// Country IP database lookup, written by Marco
// IP database is sourced from: http://software77.net/geo-ip/
// Database last updated: 19.9.2015
Class CtryDatabase extends Object
	Abstract;

struct FCountryTagEntry
{
	var const int S,E;
	var const byte C;
};
var const int StartIndex[5],EndIndex[5];
var const array<string> CN;
var const array<FCountryTagEntry> CT;

static final function string GetClientCountryStr(string IP)
{
	local int i;
	local array<string> SA;

	// Convert dotted IP-address into integer.
	ParseStringIntoArray(IP,SA,".",false);
	if (SA.Length<=1)
		return "???";
	i = int(SA[0])<<16 | int(SA[1])<<8 | int(SA[2]);

	return GetClientCountryTag(i);
}
static final function string GetClientCountryTag(int IP)
{
	local byte V;
	local int End,i;

	V = IP>>16; // Grab last IP value for lookup table.

	if (V<=50)
		V = 0;
	else if (V<=100)
		V = 1;
	else if (V<=150)
		V = 2;
	else if (V<=200)
		V = 3;
	else V = 4;

	// Speed up using lookup table.
	End = Default.EndIndex[V];

	for (i=Default.StartIndex[V]; i<End; ++i)
	{
		if (IP>=Default.CT[i].S && IP<=Default.CT[i].E) // See if entered range.
			return Default.CN[Default.CT[i].C];
	}
	return "???";
}

defaultproperties
{
	StartIndex(0)=0
	StartIndex(1)=6424
	StartIndex(2)=22318
	StartIndex(3)=34231
	StartIndex(4)=64008
	EndIndex(0)=6424
	EndIndex(1)=22318
	EndIndex(2)=34231
	EndIndex(3)=64009
	EndIndex(4)=74275
	CN(0)="AUS"
	CN(1)="CHN"
	CN(2)="JPN"
	CN(3)="THA"
	CN(4)="IND"
	CN(5)="MYS"
	CN(6)="KOR"
	CN(7)="SGP"
	CN(8)="HKG"
	CN(9)="TWN"
	CN(10)="PHL"
	CN(11)="VNM"
	CN(12)="FRA"
	CN(13)="EU"
	CN(14)="GBR"
	CN(15)="ITA"
	CN(16)="ARE"
	CN(17)="ISR"
	CN(18)="RUS"
	CN(19)="SWE"
	CN(20)="KAZ"
	CN(21)="PRT"
	CN(22)="GRC"
	CN(23)="SAU"
	CN(24)="DNK"
	CN(25)="ESP"
	CN(26)="IRN"
	CN(27)="NOR"
	CN(28)="DEU"
	CN(29)="USA"
	CN(30)="SYR"
	CN(31)="UKR"
	CN(32)="CYP"
	CN(33)="CZE"
	CN(34)="CHE"
	CN(35)="IRQ"
	CN(36)="NLD"
	CN(37)="TUR"
	CN(38)="ROU"
	CN(39)="LBN"
	CN(40)="GEO"
	CN(41)="AZE"
	CN(42)="AUT"
	CN(43)="PSE"
	CN(44)="LTU"
	CN(45)="OMN"
	CN(46)="SRB"
	CN(47)="FIN"
	CN(48)="ISL"
	CN(49)="BEL"
	CN(50)="HUN"
	CN(51)="BGR"
	CN(52)="SVN"
	CN(53)="MDA"
	CN(54)="MKD"
	CN(55)="EST"
	CN(56)="LIE"
	CN(57)="HRV"
	CN(58)="POL"
	CN(59)="BIH"
	CN(60)="LVA"
	CN(61)="JOR"
	CN(62)="KGZ"
	CN(63)="IRL"
	CN(64)="IMN"
	CN(65)="LBY"
	CN(66)="ARM"
	CN(67)="YEM"
	CN(68)="BLR"
	CN(69)="GIB"
	CN(70)="LUX"
	CN(71)="SVK"
	CN(72)="MLT"
	CN(73)="NZL"
	CN(74)="IDN"
	CN(75)="NPL"
	CN(76)="PNG"
	CN(77)="PAK"
	CN(78)="CAN"
	CN(79)="BRB"
	CN(80)="PRI"
	CN(81)="BHS"
	CN(82)="VCT"
	CN(83)="ARG"
	CN(84)="BGD"
	CN(85)="TKL"
	CN(86)="KHM"
	CN(87)="MAC"
	CN(88)="MDV"
	CN(89)="AFG"
	CN(90)="NCL"
	CN(91)="FJI"
	CN(92)="MNG"
	CN(93)="WLF"
	CN(94)="QAT"
	CN(95)="VGB"
	CN(96)="ALB"
	CN(97)="SLE"
	CN(98)="UZB"
	CN(99)="JEY"
	CN(100)="SMR"
	CN(101)="KWT"
	CN(102)="MNE"
	CN(103)="TJK"
	CN(104)="BHR"
	CN(105)="ZAF"
	CN(106)="EGY"
	CN(107)="ZWE"
	CN(108)="LBR"
	CN(109)="KEN"
	CN(110)="GHA"
	CN(111)="NGA"
	CN(112)="TZA"
	CN(113)="MUS"
	CN(114)="ZMB"
	CN(115)="MDG"
	CN(116)="AGO"
	CN(117)="NAM"
	CN(118)="CIV"
	CN(119)="SDN"
	CN(120)="CMR"
	CN(121)="MWI"
	CN(122)="GAB"
	CN(123)="MLI"
	CN(124)="BEN"
	CN(125)="TCD"
	CN(126)="BWA"
	CN(127)="CPV"
	CN(128)="RWA"
	CN(129)="COG"
	CN(130)="UGA"
	CN(131)="MOZ"
	CN(132)="GMB"
	CN(133)="LSO"
	CN(134)="MAR"
	CN(135)="DZA"
	CN(136)="GIN"
	CN(137)="COD"
	CN(138)="SWZ"
	CN(139)="BFA"
	CN(140)="SOM"
	CN(141)="NER"
	CN(142)="CAF"
	CN(143)="TGO"
	CN(144)="SSD"
	CN(145)="BDI"
	CN(146)="GNQ"
	CN(147)="SYC"
	CN(148)="SEN"
	CN(149)="MRT"
	CN(150)="DJI"
	CN(151)="REU"
	CN(152)="TUN"
	CN(153)="MYT"
	CN(154)="LAO"
	CN(155)="MMR"
	CN(156)="LKA"
	CN(157)="BRN"
	CN(158)="NRU"
	CN(159)="GUM"
	CN(160)="VUT"
	CN(161)="BTN"
	CN(162)="WSM"
	CN(163)="FSM"
	CN(164)="PYF"
	CN(165)="TLS"
	CN(166)="TON"
	CN(167)="MNP"
	CN(168)="GRL"
	CN(169)="VAT"
	CN(170)="BLZ"
	CN(171)="FRO"
	CN(172)="GGY"
	CN(173)="NIU"
	CN(174)="CYM"
	CN(175)="JAM"
	CN(176)="TTO"
	CN(177)="DOM"
	CN(178)="BMU"
	CN(179)="TCA"
	CN(180)="VIR"
	CN(181)="COL"
	CN(182)="DMA"
	CN(183)="ATG"
	CN(184)="SPM"
	CN(185)="MAF"
	CN(186)="GRD"
	CN(187)="MCO"
	CN(188)="AND"
	CN(189)="BRA"
	CN(190)="TKM"
	CN(191)="MTQ"
	CN(192)="SLB"
	CN(193)="KIR"
	CN(194)="PLW"
	CN(195)="NFK"
	CN(196)="ASM"
	CN(197)="AIA"
	CN(198)="GLP"
	CN(199)="MHL"
	CN(200)="VEN"
	CN(201)="BOL"
	CN(202)="CRI"
	CN(203)="CHL"
	CN(204)="MEX"
	CN(205)="URY"
	CN(206)="PRY"
	CN(207)="CUW"
	CN(208)="HND"
	CN(209)="PAN"
	CN(210)="SLV"
	CN(211)="SXM"
	CN(212)="PER"
	CN(213)="ECU"
	CN(214)="GUY"
	CN(215)="GTM"
	CN(216)="NIC"
	CN(217)="BES"
	CN(218)="HTI"
	CN(219)="CUB"
	CN(220)="STP"
	CN(221)="GNB"
	CN(222)="GUF"
	CN(223)="PRK"
	CN(224)="SUR"
	CN(225)="ABW"
	CN(226)="LCA"
	CN(227)="ERI"
	CN(228)="ETH"
	CN(229)="COM"
	CN(230)="KNA"
	CN(231)="MSR"
	CN(232)="TUV"
	CN(233)="IOT"
	CN(234)="COK"
	CN(235)="AFR"
	CT(0)=(S=65537,E=65539,C=1)
	CT(1)=(S=65540,E=65543,C=0)
	CT(2)=(S=65544,E=65551,C=1)
	CT(3)=(S=65552,E=65567,C=2)
	CT(4)=(S=65568,E=65599,C=1)
	CT(5)=(S=65600,E=65663,C=2)
	CT(6)=(S=65664,E=65791,C=3)
	CT(7)=(S=65792,E=65792,C=1)
	CT(8)=(S=65793,E=65793,C=0)
	CT(9)=(S=65794,E=65855,C=1)
	CT(10)=(S=65856,E=65919,C=2)
	CT(11)=(S=65920,E=66047,C=3)
	CT(12)=(S=66048,E=66050,C=1)
	CT(13)=(S=66051,E=66051,C=0)
	CT(14)=(S=66052,E=66175,C=1)
	CT(15)=(S=66176,E=66303,C=3)
	CT(16)=(S=66304,E=66559,C=1)
	CT(17)=(S=66560,E=66560,C=0)
	CT(18)=(S=66561,E=66687,C=1)
	CT(19)=(S=66688,E=66815,C=3)
	CT(20)=(S=66816,E=67071,C=2)
	CT(21)=(S=67072,E=67583,C=4)
	CT(22)=(S=67584,E=67839,C=1)
	CT(23)=(S=67840,E=68095,C=5)
	CT(24)=(S=68096,E=68105,C=1)
	CT(25)=(S=68106,E=68106,C=0)
	CT(26)=(S=68107,E=68223,C=1)
	CT(27)=(S=68224,E=68351,C=3)
	CT(28)=(S=68352,E=68607,C=6)
	CT(29)=(S=68608,E=69631,C=1)
	CT(30)=(S=69632,E=70655,C=6)
	CT(31)=(S=70656,E=70911,C=3)
	CT(32)=(S=70912,E=71167,C=2)
	CT(33)=(S=71168,E=71679,C=4)
	CT(34)=(S=71680,E=73727,C=1)
	CT(35)=(S=73728,E=73855,C=5)
	CT(36)=(S=73856,E=73919,C=7)
	CT(37)=(S=73920,E=73983,C=8)
	CT(38)=(S=73984,E=74239,C=2)
	CT(39)=(S=74240,E=74751,C=9)
	CT(40)=(S=74752,E=75007,C=8)
	CT(41)=(S=75008,E=75263,C=10)
	CT(42)=(S=75264,E=75775,C=4)
	CT(43)=(S=75776,E=77055,C=0)
	CT(44)=(S=77056,E=77311,C=1)
	CT(45)=(S=77312,E=77823,C=3)
	CT(46)=(S=77824,E=78847,C=1)
	CT(47)=(S=78848,E=79871,C=11)
	CT(48)=(S=79872,E=81919,C=1)
	CT(49)=(S=81920,E=82431,C=8)
	CT(50)=(S=82432,E=82943,C=2)
	CT(51)=(S=82944,E=83967,C=1)
	CT(52)=(S=83968,E=86015,C=2)
	CT(53)=(S=86016,E=90111,C=1)
	CT(54)=(S=90112,E=94207,C=6)
	CT(55)=(S=94208,E=95231,C=2)
	CT(56)=(S=95232,E=96255,C=1)
	CT(57)=(S=96256,E=106495,C=0)
	CT(58)=(S=106496,E=110591,C=9)
	CT(59)=(S=110592,E=111103,C=6)
	CT(60)=(S=111104,E=111487,C=0)
	CT(61)=(S=111488,E=111615,C=3)
	CT(62)=(S=111616,E=113151,C=1)
	CT(63)=(S=113152,E=113663,C=4)
	CT(64)=(S=113664,E=116735,C=1)
	CT(65)=(S=116736,E=116991,C=9)
	CT(66)=(S=116992,E=117247,C=6)
	CT(67)=(S=117248,E=118783,C=1)
	CT(68)=(S=118784,E=131071,C=6)
	CT(69)=(S=131072,E=135167,C=12)
	CT(70)=(S=135168,E=137215,C=13)
	CT(71)=(S=137216,E=139263,C=14)
	CT(72)=(S=139264,E=143359,C=15)
	CT(73)=(S=143360,E=144383,C=16)
	CT(74)=(S=144384,E=145407,C=17)
	CT(75)=(S=146432,E=147455,C=18)
	CT(76)=(S=147456,E=149503,C=19)
	CT(77)=(S=149504,E=151551,C=20)
	CT(78)=(S=151552,E=152575,C=21)
	CT(79)=(S=152576,E=153599,C=22)
	CT(80)=(S=153600,E=154623,C=23)
	CT(81)=(S=154624,E=155647,C=18)
	CT(82)=(S=155648,E=157695,C=14)
	CT(83)=(S=157696,E=159743,C=24)
	CT(84)=(S=159744,E=161791,C=15)
	CT(85)=(S=161792,E=163839,C=14)
	CT(86)=(S=163840,E=164863,C=24)
	CT(87)=(S=164864,E=165887,C=20)
	CT(88)=(S=165888,E=167935,C=25)
	CT(89)=(S=167936,E=168959,C=26)
	CT(90)=(S=168960,E=169983,C=27)
	CT(91)=(S=169984,E=171007,C=25)
	CT(92)=(S=171008,E=172031,C=15)
	CT(93)=(S=172032,E=176127,C=28)
	CT(94)=(S=176128,E=180223,C=26)
	CT(95)=(S=180224,E=182271,C=15)
	CT(96)=(S=182272,E=186367,C=28)
	CT(97)=(S=186368,E=188415,C=14)
	CT(98)=(S=188416,E=192511,C=15)
	CT(99)=(S=192512,E=194559,C=28)
	CT(100)=(S=194560,E=196607,C=19)
	CT(101)=(S=196608,E=327679,C=29)
	CT(102)=(S=327680,E=327935,C=30)
	CT(103)=(S=327936,E=327967,C=31)
	CT(104)=(S=327968,E=327975,C=25)
	CT(105)=(S=327976,E=327983,C=32)
	CT(106)=(S=327984,E=327991,C=18)
	CT(107)=(S=327992,E=327999,C=33)
	CT(108)=(S=328000,E=328031,C=28)
	CT(109)=(S=328032,E=328039,C=34)
	CT(110)=(S=328040,E=328047,C=35)
	CT(111)=(S=328048,E=328055,C=34)
	CT(112)=(S=328056,E=328191,C=28)
	CT(113)=(S=328192,E=328207,C=15)
	CT(114)=(S=328208,E=328215,C=14)
	CT(115)=(S=328216,E=328223,C=25)
	CT(116)=(S=328224,E=328255,C=18)
	CT(117)=(S=328256,E=328271,C=36)
	CT(118)=(S=328272,E=328279,C=37)
	CT(119)=(S=328280,E=328287,C=25)
	CT(120)=(S=328288,E=328319,C=14)
	CT(121)=(S=328320,E=328447,C=38)
	CT(122)=(S=328448,E=328703,C=18)
	CT(123)=(S=328704,E=329727,C=28)
	CT(124)=(S=329728,E=329823,C=18)
	CT(125)=(S=329824,E=329855,C=15)
	CT(126)=(S=329856,E=329887,C=39)
	CT(127)=(S=329888,E=329911,C=18)
	CT(128)=(S=329912,E=329919,C=25)
	CT(129)=(S=329920,E=329967,C=18)
	CT(130)=(S=329968,E=329975,C=35)
	CT(131)=(S=329984,E=330255,C=28)
	CT(132)=(S=330256,E=330271,C=14)
	CT(133)=(S=330272,E=330279,C=40)
	CT(134)=(S=330280,E=330303,C=28)
	CT(135)=(S=330304,E=330367,C=36)
	CT(136)=(S=330368,E=330375,C=12)
	CT(137)=(S=330376,E=330399,C=14)
	CT(138)=(S=330400,E=330431,C=28)
	CT(139)=(S=330432,E=330439,C=36)
	CT(140)=(S=330440,E=330447,C=25)
	CT(141)=(S=330448,E=330463,C=28)
	CT(142)=(S=330464,E=330471,C=35)
	CT(143)=(S=330472,E=330479,C=14)
	CT(144)=(S=330480,E=330495,C=41)
	CT(145)=(S=330496,E=330503,C=42)
	CT(146)=(S=330504,E=330527,C=14)
	CT(147)=(S=330528,E=330535,C=15)
	CT(148)=(S=330536,E=330543,C=43)
	CT(149)=(S=330544,E=330551,C=28)
	CT(150)=(S=330552,E=330559,C=22)
	CT(151)=(S=330560,E=330575,C=18)
	CT(152)=(S=330576,E=330591,C=14)
	CT(153)=(S=330592,E=330623,C=15)
	CT(154)=(S=330624,E=330751,C=37)
	CT(155)=(S=330752,E=331775,C=38)
	CT(156)=(S=331776,E=332799,C=18)
	CT(157)=(S=332800,E=333055,C=44)
	CT(158)=(S=333056,E=333311,C=45)
	CT(159)=(S=333312,E=333439,C=26)
	CT(160)=(S=333440,E=333447,C=17)
	CT(161)=(S=333448,E=333455,C=14)
	CT(162)=(S=333456,E=333463,C=28)
	CT(163)=(S=333464,E=333471,C=33)
	CT(164)=(S=333472,E=333503,C=46)
	CT(165)=(S=333504,E=333535,C=26)
	CT(166)=(S=333536,E=333543,C=14)
	CT(167)=(S=333544,E=333551,C=19)
	CT(168)=(S=333552,E=333559,C=28)
	CT(169)=(S=333560,E=333567,C=36)
	CT(170)=(S=333568,E=333599,C=13)
	CT(171)=(S=333600,E=333607,C=15)
	CT(172)=(S=333608,E=333615,C=12)
	CT(173)=(S=333616,E=333623,C=18)
	CT(174)=(S=333624,E=333631,C=47)
	CT(175)=(S=333632,E=333663,C=48)
	CT(176)=(S=333664,E=333679,C=18)
	CT(177)=(S=333680,E=333687,C=26)
	CT(178)=(S=333688,E=333695,C=37)
	CT(179)=(S=333696,E=333823,C=49)
	CT(180)=(S=333824,E=334847,C=37)
	CT(181)=(S=334848,E=334855,C=50)
	CT(182)=(S=334856,E=334863,C=15)
	CT(183)=(S=334864,E=334879,C=18)
	CT(184)=(S=334880,E=334887,C=26)
	CT(185)=(S=334888,E=334895,C=28)
	CT(186)=(S=334896,E=334911,C=14)
	CT(187)=(S=334912,E=334975,C=28)
	CT(188)=(S=334976,E=335039,C=17)
	CT(189)=(S=335040,E=335103,C=28)
	CT(190)=(S=335104,E=335359,C=17)
	CT(191)=(S=335360,E=335999,C=16)
	CT(192)=(S=336000,E=336007,C=51)
	CT(193)=(S=336008,E=336015,C=52)
	CT(194)=(S=336016,E=336023,C=18)
	CT(195)=(S=336024,E=336031,C=14)
	CT(196)=(S=336032,E=336039,C=28)
	CT(197)=(S=336040,E=336047,C=53)
	CT(198)=(S=336048,E=336055,C=54)
	CT(199)=(S=336056,E=336063,C=28)
	CT(200)=(S=336064,E=336127,C=45)
	CT(201)=(S=336128,E=336383,C=24)
	CT(202)=(S=336384,E=336511,C=20)
	CT(203)=(S=336512,E=336543,C=25)
	CT(204)=(S=336544,E=336551,C=43)
	CT(205)=(S=336552,E=336559,C=17)
	CT(206)=(S=336560,E=336567,C=31)
	CT(207)=(S=336568,E=336575,C=15)
	CT(208)=(S=336576,E=336607,C=26)
	CT(209)=(S=336608,E=336615,C=28)
	CT(210)=(S=336616,E=336623,C=36)
	CT(211)=(S=336624,E=336631,C=55)
	CT(212)=(S=336632,E=336639,C=56)
	CT(213)=(S=336640,E=336815,C=18)
	CT(214)=(S=336816,E=336823,C=36)
	CT(215)=(S=336824,E=336831,C=19)
	CT(216)=(S=336832,E=336839,C=49)
	CT(217)=(S=336840,E=336847,C=25)
	CT(218)=(S=336848,E=336855,C=38)
	CT(219)=(S=336856,E=336863,C=25)
	CT(220)=(S=336864,E=336895,C=28)
	CT(221)=(S=336896,E=337407,C=45)
	CT(222)=(S=337408,E=337535,C=16)
	CT(223)=(S=337536,E=337663,C=50)
	CT(224)=(S=337664,E=337791,C=12)
	CT(225)=(S=337792,E=337823,C=57)
	CT(226)=(S=337824,E=337831,C=18)
	CT(227)=(S=337832,E=337839,C=36)
	CT(228)=(S=337840,E=337847,C=14)
	CT(229)=(S=337848,E=337855,C=36)
	CT(230)=(S=337856,E=337863,C=58)
	CT(231)=(S=337864,E=337871,C=18)
	CT(232)=(S=337872,E=337879,C=25)
	CT(233)=(S=337880,E=337887,C=36)
	CT(234)=(S=337888,E=337903,C=12)
	CT(235)=(S=337904,E=337911,C=28)
	CT(236)=(S=337912,E=337919,C=14)
	CT(237)=(S=337920,E=338175,C=25)
	CT(238)=(S=338176,E=338431,C=23)
	CT(239)=(S=338432,E=338559,C=18)
	CT(240)=(S=338560,E=338567,C=14)
	CT(241)=(S=338568,E=338575,C=28)
	CT(242)=(S=338576,E=338583,C=15)
	CT(243)=(S=338584,E=338623,C=12)
	CT(244)=(S=338624,E=338655,C=35)
	CT(245)=(S=338656,E=338687,C=23)
	CT(246)=(S=338688,E=338751,C=21)
	CT(247)=(S=338752,E=338815,C=59)
	CT(248)=(S=338816,E=338847,C=18)
	CT(249)=(S=338848,E=338879,C=57)
	CT(250)=(S=338880,E=338911,C=43)
	CT(251)=(S=338912,E=338919,C=33)
	CT(252)=(S=338920,E=338927,C=15)
	CT(253)=(S=338928,E=338935,C=18)
	CT(254)=(S=338936,E=338943,C=15)
	CT(255)=(S=338944,E=338959,C=18)
	CT(256)=(S=338960,E=338975,C=14)
	CT(257)=(S=338976,E=338983,C=41)
	CT(258)=(S=338984,E=339007,C=18)
	CT(259)=(S=339008,E=339015,C=27)
	CT(260)=(S=339016,E=339023,C=36)
	CT(261)=(S=339024,E=339039,C=37)
	CT(262)=(S=339040,E=339055,C=28)
	CT(263)=(S=339056,E=339071,C=34)
	CT(264)=(S=339072,E=339079,C=40)
	CT(265)=(S=339080,E=339087,C=24)
	CT(266)=(S=339088,E=339103,C=37)
	CT(267)=(S=339104,E=339111,C=12)
	CT(268)=(S=339112,E=339119,C=18)
	CT(269)=(S=339120,E=339127,C=19)
	CT(270)=(S=339128,E=339135,C=55)
	CT(271)=(S=339136,E=339151,C=21)
	CT(272)=(S=339152,E=339159,C=42)
	CT(273)=(S=339160,E=339167,C=60)
	CT(274)=(S=339168,E=339183,C=14)
	CT(275)=(S=339184,E=339191,C=47)
	CT(276)=(S=339192,E=339199,C=14)
	CT(277)=(S=339200,E=339207,C=28)
	CT(278)=(S=339208,E=339215,C=25)
	CT(279)=(S=339216,E=339255,C=14)
	CT(280)=(S=339256,E=339263,C=33)
	CT(281)=(S=339264,E=339295,C=18)
	CT(282)=(S=339296,E=339311,C=28)
	CT(283)=(S=339312,E=339327,C=55)
	CT(284)=(S=339328,E=339343,C=61)
	CT(285)=(S=339344,E=339351,C=14)
	CT(286)=(S=339352,E=339359,C=27)
	CT(287)=(S=339360,E=339375,C=25)
	CT(288)=(S=339376,E=339383,C=28)
	CT(289)=(S=339384,E=339391,C=39)
	CT(290)=(S=339392,E=339455,C=18)
	CT(291)=(S=339456,E=339967,C=37)
	CT(292)=(S=339968,E=340991,C=12)
	CT(293)=(S=340992,E=341247,C=26)
	CT(294)=(S=341248,E=341255,C=25)
	CT(295)=(S=341256,E=341263,C=33)
	CT(296)=(S=341264,E=341279,C=18)
	CT(297)=(S=341280,E=341311,C=26)
	CT(298)=(S=341312,E=341343,C=14)
	CT(299)=(S=341344,E=341351,C=16)
	CT(300)=(S=341352,E=341359,C=34)
	CT(301)=(S=341360,E=341367,C=31)
	CT(302)=(S=341368,E=341375,C=18)
	CT(303)=(S=341376,E=341503,C=51)
	CT(304)=(S=341504,E=342015,C=22)
	CT(305)=(S=342016,E=342023,C=25)
	CT(306)=(S=342024,E=342031,C=15)
	CT(307)=(S=342032,E=342039,C=28)
	CT(308)=(S=342040,E=342047,C=31)
	CT(309)=(S=342048,E=342055,C=50)
	CT(310)=(S=342056,E=342063,C=12)
	CT(311)=(S=342064,E=342071,C=14)
	CT(312)=(S=342072,E=342079,C=25)
	CT(313)=(S=342080,E=342143,C=53)
	CT(314)=(S=342144,E=342151,C=14)
	CT(315)=(S=342152,E=342159,C=18)
	CT(316)=(S=342160,E=342175,C=24)
	CT(317)=(S=342176,E=342183,C=25)
	CT(318)=(S=342184,E=342191,C=14)
	CT(319)=(S=342192,E=342271,C=28)
	CT(320)=(S=342272,E=342279,C=39)
	CT(321)=(S=342280,E=342287,C=62)
	CT(322)=(S=342288,E=342295,C=36)
	CT(323)=(S=342296,E=342303,C=25)
	CT(324)=(S=342304,E=342311,C=26)
	CT(325)=(S=342312,E=342319,C=28)
	CT(326)=(S=342320,E=342327,C=24)
	CT(327)=(S=342328,E=342335,C=14)
	CT(328)=(S=342336,E=342343,C=31)
	CT(329)=(S=342344,E=342351,C=46)
	CT(330)=(S=342352,E=342367,C=14)
	CT(331)=(S=342368,E=342399,C=12)
	CT(332)=(S=342400,E=342463,C=58)
	CT(333)=(S=342464,E=342471,C=28)
	CT(334)=(S=342472,E=342479,C=34)
	CT(335)=(S=342480,E=342487,C=16)
	CT(336)=(S=342488,E=342495,C=18)
	CT(337)=(S=342496,E=342503,C=36)
	CT(338)=(S=342504,E=342511,C=18)
	CT(339)=(S=342512,E=342519,C=19)
	CT(340)=(S=342520,E=342527,C=36)
	CT(341)=(S=342528,E=342783,C=31)
	CT(342)=(S=342784,E=343039,C=33)
	CT(343)=(S=343040,E=343295,C=58)
	CT(344)=(S=343296,E=343303,C=15)
	CT(345)=(S=343304,E=343319,C=18)
	CT(346)=(S=343320,E=343327,C=26)
	CT(347)=(S=343328,E=343359,C=14)
	CT(348)=(S=343360,E=343367,C=28)
	CT(349)=(S=343368,E=343375,C=26)
	CT(350)=(S=343376,E=343383,C=28)
	CT(351)=(S=343384,E=343391,C=47)
	CT(352)=(S=343392,E=343407,C=51)
	CT(353)=(S=343408,E=343415,C=63)
	CT(354)=(S=343416,E=343423,C=14)
	CT(355)=(S=343424,E=343487,C=28)
	CT(356)=(S=343488,E=343495,C=14)
	CT(357)=(S=343496,E=343511,C=25)
	CT(358)=(S=343512,E=343519,C=15)
	CT(359)=(S=343520,E=343527,C=34)
	CT(360)=(S=343528,E=343535,C=18)
	CT(361)=(S=343536,E=343543,C=50)
	CT(362)=(S=343544,E=343551,C=36)
	CT(363)=(S=343552,E=343615,C=14)
	CT(364)=(S=343616,E=343623,C=28)
	CT(365)=(S=343624,E=343631,C=15)
	CT(366)=(S=343632,E=343647,C=64)
	CT(367)=(S=343648,E=343679,C=28)
	CT(368)=(S=343680,E=343703,C=35)
	CT(369)=(S=343704,E=343711,C=18)
	CT(370)=(S=343712,E=343743,C=26)
	CT(371)=(S=343744,E=343807,C=18)
	CT(372)=(S=343808,E=343815,C=65)
	CT(373)=(S=343816,E=343823,C=26)
	CT(374)=(S=343824,E=343831,C=14)
	CT(375)=(S=343832,E=343839,C=12)
	CT(376)=(S=343840,E=343871,C=37)
	CT(377)=(S=343872,E=343935,C=20)
	CT(378)=(S=343936,E=343943,C=18)
	CT(379)=(S=343944,E=343951,C=15)
	CT(380)=(S=343952,E=343959,C=14)
	CT(381)=(S=343960,E=343967,C=18)
	CT(382)=(S=343968,E=343975,C=66)
	CT(383)=(S=343976,E=343983,C=15)
	CT(384)=(S=343984,E=343991,C=28)
	CT(385)=(S=343992,E=343999,C=58)
	CT(386)=(S=344000,E=344063,C=50)
	CT(387)=(S=344064,E=346111,C=14)
	CT(388)=(S=346112,E=347135,C=18)
	CT(389)=(S=347136,E=347391,C=20)
	CT(390)=(S=347392,E=347423,C=18)
	CT(391)=(S=347424,E=347455,C=14)
	CT(392)=(S=347456,E=347519,C=15)
	CT(393)=(S=347520,E=347647,C=66)
	CT(394)=(S=347648,E=347903,C=26)
	CT(395)=(S=347904,E=347967,C=14)
	CT(396)=(S=347968,E=348031,C=36)
	CT(397)=(S=348032,E=348159,C=18)
	CT(398)=(S=348160,E=348671,C=14)
	CT(399)=(S=348672,E=348927,C=23)
	CT(400)=(S=348928,E=348935,C=36)
	CT(401)=(S=348936,E=348943,C=14)
	CT(402)=(S=348944,E=348951,C=31)
	CT(403)=(S=348952,E=348959,C=24)
	CT(404)=(S=348960,E=348975,C=38)
	CT(405)=(S=348976,E=348983,C=25)
	CT(406)=(S=348984,E=348991,C=47)
	CT(407)=(S=348992,E=349023,C=25)
	CT(408)=(S=349024,E=349031,C=14)
	CT(409)=(S=349032,E=349039,C=15)
	CT(410)=(S=349040,E=349047,C=16)
	CT(411)=(S=349048,E=349055,C=15)
	CT(412)=(S=349056,E=349119,C=28)
	CT(413)=(S=349120,E=349151,C=14)
	CT(414)=(S=349152,E=349159,C=27)
	CT(415)=(S=349160,E=349167,C=12)
	CT(416)=(S=349168,E=349183,C=63)
	CT(417)=(S=349184,E=353279,C=15)
	CT(418)=(S=353280,E=353343,C=28)
	CT(419)=(S=353344,E=353407,C=18)
	CT(420)=(S=353408,E=353423,C=28)
	CT(421)=(S=353424,E=353431,C=12)
	CT(422)=(S=353432,E=353439,C=14)
	CT(423)=(S=353440,E=353447,C=67)
	CT(424)=(S=353448,E=353455,C=14)
	CT(425)=(S=353456,E=353471,C=27)
	CT(426)=(S=353472,E=353503,C=68)
	CT(427)=(S=353504,E=353511,C=36)
	CT(428)=(S=353512,E=353519,C=15)
	CT(429)=(S=353528,E=353535,C=17)
	CT(430)=(S=353536,E=353631,C=18)
	CT(431)=(S=353632,E=353647,C=36)
	CT(432)=(S=353648,E=353663,C=55)
	CT(433)=(S=353664,E=353671,C=18)
	CT(434)=(S=353672,E=353687,C=14)
	CT(435)=(S=353688,E=353695,C=18)
	CT(436)=(S=353696,E=353703,C=28)
	CT(437)=(S=353704,E=353711,C=14)
	CT(438)=(S=353712,E=353727,C=55)
	CT(439)=(S=353728,E=353791,C=18)
	CT(440)=(S=353792,E=353823,C=15)
	CT(441)=(S=353824,E=353839,C=26)
	CT(442)=(S=353840,E=353855,C=33)
	CT(443)=(S=353856,E=353863,C=58)
	CT(444)=(S=353864,E=353871,C=12)
	CT(445)=(S=353872,E=353887,C=14)
	CT(446)=(S=353888,E=353919,C=43)
	CT(447)=(S=353920,E=353927,C=25)
	CT(448)=(S=353928,E=353935,C=36)
	CT(449)=(S=353936,E=353943,C=34)
	CT(450)=(S=353944,E=353951,C=18)
	CT(451)=(S=353952,E=353959,C=28)
	CT(452)=(S=353960,E=353967,C=14)
	CT(453)=(S=353968,E=353975,C=52)
	CT(454)=(S=353976,E=353983,C=14)
	CT(455)=(S=353984,E=354047,C=17)
	CT(456)=(S=354048,E=354303,C=24)
	CT(457)=(S=354304,E=354319,C=37)
	CT(458)=(S=354320,E=354327,C=33)
	CT(459)=(S=354328,E=354335,C=15)
	CT(460)=(S=354336,E=354367,C=31)
	CT(461)=(S=354368,E=354375,C=13)
	CT(462)=(S=354376,E=354383,C=35)
	CT(463)=(S=354384,E=354391,C=47)
	CT(464)=(S=354392,E=354399,C=34)
	CT(465)=(S=354400,E=354407,C=12)
	CT(466)=(S=354408,E=354415,C=28)
	CT(467)=(S=354416,E=354431,C=36)
	CT(468)=(S=354432,E=354439,C=30)
	CT(469)=(S=354440,E=354447,C=36)
	CT(470)=(S=354448,E=354455,C=28)
	CT(471)=(S=354456,E=354463,C=36)
	CT(472)=(S=354464,E=354495,C=51)
	CT(473)=(S=354496,E=354503,C=12)
	CT(474)=(S=354504,E=354511,C=18)
	CT(475)=(S=354512,E=354519,C=58)
	CT(476)=(S=354520,E=354527,C=42)
	CT(477)=(S=354528,E=354535,C=36)
	CT(478)=(S=354536,E=354543,C=20)
	CT(479)=(S=354544,E=354551,C=17)
	CT(480)=(S=354552,E=354559,C=58)
	CT(481)=(S=354560,E=354815,C=31)
	CT(482)=(S=354816,E=355071,C=26)
	CT(483)=(S=355072,E=355327,C=16)
	CT(484)=(S=355328,E=356351,C=23)
	CT(485)=(S=356352,E=360447,C=26)
	CT(486)=(S=360448,E=361471,C=18)
	CT(487)=(S=361472,E=361599,C=36)
	CT(488)=(S=361600,E=361663,C=42)
	CT(489)=(S=361664,E=361727,C=23)
	CT(490)=(S=361728,E=361735,C=59)
	CT(491)=(S=361736,E=361743,C=58)
	CT(492)=(S=361744,E=361751,C=14)
	CT(493)=(S=361752,E=361759,C=43)
	CT(494)=(S=361760,E=361767,C=34)
	CT(495)=(S=361768,E=361775,C=14)
	CT(496)=(S=361776,E=361791,C=15)
	CT(497)=(S=361856,E=361887,C=59)
	CT(498)=(S=361888,E=361895,C=40)
	CT(499)=(S=361896,E=361919,C=14)
	CT(500)=(S=361920,E=361951,C=19)
	CT(501)=(S=361952,E=361967,C=41)
	CT(502)=(S=361968,E=361975,C=14)
	CT(503)=(S=361976,E=361983,C=58)
	CT(504)=(S=361984,E=361991,C=49)
	CT(505)=(S=361992,E=361999,C=14)
	CT(506)=(S=362000,E=362007,C=28)
	CT(507)=(S=362008,E=362015,C=34)
	CT(508)=(S=362016,E=362031,C=25)
	CT(509)=(S=362032,E=362047,C=41)
	CT(510)=(S=362048,E=362063,C=58)
	CT(511)=(S=362064,E=362071,C=66)
	CT(512)=(S=362072,E=362079,C=63)
	CT(513)=(S=362080,E=362087,C=12)
	CT(514)=(S=362088,E=362095,C=46)
	CT(515)=(S=362096,E=362103,C=25)
	CT(516)=(S=362104,E=362111,C=15)
	CT(517)=(S=362112,E=362175,C=26)
	CT(518)=(S=362176,E=362183,C=19)
	CT(519)=(S=362184,E=362191,C=30)
	CT(520)=(S=362192,E=362199,C=58)
	CT(521)=(S=362200,E=362207,C=18)
	CT(522)=(S=362208,E=362239,C=30)
	CT(523)=(S=362240,E=362495,C=12)
	CT(524)=(S=362496,E=364543,C=18)
	CT(525)=(S=364544,E=364591,C=34)
	CT(526)=(S=364592,E=364607,C=17)
	CT(527)=(S=364608,E=364623,C=18)
	CT(528)=(S=364624,E=364639,C=14)
	CT(529)=(S=364640,E=364671,C=18)
	CT(530)=(S=364672,E=364679,C=26)
	CT(531)=(S=364680,E=364687,C=12)
	CT(532)=(S=364688,E=364695,C=25)
	CT(533)=(S=364696,E=364703,C=14)
	CT(534)=(S=364704,E=364719,C=15)
	CT(535)=(S=364720,E=364727,C=14)
	CT(536)=(S=364728,E=364735,C=15)
	CT(537)=(S=364736,E=364799,C=22)
	CT(538)=(S=364800,E=364903,C=34)
	CT(539)=(S=364904,E=364911,C=33)
	CT(540)=(S=364912,E=364919,C=26)
	CT(541)=(S=364920,E=364927,C=15)
	CT(542)=(S=364928,E=364943,C=28)
	CT(543)=(S=364944,E=364951,C=69)
	CT(544)=(S=364952,E=364959,C=27)
	CT(545)=(S=364960,E=364967,C=18)
	CT(546)=(S=364968,E=364975,C=25)
	CT(547)=(S=364976,E=364991,C=28)
	CT(548)=(S=364992,E=365055,C=18)
	CT(549)=(S=365056,E=365567,C=28)
	CT(550)=(S=365568,E=365727,C=14)
	CT(551)=(S=365728,E=365759,C=34)
	CT(552)=(S=365760,E=365823,C=50)
	CT(553)=(S=365824,E=365831,C=34)
	CT(554)=(S=365832,E=365839,C=14)
	CT(555)=(S=365840,E=365887,C=34)
	CT(556)=(S=365888,E=365919,C=59)
	CT(557)=(S=365920,E=365935,C=35)
	CT(558)=(S=365936,E=365943,C=70)
	CT(559)=(S=365944,E=365951,C=31)
	CT(560)=(S=365952,E=365959,C=42)
	CT(561)=(S=365960,E=365967,C=49)
	CT(562)=(S=365968,E=365975,C=18)
	CT(563)=(S=365976,E=365983,C=14)
	CT(564)=(S=365984,E=365991,C=58)
	CT(565)=(S=365992,E=365999,C=63)
	CT(566)=(S=366000,E=366015,C=12)
	CT(567)=(S=366016,E=366023,C=58)
	CT(568)=(S=366024,E=366031,C=18)
	CT(569)=(S=366032,E=366047,C=31)
	CT(570)=(S=366048,E=366063,C=28)
	CT(571)=(S=366064,E=366071,C=50)
	CT(572)=(S=366072,E=366207,C=14)
	CT(573)=(S=366208,E=366223,C=15)
	CT(574)=(S=366224,E=366231,C=63)
	CT(575)=(S=366232,E=366271,C=14)
	CT(576)=(S=366272,E=366335,C=19)
	CT(577)=(S=366336,E=366591,C=14)
	CT(578)=(S=366592,E=366719,C=40)
	CT(579)=(S=366720,E=366751,C=15)
	CT(580)=(S=366752,E=366759,C=58)
	CT(581)=(S=366760,E=366767,C=25)
	CT(582)=(S=366768,E=366775,C=14)
	CT(583)=(S=366776,E=366783,C=29)
	CT(584)=(S=366784,E=366815,C=14)
	CT(585)=(S=366816,E=366823,C=60)
	CT(586)=(S=366824,E=366831,C=59)
	CT(587)=(S=366832,E=366839,C=15)
	CT(588)=(S=366840,E=366847,C=16)
	CT(589)=(S=366848,E=366911,C=36)
	CT(590)=(S=366912,E=366951,C=14)
	CT(591)=(S=366952,E=366959,C=27)
	CT(592)=(S=366960,E=366975,C=34)
	CT(593)=(S=366976,E=367039,C=31)
	CT(594)=(S=367040,E=367079,C=14)
	CT(595)=(S=367080,E=367087,C=55)
	CT(596)=(S=367088,E=367095,C=34)
	CT(597)=(S=367096,E=367103,C=14)
	CT(598)=(S=367104,E=367231,C=25)
	CT(599)=(S=367232,E=367263,C=14)
	CT(600)=(S=367264,E=367271,C=28)
	CT(601)=(S=367272,E=367295,C=38)
	CT(602)=(S=367296,E=367327,C=15)
	CT(603)=(S=367328,E=367347,C=38)
	CT(604)=(S=367348,E=367351,C=19)
	CT(605)=(S=367352,E=367615,C=38)
	CT(606)=(S=367616,E=367871,C=23)
	CT(607)=(S=367872,E=367935,C=55)
	CT(608)=(S=367936,E=367943,C=14)
	CT(609)=(S=367944,E=367951,C=52)
	CT(610)=(S=367952,E=367967,C=36)
	CT(611)=(S=367968,E=367999,C=15)
	CT(612)=(S=368000,E=368127,C=38)
	CT(613)=(S=368128,E=368191,C=21)
	CT(614)=(S=368192,E=368207,C=15)
	CT(615)=(S=368208,E=368215,C=25)
	CT(616)=(S=368216,E=368223,C=14)
	CT(617)=(S=368224,E=368255,C=18)
	CT(618)=(S=368256,E=368319,C=28)
	CT(619)=(S=368320,E=368327,C=27)
	CT(620)=(S=368328,E=368335,C=12)
	CT(621)=(S=368336,E=368343,C=14)
	CT(622)=(S=368344,E=368351,C=25)
	CT(623)=(S=368352,E=368359,C=14)
	CT(624)=(S=368360,E=368367,C=18)
	CT(625)=(S=368368,E=368383,C=12)
	CT(626)=(S=368384,E=368391,C=42)
	CT(627)=(S=368392,E=368399,C=25)
	CT(628)=(S=368400,E=368407,C=40)
	CT(629)=(S=368408,E=368415,C=28)
	CT(630)=(S=368416,E=368423,C=36)
	CT(631)=(S=368424,E=368431,C=63)
	CT(632)=(S=368432,E=368439,C=26)
	CT(633)=(S=368440,E=368479,C=28)
	CT(634)=(S=368480,E=368503,C=18)
	CT(635)=(S=368504,E=368511,C=14)
	CT(636)=(S=368512,E=368575,C=25)
	CT(637)=(S=368576,E=368583,C=36)
	CT(638)=(S=368584,E=368599,C=14)
	CT(639)=(S=368600,E=368607,C=21)
	CT(640)=(S=368608,E=368615,C=14)
	CT(641)=(S=368616,E=368623,C=50)
	CT(642)=(S=368624,E=368631,C=25)
	CT(643)=(S=368632,E=368639,C=37)
	CT(644)=(S=368640,E=369151,C=26)
	CT(645)=(S=369152,E=369407,C=45)
	CT(646)=(S=369408,E=369663,C=23)
	CT(647)=(S=369664,E=370687,C=18)
	CT(648)=(S=370688,E=371711,C=15)
	CT(649)=(S=371712,E=371743,C=18)
	CT(650)=(S=371744,E=371751,C=46)
	CT(651)=(S=371752,E=371759,C=36)
	CT(652)=(S=371760,E=371775,C=14)
	CT(653)=(S=371776,E=371839,C=15)
	CT(654)=(S=371840,E=371855,C=34)
	CT(655)=(S=371856,E=371863,C=21)
	CT(656)=(S=371864,E=371887,C=14)
	CT(657)=(S=371888,E=371895,C=12)
	CT(658)=(S=371896,E=371903,C=58)
	CT(659)=(S=371904,E=371919,C=22)
	CT(660)=(S=371920,E=371927,C=15)
	CT(661)=(S=371928,E=371935,C=36)
	CT(662)=(S=371936,E=372479,C=58)
	CT(663)=(S=372480,E=372511,C=28)
	CT(664)=(S=372512,E=372519,C=15)
	CT(665)=(S=372520,E=372527,C=25)
	CT(666)=(S=372528,E=372535,C=15)
	CT(667)=(S=372536,E=372543,C=14)
	CT(668)=(S=372544,E=372559,C=15)
	CT(669)=(S=372560,E=372567,C=36)
	CT(670)=(S=372568,E=372607,C=18)
	CT(671)=(S=372608,E=372735,C=28)
	CT(672)=(S=372736,E=373247,C=37)
	CT(673)=(S=373248,E=373263,C=41)
	CT(674)=(S=373264,E=373271,C=14)
	CT(675)=(S=373272,E=373279,C=18)
	CT(676)=(S=373280,E=373287,C=36)
	CT(677)=(S=373288,E=373295,C=15)
	CT(678)=(S=373296,E=373311,C=71)
	CT(679)=(S=373312,E=373319,C=36)
	CT(680)=(S=373320,E=373327,C=19)
	CT(681)=(S=373328,E=373335,C=18)
	CT(682)=(S=373336,E=373343,C=15)
	CT(683)=(S=373344,E=373359,C=14)
	CT(684)=(S=373360,E=373367,C=36)
	CT(685)=(S=373368,E=373375,C=15)
	CT(686)=(S=373376,E=373503,C=40)
	CT(687)=(S=373504,E=373535,C=60)
	CT(688)=(S=373536,E=373575,C=63)
	CT(689)=(S=373576,E=373583,C=14)
	CT(690)=(S=373584,E=373599,C=24)
	CT(691)=(S=373600,E=373615,C=14)
	CT(692)=(S=373616,E=373623,C=19)
	CT(693)=(S=373624,E=373631,C=12)
	CT(694)=(S=373632,E=373695,C=15)
	CT(695)=(S=373696,E=373703,C=12)
	CT(696)=(S=373704,E=373759,C=28)
	CT(697)=(S=374784,E=375295,C=58)
	CT(698)=(S=375296,E=375551,C=24)
	CT(699)=(S=375552,E=375559,C=25)
	CT(700)=(S=375560,E=375567,C=40)
	CT(701)=(S=375568,E=375583,C=14)
	CT(702)=(S=375584,E=375591,C=25)
	CT(703)=(S=375592,E=375599,C=18)
	CT(704)=(S=375600,E=375607,C=58)
	CT(705)=(S=375608,E=375615,C=34)
	CT(706)=(S=375616,E=375647,C=18)
	CT(707)=(S=375648,E=375679,C=12)
	CT(708)=(S=375680,E=375807,C=50)
	CT(709)=(S=375808,E=376191,C=18)
	CT(710)=(S=376192,E=376255,C=28)
	CT(711)=(S=376256,E=376319,C=18)
	CT(712)=(S=376320,E=376575,C=26)
	CT(713)=(S=376576,E=376831,C=41)
	CT(714)=(S=376832,E=377855,C=16)
	CT(715)=(S=377856,E=378111,C=12)
	CT(716)=(S=378112,E=378367,C=41)
	CT(717)=(S=378368,E=378495,C=14)
	CT(718)=(S=378496,E=378503,C=33)
	CT(719)=(S=378504,E=378511,C=14)
	CT(720)=(S=378512,E=378527,C=42)
	CT(721)=(S=378528,E=378559,C=26)
	CT(722)=(S=378560,E=378591,C=12)
	CT(723)=(S=378592,E=378607,C=15)
	CT(724)=(S=378608,E=378615,C=61)
	CT(725)=(S=378616,E=378623,C=72)
	CT(726)=(S=378624,E=378751,C=14)
	CT(727)=(S=378752,E=378767,C=28)
	CT(728)=(S=378768,E=378783,C=36)
	CT(729)=(S=378784,E=378799,C=44)
	CT(730)=(S=378800,E=378807,C=28)
	CT(731)=(S=378816,E=378847,C=18)
	CT(732)=(S=378848,E=378855,C=25)
	CT(733)=(S=378856,E=378863,C=31)
	CT(734)=(S=378864,E=378879,C=28)
	CT(735)=(S=378880,E=378911,C=36)
	CT(736)=(S=378912,E=378943,C=18)
	CT(737)=(S=378944,E=379135,C=26)
	CT(738)=(S=379136,E=379263,C=58)
	CT(739)=(S=379264,E=379647,C=26)
	CT(740)=(S=379648,E=379903,C=22)
	CT(741)=(S=379904,E=380159,C=50)
	CT(742)=(S=380160,E=380415,C=25)
	CT(743)=(S=380416,E=380543,C=18)
	CT(744)=(S=380544,E=380607,C=50)
	CT(745)=(S=380608,E=380615,C=24)
	CT(746)=(S=380616,E=380623,C=42)
	CT(747)=(S=380624,E=380639,C=36)
	CT(748)=(S=380640,E=380647,C=21)
	CT(749)=(S=380648,E=380655,C=46)
	CT(750)=(S=380656,E=380671,C=58)
	CT(751)=(S=380672,E=380927,C=31)
	CT(752)=(S=380928,E=385023,C=26)
	CT(753)=(S=385024,E=385535,C=25)
	CT(754)=(S=385536,E=385551,C=12)
	CT(755)=(S=385552,E=385567,C=25)
	CT(756)=(S=385568,E=385575,C=19)
	CT(757)=(S=385576,E=385583,C=36)
	CT(758)=(S=385584,E=385591,C=26)
	CT(759)=(S=385592,E=385599,C=14)
	CT(760)=(S=385600,E=385671,C=58)
	CT(761)=(S=385672,E=385679,C=14)
	CT(762)=(S=385680,E=385687,C=34)
	CT(763)=(S=385688,E=385695,C=18)
	CT(764)=(S=385696,E=385703,C=27)
	CT(765)=(S=385704,E=385711,C=25)
	CT(766)=(S=385712,E=385719,C=14)
	CT(767)=(S=385720,E=385727,C=25)
	CT(768)=(S=385728,E=385791,C=37)
	CT(769)=(S=385792,E=386303,C=18)
	CT(770)=(S=386304,E=386559,C=37)
	CT(771)=(S=386560,E=387071,C=28)
	CT(772)=(S=387072,E=389119,C=26)
	CT(773)=(S=389120,E=390143,C=19)
	CT(774)=(S=390144,E=391167,C=23)
	CT(775)=(S=391168,E=391423,C=31)
	CT(776)=(S=391424,E=391551,C=21)
	CT(777)=(S=391552,E=391583,C=15)
	CT(778)=(S=391584,E=391591,C=19)
	CT(779)=(S=391592,E=391599,C=34)
	CT(780)=(S=391600,E=391615,C=52)
	CT(781)=(S=391616,E=391647,C=18)
	CT(782)=(S=391648,E=391679,C=27)
	CT(783)=(S=391680,E=391807,C=26)
	CT(784)=(S=391808,E=391839,C=20)
	CT(785)=(S=391840,E=391847,C=47)
	CT(786)=(S=391848,E=391855,C=18)
	CT(787)=(S=391856,E=391871,C=14)
	CT(788)=(S=391872,E=391903,C=41)
	CT(789)=(S=391904,E=391911,C=19)
	CT(790)=(S=391912,E=391919,C=18)
	CT(791)=(S=391920,E=391935,C=37)
	CT(792)=(S=391936,E=392191,C=20)
	CT(793)=(S=392704,E=392831,C=38)
	CT(794)=(S=392832,E=392863,C=19)
	CT(795)=(S=392864,E=392871,C=14)
	CT(796)=(S=392872,E=392879,C=19)
	CT(797)=(S=392880,E=392887,C=18)
	CT(798)=(S=392888,E=392895,C=42)
	CT(799)=(S=392896,E=392927,C=19)
	CT(800)=(S=392928,E=392959,C=18)
	CT(801)=(S=392960,E=393023,C=37)
	CT(802)=(S=393024,E=393087,C=36)
	CT(803)=(S=393088,E=393095,C=49)
	CT(804)=(S=393096,E=393103,C=14)
	CT(805)=(S=393104,E=393111,C=25)
	CT(806)=(S=393112,E=393119,C=36)
	CT(807)=(S=393120,E=393151,C=31)
	CT(808)=(S=393152,E=393215,C=18)
	CT(809)=(S=393216,E=917503,C=29)
	CT(810)=(S=917504,E=917511,C=1)
	CT(811)=(S=917512,E=917515,C=2)
	CT(812)=(S=917516,E=917519,C=1)
	CT(813)=(S=917520,E=917535,C=11)
	CT(814)=(S=917536,E=917631,C=6)
	CT(815)=(S=917632,E=917759,C=8)
	CT(816)=(S=917760,E=917763,C=1)
	CT(817)=(S=917764,E=917775,C=2)
	CT(818)=(S=917776,E=917779,C=0)
	CT(819)=(S=917780,E=917783,C=8)
	CT(820)=(S=917792,E=917855,C=73)
	CT(821)=(S=917888,E=918015,C=5)
	CT(822)=(S=918016,E=918271,C=0)
	CT(823)=(S=918272,E=918527,C=2)
	CT(824)=(S=918528,E=919551,C=6)
	CT(825)=(S=919552,E=921599,C=2)
	CT(826)=(S=921600,E=925695,C=1)
	CT(827)=(S=925696,E=942079,C=6)
	CT(828)=(S=942080,E=943103,C=4)
	CT(829)=(S=943104,E=943359,C=7)
	CT(830)=(S=943360,E=943615,C=2)
	CT(831)=(S=943616,E=943743,C=4)
	CT(832)=(S=943744,E=943747,C=1)
	CT(833)=(S=943748,E=943751,C=2)
	CT(834)=(S=943752,E=943759,C=0)
	CT(835)=(S=943760,E=943767,C=5)
	CT(836)=(S=943768,E=943771,C=74)
	CT(837)=(S=943772,E=943775,C=1)
	CT(838)=(S=943808,E=943839,C=2)
	CT(839)=(S=943840,E=943855,C=4)
	CT(840)=(S=943856,E=943871,C=8)
	CT(841)=(S=943872,E=950271,C=1)
	CT(842)=(S=950272,E=950275,C=2)
	CT(843)=(S=950276,E=950279,C=73)
	CT(844)=(S=950280,E=950283,C=3)
	CT(845)=(S=950284,E=950287,C=75)
	CT(846)=(S=950288,E=950303,C=2)
	CT(847)=(S=950304,E=950335,C=0)
	CT(848)=(S=950336,E=950399,C=2)
	CT(849)=(S=950400,E=950783,C=6)
	CT(850)=(S=950784,E=951295,C=1)
	CT(851)=(S=951296,E=951807,C=2)
	CT(852)=(S=951808,E=952319,C=1)
	CT(853)=(S=952320,E=952575,C=8)
	CT(854)=(S=952576,E=952799,C=0)
	CT(855)=(S=952800,E=952831,C=2)
	CT(856)=(S=952832,E=953087,C=6)
	CT(857)=(S=953088,E=954367,C=4)
	CT(858)=(S=954368,E=958463,C=1)
	CT(859)=(S=958464,E=966655,C=11)
	CT(860)=(S=966688,E=966703,C=2)
	CT(861)=(S=966716,E=966719,C=1)
	CT(862)=(S=966720,E=966727,C=5)
	CT(863)=(S=966728,E=966731,C=76)
	CT(864)=(S=966732,E=966735,C=1)
	CT(865)=(S=966736,E=966751,C=6)
	CT(866)=(S=966752,E=966783,C=2)
	CT(867)=(S=966784,E=966815,C=77)
	CT(868)=(S=966816,E=966847,C=0)
	CT(869)=(S=966848,E=966911,C=5)
	CT(870)=(S=966912,E=967167,C=2)
	CT(871)=(S=967168,E=967679,C=4)
	CT(872)=(S=967680,E=968191,C=1)
	CT(873)=(S=968192,E=968703,C=8)
	CT(874)=(S=968704,E=969727,C=0)
	CT(875)=(S=969728,E=970239,C=1)
	CT(876)=(S=970240,E=970495,C=6)
	CT(877)=(S=970496,E=970751,C=3)
	CT(878)=(S=970752,E=974847,C=1)
	CT(879)=(S=974848,E=983039,C=11)
	CT(880)=(S=983040,E=1511423,C=29)
	CT(881)=(S=1511424,E=1511935,C=78)
	CT(882)=(S=1511936,E=1514943,C=29)
	CT(883)=(S=1514944,E=1514975,C=78)
	CT(884)=(S=1514976,E=1530751,C=29)
	CT(885)=(S=1530752,E=1530815,C=78)
	CT(886)=(S=1530816,E=1530847,C=29)
	CT(887)=(S=1530848,E=1530879,C=78)
	CT(888)=(S=1530880,E=1531007,C=29)
	CT(889)=(S=1531008,E=1531039,C=78)
	CT(890)=(S=1531040,E=1540095,C=29)
	CT(891)=(S=1540096,E=1540096,C=78)
	CT(892)=(S=1540120,E=1540221,C=36)
	CT(893)=(S=1548288,E=1565471,C=29)
	CT(894)=(S=1565472,E=1565503,C=78)
	CT(895)=(S=1565504,E=1565647,C=29)
	CT(896)=(S=1565648,E=1565663,C=78)
	CT(897)=(S=1565664,E=1566975,C=29)
	CT(898)=(S=1566976,E=1567231,C=78)
	CT(899)=(S=1567232,E=1567551,C=29)
	CT(900)=(S=1567552,E=1567567,C=78)
	CT(901)=(S=1567568,E=1567743,C=29)
	CT(902)=(S=1567744,E=1567759,C=79)
	CT(903)=(S=1567760,E=1568543,C=29)
	CT(904)=(S=1568544,E=1568575,C=78)
	CT(905)=(S=1568576,E=1570815,C=29)
	CT(906)=(S=1570816,E=1570975,C=78)
	CT(907)=(S=1570976,E=1571183,C=29)
	CT(908)=(S=1571184,E=1571199,C=78)
	CT(909)=(S=1571200,E=1571263,C=29)
	CT(910)=(S=1571264,E=1571279,C=78)
	CT(911)=(S=1571280,E=1571743,C=29)
	CT(912)=(S=1571744,E=1571775,C=78)
	CT(913)=(S=1571776,E=1571871,C=29)
	CT(914)=(S=1571872,E=1571887,C=78)
	CT(915)=(S=1571888,E=1571935,C=29)
	CT(916)=(S=1571936,E=1571951,C=78)
	CT(917)=(S=1571952,E=1582079,C=29)
	CT(918)=(S=1582080,E=1582591,C=78)
	CT(919)=(S=1582592,E=1582735,C=29)
	CT(920)=(S=1582736,E=1582751,C=78)
	CT(921)=(S=1582752,E=1583455,C=29)
	CT(922)=(S=1583456,E=1583487,C=78)
	CT(923)=(S=1583488,E=1583679,C=80)
	CT(924)=(S=1583680,E=1585151,C=29)
	CT(925)=(S=1585152,E=1585279,C=78)
	CT(926)=(S=1585280,E=1585327,C=29)
	CT(927)=(S=1585328,E=1585343,C=78)
	CT(928)=(S=1585344,E=1585407,C=80)
	CT(929)=(S=1585408,E=1585631,C=29)
	CT(930)=(S=1585632,E=1585663,C=78)
	CT(931)=(S=1585664,E=1585695,C=29)
	CT(932)=(S=1585696,E=1585791,C=78)
	CT(933)=(S=1585792,E=1585823,C=29)
	CT(934)=(S=1585824,E=1585855,C=78)
	CT(935)=(S=1585856,E=1585919,C=80)
	CT(936)=(S=1585920,E=1585951,C=78)
	CT(937)=(S=1585952,E=1585983,C=29)
	CT(938)=(S=1585984,E=1586047,C=81)
	CT(939)=(S=1586048,E=1586143,C=29)
	CT(940)=(S=1586144,E=1586175,C=78)
	CT(941)=(S=1586176,E=1586367,C=29)
	CT(942)=(S=1586368,E=1586495,C=78)
	CT(943)=(S=1586496,E=1586511,C=29)
	CT(944)=(S=1586512,E=1586559,C=78)
	CT(945)=(S=1586560,E=1586623,C=29)
	CT(946)=(S=1586624,E=1586783,C=78)
	CT(947)=(S=1586784,E=1586879,C=29)
	CT(948)=(S=1586880,E=1586943,C=80)
	CT(949)=(S=1586944,E=1587007,C=29)
	CT(950)=(S=1587008,E=1587071,C=80)
	CT(951)=(S=1587072,E=1587103,C=29)
	CT(952)=(S=1587104,E=1587135,C=80)
	CT(953)=(S=1587136,E=1587199,C=78)
	CT(954)=(S=1587200,E=1587455,C=29)
	CT(955)=(S=1587456,E=1587711,C=78)
	CT(956)=(S=1587712,E=1589247,C=29)
	CT(957)=(S=1589248,E=1591439,C=78)
	CT(958)=(S=1591440,E=1592239,C=29)
	CT(959)=(S=1592240,E=1592255,C=78)
	CT(960)=(S=1592256,E=1592319,C=29)
	CT(961)=(S=1592320,E=1595391,C=78)
	CT(962)=(S=1595392,E=1595711,C=29)
	CT(963)=(S=1595712,E=1595775,C=78)
	CT(964)=(S=1595776,E=1595839,C=29)
	CT(965)=(S=1595840,E=1595903,C=78)
	CT(966)=(S=1595904,E=1596559,C=29)
	CT(967)=(S=1596560,E=1596575,C=82)
	CT(968)=(S=1596576,E=1598655,C=29)
	CT(969)=(S=1598656,E=1598719,C=78)
	CT(970)=(S=1598720,E=1598975,C=29)
	CT(971)=(S=1598976,E=1599039,C=78)
	CT(972)=(S=1599040,E=1599647,C=29)
	CT(973)=(S=1599648,E=1599679,C=78)
	CT(974)=(S=1599680,E=1599807,C=29)
	CT(975)=(S=1599808,E=1599871,C=78)
	CT(976)=(S=1599872,E=1600511,C=29)
	CT(977)=(S=1600512,E=1601023,C=78)
	CT(978)=(S=1601024,E=1602047,C=29)
	CT(979)=(S=1602048,E=1602303,C=78)
	CT(980)=(S=1602304,E=1604095,C=29)
	CT(981)=(S=1604096,E=1604351,C=78)
	CT(982)=(S=1604352,E=1606079,C=29)
	CT(983)=(S=1606080,E=1606095,C=78)
	CT(984)=(S=1606096,E=1606111,C=29)
	CT(985)=(S=1606112,E=1606127,C=78)
	CT(986)=(S=1606128,E=1606655,C=29)
	CT(987)=(S=1606656,E=1606911,C=36)
	CT(988)=(S=1606912,E=1607167,C=37)
	CT(989)=(S=1607168,E=1607423,C=28)
	CT(990)=(S=1607424,E=1607679,C=46)
	CT(991)=(S=1607680,E=1607967,C=29)
	CT(992)=(S=1607968,E=1608063,C=78)
	CT(993)=(S=1608064,E=1608127,C=29)
	CT(994)=(S=1608128,E=1608159,C=78)
	CT(995)=(S=1608160,E=1608191,C=80)
	CT(996)=(S=1608192,E=1608383,C=78)
	CT(997)=(S=1608384,E=1608447,C=80)
	CT(998)=(S=1608448,E=1608479,C=78)
	CT(999)=(S=1608480,E=1608511,C=29)
	CT(1000)=(S=1608512,E=1608703,C=80)
	CT(1001)=(S=1608704,E=1608927,C=29)
	CT(1002)=(S=1608928,E=1609279,C=78)
	CT(1003)=(S=1609280,E=1609295,C=29)
	CT(1004)=(S=1609296,E=1609311,C=78)
	CT(1005)=(S=1609312,E=1609599,C=29)
	CT(1006)=(S=1609600,E=1609663,C=78)
	CT(1007)=(S=1609664,E=1610239,C=29)
	CT(1008)=(S=1610240,E=1610367,C=78)
	CT(1009)=(S=1610368,E=1611263,C=29)
	CT(1010)=(S=1611264,E=1611519,C=78)
	CT(1011)=(S=1611520,E=1612031,C=29)
	CT(1012)=(S=1612032,E=1612063,C=78)
	CT(1013)=(S=1612064,E=1612127,C=29)
	CT(1014)=(S=1612128,E=1612143,C=78)
	CT(1015)=(S=1612144,E=1612927,C=29)
	CT(1016)=(S=1612928,E=1613055,C=78)
	CT(1017)=(S=1613056,E=1613071,C=29)
	CT(1018)=(S=1613072,E=1613087,C=80)
	CT(1019)=(S=1613088,E=1613119,C=29)
	CT(1020)=(S=1613120,E=1613311,C=78)
	CT(1021)=(S=1613312,E=1616831,C=29)
	CT(1022)=(S=1616832,E=1616895,C=80)
	CT(1023)=(S=1616896,E=1624063,C=29)
	CT(1024)=(S=1624064,E=1625087,C=78)
	CT(1025)=(S=1625088,E=1625231,C=29)
	CT(1026)=(S=1625232,E=1625343,C=78)
	CT(1027)=(S=1625344,E=1625599,C=29)
	CT(1028)=(S=1625600,E=1625631,C=81)
	CT(1029)=(S=1625632,E=1625855,C=29)
	CT(1030)=(S=1625856,E=1625983,C=78)
	CT(1031)=(S=1625984,E=1627135,C=29)
	CT(1032)=(S=1627136,E=1627391,C=78)
	CT(1033)=(S=1627392,E=1627455,C=29)
	CT(1034)=(S=1627456,E=1627487,C=78)
	CT(1035)=(S=1627488,E=1627903,C=29)
	CT(1036)=(S=1627904,E=1628031,C=78)
	CT(1037)=(S=1628032,E=1629695,C=29)
	CT(1038)=(S=1629696,E=1629951,C=78)
	CT(1039)=(S=1629952,E=1630335,C=29)
	CT(1040)=(S=1630336,E=1630463,C=78)
	CT(1041)=(S=1630464,E=1630591,C=29)
	CT(1042)=(S=1630592,E=1630975,C=78)
	CT(1043)=(S=1630976,E=1631007,C=80)
	CT(1044)=(S=1631008,E=1631743,C=29)
	CT(1045)=(S=1631744,E=1631775,C=8)
	CT(1046)=(S=1631776,E=1631935,C=29)
	CT(1047)=(S=1631936,E=1632015,C=78)
	CT(1048)=(S=1632016,E=1632031,C=29)
	CT(1049)=(S=1632032,E=1632063,C=81)
	CT(1050)=(S=1632064,E=1632095,C=78)
	CT(1051)=(S=1632112,E=1632255,C=29)
	CT(1052)=(S=1632256,E=1632511,C=83)
	CT(1053)=(S=1632512,E=1633047,C=29)
	CT(1054)=(S=1633048,E=1633055,C=8)
	CT(1055)=(S=1633056,E=1633087,C=78)
	CT(1056)=(S=1633088,E=1633119,C=29)
	CT(1057)=(S=1633120,E=1633279,C=78)
	CT(1058)=(S=1633280,E=1634047,C=29)
	CT(1059)=(S=1634048,E=1634063,C=78)
	CT(1060)=(S=1634064,E=1635327,C=29)
	CT(1061)=(S=1635328,E=1635423,C=78)
	CT(1062)=(S=1635424,E=1635439,C=29)
	CT(1063)=(S=1635440,E=1635455,C=78)
	CT(1064)=(S=1635456,E=1635519,C=81)
	CT(1065)=(S=1635520,E=1635583,C=78)
	CT(1066)=(S=1635584,E=1635807,C=29)
	CT(1067)=(S=1635808,E=1635823,C=78)
	CT(1068)=(S=1635824,E=1635839,C=29)
	CT(1069)=(S=1635840,E=1635935,C=78)
	CT(1070)=(S=1635936,E=1638399,C=29)
	CT(1071)=(S=1638400,E=1703935,C=14)
	CT(1072)=(S=1703936,E=1769471,C=29)
	CT(1073)=(S=1769472,E=1769475,C=2)
	CT(1074)=(S=1769476,E=1769479,C=5)
	CT(1075)=(S=1769480,E=1769483,C=7)
	CT(1076)=(S=1769484,E=1769487,C=11)
	CT(1077)=(S=1769488,E=1769519,C=2)
	CT(1078)=(S=1769520,E=1769535,C=4)
	CT(1079)=(S=1769536,E=1769567,C=0)
	CT(1080)=(S=1769568,E=1769599,C=84)
	CT(1081)=(S=1769728,E=1769983,C=6)
	CT(1082)=(S=1769984,E=1770495,C=11)
	CT(1083)=(S=1770496,E=1771519,C=4)
	CT(1084)=(S=1771520,E=1777663,C=1)
	CT(1085)=(S=1777664,E=1778175,C=0)
	CT(1086)=(S=1778176,E=1778303,C=75)
	CT(1087)=(S=1778304,E=1778351,C=2)
	CT(1088)=(S=1778352,E=1778367,C=7)
	CT(1089)=(S=1778368,E=1778399,C=2)
	CT(1090)=(S=1778400,E=1778407,C=0)
	CT(1091)=(S=1778408,E=1778415,C=1)
	CT(1092)=(S=1778416,E=1778431,C=4)
	CT(1093)=(S=1778432,E=1778687,C=6)
	CT(1094)=(S=1778688,E=1781759,C=1)
	CT(1095)=(S=1781760,E=1782271,C=4)
	CT(1096)=(S=1782272,E=1782275,C=10)
	CT(1097)=(S=1782276,E=1782279,C=4)
	CT(1098)=(S=1782284,E=1782287,C=2)
	CT(1099)=(S=1782288,E=1782303,C=74)
	CT(1100)=(S=1782304,E=1782311,C=8)
	CT(1101)=(S=1782312,E=1782319,C=1)
	CT(1102)=(S=1782320,E=1782367,C=0)
	CT(1103)=(S=1782368,E=1782399,C=2)
	CT(1104)=(S=1782400,E=1782527,C=1)
	CT(1105)=(S=1782528,E=1783295,C=9)
	CT(1106)=(S=1783296,E=1783359,C=7)
	CT(1107)=(S=1783360,E=1783367,C=0)
	CT(1108)=(S=1783368,E=1783375,C=1)
	CT(1109)=(S=1783376,E=1783391,C=0)
	CT(1110)=(S=1783392,E=1783411,C=2)
	CT(1111)=(S=1783416,E=1783419,C=77)
	CT(1112)=(S=1783420,E=1783423,C=2)
	CT(1113)=(S=1783424,E=1783439,C=0)
	CT(1114)=(S=1783448,E=1783455,C=1)
	CT(1115)=(S=1783456,E=1783487,C=4)
	CT(1116)=(S=1783488,E=1783551,C=1)
	CT(1117)=(S=1783552,E=1783807,C=3)
	CT(1118)=(S=1783808,E=1785855,C=4)
	CT(1119)=(S=1785856,E=1789951,C=11)
	CT(1120)=(S=1789952,E=1794071,C=2)
	CT(1121)=(S=1794072,E=1794079,C=85)
	CT(1122)=(S=1794080,E=1794111,C=2)
	CT(1123)=(S=1794112,E=1794115,C=73)
	CT(1124)=(S=1794116,E=1794131,C=2)
	CT(1125)=(S=1794132,E=1794135,C=4)
	CT(1126)=(S=1794144,E=1794175,C=7)
	CT(1127)=(S=1794176,E=1794239,C=6)
	CT(1128)=(S=1794240,E=1794271,C=0)
	CT(1129)=(S=1794272,E=1794303,C=9)
	CT(1130)=(S=1794304,E=1794559,C=4)
	CT(1131)=(S=1794560,E=1794751,C=2)
	CT(1132)=(S=1794752,E=1794767,C=8)
	CT(1133)=(S=1794768,E=1794815,C=1)
	CT(1134)=(S=1794816,E=1794943,C=0)
	CT(1135)=(S=1794944,E=1795071,C=1)
	CT(1136)=(S=1795072,E=1795075,C=0)
	CT(1137)=(S=1795080,E=1795083,C=2)
	CT(1138)=(S=1795084,E=1795087,C=4)
	CT(1139)=(S=1795088,E=1795095,C=8)
	CT(1140)=(S=1795100,E=1795107,C=2)
	CT(1141)=(S=1795112,E=1795119,C=3)
	CT(1142)=(S=1795120,E=1795135,C=2)
	CT(1143)=(S=1795136,E=1795199,C=9)
	CT(1144)=(S=1795200,E=1795839,C=6)
	CT(1145)=(S=1795840,E=1796095,C=1)
	CT(1146)=(S=1796096,E=1796351,C=7)
	CT(1147)=(S=1796352,E=1796607,C=9)
	CT(1148)=(S=1796608,E=1796735,C=4)
	CT(1149)=(S=1796736,E=1796799,C=1)
	CT(1150)=(S=1796800,E=1796811,C=0)
	CT(1151)=(S=1796812,E=1796815,C=1)
	CT(1152)=(S=1796816,E=1796823,C=2)
	CT(1153)=(S=1796824,E=1796831,C=10)
	CT(1154)=(S=1796832,E=1796863,C=2)
	CT(1155)=(S=1796864,E=1797119,C=4)
	CT(1156)=(S=1797120,E=1797375,C=10)
	CT(1157)=(S=1797376,E=1797407,C=4)
	CT(1158)=(S=1797408,E=1797439,C=1)
	CT(1159)=(S=1797440,E=1797471,C=10)
	CT(1160)=(S=1797472,E=1797479,C=2)
	CT(1161)=(S=1797480,E=1797483,C=7)
	CT(1162)=(S=1797484,E=1797487,C=0)
	CT(1163)=(S=1797488,E=1797495,C=86)
	CT(1164)=(S=1797496,E=1797499,C=2)
	CT(1165)=(S=1797504,E=1797631,C=87)
	CT(1166)=(S=1797632,E=1797695,C=2)
	CT(1167)=(S=1797696,E=1797711,C=8)
	CT(1168)=(S=1797712,E=1797727,C=5)
	CT(1169)=(S=1797728,E=1797751,C=2)
	CT(1170)=(S=1797752,E=1797759,C=73)
	CT(1171)=(S=1797760,E=1797887,C=10)
	CT(1172)=(S=1797888,E=1797895,C=0)
	CT(1173)=(S=1797896,E=1797899,C=86)
	CT(1174)=(S=1797900,E=1797903,C=73)
	CT(1175)=(S=1797904,E=1797919,C=75)
	CT(1176)=(S=1797920,E=1797951,C=74)
	CT(1177)=(S=1797952,E=1797959,C=0)
	CT(1178)=(S=1797964,E=1797967,C=2)
	CT(1179)=(S=1797968,E=1797983,C=0)
	CT(1180)=(S=1797984,E=1798015,C=6)
	CT(1181)=(S=1798016,E=1798047,C=2)
	CT(1182)=(S=1798048,E=1798095,C=8)
	CT(1183)=(S=1798096,E=1798127,C=7)
	CT(1184)=(S=1798128,E=1798143,C=0)
	CT(1185)=(S=1798144,E=1798207,C=1)
	CT(1186)=(S=1798208,E=1798215,C=74)
	CT(1187)=(S=1798216,E=1798219,C=0)
	CT(1188)=(S=1798220,E=1798223,C=74)
	CT(1189)=(S=1798224,E=1798239,C=1)
	CT(1190)=(S=1798240,E=1798247,C=0)
	CT(1191)=(S=1798248,E=1798271,C=2)
	CT(1192)=(S=1798272,E=1798527,C=6)
	CT(1193)=(S=1798528,E=1798591,C=1)
	CT(1194)=(S=1798592,E=1798639,C=2)
	CT(1195)=(S=1798640,E=1798647,C=0)
	CT(1196)=(S=1798648,E=1798651,C=2)
	CT(1197)=(S=1798652,E=1798655,C=4)
	CT(1198)=(S=1798656,E=1798783,C=2)
	CT(1199)=(S=1798784,E=1798847,C=88)
	CT(1200)=(S=1798848,E=1798911,C=7)
	CT(1201)=(S=1798912,E=1799039,C=1)
	CT(1202)=(S=1799040,E=1799167,C=6)
	CT(1203)=(S=1799168,E=1799183,C=2)
	CT(1204)=(S=1799184,E=1799191,C=4)
	CT(1205)=(S=1799192,E=1799199,C=2)
	CT(1206)=(S=1799200,E=1799203,C=7)
	CT(1207)=(S=1799204,E=1799207,C=0)
	CT(1208)=(S=1799212,E=1799215,C=1)
	CT(1209)=(S=1799216,E=1799223,C=4)
	CT(1210)=(S=1799224,E=1799227,C=89)
	CT(1211)=(S=1799228,E=1799231,C=86)
	CT(1212)=(S=1799232,E=1799679,C=6)
	CT(1213)=(S=1799680,E=1799695,C=2)
	CT(1214)=(S=1799696,E=1799711,C=11)
	CT(1215)=(S=1799712,E=1799743,C=4)
	CT(1216)=(S=1799744,E=1800255,C=6)
	CT(1217)=(S=1800256,E=1800511,C=2)
	CT(1218)=(S=1800512,E=1800519,C=0)
	CT(1219)=(S=1800520,E=1800527,C=1)
	CT(1220)=(S=1800528,E=1800535,C=74)
	CT(1221)=(S=1800536,E=1800539,C=73)
	CT(1222)=(S=1800540,E=1800543,C=2)
	CT(1223)=(S=1800544,E=1800547,C=73)
	CT(1224)=(S=1800548,E=1800551,C=4)
	CT(1225)=(S=1800552,E=1800555,C=0)
	CT(1226)=(S=1800556,E=1800559,C=5)
	CT(1227)=(S=1800560,E=1800567,C=0)
	CT(1228)=(S=1800568,E=1800575,C=1)
	CT(1229)=(S=1800576,E=1800703,C=2)
	CT(1230)=(S=1800704,E=1800707,C=90)
	CT(1231)=(S=1800708,E=1800715,C=2)
	CT(1232)=(S=1800716,E=1800719,C=8)
	CT(1233)=(S=1800720,E=1800735,C=76)
	CT(1234)=(S=1800736,E=1800759,C=2)
	CT(1235)=(S=1800760,E=1800763,C=5)
	CT(1236)=(S=1800768,E=1800815,C=2)
	CT(1237)=(S=1800816,E=1800831,C=0)
	CT(1238)=(S=1800832,E=1800959,C=6)
	CT(1239)=(S=1800960,E=1800967,C=74)
	CT(1240)=(S=1800968,E=1800975,C=2)
	CT(1241)=(S=1800976,E=1800979,C=3)
	CT(1242)=(S=1800980,E=1800983,C=73)
	CT(1243)=(S=1800984,E=1800991,C=0)
	CT(1244)=(S=1800992,E=1801023,C=8)
	CT(1245)=(S=1801024,E=1801087,C=4)
	CT(1246)=(S=1801088,E=1801151,C=91)
	CT(1247)=(S=1801152,E=1801167,C=8)
	CT(1248)=(S=1801168,E=1801171,C=2)
	CT(1249)=(S=1801172,E=1801175,C=92)
	CT(1250)=(S=1801176,E=1801179,C=4)
	CT(1251)=(S=1801180,E=1801183,C=74)
	CT(1252)=(S=1801216,E=1801279,C=4)
	CT(1253)=(S=1801280,E=1801295,C=10)
	CT(1254)=(S=1801296,E=1801311,C=74)
	CT(1255)=(S=1801312,E=1801343,C=0)
	CT(1256)=(S=1801344,E=1801599,C=6)
	CT(1257)=(S=1801600,E=1801663,C=7)
	CT(1258)=(S=1801664,E=1801667,C=93)
	CT(1259)=(S=1801668,E=1801671,C=7)
	CT(1260)=(S=1801672,E=1801675,C=4)
	CT(1261)=(S=1801676,E=1801679,C=2)
	CT(1262)=(S=1801680,E=1801695,C=0)
	CT(1263)=(S=1801696,E=1801727,C=5)
	CT(1264)=(S=1801728,E=1801791,C=6)
	CT(1265)=(S=1801792,E=1801871,C=2)
	CT(1266)=(S=1801872,E=1801879,C=0)
	CT(1267)=(S=1801880,E=1801883,C=10)
	CT(1268)=(S=1801884,E=1801887,C=5)
	CT(1269)=(S=1801888,E=1801903,C=2)
	CT(1270)=(S=1801904,E=1801983,C=8)
	CT(1271)=(S=1801984,E=1802175,C=2)
	CT(1272)=(S=1802176,E=1802223,C=0)
	CT(1273)=(S=1802224,E=1802239,C=2)
	CT(1274)=(S=1802240,E=1802751,C=1)
	CT(1275)=(S=1802752,E=1803007,C=3)
	CT(1276)=(S=1803008,E=1803015,C=74)
	CT(1277)=(S=1803016,E=1803019,C=2)
	CT(1278)=(S=1803020,E=1803023,C=84)
	CT(1279)=(S=1803024,E=1803039,C=2)
	CT(1280)=(S=1803040,E=1803071,C=5)
	CT(1281)=(S=1803072,E=1803135,C=0)
	CT(1282)=(S=1803136,E=1803199,C=3)
	CT(1283)=(S=1803200,E=1803215,C=2)
	CT(1284)=(S=1803216,E=1803223,C=4)
	CT(1285)=(S=1803224,E=1803227,C=0)
	CT(1286)=(S=1803228,E=1803231,C=1)
	CT(1287)=(S=1803232,E=1803255,C=2)
	CT(1288)=(S=1803256,E=1803263,C=74)
	CT(1289)=(S=1803264,E=1806335,C=2)
	CT(1290)=(S=1806336,E=1806591,C=1)
	CT(1291)=(S=1806592,E=1806847,C=3)
	CT(1292)=(S=1806848,E=1807103,C=5)
	CT(1293)=(S=1807104,E=1807167,C=9)
	CT(1294)=(S=1807168,E=1807231,C=2)
	CT(1295)=(S=1807232,E=1807359,C=84)
	CT(1296)=(S=1807360,E=1810431,C=1)
	CT(1297)=(S=1810432,E=1816575,C=6)
	CT(1298)=(S=1816576,E=1827839,C=1)
	CT(1299)=(S=1827840,E=1828863,C=2)
	CT(1300)=(S=1828864,E=1830911,C=6)
	CT(1301)=(S=1830912,E=1832959,C=9)
	CT(1302)=(S=1832960,E=1833983,C=4)
	CT(1303)=(S=1833984,E=1834239,C=73)
	CT(1304)=(S=1834240,E=1834367,C=0)
	CT(1305)=(S=1834368,E=1834495,C=2)
	CT(1306)=(S=1834496,E=1834751,C=3)
	CT(1307)=(S=1834752,E=1834815,C=77)
	CT(1308)=(S=1834816,E=1834879,C=6)
	CT(1309)=(S=1834880,E=1835007,C=4)
	CT(1310)=(S=1835008,E=2031615,C=29)
	CT(1311)=(S=2031616,E=2032255,C=58)
	CT(1312)=(S=2032256,E=2032383,C=26)
	CT(1313)=(S=2032384,E=2032391,C=37)
	CT(1314)=(S=2032392,E=2032399,C=36)
	CT(1315)=(S=2032400,E=2032407,C=18)
	CT(1316)=(S=2032408,E=2032415,C=31)
	CT(1317)=(S=2032416,E=2032447,C=71)
	CT(1318)=(S=2032448,E=2032455,C=14)
	CT(1319)=(S=2032456,E=2032463,C=24)
	CT(1320)=(S=2032464,E=2032471,C=28)
	CT(1321)=(S=2032472,E=2032479,C=54)
	CT(1322)=(S=2032480,E=2032495,C=36)
	CT(1323)=(S=2032496,E=2032511,C=25)
	CT(1324)=(S=2032512,E=2032519,C=34)
	CT(1325)=(S=2032520,E=2032527,C=12)
	CT(1326)=(S=2032528,E=2032535,C=28)
	CT(1327)=(S=2032536,E=2032543,C=19)
	CT(1328)=(S=2032544,E=2032575,C=15)
	CT(1329)=(S=2032576,E=2032591,C=31)
	CT(1330)=(S=2032592,E=2032639,C=14)
	CT(1331)=(S=2032640,E=2032895,C=25)
	CT(1332)=(S=2032896,E=2033151,C=42)
	CT(1333)=(S=2033152,E=2033215,C=14)
	CT(1334)=(S=2033216,E=2033223,C=58)
	CT(1335)=(S=2033224,E=2033231,C=14)
	CT(1336)=(S=2033232,E=2033247,C=37)
	CT(1337)=(S=2033248,E=2033279,C=31)
	CT(1338)=(S=2033280,E=2033407,C=58)
	CT(1339)=(S=2033408,E=2033415,C=36)
	CT(1340)=(S=2033416,E=2033423,C=14)
	CT(1341)=(S=2033424,E=2033439,C=47)
	CT(1342)=(S=2033440,E=2033447,C=37)
	CT(1343)=(S=2033448,E=2033455,C=58)
	CT(1344)=(S=2033456,E=2033463,C=63)
	CT(1345)=(S=2033464,E=2033471,C=34)
	CT(1346)=(S=2033472,E=2033551,C=26)
	CT(1347)=(S=2033552,E=2033567,C=15)
	CT(1348)=(S=2033568,E=2033575,C=66)
	CT(1349)=(S=2033576,E=2033583,C=54)
	CT(1350)=(S=2033584,E=2033599,C=28)
	CT(1351)=(S=2033600,E=2033631,C=52)
	CT(1352)=(S=2033632,E=2033647,C=18)
	CT(1353)=(S=2033648,E=2033655,C=33)
	CT(1354)=(S=2033656,E=2033663,C=12)
	CT(1355)=(S=2033664,E=2033919,C=18)
	CT(1356)=(S=2033920,E=2034175,C=30)
	CT(1357)=(S=2034176,E=2034191,C=18)
	CT(1358)=(S=2034192,E=2034207,C=56)
	CT(1359)=(S=2034208,E=2034223,C=14)
	CT(1360)=(S=2034224,E=2034231,C=15)
	CT(1361)=(S=2034232,E=2034239,C=33)
	CT(1362)=(S=2034240,E=2034303,C=18)
	CT(1363)=(S=2034304,E=2034463,C=34)
	CT(1364)=(S=2034464,E=2034471,C=15)
	CT(1365)=(S=2034472,E=2034479,C=38)
	CT(1366)=(S=2034480,E=2034487,C=94)
	CT(1367)=(S=2034488,E=2034495,C=36)
	CT(1368)=(S=2034496,E=2034559,C=54)
	CT(1369)=(S=2034560,E=2034687,C=58)
	CT(1370)=(S=2034688,E=2034695,C=42)
	CT(1371)=(S=2034696,E=2034703,C=15)
	CT(1372)=(S=2034704,E=2034719,C=54)
	CT(1373)=(S=2034720,E=2034751,C=28)
	CT(1374)=(S=2034752,E=2034759,C=49)
	CT(1375)=(S=2034760,E=2034815,C=14)
	CT(1376)=(S=2034816,E=2034943,C=31)
	CT(1377)=(S=2034944,E=2034951,C=14)
	CT(1378)=(S=2034952,E=2034959,C=19)
	CT(1379)=(S=2034960,E=2034967,C=18)
	CT(1380)=(S=2034968,E=2034975,C=63)
	CT(1381)=(S=2034976,E=2035007,C=18)
	CT(1382)=(S=2035008,E=2035071,C=63)
	CT(1383)=(S=2035072,E=2035079,C=18)
	CT(1384)=(S=2035080,E=2035087,C=15)
	CT(1385)=(S=2035088,E=2035095,C=18)
	CT(1386)=(S=2035096,E=2035103,C=14)
	CT(1387)=(S=2035104,E=2035111,C=43)
	CT(1388)=(S=2035112,E=2035119,C=28)
	CT(1389)=(S=2035120,E=2035127,C=18)
	CT(1390)=(S=2035136,E=2035199,C=51)
	CT(1391)=(S=2035200,E=2035210,C=38)
	CT(1392)=(S=2035211,E=2035211,C=36)
	CT(1393)=(S=2035212,E=2035231,C=38)
	CT(1394)=(S=2035232,E=2035232,C=36)
	CT(1395)=(S=2035233,E=2035243,C=38)
	CT(1396)=(S=2035244,E=2035244,C=36)
	CT(1397)=(S=2035245,E=2035245,C=38)
	CT(1398)=(S=2035246,E=2035247,C=28)
	CT(1399)=(S=2035248,E=2035248,C=36)
	CT(1400)=(S=2035249,E=2035259,C=38)
	CT(1401)=(S=2035260,E=2035263,C=25)
	CT(1402)=(S=2035264,E=2035264,C=38)
	CT(1403)=(S=2035265,E=2035267,C=14)
	CT(1404)=(S=2035268,E=2035271,C=38)
	CT(1405)=(S=2035272,E=2035279,C=95)
	CT(1406)=(S=2035280,E=2035295,C=26)
	CT(1407)=(S=2035296,E=2035302,C=38)
	CT(1408)=(S=2035303,E=2035303,C=95)
	CT(1409)=(S=2035304,E=2035311,C=38)
	CT(1410)=(S=2035312,E=2035327,C=26)
	CT(1411)=(S=2035328,E=2035343,C=15)
	CT(1412)=(S=2035344,E=2035359,C=26)
	CT(1413)=(S=2035360,E=2035367,C=38)
	CT(1414)=(S=2035368,E=2035375,C=95)
	CT(1415)=(S=2035376,E=2035379,C=44)
	CT(1416)=(S=2035380,E=2035383,C=38)
	CT(1417)=(S=2035384,E=2035391,C=15)
	CT(1418)=(S=2035392,E=2035399,C=38)
	CT(1419)=(S=2035400,E=2035407,C=25)
	CT(1420)=(S=2035408,E=2035415,C=95)
	CT(1421)=(S=2035416,E=2035439,C=38)
	CT(1422)=(S=2035440,E=2035447,C=95)
	CT(1423)=(S=2035448,E=2035454,C=38)
	CT(1424)=(S=2035455,E=2035455,C=95)
	CT(1425)=(S=2035456,E=2035463,C=25)
	CT(1426)=(S=2035464,E=2035471,C=33)
	CT(1427)=(S=2035472,E=2035479,C=18)
	CT(1428)=(S=2035480,E=2035487,C=12)
	CT(1429)=(S=2035488,E=2035519,C=19)
	CT(1430)=(S=2035520,E=2035527,C=28)
	CT(1431)=(S=2035528,E=2035535,C=15)
	CT(1432)=(S=2035536,E=2035551,C=18)
	CT(1433)=(S=2035552,E=2035559,C=15)
	CT(1434)=(S=2035560,E=2035567,C=14)
	CT(1435)=(S=2035568,E=2035575,C=48)
	CT(1436)=(S=2035576,E=2035583,C=47)
	CT(1437)=(S=2035584,E=2035711,C=52)
	CT(1438)=(S=2035712,E=2036735,C=28)
	CT(1439)=(S=2036736,E=2037247,C=36)
	CT(1440)=(S=2037248,E=2037255,C=14)
	CT(1441)=(S=2037264,E=2037271,C=49)
	CT(1442)=(S=2037272,E=2037279,C=28)
	CT(1443)=(S=2037280,E=2037287,C=18)
	CT(1444)=(S=2037288,E=2037295,C=14)
	CT(1445)=(S=2037296,E=2037311,C=96)
	CT(1446)=(S=2037312,E=2037319,C=28)
	CT(1447)=(S=2037320,E=2037327,C=14)
	CT(1448)=(S=2037328,E=2037335,C=36)
	CT(1449)=(S=2037336,E=2037343,C=32)
	CT(1450)=(S=2037344,E=2037351,C=58)
	CT(1451)=(S=2037352,E=2037359,C=14)
	CT(1452)=(S=2037360,E=2037367,C=22)
	CT(1453)=(S=2037368,E=2037375,C=70)
	CT(1454)=(S=2037376,E=2037503,C=21)
	CT(1455)=(S=2037504,E=2037759,C=18)
	CT(1456)=(S=2037760,E=2037767,C=14)
	CT(1457)=(S=2037768,E=2037775,C=34)
	CT(1458)=(S=2037776,E=2037783,C=50)
	CT(1459)=(S=2037784,E=2037791,C=18)
	CT(1460)=(S=2037792,E=2037799,C=14)
	CT(1461)=(S=2037800,E=2037807,C=25)
	CT(1462)=(S=2037808,E=2037815,C=36)
	CT(1463)=(S=2037816,E=2037823,C=55)
	CT(1464)=(S=2037824,E=2037839,C=19)
	CT(1465)=(S=2037840,E=2037847,C=14)
	CT(1466)=(S=2037848,E=2037855,C=68)
	CT(1467)=(S=2037856,E=2037863,C=28)
	CT(1468)=(S=2037864,E=2037871,C=14)
	CT(1469)=(S=2037872,E=2037879,C=49)
	CT(1470)=(S=2037880,E=2037887,C=25)
	CT(1471)=(S=2037888,E=2037895,C=27)
	CT(1472)=(S=2037896,E=2037903,C=15)
	CT(1473)=(S=2037904,E=2037911,C=28)
	CT(1474)=(S=2037912,E=2037919,C=25)
	CT(1475)=(S=2037920,E=2037927,C=36)
	CT(1476)=(S=2037928,E=2037935,C=28)
	CT(1477)=(S=2037936,E=2037943,C=71)
	CT(1478)=(S=2037944,E=2037951,C=50)
	CT(1479)=(S=2037952,E=2037959,C=60)
	CT(1480)=(S=2037960,E=2037967,C=26)
	CT(1481)=(S=2037968,E=2037975,C=31)
	CT(1482)=(S=2037976,E=2037983,C=14)
	CT(1483)=(S=2037984,E=2037991,C=23)
	CT(1484)=(S=2037992,E=2037999,C=26)
	CT(1485)=(S=2038000,E=2038015,C=12)
	CT(1486)=(S=2038016,E=2038031,C=14)
	CT(1487)=(S=2038032,E=2038039,C=24)
	CT(1488)=(S=2038040,E=2038047,C=18)
	CT(1489)=(S=2038048,E=2038055,C=15)
	CT(1490)=(S=2038056,E=2038071,C=28)
	CT(1491)=(S=2038072,E=2038079,C=36)
	CT(1492)=(S=2038080,E=2038087,C=19)
	CT(1493)=(S=2038088,E=2038095,C=43)
	CT(1494)=(S=2038096,E=2038103,C=12)
	CT(1495)=(S=2038104,E=2038111,C=26)
	CT(1496)=(S=2038112,E=2038119,C=36)
	CT(1497)=(S=2038120,E=2038127,C=26)
	CT(1498)=(S=2038128,E=2038135,C=17)
	CT(1499)=(S=2038136,E=2038143,C=34)
	CT(1500)=(S=2038144,E=2038151,C=26)
	CT(1501)=(S=2038152,E=2038159,C=35)
	CT(1502)=(S=2038160,E=2038167,C=47)
	CT(1503)=(S=2038168,E=2038175,C=28)
	CT(1504)=(S=2038176,E=2038183,C=14)
	CT(1505)=(S=2038184,E=2038191,C=37)
	CT(1506)=(S=2038192,E=2038199,C=25)
	CT(1507)=(S=2038200,E=2038215,C=14)
	CT(1508)=(S=2038216,E=2038223,C=18)
	CT(1509)=(S=2038224,E=2038231,C=25)
	CT(1510)=(S=2038232,E=2038239,C=27)
	CT(1511)=(S=2038240,E=2038247,C=18)
	CT(1512)=(S=2038248,E=2038255,C=26)
	CT(1513)=(S=2038256,E=2038263,C=18)
	CT(1514)=(S=2038264,E=2038271,C=58)
	CT(1515)=(S=2038272,E=2038783,C=15)
	CT(1516)=(S=2038784,E=2038847,C=18)
	CT(1517)=(S=2038848,E=2038879,C=14)
	CT(1518)=(S=2038880,E=2038911,C=18)
	CT(1519)=(S=2038912,E=2038943,C=33)
	CT(1520)=(S=2038944,E=2038975,C=31)
	CT(1521)=(S=2038976,E=2039007,C=18)
	CT(1522)=(S=2039008,E=2039039,C=31)
	CT(1523)=(S=2039040,E=2039071,C=62)
	CT(1524)=(S=2039072,E=2039103,C=26)
	CT(1525)=(S=2039104,E=2039135,C=16)
	CT(1526)=(S=2039136,E=2039167,C=12)
	CT(1527)=(S=2039168,E=2039295,C=18)
	CT(1528)=(S=2039296,E=2039551,C=33)
	CT(1529)=(S=2039552,E=2039583,C=53)
	CT(1530)=(S=2039584,E=2039591,C=36)
	CT(1531)=(S=2039592,E=2039599,C=15)
	CT(1532)=(S=2039600,E=2039615,C=34)
	CT(1533)=(S=2039616,E=2039623,C=18)
	CT(1534)=(S=2039624,E=2039631,C=33)
	CT(1535)=(S=2039632,E=2039639,C=24)
	CT(1536)=(S=2039640,E=2039647,C=25)
	CT(1537)=(S=2039648,E=2039679,C=31)
	CT(1538)=(S=2039680,E=2039711,C=49)
	CT(1539)=(S=2039712,E=2039719,C=19)
	CT(1540)=(S=2039720,E=2039727,C=58)
	CT(1541)=(S=2039728,E=2039743,C=67)
	CT(1542)=(S=2039744,E=2039759,C=18)
	CT(1543)=(S=2039760,E=2039767,C=28)
	CT(1544)=(S=2039768,E=2039775,C=20)
	CT(1545)=(S=2039776,E=2039807,C=33)
	CT(1546)=(S=2039808,E=2041855,C=12)
	CT(1547)=(S=2041856,E=2041863,C=26)
	CT(1548)=(S=2041864,E=2041871,C=18)
	CT(1549)=(S=2041872,E=2041879,C=31)
	CT(1550)=(S=2041880,E=2041959,C=18)
	CT(1551)=(S=2041960,E=2041967,C=31)
	CT(1552)=(S=2041968,E=2041983,C=18)
	CT(1553)=(S=2041984,E=2042047,C=31)
	CT(1554)=(S=2042112,E=2042119,C=18)
	CT(1555)=(S=2042120,E=2042127,C=31)
	CT(1556)=(S=2042128,E=2042135,C=51)
	CT(1557)=(S=2042136,E=2042143,C=58)
	CT(1558)=(S=2042144,E=2042159,C=18)
	CT(1559)=(S=2042160,E=2042167,C=31)
	CT(1560)=(S=2042168,E=2042175,C=18)
	CT(1561)=(S=2042176,E=2042191,C=31)
	CT(1562)=(S=2042192,E=2042199,C=58)
	CT(1563)=(S=2042200,E=2042207,C=31)
	CT(1564)=(S=2042208,E=2042215,C=18)
	CT(1565)=(S=2042216,E=2042231,C=31)
	CT(1566)=(S=2042232,E=2042239,C=18)
	CT(1567)=(S=2042240,E=2042247,C=31)
	CT(1568)=(S=2042248,E=2042263,C=58)
	CT(1569)=(S=2042264,E=2042271,C=62)
	CT(1570)=(S=2042272,E=2042279,C=18)
	CT(1571)=(S=2042280,E=2042295,C=58)
	CT(1572)=(S=2042296,E=2042311,C=18)
	CT(1573)=(S=2042312,E=2042319,C=33)
	CT(1574)=(S=2042320,E=2042327,C=58)
	CT(1575)=(S=2042328,E=2042335,C=31)
	CT(1576)=(S=2042336,E=2042343,C=18)
	CT(1577)=(S=2042344,E=2042351,C=58)
	CT(1578)=(S=2042352,E=2042367,C=18)
	CT(1579)=(S=2042368,E=2042399,C=58)
	CT(1580)=(S=2042400,E=2042415,C=18)
	CT(1581)=(S=2042416,E=2042447,C=31)
	CT(1582)=(S=2042448,E=2042463,C=60)
	CT(1583)=(S=2042464,E=2042479,C=18)
	CT(1584)=(S=2042480,E=2042495,C=31)
	CT(1585)=(S=2042496,E=2042527,C=38)
	CT(1586)=(S=2042528,E=2042559,C=31)
	CT(1587)=(S=2042560,E=2042623,C=18)
	CT(1588)=(S=2042624,E=2042815,C=31)
	CT(1589)=(S=2042816,E=2042847,C=18)
	CT(1590)=(S=2042848,E=2042879,C=31)
	CT(1591)=(S=2042880,E=2042895,C=18)
	CT(1592)=(S=2042896,E=2042911,C=15)
	CT(1593)=(S=2042912,E=2042927,C=34)
	CT(1594)=(S=2042928,E=2042943,C=18)
	CT(1595)=(S=2042944,E=2042959,C=96)
	CT(1596)=(S=2042960,E=2042975,C=18)
	CT(1597)=(S=2042976,E=2042991,C=44)
	CT(1598)=(S=2042992,E=2043007,C=15)
	CT(1599)=(S=2043008,E=2043023,C=17)
	CT(1600)=(S=2043024,E=2043039,C=25)
	CT(1601)=(S=2043040,E=2043055,C=15)
	CT(1602)=(S=2043056,E=2043071,C=18)
	CT(1603)=(S=2043072,E=2043087,C=37)
	CT(1604)=(S=2043088,E=2043103,C=12)
	CT(1605)=(S=2043104,E=2043119,C=19)
	CT(1606)=(S=2043120,E=2043135,C=18)
	CT(1607)=(S=2043136,E=2043263,C=27)
	CT(1608)=(S=2043264,E=2043391,C=57)
	CT(1609)=(S=2043392,E=2043647,C=50)
	CT(1610)=(S=2043648,E=2043679,C=59)
	CT(1611)=(S=2043680,E=2043711,C=26)
	CT(1612)=(S=2043712,E=2043719,C=47)
	CT(1613)=(S=2043720,E=2043727,C=25)
	CT(1614)=(S=2043728,E=2043735,C=28)
	CT(1615)=(S=2043736,E=2043743,C=16)
	CT(1616)=(S=2043744,E=2043759,C=33)
	CT(1617)=(S=2043760,E=2043839,C=18)
	CT(1618)=(S=2043840,E=2043847,C=66)
	CT(1619)=(S=2043848,E=2043855,C=19)
	CT(1620)=(S=2043856,E=2043863,C=38)
	CT(1621)=(S=2043864,E=2043871,C=14)
	CT(1622)=(S=2043872,E=2043903,C=28)
	CT(1623)=(S=2043904,E=2045951,C=14)
	CT(1624)=(S=2045952,E=2046975,C=26)
	CT(1625)=(S=2046976,E=2047999,C=58)
	CT(1626)=(S=2048000,E=2064383,C=14)
	CT(1627)=(S=2064384,E=2064415,C=58)
	CT(1628)=(S=2064416,E=2064447,C=18)
	CT(1629)=(S=2064448,E=2064511,C=31)
	CT(1630)=(S=2064512,E=2064543,C=18)
	CT(1631)=(S=2064544,E=2064575,C=31)
	CT(1632)=(S=2064576,E=2064607,C=18)
	CT(1633)=(S=2064608,E=2064639,C=31)
	CT(1634)=(S=2064640,E=2064703,C=18)
	CT(1635)=(S=2064704,E=2064767,C=31)
	CT(1636)=(S=2064768,E=2064799,C=18)
	CT(1637)=(S=2064800,E=2064831,C=31)
	CT(1638)=(S=2064832,E=2064863,C=18)
	CT(1639)=(S=2064864,E=2064895,C=31)
	CT(1640)=(S=2064896,E=2064991,C=18)
	CT(1641)=(S=2064992,E=2065007,C=58)
	CT(1642)=(S=2065008,E=2065055,C=18)
	CT(1643)=(S=2065056,E=2065063,C=71)
	CT(1644)=(S=2065064,E=2065071,C=33)
	CT(1645)=(S=2065072,E=2065087,C=26)
	CT(1646)=(S=2065088,E=2065095,C=18)
	CT(1647)=(S=2065096,E=2065103,C=68)
	CT(1648)=(S=2065104,E=2065111,C=18)
	CT(1649)=(S=2065112,E=2065119,C=58)
	CT(1650)=(S=2065120,E=2065135,C=34)
	CT(1651)=(S=2065136,E=2065143,C=38)
	CT(1652)=(S=2065144,E=2065151,C=18)
	CT(1653)=(S=2065152,E=2065159,C=53)
	CT(1654)=(S=2065160,E=2065167,C=38)
	CT(1655)=(S=2065168,E=2065191,C=31)
	CT(1656)=(S=2065192,E=2065199,C=38)
	CT(1657)=(S=2065200,E=2065215,C=58)
	CT(1658)=(S=2065216,E=2065231,C=31)
	CT(1659)=(S=2065232,E=2065247,C=18)
	CT(1660)=(S=2065248,E=2065295,C=31)
	CT(1661)=(S=2065296,E=2065311,C=18)
	CT(1662)=(S=2065312,E=2065319,C=38)
	CT(1663)=(S=2065320,E=2065327,C=58)
	CT(1664)=(S=2065328,E=2065335,C=25)
	CT(1665)=(S=2065336,E=2065375,C=18)
	CT(1666)=(S=2065376,E=2065391,C=71)
	CT(1667)=(S=2065392,E=2065399,C=15)
	CT(1668)=(S=2065400,E=2065407,C=18)
	CT(1669)=(S=2065408,E=2065415,C=14)
	CT(1670)=(S=2065424,E=2065439,C=33)
	CT(1671)=(S=2065472,E=2065479,C=58)
	CT(1672)=(S=2065480,E=2065487,C=18)
	CT(1673)=(S=2065488,E=2065503,C=38)
	CT(1674)=(S=2065504,E=2065599,C=18)
	CT(1675)=(S=2065600,E=2065607,C=38)
	CT(1676)=(S=2065608,E=2065615,C=36)
	CT(1677)=(S=2065616,E=2065663,C=18)
	CT(1678)=(S=2065664,E=2065671,C=58)
	CT(1679)=(S=2065672,E=2065679,C=33)
	CT(1680)=(S=2065680,E=2065687,C=58)
	CT(1681)=(S=2065688,E=2065695,C=38)
	CT(1682)=(S=2065696,E=2065759,C=31)
	CT(1683)=(S=2065768,E=2065775,C=18)
	CT(1684)=(S=2065776,E=2065783,C=31)
	CT(1685)=(S=2065784,E=2065791,C=18)
	CT(1686)=(S=2065792,E=2065855,C=34)
	CT(1687)=(S=2065856,E=2065863,C=38)
	CT(1688)=(S=2065864,E=2065871,C=18)
	CT(1689)=(S=2065872,E=2065887,C=20)
	CT(1690)=(S=2065888,E=2065967,C=18)
	CT(1691)=(S=2065968,E=2065983,C=58)
	CT(1692)=(S=2065984,E=2066015,C=31)
	CT(1693)=(S=2066016,E=2066023,C=97)
	CT(1694)=(S=2066024,E=2066047,C=31)
	CT(1695)=(S=2066048,E=2066119,C=18)
	CT(1696)=(S=2066120,E=2066127,C=36)
	CT(1697)=(S=2066128,E=2066135,C=31)
	CT(1698)=(S=2066136,E=2066143,C=33)
	CT(1699)=(S=2066144,E=2066175,C=18)
	CT(1700)=(S=2066176,E=2066183,C=38)
	CT(1701)=(S=2066184,E=2066191,C=18)
	CT(1702)=(S=2066192,E=2066207,C=58)
	CT(1703)=(S=2066208,E=2066303,C=18)
	CT(1704)=(S=2066304,E=2066335,C=31)
	CT(1705)=(S=2066336,E=2066351,C=58)
	CT(1706)=(S=2066352,E=2066359,C=31)
	CT(1707)=(S=2066360,E=2066367,C=18)
	CT(1708)=(S=2066368,E=2066383,C=58)
	CT(1709)=(S=2066384,E=2066391,C=98)
	CT(1710)=(S=2066392,E=2066399,C=44)
	CT(1711)=(S=2066400,E=2066415,C=18)
	CT(1712)=(S=2066416,E=2066431,C=62)
	CT(1713)=(S=2066432,E=2067455,C=36)
	CT(1714)=(S=2067456,E=2068479,C=37)
	CT(1715)=(S=2068480,E=2068735,C=31)
	CT(1716)=(S=2068736,E=2068991,C=37)
	CT(1717)=(S=2068992,E=2069247,C=40)
	CT(1718)=(S=2069248,E=2069503,C=57)
	CT(1719)=(S=2069504,E=2069759,C=33)
	CT(1720)=(S=2069760,E=2070015,C=36)
	CT(1721)=(S=2070016,E=2070271,C=28)
	CT(1722)=(S=2070272,E=2070527,C=36)
	CT(1723)=(S=2070528,E=2070783,C=22)
	CT(1724)=(S=2070784,E=2071039,C=32)
	CT(1725)=(S=2071040,E=2071295,C=17)
	CT(1726)=(S=2071296,E=2071551,C=37)
	CT(1727)=(S=2071552,E=2072575,C=15)
	CT(1728)=(S=2072576,E=2073087,C=36)
	CT(1729)=(S=2073088,E=2073599,C=18)
	CT(1730)=(S=2073600,E=2074111,C=34)
	CT(1731)=(S=2074112,E=2074623,C=23)
	CT(1732)=(S=2074624,E=2074879,C=17)
	CT(1733)=(S=2074880,E=2074911,C=20)
	CT(1734)=(S=2074912,E=2074919,C=14)
	CT(1735)=(S=2074920,E=2074927,C=12)
	CT(1736)=(S=2074928,E=2074935,C=27)
	CT(1737)=(S=2074936,E=2074943,C=36)
	CT(1738)=(S=2074944,E=2074975,C=37)
	CT(1739)=(S=2074976,E=2074983,C=14)
	CT(1740)=(S=2074984,E=2074991,C=15)
	CT(1741)=(S=2074992,E=2074999,C=14)
	CT(1742)=(S=2075000,E=2075007,C=18)
	CT(1743)=(S=2075008,E=2075135,C=33)
	CT(1744)=(S=2075136,E=2075143,C=63)
	CT(1745)=(S=2075144,E=2075151,C=12)
	CT(1746)=(S=2075152,E=2075159,C=60)
	CT(1747)=(S=2075160,E=2075167,C=19)
	CT(1748)=(S=2075168,E=2075175,C=18)
	CT(1749)=(S=2075176,E=2075183,C=15)
	CT(1750)=(S=2075184,E=2075199,C=26)
	CT(1751)=(S=2075200,E=2075231,C=71)
	CT(1752)=(S=2075240,E=2075247,C=28)
	CT(1753)=(S=2075248,E=2075255,C=18)
	CT(1754)=(S=2075256,E=2075263,C=14)
	CT(1755)=(S=2075264,E=2075295,C=31)
	CT(1756)=(S=2075296,E=2075303,C=29)
	CT(1757)=(S=2075304,E=2075311,C=18)
	CT(1758)=(S=2075312,E=2075319,C=33)
	CT(1759)=(S=2075320,E=2075327,C=18)
	CT(1760)=(S=2075328,E=2075359,C=28)
	CT(1761)=(S=2075360,E=2075519,C=41)
	CT(1762)=(S=2075520,E=2075527,C=36)
	CT(1763)=(S=2075528,E=2075535,C=15)
	CT(1764)=(S=2075536,E=2075543,C=34)
	CT(1765)=(S=2075544,E=2075551,C=96)
	CT(1766)=(S=2075552,E=2075583,C=20)
	CT(1767)=(S=2075584,E=2075591,C=18)
	CT(1768)=(S=2075592,E=2075607,C=36)
	CT(1769)=(S=2075608,E=2075615,C=26)
	CT(1770)=(S=2075616,E=2075631,C=50)
	CT(1771)=(S=2075632,E=2075647,C=34)
	CT(1772)=(S=2075648,E=2075775,C=28)
	CT(1773)=(S=2075776,E=2075783,C=18)
	CT(1774)=(S=2075784,E=2075791,C=31)
	CT(1775)=(S=2075792,E=2075799,C=70)
	CT(1776)=(S=2075800,E=2075807,C=47)
	CT(1777)=(S=2075808,E=2075815,C=12)
	CT(1778)=(S=2075824,E=2075839,C=58)
	CT(1779)=(S=2075840,E=2075871,C=18)
	CT(1780)=(S=2075880,E=2075887,C=19)
	CT(1781)=(S=2075888,E=2075903,C=14)
	CT(1782)=(S=2075904,E=2076159,C=18)
	CT(1783)=(S=2076160,E=2076671,C=58)
	CT(1784)=(S=2076672,E=2076799,C=37)
	CT(1785)=(S=2076800,E=2076927,C=59)
	CT(1786)=(S=2076928,E=2076935,C=18)
	CT(1787)=(S=2076936,E=2076943,C=44)
	CT(1788)=(S=2076944,E=2076951,C=14)
	CT(1789)=(S=2076952,E=2076959,C=33)
	CT(1790)=(S=2076960,E=2076967,C=36)
	CT(1791)=(S=2076968,E=2076975,C=15)
	CT(1792)=(S=2076976,E=2076983,C=25)
	CT(1793)=(S=2076984,E=2076991,C=22)
	CT(1794)=(S=2076992,E=2077023,C=18)
	CT(1795)=(S=2077024,E=2077031,C=25)
	CT(1796)=(S=2077032,E=2077039,C=18)
	CT(1797)=(S=2077040,E=2077047,C=15)
	CT(1798)=(S=2077048,E=2077055,C=28)
	CT(1799)=(S=2077056,E=2077183,C=37)
	CT(1800)=(S=2077184,E=2077695,C=58)
	CT(1801)=(S=2077696,E=2078207,C=18)
	CT(1802)=(S=2078208,E=2078719,C=58)
	CT(1803)=(S=2078720,E=2078783,C=28)
	CT(1804)=(S=2078784,E=2078847,C=36)
	CT(1805)=(S=2078848,E=2078911,C=26)
	CT(1806)=(S=2078912,E=2078991,C=18)
	CT(1807)=(S=2078992,E=2078999,C=15)
	CT(1808)=(S=2079000,E=2079007,C=27)
	CT(1809)=(S=2079008,E=2079039,C=14)
	CT(1810)=(S=2079040,E=2079071,C=24)
	CT(1811)=(S=2079072,E=2079079,C=15)
	CT(1812)=(S=2079080,E=2079087,C=28)
	CT(1813)=(S=2079088,E=2079103,C=59)
	CT(1814)=(S=2079104,E=2079231,C=14)
	CT(1815)=(S=2079232,E=2079263,C=37)
	CT(1816)=(S=2079264,E=2079271,C=63)
	CT(1817)=(S=2079272,E=2079279,C=15)
	CT(1818)=(S=2079280,E=2079287,C=62)
	CT(1819)=(S=2079288,E=2079311,C=18)
	CT(1820)=(S=2079312,E=2079319,C=58)
	CT(1821)=(S=2079320,E=2079327,C=27)
	CT(1822)=(S=2079328,E=2079335,C=18)
	CT(1823)=(S=2079336,E=2079343,C=22)
	CT(1824)=(S=2079344,E=2079351,C=99)
	CT(1825)=(S=2079352,E=2079359,C=28)
	CT(1826)=(S=2079360,E=2079391,C=18)
	CT(1827)=(S=2079392,E=2079407,C=36)
	CT(1828)=(S=2079408,E=2079415,C=43)
	CT(1829)=(S=2079416,E=2079423,C=33)
	CT(1830)=(S=2079424,E=2079455,C=58)
	CT(1831)=(S=2079456,E=2079487,C=14)
	CT(1832)=(S=2079488,E=2079551,C=58)
	CT(1833)=(S=2079552,E=2079579,C=28)
	CT(1834)=(S=2079580,E=2079583,C=12)
	CT(1835)=(S=2079584,E=2079591,C=18)
	CT(1836)=(S=2079592,E=2079599,C=15)
	CT(1837)=(S=2079600,E=2079615,C=28)
	CT(1838)=(S=2079616,E=2079743,C=36)
	CT(1839)=(S=2079744,E=2080767,C=15)
	CT(1840)=(S=2080768,E=2080831,C=40)
	CT(1841)=(S=2080832,E=2080863,C=33)
	CT(1842)=(S=2080864,E=2080871,C=14)
	CT(1843)=(S=2080872,E=2080879,C=18)
	CT(1844)=(S=2080880,E=2080895,C=36)
	CT(1845)=(S=2080896,E=2080959,C=18)
	CT(1846)=(S=2080960,E=2080975,C=19)
	CT(1847)=(S=2080976,E=2080983,C=37)
	CT(1848)=(S=2080984,E=2080991,C=58)
	CT(1849)=(S=2080992,E=2080999,C=19)
	CT(1850)=(S=2081000,E=2081007,C=18)
	CT(1851)=(S=2081008,E=2081015,C=14)
	CT(1852)=(S=2081016,E=2081023,C=62)
	CT(1853)=(S=2081024,E=2081039,C=14)
	CT(1854)=(S=2081040,E=2081055,C=15)
	CT(1855)=(S=2081056,E=2081063,C=100)
	CT(1856)=(S=2081064,E=2081071,C=49)
	CT(1857)=(S=2081072,E=2081079,C=12)
	CT(1858)=(S=2081080,E=2081087,C=15)
	CT(1859)=(S=2081088,E=2081103,C=30)
	CT(1860)=(S=2081104,E=2081119,C=31)
	CT(1861)=(S=2081120,E=2081127,C=58)
	CT(1862)=(S=2081128,E=2081135,C=36)
	CT(1863)=(S=2081136,E=2081143,C=26)
	CT(1864)=(S=2081144,E=2081151,C=18)
	CT(1865)=(S=2081152,E=2081167,C=14)
	CT(1866)=(S=2081168,E=2081183,C=26)
	CT(1867)=(S=2081184,E=2081191,C=42)
	CT(1868)=(S=2081192,E=2081199,C=14)
	CT(1869)=(S=2081200,E=2081207,C=49)
	CT(1870)=(S=2081208,E=2081215,C=24)
	CT(1871)=(S=2081216,E=2081223,C=44)
	CT(1872)=(S=2081224,E=2081231,C=19)
	CT(1873)=(S=2081232,E=2081239,C=34)
	CT(1874)=(S=2081240,E=2081247,C=63)
	CT(1875)=(S=2081248,E=2081255,C=25)
	CT(1876)=(S=2081256,E=2081263,C=24)
	CT(1877)=(S=2081264,E=2081271,C=12)
	CT(1878)=(S=2081272,E=2081279,C=19)
	CT(1879)=(S=2081280,E=2082815,C=15)
	CT(1880)=(S=2082816,E=2082943,C=37)
	CT(1881)=(S=2082944,E=2083007,C=63)
	CT(1882)=(S=2083008,E=2083023,C=18)
	CT(1883)=(S=2083024,E=2083031,C=36)
	CT(1884)=(S=2083032,E=2083055,C=18)
	CT(1885)=(S=2083056,E=2083063,C=25)
	CT(1886)=(S=2083064,E=2083071,C=18)
	CT(1887)=(S=2083072,E=2083327,C=36)
	CT(1888)=(S=2083328,E=2083583,C=31)
	CT(1889)=(S=2083584,E=2083839,C=101)
	CT(1890)=(S=2083840,E=2083903,C=18)
	CT(1891)=(S=2083904,E=2083911,C=28)
	CT(1892)=(S=2083912,E=2083919,C=27)
	CT(1893)=(S=2083920,E=2083927,C=12)
	CT(1894)=(S=2083928,E=2083935,C=70)
	CT(1895)=(S=2083936,E=2083951,C=18)
	CT(1896)=(S=2083952,E=2083967,C=28)
	CT(1897)=(S=2083968,E=2083999,C=36)
	CT(1898)=(S=2084000,E=2084031,C=18)
	CT(1899)=(S=2084032,E=2084095,C=102)
	CT(1900)=(S=2084096,E=2084351,C=14)
	CT(1901)=(S=2084352,E=2084607,C=37)
	CT(1902)=(S=2084608,E=2084671,C=33)
	CT(1903)=(S=2084672,E=2084687,C=14)
	CT(1904)=(S=2084688,E=2084695,C=37)
	CT(1905)=(S=2084696,E=2084703,C=25)
	CT(1906)=(S=2084704,E=2084735,C=15)
	CT(1907)=(S=2084736,E=2084863,C=18)
	CT(1908)=(S=2084864,E=2085183,C=19)
	CT(1909)=(S=2085184,E=2085199,C=44)
	CT(1910)=(S=2085200,E=2085215,C=28)
	CT(1911)=(S=2085216,E=2085231,C=32)
	CT(1912)=(S=2085232,E=2085247,C=28)
	CT(1913)=(S=2085248,E=2085255,C=49)
	CT(1914)=(S=2085256,E=2085279,C=48)
	CT(1915)=(S=2085280,E=2085295,C=28)
	CT(1916)=(S=2085296,E=2085303,C=19)
	CT(1917)=(S=2085304,E=2085311,C=28)
	CT(1918)=(S=2085312,E=2085375,C=48)
	CT(1919)=(S=2085376,E=2085383,C=12)
	CT(1920)=(S=2085384,E=2085391,C=50)
	CT(1921)=(S=2085400,E=2085407,C=14)
	CT(1922)=(S=2085408,E=2085503,C=37)
	CT(1923)=(S=2085504,E=2085511,C=14)
	CT(1924)=(S=2085512,E=2085519,C=18)
	CT(1925)=(S=2085528,E=2085535,C=37)
	CT(1926)=(S=2085536,E=2085543,C=28)
	CT(1927)=(S=2085544,E=2085551,C=18)
	CT(1928)=(S=2085552,E=2085567,C=17)
	CT(1929)=(S=2085568,E=2085599,C=18)
	CT(1930)=(S=2085600,E=2085607,C=25)
	CT(1931)=(S=2085608,E=2085623,C=14)
	CT(1932)=(S=2085624,E=2085631,C=19)
	CT(1933)=(S=2085632,E=2085759,C=18)
	CT(1934)=(S=2085760,E=2085791,C=51)
	CT(1935)=(S=2085792,E=2085799,C=14)
	CT(1936)=(S=2085800,E=2085807,C=27)
	CT(1937)=(S=2085808,E=2085815,C=28)
	CT(1938)=(S=2085816,E=2085823,C=25)
	CT(1939)=(S=2085824,E=2085887,C=19)
	CT(1940)=(S=2085888,E=2086399,C=28)
	CT(1941)=(S=2086400,E=2086527,C=101)
	CT(1942)=(S=2086528,E=2086528,C=28)
	CT(1943)=(S=2086529,E=2086529,C=37)
	CT(1944)=(S=2086530,E=2086531,C=28)
	CT(1945)=(S=2086532,E=2086533,C=26)
	CT(1946)=(S=2086534,E=2086536,C=28)
	CT(1947)=(S=2086537,E=2086538,C=58)
	CT(1948)=(S=2086539,E=2086539,C=28)
	CT(1949)=(S=2086540,E=2086540,C=58)
	CT(1950)=(S=2086541,E=2086544,C=28)
	CT(1951)=(S=2086545,E=2086545,C=58)
	CT(1952)=(S=2086546,E=2086547,C=26)
	CT(1953)=(S=2086548,E=2086551,C=28)
	CT(1954)=(S=2086552,E=2086552,C=37)
	CT(1955)=(S=2086553,E=2086553,C=28)
	CT(1956)=(S=2086554,E=2086554,C=26)
	CT(1957)=(S=2086555,E=2086567,C=28)
	CT(1958)=(S=2086568,E=2086571,C=26)
	CT(1959)=(S=2086572,E=2086575,C=28)
	CT(1960)=(S=2086576,E=2086591,C=25)
	CT(1961)=(S=2086592,E=2086599,C=28)
	CT(1962)=(S=2086600,E=2086601,C=26)
	CT(1963)=(S=2086602,E=2086627,C=28)
	CT(1964)=(S=2086628,E=2086631,C=26)
	CT(1965)=(S=2086632,E=2086655,C=28)
	CT(1966)=(S=2086656,E=2086911,C=16)
	CT(1967)=(S=2086912,E=2086927,C=14)
	CT(1968)=(S=2086928,E=2086943,C=12)
	CT(1969)=(S=2086944,E=2086951,C=19)
	CT(1970)=(S=2086952,E=2086959,C=34)
	CT(1971)=(S=2086960,E=2086967,C=14)
	CT(1972)=(S=2086968,E=2086975,C=58)
	CT(1973)=(S=2086976,E=2087039,C=32)
	CT(1974)=(S=2087040,E=2087047,C=15)
	CT(1975)=(S=2087048,E=2087055,C=19)
	CT(1976)=(S=2087056,E=2087063,C=70)
	CT(1977)=(S=2087064,E=2087071,C=15)
	CT(1978)=(S=2087072,E=2087095,C=18)
	CT(1979)=(S=2087096,E=2087103,C=59)
	CT(1980)=(S=2087104,E=2087135,C=57)
	CT(1981)=(S=2087136,E=2087143,C=19)
	CT(1982)=(S=2087144,E=2087151,C=63)
	CT(1983)=(S=2087152,E=2087167,C=15)
	CT(1984)=(S=2087168,E=2087295,C=57)
	CT(1985)=(S=2087296,E=2087327,C=14)
	CT(1986)=(S=2087328,E=2087359,C=22)
	CT(1987)=(S=2087360,E=2087367,C=47)
	CT(1988)=(S=2087368,E=2087375,C=15)
	CT(1989)=(S=2087376,E=2087383,C=26)
	CT(1990)=(S=2087384,E=2087391,C=42)
	CT(1991)=(S=2087392,E=2087399,C=101)
	CT(1992)=(S=2087400,E=2087407,C=12)
	CT(1993)=(S=2087408,E=2087415,C=63)
	CT(1994)=(S=2087424,E=2087935,C=16)
	CT(1995)=(S=2087936,E=2087951,C=28)
	CT(1996)=(S=2087952,E=2087959,C=44)
	CT(1997)=(S=2087960,E=2087983,C=28)
	CT(1998)=(S=2087984,E=2087999,C=44)
	CT(1999)=(S=2088000,E=2088007,C=28)
	CT(2000)=(S=2088008,E=2088039,C=25)
	CT(2001)=(S=2088040,E=2088047,C=44)
	CT(2002)=(S=2088048,E=2088063,C=28)
	CT(2003)=(S=2088064,E=2088071,C=38)
	CT(2004)=(S=2088072,E=2088079,C=28)
	CT(2005)=(S=2088080,E=2088087,C=58)
	CT(2006)=(S=2088088,E=2088127,C=18)
	CT(2007)=(S=2088128,E=2088319,C=14)
	CT(2008)=(S=2088320,E=2088447,C=25)
	CT(2009)=(S=2088448,E=2088463,C=33)
	CT(2010)=(S=2088464,E=2088471,C=58)
	CT(2011)=(S=2088472,E=2088479,C=34)
	CT(2012)=(S=2088480,E=2088487,C=18)
	CT(2013)=(S=2088488,E=2088495,C=96)
	CT(2014)=(S=2088496,E=2088527,C=14)
	CT(2015)=(S=2088528,E=2088575,C=25)
	CT(2016)=(S=2088576,E=2088639,C=14)
	CT(2017)=(S=2088640,E=2088647,C=12)
	CT(2018)=(S=2088648,E=2088671,C=14)
	CT(2019)=(S=2088672,E=2088703,C=61)
	CT(2020)=(S=2088704,E=2088831,C=37)
	CT(2021)=(S=2088832,E=2088863,C=59)
	CT(2022)=(S=2088864,E=2088879,C=36)
	CT(2023)=(S=2088880,E=2088895,C=43)
	CT(2024)=(S=2088896,E=2088911,C=20)
	CT(2025)=(S=2088912,E=2088927,C=59)
	CT(2026)=(S=2088928,E=2088935,C=31)
	CT(2027)=(S=2088936,E=2088943,C=50)
	CT(2028)=(S=2088944,E=2088959,C=15)
	CT(2029)=(S=2088960,E=2097151,C=28)
	CT(2030)=(S=2097152,E=2359295,C=29)
	CT(2031)=(S=2359296,E=2359299,C=1)
	CT(2032)=(S=2359300,E=2359303,C=75)
	CT(2033)=(S=2359304,E=2359807,C=1)
	CT(2034)=(S=2359808,E=2360319,C=2)
	CT(2035)=(S=2360320,E=2361343,C=1)
	CT(2036)=(S=2361344,E=2363391,C=2)
	CT(2037)=(S=2363392,E=2368799,C=1)
	CT(2038)=(S=2368800,E=2368803,C=8)
	CT(2039)=(S=2368804,E=2368805,C=1)
	CT(2040)=(S=2368806,E=2368806,C=0)
	CT(2041)=(S=2368807,E=2368831,C=1)
	CT(2042)=(S=2368832,E=2368895,C=74)
	CT(2043)=(S=2368896,E=2369023,C=86)
	CT(2044)=(S=2369024,E=2369535,C=6)
	CT(2045)=(S=2369536,E=2372607,C=1)
	CT(2046)=(S=2372608,E=2373631,C=2)
	CT(2047)=(S=2373632,E=2375679,C=1)
	CT(2048)=(S=2375680,E=2383871,C=74)
	CT(2049)=(S=2383872,E=2416639,C=1)
	CT(2050)=(S=2416640,E=2420735,C=9)
	CT(2051)=(S=2420736,E=2422783,C=2)
	CT(2052)=(S=2422784,E=2423807,C=1)
	CT(2053)=(S=2423808,E=2424319,C=75)
	CT(2054)=(S=2424320,E=2424575,C=1)
	CT(2055)=(S=2424832,E=2424839,C=63)
	CT(2056)=(S=2424840,E=2424855,C=36)
	CT(2057)=(S=2424856,E=2424863,C=55)
	CT(2058)=(S=2424864,E=2424895,C=34)
	CT(2059)=(S=2424896,E=2424903,C=102)
	CT(2060)=(S=2424904,E=2424911,C=12)
	CT(2061)=(S=2424912,E=2424927,C=36)
	CT(2062)=(S=2424928,E=2424935,C=14)
	CT(2063)=(S=2424936,E=2424943,C=25)
	CT(2064)=(S=2424944,E=2424951,C=28)
	CT(2065)=(S=2424952,E=2424959,C=18)
	CT(2066)=(S=2424960,E=2425087,C=19)
	CT(2067)=(S=2425088,E=2425175,C=18)
	CT(2068)=(S=2425176,E=2425191,C=14)
	CT(2069)=(S=2425192,E=2425199,C=36)
	CT(2070)=(S=2425200,E=2425215,C=28)
	CT(2071)=(S=2425216,E=2425231,C=18)
	CT(2072)=(S=2425232,E=2425239,C=37)
	CT(2073)=(S=2425240,E=2425247,C=14)
	CT(2074)=(S=2425248,E=2425255,C=28)
	CT(2075)=(S=2425256,E=2425263,C=12)
	CT(2076)=(S=2425264,E=2425271,C=25)
	CT(2077)=(S=2425272,E=2425279,C=47)
	CT(2078)=(S=2425280,E=2425311,C=31)
	CT(2079)=(S=2425312,E=2425319,C=14)
	CT(2080)=(S=2425320,E=2425343,C=12)
	CT(2081)=(S=2425344,E=2425855,C=19)
	CT(2082)=(S=2425856,E=2426367,C=28)
	CT(2083)=(S=2426368,E=2426623,C=22)
	CT(2084)=(S=2426624,E=2426879,C=58)
	CT(2085)=(S=2426880,E=2427007,C=43)
	CT(2086)=(S=2427008,E=2427023,C=59)
	CT(2087)=(S=2427024,E=2427039,C=18)
	CT(2088)=(S=2427040,E=2427071,C=12)
	CT(2089)=(S=2427072,E=2427135,C=58)
	CT(2090)=(S=2427136,E=2427191,C=18)
	CT(2091)=(S=2427192,E=2427199,C=14)
	CT(2092)=(S=2427200,E=2427271,C=18)
	CT(2093)=(S=2427272,E=2427279,C=12)
	CT(2094)=(S=2427280,E=2427295,C=18)
	CT(2095)=(S=2427296,E=2427303,C=36)
	CT(2096)=(S=2427304,E=2427311,C=71)
	CT(2097)=(S=2427312,E=2427319,C=32)
	CT(2098)=(S=2427320,E=2427327,C=28)
	CT(2099)=(S=2427328,E=2427335,C=33)
	CT(2100)=(S=2427336,E=2427343,C=37)
	CT(2101)=(S=2427344,E=2427351,C=50)
	CT(2102)=(S=2427352,E=2427359,C=36)
	CT(2103)=(S=2427360,E=2427375,C=15)
	CT(2104)=(S=2427376,E=2427383,C=18)
	CT(2105)=(S=2427384,E=2427391,C=26)
	CT(2106)=(S=2427392,E=2427455,C=36)
	CT(2107)=(S=2427456,E=2427463,C=28)
	CT(2108)=(S=2427464,E=2427471,C=25)
	CT(2109)=(S=2427472,E=2427487,C=15)
	CT(2110)=(S=2427488,E=2427519,C=28)
	CT(2111)=(S=2427520,E=2428927,C=25)
	CT(2112)=(S=2428928,E=2428959,C=36)
	CT(2113)=(S=2428960,E=2428991,C=23)
	CT(2114)=(S=2428992,E=2428999,C=28)
	CT(2115)=(S=2429000,E=2429007,C=12)
	CT(2116)=(S=2429008,E=2429015,C=18)
	CT(2117)=(S=2429024,E=2429055,C=47)
	CT(2118)=(S=2429056,E=2429183,C=23)
	CT(2119)=(S=2429184,E=2429311,C=68)
	CT(2120)=(S=2429312,E=2429343,C=35)
	CT(2121)=(S=2429344,E=2429359,C=50)
	CT(2122)=(S=2429360,E=2429367,C=20)
	CT(2123)=(S=2429368,E=2429375,C=14)
	CT(2124)=(S=2429376,E=2429391,C=61)
	CT(2125)=(S=2429392,E=2429407,C=36)
	CT(2126)=(S=2429408,E=2429423,C=28)
	CT(2127)=(S=2429424,E=2429431,C=31)
	CT(2128)=(S=2429432,E=2429439,C=19)
	CT(2129)=(S=2429440,E=2429567,C=18)
	CT(2130)=(S=2429576,E=2429583,C=14)
	CT(2131)=(S=2429584,E=2429591,C=63)
	CT(2132)=(S=2429592,E=2429599,C=18)
	CT(2133)=(S=2429600,E=2429615,C=12)
	CT(2134)=(S=2429616,E=2429623,C=29)
	CT(2135)=(S=2429624,E=2429631,C=46)
	CT(2136)=(S=2429632,E=2429663,C=19)
	CT(2137)=(S=2429664,E=2429679,C=52)
	CT(2138)=(S=2429680,E=2429687,C=25)
	CT(2139)=(S=2429688,E=2429703,C=18)
	CT(2140)=(S=2429704,E=2429711,C=49)
	CT(2141)=(S=2429712,E=2429719,C=47)
	CT(2142)=(S=2429720,E=2429727,C=14)
	CT(2143)=(S=2429728,E=2429759,C=18)
	CT(2144)=(S=2429760,E=2429767,C=52)
	CT(2145)=(S=2429768,E=2429775,C=18)
	CT(2146)=(S=2429776,E=2429791,C=26)
	CT(2147)=(S=2429792,E=2429799,C=63)
	CT(2148)=(S=2429800,E=2429807,C=46)
	CT(2149)=(S=2429808,E=2429823,C=17)
	CT(2150)=(S=2429824,E=2429951,C=31)
	CT(2151)=(S=2429952,E=2430975,C=18)
	CT(2152)=(S=2430976,E=2431231,C=28)
	CT(2153)=(S=2431232,E=2431271,C=31)
	CT(2154)=(S=2431272,E=2431275,C=25)
	CT(2155)=(S=2431276,E=2431279,C=14)
	CT(2156)=(S=2431280,E=2431287,C=28)
	CT(2157)=(S=2431288,E=2431295,C=12)
	CT(2158)=(S=2431296,E=2431303,C=28)
	CT(2159)=(S=2431304,E=2431311,C=12)
	CT(2160)=(S=2431312,E=2431319,C=54)
	CT(2161)=(S=2431320,E=2431327,C=14)
	CT(2162)=(S=2431328,E=2431359,C=31)
	CT(2163)=(S=2431360,E=2431487,C=28)
	CT(2164)=(S=2431488,E=2431551,C=41)
	CT(2165)=(S=2431552,E=2431559,C=96)
	CT(2166)=(S=2431560,E=2431567,C=14)
	CT(2167)=(S=2431568,E=2431575,C=96)
	CT(2168)=(S=2431576,E=2431583,C=14)
	CT(2169)=(S=2431584,E=2431591,C=54)
	CT(2170)=(S=2431592,E=2431599,C=14)
	CT(2171)=(S=2431600,E=2431615,C=13)
	CT(2172)=(S=2431616,E=2431631,C=53)
	CT(2173)=(S=2431632,E=2431639,C=17)
	CT(2174)=(S=2431640,E=2431647,C=19)
	CT(2175)=(S=2431648,E=2431655,C=58)
	CT(2176)=(S=2431656,E=2431663,C=66)
	CT(2177)=(S=2431664,E=2431679,C=12)
	CT(2178)=(S=2431680,E=2431687,C=58)
	CT(2179)=(S=2431688,E=2431695,C=28)
	CT(2180)=(S=2431696,E=2431711,C=27)
	CT(2181)=(S=2431712,E=2431719,C=36)
	CT(2182)=(S=2431720,E=2431727,C=18)
	CT(2183)=(S=2431728,E=2431735,C=12)
	CT(2184)=(S=2431736,E=2431743,C=25)
	CT(2185)=(S=2431744,E=2431999,C=26)
	CT(2186)=(S=2432000,E=2432127,C=45)
	CT(2187)=(S=2432128,E=2432135,C=27)
	CT(2188)=(S=2432136,E=2432151,C=24)
	CT(2189)=(S=2432152,E=2432159,C=58)
	CT(2190)=(S=2432160,E=2432191,C=18)
	CT(2191)=(S=2432192,E=2432255,C=21)
	CT(2192)=(S=2432256,E=2432383,C=18)
	CT(2193)=(S=2432384,E=2432511,C=25)
	CT(2194)=(S=2432512,E=2433023,C=58)
	CT(2195)=(S=2433024,E=2433071,C=26)
	CT(2196)=(S=2433072,E=2433079,C=27)
	CT(2197)=(S=2433080,E=2433087,C=12)
	CT(2198)=(S=2433088,E=2433103,C=41)
	CT(2199)=(S=2433104,E=2433135,C=15)
	CT(2200)=(S=2433136,E=2433151,C=26)
	CT(2201)=(S=2433152,E=2433279,C=22)
	CT(2202)=(S=2433280,E=2433535,C=47)
	CT(2203)=(S=2433536,E=2433567,C=37)
	CT(2204)=(S=2433568,E=2433583,C=15)
	CT(2205)=(S=2433584,E=2433599,C=36)
	CT(2206)=(S=2433600,E=2433607,C=25)
	CT(2207)=(S=2433608,E=2433615,C=15)
	CT(2208)=(S=2433616,E=2433631,C=43)
	CT(2209)=(S=2433632,E=2433663,C=53)
	CT(2210)=(S=2433664,E=2433791,C=101)
	CT(2211)=(S=2433792,E=2433799,C=12)
	CT(2212)=(S=2433800,E=2433807,C=46)
	CT(2213)=(S=2433808,E=2433823,C=47)
	CT(2214)=(S=2433824,E=2433855,C=38)
	CT(2215)=(S=2433856,E=2433863,C=46)
	CT(2216)=(S=2433864,E=2433871,C=14)
	CT(2217)=(S=2433872,E=2433879,C=47)
	CT(2218)=(S=2433880,E=2433887,C=14)
	CT(2219)=(S=2433888,E=2433895,C=24)
	CT(2220)=(S=2433896,E=2433919,C=34)
	CT(2221)=(S=2433920,E=2434047,C=25)
	CT(2222)=(S=2434048,E=2435071,C=101)
	CT(2223)=(S=2435072,E=2435583,C=45)
	CT(2224)=(S=2435584,E=2436095,C=23)
	CT(2225)=(S=2436096,E=2436103,C=28)
	CT(2226)=(S=2436104,E=2436111,C=36)
	CT(2227)=(S=2436112,E=2436127,C=33)
	CT(2228)=(S=2436128,E=2436135,C=61)
	CT(2229)=(S=2436136,E=2436143,C=18)
	CT(2230)=(S=2436144,E=2436151,C=47)
	CT(2231)=(S=2436152,E=2436159,C=26)
	CT(2232)=(S=2436160,E=2436223,C=68)
	CT(2233)=(S=2436224,E=2436287,C=27)
	CT(2234)=(S=2436288,E=2436351,C=18)
	CT(2235)=(S=2436352,E=2436607,C=68)
	CT(2236)=(S=2436608,E=2436623,C=14)
	CT(2237)=(S=2436624,E=2436631,C=63)
	CT(2238)=(S=2436632,E=2436639,C=14)
	CT(2239)=(S=2436640,E=2436655,C=17)
	CT(2240)=(S=2436656,E=2436671,C=18)
	CT(2241)=(S=2436672,E=2436679,C=50)
	CT(2242)=(S=2436680,E=2436687,C=25)
	CT(2243)=(S=2436688,E=2436695,C=33)
	CT(2244)=(S=2436696,E=2436703,C=25)
	CT(2245)=(S=2436704,E=2436711,C=14)
	CT(2246)=(S=2436712,E=2436719,C=40)
	CT(2247)=(S=2436720,E=2436735,C=19)
	CT(2248)=(S=2436736,E=2436743,C=18)
	CT(2249)=(S=2436744,E=2436751,C=36)
	CT(2250)=(S=2436760,E=2436767,C=25)
	CT(2251)=(S=2436768,E=2436799,C=19)
	CT(2252)=(S=2436800,E=2436807,C=36)
	CT(2253)=(S=2436808,E=2436815,C=14)
	CT(2254)=(S=2436816,E=2436823,C=33)
	CT(2255)=(S=2436824,E=2436863,C=31)
	CT(2256)=(S=2436864,E=2437119,C=58)
	CT(2257)=(S=2437120,E=2437183,C=33)
	CT(2258)=(S=2437184,E=2437247,C=36)
	CT(2259)=(S=2437248,E=2437343,C=30)
	CT(2260)=(S=2437344,E=2437351,C=14)
	CT(2261)=(S=2437352,E=2437359,C=57)
	CT(2262)=(S=2437360,E=2437367,C=14)
	CT(2263)=(S=2437368,E=2437375,C=18)
	CT(2264)=(S=2437376,E=2437503,C=28)
	CT(2265)=(S=2437504,E=2437519,C=24)
	CT(2266)=(S=2437520,E=2437527,C=26)
	CT(2267)=(S=2437528,E=2437535,C=28)
	CT(2268)=(S=2437536,E=2437599,C=18)
	CT(2269)=(S=2437600,E=2437607,C=36)
	CT(2270)=(S=2437608,E=2437615,C=12)
	CT(2271)=(S=2437616,E=2437631,C=19)
	CT(2272)=(S=2437632,E=2438143,C=28)
	CT(2273)=(S=2438144,E=2439167,C=31)
	CT(2274)=(S=2439168,E=2439423,C=23)
	CT(2275)=(S=2439424,E=2439679,C=31)
	CT(2276)=(S=2439680,E=2439695,C=71)
	CT(2277)=(S=2439696,E=2439703,C=37)
	CT(2278)=(S=2439704,E=2439711,C=14)
	CT(2279)=(S=2439712,E=2439719,C=18)
	CT(2280)=(S=2439720,E=2439727,C=49)
	CT(2281)=(S=2439728,E=2439743,C=28)
	CT(2282)=(S=2439744,E=2439807,C=36)
	CT(2283)=(S=2439808,E=2440191,C=12)
	CT(2284)=(S=2440192,E=2440199,C=28)
	CT(2285)=(S=2440200,E=2440207,C=12)
	CT(2286)=(S=2440208,E=2440215,C=18)
	CT(2287)=(S=2440216,E=2440223,C=58)
	CT(2288)=(S=2440224,E=2440231,C=15)
	CT(2289)=(S=2440232,E=2440239,C=17)
	CT(2290)=(S=2440240,E=2440255,C=12)
	CT(2291)=(S=2440256,E=2440319,C=14)
	CT(2292)=(S=2440320,E=2440327,C=57)
	CT(2293)=(S=2440328,E=2440335,C=51)
	CT(2294)=(S=2440336,E=2440343,C=43)
	CT(2295)=(S=2440344,E=2440351,C=12)
	CT(2296)=(S=2440352,E=2440359,C=24)
	CT(2297)=(S=2440360,E=2440367,C=28)
	CT(2298)=(S=2440368,E=2440375,C=18)
	CT(2299)=(S=2440376,E=2440383,C=12)
	CT(2300)=(S=2440384,E=2440391,C=36)
	CT(2301)=(S=2440392,E=2440399,C=28)
	CT(2302)=(S=2440400,E=2440415,C=18)
	CT(2303)=(S=2440416,E=2440447,C=51)
	CT(2304)=(S=2440448,E=2440575,C=41)
	CT(2305)=(S=2440576,E=2440591,C=28)
	CT(2306)=(S=2440592,E=2440607,C=25)
	CT(2307)=(S=2440608,E=2440623,C=71)
	CT(2308)=(S=2440624,E=2440631,C=18)
	CT(2309)=(S=2440632,E=2440639,C=42)
	CT(2310)=(S=2440640,E=2440671,C=28)
	CT(2311)=(S=2440672,E=2440679,C=17)
	CT(2312)=(S=2440680,E=2440687,C=14)
	CT(2313)=(S=2440688,E=2440695,C=12)
	CT(2314)=(S=2440696,E=2440703,C=25)
	CT(2315)=(S=2440704,E=2440959,C=49)
	CT(2316)=(S=2440960,E=2441087,C=51)
	CT(2317)=(S=2441088,E=2441215,C=26)
	CT(2318)=(S=2441216,E=2443263,C=12)
	CT(2319)=(S=2443264,E=2443295,C=25)
	CT(2320)=(S=2443296,E=2443303,C=15)
	CT(2321)=(S=2443304,E=2443311,C=31)
	CT(2322)=(S=2443312,E=2443327,C=37)
	CT(2323)=(S=2443328,E=2443359,C=18)
	CT(2324)=(S=2443360,E=2443375,C=36)
	CT(2325)=(S=2443376,E=2443383,C=14)
	CT(2326)=(S=2443384,E=2443391,C=58)
	CT(2327)=(S=2443392,E=2443407,C=41)
	CT(2328)=(S=2443408,E=2443415,C=28)
	CT(2329)=(S=2443416,E=2443423,C=14)
	CT(2330)=(S=2443424,E=2443431,C=49)
	CT(2331)=(S=2443432,E=2443439,C=58)
	CT(2332)=(S=2443440,E=2443447,C=18)
	CT(2333)=(S=2443448,E=2443455,C=55)
	CT(2334)=(S=2443456,E=2443519,C=12)
	CT(2335)=(S=2443520,E=2443775,C=31)
	CT(2336)=(S=2443776,E=2444031,C=36)
	CT(2337)=(S=2444032,E=2444039,C=18)
	CT(2338)=(S=2444040,E=2444047,C=37)
	CT(2339)=(S=2444048,E=2444063,C=53)
	CT(2340)=(S=2444064,E=2444095,C=72)
	CT(2341)=(S=2444096,E=2444159,C=53)
	CT(2342)=(S=2444160,E=2444167,C=40)
	CT(2343)=(S=2444168,E=2444175,C=42)
	CT(2344)=(S=2444176,E=2444183,C=61)
	CT(2345)=(S=2444184,E=2444191,C=36)
	CT(2346)=(S=2444192,E=2444223,C=24)
	CT(2347)=(S=2444224,E=2444239,C=18)
	CT(2348)=(S=2444240,E=2444247,C=43)
	CT(2349)=(S=2444248,E=2444255,C=31)
	CT(2350)=(S=2444256,E=2444263,C=36)
	CT(2351)=(S=2444264,E=2444271,C=14)
	CT(2352)=(S=2444272,E=2444279,C=26)
	CT(2353)=(S=2444280,E=2444287,C=18)
	CT(2354)=(S=2444288,E=2444415,C=50)
	CT(2355)=(S=2444416,E=2444479,C=18)
	CT(2356)=(S=2444480,E=2444511,C=43)
	CT(2357)=(S=2444512,E=2444543,C=23)
	CT(2358)=(S=2444544,E=2444575,C=37)
	CT(2359)=(S=2444576,E=2444591,C=12)
	CT(2360)=(S=2444592,E=2444599,C=35)
	CT(2361)=(S=2444600,E=2444607,C=36)
	CT(2362)=(S=2444608,E=2444623,C=35)
	CT(2363)=(S=2444624,E=2444639,C=36)
	CT(2364)=(S=2444640,E=2444647,C=15)
	CT(2365)=(S=2444648,E=2444655,C=18)
	CT(2366)=(S=2444656,E=2444671,C=15)
	CT(2367)=(S=2444672,E=2444679,C=18)
	CT(2368)=(S=2444680,E=2444687,C=15)
	CT(2369)=(S=2444688,E=2444695,C=14)
	CT(2370)=(S=2444696,E=2444703,C=58)
	CT(2371)=(S=2444704,E=2444711,C=15)
	CT(2372)=(S=2444712,E=2444719,C=46)
	CT(2373)=(S=2444720,E=2444743,C=14)
	CT(2374)=(S=2444744,E=2444751,C=28)
	CT(2375)=(S=2444752,E=2444767,C=15)
	CT(2376)=(S=2444768,E=2444799,C=33)
	CT(2377)=(S=2444800,E=2445311,C=18)
	CT(2378)=(S=2445312,E=2449407,C=28)
	CT(2379)=(S=2449408,E=2449727,C=24)
	CT(2380)=(S=2449728,E=2449775,C=12)
	CT(2381)=(S=2449776,E=2449787,C=38)
	CT(2382)=(S=2449788,E=2449791,C=25)
	CT(2383)=(S=2449792,E=2449919,C=36)
	CT(2384)=(S=2449920,E=2450047,C=26)
	CT(2385)=(S=2450048,E=2450063,C=47)
	CT(2386)=(S=2450064,E=2450071,C=14)
	CT(2387)=(S=2450072,E=2450079,C=103)
	CT(2388)=(S=2450080,E=2450111,C=18)
	CT(2389)=(S=2450112,E=2450119,C=42)
	CT(2390)=(S=2450120,E=2450127,C=26)
	CT(2391)=(S=2450128,E=2450143,C=58)
	CT(2392)=(S=2450144,E=2450151,C=35)
	CT(2393)=(S=2450152,E=2450159,C=13)
	CT(2394)=(S=2450160,E=2450175,C=18)
	CT(2395)=(S=2450176,E=2450303,C=20)
	CT(2396)=(S=2450304,E=2450367,C=23)
	CT(2397)=(S=2450368,E=2450375,C=22)
	CT(2398)=(S=2450376,E=2450383,C=28)
	CT(2399)=(S=2450384,E=2450391,C=15)
	CT(2400)=(S=2450392,E=2450399,C=18)
	CT(2401)=(S=2450400,E=2451455,C=15)
	CT(2402)=(S=2451456,E=2452479,C=23)
	CT(2403)=(S=2452480,E=2452991,C=58)
	CT(2404)=(S=2452992,E=2453151,C=18)
	CT(2405)=(S=2453152,E=2453183,C=61)
	CT(2406)=(S=2453184,E=2453191,C=12)
	CT(2407)=(S=2453192,E=2453199,C=34)
	CT(2408)=(S=2453200,E=2453207,C=98)
	CT(2409)=(S=2453208,E=2453215,C=63)
	CT(2410)=(S=2453216,E=2453239,C=18)
	CT(2411)=(S=2453240,E=2453247,C=14)
	CT(2412)=(S=2453248,E=2453503,C=50)
	CT(2413)=(S=2453504,E=2454023,C=18)
	CT(2414)=(S=2454024,E=2454031,C=15)
	CT(2415)=(S=2454032,E=2454047,C=18)
	CT(2416)=(S=2454048,E=2454079,C=28)
	CT(2417)=(S=2454080,E=2454087,C=12)
	CT(2418)=(S=2454088,E=2454095,C=72)
	CT(2419)=(S=2454096,E=2454103,C=47)
	CT(2420)=(S=2454104,E=2454111,C=36)
	CT(2421)=(S=2454112,E=2454143,C=28)
	CT(2422)=(S=2454144,E=2454207,C=41)
	CT(2423)=(S=2454208,E=2454271,C=26)
	CT(2424)=(S=2454272,E=2454527,C=31)
	CT(2425)=(S=2454528,E=2455551,C=15)
	CT(2426)=(S=2455552,E=2455679,C=28)
	CT(2427)=(S=2455680,E=2455711,C=38)
	CT(2428)=(S=2455712,E=2455743,C=28)
	CT(2429)=(S=2455744,E=2455807,C=38)
	CT(2430)=(S=2455808,E=2456063,C=23)
	CT(2431)=(S=2456064,E=2456191,C=18)
	CT(2432)=(S=2456192,E=2456199,C=34)
	CT(2433)=(S=2456200,E=2456207,C=37)
	CT(2434)=(S=2456208,E=2456215,C=19)
	CT(2435)=(S=2456216,E=2456223,C=17)
	CT(2436)=(S=2456224,E=2456255,C=102)
	CT(2437)=(S=2456256,E=2456263,C=14)
	CT(2438)=(S=2456264,E=2456271,C=12)
	CT(2439)=(S=2456272,E=2456279,C=14)
	CT(2440)=(S=2456288,E=2456303,C=37)
	CT(2441)=(S=2456304,E=2456311,C=24)
	CT(2442)=(S=2456312,E=2456319,C=14)
	CT(2443)=(S=2456320,E=2456383,C=37)
	CT(2444)=(S=2456384,E=2456415,C=61)
	CT(2445)=(S=2456416,E=2456423,C=37)
	CT(2446)=(S=2456424,E=2456431,C=28)
	CT(2447)=(S=2456432,E=2456439,C=14)
	CT(2448)=(S=2456440,E=2456447,C=28)
	CT(2449)=(S=2456448,E=2456511,C=19)
	CT(2450)=(S=2456512,E=2456519,C=26)
	CT(2451)=(S=2456520,E=2456527,C=58)
	CT(2452)=(S=2456528,E=2456535,C=36)
	CT(2453)=(S=2456536,E=2456543,C=18)
	CT(2454)=(S=2456544,E=2456551,C=19)
	CT(2455)=(S=2456552,E=2456559,C=49)
	CT(2456)=(S=2456560,E=2456567,C=15)
	CT(2457)=(S=2456568,E=2456575,C=14)
	CT(2458)=(S=2456576,E=2457599,C=23)
	CT(2459)=(S=2457600,E=2457727,C=58)
	CT(2460)=(S=2457728,E=2457735,C=14)
	CT(2461)=(S=2457736,E=2457743,C=15)
	CT(2462)=(S=2457744,E=2457751,C=36)
	CT(2463)=(S=2457752,E=2457767,C=58)
	CT(2464)=(S=2457768,E=2457775,C=25)
	CT(2465)=(S=2457776,E=2457783,C=34)
	CT(2466)=(S=2457784,E=2457791,C=14)
	CT(2467)=(S=2457792,E=2457799,C=63)
	CT(2468)=(S=2457800,E=2457807,C=41)
	CT(2469)=(S=2457808,E=2457823,C=24)
	CT(2470)=(S=2457824,E=2457839,C=38)
	CT(2471)=(S=2457840,E=2458111,C=26)
	CT(2472)=(S=2458112,E=2458175,C=58)
	CT(2473)=(S=2458176,E=2458239,C=37)
	CT(2474)=(S=2458240,E=2458255,C=28)
	CT(2475)=(S=2458256,E=2458263,C=25)
	CT(2476)=(S=2458264,E=2458271,C=15)
	CT(2477)=(S=2458272,E=2458303,C=47)
	CT(2478)=(S=2458304,E=2458311,C=18)
	CT(2479)=(S=2458312,E=2458319,C=26)
	CT(2480)=(S=2458320,E=2458327,C=25)
	CT(2481)=(S=2458328,E=2458335,C=15)
	CT(2482)=(S=2458336,E=2458351,C=14)
	CT(2483)=(S=2458352,E=2458359,C=51)
	CT(2484)=(S=2458360,E=2458367,C=14)
	CT(2485)=(S=2458368,E=2458495,C=104)
	CT(2486)=(S=2458496,E=2458543,C=58)
	CT(2487)=(S=2458544,E=2458551,C=28)
	CT(2488)=(S=2458552,E=2458559,C=34)
	CT(2489)=(S=2458560,E=2458591,C=18)
	CT(2490)=(S=2458592,E=2458599,C=40)
	CT(2491)=(S=2458600,E=2458615,C=28)
	CT(2492)=(S=2458616,E=2458623,C=37)
	CT(2493)=(S=2458624,E=2459647,C=25)
	CT(2494)=(S=2459648,E=2459903,C=47)
	CT(2495)=(S=2459904,E=2460159,C=26)
	CT(2496)=(S=2460160,E=2460415,C=28)
	CT(2497)=(S=2460416,E=2460447,C=36)
	CT(2498)=(S=2460448,E=2460479,C=18)
	CT(2499)=(S=2460480,E=2460487,C=13)
	CT(2500)=(S=2460488,E=2460495,C=58)
	CT(2501)=(S=2460496,E=2460503,C=18)
	CT(2502)=(S=2460504,E=2460511,C=15)
	CT(2503)=(S=2460512,E=2460527,C=31)
	CT(2504)=(S=2460528,E=2460535,C=96)
	CT(2505)=(S=2460536,E=2460543,C=25)
	CT(2506)=(S=2460552,E=2460559,C=36)
	CT(2507)=(S=2460560,E=2460567,C=58)
	CT(2508)=(S=2460568,E=2460575,C=19)
	CT(2509)=(S=2460576,E=2460607,C=31)
	CT(2510)=(S=2460608,E=2460871,C=18)
	CT(2511)=(S=2460872,E=2460879,C=12)
	CT(2512)=(S=2460880,E=2460887,C=37)
	CT(2513)=(S=2460896,E=2460903,C=12)
	CT(2514)=(S=2460904,E=2460911,C=36)
	CT(2515)=(S=2460912,E=2460927,C=61)
	CT(2516)=(S=2460928,E=2461183,C=23)
	CT(2517)=(S=2461184,E=2461439,C=17)
	CT(2518)=(S=2461440,E=2461447,C=49)
	CT(2519)=(S=2461448,E=2461471,C=18)
	CT(2520)=(S=2461472,E=2461479,C=36)
	CT(2521)=(S=2461480,E=2461503,C=28)
	CT(2522)=(S=2461504,E=2461511,C=55)
	CT(2523)=(S=2461512,E=2461519,C=50)
	CT(2524)=(S=2461520,E=2461527,C=36)
	CT(2525)=(S=2461528,E=2461535,C=31)
	CT(2526)=(S=2461536,E=2461551,C=18)
	CT(2527)=(S=2461552,E=2461559,C=33)
	CT(2528)=(S=2461560,E=2461567,C=25)
	CT(2529)=(S=2461576,E=2461583,C=14)
	CT(2530)=(S=2461584,E=2461591,C=26)
	CT(2531)=(S=2461592,E=2461599,C=40)
	CT(2532)=(S=2461600,E=2461615,C=38)
	CT(2533)=(S=2461616,E=2461631,C=42)
	CT(2534)=(S=2461632,E=2461695,C=51)
	CT(2535)=(S=2461696,E=2462719,C=18)
	CT(2536)=(S=2462720,E=2462847,C=38)
	CT(2537)=(S=2462848,E=2462855,C=36)
	CT(2538)=(S=2462856,E=2462863,C=28)
	CT(2539)=(S=2462864,E=2462871,C=27)
	CT(2540)=(S=2462872,E=2462879,C=28)
	CT(2541)=(S=2462880,E=2462887,C=36)
	CT(2542)=(S=2462888,E=2462895,C=60)
	CT(2543)=(S=2462896,E=2462903,C=49)
	CT(2544)=(S=2462904,E=2462927,C=36)
	CT(2545)=(S=2462928,E=2462935,C=37)
	CT(2546)=(S=2462936,E=2462943,C=14)
	CT(2547)=(S=2462944,E=2462951,C=15)
	CT(2548)=(S=2462952,E=2462959,C=28)
	CT(2549)=(S=2462960,E=2462967,C=14)
	CT(2550)=(S=2462976,E=2463231,C=32)
	CT(2551)=(S=2463232,E=2463743,C=20)
	CT(2552)=(S=2463744,E=2463751,C=61)
	CT(2553)=(S=2463752,E=2463759,C=36)
	CT(2554)=(S=2463760,E=2463775,C=58)
	CT(2555)=(S=2463776,E=2463799,C=14)
	CT(2556)=(S=2463800,E=2463807,C=19)
	CT(2557)=(S=2463808,E=2463815,C=48)
	CT(2558)=(S=2463816,E=2463823,C=37)
	CT(2559)=(S=2463824,E=2463839,C=25)
	CT(2560)=(S=2463840,E=2463871,C=71)
	CT(2561)=(S=2463872,E=2463903,C=25)
	CT(2562)=(S=2463904,E=2463935,C=26)
	CT(2563)=(S=2463936,E=2463999,C=14)
	CT(2564)=(S=2464000,E=2464071,C=18)
	CT(2565)=(S=2464072,E=2464079,C=14)
	CT(2566)=(S=2464080,E=2464087,C=34)
	CT(2567)=(S=2464088,E=2464095,C=25)
	CT(2568)=(S=2464096,E=2464127,C=34)
	CT(2569)=(S=2464128,E=2464131,C=26)
	CT(2570)=(S=2464132,E=2464133,C=38)
	CT(2571)=(S=2464134,E=2464134,C=44)
	CT(2572)=(S=2464135,E=2464137,C=38)
	CT(2573)=(S=2464138,E=2464139,C=19)
	CT(2574)=(S=2464140,E=2464143,C=38)
	CT(2575)=(S=2464144,E=2464151,C=95)
	CT(2576)=(S=2464152,E=2464159,C=38)
	CT(2577)=(S=2464160,E=2464167,C=95)
	CT(2578)=(S=2464168,E=2464171,C=38)
	CT(2579)=(S=2464172,E=2464173,C=19)
	CT(2580)=(S=2464174,E=2464175,C=38)
	CT(2581)=(S=2464176,E=2464191,C=26)
	CT(2582)=(S=2464192,E=2464255,C=36)
	CT(2583)=(S=2464256,E=2464767,C=37)
	CT(2584)=(S=2464768,E=2464771,C=53)
	CT(2585)=(S=2464772,E=2464773,C=38)
	CT(2586)=(S=2464774,E=2464774,C=44)
	CT(2587)=(S=2464775,E=2464775,C=38)
	CT(2588)=(S=2464776,E=2464783,C=53)
	CT(2589)=(S=2464784,E=2464799,C=26)
	CT(2590)=(S=2464800,E=2464805,C=38)
	CT(2591)=(S=2464806,E=2464807,C=99)
	CT(2592)=(S=2464808,E=2464815,C=38)
	CT(2593)=(S=2464816,E=2464831,C=26)
	CT(2594)=(S=2464832,E=2464833,C=14)
	CT(2595)=(S=2464834,E=2464863,C=38)
	CT(2596)=(S=2464864,E=2464867,C=15)
	CT(2597)=(S=2464868,E=2464871,C=53)
	CT(2598)=(S=2464872,E=2464879,C=38)
	CT(2599)=(S=2464880,E=2464911,C=26)
	CT(2600)=(S=2464912,E=2464915,C=53)
	CT(2601)=(S=2464916,E=2464919,C=15)
	CT(2602)=(S=2464920,E=2464935,C=26)
	CT(2603)=(S=2464936,E=2464939,C=15)
	CT(2604)=(S=2464940,E=2464940,C=38)
	CT(2605)=(S=2464941,E=2464941,C=14)
	CT(2606)=(S=2464942,E=2464943,C=38)
	CT(2607)=(S=2464944,E=2464947,C=26)
	CT(2608)=(S=2464948,E=2464959,C=38)
	CT(2609)=(S=2464960,E=2464975,C=19)
	CT(2610)=(S=2464976,E=2464979,C=15)
	CT(2611)=(S=2464980,E=2464983,C=26)
	CT(2612)=(S=2464984,E=2464991,C=90)
	CT(2613)=(S=2464992,E=2464999,C=38)
	CT(2614)=(S=2465000,E=2465011,C=53)
	CT(2615)=(S=2465012,E=2465015,C=38)
	CT(2616)=(S=2465016,E=2465019,C=53)
	CT(2617)=(S=2465020,E=2465023,C=38)
	CT(2618)=(S=2465024,E=2465031,C=24)
	CT(2619)=(S=2465032,E=2465039,C=12)
	CT(2620)=(S=2465040,E=2465055,C=18)
	CT(2621)=(S=2465056,E=2465063,C=14)
	CT(2622)=(S=2465064,E=2465071,C=28)
	CT(2623)=(S=2465072,E=2465079,C=14)
	CT(2624)=(S=2465080,E=2465087,C=12)
	CT(2625)=(S=2465088,E=2465151,C=55)
	CT(2626)=(S=2465152,E=2465159,C=12)
	CT(2627)=(S=2465160,E=2465167,C=51)
	CT(2628)=(S=2465168,E=2465175,C=44)
	CT(2629)=(S=2465176,E=2465183,C=70)
	CT(2630)=(S=2465184,E=2465215,C=51)
	CT(2631)=(S=2465216,E=2465223,C=33)
	CT(2632)=(S=2465224,E=2465231,C=58)
	CT(2633)=(S=2465232,E=2465247,C=66)
	CT(2634)=(S=2465248,E=2465255,C=12)
	CT(2635)=(S=2465256,E=2465263,C=18)
	CT(2636)=(S=2465264,E=2465271,C=14)
	CT(2637)=(S=2465272,E=2465279,C=28)
	CT(2638)=(S=2465280,E=2465535,C=25)
	CT(2639)=(S=2465536,E=2465791,C=15)
	CT(2640)=(S=2465792,E=2469887,C=12)
	CT(2641)=(S=2469888,E=2471935,C=15)
	CT(2642)=(S=2471936,E=2472447,C=49)
	CT(2643)=(S=2472448,E=2472479,C=42)
	CT(2644)=(S=2472480,E=2472511,C=94)
	CT(2645)=(S=2472512,E=2472575,C=66)
	CT(2646)=(S=2472576,E=2472703,C=15)
	CT(2647)=(S=2472704,E=2472959,C=12)
	CT(2648)=(S=2472960,E=2473023,C=18)
	CT(2649)=(S=2473024,E=2473039,C=36)
	CT(2650)=(S=2473040,E=2473047,C=50)
	CT(2651)=(S=2473048,E=2473055,C=12)
	CT(2652)=(S=2473056,E=2473087,C=14)
	CT(2653)=(S=2473088,E=2473215,C=33)
	CT(2654)=(S=2473216,E=2473471,C=21)
	CT(2655)=(S=2473472,E=2473599,C=18)
	CT(2656)=(S=2473600,E=2473727,C=58)
	CT(2657)=(S=2473728,E=2473791,C=50)
	CT(2658)=(S=2473792,E=2473823,C=26)
	CT(2659)=(S=2473824,E=2473855,C=14)
	CT(2660)=(S=2473856,E=2473983,C=27)
	CT(2661)=(S=2473984,E=2475007,C=18)
	CT(2662)=(S=2475008,E=2476031,C=19)
	CT(2663)=(S=2476032,E=2476095,C=27)
	CT(2664)=(S=2476096,E=2476111,C=18)
	CT(2665)=(S=2476112,E=2476119,C=15)
	CT(2666)=(S=2476120,E=2476127,C=18)
	CT(2667)=(S=2476128,E=2476135,C=28)
	CT(2668)=(S=2476136,E=2476143,C=12)
	CT(2669)=(S=2476144,E=2476151,C=14)
	CT(2670)=(S=2476152,E=2476159,C=18)
	CT(2671)=(S=2476160,E=2476287,C=45)
	CT(2672)=(S=2476288,E=2476551,C=28)
	CT(2673)=(S=2476552,E=2476559,C=18)
	CT(2674)=(S=2476560,E=2476567,C=15)
	CT(2675)=(S=2476568,E=2476575,C=27)
	CT(2676)=(S=2476576,E=2476591,C=58)
	CT(2677)=(S=2476592,E=2476599,C=37)
	CT(2678)=(S=2476600,E=2476607,C=27)
	CT(2679)=(S=2476608,E=2476671,C=61)
	CT(2680)=(S=2476672,E=2476799,C=38)
	CT(2681)=(S=2476800,E=2476831,C=31)
	CT(2682)=(S=2476832,E=2476839,C=60)
	CT(2683)=(S=2476840,E=2476847,C=14)
	CT(2684)=(S=2476848,E=2476855,C=63)
	CT(2685)=(S=2476856,E=2476863,C=36)
	CT(2686)=(S=2476864,E=2476927,C=59)
	CT(2687)=(S=2476928,E=2476991,C=14)
	CT(2688)=(S=2476992,E=2476999,C=63)
	CT(2689)=(S=2477000,E=2477007,C=18)
	CT(2690)=(S=2477008,E=2477015,C=55)
	CT(2691)=(S=2477016,E=2477023,C=36)
	CT(2692)=(S=2477024,E=2477039,C=15)
	CT(2693)=(S=2477040,E=2477047,C=18)
	CT(2694)=(S=2477048,E=2477055,C=34)
	CT(2695)=(S=2477056,E=2477311,C=18)
	CT(2696)=(S=2477312,E=2477319,C=37)
	CT(2697)=(S=2477320,E=2477327,C=33)
	CT(2698)=(S=2477328,E=2477335,C=18)
	CT(2699)=(S=2477336,E=2477343,C=59)
	CT(2700)=(S=2477344,E=2477351,C=48)
	CT(2701)=(S=2477352,E=2477359,C=12)
	CT(2702)=(S=2477360,E=2477367,C=18)
	CT(2703)=(S=2477368,E=2477375,C=14)
	CT(2704)=(S=2477376,E=2477407,C=18)
	CT(2705)=(S=2477408,E=2477423,C=50)
	CT(2706)=(S=2477424,E=2477431,C=35)
	CT(2707)=(S=2477432,E=2477439,C=24)
	CT(2708)=(S=2477440,E=2478079,C=15)
	CT(2709)=(S=2478080,E=2478111,C=19)
	CT(2710)=(S=2478112,E=2478119,C=59)
	CT(2711)=(S=2478120,E=2478127,C=20)
	CT(2712)=(S=2478128,E=2478135,C=12)
	CT(2713)=(S=2478136,E=2478143,C=49)
	CT(2714)=(S=2478144,E=2478159,C=18)
	CT(2715)=(S=2478160,E=2478167,C=14)
	CT(2716)=(S=2478168,E=2478175,C=49)
	CT(2717)=(S=2478176,E=2478183,C=18)
	CT(2718)=(S=2478184,E=2478191,C=28)
	CT(2719)=(S=2478192,E=2478199,C=36)
	CT(2720)=(S=2478200,E=2478207,C=18)
	CT(2721)=(S=2478208,E=2478335,C=94)
	CT(2722)=(S=2478336,E=2478463,C=28)
	CT(2723)=(S=2478464,E=2478495,C=58)
	CT(2724)=(S=2478496,E=2478503,C=60)
	CT(2725)=(S=2478504,E=2478511,C=51)
	CT(2726)=(S=2478512,E=2478527,C=34)
	CT(2727)=(S=2478528,E=2478535,C=14)
	CT(2728)=(S=2478536,E=2478543,C=15)
	CT(2729)=(S=2478544,E=2478559,C=14)
	CT(2730)=(S=2478560,E=2478583,C=18)
	CT(2731)=(S=2478584,E=2478591,C=39)
	CT(2732)=(S=2478592,E=2479103,C=94)
	CT(2733)=(S=2479104,E=2480127,C=68)
	CT(2734)=(S=2480128,E=2480639,C=23)
	CT(2735)=(S=2480640,E=2480767,C=25)
	CT(2736)=(S=2480768,E=2480831,C=62)
	CT(2737)=(S=2480832,E=2480847,C=37)
	CT(2738)=(S=2480848,E=2480863,C=43)
	CT(2739)=(S=2480864,E=2480871,C=16)
	CT(2740)=(S=2480872,E=2480879,C=50)
	CT(2741)=(S=2480880,E=2480887,C=36)
	CT(2742)=(S=2480888,E=2480895,C=28)
	CT(2743)=(S=2480896,E=2481151,C=47)
	CT(2744)=(S=2481152,E=2481183,C=14)
	CT(2745)=(S=2481184,E=2481191,C=36)
	CT(2746)=(S=2481192,E=2481199,C=15)
	CT(2747)=(S=2481200,E=2481215,C=12)
	CT(2748)=(S=2481216,E=2481231,C=46)
	CT(2749)=(S=2481232,E=2481239,C=18)
	CT(2750)=(S=2481240,E=2481263,C=14)
	CT(2751)=(S=2481264,E=2481279,C=61)
	CT(2752)=(S=2481280,E=2481295,C=50)
	CT(2753)=(S=2481296,E=2481303,C=36)
	CT(2754)=(S=2481304,E=2481319,C=18)
	CT(2755)=(S=2481320,E=2481327,C=36)
	CT(2756)=(S=2481328,E=2481343,C=18)
	CT(2757)=(S=2481344,E=2481407,C=50)
	CT(2758)=(S=2481408,E=2481471,C=26)
	CT(2759)=(S=2481536,E=2481567,C=31)
	CT(2760)=(S=2481568,E=2481583,C=38)
	CT(2761)=(S=2481584,E=2481591,C=102)
	CT(2762)=(S=2481592,E=2481599,C=18)
	CT(2763)=(S=2481600,E=2481607,C=28)
	CT(2764)=(S=2481608,E=2481615,C=18)
	CT(2765)=(S=2481616,E=2481623,C=50)
	CT(2766)=(S=2481624,E=2481631,C=14)
	CT(2767)=(S=2481632,E=2481639,C=19)
	CT(2768)=(S=2481640,E=2481647,C=25)
	CT(2769)=(S=2481648,E=2481663,C=33)
	CT(2770)=(S=2481664,E=2482175,C=25)
	CT(2771)=(S=2482176,E=2482431,C=23)
	CT(2772)=(S=2482432,E=2482687,C=58)
	CT(2773)=(S=2482688,E=2483199,C=15)
	CT(2774)=(S=2483200,E=2483263,C=58)
	CT(2775)=(S=2483264,E=2483271,C=20)
	CT(2776)=(S=2483272,E=2483279,C=36)
	CT(2777)=(S=2483280,E=2483295,C=18)
	CT(2778)=(S=2483296,E=2483303,C=25)
	CT(2779)=(S=2483304,E=2483311,C=27)
	CT(2780)=(S=2483312,E=2483319,C=18)
	CT(2781)=(S=2483320,E=2483391,C=28)
	CT(2782)=(S=2483392,E=2483455,C=63)
	CT(2783)=(S=2483456,E=2483711,C=31)
	CT(2784)=(S=2483712,E=2483759,C=28)
	CT(2785)=(S=2483760,E=2483775,C=14)
	CT(2786)=(S=2483776,E=2483807,C=25)
	CT(2787)=(S=2483808,E=2483815,C=36)
	CT(2788)=(S=2483816,E=2483823,C=37)
	CT(2789)=(S=2483824,E=2483831,C=70)
	CT(2790)=(S=2483832,E=2483839,C=49)
	CT(2791)=(S=2483840,E=2483847,C=36)
	CT(2792)=(S=2483848,E=2483967,C=18)
	CT(2793)=(S=2483968,E=2484223,C=101)
	CT(2794)=(S=2484224,E=2484351,C=40)
	CT(2795)=(S=2484352,E=2484479,C=18)
	CT(2796)=(S=2484480,E=2484543,C=53)
	CT(2797)=(S=2484544,E=2484559,C=19)
	CT(2798)=(S=2484560,E=2484567,C=18)
	CT(2799)=(S=2484568,E=2484575,C=47)
	CT(2800)=(S=2484576,E=2484591,C=58)
	CT(2801)=(S=2484592,E=2484599,C=18)
	CT(2802)=(S=2484600,E=2484607,C=34)
	CT(2803)=(S=2484608,E=2484735,C=40)
	CT(2804)=(S=2484736,E=2484991,C=50)
	CT(2805)=(S=2484992,E=2484999,C=42)
	CT(2806)=(S=2485000,E=2485007,C=36)
	CT(2807)=(S=2485008,E=2485023,C=26)
	CT(2808)=(S=2485024,E=2485031,C=25)
	CT(2809)=(S=2485032,E=2485039,C=14)
	CT(2810)=(S=2485040,E=2485055,C=13)
	CT(2811)=(S=2485056,E=2485063,C=18)
	CT(2812)=(S=2485064,E=2485071,C=37)
	CT(2813)=(S=2485072,E=2485079,C=36)
	CT(2814)=(S=2485080,E=2485087,C=12)
	CT(2815)=(S=2485088,E=2485103,C=33)
	CT(2816)=(S=2485104,E=2485111,C=36)
	CT(2817)=(S=2485112,E=2485119,C=14)
	CT(2818)=(S=2485120,E=2485247,C=18)
	CT(2819)=(S=2485248,E=2486271,C=35)
	CT(2820)=(S=2486272,E=2487295,C=23)
	CT(2821)=(S=2487296,E=2487359,C=13)
	CT(2822)=(S=2487360,E=2487423,C=14)
	CT(2823)=(S=2487424,E=2487551,C=57)
	CT(2824)=(S=2487552,E=2487807,C=16)
	CT(2825)=(S=2487808,E=2488063,C=53)
	CT(2826)=(S=2488064,E=2488095,C=19)
	CT(2827)=(S=2488096,E=2488111,C=36)
	CT(2828)=(S=2488112,E=2488119,C=15)
	CT(2829)=(S=2488120,E=2488127,C=58)
	CT(2830)=(S=2488128,E=2488159,C=28)
	CT(2831)=(S=2488160,E=2488183,C=37)
	CT(2832)=(S=2488184,E=2488191,C=25)
	CT(2833)=(S=2488192,E=2488831,C=58)
	CT(2834)=(S=2488832,E=2489087,C=19)
	CT(2835)=(S=2489088,E=2489215,C=36)
	CT(2836)=(S=2489216,E=2489343,C=38)
	CT(2837)=(S=2489344,E=2489359,C=14)
	CT(2838)=(S=2489360,E=2489367,C=18)
	CT(2839)=(S=2489368,E=2489407,C=14)
	CT(2840)=(S=2489408,E=2489439,C=66)
	CT(2841)=(S=2489440,E=2489447,C=25)
	CT(2842)=(S=2489448,E=2489455,C=28)
	CT(2843)=(S=2489456,E=2489463,C=27)
	CT(2844)=(S=2489464,E=2489471,C=36)
	CT(2845)=(S=2489472,E=2489503,C=34)
	CT(2846)=(S=2489504,E=2489519,C=13)
	CT(2847)=(S=2489520,E=2489527,C=34)
	CT(2848)=(S=2489528,E=2489535,C=42)
	CT(2849)=(S=2489536,E=2489543,C=14)
	CT(2850)=(S=2489544,E=2489551,C=12)
	CT(2851)=(S=2489552,E=2489559,C=27)
	CT(2852)=(S=2489568,E=2489599,C=42)
	CT(2853)=(S=2489600,E=2489855,C=27)
	CT(2854)=(S=2489856,E=2490367,C=26)
	CT(2855)=(S=2490368,E=2555903,C=29)
	CT(2856)=(S=2555904,E=2555904,C=1)
	CT(2857)=(S=2555905,E=2555905,C=0)
	CT(2858)=(S=2555906,E=2556159,C=1)
	CT(2859)=(S=2556160,E=2556415,C=9)
	CT(2860)=(S=2556416,E=2556927,C=2)
	CT(2861)=(S=2556928,E=2557951,C=6)
	CT(2862)=(S=2557952,E=2559999,C=9)
	CT(2863)=(S=2560000,E=2564095,C=6)
	CT(2864)=(S=2564096,E=2572287,C=77)
	CT(2865)=(S=2572288,E=2583807,C=1)
	CT(2866)=(S=2583808,E=2583935,C=8)
	CT(2867)=(S=2583936,E=2584063,C=7)
	CT(2868)=(S=2584064,E=2584575,C=2)
	CT(2869)=(S=2584576,E=2588671,C=6)
	CT(2870)=(S=2588672,E=2605055,C=1)
	CT(2871)=(S=2605056,E=2621439,C=74)
	CT(2872)=(S=2621440,E=2686975,C=29)
	CT(2873)=(S=2686976,E=2695167,C=105)
	CT(2874)=(S=2695168,E=2699263,C=106)
	CT(2875)=(S=2699264,E=2701631,C=105)
	CT(2876)=(S=2701632,E=2701647,C=107)
	CT(2877)=(S=2701648,E=2701663,C=108)
	CT(2878)=(S=2701664,E=2701679,C=109)
	CT(2879)=(S=2701680,E=2701683,C=105)
	CT(2880)=(S=2701684,E=2701687,C=110)
	CT(2881)=(S=2701688,E=2701691,C=111)
	CT(2882)=(S=2701692,E=2701695,C=107)
	CT(2883)=(S=2701696,E=2701759,C=105)
	CT(2884)=(S=2701760,E=2701823,C=110)
	CT(2885)=(S=2701824,E=2702079,C=111)
	CT(2886)=(S=2702080,E=2702335,C=112)
	CT(2887)=(S=2702336,E=2702591,C=113)
	CT(2888)=(S=2702592,E=2702847,C=105)
	CT(2889)=(S=2703104,E=2703167,C=114)
	CT(2890)=(S=2703168,E=2703231,C=105)
	CT(2891)=(S=2703232,E=2703263,C=115)
	CT(2892)=(S=2703264,E=2703295,C=116)
	CT(2893)=(S=2703296,E=2703359,C=117)
	CT(2894)=(S=2703360,E=2703871,C=106)
	CT(2895)=(S=2703872,E=2703935,C=118)
	CT(2896)=(S=2703936,E=2704063,C=105)
	CT(2897)=(S=2704064,E=2704127,C=110)
	CT(2898)=(S=2704128,E=2704191,C=119)
	CT(2899)=(S=2704192,E=2704255,C=106)
	CT(2900)=(S=2704256,E=2704319,C=111)
	CT(2901)=(S=2704320,E=2704383,C=120)
	CT(2902)=(S=2704384,E=2704895,C=106)
	CT(2903)=(S=2704896,E=2705023,C=121)
	CT(2904)=(S=2705024,E=2705151,C=116)
	CT(2905)=(S=2705152,E=2705279,C=105)
	CT(2906)=(S=2705280,E=2705407,C=111)
	CT(2907)=(S=2705408,E=2705471,C=116)
	CT(2908)=(S=2705472,E=2705503,C=106)
	CT(2909)=(S=2705504,E=2705535,C=114)
	CT(2910)=(S=2705536,E=2705567,C=105)
	CT(2911)=(S=2705568,E=2705599,C=109)
	CT(2912)=(S=2705600,E=2705631,C=113)
	CT(2913)=(S=2705632,E=2705663,C=122)
	CT(2914)=(S=2705696,E=2705727,C=105)
	CT(2915)=(S=2705728,E=2705759,C=112)
	CT(2916)=(S=2705760,E=2705791,C=123)
	CT(2917)=(S=2705792,E=2705823,C=111)
	CT(2918)=(S=2705824,E=2705887,C=112)
	CT(2919)=(S=2705888,E=2705919,C=111)
	CT(2920)=(S=2705920,E=2705935,C=124)
	CT(2921)=(S=2705936,E=2705951,C=115)
	CT(2922)=(S=2705952,E=2705967,C=125)
	CT(2923)=(S=2705968,E=2705983,C=126)
	CT(2924)=(S=2705984,E=2705999,C=65)
	CT(2925)=(S=2706000,E=2706015,C=110)
	CT(2926)=(S=2706032,E=2706047,C=112)
	CT(2927)=(S=2706048,E=2706063,C=127)
	CT(2928)=(S=2706064,E=2706079,C=105)
	CT(2929)=(S=2706080,E=2706095,C=128)
	CT(2930)=(S=2706096,E=2706127,C=105)
	CT(2931)=(S=2706128,E=2706143,C=115)
	CT(2932)=(S=2706144,E=2706159,C=105)
	CT(2933)=(S=2706160,E=2706175,C=116)
	CT(2934)=(S=2706176,E=2706191,C=126)
	CT(2935)=(S=2706208,E=2706223,C=109)
	CT(2936)=(S=2706224,E=2706239,C=110)
	CT(2937)=(S=2706240,E=2706255,C=129)
	CT(2938)=(S=2706256,E=2706271,C=111)
	CT(2939)=(S=2706272,E=2706287,C=105)
	CT(2940)=(S=2706288,E=2706303,C=121)
	CT(2941)=(S=2706304,E=2706319,C=105)
	CT(2942)=(S=2706320,E=2706335,C=109)
	CT(2943)=(S=2706336,E=2706351,C=130)
	CT(2944)=(S=2706352,E=2706367,C=109)
	CT(2945)=(S=2706368,E=2706383,C=111)
	CT(2946)=(S=2706384,E=2706399,C=112)
	CT(2947)=(S=2706400,E=2706415,C=105)
	CT(2948)=(S=2706416,E=2706431,C=114)
	CT(2949)=(S=2706432,E=2706439,C=131)
	CT(2950)=(S=2706440,E=2706447,C=132)
	CT(2951)=(S=2706448,E=2706455,C=133)
	CT(2952)=(S=2706456,E=2706463,C=110)
	CT(2953)=(S=2706464,E=2706471,C=105)
	CT(2954)=(S=2706472,E=2706479,C=113)
	CT(2955)=(S=2706488,E=2706495,C=105)
	CT(2956)=(S=2706496,E=2706519,C=111)
	CT(2957)=(S=2706520,E=2706527,C=112)
	CT(2958)=(S=2706528,E=2706551,C=105)
	CT(2959)=(S=2706552,E=2706559,C=122)
	CT(2960)=(S=2706560,E=2706567,C=105)
	CT(2961)=(S=2706568,E=2706575,C=129)
	CT(2962)=(S=2706576,E=2706583,C=131)
	CT(2963)=(S=2706584,E=2706591,C=111)
	CT(2964)=(S=2706600,E=2706607,C=109)
	CT(2965)=(S=2706608,E=2706615,C=105)
	CT(2966)=(S=2706616,E=2706623,C=109)
	CT(2967)=(S=2706624,E=2706631,C=111)
	CT(2968)=(S=2706632,E=2706663,C=105)
	CT(2969)=(S=2706672,E=2706679,C=126)
	CT(2970)=(S=2706680,E=2706687,C=111)
	CT(2971)=(S=2706688,E=2706695,C=114)
	CT(2972)=(S=2706696,E=2706703,C=121)
	CT(2973)=(S=2706704,E=2706711,C=115)
	CT(2974)=(S=2706712,E=2706719,C=105)
	CT(2975)=(S=2706720,E=2706727,C=131)
	CT(2976)=(S=2706728,E=2706735,C=111)
	CT(2977)=(S=2706736,E=2706751,C=105)
	CT(2978)=(S=2706752,E=2706759,C=110)
	CT(2979)=(S=2706760,E=2706767,C=130)
	CT(2980)=(S=2706768,E=2706775,C=120)
	CT(2981)=(S=2706776,E=2706783,C=126)
	CT(2982)=(S=2706784,E=2706799,C=105)
	CT(2983)=(S=2706800,E=2706807,C=134)
	CT(2984)=(S=2706808,E=2706815,C=122)
	CT(2985)=(S=2706816,E=2706823,C=131)
	CT(2986)=(S=2706824,E=2706831,C=106)
	CT(2987)=(S=2706832,E=2706839,C=113)
	CT(2988)=(S=2706840,E=2706847,C=105)
	CT(2989)=(S=2706856,E=2706863,C=111)
	CT(2990)=(S=2706864,E=2706871,C=135)
	CT(2991)=(S=2706872,E=2706879,C=136)
	CT(2992)=(S=2706880,E=2706887,C=112)
	CT(2993)=(S=2706888,E=2706895,C=105)
	CT(2994)=(S=2706896,E=2706903,C=120)
	CT(2995)=(S=2706908,E=2706911,C=137)
	CT(2996)=(S=2706912,E=2706919,C=112)
	CT(2997)=(S=2706920,E=2706927,C=138)
	CT(2998)=(S=2706928,E=2706935,C=131)
	CT(2999)=(S=2706944,E=2706947,C=112)
	CT(3000)=(S=2706948,E=2706951,C=105)
	CT(3001)=(S=2706952,E=2706955,C=111)
	CT(3002)=(S=2706960,E=2706963,C=116)
	CT(3003)=(S=2706964,E=2706967,C=106)
	CT(3004)=(S=2706968,E=2706971,C=109)
	CT(3005)=(S=2706972,E=2706983,C=105)
	CT(3006)=(S=2706984,E=2706987,C=110)
	CT(3007)=(S=2706992,E=2706999,C=139)
	CT(3008)=(S=2707000,E=2707003,C=121)
	CT(3009)=(S=2707004,E=2707007,C=106)
	CT(3010)=(S=2707008,E=2707011,C=112)
	CT(3011)=(S=2707016,E=2707019,C=140)
	CT(3012)=(S=2707020,E=2707023,C=107)
	CT(3013)=(S=2707024,E=2707027,C=111)
	CT(3014)=(S=2707028,E=2707031,C=97)
	CT(3015)=(S=2707032,E=2707035,C=111)
	CT(3016)=(S=2707036,E=2707039,C=126)
	CT(3017)=(S=2707040,E=2707043,C=122)
	CT(3018)=(S=2707044,E=2707047,C=111)
	CT(3019)=(S=2707048,E=2707051,C=137)
	CT(3020)=(S=2707052,E=2707055,C=119)
	CT(3021)=(S=2707056,E=2707059,C=114)
	CT(3022)=(S=2707060,E=2707063,C=141)
	CT(3023)=(S=2707064,E=2707067,C=142)
	CT(3024)=(S=2707068,E=2707071,C=110)
	CT(3025)=(S=2707072,E=2707075,C=105)
	CT(3026)=(S=2707080,E=2707083,C=143)
	CT(3027)=(S=2707084,E=2707087,C=105)
	CT(3028)=(S=2707088,E=2707091,C=110)
	CT(3029)=(S=2707092,E=2707095,C=106)
	CT(3030)=(S=2707096,E=2707099,C=122)
	CT(3031)=(S=2707100,E=2707103,C=111)
	CT(3032)=(S=2707108,E=2707111,C=105)
	CT(3033)=(S=2707112,E=2707115,C=112)
	CT(3034)=(S=2707116,E=2707119,C=111)
	CT(3035)=(S=2707120,E=2707123,C=109)
	CT(3036)=(S=2707128,E=2707135,C=105)
	CT(3037)=(S=2707136,E=2707143,C=137)
	CT(3038)=(S=2707144,E=2707147,C=105)
	CT(3039)=(S=2707148,E=2707151,C=120)
	CT(3040)=(S=2707152,E=2707155,C=111)
	CT(3041)=(S=2707156,E=2707159,C=105)
	CT(3042)=(S=2707160,E=2707163,C=121)
	CT(3043)=(S=2707164,E=2707167,C=105)
	CT(3044)=(S=2707168,E=2707171,C=111)
	CT(3045)=(S=2707172,E=2707179,C=105)
	CT(3046)=(S=2707184,E=2707187,C=122)
	CT(3047)=(S=2707188,E=2707191,C=105)
	CT(3048)=(S=2707192,E=2707195,C=121)
	CT(3049)=(S=2707196,E=2707207,C=111)
	CT(3050)=(S=2707208,E=2707211,C=109)
	CT(3051)=(S=2707212,E=2707215,C=107)
	CT(3052)=(S=2707216,E=2707223,C=105)
	CT(3053)=(S=2707224,E=2707227,C=144)
	CT(3054)=(S=2707228,E=2707231,C=107)
	CT(3055)=(S=2707232,E=2707235,C=126)
	CT(3056)=(S=2707236,E=2707243,C=105)
	CT(3057)=(S=2707244,E=2707247,C=145)
	CT(3058)=(S=2707248,E=2707251,C=146)
	CT(3059)=(S=2707252,E=2707255,C=117)
	CT(3060)=(S=2707256,E=2707259,C=107)
	CT(3061)=(S=2707260,E=2707263,C=147)
	CT(3062)=(S=2707264,E=2707267,C=111)
	CT(3063)=(S=2707268,E=2707271,C=112)
	CT(3064)=(S=2707272,E=2707283,C=105)
	CT(3065)=(S=2707288,E=2707291,C=121)
	CT(3066)=(S=2707292,E=2707295,C=105)
	CT(3067)=(S=2707296,E=2707299,C=110)
	CT(3068)=(S=2707300,E=2707303,C=112)
	CT(3069)=(S=2707304,E=2707311,C=105)
	CT(3070)=(S=2707312,E=2707315,C=107)
	CT(3071)=(S=2707316,E=2707319,C=111)
	CT(3072)=(S=2707320,E=2707323,C=144)
	CT(3073)=(S=2707324,E=2707327,C=127)
	CT(3074)=(S=2707328,E=2707331,C=120)
	CT(3075)=(S=2707332,E=2707335,C=107)
	CT(3076)=(S=2707336,E=2707339,C=126)
	CT(3077)=(S=2707344,E=2707347,C=111)
	CT(3078)=(S=2707348,E=2707351,C=105)
	CT(3079)=(S=2707352,E=2707355,C=111)
	CT(3080)=(S=2707356,E=2707367,C=105)
	CT(3081)=(S=2707368,E=2707371,C=109)
	CT(3082)=(S=2707372,E=2707375,C=105)
	CT(3083)=(S=2707376,E=2707379,C=111)
	CT(3084)=(S=2707380,E=2707383,C=105)
	CT(3085)=(S=2707384,E=2707387,C=112)
	CT(3086)=(S=2707388,E=2707391,C=107)
	CT(3087)=(S=2707392,E=2707395,C=105)
	CT(3088)=(S=2707396,E=2707399,C=140)
	CT(3089)=(S=2707400,E=2707403,C=136)
	CT(3090)=(S=2707404,E=2707407,C=111)
	CT(3091)=(S=2707408,E=2707411,C=106)
	CT(3092)=(S=2707412,E=2707415,C=105)
	CT(3093)=(S=2707416,E=2707419,C=124)
	CT(3094)=(S=2707420,E=2707423,C=105)
	CT(3095)=(S=2707424,E=2707427,C=145)
	CT(3096)=(S=2707428,E=2707431,C=109)
	CT(3097)=(S=2707432,E=2707435,C=137)
	CT(3098)=(S=2707436,E=2707439,C=136)
	CT(3099)=(S=2707440,E=2707443,C=105)
	CT(3100)=(S=2707444,E=2707447,C=131)
	CT(3101)=(S=2707448,E=2707451,C=105)
	CT(3102)=(S=2707452,E=2707967,C=109)
	CT(3103)=(S=2707968,E=2708479,C=148)
	CT(3104)=(S=2708480,E=2708607,C=105)
	CT(3105)=(S=2708608,E=2708639,C=113)
	CT(3106)=(S=2708640,E=2708671,C=111)
	CT(3107)=(S=2708672,E=2708703,C=130)
	CT(3108)=(S=2708704,E=2708735,C=138)
	CT(3109)=(S=2708736,E=2708895,C=105)
	CT(3110)=(S=2708896,E=2708927,C=124)
	CT(3111)=(S=2708928,E=2708959,C=107)
	CT(3112)=(S=2708960,E=2708991,C=105)
	CT(3113)=(S=2708992,E=2709023,C=108)
	CT(3114)=(S=2709024,E=2709055,C=147)
	CT(3115)=(S=2709056,E=2709119,C=105)
	CT(3116)=(S=2709120,E=2709151,C=111)
	CT(3117)=(S=2709152,E=2709183,C=112)
	CT(3118)=(S=2709184,E=2709215,C=105)
	CT(3119)=(S=2709216,E=2709247,C=124)
	CT(3120)=(S=2709248,E=2709279,C=121)
	CT(3121)=(S=2709312,E=2709343,C=111)
	CT(3122)=(S=2709344,E=2709375,C=113)
	CT(3123)=(S=2709376,E=2709407,C=134)
	CT(3124)=(S=2709408,E=2709439,C=126)
	CT(3125)=(S=2709440,E=2709503,C=105)
	CT(3126)=(S=2709504,E=2709759,C=106)
	CT(3127)=(S=2709760,E=2710271,C=109)
	CT(3128)=(S=2710272,E=2710527,C=106)
	CT(3129)=(S=2710528,E=2710655,C=134)
	CT(3130)=(S=2710656,E=2710783,C=120)
	CT(3131)=(S=2710784,E=2710911,C=112)
	CT(3132)=(S=2710912,E=2711039,C=110)
	CT(3133)=(S=2711040,E=2711295,C=131)
	CT(3134)=(S=2711296,E=2711551,C=119)
	CT(3135)=(S=2711552,E=2715647,C=135)
	CT(3136)=(S=2715648,E=2719743,C=105)
	CT(3137)=(S=2719744,E=2720767,C=106)
	CT(3138)=(S=2720768,E=2721791,C=105)
	CT(3139)=(S=2721792,E=2722047,C=113)
	CT(3140)=(S=2722048,E=2722303,C=134)
	CT(3141)=(S=2722304,E=2722335,C=130)
	CT(3142)=(S=2722336,E=2722367,C=141)
	CT(3143)=(S=2722368,E=2722375,C=105)
	CT(3144)=(S=2722376,E=2722383,C=126)
	CT(3145)=(S=2722384,E=2722391,C=128)
	CT(3146)=(S=2722392,E=2722395,C=124)
	CT(3147)=(S=2722396,E=2722399,C=105)
	CT(3148)=(S=2722400,E=2722431,C=139)
	CT(3149)=(S=2722432,E=2722463,C=149)
	CT(3150)=(S=2722464,E=2722495,C=111)
	CT(3151)=(S=2722496,E=2722511,C=112)
	CT(3152)=(S=2722512,E=2722527,C=130)
	CT(3153)=(S=2722528,E=2722543,C=131)
	CT(3154)=(S=2722544,E=2722559,C=109)
	CT(3155)=(S=2722560,E=2722623,C=110)
	CT(3156)=(S=2722624,E=2722687,C=111)
	CT(3157)=(S=2722688,E=2722815,C=109)
	CT(3158)=(S=2722816,E=2723839,C=134)
	CT(3159)=(S=2723840,E=2725887,C=105)
	CT(3160)=(S=2725888,E=2726399,C=106)
	CT(3161)=(S=2726400,E=2726655,C=105)
	CT(3162)=(S=2726656,E=2726783,C=111)
	CT(3163)=(S=2726784,E=2726911,C=106)
	CT(3164)=(S=2726912,E=2727423,C=105)
	CT(3165)=(S=2727424,E=2727935,C=122)
	CT(3166)=(S=2727936,E=2732031,C=105)
	CT(3167)=(S=2732032,E=2732287,C=106)
	CT(3168)=(S=2732288,E=2732543,C=105)
	CT(3169)=(S=2732544,E=2733055,C=106)
	CT(3170)=(S=2733056,E=2733567,C=105)
	CT(3171)=(S=2733568,E=2733823,C=117)
	CT(3172)=(S=2733824,E=2734079,C=105)
	CT(3173)=(S=2734080,E=2734335,C=111)
	CT(3174)=(S=2734336,E=2734591,C=105)
	CT(3175)=(S=2734592,E=2734847,C=128)
	CT(3176)=(S=2734848,E=2735103,C=106)
	CT(3177)=(S=2735104,E=2735167,C=115)
	CT(3178)=(S=2735168,E=2735231,C=149)
	CT(3179)=(S=2735232,E=2735295,C=112)
	CT(3180)=(S=2735296,E=2735359,C=105)
	CT(3181)=(S=2735360,E=2735391,C=111)
	CT(3182)=(S=2735392,E=2735423,C=118)
	CT(3183)=(S=2735424,E=2735455,C=105)
	CT(3184)=(S=2735520,E=2735551,C=110)
	CT(3185)=(S=2735552,E=2735583,C=137)
	CT(3186)=(S=2735584,E=2735615,C=150)
	CT(3187)=(S=2735616,E=2735647,C=111)
	CT(3188)=(S=2735648,E=2735679,C=107)
	CT(3189)=(S=2735680,E=2735683,C=124)
	CT(3190)=(S=2735684,E=2735687,C=110)
	CT(3191)=(S=2735688,E=2735691,C=129)
	CT(3192)=(S=2735692,E=2735695,C=130)
	CT(3193)=(S=2735696,E=2735699,C=137)
	CT(3194)=(S=2735700,E=2735703,C=117)
	CT(3195)=(S=2735704,E=2735707,C=110)
	CT(3196)=(S=2735708,E=2735711,C=121)
	CT(3197)=(S=2735712,E=2735743,C=117)
	CT(3198)=(S=2735744,E=2735775,C=113)
	CT(3199)=(S=2735776,E=2735807,C=131)
	CT(3200)=(S=2735808,E=2735839,C=130)
	CT(3201)=(S=2735840,E=2735843,C=120)
	CT(3202)=(S=2735844,E=2735847,C=141)
	CT(3203)=(S=2735848,E=2735851,C=137)
	CT(3204)=(S=2735852,E=2735855,C=115)
	CT(3205)=(S=2735860,E=2735863,C=126)
	CT(3206)=(S=2735864,E=2735867,C=106)
	CT(3207)=(S=2735868,E=2735871,C=109)
	CT(3208)=(S=2735872,E=2735935,C=106)
	CT(3209)=(S=2735940,E=2735943,C=118)
	CT(3210)=(S=2735944,E=2735947,C=131)
	CT(3211)=(S=2735948,E=2735951,C=130)
	CT(3212)=(S=2735952,E=2735955,C=106)
	CT(3213)=(S=2735956,E=2735959,C=124)
	CT(3214)=(S=2735960,E=2735963,C=105)
	CT(3215)=(S=2735968,E=2735971,C=110)
	CT(3216)=(S=2735972,E=2735975,C=120)
	CT(3217)=(S=2735976,E=2735979,C=108)
	CT(3218)=(S=2735988,E=2735991,C=114)
	CT(3219)=(S=2735992,E=2735995,C=147)
	CT(3220)=(S=2735996,E=2736063,C=105)
	CT(3221)=(S=2736064,E=2736071,C=109)
	CT(3222)=(S=2736072,E=2736079,C=133)
	CT(3223)=(S=2736080,E=2736083,C=111)
	CT(3224)=(S=2736084,E=2736087,C=113)
	CT(3225)=(S=2736088,E=2736091,C=126)
	CT(3226)=(S=2736092,E=2736095,C=136)
	CT(3227)=(S=2736096,E=2736103,C=113)
	CT(3228)=(S=2736104,E=2736111,C=107)
	CT(3229)=(S=2736112,E=2736119,C=110)
	CT(3230)=(S=2736120,E=2736123,C=97)
	CT(3231)=(S=2736124,E=2736127,C=135)
	CT(3232)=(S=2736128,E=2737151,C=105)
	CT(3233)=(S=2737152,E=2737407,C=106)
	CT(3234)=(S=2737408,E=2737663,C=128)
	CT(3235)=(S=2737664,E=2737919,C=105)
	CT(3236)=(S=2737920,E=2738175,C=106)
	CT(3237)=(S=2738176,E=2738687,C=135)
	CT(3238)=(S=2738688,E=2738719,C=110)
	CT(3239)=(S=2738720,E=2738751,C=105)
	CT(3240)=(S=2738752,E=2738847,C=118)
	CT(3241)=(S=2738848,E=2738879,C=119)
	CT(3242)=(S=2738880,E=2738911,C=120)
	CT(3243)=(S=2738912,E=2738943,C=130)
	CT(3244)=(S=2738944,E=2739007,C=105)
	CT(3245)=(S=2739008,E=2739071,C=111)
	CT(3246)=(S=2739072,E=2739103,C=141)
	CT(3247)=(S=2739104,E=2739119,C=105)
	CT(3248)=(S=2739120,E=2739135,C=133)
	CT(3249)=(S=2739136,E=2739151,C=123)
	CT(3250)=(S=2739152,E=2739167,C=109)
	CT(3251)=(S=2739168,E=2739183,C=139)
	CT(3252)=(S=2739184,E=2739199,C=147)
	CT(3253)=(S=2739200,E=2739231,C=138)
	CT(3254)=(S=2739232,E=2739263,C=110)
	CT(3255)=(S=2739264,E=2739295,C=120)
	CT(3256)=(S=2739296,E=2739327,C=115)
	CT(3257)=(S=2739328,E=2739359,C=112)
	CT(3258)=(S=2739360,E=2739391,C=109)
	CT(3259)=(S=2739392,E=2739423,C=105)
	CT(3260)=(S=2739424,E=2739455,C=111)
	CT(3261)=(S=2739456,E=2739487,C=120)
	CT(3262)=(S=2739488,E=2739519,C=116)
	CT(3263)=(S=2739520,E=2739551,C=120)
	CT(3264)=(S=2739552,E=2739583,C=106)
	CT(3265)=(S=2739584,E=2739615,C=117)
	CT(3266)=(S=2739616,E=2739647,C=111)
	CT(3267)=(S=2739648,E=2739679,C=134)
	CT(3268)=(S=2739680,E=2739711,C=97)
	CT(3269)=(S=2739712,E=2739743,C=111)
	CT(3270)=(S=2739744,E=2739775,C=109)
	CT(3271)=(S=2739776,E=2739807,C=118)
	CT(3272)=(S=2739808,E=2739839,C=113)
	CT(3273)=(S=2739840,E=2739871,C=106)
	CT(3274)=(S=2739872,E=2739935,C=105)
	CT(3275)=(S=2739968,E=2739999,C=118)
	CT(3276)=(S=2740000,E=2740031,C=115)
	CT(3277)=(S=2740032,E=2740095,C=109)
	CT(3278)=(S=2740096,E=2740127,C=113)
	CT(3279)=(S=2740128,E=2740159,C=143)
	CT(3280)=(S=2740160,E=2740191,C=118)
	CT(3281)=(S=2740196,E=2740199,C=116)
	CT(3282)=(S=2740200,E=2740203,C=113)
	CT(3283)=(S=2740204,E=2740207,C=105)
	CT(3284)=(S=2740208,E=2740215,C=112)
	CT(3285)=(S=2740216,E=2740219,C=111)
	CT(3286)=(S=2740220,E=2740223,C=110)
	CT(3287)=(S=2740224,E=2740287,C=105)
	CT(3288)=(S=2740288,E=2740351,C=65)
	CT(3289)=(S=2740352,E=2740415,C=148)
	CT(3290)=(S=2740416,E=2740479,C=105)
	CT(3291)=(S=2740480,E=2740543,C=109)
	CT(3292)=(S=2740544,E=2740607,C=119)
	CT(3293)=(S=2740672,E=2740735,C=106)
	CT(3294)=(S=2740736,E=2740799,C=110)
	CT(3295)=(S=2740800,E=2740863,C=135)
	CT(3296)=(S=2740864,E=2740927,C=130)
	CT(3297)=(S=2740928,E=2740991,C=116)
	CT(3298)=(S=2740992,E=2741023,C=110)
	CT(3299)=(S=2741024,E=2741055,C=138)
	CT(3300)=(S=2741056,E=2741087,C=113)
	CT(3301)=(S=2741088,E=2741119,C=120)
	CT(3302)=(S=2741120,E=2741183,C=122)
	CT(3303)=(S=2741248,E=2741375,C=109)
	CT(3304)=(S=2741376,E=2741503,C=113)
	CT(3305)=(S=2741504,E=2741631,C=105)
	CT(3306)=(S=2741632,E=2741759,C=151)
	CT(3307)=(S=2741760,E=2741887,C=148)
	CT(3308)=(S=2741888,E=2742015,C=134)
	CT(3309)=(S=2742016,E=2742159,C=109)
	CT(3310)=(S=2742160,E=2742175,C=138)
	CT(3311)=(S=2742176,E=2742191,C=110)
	CT(3312)=(S=2742192,E=2742207,C=114)
	CT(3313)=(S=2742208,E=2742223,C=109)
	CT(3314)=(S=2742224,E=2742239,C=127)
	CT(3315)=(S=2742240,E=2742247,C=109)
	CT(3316)=(S=2742248,E=2742255,C=105)
	CT(3317)=(S=2742256,E=2742259,C=106)
	CT(3318)=(S=2742260,E=2742263,C=111)
	CT(3319)=(S=2742264,E=2742267,C=113)
	CT(3320)=(S=2742268,E=2742271,C=137)
	CT(3321)=(S=2742272,E=2742303,C=114)
	CT(3322)=(S=2742304,E=2742335,C=124)
	CT(3323)=(S=2742336,E=2742367,C=114)
	CT(3324)=(S=2742368,E=2742399,C=113)
	CT(3325)=(S=2742400,E=2742415,C=105)
	CT(3326)=(S=2742416,E=2742431,C=139)
	CT(3327)=(S=2742432,E=2742447,C=111)
	CT(3328)=(S=2742464,E=2742479,C=105)
	CT(3329)=(S=2742480,E=2742487,C=126)
	CT(3330)=(S=2742488,E=2742495,C=112)
	CT(3331)=(S=2742496,E=2742499,C=134)
	CT(3332)=(S=2742500,E=2742503,C=121)
	CT(3333)=(S=2742504,E=2742507,C=120)
	CT(3334)=(S=2742508,E=2742511,C=116)
	CT(3335)=(S=2742656,E=2742687,C=120)
	CT(3336)=(S=2742688,E=2742719,C=106)
	CT(3337)=(S=2742720,E=2742723,C=105)
	CT(3338)=(S=2742728,E=2742731,C=112)
	CT(3339)=(S=2742732,E=2742735,C=111)
	CT(3340)=(S=2742736,E=2742739,C=131)
	CT(3341)=(S=2742740,E=2742743,C=113)
	CT(3342)=(S=2742744,E=2742747,C=121)
	CT(3343)=(S=2742748,E=2742751,C=109)
	CT(3344)=(S=2742752,E=2742759,C=106)
	CT(3345)=(S=2742760,E=2742767,C=130)
	CT(3346)=(S=2742768,E=2742775,C=105)
	CT(3347)=(S=2742784,E=2742847,C=119)
	CT(3348)=(S=2742848,E=2742911,C=117)
	CT(3349)=(S=2742912,E=2742975,C=106)
	CT(3350)=(S=2742976,E=2743039,C=110)
	CT(3351)=(S=2743040,E=2743103,C=148)
	CT(3352)=(S=2743104,E=2743167,C=117)
	CT(3353)=(S=2743168,E=2743295,C=111)
	CT(3354)=(S=2743296,E=2743311,C=130)
	CT(3355)=(S=2743312,E=2743327,C=107)
	CT(3356)=(S=2743328,E=2743343,C=131)
	CT(3357)=(S=2743344,E=2743359,C=110)
	CT(3358)=(S=2743360,E=2743391,C=111)
	CT(3359)=(S=2743392,E=2743407,C=147)
	CT(3360)=(S=2743408,E=2743423,C=109)
	CT(3361)=(S=2743424,E=2743439,C=112)
	CT(3362)=(S=2743440,E=2743455,C=135)
	CT(3363)=(S=2743456,E=2743471,C=131)
	CT(3364)=(S=2743472,E=2743487,C=112)
	CT(3365)=(S=2743488,E=2743503,C=131)
	CT(3366)=(S=2743504,E=2743519,C=130)
	CT(3367)=(S=2743520,E=2743551,C=113)
	CT(3368)=(S=2743552,E=2743567,C=105)
	CT(3369)=(S=2743568,E=2743583,C=135)
	CT(3370)=(S=2743584,E=2743615,C=112)
	CT(3371)=(S=2743616,E=2743631,C=131)
	CT(3372)=(S=2743632,E=2743647,C=130)
	CT(3373)=(S=2743648,E=2743663,C=121)
	CT(3374)=(S=2743680,E=2743695,C=106)
	CT(3375)=(S=2743696,E=2743711,C=107)
	CT(3376)=(S=2743712,E=2743727,C=111)
	CT(3377)=(S=2743728,E=2743743,C=123)
	CT(3378)=(S=2743744,E=2743759,C=127)
	CT(3379)=(S=2743760,E=2743791,C=105)
	CT(3380)=(S=2743792,E=2743807,C=116)
	CT(3381)=(S=2743808,E=2743815,C=113)
	CT(3382)=(S=2743816,E=2743823,C=109)
	CT(3383)=(S=2743824,E=2743831,C=114)
	CT(3384)=(S=2743832,E=2743839,C=112)
	CT(3385)=(S=2743840,E=2743847,C=105)
	CT(3386)=(S=2743848,E=2743855,C=111)
	CT(3387)=(S=2743856,E=2743863,C=105)
	CT(3388)=(S=2743864,E=2743871,C=112)
	CT(3389)=(S=2743872,E=2743879,C=111)
	CT(3390)=(S=2743880,E=2743883,C=144)
	CT(3391)=(S=2743884,E=2743887,C=111)
	CT(3392)=(S=2743896,E=2743903,C=112)
	CT(3393)=(S=2743904,E=2743911,C=113)
	CT(3394)=(S=2743912,E=2743919,C=105)
	CT(3395)=(S=2743920,E=2743927,C=146)
	CT(3396)=(S=2743928,E=2743935,C=105)
	CT(3397)=(S=2743936,E=2743943,C=106)
	CT(3398)=(S=2743944,E=2743951,C=105)
	CT(3399)=(S=2743952,E=2743959,C=111)
	CT(3400)=(S=2743968,E=2743975,C=109)
	CT(3401)=(S=2743976,E=2743983,C=106)
	CT(3402)=(S=2743984,E=2743991,C=112)
	CT(3403)=(S=2743992,E=2743999,C=121)
	CT(3404)=(S=2744000,E=2744003,C=124)
	CT(3405)=(S=2744004,E=2744007,C=137)
	CT(3406)=(S=2744008,E=2744011,C=116)
	CT(3407)=(S=2744012,E=2744015,C=147)
	CT(3408)=(S=2744016,E=2744019,C=111)
	CT(3409)=(S=2744020,E=2744023,C=105)
	CT(3410)=(S=2744024,E=2744027,C=137)
	CT(3411)=(S=2744028,E=2744039,C=105)
	CT(3412)=(S=2744040,E=2744043,C=110)
	CT(3413)=(S=2744044,E=2744047,C=116)
	CT(3414)=(S=2744048,E=2744051,C=114)
	CT(3415)=(S=2744052,E=2744055,C=128)
	CT(3416)=(S=2744056,E=2744063,C=116)
	CT(3417)=(S=2744068,E=2744071,C=112)
	CT(3418)=(S=2744072,E=2744075,C=105)
	CT(3419)=(S=2744080,E=2744083,C=112)
	CT(3420)=(S=2744084,E=2744087,C=106)
	CT(3421)=(S=2744088,E=2744091,C=105)
	CT(3422)=(S=2744092,E=2744095,C=120)
	CT(3423)=(S=2744096,E=2744103,C=105)
	CT(3424)=(S=2744104,E=2744107,C=116)
	CT(3425)=(S=2744108,E=2744111,C=111)
	CT(3426)=(S=2744112,E=2744115,C=136)
	CT(3427)=(S=2744116,E=2744119,C=106)
	CT(3428)=(S=2744120,E=2744123,C=109)
	CT(3429)=(S=2744124,E=2744127,C=105)
	CT(3430)=(S=2744128,E=2744131,C=111)
	CT(3431)=(S=2744136,E=2744139,C=126)
	CT(3432)=(S=2744140,E=2744143,C=113)
	CT(3433)=(S=2744144,E=2744147,C=117)
	CT(3434)=(S=2744148,E=2744151,C=130)
	CT(3435)=(S=2744152,E=2744155,C=129)
	CT(3436)=(S=2744156,E=2744159,C=120)
	CT(3437)=(S=2744160,E=2744163,C=149)
	CT(3438)=(S=2744164,E=2744167,C=116)
	CT(3439)=(S=2744168,E=2744171,C=137)
	CT(3440)=(S=2744172,E=2744175,C=140)
	CT(3441)=(S=2744176,E=2744179,C=109)
	CT(3442)=(S=2744180,E=2744183,C=114)
	CT(3443)=(S=2744184,E=2744187,C=105)
	CT(3444)=(S=2744188,E=2744191,C=131)
	CT(3445)=(S=2744192,E=2744195,C=111)
	CT(3446)=(S=2744196,E=2744199,C=97)
	CT(3447)=(S=2744200,E=2744203,C=111)
	CT(3448)=(S=2744204,E=2744207,C=126)
	CT(3449)=(S=2744208,E=2744211,C=111)
	CT(3450)=(S=2744212,E=2744215,C=109)
	CT(3451)=(S=2744216,E=2744219,C=131)
	CT(3452)=(S=2744220,E=2744223,C=116)
	CT(3453)=(S=2744224,E=2744227,C=119)
	CT(3454)=(S=2744228,E=2744231,C=116)
	CT(3455)=(S=2744232,E=2744235,C=122)
	CT(3456)=(S=2744236,E=2744239,C=105)
	CT(3457)=(S=2744240,E=2744243,C=135)
	CT(3458)=(S=2744244,E=2744247,C=112)
	CT(3459)=(S=2744248,E=2744251,C=142)
	CT(3460)=(S=2744252,E=2744255,C=129)
	CT(3461)=(S=2744256,E=2744259,C=126)
	CT(3462)=(S=2744260,E=2744263,C=106)
	CT(3463)=(S=2744264,E=2744267,C=119)
	CT(3464)=(S=2744268,E=2744271,C=147)
	CT(3465)=(S=2744276,E=2744279,C=132)
	CT(3466)=(S=2744280,E=2744283,C=147)
	CT(3467)=(S=2744284,E=2744291,C=113)
	CT(3468)=(S=2744292,E=2744295,C=112)
	CT(3469)=(S=2744296,E=2744299,C=139)
	CT(3470)=(S=2744300,E=2744303,C=135)
	CT(3471)=(S=2744304,E=2744307,C=106)
	CT(3472)=(S=2744308,E=2744311,C=105)
	CT(3473)=(S=2744312,E=2744315,C=124)
	CT(3474)=(S=2744316,E=2744319,C=105)
	CT(3475)=(S=2744320,E=2746367,C=152)
	CT(3476)=(S=2746368,E=2748415,C=106)
	CT(3477)=(S=2748416,E=2748927,C=119)
	CT(3478)=(S=2748928,E=2748935,C=109)
	CT(3479)=(S=2748936,E=2748939,C=112)
	CT(3480)=(S=2748940,E=2748959,C=65)
	CT(3481)=(S=2748960,E=2748967,C=120)
	CT(3482)=(S=2748968,E=2748975,C=105)
	CT(3483)=(S=2748976,E=2749007,C=111)
	CT(3484)=(S=2749008,E=2749011,C=105)
	CT(3485)=(S=2749012,E=2749015,C=137)
	CT(3486)=(S=2749016,E=2749019,C=136)
	CT(3487)=(S=2749020,E=2749023,C=111)
	CT(3488)=(S=2749024,E=2749039,C=115)
	CT(3489)=(S=2749040,E=2749043,C=110)
	CT(3490)=(S=2749044,E=2749047,C=153)
	CT(3491)=(S=2749048,E=2749051,C=105)
	CT(3492)=(S=2749052,E=2749055,C=151)
	CT(3493)=(S=2749056,E=2749059,C=137)
	CT(3494)=(S=2749064,E=2749067,C=110)
	CT(3495)=(S=2749068,E=2749071,C=128)
	CT(3496)=(S=2749072,E=2749079,C=113)
	CT(3497)=(S=2749088,E=2749103,C=105)
	CT(3498)=(S=2749104,E=2749119,C=124)
	CT(3499)=(S=2749120,E=2749183,C=105)
	CT(3500)=(S=2749184,E=2749439,C=137)
	CT(3501)=(S=2749440,E=2749695,C=120)
	CT(3502)=(S=2749952,E=2750463,C=105)
	CT(3503)=(S=2750464,E=2751487,C=134)
	CT(3504)=(S=2751488,E=2752511,C=65)
	CT(3505)=(S=2752512,E=2752515,C=1)
	CT(3506)=(S=2752516,E=2752519,C=84)
	CT(3507)=(S=2752520,E=2752539,C=1)
	CT(3508)=(S=2752540,E=2752543,C=5)
	CT(3509)=(S=2752544,E=2752575,C=1)
	CT(3510)=(S=2752576,E=2752639,C=9)
	CT(3511)=(S=2752640,E=2752827,C=1)
	CT(3512)=(S=2752828,E=2752831,C=5)
	CT(3513)=(S=2752832,E=2752895,C=11)
	CT(3514)=(S=2752896,E=2753023,C=1)
	CT(3515)=(S=2753024,E=2753535,C=8)
	CT(3516)=(S=2753536,E=2754559,C=1)
	CT(3517)=(S=2754560,E=2764799,C=6)
	CT(3518)=(S=2764800,E=2767871,C=1)
	CT(3519)=(S=2767872,E=2768383,C=7)
	CT(3520)=(S=2768384,E=2768559,C=1)
	CT(3521)=(S=2768560,E=2768563,C=74)
	CT(3522)=(S=2768564,E=2768575,C=1)
	CT(3523)=(S=2768576,E=2768639,C=0)
	CT(3524)=(S=2768640,E=2768895,C=1)
	CT(3525)=(S=2768896,E=2772991,C=9)
	CT(3526)=(S=2772992,E=2773503,C=1)
	CT(3527)=(S=2773504,E=2773759,C=6)
	CT(3528)=(S=2773760,E=2773823,C=2)
	CT(3529)=(S=2773824,E=2773843,C=1)
	CT(3530)=(S=2773844,E=2773847,C=77)
	CT(3531)=(S=2773848,E=2777087,C=1)
	CT(3532)=(S=2777088,E=2777151,C=11)
	CT(3533)=(S=2777152,E=2777191,C=1)
	CT(3534)=(S=2777192,E=2777195,C=3)
	CT(3535)=(S=2777196,E=2777599,C=1)
	CT(3536)=(S=2777600,E=2777855,C=8)
	CT(3537)=(S=2777856,E=2777971,C=1)
	CT(3538)=(S=2777972,E=2777975,C=84)
	CT(3539)=(S=2777976,E=2777983,C=1)
	CT(3540)=(S=2777984,E=2778111,C=2)
	CT(3541)=(S=2778112,E=2779135,C=1)
	CT(3542)=(S=2779136,E=2781183,C=4)
	CT(3543)=(S=2781184,E=2783231,C=11)
	CT(3544)=(S=2783232,E=2784031,C=1)
	CT(3545)=(S=2784032,E=2784035,C=2)
	CT(3546)=(S=2784036,E=2784255,C=1)
	CT(3547)=(S=2784256,E=2785279,C=2)
	CT(3548)=(S=2785280,E=2789375,C=1)
	CT(3549)=(S=2789376,E=2791423,C=2)
	CT(3550)=(S=2791424,E=2792447,C=5)
	CT(3551)=(S=2792448,E=2792479,C=1)
	CT(3552)=(S=2792480,E=2792483,C=5)
	CT(3553)=(S=2792484,E=2800507,C=1)
	CT(3554)=(S=2800508,E=2800511,C=2)
	CT(3555)=(S=2800512,E=2800639,C=1)
	CT(3556)=(S=2800640,E=2801663,C=5)
	CT(3557)=(S=2801664,E=2803711,C=1)
	CT(3558)=(S=2803712,E=2803967,C=8)
	CT(3559)=(S=2803968,E=2804095,C=1)
	CT(3560)=(S=2804096,E=2804223,C=77)
	CT(3561)=(S=2804224,E=2814207,C=1)
	CT(3562)=(S=2814208,E=2814463,C=0)
	CT(3563)=(S=2814464,E=2818047,C=1)
	CT(3564)=(S=2818048,E=2875391,C=2)
	CT(3565)=(S=2875392,E=2875395,C=4)
	CT(3566)=(S=2875396,E=2875399,C=8)
	CT(3567)=(S=2875400,E=2875403,C=4)
	CT(3568)=(S=2875404,E=2875407,C=1)
	CT(3569)=(S=2875408,E=2875411,C=74)
	CT(3570)=(S=2875412,E=2875415,C=9)
	CT(3571)=(S=2875416,E=2875419,C=1)
	CT(3572)=(S=2875420,E=2875423,C=8)
	CT(3573)=(S=2875424,E=2875427,C=2)
	CT(3574)=(S=2875428,E=2875431,C=154)
	CT(3575)=(S=2875432,E=2875435,C=155)
	CT(3576)=(S=2875436,E=2875439,C=1)
	CT(3577)=(S=2875440,E=2875443,C=8)
	CT(3578)=(S=2875444,E=2875451,C=1)
	CT(3579)=(S=2875452,E=2875455,C=0)
	CT(3580)=(S=2875456,E=2875467,C=1)
	CT(3581)=(S=2875468,E=2875471,C=0)
	CT(3582)=(S=2875472,E=2875475,C=1)
	CT(3583)=(S=2875476,E=2875479,C=155)
	CT(3584)=(S=2875480,E=2875483,C=5)
	CT(3585)=(S=2875484,E=2875487,C=8)
	CT(3586)=(S=2875488,E=2875491,C=4)
	CT(3587)=(S=2875492,E=2875495,C=1)
	CT(3588)=(S=2875496,E=2875499,C=6)
	CT(3589)=(S=2875500,E=2875511,C=84)
	CT(3590)=(S=2875512,E=2875515,C=73)
	CT(3591)=(S=2875516,E=2875519,C=156)
	CT(3592)=(S=2875520,E=2875531,C=4)
	CT(3593)=(S=2875532,E=2875539,C=1)
	CT(3594)=(S=2875540,E=2875543,C=8)
	CT(3595)=(S=2875544,E=2875547,C=7)
	CT(3596)=(S=2875548,E=2875551,C=4)
	CT(3597)=(S=2875552,E=2875555,C=1)
	CT(3598)=(S=2875556,E=2875559,C=4)
	CT(3599)=(S=2875560,E=2875563,C=74)
	CT(3600)=(S=2875564,E=2875567,C=4)
	CT(3601)=(S=2875568,E=2875571,C=1)
	CT(3602)=(S=2875572,E=2875575,C=4)
	CT(3603)=(S=2875576,E=2875579,C=1)
	CT(3604)=(S=2875580,E=2875583,C=73)
	CT(3605)=(S=2875584,E=2875587,C=90)
	CT(3606)=(S=2875588,E=2875591,C=8)
	CT(3607)=(S=2875592,E=2875611,C=1)
	CT(3608)=(S=2875612,E=2875615,C=4)
	CT(3609)=(S=2875616,E=2875619,C=1)
	CT(3610)=(S=2875620,E=2875627,C=8)
	CT(3611)=(S=2875628,E=2875631,C=77)
	CT(3612)=(S=2875632,E=2875635,C=1)
	CT(3613)=(S=2875636,E=2875639,C=8)
	CT(3614)=(S=2875640,E=2875641,C=9)
	CT(3615)=(S=2875642,E=2875643,C=73)
	CT(3616)=(S=2875644,E=2875651,C=4)
	CT(3617)=(S=2875652,E=2875655,C=7)
	CT(3618)=(S=2875656,E=2875659,C=8)
	CT(3619)=(S=2875660,E=2875663,C=0)
	CT(3620)=(S=2875664,E=2875675,C=4)
	CT(3621)=(S=2875676,E=2875679,C=8)
	CT(3622)=(S=2875680,E=2875683,C=0)
	CT(3623)=(S=2875684,E=2875687,C=5)
	CT(3624)=(S=2875688,E=2875691,C=157)
	CT(3625)=(S=2875692,E=2875695,C=8)
	CT(3626)=(S=2875696,E=2875699,C=0)
	CT(3627)=(S=2875700,E=2875703,C=4)
	CT(3628)=(S=2875704,E=2875707,C=8)
	CT(3629)=(S=2875708,E=2875711,C=0)
	CT(3630)=(S=2875712,E=2875715,C=74)
	CT(3631)=(S=2875716,E=2875723,C=4)
	CT(3632)=(S=2875724,E=2875727,C=1)
	CT(3633)=(S=2875728,E=2875731,C=4)
	CT(3634)=(S=2875732,E=2875735,C=1)
	CT(3635)=(S=2875736,E=2875739,C=2)
	CT(3636)=(S=2875740,E=2875743,C=4)
	CT(3637)=(S=2875744,E=2875747,C=77)
	CT(3638)=(S=2875748,E=2875755,C=8)
	CT(3639)=(S=2875756,E=2875759,C=5)
	CT(3640)=(S=2875760,E=2875763,C=0)
	CT(3641)=(S=2875764,E=2875767,C=4)
	CT(3642)=(S=2875768,E=2875775,C=1)
	CT(3643)=(S=2875776,E=2875779,C=0)
	CT(3644)=(S=2875780,E=2875783,C=8)
	CT(3645)=(S=2875784,E=2875787,C=157)
	CT(3646)=(S=2875788,E=2875791,C=1)
	CT(3647)=(S=2875792,E=2875795,C=0)
	CT(3648)=(S=2875796,E=2875799,C=84)
	CT(3649)=(S=2875800,E=2875803,C=7)
	CT(3650)=(S=2875808,E=2875819,C=4)
	CT(3651)=(S=2875820,E=2875823,C=1)
	CT(3652)=(S=2875824,E=2875827,C=0)
	CT(3653)=(S=2875828,E=2875835,C=1)
	CT(3654)=(S=2875836,E=2875843,C=4)
	CT(3655)=(S=2875844,E=2875847,C=8)
	CT(3656)=(S=2875848,E=2875851,C=73)
	CT(3657)=(S=2875852,E=2875855,C=84)
	CT(3658)=(S=2875856,E=2875859,C=1)
	CT(3659)=(S=2875860,E=2875863,C=4)
	CT(3660)=(S=2875864,E=2875895,C=1)
	CT(3661)=(S=2875896,E=2875899,C=4)
	CT(3662)=(S=2875900,E=2875903,C=1)
	CT(3663)=(S=2875904,E=2875905,C=0)
	CT(3664)=(S=2875906,E=2875907,C=4)
	CT(3665)=(S=2875908,E=2875911,C=8)
	CT(3666)=(S=2875912,E=2875915,C=73)
	CT(3667)=(S=2875916,E=2875919,C=86)
	CT(3668)=(S=2875920,E=2875927,C=8)
	CT(3669)=(S=2875928,E=2875935,C=4)
	CT(3670)=(S=2875936,E=2876027,C=1)
	CT(3671)=(S=2876028,E=2876031,C=8)
	CT(3672)=(S=2876032,E=2876119,C=1)
	CT(3673)=(S=2876120,E=2876123,C=73)
	CT(3674)=(S=2876124,E=2876127,C=88)
	CT(3675)=(S=2876128,E=2876131,C=5)
	CT(3676)=(S=2876132,E=2876135,C=8)
	CT(3677)=(S=2876136,E=2876139,C=9)
	CT(3678)=(S=2876140,E=2876171,C=1)
	CT(3679)=(S=2876172,E=2876175,C=8)
	CT(3680)=(S=2876176,E=2876183,C=4)
	CT(3681)=(S=2876184,E=2876187,C=9)
	CT(3682)=(S=2876188,E=2876267,C=1)
	CT(3683)=(S=2876268,E=2876271,C=0)
	CT(3684)=(S=2876272,E=2876275,C=8)
	CT(3685)=(S=2876276,E=2876283,C=6)
	CT(3686)=(S=2876284,E=2876287,C=0)
	CT(3687)=(S=2876288,E=2876295,C=4)
	CT(3688)=(S=2876296,E=2876307,C=1)
	CT(3689)=(S=2876308,E=2876311,C=74)
	CT(3690)=(S=2876312,E=2876343,C=1)
	CT(3691)=(S=2876344,E=2876347,C=4)
	CT(3692)=(S=2876348,E=2876383,C=1)
	CT(3693)=(S=2876384,E=2876387,C=4)
	CT(3694)=(S=2876388,E=2876391,C=7)
	CT(3695)=(S=2876392,E=2876395,C=1)
	CT(3696)=(S=2876396,E=2876399,C=8)
	CT(3697)=(S=2876400,E=2876403,C=7)
	CT(3698)=(S=2876404,E=2876407,C=4)
	CT(3699)=(S=2876408,E=2876487,C=1)
	CT(3700)=(S=2876488,E=2876491,C=4)
	CT(3701)=(S=2876492,E=2876495,C=1)
	CT(3702)=(S=2876496,E=2876499,C=3)
	CT(3703)=(S=2876500,E=2876503,C=154)
	CT(3704)=(S=2876504,E=2876507,C=8)
	CT(3705)=(S=2876508,E=2876515,C=4)
	CT(3706)=(S=2876516,E=2876519,C=1)
	CT(3707)=(S=2876520,E=2876523,C=2)
	CT(3708)=(S=2876524,E=2876527,C=156)
	CT(3709)=(S=2876528,E=2876531,C=4)
	CT(3710)=(S=2876532,E=2876539,C=1)
	CT(3711)=(S=2876540,E=2876543,C=8)
	CT(3712)=(S=2876544,E=2876547,C=92)
	CT(3713)=(S=2876548,E=2876555,C=1)
	CT(3714)=(S=2876556,E=2876563,C=4)
	CT(3715)=(S=2876564,E=2876571,C=1)
	CT(3716)=(S=2876572,E=2876575,C=77)
	CT(3717)=(S=2876576,E=2876579,C=6)
	CT(3718)=(S=2876584,E=2876595,C=4)
	CT(3719)=(S=2876596,E=2876599,C=1)
	CT(3720)=(S=2876600,E=2876603,C=73)
	CT(3721)=(S=2876604,E=2876607,C=1)
	CT(3722)=(S=2876608,E=2876611,C=75)
	CT(3723)=(S=2876612,E=2876615,C=3)
	CT(3724)=(S=2876616,E=2876619,C=8)
	CT(3725)=(S=2876620,E=2876623,C=1)
	CT(3726)=(S=2876624,E=2876627,C=84)
	CT(3727)=(S=2876628,E=2876631,C=7)
	CT(3728)=(S=2876632,E=2876635,C=8)
	CT(3729)=(S=2876636,E=2876647,C=4)
	CT(3730)=(S=2876648,E=2876651,C=7)
	CT(3731)=(S=2876652,E=2876655,C=4)
	CT(3732)=(S=2876656,E=2876659,C=1)
	CT(3733)=(S=2876664,E=2876671,C=5)
	CT(3734)=(S=2876672,E=2876675,C=4)
	CT(3735)=(S=2876676,E=2876679,C=8)
	CT(3736)=(S=2876680,E=2876683,C=4)
	CT(3737)=(S=2876684,E=2876687,C=84)
	CT(3738)=(S=2876688,E=2876691,C=1)
	CT(3739)=(S=2876692,E=2876695,C=74)
	CT(3740)=(S=2876696,E=2876699,C=4)
	CT(3741)=(S=2876700,E=2876703,C=8)
	CT(3742)=(S=2876704,E=2876707,C=2)
	CT(3743)=(S=2876708,E=2876711,C=8)
	CT(3744)=(S=2876712,E=2876715,C=1)
	CT(3745)=(S=2876716,E=2876719,C=3)
	CT(3746)=(S=2876720,E=2876723,C=1)
	CT(3747)=(S=2876724,E=2876727,C=8)
	CT(3748)=(S=2876728,E=2876731,C=1)
	CT(3749)=(S=2876732,E=2876739,C=0)
	CT(3750)=(S=2876744,E=2876747,C=4)
	CT(3751)=(S=2876748,E=2876751,C=3)
	CT(3752)=(S=2876752,E=2876755,C=4)
	CT(3753)=(S=2876756,E=2876759,C=7)
	CT(3754)=(S=2876760,E=2876767,C=4)
	CT(3755)=(S=2876768,E=2876771,C=1)
	CT(3756)=(S=2876772,E=2876779,C=4)
	CT(3757)=(S=2876780,E=2876783,C=1)
	CT(3758)=(S=2876784,E=2876791,C=8)
	CT(3759)=(S=2876792,E=2876795,C=1)
	CT(3760)=(S=2876796,E=2876799,C=2)
	CT(3761)=(S=2876800,E=2876803,C=7)
	CT(3762)=(S=2876804,E=2876807,C=3)
	CT(3763)=(S=2876808,E=2876819,C=1)
	CT(3764)=(S=2876820,E=2876823,C=3)
	CT(3765)=(S=2876824,E=2876827,C=8)
	CT(3766)=(S=2876828,E=2876831,C=7)
	CT(3767)=(S=2876832,E=2876835,C=4)
	CT(3768)=(S=2876840,E=2876871,C=1)
	CT(3769)=(S=2876872,E=2876875,C=4)
	CT(3770)=(S=2876876,E=2876879,C=74)
	CT(3771)=(S=2876880,E=2876883,C=84)
	CT(3772)=(S=2876884,E=2876887,C=8)
	CT(3773)=(S=2876888,E=2876895,C=1)
	CT(3774)=(S=2876896,E=2876899,C=4)
	CT(3775)=(S=2876900,E=2876903,C=75)
	CT(3776)=(S=2876904,E=2876911,C=1)
	CT(3777)=(S=2876912,E=2876919,C=2)
	CT(3778)=(S=2876920,E=2876927,C=74)
	CT(3779)=(S=2876928,E=2876931,C=6)
	CT(3780)=(S=2876932,E=2876933,C=74)
	CT(3781)=(S=2876934,E=2876934,C=158)
	CT(3782)=(S=2876935,E=2876935,C=74)
	CT(3783)=(S=2876936,E=2876947,C=8)
	CT(3784)=(S=2876948,E=2876951,C=1)
	CT(3785)=(S=2876952,E=2876955,C=7)
	CT(3786)=(S=2876956,E=2876959,C=2)
	CT(3787)=(S=2876960,E=2876963,C=1)
	CT(3788)=(S=2876964,E=2876975,C=4)
	CT(3789)=(S=2876976,E=2876979,C=0)
	CT(3790)=(S=2876980,E=2876983,C=2)
	CT(3791)=(S=2876984,E=2876987,C=7)
	CT(3792)=(S=2876988,E=2876991,C=84)
	CT(3793)=(S=2876992,E=2876995,C=0)
	CT(3794)=(S=2876996,E=2877003,C=1)
	CT(3795)=(S=2877004,E=2877011,C=6)
	CT(3796)=(S=2877012,E=2877015,C=1)
	CT(3797)=(S=2877016,E=2877019,C=8)
	CT(3798)=(S=2877020,E=2877023,C=77)
	CT(3799)=(S=2877024,E=2877027,C=8)
	CT(3800)=(S=2877028,E=2877031,C=0)
	CT(3801)=(S=2877032,E=2877035,C=4)
	CT(3802)=(S=2877036,E=2877039,C=0)
	CT(3803)=(S=2877040,E=2877047,C=8)
	CT(3804)=(S=2877048,E=2877051,C=84)
	CT(3805)=(S=2877052,E=2877055,C=1)
	CT(3806)=(S=2877056,E=2877059,C=74)
	CT(3807)=(S=2877060,E=2877063,C=4)
	CT(3808)=(S=2877064,E=2877067,C=1)
	CT(3809)=(S=2877068,E=2877075,C=8)
	CT(3810)=(S=2877076,E=2877079,C=4)
	CT(3811)=(S=2877080,E=2877083,C=74)
	CT(3812)=(S=2877084,E=2877091,C=4)
	CT(3813)=(S=2877092,E=2877095,C=8)
	CT(3814)=(S=2877096,E=2877099,C=1)
	CT(3815)=(S=2877100,E=2877103,C=4)
	CT(3816)=(S=2877104,E=2877107,C=0)
	CT(3817)=(S=2877108,E=2877111,C=7)
	CT(3818)=(S=2877112,E=2877115,C=4)
	CT(3819)=(S=2877116,E=2877119,C=8)
	CT(3820)=(S=2877120,E=2877123,C=86)
	CT(3821)=(S=2877124,E=2877131,C=4)
	CT(3822)=(S=2877132,E=2877135,C=8)
	CT(3823)=(S=2877136,E=2877139,C=2)
	CT(3824)=(S=2877140,E=2877143,C=4)
	CT(3825)=(S=2877144,E=2877147,C=6)
	CT(3826)=(S=2877148,E=2877183,C=1)
	CT(3827)=(S=2877184,E=2877187,C=8)
	CT(3828)=(S=2877188,E=2877191,C=5)
	CT(3829)=(S=2877192,E=2877195,C=8)
	CT(3830)=(S=2877196,E=2877199,C=7)
	CT(3831)=(S=2877200,E=2877203,C=8)
	CT(3832)=(S=2877204,E=2877207,C=84)
	CT(3833)=(S=2877208,E=2877211,C=0)
	CT(3834)=(S=2877212,E=2877215,C=88)
	CT(3835)=(S=2877216,E=2877231,C=1)
	CT(3836)=(S=2877232,E=2877243,C=4)
	CT(3837)=(S=2877244,E=2877247,C=77)
	CT(3838)=(S=2877248,E=2877251,C=86)
	CT(3839)=(S=2877252,E=2877255,C=0)
	CT(3840)=(S=2877256,E=2877259,C=5)
	CT(3841)=(S=2877260,E=2877263,C=84)
	CT(3842)=(S=2877264,E=2877295,C=1)
	CT(3843)=(S=2877296,E=2877299,C=92)
	CT(3844)=(S=2877300,E=2877311,C=4)
	CT(3845)=(S=2877312,E=2877313,C=74)
	CT(3846)=(S=2877314,E=2877314,C=7)
	CT(3847)=(S=2877315,E=2877315,C=89)
	CT(3848)=(S=2877316,E=2877319,C=4)
	CT(3849)=(S=2877320,E=2877367,C=1)
	CT(3850)=(S=2877368,E=2877371,C=8)
	CT(3851)=(S=2877372,E=2877375,C=2)
	CT(3852)=(S=2877376,E=2877379,C=73)
	CT(3853)=(S=2877380,E=2877383,C=8)
	CT(3854)=(S=2877384,E=2877387,C=0)
	CT(3855)=(S=2877392,E=2877395,C=75)
	CT(3856)=(S=2877396,E=2877403,C=4)
	CT(3857)=(S=2877404,E=2877407,C=2)
	CT(3858)=(S=2877408,E=2877411,C=5)
	CT(3859)=(S=2877412,E=2877415,C=10)
	CT(3860)=(S=2877416,E=2877427,C=4)
	CT(3861)=(S=2877428,E=2877431,C=2)
	CT(3862)=(S=2877432,E=2877439,C=4)
	CT(3863)=(S=2877440,E=2878463,C=2)
	CT(3864)=(S=2878464,E=2879283,C=1)
	CT(3865)=(S=2879284,E=2879295,C=4)
	CT(3866)=(S=2879296,E=2879299,C=8)
	CT(3867)=(S=2879300,E=2879303,C=4)
	CT(3868)=(S=2879304,E=2879307,C=84)
	CT(3869)=(S=2879308,E=2879315,C=4)
	CT(3870)=(S=2879316,E=2879323,C=8)
	CT(3871)=(S=2879324,E=2879331,C=73)
	CT(3872)=(S=2879332,E=2879335,C=7)
	CT(3873)=(S=2879336,E=2879339,C=2)
	CT(3874)=(S=2879340,E=2879347,C=4)
	CT(3875)=(S=2879348,E=2879355,C=1)
	CT(3876)=(S=2879356,E=2879367,C=4)
	CT(3877)=(S=2879368,E=2879371,C=0)
	CT(3878)=(S=2879372,E=2879375,C=36)
	CT(3879)=(S=2879376,E=2879379,C=4)
	CT(3880)=(S=2879380,E=2879383,C=11)
	CT(3881)=(S=2879384,E=2879387,C=4)
	CT(3882)=(S=2879388,E=2879399,C=8)
	CT(3883)=(S=2879400,E=2879403,C=4)
	CT(3884)=(S=2879404,E=2879411,C=1)
	CT(3885)=(S=2879412,E=2879415,C=73)
	CT(3886)=(S=2879416,E=2879423,C=11)
	CT(3887)=(S=2879424,E=2879451,C=4)
	CT(3888)=(S=2879452,E=2879459,C=11)
	CT(3889)=(S=2879460,E=2879463,C=4)
	CT(3890)=(S=2879464,E=2879464,C=8)
	CT(3891)=(S=2879465,E=2879467,C=5)
	CT(3892)=(S=2879468,E=2879479,C=4)
	CT(3893)=(S=2879480,E=2879483,C=73)
	CT(3894)=(S=2879484,E=2879487,C=5)
	CT(3895)=(S=2879488,E=2879491,C=1)
	CT(3896)=(S=2879492,E=2879499,C=4)
	CT(3897)=(S=2879500,E=2879507,C=8)
	CT(3898)=(S=2879508,E=2879511,C=5)
	CT(3899)=(S=2879512,E=2879515,C=9)
	CT(3900)=(S=2879516,E=2879519,C=8)
	CT(3901)=(S=2879520,E=2879523,C=73)
	CT(3902)=(S=2879524,E=2879527,C=8)
	CT(3903)=(S=2879528,E=2879531,C=0)
	CT(3904)=(S=2879532,E=2879535,C=9)
	CT(3905)=(S=2879536,E=2879539,C=1)
	CT(3906)=(S=2879540,E=2879543,C=84)
	CT(3907)=(S=2879544,E=2879551,C=1)
	CT(3908)=(S=2879552,E=2879555,C=4)
	CT(3909)=(S=2879556,E=2879567,C=1)
	CT(3910)=(S=2879568,E=2879571,C=74)
	CT(3911)=(S=2879572,E=2879575,C=1)
	CT(3912)=(S=2879576,E=2879579,C=159)
	CT(3913)=(S=2879580,E=2879583,C=8)
	CT(3914)=(S=2879584,E=2879587,C=0)
	CT(3915)=(S=2879588,E=2879591,C=84)
	CT(3916)=(S=2879592,E=2879599,C=9)
	CT(3917)=(S=2879600,E=2879603,C=4)
	CT(3918)=(S=2879604,E=2879607,C=8)
	CT(3919)=(S=2879608,E=2879611,C=0)
	CT(3920)=(S=2879612,E=2879627,C=1)
	CT(3921)=(S=2879628,E=2879631,C=160)
	CT(3922)=(S=2879632,E=2879635,C=0)
	CT(3923)=(S=2879636,E=2879639,C=74)
	CT(3924)=(S=2879640,E=2879643,C=9)
	CT(3925)=(S=2879644,E=2879711,C=1)
	CT(3926)=(S=2879712,E=2879719,C=74)
	CT(3927)=(S=2879720,E=2879723,C=0)
	CT(3928)=(S=2879724,E=2879767,C=1)
	CT(3929)=(S=2879768,E=2879775,C=4)
	CT(3930)=(S=2879776,E=2879779,C=9)
	CT(3931)=(S=2879780,E=2879787,C=4)
	CT(3932)=(S=2879788,E=2879791,C=8)
	CT(3933)=(S=2879792,E=2879795,C=1)
	CT(3934)=(S=2879796,E=2879799,C=0)
	CT(3935)=(S=2879800,E=2879803,C=3)
	CT(3936)=(S=2879804,E=2879815,C=4)
	CT(3937)=(S=2879816,E=2879819,C=8)
	CT(3938)=(S=2879820,E=2879839,C=1)
	CT(3939)=(S=2879840,E=2879843,C=5)
	CT(3940)=(S=2879844,E=2879847,C=8)
	CT(3941)=(S=2879848,E=2879855,C=6)
	CT(3942)=(S=2879856,E=2879859,C=1)
	CT(3943)=(S=2879860,E=2879879,C=4)
	CT(3944)=(S=2879880,E=2879883,C=161)
	CT(3945)=(S=2879884,E=2879895,C=4)
	CT(3946)=(S=2879896,E=2879903,C=8)
	CT(3947)=(S=2879904,E=2879907,C=9)
	CT(3948)=(S=2879908,E=2879911,C=162)
	CT(3949)=(S=2879912,E=2879931,C=1)
	CT(3950)=(S=2879932,E=2879935,C=0)
	CT(3951)=(S=2879936,E=2879939,C=4)
	CT(3952)=(S=2879940,E=2879943,C=1)
	CT(3953)=(S=2879944,E=2879947,C=0)
	CT(3954)=(S=2879948,E=2879951,C=8)
	CT(3955)=(S=2879952,E=2879987,C=1)
	CT(3956)=(S=2879988,E=2879991,C=4)
	CT(3957)=(S=2879992,E=2879999,C=1)
	CT(3958)=(S=2880000,E=2880003,C=8)
	CT(3959)=(S=2880004,E=2880007,C=7)
	CT(3960)=(S=2880008,E=2880031,C=1)
	CT(3961)=(S=2880032,E=2880035,C=8)
	CT(3962)=(S=2880036,E=2880039,C=4)
	CT(3963)=(S=2880040,E=2880043,C=0)
	CT(3964)=(S=2880044,E=2880067,C=1)
	CT(3965)=(S=2880068,E=2880071,C=0)
	CT(3966)=(S=2880072,E=2880099,C=1)
	CT(3967)=(S=2880100,E=2880103,C=77)
	CT(3968)=(S=2880104,E=2880107,C=4)
	CT(3969)=(S=2880108,E=2880111,C=8)
	CT(3970)=(S=2880112,E=2880115,C=6)
	CT(3971)=(S=2880116,E=2880127,C=4)
	CT(3972)=(S=2880128,E=2880131,C=8)
	CT(3973)=(S=2880136,E=2880139,C=2)
	CT(3974)=(S=2880140,E=2880143,C=8)
	CT(3975)=(S=2880144,E=2880171,C=1)
	CT(3976)=(S=2880172,E=2880175,C=8)
	CT(3977)=(S=2880176,E=2880179,C=77)
	CT(3978)=(S=2880180,E=2880183,C=1)
	CT(3979)=(S=2880184,E=2880187,C=7)
	CT(3980)=(S=2880188,E=2880199,C=1)
	CT(3981)=(S=2880200,E=2880203,C=8)
	CT(3982)=(S=2880204,E=2880207,C=1)
	CT(3983)=(S=2880208,E=2880215,C=4)
	CT(3984)=(S=2880216,E=2880223,C=1)
	CT(3985)=(S=2880224,E=2880231,C=4)
	CT(3986)=(S=2880232,E=2880239,C=8)
	CT(3987)=(S=2880240,E=2880243,C=92)
	CT(3988)=(S=2880244,E=2880251,C=4)
	CT(3989)=(S=2880252,E=2880255,C=1)
	CT(3990)=(S=2880256,E=2880259,C=0)
	CT(3991)=(S=2880260,E=2880275,C=1)
	CT(3992)=(S=2880276,E=2880276,C=0)
	CT(3993)=(S=2880277,E=2880279,C=73)
	CT(3994)=(S=2880280,E=2880283,C=1)
	CT(3995)=(S=2880284,E=2880291,C=8)
	CT(3996)=(S=2880292,E=2880295,C=4)
	CT(3997)=(S=2880296,E=2880311,C=8)
	CT(3998)=(S=2880312,E=2880319,C=73)
	CT(3999)=(S=2880320,E=2880331,C=8)
	CT(4000)=(S=2880332,E=2880343,C=4)
	CT(4001)=(S=2880344,E=2880347,C=1)
	CT(4002)=(S=2880348,E=2880351,C=8)
	CT(4003)=(S=2880352,E=2880355,C=75)
	CT(4004)=(S=2880356,E=2880371,C=8)
	CT(4005)=(S=2880372,E=2880375,C=0)
	CT(4006)=(S=2880376,E=2880379,C=8)
	CT(4007)=(S=2880380,E=2880383,C=10)
	CT(4008)=(S=2880384,E=2880395,C=1)
	CT(4009)=(S=2880396,E=2880399,C=74)
	CT(4010)=(S=2880400,E=2880407,C=1)
	CT(4011)=(S=2880408,E=2880411,C=74)
	CT(4012)=(S=2880412,E=2880415,C=1)
	CT(4013)=(S=2880416,E=2880419,C=92)
	CT(4014)=(S=2880420,E=2880423,C=8)
	CT(4015)=(S=2880424,E=2880427,C=1)
	CT(4016)=(S=2880428,E=2880431,C=4)
	CT(4017)=(S=2880432,E=2880435,C=8)
	CT(4018)=(S=2880436,E=2880439,C=1)
	CT(4019)=(S=2880440,E=2880443,C=74)
	CT(4020)=(S=2880444,E=2880447,C=1)
	CT(4021)=(S=2880448,E=2880451,C=8)
	CT(4022)=(S=2880452,E=2880455,C=73)
	CT(4023)=(S=2880456,E=2880459,C=0)
	CT(4024)=(S=2880460,E=2880463,C=84)
	CT(4025)=(S=2880464,E=2880467,C=8)
	CT(4026)=(S=2880468,E=2880471,C=4)
	CT(4027)=(S=2880472,E=2880475,C=6)
	CT(4028)=(S=2880476,E=2880483,C=8)
	CT(4029)=(S=2880484,E=2880491,C=1)
	CT(4030)=(S=2880492,E=2880495,C=0)
	CT(4031)=(S=2880496,E=2880499,C=8)
	CT(4032)=(S=2880500,E=2880503,C=1)
	CT(4033)=(S=2880504,E=2880507,C=8)
	CT(4034)=(S=2880508,E=2880511,C=9)
	CT(4035)=(S=2880512,E=2880767,C=2)
	CT(4036)=(S=2880768,E=2880775,C=4)
	CT(4037)=(S=2880776,E=2880779,C=77)
	CT(4038)=(S=2880780,E=2880783,C=4)
	CT(4039)=(S=2880784,E=2880787,C=7)
	CT(4040)=(S=2880788,E=2880791,C=4)
	CT(4041)=(S=2880792,E=2880799,C=0)
	CT(4042)=(S=2880800,E=2880803,C=86)
	CT(4043)=(S=2880804,E=2880811,C=0)
	CT(4044)=(S=2880812,E=2880815,C=155)
	CT(4045)=(S=2880816,E=2880819,C=8)
	CT(4046)=(S=2880820,E=2880823,C=73)
	CT(4047)=(S=2880824,E=2880827,C=91)
	CT(4048)=(S=2880828,E=2880831,C=7)
	CT(4049)=(S=2880832,E=2880847,C=0)
	CT(4050)=(S=2880848,E=2880851,C=73)
	CT(4051)=(S=2880852,E=2880855,C=75)
	CT(4052)=(S=2880856,E=2880859,C=4)
	CT(4053)=(S=2880860,E=2880863,C=75)
	CT(4054)=(S=2880864,E=2880867,C=7)
	CT(4055)=(S=2880868,E=2880871,C=4)
	CT(4056)=(S=2880872,E=2880879,C=7)
	CT(4057)=(S=2880880,E=2880883,C=0)
	CT(4058)=(S=2880884,E=2880887,C=84)
	CT(4059)=(S=2880892,E=2880895,C=5)
	CT(4060)=(S=2880896,E=2880899,C=77)
	CT(4061)=(S=2880900,E=2880903,C=0)
	CT(4062)=(S=2880904,E=2880907,C=4)
	CT(4063)=(S=2880908,E=2880911,C=84)
	CT(4064)=(S=2880912,E=2880915,C=3)
	CT(4065)=(S=2880916,E=2880919,C=4)
	CT(4066)=(S=2880920,E=2880923,C=0)
	CT(4067)=(S=2880924,E=2880927,C=4)
	CT(4068)=(S=2880928,E=2880939,C=0)
	CT(4069)=(S=2880940,E=2880943,C=73)
	CT(4070)=(S=2880944,E=2880947,C=0)
	CT(4071)=(S=2880948,E=2880959,C=74)
	CT(4072)=(S=2880960,E=2880963,C=84)
	CT(4073)=(S=2880964,E=2880967,C=5)
	CT(4074)=(S=2880968,E=2880971,C=86)
	CT(4075)=(S=2880972,E=2880975,C=77)
	CT(4076)=(S=2880976,E=2880979,C=4)
	CT(4077)=(S=2880980,E=2880983,C=90)
	CT(4078)=(S=2880984,E=2880987,C=86)
	CT(4079)=(S=2880992,E=2880995,C=0)
	CT(4080)=(S=2880996,E=2880999,C=7)
	CT(4081)=(S=2881000,E=2881003,C=84)
	CT(4082)=(S=2881004,E=2881007,C=75)
	CT(4083)=(S=2881008,E=2881011,C=73)
	CT(4084)=(S=2881012,E=2881015,C=84)
	CT(4085)=(S=2881016,E=2881019,C=74)
	CT(4086)=(S=2881020,E=2881023,C=7)
	CT(4087)=(S=2881024,E=2881123,C=1)
	CT(4088)=(S=2881124,E=2881135,C=4)
	CT(4089)=(S=2881136,E=2881139,C=7)
	CT(4090)=(S=2881140,E=2881143,C=8)
	CT(4091)=(S=2881144,E=2881151,C=4)
	CT(4092)=(S=2881152,E=2881155,C=2)
	CT(4093)=(S=2881156,E=2881159,C=8)
	CT(4094)=(S=2881160,E=2881175,C=4)
	CT(4095)=(S=2881176,E=2881179,C=6)
	CT(4096)=(S=2881180,E=2881187,C=4)
	CT(4097)=(S=2881188,E=2881191,C=5)
	CT(4098)=(S=2881192,E=2881199,C=8)
	CT(4099)=(S=2881200,E=2881203,C=5)
	CT(4100)=(S=2881204,E=2881207,C=6)
	CT(4101)=(S=2881208,E=2881211,C=3)
	CT(4102)=(S=2881212,E=2881215,C=9)
	CT(4103)=(S=2881216,E=2881223,C=2)
	CT(4104)=(S=2881224,E=2881227,C=84)
	CT(4105)=(S=2881228,E=2881235,C=8)
	CT(4106)=(S=2881236,E=2881239,C=1)
	CT(4107)=(S=2881240,E=2881243,C=9)
	CT(4108)=(S=2881244,E=2881251,C=77)
	CT(4109)=(S=2881252,E=2881255,C=1)
	CT(4110)=(S=2881256,E=2881259,C=0)
	CT(4111)=(S=2881260,E=2881279,C=4)
	CT(4112)=(S=2881280,E=2881283,C=8)
	CT(4113)=(S=2881284,E=2881291,C=1)
	CT(4114)=(S=2881292,E=2881295,C=74)
	CT(4115)=(S=2881296,E=2881299,C=10)
	CT(4116)=(S=2881300,E=2881303,C=74)
	CT(4117)=(S=2881304,E=2881307,C=87)
	CT(4118)=(S=2881308,E=2881311,C=4)
	CT(4119)=(S=2881312,E=2881319,C=74)
	CT(4120)=(S=2881320,E=2881323,C=4)
	CT(4121)=(S=2881324,E=2881331,C=1)
	CT(4122)=(S=2881332,E=2881335,C=4)
	CT(4123)=(S=2881336,E=2881339,C=3)
	CT(4124)=(S=2881340,E=2881343,C=159)
	CT(4125)=(S=2881344,E=2881347,C=0)
	CT(4126)=(S=2881348,E=2881351,C=1)
	CT(4127)=(S=2881352,E=2881355,C=2)
	CT(4128)=(S=2881356,E=2881359,C=1)
	CT(4129)=(S=2881360,E=2881363,C=0)
	CT(4130)=(S=2881364,E=2881383,C=1)
	CT(4131)=(S=2881384,E=2881387,C=6)
	CT(4132)=(S=2881388,E=2881395,C=1)
	CT(4133)=(S=2881396,E=2881399,C=0)
	CT(4134)=(S=2881400,E=2881403,C=77)
	CT(4135)=(S=2881404,E=2881407,C=73)
	CT(4136)=(S=2881408,E=2881411,C=0)
	CT(4137)=(S=2881412,E=2881415,C=8)
	CT(4138)=(S=2881416,E=2881427,C=4)
	CT(4139)=(S=2881428,E=2881435,C=1)
	CT(4140)=(S=2881436,E=2881443,C=4)
	CT(4141)=(S=2881444,E=2881447,C=8)
	CT(4142)=(S=2881448,E=2881451,C=2)
	CT(4143)=(S=2881452,E=2881455,C=73)
	CT(4144)=(S=2881456,E=2881471,C=1)
	CT(4145)=(S=2881472,E=2881475,C=6)
	CT(4146)=(S=2881476,E=2881543,C=1)
	CT(4147)=(S=2881544,E=2881547,C=8)
	CT(4148)=(S=2881548,E=2881551,C=77)
	CT(4149)=(S=2881552,E=2881555,C=9)
	CT(4150)=(S=2881556,E=2881559,C=1)
	CT(4151)=(S=2881560,E=2881563,C=74)
	CT(4152)=(S=2881564,E=2881567,C=1)
	CT(4153)=(S=2881568,E=2881575,C=4)
	CT(4154)=(S=2881576,E=2881583,C=2)
	CT(4155)=(S=2881584,E=2881587,C=1)
	CT(4156)=(S=2881588,E=2881591,C=0)
	CT(4157)=(S=2881592,E=2881595,C=1)
	CT(4158)=(S=2881596,E=2881603,C=8)
	CT(4159)=(S=2881604,E=2881611,C=4)
	CT(4160)=(S=2881612,E=2881687,C=1)
	CT(4161)=(S=2881688,E=2881691,C=4)
	CT(4162)=(S=2881692,E=2881695,C=163)
	CT(4163)=(S=2881696,E=2881699,C=2)
	CT(4164)=(S=2881700,E=2881711,C=8)
	CT(4165)=(S=2881712,E=2881747,C=1)
	CT(4166)=(S=2881748,E=2881751,C=0)
	CT(4167)=(S=2881752,E=2881755,C=2)
	CT(4168)=(S=2881756,E=2881759,C=4)
	CT(4169)=(S=2881760,E=2881763,C=8)
	CT(4170)=(S=2881764,E=2881771,C=1)
	CT(4171)=(S=2881772,E=2881779,C=4)
	CT(4172)=(S=2881780,E=2881783,C=1)
	CT(4173)=(S=2881784,E=2881787,C=8)
	CT(4174)=(S=2881788,E=2881791,C=4)
	CT(4175)=(S=2881792,E=2881803,C=1)
	CT(4176)=(S=2881804,E=2881807,C=7)
	CT(4177)=(S=2881808,E=2881815,C=8)
	CT(4178)=(S=2881816,E=2881819,C=1)
	CT(4179)=(S=2881820,E=2881823,C=8)
	CT(4180)=(S=2881824,E=2881827,C=3)
	CT(4181)=(S=2881828,E=2881831,C=7)
	CT(4182)=(S=2881832,E=2881835,C=89)
	CT(4183)=(S=2881836,E=2881839,C=2)
	CT(4184)=(S=2881840,E=2881847,C=4)
	CT(4185)=(S=2881848,E=2881863,C=3)
	CT(4186)=(S=2881864,E=2881867,C=73)
	CT(4187)=(S=2881868,E=2881875,C=8)
	CT(4188)=(S=2881876,E=2881879,C=4)
	CT(4189)=(S=2881880,E=2881883,C=8)
	CT(4190)=(S=2881884,E=2881887,C=0)
	CT(4191)=(S=2881888,E=2881895,C=8)
	CT(4192)=(S=2881896,E=2881907,C=3)
	CT(4193)=(S=2881908,E=2881911,C=0)
	CT(4194)=(S=2881912,E=2881915,C=1)
	CT(4195)=(S=2881916,E=2881919,C=8)
	CT(4196)=(S=2881920,E=2881923,C=0)
	CT(4197)=(S=2881924,E=2881931,C=1)
	CT(4198)=(S=2881932,E=2881935,C=74)
	CT(4199)=(S=2881936,E=2881963,C=1)
	CT(4200)=(S=2881964,E=2881967,C=8)
	CT(4201)=(S=2881968,E=2881971,C=164)
	CT(4202)=(S=2881972,E=2881983,C=4)
	CT(4203)=(S=2881984,E=2881987,C=1)
	CT(4204)=(S=2881988,E=2881995,C=73)
	CT(4205)=(S=2881996,E=2881999,C=8)
	CT(4206)=(S=2882000,E=2882003,C=74)
	CT(4207)=(S=2882004,E=2882007,C=7)
	CT(4208)=(S=2882008,E=2882011,C=4)
	CT(4209)=(S=2882012,E=2882015,C=8)
	CT(4210)=(S=2882016,E=2882027,C=4)
	CT(4211)=(S=2882028,E=2882031,C=1)
	CT(4212)=(S=2882032,E=2882039,C=2)
	CT(4213)=(S=2882040,E=2882043,C=7)
	CT(4214)=(S=2882044,E=2882047,C=8)
	CT(4215)=(S=2882048,E=2882051,C=7)
	CT(4216)=(S=2882052,E=2882055,C=1)
	CT(4217)=(S=2882056,E=2882059,C=8)
	CT(4218)=(S=2882060,E=2882071,C=1)
	CT(4219)=(S=2882072,E=2882075,C=0)
	CT(4220)=(S=2882076,E=2882087,C=1)
	CT(4221)=(S=2882088,E=2882091,C=4)
	CT(4222)=(S=2882092,E=2882095,C=9)
	CT(4223)=(S=2882096,E=2882099,C=8)
	CT(4224)=(S=2882100,E=2882103,C=0)
	CT(4225)=(S=2882104,E=2882115,C=8)
	CT(4226)=(S=2882116,E=2882119,C=0)
	CT(4227)=(S=2882120,E=2882123,C=1)
	CT(4228)=(S=2882128,E=2882131,C=84)
	CT(4229)=(S=2882132,E=2882135,C=77)
	CT(4230)=(S=2882136,E=2882139,C=2)
	CT(4231)=(S=2882140,E=2882143,C=0)
	CT(4232)=(S=2882144,E=2882167,C=1)
	CT(4233)=(S=2882168,E=2882171,C=4)
	CT(4234)=(S=2882172,E=2882175,C=92)
	CT(4235)=(S=2882176,E=2882179,C=1)
	CT(4236)=(S=2882180,E=2882183,C=4)
	CT(4237)=(S=2882184,E=2882187,C=89)
	CT(4238)=(S=2882188,E=2882191,C=0)
	CT(4239)=(S=2882192,E=2882199,C=1)
	CT(4240)=(S=2882200,E=2882203,C=6)
	CT(4241)=(S=2882204,E=2882207,C=4)
	CT(4242)=(S=2882208,E=2882211,C=1)
	CT(4243)=(S=2882212,E=2882215,C=4)
	CT(4244)=(S=2882216,E=2882227,C=1)
	CT(4245)=(S=2882228,E=2882231,C=4)
	CT(4246)=(S=2882232,E=2882239,C=8)
	CT(4247)=(S=2882240,E=2882243,C=7)
	CT(4248)=(S=2882244,E=2882247,C=2)
	CT(4249)=(S=2882248,E=2882251,C=1)
	CT(4250)=(S=2882252,E=2882255,C=0)
	CT(4251)=(S=2882256,E=2882259,C=4)
	CT(4252)=(S=2882260,E=2882271,C=1)
	CT(4253)=(S=2882272,E=2882275,C=10)
	CT(4254)=(S=2882276,E=2882279,C=2)
	CT(4255)=(S=2882280,E=2882282,C=0)
	CT(4256)=(S=2882283,E=2882283,C=8)
	CT(4257)=(S=2882284,E=2882287,C=1)
	CT(4258)=(S=2882288,E=2882291,C=156)
	CT(4259)=(S=2882292,E=2882295,C=1)
	CT(4260)=(S=2882296,E=2882299,C=0)
	CT(4261)=(S=2882300,E=2882307,C=4)
	CT(4262)=(S=2882308,E=2882321,C=1)
	CT(4263)=(S=2882322,E=2882323,C=4)
	CT(4264)=(S=2882324,E=2882331,C=8)
	CT(4265)=(S=2882332,E=2882335,C=6)
	CT(4266)=(S=2882336,E=2882339,C=8)
	CT(4267)=(S=2882340,E=2882343,C=1)
	CT(4268)=(S=2882344,E=2882347,C=0)
	CT(4269)=(S=2882348,E=2882351,C=8)
	CT(4270)=(S=2882352,E=2882355,C=2)
	CT(4271)=(S=2882356,E=2882359,C=0)
	CT(4272)=(S=2882360,E=2882367,C=9)
	CT(4273)=(S=2882368,E=2882371,C=7)
	CT(4274)=(S=2882372,E=2882375,C=6)
	CT(4275)=(S=2882376,E=2882379,C=4)
	CT(4276)=(S=2882380,E=2882383,C=8)
	CT(4277)=(S=2882384,E=2882399,C=4)
	CT(4278)=(S=2882400,E=2882403,C=74)
	CT(4279)=(S=2882404,E=2882407,C=4)
	CT(4280)=(S=2882408,E=2882411,C=8)
	CT(4281)=(S=2882412,E=2882415,C=0)
	CT(4282)=(S=2882416,E=2882419,C=5)
	CT(4283)=(S=2882420,E=2882423,C=1)
	CT(4284)=(S=2882424,E=2882427,C=6)
	CT(4285)=(S=2882428,E=2882431,C=4)
	CT(4286)=(S=2882432,E=2882435,C=157)
	CT(4287)=(S=2882436,E=2882439,C=8)
	CT(4288)=(S=2882440,E=2882443,C=5)
	CT(4289)=(S=2882444,E=2882447,C=8)
	CT(4290)=(S=2882448,E=2882455,C=4)
	CT(4291)=(S=2882456,E=2882459,C=0)
	CT(4292)=(S=2882460,E=2882463,C=5)
	CT(4293)=(S=2882464,E=2882467,C=8)
	CT(4294)=(S=2882468,E=2882471,C=0)
	CT(4295)=(S=2882472,E=2882483,C=4)
	CT(4296)=(S=2882484,E=2882487,C=9)
	CT(4297)=(S=2882488,E=2882491,C=2)
	CT(4298)=(S=2882492,E=2882495,C=4)
	CT(4299)=(S=2882496,E=2882499,C=1)
	CT(4300)=(S=2882500,E=2882503,C=8)
	CT(4301)=(S=2882504,E=2882507,C=0)
	CT(4302)=(S=2882508,E=2882511,C=6)
	CT(4303)=(S=2882512,E=2882515,C=5)
	CT(4304)=(S=2882516,E=2882527,C=4)
	CT(4305)=(S=2882528,E=2882535,C=8)
	CT(4306)=(S=2882536,E=2882543,C=1)
	CT(4307)=(S=2882544,E=2882547,C=0)
	CT(4308)=(S=2882548,E=2882551,C=1)
	CT(4309)=(S=2882552,E=2882555,C=8)
	CT(4310)=(S=2882556,E=2882559,C=3)
	CT(4311)=(S=2882560,E=2882563,C=0)
	CT(4312)=(S=2882564,E=2882567,C=4)
	CT(4313)=(S=2882568,E=2882571,C=74)
	CT(4314)=(S=2882572,E=2882575,C=156)
	CT(4315)=(S=2882576,E=2882579,C=86)
	CT(4316)=(S=2882580,E=2882583,C=0)
	CT(4317)=(S=2882584,E=2882595,C=4)
	CT(4318)=(S=2882596,E=2882599,C=5)
	CT(4319)=(S=2882600,E=2882603,C=1)
	CT(4320)=(S=2882604,E=2882607,C=5)
	CT(4321)=(S=2882608,E=2882611,C=1)
	CT(4322)=(S=2882612,E=2882615,C=8)
	CT(4323)=(S=2882616,E=2882619,C=1)
	CT(4324)=(S=2882620,E=2882623,C=4)
	CT(4325)=(S=2882624,E=2882631,C=73)
	CT(4326)=(S=2882632,E=2882635,C=74)
	CT(4327)=(S=2882636,E=2882639,C=0)
	CT(4328)=(S=2882640,E=2882643,C=4)
	CT(4329)=(S=2882644,E=2882647,C=8)
	CT(4330)=(S=2882648,E=2882655,C=4)
	CT(4331)=(S=2882656,E=2882659,C=0)
	CT(4332)=(S=2882660,E=2882663,C=4)
	CT(4333)=(S=2882664,E=2882667,C=74)
	CT(4334)=(S=2882668,E=2882675,C=0)
	CT(4335)=(S=2882676,E=2882679,C=4)
	CT(4336)=(S=2882680,E=2882683,C=3)
	CT(4337)=(S=2882684,E=2882691,C=0)
	CT(4338)=(S=2882692,E=2882695,C=8)
	CT(4339)=(S=2882696,E=2882699,C=74)
	CT(4340)=(S=2882700,E=2882703,C=4)
	CT(4341)=(S=2882704,E=2882707,C=74)
	CT(4342)=(S=2882708,E=2882711,C=8)
	CT(4343)=(S=2882712,E=2882715,C=4)
	CT(4344)=(S=2882716,E=2882719,C=74)
	CT(4345)=(S=2882720,E=2882739,C=8)
	CT(4346)=(S=2882740,E=2882743,C=4)
	CT(4347)=(S=2882744,E=2882747,C=74)
	CT(4348)=(S=2882748,E=2882759,C=4)
	CT(4349)=(S=2882760,E=2882763,C=8)
	CT(4350)=(S=2882764,E=2882767,C=4)
	CT(4351)=(S=2882768,E=2882771,C=8)
	CT(4352)=(S=2882772,E=2882779,C=5)
	CT(4353)=(S=2882780,E=2882781,C=4)
	CT(4354)=(S=2882782,E=2882783,C=74)
	CT(4355)=(S=2882784,E=2882787,C=1)
	CT(4356)=(S=2882788,E=2882791,C=8)
	CT(4357)=(S=2882792,E=2882795,C=5)
	CT(4358)=(S=2882796,E=2882799,C=74)
	CT(4359)=(S=2882800,E=2882803,C=4)
	CT(4360)=(S=2882804,E=2882807,C=154)
	CT(4361)=(S=2882808,E=2882811,C=4)
	CT(4362)=(S=2882812,E=2882815,C=0)
	CT(4363)=(S=2882816,E=2883071,C=2)
	CT(4364)=(S=2883072,E=2883083,C=1)
	CT(4365)=(S=2883084,E=2883087,C=77)
	CT(4366)=(S=2883088,E=2883091,C=9)
	CT(4367)=(S=2883092,E=2883095,C=73)
	CT(4368)=(S=2883096,E=2883099,C=1)
	CT(4369)=(S=2883100,E=2883107,C=4)
	CT(4370)=(S=2883108,E=2883111,C=1)
	CT(4371)=(S=2883112,E=2883115,C=4)
	CT(4372)=(S=2883116,E=2883119,C=1)
	CT(4373)=(S=2883120,E=2883123,C=4)
	CT(4374)=(S=2883124,E=2883127,C=1)
	CT(4375)=(S=2883128,E=2883131,C=165)
	CT(4376)=(S=2883132,E=2883135,C=162)
	CT(4377)=(S=2883136,E=2883139,C=1)
	CT(4378)=(S=2883140,E=2883143,C=74)
	CT(4379)=(S=2883144,E=2883147,C=1)
	CT(4380)=(S=2883148,E=2883151,C=5)
	CT(4381)=(S=2883152,E=2883155,C=74)
	CT(4382)=(S=2883156,E=2883167,C=1)
	CT(4383)=(S=2883168,E=2883171,C=84)
	CT(4384)=(S=2883172,E=2883179,C=1)
	CT(4385)=(S=2883180,E=2883183,C=4)
	CT(4386)=(S=2883184,E=2883191,C=1)
	CT(4387)=(S=2883192,E=2883195,C=5)
	CT(4388)=(S=2883196,E=2883199,C=74)
	CT(4389)=(S=2883200,E=2883203,C=1)
	CT(4390)=(S=2883204,E=2883207,C=3)
	CT(4391)=(S=2883208,E=2883231,C=1)
	CT(4392)=(S=2883232,E=2883235,C=4)
	CT(4393)=(S=2883236,E=2883239,C=8)
	CT(4394)=(S=2883240,E=2883247,C=1)
	CT(4395)=(S=2883248,E=2883251,C=4)
	CT(4396)=(S=2883252,E=2883275,C=1)
	CT(4397)=(S=2883276,E=2883279,C=4)
	CT(4398)=(S=2883280,E=2883283,C=1)
	CT(4399)=(S=2883284,E=2883287,C=4)
	CT(4400)=(S=2883288,E=2883291,C=8)
	CT(4401)=(S=2883292,E=2883315,C=1)
	CT(4402)=(S=2883316,E=2883319,C=6)
	CT(4403)=(S=2883320,E=2883339,C=1)
	CT(4404)=(S=2883340,E=2883343,C=9)
	CT(4405)=(S=2883344,E=2883347,C=1)
	CT(4406)=(S=2883348,E=2883351,C=84)
	CT(4407)=(S=2883352,E=2883355,C=73)
	CT(4408)=(S=2883356,E=2883359,C=7)
	CT(4409)=(S=2883360,E=2883363,C=0)
	CT(4410)=(S=2883364,E=2883367,C=8)
	CT(4411)=(S=2883368,E=2883371,C=2)
	CT(4412)=(S=2883372,E=2883375,C=0)
	CT(4413)=(S=2883376,E=2883379,C=1)
	CT(4414)=(S=2883380,E=2883383,C=8)
	CT(4415)=(S=2883384,E=2883387,C=4)
	CT(4416)=(S=2883388,E=2883407,C=1)
	CT(4417)=(S=2883408,E=2883411,C=8)
	CT(4418)=(S=2883412,E=2883415,C=1)
	CT(4419)=(S=2883416,E=2883423,C=9)
	CT(4420)=(S=2883424,E=2883427,C=1)
	CT(4421)=(S=2883428,E=2883431,C=7)
	CT(4422)=(S=2883432,E=2883435,C=8)
	CT(4423)=(S=2883436,E=2883439,C=1)
	CT(4424)=(S=2883440,E=2883443,C=86)
	CT(4425)=(S=2883444,E=2883451,C=8)
	CT(4426)=(S=2883452,E=2883455,C=7)
	CT(4427)=(S=2883456,E=2883459,C=0)
	CT(4428)=(S=2883460,E=2883463,C=4)
	CT(4429)=(S=2883464,E=2883467,C=0)
	CT(4430)=(S=2883468,E=2883471,C=4)
	CT(4431)=(S=2883472,E=2883475,C=1)
	CT(4432)=(S=2883476,E=2883479,C=166)
	CT(4433)=(S=2883480,E=2883483,C=7)
	CT(4434)=(S=2883484,E=2883487,C=8)
	CT(4435)=(S=2883488,E=2883491,C=73)
	CT(4436)=(S=2883492,E=2883495,C=4)
	CT(4437)=(S=2883496,E=2883499,C=1)
	CT(4438)=(S=2883500,E=2883503,C=5)
	CT(4439)=(S=2883504,E=2883507,C=1)
	CT(4440)=(S=2883508,E=2883511,C=9)
	CT(4441)=(S=2883512,E=2883515,C=1)
	CT(4442)=(S=2883516,E=2883519,C=8)
	CT(4443)=(S=2883520,E=2883523,C=1)
	CT(4444)=(S=2883524,E=2883527,C=74)
	CT(4445)=(S=2883528,E=2883543,C=1)
	CT(4446)=(S=2883544,E=2883547,C=10)
	CT(4447)=(S=2883548,E=2883551,C=4)
	CT(4448)=(S=2883552,E=2883563,C=1)
	CT(4449)=(S=2883564,E=2883567,C=90)
	CT(4450)=(S=2883568,E=2883571,C=3)
	CT(4451)=(S=2883572,E=2883575,C=1)
	CT(4452)=(S=2883576,E=2883583,C=6)
	CT(4453)=(S=2883584,E=2949631,C=29)
	CT(4454)=(S=2949632,E=2949919,C=78)
	CT(4455)=(S=2949920,E=2957759,C=29)
	CT(4456)=(S=2957760,E=2957775,C=78)
	CT(4457)=(S=2957776,E=2959375,C=29)
	CT(4458)=(S=2959376,E=2959391,C=78)
	CT(4459)=(S=2959392,E=2959423,C=29)
	CT(4460)=(S=2959424,E=2959439,C=78)
	CT(4461)=(S=2959440,E=2959711,C=29)
	CT(4462)=(S=2959712,E=2959743,C=78)
	CT(4463)=(S=2959744,E=2959879,C=29)
	CT(4464)=(S=2959880,E=2959887,C=78)
	CT(4465)=(S=2959888,E=2959895,C=29)
	CT(4466)=(S=2959896,E=2959899,C=78)
	CT(4467)=(S=2959900,E=2959907,C=29)
	CT(4468)=(S=2959908,E=2959911,C=78)
	CT(4469)=(S=2959912,E=2959943,C=29)
	CT(4470)=(S=2959944,E=2959951,C=78)
	CT(4471)=(S=2959952,E=2959967,C=29)
	CT(4472)=(S=2959968,E=2959999,C=78)
	CT(4473)=(S=2960000,E=2960383,C=29)
	CT(4474)=(S=2960384,E=2960767,C=78)
	CT(4475)=(S=2960768,E=2963983,C=29)
	CT(4476)=(S=2963984,E=2963999,C=78)
	CT(4477)=(S=2964000,E=2964047,C=29)
	CT(4478)=(S=2964048,E=2964079,C=78)
	CT(4479)=(S=2964080,E=2964159,C=29)
	CT(4480)=(S=2964160,E=2964223,C=78)
	CT(4481)=(S=2964224,E=2964735,C=29)
	CT(4482)=(S=2964736,E=2964751,C=78)
	CT(4483)=(S=2964752,E=2964783,C=29)
	CT(4484)=(S=2964784,E=2964799,C=78)
	CT(4485)=(S=2964800,E=2965055,C=29)
	CT(4486)=(S=2965056,E=2965119,C=78)
	CT(4487)=(S=2965120,E=2965183,C=29)
	CT(4488)=(S=2965184,E=2965247,C=78)
	CT(4489)=(S=2965248,E=2965503,C=29)
	CT(4490)=(S=2965504,E=2965511,C=74)
	CT(4491)=(S=2965512,E=2965523,C=4)
	CT(4492)=(S=2965524,E=2965527,C=87)
	CT(4493)=(S=2965528,E=2965531,C=77)
	CT(4494)=(S=2965532,E=2965539,C=9)
	CT(4495)=(S=2965540,E=2965543,C=0)
	CT(4496)=(S=2965544,E=2965547,C=7)
	CT(4497)=(S=2965548,E=2965551,C=74)
	CT(4498)=(S=2965552,E=2965555,C=73)
	CT(4499)=(S=2965556,E=2965559,C=8)
	CT(4500)=(S=2965560,E=2965563,C=0)
	CT(4501)=(S=2965564,E=2965567,C=73)
	CT(4502)=(S=2965568,E=2965571,C=8)
	CT(4503)=(S=2965572,E=2965575,C=0)
	CT(4504)=(S=2965576,E=2965577,C=7)
	CT(4505)=(S=2965578,E=2965579,C=9)
	CT(4506)=(S=2965580,E=2965583,C=3)
	CT(4507)=(S=2965584,E=2965587,C=10)
	CT(4508)=(S=2965588,E=2965599,C=4)
	CT(4509)=(S=2965600,E=2965607,C=74)
	CT(4510)=(S=2965608,E=2965611,C=4)
	CT(4511)=(S=2965612,E=2965615,C=8)
	CT(4512)=(S=2965616,E=2965617,C=1)
	CT(4513)=(S=2965618,E=2965619,C=4)
	CT(4514)=(S=2965620,E=2965623,C=73)
	CT(4515)=(S=2965624,E=2965627,C=10)
	CT(4516)=(S=2965628,E=2965631,C=86)
	CT(4517)=(S=2965632,E=2965635,C=7)
	CT(4518)=(S=2965636,E=2965643,C=84)
	CT(4519)=(S=2965644,E=2965651,C=6)
	CT(4520)=(S=2965652,E=2965655,C=0)
	CT(4521)=(S=2965656,E=2965659,C=6)
	CT(4522)=(S=2965660,E=2965663,C=4)
	CT(4523)=(S=2965664,E=2965667,C=75)
	CT(4524)=(S=2965668,E=2965671,C=84)
	CT(4525)=(S=2965672,E=2965675,C=5)
	CT(4526)=(S=2965676,E=2965679,C=6)
	CT(4527)=(S=2965680,E=2965683,C=4)
	CT(4528)=(S=2965684,E=2965687,C=77)
	CT(4529)=(S=2965688,E=2965691,C=3)
	CT(4530)=(S=2965692,E=2965703,C=4)
	CT(4531)=(S=2965704,E=2965707,C=73)
	CT(4532)=(S=2965708,E=2965731,C=4)
	CT(4533)=(S=2965732,E=2965739,C=9)
	CT(4534)=(S=2965740,E=2965743,C=4)
	CT(4535)=(S=2965744,E=2965751,C=8)
	CT(4536)=(S=2965752,E=2965755,C=161)
	CT(4537)=(S=2965756,E=2965759,C=74)
	CT(4538)=(S=2965760,E=2965763,C=7)
	CT(4539)=(S=2965764,E=2965767,C=73)
	CT(4540)=(S=2965768,E=2965771,C=4)
	CT(4541)=(S=2965772,E=2965775,C=73)
	CT(4542)=(S=2967552,E=2967679,C=29)
	CT(4543)=(S=2967680,E=2967935,C=78)
	CT(4544)=(S=2967936,E=2968127,C=29)
	CT(4545)=(S=2968128,E=2968143,C=78)
	CT(4546)=(S=2968144,E=2968159,C=29)
	CT(4547)=(S=2968160,E=2968191,C=78)
	CT(4548)=(S=2968192,E=2968319,C=29)
	CT(4549)=(S=2968320,E=2968575,C=78)
	CT(4550)=(S=2968576,E=2969087,C=29)
	CT(4551)=(S=2969088,E=2969151,C=78)
	CT(4552)=(S=2969152,E=2969183,C=29)
	CT(4553)=(S=2969184,E=2969215,C=78)
	CT(4554)=(S=2969216,E=2969247,C=29)
	CT(4555)=(S=2969248,E=2969343,C=78)
	CT(4556)=(S=2969344,E=2969599,C=29)
	CT(4557)=(S=2973696,E=2975743,C=106)
	CT(4558)=(S=2977792,E=2977815,C=4)
	CT(4559)=(S=2977816,E=2977819,C=8)
	CT(4560)=(S=2977820,E=2977827,C=4)
	CT(4561)=(S=2977828,E=2977831,C=7)
	CT(4562)=(S=2977832,E=2977851,C=4)
	CT(4563)=(S=2977852,E=2977855,C=2)
	CT(4564)=(S=2977856,E=2977859,C=3)
	CT(4565)=(S=2977860,E=2977863,C=4)
	CT(4566)=(S=2977864,E=2977867,C=84)
	CT(4567)=(S=2977868,E=2977873,C=7)
	CT(4568)=(S=2977874,E=2977875,C=10)
	CT(4569)=(S=2977876,E=2977879,C=74)
	CT(4570)=(S=2977880,E=2977911,C=6)
	CT(4571)=(S=2977912,E=2977915,C=2)
	CT(4572)=(S=2977916,E=2977919,C=74)
	CT(4573)=(S=2977920,E=2977923,C=2)
	CT(4574)=(S=2977924,E=2977927,C=1)
	CT(4575)=(S=2977928,E=2977931,C=4)
	CT(4576)=(S=2977932,E=2977935,C=3)
	CT(4577)=(S=2977936,E=2977943,C=4)
	CT(4578)=(S=2977944,E=2977963,C=6)
	CT(4579)=(S=2977964,E=2977967,C=4)
	CT(4580)=(S=2977968,E=2977971,C=155)
	CT(4581)=(S=2977972,E=2977979,C=4)
	CT(4582)=(S=2977980,E=2977983,C=1)
	CT(4583)=(S=2977984,E=2977987,C=4)
	CT(4584)=(S=2977988,E=2977991,C=5)
	CT(4585)=(S=2977992,E=2977995,C=4)
	CT(4586)=(S=2977996,E=2977999,C=8)
	CT(4587)=(S=2978000,E=2978015,C=1)
	CT(4588)=(S=2978016,E=2978019,C=91)
	CT(4589)=(S=2978020,E=2978031,C=1)
	CT(4590)=(S=2978032,E=2978035,C=8)
	CT(4591)=(S=2978036,E=2978039,C=0)
	CT(4592)=(S=2978040,E=2978047,C=4)
	CT(4593)=(S=2978048,E=2978051,C=8)
	CT(4594)=(S=2978052,E=2978055,C=2)
	CT(4595)=(S=2978056,E=2978059,C=73)
	CT(4596)=(S=2978060,E=2978079,C=1)
	CT(4597)=(S=2978080,E=2978083,C=8)
	CT(4598)=(S=2978084,E=2978087,C=0)
	CT(4599)=(S=2978088,E=2978091,C=1)
	CT(4600)=(S=2978092,E=2978099,C=6)
	CT(4601)=(S=2978100,E=2978107,C=1)
	CT(4602)=(S=2978108,E=2978109,C=3)
	CT(4603)=(S=2978110,E=2978115,C=4)
	CT(4604)=(S=2978116,E=2978119,C=8)
	CT(4605)=(S=2978120,E=2978123,C=1)
	CT(4606)=(S=2978124,E=2978127,C=4)
	CT(4607)=(S=2978128,E=2978131,C=8)
	CT(4608)=(S=2978132,E=2978135,C=3)
	CT(4609)=(S=2978136,E=2978139,C=4)
	CT(4610)=(S=2978140,E=2978143,C=0)
	CT(4611)=(S=2978144,E=2978155,C=4)
	CT(4612)=(S=2978156,E=2978159,C=1)
	CT(4613)=(S=2978160,E=2978163,C=8)
	CT(4614)=(S=2978164,E=2978167,C=2)
	CT(4615)=(S=2978168,E=2978171,C=4)
	CT(4616)=(S=2978172,E=2978175,C=77)
	CT(4617)=(S=2978176,E=2978179,C=8)
	CT(4618)=(S=2978180,E=2978183,C=2)
	CT(4619)=(S=2978184,E=2978187,C=4)
	CT(4620)=(S=2978188,E=2978191,C=8)
	CT(4621)=(S=2978192,E=2978199,C=1)
	CT(4622)=(S=2978200,E=2978203,C=4)
	CT(4623)=(S=2978204,E=2978207,C=9)
	CT(4624)=(S=2978208,E=2978211,C=8)
	CT(4625)=(S=2978212,E=2978215,C=2)
	CT(4626)=(S=2978216,E=2978219,C=1)
	CT(4627)=(S=2978220,E=2978223,C=2)
	CT(4628)=(S=2978224,E=2978227,C=1)
	CT(4629)=(S=2978228,E=2978231,C=4)
	CT(4630)=(S=2978232,E=2978235,C=1)
	CT(4631)=(S=2978236,E=2978239,C=4)
	CT(4632)=(S=2978240,E=2978243,C=8)
	CT(4633)=(S=2978244,E=2978247,C=0)
	CT(4634)=(S=2978248,E=2978271,C=1)
	CT(4635)=(S=2978272,E=2978275,C=4)
	CT(4636)=(S=2978276,E=2978279,C=1)
	CT(4637)=(S=2978280,E=2978283,C=0)
	CT(4638)=(S=2978284,E=2978287,C=84)
	CT(4639)=(S=2978288,E=2978291,C=1)
	CT(4640)=(S=2978292,E=2978295,C=0)
	CT(4641)=(S=2978296,E=2978299,C=4)
	CT(4642)=(S=2978300,E=2978307,C=1)
	CT(4643)=(S=2978312,E=2978315,C=8)
	CT(4644)=(S=2978316,E=2978319,C=1)
	CT(4645)=(S=2978320,E=2978323,C=2)
	CT(4646)=(S=2978324,E=2978327,C=10)
	CT(4647)=(S=2978328,E=2978331,C=8)
	CT(4648)=(S=2978332,E=2978335,C=5)
	CT(4649)=(S=2978336,E=2978339,C=1)
	CT(4650)=(S=2978340,E=2978343,C=4)
	CT(4651)=(S=2978344,E=2978347,C=1)
	CT(4652)=(S=2978348,E=2978351,C=8)
	CT(4653)=(S=2978352,E=2978355,C=4)
	CT(4654)=(S=2978356,E=2978359,C=1)
	CT(4655)=(S=2978360,E=2978387,C=4)
	CT(4656)=(S=2978388,E=2978395,C=84)
	CT(4657)=(S=2978396,E=2978399,C=8)
	CT(4658)=(S=2978400,E=2978403,C=1)
	CT(4659)=(S=2978404,E=2978407,C=5)
	CT(4660)=(S=2978408,E=2978411,C=1)
	CT(4661)=(S=2978412,E=2978419,C=2)
	CT(4662)=(S=2978420,E=2978423,C=74)
	CT(4663)=(S=2978424,E=2978431,C=77)
	CT(4664)=(S=2978432,E=2978435,C=6)
	CT(4665)=(S=2978436,E=2978439,C=77)
	CT(4666)=(S=2978440,E=2978443,C=1)
	CT(4667)=(S=2978444,E=2978463,C=4)
	CT(4668)=(S=2978464,E=2978467,C=86)
	CT(4669)=(S=2978468,E=2978479,C=8)
	CT(4670)=(S=2978480,E=2978483,C=4)
	CT(4671)=(S=2978484,E=2978487,C=84)
	CT(4672)=(S=2978488,E=2978491,C=2)
	CT(4673)=(S=2978492,E=2978495,C=0)
	CT(4674)=(S=2978496,E=2978499,C=4)
	CT(4675)=(S=2978500,E=2978507,C=1)
	CT(4676)=(S=2978508,E=2978511,C=8)
	CT(4677)=(S=2978512,E=2978515,C=2)
	CT(4678)=(S=2978516,E=2978523,C=4)
	CT(4679)=(S=2978524,E=2978527,C=0)
	CT(4680)=(S=2978528,E=2978531,C=4)
	CT(4681)=(S=2978532,E=2978535,C=1)
	CT(4682)=(S=2978536,E=2978539,C=84)
	CT(4683)=(S=2978540,E=2978543,C=1)
	CT(4684)=(S=2978544,E=2978555,C=4)
	CT(4685)=(S=2978556,E=2978559,C=1)
	CT(4686)=(S=2978560,E=2978571,C=4)
	CT(4687)=(S=2978572,E=2978575,C=84)
	CT(4688)=(S=2978576,E=2978579,C=4)
	CT(4689)=(S=2978580,E=2978583,C=0)
	CT(4690)=(S=2978584,E=2978591,C=8)
	CT(4691)=(S=2978592,E=2978595,C=2)
	CT(4692)=(S=2978596,E=2978603,C=8)
	CT(4693)=(S=2978604,E=2978607,C=1)
	CT(4694)=(S=2978608,E=2978608,C=0)
	CT(4695)=(S=2978609,E=2978609,C=77)
	CT(4696)=(S=2978610,E=2978611,C=3)
	CT(4697)=(S=2978612,E=2978623,C=4)
	CT(4698)=(S=2978624,E=2978627,C=74)
	CT(4699)=(S=2978628,E=2978631,C=0)
	CT(4700)=(S=2978632,E=2978635,C=165)
	CT(4701)=(S=2978636,E=2978643,C=4)
	CT(4702)=(S=2978644,E=2978647,C=77)
	CT(4703)=(S=2978648,E=2978659,C=4)
	CT(4704)=(S=2978660,E=2978663,C=1)
	CT(4705)=(S=2978664,E=2978671,C=4)
	CT(4706)=(S=2978672,E=2978675,C=84)
	CT(4707)=(S=2978676,E=2978677,C=0)
	CT(4708)=(S=2978678,E=2978678,C=3)
	CT(4709)=(S=2978679,E=2978679,C=4)
	CT(4710)=(S=2978680,E=2978683,C=1)
	CT(4711)=(S=2978684,E=2978687,C=8)
	CT(4712)=(S=2978688,E=2978691,C=0)
	CT(4713)=(S=2978692,E=2978695,C=1)
	CT(4714)=(S=2978696,E=2978699,C=74)
	CT(4715)=(S=2978700,E=2978703,C=4)
	CT(4716)=(S=2978704,E=2978707,C=1)
	CT(4717)=(S=2978708,E=2978711,C=4)
	CT(4718)=(S=2978712,E=2978715,C=6)
	CT(4719)=(S=2978716,E=2978719,C=1)
	CT(4720)=(S=2978720,E=2978723,C=8)
	CT(4721)=(S=2978724,E=2978727,C=1)
	CT(4722)=(S=2978728,E=2978751,C=4)
	CT(4723)=(S=2978752,E=2978755,C=2)
	CT(4724)=(S=2978756,E=2978759,C=0)
	CT(4725)=(S=2978760,E=2978763,C=1)
	CT(4726)=(S=2978764,E=2978767,C=0)
	CT(4727)=(S=2978768,E=2978771,C=4)
	CT(4728)=(S=2978772,E=2978779,C=1)
	CT(4729)=(S=2978780,E=2978783,C=4)
	CT(4730)=(S=2978784,E=2978787,C=84)
	CT(4731)=(S=2978788,E=2978791,C=1)
	CT(4732)=(S=2978792,E=2978795,C=0)
	CT(4733)=(S=2978796,E=2978799,C=1)
	CT(4734)=(S=2978800,E=2978803,C=8)
	CT(4735)=(S=2978804,E=2978811,C=1)
	CT(4736)=(S=2978812,E=2978823,C=4)
	CT(4737)=(S=2978824,E=2978827,C=8)
	CT(4738)=(S=2978828,E=2978843,C=1)
	CT(4739)=(S=2978844,E=2978847,C=4)
	CT(4740)=(S=2978848,E=2978855,C=1)
	CT(4741)=(S=2978856,E=2978859,C=8)
	CT(4742)=(S=2978860,E=2978863,C=4)
	CT(4743)=(S=2978864,E=2978867,C=2)
	CT(4744)=(S=2978868,E=2978871,C=1)
	CT(4745)=(S=2978872,E=2978875,C=4)
	CT(4746)=(S=2978876,E=2978883,C=1)
	CT(4747)=(S=2978884,E=2978887,C=4)
	CT(4748)=(S=2978888,E=2978891,C=2)
	CT(4749)=(S=2978892,E=2978899,C=8)
	CT(4750)=(S=2978900,E=2978903,C=4)
	CT(4751)=(S=2978904,E=2978907,C=7)
	CT(4752)=(S=2978908,E=2978911,C=2)
	CT(4753)=(S=2978912,E=2978919,C=1)
	CT(4754)=(S=2978920,E=2978923,C=4)
	CT(4755)=(S=2978924,E=2978927,C=0)
	CT(4756)=(S=2978928,E=2978943,C=4)
	CT(4757)=(S=2978944,E=2978945,C=89)
	CT(4758)=(S=2978946,E=2978946,C=8)
	CT(4759)=(S=2978947,E=2978947,C=0)
	CT(4760)=(S=2978948,E=2978951,C=2)
	CT(4761)=(S=2978952,E=2978955,C=7)
	CT(4762)=(S=2978956,E=2978959,C=1)
	CT(4763)=(S=2978960,E=2978963,C=8)
	CT(4764)=(S=2978964,E=2978967,C=4)
	CT(4765)=(S=2978968,E=2978971,C=1)
	CT(4766)=(S=2978972,E=2978975,C=74)
	CT(4767)=(S=2978976,E=2978983,C=4)
	CT(4768)=(S=2978984,E=2978987,C=8)
	CT(4769)=(S=2978988,E=2978995,C=5)
	CT(4770)=(S=2978996,E=2979023,C=4)
	CT(4771)=(S=2979024,E=2979027,C=1)
	CT(4772)=(S=2979028,E=2979031,C=8)
	CT(4773)=(S=2979032,E=2979035,C=3)
	CT(4774)=(S=2979036,E=2979039,C=0)
	CT(4775)=(S=2979040,E=2979043,C=2)
	CT(4776)=(S=2979044,E=2979047,C=4)
	CT(4777)=(S=2979048,E=2979051,C=77)
	CT(4778)=(S=2979052,E=2979063,C=5)
	CT(4779)=(S=2979064,E=2979067,C=84)
	CT(4780)=(S=2979068,E=2979079,C=4)
	CT(4781)=(S=2979080,E=2979083,C=1)
	CT(4782)=(S=2979084,E=2979087,C=6)
	CT(4783)=(S=2979088,E=2979091,C=2)
	CT(4784)=(S=2979092,E=2979095,C=1)
	CT(4785)=(S=2979096,E=2979099,C=0)
	CT(4786)=(S=2979100,E=2979103,C=4)
	CT(4787)=(S=2979104,E=2979107,C=92)
	CT(4788)=(S=2979108,E=2979111,C=73)
	CT(4789)=(S=2979112,E=2979115,C=1)
	CT(4790)=(S=2979116,E=2979127,C=4)
	CT(4791)=(S=2979128,E=2979135,C=84)
	CT(4792)=(S=2979136,E=2979139,C=4)
	CT(4793)=(S=2979140,E=2979143,C=1)
	CT(4794)=(S=2979144,E=2979147,C=4)
	CT(4795)=(S=2979148,E=2979155,C=11)
	CT(4796)=(S=2979156,E=2979159,C=0)
	CT(4797)=(S=2979160,E=2979163,C=77)
	CT(4798)=(S=2979164,E=2979167,C=4)
	CT(4799)=(S=2979168,E=2979171,C=8)
	CT(4800)=(S=2979172,E=2979175,C=0)
	CT(4801)=(S=2979176,E=2979179,C=77)
	CT(4802)=(S=2979180,E=2979191,C=4)
	CT(4803)=(S=2979192,E=2979195,C=5)
	CT(4804)=(S=2979196,E=2979199,C=1)
	CT(4805)=(S=2979200,E=2979211,C=4)
	CT(4806)=(S=2979212,E=2979215,C=2)
	CT(4807)=(S=2979216,E=2979219,C=8)
	CT(4808)=(S=2979220,E=2979223,C=4)
	CT(4809)=(S=2979224,E=2979227,C=75)
	CT(4810)=(S=2979228,E=2979251,C=11)
	CT(4811)=(S=2979252,E=2979259,C=4)
	CT(4812)=(S=2979260,E=2979263,C=7)
	CT(4813)=(S=2979264,E=2979267,C=8)
	CT(4814)=(S=2979268,E=2979271,C=167)
	CT(4815)=(S=2979272,E=2979299,C=4)
	CT(4816)=(S=2979300,E=2979303,C=5)
	CT(4817)=(S=2979304,E=2979307,C=4)
	CT(4818)=(S=2979308,E=2979311,C=5)
	CT(4819)=(S=2979312,E=2979319,C=91)
	CT(4820)=(S=2979320,E=2979323,C=4)
	CT(4821)=(S=2979324,E=2979327,C=1)
	CT(4822)=(S=2979328,E=2979363,C=4)
	CT(4823)=(S=2979364,E=2979367,C=10)
	CT(4824)=(S=2979368,E=2979371,C=2)
	CT(4825)=(S=2979372,E=2979387,C=4)
	CT(4826)=(S=2979388,E=2979391,C=84)
	CT(4827)=(S=2979392,E=2979395,C=0)
	CT(4828)=(S=2979396,E=2979399,C=84)
	CT(4829)=(S=2979400,E=2979439,C=4)
	CT(4830)=(S=2979440,E=2979443,C=74)
	CT(4831)=(S=2979444,E=2979451,C=4)
	CT(4832)=(S=2979452,E=2979455,C=8)
	CT(4833)=(S=2979456,E=2979459,C=4)
	CT(4834)=(S=2979460,E=2979463,C=2)
	CT(4835)=(S=2979464,E=2979479,C=11)
	CT(4836)=(S=2979480,E=2979507,C=4)
	CT(4837)=(S=2979508,E=2979511,C=7)
	CT(4838)=(S=2979512,E=2979515,C=0)
	CT(4839)=(S=2979516,E=2979519,C=73)
	CT(4840)=(S=2979520,E=2979571,C=4)
	CT(4841)=(S=2979572,E=2979575,C=84)
	CT(4842)=(S=2979576,E=2979591,C=8)
	CT(4843)=(S=2979592,E=2979623,C=4)
	CT(4844)=(S=2979624,E=2979627,C=6)
	CT(4845)=(S=2979628,E=2979635,C=4)
	CT(4846)=(S=2979636,E=2979639,C=1)
	CT(4847)=(S=2979640,E=2979643,C=4)
	CT(4848)=(S=2979644,E=2979659,C=1)
	CT(4849)=(S=2979660,E=2979671,C=11)
	CT(4850)=(S=2979672,E=2979675,C=4)
	CT(4851)=(S=2979676,E=2979683,C=8)
	CT(4852)=(S=2979684,E=2979687,C=73)
	CT(4853)=(S=2979688,E=2979691,C=1)
	CT(4854)=(S=2979692,E=2979695,C=11)
	CT(4855)=(S=2979696,E=2979699,C=4)
	CT(4856)=(S=2979700,E=2979703,C=1)
	CT(4857)=(S=2979704,E=2979707,C=84)
	CT(4858)=(S=2979708,E=2979711,C=8)
	CT(4859)=(S=2979712,E=2979727,C=4)
	CT(4860)=(S=2979728,E=2979731,C=6)
	CT(4861)=(S=2979732,E=2979735,C=4)
	CT(4862)=(S=2979736,E=2979743,C=7)
	CT(4863)=(S=2979744,E=2979747,C=1)
	CT(4864)=(S=2979748,E=2979751,C=2)
	CT(4865)=(S=2979752,E=2979783,C=4)
	CT(4866)=(S=2979784,E=2979787,C=7)
	CT(4867)=(S=2979788,E=2979795,C=2)
	CT(4868)=(S=2979796,E=2979803,C=11)
	CT(4869)=(S=2979804,E=2979807,C=0)
	CT(4870)=(S=2979808,E=2979815,C=4)
	CT(4871)=(S=2979816,E=2979819,C=1)
	CT(4872)=(S=2979820,E=2979823,C=4)
	CT(4873)=(S=2979824,E=2979827,C=11)
	CT(4874)=(S=2979828,E=2979835,C=4)
	CT(4875)=(S=2979836,E=2979847,C=8)
	CT(4876)=(S=2979848,E=2979867,C=4)
	CT(4877)=(S=2979868,E=2979871,C=8)
	CT(4878)=(S=2979872,E=2979875,C=4)
	CT(4879)=(S=2979876,E=2979876,C=0)
	CT(4880)=(S=2979877,E=2979877,C=4)
	CT(4881)=(S=2979878,E=2979879,C=84)
	CT(4882)=(S=2979880,E=2979883,C=4)
	CT(4883)=(S=2979884,E=2979887,C=2)
	CT(4884)=(S=2979888,E=2979891,C=4)
	CT(4885)=(S=2979892,E=2979895,C=8)
	CT(4886)=(S=2979896,E=2979903,C=4)
	CT(4887)=(S=2979904,E=2979911,C=6)
	CT(4888)=(S=2979912,E=2979915,C=4)
	CT(4889)=(S=2979916,E=2979923,C=8)
	CT(4890)=(S=2979924,E=2979927,C=155)
	CT(4891)=(S=2979928,E=2979935,C=4)
	CT(4892)=(S=2979936,E=2979939,C=84)
	CT(4893)=(S=2979940,E=2979943,C=1)
	CT(4894)=(S=2979944,E=2979947,C=0)
	CT(4895)=(S=2979948,E=2979951,C=10)
	CT(4896)=(S=2979952,E=2979955,C=84)
	CT(4897)=(S=2979956,E=2979959,C=73)
	CT(4898)=(S=2979960,E=2979967,C=4)
	CT(4899)=(S=2979968,E=2979971,C=5)
	CT(4900)=(S=2979972,E=2979975,C=2)
	CT(4901)=(S=2979976,E=2979979,C=73)
	CT(4902)=(S=2979980,E=2979983,C=1)
	CT(4903)=(S=2979984,E=2979987,C=4)
	CT(4904)=(S=2979988,E=2979991,C=7)
	CT(4905)=(S=2979992,E=2979995,C=8)
	CT(4906)=(S=2979996,E=2979999,C=1)
	CT(4907)=(S=2980000,E=2980003,C=4)
	CT(4908)=(S=2980004,E=2980007,C=1)
	CT(4909)=(S=2980008,E=2980011,C=8)
	CT(4910)=(S=2980012,E=2980015,C=4)
	CT(4911)=(S=2980016,E=2980019,C=2)
	CT(4912)=(S=2980020,E=2980027,C=8)
	CT(4913)=(S=2980028,E=2980031,C=4)
	CT(4914)=(S=2980032,E=2980039,C=2)
	CT(4915)=(S=2980040,E=2980043,C=0)
	CT(4916)=(S=2980044,E=2980051,C=8)
	CT(4917)=(S=2980052,E=2980055,C=4)
	CT(4918)=(S=2980056,E=2980059,C=8)
	CT(4919)=(S=2980060,E=2980063,C=1)
	CT(4920)=(S=2980064,E=2980071,C=11)
	CT(4921)=(S=2980072,E=2980079,C=4)
	CT(4922)=(S=2980080,E=2980083,C=1)
	CT(4923)=(S=2980084,E=2980087,C=74)
	CT(4924)=(S=2980088,E=2980115,C=4)
	CT(4925)=(S=2980116,E=2980119,C=1)
	CT(4926)=(S=2980120,E=2980123,C=11)
	CT(4927)=(S=2980124,E=2980127,C=4)
	CT(4928)=(S=2980128,E=2980131,C=8)
	CT(4929)=(S=2980132,E=2980135,C=5)
	CT(4930)=(S=2980136,E=2980139,C=74)
	CT(4931)=(S=2980140,E=2980143,C=4)
	CT(4932)=(S=2980144,E=2980147,C=9)
	CT(4933)=(S=2980148,E=2980151,C=1)
	CT(4934)=(S=2980152,E=2980155,C=8)
	CT(4935)=(S=2980156,E=2980159,C=3)
	CT(4936)=(S=2980160,E=2980171,C=1)
	CT(4937)=(S=2980172,E=2980175,C=4)
	CT(4938)=(S=2980176,E=2980183,C=8)
	CT(4939)=(S=2980184,E=2980187,C=156)
	CT(4940)=(S=2980188,E=2980195,C=1)
	CT(4941)=(S=2980196,E=2980199,C=4)
	CT(4942)=(S=2980200,E=2980203,C=1)
	CT(4943)=(S=2980204,E=2980227,C=4)
	CT(4944)=(S=2980228,E=2980231,C=0)
	CT(4945)=(S=2980232,E=2980239,C=8)
	CT(4946)=(S=2980240,E=2980243,C=5)
	CT(4947)=(S=2980244,E=2980247,C=2)
	CT(4948)=(S=2980248,E=2980251,C=11)
	CT(4949)=(S=2980252,E=2980255,C=4)
	CT(4950)=(S=2980256,E=2980259,C=11)
	CT(4951)=(S=2980260,E=2980263,C=6)
	CT(4952)=(S=2980264,E=2980267,C=4)
	CT(4953)=(S=2980268,E=2980275,C=1)
	CT(4954)=(S=2980276,E=2980279,C=0)
	CT(4955)=(S=2980280,E=2980283,C=2)
	CT(4956)=(S=2980284,E=2980287,C=4)
	CT(4957)=(S=2980288,E=2980307,C=8)
	CT(4958)=(S=2980308,E=2980311,C=1)
	CT(4959)=(S=2980312,E=2980315,C=74)
	CT(4960)=(S=2980316,E=2980331,C=4)
	CT(4961)=(S=2980332,E=2980335,C=86)
	CT(4962)=(S=2980336,E=2980395,C=1)
	CT(4963)=(S=2980396,E=2980399,C=4)
	CT(4964)=(S=2980400,E=2980403,C=3)
	CT(4965)=(S=2980404,E=2980407,C=8)
	CT(4966)=(S=2980408,E=2980411,C=74)
	CT(4967)=(S=2980412,E=2980471,C=1)
	CT(4968)=(S=2980472,E=2980483,C=4)
	CT(4969)=(S=2980484,E=2980491,C=8)
	CT(4970)=(S=2980492,E=2980507,C=4)
	CT(4971)=(S=2980508,E=2980511,C=8)
	CT(4972)=(S=2980512,E=2980571,C=1)
	CT(4973)=(S=2980572,E=2980575,C=11)
	CT(4974)=(S=2980576,E=2980579,C=2)
	CT(4975)=(S=2980580,E=2980583,C=0)
	CT(4976)=(S=2980584,E=2980607,C=11)
	CT(4977)=(S=2980608,E=2980627,C=4)
	CT(4978)=(S=2980628,E=2980631,C=8)
	CT(4979)=(S=2980632,E=2980635,C=4)
	CT(4980)=(S=2980636,E=2980647,C=1)
	CT(4981)=(S=2980648,E=2980651,C=84)
	CT(4982)=(S=2980652,E=2980699,C=1)
	CT(4983)=(S=2980700,E=2980703,C=4)
	CT(4984)=(S=2980704,E=2980707,C=11)
	CT(4985)=(S=2980708,E=2980711,C=5)
	CT(4986)=(S=2980712,E=2980715,C=0)
	CT(4987)=(S=2980716,E=2980719,C=4)
	CT(4988)=(S=2980720,E=2980727,C=8)
	CT(4989)=(S=2980728,E=2980731,C=1)
	CT(4990)=(S=2980732,E=2980735,C=5)
	CT(4991)=(S=2980736,E=2980747,C=1)
	CT(4992)=(S=2980748,E=2980755,C=4)
	CT(4993)=(S=2980756,E=2980767,C=1)
	CT(4994)=(S=2980768,E=2980771,C=4)
	CT(4995)=(S=2980772,E=2980795,C=1)
	CT(4996)=(S=2980796,E=2980807,C=8)
	CT(4997)=(S=2980808,E=2980811,C=87)
	CT(4998)=(S=2980812,E=2980815,C=1)
	CT(4999)=(S=2980816,E=2980819,C=2)
	CT(5000)=(S=2980820,E=2980823,C=1)
	CT(5001)=(S=2980824,E=2980827,C=4)
	CT(5002)=(S=2980828,E=2980831,C=75)
	CT(5003)=(S=2980832,E=2980867,C=1)
	CT(5004)=(S=2980868,E=2980871,C=4)
	CT(5005)=(S=2980872,E=2980875,C=8)
	CT(5006)=(S=2980876,E=2980879,C=84)
	CT(5007)=(S=2980880,E=2980883,C=10)
	CT(5008)=(S=2980884,E=2980887,C=1)
	CT(5009)=(S=2980888,E=2980891,C=8)
	CT(5010)=(S=2980892,E=2980903,C=1)
	CT(5011)=(S=2980904,E=2980907,C=0)
	CT(5012)=(S=2980908,E=2980911,C=1)
	CT(5013)=(S=2980912,E=2980915,C=4)
	CT(5014)=(S=2980916,E=2980919,C=0)
	CT(5015)=(S=2980920,E=2980923,C=10)
	CT(5016)=(S=2980924,E=2980927,C=2)
	CT(5017)=(S=2980928,E=2980931,C=8)
	CT(5018)=(S=2980932,E=2980935,C=1)
	CT(5019)=(S=2980936,E=2980939,C=8)
	CT(5020)=(S=2980940,E=2980947,C=1)
	CT(5021)=(S=2980948,E=2980959,C=11)
	CT(5022)=(S=2980960,E=2980963,C=4)
	CT(5023)=(S=2980964,E=2980967,C=1)
	CT(5024)=(S=2980968,E=2980973,C=4)
	CT(5025)=(S=2980974,E=2980975,C=0)
	CT(5026)=(S=2980976,E=2980979,C=8)
	CT(5027)=(S=2980980,E=2980987,C=4)
	CT(5028)=(S=2980988,E=2980991,C=1)
	CT(5029)=(S=2980992,E=2980995,C=4)
	CT(5030)=(S=2980996,E=2980999,C=8)
	CT(5031)=(S=2981000,E=2981003,C=2)
	CT(5032)=(S=2981004,E=2981023,C=4)
	CT(5033)=(S=2981024,E=2981027,C=8)
	CT(5034)=(S=2981028,E=2981031,C=0)
	CT(5035)=(S=2981032,E=2981035,C=84)
	CT(5036)=(S=2981036,E=2981043,C=1)
	CT(5037)=(S=2981044,E=2981063,C=4)
	CT(5038)=(S=2981064,E=2981067,C=0)
	CT(5039)=(S=2981068,E=2981071,C=2)
	CT(5040)=(S=2981072,E=2981075,C=1)
	CT(5041)=(S=2981076,E=2981079,C=0)
	CT(5042)=(S=2981080,E=2981087,C=2)
	CT(5043)=(S=2981088,E=2981111,C=4)
	CT(5044)=(S=2981112,E=2981119,C=1)
	CT(5045)=(S=2981120,E=2981131,C=8)
	CT(5046)=(S=2981132,E=2981139,C=1)
	CT(5047)=(S=2981140,E=2981143,C=7)
	CT(5048)=(S=2981144,E=2981155,C=1)
	CT(5049)=(S=2981156,E=2981163,C=4)
	CT(5050)=(S=2981164,E=2981167,C=1)
	CT(5051)=(S=2981168,E=2981171,C=8)
	CT(5052)=(S=2981172,E=2981179,C=1)
	CT(5053)=(S=2981180,E=2981183,C=4)
	CT(5054)=(S=2981184,E=2981187,C=8)
	CT(5055)=(S=2981188,E=2981191,C=4)
	CT(5056)=(S=2981192,E=2981195,C=8)
	CT(5057)=(S=2981196,E=2981227,C=1)
	CT(5058)=(S=2981228,E=2981235,C=8)
	CT(5059)=(S=2981236,E=2981239,C=4)
	CT(5060)=(S=2981240,E=2981247,C=0)
	CT(5061)=(S=2981248,E=2981255,C=4)
	CT(5062)=(S=2981256,E=2981259,C=1)
	CT(5063)=(S=2981260,E=2981279,C=4)
	CT(5064)=(S=2981280,E=2981287,C=8)
	CT(5065)=(S=2981288,E=2981291,C=73)
	CT(5066)=(S=2981292,E=2981307,C=4)
	CT(5067)=(S=2981308,E=2981311,C=2)
	CT(5068)=(S=2981312,E=2981319,C=7)
	CT(5069)=(S=2981320,E=2981331,C=11)
	CT(5070)=(S=2981332,E=2981335,C=4)
	CT(5071)=(S=2981336,E=2981339,C=8)
	CT(5072)=(S=2981340,E=2981343,C=84)
	CT(5073)=(S=2981344,E=2981347,C=89)
	CT(5074)=(S=2981348,E=2981351,C=3)
	CT(5075)=(S=2981352,E=2981355,C=6)
	CT(5076)=(S=2981356,E=2981359,C=11)
	CT(5077)=(S=2981360,E=2981363,C=2)
	CT(5078)=(S=2981364,E=2981367,C=4)
	CT(5079)=(S=2981368,E=2981371,C=10)
	CT(5080)=(S=2981372,E=2981375,C=4)
	CT(5081)=(S=2981376,E=2981383,C=8)
	CT(5082)=(S=2981384,E=2981387,C=4)
	CT(5083)=(S=2981388,E=2981391,C=8)
	CT(5084)=(S=2981392,E=2981399,C=4)
	CT(5085)=(S=2981400,E=2981403,C=0)
	CT(5086)=(S=2981404,E=2981419,C=4)
	CT(5087)=(S=2981420,E=2981423,C=0)
	CT(5088)=(S=2981424,E=2981431,C=1)
	CT(5089)=(S=2981432,E=2981435,C=74)
	CT(5090)=(S=2981436,E=2981439,C=7)
	CT(5091)=(S=2981440,E=2981447,C=4)
	CT(5092)=(S=2981448,E=2981451,C=0)
	CT(5093)=(S=2981452,E=2981455,C=8)
	CT(5094)=(S=2981456,E=2981459,C=74)
	CT(5095)=(S=2981460,E=2981463,C=2)
	CT(5096)=(S=2981464,E=2981467,C=5)
	CT(5097)=(S=2981468,E=2981475,C=11)
	CT(5098)=(S=2981476,E=2981479,C=1)
	CT(5099)=(S=2981480,E=2981483,C=4)
	CT(5100)=(S=2981484,E=2981499,C=1)
	CT(5101)=(S=2981500,E=2981503,C=0)
	CT(5102)=(S=2981504,E=2981507,C=73)
	CT(5103)=(S=2981508,E=2981511,C=74)
	CT(5104)=(S=2981512,E=2981515,C=4)
	CT(5105)=(S=2981516,E=2981519,C=157)
	CT(5106)=(S=2981520,E=2981523,C=4)
	CT(5107)=(S=2981524,E=2981527,C=6)
	CT(5108)=(S=2981528,E=2981531,C=74)
	CT(5109)=(S=2981532,E=2981535,C=2)
	CT(5110)=(S=2981536,E=2981539,C=4)
	CT(5111)=(S=2981540,E=2981543,C=7)
	CT(5112)=(S=2981544,E=2981555,C=4)
	CT(5113)=(S=2981556,E=2981559,C=8)
	CT(5114)=(S=2981560,E=2981563,C=74)
	CT(5115)=(S=2981564,E=2981567,C=4)
	CT(5116)=(S=2981568,E=2981571,C=0)
	CT(5117)=(S=2981572,E=2981583,C=4)
	CT(5118)=(S=2981584,E=2981587,C=7)
	CT(5119)=(S=2981588,E=2981591,C=1)
	CT(5120)=(S=2981592,E=2981595,C=8)
	CT(5121)=(S=2981596,E=2981599,C=1)
	CT(5122)=(S=2981600,E=2981603,C=3)
	CT(5123)=(S=2981604,E=2981607,C=8)
	CT(5124)=(S=2981608,E=2981615,C=4)
	CT(5125)=(S=2981616,E=2981619,C=8)
	CT(5126)=(S=2981620,E=2981628,C=4)
	CT(5127)=(S=2981632,E=2981635,C=8)
	CT(5128)=(S=2981636,E=2981639,C=5)
	CT(5129)=(S=2981640,E=2981647,C=1)
	CT(5130)=(S=2981648,E=2981659,C=4)
	CT(5131)=(S=2981660,E=2981663,C=2)
	CT(5132)=(S=2981664,E=2981667,C=7)
	CT(5133)=(S=2981668,E=2981671,C=1)
	CT(5134)=(S=2981672,E=2981679,C=4)
	CT(5135)=(S=2981680,E=2981683,C=84)
	CT(5136)=(S=2981684,E=2981723,C=4)
	CT(5137)=(S=2981724,E=2981727,C=8)
	CT(5138)=(S=2981728,E=2981731,C=1)
	CT(5139)=(S=2981732,E=2981743,C=4)
	CT(5140)=(S=2981744,E=2981747,C=8)
	CT(5141)=(S=2981748,E=2981751,C=1)
	CT(5142)=(S=2981752,E=2981755,C=4)
	CT(5143)=(S=2981756,E=2981763,C=1)
	CT(5144)=(S=2981764,E=2981767,C=74)
	CT(5145)=(S=2981768,E=2981771,C=4)
	CT(5146)=(S=2981772,E=2981775,C=0)
	CT(5147)=(S=2981776,E=2981783,C=1)
	CT(5148)=(S=2981784,E=2981787,C=86)
	CT(5149)=(S=2981788,E=2981791,C=1)
	CT(5150)=(S=2998272,E=3002367,C=147)
	CT(5151)=(S=3002368,E=3003391,C=110)
	CT(5152)=(S=3003392,E=3004415,C=114)
	CT(5153)=(S=3004416,E=3005439,C=134)
	CT(5154)=(S=3010560,E=3012607,C=106)
	CT(5155)=(S=3014656,E=3014911,C=18)
	CT(5156)=(S=3014912,E=3015423,C=37)
	CT(5157)=(S=3015424,E=3015679,C=18)
	CT(5158)=(S=3015680,E=3016191,C=28)
	CT(5159)=(S=3016192,E=3016447,C=25)
	CT(5160)=(S=3016448,E=3016703,C=63)
	CT(5161)=(S=3016704,E=3016959,C=18)
	CT(5162)=(S=3016960,E=3017215,C=27)
	CT(5163)=(S=3017216,E=3017471,C=51)
	CT(5164)=(S=3017472,E=3017727,C=72)
	CT(5165)=(S=3017728,E=3017983,C=22)
	CT(5166)=(S=3017984,E=3018239,C=33)
	CT(5167)=(S=3018240,E=3018495,C=34)
	CT(5168)=(S=3018496,E=3018751,C=27)
	CT(5169)=(S=3018752,E=3018759,C=14)
	CT(5170)=(S=3018760,E=3018767,C=18)
	CT(5171)=(S=3018768,E=3018775,C=168)
	CT(5172)=(S=3018776,E=3018783,C=18)
	CT(5173)=(S=3018784,E=3018791,C=14)
	CT(5174)=(S=3018792,E=3018799,C=12)
	CT(5175)=(S=3018800,E=3018807,C=27)
	CT(5176)=(S=3018808,E=3018815,C=25)
	CT(5177)=(S=3018816,E=3018823,C=18)
	CT(5178)=(S=3018824,E=3018831,C=28)
	CT(5179)=(S=3018832,E=3018839,C=14)
	CT(5180)=(S=3018840,E=3018847,C=15)
	CT(5181)=(S=3018848,E=3018855,C=18)
	CT(5182)=(S=3018856,E=3018863,C=46)
	CT(5183)=(S=3018864,E=3018871,C=55)
	CT(5184)=(S=3018872,E=3018879,C=33)
	CT(5185)=(S=3018888,E=3018895,C=32)
	CT(5186)=(S=3018896,E=3018903,C=20)
	CT(5187)=(S=3018904,E=3018911,C=34)
	CT(5188)=(S=3018912,E=3018919,C=14)
	CT(5189)=(S=3018920,E=3018927,C=15)
	CT(5190)=(S=3018928,E=3018935,C=18)
	CT(5191)=(S=3018936,E=3018943,C=14)
	CT(5192)=(S=3018944,E=3018951,C=51)
	CT(5193)=(S=3018952,E=3018959,C=34)
	CT(5194)=(S=3018960,E=3018967,C=14)
	CT(5195)=(S=3018968,E=3018975,C=28)
	CT(5196)=(S=3018976,E=3018983,C=18)
	CT(5197)=(S=3018984,E=3018991,C=19)
	CT(5198)=(S=3018992,E=3018999,C=49)
	CT(5199)=(S=3019000,E=3019023,C=36)
	CT(5200)=(S=3019024,E=3019031,C=27)
	CT(5201)=(S=3019032,E=3019039,C=36)
	CT(5202)=(S=3019040,E=3019047,C=63)
	CT(5203)=(S=3019048,E=3019063,C=18)
	CT(5204)=(S=3019064,E=3019071,C=14)
	CT(5205)=(S=3019072,E=3019079,C=12)
	CT(5206)=(S=3019080,E=3019087,C=39)
	CT(5207)=(S=3019088,E=3019095,C=12)
	CT(5208)=(S=3019096,E=3019103,C=14)
	CT(5209)=(S=3019104,E=3019111,C=18)
	CT(5210)=(S=3019112,E=3019119,C=14)
	CT(5211)=(S=3019120,E=3019127,C=58)
	CT(5212)=(S=3019128,E=3019135,C=46)
	CT(5213)=(S=3019136,E=3019143,C=37)
	CT(5214)=(S=3019144,E=3019151,C=25)
	CT(5215)=(S=3019152,E=3019159,C=46)
	CT(5216)=(S=3019160,E=3019167,C=15)
	CT(5217)=(S=3019168,E=3019175,C=14)
	CT(5218)=(S=3019176,E=3019183,C=70)
	CT(5219)=(S=3019184,E=3019191,C=14)
	CT(5220)=(S=3019192,E=3019199,C=19)
	CT(5221)=(S=3019200,E=3019207,C=47)
	CT(5222)=(S=3019208,E=3019215,C=18)
	CT(5223)=(S=3019216,E=3019231,C=14)
	CT(5224)=(S=3019232,E=3019239,C=42)
	CT(5225)=(S=3019240,E=3019247,C=14)
	CT(5226)=(S=3019248,E=3019255,C=31)
	CT(5227)=(S=3019256,E=3019263,C=18)
	CT(5228)=(S=3019264,E=3019271,C=31)
	CT(5229)=(S=3019272,E=3019279,C=14)
	CT(5230)=(S=3019280,E=3019287,C=43)
	CT(5231)=(S=3019288,E=3019295,C=15)
	CT(5232)=(S=3019296,E=3019303,C=49)
	CT(5233)=(S=3019304,E=3019311,C=25)
	CT(5234)=(S=3019312,E=3019319,C=14)
	CT(5235)=(S=3019320,E=3019327,C=28)
	CT(5236)=(S=3019328,E=3019335,C=41)
	CT(5237)=(S=3019336,E=3019343,C=25)
	CT(5238)=(S=3019344,E=3019351,C=14)
	CT(5239)=(S=3019352,E=3019367,C=12)
	CT(5240)=(S=3019368,E=3019383,C=14)
	CT(5241)=(S=3019384,E=3019399,C=12)
	CT(5242)=(S=3019400,E=3019407,C=15)
	CT(5243)=(S=3019408,E=3019415,C=71)
	CT(5244)=(S=3019416,E=3019423,C=14)
	CT(5245)=(S=3019424,E=3019431,C=23)
	CT(5246)=(S=3019432,E=3019447,C=14)
	CT(5247)=(S=3019448,E=3019455,C=28)
	CT(5248)=(S=3019456,E=3019463,C=12)
	CT(5249)=(S=3019464,E=3019471,C=18)
	CT(5250)=(S=3019472,E=3019479,C=12)
	CT(5251)=(S=3019480,E=3019487,C=14)
	CT(5252)=(S=3019488,E=3019495,C=12)
	CT(5253)=(S=3019496,E=3019503,C=15)
	CT(5254)=(S=3019504,E=3019511,C=28)
	CT(5255)=(S=3019512,E=3019519,C=26)
	CT(5256)=(S=3019520,E=3019527,C=49)
	CT(5257)=(S=3019528,E=3019535,C=52)
	CT(5258)=(S=3019536,E=3019543,C=27)
	CT(5259)=(S=3019544,E=3019551,C=12)
	CT(5260)=(S=3019552,E=3019559,C=36)
	CT(5261)=(S=3019560,E=3019567,C=20)
	CT(5262)=(S=3019568,E=3019575,C=15)
	CT(5263)=(S=3019576,E=3019583,C=28)
	CT(5264)=(S=3019584,E=3019591,C=18)
	CT(5265)=(S=3019592,E=3019599,C=16)
	CT(5266)=(S=3019600,E=3019607,C=17)
	CT(5267)=(S=3019608,E=3019615,C=28)
	CT(5268)=(S=3019616,E=3019623,C=66)
	CT(5269)=(S=3019624,E=3019631,C=38)
	CT(5270)=(S=3019632,E=3019639,C=18)
	CT(5271)=(S=3019640,E=3019655,C=12)
	CT(5272)=(S=3019656,E=3019663,C=34)
	CT(5273)=(S=3019664,E=3019671,C=60)
	CT(5274)=(S=3019672,E=3019679,C=28)
	CT(5275)=(S=3019680,E=3019687,C=14)
	CT(5276)=(S=3019688,E=3019695,C=36)
	CT(5277)=(S=3019696,E=3019703,C=12)
	CT(5278)=(S=3019704,E=3019711,C=18)
	CT(5279)=(S=3019712,E=3019719,C=39)
	CT(5280)=(S=3019720,E=3019727,C=60)
	CT(5281)=(S=3019728,E=3019735,C=63)
	CT(5282)=(S=3019736,E=3019743,C=36)
	CT(5283)=(S=3019744,E=3019751,C=96)
	CT(5284)=(S=3019752,E=3019759,C=15)
	CT(5285)=(S=3019760,E=3019767,C=49)
	CT(5286)=(S=3019768,E=3019775,C=24)
	CT(5287)=(S=3019776,E=3019791,C=37)
	CT(5288)=(S=3019792,E=3019807,C=42)
	CT(5289)=(S=3019808,E=3019823,C=28)
	CT(5290)=(S=3019824,E=3019839,C=58)
	CT(5291)=(S=3019840,E=3019855,C=18)
	CT(5292)=(S=3019856,E=3019871,C=15)
	CT(5293)=(S=3019872,E=3019887,C=39)
	CT(5294)=(S=3019888,E=3019903,C=14)
	CT(5295)=(S=3019904,E=3019919,C=38)
	CT(5296)=(S=3019920,E=3019935,C=37)
	CT(5297)=(S=3019936,E=3019951,C=12)
	CT(5298)=(S=3019952,E=3019967,C=18)
	CT(5299)=(S=3019968,E=3019983,C=103)
	CT(5300)=(S=3019984,E=3019999,C=43)
	CT(5301)=(S=3020000,E=3020015,C=14)
	CT(5302)=(S=3020016,E=3020031,C=34)
	CT(5303)=(S=3020032,E=3020047,C=28)
	CT(5304)=(S=3020048,E=3020063,C=15)
	CT(5305)=(S=3020064,E=3020079,C=24)
	CT(5306)=(S=3020080,E=3020095,C=32)
	CT(5307)=(S=3020096,E=3020111,C=18)
	CT(5308)=(S=3020112,E=3020127,C=26)
	CT(5309)=(S=3020128,E=3020143,C=19)
	CT(5310)=(S=3020144,E=3020175,C=12)
	CT(5311)=(S=3020176,E=3020207,C=36)
	CT(5312)=(S=3020208,E=3020223,C=15)
	CT(5313)=(S=3020224,E=3020239,C=12)
	CT(5314)=(S=3020240,E=3020255,C=58)
	CT(5315)=(S=3020256,E=3020271,C=36)
	CT(5316)=(S=3020272,E=3020303,C=18)
	CT(5317)=(S=3020304,E=3020319,C=34)
	CT(5318)=(S=3020320,E=3020335,C=28)
	CT(5319)=(S=3020336,E=3020351,C=18)
	CT(5320)=(S=3020352,E=3020367,C=13)
	CT(5321)=(S=3020368,E=3020383,C=12)
	CT(5322)=(S=3020384,E=3020399,C=48)
	CT(5323)=(S=3020400,E=3020415,C=19)
	CT(5324)=(S=3020416,E=3020431,C=63)
	CT(5325)=(S=3020432,E=3020447,C=46)
	CT(5326)=(S=3020448,E=3020463,C=58)
	CT(5327)=(S=3020464,E=3020479,C=36)
	CT(5328)=(S=3020480,E=3020495,C=12)
	CT(5329)=(S=3020496,E=3020511,C=55)
	CT(5330)=(S=3020512,E=3020527,C=41)
	CT(5331)=(S=3020528,E=3020543,C=18)
	CT(5332)=(S=3020544,E=3020559,C=52)
	CT(5333)=(S=3020560,E=3020575,C=104)
	CT(5334)=(S=3020576,E=3020591,C=60)
	CT(5335)=(S=3020592,E=3020607,C=33)
	CT(5336)=(S=3020608,E=3020623,C=14)
	CT(5337)=(S=3020624,E=3020639,C=36)
	CT(5338)=(S=3020640,E=3020655,C=41)
	CT(5339)=(S=3020656,E=3020671,C=61)
	CT(5340)=(S=3020672,E=3020687,C=33)
	CT(5341)=(S=3020688,E=3020703,C=18)
	CT(5342)=(S=3020704,E=3020719,C=47)
	CT(5343)=(S=3020720,E=3020735,C=18)
	CT(5344)=(S=3020736,E=3020751,C=15)
	CT(5345)=(S=3020752,E=3020767,C=28)
	CT(5346)=(S=3020768,E=3020775,C=14)
	CT(5347)=(S=3020776,E=3020783,C=28)
	CT(5348)=(S=3020784,E=3020799,C=12)
	CT(5349)=(S=3020800,E=3021823,C=25)
	CT(5350)=(S=3021824,E=3021831,C=15)
	CT(5351)=(S=3021832,E=3021839,C=58)
	CT(5352)=(S=3021840,E=3021847,C=18)
	CT(5353)=(S=3021848,E=3021855,C=15)
	CT(5354)=(S=3021856,E=3021863,C=28)
	CT(5355)=(S=3021864,E=3021871,C=49)
	CT(5356)=(S=3021872,E=3021879,C=14)
	CT(5357)=(S=3021880,E=3021887,C=60)
	CT(5358)=(S=3021888,E=3021895,C=31)
	CT(5359)=(S=3021896,E=3021903,C=26)
	CT(5360)=(S=3021904,E=3021911,C=28)
	CT(5361)=(S=3021912,E=3021919,C=18)
	CT(5362)=(S=3021920,E=3021927,C=68)
	CT(5363)=(S=3021928,E=3021935,C=33)
	CT(5364)=(S=3021936,E=3021943,C=28)
	CT(5365)=(S=3021944,E=3021951,C=15)
	CT(5366)=(S=3021952,E=3021959,C=18)
	CT(5367)=(S=3021960,E=3021967,C=43)
	CT(5368)=(S=3021968,E=3021975,C=19)
	CT(5369)=(S=3021976,E=3021983,C=48)
	CT(5370)=(S=3021984,E=3021991,C=25)
	CT(5371)=(S=3021992,E=3021999,C=12)
	CT(5372)=(S=3022000,E=3022007,C=32)
	CT(5373)=(S=3022008,E=3022015,C=28)
	CT(5374)=(S=3022016,E=3022023,C=31)
	CT(5375)=(S=3022024,E=3022031,C=34)
	CT(5376)=(S=3022032,E=3022039,C=25)
	CT(5377)=(S=3022040,E=3022047,C=50)
	CT(5378)=(S=3022048,E=3022055,C=18)
	CT(5379)=(S=3022056,E=3022063,C=37)
	CT(5380)=(S=3022064,E=3022071,C=58)
	CT(5381)=(S=3022072,E=3022079,C=14)
	CT(5382)=(S=3022080,E=3022087,C=71)
	CT(5383)=(S=3022088,E=3022095,C=18)
	CT(5384)=(S=3022096,E=3022103,C=58)
	CT(5385)=(S=3022112,E=3022119,C=42)
	CT(5386)=(S=3022120,E=3022127,C=28)
	CT(5387)=(S=3022128,E=3022135,C=25)
	CT(5388)=(S=3022136,E=3022143,C=14)
	CT(5389)=(S=3022144,E=3022151,C=28)
	CT(5390)=(S=3022152,E=3022159,C=18)
	CT(5391)=(S=3022160,E=3022167,C=23)
	CT(5392)=(S=3022168,E=3022175,C=14)
	CT(5393)=(S=3022176,E=3022183,C=28)
	CT(5394)=(S=3022184,E=3022191,C=15)
	CT(5395)=(S=3022192,E=3022199,C=18)
	CT(5396)=(S=3022200,E=3022207,C=12)
	CT(5397)=(S=3022208,E=3022215,C=31)
	CT(5398)=(S=3022216,E=3022223,C=50)
	CT(5399)=(S=3022224,E=3022231,C=58)
	CT(5400)=(S=3022232,E=3022247,C=18)
	CT(5401)=(S=3022248,E=3022255,C=54)
	CT(5402)=(S=3022256,E=3022263,C=70)
	CT(5403)=(S=3022264,E=3022271,C=33)
	CT(5404)=(S=3022272,E=3022279,C=18)
	CT(5405)=(S=3022280,E=3022287,C=15)
	CT(5406)=(S=3022288,E=3022295,C=18)
	CT(5407)=(S=3022296,E=3022303,C=27)
	CT(5408)=(S=3022304,E=3022311,C=33)
	CT(5409)=(S=3022312,E=3022319,C=18)
	CT(5410)=(S=3022320,E=3022327,C=36)
	CT(5411)=(S=3022328,E=3022335,C=19)
	CT(5412)=(S=3022336,E=3022343,C=28)
	CT(5413)=(S=3022344,E=3022351,C=14)
	CT(5414)=(S=3022352,E=3022359,C=25)
	CT(5415)=(S=3022360,E=3022367,C=28)
	CT(5416)=(S=3022368,E=3022383,C=18)
	CT(5417)=(S=3022384,E=3022391,C=14)
	CT(5418)=(S=3022392,E=3022399,C=28)
	CT(5419)=(S=3022400,E=3022407,C=33)
	CT(5420)=(S=3022408,E=3022423,C=28)
	CT(5421)=(S=3022424,E=3022431,C=33)
	CT(5422)=(S=3022432,E=3022439,C=14)
	CT(5423)=(S=3022440,E=3022447,C=25)
	CT(5424)=(S=3022448,E=3022455,C=28)
	CT(5425)=(S=3022456,E=3022463,C=24)
	CT(5426)=(S=3022464,E=3022471,C=47)
	CT(5427)=(S=3022472,E=3022479,C=14)
	CT(5428)=(S=3022480,E=3022487,C=33)
	CT(5429)=(S=3022488,E=3022495,C=18)
	CT(5430)=(S=3022496,E=3022503,C=31)
	CT(5431)=(S=3022504,E=3022511,C=15)
	CT(5432)=(S=3022512,E=3022519,C=37)
	CT(5433)=(S=3022528,E=3022535,C=14)
	CT(5434)=(S=3022536,E=3022543,C=12)
	CT(5435)=(S=3022544,E=3022551,C=24)
	CT(5436)=(S=3022552,E=3022559,C=15)
	CT(5437)=(S=3022560,E=3022567,C=35)
	CT(5438)=(S=3022568,E=3022575,C=33)
	CT(5439)=(S=3022576,E=3022591,C=15)
	CT(5440)=(S=3022592,E=3022599,C=18)
	CT(5441)=(S=3022600,E=3022607,C=36)
	CT(5442)=(S=3022608,E=3022615,C=24)
	CT(5443)=(S=3022616,E=3022623,C=18)
	CT(5444)=(S=3022624,E=3022631,C=58)
	CT(5445)=(S=3022632,E=3022639,C=12)
	CT(5446)=(S=3022640,E=3022647,C=36)
	CT(5447)=(S=3022648,E=3022655,C=25)
	CT(5448)=(S=3022656,E=3022663,C=32)
	CT(5449)=(S=3022664,E=3022671,C=35)
	CT(5450)=(S=3022672,E=3022679,C=14)
	CT(5451)=(S=3022680,E=3022687,C=49)
	CT(5452)=(S=3022688,E=3022695,C=17)
	CT(5453)=(S=3022696,E=3022703,C=15)
	CT(5454)=(S=3022704,E=3022711,C=37)
	CT(5455)=(S=3022712,E=3022719,C=28)
	CT(5456)=(S=3022720,E=3022727,C=15)
	CT(5457)=(S=3022728,E=3022735,C=57)
	CT(5458)=(S=3022736,E=3022743,C=37)
	CT(5459)=(S=3022744,E=3022751,C=18)
	CT(5460)=(S=3022752,E=3022759,C=12)
	CT(5461)=(S=3022760,E=3022767,C=14)
	CT(5462)=(S=3022768,E=3022775,C=50)
	CT(5463)=(S=3022776,E=3022783,C=27)
	CT(5464)=(S=3022784,E=3022791,C=12)
	CT(5465)=(S=3022792,E=3022799,C=14)
	CT(5466)=(S=3022800,E=3022807,C=12)
	CT(5467)=(S=3022808,E=3022815,C=28)
	CT(5468)=(S=3022816,E=3022823,C=14)
	CT(5469)=(S=3022824,E=3022831,C=13)
	CT(5470)=(S=3022832,E=3022839,C=14)
	CT(5471)=(S=3022840,E=3022847,C=15)
	CT(5472)=(S=3022848,E=3022879,C=26)
	CT(5473)=(S=3022880,E=3022911,C=24)
	CT(5474)=(S=3022912,E=3022943,C=18)
	CT(5475)=(S=3022944,E=3022975,C=61)
	CT(5476)=(S=3022976,E=3023007,C=24)
	CT(5477)=(S=3023008,E=3023039,C=41)
	CT(5478)=(S=3023040,E=3023071,C=43)
	CT(5479)=(S=3023072,E=3023135,C=14)
	CT(5480)=(S=3023136,E=3023167,C=31)
	CT(5481)=(S=3023168,E=3023199,C=63)
	CT(5482)=(S=3023200,E=3023231,C=33)
	CT(5483)=(S=3023232,E=3023263,C=14)
	CT(5484)=(S=3023264,E=3023295,C=12)
	CT(5485)=(S=3023296,E=3023327,C=102)
	CT(5486)=(S=3023328,E=3023359,C=31)
	CT(5487)=(S=3023360,E=3023391,C=14)
	CT(5488)=(S=3023392,E=3023455,C=25)
	CT(5489)=(S=3023456,E=3023487,C=26)
	CT(5490)=(S=3023488,E=3023519,C=18)
	CT(5491)=(S=3023520,E=3023551,C=26)
	CT(5492)=(S=3023552,E=3023583,C=20)
	CT(5493)=(S=3023584,E=3023615,C=71)
	CT(5494)=(S=3023616,E=3023647,C=12)
	CT(5495)=(S=3023648,E=3023679,C=28)
	CT(5496)=(S=3023680,E=3023711,C=67)
	CT(5497)=(S=3023712,E=3023743,C=25)
	CT(5498)=(S=3023744,E=3023775,C=59)
	CT(5499)=(S=3023776,E=3023807,C=51)
	CT(5500)=(S=3023808,E=3023839,C=50)
	CT(5501)=(S=3023840,E=3023871,C=31)
	CT(5502)=(S=3023872,E=3023903,C=18)
	CT(5503)=(S=3023904,E=3023935,C=33)
	CT(5504)=(S=3023936,E=3023967,C=44)
	CT(5505)=(S=3023968,E=3023999,C=26)
	CT(5506)=(S=3024000,E=3024031,C=20)
	CT(5507)=(S=3024032,E=3024063,C=59)
	CT(5508)=(S=3024064,E=3024071,C=14)
	CT(5509)=(S=3024072,E=3024075,C=169)
	CT(5510)=(S=3024076,E=3024087,C=24)
	CT(5511)=(S=3024088,E=3024095,C=55)
	CT(5512)=(S=3024096,E=3024127,C=58)
	CT(5513)=(S=3024128,E=3024159,C=15)
	CT(5514)=(S=3024160,E=3024191,C=14)
	CT(5515)=(S=3024192,E=3024255,C=25)
	CT(5516)=(S=3024256,E=3024287,C=18)
	CT(5517)=(S=3024288,E=3024319,C=14)
	CT(5518)=(S=3024320,E=3024351,C=31)
	CT(5519)=(S=3024352,E=3024383,C=15)
	CT(5520)=(S=3024384,E=3024447,C=18)
	CT(5521)=(S=3024448,E=3024479,C=23)
	CT(5522)=(S=3024480,E=3024511,C=18)
	CT(5523)=(S=3024512,E=3024543,C=26)
	CT(5524)=(S=3024544,E=3024575,C=14)
	CT(5525)=(S=3024576,E=3024607,C=36)
	CT(5526)=(S=3024608,E=3024639,C=28)
	CT(5527)=(S=3024640,E=3024703,C=18)
	CT(5528)=(S=3024704,E=3024735,C=31)
	CT(5529)=(S=3024736,E=3024767,C=19)
	CT(5530)=(S=3024768,E=3024799,C=55)
	CT(5531)=(S=3024800,E=3024831,C=33)
	CT(5532)=(S=3024832,E=3024863,C=25)
	CT(5533)=(S=3024864,E=3024895,C=18)
	CT(5534)=(S=3024896,E=3024959,C=46)
	CT(5535)=(S=3024960,E=3025023,C=51)
	CT(5536)=(S=3025024,E=3025087,C=15)
	CT(5537)=(S=3025088,E=3025151,C=45)
	CT(5538)=(S=3025152,E=3025215,C=28)
	CT(5539)=(S=3025216,E=3025279,C=18)
	CT(5540)=(S=3025280,E=3025343,C=58)
	CT(5541)=(S=3025344,E=3025407,C=26)
	CT(5542)=(S=3025408,E=3025471,C=18)
	CT(5543)=(S=3025472,E=3025535,C=104)
	CT(5544)=(S=3025536,E=3025599,C=18)
	CT(5545)=(S=3025600,E=3025663,C=20)
	CT(5546)=(S=3025664,E=3025727,C=14)
	CT(5547)=(S=3025728,E=3025791,C=43)
	CT(5548)=(S=3025792,E=3025855,C=14)
	CT(5549)=(S=3025856,E=3025983,C=18)
	CT(5550)=(S=3025984,E=3026047,C=23)
	CT(5551)=(S=3026048,E=3026111,C=36)
	CT(5552)=(S=3026112,E=3026175,C=15)
	CT(5553)=(S=3026176,E=3026239,C=18)
	CT(5554)=(S=3026240,E=3026303,C=58)
	CT(5555)=(S=3026304,E=3026367,C=37)
	CT(5556)=(S=3026368,E=3026495,C=18)
	CT(5557)=(S=3026496,E=3026559,C=31)
	CT(5558)=(S=3026560,E=3026623,C=18)
	CT(5559)=(S=3026624,E=3026687,C=19)
	CT(5560)=(S=3026688,E=3026751,C=18)
	CT(5561)=(S=3026752,E=3026815,C=51)
	CT(5562)=(S=3026816,E=3026879,C=33)
	CT(5563)=(S=3026880,E=3027199,C=18)
	CT(5564)=(S=3027200,E=3027327,C=40)
	CT(5565)=(S=3027328,E=3027455,C=38)
	CT(5566)=(S=3027456,E=3027583,C=21)
	CT(5567)=(S=3027584,E=3027711,C=18)
	CT(5568)=(S=3027712,E=3027839,C=26)
	CT(5569)=(S=3027840,E=3027967,C=63)
	CT(5570)=(S=3027968,E=3028095,C=23)
	CT(5571)=(S=3028096,E=3028223,C=18)
	CT(5572)=(S=3028224,E=3028351,C=30)
	CT(5573)=(S=3028352,E=3028479,C=68)
	CT(5574)=(S=3028480,E=3028607,C=72)
	CT(5575)=(S=3028608,E=3028735,C=52)
	CT(5576)=(S=3028736,E=3028863,C=53)
	CT(5577)=(S=3028864,E=3028991,C=51)
	CT(5578)=(S=3028992,E=3029247,C=68)
	CT(5579)=(S=3029248,E=3029375,C=42)
	CT(5580)=(S=3029376,E=3029503,C=30)
	CT(5581)=(S=3029504,E=3029631,C=13)
	CT(5582)=(S=3029632,E=3029759,C=30)
	CT(5583)=(S=3029760,E=3029887,C=19)
	CT(5584)=(S=3029888,E=3030015,C=28)
	CT(5585)=(S=3030016,E=3030143,C=43)
	CT(5586)=(S=3030144,E=3030271,C=14)
	CT(5587)=(S=3030272,E=3030655,C=18)
	CT(5588)=(S=3030656,E=3030783,C=26)
	CT(5589)=(S=3030784,E=3030911,C=31)
	CT(5590)=(S=3030912,E=3031039,C=18)
	CT(5591)=(S=3031040,E=3031551,C=14)
	CT(5592)=(S=3031552,E=3032063,C=27)
	CT(5593)=(S=3032064,E=3032575,C=14)
	CT(5594)=(S=3032576,E=3033087,C=66)
	CT(5595)=(S=3033088,E=3033599,C=18)
	CT(5596)=(S=3033600,E=3034111,C=42)
	CT(5597)=(S=3034112,E=3034623,C=58)
	CT(5598)=(S=3034624,E=3039231,C=28)
	CT(5599)=(S=3039232,E=3039487,C=31)
	CT(5600)=(S=3039488,E=3039743,C=38)
	CT(5601)=(S=3039744,E=3039999,C=31)
	CT(5602)=(S=3040000,E=3040255,C=96)
	CT(5603)=(S=3040256,E=3040511,C=26)
	CT(5604)=(S=3040512,E=3040767,C=36)
	CT(5605)=(S=3040768,E=3040863,C=38)
	CT(5606)=(S=3040864,E=3040871,C=95)
	CT(5607)=(S=3040872,E=3040887,C=38)
	CT(5608)=(S=3040888,E=3040911,C=26)
	CT(5609)=(S=3040912,E=3040919,C=38)
	CT(5610)=(S=3040920,E=3040920,C=170)
	CT(5611)=(S=3040921,E=3040927,C=38)
	CT(5612)=(S=3040928,E=3040935,C=95)
	CT(5613)=(S=3040936,E=3040944,C=38)
	CT(5614)=(S=3040945,E=3040948,C=14)
	CT(5615)=(S=3040949,E=3040951,C=38)
	CT(5616)=(S=3040952,E=3040955,C=26)
	CT(5617)=(S=3040956,E=3040957,C=15)
	CT(5618)=(S=3040958,E=3040959,C=38)
	CT(5619)=(S=3040960,E=3040991,C=14)
	CT(5620)=(S=3040992,E=3040999,C=36)
	CT(5621)=(S=3041000,E=3041006,C=38)
	CT(5622)=(S=3041007,E=3041007,C=29)
	CT(5623)=(S=3041008,E=3041018,C=38)
	CT(5624)=(S=3041019,E=3041019,C=44)
	CT(5625)=(S=3041020,E=3041023,C=38)
	CT(5626)=(S=3041024,E=3041279,C=32)
	CT(5627)=(S=3041280,E=3041535,C=37)
	CT(5628)=(S=3041536,E=3041791,C=12)
	CT(5629)=(S=3041792,E=3042047,C=37)
	CT(5630)=(S=3042048,E=3042303,C=50)
	CT(5631)=(S=3042304,E=3042559,C=38)
	CT(5632)=(S=3042560,E=3042815,C=60)
	CT(5633)=(S=3042816,E=3043071,C=28)
	CT(5634)=(S=3043072,E=3043327,C=18)
	CT(5635)=(S=3043328,E=3043839,C=58)
	CT(5636)=(S=3043840,E=3044351,C=28)
	CT(5637)=(S=3044352,E=3044863,C=17)
	CT(5638)=(S=3044864,E=3045375,C=31)
	CT(5639)=(S=3045376,E=3045887,C=17)
	CT(5640)=(S=3045888,E=3046399,C=52)
	CT(5641)=(S=3046400,E=3046911,C=42)
	CT(5642)=(S=3046912,E=3047423,C=34)
	CT(5643)=(S=3047424,E=3047679,C=28)
	CT(5644)=(S=3047680,E=3047935,C=36)
	CT(5645)=(S=3047936,E=3048191,C=66)
	CT(5646)=(S=3048192,E=3048447,C=55)
	CT(5647)=(S=3048448,E=3048703,C=47)
	CT(5648)=(S=3048704,E=3048959,C=31)
	CT(5649)=(S=3048960,E=3049215,C=58)
	CT(5650)=(S=3049216,E=3049471,C=33)
	CT(5651)=(S=3049472,E=3049727,C=25)
	CT(5652)=(S=3049728,E=3049983,C=63)
	CT(5653)=(S=3049984,E=3050239,C=18)
	CT(5654)=(S=3050240,E=3050495,C=50)
	CT(5655)=(S=3050496,E=3050751,C=34)
	CT(5656)=(S=3050752,E=3051007,C=15)
	CT(5657)=(S=3051008,E=3051263,C=28)
	CT(5658)=(S=3051264,E=3051519,C=26)
	CT(5659)=(S=3051520,E=3052031,C=36)
	CT(5660)=(S=3052032,E=3052543,C=18)
	CT(5661)=(S=3052544,E=3052559,C=58)
	CT(5662)=(S=3052560,E=3052575,C=31)
	CT(5663)=(S=3052576,E=3052591,C=26)
	CT(5664)=(S=3052592,E=3052607,C=18)
	CT(5665)=(S=3052608,E=3052639,C=58)
	CT(5666)=(S=3052640,E=3052655,C=18)
	CT(5667)=(S=3052656,E=3052671,C=31)
	CT(5668)=(S=3052672,E=3052687,C=18)
	CT(5669)=(S=3052688,E=3052719,C=58)
	CT(5670)=(S=3052720,E=3052735,C=31)
	CT(5671)=(S=3052736,E=3052799,C=18)
	CT(5672)=(S=3052800,E=3052815,C=23)
	CT(5673)=(S=3052816,E=3052831,C=48)
	CT(5674)=(S=3052832,E=3052847,C=18)
	CT(5675)=(S=3052848,E=3052863,C=31)
	CT(5676)=(S=3052864,E=3052879,C=18)
	CT(5677)=(S=3052880,E=3052911,C=31)
	CT(5678)=(S=3052912,E=3052927,C=33)
	CT(5679)=(S=3052928,E=3052943,C=18)
	CT(5680)=(S=3052944,E=3052959,C=58)
	CT(5681)=(S=3052960,E=3052991,C=31)
	CT(5682)=(S=3052992,E=3053007,C=34)
	CT(5683)=(S=3053008,E=3053023,C=58)
	CT(5684)=(S=3053024,E=3053039,C=18)
	CT(5685)=(S=3053040,E=3053055,C=58)
	CT(5686)=(S=3053056,E=3053087,C=31)
	CT(5687)=(S=3053088,E=3053119,C=52)
	CT(5688)=(S=3053120,E=3053183,C=31)
	CT(5689)=(S=3053184,E=3053247,C=18)
	CT(5690)=(S=3053248,E=3053279,C=71)
	CT(5691)=(S=3053280,E=3053327,C=18)
	CT(5692)=(S=3053328,E=3053335,C=58)
	CT(5693)=(S=3053336,E=3053343,C=18)
	CT(5694)=(S=3053344,E=3053351,C=38)
	CT(5695)=(S=3053352,E=3053367,C=31)
	CT(5696)=(S=3053368,E=3053375,C=71)
	CT(5697)=(S=3053376,E=3053383,C=18)
	CT(5698)=(S=3053384,E=3053391,C=58)
	CT(5699)=(S=3053392,E=3053399,C=31)
	CT(5700)=(S=3053400,E=3053407,C=18)
	CT(5701)=(S=3053408,E=3053415,C=49)
	CT(5702)=(S=3053416,E=3053423,C=18)
	CT(5703)=(S=3053424,E=3053431,C=58)
	CT(5704)=(S=3053432,E=3053447,C=46)
	CT(5705)=(S=3053448,E=3053455,C=58)
	CT(5706)=(S=3053456,E=3053463,C=31)
	CT(5707)=(S=3053464,E=3053471,C=18)
	CT(5708)=(S=3053472,E=3053479,C=38)
	CT(5709)=(S=3053480,E=3053487,C=18)
	CT(5710)=(S=3053488,E=3053495,C=31)
	CT(5711)=(S=3053496,E=3053503,C=58)
	CT(5712)=(S=3053504,E=3053511,C=31)
	CT(5713)=(S=3053512,E=3053519,C=42)
	CT(5714)=(S=3053520,E=3053527,C=23)
	CT(5715)=(S=3053528,E=3053535,C=58)
	CT(5716)=(S=3053536,E=3053543,C=38)
	CT(5717)=(S=3053544,E=3053551,C=52)
	CT(5718)=(S=3053552,E=3053559,C=18)
	CT(5719)=(S=3053560,E=3053567,C=31)
	CT(5720)=(S=3053568,E=3054079,C=23)
	CT(5721)=(S=3054080,E=3054591,C=37)
	CT(5722)=(S=3054592,E=3055103,C=27)
	CT(5723)=(S=3055104,E=3055679,C=18)
	CT(5724)=(S=3055680,E=3055743,C=31)
	CT(5725)=(S=3055744,E=3055935,C=18)
	CT(5726)=(S=3055936,E=3055999,C=102)
	CT(5727)=(S=3056000,E=3056063,C=18)
	CT(5728)=(S=3056064,E=3056127,C=30)
	CT(5729)=(S=3056128,E=3056191,C=31)
	CT(5730)=(S=3056192,E=3056255,C=19)
	CT(5731)=(S=3056256,E=3056319,C=12)
	CT(5732)=(S=3056320,E=3056383,C=66)
	CT(5733)=(S=3056384,E=3056447,C=52)
	CT(5734)=(S=3056448,E=3056511,C=28)
	CT(5735)=(S=3056512,E=3056575,C=18)
	CT(5736)=(S=3056576,E=3056639,C=47)
	CT(5737)=(S=3056640,E=3056703,C=52)
	CT(5738)=(S=3056704,E=3056767,C=26)
	CT(5739)=(S=3056768,E=3056831,C=31)
	CT(5740)=(S=3056832,E=3056959,C=18)
	CT(5741)=(S=3056960,E=3057023,C=12)
	CT(5742)=(S=3057024,E=3057151,C=28)
	CT(5743)=(S=3057152,E=3057215,C=53)
	CT(5744)=(S=3057216,E=3057279,C=18)
	CT(5745)=(S=3057280,E=3057311,C=36)
	CT(5746)=(S=3057312,E=3057327,C=44)
	CT(5747)=(S=3057328,E=3057343,C=36)
	CT(5748)=(S=3057344,E=3057407,C=18)
	CT(5749)=(S=3057408,E=3057471,C=28)
	CT(5750)=(S=3057472,E=3057535,C=18)
	CT(5751)=(S=3057536,E=3057599,C=28)
	CT(5752)=(S=3057600,E=3057663,C=33)
	CT(5753)=(S=3057664,E=3058687,C=58)
	CT(5754)=(S=3058688,E=3058751,C=18)
	CT(5755)=(S=3058752,E=3058783,C=31)
	CT(5756)=(S=3058784,E=3058815,C=18)
	CT(5757)=(S=3058816,E=3058847,C=31)
	CT(5758)=(S=3058848,E=3058879,C=18)
	CT(5759)=(S=3058880,E=3058943,C=31)
	CT(5760)=(S=3058944,E=3059007,C=18)
	CT(5761)=(S=3059008,E=3059119,C=31)
	CT(5762)=(S=3059120,E=3059135,C=18)
	CT(5763)=(S=3059136,E=3059151,C=33)
	CT(5764)=(S=3059152,E=3059167,C=18)
	CT(5765)=(S=3059168,E=3059183,C=47)
	CT(5766)=(S=3059184,E=3059199,C=38)
	CT(5767)=(S=3059200,E=3059207,C=58)
	CT(5768)=(S=3059208,E=3059215,C=18)
	CT(5769)=(S=3059216,E=3059223,C=33)
	CT(5770)=(S=3059224,E=3059231,C=58)
	CT(5771)=(S=3059232,E=3059239,C=33)
	CT(5772)=(S=3059240,E=3059247,C=31)
	CT(5773)=(S=3059248,E=3059255,C=18)
	CT(5774)=(S=3059256,E=3059263,C=33)
	CT(5775)=(S=3059264,E=3059271,C=31)
	CT(5776)=(S=3059272,E=3059279,C=58)
	CT(5777)=(S=3059280,E=3059295,C=18)
	CT(5778)=(S=3059296,E=3059303,C=46)
	CT(5779)=(S=3059304,E=3059319,C=18)
	CT(5780)=(S=3059320,E=3059327,C=31)
	CT(5781)=(S=3059328,E=3059335,C=36)
	CT(5782)=(S=3059336,E=3059351,C=38)
	CT(5783)=(S=3059352,E=3059359,C=33)
	CT(5784)=(S=3059360,E=3059367,C=31)
	CT(5785)=(S=3059368,E=3059383,C=58)
	CT(5786)=(S=3059384,E=3059391,C=95)
	CT(5787)=(S=3059392,E=3059399,C=18)
	CT(5788)=(S=3059400,E=3059407,C=38)
	CT(5789)=(S=3059408,E=3059415,C=58)
	CT(5790)=(S=3059416,E=3059423,C=31)
	CT(5791)=(S=3059424,E=3059439,C=58)
	CT(5792)=(S=3059440,E=3059447,C=31)
	CT(5793)=(S=3059448,E=3059455,C=18)
	CT(5794)=(S=3059456,E=3059463,C=71)
	CT(5795)=(S=3059464,E=3059467,C=34)
	CT(5796)=(S=3059468,E=3059468,C=18)
	CT(5797)=(S=3059469,E=3059469,C=14)
	CT(5798)=(S=3059470,E=3059471,C=58)
	CT(5799)=(S=3059472,E=3059479,C=31)
	CT(5800)=(S=3059480,E=3059495,C=18)
	CT(5801)=(S=3059496,E=3059503,C=58)
	CT(5802)=(S=3059504,E=3059511,C=14)
	CT(5803)=(S=3059512,E=3059519,C=28)
	CT(5804)=(S=3059520,E=3059543,C=31)
	CT(5805)=(S=3059544,E=3059551,C=33)
	CT(5806)=(S=3059552,E=3059559,C=18)
	CT(5807)=(S=3059560,E=3059575,C=58)
	CT(5808)=(S=3059576,E=3059591,C=18)
	CT(5809)=(S=3059592,E=3059607,C=31)
	CT(5810)=(S=3059608,E=3059615,C=18)
	CT(5811)=(S=3059616,E=3059623,C=31)
	CT(5812)=(S=3059624,E=3059631,C=68)
	CT(5813)=(S=3059632,E=3059639,C=33)
	CT(5814)=(S=3059640,E=3059647,C=31)
	CT(5815)=(S=3059648,E=3059679,C=18)
	CT(5816)=(S=3059680,E=3059695,C=58)
	CT(5817)=(S=3059696,E=3059711,C=31)
	CT(5818)=(S=3059712,E=3060223,C=22)
	CT(5819)=(S=3060224,E=3060735,C=49)
	CT(5820)=(S=3060736,E=3061247,C=18)
	CT(5821)=(S=3061248,E=3061255,C=12)
	CT(5822)=(S=3061256,E=3061263,C=63)
	CT(5823)=(S=3061264,E=3061271,C=28)
	CT(5824)=(S=3061272,E=3061279,C=18)
	CT(5825)=(S=3061280,E=3061287,C=21)
	CT(5826)=(S=3061288,E=3061295,C=12)
	CT(5827)=(S=3061296,E=3061303,C=68)
	CT(5828)=(S=3061304,E=3061311,C=14)
	CT(5829)=(S=3061312,E=3061319,C=37)
	CT(5830)=(S=3061320,E=3061327,C=12)
	CT(5831)=(S=3061328,E=3061335,C=31)
	CT(5832)=(S=3061336,E=3061343,C=15)
	CT(5833)=(S=3061344,E=3061351,C=58)
	CT(5834)=(S=3061352,E=3061359,C=36)
	CT(5835)=(S=3061360,E=3061367,C=47)
	CT(5836)=(S=3061368,E=3061375,C=36)
	CT(5837)=(S=3061376,E=3061383,C=18)
	CT(5838)=(S=3061384,E=3061399,C=28)
	CT(5839)=(S=3061400,E=3061407,C=15)
	CT(5840)=(S=3061408,E=3061415,C=47)
	CT(5841)=(S=3061416,E=3061423,C=66)
	CT(5842)=(S=3061424,E=3061431,C=36)
	CT(5843)=(S=3061432,E=3061439,C=48)
	CT(5844)=(S=3061440,E=3061447,C=49)
	CT(5845)=(S=3061448,E=3061455,C=19)
	CT(5846)=(S=3061456,E=3061463,C=12)
	CT(5847)=(S=3061464,E=3061471,C=36)
	CT(5848)=(S=3061472,E=3061479,C=52)
	CT(5849)=(S=3061480,E=3061487,C=15)
	CT(5850)=(S=3061488,E=3061503,C=28)
	CT(5851)=(S=3061504,E=3061511,C=18)
	CT(5852)=(S=3061512,E=3061519,C=14)
	CT(5853)=(S=3061520,E=3061527,C=22)
	CT(5854)=(S=3061528,E=3061535,C=19)
	CT(5855)=(S=3061536,E=3061543,C=12)
	CT(5856)=(S=3061544,E=3061551,C=28)
	CT(5857)=(S=3061552,E=3061559,C=12)
	CT(5858)=(S=3061560,E=3061575,C=33)
	CT(5859)=(S=3061576,E=3061583,C=47)
	CT(5860)=(S=3061584,E=3061591,C=18)
	CT(5861)=(S=3061592,E=3061599,C=17)
	CT(5862)=(S=3061600,E=3061607,C=28)
	CT(5863)=(S=3061608,E=3061615,C=14)
	CT(5864)=(S=3061616,E=3061623,C=25)
	CT(5865)=(S=3061624,E=3061631,C=96)
	CT(5866)=(S=3061632,E=3061639,C=18)
	CT(5867)=(S=3061640,E=3061647,C=24)
	CT(5868)=(S=3061648,E=3061655,C=36)
	CT(5869)=(S=3061656,E=3061663,C=28)
	CT(5870)=(S=3061664,E=3061671,C=18)
	CT(5871)=(S=3061672,E=3061679,C=27)
	CT(5872)=(S=3061680,E=3061687,C=18)
	CT(5873)=(S=3061696,E=3061711,C=14)
	CT(5874)=(S=3061712,E=3061719,C=58)
	CT(5875)=(S=3061720,E=3061727,C=60)
	CT(5876)=(S=3061728,E=3061735,C=12)
	CT(5877)=(S=3061736,E=3061743,C=15)
	CT(5878)=(S=3061744,E=3061759,C=36)
	CT(5879)=(S=3061760,E=3061887,C=23)
	CT(5880)=(S=3061888,E=3062015,C=104)
	CT(5881)=(S=3062016,E=3062143,C=31)
	CT(5882)=(S=3062144,E=3062271,C=61)
	CT(5883)=(S=3062272,E=3062399,C=58)
	CT(5884)=(S=3062400,E=3062527,C=101)
	CT(5885)=(S=3062528,E=3062655,C=18)
	CT(5886)=(S=3062656,E=3062783,C=58)
	CT(5887)=(S=3062784,E=3062911,C=18)
	CT(5888)=(S=3062912,E=3063039,C=57)
	CT(5889)=(S=3063040,E=3063167,C=28)
	CT(5890)=(S=3063168,E=3063295,C=21)
	CT(5891)=(S=3063296,E=3063423,C=22)
	CT(5892)=(S=3063424,E=3063551,C=28)
	CT(5893)=(S=3063552,E=3063679,C=68)
	CT(5894)=(S=3063680,E=3063807,C=18)
	CT(5895)=(S=3063808,E=3064319,C=12)
	CT(5896)=(S=3064320,E=3064831,C=19)
	CT(5897)=(S=3064832,E=3065343,C=37)
	CT(5898)=(S=3065344,E=3065855,C=32)
	CT(5899)=(S=3065856,E=3066879,C=31)
	CT(5900)=(S=3066880,E=3067391,C=58)
	CT(5901)=(S=3067392,E=3067903,C=42)
	CT(5902)=(S=3067904,E=3068159,C=14)
	CT(5903)=(S=3068160,E=3068415,C=26)
	CT(5904)=(S=3068416,E=3068671,C=17)
	CT(5905)=(S=3068672,E=3068927,C=31)
	CT(5906)=(S=3068928,E=3069183,C=27)
	CT(5907)=(S=3069184,E=3069439,C=30)
	CT(5908)=(S=3069440,E=3069695,C=38)
	CT(5909)=(S=3069696,E=3069951,C=58)
	CT(5910)=(S=3069952,E=3070207,C=68)
	CT(5911)=(S=3070208,E=3070463,C=54)
	CT(5912)=(S=3070464,E=3070719,C=12)
	CT(5913)=(S=3070720,E=3070975,C=31)
	CT(5914)=(S=3070976,E=3071231,C=42)
	CT(5915)=(S=3071232,E=3071487,C=37)
	CT(5916)=(S=3071488,E=3071743,C=25)
	CT(5917)=(S=3071744,E=3071999,C=28)
	CT(5918)=(S=3072000,E=3072511,C=26)
	CT(5919)=(S=3072512,E=3072519,C=14)
	CT(5920)=(S=3072520,E=3072527,C=27)
	CT(5921)=(S=3072528,E=3072535,C=33)
	CT(5922)=(S=3072536,E=3072543,C=27)
	CT(5923)=(S=3072544,E=3072559,C=25)
	CT(5924)=(S=3072560,E=3072567,C=14)
	CT(5925)=(S=3072568,E=3072575,C=36)
	CT(5926)=(S=3072576,E=3072583,C=18)
	CT(5927)=(S=3072584,E=3072591,C=14)
	CT(5928)=(S=3072592,E=3072599,C=28)
	CT(5929)=(S=3072600,E=3072607,C=36)
	CT(5930)=(S=3072608,E=3072615,C=18)
	CT(5931)=(S=3072616,E=3072623,C=12)
	CT(5932)=(S=3072624,E=3072631,C=28)
	CT(5933)=(S=3072632,E=3072639,C=14)
	CT(5934)=(S=3072640,E=3072647,C=12)
	CT(5935)=(S=3072648,E=3072655,C=55)
	CT(5936)=(S=3072656,E=3072663,C=25)
	CT(5937)=(S=3072664,E=3072671,C=19)
	CT(5938)=(S=3072672,E=3072679,C=18)
	CT(5939)=(S=3072680,E=3072687,C=14)
	CT(5940)=(S=3072688,E=3072695,C=15)
	CT(5941)=(S=3072696,E=3072703,C=14)
	CT(5942)=(S=3072704,E=3072711,C=22)
	CT(5943)=(S=3072712,E=3072719,C=15)
	CT(5944)=(S=3072720,E=3072727,C=62)
	CT(5945)=(S=3072728,E=3072735,C=14)
	CT(5946)=(S=3072736,E=3072743,C=18)
	CT(5947)=(S=3072744,E=3072751,C=36)
	CT(5948)=(S=3072752,E=3072759,C=18)
	CT(5949)=(S=3072760,E=3072767,C=14)
	CT(5950)=(S=3072768,E=3072775,C=15)
	CT(5951)=(S=3072776,E=3072783,C=33)
	CT(5952)=(S=3072784,E=3072791,C=12)
	CT(5953)=(S=3072792,E=3072799,C=18)
	CT(5954)=(S=3072800,E=3072807,C=58)
	CT(5955)=(S=3072808,E=3072823,C=14)
	CT(5956)=(S=3072824,E=3072831,C=22)
	CT(5957)=(S=3072832,E=3072847,C=19)
	CT(5958)=(S=3072848,E=3072855,C=12)
	CT(5959)=(S=3072856,E=3072871,C=28)
	CT(5960)=(S=3072872,E=3072879,C=58)
	CT(5961)=(S=3072880,E=3072887,C=171)
	CT(5962)=(S=3072888,E=3072895,C=98)
	CT(5963)=(S=3072896,E=3072903,C=28)
	CT(5964)=(S=3072904,E=3072911,C=31)
	CT(5965)=(S=3072912,E=3072919,C=14)
	CT(5966)=(S=3072920,E=3072927,C=15)
	CT(5967)=(S=3072928,E=3072935,C=18)
	CT(5968)=(S=3072936,E=3072943,C=33)
	CT(5969)=(S=3072944,E=3072951,C=71)
	CT(5970)=(S=3072952,E=3072959,C=20)
	CT(5971)=(S=3072960,E=3072967,C=27)
	CT(5972)=(S=3072968,E=3072975,C=14)
	CT(5973)=(S=3072976,E=3072983,C=18)
	CT(5974)=(S=3072984,E=3072991,C=28)
	CT(5975)=(S=3072992,E=3072999,C=34)
	CT(5976)=(S=3073000,E=3073007,C=36)
	CT(5977)=(S=3073008,E=3073015,C=58)
	CT(5978)=(S=3073016,E=3073023,C=39)
	CT(5979)=(S=3073024,E=3073039,C=18)
	CT(5980)=(S=3073040,E=3073055,C=33)
	CT(5981)=(S=3073056,E=3073071,C=14)
	CT(5982)=(S=3073072,E=3073087,C=27)
	CT(5983)=(S=3073088,E=3073103,C=18)
	CT(5984)=(S=3073104,E=3073119,C=58)
	CT(5985)=(S=3073120,E=3073151,C=18)
	CT(5986)=(S=3073152,E=3073167,C=47)
	CT(5987)=(S=3073168,E=3073183,C=17)
	CT(5988)=(S=3073184,E=3073199,C=14)
	CT(5989)=(S=3073200,E=3073215,C=41)
	CT(5990)=(S=3073216,E=3073231,C=28)
	CT(5991)=(S=3073232,E=3073247,C=71)
	CT(5992)=(S=3073248,E=3073263,C=58)
	CT(5993)=(S=3073264,E=3073279,C=15)
	CT(5994)=(S=3073280,E=3073327,C=28)
	CT(5995)=(S=3073328,E=3073343,C=31)
	CT(5996)=(S=3073344,E=3073359,C=18)
	CT(5997)=(S=3073360,E=3073375,C=15)
	CT(5998)=(S=3073376,E=3073391,C=18)
	CT(5999)=(S=3073392,E=3073407,C=33)
	CT(6000)=(S=3073408,E=3073423,C=18)
	CT(6001)=(S=3073424,E=3073439,C=58)
	CT(6002)=(S=3073440,E=3073455,C=36)
	CT(6003)=(S=3073456,E=3073471,C=18)
	CT(6004)=(S=3073472,E=3073487,C=51)
	CT(6005)=(S=3073488,E=3073503,C=18)
	CT(6006)=(S=3073504,E=3073519,C=71)
	CT(6007)=(S=3073520,E=3073535,C=57)
	CT(6008)=(S=3073536,E=3073663,C=23)
	CT(6009)=(S=3073664,E=3073791,C=19)
	CT(6010)=(S=3073792,E=3073799,C=25)
	CT(6011)=(S=3073800,E=3073807,C=14)
	CT(6012)=(S=3073808,E=3073815,C=36)
	CT(6013)=(S=3073816,E=3073831,C=15)
	CT(6014)=(S=3073832,E=3073839,C=36)
	CT(6015)=(S=3073848,E=3073855,C=58)
	CT(6016)=(S=3073856,E=3073863,C=14)
	CT(6017)=(S=3073864,E=3073871,C=33)
	CT(6018)=(S=3073872,E=3073879,C=36)
	CT(6019)=(S=3073880,E=3073887,C=28)
	CT(6020)=(S=3073888,E=3073895,C=71)
	CT(6021)=(S=3073896,E=3073903,C=38)
	CT(6022)=(S=3073904,E=3073911,C=14)
	CT(6023)=(S=3073912,E=3073919,C=25)
	CT(6024)=(S=3073920,E=3073927,C=12)
	CT(6025)=(S=3073928,E=3073935,C=28)
	CT(6026)=(S=3073936,E=3073943,C=12)
	CT(6027)=(S=3073944,E=3073951,C=58)
	CT(6028)=(S=3073952,E=3073959,C=14)
	CT(6029)=(S=3073960,E=3073967,C=18)
	CT(6030)=(S=3073968,E=3073975,C=28)
	CT(6031)=(S=3073976,E=3073983,C=14)
	CT(6032)=(S=3073984,E=3073991,C=47)
	CT(6033)=(S=3073992,E=3073999,C=34)
	CT(6034)=(S=3074000,E=3074007,C=18)
	CT(6035)=(S=3074008,E=3074015,C=12)
	CT(6036)=(S=3074016,E=3074023,C=31)
	CT(6037)=(S=3074024,E=3074031,C=28)
	CT(6038)=(S=3074032,E=3074039,C=29)
	CT(6039)=(S=3074040,E=3074047,C=36)
	CT(6040)=(S=3074048,E=3074175,C=18)
	CT(6041)=(S=3074176,E=3074183,C=58)
	CT(6042)=(S=3074184,E=3074199,C=15)
	CT(6043)=(S=3074200,E=3074207,C=51)
	CT(6044)=(S=3074208,E=3074215,C=18)
	CT(6045)=(S=3074216,E=3074223,C=25)
	CT(6046)=(S=3074224,E=3074231,C=34)
	CT(6047)=(S=3074232,E=3074239,C=28)
	CT(6048)=(S=3074240,E=3074247,C=14)
	CT(6049)=(S=3074248,E=3074271,C=18)
	CT(6050)=(S=3074272,E=3074279,C=28)
	CT(6051)=(S=3074280,E=3074287,C=31)
	CT(6052)=(S=3074288,E=3074295,C=18)
	CT(6053)=(S=3074304,E=3074367,C=51)
	CT(6054)=(S=3074368,E=3074431,C=14)
	CT(6055)=(S=3074432,E=3074495,C=15)
	CT(6056)=(S=3074496,E=3074559,C=18)
	CT(6057)=(S=3074560,E=3074591,C=37)
	CT(6058)=(S=3074592,E=3074623,C=13)
	CT(6059)=(S=3074624,E=3074655,C=57)
	CT(6060)=(S=3074656,E=3074687,C=33)
	CT(6061)=(S=3074688,E=3074719,C=25)
	CT(6062)=(S=3074720,E=3074751,C=33)
	CT(6063)=(S=3074752,E=3074815,C=15)
	CT(6064)=(S=3074816,E=3074823,C=14)
	CT(6065)=(S=3074824,E=3074831,C=37)
	CT(6066)=(S=3074832,E=3074839,C=12)
	CT(6067)=(S=3074840,E=3074847,C=28)
	CT(6068)=(S=3074848,E=3074855,C=19)
	CT(6069)=(S=3074856,E=3074863,C=36)
	CT(6070)=(S=3074864,E=3074871,C=18)
	CT(6071)=(S=3074872,E=3074879,C=34)
	CT(6072)=(S=3074880,E=3074887,C=18)
	CT(6073)=(S=3074888,E=3074895,C=62)
	CT(6074)=(S=3074896,E=3074903,C=20)
	CT(6075)=(S=3074904,E=3074911,C=23)
	CT(6076)=(S=3074912,E=3074919,C=46)
	CT(6077)=(S=3074920,E=3074935,C=28)
	CT(6078)=(S=3074936,E=3074943,C=58)
	CT(6079)=(S=3074944,E=3074951,C=172)
	CT(6080)=(S=3074952,E=3074959,C=15)
	CT(6081)=(S=3074960,E=3074967,C=34)
	CT(6082)=(S=3074968,E=3074975,C=13)
	CT(6083)=(S=3074976,E=3074983,C=18)
	CT(6084)=(S=3074984,E=3074991,C=14)
	CT(6085)=(S=3074992,E=3074999,C=12)
	CT(6086)=(S=3075000,E=3075007,C=18)
	CT(6087)=(S=3075008,E=3075023,C=28)
	CT(6088)=(S=3075024,E=3075031,C=104)
	CT(6089)=(S=3075032,E=3075039,C=18)
	CT(6090)=(S=3075040,E=3075047,C=14)
	CT(6091)=(S=3075048,E=3075055,C=19)
	CT(6092)=(S=3075056,E=3075063,C=28)
	CT(6093)=(S=3075064,E=3075071,C=18)
	CT(6094)=(S=3075072,E=3075135,C=14)
	CT(6095)=(S=3075136,E=3075199,C=19)
	CT(6096)=(S=3075200,E=3075263,C=18)
	CT(6097)=(S=3075264,E=3075327,C=28)
	CT(6098)=(S=3075328,E=3075391,C=18)
	CT(6099)=(S=3075392,E=3075455,C=51)
	CT(6100)=(S=3075456,E=3075519,C=14)
	CT(6101)=(S=3075520,E=3075583,C=28)
	CT(6102)=(S=3075584,E=3075647,C=51)
	CT(6103)=(S=3075648,E=3075711,C=58)
	CT(6104)=(S=3075712,E=3075775,C=12)
	CT(6105)=(S=3075776,E=3075839,C=58)
	CT(6106)=(S=3075840,E=3075903,C=59)
	CT(6107)=(S=3075904,E=3075967,C=19)
	CT(6108)=(S=3075968,E=3076031,C=58)
	CT(6109)=(S=3076032,E=3076095,C=48)
	CT(6110)=(S=3076096,E=3076223,C=23)
	CT(6111)=(S=3076224,E=3076351,C=46)
	CT(6112)=(S=3076352,E=3076479,C=18)
	CT(6113)=(S=3076480,E=3076607,C=66)
	CT(6114)=(S=3076608,E=3076735,C=18)
	CT(6115)=(S=3076736,E=3076863,C=58)
	CT(6116)=(S=3076864,E=3076871,C=33)
	CT(6117)=(S=3076872,E=3076879,C=18)
	CT(6118)=(S=3076880,E=3076887,C=35)
	CT(6119)=(S=3076888,E=3076895,C=36)
	CT(6120)=(S=3076896,E=3076903,C=15)
	CT(6121)=(S=3076904,E=3076911,C=36)
	CT(6122)=(S=3076912,E=3076919,C=33)
	CT(6123)=(S=3076928,E=3076935,C=71)
	CT(6124)=(S=3076936,E=3076943,C=18)
	CT(6125)=(S=3076944,E=3076959,C=28)
	CT(6126)=(S=3076960,E=3076967,C=36)
	CT(6127)=(S=3076968,E=3076975,C=42)
	CT(6128)=(S=3076976,E=3076983,C=38)
	CT(6129)=(S=3076984,E=3076991,C=28)
	CT(6130)=(S=3076992,E=3077023,C=36)
	CT(6131)=(S=3077024,E=3077119,C=18)
	CT(6132)=(S=3077120,E=3077151,C=36)
	CT(6133)=(S=3077152,E=3077167,C=14)
	CT(6134)=(S=3077168,E=3077183,C=36)
	CT(6135)=(S=3077184,E=3077215,C=43)
	CT(6136)=(S=3077216,E=3077247,C=36)
	CT(6137)=(S=3077248,E=3077375,C=28)
	CT(6138)=(S=3077376,E=3077503,C=26)
	CT(6139)=(S=3077504,E=3077511,C=18)
	CT(6140)=(S=3077512,E=3077519,C=22)
	CT(6141)=(S=3077520,E=3077527,C=34)
	CT(6142)=(S=3077528,E=3077535,C=19)
	CT(6143)=(S=3077536,E=3077543,C=37)
	CT(6144)=(S=3077544,E=3077551,C=18)
	CT(6145)=(S=3077552,E=3077559,C=28)
	CT(6146)=(S=3077560,E=3077567,C=34)
	CT(6147)=(S=3077568,E=3077575,C=58)
	CT(6148)=(S=3077576,E=3077583,C=42)
	CT(6149)=(S=3077584,E=3077591,C=63)
	CT(6150)=(S=3077592,E=3077599,C=28)
	CT(6151)=(S=3077600,E=3077615,C=12)
	CT(6152)=(S=3077616,E=3077631,C=14)
	CT(6153)=(S=3077632,E=3077759,C=19)
	CT(6154)=(S=3077760,E=3077887,C=22)
	CT(6155)=(S=3077888,E=3078015,C=14)
	CT(6156)=(S=3078016,E=3078143,C=31)
	CT(6157)=(S=3078144,E=3078175,C=14)
	CT(6158)=(S=3078176,E=3078207,C=26)
	CT(6159)=(S=3078208,E=3078239,C=52)
	CT(6160)=(S=3078240,E=3078303,C=14)
	CT(6161)=(S=3078304,E=3078335,C=58)
	CT(6162)=(S=3078336,E=3078367,C=61)
	CT(6163)=(S=3078368,E=3078399,C=14)
	CT(6164)=(S=3078400,E=3078431,C=18)
	CT(6165)=(S=3078432,E=3078463,C=36)
	CT(6166)=(S=3078464,E=3078495,C=51)
	CT(6167)=(S=3078496,E=3078527,C=26)
	CT(6168)=(S=3078528,E=3078559,C=50)
	CT(6169)=(S=3078560,E=3078591,C=44)
	CT(6170)=(S=3078592,E=3078623,C=14)
	CT(6171)=(S=3078624,E=3078655,C=27)
	CT(6172)=(S=3078656,E=3078687,C=31)
	CT(6173)=(S=3078688,E=3078751,C=18)
	CT(6174)=(S=3078752,E=3078783,C=31)
	CT(6175)=(S=3078784,E=3078815,C=18)
	CT(6176)=(S=3078816,E=3078847,C=58)
	CT(6177)=(S=3078848,E=3078879,C=27)
	CT(6178)=(S=3078880,E=3078911,C=14)
	CT(6179)=(S=3078912,E=3078943,C=50)
	CT(6180)=(S=3078944,E=3078975,C=44)
	CT(6181)=(S=3078976,E=3079007,C=18)
	CT(6182)=(S=3079008,E=3079039,C=32)
	CT(6183)=(S=3079040,E=3079071,C=23)
	CT(6184)=(S=3079072,E=3079103,C=26)
	CT(6185)=(S=3079104,E=3079135,C=62)
	CT(6186)=(S=3079136,E=3079167,C=28)
	CT(6187)=(S=3079168,E=3079183,C=19)
	CT(6188)=(S=3079184,E=3079199,C=28)
	CT(6189)=(S=3079200,E=3079215,C=96)
	CT(6190)=(S=3079216,E=3079231,C=51)
	CT(6191)=(S=3079232,E=3079263,C=14)
	CT(6192)=(S=3079264,E=3079279,C=37)
	CT(6193)=(S=3079280,E=3079295,C=18)
	CT(6194)=(S=3079296,E=3079311,C=28)
	CT(6195)=(S=3079312,E=3079327,C=15)
	CT(6196)=(S=3079328,E=3079343,C=18)
	CT(6197)=(S=3079344,E=3079359,C=12)
	CT(6198)=(S=3079360,E=3079375,C=36)
	CT(6199)=(S=3079376,E=3079391,C=31)
	CT(6200)=(S=3079392,E=3079407,C=33)
	CT(6201)=(S=3079408,E=3079423,C=18)
	CT(6202)=(S=3079424,E=3079439,C=51)
	CT(6203)=(S=3079440,E=3079455,C=28)
	CT(6204)=(S=3079456,E=3079471,C=25)
	CT(6205)=(S=3079472,E=3079503,C=28)
	CT(6206)=(S=3079504,E=3079519,C=43)
	CT(6207)=(S=3079520,E=3079535,C=33)
	CT(6208)=(S=3079536,E=3079551,C=28)
	CT(6209)=(S=3079552,E=3079567,C=35)
	CT(6210)=(S=3079568,E=3079583,C=28)
	CT(6211)=(S=3079584,E=3079599,C=49)
	CT(6212)=(S=3079600,E=3079615,C=34)
	CT(6213)=(S=3079616,E=3079631,C=19)
	CT(6214)=(S=3079632,E=3079647,C=58)
	CT(6215)=(S=3079648,E=3079663,C=25)
	CT(6216)=(S=3079664,E=3079679,C=28)
	CT(6217)=(S=3079680,E=3079687,C=52)
	CT(6218)=(S=3079688,E=3079695,C=19)
	CT(6219)=(S=3079696,E=3079711,C=18)
	CT(6220)=(S=3079712,E=3079727,C=15)
	CT(6221)=(S=3079728,E=3079735,C=37)
	CT(6222)=(S=3079736,E=3079743,C=52)
	CT(6223)=(S=3079744,E=3079751,C=33)
	CT(6224)=(S=3079752,E=3079759,C=58)
	CT(6225)=(S=3079760,E=3079775,C=18)
	CT(6226)=(S=3079776,E=3079783,C=47)
	CT(6227)=(S=3079784,E=3079791,C=15)
	CT(6228)=(S=3079792,E=3079799,C=14)
	CT(6229)=(S=3079800,E=3079807,C=28)
	CT(6230)=(S=3079808,E=3079815,C=51)
	CT(6231)=(S=3079816,E=3079823,C=28)
	CT(6232)=(S=3079824,E=3079831,C=52)
	CT(6233)=(S=3079832,E=3079839,C=15)
	CT(6234)=(S=3079840,E=3079847,C=18)
	CT(6235)=(S=3079848,E=3079855,C=36)
	CT(6236)=(S=3079856,E=3079863,C=15)
	CT(6237)=(S=3079864,E=3079887,C=14)
	CT(6238)=(S=3079888,E=3079895,C=47)
	CT(6239)=(S=3079896,E=3079903,C=18)
	CT(6240)=(S=3079904,E=3079911,C=12)
	CT(6241)=(S=3079912,E=3079919,C=15)
	CT(6242)=(S=3079920,E=3079927,C=18)
	CT(6243)=(S=3079928,E=3079943,C=14)
	CT(6244)=(S=3079944,E=3079951,C=33)
	CT(6245)=(S=3079952,E=3079959,C=27)
	CT(6246)=(S=3079960,E=3079967,C=36)
	CT(6247)=(S=3079968,E=3079975,C=31)
	CT(6248)=(S=3079976,E=3079983,C=14)
	CT(6249)=(S=3079984,E=3079991,C=12)
	CT(6250)=(S=3079992,E=3079999,C=45)
	CT(6251)=(S=3080000,E=3080007,C=98)
	CT(6252)=(S=3080008,E=3080015,C=14)
	CT(6253)=(S=3080016,E=3080023,C=15)
	CT(6254)=(S=3080024,E=3080031,C=36)
	CT(6255)=(S=3080032,E=3080039,C=18)
	CT(6256)=(S=3080040,E=3080047,C=36)
	CT(6257)=(S=3080048,E=3080055,C=14)
	CT(6258)=(S=3080056,E=3080063,C=19)
	CT(6259)=(S=3080064,E=3080071,C=12)
	CT(6260)=(S=3080072,E=3080079,C=18)
	CT(6261)=(S=3080080,E=3080087,C=96)
	CT(6262)=(S=3080088,E=3080095,C=63)
	CT(6263)=(S=3080096,E=3080103,C=12)
	CT(6264)=(S=3080104,E=3080111,C=34)
	CT(6265)=(S=3080112,E=3080119,C=12)
	CT(6266)=(S=3080120,E=3080127,C=25)
	CT(6267)=(S=3080128,E=3080135,C=47)
	CT(6268)=(S=3080136,E=3080143,C=12)
	CT(6269)=(S=3080144,E=3080151,C=44)
	CT(6270)=(S=3080152,E=3080159,C=26)
	CT(6271)=(S=3080160,E=3080167,C=33)
	CT(6272)=(S=3080168,E=3080175,C=20)
	CT(6273)=(S=3080176,E=3080183,C=15)
	CT(6274)=(S=3080184,E=3080191,C=14)
	CT(6275)=(S=3080192,E=3084287,C=78)
	CT(6276)=(S=3084288,E=3086335,C=29)
	CT(6277)=(S=3086336,E=3088383,C=78)
	CT(6278)=(S=3088384,E=3093503,C=29)
	CT(6279)=(S=3093504,E=3093759,C=78)
	CT(6280)=(S=3093760,E=3094015,C=29)
	CT(6281)=(S=3094016,E=3095039,C=78)
	CT(6282)=(S=3095040,E=3099135,C=29)
	CT(6283)=(S=3099136,E=3102719,C=78)
	CT(6284)=(S=3102720,E=3103743,C=29)
	CT(6285)=(S=3103744,E=3112959,C=1)
	CT(6286)=(S=3112960,E=3115007,C=78)
	CT(6287)=(S=3115008,E=3137535,C=29)
	CT(6288)=(S=3137536,E=3145727,C=78)
	CT(6289)=(S=3145728,E=3211263,C=29)
	CT(6290)=(S=3211264,E=3211267,C=74)
	CT(6291)=(S=3211272,E=3211279,C=0)
	CT(6292)=(S=3211280,E=3211287,C=2)
	CT(6293)=(S=3211288,E=3211291,C=74)
	CT(6294)=(S=3211292,E=3211295,C=73)
	CT(6295)=(S=3211296,E=3211327,C=84)
	CT(6296)=(S=3211328,E=3211391,C=3)
	CT(6297)=(S=3211392,E=3211519,C=92)
	CT(6298)=(S=3211520,E=3211775,C=6)
	CT(6299)=(S=3211776,E=3212287,C=0)
	CT(6300)=(S=3212288,E=3213311,C=1)
	CT(6301)=(S=3213312,E=3214335,C=6)
	CT(6302)=(S=3214336,E=3215359,C=4)
	CT(6303)=(S=3215360,E=3219455,C=6)
	CT(6304)=(S=3219456,E=3223551,C=4)
	CT(6305)=(S=3223552,E=3224063,C=3)
	CT(6306)=(S=3224064,E=3224067,C=6)
	CT(6307)=(S=3224068,E=3224075,C=74)
	CT(6308)=(S=3224076,E=3224079,C=5)
	CT(6309)=(S=3224080,E=3224127,C=6)
	CT(6310)=(S=3224128,E=3224191,C=4)
	CT(6311)=(S=3224192,E=3224255,C=6)
	CT(6312)=(S=3224256,E=3224295,C=73)
	CT(6313)=(S=3224296,E=3224299,C=3)
	CT(6314)=(S=3224300,E=3224303,C=5)
	CT(6315)=(S=3224304,E=3224319,C=73)
	CT(6316)=(S=3224320,E=3225599,C=1)
	CT(6317)=(S=3225600,E=3227647,C=6)
	CT(6318)=(S=3227648,E=3235839,C=1)
	CT(6319)=(S=3235840,E=3239935,C=2)
	CT(6320)=(S=3239936,E=3243007,C=1)
	CT(6321)=(S=3243008,E=3243519,C=5)
	CT(6322)=(S=3243520,E=3243775,C=75)
	CT(6323)=(S=3243776,E=3244031,C=0)
	CT(6324)=(S=3244032,E=3244032,C=1)
	CT(6325)=(S=3244033,E=3244033,C=0)
	CT(6326)=(S=3244034,E=3244035,C=1)
	CT(6327)=(S=3244040,E=3244043,C=7)
	CT(6328)=(S=3244044,E=3244047,C=0)
	CT(6329)=(S=3244048,E=3244063,C=2)
	CT(6330)=(S=3244064,E=3244095,C=7)
	CT(6331)=(S=3244096,E=3244127,C=9)
	CT(6332)=(S=3244128,E=3244135,C=2)
	CT(6333)=(S=3244136,E=3244139,C=159)
	CT(6334)=(S=3244140,E=3244143,C=4)
	CT(6335)=(S=3244144,E=3244159,C=9)
	CT(6336)=(S=3244160,E=3244191,C=2)
	CT(6337)=(S=3244192,E=3244207,C=4)
	CT(6338)=(S=3244208,E=3244223,C=74)
	CT(6339)=(S=3244224,E=3244255,C=6)
	CT(6340)=(S=3244256,E=3244287,C=0)
	CT(6341)=(S=3244288,E=3244543,C=2)
	CT(6342)=(S=3244544,E=3245055,C=8)
	CT(6343)=(S=3245056,E=3246079,C=2)
	CT(6344)=(S=3246080,E=3247103,C=4)
	CT(6345)=(S=3247104,E=3247615,C=1)
	CT(6346)=(S=3247616,E=3248095,C=6)
	CT(6347)=(S=3248096,E=3248099,C=0)
	CT(6348)=(S=3248100,E=3248103,C=73)
	CT(6349)=(S=3248104,E=3248107,C=8)
	CT(6350)=(S=3248108,E=3248111,C=0)
	CT(6351)=(S=3248112,E=3248119,C=2)
	CT(6352)=(S=3248120,E=3248123,C=0)
	CT(6353)=(S=3248124,E=3248127,C=4)
	CT(6354)=(S=3248128,E=3250175,C=10)
	CT(6355)=(S=3250176,E=3251199,C=1)
	CT(6356)=(S=3251204,E=3251215,C=2)
	CT(6357)=(S=3251216,E=3251231,C=0)
	CT(6358)=(S=3251232,E=3251247,C=86)
	CT(6359)=(S=3251248,E=3251251,C=173)
	CT(6360)=(S=3251252,E=3251255,C=11)
	CT(6361)=(S=3251256,E=3251263,C=74)
	CT(6362)=(S=3251264,E=3251359,C=4)
	CT(6363)=(S=3251360,E=3251455,C=2)
	CT(6364)=(S=3251456,E=3251711,C=10)
	CT(6365)=(S=3251712,E=3252223,C=9)
	CT(6366)=(S=3252224,E=3256319,C=6)
	CT(6367)=(S=3256320,E=3262463,C=0)
	CT(6368)=(S=3262464,E=3264511,C=4)
	CT(6369)=(S=3264512,E=3265535,C=1)
	CT(6370)=(S=3265536,E=3265791,C=2)
	CT(6371)=(S=3265792,E=3265807,C=8)
	CT(6372)=(S=3265808,E=3265823,C=7)
	CT(6373)=(S=3265824,E=3265855,C=4)
	CT(6374)=(S=3265856,E=3265919,C=11)
	CT(6375)=(S=3265920,E=3267583,C=9)
	CT(6376)=(S=3267584,E=3268607,C=1)
	CT(6377)=(S=3268608,E=3269631,C=73)
	CT(6378)=(S=3269632,E=3270655,C=3)
	CT(6379)=(S=3270656,E=3271679,C=1)
	CT(6380)=(S=3271680,E=3271743,C=4)
	CT(6381)=(S=3271744,E=3271871,C=6)
	CT(6382)=(S=3271872,E=3271887,C=5)
	CT(6383)=(S=3271888,E=3271891,C=11)
	CT(6384)=(S=3271892,E=3271895,C=75)
	CT(6385)=(S=3271896,E=3271903,C=74)
	CT(6386)=(S=3271904,E=3271919,C=2)
	CT(6387)=(S=3271920,E=3271935,C=0)
	CT(6388)=(S=3271936,E=3272191,C=3)
	CT(6389)=(S=3272192,E=3272223,C=2)
	CT(6390)=(S=3272224,E=3272255,C=4)
	CT(6391)=(S=3272256,E=3272447,C=6)
	CT(6392)=(S=3272448,E=3272511,C=1)
	CT(6393)=(S=3272512,E=3272575,C=2)
	CT(6394)=(S=3272576,E=3272639,C=6)
	CT(6395)=(S=3272640,E=3272703,C=1)
	CT(6396)=(S=3272704,E=3273727,C=2)
	CT(6397)=(S=3273728,E=3273983,C=75)
	CT(6398)=(S=3273984,E=3274239,C=7)
	CT(6399)=(S=3274240,E=3274367,C=6)
	CT(6400)=(S=3274368,E=3274463,C=11)
	CT(6401)=(S=3274464,E=3274495,C=1)
	CT(6402)=(S=3274496,E=3274751,C=6)
	CT(6403)=(S=3274752,E=3275263,C=4)
	CT(6404)=(S=3275264,E=3276287,C=2)
	CT(6405)=(S=3276288,E=3276543,C=6)
	CT(6406)=(S=3276544,E=3276799,C=0)
	CT(6407)=(S=3276800,E=3282303,C=29)
	CT(6408)=(S=3282304,E=3282319,C=78)
	CT(6409)=(S=3282320,E=3282335,C=29)
	CT(6410)=(S=3282336,E=3282351,C=78)
	CT(6411)=(S=3282352,E=3282399,C=29)
	CT(6412)=(S=3282400,E=3282415,C=78)
	CT(6413)=(S=3282416,E=3293183,C=29)
	CT(6414)=(S=3293184,E=3295487,C=78)
	CT(6415)=(S=3295488,E=3300351,C=29)
	CT(6416)=(S=3300352,E=3300735,C=78)
	CT(6417)=(S=3300736,E=3301887,C=29)
	CT(6418)=(S=3301888,E=3302911,C=78)
	CT(6419)=(S=3302912,E=3306415,C=29)
	CT(6420)=(S=3306416,E=3306431,C=78)
	CT(6421)=(S=3306432,E=3306879,C=29)
	CT(6422)=(S=3306880,E=3307007,C=78)
	CT(6423)=(S=3307008,E=3342335,C=29)
	CT(6424)=(S=3342336,E=3386111,C=14)
	CT(6425)=(S=3386112,E=3386367,C=63)
	CT(6426)=(S=3386368,E=3386879,C=14)
	CT(6427)=(S=3386880,E=3387391,C=27)
	CT(6428)=(S=3387392,E=3388159,C=14)
	CT(6429)=(S=3388160,E=3388287,C=15)
	CT(6430)=(S=3388288,E=3396351,C=14)
	CT(6431)=(S=3396352,E=3396607,C=23)
	CT(6432)=(S=3396608,E=3398143,C=14)
	CT(6433)=(S=3398144,E=3398399,C=23)
	CT(6434)=(S=3398400,E=3399423,C=14)
	CT(6435)=(S=3399424,E=3399679,C=23)
	CT(6436)=(S=3399680,E=3402495,C=14)
	CT(6437)=(S=3402496,E=3402751,C=23)
	CT(6438)=(S=3402752,E=3406847,C=14)
	CT(6439)=(S=3406848,E=3407359,C=23)
	CT(6440)=(S=3407360,E=3407871,C=12)
	CT(6441)=(S=3407872,E=3473407,C=29)
	CT(6442)=(S=3473408,E=3538943,C=28)
	CT(6443)=(S=3538944,E=3595775,C=29)
	CT(6444)=(S=3595776,E=3596287,C=1)
	CT(6445)=(S=3596288,E=3735551,C=29)
	CT(6446)=(S=3735552,E=3801087,C=49)
	CT(6447)=(S=3801088,E=3801599,C=2)
	CT(6448)=(S=3801600,E=3801855,C=4)
	CT(6449)=(S=3801856,E=3802623,C=2)
	CT(6450)=(S=3802624,E=3803135,C=0)
	CT(6451)=(S=3803136,E=3804159,C=3)
	CT(6452)=(S=3804160,E=3804671,C=2)
	CT(6453)=(S=3804672,E=3807743,C=1)
	CT(6454)=(S=3807744,E=3808127,C=5)
	CT(6455)=(S=3808128,E=3808255,C=77)
	CT(6456)=(S=3808256,E=3808511,C=73)
	CT(6457)=(S=3808512,E=3808767,C=6)
	CT(6458)=(S=3808768,E=3817471,C=1)
	CT(6459)=(S=3817472,E=3817599,C=3)
	CT(6460)=(S=3817600,E=3817727,C=8)
	CT(6461)=(S=3817728,E=3817759,C=7)
	CT(6462)=(S=3817760,E=3817791,C=2)
	CT(6463)=(S=3817792,E=3817855,C=6)
	CT(6464)=(S=3817856,E=3817951,C=77)
	CT(6465)=(S=3817952,E=3817959,C=84)
	CT(6466)=(S=3817960,E=3817967,C=1)
	CT(6467)=(S=3817968,E=3817975,C=74)
	CT(6468)=(S=3817976,E=3817983,C=0)
	CT(6469)=(S=3817984,E=3818495,C=1)
	CT(6470)=(S=3818496,E=3818623,C=4)
	CT(6471)=(S=3818624,E=3818751,C=1)
	CT(6472)=(S=3818752,E=3819007,C=10)
	CT(6473)=(S=3819008,E=3819263,C=2)
	CT(6474)=(S=3819264,E=3819391,C=10)
	CT(6475)=(S=3819392,E=3819519,C=5)
	CT(6476)=(S=3819520,E=3821567,C=6)
	CT(6477)=(S=3821568,E=3822079,C=2)
	CT(6478)=(S=3822080,E=3822207,C=1)
	CT(6479)=(S=3822208,E=3822271,C=3)
	CT(6480)=(S=3822272,E=3822335,C=8)
	CT(6481)=(S=3822336,E=3822591,C=1)
	CT(6482)=(S=3822656,E=3822815,C=0)
	CT(6483)=(S=3822816,E=3822831,C=73)
	CT(6484)=(S=3822832,E=3823103,C=2)
	CT(6485)=(S=3823104,E=3823359,C=9)
	CT(6486)=(S=3823360,E=3823375,C=0)
	CT(6487)=(S=3823376,E=3823391,C=2)
	CT(6488)=(S=3823392,E=3823423,C=6)
	CT(6489)=(S=3823424,E=3823487,C=1)
	CT(6490)=(S=3823488,E=3825663,C=2)
	CT(6491)=(S=3825664,E=3825823,C=0)
	CT(6492)=(S=3825824,E=3825855,C=8)
	CT(6493)=(S=3825856,E=3825919,C=0)
	CT(6494)=(S=3825920,E=3826047,C=3)
	CT(6495)=(S=3826048,E=3826175,C=84)
	CT(6496)=(S=3826176,E=3826431,C=2)
	CT(6497)=(S=3826432,E=3826559,C=9)
	CT(6498)=(S=3826560,E=3827199,C=1)
	CT(6499)=(S=3827200,E=3827711,C=6)
	CT(6500)=(S=3827712,E=3829759,C=0)
	CT(6501)=(S=3829760,E=3830271,C=2)
	CT(6502)=(S=3830272,E=3830783,C=9)
	CT(6503)=(S=3830784,E=3831807,C=1)
	CT(6504)=(S=3831808,E=3833855,C=6)
	CT(6505)=(S=3833856,E=3835903,C=1)
	CT(6506)=(S=3835904,E=3836415,C=3)
	CT(6507)=(S=3836416,E=3836607,C=2)
	CT(6508)=(S=3836608,E=3836671,C=6)
	CT(6509)=(S=3836672,E=3836927,C=5)
	CT(6510)=(S=3836928,E=3837951,C=6)
	CT(6511)=(S=3837952,E=3838207,C=1)
	CT(6512)=(S=3838208,E=3838335,C=6)
	CT(6513)=(S=3838336,E=3838367,C=0)
	CT(6514)=(S=3838368,E=3838375,C=2)
	CT(6515)=(S=3838376,E=3838383,C=74)
	CT(6516)=(S=3838384,E=3838391,C=2)
	CT(6517)=(S=3838392,E=3838399,C=84)
	CT(6518)=(S=3838400,E=3838559,C=2)
	CT(6519)=(S=3838560,E=3838591,C=4)
	CT(6520)=(S=3838592,E=3838655,C=7)
	CT(6521)=(S=3838656,E=3838719,C=6)
	CT(6522)=(S=3838720,E=3838847,C=3)
	CT(6523)=(S=3838848,E=3838879,C=89)
	CT(6524)=(S=3838880,E=3838887,C=2)
	CT(6525)=(S=3838888,E=3838895,C=84)
	CT(6526)=(S=3838896,E=3838903,C=6)
	CT(6527)=(S=3838904,E=3838911,C=74)
	CT(6528)=(S=3838912,E=3838975,C=2)
	CT(6529)=(S=3838976,E=3839999,C=6)
	CT(6530)=(S=3840000,E=3840511,C=8)
	CT(6531)=(S=3840512,E=3841023,C=1)
	CT(6532)=(S=3841024,E=3842047,C=2)
	CT(6533)=(S=3842048,E=3846143,C=0)
	CT(6534)=(S=3846144,E=3846655,C=8)
	CT(6535)=(S=3846656,E=3847167,C=0)
	CT(6536)=(S=3847168,E=3847487,C=6)
	CT(6537)=(S=3847488,E=3847519,C=0)
	CT(6538)=(S=3847520,E=3847551,C=77)
	CT(6539)=(S=3847552,E=3847679,C=3)
	CT(6540)=(S=3847680,E=3847935,C=7)
	CT(6541)=(S=3847936,E=3848191,C=2)
	CT(6542)=(S=3848192,E=3848447,C=6)
	CT(6543)=(S=3848448,E=3848703,C=7)
	CT(6544)=(S=3848704,E=3849215,C=11)
	CT(6545)=(S=3849216,E=3850239,C=2)
	CT(6546)=(S=3850240,E=3858431,C=1)
	CT(6547)=(S=3858432,E=3862527,C=6)
	CT(6548)=(S=3862528,E=3866623,C=1)
	CT(6549)=(S=3866624,E=3874815,C=6)
	CT(6550)=(S=3874816,E=3888127,C=1)
	CT(6551)=(S=3888128,E=3888799,C=2)
	CT(6552)=(S=3888800,E=3888831,C=0)
	CT(6553)=(S=3888832,E=3888895,C=6)
	CT(6554)=(S=3888896,E=3889151,C=2)
	CT(6555)=(S=3889152,E=3892223,C=4)
	CT(6556)=(S=3892224,E=3892863,C=0)
	CT(6557)=(S=3892864,E=3892991,C=9)
	CT(6558)=(S=3892992,E=3893247,C=77)
	CT(6559)=(S=3893248,E=3893759,C=9)
	CT(6560)=(S=3893760,E=3894015,C=2)
	CT(6561)=(S=3894016,E=3895295,C=1)
	CT(6562)=(S=3895296,E=3899391,C=9)
	CT(6563)=(S=3899392,E=3903487,C=2)
	CT(6564)=(S=3903488,E=3903999,C=4)
	CT(6565)=(S=3904000,E=3904511,C=2)
	CT(6566)=(S=3904512,E=3905023,C=8)
	CT(6567)=(S=3905024,E=3905279,C=6)
	CT(6568)=(S=3905280,E=3905407,C=1)
	CT(6569)=(S=3905408,E=3905471,C=2)
	CT(6570)=(S=3905472,E=3905535,C=6)
	CT(6571)=(S=3905536,E=3905647,C=84)
	CT(6572)=(S=3905664,E=3905727,C=6)
	CT(6573)=(S=3905728,E=3905791,C=8)
	CT(6574)=(S=3906048,E=3906303,C=0)
	CT(6575)=(S=3906304,E=3906559,C=1)
	CT(6576)=(S=3906560,E=3907583,C=2)
	CT(6577)=(S=3907584,E=3909119,C=4)
	CT(6578)=(S=3909120,E=3909375,C=2)
	CT(6579)=(S=3909376,E=3909631,C=0)
	CT(6580)=(S=3909632,E=3910655,C=2)
	CT(6581)=(S=3910656,E=3911679,C=1)
	CT(6582)=(S=3911680,E=3914239,C=4)
	CT(6583)=(S=3914240,E=3914751,C=6)
	CT(6584)=(S=3914752,E=3915007,C=8)
	CT(6585)=(S=3915008,E=3915263,C=7)
	CT(6586)=(S=3915264,E=3915519,C=2)
	CT(6587)=(S=3915520,E=3915647,C=1)
	CT(6588)=(S=3915648,E=3915711,C=2)
	CT(6589)=(S=3915712,E=3915759,C=0)
	CT(6590)=(S=3915760,E=3940351,C=1)
	CT(6591)=(S=3940352,E=3944447,C=2)
	CT(6592)=(S=3944448,E=3946239,C=5)
	CT(6593)=(S=3946240,E=3946495,C=1)
	CT(6594)=(S=3946496,E=3948287,C=2)
	CT(6595)=(S=3948288,E=3948543,C=1)
	CT(6596)=(S=3948544,E=3973119,C=2)
	CT(6597)=(S=3973120,E=3981311,C=1)
	CT(6598)=(S=3981312,E=3981823,C=2)
	CT(6599)=(S=3981824,E=3982335,C=1)
	CT(6600)=(S=3982336,E=3982847,C=6)
	CT(6601)=(S=3982848,E=3983359,C=9)
	CT(6602)=(S=3983360,E=3989503,C=1)
	CT(6603)=(S=3989504,E=3991551,C=0)
	CT(6604)=(S=3991552,E=3992063,C=1)
	CT(6605)=(S=3992064,E=3992319,C=73)
	CT(6606)=(S=3992320,E=3992575,C=1)
	CT(6607)=(S=3992576,E=3993599,C=2)
	CT(6608)=(S=3993600,E=3994367,C=0)
	CT(6609)=(S=3994368,E=3994623,C=4)
	CT(6610)=(S=3994624,E=3995007,C=9)
	CT(6611)=(S=3995008,E=3995135,C=1)
	CT(6612)=(S=3995136,E=3995391,C=87)
	CT(6613)=(S=3995392,E=3995647,C=1)
	CT(6614)=(S=3995648,E=3996671,C=9)
	CT(6615)=(S=3996672,E=3996927,C=1)
	CT(6616)=(S=3996928,E=3997023,C=6)
	CT(6617)=(S=3997024,E=3997055,C=74)
	CT(6618)=(S=3997056,E=3997183,C=1)
	CT(6619)=(S=3997184,E=3997311,C=4)
	CT(6620)=(S=3997312,E=3997375,C=29)
	CT(6621)=(S=3997376,E=3997439,C=2)
	CT(6622)=(S=3997440,E=3997695,C=1)
	CT(6623)=(S=3997696,E=3998719,C=4)
	CT(6624)=(S=3998720,E=3998783,C=8)
	CT(6625)=(S=3998784,E=3998799,C=155)
	CT(6626)=(S=3998800,E=3998815,C=1)
	CT(6627)=(S=3998816,E=3998847,C=5)
	CT(6628)=(S=3998848,E=3998895,C=2)
	CT(6629)=(S=3998896,E=3998911,C=1)
	CT(6630)=(S=3998912,E=3998975,C=6)
	CT(6631)=(S=3998976,E=3999103,C=74)
	CT(6632)=(S=3999104,E=3999135,C=77)
	CT(6633)=(S=3999136,E=3999167,C=6)
	CT(6634)=(S=3999168,E=3999183,C=89)
	CT(6635)=(S=3999184,E=3999199,C=90)
	CT(6636)=(S=3999200,E=3999231,C=2)
	CT(6637)=(S=3999232,E=3999423,C=5)
	CT(6638)=(S=3999424,E=3999487,C=157)
	CT(6639)=(S=3999488,E=3999615,C=2)
	CT(6640)=(S=3999616,E=3999743,C=3)
	CT(6641)=(S=3999744,E=3999807,C=2)
	CT(6642)=(S=3999808,E=3999823,C=74)
	CT(6643)=(S=3999824,E=3999839,C=2)
	CT(6644)=(S=3999840,E=3999871,C=0)
	CT(6645)=(S=3999872,E=3999903,C=4)
	CT(6646)=(S=3999904,E=3999919,C=1)
	CT(6647)=(S=3999920,E=3999935,C=0)
	CT(6648)=(S=3999936,E=3999999,C=7)
	CT(6649)=(S=4000000,E=4000127,C=10)
	CT(6650)=(S=4000128,E=4000255,C=0)
	CT(6651)=(S=4000256,E=4000511,C=8)
	CT(6652)=(S=4000512,E=4000639,C=4)
	CT(6653)=(S=4000640,E=4000719,C=2)
	CT(6654)=(S=4000720,E=4000735,C=5)
	CT(6655)=(S=4000736,E=4000767,C=11)
	CT(6656)=(S=4000768,E=4000895,C=4)
	CT(6657)=(S=4000896,E=4001023,C=2)
	CT(6658)=(S=4001024,E=4001279,C=7)
	CT(6659)=(S=4001280,E=4001343,C=0)
	CT(6660)=(S=4001344,E=4001375,C=7)
	CT(6661)=(S=4001376,E=4001407,C=0)
	CT(6662)=(S=4001408,E=4001471,C=2)
	CT(6663)=(S=4001472,E=4001479,C=10)
	CT(6664)=(S=4001536,E=4001791,C=8)
	CT(6665)=(S=4001792,E=4001919,C=7)
	CT(6666)=(S=4001920,E=4002303,C=4)
	CT(6667)=(S=4002304,E=4002559,C=8)
	CT(6668)=(S=4002560,E=4002815,C=3)
	CT(6669)=(S=4002816,E=4003071,C=9)
	CT(6670)=(S=4003072,E=4004863,C=2)
	CT(6671)=(S=4004864,E=4004991,C=1)
	CT(6672)=(S=4004992,E=4005055,C=10)
	CT(6673)=(S=4005056,E=4005087,C=0)
	CT(6674)=(S=4005088,E=4005119,C=11)
	CT(6675)=(S=4005120,E=4005247,C=0)
	CT(6676)=(S=4005248,E=4005375,C=1)
	CT(6677)=(S=4005376,E=4005887,C=9)
	CT(6678)=(S=4005888,E=4008959,C=6)
	CT(6679)=(S=4008960,E=4009343,C=2)
	CT(6680)=(S=4009344,E=4009407,C=1)
	CT(6681)=(S=4009408,E=4009439,C=2)
	CT(6682)=(S=4009440,E=4009455,C=1)
	CT(6683)=(S=4009456,E=4009463,C=2)
	CT(6684)=(S=4009464,E=4009471,C=0)
	CT(6685)=(S=4009472,E=4009855,C=2)
	CT(6686)=(S=4009856,E=4009919,C=1)
	CT(6687)=(S=4009920,E=4009983,C=6)
	CT(6688)=(S=4009984,E=4012031,C=1)
	CT(6689)=(S=4012032,E=4015103,C=9)
	CT(6690)=(S=4015104,E=4015615,C=0)
	CT(6691)=(S=4015616,E=4016127,C=9)
	CT(6692)=(S=4016128,E=4019711,C=6)
	CT(6693)=(S=4019712,E=4020159,C=2)
	CT(6694)=(S=4020160,E=4020223,C=1)
	CT(6695)=(S=4020224,E=4020479,C=0)
	CT(6696)=(S=4020480,E=4020735,C=2)
	CT(6697)=(S=4020736,E=4021247,C=3)
	CT(6698)=(S=4021248,E=4021759,C=8)
	CT(6699)=(S=4021760,E=4022015,C=74)
	CT(6700)=(S=4022016,E=4022143,C=0)
	CT(6701)=(S=4022144,E=4022271,C=4)
	CT(6702)=(S=4022272,E=4026367,C=6)
	CT(6703)=(S=4026368,E=4030463,C=2)
	CT(6704)=(S=4030464,E=4046847,C=1)
	CT(6705)=(S=4046848,E=4052991,C=2)
	CT(6706)=(S=4052992,E=4057087,C=9)
	CT(6707)=(S=4057088,E=4058623,C=1)
	CT(6708)=(S=4058624,E=4059135,C=8)
	CT(6709)=(S=4059136,E=4060159,C=1)
	CT(6710)=(S=4060160,E=4060415,C=8)
	CT(6711)=(S=4060416,E=4060543,C=2)
	CT(6712)=(S=4060544,E=4060575,C=0)
	CT(6713)=(S=4060576,E=4060591,C=156)
	CT(6714)=(S=4060592,E=4060607,C=6)
	CT(6715)=(S=4060608,E=4060639,C=2)
	CT(6716)=(S=4060640,E=4060671,C=6)
	CT(6717)=(S=4060672,E=4060927,C=4)
	CT(6718)=(S=4060928,E=4060991,C=74)
	CT(6719)=(S=4060992,E=4061087,C=6)
	CT(6720)=(S=4061088,E=4061103,C=9)
	CT(6721)=(S=4061104,E=4061119,C=84)
	CT(6722)=(S=4061120,E=4061151,C=6)
	CT(6723)=(S=4061152,E=4061183,C=4)
	CT(6724)=(S=4061184,E=4063231,C=6)
	CT(6725)=(S=4063232,E=4063487,C=17)
	CT(6726)=(S=4063488,E=4063743,C=22)
	CT(6727)=(S=4063744,E=4063999,C=34)
	CT(6728)=(S=4064000,E=4064063,C=23)
	CT(6729)=(S=4064064,E=4064127,C=14)
	CT(6730)=(S=4064128,E=4064159,C=19)
	CT(6731)=(S=4064160,E=4064191,C=58)
	CT(6732)=(S=4064192,E=4064255,C=14)
	CT(6733)=(S=4064256,E=4064287,C=12)
	CT(6734)=(S=4064288,E=4064319,C=102)
	CT(6735)=(S=4064320,E=4064351,C=13)
	CT(6736)=(S=4064352,E=4064383,C=33)
	CT(6737)=(S=4064384,E=4064511,C=49)
	CT(6738)=(S=4064512,E=4064639,C=19)
	CT(6739)=(S=4064640,E=4064767,C=18)
	CT(6740)=(S=4064768,E=4065279,C=14)
	CT(6741)=(S=4065280,E=4065311,C=12)
	CT(6742)=(S=4065312,E=4065343,C=28)
	CT(6743)=(S=4065344,E=4065375,C=109)
	CT(6744)=(S=4065376,E=4065407,C=14)
	CT(6745)=(S=4065408,E=4065535,C=28)
	CT(6746)=(S=4065536,E=4065791,C=63)
	CT(6747)=(S=4065792,E=4066303,C=15)
	CT(6748)=(S=4066304,E=4066335,C=36)
	CT(6749)=(S=4066336,E=4066367,C=24)
	CT(6750)=(S=4066368,E=4066399,C=32)
	CT(6751)=(S=4066400,E=4066415,C=119)
	CT(6752)=(S=4066416,E=4066423,C=109)
	CT(6753)=(S=4066432,E=4066559,C=34)
	CT(6754)=(S=4066560,E=4066687,C=19)
	CT(6755)=(S=4066688,E=4066719,C=14)
	CT(6756)=(S=4066720,E=4066751,C=15)
	CT(6757)=(S=4066752,E=4066783,C=42)
	CT(6758)=(S=4066784,E=4067327,C=25)
	CT(6759)=(S=4067328,E=4067359,C=31)
	CT(6760)=(S=4067360,E=4067455,C=18)
	CT(6761)=(S=4067456,E=4067583,C=27)
	CT(6762)=(S=4067584,E=4067839,C=63)
	CT(6763)=(S=4067840,E=4068351,C=15)
	CT(6764)=(S=4068352,E=4068607,C=19)
	CT(6765)=(S=4068608,E=4068735,C=58)
	CT(6766)=(S=4068736,E=4068863,C=36)
	CT(6767)=(S=4068864,E=4069119,C=25)
	CT(6768)=(S=4069120,E=4069375,C=14)
	CT(6769)=(S=4069376,E=4069407,C=28)
	CT(6770)=(S=4069408,E=4069439,C=27)
	CT(6771)=(S=4069440,E=4069471,C=33)
	CT(6772)=(S=4069472,E=4069503,C=109)
	CT(6773)=(S=4069504,E=4069631,C=14)
	CT(6774)=(S=4069632,E=4069695,C=36)
	CT(6775)=(S=4069696,E=4069887,C=14)
	CT(6776)=(S=4069888,E=4070399,C=28)
	CT(6777)=(S=4070400,E=4070655,C=21)
	CT(6778)=(S=4070656,E=4070783,C=37)
	CT(6779)=(S=4070784,E=4070911,C=58)
	CT(6780)=(S=4070912,E=4071423,C=14)
	CT(6781)=(S=4071424,E=4071455,C=34)
	CT(6782)=(S=4071456,E=4071487,C=36)
	CT(6783)=(S=4071488,E=4071519,C=18)
	CT(6784)=(S=4071520,E=4071551,C=14)
	CT(6785)=(S=4071552,E=4071679,C=25)
	CT(6786)=(S=4071680,E=4071935,C=18)
	CT(6787)=(S=4071936,E=4072447,C=12)
	CT(6788)=(S=4072448,E=4072959,C=25)
	CT(6789)=(S=4072960,E=4073215,C=22)
	CT(6790)=(S=4073216,E=4073471,C=12)
	CT(6791)=(S=4073472,E=4073503,C=28)
	CT(6792)=(S=4073504,E=4073535,C=63)
	CT(6793)=(S=4073536,E=4073567,C=33)
	CT(6794)=(S=4073568,E=4073599,C=14)
	CT(6795)=(S=4073600,E=4073727,C=42)
	CT(6796)=(S=4073728,E=4073983,C=36)
	CT(6797)=(S=4073984,E=4074495,C=25)
	CT(6798)=(S=4074496,E=4074527,C=33)
	CT(6799)=(S=4074528,E=4074559,C=28)
	CT(6800)=(S=4074560,E=4074591,C=14)
	CT(6801)=(S=4074592,E=4074623,C=51)
	CT(6802)=(S=4074624,E=4074687,C=24)
	CT(6803)=(S=4074688,E=4074751,C=47)
	CT(6804)=(S=4074752,E=4075007,C=36)
	CT(6805)=(S=4075008,E=4075519,C=42)
	CT(6806)=(S=4075520,E=4075551,C=34)
	CT(6807)=(S=4075552,E=4075583,C=15)
	CT(6808)=(S=4075584,E=4075615,C=28)
	CT(6809)=(S=4075616,E=4075647,C=34)
	CT(6810)=(S=4075648,E=4075775,C=21)
	CT(6811)=(S=4075776,E=4076127,C=14)
	CT(6812)=(S=4076128,E=4076159,C=28)
	CT(6813)=(S=4076160,E=4076191,C=14)
	CT(6814)=(S=4076192,E=4076223,C=27)
	CT(6815)=(S=4076224,E=4076287,C=14)
	CT(6816)=(S=4076288,E=4076543,C=13)
	CT(6817)=(S=4076544,E=4077567,C=28)
	CT(6818)=(S=4077568,E=4077823,C=14)
	CT(6819)=(S=4077824,E=4078079,C=25)
	CT(6820)=(S=4078080,E=4078591,C=49)
	CT(6821)=(S=4078592,E=4078719,C=14)
	CT(6822)=(S=4078720,E=4078847,C=26)
	CT(6823)=(S=4078848,E=4078879,C=18)
	CT(6824)=(S=4078880,E=4078911,C=58)
	CT(6825)=(S=4078912,E=4078975,C=47)
	CT(6826)=(S=4078976,E=4079007,C=24)
	CT(6827)=(S=4079008,E=4079039,C=45)
	CT(6828)=(S=4079040,E=4079103,C=151)
	CT(6829)=(S=4079104,E=4079231,C=13)
	CT(6830)=(S=4079232,E=4079359,C=12)
	CT(6831)=(S=4079360,E=4079423,C=27)
	CT(6832)=(S=4079424,E=4079487,C=18)
	CT(6833)=(S=4079488,E=4079551,C=60)
	CT(6834)=(S=4079552,E=4079615,C=19)
	CT(6835)=(S=4079616,E=4079647,C=18)
	CT(6836)=(S=4079648,E=4079679,C=12)
	CT(6837)=(S=4079680,E=4079743,C=31)
	CT(6838)=(S=4079744,E=4079871,C=14)
	CT(6839)=(S=4079872,E=4079903,C=19)
	CT(6840)=(S=4079904,E=4079935,C=55)
	CT(6841)=(S=4079936,E=4079999,C=19)
	CT(6842)=(S=4080000,E=4080031,C=34)
	CT(6843)=(S=4080032,E=4080063,C=71)
	CT(6844)=(S=4080064,E=4080127,C=55)
	CT(6845)=(S=4080128,E=4080383,C=24)
	CT(6846)=(S=4080384,E=4080639,C=14)
	CT(6847)=(S=4080640,E=4080671,C=28)
	CT(6848)=(S=4080672,E=4080703,C=65)
	CT(6849)=(S=4080704,E=4080735,C=22)
	CT(6850)=(S=4080736,E=4080767,C=59)
	CT(6851)=(S=4080768,E=4080799,C=18)
	CT(6852)=(S=4080800,E=4080831,C=172)
	CT(6853)=(S=4080832,E=4080863,C=42)
	CT(6854)=(S=4080864,E=4080895,C=106)
	CT(6855)=(S=4080896,E=4080927,C=18)
	CT(6856)=(S=4080928,E=4081023,C=14)
	CT(6857)=(S=4081024,E=4081031,C=15)
	CT(6858)=(S=4081032,E=4081039,C=14)
	CT(6859)=(S=4081040,E=4081047,C=28)
	CT(6860)=(S=4081048,E=4081055,C=24)
	CT(6861)=(S=4081056,E=4081087,C=36)
	CT(6862)=(S=4081088,E=4081151,C=58)
	CT(6863)=(S=4081152,E=4081407,C=27)
	CT(6864)=(S=4081408,E=4081663,C=47)
	CT(6865)=(S=4081664,E=4081759,C=28)
	CT(6866)=(S=4081760,E=4081823,C=14)
	CT(6867)=(S=4081824,E=4081855,C=31)
	CT(6868)=(S=4081856,E=4081887,C=36)
	CT(6869)=(S=4081888,E=4081919,C=47)
	CT(6870)=(S=4081920,E=4081951,C=12)
	CT(6871)=(S=4081952,E=4081983,C=47)
	CT(6872)=(S=4081984,E=4082047,C=51)
	CT(6873)=(S=4082048,E=4082111,C=14)
	CT(6874)=(S=4082112,E=4082175,C=27)
	CT(6875)=(S=4082176,E=4082559,C=22)
	CT(6876)=(S=4082560,E=4082687,C=28)
	CT(6877)=(S=4082688,E=4082943,C=18)
	CT(6878)=(S=4082944,E=4082975,C=14)
	CT(6879)=(S=4082976,E=4083007,C=15)
	CT(6880)=(S=4083008,E=4083071,C=33)
	CT(6881)=(S=4083072,E=4083103,C=50)
	CT(6882)=(S=4083104,E=4083135,C=63)
	CT(6883)=(S=4083136,E=4083199,C=50)
	CT(6884)=(S=4083200,E=4083231,C=13)
	CT(6885)=(S=4083232,E=4083263,C=18)
	CT(6886)=(S=4083264,E=4083279,C=28)
	CT(6887)=(S=4083280,E=4083295,C=18)
	CT(6888)=(S=4083296,E=4083455,C=47)
	CT(6889)=(S=4083456,E=4083711,C=24)
	CT(6890)=(S=4083712,E=4083775,C=28)
	CT(6891)=(S=4083776,E=4083807,C=33)
	CT(6892)=(S=4083808,E=4083839,C=28)
	CT(6893)=(S=4083840,E=4083871,C=47)
	CT(6894)=(S=4083872,E=4083903,C=31)
	CT(6895)=(S=4083904,E=4083935,C=19)
	CT(6896)=(S=4083936,E=4083967,C=44)
	CT(6897)=(S=4083968,E=4084735,C=25)
	CT(6898)=(S=4084736,E=4084767,C=60)
	CT(6899)=(S=4084768,E=4084799,C=20)
	CT(6900)=(S=4084800,E=4084831,C=39)
	CT(6901)=(S=4084832,E=4084863,C=18)
	CT(6902)=(S=4084864,E=4084895,C=33)
	CT(6903)=(S=4084896,E=4084959,C=14)
	CT(6904)=(S=4084960,E=4084975,C=52)
	CT(6905)=(S=4084976,E=4084983,C=36)
	CT(6906)=(S=4084984,E=4084991,C=31)
	CT(6907)=(S=4084992,E=4085119,C=60)
	CT(6908)=(S=4085120,E=4085151,C=12)
	CT(6909)=(S=4085152,E=4085503,C=15)
	CT(6910)=(S=4085504,E=4085631,C=25)
	CT(6911)=(S=4085632,E=4085759,C=58)
	CT(6912)=(S=4085760,E=4085887,C=49)
	CT(6913)=(S=4085888,E=4086015,C=19)
	CT(6914)=(S=4086016,E=4086047,C=66)
	CT(6915)=(S=4086048,E=4086079,C=27)
	CT(6916)=(S=4086080,E=4086143,C=58)
	CT(6917)=(S=4086144,E=4086175,C=14)
	CT(6918)=(S=4086176,E=4086207,C=28)
	CT(6919)=(S=4086208,E=4086271,C=18)
	CT(6920)=(S=4086272,E=4086527,C=17)
	CT(6921)=(S=4086528,E=4086783,C=28)
	CT(6922)=(S=4086784,E=4087039,C=27)
	CT(6923)=(S=4087040,E=4087071,C=28)
	CT(6924)=(S=4087072,E=4087103,C=58)
	CT(6925)=(S=4087104,E=4087167,C=42)
	CT(6926)=(S=4087168,E=4087199,C=14)
	CT(6927)=(S=4087200,E=4087231,C=25)
	CT(6928)=(S=4087232,E=4087295,C=13)
	CT(6929)=(S=4087296,E=4087551,C=15)
	CT(6930)=(S=4087552,E=4087679,C=19)
	CT(6931)=(S=4087680,E=4087807,C=13)
	CT(6932)=(S=4087808,E=4088063,C=14)
	CT(6933)=(S=4088064,E=4088127,C=15)
	CT(6934)=(S=4088128,E=4088223,C=25)
	CT(6935)=(S=4088224,E=4088319,C=27)
	CT(6936)=(S=4088320,E=4088575,C=15)
	CT(6937)=(S=4088576,E=4088703,C=25)
	CT(6938)=(S=4088704,E=4088831,C=42)
	CT(6939)=(S=4088832,E=4088895,C=36)
	CT(6940)=(S=4088896,E=4088927,C=15)
	CT(6941)=(S=4088928,E=4088959,C=25)
	CT(6942)=(S=4088960,E=4088991,C=12)
	CT(6943)=(S=4088992,E=4089055,C=14)
	CT(6944)=(S=4089056,E=4089087,C=50)
	CT(6945)=(S=4089088,E=4089119,C=15)
	CT(6946)=(S=4089120,E=4089151,C=19)
	CT(6947)=(S=4089152,E=4089215,C=15)
	CT(6948)=(S=4089216,E=4089247,C=59)
	CT(6949)=(S=4089248,E=4089279,C=25)
	CT(6950)=(S=4089280,E=4089343,C=27)
	CT(6951)=(S=4089344,E=4089471,C=13)
	CT(6952)=(S=4089472,E=4089487,C=26)
	CT(6953)=(S=4089488,E=4089495,C=19)
	CT(6954)=(S=4089496,E=4089503,C=15)
	CT(6955)=(S=4089504,E=4089535,C=27)
	CT(6956)=(S=4089536,E=4089567,C=28)
	CT(6957)=(S=4089568,E=4089599,C=12)
	CT(6958)=(S=4089600,E=4089855,C=22)
	CT(6959)=(S=4089856,E=4090111,C=28)
	CT(6960)=(S=4090112,E=4090175,C=18)
	CT(6961)=(S=4090176,E=4090239,C=14)
	CT(6962)=(S=4090240,E=4090271,C=18)
	CT(6963)=(S=4090272,E=4090303,C=14)
	CT(6964)=(S=4090304,E=4090367,C=36)
	CT(6965)=(S=4090368,E=4090431,C=47)
	CT(6966)=(S=4090432,E=4090463,C=12)
	CT(6967)=(S=4090464,E=4090495,C=18)
	CT(6968)=(S=4090496,E=4090623,C=12)
	CT(6969)=(S=4090624,E=4090879,C=24)
	CT(6970)=(S=4090880,E=4090911,C=36)
	CT(6971)=(S=4090912,E=4090943,C=28)
	CT(6972)=(S=4090944,E=4090975,C=37)
	CT(6973)=(S=4090976,E=4091007,C=46)
	CT(6974)=(S=4091008,E=4091039,C=34)
	CT(6975)=(S=4091040,E=4091071,C=58)
	CT(6976)=(S=4091072,E=4091103,C=19)
	CT(6977)=(S=4091104,E=4091135,C=15)
	CT(6978)=(S=4091136,E=4091167,C=18)
	CT(6979)=(S=4091168,E=4091199,C=19)
	CT(6980)=(S=4091200,E=4091263,C=28)
	CT(6981)=(S=4091264,E=4091295,C=33)
	CT(6982)=(S=4091296,E=4091327,C=18)
	CT(6983)=(S=4091328,E=4091391,C=36)
	CT(6984)=(S=4091392,E=4091647,C=15)
	CT(6985)=(S=4091648,E=4091775,C=28)
	CT(6986)=(S=4091776,E=4091903,C=58)
	CT(6987)=(S=4091904,E=4091935,C=49)
	CT(6988)=(S=4091936,E=4091999,C=28)
	CT(6989)=(S=4092000,E=4092031,C=18)
	CT(6990)=(S=4092032,E=4092063,C=28)
	CT(6991)=(S=4092064,E=4092095,C=36)
	CT(6992)=(S=4092096,E=4092127,C=50)
	CT(6993)=(S=4092128,E=4092159,C=36)
	CT(6994)=(S=4092160,E=4092191,C=59)
	CT(6995)=(S=4092192,E=4092287,C=18)
	CT(6996)=(S=4092288,E=4092319,C=27)
	CT(6997)=(S=4092320,E=4092351,C=47)
	CT(6998)=(S=4092352,E=4092415,C=28)
	CT(6999)=(S=4092416,E=4092671,C=106)
	CT(7000)=(S=4092672,E=4092927,C=13)
	CT(7001)=(S=4092928,E=4093055,C=42)
	CT(7002)=(S=4093056,E=4093119,C=28)
	CT(7003)=(S=4093120,E=4093151,C=24)
	CT(7004)=(S=4093152,E=4093183,C=19)
	CT(7005)=(S=4093184,E=4093215,C=28)
	CT(7006)=(S=4093216,E=4093247,C=106)
	CT(7007)=(S=4093248,E=4093311,C=18)
	CT(7008)=(S=4093312,E=4093439,C=25)
	CT(7009)=(S=4093440,E=4093695,C=18)
	CT(7010)=(S=4093696,E=4093951,C=19)
	CT(7011)=(S=4093952,E=4094207,C=23)
	CT(7012)=(S=4094208,E=4094239,C=14)
	CT(7013)=(S=4094240,E=4094271,C=47)
	CT(7014)=(S=4094272,E=4094367,C=58)
	CT(7015)=(S=4094368,E=4094399,C=24)
	CT(7016)=(S=4094400,E=4094463,C=34)
	CT(7017)=(S=4094464,E=4094471,C=31)
	CT(7018)=(S=4094472,E=4094479,C=12)
	CT(7019)=(S=4094480,E=4094487,C=60)
	CT(7020)=(S=4094504,E=4094511,C=38)
	CT(7021)=(S=4094512,E=4094519,C=18)
	CT(7022)=(S=4094520,E=4094535,C=31)
	CT(7023)=(S=4094544,E=4094551,C=28)
	CT(7024)=(S=4094552,E=4094567,C=18)
	CT(7025)=(S=4094568,E=4094575,C=31)
	CT(7026)=(S=4094576,E=4094591,C=58)
	CT(7027)=(S=4094592,E=4094615,C=18)
	CT(7028)=(S=4094616,E=4094623,C=31)
	CT(7029)=(S=4094624,E=4094631,C=24)
	CT(7030)=(S=4094632,E=4094639,C=33)
	CT(7031)=(S=4094640,E=4094663,C=18)
	CT(7032)=(S=4094664,E=4094671,C=31)
	CT(7033)=(S=4094672,E=4094679,C=18)
	CT(7034)=(S=4094696,E=4094703,C=58)
	CT(7035)=(S=4094704,E=4094711,C=18)
	CT(7036)=(S=4094712,E=4094719,C=27)
	CT(7037)=(S=4094720,E=4094975,C=15)
	CT(7038)=(S=4094976,E=4095743,C=14)
	CT(7039)=(S=4095744,E=4095999,C=19)
	CT(7040)=(S=4096000,E=4096031,C=28)
	CT(7041)=(S=4096032,E=4096063,C=17)
	CT(7042)=(S=4096064,E=4096095,C=15)
	CT(7043)=(S=4096096,E=4096127,C=28)
	CT(7044)=(S=4096128,E=4096159,C=14)
	CT(7045)=(S=4096160,E=4096191,C=28)
	CT(7046)=(S=4096192,E=4096223,C=14)
	CT(7047)=(S=4096224,E=4096255,C=27)
	CT(7048)=(S=4096256,E=4096287,C=12)
	CT(7049)=(S=4096288,E=4096319,C=33)
	CT(7050)=(S=4096320,E=4096383,C=14)
	CT(7051)=(S=4096384,E=4096415,C=36)
	CT(7052)=(S=4096416,E=4096447,C=12)
	CT(7053)=(S=4096448,E=4096511,C=58)
	CT(7054)=(S=4096512,E=4096767,C=14)
	CT(7055)=(S=4096768,E=4097023,C=36)
	CT(7056)=(S=4097024,E=4097279,C=28)
	CT(7057)=(S=4097280,E=4097311,C=14)
	CT(7058)=(S=4097344,E=4097407,C=36)
	CT(7059)=(S=4097408,E=4097439,C=58)
	CT(7060)=(S=4097440,E=4097471,C=18)
	CT(7061)=(S=4097472,E=4097535,C=36)
	CT(7062)=(S=4097536,E=4097791,C=13)
	CT(7063)=(S=4097792,E=4097919,C=106)
	CT(7064)=(S=4097920,E=4098047,C=24)
	CT(7065)=(S=4098048,E=4098559,C=14)
	CT(7066)=(S=4098560,E=4098815,C=28)
	CT(7067)=(S=4098816,E=4099071,C=106)
	CT(7068)=(S=4099072,E=4099103,C=14)
	CT(7069)=(S=4099104,E=4099135,C=42)
	CT(7070)=(S=4099136,E=4099199,C=106)
	CT(7071)=(S=4099200,E=4099263,C=36)
	CT(7072)=(S=4099264,E=4099295,C=14)
	CT(7073)=(S=4099296,E=4099327,C=18)
	CT(7074)=(S=4099328,E=4099359,C=33)
	CT(7075)=(S=4099360,E=4099391,C=28)
	CT(7076)=(S=4099392,E=4099455,C=18)
	CT(7077)=(S=4099456,E=4099487,C=27)
	CT(7078)=(S=4099488,E=4099519,C=28)
	CT(7079)=(S=4099520,E=4099583,C=58)
	CT(7080)=(S=4099584,E=4099839,C=47)
	CT(7081)=(S=4099840,E=4100383,C=28)
	CT(7082)=(S=4100384,E=4100479,C=36)
	CT(7083)=(S=4100480,E=4100511,C=48)
	CT(7084)=(S=4100512,E=4100543,C=47)
	CT(7085)=(S=4100544,E=4100607,C=36)
	CT(7086)=(S=4100608,E=4100863,C=28)
	CT(7087)=(S=4100864,E=4101119,C=12)
	CT(7088)=(S=4101120,E=4101151,C=18)
	CT(7089)=(S=4101152,E=4101183,C=27)
	CT(7090)=(S=4101184,E=4101215,C=58)
	CT(7091)=(S=4101216,E=4101247,C=47)
	CT(7092)=(S=4101248,E=4101279,C=18)
	CT(7093)=(S=4101280,E=4101311,C=36)
	CT(7094)=(S=4101312,E=4101343,C=47)
	CT(7095)=(S=4101344,E=4101375,C=18)
	CT(7096)=(S=4101376,E=4101407,C=31)
	CT(7097)=(S=4101408,E=4101439,C=14)
	CT(7098)=(S=4101440,E=4101503,C=23)
	CT(7099)=(S=4101504,E=4101631,C=15)
	CT(7100)=(S=4101632,E=4101887,C=101)
	CT(7101)=(S=4101888,E=4102143,C=25)
	CT(7102)=(S=4102144,E=4102175,C=32)
	CT(7103)=(S=4102176,E=4102239,C=18)
	CT(7104)=(S=4102240,E=4102271,C=15)
	CT(7105)=(S=4102272,E=4102303,C=58)
	CT(7106)=(S=4102304,E=4102335,C=28)
	CT(7107)=(S=4102336,E=4102367,C=18)
	CT(7108)=(S=4102368,E=4102399,C=71)
	CT(7109)=(S=4102400,E=4104191,C=28)
	CT(7110)=(S=4104192,E=4104703,C=12)
	CT(7111)=(S=4104704,E=4104959,C=54)
	CT(7112)=(S=4104960,E=4105215,C=36)
	CT(7113)=(S=4105216,E=4105343,C=13)
	CT(7114)=(S=4105344,E=4105471,C=14)
	CT(7115)=(S=4105472,E=4105535,C=18)
	CT(7116)=(S=4105536,E=4105599,C=36)
	CT(7117)=(S=4105600,E=4105663,C=47)
	CT(7118)=(S=4105664,E=4105727,C=50)
	CT(7119)=(S=4105728,E=4105983,C=49)
	CT(7120)=(S=4105984,E=4106239,C=34)
	CT(7121)=(S=4106240,E=4106303,C=33)
	CT(7122)=(S=4106304,E=4106367,C=71)
	CT(7123)=(S=4106368,E=4106399,C=19)
	CT(7124)=(S=4106400,E=4106431,C=40)
	CT(7125)=(S=4106432,E=4106463,C=28)
	CT(7126)=(S=4106464,E=4106495,C=18)
	CT(7127)=(S=4106496,E=4106527,C=28)
	CT(7128)=(S=4106528,E=4106559,C=15)
	CT(7129)=(S=4106560,E=4106623,C=21)
	CT(7130)=(S=4106624,E=4106655,C=14)
	CT(7131)=(S=4106656,E=4106687,C=71)
	CT(7132)=(S=4106688,E=4106751,C=22)
	CT(7133)=(S=4106752,E=4107519,C=14)
	CT(7134)=(S=4107520,E=4107551,C=72)
	CT(7135)=(S=4107552,E=4107583,C=111)
	CT(7136)=(S=4107584,E=4107647,C=14)
	CT(7137)=(S=4107648,E=4107679,C=18)
	CT(7138)=(S=4107680,E=4107711,C=15)
	CT(7139)=(S=4107712,E=4107775,C=14)
	CT(7140)=(S=4107776,E=4108287,C=25)
	CT(7141)=(S=4108288,E=4108319,C=18)
	CT(7142)=(S=4108320,E=4108351,C=47)
	CT(7143)=(S=4108352,E=4108415,C=51)
	CT(7144)=(S=4108416,E=4108447,C=28)
	CT(7145)=(S=4108448,E=4108479,C=71)
	CT(7146)=(S=4108480,E=4108511,C=27)
	CT(7147)=(S=4108512,E=4108543,C=28)
	CT(7148)=(S=4108544,E=4108575,C=15)
	CT(7149)=(S=4108576,E=4108607,C=18)
	CT(7150)=(S=4108608,E=4108671,C=33)
	CT(7151)=(S=4108672,E=4108799,C=36)
	CT(7152)=(S=4108800,E=4109311,C=42)
	CT(7153)=(S=4109312,E=4109567,C=13)
	CT(7154)=(S=4109568,E=4109599,C=58)
	CT(7155)=(S=4109600,E=4109631,C=18)
	CT(7156)=(S=4109632,E=4109695,C=19)
	CT(7157)=(S=4109696,E=4109727,C=28)
	CT(7158)=(S=4109728,E=4109759,C=58)
	CT(7159)=(S=4109760,E=4109831,C=19)
	CT(7160)=(S=4109832,E=4109839,C=18)
	CT(7161)=(S=4109840,E=4109847,C=14)
	CT(7162)=(S=4109848,E=4109879,C=18)
	CT(7163)=(S=4109880,E=4109887,C=49)
	CT(7164)=(S=4109888,E=4109895,C=31)
	CT(7165)=(S=4109896,E=4109903,C=18)
	CT(7166)=(S=4109904,E=4109911,C=31)
	CT(7167)=(S=4109912,E=4109919,C=18)
	CT(7168)=(S=4109926,E=4109927,C=28)
	CT(7169)=(S=4109928,E=4109935,C=58)
	CT(7170)=(S=4109936,E=4109943,C=51)
	CT(7171)=(S=4109944,E=4109951,C=31)
	CT(7172)=(S=4109952,E=4109967,C=18)
	CT(7173)=(S=4109968,E=4109975,C=58)
	CT(7174)=(S=4109976,E=4109991,C=31)
	CT(7175)=(S=4109992,E=4109999,C=24)
	CT(7176)=(S=4110000,E=4110007,C=70)
	CT(7177)=(S=4110008,E=4110015,C=34)
	CT(7178)=(S=4110016,E=4110031,C=18)
	CT(7179)=(S=4110032,E=4110039,C=58)
	CT(7180)=(S=4110040,E=4110047,C=19)
	CT(7181)=(S=4110048,E=4110055,C=58)
	CT(7182)=(S=4110056,E=4110063,C=33)
	CT(7183)=(S=4110064,E=4110071,C=36)
	CT(7184)=(S=4110072,E=4110079,C=24)
	CT(7185)=(S=4110080,E=4110207,C=18)
	CT(7186)=(S=4110208,E=4110335,C=47)
	CT(7187)=(S=4110336,E=4111359,C=13)
	CT(7188)=(S=4111360,E=4112383,C=14)
	CT(7189)=(S=4112384,E=4112415,C=34)
	CT(7190)=(S=4112416,E=4112447,C=18)
	CT(7191)=(S=4112448,E=4112511,C=36)
	CT(7192)=(S=4112512,E=4112543,C=28)
	CT(7193)=(S=4112544,E=4112575,C=18)
	CT(7194)=(S=4112576,E=4112607,C=28)
	CT(7195)=(S=4112608,E=4112639,C=18)
	CT(7196)=(S=4112640,E=4112671,C=26)
	CT(7197)=(S=4112672,E=4112703,C=12)
	CT(7198)=(S=4112704,E=4112767,C=106)
	CT(7199)=(S=4112768,E=4112799,C=46)
	CT(7200)=(S=4112832,E=4112895,C=12)
	CT(7201)=(S=4112896,E=4113407,C=36)
	CT(7202)=(S=4113408,E=4113663,C=15)
	CT(7203)=(S=4113664,E=4113695,C=28)
	CT(7204)=(S=4113696,E=4113727,C=14)
	CT(7205)=(S=4113728,E=4113791,C=49)
	CT(7206)=(S=4113792,E=4113823,C=36)
	CT(7207)=(S=4113824,E=4113855,C=47)
	CT(7208)=(S=4113856,E=4113919,C=71)
	CT(7209)=(S=4113920,E=4114431,C=24)
	CT(7210)=(S=4114432,E=4114687,C=13)
	CT(7211)=(S=4114688,E=4114703,C=25)
	CT(7212)=(S=4114704,E=4114719,C=33)
	CT(7213)=(S=4114720,E=4114751,C=19)
	CT(7214)=(S=4114752,E=4114815,C=50)
	CT(7215)=(S=4114816,E=4114847,C=12)
	CT(7216)=(S=4114848,E=4114879,C=28)
	CT(7217)=(S=4114880,E=4114943,C=35)
	CT(7218)=(S=4114944,E=4115455,C=34)
	CT(7219)=(S=4115456,E=4115487,C=47)
	CT(7220)=(S=4115488,E=4115519,C=14)
	CT(7221)=(S=4115520,E=4115551,C=36)
	CT(7222)=(S=4115552,E=4115583,C=34)
	CT(7223)=(S=4115584,E=4115615,C=51)
	CT(7224)=(S=4115616,E=4115647,C=28)
	CT(7225)=(S=4115648,E=4115679,C=25)
	CT(7226)=(S=4115680,E=4115711,C=33)
	CT(7227)=(S=4115712,E=4115743,C=15)
	CT(7228)=(S=4115744,E=4115775,C=22)
	CT(7229)=(S=4115776,E=4115839,C=49)
	CT(7230)=(S=4115840,E=4115871,C=31)
	CT(7231)=(S=4115872,E=4115903,C=18)
	CT(7232)=(S=4115904,E=4115967,C=60)
	CT(7233)=(S=4115968,E=4116223,C=28)
	CT(7234)=(S=4116224,E=4116479,C=36)
	CT(7235)=(S=4116480,E=4116735,C=13)
	CT(7236)=(S=4116736,E=4116767,C=104)
	CT(7237)=(S=4116768,E=4116799,C=28)
	CT(7238)=(S=4116800,E=4116863,C=27)
	CT(7239)=(S=4116864,E=4116895,C=98)
	CT(7240)=(S=4116896,E=4116927,C=19)
	CT(7241)=(S=4116928,E=4116991,C=33)
	CT(7242)=(S=4116992,E=4117247,C=12)
	CT(7243)=(S=4117248,E=4117535,C=15)
	CT(7244)=(S=4117536,E=4117567,C=40)
	CT(7245)=(S=4117568,E=4117599,C=36)
	CT(7246)=(S=4117600,E=4117631,C=12)
	CT(7247)=(S=4117632,E=4117663,C=36)
	CT(7248)=(S=4117664,E=4117695,C=42)
	CT(7249)=(S=4117696,E=4117727,C=44)
	CT(7250)=(S=4117728,E=4117759,C=41)
	CT(7251)=(S=4117760,E=4117887,C=18)
	CT(7252)=(S=4117888,E=4117919,C=14)
	CT(7253)=(S=4117920,E=4117951,C=12)
	CT(7254)=(S=4117952,E=4118015,C=49)
	CT(7255)=(S=4118016,E=4118271,C=28)
	CT(7256)=(S=4118272,E=4118527,C=101)
	CT(7257)=(S=4118528,E=4118559,C=36)
	CT(7258)=(S=4118560,E=4118591,C=31)
	CT(7259)=(S=4118592,E=4118623,C=38)
	CT(7260)=(S=4118624,E=4118655,C=47)
	CT(7261)=(S=4118656,E=4118687,C=14)
	CT(7262)=(S=4118688,E=4118751,C=28)
	CT(7263)=(S=4118752,E=4118783,C=14)
	CT(7264)=(S=4118784,E=4118815,C=12)
	CT(7265)=(S=4118816,E=4118847,C=28)
	CT(7266)=(S=4118848,E=4118911,C=22)
	CT(7267)=(S=4118912,E=4118943,C=41)
	CT(7268)=(S=4118944,E=4118975,C=18)
	CT(7269)=(S=4118976,E=4119039,C=38)
	CT(7270)=(S=4119040,E=4119295,C=42)
	CT(7271)=(S=4119296,E=4119551,C=17)
	CT(7272)=(S=4119552,E=4119583,C=28)
	CT(7273)=(S=4119584,E=4119615,C=18)
	CT(7274)=(S=4119616,E=4119647,C=15)
	CT(7275)=(S=4119648,E=4119679,C=26)
	CT(7276)=(S=4119680,E=4119711,C=34)
	CT(7277)=(S=4119712,E=4119743,C=19)
	CT(7278)=(S=4119744,E=4119775,C=54)
	CT(7279)=(S=4119776,E=4119807,C=47)
	CT(7280)=(S=4119808,E=4119839,C=13)
	CT(7281)=(S=4119840,E=4119871,C=31)
	CT(7282)=(S=4119872,E=4119935,C=53)
	CT(7283)=(S=4119936,E=4119967,C=51)
	CT(7284)=(S=4119968,E=4119999,C=15)
	CT(7285)=(S=4120000,E=4120575,C=36)
	CT(7286)=(S=4120576,E=4121599,C=28)
	CT(7287)=(S=4121600,E=4121855,C=22)
	CT(7288)=(S=4121856,E=4122367,C=13)
	CT(7289)=(S=4122368,E=4122399,C=18)
	CT(7290)=(S=4122400,E=4122431,C=63)
	CT(7291)=(S=4122432,E=4122495,C=38)
	CT(7292)=(S=4122496,E=4122527,C=14)
	CT(7293)=(S=4122528,E=4122559,C=18)
	CT(7294)=(S=4122560,E=4122623,C=45)
	CT(7295)=(S=4122624,E=4122879,C=14)
	CT(7296)=(S=4122880,E=4122911,C=49)
	CT(7297)=(S=4122912,E=4122943,C=12)
	CT(7298)=(S=4122944,E=4123007,C=14)
	CT(7299)=(S=4123008,E=4123135,C=58)
	CT(7300)=(S=4123136,E=4123391,C=36)
	CT(7301)=(S=4123392,E=4123647,C=49)
	CT(7302)=(S=4123648,E=4124159,C=47)
	CT(7303)=(S=4124160,E=4124415,C=36)
	CT(7304)=(S=4124416,E=4124671,C=14)
	CT(7305)=(S=4124672,E=4124703,C=46)
	CT(7306)=(S=4124704,E=4124735,C=65)
	CT(7307)=(S=4124736,E=4124767,C=47)
	CT(7308)=(S=4124768,E=4124799,C=106)
	CT(7309)=(S=4124800,E=4124831,C=28)
	CT(7310)=(S=4124832,E=4124863,C=33)
	CT(7311)=(S=4124864,E=4124895,C=34)
	CT(7312)=(S=4124896,E=4124927,C=12)
	CT(7313)=(S=4124928,E=4124959,C=15)
	CT(7314)=(S=4124960,E=4124991,C=28)
	CT(7315)=(S=4124992,E=4125055,C=12)
	CT(7316)=(S=4125056,E=4125087,C=106)
	CT(7317)=(S=4125088,E=4125119,C=14)
	CT(7318)=(S=4125120,E=4125183,C=47)
	CT(7319)=(S=4125184,E=4125695,C=24)
	CT(7320)=(S=4125696,E=4125759,C=31)
	CT(7321)=(S=4125760,E=4125823,C=12)
	CT(7322)=(S=4125824,E=4125855,C=58)
	CT(7323)=(S=4125856,E=4125887,C=14)
	CT(7324)=(S=4125888,E=4125951,C=37)
	CT(7325)=(S=4125952,E=4125983,C=34)
	CT(7326)=(S=4125984,E=4126015,C=18)
	CT(7327)=(S=4126016,E=4126079,C=33)
	CT(7328)=(S=4126080,E=4126463,C=28)
	CT(7329)=(S=4126464,E=4126719,C=19)
	CT(7330)=(S=4126720,E=4126847,C=37)
	CT(7331)=(S=4126848,E=4126975,C=47)
	CT(7332)=(S=4126976,E=4127007,C=21)
	CT(7333)=(S=4127008,E=4127039,C=15)
	CT(7334)=(S=4127040,E=4127103,C=42)
	CT(7335)=(S=4127104,E=4127135,C=18)
	CT(7336)=(S=4127136,E=4127167,C=27)
	CT(7337)=(S=4127168,E=4127231,C=14)
	CT(7338)=(S=4127232,E=4127615,C=36)
	CT(7339)=(S=4127616,E=4127743,C=134)
	CT(7340)=(S=4127744,E=4128767,C=14)
	CT(7341)=(S=4128768,E=4162543,C=29)
	CT(7342)=(S=4162544,E=4162559,C=80)
	CT(7343)=(S=4162560,E=4163327,C=29)
	CT(7344)=(S=4163328,E=4163359,C=78)
	CT(7345)=(S=4163360,E=4163567,C=29)
	CT(7346)=(S=4163568,E=4163583,C=80)
	CT(7347)=(S=4163584,E=4163695,C=29)
	CT(7348)=(S=4163696,E=4163703,C=174)
	CT(7349)=(S=4163704,E=4165279,C=29)
	CT(7350)=(S=4165280,E=4165295,C=78)
	CT(7351)=(S=4165296,E=4165439,C=29)
	CT(7352)=(S=4165440,E=4165503,C=175)
	CT(7353)=(S=4165504,E=4192863,C=29)
	CT(7354)=(S=4192864,E=4192895,C=78)
	CT(7355)=(S=4192896,E=4195391,C=29)
	CT(7356)=(S=4195392,E=4195423,C=78)
	CT(7357)=(S=4195424,E=4195775,C=29)
	CT(7358)=(S=4195776,E=4195839,C=78)
	CT(7359)=(S=4195840,E=4196031,C=29)
	CT(7360)=(S=4196032,E=4196047,C=78)
	CT(7361)=(S=4196048,E=4196223,C=29)
	CT(7362)=(S=4196224,E=4196255,C=78)
	CT(7363)=(S=4196256,E=4198175,C=29)
	CT(7364)=(S=4198176,E=4198191,C=0)
	CT(7365)=(S=4198192,E=4198223,C=78)
	CT(7366)=(S=4198224,E=4198271,C=29)
	CT(7367)=(S=4198272,E=4198303,C=78)
	CT(7368)=(S=4198304,E=4198975,C=29)
	CT(7369)=(S=4198976,E=4199007,C=78)
	CT(7370)=(S=4199008,E=4199071,C=29)
	CT(7371)=(S=4199072,E=4199103,C=78)
	CT(7372)=(S=4199104,E=4200863,C=29)
	CT(7373)=(S=4200864,E=4200895,C=78)
	CT(7374)=(S=4200896,E=4201087,C=29)
	CT(7375)=(S=4201088,E=4201151,C=78)
	CT(7376)=(S=4201152,E=4201471,C=29)
	CT(7377)=(S=4201472,E=4201503,C=27)
	CT(7378)=(S=4201504,E=4201599,C=29)
	CT(7379)=(S=4201600,E=4201615,C=176)
	CT(7380)=(S=4201632,E=4202559,C=29)
	CT(7381)=(S=4202560,E=4202623,C=177)
	CT(7382)=(S=4202624,E=4203807,C=29)
	CT(7383)=(S=4203808,E=4203823,C=178)
	CT(7384)=(S=4203824,E=4204447,C=29)
	CT(7385)=(S=4204448,E=4204479,C=78)
	CT(7386)=(S=4204480,E=4204639,C=29)
	CT(7387)=(S=4204640,E=4204671,C=78)
	CT(7388)=(S=4204672,E=4204719,C=29)
	CT(7389)=(S=4204720,E=4204735,C=78)
	CT(7390)=(S=4204736,E=4204783,C=29)
	CT(7391)=(S=4204784,E=4204799,C=78)
	CT(7392)=(S=4204800,E=4205247,C=29)
	CT(7393)=(S=4205248,E=4205311,C=78)
	CT(7394)=(S=4205312,E=4206079,C=29)
	CT(7395)=(S=4206080,E=4206143,C=78)
	CT(7396)=(S=4206144,E=4207247,C=29)
	CT(7397)=(S=4207248,E=4207263,C=78)
	CT(7398)=(S=4207264,E=4208767,C=29)
	CT(7399)=(S=4208768,E=4208799,C=78)
	CT(7400)=(S=4208800,E=4208831,C=2)
	CT(7401)=(S=4208832,E=4208863,C=29)
	CT(7402)=(S=4208864,E=4208895,C=78)
	CT(7403)=(S=4208896,E=4209007,C=29)
	CT(7404)=(S=4209008,E=4209023,C=133)
	CT(7405)=(S=4209024,E=4209535,C=29)
	CT(7406)=(S=4209536,E=4209599,C=78)
	CT(7407)=(S=4209600,E=4210847,C=29)
	CT(7408)=(S=4210848,E=4210879,C=78)
	CT(7409)=(S=4210880,E=4210911,C=29)
	CT(7410)=(S=4210912,E=4210943,C=78)
	CT(7411)=(S=4210944,E=4211199,C=29)
	CT(7412)=(S=4211200,E=4211215,C=81)
	CT(7413)=(S=4211216,E=4211231,C=78)
	CT(7414)=(S=4211232,E=4211423,C=29)
	CT(7415)=(S=4211424,E=4211439,C=78)
	CT(7416)=(S=4211440,E=4211903,C=29)
	CT(7417)=(S=4211904,E=4211919,C=78)
	CT(7418)=(S=4211920,E=4212719,C=29)
	CT(7419)=(S=4212720,E=4212735,C=78)
	CT(7420)=(S=4212736,E=4212959,C=29)
	CT(7421)=(S=4212960,E=4212991,C=78)
	CT(7422)=(S=4212992,E=4216095,C=29)
	CT(7423)=(S=4216096,E=4216127,C=78)
	CT(7424)=(S=4216128,E=4216639,C=29)
	CT(7425)=(S=4216640,E=4216831,C=78)
	CT(7426)=(S=4216832,E=4217087,C=29)
	CT(7427)=(S=4217088,E=4217103,C=80)
	CT(7428)=(S=4217104,E=4217279,C=29)
	CT(7429)=(S=4217280,E=4217295,C=80)
	CT(7430)=(S=4217296,E=4217439,C=29)
	CT(7431)=(S=4217440,E=4217455,C=78)
	CT(7432)=(S=4217456,E=4218207,C=29)
	CT(7433)=(S=4218208,E=4218223,C=78)
	CT(7434)=(S=4218224,E=4218879,C=29)
	CT(7435)=(S=4218880,E=4219391,C=174)
	CT(7436)=(S=4219392,E=4219903,C=78)
	CT(7437)=(S=4219904,E=4222491,C=29)
	CT(7438)=(S=4222492,E=4222514,C=28)
	CT(7439)=(S=4222515,E=4222527,C=29)
	CT(7440)=(S=4222528,E=4222591,C=28)
	CT(7441)=(S=4222592,E=4222607,C=29)
	CT(7442)=(S=4222608,E=4222611,C=28)
	CT(7443)=(S=4222612,E=4222616,C=29)
	CT(7444)=(S=4222617,E=4222617,C=28)
	CT(7445)=(S=4222618,E=4222655,C=29)
	CT(7446)=(S=4222656,E=4222719,C=78)
	CT(7447)=(S=4222720,E=4223487,C=29)
	CT(7448)=(S=4223488,E=4223743,C=78)
	CT(7449)=(S=4223744,E=4224527,C=29)
	CT(7450)=(S=4224528,E=4224543,C=78)
	CT(7451)=(S=4224544,E=4224863,C=29)
	CT(7452)=(S=4224864,E=4224895,C=78)
	CT(7453)=(S=4224896,E=4224959,C=29)
	CT(7454)=(S=4224960,E=4224975,C=79)
	CT(7455)=(S=4224976,E=4224991,C=78)
	CT(7456)=(S=4224992,E=4227791,C=29)
	CT(7457)=(S=4227792,E=4227807,C=78)
	CT(7458)=(S=4227808,E=4229231,C=29)
	CT(7459)=(S=4229232,E=4229247,C=78)
	CT(7460)=(S=4229248,E=4229503,C=29)
	CT(7461)=(S=4229504,E=4229631,C=78)
	CT(7462)=(S=4229632,E=4230255,C=29)
	CT(7463)=(S=4230256,E=4230271,C=78)
	CT(7464)=(S=4230272,E=4230399,C=29)
	CT(7465)=(S=4230400,E=4230527,C=78)
	CT(7466)=(S=4230528,E=4232015,C=29)
	CT(7467)=(S=4232016,E=4232031,C=178)
	CT(7468)=(S=4232032,E=4232175,C=29)
	CT(7469)=(S=4232176,E=4232191,C=78)
	CT(7470)=(S=4232192,E=4232895,C=29)
	CT(7471)=(S=4232896,E=4232959,C=81)
	CT(7472)=(S=4232960,E=4233151,C=29)
	CT(7473)=(S=4233152,E=4233215,C=78)
	CT(7474)=(S=4233216,E=4239999,C=29)
	CT(7475)=(S=4240000,E=4240031,C=78)
	CT(7476)=(S=4240032,E=4240063,C=29)
	CT(7477)=(S=4240064,E=4240095,C=80)
	CT(7478)=(S=4240096,E=4240335,C=29)
	CT(7479)=(S=4240336,E=4240351,C=78)
	CT(7480)=(S=4240352,E=4240383,C=29)
	CT(7481)=(S=4240384,E=4240639,C=78)
	CT(7482)=(S=4240640,E=4241599,C=29)
	CT(7483)=(S=4241600,E=4241631,C=78)
	CT(7484)=(S=4241632,E=4241855,C=29)
	CT(7485)=(S=4241856,E=4241887,C=80)
	CT(7486)=(S=4241888,E=4242175,C=29)
	CT(7487)=(S=4242176,E=4242239,C=78)
	CT(7488)=(S=4242240,E=4242351,C=29)
	CT(7489)=(S=4242352,E=4242367,C=78)
	CT(7490)=(S=4242368,E=4242385,C=29)
	CT(7491)=(S=4242386,E=4242387,C=78)
	CT(7492)=(S=4242388,E=4245791,C=29)
	CT(7493)=(S=4245792,E=4245823,C=78)
	CT(7494)=(S=4245824,E=4245919,C=29)
	CT(7495)=(S=4245920,E=4245983,C=78)
	CT(7496)=(S=4245984,E=4246159,C=29)
	CT(7497)=(S=4246160,E=4246175,C=78)
	CT(7498)=(S=4246176,E=4252671,C=29)
	CT(7499)=(S=4252672,E=4253695,C=78)
	CT(7500)=(S=4253696,E=4254559,C=29)
	CT(7501)=(S=4254560,E=4254575,C=78)
	CT(7502)=(S=4254576,E=4254655,C=29)
	CT(7503)=(S=4254656,E=4254687,C=78)
	CT(7504)=(S=4254688,E=4255103,C=29)
	CT(7505)=(S=4255104,E=4255231,C=80)
	CT(7506)=(S=4255232,E=4257663,C=29)
	CT(7507)=(S=4257664,E=4257695,C=78)
	CT(7508)=(S=4257696,E=4257711,C=80)
	CT(7509)=(S=4257712,E=4258383,C=29)
	CT(7510)=(S=4258384,E=4258399,C=78)
	CT(7511)=(S=4258400,E=4258623,C=29)
	CT(7512)=(S=4258624,E=4258655,C=78)
	CT(7513)=(S=4258656,E=4259199,C=29)
	CT(7514)=(S=4259200,E=4259231,C=78)
	CT(7515)=(S=4259232,E=4259327,C=29)
	CT(7516)=(S=4259328,E=4259359,C=78)
	CT(7517)=(S=4259360,E=4259551,C=29)
	CT(7518)=(S=4259552,E=4259583,C=78)
	CT(7519)=(S=4259584,E=4265471,C=29)
	CT(7520)=(S=4265472,E=4265727,C=78)
	CT(7521)=(S=4265728,E=4265919,C=29)
	CT(7522)=(S=4265920,E=4265983,C=80)
	CT(7523)=(S=4265984,E=4269599,C=29)
	CT(7524)=(S=4269600,E=4269615,C=78)
	CT(7525)=(S=4269616,E=4269631,C=29)
	CT(7526)=(S=4269632,E=4269663,C=78)
	CT(7527)=(S=4269664,E=4269759,C=29)
	CT(7528)=(S=4269760,E=4269791,C=80)
	CT(7529)=(S=4269792,E=4269823,C=29)
	CT(7530)=(S=4269824,E=4269887,C=78)
	CT(7531)=(S=4269888,E=4272255,C=29)
	CT(7532)=(S=4272256,E=4272383,C=79)
	CT(7533)=(S=4272384,E=4275647,C=29)
	CT(7534)=(S=4275648,E=4275711,C=78)
	CT(7535)=(S=4275712,E=4279103,C=29)
	CT(7536)=(S=4279104,E=4279167,C=81)
	CT(7537)=(S=4279168,E=4282335,C=29)
	CT(7538)=(S=4282336,E=4282367,C=78)
	CT(7539)=(S=4282368,E=4283391,C=29)
	CT(7540)=(S=4283392,E=4284415,C=78)
	CT(7541)=(S=4284416,E=4284703,C=29)
	CT(7542)=(S=4284704,E=4284719,C=78)
	CT(7543)=(S=4284720,E=4284863,C=29)
	CT(7544)=(S=4284864,E=4284927,C=78)
	CT(7545)=(S=4284928,E=4287999,C=29)
	CT(7546)=(S=4288000,E=4288031,C=78)
	CT(7547)=(S=4288032,E=4288159,C=29)
	CT(7548)=(S=4288160,E=4288223,C=78)
	CT(7549)=(S=4288224,E=4288399,C=29)
	CT(7550)=(S=4288400,E=4288415,C=78)
	CT(7551)=(S=4288416,E=4306207,C=29)
	CT(7552)=(S=4306208,E=4306223,C=78)
	CT(7553)=(S=4306224,E=4306687,C=29)
	CT(7554)=(S=4306688,E=4306703,C=175)
	CT(7555)=(S=4306704,E=4325167,C=29)
	CT(7556)=(S=4325168,E=4325183,C=179)
	CT(7557)=(S=4325184,E=4325295,C=29)
	CT(7558)=(S=4325296,E=4325311,C=78)
	CT(7559)=(S=4325312,E=4325343,C=29)
	CT(7560)=(S=4325344,E=4325359,C=78)
	CT(7561)=(S=4325360,E=4326959,C=29)
	CT(7562)=(S=4326960,E=4326975,C=78)
	CT(7563)=(S=4326976,E=4327423,C=29)
	CT(7564)=(S=4327424,E=4327551,C=105)
	CT(7565)=(S=4327552,E=4328223,C=29)
	CT(7566)=(S=4328224,E=4328239,C=78)
	CT(7567)=(S=4328240,E=4328255,C=29)
	CT(7568)=(S=4328256,E=4328287,C=78)
	CT(7569)=(S=4328288,E=4328335,C=29)
	CT(7570)=(S=4328336,E=4328383,C=78)
	CT(7571)=(S=4328384,E=4330047,C=29)
	CT(7572)=(S=4330048,E=4330079,C=105)
	CT(7573)=(S=4330080,E=4330175,C=29)
	CT(7574)=(S=4330176,E=4330239,C=78)
	CT(7575)=(S=4330240,E=4334719,C=29)
	CT(7576)=(S=4334720,E=4334751,C=78)
	CT(7577)=(S=4334752,E=4335231,C=29)
	CT(7578)=(S=4335232,E=4335359,C=78)
	CT(7579)=(S=4335360,E=4336431,C=29)
	CT(7580)=(S=4336432,E=4336447,C=78)
	CT(7581)=(S=4336448,E=4337151,C=29)
	CT(7582)=(S=4337152,E=4337407,C=78)
	CT(7583)=(S=4337408,E=4338047,C=29)
	CT(7584)=(S=4338048,E=4338175,C=78)
	CT(7585)=(S=4338176,E=4338431,C=80)
	CT(7586)=(S=4338432,E=4338527,C=29)
	CT(7587)=(S=4338528,E=4338575,C=78)
	CT(7588)=(S=4338576,E=4338591,C=29)
	CT(7589)=(S=4338592,E=4338607,C=78)
	CT(7590)=(S=4338608,E=4338671,C=29)
	CT(7591)=(S=4338672,E=4338687,C=78)
	CT(7592)=(S=4338688,E=4339311,C=29)
	CT(7593)=(S=4339312,E=4339327,C=175)
	CT(7594)=(S=4339328,E=4339567,C=29)
	CT(7595)=(S=4339568,E=4339583,C=178)
	CT(7596)=(S=4339584,E=4340607,C=29)
	CT(7597)=(S=4340608,E=4340671,C=78)
	CT(7598)=(S=4340704,E=4340735,C=29)
	CT(7599)=(S=4340736,E=4340799,C=83)
	CT(7600)=(S=4340800,E=4345407,C=29)
	CT(7601)=(S=4345440,E=4345471,C=78)
	CT(7602)=(S=4345472,E=4345823,C=29)
	CT(7603)=(S=4345824,E=4345855,C=78)
	CT(7604)=(S=4345856,E=4347146,C=29)
	CT(7605)=(S=4347148,E=4347148,C=78)
	CT(7606)=(S=4347150,E=4347163,C=29)
	CT(7607)=(S=4347164,E=4347167,C=78)
	CT(7608)=(S=4347168,E=4347169,C=29)
	CT(7609)=(S=4347170,E=4347170,C=78)
	CT(7610)=(S=4347171,E=4347225,C=29)
	CT(7611)=(S=4347227,E=4347227,C=78)
	CT(7612)=(S=4347228,E=4347387,C=29)
	CT(7613)=(S=4347388,E=4347389,C=78)
	CT(7614)=(S=4347390,E=4349967,C=29)
	CT(7615)=(S=4349968,E=4349983,C=78)
	CT(7616)=(S=4349984,E=4350143,C=29)
	CT(7617)=(S=4350144,E=4350175,C=7)
	CT(7618)=(S=4350176,E=4350207,C=74)
	CT(7619)=(S=4350208,E=4350223,C=29)
	CT(7620)=(S=4350224,E=4350239,C=78)
	CT(7621)=(S=4350240,E=4350271,C=29)
	CT(7622)=(S=4350272,E=4350303,C=78)
	CT(7623)=(S=4350304,E=4350463,C=29)
	CT(7624)=(S=4350464,E=4350559,C=177)
	CT(7625)=(S=4350560,E=4351551,C=29)
	CT(7626)=(S=4351552,E=4351583,C=78)
	CT(7627)=(S=4351584,E=4351775,C=29)
	CT(7628)=(S=4351776,E=4351807,C=78)
	CT(7629)=(S=4351808,E=4351935,C=29)
	CT(7630)=(S=4351936,E=4351967,C=78)
	CT(7631)=(S=4351968,E=4353663,C=29)
	CT(7632)=(S=4353664,E=4353711,C=78)
	CT(7633)=(S=4353712,E=4354223,C=29)
	CT(7634)=(S=4354224,E=4354239,C=78)
	CT(7635)=(S=4354240,E=4354415,C=29)
	CT(7636)=(S=4354416,E=4354431,C=78)
	CT(7637)=(S=4354432,E=4355967,C=29)
	CT(7638)=(S=4355968,E=4355983,C=78)
	CT(7639)=(S=4355984,E=4355999,C=29)
	CT(7640)=(S=4356000,E=4356031,C=78)
	CT(7641)=(S=4356032,E=4358175,C=29)
	CT(7642)=(S=4358176,E=4358191,C=83)
	CT(7643)=(S=4358192,E=4358223,C=29)
	CT(7644)=(S=4358224,E=4358239,C=78)
	CT(7645)=(S=4358240,E=4358527,C=29)
	CT(7646)=(S=4358528,E=4358559,C=78)
	CT(7647)=(S=4358560,E=4358575,C=80)
	CT(7648)=(S=4358576,E=4358591,C=78)
	CT(7649)=(S=4358592,E=4358655,C=29)
	CT(7650)=(S=4358656,E=4359167,C=78)
	CT(7651)=(S=4359168,E=4359429,C=29)
	CT(7652)=(S=4359430,E=4359430,C=28)
	CT(7653)=(S=4359431,E=4359431,C=29)
	CT(7654)=(S=4359432,E=4359432,C=28)
	CT(7655)=(S=4359433,E=4359436,C=29)
	CT(7656)=(S=4359437,E=4359437,C=28)
	CT(7657)=(S=4359438,E=4359480,C=29)
	CT(7658)=(S=4359481,E=4359483,C=28)
	CT(7659)=(S=4359484,E=4360031,C=29)
	CT(7660)=(S=4360032,E=4360063,C=78)
	CT(7661)=(S=4360064,E=4362879,C=29)
	CT(7662)=(S=4362880,E=4362911,C=78)
	CT(7663)=(S=4362912,E=4364351,C=29)
	CT(7664)=(S=4364352,E=4364383,C=78)
	CT(7665)=(S=4364384,E=4364895,C=29)
	CT(7666)=(S=4364896,E=4364927,C=78)
	CT(7667)=(S=4364928,E=4365951,C=29)
	CT(7668)=(S=4365952,E=4365983,C=78)
	CT(7669)=(S=4365984,E=4366111,C=29)
	CT(7670)=(S=4366112,E=4366127,C=78)
	CT(7671)=(S=4366128,E=4366187,C=29)
	CT(7672)=(S=4366188,E=4366207,C=78)
	CT(7673)=(S=4366208,E=4367103,C=29)
	CT(7674)=(S=4367104,E=4367135,C=78)
	CT(7675)=(S=4367136,E=4367167,C=29)
	CT(7676)=(S=4367168,E=4367183,C=78)
	CT(7677)=(S=4367184,E=4367295,C=29)
	CT(7678)=(S=4367296,E=4367311,C=78)
	CT(7679)=(S=4367312,E=4367807,C=29)
	CT(7680)=(S=4367808,E=4367839,C=78)
	CT(7681)=(S=4367840,E=4369039,C=29)
	CT(7682)=(S=4369040,E=4369055,C=78)
	CT(7683)=(S=4369056,E=4369075,C=29)
	CT(7684)=(S=4369076,E=4369079,C=78)
	CT(7685)=(S=4369080,E=4369199,C=29)
	CT(7686)=(S=4369200,E=4369215,C=78)
	CT(7687)=(S=4369216,E=4369247,C=29)
	CT(7688)=(S=4369248,E=4369263,C=78)
	CT(7689)=(S=4369264,E=4369279,C=29)
	CT(7690)=(S=4369280,E=4369295,C=78)
	CT(7691)=(S=4369296,E=4369311,C=29)
	CT(7692)=(S=4369312,E=4369327,C=78)
	CT(7693)=(S=4369328,E=4371519,C=29)
	CT(7694)=(S=4371520,E=4371523,C=78)
	CT(7695)=(S=4371528,E=4371919,C=29)
	CT(7696)=(S=4371920,E=4371935,C=78)
	CT(7697)=(S=4371936,E=4372223,C=29)
	CT(7698)=(S=4372224,E=4372479,C=78)
	CT(7699)=(S=4372480,E=4372767,C=29)
	CT(7700)=(S=4372768,E=4372783,C=180)
	CT(7701)=(S=4372784,E=4372799,C=78)
	CT(7702)=(S=4372800,E=4372815,C=29)
	CT(7703)=(S=4372816,E=4372831,C=78)
	CT(7704)=(S=4372832,E=4372927,C=29)
	CT(7705)=(S=4372928,E=4372959,C=78)
	CT(7706)=(S=4372960,E=4373055,C=29)
	CT(7707)=(S=4373056,E=4373087,C=78)
	CT(7708)=(S=4373088,E=4373327,C=29)
	CT(7709)=(S=4373328,E=4373343,C=78)
	CT(7710)=(S=4373344,E=4373359,C=29)
	CT(7711)=(S=4373360,E=4373375,C=78)
	CT(7712)=(S=4373376,E=4375583,C=29)
	CT(7713)=(S=4375584,E=4375599,C=78)
	CT(7714)=(S=4375600,E=4376351,C=29)
	CT(7715)=(S=4376352,E=4376367,C=78)
	CT(7716)=(S=4376368,E=4377023,C=29)
	CT(7717)=(S=4377024,E=4377087,C=78)
	CT(7718)=(S=4377088,E=4377503,C=29)
	CT(7719)=(S=4377504,E=4377567,C=78)
	CT(7720)=(S=4377568,E=4378335,C=29)
	CT(7721)=(S=4378336,E=4378367,C=78)
	CT(7722)=(S=4378368,E=4378463,C=29)
	CT(7723)=(S=4378464,E=4378495,C=78)
	CT(7724)=(S=4378496,E=4378559,C=29)
	CT(7725)=(S=4378560,E=4378591,C=78)
	CT(7726)=(S=4378592,E=4378927,C=29)
	CT(7727)=(S=4378928,E=4378943,C=78)
	CT(7728)=(S=4378944,E=4379055,C=29)
	CT(7729)=(S=4379056,E=4379071,C=78)
	CT(7730)=(S=4379072,E=4379807,C=29)
	CT(7731)=(S=4379808,E=4379839,C=78)
	CT(7732)=(S=4379840,E=4379871,C=29)
	CT(7733)=(S=4379872,E=4379903,C=78)
	CT(7734)=(S=4379904,E=4381887,C=29)
	CT(7735)=(S=4381888,E=4381903,C=78)
	CT(7736)=(S=4381904,E=4382335,C=29)
	CT(7737)=(S=4382336,E=4382463,C=78)
	CT(7738)=(S=4382464,E=4383103,C=29)
	CT(7739)=(S=4383104,E=4383167,C=78)
	CT(7740)=(S=4383168,E=4383359,C=29)
	CT(7741)=(S=4383360,E=4383391,C=78)
	CT(7742)=(S=4383392,E=4383423,C=81)
	CT(7743)=(S=4383424,E=4384503,C=29)
	CT(7744)=(S=4384504,E=4384507,C=80)
	CT(7745)=(S=4384508,E=4384575,C=29)
	CT(7746)=(S=4384576,E=4384591,C=181)
	CT(7747)=(S=4384592,E=4384671,C=29)
	CT(7748)=(S=4384672,E=4384687,C=80)
	CT(7749)=(S=4384688,E=4385295,C=29)
	CT(7750)=(S=4385296,E=4385311,C=78)
	CT(7751)=(S=4385312,E=4387199,C=29)
	CT(7752)=(S=4387200,E=4387215,C=78)
	CT(7753)=(S=4387216,E=4387423,C=29)
	CT(7754)=(S=4387424,E=4387439,C=78)
	CT(7755)=(S=4387440,E=4388031,C=29)
	CT(7756)=(S=4388032,E=4388095,C=78)
	CT(7757)=(S=4388096,E=4389023,C=29)
	CT(7758)=(S=4389024,E=4389055,C=180)
	CT(7759)=(S=4389056,E=4389087,C=78)
	CT(7760)=(S=4389088,E=4389263,C=29)
	CT(7761)=(S=4389264,E=4389279,C=175)
	CT(7762)=(S=4389280,E=4390031,C=29)
	CT(7763)=(S=4390032,E=4390047,C=78)
	CT(7764)=(S=4390048,E=4390431,C=29)
	CT(7765)=(S=4390432,E=4390463,C=78)
	CT(7766)=(S=4390464,E=4396383,C=29)
	CT(7767)=(S=4396384,E=4396399,C=78)
	CT(7768)=(S=4396400,E=4396431,C=29)
	CT(7769)=(S=4396432,E=4396447,C=78)
	CT(7770)=(S=4396448,E=4396479,C=29)
	CT(7771)=(S=4396480,E=4396543,C=78)
	CT(7772)=(S=4396544,E=4396607,C=29)
	CT(7773)=(S=4396608,E=4396655,C=78)
	CT(7774)=(S=4396656,E=4396663,C=29)
	CT(7775)=(S=4396664,E=4396687,C=78)
	CT(7776)=(S=4396688,E=4396767,C=29)
	CT(7777)=(S=4396768,E=4396783,C=78)
	CT(7778)=(S=4396784,E=4396927,C=29)
	CT(7779)=(S=4396928,E=4396943,C=78)
	CT(7780)=(S=4396944,E=4402047,C=29)
	CT(7781)=(S=4402048,E=4402063,C=78)
	CT(7782)=(S=4402064,E=4402143,C=29)
	CT(7783)=(S=4402144,E=4402159,C=78)
	CT(7784)=(S=4402160,E=4404991,C=29)
	CT(7785)=(S=4404992,E=4405055,C=78)
	CT(7786)=(S=4405056,E=4405951,C=29)
	CT(7787)=(S=4405952,E=4405983,C=78)
	CT(7788)=(S=4405984,E=4408319,C=29)
	CT(7789)=(S=4408320,E=4409343,C=78)
	CT(7790)=(S=4409344,E=4431423,C=29)
	CT(7791)=(S=4431424,E=4431439,C=78)
	CT(7792)=(S=4431440,E=4431823,C=29)
	CT(7793)=(S=4431824,E=4431839,C=78)
	CT(7794)=(S=4431840,E=4440319,C=29)
	CT(7795)=(S=4440320,E=4440575,C=78)
	CT(7796)=(S=4440576,E=4442495,C=29)
	CT(7797)=(S=4442496,E=4442559,C=78)
	CT(7798)=(S=4442560,E=4443071,C=29)
	CT(7799)=(S=4443072,E=4443135,C=80)
	CT(7800)=(S=4443136,E=4443199,C=78)
	CT(7801)=(S=4443200,E=4443327,C=29)
	CT(7802)=(S=4443328,E=4443391,C=78)
	CT(7803)=(S=4443392,E=4443455,C=29)
	CT(7804)=(S=4443456,E=4443519,C=78)
	CT(7805)=(S=4443520,E=4443839,C=29)
	CT(7806)=(S=4443840,E=4443903,C=80)
	CT(7807)=(S=4443904,E=4444159,C=29)
	CT(7808)=(S=4444160,E=4444191,C=78)
	CT(7809)=(S=4444192,E=4444799,C=29)
	CT(7810)=(S=4444800,E=4444815,C=78)
	CT(7811)=(S=4444816,E=4444831,C=29)
	CT(7812)=(S=4444832,E=4444847,C=78)
	CT(7813)=(S=4444848,E=4444879,C=29)
	CT(7814)=(S=4444880,E=4444895,C=78)
	CT(7815)=(S=4444896,E=4444991,C=29)
	CT(7816)=(S=4444992,E=4445007,C=78)
	CT(7817)=(S=4445008,E=4445023,C=29)
	CT(7818)=(S=4445024,E=4445055,C=78)
	CT(7819)=(S=4445056,E=4445119,C=29)
	CT(7820)=(S=4445120,E=4445135,C=78)
	CT(7821)=(S=4445136,E=4445167,C=29)
	CT(7822)=(S=4445168,E=4445183,C=180)
	CT(7823)=(S=4445184,E=4445215,C=78)
	CT(7824)=(S=4445216,E=4445247,C=29)
	CT(7825)=(S=4445248,E=4445279,C=78)
	CT(7826)=(S=4445280,E=4445503,C=29)
	CT(7827)=(S=4445504,E=4445551,C=78)
	CT(7828)=(S=4445552,E=4445583,C=29)
	CT(7829)=(S=4445584,E=4445599,C=175)
	CT(7830)=(S=4445600,E=4445951,C=29)
	CT(7831)=(S=4445952,E=4445967,C=78)
	CT(7832)=(S=4445968,E=4445999,C=29)
	CT(7833)=(S=4446000,E=4446015,C=78)
	CT(7834)=(S=4446016,E=4446047,C=29)
	CT(7835)=(S=4446048,E=4446059,C=80)
	CT(7836)=(S=4446060,E=4446079,C=29)
	CT(7837)=(S=4446080,E=4446111,C=78)
	CT(7838)=(S=4446112,E=4446143,C=29)
	CT(7839)=(S=4446144,E=4446159,C=78)
	CT(7840)=(S=4446160,E=4447263,C=29)
	CT(7841)=(S=4447264,E=4447279,C=78)
	CT(7842)=(S=4447280,E=4447631,C=29)
	CT(7843)=(S=4447632,E=4447647,C=78)
	CT(7844)=(S=4447648,E=4448063,C=29)
	CT(7845)=(S=4448064,E=4448111,C=78)
	CT(7846)=(S=4448112,E=4448127,C=29)
	CT(7847)=(S=4448128,E=4448159,C=78)
	CT(7848)=(S=4448160,E=4448335,C=29)
	CT(7849)=(S=4448336,E=4448351,C=78)
	CT(7850)=(S=4448352,E=4448383,C=29)
	CT(7851)=(S=4448384,E=4448511,C=80)
	CT(7852)=(S=4448512,E=4448639,C=78)
	CT(7853)=(S=4448640,E=4448895,C=29)
	CT(7854)=(S=4448896,E=4448975,C=78)
	CT(7855)=(S=4448976,E=4448991,C=29)
	CT(7856)=(S=4448992,E=4449023,C=78)
	CT(7857)=(S=4449024,E=4449919,C=29)
	CT(7858)=(S=4449920,E=4449983,C=78)
	CT(7859)=(S=4449984,E=4450063,C=29)
	CT(7860)=(S=4450064,E=4450079,C=78)
	CT(7861)=(S=4450080,E=4450255,C=29)
	CT(7862)=(S=4450256,E=4450271,C=78)
	CT(7863)=(S=4450272,E=4473391,C=29)
	CT(7864)=(S=4473392,E=4473407,C=78)
	CT(7865)=(S=4473408,E=4473631,C=29)
	CT(7866)=(S=4473632,E=4473663,C=78)
	CT(7867)=(S=4473664,E=4473855,C=29)
	CT(7868)=(S=4473856,E=4473871,C=78)
	CT(7869)=(S=4473872,E=4474127,C=29)
	CT(7870)=(S=4474128,E=4474143,C=78)
	CT(7871)=(S=4474144,E=4474239,C=29)
	CT(7872)=(S=4474240,E=4474271,C=78)
	CT(7873)=(S=4474272,E=4474303,C=29)
	CT(7874)=(S=4474304,E=4474335,C=78)
	CT(7875)=(S=4474336,E=4474511,C=29)
	CT(7876)=(S=4474512,E=4474527,C=78)
	CT(7877)=(S=4474528,E=4474623,C=29)
	CT(7878)=(S=4474624,E=4474687,C=78)
	CT(7879)=(S=4474688,E=4493311,C=29)
	CT(7880)=(S=4493312,E=4495359,C=78)
	CT(7881)=(S=4495360,E=4499567,C=29)
	CT(7882)=(S=4499568,E=4499599,C=78)
	CT(7883)=(S=4499600,E=4500287,C=29)
	CT(7884)=(S=4500288,E=4500303,C=78)
	CT(7885)=(S=4500304,E=4500447,C=29)
	CT(7886)=(S=4500448,E=4500479,C=78)
	CT(7887)=(S=4500480,E=4502271,C=29)
	CT(7888)=(S=4502272,E=4502399,C=78)
	CT(7889)=(S=4502400,E=4503039,C=29)
	CT(7890)=(S=4503040,E=4503295,C=78)
	CT(7891)=(S=4503296,E=4515903,C=29)
	CT(7892)=(S=4515904,E=4515935,C=78)
	CT(7893)=(S=4515936,E=4516783,C=29)
	CT(7894)=(S=4516784,E=4516799,C=78)
	CT(7895)=(S=4516800,E=4523167,C=29)
	CT(7896)=(S=4523168,E=4523183,C=81)
	CT(7897)=(S=4523184,E=4523215,C=29)
	CT(7898)=(S=4523216,E=4523231,C=78)
	CT(7899)=(S=4523232,E=4524015,C=29)
	CT(7900)=(S=4524016,E=4524031,C=78)
	CT(7901)=(S=4524032,E=4524671,C=29)
	CT(7902)=(S=4524672,E=4524703,C=78)
	CT(7903)=(S=4524704,E=4524767,C=29)
	CT(7904)=(S=4524768,E=4524783,C=78)
	CT(7905)=(S=4524784,E=4524799,C=29)
	CT(7906)=(S=4524800,E=4524927,C=78)
	CT(7907)=(S=4524928,E=4525087,C=29)
	CT(7908)=(S=4525088,E=4525091,C=78)
	CT(7909)=(S=4525092,E=4525107,C=29)
	CT(7910)=(S=4525108,E=4525109,C=78)
	CT(7911)=(S=4525110,E=4525159,C=29)
	CT(7912)=(S=4525160,E=4525160,C=78)
	CT(7913)=(S=4525161,E=4525161,C=29)
	CT(7914)=(S=4525162,E=4525163,C=78)
	CT(7915)=(S=4525164,E=4525167,C=182)
	CT(7916)=(S=4525168,E=4526463,C=29)
	CT(7917)=(S=4526464,E=4526527,C=78)
	CT(7918)=(S=4526528,E=4526559,C=178)
	CT(7919)=(S=4526560,E=4526591,C=78)
	CT(7920)=(S=4526592,E=4527327,C=29)
	CT(7921)=(S=4527328,E=4527359,C=78)
	CT(7922)=(S=4527360,E=4528175,C=29)
	CT(7923)=(S=4528176,E=4528191,C=78)
	CT(7924)=(S=4528192,E=4528703,C=29)
	CT(7925)=(S=4528704,E=4528735,C=78)
	CT(7926)=(S=4528736,E=4528991,C=29)
	CT(7927)=(S=4528992,E=4529023,C=78)
	CT(7928)=(S=4529024,E=4530079,C=29)
	CT(7929)=(S=4530080,E=4530175,C=78)
	CT(7930)=(S=4530176,E=4531295,C=29)
	CT(7931)=(S=4531296,E=4531327,C=78)
	CT(7932)=(S=4531328,E=4532671,C=29)
	CT(7933)=(S=4532672,E=4532687,C=78)
	CT(7934)=(S=4532688,E=4532783,C=29)
	CT(7935)=(S=4532784,E=4532799,C=78)
	CT(7936)=(S=4532800,E=4532911,C=29)
	CT(7937)=(S=4532912,E=4532927,C=78)
	CT(7938)=(S=4532928,E=4533855,C=29)
	CT(7939)=(S=4533856,E=4533887,C=78)
	CT(7940)=(S=4533888,E=4533919,C=29)
	CT(7941)=(S=4533920,E=4533951,C=78)
	CT(7942)=(S=4533952,E=4534559,C=29)
	CT(7943)=(S=4534560,E=4534591,C=78)
	CT(7944)=(S=4534592,E=4534847,C=29)
	CT(7945)=(S=4534848,E=4534863,C=183)
	CT(7946)=(S=4534864,E=4534943,C=29)
	CT(7947)=(S=4534944,E=4534975,C=78)
	CT(7948)=(S=4534976,E=4535231,C=29)
	CT(7949)=(S=4535232,E=4535295,C=78)
	CT(7950)=(S=4535296,E=4536303,C=29)
	CT(7951)=(S=4536304,E=4536319,C=78)
	CT(7952)=(S=4536320,E=4536575,C=29)
	CT(7953)=(S=4536576,E=4536591,C=78)
	CT(7954)=(S=4536592,E=4536783,C=29)
	CT(7955)=(S=4536784,E=4536799,C=78)
	CT(7956)=(S=4536800,E=4536831,C=183)
	CT(7957)=(S=4536832,E=4536847,C=29)
	CT(7958)=(S=4536848,E=4536863,C=78)
	CT(7959)=(S=4536864,E=4536927,C=29)
	CT(7960)=(S=4536928,E=4536943,C=78)
	CT(7961)=(S=4536944,E=4537407,C=29)
	CT(7962)=(S=4537408,E=4537423,C=78)
	CT(7963)=(S=4537424,E=4537567,C=29)
	CT(7964)=(S=4537568,E=4537599,C=78)
	CT(7965)=(S=4537600,E=4538143,C=29)
	CT(7966)=(S=4538144,E=4538175,C=78)
	CT(7967)=(S=4538176,E=4538191,C=138)
	CT(7968)=(S=4538192,E=4538351,C=29)
	CT(7969)=(S=4538352,E=4538367,C=78)
	CT(7970)=(S=4538368,E=4539135,C=29)
	CT(7971)=(S=4539136,E=4539151,C=78)
	CT(7972)=(S=4539152,E=4539167,C=29)
	CT(7973)=(S=4539168,E=4539183,C=105)
	CT(7974)=(S=4539184,E=4539295,C=29)
	CT(7975)=(S=4539296,E=4539327,C=78)
	CT(7976)=(S=4539328,E=4539903,C=29)
	CT(7977)=(S=4539904,E=4540159,C=78)
	CT(7978)=(S=4540160,E=4540223,C=29)
	CT(7979)=(S=4540224,E=4540255,C=78)
	CT(7980)=(S=4540256,E=4540863,C=29)
	CT(7981)=(S=4540864,E=4540927,C=79)
	CT(7982)=(S=4540928,E=4541855,C=29)
	CT(7983)=(S=4541856,E=4541887,C=78)
	CT(7984)=(S=4541888,E=4542463,C=29)
	CT(7985)=(S=4542464,E=4542527,C=79)
	CT(7986)=(S=4542528,E=4544559,C=29)
	CT(7987)=(S=4544560,E=4544575,C=78)
	CT(7988)=(S=4544576,E=4561919,C=29)
	CT(7989)=(S=4561920,E=4562943,C=78)
	CT(7990)=(S=4562944,E=4563039,C=29)
	CT(7991)=(S=4563040,E=4563071,C=175)
	CT(7992)=(S=4563072,E=4563327,C=29)
	CT(7993)=(S=4563328,E=4563359,C=78)
	CT(7994)=(S=4563360,E=4564143,C=29)
	CT(7995)=(S=4564144,E=4564159,C=78)
	CT(7996)=(S=4564160,E=4564303,C=29)
	CT(7997)=(S=4564304,E=4564319,C=78)
	CT(7998)=(S=4564320,E=4564351,C=29)
	CT(7999)=(S=4564352,E=4564479,C=78)
	CT(8000)=(S=4564480,E=4564495,C=29)
	CT(8001)=(S=4564496,E=4564511,C=78)
	CT(8002)=(S=4564512,E=4564591,C=29)
	CT(8003)=(S=4564592,E=4564607,C=78)
	CT(8004)=(S=4564608,E=4565119,C=29)
	CT(8005)=(S=4565120,E=4565183,C=78)
	CT(8006)=(S=4565184,E=4565855,C=29)
	CT(8007)=(S=4565856,E=4565871,C=78)
	CT(8008)=(S=4565872,E=4565887,C=29)
	CT(8009)=(S=4565888,E=4565919,C=78)
	CT(8010)=(S=4565920,E=4566079,C=29)
	CT(8011)=(S=4566080,E=4566143,C=78)
	CT(8012)=(S=4566144,E=4566159,C=29)
	CT(8013)=(S=4566160,E=4566207,C=78)
	CT(8014)=(S=4566208,E=4566303,C=29)
	CT(8015)=(S=4566304,E=4566335,C=78)
	CT(8016)=(S=4566336,E=4567199,C=29)
	CT(8017)=(S=4567200,E=4567231,C=78)
	CT(8018)=(S=4567232,E=4572159,C=29)
	CT(8019)=(S=4572160,E=4572191,C=78)
	CT(8020)=(S=4572192,E=4572223,C=29)
	CT(8021)=(S=4572224,E=4572255,C=78)
	CT(8022)=(S=4572256,E=4572287,C=29)
	CT(8023)=(S=4572288,E=4572351,C=78)
	CT(8024)=(S=4572352,E=4593663,C=29)
	CT(8025)=(S=4593664,E=4595711,C=78)
	CT(8026)=(S=4595712,E=4596495,C=29)
	CT(8027)=(S=4596496,E=4596511,C=78)
	CT(8028)=(S=4596512,E=4596607,C=29)
	CT(8029)=(S=4596608,E=4596623,C=80)
	CT(8030)=(S=4596624,E=4596639,C=29)
	CT(8031)=(S=4596640,E=4596655,C=78)
	CT(8032)=(S=4596656,E=4596687,C=29)
	CT(8033)=(S=4596688,E=4596703,C=78)
	CT(8034)=(S=4596704,E=4596735,C=29)
	CT(8035)=(S=4596736,E=4596751,C=184)
	CT(8036)=(S=4596752,E=4596783,C=29)
	CT(8037)=(S=4596784,E=4596799,C=78)
	CT(8038)=(S=4596800,E=4597247,C=29)
	CT(8039)=(S=4597248,E=4597375,C=78)
	CT(8040)=(S=4597376,E=4599039,C=29)
	CT(8041)=(S=4599040,E=4599295,C=80)
	CT(8042)=(S=4599296,E=4599807,C=29)
	CT(8043)=(S=4599808,E=4601855,C=78)
	CT(8044)=(S=4601856,E=4603903,C=29)
	CT(8045)=(S=4603904,E=4609023,C=78)
	CT(8046)=(S=4609024,E=4654975,C=29)
	CT(8047)=(S=4654976,E=4655103,C=78)
	CT(8048)=(S=4655104,E=4657407,C=29)
	CT(8049)=(S=4657408,E=4657663,C=78)
	CT(8050)=(S=4657664,E=4657919,C=29)
	CT(8051)=(S=4657920,E=4658047,C=78)
	CT(8052)=(S=4658064,E=4658079,C=29)
	CT(8053)=(S=4658080,E=4658111,C=78)
	CT(8054)=(S=4658112,E=4658159,C=29)
	CT(8055)=(S=4658160,E=4658175,C=78)
	CT(8056)=(S=4658176,E=4718655,C=29)
	CT(8057)=(S=4718656,E=4718671,C=78)
	CT(8058)=(S=4718672,E=4718783,C=29)
	CT(8059)=(S=4718784,E=4718815,C=78)
	CT(8060)=(S=4718816,E=4718831,C=29)
	CT(8061)=(S=4718832,E=4718847,C=78)
	CT(8062)=(S=4718848,E=4719039,C=29)
	CT(8063)=(S=4719040,E=4719071,C=78)
	CT(8064)=(S=4719072,E=4719103,C=29)
	CT(8065)=(S=4719104,E=4719167,C=78)
	CT(8066)=(S=4719168,E=4721055,C=29)
	CT(8067)=(S=4721056,E=4721087,C=78)
	CT(8068)=(S=4721088,E=4721279,C=29)
	CT(8069)=(S=4721280,E=4721327,C=78)
	CT(8070)=(S=4721328,E=4721567,C=29)
	CT(8071)=(S=4721568,E=4721599,C=78)
	CT(8072)=(S=4721600,E=4721759,C=29)
	CT(8073)=(S=4721760,E=4721775,C=78)
	CT(8074)=(S=4721776,E=4721791,C=29)
	CT(8075)=(S=4721792,E=4721855,C=78)
	CT(8076)=(S=4721856,E=4722079,C=29)
	CT(8077)=(S=4722080,E=4722111,C=78)
	CT(8078)=(S=4722112,E=4722335,C=29)
	CT(8079)=(S=4722336,E=4722351,C=78)
	CT(8080)=(S=4722352,E=4722479,C=29)
	CT(8081)=(S=4722480,E=4722495,C=78)
	CT(8082)=(S=4722496,E=4722559,C=29)
	CT(8083)=(S=4722560,E=4722591,C=78)
	CT(8084)=(S=4722592,E=4724351,C=29)
	CT(8085)=(S=4724352,E=4724383,C=79)
	CT(8086)=(S=4724384,E=4725183,C=29)
	CT(8087)=(S=4725184,E=4725247,C=78)
	CT(8088)=(S=4725248,E=4725503,C=29)
	CT(8089)=(S=4725504,E=4725727,C=175)
	CT(8090)=(S=4725728,E=4725823,C=29)
	CT(8091)=(S=4725824,E=4725855,C=78)
	CT(8092)=(S=4725856,E=4726239,C=29)
	CT(8093)=(S=4726240,E=4726271,C=78)
	CT(8094)=(S=4726272,E=4727803,C=29)
	CT(8095)=(S=4727804,E=4727805,C=8)
	CT(8096)=(S=4727806,E=4728319,C=29)
	CT(8097)=(S=4728320,E=4728831,C=78)
	CT(8098)=(S=4728832,E=4730175,C=29)
	CT(8099)=(S=4730176,E=4730239,C=78)
	CT(8100)=(S=4730240,E=4731391,C=29)
	CT(8101)=(S=4731392,E=4731519,C=80)
	CT(8102)=(S=4731520,E=4731711,C=29)
	CT(8103)=(S=4731712,E=4731775,C=79)
	CT(8104)=(S=4731776,E=4732159,C=29)
	CT(8105)=(S=4732160,E=4732319,C=78)
	CT(8106)=(S=4732320,E=4732799,C=29)
	CT(8107)=(S=4732800,E=4732863,C=78)
	CT(8108)=(S=4732864,E=4753407,C=29)
	CT(8109)=(S=4753408,E=4755455,C=78)
	CT(8110)=(S=4755456,E=4762719,C=29)
	CT(8111)=(S=4762720,E=4762751,C=78)
	CT(8112)=(S=4762752,E=4762783,C=29)
	CT(8113)=(S=4762784,E=4762799,C=78)
	CT(8114)=(S=4762800,E=4850559,C=29)
	CT(8115)=(S=4850560,E=4850623,C=78)
	CT(8116)=(S=4850624,E=4852735,C=29)
	CT(8117)=(S=4852736,E=4853759,C=78)
	CT(8118)=(S=4853760,E=4861951,C=29)
	CT(8119)=(S=4861952,E=4862463,C=78)
	CT(8120)=(S=4862464,E=4862623,C=29)
	CT(8121)=(S=4862624,E=4862655,C=78)
	CT(8122)=(S=4862656,E=4862687,C=29)
	CT(8123)=(S=4862688,E=4862703,C=78)
	CT(8124)=(S=4862704,E=4862751,C=29)
	CT(8125)=(S=4862752,E=4862783,C=78)
	CT(8126)=(S=4862784,E=4863999,C=29)
	CT(8127)=(S=4864000,E=4865023,C=78)
	CT(8128)=(S=4865024,E=4870719,C=29)
	CT(8129)=(S=4870720,E=4870751,C=78)
	CT(8130)=(S=4870752,E=4870847,C=29)
	CT(8131)=(S=4870848,E=4870879,C=78)
	CT(8132)=(S=4870880,E=4878375,C=29)
	CT(8133)=(S=4878376,E=4878383,C=78)
	CT(8134)=(S=4878384,E=4878459,C=29)
	CT(8135)=(S=4878460,E=4878463,C=78)
	CT(8136)=(S=4878464,E=4878511,C=29)
	CT(8137)=(S=4878512,E=4878515,C=78)
	CT(8138)=(S=4878516,E=4878523,C=29)
	CT(8139)=(S=4878524,E=4878525,C=78)
	CT(8140)=(S=4878526,E=4878526,C=29)
	CT(8141)=(S=4878527,E=4878527,C=78)
	CT(8142)=(S=4878528,E=4878567,C=29)
	CT(8143)=(S=4878568,E=4878575,C=185)
	CT(8144)=(S=4878576,E=4878591,C=29)
	CT(8145)=(S=4878592,E=4878595,C=78)
	CT(8146)=(S=4878596,E=4878599,C=29)
	CT(8147)=(S=4878604,E=4878607,C=78)
	CT(8148)=(S=4878608,E=4878611,C=29)
	CT(8149)=(S=4878612,E=4878615,C=78)
	CT(8150)=(S=4878616,E=4878659,C=29)
	CT(8151)=(S=4878660,E=4878667,C=78)
	CT(8152)=(S=4878668,E=4878695,C=29)
	CT(8153)=(S=4878696,E=4878699,C=95)
	CT(8154)=(S=4878700,E=4878735,C=29)
	CT(8155)=(S=4878736,E=4878739,C=78)
	CT(8156)=(S=4878740,E=4878743,C=29)
	CT(8157)=(S=4878744,E=4878747,C=78)
	CT(8158)=(S=4878748,E=4878767,C=29)
	CT(8159)=(S=4878768,E=4878775,C=78)
	CT(8160)=(S=4878776,E=4878919,C=29)
	CT(8161)=(S=4878920,E=4878927,C=78)
	CT(8162)=(S=4878928,E=4878947,C=29)
	CT(8163)=(S=4878948,E=4878951,C=78)
	CT(8164)=(S=4878952,E=4878983,C=29)
	CT(8165)=(S=4878984,E=4878987,C=78)
	CT(8166)=(S=4878988,E=4879019,C=29)
	CT(8167)=(S=4879020,E=4879023,C=78)
	CT(8168)=(S=4879024,E=4879047,C=29)
	CT(8169)=(S=4879048,E=4879051,C=179)
	CT(8170)=(S=4879052,E=4879055,C=29)
	CT(8171)=(S=4879056,E=4879063,C=78)
	CT(8172)=(S=4879064,E=4879087,C=29)
	CT(8173)=(S=4879088,E=4879091,C=178)
	CT(8174)=(S=4879092,E=4879295,C=29)
	CT(8175)=(S=4879296,E=4879303,C=78)
	CT(8176)=(S=4879304,E=4879307,C=80)
	CT(8177)=(S=4879308,E=4879311,C=78)
	CT(8178)=(S=4879312,E=4879323,C=29)
	CT(8179)=(S=4879324,E=4879327,C=78)
	CT(8180)=(S=4879328,E=4879415,C=29)
	CT(8181)=(S=4879416,E=4879419,C=175)
	CT(8182)=(S=4879420,E=4879451,C=29)
	CT(8183)=(S=4879452,E=4879455,C=185)
	CT(8184)=(S=4879456,E=4879479,C=29)
	CT(8185)=(S=4879480,E=4879483,C=78)
	CT(8186)=(S=4879484,E=4879543,C=29)
	CT(8187)=(S=4879544,E=4879551,C=78)
	CT(8188)=(S=4879552,E=4879575,C=29)
	CT(8189)=(S=4879576,E=4879583,C=78)
	CT(8190)=(S=4879584,E=4879655,C=29)
	CT(8191)=(S=4879656,E=4879663,C=78)
	CT(8192)=(S=4879664,E=4879667,C=29)
	CT(8193)=(S=4879668,E=4879671,C=78)
	CT(8194)=(S=4879672,E=4879699,C=29)
	CT(8195)=(S=4879700,E=4879703,C=186)
	CT(8196)=(S=4879704,E=4879727,C=29)
	CT(8197)=(S=4879728,E=4879735,C=174)
	CT(8198)=(S=4879736,E=4879755,C=29)
	CT(8199)=(S=4879756,E=4879759,C=78)
	CT(8200)=(S=4879760,E=4879831,C=29)
	CT(8201)=(S=4879832,E=4879839,C=174)
	CT(8202)=(S=4879840,E=4879863,C=29)
	CT(8203)=(S=4879864,E=4879871,C=78)
	CT(8204)=(S=4879872,E=4879907,C=29)
	CT(8205)=(S=4879908,E=4879911,C=78)
	CT(8206)=(S=4879912,E=4879923,C=29)
	CT(8207)=(S=4879924,E=4879927,C=78)
	CT(8208)=(S=4879928,E=4879951,C=29)
	CT(8209)=(S=4879952,E=4879955,C=89)
	CT(8210)=(S=4879956,E=4879971,C=29)
	CT(8211)=(S=4879972,E=4879975,C=78)
	CT(8212)=(S=4879976,E=4880119,C=29)
	CT(8213)=(S=4880120,E=4880123,C=78)
	CT(8214)=(S=4880124,E=4880127,C=29)
	CT(8215)=(S=4880128,E=4880131,C=78)
	CT(8216)=(S=4880132,E=4880215,C=29)
	CT(8217)=(S=4880216,E=4880223,C=78)
	CT(8218)=(S=4880224,E=4880311,C=29)
	CT(8219)=(S=4880312,E=4880319,C=78)
	CT(8220)=(S=4880320,E=4880387,C=29)
	CT(8221)=(S=4880388,E=4880391,C=78)
	CT(8222)=(S=4880392,E=4880403,C=29)
	CT(8223)=(S=4880408,E=4880415,C=80)
	CT(8224)=(S=4880416,E=4880435,C=29)
	CT(8225)=(S=4880436,E=4880439,C=78)
	CT(8226)=(S=4880440,E=4880455,C=29)
	CT(8227)=(S=4880456,E=4880459,C=78)
	CT(8228)=(S=4880464,E=4880503,C=29)
	CT(8229)=(S=4880504,E=4880507,C=9)
	CT(8230)=(S=4880508,E=4880603,C=29)
	CT(8231)=(S=4880604,E=4880607,C=78)
	CT(8232)=(S=4880608,E=4880671,C=29)
	CT(8233)=(S=4880672,E=4880675,C=78)
	CT(8234)=(S=4880676,E=4880759,C=29)
	CT(8235)=(S=4880760,E=4880763,C=78)
	CT(8236)=(S=4880764,E=4880787,C=29)
	CT(8237)=(S=4880788,E=4880791,C=78)
	CT(8238)=(S=4880792,E=4880799,C=29)
	CT(8239)=(S=4880800,E=4880807,C=78)
	CT(8240)=(S=4880808,E=4880883,C=29)
	CT(8241)=(S=4880884,E=4880887,C=78)
	CT(8242)=(S=4880888,E=4880947,C=29)
	CT(8243)=(S=4880948,E=4880951,C=78)
	CT(8244)=(S=4880952,E=4880983,C=29)
	CT(8245)=(S=4880984,E=4880991,C=186)
	CT(8246)=(S=4880992,E=4881023,C=29)
	CT(8247)=(S=4881024,E=4881031,C=78)
	CT(8248)=(S=4881032,E=4881103,C=29)
	CT(8249)=(S=4881104,E=4881111,C=78)
	CT(8250)=(S=4881112,E=4881139,C=29)
	CT(8251)=(S=4881140,E=4881143,C=78)
	CT(8252)=(S=4881144,E=4881159,C=29)
	CT(8253)=(S=4881160,E=4881163,C=78)
	CT(8254)=(S=4881164,E=4881219,C=29)
	CT(8255)=(S=4881220,E=4881223,C=78)
	CT(8256)=(S=4881224,E=4881243,C=29)
	CT(8257)=(S=4881244,E=4881247,C=78)
	CT(8258)=(S=4881256,E=4882015,C=29)
	CT(8259)=(S=4882016,E=4882047,C=78)
	CT(8260)=(S=4882048,E=4882367,C=29)
	CT(8261)=(S=4882368,E=4882431,C=78)
	CT(8262)=(S=4882432,E=4900351,C=29)
	CT(8263)=(S=4900352,E=4900607,C=78)
	CT(8264)=(S=4900608,E=4900863,C=29)
	CT(8265)=(S=4900864,E=4900895,C=78)
	CT(8266)=(S=4900896,E=4902303,C=29)
	CT(8267)=(S=4902304,E=4902335,C=78)
	CT(8268)=(S=4902336,E=4902351,C=29)
	CT(8269)=(S=4902352,E=4902367,C=78)
	CT(8270)=(S=4902368,E=4902527,C=29)
	CT(8271)=(S=4902528,E=4902559,C=78)
	CT(8272)=(S=4902560,E=4903423,C=29)
	CT(8273)=(S=4903424,E=4903679,C=78)
	CT(8274)=(S=4903680,E=4904255,C=29)
	CT(8275)=(S=4904256,E=4904319,C=80)
	CT(8276)=(S=4904320,E=4904351,C=29)
	CT(8277)=(S=4904352,E=4904383,C=78)
	CT(8278)=(S=4904384,E=4904575,C=29)
	CT(8279)=(S=4904576,E=4904607,C=78)
	CT(8280)=(S=4904608,E=4904959,C=29)
	CT(8281)=(S=4904960,E=4905215,C=78)
	CT(8282)=(S=4905216,E=4906143,C=29)
	CT(8283)=(S=4906144,E=4906175,C=78)
	CT(8284)=(S=4906176,E=4906255,C=29)
	CT(8285)=(S=4906256,E=4906271,C=78)
	CT(8286)=(S=4906272,E=4906367,C=29)
	CT(8287)=(S=4906368,E=4906383,C=78)
	CT(8288)=(S=4906384,E=4906559,C=29)
	CT(8289)=(S=4906560,E=4906591,C=174)
	CT(8290)=(S=4906592,E=4940415,C=29)
	CT(8291)=(S=4940416,E=4940431,C=78)
	CT(8292)=(S=4940432,E=4940479,C=29)
	CT(8293)=(S=4940480,E=4940495,C=78)
	CT(8294)=(S=4940496,E=4945887,C=29)
	CT(8295)=(S=4945888,E=4945919,C=78)
	CT(8296)=(S=4945920,E=4954111,C=29)
	CT(8297)=(S=4954112,E=4956159,C=78)
	CT(8298)=(S=4956160,E=4982815,C=29)
	CT(8299)=(S=4982816,E=4982831,C=178)
	CT(8300)=(S=4982832,E=4982895,C=29)
	CT(8301)=(S=4982896,E=4982911,C=78)
	CT(8302)=(S=4982912,E=4983071,C=29)
	CT(8303)=(S=4983072,E=4983103,C=78)
	CT(8304)=(S=4983104,E=4983231,C=29)
	CT(8305)=(S=4983232,E=4983263,C=78)
	CT(8306)=(S=4983264,E=4983423,C=29)
	CT(8307)=(S=4983424,E=4983487,C=78)
	CT(8308)=(S=4983488,E=4983551,C=29)
	CT(8309)=(S=4983552,E=4983679,C=78)
	CT(8310)=(S=4983680,E=4997119,C=29)
	CT(8311)=(S=4997120,E=4999167,C=78)
	CT(8312)=(S=4999168,E=4999407,C=29)
	CT(8313)=(S=4999408,E=4999423,C=80)
	CT(8314)=(S=4999424,E=4999999,C=29)
	CT(8315)=(S=5000000,E=5000127,C=78)
	CT(8316)=(S=5000128,E=5000159,C=29)
	CT(8317)=(S=5000160,E=5000191,C=78)
	CT(8318)=(S=5000192,E=5000287,C=29)
	CT(8319)=(S=5000288,E=5000319,C=78)
	CT(8320)=(S=5000320,E=5000351,C=29)
	CT(8321)=(S=5000352,E=5000383,C=183)
	CT(8322)=(S=5000384,E=5000399,C=80)
	CT(8323)=(S=5000400,E=5000495,C=29)
	CT(8324)=(S=5000496,E=5000511,C=2)
	CT(8325)=(S=5000512,E=5000543,C=78)
	CT(8326)=(S=5000544,E=5046271,C=29)
	CT(8327)=(S=5046272,E=5050367,C=28)
	CT(8328)=(S=5050368,E=5051391,C=27)
	CT(8329)=(S=5051392,E=5052927,C=28)
	CT(8330)=(S=5052928,E=5053439,C=25)
	CT(8331)=(S=5053440,E=5053951,C=54)
	CT(8332)=(S=5053952,E=5054463,C=23)
	CT(8333)=(S=5054464,E=5054975,C=58)
	CT(8334)=(S=5054976,E=5055487,C=18)
	CT(8335)=(S=5055488,E=5055615,C=38)
	CT(8336)=(S=5055616,E=5055743,C=26)
	CT(8337)=(S=5055744,E=5055871,C=28)
	CT(8338)=(S=5055872,E=5055999,C=18)
	CT(8339)=(S=5056000,E=5056127,C=52)
	CT(8340)=(S=5056128,E=5056255,C=60)
	CT(8341)=(S=5056256,E=5056383,C=18)
	CT(8342)=(S=5056384,E=5056511,C=15)
	CT(8343)=(S=5056512,E=5056639,C=18)
	CT(8344)=(S=5056640,E=5056767,C=27)
	CT(8345)=(S=5056768,E=5057023,C=18)
	CT(8346)=(S=5057024,E=5057151,C=15)
	CT(8347)=(S=5057152,E=5057279,C=39)
	CT(8348)=(S=5057280,E=5057407,C=15)
	CT(8349)=(S=5057408,E=5057535,C=18)
	CT(8350)=(S=5057536,E=5057663,C=14)
	CT(8351)=(S=5057664,E=5057791,C=30)
	CT(8352)=(S=5057792,E=5057919,C=58)
	CT(8353)=(S=5057920,E=5058047,C=18)
	CT(8354)=(S=5058048,E=5058175,C=58)
	CT(8355)=(S=5058176,E=5058303,C=46)
	CT(8356)=(S=5058304,E=5058431,C=28)
	CT(8357)=(S=5058432,E=5058559,C=31)
	CT(8358)=(S=5058560,E=5058815,C=33)
	CT(8359)=(S=5058816,E=5059071,C=22)
	CT(8360)=(S=5059072,E=5059583,C=18)
	CT(8361)=(S=5059584,E=5059839,C=31)
	CT(8362)=(S=5059840,E=5060095,C=19)
	CT(8363)=(S=5060096,E=5060351,C=21)
	CT(8364)=(S=5060352,E=5060607,C=58)
	CT(8365)=(S=5060608,E=5061631,C=34)
	CT(8366)=(S=5061632,E=5062655,C=36)
	CT(8367)=(S=5062656,E=5062783,C=23)
	CT(8368)=(S=5062784,E=5062911,C=28)
	CT(8369)=(S=5062912,E=5063167,C=58)
	CT(8370)=(S=5063168,E=5063295,C=24)
	CT(8371)=(S=5063296,E=5063423,C=18)
	CT(8372)=(S=5063424,E=5063551,C=63)
	CT(8373)=(S=5063552,E=5063679,C=68)
	CT(8374)=(S=5063680,E=5063807,C=14)
	CT(8375)=(S=5063808,E=5063935,C=24)
	CT(8376)=(S=5063936,E=5064063,C=22)
	CT(8377)=(S=5064064,E=5064191,C=104)
	CT(8378)=(S=5064192,E=5064319,C=51)
	CT(8379)=(S=5064320,E=5064447,C=44)
	CT(8380)=(S=5064448,E=5064575,C=51)
	CT(8381)=(S=5064576,E=5064703,C=72)
	CT(8382)=(S=5064704,E=5064711,C=14)
	CT(8383)=(S=5064712,E=5064719,C=28)
	CT(8384)=(S=5064720,E=5064727,C=18)
	CT(8385)=(S=5064728,E=5064743,C=15)
	CT(8386)=(S=5064744,E=5064751,C=12)
	CT(8387)=(S=5064752,E=5064759,C=24)
	CT(8388)=(S=5064760,E=5064767,C=47)
	CT(8389)=(S=5064768,E=5064775,C=34)
	CT(8390)=(S=5064776,E=5064783,C=63)
	CT(8391)=(S=5064784,E=5064791,C=18)
	CT(8392)=(S=5064792,E=5064799,C=12)
	CT(8393)=(S=5064800,E=5064807,C=19)
	CT(8394)=(S=5064808,E=5064815,C=25)
	CT(8395)=(S=5064816,E=5064823,C=36)
	CT(8396)=(S=5064824,E=5064831,C=18)
	CT(8397)=(S=5064832,E=5064839,C=28)
	CT(8398)=(S=5064840,E=5064847,C=18)
	CT(8399)=(S=5064848,E=5064863,C=36)
	CT(8400)=(S=5064864,E=5064871,C=42)
	CT(8401)=(S=5064872,E=5064879,C=36)
	CT(8402)=(S=5064880,E=5064887,C=34)
	CT(8403)=(S=5064888,E=5064895,C=37)
	CT(8404)=(S=5064896,E=5064903,C=15)
	CT(8405)=(S=5064904,E=5064911,C=14)
	CT(8406)=(S=5064912,E=5064919,C=25)
	CT(8407)=(S=5064920,E=5064927,C=28)
	CT(8408)=(S=5064928,E=5064935,C=19)
	CT(8409)=(S=5064936,E=5064943,C=49)
	CT(8410)=(S=5064944,E=5064959,C=18)
	CT(8411)=(S=5064960,E=5064975,C=14)
	CT(8412)=(S=5064984,E=5064991,C=18)
	CT(8413)=(S=5065000,E=5065007,C=25)
	CT(8414)=(S=5065008,E=5065015,C=18)
	CT(8415)=(S=5065016,E=5065023,C=15)
	CT(8416)=(S=5065024,E=5065031,C=18)
	CT(8417)=(S=5065032,E=5065039,C=14)
	CT(8418)=(S=5065040,E=5065047,C=25)
	CT(8419)=(S=5065048,E=5065055,C=18)
	CT(8420)=(S=5065056,E=5065063,C=49)
	CT(8421)=(S=5065064,E=5065071,C=18)
	CT(8422)=(S=5065072,E=5065087,C=14)
	CT(8423)=(S=5065088,E=5065095,C=20)
	CT(8424)=(S=5065096,E=5065103,C=18)
	CT(8425)=(S=5065104,E=5065119,C=14)
	CT(8426)=(S=5065120,E=5065127,C=25)
	CT(8427)=(S=5065128,E=5065135,C=14)
	CT(8428)=(S=5065136,E=5065143,C=49)
	CT(8429)=(S=5065144,E=5065151,C=99)
	CT(8430)=(S=5065152,E=5065159,C=23)
	CT(8431)=(S=5065160,E=5065167,C=25)
	CT(8432)=(S=5065168,E=5065175,C=63)
	CT(8433)=(S=5065176,E=5065183,C=37)
	CT(8434)=(S=5065184,E=5065191,C=36)
	CT(8435)=(S=5065192,E=5065199,C=18)
	CT(8436)=(S=5065200,E=5065207,C=34)
	CT(8437)=(S=5065208,E=5065215,C=28)
	CT(8438)=(S=5065216,E=5065223,C=34)
	CT(8439)=(S=5065224,E=5065231,C=18)
	CT(8440)=(S=5065232,E=5065239,C=15)
	CT(8441)=(S=5065240,E=5065247,C=18)
	CT(8442)=(S=5065248,E=5065255,C=68)
	CT(8443)=(S=5065256,E=5065263,C=40)
	CT(8444)=(S=5065264,E=5065271,C=36)
	CT(8445)=(S=5065272,E=5065279,C=32)
	CT(8446)=(S=5065280,E=5065287,C=20)
	CT(8447)=(S=5065288,E=5065295,C=18)
	CT(8448)=(S=5065296,E=5065303,C=34)
	CT(8449)=(S=5065304,E=5065311,C=49)
	CT(8450)=(S=5065312,E=5065319,C=34)
	CT(8451)=(S=5065320,E=5065327,C=14)
	CT(8452)=(S=5065328,E=5065335,C=42)
	CT(8453)=(S=5065336,E=5065343,C=14)
	CT(8454)=(S=5065344,E=5065351,C=19)
	CT(8455)=(S=5065352,E=5065359,C=28)
	CT(8456)=(S=5065360,E=5065367,C=15)
	CT(8457)=(S=5065368,E=5065375,C=12)
	CT(8458)=(S=5065376,E=5065383,C=27)
	CT(8459)=(S=5065384,E=5065391,C=34)
	CT(8460)=(S=5065392,E=5065407,C=18)
	CT(8461)=(S=5065408,E=5065415,C=14)
	CT(8462)=(S=5065424,E=5065431,C=12)
	CT(8463)=(S=5065432,E=5065439,C=27)
	CT(8464)=(S=5065440,E=5065447,C=59)
	CT(8465)=(S=5065448,E=5065455,C=28)
	CT(8466)=(S=5065456,E=5065463,C=12)
	CT(8467)=(S=5065464,E=5065471,C=46)
	CT(8468)=(S=5065472,E=5065487,C=18)
	CT(8469)=(S=5065488,E=5065503,C=42)
	CT(8470)=(S=5065504,E=5065511,C=37)
	CT(8471)=(S=5065512,E=5065519,C=44)
	CT(8472)=(S=5065520,E=5065527,C=13)
	CT(8473)=(S=5065528,E=5065543,C=12)
	CT(8474)=(S=5065544,E=5065551,C=33)
	CT(8475)=(S=5065552,E=5065559,C=24)
	CT(8476)=(S=5065560,E=5065567,C=39)
	CT(8477)=(S=5065568,E=5065575,C=63)
	CT(8478)=(S=5065576,E=5065583,C=14)
	CT(8479)=(S=5065584,E=5065591,C=34)
	CT(8480)=(S=5065592,E=5065599,C=14)
	CT(8481)=(S=5065600,E=5065607,C=18)
	CT(8482)=(S=5065608,E=5065615,C=14)
	CT(8483)=(S=5065616,E=5065623,C=31)
	CT(8484)=(S=5065624,E=5065631,C=18)
	CT(8485)=(S=5065632,E=5065639,C=24)
	CT(8486)=(S=5065640,E=5065647,C=18)
	CT(8487)=(S=5065648,E=5065655,C=25)
	CT(8488)=(S=5065656,E=5065671,C=14)
	CT(8489)=(S=5065672,E=5065679,C=28)
	CT(8490)=(S=5065680,E=5065687,C=27)
	CT(8491)=(S=5065688,E=5065695,C=37)
	CT(8492)=(S=5065696,E=5065703,C=25)
	CT(8493)=(S=5065704,E=5065711,C=14)
	CT(8494)=(S=5065712,E=5065719,C=63)
	CT(8495)=(S=5065720,E=5065727,C=28)
	CT(8496)=(S=5065728,E=5065791,C=51)
	CT(8497)=(S=5065792,E=5065855,C=14)
	CT(8498)=(S=5065856,E=5065919,C=51)
	CT(8499)=(S=5065920,E=5065983,C=28)
	CT(8500)=(S=5065984,E=5066047,C=51)
	CT(8501)=(S=5066048,E=5066111,C=26)
	CT(8502)=(S=5066112,E=5066175,C=51)
	CT(8503)=(S=5066176,E=5066239,C=59)
	CT(8504)=(S=5066240,E=5066303,C=51)
	CT(8505)=(S=5066304,E=5066367,C=33)
	CT(8506)=(S=5066368,E=5066431,C=51)
	CT(8507)=(S=5066432,E=5066495,C=59)
	CT(8508)=(S=5066496,E=5066559,C=44)
	CT(8509)=(S=5066560,E=5066623,C=37)
	CT(8510)=(S=5066624,E=5066687,C=18)
	CT(8511)=(S=5066688,E=5066751,C=58)
	CT(8512)=(S=5066752,E=5066879,C=42)
	CT(8513)=(S=5066880,E=5067007,C=19)
	CT(8514)=(S=5067008,E=5067013,C=38)
	CT(8515)=(S=5067014,E=5067015,C=19)
	CT(8516)=(S=5067016,E=5067039,C=38)
	CT(8517)=(S=5067040,E=5067055,C=26)
	CT(8518)=(S=5067056,E=5067063,C=38)
	CT(8519)=(S=5067064,E=5067071,C=41)
	CT(8520)=(S=5067072,E=5067073,C=38)
	CT(8521)=(S=5067074,E=5067075,C=14)
	CT(8522)=(S=5067076,E=5067079,C=38)
	CT(8523)=(S=5067080,E=5067081,C=14)
	CT(8524)=(S=5067082,E=5067083,C=38)
	CT(8525)=(S=5067084,E=5067091,C=26)
	CT(8526)=(S=5067092,E=5067094,C=14)
	CT(8527)=(S=5067095,E=5067096,C=38)
	CT(8528)=(S=5067097,E=5067099,C=14)
	CT(8529)=(S=5067100,E=5067119,C=38)
	CT(8530)=(S=5067120,E=5067127,C=25)
	CT(8531)=(S=5067128,E=5067129,C=170)
	CT(8532)=(S=5067130,E=5067135,C=38)
	CT(8533)=(S=5067136,E=5067143,C=26)
	CT(8534)=(S=5067144,E=5067148,C=38)
	CT(8535)=(S=5067149,E=5067149,C=25)
	CT(8536)=(S=5067150,E=5067151,C=38)
	CT(8537)=(S=5067152,E=5067167,C=26)
	CT(8538)=(S=5067168,E=5067174,C=38)
	CT(8539)=(S=5067175,E=5067183,C=95)
	CT(8540)=(S=5067184,E=5067195,C=38)
	CT(8541)=(S=5067196,E=5067197,C=19)
	CT(8542)=(S=5067198,E=5067199,C=38)
	CT(8543)=(S=5067200,E=5067231,C=26)
	CT(8544)=(S=5067232,E=5067255,C=38)
	CT(8545)=(S=5067256,E=5067263,C=41)
	CT(8546)=(S=5067264,E=5067519,C=18)
	CT(8547)=(S=5067776,E=5068031,C=12)
	CT(8548)=(S=5068032,E=5068287,C=51)
	CT(8549)=(S=5068288,E=5068415,C=14)
	CT(8550)=(S=5068416,E=5068543,C=47)
	CT(8551)=(S=5068544,E=5068551,C=58)
	CT(8552)=(S=5068552,E=5068559,C=44)
	CT(8553)=(S=5068560,E=5068567,C=23)
	CT(8554)=(S=5068568,E=5068575,C=63)
	CT(8555)=(S=5068576,E=5068591,C=31)
	CT(8556)=(S=5068592,E=5068599,C=28)
	CT(8557)=(S=5068600,E=5068607,C=36)
	CT(8558)=(S=5068608,E=5068615,C=18)
	CT(8559)=(S=5068616,E=5068623,C=58)
	CT(8560)=(S=5068624,E=5068631,C=18)
	CT(8561)=(S=5068632,E=5068639,C=38)
	CT(8562)=(S=5068640,E=5068647,C=18)
	CT(8563)=(S=5068648,E=5068655,C=12)
	CT(8564)=(S=5068656,E=5068663,C=18)
	CT(8565)=(S=5068664,E=5068671,C=58)
	CT(8566)=(S=5068672,E=5068679,C=14)
	CT(8567)=(S=5068680,E=5068687,C=58)
	CT(8568)=(S=5068688,E=5068703,C=31)
	CT(8569)=(S=5068704,E=5068711,C=36)
	CT(8570)=(S=5068712,E=5068719,C=18)
	CT(8571)=(S=5068720,E=5068727,C=14)
	CT(8572)=(S=5068728,E=5068735,C=28)
	CT(8573)=(S=5068736,E=5068743,C=31)
	CT(8574)=(S=5068744,E=5068759,C=18)
	CT(8575)=(S=5068760,E=5068767,C=58)
	CT(8576)=(S=5068768,E=5068775,C=28)
	CT(8577)=(S=5068776,E=5068791,C=33)
	CT(8578)=(S=5068792,E=5068799,C=28)
	CT(8579)=(S=5068800,E=5068863,C=18)
	CT(8580)=(S=5068864,E=5068927,C=27)
	CT(8581)=(S=5068928,E=5068991,C=58)
	CT(8582)=(S=5068992,E=5069055,C=31)
	CT(8583)=(S=5069056,E=5069119,C=15)
	CT(8584)=(S=5069120,E=5069183,C=58)
	CT(8585)=(S=5069184,E=5069247,C=14)
	CT(8586)=(S=5069248,E=5069311,C=53)
	CT(8587)=(S=5069312,E=5069375,C=28)
	CT(8588)=(S=5069376,E=5069439,C=44)
	CT(8589)=(S=5069440,E=5069503,C=28)
	CT(8590)=(S=5069504,E=5069567,C=31)
	CT(8591)=(S=5069568,E=5069631,C=58)
	CT(8592)=(S=5069632,E=5069695,C=18)
	CT(8593)=(S=5069696,E=5069759,C=31)
	CT(8594)=(S=5069760,E=5069767,C=18)
	CT(8595)=(S=5069768,E=5069775,C=21)
	CT(8596)=(S=5069776,E=5069783,C=47)
	CT(8597)=(S=5069784,E=5069791,C=19)
	CT(8598)=(S=5069792,E=5069799,C=18)
	CT(8599)=(S=5069800,E=5069807,C=28)
	CT(8600)=(S=5069808,E=5069815,C=12)
	CT(8601)=(S=5069816,E=5069823,C=14)
	CT(8602)=(S=5069824,E=5069855,C=32)
	CT(8603)=(S=5069856,E=5069887,C=58)
	CT(8604)=(S=5069888,E=5069919,C=14)
	CT(8605)=(S=5069920,E=5069983,C=37)
	CT(8606)=(S=5069984,E=5070015,C=104)
	CT(8607)=(S=5070016,E=5070047,C=33)
	CT(8608)=(S=5070048,E=5070079,C=40)
	CT(8609)=(S=5070080,E=5070111,C=60)
	CT(8610)=(S=5070112,E=5070143,C=31)
	CT(8611)=(S=5070144,E=5070175,C=36)
	CT(8612)=(S=5070176,E=5070207,C=18)
	CT(8613)=(S=5070208,E=5070239,C=14)
	CT(8614)=(S=5070240,E=5070271,C=38)
	CT(8615)=(S=5070272,E=5070303,C=33)
	CT(8616)=(S=5070304,E=5070335,C=15)
	CT(8617)=(S=5070336,E=5070399,C=18)
	CT(8618)=(S=5070400,E=5070431,C=15)
	CT(8619)=(S=5070432,E=5070463,C=18)
	CT(8620)=(S=5070464,E=5070495,C=52)
	CT(8621)=(S=5070496,E=5070559,C=18)
	CT(8622)=(S=5070560,E=5070567,C=12)
	CT(8623)=(S=5070568,E=5070575,C=27)
	CT(8624)=(S=5070576,E=5070591,C=36)
	CT(8625)=(S=5070592,E=5070599,C=103)
	CT(8626)=(S=5070600,E=5070607,C=18)
	CT(8627)=(S=5070608,E=5070615,C=32)
	CT(8628)=(S=5070616,E=5070623,C=28)
	CT(8629)=(S=5070624,E=5070631,C=14)
	CT(8630)=(S=5070632,E=5070639,C=33)
	CT(8631)=(S=5070640,E=5070647,C=58)
	CT(8632)=(S=5070648,E=5070655,C=62)
	CT(8633)=(S=5070656,E=5070663,C=12)
	CT(8634)=(S=5070664,E=5070671,C=27)
	CT(8635)=(S=5070672,E=5070679,C=14)
	CT(8636)=(S=5070680,E=5070687,C=18)
	CT(8637)=(S=5070688,E=5070695,C=36)
	CT(8638)=(S=5070696,E=5070703,C=19)
	CT(8639)=(S=5070704,E=5070711,C=14)
	CT(8640)=(S=5070712,E=5070719,C=34)
	CT(8641)=(S=5070720,E=5070727,C=18)
	CT(8642)=(S=5070728,E=5070735,C=15)
	CT(8643)=(S=5070736,E=5070743,C=47)
	CT(8644)=(S=5070744,E=5070751,C=25)
	CT(8645)=(S=5070752,E=5070759,C=63)
	CT(8646)=(S=5070760,E=5070767,C=15)
	CT(8647)=(S=5070768,E=5070775,C=14)
	CT(8648)=(S=5070776,E=5070783,C=66)
	CT(8649)=(S=5070784,E=5070791,C=33)
	CT(8650)=(S=5070792,E=5070799,C=18)
	CT(8651)=(S=5070800,E=5070807,C=15)
	CT(8652)=(S=5070808,E=5070815,C=23)
	CT(8653)=(S=5070816,E=5070823,C=36)
	CT(8654)=(S=5070824,E=5070831,C=51)
	CT(8655)=(S=5070832,E=5070839,C=47)
	CT(8656)=(S=5070840,E=5070847,C=36)
	CT(8657)=(S=5070848,E=5072895,C=14)
	CT(8658)=(S=5072896,E=5072959,C=12)
	CT(8659)=(S=5072960,E=5073023,C=26)
	CT(8660)=(S=5073024,E=5073087,C=51)
	CT(8661)=(S=5073088,E=5073151,C=33)
	CT(8662)=(S=5073152,E=5073215,C=46)
	CT(8663)=(S=5073216,E=5073279,C=47)
	CT(8664)=(S=5073280,E=5073343,C=18)
	CT(8665)=(S=5073344,E=5073407,C=19)
	CT(8666)=(S=5073408,E=5073535,C=18)
	CT(8667)=(S=5073536,E=5073599,C=27)
	CT(8668)=(S=5073600,E=5073663,C=18)
	CT(8669)=(S=5073664,E=5073727,C=19)
	CT(8670)=(S=5073728,E=5073855,C=14)
	CT(8671)=(S=5073856,E=5073919,C=63)
	CT(8672)=(S=5073920,E=5073983,C=15)
	CT(8673)=(S=5073984,E=5074047,C=18)
	CT(8674)=(S=5074048,E=5074111,C=14)
	CT(8675)=(S=5074112,E=5074175,C=18)
	CT(8676)=(S=5074176,E=5074239,C=31)
	CT(8677)=(S=5074240,E=5074303,C=36)
	CT(8678)=(S=5074304,E=5074367,C=34)
	CT(8679)=(S=5074368,E=5074431,C=47)
	CT(8680)=(S=5074432,E=5074495,C=19)
	CT(8681)=(S=5074496,E=5074559,C=39)
	CT(8682)=(S=5074560,E=5074623,C=50)
	CT(8683)=(S=5074624,E=5074687,C=27)
	CT(8684)=(S=5074688,E=5074751,C=52)
	CT(8685)=(S=5074752,E=5074879,C=50)
	CT(8686)=(S=5074880,E=5074899,C=12)
	CT(8687)=(S=5074900,E=5074915,C=14)
	CT(8688)=(S=5074916,E=5074943,C=12)
	CT(8689)=(S=5074944,E=5075967,C=58)
	CT(8690)=(S=5075968,E=5076991,C=42)
	CT(8691)=(S=5076992,E=5078015,C=31)
	CT(8692)=(S=5078016,E=5079039,C=17)
	CT(8693)=(S=5079040,E=5087231,C=12)
	CT(8694)=(S=5087232,E=5091327,C=36)
	CT(8695)=(S=5091328,E=5095423,C=28)
	CT(8696)=(S=5095424,E=5099519,C=12)
	CT(8697)=(S=5099520,E=5100543,C=25)
	CT(8698)=(S=5100544,E=5101567,C=24)
	CT(8699)=(S=5101568,E=5102591,C=19)
	CT(8700)=(S=5102592,E=5102623,C=109)
	CT(8701)=(S=5102624,E=5102655,C=18)
	CT(8702)=(S=5102656,E=5102687,C=15)
	CT(8703)=(S=5102688,E=5102719,C=42)
	CT(8704)=(S=5102720,E=5102783,C=18)
	CT(8705)=(S=5102784,E=5102815,C=98)
	CT(8706)=(S=5102816,E=5102847,C=28)
	CT(8707)=(S=5102848,E=5102879,C=59)
	CT(8708)=(S=5102880,E=5102911,C=50)
	CT(8709)=(S=5102912,E=5102943,C=44)
	CT(8710)=(S=5102944,E=5102975,C=23)
	CT(8711)=(S=5102976,E=5103007,C=18)
	CT(8712)=(S=5103008,E=5103039,C=14)
	CT(8713)=(S=5103040,E=5103071,C=18)
	CT(8714)=(S=5103072,E=5103103,C=24)
	CT(8715)=(S=5103104,E=5103135,C=102)
	CT(8716)=(S=5103136,E=5103167,C=18)
	CT(8717)=(S=5103168,E=5103199,C=36)
	CT(8718)=(S=5103200,E=5103231,C=18)
	CT(8719)=(S=5103232,E=5103263,C=31)
	CT(8720)=(S=5103264,E=5103327,C=27)
	CT(8721)=(S=5103328,E=5103359,C=58)
	CT(8722)=(S=5103360,E=5103391,C=15)
	CT(8723)=(S=5103392,E=5103423,C=47)
	CT(8724)=(S=5103424,E=5103487,C=18)
	CT(8725)=(S=5103488,E=5103519,C=37)
	CT(8726)=(S=5103520,E=5103551,C=27)
	CT(8727)=(S=5103552,E=5103615,C=58)
	CT(8728)=(S=5103616,E=5105663,C=25)
	CT(8729)=(S=5105664,E=5105727,C=18)
	CT(8730)=(S=5105728,E=5105759,C=13)
	CT(8731)=(S=5105760,E=5105791,C=23)
	CT(8732)=(S=5105792,E=5105831,C=18)
	CT(8733)=(S=5105832,E=5105839,C=14)
	CT(8734)=(S=5105840,E=5105847,C=23)
	CT(8735)=(S=5105848,E=5105855,C=18)
	CT(8736)=(S=5105856,E=5105887,C=38)
	CT(8737)=(S=5105888,E=5105919,C=28)
	CT(8738)=(S=5105920,E=5105951,C=18)
	CT(8739)=(S=5105952,E=5105983,C=36)
	CT(8740)=(S=5105984,E=5106015,C=55)
	CT(8741)=(S=5106016,E=5106047,C=12)
	CT(8742)=(S=5106048,E=5106079,C=14)
	CT(8743)=(S=5106080,E=5106143,C=18)
	CT(8744)=(S=5106144,E=5106175,C=24)
	CT(8745)=(S=5106176,E=5106207,C=18)
	CT(8746)=(S=5106208,E=5106215,C=28)
	CT(8747)=(S=5106216,E=5106223,C=33)
	CT(8748)=(S=5106224,E=5106231,C=27)
	CT(8749)=(S=5106232,E=5106239,C=34)
	CT(8750)=(S=5106240,E=5106271,C=50)
	CT(8751)=(S=5106272,E=5106303,C=47)
	CT(8752)=(S=5106304,E=5106335,C=52)
	CT(8753)=(S=5106336,E=5106367,C=24)
	CT(8754)=(S=5106368,E=5106399,C=18)
	CT(8755)=(S=5106400,E=5106431,C=71)
	CT(8756)=(S=5106432,E=5106463,C=62)
	CT(8757)=(S=5106464,E=5106495,C=36)
	CT(8758)=(S=5106496,E=5106527,C=42)
	CT(8759)=(S=5106528,E=5106559,C=53)
	CT(8760)=(S=5106560,E=5106591,C=39)
	CT(8761)=(S=5106592,E=5106623,C=28)
	CT(8762)=(S=5106624,E=5106655,C=18)
	CT(8763)=(S=5106656,E=5106687,C=19)
	CT(8764)=(S=5106688,E=5106719,C=58)
	CT(8765)=(S=5106720,E=5106783,C=18)
	CT(8766)=(S=5106784,E=5106815,C=28)
	CT(8767)=(S=5106816,E=5106847,C=18)
	CT(8768)=(S=5106848,E=5106879,C=51)
	CT(8769)=(S=5106880,E=5106911,C=33)
	CT(8770)=(S=5106912,E=5106943,C=18)
	CT(8771)=(S=5106944,E=5106975,C=58)
	CT(8772)=(S=5106976,E=5107007,C=42)
	CT(8773)=(S=5107008,E=5107039,C=26)
	CT(8774)=(S=5107040,E=5107071,C=57)
	CT(8775)=(S=5107072,E=5107103,C=33)
	CT(8776)=(S=5107104,E=5107135,C=26)
	CT(8777)=(S=5107136,E=5107167,C=34)
	CT(8778)=(S=5107168,E=5107175,C=28)
	CT(8779)=(S=5107176,E=5107191,C=25)
	CT(8780)=(S=5107192,E=5107199,C=28)
	CT(8781)=(S=5107200,E=5107231,C=15)
	CT(8782)=(S=5107232,E=5107263,C=19)
	CT(8783)=(S=5107264,E=5107295,C=51)
	CT(8784)=(S=5107296,E=5107359,C=18)
	CT(8785)=(S=5107360,E=5107391,C=14)
	CT(8786)=(S=5107392,E=5107423,C=59)
	CT(8787)=(S=5107424,E=5107455,C=18)
	CT(8788)=(S=5107456,E=5107487,C=59)
	CT(8789)=(S=5107488,E=5107519,C=34)
	CT(8790)=(S=5107520,E=5107551,C=59)
	CT(8791)=(S=5107552,E=5107583,C=14)
	CT(8792)=(S=5107584,E=5107615,C=15)
	CT(8793)=(S=5107616,E=5107647,C=31)
	CT(8794)=(S=5107648,E=5107711,C=18)
	CT(8795)=(S=5107712,E=5107727,C=14)
	CT(8796)=(S=5107728,E=5107743,C=47)
	CT(8797)=(S=5107744,E=5107759,C=20)
	CT(8798)=(S=5107760,E=5107775,C=14)
	CT(8799)=(S=5107776,E=5107791,C=19)
	CT(8800)=(S=5107792,E=5107807,C=23)
	CT(8801)=(S=5107808,E=5107823,C=33)
	CT(8802)=(S=5107824,E=5107839,C=25)
	CT(8803)=(S=5107840,E=5107855,C=23)
	CT(8804)=(S=5107856,E=5107887,C=18)
	CT(8805)=(S=5107888,E=5107903,C=33)
	CT(8806)=(S=5107904,E=5107919,C=12)
	CT(8807)=(S=5107920,E=5107935,C=19)
	CT(8808)=(S=5107936,E=5107951,C=15)
	CT(8809)=(S=5107952,E=5107967,C=55)
	CT(8810)=(S=5107968,E=5107983,C=15)
	CT(8811)=(S=5107984,E=5108015,C=18)
	CT(8812)=(S=5108016,E=5108031,C=25)
	CT(8813)=(S=5108032,E=5108047,C=61)
	CT(8814)=(S=5108048,E=5108063,C=49)
	CT(8815)=(S=5108064,E=5108079,C=27)
	CT(8816)=(S=5108080,E=5108095,C=25)
	CT(8817)=(S=5108096,E=5108111,C=19)
	CT(8818)=(S=5108112,E=5108127,C=18)
	CT(8819)=(S=5108128,E=5108143,C=60)
	CT(8820)=(S=5108144,E=5108159,C=14)
	CT(8821)=(S=5108160,E=5108175,C=44)
	CT(8822)=(S=5108176,E=5108191,C=15)
	CT(8823)=(S=5108192,E=5108207,C=36)
	CT(8824)=(S=5108208,E=5108239,C=18)
	CT(8825)=(S=5108240,E=5108255,C=96)
	CT(8826)=(S=5108256,E=5108271,C=15)
	CT(8827)=(S=5108272,E=5108287,C=40)
	CT(8828)=(S=5108288,E=5108303,C=42)
	CT(8829)=(S=5108304,E=5108319,C=33)
	CT(8830)=(S=5108320,E=5108335,C=18)
	CT(8831)=(S=5108336,E=5108351,C=36)
	CT(8832)=(S=5108352,E=5108367,C=15)
	CT(8833)=(S=5108368,E=5108383,C=50)
	CT(8834)=(S=5108384,E=5108399,C=31)
	CT(8835)=(S=5108400,E=5108415,C=15)
	CT(8836)=(S=5108416,E=5108431,C=14)
	CT(8837)=(S=5108432,E=5108447,C=100)
	CT(8838)=(S=5108448,E=5108463,C=58)
	CT(8839)=(S=5108464,E=5108479,C=14)
	CT(8840)=(S=5108480,E=5108495,C=18)
	CT(8841)=(S=5108496,E=5108511,C=46)
	CT(8842)=(S=5108512,E=5108543,C=24)
	CT(8843)=(S=5108544,E=5108559,C=72)
	CT(8844)=(S=5108576,E=5108607,C=18)
	CT(8845)=(S=5108608,E=5108623,C=24)
	CT(8846)=(S=5108624,E=5108639,C=19)
	CT(8847)=(S=5108640,E=5108655,C=36)
	CT(8848)=(S=5108656,E=5108671,C=13)
	CT(8849)=(S=5108672,E=5108687,C=25)
	CT(8850)=(S=5108688,E=5108703,C=50)
	CT(8851)=(S=5108704,E=5108719,C=36)
	CT(8852)=(S=5108720,E=5108735,C=18)
	CT(8853)=(S=5108736,E=5108751,C=19)
	CT(8854)=(S=5108752,E=5108767,C=18)
	CT(8855)=(S=5108768,E=5108783,C=31)
	CT(8856)=(S=5108784,E=5108799,C=34)
	CT(8857)=(S=5108800,E=5108815,C=18)
	CT(8858)=(S=5108816,E=5108831,C=15)
	CT(8859)=(S=5108832,E=5108847,C=28)
	CT(8860)=(S=5108848,E=5108863,C=41)
	CT(8861)=(S=5108864,E=5108879,C=14)
	CT(8862)=(S=5108880,E=5108895,C=103)
	CT(8863)=(S=5108896,E=5108911,C=18)
	CT(8864)=(S=5108912,E=5108927,C=15)
	CT(8865)=(S=5108928,E=5108943,C=51)
	CT(8866)=(S=5108944,E=5108959,C=18)
	CT(8867)=(S=5108960,E=5108975,C=19)
	CT(8868)=(S=5108976,E=5108991,C=42)
	CT(8869)=(S=5108992,E=5109007,C=61)
	CT(8870)=(S=5109008,E=5109023,C=34)
	CT(8871)=(S=5109024,E=5109039,C=28)
	CT(8872)=(S=5109040,E=5109055,C=36)
	CT(8873)=(S=5109056,E=5109071,C=14)
	CT(8874)=(S=5109072,E=5109087,C=36)
	CT(8875)=(S=5109088,E=5109103,C=20)
	CT(8876)=(S=5109104,E=5109119,C=18)
	CT(8877)=(S=5109120,E=5109135,C=12)
	CT(8878)=(S=5109136,E=5109151,C=37)
	CT(8879)=(S=5109152,E=5109167,C=18)
	CT(8880)=(S=5109168,E=5109183,C=34)
	CT(8881)=(S=5109184,E=5109215,C=18)
	CT(8882)=(S=5109216,E=5109231,C=26)
	CT(8883)=(S=5109232,E=5109247,C=58)
	CT(8884)=(S=5109248,E=5109263,C=15)
	CT(8885)=(S=5109264,E=5109311,C=14)
	CT(8886)=(S=5109312,E=5109327,C=39)
	CT(8887)=(S=5109328,E=5109343,C=12)
	CT(8888)=(S=5109344,E=5109359,C=18)
	CT(8889)=(S=5109360,E=5109375,C=28)
	CT(8890)=(S=5109376,E=5109391,C=34)
	CT(8891)=(S=5109392,E=5109407,C=18)
	CT(8892)=(S=5109408,E=5109423,C=14)
	CT(8893)=(S=5109424,E=5109439,C=25)
	CT(8894)=(S=5109440,E=5109455,C=47)
	CT(8895)=(S=5109456,E=5109471,C=51)
	CT(8896)=(S=5109472,E=5109503,C=18)
	CT(8897)=(S=5109504,E=5109519,C=28)
	CT(8898)=(S=5109520,E=5109535,C=31)
	CT(8899)=(S=5109536,E=5109567,C=18)
	CT(8900)=(S=5109568,E=5109583,C=24)
	CT(8901)=(S=5109584,E=5109591,C=28)
	CT(8902)=(S=5109592,E=5109599,C=96)
	CT(8903)=(S=5109632,E=5109647,C=18)
	CT(8904)=(S=5109648,E=5109663,C=27)
	CT(8905)=(S=5109664,E=5109679,C=18)
	CT(8906)=(S=5109680,E=5109687,C=36)
	CT(8907)=(S=5109688,E=5109695,C=18)
	CT(8908)=(S=5109696,E=5109703,C=14)
	CT(8909)=(S=5109704,E=5109711,C=46)
	CT(8910)=(S=5109712,E=5109719,C=18)
	CT(8911)=(S=5109720,E=5109727,C=31)
	CT(8912)=(S=5109728,E=5109735,C=71)
	CT(8913)=(S=5109736,E=5109751,C=18)
	CT(8914)=(S=5109752,E=5109759,C=46)
	CT(8915)=(S=5109760,E=5110783,C=36)
	CT(8916)=(S=5110784,E=5111807,C=58)
	CT(8917)=(S=5111808,E=5112831,C=57)
	CT(8918)=(S=5112832,E=5113855,C=15)
	CT(8919)=(S=5113856,E=5114879,C=58)
	CT(8920)=(S=5114880,E=5115903,C=15)
	CT(8921)=(S=5115904,E=5116927,C=63)
	CT(8922)=(S=5116928,E=5117951,C=49)
	CT(8923)=(S=5117952,E=5117959,C=18)
	CT(8924)=(S=5117960,E=5117967,C=33)
	CT(8925)=(S=5117968,E=5117975,C=63)
	CT(8926)=(S=5117976,E=5117983,C=18)
	CT(8927)=(S=5117984,E=5117991,C=12)
	CT(8928)=(S=5117992,E=5117999,C=18)
	CT(8929)=(S=5118000,E=5118007,C=53)
	CT(8930)=(S=5118008,E=5118015,C=18)
	CT(8931)=(S=5118016,E=5118023,C=50)
	CT(8932)=(S=5118024,E=5118031,C=31)
	CT(8933)=(S=5118032,E=5118039,C=15)
	CT(8934)=(S=5118056,E=5118063,C=46)
	CT(8935)=(S=5118064,E=5118071,C=14)
	CT(8936)=(S=5118072,E=5118079,C=49)
	CT(8937)=(S=5118080,E=5118087,C=12)
	CT(8938)=(S=5118088,E=5118095,C=25)
	CT(8939)=(S=5118096,E=5118103,C=27)
	CT(8940)=(S=5118104,E=5118111,C=18)
	CT(8941)=(S=5118112,E=5118119,C=58)
	CT(8942)=(S=5118120,E=5118127,C=49)
	CT(8943)=(S=5118128,E=5118135,C=18)
	CT(8944)=(S=5118136,E=5118143,C=50)
	CT(8945)=(S=5118144,E=5118151,C=55)
	CT(8946)=(S=5118152,E=5118159,C=12)
	CT(8947)=(S=5118160,E=5118167,C=64)
	CT(8948)=(S=5118168,E=5118183,C=18)
	CT(8949)=(S=5118184,E=5118191,C=33)
	CT(8950)=(S=5118192,E=5118199,C=12)
	CT(8951)=(S=5118200,E=5118207,C=14)
	CT(8952)=(S=5118208,E=5118271,C=31)
	CT(8953)=(S=5118272,E=5118399,C=18)
	CT(8954)=(S=5118400,E=5118463,C=14)
	CT(8955)=(S=5118464,E=5118527,C=27)
	CT(8956)=(S=5118528,E=5118591,C=15)
	CT(8957)=(S=5118592,E=5118719,C=31)
	CT(8958)=(S=5118720,E=5118783,C=36)
	CT(8959)=(S=5118784,E=5118847,C=47)
	CT(8960)=(S=5118848,E=5118975,C=31)
	CT(8961)=(S=5118976,E=5119039,C=58)
	CT(8962)=(S=5119040,E=5119103,C=55)
	CT(8963)=(S=5119104,E=5119167,C=59)
	CT(8964)=(S=5119168,E=5119231,C=60)
	CT(8965)=(S=5119232,E=5119359,C=18)
	CT(8966)=(S=5119360,E=5119423,C=21)
	CT(8967)=(S=5119424,E=5119487,C=49)
	CT(8968)=(S=5119488,E=5119551,C=18)
	CT(8969)=(S=5119552,E=5119615,C=58)
	CT(8970)=(S=5119616,E=5119679,C=46)
	CT(8971)=(S=5119680,E=5119743,C=31)
	CT(8972)=(S=5119744,E=5119751,C=18)
	CT(8973)=(S=5119752,E=5119759,C=70)
	CT(8974)=(S=5119760,E=5119767,C=18)
	CT(8975)=(S=5119768,E=5119775,C=33)
	CT(8976)=(S=5119776,E=5119783,C=24)
	CT(8977)=(S=5119784,E=5119791,C=12)
	CT(8978)=(S=5119792,E=5119799,C=36)
	CT(8979)=(S=5119800,E=5119807,C=38)
	CT(8980)=(S=5119808,E=5119815,C=28)
	CT(8981)=(S=5119816,E=5119823,C=18)
	CT(8982)=(S=5119824,E=5119831,C=27)
	CT(8983)=(S=5119832,E=5119839,C=58)
	CT(8984)=(S=5119840,E=5119847,C=18)
	CT(8985)=(S=5119848,E=5119855,C=14)
	CT(8986)=(S=5119856,E=5119863,C=36)
	CT(8987)=(S=5119864,E=5119871,C=58)
	CT(8988)=(S=5119872,E=5119879,C=38)
	CT(8989)=(S=5119880,E=5119903,C=58)
	CT(8990)=(S=5119904,E=5119905,C=18)
	CT(8991)=(S=5119906,E=5119907,C=58)
	CT(8992)=(S=5119908,E=5119908,C=38)
	CT(8993)=(S=5119909,E=5119909,C=19)
	CT(8994)=(S=5119910,E=5119911,C=58)
	CT(8995)=(S=5119912,E=5119919,C=28)
	CT(8996)=(S=5119920,E=5119943,C=31)
	CT(8997)=(S=5119944,E=5119951,C=18)
	CT(8998)=(S=5119952,E=5119952,C=14)
	CT(8999)=(S=5119953,E=5119953,C=58)
	CT(9000)=(S=5119954,E=5119954,C=18)
	CT(9001)=(S=5119955,E=5119955,C=14)
	CT(9002)=(S=5119956,E=5119957,C=63)
	CT(9003)=(S=5119958,E=5119959,C=58)
	CT(9004)=(S=5119960,E=5119967,C=18)
	CT(9005)=(S=5119968,E=5119975,C=44)
	CT(9006)=(S=5119976,E=5119983,C=31)
	CT(9007)=(S=5119984,E=5119991,C=18)
	CT(9008)=(S=5119992,E=5119999,C=31)
	CT(9009)=(S=5120000,E=5120511,C=14)
	CT(9010)=(S=5120512,E=5121023,C=28)
	CT(9011)=(S=5121024,E=5121535,C=18)
	CT(9012)=(S=5121536,E=5122047,C=26)
	CT(9013)=(S=5122048,E=5122055,C=54)
	CT(9014)=(S=5122056,E=5122063,C=15)
	CT(9015)=(S=5122064,E=5122071,C=14)
	CT(9016)=(S=5122072,E=5122079,C=18)
	CT(9017)=(S=5122080,E=5122087,C=14)
	CT(9018)=(S=5122088,E=5122095,C=19)
	CT(9019)=(S=5122096,E=5122103,C=12)
	CT(9020)=(S=5122104,E=5122119,C=14)
	CT(9021)=(S=5122120,E=5122127,C=34)
	CT(9022)=(S=5122128,E=5122135,C=18)
	CT(9023)=(S=5122136,E=5122143,C=12)
	CT(9024)=(S=5122144,E=5122151,C=49)
	CT(9025)=(S=5122152,E=5122155,C=18)
	CT(9026)=(S=5122156,E=5122157,C=20)
	CT(9027)=(S=5122158,E=5122158,C=18)
	CT(9028)=(S=5122159,E=5122159,C=63)
	CT(9029)=(S=5122160,E=5122167,C=14)
	CT(9030)=(S=5122168,E=5122175,C=12)
	CT(9031)=(S=5122176,E=5122183,C=15)
	CT(9032)=(S=5122184,E=5122191,C=51)
	CT(9033)=(S=5122192,E=5122207,C=14)
	CT(9034)=(S=5122208,E=5122223,C=15)
	CT(9035)=(S=5122224,E=5122231,C=39)
	CT(9036)=(S=5122232,E=5122239,C=18)
	CT(9037)=(S=5122240,E=5122247,C=36)
	CT(9038)=(S=5122248,E=5122255,C=15)
	CT(9039)=(S=5122256,E=5122263,C=42)
	CT(9040)=(S=5122264,E=5122271,C=18)
	CT(9041)=(S=5122272,E=5122279,C=37)
	CT(9042)=(S=5122280,E=5122295,C=14)
	CT(9043)=(S=5122296,E=5122303,C=48)
	CT(9044)=(S=5122304,E=5122311,C=14)
	CT(9045)=(S=5122312,E=5122327,C=33)
	CT(9046)=(S=5122328,E=5122335,C=58)
	CT(9047)=(S=5122336,E=5122343,C=18)
	CT(9048)=(S=5122344,E=5122351,C=12)
	CT(9049)=(S=5122360,E=5122367,C=39)
	CT(9050)=(S=5122368,E=5122375,C=49)
	CT(9051)=(S=5122376,E=5122383,C=36)
	CT(9052)=(S=5122392,E=5122415,C=18)
	CT(9053)=(S=5122416,E=5122423,C=42)
	CT(9054)=(S=5122424,E=5122431,C=27)
	CT(9055)=(S=5122432,E=5122439,C=12)
	CT(9056)=(S=5122440,E=5122447,C=27)
	CT(9057)=(S=5122448,E=5122455,C=42)
	CT(9058)=(S=5122456,E=5122463,C=14)
	CT(9059)=(S=5122464,E=5122471,C=12)
	CT(9060)=(S=5122472,E=5122479,C=71)
	CT(9061)=(S=5122480,E=5122487,C=18)
	CT(9062)=(S=5122488,E=5122495,C=12)
	CT(9063)=(S=5122496,E=5122503,C=18)
	CT(9064)=(S=5122504,E=5122511,C=36)
	CT(9065)=(S=5122512,E=5122519,C=14)
	CT(9066)=(S=5122520,E=5122527,C=28)
	CT(9067)=(S=5122528,E=5122535,C=14)
	CT(9068)=(S=5122536,E=5122543,C=12)
	CT(9069)=(S=5122544,E=5122551,C=19)
	CT(9070)=(S=5122552,E=5122559,C=12)
	CT(9071)=(S=5122560,E=5123071,C=28)
	CT(9072)=(S=5123072,E=5123583,C=33)
	CT(9073)=(S=5123584,E=5126143,C=28)
	CT(9074)=(S=5126144,E=5128191,C=44)
	CT(9075)=(S=5128192,E=5132287,C=19)
	CT(9076)=(S=5132288,E=5132543,C=33)
	CT(9077)=(S=5132544,E=5132799,C=18)
	CT(9078)=(S=5132800,E=5133055,C=19)
	CT(9079)=(S=5133056,E=5133311,C=51)
	CT(9080)=(S=5133312,E=5133567,C=60)
	CT(9081)=(S=5133568,E=5133823,C=18)
	CT(9082)=(S=5133824,E=5134079,C=14)
	CT(9083)=(S=5134080,E=5134335,C=22)
	CT(9084)=(S=5134336,E=5134591,C=58)
	CT(9085)=(S=5134592,E=5134847,C=101)
	CT(9086)=(S=5134848,E=5135103,C=51)
	CT(9087)=(S=5135104,E=5135359,C=27)
	CT(9088)=(S=5135360,E=5135615,C=50)
	CT(9089)=(S=5135616,E=5135871,C=23)
	CT(9090)=(S=5135872,E=5136127,C=28)
	CT(9091)=(S=5136128,E=5136383,C=23)
	CT(9092)=(S=5136384,E=5136895,C=38)
	CT(9093)=(S=5136896,E=5137407,C=71)
	CT(9094)=(S=5137408,E=5137919,C=94)
	CT(9095)=(S=5137920,E=5138431,C=33)
	CT(9096)=(S=5138432,E=5138687,C=42)
	CT(9097)=(S=5138688,E=5138943,C=14)
	CT(9098)=(S=5138944,E=5139455,C=18)
	CT(9099)=(S=5139456,E=5139471,C=19)
	CT(9100)=(S=5139472,E=5139487,C=50)
	CT(9101)=(S=5139488,E=5139503,C=22)
	CT(9102)=(S=5139504,E=5139519,C=19)
	CT(9103)=(S=5139520,E=5139551,C=18)
	CT(9104)=(S=5139552,E=5139567,C=33)
	CT(9105)=(S=5139568,E=5139583,C=13)
	CT(9106)=(S=5139584,E=5139599,C=36)
	CT(9107)=(S=5139600,E=5139615,C=33)
	CT(9108)=(S=5139616,E=5139631,C=39)
	CT(9109)=(S=5139632,E=5139647,C=33)
	CT(9110)=(S=5139648,E=5139663,C=18)
	CT(9111)=(S=5139680,E=5139695,C=12)
	CT(9112)=(S=5139696,E=5139711,C=51)
	CT(9113)=(S=5139712,E=5139727,C=36)
	CT(9114)=(S=5139728,E=5139743,C=31)
	CT(9115)=(S=5139744,E=5139759,C=18)
	CT(9116)=(S=5139760,E=5139767,C=41)
	CT(9117)=(S=5139768,E=5139775,C=28)
	CT(9118)=(S=5139776,E=5139791,C=66)
	CT(9119)=(S=5139792,E=5139807,C=12)
	CT(9120)=(S=5139808,E=5139823,C=46)
	CT(9121)=(S=5139824,E=5139855,C=18)
	CT(9122)=(S=5139856,E=5139871,C=20)
	CT(9123)=(S=5139872,E=5139903,C=14)
	CT(9124)=(S=5139904,E=5139919,C=26)
	CT(9125)=(S=5139920,E=5139935,C=24)
	CT(9126)=(S=5139936,E=5139951,C=35)
	CT(9127)=(S=5139952,E=5139967,C=12)
	CT(9128)=(S=5139968,E=5139983,C=23)
	CT(9129)=(S=5139984,E=5139999,C=72)
	CT(9130)=(S=5140000,E=5140015,C=55)
	CT(9131)=(S=5140016,E=5140031,C=18)
	CT(9132)=(S=5140032,E=5140047,C=104)
	CT(9133)=(S=5140048,E=5140063,C=19)
	CT(9134)=(S=5140064,E=5140079,C=30)
	CT(9135)=(S=5140080,E=5140095,C=26)
	CT(9136)=(S=5140096,E=5140111,C=34)
	CT(9137)=(S=5140112,E=5140127,C=18)
	CT(9138)=(S=5140128,E=5140143,C=14)
	CT(9139)=(S=5140144,E=5140159,C=15)
	CT(9140)=(S=5140160,E=5140175,C=49)
	CT(9141)=(S=5140176,E=5140191,C=33)
	CT(9142)=(S=5140192,E=5140207,C=28)
	CT(9143)=(S=5140208,E=5140223,C=18)
	CT(9144)=(S=5140224,E=5140239,C=26)
	CT(9145)=(S=5140240,E=5140255,C=31)
	CT(9146)=(S=5140256,E=5140271,C=52)
	CT(9147)=(S=5140272,E=5140287,C=41)
	CT(9148)=(S=5140288,E=5140303,C=28)
	CT(9149)=(S=5140304,E=5140319,C=18)
	CT(9150)=(S=5140320,E=5140335,C=37)
	CT(9151)=(S=5140336,E=5140351,C=33)
	CT(9152)=(S=5140352,E=5140367,C=28)
	CT(9153)=(S=5140368,E=5140383,C=18)
	CT(9154)=(S=5140384,E=5140399,C=24)
	CT(9155)=(S=5140400,E=5140415,C=31)
	CT(9156)=(S=5140416,E=5140431,C=14)
	CT(9157)=(S=5140432,E=5140447,C=31)
	CT(9158)=(S=5140448,E=5140463,C=28)
	CT(9159)=(S=5140464,E=5140479,C=18)
	CT(9160)=(S=5140480,E=5144575,C=12)
	CT(9161)=(S=5144576,E=5144703,C=51)
	CT(9162)=(S=5144704,E=5144831,C=33)
	CT(9163)=(S=5144832,E=5144959,C=49)
	CT(9164)=(S=5144960,E=5145087,C=14)
	CT(9165)=(S=5145088,E=5145215,C=21)
	CT(9166)=(S=5145216,E=5145343,C=51)
	CT(9167)=(S=5145344,E=5145471,C=50)
	CT(9168)=(S=5145472,E=5145599,C=58)
	CT(9169)=(S=5145600,E=5145727,C=42)
	CT(9170)=(S=5145728,E=5145855,C=18)
	CT(9171)=(S=5145856,E=5145983,C=72)
	CT(9172)=(S=5145984,E=5146111,C=58)
	CT(9173)=(S=5146112,E=5146239,C=15)
	CT(9174)=(S=5146240,E=5146367,C=57)
	CT(9175)=(S=5146368,E=5146495,C=37)
	CT(9176)=(S=5146496,E=5146623,C=63)
	CT(9177)=(S=5146624,E=5146687,C=14)
	CT(9178)=(S=5146688,E=5146751,C=25)
	CT(9179)=(S=5146752,E=5146815,C=33)
	CT(9180)=(S=5146816,E=5146879,C=18)
	CT(9181)=(S=5146880,E=5146943,C=31)
	CT(9182)=(S=5146944,E=5146975,C=67)
	CT(9183)=(S=5146976,E=5146983,C=28)
	CT(9184)=(S=5146984,E=5146991,C=18)
	CT(9185)=(S=5146992,E=5147007,C=14)
	CT(9186)=(S=5147008,E=5147071,C=63)
	CT(9187)=(S=5147072,E=5147135,C=21)
	CT(9188)=(S=5147136,E=5147199,C=14)
	CT(9189)=(S=5147200,E=5147263,C=28)
	CT(9190)=(S=5147264,E=5147327,C=18)
	CT(9191)=(S=5147328,E=5147391,C=23)
	CT(9192)=(S=5147392,E=5147455,C=50)
	CT(9193)=(S=5147456,E=5147519,C=18)
	CT(9194)=(S=5147520,E=5147583,C=40)
	CT(9195)=(S=5147584,E=5147711,C=18)
	CT(9196)=(S=5147712,E=5147775,C=28)
	CT(9197)=(S=5147776,E=5147839,C=36)
	CT(9198)=(S=5147840,E=5147903,C=18)
	CT(9199)=(S=5147904,E=5147967,C=14)
	CT(9200)=(S=5147968,E=5148031,C=71)
	CT(9201)=(S=5148032,E=5148095,C=70)
	CT(9202)=(S=5148096,E=5148159,C=25)
	CT(9203)=(S=5148160,E=5148223,C=51)
	CT(9204)=(S=5148224,E=5148351,C=42)
	CT(9205)=(S=5148352,E=5148415,C=18)
	CT(9206)=(S=5148416,E=5148479,C=28)
	CT(9207)=(S=5148480,E=5148543,C=24)
	CT(9208)=(S=5148544,E=5148607,C=63)
	CT(9209)=(S=5148608,E=5150719,C=14)
	CT(9210)=(S=5150720,E=5150751,C=58)
	CT(9211)=(S=5150752,E=5150783,C=36)
	CT(9212)=(S=5150784,E=5150815,C=42)
	CT(9213)=(S=5150816,E=5150847,C=15)
	CT(9214)=(S=5150848,E=5150879,C=12)
	CT(9215)=(S=5150880,E=5150911,C=31)
	CT(9216)=(S=5150912,E=5150975,C=63)
	CT(9217)=(S=5150976,E=5151007,C=18)
	CT(9218)=(S=5151008,E=5151039,C=52)
	CT(9219)=(S=5151040,E=5151071,C=28)
	CT(9220)=(S=5151072,E=5151103,C=50)
	CT(9221)=(S=5151104,E=5151135,C=18)
	CT(9222)=(S=5151136,E=5151167,C=24)
	CT(9223)=(S=5151168,E=5151199,C=63)
	CT(9224)=(S=5151200,E=5151231,C=19)
	CT(9225)=(S=5151232,E=5151263,C=51)
	CT(9226)=(S=5151264,E=5151295,C=26)
	CT(9227)=(S=5151296,E=5151327,C=58)
	CT(9228)=(S=5151328,E=5151359,C=14)
	CT(9229)=(S=5151360,E=5151391,C=60)
	CT(9230)=(S=5151392,E=5151423,C=31)
	CT(9231)=(S=5151424,E=5151487,C=101)
	CT(9232)=(S=5151488,E=5151519,C=34)
	CT(9233)=(S=5151520,E=5151551,C=102)
	CT(9234)=(S=5151552,E=5151583,C=30)
	CT(9235)=(S=5151584,E=5151615,C=58)
	CT(9236)=(S=5151616,E=5151647,C=12)
	CT(9237)=(S=5151648,E=5151711,C=18)
	CT(9238)=(S=5151712,E=5151743,C=14)
	CT(9239)=(S=5151744,E=5151775,C=27)
	CT(9240)=(S=5151776,E=5151807,C=33)
	CT(9241)=(S=5151808,E=5151839,C=14)
	CT(9242)=(S=5151840,E=5151871,C=24)
	CT(9243)=(S=5151872,E=5151903,C=33)
	CT(9244)=(S=5151904,E=5151935,C=58)
	CT(9245)=(S=5151936,E=5151967,C=19)
	CT(9246)=(S=5151968,E=5151999,C=18)
	CT(9247)=(S=5152000,E=5152031,C=54)
	CT(9248)=(S=5152032,E=5152063,C=26)
	CT(9249)=(S=5152064,E=5152095,C=44)
	CT(9250)=(S=5152096,E=5152127,C=24)
	CT(9251)=(S=5152128,E=5152191,C=33)
	CT(9252)=(S=5152192,E=5152223,C=14)
	CT(9253)=(S=5152224,E=5152255,C=18)
	CT(9254)=(S=5152256,E=5152287,C=44)
	CT(9255)=(S=5152288,E=5152351,C=14)
	CT(9256)=(S=5152352,E=5152383,C=63)
	CT(9257)=(S=5152384,E=5152415,C=32)
	CT(9258)=(S=5152416,E=5152447,C=26)
	CT(9259)=(S=5152448,E=5152479,C=18)
	CT(9260)=(S=5152480,E=5152511,C=27)
	CT(9261)=(S=5152512,E=5152543,C=12)
	CT(9262)=(S=5152544,E=5152575,C=31)
	CT(9263)=(S=5152576,E=5152607,C=58)
	CT(9264)=(S=5152608,E=5152639,C=28)
	CT(9265)=(S=5152640,E=5152671,C=51)
	CT(9266)=(S=5152672,E=5152703,C=101)
	CT(9267)=(S=5152704,E=5152735,C=15)
	CT(9268)=(S=5152736,E=5152767,C=18)
	CT(9269)=(S=5152768,E=5160959,C=37)
	CT(9270)=(S=5160960,E=5177343,C=12)
	CT(9271)=(S=5177344,E=5193727,C=15)
	CT(9272)=(S=5193728,E=5197823,C=14)
	CT(9273)=(S=5197824,E=5201919,C=12)
	CT(9274)=(S=5201920,E=5202175,C=58)
	CT(9275)=(S=5202176,E=5202431,C=63)
	CT(9276)=(S=5202432,E=5202439,C=15)
	CT(9277)=(S=5202440,E=5202447,C=18)
	CT(9278)=(S=5202448,E=5202455,C=12)
	CT(9279)=(S=5202456,E=5202463,C=44)
	CT(9280)=(S=5202464,E=5202471,C=14)
	CT(9281)=(S=5202472,E=5202479,C=15)
	CT(9282)=(S=5202480,E=5202487,C=68)
	CT(9283)=(S=5202488,E=5202495,C=12)
	CT(9284)=(S=5202496,E=5202503,C=25)
	CT(9285)=(S=5202504,E=5202511,C=33)
	CT(9286)=(S=5202512,E=5202519,C=14)
	CT(9287)=(S=5202520,E=5202527,C=18)
	CT(9288)=(S=5202528,E=5202535,C=12)
	CT(9289)=(S=5202536,E=5202543,C=51)
	CT(9290)=(S=5202544,E=5202551,C=96)
	CT(9291)=(S=5202552,E=5202559,C=16)
	CT(9292)=(S=5202560,E=5202567,C=37)
	CT(9293)=(S=5202568,E=5202575,C=18)
	CT(9294)=(S=5202576,E=5202583,C=58)
	CT(9295)=(S=5202584,E=5202591,C=33)
	CT(9296)=(S=5202592,E=5202599,C=14)
	CT(9297)=(S=5202600,E=5202615,C=36)
	CT(9298)=(S=5202624,E=5202631,C=24)
	CT(9299)=(S=5202632,E=5202647,C=18)
	CT(9300)=(S=5202648,E=5202655,C=25)
	CT(9301)=(S=5202656,E=5202663,C=28)
	CT(9302)=(S=5202664,E=5202679,C=14)
	CT(9303)=(S=5202680,E=5202687,C=18)
	CT(9304)=(S=5202688,E=5202695,C=19)
	CT(9305)=(S=5202696,E=5202703,C=15)
	CT(9306)=(S=5202704,E=5202711,C=18)
	CT(9307)=(S=5202712,E=5202719,C=36)
	CT(9308)=(S=5202720,E=5202727,C=12)
	CT(9309)=(S=5202728,E=5202735,C=14)
	CT(9310)=(S=5202736,E=5202743,C=18)
	CT(9311)=(S=5202744,E=5202751,C=54)
	CT(9312)=(S=5202752,E=5202767,C=14)
	CT(9313)=(S=5202768,E=5202791,C=28)
	CT(9314)=(S=5202792,E=5202799,C=18)
	CT(9315)=(S=5202800,E=5202807,C=15)
	CT(9316)=(S=5202808,E=5202815,C=49)
	CT(9317)=(S=5202816,E=5202823,C=36)
	CT(9318)=(S=5202824,E=5202831,C=28)
	CT(9319)=(S=5202832,E=5202839,C=14)
	CT(9320)=(S=5202840,E=5202847,C=47)
	CT(9321)=(S=5202848,E=5202855,C=12)
	CT(9322)=(S=5202856,E=5202863,C=19)
	CT(9323)=(S=5202864,E=5202871,C=37)
	CT(9324)=(S=5202872,E=5202879,C=36)
	CT(9325)=(S=5202880,E=5202887,C=14)
	CT(9326)=(S=5202888,E=5202895,C=49)
	CT(9327)=(S=5202896,E=5202903,C=25)
	CT(9328)=(S=5202904,E=5202911,C=18)
	CT(9329)=(S=5202912,E=5202919,C=19)
	CT(9330)=(S=5202920,E=5202927,C=34)
	CT(9331)=(S=5202928,E=5202935,C=18)
	CT(9332)=(S=5202936,E=5202943,C=40)
	CT(9333)=(S=5202944,E=5203199,C=51)
	CT(9334)=(S=5203200,E=5203455,C=46)
	CT(9335)=(S=5203456,E=5203711,C=19)
	CT(9336)=(S=5203712,E=5203967,C=22)
	CT(9337)=(S=5203968,E=5204479,C=18)
	CT(9338)=(S=5204480,E=5204735,C=96)
	CT(9339)=(S=5204736,E=5204991,C=22)
	CT(9340)=(S=5204992,E=5205503,C=25)
	CT(9341)=(S=5205504,E=5205519,C=58)
	CT(9342)=(S=5205520,E=5205535,C=31)
	CT(9343)=(S=5205536,E=5205551,C=33)
	CT(9344)=(S=5205568,E=5205583,C=18)
	CT(9345)=(S=5205584,E=5205599,C=28)
	CT(9346)=(S=5205600,E=5205615,C=31)
	CT(9347)=(S=5205616,E=5205631,C=51)
	CT(9348)=(S=5205632,E=5205647,C=31)
	CT(9349)=(S=5205648,E=5205663,C=46)
	CT(9350)=(S=5205680,E=5205695,C=31)
	CT(9351)=(S=5205696,E=5205711,C=58)
	CT(9352)=(S=5205712,E=5205743,C=31)
	CT(9353)=(S=5205744,E=5206015,C=18)
	CT(9354)=(S=5206016,E=5208063,C=38)
	CT(9355)=(S=5208064,E=5208191,C=18)
	CT(9356)=(S=5208192,E=5208447,C=50)
	CT(9357)=(S=5208448,E=5208575,C=14)
	CT(9358)=(S=5208576,E=5208703,C=50)
	CT(9359)=(S=5208704,E=5208831,C=18)
	CT(9360)=(S=5208832,E=5208959,C=14)
	CT(9361)=(S=5208960,E=5209087,C=37)
	CT(9362)=(S=5209088,E=5209183,C=51)
	CT(9363)=(S=5209184,E=5209215,C=58)
	CT(9364)=(S=5209216,E=5209343,C=31)
	CT(9365)=(S=5209344,E=5209471,C=63)
	CT(9366)=(S=5209472,E=5209599,C=54)
	CT(9367)=(S=5209600,E=5209727,C=18)
	CT(9368)=(S=5209728,E=5209855,C=54)
	CT(9369)=(S=5209856,E=5209983,C=26)
	CT(9370)=(S=5209984,E=5210111,C=33)
	CT(9371)=(S=5210112,E=5211135,C=22)
	CT(9372)=(S=5211136,E=5211167,C=51)
	CT(9373)=(S=5211168,E=5211199,C=12)
	CT(9374)=(S=5211200,E=5211231,C=60)
	CT(9375)=(S=5211232,E=5211263,C=18)
	CT(9376)=(S=5211264,E=5211295,C=28)
	CT(9377)=(S=5211296,E=5211327,C=44)
	CT(9378)=(S=5211328,E=5211359,C=26)
	CT(9379)=(S=5211360,E=5211391,C=49)
	CT(9380)=(S=5211392,E=5211423,C=47)
	CT(9381)=(S=5211424,E=5211455,C=28)
	CT(9382)=(S=5211456,E=5211583,C=18)
	CT(9383)=(S=5211584,E=5211615,C=58)
	CT(9384)=(S=5211616,E=5211647,C=44)
	CT(9385)=(S=5211648,E=5211679,C=18)
	CT(9386)=(S=5211680,E=5211711,C=51)
	CT(9387)=(S=5211712,E=5211743,C=18)
	CT(9388)=(S=5211744,E=5211775,C=47)
	CT(9389)=(S=5211776,E=5211807,C=61)
	CT(9390)=(S=5211808,E=5211839,C=51)
	CT(9391)=(S=5211840,E=5211871,C=55)
	CT(9392)=(S=5211872,E=5211903,C=48)
	CT(9393)=(S=5211904,E=5211935,C=27)
	CT(9394)=(S=5211936,E=5211967,C=15)
	CT(9395)=(S=5211968,E=5211999,C=18)
	CT(9396)=(S=5212000,E=5212031,C=14)
	CT(9397)=(S=5212032,E=5212063,C=60)
	CT(9398)=(S=5212064,E=5212095,C=58)
	CT(9399)=(S=5212096,E=5212127,C=31)
	CT(9400)=(S=5212128,E=5212159,C=18)
	CT(9401)=(S=5212160,E=5212287,C=19)
	CT(9402)=(S=5212288,E=5212415,C=18)
	CT(9403)=(S=5212416,E=5212543,C=15)
	CT(9404)=(S=5212544,E=5212671,C=18)
	CT(9405)=(S=5212672,E=5212927,C=19)
	CT(9406)=(S=5212928,E=5213055,C=58)
	CT(9407)=(S=5213056,E=5213183,C=18)
	CT(9408)=(S=5213184,E=5213199,C=31)
	CT(9409)=(S=5213200,E=5213215,C=18)
	CT(9410)=(S=5213216,E=5213247,C=28)
	CT(9411)=(S=5213248,E=5213263,C=18)
	CT(9412)=(S=5213264,E=5213279,C=15)
	CT(9413)=(S=5213280,E=5213295,C=18)
	CT(9414)=(S=5213296,E=5213311,C=28)
	CT(9415)=(S=5213312,E=5213327,C=63)
	CT(9416)=(S=5213328,E=5213343,C=102)
	CT(9417)=(S=5213344,E=5213359,C=53)
	CT(9418)=(S=5213360,E=5213375,C=28)
	CT(9419)=(S=5213376,E=5213391,C=14)
	CT(9420)=(S=5213392,E=5213407,C=63)
	CT(9421)=(S=5213408,E=5213423,C=20)
	CT(9422)=(S=5213424,E=5213439,C=18)
	CT(9423)=(S=5213440,E=5213455,C=12)
	CT(9424)=(S=5213456,E=5213471,C=28)
	CT(9425)=(S=5213472,E=5213487,C=14)
	CT(9426)=(S=5213488,E=5213519,C=18)
	CT(9427)=(S=5213520,E=5213535,C=34)
	CT(9428)=(S=5213536,E=5213551,C=27)
	CT(9429)=(S=5213552,E=5213567,C=54)
	CT(9430)=(S=5213568,E=5213583,C=14)
	CT(9431)=(S=5213584,E=5213599,C=47)
	CT(9432)=(S=5213600,E=5213615,C=14)
	CT(9433)=(S=5213616,E=5213631,C=15)
	CT(9434)=(S=5213632,E=5213647,C=12)
	CT(9435)=(S=5213648,E=5213679,C=18)
	CT(9436)=(S=5213680,E=5213695,C=33)
	CT(9437)=(S=5213696,E=5213711,C=59)
	CT(9438)=(S=5213712,E=5213727,C=18)
	CT(9439)=(S=5213728,E=5213743,C=28)
	CT(9440)=(S=5213744,E=5213759,C=20)
	CT(9441)=(S=5213760,E=5213775,C=36)
	CT(9442)=(S=5213776,E=5213807,C=18)
	CT(9443)=(S=5213808,E=5213823,C=44)
	CT(9444)=(S=5213824,E=5213839,C=34)
	CT(9445)=(S=5213840,E=5213855,C=33)
	CT(9446)=(S=5213856,E=5213871,C=18)
	CT(9447)=(S=5213888,E=5213903,C=31)
	CT(9448)=(S=5213904,E=5213919,C=19)
	CT(9449)=(S=5213920,E=5213935,C=24)
	CT(9450)=(S=5213936,E=5213951,C=19)
	CT(9451)=(S=5213952,E=5213983,C=18)
	CT(9452)=(S=5213984,E=5213999,C=31)
	CT(9453)=(S=5214000,E=5214015,C=42)
	CT(9454)=(S=5214016,E=5214031,C=18)
	CT(9455)=(S=5214032,E=5214039,C=14)
	CT(9456)=(S=5214040,E=5214047,C=25)
	CT(9457)=(S=5214048,E=5214063,C=102)
	CT(9458)=(S=5214064,E=5214079,C=15)
	CT(9459)=(S=5214080,E=5214095,C=12)
	CT(9460)=(S=5214096,E=5214111,C=14)
	CT(9461)=(S=5214112,E=5214127,C=59)
	CT(9462)=(S=5214128,E=5214143,C=28)
	CT(9463)=(S=5214144,E=5214159,C=63)
	CT(9464)=(S=5214160,E=5214175,C=36)
	CT(9465)=(S=5214176,E=5214191,C=18)
	CT(9466)=(S=5214192,E=5214207,C=12)
	CT(9467)=(S=5214208,E=5218303,C=25)
	CT(9468)=(S=5218304,E=5218815,C=27)
	CT(9469)=(S=5218816,E=5219327,C=58)
	CT(9470)=(S=5219328,E=5219839,C=18)
	CT(9471)=(S=5219840,E=5220351,C=22)
	CT(9472)=(S=5220352,E=5220863,C=21)
	CT(9473)=(S=5220864,E=5220871,C=23)
	CT(9474)=(S=5220872,E=5220879,C=25)
	CT(9475)=(S=5220880,E=5220887,C=19)
	CT(9476)=(S=5220888,E=5220895,C=18)
	CT(9477)=(S=5220896,E=5220903,C=64)
	CT(9478)=(S=5220904,E=5220911,C=14)
	CT(9479)=(S=5220912,E=5220919,C=23)
	CT(9480)=(S=5220920,E=5220927,C=49)
	CT(9481)=(S=5220928,E=5220935,C=25)
	CT(9482)=(S=5220936,E=5220943,C=14)
	CT(9483)=(S=5220944,E=5220951,C=27)
	CT(9484)=(S=5220952,E=5220967,C=36)
	CT(9485)=(S=5220968,E=5220975,C=68)
	CT(9486)=(S=5220976,E=5220983,C=13)
	CT(9487)=(S=5220984,E=5220991,C=58)
	CT(9488)=(S=5220992,E=5220999,C=24)
	CT(9489)=(S=5221000,E=5221031,C=18)
	CT(9490)=(S=5221032,E=5221039,C=37)
	CT(9491)=(S=5221040,E=5221047,C=14)
	CT(9492)=(S=5221048,E=5221055,C=103)
	CT(9493)=(S=5221056,E=5221063,C=14)
	CT(9494)=(S=5221064,E=5221071,C=66)
	CT(9495)=(S=5221072,E=5221079,C=42)
	CT(9496)=(S=5221080,E=5221087,C=12)
	CT(9497)=(S=5221088,E=5221095,C=53)
	CT(9498)=(S=5221096,E=5221103,C=12)
	CT(9499)=(S=5221104,E=5221111,C=63)
	CT(9500)=(S=5221112,E=5221119,C=33)
	CT(9501)=(S=5221120,E=5221127,C=58)
	CT(9502)=(S=5221128,E=5221135,C=18)
	CT(9503)=(S=5221136,E=5221143,C=37)
	CT(9504)=(S=5221144,E=5221151,C=25)
	CT(9505)=(S=5221152,E=5221159,C=14)
	CT(9506)=(S=5221160,E=5221167,C=25)
	CT(9507)=(S=5221168,E=5221175,C=96)
	CT(9508)=(S=5221176,E=5221183,C=28)
	CT(9509)=(S=5221184,E=5221191,C=49)
	CT(9510)=(S=5221192,E=5221199,C=63)
	CT(9511)=(S=5221200,E=5221207,C=27)
	CT(9512)=(S=5221208,E=5221215,C=28)
	CT(9513)=(S=5221216,E=5221223,C=48)
	CT(9514)=(S=5221224,E=5221231,C=25)
	CT(9515)=(S=5221232,E=5221239,C=18)
	CT(9516)=(S=5221240,E=5221247,C=31)
	CT(9517)=(S=5221248,E=5221263,C=50)
	CT(9518)=(S=5221264,E=5221271,C=24)
	CT(9519)=(S=5221272,E=5221279,C=14)
	CT(9520)=(S=5221280,E=5221287,C=15)
	CT(9521)=(S=5221288,E=5221295,C=18)
	CT(9522)=(S=5221296,E=5221303,C=28)
	CT(9523)=(S=5221304,E=5221311,C=33)
	CT(9524)=(S=5221312,E=5221319,C=36)
	CT(9525)=(S=5221320,E=5221327,C=28)
	CT(9526)=(S=5221328,E=5221335,C=18)
	CT(9527)=(S=5221336,E=5221343,C=25)
	CT(9528)=(S=5221344,E=5221359,C=47)
	CT(9529)=(S=5221360,E=5221367,C=104)
	CT(9530)=(S=5221368,E=5221375,C=19)
	CT(9531)=(S=5221376,E=5221503,C=18)
	CT(9532)=(S=5221504,E=5221567,C=23)
	CT(9533)=(S=5221568,E=5221631,C=50)
	CT(9534)=(S=5221632,E=5221695,C=58)
	CT(9535)=(S=5221696,E=5221759,C=18)
	CT(9536)=(S=5221760,E=5221823,C=14)
	CT(9537)=(S=5221824,E=5221887,C=61)
	CT(9538)=(S=5221920,E=5221983,C=18)
	CT(9539)=(S=5221984,E=5222015,C=42)
	CT(9540)=(S=5222016,E=5222023,C=49)
	CT(9541)=(S=5222024,E=5222039,C=28)
	CT(9542)=(S=5222040,E=5222047,C=24)
	CT(9543)=(S=5222048,E=5222055,C=26)
	CT(9544)=(S=5222056,E=5222063,C=14)
	CT(9545)=(S=5222064,E=5222071,C=27)
	CT(9546)=(S=5222072,E=5222079,C=18)
	CT(9547)=(S=5222080,E=5222143,C=12)
	CT(9548)=(S=5222144,E=5222207,C=18)
	CT(9549)=(S=5222208,E=5222271,C=46)
	CT(9550)=(S=5222272,E=5222335,C=26)
	CT(9551)=(S=5222336,E=5222399,C=58)
	CT(9552)=(S=5222400,E=5224447,C=17)
	CT(9553)=(S=5224448,E=5226495,C=58)
	CT(9554)=(S=5226496,E=5242879,C=28)
	CT(9555)=(S=5242880,E=5244927,C=14)
	CT(9556)=(S=5244928,E=5246975,C=12)
	CT(9557)=(S=5246976,E=5249023,C=15)
	CT(9558)=(S=5249024,E=5253119,C=25)
	CT(9559)=(S=5253120,E=5255167,C=14)
	CT(9560)=(S=5255168,E=5257215,C=58)
	CT(9561)=(S=5257216,E=5257727,C=36)
	CT(9562)=(S=5257728,E=5258239,C=25)
	CT(9563)=(S=5258240,E=5258751,C=36)
	CT(9564)=(S=5258752,E=5259263,C=24)
	CT(9565)=(S=5259264,E=5259279,C=47)
	CT(9566)=(S=5259280,E=5259295,C=18)
	CT(9567)=(S=5259296,E=5259311,C=25)
	CT(9568)=(S=5259312,E=5259327,C=14)
	CT(9569)=(S=5259328,E=5259343,C=50)
	CT(9570)=(S=5259344,E=5259359,C=31)
	CT(9571)=(S=5259360,E=5259375,C=18)
	CT(9572)=(S=5259376,E=5259391,C=15)
	CT(9573)=(S=5259392,E=5259407,C=42)
	CT(9574)=(S=5259408,E=5259423,C=70)
	CT(9575)=(S=5259424,E=5259439,C=18)
	CT(9576)=(S=5259440,E=5259455,C=28)
	CT(9577)=(S=5259456,E=5259471,C=27)
	CT(9578)=(S=5259488,E=5259503,C=12)
	CT(9579)=(S=5259504,E=5259535,C=36)
	CT(9580)=(S=5259536,E=5259551,C=18)
	CT(9581)=(S=5259552,E=5259567,C=28)
	CT(9582)=(S=5259568,E=5259583,C=27)
	CT(9583)=(S=5259584,E=5259615,C=59)
	CT(9584)=(S=5259616,E=5259647,C=36)
	CT(9585)=(S=5259648,E=5259663,C=49)
	CT(9586)=(S=5259664,E=5259679,C=15)
	CT(9587)=(S=5259680,E=5259695,C=59)
	CT(9588)=(S=5259696,E=5259711,C=33)
	CT(9589)=(S=5259712,E=5259727,C=19)
	CT(9590)=(S=5259744,E=5259759,C=12)
	CT(9591)=(S=5259760,E=5259775,C=14)
	CT(9592)=(S=5259776,E=5259807,C=28)
	CT(9593)=(S=5259808,E=5259839,C=42)
	CT(9594)=(S=5259840,E=5259871,C=18)
	CT(9595)=(S=5259872,E=5259887,C=28)
	CT(9596)=(S=5259888,E=5259903,C=25)
	CT(9597)=(S=5259904,E=5259919,C=49)
	CT(9598)=(S=5259920,E=5259935,C=18)
	CT(9599)=(S=5259936,E=5259951,C=47)
	CT(9600)=(S=5259952,E=5259967,C=26)
	CT(9601)=(S=5259984,E=5259999,C=15)
	CT(9602)=(S=5260000,E=5260015,C=56)
	CT(9603)=(S=5260016,E=5260031,C=55)
	CT(9604)=(S=5260032,E=5260047,C=19)
	CT(9605)=(S=5260048,E=5260063,C=28)
	CT(9606)=(S=5260080,E=5260095,C=18)
	CT(9607)=(S=5260096,E=5260127,C=13)
	CT(9608)=(S=5260128,E=5260143,C=25)
	CT(9609)=(S=5260144,E=5260159,C=15)
	CT(9610)=(S=5260160,E=5260175,C=113)
	CT(9611)=(S=5260176,E=5260191,C=34)
	CT(9612)=(S=5260192,E=5260223,C=12)
	CT(9613)=(S=5260224,E=5260239,C=19)
	CT(9614)=(S=5260240,E=5260255,C=18)
	CT(9615)=(S=5260256,E=5260271,C=28)
	CT(9616)=(S=5260272,E=5260303,C=18)
	CT(9617)=(S=5260304,E=5260351,C=14)
	CT(9618)=(S=5260352,E=5260367,C=18)
	CT(9619)=(S=5260368,E=5260383,C=14)
	CT(9620)=(S=5260384,E=5260415,C=19)
	CT(9621)=(S=5260416,E=5260431,C=25)
	CT(9622)=(S=5260432,E=5260447,C=15)
	CT(9623)=(S=5260448,E=5260463,C=28)
	CT(9624)=(S=5260464,E=5260495,C=15)
	CT(9625)=(S=5260496,E=5260511,C=14)
	CT(9626)=(S=5260512,E=5260527,C=58)
	CT(9627)=(S=5260528,E=5260543,C=18)
	CT(9628)=(S=5260544,E=5260575,C=14)
	CT(9629)=(S=5260576,E=5260591,C=28)
	CT(9630)=(S=5260592,E=5260607,C=41)
	CT(9631)=(S=5260608,E=5260639,C=36)
	CT(9632)=(S=5260640,E=5260671,C=28)
	CT(9633)=(S=5260672,E=5260687,C=14)
	CT(9634)=(S=5260688,E=5260703,C=18)
	CT(9635)=(S=5260704,E=5260719,C=47)
	CT(9636)=(S=5260720,E=5260735,C=18)
	CT(9637)=(S=5260736,E=5260751,C=28)
	CT(9638)=(S=5260752,E=5260767,C=12)
	CT(9639)=(S=5260768,E=5260783,C=19)
	CT(9640)=(S=5260784,E=5260799,C=26)
	CT(9641)=(S=5260800,E=5260815,C=24)
	CT(9642)=(S=5260816,E=5260831,C=60)
	CT(9643)=(S=5260832,E=5260847,C=12)
	CT(9644)=(S=5260848,E=5260863,C=14)
	CT(9645)=(S=5260864,E=5260895,C=31)
	CT(9646)=(S=5260896,E=5260911,C=18)
	CT(9647)=(S=5260912,E=5260927,C=15)
	CT(9648)=(S=5260928,E=5260943,C=17)
	CT(9649)=(S=5260944,E=5260959,C=19)
	CT(9650)=(S=5260960,E=5261007,C=28)
	CT(9651)=(S=5261008,E=5261023,C=12)
	CT(9652)=(S=5261024,E=5261039,C=18)
	CT(9653)=(S=5261040,E=5261055,C=46)
	CT(9654)=(S=5261056,E=5261087,C=14)
	CT(9655)=(S=5261088,E=5261103,C=18)
	CT(9656)=(S=5261104,E=5261119,C=72)
	CT(9657)=(S=5261120,E=5261151,C=24)
	CT(9658)=(S=5261152,E=5261167,C=72)
	CT(9659)=(S=5261168,E=5261183,C=26)
	CT(9660)=(S=5261184,E=5261199,C=24)
	CT(9661)=(S=5261200,E=5261215,C=59)
	CT(9662)=(S=5261216,E=5261247,C=18)
	CT(9663)=(S=5261248,E=5261263,C=47)
	CT(9664)=(S=5261264,E=5261279,C=18)
	CT(9665)=(S=5261280,E=5261295,C=15)
	CT(9666)=(S=5261296,E=5261311,C=18)
	CT(9667)=(S=5261312,E=5261327,C=19)
	CT(9668)=(S=5261328,E=5261343,C=18)
	CT(9669)=(S=5261344,E=5261359,C=58)
	CT(9670)=(S=5261360,E=5261375,C=56)
	CT(9671)=(S=5261376,E=5261407,C=51)
	CT(9672)=(S=5261408,E=5261423,C=109)
	CT(9673)=(S=5261424,E=5261439,C=18)
	CT(9674)=(S=5261440,E=5261455,C=28)
	CT(9675)=(S=5261456,E=5261471,C=24)
	CT(9676)=(S=5261472,E=5261487,C=15)
	CT(9677)=(S=5261488,E=5261503,C=62)
	CT(9678)=(S=5261504,E=5261519,C=58)
	CT(9679)=(S=5261520,E=5261551,C=18)
	CT(9680)=(S=5261552,E=5261567,C=28)
	CT(9681)=(S=5261568,E=5261583,C=31)
	CT(9682)=(S=5261584,E=5261599,C=18)
	CT(9683)=(S=5261600,E=5261615,C=28)
	CT(9684)=(S=5261616,E=5261631,C=19)
	CT(9685)=(S=5261632,E=5261663,C=18)
	CT(9686)=(S=5261664,E=5261695,C=28)
	CT(9687)=(S=5261696,E=5261711,C=36)
	CT(9688)=(S=5261712,E=5261727,C=25)
	CT(9689)=(S=5261728,E=5261743,C=18)
	CT(9690)=(S=5261744,E=5261759,C=19)
	CT(9691)=(S=5261760,E=5261775,C=18)
	CT(9692)=(S=5261776,E=5261791,C=14)
	CT(9693)=(S=5261792,E=5261807,C=15)
	CT(9694)=(S=5261808,E=5261815,C=42)
	CT(9695)=(S=5261816,E=5261855,C=14)
	CT(9696)=(S=5261856,E=5261871,C=33)
	CT(9697)=(S=5261872,E=5261887,C=38)
	CT(9698)=(S=5261888,E=5261903,C=12)
	CT(9699)=(S=5261904,E=5261919,C=23)
	CT(9700)=(S=5261920,E=5261951,C=17)
	CT(9701)=(S=5261952,E=5261983,C=34)
	CT(9702)=(S=5261984,E=5261999,C=46)
	CT(9703)=(S=5262000,E=5262015,C=15)
	CT(9704)=(S=5262016,E=5262047,C=47)
	CT(9705)=(S=5262048,E=5262079,C=14)
	CT(9706)=(S=5262080,E=5262095,C=26)
	CT(9707)=(S=5262096,E=5262111,C=19)
	CT(9708)=(S=5262112,E=5262143,C=42)
	CT(9709)=(S=5262144,E=5262159,C=14)
	CT(9710)=(S=5262160,E=5262175,C=18)
	CT(9711)=(S=5262176,E=5262191,C=47)
	CT(9712)=(S=5262192,E=5262207,C=34)
	CT(9713)=(S=5262208,E=5262223,C=18)
	CT(9714)=(S=5262224,E=5262239,C=12)
	CT(9715)=(S=5262240,E=5262271,C=106)
	CT(9716)=(S=5262272,E=5262287,C=34)
	CT(9717)=(S=5262288,E=5262303,C=14)
	CT(9718)=(S=5262304,E=5262319,C=50)
	CT(9719)=(S=5262320,E=5262335,C=42)
	CT(9720)=(S=5262336,E=5262351,C=34)
	CT(9721)=(S=5262352,E=5262367,C=15)
	CT(9722)=(S=5262368,E=5262399,C=22)
	CT(9723)=(S=5262400,E=5262431,C=15)
	CT(9724)=(S=5262432,E=5262447,C=18)
	CT(9725)=(S=5262448,E=5262463,C=14)
	CT(9726)=(S=5262464,E=5262479,C=18)
	CT(9727)=(S=5262480,E=5262495,C=19)
	CT(9728)=(S=5262496,E=5262511,C=94)
	CT(9729)=(S=5262512,E=5262527,C=18)
	CT(9730)=(S=5262528,E=5262559,C=14)
	CT(9731)=(S=5262560,E=5262591,C=18)
	CT(9732)=(S=5262592,E=5262607,C=14)
	CT(9733)=(S=5262608,E=5262623,C=28)
	CT(9734)=(S=5262624,E=5262639,C=31)
	CT(9735)=(S=5262640,E=5262655,C=40)
	CT(9736)=(S=5262656,E=5262671,C=24)
	CT(9737)=(S=5262672,E=5262687,C=14)
	CT(9738)=(S=5262688,E=5262703,C=19)
	CT(9739)=(S=5262704,E=5262719,C=50)
	CT(9740)=(S=5262720,E=5262735,C=171)
	CT(9741)=(S=5262736,E=5262751,C=54)
	CT(9742)=(S=5262752,E=5262767,C=18)
	CT(9743)=(S=5262768,E=5262783,C=39)
	CT(9744)=(S=5262784,E=5262799,C=72)
	CT(9745)=(S=5262800,E=5262815,C=28)
	CT(9746)=(S=5262816,E=5262831,C=12)
	CT(9747)=(S=5262832,E=5262847,C=14)
	CT(9748)=(S=5262848,E=5262863,C=12)
	CT(9749)=(S=5262864,E=5262879,C=19)
	CT(9750)=(S=5262880,E=5262911,C=31)
	CT(9751)=(S=5262912,E=5262927,C=96)
	CT(9752)=(S=5262928,E=5262943,C=28)
	CT(9753)=(S=5262944,E=5262975,C=18)
	CT(9754)=(S=5262992,E=5263007,C=33)
	CT(9755)=(S=5263008,E=5263039,C=28)
	CT(9756)=(S=5263040,E=5263055,C=18)
	CT(9757)=(S=5263056,E=5263071,C=19)
	CT(9758)=(S=5263072,E=5263087,C=51)
	CT(9759)=(S=5263088,E=5263103,C=18)
	CT(9760)=(S=5263120,E=5263135,C=33)
	CT(9761)=(S=5263136,E=5263151,C=36)
	CT(9762)=(S=5263152,E=5263167,C=15)
	CT(9763)=(S=5263168,E=5263183,C=18)
	CT(9764)=(S=5263184,E=5263199,C=14)
	CT(9765)=(S=5263200,E=5263215,C=36)
	CT(9766)=(S=5263216,E=5263231,C=55)
	CT(9767)=(S=5263232,E=5263247,C=14)
	CT(9768)=(S=5263248,E=5263263,C=39)
	CT(9769)=(S=5263264,E=5263279,C=19)
	CT(9770)=(S=5263280,E=5263295,C=18)
	CT(9771)=(S=5263296,E=5263311,C=36)
	CT(9772)=(S=5263312,E=5263327,C=14)
	CT(9773)=(S=5263328,E=5263343,C=28)
	CT(9774)=(S=5263344,E=5263359,C=18)
	CT(9775)=(S=5263360,E=5263391,C=24)
	CT(9776)=(S=5263392,E=5263407,C=59)
	CT(9777)=(S=5263408,E=5263423,C=57)
	CT(9778)=(S=5263424,E=5263439,C=19)
	CT(9779)=(S=5263440,E=5263455,C=25)
	CT(9780)=(S=5263456,E=5263487,C=18)
	CT(9781)=(S=5263488,E=5263519,C=51)
	CT(9782)=(S=5263520,E=5263535,C=96)
	CT(9783)=(S=5263536,E=5263551,C=14)
	CT(9784)=(S=5263552,E=5263567,C=18)
	CT(9785)=(S=5263568,E=5263583,C=98)
	CT(9786)=(S=5263584,E=5263599,C=34)
	CT(9787)=(S=5263600,E=5263615,C=42)
	CT(9788)=(S=5263616,E=5263647,C=28)
	CT(9789)=(S=5263648,E=5263679,C=60)
	CT(9790)=(S=5263680,E=5263743,C=25)
	CT(9791)=(S=5263744,E=5263759,C=42)
	CT(9792)=(S=5263760,E=5263775,C=39)
	CT(9793)=(S=5263776,E=5263807,C=47)
	CT(9794)=(S=5263808,E=5263823,C=28)
	CT(9795)=(S=5263824,E=5263839,C=18)
	CT(9796)=(S=5263840,E=5263855,C=71)
	CT(9797)=(S=5263856,E=5263871,C=28)
	CT(9798)=(S=5263872,E=5263887,C=15)
	CT(9799)=(S=5263888,E=5263903,C=58)
	CT(9800)=(S=5263904,E=5263935,C=18)
	CT(9801)=(S=5263936,E=5263951,C=36)
	CT(9802)=(S=5263952,E=5263967,C=18)
	CT(9803)=(S=5263968,E=5263983,C=24)
	CT(9804)=(S=5263984,E=5264015,C=14)
	CT(9805)=(S=5264016,E=5264031,C=33)
	CT(9806)=(S=5264032,E=5264063,C=18)
	CT(9807)=(S=5264064,E=5264071,C=31)
	CT(9808)=(S=5264072,E=5264095,C=28)
	CT(9809)=(S=5264096,E=5264111,C=12)
	CT(9810)=(S=5264112,E=5264127,C=14)
	CT(9811)=(S=5264128,E=5264143,C=47)
	CT(9812)=(S=5264144,E=5264159,C=39)
	CT(9813)=(S=5264160,E=5264191,C=34)
	CT(9814)=(S=5264192,E=5264207,C=33)
	CT(9815)=(S=5264208,E=5264223,C=19)
	CT(9816)=(S=5264224,E=5264255,C=28)
	CT(9817)=(S=5264256,E=5264271,C=40)
	CT(9818)=(S=5264272,E=5264287,C=14)
	CT(9819)=(S=5264288,E=5264303,C=15)
	CT(9820)=(S=5264304,E=5264319,C=28)
	CT(9821)=(S=5264320,E=5264335,C=18)
	CT(9822)=(S=5264336,E=5264351,C=19)
	CT(9823)=(S=5264352,E=5264383,C=18)
	CT(9824)=(S=5264384,E=5264399,C=28)
	CT(9825)=(S=5264400,E=5264415,C=13)
	CT(9826)=(S=5264416,E=5264431,C=19)
	CT(9827)=(S=5264432,E=5264479,C=14)
	CT(9828)=(S=5264480,E=5264495,C=15)
	CT(9829)=(S=5264496,E=5264511,C=18)
	CT(9830)=(S=5264512,E=5264543,C=25)
	CT(9831)=(S=5264544,E=5264559,C=14)
	CT(9832)=(S=5264560,E=5264575,C=31)
	CT(9833)=(S=5264576,E=5264607,C=28)
	CT(9834)=(S=5264608,E=5264639,C=36)
	CT(9835)=(S=5264640,E=5264655,C=28)
	CT(9836)=(S=5264656,E=5264671,C=32)
	CT(9837)=(S=5264672,E=5264687,C=36)
	CT(9838)=(S=5264688,E=5264703,C=50)
	CT(9839)=(S=5264704,E=5264735,C=14)
	CT(9840)=(S=5264736,E=5264751,C=72)
	CT(9841)=(S=5264752,E=5264767,C=19)
	CT(9842)=(S=5264768,E=5264783,C=36)
	CT(9843)=(S=5264784,E=5264799,C=18)
	CT(9844)=(S=5264800,E=5264815,C=36)
	CT(9845)=(S=5264816,E=5264831,C=18)
	CT(9846)=(S=5264832,E=5264847,C=28)
	CT(9847)=(S=5264848,E=5264863,C=14)
	CT(9848)=(S=5264864,E=5264879,C=58)
	CT(9849)=(S=5264880,E=5264895,C=18)
	CT(9850)=(S=5264896,E=5264911,C=28)
	CT(9851)=(S=5264912,E=5264927,C=15)
	CT(9852)=(S=5264928,E=5264943,C=14)
	CT(9853)=(S=5264944,E=5264959,C=15)
	CT(9854)=(S=5264960,E=5264975,C=19)
	CT(9855)=(S=5264976,E=5264991,C=28)
	CT(9856)=(S=5264992,E=5265023,C=38)
	CT(9857)=(S=5265024,E=5265039,C=27)
	CT(9858)=(S=5265040,E=5265055,C=15)
	CT(9859)=(S=5265056,E=5265087,C=13)
	CT(9860)=(S=5265088,E=5265103,C=34)
	CT(9861)=(S=5265104,E=5265119,C=27)
	CT(9862)=(S=5265120,E=5265135,C=66)
	CT(9863)=(S=5265136,E=5265151,C=71)
	CT(9864)=(S=5265152,E=5265183,C=14)
	CT(9865)=(S=5265184,E=5265199,C=58)
	CT(9866)=(S=5265200,E=5265215,C=19)
	CT(9867)=(S=5265216,E=5265247,C=110)
	CT(9868)=(S=5265248,E=5265263,C=18)
	CT(9869)=(S=5265264,E=5265279,C=28)
	CT(9870)=(S=5265280,E=5265295,C=14)
	CT(9871)=(S=5265296,E=5265311,C=18)
	CT(9872)=(S=5265312,E=5265327,C=28)
	CT(9873)=(S=5265328,E=5265343,C=33)
	CT(9874)=(S=5265344,E=5265359,C=18)
	CT(9875)=(S=5265360,E=5265375,C=71)
	CT(9876)=(S=5265376,E=5265391,C=12)
	CT(9877)=(S=5265392,E=5265407,C=59)
	CT(9878)=(S=5265412,E=5265415,C=111)
	CT(9879)=(S=5265420,E=5265423,C=135)
	CT(9880)=(S=5265424,E=5265439,C=28)
	CT(9881)=(S=5265440,E=5265455,C=49)
	CT(9882)=(S=5265456,E=5265471,C=18)
	CT(9883)=(S=5265472,E=5265487,C=28)
	CT(9884)=(S=5265488,E=5265503,C=15)
	CT(9885)=(S=5265504,E=5265535,C=19)
	CT(9886)=(S=5265536,E=5265567,C=24)
	CT(9887)=(S=5265568,E=5265583,C=15)
	CT(9888)=(S=5265584,E=5265599,C=47)
	CT(9889)=(S=5265600,E=5265631,C=14)
	CT(9890)=(S=5265632,E=5265647,C=25)
	CT(9891)=(S=5265648,E=5265663,C=104)
	CT(9892)=(S=5265664,E=5265679,C=18)
	CT(9893)=(S=5265680,E=5265695,C=24)
	CT(9894)=(S=5265696,E=5265727,C=27)
	CT(9895)=(S=5265728,E=5265743,C=28)
	CT(9896)=(S=5265744,E=5265759,C=14)
	CT(9897)=(S=5265760,E=5265775,C=42)
	CT(9898)=(S=5265776,E=5265791,C=12)
	CT(9899)=(S=5265792,E=5265823,C=18)
	CT(9900)=(S=5265824,E=5265839,C=19)
	CT(9901)=(S=5265840,E=5265855,C=39)
	CT(9902)=(S=5265856,E=5265871,C=18)
	CT(9903)=(S=5265872,E=5265887,C=34)
	CT(9904)=(S=5265888,E=5265903,C=36)
	CT(9905)=(S=5265904,E=5265919,C=28)
	CT(9906)=(S=5265920,E=5265951,C=60)
	CT(9907)=(S=5265952,E=5265983,C=70)
	CT(9908)=(S=5265984,E=5265999,C=18)
	CT(9909)=(S=5266000,E=5266015,C=96)
	CT(9910)=(S=5266016,E=5266031,C=28)
	CT(9911)=(S=5266032,E=5266047,C=18)
	CT(9912)=(S=5266048,E=5266063,C=33)
	CT(9913)=(S=5266064,E=5266079,C=28)
	CT(9914)=(S=5266080,E=5266095,C=61)
	CT(9915)=(S=5266096,E=5266111,C=18)
	CT(9916)=(S=5266112,E=5266127,C=14)
	CT(9917)=(S=5266128,E=5266143,C=52)
	CT(9918)=(S=5266144,E=5266159,C=31)
	CT(9919)=(S=5266160,E=5266175,C=18)
	CT(9920)=(S=5266176,E=5266191,C=24)
	CT(9921)=(S=5266192,E=5266207,C=18)
	CT(9922)=(S=5266208,E=5266223,C=27)
	CT(9923)=(S=5266224,E=5266239,C=15)
	CT(9924)=(S=5266240,E=5266271,C=25)
	CT(9925)=(S=5266272,E=5266287,C=28)
	CT(9926)=(S=5266288,E=5266303,C=96)
	CT(9927)=(S=5266304,E=5266319,C=24)
	CT(9928)=(S=5266320,E=5266335,C=49)
	CT(9929)=(S=5266336,E=5266367,C=31)
	CT(9930)=(S=5266368,E=5266383,C=18)
	CT(9931)=(S=5266384,E=5266399,C=22)
	CT(9932)=(S=5266400,E=5266415,C=27)
	CT(9933)=(S=5266416,E=5266431,C=13)
	CT(9934)=(S=5266432,E=5266479,C=18)
	CT(9935)=(S=5266480,E=5266495,C=28)
	CT(9936)=(S=5266496,E=5266527,C=70)
	CT(9937)=(S=5266528,E=5266543,C=18)
	CT(9938)=(S=5266544,E=5266559,C=42)
	CT(9939)=(S=5266560,E=5266575,C=19)
	CT(9940)=(S=5266576,E=5266591,C=14)
	CT(9941)=(S=5266592,E=5266607,C=18)
	CT(9942)=(S=5266608,E=5266623,C=40)
	CT(9943)=(S=5266624,E=5266655,C=18)
	CT(9944)=(S=5266656,E=5266671,C=31)
	CT(9945)=(S=5266672,E=5266687,C=33)
	CT(9946)=(S=5266688,E=5266719,C=63)
	CT(9947)=(S=5266720,E=5266735,C=42)
	CT(9948)=(S=5266736,E=5266751,C=18)
	CT(9949)=(S=5266752,E=5266767,C=15)
	CT(9950)=(S=5266768,E=5266783,C=12)
	CT(9951)=(S=5266784,E=5266799,C=18)
	CT(9952)=(S=5266800,E=5266815,C=31)
	CT(9953)=(S=5266816,E=5266831,C=15)
	CT(9954)=(S=5266832,E=5266847,C=72)
	CT(9955)=(S=5266848,E=5266863,C=14)
	CT(9956)=(S=5266864,E=5266879,C=18)
	CT(9957)=(S=5266880,E=5266895,C=14)
	CT(9958)=(S=5266896,E=5266911,C=37)
	CT(9959)=(S=5266912,E=5266943,C=46)
	CT(9960)=(S=5266944,E=5266959,C=25)
	CT(9961)=(S=5266960,E=5266975,C=58)
	CT(9962)=(S=5266976,E=5266991,C=14)
	CT(9963)=(S=5266992,E=5267007,C=71)
	CT(9964)=(S=5267008,E=5267023,C=36)
	CT(9965)=(S=5267024,E=5267039,C=28)
	CT(9966)=(S=5267040,E=5267055,C=187)
	CT(9967)=(S=5267056,E=5267071,C=15)
	CT(9968)=(S=5267072,E=5267087,C=28)
	CT(9969)=(S=5267088,E=5267103,C=34)
	CT(9970)=(S=5267104,E=5267119,C=68)
	CT(9971)=(S=5267120,E=5267135,C=12)
	CT(9972)=(S=5267136,E=5267151,C=14)
	CT(9973)=(S=5267152,E=5267167,C=19)
	CT(9974)=(S=5267168,E=5267183,C=68)
	CT(9975)=(S=5267184,E=5267199,C=53)
	CT(9976)=(S=5267200,E=5267203,C=105)
	CT(9977)=(S=5267216,E=5267231,C=51)
	CT(9978)=(S=5267232,E=5267247,C=18)
	CT(9979)=(S=5267248,E=5267263,C=14)
	CT(9980)=(S=5267264,E=5267295,C=50)
	CT(9981)=(S=5267296,E=5267327,C=33)
	CT(9982)=(S=5267328,E=5267343,C=47)
	CT(9983)=(S=5267344,E=5267359,C=28)
	CT(9984)=(S=5267360,E=5267375,C=36)
	CT(9985)=(S=5267376,E=5267407,C=14)
	CT(9986)=(S=5267408,E=5267423,C=104)
	CT(9987)=(S=5267424,E=5267439,C=52)
	CT(9988)=(S=5267440,E=5267455,C=33)
	CT(9989)=(S=5267456,E=5267967,C=38)
	CT(9990)=(S=5267968,E=5268479,C=50)
	CT(9991)=(S=5268480,E=5268991,C=36)
	CT(9992)=(S=5268992,E=5269503,C=25)
	CT(9993)=(S=5269504,E=5270015,C=15)
	CT(9994)=(S=5270016,E=5270527,C=22)
	CT(9995)=(S=5270528,E=5271551,C=42)
	CT(9996)=(S=5271552,E=5272511,C=36)
	CT(9997)=(S=5272512,E=5272543,C=18)
	CT(9998)=(S=5272544,E=5272575,C=36)
	CT(9999)=(S=5272576,E=5273087,C=15)
	CT(10000)=(S=5273088,E=5273599,C=12)
	CT(10001)=(S=5273600,E=5274623,C=42)
	CT(10002)=(S=5274624,E=5275135,C=12)
	CT(10003)=(S=5275136,E=5275647,C=36)
	CT(10004)=(S=5275648,E=5283839,C=28)
	CT(10005)=(S=5283840,E=5285887,C=24)
	CT(10006)=(S=5285888,E=5286399,C=14)
	CT(10007)=(S=5286400,E=5286655,C=19)
	CT(10008)=(S=5286656,E=5286911,C=28)
	CT(10009)=(S=5286912,E=5287167,C=21)
	CT(10010)=(S=5287168,E=5287679,C=25)
	CT(10011)=(S=5287680,E=5288447,C=14)
	CT(10012)=(S=5288448,E=5288959,C=17)
	CT(10013)=(S=5288960,E=5289983,C=15)
	CT(10014)=(S=5289984,E=5290239,C=101)
	CT(10015)=(S=5290240,E=5290495,C=12)
	CT(10016)=(S=5290496,E=5290751,C=47)
	CT(10017)=(S=5290752,E=5291007,C=28)
	CT(10018)=(S=5291008,E=5291263,C=33)
	CT(10019)=(S=5291264,E=5291519,C=14)
	CT(10020)=(S=5291520,E=5291775,C=28)
	CT(10021)=(S=5291776,E=5292031,C=26)
	CT(10022)=(S=5292032,E=5293055,C=14)
	CT(10023)=(S=5293056,E=5294079,C=24)
	CT(10024)=(S=5294080,E=5294591,C=49)
	CT(10025)=(S=5294592,E=5295103,C=27)
	CT(10026)=(S=5295104,E=5296127,C=15)
	CT(10027)=(S=5296128,E=5297151,C=24)
	CT(10028)=(S=5297152,E=5297663,C=27)
	CT(10029)=(S=5297664,E=5298175,C=12)
	CT(10030)=(S=5298176,E=5298687,C=19)
	CT(10031)=(S=5298688,E=5299199,C=34)
	CT(10032)=(S=5299200,E=5300223,C=47)
	CT(10033)=(S=5300224,E=5300479,C=25)
	CT(10034)=(S=5300480,E=5300735,C=14)
	CT(10035)=(S=5300736,E=5300991,C=28)
	CT(10036)=(S=5300992,E=5301247,C=16)
	CT(10037)=(S=5301248,E=5301503,C=28)
	CT(10038)=(S=5301504,E=5301759,C=14)
	CT(10039)=(S=5301760,E=5302015,C=17)
	CT(10040)=(S=5302016,E=5302271,C=13)
	CT(10041)=(S=5302272,E=5302399,C=27)
	CT(10042)=(S=5302400,E=5302527,C=60)
	CT(10043)=(S=5302528,E=5302655,C=63)
	CT(10044)=(S=5302656,E=5302783,C=60)
	CT(10045)=(S=5302784,E=5302911,C=18)
	CT(10046)=(S=5302912,E=5303039,C=14)
	CT(10047)=(S=5303040,E=5303167,C=55)
	CT(10048)=(S=5303168,E=5303295,C=14)
	CT(10049)=(S=5303296,E=5303423,C=12)
	CT(10050)=(S=5303424,E=5303551,C=49)
	CT(10051)=(S=5303552,E=5303679,C=18)
	CT(10052)=(S=5303680,E=5303807,C=28)
	CT(10053)=(S=5303808,E=5303871,C=14)
	CT(10054)=(S=5303872,E=5303935,C=58)
	CT(10055)=(S=5303936,E=5304063,C=34)
	CT(10056)=(S=5304064,E=5304191,C=27)
	CT(10057)=(S=5304192,E=5304319,C=13)
	CT(10058)=(S=5304320,E=5304335,C=44)
	CT(10059)=(S=5304336,E=5304351,C=22)
	CT(10060)=(S=5304352,E=5304383,C=18)
	CT(10061)=(S=5304384,E=5304399,C=23)
	CT(10062)=(S=5304400,E=5304415,C=14)
	CT(10063)=(S=5304416,E=5304447,C=18)
	CT(10064)=(S=5304448,E=5304463,C=13)
	CT(10065)=(S=5304464,E=5304479,C=46)
	CT(10066)=(S=5304480,E=5304511,C=58)
	CT(10067)=(S=5304512,E=5304527,C=113)
	CT(10068)=(S=5304528,E=5304543,C=18)
	CT(10069)=(S=5304544,E=5304559,C=42)
	CT(10070)=(S=5304560,E=5304575,C=18)
	CT(10071)=(S=5304576,E=5304591,C=20)
	CT(10072)=(S=5304592,E=5304607,C=42)
	CT(10073)=(S=5304608,E=5304623,C=20)
	CT(10074)=(S=5304624,E=5304631,C=14)
	CT(10075)=(S=5304632,E=5304639,C=28)
	CT(10076)=(S=5304640,E=5304655,C=14)
	CT(10077)=(S=5304656,E=5304671,C=27)
	CT(10078)=(S=5304672,E=5304687,C=34)
	CT(10079)=(S=5304688,E=5304703,C=56)
	CT(10080)=(S=5304704,E=5304711,C=58)
	CT(10081)=(S=5304720,E=5304735,C=104)
	CT(10082)=(S=5304736,E=5304751,C=15)
	CT(10083)=(S=5304752,E=5304767,C=40)
	CT(10084)=(S=5304768,E=5304799,C=28)
	CT(10085)=(S=5304800,E=5304815,C=15)
	CT(10086)=(S=5304816,E=5304831,C=40)
	CT(10087)=(S=5304832,E=5304847,C=26)
	CT(10088)=(S=5304848,E=5304863,C=47)
	CT(10089)=(S=5304864,E=5304879,C=71)
	CT(10090)=(S=5304880,E=5304927,C=18)
	CT(10091)=(S=5304928,E=5304943,C=31)
	CT(10092)=(S=5304944,E=5304959,C=59)
	CT(10093)=(S=5304960,E=5305023,C=28)
	CT(10094)=(S=5305024,E=5305039,C=34)
	CT(10095)=(S=5305040,E=5305055,C=20)
	CT(10096)=(S=5305056,E=5305087,C=36)
	CT(10097)=(S=5305088,E=5305103,C=18)
	CT(10098)=(S=5305104,E=5305119,C=44)
	CT(10099)=(S=5305120,E=5305151,C=28)
	CT(10100)=(S=5305152,E=5305167,C=18)
	CT(10101)=(S=5305168,E=5305183,C=21)
	CT(10102)=(S=5305184,E=5305199,C=33)
	CT(10103)=(S=5305200,E=5305215,C=24)
	CT(10104)=(S=5305216,E=5305231,C=18)
	CT(10105)=(S=5305232,E=5305247,C=31)
	CT(10106)=(S=5305248,E=5305263,C=42)
	CT(10107)=(S=5305264,E=5305279,C=14)
	CT(10108)=(S=5305280,E=5305295,C=28)
	CT(10109)=(S=5305296,E=5305311,C=34)
	CT(10110)=(S=5305328,E=5305343,C=12)
	CT(10111)=(S=5305344,E=5305359,C=14)
	CT(10112)=(S=5305360,E=5305375,C=22)
	CT(10113)=(S=5305376,E=5305391,C=18)
	CT(10114)=(S=5305392,E=5305407,C=25)
	CT(10115)=(S=5305408,E=5305439,C=19)
	CT(10116)=(S=5305440,E=5305455,C=50)
	CT(10117)=(S=5305456,E=5305471,C=15)
	CT(10118)=(S=5305472,E=5305503,C=58)
	CT(10119)=(S=5305504,E=5305519,C=17)
	CT(10120)=(S=5305520,E=5305535,C=14)
	CT(10121)=(S=5305536,E=5305551,C=19)
	CT(10122)=(S=5305552,E=5305567,C=28)
	CT(10123)=(S=5305568,E=5305583,C=18)
	CT(10124)=(S=5305584,E=5305599,C=28)
	CT(10125)=(S=5305600,E=5305615,C=25)
	CT(10126)=(S=5305616,E=5305663,C=12)
	CT(10127)=(S=5305664,E=5305679,C=28)
	CT(10128)=(S=5305680,E=5305695,C=53)
	CT(10129)=(S=5305712,E=5305727,C=31)
	CT(10130)=(S=5305728,E=5305759,C=28)
	CT(10131)=(S=5305760,E=5305775,C=22)
	CT(10132)=(S=5305776,E=5305791,C=58)
	CT(10133)=(S=5305792,E=5305807,C=42)
	CT(10134)=(S=5305808,E=5305823,C=72)
	CT(10135)=(S=5305824,E=5305839,C=19)
	CT(10136)=(S=5305840,E=5305855,C=18)
	CT(10137)=(S=5305856,E=5305871,C=135)
	CT(10138)=(S=5305872,E=5305887,C=18)
	CT(10139)=(S=5305888,E=5305919,C=28)
	CT(10140)=(S=5305920,E=5305951,C=18)
	CT(10141)=(S=5305952,E=5305967,C=70)
	CT(10142)=(S=5305968,E=5305983,C=28)
	CT(10143)=(S=5305984,E=5305999,C=17)
	CT(10144)=(S=5306000,E=5306031,C=47)
	CT(10145)=(S=5306032,E=5306063,C=36)
	CT(10146)=(S=5306064,E=5306079,C=18)
	CT(10147)=(S=5306080,E=5306095,C=52)
	CT(10148)=(S=5306096,E=5306111,C=18)
	CT(10149)=(S=5306112,E=5306143,C=14)
	CT(10150)=(S=5306144,E=5306159,C=18)
	CT(10151)=(S=5306160,E=5306175,C=14)
	CT(10152)=(S=5306176,E=5306191,C=15)
	CT(10153)=(S=5306192,E=5306207,C=14)
	CT(10154)=(S=5306208,E=5306239,C=18)
	CT(10155)=(S=5306240,E=5306271,C=14)
	CT(10156)=(S=5306272,E=5306287,C=36)
	CT(10157)=(S=5306288,E=5306303,C=18)
	CT(10158)=(S=5306304,E=5306335,C=36)
	CT(10159)=(S=5306336,E=5306351,C=12)
	CT(10160)=(S=5306352,E=5306367,C=47)
	CT(10161)=(S=5306368,E=5306383,C=111)
	CT(10162)=(S=5306384,E=5306399,C=48)
	CT(10163)=(S=5306400,E=5306415,C=36)
	CT(10164)=(S=5306416,E=5306431,C=18)
	CT(10165)=(S=5306432,E=5306447,C=143)
	CT(10166)=(S=5306448,E=5306463,C=18)
	CT(10167)=(S=5306464,E=5306495,C=47)
	CT(10168)=(S=5306496,E=5306511,C=19)
	CT(10169)=(S=5306512,E=5306527,C=18)
	CT(10170)=(S=5306528,E=5306543,C=47)
	CT(10171)=(S=5306544,E=5306559,C=14)
	CT(10172)=(S=5306560,E=5306575,C=56)
	CT(10173)=(S=5306576,E=5306591,C=12)
	CT(10174)=(S=5306592,E=5306607,C=19)
	CT(10175)=(S=5306608,E=5306623,C=47)
	CT(10176)=(S=5306624,E=5306639,C=58)
	CT(10177)=(S=5306640,E=5306655,C=20)
	CT(10178)=(S=5306656,E=5306671,C=15)
	CT(10179)=(S=5306672,E=5306687,C=14)
	CT(10180)=(S=5306688,E=5306703,C=135)
	CT(10181)=(S=5306704,E=5306719,C=68)
	CT(10182)=(S=5306720,E=5306735,C=14)
	CT(10183)=(S=5306752,E=5306783,C=18)
	CT(10184)=(S=5306784,E=5306799,C=50)
	CT(10185)=(S=5306800,E=5306831,C=18)
	CT(10186)=(S=5306832,E=5306839,C=36)
	CT(10187)=(S=5306840,E=5306847,C=14)
	CT(10188)=(S=5306848,E=5306863,C=31)
	CT(10189)=(S=5306864,E=5306879,C=63)
	CT(10190)=(S=5306880,E=5306911,C=33)
	CT(10191)=(S=5306928,E=5306943,C=60)
	CT(10192)=(S=5306944,E=5306975,C=18)
	CT(10193)=(S=5306976,E=5306991,C=14)
	CT(10194)=(S=5306992,E=5307007,C=55)
	CT(10195)=(S=5307008,E=5307023,C=28)
	CT(10196)=(S=5307024,E=5307039,C=17)
	CT(10197)=(S=5307040,E=5307071,C=18)
	CT(10198)=(S=5307072,E=5307087,C=26)
	CT(10199)=(S=5307088,E=5307119,C=18)
	CT(10200)=(S=5307120,E=5307135,C=71)
	CT(10201)=(S=5307136,E=5307167,C=14)
	CT(10202)=(S=5307168,E=5307183,C=37)
	CT(10203)=(S=5307184,E=5307199,C=18)
	CT(10204)=(S=5307200,E=5307215,C=25)
	CT(10205)=(S=5307216,E=5307231,C=28)
	CT(10206)=(S=5307232,E=5307247,C=12)
	CT(10207)=(S=5307248,E=5307295,C=18)
	CT(10208)=(S=5307296,E=5307327,C=14)
	CT(10209)=(S=5307328,E=5307343,C=19)
	CT(10210)=(S=5307344,E=5307375,C=18)
	CT(10211)=(S=5307376,E=5307391,C=33)
	CT(10212)=(S=5307392,E=5307407,C=58)
	CT(10213)=(S=5307408,E=5307423,C=18)
	CT(10214)=(S=5307424,E=5307439,C=42)
	CT(10215)=(S=5307440,E=5307455,C=50)
	CT(10216)=(S=5307456,E=5307471,C=14)
	CT(10217)=(S=5307472,E=5307487,C=36)
	CT(10218)=(S=5307488,E=5307503,C=28)
	CT(10219)=(S=5307504,E=5307519,C=14)
	CT(10220)=(S=5307520,E=5307551,C=18)
	CT(10221)=(S=5307552,E=5307583,C=19)
	CT(10222)=(S=5307584,E=5307599,C=15)
	CT(10223)=(S=5307600,E=5307615,C=19)
	CT(10224)=(S=5307616,E=5307631,C=15)
	CT(10225)=(S=5307632,E=5307647,C=31)
	CT(10226)=(S=5307648,E=5307679,C=18)
	CT(10227)=(S=5307680,E=5307695,C=15)
	CT(10228)=(S=5307696,E=5307711,C=51)
	CT(10229)=(S=5307712,E=5307727,C=25)
	CT(10230)=(S=5307728,E=5307743,C=34)
	CT(10231)=(S=5307744,E=5307775,C=14)
	CT(10232)=(S=5307776,E=5307807,C=26)
	CT(10233)=(S=5307808,E=5307823,C=57)
	CT(10234)=(S=5307824,E=5307839,C=50)
	CT(10235)=(S=5307840,E=5307855,C=25)
	CT(10236)=(S=5307856,E=5307871,C=28)
	CT(10237)=(S=5307872,E=5307887,C=18)
	CT(10238)=(S=5307888,E=5307903,C=37)
	CT(10239)=(S=5307904,E=5307919,C=31)
	CT(10240)=(S=5307920,E=5307935,C=18)
	CT(10241)=(S=5307936,E=5307951,C=27)
	CT(10242)=(S=5307952,E=5307967,C=18)
	CT(10243)=(S=5307968,E=5307983,C=34)
	CT(10244)=(S=5307984,E=5307999,C=72)
	CT(10245)=(S=5308000,E=5308031,C=18)
	CT(10246)=(S=5308032,E=5308047,C=28)
	CT(10247)=(S=5308048,E=5308063,C=14)
	CT(10248)=(S=5308064,E=5308095,C=34)
	CT(10249)=(S=5308096,E=5308111,C=15)
	CT(10250)=(S=5308112,E=5308127,C=60)
	CT(10251)=(S=5308144,E=5308159,C=19)
	CT(10252)=(S=5308160,E=5308175,C=28)
	CT(10253)=(S=5308176,E=5308191,C=18)
	CT(10254)=(S=5308192,E=5308223,C=14)
	CT(10255)=(S=5308224,E=5308239,C=31)
	CT(10256)=(S=5308240,E=5308255,C=18)
	CT(10257)=(S=5308256,E=5308271,C=34)
	CT(10258)=(S=5308272,E=5308287,C=25)
	CT(10259)=(S=5308288,E=5308319,C=18)
	CT(10260)=(S=5308320,E=5308335,C=70)
	CT(10261)=(S=5308336,E=5308351,C=18)
	CT(10262)=(S=5308352,E=5308383,C=14)
	CT(10263)=(S=5308384,E=5308399,C=60)
	CT(10264)=(S=5308400,E=5308415,C=36)
	CT(10265)=(S=5308416,E=5308479,C=25)
	CT(10266)=(S=5308480,E=5308543,C=50)
	CT(10267)=(S=5308544,E=5308607,C=27)
	CT(10268)=(S=5308608,E=5308671,C=33)
	CT(10269)=(S=5308672,E=5308735,C=12)
	CT(10270)=(S=5308736,E=5308799,C=14)
	CT(10271)=(S=5308800,E=5308991,C=18)
	CT(10272)=(S=5308992,E=5309055,C=14)
	CT(10273)=(S=5309056,E=5309119,C=28)
	CT(10274)=(S=5309120,E=5309183,C=33)
	CT(10275)=(S=5309184,E=5309247,C=28)
	CT(10276)=(S=5309248,E=5309311,C=14)
	CT(10277)=(S=5309312,E=5309375,C=18)
	CT(10278)=(S=5309376,E=5309439,C=42)
	CT(10279)=(S=5309440,E=5309503,C=105)
	CT(10280)=(S=5309504,E=5309567,C=36)
	CT(10281)=(S=5309568,E=5309631,C=32)
	CT(10282)=(S=5309632,E=5309695,C=18)
	CT(10283)=(S=5309696,E=5309759,C=17)
	CT(10284)=(S=5309760,E=5309823,C=18)
	CT(10285)=(S=5309824,E=5309887,C=14)
	CT(10286)=(S=5309888,E=5309951,C=42)
	CT(10287)=(S=5309952,E=5310015,C=34)
	CT(10288)=(S=5310016,E=5310079,C=37)
	CT(10289)=(S=5310080,E=5310143,C=58)
	CT(10290)=(S=5310144,E=5310207,C=14)
	CT(10291)=(S=5310208,E=5310271,C=28)
	CT(10292)=(S=5310272,E=5310335,C=44)
	CT(10293)=(S=5310336,E=5310399,C=24)
	CT(10294)=(S=5310400,E=5310431,C=28)
	CT(10295)=(S=5310432,E=5310463,C=34)
	CT(10296)=(S=5310464,E=5310591,C=37)
	CT(10297)=(S=5310592,E=5310719,C=19)
	CT(10298)=(S=5310720,E=5310847,C=18)
	CT(10299)=(S=5310848,E=5310975,C=25)
	CT(10300)=(S=5310976,E=5311103,C=106)
	CT(10301)=(S=5311104,E=5311231,C=42)
	CT(10302)=(S=5311232,E=5311359,C=13)
	CT(10303)=(S=5311360,E=5311487,C=49)
	CT(10304)=(S=5311488,E=5311615,C=26)
	CT(10305)=(S=5311616,E=5311743,C=38)
	CT(10306)=(S=5311744,E=5311871,C=18)
	CT(10307)=(S=5311872,E=5311999,C=34)
	CT(10308)=(S=5312000,E=5312127,C=12)
	CT(10309)=(S=5312128,E=5312255,C=28)
	CT(10310)=(S=5312256,E=5312383,C=48)
	CT(10311)=(S=5312384,E=5312511,C=58)
	CT(10312)=(S=5312512,E=5312527,C=66)
	CT(10313)=(S=5312544,E=5312559,C=42)
	CT(10314)=(S=5312560,E=5312575,C=28)
	CT(10315)=(S=5312576,E=5312591,C=47)
	CT(10316)=(S=5312592,E=5312607,C=18)
	CT(10317)=(S=5312608,E=5312623,C=42)
	CT(10318)=(S=5312624,E=5312639,C=18)
	CT(10319)=(S=5312656,E=5312671,C=42)
	CT(10320)=(S=5312672,E=5312687,C=19)
	CT(10321)=(S=5312688,E=5312703,C=28)
	CT(10322)=(S=5312704,E=5312719,C=50)
	CT(10323)=(S=5312720,E=5312735,C=23)
	CT(10324)=(S=5312736,E=5312751,C=44)
	CT(10325)=(S=5312752,E=5312767,C=40)
	CT(10326)=(S=5312768,E=5312783,C=18)
	CT(10327)=(S=5312784,E=5312799,C=34)
	CT(10328)=(S=5312800,E=5312831,C=36)
	CT(10329)=(S=5312832,E=5312847,C=14)
	CT(10330)=(S=5312848,E=5312863,C=41)
	CT(10331)=(S=5312864,E=5312895,C=28)
	CT(10332)=(S=5312896,E=5312911,C=31)
	CT(10333)=(S=5312912,E=5312927,C=18)
	CT(10334)=(S=5312928,E=5312943,C=20)
	CT(10335)=(S=5312944,E=5312959,C=50)
	CT(10336)=(S=5312960,E=5312975,C=47)
	CT(10337)=(S=5312976,E=5312991,C=28)
	CT(10338)=(S=5312992,E=5313007,C=52)
	CT(10339)=(S=5313008,E=5313023,C=63)
	CT(10340)=(S=5313024,E=5313039,C=36)
	CT(10341)=(S=5313040,E=5313055,C=34)
	CT(10342)=(S=5313056,E=5313071,C=20)
	CT(10343)=(S=5313072,E=5313087,C=46)
	CT(10344)=(S=5313088,E=5313119,C=38)
	CT(10345)=(S=5313120,E=5313135,C=28)
	CT(10346)=(S=5313136,E=5313183,C=18)
	CT(10347)=(S=5313184,E=5313199,C=36)
	CT(10348)=(S=5313200,E=5313215,C=12)
	CT(10349)=(S=5313216,E=5313247,C=58)
	CT(10350)=(S=5313248,E=5313263,C=171)
	CT(10351)=(S=5313264,E=5313279,C=36)
	CT(10352)=(S=5313280,E=5313295,C=33)
	CT(10353)=(S=5313296,E=5313311,C=12)
	CT(10354)=(S=5313312,E=5313327,C=33)
	CT(10355)=(S=5313328,E=5313343,C=14)
	CT(10356)=(S=5313344,E=5313375,C=18)
	CT(10357)=(S=5313376,E=5313391,C=25)
	CT(10358)=(S=5313392,E=5313407,C=47)
	CT(10359)=(S=5313408,E=5313423,C=18)
	CT(10360)=(S=5313424,E=5313439,C=42)
	CT(10361)=(S=5313440,E=5313455,C=25)
	CT(10362)=(S=5313456,E=5313503,C=14)
	CT(10363)=(S=5313504,E=5313535,C=24)
	CT(10364)=(S=5313536,E=5313551,C=14)
	CT(10365)=(S=5313552,E=5313567,C=62)
	CT(10366)=(S=5313568,E=5313599,C=14)
	CT(10367)=(S=5313600,E=5313615,C=36)
	CT(10368)=(S=5313616,E=5313631,C=28)
	CT(10369)=(S=5313632,E=5313647,C=18)
	CT(10370)=(S=5313648,E=5313679,C=28)
	CT(10371)=(S=5313680,E=5313695,C=55)
	CT(10372)=(S=5313696,E=5313711,C=18)
	CT(10373)=(S=5313712,E=5313727,C=99)
	CT(10374)=(S=5313728,E=5313743,C=18)
	CT(10375)=(S=5313744,E=5313759,C=12)
	CT(10376)=(S=5313760,E=5313775,C=47)
	CT(10377)=(S=5313776,E=5313791,C=21)
	CT(10378)=(S=5313792,E=5313807,C=31)
	CT(10379)=(S=5313808,E=5313823,C=15)
	CT(10380)=(S=5313824,E=5313839,C=32)
	CT(10381)=(S=5313840,E=5313855,C=23)
	CT(10382)=(S=5313856,E=5313871,C=14)
	CT(10383)=(S=5313872,E=5313887,C=41)
	CT(10384)=(S=5313888,E=5313903,C=106)
	CT(10385)=(S=5313904,E=5313919,C=42)
	CT(10386)=(S=5313920,E=5313927,C=15)
	CT(10387)=(S=5313928,E=5313935,C=36)
	CT(10388)=(S=5313936,E=5313951,C=63)
	CT(10389)=(S=5313952,E=5313967,C=37)
	CT(10390)=(S=5313968,E=5313983,C=36)
	CT(10391)=(S=5313984,E=5313999,C=58)
	CT(10392)=(S=5314000,E=5314015,C=19)
	CT(10393)=(S=5314016,E=5314031,C=18)
	CT(10394)=(S=5314032,E=5314047,C=55)
	CT(10395)=(S=5314048,E=5314063,C=18)
	CT(10396)=(S=5314064,E=5314079,C=104)
	CT(10397)=(S=5314080,E=5314095,C=14)
	CT(10398)=(S=5314096,E=5314111,C=18)
	CT(10399)=(S=5314144,E=5314159,C=37)
	CT(10400)=(S=5314160,E=5314175,C=18)
	CT(10401)=(S=5314176,E=5314191,C=31)
	CT(10402)=(S=5314192,E=5314207,C=38)
	CT(10403)=(S=5314208,E=5314223,C=47)
	CT(10404)=(S=5314224,E=5314239,C=50)
	CT(10405)=(S=5314240,E=5314271,C=18)
	CT(10406)=(S=5314272,E=5314287,C=28)
	CT(10407)=(S=5314288,E=5314303,C=47)
	CT(10408)=(S=5314304,E=5314319,C=18)
	CT(10409)=(S=5314320,E=5314335,C=31)
	CT(10410)=(S=5314336,E=5314351,C=12)
	CT(10411)=(S=5314352,E=5314367,C=14)
	CT(10412)=(S=5314368,E=5314383,C=34)
	CT(10413)=(S=5314384,E=5314399,C=15)
	CT(10414)=(S=5314400,E=5314431,C=18)
	CT(10415)=(S=5314432,E=5314447,C=24)
	CT(10416)=(S=5314448,E=5314511,C=18)
	CT(10417)=(S=5314512,E=5314527,C=14)
	CT(10418)=(S=5314528,E=5314575,C=36)
	CT(10419)=(S=5314576,E=5314591,C=38)
	CT(10420)=(S=5314592,E=5314607,C=28)
	CT(10421)=(S=5314608,E=5314623,C=36)
	CT(10422)=(S=5314624,E=5314639,C=28)
	CT(10423)=(S=5314640,E=5314655,C=18)
	CT(10424)=(S=5314656,E=5314671,C=36)
	CT(10425)=(S=5314672,E=5314703,C=18)
	CT(10426)=(S=5314704,E=5314719,C=42)
	CT(10427)=(S=5314720,E=5314735,C=25)
	CT(10428)=(S=5314736,E=5314751,C=18)
	CT(10429)=(S=5314752,E=5314767,C=14)
	CT(10430)=(S=5314768,E=5314783,C=31)
	CT(10431)=(S=5314784,E=5314799,C=15)
	CT(10432)=(S=5314800,E=5314807,C=46)
	CT(10433)=(S=5314808,E=5314815,C=63)
	CT(10434)=(S=5314816,E=5314831,C=18)
	CT(10435)=(S=5314832,E=5314847,C=33)
	CT(10436)=(S=5314848,E=5314863,C=68)
	CT(10437)=(S=5314864,E=5314879,C=18)
	CT(10438)=(S=5314896,E=5314911,C=19)
	CT(10439)=(S=5314912,E=5314927,C=15)
	CT(10440)=(S=5314928,E=5314943,C=25)
	CT(10441)=(S=5314944,E=5314959,C=28)
	CT(10442)=(S=5314960,E=5314975,C=19)
	CT(10443)=(S=5314976,E=5314991,C=18)
	CT(10444)=(S=5314992,E=5315007,C=171)
	CT(10445)=(S=5315008,E=5315023,C=12)
	CT(10446)=(S=5315024,E=5315039,C=42)
	CT(10447)=(S=5315040,E=5315055,C=31)
	CT(10448)=(S=5315056,E=5315071,C=55)
	CT(10449)=(S=5315072,E=5315103,C=58)
	CT(10450)=(S=5315104,E=5315135,C=27)
	CT(10451)=(S=5315136,E=5315143,C=111)
	CT(10452)=(S=5315144,E=5315151,C=105)
	CT(10453)=(S=5315152,E=5315167,C=18)
	CT(10454)=(S=5315168,E=5315199,C=14)
	CT(10455)=(S=5315200,E=5315229,C=18)
	CT(10456)=(S=5315230,E=5315231,C=38)
	CT(10457)=(S=5315232,E=5315247,C=28)
	CT(10458)=(S=5315248,E=5315263,C=18)
	CT(10459)=(S=5315264,E=5315271,C=15)
	CT(10460)=(S=5315272,E=5315279,C=96)
	CT(10461)=(S=5315280,E=5315295,C=36)
	CT(10462)=(S=5315296,E=5315343,C=19)
	CT(10463)=(S=5315344,E=5315359,C=12)
	CT(10464)=(S=5315360,E=5315375,C=27)
	CT(10465)=(S=5315376,E=5315391,C=18)
	CT(10466)=(S=5315392,E=5315439,C=14)
	CT(10467)=(S=5315440,E=5315455,C=28)
	CT(10468)=(S=5315456,E=5315471,C=15)
	CT(10469)=(S=5315472,E=5315487,C=18)
	CT(10470)=(S=5315488,E=5315503,C=28)
	CT(10471)=(S=5315504,E=5315519,C=15)
	CT(10472)=(S=5315520,E=5315535,C=33)
	CT(10473)=(S=5315536,E=5315551,C=24)
	CT(10474)=(S=5315552,E=5315567,C=28)
	CT(10475)=(S=5315568,E=5315599,C=18)
	CT(10476)=(S=5315600,E=5315615,C=33)
	CT(10477)=(S=5315616,E=5315647,C=26)
	CT(10478)=(S=5315648,E=5315663,C=28)
	CT(10479)=(S=5315664,E=5315679,C=36)
	CT(10480)=(S=5315680,E=5315695,C=12)
	CT(10481)=(S=5315696,E=5315711,C=61)
	CT(10482)=(S=5315712,E=5315727,C=42)
	CT(10483)=(S=5315728,E=5315743,C=14)
	CT(10484)=(S=5315744,E=5315775,C=18)
	CT(10485)=(S=5315776,E=5315791,C=12)
	CT(10486)=(S=5315792,E=5315807,C=18)
	CT(10487)=(S=5315808,E=5315823,C=28)
	CT(10488)=(S=5315840,E=5315855,C=36)
	CT(10489)=(S=5315856,E=5315871,C=44)
	CT(10490)=(S=5315872,E=5315887,C=27)
	CT(10491)=(S=5315888,E=5315935,C=14)
	CT(10492)=(S=5315936,E=5315951,C=106)
	CT(10493)=(S=5315952,E=5315967,C=18)
	CT(10494)=(S=5315968,E=5315983,C=47)
	CT(10495)=(S=5315984,E=5315999,C=34)
	CT(10496)=(S=5316000,E=5316015,C=94)
	CT(10497)=(S=5316016,E=5316079,C=15)
	CT(10498)=(S=5316080,E=5316095,C=26)
	CT(10499)=(S=5316096,E=5316127,C=15)
	CT(10500)=(S=5316128,E=5316143,C=36)
	CT(10501)=(S=5316144,E=5316159,C=18)
	CT(10502)=(S=5316160,E=5316175,C=36)
	CT(10503)=(S=5316176,E=5316191,C=68)
	CT(10504)=(S=5316192,E=5316207,C=28)
	CT(10505)=(S=5316208,E=5316223,C=18)
	CT(10506)=(S=5316224,E=5316239,C=58)
	CT(10507)=(S=5316240,E=5316255,C=28)
	CT(10508)=(S=5316256,E=5316271,C=31)
	CT(10509)=(S=5316272,E=5316319,C=18)
	CT(10510)=(S=5316320,E=5316351,C=33)
	CT(10511)=(S=5316352,E=5316383,C=14)
	CT(10512)=(S=5316384,E=5316399,C=33)
	CT(10513)=(S=5316400,E=5316415,C=47)
	CT(10514)=(S=5316416,E=5316431,C=14)
	CT(10515)=(S=5316432,E=5316447,C=24)
	CT(10516)=(S=5316448,E=5316479,C=14)
	CT(10517)=(S=5316480,E=5316495,C=42)
	CT(10518)=(S=5316496,E=5316511,C=15)
	CT(10519)=(S=5316512,E=5316543,C=26)
	CT(10520)=(S=5316560,E=5316567,C=63)
	CT(10521)=(S=5316568,E=5316575,C=36)
	CT(10522)=(S=5316576,E=5316607,C=26)
	CT(10523)=(S=5316608,E=5320703,C=25)
	CT(10524)=(S=5320704,E=5323263,C=12)
	CT(10525)=(S=5323264,E=5323775,C=49)
	CT(10526)=(S=5323776,E=5324287,C=25)
	CT(10527)=(S=5324288,E=5324799,C=34)
	CT(10528)=(S=5324800,E=5325823,C=12)
	CT(10529)=(S=5325824,E=5326847,C=36)
	CT(10530)=(S=5326848,E=5327871,C=15)
	CT(10531)=(S=5327872,E=5328895,C=14)
	CT(10532)=(S=5328896,E=5329151,C=12)
	CT(10533)=(S=5329152,E=5329407,C=15)
	CT(10534)=(S=5329408,E=5329919,C=49)
	CT(10535)=(S=5329920,E=5330175,C=21)
	CT(10536)=(S=5330176,E=5330431,C=36)
	CT(10537)=(S=5330432,E=5330943,C=14)
	CT(10538)=(S=5330944,E=5330959,C=19)
	CT(10539)=(S=5330960,E=5330991,C=28)
	CT(10540)=(S=5330992,E=5331007,C=15)
	CT(10541)=(S=5331008,E=5331023,C=27)
	CT(10542)=(S=5331024,E=5331039,C=18)
	CT(10543)=(S=5331040,E=5331055,C=12)
	CT(10544)=(S=5331056,E=5331071,C=18)
	CT(10545)=(S=5331072,E=5331087,C=71)
	CT(10546)=(S=5331088,E=5331103,C=20)
	CT(10547)=(S=5331104,E=5331119,C=14)
	CT(10548)=(S=5331120,E=5331135,C=34)
	CT(10549)=(S=5331136,E=5331151,C=62)
	CT(10550)=(S=5331152,E=5331167,C=18)
	CT(10551)=(S=5331168,E=5331199,C=15)
	CT(10552)=(S=5331200,E=5331231,C=38)
	CT(10553)=(S=5331232,E=5331247,C=25)
	CT(10554)=(S=5331248,E=5331263,C=71)
	CT(10555)=(S=5331264,E=5331287,C=18)
	CT(10556)=(S=5331288,E=5331311,C=28)
	CT(10557)=(S=5331312,E=5331327,C=18)
	CT(10558)=(S=5331328,E=5331343,C=14)
	CT(10559)=(S=5331344,E=5331359,C=19)
	CT(10560)=(S=5331360,E=5331375,C=14)
	CT(10561)=(S=5331376,E=5331391,C=18)
	CT(10562)=(S=5331392,E=5331407,C=28)
	CT(10563)=(S=5331408,E=5331423,C=66)
	CT(10564)=(S=5331424,E=5331455,C=28)
	CT(10565)=(S=5331456,E=5331471,C=18)
	CT(10566)=(S=5331472,E=5331487,C=35)
	CT(10567)=(S=5331488,E=5331503,C=28)
	CT(10568)=(S=5331504,E=5331519,C=21)
	CT(10569)=(S=5331520,E=5331535,C=47)
	CT(10570)=(S=5331536,E=5331551,C=14)
	CT(10571)=(S=5331552,E=5331567,C=25)
	CT(10572)=(S=5331568,E=5331583,C=55)
	CT(10573)=(S=5331584,E=5331599,C=14)
	CT(10574)=(S=5331600,E=5331615,C=26)
	CT(10575)=(S=5331616,E=5331631,C=33)
	CT(10576)=(S=5331632,E=5331647,C=52)
	CT(10577)=(S=5331648,E=5331663,C=28)
	CT(10578)=(S=5331664,E=5331679,C=18)
	CT(10579)=(S=5331680,E=5331695,C=31)
	CT(10580)=(S=5331696,E=5331711,C=33)
	CT(10581)=(S=5331712,E=5331727,C=19)
	CT(10582)=(S=5331728,E=5331735,C=28)
	CT(10583)=(S=5331736,E=5331743,C=67)
	CT(10584)=(S=5331744,E=5331775,C=18)
	CT(10585)=(S=5331776,E=5331791,C=12)
	CT(10586)=(S=5331792,E=5331807,C=33)
	CT(10587)=(S=5331808,E=5331823,C=14)
	CT(10588)=(S=5331824,E=5331839,C=37)
	CT(10589)=(S=5331840,E=5331871,C=26)
	CT(10590)=(S=5331872,E=5331887,C=28)
	CT(10591)=(S=5331888,E=5331903,C=18)
	CT(10592)=(S=5331904,E=5331919,C=14)
	CT(10593)=(S=5331920,E=5331935,C=33)
	CT(10594)=(S=5331936,E=5331951,C=124)
	CT(10595)=(S=5331952,E=5331967,C=14)
	CT(10596)=(S=5331968,E=5331999,C=28)
	CT(10597)=(S=5332000,E=5332015,C=15)
	CT(10598)=(S=5332016,E=5332031,C=22)
	CT(10599)=(S=5332032,E=5332047,C=19)
	CT(10600)=(S=5332048,E=5332063,C=12)
	CT(10601)=(S=5332064,E=5332079,C=34)
	CT(10602)=(S=5332080,E=5332095,C=12)
	CT(10603)=(S=5332096,E=5332111,C=25)
	CT(10604)=(S=5332112,E=5332127,C=33)
	CT(10605)=(S=5332128,E=5332143,C=28)
	CT(10606)=(S=5332144,E=5332159,C=36)
	CT(10607)=(S=5332160,E=5332161,C=14)
	CT(10608)=(S=5332162,E=5332163,C=29)
	CT(10609)=(S=5332164,E=5332167,C=19)
	CT(10610)=(S=5332168,E=5332184,C=21)
	CT(10611)=(S=5332185,E=5332185,C=14)
	CT(10612)=(S=5332186,E=5332187,C=29)
	CT(10613)=(S=5332188,E=5332191,C=19)
	CT(10614)=(S=5332192,E=5332207,C=70)
	CT(10615)=(S=5332208,E=5332223,C=42)
	CT(10616)=(S=5332224,E=5332255,C=12)
	CT(10617)=(S=5332256,E=5332271,C=15)
	CT(10618)=(S=5332272,E=5332287,C=36)
	CT(10619)=(S=5332288,E=5332319,C=59)
	CT(10620)=(S=5332320,E=5332335,C=27)
	CT(10621)=(S=5332336,E=5332351,C=28)
	CT(10622)=(S=5332352,E=5332383,C=19)
	CT(10623)=(S=5332384,E=5332399,C=27)
	CT(10624)=(S=5332400,E=5332415,C=12)
	CT(10625)=(S=5332416,E=5332431,C=50)
	CT(10626)=(S=5332432,E=5332447,C=25)
	CT(10627)=(S=5332448,E=5332463,C=14)
	CT(10628)=(S=5332464,E=5332479,C=12)
	CT(10629)=(S=5332480,E=5332495,C=59)
	CT(10630)=(S=5332496,E=5332527,C=18)
	CT(10631)=(S=5332528,E=5332543,C=42)
	CT(10632)=(S=5332544,E=5332575,C=19)
	CT(10633)=(S=5332576,E=5332607,C=34)
	CT(10634)=(S=5332608,E=5332639,C=18)
	CT(10635)=(S=5332640,E=5332655,C=19)
	CT(10636)=(S=5332656,E=5332671,C=50)
	CT(10637)=(S=5332672,E=5332703,C=14)
	CT(10638)=(S=5332704,E=5332719,C=60)
	CT(10639)=(S=5332720,E=5332735,C=50)
	CT(10640)=(S=5332736,E=5332751,C=28)
	CT(10641)=(S=5332752,E=5332783,C=18)
	CT(10642)=(S=5332784,E=5332799,C=14)
	CT(10643)=(S=5332800,E=5332815,C=25)
	CT(10644)=(S=5332816,E=5332831,C=35)
	CT(10645)=(S=5332832,E=5332847,C=33)
	CT(10646)=(S=5332848,E=5332863,C=49)
	CT(10647)=(S=5332864,E=5332879,C=18)
	CT(10648)=(S=5332880,E=5332895,C=14)
	CT(10649)=(S=5332896,E=5332911,C=40)
	CT(10650)=(S=5332912,E=5332927,C=31)
	CT(10651)=(S=5332928,E=5332943,C=58)
	CT(10652)=(S=5332944,E=5332959,C=18)
	CT(10653)=(S=5332960,E=5332975,C=98)
	CT(10654)=(S=5332976,E=5332991,C=24)
	CT(10655)=(S=5332992,E=5337087,C=14)
	CT(10656)=(S=5337088,E=5341183,C=15)
	CT(10657)=(S=5341184,E=5349375,C=14)
	CT(10658)=(S=5349376,E=5349503,C=28)
	CT(10659)=(S=5349504,E=5349631,C=50)
	CT(10660)=(S=5349632,E=5349647,C=38)
	CT(10661)=(S=5349648,E=5349679,C=18)
	CT(10662)=(S=5349680,E=5349687,C=38)
	CT(10663)=(S=5349688,E=5349689,C=56)
	CT(10664)=(S=5349690,E=5349690,C=12)
	CT(10665)=(S=5349691,E=5349691,C=38)
	CT(10666)=(S=5349692,E=5349693,C=71)
	CT(10667)=(S=5349694,E=5349694,C=58)
	CT(10668)=(S=5349695,E=5349695,C=18)
	CT(10669)=(S=5349696,E=5349727,C=33)
	CT(10670)=(S=5349728,E=5349729,C=24)
	CT(10671)=(S=5349730,E=5349731,C=18)
	CT(10672)=(S=5349732,E=5349745,C=58)
	CT(10673)=(S=5349746,E=5349746,C=18)
	CT(10674)=(S=5349747,E=5349747,C=42)
	CT(10675)=(S=5349748,E=5349759,C=18)
	CT(10676)=(S=5349760,E=5349823,C=24)
	CT(10677)=(S=5349824,E=5349839,C=58)
	CT(10678)=(S=5349840,E=5349855,C=18)
	CT(10679)=(S=5349872,E=5349887,C=51)
	CT(10680)=(S=5349888,E=5349943,C=18)
	CT(10681)=(S=5349944,E=5349967,C=31)
	CT(10682)=(S=5349968,E=5349983,C=71)
	CT(10683)=(S=5349984,E=5350015,C=31)
	CT(10684)=(S=5350016,E=5350079,C=53)
	CT(10685)=(S=5350080,E=5350095,C=33)
	CT(10686)=(S=5350096,E=5350103,C=58)
	CT(10687)=(S=5350104,E=5350143,C=31)
	CT(10688)=(S=5350144,E=5350151,C=26)
	CT(10689)=(S=5350152,E=5350159,C=58)
	CT(10690)=(S=5350160,E=5350215,C=18)
	CT(10691)=(S=5350216,E=5350223,C=60)
	CT(10692)=(S=5350224,E=5350231,C=33)
	CT(10693)=(S=5350232,E=5350239,C=31)
	CT(10694)=(S=5350240,E=5350255,C=18)
	CT(10695)=(S=5350256,E=5350303,C=31)
	CT(10696)=(S=5350304,E=5350335,C=18)
	CT(10697)=(S=5350336,E=5350351,C=58)
	CT(10698)=(S=5350352,E=5350367,C=31)
	CT(10699)=(S=5350368,E=5350399,C=18)
	CT(10700)=(S=5350400,E=5350911,C=49)
	CT(10701)=(S=5350912,E=5351423,C=27)
	CT(10702)=(S=5351424,E=5351551,C=14)
	CT(10703)=(S=5351552,E=5351679,C=58)
	CT(10704)=(S=5351680,E=5351807,C=49)
	CT(10705)=(S=5351808,E=5351935,C=28)
	CT(10706)=(S=5351936,E=5352063,C=14)
	CT(10707)=(S=5352064,E=5352191,C=19)
	CT(10708)=(S=5352192,E=5352319,C=36)
	CT(10709)=(S=5352320,E=5352447,C=14)
	CT(10710)=(S=5352448,E=5352575,C=25)
	CT(10711)=(S=5352576,E=5352703,C=19)
	CT(10712)=(S=5352704,E=5352831,C=36)
	CT(10713)=(S=5352832,E=5352959,C=28)
	CT(10714)=(S=5352960,E=5353087,C=15)
	CT(10715)=(S=5353088,E=5353215,C=14)
	CT(10716)=(S=5353216,E=5353279,C=27)
	CT(10717)=(S=5353280,E=5353343,C=36)
	CT(10718)=(S=5353344,E=5353471,C=47)
	CT(10719)=(S=5353472,E=5353983,C=18)
	CT(10720)=(S=5353984,E=5354495,C=14)
	CT(10721)=(S=5354496,E=5355007,C=38)
	CT(10722)=(S=5355008,E=5355519,C=50)
	CT(10723)=(S=5355520,E=5355775,C=25)
	CT(10724)=(S=5355776,E=5356031,C=12)
	CT(10725)=(S=5356032,E=5356255,C=22)
	CT(10726)=(S=5356256,E=5356271,C=58)
	CT(10727)=(S=5356272,E=5356287,C=19)
	CT(10728)=(S=5356288,E=5356543,C=14)
	CT(10729)=(S=5356544,E=5356799,C=49)
	CT(10730)=(S=5356800,E=5357055,C=42)
	CT(10731)=(S=5357056,E=5357311,C=58)
	CT(10732)=(S=5357312,E=5357567,C=27)
	CT(10733)=(S=5357568,E=5357823,C=134)
	CT(10734)=(S=5357824,E=5358079,C=21)
	CT(10735)=(S=5358080,E=5358335,C=12)
	CT(10736)=(S=5358336,E=5358591,C=18)
	CT(10737)=(S=5358592,E=5358847,C=38)
	CT(10738)=(S=5358848,E=5359103,C=47)
	CT(10739)=(S=5359104,E=5359359,C=60)
	CT(10740)=(S=5359360,E=5359615,C=14)
	CT(10741)=(S=5359616,E=5359647,C=18)
	CT(10742)=(S=5359648,E=5359663,C=12)
	CT(10743)=(S=5359664,E=5359679,C=33)
	CT(10744)=(S=5359680,E=5359695,C=14)
	CT(10745)=(S=5359696,E=5359711,C=18)
	CT(10746)=(S=5359712,E=5359727,C=28)
	CT(10747)=(S=5359728,E=5359743,C=18)
	CT(10748)=(S=5359744,E=5359759,C=15)
	CT(10749)=(S=5359760,E=5359775,C=18)
	CT(10750)=(S=5359776,E=5359791,C=19)
	CT(10751)=(S=5359792,E=5359807,C=12)
	CT(10752)=(S=5359808,E=5359823,C=28)
	CT(10753)=(S=5359824,E=5359839,C=18)
	CT(10754)=(S=5359840,E=5359855,C=15)
	CT(10755)=(S=5359856,E=5359871,C=18)
	CT(10756)=(S=5359872,E=5359887,C=15)
	CT(10757)=(S=5359888,E=5359903,C=18)
	CT(10758)=(S=5359904,E=5359919,C=28)
	CT(10759)=(S=5359920,E=5359935,C=33)
	CT(10760)=(S=5359936,E=5359951,C=18)
	CT(10761)=(S=5359952,E=5359967,C=49)
	CT(10762)=(S=5359968,E=5359999,C=28)
	CT(10763)=(S=5360000,E=5360015,C=14)
	CT(10764)=(S=5360016,E=5360031,C=28)
	CT(10765)=(S=5360032,E=5360047,C=14)
	CT(10766)=(S=5360048,E=5360063,C=12)
	CT(10767)=(S=5360064,E=5360079,C=34)
	CT(10768)=(S=5360080,E=5360095,C=19)
	CT(10769)=(S=5360096,E=5360111,C=28)
	CT(10770)=(S=5360112,E=5360127,C=18)
	CT(10771)=(S=5360128,E=5360639,C=25)
	CT(10772)=(S=5360640,E=5361663,C=36)
	CT(10773)=(S=5361664,E=5361791,C=15)
	CT(10774)=(S=5361792,E=5361919,C=14)
	CT(10775)=(S=5361920,E=5362047,C=47)
	CT(10776)=(S=5362048,E=5362175,C=13)
	CT(10777)=(S=5362176,E=5362303,C=58)
	CT(10778)=(S=5362304,E=5362431,C=28)
	CT(10779)=(S=5362432,E=5362559,C=18)
	CT(10780)=(S=5362560,E=5362687,C=19)
	CT(10781)=(S=5362688,E=5363711,C=37)
	CT(10782)=(S=5363712,E=5363967,C=19)
	CT(10783)=(S=5363968,E=5364223,C=42)
	CT(10784)=(S=5364224,E=5364479,C=17)
	CT(10785)=(S=5364480,E=5364735,C=58)
	CT(10786)=(S=5364736,E=5364991,C=12)
	CT(10787)=(S=5364992,E=5365247,C=34)
	CT(10788)=(S=5365248,E=5365503,C=18)
	CT(10789)=(S=5365504,E=5365759,C=42)
	CT(10790)=(S=5365760,E=5369855,C=19)
	CT(10791)=(S=5369856,E=5371903,C=49)
	CT(10792)=(S=5371904,E=5373951,C=12)
	CT(10793)=(S=5373952,E=5386239,C=14)
	CT(10794)=(S=5386240,E=5390335,C=15)
	CT(10795)=(S=5390336,E=5391359,C=12)
	CT(10796)=(S=5391360,E=5392383,C=14)
	CT(10797)=(S=5392384,E=5393407,C=36)
	CT(10798)=(S=5393408,E=5394431,C=38)
	CT(10799)=(S=5394432,E=5394943,C=17)
	CT(10800)=(S=5394944,E=5395455,C=28)
	CT(10801)=(S=5395456,E=5395967,C=15)
	CT(10802)=(S=5395968,E=5396479,C=25)
	CT(10803)=(S=5396480,E=5397503,C=15)
	CT(10804)=(S=5397504,E=5398527,C=36)
	CT(10805)=(S=5398528,E=5398591,C=19)
	CT(10806)=(S=5398592,E=5398655,C=28)
	CT(10807)=(S=5398656,E=5398719,C=12)
	CT(10808)=(S=5398720,E=5398783,C=18)
	CT(10809)=(S=5398784,E=5398847,C=12)
	CT(10810)=(S=5398848,E=5398975,C=28)
	CT(10811)=(S=5398976,E=5399039,C=18)
	CT(10812)=(S=5399040,E=5399103,C=12)
	CT(10813)=(S=5399104,E=5399167,C=28)
	CT(10814)=(S=5399168,E=5399231,C=25)
	CT(10815)=(S=5399232,E=5399239,C=14)
	CT(10816)=(S=5399240,E=5399271,C=13)
	CT(10817)=(S=5399272,E=5399295,C=14)
	CT(10818)=(S=5399296,E=5399423,C=19)
	CT(10819)=(S=5399424,E=5399487,C=33)
	CT(10820)=(S=5399488,E=5399551,C=26)
	CT(10821)=(S=5399552,E=5399615,C=33)
	CT(10822)=(S=5399616,E=5399679,C=19)
	CT(10823)=(S=5399680,E=5399743,C=18)
	CT(10824)=(S=5399744,E=5399807,C=28)
	CT(10825)=(S=5399808,E=5399871,C=12)
	CT(10826)=(S=5399872,E=5399935,C=51)
	CT(10827)=(S=5400000,E=5400063,C=36)
	CT(10828)=(S=5400064,E=5400067,C=19)
	CT(10829)=(S=5400068,E=5400068,C=21)
	CT(10830)=(S=5400069,E=5400070,C=19)
	CT(10831)=(S=5400071,E=5400071,C=21)
	CT(10832)=(S=5400072,E=5400079,C=14)
	CT(10833)=(S=5400080,E=5400127,C=21)
	CT(10834)=(S=5400128,E=5400191,C=32)
	CT(10835)=(S=5400192,E=5400255,C=17)
	CT(10836)=(S=5400256,E=5400319,C=43)
	CT(10837)=(S=5400320,E=5400383,C=42)
	CT(10838)=(S=5400384,E=5400447,C=51)
	CT(10839)=(S=5400448,E=5400511,C=24)
	CT(10840)=(S=5400512,E=5400575,C=47)
	CT(10841)=(S=5400576,E=5401599,C=15)
	CT(10842)=(S=5401600,E=5402623,C=14)
	CT(10843)=(S=5402624,E=5402687,C=18)
	CT(10844)=(S=5402688,E=5402719,C=48)
	CT(10845)=(S=5402720,E=5402751,C=14)
	CT(10846)=(S=5402752,E=5402783,C=99)
	CT(10847)=(S=5402784,E=5402791,C=39)
	CT(10848)=(S=5402792,E=5402799,C=14)
	CT(10849)=(S=5402800,E=5402815,C=18)
	CT(10850)=(S=5402816,E=5402847,C=15)
	CT(10851)=(S=5402848,E=5402879,C=14)
	CT(10852)=(S=5402880,E=5402911,C=187)
	CT(10853)=(S=5402912,E=5402943,C=33)
	CT(10854)=(S=5402944,E=5402975,C=14)
	CT(10855)=(S=5402976,E=5403007,C=28)
	CT(10856)=(S=5403008,E=5403071,C=14)
	CT(10857)=(S=5403072,E=5403103,C=15)
	CT(10858)=(S=5403104,E=5403135,C=14)
	CT(10859)=(S=5403136,E=5403167,C=18)
	CT(10860)=(S=5403168,E=5403199,C=32)
	CT(10861)=(S=5403200,E=5403231,C=96)
	CT(10862)=(S=5403232,E=5403295,C=18)
	CT(10863)=(S=5403296,E=5403327,C=67)
	CT(10864)=(S=5403328,E=5403359,C=33)
	CT(10865)=(S=5403360,E=5403391,C=18)
	CT(10866)=(S=5403392,E=5403406,C=26)
	CT(10867)=(S=5403407,E=5403407,C=36)
	CT(10868)=(S=5403408,E=5403423,C=26)
	CT(10869)=(S=5403424,E=5403455,C=20)
	CT(10870)=(S=5403456,E=5403487,C=58)
	CT(10871)=(S=5403488,E=5403519,C=28)
	CT(10872)=(S=5403520,E=5403551,C=19)
	CT(10873)=(S=5403552,E=5403583,C=15)
	CT(10874)=(S=5403584,E=5403599,C=19)
	CT(10875)=(S=5403616,E=5403647,C=14)
	CT(10876)=(S=5403648,E=5403711,C=18)
	CT(10877)=(S=5403712,E=5403743,C=27)
	CT(10878)=(S=5403744,E=5403807,C=28)
	CT(10879)=(S=5403808,E=5403839,C=25)
	CT(10880)=(S=5403840,E=5403871,C=32)
	CT(10881)=(S=5403872,E=5403903,C=47)
	CT(10882)=(S=5403904,E=5403935,C=56)
	CT(10883)=(S=5403936,E=5403967,C=27)
	CT(10884)=(S=5403968,E=5403999,C=18)
	CT(10885)=(S=5404000,E=5404031,C=19)
	CT(10886)=(S=5404032,E=5404063,C=33)
	CT(10887)=(S=5404064,E=5404095,C=18)
	CT(10888)=(S=5404096,E=5404127,C=46)
	CT(10889)=(S=5404128,E=5404159,C=31)
	CT(10890)=(S=5404160,E=5404175,C=59)
	CT(10891)=(S=5404176,E=5404183,C=31)
	CT(10892)=(S=5404184,E=5404191,C=19)
	CT(10893)=(S=5404192,E=5404223,C=28)
	CT(10894)=(S=5404224,E=5404287,C=14)
	CT(10895)=(S=5404288,E=5404319,C=18)
	CT(10896)=(S=5404320,E=5404351,C=23)
	CT(10897)=(S=5404352,E=5404383,C=47)
	CT(10898)=(S=5404384,E=5404415,C=51)
	CT(10899)=(S=5404416,E=5404447,C=28)
	CT(10900)=(S=5404448,E=5404479,C=15)
	CT(10901)=(S=5404480,E=5404511,C=51)
	CT(10902)=(S=5404512,E=5404543,C=71)
	CT(10903)=(S=5404544,E=5404575,C=18)
	CT(10904)=(S=5404576,E=5404607,C=28)
	CT(10905)=(S=5404608,E=5404639,C=15)
	CT(10906)=(S=5404640,E=5404655,C=71)
	CT(10907)=(S=5404656,E=5404671,C=33)
	CT(10908)=(S=5404672,E=5406719,C=12)
	CT(10909)=(S=5406720,E=5406847,C=111)
	CT(10910)=(S=5406848,E=5406975,C=47)
	CT(10911)=(S=5406976,E=5407103,C=13)
	CT(10912)=(S=5407104,E=5407231,C=106)
	CT(10913)=(S=5407232,E=5407295,C=47)
	CT(10914)=(S=5407296,E=5407359,C=34)
	CT(10915)=(S=5407360,E=5407487,C=25)
	CT(10916)=(S=5407488,E=5407615,C=55)
	CT(10917)=(S=5407616,E=5407743,C=50)
	CT(10918)=(S=5407744,E=5407871,C=57)
	CT(10919)=(S=5407872,E=5408127,C=14)
	CT(10920)=(S=5408128,E=5408255,C=47)
	CT(10921)=(S=5408256,E=5408383,C=27)
	CT(10922)=(S=5408384,E=5408511,C=36)
	CT(10923)=(S=5408512,E=5408639,C=28)
	CT(10924)=(S=5408640,E=5408767,C=44)
	CT(10925)=(S=5408768,E=5408831,C=14)
	CT(10926)=(S=5408832,E=5408895,C=34)
	CT(10927)=(S=5408896,E=5408959,C=19)
	CT(10928)=(S=5408960,E=5409023,C=36)
	CT(10929)=(S=5409024,E=5409087,C=38)
	CT(10930)=(S=5409088,E=5409151,C=51)
	CT(10931)=(S=5409152,E=5409215,C=18)
	CT(10932)=(S=5409216,E=5409279,C=30)
	CT(10933)=(S=5409280,E=5409343,C=18)
	CT(10934)=(S=5409344,E=5409407,C=12)
	CT(10935)=(S=5409408,E=5409471,C=13)
	CT(10936)=(S=5409472,E=5409535,C=14)
	CT(10937)=(S=5409536,E=5409599,C=58)
	CT(10938)=(S=5409600,E=5409663,C=36)
	CT(10939)=(S=5409664,E=5409727,C=58)
	CT(10940)=(S=5409728,E=5409855,C=28)
	CT(10941)=(S=5409856,E=5409919,C=18)
	CT(10942)=(S=5409920,E=5409983,C=44)
	CT(10943)=(S=5409984,E=5410047,C=18)
	CT(10944)=(S=5410048,E=5410111,C=28)
	CT(10945)=(S=5410112,E=5410175,C=47)
	CT(10946)=(S=5410176,E=5410239,C=50)
	CT(10947)=(S=5410240,E=5410303,C=63)
	CT(10948)=(S=5410304,E=5410367,C=12)
	CT(10949)=(S=5410368,E=5410431,C=33)
	CT(10950)=(S=5410432,E=5410495,C=18)
	CT(10951)=(S=5410496,E=5410559,C=14)
	CT(10952)=(S=5410560,E=5410623,C=15)
	CT(10953)=(S=5410624,E=5410687,C=49)
	CT(10954)=(S=5410688,E=5410751,C=58)
	CT(10955)=(S=5410752,E=5410815,C=24)
	CT(10956)=(S=5410816,E=5410847,C=25)
	CT(10957)=(S=5410848,E=5410879,C=28)
	CT(10958)=(S=5410880,E=5410911,C=18)
	CT(10959)=(S=5410912,E=5410943,C=25)
	CT(10960)=(S=5410944,E=5410975,C=33)
	CT(10961)=(S=5410976,E=5411007,C=50)
	CT(10962)=(S=5411008,E=5411039,C=31)
	CT(10963)=(S=5411040,E=5411071,C=14)
	CT(10964)=(S=5411072,E=5411103,C=28)
	CT(10965)=(S=5411104,E=5411135,C=14)
	CT(10966)=(S=5411136,E=5411167,C=58)
	CT(10967)=(S=5411168,E=5411199,C=15)
	CT(10968)=(S=5411200,E=5411231,C=19)
	CT(10969)=(S=5411232,E=5411263,C=15)
	CT(10970)=(S=5411264,E=5411279,C=28)
	CT(10971)=(S=5411280,E=5411295,C=14)
	CT(10972)=(S=5411296,E=5411327,C=37)
	CT(10973)=(S=5411328,E=5411359,C=51)
	CT(10974)=(S=5411360,E=5411391,C=18)
	CT(10975)=(S=5411392,E=5411423,C=27)
	CT(10976)=(S=5411424,E=5411455,C=49)
	CT(10977)=(S=5411456,E=5411487,C=14)
	CT(10978)=(S=5411488,E=5411519,C=39)
	CT(10979)=(S=5411520,E=5411551,C=34)
	CT(10980)=(S=5411552,E=5411583,C=58)
	CT(10981)=(S=5411584,E=5411615,C=14)
	CT(10982)=(S=5411616,E=5411647,C=27)
	CT(10983)=(S=5411648,E=5411711,C=18)
	CT(10984)=(S=5411712,E=5411743,C=51)
	CT(10985)=(S=5411744,E=5411775,C=55)
	CT(10986)=(S=5411776,E=5411807,C=23)
	CT(10987)=(S=5411808,E=5411839,C=24)
	CT(10988)=(S=5411840,E=5411871,C=18)
	CT(10989)=(S=5411872,E=5411903,C=14)
	CT(10990)=(S=5411904,E=5411935,C=48)
	CT(10991)=(S=5411936,E=5411967,C=94)
	CT(10992)=(S=5411968,E=5412031,C=27)
	CT(10993)=(S=5412032,E=5412063,C=36)
	CT(10994)=(S=5412064,E=5412095,C=14)
	CT(10995)=(S=5412096,E=5412127,C=52)
	CT(10996)=(S=5412128,E=5412159,C=15)
	CT(10997)=(S=5412160,E=5412191,C=28)
	CT(10998)=(S=5412192,E=5412223,C=42)
	CT(10999)=(S=5412224,E=5412255,C=18)
	CT(11000)=(S=5412256,E=5412287,C=28)
	CT(11001)=(S=5412288,E=5412319,C=18)
	CT(11002)=(S=5412320,E=5412351,C=28)
	CT(11003)=(S=5412352,E=5412383,C=13)
	CT(11004)=(S=5412384,E=5412415,C=50)
	CT(11005)=(S=5412416,E=5412447,C=37)
	CT(11006)=(S=5412448,E=5412479,C=14)
	CT(11007)=(S=5412480,E=5412511,C=36)
	CT(11008)=(S=5412512,E=5412543,C=33)
	CT(11009)=(S=5412544,E=5412575,C=42)
	CT(11010)=(S=5412576,E=5412607,C=28)
	CT(11011)=(S=5412608,E=5412639,C=12)
	CT(11012)=(S=5412640,E=5412671,C=36)
	CT(11013)=(S=5412672,E=5412703,C=149)
	CT(11014)=(S=5412704,E=5412735,C=18)
	CT(11015)=(S=5412736,E=5412767,C=37)
	CT(11016)=(S=5412768,E=5412799,C=36)
	CT(11017)=(S=5412800,E=5412831,C=18)
	CT(11018)=(S=5412832,E=5413375,C=14)
	CT(11019)=(S=5413376,E=5413887,C=21)
	CT(11020)=(S=5413888,E=5414399,C=36)
	CT(11021)=(S=5414400,E=5414911,C=25)
	CT(11022)=(S=5414912,E=5415167,C=58)
	CT(11023)=(S=5415168,E=5415423,C=36)
	CT(11024)=(S=5415424,E=5415679,C=18)
	CT(11025)=(S=5415680,E=5415927,C=14)
	CT(11026)=(S=5415928,E=5415931,C=17)
	CT(11027)=(S=5415932,E=5415935,C=14)
	CT(11028)=(S=5415936,E=5416191,C=27)
	CT(11029)=(S=5416192,E=5416447,C=28)
	CT(11030)=(S=5416448,E=5416703,C=17)
	CT(11031)=(S=5416704,E=5416959,C=23)
	CT(11032)=(S=5416960,E=5419263,C=36)
	CT(11033)=(S=5419264,E=5419519,C=58)
	CT(11034)=(S=5419520,E=5419775,C=45)
	CT(11035)=(S=5419776,E=5420031,C=18)
	CT(11036)=(S=5420032,E=5420287,C=24)
	CT(11037)=(S=5420288,E=5420543,C=47)
	CT(11038)=(S=5420544,E=5421055,C=19)
	CT(11039)=(S=5421056,E=5423103,C=15)
	CT(11040)=(S=5423104,E=5423135,C=42)
	CT(11041)=(S=5423136,E=5423167,C=52)
	CT(11042)=(S=5423168,E=5423199,C=36)
	CT(11043)=(S=5423200,E=5423231,C=14)
	CT(11044)=(S=5423232,E=5423295,C=24)
	CT(11045)=(S=5423296,E=5423327,C=28)
	CT(11046)=(S=5423328,E=5423359,C=34)
	CT(11047)=(S=5423360,E=5423423,C=15)
	CT(11048)=(S=5423424,E=5423455,C=60)
	CT(11049)=(S=5423456,E=5423487,C=31)
	CT(11050)=(S=5423488,E=5423519,C=18)
	CT(11051)=(S=5423520,E=5423551,C=19)
	CT(11052)=(S=5423552,E=5423583,C=57)
	CT(11053)=(S=5423584,E=5423615,C=28)
	CT(11054)=(S=5423616,E=5423647,C=41)
	CT(11055)=(S=5423648,E=5423679,C=104)
	CT(11056)=(S=5423680,E=5423711,C=25)
	CT(11057)=(S=5423712,E=5423743,C=28)
	CT(11058)=(S=5423744,E=5423775,C=42)
	CT(11059)=(S=5423776,E=5423807,C=18)
	CT(11060)=(S=5423808,E=5423839,C=27)
	CT(11061)=(S=5423840,E=5423903,C=18)
	CT(11062)=(S=5423904,E=5423935,C=36)
	CT(11063)=(S=5423936,E=5423967,C=28)
	CT(11064)=(S=5423968,E=5423999,C=14)
	CT(11065)=(S=5424000,E=5424031,C=63)
	CT(11066)=(S=5424032,E=5424063,C=14)
	CT(11067)=(S=5424064,E=5424095,C=47)
	CT(11068)=(S=5424096,E=5424127,C=34)
	CT(11069)=(S=5424128,E=5424143,C=36)
	CT(11070)=(S=5424144,E=5424159,C=12)
	CT(11071)=(S=5424160,E=5424191,C=13)
	CT(11072)=(S=5424192,E=5424223,C=18)
	CT(11073)=(S=5424224,E=5424255,C=19)
	CT(11074)=(S=5424256,E=5424287,C=18)
	CT(11075)=(S=5424288,E=5424319,C=19)
	CT(11076)=(S=5424320,E=5424351,C=27)
	CT(11077)=(S=5424352,E=5424383,C=14)
	CT(11078)=(S=5424384,E=5424415,C=47)
	CT(11079)=(S=5424416,E=5424447,C=58)
	CT(11080)=(S=5424448,E=5424479,C=14)
	CT(11081)=(S=5424480,E=5424511,C=12)
	CT(11082)=(S=5424512,E=5424543,C=13)
	CT(11083)=(S=5424544,E=5424575,C=34)
	CT(11084)=(S=5424576,E=5424607,C=36)
	CT(11085)=(S=5424608,E=5424639,C=19)
	CT(11086)=(S=5424640,E=5424671,C=18)
	CT(11087)=(S=5424672,E=5424703,C=25)
	CT(11088)=(S=5424704,E=5424735,C=28)
	CT(11089)=(S=5424736,E=5424767,C=25)
	CT(11090)=(S=5424768,E=5424799,C=63)
	CT(11091)=(S=5424800,E=5424831,C=18)
	CT(11092)=(S=5424832,E=5424863,C=28)
	CT(11093)=(S=5424864,E=5424895,C=14)
	CT(11094)=(S=5424896,E=5424927,C=27)
	CT(11095)=(S=5424928,E=5424959,C=25)
	CT(11096)=(S=5424960,E=5424991,C=36)
	CT(11097)=(S=5424992,E=5425023,C=18)
	CT(11098)=(S=5425024,E=5425055,C=28)
	CT(11099)=(S=5425056,E=5425087,C=47)
	CT(11100)=(S=5425088,E=5425119,C=66)
	CT(11101)=(S=5425120,E=5425151,C=47)
	CT(11102)=(S=5425152,E=5425279,C=18)
	CT(11103)=(S=5425280,E=5425407,C=20)
	CT(11104)=(S=5425408,E=5425535,C=36)
	CT(11105)=(S=5425536,E=5425663,C=106)
	CT(11106)=(S=5425664,E=5425791,C=33)
	CT(11107)=(S=5425792,E=5425919,C=18)
	CT(11108)=(S=5425920,E=5426047,C=14)
	CT(11109)=(S=5426048,E=5426175,C=47)
	CT(11110)=(S=5426176,E=5426303,C=36)
	CT(11111)=(S=5426304,E=5426431,C=18)
	CT(11112)=(S=5426432,E=5426559,C=43)
	CT(11113)=(S=5426560,E=5426687,C=16)
	CT(11114)=(S=5426688,E=5426815,C=28)
	CT(11115)=(S=5426816,E=5426943,C=14)
	CT(11116)=(S=5426944,E=5427071,C=31)
	CT(11117)=(S=5427072,E=5427199,C=28)
	CT(11118)=(S=5427200,E=5427263,C=33)
	CT(11119)=(S=5427264,E=5427327,C=18)
	CT(11120)=(S=5427328,E=5427391,C=38)
	CT(11121)=(S=5427392,E=5427455,C=46)
	CT(11122)=(S=5427456,E=5427519,C=33)
	CT(11123)=(S=5427520,E=5427583,C=18)
	CT(11124)=(S=5427584,E=5427647,C=19)
	CT(11125)=(S=5427648,E=5427711,C=68)
	CT(11126)=(S=5427712,E=5427775,C=28)
	CT(11127)=(S=5427776,E=5427839,C=36)
	CT(11128)=(S=5427840,E=5427903,C=58)
	CT(11129)=(S=5427904,E=5428031,C=28)
	CT(11130)=(S=5428032,E=5428095,C=14)
	CT(11131)=(S=5428096,E=5428159,C=40)
	CT(11132)=(S=5428160,E=5428223,C=24)
	CT(11133)=(S=5428224,E=5428287,C=28)
	CT(11134)=(S=5428288,E=5428351,C=61)
	CT(11135)=(S=5428352,E=5428415,C=49)
	CT(11136)=(S=5428416,E=5428479,C=28)
	CT(11137)=(S=5428480,E=5428543,C=43)
	CT(11138)=(S=5428544,E=5428607,C=15)
	CT(11139)=(S=5428608,E=5428735,C=25)
	CT(11140)=(S=5428736,E=5428799,C=19)
	CT(11141)=(S=5428800,E=5428863,C=52)
	CT(11142)=(S=5428864,E=5428927,C=58)
	CT(11143)=(S=5428928,E=5428991,C=28)
	CT(11144)=(S=5428992,E=5429055,C=36)
	CT(11145)=(S=5429056,E=5429119,C=98)
	CT(11146)=(S=5429120,E=5429183,C=15)
	CT(11147)=(S=5429184,E=5429247,C=47)
	CT(11148)=(S=5429248,E=5429503,C=12)
	CT(11149)=(S=5429504,E=5429759,C=36)
	CT(11150)=(S=5429760,E=5430015,C=42)
	CT(11151)=(S=5430016,E=5430271,C=14)
	CT(11152)=(S=5430272,E=5430527,C=34)
	CT(11153)=(S=5430528,E=5430783,C=48)
	CT(11154)=(S=5430784,E=5431039,C=37)
	CT(11155)=(S=5431040,E=5431295,C=25)
	CT(11156)=(S=5431296,E=5439487,C=12)
	CT(11157)=(S=5439488,E=5447679,C=58)
	CT(11158)=(S=5447680,E=5455871,C=25)
	CT(11159)=(S=5455872,E=5456383,C=42)
	CT(11160)=(S=5456384,E=5456639,C=37)
	CT(11161)=(S=5456640,E=5456895,C=14)
	CT(11162)=(S=5456896,E=5456927,C=36)
	CT(11163)=(S=5456928,E=5456959,C=18)
	CT(11164)=(S=5456960,E=5456991,C=58)
	CT(11165)=(S=5456992,E=5457023,C=12)
	CT(11166)=(S=5457024,E=5457055,C=42)
	CT(11167)=(S=5457056,E=5457087,C=101)
	CT(11168)=(S=5457088,E=5457119,C=34)
	CT(11169)=(S=5457120,E=5457151,C=19)
	CT(11170)=(S=5457152,E=5457183,C=18)
	CT(11171)=(S=5457184,E=5457215,C=33)
	CT(11172)=(S=5457216,E=5457279,C=18)
	CT(11173)=(S=5457280,E=5457311,C=98)
	CT(11174)=(S=5457312,E=5457407,C=18)
	CT(11175)=(S=5457408,E=5457919,C=63)
	CT(11176)=(S=5457920,E=5458943,C=24)
	CT(11177)=(S=5458944,E=5459967,C=34)
	CT(11178)=(S=5459968,E=5462015,C=36)
	CT(11179)=(S=5462016,E=5464063,C=24)
	CT(11180)=(S=5464064,E=5464191,C=101)
	CT(11181)=(S=5464192,E=5464319,C=36)
	CT(11182)=(S=5464320,E=5464327,C=18)
	CT(11183)=(S=5464328,E=5464335,C=36)
	CT(11184)=(S=5464344,E=5464351,C=51)
	CT(11185)=(S=5464352,E=5464359,C=14)
	CT(11186)=(S=5464360,E=5464375,C=28)
	CT(11187)=(S=5464376,E=5464383,C=12)
	CT(11188)=(S=5464384,E=5464391,C=51)
	CT(11189)=(S=5464392,E=5464399,C=28)
	CT(11190)=(S=5464400,E=5464407,C=34)
	CT(11191)=(S=5464408,E=5464415,C=14)
	CT(11192)=(S=5464416,E=5464423,C=24)
	CT(11193)=(S=5464424,E=5464431,C=18)
	CT(11194)=(S=5464440,E=5464447,C=34)
	CT(11195)=(S=5464448,E=5464575,C=25)
	CT(11196)=(S=5464576,E=5464703,C=14)
	CT(11197)=(S=5464704,E=5464831,C=36)
	CT(11198)=(S=5464832,E=5464959,C=70)
	CT(11199)=(S=5464960,E=5465087,C=60)
	CT(11200)=(S=5465088,E=5465215,C=47)
	CT(11201)=(S=5465216,E=5465343,C=14)
	CT(11202)=(S=5465344,E=5465471,C=49)
	CT(11203)=(S=5465472,E=5465599,C=23)
	CT(11204)=(S=5465600,E=5465727,C=47)
	CT(11205)=(S=5465728,E=5465855,C=18)
	CT(11206)=(S=5465856,E=5465983,C=15)
	CT(11207)=(S=5465984,E=5466111,C=38)
	CT(11208)=(S=5466112,E=5467135,C=14)
	CT(11209)=(S=5467136,E=5467647,C=27)
	CT(11210)=(S=5467648,E=5468159,C=16)
	CT(11211)=(S=5468160,E=5469183,C=12)
	CT(11212)=(S=5469184,E=5470207,C=36)
	CT(11213)=(S=5470208,E=5471231,C=26)
	CT(11214)=(S=5471232,E=5472255,C=13)
	CT(11215)=(S=5472256,E=5472511,C=36)
	CT(11216)=(S=5472512,E=5472767,C=28)
	CT(11217)=(S=5472768,E=5473023,C=17)
	CT(11218)=(S=5473024,E=5473279,C=57)
	CT(11219)=(S=5473280,E=5473535,C=21)
	CT(11220)=(S=5473536,E=5473791,C=13)
	CT(11221)=(S=5473792,E=5474047,C=49)
	CT(11222)=(S=5474048,E=5474303,C=28)
	CT(11223)=(S=5474304,E=5474311,C=14)
	CT(11224)=(S=5474312,E=5474319,C=16)
	CT(11225)=(S=5474320,E=5474327,C=36)
	CT(11226)=(S=5474328,E=5474335,C=18)
	CT(11227)=(S=5474336,E=5474343,C=42)
	CT(11228)=(S=5474344,E=5474351,C=63)
	CT(11229)=(S=5474360,E=5474367,C=104)
	CT(11230)=(S=5474368,E=5474375,C=14)
	CT(11231)=(S=5474376,E=5474391,C=28)
	CT(11232)=(S=5474392,E=5474399,C=24)
	CT(11233)=(S=5474400,E=5474407,C=28)
	CT(11234)=(S=5474408,E=5474415,C=15)
	CT(11235)=(S=5474416,E=5474423,C=18)
	CT(11236)=(S=5474424,E=5474431,C=14)
	CT(11237)=(S=5474432,E=5474439,C=28)
	CT(11238)=(S=5474440,E=5474447,C=60)
	CT(11239)=(S=5474456,E=5474463,C=58)
	CT(11240)=(S=5474464,E=5474471,C=12)
	CT(11241)=(S=5474472,E=5474479,C=63)
	CT(11242)=(S=5474480,E=5474487,C=46)
	CT(11243)=(S=5474488,E=5474495,C=25)
	CT(11244)=(S=5474496,E=5474503,C=36)
	CT(11245)=(S=5474504,E=5474511,C=33)
	CT(11246)=(S=5474512,E=5474519,C=12)
	CT(11247)=(S=5474520,E=5474527,C=49)
	CT(11248)=(S=5474528,E=5474535,C=58)
	CT(11249)=(S=5474536,E=5474551,C=18)
	CT(11250)=(S=5474552,E=5474559,C=47)
	CT(11251)=(S=5474560,E=5474567,C=14)
	CT(11252)=(S=5474568,E=5474575,C=19)
	CT(11253)=(S=5474576,E=5474583,C=36)
	CT(11254)=(S=5474584,E=5474591,C=34)
	CT(11255)=(S=5474592,E=5474599,C=28)
	CT(11256)=(S=5474600,E=5474607,C=42)
	CT(11257)=(S=5474608,E=5474615,C=18)
	CT(11258)=(S=5474616,E=5474623,C=15)
	CT(11259)=(S=5474624,E=5474631,C=28)
	CT(11260)=(S=5474632,E=5474647,C=34)
	CT(11261)=(S=5474648,E=5474655,C=31)
	CT(11262)=(S=5474656,E=5474663,C=28)
	CT(11263)=(S=5474664,E=5474671,C=15)
	CT(11264)=(S=5474672,E=5474679,C=42)
	CT(11265)=(S=5474680,E=5474687,C=24)
	CT(11266)=(S=5474688,E=5474695,C=14)
	CT(11267)=(S=5474696,E=5474711,C=36)
	CT(11268)=(S=5474712,E=5474719,C=50)
	CT(11269)=(S=5474720,E=5474727,C=15)
	CT(11270)=(S=5474728,E=5474735,C=28)
	CT(11271)=(S=5474736,E=5474743,C=14)
	CT(11272)=(S=5474744,E=5474751,C=28)
	CT(11273)=(S=5474752,E=5474759,C=36)
	CT(11274)=(S=5474760,E=5474767,C=27)
	CT(11275)=(S=5474768,E=5474775,C=14)
	CT(11276)=(S=5474776,E=5474783,C=18)
	CT(11277)=(S=5474784,E=5474791,C=14)
	CT(11278)=(S=5474792,E=5474799,C=15)
	CT(11279)=(S=5474800,E=5474807,C=12)
	CT(11280)=(S=5474808,E=5474815,C=14)
	CT(11281)=(S=5474816,E=5474823,C=36)
	CT(11282)=(S=5474824,E=5474831,C=63)
	CT(11283)=(S=5474832,E=5474855,C=14)
	CT(11284)=(S=5474856,E=5474863,C=25)
	CT(11285)=(S=5474864,E=5474871,C=18)
	CT(11286)=(S=5474872,E=5474879,C=14)
	CT(11287)=(S=5474880,E=5474943,C=28)
	CT(11288)=(S=5474944,E=5475007,C=14)
	CT(11289)=(S=5475008,E=5475071,C=25)
	CT(11290)=(S=5475072,E=5475135,C=66)
	CT(11291)=(S=5475136,E=5475199,C=57)
	CT(11292)=(S=5475200,E=5475263,C=18)
	CT(11293)=(S=5475264,E=5475327,C=15)
	CT(11294)=(S=5475328,E=5475583,C=19)
	CT(11295)=(S=5475584,E=5475647,C=28)
	CT(11296)=(S=5475648,E=5475711,C=63)
	CT(11297)=(S=5475712,E=5475839,C=12)
	CT(11298)=(S=5475840,E=5475847,C=19)
	CT(11299)=(S=5475848,E=5475855,C=18)
	CT(11300)=(S=5475856,E=5475863,C=51)
	CT(11301)=(S=5475864,E=5475879,C=14)
	CT(11302)=(S=5475880,E=5475903,C=58)
	CT(11303)=(S=5475904,E=5475911,C=14)
	CT(11304)=(S=5475912,E=5475919,C=47)
	CT(11305)=(S=5475920,E=5475927,C=28)
	CT(11306)=(S=5475928,E=5475935,C=34)
	CT(11307)=(S=5475936,E=5475943,C=27)
	CT(11308)=(S=5475944,E=5475951,C=31)
	CT(11309)=(S=5475952,E=5475967,C=58)
	CT(11310)=(S=5475968,E=5475975,C=28)
	CT(11311)=(S=5475976,E=5475983,C=58)
	CT(11312)=(S=5475984,E=5475991,C=12)
	CT(11313)=(S=5475992,E=5475999,C=58)
	CT(11314)=(S=5476000,E=5476007,C=18)
	CT(11315)=(S=5476008,E=5476047,C=58)
	CT(11316)=(S=5476048,E=5476055,C=31)
	CT(11317)=(S=5476056,E=5476063,C=58)
	CT(11318)=(S=5476064,E=5476071,C=14)
	CT(11319)=(S=5476072,E=5476079,C=31)
	CT(11320)=(S=5476080,E=5476087,C=25)
	CT(11321)=(S=5476088,E=5476095,C=72)
	CT(11322)=(S=5476096,E=5476103,C=28)
	CT(11323)=(S=5476104,E=5476111,C=111)
	CT(11324)=(S=5476112,E=5476119,C=12)
	CT(11325)=(S=5476120,E=5476127,C=126)
	CT(11326)=(S=5476128,E=5476135,C=18)
	CT(11327)=(S=5476136,E=5476143,C=58)
	CT(11328)=(S=5476144,E=5476151,C=18)
	CT(11329)=(S=5476152,E=5476159,C=47)
	CT(11330)=(S=5476160,E=5476163,C=18)
	CT(11331)=(S=5476164,E=5476167,C=53)
	CT(11332)=(S=5476176,E=5476183,C=27)
	CT(11333)=(S=5476184,E=5476191,C=47)
	CT(11334)=(S=5476192,E=5476199,C=58)
	CT(11335)=(S=5476216,E=5476223,C=24)
	CT(11336)=(S=5476224,E=5476239,C=58)
	CT(11337)=(S=5476240,E=5476247,C=51)
	CT(11338)=(S=5476248,E=5476255,C=18)
	CT(11339)=(S=5476256,E=5476263,C=58)
	CT(11340)=(S=5476264,E=5476271,C=172)
	CT(11341)=(S=5476272,E=5476279,C=51)
	CT(11342)=(S=5476280,E=5476287,C=36)
	CT(11343)=(S=5476288,E=5476295,C=18)
	CT(11344)=(S=5476296,E=5476303,C=31)
	CT(11345)=(S=5476304,E=5476311,C=28)
	CT(11346)=(S=5476312,E=5476319,C=47)
	CT(11347)=(S=5476320,E=5476327,C=14)
	CT(11348)=(S=5476328,E=5476335,C=31)
	CT(11349)=(S=5476336,E=5476343,C=13)
	CT(11350)=(S=5476344,E=5476351,C=51)
	CT(11351)=(S=5476352,E=5476415,C=12)
	CT(11352)=(S=5476416,E=5476479,C=58)
	CT(11353)=(S=5476480,E=5476543,C=21)
	CT(11354)=(S=5476544,E=5476607,C=34)
	CT(11355)=(S=5476608,E=5476671,C=19)
	CT(11356)=(S=5476672,E=5476735,C=12)
	CT(11357)=(S=5476736,E=5476799,C=58)
	CT(11358)=(S=5476800,E=5476863,C=47)
	CT(11359)=(S=5476864,E=5476927,C=14)
	CT(11360)=(S=5476928,E=5476991,C=18)
	CT(11361)=(S=5476992,E=5477119,C=47)
	CT(11362)=(S=5477120,E=5477183,C=25)
	CT(11363)=(S=5477184,E=5477247,C=15)
	CT(11364)=(S=5477248,E=5477311,C=63)
	CT(11365)=(S=5477312,E=5477375,C=26)
	CT(11366)=(S=5477376,E=5477439,C=33)
	CT(11367)=(S=5477440,E=5477503,C=51)
	CT(11368)=(S=5477504,E=5477567,C=14)
	CT(11369)=(S=5477568,E=5477631,C=47)
	CT(11370)=(S=5477632,E=5477695,C=18)
	CT(11371)=(S=5477696,E=5477759,C=36)
	CT(11372)=(S=5477760,E=5477823,C=15)
	CT(11373)=(S=5477824,E=5477887,C=18)
	CT(11374)=(S=5477888,E=5477951,C=34)
	CT(11375)=(S=5477952,E=5478015,C=47)
	CT(11376)=(S=5478016,E=5478079,C=19)
	CT(11377)=(S=5478080,E=5478143,C=25)
	CT(11378)=(S=5478144,E=5478159,C=18)
	CT(11379)=(S=5478160,E=5478175,C=28)
	CT(11380)=(S=5478176,E=5478191,C=58)
	CT(11381)=(S=5478192,E=5478207,C=28)
	CT(11382)=(S=5478208,E=5478271,C=14)
	CT(11383)=(S=5478272,E=5478335,C=24)
	CT(11384)=(S=5478336,E=5478399,C=14)
	CT(11385)=(S=5478400,E=5480447,C=12)
	CT(11386)=(S=5480448,E=5481471,C=36)
	CT(11387)=(S=5481472,E=5481727,C=42)
	CT(11388)=(S=5481728,E=5481983,C=25)
	CT(11389)=(S=5481984,E=5482015,C=19)
	CT(11390)=(S=5482016,E=5482047,C=55)
	CT(11391)=(S=5482048,E=5482079,C=14)
	CT(11392)=(S=5482080,E=5482111,C=18)
	CT(11393)=(S=5482112,E=5482143,C=34)
	CT(11394)=(S=5482144,E=5482175,C=14)
	CT(11395)=(S=5482176,E=5482207,C=38)
	CT(11396)=(S=5482208,E=5482271,C=18)
	CT(11397)=(S=5482272,E=5482303,C=12)
	CT(11398)=(S=5482304,E=5482367,C=18)
	CT(11399)=(S=5482368,E=5482399,C=12)
	CT(11400)=(S=5482400,E=5482431,C=14)
	CT(11401)=(S=5482432,E=5482463,C=36)
	CT(11402)=(S=5482464,E=5482495,C=33)
	CT(11403)=(S=5482496,E=5482559,C=32)
	CT(11404)=(S=5482560,E=5482623,C=58)
	CT(11405)=(S=5482624,E=5482687,C=71)
	CT(11406)=(S=5482688,E=5482751,C=19)
	CT(11407)=(S=5482752,E=5482815,C=28)
	CT(11408)=(S=5482816,E=5482879,C=12)
	CT(11409)=(S=5482880,E=5482943,C=28)
	CT(11410)=(S=5482944,E=5483007,C=18)
	CT(11411)=(S=5483008,E=5483071,C=28)
	CT(11412)=(S=5483072,E=5483199,C=14)
	CT(11413)=(S=5483200,E=5483263,C=31)
	CT(11414)=(S=5483264,E=5483327,C=44)
	CT(11415)=(S=5483328,E=5483391,C=18)
	CT(11416)=(S=5483392,E=5483455,C=28)
	CT(11417)=(S=5483456,E=5483519,C=22)
	CT(11418)=(S=5483520,E=5483583,C=18)
	CT(11419)=(S=5483584,E=5483647,C=19)
	CT(11420)=(S=5483648,E=5483711,C=36)
	CT(11421)=(S=5483712,E=5483775,C=34)
	CT(11422)=(S=5483776,E=5483839,C=48)
	CT(11423)=(S=5483840,E=5483903,C=12)
	CT(11424)=(S=5483904,E=5483967,C=25)
	CT(11425)=(S=5483968,E=5484031,C=34)
	CT(11426)=(S=5484032,E=5484095,C=21)
	CT(11427)=(S=5484096,E=5484159,C=19)
	CT(11428)=(S=5484160,E=5484191,C=22)
	CT(11429)=(S=5484192,E=5484223,C=63)
	CT(11430)=(S=5484224,E=5484287,C=18)
	CT(11431)=(S=5484288,E=5484351,C=15)
	CT(11432)=(S=5484352,E=5484415,C=42)
	CT(11433)=(S=5484416,E=5484479,C=58)
	CT(11434)=(S=5484480,E=5484543,C=25)
	CT(11435)=(S=5484544,E=5488639,C=19)
	CT(11436)=(S=5488640,E=5492735,C=12)
	CT(11437)=(S=5492736,E=5492991,C=33)
	CT(11438)=(S=5492992,E=5493247,C=19)
	CT(11439)=(S=5493248,E=5493503,C=63)
	CT(11440)=(S=5493504,E=5493759,C=15)
	CT(11441)=(S=5493760,E=5494015,C=22)
	CT(11442)=(S=5494016,E=5494271,C=25)
	CT(11443)=(S=5494272,E=5494527,C=12)
	CT(11444)=(S=5494528,E=5494783,C=42)
	CT(11445)=(S=5494784,E=5494815,C=47)
	CT(11446)=(S=5494816,E=5494847,C=50)
	CT(11447)=(S=5494848,E=5494879,C=14)
	CT(11448)=(S=5494880,E=5494911,C=19)
	CT(11449)=(S=5494912,E=5494943,C=14)
	CT(11450)=(S=5494944,E=5494975,C=15)
	CT(11451)=(S=5494976,E=5495007,C=42)
	CT(11452)=(S=5495008,E=5495039,C=28)
	CT(11453)=(S=5495040,E=5495103,C=18)
	CT(11454)=(S=5495104,E=5495135,C=49)
	CT(11455)=(S=5495136,E=5495167,C=14)
	CT(11456)=(S=5495168,E=5495199,C=49)
	CT(11457)=(S=5495200,E=5495231,C=14)
	CT(11458)=(S=5495232,E=5495263,C=18)
	CT(11459)=(S=5495264,E=5495295,C=25)
	CT(11460)=(S=5495296,E=5495327,C=14)
	CT(11461)=(S=5495328,E=5495359,C=28)
	CT(11462)=(S=5495360,E=5495391,C=19)
	CT(11463)=(S=5495392,E=5495423,C=58)
	CT(11464)=(S=5495424,E=5495455,C=14)
	CT(11465)=(S=5495456,E=5495487,C=42)
	CT(11466)=(S=5495488,E=5495519,C=53)
	CT(11467)=(S=5495520,E=5495551,C=31)
	CT(11468)=(S=5495552,E=5495583,C=18)
	CT(11469)=(S=5495584,E=5495615,C=14)
	CT(11470)=(S=5495616,E=5495647,C=36)
	CT(11471)=(S=5495656,E=5495663,C=28)
	CT(11472)=(S=5495664,E=5495679,C=34)
	CT(11473)=(S=5495680,E=5495711,C=18)
	CT(11474)=(S=5495712,E=5495743,C=42)
	CT(11475)=(S=5495744,E=5495775,C=19)
	CT(11476)=(S=5495776,E=5495807,C=18)
	CT(11477)=(S=5495808,E=5495839,C=15)
	CT(11478)=(S=5495840,E=5495903,C=18)
	CT(11479)=(S=5495904,E=5495935,C=58)
	CT(11480)=(S=5495936,E=5495967,C=28)
	CT(11481)=(S=5495968,E=5495999,C=18)
	CT(11482)=(S=5496000,E=5496031,C=63)
	CT(11483)=(S=5496032,E=5496095,C=18)
	CT(11484)=(S=5496096,E=5496159,C=28)
	CT(11485)=(S=5496160,E=5496191,C=15)
	CT(11486)=(S=5496192,E=5496223,C=24)
	CT(11487)=(S=5496224,E=5496287,C=18)
	CT(11488)=(S=5496288,E=5496319,C=28)
	CT(11489)=(S=5496320,E=5496351,C=18)
	CT(11490)=(S=5496352,E=5496383,C=70)
	CT(11491)=(S=5496384,E=5496447,C=18)
	CT(11492)=(S=5496448,E=5496479,C=34)
	CT(11493)=(S=5496480,E=5496511,C=51)
	CT(11494)=(S=5496512,E=5496543,C=18)
	CT(11495)=(S=5496544,E=5496575,C=14)
	CT(11496)=(S=5496576,E=5496607,C=19)
	CT(11497)=(S=5496608,E=5496639,C=36)
	CT(11498)=(S=5496640,E=5496671,C=28)
	CT(11499)=(S=5496672,E=5496703,C=14)
	CT(11500)=(S=5496704,E=5496735,C=60)
	CT(11501)=(S=5496736,E=5496767,C=21)
	CT(11502)=(S=5496768,E=5496799,C=14)
	CT(11503)=(S=5496800,E=5496831,C=21)
	CT(11504)=(S=5496832,E=5497343,C=15)
	CT(11505)=(S=5497344,E=5497855,C=19)
	CT(11506)=(S=5497856,E=5497983,C=51)
	CT(11507)=(S=5497984,E=5498111,C=34)
	CT(11508)=(S=5498112,E=5498239,C=14)
	CT(11509)=(S=5498240,E=5498367,C=18)
	CT(11510)=(S=5498368,E=5498495,C=58)
	CT(11511)=(S=5498496,E=5498751,C=25)
	CT(11512)=(S=5498752,E=5498879,C=14)
	CT(11513)=(S=5498880,E=5499135,C=36)
	CT(11514)=(S=5499136,E=5499391,C=19)
	CT(11515)=(S=5499392,E=5499647,C=18)
	CT(11516)=(S=5499648,E=5499903,C=22)
	CT(11517)=(S=5499904,E=5500159,C=28)
	CT(11518)=(S=5500160,E=5500415,C=18)
	CT(11519)=(S=5500416,E=5500671,C=58)
	CT(11520)=(S=5500672,E=5500927,C=18)
	CT(11521)=(S=5500928,E=5501055,C=33)
	CT(11522)=(S=5501056,E=5501183,C=21)
	CT(11523)=(S=5501184,E=5501311,C=60)
	CT(11524)=(S=5501312,E=5501439,C=19)
	CT(11525)=(S=5501440,E=5501471,C=27)
	CT(11526)=(S=5501472,E=5501503,C=28)
	CT(11527)=(S=5501504,E=5501535,C=58)
	CT(11528)=(S=5501536,E=5501551,C=31)
	CT(11529)=(S=5501552,E=5501567,C=58)
	CT(11530)=(S=5501568,E=5501695,C=18)
	CT(11531)=(S=5501696,E=5501703,C=28)
	CT(11532)=(S=5501704,E=5501711,C=70)
	CT(11533)=(S=5501712,E=5501719,C=12)
	CT(11534)=(S=5501720,E=5501727,C=19)
	CT(11535)=(S=5501728,E=5501735,C=58)
	CT(11536)=(S=5501736,E=5501759,C=28)
	CT(11537)=(S=5501760,E=5501775,C=18)
	CT(11538)=(S=5501776,E=5501783,C=28)
	CT(11539)=(S=5501784,E=5501791,C=60)
	CT(11540)=(S=5501792,E=5501799,C=14)
	CT(11541)=(S=5501800,E=5501807,C=58)
	CT(11542)=(S=5501808,E=5501815,C=28)
	CT(11543)=(S=5501824,E=5501951,C=27)
	CT(11544)=(S=5501952,E=5502079,C=43)
	CT(11545)=(S=5502080,E=5502335,C=14)
	CT(11546)=(S=5502336,E=5502463,C=47)
	CT(11547)=(S=5502464,E=5502591,C=28)
	CT(11548)=(S=5502592,E=5502719,C=18)
	CT(11549)=(S=5502720,E=5502847,C=36)
	CT(11550)=(S=5502848,E=5502975,C=25)
	CT(11551)=(S=5502976,E=5505023,C=19)
	CT(11552)=(S=5505024,E=5506047,C=50)
	CT(11553)=(S=5506048,E=5507071,C=12)
	CT(11554)=(S=5507072,E=5507583,C=14)
	CT(11555)=(S=5507584,E=5507839,C=58)
	CT(11556)=(S=5507840,E=5508095,C=28)
	CT(11557)=(S=5508096,E=5508607,C=14)
	CT(11558)=(S=5508608,E=5508863,C=12)
	CT(11559)=(S=5508864,E=5509119,C=44)
	CT(11560)=(S=5509120,E=5509151,C=25)
	CT(11561)=(S=5509152,E=5509183,C=71)
	CT(11562)=(S=5509184,E=5509215,C=34)
	CT(11563)=(S=5509216,E=5509247,C=33)
	CT(11564)=(S=5509248,E=5509279,C=18)
	CT(11565)=(S=5509280,E=5509311,C=24)
	CT(11566)=(S=5509312,E=5509343,C=27)
	CT(11567)=(S=5509344,E=5509375,C=28)
	CT(11568)=(S=5509376,E=5509407,C=18)
	CT(11569)=(S=5509408,E=5509439,C=15)
	CT(11570)=(S=5509440,E=5509471,C=37)
	CT(11571)=(S=5509472,E=5509503,C=28)
	CT(11572)=(S=5509504,E=5509535,C=49)
	CT(11573)=(S=5509536,E=5509567,C=28)
	CT(11574)=(S=5509568,E=5509599,C=19)
	CT(11575)=(S=5509600,E=5509631,C=18)
	CT(11576)=(S=5509632,E=5509663,C=25)
	CT(11577)=(S=5509664,E=5509727,C=61)
	CT(11578)=(S=5509728,E=5509759,C=18)
	CT(11579)=(S=5509760,E=5509791,C=15)
	CT(11580)=(S=5509792,E=5509823,C=63)
	CT(11581)=(S=5509824,E=5509855,C=14)
	CT(11582)=(S=5509856,E=5509887,C=21)
	CT(11583)=(S=5509888,E=5509919,C=28)
	CT(11584)=(S=5509920,E=5509951,C=14)
	CT(11585)=(S=5509952,E=5509983,C=33)
	CT(11586)=(S=5509984,E=5510015,C=14)
	CT(11587)=(S=5510016,E=5510047,C=19)
	CT(11588)=(S=5510048,E=5510111,C=28)
	CT(11589)=(S=5510112,E=5510143,C=14)
	CT(11590)=(S=5510144,E=5510175,C=25)
	CT(11591)=(S=5510176,E=5510207,C=34)
	CT(11592)=(S=5510208,E=5510239,C=96)
	CT(11593)=(S=5510240,E=5510271,C=27)
	CT(11594)=(S=5510272,E=5510303,C=47)
	CT(11595)=(S=5510304,E=5510335,C=42)
	CT(11596)=(S=5510336,E=5510367,C=14)
	CT(11597)=(S=5510368,E=5510399,C=52)
	CT(11598)=(S=5510400,E=5510431,C=50)
	CT(11599)=(S=5510432,E=5510463,C=28)
	CT(11600)=(S=5510464,E=5510495,C=18)
	CT(11601)=(S=5510496,E=5510527,C=33)
	CT(11602)=(S=5510528,E=5510559,C=14)
	CT(11603)=(S=5510592,E=5510623,C=51)
	CT(11604)=(S=5510624,E=5510655,C=18)
	CT(11605)=(S=5510656,E=5510687,C=51)
	CT(11606)=(S=5510688,E=5510719,C=46)
	CT(11607)=(S=5510720,E=5510751,C=12)
	CT(11608)=(S=5510752,E=5510783,C=28)
	CT(11609)=(S=5510784,E=5510815,C=18)
	CT(11610)=(S=5510816,E=5510847,C=14)
	CT(11611)=(S=5510848,E=5510879,C=18)
	CT(11612)=(S=5510880,E=5510911,C=23)
	CT(11613)=(S=5510912,E=5510943,C=14)
	CT(11614)=(S=5510944,E=5510975,C=18)
	CT(11615)=(S=5510976,E=5511007,C=28)
	CT(11616)=(S=5511008,E=5511039,C=23)
	CT(11617)=(S=5511040,E=5511071,C=19)
	CT(11618)=(S=5511072,E=5511103,C=58)
	CT(11619)=(S=5511104,E=5511135,C=21)
	CT(11620)=(S=5511136,E=5511167,C=28)
	CT(11621)=(S=5511168,E=5513215,C=36)
	CT(11622)=(S=5513216,E=5513471,C=44)
	CT(11623)=(S=5513472,E=5513727,C=15)
	CT(11624)=(S=5513728,E=5513983,C=47)
	CT(11625)=(S=5513984,E=5514239,C=36)
	CT(11626)=(S=5514240,E=5514495,C=106)
	CT(11627)=(S=5514496,E=5514751,C=12)
	CT(11628)=(S=5514752,E=5514759,C=18)
	CT(11629)=(S=5514760,E=5514767,C=27)
	CT(11630)=(S=5514768,E=5514783,C=58)
	CT(11631)=(S=5514784,E=5514799,C=14)
	CT(11632)=(S=5514800,E=5514815,C=15)
	CT(11633)=(S=5514816,E=5514831,C=28)
	CT(11634)=(S=5514832,E=5514863,C=58)
	CT(11635)=(S=5514864,E=5514879,C=42)
	CT(11636)=(S=5514880,E=5514895,C=60)
	CT(11637)=(S=5514896,E=5514911,C=27)
	CT(11638)=(S=5514912,E=5514927,C=58)
	CT(11639)=(S=5514928,E=5514943,C=18)
	CT(11640)=(S=5514944,E=5514959,C=28)
	CT(11641)=(S=5514960,E=5514975,C=58)
	CT(11642)=(S=5514976,E=5514991,C=36)
	CT(11643)=(S=5514992,E=5515007,C=51)
	CT(11644)=(S=5515008,E=5515039,C=36)
	CT(11645)=(S=5515040,E=5515071,C=12)
	CT(11646)=(S=5515072,E=5515119,C=28)
	CT(11647)=(S=5515120,E=5515127,C=13)
	CT(11648)=(S=5515128,E=5515135,C=36)
	CT(11649)=(S=5515136,E=5515143,C=18)
	CT(11650)=(S=5515144,E=5515151,C=13)
	CT(11651)=(S=5515152,E=5515159,C=34)
	CT(11652)=(S=5515160,E=5515167,C=28)
	CT(11653)=(S=5515168,E=5515183,C=58)
	CT(11654)=(S=5515184,E=5515191,C=26)
	CT(11655)=(S=5515192,E=5515199,C=15)
	CT(11656)=(S=5515216,E=5515231,C=52)
	CT(11657)=(S=5515232,E=5515239,C=25)
	CT(11658)=(S=5515240,E=5515247,C=63)
	CT(11659)=(S=5515248,E=5515263,C=18)
	CT(11660)=(S=5515264,E=5515327,C=13)
	CT(11661)=(S=5515328,E=5515391,C=51)
	CT(11662)=(S=5515392,E=5515519,C=58)
	CT(11663)=(S=5515520,E=5515647,C=52)
	CT(11664)=(S=5515648,E=5515775,C=36)
	CT(11665)=(S=5515776,E=5515903,C=18)
	CT(11666)=(S=5515904,E=5516031,C=33)
	CT(11667)=(S=5516032,E=5516159,C=14)
	CT(11668)=(S=5516160,E=5516287,C=51)
	CT(11669)=(S=5516288,E=5516415,C=37)
	CT(11670)=(S=5516416,E=5516543,C=28)
	CT(11671)=(S=5516544,E=5516799,C=14)
	CT(11672)=(S=5516800,E=5516927,C=28)
	CT(11673)=(S=5516928,E=5517055,C=44)
	CT(11674)=(S=5517056,E=5517183,C=71)
	CT(11675)=(S=5517184,E=5517187,C=18)
	CT(11676)=(S=5517188,E=5517189,C=38)
	CT(11677)=(S=5517190,E=5517201,C=18)
	CT(11678)=(S=5517202,E=5517202,C=38)
	CT(11679)=(S=5517203,E=5517209,C=18)
	CT(11680)=(S=5517210,E=5517211,C=38)
	CT(11681)=(S=5517212,E=5517221,C=18)
	CT(11682)=(S=5517222,E=5517223,C=38)
	CT(11683)=(S=5517224,E=5517234,C=18)
	CT(11684)=(S=5517235,E=5517236,C=38)
	CT(11685)=(S=5517237,E=5517239,C=18)
	CT(11686)=(S=5517240,E=5517241,C=38)
	CT(11687)=(S=5517242,E=5517244,C=18)
	CT(11688)=(S=5517245,E=5517245,C=38)
	CT(11689)=(S=5517246,E=5517247,C=18)
	CT(11690)=(S=5517248,E=5517311,C=26)
	CT(11691)=(S=5517312,E=5517823,C=27)
	CT(11692)=(S=5517824,E=5518079,C=55)
	CT(11693)=(S=5518080,E=5518143,C=37)
	CT(11694)=(S=5518144,E=5518207,C=18)
	CT(11695)=(S=5518208,E=5518271,C=14)
	CT(11696)=(S=5518272,E=5518303,C=18)
	CT(11697)=(S=5518304,E=5518335,C=63)
	CT(11698)=(S=5518336,E=5518399,C=55)
	CT(11699)=(S=5518400,E=5518463,C=18)
	CT(11700)=(S=5518464,E=5518527,C=52)
	CT(11701)=(S=5518528,E=5518655,C=27)
	CT(11702)=(S=5518656,E=5518719,C=36)
	CT(11703)=(S=5518720,E=5518783,C=13)
	CT(11704)=(S=5518784,E=5518847,C=18)
	CT(11705)=(S=5518912,E=5518975,C=98)
	CT(11706)=(S=5518976,E=5519039,C=51)
	CT(11707)=(S=5519040,E=5519103,C=18)
	CT(11708)=(S=5519104,E=5519167,C=44)
	CT(11709)=(S=5519168,E=5519231,C=19)
	CT(11710)=(S=5519232,E=5519295,C=12)
	CT(11711)=(S=5519296,E=5519359,C=34)
	CT(11712)=(S=5519360,E=5521407,C=28)
	CT(11713)=(S=5521408,E=5523455,C=14)
	CT(11714)=(S=5523456,E=5524479,C=34)
	CT(11715)=(S=5524480,E=5525503,C=25)
	CT(11716)=(S=5525504,E=5527551,C=36)
	CT(11717)=(S=5527552,E=5528063,C=25)
	CT(11718)=(S=5528064,E=5528575,C=21)
	CT(11719)=(S=5528576,E=5529087,C=14)
	CT(11720)=(S=5529088,E=5529599,C=17)
	CT(11721)=(S=5529600,E=5531647,C=12)
	CT(11722)=(S=5531648,E=5532671,C=36)
	CT(11723)=(S=5532672,E=5533695,C=17)
	CT(11724)=(S=5533696,E=5535743,C=42)
	CT(11725)=(S=5535744,E=5537791,C=25)
	CT(11726)=(S=5537792,E=5554175,C=28)
	CT(11727)=(S=5554176,E=5556223,C=49)
	CT(11728)=(S=5556224,E=5556607,C=28)
	CT(11729)=(S=5556608,E=5556671,C=18)
	CT(11730)=(S=5556672,E=5556687,C=51)
	CT(11731)=(S=5556688,E=5556703,C=58)
	CT(11732)=(S=5556704,E=5556719,C=68)
	CT(11733)=(S=5556720,E=5556735,C=18)
	CT(11734)=(S=5556736,E=5556991,C=27)
	CT(11735)=(S=5556992,E=5557247,C=63)
	CT(11736)=(S=5557248,E=5557503,C=18)
	CT(11737)=(S=5557504,E=5557535,C=58)
	CT(11738)=(S=5557536,E=5557567,C=27)
	CT(11739)=(S=5557568,E=5557599,C=13)
	CT(11740)=(S=5557600,E=5557631,C=106)
	CT(11741)=(S=5557632,E=5557663,C=12)
	CT(11742)=(S=5557664,E=5557695,C=58)
	CT(11743)=(S=5557696,E=5557759,C=22)
	CT(11744)=(S=5557760,E=5558015,C=50)
	CT(11745)=(S=5558016,E=5558271,C=13)
	CT(11746)=(S=5558272,E=5560319,C=27)
	CT(11747)=(S=5560320,E=5561343,C=19)
	CT(11748)=(S=5561344,E=5562367,C=15)
	CT(11749)=(S=5562368,E=5562879,C=50)
	CT(11750)=(S=5562880,E=5563391,C=34)
	CT(11751)=(S=5563392,E=5563903,C=17)
	CT(11752)=(S=5563904,E=5564415,C=47)
	CT(11753)=(S=5564416,E=5564543,C=25)
	CT(11754)=(S=5564544,E=5564671,C=38)
	CT(11755)=(S=5564672,E=5564799,C=106)
	CT(11756)=(S=5564800,E=5564927,C=14)
	CT(11757)=(S=5564928,E=5564943,C=58)
	CT(11758)=(S=5564944,E=5564959,C=14)
	CT(11759)=(S=5564960,E=5564975,C=58)
	CT(11760)=(S=5564976,E=5564991,C=53)
	CT(11761)=(S=5564992,E=5565007,C=47)
	CT(11762)=(S=5565008,E=5565023,C=24)
	CT(11763)=(S=5565024,E=5565039,C=31)
	CT(11764)=(S=5565040,E=5565055,C=58)
	CT(11765)=(S=5565056,E=5565183,C=27)
	CT(11766)=(S=5565184,E=5565311,C=23)
	CT(11767)=(S=5565312,E=5565439,C=28)
	CT(11768)=(S=5565440,E=5565567,C=50)
	CT(11769)=(S=5565568,E=5565695,C=25)
	CT(11770)=(S=5565696,E=5565823,C=18)
	CT(11771)=(S=5565824,E=5565951,C=60)
	CT(11772)=(S=5565952,E=5566079,C=24)
	CT(11773)=(S=5566080,E=5566207,C=51)
	CT(11774)=(S=5566208,E=5566271,C=38)
	CT(11775)=(S=5566272,E=5566335,C=12)
	CT(11776)=(S=5566336,E=5566463,C=47)
	CT(11777)=(S=5566464,E=5566527,C=44)
	CT(11778)=(S=5566528,E=5566591,C=47)
	CT(11779)=(S=5566592,E=5566655,C=15)
	CT(11780)=(S=5566656,E=5566719,C=20)
	CT(11781)=(S=5566720,E=5566783,C=26)
	CT(11782)=(S=5566784,E=5566847,C=34)
	CT(11783)=(S=5566848,E=5566975,C=36)
	CT(11784)=(S=5566976,E=5567039,C=18)
	CT(11785)=(S=5567040,E=5567103,C=33)
	CT(11786)=(S=5567104,E=5567167,C=51)
	CT(11787)=(S=5567168,E=5567231,C=18)
	CT(11788)=(S=5567232,E=5567295,C=19)
	CT(11789)=(S=5567296,E=5567359,C=38)
	CT(11790)=(S=5567360,E=5567423,C=27)
	CT(11791)=(S=5567424,E=5567487,C=36)
	CT(11792)=(S=5567488,E=5567551,C=18)
	CT(11793)=(S=5567552,E=5567615,C=33)
	CT(11794)=(S=5567616,E=5567679,C=36)
	CT(11795)=(S=5567680,E=5567743,C=19)
	CT(11796)=(S=5567744,E=5567807,C=36)
	CT(11797)=(S=5567808,E=5567871,C=71)
	CT(11798)=(S=5567872,E=5567935,C=28)
	CT(11799)=(S=5567936,E=5567999,C=60)
	CT(11800)=(S=5568000,E=5568063,C=36)
	CT(11801)=(S=5568064,E=5568071,C=28)
	CT(11802)=(S=5568072,E=5568079,C=101)
	CT(11803)=(S=5568080,E=5568087,C=14)
	CT(11804)=(S=5568088,E=5568095,C=19)
	CT(11805)=(S=5568096,E=5568103,C=57)
	CT(11806)=(S=5568104,E=5568111,C=12)
	CT(11807)=(S=5568112,E=5568119,C=14)
	CT(11808)=(S=5568120,E=5568127,C=28)
	CT(11809)=(S=5568128,E=5568135,C=19)
	CT(11810)=(S=5568136,E=5568143,C=14)
	CT(11811)=(S=5568144,E=5568151,C=15)
	CT(11812)=(S=5568152,E=5568159,C=14)
	CT(11813)=(S=5568160,E=5568167,C=33)
	CT(11814)=(S=5568168,E=5568207,C=14)
	CT(11815)=(S=5568208,E=5568215,C=25)
	CT(11816)=(S=5568216,E=5568231,C=12)
	CT(11817)=(S=5568232,E=5568239,C=34)
	CT(11818)=(S=5568240,E=5568247,C=24)
	CT(11819)=(S=5568248,E=5568255,C=28)
	CT(11820)=(S=5568256,E=5568259,C=38)
	CT(11821)=(S=5568260,E=5568263,C=95)
	CT(11822)=(S=5568264,E=5568273,C=38)
	CT(11823)=(S=5568274,E=5568275,C=25)
	CT(11824)=(S=5568276,E=5568287,C=38)
	CT(11825)=(S=5568288,E=5568295,C=13)
	CT(11826)=(S=5568296,E=5568299,C=95)
	CT(11827)=(S=5568300,E=5568303,C=13)
	CT(11828)=(S=5568304,E=5568383,C=38)
	CT(11829)=(S=5568384,E=5568447,C=27)
	CT(11830)=(S=5568448,E=5568511,C=15)
	CT(11831)=(S=5568512,E=5569535,C=47)
	CT(11832)=(S=5569536,E=5569599,C=51)
	CT(11833)=(S=5569664,E=5569727,C=18)
	CT(11834)=(S=5569728,E=5569791,C=14)
	CT(11835)=(S=5569792,E=5569855,C=34)
	CT(11836)=(S=5569856,E=5569919,C=18)
	CT(11837)=(S=5569920,E=5569983,C=15)
	CT(11838)=(S=5569984,E=5570047,C=47)
	CT(11839)=(S=5570048,E=5570111,C=22)
	CT(11840)=(S=5570112,E=5570175,C=28)
	CT(11841)=(S=5570176,E=5570239,C=15)
	CT(11842)=(S=5570240,E=5570303,C=18)
	CT(11843)=(S=5570304,E=5570367,C=72)
	CT(11844)=(S=5570368,E=5570431,C=24)
	CT(11845)=(S=5570432,E=5570495,C=104)
	CT(11846)=(S=5570496,E=5570559,C=52)
	CT(11847)=(S=5570560,E=5572607,C=34)
	CT(11848)=(S=5572608,E=5572671,C=19)
	CT(11849)=(S=5572672,E=5572735,C=28)
	CT(11850)=(S=5572800,E=5572863,C=14)
	CT(11851)=(S=5572864,E=5572927,C=38)
	CT(11852)=(S=5572928,E=5572991,C=26)
	CT(11853)=(S=5572992,E=5573055,C=103)
	CT(11854)=(S=5573056,E=5573119,C=60)
	CT(11855)=(S=5573120,E=5573167,C=52)
	CT(11856)=(S=5573168,E=5573183,C=57)
	CT(11857)=(S=5573184,E=5573247,C=49)
	CT(11858)=(S=5573248,E=5573311,C=12)
	CT(11859)=(S=5573312,E=5573375,C=28)
	CT(11860)=(S=5573376,E=5573439,C=19)
	CT(11861)=(S=5573440,E=5573503,C=58)
	CT(11862)=(S=5573504,E=5573567,C=51)
	CT(11863)=(S=5573568,E=5573631,C=19)
	CT(11864)=(S=5573632,E=5573695,C=36)
	CT(11865)=(S=5573696,E=5573759,C=14)
	CT(11866)=(S=5573760,E=5573823,C=12)
	CT(11867)=(S=5573824,E=5573887,C=18)
	CT(11868)=(S=5573888,E=5573951,C=42)
	CT(11869)=(S=5573952,E=5574015,C=33)
	CT(11870)=(S=5574016,E=5574079,C=28)
	CT(11871)=(S=5574080,E=5574143,C=14)
	CT(11872)=(S=5574144,E=5574207,C=51)
	CT(11873)=(S=5574208,E=5574271,C=58)
	CT(11874)=(S=5574272,E=5574335,C=12)
	CT(11875)=(S=5574336,E=5574399,C=28)
	CT(11876)=(S=5574400,E=5574463,C=26)
	CT(11877)=(S=5574464,E=5574591,C=18)
	CT(11878)=(S=5574592,E=5574655,C=60)
	CT(11879)=(S=5574656,E=5574911,C=28)
	CT(11880)=(S=5574912,E=5575167,C=36)
	CT(11881)=(S=5575168,E=5575423,C=15)
	CT(11882)=(S=5575424,E=5575679,C=27)
	CT(11883)=(S=5575680,E=5575935,C=15)
	CT(11884)=(S=5575936,E=5576191,C=18)
	CT(11885)=(S=5576192,E=5576447,C=28)
	CT(11886)=(S=5576448,E=5576703,C=47)
	CT(11887)=(S=5576704,E=5576831,C=24)
	CT(11888)=(S=5576832,E=5576959,C=19)
	CT(11889)=(S=5576960,E=5577215,C=28)
	CT(11890)=(S=5577216,E=5577343,C=49)
	CT(11891)=(S=5577344,E=5577471,C=18)
	CT(11892)=(S=5577472,E=5577599,C=49)
	CT(11893)=(S=5577600,E=5577727,C=24)
	CT(11894)=(S=5577728,E=5577791,C=18)
	CT(11895)=(S=5577792,E=5577855,C=49)
	CT(11896)=(S=5577856,E=5577919,C=58)
	CT(11897)=(S=5577920,E=5577983,C=18)
	CT(11898)=(S=5577984,E=5578047,C=37)
	CT(11899)=(S=5578048,E=5578111,C=47)
	CT(11900)=(S=5578112,E=5578175,C=20)
	CT(11901)=(S=5578176,E=5578239,C=55)
	CT(11902)=(S=5578240,E=5578303,C=19)
	CT(11903)=(S=5578304,E=5578367,C=54)
	CT(11904)=(S=5578368,E=5578431,C=19)
	CT(11905)=(S=5578432,E=5578495,C=18)
	CT(11906)=(S=5578496,E=5578527,C=42)
	CT(11907)=(S=5578528,E=5578591,C=14)
	CT(11908)=(S=5578592,E=5578599,C=60)
	CT(11909)=(S=5578600,E=5578607,C=12)
	CT(11910)=(S=5578608,E=5578623,C=18)
	CT(11911)=(S=5578624,E=5578631,C=25)
	CT(11912)=(S=5578632,E=5578639,C=14)
	CT(11913)=(S=5578640,E=5578647,C=12)
	CT(11914)=(S=5578648,E=5578655,C=56)
	CT(11915)=(S=5578656,E=5578663,C=15)
	CT(11916)=(S=5578664,E=5578671,C=14)
	CT(11917)=(S=5578672,E=5578679,C=18)
	CT(11918)=(S=5578680,E=5578687,C=28)
	CT(11919)=(S=5578688,E=5578719,C=19)
	CT(11920)=(S=5578720,E=5578751,C=58)
	CT(11921)=(S=5578752,E=5582847,C=15)
	CT(11922)=(S=5582848,E=5586943,C=25)
	CT(11923)=(S=5586944,E=5587455,C=17)
	CT(11924)=(S=5587456,E=5587967,C=50)
	CT(11925)=(S=5587968,E=5588479,C=12)
	CT(11926)=(S=5588480,E=5588991,C=33)
	CT(11927)=(S=5588992,E=5590015,C=22)
	CT(11928)=(S=5590016,E=5591039,C=47)
	CT(11929)=(S=5591040,E=5592063,C=24)
	CT(11930)=(S=5592064,E=5593087,C=25)
	CT(11931)=(S=5593088,E=5593119,C=28)
	CT(11932)=(S=5593120,E=5593151,C=49)
	CT(11933)=(S=5593152,E=5593183,C=27)
	CT(11934)=(S=5593184,E=5593215,C=36)
	CT(11935)=(S=5593216,E=5593247,C=21)
	CT(11936)=(S=5593248,E=5593279,C=18)
	CT(11937)=(S=5593280,E=5593311,C=15)
	CT(11938)=(S=5593312,E=5593343,C=14)
	CT(11939)=(S=5593344,E=5593375,C=27)
	CT(11940)=(S=5593376,E=5593407,C=55)
	CT(11941)=(S=5593408,E=5593439,C=19)
	CT(11942)=(S=5593440,E=5593471,C=18)
	CT(11943)=(S=5593472,E=5593503,C=15)
	CT(11944)=(S=5593504,E=5593535,C=58)
	CT(11945)=(S=5593536,E=5593567,C=27)
	CT(11946)=(S=5593568,E=5593599,C=24)
	CT(11947)=(S=5593600,E=5593631,C=34)
	CT(11948)=(S=5593632,E=5593663,C=14)
	CT(11949)=(S=5593664,E=5593695,C=36)
	CT(11950)=(S=5593696,E=5593727,C=18)
	CT(11951)=(S=5593728,E=5593759,C=42)
	CT(11952)=(S=5593760,E=5593791,C=50)
	CT(11953)=(S=5593792,E=5593823,C=31)
	CT(11954)=(S=5593824,E=5593843,C=14)
	CT(11955)=(S=5593844,E=5593847,C=13)
	CT(11956)=(S=5593848,E=5593855,C=14)
	CT(11957)=(S=5593856,E=5593887,C=63)
	CT(11958)=(S=5593888,E=5593919,C=14)
	CT(11959)=(S=5593920,E=5593951,C=25)
	CT(11960)=(S=5593952,E=5593960,C=18)
	CT(11961)=(S=5593961,E=5593961,C=38)
	CT(11962)=(S=5593962,E=5593962,C=18)
	CT(11963)=(S=5593963,E=5593967,C=38)
	CT(11964)=(S=5593968,E=5593971,C=18)
	CT(11965)=(S=5593972,E=5593973,C=38)
	CT(11966)=(S=5593974,E=5593978,C=18)
	CT(11967)=(S=5593979,E=5593983,C=38)
	CT(11968)=(S=5593984,E=5594015,C=51)
	CT(11969)=(S=5594016,E=5594047,C=49)
	CT(11970)=(S=5594048,E=5594079,C=18)
	CT(11971)=(S=5594080,E=5594111,C=14)
	CT(11972)=(S=5594112,E=5594143,C=18)
	CT(11973)=(S=5594144,E=5594175,C=33)
	CT(11974)=(S=5594176,E=5594207,C=14)
	CT(11975)=(S=5594240,E=5594271,C=36)
	CT(11976)=(S=5594272,E=5594335,C=14)
	CT(11977)=(S=5594336,E=5594367,C=59)
	CT(11978)=(S=5594368,E=5594399,C=28)
	CT(11979)=(S=5594400,E=5594431,C=18)
	CT(11980)=(S=5594432,E=5594463,C=28)
	CT(11981)=(S=5594464,E=5594495,C=33)
	CT(11982)=(S=5594496,E=5594527,C=18)
	CT(11983)=(S=5594528,E=5594559,C=33)
	CT(11984)=(S=5594560,E=5594591,C=70)
	CT(11985)=(S=5594592,E=5594623,C=27)
	CT(11986)=(S=5594624,E=5594687,C=18)
	CT(11987)=(S=5594688,E=5594719,C=57)
	CT(11988)=(S=5594720,E=5594751,C=102)
	CT(11989)=(S=5594752,E=5594783,C=59)
	CT(11990)=(S=5594784,E=5594815,C=188)
	CT(11991)=(S=5594816,E=5594847,C=15)
	CT(11992)=(S=5594848,E=5594879,C=70)
	CT(11993)=(S=5594880,E=5594911,C=14)
	CT(11994)=(S=5594912,E=5594943,C=27)
	CT(11995)=(S=5594944,E=5594975,C=51)
	CT(11996)=(S=5594976,E=5595007,C=14)
	CT(11997)=(S=5595008,E=5595071,C=18)
	CT(11998)=(S=5595072,E=5595103,C=12)
	CT(11999)=(S=5595104,E=5599231,C=37)
	CT(12000)=(S=5599232,E=5599263,C=25)
	CT(12001)=(S=5599264,E=5599295,C=18)
	CT(12002)=(S=5599296,E=5599327,C=39)
	CT(12003)=(S=5599328,E=5599359,C=18)
	CT(12004)=(S=5599360,E=5599391,C=27)
	CT(12005)=(S=5599392,E=5599423,C=19)
	CT(12006)=(S=5599424,E=5599455,C=58)
	CT(12007)=(S=5599456,E=5599487,C=28)
	CT(12008)=(S=5599488,E=5599519,C=62)
	CT(12009)=(S=5599520,E=5599551,C=18)
	CT(12010)=(S=5599552,E=5599583,C=14)
	CT(12011)=(S=5599584,E=5599615,C=43)
	CT(12012)=(S=5599616,E=5599647,C=18)
	CT(12013)=(S=5599648,E=5599679,C=27)
	CT(12014)=(S=5599680,E=5599711,C=18)
	CT(12015)=(S=5599712,E=5599743,C=36)
	CT(12016)=(S=5599744,E=5599775,C=18)
	CT(12017)=(S=5599776,E=5599807,C=57)
	CT(12018)=(S=5599808,E=5599839,C=18)
	CT(12019)=(S=5599840,E=5599871,C=43)
	CT(12020)=(S=5599872,E=5599903,C=28)
	CT(12021)=(S=5599904,E=5599935,C=18)
	CT(12022)=(S=5599936,E=5599967,C=31)
	CT(12023)=(S=5599968,E=5599999,C=40)
	CT(12024)=(S=5600000,E=5600031,C=28)
	CT(12025)=(S=5600032,E=5600063,C=14)
	CT(12026)=(S=5600064,E=5600095,C=16)
	CT(12027)=(S=5600096,E=5600127,C=60)
	CT(12028)=(S=5600128,E=5600159,C=69)
	CT(12029)=(S=5600160,E=5600191,C=18)
	CT(12030)=(S=5600192,E=5600223,C=62)
	CT(12031)=(S=5600224,E=5600255,C=18)
	CT(12032)=(S=5600256,E=5600287,C=14)
	CT(12033)=(S=5600288,E=5600319,C=12)
	CT(12034)=(S=5600320,E=5600351,C=48)
	CT(12035)=(S=5600352,E=5600383,C=18)
	CT(12036)=(S=5600384,E=5600415,C=15)
	CT(12037)=(S=5600416,E=5600447,C=14)
	CT(12038)=(S=5600448,E=5600479,C=28)
	CT(12039)=(S=5600480,E=5600511,C=15)
	CT(12040)=(S=5600512,E=5600543,C=58)
	CT(12041)=(S=5600544,E=5600575,C=40)
	CT(12042)=(S=5600576,E=5600607,C=18)
	CT(12043)=(S=5600608,E=5600639,C=20)
	CT(12044)=(S=5600640,E=5600671,C=12)
	CT(12045)=(S=5600672,E=5600735,C=19)
	CT(12046)=(S=5600768,E=5600799,C=14)
	CT(12047)=(S=5600800,E=5600831,C=12)
	CT(12048)=(S=5600832,E=5600863,C=51)
	CT(12049)=(S=5600864,E=5600895,C=40)
	CT(12050)=(S=5600896,E=5600903,C=33)
	CT(12051)=(S=5600904,E=5600911,C=18)
	CT(12052)=(S=5600912,E=5600919,C=15)
	CT(12053)=(S=5600920,E=5600927,C=14)
	CT(12054)=(S=5600928,E=5600935,C=28)
	CT(12055)=(S=5600936,E=5600943,C=14)
	CT(12056)=(S=5600944,E=5600951,C=18)
	CT(12057)=(S=5600952,E=5600959,C=57)
	CT(12058)=(S=5600960,E=5600967,C=51)
	CT(12059)=(S=5600968,E=5600975,C=19)
	CT(12060)=(S=5600976,E=5600983,C=49)
	CT(12061)=(S=5600984,E=5600991,C=34)
	CT(12062)=(S=5600992,E=5600999,C=18)
	CT(12063)=(S=5601000,E=5601007,C=14)
	CT(12064)=(S=5601008,E=5601015,C=25)
	CT(12065)=(S=5601016,E=5601023,C=28)
	CT(12066)=(S=5601024,E=5601031,C=34)
	CT(12067)=(S=5601032,E=5601039,C=50)
	CT(12068)=(S=5601040,E=5601055,C=14)
	CT(12069)=(S=5601056,E=5601063,C=37)
	CT(12070)=(S=5601064,E=5601071,C=15)
	CT(12071)=(S=5601072,E=5601079,C=36)
	CT(12072)=(S=5601080,E=5601087,C=14)
	CT(12073)=(S=5601088,E=5601095,C=37)
	CT(12074)=(S=5601096,E=5601103,C=18)
	CT(12075)=(S=5601104,E=5601111,C=14)
	CT(12076)=(S=5601112,E=5601119,C=33)
	CT(12077)=(S=5601120,E=5601127,C=14)
	CT(12078)=(S=5601128,E=5601135,C=36)
	CT(12079)=(S=5601136,E=5601143,C=14)
	CT(12080)=(S=5601144,E=5601151,C=72)
	CT(12081)=(S=5601152,E=5601159,C=19)
	CT(12082)=(S=5601160,E=5601167,C=27)
	CT(12083)=(S=5601168,E=5601175,C=18)
	CT(12084)=(S=5601176,E=5601191,C=28)
	CT(12085)=(S=5601192,E=5601199,C=12)
	CT(12086)=(S=5601200,E=5601207,C=15)
	CT(12087)=(S=5601208,E=5601215,C=49)
	CT(12088)=(S=5601216,E=5601223,C=25)
	CT(12089)=(S=5601224,E=5601231,C=36)
	CT(12090)=(S=5601232,E=5601239,C=28)
	CT(12091)=(S=5601240,E=5601247,C=49)
	CT(12092)=(S=5601248,E=5601255,C=14)
	CT(12093)=(S=5601256,E=5601263,C=34)
	CT(12094)=(S=5601264,E=5601271,C=49)
	CT(12095)=(S=5601272,E=5601279,C=14)
	CT(12096)=(S=5601280,E=5602303,C=38)
	CT(12097)=(S=5602304,E=5603327,C=42)
	CT(12098)=(S=5603328,E=5603583,C=58)
	CT(12099)=(S=5603584,E=5603711,C=24)
	CT(12100)=(S=5603712,E=5603839,C=23)
	CT(12101)=(S=5603840,E=5603967,C=51)
	CT(12102)=(S=5603968,E=5604095,C=17)
	CT(12103)=(S=5604096,E=5604223,C=47)
	CT(12104)=(S=5604224,E=5604351,C=28)
	CT(12105)=(S=5604352,E=5604479,C=41)
	CT(12106)=(S=5604480,E=5604607,C=33)
	CT(12107)=(S=5604608,E=5604735,C=14)
	CT(12108)=(S=5604736,E=5604863,C=26)
	CT(12109)=(S=5604864,E=5604991,C=47)
	CT(12110)=(S=5604992,E=5605119,C=63)
	CT(12111)=(S=5605120,E=5605247,C=33)
	CT(12112)=(S=5605248,E=5605375,C=71)
	CT(12113)=(S=5605376,E=5605887,C=25)
	CT(12114)=(S=5605888,E=5606399,C=21)
	CT(12115)=(S=5606400,E=5607423,C=18)
	CT(12116)=(S=5607424,E=5609471,C=36)
	CT(12117)=(S=5609472,E=5609727,C=25)
	CT(12118)=(S=5609728,E=5609983,C=37)
	CT(12119)=(S=5609984,E=5610239,C=45)
	CT(12120)=(S=5610240,E=5610495,C=25)
	CT(12121)=(S=5610496,E=5611007,C=47)
	CT(12122)=(S=5611008,E=5611015,C=28)
	CT(12123)=(S=5611024,E=5611031,C=12)
	CT(12124)=(S=5611032,E=5611039,C=34)
	CT(12125)=(S=5611040,E=5611047,C=59)
	CT(12126)=(S=5611048,E=5611055,C=14)
	CT(12127)=(S=5611056,E=5611063,C=18)
	CT(12128)=(S=5611064,E=5611079,C=14)
	CT(12129)=(S=5611080,E=5611087,C=60)
	CT(12130)=(S=5611088,E=5611095,C=15)
	CT(12131)=(S=5611096,E=5611103,C=18)
	CT(12132)=(S=5611104,E=5611111,C=37)
	CT(12133)=(S=5611112,E=5611119,C=49)
	CT(12134)=(S=5611120,E=5611135,C=12)
	CT(12135)=(S=5611136,E=5611143,C=104)
	CT(12136)=(S=5611144,E=5611151,C=14)
	CT(12137)=(S=5611152,E=5611159,C=41)
	CT(12138)=(S=5611160,E=5611167,C=14)
	CT(12139)=(S=5611168,E=5611175,C=36)
	CT(12140)=(S=5611176,E=5611183,C=25)
	CT(12141)=(S=5611184,E=5611191,C=28)
	CT(12142)=(S=5611192,E=5611199,C=18)
	CT(12143)=(S=5611200,E=5611207,C=28)
	CT(12144)=(S=5611208,E=5611215,C=36)
	CT(12145)=(S=5611216,E=5611223,C=49)
	CT(12146)=(S=5611224,E=5611231,C=38)
	CT(12147)=(S=5611232,E=5611239,C=42)
	CT(12148)=(S=5611240,E=5611247,C=34)
	CT(12149)=(S=5611248,E=5611255,C=18)
	CT(12150)=(S=5611256,E=5611263,C=36)
	CT(12151)=(S=5611264,E=5611271,C=31)
	CT(12152)=(S=5611272,E=5611279,C=28)
	CT(12153)=(S=5611280,E=5611287,C=63)
	CT(12154)=(S=5611288,E=5611295,C=20)
	CT(12155)=(S=5611296,E=5611311,C=18)
	CT(12156)=(S=5611312,E=5611319,C=50)
	CT(12157)=(S=5611320,E=5611327,C=14)
	CT(12158)=(S=5611328,E=5611343,C=37)
	CT(12159)=(S=5611344,E=5611359,C=14)
	CT(12160)=(S=5611360,E=5611367,C=36)
	CT(12161)=(S=5611368,E=5611375,C=71)
	CT(12162)=(S=5611376,E=5611383,C=36)
	CT(12163)=(S=5611384,E=5611391,C=69)
	CT(12164)=(S=5611392,E=5611407,C=14)
	CT(12165)=(S=5611408,E=5611415,C=15)
	CT(12166)=(S=5611416,E=5611423,C=14)
	CT(12167)=(S=5611424,E=5611431,C=17)
	CT(12168)=(S=5611432,E=5611439,C=14)
	CT(12169)=(S=5611440,E=5611447,C=15)
	CT(12170)=(S=5611448,E=5611455,C=14)
	CT(12171)=(S=5611456,E=5611471,C=15)
	CT(12172)=(S=5611472,E=5611479,C=14)
	CT(12173)=(S=5611480,E=5611487,C=61)
	CT(12174)=(S=5611488,E=5611495,C=18)
	CT(12175)=(S=5611496,E=5611503,C=36)
	CT(12176)=(S=5611504,E=5611511,C=28)
	CT(12177)=(S=5611512,E=5611519,C=14)
	CT(12178)=(S=5611520,E=5612543,C=33)
	CT(12179)=(S=5612544,E=5613567,C=27)
	CT(12180)=(S=5613568,E=5614591,C=12)
	CT(12181)=(S=5614592,E=5615615,C=18)
	CT(12182)=(S=5615616,E=5617663,C=28)
	CT(12183)=(S=5617664,E=5617919,C=36)
	CT(12184)=(S=5617920,E=5618175,C=26)
	CT(12185)=(S=5618176,E=5618431,C=38)
	CT(12186)=(S=5618432,E=5618687,C=51)
	CT(12187)=(S=5618688,E=5618751,C=47)
	CT(12188)=(S=5618752,E=5618879,C=19)
	CT(12189)=(S=5618880,E=5619199,C=14)
	CT(12190)=(S=5619200,E=5619455,C=28)
	CT(12191)=(S=5619456,E=5619711,C=24)
	CT(12192)=(S=5619712,E=5619775,C=18)
	CT(12193)=(S=5619776,E=5619839,C=25)
	CT(12194)=(S=5619840,E=5619903,C=18)
	CT(12195)=(S=5619904,E=5619967,C=12)
	CT(12196)=(S=5619968,E=5620031,C=33)
	CT(12197)=(S=5620032,E=5620095,C=18)
	CT(12198)=(S=5620096,E=5620111,C=42)
	CT(12199)=(S=5620112,E=5620127,C=14)
	CT(12200)=(S=5620128,E=5620159,C=42)
	CT(12201)=(S=5620160,E=5620223,C=58)
	CT(12202)=(S=5620224,E=5620287,C=19)
	CT(12203)=(S=5620288,E=5620351,C=23)
	CT(12204)=(S=5620352,E=5620415,C=19)
	CT(12205)=(S=5620416,E=5620419,C=47)
	CT(12206)=(S=5620420,E=5620423,C=35)
	CT(12207)=(S=5620424,E=5620427,C=55)
	CT(12208)=(S=5620428,E=5620463,C=47)
	CT(12209)=(S=5620464,E=5620471,C=58)
	CT(12210)=(S=5620472,E=5620475,C=35)
	CT(12211)=(S=5620476,E=5620477,C=28)
	CT(12212)=(S=5620478,E=5620479,C=24)
	CT(12213)=(S=5620480,E=5620543,C=19)
	CT(12214)=(S=5620544,E=5620607,C=28)
	CT(12215)=(S=5620608,E=5620671,C=18)
	CT(12216)=(S=5620672,E=5620735,C=34)
	CT(12217)=(S=5620736,E=5620799,C=51)
	CT(12218)=(S=5620800,E=5620863,C=27)
	CT(12219)=(S=5620864,E=5620927,C=51)
	CT(12220)=(S=5620928,E=5620991,C=55)
	CT(12221)=(S=5620992,E=5621119,C=28)
	CT(12222)=(S=5621120,E=5621183,C=19)
	CT(12223)=(S=5621184,E=5621247,C=48)
	CT(12224)=(S=5621248,E=5621311,C=26)
	CT(12225)=(S=5621312,E=5621375,C=18)
	CT(12226)=(S=5621376,E=5621439,C=31)
	CT(12227)=(S=5621440,E=5621503,C=58)
	CT(12228)=(S=5621504,E=5621567,C=42)
	CT(12229)=(S=5621568,E=5621695,C=28)
	CT(12230)=(S=5621696,E=5621759,C=14)
	CT(12231)=(S=5621760,E=5622015,C=27)
	CT(12232)=(S=5622016,E=5622271,C=49)
	CT(12233)=(S=5622272,E=5622287,C=18)
	CT(12234)=(S=5622288,E=5622303,C=24)
	CT(12235)=(S=5622304,E=5622335,C=58)
	CT(12236)=(S=5622336,E=5622351,C=24)
	CT(12237)=(S=5622368,E=5622383,C=58)
	CT(12238)=(S=5622384,E=5622399,C=46)
	CT(12239)=(S=5622400,E=5622415,C=18)
	CT(12240)=(S=5622416,E=5622431,C=58)
	CT(12241)=(S=5622448,E=5622463,C=18)
	CT(12242)=(S=5622480,E=5622495,C=58)
	CT(12243)=(S=5622496,E=5622527,C=18)
	CT(12244)=(S=5622528,E=5622591,C=36)
	CT(12245)=(S=5622592,E=5622655,C=12)
	CT(12246)=(S=5622656,E=5622783,C=24)
	CT(12247)=(S=5622784,E=5622803,C=38)
	CT(12248)=(S=5622804,E=5622805,C=12)
	CT(12249)=(S=5622806,E=5622807,C=38)
	CT(12250)=(S=5622808,E=5622809,C=12)
	CT(12251)=(S=5622810,E=5622813,C=38)
	CT(12252)=(S=5622814,E=5622815,C=26)
	CT(12253)=(S=5622816,E=5622819,C=38)
	CT(12254)=(S=5622820,E=5622823,C=25)
	CT(12255)=(S=5622824,E=5622837,C=38)
	CT(12256)=(S=5622838,E=5622839,C=28)
	CT(12257)=(S=5622840,E=5622847,C=38)
	CT(12258)=(S=5622848,E=5622851,C=95)
	CT(12259)=(S=5622852,E=5622857,C=38)
	CT(12260)=(S=5622858,E=5622858,C=170)
	CT(12261)=(S=5622859,E=5622859,C=38)
	CT(12262)=(S=5622860,E=5622861,C=26)
	CT(12263)=(S=5622862,E=5622863,C=38)
	CT(12264)=(S=5622864,E=5622879,C=26)
	CT(12265)=(S=5622880,E=5622881,C=28)
	CT(12266)=(S=5622882,E=5622887,C=38)
	CT(12267)=(S=5622888,E=5622889,C=26)
	CT(12268)=(S=5622890,E=5622895,C=38)
	CT(12269)=(S=5622896,E=5622899,C=95)
	CT(12270)=(S=5622900,E=5622903,C=38)
	CT(12271)=(S=5622904,E=5622905,C=24)
	CT(12272)=(S=5622906,E=5622911,C=38)
	CT(12273)=(S=5622912,E=5622915,C=26)
	CT(12274)=(S=5622916,E=5622917,C=24)
	CT(12275)=(S=5622918,E=5622919,C=38)
	CT(12276)=(S=5622920,E=5622921,C=24)
	CT(12277)=(S=5622922,E=5622923,C=38)
	CT(12278)=(S=5622924,E=5622927,C=25)
	CT(12279)=(S=5622928,E=5622931,C=38)
	CT(12280)=(S=5622932,E=5622935,C=95)
	CT(12281)=(S=5622936,E=5622943,C=38)
	CT(12282)=(S=5622944,E=5622947,C=95)
	CT(12283)=(S=5622948,E=5622977,C=38)
	CT(12284)=(S=5622978,E=5622979,C=24)
	CT(12285)=(S=5622980,E=5622983,C=38)
	CT(12286)=(S=5622984,E=5622991,C=25)
	CT(12287)=(S=5622992,E=5623007,C=26)
	CT(12288)=(S=5623008,E=5623039,C=38)
	CT(12289)=(S=5623040,E=5623295,C=28)
	CT(12290)=(S=5623296,E=5623551,C=44)
	CT(12291)=(S=5623552,E=5623807,C=33)
	CT(12292)=(S=5623808,E=5624319,C=23)
	CT(12293)=(S=5624320,E=5624831,C=14)
	CT(12294)=(S=5624832,E=5625983,C=28)
	CT(12295)=(S=5625984,E=5626111,C=71)
	CT(12296)=(S=5626112,E=5626239,C=47)
	CT(12297)=(S=5626240,E=5626367,C=51)
	CT(12298)=(S=5626368,E=5626495,C=34)
	CT(12299)=(S=5626496,E=5626623,C=24)
	CT(12300)=(S=5626624,E=5626751,C=25)
	CT(12301)=(S=5626752,E=5626879,C=58)
	CT(12302)=(S=5626880,E=5627007,C=48)
	CT(12303)=(S=5627008,E=5627135,C=28)
	CT(12304)=(S=5627136,E=5627263,C=27)
	CT(12305)=(S=5627264,E=5627519,C=58)
	CT(12306)=(S=5627520,E=5627551,C=14)
	CT(12307)=(S=5627552,E=5627583,C=46)
	CT(12308)=(S=5627584,E=5627591,C=36)
	CT(12309)=(S=5627592,E=5627599,C=46)
	CT(12310)=(S=5627600,E=5627615,C=28)
	CT(12311)=(S=5627616,E=5627623,C=36)
	CT(12312)=(S=5627624,E=5627627,C=55)
	CT(12313)=(S=5627628,E=5627775,C=36)
	CT(12314)=(S=5627776,E=5627903,C=31)
	CT(12315)=(S=5627904,E=5629951,C=19)
	CT(12316)=(S=5629952,E=5629983,C=28)
	CT(12317)=(S=5629984,E=5630015,C=14)
	CT(12318)=(S=5630016,E=5630047,C=42)
	CT(12319)=(S=5630048,E=5630079,C=18)
	CT(12320)=(S=5630080,E=5630111,C=44)
	CT(12321)=(S=5630112,E=5630143,C=19)
	CT(12322)=(S=5630144,E=5630175,C=72)
	CT(12323)=(S=5630176,E=5630207,C=58)
	CT(12324)=(S=5630208,E=5630271,C=28)
	CT(12325)=(S=5630272,E=5630303,C=18)
	CT(12326)=(S=5630304,E=5630335,C=42)
	CT(12327)=(S=5630336,E=5630367,C=18)
	CT(12328)=(S=5630368,E=5630399,C=14)
	CT(12329)=(S=5630400,E=5630431,C=12)
	CT(12330)=(S=5630432,E=5630463,C=24)
	CT(12331)=(S=5630464,E=5630527,C=18)
	CT(12332)=(S=5630528,E=5630559,C=14)
	CT(12333)=(S=5630560,E=5630591,C=18)
	CT(12334)=(S=5630592,E=5630623,C=14)
	CT(12335)=(S=5630624,E=5630655,C=60)
	CT(12336)=(S=5630656,E=5630687,C=49)
	CT(12337)=(S=5630688,E=5630719,C=36)
	CT(12338)=(S=5630720,E=5630751,C=19)
	CT(12339)=(S=5630752,E=5630783,C=18)
	CT(12340)=(S=5630784,E=5630815,C=37)
	CT(12341)=(S=5630816,E=5630847,C=14)
	CT(12342)=(S=5630848,E=5630879,C=15)
	CT(12343)=(S=5630880,E=5630943,C=18)
	CT(12344)=(S=5630944,E=5630975,C=24)
	CT(12345)=(S=5630976,E=5631007,C=18)
	CT(12346)=(S=5631008,E=5631039,C=28)
	CT(12347)=(S=5631040,E=5631071,C=24)
	CT(12348)=(S=5631072,E=5631111,C=14)
	CT(12349)=(S=5631112,E=5631119,C=36)
	CT(12350)=(S=5631120,E=5631127,C=14)
	CT(12351)=(S=5631128,E=5631135,C=12)
	CT(12352)=(S=5631136,E=5631167,C=18)
	CT(12353)=(S=5631168,E=5631199,C=28)
	CT(12354)=(S=5631200,E=5631263,C=42)
	CT(12355)=(S=5631264,E=5631295,C=18)
	CT(12356)=(S=5631296,E=5631327,C=28)
	CT(12357)=(S=5631328,E=5631359,C=12)
	CT(12358)=(S=5631360,E=5631391,C=23)
	CT(12359)=(S=5631392,E=5631423,C=58)
	CT(12360)=(S=5631424,E=5631455,C=14)
	CT(12361)=(S=5631456,E=5631487,C=71)
	CT(12362)=(S=5631488,E=5631519,C=25)
	CT(12363)=(S=5631520,E=5631551,C=40)
	CT(12364)=(S=5631552,E=5631583,C=50)
	CT(12365)=(S=5631584,E=5631615,C=31)
	CT(12366)=(S=5631616,E=5631647,C=28)
	CT(12367)=(S=5631648,E=5631679,C=42)
	CT(12368)=(S=5631680,E=5631711,C=19)
	CT(12369)=(S=5631712,E=5631743,C=28)
	CT(12370)=(S=5631744,E=5631775,C=101)
	CT(12371)=(S=5631776,E=5631807,C=18)
	CT(12372)=(S=5631808,E=5631839,C=33)
	CT(12373)=(S=5631840,E=5631871,C=28)
	CT(12374)=(S=5631872,E=5631903,C=51)
	CT(12375)=(S=5631904,E=5631935,C=15)
	CT(12376)=(S=5631936,E=5631967,C=26)
	CT(12377)=(S=5631968,E=5631999,C=33)
	CT(12378)=(S=5632000,E=5634047,C=21)
	CT(12379)=(S=5634048,E=5634303,C=71)
	CT(12380)=(S=5634304,E=5634559,C=18)
	CT(12381)=(S=5634560,E=5634815,C=17)
	CT(12382)=(S=5634816,E=5635071,C=25)
	CT(12383)=(S=5635072,E=5635327,C=27)
	CT(12384)=(S=5635328,E=5635583,C=55)
	CT(12385)=(S=5635584,E=5635839,C=60)
	CT(12386)=(S=5635840,E=5635855,C=58)
	CT(12387)=(S=5635856,E=5635871,C=28)
	CT(12388)=(S=5635872,E=5635887,C=27)
	CT(12389)=(S=5635888,E=5635903,C=44)
	CT(12390)=(S=5635904,E=5635919,C=60)
	CT(12391)=(S=5635936,E=5635951,C=44)
	CT(12392)=(S=5635952,E=5635967,C=38)
	CT(12393)=(S=5635968,E=5635983,C=51)
	CT(12394)=(S=5635984,E=5635999,C=42)
	CT(12395)=(S=5636000,E=5636015,C=51)
	CT(12396)=(S=5636016,E=5636023,C=31)
	CT(12397)=(S=5636024,E=5636031,C=18)
	CT(12398)=(S=5636032,E=5636047,C=49)
	CT(12399)=(S=5636048,E=5636063,C=36)
	CT(12400)=(S=5636064,E=5636079,C=14)
	CT(12401)=(S=5636080,E=5636095,C=58)
	CT(12402)=(S=5636096,E=5644287,C=14)
	CT(12403)=(S=5644288,E=5644799,C=42)
	CT(12404)=(S=5644800,E=5645311,C=38)
	CT(12405)=(S=5645312,E=5645823,C=94)
	CT(12406)=(S=5645824,E=5646079,C=44)
	CT(12407)=(S=5646080,E=5646335,C=49)
	CT(12408)=(S=5646336,E=5648383,C=63)
	CT(12409)=(S=5648384,E=5648639,C=24)
	CT(12410)=(S=5648640,E=5648895,C=33)
	CT(12411)=(S=5648896,E=5649151,C=47)
	CT(12412)=(S=5649152,E=5649407,C=23)
	CT(12413)=(S=5649408,E=5649663,C=24)
	CT(12414)=(S=5649664,E=5650175,C=14)
	CT(12415)=(S=5650176,E=5650431,C=26)
	CT(12416)=(S=5650432,E=5650559,C=28)
	CT(12417)=(S=5650560,E=5650687,C=42)
	CT(12418)=(S=5650688,E=5650815,C=26)
	CT(12419)=(S=5650816,E=5650943,C=68)
	CT(12420)=(S=5650944,E=5651071,C=52)
	CT(12421)=(S=5651072,E=5651199,C=24)
	CT(12422)=(S=5651200,E=5651327,C=42)
	CT(12423)=(S=5651328,E=5651455,C=50)
	CT(12424)=(S=5651456,E=5651583,C=23)
	CT(12425)=(S=5651584,E=5651711,C=47)
	CT(12426)=(S=5651712,E=5651839,C=52)
	CT(12427)=(S=5651840,E=5651967,C=33)
	CT(12428)=(S=5652032,E=5652095,C=18)
	CT(12429)=(S=5652096,E=5652159,C=27)
	CT(12430)=(S=5652160,E=5652223,C=94)
	CT(12431)=(S=5652224,E=5652287,C=14)
	CT(12432)=(S=5652288,E=5652383,C=58)
	CT(12433)=(S=5652384,E=5652415,C=60)
	CT(12434)=(S=5652416,E=5652447,C=33)
	CT(12435)=(S=5652448,E=5656575,C=12)
	CT(12436)=(S=5656576,E=5660671,C=36)
	CT(12437)=(S=5660672,E=5661695,C=16)
	CT(12438)=(S=5661696,E=5661951,C=44)
	CT(12439)=(S=5661952,E=5662207,C=50)
	CT(12440)=(S=5662208,E=5662463,C=18)
	CT(12441)=(S=5662464,E=5662719,C=28)
	CT(12442)=(S=5662720,E=5662722,C=14)
	CT(12443)=(S=5662723,E=5662734,C=38)
	CT(12444)=(S=5662735,E=5662735,C=36)
	CT(12445)=(S=5662736,E=5662739,C=38)
	CT(12446)=(S=5662740,E=5662743,C=95)
	CT(12447)=(S=5662744,E=5662747,C=25)
	CT(12448)=(S=5662748,E=5662751,C=38)
	CT(12449)=(S=5662752,E=5662767,C=26)
	CT(12450)=(S=5662768,E=5662783,C=19)
	CT(12451)=(S=5662784,E=5662787,C=38)
	CT(12452)=(S=5662788,E=5662791,C=95)
	CT(12453)=(S=5662792,E=5662799,C=38)
	CT(12454)=(S=5662800,E=5662831,C=26)
	CT(12455)=(S=5662832,E=5662839,C=38)
	CT(12456)=(S=5662840,E=5662843,C=95)
	CT(12457)=(S=5662844,E=5662845,C=38)
	CT(12458)=(S=5662846,E=5662846,C=14)
	CT(12459)=(S=5662847,E=5662871,C=38)
	CT(12460)=(S=5662872,E=5662879,C=19)
	CT(12461)=(S=5662880,E=5662887,C=43)
	CT(12462)=(S=5662888,E=5662919,C=38)
	CT(12463)=(S=5662920,E=5662927,C=19)
	CT(12464)=(S=5662928,E=5662928,C=14)
	CT(12465)=(S=5662929,E=5662931,C=38)
	CT(12466)=(S=5662932,E=5662933,C=14)
	CT(12467)=(S=5662934,E=5662941,C=38)
	CT(12468)=(S=5662942,E=5662945,C=14)
	CT(12469)=(S=5662946,E=5662951,C=38)
	CT(12470)=(S=5662952,E=5662967,C=26)
	CT(12471)=(S=5662968,E=5662971,C=95)
	CT(12472)=(S=5662972,E=5662973,C=12)
	CT(12473)=(S=5662974,E=5662976,C=38)
	CT(12474)=(S=5662977,E=5662977,C=36)
	CT(12475)=(S=5662978,E=5662978,C=14)
	CT(12476)=(S=5662979,E=5662980,C=38)
	CT(12477)=(S=5662981,E=5662981,C=36)
	CT(12478)=(S=5662982,E=5662982,C=14)
	CT(12479)=(S=5662983,E=5662987,C=38)
	CT(12480)=(S=5662988,E=5662988,C=14)
	CT(12481)=(S=5662989,E=5662993,C=38)
	CT(12482)=(S=5662994,E=5662994,C=28)
	CT(12483)=(S=5662995,E=5662995,C=38)
	CT(12484)=(S=5662996,E=5662999,C=95)
	CT(12485)=(S=5663000,E=5663007,C=38)
	CT(12486)=(S=5663008,E=5663015,C=13)
	CT(12487)=(S=5663016,E=5663023,C=26)
	CT(12488)=(S=5663024,E=5663031,C=15)
	CT(12489)=(S=5663032,E=5663087,C=38)
	CT(12490)=(S=5663088,E=5663095,C=19)
	CT(12491)=(S=5663096,E=5663103,C=38)
	CT(12492)=(S=5663104,E=5663119,C=26)
	CT(12493)=(S=5663120,E=5663131,C=38)
	CT(12494)=(S=5663132,E=5663135,C=25)
	CT(12495)=(S=5663136,E=5663151,C=38)
	CT(12496)=(S=5663152,E=5663152,C=14)
	CT(12497)=(S=5663153,E=5663153,C=38)
	CT(12498)=(S=5663154,E=5663154,C=14)
	CT(12499)=(S=5663155,E=5663159,C=38)
	CT(12500)=(S=5663160,E=5663161,C=14)
	CT(12501)=(S=5663162,E=5663163,C=38)
	CT(12502)=(S=5663164,E=5663167,C=95)
	CT(12503)=(S=5663168,E=5663171,C=38)
	CT(12504)=(S=5663172,E=5663172,C=14)
	CT(12505)=(S=5663173,E=5663175,C=38)
	CT(12506)=(S=5663176,E=5663179,C=95)
	CT(12507)=(S=5663180,E=5663187,C=38)
	CT(12508)=(S=5663188,E=5663189,C=12)
	CT(12509)=(S=5663190,E=5663200,C=38)
	CT(12510)=(S=5663201,E=5663201,C=14)
	CT(12511)=(S=5663202,E=5663202,C=38)
	CT(12512)=(S=5663203,E=5663203,C=13)
	CT(12513)=(S=5663204,E=5663204,C=38)
	CT(12514)=(S=5663205,E=5663206,C=14)
	CT(12515)=(S=5663207,E=5663210,C=38)
	CT(12516)=(S=5663211,E=5663211,C=170)
	CT(12517)=(S=5663212,E=5663215,C=38)
	CT(12518)=(S=5663216,E=5663216,C=28)
	CT(12519)=(S=5663217,E=5663217,C=38)
	CT(12520)=(S=5663218,E=5663219,C=14)
	CT(12521)=(S=5663220,E=5663223,C=38)
	CT(12522)=(S=5663224,E=5663227,C=95)
	CT(12523)=(S=5663228,E=5663229,C=38)
	CT(12524)=(S=5663230,E=5663231,C=12)
	CT(12525)=(S=5663232,E=5663239,C=38)
	CT(12526)=(S=5663240,E=5663242,C=14)
	CT(12527)=(S=5663243,E=5663244,C=38)
	CT(12528)=(S=5663245,E=5663247,C=14)
	CT(12529)=(S=5663248,E=5663251,C=95)
	CT(12530)=(S=5663252,E=5663254,C=38)
	CT(12531)=(S=5663255,E=5663255,C=29)
	CT(12532)=(S=5663256,E=5663257,C=26)
	CT(12533)=(S=5663258,E=5663324,C=38)
	CT(12534)=(S=5663325,E=5663325,C=170)
	CT(12535)=(S=5663326,E=5663327,C=14)
	CT(12536)=(S=5663328,E=5663328,C=24)
	CT(12537)=(S=5663329,E=5663331,C=38)
	CT(12538)=(S=5663332,E=5663333,C=14)
	CT(12539)=(S=5663334,E=5663334,C=13)
	CT(12540)=(S=5663335,E=5663336,C=38)
	CT(12541)=(S=5663337,E=5663339,C=14)
	CT(12542)=(S=5663340,E=5663340,C=38)
	CT(12543)=(S=5663341,E=5663343,C=14)
	CT(12544)=(S=5663344,E=5663345,C=38)
	CT(12545)=(S=5663346,E=5663350,C=14)
	CT(12546)=(S=5663351,E=5663362,C=38)
	CT(12547)=(S=5663363,E=5663363,C=170)
	CT(12548)=(S=5663364,E=5663369,C=38)
	CT(12549)=(S=5663370,E=5663373,C=14)
	CT(12550)=(S=5663374,E=5663391,C=38)
	CT(12551)=(S=5663392,E=5663399,C=19)
	CT(12552)=(S=5663400,E=5663405,C=38)
	CT(12553)=(S=5663406,E=5663407,C=14)
	CT(12554)=(S=5663408,E=5663423,C=38)
	CT(12555)=(S=5663424,E=5663431,C=26)
	CT(12556)=(S=5663432,E=5663487,C=38)
	CT(12557)=(S=5663488,E=5663503,C=26)
	CT(12558)=(S=5663504,E=5663513,C=38)
	CT(12559)=(S=5663514,E=5663517,C=14)
	CT(12560)=(S=5663518,E=5663529,C=38)
	CT(12561)=(S=5663530,E=5663530,C=36)
	CT(12562)=(S=5663531,E=5663535,C=38)
	CT(12563)=(S=5663536,E=5663537,C=19)
	CT(12564)=(S=5663538,E=5663557,C=38)
	CT(12565)=(S=5663558,E=5663560,C=14)
	CT(12566)=(S=5663561,E=5663561,C=38)
	CT(12567)=(S=5663562,E=5663563,C=14)
	CT(12568)=(S=5663564,E=5663567,C=38)
	CT(12569)=(S=5663568,E=5663583,C=26)
	CT(12570)=(S=5663584,E=5663587,C=15)
	CT(12571)=(S=5663588,E=5663589,C=38)
	CT(12572)=(S=5663590,E=5663590,C=44)
	CT(12573)=(S=5663591,E=5663607,C=38)
	CT(12574)=(S=5663608,E=5663615,C=49)
	CT(12575)=(S=5663616,E=5663631,C=14)
	CT(12576)=(S=5663632,E=5663647,C=26)
	CT(12577)=(S=5663648,E=5663659,C=38)
	CT(12578)=(S=5663660,E=5663663,C=26)
	CT(12579)=(S=5663664,E=5663676,C=38)
	CT(12580)=(S=5663677,E=5663678,C=12)
	CT(12581)=(S=5663679,E=5663679,C=28)
	CT(12582)=(S=5663680,E=5663680,C=38)
	CT(12583)=(S=5663681,E=5663683,C=14)
	CT(12584)=(S=5663684,E=5663687,C=38)
	CT(12585)=(S=5663688,E=5663695,C=36)
	CT(12586)=(S=5663696,E=5663711,C=26)
	CT(12587)=(S=5663712,E=5663729,C=38)
	CT(12588)=(S=5663730,E=5663731,C=19)
	CT(12589)=(S=5663732,E=5663743,C=38)
	CT(12590)=(S=5663744,E=5663871,C=61)
	CT(12591)=(S=5663872,E=5663999,C=37)
	CT(12592)=(S=5664000,E=5664031,C=36)
	CT(12593)=(S=5664032,E=5664063,C=26)
	CT(12594)=(S=5664064,E=5664095,C=50)
	CT(12595)=(S=5664096,E=5664127,C=25)
	CT(12596)=(S=5664128,E=5664159,C=15)
	CT(12597)=(S=5664160,E=5664191,C=25)
	CT(12598)=(S=5664192,E=5664223,C=18)
	CT(12599)=(S=5664224,E=5664255,C=28)
	CT(12600)=(S=5664256,E=5664287,C=18)
	CT(12601)=(S=5664288,E=5664319,C=55)
	CT(12602)=(S=5664320,E=5664351,C=28)
	CT(12603)=(S=5664352,E=5664383,C=18)
	CT(12604)=(S=5664384,E=5664415,C=15)
	CT(12605)=(S=5664416,E=5664479,C=18)
	CT(12606)=(S=5664480,E=5664511,C=71)
	CT(12607)=(S=5664512,E=5664543,C=18)
	CT(12608)=(S=5664544,E=5664575,C=42)
	CT(12609)=(S=5664576,E=5664607,C=31)
	CT(12610)=(S=5664608,E=5664639,C=58)
	CT(12611)=(S=5664640,E=5664655,C=34)
	CT(12612)=(S=5664656,E=5664663,C=35)
	CT(12613)=(S=5664664,E=5664671,C=28)
	CT(12614)=(S=5664672,E=5664703,C=14)
	CT(12615)=(S=5664704,E=5664711,C=23)
	CT(12616)=(S=5664712,E=5664719,C=58)
	CT(12617)=(S=5664728,E=5664735,C=14)
	CT(12618)=(S=5664736,E=5664743,C=31)
	CT(12619)=(S=5664744,E=5664751,C=28)
	CT(12620)=(S=5664752,E=5664759,C=58)
	CT(12621)=(S=5664760,E=5664767,C=28)
	CT(12622)=(S=5664768,E=5665279,C=14)
	CT(12623)=(S=5665280,E=5665791,C=47)
	CT(12624)=(S=5665792,E=5666815,C=34)
	CT(12625)=(S=5666816,E=5668863,C=38)
	CT(12626)=(S=5668864,E=5685247,C=14)
	CT(12627)=(S=5685248,E=5701631,C=12)
	CT(12628)=(S=5701632,E=5709823,C=15)
	CT(12629)=(S=5709824,E=5713919,C=63)
	CT(12630)=(S=5713920,E=5718015,C=24)
	CT(12631)=(S=5718016,E=5719039,C=49)
	CT(12632)=(S=5719040,E=5720063,C=17)
	CT(12633)=(S=5720064,E=5720575,C=24)
	CT(12634)=(S=5720576,E=5721087,C=14)
	CT(12635)=(S=5721088,E=5721103,C=18)
	CT(12636)=(S=5721104,E=5721119,C=29)
	CT(12637)=(S=5721120,E=5721151,C=20)
	CT(12638)=(S=5721152,E=5721343,C=31)
	CT(12639)=(S=5721344,E=5722111,C=28)
	CT(12640)=(S=5722112,E=5724159,C=14)
	CT(12641)=(S=5724160,E=5725183,C=12)
	CT(12642)=(S=5725184,E=5726207,C=47)
	CT(12643)=(S=5726208,E=5726335,C=58)
	CT(12644)=(S=5726336,E=5726463,C=19)
	CT(12645)=(S=5726464,E=5726591,C=50)
	CT(12646)=(S=5726592,E=5726719,C=51)
	CT(12647)=(S=5726720,E=5726847,C=55)
	CT(12648)=(S=5726848,E=5726975,C=12)
	CT(12649)=(S=5726976,E=5727039,C=58)
	CT(12650)=(S=5727040,E=5727071,C=60)
	CT(12651)=(S=5727072,E=5727103,C=58)
	CT(12652)=(S=5727104,E=5727231,C=19)
	CT(12653)=(S=5727232,E=5727359,C=12)
	CT(12654)=(S=5727360,E=5727487,C=47)
	CT(12655)=(S=5727488,E=5727503,C=36)
	CT(12656)=(S=5727504,E=5727535,C=58)
	CT(12657)=(S=5727536,E=5727551,C=22)
	CT(12658)=(S=5727552,E=5727567,C=58)
	CT(12659)=(S=5727584,E=5727599,C=14)
	CT(12660)=(S=5727600,E=5727615,C=50)
	CT(12661)=(S=5727616,E=5727743,C=23)
	CT(12662)=(S=5727744,E=5727871,C=14)
	CT(12663)=(S=5727872,E=5727999,C=34)
	CT(12664)=(S=5728000,E=5728127,C=21)
	CT(12665)=(S=5728128,E=5728255,C=18)
	CT(12666)=(S=5728256,E=5728511,C=24)
	CT(12667)=(S=5728512,E=5728767,C=58)
	CT(12668)=(S=5728768,E=5729023,C=28)
	CT(12669)=(S=5729024,E=5729279,C=26)
	CT(12670)=(S=5729280,E=5729535,C=47)
	CT(12671)=(S=5729536,E=5729791,C=23)
	CT(12672)=(S=5729792,E=5730047,C=60)
	CT(12673)=(S=5730048,E=5730303,C=25)
	CT(12674)=(S=5730304,E=5731327,C=14)
	CT(12675)=(S=5731328,E=5731391,C=24)
	CT(12676)=(S=5731392,E=5731455,C=51)
	CT(12677)=(S=5731456,E=5731519,C=46)
	CT(12678)=(S=5731520,E=5731583,C=58)
	CT(12679)=(S=5731584,E=5731647,C=18)
	CT(12680)=(S=5731648,E=5731711,C=14)
	CT(12681)=(S=5731712,E=5731775,C=18)
	CT(12682)=(S=5731776,E=5731839,C=14)
	CT(12683)=(S=5731840,E=5731903,C=27)
	CT(12684)=(S=5731904,E=5731967,C=28)
	CT(12685)=(S=5731968,E=5732031,C=51)
	CT(12686)=(S=5732032,E=5732095,C=18)
	CT(12687)=(S=5732096,E=5732159,C=58)
	CT(12688)=(S=5732160,E=5732223,C=51)
	CT(12689)=(S=5732224,E=5732255,C=52)
	CT(12690)=(S=5732256,E=5732287,C=55)
	CT(12691)=(S=5732288,E=5732319,C=28)
	CT(12692)=(S=5732320,E=5732351,C=18)
	CT(12693)=(S=5732352,E=5732863,C=51)
	CT(12694)=(S=5732864,E=5733375,C=28)
	CT(12695)=(S=5733376,E=5733887,C=14)
	CT(12696)=(S=5733888,E=5734143,C=51)
	CT(12697)=(S=5734144,E=5734399,C=14)
	CT(12698)=(S=5734400,E=5750783,C=28)
	CT(12699)=(S=5750784,E=5751039,C=63)
	CT(12700)=(S=5751040,E=5751295,C=28)
	CT(12701)=(S=5751296,E=5751551,C=14)
	CT(12702)=(S=5751552,E=5751807,C=36)
	CT(12703)=(S=5751808,E=5752063,C=21)
	CT(12704)=(S=5752064,E=5752319,C=71)
	CT(12705)=(S=5752320,E=5752575,C=63)
	CT(12706)=(S=5752576,E=5752831,C=58)
	CT(12707)=(S=5752832,E=5753343,C=16)
	CT(12708)=(S=5753344,E=5753855,C=22)
	CT(12709)=(S=5753856,E=5754879,C=58)
	CT(12710)=(S=5754880,E=5756927,C=36)
	CT(12711)=(S=5756928,E=5758975,C=25)
	CT(12712)=(S=5758976,E=5759103,C=14)
	CT(12713)=(S=5759104,E=5759359,C=18)
	CT(12714)=(S=5759360,E=5759487,C=28)
	CT(12715)=(S=5759488,E=5759615,C=60)
	CT(12716)=(S=5759616,E=5759743,C=18)
	CT(12717)=(S=5759744,E=5759871,C=19)
	CT(12718)=(S=5759872,E=5759999,C=51)
	CT(12719)=(S=5760000,E=5760127,C=18)
	CT(12720)=(S=5760128,E=5760255,C=32)
	CT(12721)=(S=5760256,E=5760383,C=50)
	CT(12722)=(S=5760384,E=5760511,C=18)
	CT(12723)=(S=5760512,E=5760639,C=28)
	CT(12724)=(S=5760640,E=5760767,C=23)
	CT(12725)=(S=5760768,E=5761023,C=12)
	CT(12726)=(S=5761024,E=5761279,C=63)
	CT(12727)=(S=5761280,E=5761535,C=36)
	CT(12728)=(S=5761536,E=5761791,C=28)
	CT(12729)=(S=5761792,E=5762047,C=25)
	CT(12730)=(S=5762048,E=5762055,C=36)
	CT(12731)=(S=5762056,E=5762079,C=18)
	CT(12732)=(S=5762080,E=5762087,C=32)
	CT(12733)=(S=5762088,E=5762095,C=18)
	CT(12734)=(S=5762096,E=5762103,C=104)
	CT(12735)=(S=5762104,E=5762119,C=14)
	CT(12736)=(S=5762120,E=5762127,C=12)
	CT(12737)=(S=5762128,E=5762135,C=18)
	CT(12738)=(S=5762136,E=5762143,C=21)
	CT(12739)=(S=5762144,E=5762151,C=36)
	CT(12740)=(S=5762152,E=5762159,C=28)
	CT(12741)=(S=5762160,E=5762167,C=42)
	CT(12742)=(S=5762168,E=5762175,C=101)
	CT(12743)=(S=5762176,E=5762183,C=14)
	CT(12744)=(S=5762184,E=5762191,C=104)
	CT(12745)=(S=5762192,E=5762199,C=14)
	CT(12746)=(S=5762200,E=5762207,C=47)
	CT(12747)=(S=5762208,E=5762215,C=15)
	CT(12748)=(S=5762216,E=5762223,C=47)
	CT(12749)=(S=5762224,E=5762231,C=49)
	CT(12750)=(S=5762232,E=5762239,C=18)
	CT(12751)=(S=5762240,E=5762247,C=33)
	CT(12752)=(S=5762248,E=5762255,C=34)
	CT(12753)=(S=5762256,E=5762263,C=14)
	CT(12754)=(S=5762264,E=5762271,C=25)
	CT(12755)=(S=5762272,E=5762279,C=47)
	CT(12756)=(S=5762280,E=5762287,C=61)
	CT(12757)=(S=5762288,E=5762295,C=25)
	CT(12758)=(S=5762296,E=5762303,C=14)
	CT(12759)=(S=5762304,E=5762311,C=36)
	CT(12760)=(S=5762312,E=5762319,C=49)
	CT(12761)=(S=5762320,E=5762327,C=14)
	CT(12762)=(S=5762328,E=5762335,C=36)
	CT(12763)=(S=5762336,E=5762343,C=48)
	CT(12764)=(S=5762344,E=5762351,C=18)
	CT(12765)=(S=5762352,E=5762383,C=14)
	CT(12766)=(S=5762384,E=5762391,C=25)
	CT(12767)=(S=5762392,E=5762399,C=28)
	CT(12768)=(S=5762400,E=5762407,C=36)
	CT(12769)=(S=5762408,E=5762415,C=38)
	CT(12770)=(S=5762416,E=5762423,C=18)
	CT(12771)=(S=5762424,E=5762439,C=28)
	CT(12772)=(S=5762440,E=5762447,C=18)
	CT(12773)=(S=5762448,E=5762455,C=28)
	CT(12774)=(S=5762456,E=5762463,C=19)
	CT(12775)=(S=5762464,E=5762471,C=14)
	CT(12776)=(S=5762472,E=5762479,C=34)
	CT(12777)=(S=5762480,E=5762487,C=28)
	CT(12778)=(S=5762488,E=5762495,C=12)
	CT(12779)=(S=5762496,E=5762503,C=104)
	CT(12780)=(S=5762504,E=5762511,C=46)
	CT(12781)=(S=5762512,E=5762519,C=19)
	CT(12782)=(S=5762520,E=5762527,C=28)
	CT(12783)=(S=5762528,E=5762535,C=16)
	CT(12784)=(S=5762536,E=5762543,C=98)
	CT(12785)=(S=5762544,E=5762551,C=28)
	CT(12786)=(S=5762552,E=5762559,C=12)
	CT(12787)=(S=5762560,E=5762591,C=15)
	CT(12788)=(S=5762592,E=5762631,C=27)
	CT(12789)=(S=5762632,E=5762639,C=14)
	CT(12790)=(S=5762640,E=5762647,C=63)
	CT(12791)=(S=5762648,E=5762655,C=25)
	CT(12792)=(S=5762656,E=5762663,C=18)
	CT(12793)=(S=5762664,E=5762671,C=187)
	CT(12794)=(S=5762672,E=5762687,C=28)
	CT(12795)=(S=5762688,E=5762695,C=61)
	CT(12796)=(S=5762696,E=5762703,C=28)
	CT(12797)=(S=5762704,E=5762711,C=12)
	CT(12798)=(S=5762712,E=5762719,C=31)
	CT(12799)=(S=5762720,E=5762727,C=49)
	CT(12800)=(S=5762728,E=5762735,C=36)
	CT(12801)=(S=5762736,E=5762743,C=12)
	CT(12802)=(S=5762744,E=5762759,C=28)
	CT(12803)=(S=5762760,E=5762767,C=14)
	CT(12804)=(S=5762768,E=5762775,C=46)
	CT(12805)=(S=5762776,E=5762783,C=14)
	CT(12806)=(S=5762784,E=5762791,C=49)
	CT(12807)=(S=5762792,E=5762799,C=18)
	CT(12808)=(S=5762800,E=5762807,C=25)
	CT(12809)=(S=5762808,E=5762815,C=24)
	CT(12810)=(S=5762816,E=5762823,C=18)
	CT(12811)=(S=5762824,E=5762831,C=36)
	CT(12812)=(S=5762832,E=5762839,C=14)
	CT(12813)=(S=5762840,E=5762855,C=18)
	CT(12814)=(S=5762856,E=5762863,C=58)
	CT(12815)=(S=5762872,E=5762879,C=71)
	CT(12816)=(S=5762880,E=5762887,C=24)
	CT(12817)=(S=5762888,E=5762895,C=58)
	CT(12818)=(S=5762896,E=5762903,C=44)
	CT(12819)=(S=5762904,E=5762911,C=58)
	CT(12820)=(S=5762912,E=5762919,C=36)
	CT(12821)=(S=5762920,E=5762927,C=18)
	CT(12822)=(S=5762928,E=5762935,C=44)
	CT(12823)=(S=5762936,E=5762943,C=47)
	CT(12824)=(S=5762944,E=5762951,C=28)
	CT(12825)=(S=5762952,E=5762959,C=29)
	CT(12826)=(S=5762960,E=5762967,C=18)
	CT(12827)=(S=5762968,E=5762975,C=51)
	CT(12828)=(S=5762976,E=5762983,C=38)
	CT(12829)=(S=5762984,E=5762999,C=58)
	CT(12830)=(S=5763000,E=5763007,C=28)
	CT(12831)=(S=5763008,E=5763015,C=58)
	CT(12832)=(S=5763016,E=5763023,C=56)
	CT(12833)=(S=5763024,E=5763031,C=34)
	CT(12834)=(S=5763032,E=5763039,C=58)
	CT(12835)=(S=5763040,E=5763047,C=38)
	CT(12836)=(S=5763048,E=5763055,C=22)
	CT(12837)=(S=5763056,E=5763063,C=58)
	CT(12838)=(S=5763064,E=5763064,C=38)
	CT(12839)=(S=5763065,E=5763069,C=13)
	CT(12840)=(S=5763070,E=5763071,C=38)
	CT(12841)=(S=5763072,E=5763135,C=18)
	CT(12842)=(S=5763136,E=5763199,C=12)
	CT(12843)=(S=5763200,E=5763263,C=18)
	CT(12844)=(S=5763264,E=5763327,C=70)
	CT(12845)=(S=5763328,E=5763391,C=15)
	CT(12846)=(S=5763392,E=5763455,C=19)
	CT(12847)=(S=5763456,E=5763519,C=66)
	CT(12848)=(S=5763520,E=5763583,C=18)
	CT(12849)=(S=5763584,E=5763647,C=50)
	CT(12850)=(S=5763648,E=5763711,C=18)
	CT(12851)=(S=5763712,E=5763839,C=14)
	CT(12852)=(S=5763840,E=5763847,C=38)
	CT(12853)=(S=5763848,E=5763903,C=20)
	CT(12854)=(S=5763904,E=5763967,C=22)
	CT(12855)=(S=5763968,E=5764031,C=42)
	CT(12856)=(S=5764032,E=5764095,C=14)
	CT(12857)=(S=5764096,E=5764159,C=18)
	CT(12858)=(S=5764160,E=5764223,C=99)
	CT(12859)=(S=5764224,E=5764287,C=31)
	CT(12860)=(S=5764288,E=5764351,C=71)
	CT(12861)=(S=5764352,E=5764415,C=28)
	CT(12862)=(S=5764416,E=5764479,C=34)
	CT(12863)=(S=5764480,E=5764543,C=18)
	CT(12864)=(S=5764544,E=5764607,C=14)
	CT(12865)=(S=5764608,E=5764671,C=51)
	CT(12866)=(S=5764672,E=5764735,C=14)
	CT(12867)=(S=5764736,E=5764799,C=60)
	CT(12868)=(S=5764800,E=5764863,C=58)
	CT(12869)=(S=5764864,E=5764927,C=20)
	CT(12870)=(S=5764928,E=5764991,C=44)
	CT(12871)=(S=5764992,E=5765023,C=14)
	CT(12872)=(S=5765024,E=5765055,C=26)
	CT(12873)=(S=5765056,E=5765087,C=42)
	CT(12874)=(S=5765088,E=5765095,C=12)
	CT(12875)=(S=5765096,E=5765103,C=15)
	CT(12876)=(S=5765104,E=5765111,C=14)
	CT(12877)=(S=5765112,E=5765119,C=51)
	CT(12878)=(S=5765120,E=5765151,C=27)
	CT(12879)=(S=5765152,E=5765183,C=15)
	CT(12880)=(S=5765184,E=5765215,C=58)
	CT(12881)=(S=5765216,E=5765247,C=14)
	CT(12882)=(S=5765248,E=5765279,C=26)
	CT(12883)=(S=5765280,E=5765311,C=53)
	CT(12884)=(S=5765312,E=5765343,C=14)
	CT(12885)=(S=5765344,E=5765439,C=18)
	CT(12886)=(S=5765440,E=5765471,C=14)
	CT(12887)=(S=5765472,E=5765503,C=36)
	CT(12888)=(S=5765504,E=5765535,C=33)
	CT(12889)=(S=5765536,E=5765567,C=19)
	CT(12890)=(S=5765568,E=5765663,C=18)
	CT(12891)=(S=5765664,E=5765695,C=46)
	CT(12892)=(S=5765696,E=5765727,C=15)
	CT(12893)=(S=5765728,E=5765759,C=59)
	CT(12894)=(S=5765760,E=5765791,C=36)
	CT(12895)=(S=5765792,E=5765823,C=58)
	CT(12896)=(S=5765824,E=5765887,C=18)
	CT(12897)=(S=5765888,E=5765919,C=37)
	CT(12898)=(S=5765920,E=5765951,C=36)
	CT(12899)=(S=5765952,E=5766079,C=18)
	CT(12900)=(S=5766080,E=5766111,C=19)
	CT(12901)=(S=5766112,E=5766143,C=58)
	CT(12902)=(S=5766144,E=5766175,C=12)
	CT(12903)=(S=5766176,E=5766207,C=14)
	CT(12904)=(S=5766208,E=5766239,C=27)
	CT(12905)=(S=5766240,E=5766271,C=104)
	CT(12906)=(S=5766272,E=5766303,C=57)
	CT(12907)=(S=5766304,E=5766335,C=51)
	CT(12908)=(S=5766336,E=5766367,C=63)
	CT(12909)=(S=5766368,E=5766399,C=68)
	CT(12910)=(S=5766400,E=5766431,C=18)
	CT(12911)=(S=5766432,E=5766463,C=40)
	CT(12912)=(S=5766464,E=5766495,C=19)
	CT(12913)=(S=5766496,E=5766527,C=15)
	CT(12914)=(S=5766528,E=5766559,C=36)
	CT(12915)=(S=5766560,E=5766591,C=28)
	CT(12916)=(S=5766592,E=5766623,C=18)
	CT(12917)=(S=5766624,E=5766631,C=25)
	CT(12918)=(S=5766632,E=5766639,C=12)
	CT(12919)=(S=5766640,E=5766655,C=28)
	CT(12920)=(S=5766656,E=5766687,C=14)
	CT(12921)=(S=5766688,E=5766719,C=27)
	CT(12922)=(S=5766720,E=5766751,C=14)
	CT(12923)=(S=5766752,E=5766783,C=70)
	CT(12924)=(S=5766784,E=5766815,C=18)
	CT(12925)=(S=5766816,E=5766847,C=51)
	CT(12926)=(S=5766848,E=5766879,C=47)
	CT(12927)=(S=5766880,E=5766911,C=187)
	CT(12928)=(S=5766912,E=5766943,C=18)
	CT(12929)=(S=5766944,E=5766975,C=36)
	CT(12930)=(S=5766976,E=5767007,C=53)
	CT(12931)=(S=5767008,E=5767039,C=109)
	CT(12932)=(S=5767040,E=5767071,C=12)
	CT(12933)=(S=5767072,E=5767103,C=19)
	CT(12934)=(S=5767104,E=5767135,C=20)
	CT(12935)=(S=5767136,E=5767167,C=18)
	CT(12936)=(S=5767168,E=5775359,C=25)
	CT(12937)=(S=5775360,E=5783551,C=15)
	CT(12938)=(S=5783552,E=5787647,C=28)
	CT(12939)=(S=5787648,E=5787679,C=19)
	CT(12940)=(S=5787680,E=5787711,C=18)
	CT(12941)=(S=5787712,E=5787743,C=15)
	CT(12942)=(S=5787744,E=5787807,C=51)
	CT(12943)=(S=5787808,E=5787831,C=19)
	CT(12944)=(S=5787832,E=5787839,C=14)
	CT(12945)=(S=5787840,E=5787871,C=28)
	CT(12946)=(S=5787872,E=5787903,C=71)
	CT(12947)=(S=5787904,E=5787935,C=23)
	CT(12948)=(S=5787936,E=5787967,C=18)
	CT(12949)=(S=5787968,E=5787999,C=33)
	CT(12950)=(S=5788000,E=5788031,C=63)
	CT(12951)=(S=5788032,E=5788063,C=14)
	CT(12952)=(S=5788064,E=5788095,C=15)
	CT(12953)=(S=5788096,E=5788127,C=18)
	CT(12954)=(S=5788128,E=5788159,C=31)
	CT(12955)=(S=5788160,E=5788191,C=14)
	CT(12956)=(S=5788192,E=5788223,C=49)
	CT(12957)=(S=5788224,E=5788255,C=18)
	CT(12958)=(S=5788256,E=5788287,C=56)
	CT(12959)=(S=5788288,E=5788319,C=14)
	CT(12960)=(S=5788320,E=5788351,C=18)
	CT(12961)=(S=5788352,E=5788383,C=25)
	CT(12962)=(S=5788384,E=5788415,C=28)
	CT(12963)=(S=5788416,E=5788447,C=168)
	CT(12964)=(S=5788448,E=5788479,C=19)
	CT(12965)=(S=5788480,E=5788511,C=24)
	CT(12966)=(S=5788512,E=5788543,C=14)
	CT(12967)=(S=5788544,E=5788575,C=19)
	CT(12968)=(S=5788576,E=5788607,C=33)
	CT(12969)=(S=5788608,E=5788639,C=18)
	CT(12970)=(S=5788640,E=5788671,C=33)
	CT(12971)=(S=5788672,E=5788703,C=34)
	CT(12972)=(S=5788704,E=5788735,C=27)
	CT(12973)=(S=5788736,E=5788767,C=25)
	CT(12974)=(S=5788768,E=5788799,C=23)
	CT(12975)=(S=5788800,E=5788831,C=28)
	CT(12976)=(S=5788832,E=5788863,C=27)
	CT(12977)=(S=5788864,E=5788895,C=18)
	CT(12978)=(S=5788896,E=5788927,C=19)
	CT(12979)=(S=5788928,E=5788959,C=34)
	CT(12980)=(S=5788960,E=5788991,C=171)
	CT(12981)=(S=5788992,E=5789023,C=36)
	CT(12982)=(S=5789024,E=5789055,C=54)
	CT(12983)=(S=5789056,E=5789087,C=47)
	CT(12984)=(S=5789088,E=5789151,C=18)
	CT(12985)=(S=5789152,E=5789183,C=23)
	CT(12986)=(S=5789184,E=5789215,C=30)
	CT(12987)=(S=5789216,E=5789247,C=19)
	CT(12988)=(S=5789248,E=5789279,C=18)
	CT(12989)=(S=5789280,E=5789311,C=33)
	CT(12990)=(S=5789312,E=5789343,C=47)
	CT(12991)=(S=5789344,E=5789375,C=15)
	CT(12992)=(S=5789376,E=5789407,C=18)
	CT(12993)=(S=5789408,E=5789439,C=12)
	CT(12994)=(S=5789440,E=5789471,C=51)
	CT(12995)=(S=5789472,E=5789503,C=27)
	CT(12996)=(S=5789504,E=5789535,C=18)
	CT(12997)=(S=5789536,E=5789567,C=15)
	CT(12998)=(S=5789568,E=5789599,C=25)
	CT(12999)=(S=5789600,E=5789631,C=63)
	CT(13000)=(S=5789632,E=5789663,C=25)
	CT(13001)=(S=5789664,E=5789695,C=50)
	CT(13002)=(S=5789696,E=5791743,C=27)
	CT(13003)=(S=5791744,E=5792767,C=14)
	CT(13004)=(S=5792768,E=5793791,C=33)
	CT(13005)=(S=5793792,E=5795839,C=14)
	CT(13006)=(S=5795840,E=5796863,C=47)
	CT(13007)=(S=5796864,E=5797375,C=42)
	CT(13008)=(S=5797376,E=5797887,C=44)
	CT(13009)=(S=5797888,E=5799935,C=12)
	CT(13010)=(S=5799936,E=5800191,C=28)
	CT(13011)=(S=5800192,E=5800447,C=19)
	CT(13012)=(S=5800448,E=5800703,C=28)
	CT(13013)=(S=5800704,E=5800959,C=19)
	CT(13014)=(S=5800960,E=5801215,C=50)
	CT(13015)=(S=5801216,E=5801359,C=28)
	CT(13016)=(S=5801360,E=5801367,C=42)
	CT(13017)=(S=5801368,E=5801727,C=28)
	CT(13018)=(S=5801728,E=5801743,C=18)
	CT(13019)=(S=5801744,E=5801759,C=31)
	CT(13020)=(S=5801760,E=5801775,C=26)
	CT(13021)=(S=5801776,E=5801791,C=18)
	CT(13022)=(S=5801808,E=5801823,C=31)
	CT(13023)=(S=5801824,E=5801839,C=38)
	CT(13024)=(S=5801840,E=5801855,C=31)
	CT(13025)=(S=5801856,E=5801887,C=60)
	CT(13026)=(S=5801888,E=5801919,C=58)
	CT(13027)=(S=5801920,E=5801983,C=31)
	CT(13028)=(S=5801984,E=5804031,C=12)
	CT(13029)=(S=5804032,E=5804543,C=14)
	CT(13030)=(S=5804544,E=5804799,C=33)
	CT(13031)=(S=5804800,E=5804927,C=15)
	CT(13032)=(S=5804928,E=5805055,C=18)
	CT(13033)=(S=5805056,E=5805183,C=25)
	CT(13034)=(S=5805184,E=5805311,C=47)
	CT(13035)=(S=5805312,E=5805439,C=48)
	CT(13036)=(S=5805440,E=5805567,C=15)
	CT(13037)=(S=5805568,E=5805695,C=28)
	CT(13038)=(S=5805696,E=5805831,C=14)
	CT(13039)=(S=5805832,E=5805839,C=18)
	CT(13040)=(S=5805840,E=5805847,C=25)
	CT(13041)=(S=5805848,E=5805855,C=63)
	CT(13042)=(S=5805856,E=5805863,C=36)
	CT(13043)=(S=5805864,E=5805871,C=14)
	CT(13044)=(S=5805872,E=5805879,C=48)
	CT(13045)=(S=5805880,E=5805887,C=50)
	CT(13046)=(S=5805888,E=5805895,C=28)
	CT(13047)=(S=5805896,E=5805903,C=42)
	CT(13048)=(S=5805904,E=5805911,C=63)
	CT(13049)=(S=5805912,E=5805919,C=18)
	CT(13050)=(S=5805920,E=5805927,C=50)
	CT(13051)=(S=5805928,E=5805943,C=18)
	CT(13052)=(S=5805944,E=5805951,C=36)
	CT(13053)=(S=5805952,E=5805959,C=14)
	CT(13054)=(S=5805960,E=5805967,C=58)
	CT(13055)=(S=5805968,E=5805975,C=34)
	CT(13056)=(S=5805976,E=5805983,C=14)
	CT(13057)=(S=5805984,E=5805991,C=27)
	CT(13058)=(S=5805992,E=5805999,C=19)
	CT(13059)=(S=5806000,E=5806007,C=20)
	CT(13060)=(S=5806008,E=5806015,C=18)
	CT(13061)=(S=5806016,E=5806023,C=41)
	CT(13062)=(S=5806024,E=5806031,C=18)
	CT(13063)=(S=5806032,E=5806039,C=25)
	CT(13064)=(S=5806040,E=5806047,C=14)
	CT(13065)=(S=5806048,E=5806055,C=29)
	CT(13066)=(S=5806056,E=5806063,C=12)
	CT(13067)=(S=5806064,E=5806071,C=49)
	CT(13068)=(S=5806072,E=5806079,C=18)
	CT(13069)=(S=5806080,E=5806591,C=28)
	CT(13070)=(S=5806592,E=5807103,C=31)
	CT(13071)=(S=5807104,E=5807359,C=58)
	CT(13072)=(S=5807360,E=5807615,C=21)
	CT(13073)=(S=5807616,E=5807871,C=38)
	CT(13074)=(S=5807872,E=5808127,C=36)
	CT(13075)=(S=5808128,E=5816319,C=12)
	CT(13076)=(S=5816320,E=5817343,C=47)
	CT(13077)=(S=5817344,E=5817599,C=55)
	CT(13078)=(S=5817600,E=5817727,C=22)
	CT(13079)=(S=5817728,E=5817855,C=49)
	CT(13080)=(S=5817856,E=5818111,C=28)
	CT(13081)=(S=5818112,E=5818367,C=58)
	CT(13082)=(S=5818368,E=5818495,C=52)
	CT(13083)=(S=5818496,E=5818623,C=18)
	CT(13084)=(S=5818624,E=5818751,C=104)
	CT(13085)=(S=5818752,E=5818879,C=18)
	CT(13086)=(S=5818880,E=5819007,C=15)
	CT(13087)=(S=5819008,E=5819039,C=14)
	CT(13088)=(S=5819040,E=5819047,C=36)
	CT(13089)=(S=5819048,E=5819115,C=14)
	CT(13090)=(S=5819116,E=5819119,C=12)
	CT(13091)=(S=5819120,E=5819135,C=14)
	CT(13092)=(S=5819136,E=5819263,C=72)
	CT(13093)=(S=5819264,E=5819391,C=51)
	CT(13094)=(S=5819392,E=5819519,C=18)
	CT(13095)=(S=5819520,E=5819647,C=20)
	CT(13096)=(S=5819648,E=5819775,C=28)
	CT(13097)=(S=5819776,E=5820031,C=18)
	CT(13098)=(S=5820032,E=5820159,C=19)
	CT(13099)=(S=5820160,E=5820287,C=57)
	CT(13100)=(S=5820288,E=5820415,C=70)
	CT(13101)=(S=5820416,E=5820479,C=36)
	CT(13102)=(S=5820480,E=5820543,C=33)
	CT(13103)=(S=5820544,E=5820607,C=28)
	CT(13104)=(S=5820608,E=5820671,C=14)
	CT(13105)=(S=5820672,E=5820735,C=28)
	CT(13106)=(S=5820736,E=5820799,C=187)
	CT(13107)=(S=5820800,E=5820863,C=23)
	CT(13108)=(S=5820864,E=5820927,C=50)
	CT(13109)=(S=5820928,E=5820991,C=18)
	CT(13110)=(S=5820992,E=5821055,C=21)
	CT(13111)=(S=5821056,E=5821119,C=14)
	CT(13112)=(S=5821120,E=5821183,C=40)
	CT(13113)=(S=5821184,E=5821311,C=14)
	CT(13114)=(S=5821312,E=5821375,C=36)
	CT(13115)=(S=5821376,E=5821439,C=14)
	CT(13116)=(S=5821440,E=5821503,C=71)
	CT(13117)=(S=5821504,E=5821567,C=24)
	CT(13118)=(S=5821568,E=5821591,C=14)
	CT(13119)=(S=5821592,E=5821595,C=12)
	CT(13120)=(S=5821596,E=5821631,C=14)
	CT(13121)=(S=5821632,E=5821695,C=18)
	CT(13122)=(S=5821696,E=5821823,C=23)
	CT(13123)=(S=5821824,E=5821887,C=15)
	CT(13124)=(S=5821888,E=5821951,C=51)
	CT(13125)=(S=5821952,E=5822015,C=28)
	CT(13126)=(S=5822016,E=5822079,C=31)
	CT(13127)=(S=5822080,E=5822143,C=21)
	CT(13128)=(S=5822144,E=5822271,C=14)
	CT(13129)=(S=5822272,E=5822335,C=28)
	CT(13130)=(S=5822336,E=5822399,C=18)
	CT(13131)=(S=5822400,E=5822463,C=28)
	CT(13132)=(S=5822464,E=5822719,C=44)
	CT(13133)=(S=5822720,E=5822975,C=28)
	CT(13134)=(S=5822976,E=5823231,C=22)
	CT(13135)=(S=5823232,E=5823487,C=12)
	CT(13136)=(S=5823488,E=5823743,C=58)
	CT(13137)=(S=5823744,E=5823999,C=13)
	CT(13138)=(S=5824000,E=5824511,C=44)
	CT(13139)=(S=5824512,E=5832703,C=37)
	CT(13140)=(S=5832704,E=5833215,C=28)
	CT(13141)=(S=5833216,E=5833727,C=12)
	CT(13142)=(S=5833728,E=5834239,C=23)
	CT(13143)=(S=5834240,E=5834751,C=25)
	CT(13144)=(S=5834752,E=5835775,C=27)
	CT(13145)=(S=5835776,E=5836799,C=28)
	CT(13146)=(S=5836800,E=5836831,C=12)
	CT(13147)=(S=5836832,E=5836863,C=18)
	CT(13148)=(S=5836864,E=5836895,C=63)
	CT(13149)=(S=5836896,E=5836927,C=18)
	CT(13150)=(S=5836928,E=5836959,C=28)
	CT(13151)=(S=5836960,E=5836991,C=14)
	CT(13152)=(S=5836992,E=5837023,C=42)
	CT(13153)=(S=5837024,E=5837055,C=14)
	CT(13154)=(S=5837056,E=5837087,C=57)
	CT(13155)=(S=5837088,E=5837119,C=18)
	CT(13156)=(S=5837120,E=5837151,C=47)
	CT(13157)=(S=5837152,E=5837183,C=14)
	CT(13158)=(S=5837184,E=5837215,C=48)
	CT(13159)=(S=5837216,E=5837247,C=15)
	CT(13160)=(S=5837248,E=5837279,C=25)
	CT(13161)=(S=5837280,E=5837311,C=58)
	CT(13162)=(S=5837312,E=5837343,C=38)
	CT(13163)=(S=5837344,E=5837375,C=57)
	CT(13164)=(S=5837376,E=5837407,C=63)
	CT(13165)=(S=5837408,E=5837439,C=19)
	CT(13166)=(S=5837440,E=5837471,C=18)
	CT(13167)=(S=5837472,E=5837503,C=36)
	CT(13168)=(S=5837504,E=5837535,C=60)
	CT(13169)=(S=5837536,E=5837567,C=47)
	CT(13170)=(S=5837568,E=5837599,C=37)
	CT(13171)=(S=5837632,E=5837663,C=63)
	CT(13172)=(S=5837664,E=5837695,C=31)
	CT(13173)=(S=5837696,E=5837727,C=24)
	CT(13174)=(S=5837728,E=5837791,C=18)
	CT(13175)=(S=5837792,E=5837823,C=28)
	CT(13176)=(S=5837824,E=5837887,C=18)
	CT(13177)=(S=5837888,E=5837919,C=36)
	CT(13178)=(S=5837920,E=5837983,C=18)
	CT(13179)=(S=5837984,E=5838015,C=36)
	CT(13180)=(S=5838016,E=5838047,C=14)
	CT(13181)=(S=5838048,E=5838079,C=27)
	CT(13182)=(S=5838080,E=5838111,C=14)
	CT(13183)=(S=5838112,E=5838143,C=28)
	CT(13184)=(S=5838144,E=5838175,C=31)
	CT(13185)=(S=5838176,E=5838207,C=38)
	CT(13186)=(S=5838208,E=5838239,C=18)
	CT(13187)=(S=5838240,E=5838271,C=36)
	CT(13188)=(S=5838272,E=5838303,C=15)
	CT(13189)=(S=5838304,E=5838335,C=14)
	CT(13190)=(S=5838336,E=5838343,C=18)
	CT(13191)=(S=5838344,E=5838351,C=25)
	CT(13192)=(S=5838352,E=5838367,C=18)
	CT(13193)=(S=5838368,E=5838375,C=58)
	CT(13194)=(S=5838376,E=5838383,C=31)
	CT(13195)=(S=5838384,E=5838384,C=33)
	CT(13196)=(S=5838385,E=5838385,C=38)
	CT(13197)=(S=5838386,E=5838386,C=18)
	CT(13198)=(S=5838387,E=5838387,C=63)
	CT(13199)=(S=5838388,E=5838399,C=18)
	CT(13200)=(S=5838400,E=5838431,C=33)
	CT(13201)=(S=5838432,E=5838463,C=28)
	CT(13202)=(S=5838464,E=5838527,C=18)
	CT(13203)=(S=5838528,E=5838535,C=58)
	CT(13204)=(S=5838536,E=5838543,C=31)
	CT(13205)=(S=5838544,E=5838551,C=58)
	CT(13206)=(S=5838552,E=5838575,C=18)
	CT(13207)=(S=5838576,E=5838591,C=31)
	CT(13208)=(S=5838592,E=5838655,C=18)
	CT(13209)=(S=5838656,E=5838687,C=46)
	CT(13210)=(S=5838688,E=5838815,C=18)
	CT(13211)=(S=5838816,E=5838847,C=24)
	CT(13212)=(S=5838848,E=5839103,C=33)
	CT(13213)=(S=5839104,E=5839231,C=51)
	CT(13214)=(S=5839232,E=5839359,C=58)
	CT(13215)=(S=5839360,E=5839487,C=42)
	CT(13216)=(S=5839488,E=5839615,C=21)
	CT(13217)=(S=5839616,E=5839743,C=47)
	CT(13218)=(S=5839744,E=5839871,C=28)
	CT(13219)=(S=5839872,E=5839999,C=53)
	CT(13220)=(S=5840000,E=5840007,C=20)
	CT(13221)=(S=5840008,E=5840015,C=28)
	CT(13222)=(S=5840016,E=5840023,C=12)
	CT(13223)=(S=5840024,E=5840031,C=34)
	CT(13224)=(S=5840032,E=5840039,C=18)
	CT(13225)=(S=5840040,E=5840047,C=14)
	CT(13226)=(S=5840048,E=5840055,C=63)
	CT(13227)=(S=5840056,E=5840063,C=14)
	CT(13228)=(S=5840064,E=5840071,C=18)
	CT(13229)=(S=5840072,E=5840079,C=31)
	CT(13230)=(S=5840080,E=5840087,C=14)
	CT(13231)=(S=5840088,E=5840095,C=61)
	CT(13232)=(S=5840096,E=5840103,C=58)
	CT(13233)=(S=5840104,E=5840111,C=14)
	CT(13234)=(S=5840112,E=5840119,C=28)
	CT(13235)=(S=5840120,E=5840127,C=18)
	CT(13236)=(S=5840128,E=5840255,C=33)
	CT(13237)=(S=5840256,E=5840383,C=25)
	CT(13238)=(S=5840384,E=5840511,C=13)
	CT(13239)=(S=5840512,E=5840639,C=36)
	CT(13240)=(S=5840640,E=5840647,C=28)
	CT(13241)=(S=5840648,E=5840655,C=33)
	CT(13242)=(S=5840656,E=5840679,C=18)
	CT(13243)=(S=5840680,E=5840687,C=33)
	CT(13244)=(S=5840688,E=5840695,C=15)
	CT(13245)=(S=5840696,E=5840703,C=36)
	CT(13246)=(S=5840704,E=5840711,C=58)
	CT(13247)=(S=5840712,E=5840719,C=15)
	CT(13248)=(S=5840720,E=5840735,C=18)
	CT(13249)=(S=5840736,E=5840743,C=36)
	CT(13250)=(S=5840744,E=5840759,C=18)
	CT(13251)=(S=5840768,E=5840775,C=42)
	CT(13252)=(S=5840776,E=5840783,C=28)
	CT(13253)=(S=5840784,E=5840791,C=12)
	CT(13254)=(S=5840792,E=5840799,C=54)
	CT(13255)=(S=5840800,E=5840807,C=33)
	CT(13256)=(S=5840808,E=5840815,C=63)
	CT(13257)=(S=5840816,E=5840823,C=22)
	CT(13258)=(S=5840824,E=5840831,C=18)
	CT(13259)=(S=5840832,E=5840839,C=104)
	CT(13260)=(S=5840840,E=5840847,C=15)
	CT(13261)=(S=5840848,E=5840855,C=14)
	CT(13262)=(S=5840856,E=5840863,C=49)
	CT(13263)=(S=5840864,E=5840871,C=21)
	CT(13264)=(S=5840872,E=5840879,C=14)
	CT(13265)=(S=5840880,E=5840887,C=18)
	CT(13266)=(S=5840888,E=5840895,C=19)
	CT(13267)=(S=5840896,E=5840927,C=26)
	CT(13268)=(S=5840928,E=5840935,C=19)
	CT(13269)=(S=5840936,E=5840936,C=170)
	CT(13270)=(S=5840937,E=5840938,C=38)
	CT(13271)=(S=5840939,E=5840939,C=14)
	CT(13272)=(S=5840940,E=5840959,C=38)
	CT(13273)=(S=5840960,E=5840967,C=95)
	CT(13274)=(S=5840968,E=5840975,C=19)
	CT(13275)=(S=5840976,E=5840991,C=38)
	CT(13276)=(S=5840992,E=5841007,C=26)
	CT(13277)=(S=5841008,E=5841015,C=38)
	CT(13278)=(S=5841016,E=5841019,C=95)
	CT(13279)=(S=5841020,E=5841055,C=38)
	CT(13280)=(S=5841056,E=5841063,C=25)
	CT(13281)=(S=5841064,E=5841065,C=15)
	CT(13282)=(S=5841066,E=5841066,C=14)
	CT(13283)=(S=5841067,E=5841071,C=38)
	CT(13284)=(S=5841072,E=5841079,C=95)
	CT(13285)=(S=5841080,E=5841081,C=38)
	CT(13286)=(S=5841082,E=5841083,C=15)
	CT(13287)=(S=5841084,E=5841087,C=38)
	CT(13288)=(S=5841088,E=5841091,C=95)
	CT(13289)=(S=5841092,E=5841093,C=26)
	CT(13290)=(S=5841094,E=5841103,C=38)
	CT(13291)=(S=5841104,E=5841111,C=95)
	CT(13292)=(S=5841112,E=5841143,C=38)
	CT(13293)=(S=5841144,E=5841147,C=95)
	CT(13294)=(S=5841148,E=5841157,C=38)
	CT(13295)=(S=5841158,E=5841159,C=13)
	CT(13296)=(S=5841160,E=5841167,C=38)
	CT(13297)=(S=5841168,E=5841168,C=28)
	CT(13298)=(S=5841169,E=5841169,C=38)
	CT(13299)=(S=5841170,E=5841171,C=26)
	CT(13300)=(S=5841172,E=5841191,C=38)
	CT(13301)=(S=5841192,E=5841195,C=95)
	CT(13302)=(S=5841196,E=5841197,C=38)
	CT(13303)=(S=5841198,E=5841199,C=14)
	CT(13304)=(S=5841200,E=5841215,C=38)
	CT(13305)=(S=5841216,E=5841216,C=170)
	CT(13306)=(S=5841217,E=5841217,C=38)
	CT(13307)=(S=5841218,E=5841219,C=35)
	CT(13308)=(S=5841220,E=5841223,C=95)
	CT(13309)=(S=5841224,E=5841251,C=38)
	CT(13310)=(S=5841252,E=5841255,C=26)
	CT(13311)=(S=5841256,E=5841259,C=38)
	CT(13312)=(S=5841260,E=5841263,C=25)
	CT(13313)=(S=5841264,E=5841267,C=38)
	CT(13314)=(S=5841268,E=5841268,C=14)
	CT(13315)=(S=5841269,E=5841279,C=38)
	CT(13316)=(S=5841280,E=5841281,C=26)
	CT(13317)=(S=5841282,E=5841285,C=38)
	CT(13318)=(S=5841286,E=5841286,C=14)
	CT(13319)=(S=5841287,E=5841287,C=38)
	CT(13320)=(S=5841288,E=5841295,C=19)
	CT(13321)=(S=5841296,E=5841313,C=38)
	CT(13322)=(S=5841314,E=5841314,C=14)
	CT(13323)=(S=5841315,E=5841315,C=38)
	CT(13324)=(S=5841316,E=5841317,C=14)
	CT(13325)=(S=5841318,E=5841335,C=38)
	CT(13326)=(S=5841336,E=5841343,C=25)
	CT(13327)=(S=5841344,E=5841349,C=38)
	CT(13328)=(S=5841350,E=5841353,C=14)
	CT(13329)=(S=5841354,E=5841355,C=38)
	CT(13330)=(S=5841356,E=5841357,C=26)
	CT(13331)=(S=5841358,E=5841358,C=14)
	CT(13332)=(S=5841359,E=5841359,C=38)
	CT(13333)=(S=5841360,E=5841375,C=20)
	CT(13334)=(S=5841376,E=5841385,C=38)
	CT(13335)=(S=5841386,E=5841387,C=26)
	CT(13336)=(S=5841388,E=5841391,C=38)
	CT(13337)=(S=5841392,E=5841393,C=26)
	CT(13338)=(S=5841394,E=5841401,C=38)
	CT(13339)=(S=5841402,E=5841403,C=14)
	CT(13340)=(S=5841404,E=5841405,C=38)
	CT(13341)=(S=5841406,E=5841407,C=14)
	CT(13342)=(S=5841408,E=5841427,C=38)
	CT(13343)=(S=5841428,E=5841429,C=26)
	CT(13344)=(S=5841430,E=5841435,C=38)
	CT(13345)=(S=5841436,E=5841439,C=25)
	CT(13346)=(S=5841440,E=5841471,C=26)
	CT(13347)=(S=5841472,E=5841495,C=38)
	CT(13348)=(S=5841496,E=5841497,C=26)
	CT(13349)=(S=5841498,E=5841501,C=38)
	CT(13350)=(S=5841502,E=5841503,C=26)
	CT(13351)=(S=5841504,E=5841509,C=38)
	CT(13352)=(S=5841510,E=5841510,C=14)
	CT(13353)=(S=5841511,E=5841511,C=38)
	CT(13354)=(S=5841512,E=5841513,C=14)
	CT(13355)=(S=5841514,E=5841515,C=38)
	CT(13356)=(S=5841516,E=5841516,C=14)
	CT(13357)=(S=5841517,E=5841518,C=38)
	CT(13358)=(S=5841519,E=5841519,C=170)
	CT(13359)=(S=5841520,E=5841527,C=38)
	CT(13360)=(S=5841528,E=5841531,C=25)
	CT(13361)=(S=5841532,E=5841535,C=38)
	CT(13362)=(S=5841536,E=5841567,C=26)
	CT(13363)=(S=5841568,E=5841571,C=38)
	CT(13364)=(S=5841572,E=5841575,C=95)
	CT(13365)=(S=5841576,E=5841577,C=26)
	CT(13366)=(S=5841578,E=5841579,C=38)
	CT(13367)=(S=5841580,E=5841580,C=44)
	CT(13368)=(S=5841581,E=5841583,C=38)
	CT(13369)=(S=5841584,E=5841585,C=26)
	CT(13370)=(S=5841586,E=5841591,C=38)
	CT(13371)=(S=5841592,E=5841599,C=19)
	CT(13372)=(S=5841600,E=5841607,C=38)
	CT(13373)=(S=5841608,E=5841609,C=26)
	CT(13374)=(S=5841610,E=5841647,C=38)
	CT(13375)=(S=5841648,E=5841655,C=19)
	CT(13376)=(S=5841656,E=5841663,C=26)
	CT(13377)=(S=5841664,E=5841665,C=38)
	CT(13378)=(S=5841666,E=5841667,C=28)
	CT(13379)=(S=5841668,E=5841691,C=38)
	CT(13380)=(S=5841692,E=5841695,C=95)
	CT(13381)=(S=5841696,E=5841717,C=38)
	CT(13382)=(S=5841718,E=5841718,C=14)
	CT(13383)=(S=5841719,E=5841721,C=38)
	CT(13384)=(S=5841722,E=5841723,C=26)
	CT(13385)=(S=5841724,E=5841727,C=38)
	CT(13386)=(S=5841728,E=5841735,C=26)
	CT(13387)=(S=5841736,E=5841752,C=38)
	CT(13388)=(S=5841753,E=5841754,C=14)
	CT(13389)=(S=5841755,E=5841767,C=38)
	CT(13390)=(S=5841768,E=5841775,C=95)
	CT(13391)=(S=5841776,E=5841783,C=38)
	CT(13392)=(S=5841784,E=5841787,C=26)
	CT(13393)=(S=5841788,E=5841789,C=38)
	CT(13394)=(S=5841790,E=5841790,C=44)
	CT(13395)=(S=5841791,E=5841795,C=38)
	CT(13396)=(S=5841796,E=5841797,C=26)
	CT(13397)=(S=5841798,E=5841807,C=38)
	CT(13398)=(S=5841808,E=5841811,C=58)
	CT(13399)=(S=5841812,E=5841815,C=25)
	CT(13400)=(S=5841816,E=5841816,C=38)
	CT(13401)=(S=5841817,E=5841817,C=14)
	CT(13402)=(S=5841818,E=5841819,C=38)
	CT(13403)=(S=5841820,E=5841821,C=26)
	CT(13404)=(S=5841822,E=5841831,C=38)
	CT(13405)=(S=5841832,E=5841835,C=15)
	CT(13406)=(S=5841836,E=5841837,C=38)
	CT(13407)=(S=5841838,E=5841839,C=28)
	CT(13408)=(S=5841840,E=5841841,C=26)
	CT(13409)=(S=5841842,E=5841842,C=170)
	CT(13410)=(S=5841843,E=5841843,C=38)
	CT(13411)=(S=5841844,E=5841847,C=26)
	CT(13412)=(S=5841848,E=5841857,C=38)
	CT(13413)=(S=5841858,E=5841859,C=26)
	CT(13414)=(S=5841860,E=5841863,C=95)
	CT(13415)=(S=5841864,E=5841935,C=38)
	CT(13416)=(S=5841936,E=5841937,C=26)
	CT(13417)=(S=5841938,E=5841951,C=38)
	CT(13418)=(S=5841952,E=5841955,C=95)
	CT(13419)=(S=5841956,E=5841956,C=36)
	CT(13420)=(S=5841957,E=5841957,C=38)
	CT(13421)=(S=5841958,E=5841959,C=14)
	CT(13422)=(S=5841960,E=5841967,C=38)
	CT(13423)=(S=5841968,E=5841983,C=26)
	CT(13424)=(S=5841984,E=5841991,C=95)
	CT(13425)=(S=5841992,E=5841999,C=38)
	CT(13426)=(S=5842000,E=5842007,C=25)
	CT(13427)=(S=5842008,E=5842009,C=38)
	CT(13428)=(S=5842010,E=5842011,C=19)
	CT(13429)=(S=5842012,E=5842015,C=38)
	CT(13430)=(S=5842016,E=5842031,C=26)
	CT(13431)=(S=5842032,E=5842055,C=38)
	CT(13432)=(S=5842056,E=5842056,C=14)
	CT(13433)=(S=5842057,E=5842057,C=38)
	CT(13434)=(S=5842058,E=5842059,C=14)
	CT(13435)=(S=5842060,E=5842060,C=38)
	CT(13436)=(S=5842061,E=5842061,C=14)
	CT(13437)=(S=5842062,E=5842095,C=38)
	CT(13438)=(S=5842096,E=5842111,C=26)
	CT(13439)=(S=5842112,E=5842113,C=38)
	CT(13440)=(S=5842114,E=5842115,C=26)
	CT(13441)=(S=5842116,E=5842127,C=38)
	CT(13442)=(S=5842128,E=5842143,C=15)
	CT(13443)=(S=5842144,E=5842145,C=38)
	CT(13444)=(S=5842146,E=5842147,C=26)
	CT(13445)=(S=5842148,E=5842155,C=38)
	CT(13446)=(S=5842156,E=5842159,C=95)
	CT(13447)=(S=5842160,E=5842171,C=38)
	CT(13448)=(S=5842172,E=5842173,C=26)
	CT(13449)=(S=5842174,E=5842175,C=38)
	CT(13450)=(S=5842176,E=5842191,C=26)
	CT(13451)=(S=5842192,E=5842199,C=41)
	CT(13452)=(S=5842200,E=5842205,C=38)
	CT(13453)=(S=5842206,E=5842207,C=26)
	CT(13454)=(S=5842208,E=5842215,C=41)
	CT(13455)=(S=5842216,E=5842217,C=38)
	CT(13456)=(S=5842218,E=5842219,C=26)
	CT(13457)=(S=5842220,E=5842239,C=38)
	CT(13458)=(S=5842240,E=5842243,C=95)
	CT(13459)=(S=5842244,E=5842255,C=38)
	CT(13460)=(S=5842256,E=5842259,C=25)
	CT(13461)=(S=5842260,E=5842267,C=95)
	CT(13462)=(S=5842268,E=5842270,C=14)
	CT(13463)=(S=5842271,E=5842271,C=38)
	CT(13464)=(S=5842272,E=5842272,C=14)
	CT(13465)=(S=5842273,E=5842277,C=38)
	CT(13466)=(S=5842278,E=5842279,C=26)
	CT(13467)=(S=5842280,E=5842304,C=38)
	CT(13468)=(S=5842305,E=5842310,C=14)
	CT(13469)=(S=5842311,E=5842319,C=38)
	CT(13470)=(S=5842320,E=5842331,C=26)
	CT(13471)=(S=5842332,E=5842343,C=38)
	CT(13472)=(S=5842344,E=5842347,C=26)
	CT(13473)=(S=5842348,E=5842351,C=38)
	CT(13474)=(S=5842352,E=5842359,C=19)
	CT(13475)=(S=5842360,E=5842373,C=38)
	CT(13476)=(S=5842374,E=5842375,C=26)
	CT(13477)=(S=5842376,E=5842383,C=38)
	CT(13478)=(S=5842384,E=5842387,C=26)
	CT(13479)=(S=5842388,E=5842393,C=38)
	CT(13480)=(S=5842394,E=5842395,C=26)
	CT(13481)=(S=5842396,E=5842403,C=38)
	CT(13482)=(S=5842404,E=5842407,C=95)
	CT(13483)=(S=5842408,E=5842411,C=38)
	CT(13484)=(S=5842412,E=5842415,C=95)
	CT(13485)=(S=5842416,E=5842431,C=26)
	CT(13486)=(S=5842432,E=5842455,C=38)
	CT(13487)=(S=5842456,E=5842457,C=26)
	CT(13488)=(S=5842458,E=5842458,C=14)
	CT(13489)=(S=5842459,E=5842459,C=38)
	CT(13490)=(S=5842460,E=5842463,C=95)
	CT(13491)=(S=5842464,E=5842471,C=38)
	CT(13492)=(S=5842472,E=5842479,C=25)
	CT(13493)=(S=5842480,E=5842511,C=38)
	CT(13494)=(S=5842512,E=5842527,C=26)
	CT(13495)=(S=5842528,E=5842531,C=95)
	CT(13496)=(S=5842532,E=5842533,C=38)
	CT(13497)=(S=5842534,E=5842535,C=26)
	CT(13498)=(S=5842536,E=5842575,C=38)
	CT(13499)=(S=5842576,E=5842583,C=15)
	CT(13500)=(S=5842584,E=5842591,C=38)
	CT(13501)=(S=5842592,E=5842595,C=95)
	CT(13502)=(S=5842596,E=5842615,C=38)
	CT(13503)=(S=5842616,E=5842631,C=26)
	CT(13504)=(S=5842632,E=5842643,C=38)
	CT(13505)=(S=5842644,E=5842647,C=26)
	CT(13506)=(S=5842648,E=5842655,C=95)
	CT(13507)=(S=5842656,E=5842669,C=38)
	CT(13508)=(S=5842670,E=5842671,C=12)
	CT(13509)=(S=5842672,E=5842672,C=14)
	CT(13510)=(S=5842673,E=5842673,C=38)
	CT(13511)=(S=5842674,E=5842679,C=26)
	CT(13512)=(S=5842680,E=5842695,C=38)
	CT(13513)=(S=5842696,E=5842699,C=26)
	CT(13514)=(S=5842700,E=5842703,C=38)
	CT(13515)=(S=5842704,E=5842711,C=25)
	CT(13516)=(S=5842712,E=5842756,C=38)
	CT(13517)=(S=5842757,E=5842757,C=14)
	CT(13518)=(S=5842758,E=5842791,C=38)
	CT(13519)=(S=5842792,E=5842795,C=95)
	CT(13520)=(S=5842796,E=5842809,C=38)
	CT(13521)=(S=5842810,E=5842810,C=44)
	CT(13522)=(S=5842811,E=5842815,C=38)
	CT(13523)=(S=5842816,E=5842823,C=28)
	CT(13524)=(S=5842824,E=5842831,C=95)
	CT(13525)=(S=5842832,E=5842833,C=38)
	CT(13526)=(S=5842834,E=5842834,C=14)
	CT(13527)=(S=5842835,E=5842835,C=29)
	CT(13528)=(S=5842836,E=5842839,C=38)
	CT(13529)=(S=5842840,E=5842847,C=25)
	CT(13530)=(S=5842848,E=5842855,C=38)
	CT(13531)=(S=5842856,E=5842859,C=58)
	CT(13532)=(S=5842860,E=5842867,C=38)
	CT(13533)=(S=5842868,E=5842871,C=95)
	CT(13534)=(S=5842872,E=5842873,C=38)
	CT(13535)=(S=5842874,E=5842875,C=26)
	CT(13536)=(S=5842876,E=5842895,C=38)
	CT(13537)=(S=5842896,E=5842899,C=95)
	CT(13538)=(S=5842900,E=5842900,C=14)
	CT(13539)=(S=5842901,E=5842902,C=38)
	CT(13540)=(S=5842903,E=5842903,C=14)
	CT(13541)=(S=5842904,E=5842928,C=38)
	CT(13542)=(S=5842929,E=5842929,C=14)
	CT(13543)=(S=5842930,E=5842930,C=38)
	CT(13544)=(S=5842931,E=5842932,C=14)
	CT(13545)=(S=5842933,E=5842934,C=38)
	CT(13546)=(S=5842935,E=5842935,C=14)
	CT(13547)=(S=5842936,E=5842943,C=38)
	CT(13548)=(S=5842944,E=5842951,C=95)
	CT(13549)=(S=5842952,E=5842981,C=38)
	CT(13550)=(S=5842982,E=5842983,C=26)
	CT(13551)=(S=5842984,E=5842987,C=38)
	CT(13552)=(S=5842988,E=5842989,C=14)
	CT(13553)=(S=5842990,E=5842991,C=38)
	CT(13554)=(S=5842992,E=5843007,C=20)
	CT(13555)=(S=5843008,E=5843021,C=38)
	CT(13556)=(S=5843022,E=5843023,C=26)
	CT(13557)=(S=5843024,E=5843027,C=38)
	CT(13558)=(S=5843028,E=5843031,C=25)
	CT(13559)=(S=5843032,E=5843033,C=38)
	CT(13560)=(S=5843034,E=5843035,C=26)
	CT(13561)=(S=5843036,E=5843036,C=38)
	CT(13562)=(S=5843037,E=5843039,C=14)
	CT(13563)=(S=5843040,E=5843049,C=38)
	CT(13564)=(S=5843050,E=5843051,C=26)
	CT(13565)=(S=5843052,E=5843053,C=38)
	CT(13566)=(S=5843054,E=5843055,C=26)
	CT(13567)=(S=5843056,E=5843071,C=38)
	CT(13568)=(S=5843072,E=5843073,C=26)
	CT(13569)=(S=5843074,E=5843077,C=38)
	CT(13570)=(S=5843078,E=5843079,C=28)
	CT(13571)=(S=5843080,E=5843083,C=38)
	CT(13572)=(S=5843084,E=5843085,C=14)
	CT(13573)=(S=5843086,E=5843095,C=38)
	CT(13574)=(S=5843096,E=5843103,C=26)
	CT(13575)=(S=5843104,E=5843115,C=38)
	CT(13576)=(S=5843116,E=5843119,C=15)
	CT(13577)=(S=5843120,E=5843150,C=38)
	CT(13578)=(S=5843151,E=5843153,C=14)
	CT(13579)=(S=5843154,E=5843159,C=38)
	CT(13580)=(S=5843160,E=5843161,C=19)
	CT(13581)=(S=5843162,E=5843183,C=38)
	CT(13582)=(S=5843184,E=5843199,C=26)
	CT(13583)=(S=5843200,E=5843207,C=41)
	CT(13584)=(S=5843208,E=5843223,C=26)
	CT(13585)=(S=5843224,E=5843224,C=14)
	CT(13586)=(S=5843225,E=5843226,C=38)
	CT(13587)=(S=5843227,E=5843229,C=14)
	CT(13588)=(S=5843230,E=5843231,C=38)
	CT(13589)=(S=5843232,E=5843233,C=26)
	CT(13590)=(S=5843234,E=5843243,C=38)
	CT(13591)=(S=5843244,E=5843247,C=95)
	CT(13592)=(S=5843248,E=5843248,C=38)
	CT(13593)=(S=5843249,E=5843251,C=14)
	CT(13594)=(S=5843252,E=5843255,C=95)
	CT(13595)=(S=5843256,E=5843257,C=38)
	CT(13596)=(S=5843258,E=5843259,C=26)
	CT(13597)=(S=5843260,E=5843359,C=38)
	CT(13598)=(S=5843360,E=5843367,C=19)
	CT(13599)=(S=5843368,E=5843383,C=38)
	CT(13600)=(S=5843384,E=5843387,C=26)
	CT(13601)=(S=5843388,E=5843388,C=38)
	CT(13602)=(S=5843389,E=5843391,C=14)
	CT(13603)=(S=5843392,E=5843447,C=26)
	CT(13604)=(S=5843448,E=5843463,C=38)
	CT(13605)=(S=5843464,E=5843464,C=44)
	CT(13606)=(S=5843465,E=5843465,C=38)
	CT(13607)=(S=5843466,E=5843466,C=14)
	CT(13608)=(S=5843467,E=5843487,C=38)
	CT(13609)=(S=5843488,E=5843489,C=26)
	CT(13610)=(S=5843490,E=5843499,C=38)
	CT(13611)=(S=5843500,E=5843503,C=26)
	CT(13612)=(S=5843504,E=5843511,C=38)
	CT(13613)=(S=5843512,E=5843513,C=26)
	CT(13614)=(S=5843514,E=5843523,C=38)
	CT(13615)=(S=5843524,E=5843525,C=26)
	CT(13616)=(S=5843526,E=5843551,C=38)
	CT(13617)=(S=5843552,E=5843559,C=26)
	CT(13618)=(S=5843560,E=5843569,C=38)
	CT(13619)=(S=5843570,E=5843570,C=14)
	CT(13620)=(S=5843571,E=5843591,C=38)
	CT(13621)=(S=5843592,E=5843595,C=26)
	CT(13622)=(S=5843596,E=5843597,C=19)
	CT(13623)=(S=5843598,E=5843605,C=38)
	CT(13624)=(S=5843606,E=5843607,C=26)
	CT(13625)=(S=5843608,E=5843610,C=14)
	CT(13626)=(S=5843611,E=5843619,C=38)
	CT(13627)=(S=5843620,E=5843623,C=95)
	CT(13628)=(S=5843624,E=5843627,C=36)
	CT(13629)=(S=5843628,E=5843639,C=38)
	CT(13630)=(S=5843640,E=5843647,C=26)
	CT(13631)=(S=5843648,E=5843651,C=38)
	CT(13632)=(S=5843652,E=5843655,C=26)
	CT(13633)=(S=5843656,E=5843663,C=38)
	CT(13634)=(S=5843664,E=5843667,C=95)
	CT(13635)=(S=5843668,E=5843683,C=38)
	CT(13636)=(S=5843684,E=5843685,C=26)
	CT(13637)=(S=5843686,E=5843691,C=38)
	CT(13638)=(S=5843692,E=5843695,C=95)
	CT(13639)=(S=5843696,E=5843711,C=38)
	CT(13640)=(S=5843712,E=5843727,C=26)
	CT(13641)=(S=5843728,E=5843743,C=38)
	CT(13642)=(S=5843744,E=5843744,C=44)
	CT(13643)=(S=5843745,E=5843747,C=38)
	CT(13644)=(S=5843748,E=5843749,C=26)
	CT(13645)=(S=5843750,E=5843767,C=38)
	CT(13646)=(S=5843768,E=5843771,C=95)
	CT(13647)=(S=5843772,E=5843772,C=36)
	CT(13648)=(S=5843773,E=5843781,C=38)
	CT(13649)=(S=5843782,E=5843783,C=26)
	CT(13650)=(S=5843784,E=5843785,C=38)
	CT(13651)=(S=5843786,E=5843789,C=25)
	CT(13652)=(S=5843790,E=5843795,C=38)
	CT(13653)=(S=5843796,E=5843799,C=25)
	CT(13654)=(S=5843800,E=5843815,C=26)
	CT(13655)=(S=5843816,E=5843823,C=38)
	CT(13656)=(S=5843824,E=5843831,C=95)
	CT(13657)=(S=5843832,E=5843893,C=38)
	CT(13658)=(S=5843894,E=5843895,C=26)
	CT(13659)=(S=5843896,E=5843899,C=95)
	CT(13660)=(S=5843900,E=5843901,C=26)
	CT(13661)=(S=5843902,E=5843907,C=38)
	CT(13662)=(S=5843908,E=5843909,C=25)
	CT(13663)=(S=5843910,E=5843915,C=38)
	CT(13664)=(S=5843916,E=5843917,C=26)
	CT(13665)=(S=5843918,E=5843927,C=38)
	CT(13666)=(S=5843928,E=5843943,C=26)
	CT(13667)=(S=5843944,E=5843951,C=19)
	CT(13668)=(S=5843952,E=5843967,C=38)
	CT(13669)=(S=5843968,E=5843975,C=19)
	CT(13670)=(S=5843976,E=5843976,C=28)
	CT(13671)=(S=5843977,E=5843983,C=38)
	CT(13672)=(S=5843984,E=5843999,C=95)
	CT(13673)=(S=5844000,E=5844007,C=38)
	CT(13674)=(S=5844008,E=5844011,C=95)
	CT(13675)=(S=5844012,E=5844031,C=38)
	CT(13676)=(S=5844032,E=5844039,C=25)
	CT(13677)=(S=5844040,E=5844043,C=95)
	CT(13678)=(S=5844044,E=5844059,C=38)
	CT(13679)=(S=5844060,E=5844067,C=95)
	CT(13680)=(S=5844068,E=5844070,C=38)
	CT(13681)=(S=5844071,E=5844071,C=36)
	CT(13682)=(S=5844072,E=5844079,C=38)
	CT(13683)=(S=5844080,E=5844081,C=26)
	CT(13684)=(S=5844082,E=5844085,C=38)
	CT(13685)=(S=5844086,E=5844087,C=26)
	CT(13686)=(S=5844088,E=5844095,C=38)
	CT(13687)=(S=5844096,E=5844103,C=26)
	CT(13688)=(S=5844104,E=5844113,C=38)
	CT(13689)=(S=5844114,E=5844115,C=26)
	CT(13690)=(S=5844116,E=5844123,C=38)
	CT(13691)=(S=5844124,E=5844127,C=95)
	CT(13692)=(S=5844128,E=5844143,C=38)
	CT(13693)=(S=5844144,E=5844151,C=26)
	CT(13694)=(S=5844152,E=5844157,C=38)
	CT(13695)=(S=5844158,E=5844159,C=26)
	CT(13696)=(S=5844160,E=5844169,C=38)
	CT(13697)=(S=5844170,E=5844171,C=26)
	CT(13698)=(S=5844172,E=5844183,C=38)
	CT(13699)=(S=5844184,E=5844191,C=95)
	CT(13700)=(S=5844192,E=5844207,C=38)
	CT(13701)=(S=5844208,E=5844211,C=26)
	CT(13702)=(S=5844212,E=5844215,C=38)
	CT(13703)=(S=5844216,E=5844223,C=19)
	CT(13704)=(S=5844224,E=5844227,C=53)
	CT(13705)=(S=5844228,E=5844259,C=38)
	CT(13706)=(S=5844260,E=5844263,C=25)
	CT(13707)=(S=5844264,E=5844271,C=38)
	CT(13708)=(S=5844272,E=5844287,C=26)
	CT(13709)=(S=5844288,E=5844290,C=38)
	CT(13710)=(S=5844291,E=5844291,C=36)
	CT(13711)=(S=5844292,E=5844293,C=26)
	CT(13712)=(S=5844294,E=5844303,C=38)
	CT(13713)=(S=5844304,E=5844305,C=26)
	CT(13714)=(S=5844306,E=5844306,C=14)
	CT(13715)=(S=5844307,E=5844335,C=38)
	CT(13716)=(S=5844336,E=5844343,C=26)
	CT(13717)=(S=5844344,E=5844347,C=95)
	CT(13718)=(S=5844348,E=5844349,C=38)
	CT(13719)=(S=5844350,E=5844351,C=26)
	CT(13720)=(S=5844352,E=5844375,C=38)
	CT(13721)=(S=5844376,E=5844383,C=26)
	CT(13722)=(S=5844384,E=5844449,C=38)
	CT(13723)=(S=5844450,E=5844451,C=19)
	CT(13724)=(S=5844452,E=5844453,C=38)
	CT(13725)=(S=5844454,E=5844455,C=26)
	CT(13726)=(S=5844456,E=5844463,C=19)
	CT(13727)=(S=5844464,E=5844471,C=25)
	CT(13728)=(S=5844472,E=5844487,C=38)
	CT(13729)=(S=5844488,E=5844491,C=25)
	CT(13730)=(S=5844492,E=5844495,C=38)
	CT(13731)=(S=5844496,E=5844503,C=19)
	CT(13732)=(S=5844504,E=5844523,C=38)
	CT(13733)=(S=5844524,E=5844525,C=26)
	CT(13734)=(S=5844526,E=5844526,C=38)
	CT(13735)=(S=5844527,E=5844527,C=14)
	CT(13736)=(S=5844528,E=5844539,C=38)
	CT(13737)=(S=5844540,E=5844541,C=26)
	CT(13738)=(S=5844542,E=5844543,C=38)
	CT(13739)=(S=5844544,E=5844559,C=15)
	CT(13740)=(S=5844560,E=5844573,C=38)
	CT(13741)=(S=5844574,E=5844575,C=26)
	CT(13742)=(S=5844576,E=5844583,C=38)
	CT(13743)=(S=5844584,E=5844591,C=15)
	CT(13744)=(S=5844592,E=5844615,C=38)
	CT(13745)=(S=5844616,E=5844623,C=25)
	CT(13746)=(S=5844624,E=5844655,C=38)
	CT(13747)=(S=5844656,E=5844663,C=43)
	CT(13748)=(S=5844664,E=5844671,C=26)
	CT(13749)=(S=5844672,E=5844679,C=15)
	CT(13750)=(S=5844680,E=5844687,C=35)
	CT(13751)=(S=5844688,E=5844695,C=38)
	CT(13752)=(S=5844696,E=5844699,C=26)
	CT(13753)=(S=5844700,E=5844739,C=38)
	CT(13754)=(S=5844740,E=5844743,C=25)
	CT(13755)=(S=5844744,E=5844751,C=38)
	CT(13756)=(S=5844752,E=5844767,C=95)
	CT(13757)=(S=5844768,E=5844779,C=38)
	CT(13758)=(S=5844780,E=5844783,C=95)
	CT(13759)=(S=5844784,E=5844799,C=38)
	CT(13760)=(S=5844800,E=5844815,C=26)
	CT(13761)=(S=5844816,E=5844863,C=38)
	CT(13762)=(S=5844864,E=5844895,C=26)
	CT(13763)=(S=5844896,E=5844919,C=38)
	CT(13764)=(S=5844920,E=5844927,C=19)
	CT(13765)=(S=5844928,E=5844931,C=38)
	CT(13766)=(S=5844932,E=5844939,C=26)
	CT(13767)=(S=5844940,E=5844991,C=38)
	CT(13768)=(S=5844992,E=5849087,C=28)
	CT(13769)=(S=5849088,E=5853183,C=58)
	CT(13770)=(S=5853184,E=5857279,C=12)
	CT(13771)=(S=5857280,E=5857791,C=15)
	CT(13772)=(S=5857792,E=5858303,C=36)
	CT(13773)=(S=5858304,E=5858815,C=63)
	CT(13774)=(S=5858816,E=5859327,C=33)
	CT(13775)=(S=5859328,E=5859359,C=42)
	CT(13776)=(S=5859360,E=5859391,C=14)
	CT(13777)=(S=5859392,E=5859455,C=18)
	CT(13778)=(S=5859456,E=5859487,C=48)
	CT(13779)=(S=5859488,E=5859519,C=36)
	CT(13780)=(S=5859520,E=5859551,C=24)
	CT(13781)=(S=5859552,E=5859615,C=14)
	CT(13782)=(S=5859616,E=5859647,C=27)
	CT(13783)=(S=5859648,E=5859679,C=15)
	CT(13784)=(S=5859680,E=5859711,C=14)
	CT(13785)=(S=5859712,E=5859743,C=18)
	CT(13786)=(S=5859744,E=5859775,C=42)
	CT(13787)=(S=5859776,E=5859807,C=36)
	CT(13788)=(S=5859808,E=5859839,C=31)
	CT(13789)=(S=5859840,E=5859871,C=37)
	CT(13790)=(S=5859872,E=5859903,C=47)
	CT(13791)=(S=5859904,E=5859935,C=28)
	CT(13792)=(S=5859936,E=5859967,C=51)
	CT(13793)=(S=5859968,E=5859975,C=14)
	CT(13794)=(S=5859976,E=5859983,C=63)
	CT(13795)=(S=5859984,E=5859991,C=14)
	CT(13796)=(S=5859992,E=5859999,C=28)
	CT(13797)=(S=5860000,E=5860007,C=36)
	CT(13798)=(S=5860008,E=5860015,C=18)
	CT(13799)=(S=5860016,E=5860023,C=14)
	CT(13800)=(S=5860024,E=5860031,C=12)
	CT(13801)=(S=5860032,E=5860039,C=18)
	CT(13802)=(S=5860040,E=5860047,C=70)
	CT(13803)=(S=5860048,E=5860055,C=42)
	CT(13804)=(S=5860056,E=5860063,C=28)
	CT(13805)=(S=5860064,E=5860071,C=14)
	CT(13806)=(S=5860072,E=5860079,C=20)
	CT(13807)=(S=5860080,E=5860087,C=49)
	CT(13808)=(S=5860088,E=5860095,C=18)
	CT(13809)=(S=5860096,E=5860103,C=14)
	CT(13810)=(S=5860104,E=5860111,C=101)
	CT(13811)=(S=5860112,E=5860119,C=14)
	CT(13812)=(S=5860128,E=5860135,C=18)
	CT(13813)=(S=5860136,E=5860143,C=14)
	CT(13814)=(S=5860144,E=5860151,C=25)
	CT(13815)=(S=5860152,E=5860159,C=14)
	CT(13816)=(S=5860160,E=5860167,C=28)
	CT(13817)=(S=5860168,E=5860175,C=14)
	CT(13818)=(S=5860176,E=5860183,C=18)
	CT(13819)=(S=5860184,E=5860191,C=15)
	CT(13820)=(S=5860192,E=5860199,C=20)
	CT(13821)=(S=5860200,E=5860207,C=15)
	CT(13822)=(S=5860208,E=5860223,C=18)
	CT(13823)=(S=5860224,E=5860231,C=28)
	CT(13824)=(S=5860232,E=5860239,C=18)
	CT(13825)=(S=5860240,E=5860247,C=71)
	CT(13826)=(S=5860248,E=5860255,C=58)
	CT(13827)=(S=5860256,E=5860263,C=28)
	CT(13828)=(S=5860264,E=5860271,C=12)
	CT(13829)=(S=5860272,E=5860279,C=25)
	CT(13830)=(S=5860280,E=5860287,C=28)
	CT(13831)=(S=5860288,E=5860303,C=18)
	CT(13832)=(S=5860304,E=5860319,C=19)
	CT(13833)=(S=5860320,E=5860327,C=37)
	CT(13834)=(S=5860328,E=5860335,C=14)
	CT(13835)=(S=5860336,E=5860343,C=25)
	CT(13836)=(S=5860344,E=5860351,C=50)
	CT(13837)=(S=5860352,E=5860415,C=23)
	CT(13838)=(S=5860416,E=5860479,C=18)
	CT(13839)=(S=5860480,E=5860543,C=39)
	CT(13840)=(S=5860544,E=5860607,C=58)
	CT(13841)=(S=5860608,E=5860671,C=18)
	CT(13842)=(S=5860672,E=5860735,C=21)
	CT(13843)=(S=5860736,E=5860927,C=18)
	CT(13844)=(S=5860928,E=5860991,C=31)
	CT(13845)=(S=5860992,E=5861055,C=28)
	CT(13846)=(S=5861056,E=5861119,C=46)
	CT(13847)=(S=5861120,E=5861183,C=60)
	CT(13848)=(S=5861184,E=5861247,C=33)
	CT(13849)=(S=5861248,E=5861311,C=18)
	CT(13850)=(S=5861312,E=5861375,C=59)
	CT(13851)=(S=5861376,E=5861887,C=18)
	CT(13852)=(S=5861888,E=5862399,C=21)
	CT(13853)=(S=5862400,E=5862911,C=44)
	CT(13854)=(S=5862912,E=5863423,C=15)
	CT(13855)=(S=5863424,E=5864447,C=38)
	CT(13856)=(S=5864448,E=5865471,C=63)
	CT(13857)=(S=5865472,E=5866495,C=25)
	CT(13858)=(S=5866496,E=5867519,C=50)
	CT(13859)=(S=5867520,E=5868031,C=38)
	CT(13860)=(S=5868032,E=5868543,C=17)
	CT(13861)=(S=5868544,E=5869055,C=25)
	CT(13862)=(S=5869056,E=5869567,C=52)
	CT(13863)=(S=5869568,E=5869631,C=28)
	CT(13864)=(S=5869632,E=5869695,C=23)
	CT(13865)=(S=5869696,E=5869759,C=26)
	CT(13866)=(S=5869760,E=5869823,C=42)
	CT(13867)=(S=5869824,E=5869887,C=28)
	CT(13868)=(S=5869888,E=5869951,C=14)
	CT(13869)=(S=5869952,E=5869983,C=18)
	CT(13870)=(S=5869984,E=5869991,C=15)
	CT(13871)=(S=5869992,E=5869999,C=14)
	CT(13872)=(S=5870000,E=5870007,C=34)
	CT(13873)=(S=5870008,E=5870015,C=37)
	CT(13874)=(S=5870016,E=5870079,C=14)
	CT(13875)=(S=5870080,E=5870143,C=36)
	CT(13876)=(S=5870144,E=5870207,C=98)
	CT(13877)=(S=5870208,E=5870271,C=59)
	CT(13878)=(S=5870272,E=5870335,C=28)
	CT(13879)=(S=5870336,E=5870399,C=23)
	CT(13880)=(S=5870400,E=5870463,C=50)
	CT(13881)=(S=5870464,E=5870527,C=19)
	CT(13882)=(S=5870528,E=5870591,C=41)
	CT(13883)=(S=5870592,E=5870655,C=104)
	CT(13884)=(S=5870656,E=5870719,C=50)
	CT(13885)=(S=5870720,E=5870783,C=15)
	CT(13886)=(S=5870784,E=5870847,C=18)
	CT(13887)=(S=5870848,E=5870911,C=38)
	CT(13888)=(S=5870912,E=5870975,C=53)
	CT(13889)=(S=5870976,E=5871039,C=63)
	CT(13890)=(S=5871040,E=5871103,C=28)
	CT(13891)=(S=5871104,E=5871119,C=40)
	CT(13892)=(S=5871120,E=5871135,C=14)
	CT(13893)=(S=5871136,E=5871167,C=15)
	CT(13894)=(S=5871168,E=5871295,C=24)
	CT(13895)=(S=5871296,E=5871359,C=19)
	CT(13896)=(S=5871360,E=5871423,C=58)
	CT(13897)=(S=5871424,E=5871487,C=14)
	CT(13898)=(S=5871488,E=5871551,C=18)
	CT(13899)=(S=5871552,E=5871615,C=27)
	CT(13900)=(S=5871616,E=5872639,C=21)
	CT(13901)=(S=5872640,E=5873663,C=12)
	CT(13902)=(S=5873664,E=5873791,C=19)
	CT(13903)=(S=5873792,E=5873919,C=48)
	CT(13904)=(S=5873920,E=5874175,C=58)
	CT(13905)=(S=5874176,E=5874303,C=27)
	CT(13906)=(S=5874304,E=5874431,C=31)
	CT(13907)=(S=5874432,E=5874559,C=18)
	CT(13908)=(S=5874560,E=5874687,C=28)
	CT(13909)=(S=5874688,E=5874943,C=57)
	CT(13910)=(S=5874944,E=5875071,C=26)
	CT(13911)=(S=5875072,E=5875199,C=38)
	CT(13912)=(S=5875200,E=5875327,C=47)
	CT(13913)=(S=5875328,E=5875455,C=28)
	CT(13914)=(S=5875456,E=5875583,C=58)
	CT(13915)=(S=5875584,E=5875967,C=14)
	CT(13916)=(S=5875968,E=5876223,C=18)
	CT(13917)=(S=5876224,E=5876479,C=12)
	CT(13918)=(S=5876480,E=5876735,C=58)
	CT(13919)=(S=5876736,E=5876991,C=57)
	CT(13920)=(S=5876992,E=5877247,C=71)
	CT(13921)=(S=5877248,E=5877503,C=58)
	CT(13922)=(S=5877504,E=5877759,C=18)
	CT(13923)=(S=5877760,E=5878271,C=33)
	CT(13924)=(S=5878272,E=5878783,C=18)
	CT(13925)=(S=5878784,E=5879295,C=21)
	CT(13926)=(S=5879296,E=5879807,C=28)
	CT(13927)=(S=5879808,E=5879839,C=18)
	CT(13928)=(S=5879840,E=5879871,C=63)
	CT(13929)=(S=5879872,E=5879903,C=31)
	CT(13930)=(S=5879904,E=5879935,C=15)
	CT(13931)=(S=5879936,E=5879967,C=24)
	CT(13932)=(S=5879968,E=5879999,C=36)
	CT(13933)=(S=5880000,E=5880031,C=26)
	CT(13934)=(S=5880032,E=5880063,C=18)
	CT(13935)=(S=5880064,E=5880095,C=31)
	CT(13936)=(S=5880096,E=5880127,C=12)
	CT(13937)=(S=5880128,E=5880159,C=18)
	CT(13938)=(S=5880160,E=5880191,C=42)
	CT(13939)=(S=5880192,E=5880223,C=63)
	CT(13940)=(S=5880224,E=5880255,C=12)
	CT(13941)=(S=5880256,E=5880287,C=54)
	CT(13942)=(S=5880288,E=5880319,C=33)
	CT(13943)=(S=5880320,E=5880351,C=58)
	CT(13944)=(S=5880352,E=5880383,C=100)
	CT(13945)=(S=5880384,E=5880415,C=15)
	CT(13946)=(S=5880416,E=5880447,C=50)
	CT(13947)=(S=5880448,E=5880479,C=28)
	CT(13948)=(S=5880480,E=5880511,C=24)
	CT(13949)=(S=5880512,E=5880519,C=34)
	CT(13950)=(S=5880520,E=5880527,C=51)
	CT(13951)=(S=5880528,E=5880535,C=34)
	CT(13952)=(S=5880536,E=5880543,C=42)
	CT(13953)=(S=5880544,E=5880575,C=18)
	CT(13954)=(S=5880576,E=5880607,C=31)
	CT(13955)=(S=5880608,E=5880639,C=53)
	CT(13956)=(S=5880640,E=5880703,C=14)
	CT(13957)=(S=5880704,E=5880775,C=33)
	CT(13958)=(S=5880776,E=5880791,C=28)
	CT(13959)=(S=5880792,E=5880799,C=39)
	CT(13960)=(S=5880800,E=5880831,C=58)
	CT(13961)=(S=5880832,E=5880863,C=36)
	CT(13962)=(S=5880864,E=5880895,C=102)
	CT(13963)=(S=5880896,E=5880903,C=42)
	CT(13964)=(S=5880904,E=5880927,C=24)
	CT(13965)=(S=5880928,E=5880959,C=18)
	CT(13966)=(S=5880960,E=5880991,C=15)
	CT(13967)=(S=5880992,E=5881023,C=18)
	CT(13968)=(S=5881024,E=5881055,C=58)
	CT(13969)=(S=5881056,E=5881119,C=18)
	CT(13970)=(S=5881120,E=5881151,C=15)
	CT(13971)=(S=5881152,E=5881183,C=67)
	CT(13972)=(S=5881184,E=5881279,C=18)
	CT(13973)=(S=5881280,E=5881311,C=19)
	CT(13974)=(S=5881312,E=5881343,C=23)
	CT(13975)=(S=5881344,E=5881375,C=47)
	CT(13976)=(S=5881376,E=5881439,C=33)
	CT(13977)=(S=5881440,E=5881471,C=44)
	CT(13978)=(S=5881504,E=5881535,C=15)
	CT(13979)=(S=5881536,E=5881567,C=51)
	CT(13980)=(S=5881568,E=5881599,C=18)
	CT(13981)=(S=5881600,E=5881631,C=27)
	CT(13982)=(S=5881632,E=5881663,C=63)
	CT(13983)=(S=5881664,E=5881695,C=28)
	CT(13984)=(S=5881696,E=5881727,C=60)
	CT(13985)=(S=5881728,E=5881791,C=58)
	CT(13986)=(S=5881792,E=5881823,C=14)
	CT(13987)=(S=5881824,E=5881855,C=18)
	CT(13988)=(S=5881856,E=5882879,C=14)
	CT(13989)=(S=5882880,E=5883135,C=26)
	CT(13990)=(S=5883136,E=5883391,C=28)
	CT(13991)=(S=5883392,E=5883903,C=26)
	CT(13992)=(S=5883904,E=5884031,C=36)
	CT(13993)=(S=5884032,E=5884047,C=14)
	CT(13994)=(S=5884048,E=5884071,C=58)
	CT(13995)=(S=5884072,E=5884079,C=28)
	CT(13996)=(S=5884080,E=5884095,C=18)
	CT(13997)=(S=5884096,E=5884103,C=47)
	CT(13998)=(S=5884104,E=5884111,C=36)
	CT(13999)=(S=5884112,E=5884119,C=58)
	CT(14000)=(S=5884120,E=5884127,C=14)
	CT(14001)=(S=5884128,E=5884135,C=58)
	CT(14002)=(S=5884136,E=5884143,C=31)
	CT(14003)=(S=5884144,E=5884151,C=38)
	CT(14004)=(S=5884152,E=5884159,C=31)
	CT(14005)=(S=5884160,E=5884287,C=60)
	CT(14006)=(S=5884288,E=5884415,C=57)
	CT(14007)=(S=5884416,E=5884543,C=28)
	CT(14008)=(S=5884544,E=5884671,C=14)
	CT(14009)=(S=5884672,E=5884799,C=101)
	CT(14010)=(S=5884800,E=5884927,C=33)
	CT(14011)=(S=5884928,E=5885055,C=18)
	CT(14012)=(S=5885056,E=5885087,C=28)
	CT(14013)=(S=5885088,E=5885183,C=63)
	CT(14014)=(S=5885184,E=5885311,C=54)
	CT(14015)=(S=5885312,E=5885439,C=36)
	CT(14016)=(S=5885440,E=5885503,C=58)
	CT(14017)=(S=5885504,E=5885567,C=34)
	CT(14018)=(S=5885568,E=5885703,C=14)
	CT(14019)=(S=5885712,E=5885719,C=19)
	CT(14020)=(S=5885720,E=5885727,C=36)
	CT(14021)=(S=5885728,E=5885743,C=18)
	CT(14022)=(S=5885744,E=5885751,C=14)
	CT(14023)=(S=5885752,E=5885759,C=63)
	CT(14024)=(S=5885760,E=5885775,C=18)
	CT(14025)=(S=5885776,E=5885783,C=15)
	CT(14026)=(S=5885784,E=5885799,C=18)
	CT(14027)=(S=5885800,E=5885807,C=15)
	CT(14028)=(S=5885808,E=5885815,C=27)
	CT(14029)=(S=5885816,E=5885823,C=42)
	CT(14030)=(S=5885824,E=5885831,C=36)
	CT(14031)=(S=5885832,E=5885839,C=14)
	CT(14032)=(S=5885840,E=5885847,C=42)
	CT(14033)=(S=5885848,E=5885871,C=14)
	CT(14034)=(S=5885880,E=5885887,C=49)
	CT(14035)=(S=5885888,E=5885895,C=102)
	CT(14036)=(S=5885896,E=5885903,C=28)
	CT(14037)=(S=5885904,E=5885911,C=14)
	CT(14038)=(S=5885912,E=5885919,C=18)
	CT(14039)=(S=5885920,E=5885935,C=34)
	CT(14040)=(S=5885936,E=5885943,C=60)
	CT(14041)=(S=5885944,E=5885951,C=28)
	CT(14042)=(S=5885952,E=5886207,C=18)
	CT(14043)=(S=5886208,E=5886463,C=31)
	CT(14044)=(S=5886464,E=5886719,C=22)
	CT(14045)=(S=5886720,E=5886975,C=94)
	CT(14046)=(S=5886976,E=5887231,C=52)
	CT(14047)=(S=5887232,E=5887487,C=14)
	CT(14048)=(S=5887488,E=5887743,C=21)
	CT(14049)=(S=5887744,E=5887999,C=51)
	CT(14050)=(S=5888000,E=5888255,C=46)
	CT(14051)=(S=5888256,E=5888511,C=34)
	CT(14052)=(S=5888512,E=5888767,C=20)
	CT(14053)=(S=5888768,E=5888771,C=55)
	CT(14054)=(S=5888772,E=5888803,C=20)
	CT(14055)=(S=5888804,E=5888831,C=41)
	CT(14056)=(S=5888832,E=5888895,C=26)
	CT(14057)=(S=5888896,E=5888899,C=55)
	CT(14058)=(S=5888900,E=5888901,C=41)
	CT(14059)=(S=5888902,E=5888922,C=55)
	CT(14060)=(S=5888923,E=5888928,C=41)
	CT(14061)=(S=5888929,E=5888930,C=55)
	CT(14062)=(S=5888931,E=5888935,C=41)
	CT(14063)=(S=5888936,E=5888936,C=55)
	CT(14064)=(S=5888937,E=5888959,C=41)
	CT(14065)=(S=5888960,E=5889023,C=26)
	CT(14066)=(S=5889024,E=5889279,C=36)
	CT(14067)=(S=5889280,E=5889295,C=28)
	CT(14068)=(S=5889296,E=5889311,C=18)
	CT(14069)=(S=5889312,E=5889327,C=15)
	CT(14070)=(S=5889328,E=5889343,C=18)
	CT(14071)=(S=5889344,E=5889359,C=55)
	CT(14072)=(S=5889360,E=5889375,C=26)
	CT(14073)=(S=5889376,E=5889391,C=27)
	CT(14074)=(S=5889392,E=5889407,C=60)
	CT(14075)=(S=5889408,E=5889439,C=15)
	CT(14076)=(S=5889440,E=5889455,C=24)
	CT(14077)=(S=5889456,E=5889471,C=36)
	CT(14078)=(S=5889472,E=5889487,C=18)
	CT(14079)=(S=5889488,E=5889503,C=33)
	CT(14080)=(S=5889504,E=5889519,C=18)
	CT(14081)=(S=5889520,E=5889535,C=19)
	CT(14082)=(S=5889536,E=5889663,C=14)
	CT(14083)=(S=5889664,E=5889919,C=18)
	CT(14084)=(S=5889920,E=5890047,C=50)
	CT(14085)=(S=5890048,E=5891071,C=12)
	CT(14086)=(S=5891072,E=5892095,C=58)
	CT(14087)=(S=5892096,E=5892159,C=40)
	CT(14088)=(S=5892160,E=5892351,C=18)
	CT(14089)=(S=5892352,E=5892415,C=24)
	CT(14090)=(S=5892416,E=5892447,C=28)
	CT(14091)=(S=5892448,E=5892455,C=27)
	CT(14092)=(S=5892456,E=5892463,C=58)
	CT(14093)=(S=5892464,E=5892479,C=52)
	CT(14094)=(S=5892480,E=5892543,C=33)
	CT(14095)=(S=5892544,E=5892607,C=19)
	CT(14096)=(S=5892608,E=5892671,C=14)
	CT(14097)=(S=5892672,E=5892735,C=63)
	CT(14098)=(S=5892736,E=5892799,C=12)
	CT(14099)=(S=5892800,E=5892863,C=58)
	CT(14100)=(S=5892864,E=5892927,C=33)
	CT(14101)=(S=5892928,E=5892991,C=26)
	CT(14102)=(S=5892992,E=5893055,C=18)
	CT(14103)=(S=5893056,E=5893119,C=55)
	CT(14104)=(S=5893120,E=5893183,C=19)
	CT(14105)=(S=5893184,E=5893247,C=47)
	CT(14106)=(S=5893248,E=5893311,C=34)
	CT(14107)=(S=5893312,E=5893375,C=98)
	CT(14108)=(S=5893376,E=5893439,C=18)
	CT(14109)=(S=5893440,E=5893503,C=12)
	CT(14110)=(S=5893504,E=5893567,C=23)
	CT(14111)=(S=5893568,E=5893631,C=62)
	CT(14112)=(S=5893632,E=5893695,C=58)
	CT(14113)=(S=5893696,E=5893759,C=28)
	CT(14114)=(S=5893760,E=5893823,C=14)
	CT(14115)=(S=5893824,E=5893887,C=38)
	CT(14116)=(S=5893888,E=5893951,C=33)
	CT(14117)=(S=5893952,E=5894015,C=58)
	CT(14118)=(S=5894016,E=5894079,C=18)
	CT(14119)=(S=5894080,E=5894143,C=24)
	CT(14120)=(S=5894144,E=5895167,C=14)
	CT(14121)=(S=5895168,E=5896191,C=28)
	CT(14122)=(S=5896192,E=5896207,C=27)
	CT(14123)=(S=5896208,E=5896223,C=14)
	CT(14124)=(S=5896224,E=5896239,C=34)
	CT(14125)=(S=5896240,E=5896263,C=14)
	CT(14126)=(S=5896264,E=5896271,C=12)
	CT(14127)=(S=5896272,E=5896287,C=60)
	CT(14128)=(S=5896288,E=5896303,C=25)
	CT(14129)=(S=5896304,E=5896319,C=18)
	CT(14130)=(S=5896320,E=5896335,C=36)
	CT(14131)=(S=5896336,E=5896351,C=56)
	CT(14132)=(S=5896352,E=5896367,C=36)
	CT(14133)=(S=5896368,E=5896399,C=18)
	CT(14134)=(S=5896400,E=5896415,C=12)
	CT(14135)=(S=5896416,E=5896431,C=18)
	CT(14136)=(S=5896432,E=5896447,C=33)
	CT(14137)=(S=5896448,E=5896463,C=24)
	CT(14138)=(S=5896464,E=5896479,C=18)
	CT(14139)=(S=5896480,E=5896495,C=34)
	CT(14140)=(S=5896496,E=5896511,C=18)
	CT(14141)=(S=5896512,E=5896527,C=14)
	CT(14142)=(S=5896528,E=5896543,C=44)
	CT(14143)=(S=5896544,E=5896559,C=57)
	CT(14144)=(S=5896560,E=5896623,C=18)
	CT(14145)=(S=5896624,E=5896631,C=15)
	CT(14146)=(S=5896632,E=5896639,C=57)
	CT(14147)=(S=5896640,E=5896655,C=66)
	CT(14148)=(S=5896656,E=5896671,C=39)
	CT(14149)=(S=5896672,E=5896735,C=18)
	CT(14150)=(S=5896736,E=5896751,C=14)
	CT(14151)=(S=5896752,E=5896767,C=47)
	CT(14152)=(S=5896768,E=5896783,C=36)
	CT(14153)=(S=5896784,E=5896799,C=20)
	CT(14154)=(S=5896800,E=5896815,C=33)
	CT(14155)=(S=5896816,E=5896831,C=27)
	CT(14156)=(S=5896832,E=5896847,C=28)
	CT(14157)=(S=5896848,E=5896879,C=18)
	CT(14158)=(S=5896880,E=5896895,C=36)
	CT(14159)=(S=5896896,E=5896911,C=58)
	CT(14160)=(S=5896912,E=5896943,C=18)
	CT(14161)=(S=5896944,E=5896959,C=33)
	CT(14162)=(S=5896960,E=5896975,C=36)
	CT(14163)=(S=5896976,E=5896991,C=31)
	CT(14164)=(S=5896992,E=5897007,C=50)
	CT(14165)=(S=5897008,E=5897023,C=12)
	CT(14166)=(S=5897024,E=5897039,C=18)
	CT(14167)=(S=5897040,E=5897055,C=101)
	CT(14168)=(S=5897056,E=5897071,C=18)
	CT(14169)=(S=5897072,E=5897087,C=14)
	CT(14170)=(S=5897088,E=5897103,C=28)
	CT(14171)=(S=5897104,E=5897135,C=18)
	CT(14172)=(S=5897136,E=5897151,C=15)
	CT(14173)=(S=5897152,E=5897183,C=14)
	CT(14174)=(S=5897184,E=5897199,C=25)
	CT(14175)=(S=5897200,E=5897215,C=34)
	CT(14176)=(S=5897216,E=5897279,C=31)
	CT(14177)=(S=5897280,E=5897343,C=18)
	CT(14178)=(S=5897344,E=5897407,C=37)
	CT(14179)=(S=5897408,E=5897471,C=51)
	CT(14180)=(S=5897472,E=5897535,C=18)
	CT(14181)=(S=5897536,E=5897599,C=19)
	CT(14182)=(S=5897600,E=5897663,C=51)
	CT(14183)=(S=5897664,E=5897727,C=18)
	CT(14184)=(S=5897728,E=5897855,C=27)
	CT(14185)=(S=5897856,E=5897919,C=60)
	CT(14186)=(S=5897920,E=5897983,C=18)
	CT(14187)=(S=5897984,E=5898047,C=36)
	CT(14188)=(S=5898048,E=5898111,C=18)
	CT(14189)=(S=5898112,E=5898175,C=14)
	CT(14190)=(S=5898176,E=5898183,C=34)
	CT(14191)=(S=5898184,E=5898207,C=36)
	CT(14192)=(S=5898208,E=5898231,C=19)
	CT(14193)=(S=5898232,E=5898239,C=36)
	CT(14194)=(S=5898240,E=5931007,C=12)
	CT(14195)=(S=5931008,E=5935359,C=19)
	CT(14196)=(S=5935360,E=5935615,C=36)
	CT(14197)=(S=5935616,E=5935871,C=42)
	CT(14198)=(S=5935872,E=5936127,C=15)
	CT(14199)=(S=5936128,E=5936383,C=23)
	CT(14200)=(S=5936384,E=5936639,C=27)
	CT(14201)=(S=5936640,E=5937151,C=18)
	CT(14202)=(S=5937152,E=5937279,C=14)
	CT(14203)=(S=5937280,E=5937407,C=42)
	CT(14204)=(S=5937408,E=5937535,C=28)
	CT(14205)=(S=5937536,E=5937663,C=30)
	CT(14206)=(S=5937664,E=5937791,C=18)
	CT(14207)=(S=5937792,E=5937919,C=51)
	CT(14208)=(S=5937920,E=5938047,C=14)
	CT(14209)=(S=5938048,E=5938175,C=18)
	CT(14210)=(S=5938176,E=5938303,C=58)
	CT(14211)=(S=5938304,E=5938559,C=18)
	CT(14212)=(S=5938560,E=5938687,C=52)
	CT(14213)=(S=5938688,E=5939199,C=37)
	CT(14214)=(S=5939200,E=5943295,C=25)
	CT(14215)=(S=5943296,E=5945343,C=33)
	CT(14216)=(S=5945344,E=5945855,C=24)
	CT(14217)=(S=5945856,E=5946367,C=28)
	CT(14218)=(S=5946368,E=5946879,C=18)
	CT(14219)=(S=5946880,E=5947391,C=55)
	CT(14220)=(S=5947392,E=5955583,C=14)
	CT(14221)=(S=5955584,E=5959679,C=19)
	CT(14222)=(S=5959680,E=5963775,C=14)
	CT(14223)=(S=5963776,E=5981183,C=28)
	CT(14224)=(S=5981184,E=5982207,C=12)
	CT(14225)=(S=5982208,E=5983231,C=16)
	CT(14226)=(S=5983232,E=5984255,C=18)
	CT(14227)=(S=5984256,E=5984767,C=15)
	CT(14228)=(S=5984768,E=5985279,C=50)
	CT(14229)=(S=5985280,E=5985791,C=14)
	CT(14230)=(S=5985792,E=5986303,C=49)
	CT(14231)=(S=5986304,E=5986559,C=12)
	CT(14232)=(S=5986560,E=5986815,C=28)
	CT(14233)=(S=5986816,E=5986823,C=25)
	CT(14234)=(S=5986824,E=5986839,C=31)
	CT(14235)=(S=5986840,E=5986847,C=19)
	CT(14236)=(S=5986848,E=5986855,C=18)
	CT(14237)=(S=5986856,E=5986863,C=27)
	CT(14238)=(S=5986864,E=5986871,C=47)
	CT(14239)=(S=5986872,E=5986879,C=58)
	CT(14240)=(S=5986880,E=5986887,C=27)
	CT(14241)=(S=5986888,E=5986911,C=58)
	CT(14242)=(S=5986912,E=5986919,C=12)
	CT(14243)=(S=5986920,E=5986927,C=27)
	CT(14244)=(S=5986928,E=5986935,C=58)
	CT(14245)=(S=5986936,E=5986943,C=38)
	CT(14246)=(S=5986944,E=5986959,C=17)
	CT(14247)=(S=5986960,E=5986975,C=28)
	CT(14248)=(S=5986976,E=5987007,C=58)
	CT(14249)=(S=5987008,E=5987039,C=31)
	CT(14250)=(S=5987040,E=5987071,C=60)
	CT(14251)=(S=5987072,E=5987327,C=12)
	CT(14252)=(S=5987328,E=5987583,C=51)
	CT(14253)=(S=5987584,E=5987839,C=37)
	CT(14254)=(S=5987840,E=5988095,C=58)
	CT(14255)=(S=5988096,E=5988351,C=19)
	CT(14256)=(S=5988352,E=5988863,C=28)
	CT(14257)=(S=5988864,E=5989375,C=26)
	CT(14258)=(S=5989376,E=5989887,C=24)
	CT(14259)=(S=5989888,E=5989895,C=14)
	CT(14260)=(S=5989896,E=5989903,C=28)
	CT(14261)=(S=5989904,E=5989911,C=23)
	CT(14262)=(S=5989912,E=5989919,C=27)
	CT(14263)=(S=5989920,E=5989927,C=101)
	CT(14264)=(S=5989928,E=5989935,C=36)
	CT(14265)=(S=5989936,E=5989943,C=15)
	CT(14266)=(S=5989944,E=5989959,C=14)
	CT(14267)=(S=5989960,E=5989967,C=18)
	CT(14268)=(S=5989968,E=5989975,C=63)
	CT(14269)=(S=5989976,E=5989983,C=24)
	CT(14270)=(S=5989984,E=5989991,C=33)
	CT(14271)=(S=5989992,E=5990007,C=58)
	CT(14272)=(S=5990008,E=5990015,C=16)
	CT(14273)=(S=5990016,E=5990023,C=15)
	CT(14274)=(S=5990024,E=5990031,C=28)
	CT(14275)=(S=5990032,E=5990039,C=101)
	CT(14276)=(S=5990040,E=5990047,C=18)
	CT(14277)=(S=5990048,E=5990055,C=37)
	CT(14278)=(S=5990056,E=5990063,C=46)
	CT(14279)=(S=5990064,E=5990071,C=31)
	CT(14280)=(S=5990072,E=5990079,C=14)
	CT(14281)=(S=5990080,E=5990087,C=34)
	CT(14282)=(S=5990088,E=5990095,C=18)
	CT(14283)=(S=5990096,E=5990103,C=36)
	CT(14284)=(S=5990104,E=5990111,C=18)
	CT(14285)=(S=5990112,E=5990119,C=46)
	CT(14286)=(S=5990120,E=5990127,C=101)
	CT(14287)=(S=5990128,E=5990135,C=25)
	CT(14288)=(S=5990136,E=5990143,C=12)
	CT(14289)=(S=5990144,E=5990151,C=63)
	CT(14290)=(S=5990160,E=5990167,C=36)
	CT(14291)=(S=5990168,E=5990175,C=66)
	CT(14292)=(S=5990176,E=5990183,C=14)
	CT(14293)=(S=5990184,E=5990191,C=12)
	CT(14294)=(S=5990192,E=5990199,C=25)
	CT(14295)=(S=5990200,E=5990207,C=66)
	CT(14296)=(S=5990208,E=5990231,C=18)
	CT(14297)=(S=5990232,E=5990239,C=72)
	CT(14298)=(S=5990240,E=5990247,C=44)
	CT(14299)=(S=5990248,E=5990255,C=18)
	CT(14300)=(S=5990256,E=5990263,C=28)
	CT(14301)=(S=5990264,E=5990271,C=31)
	CT(14302)=(S=5990272,E=5990279,C=14)
	CT(14303)=(S=5990280,E=5990287,C=12)
	CT(14304)=(S=5990296,E=5990303,C=18)
	CT(14305)=(S=5990304,E=5990311,C=33)
	CT(14306)=(S=5990312,E=5990335,C=14)
	CT(14307)=(S=5990336,E=5990359,C=18)
	CT(14308)=(S=5990360,E=5990367,C=14)
	CT(14309)=(S=5990368,E=5990375,C=18)
	CT(14310)=(S=5990376,E=5990383,C=12)
	CT(14311)=(S=5990384,E=5990391,C=28)
	CT(14312)=(S=5990392,E=5990399,C=42)
	CT(14313)=(S=5990400,E=5990655,C=50)
	CT(14314)=(S=5990656,E=5990783,C=60)
	CT(14315)=(S=5990784,E=5990855,C=18)
	CT(14316)=(S=5990856,E=5990863,C=60)
	CT(14317)=(S=5990864,E=5990871,C=19)
	CT(14318)=(S=5990872,E=5990879,C=36)
	CT(14319)=(S=5990880,E=5990887,C=14)
	CT(14320)=(S=5990888,E=5990895,C=18)
	CT(14321)=(S=5990896,E=5990903,C=28)
	CT(14322)=(S=5990904,E=5990911,C=47)
	CT(14323)=(S=5990912,E=5990927,C=28)
	CT(14324)=(S=5990928,E=5990935,C=25)
	CT(14325)=(S=5990936,E=5990943,C=58)
	CT(14326)=(S=5990944,E=5990975,C=35)
	CT(14327)=(S=5990976,E=5991007,C=26)
	CT(14328)=(S=5991008,E=5991023,C=61)
	CT(14329)=(S=5991024,E=5991031,C=18)
	CT(14330)=(S=5991032,E=5991103,C=28)
	CT(14331)=(S=5991104,E=5991111,C=19)
	CT(14332)=(S=5991112,E=5991119,C=18)
	CT(14333)=(S=5991120,E=5991127,C=32)
	CT(14334)=(S=5991128,E=5991135,C=12)
	CT(14335)=(S=5991136,E=5991143,C=36)
	CT(14336)=(S=5991144,E=5991151,C=18)
	CT(14337)=(S=5991152,E=5991167,C=33)
	CT(14338)=(S=5991168,E=5991295,C=18)
	CT(14339)=(S=5991296,E=5991423,C=26)
	CT(14340)=(S=5991424,E=5991487,C=18)
	CT(14341)=(S=5991488,E=5991551,C=28)
	CT(14342)=(S=5991552,E=5991583,C=26)
	CT(14343)=(S=5991584,E=5991591,C=14)
	CT(14344)=(S=5991592,E=5991599,C=18)
	CT(14345)=(S=5991600,E=5991607,C=55)
	CT(14346)=(S=5991608,E=5991615,C=34)
	CT(14347)=(S=5991616,E=5991679,C=35)
	CT(14348)=(S=5991680,E=5991695,C=14)
	CT(14349)=(S=5991696,E=5991711,C=28)
	CT(14350)=(S=5991712,E=5991719,C=33)
	CT(14351)=(S=5991720,E=5991727,C=14)
	CT(14352)=(S=5991728,E=5991743,C=15)
	CT(14353)=(S=5991744,E=5991775,C=18)
	CT(14354)=(S=5991776,E=5991783,C=25)
	CT(14355)=(S=5991784,E=5991791,C=21)
	CT(14356)=(S=5991792,E=5991799,C=29)
	CT(14357)=(S=5991800,E=5991807,C=12)
	CT(14358)=(S=5991808,E=5991839,C=18)
	CT(14359)=(S=5991840,E=5991847,C=25)
	CT(14360)=(S=5991848,E=5991855,C=15)
	CT(14361)=(S=5991856,E=5991871,C=12)
	CT(14362)=(S=5991872,E=5991879,C=15)
	CT(14363)=(S=5991880,E=5991887,C=18)
	CT(14364)=(S=5991888,E=5991895,C=37)
	CT(14365)=(S=5991896,E=5991903,C=89)
	CT(14366)=(S=5991904,E=5991911,C=18)
	CT(14367)=(S=5991912,E=5991927,C=14)
	CT(14368)=(S=5991928,E=5991935,C=69)
	CT(14369)=(S=5991936,E=5992447,C=14)
	CT(14370)=(S=5992448,E=5993471,C=42)
	CT(14371)=(S=5993472,E=5993983,C=25)
	CT(14372)=(S=5993984,E=5994495,C=42)
	CT(14373)=(S=5994496,E=5994751,C=50)
	CT(14374)=(S=5994752,E=5995007,C=12)
	CT(14375)=(S=5995008,E=5995263,C=18)
	CT(14376)=(S=5995264,E=5995279,C=38)
	CT(14377)=(S=5995280,E=5995295,C=18)
	CT(14378)=(S=5995296,E=5995327,C=27)
	CT(14379)=(S=5995328,E=5995343,C=60)
	CT(14380)=(S=5995344,E=5995359,C=18)
	CT(14381)=(S=5995360,E=5995375,C=34)
	CT(14382)=(S=5995376,E=5995391,C=12)
	CT(14383)=(S=5995392,E=5995407,C=47)
	CT(14384)=(S=5995408,E=5995423,C=31)
	CT(14385)=(S=5995424,E=5995455,C=58)
	CT(14386)=(S=5995456,E=5995471,C=19)
	CT(14387)=(S=5995472,E=5995487,C=58)
	CT(14388)=(S=5995488,E=5995503,C=63)
	CT(14389)=(S=5995504,E=5995519,C=36)
	CT(14390)=(S=5995520,E=5995775,C=31)
	CT(14391)=(S=5995776,E=5996031,C=14)
	CT(14392)=(S=5996032,E=5996047,C=27)
	CT(14393)=(S=5996048,E=5996287,C=25)
	CT(14394)=(S=5996288,E=5996543,C=71)
	CT(14395)=(S=5996544,E=5997567,C=42)
	CT(14396)=(S=5997568,E=5997823,C=22)
	CT(14397)=(S=5997824,E=5997855,C=18)
	CT(14398)=(S=5997856,E=5997887,C=24)
	CT(14399)=(S=5997888,E=5997951,C=42)
	CT(14400)=(S=5997952,E=5998079,C=26)
	CT(14401)=(S=5998080,E=5998335,C=51)
	CT(14402)=(S=5998336,E=5998351,C=14)
	CT(14403)=(S=5998352,E=5998367,C=60)
	CT(14404)=(S=5998368,E=5998399,C=27)
	CT(14405)=(S=5998400,E=5998415,C=34)
	CT(14406)=(S=5998416,E=5998431,C=60)
	CT(14407)=(S=5998432,E=5998447,C=17)
	CT(14408)=(S=5998448,E=5998463,C=50)
	CT(14409)=(S=5998464,E=5998479,C=27)
	CT(14410)=(S=5998480,E=5998495,C=18)
	CT(14411)=(S=5998496,E=5998511,C=42)
	CT(14412)=(S=5998512,E=5998527,C=12)
	CT(14413)=(S=5998528,E=5998543,C=20)
	CT(14414)=(S=5998544,E=5998559,C=18)
	CT(14415)=(S=5998560,E=5998575,C=14)
	CT(14416)=(S=5998576,E=5998591,C=41)
	CT(14417)=(S=5998592,E=5998719,C=14)
	CT(14418)=(S=5998720,E=5998975,C=28)
	CT(14419)=(S=5998976,E=5999103,C=50)
	CT(14420)=(S=5999104,E=5999231,C=34)
	CT(14421)=(S=5999232,E=5999359,C=22)
	CT(14422)=(S=5999360,E=5999487,C=33)
	CT(14423)=(S=5999488,E=5999615,C=51)
	CT(14424)=(S=5999616,E=5999743,C=22)
	CT(14425)=(S=5999744,E=5999871,C=101)
	CT(14426)=(S=5999872,E=5999999,C=42)
	CT(14427)=(S=6000000,E=6000127,C=36)
	CT(14428)=(S=6000128,E=6000143,C=60)
	CT(14429)=(S=6000144,E=6000159,C=42)
	CT(14430)=(S=6000160,E=6000175,C=14)
	CT(14431)=(S=6000176,E=6000191,C=28)
	CT(14432)=(S=6000192,E=6000199,C=15)
	CT(14433)=(S=6000200,E=6000223,C=18)
	CT(14434)=(S=6000224,E=6000239,C=63)
	CT(14435)=(S=6000240,E=6000255,C=19)
	CT(14436)=(S=6000256,E=6000256,C=14)
	CT(14437)=(S=6000257,E=6000258,C=70)
	CT(14438)=(S=6000259,E=6000260,C=36)
	CT(14439)=(S=6000262,E=6000262,C=14)
	CT(14440)=(S=6000263,E=6000263,C=18)
	CT(14441)=(S=6000264,E=6000264,C=24)
	CT(14442)=(S=6000265,E=6000265,C=58)
	CT(14443)=(S=6000266,E=6000266,C=18)
	CT(14444)=(S=6000267,E=6000267,C=57)
	CT(14445)=(S=6000269,E=6000269,C=18)
	CT(14446)=(S=6000270,E=6000270,C=37)
	CT(14447)=(S=6000271,E=6000271,C=15)
	CT(14448)=(S=6000272,E=6000287,C=18)
	CT(14449)=(S=6000288,E=6000303,C=31)
	CT(14450)=(S=6000304,E=6000319,C=19)
	CT(14451)=(S=6000320,E=6000335,C=58)
	CT(14452)=(S=6000336,E=6000351,C=25)
	CT(14453)=(S=6000352,E=6000367,C=63)
	CT(14454)=(S=6000368,E=6000383,C=36)
	CT(14455)=(S=6000384,E=6000447,C=18)
	CT(14456)=(S=6000448,E=6000463,C=14)
	CT(14457)=(S=6000464,E=6000479,C=28)
	CT(14458)=(S=6000480,E=6000495,C=42)
	CT(14459)=(S=6000496,E=6000511,C=24)
	CT(14460)=(S=6000512,E=6000559,C=18)
	CT(14461)=(S=6000560,E=6000575,C=14)
	CT(14462)=(S=6000576,E=6000591,C=15)
	CT(14463)=(S=6000592,E=6000607,C=46)
	CT(14464)=(S=6000608,E=6000623,C=17)
	CT(14465)=(S=6000624,E=6000639,C=28)
	CT(14466)=(S=6000640,E=6000703,C=30)
	CT(14467)=(S=6000704,E=6000767,C=50)
	CT(14468)=(S=6000768,E=6000831,C=18)
	CT(14469)=(S=6000832,E=6000895,C=24)
	CT(14470)=(S=6000896,E=6000959,C=19)
	CT(14471)=(S=6000960,E=6001023,C=47)
	CT(14472)=(S=6001024,E=6001087,C=58)
	CT(14473)=(S=6001088,E=6001151,C=31)
	CT(14474)=(S=6001152,E=6001215,C=18)
	CT(14475)=(S=6001216,E=6001247,C=55)
	CT(14476)=(S=6001248,E=6001255,C=25)
	CT(14477)=(S=6001256,E=6001271,C=14)
	CT(14478)=(S=6001272,E=6001279,C=33)
	CT(14479)=(S=6001280,E=6001343,C=50)
	CT(14480)=(S=6001344,E=6001407,C=58)
	CT(14481)=(S=6001408,E=6001471,C=18)
	CT(14482)=(S=6001472,E=6001535,C=20)
	CT(14483)=(S=6001536,E=6001599,C=23)
	CT(14484)=(S=6001600,E=6001663,C=50)
	CT(14485)=(S=6001664,E=6001727,C=71)
	CT(14486)=(S=6001728,E=6001791,C=46)
	CT(14487)=(S=6001792,E=6001855,C=51)
	CT(14488)=(S=6001856,E=6001919,C=36)
	CT(14489)=(S=6001920,E=6001983,C=19)
	CT(14490)=(S=6001984,E=6002047,C=18)
	CT(14491)=(S=6002048,E=6002111,C=68)
	CT(14492)=(S=6002112,E=6002175,C=58)
	CT(14493)=(S=6002176,E=6002239,C=47)
	CT(14494)=(S=6002240,E=6002303,C=46)
	CT(14495)=(S=6002304,E=6002399,C=58)
	CT(14496)=(S=6002400,E=6002431,C=24)
	CT(14497)=(S=6002432,E=6002447,C=14)
	CT(14498)=(S=6002448,E=6002463,C=28)
	CT(14499)=(S=6002464,E=6002479,C=18)
	CT(14500)=(S=6002480,E=6002511,C=12)
	CT(14501)=(S=6002512,E=6002527,C=37)
	CT(14502)=(S=6002528,E=6002543,C=25)
	CT(14503)=(S=6002544,E=6002559,C=12)
	CT(14504)=(S=6002560,E=6002575,C=40)
	CT(14505)=(S=6002576,E=6002591,C=28)
	CT(14506)=(S=6002592,E=6002607,C=23)
	CT(14507)=(S=6002608,E=6002639,C=18)
	CT(14508)=(S=6002640,E=6002655,C=14)
	CT(14509)=(S=6002656,E=6002671,C=39)
	CT(14510)=(S=6002672,E=6002687,C=14)
	CT(14511)=(S=6002688,E=6004735,C=47)
	CT(14512)=(S=6004736,E=6008831,C=12)
	CT(14513)=(S=6008832,E=6010879,C=49)
	CT(14514)=(S=6010880,E=6010911,C=36)
	CT(14515)=(S=6010912,E=6010943,C=28)
	CT(14516)=(S=6010944,E=6010975,C=26)
	CT(14517)=(S=6010976,E=6011007,C=40)
	CT(14518)=(S=6011008,E=6011039,C=27)
	CT(14519)=(S=6011040,E=6011071,C=28)
	CT(14520)=(S=6011072,E=6011103,C=32)
	CT(14521)=(S=6011104,E=6011135,C=18)
	CT(14522)=(S=6011136,E=6011167,C=20)
	CT(14523)=(S=6011168,E=6011231,C=18)
	CT(14524)=(S=6011232,E=6011263,C=46)
	CT(14525)=(S=6011264,E=6011295,C=26)
	CT(14526)=(S=6011296,E=6011319,C=14)
	CT(14527)=(S=6011320,E=6011327,C=58)
	CT(14528)=(S=6011328,E=6011359,C=52)
	CT(14529)=(S=6011360,E=6011391,C=18)
	CT(14530)=(S=6011392,E=6011423,C=14)
	CT(14531)=(S=6011424,E=6011455,C=28)
	CT(14532)=(S=6011456,E=6011487,C=27)
	CT(14533)=(S=6011488,E=6011519,C=18)
	CT(14534)=(S=6011520,E=6011551,C=42)
	CT(14535)=(S=6011552,E=6011583,C=28)
	CT(14536)=(S=6011584,E=6011615,C=26)
	CT(14537)=(S=6011616,E=6011647,C=61)
	CT(14538)=(S=6011648,E=6011679,C=68)
	CT(14539)=(S=6011680,E=6011711,C=33)
	CT(14540)=(S=6011712,E=6011743,C=188)
	CT(14541)=(S=6011744,E=6011775,C=96)
	CT(14542)=(S=6011776,E=6011807,C=46)
	CT(14543)=(S=6011808,E=6011839,C=44)
	CT(14544)=(S=6011840,E=6011871,C=15)
	CT(14545)=(S=6011872,E=6011903,C=58)
	CT(14546)=(S=6011904,E=6011935,C=15)
	CT(14547)=(S=6011936,E=6011967,C=60)
	CT(14548)=(S=6011968,E=6011999,C=12)
	CT(14549)=(S=6012000,E=6012031,C=58)
	CT(14550)=(S=6012032,E=6012063,C=98)
	CT(14551)=(S=6012064,E=6012095,C=18)
	CT(14552)=(S=6012160,E=6012167,C=58)
	CT(14553)=(S=6012168,E=6012175,C=34)
	CT(14554)=(S=6012176,E=6012199,C=58)
	CT(14555)=(S=6012200,E=6012207,C=19)
	CT(14556)=(S=6012208,E=6012215,C=18)
	CT(14557)=(S=6012216,E=6012223,C=58)
	CT(14558)=(S=6012224,E=6012231,C=63)
	CT(14559)=(S=6012232,E=6012239,C=58)
	CT(14560)=(S=6012240,E=6012247,C=18)
	CT(14561)=(S=6012248,E=6012255,C=14)
	CT(14562)=(S=6012256,E=6012263,C=36)
	CT(14563)=(S=6012264,E=6012271,C=12)
	CT(14564)=(S=6012272,E=6012279,C=18)
	CT(14565)=(S=6012280,E=6012287,C=27)
	CT(14566)=(S=6012288,E=6012295,C=31)
	CT(14567)=(S=6012296,E=6012303,C=58)
	CT(14568)=(S=6012304,E=6012311,C=14)
	CT(14569)=(S=6012312,E=6012319,C=31)
	CT(14570)=(S=6012320,E=6012327,C=18)
	CT(14571)=(S=6012328,E=6012343,C=27)
	CT(14572)=(S=6012352,E=6012359,C=28)
	CT(14573)=(S=6012360,E=6012367,C=24)
	CT(14574)=(S=6012368,E=6012375,C=47)
	CT(14575)=(S=6012376,E=6012383,C=58)
	CT(14576)=(S=6012384,E=6012391,C=36)
	CT(14577)=(S=6012392,E=6012407,C=18)
	CT(14578)=(S=6012408,E=6012415,C=58)
	CT(14579)=(S=6012416,E=6012447,C=34)
	CT(14580)=(S=6012448,E=6012479,C=60)
	CT(14581)=(S=6012480,E=6012511,C=18)
	CT(14582)=(S=6012512,E=6012527,C=36)
	CT(14583)=(S=6012528,E=6012543,C=18)
	CT(14584)=(S=6012544,E=6012551,C=33)
	CT(14585)=(S=6012552,E=6012559,C=19)
	CT(14586)=(S=6012560,E=6012575,C=28)
	CT(14587)=(S=6012576,E=6012583,C=14)
	CT(14588)=(S=6012584,E=6012591,C=12)
	CT(14589)=(S=6012592,E=6012599,C=18)
	CT(14590)=(S=6012600,E=6012607,C=50)
	CT(14591)=(S=6012608,E=6012615,C=47)
	CT(14592)=(S=6012616,E=6012623,C=28)
	CT(14593)=(S=6012624,E=6012631,C=49)
	CT(14594)=(S=6012632,E=6012639,C=70)
	CT(14595)=(S=6012640,E=6012647,C=28)
	CT(14596)=(S=6012648,E=6012655,C=18)
	CT(14597)=(S=6012656,E=6012671,C=28)
	CT(14598)=(S=6012672,E=6012735,C=59)
	CT(14599)=(S=6012736,E=6012799,C=71)
	CT(14600)=(S=6012800,E=6012815,C=19)
	CT(14601)=(S=6012816,E=6012831,C=12)
	CT(14602)=(S=6012832,E=6012847,C=37)
	CT(14603)=(S=6012848,E=6012863,C=18)
	CT(14604)=(S=6012864,E=6012879,C=41)
	CT(14605)=(S=6012880,E=6012895,C=51)
	CT(14606)=(S=6012896,E=6012927,C=18)
	CT(14607)=(S=6012928,E=6012931,C=58)
	CT(14608)=(S=6012940,E=6012947,C=28)
	CT(14609)=(S=6012948,E=6012951,C=18)
	CT(14610)=(S=6012956,E=6012959,C=19)
	CT(14611)=(S=6012960,E=6012963,C=33)
	CT(14612)=(S=6012964,E=6012967,C=36)
	CT(14613)=(S=6012968,E=6012971,C=28)
	CT(14614)=(S=6012972,E=6012975,C=31)
	CT(14615)=(S=6012976,E=6012979,C=106)
	CT(14616)=(S=6012980,E=6012983,C=24)
	CT(14617)=(S=6012984,E=6012987,C=58)
	CT(14618)=(S=6012988,E=6012991,C=18)
	CT(14619)=(S=6012992,E=6012995,C=62)
	CT(14620)=(S=6012996,E=6013003,C=18)
	CT(14621)=(S=6013004,E=6013007,C=58)
	CT(14622)=(S=6013008,E=6013011,C=18)
	CT(14623)=(S=6013012,E=6013015,C=31)
	CT(14624)=(S=6013016,E=6013019,C=58)
	CT(14625)=(S=6013020,E=6013027,C=18)
	CT(14626)=(S=6013028,E=6013031,C=34)
	CT(14627)=(S=6013032,E=6013033,C=31)
	CT(14628)=(S=6013036,E=6013039,C=25)
	CT(14629)=(S=6013040,E=6013043,C=19)
	CT(14630)=(S=6013044,E=6013047,C=36)
	CT(14631)=(S=6013048,E=6013051,C=58)
	CT(14632)=(S=6013052,E=6013055,C=15)
	CT(14633)=(S=6013056,E=6013067,C=31)
	CT(14634)=(S=6013068,E=6013071,C=38)
	CT(14635)=(S=6013072,E=6013075,C=58)
	CT(14636)=(S=6013076,E=6013079,C=18)
	CT(14637)=(S=6013080,E=6013091,C=31)
	CT(14638)=(S=6013092,E=6013095,C=58)
	CT(14639)=(S=6013096,E=6013103,C=18)
	CT(14640)=(S=6013108,E=6013115,C=31)
	CT(14641)=(S=6013116,E=6013119,C=18)
	CT(14642)=(S=6013120,E=6013123,C=14)
	CT(14643)=(S=6013124,E=6013127,C=58)
	CT(14644)=(S=6013128,E=6013129,C=31)
	CT(14645)=(S=6013130,E=6013131,C=28)
	CT(14646)=(S=6013132,E=6013135,C=58)
	CT(14647)=(S=6013136,E=6013139,C=14)
	CT(14648)=(S=6013140,E=6013143,C=24)
	CT(14649)=(S=6013144,E=6013147,C=31)
	CT(14650)=(S=6013148,E=6013151,C=27)
	CT(14651)=(S=6013152,E=6013159,C=58)
	CT(14652)=(S=6013160,E=6013163,C=14)
	CT(14653)=(S=6013164,E=6013167,C=51)
	CT(14654)=(S=6013168,E=6013175,C=18)
	CT(14655)=(S=6013176,E=6013179,C=58)
	CT(14656)=(S=6013180,E=6013183,C=19)
	CT(14657)=(S=6013184,E=6013187,C=27)
	CT(14658)=(S=6013192,E=6013195,C=31)
	CT(14659)=(S=6013196,E=6013199,C=58)
	CT(14660)=(S=6013204,E=6013207,C=34)
	CT(14661)=(S=6013208,E=6013211,C=46)
	CT(14662)=(S=6013212,E=6013215,C=38)
	CT(14663)=(S=6013216,E=6013219,C=31)
	CT(14664)=(S=6013220,E=6013223,C=18)
	CT(14665)=(S=6013224,E=6013225,C=58)
	CT(14666)=(S=6013226,E=6013227,C=49)
	CT(14667)=(S=6013228,E=6013231,C=15)
	CT(14668)=(S=6013232,E=6013235,C=17)
	CT(14669)=(S=6013236,E=6013239,C=58)
	CT(14670)=(S=6013240,E=6013241,C=12)
	CT(14671)=(S=6013242,E=6013243,C=18)
	CT(14672)=(S=6013248,E=6013251,C=60)
	CT(14673)=(S=6013252,E=6013255,C=31)
	CT(14674)=(S=6013256,E=6013256,C=28)
	CT(14675)=(S=6013257,E=6013257,C=58)
	CT(14676)=(S=6013258,E=6013258,C=18)
	CT(14677)=(S=6013259,E=6013259,C=46)
	CT(14678)=(S=6013260,E=6013267,C=31)
	CT(14679)=(S=6013268,E=6013271,C=58)
	CT(14680)=(S=6013272,E=6013273,C=18)
	CT(14681)=(S=6013274,E=6013275,C=58)
	CT(14682)=(S=6013276,E=6013279,C=25)
	CT(14683)=(S=6013280,E=6013283,C=58)
	CT(14684)=(S=6013288,E=6013289,C=14)
	CT(14685)=(S=6013290,E=6013291,C=12)
	CT(14686)=(S=6013292,E=6013293,C=14)
	CT(14687)=(S=6013294,E=6013295,C=18)
	CT(14688)=(S=6013304,E=6013307,C=58)
	CT(14689)=(S=6013308,E=6013315,C=31)
	CT(14690)=(S=6013316,E=6013319,C=33)
	CT(14691)=(S=6013320,E=6013323,C=24)
	CT(14692)=(S=6013324,E=6013327,C=18)
	CT(14693)=(S=6013328,E=6013331,C=58)
	CT(14694)=(S=6013332,E=6013335,C=18)
	CT(14695)=(S=6013340,E=6013343,C=51)
	CT(14696)=(S=6013344,E=6013347,C=58)
	CT(14697)=(S=6013348,E=6013351,C=31)
	CT(14698)=(S=6013352,E=6013355,C=18)
	CT(14699)=(S=6013356,E=6013359,C=31)
	CT(14700)=(S=6013360,E=6013363,C=18)
	CT(14701)=(S=6013364,E=6013367,C=14)
	CT(14702)=(S=6013368,E=6013371,C=58)
	CT(14703)=(S=6013372,E=6013375,C=63)
	CT(14704)=(S=6013380,E=6013383,C=58)
	CT(14705)=(S=6013384,E=6013387,C=51)
	CT(14706)=(S=6013388,E=6013391,C=31)
	CT(14707)=(S=6013392,E=6013395,C=58)
	CT(14708)=(S=6013396,E=6013415,C=18)
	CT(14709)=(S=6013416,E=6013419,C=31)
	CT(14710)=(S=6013420,E=6013427,C=18)
	CT(14711)=(S=6013428,E=6013431,C=14)
	CT(14712)=(S=6013436,E=6013439,C=31)
	CT(14713)=(S=6013442,E=6013443,C=18)
	CT(14714)=(S=6013444,E=6013445,C=17)
	CT(14715)=(S=6013446,E=6013447,C=28)
	CT(14716)=(S=6013448,E=6013449,C=36)
	CT(14717)=(S=6013450,E=6013457,C=18)
	CT(14718)=(S=6013458,E=6013459,C=58)
	CT(14719)=(S=6013460,E=6013461,C=47)
	CT(14720)=(S=6013462,E=6013467,C=19)
	CT(14721)=(S=6013468,E=6013471,C=38)
	CT(14722)=(S=6013472,E=6013473,C=42)
	CT(14723)=(S=6013474,E=6013475,C=31)
	CT(14724)=(S=6013476,E=6013477,C=24)
	CT(14725)=(S=6013478,E=6013479,C=19)
	CT(14726)=(S=6013480,E=6013481,C=31)
	CT(14727)=(S=6013482,E=6013485,C=38)
	CT(14728)=(S=6013486,E=6013487,C=14)
	CT(14729)=(S=6013488,E=6013489,C=34)
	CT(14730)=(S=6013490,E=6013497,C=31)
	CT(14731)=(S=6013498,E=6013499,C=18)
	CT(14732)=(S=6013500,E=6013501,C=12)
	CT(14733)=(S=6013502,E=6013503,C=25)
	CT(14734)=(S=6013504,E=6013505,C=18)
	CT(14735)=(S=6013506,E=6013507,C=19)
	CT(14736)=(S=6013508,E=6013511,C=14)
	CT(14737)=(S=6013512,E=6013513,C=31)
	CT(14738)=(S=6013514,E=6013515,C=14)
	CT(14739)=(S=6013516,E=6013517,C=60)
	CT(14740)=(S=6013518,E=6013521,C=31)
	CT(14741)=(S=6013522,E=6013523,C=46)
	CT(14742)=(S=6013524,E=6013525,C=28)
	CT(14743)=(S=6013526,E=6013527,C=49)
	CT(14744)=(S=6013528,E=6013529,C=31)
	CT(14745)=(S=6013530,E=6013531,C=28)
	CT(14746)=(S=6013532,E=6013533,C=38)
	CT(14747)=(S=6013534,E=6013535,C=24)
	CT(14748)=(S=6013536,E=6013541,C=12)
	CT(14749)=(S=6013542,E=6013543,C=101)
	CT(14750)=(S=6013546,E=6013547,C=58)
	CT(14751)=(S=6013548,E=6013549,C=28)
	CT(14752)=(S=6013550,E=6013553,C=18)
	CT(14753)=(S=6013554,E=6013555,C=17)
	CT(14754)=(S=6013556,E=6013557,C=14)
	CT(14755)=(S=6013560,E=6013561,C=18)
	CT(14756)=(S=6013562,E=6013563,C=58)
	CT(14757)=(S=6013566,E=6013567,C=14)
	CT(14758)=(S=6013568,E=6013569,C=12)
	CT(14759)=(S=6013570,E=6013571,C=101)
	CT(14760)=(S=6013572,E=6013573,C=28)
	CT(14761)=(S=6013574,E=6013575,C=31)
	CT(14762)=(S=6013576,E=6013577,C=36)
	CT(14763)=(S=6013578,E=6013579,C=58)
	CT(14764)=(S=6013582,E=6013583,C=15)
	CT(14765)=(S=6013584,E=6013585,C=58)
	CT(14766)=(S=6013586,E=6013587,C=34)
	CT(14767)=(S=6013588,E=6013589,C=38)
	CT(14768)=(S=6013590,E=6013593,C=14)
	CT(14769)=(S=6013594,E=6013595,C=24)
	CT(14770)=(S=6013596,E=6013597,C=58)
	CT(14771)=(S=6013598,E=6013599,C=14)
	CT(14772)=(S=6013600,E=6013601,C=18)
	CT(14773)=(S=6013602,E=6013603,C=31)
	CT(14774)=(S=6013604,E=6013605,C=18)
	CT(14775)=(S=6013608,E=6013609,C=31)
	CT(14776)=(S=6013612,E=6013615,C=18)
	CT(14777)=(S=6013616,E=6013616,C=60)
	CT(14778)=(S=6013617,E=6013617,C=38)
	CT(14779)=(S=6013618,E=6013619,C=12)
	CT(14780)=(S=6013620,E=6013621,C=28)
	CT(14781)=(S=6013624,E=6013625,C=18)
	CT(14782)=(S=6013626,E=6013627,C=19)
	CT(14783)=(S=6013628,E=6013629,C=58)
	CT(14784)=(S=6013630,E=6013631,C=18)
	CT(14785)=(S=6013636,E=6013637,C=34)
	CT(14786)=(S=6013638,E=6013639,C=58)
	CT(14787)=(S=6013640,E=6013641,C=18)
	CT(14788)=(S=6013642,E=6013643,C=49)
	CT(14789)=(S=6013644,E=6013645,C=36)
	CT(14790)=(S=6013646,E=6013647,C=18)
	CT(14791)=(S=6013648,E=6013649,C=12)
	CT(14792)=(S=6013650,E=6013651,C=58)
	CT(14793)=(S=6013652,E=6013653,C=18)
	CT(14794)=(S=6013656,E=6013657,C=46)
	CT(14795)=(S=6013658,E=6013659,C=36)
	CT(14796)=(S=6013660,E=6013661,C=14)
	CT(14797)=(S=6013662,E=6013663,C=28)
	CT(14798)=(S=6013664,E=6013665,C=36)
	CT(14799)=(S=6013666,E=6013667,C=18)
	CT(14800)=(S=6013668,E=6013669,C=58)
	CT(14801)=(S=6013670,E=6013673,C=28)
	CT(14802)=(S=6013674,E=6013675,C=38)
	CT(14803)=(S=6013676,E=6013677,C=49)
	CT(14804)=(S=6013678,E=6013679,C=31)
	CT(14805)=(S=6013682,E=6013683,C=57)
	CT(14806)=(S=6013684,E=6013687,C=18)
	CT(14807)=(S=6013688,E=6013689,C=42)
	CT(14808)=(S=6013690,E=6013691,C=31)
	CT(14809)=(S=6013692,E=6013695,C=18)
	CT(14810)=(S=6013696,E=6013697,C=36)
	CT(14811)=(S=6013698,E=6013699,C=31)
	CT(14812)=(S=6013700,E=6013703,C=38)
	CT(14813)=(S=6013704,E=6013705,C=27)
	CT(14814)=(S=6013706,E=6013709,C=31)
	CT(14815)=(S=6013710,E=6013711,C=38)
	CT(14816)=(S=6013712,E=6013713,C=18)
	CT(14817)=(S=6013714,E=6013715,C=70)
	CT(14818)=(S=6013716,E=6013717,C=31)
	CT(14819)=(S=6013718,E=6013719,C=18)
	CT(14820)=(S=6013720,E=6013721,C=51)
	CT(14821)=(S=6013722,E=6013723,C=58)
	CT(14822)=(S=6013724,E=6013725,C=38)
	CT(14823)=(S=6013726,E=6013727,C=18)
	CT(14824)=(S=6013728,E=6013729,C=19)
	CT(14825)=(S=6013730,E=6013731,C=25)
	CT(14826)=(S=6013732,E=6013733,C=12)
	CT(14827)=(S=6013734,E=6013735,C=46)
	CT(14828)=(S=6013736,E=6013737,C=42)
	CT(14829)=(S=6013738,E=6013739,C=58)
	CT(14830)=(S=6013740,E=6013741,C=38)
	CT(14831)=(S=6013742,E=6013743,C=58)
	CT(14832)=(S=6013744,E=6013745,C=28)
	CT(14833)=(S=6013748,E=6013749,C=31)
	CT(14834)=(S=6013750,E=6013751,C=28)
	CT(14835)=(S=6013752,E=6013753,C=58)
	CT(14836)=(S=6013754,E=6013755,C=24)
	CT(14837)=(S=6013756,E=6013757,C=18)
	CT(14838)=(S=6013758,E=6013759,C=38)
	CT(14839)=(S=6013764,E=6013765,C=31)
	CT(14840)=(S=6013766,E=6013767,C=14)
	CT(14841)=(S=6013768,E=6013769,C=12)
	CT(14842)=(S=6013770,E=6013771,C=31)
	CT(14843)=(S=6013772,E=6013773,C=19)
	CT(14844)=(S=6013774,E=6013775,C=28)
	CT(14845)=(S=6013776,E=6013777,C=36)
	CT(14846)=(S=6013778,E=6013779,C=14)
	CT(14847)=(S=6013780,E=6013781,C=38)
	CT(14848)=(S=6013782,E=6013783,C=31)
	CT(14849)=(S=6013784,E=6013785,C=23)
	CT(14850)=(S=6013786,E=6013787,C=31)
	CT(14851)=(S=6013788,E=6013789,C=58)
	CT(14852)=(S=6013790,E=6013791,C=42)
	CT(14853)=(S=6013792,E=6013793,C=31)
	CT(14854)=(S=6013794,E=6013795,C=38)
	CT(14855)=(S=6013796,E=6013799,C=18)
	CT(14856)=(S=6013800,E=6013801,C=28)
	CT(14857)=(S=6013802,E=6013803,C=33)
	CT(14858)=(S=6013804,E=6013805,C=58)
	CT(14859)=(S=6013806,E=6013807,C=18)
	CT(14860)=(S=6013808,E=6013809,C=14)
	CT(14861)=(S=6013810,E=6013811,C=18)
	CT(14862)=(S=6013812,E=6013813,C=42)
	CT(14863)=(S=6013814,E=6013815,C=36)
	CT(14864)=(S=6013816,E=6013817,C=31)
	CT(14865)=(S=6013818,E=6013819,C=14)
	CT(14866)=(S=6013820,E=6013829,C=18)
	CT(14867)=(S=6013830,E=6013831,C=58)
	CT(14868)=(S=6013832,E=6013833,C=18)
	CT(14869)=(S=6013834,E=6013835,C=37)
	CT(14870)=(S=6013836,E=6013837,C=28)
	CT(14871)=(S=6013838,E=6013839,C=34)
	CT(14872)=(S=6013840,E=6013841,C=38)
	CT(14873)=(S=6013842,E=6013843,C=52)
	CT(14874)=(S=6013844,E=6013845,C=18)
	CT(14875)=(S=6013846,E=6013847,C=28)
	CT(14876)=(S=6013848,E=6013849,C=22)
	CT(14877)=(S=6013850,E=6013851,C=19)
	CT(14878)=(S=6013852,E=6013853,C=31)
	CT(14879)=(S=6013854,E=6013855,C=58)
	CT(14880)=(S=6013856,E=6013857,C=36)
	CT(14881)=(S=6013858,E=6013859,C=17)
	CT(14882)=(S=6013860,E=6013861,C=12)
	CT(14883)=(S=6013862,E=6013863,C=13)
	CT(14884)=(S=6013864,E=6013865,C=14)
	CT(14885)=(S=6013866,E=6013867,C=18)
	CT(14886)=(S=6013868,E=6013869,C=31)
	CT(14887)=(S=6013872,E=6013873,C=46)
	CT(14888)=(S=6013874,E=6013875,C=18)
	CT(14889)=(S=6013876,E=6013877,C=38)
	CT(14890)=(S=6013878,E=6013879,C=14)
	CT(14891)=(S=6013880,E=6013881,C=31)
	CT(14892)=(S=6013882,E=6013883,C=14)
	CT(14893)=(S=6013884,E=6013885,C=34)
	CT(14894)=(S=6013886,E=6013887,C=58)
	CT(14895)=(S=6013888,E=6013889,C=14)
	CT(14896)=(S=6013890,E=6013893,C=38)
	CT(14897)=(S=6013894,E=6013895,C=42)
	CT(14898)=(S=6013896,E=6013897,C=36)
	CT(14899)=(S=6013898,E=6013899,C=33)
	CT(14900)=(S=6013900,E=6013901,C=18)
	CT(14901)=(S=6013902,E=6013903,C=12)
	CT(14902)=(S=6013904,E=6013907,C=18)
	CT(14903)=(S=6013910,E=6013911,C=31)
	CT(14904)=(S=6013912,E=6013913,C=38)
	CT(14905)=(S=6013914,E=6013917,C=28)
	CT(14906)=(S=6013920,E=6013921,C=19)
	CT(14907)=(S=6013922,E=6013923,C=20)
	CT(14908)=(S=6013924,E=6013925,C=14)
	CT(14909)=(S=6013926,E=6013927,C=31)
	CT(14910)=(S=6013928,E=6013929,C=58)
	CT(14911)=(S=6013930,E=6013933,C=36)
	CT(14912)=(S=6013934,E=6013935,C=34)
	CT(14913)=(S=6013936,E=6013937,C=28)
	CT(14914)=(S=6013938,E=6013939,C=36)
	CT(14915)=(S=6013940,E=6013941,C=31)
	CT(14916)=(S=6013942,E=6013943,C=55)
	CT(14917)=(S=6013944,E=6013945,C=31)
	CT(14918)=(S=6013946,E=6013947,C=58)
	CT(14919)=(S=6013948,E=6013951,C=18)
	CT(14920)=(S=6013952,E=6013955,C=31)
	CT(14921)=(S=6013956,E=6013959,C=18)
	CT(14922)=(S=6013960,E=6013967,C=58)
	CT(14923)=(S=6013972,E=6013975,C=14)
	CT(14924)=(S=6013976,E=6013979,C=18)
	CT(14925)=(S=6013980,E=6013983,C=58)
	CT(14926)=(S=6013984,E=6013987,C=18)
	CT(14927)=(S=6013988,E=6013991,C=66)
	CT(14928)=(S=6013996,E=6013999,C=38)
	CT(14929)=(S=6014000,E=6014003,C=58)
	CT(14930)=(S=6014004,E=6014007,C=31)
	CT(14931)=(S=6014008,E=6014011,C=58)
	CT(14932)=(S=6014012,E=6014015,C=31)
	CT(14933)=(S=6014024,E=6014027,C=18)
	CT(14934)=(S=6014028,E=6014031,C=98)
	CT(14935)=(S=6014032,E=6014035,C=31)
	CT(14936)=(S=6014036,E=6014039,C=18)
	CT(14937)=(S=6014040,E=6014055,C=31)
	CT(14938)=(S=6014056,E=6014059,C=36)
	CT(14939)=(S=6014060,E=6014063,C=58)
	CT(14940)=(S=6014064,E=6014067,C=50)
	CT(14941)=(S=6014068,E=6014071,C=28)
	CT(14942)=(S=6014072,E=6014075,C=31)
	CT(14943)=(S=6014076,E=6014079,C=51)
	CT(14944)=(S=6014080,E=6014083,C=25)
	CT(14945)=(S=6014084,E=6014087,C=31)
	CT(14946)=(S=6014088,E=6014091,C=18)
	CT(14947)=(S=6014092,E=6014099,C=28)
	CT(14948)=(S=6014100,E=6014103,C=31)
	CT(14949)=(S=6014104,E=6014107,C=18)
	CT(14950)=(S=6014108,E=6014119,C=31)
	CT(14951)=(S=6014120,E=6014123,C=36)
	CT(14952)=(S=6014124,E=6014127,C=24)
	CT(14953)=(S=6014128,E=6014131,C=31)
	CT(14954)=(S=6014136,E=6014139,C=14)
	CT(14955)=(S=6014140,E=6014143,C=18)
	CT(14956)=(S=6014144,E=6014151,C=31)
	CT(14957)=(S=6014156,E=6014159,C=28)
	CT(14958)=(S=6014160,E=6014163,C=18)
	CT(14959)=(S=6014164,E=6014167,C=58)
	CT(14960)=(S=6014176,E=6014179,C=51)
	CT(14961)=(S=6014180,E=6014183,C=31)
	CT(14962)=(S=6014188,E=6014191,C=18)
	CT(14963)=(S=6014192,E=6014195,C=19)
	CT(14964)=(S=6014196,E=6014207,C=18)
	CT(14965)=(S=6014212,E=6014215,C=31)
	CT(14966)=(S=6014216,E=6014219,C=18)
	CT(14967)=(S=6014220,E=6014223,C=58)
	CT(14968)=(S=6014224,E=6014227,C=31)
	CT(14969)=(S=6014228,E=6014231,C=18)
	CT(14970)=(S=6014232,E=6014235,C=31)
	CT(14971)=(S=6014236,E=6014239,C=28)
	CT(14972)=(S=6014240,E=6014243,C=14)
	CT(14973)=(S=6014252,E=6014253,C=32)
	CT(14974)=(S=6014254,E=6014255,C=39)
	CT(14975)=(S=6014256,E=6014259,C=31)
	CT(14976)=(S=6014260,E=6014263,C=18)
	CT(14977)=(S=6014264,E=6014267,C=31)
	CT(14978)=(S=6014268,E=6014271,C=17)
	CT(14979)=(S=6014272,E=6014279,C=18)
	CT(14980)=(S=6014280,E=6014283,C=36)
	CT(14981)=(S=6014284,E=6014287,C=18)
	CT(14982)=(S=6014288,E=6014291,C=31)
	CT(14983)=(S=6014292,E=6014295,C=47)
	CT(14984)=(S=6014296,E=6014299,C=58)
	CT(14985)=(S=6014300,E=6014303,C=14)
	CT(14986)=(S=6014304,E=6014307,C=18)
	CT(14987)=(S=6014308,E=6014311,C=17)
	CT(14988)=(S=6014312,E=6014315,C=18)
	CT(14989)=(S=6014316,E=6014319,C=49)
	CT(14990)=(S=6014320,E=6014323,C=18)
	CT(14991)=(S=6014324,E=6014327,C=33)
	CT(14992)=(S=6014332,E=6014335,C=18)
	CT(14993)=(S=6014336,E=6014343,C=31)
	CT(14994)=(S=6014344,E=6014347,C=12)
	CT(14995)=(S=6014348,E=6014351,C=14)
	CT(14996)=(S=6014352,E=6014355,C=31)
	CT(14997)=(S=6014356,E=6014363,C=58)
	CT(14998)=(S=6014364,E=6014367,C=31)
	CT(14999)=(S=6014368,E=6014371,C=18)
	CT(15000)=(S=6014372,E=6014375,C=12)
	CT(15001)=(S=6014376,E=6014379,C=31)
	CT(15002)=(S=6014380,E=6014391,C=18)
	CT(15003)=(S=6014392,E=6014395,C=31)
	CT(15004)=(S=6014396,E=6014415,C=18)
	CT(15005)=(S=6014416,E=6014419,C=47)
	CT(15006)=(S=6014420,E=6014423,C=18)
	CT(15007)=(S=6014424,E=6014431,C=31)
	CT(15008)=(S=6014432,E=6014435,C=58)
	CT(15009)=(S=6014436,E=6014439,C=14)
	CT(15010)=(S=6014440,E=6014443,C=18)
	CT(15011)=(S=6014444,E=6014447,C=31)
	CT(15012)=(S=6014452,E=6014455,C=38)
	CT(15013)=(S=6014456,E=6014459,C=24)
	CT(15014)=(S=6014460,E=6014463,C=31)
	CT(15015)=(S=6014464,E=6014464,C=52)
	CT(15016)=(S=6014465,E=6014465,C=31)
	CT(15017)=(S=6014466,E=6014466,C=28)
	CT(15018)=(S=6014467,E=6014467,C=14)
	CT(15019)=(S=6014468,E=6014468,C=31)
	CT(15020)=(S=6014469,E=6014469,C=17)
	CT(15021)=(S=6014470,E=6014470,C=34)
	CT(15022)=(S=6014471,E=6014471,C=50)
	CT(15023)=(S=6014472,E=6014472,C=34)
	CT(15024)=(S=6014473,E=6014473,C=14)
	CT(15025)=(S=6014474,E=6014474,C=31)
	CT(15026)=(S=6014475,E=6014476,C=34)
	CT(15027)=(S=6014477,E=6014477,C=14)
	CT(15028)=(S=6014478,E=6014478,C=18)
	CT(15029)=(S=6014479,E=6014480,C=58)
	CT(15030)=(S=6014481,E=6014481,C=44)
	CT(15031)=(S=6014482,E=6014482,C=14)
	CT(15032)=(S=6014483,E=6014483,C=16)
	CT(15033)=(S=6014484,E=6014484,C=58)
	CT(15034)=(S=6014485,E=6014485,C=28)
	CT(15035)=(S=6014486,E=6014486,C=14)
	CT(15036)=(S=6014487,E=6014487,C=38)
	CT(15037)=(S=6014488,E=6014488,C=63)
	CT(15038)=(S=6014489,E=6014489,C=42)
	CT(15039)=(S=6014490,E=6014490,C=49)
	CT(15040)=(S=6014491,E=6014491,C=36)
	CT(15041)=(S=6014493,E=6014493,C=14)
	CT(15042)=(S=6014494,E=6014494,C=12)
	CT(15043)=(S=6014495,E=6014495,C=31)
	CT(15044)=(S=6014496,E=6014496,C=28)
	CT(15045)=(S=6014497,E=6014497,C=18)
	CT(15046)=(S=6014498,E=6014498,C=31)
	CT(15047)=(S=6014499,E=6014499,C=28)
	CT(15048)=(S=6014500,E=6014500,C=31)
	CT(15049)=(S=6014502,E=6014503,C=18)
	CT(15050)=(S=6014504,E=6014504,C=31)
	CT(15051)=(S=6014506,E=6014506,C=24)
	CT(15052)=(S=6014507,E=6014507,C=28)
	CT(15053)=(S=6014508,E=6014508,C=12)
	CT(15054)=(S=6014509,E=6014509,C=42)
	CT(15055)=(S=6014510,E=6014510,C=18)
	CT(15056)=(S=6014511,E=6014511,C=21)
	CT(15057)=(S=6014513,E=6014513,C=37)
	CT(15058)=(S=6014514,E=6014514,C=31)
	CT(15059)=(S=6014515,E=6014515,C=18)
	CT(15060)=(S=6014516,E=6014516,C=52)
	CT(15061)=(S=6014517,E=6014517,C=58)
	CT(15062)=(S=6014518,E=6014518,C=18)
	CT(15063)=(S=6014519,E=6014520,C=58)
	CT(15064)=(S=6014521,E=6014521,C=28)
	CT(15065)=(S=6014522,E=6014522,C=34)
	CT(15066)=(S=6014523,E=6014523,C=54)
	CT(15067)=(S=6014524,E=6014524,C=28)
	CT(15068)=(S=6014525,E=6014525,C=37)
	CT(15069)=(S=6014526,E=6014526,C=23)
	CT(15070)=(S=6014527,E=6014527,C=20)
	CT(15071)=(S=6014528,E=6014528,C=63)
	CT(15072)=(S=6014529,E=6014529,C=58)
	CT(15073)=(S=6014530,E=6014531,C=28)
	CT(15074)=(S=6014532,E=6014533,C=18)
	CT(15075)=(S=6014534,E=6014534,C=17)
	CT(15076)=(S=6014535,E=6014535,C=18)
	CT(15077)=(S=6014536,E=6014537,C=34)
	CT(15078)=(S=6014538,E=6014538,C=58)
	CT(15079)=(S=6014539,E=6014539,C=18)
	CT(15080)=(S=6014540,E=6014540,C=58)
	CT(15081)=(S=6014542,E=6014542,C=28)
	CT(15082)=(S=6014543,E=6014543,C=34)
	CT(15083)=(S=6014544,E=6014544,C=58)
	CT(15084)=(S=6014545,E=6014545,C=14)
	CT(15085)=(S=6014546,E=6014546,C=28)
	CT(15086)=(S=6014547,E=6014547,C=31)
	CT(15087)=(S=6014548,E=6014548,C=15)
	CT(15088)=(S=6014549,E=6014549,C=18)
	CT(15089)=(S=6014550,E=6014550,C=28)
	CT(15090)=(S=6014551,E=6014551,C=36)
	CT(15091)=(S=6014552,E=6014552,C=12)
	CT(15092)=(S=6014553,E=6014553,C=58)
	CT(15093)=(S=6014554,E=6014554,C=21)
	CT(15094)=(S=6014555,E=6014555,C=60)
	CT(15095)=(S=6014556,E=6014556,C=19)
	CT(15096)=(S=6014557,E=6014557,C=42)
	CT(15097)=(S=6014558,E=6014558,C=58)
	CT(15098)=(S=6014559,E=6014559,C=19)
	CT(15099)=(S=6014560,E=6014560,C=52)
	CT(15100)=(S=6014561,E=6014561,C=58)
	CT(15101)=(S=6014562,E=6014562,C=18)
	CT(15102)=(S=6014563,E=6014563,C=14)
	CT(15103)=(S=6014564,E=6014564,C=58)
	CT(15104)=(S=6014565,E=6014565,C=31)
	CT(15105)=(S=6014566,E=6014566,C=23)
	CT(15106)=(S=6014567,E=6014567,C=36)
	CT(15107)=(S=6014568,E=6014568,C=18)
	CT(15108)=(S=6014569,E=6014569,C=12)
	CT(15109)=(S=6014570,E=6014570,C=36)
	CT(15110)=(S=6014571,E=6014571,C=14)
	CT(15111)=(S=6014574,E=6014575,C=58)
	CT(15112)=(S=6014576,E=6014576,C=52)
	CT(15113)=(S=6014577,E=6014578,C=28)
	CT(15114)=(S=6014580,E=6014580,C=31)
	CT(15115)=(S=6014581,E=6014581,C=24)
	CT(15116)=(S=6014582,E=6014582,C=58)
	CT(15117)=(S=6014583,E=6014583,C=51)
	CT(15118)=(S=6014584,E=6014584,C=47)
	CT(15119)=(S=6014585,E=6014585,C=58)
	CT(15120)=(S=6014586,E=6014586,C=12)
	CT(15121)=(S=6014587,E=6014587,C=27)
	CT(15122)=(S=6014588,E=6014588,C=37)
	CT(15123)=(S=6014589,E=6014589,C=12)
	CT(15124)=(S=6014590,E=6014590,C=49)
	CT(15125)=(S=6014591,E=6014591,C=18)
	CT(15126)=(S=6014592,E=6014592,C=42)
	CT(15127)=(S=6014593,E=6014593,C=17)
	CT(15128)=(S=6014594,E=6014594,C=18)
	CT(15129)=(S=6014595,E=6014595,C=50)
	CT(15130)=(S=6014596,E=6014596,C=51)
	CT(15131)=(S=6014597,E=6014597,C=31)
	CT(15132)=(S=6014598,E=6014598,C=101)
	CT(15133)=(S=6014599,E=6014599,C=19)
	CT(15134)=(S=6014600,E=6014600,C=38)
	CT(15135)=(S=6014601,E=6014601,C=28)
	CT(15136)=(S=6014602,E=6014602,C=24)
	CT(15137)=(S=6014603,E=6014603,C=28)
	CT(15138)=(S=6014604,E=6014604,C=31)
	CT(15139)=(S=6014605,E=6014605,C=36)
	CT(15140)=(S=6014606,E=6014606,C=42)
	CT(15141)=(S=6014607,E=6014607,C=31)
	CT(15142)=(S=6014608,E=6014608,C=24)
	CT(15143)=(S=6014609,E=6014610,C=58)
	CT(15144)=(S=6014612,E=6014612,C=42)
	CT(15145)=(S=6014613,E=6014613,C=18)
	CT(15146)=(S=6014614,E=6014614,C=58)
	CT(15147)=(S=6014615,E=6014615,C=36)
	CT(15148)=(S=6014616,E=6014616,C=28)
	CT(15149)=(S=6014617,E=6014617,C=31)
	CT(15150)=(S=6014618,E=6014618,C=34)
	CT(15151)=(S=6014619,E=6014619,C=18)
	CT(15152)=(S=6014620,E=6014620,C=60)
	CT(15153)=(S=6014621,E=6014621,C=28)
	CT(15154)=(S=6014623,E=6014623,C=12)
	CT(15155)=(S=6014624,E=6014624,C=38)
	CT(15156)=(S=6014625,E=6014625,C=58)
	CT(15157)=(S=6014626,E=6014626,C=38)
	CT(15158)=(S=6014627,E=6014627,C=28)
	CT(15159)=(S=6014628,E=6014628,C=19)
	CT(15160)=(S=6014629,E=6014629,C=14)
	CT(15161)=(S=6014630,E=6014630,C=69)
	CT(15162)=(S=6014632,E=6014633,C=24)
	CT(15163)=(S=6014635,E=6014635,C=18)
	CT(15164)=(S=6014636,E=6014636,C=24)
	CT(15165)=(S=6014637,E=6014637,C=34)
	CT(15166)=(S=6014638,E=6014638,C=36)
	CT(15167)=(S=6014639,E=6014639,C=31)
	CT(15168)=(S=6014640,E=6014640,C=27)
	CT(15169)=(S=6014641,E=6014641,C=58)
	CT(15170)=(S=6014642,E=6014642,C=36)
	CT(15171)=(S=6014643,E=6014643,C=58)
	CT(15172)=(S=6014644,E=6014644,C=14)
	CT(15173)=(S=6014645,E=6014645,C=51)
	CT(15174)=(S=6014646,E=6014646,C=21)
	CT(15175)=(S=6014647,E=6014647,C=22)
	CT(15176)=(S=6014648,E=6014648,C=18)
	CT(15177)=(S=6014650,E=6014650,C=58)
	CT(15178)=(S=6014651,E=6014651,C=28)
	CT(15179)=(S=6014652,E=6014652,C=31)
	CT(15180)=(S=6014653,E=6014653,C=28)
	CT(15181)=(S=6014654,E=6014654,C=52)
	CT(15182)=(S=6014655,E=6014655,C=12)
	CT(15183)=(S=6014657,E=6014657,C=19)
	CT(15184)=(S=6014658,E=6014658,C=58)
	CT(15185)=(S=6014659,E=6014659,C=42)
	CT(15186)=(S=6014661,E=6014661,C=28)
	CT(15187)=(S=6014662,E=6014662,C=50)
	CT(15188)=(S=6014663,E=6014664,C=12)
	CT(15189)=(S=6014665,E=6014665,C=27)
	CT(15190)=(S=6014666,E=6014666,C=19)
	CT(15191)=(S=6014667,E=6014667,C=49)
	CT(15192)=(S=6014668,E=6014668,C=24)
	CT(15193)=(S=6014669,E=6014669,C=17)
	CT(15194)=(S=6014670,E=6014670,C=55)
	CT(15195)=(S=6014671,E=6014671,C=72)
	CT(15196)=(S=6014673,E=6014673,C=58)
	CT(15197)=(S=6014675,E=6014675,C=49)
	CT(15198)=(S=6014676,E=6014677,C=18)
	CT(15199)=(S=6014678,E=6014678,C=52)
	CT(15200)=(S=6014679,E=6014679,C=31)
	CT(15201)=(S=6014680,E=6014680,C=57)
	CT(15202)=(S=6014681,E=6014681,C=58)
	CT(15203)=(S=6014683,E=6014683,C=59)
	CT(15204)=(S=6014685,E=6014685,C=18)
	CT(15205)=(S=6014686,E=6014686,C=58)
	CT(15206)=(S=6014687,E=6014687,C=19)
	CT(15207)=(S=6014688,E=6014688,C=28)
	CT(15208)=(S=6014689,E=6014689,C=18)
	CT(15209)=(S=6014690,E=6014690,C=58)
	CT(15210)=(S=6014691,E=6014691,C=24)
	CT(15211)=(S=6014692,E=6014692,C=28)
	CT(15212)=(S=6014693,E=6014693,C=34)
	CT(15213)=(S=6014695,E=6014695,C=18)
	CT(15214)=(S=6014696,E=6014696,C=58)
	CT(15215)=(S=6014697,E=6014697,C=31)
	CT(15216)=(S=6014698,E=6014698,C=36)
	CT(15217)=(S=6014699,E=6014699,C=31)
	CT(15218)=(S=6014700,E=6014700,C=53)
	CT(15219)=(S=6014701,E=6014701,C=24)
	CT(15220)=(S=6014702,E=6014702,C=28)
	CT(15221)=(S=6014703,E=6014703,C=21)
	CT(15222)=(S=6014704,E=6014704,C=42)
	CT(15223)=(S=6014705,E=6014705,C=14)
	CT(15224)=(S=6014706,E=6014706,C=28)
	CT(15225)=(S=6014707,E=6014707,C=49)
	CT(15226)=(S=6014708,E=6014708,C=14)
	CT(15227)=(S=6014709,E=6014710,C=58)
	CT(15228)=(S=6014711,E=6014711,C=31)
	CT(15229)=(S=6014712,E=6014712,C=58)
	CT(15230)=(S=6014713,E=6014713,C=31)
	CT(15231)=(S=6014714,E=6014714,C=28)
	CT(15232)=(S=6014715,E=6014715,C=23)
	CT(15233)=(S=6014716,E=6014716,C=24)
	CT(15234)=(S=6014717,E=6014717,C=28)
	CT(15235)=(S=6014719,E=6014719,C=14)
	CT(15236)=(S=6014720,E=6014720,C=12)
	CT(15237)=(S=6014721,E=6014721,C=14)
	CT(15238)=(S=6014722,E=6014722,C=15)
	CT(15239)=(S=6014723,E=6014723,C=49)
	CT(15240)=(S=6014724,E=6014724,C=18)
	CT(15241)=(S=6014725,E=6014725,C=42)
	CT(15242)=(S=6014726,E=6014726,C=12)
	CT(15243)=(S=6014727,E=6014727,C=22)
	CT(15244)=(S=6014728,E=6014728,C=28)
	CT(15245)=(S=6014730,E=6014730,C=18)
	CT(15246)=(S=6014731,E=6014731,C=12)
	CT(15247)=(S=6014732,E=6014732,C=58)
	CT(15248)=(S=6014733,E=6014733,C=31)
	CT(15249)=(S=6014734,E=6014734,C=18)
	CT(15250)=(S=6014735,E=6014735,C=49)
	CT(15251)=(S=6014736,E=6014736,C=18)
	CT(15252)=(S=6014737,E=6014737,C=46)
	CT(15253)=(S=6014739,E=6014739,C=14)
	CT(15254)=(S=6014740,E=6014740,C=20)
	CT(15255)=(S=6014741,E=6014741,C=28)
	CT(15256)=(S=6014742,E=6014742,C=58)
	CT(15257)=(S=6014743,E=6014743,C=52)
	CT(15258)=(S=6014744,E=6014744,C=14)
	CT(15259)=(S=6014745,E=6014745,C=58)
	CT(15260)=(S=6014746,E=6014746,C=18)
	CT(15261)=(S=6014748,E=6014748,C=31)
	CT(15262)=(S=6014749,E=6014749,C=17)
	CT(15263)=(S=6014751,E=6014751,C=71)
	CT(15264)=(S=6014753,E=6014753,C=31)
	CT(15265)=(S=6014754,E=6014754,C=18)
	CT(15266)=(S=6014755,E=6014755,C=31)
	CT(15267)=(S=6014756,E=6014756,C=51)
	CT(15268)=(S=6014757,E=6014757,C=31)
	CT(15269)=(S=6014758,E=6014758,C=66)
	CT(15270)=(S=6014759,E=6014759,C=39)
	CT(15271)=(S=6014760,E=6014760,C=18)
	CT(15272)=(S=6014761,E=6014761,C=38)
	CT(15273)=(S=6014763,E=6014763,C=34)
	CT(15274)=(S=6014765,E=6014765,C=36)
	CT(15275)=(S=6014766,E=6014766,C=18)
	CT(15276)=(S=6014768,E=6014768,C=58)
	CT(15277)=(S=6014769,E=6014769,C=38)
	CT(15278)=(S=6014770,E=6014770,C=36)
	CT(15279)=(S=6014771,E=6014771,C=58)
	CT(15280)=(S=6014772,E=6014772,C=50)
	CT(15281)=(S=6014773,E=6014773,C=17)
	CT(15282)=(S=6014774,E=6014774,C=31)
	CT(15283)=(S=6014775,E=6014775,C=44)
	CT(15284)=(S=6014776,E=6014776,C=69)
	CT(15285)=(S=6014777,E=6014777,C=58)
	CT(15286)=(S=6014778,E=6014779,C=36)
	CT(15287)=(S=6014780,E=6014780,C=28)
	CT(15288)=(S=6014781,E=6014781,C=52)
	CT(15289)=(S=6014782,E=6014782,C=18)
	CT(15290)=(S=6014783,E=6014783,C=27)
	CT(15291)=(S=6014784,E=6014784,C=19)
	CT(15292)=(S=6014785,E=6014785,C=28)
	CT(15293)=(S=6014786,E=6014786,C=34)
	CT(15294)=(S=6014787,E=6014787,C=18)
	CT(15295)=(S=6014788,E=6014788,C=19)
	CT(15296)=(S=6014789,E=6014789,C=17)
	CT(15297)=(S=6014790,E=6014790,C=42)
	CT(15298)=(S=6014791,E=6014791,C=14)
	CT(15299)=(S=6014792,E=6014792,C=50)
	CT(15300)=(S=6014793,E=6014793,C=37)
	CT(15301)=(S=6014794,E=6014794,C=63)
	CT(15302)=(S=6014795,E=6014795,C=28)
	CT(15303)=(S=6014796,E=6014796,C=34)
	CT(15304)=(S=6014797,E=6014797,C=57)
	CT(15305)=(S=6014798,E=6014798,C=14)
	CT(15306)=(S=6014799,E=6014799,C=57)
	CT(15307)=(S=6014800,E=6014800,C=18)
	CT(15308)=(S=6014803,E=6014803,C=28)
	CT(15309)=(S=6014804,E=6014804,C=36)
	CT(15310)=(S=6014805,E=6014805,C=34)
	CT(15311)=(S=6014806,E=6014806,C=60)
	CT(15312)=(S=6014808,E=6014808,C=28)
	CT(15313)=(S=6014809,E=6014809,C=58)
	CT(15314)=(S=6014811,E=6014813,C=31)
	CT(15315)=(S=6014814,E=6014814,C=17)
	CT(15316)=(S=6014815,E=6014815,C=58)
	CT(15317)=(S=6014816,E=6014816,C=22)
	CT(15318)=(S=6014817,E=6014817,C=58)
	CT(15319)=(S=6014818,E=6014818,C=34)
	CT(15320)=(S=6014819,E=6014820,C=17)
	CT(15321)=(S=6014821,E=6014821,C=58)
	CT(15322)=(S=6014823,E=6014823,C=12)
	CT(15323)=(S=6014824,E=6014824,C=38)
	CT(15324)=(S=6014825,E=6014825,C=58)
	CT(15325)=(S=6014826,E=6014826,C=31)
	CT(15326)=(S=6014827,E=6014827,C=23)
	CT(15327)=(S=6014828,E=6014828,C=36)
	CT(15328)=(S=6014829,E=6014829,C=187)
	CT(15329)=(S=6014830,E=6014830,C=14)
	CT(15330)=(S=6014831,E=6014831,C=37)
	CT(15331)=(S=6014832,E=6014832,C=31)
	CT(15332)=(S=6014833,E=6014833,C=24)
	CT(15333)=(S=6014834,E=6014834,C=18)
	CT(15334)=(S=6014835,E=6014835,C=31)
	CT(15335)=(S=6014836,E=6014836,C=50)
	CT(15336)=(S=6014837,E=6014837,C=58)
	CT(15337)=(S=6014838,E=6014838,C=14)
	CT(15338)=(S=6014839,E=6014839,C=17)
	CT(15339)=(S=6014840,E=6014840,C=25)
	CT(15340)=(S=6014841,E=6014841,C=24)
	CT(15341)=(S=6014842,E=6014842,C=25)
	CT(15342)=(S=6014844,E=6014844,C=52)
	CT(15343)=(S=6014845,E=6014845,C=14)
	CT(15344)=(S=6014846,E=6014846,C=60)
	CT(15345)=(S=6014847,E=6014847,C=58)
	CT(15346)=(S=6014848,E=6014848,C=51)
	CT(15347)=(S=6014849,E=6014850,C=14)
	CT(15348)=(S=6014851,E=6014851,C=52)
	CT(15349)=(S=6014852,E=6014852,C=58)
	CT(15350)=(S=6014853,E=6014853,C=38)
	CT(15351)=(S=6014854,E=6014854,C=48)
	CT(15352)=(S=6014855,E=6014855,C=36)
	CT(15353)=(S=6014856,E=6014856,C=14)
	CT(15354)=(S=6014857,E=6014857,C=36)
	CT(15355)=(S=6014858,E=6014859,C=31)
	CT(15356)=(S=6014860,E=6014861,C=58)
	CT(15357)=(S=6014862,E=6014862,C=52)
	CT(15358)=(S=6014863,E=6014864,C=31)
	CT(15359)=(S=6014865,E=6014865,C=42)
	CT(15360)=(S=6014866,E=6014866,C=34)
	CT(15361)=(S=6014868,E=6014868,C=47)
	CT(15362)=(S=6014869,E=6014869,C=18)
	CT(15363)=(S=6014870,E=6014870,C=51)
	CT(15364)=(S=6014871,E=6014871,C=14)
	CT(15365)=(S=6014872,E=6014872,C=12)
	CT(15366)=(S=6014873,E=6014873,C=18)
	CT(15367)=(S=6014874,E=6014874,C=14)
	CT(15368)=(S=6014875,E=6014875,C=50)
	CT(15369)=(S=6014876,E=6014876,C=18)
	CT(15370)=(S=6014877,E=6014877,C=14)
	CT(15371)=(S=6014878,E=6014878,C=28)
	CT(15372)=(S=6014879,E=6014879,C=14)
	CT(15373)=(S=6014881,E=6014881,C=52)
	CT(15374)=(S=6014882,E=6014882,C=28)
	CT(15375)=(S=6014884,E=6014884,C=33)
	CT(15376)=(S=6014885,E=6014885,C=28)
	CT(15377)=(S=6014886,E=6014886,C=37)
	CT(15378)=(S=6014887,E=6014887,C=36)
	CT(15379)=(S=6014889,E=6014889,C=28)
	CT(15380)=(S=6014890,E=6014890,C=15)
	CT(15381)=(S=6014891,E=6014891,C=18)
	CT(15382)=(S=6014893,E=6014893,C=15)
	CT(15383)=(S=6014894,E=6014894,C=18)
	CT(15384)=(S=6014895,E=6014895,C=42)
	CT(15385)=(S=6014896,E=6014896,C=58)
	CT(15386)=(S=6014897,E=6014897,C=36)
	CT(15387)=(S=6014898,E=6014898,C=18)
	CT(15388)=(S=6014900,E=6014901,C=14)
	CT(15389)=(S=6014902,E=6014902,C=31)
	CT(15390)=(S=6014903,E=6014903,C=14)
	CT(15391)=(S=6014906,E=6014906,C=34)
	CT(15392)=(S=6014907,E=6014907,C=23)
	CT(15393)=(S=6014908,E=6014908,C=31)
	CT(15394)=(S=6014909,E=6014909,C=18)
	CT(15395)=(S=6014910,E=6014910,C=24)
	CT(15396)=(S=6014911,E=6014911,C=37)
	CT(15397)=(S=6014912,E=6014912,C=28)
	CT(15398)=(S=6014914,E=6014914,C=31)
	CT(15399)=(S=6014915,E=6014915,C=27)
	CT(15400)=(S=6014916,E=6014917,C=18)
	CT(15401)=(S=6014918,E=6014918,C=34)
	CT(15402)=(S=6014919,E=6014919,C=38)
	CT(15403)=(S=6014920,E=6014920,C=42)
	CT(15404)=(S=6014921,E=6014921,C=52)
	CT(15405)=(S=6014922,E=6014922,C=17)
	CT(15406)=(S=6014923,E=6014923,C=58)
	CT(15407)=(S=6014924,E=6014924,C=25)
	CT(15408)=(S=6014925,E=6014925,C=18)
	CT(15409)=(S=6014926,E=6014926,C=31)
	CT(15410)=(S=6014927,E=6014927,C=33)
	CT(15411)=(S=6014928,E=6014928,C=24)
	CT(15412)=(S=6014929,E=6014929,C=28)
	CT(15413)=(S=6014930,E=6014930,C=38)
	CT(15414)=(S=6014932,E=6014932,C=14)
	CT(15415)=(S=6014933,E=6014933,C=63)
	CT(15416)=(S=6014934,E=6014934,C=28)
	CT(15417)=(S=6014935,E=6014935,C=32)
	CT(15418)=(S=6014936,E=6014936,C=38)
	CT(15419)=(S=6014937,E=6014937,C=24)
	CT(15420)=(S=6014938,E=6014938,C=34)
	CT(15421)=(S=6014939,E=6014939,C=36)
	CT(15422)=(S=6014940,E=6014940,C=50)
	CT(15423)=(S=6014941,E=6014941,C=18)
	CT(15424)=(S=6014942,E=6014942,C=31)
	CT(15425)=(S=6014943,E=6014944,C=18)
	CT(15426)=(S=6014945,E=6014945,C=28)
	CT(15427)=(S=6014946,E=6014946,C=66)
	CT(15428)=(S=6014947,E=6014947,C=49)
	CT(15429)=(S=6014948,E=6014948,C=34)
	CT(15430)=(S=6014949,E=6014949,C=18)
	CT(15431)=(S=6014950,E=6014950,C=25)
	CT(15432)=(S=6014951,E=6014951,C=54)
	CT(15433)=(S=6014952,E=6014952,C=18)
	CT(15434)=(S=6014953,E=6014954,C=12)
	CT(15435)=(S=6014955,E=6014955,C=52)
	CT(15436)=(S=6014956,E=6014956,C=36)
	CT(15437)=(S=6014957,E=6014957,C=51)
	CT(15438)=(S=6014958,E=6014958,C=19)
	CT(15439)=(S=6014959,E=6014959,C=28)
	CT(15440)=(S=6014960,E=6014960,C=58)
	CT(15441)=(S=6014961,E=6014961,C=28)
	CT(15442)=(S=6014962,E=6014962,C=12)
	CT(15443)=(S=6014963,E=6014963,C=38)
	CT(15444)=(S=6014964,E=6014964,C=18)
	CT(15445)=(S=6014965,E=6014965,C=31)
	CT(15446)=(S=6014966,E=6014966,C=58)
	CT(15447)=(S=6014967,E=6014967,C=28)
	CT(15448)=(S=6014969,E=6014969,C=14)
	CT(15449)=(S=6014970,E=6014970,C=58)
	CT(15450)=(S=6014971,E=6014971,C=18)
	CT(15451)=(S=6014972,E=6014972,C=14)
	CT(15452)=(S=6014973,E=6014973,C=18)
	CT(15453)=(S=6014974,E=6014975,C=12)
	CT(15454)=(S=6014976,E=6014991,C=31)
	CT(15455)=(S=6014992,E=6014995,C=36)
	CT(15456)=(S=6014996,E=6014999,C=22)
	CT(15457)=(S=6015000,E=6015003,C=58)
	CT(15458)=(S=6015004,E=6015007,C=18)
	CT(15459)=(S=6015008,E=6015015,C=58)
	CT(15460)=(S=6015016,E=6015023,C=31)
	CT(15461)=(S=6015024,E=6015027,C=36)
	CT(15462)=(S=6015028,E=6015036,C=31)
	CT(15463)=(S=6015037,E=6015037,C=14)
	CT(15464)=(S=6015038,E=6015039,C=18)
	CT(15465)=(S=6015040,E=6015043,C=60)
	CT(15466)=(S=6015044,E=6015047,C=58)
	CT(15467)=(S=6015048,E=6015051,C=31)
	CT(15468)=(S=6015052,E=6015055,C=58)
	CT(15469)=(S=6015056,E=6015059,C=31)
	CT(15470)=(S=6015060,E=6015063,C=18)
	CT(15471)=(S=6015068,E=6015071,C=58)
	CT(15472)=(S=6015072,E=6015075,C=38)
	CT(15473)=(S=6015080,E=6015083,C=58)
	CT(15474)=(S=6015084,E=6015087,C=28)
	CT(15475)=(S=6015088,E=6015091,C=31)
	CT(15476)=(S=6015092,E=6015095,C=25)
	CT(15477)=(S=6015096,E=6015099,C=38)
	CT(15478)=(S=6015100,E=6015103,C=31)
	CT(15479)=(S=6015104,E=6015107,C=15)
	CT(15480)=(S=6015108,E=6015111,C=38)
	CT(15481)=(S=6015112,E=6015115,C=31)
	CT(15482)=(S=6015116,E=6015119,C=25)
	CT(15483)=(S=6015120,E=6015123,C=17)
	CT(15484)=(S=6015128,E=6015131,C=19)
	CT(15485)=(S=6015132,E=6015139,C=31)
	CT(15486)=(S=6015140,E=6015143,C=53)
	CT(15487)=(S=6015144,E=6015147,C=25)
	CT(15488)=(S=6015148,E=6015151,C=58)
	CT(15489)=(S=6015156,E=6015159,C=31)
	CT(15490)=(S=6015160,E=6015163,C=58)
	CT(15491)=(S=6015168,E=6015171,C=46)
	CT(15492)=(S=6015172,E=6015175,C=72)
	CT(15493)=(S=6015176,E=6015179,C=31)
	CT(15494)=(S=6015180,E=6015183,C=12)
	CT(15495)=(S=6015184,E=6015187,C=15)
	CT(15496)=(S=6015188,E=6015191,C=31)
	CT(15497)=(S=6015192,E=6015195,C=18)
	CT(15498)=(S=6015196,E=6015199,C=31)
	CT(15499)=(S=6015200,E=6015207,C=18)
	CT(15500)=(S=6015208,E=6015211,C=31)
	CT(15501)=(S=6015212,E=6015215,C=58)
	CT(15502)=(S=6015224,E=6015231,C=31)
	CT(15503)=(S=6015232,E=6015243,C=18)
	CT(15504)=(S=6015244,E=6015247,C=60)
	CT(15505)=(S=6015248,E=6015251,C=58)
	CT(15506)=(S=6015252,E=6015255,C=33)
	CT(15507)=(S=6015256,E=6015275,C=31)
	CT(15508)=(S=6015276,E=6015279,C=58)
	CT(15509)=(S=6015280,E=6015287,C=18)
	CT(15510)=(S=6015288,E=6015291,C=34)
	CT(15511)=(S=6015292,E=6015295,C=19)
	CT(15512)=(S=6015300,E=6015303,C=31)
	CT(15513)=(S=6015304,E=6015307,C=18)
	CT(15514)=(S=6015308,E=6015315,C=38)
	CT(15515)=(S=6015316,E=6015319,C=31)
	CT(15516)=(S=6015320,E=6015323,C=58)
	CT(15517)=(S=6015324,E=6015327,C=19)
	CT(15518)=(S=6015328,E=6015331,C=18)
	CT(15519)=(S=6015332,E=6015335,C=28)
	CT(15520)=(S=6015340,E=6015343,C=31)
	CT(15521)=(S=6015344,E=6015351,C=18)
	CT(15522)=(S=6015352,E=6015355,C=58)
	CT(15523)=(S=6015360,E=6015363,C=28)
	CT(15524)=(S=6015364,E=6015367,C=25)
	CT(15525)=(S=6015368,E=6015371,C=46)
	CT(15526)=(S=6015376,E=6015379,C=31)
	CT(15527)=(S=6015380,E=6015383,C=34)
	CT(15528)=(S=6015384,E=6015387,C=58)
	CT(15529)=(S=6015388,E=6015391,C=31)
	CT(15530)=(S=6015392,E=6015395,C=14)
	CT(15531)=(S=6015396,E=6015399,C=36)
	CT(15532)=(S=6015400,E=6015403,C=31)
	CT(15533)=(S=6015404,E=6015407,C=51)
	CT(15534)=(S=6015408,E=6015415,C=31)
	CT(15535)=(S=6015416,E=6015419,C=18)
	CT(15536)=(S=6015420,E=6015423,C=31)
	CT(15537)=(S=6015424,E=6015427,C=38)
	CT(15538)=(S=6015432,E=6015432,C=14)
	CT(15539)=(S=6015433,E=6015433,C=18)
	CT(15540)=(S=6015434,E=6015435,C=31)
	CT(15541)=(S=6015436,E=6015443,C=18)
	CT(15542)=(S=6015444,E=6015445,C=42)
	CT(15543)=(S=6015446,E=6015447,C=18)
	CT(15544)=(S=6015448,E=6015455,C=38)
	CT(15545)=(S=6015456,E=6015459,C=31)
	CT(15546)=(S=6015460,E=6015463,C=18)
	CT(15547)=(S=6015464,E=6015468,C=31)
	CT(15548)=(S=6015469,E=6015471,C=18)
	CT(15549)=(S=6015472,E=6015479,C=31)
	CT(15550)=(S=6015480,E=6015487,C=18)
	CT(15551)=(S=6015488,E=6015491,C=31)
	CT(15552)=(S=6015492,E=6015495,C=18)
	CT(15553)=(S=6015496,E=6015499,C=31)
	CT(15554)=(S=6015500,E=6015527,C=18)
	CT(15555)=(S=6015528,E=6015531,C=28)
	CT(15556)=(S=6015532,E=6015535,C=18)
	CT(15557)=(S=6015536,E=6015539,C=28)
	CT(15558)=(S=6015540,E=6015543,C=33)
	CT(15559)=(S=6015544,E=6015547,C=31)
	CT(15560)=(S=6015548,E=6015551,C=95)
	CT(15561)=(S=6015552,E=6015555,C=52)
	CT(15562)=(S=6015556,E=6015559,C=18)
	CT(15563)=(S=6015560,E=6015563,C=31)
	CT(15564)=(S=6015564,E=6015567,C=18)
	CT(15565)=(S=6015568,E=6015571,C=19)
	CT(15566)=(S=6015572,E=6015575,C=18)
	CT(15567)=(S=6015576,E=6015579,C=38)
	CT(15568)=(S=6015580,E=6015587,C=18)
	CT(15569)=(S=6015588,E=6015591,C=58)
	CT(15570)=(S=6015592,E=6015599,C=31)
	CT(15571)=(S=6015600,E=6015603,C=47)
	CT(15572)=(S=6015604,E=6015607,C=42)
	CT(15573)=(S=6015608,E=6015611,C=34)
	CT(15574)=(S=6015612,E=6015615,C=58)
	CT(15575)=(S=6015616,E=6015623,C=31)
	CT(15576)=(S=6015624,E=6015627,C=14)
	CT(15577)=(S=6015628,E=6015631,C=18)
	CT(15578)=(S=6015632,E=6015635,C=31)
	CT(15579)=(S=6015636,E=6015639,C=18)
	CT(15580)=(S=6015640,E=6015643,C=58)
	CT(15581)=(S=6015644,E=6015647,C=19)
	CT(15582)=(S=6015648,E=6015651,C=31)
	CT(15583)=(S=6015652,E=6015655,C=18)
	CT(15584)=(S=6015656,E=6015659,C=17)
	CT(15585)=(S=6015660,E=6015667,C=58)
	CT(15586)=(S=6015668,E=6015679,C=18)
	CT(15587)=(S=6015680,E=6015683,C=58)
	CT(15588)=(S=6015684,E=6015687,C=18)
	CT(15589)=(S=6015688,E=6015691,C=13)
	CT(15590)=(S=6015692,E=6015695,C=18)
	CT(15591)=(S=6015696,E=6015703,C=31)
	CT(15592)=(S=6015704,E=6015711,C=18)
	CT(15593)=(S=6015712,E=6015715,C=14)
	CT(15594)=(S=6015716,E=6015719,C=58)
	CT(15595)=(S=6015720,E=6015723,C=18)
	CT(15596)=(S=6015724,E=6015727,C=14)
	CT(15597)=(S=6015728,E=6015731,C=31)
	CT(15598)=(S=6015740,E=6015747,C=18)
	CT(15599)=(S=6015748,E=6015751,C=31)
	CT(15600)=(S=6015752,E=6015755,C=18)
	CT(15601)=(S=6015756,E=6015759,C=31)
	CT(15602)=(S=6015760,E=6015763,C=58)
	CT(15603)=(S=6015764,E=6015767,C=18)
	CT(15604)=(S=6015768,E=6015771,C=31)
	CT(15605)=(S=6015772,E=6015775,C=55)
	CT(15606)=(S=6015776,E=6015779,C=12)
	CT(15607)=(S=6015780,E=6015783,C=18)
	CT(15608)=(S=6015784,E=6015787,C=19)
	CT(15609)=(S=6015788,E=6015791,C=18)
	CT(15610)=(S=6015792,E=6015795,C=31)
	CT(15611)=(S=6015796,E=6015799,C=58)
	CT(15612)=(S=6015800,E=6015803,C=14)
	CT(15613)=(S=6015804,E=6015807,C=31)
	CT(15614)=(S=6015808,E=6015811,C=18)
	CT(15615)=(S=6015812,E=6015815,C=60)
	CT(15616)=(S=6015816,E=6015819,C=14)
	CT(15617)=(S=6015824,E=6015831,C=18)
	CT(15618)=(S=6015832,E=6015835,C=31)
	CT(15619)=(S=6015840,E=6015843,C=27)
	CT(15620)=(S=6015848,E=6015848,C=58)
	CT(15621)=(S=6015849,E=6015849,C=18)
	CT(15622)=(S=6015850,E=6015850,C=36)
	CT(15623)=(S=6015851,E=6015851,C=31)
	CT(15624)=(S=6015852,E=6015855,C=28)
	CT(15625)=(S=6015856,E=6015859,C=31)
	CT(15626)=(S=6015860,E=6015863,C=27)
	CT(15627)=(S=6015864,E=6015867,C=13)
	CT(15628)=(S=6015868,E=6015871,C=18)
	CT(15629)=(S=6015872,E=6015875,C=36)
	CT(15630)=(S=6015876,E=6015879,C=58)
	CT(15631)=(S=6015880,E=6015891,C=31)
	CT(15632)=(S=6015892,E=6015895,C=12)
	CT(15633)=(S=6015896,E=6015899,C=18)
	CT(15634)=(S=6015900,E=6015903,C=58)
	CT(15635)=(S=6015904,E=6015907,C=19)
	CT(15636)=(S=6015908,E=6015911,C=31)
	CT(15637)=(S=6015912,E=6015915,C=18)
	CT(15638)=(S=6015916,E=6015919,C=98)
	CT(15639)=(S=6015920,E=6015927,C=18)
	CT(15640)=(S=6015928,E=6015931,C=12)
	CT(15641)=(S=6015932,E=6015943,C=18)
	CT(15642)=(S=6015944,E=6015947,C=28)
	CT(15643)=(S=6015948,E=6015951,C=27)
	CT(15644)=(S=6015952,E=6015955,C=18)
	CT(15645)=(S=6015956,E=6015959,C=28)
	CT(15646)=(S=6015960,E=6015967,C=58)
	CT(15647)=(S=6015968,E=6015987,C=18)
	CT(15648)=(S=6015988,E=6015995,C=58)
	CT(15649)=(S=6015996,E=6015999,C=18)
	CT(15650)=(S=6016000,E=6016003,C=19)
	CT(15651)=(S=6016004,E=6016011,C=28)
	CT(15652)=(S=6016012,E=6016015,C=31)
	CT(15653)=(S=6016016,E=6016027,C=18)
	CT(15654)=(S=6016028,E=6016035,C=28)
	CT(15655)=(S=6016036,E=6016039,C=31)
	CT(15656)=(S=6016056,E=6016059,C=18)
	CT(15657)=(S=6016060,E=6016063,C=31)
	CT(15658)=(S=6016064,E=6016067,C=60)
	CT(15659)=(S=6016068,E=6016075,C=18)
	CT(15660)=(S=6016076,E=6016079,C=31)
	CT(15661)=(S=6016080,E=6016083,C=34)
	CT(15662)=(S=6016084,E=6016087,C=31)
	CT(15663)=(S=6016088,E=6016091,C=58)
	CT(15664)=(S=6016092,E=6016095,C=31)
	CT(15665)=(S=6016096,E=6016111,C=18)
	CT(15666)=(S=6016112,E=6016115,C=31)
	CT(15667)=(S=6016116,E=6016119,C=12)
	CT(15668)=(S=6016120,E=6016123,C=31)
	CT(15669)=(S=6016124,E=6016127,C=14)
	CT(15670)=(S=6016128,E=6016131,C=18)
	CT(15671)=(S=6016132,E=6016135,C=31)
	CT(15672)=(S=6016136,E=6016139,C=18)
	CT(15673)=(S=6016140,E=6016143,C=58)
	CT(15674)=(S=6016144,E=6016151,C=18)
	CT(15675)=(S=6016152,E=6016155,C=58)
	CT(15676)=(S=6016156,E=6016159,C=51)
	CT(15677)=(S=6016160,E=6016163,C=58)
	CT(15678)=(S=6016164,E=6016167,C=18)
	CT(15679)=(S=6016168,E=6016171,C=28)
	CT(15680)=(S=6016172,E=6016175,C=14)
	CT(15681)=(S=6016176,E=6016179,C=18)
	CT(15682)=(S=6016180,E=6016183,C=31)
	CT(15683)=(S=6016184,E=6016187,C=18)
	CT(15684)=(S=6016188,E=6016191,C=66)
	CT(15685)=(S=6016192,E=6016195,C=42)
	CT(15686)=(S=6016196,E=6016199,C=31)
	CT(15687)=(S=6016200,E=6016203,C=18)
	CT(15688)=(S=6016204,E=6016207,C=58)
	CT(15689)=(S=6016208,E=6016211,C=14)
	CT(15690)=(S=6016212,E=6016215,C=31)
	CT(15691)=(S=6016216,E=6016219,C=42)
	CT(15692)=(S=6016228,E=6016235,C=18)
	CT(15693)=(S=6016236,E=6016239,C=98)
	CT(15694)=(S=6016240,E=6016243,C=18)
	CT(15695)=(S=6016244,E=6016247,C=14)
	CT(15696)=(S=6016248,E=6016251,C=31)
	CT(15697)=(S=6016252,E=6016259,C=18)
	CT(15698)=(S=6016260,E=6016263,C=31)
	CT(15699)=(S=6016264,E=6016267,C=18)
	CT(15700)=(S=6016268,E=6016271,C=28)
	CT(15701)=(S=6016272,E=6016275,C=31)
	CT(15702)=(S=6016276,E=6016279,C=58)
	CT(15703)=(S=6016280,E=6016283,C=18)
	CT(15704)=(S=6016284,E=6016287,C=28)
	CT(15705)=(S=6016288,E=6016291,C=36)
	CT(15706)=(S=6016292,E=6016295,C=28)
	CT(15707)=(S=6016296,E=6016299,C=14)
	CT(15708)=(S=6016300,E=6016303,C=18)
	CT(15709)=(S=6016304,E=6016307,C=62)
	CT(15710)=(S=6016308,E=6016311,C=18)
	CT(15711)=(S=6016312,E=6016315,C=28)
	CT(15712)=(S=6016316,E=6016319,C=19)
	CT(15713)=(S=6016320,E=6016323,C=31)
	CT(15714)=(S=6016324,E=6016327,C=59)
	CT(15715)=(S=6016328,E=6016331,C=58)
	CT(15716)=(S=6016332,E=6016335,C=28)
	CT(15717)=(S=6016336,E=6016339,C=15)
	CT(15718)=(S=6016340,E=6016343,C=18)
	CT(15719)=(S=6016344,E=6016347,C=58)
	CT(15720)=(S=6016348,E=6016351,C=18)
	CT(15721)=(S=6016352,E=6016355,C=36)
	CT(15722)=(S=6016356,E=6016359,C=29)
	CT(15723)=(S=6016368,E=6016371,C=21)
	CT(15724)=(S=6016372,E=6016375,C=12)
	CT(15725)=(S=6016376,E=6016387,C=18)
	CT(15726)=(S=6016388,E=6016399,C=66)
	CT(15727)=(S=6016400,E=6016403,C=18)
	CT(15728)=(S=6016404,E=6016407,C=34)
	CT(15729)=(S=6016408,E=6016411,C=17)
	CT(15730)=(S=6016412,E=6016415,C=31)
	CT(15731)=(S=6016416,E=6016419,C=18)
	CT(15732)=(S=6016420,E=6016423,C=31)
	CT(15733)=(S=6016424,E=6016427,C=18)
	CT(15734)=(S=6016428,E=6016431,C=28)
	CT(15735)=(S=6016432,E=6016439,C=18)
	CT(15736)=(S=6016440,E=6016443,C=27)
	CT(15737)=(S=6016448,E=6016450,C=36)
	CT(15738)=(S=6016451,E=6016451,C=33)
	CT(15739)=(S=6016456,E=6016459,C=18)
	CT(15740)=(S=6016460,E=6016463,C=31)
	CT(15741)=(S=6016464,E=6016467,C=18)
	CT(15742)=(S=6016468,E=6016471,C=36)
	CT(15743)=(S=6016472,E=6016475,C=31)
	CT(15744)=(S=6016476,E=6016479,C=14)
	CT(15745)=(S=6016480,E=6016483,C=18)
	CT(15746)=(S=6016484,E=6016487,C=58)
	CT(15747)=(S=6016488,E=6016491,C=36)
	CT(15748)=(S=6016492,E=6016499,C=18)
	CT(15749)=(S=6016500,E=6016503,C=58)
	CT(15750)=(S=6016504,E=6016507,C=14)
	CT(15751)=(S=6016508,E=6016511,C=18)
	CT(15752)=(S=6016512,E=6016513,C=14)
	CT(15753)=(S=6016514,E=6016515,C=18)
	CT(15754)=(S=6016516,E=6016517,C=31)
	CT(15755)=(S=6016518,E=6016519,C=58)
	CT(15756)=(S=6016520,E=6016521,C=42)
	CT(15757)=(S=6016522,E=6016523,C=36)
	CT(15758)=(S=6016524,E=6016531,C=18)
	CT(15759)=(S=6016532,E=6016533,C=51)
	CT(15760)=(S=6016534,E=6016535,C=18)
	CT(15761)=(S=6016536,E=6016537,C=34)
	CT(15762)=(S=6016538,E=6016539,C=58)
	CT(15763)=(S=6016540,E=6016541,C=28)
	CT(15764)=(S=6016542,E=6016545,C=31)
	CT(15765)=(S=6016546,E=6016547,C=27)
	CT(15766)=(S=6016548,E=6016549,C=38)
	CT(15767)=(S=6016550,E=6016551,C=58)
	CT(15768)=(S=6016552,E=6016557,C=38)
	CT(15769)=(S=6016558,E=6016559,C=28)
	CT(15770)=(S=6016560,E=6016561,C=31)
	CT(15771)=(S=6016562,E=6016563,C=12)
	CT(15772)=(S=6016564,E=6016565,C=34)
	CT(15773)=(S=6016566,E=6016571,C=18)
	CT(15774)=(S=6016572,E=6016573,C=28)
	CT(15775)=(S=6016574,E=6016575,C=18)
	CT(15776)=(S=6016576,E=6016577,C=58)
	CT(15777)=(S=6016578,E=6016579,C=18)
	CT(15778)=(S=6016582,E=6016583,C=28)
	CT(15779)=(S=6016584,E=6016585,C=18)
	CT(15780)=(S=6016586,E=6016587,C=15)
	CT(15781)=(S=6016588,E=6016591,C=38)
	CT(15782)=(S=6016592,E=6016593,C=36)
	CT(15783)=(S=6016594,E=6016595,C=38)
	CT(15784)=(S=6016596,E=6016597,C=14)
	CT(15785)=(S=6016598,E=6016599,C=58)
	CT(15786)=(S=6016600,E=6016601,C=20)
	CT(15787)=(S=6016602,E=6016602,C=58)
	CT(15788)=(S=6016603,E=6016603,C=33)
	CT(15789)=(S=6016606,E=6016607,C=14)
	CT(15790)=(S=6016608,E=6016609,C=58)
	CT(15791)=(S=6016610,E=6016611,C=28)
	CT(15792)=(S=6016612,E=6016613,C=18)
	CT(15793)=(S=6016614,E=6016615,C=42)
	CT(15794)=(S=6016616,E=6016617,C=34)
	CT(15795)=(S=6016618,E=6016619,C=18)
	CT(15796)=(S=6016620,E=6016621,C=58)
	CT(15797)=(S=6016622,E=6016623,C=31)
	CT(15798)=(S=6016624,E=6016625,C=42)
	CT(15799)=(S=6016626,E=6016627,C=58)
	CT(15800)=(S=6016628,E=6016629,C=18)
	CT(15801)=(S=6016630,E=6016631,C=14)
	CT(15802)=(S=6016632,E=6016633,C=18)
	CT(15803)=(S=6016634,E=6016635,C=26)
	CT(15804)=(S=6016636,E=6016637,C=31)
	CT(15805)=(S=6016638,E=6016639,C=18)
	CT(15806)=(S=6016640,E=6016641,C=15)
	CT(15807)=(S=6016642,E=6016643,C=18)
	CT(15808)=(S=6016644,E=6016645,C=44)
	CT(15809)=(S=6016646,E=6016647,C=23)
	CT(15810)=(S=6016648,E=6016649,C=36)
	CT(15811)=(S=6016650,E=6016651,C=51)
	CT(15812)=(S=6016652,E=6016653,C=38)
	CT(15813)=(S=6016654,E=6016655,C=28)
	CT(15814)=(S=6016656,E=6016657,C=14)
	CT(15815)=(S=6016658,E=6016663,C=18)
	CT(15816)=(S=6016664,E=6016665,C=28)
	CT(15817)=(S=6016666,E=6016667,C=58)
	CT(15818)=(S=6016668,E=6016669,C=12)
	CT(15819)=(S=6016670,E=6016673,C=38)
	CT(15820)=(S=6016674,E=6016675,C=18)
	CT(15821)=(S=6016676,E=6016677,C=24)
	CT(15822)=(S=6016678,E=6016679,C=18)
	CT(15823)=(S=6016680,E=6016681,C=14)
	CT(15824)=(S=6016682,E=6016683,C=17)
	CT(15825)=(S=6016684,E=6016685,C=58)
	CT(15826)=(S=6016686,E=6016687,C=19)
	CT(15827)=(S=6016688,E=6016689,C=14)
	CT(15828)=(S=6016690,E=6016693,C=58)
	CT(15829)=(S=6016694,E=6016695,C=14)
	CT(15830)=(S=6016696,E=6016697,C=28)
	CT(15831)=(S=6016698,E=6016699,C=31)
	CT(15832)=(S=6016700,E=6016701,C=28)
	CT(15833)=(S=6016702,E=6016703,C=22)
	CT(15834)=(S=6016704,E=6016705,C=42)
	CT(15835)=(S=6016706,E=6016707,C=38)
	CT(15836)=(S=6016708,E=6016709,C=18)
	CT(15837)=(S=6016710,E=6016711,C=12)
	CT(15838)=(S=6016712,E=6016713,C=31)
	CT(15839)=(S=6016714,E=6016717,C=18)
	CT(15840)=(S=6016718,E=6016719,C=38)
	CT(15841)=(S=6016720,E=6016721,C=53)
	CT(15842)=(S=6016722,E=6016723,C=58)
	CT(15843)=(S=6016724,E=6016725,C=31)
	CT(15844)=(S=6016726,E=6016727,C=28)
	CT(15845)=(S=6016728,E=6016729,C=18)
	CT(15846)=(S=6016730,E=6016731,C=31)
	CT(15847)=(S=6016732,E=6016735,C=18)
	CT(15848)=(S=6016736,E=6016737,C=42)
	CT(15849)=(S=6016738,E=6016739,C=31)
	CT(15850)=(S=6016740,E=6016741,C=28)
	CT(15851)=(S=6016742,E=6016743,C=18)
	CT(15852)=(S=6016744,E=6016745,C=36)
	CT(15853)=(S=6016746,E=6016747,C=14)
	CT(15854)=(S=6016748,E=6016751,C=42)
	CT(15855)=(S=6016752,E=6016755,C=18)
	CT(15856)=(S=6016756,E=6016757,C=58)
	CT(15857)=(S=6016758,E=6016759,C=36)
	CT(15858)=(S=6016760,E=6016761,C=18)
	CT(15859)=(S=6016762,E=6016763,C=49)
	CT(15860)=(S=6016764,E=6016765,C=31)
	CT(15861)=(S=6016766,E=6016767,C=18)
	CT(15862)=(S=6016770,E=6016771,C=24)
	CT(15863)=(S=6016772,E=6016777,C=31)
	CT(15864)=(S=6016778,E=6016783,C=58)
	CT(15865)=(S=6016784,E=6016785,C=28)
	CT(15866)=(S=6016786,E=6016789,C=12)
	CT(15867)=(S=6016790,E=6016791,C=57)
	CT(15868)=(S=6016792,E=6016793,C=18)
	CT(15869)=(S=6016794,E=6016795,C=31)
	CT(15870)=(S=6016796,E=6016797,C=62)
	CT(15871)=(S=6016798,E=6016799,C=18)
	CT(15872)=(S=6016800,E=6016801,C=24)
	CT(15873)=(S=6016802,E=6016803,C=17)
	CT(15874)=(S=6016804,E=6016807,C=14)
	CT(15875)=(S=6016808,E=6016811,C=18)
	CT(15876)=(S=6016812,E=6016815,C=31)
	CT(15877)=(S=6016816,E=6016817,C=58)
	CT(15878)=(S=6016818,E=6016819,C=14)
	CT(15879)=(S=6016820,E=6016821,C=58)
	CT(15880)=(S=6016822,E=6016823,C=31)
	CT(15881)=(S=6016824,E=6016825,C=14)
	CT(15882)=(S=6016826,E=6016827,C=18)
	CT(15883)=(S=6016828,E=6016829,C=31)
	CT(15884)=(S=6016830,E=6016831,C=46)
	CT(15885)=(S=6016832,E=6016833,C=58)
	CT(15886)=(S=6016834,E=6016835,C=31)
	CT(15887)=(S=6016836,E=6016837,C=58)
	CT(15888)=(S=6016840,E=6016847,C=18)
	CT(15889)=(S=6016848,E=6016849,C=19)
	CT(15890)=(S=6016850,E=6016855,C=18)
	CT(15891)=(S=6016856,E=6016857,C=31)
	CT(15892)=(S=6016858,E=6016859,C=17)
	CT(15893)=(S=6016860,E=6016861,C=28)
	CT(15894)=(S=6016862,E=6016863,C=38)
	CT(15895)=(S=6016866,E=6016867,C=31)
	CT(15896)=(S=6016868,E=6016869,C=18)
	CT(15897)=(S=6016870,E=6016871,C=38)
	CT(15898)=(S=6016872,E=6016875,C=31)
	CT(15899)=(S=6016878,E=6016879,C=14)
	CT(15900)=(S=6016880,E=6016881,C=19)
	CT(15901)=(S=6016882,E=6016883,C=18)
	CT(15902)=(S=6016884,E=6016885,C=31)
	CT(15903)=(S=6016886,E=6016887,C=28)
	CT(15904)=(S=6016888,E=6016889,C=38)
	CT(15905)=(S=6016890,E=6016891,C=31)
	CT(15906)=(S=6016892,E=6016893,C=38)
	CT(15907)=(S=6016894,E=6016895,C=58)
	CT(15908)=(S=6016896,E=6016897,C=28)
	CT(15909)=(S=6016898,E=6016899,C=56)
	CT(15910)=(S=6016900,E=6016901,C=18)
	CT(15911)=(S=6016902,E=6016903,C=42)
	CT(15912)=(S=6016904,E=6016905,C=18)
	CT(15913)=(S=6016906,E=6016907,C=26)
	CT(15914)=(S=6016908,E=6016913,C=18)
	CT(15915)=(S=6016914,E=6016915,C=31)
	CT(15916)=(S=6016916,E=6016917,C=58)
	CT(15917)=(S=6016918,E=6016919,C=15)
	CT(15918)=(S=6016920,E=6016921,C=14)
	CT(15919)=(S=6016924,E=6016925,C=58)
	CT(15920)=(S=6016926,E=6016927,C=27)
	CT(15921)=(S=6016928,E=6016929,C=38)
	CT(15922)=(S=6016930,E=6016931,C=28)
	CT(15923)=(S=6016932,E=6016933,C=18)
	CT(15924)=(S=6016934,E=6016937,C=58)
	CT(15925)=(S=6016938,E=6016939,C=18)
	CT(15926)=(S=6016944,E=6016945,C=12)
	CT(15927)=(S=6016948,E=6016949,C=18)
	CT(15928)=(S=6016950,E=6016951,C=31)
	CT(15929)=(S=6016952,E=6016955,C=58)
	CT(15930)=(S=6016956,E=6016957,C=33)
	CT(15931)=(S=6016958,E=6016959,C=51)
	CT(15932)=(S=6016960,E=6016961,C=14)
	CT(15933)=(S=6016962,E=6016963,C=31)
	CT(15934)=(S=6016966,E=6016967,C=14)
	CT(15935)=(S=6016968,E=6016971,C=58)
	CT(15936)=(S=6016972,E=6016973,C=31)
	CT(15937)=(S=6016974,E=6016975,C=17)
	CT(15938)=(S=6016976,E=6016977,C=12)
	CT(15939)=(S=6016978,E=6016979,C=31)
	CT(15940)=(S=6016980,E=6016981,C=14)
	CT(15941)=(S=6016982,E=6016983,C=24)
	CT(15942)=(S=6016984,E=6016985,C=38)
	CT(15943)=(S=6016986,E=6016987,C=31)
	CT(15944)=(S=6016988,E=6016989,C=14)
	CT(15945)=(S=6016992,E=6016993,C=31)
	CT(15946)=(S=6016994,E=6016995,C=27)
	CT(15947)=(S=6016996,E=6016997,C=24)
	CT(15948)=(S=6017000,E=6017001,C=36)
	CT(15949)=(S=6017004,E=6017005,C=22)
	CT(15950)=(S=6017006,E=6017007,C=18)
	CT(15951)=(S=6017008,E=6017009,C=17)
	CT(15952)=(S=6017010,E=6017011,C=58)
	CT(15953)=(S=6017012,E=6017013,C=31)
	CT(15954)=(S=6017014,E=6017017,C=18)
	CT(15955)=(S=6017018,E=6017019,C=28)
	CT(15956)=(S=6017020,E=6017021,C=18)
	CT(15957)=(S=6017022,E=6017023,C=12)
	CT(15958)=(S=6017024,E=6017024,C=18)
	CT(15959)=(S=6017025,E=6017025,C=14)
	CT(15960)=(S=6017026,E=6017026,C=19)
	CT(15961)=(S=6017027,E=6017027,C=14)
	CT(15962)=(S=6017028,E=6017028,C=23)
	CT(15963)=(S=6017029,E=6017029,C=28)
	CT(15964)=(S=6017030,E=6017030,C=14)
	CT(15965)=(S=6017031,E=6017031,C=42)
	CT(15966)=(S=6017032,E=6017032,C=27)
	CT(15967)=(S=6017033,E=6017033,C=12)
	CT(15968)=(S=6017034,E=6017034,C=58)
	CT(15969)=(S=6017035,E=6017035,C=14)
	CT(15970)=(S=6017036,E=6017036,C=49)
	CT(15971)=(S=6017037,E=6017037,C=28)
	CT(15972)=(S=6017038,E=6017038,C=15)
	CT(15973)=(S=6017039,E=6017039,C=55)
	CT(15974)=(S=6017041,E=6017042,C=28)
	CT(15975)=(S=6017044,E=6017044,C=18)
	CT(15976)=(S=6017046,E=6017046,C=48)
	CT(15977)=(S=6017047,E=6017047,C=18)
	CT(15978)=(S=6017048,E=6017048,C=27)
	CT(15979)=(S=6017049,E=6017049,C=31)
	CT(15980)=(S=6017050,E=6017050,C=55)
	CT(15981)=(S=6017051,E=6017051,C=52)
	CT(15982)=(S=6017052,E=6017052,C=14)
	CT(15983)=(S=6017053,E=6017053,C=18)
	CT(15984)=(S=6017054,E=6017054,C=34)
	CT(15985)=(S=6017056,E=6017056,C=58)
	CT(15986)=(S=6017057,E=6017057,C=18)
	CT(15987)=(S=6017058,E=6017058,C=42)
	CT(15988)=(S=6017059,E=6017059,C=18)
	CT(15989)=(S=6017060,E=6017060,C=50)
	CT(15990)=(S=6017061,E=6017061,C=49)
	CT(15991)=(S=6017062,E=6017062,C=28)
	CT(15992)=(S=6017063,E=6017063,C=18)
	CT(15993)=(S=6017064,E=6017064,C=12)
	CT(15994)=(S=6017065,E=6017065,C=71)
	CT(15995)=(S=6017066,E=6017067,C=18)
	CT(15996)=(S=6017068,E=6017068,C=58)
	CT(15997)=(S=6017069,E=6017071,C=28)
	CT(15998)=(S=6017072,E=6017072,C=39)
	CT(15999)=(S=6017073,E=6017073,C=36)
	CT(16000)=(S=6017074,E=6017074,C=58)
	CT(16001)=(S=6017075,E=6017075,C=49)
	CT(16002)=(S=6017076,E=6017076,C=31)
	CT(16003)=(S=6017077,E=6017077,C=14)
	CT(16004)=(S=6017078,E=6017078,C=27)
	CT(16005)=(S=6017079,E=6017079,C=58)
	CT(16006)=(S=6017080,E=6017080,C=13)
	CT(16007)=(S=6017081,E=6017081,C=22)
	CT(16008)=(S=6017082,E=6017082,C=28)
	CT(16009)=(S=6017083,E=6017083,C=31)
	CT(16010)=(S=6017084,E=6017084,C=36)
	CT(16011)=(S=6017085,E=6017085,C=37)
	CT(16012)=(S=6017086,E=6017086,C=36)
	CT(16013)=(S=6017087,E=6017087,C=51)
	CT(16014)=(S=6017088,E=6017088,C=18)
	CT(16015)=(S=6017089,E=6017089,C=31)
	CT(16016)=(S=6017090,E=6017091,C=14)
	CT(16017)=(S=6017092,E=6017092,C=28)
	CT(16018)=(S=6017094,E=6017094,C=37)
	CT(16019)=(S=6017095,E=6017095,C=24)
	CT(16020)=(S=6017096,E=6017096,C=58)
	CT(16021)=(S=6017097,E=6017097,C=18)
	CT(16022)=(S=6017098,E=6017098,C=28)
	CT(16023)=(S=6017099,E=6017099,C=38)
	CT(16024)=(S=6017100,E=6017100,C=66)
	CT(16025)=(S=6017101,E=6017101,C=25)
	CT(16026)=(S=6017102,E=6017102,C=58)
	CT(16027)=(S=6017103,E=6017103,C=34)
	CT(16028)=(S=6017104,E=6017104,C=42)
	CT(16029)=(S=6017105,E=6017105,C=19)
	CT(16030)=(S=6017106,E=6017106,C=34)
	CT(16031)=(S=6017107,E=6017107,C=28)
	CT(16032)=(S=6017108,E=6017108,C=18)
	CT(16033)=(S=6017109,E=6017109,C=13)
	CT(16034)=(S=6017110,E=6017110,C=17)
	CT(16035)=(S=6017111,E=6017111,C=36)
	CT(16036)=(S=6017112,E=6017112,C=52)
	CT(16037)=(S=6017113,E=6017113,C=46)
	CT(16038)=(S=6017114,E=6017114,C=38)
	CT(16039)=(S=6017115,E=6017115,C=42)
	CT(16040)=(S=6017116,E=6017116,C=38)
	CT(16041)=(S=6017117,E=6017117,C=58)
	CT(16042)=(S=6017118,E=6017118,C=31)
	CT(16043)=(S=6017119,E=6017119,C=14)
	CT(16044)=(S=6017120,E=6017120,C=39)
	CT(16045)=(S=6017121,E=6017121,C=31)
	CT(16046)=(S=6017122,E=6017122,C=42)
	CT(16047)=(S=6017123,E=6017123,C=14)
	CT(16048)=(S=6017124,E=6017124,C=38)
	CT(16049)=(S=6017125,E=6017125,C=52)
	CT(16050)=(S=6017126,E=6017126,C=12)
	CT(16051)=(S=6017127,E=6017127,C=19)
	CT(16052)=(S=6017129,E=6017129,C=14)
	CT(16053)=(S=6017130,E=6017130,C=58)
	CT(16054)=(S=6017131,E=6017131,C=28)
	CT(16055)=(S=6017132,E=6017133,C=12)
	CT(16056)=(S=6017134,E=6017134,C=36)
	CT(16057)=(S=6017135,E=6017135,C=14)
	CT(16058)=(S=6017136,E=6017136,C=36)
	CT(16059)=(S=6017138,E=6017138,C=14)
	CT(16060)=(S=6017139,E=6017140,C=31)
	CT(16061)=(S=6017141,E=6017141,C=14)
	CT(16062)=(S=6017142,E=6017142,C=17)
	CT(16063)=(S=6017143,E=6017143,C=18)
	CT(16064)=(S=6017144,E=6017144,C=38)
	CT(16065)=(S=6017145,E=6017145,C=18)
	CT(16066)=(S=6017146,E=6017146,C=14)
	CT(16067)=(S=6017147,E=6017147,C=38)
	CT(16068)=(S=6017148,E=6017148,C=14)
	CT(16069)=(S=6017149,E=6017149,C=52)
	CT(16070)=(S=6017150,E=6017150,C=18)
	CT(16071)=(S=6017151,E=6017151,C=31)
	CT(16072)=(S=6017153,E=6017153,C=17)
	CT(16073)=(S=6017154,E=6017154,C=34)
	CT(16074)=(S=6017155,E=6017155,C=36)
	CT(16075)=(S=6017156,E=6017156,C=18)
	CT(16076)=(S=6017157,E=6017157,C=58)
	CT(16077)=(S=6017158,E=6017158,C=18)
	CT(16078)=(S=6017159,E=6017159,C=58)
	CT(16079)=(S=6017160,E=6017160,C=14)
	CT(16080)=(S=6017162,E=6017162,C=31)
	CT(16081)=(S=6017163,E=6017164,C=17)
	CT(16082)=(S=6017165,E=6017165,C=14)
	CT(16083)=(S=6017166,E=6017166,C=38)
	CT(16084)=(S=6017167,E=6017167,C=24)
	CT(16085)=(S=6017168,E=6017168,C=40)
	CT(16086)=(S=6017169,E=6017169,C=31)
	CT(16087)=(S=6017170,E=6017170,C=14)
	CT(16088)=(S=6017171,E=6017171,C=17)
	CT(16089)=(S=6017172,E=6017172,C=27)
	CT(16090)=(S=6017173,E=6017173,C=66)
	CT(16091)=(S=6017174,E=6017174,C=58)
	CT(16092)=(S=6017175,E=6017175,C=18)
	CT(16093)=(S=6017176,E=6017176,C=15)
	CT(16094)=(S=6017177,E=6017178,C=31)
	CT(16095)=(S=6017179,E=6017179,C=28)
	CT(16096)=(S=6017180,E=6017180,C=23)
	CT(16097)=(S=6017181,E=6017181,C=24)
	CT(16098)=(S=6017182,E=6017184,C=28)
	CT(16099)=(S=6017185,E=6017185,C=18)
	CT(16100)=(S=6017187,E=6017187,C=14)
	CT(16101)=(S=6017188,E=6017188,C=36)
	CT(16102)=(S=6017189,E=6017189,C=26)
	CT(16103)=(S=6017190,E=6017190,C=18)
	CT(16104)=(S=6017191,E=6017191,C=36)
	CT(16105)=(S=6017192,E=6017192,C=52)
	CT(16106)=(S=6017193,E=6017193,C=42)
	CT(16107)=(S=6017194,E=6017194,C=14)
	CT(16108)=(S=6017195,E=6017195,C=18)
	CT(16109)=(S=6017196,E=6017196,C=12)
	CT(16110)=(S=6017197,E=6017197,C=34)
	CT(16111)=(S=6017198,E=6017198,C=19)
	CT(16112)=(S=6017199,E=6017199,C=36)
	CT(16113)=(S=6017200,E=6017200,C=42)
	CT(16114)=(S=6017201,E=6017201,C=14)
	CT(16115)=(S=6017202,E=6017203,C=38)
	CT(16116)=(S=6017204,E=6017204,C=34)
	CT(16117)=(S=6017205,E=6017205,C=12)
	CT(16118)=(S=6017206,E=6017206,C=58)
	CT(16119)=(S=6017207,E=6017207,C=18)
	CT(16120)=(S=6017208,E=6017208,C=14)
	CT(16121)=(S=6017209,E=6017209,C=18)
	CT(16122)=(S=6017210,E=6017210,C=58)
	CT(16123)=(S=6017211,E=6017211,C=18)
	CT(16124)=(S=6017212,E=6017212,C=19)
	CT(16125)=(S=6017213,E=6017214,C=58)
	CT(16126)=(S=6017215,E=6017217,C=28)
	CT(16127)=(S=6017218,E=6017218,C=31)
	CT(16128)=(S=6017219,E=6017219,C=15)
	CT(16129)=(S=6017220,E=6017220,C=36)
	CT(16130)=(S=6017222,E=6017222,C=31)
	CT(16131)=(S=6017223,E=6017223,C=37)
	CT(16132)=(S=6017224,E=6017224,C=52)
	CT(16133)=(S=6017225,E=6017225,C=42)
	CT(16134)=(S=6017226,E=6017226,C=28)
	CT(16135)=(S=6017227,E=6017227,C=101)
	CT(16136)=(S=6017228,E=6017228,C=34)
	CT(16137)=(S=6017229,E=6017229,C=18)
	CT(16138)=(S=6017231,E=6017231,C=36)
	CT(16139)=(S=6017233,E=6017233,C=12)
	CT(16140)=(S=6017234,E=6017234,C=18)
	CT(16141)=(S=6017235,E=6017235,C=49)
	CT(16142)=(S=6017236,E=6017236,C=28)
	CT(16143)=(S=6017237,E=6017237,C=38)
	CT(16144)=(S=6017238,E=6017238,C=12)
	CT(16145)=(S=6017239,E=6017239,C=38)
	CT(16146)=(S=6017240,E=6017240,C=18)
	CT(16147)=(S=6017241,E=6017241,C=42)
	CT(16148)=(S=6017242,E=6017242,C=17)
	CT(16149)=(S=6017243,E=6017243,C=14)
	CT(16150)=(S=6017244,E=6017244,C=49)
	CT(16151)=(S=6017245,E=6017245,C=19)
	CT(16152)=(S=6017246,E=6017246,C=12)
	CT(16153)=(S=6017247,E=6017247,C=38)
	CT(16154)=(S=6017248,E=6017248,C=12)
	CT(16155)=(S=6017249,E=6017249,C=52)
	CT(16156)=(S=6017250,E=6017250,C=12)
	CT(16157)=(S=6017251,E=6017251,C=34)
	CT(16158)=(S=6017253,E=6017253,C=36)
	CT(16159)=(S=6017255,E=6017255,C=31)
	CT(16160)=(S=6017256,E=6017256,C=18)
	CT(16161)=(S=6017257,E=6017257,C=57)
	CT(16162)=(S=6017258,E=6017259,C=38)
	CT(16163)=(S=6017261,E=6017261,C=12)
	CT(16164)=(S=6017262,E=6017262,C=18)
	CT(16165)=(S=6017263,E=6017263,C=14)
	CT(16166)=(S=6017265,E=6017265,C=36)
	CT(16167)=(S=6017266,E=6017267,C=14)
	CT(16168)=(S=6017268,E=6017268,C=28)
	CT(16169)=(S=6017269,E=6017269,C=14)
	CT(16170)=(S=6017271,E=6017271,C=58)
	CT(16171)=(S=6017272,E=6017273,C=18)
	CT(16172)=(S=6017274,E=6017274,C=58)
	CT(16173)=(S=6017275,E=6017275,C=31)
	CT(16174)=(S=6017276,E=6017276,C=14)
	CT(16175)=(S=6017277,E=6017278,C=18)
	CT(16176)=(S=6017279,E=6017279,C=59)
	CT(16177)=(S=6017280,E=6017280,C=19)
	CT(16178)=(S=6017281,E=6017281,C=31)
	CT(16179)=(S=6017282,E=6017282,C=42)
	CT(16180)=(S=6017283,E=6017283,C=25)
	CT(16181)=(S=6017284,E=6017284,C=58)
	CT(16182)=(S=6017285,E=6017285,C=38)
	CT(16183)=(S=6017286,E=6017286,C=29)
	CT(16184)=(S=6017287,E=6017287,C=36)
	CT(16185)=(S=6017288,E=6017288,C=51)
	CT(16186)=(S=6017289,E=6017289,C=18)
	CT(16187)=(S=6017290,E=6017290,C=63)
	CT(16188)=(S=6017291,E=6017291,C=31)
	CT(16189)=(S=6017292,E=6017293,C=36)
	CT(16190)=(S=6017295,E=6017295,C=12)
	CT(16191)=(S=6017296,E=6017297,C=18)
	CT(16192)=(S=6017298,E=6017298,C=52)
	CT(16193)=(S=6017299,E=6017299,C=12)
	CT(16194)=(S=6017300,E=6017300,C=58)
	CT(16195)=(S=6017303,E=6017303,C=38)
	CT(16196)=(S=6017304,E=6017304,C=31)
	CT(16197)=(S=6017305,E=6017305,C=12)
	CT(16198)=(S=6017306,E=6017306,C=28)
	CT(16199)=(S=6017307,E=6017307,C=58)
	CT(16200)=(S=6017308,E=6017308,C=34)
	CT(16201)=(S=6017309,E=6017309,C=47)
	CT(16202)=(S=6017310,E=6017310,C=27)
	CT(16203)=(S=6017311,E=6017311,C=14)
	CT(16204)=(S=6017312,E=6017312,C=57)
	CT(16205)=(S=6017313,E=6017313,C=42)
	CT(16206)=(S=6017314,E=6017314,C=14)
	CT(16207)=(S=6017315,E=6017315,C=12)
	CT(16208)=(S=6017316,E=6017316,C=31)
	CT(16209)=(S=6017317,E=6017317,C=14)
	CT(16210)=(S=6017318,E=6017318,C=66)
	CT(16211)=(S=6017319,E=6017319,C=24)
	CT(16212)=(S=6017321,E=6017321,C=34)
	CT(16213)=(S=6017322,E=6017322,C=28)
	CT(16214)=(S=6017323,E=6017323,C=14)
	CT(16215)=(S=6017324,E=6017324,C=12)
	CT(16216)=(S=6017325,E=6017325,C=38)
	CT(16217)=(S=6017326,E=6017327,C=22)
	CT(16218)=(S=6017329,E=6017329,C=52)
	CT(16219)=(S=6017330,E=6017330,C=14)
	CT(16220)=(S=6017331,E=6017331,C=31)
	CT(16221)=(S=6017332,E=6017332,C=28)
	CT(16222)=(S=6017333,E=6017333,C=34)
	CT(16223)=(S=6017334,E=6017335,C=31)
	CT(16224)=(S=6017336,E=6017336,C=58)
	CT(16225)=(S=6017337,E=6017337,C=29)
	CT(16226)=(S=6017339,E=6017339,C=18)
	CT(16227)=(S=6017340,E=6017340,C=14)
	CT(16228)=(S=6017341,E=6017341,C=38)
	CT(16229)=(S=6017344,E=6017344,C=31)
	CT(16230)=(S=6017345,E=6017345,C=18)
	CT(16231)=(S=6017346,E=6017346,C=53)
	CT(16232)=(S=6017347,E=6017350,C=18)
	CT(16233)=(S=6017351,E=6017351,C=14)
	CT(16234)=(S=6017352,E=6017352,C=18)
	CT(16235)=(S=6017353,E=6017353,C=14)
	CT(16236)=(S=6017354,E=6017354,C=63)
	CT(16237)=(S=6017355,E=6017355,C=42)
	CT(16238)=(S=6017356,E=6017357,C=18)
	CT(16239)=(S=6017358,E=6017358,C=12)
	CT(16240)=(S=6017359,E=6017359,C=14)
	CT(16241)=(S=6017360,E=6017360,C=18)
	CT(16242)=(S=6017361,E=6017361,C=28)
	CT(16243)=(S=6017362,E=6017363,C=14)
	CT(16244)=(S=6017364,E=6017364,C=34)
	CT(16245)=(S=6017365,E=6017365,C=18)
	CT(16246)=(S=6017366,E=6017366,C=14)
	CT(16247)=(S=6017367,E=6017367,C=28)
	CT(16248)=(S=6017368,E=6017368,C=27)
	CT(16249)=(S=6017369,E=6017369,C=42)
	CT(16250)=(S=6017370,E=6017370,C=31)
	CT(16251)=(S=6017371,E=6017371,C=18)
	CT(16252)=(S=6017372,E=6017372,C=28)
	CT(16253)=(S=6017373,E=6017373,C=58)
	CT(16254)=(S=6017374,E=6017374,C=18)
	CT(16255)=(S=6017375,E=6017375,C=58)
	CT(16256)=(S=6017376,E=6017376,C=31)
	CT(16257)=(S=6017377,E=6017377,C=15)
	CT(16258)=(S=6017378,E=6017378,C=34)
	CT(16259)=(S=6017381,E=6017381,C=33)
	CT(16260)=(S=6017382,E=6017382,C=38)
	CT(16261)=(S=6017383,E=6017383,C=28)
	CT(16262)=(S=6017384,E=6017384,C=14)
	CT(16263)=(S=6017385,E=6017385,C=66)
	CT(16264)=(S=6017386,E=6017386,C=63)
	CT(16265)=(S=6017387,E=6017387,C=12)
	CT(16266)=(S=6017388,E=6017388,C=25)
	CT(16267)=(S=6017389,E=6017389,C=38)
	CT(16268)=(S=6017390,E=6017390,C=36)
	CT(16269)=(S=6017391,E=6017391,C=34)
	CT(16270)=(S=6017392,E=6017392,C=42)
	CT(16271)=(S=6017393,E=6017393,C=17)
	CT(16272)=(S=6017394,E=6017394,C=50)
	CT(16273)=(S=6017396,E=6017396,C=58)
	CT(16274)=(S=6017397,E=6017397,C=12)
	CT(16275)=(S=6017398,E=6017398,C=31)
	CT(16276)=(S=6017399,E=6017399,C=18)
	CT(16277)=(S=6017400,E=6017401,C=28)
	CT(16278)=(S=6017402,E=6017402,C=18)
	CT(16279)=(S=6017403,E=6017403,C=38)
	CT(16280)=(S=6017404,E=6017404,C=18)
	CT(16281)=(S=6017405,E=6017405,C=14)
	CT(16282)=(S=6017406,E=6017406,C=31)
	CT(16283)=(S=6017407,E=6017410,C=18)
	CT(16284)=(S=6017411,E=6017411,C=40)
	CT(16285)=(S=6017412,E=6017412,C=52)
	CT(16286)=(S=6017413,E=6017413,C=49)
	CT(16287)=(S=6017414,E=6017414,C=34)
	CT(16288)=(S=6017415,E=6017415,C=25)
	CT(16289)=(S=6017416,E=6017416,C=57)
	CT(16290)=(S=6017417,E=6017417,C=36)
	CT(16291)=(S=6017418,E=6017419,C=31)
	CT(16292)=(S=6017420,E=6017420,C=14)
	CT(16293)=(S=6017421,E=6017421,C=58)
	CT(16294)=(S=6017422,E=6017422,C=14)
	CT(16295)=(S=6017423,E=6017423,C=36)
	CT(16296)=(S=6017424,E=6017424,C=14)
	CT(16297)=(S=6017425,E=6017425,C=52)
	CT(16298)=(S=6017426,E=6017426,C=51)
	CT(16299)=(S=6017427,E=6017428,C=18)
	CT(16300)=(S=6017430,E=6017430,C=52)
	CT(16301)=(S=6017432,E=6017433,C=58)
	CT(16302)=(S=6017434,E=6017434,C=12)
	CT(16303)=(S=6017435,E=6017435,C=58)
	CT(16304)=(S=6017436,E=6017436,C=24)
	CT(16305)=(S=6017437,E=6017437,C=31)
	CT(16306)=(S=6017438,E=6017438,C=34)
	CT(16307)=(S=6017439,E=6017440,C=28)
	CT(16308)=(S=6017441,E=6017442,C=14)
	CT(16309)=(S=6017444,E=6017444,C=36)
	CT(16310)=(S=6017445,E=6017445,C=31)
	CT(16311)=(S=6017446,E=6017446,C=24)
	CT(16312)=(S=6017447,E=6017447,C=18)
	CT(16313)=(S=6017448,E=6017448,C=36)
	CT(16314)=(S=6017449,E=6017450,C=34)
	CT(16315)=(S=6017451,E=6017451,C=38)
	CT(16316)=(S=6017452,E=6017452,C=101)
	CT(16317)=(S=6017453,E=6017453,C=22)
	CT(16318)=(S=6017454,E=6017455,C=12)
	CT(16319)=(S=6017456,E=6017456,C=31)
	CT(16320)=(S=6017457,E=6017457,C=58)
	CT(16321)=(S=6017458,E=6017458,C=14)
	CT(16322)=(S=6017460,E=6017460,C=58)
	CT(16323)=(S=6017461,E=6017461,C=52)
	CT(16324)=(S=6017462,E=6017462,C=17)
	CT(16325)=(S=6017465,E=6017465,C=28)
	CT(16326)=(S=6017467,E=6017467,C=14)
	CT(16327)=(S=6017468,E=6017468,C=52)
	CT(16328)=(S=6017469,E=6017469,C=38)
	CT(16329)=(S=6017470,E=6017470,C=24)
	CT(16330)=(S=6017471,E=6017471,C=12)
	CT(16331)=(S=6017472,E=6017472,C=36)
	CT(16332)=(S=6017473,E=6017473,C=31)
	CT(16333)=(S=6017474,E=6017474,C=15)
	CT(16334)=(S=6017475,E=6017475,C=38)
	CT(16335)=(S=6017476,E=6017477,C=14)
	CT(16336)=(S=6017478,E=6017478,C=38)
	CT(16337)=(S=6017479,E=6017479,C=58)
	CT(16338)=(S=6017480,E=6017480,C=14)
	CT(16339)=(S=6017481,E=6017483,C=31)
	CT(16340)=(S=6017484,E=6017484,C=34)
	CT(16341)=(S=6017485,E=6017485,C=28)
	CT(16342)=(S=6017486,E=6017486,C=31)
	CT(16343)=(S=6017487,E=6017487,C=52)
	CT(16344)=(S=6017488,E=6017488,C=36)
	CT(16345)=(S=6017490,E=6017490,C=31)
	CT(16346)=(S=6017491,E=6017491,C=28)
	CT(16347)=(S=6017492,E=6017492,C=27)
	CT(16348)=(S=6017493,E=6017493,C=36)
	CT(16349)=(S=6017494,E=6017494,C=31)
	CT(16350)=(S=6017496,E=6017496,C=57)
	CT(16351)=(S=6017497,E=6017497,C=14)
	CT(16352)=(S=6017498,E=6017498,C=18)
	CT(16353)=(S=6017499,E=6017499,C=42)
	CT(16354)=(S=6017500,E=6017500,C=14)
	CT(16355)=(S=6017501,E=6017502,C=58)
	CT(16356)=(S=6017503,E=6017505,C=14)
	CT(16357)=(S=6017506,E=6017506,C=31)
	CT(16358)=(S=6017508,E=6017508,C=24)
	CT(16359)=(S=6017509,E=6017509,C=12)
	CT(16360)=(S=6017511,E=6017511,C=28)
	CT(16361)=(S=6017512,E=6017512,C=19)
	CT(16362)=(S=6017513,E=6017514,C=18)
	CT(16363)=(S=6017515,E=6017515,C=31)
	CT(16364)=(S=6017516,E=6017516,C=24)
	CT(16365)=(S=6017517,E=6017517,C=52)
	CT(16366)=(S=6017518,E=6017518,C=14)
	CT(16367)=(S=6017519,E=6017519,C=31)
	CT(16368)=(S=6017520,E=6017520,C=19)
	CT(16369)=(S=6017521,E=6017521,C=38)
	CT(16370)=(S=6017522,E=6017522,C=26)
	CT(16371)=(S=6017523,E=6017523,C=25)
	CT(16372)=(S=6017524,E=6017524,C=14)
	CT(16373)=(S=6017525,E=6017525,C=12)
	CT(16374)=(S=6017526,E=6017526,C=14)
	CT(16375)=(S=6017527,E=6017527,C=71)
	CT(16376)=(S=6017528,E=6017528,C=43)
	CT(16377)=(S=6017529,E=6017529,C=31)
	CT(16378)=(S=6017530,E=6017530,C=28)
	CT(16379)=(S=6017531,E=6017531,C=52)
	CT(16380)=(S=6017532,E=6017532,C=14)
	CT(16381)=(S=6017533,E=6017533,C=23)
	CT(16382)=(S=6017534,E=6017534,C=28)
	CT(16383)=(S=6017535,E=6017535,C=38)
	CT(16384)=(S=6017536,E=6017543,C=18)
	CT(16385)=(S=6017544,E=6017547,C=31)
	CT(16386)=(S=6017548,E=6017551,C=58)
	CT(16387)=(S=6017552,E=6017555,C=33)
	CT(16388)=(S=6017556,E=6017559,C=31)
	CT(16389)=(S=6017560,E=6017563,C=18)
	CT(16390)=(S=6017564,E=6017575,C=31)
	CT(16391)=(S=6017576,E=6017579,C=66)
	CT(16392)=(S=6017580,E=6017583,C=18)
	CT(16393)=(S=6017584,E=6017587,C=58)
	CT(16394)=(S=6017588,E=6017591,C=18)
	CT(16395)=(S=6017596,E=6017599,C=19)
	CT(16396)=(S=6017600,E=6017603,C=13)
	CT(16397)=(S=6017604,E=6017607,C=14)
	CT(16398)=(S=6017608,E=6017611,C=18)
	CT(16399)=(S=6017612,E=6017615,C=31)
	CT(16400)=(S=6017616,E=6017619,C=38)
	CT(16401)=(S=6017620,E=6017623,C=18)
	CT(16402)=(S=6017624,E=6017627,C=51)
	CT(16403)=(S=6017628,E=6017643,C=18)
	CT(16404)=(S=6017644,E=6017647,C=31)
	CT(16405)=(S=6017648,E=6017651,C=18)
	CT(16406)=(S=6017652,E=6017659,C=31)
	CT(16407)=(S=6017660,E=6017663,C=18)
	CT(16408)=(S=6017664,E=6017667,C=58)
	CT(16409)=(S=6017668,E=6017671,C=18)
	CT(16410)=(S=6017672,E=6017675,C=96)
	CT(16411)=(S=6017676,E=6017679,C=42)
	CT(16412)=(S=6017680,E=6017687,C=31)
	CT(16413)=(S=6017688,E=6017691,C=18)
	CT(16414)=(S=6017692,E=6017695,C=31)
	CT(16415)=(S=6017696,E=6017699,C=18)
	CT(16416)=(S=6017700,E=6017703,C=31)
	CT(16417)=(S=6017704,E=6017707,C=18)
	CT(16418)=(S=6017708,E=6017711,C=20)
	CT(16419)=(S=6017712,E=6017715,C=31)
	CT(16420)=(S=6017716,E=6017719,C=71)
	CT(16421)=(S=6017720,E=6017723,C=18)
	CT(16422)=(S=6017724,E=6017727,C=31)
	CT(16423)=(S=6017728,E=6017743,C=18)
	CT(16424)=(S=6017744,E=6017747,C=58)
	CT(16425)=(S=6017748,E=6017751,C=28)
	CT(16426)=(S=6017752,E=6017755,C=18)
	CT(16427)=(S=6017756,E=6017759,C=42)
	CT(16428)=(S=6017760,E=6017767,C=18)
	CT(16429)=(S=6017768,E=6017771,C=63)
	CT(16430)=(S=6017772,E=6017783,C=58)
	CT(16431)=(S=6017784,E=6017787,C=31)
	CT(16432)=(S=6017788,E=6017795,C=18)
	CT(16433)=(S=6017796,E=6017799,C=98)
	CT(16434)=(S=6017800,E=6017803,C=14)
	CT(16435)=(S=6017804,E=6017811,C=31)
	CT(16436)=(S=6017812,E=6017823,C=18)
	CT(16437)=(S=6017824,E=6017827,C=42)
	CT(16438)=(S=6017828,E=6017831,C=18)
	CT(16439)=(S=6017832,E=6017835,C=28)
	CT(16440)=(S=6017836,E=6017839,C=18)
	CT(16441)=(S=6017840,E=6017843,C=53)
	CT(16442)=(S=6017844,E=6017855,C=18)
	CT(16443)=(S=6017860,E=6017863,C=31)
	CT(16444)=(S=6017864,E=6017867,C=36)
	CT(16445)=(S=6017868,E=6017875,C=18)
	CT(16446)=(S=6017876,E=6017879,C=58)
	CT(16447)=(S=6017884,E=6017887,C=18)
	CT(16448)=(S=6017888,E=6017891,C=63)
	CT(16449)=(S=6017892,E=6017895,C=58)
	CT(16450)=(S=6017896,E=6017899,C=18)
	CT(16451)=(S=6017900,E=6017903,C=51)
	CT(16452)=(S=6017904,E=6017907,C=28)
	CT(16453)=(S=6017908,E=6017915,C=31)
	CT(16454)=(S=6017916,E=6017923,C=18)
	CT(16455)=(S=6017924,E=6017927,C=31)
	CT(16456)=(S=6017928,E=6017943,C=18)
	CT(16457)=(S=6017948,E=6017951,C=15)
	CT(16458)=(S=6017956,E=6017959,C=12)
	CT(16459)=(S=6017960,E=6017963,C=19)
	CT(16460)=(S=6017964,E=6017971,C=31)
	CT(16461)=(S=6017972,E=6017979,C=18)
	CT(16462)=(S=6017980,E=6017983,C=51)
	CT(16463)=(S=6017984,E=6017987,C=18)
	CT(16464)=(S=6017992,E=6017995,C=53)
	CT(16465)=(S=6017996,E=6017999,C=31)
	CT(16466)=(S=6018000,E=6018003,C=18)
	CT(16467)=(S=6018004,E=6018007,C=31)
	CT(16468)=(S=6018008,E=6018011,C=18)
	CT(16469)=(S=6018012,E=6018015,C=58)
	CT(16470)=(S=6018020,E=6018023,C=18)
	CT(16471)=(S=6018024,E=6018027,C=51)
	CT(16472)=(S=6018028,E=6018031,C=18)
	CT(16473)=(S=6018032,E=6018035,C=42)
	CT(16474)=(S=6018036,E=6018039,C=44)
	CT(16475)=(S=6018044,E=6018047,C=58)
	CT(16476)=(S=6018048,E=6018048,C=36)
	CT(16477)=(S=6018049,E=6018049,C=39)
	CT(16478)=(S=6018050,E=6018050,C=58)
	CT(16479)=(S=6018051,E=6018051,C=28)
	CT(16480)=(S=6018052,E=6018052,C=59)
	CT(16481)=(S=6018053,E=6018053,C=58)
	CT(16482)=(S=6018054,E=6018055,C=18)
	CT(16483)=(S=6018056,E=6018056,C=101)
	CT(16484)=(S=6018057,E=6018057,C=58)
	CT(16485)=(S=6018058,E=6018058,C=18)
	CT(16486)=(S=6018060,E=6018060,C=14)
	CT(16487)=(S=6018061,E=6018061,C=51)
	CT(16488)=(S=6018063,E=6018063,C=46)
	CT(16489)=(S=6018064,E=6018064,C=26)
	CT(16490)=(S=6018065,E=6018065,C=51)
	CT(16491)=(S=6018066,E=6018066,C=14)
	CT(16492)=(S=6018069,E=6018069,C=12)
	CT(16493)=(S=6018070,E=6018070,C=31)
	CT(16494)=(S=6018071,E=6018071,C=38)
	CT(16495)=(S=6018072,E=6018072,C=52)
	CT(16496)=(S=6018073,E=6018073,C=58)
	CT(16497)=(S=6018074,E=6018074,C=12)
	CT(16498)=(S=6018076,E=6018076,C=24)
	CT(16499)=(S=6018077,E=6018077,C=34)
	CT(16500)=(S=6018078,E=6018078,C=58)
	CT(16501)=(S=6018079,E=6018079,C=57)
	CT(16502)=(S=6018080,E=6018080,C=101)
	CT(16503)=(S=6018081,E=6018081,C=58)
	CT(16504)=(S=6018082,E=6018082,C=31)
	CT(16505)=(S=6018083,E=6018083,C=38)
	CT(16506)=(S=6018084,E=6018084,C=24)
	CT(16507)=(S=6018085,E=6018085,C=51)
	CT(16508)=(S=6018087,E=6018087,C=38)
	CT(16509)=(S=6018088,E=6018088,C=34)
	CT(16510)=(S=6018090,E=6018090,C=36)
	CT(16511)=(S=6018091,E=6018091,C=34)
	CT(16512)=(S=6018092,E=6018092,C=57)
	CT(16513)=(S=6018094,E=6018094,C=42)
	CT(16514)=(S=6018095,E=6018095,C=18)
	CT(16515)=(S=6018096,E=6018096,C=24)
	CT(16516)=(S=6018098,E=6018098,C=18)
	CT(16517)=(S=6018100,E=6018100,C=38)
	CT(16518)=(S=6018101,E=6018101,C=16)
	CT(16519)=(S=6018104,E=6018104,C=31)
	CT(16520)=(S=6018105,E=6018105,C=18)
	CT(16521)=(S=6018106,E=6018106,C=12)
	CT(16522)=(S=6018107,E=6018107,C=38)
	CT(16523)=(S=6018108,E=6018109,C=18)
	CT(16524)=(S=6018110,E=6018110,C=38)
	CT(16525)=(S=6018111,E=6018111,C=60)
	CT(16526)=(S=6018112,E=6018112,C=18)
	CT(16527)=(S=6018114,E=6018114,C=38)
	CT(16528)=(S=6018115,E=6018115,C=23)
	CT(16529)=(S=6018116,E=6018116,C=18)
	CT(16530)=(S=6018117,E=6018117,C=46)
	CT(16531)=(S=6018118,E=6018118,C=38)
	CT(16532)=(S=6018119,E=6018119,C=66)
	CT(16533)=(S=6018122,E=6018122,C=38)
	CT(16534)=(S=6018123,E=6018123,C=34)
	CT(16535)=(S=6018124,E=6018124,C=17)
	CT(16536)=(S=6018125,E=6018125,C=34)
	CT(16537)=(S=6018126,E=6018126,C=12)
	CT(16538)=(S=6018127,E=6018127,C=25)
	CT(16539)=(S=6018128,E=6018128,C=31)
	CT(16540)=(S=6018129,E=6018130,C=18)
	CT(16541)=(S=6018131,E=6018131,C=28)
	CT(16542)=(S=6018132,E=6018132,C=58)
	CT(16543)=(S=6018133,E=6018133,C=28)
	CT(16544)=(S=6018134,E=6018134,C=58)
	CT(16545)=(S=6018135,E=6018135,C=28)
	CT(16546)=(S=6018136,E=6018136,C=63)
	CT(16547)=(S=6018137,E=6018137,C=98)
	CT(16548)=(S=6018138,E=6018138,C=14)
	CT(16549)=(S=6018139,E=6018139,C=34)
	CT(16550)=(S=6018140,E=6018140,C=27)
	CT(16551)=(S=6018141,E=6018142,C=14)
	CT(16552)=(S=6018143,E=6018143,C=28)
	CT(16553)=(S=6018144,E=6018144,C=36)
	CT(16554)=(S=6018145,E=6018145,C=18)
	CT(16555)=(S=6018146,E=6018146,C=49)
	CT(16556)=(S=6018147,E=6018147,C=36)
	CT(16557)=(S=6018149,E=6018149,C=38)
	CT(16558)=(S=6018150,E=6018150,C=43)
	CT(16559)=(S=6018151,E=6018151,C=19)
	CT(16560)=(S=6018152,E=6018152,C=18)
	CT(16561)=(S=6018153,E=6018153,C=14)
	CT(16562)=(S=6018154,E=6018154,C=18)
	CT(16563)=(S=6018156,E=6018156,C=14)
	CT(16564)=(S=6018157,E=6018157,C=34)
	CT(16565)=(S=6018158,E=6018159,C=38)
	CT(16566)=(S=6018160,E=6018160,C=71)
	CT(16567)=(S=6018161,E=6018161,C=28)
	CT(16568)=(S=6018162,E=6018162,C=17)
	CT(16569)=(S=6018163,E=6018163,C=14)
	CT(16570)=(S=6018164,E=6018164,C=12)
	CT(16571)=(S=6018165,E=6018165,C=34)
	CT(16572)=(S=6018166,E=6018166,C=18)
	CT(16573)=(S=6018167,E=6018167,C=12)
	CT(16574)=(S=6018168,E=6018168,C=38)
	CT(16575)=(S=6018169,E=6018169,C=34)
	CT(16576)=(S=6018170,E=6018170,C=54)
	CT(16577)=(S=6018171,E=6018171,C=18)
	CT(16578)=(S=6018172,E=6018173,C=31)
	CT(16579)=(S=6018174,E=6018174,C=37)
	CT(16580)=(S=6018175,E=6018175,C=58)
	CT(16581)=(S=6018176,E=6018176,C=12)
	CT(16582)=(S=6018177,E=6018177,C=24)
	CT(16583)=(S=6018178,E=6018178,C=28)
	CT(16584)=(S=6018179,E=6018179,C=14)
	CT(16585)=(S=6018180,E=6018180,C=13)
	CT(16586)=(S=6018181,E=6018181,C=18)
	CT(16587)=(S=6018182,E=6018182,C=52)
	CT(16588)=(S=6018184,E=6018184,C=42)
	CT(16589)=(S=6018185,E=6018185,C=18)
	CT(16590)=(S=6018186,E=6018186,C=14)
	CT(16591)=(S=6018187,E=6018188,C=36)
	CT(16592)=(S=6018191,E=6018191,C=38)
	CT(16593)=(S=6018192,E=6018192,C=28)
	CT(16594)=(S=6018193,E=6018193,C=32)
	CT(16595)=(S=6018194,E=6018194,C=18)
	CT(16596)=(S=6018195,E=6018195,C=12)
	CT(16597)=(S=6018196,E=6018196,C=38)
	CT(16598)=(S=6018197,E=6018199,C=18)
	CT(16599)=(S=6018200,E=6018200,C=12)
	CT(16600)=(S=6018202,E=6018202,C=31)
	CT(16601)=(S=6018203,E=6018203,C=58)
	CT(16602)=(S=6018204,E=6018204,C=18)
	CT(16603)=(S=6018205,E=6018205,C=12)
	CT(16604)=(S=6018206,E=6018206,C=18)
	CT(16605)=(S=6018207,E=6018207,C=38)
	CT(16606)=(S=6018208,E=6018208,C=14)
	CT(16607)=(S=6018209,E=6018210,C=38)
	CT(16608)=(S=6018211,E=6018211,C=51)
	CT(16609)=(S=6018212,E=6018212,C=38)
	CT(16610)=(S=6018213,E=6018213,C=58)
	CT(16611)=(S=6018214,E=6018214,C=14)
	CT(16612)=(S=6018215,E=6018215,C=15)
	CT(16613)=(S=6018216,E=6018216,C=18)
	CT(16614)=(S=6018217,E=6018218,C=58)
	CT(16615)=(S=6018219,E=6018219,C=18)
	CT(16616)=(S=6018220,E=6018220,C=58)
	CT(16617)=(S=6018221,E=6018222,C=31)
	CT(16618)=(S=6018223,E=6018223,C=25)
	CT(16619)=(S=6018224,E=6018224,C=36)
	CT(16620)=(S=6018225,E=6018225,C=31)
	CT(16621)=(S=6018226,E=6018226,C=37)
	CT(16622)=(S=6018227,E=6018227,C=18)
	CT(16623)=(S=6018228,E=6018228,C=98)
	CT(16624)=(S=6018229,E=6018229,C=28)
	CT(16625)=(S=6018230,E=6018230,C=14)
	CT(16626)=(S=6018231,E=6018232,C=36)
	CT(16627)=(S=6018233,E=6018234,C=49)
	CT(16628)=(S=6018235,E=6018235,C=42)
	CT(16629)=(S=6018236,E=6018236,C=19)
	CT(16630)=(S=6018237,E=6018237,C=17)
	CT(16631)=(S=6018238,E=6018240,C=38)
	CT(16632)=(S=6018241,E=6018241,C=20)
	CT(16633)=(S=6018242,E=6018242,C=31)
	CT(16634)=(S=6018243,E=6018243,C=18)
	CT(16635)=(S=6018244,E=6018244,C=34)
	CT(16636)=(S=6018245,E=6018245,C=18)
	CT(16637)=(S=6018246,E=6018246,C=60)
	CT(16638)=(S=6018247,E=6018247,C=52)
	CT(16639)=(S=6018248,E=6018248,C=34)
	CT(16640)=(S=6018249,E=6018249,C=51)
	CT(16641)=(S=6018250,E=6018250,C=12)
	CT(16642)=(S=6018251,E=6018251,C=31)
	CT(16643)=(S=6018252,E=6018252,C=28)
	CT(16644)=(S=6018253,E=6018253,C=12)
	CT(16645)=(S=6018254,E=6018254,C=28)
	CT(16646)=(S=6018255,E=6018255,C=19)
	CT(16647)=(S=6018256,E=6018256,C=52)
	CT(16648)=(S=6018257,E=6018257,C=19)
	CT(16649)=(S=6018258,E=6018258,C=14)
	CT(16650)=(S=6018259,E=6018259,C=12)
	CT(16651)=(S=6018260,E=6018260,C=14)
	CT(16652)=(S=6018261,E=6018261,C=40)
	CT(16653)=(S=6018262,E=6018262,C=31)
	CT(16654)=(S=6018263,E=6018263,C=36)
	CT(16655)=(S=6018264,E=6018264,C=14)
	CT(16656)=(S=6018265,E=6018266,C=18)
	CT(16657)=(S=6018267,E=6018267,C=15)
	CT(16658)=(S=6018268,E=6018269,C=18)
	CT(16659)=(S=6018270,E=6018270,C=14)
	CT(16660)=(S=6018271,E=6018271,C=58)
	CT(16661)=(S=6018272,E=6018272,C=24)
	CT(16662)=(S=6018273,E=6018273,C=40)
	CT(16663)=(S=6018275,E=6018275,C=38)
	CT(16664)=(S=6018276,E=6018276,C=19)
	CT(16665)=(S=6018277,E=6018277,C=14)
	CT(16666)=(S=6018278,E=6018278,C=18)
	CT(16667)=(S=6018280,E=6018280,C=44)
	CT(16668)=(S=6018281,E=6018281,C=51)
	CT(16669)=(S=6018282,E=6018282,C=34)
	CT(16670)=(S=6018283,E=6018283,C=51)
	CT(16671)=(S=6018284,E=6018284,C=12)
	CT(16672)=(S=6018285,E=6018285,C=36)
	CT(16673)=(S=6018286,E=6018286,C=12)
	CT(16674)=(S=6018287,E=6018287,C=18)
	CT(16675)=(S=6018289,E=6018289,C=15)
	CT(16676)=(S=6018290,E=6018290,C=58)
	CT(16677)=(S=6018291,E=6018291,C=28)
	CT(16678)=(S=6018292,E=6018292,C=55)
	CT(16679)=(S=6018293,E=6018293,C=28)
	CT(16680)=(S=6018295,E=6018295,C=57)
	CT(16681)=(S=6018296,E=6018296,C=31)
	CT(16682)=(S=6018297,E=6018297,C=15)
	CT(16683)=(S=6018298,E=6018298,C=50)
	CT(16684)=(S=6018299,E=6018299,C=52)
	CT(16685)=(S=6018300,E=6018300,C=26)
	CT(16686)=(S=6018301,E=6018301,C=31)
	CT(16687)=(S=6018302,E=6018302,C=38)
	CT(16688)=(S=6018303,E=6018303,C=12)
	CT(16689)=(S=6018304,E=6018304,C=19)
	CT(16690)=(S=6018305,E=6018305,C=37)
	CT(16691)=(S=6018306,E=6018306,C=34)
	CT(16692)=(S=6018307,E=6018307,C=14)
	CT(16693)=(S=6018308,E=6018308,C=34)
	CT(16694)=(S=6018309,E=6018309,C=18)
	CT(16695)=(S=6018310,E=6018310,C=42)
	CT(16696)=(S=6018311,E=6018311,C=15)
	CT(16697)=(S=6018312,E=6018312,C=31)
	CT(16698)=(S=6018313,E=6018313,C=50)
	CT(16699)=(S=6018314,E=6018314,C=33)
	CT(16700)=(S=6018315,E=6018315,C=38)
	CT(16701)=(S=6018316,E=6018316,C=51)
	CT(16702)=(S=6018317,E=6018317,C=36)
	CT(16703)=(S=6018318,E=6018318,C=18)
	CT(16704)=(S=6018319,E=6018319,C=12)
	CT(16705)=(S=6018320,E=6018320,C=38)
	CT(16706)=(S=6018321,E=6018321,C=24)
	CT(16707)=(S=6018322,E=6018322,C=23)
	CT(16708)=(S=6018323,E=6018323,C=38)
	CT(16709)=(S=6018324,E=6018326,C=28)
	CT(16710)=(S=6018327,E=6018327,C=18)
	CT(16711)=(S=6018328,E=6018328,C=34)
	CT(16712)=(S=6018329,E=6018330,C=12)
	CT(16713)=(S=6018331,E=6018331,C=28)
	CT(16714)=(S=6018332,E=6018332,C=14)
	CT(16715)=(S=6018333,E=6018333,C=18)
	CT(16716)=(S=6018334,E=6018334,C=25)
	CT(16717)=(S=6018335,E=6018335,C=98)
	CT(16718)=(S=6018336,E=6018336,C=34)
	CT(16719)=(S=6018337,E=6018337,C=31)
	CT(16720)=(S=6018338,E=6018338,C=38)
	CT(16721)=(S=6018340,E=6018340,C=28)
	CT(16722)=(S=6018341,E=6018341,C=36)
	CT(16723)=(S=6018342,E=6018342,C=58)
	CT(16724)=(S=6018343,E=6018343,C=18)
	CT(16725)=(S=6018344,E=6018344,C=50)
	CT(16726)=(S=6018345,E=6018345,C=12)
	CT(16727)=(S=6018346,E=6018346,C=14)
	CT(16728)=(S=6018347,E=6018347,C=55)
	CT(16729)=(S=6018348,E=6018348,C=18)
	CT(16730)=(S=6018349,E=6018349,C=14)
	CT(16731)=(S=6018350,E=6018350,C=25)
	CT(16732)=(S=6018351,E=6018351,C=27)
	CT(16733)=(S=6018352,E=6018352,C=28)
	CT(16734)=(S=6018353,E=6018353,C=63)
	CT(16735)=(S=6018355,E=6018355,C=18)
	CT(16736)=(S=6018356,E=6018356,C=22)
	CT(16737)=(S=6018357,E=6018357,C=36)
	CT(16738)=(S=6018358,E=6018358,C=42)
	CT(16739)=(S=6018359,E=6018359,C=38)
	CT(16740)=(S=6018360,E=6018360,C=28)
	CT(16741)=(S=6018361,E=6018361,C=38)
	CT(16742)=(S=6018362,E=6018362,C=58)
	CT(16743)=(S=6018363,E=6018363,C=31)
	CT(16744)=(S=6018364,E=6018365,C=28)
	CT(16745)=(S=6018366,E=6018366,C=31)
	CT(16746)=(S=6018367,E=6018367,C=14)
	CT(16747)=(S=6018368,E=6018368,C=58)
	CT(16748)=(S=6018369,E=6018369,C=21)
	CT(16749)=(S=6018370,E=6018370,C=24)
	CT(16750)=(S=6018371,E=6018371,C=18)
	CT(16751)=(S=6018372,E=6018372,C=14)
	CT(16752)=(S=6018373,E=6018373,C=36)
	CT(16753)=(S=6018374,E=6018374,C=24)
	CT(16754)=(S=6018375,E=6018375,C=58)
	CT(16755)=(S=6018377,E=6018377,C=63)
	CT(16756)=(S=6018380,E=6018380,C=38)
	CT(16757)=(S=6018381,E=6018381,C=42)
	CT(16758)=(S=6018382,E=6018382,C=36)
	CT(16759)=(S=6018383,E=6018383,C=12)
	CT(16760)=(S=6018384,E=6018384,C=52)
	CT(16761)=(S=6018385,E=6018385,C=14)
	CT(16762)=(S=6018386,E=6018386,C=12)
	CT(16763)=(S=6018387,E=6018387,C=42)
	CT(16764)=(S=6018388,E=6018388,C=57)
	CT(16765)=(S=6018389,E=6018389,C=36)
	CT(16766)=(S=6018390,E=6018390,C=53)
	CT(16767)=(S=6018391,E=6018391,C=18)
	CT(16768)=(S=6018392,E=6018392,C=58)
	CT(16769)=(S=6018393,E=6018393,C=52)
	CT(16770)=(S=6018394,E=6018394,C=12)
	CT(16771)=(S=6018395,E=6018395,C=28)
	CT(16772)=(S=6018396,E=6018398,C=31)
	CT(16773)=(S=6018399,E=6018399,C=36)
	CT(16774)=(S=6018400,E=6018400,C=58)
	CT(16775)=(S=6018401,E=6018401,C=47)
	CT(16776)=(S=6018402,E=6018403,C=28)
	CT(16777)=(S=6018404,E=6018404,C=34)
	CT(16778)=(S=6018405,E=6018405,C=38)
	CT(16779)=(S=6018406,E=6018406,C=12)
	CT(16780)=(S=6018407,E=6018407,C=28)
	CT(16781)=(S=6018408,E=6018408,C=18)
	CT(16782)=(S=6018409,E=6018410,C=60)
	CT(16783)=(S=6018411,E=6018412,C=58)
	CT(16784)=(S=6018413,E=6018413,C=18)
	CT(16785)=(S=6018414,E=6018414,C=14)
	CT(16786)=(S=6018416,E=6018416,C=36)
	CT(16787)=(S=6018417,E=6018418,C=18)
	CT(16788)=(S=6018419,E=6018419,C=36)
	CT(16789)=(S=6018420,E=6018420,C=28)
	CT(16790)=(S=6018422,E=6018422,C=58)
	CT(16791)=(S=6018423,E=6018425,C=31)
	CT(16792)=(S=6018426,E=6018426,C=33)
	CT(16793)=(S=6018427,E=6018428,C=58)
	CT(16794)=(S=6018429,E=6018429,C=31)
	CT(16795)=(S=6018430,E=6018430,C=18)
	CT(16796)=(S=6018431,E=6018431,C=27)
	CT(16797)=(S=6018432,E=6018432,C=34)
	CT(16798)=(S=6018433,E=6018433,C=15)
	CT(16799)=(S=6018434,E=6018434,C=18)
	CT(16800)=(S=6018435,E=6018435,C=52)
	CT(16801)=(S=6018436,E=6018436,C=42)
	CT(16802)=(S=6018437,E=6018437,C=34)
	CT(16803)=(S=6018438,E=6018438,C=36)
	CT(16804)=(S=6018439,E=6018439,C=38)
	CT(16805)=(S=6018440,E=6018440,C=34)
	CT(16806)=(S=6018441,E=6018441,C=58)
	CT(16807)=(S=6018443,E=6018443,C=18)
	CT(16808)=(S=6018444,E=6018444,C=14)
	CT(16809)=(S=6018445,E=6018446,C=18)
	CT(16810)=(S=6018447,E=6018447,C=33)
	CT(16811)=(S=6018448,E=6018448,C=18)
	CT(16812)=(S=6018449,E=6018450,C=12)
	CT(16813)=(S=6018451,E=6018451,C=52)
	CT(16814)=(S=6018453,E=6018453,C=31)
	CT(16815)=(S=6018454,E=6018454,C=50)
	CT(16816)=(S=6018456,E=6018456,C=34)
	CT(16817)=(S=6018457,E=6018457,C=15)
	CT(16818)=(S=6018458,E=6018458,C=31)
	CT(16819)=(S=6018459,E=6018459,C=19)
	CT(16820)=(S=6018460,E=6018460,C=58)
	CT(16821)=(S=6018461,E=6018461,C=176)
	CT(16822)=(S=6018462,E=6018462,C=18)
	CT(16823)=(S=6018463,E=6018463,C=34)
	CT(16824)=(S=6018464,E=6018464,C=33)
	CT(16825)=(S=6018465,E=6018465,C=47)
	CT(16826)=(S=6018466,E=6018466,C=58)
	CT(16827)=(S=6018467,E=6018467,C=18)
	CT(16828)=(S=6018468,E=6018468,C=14)
	CT(16829)=(S=6018469,E=6018469,C=36)
	CT(16830)=(S=6018470,E=6018473,C=18)
	CT(16831)=(S=6018474,E=6018474,C=58)
	CT(16832)=(S=6018475,E=6018475,C=34)
	CT(16833)=(S=6018476,E=6018476,C=18)
	CT(16834)=(S=6018477,E=6018477,C=14)
	CT(16835)=(S=6018479,E=6018479,C=31)
	CT(16836)=(S=6018480,E=6018480,C=17)
	CT(16837)=(S=6018481,E=6018481,C=28)
	CT(16838)=(S=6018482,E=6018482,C=36)
	CT(16839)=(S=6018483,E=6018483,C=19)
	CT(16840)=(S=6018484,E=6018484,C=12)
	CT(16841)=(S=6018485,E=6018485,C=53)
	CT(16842)=(S=6018486,E=6018486,C=31)
	CT(16843)=(S=6018487,E=6018487,C=14)
	CT(16844)=(S=6018488,E=6018488,C=18)
	CT(16845)=(S=6018489,E=6018489,C=31)
	CT(16846)=(S=6018490,E=6018490,C=18)
	CT(16847)=(S=6018491,E=6018491,C=31)
	CT(16848)=(S=6018492,E=6018492,C=14)
	CT(16849)=(S=6018493,E=6018493,C=18)
	CT(16850)=(S=6018494,E=6018494,C=36)
	CT(16851)=(S=6018495,E=6018495,C=35)
	CT(16852)=(S=6018496,E=6018496,C=31)
	CT(16853)=(S=6018497,E=6018497,C=58)
	CT(16854)=(S=6018498,E=6018498,C=34)
	CT(16855)=(S=6018499,E=6018499,C=36)
	CT(16856)=(S=6018500,E=6018500,C=18)
	CT(16857)=(S=6018501,E=6018501,C=15)
	CT(16858)=(S=6018502,E=6018502,C=28)
	CT(16859)=(S=6018503,E=6018506,C=14)
	CT(16860)=(S=6018507,E=6018507,C=27)
	CT(16861)=(S=6018508,E=6018508,C=42)
	CT(16862)=(S=6018509,E=6018509,C=23)
	CT(16863)=(S=6018510,E=6018510,C=58)
	CT(16864)=(S=6018511,E=6018511,C=28)
	CT(16865)=(S=6018512,E=6018512,C=19)
	CT(16866)=(S=6018513,E=6018513,C=31)
	CT(16867)=(S=6018514,E=6018514,C=15)
	CT(16868)=(S=6018515,E=6018515,C=33)
	CT(16869)=(S=6018516,E=6018516,C=14)
	CT(16870)=(S=6018517,E=6018517,C=23)
	CT(16871)=(S=6018518,E=6018518,C=36)
	CT(16872)=(S=6018519,E=6018519,C=38)
	CT(16873)=(S=6018520,E=6018520,C=14)
	CT(16874)=(S=6018522,E=6018523,C=36)
	CT(16875)=(S=6018525,E=6018525,C=28)
	CT(16876)=(S=6018526,E=6018526,C=42)
	CT(16877)=(S=6018528,E=6018528,C=27)
	CT(16878)=(S=6018529,E=6018529,C=14)
	CT(16879)=(S=6018530,E=6018530,C=58)
	CT(16880)=(S=6018531,E=6018531,C=34)
	CT(16881)=(S=6018532,E=6018532,C=22)
	CT(16882)=(S=6018533,E=6018533,C=12)
	CT(16883)=(S=6018534,E=6018534,C=14)
	CT(16884)=(S=6018535,E=6018535,C=31)
	CT(16885)=(S=6018536,E=6018536,C=28)
	CT(16886)=(S=6018537,E=6018537,C=62)
	CT(16887)=(S=6018538,E=6018539,C=42)
	CT(16888)=(S=6018540,E=6018540,C=12)
	CT(16889)=(S=6018541,E=6018543,C=58)
	CT(16890)=(S=6018544,E=6018544,C=18)
	CT(16891)=(S=6018545,E=6018545,C=52)
	CT(16892)=(S=6018546,E=6018546,C=12)
	CT(16893)=(S=6018547,E=6018547,C=18)
	CT(16894)=(S=6018548,E=6018548,C=33)
	CT(16895)=(S=6018549,E=6018549,C=37)
	CT(16896)=(S=6018550,E=6018550,C=19)
	CT(16897)=(S=6018551,E=6018551,C=18)
	CT(16898)=(S=6018552,E=6018552,C=98)
	CT(16899)=(S=6018553,E=6018553,C=18)
	CT(16900)=(S=6018554,E=6018554,C=19)
	CT(16901)=(S=6018555,E=6018555,C=49)
	CT(16902)=(S=6018556,E=6018556,C=28)
	CT(16903)=(S=6018557,E=6018558,C=37)
	CT(16904)=(S=6018559,E=6018563,C=58)
	CT(16905)=(S=6018564,E=6018567,C=14)
	CT(16906)=(S=6018568,E=6018571,C=28)
	CT(16907)=(S=6018572,E=6018575,C=38)
	CT(16908)=(S=6018576,E=6018579,C=31)
	CT(16909)=(S=6018580,E=6018583,C=24)
	CT(16910)=(S=6018584,E=6018587,C=58)
	CT(16911)=(S=6018588,E=6018591,C=31)
	CT(16912)=(S=6018596,E=6018599,C=58)
	CT(16913)=(S=6018600,E=6018603,C=18)
	CT(16914)=(S=6018604,E=6018607,C=70)
	CT(16915)=(S=6018608,E=6018611,C=31)
	CT(16916)=(S=6018612,E=6018615,C=58)
	CT(16917)=(S=6018616,E=6018619,C=18)
	CT(16918)=(S=6018620,E=6018623,C=15)
	CT(16919)=(S=6018628,E=6018631,C=18)
	CT(16920)=(S=6018632,E=6018635,C=15)
	CT(16921)=(S=6018636,E=6018639,C=18)
	CT(16922)=(S=6018640,E=6018647,C=31)
	CT(16923)=(S=6018648,E=6018651,C=19)
	CT(16924)=(S=6018656,E=6018659,C=18)
	CT(16925)=(S=6018660,E=6018663,C=17)
	CT(16926)=(S=6018664,E=6018667,C=57)
	CT(16927)=(S=6018668,E=6018671,C=36)
	CT(16928)=(S=6018672,E=6018675,C=31)
	CT(16929)=(S=6018676,E=6018683,C=18)
	CT(16930)=(S=6018684,E=6018699,C=31)
	CT(16931)=(S=6018700,E=6018711,C=18)
	CT(16932)=(S=6018712,E=6018715,C=101)
	CT(16933)=(S=6018716,E=6018719,C=33)
	CT(16934)=(S=6018720,E=6018723,C=31)
	CT(16935)=(S=6018724,E=6018727,C=47)
	CT(16936)=(S=6018728,E=6018731,C=34)
	CT(16937)=(S=6018732,E=6018735,C=18)
	CT(16938)=(S=6018736,E=6018743,C=31)
	CT(16939)=(S=6018744,E=6018747,C=18)
	CT(16940)=(S=6018748,E=6018751,C=38)
	CT(16941)=(S=6018752,E=6018755,C=33)
	CT(16942)=(S=6018756,E=6018759,C=18)
	CT(16943)=(S=6018760,E=6018763,C=53)
	CT(16944)=(S=6018764,E=6018767,C=18)
	CT(16945)=(S=6018768,E=6018771,C=31)
	CT(16946)=(S=6018772,E=6018787,C=18)
	CT(16947)=(S=6018788,E=6018791,C=14)
	CT(16948)=(S=6018792,E=6018795,C=31)
	CT(16949)=(S=6018796,E=6018799,C=58)
	CT(16950)=(S=6018800,E=6018803,C=18)
	CT(16951)=(S=6018804,E=6018811,C=31)
	CT(16952)=(S=6018812,E=6018815,C=28)
	CT(16953)=(S=6018816,E=6018819,C=58)
	CT(16954)=(S=6018820,E=6018823,C=36)
	CT(16955)=(S=6018824,E=6018827,C=31)
	CT(16956)=(S=6018828,E=6018831,C=33)
	CT(16957)=(S=6018832,E=6018839,C=18)
	CT(16958)=(S=6018840,E=6018843,C=31)
	CT(16959)=(S=6018844,E=6018847,C=18)
	CT(16960)=(S=6018848,E=6018851,C=58)
	CT(16961)=(S=6018852,E=6018859,C=18)
	CT(16962)=(S=6018860,E=6018863,C=58)
	CT(16963)=(S=6018864,E=6018867,C=18)
	CT(16964)=(S=6018868,E=6018879,C=31)
	CT(16965)=(S=6018880,E=6018883,C=25)
	CT(16966)=(S=6018884,E=6018887,C=31)
	CT(16967)=(S=6018888,E=6018891,C=28)
	CT(16968)=(S=6018892,E=6018895,C=18)
	CT(16969)=(S=6018896,E=6018899,C=58)
	CT(16970)=(S=6018900,E=6018907,C=18)
	CT(16971)=(S=6018908,E=6018911,C=19)
	CT(16972)=(S=6018912,E=6018915,C=38)
	CT(16973)=(S=6018916,E=6018919,C=28)
	CT(16974)=(S=6018920,E=6018923,C=58)
	CT(16975)=(S=6018924,E=6018931,C=18)
	CT(16976)=(S=6018932,E=6018935,C=28)
	CT(16977)=(S=6018936,E=6018939,C=18)
	CT(16978)=(S=6018940,E=6018943,C=31)
	CT(16979)=(S=6018944,E=6018947,C=18)
	CT(16980)=(S=6018948,E=6018951,C=47)
	CT(16981)=(S=6018952,E=6018959,C=18)
	CT(16982)=(S=6018960,E=6018963,C=31)
	CT(16983)=(S=6018964,E=6018967,C=36)
	CT(16984)=(S=6018968,E=6018971,C=18)
	CT(16985)=(S=6018972,E=6018975,C=36)
	CT(16986)=(S=6018976,E=6018979,C=24)
	CT(16987)=(S=6018980,E=6018983,C=14)
	CT(16988)=(S=6018984,E=6018987,C=18)
	CT(16989)=(S=6018988,E=6018991,C=20)
	CT(16990)=(S=6018992,E=6018995,C=68)
	CT(16991)=(S=6018996,E=6019003,C=14)
	CT(16992)=(S=6019004,E=6019031,C=18)
	CT(16993)=(S=6019032,E=6019035,C=51)
	CT(16994)=(S=6019036,E=6019043,C=18)
	CT(16995)=(S=6019044,E=6019047,C=58)
	CT(16996)=(S=6019048,E=6019051,C=18)
	CT(16997)=(S=6019052,E=6019055,C=58)
	CT(16998)=(S=6019056,E=6019059,C=42)
	CT(16999)=(S=6019060,E=6019063,C=18)
	CT(17000)=(S=6019064,E=6019067,C=31)
	CT(17001)=(S=6019068,E=6019071,C=18)
	CT(17002)=(S=6019072,E=6019072,C=19)
	CT(17003)=(S=6019073,E=6019074,C=60)
	CT(17004)=(S=6019075,E=6019075,C=18)
	CT(17005)=(S=6019076,E=6019076,C=26)
	CT(17006)=(S=6019077,E=6019077,C=58)
	CT(17007)=(S=6019078,E=6019078,C=63)
	CT(17008)=(S=6019079,E=6019079,C=19)
	CT(17009)=(S=6019080,E=6019080,C=38)
	CT(17010)=(S=6019081,E=6019081,C=14)
	CT(17011)=(S=6019082,E=6019082,C=49)
	CT(17012)=(S=6019084,E=6019084,C=25)
	CT(17013)=(S=6019085,E=6019085,C=28)
	CT(17014)=(S=6019086,E=6019086,C=24)
	CT(17015)=(S=6019087,E=6019087,C=31)
	CT(17016)=(S=6019088,E=6019088,C=58)
	CT(17017)=(S=6019089,E=6019089,C=15)
	CT(17018)=(S=6019090,E=6019090,C=14)
	CT(17019)=(S=6019091,E=6019091,C=12)
	CT(17020)=(S=6019092,E=6019092,C=49)
	CT(17021)=(S=6019093,E=6019093,C=53)
	CT(17022)=(S=6019094,E=6019094,C=31)
	CT(17023)=(S=6019095,E=6019095,C=28)
	CT(17024)=(S=6019096,E=6019096,C=19)
	CT(17025)=(S=6019097,E=6019097,C=58)
	CT(17026)=(S=6019098,E=6019098,C=12)
	CT(17027)=(S=6019099,E=6019099,C=38)
	CT(17028)=(S=6019100,E=6019100,C=31)
	CT(17029)=(S=6019101,E=6019101,C=12)
	CT(17030)=(S=6019102,E=6019102,C=58)
	CT(17031)=(S=6019103,E=6019103,C=17)
	CT(17032)=(S=6019104,E=6019104,C=19)
	CT(17033)=(S=6019105,E=6019105,C=28)
	CT(17034)=(S=6019106,E=6019106,C=36)
	CT(17035)=(S=6019107,E=6019107,C=28)
	CT(17036)=(S=6019108,E=6019108,C=34)
	CT(17037)=(S=6019109,E=6019109,C=14)
	CT(17038)=(S=6019110,E=6019110,C=15)
	CT(17039)=(S=6019112,E=6019112,C=14)
	CT(17040)=(S=6019113,E=6019113,C=49)
	CT(17041)=(S=6019114,E=6019114,C=14)
	CT(17042)=(S=6019115,E=6019115,C=12)
	CT(17043)=(S=6019116,E=6019116,C=50)
	CT(17044)=(S=6019117,E=6019117,C=28)
	CT(17045)=(S=6019118,E=6019118,C=18)
	CT(17046)=(S=6019119,E=6019119,C=38)
	CT(17047)=(S=6019120,E=6019122,C=18)
	CT(17048)=(S=6019123,E=6019123,C=58)
	CT(17049)=(S=6019124,E=6019124,C=38)
	CT(17050)=(S=6019125,E=6019125,C=18)
	CT(17051)=(S=6019126,E=6019126,C=52)
	CT(17052)=(S=6019127,E=6019127,C=14)
	CT(17053)=(S=6019128,E=6019128,C=13)
	CT(17054)=(S=6019129,E=6019129,C=14)
	CT(17055)=(S=6019130,E=6019130,C=12)
	CT(17056)=(S=6019131,E=6019131,C=18)
	CT(17057)=(S=6019132,E=6019132,C=50)
	CT(17058)=(S=6019133,E=6019133,C=31)
	CT(17059)=(S=6019134,E=6019134,C=19)
	CT(17060)=(S=6019135,E=6019135,C=25)
	CT(17061)=(S=6019136,E=6019136,C=58)
	CT(17062)=(S=6019137,E=6019137,C=19)
	CT(17063)=(S=6019138,E=6019138,C=18)
	CT(17064)=(S=6019139,E=6019139,C=12)
	CT(17065)=(S=6019140,E=6019141,C=18)
	CT(17066)=(S=6019142,E=6019142,C=50)
	CT(17067)=(S=6019143,E=6019143,C=51)
	CT(17068)=(S=6019144,E=6019144,C=18)
	CT(17069)=(S=6019145,E=6019145,C=60)
	CT(17070)=(S=6019146,E=6019146,C=52)
	CT(17071)=(S=6019147,E=6019147,C=38)
	CT(17072)=(S=6019148,E=6019148,C=22)
	CT(17073)=(S=6019149,E=6019149,C=47)
	CT(17074)=(S=6019150,E=6019150,C=38)
	CT(17075)=(S=6019151,E=6019151,C=34)
	CT(17076)=(S=6019152,E=6019152,C=38)
	CT(17077)=(S=6019153,E=6019153,C=18)
	CT(17078)=(S=6019154,E=6019154,C=14)
	CT(17079)=(S=6019155,E=6019157,C=12)
	CT(17080)=(S=6019158,E=6019158,C=55)
	CT(17081)=(S=6019159,E=6019159,C=52)
	CT(17082)=(S=6019160,E=6019161,C=14)
	CT(17083)=(S=6019162,E=6019162,C=28)
	CT(17084)=(S=6019163,E=6019163,C=37)
	CT(17085)=(S=6019164,E=6019164,C=42)
	CT(17086)=(S=6019165,E=6019165,C=14)
	CT(17087)=(S=6019166,E=6019166,C=31)
	CT(17088)=(S=6019167,E=6019167,C=51)
	CT(17089)=(S=6019168,E=6019168,C=18)
	CT(17090)=(S=6019169,E=6019169,C=53)
	CT(17091)=(S=6019170,E=6019170,C=37)
	CT(17092)=(S=6019171,E=6019171,C=58)
	CT(17093)=(S=6019172,E=6019172,C=36)
	CT(17094)=(S=6019173,E=6019173,C=24)
	CT(17095)=(S=6019174,E=6019174,C=15)
	CT(17096)=(S=6019175,E=6019175,C=24)
	CT(17097)=(S=6019176,E=6019176,C=36)
	CT(17098)=(S=6019177,E=6019177,C=14)
	CT(17099)=(S=6019178,E=6019178,C=31)
	CT(17100)=(S=6019179,E=6019179,C=12)
	CT(17101)=(S=6019180,E=6019180,C=28)
	CT(17102)=(S=6019181,E=6019181,C=52)
	CT(17103)=(S=6019182,E=6019182,C=12)
	CT(17104)=(S=6019183,E=6019183,C=19)
	CT(17105)=(S=6019184,E=6019184,C=14)
	CT(17106)=(S=6019185,E=6019185,C=36)
	CT(17107)=(S=6019186,E=6019186,C=18)
	CT(17108)=(S=6019187,E=6019187,C=58)
	CT(17109)=(S=6019188,E=6019188,C=14)
	CT(17110)=(S=6019189,E=6019189,C=21)
	CT(17111)=(S=6019190,E=6019190,C=24)
	CT(17112)=(S=6019191,E=6019191,C=37)
	CT(17113)=(S=6019192,E=6019192,C=42)
	CT(17114)=(S=6019193,E=6019195,C=18)
	CT(17115)=(S=6019196,E=6019197,C=58)
	CT(17116)=(S=6019198,E=6019198,C=18)
	CT(17117)=(S=6019199,E=6019199,C=15)
	CT(17118)=(S=6019200,E=6019200,C=28)
	CT(17119)=(S=6019201,E=6019202,C=18)
	CT(17120)=(S=6019203,E=6019204,C=36)
	CT(17121)=(S=6019205,E=6019205,C=31)
	CT(17122)=(S=6019206,E=6019206,C=58)
	CT(17123)=(S=6019207,E=6019207,C=31)
	CT(17124)=(S=6019208,E=6019208,C=47)
	CT(17125)=(S=6019209,E=6019209,C=14)
	CT(17126)=(S=6019210,E=6019210,C=38)
	CT(17127)=(S=6019211,E=6019212,C=12)
	CT(17128)=(S=6019213,E=6019214,C=36)
	CT(17129)=(S=6019215,E=6019215,C=14)
	CT(17130)=(S=6019216,E=6019216,C=38)
	CT(17131)=(S=6019217,E=6019217,C=49)
	CT(17132)=(S=6019218,E=6019218,C=14)
	CT(17133)=(S=6019219,E=6019219,C=18)
	CT(17134)=(S=6019220,E=6019220,C=37)
	CT(17135)=(S=6019221,E=6019221,C=58)
	CT(17136)=(S=6019222,E=6019222,C=50)
	CT(17137)=(S=6019223,E=6019224,C=38)
	CT(17138)=(S=6019225,E=6019226,C=72)
	CT(17139)=(S=6019227,E=6019227,C=25)
	CT(17140)=(S=6019228,E=6019229,C=38)
	CT(17141)=(S=6019230,E=6019230,C=28)
	CT(17142)=(S=6019231,E=6019231,C=14)
	CT(17143)=(S=6019232,E=6019232,C=58)
	CT(17144)=(S=6019233,E=6019233,C=31)
	CT(17145)=(S=6019234,E=6019234,C=28)
	CT(17146)=(S=6019235,E=6019235,C=44)
	CT(17147)=(S=6019236,E=6019236,C=18)
	CT(17148)=(S=6019237,E=6019237,C=31)
	CT(17149)=(S=6019238,E=6019238,C=53)
	CT(17150)=(S=6019239,E=6019239,C=18)
	CT(17151)=(S=6019240,E=6019240,C=33)
	CT(17152)=(S=6019242,E=6019242,C=37)
	CT(17153)=(S=6019243,E=6019243,C=12)
	CT(17154)=(S=6019244,E=6019244,C=52)
	CT(17155)=(S=6019245,E=6019245,C=31)
	CT(17156)=(S=6019246,E=6019246,C=51)
	CT(17157)=(S=6019247,E=6019247,C=18)
	CT(17158)=(S=6019248,E=6019248,C=40)
	CT(17159)=(S=6019249,E=6019249,C=38)
	CT(17160)=(S=6019250,E=6019250,C=20)
	CT(17161)=(S=6019251,E=6019251,C=33)
	CT(17162)=(S=6019252,E=6019252,C=22)
	CT(17163)=(S=6019253,E=6019253,C=14)
	CT(17164)=(S=6019254,E=6019254,C=24)
	CT(17165)=(S=6019255,E=6019255,C=14)
	CT(17166)=(S=6019256,E=6019256,C=36)
	CT(17167)=(S=6019257,E=6019257,C=33)
	CT(17168)=(S=6019258,E=6019258,C=12)
	CT(17169)=(S=6019259,E=6019259,C=18)
	CT(17170)=(S=6019260,E=6019260,C=101)
	CT(17171)=(S=6019261,E=6019261,C=20)
	CT(17172)=(S=6019263,E=6019264,C=58)
	CT(17173)=(S=6019265,E=6019265,C=14)
	CT(17174)=(S=6019266,E=6019266,C=58)
	CT(17175)=(S=6019267,E=6019267,C=12)
	CT(17176)=(S=6019268,E=6019268,C=31)
	CT(17177)=(S=6019269,E=6019269,C=14)
	CT(17178)=(S=6019270,E=6019270,C=18)
	CT(17179)=(S=6019271,E=6019271,C=38)
	CT(17180)=(S=6019272,E=6019272,C=36)
	CT(17181)=(S=6019273,E=6019273,C=37)
	CT(17182)=(S=6019274,E=6019274,C=58)
	CT(17183)=(S=6019275,E=6019275,C=12)
	CT(17184)=(S=6019276,E=6019276,C=58)
	CT(17185)=(S=6019277,E=6019277,C=18)
	CT(17186)=(S=6019278,E=6019278,C=22)
	CT(17187)=(S=6019279,E=6019279,C=36)
	CT(17188)=(S=6019280,E=6019280,C=18)
	CT(17189)=(S=6019281,E=6019281,C=12)
	CT(17190)=(S=6019282,E=6019282,C=32)
	CT(17191)=(S=6019283,E=6019284,C=18)
	CT(17192)=(S=6019285,E=6019286,C=58)
	CT(17193)=(S=6019287,E=6019287,C=18)
	CT(17194)=(S=6019288,E=6019288,C=28)
	CT(17195)=(S=6019289,E=6019289,C=18)
	CT(17196)=(S=6019290,E=6019291,C=25)
	CT(17197)=(S=6019292,E=6019292,C=20)
	CT(17198)=(S=6019293,E=6019293,C=12)
	CT(17199)=(S=6019294,E=6019294,C=17)
	CT(17200)=(S=6019295,E=6019295,C=37)
	CT(17201)=(S=6019296,E=6019296,C=58)
	CT(17202)=(S=6019297,E=6019297,C=38)
	CT(17203)=(S=6019298,E=6019298,C=24)
	CT(17204)=(S=6019299,E=6019299,C=38)
	CT(17205)=(S=6019300,E=6019300,C=12)
	CT(17206)=(S=6019301,E=6019301,C=34)
	CT(17207)=(S=6019302,E=6019302,C=42)
	CT(17208)=(S=6019303,E=6019304,C=31)
	CT(17209)=(S=6019305,E=6019305,C=38)
	CT(17210)=(S=6019306,E=6019306,C=12)
	CT(17211)=(S=6019307,E=6019307,C=28)
	CT(17212)=(S=6019308,E=6019308,C=63)
	CT(17213)=(S=6019309,E=6019309,C=31)
	CT(17214)=(S=6019310,E=6019310,C=49)
	CT(17215)=(S=6019311,E=6019311,C=58)
	CT(17216)=(S=6019312,E=6019312,C=31)
	CT(17217)=(S=6019313,E=6019313,C=63)
	CT(17218)=(S=6019314,E=6019314,C=42)
	CT(17219)=(S=6019315,E=6019315,C=28)
	CT(17220)=(S=6019316,E=6019316,C=31)
	CT(17221)=(S=6019317,E=6019317,C=28)
	CT(17222)=(S=6019318,E=6019318,C=14)
	CT(17223)=(S=6019319,E=6019319,C=42)
	CT(17224)=(S=6019320,E=6019320,C=28)
	CT(17225)=(S=6019321,E=6019321,C=42)
	CT(17226)=(S=6019322,E=6019322,C=18)
	CT(17227)=(S=6019323,E=6019323,C=31)
	CT(17228)=(S=6019324,E=6019324,C=17)
	CT(17229)=(S=6019325,E=6019325,C=51)
	CT(17230)=(S=6019326,E=6019326,C=58)
	CT(17231)=(S=6019327,E=6019327,C=48)
	CT(17232)=(S=6019328,E=6019329,C=58)
	CT(17233)=(S=6019330,E=6019331,C=18)
	CT(17234)=(S=6019332,E=6019333,C=31)
	CT(17235)=(S=6019334,E=6019335,C=18)
	CT(17236)=(S=6019336,E=6019341,C=31)
	CT(17237)=(S=6019342,E=6019343,C=46)
	CT(17238)=(S=6019344,E=6019345,C=18)
	CT(17239)=(S=6019346,E=6019347,C=58)
	CT(17240)=(S=6019348,E=6019349,C=18)
	CT(17241)=(S=6019350,E=6019353,C=58)
	CT(17242)=(S=6019354,E=6019357,C=38)
	CT(17243)=(S=6019358,E=6019359,C=36)
	CT(17244)=(S=6019360,E=6019361,C=71)
	CT(17245)=(S=6019362,E=6019363,C=18)
	CT(17246)=(S=6019364,E=6019365,C=28)
	CT(17247)=(S=6019366,E=6019367,C=42)
	CT(17248)=(S=6019368,E=6019369,C=58)
	CT(17249)=(S=6019370,E=6019371,C=18)
	CT(17250)=(S=6019372,E=6019373,C=12)
	CT(17251)=(S=6019374,E=6019375,C=18)
	CT(17252)=(S=6019376,E=6019377,C=58)
	CT(17253)=(S=6019378,E=6019379,C=18)
	CT(17254)=(S=6019380,E=6019381,C=33)
	CT(17255)=(S=6019382,E=6019383,C=42)
	CT(17256)=(S=6019384,E=6019385,C=36)
	CT(17257)=(S=6019386,E=6019389,C=31)
	CT(17258)=(S=6019390,E=6019391,C=18)
	CT(17259)=(S=6019392,E=6019393,C=26)
	CT(17260)=(S=6019394,E=6019397,C=31)
	CT(17261)=(S=6019398,E=6019399,C=18)
	CT(17262)=(S=6019400,E=6019401,C=14)
	CT(17263)=(S=6019402,E=6019405,C=18)
	CT(17264)=(S=6019406,E=6019407,C=31)
	CT(17265)=(S=6019410,E=6019411,C=38)
	CT(17266)=(S=6019412,E=6019413,C=14)
	CT(17267)=(S=6019414,E=6019415,C=28)
	CT(17268)=(S=6019416,E=6019417,C=63)
	CT(17269)=(S=6019418,E=6019419,C=31)
	CT(17270)=(S=6019420,E=6019421,C=52)
	CT(17271)=(S=6019422,E=6019423,C=12)
	CT(17272)=(S=6019424,E=6019425,C=33)
	CT(17273)=(S=6019426,E=6019427,C=58)
	CT(17274)=(S=6019428,E=6019429,C=31)
	CT(17275)=(S=6019430,E=6019431,C=18)
	CT(17276)=(S=6019432,E=6019433,C=58)
	CT(17277)=(S=6019434,E=6019435,C=31)
	CT(17278)=(S=6019436,E=6019439,C=18)
	CT(17279)=(S=6019440,E=6019441,C=31)
	CT(17280)=(S=6019442,E=6019443,C=58)
	CT(17281)=(S=6019444,E=6019445,C=47)
	CT(17282)=(S=6019446,E=6019447,C=42)
	CT(17283)=(S=6019448,E=6019449,C=18)
	CT(17284)=(S=6019450,E=6019451,C=52)
	CT(17285)=(S=6019452,E=6019453,C=18)
	CT(17286)=(S=6019454,E=6019455,C=52)
	CT(17287)=(S=6019456,E=6019456,C=34)
	CT(17288)=(S=6019457,E=6019457,C=18)
	CT(17289)=(S=6019458,E=6019458,C=38)
	CT(17290)=(S=6019459,E=6019459,C=28)
	CT(17291)=(S=6019460,E=6019460,C=58)
	CT(17292)=(S=6019461,E=6019461,C=24)
	CT(17293)=(S=6019462,E=6019463,C=14)
	CT(17294)=(S=6019464,E=6019465,C=18)
	CT(17295)=(S=6019466,E=6019466,C=102)
	CT(17296)=(S=6019468,E=6019468,C=12)
	CT(17297)=(S=6019469,E=6019469,C=28)
	CT(17298)=(S=6019470,E=6019470,C=58)
	CT(17299)=(S=6019471,E=6019471,C=18)
	CT(17300)=(S=6019472,E=6019472,C=31)
	CT(17301)=(S=6019473,E=6019473,C=28)
	CT(17302)=(S=6019474,E=6019474,C=36)
	CT(17303)=(S=6019475,E=6019475,C=37)
	CT(17304)=(S=6019476,E=6019476,C=51)
	CT(17305)=(S=6019477,E=6019477,C=42)
	CT(17306)=(S=6019478,E=6019478,C=58)
	CT(17307)=(S=6019479,E=6019479,C=12)
	CT(17308)=(S=6019480,E=6019480,C=58)
	CT(17309)=(S=6019481,E=6019481,C=31)
	CT(17310)=(S=6019482,E=6019482,C=12)
	CT(17311)=(S=6019483,E=6019483,C=22)
	CT(17312)=(S=6019484,E=6019484,C=18)
	CT(17313)=(S=6019485,E=6019485,C=66)
	CT(17314)=(S=6019486,E=6019486,C=58)
	CT(17315)=(S=6019487,E=6019487,C=18)
	CT(17316)=(S=6019488,E=6019488,C=14)
	CT(17317)=(S=6019489,E=6019489,C=58)
	CT(17318)=(S=6019490,E=6019490,C=31)
	CT(17319)=(S=6019491,E=6019491,C=58)
	CT(17320)=(S=6019492,E=6019492,C=12)
	CT(17321)=(S=6019493,E=6019493,C=28)
	CT(17322)=(S=6019494,E=6019494,C=31)
	CT(17323)=(S=6019496,E=6019496,C=12)
	CT(17324)=(S=6019497,E=6019497,C=50)
	CT(17325)=(S=6019498,E=6019498,C=34)
	CT(17326)=(S=6019499,E=6019499,C=58)
	CT(17327)=(S=6019500,E=6019500,C=12)
	CT(17328)=(S=6019501,E=6019501,C=42)
	CT(17329)=(S=6019502,E=6019504,C=28)
	CT(17330)=(S=6019505,E=6019505,C=39)
	CT(17331)=(S=6019506,E=6019506,C=18)
	CT(17332)=(S=6019507,E=6019507,C=31)
	CT(17333)=(S=6019508,E=6019508,C=18)
	CT(17334)=(S=6019509,E=6019509,C=19)
	CT(17335)=(S=6019510,E=6019510,C=58)
	CT(17336)=(S=6019511,E=6019511,C=31)
	CT(17337)=(S=6019512,E=6019512,C=18)
	CT(17338)=(S=6019513,E=6019513,C=35)
	CT(17339)=(S=6019514,E=6019514,C=19)
	CT(17340)=(S=6019515,E=6019515,C=38)
	CT(17341)=(S=6019516,E=6019516,C=28)
	CT(17342)=(S=6019517,E=6019517,C=36)
	CT(17343)=(S=6019518,E=6019518,C=18)
	CT(17344)=(S=6019519,E=6019519,C=36)
	CT(17345)=(S=6019520,E=6019520,C=18)
	CT(17346)=(S=6019521,E=6019521,C=37)
	CT(17347)=(S=6019522,E=6019524,C=18)
	CT(17348)=(S=6019525,E=6019525,C=31)
	CT(17349)=(S=6019526,E=6019526,C=58)
	CT(17350)=(S=6019527,E=6019527,C=28)
	CT(17351)=(S=6019529,E=6019529,C=24)
	CT(17352)=(S=6019530,E=6019530,C=18)
	CT(17353)=(S=6019531,E=6019531,C=38)
	CT(17354)=(S=6019532,E=6019532,C=58)
	CT(17355)=(S=6019533,E=6019533,C=51)
	CT(17356)=(S=6019534,E=6019534,C=28)
	CT(17357)=(S=6019536,E=6019536,C=58)
	CT(17358)=(S=6019537,E=6019537,C=19)
	CT(17359)=(S=6019538,E=6019538,C=36)
	CT(17360)=(S=6019539,E=6019539,C=47)
	CT(17361)=(S=6019540,E=6019540,C=18)
	CT(17362)=(S=6019541,E=6019541,C=42)
	CT(17363)=(S=6019542,E=6019542,C=28)
	CT(17364)=(S=6019543,E=6019543,C=51)
	CT(17365)=(S=6019544,E=6019544,C=14)
	CT(17366)=(S=6019545,E=6019546,C=58)
	CT(17367)=(S=6019547,E=6019547,C=17)
	CT(17368)=(S=6019548,E=6019548,C=18)
	CT(17369)=(S=6019549,E=6019549,C=58)
	CT(17370)=(S=6019550,E=6019550,C=52)
	CT(17371)=(S=6019552,E=6019552,C=58)
	CT(17372)=(S=6019553,E=6019554,C=42)
	CT(17373)=(S=6019555,E=6019555,C=18)
	CT(17374)=(S=6019556,E=6019556,C=58)
	CT(17375)=(S=6019557,E=6019557,C=42)
	CT(17376)=(S=6019558,E=6019558,C=28)
	CT(17377)=(S=6019559,E=6019559,C=14)
	CT(17378)=(S=6019560,E=6019560,C=28)
	CT(17379)=(S=6019561,E=6019561,C=18)
	CT(17380)=(S=6019562,E=6019562,C=12)
	CT(17381)=(S=6019563,E=6019563,C=36)
	CT(17382)=(S=6019564,E=6019564,C=18)
	CT(17383)=(S=6019565,E=6019565,C=14)
	CT(17384)=(S=6019566,E=6019566,C=37)
	CT(17385)=(S=6019568,E=6019568,C=31)
	CT(17386)=(S=6019569,E=6019569,C=34)
	CT(17387)=(S=6019570,E=6019570,C=58)
	CT(17388)=(S=6019571,E=6019571,C=22)
	CT(17389)=(S=6019572,E=6019572,C=31)
	CT(17390)=(S=6019573,E=6019573,C=14)
	CT(17391)=(S=6019574,E=6019574,C=32)
	CT(17392)=(S=6019575,E=6019575,C=34)
	CT(17393)=(S=6019576,E=6019576,C=47)
	CT(17394)=(S=6019579,E=6019579,C=28)
	CT(17395)=(S=6019580,E=6019580,C=47)
	CT(17396)=(S=6019582,E=6019582,C=31)
	CT(17397)=(S=6019583,E=6019583,C=52)
	CT(17398)=(S=6019584,E=6019587,C=38)
	CT(17399)=(S=6019588,E=6019591,C=66)
	CT(17400)=(S=6019592,E=6019595,C=18)
	CT(17401)=(S=6019596,E=6019603,C=31)
	CT(17402)=(S=6019608,E=6019611,C=18)
	CT(17403)=(S=6019612,E=6019619,C=31)
	CT(17404)=(S=6019624,E=6019627,C=18)
	CT(17405)=(S=6019628,E=6019635,C=31)
	CT(17406)=(S=6019636,E=6019639,C=60)
	CT(17407)=(S=6019644,E=6019655,C=58)
	CT(17408)=(S=6019656,E=6019659,C=31)
	CT(17409)=(S=6019660,E=6019663,C=58)
	CT(17410)=(S=6019664,E=6019667,C=51)
	CT(17411)=(S=6019668,E=6019671,C=18)
	CT(17412)=(S=6019672,E=6019675,C=31)
	CT(17413)=(S=6019676,E=6019679,C=18)
	CT(17414)=(S=6019680,E=6019683,C=31)
	CT(17415)=(S=6019684,E=6019687,C=18)
	CT(17416)=(S=6019688,E=6019691,C=31)
	CT(17417)=(S=6019692,E=6019695,C=18)
	CT(17418)=(S=6019696,E=6019699,C=28)
	CT(17419)=(S=6019700,E=6019703,C=18)
	CT(17420)=(S=6019704,E=6019707,C=33)
	CT(17421)=(S=6019708,E=6019711,C=36)
	CT(17422)=(S=6019712,E=6019731,C=18)
	CT(17423)=(S=6019732,E=6019735,C=36)
	CT(17424)=(S=6019736,E=6019739,C=38)
	CT(17425)=(S=6019740,E=6019743,C=58)
	CT(17426)=(S=6019744,E=6019747,C=18)
	CT(17427)=(S=6019748,E=6019751,C=58)
	CT(17428)=(S=6019752,E=6019755,C=103)
	CT(17429)=(S=6019756,E=6019759,C=46)
	CT(17430)=(S=6019760,E=6019763,C=18)
	CT(17431)=(S=6019764,E=6019767,C=66)
	CT(17432)=(S=6019768,E=6019771,C=27)
	CT(17433)=(S=6019772,E=6019775,C=33)
	CT(17434)=(S=6019776,E=6019779,C=31)
	CT(17435)=(S=6019780,E=6019787,C=58)
	CT(17436)=(S=6019788,E=6019791,C=36)
	CT(17437)=(S=6019792,E=6019795,C=58)
	CT(17438)=(S=6019796,E=6019799,C=31)
	CT(17439)=(S=6019800,E=6019803,C=58)
	CT(17440)=(S=6019804,E=6019807,C=18)
	CT(17441)=(S=6019808,E=6019811,C=15)
	CT(17442)=(S=6019812,E=6019815,C=18)
	CT(17443)=(S=6019816,E=6019819,C=33)
	CT(17444)=(S=6019820,E=6019823,C=38)
	CT(17445)=(S=6019824,E=6019827,C=58)
	CT(17446)=(S=6019828,E=6019835,C=18)
	CT(17447)=(S=6019836,E=6019839,C=38)
	CT(17448)=(S=6019840,E=6019843,C=58)
	CT(17449)=(S=6019844,E=6019847,C=18)
	CT(17450)=(S=6019848,E=6019851,C=28)
	CT(17451)=(S=6019852,E=6019855,C=18)
	CT(17452)=(S=6019856,E=6019859,C=38)
	CT(17453)=(S=6019860,E=6019867,C=18)
	CT(17454)=(S=6019868,E=6019875,C=31)
	CT(17455)=(S=6019876,E=6019879,C=18)
	CT(17456)=(S=6019880,E=6019883,C=14)
	CT(17457)=(S=6019884,E=6019887,C=33)
	CT(17458)=(S=6019888,E=6019891,C=18)
	CT(17459)=(S=6019892,E=6019895,C=31)
	CT(17460)=(S=6019896,E=6019899,C=18)
	CT(17461)=(S=6019900,E=6019903,C=31)
	CT(17462)=(S=6019904,E=6019907,C=18)
	CT(17463)=(S=6019908,E=6019911,C=42)
	CT(17464)=(S=6019912,E=6019915,C=18)
	CT(17465)=(S=6019916,E=6019919,C=36)
	CT(17466)=(S=6019920,E=6019923,C=31)
	CT(17467)=(S=6019924,E=6019927,C=50)
	CT(17468)=(S=6019928,E=6019931,C=18)
	CT(17469)=(S=6019932,E=6019935,C=38)
	CT(17470)=(S=6019936,E=6019943,C=18)
	CT(17471)=(S=6019944,E=6019947,C=58)
	CT(17472)=(S=6019948,E=6019951,C=36)
	CT(17473)=(S=6019952,E=6019955,C=58)
	CT(17474)=(S=6019956,E=6019959,C=25)
	CT(17475)=(S=6019960,E=6019963,C=58)
	CT(17476)=(S=6019964,E=6019967,C=18)
	CT(17477)=(S=6019968,E=6019971,C=58)
	CT(17478)=(S=6019972,E=6019975,C=71)
	CT(17479)=(S=6019976,E=6019979,C=18)
	CT(17480)=(S=6019980,E=6019987,C=31)
	CT(17481)=(S=6019988,E=6019991,C=18)
	CT(17482)=(S=6019992,E=6019995,C=24)
	CT(17483)=(S=6019996,E=6020007,C=18)
	CT(17484)=(S=6020008,E=6020011,C=31)
	CT(17485)=(S=6020012,E=6020015,C=47)
	CT(17486)=(S=6020016,E=6020023,C=18)
	CT(17487)=(S=6020024,E=6020027,C=58)
	CT(17488)=(S=6020028,E=6020035,C=18)
	CT(17489)=(S=6020036,E=6020039,C=31)
	CT(17490)=(S=6020040,E=6020047,C=18)
	CT(17491)=(S=6020048,E=6020051,C=58)
	CT(17492)=(S=6020052,E=6020055,C=38)
	CT(17493)=(S=6020056,E=6020059,C=19)
	CT(17494)=(S=6020060,E=6020063,C=31)
	CT(17495)=(S=6020064,E=6020071,C=18)
	CT(17496)=(S=6020072,E=6020075,C=31)
	CT(17497)=(S=6020076,E=6020079,C=50)
	CT(17498)=(S=6020080,E=6020087,C=33)
	CT(17499)=(S=6020088,E=6020091,C=18)
	CT(17500)=(S=6020092,E=6020095,C=31)
	CT(17501)=(S=6020096,E=6020096,C=58)
	CT(17502)=(S=6020097,E=6020097,C=18)
	CT(17503)=(S=6020098,E=6020098,C=42)
	CT(17504)=(S=6020099,E=6020099,C=31)
	CT(17505)=(S=6020100,E=6020100,C=15)
	CT(17506)=(S=6020101,E=6020101,C=18)
	CT(17507)=(S=6020102,E=6020102,C=31)
	CT(17508)=(S=6020103,E=6020103,C=24)
	CT(17509)=(S=6020104,E=6020104,C=42)
	CT(17510)=(S=6020105,E=6020105,C=17)
	CT(17511)=(S=6020106,E=6020106,C=58)
	CT(17512)=(S=6020107,E=6020108,C=18)
	CT(17513)=(S=6020109,E=6020110,C=38)
	CT(17514)=(S=6020111,E=6020111,C=14)
	CT(17515)=(S=6020112,E=6020112,C=18)
	CT(17516)=(S=6020113,E=6020114,C=58)
	CT(17517)=(S=6020115,E=6020115,C=19)
	CT(17518)=(S=6020116,E=6020116,C=44)
	CT(17519)=(S=6020117,E=6020117,C=36)
	CT(17520)=(S=6020118,E=6020118,C=17)
	CT(17521)=(S=6020119,E=6020119,C=34)
	CT(17522)=(S=6020120,E=6020121,C=14)
	CT(17523)=(S=6020122,E=6020122,C=38)
	CT(17524)=(S=6020123,E=6020123,C=12)
	CT(17525)=(S=6020125,E=6020125,C=50)
	CT(17526)=(S=6020126,E=6020126,C=36)
	CT(17527)=(S=6020127,E=6020127,C=18)
	CT(17528)=(S=6020128,E=6020128,C=58)
	CT(17529)=(S=6020129,E=6020129,C=18)
	CT(17530)=(S=6020130,E=6020130,C=28)
	CT(17531)=(S=6020131,E=6020131,C=31)
	CT(17532)=(S=6020132,E=6020132,C=28)
	CT(17533)=(S=6020133,E=6020133,C=36)
	CT(17534)=(S=6020134,E=6020134,C=18)
	CT(17535)=(S=6020135,E=6020135,C=58)
	CT(17536)=(S=6020136,E=6020136,C=34)
	CT(17537)=(S=6020137,E=6020137,C=32)
	CT(17538)=(S=6020138,E=6020138,C=14)
	CT(17539)=(S=6020139,E=6020139,C=60)
	CT(17540)=(S=6020140,E=6020140,C=58)
	CT(17541)=(S=6020141,E=6020141,C=63)
	CT(17542)=(S=6020142,E=6020142,C=18)
	CT(17543)=(S=6020143,E=6020143,C=19)
	CT(17544)=(S=6020144,E=6020144,C=38)
	CT(17545)=(S=6020145,E=6020145,C=28)
	CT(17546)=(S=6020146,E=6020146,C=37)
	CT(17547)=(S=6020147,E=6020147,C=20)
	CT(17548)=(S=6020148,E=6020148,C=14)
	CT(17549)=(S=6020149,E=6020149,C=36)
	CT(17550)=(S=6020150,E=6020150,C=58)
	CT(17551)=(S=6020151,E=6020151,C=52)
	CT(17552)=(S=6020152,E=6020152,C=28)
	CT(17553)=(S=6020153,E=6020153,C=18)
	CT(17554)=(S=6020154,E=6020154,C=19)
	CT(17555)=(S=6020155,E=6020155,C=44)
	CT(17556)=(S=6020156,E=6020157,C=31)
	CT(17557)=(S=6020158,E=6020158,C=18)
	CT(17558)=(S=6020159,E=6020159,C=53)
	CT(17559)=(S=6020160,E=6020160,C=34)
	CT(17560)=(S=6020161,E=6020161,C=37)
	CT(17561)=(S=6020162,E=6020162,C=25)
	CT(17562)=(S=6020163,E=6020163,C=28)
	CT(17563)=(S=6020164,E=6020164,C=49)
	CT(17564)=(S=6020165,E=6020165,C=18)
	CT(17565)=(S=6020166,E=6020166,C=14)
	CT(17566)=(S=6020168,E=6020168,C=38)
	CT(17567)=(S=6020169,E=6020169,C=58)
	CT(17568)=(S=6020170,E=6020170,C=28)
	CT(17569)=(S=6020171,E=6020171,C=38)
	CT(17570)=(S=6020172,E=6020172,C=18)
	CT(17571)=(S=6020173,E=6020173,C=34)
	CT(17572)=(S=6020174,E=6020174,C=52)
	CT(17573)=(S=6020175,E=6020175,C=26)
	CT(17574)=(S=6020176,E=6020176,C=31)
	CT(17575)=(S=6020177,E=6020177,C=18)
	CT(17576)=(S=6020178,E=6020178,C=36)
	CT(17577)=(S=6020179,E=6020179,C=14)
	CT(17578)=(S=6020181,E=6020181,C=12)
	CT(17579)=(S=6020182,E=6020182,C=38)
	CT(17580)=(S=6020183,E=6020183,C=18)
	CT(17581)=(S=6020184,E=6020184,C=36)
	CT(17582)=(S=6020185,E=6020185,C=42)
	CT(17583)=(S=6020187,E=6020187,C=31)
	CT(17584)=(S=6020188,E=6020189,C=18)
	CT(17585)=(S=6020190,E=6020190,C=53)
	CT(17586)=(S=6020191,E=6020191,C=54)
	CT(17587)=(S=6020192,E=6020192,C=38)
	CT(17588)=(S=6020193,E=6020193,C=58)
	CT(17589)=(S=6020194,E=6020194,C=19)
	CT(17590)=(S=6020195,E=6020196,C=34)
	CT(17591)=(S=6020197,E=6020197,C=36)
	CT(17592)=(S=6020198,E=6020198,C=49)
	CT(17593)=(S=6020199,E=6020199,C=58)
	CT(17594)=(S=6020200,E=6020200,C=31)
	CT(17595)=(S=6020201,E=6020201,C=58)
	CT(17596)=(S=6020202,E=6020202,C=31)
	CT(17597)=(S=6020203,E=6020203,C=52)
	CT(17598)=(S=6020204,E=6020204,C=22)
	CT(17599)=(S=6020205,E=6020205,C=18)
	CT(17600)=(S=6020206,E=6020206,C=48)
	CT(17601)=(S=6020207,E=6020207,C=49)
	CT(17602)=(S=6020208,E=6020208,C=18)
	CT(17603)=(S=6020209,E=6020209,C=31)
	CT(17604)=(S=6020210,E=6020210,C=44)
	CT(17605)=(S=6020211,E=6020211,C=28)
	CT(17606)=(S=6020212,E=6020212,C=12)
	CT(17607)=(S=6020213,E=6020213,C=19)
	CT(17608)=(S=6020214,E=6020214,C=14)
	CT(17609)=(S=6020215,E=6020215,C=52)
	CT(17610)=(S=6020216,E=6020216,C=18)
	CT(17611)=(S=6020217,E=6020217,C=38)
	CT(17612)=(S=6020218,E=6020218,C=33)
	CT(17613)=(S=6020219,E=6020219,C=36)
	CT(17614)=(S=6020220,E=6020220,C=66)
	CT(17615)=(S=6020221,E=6020221,C=28)
	CT(17616)=(S=6020223,E=6020223,C=14)
	CT(17617)=(S=6020224,E=6020224,C=34)
	CT(17618)=(S=6020225,E=6020225,C=28)
	CT(17619)=(S=6020226,E=6020226,C=38)
	CT(17620)=(S=6020227,E=6020227,C=18)
	CT(17621)=(S=6020228,E=6020228,C=14)
	CT(17622)=(S=6020229,E=6020229,C=18)
	CT(17623)=(S=6020230,E=6020230,C=28)
	CT(17624)=(S=6020231,E=6020231,C=18)
	CT(17625)=(S=6020232,E=6020232,C=49)
	CT(17626)=(S=6020233,E=6020233,C=18)
	CT(17627)=(S=6020234,E=6020234,C=48)
	CT(17628)=(S=6020235,E=6020235,C=72)
	CT(17629)=(S=6020236,E=6020236,C=31)
	CT(17630)=(S=6020237,E=6020237,C=14)
	CT(17631)=(S=6020238,E=6020238,C=15)
	CT(17632)=(S=6020239,E=6020239,C=27)
	CT(17633)=(S=6020240,E=6020240,C=53)
	CT(17634)=(S=6020241,E=6020242,C=18)
	CT(17635)=(S=6020243,E=6020243,C=36)
	CT(17636)=(S=6020244,E=6020244,C=28)
	CT(17637)=(S=6020245,E=6020245,C=38)
	CT(17638)=(S=6020246,E=6020246,C=19)
	CT(17639)=(S=6020247,E=6020247,C=31)
	CT(17640)=(S=6020248,E=6020248,C=18)
	CT(17641)=(S=6020249,E=6020249,C=28)
	CT(17642)=(S=6020250,E=6020250,C=49)
	CT(17643)=(S=6020251,E=6020251,C=18)
	CT(17644)=(S=6020252,E=6020252,C=52)
	CT(17645)=(S=6020253,E=6020253,C=28)
	CT(17646)=(S=6020254,E=6020254,C=25)
	CT(17647)=(S=6020255,E=6020255,C=28)
	CT(17648)=(S=6020256,E=6020256,C=14)
	CT(17649)=(S=6020257,E=6020257,C=31)
	CT(17650)=(S=6020258,E=6020258,C=34)
	CT(17651)=(S=6020259,E=6020259,C=31)
	CT(17652)=(S=6020260,E=6020260,C=58)
	CT(17653)=(S=6020261,E=6020261,C=27)
	CT(17654)=(S=6020262,E=6020262,C=18)
	CT(17655)=(S=6020263,E=6020263,C=38)
	CT(17656)=(S=6020264,E=6020264,C=15)
	CT(17657)=(S=6020265,E=6020266,C=18)
	CT(17658)=(S=6020267,E=6020267,C=12)
	CT(17659)=(S=6020268,E=6020268,C=50)
	CT(17660)=(S=6020269,E=6020269,C=18)
	CT(17661)=(S=6020270,E=6020270,C=31)
	CT(17662)=(S=6020271,E=6020271,C=14)
	CT(17663)=(S=6020273,E=6020273,C=58)
	CT(17664)=(S=6020274,E=6020274,C=14)
	CT(17665)=(S=6020275,E=6020275,C=42)
	CT(17666)=(S=6020276,E=6020276,C=14)
	CT(17667)=(S=6020277,E=6020277,C=18)
	CT(17668)=(S=6020278,E=6020278,C=37)
	CT(17669)=(S=6020279,E=6020279,C=38)
	CT(17670)=(S=6020280,E=6020281,C=36)
	CT(17671)=(S=6020283,E=6020283,C=102)
	CT(17672)=(S=6020284,E=6020284,C=18)
	CT(17673)=(S=6020285,E=6020285,C=51)
	CT(17674)=(S=6020286,E=6020286,C=31)
	CT(17675)=(S=6020287,E=6020287,C=49)
	CT(17676)=(S=6020288,E=6020288,C=19)
	CT(17677)=(S=6020289,E=6020289,C=17)
	CT(17678)=(S=6020290,E=6020290,C=52)
	CT(17679)=(S=6020291,E=6020291,C=61)
	CT(17680)=(S=6020292,E=6020292,C=27)
	CT(17681)=(S=6020293,E=6020293,C=12)
	CT(17682)=(S=6020294,E=6020294,C=31)
	CT(17683)=(S=6020295,E=6020295,C=14)
	CT(17684)=(S=6020296,E=6020296,C=12)
	CT(17685)=(S=6020297,E=6020297,C=33)
	CT(17686)=(S=6020300,E=6020300,C=31)
	CT(17687)=(S=6020301,E=6020301,C=58)
	CT(17688)=(S=6020302,E=6020302,C=18)
	CT(17689)=(S=6020303,E=6020303,C=53)
	CT(17690)=(S=6020304,E=6020304,C=22)
	CT(17691)=(S=6020305,E=6020305,C=44)
	CT(17692)=(S=6020306,E=6020306,C=42)
	CT(17693)=(S=6020307,E=6020307,C=18)
	CT(17694)=(S=6020308,E=6020308,C=14)
	CT(17695)=(S=6020309,E=6020309,C=52)
	CT(17696)=(S=6020310,E=6020310,C=14)
	CT(17697)=(S=6020311,E=6020311,C=18)
	CT(17698)=(S=6020312,E=6020312,C=14)
	CT(17699)=(S=6020313,E=6020313,C=27)
	CT(17700)=(S=6020314,E=6020316,C=18)
	CT(17701)=(S=6020317,E=6020317,C=52)
	CT(17702)=(S=6020318,E=6020318,C=58)
	CT(17703)=(S=6020319,E=6020319,C=18)
	CT(17704)=(S=6020320,E=6020320,C=63)
	CT(17705)=(S=6020321,E=6020321,C=58)
	CT(17706)=(S=6020322,E=6020322,C=25)
	CT(17707)=(S=6020323,E=6020323,C=19)
	CT(17708)=(S=6020324,E=6020324,C=28)
	CT(17709)=(S=6020326,E=6020326,C=31)
	CT(17710)=(S=6020327,E=6020327,C=14)
	CT(17711)=(S=6020328,E=6020328,C=38)
	CT(17712)=(S=6020329,E=6020329,C=18)
	CT(17713)=(S=6020330,E=6020330,C=14)
	CT(17714)=(S=6020331,E=6020331,C=31)
	CT(17715)=(S=6020332,E=6020332,C=22)
	CT(17716)=(S=6020333,E=6020333,C=71)
	CT(17717)=(S=6020334,E=6020334,C=58)
	CT(17718)=(S=6020335,E=6020335,C=42)
	CT(17719)=(S=6020336,E=6020336,C=52)
	CT(17720)=(S=6020337,E=6020337,C=47)
	CT(17721)=(S=6020338,E=6020338,C=37)
	CT(17722)=(S=6020339,E=6020339,C=27)
	CT(17723)=(S=6020340,E=6020340,C=28)
	CT(17724)=(S=6020341,E=6020342,C=12)
	CT(17725)=(S=6020343,E=6020343,C=50)
	CT(17726)=(S=6020344,E=6020344,C=12)
	CT(17727)=(S=6020345,E=6020345,C=36)
	CT(17728)=(S=6020346,E=6020346,C=68)
	CT(17729)=(S=6020347,E=6020347,C=50)
	CT(17730)=(S=6020348,E=6020349,C=31)
	CT(17731)=(S=6020350,E=6020350,C=19)
	CT(17732)=(S=6020351,E=6020351,C=31)
	CT(17733)=(S=6020352,E=6020353,C=18)
	CT(17734)=(S=6020354,E=6020355,C=33)
	CT(17735)=(S=6020356,E=6020357,C=71)
	CT(17736)=(S=6020358,E=6020359,C=31)
	CT(17737)=(S=6020360,E=6020361,C=28)
	CT(17738)=(S=6020362,E=6020363,C=72)
	CT(17739)=(S=6020364,E=6020365,C=28)
	CT(17740)=(S=6020366,E=6020367,C=31)
	CT(17741)=(S=6020368,E=6020373,C=18)
	CT(17742)=(S=6020374,E=6020375,C=23)
	CT(17743)=(S=6020376,E=6020377,C=18)
	CT(17744)=(S=6020378,E=6020379,C=58)
	CT(17745)=(S=6020380,E=6020383,C=18)
	CT(17746)=(S=6020384,E=6020385,C=15)
	CT(17747)=(S=6020386,E=6020389,C=18)
	CT(17748)=(S=6020390,E=6020391,C=33)
	CT(17749)=(S=6020392,E=6020395,C=18)
	CT(17750)=(S=6020396,E=6020397,C=58)
	CT(17751)=(S=6020398,E=6020399,C=33)
	CT(17752)=(S=6020400,E=6020401,C=18)
	CT(17753)=(S=6020402,E=6020403,C=24)
	CT(17754)=(S=6020404,E=6020405,C=31)
	CT(17755)=(S=6020406,E=6020407,C=14)
	CT(17756)=(S=6020408,E=6020409,C=18)
	CT(17757)=(S=6020410,E=6020411,C=28)
	CT(17758)=(S=6020412,E=6020413,C=18)
	CT(17759)=(S=6020414,E=6020415,C=60)
	CT(17760)=(S=6020416,E=6020417,C=31)
	CT(17761)=(S=6020418,E=6020419,C=47)
	CT(17762)=(S=6020420,E=6020420,C=18)
	CT(17763)=(S=6020421,E=6020421,C=36)
	CT(17764)=(S=6020422,E=6020423,C=18)
	CT(17765)=(S=6020424,E=6020425,C=33)
	CT(17766)=(S=6020426,E=6020427,C=31)
	CT(17767)=(S=6020430,E=6020431,C=38)
	CT(17768)=(S=6020432,E=6020433,C=18)
	CT(17769)=(S=6020434,E=6020435,C=14)
	CT(17770)=(S=6020436,E=6020437,C=31)
	CT(17771)=(S=6020438,E=6020439,C=18)
	CT(17772)=(S=6020440,E=6020441,C=58)
	CT(17773)=(S=6020442,E=6020443,C=18)
	CT(17774)=(S=6020446,E=6020447,C=38)
	CT(17775)=(S=6020448,E=6020449,C=36)
	CT(17776)=(S=6020450,E=6020450,C=18)
	CT(17777)=(S=6020451,E=6020451,C=53)
	CT(17778)=(S=6020452,E=6020453,C=42)
	CT(17779)=(S=6020454,E=6020455,C=18)
	CT(17780)=(S=6020456,E=6020457,C=28)
	CT(17781)=(S=6020458,E=6020461,C=18)
	CT(17782)=(S=6020462,E=6020465,C=71)
	CT(17783)=(S=6020466,E=6020467,C=58)
	CT(17784)=(S=6020470,E=6020471,C=34)
	CT(17785)=(S=6020472,E=6020473,C=28)
	CT(17786)=(S=6020474,E=6020475,C=58)
	CT(17787)=(S=6020476,E=6020477,C=31)
	CT(17788)=(S=6020478,E=6020479,C=58)
	CT(17789)=(S=6020480,E=6020481,C=101)
	CT(17790)=(S=6020482,E=6020483,C=27)
	CT(17791)=(S=6020484,E=6020485,C=31)
	CT(17792)=(S=6020486,E=6020487,C=46)
	CT(17793)=(S=6020488,E=6020489,C=31)
	CT(17794)=(S=6020490,E=6020491,C=42)
	CT(17795)=(S=6020492,E=6020493,C=18)
	CT(17796)=(S=6020494,E=6020495,C=24)
	CT(17797)=(S=6020496,E=6020497,C=58)
	CT(17798)=(S=6020498,E=6020499,C=12)
	CT(17799)=(S=6020500,E=6020501,C=33)
	CT(17800)=(S=6020502,E=6020503,C=36)
	CT(17801)=(S=6020504,E=6020505,C=14)
	CT(17802)=(S=6020506,E=6020507,C=31)
	CT(17803)=(S=6020508,E=6020509,C=18)
	CT(17804)=(S=6020510,E=6020511,C=58)
	CT(17805)=(S=6020512,E=6020513,C=36)
	CT(17806)=(S=6020514,E=6020517,C=18)
	CT(17807)=(S=6020518,E=6020519,C=31)
	CT(17808)=(S=6020520,E=6020521,C=14)
	CT(17809)=(S=6020522,E=6020523,C=52)
	CT(17810)=(S=6020524,E=6020525,C=58)
	CT(17811)=(S=6020526,E=6020529,C=18)
	CT(17812)=(S=6020530,E=6020531,C=31)
	CT(17813)=(S=6020532,E=6020533,C=18)
	CT(17814)=(S=6020534,E=6020535,C=28)
	CT(17815)=(S=6020536,E=6020537,C=23)
	CT(17816)=(S=6020538,E=6020539,C=55)
	CT(17817)=(S=6020540,E=6020541,C=18)
	CT(17818)=(S=6020542,E=6020543,C=31)
	CT(17819)=(S=6020544,E=6020545,C=34)
	CT(17820)=(S=6020546,E=6020547,C=18)
	CT(17821)=(S=6020548,E=6020549,C=24)
	CT(17822)=(S=6020550,E=6020553,C=18)
	CT(17823)=(S=6020554,E=6020555,C=23)
	CT(17824)=(S=6020556,E=6020557,C=28)
	CT(17825)=(S=6020558,E=6020559,C=24)
	CT(17826)=(S=6020560,E=6020561,C=36)
	CT(17827)=(S=6020562,E=6020563,C=31)
	CT(17828)=(S=6020564,E=6020565,C=33)
	CT(17829)=(S=6020566,E=6020567,C=58)
	CT(17830)=(S=6020568,E=6020569,C=46)
	CT(17831)=(S=6020570,E=6020571,C=31)
	CT(17832)=(S=6020572,E=6020573,C=101)
	CT(17833)=(S=6020574,E=6020575,C=32)
	CT(17834)=(S=6020576,E=6020577,C=12)
	CT(17835)=(S=6020578,E=6020579,C=28)
	CT(17836)=(S=6020580,E=6020581,C=66)
	CT(17837)=(S=6020582,E=6020583,C=36)
	CT(17838)=(S=6020584,E=6020585,C=18)
	CT(17839)=(S=6020586,E=6020587,C=31)
	CT(17840)=(S=6020588,E=6020589,C=15)
	CT(17841)=(S=6020590,E=6020591,C=47)
	CT(17842)=(S=6020592,E=6020593,C=38)
	CT(17843)=(S=6020594,E=6020595,C=18)
	CT(17844)=(S=6020596,E=6020597,C=12)
	CT(17845)=(S=6020598,E=6020601,C=18)
	CT(17846)=(S=6020602,E=6020603,C=28)
	CT(17847)=(S=6020604,E=6020605,C=31)
	CT(17848)=(S=6020606,E=6020607,C=12)
	CT(17849)=(S=6020608,E=6020611,C=31)
	CT(17850)=(S=6020612,E=6020615,C=46)
	CT(17851)=(S=6020616,E=6020619,C=14)
	CT(17852)=(S=6020620,E=6020627,C=31)
	CT(17853)=(S=6020628,E=6020631,C=51)
	CT(17854)=(S=6020632,E=6020635,C=58)
	CT(17855)=(S=6020636,E=6020639,C=18)
	CT(17856)=(S=6020640,E=6020651,C=31)
	CT(17857)=(S=6020652,E=6020655,C=28)
	CT(17858)=(S=6020656,E=6020659,C=31)
	CT(17859)=(S=6020660,E=6020663,C=33)
	CT(17860)=(S=6020664,E=6020667,C=58)
	CT(17861)=(S=6020668,E=6020675,C=31)
	CT(17862)=(S=6020676,E=6020683,C=58)
	CT(17863)=(S=6020684,E=6020691,C=31)
	CT(17864)=(S=6020692,E=6020695,C=34)
	CT(17865)=(S=6020696,E=6020699,C=20)
	CT(17866)=(S=6020700,E=6020703,C=42)
	CT(17867)=(S=6020704,E=6020707,C=28)
	CT(17868)=(S=6020708,E=6020711,C=58)
	CT(17869)=(S=6020712,E=6020715,C=71)
	CT(17870)=(S=6020716,E=6020719,C=58)
	CT(17871)=(S=6020720,E=6020723,C=31)
	CT(17872)=(S=6020724,E=6020727,C=58)
	CT(17873)=(S=6020728,E=6020731,C=18)
	CT(17874)=(S=6020732,E=6020735,C=58)
	CT(17875)=(S=6020736,E=6020739,C=18)
	CT(17876)=(S=6020740,E=6020743,C=34)
	CT(17877)=(S=6020744,E=6020747,C=31)
	CT(17878)=(S=6020748,E=6020759,C=58)
	CT(17879)=(S=6020760,E=6020763,C=31)
	CT(17880)=(S=6020764,E=6020767,C=18)
	CT(17881)=(S=6020768,E=6020771,C=58)
	CT(17882)=(S=6020772,E=6020775,C=18)
	CT(17883)=(S=6020776,E=6020783,C=31)
	CT(17884)=(S=6020784,E=6020787,C=28)
	CT(17885)=(S=6020788,E=6020791,C=31)
	CT(17886)=(S=6020792,E=6020795,C=58)
	CT(17887)=(S=6020796,E=6020799,C=47)
	CT(17888)=(S=6020800,E=6020803,C=58)
	CT(17889)=(S=6020804,E=6020807,C=26)
	CT(17890)=(S=6020808,E=6020811,C=23)
	CT(17891)=(S=6020812,E=6020815,C=33)
	CT(17892)=(S=6020816,E=6020819,C=18)
	CT(17893)=(S=6020820,E=6020823,C=58)
	CT(17894)=(S=6020824,E=6020827,C=18)
	CT(17895)=(S=6020828,E=6020831,C=31)
	CT(17896)=(S=6020832,E=6020835,C=58)
	CT(17897)=(S=6020836,E=6020839,C=18)
	CT(17898)=(S=6020840,E=6020843,C=28)
	CT(17899)=(S=6020844,E=6020851,C=18)
	CT(17900)=(S=6020852,E=6020863,C=31)
	CT(17901)=(S=6020864,E=6020864,C=37)
	CT(17902)=(S=6020865,E=6020865,C=18)
	CT(17903)=(S=6020866,E=6020866,C=28)
	CT(17904)=(S=6020867,E=6020868,C=58)
	CT(17905)=(S=6020869,E=6020869,C=12)
	CT(17906)=(S=6020870,E=6020870,C=58)
	CT(17907)=(S=6020871,E=6020871,C=18)
	CT(17908)=(S=6020872,E=6020872,C=37)
	CT(17909)=(S=6020873,E=6020873,C=63)
	CT(17910)=(S=6020874,E=6020874,C=14)
	CT(17911)=(S=6020875,E=6020875,C=17)
	CT(17912)=(S=6020876,E=6020876,C=15)
	CT(17913)=(S=6020877,E=6020878,C=58)
	CT(17914)=(S=6020879,E=6020879,C=18)
	CT(17915)=(S=6020880,E=6020880,C=14)
	CT(17916)=(S=6020881,E=6020881,C=34)
	CT(17917)=(S=6020882,E=6020882,C=28)
	CT(17918)=(S=6020883,E=6020883,C=31)
	CT(17919)=(S=6020884,E=6020884,C=28)
	CT(17920)=(S=6020885,E=6020885,C=49)
	CT(17921)=(S=6020886,E=6020886,C=12)
	CT(17922)=(S=6020887,E=6020887,C=58)
	CT(17923)=(S=6020888,E=6020888,C=57)
	CT(17924)=(S=6020889,E=6020889,C=18)
	CT(17925)=(S=6020890,E=6020890,C=52)
	CT(17926)=(S=6020891,E=6020891,C=31)
	CT(17927)=(S=6020893,E=6020893,C=57)
	CT(17928)=(S=6020894,E=6020894,C=31)
	CT(17929)=(S=6020895,E=6020895,C=58)
	CT(17930)=(S=6020896,E=6020896,C=18)
	CT(17931)=(S=6020897,E=6020897,C=58)
	CT(17932)=(S=6020898,E=6020898,C=18)
	CT(17933)=(S=6020899,E=6020899,C=12)
	CT(17934)=(S=6020900,E=6020900,C=27)
	CT(17935)=(S=6020901,E=6020901,C=18)
	CT(17936)=(S=6020902,E=6020902,C=42)
	CT(17937)=(S=6020903,E=6020903,C=52)
	CT(17938)=(S=6020904,E=6020904,C=12)
	CT(17939)=(S=6020905,E=6020905,C=28)
	CT(17940)=(S=6020906,E=6020906,C=22)
	CT(17941)=(S=6020907,E=6020907,C=52)
	CT(17942)=(S=6020908,E=6020908,C=18)
	CT(17943)=(S=6020909,E=6020909,C=38)
	CT(17944)=(S=6020910,E=6020910,C=14)
	CT(17945)=(S=6020911,E=6020911,C=19)
	CT(17946)=(S=6020912,E=6020912,C=31)
	CT(17947)=(S=6020913,E=6020913,C=52)
	CT(17948)=(S=6020914,E=6020914,C=38)
	CT(17949)=(S=6020915,E=6020915,C=19)
	CT(17950)=(S=6020916,E=6020916,C=58)
	CT(17951)=(S=6020917,E=6020917,C=31)
	CT(17952)=(S=6020918,E=6020918,C=17)
	CT(17953)=(S=6020919,E=6020919,C=58)
	CT(17954)=(S=6020920,E=6020920,C=36)
	CT(17955)=(S=6020921,E=6020921,C=58)
	CT(17956)=(S=6020922,E=6020922,C=14)
	CT(17957)=(S=6020923,E=6020923,C=31)
	CT(17958)=(S=6020924,E=6020924,C=18)
	CT(17959)=(S=6020925,E=6020925,C=25)
	CT(17960)=(S=6020926,E=6020926,C=12)
	CT(17961)=(S=6020927,E=6020927,C=18)
	CT(17962)=(S=6020928,E=6020928,C=58)
	CT(17963)=(S=6020929,E=6020929,C=57)
	CT(17964)=(S=6020930,E=6020930,C=51)
	CT(17965)=(S=6020931,E=6020931,C=14)
	CT(17966)=(S=6020932,E=6020932,C=58)
	CT(17967)=(S=6020933,E=6020933,C=71)
	CT(17968)=(S=6020935,E=6020935,C=12)
	CT(17969)=(S=6020936,E=6020936,C=58)
	CT(17970)=(S=6020937,E=6020937,C=12)
	CT(17971)=(S=6020938,E=6020938,C=33)
	CT(17972)=(S=6020940,E=6020940,C=12)
	CT(17973)=(S=6020941,E=6020941,C=31)
	CT(17974)=(S=6020942,E=6020942,C=58)
	CT(17975)=(S=6020943,E=6020943,C=34)
	CT(17976)=(S=6020944,E=6020944,C=52)
	CT(17977)=(S=6020945,E=6020945,C=58)
	CT(17978)=(S=6020946,E=6020946,C=36)
	CT(17979)=(S=6020947,E=6020947,C=58)
	CT(17980)=(S=6020948,E=6020948,C=12)
	CT(17981)=(S=6020949,E=6020949,C=24)
	CT(17982)=(S=6020950,E=6020950,C=31)
	CT(17983)=(S=6020951,E=6020951,C=63)
	CT(17984)=(S=6020952,E=6020953,C=31)
	CT(17985)=(S=6020954,E=6020954,C=18)
	CT(17986)=(S=6020955,E=6020955,C=42)
	CT(17987)=(S=6020956,E=6020956,C=19)
	CT(17988)=(S=6020957,E=6020957,C=18)
	CT(17989)=(S=6020958,E=6020958,C=14)
	CT(17990)=(S=6020959,E=6020959,C=36)
	CT(17991)=(S=6020960,E=6020960,C=28)
	CT(17992)=(S=6020961,E=6020961,C=31)
	CT(17993)=(S=6020962,E=6020962,C=18)
	CT(17994)=(S=6020964,E=6020964,C=27)
	CT(17995)=(S=6020965,E=6020965,C=42)
	CT(17996)=(S=6020966,E=6020967,C=31)
	CT(17997)=(S=6020968,E=6020968,C=22)
	CT(17998)=(S=6020969,E=6020969,C=18)
	CT(17999)=(S=6020970,E=6020970,C=17)
	CT(18000)=(S=6020971,E=6020971,C=47)
	CT(18001)=(S=6020972,E=6020973,C=18)
	CT(18002)=(S=6020974,E=6020974,C=31)
	CT(18003)=(S=6020975,E=6020975,C=14)
	CT(18004)=(S=6020976,E=6020976,C=31)
	CT(18005)=(S=6020977,E=6020977,C=52)
	CT(18006)=(S=6020978,E=6020978,C=28)
	CT(18007)=(S=6020979,E=6020979,C=52)
	CT(18008)=(S=6020980,E=6020980,C=47)
	CT(18009)=(S=6020981,E=6020981,C=38)
	CT(18010)=(S=6020982,E=6020982,C=18)
	CT(18011)=(S=6020983,E=6020983,C=58)
	CT(18012)=(S=6020984,E=6020984,C=18)
	CT(18013)=(S=6020985,E=6020985,C=34)
	CT(18014)=(S=6020986,E=6020986,C=31)
	CT(18015)=(S=6020987,E=6020987,C=25)
	CT(18016)=(S=6020988,E=6020988,C=36)
	CT(18017)=(S=6020989,E=6020989,C=12)
	CT(18018)=(S=6020990,E=6020990,C=14)
	CT(18019)=(S=6020991,E=6020991,C=31)
	CT(18020)=(S=6020992,E=6020992,C=18)
	CT(18021)=(S=6020993,E=6020993,C=28)
	CT(18022)=(S=6020994,E=6020994,C=58)
	CT(18023)=(S=6020995,E=6020995,C=36)
	CT(18024)=(S=6020996,E=6020996,C=22)
	CT(18025)=(S=6020997,E=6020997,C=31)
	CT(18026)=(S=6020998,E=6020999,C=58)
	CT(18027)=(S=6021000,E=6021000,C=31)
	CT(18028)=(S=6021001,E=6021002,C=18)
	CT(18029)=(S=6021003,E=6021003,C=12)
	CT(18030)=(S=6021004,E=6021004,C=36)
	CT(18031)=(S=6021005,E=6021005,C=28)
	CT(18032)=(S=6021006,E=6021006,C=58)
	CT(18033)=(S=6021007,E=6021007,C=25)
	CT(18034)=(S=6021008,E=6021008,C=18)
	CT(18035)=(S=6021009,E=6021009,C=28)
	CT(18036)=(S=6021010,E=6021010,C=14)
	CT(18037)=(S=6021011,E=6021011,C=42)
	CT(18038)=(S=6021012,E=6021012,C=38)
	CT(18039)=(S=6021013,E=6021013,C=31)
	CT(18040)=(S=6021014,E=6021014,C=12)
	CT(18041)=(S=6021015,E=6021015,C=28)
	CT(18042)=(S=6021016,E=6021016,C=31)
	CT(18043)=(S=6021017,E=6021017,C=18)
	CT(18044)=(S=6021018,E=6021018,C=34)
	CT(18045)=(S=6021019,E=6021019,C=15)
	CT(18046)=(S=6021020,E=6021020,C=38)
	CT(18047)=(S=6021021,E=6021021,C=37)
	CT(18048)=(S=6021022,E=6021022,C=38)
	CT(18049)=(S=6021023,E=6021023,C=12)
	CT(18050)=(S=6021024,E=6021024,C=58)
	CT(18051)=(S=6021025,E=6021025,C=64)
	CT(18052)=(S=6021026,E=6021026,C=46)
	CT(18053)=(S=6021028,E=6021028,C=32)
	CT(18054)=(S=6021029,E=6021029,C=60)
	CT(18055)=(S=6021030,E=6021030,C=52)
	CT(18056)=(S=6021031,E=6021032,C=58)
	CT(18057)=(S=6021034,E=6021034,C=18)
	CT(18058)=(S=6021035,E=6021035,C=58)
	CT(18059)=(S=6021036,E=6021036,C=36)
	CT(18060)=(S=6021037,E=6021037,C=71)
	CT(18061)=(S=6021038,E=6021038,C=52)
	CT(18062)=(S=6021039,E=6021039,C=58)
	CT(18063)=(S=6021040,E=6021040,C=18)
	CT(18064)=(S=6021041,E=6021041,C=31)
	CT(18065)=(S=6021042,E=6021042,C=18)
	CT(18066)=(S=6021043,E=6021043,C=34)
	CT(18067)=(S=6021044,E=6021044,C=31)
	CT(18068)=(S=6021045,E=6021045,C=18)
	CT(18069)=(S=6021046,E=6021046,C=52)
	CT(18070)=(S=6021047,E=6021047,C=38)
	CT(18071)=(S=6021048,E=6021048,C=58)
	CT(18072)=(S=6021049,E=6021049,C=38)
	CT(18073)=(S=6021050,E=6021050,C=34)
	CT(18074)=(S=6021051,E=6021051,C=36)
	CT(18075)=(S=6021052,E=6021052,C=18)
	CT(18076)=(S=6021053,E=6021053,C=52)
	CT(18077)=(S=6021054,E=6021054,C=38)
	CT(18078)=(S=6021055,E=6021055,C=36)
	CT(18079)=(S=6021056,E=6021056,C=24)
	CT(18080)=(S=6021057,E=6021057,C=52)
	CT(18081)=(S=6021058,E=6021058,C=18)
	CT(18082)=(S=6021059,E=6021059,C=49)
	CT(18083)=(S=6021060,E=6021060,C=58)
	CT(18084)=(S=6021061,E=6021061,C=52)
	CT(18085)=(S=6021062,E=6021062,C=28)
	CT(18086)=(S=6021063,E=6021063,C=18)
	CT(18087)=(S=6021064,E=6021064,C=38)
	CT(18088)=(S=6021065,E=6021065,C=42)
	CT(18089)=(S=6021066,E=6021066,C=28)
	CT(18090)=(S=6021067,E=6021067,C=14)
	CT(18091)=(S=6021068,E=6021068,C=28)
	CT(18092)=(S=6021069,E=6021069,C=31)
	CT(18093)=(S=6021070,E=6021070,C=25)
	CT(18094)=(S=6021071,E=6021071,C=12)
	CT(18095)=(S=6021072,E=6021072,C=32)
	CT(18096)=(S=6021073,E=6021073,C=71)
	CT(18097)=(S=6021074,E=6021074,C=23)
	CT(18098)=(S=6021075,E=6021075,C=28)
	CT(18099)=(S=6021076,E=6021076,C=58)
	CT(18100)=(S=6021077,E=6021077,C=27)
	CT(18101)=(S=6021078,E=6021078,C=31)
	CT(18102)=(S=6021079,E=6021079,C=39)
	CT(18103)=(S=6021080,E=6021080,C=31)
	CT(18104)=(S=6021081,E=6021081,C=18)
	CT(18105)=(S=6021082,E=6021082,C=14)
	CT(18106)=(S=6021083,E=6021083,C=19)
	CT(18107)=(S=6021084,E=6021084,C=36)
	CT(18108)=(S=6021085,E=6021085,C=58)
	CT(18109)=(S=6021086,E=6021086,C=50)
	CT(18110)=(S=6021087,E=6021087,C=31)
	CT(18111)=(S=6021088,E=6021088,C=18)
	CT(18112)=(S=6021089,E=6021089,C=58)
	CT(18113)=(S=6021090,E=6021090,C=18)
	CT(18114)=(S=6021091,E=6021091,C=31)
	CT(18115)=(S=6021092,E=6021092,C=28)
	CT(18116)=(S=6021093,E=6021093,C=18)
	CT(18117)=(S=6021094,E=6021094,C=52)
	CT(18118)=(S=6021095,E=6021096,C=19)
	CT(18119)=(S=6021097,E=6021097,C=36)
	CT(18120)=(S=6021098,E=6021098,C=31)
	CT(18121)=(S=6021099,E=6021099,C=24)
	CT(18122)=(S=6021100,E=6021100,C=60)
	CT(18123)=(S=6021101,E=6021101,C=57)
	CT(18124)=(S=6021102,E=6021102,C=52)
	CT(18125)=(S=6021103,E=6021103,C=18)
	CT(18126)=(S=6021104,E=6021104,C=12)
	CT(18127)=(S=6021105,E=6021105,C=58)
	CT(18128)=(S=6021106,E=6021106,C=18)
	CT(18129)=(S=6021107,E=6021107,C=55)
	CT(18130)=(S=6021108,E=6021108,C=18)
	CT(18131)=(S=6021109,E=6021109,C=40)
	CT(18132)=(S=6021110,E=6021110,C=18)
	CT(18133)=(S=6021111,E=6021111,C=28)
	CT(18134)=(S=6021112,E=6021112,C=42)
	CT(18135)=(S=6021113,E=6021113,C=57)
	CT(18136)=(S=6021114,E=6021114,C=58)
	CT(18137)=(S=6021115,E=6021116,C=18)
	CT(18138)=(S=6021117,E=6021117,C=12)
	CT(18139)=(S=6021118,E=6021118,C=18)
	CT(18140)=(S=6021119,E=6021119,C=42)
	CT(18141)=(S=6021120,E=6021121,C=60)
	CT(18142)=(S=6021122,E=6021123,C=31)
	CT(18143)=(S=6021124,E=6021125,C=47)
	CT(18144)=(S=6021126,E=6021127,C=12)
	CT(18145)=(S=6021128,E=6021131,C=31)
	CT(18146)=(S=6021132,E=6021133,C=60)
	CT(18147)=(S=6021134,E=6021135,C=18)
	CT(18148)=(S=6021136,E=6021137,C=31)
	CT(18149)=(S=6021138,E=6021143,C=18)
	CT(18150)=(S=6021144,E=6021145,C=31)
	CT(18151)=(S=6021146,E=6021147,C=14)
	CT(18152)=(S=6021148,E=6021149,C=58)
	CT(18153)=(S=6021150,E=6021151,C=31)
	CT(18154)=(S=6021152,E=6021155,C=18)
	CT(18155)=(S=6021156,E=6021157,C=19)
	CT(18156)=(S=6021158,E=6021159,C=58)
	CT(18157)=(S=6021160,E=6021161,C=38)
	CT(18158)=(S=6021162,E=6021163,C=71)
	CT(18159)=(S=6021164,E=6021165,C=31)
	CT(18160)=(S=6021166,E=6021167,C=58)
	CT(18161)=(S=6021168,E=6021169,C=33)
	CT(18162)=(S=6021170,E=6021171,C=38)
	CT(18163)=(S=6021172,E=6021173,C=18)
	CT(18164)=(S=6021174,E=6021175,C=36)
	CT(18165)=(S=6021176,E=6021177,C=58)
	CT(18166)=(S=6021178,E=6021179,C=33)
	CT(18167)=(S=6021180,E=6021181,C=58)
	CT(18168)=(S=6021182,E=6021183,C=18)
	CT(18169)=(S=6021186,E=6021187,C=42)
	CT(18170)=(S=6021188,E=6021189,C=31)
	CT(18171)=(S=6021190,E=6021191,C=42)
	CT(18172)=(S=6021192,E=6021193,C=15)
	CT(18173)=(S=6021194,E=6021195,C=31)
	CT(18174)=(S=6021196,E=6021197,C=18)
	CT(18175)=(S=6021198,E=6021199,C=31)
	CT(18176)=(S=6021200,E=6021203,C=18)
	CT(18177)=(S=6021204,E=6021205,C=31)
	CT(18178)=(S=6021206,E=6021207,C=18)
	CT(18179)=(S=6021208,E=6021209,C=47)
	CT(18180)=(S=6021210,E=6021211,C=33)
	CT(18181)=(S=6021212,E=6021213,C=44)
	CT(18182)=(S=6021214,E=6021215,C=34)
	CT(18183)=(S=6021216,E=6021217,C=31)
	CT(18184)=(S=6021218,E=6021223,C=18)
	CT(18185)=(S=6021224,E=6021225,C=58)
	CT(18186)=(S=6021226,E=6021227,C=18)
	CT(18187)=(S=6021228,E=6021229,C=14)
	CT(18188)=(S=6021230,E=6021231,C=26)
	CT(18189)=(S=6021232,E=6021233,C=31)
	CT(18190)=(S=6021234,E=6021235,C=18)
	CT(18191)=(S=6021236,E=6021241,C=58)
	CT(18192)=(S=6021242,E=6021243,C=36)
	CT(18193)=(S=6021244,E=6021247,C=18)
	CT(18194)=(S=6021248,E=6021249,C=58)
	CT(18195)=(S=6021250,E=6021253,C=18)
	CT(18196)=(S=6021254,E=6021255,C=44)
	CT(18197)=(S=6021256,E=6021259,C=18)
	CT(18198)=(S=6021260,E=6021261,C=36)
	CT(18199)=(S=6021262,E=6021267,C=58)
	CT(18200)=(S=6021268,E=6021269,C=12)
	CT(18201)=(S=6021270,E=6021271,C=25)
	CT(18202)=(S=6021272,E=6021273,C=14)
	CT(18203)=(S=6021274,E=6021275,C=18)
	CT(18204)=(S=6021276,E=6021277,C=31)
	CT(18205)=(S=6021278,E=6021279,C=18)
	CT(18206)=(S=6021280,E=6021281,C=36)
	CT(18207)=(S=6021282,E=6021283,C=12)
	CT(18208)=(S=6021284,E=6021285,C=36)
	CT(18209)=(S=6021286,E=6021287,C=18)
	CT(18210)=(S=6021288,E=6021289,C=31)
	CT(18211)=(S=6021290,E=6021291,C=18)
	CT(18212)=(S=6021292,E=6021293,C=52)
	CT(18213)=(S=6021294,E=6021295,C=24)
	CT(18214)=(S=6021296,E=6021297,C=26)
	CT(18215)=(S=6021298,E=6021299,C=31)
	CT(18216)=(S=6021300,E=6021301,C=58)
	CT(18217)=(S=6021302,E=6021303,C=18)
	CT(18218)=(S=6021304,E=6021305,C=58)
	CT(18219)=(S=6021306,E=6021307,C=14)
	CT(18220)=(S=6021308,E=6021309,C=55)
	CT(18221)=(S=6021310,E=6021311,C=14)
	CT(18222)=(S=6021312,E=6021313,C=58)
	CT(18223)=(S=6021314,E=6021315,C=18)
	CT(18224)=(S=6021316,E=6021319,C=58)
	CT(18225)=(S=6021320,E=6021323,C=31)
	CT(18226)=(S=6021324,E=6021329,C=18)
	CT(18227)=(S=6021330,E=6021331,C=24)
	CT(18228)=(S=6021332,E=6021333,C=38)
	CT(18229)=(S=6021334,E=6021337,C=58)
	CT(18230)=(S=6021338,E=6021339,C=18)
	CT(18231)=(S=6021340,E=6021341,C=58)
	CT(18232)=(S=6021342,E=6021343,C=31)
	CT(18233)=(S=6021344,E=6021345,C=58)
	CT(18234)=(S=6021346,E=6021347,C=28)
	CT(18235)=(S=6021350,E=6021351,C=12)
	CT(18236)=(S=6021352,E=6021355,C=18)
	CT(18237)=(S=6021356,E=6021357,C=58)
	CT(18238)=(S=6021358,E=6021359,C=52)
	CT(18239)=(S=6021360,E=6021361,C=25)
	CT(18240)=(S=6021362,E=6021363,C=36)
	CT(18241)=(S=6021364,E=6021365,C=19)
	CT(18242)=(S=6021368,E=6021369,C=36)
	CT(18243)=(S=6021370,E=6021371,C=46)
	CT(18244)=(S=6021372,E=6021375,C=31)
	CT(18245)=(S=6021376,E=6021379,C=14)
	CT(18246)=(S=6021380,E=6021383,C=31)
	CT(18247)=(S=6021384,E=6021387,C=58)
	CT(18248)=(S=6021388,E=6021391,C=38)
	CT(18249)=(S=6021392,E=6021395,C=14)
	CT(18250)=(S=6021396,E=6021403,C=18)
	CT(18251)=(S=6021404,E=6021407,C=58)
	CT(18252)=(S=6021408,E=6021411,C=38)
	CT(18253)=(S=6021412,E=6021427,C=31)
	CT(18254)=(S=6021428,E=6021431,C=26)
	CT(18255)=(S=6021432,E=6021435,C=31)
	CT(18256)=(S=6021436,E=6021439,C=27)
	CT(18257)=(S=6021440,E=6021443,C=58)
	CT(18258)=(S=6021444,E=6021447,C=47)
	CT(18259)=(S=6021448,E=6021451,C=31)
	CT(18260)=(S=6021452,E=6021455,C=18)
	CT(18261)=(S=6021456,E=6021459,C=31)
	CT(18262)=(S=6021460,E=6021463,C=33)
	CT(18263)=(S=6021464,E=6021471,C=31)
	CT(18264)=(S=6021472,E=6021475,C=52)
	CT(18265)=(S=6021476,E=6021479,C=18)
	CT(18266)=(S=6021480,E=6021483,C=44)
	CT(18267)=(S=6021484,E=6021487,C=31)
	CT(18268)=(S=6021488,E=6021491,C=18)
	CT(18269)=(S=6021492,E=6021499,C=31)
	CT(18270)=(S=6021500,E=6021507,C=18)
	CT(18271)=(S=6021508,E=6021511,C=58)
	CT(18272)=(S=6021512,E=6021523,C=31)
	CT(18273)=(S=6021524,E=6021527,C=18)
	CT(18274)=(S=6021528,E=6021531,C=31)
	CT(18275)=(S=6021532,E=6021535,C=58)
	CT(18276)=(S=6021536,E=6021551,C=31)
	CT(18277)=(S=6021552,E=6021555,C=58)
	CT(18278)=(S=6021556,E=6021559,C=15)
	CT(18279)=(S=6021560,E=6021567,C=58)
	CT(18280)=(S=6021568,E=6021571,C=18)
	CT(18281)=(S=6021572,E=6021579,C=31)
	CT(18282)=(S=6021580,E=6021583,C=38)
	CT(18283)=(S=6021584,E=6021595,C=18)
	CT(18284)=(S=6021596,E=6021611,C=31)
	CT(18285)=(S=6021612,E=6021615,C=18)
	CT(18286)=(S=6021616,E=6021619,C=58)
	CT(18287)=(S=6021620,E=6021623,C=15)
	CT(18288)=(S=6021624,E=6021627,C=29)
	CT(18289)=(S=6021628,E=6021631,C=18)
	CT(18290)=(S=6021632,E=6021633,C=14)
	CT(18291)=(S=6021634,E=6021635,C=38)
	CT(18292)=(S=6021636,E=6021637,C=31)
	CT(18293)=(S=6021638,E=6021639,C=58)
	CT(18294)=(S=6021640,E=6021641,C=57)
	CT(18295)=(S=6021642,E=6021645,C=18)
	CT(18296)=(S=6021646,E=6021647,C=31)
	CT(18297)=(S=6021648,E=6021649,C=58)
	CT(18298)=(S=6021650,E=6021651,C=12)
	CT(18299)=(S=6021652,E=6021653,C=54)
	CT(18300)=(S=6021654,E=6021655,C=58)
	CT(18301)=(S=6021656,E=6021657,C=14)
	CT(18302)=(S=6021658,E=6021659,C=58)
	CT(18303)=(S=6021660,E=6021661,C=12)
	CT(18304)=(S=6021662,E=6021663,C=18)
	CT(18305)=(S=6021664,E=6021665,C=60)
	CT(18306)=(S=6021666,E=6021667,C=31)
	CT(18307)=(S=6021668,E=6021669,C=19)
	CT(18308)=(S=6021670,E=6021673,C=58)
	CT(18309)=(S=6021674,E=6021677,C=31)
	CT(18310)=(S=6021678,E=6021679,C=28)
	CT(18311)=(S=6021680,E=6021681,C=18)
	CT(18312)=(S=6021682,E=6021683,C=58)
	CT(18313)=(S=6021684,E=6021687,C=38)
	CT(18314)=(S=6021688,E=6021691,C=31)
	CT(18315)=(S=6021692,E=6021699,C=18)
	CT(18316)=(S=6021700,E=6021703,C=58)
	CT(18317)=(S=6021705,E=6021705,C=18)
	CT(18318)=(S=6021706,E=6021707,C=15)
	CT(18319)=(S=6021709,E=6021709,C=52)
	CT(18320)=(S=6021711,E=6021711,C=50)
	CT(18321)=(S=6021712,E=6021715,C=18)
	CT(18322)=(S=6021720,E=6021723,C=28)
	CT(18323)=(S=6021724,E=6021729,C=18)
	CT(18324)=(S=6021730,E=6021730,C=12)
	CT(18325)=(S=6021731,E=6021731,C=36)
	CT(18326)=(S=6021732,E=6021735,C=58)
	CT(18327)=(S=6021736,E=6021739,C=18)
	CT(18328)=(S=6021740,E=6021741,C=15)
	CT(18329)=(S=6021742,E=6021743,C=58)
	CT(18330)=(S=6021744,E=6021747,C=71)
	CT(18331)=(S=6021748,E=6021748,C=18)
	CT(18332)=(S=6021749,E=6021749,C=58)
	CT(18333)=(S=6021750,E=6021751,C=19)
	CT(18334)=(S=6021752,E=6021755,C=18)
	CT(18335)=(S=6021756,E=6021759,C=33)
	CT(18336)=(S=6021760,E=6021763,C=18)
	CT(18337)=(S=6021764,E=6021767,C=31)
	CT(18338)=(S=6021768,E=6021769,C=18)
	CT(18339)=(S=6021770,E=6021771,C=31)
	CT(18340)=(S=6021772,E=6021775,C=18)
	CT(18341)=(S=6021776,E=6021777,C=19)
	CT(18342)=(S=6021778,E=6021778,C=58)
	CT(18343)=(S=6021779,E=6021779,C=15)
	CT(18344)=(S=6021780,E=6021787,C=18)
	CT(18345)=(S=6021788,E=6021791,C=42)
	CT(18346)=(S=6021792,E=6021792,C=14)
	CT(18347)=(S=6021793,E=6021793,C=18)
	CT(18348)=(S=6021794,E=6021795,C=28)
	CT(18349)=(S=6021796,E=6021799,C=18)
	CT(18350)=(S=6021800,E=6021801,C=20)
	CT(18351)=(S=6021802,E=6021802,C=57)
	CT(18352)=(S=6021803,E=6021807,C=18)
	CT(18353)=(S=6021808,E=6021809,C=25)
	CT(18354)=(S=6021810,E=6021810,C=18)
	CT(18355)=(S=6021811,E=6021811,C=40)
	CT(18356)=(S=6021812,E=6021812,C=19)
	CT(18357)=(S=6021813,E=6021813,C=14)
	CT(18358)=(S=6021814,E=6021815,C=49)
	CT(18359)=(S=6021817,E=6021818,C=18)
	CT(18360)=(S=6021819,E=6021819,C=31)
	CT(18361)=(S=6021820,E=6021821,C=18)
	CT(18362)=(S=6021822,E=6021824,C=31)
	CT(18363)=(S=6021825,E=6021825,C=12)
	CT(18364)=(S=6021826,E=6021831,C=58)
	CT(18365)=(S=6021832,E=6021832,C=18)
	CT(18366)=(S=6021833,E=6021833,C=31)
	CT(18367)=(S=6021834,E=6021835,C=34)
	CT(18368)=(S=6021836,E=6021837,C=18)
	CT(18369)=(S=6021838,E=6021838,C=14)
	CT(18370)=(S=6021839,E=6021839,C=31)
	CT(18371)=(S=6021840,E=6021843,C=18)
	CT(18372)=(S=6021844,E=6021845,C=31)
	CT(18373)=(S=6021846,E=6021847,C=12)
	CT(18374)=(S=6021848,E=6021851,C=33)
	CT(18375)=(S=6021852,E=6021852,C=96)
	CT(18376)=(S=6021854,E=6021855,C=18)
	CT(18377)=(S=6021856,E=6021857,C=26)
	CT(18378)=(S=6021858,E=6021859,C=51)
	CT(18379)=(S=6021860,E=6021865,C=58)
	CT(18380)=(S=6021866,E=6021866,C=18)
	CT(18381)=(S=6021867,E=6021867,C=66)
	CT(18382)=(S=6021868,E=6021871,C=58)
	CT(18383)=(S=6021872,E=6021875,C=46)
	CT(18384)=(S=6021876,E=6021876,C=57)
	CT(18385)=(S=6021877,E=6021877,C=17)
	CT(18386)=(S=6021878,E=6021878,C=52)
	CT(18387)=(S=6021879,E=6021879,C=18)
	CT(18388)=(S=6021880,E=6021881,C=28)
	CT(18389)=(S=6021882,E=6021882,C=18)
	CT(18390)=(S=6021883,E=6021883,C=58)
	CT(18391)=(S=6021884,E=6021887,C=31)
	CT(18392)=(S=6021888,E=6021891,C=58)
	CT(18393)=(S=6021892,E=6021894,C=37)
	CT(18394)=(S=6021895,E=6021895,C=33)
	CT(18395)=(S=6021896,E=6021901,C=18)
	CT(18396)=(S=6021902,E=6021903,C=98)
	CT(18397)=(S=6021904,E=6021907,C=18)
	CT(18398)=(S=6021908,E=6021908,C=28)
	CT(18399)=(S=6021909,E=6021909,C=58)
	CT(18400)=(S=6021910,E=6021910,C=23)
	CT(18401)=(S=6021911,E=6021911,C=18)
	CT(18402)=(S=6021912,E=6021913,C=23)
	CT(18403)=(S=6021914,E=6021914,C=14)
	CT(18404)=(S=6021915,E=6021915,C=36)
	CT(18405)=(S=6021916,E=6021920,C=18)
	CT(18406)=(S=6021921,E=6021921,C=38)
	CT(18407)=(S=6021922,E=6021923,C=18)
	CT(18408)=(S=6021924,E=6021929,C=58)
	CT(18409)=(S=6021930,E=6021930,C=31)
	CT(18410)=(S=6021931,E=6021931,C=52)
	CT(18411)=(S=6021932,E=6021935,C=18)
	CT(18412)=(S=6021936,E=6021936,C=34)
	CT(18413)=(S=6021937,E=6021937,C=19)
	CT(18414)=(S=6021938,E=6021943,C=18)
	CT(18415)=(S=6021944,E=6021949,C=58)
	CT(18416)=(S=6021950,E=6021950,C=14)
	CT(18417)=(S=6021951,E=6021955,C=58)
	CT(18418)=(S=6021956,E=6021956,C=18)
	CT(18419)=(S=6021957,E=6021957,C=31)
	CT(18420)=(S=6021958,E=6021959,C=17)
	CT(18421)=(S=6021960,E=6021961,C=38)
	CT(18422)=(S=6021962,E=6021962,C=18)
	CT(18423)=(S=6021963,E=6021963,C=36)
	CT(18424)=(S=6021964,E=6021965,C=58)
	CT(18425)=(S=6021966,E=6021966,C=14)
	CT(18426)=(S=6021967,E=6021967,C=18)
	CT(18427)=(S=6021968,E=6021971,C=58)
	CT(18428)=(S=6021972,E=6021975,C=18)
	CT(18429)=(S=6021976,E=6021977,C=58)
	CT(18430)=(S=6021978,E=6021978,C=38)
	CT(18431)=(S=6021979,E=6021983,C=18)
	CT(18432)=(S=6021984,E=6021985,C=33)
	CT(18433)=(S=6021986,E=6021987,C=28)
	CT(18434)=(S=6021988,E=6021988,C=21)
	CT(18435)=(S=6021989,E=6021989,C=60)
	CT(18436)=(S=6021990,E=6021995,C=58)
	CT(18437)=(S=6021996,E=6021996,C=22)
	CT(18438)=(S=6021997,E=6021997,C=15)
	CT(18439)=(S=6021998,E=6021999,C=52)
	CT(18440)=(S=6022000,E=6022001,C=18)
	CT(18441)=(S=6022002,E=6022002,C=36)
	CT(18442)=(S=6022003,E=6022008,C=58)
	CT(18443)=(S=6022009,E=6022009,C=53)
	CT(18444)=(S=6022010,E=6022011,C=58)
	CT(18445)=(S=6022012,E=6022015,C=14)
	CT(18446)=(S=6022016,E=6022018,C=42)
	CT(18447)=(S=6022019,E=6022019,C=38)
	CT(18448)=(S=6022022,E=6022023,C=33)
	CT(18449)=(S=6022024,E=6022024,C=18)
	CT(18450)=(S=6022026,E=6022027,C=50)
	CT(18451)=(S=6022028,E=6022031,C=18)
	CT(18452)=(S=6022032,E=6022035,C=20)
	CT(18453)=(S=6022036,E=6022037,C=18)
	CT(18454)=(S=6022038,E=6022039,C=31)
	CT(18455)=(S=6022040,E=6022043,C=18)
	CT(18456)=(S=6022044,E=6022047,C=58)
	CT(18457)=(S=6022048,E=6022051,C=18)
	CT(18458)=(S=6022052,E=6022053,C=17)
	CT(18459)=(S=6022054,E=6022054,C=38)
	CT(18460)=(S=6022055,E=6022055,C=58)
	CT(18461)=(S=6022056,E=6022063,C=18)
	CT(18462)=(S=6022064,E=6022064,C=58)
	CT(18463)=(S=6022065,E=6022065,C=24)
	CT(18464)=(S=6022066,E=6022066,C=14)
	CT(18465)=(S=6022067,E=6022067,C=36)
	CT(18466)=(S=6022068,E=6022071,C=31)
	CT(18467)=(S=6022072,E=6022072,C=18)
	CT(18468)=(S=6022073,E=6022073,C=53)
	CT(18469)=(S=6022074,E=6022074,C=19)
	CT(18470)=(S=6022075,E=6022075,C=47)
	CT(18471)=(S=6022076,E=6022079,C=18)
	CT(18472)=(S=6022080,E=6022081,C=47)
	CT(18473)=(S=6022082,E=6022082,C=12)
	CT(18474)=(S=6022083,E=6022083,C=33)
	CT(18475)=(S=6022084,E=6022085,C=58)
	CT(18476)=(S=6022086,E=6022086,C=18)
	CT(18477)=(S=6022087,E=6022091,C=58)
	CT(18478)=(S=6022092,E=6022093,C=42)
	CT(18479)=(S=6022094,E=6022095,C=31)
	CT(18480)=(S=6022096,E=6022096,C=19)
	CT(18481)=(S=6022097,E=6022097,C=49)
	CT(18482)=(S=6022098,E=6022099,C=18)
	CT(18483)=(S=6022100,E=6022107,C=58)
	CT(18484)=(S=6022108,E=6022111,C=14)
	CT(18485)=(S=6022112,E=6022115,C=71)
	CT(18486)=(S=6022116,E=6022116,C=18)
	CT(18487)=(S=6022117,E=6022117,C=36)
	CT(18488)=(S=6022118,E=6022119,C=42)
	CT(18489)=(S=6022120,E=6022123,C=33)
	CT(18490)=(S=6022124,E=6022127,C=42)
	CT(18491)=(S=6022128,E=6022132,C=18)
	CT(18492)=(S=6022133,E=6022133,C=14)
	CT(18493)=(S=6022136,E=6022139,C=27)
	CT(18494)=(S=6022140,E=6022140,C=34)
	CT(18495)=(S=6022142,E=6022142,C=31)
	CT(18496)=(S=6022143,E=6022143,C=40)
	CT(18497)=(S=6022144,E=6022147,C=14)
	CT(18498)=(S=6022148,E=6022151,C=60)
	CT(18499)=(S=6022152,E=6022153,C=58)
	CT(18500)=(S=6022154,E=6022154,C=18)
	CT(18501)=(S=6022155,E=6022155,C=58)
	CT(18502)=(S=6022156,E=6022159,C=31)
	CT(18503)=(S=6022160,E=6022165,C=46)
	CT(18504)=(S=6022166,E=6022167,C=62)
	CT(18505)=(S=6022168,E=6022169,C=63)
	CT(18506)=(S=6022170,E=6022171,C=18)
	CT(18507)=(S=6022172,E=6022172,C=58)
	CT(18508)=(S=6022173,E=6022173,C=38)
	CT(18509)=(S=6022174,E=6022175,C=18)
	CT(18510)=(S=6022176,E=6022179,C=58)
	CT(18511)=(S=6022180,E=6022181,C=38)
	CT(18512)=(S=6022182,E=6022182,C=51)
	CT(18513)=(S=6022183,E=6022183,C=20)
	CT(18514)=(S=6022184,E=6022185,C=18)
	CT(18515)=(S=6022186,E=6022187,C=58)
	CT(18516)=(S=6022188,E=6022191,C=33)
	CT(18517)=(S=6022192,E=6022195,C=18)
	CT(18518)=(S=6022196,E=6022197,C=28)
	CT(18519)=(S=6022198,E=6022199,C=18)
	CT(18520)=(S=6022200,E=6022200,C=58)
	CT(18521)=(S=6022201,E=6022201,C=18)
	CT(18522)=(S=6022202,E=6022203,C=31)
	CT(18523)=(S=6022204,E=6022205,C=34)
	CT(18524)=(S=6022206,E=6022211,C=18)
	CT(18525)=(S=6022212,E=6022215,C=31)
	CT(18526)=(S=6022216,E=6022219,C=14)
	CT(18527)=(S=6022220,E=6022221,C=18)
	CT(18528)=(S=6022222,E=6022223,C=14)
	CT(18529)=(S=6022224,E=6022227,C=58)
	CT(18530)=(S=6022228,E=6022231,C=44)
	CT(18531)=(S=6022232,E=6022233,C=58)
	CT(18532)=(S=6022234,E=6022234,C=19)
	CT(18533)=(S=6022235,E=6022235,C=25)
	CT(18534)=(S=6022236,E=6022239,C=58)
	CT(18535)=(S=6022240,E=6022243,C=18)
	CT(18536)=(S=6022244,E=6022244,C=33)
	CT(18537)=(S=6022245,E=6022245,C=28)
	CT(18538)=(S=6022246,E=6022247,C=18)
	CT(18539)=(S=6022248,E=6022251,C=28)
	CT(18540)=(S=6022252,E=6022255,C=53)
	CT(18541)=(S=6022256,E=6022257,C=58)
	CT(18542)=(S=6022258,E=6022258,C=24)
	CT(18543)=(S=6022259,E=6022259,C=14)
	CT(18544)=(S=6022260,E=6022262,C=18)
	CT(18545)=(S=6022263,E=6022263,C=38)
	CT(18546)=(S=6022264,E=6022265,C=18)
	CT(18547)=(S=6022266,E=6022267,C=31)
	CT(18548)=(S=6022268,E=6022268,C=18)
	CT(18549)=(S=6022269,E=6022269,C=58)
	CT(18550)=(S=6022270,E=6022271,C=17)
	CT(18551)=(S=6022272,E=6022275,C=38)
	CT(18552)=(S=6022279,E=6022279,C=18)
	CT(18553)=(S=6022280,E=6022283,C=58)
	CT(18554)=(S=6022284,E=6022289,C=38)
	CT(18555)=(S=6022290,E=6022291,C=31)
	CT(18556)=(S=6022292,E=6022293,C=18)
	CT(18557)=(S=6022294,E=6022294,C=28)
	CT(18558)=(S=6022295,E=6022295,C=36)
	CT(18559)=(S=6022296,E=6022299,C=28)
	CT(18560)=(S=6022300,E=6022303,C=58)
	CT(18561)=(S=6022305,E=6022305,C=18)
	CT(18562)=(S=6022306,E=6022307,C=17)
	CT(18563)=(S=6022308,E=6022311,C=71)
	CT(18564)=(S=6022312,E=6022312,C=12)
	CT(18565)=(S=6022313,E=6022313,C=37)
	CT(18566)=(S=6022314,E=6022315,C=28)
	CT(18567)=(S=6022320,E=6022320,C=36)
	CT(18568)=(S=6022321,E=6022328,C=18)
	CT(18569)=(S=6022329,E=6022329,C=58)
	CT(18570)=(S=6022330,E=6022331,C=14)
	CT(18571)=(S=6022332,E=6022332,C=12)
	CT(18572)=(S=6022333,E=6022333,C=26)
	CT(18573)=(S=6022334,E=6022335,C=28)
	CT(18574)=(S=6022336,E=6022336,C=20)
	CT(18575)=(S=6022337,E=6022339,C=19)
	CT(18576)=(S=6022340,E=6022343,C=58)
	CT(18577)=(S=6022344,E=6022347,C=18)
	CT(18578)=(S=6022348,E=6022348,C=58)
	CT(18579)=(S=6022349,E=6022349,C=18)
	CT(18580)=(S=6022350,E=6022350,C=14)
	CT(18581)=(S=6022351,E=6022351,C=52)
	CT(18582)=(S=6022352,E=6022355,C=18)
	CT(18583)=(S=6022356,E=6022357,C=50)
	CT(18584)=(S=6022358,E=6022358,C=27)
	CT(18585)=(S=6022359,E=6022359,C=49)
	CT(18586)=(S=6022360,E=6022363,C=33)
	CT(18587)=(S=6022364,E=6022372,C=18)
	CT(18588)=(S=6022373,E=6022373,C=31)
	CT(18589)=(S=6022374,E=6022374,C=18)
	CT(18590)=(S=6022376,E=6022378,C=14)
	CT(18591)=(S=6022379,E=6022379,C=28)
	CT(18592)=(S=6022380,E=6022380,C=31)
	CT(18593)=(S=6022381,E=6022381,C=58)
	CT(18594)=(S=6022382,E=6022383,C=18)
	CT(18595)=(S=6022384,E=6022385,C=25)
	CT(18596)=(S=6022386,E=6022386,C=14)
	CT(18597)=(S=6022387,E=6022387,C=28)
	CT(18598)=(S=6022388,E=6022389,C=71)
	CT(18599)=(S=6022390,E=6022391,C=18)
	CT(18600)=(S=6022392,E=6022392,C=17)
	CT(18601)=(S=6022393,E=6022393,C=38)
	CT(18602)=(S=6022394,E=6022394,C=18)
	CT(18603)=(S=6022395,E=6022395,C=14)
	CT(18604)=(S=6022396,E=6022398,C=46)
	CT(18605)=(S=6022399,E=6022399,C=37)
	CT(18606)=(S=6022400,E=6022401,C=38)
	CT(18607)=(S=6022402,E=6022402,C=18)
	CT(18608)=(S=6022403,E=6022403,C=28)
	CT(18609)=(S=6022404,E=6022404,C=14)
	CT(18610)=(S=6022406,E=6022407,C=38)
	CT(18611)=(S=6022408,E=6022411,C=18)
	CT(18612)=(S=6022412,E=6022419,C=38)
	CT(18613)=(S=6022420,E=6022420,C=12)
	CT(18614)=(S=6022421,E=6022421,C=27)
	CT(18615)=(S=6022422,E=6022422,C=58)
	CT(18616)=(S=6022423,E=6022423,C=36)
	CT(18617)=(S=6022424,E=6022424,C=33)
	CT(18618)=(S=6022426,E=6022427,C=58)
	CT(18619)=(S=6022428,E=6022429,C=71)
	CT(18620)=(S=6022430,E=6022431,C=38)
	CT(18621)=(S=6022432,E=6022433,C=23)
	CT(18622)=(S=6022434,E=6022435,C=37)
	CT(18623)=(S=6022436,E=6022439,C=71)
	CT(18624)=(S=6022440,E=6022443,C=58)
	CT(18625)=(S=6022444,E=6022445,C=37)
	CT(18626)=(S=6022448,E=6022448,C=18)
	CT(18627)=(S=6022449,E=6022449,C=36)
	CT(18628)=(S=6022450,E=6022451,C=18)
	CT(18629)=(S=6022452,E=6022455,C=31)
	CT(18630)=(S=6022457,E=6022457,C=42)
	CT(18631)=(S=6022458,E=6022458,C=25)
	CT(18632)=(S=6022459,E=6022459,C=18)
	CT(18633)=(S=6022460,E=6022463,C=36)
	CT(18634)=(S=6022464,E=6022465,C=16)
	CT(18635)=(S=6022466,E=6022467,C=31)
	CT(18636)=(S=6022468,E=6022471,C=58)
	CT(18637)=(S=6022472,E=6022472,C=52)
	CT(18638)=(S=6022473,E=6022473,C=14)
	CT(18639)=(S=6022474,E=6022475,C=18)
	CT(18640)=(S=6022476,E=6022479,C=31)
	CT(18641)=(S=6022480,E=6022481,C=15)
	CT(18642)=(S=6022482,E=6022489,C=58)
	CT(18643)=(S=6022492,E=6022492,C=14)
	CT(18644)=(S=6022493,E=6022493,C=50)
	CT(18645)=(S=6022494,E=6022494,C=18)
	CT(18646)=(S=6022495,E=6022495,C=12)
	CT(18647)=(S=6022496,E=6022500,C=18)
	CT(18648)=(S=6022501,E=6022501,C=31)
	CT(18649)=(S=6022502,E=6022503,C=34)
	CT(18650)=(S=6022504,E=6022505,C=62)
	CT(18651)=(S=6022506,E=6022507,C=58)
	CT(18652)=(S=6022508,E=6022513,C=18)
	CT(18653)=(S=6022514,E=6022514,C=19)
	CT(18654)=(S=6022515,E=6022515,C=31)
	CT(18655)=(S=6022516,E=6022519,C=18)
	CT(18656)=(S=6022520,E=6022523,C=31)
	CT(18657)=(S=6022524,E=6022527,C=14)
	CT(18658)=(S=6022528,E=6022529,C=31)
	CT(18659)=(S=6022530,E=6022536,C=12)
	CT(18660)=(S=6022537,E=6022539,C=47)
	CT(18661)=(S=6022540,E=6022540,C=34)
	CT(18662)=(S=6022541,E=6022541,C=19)
	CT(18663)=(S=6022542,E=6022543,C=27)
	CT(18664)=(S=6022544,E=6022545,C=36)
	CT(18665)=(S=6022546,E=6022546,C=18)
	CT(18666)=(S=6022547,E=6022547,C=38)
	CT(18667)=(S=6022548,E=6022551,C=33)
	CT(18668)=(S=6022552,E=6022553,C=36)
	CT(18669)=(S=6022554,E=6022554,C=18)
	CT(18670)=(S=6022555,E=6022555,C=37)
	CT(18671)=(S=6022556,E=6022559,C=38)
	CT(18672)=(S=6022560,E=6022565,C=98)
	CT(18673)=(S=6022566,E=6022566,C=14)
	CT(18674)=(S=6022567,E=6022567,C=31)
	CT(18675)=(S=6022568,E=6022571,C=28)
	CT(18676)=(S=6022572,E=6022572,C=12)
	CT(18677)=(S=6022573,E=6022575,C=14)
	CT(18678)=(S=6022576,E=6022577,C=18)
	CT(18679)=(S=6022578,E=6022579,C=28)
	CT(18680)=(S=6022580,E=6022580,C=25)
	CT(18681)=(S=6022581,E=6022581,C=14)
	CT(18682)=(S=6022583,E=6022583,C=38)
	CT(18683)=(S=6022584,E=6022584,C=37)
	CT(18684)=(S=6022585,E=6022585,C=18)
	CT(18685)=(S=6022586,E=6022586,C=38)
	CT(18686)=(S=6022587,E=6022587,C=14)
	CT(18687)=(S=6022588,E=6022591,C=18)
	CT(18688)=(S=6022592,E=6022592,C=63)
	CT(18689)=(S=6022593,E=6022593,C=58)
	CT(18690)=(S=6022594,E=6022595,C=28)
	CT(18691)=(S=6022596,E=6022600,C=18)
	CT(18692)=(S=6022601,E=6022601,C=14)
	CT(18693)=(S=6022602,E=6022603,C=18)
	CT(18694)=(S=6022604,E=6022607,C=31)
	CT(18695)=(S=6022608,E=6022609,C=18)
	CT(18696)=(S=6022610,E=6022610,C=31)
	CT(18697)=(S=6022611,E=6022611,C=18)
	CT(18698)=(S=6022612,E=6022613,C=58)
	CT(18699)=(S=6022614,E=6022615,C=26)
	CT(18700)=(S=6022616,E=6022619,C=31)
	CT(18701)=(S=6022620,E=6022620,C=23)
	CT(18702)=(S=6022621,E=6022621,C=18)
	CT(18703)=(S=6022622,E=6022623,C=14)
	CT(18704)=(S=6022624,E=6022625,C=62)
	CT(18705)=(S=6022626,E=6022626,C=31)
	CT(18706)=(S=6022627,E=6022627,C=12)
	CT(18707)=(S=6022628,E=6022628,C=38)
	CT(18708)=(S=6022629,E=6022629,C=57)
	CT(18709)=(S=6022630,E=6022631,C=12)
	CT(18710)=(S=6022632,E=6022632,C=36)
	CT(18711)=(S=6022633,E=6022635,C=18)
	CT(18712)=(S=6022636,E=6022636,C=34)
	CT(18713)=(S=6022637,E=6022637,C=18)
	CT(18714)=(S=6022638,E=6022638,C=58)
	CT(18715)=(S=6022639,E=6022639,C=25)
	CT(18716)=(S=6022640,E=6022643,C=31)
	CT(18717)=(S=6022644,E=6022645,C=18)
	CT(18718)=(S=6022646,E=6022647,C=14)
	CT(18719)=(S=6022649,E=6022651,C=58)
	CT(18720)=(S=6022652,E=6022655,C=33)
	CT(18721)=(S=6022656,E=6022657,C=12)
	CT(18722)=(S=6022658,E=6022659,C=42)
	CT(18723)=(S=6022660,E=6022661,C=47)
	CT(18724)=(S=6022662,E=6022663,C=51)
	CT(18725)=(S=6022664,E=6022665,C=58)
	CT(18726)=(S=6022666,E=6022667,C=18)
	CT(18727)=(S=6022668,E=6022671,C=44)
	CT(18728)=(S=6022672,E=6022672,C=14)
	CT(18729)=(S=6022673,E=6022673,C=58)
	CT(18730)=(S=6022674,E=6022677,C=18)
	CT(18731)=(S=6022678,E=6022678,C=28)
	CT(18732)=(S=6022679,E=6022679,C=18)
	CT(18733)=(S=6022680,E=6022680,C=58)
	CT(18734)=(S=6022681,E=6022681,C=31)
	CT(18735)=(S=6022682,E=6022682,C=18)
	CT(18736)=(S=6022684,E=6022687,C=31)
	CT(18737)=(S=6022688,E=6022688,C=26)
	CT(18738)=(S=6022689,E=6022691,C=36)
	CT(18739)=(S=6022692,E=6022693,C=58)
	CT(18740)=(S=6022694,E=6022695,C=14)
	CT(18741)=(S=6022696,E=6022696,C=21)
	CT(18742)=(S=6022697,E=6022697,C=16)
	CT(18743)=(S=6022698,E=6022699,C=58)
	CT(18744)=(S=6022700,E=6022701,C=71)
	CT(18745)=(S=6022702,E=6022702,C=36)
	CT(18746)=(S=6022703,E=6022706,C=18)
	CT(18747)=(S=6022707,E=6022707,C=36)
	CT(18748)=(S=6022708,E=6022709,C=44)
	CT(18749)=(S=6022710,E=6022710,C=18)
	CT(18750)=(S=6022711,E=6022713,C=25)
	CT(18751)=(S=6022714,E=6022715,C=58)
	CT(18752)=(S=6022716,E=6022719,C=18)
	CT(18753)=(S=6022720,E=6022720,C=49)
	CT(18754)=(S=6022721,E=6022721,C=18)
	CT(18755)=(S=6022722,E=6022723,C=12)
	CT(18756)=(S=6022724,E=6022727,C=24)
	CT(18757)=(S=6022728,E=6022728,C=18)
	CT(18758)=(S=6022729,E=6022729,C=37)
	CT(18759)=(S=6022730,E=6022733,C=58)
	CT(18760)=(S=6022734,E=6022734,C=19)
	CT(18761)=(S=6022735,E=6022735,C=17)
	CT(18762)=(S=6022736,E=6022736,C=53)
	CT(18763)=(S=6022737,E=6022737,C=37)
	CT(18764)=(S=6022738,E=6022738,C=18)
	CT(18765)=(S=6022739,E=6022739,C=28)
	CT(18766)=(S=6022740,E=6022740,C=18)
	CT(18767)=(S=6022741,E=6022741,C=37)
	CT(18768)=(S=6022742,E=6022745,C=58)
	CT(18769)=(S=6022746,E=6022746,C=52)
	CT(18770)=(S=6022747,E=6022747,C=18)
	CT(18771)=(S=6022748,E=6022749,C=31)
	CT(18772)=(S=6022750,E=6022751,C=18)
	CT(18773)=(S=6022752,E=6022752,C=15)
	CT(18774)=(S=6022753,E=6022753,C=14)
	CT(18775)=(S=6022754,E=6022755,C=58)
	CT(18776)=(S=6022756,E=6022757,C=18)
	CT(18777)=(S=6022758,E=6022759,C=15)
	CT(18778)=(S=6022760,E=6022761,C=25)
	CT(18779)=(S=6022762,E=6022762,C=60)
	CT(18780)=(S=6022763,E=6022763,C=18)
	CT(18781)=(S=6022764,E=6022765,C=19)
	CT(18782)=(S=6022766,E=6022766,C=18)
	CT(18783)=(S=6022768,E=6022776,C=58)
	CT(18784)=(S=6022777,E=6022777,C=31)
	CT(18785)=(S=6022778,E=6022778,C=58)
	CT(18786)=(S=6022779,E=6022779,C=38)
	CT(18787)=(S=6022780,E=6022781,C=23)
	CT(18788)=(S=6022782,E=6022788,C=19)
	CT(18789)=(S=6022789,E=6022791,C=18)
	CT(18790)=(S=6022796,E=6022797,C=34)
	CT(18791)=(S=6022798,E=6022798,C=42)
	CT(18792)=(S=6022800,E=6022800,C=33)
	CT(18793)=(S=6022801,E=6022801,C=15)
	CT(18794)=(S=6022802,E=6022802,C=18)
	CT(18795)=(S=6022804,E=6022804,C=58)
	CT(18796)=(S=6022805,E=6022811,C=18)
	CT(18797)=(S=6022812,E=6022813,C=58)
	CT(18798)=(S=6022814,E=6022814,C=18)
	CT(18799)=(S=6022815,E=6022815,C=58)
	CT(18800)=(S=6022816,E=6022817,C=28)
	CT(18801)=(S=6022818,E=6022823,C=58)
	CT(18802)=(S=6022824,E=6022824,C=14)
	CT(18803)=(S=6022825,E=6022825,C=12)
	CT(18804)=(S=6022826,E=6022827,C=49)
	CT(18805)=(S=6022828,E=6022831,C=28)
	CT(18806)=(S=6022832,E=6022835,C=49)
	CT(18807)=(S=6022836,E=6022836,C=58)
	CT(18808)=(S=6022837,E=6022837,C=14)
	CT(18809)=(S=6022840,E=6022843,C=18)
	CT(18810)=(S=6022844,E=6022844,C=50)
	CT(18811)=(S=6022845,E=6022845,C=14)
	CT(18812)=(S=6022846,E=6022847,C=18)
	CT(18813)=(S=6022848,E=6022851,C=51)
	CT(18814)=(S=6022852,E=6022853,C=18)
	CT(18815)=(S=6022854,E=6022855,C=31)
	CT(18816)=(S=6022856,E=6022861,C=58)
	CT(18817)=(S=6022862,E=6022863,C=38)
	CT(18818)=(S=6022864,E=6022867,C=18)
	CT(18819)=(S=6022868,E=6022868,C=12)
	CT(18820)=(S=6022869,E=6022869,C=101)
	CT(18821)=(S=6022870,E=6022870,C=53)
	CT(18822)=(S=6022871,E=6022874,C=18)
	CT(18823)=(S=6022875,E=6022875,C=54)
	CT(18824)=(S=6022876,E=6022876,C=31)
	CT(18825)=(S=6022877,E=6022879,C=58)
	CT(18826)=(S=6022880,E=6022880,C=18)
	CT(18827)=(S=6022881,E=6022881,C=101)
	CT(18828)=(S=6022882,E=6022883,C=18)
	CT(18829)=(S=6022886,E=6022887,C=51)
	CT(18830)=(S=6022888,E=6022888,C=38)
	CT(18831)=(S=6022889,E=6022889,C=58)
	CT(18832)=(S=6022890,E=6022890,C=38)
	CT(18833)=(S=6022891,E=6022891,C=12)
	CT(18834)=(S=6022892,E=6022892,C=17)
	CT(18835)=(S=6022893,E=6022893,C=36)
	CT(18836)=(S=6022894,E=6022894,C=52)
	CT(18837)=(S=6022895,E=6022895,C=24)
	CT(18838)=(S=6022896,E=6022897,C=38)
	CT(18839)=(S=6022898,E=6022898,C=42)
	CT(18840)=(S=6022899,E=6022899,C=14)
	CT(18841)=(S=6022900,E=6022901,C=36)
	CT(18842)=(S=6022902,E=6022902,C=14)
	CT(18843)=(S=6022903,E=6022903,C=18)
	CT(18844)=(S=6022904,E=6022904,C=38)
	CT(18845)=(S=6022905,E=6022905,C=34)
	CT(18846)=(S=6022906,E=6022906,C=25)
	CT(18847)=(S=6022907,E=6022907,C=14)
	CT(18848)=(S=6022908,E=6022909,C=18)
	CT(18849)=(S=6022910,E=6022911,C=96)
	CT(18850)=(S=6022912,E=6022912,C=38)
	CT(18851)=(S=6022913,E=6022913,C=31)
	CT(18852)=(S=6022914,E=6022914,C=58)
	CT(18853)=(S=6022915,E=6022915,C=12)
	CT(18854)=(S=6022916,E=6022918,C=36)
	CT(18855)=(S=6022919,E=6022919,C=38)
	CT(18856)=(S=6022920,E=6022920,C=18)
	CT(18857)=(S=6022921,E=6022921,C=58)
	CT(18858)=(S=6022922,E=6022922,C=72)
	CT(18859)=(S=6022923,E=6022923,C=16)
	CT(18860)=(S=6022924,E=6022927,C=58)
	CT(18861)=(S=6022928,E=6022929,C=18)
	CT(18862)=(S=6022930,E=6022933,C=58)
	CT(18863)=(S=6022934,E=6022934,C=42)
	CT(18864)=(S=6022935,E=6022939,C=58)
	CT(18865)=(S=6022940,E=6022943,C=33)
	CT(18866)=(S=6022944,E=6022945,C=58)
	CT(18867)=(S=6022946,E=6022947,C=18)
	CT(18868)=(S=6022948,E=6022951,C=58)
	CT(18869)=(S=6022952,E=6022955,C=31)
	CT(18870)=(S=6022956,E=6022957,C=58)
	CT(18871)=(S=6022958,E=6022958,C=14)
	CT(18872)=(S=6022959,E=6022959,C=18)
	CT(18873)=(S=6022960,E=6022963,C=38)
	CT(18874)=(S=6022964,E=6022965,C=58)
	CT(18875)=(S=6022966,E=6022967,C=31)
	CT(18876)=(S=6022968,E=6022971,C=98)
	CT(18877)=(S=6022972,E=6022973,C=58)
	CT(18878)=(S=6022974,E=6022974,C=15)
	CT(18879)=(S=6022975,E=6022975,C=19)
	CT(18880)=(S=6022976,E=6022979,C=18)
	CT(18881)=(S=6022980,E=6022980,C=60)
	CT(18882)=(S=6022981,E=6022981,C=31)
	CT(18883)=(S=6022982,E=6022987,C=58)
	CT(18884)=(S=6022988,E=6022991,C=44)
	CT(18885)=(S=6022992,E=6022995,C=58)
	CT(18886)=(S=6022996,E=6022999,C=31)
	CT(18887)=(S=6023000,E=6023000,C=46)
	CT(18888)=(S=6023002,E=6023003,C=14)
	CT(18889)=(S=6023004,E=6023005,C=58)
	CT(18890)=(S=6023006,E=6023008,C=18)
	CT(18891)=(S=6023009,E=6023009,C=28)
	CT(18892)=(S=6023010,E=6023011,C=31)
	CT(18893)=(S=6023012,E=6023019,C=58)
	CT(18894)=(S=6023020,E=6023021,C=49)
	CT(18895)=(S=6023022,E=6023023,C=58)
	CT(18896)=(S=6023024,E=6023027,C=18)
	CT(18897)=(S=6023028,E=6023037,C=58)
	CT(18898)=(S=6023039,E=6023039,C=31)
	CT(18899)=(S=6023040,E=6023049,C=18)
	CT(18900)=(S=6023050,E=6023050,C=38)
	CT(18901)=(S=6023051,E=6023051,C=12)
	CT(18902)=(S=6023052,E=6023052,C=58)
	CT(18903)=(S=6023053,E=6023053,C=18)
	CT(18904)=(S=6023054,E=6023055,C=31)
	CT(18905)=(S=6023056,E=6023057,C=14)
	CT(18906)=(S=6023058,E=6023058,C=18)
	CT(18907)=(S=6023059,E=6023059,C=24)
	CT(18908)=(S=6023060,E=6023063,C=31)
	CT(18909)=(S=6023064,E=6023064,C=58)
	CT(18910)=(S=6023065,E=6023065,C=19)
	CT(18911)=(S=6023066,E=6023067,C=14)
	CT(18912)=(S=6023069,E=6023069,C=58)
	CT(18913)=(S=6023070,E=6023071,C=46)
	CT(18914)=(S=6023072,E=6023072,C=31)
	CT(18915)=(S=6023073,E=6023075,C=18)
	CT(18916)=(S=6023076,E=6023079,C=15)
	CT(18917)=(S=6023080,E=6023081,C=58)
	CT(18918)=(S=6023082,E=6023082,C=31)
	CT(18919)=(S=6023083,E=6023083,C=33)
	CT(18920)=(S=6023084,E=6023087,C=36)
	CT(18921)=(S=6023088,E=6023089,C=18)
	CT(18922)=(S=6023090,E=6023090,C=58)
	CT(18923)=(S=6023091,E=6023091,C=15)
	CT(18924)=(S=6023092,E=6023095,C=18)
	CT(18925)=(S=6023096,E=6023096,C=36)
	CT(18926)=(S=6023097,E=6023097,C=31)
	CT(18927)=(S=6023098,E=6023103,C=18)
	CT(18928)=(S=6023104,E=6023107,C=17)
	CT(18929)=(S=6023108,E=6023111,C=18)
	CT(18930)=(S=6023116,E=6023116,C=34)
	CT(18931)=(S=6023117,E=6023117,C=58)
	CT(18932)=(S=6023118,E=6023118,C=31)
	CT(18933)=(S=6023119,E=6023123,C=38)
	CT(18934)=(S=6023124,E=6023124,C=24)
	CT(18935)=(S=6023125,E=6023127,C=18)
	CT(18936)=(S=6023128,E=6023129,C=21)
	CT(18937)=(S=6023130,E=6023131,C=18)
	CT(18938)=(S=6023132,E=6023132,C=28)
	CT(18939)=(S=6023133,E=6023133,C=58)
	CT(18940)=(S=6023134,E=6023134,C=18)
	CT(18941)=(S=6023135,E=6023139,C=38)
	CT(18942)=(S=6023140,E=6023140,C=12)
	CT(18943)=(S=6023141,E=6023141,C=31)
	CT(18944)=(S=6023142,E=6023143,C=58)
	CT(18945)=(S=6023144,E=6023144,C=38)
	CT(18946)=(S=6023145,E=6023151,C=18)
	CT(18947)=(S=6023152,E=6023152,C=31)
	CT(18948)=(S=6023153,E=6023154,C=58)
	CT(18949)=(S=6023155,E=6023155,C=28)
	CT(18950)=(S=6023156,E=6023163,C=58)
	CT(18951)=(S=6023164,E=6023167,C=18)
	CT(18952)=(S=6023168,E=6023171,C=31)
	CT(18953)=(S=6023172,E=6023173,C=58)
	CT(18954)=(S=6023174,E=6023183,C=18)
	CT(18955)=(S=6023184,E=6023188,C=38)
	CT(18956)=(S=6023189,E=6023189,C=31)
	CT(18957)=(S=6023190,E=6023190,C=28)
	CT(18958)=(S=6023191,E=6023191,C=12)
	CT(18959)=(S=6023192,E=6023193,C=36)
	CT(18960)=(S=6023194,E=6023194,C=40)
	CT(18961)=(S=6023195,E=6023195,C=58)
	CT(18962)=(S=6023196,E=6023199,C=31)
	CT(18963)=(S=6023200,E=6023202,C=18)
	CT(18964)=(S=6023203,E=6023203,C=25)
	CT(18965)=(S=6023204,E=6023204,C=14)
	CT(18966)=(S=6023205,E=6023205,C=34)
	CT(18967)=(S=6023206,E=6023206,C=58)
	CT(18968)=(S=6023207,E=6023207,C=18)
	CT(18969)=(S=6023208,E=6023211,C=12)
	CT(18970)=(S=6023212,E=6023221,C=58)
	CT(18971)=(S=6023222,E=6023222,C=28)
	CT(18972)=(S=6023223,E=6023225,C=38)
	CT(18973)=(S=6023226,E=6023226,C=58)
	CT(18974)=(S=6023227,E=6023229,C=19)
	CT(18975)=(S=6023230,E=6023231,C=58)
	CT(18976)=(S=6023232,E=6023237,C=26)
	CT(18977)=(S=6023238,E=6023238,C=18)
	CT(18978)=(S=6023239,E=6023239,C=25)
	CT(18979)=(S=6023240,E=6023243,C=26)
	CT(18980)=(S=6023244,E=6023247,C=25)
	CT(18981)=(S=6023248,E=6023248,C=18)
	CT(18982)=(S=6023249,E=6023249,C=25)
	CT(18983)=(S=6023250,E=6023250,C=33)
	CT(18984)=(S=6023251,E=6023251,C=27)
	CT(18985)=(S=6023252,E=6023255,C=33)
	CT(18986)=(S=6023256,E=6023257,C=70)
	CT(18987)=(S=6023258,E=6023259,C=58)
	CT(18988)=(S=6023260,E=6023263,C=18)
	CT(18989)=(S=6023264,E=6023265,C=28)
	CT(18990)=(S=6023266,E=6023266,C=38)
	CT(18991)=(S=6023268,E=6023269,C=39)
	CT(18992)=(S=6023270,E=6023270,C=58)
	CT(18993)=(S=6023271,E=6023271,C=31)
	CT(18994)=(S=6023272,E=6023272,C=14)
	CT(18995)=(S=6023273,E=6023273,C=36)
	CT(18996)=(S=6023274,E=6023275,C=46)
	CT(18997)=(S=6023276,E=6023279,C=18)
	CT(18998)=(S=6023280,E=6023280,C=58)
	CT(18999)=(S=6023281,E=6023283,C=18)
	CT(19000)=(S=6023284,E=6023287,C=31)
	CT(19001)=(S=6023288,E=6023291,C=18)
	CT(19002)=(S=6023292,E=6023293,C=14)
	CT(19003)=(S=6023294,E=6023294,C=18)
	CT(19004)=(S=6023295,E=6023296,C=31)
	CT(19005)=(S=6023298,E=6023298,C=36)
	CT(19006)=(S=6023299,E=6023303,C=18)
	CT(19007)=(S=6023304,E=6023307,C=36)
	CT(19008)=(S=6023308,E=6023311,C=58)
	CT(19009)=(S=6023312,E=6023315,C=18)
	CT(19010)=(S=6023316,E=6023319,C=14)
	CT(19011)=(S=6023320,E=6023320,C=15)
	CT(19012)=(S=6023321,E=6023321,C=14)
	CT(19013)=(S=6023322,E=6023324,C=47)
	CT(19014)=(S=6023325,E=6023327,C=31)
	CT(19015)=(S=6023328,E=6023329,C=58)
	CT(19016)=(S=6023330,E=6023330,C=18)
	CT(19017)=(S=6023331,E=6023331,C=42)
	CT(19018)=(S=6023332,E=6023335,C=58)
	CT(19019)=(S=6023336,E=6023341,C=18)
	CT(19020)=(S=6023342,E=6023342,C=37)
	CT(19021)=(S=6023343,E=6023344,C=58)
	CT(19022)=(S=6023345,E=6023345,C=31)
	CT(19023)=(S=6023346,E=6023347,C=18)
	CT(19024)=(S=6023348,E=6023348,C=15)
	CT(19025)=(S=6023349,E=6023349,C=14)
	CT(19026)=(S=6023350,E=6023350,C=38)
	CT(19027)=(S=6023351,E=6023351,C=58)
	CT(19028)=(S=6023352,E=6023354,C=38)
	CT(19029)=(S=6023355,E=6023355,C=12)
	CT(19030)=(S=6023356,E=6023357,C=18)
	CT(19031)=(S=6023358,E=6023363,C=58)
	CT(19032)=(S=6023364,E=6023366,C=18)
	CT(19033)=(S=6023367,E=6023367,C=31)
	CT(19034)=(S=6023368,E=6023369,C=46)
	CT(19035)=(S=6023370,E=6023375,C=18)
	CT(19036)=(S=6023377,E=6023377,C=51)
	CT(19037)=(S=6023378,E=6023379,C=38)
	CT(19038)=(S=6023380,E=6023383,C=33)
	CT(19039)=(S=6023384,E=6023385,C=18)
	CT(19040)=(S=6023386,E=6023391,C=33)
	CT(19041)=(S=6023392,E=6023392,C=12)
	CT(19042)=(S=6023393,E=6023393,C=18)
	CT(19043)=(S=6023394,E=6023396,C=38)
	CT(19044)=(S=6023397,E=6023397,C=56)
	CT(19045)=(S=6023398,E=6023399,C=18)
	CT(19046)=(S=6023400,E=6023401,C=42)
	CT(19047)=(S=6023402,E=6023402,C=52)
	CT(19048)=(S=6023404,E=6023405,C=46)
	CT(19049)=(S=6023406,E=6023406,C=31)
	CT(19050)=(S=6023407,E=6023407,C=52)
	CT(19051)=(S=6023408,E=6023409,C=31)
	CT(19052)=(S=6023410,E=6023410,C=12)
	CT(19053)=(S=6023411,E=6023414,C=18)
	CT(19054)=(S=6023415,E=6023415,C=33)
	CT(19055)=(S=6023416,E=6023419,C=58)
	CT(19056)=(S=6023420,E=6023421,C=18)
	CT(19057)=(S=6023422,E=6023427,C=38)
	CT(19058)=(S=6023428,E=6023428,C=18)
	CT(19059)=(S=6023429,E=6023429,C=14)
	CT(19060)=(S=6023430,E=6023430,C=36)
	CT(19061)=(S=6023431,E=6023431,C=42)
	CT(19062)=(S=6023432,E=6023435,C=28)
	CT(19063)=(S=6023436,E=6023437,C=42)
	CT(19064)=(S=6023438,E=6023438,C=27)
	CT(19065)=(S=6023439,E=6023439,C=40)
	CT(19066)=(S=6023440,E=6023440,C=14)
	CT(19067)=(S=6023441,E=6023441,C=31)
	CT(19068)=(S=6023442,E=6023443,C=58)
	CT(19069)=(S=6023445,E=6023445,C=50)
	CT(19070)=(S=6023446,E=6023447,C=36)
	CT(19071)=(S=6023448,E=6023448,C=46)
	CT(19072)=(S=6023449,E=6023449,C=58)
	CT(19073)=(S=6023450,E=6023451,C=38)
	CT(19074)=(S=6023452,E=6023455,C=18)
	CT(19075)=(S=6023456,E=6023456,C=51)
	CT(19076)=(S=6023457,E=6023457,C=14)
	CT(19077)=(S=6023458,E=6023458,C=58)
	CT(19078)=(S=6023459,E=6023459,C=28)
	CT(19079)=(S=6023460,E=6023461,C=57)
	CT(19080)=(S=6023462,E=6023471,C=18)
	CT(19081)=(S=6023472,E=6023473,C=31)
	CT(19082)=(S=6023474,E=6023474,C=19)
	CT(19083)=(S=6023475,E=6023475,C=12)
	CT(19084)=(S=6023476,E=6023477,C=36)
	CT(19085)=(S=6023478,E=6023479,C=18)
	CT(19086)=(S=6023480,E=6023483,C=26)
	CT(19087)=(S=6023484,E=6023484,C=58)
	CT(19088)=(S=6023485,E=6023485,C=18)
	CT(19089)=(S=6023486,E=6023486,C=14)
	CT(19090)=(S=6023487,E=6023491,C=18)
	CT(19091)=(S=6023492,E=6023492,C=12)
	CT(19092)=(S=6023493,E=6023503,C=58)
	CT(19093)=(S=6023504,E=6023504,C=37)
	CT(19094)=(S=6023505,E=6023505,C=14)
	CT(19095)=(S=6023506,E=6023506,C=18)
	CT(19096)=(S=6023507,E=6023507,C=14)
	CT(19097)=(S=6023508,E=6023511,C=28)
	CT(19098)=(S=6023512,E=6023512,C=18)
	CT(19099)=(S=6023514,E=6023515,C=31)
	CT(19100)=(S=6023516,E=6023519,C=18)
	CT(19101)=(S=6023520,E=6023527,C=31)
	CT(19102)=(S=6023528,E=6023528,C=58)
	CT(19103)=(S=6023529,E=6023529,C=36)
	CT(19104)=(S=6023530,E=6023530,C=38)
	CT(19105)=(S=6023531,E=6023531,C=58)
	CT(19106)=(S=6023532,E=6023535,C=31)
	CT(19107)=(S=6023536,E=6023539,C=58)
	CT(19108)=(S=6023540,E=6023541,C=47)
	CT(19109)=(S=6023542,E=6023542,C=49)
	CT(19110)=(S=6023543,E=6023546,C=12)
	CT(19111)=(S=6023547,E=6023547,C=31)
	CT(19112)=(S=6023548,E=6023548,C=14)
	CT(19113)=(S=6023549,E=6023549,C=47)
	CT(19114)=(S=6023550,E=6023550,C=34)
	CT(19115)=(S=6023551,E=6023551,C=31)
	CT(19116)=(S=6023552,E=6023553,C=14)
	CT(19117)=(S=6023556,E=6023559,C=18)
	CT(19118)=(S=6023560,E=6023563,C=58)
	CT(19119)=(S=6023564,E=6023564,C=31)
	CT(19120)=(S=6023565,E=6023565,C=23)
	CT(19121)=(S=6023566,E=6023567,C=24)
	CT(19122)=(S=6023568,E=6023575,C=18)
	CT(19123)=(S=6023576,E=6023579,C=14)
	CT(19124)=(S=6023580,E=6023586,C=58)
	CT(19125)=(S=6023587,E=6023587,C=52)
	CT(19126)=(S=6023588,E=6023595,C=18)
	CT(19127)=(S=6023596,E=6023597,C=31)
	CT(19128)=(S=6023598,E=6023598,C=23)
	CT(19129)=(S=6023599,E=6023599,C=18)
	CT(19130)=(S=6023600,E=6023603,C=44)
	CT(19131)=(S=6023604,E=6023605,C=15)
	CT(19132)=(S=6023606,E=6023606,C=34)
	CT(19133)=(S=6023607,E=6023607,C=14)
	CT(19134)=(S=6023608,E=6023611,C=58)
	CT(19135)=(S=6023612,E=6023613,C=18)
	CT(19136)=(S=6023614,E=6023614,C=38)
	CT(19137)=(S=6023615,E=6023615,C=42)
	CT(19138)=(S=6023616,E=6023619,C=38)
	CT(19139)=(S=6023620,E=6023623,C=58)
	CT(19140)=(S=6023624,E=6023627,C=38)
	CT(19141)=(S=6023628,E=6023629,C=58)
	CT(19142)=(S=6023630,E=6023631,C=49)
	CT(19143)=(S=6023632,E=6023635,C=33)
	CT(19144)=(S=6023636,E=6023637,C=18)
	CT(19145)=(S=6023638,E=6023639,C=60)
	CT(19146)=(S=6023640,E=6023647,C=18)
	CT(19147)=(S=6023648,E=6023649,C=36)
	CT(19148)=(S=6023650,E=6023650,C=18)
	CT(19149)=(S=6023651,E=6023651,C=42)
	CT(19150)=(S=6023652,E=6023653,C=58)
	CT(19151)=(S=6023654,E=6023655,C=18)
	CT(19152)=(S=6023656,E=6023661,C=58)
	CT(19153)=(S=6023662,E=6023671,C=18)
	CT(19154)=(S=6023672,E=6023672,C=71)
	CT(19155)=(S=6023673,E=6023673,C=28)
	CT(19156)=(S=6023674,E=6023675,C=58)
	CT(19157)=(S=6023676,E=6023679,C=15)
	CT(19158)=(S=6023680,E=6023687,C=31)
	CT(19159)=(S=6023688,E=6023688,C=58)
	CT(19160)=(S=6023689,E=6023689,C=12)
	CT(19161)=(S=6023690,E=6023691,C=18)
	CT(19162)=(S=6023692,E=6023695,C=58)
	CT(19163)=(S=6023696,E=6023696,C=18)
	CT(19164)=(S=6023697,E=6023697,C=36)
	CT(19165)=(S=6023698,E=6023698,C=14)
	CT(19166)=(S=6023699,E=6023699,C=51)
	CT(19167)=(S=6023700,E=6023700,C=31)
	CT(19168)=(S=6023701,E=6023701,C=51)
	CT(19169)=(S=6023702,E=6023702,C=28)
	CT(19170)=(S=6023703,E=6023703,C=14)
	CT(19171)=(S=6023704,E=6023709,C=31)
	CT(19172)=(S=6023710,E=6023710,C=28)
	CT(19173)=(S=6023711,E=6023715,C=31)
	CT(19174)=(S=6023716,E=6023716,C=1)
	CT(19175)=(S=6023717,E=6023717,C=31)
	CT(19176)=(S=6023718,E=6023719,C=28)
	CT(19177)=(S=6023720,E=6023723,C=33)
	CT(19178)=(S=6023724,E=6023725,C=58)
	CT(19179)=(S=6023726,E=6023726,C=57)
	CT(19180)=(S=6023727,E=6023727,C=31)
	CT(19181)=(S=6023728,E=6023728,C=14)
	CT(19182)=(S=6023729,E=6023729,C=58)
	CT(19183)=(S=6023730,E=6023731,C=36)
	CT(19184)=(S=6023732,E=6023732,C=34)
	CT(19185)=(S=6023733,E=6023733,C=18)
	CT(19186)=(S=6023734,E=6023734,C=63)
	CT(19187)=(S=6023735,E=6023739,C=38)
	CT(19188)=(S=6023740,E=6023745,C=18)
	CT(19189)=(S=6023746,E=6023747,C=58)
	CT(19190)=(S=6023748,E=6023751,C=18)
	CT(19191)=(S=6023752,E=6023759,C=31)
	CT(19192)=(S=6023760,E=6023763,C=18)
	CT(19193)=(S=6023764,E=6023771,C=31)
	CT(19194)=(S=6023772,E=6023775,C=51)
	CT(19195)=(S=6023776,E=6023777,C=18)
	CT(19196)=(S=6023778,E=6023779,C=31)
	CT(19197)=(S=6023780,E=6023783,C=58)
	CT(19198)=(S=6023784,E=6023787,C=12)
	CT(19199)=(S=6023788,E=6023791,C=18)
	CT(19200)=(S=6023792,E=6023795,C=58)
	CT(19201)=(S=6023796,E=6023799,C=18)
	CT(19202)=(S=6023800,E=6023803,C=38)
	CT(19203)=(S=6023804,E=6023811,C=58)
	CT(19204)=(S=6023812,E=6023815,C=46)
	CT(19205)=(S=6023816,E=6023821,C=31)
	CT(19206)=(S=6023822,E=6023824,C=58)
	CT(19207)=(S=6023825,E=6023825,C=18)
	CT(19208)=(S=6023826,E=6023826,C=58)
	CT(19209)=(S=6023827,E=6023827,C=18)
	CT(19210)=(S=6023828,E=6023828,C=36)
	CT(19211)=(S=6023829,E=6023829,C=24)
	CT(19212)=(S=6023830,E=6023831,C=38)
	CT(19213)=(S=6023832,E=6023835,C=18)
	CT(19214)=(S=6023836,E=6023839,C=38)
	CT(19215)=(S=6023840,E=6023840,C=34)
	CT(19216)=(S=6023841,E=6023841,C=18)
	CT(19217)=(S=6023842,E=6023843,C=33)
	CT(19218)=(S=6023844,E=6023847,C=18)
	CT(19219)=(S=6023848,E=6023849,C=38)
	CT(19220)=(S=6023850,E=6023850,C=18)
	CT(19221)=(S=6023851,E=6023851,C=28)
	CT(19222)=(S=6023852,E=6023855,C=18)
	CT(19223)=(S=6023856,E=6023856,C=58)
	CT(19224)=(S=6023857,E=6023858,C=18)
	CT(19225)=(S=6023859,E=6023859,C=42)
	CT(19226)=(S=6023860,E=6023863,C=18)
	CT(19227)=(S=6023864,E=6023865,C=14)
	CT(19228)=(S=6023866,E=6023867,C=58)
	CT(19229)=(S=6023868,E=6023868,C=18)
	CT(19230)=(S=6023869,E=6023869,C=34)
	CT(19231)=(S=6023870,E=6023871,C=46)
	CT(19232)=(S=6023872,E=6023872,C=19)
	CT(19233)=(S=6023873,E=6023873,C=36)
	CT(19234)=(S=6023874,E=6023875,C=12)
	CT(19235)=(S=6023876,E=6023879,C=31)
	CT(19236)=(S=6023880,E=6023880,C=44)
	CT(19237)=(S=6023881,E=6023881,C=14)
	CT(19238)=(S=6023882,E=6023882,C=18)
	CT(19239)=(S=6023883,E=6023883,C=49)
	CT(19240)=(S=6023884,E=6023885,C=47)
	CT(19241)=(S=6023886,E=6023886,C=36)
	CT(19242)=(S=6023887,E=6023887,C=58)
	CT(19243)=(S=6023888,E=6023891,C=31)
	CT(19244)=(S=6023892,E=6023892,C=38)
	CT(19245)=(S=6023893,E=6023893,C=28)
	CT(19246)=(S=6023894,E=6023894,C=14)
	CT(19247)=(S=6023895,E=6023896,C=28)
	CT(19248)=(S=6023897,E=6023897,C=58)
	CT(19249)=(S=6023898,E=6023899,C=98)
	CT(19250)=(S=6023900,E=6023903,C=38)
	CT(19251)=(S=6023904,E=6023905,C=18)
	CT(19252)=(S=6023906,E=6023907,C=31)
	CT(19253)=(S=6023908,E=6023908,C=40)
	CT(19254)=(S=6023909,E=6023909,C=42)
	CT(19255)=(S=6023910,E=6023911,C=49)
	CT(19256)=(S=6023912,E=6023912,C=58)
	CT(19257)=(S=6023913,E=6023919,C=12)
	CT(19258)=(S=6023920,E=6023920,C=19)
	CT(19259)=(S=6023921,E=6023923,C=38)
	CT(19260)=(S=6023924,E=6023924,C=28)
	CT(19261)=(S=6023925,E=6023927,C=58)
	CT(19262)=(S=6023928,E=6023931,C=33)
	CT(19263)=(S=6023932,E=6023937,C=58)
	CT(19264)=(S=6023938,E=6023938,C=18)
	CT(19265)=(S=6023939,E=6023941,C=38)
	CT(19266)=(S=6023942,E=6023942,C=48)
	CT(19267)=(S=6023943,E=6023943,C=18)
	CT(19268)=(S=6023944,E=6023947,C=28)
	CT(19269)=(S=6023948,E=6023951,C=31)
	CT(19270)=(S=6023952,E=6023961,C=58)
	CT(19271)=(S=6023962,E=6023969,C=18)
	CT(19272)=(S=6023970,E=6023971,C=28)
	CT(19273)=(S=6023972,E=6023975,C=103)
	CT(19274)=(S=6023976,E=6023977,C=25)
	CT(19275)=(S=6023978,E=6023978,C=18)
	CT(19276)=(S=6023979,E=6023979,C=38)
	CT(19277)=(S=6023980,E=6023980,C=14)
	CT(19278)=(S=6023981,E=6023981,C=28)
	CT(19279)=(S=6023982,E=6023982,C=31)
	CT(19280)=(S=6023983,E=6023983,C=28)
	CT(19281)=(S=6023984,E=6023987,C=25)
	CT(19282)=(S=6023988,E=6023991,C=71)
	CT(19283)=(S=6023992,E=6023995,C=14)
	CT(19284)=(S=6023996,E=6023999,C=27)
	CT(19285)=(S=6024000,E=6024000,C=37)
	CT(19286)=(S=6024001,E=6024001,C=14)
	CT(19287)=(S=6024002,E=6024002,C=18)
	CT(19288)=(S=6024003,E=6024007,C=31)
	CT(19289)=(S=6024008,E=6024013,C=18)
	CT(19290)=(S=6024014,E=6024015,C=12)
	CT(19291)=(S=6024016,E=6024017,C=18)
	CT(19292)=(S=6024018,E=6024018,C=52)
	CT(19293)=(S=6024019,E=6024019,C=36)
	CT(19294)=(S=6024020,E=6024023,C=31)
	CT(19295)=(S=6024024,E=6024031,C=18)
	CT(19296)=(S=6024032,E=6024035,C=63)
	CT(19297)=(S=6024036,E=6024036,C=24)
	CT(19298)=(S=6024037,E=6024037,C=18)
	CT(19299)=(S=6024038,E=6024038,C=31)
	CT(19300)=(S=6024039,E=6024039,C=18)
	CT(19301)=(S=6024040,E=6024041,C=37)
	CT(19302)=(S=6024042,E=6024042,C=18)
	CT(19303)=(S=6024043,E=6024043,C=17)
	CT(19304)=(S=6024044,E=6024047,C=25)
	CT(19305)=(S=6024048,E=6024048,C=18)
	CT(19306)=(S=6024049,E=6024049,C=38)
	CT(19307)=(S=6024050,E=6024051,C=28)
	CT(19308)=(S=6024052,E=6024055,C=41)
	CT(19309)=(S=6024056,E=6024057,C=15)
	CT(19310)=(S=6024058,E=6024059,C=38)
	CT(19311)=(S=6024060,E=6024062,C=58)
	CT(19312)=(S=6024063,E=6024063,C=18)
	CT(19313)=(S=6024064,E=6024065,C=31)
	CT(19314)=(S=6024066,E=6024067,C=14)
	CT(19315)=(S=6024068,E=6024071,C=36)
	CT(19316)=(S=6024072,E=6024073,C=18)
	CT(19317)=(S=6024074,E=6024074,C=12)
	CT(19318)=(S=6024075,E=6024075,C=24)
	CT(19319)=(S=6024076,E=6024076,C=14)
	CT(19320)=(S=6024077,E=6024077,C=12)
	CT(19321)=(S=6024078,E=6024079,C=31)
	CT(19322)=(S=6024080,E=6024083,C=18)
	CT(19323)=(S=6024085,E=6024085,C=58)
	CT(19324)=(S=6024086,E=6024087,C=17)
	CT(19325)=(S=6024088,E=6024091,C=58)
	CT(19326)=(S=6024092,E=6024094,C=18)
	CT(19327)=(S=6024095,E=6024095,C=16)
	CT(19328)=(S=6024096,E=6024099,C=58)
	CT(19329)=(S=6024100,E=6024100,C=51)
	CT(19330)=(S=6024101,E=6024103,C=38)
	CT(19331)=(S=6024104,E=6024104,C=19)
	CT(19332)=(S=6024105,E=6024105,C=18)
	CT(19333)=(S=6024106,E=6024107,C=57)
	CT(19334)=(S=6024108,E=6024109,C=14)
	CT(19335)=(S=6024110,E=6024110,C=36)
	CT(19336)=(S=6024111,E=6024111,C=18)
	CT(19337)=(S=6024112,E=6024115,C=20)
	CT(19338)=(S=6024116,E=6024117,C=18)
	CT(19339)=(S=6024118,E=6024119,C=33)
	CT(19340)=(S=6024120,E=6024127,C=18)
	CT(19341)=(S=6024128,E=6024131,C=58)
	CT(19342)=(S=6024132,E=6024135,C=31)
	CT(19343)=(S=6024136,E=6024137,C=33)
	CT(19344)=(S=6024138,E=6024138,C=58)
	CT(19345)=(S=6024139,E=6024139,C=18)
	CT(19346)=(S=6024140,E=6024143,C=58)
	CT(19347)=(S=6024144,E=6024147,C=14)
	CT(19348)=(S=6024148,E=6024148,C=34)
	CT(19349)=(S=6024149,E=6024149,C=18)
	CT(19350)=(S=6024150,E=6024150,C=14)
	CT(19351)=(S=6024151,E=6024151,C=12)
	CT(19352)=(S=6024152,E=6024155,C=58)
	CT(19353)=(S=6024156,E=6024156,C=14)
	CT(19354)=(S=6024157,E=6024157,C=18)
	CT(19355)=(S=6024158,E=6024159,C=58)
	CT(19356)=(S=6024160,E=6024163,C=31)
	CT(19357)=(S=6024164,E=6024167,C=58)
	CT(19358)=(S=6024168,E=6024168,C=24)
	CT(19359)=(S=6024169,E=6024169,C=18)
	CT(19360)=(S=6024170,E=6024171,C=14)
	CT(19361)=(S=6024172,E=6024172,C=28)
	CT(19362)=(S=6024173,E=6024173,C=36)
	CT(19363)=(S=6024174,E=6024175,C=58)
	CT(19364)=(S=6024176,E=6024176,C=28)
	CT(19365)=(S=6024178,E=6024178,C=52)
	CT(19366)=(S=6024179,E=6024179,C=28)
	CT(19367)=(S=6024180,E=6024183,C=18)
	CT(19368)=(S=6024184,E=6024187,C=51)
	CT(19369)=(S=6024188,E=6024192,C=58)
	CT(19370)=(S=6024193,E=6024193,C=52)
	CT(19371)=(S=6024194,E=6024195,C=18)
	CT(19372)=(S=6024196,E=6024201,C=58)
	CT(19373)=(S=6024202,E=6024204,C=18)
	CT(19374)=(S=6024205,E=6024205,C=58)
	CT(19375)=(S=6024206,E=6024207,C=18)
	CT(19376)=(S=6024208,E=6024208,C=24)
	CT(19377)=(S=6024209,E=6024210,C=14)
	CT(19378)=(S=6024211,E=6024211,C=36)
	CT(19379)=(S=6024212,E=6024213,C=58)
	CT(19380)=(S=6024214,E=6024216,C=18)
	CT(19381)=(S=6024217,E=6024217,C=12)
	CT(19382)=(S=6024218,E=6024219,C=14)
	CT(19383)=(S=6024220,E=6024220,C=24)
	CT(19384)=(S=6024221,E=6024221,C=19)
	CT(19385)=(S=6024222,E=6024223,C=18)
	CT(19386)=(S=6024224,E=6024227,C=58)
	CT(19387)=(S=6024228,E=6024228,C=31)
	CT(19388)=(S=6024229,E=6024229,C=57)
	CT(19389)=(S=6024230,E=6024231,C=55)
	CT(19390)=(S=6024232,E=6024235,C=33)
	CT(19391)=(S=6024236,E=6024236,C=55)
	CT(19392)=(S=6024237,E=6024237,C=34)
	CT(19393)=(S=6024238,E=6024239,C=38)
	CT(19394)=(S=6024240,E=6024243,C=18)
	CT(19395)=(S=6024244,E=6024247,C=58)
	CT(19396)=(S=6024248,E=6024248,C=14)
	CT(19397)=(S=6024249,E=6024249,C=58)
	CT(19398)=(S=6024250,E=6024250,C=38)
	CT(19399)=(S=6024251,E=6024251,C=25)
	CT(19400)=(S=6024252,E=6024255,C=18)
	CT(19401)=(S=6024256,E=6024263,C=71)
	CT(19402)=(S=6024264,E=6024267,C=58)
	CT(19403)=(S=6024268,E=6024269,C=38)
	CT(19404)=(S=6024270,E=6024270,C=34)
	CT(19405)=(S=6024271,E=6024271,C=38)
	CT(19406)=(S=6024272,E=6024273,C=18)
	CT(19407)=(S=6024274,E=6024274,C=71)
	CT(19408)=(S=6024275,E=6024275,C=36)
	CT(19409)=(S=6024276,E=6024279,C=58)
	CT(19410)=(S=6024280,E=6024283,C=38)
	CT(19411)=(S=6024284,E=6024287,C=18)
	CT(19412)=(S=6024288,E=6024291,C=31)
	CT(19413)=(S=6024292,E=6024296,C=18)
	CT(19414)=(S=6024297,E=6024297,C=58)
	CT(19415)=(S=6024298,E=6024299,C=28)
	CT(19416)=(S=6024300,E=6024306,C=18)
	CT(19417)=(S=6024308,E=6024308,C=36)
	CT(19418)=(S=6024309,E=6024309,C=14)
	CT(19419)=(S=6024310,E=6024311,C=31)
	CT(19420)=(S=6024312,E=6024312,C=18)
	CT(19421)=(S=6024314,E=6024314,C=28)
	CT(19422)=(S=6024315,E=6024315,C=24)
	CT(19423)=(S=6024316,E=6024316,C=18)
	CT(19424)=(S=6024317,E=6024317,C=58)
	CT(19425)=(S=6024318,E=6024319,C=31)
	CT(19426)=(S=6024320,E=6024325,C=58)
	CT(19427)=(S=6024326,E=6024327,C=31)
	CT(19428)=(S=6024328,E=6024328,C=18)
	CT(19429)=(S=6024329,E=6024329,C=58)
	CT(19430)=(S=6024330,E=6024331,C=31)
	CT(19431)=(S=6024332,E=6024335,C=18)
	CT(19432)=(S=6024336,E=6024339,C=51)
	CT(19433)=(S=6024340,E=6024343,C=18)
	CT(19434)=(S=6024344,E=6024344,C=14)
	CT(19435)=(S=6024345,E=6024345,C=12)
	CT(19436)=(S=6024346,E=6024347,C=18)
	CT(19437)=(S=6024348,E=6024351,C=31)
	CT(19438)=(S=6024352,E=6024352,C=42)
	CT(19439)=(S=6024353,E=6024353,C=12)
	CT(19440)=(S=6024354,E=6024354,C=16)
	CT(19441)=(S=6024355,E=6024355,C=58)
	CT(19442)=(S=6024356,E=6024359,C=18)
	CT(19443)=(S=6024360,E=6024361,C=26)
	CT(19444)=(S=6024362,E=6024362,C=38)
	CT(19445)=(S=6024363,E=6024363,C=31)
	CT(19446)=(S=6024364,E=6024367,C=58)
	CT(19447)=(S=6024368,E=6024373,C=18)
	CT(19448)=(S=6024374,E=6024374,C=50)
	CT(19449)=(S=6024375,E=6024377,C=18)
	CT(19450)=(S=6024378,E=6024378,C=24)
	CT(19451)=(S=6024379,E=6024379,C=14)
	CT(19452)=(S=6024380,E=6024380,C=60)
	CT(19453)=(S=6024381,E=6024381,C=36)
	CT(19454)=(S=6024382,E=6024383,C=14)
	CT(19455)=(S=6024384,E=6024386,C=36)
	CT(19456)=(S=6024387,E=6024393,C=18)
	CT(19457)=(S=6024394,E=6024395,C=36)
	CT(19458)=(S=6024396,E=6024397,C=18)
	CT(19459)=(S=6024398,E=6024398,C=14)
	CT(19460)=(S=6024399,E=6024401,C=19)
	CT(19461)=(S=6024402,E=6024403,C=24)
	CT(19462)=(S=6024404,E=6024404,C=19)
	CT(19463)=(S=6024405,E=6024405,C=31)
	CT(19464)=(S=6024406,E=6024407,C=36)
	CT(19465)=(S=6024408,E=6024411,C=38)
	CT(19466)=(S=6024412,E=6024412,C=18)
	CT(19467)=(S=6024413,E=6024413,C=31)
	CT(19468)=(S=6024414,E=6024414,C=55)
	CT(19469)=(S=6024415,E=6024415,C=38)
	CT(19470)=(S=6024416,E=6024419,C=31)
	CT(19471)=(S=6024420,E=6024420,C=38)
	CT(19472)=(S=6024421,E=6024421,C=58)
	CT(19473)=(S=6024422,E=6024423,C=38)
	CT(19474)=(S=6024424,E=6024425,C=58)
	CT(19475)=(S=6024426,E=6024427,C=18)
	CT(19476)=(S=6024428,E=6024428,C=39)
	CT(19477)=(S=6024429,E=6024429,C=38)
	CT(19478)=(S=6024430,E=6024430,C=18)
	CT(19479)=(S=6024431,E=6024431,C=12)
	CT(19480)=(S=6024432,E=6024435,C=38)
	CT(19481)=(S=6024436,E=6024437,C=49)
	CT(19482)=(S=6024438,E=6024439,C=18)
	CT(19483)=(S=6024440,E=6024443,C=31)
	CT(19484)=(S=6024444,E=6024445,C=18)
	CT(19485)=(S=6024446,E=6024447,C=12)
	CT(19486)=(S=6024448,E=6024463,C=31)
	CT(19487)=(S=6024464,E=6024465,C=15)
	CT(19488)=(S=6024466,E=6024467,C=58)
	CT(19489)=(S=6024468,E=6024468,C=18)
	CT(19490)=(S=6024469,E=6024471,C=23)
	CT(19491)=(S=6024472,E=6024475,C=31)
	CT(19492)=(S=6024476,E=6024479,C=23)
	CT(19493)=(S=6024480,E=6024480,C=18)
	CT(19494)=(S=6024481,E=6024481,C=52)
	CT(19495)=(S=6024482,E=6024483,C=27)
	CT(19496)=(S=6024484,E=6024484,C=58)
	CT(19497)=(S=6024485,E=6024485,C=18)
	CT(19498)=(S=6024486,E=6024487,C=12)
	CT(19499)=(S=6024488,E=6024489,C=31)
	CT(19500)=(S=6024490,E=6024490,C=60)
	CT(19501)=(S=6024491,E=6024491,C=36)
	CT(19502)=(S=6024492,E=6024497,C=38)
	CT(19503)=(S=6024498,E=6024498,C=43)
	CT(19504)=(S=6024499,E=6024499,C=33)
	CT(19505)=(S=6024500,E=6024500,C=58)
	CT(19506)=(S=6024501,E=6024503,C=18)
	CT(19507)=(S=6024504,E=6024507,C=51)
	CT(19508)=(S=6024508,E=6024511,C=58)
	CT(19509)=(S=6024512,E=6024512,C=31)
	CT(19510)=(S=6024513,E=6024513,C=42)
	CT(19511)=(S=6024514,E=6024515,C=19)
	CT(19512)=(S=6024516,E=6024521,C=58)
	CT(19513)=(S=6024522,E=6024523,C=14)
	CT(19514)=(S=6024524,E=6024524,C=18)
	CT(19515)=(S=6024525,E=6024525,C=58)
	CT(19516)=(S=6024526,E=6024526,C=36)
	CT(19517)=(S=6024527,E=6024529,C=58)
	CT(19518)=(S=6024530,E=6024531,C=18)
	CT(19519)=(S=6024532,E=6024534,C=58)
	CT(19520)=(S=6024535,E=6024535,C=14)
	CT(19521)=(S=6024536,E=6024537,C=28)
	CT(19522)=(S=6024538,E=6024538,C=19)
	CT(19523)=(S=6024539,E=6024541,C=38)
	CT(19524)=(S=6024542,E=6024542,C=18)
	CT(19525)=(S=6024543,E=6024543,C=19)
	CT(19526)=(S=6024544,E=6024544,C=42)
	CT(19527)=(S=6024545,E=6024545,C=18)
	CT(19528)=(S=6024546,E=6024547,C=60)
	CT(19529)=(S=6024548,E=6024549,C=28)
	CT(19530)=(S=6024550,E=6024551,C=51)
	CT(19531)=(S=6024552,E=6024553,C=18)
	CT(19532)=(S=6024554,E=6024554,C=19)
	CT(19533)=(S=6024555,E=6024559,C=31)
	CT(19534)=(S=6024560,E=6024561,C=58)
	CT(19535)=(S=6024562,E=6024564,C=31)
	CT(19536)=(S=6024565,E=6024565,C=28)
	CT(19537)=(S=6024566,E=6024566,C=58)
	CT(19538)=(S=6024567,E=6024577,C=31)
	CT(19539)=(S=6024578,E=6024579,C=38)
	CT(19540)=(S=6024580,E=6024583,C=52)
	CT(19541)=(S=6024584,E=6024587,C=58)
	CT(19542)=(S=6024588,E=6024589,C=19)
	CT(19543)=(S=6024590,E=6024590,C=31)
	CT(19544)=(S=6024591,E=6024591,C=42)
	CT(19545)=(S=6024592,E=6024596,C=18)
	CT(19546)=(S=6024597,E=6024597,C=58)
	CT(19547)=(S=6024598,E=6024598,C=18)
	CT(19548)=(S=6024599,E=6024599,C=47)
	CT(19549)=(S=6024600,E=6024600,C=31)
	CT(19550)=(S=6024601,E=6024601,C=18)
	CT(19551)=(S=6024602,E=6024609,C=58)
	CT(19552)=(S=6024610,E=6024611,C=18)
	CT(19553)=(S=6024612,E=6024615,C=31)
	CT(19554)=(S=6024616,E=6024618,C=71)
	CT(19555)=(S=6024619,E=6024621,C=58)
	CT(19556)=(S=6024622,E=6024623,C=13)
	CT(19557)=(S=6024624,E=6024624,C=14)
	CT(19558)=(S=6024625,E=6024625,C=12)
	CT(19559)=(S=6024626,E=6024629,C=18)
	CT(19560)=(S=6024630,E=6024631,C=31)
	CT(19561)=(S=6024632,E=6024632,C=12)
	CT(19562)=(S=6024633,E=6024633,C=14)
	CT(19563)=(S=6024634,E=6024637,C=18)
	CT(19564)=(S=6024638,E=6024638,C=58)
	CT(19565)=(S=6024639,E=6024639,C=19)
	CT(19566)=(S=6024640,E=6024640,C=18)
	CT(19567)=(S=6024641,E=6024643,C=38)
	CT(19568)=(S=6024644,E=6024644,C=58)
	CT(19569)=(S=6024645,E=6024645,C=14)
	CT(19570)=(S=6024646,E=6024651,C=18)
	CT(19571)=(S=6024652,E=6024656,C=31)
	CT(19572)=(S=6024657,E=6024657,C=18)
	CT(19573)=(S=6024658,E=6024659,C=14)
	CT(19574)=(S=6024660,E=6024660,C=12)
	CT(19575)=(S=6024661,E=6024661,C=18)
	CT(19576)=(S=6024662,E=6024662,C=28)
	CT(19577)=(S=6024663,E=6024663,C=18)
	CT(19578)=(S=6024664,E=6024665,C=37)
	CT(19579)=(S=6024666,E=6024666,C=34)
	CT(19580)=(S=6024667,E=6024667,C=58)
	CT(19581)=(S=6024668,E=6024671,C=18)
	CT(19582)=(S=6024672,E=6024677,C=58)
	CT(19583)=(S=6024678,E=6024678,C=18)
	CT(19584)=(S=6024679,E=6024679,C=14)
	CT(19585)=(S=6024680,E=6024683,C=31)
	CT(19586)=(S=6024684,E=6024687,C=33)
	CT(19587)=(S=6024688,E=6024695,C=31)
	CT(19588)=(S=6024696,E=6024696,C=38)
	CT(19589)=(S=6024697,E=6024697,C=18)
	CT(19590)=(S=6024698,E=6024698,C=13)
	CT(19591)=(S=6024699,E=6024699,C=17)
	CT(19592)=(S=6024700,E=6024701,C=18)
	CT(19593)=(S=6024702,E=6024704,C=26)
	CT(19594)=(S=6024705,E=6024707,C=18)
	CT(19595)=(S=6024708,E=6024709,C=31)
	CT(19596)=(S=6024710,E=6024711,C=46)
	CT(19597)=(S=6024712,E=6024712,C=34)
	CT(19598)=(S=6024713,E=6024713,C=31)
	CT(19599)=(S=6024714,E=6024715,C=18)
	CT(19600)=(S=6024716,E=6024719,C=53)
	CT(19601)=(S=6024720,E=6024721,C=31)
	CT(19602)=(S=6024723,E=6024723,C=42)
	CT(19603)=(S=6024724,E=6024725,C=18)
	CT(19604)=(S=6024726,E=6024726,C=14)
	CT(19605)=(S=6024727,E=6024727,C=31)
	CT(19606)=(S=6024728,E=6024733,C=18)
	CT(19607)=(S=6024734,E=6024736,C=49)
	CT(19608)=(S=6024737,E=6024739,C=18)
	CT(19609)=(S=6024740,E=6024740,C=58)
	CT(19610)=(S=6024741,E=6024747,C=38)
	CT(19611)=(S=6024748,E=6024753,C=18)
	CT(19612)=(S=6024754,E=6024754,C=38)
	CT(19613)=(S=6024755,E=6024755,C=19)
	CT(19614)=(S=6024756,E=6024764,C=58)
	CT(19615)=(S=6024765,E=6024765,C=46)
	CT(19616)=(S=6024766,E=6024767,C=58)
	CT(19617)=(S=6024768,E=6024773,C=18)
	CT(19618)=(S=6024774,E=6024775,C=38)
	CT(19619)=(S=6024776,E=6024777,C=12)
	CT(19620)=(S=6024778,E=6024779,C=31)
	CT(19621)=(S=6024780,E=6024782,C=58)
	CT(19622)=(S=6024783,E=6024785,C=18)
	CT(19623)=(S=6024786,E=6024786,C=31)
	CT(19624)=(S=6024789,E=6024789,C=58)
	CT(19625)=(S=6024790,E=6024795,C=18)
	CT(19626)=(S=6024798,E=6024799,C=53)
	CT(19627)=(S=6024800,E=6024803,C=18)
	CT(19628)=(S=6024804,E=6024807,C=31)
	CT(19629)=(S=6024808,E=6024809,C=33)
	CT(19630)=(S=6024810,E=6024810,C=51)
	CT(19631)=(S=6024811,E=6024811,C=58)
	CT(19632)=(S=6024812,E=6024816,C=18)
	CT(19633)=(S=6024817,E=6024817,C=23)
	CT(19634)=(S=6024818,E=6024819,C=18)
	CT(19635)=(S=6024820,E=6024823,C=15)
	CT(19636)=(S=6024824,E=6024826,C=18)
	CT(19637)=(S=6024827,E=6024827,C=14)
	CT(19638)=(S=6024828,E=6024829,C=58)
	CT(19639)=(S=6024830,E=6024830,C=25)
	CT(19640)=(S=6024831,E=6024833,C=18)
	CT(19641)=(S=6024834,E=6024834,C=14)
	CT(19642)=(S=6024835,E=6024835,C=12)
	CT(19643)=(S=6024836,E=6024836,C=31)
	CT(19644)=(S=6024837,E=6024837,C=33)
	CT(19645)=(S=6024838,E=6024839,C=58)
	CT(19646)=(S=6024840,E=6024843,C=33)
	CT(19647)=(S=6024844,E=6024844,C=18)
	CT(19648)=(S=6024845,E=6024845,C=36)
	CT(19649)=(S=6024846,E=6024847,C=58)
	CT(19650)=(S=6024848,E=6024848,C=34)
	CT(19651)=(S=6024849,E=6024849,C=58)
	CT(19652)=(S=6024850,E=6024851,C=31)
	CT(19653)=(S=6024852,E=6024853,C=38)
	CT(19654)=(S=6024854,E=6024855,C=12)
	CT(19655)=(S=6024856,E=6024857,C=24)
	CT(19656)=(S=6024858,E=6024859,C=36)
	CT(19657)=(S=6024860,E=6024863,C=18)
	CT(19658)=(S=6024864,E=6024869,C=14)
	CT(19659)=(S=6024870,E=6024871,C=58)
	CT(19660)=(S=6024872,E=6024875,C=18)
	CT(19661)=(S=6024876,E=6024879,C=58)
	CT(19662)=(S=6024880,E=6024881,C=36)
	CT(19663)=(S=6024882,E=6024883,C=58)
	CT(19664)=(S=6024884,E=6024885,C=28)
	CT(19665)=(S=6024886,E=6024889,C=18)
	CT(19666)=(S=6024890,E=6024891,C=38)
	CT(19667)=(S=6024892,E=6024895,C=18)
	CT(19668)=(S=6024896,E=6024899,C=31)
	CT(19669)=(S=6024900,E=6024902,C=14)
	CT(19670)=(S=6024903,E=6024903,C=51)
	CT(19671)=(S=6024904,E=6024907,C=18)
	CT(19672)=(S=6024908,E=6024909,C=51)
	CT(19673)=(S=6024910,E=6024910,C=24)
	CT(19674)=(S=6024911,E=6024911,C=18)
	CT(19675)=(S=6024912,E=6024913,C=14)
	CT(19676)=(S=6024914,E=6024915,C=18)
	CT(19677)=(S=6024916,E=6024916,C=24)
	CT(19678)=(S=6024917,E=6024917,C=18)
	CT(19679)=(S=6024918,E=6024919,C=14)
	CT(19680)=(S=6024920,E=6024920,C=58)
	CT(19681)=(S=6024921,E=6024921,C=18)
	CT(19682)=(S=6024922,E=6024922,C=38)
	CT(19683)=(S=6024923,E=6024923,C=49)
	CT(19684)=(S=6024924,E=6024924,C=12)
	CT(19685)=(S=6024925,E=6024925,C=14)
	CT(19686)=(S=6024926,E=6024928,C=18)
	CT(19687)=(S=6024929,E=6024929,C=42)
	CT(19688)=(S=6024930,E=6024930,C=49)
	CT(19689)=(S=6024931,E=6024935,C=18)
	CT(19690)=(S=6024936,E=6024939,C=58)
	CT(19691)=(S=6024940,E=6024940,C=28)
	CT(19692)=(S=6024941,E=6024941,C=18)
	CT(19693)=(S=6024942,E=6024943,C=34)
	CT(19694)=(S=6024944,E=6024947,C=58)
	CT(19695)=(S=6024948,E=6024949,C=31)
	CT(19696)=(S=6024950,E=6024951,C=58)
	CT(19697)=(S=6024952,E=6024953,C=18)
	CT(19698)=(S=6024954,E=6024954,C=15)
	CT(19699)=(S=6024955,E=6024955,C=51)
	CT(19700)=(S=6024956,E=6024957,C=18)
	CT(19701)=(S=6024958,E=6024958,C=58)
	CT(19702)=(S=6024959,E=6024959,C=51)
	CT(19703)=(S=6024960,E=6024963,C=38)
	CT(19704)=(S=6024964,E=6024965,C=18)
	CT(19705)=(S=6024966,E=6024967,C=12)
	CT(19706)=(S=6024968,E=6024971,C=53)
	CT(19707)=(S=6024972,E=6024972,C=38)
	CT(19708)=(S=6024973,E=6024973,C=51)
	CT(19709)=(S=6024974,E=6024974,C=26)
	CT(19710)=(S=6024976,E=6024979,C=18)
	CT(19711)=(S=6024980,E=6024980,C=28)
	CT(19712)=(S=6024981,E=6024981,C=25)
	CT(19713)=(S=6024982,E=6024982,C=18)
	CT(19714)=(S=6024983,E=6024983,C=42)
	CT(19715)=(S=6024984,E=6024984,C=18)
	CT(19716)=(S=6024985,E=6024985,C=27)
	CT(19717)=(S=6024986,E=6024987,C=18)
	CT(19718)=(S=6024988,E=6024991,C=58)
	CT(19719)=(S=6024992,E=6024993,C=36)
	CT(19720)=(S=6024994,E=6024995,C=28)
	CT(19721)=(S=6024996,E=6024996,C=58)
	CT(19722)=(S=6024998,E=6024999,C=24)
	CT(19723)=(S=6025000,E=6025005,C=18)
	CT(19724)=(S=6025006,E=6025006,C=14)
	CT(19725)=(S=6025007,E=6025007,C=51)
	CT(19726)=(S=6025008,E=6025011,C=18)
	CT(19727)=(S=6025012,E=6025012,C=34)
	CT(19728)=(S=6025013,E=6025013,C=31)
	CT(19729)=(S=6025014,E=6025015,C=26)
	CT(19730)=(S=6025016,E=6025020,C=18)
	CT(19731)=(S=6025021,E=6025021,C=58)
	CT(19732)=(S=6025022,E=6025022,C=14)
	CT(19733)=(S=6025023,E=6025023,C=18)
	CT(19734)=(S=6025024,E=6025025,C=25)
	CT(19735)=(S=6025026,E=6025027,C=58)
	CT(19736)=(S=6025028,E=6025031,C=18)
	CT(19737)=(S=6025032,E=6025033,C=14)
	CT(19738)=(S=6025034,E=6025034,C=31)
	CT(19739)=(S=6025035,E=6025035,C=34)
	CT(19740)=(S=6025036,E=6025039,C=31)
	CT(19741)=(S=6025040,E=6025043,C=18)
	CT(19742)=(S=6025044,E=6025044,C=28)
	CT(19743)=(S=6025045,E=6025045,C=18)
	CT(19744)=(S=6025046,E=6025046,C=55)
	CT(19745)=(S=6025047,E=6025047,C=14)
	CT(19746)=(S=6025048,E=6025049,C=18)
	CT(19747)=(S=6025050,E=6025050,C=58)
	CT(19748)=(S=6025051,E=6025051,C=14)
	CT(19749)=(S=6025052,E=6025053,C=28)
	CT(19750)=(S=6025054,E=6025054,C=18)
	CT(19751)=(S=6025055,E=6025055,C=31)
	CT(19752)=(S=6025056,E=6025057,C=52)
	CT(19753)=(S=6025058,E=6025059,C=18)
	CT(19754)=(S=6025060,E=6025060,C=24)
	CT(19755)=(S=6025061,E=6025061,C=20)
	CT(19756)=(S=6025062,E=6025063,C=18)
	CT(19757)=(S=6025064,E=6025067,C=31)
	CT(19758)=(S=6025068,E=6025071,C=26)
	CT(19759)=(S=6025072,E=6025072,C=18)
	CT(19760)=(S=6025073,E=6025080,C=58)
	CT(19761)=(S=6025081,E=6025081,C=18)
	CT(19762)=(S=6025082,E=6025082,C=24)
	CT(19763)=(S=6025083,E=6025083,C=38)
	CT(19764)=(S=6025084,E=6025085,C=14)
	CT(19765)=(S=6025086,E=6025087,C=18)
	CT(19766)=(S=6025088,E=6025088,C=38)
	CT(19767)=(S=6025089,E=6025089,C=58)
	CT(19768)=(S=6025091,E=6025095,C=18)
	CT(19769)=(S=6025096,E=6025099,C=31)
	CT(19770)=(S=6025100,E=6025103,C=18)
	CT(19771)=(S=6025104,E=6025104,C=58)
	CT(19772)=(S=6025105,E=6025105,C=14)
	CT(19773)=(S=6025106,E=6025106,C=21)
	CT(19774)=(S=6025107,E=6025107,C=63)
	CT(19775)=(S=6025108,E=6025109,C=51)
	CT(19776)=(S=6025110,E=6025110,C=31)
	CT(19777)=(S=6025111,E=6025111,C=46)
	CT(19778)=(S=6025112,E=6025115,C=58)
	CT(19779)=(S=6025116,E=6025116,C=14)
	CT(19780)=(S=6025118,E=6025119,C=18)
	CT(19781)=(S=6025120,E=6025123,C=31)
	CT(19782)=(S=6025124,E=6025127,C=38)
	CT(19783)=(S=6025128,E=6025131,C=58)
	CT(19784)=(S=6025132,E=6025132,C=18)
	CT(19785)=(S=6025133,E=6025133,C=38)
	CT(19786)=(S=6025134,E=6025135,C=36)
	CT(19787)=(S=6025136,E=6025136,C=18)
	CT(19788)=(S=6025137,E=6025137,C=28)
	CT(19789)=(S=6025138,E=6025139,C=38)
	CT(19790)=(S=6025140,E=6025143,C=58)
	CT(19791)=(S=6025144,E=6025144,C=18)
	CT(19792)=(S=6025145,E=6025145,C=58)
	CT(19793)=(S=6025146,E=6025146,C=36)
	CT(19794)=(S=6025147,E=6025147,C=31)
	CT(19795)=(S=6025148,E=6025149,C=19)
	CT(19796)=(S=6025150,E=6025150,C=31)
	CT(19797)=(S=6025151,E=6025151,C=18)
	CT(19798)=(S=6025152,E=6025152,C=20)
	CT(19799)=(S=6025153,E=6025153,C=52)
	CT(19800)=(S=6025154,E=6025154,C=38)
	CT(19801)=(S=6025155,E=6025155,C=32)
	CT(19802)=(S=6025156,E=6025159,C=58)
	CT(19803)=(S=6025160,E=6025163,C=33)
	CT(19804)=(S=6025164,E=6025164,C=37)
	CT(19805)=(S=6025165,E=6025165,C=58)
	CT(19806)=(S=6025166,E=6025167,C=40)
	CT(19807)=(S=6025168,E=6025171,C=14)
	CT(19808)=(S=6025172,E=6025173,C=36)
	CT(19809)=(S=6025174,E=6025174,C=26)
	CT(19810)=(S=6025175,E=6025175,C=18)
	CT(19811)=(S=6025176,E=6025177,C=19)
	CT(19812)=(S=6025178,E=6025179,C=17)
	CT(19813)=(S=6025180,E=6025186,C=38)
	CT(19814)=(S=6025187,E=6025189,C=18)
	CT(19815)=(S=6025190,E=6025190,C=38)
	CT(19816)=(S=6025191,E=6025191,C=42)
	CT(19817)=(S=6025192,E=6025195,C=31)
	CT(19818)=(S=6025196,E=6025197,C=33)
	CT(19819)=(S=6025198,E=6025199,C=18)
	CT(19820)=(S=6025200,E=6025200,C=51)
	CT(19821)=(S=6025201,E=6025201,C=31)
	CT(19822)=(S=6025202,E=6025202,C=37)
	CT(19823)=(S=6025203,E=6025203,C=52)
	CT(19824)=(S=6025204,E=6025211,C=58)
	CT(19825)=(S=6025212,E=6025213,C=27)
	CT(19826)=(S=6025214,E=6025214,C=58)
	CT(19827)=(S=6025215,E=6025215,C=19)
	CT(19828)=(S=6025216,E=6025216,C=34)
	CT(19829)=(S=6025217,E=6025217,C=38)
	CT(19830)=(S=6025218,E=6025219,C=14)
	CT(19831)=(S=6025220,E=6025221,C=38)
	CT(19832)=(S=6025222,E=6025223,C=46)
	CT(19833)=(S=6025224,E=6025227,C=38)
	CT(19834)=(S=6025228,E=6025231,C=98)
	CT(19835)=(S=6025232,E=6025232,C=18)
	CT(19836)=(S=6025233,E=6025233,C=14)
	CT(19837)=(S=6025234,E=6025235,C=31)
	CT(19838)=(S=6025236,E=6025239,C=33)
	CT(19839)=(S=6025240,E=6025240,C=14)
	CT(19840)=(S=6025241,E=6025241,C=18)
	CT(19841)=(S=6025242,E=6025242,C=37)
	CT(19842)=(S=6025243,E=6025243,C=40)
	CT(19843)=(S=6025244,E=6025247,C=58)
	CT(19844)=(S=6025248,E=6025248,C=12)
	CT(19845)=(S=6025249,E=6025249,C=25)
	CT(19846)=(S=6025250,E=6025250,C=28)
	CT(19847)=(S=6025251,E=6025251,C=33)
	CT(19848)=(S=6025252,E=6025252,C=18)
	CT(19849)=(S=6025253,E=6025253,C=37)
	CT(19850)=(S=6025254,E=6025255,C=18)
	CT(19851)=(S=6025256,E=6025259,C=46)
	CT(19852)=(S=6025260,E=6025260,C=58)
	CT(19853)=(S=6025261,E=6025261,C=18)
	CT(19854)=(S=6025262,E=6025263,C=58)
	CT(19855)=(S=6025264,E=6025267,C=31)
	CT(19856)=(S=6025268,E=6025271,C=52)
	CT(19857)=(S=6025272,E=6025275,C=25)
	CT(19858)=(S=6025276,E=6025279,C=26)
	CT(19859)=(S=6025280,E=6025283,C=19)
	CT(19860)=(S=6025284,E=6025284,C=58)
	CT(19861)=(S=6025285,E=6025285,C=39)
	CT(19862)=(S=6025286,E=6025286,C=58)
	CT(19863)=(S=6025287,E=6025287,C=14)
	CT(19864)=(S=6025288,E=6025288,C=12)
	CT(19865)=(S=6025289,E=6025289,C=16)
	CT(19866)=(S=6025290,E=6025291,C=18)
	CT(19867)=(S=6025292,E=6025292,C=58)
	CT(19868)=(S=6025293,E=6025293,C=28)
	CT(19869)=(S=6025294,E=6025295,C=12)
	CT(19870)=(S=6025296,E=6025299,C=39)
	CT(19871)=(S=6025300,E=6025303,C=18)
	CT(19872)=(S=6025304,E=6025304,C=24)
	CT(19873)=(S=6025306,E=6025308,C=18)
	CT(19874)=(S=6025309,E=6025309,C=58)
	CT(19875)=(S=6025310,E=6025310,C=38)
	CT(19876)=(S=6025311,E=6025315,C=18)
	CT(19877)=(S=6025316,E=6025317,C=58)
	CT(19878)=(S=6025318,E=6025319,C=18)
	CT(19879)=(S=6025320,E=6025321,C=34)
	CT(19880)=(S=6025322,E=6025323,C=58)
	CT(19881)=(S=6025324,E=6025324,C=37)
	CT(19882)=(S=6025325,E=6025325,C=12)
	CT(19883)=(S=6025326,E=6025331,C=18)
	CT(19884)=(S=6025332,E=6025335,C=36)
	CT(19885)=(S=6025336,E=6025344,C=18)
	CT(19886)=(S=6025345,E=6025345,C=28)
	CT(19887)=(S=6025346,E=6025355,C=58)
	CT(19888)=(S=6025356,E=6025360,C=18)
	CT(19889)=(S=6025361,E=6025361,C=33)
	CT(19890)=(S=6025362,E=6025363,C=17)
	CT(19891)=(S=6025364,E=6025364,C=43)
	CT(19892)=(S=6025365,E=6025367,C=38)
	CT(19893)=(S=6025368,E=6025369,C=58)
	CT(19894)=(S=6025370,E=6025370,C=38)
	CT(19895)=(S=6025371,E=6025371,C=18)
	CT(19896)=(S=6025372,E=6025375,C=38)
	CT(19897)=(S=6025376,E=6025377,C=28)
	CT(19898)=(S=6025378,E=6025379,C=18)
	CT(19899)=(S=6025380,E=6025380,C=28)
	CT(19900)=(S=6025381,E=6025381,C=33)
	CT(19901)=(S=6025382,E=6025383,C=15)
	CT(19902)=(S=6025384,E=6025388,C=18)
	CT(19903)=(S=6025389,E=6025389,C=25)
	CT(19904)=(S=6025390,E=6025392,C=14)
	CT(19905)=(S=6025393,E=6025393,C=38)
	CT(19906)=(S=6025394,E=6025394,C=14)
	CT(19907)=(S=6025395,E=6025395,C=18)
	CT(19908)=(S=6025396,E=6025399,C=26)
	CT(19909)=(S=6025400,E=6025400,C=38)
	CT(19910)=(S=6025401,E=6025403,C=24)
	CT(19911)=(S=6025404,E=6025404,C=38)
	CT(19912)=(S=6025405,E=6025405,C=24)
	CT(19913)=(S=6025406,E=6025407,C=31)
	CT(19914)=(S=6025408,E=6025408,C=14)
	CT(19915)=(S=6025409,E=6025409,C=58)
	CT(19916)=(S=6025410,E=6025410,C=19)
	CT(19917)=(S=6025411,E=6025411,C=27)
	CT(19918)=(S=6025412,E=6025415,C=58)
	CT(19919)=(S=6025416,E=6025417,C=36)
	CT(19920)=(S=6025418,E=6025418,C=18)
	CT(19921)=(S=6025419,E=6025419,C=32)
	CT(19922)=(S=6025420,E=6025423,C=27)
	CT(19923)=(S=6025424,E=6025427,C=18)
	CT(19924)=(S=6025428,E=6025429,C=32)
	CT(19925)=(S=6025430,E=6025431,C=18)
	CT(19926)=(S=6025432,E=6025432,C=52)
	CT(19927)=(S=6025433,E=6025433,C=28)
	CT(19928)=(S=6025434,E=6025436,C=18)
	CT(19929)=(S=6025437,E=6025437,C=14)
	CT(19930)=(S=6025438,E=6025438,C=18)
	CT(19931)=(S=6025439,E=6025439,C=58)
	CT(19932)=(S=6025440,E=6025440,C=14)
	CT(19933)=(S=6025441,E=6025441,C=18)
	CT(19934)=(S=6025442,E=6025443,C=14)
	CT(19935)=(S=6025444,E=6025444,C=12)
	CT(19936)=(S=6025445,E=6025445,C=14)
	CT(19937)=(S=6025446,E=6025447,C=15)
	CT(19938)=(S=6025448,E=6025449,C=18)
	CT(19939)=(S=6025450,E=6025451,C=17)
	CT(19940)=(S=6025452,E=6025453,C=38)
	CT(19941)=(S=6025454,E=6025455,C=58)
	CT(19942)=(S=6025456,E=6025456,C=36)
	CT(19943)=(S=6025457,E=6025457,C=12)
	CT(19944)=(S=6025458,E=6025459,C=18)
	CT(19945)=(S=6025460,E=6025460,C=31)
	CT(19946)=(S=6025461,E=6025461,C=71)
	CT(19947)=(S=6025462,E=6025463,C=60)
	CT(19948)=(S=6025464,E=6025467,C=58)
	CT(19949)=(S=6025468,E=6025469,C=19)
	CT(19950)=(S=6025470,E=6025470,C=58)
	CT(19951)=(S=6025471,E=6025471,C=51)
	CT(19952)=(S=6025472,E=6025473,C=14)
	CT(19953)=(S=6025474,E=6025474,C=58)
	CT(19954)=(S=6025475,E=6025475,C=21)
	CT(19955)=(S=6025476,E=6025476,C=31)
	CT(19956)=(S=6025477,E=6025477,C=58)
	CT(19957)=(S=6025478,E=6025479,C=36)
	CT(19958)=(S=6025480,E=6025483,C=33)
	CT(19959)=(S=6025484,E=6025491,C=18)
	CT(19960)=(S=6025492,E=6025493,C=26)
	CT(19961)=(S=6025494,E=6025494,C=18)
	CT(19962)=(S=6025495,E=6025495,C=33)
	CT(19963)=(S=6025496,E=6025499,C=58)
	CT(19964)=(S=6025500,E=6025500,C=70)
	CT(19965)=(S=6025501,E=6025501,C=36)
	CT(19966)=(S=6025502,E=6025502,C=49)
	CT(19967)=(S=6025503,E=6025513,C=58)
	CT(19968)=(S=6025514,E=6025514,C=12)
	CT(19969)=(S=6025515,E=6025515,C=18)
	CT(19970)=(S=6025516,E=6025516,C=28)
	CT(19971)=(S=6025517,E=6025519,C=18)
	CT(19972)=(S=6025520,E=6025523,C=58)
	CT(19973)=(S=6025524,E=6025524,C=28)
	CT(19974)=(S=6025525,E=6025525,C=57)
	CT(19975)=(S=6025526,E=6025527,C=28)
	CT(19976)=(S=6025528,E=6025528,C=14)
	CT(19977)=(S=6025529,E=6025529,C=12)
	CT(19978)=(S=6025530,E=6025531,C=18)
	CT(19979)=(S=6025532,E=6025535,C=58)
	CT(19980)=(S=6025536,E=6025539,C=60)
	CT(19981)=(S=6025540,E=6025541,C=18)
	CT(19982)=(S=6025542,E=6025542,C=14)
	CT(19983)=(S=6025543,E=6025543,C=31)
	CT(19984)=(S=6025544,E=6025547,C=28)
	CT(19985)=(S=6025548,E=6025549,C=15)
	CT(19986)=(S=6025550,E=6025550,C=14)
	CT(19987)=(S=6025551,E=6025551,C=49)
	CT(19988)=(S=6025552,E=6025555,C=31)
	CT(19989)=(S=6025556,E=6025557,C=18)
	CT(19990)=(S=6025558,E=6025559,C=14)
	CT(19991)=(S=6025560,E=6025563,C=18)
	CT(19992)=(S=6025564,E=6025564,C=26)
	CT(19993)=(S=6025566,E=6025567,C=22)
	CT(19994)=(S=6025568,E=6025727,C=31)
	CT(19995)=(S=6025728,E=6025759,C=41)
	CT(19996)=(S=6025760,E=6025771,C=18)
	CT(19997)=(S=6025772,E=6025773,C=26)
	CT(19998)=(S=6025774,E=6025779,C=38)
	CT(19999)=(S=6025780,E=6025783,C=31)
	CT(20000)=(S=6025784,E=6025791,C=58)
	CT(20001)=(S=6025792,E=6025887,C=53)
	CT(20002)=(S=6025888,E=6025888,C=36)
	CT(20003)=(S=6025889,E=6025889,C=18)
	CT(20004)=(S=6025890,E=6025890,C=12)
	CT(20005)=(S=6025891,E=6025899,C=18)
	CT(20006)=(S=6025900,E=6025900,C=14)
	CT(20007)=(S=6025901,E=6025901,C=28)
	CT(20008)=(S=6025902,E=6025903,C=58)
	CT(20009)=(S=6025904,E=6025919,C=18)
	CT(20010)=(S=6025920,E=6025927,C=31)
	CT(20011)=(S=6025928,E=6025935,C=27)
	CT(20012)=(S=6025936,E=6025939,C=31)
	CT(20013)=(S=6025940,E=6025940,C=28)
	CT(20014)=(S=6025941,E=6025941,C=18)
	CT(20015)=(S=6025942,E=6025943,C=4)
	CT(20016)=(S=6025944,E=6025944,C=38)
	CT(20017)=(S=6025945,E=6025945,C=16)
	CT(20018)=(S=6025946,E=6025949,C=58)
	CT(20019)=(S=6025950,E=6025950,C=31)
	CT(20020)=(S=6025951,E=6025951,C=36)
	CT(20021)=(S=6025952,E=6026015,C=31)
	CT(20022)=(S=6026016,E=6026047,C=18)
	CT(20023)=(S=6026048,E=6026049,C=36)
	CT(20024)=(S=6026050,E=6026051,C=58)
	CT(20025)=(S=6026052,E=6026052,C=18)
	CT(20026)=(S=6026053,E=6026053,C=27)
	CT(20027)=(S=6026054,E=6026054,C=28)
	CT(20028)=(S=6026055,E=6026055,C=34)
	CT(20029)=(S=6026056,E=6026059,C=14)
	CT(20030)=(S=6026060,E=6026061,C=12)
	CT(20031)=(S=6026062,E=6026063,C=42)
	CT(20032)=(S=6026064,E=6026097,C=18)
	CT(20033)=(S=6026098,E=6026098,C=14)
	CT(20034)=(S=6026099,E=6026105,C=18)
	CT(20035)=(S=6026106,E=6026106,C=71)
	CT(20036)=(S=6026107,E=6026107,C=46)
	CT(20037)=(S=6026108,E=6026109,C=18)
	CT(20038)=(S=6026110,E=6026111,C=26)
	CT(20039)=(S=6026112,E=6026143,C=18)
	CT(20040)=(S=6026144,E=6026159,C=26)
	CT(20041)=(S=6026160,E=6026175,C=18)
	CT(20042)=(S=6026176,E=6026207,C=31)
	CT(20043)=(S=6026208,E=6026239,C=18)
	CT(20044)=(S=6026240,E=6026303,C=31)
	CT(20045)=(S=6026304,E=6026307,C=18)
	CT(20046)=(S=6026308,E=6026308,C=14)
	CT(20047)=(S=6026309,E=6026309,C=31)
	CT(20048)=(S=6026310,E=6026311,C=58)
	CT(20049)=(S=6026312,E=6026335,C=18)
	CT(20050)=(S=6026336,E=6026351,C=38)
	CT(20051)=(S=6026352,E=6026355,C=18)
	CT(20052)=(S=6026356,E=6026356,C=37)
	CT(20053)=(S=6026357,E=6026357,C=14)
	CT(20054)=(S=6026358,E=6026359,C=31)
	CT(20055)=(S=6026360,E=6026363,C=71)
	CT(20056)=(S=6026364,E=6026364,C=31)
	CT(20057)=(S=6026365,E=6026365,C=28)
	CT(20058)=(S=6026366,E=6026366,C=14)
	CT(20059)=(S=6026367,E=6026399,C=31)
	CT(20060)=(S=6026400,E=6026419,C=18)
	CT(20061)=(S=6026420,E=6026420,C=28)
	CT(20062)=(S=6026421,E=6026421,C=51)
	CT(20063)=(S=6026422,E=6026423,C=18)
	CT(20064)=(S=6026424,E=6026431,C=58)
	CT(20065)=(S=6026432,E=6026447,C=18)
	CT(20066)=(S=6026448,E=6026463,C=58)
	CT(20067)=(S=6026464,E=6026464,C=36)
	CT(20068)=(S=6026465,E=6026465,C=58)
	CT(20069)=(S=6026466,E=6026467,C=37)
	CT(20070)=(S=6026468,E=6026468,C=18)
	CT(20071)=(S=6026469,E=6026469,C=14)
	CT(20072)=(S=6026470,E=6026471,C=58)
	CT(20073)=(S=6026472,E=6026475,C=31)
	CT(20074)=(S=6026476,E=6026477,C=14)
	CT(20075)=(S=6026478,E=6026478,C=12)
	CT(20076)=(S=6026479,E=6026479,C=14)
	CT(20077)=(S=6026480,E=6026484,C=18)
	CT(20078)=(S=6026485,E=6026487,C=14)
	CT(20079)=(S=6026488,E=6026491,C=58)
	CT(20080)=(S=6026492,E=6026495,C=18)
	CT(20081)=(S=6026496,E=6026527,C=33)
	CT(20082)=(S=6026528,E=6026567,C=18)
	CT(20083)=(S=6026568,E=6026575,C=31)
	CT(20084)=(S=6026576,E=6026583,C=58)
	CT(20085)=(S=6026584,E=6026591,C=38)
	CT(20086)=(S=6026592,E=6026623,C=33)
	CT(20087)=(S=6026624,E=6026655,C=18)
	CT(20088)=(S=6026656,E=6026687,C=38)
	CT(20089)=(S=6026688,E=6026688,C=51)
	CT(20090)=(S=6026689,E=6026689,C=52)
	CT(20091)=(S=6026690,E=6026690,C=60)
	CT(20092)=(S=6026691,E=6026691,C=22)
	CT(20093)=(S=6026692,E=6026705,C=38)
	CT(20094)=(S=6026706,E=6026707,C=18)
	CT(20095)=(S=6026708,E=6026708,C=14)
	CT(20096)=(S=6026709,E=6026709,C=31)
	CT(20097)=(S=6026710,E=6026710,C=46)
	CT(20098)=(S=6026711,E=6026711,C=28)
	CT(20099)=(S=6026712,E=6026713,C=31)
	CT(20100)=(S=6026714,E=6026714,C=24)
	CT(20101)=(S=6026715,E=6026715,C=58)
	CT(20102)=(S=6026716,E=6026716,C=28)
	CT(20103)=(S=6026717,E=6026717,C=31)
	CT(20104)=(S=6026718,E=6026718,C=32)
	CT(20105)=(S=6026719,E=6026719,C=31)
	CT(20106)=(S=6026752,E=6026755,C=18)
	CT(20107)=(S=6026756,E=6026759,C=31)
	CT(20108)=(S=6026760,E=6026763,C=14)
	CT(20109)=(S=6026764,E=6026764,C=51)
	CT(20110)=(S=6026766,E=6026766,C=101)
	CT(20111)=(S=6026767,E=6026767,C=58)
	CT(20112)=(S=6026768,E=6026777,C=18)
	CT(20113)=(S=6026778,E=6026781,C=14)
	CT(20114)=(S=6026782,E=6026783,C=18)
	CT(20115)=(S=6026784,E=6026815,C=31)
	CT(20116)=(S=6026816,E=6026831,C=58)
	CT(20117)=(S=6026832,E=6026855,C=18)
	CT(20118)=(S=6026856,E=6026863,C=58)
	CT(20119)=(S=6026864,E=6026879,C=18)
	CT(20120)=(S=6026880,E=6026911,C=31)
	CT(20121)=(S=6026912,E=6026919,C=18)
	CT(20122)=(S=6026920,E=6026923,C=58)
	CT(20123)=(S=6026924,E=6026953,C=38)
	CT(20124)=(S=6026954,E=6026954,C=28)
	CT(20125)=(S=6026955,E=6026959,C=38)
	CT(20126)=(S=6026960,E=6026967,C=58)
	CT(20127)=(S=6026968,E=6026969,C=38)
	CT(20128)=(S=6026970,E=6026970,C=17)
	CT(20129)=(S=6026971,E=6026971,C=18)
	CT(20130)=(S=6026972,E=6027007,C=31)
	CT(20131)=(S=6027008,E=6027039,C=18)
	CT(20132)=(S=6027040,E=6027071,C=31)
	CT(20133)=(S=6027072,E=6027073,C=33)
	CT(20134)=(S=6027074,E=6027075,C=26)
	CT(20135)=(S=6027076,E=6027079,C=18)
	CT(20136)=(S=6027080,E=6027080,C=60)
	CT(20137)=(S=6027081,E=6027081,C=14)
	CT(20138)=(S=6027082,E=6027082,C=27)
	CT(20139)=(S=6027083,E=6027083,C=12)
	CT(20140)=(S=6027084,E=6027103,C=31)
	CT(20141)=(S=6027104,E=6027151,C=18)
	CT(20142)=(S=6027152,E=6027153,C=28)
	CT(20143)=(S=6027154,E=6027154,C=38)
	CT(20144)=(S=6027155,E=6027155,C=58)
	CT(20145)=(S=6027156,E=6027167,C=18)
	CT(20146)=(S=6027168,E=6027199,C=31)
	CT(20147)=(S=6027200,E=6027228,C=18)
	CT(20148)=(S=6027229,E=6027229,C=31)
	CT(20149)=(S=6027230,E=6027231,C=18)
	CT(20150)=(S=6027232,E=6027235,C=31)
	CT(20151)=(S=6027236,E=6027239,C=27)
	CT(20152)=(S=6027240,E=6027247,C=18)
	CT(20153)=(S=6027248,E=6027255,C=38)
	CT(20154)=(S=6027256,E=6027259,C=18)
	CT(20155)=(S=6027260,E=6027260,C=58)
	CT(20156)=(S=6027261,E=6027261,C=34)
	CT(20157)=(S=6027262,E=6027263,C=18)
	CT(20158)=(S=6027264,E=6027775,C=28)
	CT(20159)=(S=6027776,E=6027839,C=31)
	CT(20160)=(S=6027840,E=6027903,C=28)
	CT(20161)=(S=6027904,E=6027935,C=38)
	CT(20162)=(S=6027936,E=6027967,C=13)
	CT(20163)=(S=6027968,E=6027999,C=19)
	CT(20164)=(S=6028000,E=6028015,C=53)
	CT(20165)=(S=6028016,E=6028020,C=38)
	CT(20166)=(S=6028021,E=6028021,C=53)
	CT(20167)=(S=6028022,E=6028031,C=38)
	CT(20168)=(S=6028032,E=6028287,C=26)
	CT(20169)=(S=6028288,E=6029311,C=15)
	CT(20170)=(S=6029312,E=6037503,C=14)
	CT(20171)=(S=6037504,E=6038527,C=19)
	CT(20172)=(S=6038528,E=6038655,C=18)
	CT(20173)=(S=6038656,E=6038783,C=59)
	CT(20174)=(S=6038784,E=6038911,C=52)
	CT(20175)=(S=6038912,E=6039039,C=18)
	CT(20176)=(S=6039040,E=6039167,C=33)
	CT(20177)=(S=6039168,E=6039295,C=18)
	CT(20178)=(S=6039296,E=6039311,C=19)
	CT(20179)=(S=6039312,E=6039327,C=28)
	CT(20180)=(S=6039328,E=6039343,C=19)
	CT(20181)=(S=6039344,E=6039359,C=53)
	CT(20182)=(S=6039360,E=6039375,C=18)
	CT(20183)=(S=6039376,E=6039391,C=41)
	CT(20184)=(S=6039392,E=6039407,C=18)
	CT(20185)=(S=6039408,E=6039423,C=14)
	CT(20186)=(S=6039424,E=6039439,C=18)
	CT(20187)=(S=6039440,E=6039455,C=15)
	CT(20188)=(S=6039456,E=6039471,C=42)
	CT(20189)=(S=6039472,E=6039487,C=63)
	CT(20190)=(S=6039488,E=6039503,C=14)
	CT(20191)=(S=6039504,E=6039535,C=18)
	CT(20192)=(S=6039536,E=6039551,C=12)
	CT(20193)=(S=6039552,E=6040063,C=14)
	CT(20194)=(S=6040064,E=6040095,C=18)
	CT(20195)=(S=6040096,E=6040103,C=37)
	CT(20196)=(S=6040104,E=6040111,C=18)
	CT(20197)=(S=6040112,E=6040119,C=26)
	CT(20198)=(S=6040120,E=6040127,C=14)
	CT(20199)=(S=6040128,E=6040135,C=27)
	CT(20200)=(S=6040136,E=6040143,C=19)
	CT(20201)=(S=6040144,E=6040151,C=15)
	CT(20202)=(S=6040152,E=6040159,C=18)
	CT(20203)=(S=6040160,E=6040167,C=15)
	CT(20204)=(S=6040168,E=6040175,C=34)
	CT(20205)=(S=6040176,E=6040183,C=58)
	CT(20206)=(S=6040184,E=6040191,C=14)
	CT(20207)=(S=6040192,E=6040199,C=18)
	CT(20208)=(S=6040200,E=6040207,C=42)
	CT(20209)=(S=6040208,E=6040215,C=14)
	CT(20210)=(S=6040216,E=6040223,C=19)
	CT(20211)=(S=6040224,E=6040231,C=18)
	CT(20212)=(S=6040232,E=6040239,C=14)
	CT(20213)=(S=6040240,E=6040247,C=70)
	CT(20214)=(S=6040248,E=6040255,C=34)
	CT(20215)=(S=6040256,E=6040263,C=28)
	CT(20216)=(S=6040264,E=6040279,C=18)
	CT(20217)=(S=6040280,E=6040287,C=12)
	CT(20218)=(S=6040288,E=6040295,C=28)
	CT(20219)=(S=6040296,E=6040303,C=36)
	CT(20220)=(S=6040304,E=6040311,C=34)
	CT(20221)=(S=6040312,E=6040319,C=46)
	CT(20222)=(S=6040320,E=6040327,C=18)
	CT(20223)=(S=6040328,E=6040335,C=27)
	CT(20224)=(S=6040336,E=6040343,C=25)
	CT(20225)=(S=6040344,E=6040351,C=33)
	CT(20226)=(S=6040352,E=6040359,C=19)
	CT(20227)=(S=6040360,E=6040367,C=14)
	CT(20228)=(S=6040368,E=6040375,C=28)
	CT(20229)=(S=6040376,E=6040383,C=33)
	CT(20230)=(S=6040384,E=6040391,C=14)
	CT(20231)=(S=6040392,E=6040399,C=24)
	CT(20232)=(S=6040400,E=6040407,C=37)
	CT(20233)=(S=6040408,E=6040415,C=24)
	CT(20234)=(S=6040416,E=6040423,C=42)
	CT(20235)=(S=6040424,E=6040431,C=28)
	CT(20236)=(S=6040432,E=6040439,C=58)
	CT(20237)=(S=6040440,E=6040447,C=24)
	CT(20238)=(S=6040448,E=6040455,C=12)
	CT(20239)=(S=6040456,E=6040463,C=66)
	CT(20240)=(S=6040464,E=6040471,C=58)
	CT(20241)=(S=6040472,E=6040487,C=18)
	CT(20242)=(S=6040488,E=6040495,C=23)
	CT(20243)=(S=6040496,E=6040503,C=24)
	CT(20244)=(S=6040504,E=6040511,C=18)
	CT(20245)=(S=6040512,E=6040519,C=48)
	CT(20246)=(S=6040520,E=6040527,C=50)
	CT(20247)=(S=6040528,E=6040535,C=14)
	CT(20248)=(S=6040536,E=6040543,C=34)
	CT(20249)=(S=6040544,E=6040551,C=25)
	CT(20250)=(S=6040552,E=6040559,C=24)
	CT(20251)=(S=6040560,E=6040567,C=15)
	CT(20252)=(S=6040568,E=6040575,C=12)
	CT(20253)=(S=6040576,E=6041087,C=37)
	CT(20254)=(S=6041088,E=6041599,C=20)
	CT(20255)=(S=6041600,E=6041663,C=23)
	CT(20256)=(S=6041664,E=6041727,C=14)
	CT(20257)=(S=6041728,E=6041791,C=49)
	CT(20258)=(S=6041792,E=6041855,C=36)
	CT(20259)=(S=6041856,E=6041919,C=60)
	CT(20260)=(S=6041920,E=6041983,C=12)
	CT(20261)=(S=6041984,E=6042047,C=18)
	CT(20262)=(S=6042048,E=6042111,C=20)
	CT(20263)=(S=6042112,E=6042175,C=26)
	CT(20264)=(S=6042176,E=6042239,C=28)
	CT(20265)=(S=6042240,E=6042431,C=18)
	CT(20266)=(S=6042432,E=6042495,C=40)
	CT(20267)=(S=6042496,E=6042559,C=28)
	CT(20268)=(S=6042560,E=6042623,C=63)
	CT(20269)=(S=6042624,E=6042687,C=71)
	CT(20270)=(S=6042688,E=6042751,C=14)
	CT(20271)=(S=6042752,E=6042815,C=31)
	CT(20272)=(S=6042816,E=6042879,C=50)
	CT(20273)=(S=6042880,E=6042943,C=54)
	CT(20274)=(S=6042944,E=6043007,C=18)
	CT(20275)=(S=6043008,E=6043071,C=52)
	CT(20276)=(S=6043072,E=6043103,C=25)
	CT(20277)=(S=6043104,E=6043111,C=18)
	CT(20278)=(S=6043112,E=6043119,C=19)
	CT(20279)=(S=6043120,E=6043127,C=14)
	CT(20280)=(S=6043128,E=6043135,C=36)
	CT(20281)=(S=6043136,E=6043199,C=25)
	CT(20282)=(S=6043200,E=6043263,C=18)
	CT(20283)=(S=6043264,E=6043327,C=14)
	CT(20284)=(S=6043328,E=6043391,C=40)
	CT(20285)=(S=6043392,E=6043455,C=18)
	CT(20286)=(S=6043456,E=6043519,C=54)
	CT(20287)=(S=6043520,E=6043551,C=38)
	CT(20288)=(S=6043552,E=6043583,C=20)
	CT(20289)=(S=6043584,E=6043647,C=58)
	CT(20290)=(S=6043648,E=6044671,C=25)
	CT(20291)=(S=6044672,E=6044687,C=42)
	CT(20292)=(S=6044688,E=6044703,C=96)
	CT(20293)=(S=6044720,E=6044735,C=71)
	CT(20294)=(S=6044736,E=6044751,C=15)
	CT(20295)=(S=6044752,E=6044767,C=18)
	CT(20296)=(S=6044768,E=6044799,C=14)
	CT(20297)=(S=6044800,E=6044815,C=58)
	CT(20298)=(S=6044816,E=6044831,C=24)
	CT(20299)=(S=6044832,E=6044847,C=25)
	CT(20300)=(S=6044848,E=6044863,C=31)
	CT(20301)=(S=6044864,E=6044879,C=63)
	CT(20302)=(S=6044880,E=6044895,C=28)
	CT(20303)=(S=6044896,E=6044911,C=46)
	CT(20304)=(S=6044912,E=6044927,C=14)
	CT(20305)=(S=6044928,E=6044943,C=37)
	CT(20306)=(S=6044944,E=6044959,C=18)
	CT(20307)=(S=6044960,E=6044975,C=44)
	CT(20308)=(S=6044976,E=6044991,C=42)
	CT(20309)=(S=6044992,E=6045007,C=18)
	CT(20310)=(S=6045008,E=6045023,C=33)
	CT(20311)=(S=6045024,E=6045055,C=50)
	CT(20312)=(S=6045056,E=6045071,C=12)
	CT(20313)=(S=6045072,E=6045087,C=13)
	CT(20314)=(S=6045088,E=6045103,C=12)
	CT(20315)=(S=6045104,E=6045119,C=26)
	CT(20316)=(S=6045120,E=6045135,C=63)
	CT(20317)=(S=6045136,E=6045151,C=42)
	CT(20318)=(S=6045152,E=6045167,C=17)
	CT(20319)=(S=6045168,E=6045183,C=36)
	CT(20320)=(S=6045184,E=6045199,C=14)
	CT(20321)=(S=6045200,E=6045215,C=42)
	CT(20322)=(S=6045216,E=6045231,C=27)
	CT(20323)=(S=6045232,E=6045247,C=18)
	CT(20324)=(S=6045248,E=6045263,C=62)
	CT(20325)=(S=6045264,E=6045279,C=15)
	CT(20326)=(S=6045280,E=6045295,C=55)
	CT(20327)=(S=6045312,E=6045327,C=44)
	CT(20328)=(S=6045328,E=6045343,C=18)
	CT(20329)=(S=6045344,E=6045359,C=39)
	CT(20330)=(S=6045360,E=6045375,C=26)
	CT(20331)=(S=6045376,E=6045391,C=24)
	CT(20332)=(S=6045392,E=6045407,C=99)
	CT(20333)=(S=6045408,E=6045423,C=33)
	CT(20334)=(S=6045424,E=6045439,C=51)
	CT(20335)=(S=6045440,E=6045455,C=37)
	CT(20336)=(S=6045456,E=6045471,C=52)
	CT(20337)=(S=6045472,E=6045487,C=58)
	CT(20338)=(S=6045488,E=6045503,C=33)
	CT(20339)=(S=6045504,E=6045519,C=18)
	CT(20340)=(S=6045520,E=6045535,C=60)
	CT(20341)=(S=6045536,E=6045551,C=18)
	CT(20342)=(S=6045552,E=6045563,C=60)
	CT(20343)=(S=6045564,E=6045567,C=25)
	CT(20344)=(S=6045568,E=6045583,C=14)
	CT(20345)=(S=6045584,E=6045599,C=42)
	CT(20346)=(S=6045600,E=6045615,C=36)
	CT(20347)=(S=6045616,E=6045647,C=18)
	CT(20348)=(S=6045648,E=6045663,C=42)
	CT(20349)=(S=6045664,E=6045679,C=15)
	CT(20350)=(S=6045680,E=6045695,C=50)
	CT(20351)=(S=6045696,E=6047743,C=36)
	CT(20352)=(S=6047744,E=6049791,C=28)
	CT(20353)=(S=6049792,E=6051839,C=38)
	CT(20354)=(S=6051840,E=6053887,C=12)
	CT(20355)=(S=6053888,E=6054911,C=16)
	CT(20356)=(S=6054912,E=6055423,C=18)
	CT(20357)=(S=6055424,E=6055935,C=12)
	CT(20358)=(S=6055936,E=6056959,C=34)
	CT(20359)=(S=6056960,E=6057983,C=36)
	CT(20360)=(S=6057984,E=6058495,C=31)
	CT(20361)=(S=6058496,E=6058511,C=38)
	CT(20362)=(S=6058512,E=6058527,C=26)
	CT(20363)=(S=6058528,E=6058530,C=38)
	CT(20364)=(S=6058531,E=6058531,C=36)
	CT(20365)=(S=6058532,E=6058532,C=95)
	CT(20366)=(S=6058533,E=6058535,C=38)
	CT(20367)=(S=6058536,E=6058543,C=43)
	CT(20368)=(S=6058544,E=6058547,C=13)
	CT(20369)=(S=6058548,E=6058548,C=38)
	CT(20370)=(S=6058549,E=6058549,C=95)
	CT(20371)=(S=6058550,E=6058551,C=38)
	CT(20372)=(S=6058552,E=6058559,C=95)
	CT(20373)=(S=6058560,E=6058578,C=38)
	CT(20374)=(S=6058579,E=6058579,C=95)
	CT(20375)=(S=6058580,E=6058587,C=38)
	CT(20376)=(S=6058588,E=6058588,C=36)
	CT(20377)=(S=6058589,E=6059007,C=38)
	CT(20378)=(S=6059008,E=6059519,C=28)
	CT(20379)=(S=6059520,E=6060031,C=22)
	CT(20380)=(S=6060032,E=6060543,C=36)
	CT(20381)=(S=6060544,E=6061055,C=13)
	CT(20382)=(S=6061056,E=6062079,C=18)
	CT(20383)=(S=6062080,E=6078463,C=12)
	CT(20384)=(S=6078464,E=6082303,C=28)
	CT(20385)=(S=6082304,E=6082559,C=14)
	CT(20386)=(S=6082560,E=6085631,C=28)
	CT(20387)=(S=6085632,E=6086143,C=27)
	CT(20388)=(S=6086144,E=6086399,C=12)
	CT(20389)=(S=6086400,E=6086527,C=42)
	CT(20390)=(S=6086528,E=6086655,C=15)
	CT(20391)=(S=6086656,E=6088703,C=28)
	CT(20392)=(S=6088704,E=6089471,C=14)
	CT(20393)=(S=6089472,E=6089727,C=36)
	CT(20394)=(S=6089728,E=6090751,C=14)
	CT(20395)=(S=6090752,E=6090783,C=17)
	CT(20396)=(S=6090784,E=6090815,C=59)
	CT(20397)=(S=6090816,E=6090847,C=60)
	CT(20398)=(S=6090848,E=6090879,C=31)
	CT(20399)=(S=6090880,E=6090895,C=18)
	CT(20400)=(S=6090896,E=6090903,C=36)
	CT(20401)=(S=6090904,E=6090911,C=47)
	CT(20402)=(S=6090912,E=6090943,C=33)
	CT(20403)=(S=6090944,E=6090975,C=18)
	CT(20404)=(S=6090976,E=6091007,C=71)
	CT(20405)=(S=6091008,E=6091039,C=18)
	CT(20406)=(S=6091040,E=6091071,C=61)
	CT(20407)=(S=6091072,E=6091103,C=40)
	CT(20408)=(S=6091104,E=6091135,C=18)
	CT(20409)=(S=6091136,E=6091167,C=59)
	CT(20410)=(S=6091168,E=6091199,C=18)
	CT(20411)=(S=6091200,E=6091231,C=19)
	CT(20412)=(S=6091232,E=6091359,C=18)
	CT(20413)=(S=6091360,E=6091391,C=31)
	CT(20414)=(S=6091392,E=6091423,C=14)
	CT(20415)=(S=6091456,E=6091487,C=26)
	CT(20416)=(S=6091488,E=6091519,C=57)
	CT(20417)=(S=6091520,E=6091551,C=12)
	CT(20418)=(S=6091552,E=6091583,C=42)
	CT(20419)=(S=6091584,E=6091647,C=18)
	CT(20420)=(S=6091648,E=6091679,C=15)
	CT(20421)=(S=6091680,E=6091711,C=18)
	CT(20422)=(S=6091712,E=6091743,C=33)
	CT(20423)=(S=6091744,E=6091775,C=24)
	CT(20424)=(S=6091776,E=6091807,C=19)
	CT(20425)=(S=6091808,E=6091839,C=58)
	CT(20426)=(S=6091840,E=6091871,C=52)
	CT(20427)=(S=6091872,E=6091903,C=31)
	CT(20428)=(S=6091904,E=6091935,C=46)
	CT(20429)=(S=6091936,E=6091967,C=14)
	CT(20430)=(S=6091968,E=6091999,C=19)
	CT(20431)=(S=6092000,E=6092031,C=18)
	CT(20432)=(S=6092032,E=6092063,C=71)
	CT(20433)=(S=6092064,E=6092095,C=18)
	CT(20434)=(S=6092096,E=6092127,C=50)
	CT(20435)=(S=6092128,E=6092159,C=62)
	CT(20436)=(S=6092160,E=6092191,C=12)
	CT(20437)=(S=6092192,E=6092223,C=15)
	CT(20438)=(S=6092224,E=6092255,C=71)
	CT(20439)=(S=6092256,E=6092287,C=14)
	CT(20440)=(S=6092288,E=6092319,C=24)
	CT(20441)=(S=6092320,E=6092415,C=15)
	CT(20442)=(S=6092416,E=6092511,C=18)
	CT(20443)=(S=6092512,E=6092543,C=63)
	CT(20444)=(S=6092544,E=6092799,C=51)
	CT(20445)=(S=6092800,E=6092927,C=42)
	CT(20446)=(S=6092928,E=6093055,C=18)
	CT(20447)=(S=6093120,E=6093183,C=31)
	CT(20448)=(S=6093184,E=6093311,C=50)
	CT(20449)=(S=6093312,E=6093439,C=21)
	CT(20450)=(S=6093440,E=6093567,C=70)
	CT(20451)=(S=6093568,E=6093695,C=72)
	CT(20452)=(S=6093696,E=6093823,C=63)
	CT(20453)=(S=6093824,E=6093951,C=28)
	CT(20454)=(S=6093952,E=6094079,C=18)
	CT(20455)=(S=6094080,E=6094207,C=61)
	CT(20456)=(S=6094208,E=6094335,C=33)
	CT(20457)=(S=6094336,E=6094463,C=36)
	CT(20458)=(S=6094464,E=6094591,C=19)
	CT(20459)=(S=6094592,E=6094847,C=18)
	CT(20460)=(S=6094848,E=6103039,C=12)
	CT(20461)=(S=6103040,E=6113279,C=15)
	CT(20462)=(S=6113280,E=6115327,C=31)
	CT(20463)=(S=6115328,E=6115839,C=18)
	CT(20464)=(S=6115840,E=6116351,C=42)
	CT(20465)=(S=6116352,E=6116863,C=68)
	CT(20466)=(S=6116864,E=6117375,C=46)
	CT(20467)=(S=6117376,E=6117407,C=18)
	CT(20468)=(S=6117408,E=6117423,C=15)
	CT(20469)=(S=6117424,E=6117439,C=31)
	CT(20470)=(S=6117440,E=6117455,C=18)
	CT(20471)=(S=6117456,E=6117471,C=41)
	CT(20472)=(S=6117472,E=6117503,C=15)
	CT(20473)=(S=6117504,E=6117519,C=18)
	CT(20474)=(S=6117520,E=6117535,C=36)
	CT(20475)=(S=6117536,E=6117567,C=18)
	CT(20476)=(S=6117568,E=6117583,C=50)
	CT(20477)=(S=6117584,E=6117599,C=18)
	CT(20478)=(S=6117600,E=6117615,C=28)
	CT(20479)=(S=6117616,E=6117631,C=34)
	CT(20480)=(S=6117632,E=6117647,C=28)
	CT(20481)=(S=6117648,E=6117663,C=37)
	CT(20482)=(S=6117664,E=6117679,C=27)
	CT(20483)=(S=6117680,E=6117695,C=15)
	CT(20484)=(S=6117696,E=6117711,C=37)
	CT(20485)=(S=6117712,E=6117727,C=14)
	CT(20486)=(S=6117728,E=6117743,C=33)
	CT(20487)=(S=6117744,E=6117759,C=27)
	CT(20488)=(S=6117760,E=6117775,C=14)
	CT(20489)=(S=6117776,E=6117791,C=33)
	CT(20490)=(S=6117792,E=6117807,C=50)
	CT(20491)=(S=6117808,E=6117823,C=18)
	CT(20492)=(S=6117824,E=6117839,C=58)
	CT(20493)=(S=6117840,E=6117855,C=31)
	CT(20494)=(S=6117856,E=6117871,C=37)
	CT(20495)=(S=6117872,E=6117887,C=63)
	CT(20496)=(S=6117888,E=6117903,C=24)
	CT(20497)=(S=6117904,E=6117919,C=25)
	CT(20498)=(S=6117920,E=6117935,C=18)
	CT(20499)=(S=6117936,E=6117951,C=47)
	CT(20500)=(S=6117968,E=6117999,C=18)
	CT(20501)=(S=6118000,E=6118015,C=24)
	CT(20502)=(S=6118016,E=6118031,C=28)
	CT(20503)=(S=6118032,E=6118047,C=19)
	CT(20504)=(S=6118048,E=6118063,C=33)
	CT(20505)=(S=6118064,E=6118079,C=28)
	CT(20506)=(S=6118080,E=6118095,C=14)
	CT(20507)=(S=6118096,E=6118159,C=18)
	CT(20508)=(S=6118160,E=6118191,C=14)
	CT(20509)=(S=6118192,E=6118207,C=33)
	CT(20510)=(S=6118208,E=6118223,C=37)
	CT(20511)=(S=6118224,E=6118239,C=28)
	CT(20512)=(S=6118240,E=6118255,C=27)
	CT(20513)=(S=6118256,E=6118271,C=18)
	CT(20514)=(S=6118272,E=6118287,C=15)
	CT(20515)=(S=6118288,E=6118303,C=33)
	CT(20516)=(S=6118304,E=6118319,C=18)
	CT(20517)=(S=6118320,E=6118335,C=47)
	CT(20518)=(S=6118336,E=6118351,C=35)
	CT(20519)=(S=6118352,E=6118367,C=58)
	CT(20520)=(S=6118368,E=6118383,C=18)
	CT(20521)=(S=6118384,E=6118399,C=33)
	CT(20522)=(S=6118400,E=6118407,C=19)
	CT(20523)=(S=6118408,E=6118415,C=63)
	CT(20524)=(S=6118416,E=6118423,C=49)
	CT(20525)=(S=6118424,E=6118431,C=36)
	CT(20526)=(S=6118432,E=6118439,C=18)
	CT(20527)=(S=6118440,E=6118447,C=14)
	CT(20528)=(S=6118448,E=6118455,C=33)
	CT(20529)=(S=6118456,E=6118463,C=50)
	CT(20530)=(S=6118464,E=6118471,C=18)
	CT(20531)=(S=6118472,E=6118479,C=15)
	CT(20532)=(S=6118480,E=6118495,C=18)
	CT(20533)=(S=6118496,E=6118503,C=36)
	CT(20534)=(S=6118504,E=6118511,C=12)
	CT(20535)=(S=6118512,E=6118519,C=18)
	CT(20536)=(S=6118520,E=6118527,C=14)
	CT(20537)=(S=6118528,E=6118535,C=28)
	CT(20538)=(S=6118536,E=6118543,C=18)
	CT(20539)=(S=6118544,E=6118551,C=28)
	CT(20540)=(S=6118552,E=6118559,C=15)
	CT(20541)=(S=6118560,E=6118567,C=42)
	CT(20542)=(S=6118568,E=6118575,C=25)
	CT(20543)=(S=6118576,E=6118583,C=18)
	CT(20544)=(S=6118584,E=6118591,C=15)
	CT(20545)=(S=6118592,E=6118607,C=18)
	CT(20546)=(S=6118608,E=6118615,C=14)
	CT(20547)=(S=6118616,E=6118623,C=18)
	CT(20548)=(S=6118632,E=6118639,C=25)
	CT(20549)=(S=6118640,E=6118647,C=15)
	CT(20550)=(S=6118648,E=6118655,C=46)
	CT(20551)=(S=6118656,E=6118663,C=28)
	CT(20552)=(S=6118664,E=6118671,C=51)
	CT(20553)=(S=6118672,E=6118679,C=15)
	CT(20554)=(S=6118680,E=6118687,C=37)
	CT(20555)=(S=6118688,E=6118695,C=33)
	CT(20556)=(S=6118696,E=6118703,C=12)
	CT(20557)=(S=6118704,E=6118711,C=70)
	CT(20558)=(S=6118712,E=6118719,C=44)
	CT(20559)=(S=6118720,E=6118727,C=25)
	CT(20560)=(S=6118728,E=6118735,C=27)
	CT(20561)=(S=6118736,E=6118743,C=14)
	CT(20562)=(S=6118744,E=6118751,C=18)
	CT(20563)=(S=6118752,E=6118759,C=27)
	CT(20564)=(S=6118768,E=6118775,C=25)
	CT(20565)=(S=6118776,E=6118783,C=36)
	CT(20566)=(S=6118784,E=6118791,C=14)
	CT(20567)=(S=6118792,E=6118799,C=18)
	CT(20568)=(S=6118800,E=6118807,C=61)
	CT(20569)=(S=6118808,E=6118815,C=12)
	CT(20570)=(S=6118816,E=6118823,C=15)
	CT(20571)=(S=6118824,E=6118831,C=18)
	CT(20572)=(S=6118832,E=6118839,C=25)
	CT(20573)=(S=6118840,E=6118847,C=12)
	CT(20574)=(S=6118848,E=6118855,C=46)
	CT(20575)=(S=6118856,E=6118863,C=15)
	CT(20576)=(S=6118864,E=6118895,C=14)
	CT(20577)=(S=6118904,E=6118919,C=28)
	CT(20578)=(S=6118920,E=6118927,C=27)
	CT(20579)=(S=6118928,E=6118935,C=58)
	CT(20580)=(S=6118936,E=6118951,C=15)
	CT(20581)=(S=6118952,E=6118959,C=17)
	CT(20582)=(S=6118960,E=6118967,C=35)
	CT(20583)=(S=6118968,E=6118975,C=21)
	CT(20584)=(S=6118976,E=6118983,C=34)
	CT(20585)=(S=6118984,E=6118991,C=14)
	CT(20586)=(S=6118992,E=6118999,C=28)
	CT(20587)=(S=6119000,E=6119007,C=15)
	CT(20588)=(S=6119008,E=6119015,C=25)
	CT(20589)=(S=6119016,E=6119023,C=49)
	CT(20590)=(S=6119024,E=6119031,C=63)
	CT(20591)=(S=6119032,E=6119039,C=14)
	CT(20592)=(S=6119040,E=6119047,C=28)
	CT(20593)=(S=6119048,E=6119055,C=51)
	CT(20594)=(S=6119056,E=6119071,C=18)
	CT(20595)=(S=6119072,E=6119079,C=12)
	CT(20596)=(S=6119080,E=6119087,C=19)
	CT(20597)=(S=6119088,E=6119095,C=18)
	CT(20598)=(S=6119096,E=6119103,C=58)
	CT(20599)=(S=6119104,E=6119111,C=37)
	CT(20600)=(S=6119112,E=6119119,C=36)
	CT(20601)=(S=6119120,E=6119127,C=19)
	CT(20602)=(S=6119128,E=6119135,C=66)
	CT(20603)=(S=6119136,E=6119143,C=36)
	CT(20604)=(S=6119144,E=6119151,C=18)
	CT(20605)=(S=6119152,E=6119159,C=34)
	CT(20606)=(S=6119160,E=6119167,C=37)
	CT(20607)=(S=6119168,E=6119175,C=36)
	CT(20608)=(S=6119176,E=6119183,C=14)
	CT(20609)=(S=6119184,E=6119191,C=172)
	CT(20610)=(S=6119192,E=6119199,C=104)
	CT(20611)=(S=6119200,E=6119207,C=33)
	CT(20612)=(S=6119208,E=6119215,C=18)
	CT(20613)=(S=6119216,E=6119223,C=49)
	CT(20614)=(S=6119224,E=6119231,C=12)
	CT(20615)=(S=6119232,E=6119239,C=54)
	CT(20616)=(S=6119240,E=6119247,C=48)
	CT(20617)=(S=6119248,E=6119255,C=63)
	CT(20618)=(S=6119256,E=6119263,C=58)
	CT(20619)=(S=6119264,E=6119271,C=18)
	CT(20620)=(S=6119272,E=6119279,C=14)
	CT(20621)=(S=6119280,E=6119287,C=36)
	CT(20622)=(S=6119288,E=6119295,C=14)
	CT(20623)=(S=6119296,E=6119303,C=28)
	CT(20624)=(S=6119304,E=6119311,C=18)
	CT(20625)=(S=6119312,E=6119319,C=36)
	CT(20626)=(S=6119320,E=6119343,C=18)
	CT(20627)=(S=6119344,E=6119351,C=37)
	CT(20628)=(S=6119352,E=6119359,C=31)
	CT(20629)=(S=6119360,E=6119367,C=18)
	CT(20630)=(S=6119368,E=6119375,C=61)
	CT(20631)=(S=6119376,E=6119391,C=15)
	CT(20632)=(S=6119392,E=6119399,C=48)
	CT(20633)=(S=6119400,E=6119407,C=12)
	CT(20634)=(S=6119408,E=6119415,C=20)
	CT(20635)=(S=6119416,E=6119423,C=36)
	CT(20636)=(S=6119424,E=6119935,C=14)
	CT(20637)=(S=6119936,E=6120191,C=23)
	CT(20638)=(S=6120192,E=6120447,C=33)
	CT(20639)=(S=6120448,E=6120703,C=18)
	CT(20640)=(S=6120704,E=6120959,C=15)
	CT(20641)=(S=6120960,E=6121215,C=21)
	CT(20642)=(S=6121216,E=6121471,C=52)
	CT(20643)=(S=6121472,E=6121727,C=28)
	CT(20644)=(S=6121728,E=6121983,C=58)
	CT(20645)=(S=6121984,E=6122239,C=47)
	CT(20646)=(S=6122240,E=6122495,C=63)
	CT(20647)=(S=6122496,E=6122751,C=21)
	CT(20648)=(S=6122752,E=6123007,C=32)
	CT(20649)=(S=6123008,E=6123263,C=26)
	CT(20650)=(S=6123264,E=6123519,C=42)
	CT(20651)=(S=6123520,E=6123775,C=23)
	CT(20652)=(S=6123776,E=6123823,C=38)
	CT(20653)=(S=6123824,E=6123827,C=25)
	CT(20654)=(S=6123828,E=6123885,C=38)
	CT(20655)=(S=6123886,E=6123887,C=14)
	CT(20656)=(S=6123888,E=6123895,C=53)
	CT(20657)=(S=6123896,E=6123899,C=95)
	CT(20658)=(S=6123900,E=6123902,C=38)
	CT(20659)=(S=6123903,E=6123903,C=14)
	CT(20660)=(S=6123904,E=6123991,C=38)
	CT(20661)=(S=6123992,E=6123995,C=95)
	CT(20662)=(S=6123996,E=6123999,C=38)
	CT(20663)=(S=6124000,E=6124015,C=26)
	CT(20664)=(S=6124016,E=6124047,C=38)
	CT(20665)=(S=6124048,E=6124063,C=26)
	CT(20666)=(S=6124064,E=6124082,C=38)
	CT(20667)=(S=6124083,E=6124086,C=14)
	CT(20668)=(S=6124087,E=6124089,C=38)
	CT(20669)=(S=6124090,E=6124091,C=14)
	CT(20670)=(S=6124092,E=6124095,C=25)
	CT(20671)=(S=6124096,E=6124117,C=38)
	CT(20672)=(S=6124118,E=6124119,C=14)
	CT(20673)=(S=6124120,E=6124135,C=38)
	CT(20674)=(S=6124136,E=6124143,C=26)
	CT(20675)=(S=6124144,E=6124175,C=38)
	CT(20676)=(S=6124176,E=6124191,C=25)
	CT(20677)=(S=6124192,E=6124215,C=38)
	CT(20678)=(S=6124216,E=6124217,C=14)
	CT(20679)=(S=6124218,E=6124227,C=38)
	CT(20680)=(S=6124228,E=6124231,C=95)
	CT(20681)=(S=6124232,E=6124239,C=25)
	CT(20682)=(S=6124240,E=6124255,C=38)
	CT(20683)=(S=6124256,E=6124263,C=25)
	CT(20684)=(S=6124264,E=6124265,C=38)
	CT(20685)=(S=6124266,E=6124267,C=14)
	CT(20686)=(S=6124268,E=6124283,C=38)
	CT(20687)=(S=6124284,E=6124287,C=25)
	CT(20688)=(S=6124288,E=6124320,C=38)
	CT(20689)=(S=6124321,E=6124321,C=28)
	CT(20690)=(S=6124322,E=6124325,C=38)
	CT(20691)=(S=6124326,E=6124326,C=170)
	CT(20692)=(S=6124327,E=6124346,C=38)
	CT(20693)=(S=6124347,E=6124347,C=14)
	CT(20694)=(S=6124348,E=6124349,C=36)
	CT(20695)=(S=6124350,E=6124351,C=15)
	CT(20696)=(S=6124352,E=6124383,C=38)
	CT(20697)=(S=6124384,E=6124385,C=12)
	CT(20698)=(S=6124386,E=6124407,C=38)
	CT(20699)=(S=6124408,E=6124415,C=26)
	CT(20700)=(S=6124416,E=6124431,C=38)
	CT(20701)=(S=6124432,E=6124439,C=53)
	CT(20702)=(S=6124440,E=6124443,C=38)
	CT(20703)=(S=6124444,E=6124447,C=25)
	CT(20704)=(S=6124448,E=6124455,C=19)
	CT(20705)=(S=6124456,E=6124488,C=38)
	CT(20706)=(S=6124489,E=6124489,C=36)
	CT(20707)=(S=6124490,E=6124491,C=38)
	CT(20708)=(S=6124492,E=6124495,C=25)
	CT(20709)=(S=6124496,E=6124499,C=38)
	CT(20710)=(S=6124500,E=6124503,C=25)
	CT(20711)=(S=6124504,E=6124527,C=53)
	CT(20712)=(S=6124528,E=6124799,C=38)
	CT(20713)=(S=6124800,E=6124847,C=26)
	CT(20714)=(S=6124848,E=6124895,C=38)
	CT(20715)=(S=6124896,E=6124927,C=26)
	CT(20716)=(S=6124928,E=6124936,C=38)
	CT(20717)=(S=6124937,E=6124937,C=36)
	CT(20718)=(S=6124938,E=6124951,C=38)
	CT(20719)=(S=6124952,E=6124953,C=15)
	CT(20720)=(S=6124954,E=6124971,C=38)
	CT(20721)=(S=6124972,E=6124973,C=15)
	CT(20722)=(S=6124974,E=6124975,C=38)
	CT(20723)=(S=6124976,E=6124991,C=26)
	CT(20724)=(S=6124992,E=6125055,C=36)
	CT(20725)=(S=6125056,E=6125087,C=28)
	CT(20726)=(S=6125088,E=6125091,C=12)
	CT(20727)=(S=6125092,E=6125093,C=38)
	CT(20728)=(S=6125094,E=6125094,C=44)
	CT(20729)=(S=6125095,E=6125119,C=38)
	CT(20730)=(S=6125120,E=6125135,C=95)
	CT(20731)=(S=6125136,E=6125151,C=38)
	CT(20732)=(S=6125152,E=6125231,C=26)
	CT(20733)=(S=6125232,E=6125235,C=38)
	CT(20734)=(S=6125236,E=6125243,C=53)
	CT(20735)=(S=6125244,E=6125339,C=38)
	CT(20736)=(S=6125340,E=6125343,C=25)
	CT(20737)=(S=6125344,E=6125407,C=26)
	CT(20738)=(S=6125408,E=6125423,C=38)
	CT(20739)=(S=6125424,E=6125425,C=14)
	CT(20740)=(S=6125426,E=6125429,C=38)
	CT(20741)=(S=6125430,E=6125433,C=14)
	CT(20742)=(S=6125434,E=6125435,C=38)
	CT(20743)=(S=6125436,E=6125437,C=14)
	CT(20744)=(S=6125438,E=6125467,C=38)
	CT(20745)=(S=6125468,E=6125471,C=25)
	CT(20746)=(S=6125472,E=6125535,C=38)
	CT(20747)=(S=6125536,E=6125551,C=13)
	CT(20748)=(S=6125552,E=6125567,C=28)
	CT(20749)=(S=6125568,E=6125695,C=38)
	CT(20750)=(S=6125696,E=6125823,C=18)
	CT(20751)=(S=6125824,E=6125951,C=14)
	CT(20752)=(S=6125952,E=6126079,C=12)
	CT(20753)=(S=6126080,E=6126207,C=28)
	CT(20754)=(S=6126208,E=6126335,C=38)
	CT(20755)=(S=6126336,E=6126463,C=51)
	CT(20756)=(S=6126464,E=6126719,C=18)
	CT(20757)=(S=6126720,E=6126847,C=27)
	CT(20758)=(S=6126848,E=6126975,C=68)
	CT(20759)=(S=6126976,E=6127103,C=36)
	CT(20760)=(S=6127104,E=6127167,C=26)
	CT(20761)=(S=6127168,E=6127231,C=31)
	CT(20762)=(S=6127232,E=6127359,C=39)
	CT(20763)=(S=6127360,E=6127487,C=31)
	CT(20764)=(S=6127488,E=6129663,C=28)
	CT(20765)=(S=6129664,E=6131711,C=57)
	CT(20766)=(S=6131712,E=6133759,C=15)
	CT(20767)=(S=6133760,E=6133887,C=14)
	CT(20768)=(S=6133888,E=6134015,C=51)
	CT(20769)=(S=6134016,E=6134143,C=33)
	CT(20770)=(S=6134144,E=6134271,C=18)
	CT(20771)=(S=6134272,E=6134399,C=36)
	CT(20772)=(S=6134400,E=6134527,C=58)
	CT(20773)=(S=6134528,E=6134655,C=37)
	CT(20774)=(S=6134656,E=6134783,C=51)
	CT(20775)=(S=6134784,E=6135039,C=25)
	CT(20776)=(S=6135040,E=6135047,C=36)
	CT(20777)=(S=6135048,E=6135055,C=31)
	CT(20778)=(S=6135056,E=6135063,C=18)
	CT(20779)=(S=6135064,E=6135071,C=31)
	CT(20780)=(S=6135072,E=6135079,C=34)
	CT(20781)=(S=6135080,E=6135087,C=18)
	CT(20782)=(S=6135088,E=6135095,C=28)
	CT(20783)=(S=6135096,E=6135103,C=18)
	CT(20784)=(S=6135104,E=6135119,C=58)
	CT(20785)=(S=6135120,E=6135127,C=17)
	CT(20786)=(S=6135128,E=6135159,C=58)
	CT(20787)=(S=6135160,E=6135167,C=18)
	CT(20788)=(S=6135168,E=6135175,C=33)
	CT(20789)=(S=6135176,E=6135183,C=36)
	CT(20790)=(S=6135184,E=6135191,C=18)
	CT(20791)=(S=6135192,E=6135199,C=58)
	CT(20792)=(S=6135200,E=6135223,C=18)
	CT(20793)=(S=6135232,E=6135239,C=59)
	CT(20794)=(S=6135240,E=6135255,C=18)
	CT(20795)=(S=6135256,E=6135263,C=14)
	CT(20796)=(S=6135264,E=6135295,C=18)
	CT(20797)=(S=6135296,E=6135359,C=12)
	CT(20798)=(S=6135360,E=6135423,C=19)
	CT(20799)=(S=6135424,E=6135495,C=18)
	CT(20800)=(S=6135496,E=6135519,C=36)
	CT(20801)=(S=6135520,E=6135527,C=18)
	CT(20802)=(S=6135528,E=6135531,C=58)
	CT(20803)=(S=6135532,E=6135535,C=189)
	CT(20804)=(S=6135536,E=6135551,C=18)
	CT(20805)=(S=6135552,E=6135615,C=58)
	CT(20806)=(S=6135616,E=6135647,C=57)
	CT(20807)=(S=6135648,E=6135679,C=28)
	CT(20808)=(S=6135680,E=6135743,C=58)
	CT(20809)=(S=6135744,E=6135751,C=96)
	CT(20810)=(S=6135752,E=6135759,C=14)
	CT(20811)=(S=6135760,E=6135767,C=25)
	CT(20812)=(S=6135768,E=6135799,C=18)
	CT(20813)=(S=6135800,E=6135807,C=28)
	CT(20814)=(S=6135808,E=6137855,C=24)
	CT(20815)=(S=6137856,E=6138367,C=23)
	CT(20816)=(S=6138368,E=6138879,C=33)
	CT(20817)=(S=6138880,E=6139391,C=17)
	CT(20818)=(S=6139392,E=6139399,C=25)
	CT(20819)=(S=6139400,E=6139407,C=14)
	CT(20820)=(S=6139408,E=6139415,C=34)
	CT(20821)=(S=6139416,E=6139423,C=58)
	CT(20822)=(S=6139424,E=6139431,C=12)
	CT(20823)=(S=6139432,E=6139439,C=70)
	CT(20824)=(S=6139440,E=6139455,C=18)
	CT(20825)=(S=6139456,E=6139463,C=15)
	CT(20826)=(S=6139464,E=6139471,C=18)
	CT(20827)=(S=6139472,E=6139479,C=14)
	CT(20828)=(S=6139480,E=6139487,C=36)
	CT(20829)=(S=6139488,E=6139495,C=14)
	CT(20830)=(S=6139496,E=6139503,C=36)
	CT(20831)=(S=6139504,E=6139511,C=18)
	CT(20832)=(S=6139512,E=6139519,C=22)
	CT(20833)=(S=6139520,E=6139527,C=18)
	CT(20834)=(S=6139528,E=6139535,C=14)
	CT(20835)=(S=6139536,E=6139543,C=12)
	CT(20836)=(S=6139544,E=6139551,C=14)
	CT(20837)=(S=6139552,E=6139559,C=38)
	CT(20838)=(S=6139560,E=6139567,C=63)
	CT(20839)=(S=6139568,E=6139575,C=71)
	CT(20840)=(S=6139576,E=6139583,C=34)
	CT(20841)=(S=6139584,E=6139591,C=47)
	CT(20842)=(S=6139592,E=6139599,C=36)
	CT(20843)=(S=6139600,E=6139607,C=13)
	CT(20844)=(S=6139608,E=6139615,C=14)
	CT(20845)=(S=6139616,E=6139623,C=18)
	CT(20846)=(S=6139624,E=6139631,C=14)
	CT(20847)=(S=6139632,E=6139639,C=18)
	CT(20848)=(S=6139640,E=6139647,C=14)
	CT(20849)=(S=6139648,E=6139679,C=18)
	CT(20850)=(S=6139680,E=6139711,C=17)
	CT(20851)=(S=6139712,E=6139791,C=58)
	CT(20852)=(S=6139792,E=6139807,C=13)
	CT(20853)=(S=6139808,E=6139823,C=58)
	CT(20854)=(S=6139824,E=6139839,C=36)
	CT(20855)=(S=6139840,E=6139903,C=31)
	CT(20856)=(S=6139904,E=6139967,C=12)
	CT(20857)=(S=6139968,E=6140031,C=24)
	CT(20858)=(S=6140032,E=6140095,C=18)
	CT(20859)=(S=6140096,E=6140159,C=58)
	CT(20860)=(S=6140160,E=6140223,C=18)
	CT(20861)=(S=6140288,E=6140351,C=40)
	CT(20862)=(S=6140352,E=6140415,C=60)
	CT(20863)=(S=6140416,E=6140479,C=23)
	CT(20864)=(S=6140480,E=6140543,C=18)
	CT(20865)=(S=6140544,E=6140607,C=24)
	CT(20866)=(S=6140608,E=6140671,C=31)
	CT(20867)=(S=6140672,E=6140735,C=19)
	CT(20868)=(S=6140736,E=6140799,C=18)
	CT(20869)=(S=6140800,E=6140863,C=15)
	CT(20870)=(S=6140864,E=6140927,C=58)
	CT(20871)=(S=6140928,E=6140991,C=18)
	CT(20872)=(S=6140992,E=6140999,C=36)
	CT(20873)=(S=6141000,E=6141007,C=28)
	CT(20874)=(S=6141008,E=6141015,C=24)
	CT(20875)=(S=6141016,E=6141023,C=34)
	CT(20876)=(S=6141024,E=6141055,C=59)
	CT(20877)=(S=6141056,E=6141063,C=14)
	CT(20878)=(S=6141064,E=6141071,C=18)
	CT(20879)=(S=6141072,E=6141079,C=59)
	CT(20880)=(S=6141080,E=6141087,C=28)
	CT(20881)=(S=6141088,E=6141119,C=58)
	CT(20882)=(S=6141120,E=6141183,C=31)
	CT(20883)=(S=6141184,E=6141247,C=28)
	CT(20884)=(S=6141248,E=6141311,C=33)
	CT(20885)=(S=6141312,E=6141375,C=58)
	CT(20886)=(S=6141376,E=6141503,C=18)
	CT(20887)=(S=6141504,E=6141567,C=37)
	CT(20888)=(S=6141568,E=6141631,C=19)
	CT(20889)=(S=6141632,E=6141695,C=12)
	CT(20890)=(S=6141696,E=6141759,C=29)
	CT(20891)=(S=6141760,E=6141823,C=18)
	CT(20892)=(S=6141824,E=6141887,C=51)
	CT(20893)=(S=6141888,E=6141951,C=31)
	CT(20894)=(S=6141952,E=6141967,C=43)
	CT(20895)=(S=6141968,E=6141983,C=34)
	CT(20896)=(S=6141984,E=6141999,C=12)
	CT(20897)=(S=6142000,E=6142015,C=28)
	CT(20898)=(S=6142016,E=6142031,C=71)
	CT(20899)=(S=6142032,E=6142047,C=46)
	CT(20900)=(S=6142048,E=6142063,C=36)
	CT(20901)=(S=6142064,E=6142079,C=27)
	CT(20902)=(S=6142080,E=6142095,C=28)
	CT(20903)=(S=6142096,E=6142111,C=37)
	CT(20904)=(S=6142112,E=6142127,C=18)
	CT(20905)=(S=6142128,E=6142143,C=28)
	CT(20906)=(S=6142144,E=6142159,C=24)
	CT(20907)=(S=6142160,E=6142175,C=13)
	CT(20908)=(S=6142176,E=6142191,C=41)
	CT(20909)=(S=6142192,E=6142207,C=14)
	CT(20910)=(S=6142208,E=6142223,C=33)
	CT(20911)=(S=6142224,E=6142239,C=18)
	CT(20912)=(S=6142240,E=6142255,C=66)
	CT(20913)=(S=6142256,E=6142271,C=33)
	CT(20914)=(S=6142272,E=6142287,C=20)
	CT(20915)=(S=6142288,E=6142303,C=39)
	CT(20916)=(S=6142304,E=6142319,C=33)
	CT(20917)=(S=6142320,E=6142335,C=15)
	CT(20918)=(S=6142336,E=6142351,C=42)
	CT(20919)=(S=6142352,E=6142415,C=18)
	CT(20920)=(S=6142416,E=6142431,C=31)
	CT(20921)=(S=6142432,E=6142447,C=39)
	CT(20922)=(S=6142448,E=6142463,C=55)
	CT(20923)=(S=6142464,E=6142479,C=28)
	CT(20924)=(S=6142480,E=6142511,C=14)
	CT(20925)=(S=6142512,E=6142527,C=18)
	CT(20926)=(S=6142528,E=6142543,C=46)
	CT(20927)=(S=6142544,E=6142559,C=15)
	CT(20928)=(S=6142560,E=6142575,C=18)
	CT(20929)=(S=6142576,E=6142591,C=37)
	CT(20930)=(S=6142592,E=6142607,C=15)
	CT(20931)=(S=6142608,E=6142623,C=14)
	CT(20932)=(S=6142624,E=6142639,C=28)
	CT(20933)=(S=6142640,E=6142655,C=36)
	CT(20934)=(S=6142656,E=6142671,C=28)
	CT(20935)=(S=6142672,E=6142687,C=40)
	CT(20936)=(S=6142688,E=6142703,C=18)
	CT(20937)=(S=6142704,E=6142719,C=15)
	CT(20938)=(S=6142720,E=6142727,C=14)
	CT(20939)=(S=6142728,E=6142735,C=36)
	CT(20940)=(S=6142736,E=6142743,C=14)
	CT(20941)=(S=6142744,E=6142751,C=15)
	CT(20942)=(S=6142752,E=6142759,C=35)
	CT(20943)=(S=6142760,E=6142767,C=12)
	CT(20944)=(S=6142768,E=6142775,C=36)
	CT(20945)=(S=6142776,E=6142783,C=28)
	CT(20946)=(S=6142784,E=6142791,C=37)
	CT(20947)=(S=6142792,E=6142799,C=18)
	CT(20948)=(S=6142800,E=6142807,C=27)
	CT(20949)=(S=6142808,E=6142815,C=15)
	CT(20950)=(S=6142816,E=6142823,C=18)
	CT(20951)=(S=6142824,E=6142831,C=33)
	CT(20952)=(S=6142832,E=6142839,C=28)
	CT(20953)=(S=6142840,E=6142847,C=18)
	CT(20954)=(S=6142856,E=6142863,C=51)
	CT(20955)=(S=6142864,E=6142871,C=14)
	CT(20956)=(S=6142872,E=6142879,C=18)
	CT(20957)=(S=6142880,E=6142887,C=66)
	CT(20958)=(S=6142888,E=6142895,C=14)
	CT(20959)=(S=6142896,E=6142911,C=18)
	CT(20960)=(S=6142912,E=6142919,C=34)
	CT(20961)=(S=6142920,E=6142927,C=37)
	CT(20962)=(S=6142928,E=6142935,C=34)
	CT(20963)=(S=6142936,E=6142943,C=36)
	CT(20964)=(S=6142944,E=6142951,C=14)
	CT(20965)=(S=6142952,E=6142959,C=28)
	CT(20966)=(S=6142960,E=6142967,C=46)
	CT(20967)=(S=6142968,E=6142975,C=28)
	CT(20968)=(S=6142976,E=6142983,C=19)
	CT(20969)=(S=6142984,E=6142991,C=40)
	CT(20970)=(S=6142992,E=6142999,C=18)
	CT(20971)=(S=6143000,E=6143007,C=28)
	CT(20972)=(S=6143008,E=6143015,C=31)
	CT(20973)=(S=6143016,E=6143023,C=18)
	CT(20974)=(S=6143024,E=6143031,C=25)
	CT(20975)=(S=6143040,E=6143047,C=36)
	CT(20976)=(S=6143048,E=6143055,C=34)
	CT(20977)=(S=6143056,E=6143063,C=18)
	CT(20978)=(S=6143064,E=6143071,C=27)
	CT(20979)=(S=6143072,E=6143079,C=15)
	CT(20980)=(S=6143080,E=6143087,C=28)
	CT(20981)=(S=6143088,E=6143095,C=15)
	CT(20982)=(S=6143096,E=6143103,C=18)
	CT(20983)=(S=6143104,E=6143111,C=29)
	CT(20984)=(S=6143112,E=6143119,C=25)
	CT(20985)=(S=6143120,E=6143127,C=12)
	CT(20986)=(S=6143128,E=6143135,C=18)
	CT(20987)=(S=6143136,E=6143143,C=44)
	CT(20988)=(S=6143144,E=6143151,C=12)
	CT(20989)=(S=6143152,E=6143159,C=14)
	CT(20990)=(S=6143160,E=6143167,C=18)
	CT(20991)=(S=6143168,E=6143175,C=104)
	CT(20992)=(S=6143176,E=6143191,C=18)
	CT(20993)=(S=6143192,E=6143199,C=28)
	CT(20994)=(S=6143200,E=6143207,C=15)
	CT(20995)=(S=6143208,E=6143215,C=27)
	CT(20996)=(S=6143216,E=6143223,C=28)
	CT(20997)=(S=6143224,E=6143231,C=36)
	CT(20998)=(S=6143232,E=6143239,C=14)
	CT(20999)=(S=6143240,E=6143255,C=18)
	CT(21000)=(S=6143256,E=6143263,C=42)
	CT(21001)=(S=6143264,E=6143271,C=25)
	CT(21002)=(S=6143272,E=6143295,C=18)
	CT(21003)=(S=6143296,E=6143303,C=34)
	CT(21004)=(S=6143304,E=6143319,C=18)
	CT(21005)=(S=6143320,E=6143327,C=25)
	CT(21006)=(S=6143328,E=6143335,C=23)
	CT(21007)=(S=6143336,E=6143343,C=14)
	CT(21008)=(S=6143344,E=6143351,C=50)
	CT(21009)=(S=6143360,E=6143367,C=36)
	CT(21010)=(S=6143368,E=6143375,C=15)
	CT(21011)=(S=6143376,E=6143383,C=18)
	CT(21012)=(S=6143384,E=6143391,C=28)
	CT(21013)=(S=6143392,E=6143399,C=14)
	CT(21014)=(S=6143400,E=6143407,C=28)
	CT(21015)=(S=6143408,E=6143415,C=25)
	CT(21016)=(S=6143416,E=6143423,C=15)
	CT(21017)=(S=6143424,E=6143431,C=23)
	CT(21018)=(S=6143432,E=6143439,C=18)
	CT(21019)=(S=6143440,E=6143447,C=15)
	CT(21020)=(S=6143448,E=6143455,C=18)
	CT(21021)=(S=6143456,E=6143463,C=68)
	CT(21022)=(S=6143464,E=6143471,C=34)
	CT(21023)=(S=6143472,E=6143479,C=19)
	CT(21024)=(S=6143480,E=6143487,C=18)
	CT(21025)=(S=6143488,E=6143495,C=50)
	CT(21026)=(S=6143504,E=6143511,C=18)
	CT(21027)=(S=6143512,E=6143519,C=26)
	CT(21028)=(S=6143528,E=6143535,C=31)
	CT(21029)=(S=6143536,E=6143551,C=33)
	CT(21030)=(S=6143552,E=6143559,C=28)
	CT(21031)=(S=6143560,E=6143567,C=34)
	CT(21032)=(S=6143568,E=6143575,C=27)
	CT(21033)=(S=6143576,E=6143583,C=28)
	CT(21034)=(S=6143584,E=6143591,C=47)
	CT(21035)=(S=6143592,E=6143599,C=18)
	CT(21036)=(S=6143600,E=6143607,C=14)
	CT(21037)=(S=6143608,E=6143615,C=37)
	CT(21038)=(S=6143616,E=6143623,C=33)
	CT(21039)=(S=6143624,E=6143631,C=36)
	CT(21040)=(S=6143632,E=6143639,C=38)
	CT(21041)=(S=6143640,E=6143647,C=25)
	CT(21042)=(S=6143648,E=6143655,C=46)
	CT(21043)=(S=6143656,E=6143663,C=14)
	CT(21044)=(S=6143664,E=6143671,C=31)
	CT(21045)=(S=6143672,E=6143679,C=36)
	CT(21046)=(S=6143680,E=6143687,C=19)
	CT(21047)=(S=6143688,E=6143695,C=18)
	CT(21048)=(S=6143696,E=6143703,C=12)
	CT(21049)=(S=6143704,E=6143711,C=37)
	CT(21050)=(S=6143712,E=6143719,C=18)
	CT(21051)=(S=6143720,E=6143727,C=28)
	CT(21052)=(S=6143728,E=6143735,C=18)
	CT(21053)=(S=6143736,E=6143743,C=28)
	CT(21054)=(S=6143744,E=6143751,C=36)
	CT(21055)=(S=6143752,E=6143767,C=18)
	CT(21056)=(S=6143768,E=6143783,C=14)
	CT(21057)=(S=6143784,E=6143791,C=12)
	CT(21058)=(S=6143792,E=6143799,C=28)
	CT(21059)=(S=6143800,E=6143807,C=18)
	CT(21060)=(S=6143808,E=6143815,C=101)
	CT(21061)=(S=6143816,E=6143823,C=18)
	CT(21062)=(S=6143824,E=6143831,C=12)
	CT(21063)=(S=6143832,E=6143839,C=34)
	CT(21064)=(S=6143840,E=6143847,C=68)
	CT(21065)=(S=6143848,E=6143855,C=18)
	CT(21066)=(S=6143864,E=6143871,C=47)
	CT(21067)=(S=6143872,E=6143879,C=36)
	CT(21068)=(S=6143880,E=6143887,C=25)
	CT(21069)=(S=6143888,E=6143895,C=63)
	CT(21070)=(S=6143896,E=6143903,C=24)
	CT(21071)=(S=6143904,E=6143911,C=28)
	CT(21072)=(S=6143912,E=6143919,C=13)
	CT(21073)=(S=6143920,E=6143927,C=61)
	CT(21074)=(S=6143928,E=6143935,C=12)
	CT(21075)=(S=6143936,E=6143943,C=69)
	CT(21076)=(S=6143944,E=6143947,C=99)
	CT(21077)=(S=6143948,E=6143951,C=14)
	CT(21078)=(S=6143952,E=6143959,C=28)
	CT(21079)=(S=6143960,E=6143967,C=13)
	CT(21080)=(S=6143968,E=6143975,C=20)
	CT(21081)=(S=6143976,E=6143983,C=19)
	CT(21082)=(S=6143984,E=6143991,C=15)
	CT(21083)=(S=6143992,E=6143999,C=34)
	CT(21084)=(S=6144000,E=6160383,C=28)
	CT(21085)=(S=6160384,E=6164479,C=14)
	CT(21086)=(S=6164480,E=6164607,C=28)
	CT(21087)=(S=6164608,E=6164735,C=34)
	CT(21088)=(S=6164736,E=6164991,C=72)
	CT(21089)=(S=6164992,E=6165247,C=24)
	CT(21090)=(S=6165248,E=6165503,C=18)
	CT(21091)=(S=6165504,E=6165759,C=41)
	CT(21092)=(S=6165760,E=6166015,C=50)
	CT(21093)=(S=6166016,E=6166271,C=47)
	CT(21094)=(S=6166272,E=6166527,C=12)
	CT(21095)=(S=6166528,E=6166551,C=26)
	CT(21096)=(S=6166552,E=6166607,C=38)
	CT(21097)=(S=6166608,E=6166631,C=26)
	CT(21098)=(S=6166632,E=6166639,C=38)
	CT(21099)=(S=6166640,E=6166655,C=25)
	CT(21100)=(S=6166656,E=6167039,C=18)
	CT(21101)=(S=6167040,E=6167167,C=51)
	CT(21102)=(S=6167168,E=6167295,C=18)
	CT(21103)=(S=6167296,E=6167423,C=31)
	CT(21104)=(S=6167424,E=6167551,C=50)
	CT(21105)=(S=6167552,E=6167935,C=18)
	CT(21106)=(S=6167936,E=6168063,C=101)
	CT(21107)=(S=6168064,E=6168191,C=14)
	CT(21108)=(S=6168192,E=6168319,C=60)
	CT(21109)=(S=6168320,E=6168383,C=14)
	CT(21110)=(S=6168384,E=6168447,C=28)
	CT(21111)=(S=6168448,E=6168575,C=18)
	CT(21112)=(S=6168576,E=6170623,C=15)
	CT(21113)=(S=6170624,E=6170879,C=58)
	CT(21114)=(S=6170880,E=6171135,C=18)
	CT(21115)=(S=6171136,E=6171391,C=58)
	CT(21116)=(S=6171392,E=6171647,C=40)
	CT(21117)=(S=6171648,E=6171903,C=50)
	CT(21118)=(S=6171904,E=6171935,C=18)
	CT(21119)=(S=6171936,E=6172063,C=31)
	CT(21120)=(S=6172064,E=6172127,C=18)
	CT(21121)=(S=6172128,E=6172159,C=28)
	CT(21122)=(S=6172160,E=6172167,C=21)
	CT(21123)=(S=6172168,E=6172171,C=19)
	CT(21124)=(S=6172172,E=6172199,C=21)
	CT(21125)=(S=6172200,E=6172223,C=19)
	CT(21126)=(S=6172224,E=6172255,C=21)
	CT(21127)=(S=6172256,E=6172271,C=14)
	CT(21128)=(S=6172272,E=6172287,C=19)
	CT(21129)=(S=6172288,E=6172295,C=21)
	CT(21130)=(S=6172296,E=6172300,C=19)
	CT(21131)=(S=6172301,E=6172302,C=21)
	CT(21132)=(S=6172303,E=6172303,C=19)
	CT(21133)=(S=6172304,E=6172319,C=36)
	CT(21134)=(S=6172320,E=6172343,C=21)
	CT(21135)=(S=6172344,E=6172347,C=14)
	CT(21136)=(S=6172348,E=6172351,C=21)
	CT(21137)=(S=6172352,E=6172355,C=14)
	CT(21138)=(S=6172356,E=6172366,C=21)
	CT(21139)=(S=6172367,E=6172367,C=14)
	CT(21140)=(S=6172368,E=6172379,C=21)
	CT(21141)=(S=6172380,E=6172380,C=29)
	CT(21142)=(S=6172381,E=6172381,C=14)
	CT(21143)=(S=6172382,E=6172383,C=29)
	CT(21144)=(S=6172384,E=6172399,C=21)
	CT(21145)=(S=6172400,E=6172403,C=19)
	CT(21146)=(S=6172404,E=6172407,C=14)
	CT(21147)=(S=6172408,E=6172410,C=19)
	CT(21148)=(S=6172411,E=6172411,C=21)
	CT(21149)=(S=6172412,E=6172415,C=19)
	CT(21150)=(S=6172416,E=6172671,C=15)
	CT(21151)=(S=6172672,E=6173183,C=23)
	CT(21152)=(S=6173184,E=6173695,C=18)
	CT(21153)=(S=6173696,E=6174207,C=38)
	CT(21154)=(S=6174208,E=6174719,C=37)
	CT(21155)=(S=6174720,E=6175743,C=16)
	CT(21156)=(S=6175744,E=6176767,C=21)
	CT(21157)=(S=6176768,E=6178815,C=22)
	CT(21158)=(S=6178816,E=6178879,C=18)
	CT(21159)=(S=6178880,E=6178943,C=58)
	CT(21160)=(S=6178944,E=6179007,C=51)
	CT(21161)=(S=6179008,E=6179071,C=14)
	CT(21162)=(S=6179072,E=6179135,C=51)
	CT(21163)=(S=6179136,E=6179199,C=15)
	CT(21164)=(S=6179200,E=6179263,C=37)
	CT(21165)=(S=6179264,E=6179327,C=18)
	CT(21166)=(S=6179328,E=6179391,C=15)
	CT(21167)=(S=6179392,E=6179455,C=31)
	CT(21168)=(S=6179456,E=6179519,C=26)
	CT(21169)=(S=6179520,E=6179583,C=33)
	CT(21170)=(S=6179584,E=6179647,C=18)
	CT(21171)=(S=6179648,E=6179711,C=58)
	CT(21172)=(S=6179712,E=6179775,C=18)
	CT(21173)=(S=6179776,E=6179839,C=36)
	CT(21174)=(S=6179840,E=6179903,C=104)
	CT(21175)=(S=6179904,E=6179967,C=31)
	CT(21176)=(S=6179968,E=6180031,C=25)
	CT(21177)=(S=6180032,E=6180095,C=14)
	CT(21178)=(S=6180096,E=6180287,C=18)
	CT(21179)=(S=6180288,E=6180351,C=23)
	CT(21180)=(S=6180352,E=6180415,C=34)
	CT(21181)=(S=6180416,E=6180479,C=37)
	CT(21182)=(S=6180480,E=6180543,C=58)
	CT(21183)=(S=6180544,E=6180671,C=18)
	CT(21184)=(S=6180672,E=6180735,C=37)
	CT(21185)=(S=6180736,E=6180799,C=28)
	CT(21186)=(S=6180800,E=6180863,C=104)
	CT(21187)=(S=6180864,E=6184959,C=15)
	CT(21188)=(S=6184960,E=6185983,C=23)
	CT(21189)=(S=6185984,E=6185999,C=60)
	CT(21190)=(S=6186000,E=6186015,C=36)
	CT(21191)=(S=6186016,E=6186031,C=15)
	CT(21192)=(S=6186032,E=6186047,C=46)
	CT(21193)=(S=6186048,E=6186063,C=28)
	CT(21194)=(S=6186064,E=6186079,C=18)
	CT(21195)=(S=6186080,E=6186095,C=54)
	CT(21196)=(S=6186096,E=6186111,C=36)
	CT(21197)=(S=6186112,E=6186127,C=28)
	CT(21198)=(S=6186128,E=6186143,C=34)
	CT(21199)=(S=6186144,E=6186159,C=12)
	CT(21200)=(S=6186160,E=6186191,C=18)
	CT(21201)=(S=6186192,E=6186207,C=31)
	CT(21202)=(S=6186208,E=6186223,C=40)
	CT(21203)=(S=6186224,E=6186239,C=28)
	CT(21204)=(S=6186240,E=6186255,C=47)
	CT(21205)=(S=6186256,E=6186271,C=58)
	CT(21206)=(S=6186272,E=6186287,C=28)
	CT(21207)=(S=6186288,E=6186303,C=15)
	CT(21208)=(S=6186304,E=6186319,C=18)
	CT(21209)=(S=6186320,E=6186335,C=37)
	CT(21210)=(S=6186336,E=6186351,C=18)
	CT(21211)=(S=6186368,E=6186383,C=26)
	CT(21212)=(S=6186384,E=6186415,C=14)
	CT(21213)=(S=6186416,E=6186431,C=26)
	CT(21214)=(S=6186432,E=6186447,C=51)
	CT(21215)=(S=6186448,E=6186463,C=24)
	CT(21216)=(S=6186464,E=6186479,C=60)
	CT(21217)=(S=6186480,E=6186495,C=26)
	CT(21218)=(S=6186496,E=6186511,C=37)
	CT(21219)=(S=6186512,E=6186527,C=18)
	CT(21220)=(S=6186528,E=6186543,C=27)
	CT(21221)=(S=6186544,E=6186559,C=36)
	CT(21222)=(S=6186560,E=6186575,C=37)
	CT(21223)=(S=6186576,E=6186591,C=18)
	CT(21224)=(S=6186592,E=6186607,C=42)
	CT(21225)=(S=6186608,E=6186623,C=18)
	CT(21226)=(S=6186624,E=6186639,C=15)
	CT(21227)=(S=6186640,E=6186655,C=14)
	CT(21228)=(S=6186656,E=6186671,C=13)
	CT(21229)=(S=6186672,E=6186687,C=190)
	CT(21230)=(S=6186688,E=6186703,C=63)
	CT(21231)=(S=6186704,E=6186719,C=28)
	CT(21232)=(S=6186720,E=6186735,C=102)
	CT(21233)=(S=6186736,E=6186751,C=14)
	CT(21234)=(S=6186752,E=6186767,C=53)
	CT(21235)=(S=6186768,E=6186783,C=36)
	CT(21236)=(S=6186784,E=6186799,C=37)
	CT(21237)=(S=6186800,E=6186815,C=60)
	CT(21238)=(S=6186816,E=6186831,C=52)
	CT(21239)=(S=6186832,E=6186847,C=18)
	CT(21240)=(S=6186848,E=6186863,C=34)
	CT(21241)=(S=6186880,E=6186895,C=12)
	CT(21242)=(S=6186896,E=6186911,C=36)
	CT(21243)=(S=6186912,E=6186927,C=28)
	CT(21244)=(S=6186928,E=6186943,C=18)
	CT(21245)=(S=6186944,E=6186959,C=19)
	CT(21246)=(S=6186960,E=6186975,C=70)
	CT(21247)=(S=6186976,E=6186991,C=18)
	CT(21248)=(S=6186992,E=6187007,C=20)
	CT(21249)=(S=6187008,E=6189055,C=49)
	CT(21250)=(S=6189056,E=6190079,C=33)
	CT(21251)=(S=6190080,E=6191103,C=14)
	CT(21252)=(S=6191104,E=6192127,C=37)
	CT(21253)=(S=6192128,E=6192151,C=58)
	CT(21254)=(S=6192152,E=6192159,C=25)
	CT(21255)=(S=6192160,E=6192167,C=15)
	CT(21256)=(S=6192168,E=6192175,C=33)
	CT(21257)=(S=6192176,E=6192183,C=15)
	CT(21258)=(S=6192184,E=6192191,C=36)
	CT(21259)=(S=6192192,E=6192207,C=15)
	CT(21260)=(S=6192208,E=6192215,C=12)
	CT(21261)=(S=6192216,E=6192223,C=36)
	CT(21262)=(S=6192232,E=6192239,C=33)
	CT(21263)=(S=6192240,E=6192255,C=36)
	CT(21264)=(S=6192256,E=6192263,C=12)
	CT(21265)=(S=6192264,E=6192271,C=36)
	CT(21266)=(S=6192272,E=6192279,C=71)
	CT(21267)=(S=6192280,E=6192287,C=12)
	CT(21268)=(S=6192288,E=6192295,C=31)
	CT(21269)=(S=6192296,E=6192311,C=18)
	CT(21270)=(S=6192312,E=6192319,C=33)
	CT(21271)=(S=6192320,E=6192335,C=18)
	CT(21272)=(S=6192336,E=6192343,C=28)
	CT(21273)=(S=6192344,E=6192351,C=191)
	CT(21274)=(S=6192352,E=6192359,C=18)
	CT(21275)=(S=6192360,E=6192367,C=12)
	CT(21276)=(S=6192368,E=6192375,C=18)
	CT(21277)=(S=6192376,E=6192383,C=33)
	CT(21278)=(S=6192384,E=6192399,C=18)
	CT(21279)=(S=6192400,E=6192407,C=14)
	CT(21280)=(S=6192408,E=6192415,C=28)
	CT(21281)=(S=6192416,E=6192423,C=14)
	CT(21282)=(S=6192424,E=6192439,C=18)
	CT(21283)=(S=6192440,E=6192447,C=64)
	CT(21284)=(S=6192448,E=6192463,C=28)
	CT(21285)=(S=6192464,E=6192471,C=15)
	CT(21286)=(S=6192472,E=6192479,C=18)
	CT(21287)=(S=6192488,E=6192495,C=12)
	CT(21288)=(S=6192496,E=6192503,C=96)
	CT(21289)=(S=6192504,E=6192511,C=31)
	CT(21290)=(S=6192512,E=6192519,C=14)
	CT(21291)=(S=6192520,E=6192527,C=25)
	CT(21292)=(S=6192528,E=6192535,C=32)
	CT(21293)=(S=6192536,E=6192543,C=28)
	CT(21294)=(S=6192544,E=6192559,C=12)
	CT(21295)=(S=6192560,E=6192567,C=50)
	CT(21296)=(S=6192568,E=6192575,C=18)
	CT(21297)=(S=6192576,E=6192583,C=14)
	CT(21298)=(S=6192584,E=6192591,C=70)
	CT(21299)=(S=6192592,E=6192599,C=28)
	CT(21300)=(S=6192600,E=6192607,C=33)
	CT(21301)=(S=6192608,E=6192615,C=94)
	CT(21302)=(S=6192616,E=6192623,C=15)
	CT(21303)=(S=6192624,E=6192631,C=18)
	CT(21304)=(S=6192632,E=6192639,C=50)
	CT(21305)=(S=6192640,E=6192647,C=24)
	CT(21306)=(S=6192648,E=6192655,C=15)
	CT(21307)=(S=6192656,E=6192663,C=34)
	CT(21308)=(S=6192664,E=6192671,C=18)
	CT(21309)=(S=6192672,E=6192679,C=36)
	CT(21310)=(S=6192680,E=6192687,C=14)
	CT(21311)=(S=6192688,E=6192695,C=49)
	CT(21312)=(S=6192696,E=6192703,C=15)
	CT(21313)=(S=6192704,E=6192711,C=36)
	CT(21314)=(S=6192712,E=6192719,C=28)
	CT(21315)=(S=6192720,E=6192727,C=19)
	CT(21316)=(S=6192728,E=6192735,C=18)
	CT(21317)=(S=6192736,E=6192743,C=42)
	CT(21318)=(S=6192744,E=6192751,C=14)
	CT(21319)=(S=6192752,E=6192759,C=12)
	CT(21320)=(S=6192760,E=6192767,C=14)
	CT(21321)=(S=6192768,E=6192775,C=34)
	CT(21322)=(S=6192776,E=6192783,C=15)
	CT(21323)=(S=6192784,E=6192791,C=21)
	CT(21324)=(S=6192792,E=6192799,C=18)
	CT(21325)=(S=6192800,E=6192807,C=47)
	CT(21326)=(S=6192808,E=6192815,C=21)
	CT(21327)=(S=6192816,E=6192823,C=24)
	CT(21328)=(S=6192824,E=6192831,C=15)
	CT(21329)=(S=6192832,E=6192839,C=14)
	CT(21330)=(S=6192840,E=6192847,C=18)
	CT(21331)=(S=6192848,E=6192855,C=14)
	CT(21332)=(S=6192856,E=6192871,C=19)
	CT(21333)=(S=6192872,E=6192879,C=14)
	CT(21334)=(S=6192880,E=6192887,C=25)
	CT(21335)=(S=6192888,E=6192895,C=34)
	CT(21336)=(S=6192896,E=6192903,C=46)
	CT(21337)=(S=6192904,E=6192911,C=12)
	CT(21338)=(S=6192912,E=6192919,C=28)
	CT(21339)=(S=6192920,E=6192927,C=52)
	CT(21340)=(S=6192928,E=6192935,C=19)
	CT(21341)=(S=6192936,E=6192943,C=15)
	CT(21342)=(S=6192944,E=6192951,C=24)
	CT(21343)=(S=6192952,E=6192959,C=27)
	CT(21344)=(S=6192960,E=6192967,C=18)
	CT(21345)=(S=6192968,E=6192975,C=13)
	CT(21346)=(S=6192976,E=6192983,C=15)
	CT(21347)=(S=6192984,E=6192991,C=18)
	CT(21348)=(S=6192992,E=6192999,C=14)
	CT(21349)=(S=6193000,E=6193007,C=58)
	CT(21350)=(S=6193008,E=6193015,C=14)
	CT(21351)=(S=6193016,E=6193023,C=27)
	CT(21352)=(S=6193024,E=6193031,C=33)
	CT(21353)=(S=6193032,E=6193055,C=18)
	CT(21354)=(S=6193056,E=6193063,C=25)
	CT(21355)=(S=6193064,E=6193079,C=18)
	CT(21356)=(S=6193080,E=6193095,C=25)
	CT(21357)=(S=6193096,E=6193103,C=18)
	CT(21358)=(S=6193104,E=6193111,C=61)
	CT(21359)=(S=6193112,E=6193119,C=18)
	CT(21360)=(S=6193120,E=6193127,C=28)
	CT(21361)=(S=6193128,E=6193135,C=15)
	CT(21362)=(S=6193136,E=6193151,C=18)
	CT(21363)=(S=6193152,E=6193663,C=101)
	CT(21364)=(S=6193664,E=6194175,C=31)
	CT(21365)=(S=6194176,E=6194687,C=21)
	CT(21366)=(S=6194688,E=6195199,C=28)
	CT(21367)=(S=6195200,E=6195231,C=42)
	CT(21368)=(S=6195232,E=6195263,C=14)
	CT(21369)=(S=6195264,E=6195295,C=19)
	CT(21370)=(S=6195296,E=6195327,C=42)
	CT(21371)=(S=6195328,E=6195359,C=71)
	CT(21372)=(S=6195360,E=6195391,C=28)
	CT(21373)=(S=6195392,E=6195423,C=18)
	CT(21374)=(S=6195424,E=6195455,C=14)
	CT(21375)=(S=6195456,E=6195551,C=18)
	CT(21376)=(S=6195552,E=6195583,C=19)
	CT(21377)=(S=6195584,E=6195599,C=24)
	CT(21378)=(S=6195600,E=6195615,C=28)
	CT(21379)=(S=6195616,E=6195647,C=40)
	CT(21380)=(S=6195648,E=6195743,C=18)
	CT(21381)=(S=6195744,E=6195775,C=15)
	CT(21382)=(S=6195776,E=6195807,C=24)
	CT(21383)=(S=6195808,E=6195839,C=33)
	CT(21384)=(S=6195840,E=6195871,C=18)
	CT(21385)=(S=6195872,E=6195903,C=15)
	CT(21386)=(S=6195904,E=6195935,C=37)
	CT(21387)=(S=6195936,E=6195967,C=72)
	CT(21388)=(S=6195968,E=6195999,C=28)
	CT(21389)=(S=6196000,E=6196031,C=49)
	CT(21390)=(S=6196032,E=6196063,C=27)
	CT(21391)=(S=6196064,E=6196095,C=18)
	CT(21392)=(S=6196096,E=6196127,C=53)
	CT(21393)=(S=6196128,E=6196159,C=26)
	CT(21394)=(S=6196160,E=6196191,C=51)
	CT(21395)=(S=6196192,E=6196223,C=18)
	CT(21396)=(S=6196224,E=6196255,C=12)
	CT(21397)=(S=6196256,E=6196287,C=19)
	CT(21398)=(S=6196288,E=6196319,C=52)
	CT(21399)=(S=6196320,E=6196351,C=13)
	CT(21400)=(S=6196352,E=6196383,C=18)
	CT(21401)=(S=6196384,E=6196415,C=49)
	CT(21402)=(S=6196416,E=6196479,C=18)
	CT(21403)=(S=6196480,E=6196511,C=15)
	CT(21404)=(S=6196512,E=6196543,C=18)
	CT(21405)=(S=6196544,E=6196575,C=98)
	CT(21406)=(S=6196576,E=6196607,C=18)
	CT(21407)=(S=6196608,E=6196639,C=58)
	CT(21408)=(S=6196640,E=6196671,C=18)
	CT(21409)=(S=6196672,E=6196703,C=30)
	CT(21410)=(S=6196704,E=6196767,C=18)
	CT(21411)=(S=6196768,E=6196799,C=61)
	CT(21412)=(S=6196800,E=6196831,C=14)
	CT(21413)=(S=6196832,E=6196863,C=25)
	CT(21414)=(S=6196872,E=6196879,C=18)
	CT(21415)=(S=6196880,E=6196887,C=15)
	CT(21416)=(S=6196888,E=6196895,C=48)
	CT(21417)=(S=6196896,E=6196903,C=42)
	CT(21418)=(S=6196904,E=6196927,C=14)
	CT(21419)=(S=6196928,E=6196935,C=15)
	CT(21420)=(S=6196936,E=6196943,C=25)
	CT(21421)=(S=6196944,E=6196951,C=36)
	CT(21422)=(S=6196952,E=6196959,C=28)
	CT(21423)=(S=6196960,E=6196967,C=19)
	CT(21424)=(S=6196968,E=6196975,C=33)
	CT(21425)=(S=6196976,E=6196983,C=36)
	CT(21426)=(S=6196984,E=6196991,C=12)
	CT(21427)=(S=6196992,E=6196999,C=34)
	CT(21428)=(S=6197000,E=6197007,C=24)
	CT(21429)=(S=6197008,E=6197015,C=12)
	CT(21430)=(S=6197016,E=6197023,C=25)
	CT(21431)=(S=6197024,E=6197055,C=18)
	CT(21432)=(S=6197056,E=6197063,C=27)
	CT(21433)=(S=6197064,E=6197071,C=25)
	CT(21434)=(S=6197072,E=6197079,C=12)
	CT(21435)=(S=6197080,E=6197087,C=34)
	CT(21436)=(S=6197088,E=6197095,C=15)
	CT(21437)=(S=6197096,E=6197103,C=14)
	CT(21438)=(S=6197104,E=6197111,C=12)
	CT(21439)=(S=6197112,E=6197119,C=14)
	CT(21440)=(S=6197120,E=6197127,C=18)
	CT(21441)=(S=6197128,E=6197135,C=25)
	CT(21442)=(S=6197136,E=6197143,C=18)
	CT(21443)=(S=6197144,E=6197151,C=15)
	CT(21444)=(S=6197160,E=6197167,C=33)
	CT(21445)=(S=6197168,E=6197175,C=22)
	CT(21446)=(S=6197176,E=6197183,C=49)
	CT(21447)=(S=6197184,E=6197191,C=62)
	CT(21448)=(S=6197192,E=6197199,C=25)
	CT(21449)=(S=6197200,E=6197207,C=36)
	CT(21450)=(S=6197208,E=6197215,C=12)
	CT(21451)=(S=6197216,E=6197223,C=23)
	CT(21452)=(S=6197224,E=6197231,C=33)
	CT(21453)=(S=6197232,E=6197239,C=18)
	CT(21454)=(S=6197240,E=6197247,C=14)
	CT(21455)=(S=6197248,E=6199295,C=24)
	CT(21456)=(S=6199296,E=6199551,C=58)
	CT(21457)=(S=6199552,E=6199807,C=31)
	CT(21458)=(S=6199808,E=6199823,C=18)
	CT(21459)=(S=6199824,E=6199839,C=58)
	CT(21460)=(S=6199840,E=6199855,C=31)
	CT(21461)=(S=6199856,E=6199863,C=28)
	CT(21462)=(S=6199864,E=6199871,C=58)
	CT(21463)=(S=6199872,E=6199903,C=18)
	CT(21464)=(S=6199904,E=6199911,C=58)
	CT(21465)=(S=6199912,E=6199919,C=46)
	CT(21466)=(S=6199920,E=6199935,C=31)
	CT(21467)=(S=6199936,E=6199999,C=18)
	CT(21468)=(S=6200000,E=6200063,C=31)
	CT(21469)=(S=6200064,E=6200575,C=51)
	CT(21470)=(S=6200576,E=6200831,C=36)
	CT(21471)=(S=6200832,E=6200847,C=18)
	CT(21472)=(S=6200864,E=6200879,C=31)
	CT(21473)=(S=6200880,E=6200895,C=98)
	CT(21474)=(S=6200896,E=6200927,C=31)
	CT(21475)=(S=6200928,E=6200959,C=18)
	CT(21476)=(S=6200960,E=6200975,C=58)
	CT(21477)=(S=6200976,E=6200991,C=31)
	CT(21478)=(S=6200992,E=6201055,C=18)
	CT(21479)=(S=6201056,E=6201071,C=58)
	CT(21480)=(S=6201088,E=6201215,C=18)
	CT(21481)=(S=6201216,E=6201343,C=17)
	CT(21482)=(S=6201344,E=6203391,C=15)
	CT(21483)=(S=6203392,E=6203647,C=36)
	CT(21484)=(S=6203648,E=6205439,C=14)
	CT(21485)=(S=6205440,E=6205447,C=38)
	CT(21486)=(S=6205448,E=6205479,C=26)
	CT(21487)=(S=6205480,E=6205487,C=38)
	CT(21488)=(S=6205488,E=6205503,C=26)
	CT(21489)=(S=6205504,E=6205519,C=38)
	CT(21490)=(S=6205520,E=6205535,C=26)
	CT(21491)=(S=6205536,E=6205545,C=38)
	CT(21492)=(S=6205546,E=6205546,C=44)
	CT(21493)=(S=6205547,E=6205547,C=95)
	CT(21494)=(S=6205548,E=6205575,C=38)
	CT(21495)=(S=6205576,E=6205583,C=25)
	CT(21496)=(S=6205584,E=6205607,C=38)
	CT(21497)=(S=6205608,E=6205611,C=13)
	CT(21498)=(S=6205612,E=6205615,C=38)
	CT(21499)=(S=6205616,E=6205619,C=25)
	CT(21500)=(S=6205620,E=6205630,C=38)
	CT(21501)=(S=6205631,E=6205631,C=14)
	CT(21502)=(S=6205632,E=6205663,C=38)
	CT(21503)=(S=6205664,E=6205671,C=19)
	CT(21504)=(S=6205672,E=6205679,C=38)
	CT(21505)=(S=6205680,E=6205695,C=28)
	CT(21506)=(S=6205696,E=6205699,C=38)
	CT(21507)=(S=6205700,E=6205701,C=13)
	CT(21508)=(S=6205702,E=6205707,C=38)
	CT(21509)=(S=6205708,E=6205708,C=36)
	CT(21510)=(S=6205709,E=6205719,C=38)
	CT(21511)=(S=6205720,E=6205721,C=95)
	CT(21512)=(S=6205722,E=6205767,C=38)
	CT(21513)=(S=6205768,E=6205775,C=26)
	CT(21514)=(S=6205776,E=6205799,C=38)
	CT(21515)=(S=6205800,E=6205801,C=95)
	CT(21516)=(S=6205802,E=6205818,C=38)
	CT(21517)=(S=6205819,E=6205819,C=36)
	CT(21518)=(S=6205820,E=6205822,C=14)
	CT(21519)=(S=6205823,E=6205847,C=38)
	CT(21520)=(S=6205848,E=6205855,C=25)
	CT(21521)=(S=6205856,E=6205951,C=38)
	CT(21522)=(S=6205952,E=6206463,C=31)
	CT(21523)=(S=6206464,E=6206975,C=18)
	CT(21524)=(S=6206976,E=6207743,C=26)
	CT(21525)=(S=6207744,E=6207807,C=45)
	CT(21526)=(S=6207808,E=6207815,C=12)
	CT(21527)=(S=6207816,E=6207823,C=15)
	CT(21528)=(S=6207824,E=6207831,C=19)
	CT(21529)=(S=6207832,E=6207839,C=28)
	CT(21530)=(S=6207840,E=6207871,C=15)
	CT(21531)=(S=6207872,E=6207999,C=14)
	CT(21532)=(S=6208000,E=6208127,C=27)
	CT(21533)=(S=6208128,E=6208255,C=28)
	CT(21534)=(S=6208256,E=6208383,C=39)
	CT(21535)=(S=6208384,E=6208511,C=101)
	CT(21536)=(S=6208512,E=6208639,C=18)
	CT(21537)=(S=6208640,E=6208767,C=17)
	CT(21538)=(S=6208768,E=6208895,C=24)
	CT(21539)=(S=6208896,E=6209023,C=46)
	CT(21540)=(S=6209024,E=6209151,C=18)
	CT(21541)=(S=6209152,E=6209279,C=51)
	CT(21542)=(S=6209280,E=6209407,C=18)
	CT(21543)=(S=6209408,E=6209535,C=19)
	CT(21544)=(S=6209536,E=6211071,C=14)
	CT(21545)=(S=6211072,E=6211079,C=18)
	CT(21546)=(S=6211080,E=6211087,C=14)
	CT(21547)=(S=6211088,E=6211095,C=18)
	CT(21548)=(S=6211096,E=6211103,C=36)
	CT(21549)=(S=6211104,E=6211127,C=18)
	CT(21550)=(S=6211128,E=6211135,C=28)
	CT(21551)=(S=6211136,E=6211143,C=19)
	CT(21552)=(S=6211144,E=6211151,C=15)
	CT(21553)=(S=6211152,E=6211159,C=14)
	CT(21554)=(S=6211160,E=6211167,C=25)
	CT(21555)=(S=6211168,E=6211175,C=15)
	CT(21556)=(S=6211176,E=6211183,C=18)
	CT(21557)=(S=6211184,E=6211191,C=33)
	CT(21558)=(S=6211192,E=6211199,C=63)
	CT(21559)=(S=6211200,E=6211207,C=18)
	CT(21560)=(S=6211208,E=6211215,C=42)
	CT(21561)=(S=6211216,E=6211223,C=12)
	CT(21562)=(S=6211224,E=6211231,C=36)
	CT(21563)=(S=6211232,E=6211239,C=49)
	CT(21564)=(S=6211240,E=6211247,C=14)
	CT(21565)=(S=6211248,E=6211255,C=12)
	CT(21566)=(S=6211256,E=6211263,C=14)
	CT(21567)=(S=6211264,E=6211271,C=18)
	CT(21568)=(S=6211272,E=6211279,C=12)
	CT(21569)=(S=6211280,E=6211287,C=15)
	CT(21570)=(S=6211288,E=6211295,C=18)
	CT(21571)=(S=6211296,E=6211303,C=33)
	CT(21572)=(S=6211304,E=6211311,C=18)
	CT(21573)=(S=6211312,E=6211319,C=33)
	CT(21574)=(S=6211320,E=6211327,C=34)
	CT(21575)=(S=6211328,E=6211335,C=18)
	CT(21576)=(S=6211336,E=6211343,C=15)
	CT(21577)=(S=6211344,E=6211351,C=103)
	CT(21578)=(S=6211352,E=6211359,C=14)
	CT(21579)=(S=6211360,E=6211367,C=37)
	CT(21580)=(S=6211368,E=6211375,C=33)
	CT(21581)=(S=6211376,E=6211383,C=50)
	CT(21582)=(S=6211384,E=6211391,C=47)
	CT(21583)=(S=6211392,E=6211407,C=18)
	CT(21584)=(S=6211408,E=6211415,C=15)
	CT(21585)=(S=6211416,E=6211423,C=28)
	CT(21586)=(S=6211432,E=6211439,C=18)
	CT(21587)=(S=6211440,E=6211447,C=47)
	CT(21588)=(S=6211448,E=6211455,C=12)
	CT(21589)=(S=6211456,E=6211479,C=14)
	CT(21590)=(S=6211480,E=6211487,C=12)
	CT(21591)=(S=6211488,E=6211495,C=31)
	CT(21592)=(S=6211496,E=6211503,C=42)
	CT(21593)=(S=6211504,E=6211511,C=50)
	CT(21594)=(S=6211512,E=6211519,C=14)
	CT(21595)=(S=6211520,E=6211527,C=33)
	CT(21596)=(S=6211528,E=6211535,C=37)
	CT(21597)=(S=6211536,E=6211551,C=18)
	CT(21598)=(S=6211552,E=6211559,C=63)
	CT(21599)=(S=6211560,E=6211567,C=14)
	CT(21600)=(S=6211568,E=6211575,C=28)
	CT(21601)=(S=6211576,E=6211583,C=36)
	CT(21602)=(S=6211584,E=6213631,C=16)
	CT(21603)=(S=6213632,E=6215679,C=36)
	CT(21604)=(S=6215680,E=6217727,C=28)
	CT(21605)=(S=6217728,E=6218751,C=49)
	CT(21606)=(S=6218752,E=6218767,C=25)
	CT(21607)=(S=6218768,E=6218783,C=66)
	CT(21608)=(S=6218784,E=6218799,C=14)
	CT(21609)=(S=6218800,E=6218815,C=34)
	CT(21610)=(S=6218816,E=6218831,C=14)
	CT(21611)=(S=6218832,E=6218847,C=71)
	CT(21612)=(S=6218848,E=6218863,C=28)
	CT(21613)=(S=6218864,E=6218879,C=18)
	CT(21614)=(S=6218880,E=6218895,C=36)
	CT(21615)=(S=6218896,E=6218911,C=34)
	CT(21616)=(S=6218912,E=6218927,C=18)
	CT(21617)=(S=6218928,E=6218943,C=12)
	CT(21618)=(S=6218944,E=6218959,C=18)
	CT(21619)=(S=6218960,E=6218975,C=36)
	CT(21620)=(S=6218976,E=6218991,C=46)
	CT(21621)=(S=6218992,E=6219007,C=18)
	CT(21622)=(S=6219008,E=6219023,C=32)
	CT(21623)=(S=6219024,E=6219039,C=18)
	CT(21624)=(S=6219040,E=6219055,C=71)
	CT(21625)=(S=6219056,E=6219071,C=36)
	CT(21626)=(S=6219072,E=6219087,C=14)
	CT(21627)=(S=6219088,E=6219103,C=33)
	CT(21628)=(S=6219104,E=6219119,C=18)
	CT(21629)=(S=6219120,E=6219135,C=46)
	CT(21630)=(S=6219136,E=6219151,C=14)
	CT(21631)=(S=6219152,E=6219167,C=28)
	CT(21632)=(S=6219168,E=6219183,C=14)
	CT(21633)=(S=6219184,E=6219199,C=28)
	CT(21634)=(S=6219200,E=6219215,C=25)
	CT(21635)=(S=6219216,E=6219231,C=58)
	CT(21636)=(S=6219232,E=6219279,C=18)
	CT(21637)=(S=6219280,E=6219295,C=58)
	CT(21638)=(S=6219296,E=6219311,C=18)
	CT(21639)=(S=6219312,E=6219327,C=28)
	CT(21640)=(S=6219328,E=6219343,C=15)
	CT(21641)=(S=6219344,E=6219359,C=17)
	CT(21642)=(S=6219360,E=6219375,C=63)
	CT(21643)=(S=6219376,E=6219407,C=18)
	CT(21644)=(S=6219408,E=6219423,C=33)
	CT(21645)=(S=6219424,E=6219439,C=18)
	CT(21646)=(S=6219440,E=6219455,C=46)
	CT(21647)=(S=6219456,E=6219471,C=31)
	CT(21648)=(S=6219472,E=6219487,C=34)
	CT(21649)=(S=6219488,E=6219503,C=98)
	CT(21650)=(S=6219504,E=6219519,C=18)
	CT(21651)=(S=6219520,E=6219535,C=58)
	CT(21652)=(S=6219536,E=6219551,C=18)
	CT(21653)=(S=6219552,E=6219567,C=31)
	CT(21654)=(S=6219568,E=6219583,C=58)
	CT(21655)=(S=6219584,E=6219599,C=31)
	CT(21656)=(S=6219600,E=6219615,C=34)
	CT(21657)=(S=6219616,E=6219631,C=24)
	CT(21658)=(S=6219632,E=6219695,C=18)
	CT(21659)=(S=6219696,E=6219711,C=31)
	CT(21660)=(S=6219712,E=6219727,C=18)
	CT(21661)=(S=6219744,E=6219759,C=58)
	CT(21662)=(S=6219760,E=6219775,C=36)
	CT(21663)=(S=6219776,E=6219803,C=18)
	CT(21664)=(S=6219804,E=6219805,C=95)
	CT(21665)=(S=6219806,E=6219806,C=53)
	CT(21666)=(S=6219807,E=6219807,C=18)
	CT(21667)=(S=6219808,E=6219815,C=58)
	CT(21668)=(S=6219816,E=6219847,C=18)
	CT(21669)=(S=6219848,E=6219855,C=31)
	CT(21670)=(S=6219856,E=6219863,C=22)
	CT(21671)=(S=6219864,E=6219871,C=18)
	CT(21672)=(S=6219872,E=6219879,C=38)
	CT(21673)=(S=6219880,E=6219887,C=18)
	CT(21674)=(S=6219888,E=6219895,C=36)
	CT(21675)=(S=6219896,E=6219903,C=44)
	CT(21676)=(S=6219904,E=6219919,C=18)
	CT(21677)=(S=6219920,E=6219935,C=58)
	CT(21678)=(S=6219936,E=6219943,C=36)
	CT(21679)=(S=6219944,E=6219951,C=26)
	CT(21680)=(S=6219952,E=6219959,C=31)
	CT(21681)=(S=6219960,E=6219967,C=18)
	CT(21682)=(S=6219968,E=6219975,C=28)
	CT(21683)=(S=6219976,E=6219983,C=18)
	CT(21684)=(S=6219984,E=6219991,C=31)
	CT(21685)=(S=6219992,E=6219999,C=58)
	CT(21686)=(S=6220000,E=6220007,C=44)
	CT(21687)=(S=6220008,E=6220287,C=18)
	CT(21688)=(S=6220288,E=6220543,C=19)
	CT(21689)=(S=6220544,E=6220799,C=37)
	CT(21690)=(S=6220800,E=6220927,C=14)
	CT(21691)=(S=6220928,E=6221055,C=51)
	CT(21692)=(S=6221056,E=6221183,C=47)
	CT(21693)=(S=6221184,E=6221311,C=18)
	CT(21694)=(S=6221312,E=6221823,C=12)
	CT(21695)=(S=6221824,E=6221887,C=58)
	CT(21696)=(S=6221888,E=6221951,C=18)
	CT(21697)=(S=6221952,E=6222015,C=31)
	CT(21698)=(S=6222016,E=6222079,C=40)
	CT(21699)=(S=6222080,E=6222143,C=18)
	CT(21700)=(S=6222144,E=6222207,C=33)
	CT(21701)=(S=6222208,E=6222271,C=26)
	CT(21702)=(S=6222272,E=6222399,C=18)
	CT(21703)=(S=6222400,E=6222463,C=33)
	CT(21704)=(S=6222464,E=6222527,C=18)
	CT(21705)=(S=6222528,E=6222591,C=70)
	CT(21706)=(S=6222592,E=6222655,C=18)
	CT(21707)=(S=6222656,E=6222719,C=53)
	CT(21708)=(S=6222720,E=6222847,C=18)
	CT(21709)=(S=6222848,E=6222911,C=31)
	CT(21710)=(S=6222912,E=6222975,C=44)
	CT(21711)=(S=6222976,E=6223039,C=31)
	CT(21712)=(S=6223040,E=6223103,C=34)
	CT(21713)=(S=6223104,E=6223167,C=19)
	CT(21714)=(S=6223168,E=6223231,C=14)
	CT(21715)=(S=6223232,E=6223295,C=18)
	CT(21716)=(S=6223296,E=6223359,C=42)
	CT(21717)=(S=6223360,E=6223423,C=27)
	CT(21718)=(S=6223424,E=6223487,C=19)
	CT(21719)=(S=6223488,E=6223551,C=58)
	CT(21720)=(S=6223552,E=6223615,C=55)
	CT(21721)=(S=6223616,E=6223623,C=36)
	CT(21722)=(S=6223624,E=6223631,C=15)
	CT(21723)=(S=6223632,E=6223639,C=18)
	CT(21724)=(S=6223640,E=6223647,C=12)
	CT(21725)=(S=6223648,E=6223655,C=33)
	CT(21726)=(S=6223656,E=6223663,C=28)
	CT(21727)=(S=6223664,E=6223671,C=63)
	CT(21728)=(S=6223672,E=6223679,C=18)
	CT(21729)=(S=6223680,E=6223687,C=20)
	CT(21730)=(S=6223688,E=6223695,C=36)
	CT(21731)=(S=6223696,E=6223703,C=14)
	CT(21732)=(S=6223704,E=6223711,C=50)
	CT(21733)=(S=6223712,E=6223719,C=14)
	CT(21734)=(S=6223720,E=6223727,C=18)
	CT(21735)=(S=6223728,E=6223735,C=28)
	CT(21736)=(S=6223736,E=6223743,C=18)
	CT(21737)=(S=6223744,E=6223751,C=20)
	CT(21738)=(S=6223760,E=6223767,C=42)
	CT(21739)=(S=6223768,E=6223775,C=14)
	CT(21740)=(S=6223776,E=6223783,C=12)
	CT(21741)=(S=6223784,E=6223791,C=19)
	CT(21742)=(S=6223792,E=6223799,C=12)
	CT(21743)=(S=6223800,E=6223807,C=15)
	CT(21744)=(S=6223808,E=6223815,C=36)
	CT(21745)=(S=6223816,E=6223823,C=46)
	CT(21746)=(S=6223824,E=6223831,C=18)
	CT(21747)=(S=6223832,E=6223839,C=34)
	CT(21748)=(S=6223840,E=6223847,C=31)
	CT(21749)=(S=6223848,E=6223855,C=12)
	CT(21750)=(S=6223856,E=6223871,C=18)
	CT(21751)=(S=6223872,E=6223999,C=31)
	CT(21752)=(S=6224000,E=6224127,C=50)
	CT(21753)=(S=6224128,E=6224255,C=61)
	CT(21754)=(S=6224256,E=6224383,C=28)
	CT(21755)=(S=6224384,E=6224511,C=59)
	CT(21756)=(S=6224512,E=6224575,C=57)
	CT(21757)=(S=6224576,E=6224607,C=28)
	CT(21758)=(S=6224608,E=6224623,C=14)
	CT(21759)=(S=6224624,E=6224631,C=33)
	CT(21760)=(S=6224632,E=6224639,C=36)
	CT(21761)=(S=6224640,E=6224767,C=18)
	CT(21762)=(S=6224768,E=6224895,C=58)
	CT(21763)=(S=6224896,E=6225023,C=70)
	CT(21764)=(S=6225024,E=6225151,C=30)
	CT(21765)=(S=6225152,E=6225279,C=18)
	CT(21766)=(S=6225280,E=6225407,C=57)
	CT(21767)=(S=6225408,E=6225535,C=19)
	CT(21768)=(S=6225536,E=6225663,C=58)
	CT(21769)=(S=6225664,E=6225791,C=18)
	CT(21770)=(S=6225792,E=6225919,C=19)
	CT(21771)=(S=6225920,E=6230015,C=37)
	CT(21772)=(S=6230016,E=6232063,C=25)
	CT(21773)=(S=6232064,E=6234367,C=18)
	CT(21774)=(S=6234368,E=6234623,C=28)
	CT(21775)=(S=6234624,E=6234879,C=27)
	CT(21776)=(S=6234880,E=6235135,C=17)
	CT(21777)=(S=6235136,E=6235391,C=36)
	CT(21778)=(S=6235392,E=6235647,C=18)
	CT(21779)=(S=6235648,E=6235903,C=26)
	CT(21780)=(S=6235904,E=6236159,C=25)
	CT(21781)=(S=6236160,E=6236671,C=58)
	CT(21782)=(S=6236672,E=6237183,C=51)
	CT(21783)=(S=6237184,E=6237695,C=63)
	CT(21784)=(S=6237696,E=6238207,C=33)
	CT(21785)=(S=6238208,E=6239231,C=58)
	CT(21786)=(S=6239232,E=6240255,C=18)
	CT(21787)=(S=6240256,E=6241279,C=20)
	CT(21788)=(S=6241280,E=6242303,C=25)
	CT(21789)=(S=6242304,E=6242431,C=26)
	CT(21790)=(S=6242432,E=6242559,C=18)
	CT(21791)=(S=6242560,E=6242687,C=53)
	CT(21792)=(S=6242688,E=6242815,C=37)
	CT(21793)=(S=6242816,E=6242943,C=101)
	CT(21794)=(S=6242944,E=6243071,C=18)
	CT(21795)=(S=6243072,E=6243199,C=31)
	CT(21796)=(S=6243200,E=6243327,C=18)
	CT(21797)=(S=6243328,E=6243455,C=60)
	CT(21798)=(S=6243456,E=6243583,C=18)
	CT(21799)=(S=6243584,E=6243711,C=21)
	CT(21800)=(S=6243712,E=6243839,C=31)
	CT(21801)=(S=6243840,E=6243967,C=18)
	CT(21802)=(S=6243968,E=6244095,C=37)
	CT(21803)=(S=6244096,E=6244863,C=18)
	CT(21804)=(S=6244864,E=6245375,C=15)
	CT(21805)=(S=6245376,E=6245887,C=38)
	CT(21806)=(S=6245888,E=6246399,C=18)
	CT(21807)=(S=6246400,E=6246463,C=19)
	CT(21808)=(S=6246464,E=6246527,C=18)
	CT(21809)=(S=6246528,E=6246591,C=26)
	CT(21810)=(S=6246592,E=6246655,C=33)
	CT(21811)=(S=6246656,E=6246719,C=31)
	CT(21812)=(S=6246720,E=6246783,C=26)
	CT(21813)=(S=6246784,E=6246847,C=12)
	CT(21814)=(S=6246848,E=6246911,C=18)
	CT(21815)=(S=6246912,E=6247039,C=26)
	CT(21816)=(S=6247040,E=6247103,C=33)
	CT(21817)=(S=6247104,E=6247359,C=18)
	CT(21818)=(S=6247360,E=6247423,C=63)
	CT(21819)=(S=6247424,E=6247487,C=18)
	CT(21820)=(S=6247488,E=6247551,C=104)
	CT(21821)=(S=6247552,E=6247679,C=18)
	CT(21822)=(S=6247680,E=6247743,C=13)
	CT(21823)=(S=6247744,E=6247775,C=18)
	CT(21824)=(S=6247776,E=6247807,C=190)
	CT(21825)=(S=6247808,E=6247871,C=46)
	CT(21826)=(S=6247872,E=6247935,C=33)
	CT(21827)=(S=6247936,E=6247999,C=54)
	CT(21828)=(S=6248000,E=6248063,C=17)
	CT(21829)=(S=6248064,E=6248127,C=41)
	CT(21830)=(S=6248128,E=6248191,C=18)
	CT(21831)=(S=6248192,E=6248255,C=51)
	CT(21832)=(S=6248256,E=6248287,C=62)
	CT(21833)=(S=6248288,E=6248303,C=14)
	CT(21834)=(S=6248304,E=6248311,C=30)
	CT(21835)=(S=6248312,E=6248319,C=15)
	CT(21836)=(S=6248320,E=6248383,C=52)
	CT(21837)=(S=6248384,E=6248447,C=51)
	CT(21838)=(S=6248448,E=6249471,C=28)
	CT(21839)=(S=6249472,E=6250495,C=21)
	CT(21840)=(S=6250496,E=6251519,C=36)
	CT(21841)=(S=6251520,E=6252031,C=13)
	CT(21842)=(S=6252032,E=6252543,C=71)
	CT(21843)=(S=6252544,E=6252671,C=40)
	CT(21844)=(S=6252672,E=6252927,C=18)
	CT(21845)=(S=6252928,E=6253055,C=71)
	CT(21846)=(S=6253056,E=6253439,C=18)
	CT(21847)=(S=6253440,E=6253567,C=96)
	CT(21848)=(S=6253568,E=6253695,C=58)
	CT(21849)=(S=6253696,E=6253823,C=18)
	CT(21850)=(S=6253824,E=6253951,C=19)
	CT(21851)=(S=6253952,E=6254079,C=31)
	CT(21852)=(S=6254080,E=6254207,C=18)
	CT(21853)=(S=6254208,E=6254335,C=15)
	CT(21854)=(S=6254336,E=6254463,C=51)
	CT(21855)=(S=6254464,E=6254591,C=31)
	CT(21856)=(S=6254592,E=6256639,C=28)
	CT(21857)=(S=6256640,E=6258687,C=25)
	CT(21858)=(S=6258688,E=6258695,C=36)
	CT(21859)=(S=6258696,E=6258703,C=14)
	CT(21860)=(S=6258704,E=6258711,C=28)
	CT(21861)=(S=6258712,E=6258719,C=24)
	CT(21862)=(S=6258720,E=6258727,C=34)
	CT(21863)=(S=6258728,E=6258743,C=14)
	CT(21864)=(S=6258744,E=6258751,C=37)
	CT(21865)=(S=6258752,E=6258759,C=68)
	CT(21866)=(S=6258760,E=6258767,C=12)
	CT(21867)=(S=6258768,E=6258775,C=34)
	CT(21868)=(S=6258776,E=6258783,C=36)
	CT(21869)=(S=6258784,E=6258791,C=49)
	CT(21870)=(S=6258792,E=6258799,C=27)
	CT(21871)=(S=6258800,E=6258807,C=19)
	CT(21872)=(S=6258808,E=6258815,C=70)
	CT(21873)=(S=6258816,E=6258823,C=14)
	CT(21874)=(S=6258824,E=6258831,C=18)
	CT(21875)=(S=6258832,E=6258839,C=12)
	CT(21876)=(S=6258840,E=6258847,C=25)
	CT(21877)=(S=6258848,E=6258855,C=18)
	CT(21878)=(S=6258856,E=6258863,C=38)
	CT(21879)=(S=6258864,E=6258871,C=18)
	CT(21880)=(S=6258872,E=6258879,C=54)
	CT(21881)=(S=6258880,E=6258887,C=63)
	CT(21882)=(S=6258888,E=6258903,C=28)
	CT(21883)=(S=6258904,E=6258911,C=14)
	CT(21884)=(S=6258912,E=6258919,C=18)
	CT(21885)=(S=6258920,E=6258927,C=57)
	CT(21886)=(S=6258928,E=6258935,C=18)
	CT(21887)=(S=6258936,E=6258943,C=28)
	CT(21888)=(S=6258944,E=6258951,C=39)
	CT(21889)=(S=6258952,E=6258959,C=23)
	CT(21890)=(S=6258960,E=6258967,C=14)
	CT(21891)=(S=6258968,E=6258975,C=36)
	CT(21892)=(S=6258976,E=6258983,C=17)
	CT(21893)=(S=6258984,E=6258991,C=22)
	CT(21894)=(S=6258992,E=6258999,C=28)
	CT(21895)=(S=6259000,E=6259007,C=18)
	CT(21896)=(S=6259008,E=6259015,C=14)
	CT(21897)=(S=6259016,E=6259023,C=18)
	CT(21898)=(S=6259024,E=6259031,C=36)
	CT(21899)=(S=6259032,E=6259039,C=18)
	CT(21900)=(S=6259040,E=6259047,C=33)
	CT(21901)=(S=6259048,E=6259055,C=14)
	CT(21902)=(S=6259056,E=6259063,C=25)
	CT(21903)=(S=6259064,E=6259079,C=36)
	CT(21904)=(S=6259080,E=6259095,C=18)
	CT(21905)=(S=6259096,E=6259103,C=25)
	CT(21906)=(S=6259104,E=6259127,C=18)
	CT(21907)=(S=6259128,E=6259135,C=14)
	CT(21908)=(S=6259136,E=6259143,C=55)
	CT(21909)=(S=6259144,E=6259151,C=42)
	CT(21910)=(S=6259152,E=6259159,C=28)
	CT(21911)=(S=6259160,E=6259167,C=25)
	CT(21912)=(S=6259168,E=6259175,C=58)
	CT(21913)=(S=6259176,E=6259183,C=18)
	CT(21914)=(S=6259184,E=6259191,C=33)
	CT(21915)=(S=6259192,E=6259199,C=25)
	CT(21916)=(S=6259208,E=6259215,C=12)
	CT(21917)=(S=6259216,E=6259223,C=28)
	CT(21918)=(S=6259224,E=6259231,C=47)
	CT(21919)=(S=6259232,E=6259239,C=60)
	CT(21920)=(S=6259240,E=6259247,C=49)
	CT(21921)=(S=6259248,E=6259255,C=25)
	CT(21922)=(S=6259256,E=6259263,C=26)
	CT(21923)=(S=6259264,E=6259271,C=19)
	CT(21924)=(S=6259272,E=6259279,C=14)
	CT(21925)=(S=6259280,E=6259287,C=68)
	CT(21926)=(S=6259288,E=6259295,C=43)
	CT(21927)=(S=6259296,E=6259311,C=14)
	CT(21928)=(S=6259312,E=6259319,C=13)
	CT(21929)=(S=6259320,E=6259327,C=42)
	CT(21930)=(S=6259328,E=6259335,C=18)
	CT(21931)=(S=6259336,E=6259343,C=15)
	CT(21932)=(S=6259344,E=6259351,C=14)
	CT(21933)=(S=6259352,E=6259359,C=12)
	CT(21934)=(S=6259360,E=6259367,C=28)
	CT(21935)=(S=6259368,E=6259375,C=37)
	CT(21936)=(S=6259376,E=6259383,C=18)
	CT(21937)=(S=6259384,E=6259391,C=70)
	CT(21938)=(S=6259392,E=6259399,C=14)
	CT(21939)=(S=6259400,E=6259407,C=28)
	CT(21940)=(S=6259408,E=6259415,C=24)
	CT(21941)=(S=6259416,E=6259423,C=27)
	CT(21942)=(S=6259432,E=6259439,C=36)
	CT(21943)=(S=6259440,E=6259447,C=26)
	CT(21944)=(S=6259448,E=6259455,C=28)
	CT(21945)=(S=6259456,E=6259471,C=18)
	CT(21946)=(S=6259472,E=6259479,C=12)
	CT(21947)=(S=6259480,E=6259487,C=18)
	CT(21948)=(S=6259488,E=6259495,C=58)
	CT(21949)=(S=6259496,E=6259503,C=15)
	CT(21950)=(S=6259504,E=6259511,C=50)
	CT(21951)=(S=6259512,E=6259519,C=15)
	CT(21952)=(S=6259520,E=6259527,C=14)
	CT(21953)=(S=6259528,E=6259535,C=18)
	CT(21954)=(S=6259536,E=6259543,C=14)
	CT(21955)=(S=6259544,E=6259551,C=18)
	CT(21956)=(S=6259552,E=6259559,C=28)
	CT(21957)=(S=6259560,E=6259567,C=14)
	CT(21958)=(S=6259568,E=6259575,C=60)
	CT(21959)=(S=6259576,E=6259583,C=28)
	CT(21960)=(S=6259584,E=6259591,C=71)
	CT(21961)=(S=6259592,E=6259599,C=12)
	CT(21962)=(S=6259600,E=6259607,C=18)
	CT(21963)=(S=6259608,E=6259615,C=14)
	CT(21964)=(S=6259616,E=6259631,C=25)
	CT(21965)=(S=6259632,E=6259639,C=18)
	CT(21966)=(S=6259640,E=6259647,C=69)
	CT(21967)=(S=6259648,E=6259655,C=42)
	CT(21968)=(S=6259656,E=6259663,C=19)
	CT(21969)=(S=6259664,E=6259671,C=18)
	CT(21970)=(S=6259672,E=6259679,C=14)
	CT(21971)=(S=6259680,E=6259687,C=18)
	CT(21972)=(S=6259688,E=6259695,C=72)
	CT(21973)=(S=6259696,E=6259703,C=12)
	CT(21974)=(S=6259704,E=6259711,C=14)
	CT(21975)=(S=6259712,E=6260735,C=31)
	CT(21976)=(S=6260736,E=6260863,C=21)
	CT(21977)=(S=6260864,E=6260991,C=12)
	CT(21978)=(S=6260992,E=6261119,C=18)
	CT(21979)=(S=6261120,E=6261247,C=40)
	CT(21980)=(S=6261248,E=6261375,C=12)
	CT(21981)=(S=6261376,E=6261439,C=14)
	CT(21982)=(S=6261440,E=6261447,C=50)
	CT(21983)=(S=6261448,E=6261455,C=14)
	CT(21984)=(S=6261456,E=6261471,C=24)
	CT(21985)=(S=6261472,E=6261759,C=18)
	CT(21986)=(S=6261760,E=6261775,C=12)
	CT(21987)=(S=6261776,E=6261791,C=18)
	CT(21988)=(S=6261792,E=6261807,C=50)
	CT(21989)=(S=6261808,E=6261823,C=14)
	CT(21990)=(S=6261824,E=6261839,C=19)
	CT(21991)=(S=6261840,E=6261855,C=18)
	CT(21992)=(S=6261856,E=6261871,C=30)
	CT(21993)=(S=6261872,E=6261887,C=46)
	CT(21994)=(S=6261888,E=6261903,C=15)
	CT(21995)=(S=6261904,E=6261919,C=18)
	CT(21996)=(S=6261920,E=6261935,C=61)
	CT(21997)=(S=6261936,E=6261951,C=19)
	CT(21998)=(S=6261952,E=6261967,C=66)
	CT(21999)=(S=6261968,E=6261983,C=51)
	CT(22000)=(S=6261984,E=6261999,C=14)
	CT(22001)=(S=6262000,E=6262015,C=33)
	CT(22002)=(S=6262016,E=6262031,C=18)
	CT(22003)=(S=6262032,E=6262047,C=14)
	CT(22004)=(S=6262048,E=6262063,C=15)
	CT(22005)=(S=6262064,E=6262079,C=39)
	CT(22006)=(S=6262080,E=6262111,C=27)
	CT(22007)=(S=6262112,E=6262143,C=12)
	CT(22008)=(S=6262144,E=6262159,C=20)
	CT(22009)=(S=6262160,E=6262191,C=14)
	CT(22010)=(S=6262192,E=6262223,C=18)
	CT(22011)=(S=6262224,E=6262239,C=61)
	CT(22012)=(S=6262240,E=6262255,C=18)
	CT(22013)=(S=6262256,E=6262271,C=101)
	CT(22014)=(S=6262272,E=6262287,C=25)
	CT(22015)=(S=6262288,E=6262303,C=17)
	CT(22016)=(S=6262304,E=6262319,C=18)
	CT(22017)=(S=6262320,E=6262335,C=36)
	CT(22018)=(S=6262336,E=6262351,C=28)
	CT(22019)=(S=6262352,E=6262367,C=103)
	CT(22020)=(S=6262368,E=6262383,C=36)
	CT(22021)=(S=6262384,E=6262399,C=63)
	CT(22022)=(S=6262400,E=6262415,C=37)
	CT(22023)=(S=6262416,E=6262431,C=14)
	CT(22024)=(S=6262432,E=6262447,C=12)
	CT(22025)=(S=6262448,E=6262463,C=15)
	CT(22026)=(S=6262464,E=6262479,C=18)
	CT(22027)=(S=6262480,E=6262495,C=15)
	CT(22028)=(S=6262496,E=6262511,C=26)
	CT(22029)=(S=6262512,E=6262527,C=58)
	CT(22030)=(S=6262528,E=6262559,C=18)
	CT(22031)=(S=6262560,E=6262575,C=28)
	CT(22032)=(S=6262576,E=6262591,C=34)
	CT(22033)=(S=6262592,E=6262607,C=19)
	CT(22034)=(S=6262608,E=6262623,C=42)
	CT(22035)=(S=6262624,E=6262639,C=15)
	CT(22036)=(S=6262640,E=6262655,C=18)
	CT(22037)=(S=6262656,E=6262671,C=33)
	CT(22038)=(S=6262672,E=6262687,C=52)
	CT(22039)=(S=6262688,E=6262703,C=28)
	CT(22040)=(S=6262704,E=6262719,C=18)
	CT(22041)=(S=6262720,E=6262735,C=19)
	CT(22042)=(S=6262736,E=6262751,C=18)
	CT(22043)=(S=6262752,E=6262767,C=42)
	CT(22044)=(S=6262768,E=6262783,C=58)
	CT(22045)=(S=6262784,E=6264831,C=14)
	CT(22046)=(S=6264832,E=6264895,C=18)
	CT(22047)=(S=6264896,E=6264959,C=34)
	CT(22048)=(S=6264960,E=6265023,C=36)
	CT(22049)=(S=6265024,E=6265087,C=14)
	CT(22050)=(S=6265088,E=6265151,C=55)
	CT(22051)=(S=6265152,E=6265215,C=53)
	CT(22052)=(S=6265216,E=6265343,C=18)
	CT(22053)=(S=6265344,E=6265407,C=24)
	CT(22054)=(S=6265408,E=6265535,C=18)
	CT(22055)=(S=6265536,E=6265599,C=14)
	CT(22056)=(S=6265600,E=6265663,C=102)
	CT(22057)=(S=6265664,E=6265727,C=58)
	CT(22058)=(S=6265728,E=6265791,C=36)
	CT(22059)=(S=6265792,E=6265855,C=19)
	CT(22060)=(S=6265856,E=6265919,C=54)
	CT(22061)=(S=6265920,E=6265983,C=18)
	CT(22062)=(S=6265984,E=6266047,C=59)
	CT(22063)=(S=6266048,E=6266059,C=28)
	CT(22064)=(S=6266060,E=6266063,C=25)
	CT(22065)=(S=6266064,E=6266095,C=28)
	CT(22066)=(S=6266096,E=6266103,C=58)
	CT(22067)=(S=6266104,E=6266175,C=28)
	CT(22068)=(S=6266176,E=6266239,C=15)
	CT(22069)=(S=6266240,E=6266367,C=12)
	CT(22070)=(S=6266368,E=6266431,C=31)
	CT(22071)=(S=6266432,E=6266495,C=58)
	CT(22072)=(S=6266496,E=6266559,C=51)
	CT(22073)=(S=6266560,E=6266623,C=18)
	CT(22074)=(S=6266624,E=6266687,C=30)
	CT(22075)=(S=6266688,E=6266751,C=35)
	CT(22076)=(S=6266752,E=6266815,C=18)
	CT(22077)=(S=6266816,E=6266879,C=52)
	CT(22078)=(S=6266880,E=6267135,C=58)
	CT(22079)=(S=6267136,E=6267391,C=18)
	CT(22080)=(S=6267392,E=6267647,C=26)
	CT(22081)=(S=6267648,E=6267903,C=18)
	CT(22082)=(S=6267904,E=6268159,C=31)
	CT(22083)=(S=6268160,E=6268415,C=18)
	CT(22084)=(S=6268416,E=6268671,C=24)
	CT(22085)=(S=6268672,E=6268927,C=18)
	CT(22086)=(S=6268928,E=6268959,C=12)
	CT(22087)=(S=6268960,E=6268991,C=50)
	CT(22088)=(S=6268992,E=6269023,C=46)
	CT(22089)=(S=6269024,E=6269055,C=57)
	CT(22090)=(S=6269056,E=6269119,C=28)
	CT(22091)=(S=6269120,E=6269151,C=33)
	CT(22092)=(S=6269152,E=6269183,C=51)
	CT(22093)=(S=6269184,E=6269215,C=31)
	CT(22094)=(S=6269216,E=6269247,C=27)
	CT(22095)=(S=6269248,E=6269279,C=15)
	CT(22096)=(S=6269280,E=6269343,C=18)
	CT(22097)=(S=6269344,E=6269375,C=28)
	CT(22098)=(S=6269376,E=6269407,C=51)
	CT(22099)=(S=6269408,E=6269439,C=25)
	CT(22100)=(S=6269440,E=6269503,C=12)
	CT(22101)=(S=6269504,E=6269535,C=36)
	CT(22102)=(S=6269536,E=6269631,C=18)
	CT(22103)=(S=6269632,E=6269663,C=35)
	CT(22104)=(S=6269664,E=6269695,C=71)
	CT(22105)=(S=6269696,E=6269727,C=18)
	CT(22106)=(S=6269728,E=6269759,C=15)
	CT(22107)=(S=6269760,E=6269791,C=50)
	CT(22108)=(S=6269792,E=6269823,C=58)
	CT(22109)=(S=6269824,E=6269855,C=12)
	CT(22110)=(S=6269856,E=6269887,C=49)
	CT(22111)=(S=6269888,E=6269919,C=58)
	CT(22112)=(S=6269920,E=6269951,C=18)
	CT(22113)=(S=6269952,E=6269983,C=14)
	CT(22114)=(S=6269984,E=6270015,C=18)
	CT(22115)=(S=6270016,E=6270047,C=14)
	CT(22116)=(S=6270048,E=6270111,C=18)
	CT(22117)=(S=6270112,E=6270143,C=21)
	CT(22118)=(S=6270144,E=6270175,C=61)
	CT(22119)=(S=6270176,E=6270207,C=14)
	CT(22120)=(S=6270208,E=6270239,C=37)
	CT(22121)=(S=6270240,E=6270271,C=44)
	CT(22122)=(S=6270272,E=6270303,C=33)
	CT(22123)=(S=6270304,E=6270335,C=28)
	CT(22124)=(S=6270336,E=6270367,C=18)
	CT(22125)=(S=6270368,E=6270399,C=37)
	CT(22126)=(S=6270400,E=6270431,C=33)
	CT(22127)=(S=6270432,E=6270463,C=37)
	CT(22128)=(S=6270464,E=6270495,C=15)
	CT(22129)=(S=6270496,E=6270527,C=58)
	CT(22130)=(S=6270536,E=6270543,C=12)
	CT(22131)=(S=6270544,E=6270551,C=28)
	CT(22132)=(S=6270552,E=6270591,C=18)
	CT(22133)=(S=6270592,E=6270623,C=28)
	CT(22134)=(S=6270624,E=6270655,C=12)
	CT(22135)=(S=6270656,E=6270687,C=18)
	CT(22136)=(S=6270688,E=6270719,C=34)
	CT(22137)=(S=6270720,E=6270751,C=58)
	CT(22138)=(S=6270752,E=6270783,C=17)
	CT(22139)=(S=6270784,E=6270815,C=101)
	CT(22140)=(S=6270816,E=6270847,C=47)
	CT(22141)=(S=6270848,E=6270879,C=14)
	CT(22142)=(S=6270880,E=6270911,C=12)
	CT(22143)=(S=6270912,E=6270943,C=14)
	CT(22144)=(S=6270944,E=6270975,C=18)
	CT(22145)=(S=6270976,E=6271103,C=12)
	CT(22146)=(S=6271104,E=6271231,C=52)
	CT(22147)=(S=6271232,E=6271359,C=14)
	CT(22148)=(S=6271360,E=6271487,C=38)
	CT(22149)=(S=6271488,E=6271615,C=58)
	CT(22150)=(S=6271616,E=6271743,C=57)
	CT(22151)=(S=6271744,E=6271871,C=18)
	CT(22152)=(S=6271872,E=6271999,C=22)
	CT(22153)=(S=6272000,E=6272127,C=46)
	CT(22154)=(S=6272128,E=6272255,C=54)
	CT(22155)=(S=6272256,E=6272475,C=18)
	CT(22156)=(S=6272476,E=6272479,C=12)
	CT(22157)=(S=6272480,E=6272511,C=18)
	CT(22158)=(S=6272512,E=6272535,C=20)
	CT(22159)=(S=6272536,E=6272543,C=58)
	CT(22160)=(S=6272544,E=6272639,C=18)
	CT(22161)=(S=6272640,E=6272767,C=49)
	CT(22162)=(S=6272768,E=6272775,C=17)
	CT(22163)=(S=6272776,E=6272815,C=18)
	CT(22164)=(S=6272816,E=6272831,C=34)
	CT(22165)=(S=6272832,E=6272895,C=18)
	CT(22166)=(S=6272896,E=6273023,C=37)
	CT(22167)=(S=6273024,E=6274047,C=23)
	CT(22168)=(S=6274048,E=6275071,C=18)
	CT(22169)=(S=6275072,E=6279167,C=19)
	CT(22170)=(S=6279168,E=6279423,C=28)
	CT(22171)=(S=6279424,E=6279679,C=19)
	CT(22172)=(S=6279680,E=6279935,C=15)
	CT(22173)=(S=6279936,E=6280191,C=36)
	CT(22174)=(S=6280192,E=6280447,C=106)
	CT(22175)=(S=6280448,E=6280703,C=18)
	CT(22176)=(S=6280704,E=6280959,C=25)
	CT(22177)=(S=6280960,E=6280963,C=18)
	CT(22178)=(S=6280964,E=6280967,C=36)
	CT(22179)=(S=6280968,E=6280975,C=18)
	CT(22180)=(S=6280976,E=6280979,C=19)
	CT(22181)=(S=6280980,E=6280991,C=58)
	CT(22182)=(S=6280992,E=6280995,C=28)
	CT(22183)=(S=6280996,E=6281003,C=31)
	CT(22184)=(S=6281004,E=6281007,C=60)
	CT(22185)=(S=6281008,E=6281011,C=31)
	CT(22186)=(S=6281012,E=6281015,C=58)
	CT(22187)=(S=6281016,E=6281019,C=18)
	CT(22188)=(S=6281020,E=6281023,C=25)
	CT(22189)=(S=6281024,E=6281031,C=18)
	CT(22190)=(S=6281032,E=6281035,C=12)
	CT(22191)=(S=6281036,E=6281039,C=58)
	CT(22192)=(S=6281044,E=6281047,C=18)
	CT(22193)=(S=6281048,E=6281051,C=14)
	CT(22194)=(S=6281052,E=6281055,C=31)
	CT(22195)=(S=6281056,E=6281063,C=18)
	CT(22196)=(S=6281064,E=6281067,C=31)
	CT(22197)=(S=6281068,E=6281071,C=18)
	CT(22198)=(S=6281072,E=6281079,C=31)
	CT(22199)=(S=6281080,E=6281083,C=25)
	CT(22200)=(S=6281084,E=6281087,C=31)
	CT(22201)=(S=6281088,E=6281091,C=43)
	CT(22202)=(S=6281092,E=6281095,C=55)
	CT(22203)=(S=6281096,E=6281099,C=18)
	CT(22204)=(S=6281104,E=6281107,C=31)
	CT(22205)=(S=6281108,E=6281115,C=18)
	CT(22206)=(S=6281116,E=6281119,C=31)
	CT(22207)=(S=6281120,E=6281123,C=26)
	CT(22208)=(S=6281124,E=6281127,C=31)
	CT(22209)=(S=6281128,E=6281131,C=19)
	CT(22210)=(S=6281132,E=6281132,C=18)
	CT(22211)=(S=6281133,E=6281133,C=26)
	CT(22212)=(S=6281134,E=6281135,C=14)
	CT(22213)=(S=6281136,E=6281143,C=18)
	CT(22214)=(S=6281144,E=6281151,C=36)
	CT(22215)=(S=6281152,E=6281159,C=58)
	CT(22216)=(S=6281160,E=6281171,C=18)
	CT(22217)=(S=6281172,E=6281175,C=28)
	CT(22218)=(S=6281176,E=6281179,C=31)
	CT(22219)=(S=6281180,E=6281182,C=38)
	CT(22220)=(S=6281183,E=6281183,C=31)
	CT(22221)=(S=6281184,E=6281187,C=14)
	CT(22222)=(S=6281188,E=6281191,C=18)
	CT(22223)=(S=6281192,E=6281195,C=58)
	CT(22224)=(S=6281196,E=6281196,C=12)
	CT(22225)=(S=6281197,E=6281197,C=14)
	CT(22226)=(S=6281198,E=6281199,C=18)
	CT(22227)=(S=6281200,E=6281203,C=33)
	CT(22228)=(S=6281204,E=6281207,C=62)
	CT(22229)=(S=6281208,E=6281215,C=18)
	CT(22230)=(S=6281216,E=6281727,C=31)
	CT(22231)=(S=6281728,E=6282239,C=23)
	CT(22232)=(S=6282240,E=6282751,C=18)
	CT(22233)=(S=6282752,E=6283263,C=28)
	CT(22234)=(S=6283264,E=6291455,C=15)
	CT(22235)=(S=6291456,E=6291711,C=29)
	CT(22236)=(S=6291712,E=6291967,C=78)
	CT(22237)=(S=6291968,E=6293823,C=29)
	CT(22238)=(S=6293824,E=6293855,C=86)
	CT(22239)=(S=6293856,E=6293887,C=78)
	CT(22240)=(S=6293888,E=6293935,C=7)
	CT(22241)=(S=6293936,E=6293951,C=8)
	CT(22242)=(S=6293952,E=6296575,C=29)
	CT(22243)=(S=6296576,E=6297599,C=78)
	CT(22244)=(S=6297600,E=6299263,C=29)
	CT(22245)=(S=6299264,E=6299359,C=78)
	CT(22246)=(S=6299360,E=6302623,C=29)
	CT(22247)=(S=6302624,E=6302655,C=175)
	CT(22248)=(S=6302656,E=6302687,C=29)
	CT(22249)=(S=6302688,E=6302703,C=78)
	CT(22250)=(S=6302704,E=6302751,C=29)
	CT(22251)=(S=6302752,E=6302847,C=78)
	CT(22252)=(S=6302848,E=6302911,C=29)
	CT(22253)=(S=6302912,E=6302991,C=78)
	CT(22254)=(S=6302992,E=6303151,C=29)
	CT(22255)=(S=6303152,E=6303183,C=78)
	CT(22256)=(S=6303184,E=6303263,C=29)
	CT(22257)=(S=6303264,E=6303295,C=78)
	CT(22258)=(S=6303296,E=6303423,C=29)
	CT(22259)=(S=6303424,E=6303439,C=78)
	CT(22260)=(S=6303440,E=6303663,C=29)
	CT(22261)=(S=6303664,E=6303679,C=78)
	CT(22262)=(S=6303680,E=6303743,C=29)
	CT(22263)=(S=6303744,E=6305791,C=78)
	CT(22264)=(S=6305792,E=6307583,C=29)
	CT(22265)=(S=6307584,E=6307647,C=78)
	CT(22266)=(S=6307648,E=6307711,C=29)
	CT(22267)=(S=6307712,E=6307759,C=78)
	CT(22268)=(S=6307760,E=6323583,C=29)
	CT(22269)=(S=6323584,E=6323599,C=78)
	CT(22270)=(S=6323600,E=6323631,C=29)
	CT(22271)=(S=6323632,E=6323679,C=78)
	CT(22272)=(S=6323680,E=6323695,C=29)
	CT(22273)=(S=6323696,E=6323711,C=78)
	CT(22274)=(S=6323712,E=6324159,C=29)
	CT(22275)=(S=6324160,E=6324223,C=78)
	CT(22276)=(S=6324224,E=6376191,C=29)
	CT(22277)=(S=6376192,E=6376255,C=78)
	CT(22278)=(S=6376256,E=6376383,C=29)
	CT(22279)=(S=6376384,E=6376415,C=78)
	CT(22280)=(S=6376416,E=6384431,C=29)
	CT(22281)=(S=6384432,E=6384447,C=78)
	CT(22282)=(S=6384448,E=6384559,C=29)
	CT(22283)=(S=6384560,E=6384575,C=78)
	CT(22284)=(S=6384576,E=6384591,C=29)
	CT(22285)=(S=6384592,E=6384607,C=78)
	CT(22286)=(S=6384608,E=6384639,C=29)
	CT(22287)=(S=6384640,E=6385663,C=78)
	CT(22288)=(S=6385664,E=6454271,C=29)
	CT(22289)=(S=6454272,E=6454335,C=78)
	CT(22290)=(S=6454336,E=6459119,C=29)
	CT(22291)=(S=6459120,E=6459135,C=78)
	CT(22292)=(S=6459136,E=6459199,C=29)
	CT(22293)=(S=6459200,E=6459247,C=78)
	CT(22294)=(S=6459248,E=6459263,C=29)
	CT(22295)=(S=6459264,E=6459279,C=78)
	CT(22296)=(S=6459280,E=6459343,C=29)
	CT(22297)=(S=6459344,E=6459359,C=78)
	CT(22298)=(S=6459360,E=6459375,C=29)
	CT(22299)=(S=6459376,E=6459391,C=78)
	CT(22300)=(S=6459392,E=6463055,C=29)
	CT(22301)=(S=6463056,E=6463071,C=78)
	CT(22302)=(S=6463072,E=6463087,C=95)
	CT(22303)=(S=6463088,E=6463103,C=29)
	CT(22304)=(S=6463104,E=6463119,C=78)
	CT(22305)=(S=6463120,E=6463471,C=29)
	CT(22306)=(S=6463472,E=6463487,C=78)
	CT(22307)=(S=6463488,E=6537215,C=29)
	CT(22308)=(S=6537216,E=6537343,C=78)
	CT(22309)=(S=6537344,E=6539007,C=29)
	CT(22310)=(S=6539008,E=6539263,C=78)
	CT(22311)=(S=6539264,E=6541311,C=29)
	CT(22312)=(S=6541312,E=6553599,C=78)
	CT(22313)=(S=6553600,E=6564591,C=29)
	CT(22314)=(S=6564592,E=6564607,C=78)
	CT(22315)=(S=6564608,E=6564703,C=29)
	CT(22316)=(S=6564704,E=6564735,C=78)
	CT(22317)=(S=6564736,E=6619135,C=29)
	CT(22318)=(S=6619136,E=6619139,C=1)
	CT(22319)=(S=6619140,E=6619143,C=74)
	CT(22320)=(S=6619144,E=6619167,C=2)
	CT(22321)=(S=6619168,E=6619199,C=4)
	CT(22322)=(S=6619200,E=6619263,C=0)
	CT(22323)=(S=6619264,E=6619391,C=9)
	CT(22324)=(S=6619392,E=6619395,C=1)
	CT(22325)=(S=6619396,E=6619399,C=8)
	CT(22326)=(S=6619400,E=6619407,C=6)
	CT(22327)=(S=6619408,E=6619423,C=8)
	CT(22328)=(S=6619424,E=6619455,C=6)
	CT(22329)=(S=6619456,E=6619647,C=2)
	CT(22330)=(S=6619648,E=6619775,C=4)
	CT(22331)=(S=6619776,E=6619807,C=2)
	CT(22332)=(S=6619808,E=6619815,C=84)
	CT(22333)=(S=6619816,E=6619819,C=0)
	CT(22334)=(S=6619820,E=6619823,C=1)
	CT(22335)=(S=6619824,E=6619839,C=156)
	CT(22336)=(S=6619840,E=6619903,C=0)
	CT(22337)=(S=6619904,E=6620159,C=9)
	CT(22338)=(S=6620160,E=6621183,C=1)
	CT(22339)=(S=6621184,E=6623231,C=9)
	CT(22340)=(S=6623232,E=6631935,C=1)
	CT(22341)=(S=6631936,E=6631939,C=74)
	CT(22342)=(S=6631940,E=6631943,C=2)
	CT(22343)=(S=6631952,E=6631967,C=74)
	CT(22344)=(S=6631968,E=6631991,C=2)
	CT(22345)=(S=6631992,E=6631995,C=1)
	CT(22346)=(S=6631996,E=6631999,C=2)
	CT(22347)=(S=6632000,E=6632063,C=77)
	CT(22348)=(S=6632064,E=6632191,C=2)
	CT(22349)=(S=6632192,E=6632447,C=3)
	CT(22350)=(S=6632448,E=6632703,C=1)
	CT(22351)=(S=6632704,E=6632767,C=11)
	CT(22352)=(S=6632768,E=6632799,C=6)
	CT(22353)=(S=6632800,E=6632803,C=2)
	CT(22354)=(S=6632804,E=6632807,C=1)
	CT(22355)=(S=6632808,E=6632831,C=2)
	CT(22356)=(S=6632832,E=6632959,C=0)
	CT(22357)=(S=6632960,E=6633215,C=1)
	CT(22358)=(S=6633216,E=6633343,C=6)
	CT(22359)=(S=6633344,E=6633439,C=2)
	CT(22360)=(S=6633440,E=6633447,C=1)
	CT(22361)=(S=6633448,E=6633471,C=2)
	CT(22362)=(S=6633472,E=6635519,C=4)
	CT(22363)=(S=6635520,E=6639107,C=1)
	CT(22364)=(S=6639108,E=6639111,C=2)
	CT(22365)=(S=6639112,E=6639119,C=154)
	CT(22366)=(S=6639120,E=6639135,C=10)
	CT(22367)=(S=6639136,E=6639167,C=1)
	CT(22368)=(S=6639168,E=6639231,C=7)
	CT(22369)=(S=6639232,E=6639359,C=8)
	CT(22370)=(S=6639360,E=6639615,C=6)
	CT(22371)=(S=6639616,E=6643723,C=1)
	CT(22372)=(S=6643724,E=6643727,C=11)
	CT(22373)=(S=6643728,E=6643743,C=1)
	CT(22374)=(S=6643744,E=6643775,C=2)
	CT(22375)=(S=6643776,E=6643839,C=11)
	CT(22376)=(S=6643840,E=6643967,C=1)
	CT(22377)=(S=6643968,E=6644223,C=2)
	CT(22378)=(S=6644224,E=6644479,C=73)
	CT(22379)=(S=6644480,E=6644543,C=11)
	CT(22380)=(S=6644544,E=6644575,C=5)
	CT(22381)=(S=6644576,E=6644607,C=1)
	CT(22382)=(S=6644608,E=6644735,C=159)
	CT(22383)=(S=6644736,E=6644895,C=73)
	CT(22384)=(S=6644896,E=6644991,C=7)
	CT(22385)=(S=6644992,E=6645055,C=90)
	CT(22386)=(S=6645056,E=6645087,C=1)
	CT(22387)=(S=6645088,E=6645091,C=8)
	CT(22388)=(S=6645092,E=6645092,C=1)
	CT(22389)=(S=6645093,E=6645093,C=0)
	CT(22390)=(S=6645094,E=6645119,C=1)
	CT(22391)=(S=6645120,E=6645247,C=6)
	CT(22392)=(S=6645248,E=6645311,C=2)
	CT(22393)=(S=6645312,E=6645343,C=1)
	CT(22394)=(S=6645344,E=6645347,C=2)
	CT(22395)=(S=6645348,E=6645350,C=1)
	CT(22396)=(S=6645351,E=6645351,C=0)
	CT(22397)=(S=6645352,E=6645375,C=1)
	CT(22398)=(S=6645376,E=6645503,C=2)
	CT(22399)=(S=6645504,E=6645759,C=0)
	CT(22400)=(S=6645760,E=6646783,C=1)
	CT(22401)=(S=6646784,E=6647295,C=3)
	CT(22402)=(S=6647296,E=6647359,C=2)
	CT(22403)=(S=6647360,E=6647407,C=1)
	CT(22404)=(S=6647408,E=6647411,C=73)
	CT(22405)=(S=6647412,E=6647423,C=1)
	CT(22406)=(S=6647424,E=6647807,C=2)
	CT(22407)=(S=6647808,E=6649855,C=0)
	CT(22408)=(S=6649856,E=6651647,C=1)
	CT(22409)=(S=6651648,E=6651903,C=7)
	CT(22410)=(S=6651904,E=6651907,C=1)
	CT(22411)=(S=6651908,E=6651911,C=0)
	CT(22412)=(S=6651912,E=6651967,C=1)
	CT(22413)=(S=6651968,E=6652031,C=74)
	CT(22414)=(S=6652032,E=6652159,C=2)
	CT(22415)=(S=6652160,E=6653951,C=1)
	CT(22416)=(S=6653952,E=6654975,C=9)
	CT(22417)=(S=6654976,E=6655999,C=2)
	CT(22418)=(S=6656000,E=6660095,C=1)
	CT(22419)=(S=6660096,E=6668287,C=0)
	CT(22420)=(S=6668288,E=6670847,C=1)
	CT(22421)=(S=6670848,E=6671103,C=6)
	CT(22422)=(S=6671104,E=6671231,C=2)
	CT(22423)=(S=6671232,E=6671271,C=1)
	CT(22424)=(S=6671272,E=6671275,C=74)
	CT(22425)=(S=6671276,E=6671295,C=1)
	CT(22426)=(S=6671296,E=6671359,C=2)
	CT(22427)=(S=6671360,E=6672383,C=1)
	CT(22428)=(S=6672384,E=6676479,C=4)
	CT(22429)=(S=6676480,E=6679039,C=1)
	CT(22430)=(S=6679040,E=6679103,C=7)
	CT(22431)=(S=6679104,E=6679111,C=1)
	CT(22432)=(S=6679112,E=6679115,C=0)
	CT(22433)=(S=6679116,E=6679167,C=1)
	CT(22434)=(S=6679168,E=6679295,C=0)
	CT(22435)=(S=6679296,E=6679551,C=6)
	CT(22436)=(S=6679552,E=6683135,C=1)
	CT(22437)=(S=6683136,E=6683391,C=6)
	CT(22438)=(S=6683392,E=6683395,C=1)
	CT(22439)=(S=6683396,E=6683399,C=75)
	CT(22440)=(S=6683400,E=6684415,C=1)
	CT(22441)=(S=6684416,E=6684671,C=74)
	CT(22442)=(S=6750208,E=6750468,C=0)
	CT(22443)=(S=6750469,E=6750469,C=7)
	CT(22444)=(S=6750470,E=6750470,C=4)
	CT(22445)=(S=6750471,E=6750471,C=0)
	CT(22446)=(S=6750472,E=6750475,C=1)
	CT(22447)=(S=6750476,E=6750483,C=8)
	CT(22448)=(S=6750484,E=6750491,C=1)
	CT(22449)=(S=6750492,E=6750495,C=154)
	CT(22450)=(S=6750496,E=6750515,C=8)
	CT(22451)=(S=6750516,E=6750527,C=0)
	CT(22452)=(S=6750528,E=6750531,C=8)
	CT(22453)=(S=6750532,E=6750535,C=5)
	CT(22454)=(S=6750536,E=6750539,C=1)
	CT(22455)=(S=6750540,E=6750543,C=0)
	CT(22456)=(S=6750544,E=6750547,C=4)
	CT(22457)=(S=6750548,E=6750551,C=7)
	CT(22458)=(S=6750552,E=6750555,C=1)
	CT(22459)=(S=6750556,E=6750559,C=75)
	CT(22460)=(S=6750560,E=6750563,C=167)
	CT(22461)=(S=6750564,E=6750567,C=4)
	CT(22462)=(S=6750568,E=6750571,C=5)
	CT(22463)=(S=6750572,E=6750575,C=0)
	CT(22464)=(S=6750576,E=6750579,C=4)
	CT(22465)=(S=6750580,E=6750583,C=10)
	CT(22466)=(S=6750584,E=6750587,C=2)
	CT(22467)=(S=6750588,E=6750595,C=4)
	CT(22468)=(S=6750596,E=6750599,C=5)
	CT(22469)=(S=6750600,E=6750603,C=7)
	CT(22470)=(S=6750604,E=6750607,C=2)
	CT(22471)=(S=6750608,E=6750619,C=5)
	CT(22472)=(S=6750620,E=6750623,C=8)
	CT(22473)=(S=6750624,E=6750631,C=3)
	CT(22474)=(S=6750632,E=6750635,C=1)
	CT(22475)=(S=6750636,E=6750639,C=7)
	CT(22476)=(S=6750640,E=6750643,C=156)
	CT(22477)=(S=6750644,E=6750647,C=91)
	CT(22478)=(S=6750648,E=6750659,C=0)
	CT(22479)=(S=6750660,E=6750663,C=4)
	CT(22480)=(S=6750664,E=6750667,C=11)
	CT(22481)=(S=6750668,E=6750671,C=0)
	CT(22482)=(S=6750672,E=6750675,C=11)
	CT(22483)=(S=6750676,E=6750683,C=0)
	CT(22484)=(S=6750688,E=6750691,C=8)
	CT(22485)=(S=6750692,E=6750695,C=0)
	CT(22486)=(S=6750696,E=6750699,C=154)
	CT(22487)=(S=6750700,E=6750703,C=11)
	CT(22488)=(S=6750704,E=6750707,C=0)
	CT(22489)=(S=6750708,E=6750795,C=2)
	CT(22490)=(S=6750796,E=6750799,C=6)
	CT(22491)=(S=6750800,E=6750803,C=4)
	CT(22492)=(S=6750804,E=6750807,C=6)
	CT(22493)=(S=6750808,E=6750811,C=192)
	CT(22494)=(S=6750812,E=6750815,C=6)
	CT(22495)=(S=6750816,E=6750823,C=2)
	CT(22496)=(S=6750824,E=6750827,C=7)
	CT(22497)=(S=6750828,E=6750831,C=1)
	CT(22498)=(S=6750832,E=6750835,C=3)
	CT(22499)=(S=6750836,E=6750839,C=0)
	CT(22500)=(S=6750840,E=6750843,C=73)
	CT(22501)=(S=6750844,E=6750851,C=2)
	CT(22502)=(S=6750852,E=6750855,C=4)
	CT(22503)=(S=6750856,E=6750859,C=2)
	CT(22504)=(S=6750860,E=6750863,C=73)
	CT(22505)=(S=6750864,E=6750867,C=74)
	CT(22506)=(S=6750868,E=6750875,C=156)
	CT(22507)=(S=6750876,E=6750879,C=1)
	CT(22508)=(S=6750880,E=6750883,C=2)
	CT(22509)=(S=6750884,E=6750887,C=1)
	CT(22510)=(S=6750888,E=6750899,C=0)
	CT(22511)=(S=6750900,E=6750903,C=7)
	CT(22512)=(S=6750904,E=6750907,C=90)
	CT(22513)=(S=6750908,E=6750919,C=0)
	CT(22514)=(S=6750920,E=6750935,C=1)
	CT(22515)=(S=6750936,E=6750939,C=9)
	CT(22516)=(S=6750940,E=6750951,C=11)
	CT(22517)=(S=6750952,E=6750963,C=4)
	CT(22518)=(S=6750964,E=6750995,C=2)
	CT(22519)=(S=6750996,E=6750999,C=5)
	CT(22520)=(S=6751000,E=6751003,C=8)
	CT(22521)=(S=6751008,E=6751011,C=4)
	CT(22522)=(S=6751012,E=6751015,C=6)
	CT(22523)=(S=6751016,E=6751019,C=4)
	CT(22524)=(S=6751020,E=6751023,C=74)
	CT(22525)=(S=6751024,E=6751031,C=2)
	CT(22526)=(S=6751032,E=6751035,C=74)
	CT(22527)=(S=6751036,E=6751039,C=7)
	CT(22528)=(S=6751040,E=6751043,C=3)
	CT(22529)=(S=6751044,E=6751047,C=74)
	CT(22530)=(S=6751048,E=6751051,C=5)
	CT(22531)=(S=6751052,E=6751055,C=74)
	CT(22532)=(S=6751056,E=6751059,C=10)
	CT(22533)=(S=6751060,E=6751119,C=1)
	CT(22534)=(S=6751120,E=6751123,C=0)
	CT(22535)=(S=6751124,E=6751135,C=1)
	CT(22536)=(S=6751136,E=6751139,C=0)
	CT(22537)=(S=6751140,E=6751143,C=7)
	CT(22538)=(S=6751144,E=6751147,C=76)
	CT(22539)=(S=6751148,E=6751151,C=5)
	CT(22540)=(S=6751152,E=6751155,C=3)
	CT(22541)=(S=6751156,E=6751167,C=2)
	CT(22542)=(S=6751168,E=6751171,C=9)
	CT(22543)=(S=6751172,E=6751175,C=0)
	CT(22544)=(S=6751176,E=6751179,C=7)
	CT(22545)=(S=6751180,E=6751183,C=4)
	CT(22546)=(S=6751184,E=6751191,C=74)
	CT(22547)=(S=6751192,E=6751195,C=0)
	CT(22548)=(S=6751196,E=6751199,C=74)
	CT(22549)=(S=6751200,E=6751203,C=84)
	CT(22550)=(S=6751204,E=6751207,C=4)
	CT(22551)=(S=6751208,E=6751211,C=73)
	CT(22552)=(S=6751212,E=6751215,C=0)
	CT(22553)=(S=6751216,E=6751219,C=159)
	CT(22554)=(S=6751220,E=6751231,C=11)
	CT(22555)=(S=6751232,E=6751235,C=74)
	CT(22556)=(S=6751236,E=6751239,C=5)
	CT(22557)=(S=6751240,E=6751243,C=2)
	CT(22558)=(S=6751244,E=6751247,C=7)
	CT(22559)=(S=6751248,E=6751251,C=0)
	CT(22560)=(S=6751260,E=6751263,C=5)
	CT(22561)=(S=6751264,E=6751267,C=2)
	CT(22562)=(S=6751268,E=6751271,C=7)
	CT(22563)=(S=6751272,E=6751275,C=2)
	CT(22564)=(S=6751276,E=6751279,C=5)
	CT(22565)=(S=6751280,E=6751283,C=6)
	CT(22566)=(S=6751284,E=6751287,C=74)
	CT(22567)=(S=6751288,E=6751291,C=1)
	CT(22568)=(S=6751292,E=6751295,C=0)
	CT(22569)=(S=6751296,E=6751299,C=84)
	CT(22570)=(S=6751300,E=6751303,C=2)
	CT(22571)=(S=6751304,E=6751307,C=164)
	CT(22572)=(S=6751308,E=6751315,C=2)
	CT(22573)=(S=6751316,E=6751319,C=0)
	CT(22574)=(S=6751320,E=6751323,C=5)
	CT(22575)=(S=6751324,E=6751327,C=77)
	CT(22576)=(S=6751328,E=6751331,C=7)
	CT(22577)=(S=6751332,E=6751335,C=0)
	CT(22578)=(S=6751336,E=6751339,C=9)
	CT(22579)=(S=6751340,E=6751343,C=7)
	CT(22580)=(S=6751344,E=6751345,C=8)
	CT(22581)=(S=6751346,E=6751347,C=7)
	CT(22582)=(S=6751348,E=6751351,C=84)
	CT(22583)=(S=6751352,E=6751352,C=0)
	CT(22584)=(S=6751353,E=6751353,C=73)
	CT(22585)=(S=6751354,E=6751355,C=0)
	CT(22586)=(S=6751356,E=6751359,C=7)
	CT(22587)=(S=6751360,E=6751363,C=11)
	CT(22588)=(S=6751364,E=6751366,C=0)
	CT(22589)=(S=6751367,E=6751367,C=75)
	CT(22590)=(S=6751368,E=6751371,C=73)
	CT(22591)=(S=6751372,E=6751375,C=5)
	CT(22592)=(S=6751376,E=6751379,C=84)
	CT(22593)=(S=6751380,E=6751383,C=6)
	CT(22594)=(S=6751384,E=6751387,C=73)
	CT(22595)=(S=6751388,E=6751391,C=3)
	CT(22596)=(S=6751392,E=6751395,C=0)
	CT(22597)=(S=6751396,E=6751399,C=74)
	CT(22598)=(S=6751400,E=6751403,C=1)
	CT(22599)=(S=6751404,E=6751405,C=0)
	CT(22600)=(S=6751406,E=6751407,C=74)
	CT(22601)=(S=6751408,E=6751415,C=6)
	CT(22602)=(S=6751416,E=6751419,C=1)
	CT(22603)=(S=6751420,E=6751423,C=157)
	CT(22604)=(S=6751424,E=6751427,C=0)
	CT(22605)=(S=6751428,E=6751431,C=7)
	CT(22606)=(S=6751432,E=6751435,C=2)
	CT(22607)=(S=6751436,E=6751436,C=4)
	CT(22608)=(S=6751437,E=6751437,C=0)
	CT(22609)=(S=6751438,E=6751439,C=74)
	CT(22610)=(S=6751440,E=6751443,C=4)
	CT(22611)=(S=6751444,E=6751447,C=0)
	CT(22612)=(S=6751448,E=6751451,C=3)
	CT(22613)=(S=6751452,E=6751455,C=4)
	CT(22614)=(S=6751460,E=6751463,C=3)
	CT(22615)=(S=6751464,E=6751464,C=7)
	CT(22616)=(S=6751465,E=6751465,C=4)
	CT(22617)=(S=6751466,E=6751471,C=0)
	CT(22618)=(S=6751472,E=6751475,C=74)
	CT(22619)=(S=6751476,E=6751479,C=8)
	CT(22620)=(S=6751480,E=6751487,C=4)
	CT(22621)=(S=6751488,E=6751495,C=10)
	CT(22622)=(S=6751496,E=6751499,C=7)
	CT(22623)=(S=6751500,E=6751503,C=8)
	CT(22624)=(S=6751504,E=6751507,C=4)
	CT(22625)=(S=6751508,E=6751511,C=2)
	CT(22626)=(S=6751512,E=6751515,C=3)
	CT(22627)=(S=6751516,E=6751516,C=73)
	CT(22628)=(S=6751517,E=6751517,C=0)
	CT(22629)=(S=6751518,E=6751519,C=11)
	CT(22630)=(S=6751520,E=6751523,C=9)
	CT(22631)=(S=6751524,E=6751527,C=1)
	CT(22632)=(S=6751528,E=6751531,C=2)
	CT(22633)=(S=6751532,E=6751535,C=7)
	CT(22634)=(S=6751536,E=6751539,C=74)
	CT(22635)=(S=6751540,E=6751547,C=1)
	CT(22636)=(S=6751548,E=6751551,C=10)
	CT(22637)=(S=6751552,E=6751555,C=2)
	CT(22638)=(S=6751556,E=6751557,C=0)
	CT(22639)=(S=6751558,E=6751558,C=4)
	CT(22640)=(S=6751559,E=6751563,C=73)
	CT(22641)=(S=6751564,E=6751565,C=0)
	CT(22642)=(S=6751566,E=6751566,C=4)
	CT(22643)=(S=6751568,E=6751571,C=73)
	CT(22644)=(S=6751572,E=6751575,C=4)
	CT(22645)=(S=6751576,E=6751587,C=0)
	CT(22646)=(S=6751588,E=6751591,C=9)
	CT(22647)=(S=6751593,E=6751593,C=5)
	CT(22648)=(S=6751595,E=6751595,C=74)
	CT(22649)=(S=6751596,E=6751599,C=73)
	CT(22650)=(S=6751600,E=6751603,C=4)
	CT(22651)=(S=6751604,E=6751607,C=2)
	CT(22652)=(S=6751608,E=6751611,C=3)
	CT(22653)=(S=6751612,E=6751615,C=86)
	CT(22654)=(S=6751616,E=6751619,C=6)
	CT(22655)=(S=6751620,E=6751623,C=4)
	CT(22656)=(S=6751624,E=6751627,C=77)
	CT(22657)=(S=6751628,E=6751631,C=2)
	CT(22658)=(S=6751632,E=6751635,C=6)
	CT(22659)=(S=6751636,E=6751636,C=74)
	CT(22660)=(S=6751637,E=6751637,C=8)
	CT(22661)=(S=6751638,E=6751638,C=75)
	CT(22662)=(S=6751639,E=6751639,C=7)
	CT(22663)=(S=6751644,E=6751647,C=73)
	CT(22664)=(S=6751648,E=6751651,C=2)
	CT(22665)=(S=6751652,E=6751655,C=0)
	CT(22666)=(S=6751660,E=6751663,C=89)
	CT(22667)=(S=6751664,E=6751667,C=2)
	CT(22668)=(S=6751668,E=6751671,C=5)
	CT(22669)=(S=6751672,E=6751679,C=4)
	CT(22670)=(S=6751684,E=6751685,C=89)
	CT(22671)=(S=6751686,E=6751686,C=8)
	CT(22672)=(S=6751687,E=6751687,C=7)
	CT(22673)=(S=6751688,E=6751691,C=4)
	CT(22674)=(S=6751692,E=6751699,C=11)
	CT(22675)=(S=6751700,E=6751702,C=0)
	CT(22676)=(S=6751703,E=6751703,C=74)
	CT(22677)=(S=6751704,E=6751707,C=4)
	CT(22678)=(S=6751708,E=6751711,C=8)
	CT(22679)=(S=6751712,E=6751715,C=73)
	CT(22680)=(S=6751716,E=6751717,C=75)
	CT(22681)=(S=6751720,E=6751723,C=84)
	CT(22682)=(S=6751724,E=6751727,C=5)
	CT(22683)=(S=6751728,E=6751731,C=7)
	CT(22684)=(S=6751732,E=6751735,C=4)
	CT(22685)=(S=6751736,E=6751739,C=2)
	CT(22686)=(S=6751740,E=6751743,C=1)
	CT(22687)=(S=6751744,E=6751745,C=74)
	CT(22688)=(S=6751746,E=6751746,C=4)
	CT(22689)=(S=6751748,E=6751751,C=8)
	CT(22690)=(S=6751752,E=6751755,C=86)
	CT(22691)=(S=6751756,E=6751759,C=0)
	CT(22692)=(S=6751760,E=6751767,C=2)
	CT(22693)=(S=6751768,E=6751769,C=0)
	CT(22694)=(S=6751770,E=6751771,C=4)
	CT(22695)=(S=6751772,E=6751775,C=73)
	CT(22696)=(S=6751776,E=6751777,C=4)
	CT(22697)=(S=6751778,E=6751778,C=74)
	CT(22698)=(S=6751779,E=6751779,C=0)
	CT(22699)=(S=6751780,E=6751787,C=74)
	CT(22700)=(S=6751788,E=6751791,C=2)
	CT(22701)=(S=6751792,E=6751795,C=8)
	CT(22702)=(S=6751796,E=6751799,C=74)
	CT(22703)=(S=6751800,E=6751807,C=2)
	CT(22704)=(S=6751808,E=6751811,C=73)
	CT(22705)=(S=6751812,E=6751815,C=2)
	CT(22706)=(S=6751816,E=6751819,C=6)
	CT(22707)=(S=6751820,E=6751823,C=1)
	CT(22708)=(S=6751824,E=6751827,C=6)
	CT(22709)=(S=6751828,E=6751831,C=8)
	CT(22710)=(S=6751832,E=6751832,C=4)
	CT(22711)=(S=6751833,E=6751833,C=73)
	CT(22712)=(S=6751836,E=6751839,C=2)
	CT(22713)=(S=6751840,E=6751843,C=10)
	CT(22714)=(S=6751844,E=6751847,C=6)
	CT(22715)=(S=6751849,E=6751849,C=0)
	CT(22716)=(S=6751850,E=6751851,C=7)
	CT(22717)=(S=6751856,E=6751859,C=2)
	CT(22718)=(S=6751860,E=6751860,C=0)
	CT(22719)=(S=6751861,E=6751862,C=74)
	CT(22720)=(S=6751863,E=6751863,C=4)
	CT(22721)=(S=6751868,E=6751875,C=8)
	CT(22722)=(S=6751876,E=6751879,C=4)
	CT(22723)=(S=6751880,E=6751887,C=8)
	CT(22724)=(S=6751888,E=6751891,C=73)
	CT(22725)=(S=6751892,E=6751895,C=7)
	CT(22726)=(S=6751896,E=6751899,C=8)
	CT(22727)=(S=6751900,E=6751903,C=4)
	CT(22728)=(S=6751904,E=6751905,C=8)
	CT(22729)=(S=6751906,E=6751906,C=3)
	CT(22730)=(S=6751907,E=6751907,C=5)
	CT(22731)=(S=6751908,E=6751908,C=4)
	CT(22732)=(S=6751909,E=6751909,C=3)
	CT(22733)=(S=6751910,E=6751910,C=8)
	CT(22734)=(S=6751911,E=6751915,C=4)
	CT(22735)=(S=6751916,E=6751919,C=6)
	CT(22736)=(S=6751920,E=6751923,C=8)
	CT(22737)=(S=6751924,E=6751924,C=74)
	CT(22738)=(S=6751925,E=6751925,C=10)
	CT(22739)=(S=6751926,E=6751927,C=3)
	CT(22740)=(S=6751928,E=6751931,C=4)
	CT(22741)=(S=6751932,E=6751935,C=0)
	CT(22742)=(S=6751936,E=6751939,C=8)
	CT(22743)=(S=6751940,E=6751943,C=5)
	CT(22744)=(S=6751944,E=6751951,C=74)
	CT(22745)=(S=6751952,E=6751955,C=0)
	CT(22746)=(S=6751956,E=6751957,C=73)
	CT(22747)=(S=6751958,E=6751958,C=74)
	CT(22748)=(S=6751959,E=6751963,C=0)
	CT(22749)=(S=6751964,E=6751967,C=1)
	CT(22750)=(S=6751968,E=6751971,C=2)
	CT(22751)=(S=6751976,E=6751979,C=73)
	CT(22752)=(S=6751980,E=6751983,C=5)
	CT(22753)=(S=6751984,E=6751987,C=2)
	CT(22754)=(S=6751988,E=6751991,C=5)
	CT(22755)=(S=6751992,E=6751995,C=10)
	CT(22756)=(S=6751996,E=6751999,C=0)
	CT(22757)=(S=6752000,E=6752001,C=74)
	CT(22758)=(S=6752002,E=6752002,C=7)
	CT(22759)=(S=6752003,E=6752003,C=4)
	CT(22760)=(S=6752004,E=6752007,C=1)
	CT(22761)=(S=6752008,E=6752011,C=7)
	CT(22762)=(S=6752012,E=6752015,C=74)
	CT(22763)=(S=6752016,E=6752019,C=3)
	CT(22764)=(S=6752020,E=6752023,C=2)
	CT(22765)=(S=6752024,E=6752027,C=86)
	CT(22766)=(S=6752028,E=6752031,C=1)
	CT(22767)=(S=6752032,E=6752035,C=6)
	CT(22768)=(S=6752036,E=6752043,C=11)
	CT(22769)=(S=6752044,E=6752051,C=0)
	CT(22770)=(S=6752052,E=6752053,C=74)
	CT(22771)=(S=6752054,E=6752055,C=8)
	CT(22772)=(S=6752056,E=6752059,C=3)
	CT(22773)=(S=6752060,E=6752063,C=77)
	CT(22774)=(S=6752064,E=6752067,C=4)
	CT(22775)=(S=6752068,E=6752071,C=5)
	CT(22776)=(S=6752072,E=6752075,C=0)
	CT(22777)=(S=6752076,E=6752079,C=77)
	CT(22778)=(S=6752080,E=6752083,C=4)
	CT(22779)=(S=6752084,E=6752087,C=74)
	CT(22780)=(S=6752088,E=6752091,C=0)
	CT(22781)=(S=6752092,E=6752095,C=2)
	CT(22782)=(S=6752096,E=6752097,C=73)
	CT(22783)=(S=6752098,E=6752099,C=0)
	CT(22784)=(S=6752100,E=6752103,C=159)
	CT(22785)=(S=6752104,E=6752107,C=89)
	CT(22786)=(S=6752108,E=6752111,C=4)
	CT(22787)=(S=6752112,E=6752115,C=84)
	CT(22788)=(S=6752116,E=6752119,C=2)
	CT(22789)=(S=6752124,E=6752127,C=7)
	CT(22790)=(S=6752128,E=6752131,C=4)
	CT(22791)=(S=6752132,E=6752135,C=2)
	CT(22792)=(S=6752136,E=6752139,C=0)
	CT(22793)=(S=6752144,E=6752144,C=86)
	CT(22794)=(S=6752145,E=6752145,C=4)
	CT(22795)=(S=6752146,E=6752146,C=10)
	CT(22796)=(S=6752147,E=6752147,C=0)
	CT(22797)=(S=6752148,E=6752151,C=4)
	CT(22798)=(S=6752152,E=6752155,C=2)
	CT(22799)=(S=6752156,E=6752159,C=5)
	CT(22800)=(S=6752160,E=6752163,C=8)
	CT(22801)=(S=6752164,E=6752167,C=0)
	CT(22802)=(S=6752168,E=6752171,C=73)
	CT(22803)=(S=6752172,E=6752172,C=11)
	CT(22804)=(S=6752173,E=6752173,C=74)
	CT(22805)=(S=6752174,E=6752175,C=11)
	CT(22806)=(S=6752176,E=6752176,C=74)
	CT(22807)=(S=6752177,E=6752177,C=11)
	CT(22808)=(S=6752178,E=6752179,C=74)
	CT(22809)=(S=6752180,E=6752183,C=4)
	CT(22810)=(S=6752184,E=6752187,C=74)
	CT(22811)=(S=6752188,E=6752195,C=0)
	CT(22812)=(S=6752196,E=6752196,C=11)
	CT(22813)=(S=6752197,E=6752197,C=160)
	CT(22814)=(S=6752198,E=6752199,C=2)
	CT(22815)=(S=6752200,E=6752203,C=7)
	CT(22816)=(S=6752204,E=6752207,C=4)
	CT(22817)=(S=6752208,E=6752211,C=2)
	CT(22818)=(S=6752212,E=6752223,C=1)
	CT(22819)=(S=6752224,E=6752224,C=10)
	CT(22820)=(S=6752225,E=6752231,C=74)
	CT(22821)=(S=6752232,E=6752235,C=0)
	CT(22822)=(S=6752236,E=6752239,C=2)
	CT(22823)=(S=6752240,E=6752243,C=8)
	CT(22824)=(S=6752244,E=6752247,C=6)
	CT(22825)=(S=6752248,E=6752251,C=84)
	CT(22826)=(S=6752252,E=6752255,C=161)
	CT(22827)=(S=6752260,E=6752267,C=1)
	CT(22828)=(S=6752268,E=6752268,C=74)
	CT(22829)=(S=6752269,E=6752269,C=11)
	CT(22830)=(S=6752270,E=6752271,C=77)
	CT(22831)=(S=6752272,E=6752275,C=73)
	CT(22832)=(S=6752276,E=6752279,C=2)
	CT(22833)=(S=6752280,E=6752283,C=5)
	CT(22834)=(S=6752284,E=6752285,C=0)
	CT(22835)=(S=6752286,E=6752287,C=7)
	CT(22836)=(S=6752288,E=6752291,C=1)
	CT(22837)=(S=6752292,E=6752303,C=4)
	CT(22838)=(S=6752304,E=6752307,C=2)
	CT(22839)=(S=6752308,E=6752311,C=1)
	CT(22840)=(S=6752312,E=6752315,C=74)
	CT(22841)=(S=6752316,E=6752316,C=92)
	CT(22842)=(S=6752317,E=6752317,C=0)
	CT(22843)=(S=6752318,E=6752318,C=4)
	CT(22844)=(S=6752319,E=6752319,C=74)
	CT(22845)=(S=6752320,E=6752323,C=2)
	CT(22846)=(S=6752328,E=6752329,C=73)
	CT(22847)=(S=6752330,E=6752335,C=74)
	CT(22848)=(S=6752336,E=6752339,C=73)
	CT(22849)=(S=6752340,E=6752351,C=2)
	CT(22850)=(S=6752352,E=6752355,C=74)
	CT(22851)=(S=6752356,E=6752359,C=6)
	CT(22852)=(S=6752360,E=6752363,C=9)
	CT(22853)=(S=6752364,E=6752367,C=1)
	CT(22854)=(S=6752368,E=6752371,C=77)
	CT(22855)=(S=6752372,E=6752375,C=4)
	CT(22856)=(S=6752376,E=6752376,C=7)
	CT(22857)=(S=6752378,E=6752378,C=74)
	CT(22858)=(S=6752379,E=6752383,C=4)
	CT(22859)=(S=6752384,E=6752387,C=3)
	CT(22860)=(S=6752388,E=6752395,C=0)
	CT(22861)=(S=6752396,E=6752399,C=73)
	CT(22862)=(S=6752400,E=6752403,C=5)
	CT(22863)=(S=6752404,E=6752407,C=4)
	CT(22864)=(S=6752408,E=6752411,C=7)
	CT(22865)=(S=6752412,E=6752415,C=1)
	CT(22866)=(S=6752416,E=6752419,C=5)
	CT(22867)=(S=6752420,E=6752423,C=4)
	CT(22868)=(S=6752424,E=6752431,C=0)
	CT(22869)=(S=6752432,E=6752439,C=7)
	CT(22870)=(S=6752440,E=6752443,C=2)
	CT(22871)=(S=6752444,E=6752445,C=4)
	CT(22872)=(S=6752446,E=6752446,C=2)
	CT(22873)=(S=6752447,E=6752447,C=8)
	CT(22874)=(S=6752448,E=6752451,C=4)
	CT(22875)=(S=6752452,E=6752455,C=2)
	CT(22876)=(S=6752456,E=6752463,C=1)
	CT(22877)=(S=6752464,E=6752467,C=3)
	CT(22878)=(S=6752468,E=6752469,C=7)
	CT(22879)=(S=6752470,E=6752470,C=77)
	CT(22880)=(S=6752471,E=6752475,C=4)
	CT(22881)=(S=6752476,E=6752479,C=1)
	CT(22882)=(S=6752480,E=6752484,C=74)
	CT(22883)=(S=6752485,E=6752485,C=0)
	CT(22884)=(S=6752486,E=6752486,C=6)
	CT(22885)=(S=6752487,E=6752487,C=77)
	CT(22886)=(S=6752488,E=6752491,C=73)
	CT(22887)=(S=6752492,E=6752493,C=0)
	CT(22888)=(S=6752494,E=6752494,C=74)
	CT(22889)=(S=6752495,E=6752495,C=0)
	CT(22890)=(S=6752496,E=6752499,C=2)
	CT(22891)=(S=6752500,E=6752503,C=74)
	CT(22892)=(S=6752504,E=6752507,C=4)
	CT(22893)=(S=6752508,E=6752511,C=73)
	CT(22894)=(S=6752512,E=6752519,C=11)
	CT(22895)=(S=6752520,E=6752523,C=1)
	CT(22896)=(S=6752524,E=6752527,C=4)
	CT(22897)=(S=6752528,E=6752531,C=0)
	CT(22898)=(S=6752534,E=6752534,C=74)
	CT(22899)=(S=6752535,E=6752535,C=77)
	CT(22900)=(S=6752540,E=6752543,C=2)
	CT(22901)=(S=6752544,E=6752547,C=6)
	CT(22902)=(S=6752548,E=6752551,C=74)
	CT(22903)=(S=6752552,E=6752555,C=73)
	CT(22904)=(S=6752556,E=6752559,C=74)
	CT(22905)=(S=6752562,E=6752562,C=192)
	CT(22906)=(S=6752564,E=6752583,C=0)
	CT(22907)=(S=6752584,E=6752587,C=4)
	CT(22908)=(S=6752588,E=6752599,C=11)
	CT(22909)=(S=6752600,E=6752603,C=92)
	CT(22910)=(S=6752604,E=6752611,C=2)
	CT(22911)=(S=6752612,E=6752615,C=7)
	CT(22912)=(S=6752616,E=6752617,C=84)
	CT(22913)=(S=6752618,E=6752618,C=73)
	CT(22914)=(S=6752624,E=6752627,C=84)
	CT(22915)=(S=6752628,E=6752631,C=9)
	CT(22916)=(S=6752632,E=6752635,C=2)
	CT(22917)=(S=6752636,E=6752639,C=74)
	CT(22918)=(S=6752640,E=6752643,C=6)
	CT(22919)=(S=6752648,E=6752651,C=84)
	CT(22920)=(S=6752652,E=6752655,C=76)
	CT(22921)=(S=6752656,E=6752659,C=74)
	CT(22922)=(S=6752660,E=6752663,C=5)
	CT(22923)=(S=6752664,E=6752667,C=1)
	CT(22924)=(S=6752672,E=6752675,C=0)
	CT(22925)=(S=6752676,E=6752679,C=2)
	CT(22926)=(S=6752680,E=6752683,C=0)
	CT(22927)=(S=6752684,E=6752692,C=2)
	CT(22928)=(S=6752693,E=6752693,C=84)
	CT(22929)=(S=6752694,E=6752694,C=77)
	CT(22930)=(S=6752696,E=6752699,C=84)
	CT(22931)=(S=6752700,E=6752703,C=86)
	CT(22932)=(S=6752708,E=6752727,C=11)
	CT(22933)=(S=6752728,E=6752731,C=73)
	CT(22934)=(S=6752732,E=6752733,C=84)
	CT(22935)=(S=6752734,E=6752735,C=4)
	CT(22936)=(S=6752736,E=6752736,C=84)
	CT(22937)=(S=6752737,E=6752737,C=4)
	CT(22938)=(S=6752738,E=6752738,C=76)
	CT(22939)=(S=6752739,E=6752739,C=74)
	CT(22940)=(S=6752740,E=6752743,C=162)
	CT(22941)=(S=6752744,E=6752747,C=4)
	CT(22942)=(S=6752748,E=6752751,C=73)
	CT(22943)=(S=6752752,E=6752755,C=0)
	CT(22944)=(S=6752756,E=6752759,C=8)
	CT(22945)=(S=6752760,E=6752771,C=1)
	CT(22946)=(S=6752772,E=6752775,C=9)
	CT(22947)=(S=6752776,E=6752776,C=0)
	CT(22948)=(S=6752777,E=6752777,C=4)
	CT(22949)=(S=6752779,E=6752779,C=0)
	CT(22950)=(S=6752780,E=6752783,C=8)
	CT(22951)=(S=6752784,E=6752787,C=1)
	CT(22952)=(S=6752788,E=6752791,C=92)
	CT(22953)=(S=6752792,E=6752795,C=4)
	CT(22954)=(S=6752796,E=6752799,C=75)
	CT(22955)=(S=6752800,E=6752803,C=0)
	CT(22956)=(S=6752804,E=6752807,C=2)
	CT(22957)=(S=6752808,E=6752811,C=5)
	CT(22958)=(S=6752816,E=6752819,C=0)
	CT(22959)=(S=6752820,E=6752823,C=84)
	CT(22960)=(S=6752824,E=6752824,C=74)
	CT(22961)=(S=6752825,E=6752825,C=0)
	CT(22962)=(S=6752826,E=6752835,C=74)
	CT(22963)=(S=6752836,E=6752839,C=2)
	CT(22964)=(S=6752840,E=6752843,C=0)
	CT(22965)=(S=6752844,E=6752847,C=84)
	CT(22966)=(S=6752848,E=6752849,C=74)
	CT(22967)=(S=6752850,E=6752851,C=0)
	CT(22968)=(S=6752852,E=6752855,C=1)
	CT(22969)=(S=6752856,E=6752859,C=11)
	CT(22970)=(S=6752860,E=6752863,C=6)
	CT(22971)=(S=6752864,E=6752867,C=74)
	CT(22972)=(S=6752868,E=6752871,C=7)
	CT(22973)=(S=6752872,E=6752875,C=74)
	CT(22974)=(S=6752876,E=6752876,C=8)
	CT(22975)=(S=6752878,E=6752878,C=74)
	CT(22976)=(S=6752879,E=6752879,C=1)
	CT(22977)=(S=6752880,E=6752883,C=2)
	CT(22978)=(S=6752884,E=6752887,C=4)
	CT(22979)=(S=6752888,E=6752891,C=74)
	CT(22980)=(S=6752892,E=6752893,C=8)
	CT(22981)=(S=6752894,E=6752894,C=0)
	CT(22982)=(S=6752895,E=6752895,C=4)
	CT(22983)=(S=6752896,E=6752899,C=74)
	CT(22984)=(S=6752900,E=6752903,C=4)
	CT(22985)=(S=6752904,E=6752907,C=74)
	CT(22986)=(S=6752908,E=6752911,C=1)
	CT(22987)=(S=6752912,E=6752919,C=74)
	CT(22988)=(S=6752920,E=6752923,C=10)
	CT(22989)=(S=6752924,E=6752931,C=5)
	CT(22990)=(S=6752932,E=6752935,C=2)
	CT(22991)=(S=6752936,E=6752936,C=4)
	CT(22992)=(S=6752937,E=6752939,C=74)
	CT(22993)=(S=6752944,E=6752947,C=10)
	CT(22994)=(S=6752948,E=6752951,C=4)
	CT(22995)=(S=6752952,E=6752955,C=74)
	CT(22996)=(S=6752956,E=6752959,C=4)
	CT(22997)=(S=6752960,E=6752963,C=2)
	CT(22998)=(S=6752964,E=6752964,C=5)
	CT(22999)=(S=6752965,E=6752965,C=8)
	CT(23000)=(S=6752966,E=6752966,C=5)
	CT(23001)=(S=6752967,E=6752967,C=8)
	CT(23002)=(S=6752968,E=6752971,C=10)
	CT(23003)=(S=6752972,E=6752975,C=9)
	CT(23004)=(S=6752976,E=6752979,C=4)
	CT(23005)=(S=6752980,E=6752983,C=11)
	CT(23006)=(S=6752984,E=6752987,C=6)
	CT(23007)=(S=6752988,E=6752989,C=7)
	CT(23008)=(S=6752991,E=6752991,C=74)
	CT(23009)=(S=6752992,E=6752995,C=4)
	CT(23010)=(S=6752996,E=6752999,C=3)
	CT(23011)=(S=6753000,E=6753000,C=0)
	CT(23012)=(S=6753001,E=6753001,C=73)
	CT(23013)=(S=6753002,E=6753003,C=4)
	CT(23014)=(S=6753008,E=6753015,C=2)
	CT(23015)=(S=6753016,E=6753020,C=4)
	CT(23016)=(S=6753021,E=6753021,C=74)
	CT(23017)=(S=6753022,E=6753023,C=10)
	CT(23018)=(S=6753028,E=6753031,C=2)
	CT(23019)=(S=6753032,E=6753035,C=0)
	CT(23020)=(S=6753036,E=6753039,C=3)
	CT(23021)=(S=6753044,E=6753044,C=4)
	CT(23022)=(S=6753046,E=6753047,C=74)
	CT(23023)=(S=6753048,E=6753051,C=6)
	CT(23024)=(S=6753052,E=6753055,C=74)
	CT(23025)=(S=6753056,E=6753059,C=156)
	CT(23026)=(S=6753060,E=6753063,C=8)
	CT(23027)=(S=6753064,E=6753067,C=10)
	CT(23028)=(S=6753068,E=6753071,C=6)
	CT(23029)=(S=6753072,E=6753075,C=7)
	CT(23030)=(S=6753076,E=6753079,C=73)
	CT(23031)=(S=6753080,E=6753083,C=6)
	CT(23032)=(S=6753084,E=6753087,C=77)
	CT(23033)=(S=6753088,E=6753091,C=10)
	CT(23034)=(S=6753092,E=6753095,C=77)
	CT(23035)=(S=6753098,E=6753099,C=74)
	CT(23036)=(S=6753100,E=6753103,C=0)
	CT(23037)=(S=6753104,E=6753111,C=4)
	CT(23038)=(S=6753112,E=6753119,C=8)
	CT(23039)=(S=6753120,E=6753123,C=74)
	CT(23040)=(S=6753124,E=6753129,C=8)
	CT(23041)=(S=6753130,E=6753131,C=74)
	CT(23042)=(S=6753132,E=6753135,C=8)
	CT(23043)=(S=6753136,E=6753139,C=10)
	CT(23044)=(S=6753140,E=6753143,C=4)
	CT(23045)=(S=6753144,E=6753147,C=2)
	CT(23046)=(S=6753148,E=6753151,C=73)
	CT(23047)=(S=6753152,E=6753155,C=6)
	CT(23048)=(S=6753156,E=6753159,C=74)
	CT(23049)=(S=6753160,E=6753163,C=84)
	CT(23050)=(S=6753164,E=6753165,C=7)
	CT(23051)=(S=6753166,E=6753166,C=0)
	CT(23052)=(S=6753167,E=6753167,C=7)
	CT(23053)=(S=6753168,E=6753171,C=0)
	CT(23054)=(S=6753172,E=6753173,C=74)
	CT(23055)=(S=6753174,E=6753175,C=7)
	CT(23056)=(S=6753176,E=6753179,C=4)
	CT(23057)=(S=6753180,E=6753183,C=2)
	CT(23058)=(S=6753184,E=6753191,C=0)
	CT(23059)=(S=6753196,E=6753199,C=11)
	CT(23060)=(S=6753200,E=6753201,C=74)
	CT(23061)=(S=6753202,E=6753202,C=73)
	CT(23062)=(S=6753203,E=6753203,C=74)
	CT(23063)=(S=6753204,E=6753207,C=1)
	CT(23064)=(S=6753208,E=6753209,C=0)
	CT(23065)=(S=6753210,E=6753211,C=74)
	CT(23066)=(S=6753212,E=6753215,C=7)
	CT(23067)=(S=6753216,E=6753219,C=92)
	CT(23068)=(S=6753220,E=6753223,C=2)
	CT(23069)=(S=6753224,E=6753227,C=7)
	CT(23070)=(S=6753228,E=6753231,C=0)
	CT(23071)=(S=6753232,E=6753235,C=2)
	CT(23072)=(S=6753236,E=6753239,C=0)
	CT(23073)=(S=6753240,E=6753243,C=86)
	CT(23074)=(S=6753244,E=6753244,C=7)
	CT(23075)=(S=6753245,E=6753245,C=2)
	CT(23076)=(S=6753246,E=6753247,C=74)
	CT(23077)=(S=6753248,E=6753251,C=4)
	CT(23078)=(S=6753252,E=6753255,C=8)
	CT(23079)=(S=6753256,E=6753259,C=4)
	CT(23080)=(S=6753260,E=6753263,C=2)
	CT(23081)=(S=6753264,E=6753267,C=73)
	CT(23082)=(S=6753268,E=6753271,C=8)
	CT(23083)=(S=6753272,E=6753275,C=6)
	CT(23084)=(S=6753276,E=6753279,C=74)
	CT(23085)=(S=6753281,E=6753281,C=4)
	CT(23086)=(S=6753282,E=6753282,C=7)
	CT(23087)=(S=6753283,E=6753287,C=0)
	CT(23088)=(S=6753288,E=6753290,C=73)
	CT(23089)=(S=6753291,E=6753291,C=0)
	CT(23090)=(S=6753292,E=6753295,C=74)
	CT(23091)=(S=6753296,E=6753299,C=2)
	CT(23092)=(S=6753300,E=6753301,C=74)
	CT(23093)=(S=6753302,E=6753302,C=0)
	CT(23094)=(S=6753303,E=6753303,C=74)
	CT(23095)=(S=6753304,E=6753307,C=4)
	CT(23096)=(S=6753308,E=6753311,C=74)
	CT(23097)=(S=6753312,E=6753315,C=1)
	CT(23098)=(S=6753316,E=6753319,C=0)
	CT(23099)=(S=6753320,E=6753323,C=77)
	CT(23100)=(S=6753324,E=6753327,C=84)
	CT(23101)=(S=6753332,E=6753335,C=2)
	CT(23102)=(S=6753336,E=6753337,C=0)
	CT(23103)=(S=6753338,E=6753338,C=77)
	CT(23104)=(S=6753339,E=6753339,C=0)
	CT(23105)=(S=6753340,E=6753343,C=8)
	CT(23106)=(S=6753344,E=6753347,C=5)
	CT(23107)=(S=6753348,E=6753351,C=1)
	CT(23108)=(S=6753352,E=6753355,C=84)
	CT(23109)=(S=6753360,E=6753361,C=74)
	CT(23110)=(S=6753362,E=6753363,C=84)
	CT(23111)=(S=6753364,E=6753364,C=74)
	CT(23112)=(S=6753365,E=6753367,C=0)
	CT(23113)=(S=6753368,E=6753371,C=10)
	CT(23114)=(S=6753376,E=6753379,C=89)
	CT(23115)=(S=6753380,E=6753383,C=0)
	CT(23116)=(S=6753384,E=6753387,C=11)
	CT(23117)=(S=6753388,E=6753391,C=0)
	CT(23118)=(S=6753392,E=6753395,C=74)
	CT(23119)=(S=6753396,E=6753399,C=4)
	CT(23120)=(S=6753400,E=6753403,C=77)
	CT(23121)=(S=6753404,E=6753407,C=4)
	CT(23122)=(S=6753408,E=6753411,C=0)
	CT(23123)=(S=6753412,E=6753415,C=4)
	CT(23124)=(S=6753416,E=6753419,C=1)
	CT(23125)=(S=6753420,E=6753423,C=8)
	CT(23126)=(S=6753424,E=6753427,C=5)
	CT(23127)=(S=6753428,E=6753431,C=8)
	CT(23128)=(S=6753432,E=6753435,C=5)
	CT(23129)=(S=6753436,E=6753437,C=4)
	CT(23130)=(S=6753438,E=6753438,C=2)
	CT(23131)=(S=6753440,E=6753443,C=86)
	CT(23132)=(S=6753444,E=6753444,C=74)
	CT(23133)=(S=6753445,E=6753445,C=0)
	CT(23134)=(S=6753446,E=6753446,C=84)
	CT(23135)=(S=6753447,E=6753447,C=74)
	CT(23136)=(S=6753448,E=6753451,C=4)
	CT(23137)=(S=6753452,E=6753459,C=84)
	CT(23138)=(S=6753460,E=6753463,C=2)
	CT(23139)=(S=6753464,E=6753467,C=1)
	CT(23140)=(S=6753468,E=6753471,C=0)
	CT(23141)=(S=6753472,E=6753475,C=4)
	CT(23142)=(S=6753480,E=6753483,C=0)
	CT(23143)=(S=6753484,E=6753487,C=84)
	CT(23144)=(S=6753488,E=6753489,C=157)
	CT(23145)=(S=6753490,E=6753490,C=7)
	CT(23146)=(S=6753491,E=6753491,C=4)
	CT(23147)=(S=6753492,E=6753495,C=7)
	CT(23148)=(S=6753496,E=6753499,C=8)
	CT(23149)=(S=6753500,E=6753503,C=74)
	CT(23150)=(S=6753504,E=6753507,C=2)
	CT(23151)=(S=6753508,E=6753511,C=84)
	CT(23152)=(S=6753512,E=6753515,C=1)
	CT(23153)=(S=6753516,E=6753519,C=84)
	CT(23154)=(S=6753520,E=6753523,C=74)
	CT(23155)=(S=6753524,E=6753524,C=7)
	CT(23156)=(S=6753525,E=6753525,C=4)
	CT(23157)=(S=6753528,E=6753535,C=6)
	CT(23158)=(S=6753538,E=6753539,C=4)
	CT(23159)=(S=6753544,E=6753547,C=73)
	CT(23160)=(S=6753552,E=6753563,C=2)
	CT(23161)=(S=6753564,E=6753567,C=3)
	CT(23162)=(S=6753572,E=6753574,C=74)
	CT(23163)=(S=6753575,E=6753575,C=73)
	CT(23164)=(S=6753576,E=6753583,C=2)
	CT(23165)=(S=6753584,E=6753587,C=8)
	CT(23166)=(S=6753588,E=6753591,C=6)
	CT(23167)=(S=6753592,E=6753595,C=0)
	CT(23168)=(S=6753596,E=6753599,C=2)
	CT(23169)=(S=6753600,E=6753603,C=89)
	CT(23170)=(S=6753604,E=6753607,C=0)
	CT(23171)=(S=6753611,E=6753611,C=5)
	CT(23172)=(S=6753612,E=6753615,C=11)
	CT(23173)=(S=6753616,E=6753623,C=0)
	CT(23174)=(S=6753624,E=6753627,C=154)
	CT(23175)=(S=6753628,E=6753631,C=2)
	CT(23176)=(S=6753632,E=6753635,C=4)
	CT(23177)=(S=6753636,E=6753639,C=0)
	CT(23178)=(S=6753644,E=6753647,C=4)
	CT(23179)=(S=6753652,E=6753655,C=73)
	CT(23180)=(S=6753656,E=6753659,C=5)
	CT(23181)=(S=6753660,E=6753663,C=1)
	CT(23182)=(S=6753664,E=6753667,C=7)
	CT(23183)=(S=6753670,E=6753670,C=10)
	CT(23184)=(S=6753671,E=6753671,C=4)
	CT(23185)=(S=6753672,E=6753679,C=2)
	CT(23186)=(S=6753680,E=6753683,C=1)
	CT(23187)=(S=6753684,E=6753687,C=84)
	CT(23188)=(S=6753688,E=6753691,C=2)
	CT(23189)=(S=6753692,E=6753695,C=84)
	CT(23190)=(S=6753696,E=6753699,C=6)
	CT(23191)=(S=6753700,E=6753707,C=2)
	CT(23192)=(S=6753708,E=6753711,C=7)
	CT(23193)=(S=6753712,E=6753713,C=8)
	CT(23194)=(S=6753714,E=6753716,C=4)
	CT(23195)=(S=6753717,E=6753717,C=74)
	CT(23196)=(S=6753718,E=6753718,C=7)
	CT(23197)=(S=6753719,E=6753719,C=4)
	CT(23198)=(S=6753722,E=6753723,C=73)
	CT(23199)=(S=6753724,E=6753727,C=8)
	CT(23200)=(S=6753728,E=6753731,C=2)
	CT(23201)=(S=6753732,E=6753735,C=1)
	CT(23202)=(S=6753736,E=6753739,C=74)
	CT(23203)=(S=6753740,E=6753747,C=2)
	CT(23204)=(S=6753748,E=6753751,C=3)
	CT(23205)=(S=6753752,E=6753754,C=0)
	CT(23206)=(S=6753755,E=6753755,C=4)
	CT(23207)=(S=6753760,E=6753763,C=84)
	CT(23208)=(S=6753764,E=6753767,C=3)
	CT(23209)=(S=6753768,E=6753771,C=8)
	CT(23210)=(S=6753772,E=6753775,C=84)
	CT(23211)=(S=6753776,E=6753779,C=4)
	CT(23212)=(S=6753780,E=6753783,C=1)
	CT(23213)=(S=6753784,E=6753791,C=2)
	CT(23214)=(S=6753793,E=6753793,C=0)
	CT(23215)=(S=6753794,E=6753794,C=4)
	CT(23216)=(S=6753795,E=6753795,C=73)
	CT(23217)=(S=6753796,E=6753799,C=0)
	CT(23218)=(S=6753800,E=6753803,C=3)
	CT(23219)=(S=6753804,E=6753807,C=2)
	CT(23220)=(S=6753808,E=6753813,C=74)
	CT(23221)=(S=6753814,E=6753815,C=0)
	CT(23222)=(S=6753820,E=6753823,C=5)
	CT(23223)=(S=6753828,E=6753831,C=92)
	CT(23224)=(S=6753832,E=6753835,C=73)
	CT(23225)=(S=6753836,E=6753837,C=74)
	CT(23226)=(S=6753838,E=6753838,C=4)
	CT(23227)=(S=6753839,E=6753847,C=0)
	CT(23228)=(S=6753848,E=6753855,C=10)
	CT(23229)=(S=6753856,E=6753859,C=0)
	CT(23230)=(S=6753860,E=6753863,C=73)
	CT(23231)=(S=6753864,E=6753865,C=84)
	CT(23232)=(S=6753868,E=6753871,C=7)
	CT(23233)=(S=6753876,E=6753879,C=1)
	CT(23234)=(S=6753880,E=6753883,C=76)
	CT(23235)=(S=6753884,E=6753887,C=0)
	CT(23236)=(S=6753888,E=6753890,C=4)
	CT(23237)=(S=6753891,E=6753891,C=7)
	CT(23238)=(S=6753900,E=6753900,C=74)
	CT(23239)=(S=6753901,E=6753901,C=84)
	CT(23240)=(S=6753902,E=6753903,C=74)
	CT(23241)=(S=6753904,E=6753907,C=1)
	CT(23242)=(S=6753908,E=6753911,C=8)
	CT(23243)=(S=6753912,E=6753919,C=4)
	CT(23244)=(S=6753920,E=6753923,C=84)
	CT(23245)=(S=6753924,E=6753931,C=1)
	CT(23246)=(S=6753932,E=6753935,C=73)
	CT(23247)=(S=6753936,E=6753939,C=8)
	CT(23248)=(S=6753940,E=6753947,C=2)
	CT(23249)=(S=6753948,E=6753951,C=1)
	CT(23250)=(S=6753952,E=6753955,C=4)
	CT(23251)=(S=6753956,E=6753959,C=3)
	CT(23252)=(S=6753960,E=6753963,C=73)
	CT(23253)=(S=6753964,E=6753967,C=0)
	CT(23254)=(S=6753968,E=6753971,C=2)
	CT(23255)=(S=6753972,E=6753975,C=4)
	CT(23256)=(S=6753976,E=6753976,C=5)
	CT(23257)=(S=6753977,E=6753977,C=7)
	CT(23258)=(S=6753978,E=6753979,C=0)
	CT(23259)=(S=6753980,E=6753983,C=4)
	CT(23260)=(S=6753984,E=6753984,C=74)
	CT(23261)=(S=6753985,E=6753985,C=84)
	CT(23262)=(S=6753986,E=6753986,C=10)
	CT(23263)=(S=6753987,E=6753987,C=74)
	CT(23264)=(S=6753992,E=6753995,C=0)
	CT(23265)=(S=6753996,E=6753999,C=84)
	CT(23266)=(S=6754000,E=6754003,C=4)
	CT(23267)=(S=6754004,E=6754007,C=7)
	CT(23268)=(S=6754008,E=6754011,C=73)
	CT(23269)=(S=6754012,E=6754015,C=8)
	CT(23270)=(S=6754016,E=6754019,C=7)
	CT(23271)=(S=6754020,E=6754020,C=0)
	CT(23272)=(S=6754021,E=6754021,C=74)
	CT(23273)=(S=6754022,E=6754022,C=0)
	CT(23274)=(S=6754023,E=6754023,C=77)
	CT(23275)=(S=6754024,E=6754027,C=4)
	CT(23276)=(S=6754028,E=6754031,C=7)
	CT(23277)=(S=6754032,E=6754035,C=1)
	CT(23278)=(S=6754036,E=6754039,C=7)
	CT(23279)=(S=6754040,E=6754043,C=86)
	CT(23280)=(S=6754044,E=6754047,C=0)
	CT(23281)=(S=6754048,E=6754051,C=2)
	CT(23282)=(S=6754052,E=6754059,C=1)
	CT(23283)=(S=6754060,E=6754063,C=2)
	CT(23284)=(S=6754064,E=6754067,C=1)
	CT(23285)=(S=6754068,E=6754079,C=8)
	CT(23286)=(S=6754080,E=6754083,C=2)
	CT(23287)=(S=6754084,E=6754085,C=74)
	CT(23288)=(S=6754086,E=6754086,C=8)
	CT(23289)=(S=6754087,E=6754087,C=2)
	CT(23290)=(S=6754088,E=6754091,C=84)
	CT(23291)=(S=6754092,E=6754095,C=0)
	CT(23292)=(S=6754096,E=6754099,C=11)
	CT(23293)=(S=6754100,E=6754115,C=4)
	CT(23294)=(S=6754116,E=6754119,C=2)
	CT(23295)=(S=6754120,E=6754121,C=0)
	CT(23296)=(S=6754122,E=6754122,C=4)
	CT(23297)=(S=6754123,E=6754123,C=73)
	CT(23298)=(S=6754124,E=6754127,C=2)
	CT(23299)=(S=6754128,E=6754131,C=4)
	CT(23300)=(S=6754132,E=6754135,C=8)
	CT(23301)=(S=6754136,E=6754139,C=3)
	CT(23302)=(S=6754144,E=6754147,C=1)
	CT(23303)=(S=6754148,E=6754151,C=7)
	CT(23304)=(S=6754152,E=6754155,C=5)
	CT(23305)=(S=6754156,E=6754159,C=2)
	CT(23306)=(S=6754160,E=6754163,C=76)
	CT(23307)=(S=6754164,E=6754167,C=2)
	CT(23308)=(S=6754168,E=6754171,C=8)
	CT(23309)=(S=6754172,E=6754172,C=4)
	CT(23310)=(S=6754173,E=6754173,C=7)
	CT(23311)=(S=6754174,E=6754175,C=73)
	CT(23312)=(S=6754176,E=6754179,C=0)
	CT(23313)=(S=6754180,E=6754183,C=8)
	CT(23314)=(S=6754184,E=6754187,C=5)
	CT(23315)=(S=6754188,E=6754190,C=84)
	CT(23316)=(S=6754191,E=6754191,C=74)
	CT(23317)=(S=6754192,E=6754195,C=3)
	CT(23318)=(S=6754196,E=6754199,C=0)
	CT(23319)=(S=6754200,E=6754203,C=73)
	CT(23320)=(S=6754204,E=6754205,C=0)
	CT(23321)=(S=6754206,E=6754206,C=8)
	CT(23322)=(S=6754207,E=6754207,C=4)
	CT(23323)=(S=6754208,E=6754211,C=2)
	CT(23324)=(S=6754212,E=6754215,C=84)
	CT(23325)=(S=6754216,E=6754219,C=0)
	CT(23326)=(S=6754220,E=6754223,C=74)
	CT(23327)=(S=6754224,E=6754224,C=4)
	CT(23328)=(S=6754225,E=6754225,C=0)
	CT(23329)=(S=6754226,E=6754226,C=73)
	CT(23330)=(S=6754227,E=6754227,C=4)
	CT(23331)=(S=6754228,E=6754235,C=8)
	CT(23332)=(S=6754236,E=6754239,C=2)
	CT(23333)=(S=6754240,E=6754247,C=8)
	CT(23334)=(S=6754248,E=6754251,C=1)
	CT(23335)=(S=6754252,E=6754259,C=0)
	CT(23336)=(S=6754260,E=6754263,C=2)
	CT(23337)=(S=6754264,E=6754267,C=8)
	CT(23338)=(S=6754268,E=6754273,C=84)
	CT(23339)=(S=6754275,E=6754275,C=4)
	CT(23340)=(S=6754280,E=6754283,C=7)
	CT(23341)=(S=6754284,E=6754285,C=0)
	CT(23342)=(S=6754286,E=6754287,C=89)
	CT(23343)=(S=6754288,E=6754291,C=74)
	CT(23344)=(S=6754292,E=6754295,C=84)
	CT(23345)=(S=6754296,E=6754298,C=4)
	CT(23346)=(S=6754299,E=6754299,C=74)
	CT(23347)=(S=6754304,E=6754307,C=11)
	CT(23348)=(S=6754308,E=6754311,C=4)
	CT(23349)=(S=6754312,E=6754315,C=2)
	CT(23350)=(S=6754316,E=6754318,C=4)
	CT(23351)=(S=6754319,E=6754319,C=160)
	CT(23352)=(S=6754320,E=6754323,C=7)
	CT(23353)=(S=6754324,E=6754327,C=73)
	CT(23354)=(S=6754330,E=6754331,C=8)
	CT(23355)=(S=6754332,E=6754339,C=4)
	CT(23356)=(S=6754340,E=6754343,C=2)
	CT(23357)=(S=6754344,E=6754347,C=5)
	CT(23358)=(S=6754348,E=6754351,C=4)
	CT(23359)=(S=6754352,E=6754353,C=74)
	CT(23360)=(S=6754354,E=6754354,C=4)
	CT(23361)=(S=6754355,E=6754355,C=0)
	CT(23362)=(S=6754356,E=6754359,C=1)
	CT(23363)=(S=6754360,E=6754363,C=4)
	CT(23364)=(S=6754364,E=6754367,C=86)
	CT(23365)=(S=6754368,E=6754371,C=0)
	CT(23366)=(S=6754372,E=6754375,C=4)
	CT(23367)=(S=6754376,E=6754379,C=84)
	CT(23368)=(S=6754380,E=6754380,C=4)
	CT(23369)=(S=6754381,E=6754383,C=74)
	CT(23370)=(S=6754384,E=6754395,C=1)
	CT(23371)=(S=6754396,E=6754403,C=0)
	CT(23372)=(S=6754404,E=6754405,C=4)
	CT(23373)=(S=6754406,E=6754407,C=7)
	CT(23374)=(S=6754408,E=6754411,C=4)
	CT(23375)=(S=6754412,E=6754415,C=1)
	CT(23376)=(S=6754416,E=6754419,C=74)
	CT(23377)=(S=6754420,E=6754423,C=8)
	CT(23378)=(S=6754424,E=6754427,C=157)
	CT(23379)=(S=6754428,E=6754431,C=1)
	CT(23380)=(S=6754432,E=6754435,C=0)
	CT(23381)=(S=6754436,E=6754437,C=74)
	CT(23382)=(S=6754438,E=6754439,C=0)
	CT(23383)=(S=6754440,E=6754443,C=74)
	CT(23384)=(S=6754444,E=6754451,C=4)
	CT(23385)=(S=6754452,E=6754455,C=0)
	CT(23386)=(S=6754456,E=6754459,C=84)
	CT(23387)=(S=6754460,E=6754463,C=76)
	CT(23388)=(S=6754464,E=6754467,C=2)
	CT(23389)=(S=6754468,E=6754471,C=4)
	CT(23390)=(S=6754472,E=6754475,C=10)
	CT(23391)=(S=6754476,E=6754479,C=73)
	CT(23392)=(S=6754480,E=6754483,C=4)
	CT(23393)=(S=6754484,E=6754485,C=73)
	CT(23394)=(S=6754486,E=6754487,C=5)
	CT(23395)=(S=6754488,E=6754491,C=8)
	CT(23396)=(S=6754492,E=6754495,C=74)
	CT(23397)=(S=6754496,E=6754499,C=0)
	CT(23398)=(S=6754500,E=6754500,C=4)
	CT(23399)=(S=6754501,E=6754503,C=74)
	CT(23400)=(S=6754504,E=6754507,C=4)
	CT(23401)=(S=6754508,E=6754511,C=3)
	CT(23402)=(S=6754512,E=6754513,C=4)
	CT(23403)=(S=6754514,E=6754515,C=0)
	CT(23404)=(S=6754516,E=6754519,C=2)
	CT(23405)=(S=6754520,E=6754523,C=8)
	CT(23406)=(S=6754524,E=6754524,C=74)
	CT(23407)=(S=6754525,E=6754526,C=4)
	CT(23408)=(S=6754527,E=6754527,C=74)
	CT(23409)=(S=6754528,E=6754531,C=2)
	CT(23410)=(S=6754532,E=6754535,C=8)
	CT(23411)=(S=6754536,E=6754539,C=74)
	CT(23412)=(S=6754540,E=6754543,C=2)
	CT(23413)=(S=6754544,E=6754551,C=9)
	CT(23414)=(S=6754552,E=6754555,C=2)
	CT(23415)=(S=6754556,E=6754559,C=8)
	CT(23416)=(S=6754560,E=6754563,C=4)
	CT(23417)=(S=6754564,E=6754567,C=2)
	CT(23418)=(S=6754568,E=6754571,C=9)
	CT(23419)=(S=6754576,E=6754577,C=0)
	CT(23420)=(S=6754578,E=6754578,C=4)
	CT(23421)=(S=6754579,E=6754579,C=5)
	CT(23422)=(S=6754580,E=6754583,C=10)
	CT(23423)=(S=6754584,E=6754587,C=157)
	CT(23424)=(S=6754588,E=6754595,C=8)
	CT(23425)=(S=6754596,E=6754599,C=84)
	CT(23426)=(S=6754600,E=6754603,C=1)
	CT(23427)=(S=6754604,E=6754607,C=90)
	CT(23428)=(S=6754608,E=6754611,C=4)
	CT(23429)=(S=6754612,E=6754615,C=74)
	CT(23430)=(S=6754616,E=6754617,C=0)
	CT(23431)=(S=6754618,E=6754618,C=74)
	CT(23432)=(S=6754619,E=6754619,C=0)
	CT(23433)=(S=6754620,E=6754623,C=89)
	CT(23434)=(S=6754628,E=6754631,C=84)
	CT(23435)=(S=6754632,E=6754635,C=8)
	CT(23436)=(S=6754636,E=6754637,C=74)
	CT(23437)=(S=6754638,E=6754638,C=5)
	CT(23438)=(S=6754639,E=6754639,C=74)
	CT(23439)=(S=6754640,E=6754647,C=4)
	CT(23440)=(S=6754648,E=6754651,C=11)
	CT(23441)=(S=6754652,E=6754655,C=8)
	CT(23442)=(S=6754656,E=6754667,C=4)
	CT(23443)=(S=6754668,E=6754669,C=92)
	CT(23444)=(S=6754672,E=6754675,C=159)
	CT(23445)=(S=6754676,E=6754679,C=8)
	CT(23446)=(S=6754680,E=6754683,C=1)
	CT(23447)=(S=6754684,E=6754685,C=5)
	CT(23448)=(S=6754686,E=6754686,C=2)
	CT(23449)=(S=6754688,E=6754691,C=4)
	CT(23450)=(S=6754692,E=6754695,C=0)
	CT(23451)=(S=6754700,E=6754707,C=7)
	CT(23452)=(S=6754708,E=6754711,C=2)
	CT(23453)=(S=6754712,E=6754715,C=0)
	CT(23454)=(S=6754716,E=6754719,C=4)
	CT(23455)=(S=6754720,E=6754723,C=1)
	CT(23456)=(S=6754724,E=6754724,C=74)
	CT(23457)=(S=6754725,E=6754727,C=89)
	CT(23458)=(S=6754728,E=6754731,C=5)
	CT(23459)=(S=6754732,E=6754735,C=74)
	CT(23460)=(S=6754736,E=6754739,C=2)
	CT(23461)=(S=6754740,E=6754741,C=84)
	CT(23462)=(S=6754742,E=6754743,C=74)
	CT(23463)=(S=6754744,E=6754751,C=2)
	CT(23464)=(S=6754752,E=6754752,C=74)
	CT(23465)=(S=6754753,E=6754753,C=0)
	CT(23466)=(S=6754754,E=6754755,C=4)
	CT(23467)=(S=6754756,E=6754759,C=8)
	CT(23468)=(S=6754760,E=6754763,C=77)
	CT(23469)=(S=6754764,E=6754767,C=1)
	CT(23470)=(S=6754768,E=6754771,C=5)
	CT(23471)=(S=6754772,E=6754775,C=86)
	CT(23472)=(S=6754776,E=6754787,C=0)
	CT(23473)=(S=6754788,E=6754791,C=1)
	CT(23474)=(S=6754792,E=6754795,C=0)
	CT(23475)=(S=6754796,E=6754799,C=11)
	CT(23476)=(S=6754800,E=6754803,C=9)
	CT(23477)=(S=6754804,E=6754807,C=74)
	CT(23478)=(S=6754808,E=6754808,C=10)
	CT(23479)=(S=6754810,E=6754815,C=0)
	CT(23480)=(S=6754816,E=6754819,C=5)
	CT(23481)=(S=6754820,E=6754823,C=11)
	CT(23482)=(S=6754824,E=6754831,C=77)
	CT(23483)=(S=6754832,E=6754835,C=74)
	CT(23484)=(S=6754836,E=6754839,C=77)
	CT(23485)=(S=6754840,E=6754843,C=2)
	CT(23486)=(S=6754844,E=6754851,C=74)
	CT(23487)=(S=6754852,E=6754871,C=0)
	CT(23488)=(S=6754872,E=6754875,C=73)
	CT(23489)=(S=6754876,E=6754879,C=2)
	CT(23490)=(S=6754880,E=6754891,C=4)
	CT(23491)=(S=6754900,E=6754903,C=0)
	CT(23492)=(S=6754904,E=6754911,C=5)
	CT(23493)=(S=6754912,E=6754915,C=74)
	CT(23494)=(S=6754916,E=6754919,C=2)
	CT(23495)=(S=6754920,E=6754923,C=4)
	CT(23496)=(S=6754924,E=6754927,C=0)
	CT(23497)=(S=6754928,E=6754931,C=2)
	CT(23498)=(S=6754932,E=6754932,C=77)
	CT(23499)=(S=6754933,E=6754933,C=74)
	CT(23500)=(S=6754934,E=6754935,C=73)
	CT(23501)=(S=6754936,E=6754939,C=4)
	CT(23502)=(S=6754940,E=6754943,C=0)
	CT(23503)=(S=6754944,E=6754947,C=9)
	CT(23504)=(S=6754948,E=6754951,C=74)
	CT(23505)=(S=6754952,E=6754955,C=73)
	CT(23506)=(S=6754956,E=6754959,C=4)
	CT(23507)=(S=6754960,E=6754963,C=74)
	CT(23508)=(S=6754964,E=6754967,C=0)
	CT(23509)=(S=6754968,E=6754968,C=4)
	CT(23510)=(S=6754969,E=6754969,C=74)
	CT(23511)=(S=6754970,E=6754971,C=0)
	CT(23512)=(S=6754972,E=6754975,C=4)
	CT(23513)=(S=6754976,E=6754979,C=89)
	CT(23514)=(S=6754980,E=6754987,C=4)
	CT(23515)=(S=6754988,E=6754991,C=157)
	CT(23516)=(S=6754992,E=6754995,C=11)
	CT(23517)=(S=6755000,E=6755003,C=8)
	CT(23518)=(S=6755004,E=6755005,C=0)
	CT(23519)=(S=6755006,E=6755007,C=74)
	CT(23520)=(S=6755008,E=6755011,C=1)
	CT(23521)=(S=6755012,E=6755015,C=0)
	CT(23522)=(S=6755016,E=6755019,C=73)
	CT(23523)=(S=6755020,E=6755023,C=0)
	CT(23524)=(S=6755024,E=6755031,C=1)
	CT(23525)=(S=6755032,E=6755035,C=4)
	CT(23526)=(S=6755036,E=6755039,C=74)
	CT(23527)=(S=6755040,E=6755043,C=1)
	CT(23528)=(S=6755044,E=6755051,C=2)
	CT(23529)=(S=6755052,E=6755056,C=74)
	CT(23530)=(S=6755057,E=6755057,C=0)
	CT(23531)=(S=6755058,E=6755058,C=4)
	CT(23532)=(S=6755059,E=6755059,C=77)
	CT(23533)=(S=6755060,E=6755063,C=5)
	CT(23534)=(S=6755068,E=6755071,C=0)
	CT(23535)=(S=6755076,E=6755079,C=73)
	CT(23536)=(S=6755080,E=6755081,C=7)
	CT(23537)=(S=6755082,E=6755083,C=73)
	CT(23538)=(S=6755084,E=6755087,C=1)
	CT(23539)=(S=6755088,E=6755088,C=10)
	CT(23540)=(S=6755089,E=6755091,C=4)
	CT(23541)=(S=6755092,E=6755095,C=7)
	CT(23542)=(S=6755096,E=6755099,C=8)
	CT(23543)=(S=6755100,E=6755103,C=2)
	CT(23544)=(S=6755104,E=6755107,C=10)
	CT(23545)=(S=6755109,E=6755109,C=74)
	CT(23546)=(S=6755110,E=6755110,C=8)
	CT(23547)=(S=6755112,E=6755119,C=1)
	CT(23548)=(S=6755120,E=6755123,C=2)
	CT(23549)=(S=6755124,E=6755127,C=4)
	CT(23550)=(S=6755128,E=6755131,C=74)
	CT(23551)=(S=6755132,E=6755135,C=0)
	CT(23552)=(S=6755136,E=6755147,C=1)
	CT(23553)=(S=6755148,E=6755153,C=74)
	CT(23554)=(S=6755154,E=6755155,C=7)
	CT(23555)=(S=6755156,E=6755159,C=8)
	CT(23556)=(S=6755160,E=6755163,C=4)
	CT(23557)=(S=6755164,E=6755167,C=0)
	CT(23558)=(S=6755168,E=6755171,C=11)
	CT(23559)=(S=6755172,E=6755175,C=73)
	CT(23560)=(S=6755176,E=6755179,C=3)
	CT(23561)=(S=6755180,E=6755183,C=74)
	CT(23562)=(S=6755184,E=6755187,C=0)
	CT(23563)=(S=6755188,E=6755195,C=2)
	CT(23564)=(S=6755196,E=6755199,C=6)
	CT(23565)=(S=6755200,E=6755201,C=4)
	CT(23566)=(S=6755202,E=6755203,C=84)
	CT(23567)=(S=6755204,E=6755215,C=4)
	CT(23568)=(S=6755216,E=6755219,C=0)
	CT(23569)=(S=6755220,E=6755227,C=4)
	CT(23570)=(S=6755228,E=6755235,C=2)
	CT(23571)=(S=6755236,E=6755239,C=11)
	CT(23572)=(S=6755240,E=6755241,C=4)
	CT(23573)=(S=6755242,E=6755247,C=0)
	CT(23574)=(S=6755248,E=6755259,C=74)
	CT(23575)=(S=6755260,E=6755263,C=8)
	CT(23576)=(S=6755264,E=6755267,C=0)
	CT(23577)=(S=6755268,E=6755275,C=4)
	CT(23578)=(S=6755276,E=6755283,C=74)
	CT(23579)=(S=6755284,E=6755287,C=4)
	CT(23580)=(S=6755288,E=6755288,C=0)
	CT(23581)=(S=6755289,E=6755289,C=4)
	CT(23582)=(S=6755290,E=6755290,C=0)
	CT(23583)=(S=6755292,E=6755295,C=11)
	CT(23584)=(S=6755296,E=6755299,C=4)
	CT(23585)=(S=6755300,E=6755303,C=74)
	CT(23586)=(S=6755304,E=6755307,C=1)
	CT(23587)=(S=6755308,E=6755311,C=8)
	CT(23588)=(S=6755312,E=6755315,C=2)
	CT(23589)=(S=6755316,E=6755319,C=0)
	CT(23590)=(S=6755320,E=6755323,C=4)
	CT(23591)=(S=6755324,E=6755327,C=84)
	CT(23592)=(S=6755328,E=6755331,C=77)
	CT(23593)=(S=6755332,E=6755339,C=2)
	CT(23594)=(S=6755340,E=6755343,C=1)
	CT(23595)=(S=6755344,E=6755348,C=0)
	CT(23596)=(S=6755349,E=6755349,C=74)
	CT(23597)=(S=6755350,E=6755351,C=0)
	CT(23598)=(S=6755352,E=6755355,C=157)
	CT(23599)=(S=6755356,E=6755359,C=4)
	CT(23600)=(S=6755360,E=6755363,C=1)
	CT(23601)=(S=6755364,E=6755367,C=2)
	CT(23602)=(S=6755368,E=6755371,C=9)
	CT(23603)=(S=6755376,E=6755377,C=0)
	CT(23604)=(S=6755378,E=6755378,C=7)
	CT(23605)=(S=6755379,E=6755379,C=74)
	CT(23606)=(S=6755380,E=6755383,C=84)
	CT(23607)=(S=6755385,E=6755385,C=4)
	CT(23608)=(S=6755387,E=6755387,C=7)
	CT(23609)=(S=6755388,E=6755391,C=8)
	CT(23610)=(S=6755392,E=6755395,C=4)
	CT(23611)=(S=6755400,E=6755403,C=2)
	CT(23612)=(S=6755404,E=6755407,C=76)
	CT(23613)=(S=6755408,E=6755411,C=86)
	CT(23614)=(S=6755412,E=6755415,C=2)
	CT(23615)=(S=6755416,E=6755416,C=73)
	CT(23616)=(S=6755417,E=6755417,C=84)
	CT(23617)=(S=6755418,E=6755419,C=74)
	CT(23618)=(S=6755420,E=6755423,C=7)
	CT(23619)=(S=6755424,E=6755427,C=8)
	CT(23620)=(S=6755432,E=6755433,C=4)
	CT(23621)=(S=6755434,E=6755435,C=74)
	CT(23622)=(S=6755436,E=6755439,C=2)
	CT(23623)=(S=6755440,E=6755443,C=1)
	CT(23624)=(S=6755444,E=6755447,C=6)
	CT(23625)=(S=6755448,E=6755451,C=3)
	CT(23626)=(S=6755452,E=6755452,C=158)
	CT(23627)=(S=6755453,E=6755455,C=4)
	CT(23628)=(S=6755456,E=6755459,C=1)
	CT(23629)=(S=6755460,E=6755463,C=77)
	CT(23630)=(S=6755464,E=6755467,C=73)
	CT(23631)=(S=6755468,E=6755471,C=84)
	CT(23632)=(S=6755472,E=6755479,C=11)
	CT(23633)=(S=6755480,E=6755483,C=92)
	CT(23634)=(S=6755484,E=6755487,C=2)
	CT(23635)=(S=6755488,E=6755491,C=1)
	CT(23636)=(S=6755492,E=6755492,C=4)
	CT(23637)=(S=6755493,E=6755493,C=8)
	CT(23638)=(S=6755494,E=6755495,C=74)
	CT(23639)=(S=6755496,E=6755499,C=7)
	CT(23640)=(S=6755500,E=6755503,C=8)
	CT(23641)=(S=6755504,E=6755507,C=9)
	CT(23642)=(S=6755508,E=6755511,C=84)
	CT(23643)=(S=6755512,E=6755519,C=74)
	CT(23644)=(S=6755520,E=6755523,C=8)
	CT(23645)=(S=6755524,E=6755525,C=74)
	CT(23646)=(S=6755526,E=6755531,C=0)
	CT(23647)=(S=6755532,E=6755535,C=3)
	CT(23648)=(S=6755536,E=6755539,C=0)
	CT(23649)=(S=6755540,E=6755543,C=4)
	CT(23650)=(S=6755544,E=6755544,C=0)
	CT(23651)=(S=6755545,E=6755545,C=7)
	CT(23652)=(S=6755546,E=6755546,C=4)
	CT(23653)=(S=6755548,E=6755551,C=8)
	CT(23654)=(S=6755552,E=6755555,C=5)
	CT(23655)=(S=6755556,E=6755559,C=8)
	CT(23656)=(S=6755560,E=6755561,C=160)
	CT(23657)=(S=6755562,E=6755563,C=0)
	CT(23658)=(S=6755564,E=6755567,C=8)
	CT(23659)=(S=6755568,E=6755571,C=2)
	CT(23660)=(S=6755572,E=6755575,C=0)
	CT(23661)=(S=6755576,E=6755579,C=1)
	CT(23662)=(S=6755580,E=6755587,C=0)
	CT(23663)=(S=6755596,E=6755599,C=10)
	CT(23664)=(S=6755600,E=6755600,C=4)
	CT(23665)=(S=6755601,E=6755601,C=0)
	CT(23666)=(S=6755602,E=6755603,C=4)
	CT(23667)=(S=6755604,E=6755607,C=0)
	CT(23668)=(S=6755608,E=6755611,C=3)
	CT(23669)=(S=6755612,E=6755615,C=7)
	CT(23670)=(S=6755616,E=6755619,C=5)
	CT(23671)=(S=6755620,E=6755623,C=2)
	CT(23672)=(S=6755624,E=6755627,C=84)
	CT(23673)=(S=6755628,E=6755631,C=5)
	CT(23674)=(S=6755632,E=6755635,C=0)
	CT(23675)=(S=6755636,E=6755639,C=4)
	CT(23676)=(S=6755640,E=6755641,C=0)
	CT(23677)=(S=6755642,E=6755643,C=4)
	CT(23678)=(S=6755644,E=6755647,C=9)
	CT(23679)=(S=6755648,E=6755649,C=7)
	CT(23680)=(S=6755650,E=6755651,C=4)
	CT(23681)=(S=6755652,E=6755655,C=7)
	CT(23682)=(S=6755656,E=6755659,C=84)
	CT(23683)=(S=6755660,E=6755663,C=4)
	CT(23684)=(S=6755664,E=6755665,C=7)
	CT(23685)=(S=6755666,E=6755667,C=4)
	CT(23686)=(S=6755668,E=6755671,C=74)
	CT(23687)=(S=6755672,E=6755675,C=5)
	CT(23688)=(S=6755676,E=6755679,C=74)
	CT(23689)=(S=6755680,E=6755691,C=2)
	CT(23690)=(S=6755692,E=6755695,C=0)
	CT(23691)=(S=6755696,E=6755703,C=1)
	CT(23692)=(S=6755704,E=6755707,C=11)
	CT(23693)=(S=6755708,E=6755711,C=4)
	CT(23694)=(S=6755712,E=6755719,C=0)
	CT(23695)=(S=6755720,E=6755727,C=1)
	CT(23696)=(S=6755728,E=6755731,C=0)
	CT(23697)=(S=6755732,E=6755735,C=11)
	CT(23698)=(S=6755736,E=6755739,C=5)
	CT(23699)=(S=6755740,E=6755743,C=0)
	CT(23700)=(S=6755744,E=6755747,C=4)
	CT(23701)=(S=6755748,E=6755751,C=156)
	CT(23702)=(S=6755752,E=6755755,C=10)
	CT(23703)=(S=6755756,E=6755759,C=73)
	CT(23704)=(S=6755760,E=6755763,C=1)
	CT(23705)=(S=6755764,E=6755767,C=5)
	CT(23706)=(S=6755768,E=6755771,C=4)
	CT(23707)=(S=6755772,E=6755775,C=0)
	CT(23708)=(S=6755777,E=6755778,C=73)
	CT(23709)=(S=6755779,E=6755779,C=4)
	CT(23710)=(S=6755780,E=6755783,C=9)
	CT(23711)=(S=6755784,E=6755787,C=6)
	CT(23712)=(S=6755788,E=6755791,C=74)
	CT(23713)=(S=6755792,E=6755795,C=1)
	CT(23714)=(S=6755796,E=6755803,C=74)
	CT(23715)=(S=6755804,E=6755807,C=8)
	CT(23716)=(S=6755808,E=6755813,C=74)
	CT(23717)=(S=6755814,E=6755815,C=192)
	CT(23718)=(S=6755816,E=6755819,C=4)
	CT(23719)=(S=6755824,E=6755827,C=1)
	CT(23720)=(S=6755828,E=6755831,C=8)
	CT(23721)=(S=6755832,E=6755835,C=192)
	CT(23722)=(S=6755836,E=6755839,C=5)
	CT(23723)=(S=6755840,E=6755935,C=1)
	CT(23724)=(S=6755936,E=6755939,C=8)
	CT(23725)=(S=6755940,E=6755967,C=1)
	CT(23726)=(S=6755968,E=6755971,C=7)
	CT(23727)=(S=6755972,E=6755979,C=74)
	CT(23728)=(S=6755984,E=6755987,C=0)
	CT(23729)=(S=6755988,E=6755991,C=5)
	CT(23730)=(S=6755992,E=6755995,C=2)
	CT(23731)=(S=6755996,E=6755999,C=9)
	CT(23732)=(S=6756000,E=6756003,C=8)
	CT(23733)=(S=6756004,E=6756006,C=74)
	CT(23734)=(S=6756007,E=6756007,C=2)
	CT(23735)=(S=6756008,E=6756011,C=7)
	CT(23736)=(S=6756012,E=6756015,C=4)
	CT(23737)=(S=6756016,E=6756019,C=0)
	CT(23738)=(S=6756020,E=6756023,C=3)
	CT(23739)=(S=6756024,E=6756027,C=0)
	CT(23740)=(S=6756028,E=6756031,C=1)
	CT(23741)=(S=6756032,E=6756035,C=4)
	CT(23742)=(S=6756036,E=6756039,C=0)
	CT(23743)=(S=6756040,E=6756043,C=8)
	CT(23744)=(S=6756049,E=6756051,C=0)
	CT(23745)=(S=6756056,E=6756059,C=5)
	CT(23746)=(S=6756060,E=6756063,C=6)
	CT(23747)=(S=6756064,E=6756067,C=0)
	CT(23748)=(S=6756068,E=6756071,C=1)
	CT(23749)=(S=6756072,E=6756073,C=74)
	CT(23750)=(S=6756074,E=6756075,C=73)
	CT(23751)=(S=6756076,E=6756079,C=4)
	CT(23752)=(S=6756080,E=6756091,C=74)
	CT(23753)=(S=6756092,E=6756095,C=1)
	CT(23754)=(S=6756096,E=6756099,C=0)
	CT(23755)=(S=6756100,E=6756103,C=2)
	CT(23756)=(S=6756104,E=6756107,C=1)
	CT(23757)=(S=6756108,E=6756111,C=2)
	CT(23758)=(S=6756112,E=6756115,C=73)
	CT(23759)=(S=6756116,E=6756119,C=74)
	CT(23760)=(S=6756120,E=6756125,C=4)
	CT(23761)=(S=6756126,E=6756131,C=74)
	CT(23762)=(S=6756132,E=6756135,C=89)
	CT(23763)=(S=6756136,E=6756139,C=0)
	CT(23764)=(S=6756140,E=6756143,C=5)
	CT(23765)=(S=6756144,E=6756147,C=92)
	CT(23766)=(S=6756148,E=6756151,C=90)
	CT(23767)=(S=6756152,E=6756155,C=1)
	CT(23768)=(S=6756156,E=6756159,C=84)
	CT(23769)=(S=6756160,E=6756163,C=0)
	CT(23770)=(S=6756164,E=6756165,C=8)
	CT(23771)=(S=6756166,E=6756166,C=0)
	CT(23772)=(S=6756168,E=6756171,C=2)
	CT(23773)=(S=6756172,E=6756175,C=7)
	CT(23774)=(S=6756176,E=6756183,C=6)
	CT(23775)=(S=6756184,E=6756187,C=2)
	CT(23776)=(S=6756188,E=6756191,C=7)
	CT(23777)=(S=6756192,E=6756195,C=10)
	CT(23778)=(S=6756196,E=6756199,C=74)
	CT(23779)=(S=6756200,E=6756203,C=0)
	CT(23780)=(S=6756204,E=6756207,C=9)
	CT(23781)=(S=6756208,E=6756211,C=0)
	CT(23782)=(S=6756212,E=6756213,C=74)
	CT(23783)=(S=6756214,E=6756215,C=4)
	CT(23784)=(S=6756216,E=6756219,C=3)
	CT(23785)=(S=6756220,E=6756223,C=4)
	CT(23786)=(S=6756224,E=6756227,C=2)
	CT(23787)=(S=6756228,E=6756235,C=86)
	CT(23788)=(S=6756236,E=6756236,C=4)
	CT(23789)=(S=6756237,E=6756237,C=74)
	CT(23790)=(S=6756238,E=6756239,C=73)
	CT(23791)=(S=6756240,E=6756243,C=11)
	CT(23792)=(S=6756246,E=6756246,C=4)
	CT(23793)=(S=6756247,E=6756247,C=74)
	CT(23794)=(S=6756248,E=6756251,C=0)
	CT(23795)=(S=6756252,E=6756255,C=11)
	CT(23796)=(S=6756256,E=6756263,C=1)
	CT(23797)=(S=6756264,E=6756267,C=8)
	CT(23798)=(S=6756271,E=6756271,C=74)
	CT(23799)=(S=6756272,E=6756275,C=1)
	CT(23800)=(S=6756276,E=6756279,C=74)
	CT(23801)=(S=6756280,E=6756287,C=0)
	CT(23802)=(S=6756288,E=6756291,C=8)
	CT(23803)=(S=6756292,E=6756295,C=2)
	CT(23804)=(S=6756296,E=6756299,C=74)
	CT(23805)=(S=6756304,E=6756307,C=7)
	CT(23806)=(S=6756308,E=6756308,C=73)
	CT(23807)=(S=6756309,E=6756309,C=86)
	CT(23808)=(S=6756310,E=6756310,C=4)
	CT(23809)=(S=6756312,E=6756315,C=0)
	CT(23810)=(S=6756316,E=6756319,C=2)
	CT(23811)=(S=6756320,E=6756323,C=74)
	CT(23812)=(S=6756324,E=6756327,C=1)
	CT(23813)=(S=6756328,E=6756331,C=74)
	CT(23814)=(S=6756332,E=6756335,C=4)
	CT(23815)=(S=6756336,E=6756340,C=74)
	CT(23816)=(S=6756341,E=6756341,C=0)
	CT(23817)=(S=6756342,E=6756342,C=3)
	CT(23818)=(S=6756343,E=6756343,C=89)
	CT(23819)=(S=6756344,E=6756351,C=2)
	CT(23820)=(S=6756352,E=6756355,C=8)
	CT(23821)=(S=6756356,E=6756359,C=7)
	CT(23822)=(S=6756360,E=6756363,C=6)
	CT(23823)=(S=6756364,E=6756365,C=74)
	CT(23824)=(S=6756366,E=6756366,C=73)
	CT(23825)=(S=6756367,E=6756367,C=8)
	CT(23826)=(S=6756368,E=6756371,C=10)
	CT(23827)=(S=6756372,E=6756375,C=4)
	CT(23828)=(S=6756380,E=6756383,C=73)
	CT(23829)=(S=6756384,E=6756387,C=86)
	CT(23830)=(S=6756388,E=6756391,C=4)
	CT(23831)=(S=6756392,E=6756395,C=73)
	CT(23832)=(S=6756396,E=6756397,C=7)
	CT(23833)=(S=6756398,E=6756398,C=4)
	CT(23834)=(S=6756399,E=6756399,C=0)
	CT(23835)=(S=6756400,E=6756403,C=74)
	CT(23836)=(S=6756404,E=6756407,C=2)
	CT(23837)=(S=6756408,E=6756411,C=74)
	CT(23838)=(S=6756412,E=6756415,C=4)
	CT(23839)=(S=6756416,E=6756419,C=0)
	CT(23840)=(S=6756420,E=6756423,C=2)
	CT(23841)=(S=6756424,E=6756428,C=74)
	CT(23842)=(S=6756429,E=6756429,C=7)
	CT(23843)=(S=6756430,E=6756431,C=76)
	CT(23844)=(S=6756432,E=6756443,C=4)
	CT(23845)=(S=6756444,E=6756447,C=8)
	CT(23846)=(S=6756448,E=6756451,C=77)
	CT(23847)=(S=6756452,E=6756455,C=9)
	CT(23848)=(S=6756456,E=6756459,C=74)
	CT(23849)=(S=6756460,E=6756463,C=4)
	CT(23850)=(S=6756464,E=6756467,C=90)
	CT(23851)=(S=6756468,E=6756471,C=1)
	CT(23852)=(S=6756472,E=6756475,C=0)
	CT(23853)=(S=6756476,E=6756479,C=4)
	CT(23854)=(S=6756480,E=6756483,C=1)
	CT(23855)=(S=6756484,E=6756485,C=0)
	CT(23856)=(S=6756486,E=6756486,C=5)
	CT(23857)=(S=6756487,E=6756487,C=4)
	CT(23858)=(S=6756488,E=6756495,C=73)
	CT(23859)=(S=6756496,E=6756499,C=1)
	CT(23860)=(S=6756500,E=6756503,C=74)
	CT(23861)=(S=6756504,E=6756507,C=8)
	CT(23862)=(S=6756508,E=6756511,C=7)
	CT(23863)=(S=6756512,E=6756515,C=2)
	CT(23864)=(S=6756516,E=6756519,C=7)
	CT(23865)=(S=6756520,E=6756523,C=2)
	CT(23866)=(S=6756524,E=6756527,C=4)
	CT(23867)=(S=6756528,E=6756531,C=1)
	CT(23868)=(S=6756532,E=6756535,C=4)
	CT(23869)=(S=6756536,E=6756539,C=1)
	CT(23870)=(S=6756540,E=6756543,C=4)
	CT(23871)=(S=6756544,E=6756547,C=2)
	CT(23872)=(S=6756548,E=6756551,C=0)
	CT(23873)=(S=6756552,E=6756555,C=4)
	CT(23874)=(S=6756556,E=6756559,C=8)
	CT(23875)=(S=6756560,E=6756563,C=0)
	CT(23876)=(S=6756564,E=6756567,C=74)
	CT(23877)=(S=6756568,E=6756571,C=8)
	CT(23878)=(S=6756572,E=6756575,C=1)
	CT(23879)=(S=6756576,E=6756579,C=8)
	CT(23880)=(S=6756580,E=6756583,C=1)
	CT(23881)=(S=6756584,E=6756591,C=4)
	CT(23882)=(S=6756592,E=6756595,C=0)
	CT(23883)=(S=6756596,E=6756599,C=11)
	CT(23884)=(S=6756600,E=6756607,C=1)
	CT(23885)=(S=6756608,E=6756615,C=4)
	CT(23886)=(S=6756616,E=6756617,C=1)
	CT(23887)=(S=6756618,E=6756619,C=4)
	CT(23888)=(S=6756620,E=6756623,C=155)
	CT(23889)=(S=6756624,E=6756627,C=6)
	CT(23890)=(S=6756628,E=6756651,C=1)
	CT(23891)=(S=6756652,E=6756655,C=4)
	CT(23892)=(S=6756656,E=6756659,C=1)
	CT(23893)=(S=6756661,E=6756661,C=0)
	CT(23894)=(S=6756662,E=6756663,C=74)
	CT(23895)=(S=6756664,E=6756667,C=0)
	CT(23896)=(S=6756668,E=6756671,C=8)
	CT(23897)=(S=6756672,E=6756679,C=1)
	CT(23898)=(S=6756680,E=6756683,C=8)
	CT(23899)=(S=6756684,E=6756687,C=155)
	CT(23900)=(S=6756688,E=6756691,C=84)
	CT(23901)=(S=6756692,E=6756695,C=4)
	CT(23902)=(S=6756696,E=6756699,C=2)
	CT(23903)=(S=6756700,E=6756703,C=4)
	CT(23904)=(S=6756704,E=6756707,C=3)
	CT(23905)=(S=6756708,E=6756711,C=2)
	CT(23906)=(S=6756712,E=6756715,C=8)
	CT(23907)=(S=6756716,E=6756719,C=74)
	CT(23908)=(S=6756720,E=6756722,C=0)
	CT(23909)=(S=6756724,E=6756727,C=73)
	CT(23910)=(S=6756732,E=6756743,C=4)
	CT(23911)=(S=6756744,E=6756747,C=77)
	CT(23912)=(S=6756748,E=6756751,C=193)
	CT(23913)=(S=6756756,E=6756759,C=1)
	CT(23914)=(S=6756760,E=6756763,C=4)
	CT(23915)=(S=6756764,E=6756767,C=1)
	CT(23916)=(S=6756768,E=6756771,C=0)
	CT(23917)=(S=6756772,E=6756772,C=84)
	CT(23918)=(S=6756773,E=6756775,C=74)
	CT(23919)=(S=6756776,E=6756783,C=4)
	CT(23920)=(S=6756784,E=6756787,C=10)
	CT(23921)=(S=6756788,E=6756791,C=0)
	CT(23922)=(S=6756792,E=6756795,C=84)
	CT(23923)=(S=6756796,E=6756799,C=73)
	CT(23924)=(S=6756800,E=6756805,C=74)
	CT(23925)=(S=6756806,E=6756807,C=10)
	CT(23926)=(S=6756808,E=6756811,C=7)
	CT(23927)=(S=6756812,E=6756815,C=4)
	CT(23928)=(S=6756816,E=6756819,C=74)
	CT(23929)=(S=6756820,E=6756823,C=0)
	CT(23930)=(S=6756824,E=6756827,C=1)
	CT(23931)=(S=6756828,E=6756828,C=4)
	CT(23932)=(S=6756829,E=6756829,C=0)
	CT(23933)=(S=6756830,E=6756831,C=74)
	CT(23934)=(S=6756832,E=6756835,C=4)
	CT(23935)=(S=6756836,E=6756837,C=160)
	CT(23936)=(S=6756838,E=6756839,C=4)
	CT(23937)=(S=6756840,E=6756847,C=9)
	CT(23938)=(S=6756848,E=6756851,C=3)
	CT(23939)=(S=6756852,E=6756855,C=73)
	CT(23940)=(S=6756856,E=6756859,C=84)
	CT(23941)=(S=6756860,E=6756863,C=8)
	CT(23942)=(S=6756864,E=6756867,C=1)
	CT(23943)=(S=6756868,E=6756871,C=74)
	CT(23944)=(S=6756872,E=6756875,C=4)
	CT(23945)=(S=6756876,E=6756879,C=74)
	CT(23946)=(S=6756880,E=6756883,C=73)
	CT(23947)=(S=6756884,E=6756887,C=3)
	CT(23948)=(S=6756888,E=6756891,C=7)
	CT(23949)=(S=6756892,E=6756899,C=2)
	CT(23950)=(S=6756900,E=6756903,C=10)
	CT(23951)=(S=6756904,E=6756907,C=7)
	CT(23952)=(S=6756908,E=6756911,C=5)
	CT(23953)=(S=6756912,E=6756923,C=4)
	CT(23954)=(S=6756924,E=6756927,C=0)
	CT(23955)=(S=6756928,E=6756931,C=1)
	CT(23956)=(S=6756932,E=6756935,C=0)
	CT(23957)=(S=6756936,E=6756937,C=4)
	CT(23958)=(S=6756938,E=6756939,C=5)
	CT(23959)=(S=6756944,E=6756951,C=77)
	CT(23960)=(S=6756952,E=6756955,C=0)
	CT(23961)=(S=6756956,E=6756958,C=8)
	CT(23962)=(S=6756959,E=6756959,C=165)
	CT(23963)=(S=6756960,E=6756963,C=4)
	CT(23964)=(S=6756964,E=6756966,C=74)
	CT(23965)=(S=6756967,E=6756967,C=73)
	CT(23966)=(S=6756968,E=6756975,C=4)
	CT(23967)=(S=6756976,E=6756979,C=84)
	CT(23968)=(S=6756984,E=6756991,C=8)
	CT(23969)=(S=6756992,E=6756995,C=74)
	CT(23970)=(S=6757000,E=6757003,C=84)
	CT(23971)=(S=6757004,E=6757007,C=4)
	CT(23972)=(S=6757008,E=6757011,C=8)
	CT(23973)=(S=6757012,E=6757015,C=5)
	CT(23974)=(S=6757016,E=6757019,C=2)
	CT(23975)=(S=6757020,E=6757027,C=1)
	CT(23976)=(S=6757028,E=6757031,C=2)
	CT(23977)=(S=6757032,E=6757033,C=4)
	CT(23978)=(S=6757034,E=6757035,C=5)
	CT(23979)=(S=6757036,E=6757039,C=0)
	CT(23980)=(S=6757040,E=6757043,C=5)
	CT(23981)=(S=6757044,E=6757047,C=3)
	CT(23982)=(S=6757048,E=6757051,C=77)
	CT(23983)=(S=6757052,E=6757055,C=2)
	CT(23984)=(S=6757056,E=6757059,C=92)
	CT(23985)=(S=6757060,E=6757063,C=5)
	CT(23986)=(S=6757064,E=6757065,C=4)
	CT(23987)=(S=6757066,E=6757067,C=0)
	CT(23988)=(S=6757068,E=6757071,C=4)
	CT(23989)=(S=6757072,E=6757079,C=74)
	CT(23990)=(S=6757080,E=6757083,C=4)
	CT(23991)=(S=6757084,E=6757087,C=8)
	CT(23992)=(S=6757088,E=6757091,C=4)
	CT(23993)=(S=6757092,E=6757095,C=1)
	CT(23994)=(S=6757096,E=6757099,C=4)
	CT(23995)=(S=6757100,E=6757103,C=0)
	CT(23996)=(S=6757104,E=6757107,C=1)
	CT(23997)=(S=6757108,E=6757111,C=84)
	CT(23998)=(S=6757112,E=6757115,C=5)
	CT(23999)=(S=6757116,E=6757119,C=11)
	CT(24000)=(S=6757120,E=6757120,C=8)
	CT(24001)=(S=6757121,E=6757121,C=0)
	CT(24002)=(S=6757122,E=6757122,C=4)
	CT(24003)=(S=6757123,E=6757123,C=74)
	CT(24004)=(S=6757124,E=6757127,C=1)
	CT(24005)=(S=6757128,E=6757131,C=4)
	CT(24006)=(S=6757132,E=6757135,C=1)
	CT(24007)=(S=6757136,E=6757139,C=4)
	CT(24008)=(S=6757140,E=6757143,C=77)
	CT(24009)=(S=6757144,E=6757147,C=1)
	CT(24010)=(S=6757148,E=6757155,C=0)
	CT(24011)=(S=6757156,E=6757156,C=74)
	CT(24012)=(S=6757157,E=6757157,C=73)
	CT(24013)=(S=6757158,E=6757159,C=0)
	CT(24014)=(S=6757160,E=6757163,C=2)
	CT(24015)=(S=6757164,E=6757167,C=7)
	CT(24016)=(S=6757172,E=6757175,C=73)
	CT(24017)=(S=6757176,E=6757179,C=1)
	CT(24018)=(S=6757180,E=6757187,C=11)
	CT(24019)=(S=6757188,E=6757191,C=2)
	CT(24020)=(S=6757192,E=6757195,C=5)
	CT(24021)=(S=6757200,E=6757203,C=2)
	CT(24022)=(S=6757204,E=6757207,C=4)
	CT(24023)=(S=6757208,E=6757211,C=73)
	CT(24024)=(S=6757212,E=6757215,C=0)
	CT(24025)=(S=6757216,E=6757219,C=1)
	CT(24026)=(S=6757220,E=6757223,C=3)
	CT(24027)=(S=6757224,E=6757227,C=5)
	CT(24028)=(S=6757228,E=6757231,C=8)
	CT(24029)=(S=6757232,E=6757235,C=5)
	CT(24030)=(S=6757236,E=6757239,C=155)
	CT(24031)=(S=6757240,E=6757240,C=4)
	CT(24032)=(S=6757241,E=6757241,C=73)
	CT(24033)=(S=6757242,E=6757243,C=10)
	CT(24034)=(S=6757244,E=6757247,C=8)
	CT(24035)=(S=6757248,E=6757251,C=6)
	CT(24036)=(S=6757252,E=6757255,C=0)
	CT(24037)=(S=6757256,E=6757259,C=2)
	CT(24038)=(S=6757260,E=6757263,C=4)
	CT(24039)=(S=6757268,E=6757271,C=6)
	CT(24040)=(S=6757272,E=6757275,C=0)
	CT(24041)=(S=6757276,E=6757279,C=2)
	CT(24042)=(S=6757280,E=6757283,C=0)
	CT(24043)=(S=6757284,E=6757291,C=4)
	CT(24044)=(S=6757292,E=6757295,C=0)
	CT(24045)=(S=6757296,E=6757303,C=8)
	CT(24046)=(S=6757308,E=6757311,C=0)
	CT(24047)=(S=6757312,E=6757315,C=2)
	CT(24048)=(S=6757316,E=6757319,C=0)
	CT(24049)=(S=6757320,E=6757323,C=3)
	CT(24050)=(S=6757324,E=6757325,C=5)
	CT(24051)=(S=6757326,E=6757327,C=74)
	CT(24052)=(S=6757328,E=6757331,C=1)
	CT(24053)=(S=6757336,E=6757339,C=73)
	CT(24054)=(S=6757340,E=6757343,C=2)
	CT(24055)=(S=6757344,E=6757347,C=0)
	CT(24056)=(S=6757348,E=6757351,C=7)
	CT(24057)=(S=6757352,E=6757355,C=4)
	CT(24058)=(S=6757356,E=6757359,C=11)
	CT(24059)=(S=6757360,E=6757363,C=1)
	CT(24060)=(S=6757364,E=6757367,C=2)
	CT(24061)=(S=6757368,E=6757373,C=74)
	CT(24062)=(S=6757375,E=6757375,C=4)
	CT(24063)=(S=6757376,E=6757379,C=86)
	CT(24064)=(S=6757380,E=6757387,C=1)
	CT(24065)=(S=6757388,E=6757391,C=74)
	CT(24066)=(S=6757392,E=6757395,C=10)
	CT(24067)=(S=6757396,E=6757396,C=0)
	CT(24068)=(S=6757397,E=6757399,C=74)
	CT(24069)=(S=6757400,E=6757407,C=2)
	CT(24070)=(S=6757408,E=6757415,C=11)
	CT(24071)=(S=6757416,E=6757419,C=0)
	CT(24072)=(S=6757420,E=6757423,C=8)
	CT(24073)=(S=6757424,E=6757427,C=0)
	CT(24074)=(S=6757430,E=6757431,C=7)
	CT(24075)=(S=6757432,E=6757435,C=74)
	CT(24076)=(S=6757436,E=6757443,C=6)
	CT(24077)=(S=6757448,E=6757448,C=0)
	CT(24078)=(S=6757449,E=6757449,C=2)
	CT(24079)=(S=6757450,E=6757451,C=74)
	CT(24080)=(S=6757452,E=6757455,C=8)
	CT(24081)=(S=6757456,E=6757459,C=74)
	CT(24082)=(S=6757460,E=6757463,C=75)
	CT(24083)=(S=6757464,E=6757467,C=5)
	CT(24084)=(S=6757468,E=6757468,C=0)
	CT(24085)=(S=6757469,E=6757471,C=74)
	CT(24086)=(S=6757472,E=6757475,C=2)
	CT(24087)=(S=6757476,E=6757479,C=3)
	CT(24088)=(S=6757480,E=6757480,C=74)
	CT(24089)=(S=6757481,E=6757481,C=4)
	CT(24090)=(S=6757482,E=6757495,C=74)
	CT(24091)=(S=6757496,E=6757499,C=84)
	CT(24092)=(S=6757500,E=6757503,C=3)
	CT(24093)=(S=6757504,E=6757507,C=0)
	CT(24094)=(S=6757508,E=6757511,C=89)
	CT(24095)=(S=6757512,E=6757515,C=11)
	CT(24096)=(S=6757516,E=6757519,C=4)
	CT(24097)=(S=6757520,E=6757523,C=8)
	CT(24098)=(S=6757524,E=6757525,C=74)
	CT(24099)=(S=6757526,E=6757531,C=77)
	CT(24100)=(S=6757532,E=6757535,C=4)
	CT(24101)=(S=6757536,E=6757536,C=0)
	CT(24102)=(S=6757537,E=6757537,C=74)
	CT(24103)=(S=6757538,E=6757538,C=5)
	CT(24104)=(S=6757539,E=6757539,C=74)
	CT(24105)=(S=6757540,E=6757547,C=2)
	CT(24106)=(S=6757548,E=6757551,C=11)
	CT(24107)=(S=6757552,E=6757555,C=74)
	CT(24108)=(S=6757556,E=6757559,C=4)
	CT(24109)=(S=6757564,E=6757571,C=74)
	CT(24110)=(S=6757572,E=6757575,C=0)
	CT(24111)=(S=6757576,E=6757577,C=9)
	CT(24112)=(S=6757578,E=6757579,C=10)
	CT(24113)=(S=6757580,E=6757583,C=1)
	CT(24114)=(S=6757584,E=6757587,C=7)
	CT(24115)=(S=6757592,E=6757593,C=0)
	CT(24116)=(S=6757594,E=6757603,C=74)
	CT(24117)=(S=6757604,E=6757615,C=2)
	CT(24118)=(S=6757616,E=6757617,C=4)
	CT(24119)=(S=6757618,E=6757619,C=86)
	CT(24120)=(S=6757624,E=6757627,C=2)
	CT(24121)=(S=6757628,E=6757631,C=4)
	CT(24122)=(S=6757636,E=6757639,C=74)
	CT(24123)=(S=6757640,E=6757643,C=84)
	CT(24124)=(S=6757644,E=6757647,C=8)
	CT(24125)=(S=6757648,E=6757651,C=1)
	CT(24126)=(S=6757652,E=6757655,C=10)
	CT(24127)=(S=6757656,E=6757659,C=7)
	CT(24128)=(S=6757660,E=6757661,C=4)
	CT(24129)=(S=6757662,E=6757663,C=73)
	CT(24130)=(S=6757664,E=6757671,C=4)
	CT(24131)=(S=6757672,E=6757675,C=0)
	CT(24132)=(S=6757676,E=6757679,C=4)
	CT(24133)=(S=6757680,E=6757683,C=2)
	CT(24134)=(S=6757684,E=6757687,C=0)
	CT(24135)=(S=6757688,E=6757691,C=5)
	CT(24136)=(S=6757692,E=6757695,C=156)
	CT(24137)=(S=6757696,E=6757699,C=0)
	CT(24138)=(S=6757700,E=6757703,C=7)
	CT(24139)=(S=6757704,E=6757707,C=8)
	CT(24140)=(S=6757708,E=6757711,C=0)
	CT(24141)=(S=6757712,E=6757715,C=10)
	CT(24142)=(S=6757716,E=6757721,C=0)
	CT(24143)=(S=6757724,E=6757727,C=8)
	CT(24144)=(S=6757728,E=6757731,C=4)
	CT(24145)=(S=6757732,E=6757735,C=0)
	CT(24146)=(S=6757736,E=6757737,C=84)
	CT(24147)=(S=6757738,E=6757743,C=7)
	CT(24148)=(S=6757744,E=6757744,C=0)
	CT(24149)=(S=6757745,E=6757745,C=2)
	CT(24150)=(S=6757746,E=6757746,C=7)
	CT(24151)=(S=6757747,E=6757747,C=0)
	CT(24152)=(S=6757748,E=6757751,C=4)
	CT(24153)=(S=6757752,E=6757755,C=74)
	CT(24154)=(S=6757756,E=6757759,C=84)
	CT(24155)=(S=6757760,E=6757771,C=1)
	CT(24156)=(S=6757772,E=6757775,C=8)
	CT(24157)=(S=6757776,E=6757779,C=92)
	CT(24158)=(S=6757780,E=6757783,C=74)
	CT(24159)=(S=6757784,E=6757787,C=90)
	CT(24160)=(S=6757788,E=6757791,C=4)
	CT(24161)=(S=6757792,E=6757794,C=0)
	CT(24162)=(S=6757795,E=6757795,C=77)
	CT(24163)=(S=6757796,E=6757799,C=74)
	CT(24164)=(S=6757800,E=6757801,C=0)
	CT(24165)=(S=6757802,E=6757803,C=74)
	CT(24166)=(S=6757804,E=6757807,C=0)
	CT(24167)=(S=6757808,E=6757815,C=2)
	CT(24168)=(S=6757816,E=6757819,C=74)
	CT(24169)=(S=6757820,E=6757823,C=3)
	CT(24170)=(S=6757824,E=6757827,C=0)
	CT(24171)=(S=6757828,E=6757828,C=74)
	CT(24172)=(S=6757829,E=6757829,C=4)
	CT(24173)=(S=6757830,E=6757830,C=5)
	CT(24174)=(S=6757831,E=6757831,C=92)
	CT(24175)=(S=6757832,E=6757839,C=0)
	CT(24176)=(S=6757840,E=6757843,C=4)
	CT(24177)=(S=6757844,E=6757847,C=74)
	CT(24178)=(S=6757848,E=6757851,C=0)
	CT(24179)=(S=6757852,E=6757855,C=4)
	CT(24180)=(S=6757856,E=6757859,C=161)
	CT(24181)=(S=6757860,E=6757863,C=74)
	CT(24182)=(S=6757864,E=6757867,C=4)
	CT(24183)=(S=6757872,E=6757875,C=5)
	CT(24184)=(S=6757876,E=6757879,C=73)
	CT(24185)=(S=6757880,E=6757881,C=4)
	CT(24186)=(S=6757882,E=6757887,C=10)
	CT(24187)=(S=6757888,E=6757891,C=74)
	CT(24188)=(S=6757892,E=6757895,C=8)
	CT(24189)=(S=6757896,E=6757899,C=2)
	CT(24190)=(S=6757900,E=6757903,C=4)
	CT(24191)=(S=6757904,E=6757907,C=8)
	CT(24192)=(S=6757908,E=6757911,C=1)
	CT(24193)=(S=6757912,E=6757915,C=8)
	CT(24194)=(S=6757916,E=6757919,C=84)
	CT(24195)=(S=6757920,E=6757923,C=7)
	CT(24196)=(S=6757924,E=6757927,C=11)
	CT(24197)=(S=6757928,E=6757931,C=8)
	CT(24198)=(S=6757932,E=6757935,C=9)
	CT(24199)=(S=6757936,E=6757939,C=8)
	CT(24200)=(S=6757940,E=6757943,C=2)
	CT(24201)=(S=6757944,E=6757947,C=74)
	CT(24202)=(S=6757948,E=6757951,C=2)
	CT(24203)=(S=6757956,E=6757959,C=8)
	CT(24204)=(S=6757960,E=6757963,C=2)
	CT(24205)=(S=6757964,E=6757967,C=8)
	CT(24206)=(S=6757968,E=6757971,C=4)
	CT(24207)=(S=6757972,E=6757983,C=74)
	CT(24208)=(S=6757984,E=6757987,C=1)
	CT(24209)=(S=6757988,E=6757995,C=8)
	CT(24210)=(S=6757996,E=6757999,C=6)
	CT(24211)=(S=6758000,E=6758003,C=165)
	CT(24212)=(S=6758008,E=6758008,C=0)
	CT(24213)=(S=6758009,E=6758009,C=7)
	CT(24214)=(S=6758010,E=6758011,C=74)
	CT(24215)=(S=6758012,E=6758015,C=3)
	CT(24216)=(S=6758016,E=6758023,C=9)
	CT(24217)=(S=6758024,E=6758027,C=89)
	CT(24218)=(S=6758028,E=6758031,C=4)
	CT(24219)=(S=6758032,E=6758035,C=74)
	CT(24220)=(S=6758036,E=6758039,C=1)
	CT(24221)=(S=6758040,E=6758043,C=4)
	CT(24222)=(S=6758044,E=6758051,C=6)
	CT(24223)=(S=6758052,E=6758055,C=2)
	CT(24224)=(S=6758056,E=6758056,C=0)
	CT(24225)=(S=6758057,E=6758057,C=84)
	CT(24226)=(S=6758058,E=6758071,C=74)
	CT(24227)=(S=6758072,E=6758075,C=0)
	CT(24228)=(S=6758076,E=6758079,C=84)
	CT(24229)=(S=6758080,E=6758083,C=4)
	CT(24230)=(S=6758088,E=6758091,C=1)
	CT(24231)=(S=6758092,E=6758095,C=6)
	CT(24232)=(S=6758096,E=6758099,C=7)
	CT(24233)=(S=6758100,E=6758100,C=8)
	CT(24234)=(S=6758102,E=6758102,C=0)
	CT(24235)=(S=6758103,E=6758103,C=7)
	CT(24236)=(S=6758108,E=6758111,C=74)
	CT(24237)=(S=6758116,E=6758119,C=1)
	CT(24238)=(S=6758120,E=6758123,C=8)
	CT(24239)=(S=6758124,E=6758127,C=1)
	CT(24240)=(S=6758128,E=6758131,C=8)
	CT(24241)=(S=6758132,E=6758135,C=74)
	CT(24242)=(S=6758136,E=6758139,C=194)
	CT(24243)=(S=6758144,E=6758147,C=1)
	CT(24244)=(S=6758148,E=6758151,C=0)
	CT(24245)=(S=6758152,E=6758155,C=8)
	CT(24246)=(S=6758156,E=6758159,C=2)
	CT(24247)=(S=6758160,E=6758167,C=8)
	CT(24248)=(S=6758168,E=6758169,C=7)
	CT(24249)=(S=6758170,E=6758175,C=0)
	CT(24250)=(S=6758176,E=6758177,C=74)
	CT(24251)=(S=6758178,E=6758179,C=5)
	CT(24252)=(S=6758180,E=6758183,C=74)
	CT(24253)=(S=6758184,E=6758187,C=2)
	CT(24254)=(S=6758188,E=6758191,C=74)
	CT(24255)=(S=6758192,E=6758215,C=1)
	CT(24256)=(S=6758216,E=6758219,C=8)
	CT(24257)=(S=6758220,E=6758223,C=2)
	CT(24258)=(S=6758224,E=6758227,C=77)
	CT(24259)=(S=6758228,E=6758231,C=88)
	CT(24260)=(S=6758232,E=6758235,C=0)
	CT(24261)=(S=6758236,E=6758239,C=77)
	CT(24262)=(S=6758240,E=6758243,C=8)
	CT(24263)=(S=6758244,E=6758251,C=77)
	CT(24264)=(S=6758252,E=6758252,C=84)
	CT(24265)=(S=6758253,E=6758255,C=74)
	CT(24266)=(S=6758256,E=6758259,C=0)
	CT(24267)=(S=6758260,E=6758263,C=2)
	CT(24268)=(S=6758264,E=6758271,C=11)
	CT(24269)=(S=6758272,E=6758275,C=86)
	CT(24270)=(S=6758276,E=6758279,C=74)
	CT(24271)=(S=6758280,E=6758283,C=8)
	CT(24272)=(S=6758284,E=6758287,C=2)
	CT(24273)=(S=6758288,E=6758291,C=4)
	CT(24274)=(S=6758292,E=6758295,C=1)
	CT(24275)=(S=6758296,E=6758299,C=84)
	CT(24276)=(S=6758300,E=6758303,C=74)
	CT(24277)=(S=6758304,E=6758307,C=1)
	CT(24278)=(S=6758308,E=6758311,C=74)
	CT(24279)=(S=6758312,E=6758315,C=1)
	CT(24280)=(S=6758316,E=6758319,C=7)
	CT(24281)=(S=6758320,E=6758323,C=84)
	CT(24282)=(S=6758324,E=6758327,C=6)
	CT(24283)=(S=6758328,E=6758331,C=8)
	CT(24284)=(S=6758332,E=6758335,C=4)
	CT(24285)=(S=6758336,E=6758339,C=2)
	CT(24286)=(S=6758340,E=6758343,C=9)
	CT(24287)=(S=6758344,E=6758347,C=1)
	CT(24288)=(S=6758348,E=6758351,C=74)
	CT(24289)=(S=6758352,E=6758355,C=84)
	CT(24290)=(S=6758356,E=6758359,C=4)
	CT(24291)=(S=6758360,E=6758363,C=2)
	CT(24292)=(S=6758364,E=6758367,C=8)
	CT(24293)=(S=6758368,E=6758371,C=74)
	CT(24294)=(S=6758372,E=6758375,C=7)
	CT(24295)=(S=6758376,E=6758379,C=74)
	CT(24296)=(S=6758384,E=6758387,C=8)
	CT(24297)=(S=6758388,E=6758391,C=2)
	CT(24298)=(S=6758392,E=6758395,C=74)
	CT(24299)=(S=6758396,E=6758399,C=2)
	CT(24300)=(S=6758400,E=6759219,C=1)
	CT(24301)=(S=6759220,E=6759223,C=4)
	CT(24302)=(S=6759224,E=6759227,C=7)
	CT(24303)=(S=6759228,E=6759231,C=2)
	CT(24304)=(S=6759232,E=6759235,C=11)
	CT(24305)=(S=6759236,E=6759239,C=4)
	CT(24306)=(S=6759240,E=6759247,C=8)
	CT(24307)=(S=6759248,E=6759251,C=0)
	CT(24308)=(S=6759252,E=6759255,C=8)
	CT(24309)=(S=6759256,E=6759257,C=73)
	CT(24310)=(S=6759258,E=6759258,C=4)
	CT(24311)=(S=6759259,E=6759259,C=2)
	CT(24312)=(S=6759260,E=6759263,C=0)
	CT(24313)=(S=6759264,E=6759267,C=3)
	CT(24314)=(S=6759268,E=6759271,C=4)
	CT(24315)=(S=6759272,E=6759275,C=1)
	CT(24316)=(S=6759276,E=6759279,C=84)
	CT(24317)=(S=6759280,E=6759281,C=0)
	CT(24318)=(S=6759282,E=6759283,C=160)
	CT(24319)=(S=6759284,E=6759287,C=1)
	CT(24320)=(S=6759288,E=6759289,C=4)
	CT(24321)=(S=6759290,E=6759290,C=8)
	CT(24322)=(S=6759291,E=6759291,C=4)
	CT(24323)=(S=6759292,E=6759295,C=8)
	CT(24324)=(S=6759296,E=6759299,C=0)
	CT(24325)=(S=6759300,E=6759303,C=4)
	CT(24326)=(S=6759304,E=6759307,C=5)
	CT(24327)=(S=6759308,E=6759309,C=74)
	CT(24328)=(S=6759310,E=6759310,C=4)
	CT(24329)=(S=6759311,E=6759311,C=5)
	CT(24330)=(S=6759312,E=6759319,C=8)
	CT(24331)=(S=6759320,E=6759323,C=0)
	CT(24332)=(S=6759324,E=6759327,C=84)
	CT(24333)=(S=6759328,E=6759333,C=4)
	CT(24334)=(S=6759334,E=6759335,C=89)
	CT(24335)=(S=6759336,E=6759339,C=84)
	CT(24336)=(S=6759340,E=6759343,C=8)
	CT(24337)=(S=6759344,E=6759347,C=0)
	CT(24338)=(S=6759348,E=6759351,C=7)
	CT(24339)=(S=6759352,E=6759355,C=8)
	CT(24340)=(S=6759356,E=6759359,C=2)
	CT(24341)=(S=6759360,E=6759367,C=4)
	CT(24342)=(S=6759368,E=6759371,C=1)
	CT(24343)=(S=6759372,E=6759375,C=9)
	CT(24344)=(S=6759376,E=6759383,C=77)
	CT(24345)=(S=6759384,E=6759384,C=73)
	CT(24346)=(S=6759385,E=6759385,C=4)
	CT(24347)=(S=6759386,E=6759386,C=74)
	CT(24348)=(S=6759387,E=6759387,C=0)
	CT(24349)=(S=6759388,E=6759391,C=1)
	CT(24350)=(S=6759392,E=6759399,C=0)
	CT(24351)=(S=6759400,E=6759403,C=4)
	CT(24352)=(S=6759404,E=6759407,C=74)
	CT(24353)=(S=6759408,E=6759415,C=4)
	CT(24354)=(S=6759416,E=6759419,C=7)
	CT(24355)=(S=6759420,E=6759423,C=8)
	CT(24356)=(S=6759424,E=6759431,C=0)
	CT(24357)=(S=6759432,E=6759435,C=74)
	CT(24358)=(S=6759436,E=6759437,C=0)
	CT(24359)=(S=6759438,E=6759439,C=74)
	CT(24360)=(S=6759440,E=6759443,C=10)
	CT(24361)=(S=6759444,E=6759447,C=1)
	CT(24362)=(S=6759448,E=6759451,C=84)
	CT(24363)=(S=6759452,E=6759455,C=1)
	CT(24364)=(S=6759456,E=6759459,C=74)
	CT(24365)=(S=6759460,E=6759463,C=1)
	CT(24366)=(S=6759464,E=6759467,C=74)
	CT(24367)=(S=6759468,E=6759475,C=4)
	CT(24368)=(S=6759476,E=6759479,C=8)
	CT(24369)=(S=6759480,E=6759491,C=1)
	CT(24370)=(S=6759492,E=6759492,C=74)
	CT(24371)=(S=6759493,E=6759493,C=0)
	CT(24372)=(S=6759494,E=6759494,C=8)
	CT(24373)=(S=6759495,E=6759495,C=4)
	CT(24374)=(S=6759496,E=6759499,C=1)
	CT(24375)=(S=6759500,E=6759511,C=4)
	CT(24376)=(S=6759512,E=6759515,C=8)
	CT(24377)=(S=6759516,E=6759519,C=7)
	CT(24378)=(S=6759520,E=6759523,C=1)
	CT(24379)=(S=6759524,E=6759527,C=84)
	CT(24380)=(S=6759528,E=6759531,C=4)
	CT(24381)=(S=6759532,E=6759539,C=8)
	CT(24382)=(S=6759540,E=6759543,C=9)
	CT(24383)=(S=6759544,E=6759551,C=4)
	CT(24384)=(S=6759552,E=6759552,C=73)
	CT(24385)=(S=6759553,E=6759555,C=0)
	CT(24386)=(S=6759556,E=6759563,C=1)
	CT(24387)=(S=6759564,E=6759567,C=2)
	CT(24388)=(S=6759568,E=6759571,C=160)
	CT(24389)=(S=6759572,E=6759572,C=0)
	CT(24390)=(S=6759573,E=6759573,C=4)
	CT(24391)=(S=6759574,E=6759575,C=10)
	CT(24392)=(S=6759576,E=6759579,C=0)
	CT(24393)=(S=6759580,E=6759583,C=74)
	CT(24394)=(S=6759584,E=6759671,C=1)
	CT(24395)=(S=6759672,E=6759673,C=0)
	CT(24396)=(S=6759674,E=6759674,C=73)
	CT(24397)=(S=6759675,E=6759675,C=4)
	CT(24398)=(S=6759676,E=6759679,C=0)
	CT(24399)=(S=6759680,E=6759683,C=1)
	CT(24400)=(S=6759684,E=6759687,C=8)
	CT(24401)=(S=6759688,E=6759689,C=0)
	CT(24402)=(S=6759690,E=6759691,C=2)
	CT(24403)=(S=6759692,E=6759699,C=1)
	CT(24404)=(S=6759700,E=6759701,C=4)
	CT(24405)=(S=6759702,E=6759703,C=0)
	CT(24406)=(S=6759704,E=6759707,C=1)
	CT(24407)=(S=6759708,E=6759715,C=11)
	CT(24408)=(S=6759716,E=6759719,C=9)
	CT(24409)=(S=6759720,E=6759723,C=8)
	CT(24410)=(S=6759724,E=6759727,C=1)
	CT(24411)=(S=6759728,E=6759731,C=10)
	CT(24412)=(S=6759732,E=6759739,C=1)
	CT(24413)=(S=6759740,E=6759743,C=8)
	CT(24414)=(S=6759744,E=6759747,C=2)
	CT(24415)=(S=6759748,E=6759751,C=0)
	CT(24416)=(S=6759752,E=6759755,C=1)
	CT(24417)=(S=6759756,E=6759763,C=4)
	CT(24418)=(S=6759764,E=6759767,C=8)
	CT(24419)=(S=6759768,E=6759771,C=4)
	CT(24420)=(S=6759772,E=6759775,C=2)
	CT(24421)=(S=6759776,E=6759777,C=5)
	CT(24422)=(S=6759778,E=6759778,C=74)
	CT(24423)=(S=6759779,E=6759779,C=0)
	CT(24424)=(S=6759780,E=6759787,C=1)
	CT(24425)=(S=6759788,E=6759799,C=2)
	CT(24426)=(S=6759800,E=6759803,C=0)
	CT(24427)=(S=6759804,E=6759807,C=1)
	CT(24428)=(S=6759808,E=6759811,C=0)
	CT(24429)=(S=6759812,E=6759815,C=2)
	CT(24430)=(S=6759816,E=6759847,C=1)
	CT(24431)=(S=6759848,E=6759851,C=74)
	CT(24432)=(S=6759852,E=6759859,C=1)
	CT(24433)=(S=6759860,E=6759863,C=4)
	CT(24434)=(S=6759864,E=6759867,C=8)
	CT(24435)=(S=6759876,E=6759879,C=7)
	CT(24436)=(S=6759880,E=6759881,C=4)
	CT(24437)=(S=6759882,E=6759883,C=0)
	CT(24438)=(S=6759884,E=6759887,C=73)
	CT(24439)=(S=6759888,E=6759903,C=1)
	CT(24440)=(S=6759904,E=6759907,C=74)
	CT(24441)=(S=6759908,E=6759908,C=0)
	CT(24442)=(S=6759909,E=6759911,C=74)
	CT(24443)=(S=6759912,E=6759915,C=8)
	CT(24444)=(S=6759916,E=6759923,C=2)
	CT(24445)=(S=6759924,E=6759927,C=74)
	CT(24446)=(S=6759928,E=6759939,C=1)
	CT(24447)=(S=6759940,E=6759944,C=4)
	CT(24448)=(S=6759945,E=6759947,C=0)
	CT(24449)=(S=6759948,E=6759951,C=4)
	CT(24450)=(S=6759952,E=6759955,C=84)
	CT(24451)=(S=6759956,E=6759959,C=8)
	CT(24452)=(S=6759960,E=6759963,C=6)
	CT(24453)=(S=6759964,E=6759967,C=8)
	CT(24454)=(S=6759968,E=6759971,C=1)
	CT(24455)=(S=6759972,E=6759975,C=4)
	CT(24456)=(S=6759976,E=6759983,C=1)
	CT(24457)=(S=6759984,E=6759985,C=4)
	CT(24458)=(S=6759988,E=6759991,C=2)
	CT(24459)=(S=6759992,E=6759995,C=1)
	CT(24460)=(S=6759996,E=6759999,C=5)
	CT(24461)=(S=6760000,E=6760003,C=8)
	CT(24462)=(S=6760004,E=6760007,C=4)
	CT(24463)=(S=6760008,E=6760011,C=7)
	CT(24464)=(S=6760012,E=6760015,C=1)
	CT(24465)=(S=6760016,E=6760019,C=8)
	CT(24466)=(S=6760020,E=6760023,C=1)
	CT(24467)=(S=6760024,E=6760025,C=4)
	CT(24468)=(S=6760026,E=6760027,C=8)
	CT(24469)=(S=6760028,E=6760035,C=1)
	CT(24470)=(S=6760036,E=6760039,C=74)
	CT(24471)=(S=6760040,E=6760043,C=8)
	CT(24472)=(S=6760044,E=6760047,C=4)
	CT(24473)=(S=6760048,E=6760051,C=86)
	CT(24474)=(S=6760052,E=6760055,C=1)
	CT(24475)=(S=6760056,E=6760059,C=0)
	CT(24476)=(S=6760060,E=6760067,C=4)
	CT(24477)=(S=6760068,E=6760071,C=1)
	CT(24478)=(S=6760072,E=6760075,C=11)
	CT(24479)=(S=6760076,E=6760079,C=1)
	CT(24480)=(S=6760080,E=6760080,C=8)
	CT(24481)=(S=6760081,E=6760081,C=0)
	CT(24482)=(S=6760082,E=6760083,C=9)
	CT(24483)=(S=6760084,E=6760085,C=0)
	CT(24484)=(S=6760086,E=6760086,C=4)
	CT(24485)=(S=6760087,E=6760087,C=10)
	CT(24486)=(S=6760088,E=6760099,C=8)
	CT(24487)=(S=6760100,E=6760103,C=73)
	CT(24488)=(S=6760104,E=6760107,C=8)
	CT(24489)=(S=6760108,E=6760111,C=10)
	CT(24490)=(S=6760112,E=6760115,C=4)
	CT(24491)=(S=6760116,E=6760119,C=2)
	CT(24492)=(S=6760120,E=6760123,C=7)
	CT(24493)=(S=6760124,E=6760124,C=10)
	CT(24494)=(S=6760125,E=6760127,C=4)
	CT(24495)=(S=6760128,E=6760131,C=5)
	CT(24496)=(S=6760132,E=6760135,C=75)
	CT(24497)=(S=6760136,E=6760143,C=4)
	CT(24498)=(S=6760144,E=6760147,C=2)
	CT(24499)=(S=6760148,E=6760151,C=10)
	CT(24500)=(S=6760152,E=6760153,C=0)
	CT(24501)=(S=6760154,E=6760155,C=160)
	CT(24502)=(S=6760156,E=6760171,C=1)
	CT(24503)=(S=6760172,E=6760175,C=75)
	CT(24504)=(S=6760176,E=6760183,C=4)
	CT(24505)=(S=6760184,E=6760187,C=2)
	CT(24506)=(S=6760188,E=6760191,C=1)
	CT(24507)=(S=6760192,E=6760197,C=4)
	CT(24508)=(S=6760198,E=6760198,C=5)
	CT(24509)=(S=6760199,E=6760199,C=7)
	CT(24510)=(S=6760200,E=6760203,C=4)
	CT(24511)=(S=6760204,E=6760207,C=74)
	CT(24512)=(S=6760208,E=6760211,C=1)
	CT(24513)=(S=6760212,E=6760215,C=73)
	CT(24514)=(S=6760216,E=6760219,C=8)
	CT(24515)=(S=6760220,E=6760223,C=4)
	CT(24516)=(S=6760224,E=6760227,C=9)
	CT(24517)=(S=6760228,E=6760231,C=6)
	CT(24518)=(S=6760232,E=6760233,C=8)
	CT(24519)=(S=6760234,E=6760235,C=7)
	CT(24520)=(S=6760236,E=6760239,C=73)
	CT(24521)=(S=6760240,E=6760244,C=74)
	CT(24522)=(S=6760245,E=6760245,C=0)
	CT(24523)=(S=6760246,E=6760246,C=4)
	CT(24524)=(S=6760247,E=6760247,C=0)
	CT(24525)=(S=6760248,E=6760251,C=8)
	CT(24526)=(S=6760252,E=6760255,C=0)
	CT(24527)=(S=6760256,E=6760259,C=1)
	CT(24528)=(S=6760260,E=6760271,C=8)
	CT(24529)=(S=6760272,E=6760275,C=77)
	CT(24530)=(S=6760276,E=6760279,C=0)
	CT(24531)=(S=6760280,E=6760283,C=1)
	CT(24532)=(S=6760284,E=6760291,C=11)
	CT(24533)=(S=6760292,E=6760303,C=1)
	CT(24534)=(S=6760304,E=6760304,C=0)
	CT(24535)=(S=6760305,E=6760305,C=2)
	CT(24536)=(S=6760306,E=6760307,C=73)
	CT(24537)=(S=6760308,E=6760311,C=4)
	CT(24538)=(S=6760312,E=6760313,C=0)
	CT(24539)=(S=6760314,E=6760315,C=5)
	CT(24540)=(S=6760316,E=6760319,C=2)
	CT(24541)=(S=6760320,E=6760327,C=4)
	CT(24542)=(S=6760328,E=6760330,C=0)
	CT(24543)=(S=6760331,E=6760331,C=4)
	CT(24544)=(S=6760332,E=6760335,C=5)
	CT(24545)=(S=6760336,E=6760339,C=1)
	CT(24546)=(S=6760340,E=6760347,C=8)
	CT(24547)=(S=6760348,E=6760351,C=4)
	CT(24548)=(S=6760352,E=6760383,C=1)
	CT(24549)=(S=6760384,E=6760391,C=2)
	CT(24550)=(S=6760392,E=6760427,C=1)
	CT(24551)=(S=6760428,E=6760429,C=4)
	CT(24552)=(S=6760430,E=6760431,C=0)
	CT(24553)=(S=6760432,E=6760439,C=4)
	CT(24554)=(S=6760440,E=6760442,C=0)
	CT(24555)=(S=6760443,E=6760443,C=4)
	CT(24556)=(S=6760444,E=6760447,C=163)
	CT(24557)=(S=6760448,E=6760451,C=0)
	CT(24558)=(S=6760452,E=6760459,C=8)
	CT(24559)=(S=6760460,E=6760495,C=1)
	CT(24560)=(S=6760496,E=6760499,C=4)
	CT(24561)=(S=6760500,E=6760501,C=0)
	CT(24562)=(S=6760502,E=6760503,C=74)
	CT(24563)=(S=6760504,E=6760515,C=4)
	CT(24564)=(S=6760516,E=6760517,C=74)
	CT(24565)=(S=6760518,E=6760519,C=0)
	CT(24566)=(S=6760520,E=6760523,C=4)
	CT(24567)=(S=6760524,E=6760527,C=84)
	CT(24568)=(S=6760528,E=6760528,C=4)
	CT(24569)=(S=6760529,E=6760529,C=73)
	CT(24570)=(S=6760530,E=6760531,C=84)
	CT(24571)=(S=6760532,E=6760535,C=8)
	CT(24572)=(S=6760536,E=6760539,C=1)
	CT(24573)=(S=6760540,E=6760543,C=74)
	CT(24574)=(S=6760544,E=6760547,C=2)
	CT(24575)=(S=6760548,E=6760551,C=1)
	CT(24576)=(S=6760552,E=6760555,C=2)
	CT(24577)=(S=6760556,E=6760556,C=74)
	CT(24578)=(S=6760557,E=6760557,C=0)
	CT(24579)=(S=6760558,E=6760559,C=8)
	CT(24580)=(S=6760560,E=6760563,C=1)
	CT(24581)=(S=6760564,E=6760567,C=3)
	CT(24582)=(S=6760568,E=6760571,C=74)
	CT(24583)=(S=6760572,E=6760579,C=2)
	CT(24584)=(S=6760580,E=6760599,C=3)
	CT(24585)=(S=6760600,E=6760603,C=8)
	CT(24586)=(S=6760604,E=6760607,C=2)
	CT(24587)=(S=6760608,E=6760611,C=8)
	CT(24588)=(S=6760612,E=6760613,C=10)
	CT(24589)=(S=6760614,E=6760615,C=0)
	CT(24590)=(S=6760616,E=6760619,C=4)
	CT(24591)=(S=6760620,E=6760627,C=8)
	CT(24592)=(S=6760628,E=6760639,C=3)
	CT(24593)=(S=6760640,E=6760643,C=1)
	CT(24594)=(S=6760644,E=6760655,C=4)
	CT(24595)=(S=6760656,E=6760659,C=8)
	CT(24596)=(S=6760660,E=6760663,C=1)
	CT(24597)=(S=6760664,E=6760667,C=2)
	CT(24598)=(S=6760668,E=6760671,C=1)
	CT(24599)=(S=6760672,E=6760672,C=0)
	CT(24600)=(S=6760673,E=6760673,C=8)
	CT(24601)=(S=6760674,E=6760675,C=84)
	CT(24602)=(S=6760676,E=6760707,C=1)
	CT(24603)=(S=6760708,E=6760711,C=8)
	CT(24604)=(S=6760712,E=6760713,C=89)
	CT(24605)=(S=6760714,E=6760715,C=4)
	CT(24606)=(S=6760716,E=6760719,C=7)
	CT(24607)=(S=6760720,E=6760723,C=1)
	CT(24608)=(S=6760724,E=6760751,C=4)
	CT(24609)=(S=6760752,E=6760755,C=8)
	CT(24610)=(S=6760756,E=6760759,C=1)
	CT(24611)=(S=6760760,E=6760761,C=4)
	CT(24612)=(S=6760762,E=6760763,C=3)
	CT(24613)=(S=6760764,E=6760767,C=2)
	CT(24614)=(S=6760768,E=6760771,C=8)
	CT(24615)=(S=6760772,E=6760775,C=7)
	CT(24616)=(S=6760776,E=6760779,C=2)
	CT(24617)=(S=6760780,E=6760783,C=7)
	CT(24618)=(S=6760784,E=6760787,C=8)
	CT(24619)=(S=6760788,E=6760791,C=3)
	CT(24620)=(S=6760792,E=6760807,C=4)
	CT(24621)=(S=6760808,E=6760811,C=8)
	CT(24622)=(S=6760812,E=6760812,C=0)
	CT(24623)=(S=6760813,E=6760814,C=74)
	CT(24624)=(S=6760815,E=6760815,C=84)
	CT(24625)=(S=6760816,E=6760819,C=2)
	CT(24626)=(S=6760820,E=6760823,C=1)
	CT(24627)=(S=6760824,E=6760827,C=74)
	CT(24628)=(S=6760828,E=6760831,C=8)
	CT(24629)=(S=6760832,E=6760835,C=7)
	CT(24630)=(S=6760836,E=6760839,C=8)
	CT(24631)=(S=6760840,E=6760843,C=2)
	CT(24632)=(S=6760844,E=6760847,C=1)
	CT(24633)=(S=6760848,E=6760851,C=4)
	CT(24634)=(S=6760852,E=6760859,C=1)
	CT(24635)=(S=6760860,E=6760863,C=0)
	CT(24636)=(S=6760864,E=6760871,C=1)
	CT(24637)=(S=6760872,E=6760872,C=4)
	CT(24638)=(S=6760873,E=6760873,C=74)
	CT(24639)=(S=6760874,E=6760874,C=4)
	CT(24640)=(S=6760875,E=6760875,C=74)
	CT(24641)=(S=6760876,E=6760879,C=8)
	CT(24642)=(S=6760880,E=6760883,C=0)
	CT(24643)=(S=6760884,E=6760891,C=8)
	CT(24644)=(S=6760892,E=6760899,C=74)
	CT(24645)=(S=6760900,E=6760903,C=4)
	CT(24646)=(S=6760904,E=6760915,C=74)
	CT(24647)=(S=6760916,E=6760917,C=84)
	CT(24648)=(S=6760918,E=6760919,C=4)
	CT(24649)=(S=6760920,E=6760923,C=0)
	CT(24650)=(S=6760924,E=6760939,C=1)
	CT(24651)=(S=6760940,E=6760943,C=4)
	CT(24652)=(S=6760944,E=6760947,C=2)
	CT(24653)=(S=6760948,E=6760949,C=84)
	CT(24654)=(S=6760950,E=6760951,C=74)
	CT(24655)=(S=6760952,E=6760955,C=8)
	CT(24656)=(S=6760956,E=6760959,C=2)
	CT(24657)=(S=6760960,E=6760963,C=89)
	CT(24658)=(S=6760964,E=6760967,C=2)
	CT(24659)=(S=6760968,E=6760971,C=1)
	CT(24660)=(S=6760972,E=6760975,C=8)
	CT(24661)=(S=6760976,E=6760978,C=4)
	CT(24662)=(S=6760979,E=6760983,C=0)
	CT(24663)=(S=6760984,E=6760995,C=1)
	CT(24664)=(S=6760996,E=6760999,C=8)
	CT(24665)=(S=6761000,E=6761003,C=74)
	CT(24666)=(S=6761004,E=6761007,C=2)
	CT(24667)=(S=6761008,E=6761010,C=0)
	CT(24668)=(S=6761011,E=6761011,C=4)
	CT(24669)=(S=6761012,E=6761015,C=2)
	CT(24670)=(S=6761016,E=6761019,C=11)
	CT(24671)=(S=6761020,E=6761023,C=8)
	CT(24672)=(S=6761024,E=6761031,C=1)
	CT(24673)=(S=6761032,E=6761035,C=4)
	CT(24674)=(S=6761036,E=6761039,C=1)
	CT(24675)=(S=6761040,E=6761043,C=2)
	CT(24676)=(S=6761044,E=6761051,C=4)
	CT(24677)=(S=6761052,E=6761059,C=10)
	CT(24678)=(S=6761060,E=6761063,C=8)
	CT(24679)=(S=6761064,E=6761067,C=1)
	CT(24680)=(S=6761068,E=6761071,C=0)
	CT(24681)=(S=6761072,E=6761073,C=9)
	CT(24682)=(S=6761074,E=6761075,C=4)
	CT(24683)=(S=6761076,E=6761083,C=74)
	CT(24684)=(S=6761084,E=6761087,C=4)
	CT(24685)=(S=6761088,E=6761091,C=3)
	CT(24686)=(S=6761092,E=6761095,C=0)
	CT(24687)=(S=6761096,E=6761099,C=7)
	CT(24688)=(S=6761100,E=6761103,C=0)
	CT(24689)=(S=6761104,E=6761111,C=9)
	CT(24690)=(S=6761112,E=6761115,C=84)
	CT(24691)=(S=6761116,E=6761127,C=4)
	CT(24692)=(S=6761128,E=6761131,C=2)
	CT(24693)=(S=6761132,E=6761135,C=4)
	CT(24694)=(S=6761136,E=6761139,C=8)
	CT(24695)=(S=6761140,E=6761143,C=1)
	CT(24696)=(S=6761144,E=6761147,C=6)
	CT(24697)=(S=6761148,E=6761148,C=4)
	CT(24698)=(S=6761149,E=6761149,C=0)
	CT(24699)=(S=6761150,E=6761150,C=73)
	CT(24700)=(S=6761151,E=6761151,C=8)
	CT(24701)=(S=6761152,E=6761159,C=4)
	CT(24702)=(S=6761160,E=6761163,C=2)
	CT(24703)=(S=6761164,E=6761164,C=0)
	CT(24704)=(S=6761165,E=6761165,C=74)
	CT(24705)=(S=6761166,E=6761167,C=4)
	CT(24706)=(S=6761168,E=6761171,C=157)
	CT(24707)=(S=6761172,E=6761175,C=5)
	CT(24708)=(S=6761176,E=6761179,C=155)
	CT(24709)=(S=6761180,E=6761183,C=2)
	CT(24710)=(S=6761184,E=6761187,C=0)
	CT(24711)=(S=6761188,E=6761191,C=9)
	CT(24712)=(S=6761192,E=6761195,C=1)
	CT(24713)=(S=6761196,E=6761199,C=0)
	CT(24714)=(S=6761200,E=6761203,C=2)
	CT(24715)=(S=6761204,E=6761207,C=6)
	CT(24716)=(S=6761208,E=6761211,C=4)
	CT(24717)=(S=6761212,E=6761217,C=74)
	CT(24718)=(S=6761218,E=6761223,C=4)
	CT(24719)=(S=6761224,E=6761231,C=8)
	CT(24720)=(S=6761232,E=6761235,C=1)
	CT(24721)=(S=6761236,E=6761239,C=0)
	CT(24722)=(S=6761240,E=6761243,C=1)
	CT(24723)=(S=6761244,E=6761247,C=8)
	CT(24724)=(S=6761248,E=6761253,C=4)
	CT(24725)=(S=6761254,E=6761255,C=0)
	CT(24726)=(S=6761256,E=6761259,C=4)
	CT(24727)=(S=6761260,E=6761263,C=74)
	CT(24728)=(S=6761264,E=6761267,C=9)
	CT(24729)=(S=6761268,E=6761271,C=84)
	CT(24730)=(S=6761272,E=6761275,C=0)
	CT(24731)=(S=6761276,E=6761279,C=8)
	CT(24732)=(S=6761280,E=6761280,C=6)
	CT(24733)=(S=6761281,E=6761281,C=4)
	CT(24734)=(S=6761282,E=6761283,C=74)
	CT(24735)=(S=6761284,E=6761287,C=8)
	CT(24736)=(S=6761288,E=6761291,C=2)
	CT(24737)=(S=6761292,E=6761295,C=154)
	CT(24738)=(S=6761296,E=6761299,C=4)
	CT(24739)=(S=6761300,E=6761303,C=1)
	CT(24740)=(S=6761304,E=6761308,C=0)
	CT(24741)=(S=6761309,E=6761309,C=84)
	CT(24742)=(S=6761310,E=6761311,C=4)
	CT(24743)=(S=6761312,E=6761323,C=1)
	CT(24744)=(S=6761324,E=6761327,C=5)
	CT(24745)=(S=6761328,E=6761331,C=4)
	CT(24746)=(S=6761332,E=6761335,C=0)
	CT(24747)=(S=6761336,E=6761339,C=6)
	CT(24748)=(S=6761340,E=6761343,C=1)
	CT(24749)=(S=6761344,E=6761347,C=74)
	CT(24750)=(S=6761348,E=6761351,C=1)
	CT(24751)=(S=6761352,E=6761352,C=3)
	CT(24752)=(S=6761353,E=6761355,C=0)
	CT(24753)=(S=6761356,E=6761359,C=2)
	CT(24754)=(S=6761360,E=6761363,C=76)
	CT(24755)=(S=6761364,E=6761367,C=84)
	CT(24756)=(S=6761368,E=6761371,C=4)
	CT(24757)=(S=6761372,E=6761375,C=90)
	CT(24758)=(S=6761376,E=6761379,C=4)
	CT(24759)=(S=6761380,E=6761383,C=1)
	CT(24760)=(S=6761384,E=6761387,C=75)
	CT(24761)=(S=6761388,E=6761391,C=8)
	CT(24762)=(S=6761392,E=6761399,C=0)
	CT(24763)=(S=6761400,E=6761403,C=1)
	CT(24764)=(S=6761404,E=6761407,C=7)
	CT(24765)=(S=6761408,E=6761415,C=1)
	CT(24766)=(S=6761416,E=6761419,C=4)
	CT(24767)=(S=6761420,E=6761421,C=195)
	CT(24768)=(S=6761422,E=6761423,C=73)
	CT(24769)=(S=6761424,E=6761427,C=1)
	CT(24770)=(S=6761428,E=6761431,C=73)
	CT(24771)=(S=6761432,E=6761435,C=8)
	CT(24772)=(S=6761436,E=6761443,C=1)
	CT(24773)=(S=6761444,E=6761444,C=4)
	CT(24774)=(S=6761445,E=6761445,C=3)
	CT(24775)=(S=6761446,E=6761446,C=84)
	CT(24776)=(S=6761447,E=6761447,C=4)
	CT(24777)=(S=6761448,E=6761451,C=1)
	CT(24778)=(S=6761452,E=6761455,C=8)
	CT(24779)=(S=6761456,E=6761459,C=1)
	CT(24780)=(S=6761460,E=6761463,C=8)
	CT(24781)=(S=6761464,E=6761468,C=4)
	CT(24782)=(S=6761469,E=6761471,C=0)
	CT(24783)=(S=6761472,E=6761475,C=4)
	CT(24784)=(S=6761476,E=6761479,C=7)
	CT(24785)=(S=6761480,E=6761483,C=2)
	CT(24786)=(S=6761484,E=6761487,C=4)
	CT(24787)=(S=6761488,E=6761489,C=2)
	CT(24788)=(S=6761490,E=6761491,C=4)
	CT(24789)=(S=6761492,E=6761495,C=8)
	CT(24790)=(S=6761496,E=6761497,C=0)
	CT(24791)=(S=6761498,E=6761499,C=74)
	CT(24792)=(S=6761500,E=6761503,C=8)
	CT(24793)=(S=6761504,E=6761507,C=0)
	CT(24794)=(S=6761508,E=6761511,C=74)
	CT(24795)=(S=6761512,E=6761515,C=0)
	CT(24796)=(S=6761516,E=6761516,C=3)
	CT(24797)=(S=6761517,E=6761517,C=73)
	CT(24798)=(S=6761518,E=6761519,C=0)
	CT(24799)=(S=6761520,E=6761527,C=4)
	CT(24800)=(S=6761528,E=6761531,C=1)
	CT(24801)=(S=6761532,E=6761539,C=8)
	CT(24802)=(S=6761540,E=6761543,C=7)
	CT(24803)=(S=6761544,E=6761551,C=84)
	CT(24804)=(S=6761552,E=6761555,C=1)
	CT(24805)=(S=6761556,E=6761559,C=2)
	CT(24806)=(S=6761560,E=6761563,C=1)
	CT(24807)=(S=6761564,E=6761567,C=0)
	CT(24808)=(S=6761568,E=6761571,C=4)
	CT(24809)=(S=6761572,E=6761572,C=74)
	CT(24810)=(S=6761573,E=6761573,C=0)
	CT(24811)=(S=6761574,E=6761575,C=2)
	CT(24812)=(S=6761576,E=6761579,C=4)
	CT(24813)=(S=6761580,E=6761580,C=5)
	CT(24814)=(S=6761581,E=6761581,C=8)
	CT(24815)=(S=6761582,E=6761583,C=73)
	CT(24816)=(S=6761584,E=6761587,C=7)
	CT(24817)=(S=6761588,E=6761591,C=4)
	CT(24818)=(S=6761592,E=6761599,C=1)
	CT(24819)=(S=6761600,E=6761603,C=2)
	CT(24820)=(S=6761604,E=6761607,C=1)
	CT(24821)=(S=6761608,E=6761615,C=4)
	CT(24822)=(S=6761616,E=6761619,C=1)
	CT(24823)=(S=6761620,E=6761620,C=0)
	CT(24824)=(S=6761621,E=6761621,C=74)
	CT(24825)=(S=6761622,E=6761623,C=84)
	CT(24826)=(S=6761624,E=6761627,C=1)
	CT(24827)=(S=6761628,E=6761631,C=4)
	CT(24828)=(S=6761632,E=6761635,C=8)
	CT(24829)=(S=6761636,E=6761639,C=73)
	CT(24830)=(S=6761640,E=6761643,C=1)
	CT(24831)=(S=6761644,E=6761647,C=4)
	CT(24832)=(S=6761648,E=6761679,C=1)
	CT(24833)=(S=6761680,E=6761687,C=2)
	CT(24834)=(S=6761688,E=6761691,C=3)
	CT(24835)=(S=6761692,E=6761692,C=7)
	CT(24836)=(S=6761693,E=6761693,C=4)
	CT(24837)=(S=6761694,E=6761694,C=73)
	CT(24838)=(S=6761695,E=6761695,C=77)
	CT(24839)=(S=6761696,E=6761699,C=1)
	CT(24840)=(S=6761700,E=6761703,C=0)
	CT(24841)=(S=6761704,E=6761707,C=8)
	CT(24842)=(S=6761708,E=6761791,C=1)
	CT(24843)=(S=6761792,E=6761799,C=8)
	CT(24844)=(S=6761800,E=6761955,C=1)
	CT(24845)=(S=6761956,E=6761967,C=11)
	CT(24846)=(S=6761968,E=6761971,C=73)
	CT(24847)=(S=6761972,E=6761975,C=2)
	CT(24848)=(S=6761976,E=6761979,C=1)
	CT(24849)=(S=6761980,E=6761981,C=10)
	CT(24850)=(S=6761982,E=6761983,C=0)
	CT(24851)=(S=6761984,E=6761987,C=1)
	CT(24852)=(S=6761988,E=6761995,C=2)
	CT(24853)=(S=6761996,E=6762123,C=1)
	CT(24854)=(S=6762124,E=6762127,C=8)
	CT(24855)=(S=6762128,E=6762131,C=9)
	CT(24856)=(S=6762132,E=6762135,C=84)
	CT(24857)=(S=6762136,E=6762167,C=1)
	CT(24858)=(S=6762168,E=6762171,C=8)
	CT(24859)=(S=6762172,E=6762175,C=9)
	CT(24860)=(S=6762176,E=6762182,C=4)
	CT(24861)=(S=6762183,E=6762183,C=0)
	CT(24862)=(S=6762184,E=6762187,C=4)
	CT(24863)=(S=6762188,E=6762191,C=2)
	CT(24864)=(S=6762192,E=6762195,C=89)
	CT(24865)=(S=6762196,E=6762199,C=0)
	CT(24866)=(S=6762200,E=6762203,C=164)
	CT(24867)=(S=6762204,E=6762207,C=0)
	CT(24868)=(S=6762208,E=6762211,C=8)
	CT(24869)=(S=6762212,E=6762215,C=0)
	CT(24870)=(S=6762216,E=6762227,C=4)
	CT(24871)=(S=6762228,E=6762235,C=1)
	CT(24872)=(S=6762236,E=6762239,C=7)
	CT(24873)=(S=6762240,E=6762241,C=0)
	CT(24874)=(S=6762242,E=6762242,C=2)
	CT(24875)=(S=6762243,E=6762243,C=73)
	CT(24876)=(S=6762244,E=6762247,C=1)
	CT(24877)=(S=6762248,E=6762251,C=2)
	CT(24878)=(S=6762252,E=6762259,C=4)
	CT(24879)=(S=6762260,E=6762263,C=1)
	CT(24880)=(S=6762264,E=6762267,C=8)
	CT(24881)=(S=6762268,E=6762271,C=7)
	CT(24882)=(S=6762272,E=6762275,C=4)
	CT(24883)=(S=6762276,E=6762283,C=1)
	CT(24884)=(S=6762284,E=6762287,C=4)
	CT(24885)=(S=6762288,E=6762291,C=1)
	CT(24886)=(S=6762292,E=6762299,C=4)
	CT(24887)=(S=6762300,E=6762300,C=74)
	CT(24888)=(S=6762301,E=6762301,C=8)
	CT(24889)=(S=6762302,E=6762303,C=0)
	CT(24890)=(S=6762304,E=6762315,C=4)
	CT(24891)=(S=6762316,E=6762317,C=7)
	CT(24892)=(S=6762318,E=6762319,C=4)
	CT(24893)=(S=6762320,E=6762323,C=1)
	CT(24894)=(S=6762324,E=6762327,C=3)
	CT(24895)=(S=6762328,E=6762329,C=8)
	CT(24896)=(S=6762330,E=6762335,C=4)
	CT(24897)=(S=6762336,E=6762339,C=1)
	CT(24898)=(S=6762340,E=6762343,C=2)
	CT(24899)=(S=6762344,E=6762347,C=4)
	CT(24900)=(S=6762348,E=6762351,C=1)
	CT(24901)=(S=6762352,E=6762355,C=8)
	CT(24902)=(S=6762356,E=6762363,C=1)
	CT(24903)=(S=6762364,E=6762371,C=4)
	CT(24904)=(S=6762372,E=6762375,C=74)
	CT(24905)=(S=6762376,E=6762383,C=1)
	CT(24906)=(S=6762384,E=6762387,C=77)
	CT(24907)=(S=6762388,E=6762393,C=4)
	CT(24908)=(S=6762394,E=6762395,C=5)
	CT(24909)=(S=6762396,E=6762415,C=4)
	CT(24910)=(S=6762416,E=6762419,C=8)
	CT(24911)=(S=6762420,E=6762423,C=2)
	CT(24912)=(S=6762424,E=6762425,C=155)
	CT(24913)=(S=6762426,E=6762426,C=4)
	CT(24914)=(S=6762427,E=6762427,C=0)
	CT(24915)=(S=6762428,E=6762431,C=86)
	CT(24916)=(S=6762432,E=6762435,C=11)
	CT(24917)=(S=6762436,E=6762439,C=4)
	CT(24918)=(S=6762440,E=6762443,C=1)
	CT(24919)=(S=6762444,E=6762447,C=73)
	CT(24920)=(S=6762448,E=6762451,C=7)
	CT(24921)=(S=6762452,E=6762455,C=1)
	CT(24922)=(S=6762456,E=6762459,C=4)
	CT(24923)=(S=6762460,E=6762463,C=1)
	CT(24924)=(S=6762464,E=6762467,C=7)
	CT(24925)=(S=6762468,E=6762471,C=74)
	CT(24926)=(S=6762472,E=6762472,C=7)
	CT(24927)=(S=6762473,E=6762475,C=0)
	CT(24928)=(S=6762476,E=6762479,C=4)
	CT(24929)=(S=6762480,E=6762484,C=7)
	CT(24930)=(S=6762485,E=6762487,C=0)
	CT(24931)=(S=6762488,E=6762491,C=1)
	CT(24932)=(S=6762492,E=6762499,C=5)
	CT(24933)=(S=6762500,E=6762503,C=8)
	CT(24934)=(S=6762504,E=6762507,C=73)
	CT(24935)=(S=6762508,E=6762508,C=3)
	CT(24936)=(S=6762509,E=6762509,C=74)
	CT(24937)=(S=6762510,E=6762511,C=0)
	CT(24938)=(S=6762512,E=6762515,C=84)
	CT(24939)=(S=6762516,E=6762519,C=1)
	CT(24940)=(S=6762520,E=6762521,C=77)
	CT(24941)=(S=6762522,E=6762522,C=8)
	CT(24942)=(S=6762523,E=6762523,C=74)
	CT(24943)=(S=6762524,E=6762525,C=8)
	CT(24944)=(S=6762526,E=6762526,C=10)
	CT(24945)=(S=6762527,E=6762527,C=2)
	CT(24946)=(S=6762528,E=6762531,C=4)
	CT(24947)=(S=6762532,E=6762539,C=2)
	CT(24948)=(S=6762540,E=6762543,C=4)
	CT(24949)=(S=6762544,E=6762545,C=8)
	CT(24950)=(S=6762546,E=6762547,C=4)
	CT(24951)=(S=6762548,E=6762551,C=1)
	CT(24952)=(S=6762552,E=6762571,C=4)
	CT(24953)=(S=6762572,E=6762583,C=11)
	CT(24954)=(S=6762584,E=6762584,C=75)
	CT(24955)=(S=6762585,E=6762585,C=0)
	CT(24956)=(S=6762586,E=6762587,C=4)
	CT(24957)=(S=6762588,E=6762591,C=1)
	CT(24958)=(S=6762592,E=6762611,C=4)
	CT(24959)=(S=6762612,E=6762612,C=92)
	CT(24960)=(S=6762613,E=6762613,C=0)
	CT(24961)=(S=6762614,E=6762614,C=73)
	CT(24962)=(S=6762615,E=6762615,C=84)
	CT(24963)=(S=6762616,E=6762619,C=2)
	CT(24964)=(S=6762620,E=6762623,C=4)
	CT(24965)=(S=6762624,E=6762627,C=74)
	CT(24966)=(S=6762628,E=6762631,C=8)
	CT(24967)=(S=6762632,E=6762635,C=2)
	CT(24968)=(S=6762636,E=6762639,C=8)
	CT(24969)=(S=6762640,E=6762655,C=1)
	CT(24970)=(S=6762656,E=6762663,C=2)
	CT(24971)=(S=6762664,E=6762667,C=8)
	CT(24972)=(S=6762668,E=6762671,C=7)
	CT(24973)=(S=6762672,E=6762683,C=4)
	CT(24974)=(S=6762684,E=6762691,C=11)
	CT(24975)=(S=6762692,E=6762695,C=4)
	CT(24976)=(S=6762696,E=6762697,C=0)
	CT(24977)=(S=6762698,E=6762699,C=1)
	CT(24978)=(S=6762700,E=6762703,C=3)
	CT(24979)=(S=6762704,E=6762707,C=0)
	CT(24980)=(S=6762708,E=6762709,C=4)
	CT(24981)=(S=6762710,E=6762711,C=0)
	CT(24982)=(S=6762712,E=6762743,C=1)
	CT(24983)=(S=6762744,E=6762747,C=2)
	CT(24984)=(S=6762748,E=6762751,C=4)
	CT(24985)=(S=6762752,E=6762755,C=2)
	CT(24986)=(S=6762756,E=6762759,C=0)
	CT(24987)=(S=6762760,E=6762763,C=8)
	CT(24988)=(S=6762764,E=6762767,C=1)
	CT(24989)=(S=6762768,E=6762771,C=0)
	CT(24990)=(S=6762772,E=6762775,C=1)
	CT(24991)=(S=6762776,E=6762779,C=5)
	CT(24992)=(S=6762780,E=6762783,C=74)
	CT(24993)=(S=6762784,E=6762787,C=7)
	CT(24994)=(S=6762788,E=6762788,C=10)
	CT(24995)=(S=6762789,E=6762789,C=74)
	CT(24996)=(S=6762790,E=6762791,C=0)
	CT(24997)=(S=6762792,E=6762795,C=86)
	CT(24998)=(S=6762796,E=6762799,C=6)
	CT(24999)=(S=6762800,E=6762811,C=4)
	CT(25000)=(S=6762812,E=6762815,C=8)
	CT(25001)=(S=6762816,E=6762819,C=2)
	CT(25002)=(S=6762820,E=6762820,C=4)
	CT(25003)=(S=6762821,E=6762821,C=77)
	CT(25004)=(S=6762822,E=6762823,C=10)
	CT(25005)=(S=6762824,E=6762831,C=1)
	CT(25006)=(S=6762832,E=6762835,C=0)
	CT(25007)=(S=6762836,E=6762839,C=7)
	CT(25008)=(S=6762840,E=6762843,C=2)
	CT(25009)=(S=6762844,E=6762851,C=1)
	CT(25010)=(S=6762852,E=6762855,C=10)
	CT(25011)=(S=6762856,E=6762857,C=74)
	CT(25012)=(S=6762858,E=6762859,C=0)
	CT(25013)=(S=6762860,E=6762863,C=1)
	CT(25014)=(S=6762864,E=6762867,C=7)
	CT(25015)=(S=6762868,E=6762877,C=4)
	CT(25016)=(S=6762878,E=6762878,C=8)
	CT(25017)=(S=6762879,E=6762879,C=0)
	CT(25018)=(S=6762880,E=6762883,C=1)
	CT(25019)=(S=6762884,E=6762887,C=8)
	CT(25020)=(S=6762888,E=6762891,C=77)
	CT(25021)=(S=6762892,E=6762895,C=2)
	CT(25022)=(S=6762896,E=6762898,C=0)
	CT(25023)=(S=6762899,E=6762899,C=4)
	CT(25024)=(S=6762900,E=6762901,C=3)
	CT(25025)=(S=6762902,E=6762902,C=73)
	CT(25026)=(S=6762903,E=6762903,C=0)
	CT(25027)=(S=6762904,E=6762907,C=4)
	CT(25028)=(S=6762908,E=6762915,C=8)
	CT(25029)=(S=6762916,E=6762919,C=7)
	CT(25030)=(S=6762920,E=6762923,C=84)
	CT(25031)=(S=6762924,E=6762924,C=73)
	CT(25032)=(S=6762925,E=6762927,C=158)
	CT(25033)=(S=6762928,E=6762935,C=1)
	CT(25034)=(S=6762936,E=6762947,C=74)
	CT(25035)=(S=6762948,E=6762951,C=1)
	CT(25036)=(S=6762952,E=6762955,C=84)
	CT(25037)=(S=6762956,E=6762956,C=4)
	CT(25038)=(S=6762957,E=6762957,C=73)
	CT(25039)=(S=6762958,E=6762958,C=4)
	CT(25040)=(S=6762959,E=6762959,C=76)
	CT(25041)=(S=6762960,E=6762967,C=8)
	CT(25042)=(S=6762968,E=6762971,C=2)
	CT(25043)=(S=6762972,E=6762975,C=74)
	CT(25044)=(S=6762976,E=6762979,C=4)
	CT(25045)=(S=6762980,E=6762983,C=74)
	CT(25046)=(S=6762984,E=6762997,C=4)
	CT(25047)=(S=6762998,E=6762999,C=8)
	CT(25048)=(S=6763000,E=6763003,C=1)
	CT(25049)=(S=6763004,E=6763007,C=4)
	CT(25050)=(S=6763008,E=6763011,C=7)
	CT(25051)=(S=6763012,E=6763015,C=4)
	CT(25052)=(S=6763016,E=6763019,C=8)
	CT(25053)=(S=6763020,E=6763021,C=0)
	CT(25054)=(S=6763022,E=6763023,C=7)
	CT(25055)=(S=6763024,E=6763027,C=8)
	CT(25056)=(S=6763028,E=6763031,C=2)
	CT(25057)=(S=6763032,E=6763035,C=74)
	CT(25058)=(S=6763036,E=6763039,C=2)
	CT(25059)=(S=6763040,E=6763041,C=4)
	CT(25060)=(S=6763042,E=6763043,C=0)
	CT(25061)=(S=6763044,E=6763047,C=1)
	CT(25062)=(S=6763048,E=6763051,C=6)
	CT(25063)=(S=6763052,E=6763083,C=1)
	CT(25064)=(S=6763084,E=6763091,C=4)
	CT(25065)=(S=6763092,E=6763095,C=2)
	CT(25066)=(S=6763096,E=6763099,C=8)
	CT(25067)=(S=6763100,E=6763103,C=7)
	CT(25068)=(S=6763104,E=6763107,C=8)
	CT(25069)=(S=6763108,E=6763111,C=0)
	CT(25070)=(S=6763112,E=6763115,C=88)
	CT(25071)=(S=6763116,E=6763135,C=1)
	CT(25072)=(S=6763136,E=6763139,C=74)
	CT(25073)=(S=6763140,E=6763151,C=1)
	CT(25074)=(S=6763152,E=6763163,C=4)
	CT(25075)=(S=6763164,E=6763167,C=77)
	CT(25076)=(S=6763168,E=6763171,C=4)
	CT(25077)=(S=6763172,E=6763175,C=5)
	CT(25078)=(S=6763176,E=6763179,C=84)
	CT(25079)=(S=6763180,E=6763211,C=1)
	CT(25080)=(S=6763212,E=6763215,C=92)
	CT(25081)=(S=6763216,E=6763223,C=4)
	CT(25082)=(S=6763224,E=6763226,C=74)
	CT(25083)=(S=6763227,E=6763227,C=4)
	CT(25084)=(S=6763228,E=6763259,C=1)
	CT(25085)=(S=6763260,E=6763263,C=8)
	CT(25086)=(S=6763264,E=6763265,C=4)
	CT(25087)=(S=6763266,E=6763267,C=84)
	CT(25088)=(S=6763268,E=6763271,C=2)
	CT(25089)=(S=6763272,E=6763273,C=4)
	CT(25090)=(S=6763274,E=6763275,C=3)
	CT(25091)=(S=6763276,E=6763279,C=8)
	CT(25092)=(S=6763280,E=6763283,C=75)
	CT(25093)=(S=6763284,E=6763299,C=4)
	CT(25094)=(S=6763300,E=6763303,C=2)
	CT(25095)=(S=6763304,E=6763304,C=0)
	CT(25096)=(S=6763305,E=6763305,C=5)
	CT(25097)=(S=6763306,E=6763306,C=0)
	CT(25098)=(S=6763307,E=6763307,C=5)
	CT(25099)=(S=6763308,E=6763311,C=74)
	CT(25100)=(S=6763312,E=6763315,C=5)
	CT(25101)=(S=6763316,E=6763323,C=4)
	CT(25102)=(S=6763324,E=6763324,C=92)
	CT(25103)=(S=6763325,E=6763325,C=0)
	CT(25104)=(S=6763326,E=6763326,C=7)
	CT(25105)=(S=6763327,E=6763327,C=4)
	CT(25106)=(S=6763328,E=6763331,C=3)
	CT(25107)=(S=6763332,E=6763335,C=0)
	CT(25108)=(S=6763336,E=6763343,C=4)
	CT(25109)=(S=6763344,E=6763351,C=2)
	CT(25110)=(S=6763352,E=6763363,C=4)
	CT(25111)=(S=6763364,E=6763365,C=73)
	CT(25112)=(S=6763366,E=6763366,C=4)
	CT(25113)=(S=6763367,E=6763367,C=74)
	CT(25114)=(S=6763368,E=6763371,C=84)
	CT(25115)=(S=6763372,E=6763375,C=8)
	CT(25116)=(S=6763376,E=6763379,C=0)
	CT(25117)=(S=6763380,E=6763383,C=4)
	CT(25118)=(S=6763384,E=6763387,C=84)
	CT(25119)=(S=6763388,E=6763391,C=86)
	CT(25120)=(S=6763392,E=6763392,C=4)
	CT(25121)=(S=6763393,E=6763393,C=73)
	CT(25122)=(S=6763394,E=6763394,C=0)
	CT(25123)=(S=6763395,E=6763395,C=74)
	CT(25124)=(S=6763396,E=6763403,C=4)
	CT(25125)=(S=6763404,E=6763407,C=9)
	CT(25126)=(S=6763408,E=6763411,C=8)
	CT(25127)=(S=6763412,E=6763419,C=4)
	CT(25128)=(S=6763420,E=6763421,C=73)
	CT(25129)=(S=6763422,E=6763422,C=74)
	CT(25130)=(S=6763423,E=6763423,C=0)
	CT(25131)=(S=6763424,E=6763427,C=7)
	CT(25132)=(S=6763428,E=6763431,C=74)
	CT(25133)=(S=6763432,E=6763443,C=6)
	CT(25134)=(S=6763444,E=6763447,C=2)
	CT(25135)=(S=6763448,E=6763467,C=6)
	CT(25136)=(S=6763468,E=6763471,C=2)
	CT(25137)=(S=6763472,E=6763475,C=4)
	CT(25138)=(S=6763476,E=6763479,C=2)
	CT(25139)=(S=6763480,E=6763483,C=4)
	CT(25140)=(S=6763484,E=6763484,C=77)
	CT(25141)=(S=6763485,E=6763485,C=0)
	CT(25142)=(S=6763486,E=6763487,C=4)
	CT(25143)=(S=6763488,E=6763491,C=3)
	CT(25144)=(S=6763492,E=6763495,C=84)
	CT(25145)=(S=6763496,E=6763503,C=4)
	CT(25146)=(S=6763504,E=6763519,C=6)
	CT(25147)=(S=6763520,E=6763520,C=4)
	CT(25148)=(S=6763521,E=6763521,C=0)
	CT(25149)=(S=6763522,E=6763523,C=74)
	CT(25150)=(S=6763524,E=6763525,C=0)
	CT(25151)=(S=6763526,E=6763526,C=4)
	CT(25152)=(S=6763527,E=6763527,C=74)
	CT(25153)=(S=6763528,E=6763531,C=4)
	CT(25154)=(S=6763532,E=6763535,C=155)
	CT(25155)=(S=6763536,E=6763539,C=74)
	CT(25156)=(S=6763540,E=6763547,C=2)
	CT(25157)=(S=6763548,E=6763551,C=75)
	CT(25158)=(S=6763552,E=6763555,C=7)
	CT(25159)=(S=6763556,E=6763559,C=4)
	CT(25160)=(S=6763560,E=6763563,C=1)
	CT(25161)=(S=6763564,E=6763567,C=74)
	CT(25162)=(S=6763568,E=6763575,C=4)
	CT(25163)=(S=6763576,E=6763579,C=5)
	CT(25164)=(S=6763580,E=6763581,C=74)
	CT(25165)=(S=6763582,E=6763582,C=10)
	CT(25166)=(S=6763583,E=6763583,C=0)
	CT(25167)=(S=6763584,E=6763591,C=74)
	CT(25168)=(S=6763592,E=6763607,C=1)
	CT(25169)=(S=6763608,E=6763611,C=91)
	CT(25170)=(S=6763612,E=6763615,C=11)
	CT(25171)=(S=6763616,E=6763627,C=1)
	CT(25172)=(S=6763628,E=6763631,C=2)
	CT(25173)=(S=6763632,E=6763635,C=8)
	CT(25174)=(S=6763636,E=6763639,C=0)
	CT(25175)=(S=6763640,E=6763643,C=74)
	CT(25176)=(S=6763644,E=6763647,C=4)
	CT(25177)=(S=6763648,E=6763652,C=2)
	CT(25178)=(S=6763653,E=6763653,C=7)
	CT(25179)=(S=6763654,E=6763655,C=84)
	CT(25180)=(S=6763656,E=6763659,C=4)
	CT(25181)=(S=6763660,E=6763663,C=84)
	CT(25182)=(S=6763664,E=6763667,C=74)
	CT(25183)=(S=6763668,E=6763671,C=2)
	CT(25184)=(S=6763672,E=6763675,C=8)
	CT(25185)=(S=6763676,E=6763679,C=73)
	CT(25186)=(S=6763680,E=6763687,C=1)
	CT(25187)=(S=6763688,E=6763689,C=0)
	CT(25188)=(S=6763690,E=6763691,C=74)
	CT(25189)=(S=6763692,E=6763699,C=1)
	CT(25190)=(S=6763700,E=6763703,C=4)
	CT(25191)=(S=6763704,E=6763707,C=1)
	CT(25192)=(S=6763708,E=6763711,C=8)
	CT(25193)=(S=6763712,E=6763712,C=5)
	CT(25194)=(S=6763713,E=6763713,C=0)
	CT(25195)=(S=6763714,E=6763715,C=4)
	CT(25196)=(S=6763716,E=6763719,C=1)
	CT(25197)=(S=6763720,E=6763723,C=6)
	CT(25198)=(S=6763724,E=6763727,C=73)
	CT(25199)=(S=6763728,E=6763731,C=4)
	CT(25200)=(S=6763732,E=6763732,C=0)
	CT(25201)=(S=6763733,E=6763733,C=74)
	CT(25202)=(S=6763734,E=6763739,C=8)
	CT(25203)=(S=6763740,E=6763747,C=4)
	CT(25204)=(S=6763748,E=6763751,C=155)
	CT(25205)=(S=6763752,E=6763763,C=2)
	CT(25206)=(S=6763764,E=6763775,C=4)
	CT(25207)=(S=6763776,E=6763779,C=74)
	CT(25208)=(S=6763780,E=6763783,C=1)
	CT(25209)=(S=6763784,E=6763787,C=8)
	CT(25210)=(S=6763788,E=6763791,C=2)
	CT(25211)=(S=6763792,E=6763795,C=89)
	CT(25212)=(S=6763796,E=6763796,C=74)
	CT(25213)=(S=6763797,E=6763797,C=4)
	CT(25214)=(S=6763798,E=6763799,C=73)
	CT(25215)=(S=6763800,E=6763803,C=89)
	CT(25216)=(S=6763804,E=6763807,C=2)
	CT(25217)=(S=6763808,E=6763811,C=5)
	CT(25218)=(S=6763812,E=6763815,C=84)
	CT(25219)=(S=6763816,E=6763819,C=4)
	CT(25220)=(S=6763820,E=6763823,C=77)
	CT(25221)=(S=6763824,E=6763827,C=8)
	CT(25222)=(S=6763828,E=6763831,C=4)
	CT(25223)=(S=6763832,E=6763835,C=10)
	CT(25224)=(S=6763836,E=6763839,C=4)
	CT(25225)=(S=6763840,E=6763847,C=1)
	CT(25226)=(S=6763848,E=6763851,C=4)
	CT(25227)=(S=6763852,E=6763855,C=74)
	CT(25228)=(S=6763856,E=6763859,C=4)
	CT(25229)=(S=6763860,E=6763861,C=84)
	CT(25230)=(S=6763862,E=6763863,C=4)
	CT(25231)=(S=6763864,E=6763867,C=11)
	CT(25232)=(S=6763868,E=6763871,C=1)
	CT(25233)=(S=6763872,E=6763875,C=2)
	CT(25234)=(S=6763876,E=6763879,C=1)
	CT(25235)=(S=6763880,E=6763889,C=4)
	CT(25236)=(S=6763890,E=6763890,C=6)
	CT(25237)=(S=6763891,E=6763895,C=0)
	CT(25238)=(S=6763896,E=6763899,C=2)
	CT(25239)=(S=6763900,E=6763923,C=1)
	CT(25240)=(S=6763924,E=6763927,C=73)
	CT(25241)=(S=6763928,E=6763929,C=8)
	CT(25242)=(S=6763930,E=6763931,C=10)
	CT(25243)=(S=6763932,E=6763935,C=4)
	CT(25244)=(S=6763936,E=6763939,C=1)
	CT(25245)=(S=6763940,E=6763943,C=4)
	CT(25246)=(S=6763944,E=6763947,C=11)
	CT(25247)=(S=6763948,E=6763951,C=7)
	CT(25248)=(S=6763952,E=6763955,C=76)
	CT(25249)=(S=6763956,E=6763959,C=1)
	CT(25250)=(S=6763960,E=6763961,C=74)
	CT(25251)=(S=6763962,E=6763962,C=5)
	CT(25252)=(S=6763963,E=6763963,C=0)
	CT(25253)=(S=6763964,E=6763971,C=74)
	CT(25254)=(S=6763972,E=6763979,C=0)
	CT(25255)=(S=6763980,E=6763995,C=1)
	CT(25256)=(S=6763996,E=6763999,C=10)
	CT(25257)=(S=6764000,E=6764003,C=8)
	CT(25258)=(S=6764004,E=6764007,C=11)
	CT(25259)=(S=6764008,E=6764011,C=4)
	CT(25260)=(S=6764012,E=6764015,C=1)
	CT(25261)=(S=6764016,E=6764023,C=4)
	CT(25262)=(S=6764024,E=6764027,C=1)
	CT(25263)=(S=6764028,E=6764031,C=11)
	CT(25264)=(S=6764032,E=6764035,C=74)
	CT(25265)=(S=6764036,E=6764039,C=8)
	CT(25266)=(S=6764040,E=6764043,C=1)
	CT(25267)=(S=6764044,E=6764067,C=4)
	CT(25268)=(S=6764068,E=6764075,C=84)
	CT(25269)=(S=6764076,E=6764079,C=8)
	CT(25270)=(S=6764080,E=6764083,C=1)
	CT(25271)=(S=6764084,E=6764087,C=5)
	CT(25272)=(S=6764088,E=6764091,C=2)
	CT(25273)=(S=6764092,E=6764095,C=1)
	CT(25274)=(S=6764096,E=6764099,C=4)
	CT(25275)=(S=6764100,E=6764103,C=0)
	CT(25276)=(S=6764104,E=6764107,C=6)
	CT(25277)=(S=6764108,E=6764119,C=77)
	CT(25278)=(S=6764120,E=6764123,C=2)
	CT(25279)=(S=6764124,E=6764127,C=74)
	CT(25280)=(S=6764128,E=6764141,C=4)
	CT(25281)=(S=6764142,E=6764142,C=5)
	CT(25282)=(S=6764143,E=6764143,C=0)
	CT(25283)=(S=6764144,E=6764147,C=86)
	CT(25284)=(S=6764148,E=6764163,C=8)
	CT(25285)=(S=6764164,E=6764165,C=0)
	CT(25286)=(S=6764166,E=6764166,C=74)
	CT(25287)=(S=6764167,E=6764173,C=4)
	CT(25288)=(S=6764174,E=6764174,C=10)
	CT(25289)=(S=6764175,E=6764179,C=4)
	CT(25290)=(S=6764180,E=6764183,C=84)
	CT(25291)=(S=6764184,E=6764187,C=7)
	CT(25292)=(S=6764188,E=6764191,C=2)
	CT(25293)=(S=6764192,E=6764199,C=1)
	CT(25294)=(S=6764200,E=6764203,C=4)
	CT(25295)=(S=6764204,E=6764204,C=74)
	CT(25296)=(S=6764205,E=6764207,C=4)
	CT(25297)=(S=6764208,E=6764211,C=8)
	CT(25298)=(S=6764212,E=6764215,C=4)
	CT(25299)=(S=6764216,E=6764219,C=2)
	CT(25300)=(S=6764220,E=6764223,C=4)
	CT(25301)=(S=6764224,E=6764227,C=8)
	CT(25302)=(S=6764228,E=6764235,C=4)
	CT(25303)=(S=6764236,E=6764243,C=2)
	CT(25304)=(S=6764244,E=6764247,C=1)
	CT(25305)=(S=6764248,E=6764251,C=74)
	CT(25306)=(S=6764252,E=6764255,C=4)
	CT(25307)=(S=6764256,E=6764256,C=73)
	CT(25308)=(S=6764257,E=6764259,C=74)
	CT(25309)=(S=6764260,E=6764263,C=1)
	CT(25310)=(S=6764264,E=6764267,C=0)
	CT(25311)=(S=6764268,E=6764269,C=4)
	CT(25312)=(S=6764270,E=6764271,C=0)
	CT(25313)=(S=6764272,E=6764275,C=1)
	CT(25314)=(S=6764276,E=6764279,C=4)
	CT(25315)=(S=6764280,E=6764287,C=11)
	CT(25316)=(S=6764288,E=6764291,C=3)
	CT(25317)=(S=6764292,E=6764295,C=2)
	CT(25318)=(S=6764296,E=6764299,C=8)
	CT(25319)=(S=6764300,E=6764303,C=2)
	CT(25320)=(S=6764304,E=6764307,C=0)
	CT(25321)=(S=6764308,E=6764311,C=8)
	CT(25322)=(S=6764312,E=6764315,C=1)
	CT(25323)=(S=6764316,E=6764319,C=8)
	CT(25324)=(S=6764320,E=6764320,C=0)
	CT(25325)=(S=6764321,E=6764321,C=74)
	CT(25326)=(S=6764322,E=6764322,C=0)
	CT(25327)=(S=6764323,E=6764323,C=6)
	CT(25328)=(S=6764324,E=6764327,C=74)
	CT(25329)=(S=6764328,E=6764331,C=4)
	CT(25330)=(S=6764332,E=6764335,C=0)
	CT(25331)=(S=6764336,E=6764339,C=165)
	CT(25332)=(S=6764340,E=6764341,C=0)
	CT(25333)=(S=6764342,E=6764343,C=3)
	CT(25334)=(S=6764344,E=6764345,C=74)
	CT(25335)=(S=6764346,E=6764346,C=0)
	CT(25336)=(S=6764347,E=6764347,C=3)
	CT(25337)=(S=6764348,E=6764355,C=4)
	CT(25338)=(S=6764356,E=6764359,C=77)
	CT(25339)=(S=6764360,E=6764363,C=4)
	CT(25340)=(S=6764364,E=6764367,C=0)
	CT(25341)=(S=6764368,E=6764371,C=1)
	CT(25342)=(S=6764372,E=6764379,C=4)
	CT(25343)=(S=6764380,E=6764383,C=0)
	CT(25344)=(S=6764384,E=6764399,C=4)
	CT(25345)=(S=6764400,E=6764401,C=8)
	CT(25346)=(S=6764402,E=6764407,C=0)
	CT(25347)=(S=6764408,E=6764411,C=1)
	CT(25348)=(S=6764412,E=6764415,C=7)
	CT(25349)=(S=6764416,E=6764419,C=8)
	CT(25350)=(S=6764420,E=6764420,C=0)
	CT(25351)=(S=6764421,E=6764421,C=77)
	CT(25352)=(S=6764422,E=6764423,C=0)
	CT(25353)=(S=6764424,E=6764425,C=77)
	CT(25354)=(S=6764426,E=6764426,C=7)
	CT(25355)=(S=6764427,E=6764427,C=74)
	CT(25356)=(S=6764428,E=6764431,C=3)
	CT(25357)=(S=6764432,E=6764435,C=84)
	CT(25358)=(S=6764436,E=6764439,C=0)
	CT(25359)=(S=6764440,E=6764443,C=1)
	CT(25360)=(S=6764444,E=6764447,C=2)
	CT(25361)=(S=6764448,E=6764448,C=74)
	CT(25362)=(S=6764449,E=6764449,C=10)
	CT(25363)=(S=6764450,E=6764451,C=0)
	CT(25364)=(S=6764452,E=6764459,C=4)
	CT(25365)=(S=6764460,E=6764463,C=1)
	CT(25366)=(S=6764464,E=6764465,C=0)
	CT(25367)=(S=6764466,E=6764466,C=89)
	CT(25368)=(S=6764467,E=6764467,C=73)
	CT(25369)=(S=6764468,E=6764475,C=4)
	CT(25370)=(S=6764476,E=6764479,C=6)
	CT(25371)=(S=6764480,E=6764487,C=2)
	CT(25372)=(S=6764488,E=6764491,C=8)
	CT(25373)=(S=6764492,E=6764499,C=1)
	CT(25374)=(S=6764500,E=6764500,C=0)
	CT(25375)=(S=6764501,E=6764503,C=4)
	CT(25376)=(S=6764504,E=6764504,C=74)
	CT(25377)=(S=6764505,E=6764505,C=0)
	CT(25378)=(S=6764506,E=6764506,C=8)
	CT(25379)=(S=6764507,E=6764507,C=0)
	CT(25380)=(S=6764508,E=6764511,C=74)
	CT(25381)=(S=6764512,E=6764515,C=0)
	CT(25382)=(S=6764516,E=6764519,C=1)
	CT(25383)=(S=6764520,E=6764523,C=0)
	CT(25384)=(S=6764524,E=6764531,C=1)
	CT(25385)=(S=6764532,E=6764543,C=4)
	CT(25386)=(S=6764544,E=6764547,C=0)
	CT(25387)=(S=6764548,E=6764551,C=84)
	CT(25388)=(S=6764552,E=6764555,C=1)
	CT(25389)=(S=6764556,E=6764559,C=0)
	CT(25390)=(S=6764560,E=6764567,C=1)
	CT(25391)=(S=6764568,E=6764571,C=8)
	CT(25392)=(S=6764572,E=6764574,C=4)
	CT(25393)=(S=6764575,E=6764575,C=0)
	CT(25394)=(S=6764576,E=6764579,C=1)
	CT(25395)=(S=6764580,E=6764587,C=4)
	CT(25396)=(S=6764588,E=6764591,C=8)
	CT(25397)=(S=6764592,E=6764595,C=2)
	CT(25398)=(S=6764596,E=6764607,C=1)
	CT(25399)=(S=6764608,E=6764613,C=4)
	CT(25400)=(S=6764614,E=6764614,C=74)
	CT(25401)=(S=6764615,E=6764615,C=0)
	CT(25402)=(S=6764616,E=6764623,C=1)
	CT(25403)=(S=6764624,E=6764627,C=2)
	CT(25404)=(S=6764628,E=6764639,C=4)
	CT(25405)=(S=6764640,E=6764643,C=2)
	CT(25406)=(S=6764644,E=6764651,C=1)
	CT(25407)=(S=6764652,E=6764655,C=2)
	CT(25408)=(S=6764656,E=6764663,C=8)
	CT(25409)=(S=6764664,E=6764667,C=4)
	CT(25410)=(S=6764668,E=6764671,C=7)
	CT(25411)=(S=6764672,E=6764675,C=10)
	CT(25412)=(S=6764676,E=6764679,C=2)
	CT(25413)=(S=6764680,E=6764683,C=8)
	CT(25414)=(S=6764684,E=6764687,C=1)
	CT(25415)=(S=6764688,E=6764693,C=74)
	CT(25416)=(S=6764694,E=6764695,C=0)
	CT(25417)=(S=6764696,E=6764699,C=1)
	CT(25418)=(S=6764700,E=6764715,C=11)
	CT(25419)=(S=6764716,E=6764717,C=4)
	CT(25420)=(S=6764718,E=6764718,C=86)
	CT(25421)=(S=6764719,E=6764727,C=4)
	CT(25422)=(S=6764728,E=6764731,C=1)
	CT(25423)=(S=6764732,E=6764735,C=74)
	CT(25424)=(S=6764736,E=6764739,C=8)
	CT(25425)=(S=6764740,E=6764743,C=4)
	CT(25426)=(S=6764744,E=6764747,C=1)
	CT(25427)=(S=6764748,E=6764751,C=74)
	CT(25428)=(S=6764752,E=6764753,C=84)
	CT(25429)=(S=6764754,E=6764755,C=4)
	CT(25430)=(S=6764756,E=6764759,C=2)
	CT(25431)=(S=6764760,E=6764763,C=1)
	CT(25432)=(S=6764764,E=6764775,C=4)
	CT(25433)=(S=6764776,E=6764779,C=74)
	CT(25434)=(S=6764780,E=6764799,C=4)
	CT(25435)=(S=6764800,E=6764803,C=0)
	CT(25436)=(S=6764804,E=6764807,C=2)
	CT(25437)=(S=6764808,E=6764811,C=74)
	CT(25438)=(S=6764812,E=6764815,C=1)
	CT(25439)=(S=6764816,E=6764819,C=4)
	CT(25440)=(S=6764820,E=6764823,C=84)
	CT(25441)=(S=6764824,E=6764827,C=8)
	CT(25442)=(S=6764828,E=6764831,C=2)
	CT(25443)=(S=6764832,E=6764835,C=0)
	CT(25444)=(S=6764836,E=6764839,C=74)
	CT(25445)=(S=6764840,E=6764843,C=84)
	CT(25446)=(S=6764844,E=6764851,C=2)
	CT(25447)=(S=6764852,E=6764859,C=1)
	CT(25448)=(S=6764860,E=6764863,C=6)
	CT(25449)=(S=6764864,E=6764867,C=4)
	CT(25450)=(S=6764868,E=6764868,C=0)
	CT(25451)=(S=6764869,E=6764871,C=4)
	CT(25452)=(S=6764872,E=6764875,C=7)
	CT(25453)=(S=6764876,E=6764879,C=1)
	CT(25454)=(S=6764880,E=6764888,C=4)
	CT(25455)=(S=6764889,E=6764889,C=0)
	CT(25456)=(S=6764890,E=6764890,C=8)
	CT(25457)=(S=6764891,E=6764891,C=0)
	CT(25458)=(S=6764892,E=6764895,C=92)
	CT(25459)=(S=6764896,E=6764903,C=2)
	CT(25460)=(S=6764904,E=6764907,C=11)
	CT(25461)=(S=6764908,E=6764911,C=1)
	CT(25462)=(S=6764912,E=6764915,C=11)
	CT(25463)=(S=6764916,E=6764923,C=84)
	CT(25464)=(S=6764924,E=6764927,C=4)
	CT(25465)=(S=6764928,E=6764931,C=2)
	CT(25466)=(S=6764932,E=6764935,C=4)
	CT(25467)=(S=6764936,E=6764939,C=1)
	CT(25468)=(S=6764940,E=6764943,C=4)
	CT(25469)=(S=6764944,E=6764947,C=2)
	CT(25470)=(S=6764948,E=6764949,C=0)
	CT(25471)=(S=6764950,E=6764950,C=4)
	CT(25472)=(S=6764951,E=6764951,C=84)
	CT(25473)=(S=6764952,E=6764955,C=77)
	CT(25474)=(S=6764956,E=6764959,C=7)
	CT(25475)=(S=6764960,E=6764963,C=4)
	CT(25476)=(S=6764964,E=6764967,C=0)
	CT(25477)=(S=6764968,E=6764971,C=77)
	CT(25478)=(S=6764972,E=6764987,C=4)
	CT(25479)=(S=6764988,E=6764991,C=5)
	CT(25480)=(S=6764992,E=6764995,C=74)
	CT(25481)=(S=6764996,E=6764999,C=1)
	CT(25482)=(S=6765000,E=6765003,C=8)
	CT(25483)=(S=6765004,E=6765007,C=75)
	CT(25484)=(S=6765008,E=6765011,C=11)
	CT(25485)=(S=6765012,E=6765019,C=4)
	CT(25486)=(S=6765020,E=6765023,C=11)
	CT(25487)=(S=6765024,E=6765025,C=8)
	CT(25488)=(S=6765026,E=6765026,C=4)
	CT(25489)=(S=6765027,E=6765027,C=10)
	CT(25490)=(S=6765028,E=6765031,C=8)
	CT(25491)=(S=6765032,E=6765035,C=167)
	CT(25492)=(S=6765036,E=6765071,C=4)
	CT(25493)=(S=6765072,E=6765075,C=5)
	CT(25494)=(S=6765076,E=6765079,C=91)
	CT(25495)=(S=6765080,E=6765083,C=1)
	CT(25496)=(S=6765084,E=6765087,C=4)
	CT(25497)=(S=6765088,E=6765088,C=0)
	CT(25498)=(S=6765089,E=6765089,C=74)
	CT(25499)=(S=6765090,E=6765091,C=0)
	CT(25500)=(S=6765092,E=6765123,C=4)
	CT(25501)=(S=6765124,E=6765127,C=2)
	CT(25502)=(S=6765128,E=6765131,C=84)
	CT(25503)=(S=6765132,E=6765147,C=4)
	CT(25504)=(S=6765148,E=6765151,C=84)
	CT(25505)=(S=6765152,E=6765155,C=4)
	CT(25506)=(S=6765156,E=6765163,C=74)
	CT(25507)=(S=6765164,E=6765165,C=84)
	CT(25508)=(S=6765166,E=6765166,C=7)
	CT(25509)=(S=6765167,E=6765167,C=74)
	CT(25510)=(S=6765168,E=6765168,C=7)
	CT(25511)=(S=6765169,E=6765169,C=5)
	CT(25512)=(S=6765170,E=6765199,C=4)
	CT(25513)=(S=6765200,E=6765201,C=75)
	CT(25514)=(S=6765202,E=6765203,C=74)
	CT(25515)=(S=6765204,E=6765207,C=3)
	CT(25516)=(S=6765208,E=6765215,C=4)
	CT(25517)=(S=6765216,E=6765219,C=74)
	CT(25518)=(S=6765220,E=6765231,C=4)
	CT(25519)=(S=6765232,E=6765235,C=8)
	CT(25520)=(S=6765236,E=6765236,C=4)
	CT(25521)=(S=6765237,E=6765237,C=8)
	CT(25522)=(S=6765238,E=6765239,C=1)
	CT(25523)=(S=6765240,E=6765247,C=4)
	CT(25524)=(S=6765248,E=6765255,C=0)
	CT(25525)=(S=6765256,E=6765267,C=4)
	CT(25526)=(S=6765268,E=6765279,C=0)
	CT(25527)=(S=6765280,E=6765283,C=86)
	CT(25528)=(S=6765284,E=6765287,C=73)
	CT(25529)=(S=6765288,E=6765288,C=0)
	CT(25530)=(S=6765289,E=6765343,C=4)
	CT(25531)=(S=6765344,E=6765347,C=2)
	CT(25532)=(S=6765348,E=6765351,C=84)
	CT(25533)=(S=6765352,E=6765363,C=8)
	CT(25534)=(S=6765364,E=6765367,C=154)
	CT(25535)=(S=6765368,E=6765387,C=4)
	CT(25536)=(S=6765388,E=6765391,C=1)
	CT(25537)=(S=6765392,E=6765400,C=4)
	CT(25538)=(S=6765401,E=6765401,C=2)
	CT(25539)=(S=6765402,E=6765403,C=8)
	CT(25540)=(S=6765404,E=6765407,C=6)
	CT(25541)=(S=6765408,E=6765411,C=4)
	CT(25542)=(S=6765412,E=6765415,C=1)
	CT(25543)=(S=6765416,E=6765419,C=4)
	CT(25544)=(S=6765420,E=6765423,C=0)
	CT(25545)=(S=6765424,E=6765443,C=1)
	CT(25546)=(S=6765444,E=6765447,C=4)
	CT(25547)=(S=6765448,E=6765451,C=2)
	CT(25548)=(S=6765456,E=6765459,C=8)
	CT(25549)=(S=6765460,E=6765463,C=1)
	CT(25550)=(S=6765464,E=6765467,C=4)
	CT(25551)=(S=6765468,E=6765471,C=6)
	CT(25552)=(S=6765472,E=6765475,C=0)
	CT(25553)=(S=6765476,E=6765479,C=1)
	CT(25554)=(S=6765480,E=6765483,C=84)
	CT(25555)=(S=6765484,E=6765487,C=86)
	CT(25556)=(S=6765488,E=6765491,C=84)
	CT(25557)=(S=6765492,E=6765493,C=4)
	CT(25558)=(S=6765494,E=6765495,C=0)
	CT(25559)=(S=6765496,E=6765527,C=4)
	CT(25560)=(S=6765528,E=6765531,C=1)
	CT(25561)=(S=6765532,E=6765535,C=9)
	CT(25562)=(S=6765536,E=6765544,C=4)
	CT(25563)=(S=6765545,E=6765545,C=84)
	CT(25564)=(S=6765546,E=6765547,C=74)
	CT(25565)=(S=6765548,E=6765571,C=4)
	CT(25566)=(S=6765572,E=6765575,C=2)
	CT(25567)=(S=6765576,E=6765579,C=7)
	CT(25568)=(S=6765580,E=6765583,C=2)
	CT(25569)=(S=6765584,E=6765587,C=11)
	CT(25570)=(S=6765588,E=6765588,C=0)
	CT(25571)=(S=6765589,E=6765589,C=4)
	CT(25572)=(S=6765590,E=6765591,C=0)
	CT(25573)=(S=6765592,E=6765595,C=5)
	CT(25574)=(S=6765596,E=6765599,C=4)
	CT(25575)=(S=6765600,E=6765603,C=1)
	CT(25576)=(S=6765604,E=6765607,C=2)
	CT(25577)=(S=6765608,E=6765611,C=4)
	CT(25578)=(S=6765612,E=6765615,C=1)
	CT(25579)=(S=6765616,E=6765623,C=4)
	CT(25580)=(S=6765624,E=6765635,C=8)
	CT(25581)=(S=6765636,E=6765655,C=4)
	CT(25582)=(S=6765656,E=6765663,C=2)
	CT(25583)=(S=6765664,E=6765664,C=0)
	CT(25584)=(S=6765665,E=6765665,C=92)
	CT(25585)=(S=6765666,E=6765666,C=0)
	CT(25586)=(S=6765667,E=6765667,C=73)
	CT(25587)=(S=6765668,E=6765672,C=4)
	CT(25588)=(S=6765673,E=6765675,C=0)
	CT(25589)=(S=6765676,E=6765679,C=8)
	CT(25590)=(S=6765680,E=6765687,C=4)
	CT(25591)=(S=6765688,E=6765695,C=6)
	CT(25592)=(S=6765696,E=6765699,C=4)
	CT(25593)=(S=6765700,E=6765702,C=0)
	CT(25594)=(S=6765703,E=6765711,C=4)
	CT(25595)=(S=6765712,E=6765719,C=8)
	CT(25596)=(S=6765720,E=6765723,C=0)
	CT(25597)=(S=6765724,E=6765727,C=4)
	CT(25598)=(S=6765728,E=6765731,C=84)
	CT(25599)=(S=6765732,E=6765735,C=1)
	CT(25600)=(S=6765736,E=6765739,C=10)
	CT(25601)=(S=6765740,E=6765743,C=84)
	CT(25602)=(S=6765744,E=6765747,C=4)
	CT(25603)=(S=6765748,E=6765751,C=74)
	CT(25604)=(S=6765752,E=6765752,C=10)
	CT(25605)=(S=6765753,E=6765753,C=3)
	CT(25606)=(S=6765754,E=6765755,C=4)
	CT(25607)=(S=6765756,E=6765759,C=2)
	CT(25608)=(S=6765760,E=6765763,C=73)
	CT(25609)=(S=6765764,E=6765767,C=4)
	CT(25610)=(S=6765768,E=6765771,C=0)
	CT(25611)=(S=6765772,E=6765775,C=2)
	CT(25612)=(S=6765776,E=6765783,C=4)
	CT(25613)=(S=6765784,E=6765784,C=0)
	CT(25614)=(S=6765785,E=6765785,C=7)
	CT(25615)=(S=6765786,E=6765787,C=4)
	CT(25616)=(S=6765788,E=6765791,C=8)
	CT(25617)=(S=6765792,E=6765795,C=2)
	CT(25618)=(S=6765796,E=6765799,C=1)
	CT(25619)=(S=6765800,E=6765803,C=8)
	CT(25620)=(S=6765804,E=6765807,C=1)
	CT(25621)=(S=6765808,E=6765811,C=0)
	CT(25622)=(S=6765812,E=6765815,C=4)
	CT(25623)=(S=6765816,E=6765819,C=8)
	CT(25624)=(S=6765820,E=6765823,C=10)
	CT(25625)=(S=6765824,E=6765827,C=8)
	CT(25626)=(S=6765828,E=6765831,C=7)
	CT(25627)=(S=6765832,E=6765835,C=155)
	CT(25628)=(S=6765836,E=6765839,C=4)
	CT(25629)=(S=6765840,E=6765843,C=74)
	CT(25630)=(S=6765844,E=6765847,C=8)
	CT(25631)=(S=6765848,E=6765851,C=86)
	CT(25632)=(S=6765852,E=6765855,C=8)
	CT(25633)=(S=6765856,E=6765859,C=4)
	CT(25634)=(S=6765860,E=6765863,C=8)
	CT(25635)=(S=6765864,E=6765864,C=0)
	CT(25636)=(S=6765865,E=6765865,C=74)
	CT(25637)=(S=6765866,E=6765867,C=0)
	CT(25638)=(S=6765868,E=6765875,C=11)
	CT(25639)=(S=6765876,E=6765883,C=4)
	CT(25640)=(S=6765884,E=6765887,C=1)
	CT(25641)=(S=6765888,E=6765892,C=4)
	CT(25642)=(S=6765893,E=6765893,C=73)
	CT(25643)=(S=6765894,E=6765894,C=0)
	CT(25644)=(S=6765895,E=6765895,C=74)
	CT(25645)=(S=6765896,E=6765915,C=4)
	CT(25646)=(S=6765916,E=6765919,C=86)
	CT(25647)=(S=6765920,E=6765923,C=0)
	CT(25648)=(S=6765924,E=6765927,C=4)
	CT(25649)=(S=6765928,E=6765931,C=1)
	CT(25650)=(S=6765932,E=6765939,C=4)
	CT(25651)=(S=6765940,E=6765943,C=74)
	CT(25652)=(S=6765944,E=6765947,C=8)
	CT(25653)=(S=6765948,E=6765951,C=5)
	CT(25654)=(S=6765952,E=6765955,C=0)
	CT(25655)=(S=6765956,E=6765959,C=4)
	CT(25656)=(S=6765960,E=6765963,C=9)
	CT(25657)=(S=6765964,E=6765967,C=1)
	CT(25658)=(S=6765968,E=6765971,C=8)
	CT(25659)=(S=6765972,E=6765974,C=4)
	CT(25660)=(S=6765975,E=6765975,C=2)
	CT(25661)=(S=6765976,E=6765987,C=1)
	CT(25662)=(S=6765988,E=6765991,C=4)
	CT(25663)=(S=6765992,E=6765995,C=8)
	CT(25664)=(S=6765996,E=6766003,C=1)
	CT(25665)=(S=6766004,E=6766007,C=4)
	CT(25666)=(S=6766008,E=6766015,C=1)
	CT(25667)=(S=6766016,E=6766017,C=7)
	CT(25668)=(S=6766018,E=6766043,C=4)
	CT(25669)=(S=6766044,E=6766047,C=8)
	CT(25670)=(S=6766048,E=6766053,C=0)
	CT(25671)=(S=6766054,E=6766055,C=4)
	CT(25672)=(S=6766056,E=6766059,C=2)
	CT(25673)=(S=6766060,E=6766067,C=8)
	CT(25674)=(S=6766068,E=6766071,C=2)
	CT(25675)=(S=6766072,E=6766075,C=74)
	CT(25676)=(S=6766076,E=6766079,C=4)
	CT(25677)=(S=6766080,E=6766083,C=10)
	CT(25678)=(S=6766084,E=6766087,C=7)
	CT(25679)=(S=6766088,E=6766091,C=11)
	CT(25680)=(S=6766092,E=6766103,C=4)
	CT(25681)=(S=6766104,E=6766107,C=1)
	CT(25682)=(S=6766108,E=6766109,C=0)
	CT(25683)=(S=6766110,E=6766111,C=10)
	CT(25684)=(S=6766112,E=6766119,C=0)
	CT(25685)=(S=6766120,E=6766123,C=4)
	CT(25686)=(S=6766124,E=6766131,C=8)
	CT(25687)=(S=6766132,E=6766135,C=1)
	CT(25688)=(S=6766136,E=6766151,C=4)
	CT(25689)=(S=6766152,E=6766171,C=1)
	CT(25690)=(S=6766172,E=6766175,C=4)
	CT(25691)=(S=6766176,E=6766215,C=1)
	CT(25692)=(S=6766216,E=6766219,C=4)
	CT(25693)=(S=6766220,E=6766225,C=84)
	CT(25694)=(S=6766226,E=6766227,C=4)
	CT(25695)=(S=6766228,E=6766231,C=8)
	CT(25696)=(S=6766232,E=6766235,C=10)
	CT(25697)=(S=6766236,E=6766275,C=1)
	CT(25698)=(S=6766276,E=6766277,C=74)
	CT(25699)=(S=6766278,E=6766279,C=4)
	CT(25700)=(S=6766280,E=6766281,C=0)
	CT(25701)=(S=6766282,E=6766283,C=74)
	CT(25702)=(S=6766284,E=6766307,C=1)
	CT(25703)=(S=6766308,E=6766311,C=6)
	CT(25704)=(S=6766312,E=6766315,C=77)
	CT(25705)=(S=6766316,E=6766319,C=4)
	CT(25706)=(S=6766320,E=6766323,C=7)
	CT(25707)=(S=6766324,E=6766327,C=2)
	CT(25708)=(S=6766328,E=6766335,C=4)
	CT(25709)=(S=6766336,E=6766343,C=8)
	CT(25710)=(S=6766344,E=6766361,C=4)
	CT(25711)=(S=6766362,E=6766362,C=0)
	CT(25712)=(S=6766363,E=6766363,C=162)
	CT(25713)=(S=6766364,E=6766367,C=4)
	CT(25714)=(S=6766368,E=6766427,C=1)
	CT(25715)=(S=6766428,E=6766431,C=73)
	CT(25716)=(S=6766432,E=6766435,C=74)
	CT(25717)=(S=6766436,E=6766436,C=0)
	CT(25718)=(S=6766437,E=6766437,C=4)
	CT(25719)=(S=6766438,E=6766438,C=10)
	CT(25720)=(S=6766439,E=6766439,C=0)
	CT(25721)=(S=6766440,E=6766459,C=11)
	CT(25722)=(S=6766460,E=6766467,C=4)
	CT(25723)=(S=6766468,E=6766475,C=8)
	CT(25724)=(S=6766476,E=6766483,C=1)
	CT(25725)=(S=6766484,E=6766487,C=4)
	CT(25726)=(S=6766488,E=6766491,C=1)
	CT(25727)=(S=6766492,E=6766495,C=84)
	CT(25728)=(S=6766496,E=6766523,C=1)
	CT(25729)=(S=6766524,E=6766524,C=154)
	CT(25730)=(S=6766525,E=6766525,C=0)
	CT(25731)=(S=6766526,E=6766526,C=86)
	CT(25732)=(S=6766527,E=6766527,C=0)
	CT(25733)=(S=6766528,E=6766547,C=1)
	CT(25734)=(S=6766548,E=6766551,C=11)
	CT(25735)=(S=6766552,E=6766563,C=2)
	CT(25736)=(S=6766564,E=6766567,C=0)
	CT(25737)=(S=6766568,E=6766568,C=4)
	CT(25738)=(S=6766569,E=6766569,C=74)
	CT(25739)=(S=6766570,E=6766571,C=84)
	CT(25740)=(S=6766572,E=6766575,C=8)
	CT(25741)=(S=6766576,E=6799391,C=1)
	CT(25742)=(S=6799392,E=6799395,C=4)
	CT(25743)=(S=6799396,E=6799403,C=8)
	CT(25744)=(S=6799404,E=6799407,C=87)
	CT(25745)=(S=6799408,E=6799419,C=1)
	CT(25746)=(S=6799420,E=6799427,C=4)
	CT(25747)=(S=6799428,E=6799431,C=2)
	CT(25748)=(S=6799432,E=6799435,C=4)
	CT(25749)=(S=6799436,E=6799439,C=75)
	CT(25750)=(S=6799440,E=6799440,C=73)
	CT(25751)=(S=6799441,E=6799441,C=89)
	CT(25752)=(S=6799442,E=6799443,C=0)
	CT(25753)=(S=6799444,E=6799475,C=1)
	CT(25754)=(S=6799476,E=6799479,C=4)
	CT(25755)=(S=6799480,E=6799483,C=2)
	CT(25756)=(S=6799484,E=6799487,C=10)
	CT(25757)=(S=6799488,E=6799507,C=1)
	CT(25758)=(S=6799508,E=6799511,C=2)
	CT(25759)=(S=6799512,E=6799515,C=0)
	CT(25760)=(S=6799516,E=6799519,C=84)
	CT(25761)=(S=6799520,E=6799523,C=8)
	CT(25762)=(S=6799524,E=6799527,C=1)
	CT(25763)=(S=6799528,E=6799532,C=4)
	CT(25764)=(S=6799533,E=6799533,C=0)
	CT(25765)=(S=6799534,E=6799534,C=155)
	CT(25766)=(S=6799535,E=6799535,C=0)
	CT(25767)=(S=6799536,E=6799539,C=8)
	CT(25768)=(S=6799540,E=6799543,C=86)
	CT(25769)=(S=6799544,E=6799547,C=10)
	CT(25770)=(S=6799548,E=6799551,C=1)
	CT(25771)=(S=6799552,E=6799556,C=0)
	CT(25772)=(S=6799557,E=6799557,C=73)
	CT(25773)=(S=6799558,E=6799559,C=4)
	CT(25774)=(S=6799560,E=6799563,C=8)
	CT(25775)=(S=6799564,E=6799567,C=4)
	CT(25776)=(S=6799568,E=6799579,C=1)
	CT(25777)=(S=6799580,E=6799583,C=4)
	CT(25778)=(S=6799584,E=6799587,C=8)
	CT(25779)=(S=6799588,E=6799595,C=2)
	CT(25780)=(S=6799596,E=6799599,C=11)
	CT(25781)=(S=6799600,E=6799603,C=73)
	CT(25782)=(S=6799604,E=6799611,C=4)
	CT(25783)=(S=6799612,E=6799615,C=1)
	CT(25784)=(S=6799616,E=6799619,C=4)
	CT(25785)=(S=6799620,E=6799623,C=7)
	CT(25786)=(S=6799624,E=6799625,C=73)
	CT(25787)=(S=6799626,E=6799627,C=0)
	CT(25788)=(S=6799628,E=6799631,C=2)
	CT(25789)=(S=6799632,E=6799632,C=73)
	CT(25790)=(S=6799633,E=6799633,C=0)
	CT(25791)=(S=6799634,E=6799634,C=77)
	CT(25792)=(S=6799635,E=6799635,C=74)
	CT(25793)=(S=6799636,E=6799647,C=4)
	CT(25794)=(S=6799648,E=6799651,C=8)
	CT(25795)=(S=6799652,E=6799653,C=4)
	CT(25796)=(S=6799654,E=6799655,C=73)
	CT(25797)=(S=6799656,E=6799663,C=1)
	CT(25798)=(S=6799664,E=6799683,C=4)
	CT(25799)=(S=6799684,E=6799687,C=8)
	CT(25800)=(S=6799688,E=6799691,C=2)
	CT(25801)=(S=6799692,E=6799692,C=0)
	CT(25802)=(S=6799693,E=6799693,C=4)
	CT(25803)=(S=6799694,E=6799694,C=0)
	CT(25804)=(S=6799695,E=6799695,C=4)
	CT(25805)=(S=6799696,E=6799699,C=0)
	CT(25806)=(S=6799700,E=6799731,C=4)
	CT(25807)=(S=6799732,E=6799735,C=74)
	CT(25808)=(S=6799736,E=6799743,C=1)
	CT(25809)=(S=6799744,E=6799751,C=8)
	CT(25810)=(S=6799752,E=6799755,C=73)
	CT(25811)=(S=6799756,E=6799767,C=1)
	CT(25812)=(S=6799768,E=6799775,C=4)
	CT(25813)=(S=6799776,E=6799779,C=1)
	CT(25814)=(S=6799780,E=6799780,C=73)
	CT(25815)=(S=6799781,E=6799781,C=10)
	CT(25816)=(S=6799782,E=6799783,C=0)
	CT(25817)=(S=6799784,E=6799787,C=84)
	CT(25818)=(S=6799788,E=6799791,C=8)
	CT(25819)=(S=6799792,E=6799795,C=2)
	CT(25820)=(S=6799796,E=6799799,C=86)
	CT(25821)=(S=6799800,E=6799803,C=2)
	CT(25822)=(S=6799804,E=6799811,C=1)
	CT(25823)=(S=6799812,E=6799819,C=4)
	CT(25824)=(S=6799820,E=6799823,C=86)
	CT(25825)=(S=6799824,E=6799827,C=0)
	CT(25826)=(S=6799828,E=6799859,C=1)
	CT(25827)=(S=6799860,E=6799867,C=8)
	CT(25828)=(S=6799868,E=6799871,C=4)
	CT(25829)=(S=6799872,E=6799879,C=0)
	CT(25830)=(S=6799880,E=6799887,C=4)
	CT(25831)=(S=6799888,E=6799891,C=1)
	CT(25832)=(S=6799892,E=6799907,C=4)
	CT(25833)=(S=6799908,E=6799915,C=8)
	CT(25834)=(S=6799916,E=6799919,C=4)
	CT(25835)=(S=6799920,E=6799923,C=0)
	CT(25836)=(S=6799924,E=6799939,C=4)
	CT(25837)=(S=6799940,E=6799940,C=0)
	CT(25838)=(S=6799941,E=6799943,C=4)
	CT(25839)=(S=6799944,E=6799947,C=2)
	CT(25840)=(S=6799948,E=6799951,C=7)
	CT(25841)=(S=6799952,E=6799955,C=5)
	CT(25842)=(S=6799956,E=6799959,C=86)
	CT(25843)=(S=6799960,E=6799971,C=4)
	CT(25844)=(S=6799972,E=6799979,C=8)
	CT(25845)=(S=6799980,E=6799983,C=6)
	CT(25846)=(S=6799984,E=6799992,C=4)
	CT(25847)=(S=6799993,E=6799993,C=0)
	CT(25848)=(S=6799994,E=6799995,C=4)
	CT(25849)=(S=6799996,E=6799999,C=0)
	CT(25850)=(S=6800000,E=6800001,C=4)
	CT(25851)=(S=6800002,E=6800002,C=74)
	CT(25852)=(S=6800003,E=6800019,C=4)
	CT(25853)=(S=6800020,E=6800023,C=8)
	CT(25854)=(S=6800024,E=6800035,C=4)
	CT(25855)=(S=6800036,E=6800039,C=2)
	CT(25856)=(S=6800040,E=6800043,C=36)
	CT(25857)=(S=6800044,E=6800047,C=74)
	CT(25858)=(S=6800048,E=6800051,C=92)
	CT(25859)=(S=6800052,E=6800055,C=4)
	CT(25860)=(S=6800060,E=6800063,C=11)
	CT(25861)=(S=6800064,E=6800083,C=4)
	CT(25862)=(S=6800084,E=6800085,C=8)
	CT(25863)=(S=6800086,E=6800087,C=0)
	CT(25864)=(S=6800088,E=6800099,C=4)
	CT(25865)=(S=6800100,E=6800100,C=7)
	CT(25866)=(S=6800101,E=6800101,C=0)
	CT(25867)=(S=6800102,E=6800103,C=1)
	CT(25868)=(S=6800104,E=6800107,C=4)
	CT(25869)=(S=6800108,E=6800111,C=8)
	CT(25870)=(S=6800112,E=6800115,C=4)
	CT(25871)=(S=6800116,E=6800119,C=8)
	CT(25872)=(S=6800120,E=6800123,C=4)
	CT(25873)=(S=6800124,E=6800127,C=6)
	CT(25874)=(S=6800128,E=6800131,C=84)
	CT(25875)=(S=6800132,E=6800135,C=8)
	CT(25876)=(S=6800136,E=6800139,C=73)
	CT(25877)=(S=6800140,E=6800143,C=0)
	CT(25878)=(S=6800144,E=6800144,C=4)
	CT(25879)=(S=6800145,E=6800145,C=0)
	CT(25880)=(S=6800146,E=6800147,C=74)
	CT(25881)=(S=6800148,E=6800151,C=5)
	CT(25882)=(S=6800152,E=6800155,C=4)
	CT(25883)=(S=6800156,E=6800156,C=7)
	CT(25884)=(S=6800157,E=6800159,C=74)
	CT(25885)=(S=6800160,E=6800163,C=7)
	CT(25886)=(S=6800164,E=6800175,C=4)
	CT(25887)=(S=6800176,E=6800179,C=8)
	CT(25888)=(S=6800180,E=6800181,C=0)
	CT(25889)=(S=6800182,E=6800183,C=84)
	CT(25890)=(S=6800184,E=6800187,C=74)
	CT(25891)=(S=6800188,E=6800192,C=0)
	CT(25892)=(S=6800193,E=6800193,C=74)
	CT(25893)=(S=6800194,E=6800195,C=0)
	CT(25894)=(S=6800196,E=6800215,C=4)
	CT(25895)=(S=6800216,E=6800219,C=74)
	CT(25896)=(S=6800220,E=6800227,C=4)
	CT(25897)=(S=6800228,E=6800231,C=7)
	CT(25898)=(S=6800232,E=6800235,C=1)
	CT(25899)=(S=6800236,E=6800239,C=8)
	CT(25900)=(S=6800240,E=6800243,C=1)
	CT(25901)=(S=6800244,E=6800247,C=8)
	CT(25902)=(S=6800248,E=6800255,C=4)
	CT(25903)=(S=6800256,E=6800259,C=0)
	CT(25904)=(S=6800260,E=6800263,C=8)
	CT(25905)=(S=6800264,E=6800267,C=1)
	CT(25906)=(S=6800268,E=6800269,C=84)
	CT(25907)=(S=6800270,E=6800270,C=74)
	CT(25908)=(S=6800271,E=6800271,C=5)
	CT(25909)=(S=6800272,E=6800275,C=2)
	CT(25910)=(S=6800276,E=6800283,C=1)
	CT(25911)=(S=6800284,E=6800287,C=74)
	CT(25912)=(S=6800288,E=6800291,C=1)
	CT(25913)=(S=6800292,E=6800297,C=74)
	CT(25914)=(S=6800298,E=6800299,C=0)
	CT(25915)=(S=6800300,E=6800315,C=4)
	CT(25916)=(S=6800316,E=6800319,C=7)
	CT(25917)=(S=6800320,E=6800323,C=1)
	CT(25918)=(S=6800324,E=6800331,C=4)
	CT(25919)=(S=6800332,E=6800333,C=84)
	CT(25920)=(S=6800334,E=6800335,C=0)
	CT(25921)=(S=6800336,E=6800363,C=4)
	CT(25922)=(S=6800364,E=6800371,C=11)
	CT(25923)=(S=6800372,E=6800395,C=4)
	CT(25924)=(S=6800396,E=6800397,C=3)
	CT(25925)=(S=6800398,E=6800398,C=74)
	CT(25926)=(S=6800399,E=6800399,C=4)
	CT(25927)=(S=6800400,E=6800403,C=11)
	CT(25928)=(S=6800404,E=6800407,C=8)
	CT(25929)=(S=6800408,E=6800409,C=73)
	CT(25930)=(S=6800410,E=6800419,C=4)
	CT(25931)=(S=6800420,E=6800423,C=8)
	CT(25932)=(S=6800424,E=6800425,C=77)
	CT(25933)=(S=6800426,E=6800431,C=74)
	CT(25934)=(S=6800432,E=6800435,C=8)
	CT(25935)=(S=6800436,E=6800439,C=4)
	CT(25936)=(S=6800440,E=6800443,C=10)
	CT(25937)=(S=6800444,E=6800451,C=1)
	CT(25938)=(S=6800452,E=6800455,C=0)
	CT(25939)=(S=6800456,E=6800459,C=1)
	CT(25940)=(S=6800460,E=6800463,C=4)
	CT(25941)=(S=6800464,E=6800468,C=0)
	CT(25942)=(S=6800472,E=6800483,C=1)
	CT(25943)=(S=6807556,E=6807559,C=4)
	CT(25944)=(S=6807560,E=6807563,C=2)
	CT(25945)=(S=6807564,E=6807567,C=77)
	CT(25946)=(S=6807572,E=6807572,C=8)
	CT(25947)=(S=6807573,E=6807573,C=0)
	CT(25948)=(S=6807574,E=6807574,C=8)
	CT(25949)=(S=6807575,E=6807579,C=4)
	CT(25950)=(S=6807580,E=6807583,C=86)
	CT(25951)=(S=6807584,E=6807591,C=4)
	CT(25952)=(S=6807592,E=6807599,C=1)
	CT(25953)=(S=6807600,E=6807600,C=4)
	CT(25954)=(S=6807601,E=6807601,C=0)
	CT(25955)=(S=6807602,E=6807603,C=4)
	CT(25956)=(S=6807604,E=6807605,C=0)
	CT(25957)=(S=6807606,E=6807606,C=4)
	CT(25958)=(S=6807607,E=6807607,C=0)
	CT(25959)=(S=6807608,E=6807611,C=8)
	CT(25960)=(S=6807612,E=6807615,C=1)
	CT(25961)=(S=6807616,E=6807619,C=74)
	CT(25962)=(S=6807620,E=6807623,C=4)
	CT(25963)=(S=6807624,E=6807627,C=8)
	CT(25964)=(S=6807628,E=6807631,C=73)
	CT(25965)=(S=6807632,E=6807635,C=1)
	CT(25966)=(S=6807636,E=6807643,C=0)
	CT(25967)=(S=6807644,E=6807647,C=2)
	CT(25968)=(S=6807648,E=6807651,C=157)
	CT(25969)=(S=6807652,E=6807655,C=74)
	CT(25970)=(S=6807656,E=6807656,C=0)
	CT(25971)=(S=6807657,E=6807657,C=4)
	CT(25972)=(S=6807658,E=6807658,C=8)
	CT(25973)=(S=6807659,E=6807659,C=73)
	CT(25974)=(S=6807660,E=6807663,C=4)
	CT(25975)=(S=6807664,E=6807667,C=0)
	CT(25976)=(S=6807668,E=6807671,C=7)
	CT(25977)=(S=6807672,E=6807678,C=0)
	CT(25978)=(S=6807679,E=6807679,C=89)
	CT(25979)=(S=6807680,E=6807683,C=73)
	CT(25980)=(S=6807684,E=6807687,C=9)
	CT(25981)=(S=6807688,E=6807693,C=74)
	CT(25982)=(S=6807694,E=6807694,C=0)
	CT(25983)=(S=6807695,E=6807695,C=2)
	CT(25984)=(S=6807696,E=6807711,C=4)
	CT(25985)=(S=6807712,E=6807712,C=0)
	CT(25986)=(S=6807713,E=6807713,C=73)
	CT(25987)=(S=6807714,E=6807715,C=0)
	CT(25988)=(S=6807716,E=6807719,C=7)
	CT(25989)=(S=6807720,E=6807723,C=11)
	CT(25990)=(S=6807724,E=6807732,C=74)
	CT(25991)=(S=6807733,E=6807733,C=4)
	CT(25992)=(S=6807734,E=6807735,C=0)
	CT(25993)=(S=6807736,E=6807743,C=4)
	CT(25994)=(S=6807744,E=6807747,C=9)
	CT(25995)=(S=6807748,E=6807751,C=5)
	CT(25996)=(S=6807752,E=6807759,C=9)
	CT(25997)=(S=6807760,E=6807765,C=0)
	CT(25998)=(S=6807766,E=6807767,C=89)
	CT(25999)=(S=6807768,E=6807771,C=4)
	CT(26000)=(S=6807772,E=6807787,C=1)
	CT(26001)=(S=6807788,E=6807799,C=4)
	CT(26002)=(S=6807800,E=6807803,C=8)
	CT(26003)=(S=6807804,E=6807807,C=2)
	CT(26004)=(S=6807808,E=6807811,C=9)
	CT(26005)=(S=6807812,E=6807815,C=73)
	CT(26006)=(S=6807816,E=6807819,C=8)
	CT(26007)=(S=6807820,E=6807823,C=4)
	CT(26008)=(S=6807824,E=6807831,C=8)
	CT(26009)=(S=6807832,E=6807839,C=4)
	CT(26010)=(S=6807840,E=6807843,C=0)
	CT(26011)=(S=6807844,E=6807847,C=10)
	CT(26012)=(S=6807848,E=6807851,C=4)
	CT(26013)=(S=6807852,E=6807853,C=0)
	CT(26014)=(S=6807854,E=6807855,C=4)
	CT(26015)=(S=6807856,E=6807859,C=77)
	CT(26016)=(S=6807860,E=6807863,C=2)
	CT(26017)=(S=6807864,E=6807871,C=4)
	CT(26018)=(S=6807872,E=6807873,C=7)
	CT(26019)=(S=6807874,E=6807874,C=74)
	CT(26020)=(S=6807875,E=6807879,C=0)
	CT(26021)=(S=6807880,E=6807885,C=4)
	CT(26022)=(S=6807886,E=6807887,C=86)
	CT(26023)=(S=6807888,E=6807891,C=2)
	CT(26024)=(S=6807892,E=6807895,C=1)
	CT(26025)=(S=6807896,E=6807899,C=74)
	CT(26026)=(S=6807900,E=6807903,C=84)
	CT(26027)=(S=6807904,E=6807904,C=0)
	CT(26028)=(S=6807905,E=6807905,C=4)
	CT(26029)=(S=6807906,E=6807906,C=74)
	CT(26030)=(S=6807907,E=6807923,C=4)
	CT(26031)=(S=6807928,E=6807931,C=2)
	CT(26032)=(S=6807932,E=6807943,C=4)
	CT(26033)=(S=6807944,E=6807947,C=10)
	CT(26034)=(S=6807948,E=6807951,C=0)
	CT(26035)=(S=6807952,E=6807955,C=2)
	CT(26036)=(S=6807956,E=6807959,C=74)
	CT(26037)=(S=6807960,E=6807963,C=2)
	CT(26038)=(S=6807964,E=6807975,C=0)
	CT(26039)=(S=6807976,E=6807977,C=3)
	CT(26040)=(S=6807978,E=6807981,C=74)
	CT(26041)=(S=6807982,E=6807982,C=4)
	CT(26042)=(S=6807983,E=6807983,C=74)
	CT(26043)=(S=6807984,E=6807991,C=4)
	CT(26044)=(S=6807992,E=6807993,C=74)
	CT(26045)=(S=6807994,E=6807995,C=0)
	CT(26046)=(S=6807996,E=6807999,C=4)
	CT(26047)=(S=6808000,E=6808003,C=2)
	CT(26048)=(S=6808004,E=6808011,C=8)
	CT(26049)=(S=6808012,E=6808015,C=4)
	CT(26050)=(S=6808016,E=6808017,C=74)
	CT(26051)=(S=6808018,E=6808019,C=73)
	CT(26052)=(S=6808020,E=6808023,C=8)
	CT(26053)=(S=6808024,E=6808027,C=89)
	CT(26054)=(S=6808028,E=6808031,C=77)
	CT(26055)=(S=6808032,E=6808035,C=4)
	CT(26056)=(S=6808036,E=6808039,C=84)
	CT(26057)=(S=6808040,E=6808043,C=0)
	CT(26058)=(S=6808044,E=6808047,C=11)
	CT(26059)=(S=6808048,E=6808049,C=0)
	CT(26060)=(S=6808050,E=6808050,C=74)
	CT(26061)=(S=6808051,E=6808051,C=0)
	CT(26062)=(S=6808052,E=6808055,C=75)
	CT(26063)=(S=6808056,E=6808059,C=8)
	CT(26064)=(S=6808060,E=6808063,C=2)
	CT(26065)=(S=6808064,E=6808071,C=4)
	CT(26066)=(S=6808072,E=6808074,C=0)
	CT(26067)=(S=6808075,E=6808075,C=4)
	CT(26068)=(S=6808076,E=6808079,C=10)
	CT(26069)=(S=6808080,E=6808083,C=1)
	CT(26070)=(S=6808084,E=6808085,C=4)
	CT(26071)=(S=6808086,E=6808087,C=160)
	CT(26072)=(S=6808088,E=6808095,C=4)
	CT(26073)=(S=6808096,E=6808099,C=73)
	CT(26074)=(S=6808100,E=6808103,C=10)
	CT(26075)=(S=6808104,E=6808107,C=1)
	CT(26076)=(S=6808108,E=6808111,C=2)
	CT(26077)=(S=6808112,E=6808115,C=74)
	CT(26078)=(S=6808116,E=6808116,C=4)
	CT(26079)=(S=6808117,E=6808117,C=0)
	CT(26080)=(S=6808118,E=6808118,C=2)
	CT(26081)=(S=6808119,E=6808119,C=74)
	CT(26082)=(S=6808120,E=6808127,C=1)
	CT(26083)=(S=6808128,E=6808131,C=73)
	CT(26084)=(S=6808132,E=6808135,C=0)
	CT(26085)=(S=6808136,E=6808143,C=6)
	CT(26086)=(S=6808144,E=6808147,C=1)
	CT(26087)=(S=6808148,E=6808155,C=4)
	CT(26088)=(S=6808156,E=6808159,C=8)
	CT(26089)=(S=6808160,E=6808163,C=6)
	CT(26090)=(S=6808164,E=6808167,C=9)
	CT(26091)=(S=6808168,E=6808171,C=0)
	CT(26092)=(S=6808172,E=6808175,C=11)
	CT(26093)=(S=6808176,E=6808179,C=73)
	CT(26094)=(S=6808180,E=6808183,C=1)
	CT(26095)=(S=6808184,E=6808185,C=7)
	CT(26096)=(S=6808186,E=6808186,C=84)
	CT(26097)=(S=6808187,E=6808187,C=0)
	CT(26098)=(S=6808188,E=6808191,C=8)
	CT(26099)=(S=6808192,E=6808195,C=0)
	CT(26100)=(S=6808196,E=6808199,C=1)
	CT(26101)=(S=6808200,E=6808203,C=74)
	CT(26102)=(S=6808204,E=6808215,C=4)
	CT(26103)=(S=6808216,E=6808219,C=8)
	CT(26104)=(S=6808220,E=6808223,C=1)
	CT(26105)=(S=6808224,E=6808231,C=0)
	CT(26106)=(S=6808232,E=6808235,C=4)
	CT(26107)=(S=6808236,E=6808237,C=7)
	CT(26108)=(S=6808238,E=6808238,C=74)
	CT(26109)=(S=6808239,E=6808239,C=4)
	CT(26110)=(S=6808240,E=6808243,C=0)
	CT(26111)=(S=6808244,E=6808247,C=1)
	CT(26112)=(S=6808248,E=6808259,C=4)
	CT(26113)=(S=6808260,E=6808263,C=1)
	CT(26114)=(S=6808264,E=6808275,C=4)
	CT(26115)=(S=6808276,E=6808279,C=9)
	CT(26116)=(S=6808280,E=6808281,C=77)
	CT(26117)=(S=6808282,E=6808282,C=74)
	CT(26118)=(S=6808283,E=6808287,C=0)
	CT(26119)=(S=6808288,E=6808295,C=4)
	CT(26120)=(S=6808296,E=6808299,C=74)
	CT(26121)=(S=6808300,E=6808307,C=4)
	CT(26122)=(S=6808308,E=6808309,C=0)
	CT(26123)=(S=6808310,E=6808310,C=7)
	CT(26124)=(S=6808311,E=6808311,C=0)
	CT(26125)=(S=6808312,E=6808315,C=11)
	CT(26126)=(S=6808316,E=6808319,C=73)
	CT(26127)=(S=6808320,E=6808320,C=4)
	CT(26128)=(S=6808322,E=6808323,C=10)
	CT(26129)=(S=6808328,E=6808331,C=2)
	CT(26130)=(S=6808332,E=6808333,C=7)
	CT(26131)=(S=6808334,E=6808334,C=74)
	CT(26132)=(S=6808335,E=6808335,C=73)
	CT(26133)=(S=6808336,E=6808339,C=89)
	CT(26134)=(S=6808340,E=6808347,C=0)
	CT(26135)=(S=6808348,E=6808351,C=73)
	CT(26136)=(S=6808352,E=6808355,C=9)
	CT(26137)=(S=6808356,E=6808359,C=4)
	CT(26138)=(S=6808360,E=6808363,C=2)
	CT(26139)=(S=6808364,E=6808367,C=10)
	CT(26140)=(S=6808368,E=6808371,C=1)
	CT(26141)=(S=6808372,E=6808379,C=4)
	CT(26142)=(S=6808380,E=6808381,C=0)
	CT(26143)=(S=6808382,E=6808382,C=4)
	CT(26144)=(S=6808383,E=6808387,C=0)
	CT(26145)=(S=6808388,E=6808391,C=4)
	CT(26146)=(S=6808392,E=6808403,C=1)
	CT(26147)=(S=6808404,E=6808411,C=8)
	CT(26148)=(S=6808416,E=6808419,C=4)
	CT(26149)=(S=6808420,E=6808423,C=1)
	CT(26150)=(S=6808424,E=6808427,C=8)
	CT(26151)=(S=6808428,E=6808431,C=0)
	CT(26152)=(S=6808432,E=6808435,C=11)
	CT(26153)=(S=6808436,E=6808439,C=4)
	CT(26154)=(S=6808440,E=6808443,C=1)
	CT(26155)=(S=6808444,E=6808447,C=4)
	CT(26156)=(S=6808448,E=6808451,C=8)
	CT(26157)=(S=6808452,E=6808459,C=1)
	CT(26158)=(S=6808460,E=6808460,C=5)
	CT(26159)=(S=6808461,E=6808469,C=74)
	CT(26160)=(S=6808470,E=6808479,C=4)
	CT(26161)=(S=6808480,E=6808483,C=2)
	CT(26162)=(S=6808484,E=6808487,C=7)
	CT(26163)=(S=6808488,E=6808489,C=0)
	CT(26164)=(S=6808490,E=6808491,C=73)
	CT(26165)=(S=6808492,E=6808499,C=7)
	CT(26166)=(S=6808508,E=6808511,C=2)
	CT(26167)=(S=6808512,E=6808515,C=8)
	CT(26168)=(S=6808516,E=6808519,C=1)
	CT(26169)=(S=6808520,E=6808523,C=73)
	CT(26170)=(S=6808524,E=6808527,C=1)
	CT(26171)=(S=6808528,E=6808531,C=4)
	CT(26172)=(S=6808532,E=6808535,C=1)
	CT(26173)=(S=6808536,E=6808539,C=11)
	CT(26174)=(S=6808540,E=6808543,C=74)
	CT(26175)=(S=6808544,E=6808547,C=9)
	CT(26176)=(S=6808548,E=6808551,C=1)
	CT(26177)=(S=6808552,E=6808552,C=0)
	CT(26178)=(S=6808553,E=6808553,C=10)
	CT(26179)=(S=6808554,E=6808555,C=0)
	CT(26180)=(S=6808556,E=6808559,C=4)
	CT(26181)=(S=6808560,E=6808563,C=74)
	CT(26182)=(S=6808564,E=6808567,C=156)
	CT(26183)=(S=6808568,E=6808571,C=2)
	CT(26184)=(S=6808572,E=6808575,C=74)
	CT(26185)=(S=6808576,E=6808579,C=84)
	CT(26186)=(S=6808580,E=6808583,C=3)
	CT(26187)=(S=6808588,E=6808591,C=1)
	CT(26188)=(S=6808592,E=6808592,C=0)
	CT(26189)=(S=6808593,E=6808593,C=74)
	CT(26190)=(S=6808594,E=6808595,C=4)
	CT(26191)=(S=6808596,E=6808599,C=11)
	CT(26192)=(S=6808600,E=6808603,C=74)
	CT(26193)=(S=6808604,E=6808607,C=1)
	CT(26194)=(S=6808608,E=6808624,C=4)
	CT(26195)=(S=6808625,E=6808625,C=84)
	CT(26196)=(S=6808626,E=6808627,C=4)
	CT(26197)=(S=6808628,E=6808635,C=5)
	CT(26198)=(S=6808636,E=6808639,C=2)
	CT(26199)=(S=6808640,E=6808643,C=8)
	CT(26200)=(S=6808644,E=6808647,C=1)
	CT(26201)=(S=6808648,E=6808655,C=4)
	CT(26202)=(S=6808656,E=6808657,C=74)
	CT(26203)=(S=6808658,E=6808658,C=8)
	CT(26204)=(S=6808659,E=6808659,C=4)
	CT(26205)=(S=6808664,E=6808667,C=1)
	CT(26206)=(S=6808668,E=6808675,C=8)
	CT(26207)=(S=6808676,E=6808676,C=7)
	CT(26208)=(S=6808677,E=6808677,C=73)
	CT(26209)=(S=6808678,E=6808678,C=10)
	CT(26210)=(S=6808679,E=6808679,C=4)
	CT(26211)=(S=6808684,E=6808687,C=8)
	CT(26212)=(S=6808688,E=6808691,C=4)
	CT(26213)=(S=6808692,E=6808695,C=74)
	CT(26214)=(S=6808696,E=6808699,C=84)
	CT(26215)=(S=6808700,E=6808703,C=0)
	CT(26216)=(S=6808704,E=6808707,C=1)
	CT(26217)=(S=6808716,E=6808731,C=4)
	CT(26218)=(S=6808732,E=6808735,C=77)
	CT(26219)=(S=6808736,E=6808739,C=1)
	CT(26220)=(S=6808740,E=6808743,C=0)
	CT(26221)=(S=6808748,E=6808751,C=4)
	CT(26222)=(S=6808752,E=6808755,C=1)
	CT(26223)=(S=6808756,E=6808759,C=8)
	CT(26224)=(S=6808760,E=6808763,C=4)
	CT(26225)=(S=6808764,E=6808767,C=0)
	CT(26226)=(S=6808768,E=6808769,C=4)
	CT(26227)=(S=6808770,E=6808771,C=0)
	CT(26228)=(S=6808772,E=6808775,C=2)
	CT(26229)=(S=6808776,E=6808779,C=84)
	CT(26230)=(S=6808780,E=6808787,C=1)
	CT(26231)=(S=6808788,E=6808791,C=0)
	CT(26232)=(S=6808792,E=6808795,C=9)
	CT(26233)=(S=6808796,E=6808799,C=4)
	CT(26234)=(S=6808804,E=6808811,C=1)
	CT(26235)=(S=6808812,E=6808824,C=74)
	CT(26236)=(S=6808825,E=6808827,C=4)
	CT(26237)=(S=6808828,E=6808831,C=154)
	CT(26238)=(S=6808832,E=6808833,C=0)
	CT(26239)=(S=6808834,E=6808835,C=4)
	CT(26240)=(S=6808836,E=6808836,C=0)
	CT(26241)=(S=6808837,E=6808837,C=4)
	CT(26242)=(S=6808838,E=6808839,C=0)
	CT(26243)=(S=6808840,E=6808843,C=7)
	CT(26244)=(S=6808844,E=6808845,C=74)
	CT(26245)=(S=6808846,E=6808847,C=4)
	CT(26246)=(S=6808848,E=6808851,C=9)
	CT(26247)=(S=6808852,E=6808855,C=1)
	CT(26248)=(S=6808856,E=6808859,C=4)
	CT(26249)=(S=6808860,E=6808863,C=8)
	CT(26250)=(S=6808864,E=6808867,C=5)
	CT(26251)=(S=6808868,E=6808871,C=4)
	CT(26252)=(S=6808872,E=6808875,C=11)
	CT(26253)=(S=6808876,E=6808879,C=84)
	CT(26254)=(S=6808880,E=6808882,C=0)
	CT(26255)=(S=6808883,E=6808883,C=4)
	CT(26256)=(S=6808884,E=6808887,C=8)
	CT(26257)=(S=6808888,E=6808891,C=5)
	CT(26258)=(S=6808896,E=6808899,C=4)
	CT(26259)=(S=6808904,E=6808909,C=74)
	CT(26260)=(S=6808910,E=6808911,C=4)
	CT(26261)=(S=6808912,E=6808919,C=84)
	CT(26262)=(S=6808920,E=6808923,C=7)
	CT(26263)=(S=6808924,E=6808927,C=3)
	CT(26264)=(S=6808928,E=6808928,C=7)
	CT(26265)=(S=6808929,E=6808929,C=0)
	CT(26266)=(S=6808930,E=6808931,C=73)
	CT(26267)=(S=6808932,E=6808935,C=9)
	CT(26268)=(S=6808936,E=6808939,C=10)
	CT(26269)=(S=6808940,E=6808943,C=3)
	CT(26270)=(S=6808944,E=6808947,C=5)
	CT(26271)=(S=6808948,E=6808951,C=8)
	CT(26272)=(S=6808952,E=6808955,C=92)
	CT(26273)=(S=6808956,E=6808959,C=9)
	CT(26274)=(S=6808960,E=6808960,C=7)
	CT(26275)=(S=6808961,E=6808961,C=4)
	CT(26276)=(S=6808962,E=6808962,C=73)
	CT(26277)=(S=6808963,E=6808963,C=4)
	CT(26278)=(S=6808964,E=6808967,C=9)
	CT(26279)=(S=6808968,E=6808971,C=1)
	CT(26280)=(S=6808972,E=6808979,C=8)
	CT(26281)=(S=6808980,E=6808983,C=1)
	CT(26282)=(S=6808988,E=6808991,C=6)
	CT(26283)=(S=6808992,E=6808999,C=74)
	CT(26284)=(S=6809000,E=6809003,C=4)
	CT(26285)=(S=6809004,E=6809007,C=1)
	CT(26286)=(S=6809008,E=6809011,C=92)
	CT(26287)=(S=6809012,E=6809015,C=8)
	CT(26288)=(S=6809016,E=6809017,C=4)
	CT(26289)=(S=6809018,E=6809018,C=7)
	CT(26290)=(S=6809019,E=6809019,C=0)
	CT(26291)=(S=6809020,E=6809023,C=8)
	CT(26292)=(S=6809024,E=6809027,C=11)
	CT(26293)=(S=6809028,E=6809031,C=8)
	CT(26294)=(S=6809032,E=6809035,C=74)
	CT(26295)=(S=6809036,E=6809039,C=7)
	CT(26296)=(S=6809040,E=6809043,C=4)
	CT(26297)=(S=6809044,E=6809055,C=1)
	CT(26298)=(S=6809056,E=6809059,C=4)
	CT(26299)=(S=6809060,E=6809063,C=1)
	CT(26300)=(S=6809064,E=6809067,C=4)
	CT(26301)=(S=6809068,E=6809075,C=1)
	CT(26302)=(S=6809076,E=6809079,C=4)
	CT(26303)=(S=6809080,E=6809081,C=73)
	CT(26304)=(S=6809082,E=6809083,C=4)
	CT(26305)=(S=6809084,E=6809087,C=2)
	CT(26306)=(S=6809088,E=6809091,C=1)
	CT(26307)=(S=6809092,E=6809095,C=84)
	CT(26308)=(S=6809096,E=6809099,C=2)
	CT(26309)=(S=6809108,E=6809111,C=4)
	CT(26310)=(S=6809112,E=6809115,C=2)
	CT(26311)=(S=6809116,E=6809119,C=1)
	CT(26312)=(S=6809120,E=6809120,C=8)
	CT(26313)=(S=6809121,E=6809121,C=4)
	CT(26314)=(S=6809122,E=6809123,C=8)
	CT(26315)=(S=6809124,E=6809127,C=7)
	CT(26316)=(S=6809128,E=6809135,C=1)
	CT(26317)=(S=6809136,E=6809137,C=74)
	CT(26318)=(S=6809138,E=6809139,C=7)
	CT(26319)=(S=6809140,E=6809143,C=4)
	CT(26320)=(S=6809144,E=6809147,C=5)
	CT(26321)=(S=6809148,E=6809151,C=84)
	CT(26322)=(S=6809152,E=6809155,C=157)
	CT(26323)=(S=6809160,E=6809163,C=0)
	CT(26324)=(S=6809164,E=6809167,C=73)
	CT(26325)=(S=6809168,E=6809168,C=0)
	CT(26326)=(S=6809169,E=6809169,C=73)
	CT(26327)=(S=6809170,E=6809170,C=92)
	CT(26328)=(S=6809171,E=6809171,C=74)
	CT(26329)=(S=6809172,E=6809175,C=4)
	CT(26330)=(S=6809176,E=6809179,C=6)
	CT(26331)=(S=6809180,E=6809183,C=2)
	CT(26332)=(S=6809184,E=6809187,C=1)
	CT(26333)=(S=6809189,E=6809191,C=4)
	CT(26334)=(S=6809192,E=6809195,C=84)
	CT(26335)=(S=6809196,E=6809199,C=8)
	CT(26336)=(S=6809200,E=6809203,C=6)
	CT(26337)=(S=6809208,E=6809211,C=8)
	CT(26338)=(S=6809216,E=6809219,C=2)
	CT(26339)=(S=6809220,E=6809223,C=3)
	CT(26340)=(S=6809224,E=6809227,C=8)
	CT(26341)=(S=6809232,E=6809243,C=4)
	CT(26342)=(S=6809244,E=6809247,C=0)
	CT(26343)=(S=6809248,E=6809251,C=8)
	CT(26344)=(S=6809254,E=6809255,C=4)
	CT(26345)=(S=6809256,E=6809259,C=2)
	CT(26346)=(S=6809264,E=6809267,C=0)
	CT(26347)=(S=6809268,E=6809271,C=84)
	CT(26348)=(S=6809272,E=6809275,C=0)
	CT(26349)=(S=6809276,E=6809279,C=74)
	CT(26350)=(S=6809280,E=6809283,C=8)
	CT(26351)=(S=6809284,E=6809303,C=1)
	CT(26352)=(S=6809304,E=6809307,C=8)
	CT(26353)=(S=6809308,E=6809315,C=4)
	CT(26354)=(S=6809316,E=6809319,C=8)
	CT(26355)=(S=6809320,E=6809323,C=7)
	CT(26356)=(S=6809324,E=6809327,C=1)
	CT(26357)=(S=6809328,E=6809331,C=8)
	CT(26358)=(S=6809332,E=6809335,C=2)
	CT(26359)=(S=6809336,E=6809339,C=7)
	CT(26360)=(S=6809340,E=6809343,C=89)
	CT(26361)=(S=6809344,E=6809355,C=4)
	CT(26362)=(S=6809356,E=6809359,C=8)
	CT(26363)=(S=6809360,E=6809367,C=1)
	CT(26364)=(S=6809368,E=6809369,C=2)
	CT(26365)=(S=6809370,E=6809370,C=0)
	CT(26366)=(S=6809371,E=6809371,C=7)
	CT(26367)=(S=6809372,E=6809375,C=8)
	CT(26368)=(S=6809376,E=6809379,C=4)
	CT(26369)=(S=6809380,E=6809383,C=10)
	CT(26370)=(S=6809384,E=6809391,C=4)
	CT(26371)=(S=6809392,E=6809395,C=9)
	CT(26372)=(S=6809400,E=6809403,C=8)
	CT(26373)=(S=6809404,E=6809407,C=86)
	CT(26374)=(S=6809408,E=6809415,C=1)
	CT(26375)=(S=6809416,E=6809419,C=2)
	CT(26376)=(S=6809420,E=6809423,C=4)
	CT(26377)=(S=6809424,E=6809427,C=74)
	CT(26378)=(S=6809428,E=6809431,C=8)
	CT(26379)=(S=6809432,E=6809435,C=0)
	CT(26380)=(S=6809436,E=6809439,C=155)
	CT(26381)=(S=6809440,E=6809441,C=7)
	CT(26382)=(S=6809442,E=6809443,C=2)
	CT(26383)=(S=6809444,E=6809447,C=4)
	CT(26384)=(S=6809448,E=6809451,C=8)
	CT(26385)=(S=6809452,E=6809459,C=74)
	CT(26386)=(S=6809460,E=6809463,C=4)
	CT(26387)=(S=6809464,E=6809467,C=74)
	CT(26388)=(S=6809468,E=6809471,C=4)
	CT(26389)=(S=6809472,E=6809475,C=6)
	CT(26390)=(S=6809476,E=6809477,C=4)
	CT(26391)=(S=6809478,E=6809478,C=10)
	CT(26392)=(S=6809479,E=6809479,C=84)
	CT(26393)=(S=6809480,E=6809483,C=89)
	CT(26394)=(S=6809484,E=6809487,C=8)
	CT(26395)=(S=6809488,E=6809491,C=1)
	CT(26396)=(S=6809492,E=6809495,C=11)
	CT(26397)=(S=6809496,E=6809499,C=8)
	CT(26398)=(S=6809500,E=6809501,C=73)
	CT(26399)=(S=6809502,E=6809503,C=0)
	CT(26400)=(S=6809504,E=6809507,C=84)
	CT(26401)=(S=6809508,E=6809511,C=8)
	CT(26402)=(S=6809512,E=6809513,C=0)
	CT(26403)=(S=6809514,E=6809515,C=74)
	CT(26404)=(S=6809516,E=6809519,C=0)
	CT(26405)=(S=6809520,E=6809523,C=84)
	CT(26406)=(S=6809524,E=6809531,C=1)
	CT(26407)=(S=6809532,E=6809535,C=11)
	CT(26408)=(S=6809536,E=6809539,C=2)
	CT(26409)=(S=6809540,E=6809547,C=74)
	CT(26410)=(S=6809548,E=6809551,C=0)
	CT(26411)=(S=6809552,E=6809563,C=4)
	CT(26412)=(S=6809564,E=6809565,C=7)
	CT(26413)=(S=6809566,E=6809567,C=4)
	CT(26414)=(S=6809568,E=6809571,C=5)
	CT(26415)=(S=6809572,E=6809575,C=84)
	CT(26416)=(S=6809576,E=6809579,C=4)
	CT(26417)=(S=6809580,E=6809583,C=2)
	CT(26418)=(S=6809584,E=6809587,C=10)
	CT(26419)=(S=6809588,E=6809591,C=1)
	CT(26420)=(S=6809592,E=6809594,C=7)
	CT(26421)=(S=6809595,E=6809595,C=5)
	CT(26422)=(S=6809596,E=6809603,C=8)
	CT(26423)=(S=6809604,E=6809607,C=1)
	CT(26424)=(S=6809608,E=6809611,C=4)
	CT(26425)=(S=6809612,E=6809615,C=2)
	CT(26426)=(S=6809616,E=6809619,C=8)
	CT(26427)=(S=6809620,E=6809627,C=4)
	CT(26428)=(S=6809628,E=6809629,C=5)
	CT(26429)=(S=6809630,E=6809630,C=74)
	CT(26430)=(S=6809631,E=6809631,C=0)
	CT(26431)=(S=6809632,E=6809633,C=74)
	CT(26432)=(S=6809634,E=6809639,C=8)
	CT(26433)=(S=6809640,E=6809643,C=3)
	CT(26434)=(S=6809644,E=6809647,C=7)
	CT(26435)=(S=6809648,E=6809651,C=5)
	CT(26436)=(S=6809652,E=6809663,C=11)
	CT(26437)=(S=6809664,E=6809667,C=74)
	CT(26438)=(S=6809668,E=6809671,C=8)
	CT(26439)=(S=6809672,E=6809675,C=2)
	CT(26440)=(S=6809680,E=6809683,C=154)
	CT(26441)=(S=6809684,E=6809691,C=8)
	CT(26442)=(S=6809692,E=6809699,C=0)
	CT(26443)=(S=6809700,E=6809703,C=84)
	CT(26444)=(S=6809708,E=6809711,C=73)
	CT(26445)=(S=6809712,E=6809715,C=4)
	CT(26446)=(S=6809716,E=6809719,C=0)
	CT(26447)=(S=6809720,E=6809723,C=11)
	CT(26448)=(S=6809724,E=6809731,C=4)
	CT(26449)=(S=6809732,E=6809735,C=0)
	CT(26450)=(S=6809736,E=6809739,C=4)
	CT(26451)=(S=6809740,E=6809741,C=0)
	CT(26452)=(S=6809744,E=6809747,C=1)
	CT(26453)=(S=6809748,E=6809751,C=4)
	CT(26454)=(S=6809752,E=6809755,C=75)
	CT(26455)=(S=6809756,E=6809758,C=73)
	CT(26456)=(S=6809759,E=6809771,C=0)
	CT(26457)=(S=6809772,E=6809779,C=4)
	CT(26458)=(S=6809780,E=6809783,C=73)
	CT(26459)=(S=6809784,E=6809787,C=4)
	CT(26460)=(S=6809792,E=6809795,C=86)
	CT(26461)=(S=6809796,E=6809799,C=0)
	CT(26462)=(S=6809800,E=6809803,C=2)
	CT(26463)=(S=6809804,E=6809805,C=73)
	CT(26464)=(S=6809806,E=6809811,C=0)
	CT(26465)=(S=6809812,E=6809815,C=1)
	CT(26466)=(S=6809816,E=6809818,C=0)
	CT(26467)=(S=6809819,E=6809819,C=5)
	CT(26468)=(S=6809820,E=6809823,C=8)
	CT(26469)=(S=6809824,E=6809824,C=4)
	CT(26470)=(S=6809825,E=6809825,C=77)
	CT(26471)=(S=6809826,E=6809827,C=73)
	CT(26472)=(S=6809828,E=6809831,C=75)
	CT(26473)=(S=6809832,E=6809839,C=4)
	CT(26474)=(S=6809840,E=6809840,C=74)
	CT(26475)=(S=6809841,E=6809841,C=4)
	CT(26476)=(S=6809842,E=6809843,C=74)
	CT(26477)=(S=6809844,E=6809847,C=4)
	CT(26478)=(S=6809852,E=6809855,C=0)
	CT(26479)=(S=6809856,E=6809859,C=5)
	CT(26480)=(S=6809860,E=6809863,C=1)
	CT(26481)=(S=6809868,E=6809871,C=7)
	CT(26482)=(S=6809872,E=6809875,C=0)
	CT(26483)=(S=6809876,E=6809879,C=73)
	CT(26484)=(S=6809880,E=6809883,C=4)
	CT(26485)=(S=6809884,E=6809887,C=0)
	CT(26486)=(S=6809888,E=6809895,C=10)
	CT(26487)=(S=6809900,E=6809903,C=1)
	CT(26488)=(S=6809904,E=6809907,C=11)
	CT(26489)=(S=6809908,E=6809911,C=1)
	CT(26490)=(S=6809912,E=6809915,C=75)
	CT(26491)=(S=6809916,E=6809919,C=73)
	CT(26492)=(S=6809920,E=6809923,C=4)
	CT(26493)=(S=6809924,E=6809927,C=9)
	CT(26494)=(S=6809928,E=6809928,C=8)
	CT(26495)=(S=6809929,E=6809929,C=4)
	CT(26496)=(S=6809930,E=6809931,C=0)
	CT(26497)=(S=6809932,E=6809935,C=4)
	CT(26498)=(S=6809936,E=6809939,C=3)
	CT(26499)=(S=6809940,E=6809943,C=4)
	CT(26500)=(S=6809944,E=6809947,C=74)
	CT(26501)=(S=6809948,E=6809951,C=4)
	CT(26502)=(S=6809952,E=6809955,C=8)
	CT(26503)=(S=6809956,E=6809956,C=74)
	CT(26504)=(S=6809957,E=6809957,C=4)
	CT(26505)=(S=6809958,E=6809959,C=74)
	CT(26506)=(S=6809960,E=6809963,C=1)
	CT(26507)=(S=6809964,E=6809967,C=74)
	CT(26508)=(S=6809968,E=6809976,C=4)
	CT(26509)=(S=6809977,E=6809977,C=84)
	CT(26510)=(S=6809978,E=6809983,C=4)
	CT(26511)=(S=6809984,E=6809987,C=1)
	CT(26512)=(S=6809988,E=6809991,C=73)
	CT(26513)=(S=6809992,E=6809995,C=1)
	CT(26514)=(S=6809998,E=6810003,C=74)
	CT(26515)=(S=6810004,E=6810007,C=0)
	CT(26516)=(S=6810008,E=6810015,C=74)
	CT(26517)=(S=6810016,E=6810019,C=4)
	CT(26518)=(S=6810020,E=6810023,C=8)
	CT(26519)=(S=6810024,E=6810027,C=4)
	CT(26520)=(S=6810028,E=6810031,C=7)
	CT(26521)=(S=6810032,E=6810035,C=3)
	CT(26522)=(S=6810036,E=6810037,C=5)
	CT(26523)=(S=6810038,E=6810038,C=75)
	CT(26524)=(S=6810039,E=6810039,C=89)
	CT(26525)=(S=6810040,E=6810043,C=8)
	CT(26526)=(S=6810044,E=6810047,C=87)
	CT(26527)=(S=6810048,E=6810051,C=3)
	CT(26528)=(S=6810052,E=6810055,C=2)
	CT(26529)=(S=6810060,E=6810063,C=155)
	CT(26530)=(S=6810064,E=6810067,C=0)
	CT(26531)=(S=6810072,E=6810075,C=4)
	CT(26532)=(S=6810076,E=6810079,C=73)
	CT(26533)=(S=6810080,E=6810083,C=0)
	CT(26534)=(S=6810084,E=6810087,C=1)
	CT(26535)=(S=6810088,E=6810091,C=8)
	CT(26536)=(S=6810092,E=6810095,C=0)
	CT(26537)=(S=6810096,E=6810097,C=7)
	CT(26538)=(S=6810098,E=6810098,C=0)
	CT(26539)=(S=6810099,E=6810103,C=4)
	CT(26540)=(S=6810104,E=6810107,C=8)
	CT(26541)=(S=6810112,E=6810115,C=1)
	CT(26542)=(S=6810116,E=6810119,C=6)
	CT(26543)=(S=6810124,E=6810124,C=77)
	CT(26544)=(S=6810125,E=6810127,C=0)
	CT(26545)=(S=6810128,E=6810131,C=8)
	CT(26546)=(S=6810132,E=6810135,C=1)
	CT(26547)=(S=6810136,E=6810139,C=84)
	CT(26548)=(S=6810140,E=6810143,C=2)
	CT(26549)=(S=6810144,E=6810147,C=8)
	CT(26550)=(S=6810148,E=6810151,C=11)
	CT(26551)=(S=6810152,E=6810155,C=9)
	CT(26552)=(S=6810160,E=6810163,C=76)
	CT(26553)=(S=6810164,E=6810167,C=8)
	CT(26554)=(S=6810168,E=6810171,C=1)
	CT(26555)=(S=6810172,E=6810175,C=0)
	CT(26556)=(S=6810176,E=6810183,C=4)
	CT(26557)=(S=6810188,E=6810195,C=9)
	CT(26558)=(S=6810196,E=6810199,C=7)
	CT(26559)=(S=6810200,E=6810203,C=11)
	CT(26560)=(S=6810204,E=6810204,C=0)
	CT(26561)=(S=6810205,E=6810206,C=4)
	CT(26562)=(S=6810207,E=6810207,C=0)
	CT(26563)=(S=6810208,E=6810211,C=8)
	CT(26564)=(S=6810212,E=6810215,C=5)
	CT(26565)=(S=6810216,E=6810219,C=7)
	CT(26566)=(S=6810220,E=6810225,C=0)
	CT(26567)=(S=6810226,E=6810229,C=4)
	CT(26568)=(S=6810230,E=6810231,C=0)
	CT(26569)=(S=6810232,E=6810235,C=74)
	CT(26570)=(S=6810236,E=6810243,C=1)
	CT(26571)=(S=6810244,E=6810247,C=73)
	CT(26572)=(S=6810248,E=6810251,C=4)
	CT(26573)=(S=6810252,E=6810255,C=8)
	CT(26574)=(S=6810256,E=6810259,C=77)
	CT(26575)=(S=6810260,E=6810260,C=7)
	CT(26576)=(S=6810261,E=6810274,C=4)
	CT(26577)=(S=6810275,E=6810275,C=0)
	CT(26578)=(S=6810280,E=6810283,C=7)
	CT(26579)=(S=6810284,E=6810287,C=1)
	CT(26580)=(S=6810288,E=6810291,C=2)
	CT(26581)=(S=6810292,E=6810295,C=1)
	CT(26582)=(S=6810296,E=6810303,C=4)
	CT(26583)=(S=6810304,E=6810305,C=84)
	CT(26584)=(S=6810306,E=6810306,C=0)
	CT(26585)=(S=6810307,E=6810307,C=74)
	CT(26586)=(S=6810308,E=6810311,C=2)
	CT(26587)=(S=6810312,E=6810315,C=84)
	CT(26588)=(S=6810316,E=6810319,C=9)
	CT(26589)=(S=6810328,E=6810331,C=74)
	CT(26590)=(S=6810332,E=6810335,C=8)
	CT(26591)=(S=6810336,E=6810343,C=9)
	CT(26592)=(S=6810344,E=6810347,C=0)
	CT(26593)=(S=6810348,E=6810355,C=4)
	CT(26594)=(S=6810356,E=6810359,C=1)
	CT(26595)=(S=6810360,E=6810363,C=0)
	CT(26596)=(S=6810364,E=6810367,C=74)
	CT(26597)=(S=6810368,E=6810371,C=4)
	CT(26598)=(S=6810372,E=6810375,C=8)
	CT(26599)=(S=6810376,E=6810379,C=4)
	CT(26600)=(S=6810380,E=6810383,C=2)
	CT(26601)=(S=6810384,E=6810387,C=1)
	CT(26602)=(S=6810388,E=6810388,C=0)
	CT(26603)=(S=6810389,E=6810389,C=74)
	CT(26604)=(S=6810390,E=6810390,C=0)
	CT(26605)=(S=6810391,E=6810391,C=4)
	CT(26606)=(S=6810392,E=6810395,C=6)
	CT(26607)=(S=6810396,E=6810399,C=5)
	CT(26608)=(S=6810400,E=6810403,C=74)
	CT(26609)=(S=6810404,E=6810407,C=0)
	CT(26610)=(S=6810408,E=6810411,C=2)
	CT(26611)=(S=6810412,E=6810415,C=8)
	CT(26612)=(S=6810416,E=6810419,C=1)
	CT(26613)=(S=6810420,E=6810423,C=0)
	CT(26614)=(S=6810424,E=6810431,C=1)
	CT(26615)=(S=6810432,E=6810432,C=4)
	CT(26616)=(S=6810433,E=6810433,C=0)
	CT(26617)=(S=6810434,E=6810435,C=74)
	CT(26618)=(S=6810436,E=6810439,C=4)
	CT(26619)=(S=6810448,E=6810455,C=1)
	CT(26620)=(S=6810456,E=6810459,C=9)
	CT(26621)=(S=6810460,E=6810467,C=0)
	CT(26622)=(S=6810468,E=6810471,C=8)
	CT(26623)=(S=6810472,E=6810475,C=4)
	CT(26624)=(S=6810476,E=6810476,C=0)
	CT(26625)=(S=6810477,E=6810477,C=74)
	CT(26626)=(S=6810478,E=6810479,C=93)
	CT(26627)=(S=6810480,E=6810483,C=2)
	CT(26628)=(S=6810484,E=6810487,C=0)
	CT(26629)=(S=6810488,E=6810491,C=4)
	CT(26630)=(S=6810492,E=6810495,C=2)
	CT(26631)=(S=6810496,E=6810519,C=1)
	CT(26632)=(S=6810520,E=6810521,C=74)
	CT(26633)=(S=6810522,E=6810522,C=0)
	CT(26634)=(S=6810523,E=6810527,C=4)
	CT(26635)=(S=6810528,E=6810531,C=2)
	CT(26636)=(S=6810536,E=6810543,C=8)
	CT(26637)=(S=6810544,E=6810547,C=89)
	CT(26638)=(S=6810548,E=6810551,C=2)
	CT(26639)=(S=6810552,E=6810555,C=1)
	CT(26640)=(S=6810556,E=6810559,C=0)
	CT(26641)=(S=6810560,E=6810563,C=1)
	CT(26642)=(S=6810564,E=6810567,C=75)
	CT(26643)=(S=6810568,E=6810571,C=1)
	CT(26644)=(S=6810572,E=6810573,C=7)
	CT(26645)=(S=6810574,E=6810575,C=4)
	CT(26646)=(S=6810576,E=6810583,C=11)
	CT(26647)=(S=6810588,E=6810723,C=1)
	CT(26648)=(S=6810724,E=6810727,C=8)
	CT(26649)=(S=6810728,E=6810741,C=4)
	CT(26650)=(S=6810742,E=6810742,C=0)
	CT(26651)=(S=6810743,E=6810743,C=7)
	CT(26652)=(S=6810744,E=6810747,C=1)
	CT(26653)=(S=6810748,E=6810751,C=4)
	CT(26654)=(S=6810752,E=6810755,C=2)
	CT(26655)=(S=6810756,E=6810759,C=77)
	CT(26656)=(S=6810760,E=6810763,C=8)
	CT(26657)=(S=6810764,E=6810771,C=2)
	CT(26658)=(S=6810772,E=6810775,C=74)
	CT(26659)=(S=6810776,E=6810783,C=4)
	CT(26660)=(S=6810784,E=6810785,C=74)
	CT(26661)=(S=6810786,E=6810787,C=0)
	CT(26662)=(S=6810788,E=6810799,C=4)
	CT(26663)=(S=6810800,E=6810803,C=10)
	CT(26664)=(S=6810804,E=6810807,C=2)
	CT(26665)=(S=6810808,E=6810811,C=1)
	CT(26666)=(S=6810812,E=6810816,C=74)
	CT(26667)=(S=6810817,E=6810819,C=4)
	CT(26668)=(S=6810820,E=6810823,C=7)
	CT(26669)=(S=6810824,E=6810824,C=4)
	CT(26670)=(S=6810825,E=6810825,C=74)
	CT(26671)=(S=6810826,E=6810827,C=0)
	CT(26672)=(S=6810828,E=6810835,C=4)
	CT(26673)=(S=6810836,E=6810839,C=0)
	CT(26674)=(S=6810840,E=6810843,C=8)
	CT(26675)=(S=6810844,E=6810847,C=1)
	CT(26676)=(S=6810848,E=6810851,C=2)
	CT(26677)=(S=6810852,E=6810855,C=5)
	CT(26678)=(S=6810856,E=6810859,C=1)
	CT(26679)=(S=6810860,E=6810863,C=4)
	CT(26680)=(S=6810864,E=6810895,C=1)
	CT(26681)=(S=6810896,E=6810899,C=0)
	CT(26682)=(S=6810900,E=6810903,C=6)
	CT(26683)=(S=6810904,E=6810911,C=1)
	CT(26684)=(S=6810912,E=6810915,C=74)
	CT(26685)=(S=6810916,E=6810919,C=84)
	CT(26686)=(S=6810920,E=6810923,C=73)
	CT(26687)=(S=6810924,E=6810931,C=4)
	CT(26688)=(S=6810932,E=6810933,C=86)
	CT(26689)=(S=6810934,E=6810935,C=8)
	CT(26690)=(S=6810936,E=6810939,C=4)
	CT(26691)=(S=6810940,E=6810947,C=11)
	CT(26692)=(S=6810948,E=6810951,C=1)
	CT(26693)=(S=6810952,E=6810955,C=8)
	CT(26694)=(S=6810960,E=6810960,C=7)
	CT(26695)=(S=6810961,E=6810962,C=0)
	CT(26696)=(S=6810963,E=6810963,C=89)
	CT(26697)=(S=6810964,E=6810967,C=77)
	CT(26698)=(S=6810968,E=6810971,C=1)
	CT(26699)=(S=6810972,E=6810975,C=0)
	CT(26700)=(S=6810976,E=6810979,C=11)
	CT(26701)=(S=6810980,E=6810983,C=87)
	CT(26702)=(S=6810984,E=6810991,C=0)
	CT(26703)=(S=6810992,E=6810999,C=4)
	CT(26704)=(S=6811000,E=6811003,C=2)
	CT(26705)=(S=6811004,E=6811007,C=87)
	CT(26706)=(S=6811008,E=6811011,C=8)
	CT(26707)=(S=6811012,E=6811013,C=7)
	CT(26708)=(S=6811014,E=6811014,C=74)
	CT(26709)=(S=6811015,E=6811015,C=4)
	CT(26710)=(S=6811016,E=6811019,C=2)
	CT(26711)=(S=6811020,E=6811021,C=74)
	CT(26712)=(S=6811022,E=6811023,C=7)
	CT(26713)=(S=6811024,E=6811031,C=11)
	CT(26714)=(S=6811032,E=6811035,C=1)
	CT(26715)=(S=6811036,E=6811039,C=4)
	CT(26716)=(S=6811040,E=6811043,C=0)
	CT(26717)=(S=6811044,E=6811047,C=73)
	CT(26718)=(S=6811048,E=6811049,C=7)
	CT(26719)=(S=6811050,E=6811050,C=4)
	CT(26720)=(S=6811051,E=6811051,C=0)
	CT(26721)=(S=6811052,E=6811055,C=4)
	CT(26722)=(S=6811056,E=6811143,C=1)
	CT(26723)=(S=6811144,E=6811151,C=4)
	CT(26724)=(S=6811152,E=6811195,C=1)
	CT(26725)=(S=6811196,E=6811196,C=73)
	CT(26726)=(S=6811197,E=6811197,C=4)
	CT(26727)=(S=6811198,E=6811198,C=0)
	CT(26728)=(S=6811199,E=6811199,C=84)
	CT(26729)=(S=6811200,E=6811203,C=86)
	CT(26730)=(S=6811204,E=6811219,C=11)
	CT(26731)=(S=6811220,E=6811223,C=73)
	CT(26732)=(S=6811224,E=6811235,C=1)
	CT(26733)=(S=6811236,E=6811247,C=4)
	CT(26734)=(S=6811248,E=6811251,C=2)
	CT(26735)=(S=6811252,E=6811255,C=165)
	CT(26736)=(S=6811256,E=6811259,C=0)
	CT(26737)=(S=6811260,E=6811263,C=5)
	CT(26738)=(S=6811264,E=6811267,C=8)
	CT(26739)=(S=6811268,E=6811271,C=1)
	CT(26740)=(S=6811272,E=6811275,C=74)
	CT(26741)=(S=6811276,E=6811283,C=1)
	CT(26742)=(S=6811284,E=6811287,C=5)
	CT(26743)=(S=6811288,E=6811291,C=4)
	CT(26744)=(S=6811292,E=6811293,C=196)
	CT(26745)=(S=6811294,E=6811295,C=4)
	CT(26746)=(S=6811296,E=6811327,C=1)
	CT(26747)=(S=6811328,E=6811329,C=4)
	CT(26748)=(S=6811330,E=6811330,C=73)
	CT(26749)=(S=6811331,E=6811331,C=0)
	CT(26750)=(S=6811332,E=6811335,C=1)
	CT(26751)=(S=6811336,E=6811339,C=74)
	CT(26752)=(S=6811340,E=6811343,C=1)
	CT(26753)=(S=6811344,E=6811351,C=11)
	CT(26754)=(S=6811352,E=6811359,C=4)
	CT(26755)=(S=6811360,E=6811363,C=8)
	CT(26756)=(S=6811364,E=6811371,C=4)
	CT(26757)=(S=6811372,E=6811375,C=2)
	CT(26758)=(S=6811376,E=6811379,C=4)
	CT(26759)=(S=6811380,E=6811383,C=2)
	CT(26760)=(S=6811384,E=6811387,C=6)
	CT(26761)=(S=6811388,E=6811395,C=1)
	CT(26762)=(S=6811396,E=6811399,C=84)
	CT(26763)=(S=6811400,E=6811403,C=73)
	CT(26764)=(S=6811404,E=6811407,C=8)
	CT(26765)=(S=6811408,E=6811411,C=2)
	CT(26766)=(S=6811412,E=6811415,C=7)
	CT(26767)=(S=6811416,E=6811419,C=0)
	CT(26768)=(S=6811420,E=6811423,C=8)
	CT(26769)=(S=6811424,E=6811427,C=11)
	CT(26770)=(S=6811428,E=6811431,C=4)
	CT(26771)=(S=6811432,E=6811439,C=1)
	CT(26772)=(S=6811440,E=6811441,C=0)
	CT(26773)=(S=6811442,E=6811442,C=4)
	CT(26774)=(S=6811443,E=6811443,C=0)
	CT(26775)=(S=6811444,E=6811447,C=86)
	CT(26776)=(S=6811448,E=6811451,C=4)
	CT(26777)=(S=6811452,E=6811455,C=9)
	CT(26778)=(S=6811456,E=6811459,C=8)
	CT(26779)=(S=6811460,E=6811463,C=1)
	CT(26780)=(S=6811464,E=6811467,C=8)
	CT(26781)=(S=6811468,E=6811469,C=0)
	CT(26782)=(S=6811470,E=6811470,C=4)
	CT(26783)=(S=6811471,E=6811471,C=0)
	CT(26784)=(S=6811472,E=6811479,C=4)
	CT(26785)=(S=6811480,E=6811487,C=2)
	CT(26786)=(S=6811488,E=6811491,C=1)
	CT(26787)=(S=6811492,E=6811495,C=8)
	CT(26788)=(S=6811496,E=6811497,C=73)
	CT(26789)=(S=6811498,E=6811499,C=0)
	CT(26790)=(S=6811500,E=6811503,C=2)
	CT(26791)=(S=6811504,E=6811507,C=6)
	CT(26792)=(S=6811508,E=6811515,C=11)
	CT(26793)=(S=6811516,E=6811517,C=4)
	CT(26794)=(S=6811518,E=6811518,C=8)
	CT(26795)=(S=6811519,E=6811519,C=5)
	CT(26796)=(S=6811520,E=6811527,C=0)
	CT(26797)=(S=6811528,E=6811543,C=4)
	CT(26798)=(S=6811544,E=6811551,C=1)
	CT(26799)=(S=6811552,E=6811555,C=166)
	CT(26800)=(S=6811556,E=6811556,C=8)
	CT(26801)=(S=6811557,E=6811557,C=74)
	CT(26802)=(S=6811558,E=6811559,C=0)
	CT(26803)=(S=6811560,E=6811567,C=4)
	CT(26804)=(S=6811568,E=6811579,C=1)
	CT(26805)=(S=6811580,E=6811583,C=74)
	CT(26806)=(S=6811584,E=6811591,C=1)
	CT(26807)=(S=6811592,E=6811595,C=10)
	CT(26808)=(S=6811596,E=6811603,C=1)
	CT(26809)=(S=6811604,E=6811604,C=0)
	CT(26810)=(S=6811606,E=6811607,C=73)
	CT(26811)=(S=6811608,E=6811611,C=2)
	CT(26812)=(S=6811612,E=6811615,C=7)
	CT(26813)=(S=6811616,E=6811619,C=1)
	CT(26814)=(S=6811620,E=6811623,C=73)
	CT(26815)=(S=6811624,E=6811627,C=4)
	CT(26816)=(S=6811628,E=6811635,C=6)
	CT(26817)=(S=6811636,E=6811639,C=1)
	CT(26818)=(S=6811640,E=6811643,C=5)
	CT(26819)=(S=6811644,E=6811647,C=84)
	CT(26820)=(S=6811648,E=6811655,C=2)
	CT(26821)=(S=6811656,E=6811659,C=4)
	CT(26822)=(S=6811660,E=6811663,C=2)
	CT(26823)=(S=6811664,E=6811667,C=1)
	CT(26824)=(S=6811668,E=6811675,C=2)
	CT(26825)=(S=6811676,E=6811679,C=6)
	CT(26826)=(S=6811680,E=6811683,C=4)
	CT(26827)=(S=6811684,E=6811687,C=1)
	CT(26828)=(S=6811688,E=6811691,C=8)
	CT(26829)=(S=6811692,E=6811695,C=84)
	CT(26830)=(S=6811696,E=6811699,C=6)
	CT(26831)=(S=6811700,E=6811703,C=73)
	CT(26832)=(S=6811704,E=6811707,C=2)
	CT(26833)=(S=6811708,E=6811711,C=3)
	CT(26834)=(S=6811712,E=6811719,C=8)
	CT(26835)=(S=6811720,E=6811723,C=1)
	CT(26836)=(S=6811724,E=6811727,C=4)
	CT(26837)=(S=6811728,E=6811731,C=8)
	CT(26838)=(S=6811732,E=6811735,C=1)
	CT(26839)=(S=6811736,E=6811736,C=0)
	CT(26840)=(S=6811737,E=6811739,C=4)
	CT(26841)=(S=6811740,E=6811743,C=8)
	CT(26842)=(S=6811744,E=6811755,C=4)
	CT(26843)=(S=6811756,E=6811759,C=74)
	CT(26844)=(S=6811760,E=6811763,C=86)
	CT(26845)=(S=6811764,E=6811767,C=2)
	CT(26846)=(S=6811772,E=6811775,C=1)
	CT(26847)=(S=6811776,E=6811779,C=7)
	CT(26848)=(S=6811780,E=6811780,C=0)
	CT(26849)=(S=6811781,E=6811781,C=73)
	CT(26850)=(S=6811782,E=6811783,C=0)
	CT(26851)=(S=6811784,E=6811787,C=4)
	CT(26852)=(S=6811788,E=6811791,C=8)
	CT(26853)=(S=6811792,E=6811795,C=4)
	CT(26854)=(S=6811796,E=6811799,C=8)
	CT(26855)=(S=6811800,E=6811803,C=73)
	CT(26856)=(S=6811804,E=6811807,C=1)
	CT(26857)=(S=6811808,E=6811819,C=4)
	CT(26858)=(S=6811820,E=6811823,C=1)
	CT(26859)=(S=6811824,E=6811827,C=5)
	CT(26860)=(S=6811828,E=6811831,C=8)
	CT(26861)=(S=6811832,E=6811835,C=73)
	CT(26862)=(S=6811840,E=6811843,C=4)
	CT(26863)=(S=6811844,E=6811851,C=8)
	CT(26864)=(S=6811852,E=6811867,C=4)
	CT(26865)=(S=6811868,E=6811868,C=77)
	CT(26866)=(S=6811869,E=6811869,C=8)
	CT(26867)=(S=6811870,E=6811879,C=0)
	CT(26868)=(S=6811880,E=6811887,C=4)
	CT(26869)=(S=6811888,E=6811891,C=154)
	CT(26870)=(S=6811892,E=6811895,C=1)
	CT(26871)=(S=6811896,E=6811899,C=84)
	CT(26872)=(S=6811900,E=6811903,C=2)
	CT(26873)=(S=6811904,E=6811907,C=0)
	CT(26874)=(S=6811908,E=6811909,C=74)
	CT(26875)=(S=6811910,E=6811911,C=0)
	CT(26876)=(S=6811912,E=6811915,C=4)
	CT(26877)=(S=6811916,E=6811919,C=1)
	CT(26878)=(S=6811920,E=6811923,C=8)
	CT(26879)=(S=6811924,E=6811927,C=4)
	CT(26880)=(S=6811928,E=6811928,C=74)
	CT(26881)=(S=6811929,E=6811929,C=4)
	CT(26882)=(S=6811930,E=6811930,C=0)
	CT(26883)=(S=6811931,E=6811931,C=156)
	CT(26884)=(S=6811932,E=6811939,C=3)
	CT(26885)=(S=6811940,E=6811943,C=73)
	CT(26886)=(S=6811944,E=6811951,C=4)
	CT(26887)=(S=6811952,E=6811955,C=8)
	CT(26888)=(S=6811956,E=6811959,C=4)
	CT(26889)=(S=6811960,E=6811963,C=73)
	CT(26890)=(S=6811964,E=6811967,C=7)
	CT(26891)=(S=6811968,E=6811971,C=2)
	CT(26892)=(S=6811972,E=6811975,C=9)
	CT(26893)=(S=6811976,E=6811979,C=1)
	CT(26894)=(S=6811980,E=6811983,C=6)
	CT(26895)=(S=6811988,E=6811995,C=73)
	CT(26896)=(S=6811996,E=6812003,C=1)
	CT(26897)=(S=6812004,E=6812007,C=0)
	CT(26898)=(S=6812008,E=6812008,C=2)
	CT(26899)=(S=6812009,E=6812011,C=7)
	CT(26900)=(S=6812012,E=6812015,C=8)
	CT(26901)=(S=6812016,E=6812019,C=4)
	CT(26902)=(S=6812020,E=6812023,C=6)
	CT(26903)=(S=6812024,E=6812027,C=73)
	CT(26904)=(S=6812028,E=6812035,C=2)
	CT(26905)=(S=6812036,E=6812051,C=4)
	CT(26906)=(S=6812052,E=6812053,C=0)
	CT(26907)=(S=6812054,E=6812055,C=5)
	CT(26908)=(S=6812056,E=6812059,C=2)
	CT(26909)=(S=6812060,E=6812063,C=89)
	CT(26910)=(S=6812064,E=6812067,C=1)
	CT(26911)=(S=6812068,E=6812071,C=8)
	CT(26912)=(S=6812072,E=6812075,C=4)
	CT(26913)=(S=6812076,E=6812079,C=2)
	CT(26914)=(S=6812080,E=6812081,C=0)
	CT(26915)=(S=6812082,E=6812083,C=74)
	CT(26916)=(S=6812084,E=6812087,C=4)
	CT(26917)=(S=6812088,E=6812095,C=1)
	CT(26918)=(S=6812096,E=6812099,C=2)
	CT(26919)=(S=6812100,E=6812107,C=0)
	CT(26920)=(S=6812108,E=6812111,C=74)
	CT(26921)=(S=6812112,E=6812115,C=2)
	CT(26922)=(S=6812120,E=6812123,C=0)
	CT(26923)=(S=6812124,E=6812127,C=1)
	CT(26924)=(S=6812128,E=6812143,C=4)
	CT(26925)=(S=6812144,E=6812145,C=0)
	CT(26926)=(S=6812146,E=6812151,C=4)
	CT(26927)=(S=6812152,E=6812155,C=11)
	CT(26928)=(S=6812156,E=6812159,C=0)
	CT(26929)=(S=6812160,E=6812163,C=8)
	CT(26930)=(S=6812164,E=6812167,C=2)
	CT(26931)=(S=6812168,E=6812171,C=1)
	CT(26932)=(S=6812172,E=6812175,C=86)
	CT(26933)=(S=6812176,E=6812179,C=5)
	CT(26934)=(S=6812180,E=6812183,C=84)
	CT(26935)=(S=6812184,E=6812187,C=0)
	CT(26936)=(S=6812188,E=6812191,C=2)
	CT(26937)=(S=6812192,E=6812195,C=0)
	CT(26938)=(S=6812196,E=6812199,C=73)
	CT(26939)=(S=6812200,E=6812203,C=86)
	CT(26940)=(S=6812204,E=6812207,C=92)
	CT(26941)=(S=6812208,E=6812208,C=73)
	CT(26942)=(S=6812209,E=6812209,C=0)
	CT(26943)=(S=6812210,E=6812211,C=89)
	CT(26944)=(S=6812212,E=6812215,C=11)
	CT(26945)=(S=6812216,E=6812219,C=86)
	CT(26946)=(S=6812220,E=6812223,C=4)
	CT(26947)=(S=6812224,E=6812227,C=1)
	CT(26948)=(S=6812228,E=6812231,C=73)
	CT(26949)=(S=6812232,E=6812235,C=8)
	CT(26950)=(S=6812236,E=6812239,C=4)
	CT(26951)=(S=6812240,E=6812243,C=0)
	CT(26952)=(S=6812248,E=6812251,C=7)
	CT(26953)=(S=6812252,E=6812255,C=0)
	CT(26954)=(S=6812256,E=6812259,C=155)
	CT(26955)=(S=6812260,E=6812263,C=8)
	CT(26956)=(S=6812264,E=6812267,C=74)
	CT(26957)=(S=6812268,E=6812271,C=8)
	CT(26958)=(S=6812272,E=6812275,C=2)
	CT(26959)=(S=6812276,E=6812283,C=4)
	CT(26960)=(S=6812284,E=6812285,C=74)
	CT(26961)=(S=6812286,E=6812287,C=166)
	CT(26962)=(S=6812288,E=6812295,C=1)
	CT(26963)=(S=6812300,E=6812303,C=8)
	CT(26964)=(S=6812304,E=6812307,C=3)
	CT(26965)=(S=6812308,E=6812309,C=8)
	CT(26966)=(S=6812310,E=6812311,C=7)
	CT(26967)=(S=6812312,E=6812319,C=4)
	CT(26968)=(S=6812320,E=6812323,C=1)
	CT(26969)=(S=6812324,E=6812327,C=76)
	CT(26970)=(S=6812328,E=6812339,C=1)
	CT(26971)=(S=6812340,E=6812341,C=74)
	CT(26972)=(S=6812342,E=6812343,C=0)
	CT(26973)=(S=6812344,E=6812359,C=4)
	CT(26974)=(S=6812360,E=6812363,C=1)
	CT(26975)=(S=6812364,E=6812369,C=74)
	CT(26976)=(S=6812370,E=6812371,C=0)
	CT(26977)=(S=6812372,E=6812375,C=1)
	CT(26978)=(S=6812376,E=6812379,C=84)
	CT(26979)=(S=6812380,E=6812383,C=1)
	CT(26980)=(S=6812384,E=6812387,C=4)
	CT(26981)=(S=6812388,E=6812391,C=73)
	CT(26982)=(S=6812392,E=6812392,C=4)
	CT(26983)=(S=6812393,E=6812393,C=74)
	CT(26984)=(S=6812394,E=6812394,C=4)
	CT(26985)=(S=6812395,E=6812395,C=0)
	CT(26986)=(S=6812396,E=6812399,C=4)
	CT(26987)=(S=6812400,E=6812403,C=1)
	CT(26988)=(S=6812404,E=6812407,C=73)
	CT(26989)=(S=6812408,E=6812411,C=0)
	CT(26990)=(S=6812412,E=6812415,C=2)
	CT(26991)=(S=6812416,E=6812419,C=8)
	CT(26992)=(S=6812424,E=6812427,C=4)
	CT(26993)=(S=6812428,E=6812431,C=7)
	CT(26994)=(S=6812432,E=6812433,C=0)
	CT(26995)=(S=6812434,E=6812434,C=8)
	CT(26996)=(S=6812435,E=6812435,C=4)
	CT(26997)=(S=6812436,E=6812439,C=6)
	CT(26998)=(S=6812440,E=6812443,C=1)
	CT(26999)=(S=6812444,E=6812479,C=4)
	CT(27000)=(S=6812480,E=6812483,C=0)
	CT(27001)=(S=6812484,E=6812491,C=4)
	CT(27002)=(S=6812492,E=6812499,C=84)
	CT(27003)=(S=6812500,E=6812503,C=0)
	CT(27004)=(S=6812504,E=6812505,C=4)
	CT(27005)=(S=6812506,E=6812506,C=73)
	CT(27006)=(S=6812507,E=6812507,C=0)
	CT(27007)=(S=6812508,E=6812511,C=8)
	CT(27008)=(S=6812512,E=6812515,C=73)
	CT(27009)=(S=6812516,E=6812519,C=8)
	CT(27010)=(S=6812520,E=6812523,C=11)
	CT(27011)=(S=6812524,E=6812525,C=74)
	CT(27012)=(S=6812526,E=6812526,C=0)
	CT(27013)=(S=6812527,E=6812535,C=4)
	CT(27014)=(S=6812536,E=6812539,C=2)
	CT(27015)=(S=6812544,E=6812547,C=8)
	CT(27016)=(S=6812548,E=6812551,C=4)
	CT(27017)=(S=6812552,E=6812555,C=1)
	CT(27018)=(S=6812556,E=6812559,C=84)
	CT(27019)=(S=6812560,E=6812563,C=4)
	CT(27020)=(S=6812564,E=6812567,C=0)
	CT(27021)=(S=6812568,E=6812571,C=2)
	CT(27022)=(S=6812572,E=6812575,C=5)
	CT(27023)=(S=6812576,E=6812579,C=73)
	CT(27024)=(S=6812580,E=6812583,C=8)
	CT(27025)=(S=6812584,E=6812587,C=3)
	CT(27026)=(S=6812588,E=6812591,C=7)
	CT(27027)=(S=6812592,E=6812592,C=4)
	CT(27028)=(S=6812593,E=6812594,C=74)
	CT(27029)=(S=6812595,E=6812595,C=89)
	CT(27030)=(S=6812596,E=6812599,C=8)
	CT(27031)=(S=6812600,E=6812603,C=4)
	CT(27032)=(S=6812608,E=6812611,C=5)
	CT(27033)=(S=6812612,E=6812615,C=2)
	CT(27034)=(S=6812616,E=6812619,C=6)
	CT(27035)=(S=6812620,E=6812623,C=73)
	CT(27036)=(S=6812624,E=6812627,C=2)
	CT(27037)=(S=6812628,E=6812631,C=5)
	CT(27038)=(S=6812632,E=6812635,C=11)
	CT(27039)=(S=6812636,E=6812639,C=7)
	CT(27040)=(S=6812640,E=6812653,C=4)
	CT(27041)=(S=6812654,E=6812654,C=10)
	CT(27042)=(S=6812655,E=6812655,C=0)
	CT(27043)=(S=6812656,E=6812659,C=2)
	CT(27044)=(S=6812660,E=6812661,C=0)
	CT(27045)=(S=6812662,E=6812663,C=74)
	CT(27046)=(S=6812664,E=6812667,C=9)
	CT(27047)=(S=6812668,E=6812671,C=1)
	CT(27048)=(S=6812672,E=6812675,C=8)
	CT(27049)=(S=6812680,E=6812683,C=7)
	CT(27050)=(S=6812684,E=6812687,C=84)
	CT(27051)=(S=6812688,E=6812691,C=1)
	CT(27052)=(S=6812692,E=6812695,C=84)
	CT(27053)=(S=6812696,E=6812699,C=8)
	CT(27054)=(S=6812700,E=6812703,C=10)
	CT(27055)=(S=6812708,E=6812711,C=74)
	CT(27056)=(S=6812712,E=6812715,C=6)
	CT(27057)=(S=6812716,E=6812719,C=8)
	CT(27058)=(S=6812720,E=6812720,C=4)
	CT(27059)=(S=6812721,E=6812721,C=5)
	CT(27060)=(S=6812722,E=6812722,C=73)
	CT(27061)=(S=6812723,E=6812723,C=74)
	CT(27062)=(S=6812724,E=6812729,C=8)
	CT(27063)=(S=6812730,E=6812759,C=1)
	CT(27064)=(S=6812760,E=6812763,C=8)
	CT(27065)=(S=6812764,E=6812767,C=2)
	CT(27066)=(S=6812768,E=6812771,C=74)
	CT(27067)=(S=6812772,E=6812775,C=7)
	CT(27068)=(S=6812776,E=6812779,C=2)
	CT(27069)=(S=6812780,E=6812783,C=6)
	CT(27070)=(S=6812792,E=6812799,C=4)
	CT(27071)=(S=6812800,E=6812803,C=0)
	CT(27072)=(S=6812804,E=6812807,C=77)
	CT(27073)=(S=6812808,E=6812811,C=11)
	CT(27074)=(S=6812812,E=6812815,C=84)
	CT(27075)=(S=6812816,E=6812819,C=89)
	CT(27076)=(S=6812820,E=6812823,C=8)
	CT(27077)=(S=6812824,E=6812827,C=4)
	CT(27078)=(S=6812828,E=6812831,C=8)
	CT(27079)=(S=6812832,E=6812833,C=74)
	CT(27080)=(S=6812834,E=6812834,C=84)
	CT(27081)=(S=6812835,E=6812835,C=5)
	CT(27082)=(S=6812836,E=6812839,C=1)
	CT(27083)=(S=6812840,E=6812843,C=4)
	CT(27084)=(S=6812844,E=6812851,C=77)
	CT(27085)=(S=6812852,E=6812855,C=73)
	CT(27086)=(S=6812856,E=6812859,C=84)
	CT(27087)=(S=6812860,E=6812861,C=5)
	CT(27088)=(S=6812862,E=6812862,C=4)
	CT(27089)=(S=6812863,E=6812863,C=5)
	CT(27090)=(S=6812868,E=6812875,C=0)
	CT(27091)=(S=6812876,E=6812879,C=74)
	CT(27092)=(S=6812880,E=6812883,C=2)
	CT(27093)=(S=6812884,E=6812887,C=73)
	CT(27094)=(S=6812888,E=6812895,C=2)
	CT(27095)=(S=6812896,E=6812899,C=8)
	CT(27096)=(S=6812900,E=6812903,C=91)
	CT(27097)=(S=6812904,E=6812907,C=1)
	CT(27098)=(S=6812908,E=6812911,C=0)
	CT(27099)=(S=6812912,E=6812916,C=4)
	CT(27100)=(S=6812917,E=6812917,C=74)
	CT(27101)=(S=6812918,E=6812919,C=7)
	CT(27102)=(S=6812920,E=6812923,C=86)
	CT(27103)=(S=6812924,E=6812927,C=1)
	CT(27104)=(S=6812928,E=6812931,C=4)
	CT(27105)=(S=6812932,E=6812935,C=2)
	CT(27106)=(S=6812936,E=6812950,C=4)
	CT(27107)=(S=6812951,E=6812951,C=1)
	CT(27108)=(S=6812952,E=6812955,C=5)
	CT(27109)=(S=6812956,E=6812959,C=8)
	CT(27110)=(S=6812960,E=6812963,C=4)
	CT(27111)=(S=6812964,E=6812967,C=7)
	CT(27112)=(S=6812968,E=6812971,C=2)
	CT(27113)=(S=6812972,E=6812979,C=4)
	CT(27114)=(S=6812980,E=6812983,C=1)
	CT(27115)=(S=6812984,E=6812987,C=2)
	CT(27116)=(S=6812988,E=6812991,C=1)
	CT(27117)=(S=6812992,E=6812999,C=4)
	CT(27118)=(S=6813000,E=6813001,C=74)
	CT(27119)=(S=6813002,E=6813002,C=4)
	CT(27120)=(S=6813003,E=6813003,C=0)
	CT(27121)=(S=6813004,E=6813007,C=86)
	CT(27122)=(S=6813008,E=6813011,C=1)
	CT(27123)=(S=6813012,E=6813015,C=0)
	CT(27124)=(S=6813016,E=6813019,C=5)
	CT(27125)=(S=6813020,E=6813023,C=7)
	CT(27126)=(S=6813024,E=6813027,C=84)
	CT(27127)=(S=6813028,E=6813035,C=4)
	CT(27128)=(S=6813036,E=6813039,C=84)
	CT(27129)=(S=6813040,E=6813047,C=4)
	CT(27130)=(S=6813048,E=6813049,C=5)
	CT(27131)=(S=6813050,E=6813051,C=74)
	CT(27132)=(S=6813052,E=6813059,C=1)
	CT(27133)=(S=6813060,E=6813063,C=77)
	CT(27134)=(S=6813064,E=6813067,C=74)
	CT(27135)=(S=6813068,E=6813071,C=84)
	CT(27136)=(S=6813072,E=6813072,C=4)
	CT(27137)=(S=6813073,E=6813074,C=0)
	CT(27138)=(S=6813075,E=6813075,C=4)
	CT(27139)=(S=6813076,E=6813079,C=11)
	CT(27140)=(S=6813080,E=6813083,C=0)
	CT(27141)=(S=6813084,E=6813087,C=4)
	CT(27142)=(S=6813088,E=6813091,C=166)
	CT(27143)=(S=6813092,E=6813095,C=154)
	CT(27144)=(S=6813096,E=6813099,C=2)
	CT(27145)=(S=6813100,E=6813103,C=73)
	CT(27146)=(S=6813105,E=6813105,C=74)
	CT(27147)=(S=6813106,E=6813107,C=84)
	CT(27148)=(S=6813108,E=6813111,C=74)
	CT(27149)=(S=6813112,E=6813117,C=4)
	CT(27150)=(S=6813118,E=6813119,C=8)
	CT(27151)=(S=6813120,E=6813123,C=77)
	CT(27152)=(S=6813124,E=6813131,C=4)
	CT(27153)=(S=6813132,E=6813135,C=84)
	CT(27154)=(S=6813136,E=6813139,C=8)
	CT(27155)=(S=6813140,E=6813150,C=0)
	CT(27156)=(S=6813151,E=6813151,C=7)
	CT(27157)=(S=6813152,E=6813152,C=2)
	CT(27158)=(S=6813153,E=6813154,C=74)
	CT(27159)=(S=6813155,E=6813155,C=0)
	CT(27160)=(S=6813156,E=6813167,C=2)
	CT(27161)=(S=6813168,E=6813171,C=161)
	CT(27162)=(S=6813172,E=6813183,C=11)
	CT(27163)=(S=6813184,E=6813187,C=74)
	CT(27164)=(S=6813188,E=6813191,C=0)
	CT(27165)=(S=6813192,E=6813199,C=1)
	CT(27166)=(S=6813200,E=6813203,C=3)
	CT(27167)=(S=6813204,E=6813207,C=10)
	CT(27168)=(S=6813208,E=6813211,C=3)
	CT(27169)=(S=6813212,E=6813220,C=0)
	CT(27170)=(S=6813221,E=6813221,C=7)
	CT(27171)=(S=6813222,E=6813222,C=8)
	CT(27172)=(S=6813223,E=6813223,C=2)
	CT(27173)=(S=6813228,E=6813233,C=4)
	CT(27174)=(S=6813234,E=6813234,C=3)
	CT(27175)=(S=6813235,E=6813235,C=0)
	CT(27176)=(S=6813236,E=6813239,C=73)
	CT(27177)=(S=6813240,E=6813243,C=6)
	CT(27178)=(S=6813244,E=6813247,C=2)
	CT(27179)=(S=6813248,E=6813251,C=4)
	CT(27180)=(S=6813252,E=6813255,C=2)
	CT(27181)=(S=6813260,E=6813260,C=74)
	CT(27182)=(S=6813261,E=6813261,C=8)
	CT(27183)=(S=6813262,E=6813262,C=7)
	CT(27184)=(S=6813263,E=6813263,C=8)
	CT(27185)=(S=6813264,E=6813267,C=2)
	CT(27186)=(S=6813268,E=6813271,C=4)
	CT(27187)=(S=6813272,E=6813275,C=5)
	CT(27188)=(S=6813276,E=6813279,C=7)
	CT(27189)=(S=6813280,E=6813287,C=0)
	CT(27190)=(S=6813288,E=6813288,C=11)
	CT(27191)=(S=6813290,E=6813290,C=4)
	CT(27192)=(S=6813291,E=6813291,C=74)
	CT(27193)=(S=6813292,E=6813295,C=77)
	CT(27194)=(S=6813296,E=6813299,C=5)
	CT(27195)=(S=6813300,E=6813303,C=74)
	CT(27196)=(S=6813304,E=6813311,C=1)
	CT(27197)=(S=6813312,E=6813312,C=0)
	CT(27198)=(S=6813313,E=6813313,C=74)
	CT(27199)=(S=6813314,E=6813315,C=0)
	CT(27200)=(S=6813316,E=6813319,C=1)
	CT(27201)=(S=6813320,E=6813323,C=0)
	CT(27202)=(S=6813324,E=6813327,C=7)
	CT(27203)=(S=6813328,E=6813331,C=86)
	CT(27204)=(S=6813332,E=6813335,C=2)
	CT(27205)=(S=6813336,E=6813343,C=1)
	CT(27206)=(S=6813344,E=6813347,C=8)
	CT(27207)=(S=6813348,E=6813351,C=2)
	CT(27208)=(S=6813352,E=6813352,C=0)
	CT(27209)=(S=6813353,E=6813353,C=74)
	CT(27210)=(S=6813354,E=6813355,C=4)
	CT(27211)=(S=6813360,E=6813363,C=8)
	CT(27212)=(S=6813364,E=6813367,C=73)
	CT(27213)=(S=6813368,E=6813369,C=74)
	CT(27214)=(S=6813370,E=6813370,C=0)
	CT(27215)=(S=6813372,E=6813375,C=3)
	CT(27216)=(S=6813380,E=6813383,C=4)
	CT(27217)=(S=6813384,E=6813387,C=7)
	CT(27218)=(S=6813388,E=6813391,C=5)
	CT(27219)=(S=6813392,E=6813395,C=9)
	CT(27220)=(S=6813396,E=6813399,C=4)
	CT(27221)=(S=6813400,E=6813400,C=8)
	CT(27222)=(S=6813401,E=6813401,C=5)
	CT(27223)=(S=6813404,E=6813407,C=11)
	CT(27224)=(S=6813408,E=6813411,C=4)
	CT(27225)=(S=6813412,E=6813415,C=86)
	CT(27226)=(S=6813416,E=6813416,C=2)
	CT(27227)=(S=6813417,E=6813419,C=0)
	CT(27228)=(S=6813420,E=6813423,C=6)
	CT(27229)=(S=6813424,E=6813427,C=4)
	CT(27230)=(S=6813428,E=6813431,C=8)
	CT(27231)=(S=6813432,E=6813432,C=29)
	CT(27232)=(S=6813433,E=6813435,C=73)
	CT(27233)=(S=6813436,E=6813439,C=2)
	CT(27234)=(S=6813440,E=6813443,C=0)
	CT(27235)=(S=6813448,E=6813467,C=74)
	CT(27236)=(S=6813468,E=6813471,C=4)
	CT(27237)=(S=6813472,E=6813475,C=74)
	CT(27238)=(S=6813478,E=6813479,C=10)
	CT(27239)=(S=6813480,E=6813483,C=74)
	CT(27240)=(S=6813484,E=6813487,C=84)
	CT(27241)=(S=6813488,E=6813491,C=156)
	CT(27242)=(S=6813496,E=6813499,C=73)
	CT(27243)=(S=6813500,E=6813503,C=3)
	CT(27244)=(S=6813504,E=6813505,C=0)
	CT(27245)=(S=6813506,E=6813506,C=77)
	CT(27246)=(S=6813507,E=6813507,C=0)
	CT(27247)=(S=6813508,E=6813511,C=2)
	CT(27248)=(S=6813512,E=6813515,C=73)
	CT(27249)=(S=6813516,E=6813519,C=0)
	CT(27250)=(S=6813520,E=6813523,C=4)
	CT(27251)=(S=6813524,E=6813531,C=2)
	CT(27252)=(S=6813532,E=6813535,C=8)
	CT(27253)=(S=6813536,E=6813539,C=4)
	CT(27254)=(S=6813540,E=6813543,C=74)
	CT(27255)=(S=6813544,E=6813547,C=4)
	CT(27256)=(S=6813548,E=6813551,C=8)
	CT(27257)=(S=6813552,E=6813555,C=9)
	CT(27258)=(S=6813556,E=6813556,C=0)
	CT(27259)=(S=6813557,E=6813557,C=74)
	CT(27260)=(S=6813558,E=6813559,C=0)
	CT(27261)=(S=6813560,E=6813563,C=74)
	CT(27262)=(S=6813564,E=6813565,C=77)
	CT(27263)=(S=6813566,E=6813567,C=74)
	CT(27264)=(S=6813568,E=6813571,C=0)
	CT(27265)=(S=6813572,E=6813575,C=7)
	CT(27266)=(S=6813576,E=6813587,C=8)
	CT(27267)=(S=6813588,E=6813591,C=4)
	CT(27268)=(S=6813592,E=6813595,C=73)
	CT(27269)=(S=6813596,E=6813607,C=8)
	CT(27270)=(S=6813608,E=6813619,C=1)
	CT(27271)=(S=6813622,E=6813623,C=74)
	CT(27272)=(S=6813624,E=6813627,C=0)
	CT(27273)=(S=6813628,E=6813631,C=2)
	CT(27274)=(S=6813632,E=6813635,C=73)
	CT(27275)=(S=6813636,E=6813637,C=74)
	CT(27276)=(S=6813638,E=6813638,C=89)
	CT(27277)=(S=6813639,E=6813639,C=7)
	CT(27278)=(S=6813640,E=6813643,C=1)
	CT(27279)=(S=6813644,E=6813647,C=7)
	CT(27280)=(S=6813648,E=6813650,C=4)
	CT(27281)=(S=6813651,E=6813651,C=74)
	CT(27282)=(S=6813652,E=6813655,C=1)
	CT(27283)=(S=6813656,E=6813659,C=74)
	CT(27284)=(S=6813660,E=6813663,C=6)
	CT(27285)=(S=6813664,E=6813665,C=0)
	CT(27286)=(S=6813666,E=6813667,C=74)
	CT(27287)=(S=6813668,E=6813671,C=8)
	CT(27288)=(S=6813672,E=6813675,C=6)
	CT(27289)=(S=6813676,E=6813677,C=0)
	CT(27290)=(S=6813678,E=6813679,C=84)
	CT(27291)=(S=6813680,E=6813683,C=2)
	CT(27292)=(S=6813684,E=6813685,C=74)
	CT(27293)=(S=6813686,E=6813686,C=4)
	CT(27294)=(S=6813687,E=6813687,C=73)
	CT(27295)=(S=6813688,E=6813691,C=0)
	CT(27296)=(S=6813692,E=6813695,C=10)
	CT(27297)=(S=6813696,E=6813697,C=1)
	CT(27298)=(S=6813698,E=6813703,C=0)
	CT(27299)=(S=6813704,E=6813707,C=2)
	CT(27300)=(S=6813708,E=6813711,C=84)
	CT(27301)=(S=6813712,E=6813719,C=0)
	CT(27302)=(S=6813720,E=6813720,C=4)
	CT(27303)=(S=6813721,E=6813721,C=74)
	CT(27304)=(S=6813722,E=6813723,C=0)
	CT(27305)=(S=6813724,E=6813731,C=4)
	CT(27306)=(S=6813732,E=6813735,C=8)
	CT(27307)=(S=6813736,E=6813739,C=86)
	CT(27308)=(S=6813740,E=6813743,C=0)
	CT(27309)=(S=6813744,E=6813747,C=2)
	CT(27310)=(S=6813752,E=6813755,C=74)
	CT(27311)=(S=6813756,E=6813759,C=4)
	CT(27312)=(S=6813760,E=6813763,C=1)
	CT(27313)=(S=6813764,E=6813767,C=2)
	CT(27314)=(S=6813768,E=6813771,C=4)
	CT(27315)=(S=6813772,E=6813775,C=8)
	CT(27316)=(S=6813776,E=6813791,C=4)
	CT(27317)=(S=6813792,E=6813793,C=0)
	CT(27318)=(S=6813794,E=6813795,C=7)
	CT(27319)=(S=6813796,E=6813799,C=1)
	CT(27320)=(S=6813800,E=6813803,C=6)
	CT(27321)=(S=6813804,E=6813807,C=8)
	CT(27322)=(S=6813808,E=6813819,C=4)
	CT(27323)=(S=6813820,E=6813823,C=1)
	CT(27324)=(S=6813824,E=6813827,C=0)
	CT(27325)=(S=6813828,E=6813831,C=7)
	CT(27326)=(S=6813836,E=6813839,C=8)
	CT(27327)=(S=6813840,E=6813841,C=7)
	CT(27328)=(S=6813842,E=6813842,C=74)
	CT(27329)=(S=6813843,E=6813843,C=0)
	CT(27330)=(S=6813844,E=6813847,C=2)
	CT(27331)=(S=6813848,E=6813851,C=1)
	CT(27332)=(S=6813852,E=6813855,C=4)
	CT(27333)=(S=6813856,E=6813863,C=11)
	CT(27334)=(S=6813864,E=6813867,C=1)
	CT(27335)=(S=6813868,E=6813871,C=4)
	CT(27336)=(S=6813872,E=6813875,C=73)
	CT(27337)=(S=6813880,E=6813883,C=8)
	CT(27338)=(S=6813884,E=6813884,C=0)
	CT(27339)=(S=6813885,E=6813887,C=73)
	CT(27340)=(S=6813888,E=6813891,C=1)
	CT(27341)=(S=6813896,E=6813896,C=73)
	CT(27342)=(S=6813897,E=6813897,C=0)
	CT(27343)=(S=6813900,E=6813903,C=84)
	CT(27344)=(S=6813904,E=6813907,C=4)
	CT(27345)=(S=6813908,E=6813911,C=1)
	CT(27346)=(S=6813912,E=6813915,C=74)
	CT(27347)=(S=6813920,E=6813927,C=1)
	CT(27348)=(S=6813928,E=6813931,C=4)
	CT(27349)=(S=6813932,E=6813935,C=84)
	CT(27350)=(S=6813936,E=6813939,C=8)
	CT(27351)=(S=6813940,E=6813943,C=4)
	CT(27352)=(S=6813944,E=6813944,C=74)
	CT(27353)=(S=6813945,E=6813945,C=0)
	CT(27354)=(S=6813947,E=6813947,C=2)
	CT(27355)=(S=6813948,E=6813951,C=8)
	CT(27356)=(S=6813952,E=6813955,C=84)
	CT(27357)=(S=6813956,E=6813959,C=4)
	CT(27358)=(S=6813964,E=6813967,C=1)
	CT(27359)=(S=6813968,E=6813971,C=4)
	CT(27360)=(S=6813972,E=6813975,C=11)
	CT(27361)=(S=6813976,E=6813979,C=4)
	CT(27362)=(S=6813980,E=6813983,C=6)
	CT(27363)=(S=6813984,E=6813987,C=8)
	CT(27364)=(S=6813988,E=6813999,C=4)
	CT(27365)=(S=6814000,E=6814003,C=0)
	CT(27366)=(S=6814004,E=6814007,C=1)
	CT(27367)=(S=6814008,E=6814008,C=84)
	CT(27368)=(S=6814009,E=6814009,C=0)
	CT(27369)=(S=6814010,E=6814011,C=74)
	CT(27370)=(S=6814012,E=6814015,C=73)
	CT(27371)=(S=6814016,E=6814017,C=74)
	CT(27372)=(S=6814018,E=6814023,C=0)
	CT(27373)=(S=6814024,E=6814027,C=8)
	CT(27374)=(S=6814028,E=6814035,C=4)
	CT(27375)=(S=6814036,E=6814039,C=5)
	CT(27376)=(S=6814040,E=6814043,C=4)
	CT(27377)=(S=6814044,E=6814047,C=5)
	CT(27378)=(S=6814048,E=6814051,C=4)
	CT(27379)=(S=6814052,E=6814055,C=11)
	CT(27380)=(S=6814060,E=6814063,C=8)
	CT(27381)=(S=6814064,E=6814071,C=2)
	CT(27382)=(S=6814072,E=6814075,C=4)
	CT(27383)=(S=6814076,E=6814079,C=0)
	CT(27384)=(S=6814080,E=6814091,C=1)
	CT(27385)=(S=6814092,E=6814095,C=0)
	CT(27386)=(S=6814096,E=6814099,C=1)
	CT(27387)=(S=6814100,E=6814103,C=0)
	CT(27388)=(S=6814104,E=6814107,C=77)
	CT(27389)=(S=6814112,E=6814115,C=7)
	CT(27390)=(S=6814116,E=6814131,C=1)
	CT(27391)=(S=6814132,E=6814135,C=4)
	CT(27392)=(S=6814136,E=6814139,C=2)
	CT(27393)=(S=6814140,E=6814147,C=1)
	CT(27394)=(S=6814148,E=6814151,C=8)
	CT(27395)=(S=6814152,E=6814155,C=2)
	CT(27396)=(S=6814156,E=6814159,C=4)
	CT(27397)=(S=6814160,E=6814161,C=5)
	CT(27398)=(S=6814162,E=6814162,C=74)
	CT(27399)=(S=6814163,E=6814163,C=4)
	CT(27400)=(S=6814164,E=6814167,C=2)
	CT(27401)=(S=6814172,E=6814177,C=0)
	CT(27402)=(S=6814178,E=6814178,C=4)
	CT(27403)=(S=6814179,E=6814179,C=74)
	CT(27404)=(S=6814180,E=6814183,C=77)
	CT(27405)=(S=6814184,E=6814195,C=4)
	CT(27406)=(S=6814196,E=6814199,C=1)
	CT(27407)=(S=6814200,E=6814203,C=4)
	CT(27408)=(S=6814204,E=6814207,C=1)
	CT(27409)=(S=6814216,E=6814217,C=0)
	CT(27410)=(S=6814218,E=6814219,C=4)
	CT(27411)=(S=6814220,E=6814223,C=8)
	CT(27412)=(S=6814224,E=6814227,C=4)
	CT(27413)=(S=6814228,E=6814231,C=0)
	CT(27414)=(S=6814232,E=6814235,C=11)
	CT(27415)=(S=6814236,E=6814239,C=4)
	CT(27416)=(S=6814240,E=6814243,C=1)
	CT(27417)=(S=6814248,E=6814248,C=4)
	CT(27418)=(S=6814249,E=6814249,C=8)
	CT(27419)=(S=6814250,E=6814251,C=84)
	CT(27420)=(S=6814252,E=6814255,C=4)
	CT(27421)=(S=6814256,E=6814259,C=73)
	CT(27422)=(S=6814260,E=6814263,C=8)
	CT(27423)=(S=6814264,E=6814267,C=7)
	CT(27424)=(S=6814268,E=6814269,C=0)
	CT(27425)=(S=6814270,E=6814270,C=4)
	CT(27426)=(S=6814271,E=6814271,C=0)
	CT(27427)=(S=6814272,E=6814275,C=7)
	CT(27428)=(S=6814276,E=6814279,C=84)
	CT(27429)=(S=6814280,E=6814283,C=2)
	CT(27430)=(S=6814284,E=6814295,C=4)
	CT(27431)=(S=6814296,E=6814299,C=73)
	CT(27432)=(S=6814300,E=6814307,C=2)
	CT(27433)=(S=6814308,E=6814311,C=73)
	CT(27434)=(S=6814312,E=6814315,C=1)
	CT(27435)=(S=6814316,E=6814319,C=2)
	CT(27436)=(S=6814320,E=6814323,C=4)
	CT(27437)=(S=6814324,E=6814327,C=73)
	CT(27438)=(S=6814328,E=6814331,C=2)
	CT(27439)=(S=6814332,E=6814335,C=1)
	CT(27440)=(S=6814336,E=6814339,C=3)
	CT(27441)=(S=6814340,E=6814343,C=75)
	CT(27442)=(S=6814344,E=6814347,C=4)
	CT(27443)=(S=6814348,E=6814351,C=7)
	CT(27444)=(S=6814352,E=6814379,C=4)
	CT(27445)=(S=6814380,E=6814383,C=2)
	CT(27446)=(S=6814388,E=6814391,C=1)
	CT(27447)=(S=6814392,E=6814394,C=4)
	CT(27448)=(S=6814395,E=6814395,C=5)
	CT(27449)=(S=6814396,E=6814399,C=4)
	CT(27450)=(S=6814400,E=6814403,C=1)
	CT(27451)=(S=6814404,E=6814405,C=3)
	CT(27452)=(S=6814406,E=6814406,C=73)
	CT(27453)=(S=6814407,E=6814407,C=74)
	CT(27454)=(S=6814408,E=6814411,C=2)
	CT(27455)=(S=6814412,E=6814415,C=73)
	CT(27456)=(S=6814416,E=6814419,C=4)
	CT(27457)=(S=6814420,E=6814423,C=0)
	CT(27458)=(S=6814424,E=6814427,C=1)
	CT(27459)=(S=6814430,E=6814431,C=77)
	CT(27460)=(S=6814432,E=6814435,C=1)
	CT(27461)=(S=6814436,E=6814443,C=73)
	CT(27462)=(S=6814444,E=6814447,C=1)
	CT(27463)=(S=6814448,E=6814455,C=0)
	CT(27464)=(S=6814456,E=6814463,C=1)
	CT(27465)=(S=6814464,E=6814467,C=5)
	CT(27466)=(S=6814468,E=6814471,C=0)
	CT(27467)=(S=6814472,E=6814475,C=5)
	CT(27468)=(S=6814476,E=6814477,C=0)
	CT(27469)=(S=6814478,E=6814478,C=74)
	CT(27470)=(S=6814479,E=6814479,C=0)
	CT(27471)=(S=6814480,E=6814495,C=4)
	CT(27472)=(S=6814496,E=6814499,C=1)
	CT(27473)=(S=6814500,E=6814503,C=8)
	CT(27474)=(S=6814504,E=6814505,C=74)
	CT(27475)=(S=6814506,E=6814506,C=0)
	CT(27476)=(S=6814507,E=6814507,C=4)
	CT(27477)=(S=6814508,E=6814509,C=74)
	CT(27478)=(S=6814510,E=6814510,C=4)
	CT(27479)=(S=6814511,E=6814511,C=0)
	CT(27480)=(S=6814512,E=6814527,C=4)
	CT(27481)=(S=6814528,E=6814531,C=0)
	CT(27482)=(S=6814532,E=6814535,C=3)
	CT(27483)=(S=6814536,E=6814543,C=2)
	CT(27484)=(S=6814544,E=6814547,C=4)
	CT(27485)=(S=6814548,E=6814551,C=1)
	CT(27486)=(S=6814552,E=6814555,C=8)
	CT(27487)=(S=6814556,E=6814559,C=4)
	CT(27488)=(S=6814560,E=6814563,C=1)
	CT(27489)=(S=6814564,E=6814567,C=4)
	CT(27490)=(S=6814568,E=6814571,C=6)
	CT(27491)=(S=6814572,E=6814573,C=7)
	CT(27492)=(S=6814574,E=6814575,C=0)
	CT(27493)=(S=6814576,E=6814579,C=8)
	CT(27494)=(S=6814580,E=6814583,C=2)
	CT(27495)=(S=6814584,E=6814587,C=84)
	CT(27496)=(S=6814588,E=6814595,C=1)
	CT(27497)=(S=6814596,E=6814597,C=194)
	CT(27498)=(S=6814598,E=6814611,C=0)
	CT(27499)=(S=6814612,E=6814615,C=4)
	CT(27500)=(S=6814616,E=6814619,C=74)
	CT(27501)=(S=6814620,E=6814623,C=2)
	CT(27502)=(S=6814624,E=6814627,C=1)
	CT(27503)=(S=6814628,E=6814631,C=8)
	CT(27504)=(S=6814632,E=6814632,C=73)
	CT(27505)=(S=6814633,E=6814639,C=0)
	CT(27506)=(S=6814640,E=6814643,C=89)
	CT(27507)=(S=6814644,E=6814647,C=74)
	CT(27508)=(S=6814648,E=6814655,C=4)
	CT(27509)=(S=6814660,E=6814661,C=7)
	CT(27510)=(S=6814662,E=6814663,C=8)
	CT(27511)=(S=6814664,E=6814667,C=5)
	CT(27512)=(S=6814668,E=6814671,C=1)
	CT(27513)=(S=6814672,E=6814695,C=4)
	CT(27514)=(S=6814696,E=6814699,C=84)
	CT(27515)=(S=6814700,E=6814707,C=1)
	CT(27516)=(S=6814712,E=6814715,C=4)
	CT(27517)=(S=6814716,E=6814719,C=2)
	CT(27518)=(S=6814720,E=6814723,C=11)
	CT(27519)=(S=6814724,E=6814727,C=4)
	CT(27520)=(S=6814730,E=6814731,C=0)
	CT(27521)=(S=6814732,E=6814733,C=74)
	CT(27522)=(S=6814734,E=6814735,C=0)
	CT(27523)=(S=6814736,E=6814739,C=8)
	CT(27524)=(S=6814740,E=6814743,C=74)
	CT(27525)=(S=6814744,E=6814747,C=4)
	CT(27526)=(S=6814748,E=6814759,C=1)
	CT(27527)=(S=6814760,E=6814767,C=4)
	CT(27528)=(S=6814768,E=6814768,C=10)
	CT(27529)=(S=6814769,E=6814769,C=7)
	CT(27530)=(S=6814770,E=6814771,C=74)
	CT(27531)=(S=6814776,E=6814779,C=0)
	CT(27532)=(S=6814780,E=6814783,C=4)
	CT(27533)=(S=6814784,E=6814787,C=1)
	CT(27534)=(S=6814788,E=6814791,C=9)
	CT(27535)=(S=6814792,E=6814799,C=8)
	CT(27536)=(S=6814800,E=6814803,C=77)
	CT(27537)=(S=6814804,E=6814804,C=161)
	CT(27538)=(S=6814805,E=6814805,C=74)
	CT(27539)=(S=6814806,E=6814806,C=4)
	CT(27540)=(S=6814807,E=6814807,C=0)
	CT(27541)=(S=6814808,E=6814811,C=8)
	CT(27542)=(S=6814812,E=6814815,C=4)
	CT(27543)=(S=6814820,E=6814821,C=74)
	CT(27544)=(S=6814822,E=6814823,C=0)
	CT(27545)=(S=6814824,E=6814827,C=1)
	CT(27546)=(S=6814828,E=6814831,C=4)
	CT(27547)=(S=6814832,E=6814835,C=7)
	CT(27548)=(S=6814840,E=6814847,C=2)
	CT(27549)=(S=6814852,E=6814855,C=8)
	CT(27550)=(S=6814856,E=6814859,C=2)
	CT(27551)=(S=6814860,E=6814871,C=4)
	CT(27552)=(S=6814872,E=6814875,C=0)
	CT(27553)=(S=6814876,E=6814881,C=4)
	CT(27554)=(S=6814882,E=6814882,C=8)
	CT(27555)=(S=6814883,E=6814883,C=74)
	CT(27556)=(S=6814884,E=6814891,C=4)
	CT(27557)=(S=6814892,E=6814895,C=1)
	CT(27558)=(S=6814900,E=6814903,C=2)
	CT(27559)=(S=6814904,E=6814907,C=4)
	CT(27560)=(S=6814908,E=6814909,C=74)
	CT(27561)=(S=6814910,E=6814911,C=0)
	CT(27562)=(S=6814912,E=6814915,C=4)
	CT(27563)=(S=6814920,E=6814923,C=7)
	CT(27564)=(S=6814924,E=6814931,C=1)
	CT(27565)=(S=6814932,E=6814939,C=4)
	CT(27566)=(S=6814940,E=6814943,C=36)
	CT(27567)=(S=6814948,E=6814951,C=6)
	CT(27568)=(S=6814952,E=6814955,C=1)
	CT(27569)=(S=6814956,E=6814959,C=4)
	CT(27570)=(S=6814960,E=6814963,C=8)
	CT(27571)=(S=6814964,E=6814964,C=9)
	CT(27572)=(S=6814965,E=6814965,C=4)
	CT(27573)=(S=6814966,E=6814966,C=0)
	CT(27574)=(S=6814967,E=6814967,C=73)
	CT(27575)=(S=6814968,E=6814971,C=1)
	CT(27576)=(S=6814972,E=6814975,C=11)
	CT(27577)=(S=6814976,E=6814979,C=74)
	CT(27578)=(S=6814980,E=6814983,C=1)
	CT(27579)=(S=6814984,E=6814987,C=8)
	CT(27580)=(S=6814988,E=6814991,C=5)
	CT(27581)=(S=6814992,E=6814995,C=2)
	CT(27582)=(S=6814996,E=6814999,C=4)
	CT(27583)=(S=6815000,E=6815003,C=7)
	CT(27584)=(S=6815004,E=6815013,C=4)
	CT(27585)=(S=6815014,E=6815015,C=84)
	CT(27586)=(S=6815016,E=6815019,C=8)
	CT(27587)=(S=6815020,E=6815023,C=84)
	CT(27588)=(S=6815028,E=6815031,C=4)
	CT(27589)=(S=6815032,E=6815035,C=6)
	CT(27590)=(S=6815036,E=6815039,C=1)
	CT(27591)=(S=6815040,E=6815043,C=0)
	CT(27592)=(S=6815044,E=6815045,C=74)
	CT(27593)=(S=6815046,E=6815047,C=4)
	CT(27594)=(S=6815048,E=6815051,C=3)
	CT(27595)=(S=6815052,E=6815061,C=2)
	CT(27596)=(S=6815062,E=6815063,C=74)
	CT(27597)=(S=6815064,E=6815067,C=11)
	CT(27598)=(S=6815068,E=6815071,C=0)
	CT(27599)=(S=6815072,E=6815075,C=8)
	CT(27600)=(S=6815080,E=6815080,C=4)
	CT(27601)=(S=6815081,E=6815081,C=5)
	CT(27602)=(S=6815082,E=6815083,C=74)
	CT(27603)=(S=6815084,E=6815087,C=5)
	CT(27604)=(S=6815088,E=6815091,C=74)
	CT(27605)=(S=6815092,E=6815092,C=73)
	CT(27606)=(S=6815093,E=6815093,C=3)
	CT(27607)=(S=6815094,E=6815095,C=0)
	CT(27608)=(S=6815096,E=6815099,C=8)
	CT(27609)=(S=6815100,E=6815100,C=74)
	CT(27610)=(S=6815101,E=6815101,C=4)
	CT(27611)=(S=6815102,E=6815102,C=0)
	CT(27612)=(S=6815103,E=6815103,C=10)
	CT(27613)=(S=6815108,E=6815111,C=3)
	CT(27614)=(S=6815112,E=6815115,C=73)
	CT(27615)=(S=6815116,E=6815119,C=8)
	CT(27616)=(S=6815120,E=6815123,C=7)
	CT(27617)=(S=6815124,E=6815131,C=4)
	CT(27618)=(S=6815132,E=6815135,C=77)
	CT(27619)=(S=6815136,E=6815139,C=84)
	CT(27620)=(S=6815142,E=6815151,C=4)
	CT(27621)=(S=6815152,E=6815155,C=2)
	CT(27622)=(S=6815156,E=6815159,C=86)
	CT(27623)=(S=6815160,E=6815163,C=9)
	CT(27624)=(S=6815164,E=6815167,C=2)
	CT(27625)=(S=6815168,E=6815175,C=0)
	CT(27626)=(S=6815176,E=6815179,C=4)
	CT(27627)=(S=6815180,E=6815183,C=1)
	CT(27628)=(S=6815184,E=6815187,C=4)
	CT(27629)=(S=6815188,E=6815191,C=74)
	CT(27630)=(S=6815192,E=6815195,C=2)
	CT(27631)=(S=6815196,E=6815203,C=1)
	CT(27632)=(S=6815204,E=6815207,C=4)
	CT(27633)=(S=6815208,E=6815211,C=1)
	CT(27634)=(S=6815212,E=6815215,C=8)
	CT(27635)=(S=6815220,E=6815223,C=10)
	CT(27636)=(S=6815224,E=6815227,C=8)
	CT(27637)=(S=6815228,E=6815231,C=86)
	CT(27638)=(S=6815232,E=6815235,C=73)
	CT(27639)=(S=6815236,E=6815239,C=0)
	CT(27640)=(S=6815240,E=6815243,C=1)
	CT(27641)=(S=6815244,E=6815251,C=11)
	CT(27642)=(S=6815252,E=6815255,C=1)
	CT(27643)=(S=6815256,E=6815256,C=0)
	CT(27644)=(S=6815257,E=6815257,C=74)
	CT(27645)=(S=6815258,E=6815258,C=84)
	CT(27646)=(S=6815259,E=6815267,C=4)
	CT(27647)=(S=6815268,E=6815271,C=10)
	CT(27648)=(S=6815272,E=6815275,C=11)
	CT(27649)=(S=6815276,E=6815279,C=7)
	CT(27650)=(S=6815280,E=6815281,C=74)
	CT(27651)=(S=6815282,E=6815282,C=0)
	CT(27652)=(S=6815283,E=6815283,C=74)
	CT(27653)=(S=6815284,E=6815291,C=4)
	CT(27654)=(S=6815292,E=6815295,C=2)
	CT(27655)=(S=6815296,E=6815311,C=1)
	CT(27656)=(S=6815312,E=6815315,C=4)
	CT(27657)=(S=6815316,E=6815319,C=84)
	CT(27658)=(S=6815320,E=6815325,C=0)
	CT(27659)=(S=6815326,E=6815326,C=84)
	CT(27660)=(S=6815327,E=6815327,C=74)
	CT(27661)=(S=6815328,E=6815335,C=4)
	CT(27662)=(S=6815336,E=6815339,C=74)
	CT(27663)=(S=6815340,E=6815343,C=8)
	CT(27664)=(S=6815344,E=6815347,C=1)
	CT(27665)=(S=6815348,E=6815351,C=8)
	CT(27666)=(S=6815352,E=6815355,C=92)
	CT(27667)=(S=6815356,E=6815357,C=4)
	CT(27668)=(S=6815358,E=6815359,C=74)
	CT(27669)=(S=6815360,E=6815363,C=4)
	CT(27670)=(S=6815364,E=6815367,C=73)
	CT(27671)=(S=6815368,E=6815375,C=0)
	CT(27672)=(S=6815376,E=6815379,C=2)
	CT(27673)=(S=6815380,E=6815383,C=8)
	CT(27674)=(S=6815384,E=6815387,C=7)
	CT(27675)=(S=6815388,E=6815391,C=73)
	CT(27676)=(S=6815392,E=6815395,C=4)
	CT(27677)=(S=6815396,E=6815397,C=0)
	CT(27678)=(S=6815398,E=6815398,C=89)
	CT(27679)=(S=6815399,E=6815399,C=84)
	CT(27680)=(S=6815400,E=6815403,C=74)
	CT(27681)=(S=6815404,E=6815407,C=4)
	CT(27682)=(S=6815408,E=6815411,C=1)
	CT(27683)=(S=6815412,E=6815419,C=75)
	CT(27684)=(S=6815420,E=6815423,C=1)
	CT(27685)=(S=6815426,E=6815427,C=73)
	CT(27686)=(S=6815428,E=6815428,C=1)
	CT(27687)=(S=6815432,E=6815439,C=4)
	CT(27688)=(S=6815440,E=6815443,C=8)
	CT(27689)=(S=6815444,E=6815447,C=7)
	CT(27690)=(S=6815448,E=6815451,C=11)
	CT(27691)=(S=6815452,E=6815455,C=1)
	CT(27692)=(S=6815456,E=6815459,C=164)
	CT(27693)=(S=6815460,E=6815461,C=73)
	CT(27694)=(S=6815462,E=6815462,C=4)
	CT(27695)=(S=6815463,E=6815463,C=73)
	CT(27696)=(S=6815464,E=6815467,C=164)
	CT(27697)=(S=6815468,E=6815479,C=4)
	CT(27698)=(S=6815484,E=6815487,C=8)
	CT(27699)=(S=6815488,E=6815491,C=2)
	CT(27700)=(S=6815492,E=6815495,C=77)
	CT(27701)=(S=6815496,E=6815499,C=4)
	CT(27702)=(S=6815500,E=6815500,C=74)
	CT(27703)=(S=6815501,E=6815501,C=3)
	CT(27704)=(S=6815502,E=6815503,C=74)
	CT(27705)=(S=6815504,E=6815507,C=0)
	CT(27706)=(S=6815508,E=6815511,C=2)
	CT(27707)=(S=6815512,E=6815515,C=73)
	CT(27708)=(S=6815516,E=6815516,C=4)
	CT(27709)=(S=6815517,E=6815517,C=74)
	CT(27710)=(S=6815518,E=6815518,C=84)
	CT(27711)=(S=6815519,E=6815519,C=0)
	CT(27712)=(S=6815520,E=6815523,C=7)
	CT(27713)=(S=6815524,E=6815531,C=4)
	CT(27714)=(S=6815532,E=6815535,C=8)
	CT(27715)=(S=6815536,E=6815539,C=0)
	CT(27716)=(S=6815540,E=6815541,C=74)
	CT(27717)=(S=6815542,E=6815543,C=0)
	CT(27718)=(S=6815548,E=6815551,C=8)
	CT(27719)=(S=6815552,E=6815555,C=2)
	CT(27720)=(S=6815556,E=6815559,C=1)
	CT(27721)=(S=6815560,E=6815571,C=4)
	CT(27722)=(S=6815572,E=6815575,C=11)
	CT(27723)=(S=6815576,E=6815583,C=1)
	CT(27724)=(S=6815584,E=6815595,C=4)
	CT(27725)=(S=6815596,E=6815599,C=77)
	CT(27726)=(S=6815600,E=6815603,C=4)
	CT(27727)=(S=6815604,E=6815607,C=86)
	CT(27728)=(S=6815608,E=6815613,C=74)
	CT(27729)=(S=6815614,E=6815614,C=75)
	CT(27730)=(S=6815615,E=6815615,C=8)
	CT(27731)=(S=6815616,E=6815619,C=77)
	CT(27732)=(S=6815624,E=6815631,C=1)
	CT(27733)=(S=6815632,E=6815635,C=4)
	CT(27734)=(S=6815636,E=6815639,C=77)
	CT(27735)=(S=6815640,E=6815643,C=10)
	CT(27736)=(S=6815644,E=6815645,C=74)
	CT(27737)=(S=6815646,E=6815657,C=4)
	CT(27738)=(S=6815658,E=6815659,C=5)
	CT(27739)=(S=6815660,E=6815663,C=155)
	CT(27740)=(S=6815664,E=6815667,C=8)
	CT(27741)=(S=6815668,E=6815671,C=4)
	CT(27742)=(S=6815672,E=6815675,C=1)
	CT(27743)=(S=6815676,E=6815687,C=4)
	CT(27744)=(S=6815688,E=6815691,C=1)
	CT(27745)=(S=6815692,E=6815695,C=4)
	CT(27746)=(S=6815696,E=6815703,C=1)
	CT(27747)=(S=6815704,E=6815715,C=4)
	CT(27748)=(S=6815716,E=6815719,C=1)
	CT(27749)=(S=6815720,E=6815723,C=4)
	CT(27750)=(S=6815724,E=6815727,C=11)
	CT(27751)=(S=6815728,E=6815731,C=74)
	CT(27752)=(S=6815732,E=6815735,C=2)
	CT(27753)=(S=6815736,E=6815737,C=73)
	CT(27754)=(S=6815738,E=6815738,C=7)
	CT(27755)=(S=6815739,E=6815739,C=0)
	CT(27756)=(S=6815740,E=6815743,C=8)
	CT(27757)=(S=6815744,E=6824967,C=29)
	CT(27758)=(S=6824968,E=6824975,C=78)
	CT(27759)=(S=6824976,E=6825059,C=29)
	CT(27760)=(S=6825060,E=6825063,C=78)
	CT(27761)=(S=6825064,E=6825075,C=29)
	CT(27762)=(S=6825076,E=6825079,C=78)
	CT(27763)=(S=6825080,E=6825107,C=29)
	CT(27764)=(S=6825108,E=6825111,C=78)
	CT(27765)=(S=6825112,E=6825235,C=29)
	CT(27766)=(S=6825236,E=6825239,C=78)
	CT(27767)=(S=6825240,E=6825275,C=29)
	CT(27768)=(S=6825276,E=6825283,C=78)
	CT(27769)=(S=6825284,E=6825287,C=29)
	CT(27770)=(S=6825288,E=6825291,C=78)
	CT(27771)=(S=6825292,E=6825295,C=29)
	CT(27772)=(S=6825296,E=6825299,C=78)
	CT(27773)=(S=6825300,E=6825327,C=29)
	CT(27774)=(S=6825328,E=6825331,C=78)
	CT(27775)=(S=6825332,E=6825363,C=29)
	CT(27776)=(S=6825364,E=6825367,C=78)
	CT(27777)=(S=6825368,E=6825407,C=29)
	CT(27778)=(S=6825408,E=6825423,C=78)
	CT(27779)=(S=6825424,E=6825455,C=29)
	CT(27780)=(S=6825456,E=6825459,C=78)
	CT(27781)=(S=6825460,E=6848591,C=29)
	CT(27782)=(S=6848592,E=6848607,C=78)
	CT(27783)=(S=6848608,E=6848655,C=29)
	CT(27784)=(S=6848656,E=6848671,C=78)
	CT(27785)=(S=6848672,E=6848751,C=29)
	CT(27786)=(S=6848752,E=6848767,C=78)
	CT(27787)=(S=6848768,E=6848863,C=29)
	CT(27788)=(S=6848864,E=6848895,C=78)
	CT(27789)=(S=6848896,E=6848991,C=29)
	CT(27790)=(S=6848992,E=6849007,C=78)
	CT(27791)=(S=6849008,E=6852095,C=29)
	CT(27792)=(S=6852096,E=6852223,C=78)
	CT(27793)=(S=6852224,E=6852463,C=29)
	CT(27794)=(S=6852464,E=6852479,C=95)
	CT(27795)=(S=6852480,E=6852863,C=29)
	CT(27796)=(S=6852864,E=6852879,C=78)
	CT(27797)=(S=6852880,E=6854659,C=29)
	CT(27798)=(S=6854660,E=6854663,C=78)
	CT(27799)=(S=6854664,E=6854683,C=29)
	CT(27800)=(S=6854684,E=6854687,C=78)
	CT(27801)=(S=6854688,E=6854775,C=29)
	CT(27802)=(S=6854776,E=6854779,C=78)
	CT(27803)=(S=6854780,E=6854807,C=29)
	CT(27804)=(S=6854808,E=6854811,C=78)
	CT(27805)=(S=6854812,E=6854815,C=29)
	CT(27806)=(S=6854816,E=6854827,C=78)
	CT(27807)=(S=6854828,E=6854863,C=29)
	CT(27808)=(S=6854864,E=6854867,C=78)
	CT(27809)=(S=6854868,E=6854879,C=29)
	CT(27810)=(S=6854880,E=6854883,C=78)
	CT(27811)=(S=6854884,E=6854891,C=29)
	CT(27812)=(S=6854892,E=6854895,C=175)
	CT(27813)=(S=6854896,E=6854935,C=29)
	CT(27814)=(S=6854936,E=6854939,C=78)
	CT(27815)=(S=6854940,E=6854959,C=29)
	CT(27816)=(S=6854960,E=6854963,C=78)
	CT(27817)=(S=6854964,E=6854967,C=29)
	CT(27818)=(S=6854968,E=6854971,C=78)
	CT(27819)=(S=6854972,E=6855039,C=29)
	CT(27820)=(S=6855040,E=6855047,C=79)
	CT(27821)=(S=6855048,E=6855055,C=78)
	CT(27822)=(S=6855056,E=6855075,C=29)
	CT(27823)=(S=6855076,E=6855079,C=78)
	CT(27824)=(S=6855080,E=6855123,C=29)
	CT(27825)=(S=6855124,E=6855127,C=78)
	CT(27826)=(S=6855128,E=6855159,C=29)
	CT(27827)=(S=6855160,E=6855163,C=182)
	CT(27828)=(S=6855164,E=6855711,C=29)
	CT(27829)=(S=6855712,E=6855727,C=78)
	CT(27830)=(S=6855728,E=6855743,C=29)
	CT(27831)=(S=6855744,E=6855759,C=78)
	CT(27832)=(S=6855760,E=6855935,C=29)
	CT(27833)=(S=6855936,E=6856575,C=78)
	CT(27834)=(S=6856576,E=6856735,C=29)
	CT(27835)=(S=6856736,E=6856751,C=78)
	CT(27836)=(S=6856752,E=6856895,C=29)
	CT(27837)=(S=6856896,E=6856927,C=78)
	CT(27838)=(S=6856928,E=6857471,C=29)
	CT(27839)=(S=6857472,E=6857727,C=78)
	CT(27840)=(S=6857728,E=6858271,C=29)
	CT(27841)=(S=6858272,E=6858287,C=81)
	CT(27842)=(S=6858288,E=6858591,C=29)
	CT(27843)=(S=6858592,E=6858623,C=78)
	CT(27844)=(S=6858624,E=6859567,C=29)
	CT(27845)=(S=6859568,E=6859583,C=78)
	CT(27846)=(S=6859584,E=6859711,C=29)
	CT(27847)=(S=6859712,E=6859727,C=78)
	CT(27848)=(S=6859728,E=6864911,C=29)
	CT(27849)=(S=6864912,E=6864919,C=78)
	CT(27850)=(S=6864920,E=6864939,C=29)
	CT(27851)=(S=6864940,E=6864943,C=78)
	CT(27852)=(S=6864944,E=6864947,C=29)
	CT(27853)=(S=6864948,E=6864951,C=78)
	CT(27854)=(S=6864952,E=6865011,C=29)
	CT(27855)=(S=6865012,E=6865023,C=78)
	CT(27856)=(S=6865024,E=6865027,C=29)
	CT(27857)=(S=6865028,E=6865031,C=78)
	CT(27858)=(S=6865032,E=6865039,C=29)
	CT(27859)=(S=6865040,E=6865043,C=78)
	CT(27860)=(S=6865044,E=6865047,C=29)
	CT(27861)=(S=6865048,E=6865051,C=78)
	CT(27862)=(S=6865052,E=6865067,C=29)
	CT(27863)=(S=6865068,E=6865071,C=78)
	CT(27864)=(S=6865072,E=6865079,C=29)
	CT(27865)=(S=6865080,E=6865087,C=180)
	CT(27866)=(S=6865088,E=6865127,C=29)
	CT(27867)=(S=6865128,E=6865131,C=78)
	CT(27868)=(S=6865132,E=6865183,C=29)
	CT(27869)=(S=6865184,E=6865191,C=78)
	CT(27870)=(S=6865192,E=6865199,C=29)
	CT(27871)=(S=6865200,E=6865223,C=78)
	CT(27872)=(S=6865224,E=6865243,C=29)
	CT(27873)=(S=6865244,E=6865247,C=78)
	CT(27874)=(S=6865248,E=6865255,C=29)
	CT(27875)=(S=6865256,E=6865259,C=78)
	CT(27876)=(S=6865260,E=6865283,C=29)
	CT(27877)=(S=6865284,E=6865287,C=78)
	CT(27878)=(S=6865288,E=6865323,C=29)
	CT(27879)=(S=6865324,E=6865327,C=78)
	CT(27880)=(S=6865328,E=6865347,C=29)
	CT(27881)=(S=6865348,E=6865351,C=197)
	CT(27882)=(S=6865352,E=6865359,C=29)
	CT(27883)=(S=6865360,E=6865367,C=78)
	CT(27884)=(S=6865368,E=6865439,C=29)
	CT(27885)=(S=6865440,E=6865503,C=78)
	CT(27886)=(S=6865504,E=6865791,C=29)
	CT(27887)=(S=6865792,E=6865919,C=78)
	CT(27888)=(S=6865920,E=6866943,C=29)
	CT(27889)=(S=6866944,E=6866959,C=78)
	CT(27890)=(S=6866960,E=6867023,C=29)
	CT(27891)=(S=6867024,E=6867039,C=78)
	CT(27892)=(S=6867040,E=6867055,C=79)
	CT(27893)=(S=6867056,E=6868223,C=29)
	CT(27894)=(S=6868224,E=6868479,C=78)
	CT(27895)=(S=6868480,E=6871595,C=29)
	CT(27896)=(S=6871596,E=6871599,C=78)
	CT(27897)=(S=6871600,E=6871607,C=29)
	CT(27898)=(S=6871608,E=6871615,C=78)
	CT(27899)=(S=6871616,E=6871635,C=29)
	CT(27900)=(S=6871636,E=6871639,C=78)
	CT(27901)=(S=6871640,E=6871643,C=29)
	CT(27902)=(S=6871644,E=6871647,C=78)
	CT(27903)=(S=6871648,E=6871675,C=29)
	CT(27904)=(S=6871676,E=6871679,C=78)
	CT(27905)=(S=6871680,E=6871719,C=29)
	CT(27906)=(S=6871720,E=6871727,C=178)
	CT(27907)=(S=6871728,E=6871731,C=79)
	CT(27908)=(S=6871732,E=6871767,C=29)
	CT(27909)=(S=6871768,E=6871771,C=82)
	CT(27910)=(S=6871772,E=6871791,C=29)
	CT(27911)=(S=6871792,E=6871795,C=80)
	CT(27912)=(S=6871796,E=6871815,C=29)
	CT(27913)=(S=6871816,E=6871823,C=78)
	CT(27914)=(S=6871824,E=6871831,C=29)
	CT(27915)=(S=6871832,E=6871835,C=82)
	CT(27916)=(S=6871836,E=6871839,C=80)
	CT(27917)=(S=6871840,E=6871879,C=29)
	CT(27918)=(S=6871880,E=6871883,C=95)
	CT(27919)=(S=6871884,E=6871907,C=29)
	CT(27920)=(S=6871908,E=6871911,C=78)
	CT(27921)=(S=6871912,E=6871927,C=29)
	CT(27922)=(S=6871928,E=6871943,C=78)
	CT(27923)=(S=6871944,E=6871963,C=29)
	CT(27924)=(S=6871964,E=6871967,C=78)
	CT(27925)=(S=6871968,E=6871995,C=29)
	CT(27926)=(S=6871996,E=6872003,C=78)
	CT(27927)=(S=6872004,E=6872007,C=29)
	CT(27928)=(S=6872008,E=6872015,C=78)
	CT(27929)=(S=6872016,E=6872027,C=29)
	CT(27930)=(S=6872028,E=6872031,C=78)
	CT(27931)=(S=6872032,E=6872319,C=29)
	CT(27932)=(S=6872320,E=6872447,C=78)
	CT(27933)=(S=6872448,E=6872687,C=29)
	CT(27934)=(S=6872688,E=6872703,C=78)
	CT(27935)=(S=6872704,E=6873183,C=29)
	CT(27936)=(S=6873184,E=6873279,C=78)
	CT(27937)=(S=6873280,E=6875183,C=29)
	CT(27938)=(S=6875184,E=6875199,C=78)
	CT(27939)=(S=6875200,E=6875455,C=29)
	CT(27940)=(S=6875456,E=6875519,C=78)
	CT(27941)=(S=6875520,E=6875647,C=29)
	CT(27942)=(S=6875648,E=6875903,C=78)
	CT(27943)=(S=6875904,E=6877567,C=29)
	CT(27944)=(S=6877568,E=6877647,C=78)
	CT(27945)=(S=6877648,E=6877695,C=29)
	CT(27946)=(S=6877696,E=6877951,C=78)
	CT(27947)=(S=6877952,E=6878031,C=29)
	CT(27948)=(S=6878032,E=6878063,C=78)
	CT(27949)=(S=6878064,E=6878271,C=29)
	CT(27950)=(S=6878272,E=6878279,C=78)
	CT(27951)=(S=6878280,E=6878387,C=29)
	CT(27952)=(S=6878388,E=6878391,C=80)
	CT(27953)=(S=6878392,E=6878395,C=29)
	CT(27954)=(S=6878396,E=6878399,C=78)
	CT(27955)=(S=6878400,E=6878403,C=29)
	CT(27956)=(S=6878404,E=6878407,C=78)
	CT(27957)=(S=6878408,E=6878431,C=29)
	CT(27958)=(S=6878432,E=6878439,C=175)
	CT(27959)=(S=6878440,E=6878451,C=29)
	CT(27960)=(S=6878452,E=6878455,C=78)
	CT(27961)=(S=6878456,E=6878463,C=29)
	CT(27962)=(S=6878464,E=6878471,C=78)
	CT(27963)=(S=6878472,E=6878511,C=29)
	CT(27964)=(S=6878512,E=6878515,C=186)
	CT(27965)=(S=6878516,E=6878527,C=29)
	CT(27966)=(S=6878528,E=6878531,C=78)
	CT(27967)=(S=6878532,E=6878551,C=29)
	CT(27968)=(S=6878552,E=6878555,C=95)
	CT(27969)=(S=6878556,E=6878559,C=186)
	CT(27970)=(S=6878560,E=6878575,C=29)
	CT(27971)=(S=6878576,E=6878583,C=191)
	CT(27972)=(S=6878584,E=6878607,C=29)
	CT(27973)=(S=6878608,E=6878611,C=78)
	CT(27974)=(S=6878612,E=6878615,C=29)
	CT(27975)=(S=6878616,E=6878631,C=78)
	CT(27976)=(S=6878632,E=6878663,C=29)
	CT(27977)=(S=6878664,E=6878667,C=78)
	CT(27978)=(S=6878668,E=6878671,C=182)
	CT(27979)=(S=6878672,E=6878687,C=29)
	CT(27980)=(S=6878688,E=6878691,C=78)
	CT(27981)=(S=6878692,E=6878847,C=29)
	CT(27982)=(S=6878848,E=6878975,C=78)
	CT(27983)=(S=6878976,E=6879199,C=29)
	CT(27984)=(S=6879200,E=6879231,C=78)
	CT(27985)=(S=6879232,E=6879679,C=29)
	CT(27986)=(S=6879680,E=6879743,C=78)
	CT(27987)=(S=6879744,E=6879775,C=198)
	CT(27988)=(S=6879776,E=6879807,C=29)
	CT(27989)=(S=6879808,E=6879839,C=78)
	CT(27990)=(S=6879840,E=6880031,C=29)
	CT(27991)=(S=6880032,E=6880047,C=78)
	CT(27992)=(S=6880048,E=6880095,C=29)
	CT(27993)=(S=6880096,E=6880111,C=78)
	CT(27994)=(S=6880112,E=6880775,C=29)
	CT(27995)=(S=6880776,E=6880779,C=78)
	CT(27996)=(S=6880780,E=6880799,C=29)
	CT(27997)=(S=6880800,E=6880803,C=78)
	CT(27998)=(S=6880804,E=6880815,C=29)
	CT(27999)=(S=6880816,E=6880819,C=78)
	CT(28000)=(S=6880820,E=6880855,C=29)
	CT(28001)=(S=6880856,E=6880863,C=78)
	CT(28002)=(S=6880864,E=6880867,C=95)
	CT(28003)=(S=6880868,E=6880919,C=29)
	CT(28004)=(S=6880920,E=6880927,C=78)
	CT(28005)=(S=6880928,E=6880947,C=29)
	CT(28006)=(S=6880948,E=6880951,C=78)
	CT(28007)=(S=6880952,E=6880955,C=29)
	CT(28008)=(S=6880956,E=6880959,C=78)
	CT(28009)=(S=6880960,E=6880963,C=29)
	CT(28010)=(S=6880964,E=6880967,C=78)
	CT(28011)=(S=6880968,E=6881031,C=29)
	CT(28012)=(S=6881032,E=6881039,C=78)
	CT(28013)=(S=6881040,E=6881071,C=29)
	CT(28014)=(S=6881072,E=6881075,C=78)
	CT(28015)=(S=6881076,E=6881159,C=29)
	CT(28016)=(S=6881160,E=6881167,C=95)
	CT(28017)=(S=6881168,E=6881175,C=29)
	CT(28018)=(S=6881176,E=6881183,C=78)
	CT(28019)=(S=6881184,E=6881199,C=29)
	CT(28020)=(S=6881200,E=6881203,C=197)
	CT(28021)=(S=6881204,E=6881207,C=80)
	CT(28022)=(S=6881208,E=6881231,C=29)
	CT(28023)=(S=6881232,E=6881235,C=78)
	CT(28024)=(S=6881236,E=6881255,C=29)
	CT(28025)=(S=6881256,E=6881259,C=82)
	CT(28026)=(S=6881260,E=6881275,C=29)
	CT(28027)=(S=6881276,E=6881279,C=82)
	CT(28028)=(S=6881280,E=6885375,C=105)
	CT(28029)=(S=6885376,E=6889471,C=113)
	CT(28030)=(S=6889472,E=6893567,C=106)
	CT(28031)=(S=6893568,E=6897663,C=109)
	CT(28032)=(S=6897664,E=6901759,C=134)
	CT(28033)=(S=6901760,E=6905855,C=106)
	CT(28034)=(S=6905856,E=6909951,C=135)
	CT(28035)=(S=6909952,E=6914047,C=111)
	CT(28036)=(S=6914048,E=6922239,C=134)
	CT(28037)=(S=6922240,E=6924287,C=109)
	CT(28038)=(S=6924288,E=6926335,C=116)
	CT(28039)=(S=6926336,E=6926847,C=105)
	CT(28040)=(S=6926848,E=6927359,C=128)
	CT(28041)=(S=6927360,E=6928383,C=106)
	CT(28042)=(S=6928384,E=6929407,C=105)
	CT(28043)=(S=6929408,E=6930431,C=134)
	CT(28044)=(S=6930432,E=6934527,C=106)
	CT(28045)=(S=6934528,E=6940159,C=105)
	CT(28046)=(S=6940160,E=6940671,C=109)
	CT(28047)=(S=6940672,E=6940927,C=117)
	CT(28048)=(S=6940928,E=6941183,C=105)
	CT(28049)=(S=6941184,E=6941439,C=121)
	CT(28050)=(S=6941440,E=6941567,C=118)
	CT(28051)=(S=6941568,E=6941583,C=135)
	CT(28052)=(S=6941584,E=6941599,C=113)
	CT(28053)=(S=6941600,E=6941615,C=105)
	CT(28054)=(S=6941616,E=6941631,C=139)
	CT(28055)=(S=6941632,E=6941647,C=111)
	CT(28056)=(S=6941648,E=6941655,C=144)
	CT(28057)=(S=6941656,E=6941663,C=131)
	CT(28058)=(S=6941664,E=6941679,C=146)
	CT(28059)=(S=6941680,E=6941695,C=126)
	CT(28060)=(S=6941696,E=6942207,C=105)
	CT(28061)=(S=6942208,E=6942719,C=119)
	CT(28062)=(S=6942720,E=6946815,C=105)
	CT(28063)=(S=6946816,E=6946816,C=1)
	CT(28064)=(S=6946817,E=6946817,C=0)
	CT(28065)=(S=6946818,E=6946943,C=1)
	CT(28066)=(S=6946944,E=6947071,C=3)
	CT(28067)=(S=6947072,E=6947327,C=9)
	CT(28068)=(S=6947328,E=6949375,C=1)
	CT(28069)=(S=6949376,E=6949503,C=6)
	CT(28070)=(S=6949504,E=6949631,C=7)
	CT(28071)=(S=6949632,E=6959871,C=1)
	CT(28072)=(S=6959872,E=6960127,C=4)
	CT(28073)=(S=6960128,E=6963199,C=1)
	CT(28074)=(S=6963200,E=6963711,C=9)
	CT(28075)=(S=6963712,E=6964223,C=4)
	CT(28076)=(S=6964224,E=6965247,C=0)
	CT(28077)=(S=6965248,E=6965759,C=2)
	CT(28078)=(S=6965760,E=6966271,C=1)
	CT(28079)=(S=6966272,E=6967295,C=4)
	CT(28080)=(S=6967296,E=6971391,C=1)
	CT(28081)=(S=6971392,E=6973439,C=6)
	CT(28082)=(S=6973440,E=6974463,C=9)
	CT(28083)=(S=6974464,E=6979583,C=1)
	CT(28084)=(S=6979584,E=6995967,C=2)
	CT(28085)=(S=6995968,E=7004159,C=4)
	CT(28086)=(S=7004160,E=7008255,C=1)
	CT(28087)=(S=7008256,E=7012351,C=6)
	CT(28088)=(S=7012352,E=7050975,C=29)
	CT(28089)=(S=7050976,E=7051007,C=78)
	CT(28090)=(S=7051008,E=7051343,C=29)
	CT(28091)=(S=7051344,E=7051359,C=78)
	CT(28092)=(S=7051360,E=7053055,C=29)
	CT(28093)=(S=7053056,E=7053311,C=78)
	CT(28094)=(S=7053312,E=7053567,C=29)
	CT(28095)=(S=7053568,E=7053583,C=78)
	CT(28096)=(S=7053584,E=7053599,C=29)
	CT(28097)=(S=7053600,E=7053615,C=78)
	CT(28098)=(S=7053616,E=7053631,C=29)
	CT(28099)=(S=7053632,E=7053647,C=78)
	CT(28100)=(S=7053648,E=7053663,C=29)
	CT(28101)=(S=7053664,E=7053679,C=78)
	CT(28102)=(S=7053680,E=7056255,C=29)
	CT(28103)=(S=7056256,E=7056383,C=78)
	CT(28104)=(S=7056384,E=7058303,C=29)
	CT(28105)=(S=7058304,E=7058431,C=78)
	CT(28106)=(S=7058432,E=7059119,C=29)
	CT(28107)=(S=7059120,E=7059135,C=78)
	CT(28108)=(S=7059136,E=7060927,C=29)
	CT(28109)=(S=7060928,E=7061119,C=78)
	CT(28110)=(S=7061120,E=7061455,C=29)
	CT(28111)=(S=7061456,E=7061471,C=198)
	CT(28112)=(S=7061472,E=7093055,C=29)
	CT(28113)=(S=7093056,E=7093071,C=78)
	CT(28114)=(S=7093072,E=7093247,C=29)
	CT(28115)=(S=7093248,E=7093279,C=78)
	CT(28116)=(S=7093280,E=7093407,C=29)
	CT(28117)=(S=7093408,E=7093439,C=78)
	CT(28118)=(S=7093440,E=7093471,C=29)
	CT(28119)=(S=7093472,E=7093503,C=81)
	CT(28120)=(S=7093504,E=7094015,C=29)
	CT(28121)=(S=7094016,E=7094271,C=78)
	CT(28122)=(S=7094272,E=7118863,C=29)
	CT(28123)=(S=7118864,E=7118879,C=78)
	CT(28124)=(S=7118880,E=7118927,C=29)
	CT(28125)=(S=7118928,E=7118959,C=78)
	CT(28126)=(S=7118960,E=7119087,C=29)
	CT(28127)=(S=7119088,E=7119103,C=78)
	CT(28128)=(S=7119104,E=7119135,C=29)
	CT(28129)=(S=7119136,E=7119151,C=78)
	CT(28130)=(S=7119152,E=7119215,C=29)
	CT(28131)=(S=7119216,E=7119231,C=78)
	CT(28132)=(S=7119232,E=7119263,C=29)
	CT(28133)=(S=7119264,E=7119279,C=78)
	CT(28134)=(S=7119280,E=7119423,C=29)
	CT(28135)=(S=7119424,E=7119551,C=78)
	CT(28136)=(S=7119552,E=7119743,C=29)
	CT(28137)=(S=7119744,E=7119807,C=78)
	CT(28138)=(S=7119808,E=7120895,C=29)
	CT(28139)=(S=7120896,E=7121023,C=78)
	CT(28140)=(S=7121024,E=7121535,C=29)
	CT(28141)=(S=7121536,E=7121599,C=78)
	CT(28142)=(S=7121600,E=7121727,C=29)
	CT(28143)=(S=7121728,E=7121759,C=78)
	CT(28144)=(S=7121760,E=7121919,C=29)
	CT(28145)=(S=7121920,E=7122431,C=78)
	CT(28146)=(S=7122432,E=7122447,C=29)
	CT(28147)=(S=7122448,E=7122463,C=78)
	CT(28148)=(S=7122464,E=7122559,C=29)
	CT(28149)=(S=7122560,E=7122575,C=78)
	CT(28150)=(S=7122576,E=7122591,C=29)
	CT(28151)=(S=7122592,E=7122607,C=78)
	CT(28152)=(S=7122608,E=7122767,C=29)
	CT(28153)=(S=7122768,E=7122783,C=78)
	CT(28154)=(S=7122784,E=7122911,C=29)
	CT(28155)=(S=7122912,E=7122927,C=78)
	CT(28156)=(S=7122928,E=7123967,C=29)
	CT(28157)=(S=7123968,E=7124479,C=78)
	CT(28158)=(S=7124480,E=7143423,C=29)
	CT(28159)=(S=7143424,E=7151615,C=12)
	CT(28160)=(S=7151616,E=7153663,C=36)
	CT(28161)=(S=7153664,E=7155711,C=28)
	CT(28162)=(S=7155712,E=7156735,C=21)
	CT(28163)=(S=7156736,E=7157759,C=15)
	CT(28164)=(S=7157760,E=7158783,C=19)
	CT(28165)=(S=7158784,E=7158911,C=57)
	CT(28166)=(S=7158912,E=7159039,C=18)
	CT(28167)=(S=7159040,E=7159167,C=50)
	CT(28168)=(S=7159168,E=7159295,C=18)
	CT(28169)=(S=7159296,E=7159423,C=12)
	CT(28170)=(S=7159424,E=7159551,C=18)
	CT(28171)=(S=7159552,E=7159679,C=104)
	CT(28172)=(S=7159680,E=7159807,C=18)
	CT(28173)=(S=7159808,E=7160831,C=17)
	CT(28174)=(S=7160832,E=7160847,C=14)
	CT(28175)=(S=7160848,E=7160855,C=18)
	CT(28176)=(S=7160856,E=7160863,C=15)
	CT(28177)=(S=7160864,E=7160871,C=14)
	CT(28178)=(S=7160872,E=7160879,C=31)
	CT(28179)=(S=7160880,E=7160887,C=28)
	CT(28180)=(S=7160888,E=7160895,C=36)
	CT(28181)=(S=7160896,E=7160903,C=14)
	CT(28182)=(S=7160904,E=7160911,C=18)
	CT(28183)=(S=7160912,E=7160919,C=25)
	CT(28184)=(S=7160920,E=7160927,C=27)
	CT(28185)=(S=7160928,E=7160935,C=28)
	CT(28186)=(S=7160936,E=7160943,C=42)
	CT(28187)=(S=7160944,E=7160951,C=18)
	CT(28188)=(S=7160952,E=7160959,C=66)
	CT(28189)=(S=7160960,E=7160967,C=47)
	CT(28190)=(S=7160968,E=7160975,C=18)
	CT(28191)=(S=7160976,E=7160983,C=22)
	CT(28192)=(S=7160984,E=7160991,C=15)
	CT(28193)=(S=7160992,E=7160999,C=49)
	CT(28194)=(S=7161000,E=7161007,C=18)
	CT(28195)=(S=7161008,E=7161015,C=27)
	CT(28196)=(S=7161016,E=7161023,C=18)
	CT(28197)=(S=7161024,E=7161031,C=14)
	CT(28198)=(S=7161032,E=7161047,C=18)
	CT(28199)=(S=7161048,E=7161063,C=28)
	CT(28200)=(S=7161064,E=7161071,C=103)
	CT(28201)=(S=7161072,E=7161079,C=14)
	CT(28202)=(S=7161080,E=7161087,C=47)
	CT(28203)=(S=7161088,E=7161095,C=96)
	CT(28204)=(S=7161096,E=7161103,C=25)
	CT(28205)=(S=7161104,E=7161119,C=18)
	CT(28206)=(S=7161120,E=7161127,C=47)
	CT(28207)=(S=7161128,E=7161135,C=54)
	CT(28208)=(S=7161136,E=7161143,C=14)
	CT(28209)=(S=7161144,E=7161151,C=33)
	CT(28210)=(S=7161152,E=7161159,C=28)
	CT(28211)=(S=7161160,E=7161167,C=18)
	CT(28212)=(S=7161168,E=7161175,C=14)
	CT(28213)=(S=7161176,E=7161183,C=58)
	CT(28214)=(S=7161184,E=7161191,C=28)
	CT(28215)=(S=7161192,E=7161199,C=14)
	CT(28216)=(S=7161200,E=7161207,C=50)
	CT(28217)=(S=7161208,E=7161215,C=47)
	CT(28218)=(S=7161216,E=7161223,C=15)
	CT(28219)=(S=7161224,E=7161231,C=13)
	CT(28220)=(S=7161232,E=7161239,C=15)
	CT(28221)=(S=7161240,E=7161247,C=34)
	CT(28222)=(S=7161248,E=7161255,C=96)
	CT(28223)=(S=7161256,E=7161263,C=42)
	CT(28224)=(S=7161264,E=7161271,C=18)
	CT(28225)=(S=7161272,E=7161287,C=12)
	CT(28226)=(S=7161288,E=7161295,C=15)
	CT(28227)=(S=7161296,E=7161303,C=33)
	CT(28228)=(S=7161304,E=7161311,C=49)
	CT(28229)=(S=7161312,E=7161319,C=36)
	CT(28230)=(S=7161320,E=7161327,C=14)
	CT(28231)=(S=7161328,E=7161335,C=70)
	CT(28232)=(S=7161336,E=7161343,C=42)
	CT(28233)=(S=7161344,E=7161359,C=36)
	CT(28234)=(S=7161360,E=7161367,C=12)
	CT(28235)=(S=7161368,E=7161375,C=18)
	CT(28236)=(S=7161376,E=7161383,C=25)
	CT(28237)=(S=7161384,E=7161391,C=14)
	CT(28238)=(S=7161392,E=7161399,C=24)
	CT(28239)=(S=7161400,E=7161407,C=14)
	CT(28240)=(S=7161408,E=7161415,C=16)
	CT(28241)=(S=7161416,E=7161423,C=15)
	CT(28242)=(S=7161424,E=7161431,C=27)
	CT(28243)=(S=7161432,E=7161439,C=29)
	CT(28244)=(S=7161440,E=7161447,C=42)
	CT(28245)=(S=7161448,E=7161455,C=31)
	CT(28246)=(S=7161456,E=7161463,C=34)
	CT(28247)=(S=7161464,E=7161471,C=15)
	CT(28248)=(S=7161472,E=7161479,C=25)
	CT(28249)=(S=7161480,E=7161495,C=14)
	CT(28250)=(S=7161496,E=7161503,C=34)
	CT(28251)=(S=7161504,E=7161511,C=47)
	CT(28252)=(S=7161512,E=7161519,C=12)
	CT(28253)=(S=7161520,E=7161527,C=47)
	CT(28254)=(S=7161528,E=7161535,C=18)
	CT(28255)=(S=7161536,E=7161543,C=28)
	CT(28256)=(S=7161544,E=7161551,C=15)
	CT(28257)=(S=7161552,E=7161559,C=63)
	CT(28258)=(S=7161560,E=7161567,C=28)
	CT(28259)=(S=7161568,E=7161575,C=14)
	CT(28260)=(S=7161576,E=7161583,C=18)
	CT(28261)=(S=7161584,E=7161591,C=15)
	CT(28262)=(S=7161592,E=7161599,C=18)
	CT(28263)=(S=7161600,E=7161607,C=36)
	CT(28264)=(S=7161608,E=7161615,C=18)
	CT(28265)=(S=7161616,E=7161631,C=28)
	CT(28266)=(S=7161632,E=7161639,C=20)
	CT(28267)=(S=7161640,E=7161647,C=21)
	CT(28268)=(S=7161648,E=7161655,C=36)
	CT(28269)=(S=7161656,E=7161663,C=24)
	CT(28270)=(S=7161664,E=7161671,C=71)
	CT(28271)=(S=7161680,E=7161687,C=14)
	CT(28272)=(S=7161688,E=7161703,C=34)
	CT(28273)=(S=7161704,E=7161711,C=36)
	CT(28274)=(S=7161712,E=7161719,C=28)
	CT(28275)=(S=7161720,E=7161727,C=14)
	CT(28276)=(S=7161728,E=7161743,C=12)
	CT(28277)=(S=7161744,E=7161751,C=14)
	CT(28278)=(S=7161760,E=7161767,C=21)
	CT(28279)=(S=7161768,E=7161775,C=14)
	CT(28280)=(S=7161776,E=7161783,C=18)
	CT(28281)=(S=7161784,E=7161791,C=36)
	CT(28282)=(S=7161792,E=7161807,C=18)
	CT(28283)=(S=7161808,E=7161815,C=33)
	CT(28284)=(S=7161816,E=7161823,C=28)
	CT(28285)=(S=7161824,E=7161831,C=62)
	CT(28286)=(S=7161832,E=7161839,C=15)
	CT(28287)=(S=7161840,E=7161847,C=18)
	CT(28288)=(S=7161856,E=7161871,C=33)
	CT(28289)=(S=7161872,E=7161887,C=14)
	CT(28290)=(S=7161888,E=7161903,C=36)
	CT(28291)=(S=7161904,E=7161919,C=46)
	CT(28292)=(S=7161920,E=7161935,C=18)
	CT(28293)=(S=7161936,E=7161951,C=36)
	CT(28294)=(S=7161952,E=7161967,C=102)
	CT(28295)=(S=7161968,E=7161983,C=31)
	CT(28296)=(S=7161984,E=7161999,C=34)
	CT(28297)=(S=7162000,E=7162015,C=13)
	CT(28298)=(S=7162016,E=7162047,C=36)
	CT(28299)=(S=7162048,E=7162063,C=26)
	CT(28300)=(S=7162064,E=7162079,C=14)
	CT(28301)=(S=7162080,E=7162127,C=18)
	CT(28302)=(S=7162128,E=7162143,C=28)
	CT(28303)=(S=7162144,E=7162159,C=18)
	CT(28304)=(S=7162160,E=7162175,C=28)
	CT(28305)=(S=7162176,E=7162191,C=14)
	CT(28306)=(S=7162192,E=7162207,C=15)
	CT(28307)=(S=7162208,E=7162223,C=60)
	CT(28308)=(S=7162224,E=7162239,C=14)
	CT(28309)=(S=7162256,E=7162271,C=42)
	CT(28310)=(S=7162272,E=7162287,C=14)
	CT(28311)=(S=7162288,E=7162303,C=15)
	CT(28312)=(S=7162304,E=7162319,C=18)
	CT(28313)=(S=7162320,E=7162335,C=33)
	CT(28314)=(S=7162336,E=7162351,C=51)
	CT(28315)=(S=7162352,E=7162367,C=43)
	CT(28316)=(S=7162368,E=7162383,C=19)
	CT(28317)=(S=7162384,E=7162399,C=36)
	CT(28318)=(S=7162400,E=7162415,C=67)
	CT(28319)=(S=7162416,E=7162431,C=50)
	CT(28320)=(S=7162432,E=7162447,C=103)
	CT(28321)=(S=7162448,E=7162463,C=12)
	CT(28322)=(S=7162464,E=7162479,C=58)
	CT(28323)=(S=7162480,E=7162511,C=18)
	CT(28324)=(S=7162512,E=7162527,C=71)
	CT(28325)=(S=7162528,E=7162543,C=18)
	CT(28326)=(S=7162544,E=7162559,C=27)
	CT(28327)=(S=7162560,E=7162575,C=14)
	CT(28328)=(S=7162576,E=7162591,C=18)
	CT(28329)=(S=7162592,E=7162607,C=26)
	CT(28330)=(S=7162608,E=7162623,C=14)
	CT(28331)=(S=7162624,E=7162639,C=33)
	CT(28332)=(S=7162640,E=7162655,C=28)
	CT(28333)=(S=7162656,E=7162671,C=66)
	CT(28334)=(S=7162672,E=7162687,C=103)
	CT(28335)=(S=7162688,E=7162703,C=39)
	CT(28336)=(S=7162704,E=7162735,C=28)
	CT(28337)=(S=7162736,E=7162751,C=15)
	CT(28338)=(S=7162752,E=7162767,C=18)
	CT(28339)=(S=7162768,E=7162783,C=33)
	CT(28340)=(S=7162784,E=7162799,C=14)
	CT(28341)=(S=7162800,E=7162815,C=28)
	CT(28342)=(S=7162816,E=7162831,C=31)
	CT(28343)=(S=7162832,E=7162847,C=28)
	CT(28344)=(S=7162848,E=7162863,C=47)
	CT(28345)=(S=7162864,E=7162879,C=18)
	CT(28346)=(S=7162880,E=7163903,C=63)
	CT(28347)=(S=7163904,E=7164415,C=33)
	CT(28348)=(S=7164416,E=7164927,C=23)
	CT(28349)=(S=7164928,E=7165439,C=28)
	CT(28350)=(S=7165440,E=7165951,C=31)
	CT(28351)=(S=7165952,E=7166463,C=49)
	CT(28352)=(S=7166464,E=7166975,C=28)
	CT(28353)=(S=7166976,E=7167487,C=46)
	CT(28354)=(S=7167488,E=7167519,C=18)
	CT(28355)=(S=7167520,E=7167551,C=12)
	CT(28356)=(S=7167552,E=7167583,C=18)
	CT(28357)=(S=7167584,E=7167615,C=46)
	CT(28358)=(S=7167616,E=7167631,C=14)
	CT(28359)=(S=7167632,E=7167647,C=38)
	CT(28360)=(S=7167648,E=7167663,C=31)
	CT(28361)=(S=7167664,E=7167695,C=18)
	CT(28362)=(S=7167696,E=7167711,C=31)
	CT(28363)=(S=7167712,E=7167727,C=46)
	CT(28364)=(S=7167728,E=7167751,C=58)
	CT(28365)=(S=7167752,E=7167759,C=38)
	CT(28366)=(S=7167760,E=7167767,C=18)
	CT(28367)=(S=7167768,E=7167775,C=58)
	CT(28368)=(S=7167776,E=7167799,C=31)
	CT(28369)=(S=7167800,E=7167815,C=26)
	CT(28370)=(S=7167816,E=7167831,C=18)
	CT(28371)=(S=7167832,E=7167839,C=58)
	CT(28372)=(S=7167840,E=7167855,C=18)
	CT(28373)=(S=7167856,E=7167863,C=58)
	CT(28374)=(S=7167872,E=7167879,C=17)
	CT(28375)=(S=7167880,E=7167903,C=58)
	CT(28376)=(S=7167904,E=7167911,C=18)
	CT(28377)=(S=7167912,E=7167927,C=58)
	CT(28378)=(S=7167928,E=7167935,C=14)
	CT(28379)=(S=7167936,E=7167943,C=18)
	CT(28380)=(S=7167944,E=7167951,C=58)
	CT(28381)=(S=7167952,E=7167975,C=18)
	CT(28382)=(S=7167976,E=7167983,C=58)
	CT(28383)=(S=7167984,E=7167991,C=34)
	CT(28384)=(S=7167992,E=7167999,C=18)
	CT(28385)=(S=7168000,E=7170047,C=38)
	CT(28386)=(S=7170048,E=7170079,C=19)
	CT(28387)=(S=7170080,E=7170111,C=28)
	CT(28388)=(S=7170112,E=7170175,C=14)
	CT(28389)=(S=7170176,E=7170207,C=96)
	CT(28390)=(S=7170208,E=7170239,C=31)
	CT(28391)=(S=7170240,E=7170271,C=51)
	CT(28392)=(S=7170272,E=7170303,C=15)
	CT(28393)=(S=7170304,E=7170335,C=50)
	CT(28394)=(S=7170336,E=7170367,C=33)
	CT(28395)=(S=7170368,E=7170399,C=18)
	CT(28396)=(S=7170400,E=7170431,C=19)
	CT(28397)=(S=7170432,E=7170495,C=18)
	CT(28398)=(S=7170496,E=7170527,C=59)
	CT(28399)=(S=7170528,E=7170559,C=32)
	CT(28400)=(S=7170560,E=7170591,C=31)
	CT(28401)=(S=7170592,E=7170623,C=34)
	CT(28402)=(S=7170624,E=7170655,C=12)
	CT(28403)=(S=7170656,E=7170687,C=63)
	CT(28404)=(S=7170688,E=7170719,C=18)
	CT(28405)=(S=7170720,E=7170751,C=13)
	CT(28406)=(S=7170752,E=7170783,C=18)
	CT(28407)=(S=7170784,E=7170815,C=46)
	CT(28408)=(S=7170816,E=7170847,C=58)
	CT(28409)=(S=7170848,E=7170879,C=14)
	CT(28410)=(S=7170880,E=7170911,C=51)
	CT(28411)=(S=7170912,E=7170943,C=25)
	CT(28412)=(S=7170944,E=7170975,C=61)
	CT(28413)=(S=7170976,E=7171007,C=18)
	CT(28414)=(S=7171008,E=7171039,C=33)
	CT(28415)=(S=7171040,E=7171071,C=61)
	CT(28416)=(S=7171072,E=7171103,C=47)
	CT(28417)=(S=7171104,E=7171135,C=18)
	CT(28418)=(S=7171136,E=7171167,C=31)
	CT(28419)=(S=7171168,E=7171199,C=33)
	CT(28420)=(S=7171200,E=7171231,C=14)
	CT(28421)=(S=7171232,E=7171263,C=26)
	CT(28422)=(S=7171264,E=7171295,C=27)
	CT(28423)=(S=7171296,E=7171327,C=31)
	CT(28424)=(S=7171328,E=7171359,C=28)
	CT(28425)=(S=7171360,E=7171391,C=26)
	CT(28426)=(S=7171392,E=7171423,C=27)
	CT(28427)=(S=7171424,E=7171455,C=36)
	CT(28428)=(S=7171456,E=7171519,C=14)
	CT(28429)=(S=7171520,E=7171527,C=46)
	CT(28430)=(S=7171528,E=7171535,C=28)
	CT(28431)=(S=7171536,E=7171551,C=18)
	CT(28432)=(S=7171552,E=7171583,C=14)
	CT(28433)=(S=7171584,E=7171615,C=60)
	CT(28434)=(S=7171616,E=7171647,C=18)
	CT(28435)=(S=7171648,E=7171679,C=31)
	CT(28436)=(S=7171680,E=7171711,C=39)
	CT(28437)=(S=7171712,E=7171743,C=50)
	CT(28438)=(S=7171744,E=7171775,C=26)
	CT(28439)=(S=7171776,E=7171807,C=58)
	CT(28440)=(S=7171808,E=7171839,C=32)
	CT(28441)=(S=7171840,E=7171871,C=18)
	CT(28442)=(S=7171872,E=7171903,C=26)
	CT(28443)=(S=7171904,E=7171935,C=18)
	CT(28444)=(S=7171936,E=7171967,C=188)
	CT(28445)=(S=7171968,E=7172031,C=18)
	CT(28446)=(S=7172032,E=7172063,C=14)
	CT(28447)=(S=7172064,E=7172095,C=46)
	CT(28448)=(S=7172096,E=7174143,C=15)
	CT(28449)=(S=7174144,E=7174207,C=18)
	CT(28450)=(S=7174208,E=7174271,C=70)
	CT(28451)=(S=7174272,E=7174335,C=18)
	CT(28452)=(S=7174336,E=7174399,C=51)
	CT(28453)=(S=7174400,E=7174527,C=46)
	CT(28454)=(S=7174528,E=7174655,C=51)
	CT(28455)=(S=7174656,E=7174719,C=31)
	CT(28456)=(S=7174720,E=7174783,C=46)
	CT(28457)=(S=7174784,E=7174847,C=12)
	CT(28458)=(S=7174848,E=7174911,C=26)
	CT(28459)=(S=7174912,E=7174975,C=52)
	CT(28460)=(S=7174976,E=7175039,C=14)
	CT(28461)=(S=7175040,E=7175103,C=18)
	CT(28462)=(S=7175104,E=7175167,C=33)
	CT(28463)=(S=7175168,E=7175295,C=18)
	CT(28464)=(S=7175296,E=7175359,C=19)
	CT(28465)=(S=7175360,E=7175423,C=18)
	CT(28466)=(S=7175424,E=7175487,C=63)
	CT(28467)=(S=7175488,E=7175551,C=28)
	CT(28468)=(S=7175552,E=7175615,C=26)
	CT(28469)=(S=7175616,E=7175679,C=58)
	CT(28470)=(S=7175680,E=7175743,C=18)
	CT(28471)=(S=7175744,E=7175807,C=42)
	CT(28472)=(S=7175808,E=7175871,C=68)
	CT(28473)=(S=7175872,E=7175935,C=18)
	CT(28474)=(S=7175936,E=7175999,C=41)
	CT(28475)=(S=7176000,E=7176063,C=35)
	CT(28476)=(S=7176064,E=7176127,C=18)
	CT(28477)=(S=7176128,E=7176191,C=52)
	CT(28478)=(S=7176192,E=7180287,C=49)
	CT(28479)=(S=7180288,E=7184383,C=14)
	CT(28480)=(S=7184384,E=7184511,C=51)
	CT(28481)=(S=7184512,E=7184639,C=17)
	CT(28482)=(S=7184640,E=7184767,C=18)
	CT(28483)=(S=7184768,E=7184895,C=104)
	CT(28484)=(S=7184896,E=7185023,C=31)
	CT(28485)=(S=7185024,E=7185151,C=26)
	CT(28486)=(S=7185152,E=7185279,C=27)
	CT(28487)=(S=7185280,E=7185343,C=59)
	CT(28488)=(S=7185344,E=7185351,C=20)
	CT(28489)=(S=7185352,E=7185359,C=27)
	CT(28490)=(S=7185360,E=7185367,C=47)
	CT(28491)=(S=7185368,E=7185375,C=18)
	CT(28492)=(S=7185376,E=7185391,C=38)
	CT(28493)=(S=7185392,E=7185399,C=18)
	CT(28494)=(S=7185400,E=7185407,C=47)
	CT(28495)=(S=7185408,E=7185535,C=33)
	CT(28496)=(S=7185536,E=7185663,C=34)
	CT(28497)=(S=7185664,E=7185791,C=18)
	CT(28498)=(S=7185792,E=7185919,C=59)
	CT(28499)=(S=7185920,E=7185951,C=23)
	CT(28500)=(S=7185952,E=7186007,C=38)
	CT(28501)=(S=7186008,E=7186047,C=23)
	CT(28502)=(S=7186048,E=7186175,C=38)
	CT(28503)=(S=7186176,E=7186303,C=25)
	CT(28504)=(S=7186304,E=7186431,C=18)
	CT(28505)=(S=7186432,E=7186559,C=15)
	CT(28506)=(S=7186560,E=7186687,C=18)
	CT(28507)=(S=7186688,E=7186783,C=14)
	CT(28508)=(S=7186784,E=7186815,C=27)
	CT(28509)=(S=7186816,E=7187071,C=18)
	CT(28510)=(S=7187072,E=7187199,C=14)
	CT(28511)=(S=7187200,E=7187327,C=18)
	CT(28512)=(S=7187328,E=7187455,C=23)
	CT(28513)=(S=7187456,E=7187583,C=18)
	CT(28514)=(S=7187584,E=7187711,C=40)
	CT(28515)=(S=7187712,E=7187839,C=18)
	CT(28516)=(S=7187840,E=7187967,C=58)
	CT(28517)=(S=7187968,E=7188095,C=18)
	CT(28518)=(S=7188096,E=7188223,C=14)
	CT(28519)=(S=7188224,E=7188351,C=59)
	CT(28520)=(S=7188352,E=7188735,C=14)
	CT(28521)=(S=7188736,E=7188991,C=16)
	CT(28522)=(S=7188992,E=7189247,C=22)
	CT(28523)=(S=7189248,E=7189503,C=27)
	CT(28524)=(S=7189504,E=7190015,C=14)
	CT(28525)=(S=7190016,E=7190271,C=52)
	CT(28526)=(S=7190272,E=7190527,C=33)
	CT(28527)=(S=7190528,E=7190783,C=18)
	CT(28528)=(S=7190784,E=7191039,C=53)
	CT(28529)=(S=7191040,E=7191295,C=17)
	CT(28530)=(S=7191296,E=7191807,C=18)
	CT(28531)=(S=7191808,E=7192063,C=27)
	CT(28532)=(S=7192064,E=7192319,C=12)
	CT(28533)=(S=7192320,E=7192575,C=18)
	CT(28534)=(S=7192576,E=7193087,C=28)
	CT(28535)=(S=7193088,E=7193599,C=18)
	CT(28536)=(S=7193600,E=7193615,C=58)
	CT(28537)=(S=7193616,E=7193631,C=18)
	CT(28538)=(S=7193632,E=7193663,C=58)
	CT(28539)=(S=7193664,E=7193679,C=18)
	CT(28540)=(S=7193680,E=7193695,C=58)
	CT(28541)=(S=7193696,E=7193711,C=18)
	CT(28542)=(S=7193712,E=7193727,C=58)
	CT(28543)=(S=7193728,E=7193743,C=18)
	CT(28544)=(S=7193744,E=7193759,C=58)
	CT(28545)=(S=7193776,E=7193791,C=31)
	CT(28546)=(S=7193792,E=7193823,C=18)
	CT(28547)=(S=7193824,E=7193855,C=58)
	CT(28548)=(S=7193856,E=7193863,C=27)
	CT(28549)=(S=7193864,E=7193887,C=18)
	CT(28550)=(S=7193888,E=7193903,C=58)
	CT(28551)=(S=7193904,E=7193911,C=18)
	CT(28552)=(S=7193912,E=7193927,C=58)
	CT(28553)=(S=7193928,E=7193935,C=18)
	CT(28554)=(S=7193936,E=7193943,C=38)
	CT(28555)=(S=7193944,E=7193951,C=18)
	CT(28556)=(S=7193952,E=7193959,C=58)
	CT(28557)=(S=7193960,E=7193967,C=12)
	CT(28558)=(S=7193968,E=7194015,C=18)
	CT(28559)=(S=7194016,E=7194017,C=19)
	CT(28560)=(S=7194018,E=7194019,C=18)
	CT(28561)=(S=7194020,E=7194020,C=58)
	CT(28562)=(S=7194021,E=7194021,C=49)
	CT(28563)=(S=7194022,E=7194023,C=31)
	CT(28564)=(S=7194024,E=7194031,C=58)
	CT(28565)=(S=7194032,E=7194039,C=12)
	CT(28566)=(S=7194040,E=7194047,C=58)
	CT(28567)=(S=7194048,E=7194063,C=18)
	CT(28568)=(S=7194064,E=7194071,C=60)
	CT(28569)=(S=7194072,E=7194079,C=31)
	CT(28570)=(S=7194080,E=7194087,C=18)
	CT(28571)=(S=7194088,E=7194095,C=38)
	CT(28572)=(S=7194096,E=7194103,C=12)
	CT(28573)=(S=7194104,E=7194111,C=18)
	CT(28574)=(S=7194112,E=7194143,C=46)
	CT(28575)=(S=7194144,E=7194175,C=53)
	CT(28576)=(S=7194176,E=7194239,C=18)
	CT(28577)=(S=7194240,E=7194271,C=24)
	CT(28578)=(S=7194272,E=7194367,C=18)
	CT(28579)=(S=7194368,E=7194399,C=58)
	CT(28580)=(S=7194400,E=7194431,C=50)
	CT(28581)=(S=7194432,E=7194463,C=58)
	CT(28582)=(S=7194464,E=7194527,C=51)
	CT(28583)=(S=7194528,E=7194559,C=28)
	CT(28584)=(S=7194560,E=7194591,C=27)
	CT(28585)=(S=7194592,E=7194623,C=51)
	CT(28586)=(S=7194624,E=7194655,C=14)
	CT(28587)=(S=7194656,E=7194687,C=72)
	CT(28588)=(S=7194688,E=7194719,C=58)
	CT(28589)=(S=7194720,E=7194751,C=18)
	CT(28590)=(S=7194752,E=7194783,C=31)
	CT(28591)=(S=7194784,E=7194815,C=67)
	CT(28592)=(S=7194816,E=7194847,C=36)
	CT(28593)=(S=7194848,E=7194879,C=31)
	CT(28594)=(S=7194880,E=7194911,C=26)
	CT(28595)=(S=7194912,E=7194943,C=20)
	CT(28596)=(S=7194944,E=7195007,C=18)
	CT(28597)=(S=7195008,E=7195039,C=36)
	CT(28598)=(S=7195040,E=7195071,C=62)
	CT(28599)=(S=7195072,E=7195103,C=18)
	CT(28600)=(S=7195104,E=7195135,C=31)
	CT(28601)=(S=7195136,E=7195199,C=18)
	CT(28602)=(S=7195200,E=7195231,C=33)
	CT(28603)=(S=7195232,E=7195247,C=36)
	CT(28604)=(S=7195248,E=7195255,C=64)
	CT(28605)=(S=7195256,E=7195263,C=52)
	CT(28606)=(S=7195264,E=7195295,C=24)
	CT(28607)=(S=7195296,E=7195327,C=18)
	CT(28608)=(S=7195328,E=7195359,C=34)
	CT(28609)=(S=7195360,E=7195391,C=14)
	CT(28610)=(S=7195392,E=7195423,C=27)
	CT(28611)=(S=7195424,E=7195455,C=34)
	CT(28612)=(S=7195456,E=7195487,C=12)
	CT(28613)=(S=7195488,E=7195519,C=14)
	CT(28614)=(S=7195520,E=7195583,C=26)
	CT(28615)=(S=7195584,E=7195615,C=18)
	CT(28616)=(S=7195616,E=7195647,C=191)
	CT(28617)=(S=7195648,E=7195775,C=14)
	CT(28618)=(S=7195776,E=7195903,C=47)
	CT(28619)=(S=7195904,E=7195911,C=12)
	CT(28620)=(S=7195912,E=7195919,C=27)
	CT(28621)=(S=7195920,E=7195927,C=15)
	CT(28622)=(S=7195928,E=7195935,C=16)
	CT(28623)=(S=7195936,E=7195943,C=59)
	CT(28624)=(S=7195944,E=7195951,C=40)
	CT(28625)=(S=7195952,E=7195959,C=58)
	CT(28626)=(S=7195960,E=7195967,C=18)
	CT(28627)=(S=7195968,E=7195975,C=12)
	CT(28628)=(S=7195976,E=7195983,C=33)
	CT(28629)=(S=7195984,E=7195991,C=18)
	CT(28630)=(S=7195992,E=7195999,C=38)
	CT(28631)=(S=7196000,E=7196007,C=47)
	CT(28632)=(S=7196008,E=7196015,C=15)
	CT(28633)=(S=7196016,E=7196023,C=35)
	CT(28634)=(S=7196024,E=7196031,C=60)
	CT(28635)=(S=7196032,E=7196039,C=28)
	CT(28636)=(S=7196040,E=7196047,C=25)
	CT(28637)=(S=7196048,E=7196055,C=18)
	CT(28638)=(S=7196056,E=7196063,C=14)
	CT(28639)=(S=7196064,E=7196071,C=41)
	CT(28640)=(S=7196072,E=7196079,C=34)
	CT(28641)=(S=7196080,E=7196087,C=28)
	CT(28642)=(S=7196088,E=7196095,C=14)
	CT(28643)=(S=7196096,E=7196103,C=36)
	CT(28644)=(S=7196104,E=7196111,C=34)
	CT(28645)=(S=7196112,E=7196119,C=41)
	CT(28646)=(S=7196120,E=7196127,C=19)
	CT(28647)=(S=7196128,E=7196135,C=14)
	CT(28648)=(S=7196136,E=7196143,C=44)
	CT(28649)=(S=7196144,E=7196151,C=33)
	CT(28650)=(S=7196152,E=7196159,C=18)
	CT(28651)=(S=7196160,E=7196191,C=38)
	CT(28652)=(S=7196192,E=7196223,C=31)
	CT(28653)=(S=7196224,E=7196255,C=23)
	CT(28654)=(S=7196256,E=7196287,C=46)
	CT(28655)=(S=7196288,E=7196319,C=18)
	CT(28656)=(S=7196320,E=7196351,C=13)
	CT(28657)=(S=7196352,E=7196383,C=58)
	CT(28658)=(S=7196416,E=7196431,C=18)
	CT(28659)=(S=7196432,E=7196447,C=14)
	CT(28660)=(S=7196448,E=7196463,C=46)
	CT(28661)=(S=7196464,E=7196479,C=58)
	CT(28662)=(S=7196496,E=7196511,C=18)
	CT(28663)=(S=7196512,E=7196527,C=58)
	CT(28664)=(S=7196528,E=7196543,C=31)
	CT(28665)=(S=7196544,E=7196559,C=38)
	CT(28666)=(S=7196560,E=7196575,C=58)
	CT(28667)=(S=7196576,E=7196607,C=18)
	CT(28668)=(S=7196608,E=7196623,C=31)
	CT(28669)=(S=7196624,E=7196639,C=38)
	CT(28670)=(S=7196640,E=7196655,C=18)
	CT(28671)=(S=7196656,E=7196671,C=98)
	CT(28672)=(S=7196672,E=7200767,C=12)
	CT(28673)=(S=7200768,E=7200831,C=35)
	CT(28674)=(S=7200832,E=7200895,C=33)
	CT(28675)=(S=7200896,E=7201023,C=14)
	CT(28676)=(S=7201024,E=7201087,C=18)
	CT(28677)=(S=7201088,E=7201151,C=19)
	CT(28678)=(S=7201152,E=7201215,C=26)
	CT(28679)=(S=7201216,E=7201279,C=18)
	CT(28680)=(S=7201280,E=7201343,C=17)
	CT(28681)=(S=7201344,E=7201407,C=18)
	CT(28682)=(S=7201408,E=7201471,C=28)
	CT(28683)=(S=7201472,E=7201535,C=18)
	CT(28684)=(S=7201536,E=7201599,C=57)
	CT(28685)=(S=7201600,E=7201663,C=31)
	CT(28686)=(S=7201664,E=7201727,C=25)
	CT(28687)=(S=7201728,E=7201791,C=18)
	CT(28688)=(S=7201792,E=7201855,C=14)
	CT(28689)=(S=7201856,E=7201919,C=102)
	CT(28690)=(S=7201920,E=7201983,C=19)
	CT(28691)=(S=7201984,E=7202047,C=37)
	CT(28692)=(S=7202048,E=7202079,C=31)
	CT(28693)=(S=7202080,E=7202207,C=18)
	CT(28694)=(S=7202208,E=7202239,C=20)
	CT(28695)=(S=7202240,E=7202271,C=60)
	CT(28696)=(S=7202272,E=7202303,C=18)
	CT(28697)=(S=7202304,E=7202367,C=71)
	CT(28698)=(S=7202368,E=7202431,C=26)
	CT(28699)=(S=7202432,E=7202495,C=18)
	CT(28700)=(S=7202496,E=7202497,C=26)
	CT(28701)=(S=7202498,E=7202499,C=28)
	CT(28702)=(S=7202500,E=7202500,C=37)
	CT(28703)=(S=7202501,E=7202545,C=28)
	CT(28704)=(S=7202546,E=7202546,C=26)
	CT(28705)=(S=7202547,E=7202559,C=28)
	CT(28706)=(S=7202560,E=7202623,C=58)
	CT(28707)=(S=7202624,E=7202687,C=14)
	CT(28708)=(S=7202688,E=7202751,C=33)
	CT(28709)=(S=7202752,E=7202815,C=14)
	CT(28710)=(S=7202816,E=7202823,C=20)
	CT(28711)=(S=7202824,E=7202831,C=36)
	CT(28712)=(S=7202832,E=7202839,C=25)
	CT(28713)=(S=7202840,E=7202847,C=58)
	CT(28714)=(S=7202848,E=7202855,C=15)
	CT(28715)=(S=7202856,E=7202863,C=36)
	CT(28716)=(S=7202864,E=7202871,C=15)
	CT(28717)=(S=7202872,E=7202879,C=14)
	CT(28718)=(S=7202880,E=7202887,C=25)
	CT(28719)=(S=7202888,E=7202895,C=24)
	CT(28720)=(S=7202896,E=7202903,C=47)
	CT(28721)=(S=7202904,E=7202911,C=34)
	CT(28722)=(S=7202912,E=7202919,C=12)
	CT(28723)=(S=7202920,E=7202927,C=18)
	CT(28724)=(S=7202928,E=7202935,C=25)
	CT(28725)=(S=7202936,E=7202951,C=28)
	CT(28726)=(S=7202952,E=7202959,C=15)
	CT(28727)=(S=7202960,E=7202967,C=33)
	CT(28728)=(S=7202968,E=7202975,C=14)
	CT(28729)=(S=7202984,E=7202991,C=28)
	CT(28730)=(S=7202992,E=7202999,C=14)
	CT(28731)=(S=7203000,E=7203007,C=18)
	CT(28732)=(S=7203008,E=7203015,C=12)
	CT(28733)=(S=7203016,E=7203023,C=50)
	CT(28734)=(S=7203024,E=7203031,C=25)
	CT(28735)=(S=7203032,E=7203039,C=37)
	CT(28736)=(S=7203040,E=7203047,C=36)
	CT(28737)=(S=7203048,E=7203055,C=12)
	CT(28738)=(S=7203056,E=7203063,C=58)
	CT(28739)=(S=7203064,E=7203071,C=18)
	CT(28740)=(S=7203072,E=7203079,C=27)
	CT(28741)=(S=7203080,E=7203087,C=14)
	CT(28742)=(S=7203088,E=7203095,C=39)
	CT(28743)=(S=7203096,E=7203103,C=42)
	CT(28744)=(S=7203104,E=7203127,C=14)
	CT(28745)=(S=7203128,E=7203135,C=27)
	CT(28746)=(S=7203136,E=7203151,C=33)
	CT(28747)=(S=7203152,E=7203159,C=100)
	CT(28748)=(S=7203160,E=7203167,C=58)
	CT(28749)=(S=7203168,E=7203175,C=15)
	CT(28750)=(S=7203184,E=7203191,C=14)
	CT(28751)=(S=7203192,E=7203207,C=15)
	CT(28752)=(S=7203208,E=7203231,C=28)
	CT(28753)=(S=7203232,E=7203239,C=33)
	CT(28754)=(S=7203240,E=7203247,C=18)
	CT(28755)=(S=7203248,E=7203255,C=13)
	CT(28756)=(S=7203256,E=7203263,C=46)
	CT(28757)=(S=7203264,E=7203271,C=53)
	CT(28758)=(S=7203272,E=7203287,C=18)
	CT(28759)=(S=7203288,E=7203295,C=15)
	CT(28760)=(S=7203296,E=7203319,C=18)
	CT(28761)=(S=7203320,E=7203327,C=28)
	CT(28762)=(S=7203328,E=7203335,C=12)
	CT(28763)=(S=7203336,E=7203343,C=18)
	CT(28764)=(S=7203344,E=7203351,C=17)
	CT(28765)=(S=7203352,E=7203367,C=18)
	CT(28766)=(S=7203368,E=7203375,C=33)
	CT(28767)=(S=7203376,E=7203391,C=15)
	CT(28768)=(S=7203392,E=7203399,C=25)
	CT(28769)=(S=7203400,E=7203407,C=14)
	CT(28770)=(S=7203408,E=7203423,C=25)
	CT(28771)=(S=7203424,E=7203431,C=63)
	CT(28772)=(S=7203432,E=7203439,C=28)
	CT(28773)=(S=7203440,E=7203447,C=40)
	CT(28774)=(S=7203448,E=7203455,C=28)
	CT(28775)=(S=7203456,E=7203463,C=18)
	CT(28776)=(S=7203464,E=7203471,C=15)
	CT(28777)=(S=7203472,E=7203479,C=42)
	CT(28778)=(S=7203480,E=7203487,C=18)
	CT(28779)=(S=7203488,E=7203495,C=12)
	CT(28780)=(S=7203496,E=7203511,C=14)
	CT(28781)=(S=7203512,E=7203519,C=28)
	CT(28782)=(S=7203520,E=7203535,C=14)
	CT(28783)=(S=7203536,E=7203543,C=25)
	CT(28784)=(S=7203544,E=7203551,C=28)
	CT(28785)=(S=7203552,E=7203559,C=14)
	CT(28786)=(S=7203560,E=7203567,C=96)
	CT(28787)=(S=7203568,E=7203575,C=47)
	CT(28788)=(S=7203576,E=7203583,C=28)
	CT(28789)=(S=7203584,E=7203591,C=33)
	CT(28790)=(S=7203592,E=7203599,C=31)
	CT(28791)=(S=7203600,E=7203607,C=25)
	CT(28792)=(S=7203608,E=7203615,C=18)
	CT(28793)=(S=7203616,E=7203623,C=36)
	CT(28794)=(S=7203624,E=7203631,C=28)
	CT(28795)=(S=7203632,E=7203639,C=36)
	CT(28796)=(S=7203640,E=7203647,C=28)
	CT(28797)=(S=7203648,E=7203655,C=44)
	CT(28798)=(S=7203656,E=7203663,C=36)
	CT(28799)=(S=7203664,E=7203671,C=12)
	CT(28800)=(S=7203672,E=7203679,C=18)
	CT(28801)=(S=7203680,E=7203687,C=46)
	CT(28802)=(S=7203688,E=7203695,C=100)
	CT(28803)=(S=7203696,E=7203703,C=27)
	CT(28804)=(S=7203704,E=7203711,C=14)
	CT(28805)=(S=7203712,E=7203719,C=25)
	CT(28806)=(S=7203720,E=7203727,C=28)
	CT(28807)=(S=7203728,E=7203735,C=14)
	CT(28808)=(S=7203736,E=7203743,C=15)
	CT(28809)=(S=7203744,E=7203751,C=18)
	CT(28810)=(S=7203752,E=7203759,C=19)
	CT(28811)=(S=7203760,E=7203767,C=33)
	CT(28812)=(S=7203768,E=7203775,C=18)
	CT(28813)=(S=7203776,E=7203783,C=41)
	CT(28814)=(S=7203784,E=7203791,C=69)
	CT(28815)=(S=7203792,E=7203807,C=18)
	CT(28816)=(S=7203808,E=7203823,C=28)
	CT(28817)=(S=7203824,E=7203831,C=55)
	CT(28818)=(S=7203832,E=7203839,C=37)
	CT(28819)=(S=7203840,E=7203871,C=15)
	CT(28820)=(S=7203872,E=7203887,C=96)
	CT(28821)=(S=7203904,E=7203919,C=18)
	CT(28822)=(S=7203920,E=7203935,C=36)
	CT(28823)=(S=7203936,E=7203951,C=18)
	CT(28824)=(S=7203952,E=7203967,C=71)
	CT(28825)=(S=7203968,E=7203983,C=49)
	CT(28826)=(S=7203984,E=7203999,C=28)
	CT(28827)=(S=7204000,E=7204015,C=14)
	CT(28828)=(S=7204016,E=7204031,C=24)
	CT(28829)=(S=7204032,E=7204047,C=12)
	CT(28830)=(S=7204048,E=7204111,C=18)
	CT(28831)=(S=7204112,E=7204119,C=14)
	CT(28832)=(S=7204120,E=7204123,C=13)
	CT(28833)=(S=7204124,E=7204127,C=14)
	CT(28834)=(S=7204128,E=7204143,C=59)
	CT(28835)=(S=7204144,E=7204159,C=34)
	CT(28836)=(S=7204160,E=7204175,C=36)
	CT(28837)=(S=7204176,E=7204191,C=31)
	CT(28838)=(S=7204192,E=7204207,C=18)
	CT(28839)=(S=7204208,E=7204223,C=41)
	CT(28840)=(S=7204224,E=7204239,C=28)
	CT(28841)=(S=7204240,E=7204255,C=18)
	CT(28842)=(S=7204256,E=7204271,C=15)
	CT(28843)=(S=7204272,E=7204287,C=28)
	CT(28844)=(S=7204288,E=7204303,C=35)
	CT(28845)=(S=7204304,E=7204319,C=36)
	CT(28846)=(S=7204320,E=7204335,C=18)
	CT(28847)=(S=7204336,E=7204367,C=12)
	CT(28848)=(S=7204368,E=7204383,C=15)
	CT(28849)=(S=7204384,E=7204399,C=33)
	CT(28850)=(S=7204400,E=7204415,C=24)
	CT(28851)=(S=7204416,E=7204431,C=14)
	CT(28852)=(S=7204432,E=7204463,C=18)
	CT(28853)=(S=7204464,E=7204479,C=15)
	CT(28854)=(S=7204480,E=7204495,C=14)
	CT(28855)=(S=7204496,E=7204511,C=30)
	CT(28856)=(S=7204512,E=7204527,C=20)
	CT(28857)=(S=7204528,E=7204543,C=26)
	CT(28858)=(S=7204544,E=7204559,C=18)
	CT(28859)=(S=7204560,E=7204575,C=33)
	CT(28860)=(S=7204576,E=7204591,C=40)
	CT(28861)=(S=7204592,E=7204607,C=18)
	CT(28862)=(S=7204608,E=7204623,C=26)
	CT(28863)=(S=7204624,E=7204639,C=41)
	CT(28864)=(S=7204640,E=7204655,C=20)
	CT(28865)=(S=7204656,E=7204671,C=28)
	CT(28866)=(S=7204672,E=7204687,C=33)
	CT(28867)=(S=7204688,E=7204719,C=14)
	CT(28868)=(S=7204720,E=7204735,C=12)
	CT(28869)=(S=7204736,E=7204751,C=18)
	CT(28870)=(S=7204752,E=7204767,C=12)
	CT(28871)=(S=7204768,E=7204783,C=28)
	CT(28872)=(S=7204784,E=7204799,C=52)
	CT(28873)=(S=7204800,E=7204815,C=28)
	CT(28874)=(S=7204816,E=7204831,C=18)
	CT(28875)=(S=7204832,E=7204847,C=27)
	CT(28876)=(S=7204848,E=7204863,C=15)
	CT(28877)=(S=7204864,E=7205119,C=47)
	CT(28878)=(S=7205120,E=7205375,C=58)
	CT(28879)=(S=7205376,E=7205631,C=22)
	CT(28880)=(S=7205632,E=7205887,C=58)
	CT(28881)=(S=7205888,E=7206143,C=18)
	CT(28882)=(S=7206144,E=7206399,C=46)
	CT(28883)=(S=7206400,E=7206655,C=14)
	CT(28884)=(S=7206656,E=7206911,C=27)
	CT(28885)=(S=7206912,E=7207167,C=18)
	CT(28886)=(S=7207168,E=7207423,C=14)
	CT(28887)=(S=7207424,E=7207679,C=28)
	CT(28888)=(S=7207680,E=7207935,C=31)
	CT(28889)=(S=7207936,E=7208191,C=18)
	CT(28890)=(S=7208192,E=7208447,C=17)
	CT(28891)=(S=7208448,E=7208703,C=31)
	CT(28892)=(S=7208704,E=7208959,C=63)
	CT(28893)=(S=7208960,E=7209983,C=2)
	CT(28894)=(S=7209984,E=7210015,C=8)
	CT(28895)=(S=7210016,E=7210023,C=2)
	CT(28896)=(S=7210024,E=7210031,C=5)
	CT(28897)=(S=7210032,E=7210047,C=2)
	CT(28898)=(S=7210048,E=7210111,C=6)
	CT(28899)=(S=7210112,E=7210303,C=2)
	CT(28900)=(S=7210304,E=7210311,C=10)
	CT(28901)=(S=7210312,E=7210319,C=4)
	CT(28902)=(S=7210320,E=7210335,C=0)
	CT(28903)=(S=7210336,E=7210351,C=74)
	CT(28904)=(S=7210352,E=7210355,C=162)
	CT(28905)=(S=7210356,E=7210359,C=0)
	CT(28906)=(S=7210360,E=7210367,C=2)
	CT(28907)=(S=7210368,E=7210495,C=6)
	CT(28908)=(S=7210496,E=7211007,C=1)
	CT(28909)=(S=7211008,E=7213055,C=6)
	CT(28910)=(S=7213056,E=7214079,C=1)
	CT(28911)=(S=7214080,E=7215103,C=0)
	CT(28912)=(S=7215104,E=7217151,C=9)
	CT(28913)=(S=7217152,E=7217663,C=0)
	CT(28914)=(S=7217664,E=7217695,C=75)
	CT(28915)=(S=7217696,E=7217703,C=77)
	CT(28916)=(S=7217712,E=7217715,C=73)
	CT(28917)=(S=7217716,E=7217719,C=0)
	CT(28918)=(S=7217720,E=7217727,C=8)
	CT(28919)=(S=7217728,E=7217791,C=6)
	CT(28920)=(S=7217792,E=7217919,C=3)
	CT(28921)=(S=7217920,E=7217983,C=6)
	CT(28922)=(S=7217984,E=7217999,C=11)
	CT(28923)=(S=7218000,E=7218007,C=74)
	CT(28924)=(S=7218008,E=7218015,C=91)
	CT(28925)=(S=7218016,E=7218175,C=6)
	CT(28926)=(S=7218176,E=7219199,C=77)
	CT(28927)=(S=7219200,E=7220223,C=1)
	CT(28928)=(S=7220240,E=7220243,C=73)
	CT(28929)=(S=7220244,E=7220247,C=2)
	CT(28930)=(S=7220248,E=7220255,C=0)
	CT(28931)=(S=7220256,E=7220287,C=6)
	CT(28932)=(S=7220288,E=7220319,C=2)
	CT(28933)=(S=7220320,E=7220335,C=10)
	CT(28934)=(S=7220336,E=7220351,C=75)
	CT(28935)=(S=7220352,E=7220367,C=10)
	CT(28936)=(S=7220368,E=7220383,C=1)
	CT(28937)=(S=7220384,E=7220391,C=0)
	CT(28938)=(S=7220392,E=7220399,C=74)
	CT(28939)=(S=7220400,E=7220407,C=2)
	CT(28940)=(S=7220408,E=7220415,C=11)
	CT(28941)=(S=7220416,E=7221247,C=6)
	CT(28942)=(S=7221248,E=7221503,C=1)
	CT(28943)=(S=7221504,E=7221759,C=3)
	CT(28944)=(S=7221760,E=7221775,C=2)
	CT(28945)=(S=7221776,E=7221783,C=0)
	CT(28946)=(S=7221784,E=7221791,C=2)
	CT(28947)=(S=7221792,E=7221823,C=4)
	CT(28948)=(S=7221824,E=7221839,C=2)
	CT(28949)=(S=7221840,E=7221847,C=74)
	CT(28950)=(S=7221848,E=7221887,C=2)
	CT(28951)=(S=7221888,E=7221951,C=9)
	CT(28952)=(S=7221952,E=7221983,C=2)
	CT(28953)=(S=7221984,E=7221999,C=10)
	CT(28954)=(S=7222000,E=7222015,C=2)
	CT(28955)=(S=7222016,E=7222783,C=1)
	CT(28956)=(S=7222784,E=7222911,C=2)
	CT(28957)=(S=7222912,E=7223295,C=10)
	CT(28958)=(S=7223296,E=7225855,C=1)
	CT(28959)=(S=7225856,E=7226367,C=2)
	CT(28960)=(S=7226368,E=7227391,C=6)
	CT(28961)=(S=7227392,E=7227903,C=1)
	CT(28962)=(S=7227904,E=7228031,C=2)
	CT(28963)=(S=7228032,E=7228095,C=5)
	CT(28964)=(S=7228096,E=7228127,C=86)
	CT(28965)=(S=7228128,E=7228159,C=2)
	CT(28966)=(S=7228160,E=7228479,C=1)
	CT(28967)=(S=7228480,E=7228543,C=6)
	CT(28968)=(S=7228552,E=7228555,C=0)
	CT(28969)=(S=7228556,E=7228559,C=6)
	CT(28970)=(S=7228560,E=7228567,C=74)
	CT(28971)=(S=7228568,E=7228571,C=3)
	CT(28972)=(S=7228572,E=7228575,C=1)
	CT(28973)=(S=7228576,E=7228591,C=4)
	CT(28974)=(S=7228592,E=7228595,C=0)
	CT(28975)=(S=7228596,E=7228599,C=2)
	CT(28976)=(S=7228600,E=7228603,C=1)
	CT(28977)=(S=7228604,E=7228607,C=8)
	CT(28978)=(S=7228608,E=7228799,C=1)
	CT(28979)=(S=7228800,E=7229183,C=3)
	CT(28980)=(S=7229184,E=7229439,C=8)
	CT(28981)=(S=7229440,E=7232511,C=1)
	CT(28982)=(S=7232512,E=7232527,C=2)
	CT(28983)=(S=7232528,E=7232531,C=73)
	CT(28984)=(S=7232532,E=7232535,C=6)
	CT(28985)=(S=7232536,E=7232543,C=10)
	CT(28986)=(S=7232544,E=7232575,C=2)
	CT(28987)=(S=7232584,E=7232587,C=74)
	CT(28988)=(S=7232588,E=7232591,C=8)
	CT(28989)=(S=7232592,E=7232607,C=2)
	CT(28990)=(S=7232608,E=7232639,C=7)
	CT(28991)=(S=7232640,E=7232767,C=6)
	CT(28992)=(S=7232768,E=7232775,C=2)
	CT(28993)=(S=7232776,E=7232779,C=0)
	CT(28994)=(S=7232780,E=7232783,C=74)
	CT(28995)=(S=7232784,E=7232791,C=0)
	CT(28996)=(S=7232792,E=7232799,C=6)
	CT(28997)=(S=7232800,E=7232831,C=1)
	CT(28998)=(S=7232832,E=7232863,C=10)
	CT(28999)=(S=7232864,E=7232879,C=2)
	CT(29000)=(S=7232880,E=7232959,C=6)
	CT(29001)=(S=7232960,E=7233023,C=77)
	CT(29002)=(S=7233024,E=7241727,C=1)
	CT(29003)=(S=7241728,E=7243775,C=2)
	CT(29004)=(S=7243776,E=7244799,C=74)
	CT(29005)=(S=7244800,E=7247871,C=0)
	CT(29006)=(S=7247872,E=7249407,C=1)
	CT(29007)=(S=7249408,E=7249663,C=2)
	CT(29008)=(S=7249664,E=7249919,C=5)
	CT(29009)=(S=7249920,E=7250943,C=2)
	CT(29010)=(S=7250944,E=7251199,C=3)
	CT(29011)=(S=7251200,E=7251231,C=6)
	CT(29012)=(S=7251232,E=7251263,C=1)
	CT(29013)=(S=7251264,E=7251327,C=6)
	CT(29014)=(S=7251328,E=7251455,C=2)
	CT(29015)=(S=7251456,E=7251967,C=1)
	CT(29016)=(S=7251968,E=7252991,C=3)
	CT(29017)=(S=7252992,E=7253007,C=2)
	CT(29018)=(S=7253008,E=7253023,C=4)
	CT(29019)=(S=7253024,E=7253043,C=2)
	CT(29020)=(S=7253044,E=7253047,C=4)
	CT(29021)=(S=7253048,E=7253055,C=2)
	CT(29022)=(S=7253056,E=7253119,C=6)
	CT(29023)=(S=7253120,E=7253183,C=4)
	CT(29024)=(S=7253184,E=7253295,C=1)
	CT(29025)=(S=7253296,E=7253311,C=8)
	CT(29026)=(S=7253312,E=7253375,C=1)
	CT(29027)=(S=7253376,E=7253407,C=0)
	CT(29028)=(S=7253408,E=7253423,C=73)
	CT(29029)=(S=7253424,E=7253439,C=4)
	CT(29030)=(S=7253440,E=7253471,C=1)
	CT(29031)=(S=7253472,E=7253487,C=0)
	CT(29032)=(S=7253488,E=7253503,C=2)
	CT(29033)=(S=7253504,E=7254015,C=0)
	CT(29034)=(S=7254016,E=7266303,C=1)
	CT(29035)=(S=7266304,E=7267327,C=4)
	CT(29036)=(S=7267328,E=7268351,C=1)
	CT(29037)=(S=7268352,E=7268383,C=2)
	CT(29038)=(S=7268384,E=7268415,C=1)
	CT(29039)=(S=7268416,E=7268447,C=74)
	CT(29040)=(S=7268448,E=7268463,C=6)
	CT(29041)=(S=7268464,E=7268471,C=0)
	CT(29042)=(S=7268472,E=7268491,C=2)
	CT(29043)=(S=7268492,E=7268495,C=0)
	CT(29044)=(S=7268496,E=7268499,C=73)
	CT(29045)=(S=7268500,E=7268503,C=0)
	CT(29046)=(S=7268504,E=7268511,C=2)
	CT(29047)=(S=7268512,E=7268527,C=10)
	CT(29048)=(S=7268528,E=7268531,C=0)
	CT(29049)=(S=7268532,E=7268535,C=4)
	CT(29050)=(S=7268536,E=7268539,C=0)
	CT(29051)=(S=7268540,E=7268543,C=77)
	CT(29052)=(S=7268544,E=7268591,C=2)
	CT(29053)=(S=7268592,E=7268599,C=0)
	CT(29054)=(S=7268600,E=7268607,C=4)
	CT(29055)=(S=7268608,E=7268863,C=2)
	CT(29056)=(S=7268864,E=7269375,C=4)
	CT(29057)=(S=7269376,E=7269887,C=1)
	CT(29058)=(S=7269888,E=7270399,C=0)
	CT(29059)=(S=7270400,E=7290879,C=1)
	CT(29060)=(S=7290880,E=7291135,C=2)
	CT(29061)=(S=7291136,E=7291263,C=7)
	CT(29062)=(S=7291264,E=7291327,C=6)
	CT(29063)=(S=7291328,E=7291359,C=2)
	CT(29064)=(S=7291360,E=7291375,C=73)
	CT(29065)=(S=7291376,E=7291391,C=11)
	CT(29066)=(S=7291392,E=7291647,C=1)
	CT(29067)=(S=7291648,E=7291679,C=0)
	CT(29068)=(S=7291680,E=7291695,C=5)
	CT(29069)=(S=7291696,E=7291711,C=9)
	CT(29070)=(S=7291712,E=7291743,C=74)
	CT(29071)=(S=7291744,E=7291759,C=86)
	CT(29072)=(S=7291760,E=7291839,C=2)
	CT(29073)=(S=7291840,E=7291855,C=1)
	CT(29074)=(S=7291856,E=7291903,C=6)
	CT(29075)=(S=7291904,E=7291919,C=8)
	CT(29076)=(S=7291920,E=7291927,C=2)
	CT(29077)=(S=7291928,E=7291935,C=74)
	CT(29078)=(S=7291936,E=7291967,C=10)
	CT(29079)=(S=7291968,E=7291999,C=1)
	CT(29080)=(S=7292000,E=7292015,C=77)
	CT(29081)=(S=7292016,E=7292031,C=74)
	CT(29082)=(S=7292032,E=7292159,C=2)
	CT(29083)=(S=7292160,E=7292415,C=73)
	CT(29084)=(S=7292416,E=7292927,C=9)
	CT(29085)=(S=7292928,E=7294975,C=1)
	CT(29086)=(S=7294976,E=7295999,C=9)
	CT(29087)=(S=7296000,E=7296255,C=3)
	CT(29088)=(S=7296256,E=7296511,C=1)
	CT(29089)=(S=7296512,E=7297023,C=2)
	CT(29090)=(S=7297024,E=7297279,C=77)
	CT(29091)=(S=7297280,E=7297663,C=2)
	CT(29092)=(S=7297664,E=7297695,C=5)
	CT(29093)=(S=7297696,E=7297703,C=2)
	CT(29094)=(S=7297704,E=7297711,C=4)
	CT(29095)=(S=7297712,E=7297727,C=86)
	CT(29096)=(S=7297728,E=7297791,C=7)
	CT(29097)=(S=7297792,E=7297919,C=4)
	CT(29098)=(S=7297920,E=7297983,C=6)
	CT(29099)=(S=7297984,E=7298015,C=1)
	CT(29100)=(S=7298016,E=7298023,C=4)
	CT(29101)=(S=7298024,E=7298027,C=11)
	CT(29102)=(S=7298028,E=7298031,C=8)
	CT(29103)=(S=7298032,E=7298047,C=2)
	CT(29104)=(S=7298048,E=7298175,C=4)
	CT(29105)=(S=7298176,E=7298207,C=77)
	CT(29106)=(S=7298208,E=7298223,C=74)
	CT(29107)=(S=7298224,E=7298227,C=0)
	CT(29108)=(S=7298228,E=7298231,C=3)
	CT(29109)=(S=7298236,E=7298239,C=6)
	CT(29110)=(S=7298240,E=7298271,C=7)
	CT(29111)=(S=7298272,E=7298287,C=8)
	CT(29112)=(S=7298288,E=7298291,C=86)
	CT(29113)=(S=7298292,E=7298295,C=2)
	CT(29114)=(S=7298304,E=7298559,C=4)
	CT(29115)=(S=7298560,E=7299071,C=74)
	CT(29116)=(S=7299072,E=7303167,C=2)
	CT(29117)=(S=7303168,E=7304703,C=1)
	CT(29118)=(S=7304704,E=7304831,C=6)
	CT(29119)=(S=7304832,E=7304863,C=86)
	CT(29120)=(S=7304864,E=7304879,C=0)
	CT(29121)=(S=7304880,E=7304895,C=4)
	CT(29122)=(S=7304896,E=7304903,C=0)
	CT(29123)=(S=7304904,E=7304911,C=1)
	CT(29124)=(S=7304912,E=7304915,C=0)
	CT(29125)=(S=7304916,E=7304919,C=4)
	CT(29126)=(S=7304920,E=7304943,C=0)
	CT(29127)=(S=7304944,E=7304959,C=4)
	CT(29128)=(S=7304960,E=7304991,C=2)
	CT(29129)=(S=7304992,E=7305023,C=75)
	CT(29130)=(S=7305024,E=7305119,C=1)
	CT(29131)=(S=7305120,E=7305151,C=77)
	CT(29132)=(S=7305152,E=7305215,C=4)
	CT(29133)=(S=7305216,E=7306495,C=1)
	CT(29134)=(S=7306496,E=7306559,C=2)
	CT(29135)=(S=7306560,E=7306623,C=10)
	CT(29136)=(S=7306624,E=7306631,C=9)
	CT(29137)=(S=7306632,E=7306639,C=4)
	CT(29138)=(S=7306640,E=7306647,C=2)
	CT(29139)=(S=7306648,E=7306655,C=89)
	CT(29140)=(S=7306656,E=7306687,C=0)
	CT(29141)=(S=7306688,E=7306751,C=4)
	CT(29142)=(S=7306752,E=7317503,C=1)
	CT(29143)=(S=7317504,E=7318015,C=2)
	CT(29144)=(S=7318016,E=7318271,C=1)
	CT(29145)=(S=7318272,E=7318399,C=6)
	CT(29146)=(S=7318400,E=7318527,C=2)
	CT(29147)=(S=7318528,E=7321599,C=1)
	CT(29148)=(S=7321600,E=7322111,C=9)
	CT(29149)=(S=7322112,E=7322623,C=1)
	CT(29150)=(S=7322624,E=7323647,C=2)
	CT(29151)=(S=7323648,E=7329791,C=1)
	CT(29152)=(S=7329792,E=7330303,C=2)
	CT(29153)=(S=7330304,E=7330815,C=6)
	CT(29154)=(S=7330816,E=7331071,C=0)
	CT(29155)=(S=7331072,E=7331079,C=84)
	CT(29156)=(S=7331080,E=7331099,C=7)
	CT(29157)=(S=7331100,E=7331100,C=1)
	CT(29158)=(S=7331101,E=7331103,C=7)
	CT(29159)=(S=7331104,E=7331111,C=6)
	CT(29160)=(S=7331112,E=7331115,C=74)
	CT(29161)=(S=7331116,E=7331119,C=7)
	CT(29162)=(S=7331120,E=7331127,C=5)
	CT(29163)=(S=7331128,E=7331131,C=0)
	CT(29164)=(S=7331132,E=7331135,C=2)
	CT(29165)=(S=7331136,E=7331199,C=7)
	CT(29166)=(S=7331200,E=7331583,C=1)
	CT(29167)=(S=7331616,E=7331647,C=3)
	CT(29168)=(S=7331648,E=7331711,C=7)
	CT(29169)=(S=7331712,E=7331775,C=156)
	CT(29170)=(S=7331776,E=7331807,C=2)
	CT(29171)=(S=7331808,E=7331823,C=0)
	CT(29172)=(S=7331824,E=7331835,C=1)
	CT(29173)=(S=7331836,E=7331839,C=74)
	CT(29174)=(S=7331840,E=7333887,C=1)
	CT(29175)=(S=7333888,E=7334719,C=2)
	CT(29176)=(S=7334720,E=7334731,C=4)
	CT(29177)=(S=7334732,E=7334735,C=2)
	CT(29178)=(S=7334736,E=7334751,C=10)
	CT(29179)=(S=7334752,E=7334783,C=1)
	CT(29180)=(S=7334784,E=7334787,C=0)
	CT(29181)=(S=7334788,E=7334791,C=5)
	CT(29182)=(S=7334792,E=7334795,C=7)
	CT(29183)=(S=7334796,E=7334803,C=0)
	CT(29184)=(S=7334804,E=7334807,C=4)
	CT(29185)=(S=7334808,E=7334811,C=8)
	CT(29186)=(S=7334812,E=7334847,C=1)
	CT(29187)=(S=7334848,E=7334911,C=9)
	CT(29188)=(S=7334912,E=7335935,C=2)
	CT(29189)=(S=7335936,E=7340031,C=9)
	CT(29190)=(S=7340032,E=7357439,C=1)
	CT(29191)=(S=7357440,E=7358463,C=2)
	CT(29192)=(S=7358464,E=7358479,C=92)
	CT(29193)=(S=7358480,E=7358527,C=6)
	CT(29194)=(S=7358528,E=7358591,C=11)
	CT(29195)=(S=7358592,E=7358719,C=6)
	CT(29196)=(S=7358720,E=7359487,C=1)
	CT(29197)=(S=7359488,E=7359999,C=6)
	CT(29198)=(S=7360000,E=7360015,C=11)
	CT(29199)=(S=7360016,E=7360031,C=9)
	CT(29200)=(S=7360032,E=7360047,C=74)
	CT(29201)=(S=7360048,E=7360063,C=10)
	CT(29202)=(S=7360064,E=7360095,C=9)
	CT(29203)=(S=7360096,E=7360103,C=74)
	CT(29204)=(S=7360104,E=7360111,C=9)
	CT(29205)=(S=7360112,E=7360127,C=2)
	CT(29206)=(S=7360128,E=7360191,C=74)
	CT(29207)=(S=7360192,E=7360223,C=2)
	CT(29208)=(S=7360224,E=7360255,C=7)
	CT(29209)=(S=7360256,E=7360511,C=4)
	CT(29210)=(S=7360512,E=7366655,C=1)
	CT(29211)=(S=7366656,E=7367167,C=9)
	CT(29212)=(S=7367168,E=7367935,C=6)
	CT(29213)=(S=7367936,E=7367951,C=10)
	CT(29214)=(S=7367952,E=7367959,C=74)
	CT(29215)=(S=7367960,E=7367967,C=2)
	CT(29216)=(S=7367968,E=7367999,C=6)
	CT(29217)=(S=7368000,E=7368023,C=73)
	CT(29218)=(S=7368024,E=7368031,C=11)
	CT(29219)=(S=7368032,E=7368063,C=4)
	CT(29220)=(S=7368064,E=7368191,C=1)
	CT(29221)=(S=7368192,E=7368447,C=4)
	CT(29222)=(S=7368448,E=7370239,C=1)
	CT(29223)=(S=7370240,E=7371007,C=8)
	CT(29224)=(S=7371008,E=7371039,C=6)
	CT(29225)=(S=7371040,E=7371055,C=9)
	CT(29226)=(S=7371056,E=7371071,C=4)
	CT(29227)=(S=7371072,E=7371135,C=9)
	CT(29228)=(S=7371136,E=7371167,C=3)
	CT(29229)=(S=7371168,E=7371199,C=8)
	CT(29230)=(S=7371200,E=7371263,C=6)
	CT(29231)=(S=7371264,E=7374079,C=1)
	CT(29232)=(S=7374080,E=7374271,C=6)
	CT(29233)=(S=7374272,E=7374335,C=4)
	CT(29234)=(S=7374336,E=7374847,C=156)
	CT(29235)=(S=7374848,E=7374975,C=2)
	CT(29236)=(S=7374976,E=7375103,C=6)
	CT(29237)=(S=7375104,E=7375119,C=84)
	CT(29238)=(S=7375120,E=7375135,C=8)
	CT(29239)=(S=7375136,E=7375151,C=2)
	CT(29240)=(S=7375152,E=7375159,C=1)
	CT(29241)=(S=7375160,E=7375231,C=2)
	CT(29242)=(S=7375232,E=7375247,C=11)
	CT(29243)=(S=7375248,E=7375263,C=2)
	CT(29244)=(S=7375264,E=7375279,C=5)
	CT(29245)=(S=7375280,E=7375287,C=6)
	CT(29246)=(S=7375288,E=7375935,C=2)
	CT(29247)=(S=7375936,E=7375999,C=6)
	CT(29248)=(S=7376000,E=7376015,C=2)
	CT(29249)=(S=7376016,E=7376031,C=6)
	CT(29250)=(S=7376032,E=7376039,C=74)
	CT(29251)=(S=7376040,E=7376047,C=2)
	CT(29252)=(S=7376048,E=7376055,C=0)
	CT(29253)=(S=7376056,E=7376059,C=7)
	CT(29254)=(S=7376064,E=7376127,C=6)
	CT(29255)=(S=7376128,E=7376383,C=0)
	CT(29256)=(S=7376384,E=7376895,C=3)
	CT(29257)=(S=7376896,E=7389183,C=6)
	CT(29258)=(S=7389184,E=7390207,C=1)
	CT(29259)=(S=7390208,E=7390399,C=4)
	CT(29260)=(S=7390400,E=7390431,C=6)
	CT(29261)=(S=7390432,E=7390463,C=10)
	CT(29262)=(S=7390464,E=7390719,C=11)
	CT(29263)=(S=7390720,E=7391103,C=10)
	CT(29264)=(S=7391104,E=7391231,C=7)
	CT(29265)=(S=7391232,E=7394303,C=10)
	CT(29266)=(S=7394304,E=7394591,C=6)
	CT(29267)=(S=7394592,E=7394607,C=0)
	CT(29268)=(S=7394608,E=7394623,C=9)
	CT(29269)=(S=7394624,E=7394639,C=8)
	CT(29270)=(S=7394640,E=7394655,C=11)
	CT(29271)=(S=7394656,E=7394687,C=8)
	CT(29272)=(S=7394688,E=7394815,C=0)
	CT(29273)=(S=7394816,E=7395071,C=6)
	CT(29274)=(S=7395072,E=7395327,C=74)
	CT(29275)=(S=7395328,E=7397375,C=6)
	CT(29276)=(S=7397376,E=7408127,C=1)
	CT(29277)=(S=7408128,E=7408191,C=6)
	CT(29278)=(S=7408192,E=7408255,C=7)
	CT(29279)=(S=7408256,E=7408383,C=8)
	CT(29280)=(S=7408384,E=7408511,C=84)
	CT(29281)=(S=7408512,E=7408543,C=74)
	CT(29282)=(S=7408544,E=7408559,C=7)
	CT(29283)=(S=7408560,E=7408567,C=74)
	CT(29284)=(S=7408568,E=7408571,C=0)
	CT(29285)=(S=7408572,E=7408575,C=5)
	CT(29286)=(S=7408576,E=7408607,C=1)
	CT(29287)=(S=7408608,E=7408615,C=4)
	CT(29288)=(S=7408616,E=7408619,C=0)
	CT(29289)=(S=7408620,E=7408623,C=5)
	CT(29290)=(S=7408624,E=7408631,C=160)
	CT(29291)=(S=7408632,E=7408639,C=7)
	CT(29292)=(S=7408640,E=7410431,C=1)
	CT(29293)=(S=7410432,E=7410687,C=4)
	CT(29294)=(S=7410688,E=7410703,C=0)
	CT(29295)=(S=7410704,E=7410707,C=4)
	CT(29296)=(S=7410708,E=7410711,C=8)
	CT(29297)=(S=7410712,E=7410715,C=0)
	CT(29298)=(S=7410716,E=7410719,C=74)
	CT(29299)=(S=7410720,E=7410751,C=90)
	CT(29300)=(S=7410752,E=7410783,C=91)
	CT(29301)=(S=7410784,E=7410815,C=11)
	CT(29302)=(S=7410816,E=7410819,C=0)
	CT(29303)=(S=7410820,E=7410823,C=8)
	CT(29304)=(S=7410824,E=7410831,C=74)
	CT(29305)=(S=7410832,E=7410839,C=10)
	CT(29306)=(S=7410840,E=7410843,C=2)
	CT(29307)=(S=7410848,E=7410879,C=10)
	CT(29308)=(S=7410880,E=7410943,C=2)
	CT(29309)=(S=7410944,E=7410975,C=6)
	CT(29310)=(S=7410976,E=7410991,C=0)
	CT(29311)=(S=7410992,E=7411007,C=2)
	CT(29312)=(S=7411008,E=7411023,C=4)
	CT(29313)=(S=7411024,E=7411039,C=9)
	CT(29314)=(S=7411040,E=7411071,C=90)
	CT(29315)=(S=7411072,E=7411103,C=5)
	CT(29316)=(S=7411104,E=7411135,C=9)
	CT(29317)=(S=7411136,E=7411167,C=8)
	CT(29318)=(S=7411168,E=7411171,C=73)
	CT(29319)=(S=7411172,E=7411175,C=84)
	CT(29320)=(S=7411184,E=7411191,C=3)
	CT(29321)=(S=7411192,E=7411199,C=2)
	CT(29322)=(S=7411200,E=7411583,C=11)
	CT(29323)=(S=7411584,E=7411711,C=5)
	CT(29324)=(S=7411712,E=7412735,C=1)
	CT(29325)=(S=7412736,E=7412991,C=8)
	CT(29326)=(S=7412992,E=7413119,C=29)
	CT(29327)=(S=7413120,E=7413199,C=6)
	CT(29328)=(S=7413200,E=7413207,C=0)
	CT(29329)=(S=7413208,E=7413215,C=2)
	CT(29330)=(S=7413216,E=7413231,C=7)
	CT(29331)=(S=7413232,E=7413239,C=0)
	CT(29332)=(S=7413240,E=7413243,C=86)
	CT(29333)=(S=7413244,E=7413247,C=8)
	CT(29334)=(S=7413248,E=7413375,C=6)
	CT(29335)=(S=7413376,E=7413503,C=4)
	CT(29336)=(S=7413504,E=7413759,C=1)
	CT(29337)=(S=7413760,E=7416831,C=2)
	CT(29338)=(S=7416832,E=7418879,C=1)
	CT(29339)=(S=7418880,E=7418895,C=0)
	CT(29340)=(S=7418896,E=7418911,C=2)
	CT(29341)=(S=7418912,E=7418943,C=11)
	CT(29342)=(S=7418944,E=7419007,C=87)
	CT(29343)=(S=7419008,E=7419011,C=0)
	CT(29344)=(S=7419012,E=7419015,C=8)
	CT(29345)=(S=7419016,E=7419023,C=6)
	CT(29346)=(S=7419024,E=7419027,C=0)
	CT(29347)=(S=7419028,E=7419031,C=74)
	CT(29348)=(S=7419032,E=7419035,C=7)
	CT(29349)=(S=7419036,E=7419039,C=2)
	CT(29350)=(S=7419040,E=7419071,C=1)
	CT(29351)=(S=7419072,E=7419103,C=6)
	CT(29352)=(S=7419112,E=7419119,C=0)
	CT(29353)=(S=7419120,E=7419135,C=2)
	CT(29354)=(S=7419136,E=7419391,C=3)
	CT(29355)=(S=7419392,E=7420799,C=1)
	CT(29356)=(S=7420800,E=7420863,C=6)
	CT(29357)=(S=7420864,E=7420895,C=156)
	CT(29358)=(S=7420896,E=7420899,C=1)
	CT(29359)=(S=7420900,E=7420903,C=2)
	CT(29360)=(S=7420904,E=7420911,C=74)
	CT(29361)=(S=7420912,E=7420927,C=8)
	CT(29362)=(S=7420928,E=7421215,C=6)
	CT(29363)=(S=7421216,E=7421247,C=10)
	CT(29364)=(S=7421248,E=7421287,C=0)
	CT(29365)=(S=7421288,E=7421291,C=6)
	CT(29366)=(S=7421292,E=7421295,C=11)
	CT(29367)=(S=7421296,E=7421311,C=2)
	CT(29368)=(S=7421312,E=7421439,C=9)
	CT(29369)=(S=7421440,E=7438847,C=1)
	CT(29370)=(S=7438848,E=7438911,C=2)
	CT(29371)=(S=7438912,E=7438943,C=6)
	CT(29372)=(S=7438944,E=7438967,C=1)
	CT(29373)=(S=7438968,E=7438971,C=73)
	CT(29374)=(S=7438972,E=7438975,C=86)
	CT(29375)=(S=7438976,E=7439359,C=6)
	CT(29376)=(S=7439360,E=7442431,C=1)
	CT(29377)=(S=7442432,E=7446527,C=2)
	CT(29378)=(S=7446528,E=7454719,C=11)
	CT(29379)=(S=7454720,E=7454783,C=0)
	CT(29380)=(S=7454784,E=7454847,C=6)
	CT(29381)=(S=7454848,E=7454975,C=2)
	CT(29382)=(S=7454976,E=7455231,C=4)
	CT(29383)=(S=7455232,E=7455743,C=1)
	CT(29384)=(S=7455744,E=7455999,C=9)
	CT(29385)=(S=7456000,E=7456015,C=0)
	CT(29386)=(S=7456016,E=7456031,C=2)
	CT(29387)=(S=7456032,E=7456039,C=7)
	CT(29388)=(S=7456040,E=7456047,C=2)
	CT(29389)=(S=7456048,E=7456055,C=77)
	CT(29390)=(S=7456056,E=7456063,C=2)
	CT(29391)=(S=7456064,E=7456067,C=73)
	CT(29392)=(S=7456068,E=7456071,C=164)
	CT(29393)=(S=7456072,E=7456079,C=10)
	CT(29394)=(S=7456080,E=7456095,C=6)
	CT(29395)=(S=7456096,E=7456099,C=73)
	CT(29396)=(S=7456100,E=7456103,C=1)
	CT(29397)=(S=7456104,E=7456107,C=4)
	CT(29398)=(S=7456108,E=7456111,C=74)
	CT(29399)=(S=7456112,E=7456127,C=0)
	CT(29400)=(S=7456128,E=7456255,C=2)
	CT(29401)=(S=7456256,E=7456639,C=6)
	CT(29402)=(S=7456640,E=7456767,C=75)
	CT(29403)=(S=7456768,E=7457535,C=1)
	CT(29404)=(S=7457536,E=7457663,C=0)
	CT(29405)=(S=7457664,E=7457791,C=77)
	CT(29406)=(S=7457792,E=7458815,C=1)
	CT(29407)=(S=7458816,E=7458879,C=2)
	CT(29408)=(S=7458880,E=7458911,C=8)
	CT(29409)=(S=7458912,E=7459327,C=1)
	CT(29410)=(S=7459328,E=7459839,C=5)
	CT(29411)=(S=7459840,E=7459903,C=1)
	CT(29412)=(S=7459904,E=7459935,C=4)
	CT(29413)=(S=7459936,E=7459939,C=0)
	CT(29414)=(S=7459940,E=7459943,C=1)
	CT(29415)=(S=7459944,E=7459947,C=2)
	CT(29416)=(S=7459948,E=7459951,C=5)
	CT(29417)=(S=7459952,E=7459967,C=74)
	CT(29418)=(S=7459968,E=7459999,C=2)
	CT(29419)=(S=7460000,E=7460007,C=74)
	CT(29420)=(S=7460008,E=7460015,C=0)
	CT(29421)=(S=7460016,E=7460023,C=10)
	CT(29422)=(S=7460024,E=7460031,C=1)
	CT(29423)=(S=7460032,E=7460095,C=8)
	CT(29424)=(S=7460096,E=7460223,C=1)
	CT(29425)=(S=7460224,E=7460351,C=2)
	CT(29426)=(S=7460352,E=7460863,C=1)
	CT(29427)=(S=7460864,E=7461375,C=6)
	CT(29428)=(S=7461376,E=7470079,C=1)
	CT(29429)=(S=7470080,E=7471103,C=8)
	CT(29430)=(S=7471104,E=7475199,C=74)
	CT(29431)=(S=7475200,E=7476991,C=2)
	CT(29432)=(S=7476992,E=7477247,C=73)
	CT(29433)=(S=7477248,E=7478271,C=9)
	CT(29434)=(S=7478272,E=7478527,C=1)
	CT(29435)=(S=7478528,E=7478719,C=6)
	CT(29436)=(S=7478720,E=7478751,C=7)
	CT(29437)=(S=7478784,E=7478815,C=6)
	CT(29438)=(S=7478816,E=7478831,C=9)
	CT(29439)=(S=7478832,E=7478847,C=6)
	CT(29440)=(S=7478848,E=7478855,C=0)
	CT(29441)=(S=7478856,E=7478863,C=4)
	CT(29442)=(S=7478864,E=7478879,C=74)
	CT(29443)=(S=7478880,E=7478911,C=0)
	CT(29444)=(S=7478912,E=7479039,C=6)
	CT(29445)=(S=7479040,E=7479071,C=84)
	CT(29446)=(S=7479072,E=7479103,C=6)
	CT(29447)=(S=7479112,E=7479119,C=0)
	CT(29448)=(S=7479120,E=7479135,C=2)
	CT(29449)=(S=7479136,E=7479143,C=0)
	CT(29450)=(S=7479144,E=7479151,C=2)
	CT(29451)=(S=7479152,E=7479167,C=6)
	CT(29452)=(S=7479168,E=7479231,C=4)
	CT(29453)=(S=7479232,E=7479247,C=0)
	CT(29454)=(S=7479248,E=7479255,C=73)
	CT(29455)=(S=7479256,E=7479263,C=2)
	CT(29456)=(S=7479264,E=7479279,C=4)
	CT(29457)=(S=7479280,E=7479287,C=74)
	CT(29458)=(S=7479288,E=7479295,C=4)
	CT(29459)=(S=7479296,E=7483391,C=9)
	CT(29460)=(S=7483392,E=7484415,C=2)
	CT(29461)=(S=7484416,E=7484927,C=6)
	CT(29462)=(S=7484928,E=7485439,C=1)
	CT(29463)=(S=7485440,E=7486463,C=74)
	CT(29464)=(S=7486464,E=7488767,C=1)
	CT(29465)=(S=7488768,E=7488943,C=2)
	CT(29466)=(S=7488944,E=7488991,C=90)
	CT(29467)=(S=7488992,E=7489023,C=4)
	CT(29468)=(S=7489024,E=7489535,C=6)
	CT(29469)=(S=7489536,E=7491327,C=0)
	CT(29470)=(S=7491328,E=7491391,C=74)
	CT(29471)=(S=7491392,E=7491455,C=1)
	CT(29472)=(S=7491456,E=7491583,C=4)
	CT(29473)=(S=7491584,E=7498751,C=1)
	CT(29474)=(S=7498752,E=7498943,C=6)
	CT(29475)=(S=7498944,E=7499007,C=10)
	CT(29476)=(S=7499008,E=7499263,C=3)
	CT(29477)=(S=7499264,E=7499279,C=1)
	CT(29478)=(S=7499280,E=7499287,C=74)
	CT(29479)=(S=7499288,E=7499295,C=6)
	CT(29480)=(S=7499296,E=7499303,C=73)
	CT(29481)=(S=7499304,E=7499311,C=74)
	CT(29482)=(S=7499312,E=7499327,C=2)
	CT(29483)=(S=7499328,E=7499391,C=1)
	CT(29484)=(S=7499392,E=7499519,C=6)
	CT(29485)=(S=7499520,E=7499551,C=1)
	CT(29486)=(S=7499552,E=7499583,C=6)
	CT(29487)=(S=7499584,E=7499647,C=2)
	CT(29488)=(S=7499648,E=7499679,C=0)
	CT(29489)=(S=7499680,E=7499711,C=1)
	CT(29490)=(S=7499712,E=7499775,C=6)
	CT(29491)=(S=7499776,E=7501823,C=1)
	CT(29492)=(S=7501824,E=7503871,C=74)
	CT(29493)=(S=7503872,E=7504127,C=3)
	CT(29494)=(S=7504128,E=7504135,C=2)
	CT(29495)=(S=7504136,E=7504143,C=84)
	CT(29496)=(S=7504144,E=7504151,C=74)
	CT(29497)=(S=7504152,E=7504159,C=154)
	CT(29498)=(S=7504160,E=7504175,C=7)
	CT(29499)=(S=7504176,E=7504191,C=2)
	CT(29500)=(S=7504192,E=7504255,C=6)
	CT(29501)=(S=7504256,E=7504319,C=0)
	CT(29502)=(S=7504320,E=7504383,C=6)
	CT(29503)=(S=7504384,E=7504639,C=84)
	CT(29504)=(S=7504640,E=7504895,C=3)
	CT(29505)=(S=7504896,E=7505151,C=1)
	CT(29506)=(S=7505152,E=7505407,C=5)
	CT(29507)=(S=7505408,E=7505423,C=73)
	CT(29508)=(S=7505440,E=7505471,C=2)
	CT(29509)=(S=7505472,E=7505487,C=74)
	CT(29510)=(S=7505488,E=7505495,C=8)
	CT(29511)=(S=7505496,E=7505503,C=84)
	CT(29512)=(S=7505504,E=7505567,C=2)
	CT(29513)=(S=7505568,E=7505591,C=73)
	CT(29514)=(S=7505592,E=7505599,C=86)
	CT(29515)=(S=7505600,E=7505663,C=2)
	CT(29516)=(S=7505664,E=7505919,C=1)
	CT(29517)=(S=7505920,E=7506431,C=9)
	CT(29518)=(S=7506432,E=7506943,C=1)
	CT(29519)=(S=7506944,E=7507199,C=9)
	CT(29520)=(S=7507200,E=7507231,C=6)
	CT(29521)=(S=7507232,E=7507239,C=2)
	CT(29522)=(S=7507240,E=7507247,C=6)
	CT(29523)=(S=7507248,E=7507263,C=74)
	CT(29524)=(S=7507264,E=7507271,C=1)
	CT(29525)=(S=7507272,E=7507279,C=7)
	CT(29526)=(S=7507288,E=7507295,C=74)
	CT(29527)=(S=7507296,E=7507311,C=0)
	CT(29528)=(S=7507312,E=7507327,C=73)
	CT(29529)=(S=7507328,E=7507391,C=1)
	CT(29530)=(S=7507392,E=7507407,C=0)
	CT(29531)=(S=7507408,E=7507415,C=7)
	CT(29532)=(S=7507416,E=7507423,C=10)
	CT(29533)=(S=7507424,E=7507455,C=6)
	CT(29534)=(S=7507456,E=7507591,C=2)
	CT(29535)=(S=7507592,E=7507599,C=4)
	CT(29536)=(S=7507600,E=7507615,C=8)
	CT(29537)=(S=7507616,E=7507631,C=0)
	CT(29538)=(S=7507632,E=7507647,C=2)
	CT(29539)=(S=7507648,E=7507711,C=159)
	CT(29540)=(S=7507712,E=7507967,C=4)
	CT(29541)=(S=7507968,E=7521279,C=2)
	CT(29542)=(S=7521280,E=7521791,C=1)
	CT(29543)=(S=7521792,E=7521919,C=0)
	CT(29544)=(S=7521920,E=7521951,C=10)
	CT(29545)=(S=7521952,E=7521983,C=9)
	CT(29546)=(S=7521984,E=7522023,C=2)
	CT(29547)=(S=7522024,E=7522031,C=77)
	CT(29548)=(S=7522032,E=7522039,C=74)
	CT(29549)=(S=7522040,E=7522047,C=1)
	CT(29550)=(S=7522048,E=7522111,C=6)
	CT(29551)=(S=7522112,E=7522127,C=8)
	CT(29552)=(S=7522128,E=7522175,C=74)
	CT(29553)=(S=7522176,E=7524351,C=6)
	CT(29554)=(S=7524352,E=7536639,C=1)
	CT(29555)=(S=7536640,E=7542783,C=6)
	CT(29556)=(S=7542784,E=7544319,C=1)
	CT(29557)=(S=7544320,E=7544351,C=2)
	CT(29558)=(S=7544352,E=7544383,C=0)
	CT(29559)=(S=7544384,E=7544447,C=9)
	CT(29560)=(S=7544448,E=7544663,C=2)
	CT(29561)=(S=7544664,E=7544671,C=0)
	CT(29562)=(S=7544672,E=7544703,C=6)
	CT(29563)=(S=7544704,E=7544767,C=3)
	CT(29564)=(S=7544768,E=7544799,C=2)
	CT(29565)=(S=7544800,E=7544831,C=7)
	CT(29566)=(S=7544832,E=7545855,C=1)
	CT(29567)=(S=7545856,E=7546879,C=2)
	CT(29568)=(S=7546880,E=7547391,C=6)
	CT(29569)=(S=7547392,E=7547423,C=0)
	CT(29570)=(S=7547456,E=7547471,C=77)
	CT(29571)=(S=7547472,E=7547487,C=9)
	CT(29572)=(S=7547488,E=7547503,C=2)
	CT(29573)=(S=7547504,E=7547511,C=9)
	CT(29574)=(S=7547512,E=7547519,C=10)
	CT(29575)=(S=7547520,E=7547647,C=7)
	CT(29576)=(S=7547648,E=7547903,C=9)
	CT(29577)=(S=7547904,E=7553023,C=1)
	CT(29578)=(S=7553024,E=7553279,C=0)
	CT(29579)=(S=7553280,E=7553535,C=2)
	CT(29580)=(S=7553536,E=7553791,C=7)
	CT(29581)=(S=7553792,E=7554047,C=3)
	CT(29582)=(S=7554048,E=7554303,C=6)
	CT(29583)=(S=7554304,E=7554367,C=0)
	CT(29584)=(S=7554368,E=7554383,C=1)
	CT(29585)=(S=7554384,E=7554399,C=4)
	CT(29586)=(S=7554400,E=7554431,C=6)
	CT(29587)=(S=7554432,E=7554463,C=4)
	CT(29588)=(S=7554464,E=7554495,C=73)
	CT(29589)=(S=7554496,E=7554511,C=2)
	CT(29590)=(S=7554512,E=7554519,C=84)
	CT(29591)=(S=7554520,E=7554527,C=74)
	CT(29592)=(S=7554528,E=7554543,C=2)
	CT(29593)=(S=7554544,E=7554559,C=4)
	CT(29594)=(S=7554560,E=7554815,C=0)
	CT(29595)=(S=7554816,E=7555071,C=6)
	CT(29596)=(S=7555072,E=7557119,C=11)
	CT(29597)=(S=7557120,E=7558143,C=9)
	CT(29598)=(S=7558144,E=7558207,C=1)
	CT(29599)=(S=7558208,E=7558271,C=154)
	CT(29600)=(S=7558272,E=7558303,C=88)
	CT(29601)=(S=7558304,E=7558311,C=6)
	CT(29602)=(S=7558312,E=7558319,C=10)
	CT(29603)=(S=7558320,E=7558327,C=11)
	CT(29604)=(S=7558328,E=7558335,C=2)
	CT(29605)=(S=7558336,E=7558367,C=1)
	CT(29606)=(S=7558368,E=7558463,C=10)
	CT(29607)=(S=7558464,E=7558495,C=74)
	CT(29608)=(S=7558496,E=7558527,C=2)
	CT(29609)=(S=7558528,E=7558528,C=5)
	CT(29610)=(S=7558529,E=7558535,C=0)
	CT(29611)=(S=7558536,E=7558543,C=2)
	CT(29612)=(S=7558544,E=7558559,C=9)
	CT(29613)=(S=7558560,E=7558591,C=6)
	CT(29614)=(S=7558592,E=7558655,C=1)
	CT(29615)=(S=7558656,E=7558911,C=6)
	CT(29616)=(S=7558912,E=7559167,C=3)
	CT(29617)=(S=7559168,E=7561215,C=6)
	CT(29618)=(S=7561216,E=7562239,C=4)
	CT(29619)=(S=7562240,E=7564287,C=1)
	CT(29620)=(S=7564288,E=7567359,C=4)
	CT(29621)=(S=7567360,E=7568383,C=1)
	CT(29622)=(S=7568384,E=7568399,C=0)
	CT(29623)=(S=7568400,E=7568415,C=1)
	CT(29624)=(S=7568416,E=7568423,C=0)
	CT(29625)=(S=7568424,E=7568431,C=4)
	CT(29626)=(S=7568432,E=7568447,C=2)
	CT(29627)=(S=7568448,E=7568479,C=74)
	CT(29628)=(S=7568480,E=7568511,C=4)
	CT(29629)=(S=7568512,E=7568895,C=2)
	CT(29630)=(S=7568896,E=7569023,C=8)
	CT(29631)=(S=7569024,E=7569031,C=73)
	CT(29632)=(S=7569032,E=7569055,C=2)
	CT(29633)=(S=7569056,E=7569087,C=90)
	CT(29634)=(S=7569088,E=7569151,C=6)
	CT(29635)=(S=7569152,E=7569375,C=84)
	CT(29636)=(S=7569376,E=7569407,C=2)
	CT(29637)=(S=7569408,E=7570431,C=0)
	CT(29638)=(S=7570432,E=7571455,C=5)
	CT(29639)=(S=7571456,E=7574015,C=6)
	CT(29640)=(S=7574016,E=7574079,C=2)
	CT(29641)=(S=7574080,E=7574111,C=0)
	CT(29642)=(S=7574112,E=7574127,C=5)
	CT(29643)=(S=7574128,E=7574135,C=7)
	CT(29644)=(S=7574136,E=7574143,C=11)
	CT(29645)=(S=7574144,E=7574527,C=10)
	CT(29646)=(S=7574528,E=7577599,C=1)
	CT(29647)=(S=7577600,E=7577727,C=6)
	CT(29648)=(S=7577728,E=7577791,C=8)
	CT(29649)=(S=7577792,E=7577855,C=4)
	CT(29650)=(S=7577856,E=7578111,C=6)
	CT(29651)=(S=7578112,E=7578623,C=2)
	CT(29652)=(S=7578624,E=7578879,C=5)
	CT(29653)=(S=7578880,E=7579039,C=2)
	CT(29654)=(S=7579040,E=7579047,C=11)
	CT(29655)=(S=7579048,E=7579055,C=0)
	CT(29656)=(S=7579056,E=7579063,C=6)
	CT(29657)=(S=7579064,E=7579071,C=2)
	CT(29658)=(S=7579072,E=7579135,C=9)
	CT(29659)=(S=7579136,E=7579199,C=0)
	CT(29660)=(S=7579200,E=7579231,C=1)
	CT(29661)=(S=7579232,E=7579263,C=74)
	CT(29662)=(S=7579280,E=7579391,C=2)
	CT(29663)=(S=7579392,E=7579519,C=77)
	CT(29664)=(S=7579520,E=7579647,C=2)
	CT(29665)=(S=7579648,E=7581695,C=1)
	CT(29666)=(S=7581696,E=7582207,C=2)
	CT(29667)=(S=7582208,E=7582223,C=8)
	CT(29668)=(S=7582224,E=7582231,C=0)
	CT(29669)=(S=7582232,E=7582235,C=86)
	CT(29670)=(S=7582236,E=7582239,C=0)
	CT(29671)=(S=7582240,E=7582255,C=6)
	CT(29672)=(S=7582256,E=7582263,C=74)
	CT(29673)=(S=7582264,E=7582271,C=3)
	CT(29674)=(S=7582272,E=7582303,C=6)
	CT(29675)=(S=7582304,E=7582311,C=4)
	CT(29676)=(S=7582312,E=7582319,C=0)
	CT(29677)=(S=7582320,E=7582327,C=2)
	CT(29678)=(S=7582328,E=7582463,C=74)
	CT(29679)=(S=7582464,E=7582719,C=2)
	CT(29680)=(S=7582720,E=7583743,C=1)
	CT(29681)=(S=7583744,E=7584255,C=4)
	CT(29682)=(S=7584256,E=7584447,C=77)
	CT(29683)=(S=7584448,E=7584511,C=0)
	CT(29684)=(S=7584528,E=7584543,C=75)
	CT(29685)=(S=7584544,E=7584575,C=4)
	CT(29686)=(S=7584576,E=7584591,C=2)
	CT(29687)=(S=7584592,E=7584599,C=6)
	CT(29688)=(S=7584600,E=7584603,C=92)
	CT(29689)=(S=7584604,E=7584607,C=2)
	CT(29690)=(S=7584608,E=7584639,C=73)
	CT(29691)=(S=7584640,E=7584767,C=0)
	CT(29692)=(S=7584768,E=7585279,C=73)
	CT(29693)=(S=7585280,E=7598079,C=1)
	CT(29694)=(S=7598080,E=7602175,C=4)
	CT(29695)=(S=7602176,E=7602183,C=74)
	CT(29696)=(S=7602184,E=7602191,C=1)
	CT(29697)=(S=7602192,E=7602199,C=0)
	CT(29698)=(S=7602200,E=7602207,C=1)
	CT(29699)=(S=7602208,E=7602239,C=77)
	CT(29700)=(S=7602240,E=7602271,C=7)
	CT(29701)=(S=7602272,E=7602303,C=5)
	CT(29702)=(S=7602304,E=7602431,C=2)
	CT(29703)=(S=7602432,E=7605247,C=1)
	CT(29704)=(S=7605248,E=7605279,C=2)
	CT(29705)=(S=7605280,E=7605287,C=84)
	CT(29706)=(S=7605288,E=7605295,C=74)
	CT(29707)=(S=7605296,E=7605303,C=7)
	CT(29708)=(S=7605304,E=7605311,C=73)
	CT(29709)=(S=7605312,E=7605375,C=156)
	CT(29710)=(S=7605376,E=7605503,C=7)
	CT(29711)=(S=7605504,E=7605759,C=1)
	CT(29712)=(S=7605760,E=7606271,C=7)
	CT(29713)=(S=7606272,E=7610367,C=1)
	CT(29714)=(S=7610368,E=7614463,C=6)
	CT(29715)=(S=7614464,E=7614975,C=8)
	CT(29716)=(S=7614976,E=7614991,C=1)
	CT(29717)=(S=7614992,E=7614999,C=0)
	CT(29718)=(S=7615000,E=7615007,C=74)
	CT(29719)=(S=7615008,E=7615023,C=9)
	CT(29720)=(S=7615024,E=7615031,C=2)
	CT(29721)=(S=7615032,E=7615039,C=8)
	CT(29722)=(S=7615040,E=7615103,C=4)
	CT(29723)=(S=7615104,E=7615231,C=10)
	CT(29724)=(S=7615232,E=7615487,C=7)
	CT(29725)=(S=7615488,E=7617023,C=1)
	CT(29726)=(S=7617024,E=7617151,C=77)
	CT(29727)=(S=7617152,E=7617167,C=1)
	CT(29728)=(S=7617168,E=7617215,C=2)
	CT(29729)=(S=7617216,E=7617223,C=74)
	CT(29730)=(S=7617224,E=7617231,C=84)
	CT(29731)=(S=7617232,E=7617247,C=1)
	CT(29732)=(S=7617248,E=7617279,C=3)
	CT(29733)=(S=7617280,E=7617535,C=9)
	CT(29734)=(S=7617536,E=7618559,C=1)
	CT(29735)=(S=7618560,E=7619071,C=2)
	CT(29736)=(S=7619072,E=7619199,C=1)
	CT(29737)=(S=7619200,E=7619231,C=4)
	CT(29738)=(S=7619232,E=7619247,C=0)
	CT(29739)=(S=7619264,E=7619271,C=75)
	CT(29740)=(S=7619272,E=7619279,C=74)
	CT(29741)=(S=7619280,E=7619295,C=8)
	CT(29742)=(S=7619296,E=7619303,C=0)
	CT(29743)=(S=7619304,E=7619311,C=8)
	CT(29744)=(S=7619312,E=7619319,C=73)
	CT(29745)=(S=7619320,E=7619327,C=10)
	CT(29746)=(S=7619328,E=7619455,C=6)
	CT(29747)=(S=7619456,E=7619583,C=2)
	CT(29748)=(S=7619584,E=7619615,C=159)
	CT(29749)=(S=7619616,E=7619647,C=6)
	CT(29750)=(S=7619648,E=7619711,C=4)
	CT(29751)=(S=7619712,E=7619719,C=11)
	CT(29752)=(S=7619728,E=7619743,C=3)
	CT(29753)=(S=7619744,E=7619759,C=74)
	CT(29754)=(S=7619768,E=7619775,C=8)
	CT(29755)=(S=7619776,E=7619791,C=84)
	CT(29756)=(S=7619792,E=7619799,C=75)
	CT(29757)=(S=7619800,E=7619807,C=2)
	CT(29758)=(S=7619808,E=7619815,C=74)
	CT(29759)=(S=7619816,E=7619823,C=6)
	CT(29760)=(S=7619824,E=7619831,C=4)
	CT(29761)=(S=7619832,E=7619839,C=74)
	CT(29762)=(S=7619840,E=7620223,C=1)
	CT(29763)=(S=7620224,E=7620351,C=2)
	CT(29764)=(S=7620352,E=7620607,C=77)
	CT(29765)=(S=7620608,E=7621631,C=4)
	CT(29766)=(S=7621632,E=7622655,C=1)
	CT(29767)=(S=7622656,E=7623679,C=2)
	CT(29768)=(S=7623680,E=7623935,C=6)
	CT(29769)=(S=7623936,E=7624191,C=1)
	CT(29770)=(S=7624192,E=7625087,C=7)
	CT(29771)=(S=7625088,E=7625103,C=9)
	CT(29772)=(S=7625104,E=7625119,C=1)
	CT(29773)=(S=7625120,E=7625151,C=6)
	CT(29774)=(S=7625152,E=7625183,C=2)
	CT(29775)=(S=7625184,E=7625199,C=73)
	CT(29776)=(S=7625200,E=7625215,C=2)
	CT(29777)=(S=7625216,E=7625279,C=0)
	CT(29778)=(S=7625280,E=7625287,C=8)
	CT(29779)=(S=7625288,E=7625291,C=0)
	CT(29780)=(S=7625292,E=7625295,C=73)
	CT(29781)=(S=7625296,E=7625311,C=1)
	CT(29782)=(S=7625312,E=7625343,C=77)
	CT(29783)=(S=7625344,E=7625359,C=73)
	CT(29784)=(S=7625360,E=7625375,C=2)
	CT(29785)=(S=7625376,E=7625399,C=74)
	CT(29786)=(S=7625400,E=7625407,C=1)
	CT(29787)=(S=7625408,E=7625423,C=2)
	CT(29788)=(S=7625424,E=7625431,C=74)
	CT(29789)=(S=7625432,E=7625439,C=6)
	CT(29790)=(S=7625440,E=7625455,C=75)
	CT(29791)=(S=7625456,E=7625471,C=4)
	CT(29792)=(S=7625472,E=7625983,C=2)
	CT(29793)=(S=7625984,E=7626111,C=10)
	CT(29794)=(S=7626112,E=7626127,C=73)
	CT(29795)=(S=7626128,E=7626143,C=2)
	CT(29796)=(S=7626144,E=7626239,C=6)
	CT(29797)=(S=7626240,E=7626495,C=2)
	CT(29798)=(S=7626496,E=7626751,C=1)
	CT(29799)=(S=7626752,E=7630847,C=11)
	CT(29800)=(S=7630848,E=7632383,C=1)
	CT(29801)=(S=7632384,E=7632511,C=11)
	CT(29802)=(S=7632512,E=7632575,C=9)
	CT(29803)=(S=7632576,E=7632623,C=2)
	CT(29804)=(S=7632624,E=7632639,C=0)
	CT(29805)=(S=7632640,E=7632895,C=4)
	CT(29806)=(S=7632896,E=7634943,C=6)
	CT(29807)=(S=7634944,E=7651583,C=1)
	CT(29808)=(S=7651584,E=7651591,C=7)
	CT(29809)=(S=7651592,E=7651599,C=87)
	CT(29810)=(S=7651600,E=7651647,C=1)
	CT(29811)=(S=7651648,E=7651663,C=11)
	CT(29812)=(S=7651664,E=7651679,C=6)
	CT(29813)=(S=7651680,E=7651711,C=2)
	CT(29814)=(S=7651712,E=7651727,C=4)
	CT(29815)=(S=7651728,E=7651735,C=73)
	CT(29816)=(S=7651740,E=7651741,C=5)
	CT(29817)=(S=7651742,E=7651743,C=8)
	CT(29818)=(S=7651744,E=7651747,C=4)
	CT(29819)=(S=7651752,E=7651759,C=84)
	CT(29820)=(S=7651760,E=7651767,C=1)
	CT(29821)=(S=7651768,E=7651771,C=0)
	CT(29822)=(S=7651772,E=7651775,C=74)
	CT(29823)=(S=7651776,E=7651799,C=73)
	CT(29824)=(S=7651800,E=7651807,C=84)
	CT(29825)=(S=7651808,E=7651839,C=2)
	CT(29826)=(S=7651840,E=7652607,C=1)
	CT(29827)=(S=7652608,E=7652735,C=5)
	CT(29828)=(S=7652736,E=7652743,C=74)
	CT(29829)=(S=7652744,E=7652751,C=2)
	CT(29830)=(S=7652752,E=7652759,C=0)
	CT(29831)=(S=7652776,E=7652783,C=2)
	CT(29832)=(S=7652784,E=7652799,C=8)
	CT(29833)=(S=7652800,E=7652863,C=7)
	CT(29834)=(S=7652864,E=7653279,C=1)
	CT(29835)=(S=7653280,E=7653287,C=6)
	CT(29836)=(S=7653288,E=7653295,C=4)
	CT(29837)=(S=7653296,E=7653311,C=2)
	CT(29838)=(S=7653312,E=7653319,C=74)
	CT(29839)=(S=7653328,E=7653343,C=73)
	CT(29840)=(S=7653344,E=7653359,C=0)
	CT(29841)=(S=7653360,E=7653375,C=2)
	CT(29842)=(S=7653376,E=7653887,C=6)
	CT(29843)=(S=7653888,E=7654399,C=4)
	CT(29844)=(S=7654400,E=7656447,C=1)
	CT(29845)=(S=7656448,E=7656479,C=6)
	CT(29846)=(S=7656480,E=7656511,C=11)
	CT(29847)=(S=7656512,E=7656519,C=0)
	CT(29848)=(S=7656520,E=7656527,C=74)
	CT(29849)=(S=7656528,E=7656543,C=9)
	CT(29850)=(S=7656544,E=7656551,C=74)
	CT(29851)=(S=7656552,E=7656559,C=84)
	CT(29852)=(S=7656560,E=7656575,C=8)
	CT(29853)=(S=7656576,E=7656607,C=86)
	CT(29854)=(S=7656608,E=7656623,C=1)
	CT(29855)=(S=7656624,E=7656631,C=4)
	CT(29856)=(S=7656632,E=7656639,C=84)
	CT(29857)=(S=7656640,E=7656723,C=0)
	CT(29858)=(S=7656724,E=7656727,C=2)
	CT(29859)=(S=7656728,E=7656767,C=74)
	CT(29860)=(S=7656768,E=7656959,C=1)
	CT(29861)=(S=7656960,E=7656975,C=9)
	CT(29862)=(S=7656976,E=7656983,C=2)
	CT(29863)=(S=7656984,E=7656991,C=4)
	CT(29864)=(S=7656992,E=7657039,C=1)
	CT(29865)=(S=7657040,E=7657055,C=2)
	CT(29866)=(S=7657056,E=7657087,C=7)
	CT(29867)=(S=7657088,E=7658495,C=1)
	CT(29868)=(S=7658496,E=7659519,C=2)
	CT(29869)=(S=7659520,E=7663615,C=1)
	CT(29870)=(S=7663616,E=7663871,C=0)
	CT(29871)=(S=7663872,E=7664127,C=9)
	CT(29872)=(S=7664128,E=7666175,C=1)
	CT(29873)=(S=7666176,E=7666495,C=0)
	CT(29874)=(S=7666496,E=7666559,C=1)
	CT(29875)=(S=7666560,E=7666635,C=73)
	CT(29876)=(S=7666636,E=7666639,C=8)
	CT(29877)=(S=7666640,E=7666655,C=7)
	CT(29878)=(S=7666656,E=7666687,C=8)
	CT(29879)=(S=7666688,E=7667199,C=1)
	CT(29880)=(S=7667200,E=7667295,C=2)
	CT(29881)=(S=7667296,E=7667327,C=74)
	CT(29882)=(S=7667328,E=7667455,C=1)
	CT(29883)=(S=7667456,E=7667519,C=0)
	CT(29884)=(S=7667520,E=7667583,C=6)
	CT(29885)=(S=7667584,E=7667711,C=1)
	CT(29886)=(S=7667712,E=7669759,C=11)
	CT(29887)=(S=7669760,E=7671807,C=1)
	CT(29888)=(S=7671808,E=7672319,C=6)
	CT(29889)=(S=7672320,E=7672335,C=8)
	CT(29890)=(S=7672336,E=7672343,C=74)
	CT(29891)=(S=7672344,E=7672351,C=2)
	CT(29892)=(S=7672352,E=7672383,C=7)
	CT(29893)=(S=7672384,E=7672399,C=8)
	CT(29894)=(S=7672400,E=7672407,C=73)
	CT(29895)=(S=7672408,E=7672415,C=2)
	CT(29896)=(S=7672416,E=7672447,C=8)
	CT(29897)=(S=7672448,E=7672543,C=2)
	CT(29898)=(S=7672544,E=7672551,C=84)
	CT(29899)=(S=7672552,E=7672559,C=0)
	CT(29900)=(S=7672560,E=7672575,C=77)
	CT(29901)=(S=7672576,E=7672831,C=9)
	CT(29902)=(S=7672832,E=7672847,C=0)
	CT(29903)=(S=7672848,E=7672863,C=77)
	CT(29904)=(S=7672864,E=7672871,C=93)
	CT(29905)=(S=7672872,E=7672879,C=7)
	CT(29906)=(S=7672880,E=7672895,C=74)
	CT(29907)=(S=7672896,E=7672903,C=0)
	CT(29908)=(S=7672904,E=7672911,C=2)
	CT(29909)=(S=7672912,E=7672927,C=6)
	CT(29910)=(S=7672928,E=7672943,C=2)
	CT(29911)=(S=7672944,E=7672951,C=86)
	CT(29912)=(S=7672952,E=7672959,C=159)
	CT(29913)=(S=7672960,E=7673023,C=7)
	CT(29914)=(S=7673024,E=7673087,C=6)
	CT(29915)=(S=7673088,E=7679487,C=1)
	CT(29916)=(S=7679488,E=7679743,C=2)
	CT(29917)=(S=7679744,E=7679999,C=3)
	CT(29918)=(S=7680000,E=7681023,C=1)
	CT(29919)=(S=7681024,E=7681279,C=6)
	CT(29920)=(S=7681280,E=7681319,C=2)
	CT(29921)=(S=7681328,E=7681343,C=1)
	CT(29922)=(S=7681344,E=7681407,C=6)
	CT(29923)=(S=7681408,E=7681423,C=0)
	CT(29924)=(S=7681432,E=7681435,C=5)
	CT(29925)=(S=7681436,E=7681439,C=73)
	CT(29926)=(S=7681440,E=7681455,C=0)
	CT(29927)=(S=7681456,E=7681471,C=1)
	CT(29928)=(S=7681472,E=7681535,C=6)
	CT(29929)=(S=7681536,E=7681791,C=74)
	CT(29930)=(S=7681792,E=7681919,C=2)
	CT(29931)=(S=7681920,E=7681983,C=6)
	CT(29932)=(S=7681984,E=7681999,C=89)
	CT(29933)=(S=7682000,E=7682015,C=2)
	CT(29934)=(S=7682016,E=7682031,C=0)
	CT(29935)=(S=7682032,E=7682039,C=4)
	CT(29936)=(S=7682040,E=7682047,C=86)
	CT(29937)=(S=7682048,E=7682303,C=9)
	CT(29938)=(S=7682304,E=7682687,C=1)
	CT(29939)=(S=7682688,E=7682703,C=6)
	CT(29940)=(S=7682704,E=7682719,C=2)
	CT(29941)=(S=7682720,E=7682751,C=7)
	CT(29942)=(S=7682752,E=7682783,C=10)
	CT(29943)=(S=7682784,E=7682799,C=2)
	CT(29944)=(S=7682800,E=7682807,C=84)
	CT(29945)=(S=7682808,E=7682815,C=0)
	CT(29946)=(S=7682816,E=7686655,C=1)
	CT(29947)=(S=7686656,E=7686719,C=2)
	CT(29948)=(S=7686720,E=7686751,C=1)
	CT(29949)=(S=7686752,E=7686767,C=0)
	CT(29950)=(S=7686768,E=7686783,C=74)
	CT(29951)=(S=7686784,E=7692287,C=1)
	CT(29952)=(S=7692288,E=7693311,C=4)
	CT(29953)=(S=7693312,E=7693823,C=1)
	CT(29954)=(S=7693824,E=7693887,C=77)
	CT(29955)=(S=7693888,E=7693951,C=74)
	CT(29956)=(S=7693952,E=7693983,C=0)
	CT(29957)=(S=7693984,E=7693991,C=74)
	CT(29958)=(S=7693992,E=7694047,C=2)
	CT(29959)=(S=7694048,E=7694095,C=74)
	CT(29960)=(S=7694096,E=7694111,C=1)
	CT(29961)=(S=7694112,E=7694119,C=74)
	CT(29962)=(S=7694120,E=7694127,C=1)
	CT(29963)=(S=7694128,E=7694151,C=74)
	CT(29964)=(S=7694152,E=7694159,C=1)
	CT(29965)=(S=7694160,E=7694167,C=84)
	CT(29966)=(S=7694168,E=7694175,C=199)
	CT(29967)=(S=7694176,E=7694191,C=9)
	CT(29968)=(S=7694192,E=7694199,C=0)
	CT(29969)=(S=7694200,E=7694207,C=4)
	CT(29970)=(S=7694208,E=7694223,C=1)
	CT(29971)=(S=7694224,E=7694239,C=8)
	CT(29972)=(S=7694240,E=7694247,C=2)
	CT(29973)=(S=7694248,E=7694255,C=74)
	CT(29974)=(S=7694256,E=7694271,C=2)
	CT(29975)=(S=7694272,E=7694335,C=11)
	CT(29976)=(S=7694336,E=7694495,C=2)
	CT(29977)=(S=7694496,E=7694503,C=0)
	CT(29978)=(S=7694504,E=7694511,C=1)
	CT(29979)=(S=7694512,E=7694519,C=73)
	CT(29980)=(S=7694520,E=7694523,C=7)
	CT(29981)=(S=7694524,E=7694527,C=8)
	CT(29982)=(S=7694528,E=7694559,C=74)
	CT(29983)=(S=7694560,E=7694567,C=89)
	CT(29984)=(S=7694568,E=7694575,C=4)
	CT(29985)=(S=7694576,E=7694591,C=10)
	CT(29986)=(S=7694592,E=7694847,C=7)
	CT(29987)=(S=7694848,E=7695359,C=1)
	CT(29988)=(S=7695360,E=7695871,C=2)
	CT(29989)=(S=7695872,E=7696383,C=6)
	CT(29990)=(S=7696384,E=7698431,C=1)
	CT(29991)=(S=7698432,E=7698439,C=7)
	CT(29992)=(S=7698440,E=7698455,C=0)
	CT(29993)=(S=7698456,E=7698463,C=86)
	CT(29994)=(S=7698464,E=7698471,C=73)
	CT(29995)=(S=7698472,E=7698479,C=0)
	CT(29996)=(S=7698480,E=7698487,C=2)
	CT(29997)=(S=7698496,E=7698887,C=1)
	CT(29998)=(S=7698888,E=7698895,C=74)
	CT(29999)=(S=7698896,E=7698911,C=3)
	CT(30000)=(S=7698912,E=7698927,C=75)
	CT(30001)=(S=7698928,E=7698935,C=7)
	CT(30002)=(S=7698936,E=7698943,C=0)
	CT(30003)=(S=7698944,E=7699071,C=11)
	CT(30004)=(S=7699072,E=7699199,C=1)
	CT(30005)=(S=7699200,E=7699455,C=6)
	CT(30006)=(S=7699456,E=7716863,C=1)
	CT(30007)=(S=7716864,E=7733247,C=4)
	CT(30008)=(S=7733248,E=7739391,C=2)
	CT(30009)=(S=7739392,E=7740159,C=1)
	CT(30010)=(S=7740160,E=7740415,C=2)
	CT(30011)=(S=7740416,E=7741439,C=1)
	CT(30012)=(S=7741440,E=7749631,C=6)
	CT(30013)=(S=7749632,E=7750399,C=1)
	CT(30014)=(S=7750400,E=7750463,C=0)
	CT(30015)=(S=7750464,E=7750511,C=2)
	CT(30016)=(S=7750512,E=7750527,C=1)
	CT(30017)=(S=7750528,E=7750591,C=6)
	CT(30018)=(S=7750592,E=7750599,C=73)
	CT(30019)=(S=7750600,E=7750607,C=86)
	CT(30020)=(S=7750608,E=7750623,C=0)
	CT(30021)=(S=7750624,E=7750655,C=4)
	CT(30022)=(S=7750656,E=7751679,C=11)
	CT(30023)=(S=7751680,E=7754239,C=1)
	CT(30024)=(S=7754240,E=7754271,C=74)
	CT(30025)=(S=7754272,E=7754287,C=0)
	CT(30026)=(S=7754288,E=7754367,C=2)
	CT(30027)=(S=7754368,E=7754495,C=73)
	CT(30028)=(S=7754496,E=7754751,C=2)
	CT(30029)=(S=7754752,E=7755263,C=1)
	CT(30030)=(S=7755264,E=7755775,C=2)
	CT(30031)=(S=7755776,E=7755783,C=4)
	CT(30032)=(S=7755784,E=7755791,C=10)
	CT(30033)=(S=7755792,E=7755807,C=0)
	CT(30034)=(S=7755808,E=7756287,C=1)
	CT(30035)=(S=7756288,E=7756543,C=73)
	CT(30036)=(S=7756544,E=7756671,C=6)
	CT(30037)=(S=7756672,E=7756687,C=74)
	CT(30038)=(S=7756688,E=7756703,C=6)
	CT(30039)=(S=7756704,E=7756719,C=75)
	CT(30040)=(S=7756720,E=7756735,C=4)
	CT(30041)=(S=7756736,E=7756767,C=2)
	CT(30042)=(S=7756768,E=7756775,C=74)
	CT(30043)=(S=7756776,E=7756783,C=4)
	CT(30044)=(S=7756784,E=7756799,C=1)
	CT(30045)=(S=7756800,E=7757311,C=73)
	CT(30046)=(S=7757312,E=7757823,C=4)
	CT(30047)=(S=7757824,E=7758591,C=74)
	CT(30048)=(S=7758592,E=7758655,C=8)
	CT(30049)=(S=7758656,E=7758719,C=74)
	CT(30050)=(S=7758720,E=7758847,C=9)
	CT(30051)=(S=7758848,E=7759359,C=5)
	CT(30052)=(S=7759360,E=7759367,C=11)
	CT(30053)=(S=7759368,E=7759375,C=8)
	CT(30054)=(S=7759376,E=7759399,C=1)
	CT(30055)=(S=7759400,E=7759423,C=2)
	CT(30056)=(S=7759424,E=7759487,C=0)
	CT(30057)=(S=7759488,E=7759615,C=4)
	CT(30058)=(S=7759616,E=7759751,C=2)
	CT(30059)=(S=7759752,E=7759759,C=161)
	CT(30060)=(S=7759760,E=7759775,C=8)
	CT(30061)=(S=7759808,E=7759839,C=6)
	CT(30062)=(S=7759840,E=7759855,C=77)
	CT(30063)=(S=7759856,E=7759863,C=8)
	CT(30064)=(S=7759864,E=7759871,C=157)
	CT(30065)=(S=7759872,E=7760639,C=2)
	CT(30066)=(S=7760640,E=7760703,C=0)
	CT(30067)=(S=7760704,E=7760767,C=11)
	CT(30068)=(S=7760768,E=7760783,C=77)
	CT(30069)=(S=7760784,E=7760791,C=0)
	CT(30070)=(S=7760792,E=7760799,C=2)
	CT(30071)=(S=7760800,E=7760815,C=6)
	CT(30072)=(S=7760824,E=7760831,C=0)
	CT(30073)=(S=7760832,E=7760895,C=5)
	CT(30074)=(S=7760896,E=7761919,C=2)
	CT(30075)=(S=7761920,E=7765759,C=1)
	CT(30076)=(S=7765760,E=7765887,C=0)
	CT(30077)=(S=7765888,E=7765919,C=1)
	CT(30078)=(S=7765920,E=7765951,C=0)
	CT(30079)=(S=7765952,E=7767039,C=6)
	CT(30080)=(S=7767040,E=7768063,C=1)
	CT(30081)=(S=7768064,E=7768575,C=74)
	CT(30082)=(S=7768576,E=7768991,C=0)
	CT(30083)=(S=7768992,E=7769023,C=7)
	CT(30084)=(S=7769024,E=7769087,C=6)
	CT(30085)=(S=7769088,E=7770111,C=8)
	CT(30086)=(S=7770112,E=7771135,C=1)
	CT(30087)=(S=7771136,E=7771647,C=73)
	CT(30088)=(S=7771648,E=7771903,C=9)
	CT(30089)=(S=7771904,E=7772111,C=2)
	CT(30090)=(S=7772112,E=7772115,C=4)
	CT(30091)=(S=7772116,E=7772119,C=74)
	CT(30092)=(S=7772120,E=7772123,C=0)
	CT(30093)=(S=7772124,E=7772127,C=74)
	CT(30094)=(S=7772128,E=7774207,C=2)
	CT(30095)=(S=7774208,E=7777279,C=9)
	CT(30096)=(S=7777280,E=7778303,C=3)
	CT(30097)=(S=7778304,E=7778559,C=6)
	CT(30098)=(S=7778560,E=7778815,C=2)
	CT(30099)=(S=7778816,E=7779071,C=1)
	CT(30100)=(S=7779072,E=7779295,C=84)
	CT(30101)=(S=7779296,E=7779327,C=90)
	CT(30102)=(S=7779328,E=7780607,C=1)
	CT(30103)=(S=7780608,E=7780863,C=4)
	CT(30104)=(S=7780864,E=7781631,C=1)
	CT(30105)=(S=7781632,E=7781887,C=7)
	CT(30106)=(S=7781888,E=7784447,C=1)
	CT(30107)=(S=7784448,E=7784959,C=7)
	CT(30108)=(S=7784960,E=7786495,C=1)
	CT(30109)=(S=7786496,E=7787519,C=0)
	CT(30110)=(S=7787520,E=7788031,C=1)
	CT(30111)=(S=7788032,E=7788543,C=7)
	CT(30112)=(S=7788544,E=7790591,C=6)
	CT(30113)=(S=7790592,E=7792383,C=1)
	CT(30114)=(S=7792384,E=7793151,C=9)
	CT(30115)=(S=7793152,E=7793663,C=6)
	CT(30116)=(S=7793664,E=7794431,C=2)
	CT(30117)=(S=7794432,E=7794687,C=1)
	CT(30118)=(S=7794688,E=7795199,C=2)
	CT(30119)=(S=7795200,E=7795455,C=1)
	CT(30120)=(S=7795456,E=7795711,C=2)
	CT(30121)=(S=7795712,E=7799327,C=1)
	CT(30122)=(S=7799328,E=7799335,C=0)
	CT(30123)=(S=7799336,E=7799351,C=74)
	CT(30124)=(S=7799352,E=7799359,C=0)
	CT(30125)=(S=7799360,E=7799391,C=74)
	CT(30126)=(S=7799392,E=7799423,C=161)
	CT(30127)=(S=7799424,E=7801087,C=1)
	CT(30128)=(S=7801088,E=7801343,C=8)
	CT(30129)=(S=7801344,E=7801471,C=1)
	CT(30130)=(S=7801472,E=7801487,C=0)
	CT(30131)=(S=7801488,E=7801503,C=8)
	CT(30132)=(S=7801504,E=7801511,C=2)
	CT(30133)=(S=7801512,E=7801515,C=8)
	CT(30134)=(S=7801516,E=7801519,C=84)
	CT(30135)=(S=7801520,E=7801527,C=74)
	CT(30136)=(S=7801528,E=7801599,C=2)
	CT(30137)=(S=7801600,E=7801727,C=0)
	CT(30138)=(S=7801728,E=7801855,C=74)
	CT(30139)=(S=7801856,E=7802367,C=0)
	CT(30140)=(S=7802368,E=7802623,C=9)
	CT(30141)=(S=7802624,E=7802655,C=73)
	CT(30142)=(S=7802656,E=7802687,C=2)
	CT(30143)=(S=7802688,E=7802703,C=0)
	CT(30144)=(S=7802704,E=7802719,C=86)
	CT(30145)=(S=7802720,E=7802735,C=0)
	CT(30146)=(S=7802736,E=7802751,C=2)
	CT(30147)=(S=7802752,E=7802759,C=74)
	CT(30148)=(S=7802760,E=7802767,C=1)
	CT(30149)=(S=7802768,E=7802775,C=2)
	CT(30150)=(S=7802776,E=7802783,C=84)
	CT(30151)=(S=7802784,E=7802815,C=11)
	CT(30152)=(S=7802816,E=7802879,C=9)
	CT(30153)=(S=7802880,E=7803135,C=1)
	CT(30154)=(S=7803136,E=7803167,C=6)
	CT(30155)=(S=7803168,E=7803199,C=0)
	CT(30156)=(S=7803200,E=7803263,C=6)
	CT(30157)=(S=7803264,E=7803327,C=0)
	CT(30158)=(S=7803328,E=7803391,C=11)
	CT(30159)=(S=7803392,E=7803439,C=0)
	CT(30160)=(S=7803440,E=7803455,C=4)
	CT(30161)=(S=7803456,E=7803519,C=6)
	CT(30162)=(S=7803520,E=7803535,C=11)
	CT(30163)=(S=7803536,E=7803543,C=84)
	CT(30164)=(S=7803544,E=7803551,C=74)
	CT(30165)=(S=7803552,E=7803575,C=2)
	CT(30166)=(S=7803576,E=7803583,C=11)
	CT(30167)=(S=7803584,E=7803607,C=1)
	CT(30168)=(S=7803608,E=7803615,C=2)
	CT(30169)=(S=7803616,E=7804927,C=1)
	CT(30170)=(S=7804928,E=7805695,C=2)
	CT(30171)=(S=7805696,E=7805759,C=7)
	CT(30172)=(S=7805760,E=7806463,C=1)
	CT(30173)=(S=7806464,E=7806471,C=0)
	CT(30174)=(S=7806472,E=7806479,C=2)
	CT(30175)=(S=7806480,E=7806495,C=9)
	CT(30176)=(S=7806496,E=7806511,C=84)
	CT(30177)=(S=7806512,E=7806527,C=1)
	CT(30178)=(S=7806528,E=7806591,C=77)
	CT(30179)=(S=7806592,E=7806655,C=6)
	CT(30180)=(S=7806656,E=7806719,C=2)
	CT(30181)=(S=7806720,E=7806847,C=3)
	CT(30182)=(S=7806848,E=7806879,C=2)
	CT(30183)=(S=7806880,E=7806895,C=4)
	CT(30184)=(S=7806896,E=7806911,C=9)
	CT(30185)=(S=7806912,E=7806943,C=1)
	CT(30186)=(S=7806944,E=7806951,C=0)
	CT(30187)=(S=7806952,E=7806959,C=7)
	CT(30188)=(S=7806960,E=7806975,C=6)
	CT(30189)=(S=7806976,E=7809103,C=1)
	CT(30190)=(S=7809104,E=7809119,C=84)
	CT(30191)=(S=7809120,E=7809127,C=92)
	CT(30192)=(S=7809128,E=7809135,C=0)
	CT(30193)=(S=7809136,E=7809151,C=5)
	CT(30194)=(S=7809152,E=7809567,C=1)
	CT(30195)=(S=7809600,E=7809663,C=3)
	CT(30196)=(S=7809664,E=7809679,C=1)
	CT(30197)=(S=7809680,E=7809687,C=8)
	CT(30198)=(S=7809688,E=7809695,C=4)
	CT(30199)=(S=7809696,E=7809727,C=6)
	CT(30200)=(S=7809728,E=7809759,C=7)
	CT(30201)=(S=7809760,E=7809791,C=1)
	CT(30202)=(S=7809792,E=7810047,C=4)
	CT(30203)=(S=7810048,E=7810559,C=1)
	CT(30204)=(S=7810560,E=7810815,C=3)
	CT(30205)=(S=7810816,E=7810895,C=2)
	CT(30206)=(S=7810896,E=7810903,C=8)
	CT(30207)=(S=7810904,E=7810911,C=74)
	CT(30208)=(S=7810912,E=7810927,C=2)
	CT(30209)=(S=7810928,E=7810943,C=73)
	CT(30210)=(S=7810944,E=7811071,C=2)
	CT(30211)=(S=7811072,E=7813119,C=1)
	CT(30212)=(S=7813120,E=7813247,C=7)
	CT(30213)=(S=7813248,E=7813375,C=6)
	CT(30214)=(S=7813376,E=7813887,C=1)
	CT(30215)=(S=7813888,E=7813951,C=6)
	CT(30216)=(S=7813952,E=7813959,C=2)
	CT(30217)=(S=7813960,E=7813967,C=8)
	CT(30218)=(S=7813968,E=7813975,C=89)
	CT(30219)=(S=7813976,E=7813983,C=2)
	CT(30220)=(S=7813984,E=7814015,C=3)
	CT(30221)=(S=7814016,E=7814911,C=1)
	CT(30222)=(S=7814912,E=7814927,C=10)
	CT(30223)=(S=7814928,E=7814943,C=2)
	CT(30224)=(S=7814944,E=7814975,C=1)
	CT(30225)=(S=7814976,E=7815007,C=3)
	CT(30226)=(S=7815008,E=7815023,C=8)
	CT(30227)=(S=7815024,E=7815039,C=2)
	CT(30228)=(S=7815040,E=7815055,C=77)
	CT(30229)=(S=7815056,E=7815111,C=2)
	CT(30230)=(S=7815112,E=7815135,C=0)
	CT(30231)=(S=7815136,E=7817215,C=6)
	CT(30232)=(S=7817216,E=7817471,C=2)
	CT(30233)=(S=7817472,E=7817599,C=77)
	CT(30234)=(S=7817600,E=7818047,C=7)
	CT(30235)=(S=7818048,E=7818175,C=6)
	CT(30236)=(S=7818176,E=7818191,C=7)
	CT(30237)=(S=7818192,E=7818207,C=1)
	CT(30238)=(S=7818208,E=7818223,C=2)
	CT(30239)=(S=7818224,E=7818239,C=9)
	CT(30240)=(S=7818240,E=7818495,C=3)
	CT(30241)=(S=7818496,E=7818591,C=0)
	CT(30242)=(S=7818592,E=7818623,C=6)
	CT(30243)=(S=7818624,E=7818751,C=9)
	CT(30244)=(S=7818752,E=7819519,C=1)
	CT(30245)=(S=7819520,E=7819775,C=7)
	CT(30246)=(S=7819776,E=7819783,C=0)
	CT(30247)=(S=7819784,E=7819807,C=2)
	CT(30248)=(S=7819808,E=7819839,C=6)
	CT(30249)=(S=7819840,E=7819903,C=4)
	CT(30250)=(S=7819904,E=7819919,C=11)
	CT(30251)=(S=7819920,E=7819927,C=0)
	CT(30252)=(S=7819928,E=7819983,C=2)
	CT(30253)=(S=7819984,E=7819999,C=1)
	CT(30254)=(S=7820000,E=7820023,C=74)
	CT(30255)=(S=7820024,E=7820031,C=86)
	CT(30256)=(S=7820032,E=7820287,C=2)
	CT(30257)=(S=7820288,E=7822335,C=1)
	CT(30258)=(S=7822336,E=7823359,C=10)
	CT(30259)=(S=7823360,E=7825407,C=1)
	CT(30260)=(S=7825408,E=7826431,C=2)
	CT(30261)=(S=7826432,E=7826943,C=1)
	CT(30262)=(S=7826944,E=7827007,C=5)
	CT(30263)=(S=7827008,E=7827031,C=74)
	CT(30264)=(S=7827032,E=7827039,C=2)
	CT(30265)=(S=7827040,E=7827055,C=5)
	CT(30266)=(S=7827056,E=7827071,C=74)
	CT(30267)=(S=7827072,E=7827199,C=5)
	CT(30268)=(S=7827200,E=7827455,C=10)
	CT(30269)=(S=7827456,E=7836671,C=1)
	CT(30270)=(S=7836672,E=7836735,C=84)
	CT(30271)=(S=7836736,E=7836767,C=0)
	CT(30272)=(S=7836768,E=7836775,C=3)
	CT(30273)=(S=7836776,E=7836783,C=2)
	CT(30274)=(S=7836784,E=7836831,C=6)
	CT(30275)=(S=7836832,E=7836863,C=1)
	CT(30276)=(S=7836864,E=7836927,C=2)
	CT(30277)=(S=7836928,E=7837183,C=6)
	CT(30278)=(S=7837184,E=7837439,C=2)
	CT(30279)=(S=7837440,E=7837631,C=4)
	CT(30280)=(S=7837632,E=7837695,C=1)
	CT(30281)=(S=7837696,E=7839871,C=77)
	CT(30282)=(S=7839872,E=7839935,C=157)
	CT(30283)=(S=7839936,E=7839943,C=4)
	CT(30284)=(S=7839944,E=7839951,C=1)
	CT(30285)=(S=7839952,E=7839967,C=3)
	CT(30286)=(S=7839968,E=7839983,C=0)
	CT(30287)=(S=7839984,E=7839999,C=9)
	CT(30288)=(S=7840000,E=7840031,C=6)
	CT(30289)=(S=7840032,E=7840095,C=0)
	CT(30290)=(S=7840104,E=7840119,C=2)
	CT(30291)=(S=7840128,E=7841791,C=1)
	CT(30292)=(S=7841792,E=7843839,C=2)
	CT(30293)=(S=7843840,E=7847935,C=1)
	CT(30294)=(S=7847936,E=7856127,C=6)
	CT(30295)=(S=7856128,E=7856271,C=73)
	CT(30296)=(S=7856272,E=7856287,C=0)
	CT(30297)=(S=7856288,E=7856383,C=2)
	CT(30298)=(S=7856384,E=7856639,C=0)
	CT(30299)=(S=7856640,E=7857151,C=4)
	CT(30300)=(S=7857152,E=7858175,C=2)
	CT(30301)=(S=7858176,E=7858687,C=1)
	CT(30302)=(S=7858688,E=7858943,C=7)
	CT(30303)=(S=7858944,E=7858959,C=156)
	CT(30304)=(S=7858960,E=7858975,C=74)
	CT(30305)=(S=7858976,E=7858983,C=73)
	CT(30306)=(S=7858984,E=7858991,C=8)
	CT(30307)=(S=7858992,E=7859007,C=4)
	CT(30308)=(S=7859008,E=7859047,C=91)
	CT(30309)=(S=7859048,E=7859071,C=2)
	CT(30310)=(S=7859072,E=7859135,C=1)
	CT(30311)=(S=7859136,E=7859151,C=6)
	CT(30312)=(S=7859152,E=7859167,C=74)
	CT(30313)=(S=7859168,E=7859183,C=2)
	CT(30314)=(S=7859184,E=7859191,C=6)
	CT(30315)=(S=7859192,E=7859199,C=74)
	CT(30316)=(S=7859200,E=7859711,C=8)
	CT(30317)=(S=7859712,E=7861759,C=2)
	CT(30318)=(S=7861760,E=7862271,C=8)
	CT(30319)=(S=7862272,E=7863295,C=1)
	CT(30320)=(S=7863296,E=7863327,C=0)
	CT(30321)=(S=7863328,E=7863359,C=2)
	CT(30322)=(S=7863360,E=7863391,C=0)
	CT(30323)=(S=7863392,E=7863399,C=1)
	CT(30324)=(S=7863400,E=7863407,C=74)
	CT(30325)=(S=7863408,E=7863423,C=163)
	CT(30326)=(S=7863424,E=7863431,C=74)
	CT(30327)=(S=7863432,E=7863439,C=8)
	CT(30328)=(S=7863440,E=7863455,C=4)
	CT(30329)=(S=7863456,E=7863471,C=74)
	CT(30330)=(S=7863472,E=7863487,C=0)
	CT(30331)=(S=7863488,E=7863519,C=4)
	CT(30332)=(S=7863520,E=7863535,C=76)
	CT(30333)=(S=7863536,E=7868415,C=1)
	CT(30334)=(S=7868416,E=7870463,C=0)
	CT(30335)=(S=7870464,E=7871487,C=1)
	CT(30336)=(S=7871488,E=7871743,C=10)
	CT(30337)=(S=7871744,E=7871807,C=0)
	CT(30338)=(S=7871808,E=7871871,C=10)
	CT(30339)=(S=7871872,E=7871887,C=6)
	CT(30340)=(S=7871888,E=7871895,C=2)
	CT(30341)=(S=7871896,E=7871903,C=74)
	CT(30342)=(S=7871904,E=7871935,C=2)
	CT(30343)=(S=7871936,E=7871967,C=4)
	CT(30344)=(S=7871968,E=7871975,C=74)
	CT(30345)=(S=7871976,E=7871983,C=4)
	CT(30346)=(S=7871984,E=7871999,C=0)
	CT(30347)=(S=7872000,E=7877119,C=1)
	CT(30348)=(S=7877120,E=7877151,C=84)
	CT(30349)=(S=7877152,E=7877167,C=7)
	CT(30350)=(S=7877168,E=7877183,C=5)
	CT(30351)=(S=7877184,E=7877279,C=6)
	CT(30352)=(S=7877280,E=7877295,C=2)
	CT(30353)=(S=7877296,E=7877303,C=84)
	CT(30354)=(S=7877304,E=7877311,C=11)
	CT(30355)=(S=7877312,E=7877631,C=2)
	CT(30356)=(S=7877632,E=7878655,C=1)
	CT(30357)=(S=7878656,E=7880703,C=4)
	CT(30358)=(S=7880704,E=7882751,C=1)
	CT(30359)=(S=7882752,E=7882767,C=2)
	CT(30360)=(S=7882768,E=7882783,C=10)
	CT(30361)=(S=7882784,E=7882815,C=1)
	CT(30362)=(S=7882816,E=7882831,C=2)
	CT(30363)=(S=7882832,E=7882839,C=11)
	CT(30364)=(S=7882840,E=7882847,C=4)
	CT(30365)=(S=7882848,E=7882879,C=11)
	CT(30366)=(S=7882880,E=7883007,C=1)
	CT(30367)=(S=7883008,E=7883263,C=6)
	CT(30368)=(S=7883264,E=7883775,C=2)
	CT(30369)=(S=7883776,E=7886847,C=1)
	CT(30370)=(S=7886848,E=7886855,C=0)
	CT(30371)=(S=7886856,E=7886863,C=1)
	CT(30372)=(S=7886864,E=7886879,C=2)
	CT(30373)=(S=7886880,E=7886895,C=4)
	CT(30374)=(S=7886896,E=7886911,C=2)
	CT(30375)=(S=7886912,E=7887023,C=0)
	CT(30376)=(S=7887024,E=7887039,C=4)
	CT(30377)=(S=7887040,E=7887103,C=8)
	CT(30378)=(S=7887104,E=7887167,C=10)
	CT(30379)=(S=7887168,E=7887175,C=2)
	CT(30380)=(S=7887176,E=7887183,C=4)
	CT(30381)=(S=7887184,E=7887191,C=73)
	CT(30382)=(S=7887192,E=7887199,C=74)
	CT(30383)=(S=7887200,E=7887231,C=75)
	CT(30384)=(S=7887232,E=7887359,C=2)
	CT(30385)=(S=7887360,E=7888127,C=1)
	CT(30386)=(S=7888128,E=7888383,C=2)
	CT(30387)=(S=7888384,E=7888895,C=1)
	CT(30388)=(S=7888896,E=7897087,C=9)
	CT(30389)=(S=7897088,E=7899135,C=1)
	CT(30390)=(S=7899136,E=7899143,C=73)
	CT(30391)=(S=7899144,E=7899151,C=2)
	CT(30392)=(S=7899160,E=7899167,C=86)
	CT(30393)=(S=7899168,E=7899183,C=8)
	CT(30394)=(S=7899184,E=7899199,C=73)
	CT(30395)=(S=7899200,E=7899263,C=6)
	CT(30396)=(S=7899264,E=7899327,C=1)
	CT(30397)=(S=7899328,E=7899391,C=2)
	CT(30398)=(S=7899392,E=7899519,C=1)
	CT(30399)=(S=7899520,E=7899647,C=2)
	CT(30400)=(S=7899664,E=7899679,C=73)
	CT(30401)=(S=7899680,E=7899711,C=0)
	CT(30402)=(S=7899712,E=7899727,C=11)
	CT(30403)=(S=7899728,E=7899743,C=5)
	CT(30404)=(S=7899744,E=7899775,C=4)
	CT(30405)=(S=7899776,E=7899839,C=2)
	CT(30406)=(S=7899840,E=7899903,C=4)
	CT(30407)=(S=7899904,E=7900671,C=5)
	CT(30408)=(S=7900672,E=7900927,C=6)
	CT(30409)=(S=7900928,E=7901055,C=2)
	CT(30410)=(S=7901056,E=7901087,C=1)
	CT(30411)=(S=7901088,E=7901183,C=6)
	CT(30412)=(S=7901184,E=7905279,C=0)
	CT(30413)=(S=7905280,E=7913471,C=74)
	CT(30414)=(S=7913472,E=7929855,C=1)
	CT(30415)=(S=7929856,E=7929863,C=0)
	CT(30416)=(S=7929864,E=7929887,C=1)
	CT(30417)=(S=7929888,E=7929919,C=2)
	CT(30418)=(S=7929920,E=7930111,C=6)
	CT(30419)=(S=7930112,E=7930175,C=10)
	CT(30420)=(S=7930176,E=7930239,C=6)
	CT(30421)=(S=7930240,E=7930879,C=2)
	CT(30422)=(S=7930880,E=7931391,C=1)
	CT(30423)=(S=7931392,E=7931903,C=7)
	CT(30424)=(S=7931904,E=7941119,C=1)
	CT(30425)=(S=7941120,E=7941631,C=0)
	CT(30426)=(S=7941632,E=7942655,C=1)
	CT(30427)=(S=7942656,E=7942663,C=4)
	CT(30428)=(S=7942664,E=7942671,C=1)
	CT(30429)=(S=7942672,E=7942687,C=6)
	CT(30430)=(S=7942688,E=7942695,C=74)
	CT(30431)=(S=7942696,E=7942719,C=2)
	CT(30432)=(S=7942720,E=7942783,C=6)
	CT(30433)=(S=7942784,E=7942799,C=74)
	CT(30434)=(S=7942800,E=7942807,C=9)
	CT(30435)=(S=7942808,E=7942823,C=2)
	CT(30436)=(S=7942832,E=7942847,C=9)
	CT(30437)=(S=7942848,E=7942879,C=0)
	CT(30438)=(S=7942880,E=7942911,C=6)
	CT(30439)=(S=7942912,E=7943167,C=1)
	CT(30440)=(S=7943168,E=7943311,C=74)
	CT(30441)=(S=7943312,E=7943327,C=77)
	CT(30442)=(S=7943328,E=7943359,C=1)
	CT(30443)=(S=7943360,E=7943367,C=0)
	CT(30444)=(S=7943368,E=7943375,C=7)
	CT(30445)=(S=7943376,E=7943423,C=1)
	CT(30446)=(S=7943424,E=7943679,C=6)
	CT(30447)=(S=7943680,E=7943807,C=10)
	CT(30448)=(S=7943808,E=7943839,C=0)
	CT(30449)=(S=7943848,E=7943855,C=8)
	CT(30450)=(S=7943856,E=7943863,C=1)
	CT(30451)=(S=7943872,E=7943935,C=6)
	CT(30452)=(S=7943936,E=7943999,C=1)
	CT(30453)=(S=7944000,E=7944127,C=6)
	CT(30454)=(S=7944128,E=7944191,C=159)
	CT(30455)=(S=7944192,E=7944831,C=1)
	CT(30456)=(S=7944832,E=7944839,C=2)
	CT(30457)=(S=7944840,E=7944871,C=1)
	CT(30458)=(S=7944872,E=7944879,C=4)
	CT(30459)=(S=7944880,E=7944887,C=2)
	CT(30460)=(S=7944888,E=7944895,C=74)
	CT(30461)=(S=7944896,E=7944959,C=10)
	CT(30462)=(S=7944960,E=7946239,C=1)
	CT(30463)=(S=7946240,E=7947263,C=6)
	CT(30464)=(S=7947264,E=7948287,C=1)
	CT(30465)=(S=7948288,E=7949311,C=73)
	CT(30466)=(S=7949312,E=7949823,C=1)
	CT(30467)=(S=7949824,E=7950079,C=6)
	CT(30468)=(S=7950080,E=7950207,C=0)
	CT(30469)=(S=7950208,E=7950271,C=1)
	CT(30470)=(S=7950272,E=7950335,C=73)
	CT(30471)=(S=7950336,E=7952383,C=2)
	CT(30472)=(S=7952384,E=7952639,C=6)
	CT(30473)=(S=7952640,E=7952895,C=1)
	CT(30474)=(S=7952896,E=7953407,C=0)
	CT(30475)=(S=7953408,E=7954431,C=2)
	CT(30476)=(S=7954432,E=7954943,C=10)
	CT(30477)=(S=7954944,E=7955455,C=73)
	CT(30478)=(S=7955456,E=7955463,C=74)
	CT(30479)=(S=7955464,E=7955471,C=2)
	CT(30480)=(S=7955472,E=7955487,C=74)
	CT(30481)=(S=7955488,E=7955503,C=7)
	CT(30482)=(S=7955504,E=7955511,C=89)
	CT(30483)=(S=7955512,E=7955519,C=2)
	CT(30484)=(S=7955520,E=7955583,C=6)
	CT(30485)=(S=7955584,E=7955775,C=1)
	CT(30486)=(S=7955776,E=7955839,C=2)
	CT(30487)=(S=7955840,E=7955847,C=74)
	CT(30488)=(S=7955848,E=7955855,C=0)
	CT(30489)=(S=7955856,E=7955871,C=4)
	CT(30490)=(S=7955872,E=7955879,C=0)
	CT(30491)=(S=7955880,E=7955887,C=8)
	CT(30492)=(S=7955888,E=7955895,C=92)
	CT(30493)=(S=7955896,E=7955903,C=74)
	CT(30494)=(S=7955904,E=7955919,C=6)
	CT(30495)=(S=7955920,E=7955935,C=1)
	CT(30496)=(S=7955936,E=7955967,C=6)
	CT(30497)=(S=7955968,E=7960575,C=2)
	CT(30498)=(S=7960576,E=7961599,C=5)
	CT(30499)=(S=7961600,E=7962367,C=6)
	CT(30500)=(S=7962368,E=7962399,C=10)
	CT(30501)=(S=7962400,E=7962431,C=89)
	CT(30502)=(S=7962432,E=7962559,C=6)
	CT(30503)=(S=7962560,E=7962591,C=0)
	CT(30504)=(S=7962592,E=7962623,C=8)
	CT(30505)=(S=7962624,E=7979007,C=6)
	CT(30506)=(S=7979008,E=7981055,C=1)
	CT(30507)=(S=7981056,E=7981103,C=0)
	CT(30508)=(S=7981104,E=7981115,C=4)
	CT(30509)=(S=7981116,E=7981119,C=84)
	CT(30510)=(S=7981120,E=7981183,C=6)
	CT(30511)=(S=7981184,E=7981247,C=2)
	CT(30512)=(S=7981248,E=7981255,C=1)
	CT(30513)=(S=7981256,E=7981263,C=2)
	CT(30514)=(S=7981264,E=7981271,C=0)
	CT(30515)=(S=7981272,E=7981279,C=2)
	CT(30516)=(S=7981280,E=7981295,C=0)
	CT(30517)=(S=7981296,E=7981311,C=7)
	CT(30518)=(S=7981312,E=7981567,C=1)
	CT(30519)=(S=7981568,E=7982079,C=8)
	CT(30520)=(S=7982080,E=7983103,C=1)
	CT(30521)=(S=7983104,E=7987199,C=0)
	CT(30522)=(S=7987200,E=7991295,C=1)
	CT(30523)=(S=7991296,E=7993343,C=4)
	CT(30524)=(S=7993344,E=7994367,C=1)
	CT(30525)=(S=7994368,E=7994943,C=6)
	CT(30526)=(S=7994944,E=7995007,C=9)
	CT(30527)=(S=7995008,E=7995135,C=6)
	CT(30528)=(S=7995136,E=7995391,C=1)
	CT(30529)=(S=7995392,E=7995399,C=3)
	CT(30530)=(S=7995400,E=7995407,C=6)
	CT(30531)=(S=7995408,E=7995423,C=5)
	CT(30532)=(S=7995424,E=7995455,C=6)
	CT(30533)=(S=7995456,E=7995647,C=1)
	CT(30534)=(S=7995648,E=7995903,C=2)
	CT(30535)=(S=7995904,E=7996415,C=10)
	CT(30536)=(S=7996416,E=7998335,C=1)
	CT(30537)=(S=7998336,E=7998463,C=7)
	CT(30538)=(S=7998464,E=7999231,C=1)
	CT(30539)=(S=7999232,E=7999487,C=4)
	CT(30540)=(S=7999488,E=8003583,C=2)
	CT(30541)=(S=8003584,E=8007679,C=6)
	CT(30542)=(S=8007680,E=8007999,C=1)
	CT(30543)=(S=8008000,E=8008063,C=6)
	CT(30544)=(S=8008064,E=8008127,C=0)
	CT(30545)=(S=8008128,E=8008135,C=2)
	CT(30546)=(S=8008136,E=8008143,C=0)
	CT(30547)=(S=8008144,E=8008159,C=10)
	CT(30548)=(S=8008160,E=8008167,C=74)
	CT(30549)=(S=8008168,E=8008191,C=2)
	CT(30550)=(S=8008192,E=8008199,C=7)
	CT(30551)=(S=8008200,E=8008207,C=4)
	CT(30552)=(S=8008208,E=8008255,C=2)
	CT(30553)=(S=8008256,E=8008319,C=7)
	CT(30554)=(S=8008320,E=8008447,C=4)
	CT(30555)=(S=8008448,E=8008703,C=1)
	CT(30556)=(S=8008704,E=8009727,C=10)
	CT(30557)=(S=8009728,E=8011775,C=73)
	CT(30558)=(S=8011776,E=8020479,C=1)
	CT(30559)=(S=8020480,E=8020735,C=4)
	CT(30560)=(S=8020736,E=8020799,C=9)
	CT(30561)=(S=8020800,E=8020831,C=0)
	CT(30562)=(S=8020832,E=8020839,C=84)
	CT(30563)=(S=8020840,E=8020847,C=2)
	CT(30564)=(S=8020848,E=8020859,C=0)
	CT(30565)=(S=8020860,E=8020863,C=4)
	CT(30566)=(S=8020864,E=8020991,C=6)
	CT(30567)=(S=8020992,E=8021007,C=0)
	CT(30568)=(S=8021008,E=8021023,C=2)
	CT(30569)=(S=8021024,E=8021055,C=6)
	CT(30570)=(S=8021056,E=8021119,C=9)
	CT(30571)=(S=8021120,E=8021247,C=87)
	CT(30572)=(S=8021248,E=8021503,C=6)
	CT(30573)=(S=8021504,E=8021519,C=1)
	CT(30574)=(S=8021520,E=8021527,C=0)
	CT(30575)=(S=8021528,E=8021535,C=4)
	CT(30576)=(S=8021536,E=8021543,C=84)
	CT(30577)=(S=8021544,E=8021559,C=74)
	CT(30578)=(S=8021560,E=8021567,C=2)
	CT(30579)=(S=8021568,E=8021599,C=1)
	CT(30580)=(S=8021600,E=8021611,C=0)
	CT(30581)=(S=8021612,E=8021613,C=73)
	CT(30582)=(S=8021614,E=8021615,C=0)
	CT(30583)=(S=8021616,E=8021619,C=11)
	CT(30584)=(S=8021620,E=8021631,C=4)
	CT(30585)=(S=8021632,E=8022015,C=2)
	CT(30586)=(S=8022016,E=8024063,C=0)
	CT(30587)=(S=8024064,E=8025087,C=1)
	CT(30588)=(S=8025088,E=8025855,C=9)
	CT(30589)=(S=8025856,E=8026111,C=1)
	CT(30590)=(S=8026112,E=8028159,C=9)
	CT(30591)=(S=8028160,E=8028175,C=2)
	CT(30592)=(S=8028176,E=8028191,C=74)
	CT(30593)=(S=8028192,E=8028239,C=6)
	CT(30594)=(S=8028240,E=8028247,C=9)
	CT(30595)=(S=8028248,E=8028255,C=0)
	CT(30596)=(S=8028264,E=8028271,C=8)
	CT(30597)=(S=8028272,E=8028279,C=2)
	CT(30598)=(S=8028280,E=8028287,C=1)
	CT(30599)=(S=8028288,E=8028415,C=6)
	CT(30600)=(S=8028416,E=8028479,C=11)
	CT(30601)=(S=8028480,E=8028511,C=77)
	CT(30602)=(S=8028512,E=8028535,C=74)
	CT(30603)=(S=8028536,E=8028543,C=5)
	CT(30604)=(S=8028544,E=8028575,C=0)
	CT(30605)=(S=8028576,E=8028607,C=2)
	CT(30606)=(S=8028608,E=8028623,C=74)
	CT(30607)=(S=8028624,E=8028631,C=6)
	CT(30608)=(S=8028632,E=8028639,C=0)
	CT(30609)=(S=8028640,E=8028655,C=2)
	CT(30610)=(S=8028656,E=8028671,C=6)
	CT(30611)=(S=8028672,E=8030207,C=2)
	CT(30612)=(S=8030208,E=8032255,C=1)
	CT(30613)=(S=8032256,E=8032263,C=74)
	CT(30614)=(S=8032264,E=8032271,C=84)
	CT(30615)=(S=8032272,E=8032279,C=4)
	CT(30616)=(S=8032280,E=8032287,C=3)
	CT(30617)=(S=8032288,E=8032319,C=9)
	CT(30618)=(S=8032320,E=8032383,C=10)
	CT(30619)=(S=8032384,E=8032511,C=1)
	CT(30620)=(S=8032512,E=8032767,C=2)
	CT(30621)=(S=8032768,E=8033279,C=9)
	CT(30622)=(S=8033280,E=8034303,C=0)
	CT(30623)=(S=8034304,E=8034351,C=2)
	CT(30624)=(S=8034352,E=8034359,C=84)
	CT(30625)=(S=8034360,E=8034399,C=2)
	CT(30626)=(S=8034400,E=8034431,C=6)
	CT(30627)=(S=8034432,E=8034495,C=2)
	CT(30628)=(S=8034496,E=8034559,C=1)
	CT(30629)=(S=8034560,E=8034815,C=6)
	CT(30630)=(S=8034816,E=8035327,C=3)
	CT(30631)=(S=8035328,E=8036351,C=1)
	CT(30632)=(S=8036352,E=8043519,C=4)
	CT(30633)=(S=8043520,E=8045567,C=1)
	CT(30634)=(S=8045568,E=8046079,C=2)
	CT(30635)=(S=8046080,E=8046335,C=1)
	CT(30636)=(S=8046336,E=8046399,C=0)
	CT(30637)=(S=8046400,E=8046591,C=6)
	CT(30638)=(S=8046592,E=8046607,C=74)
	CT(30639)=(S=8046608,E=8046615,C=4)
	CT(30640)=(S=8046616,E=8046623,C=0)
	CT(30641)=(S=8046624,E=8046631,C=2)
	CT(30642)=(S=8046640,E=8046647,C=74)
	CT(30643)=(S=8046648,E=8046655,C=0)
	CT(30644)=(S=8046656,E=8046719,C=1)
	CT(30645)=(S=8046720,E=8046723,C=2)
	CT(30646)=(S=8046724,E=8046727,C=8)
	CT(30647)=(S=8046728,E=8046735,C=2)
	CT(30648)=(S=8046736,E=8046743,C=74)
	CT(30649)=(S=8046744,E=8046751,C=9)
	CT(30650)=(S=8046752,E=8046783,C=0)
	CT(30651)=(S=8046784,E=8046855,C=2)
	CT(30652)=(S=8046856,E=8046863,C=11)
	CT(30653)=(S=8046864,E=8046879,C=92)
	CT(30654)=(S=8046880,E=8046895,C=8)
	CT(30655)=(S=8046896,E=8046911,C=1)
	CT(30656)=(S=8046912,E=8046975,C=0)
	CT(30657)=(S=8046976,E=8047103,C=9)
	CT(30658)=(S=8047104,E=8047119,C=0)
	CT(30659)=(S=8047120,E=8047135,C=2)
	CT(30660)=(S=8047136,E=8047167,C=6)
	CT(30661)=(S=8047168,E=8047199,C=0)
	CT(30662)=(S=8047200,E=8047231,C=10)
	CT(30663)=(S=8047232,E=8047615,C=6)
	CT(30664)=(S=8047616,E=8048639,C=1)
	CT(30665)=(S=8048640,E=8052735,C=2)
	CT(30666)=(S=8052736,E=8058879,C=1)
	CT(30667)=(S=8058880,E=8058895,C=84)
	CT(30668)=(S=8058896,E=8058903,C=8)
	CT(30669)=(S=8058904,E=8058911,C=1)
	CT(30670)=(S=8058912,E=8058927,C=74)
	CT(30671)=(S=8058928,E=8058943,C=1)
	CT(30672)=(S=8058944,E=8058975,C=2)
	CT(30673)=(S=8058976,E=8059007,C=155)
	CT(30674)=(S=8059008,E=8059071,C=2)
	CT(30675)=(S=8059072,E=8059135,C=7)
	CT(30676)=(S=8059136,E=8059903,C=2)
	CT(30677)=(S=8059904,E=8059919,C=0)
	CT(30678)=(S=8059920,E=8059927,C=73)
	CT(30679)=(S=8059928,E=8059935,C=7)
	CT(30680)=(S=8059936,E=8059967,C=2)
	CT(30681)=(S=8059968,E=8060031,C=6)
	CT(30682)=(S=8060032,E=8060047,C=2)
	CT(30683)=(S=8060048,E=8060055,C=0)
	CT(30684)=(S=8060056,E=8060063,C=73)
	CT(30685)=(S=8060064,E=8060079,C=9)
	CT(30686)=(S=8060080,E=8060087,C=86)
	CT(30687)=(S=8060088,E=8060095,C=73)
	CT(30688)=(S=8060096,E=8060127,C=6)
	CT(30689)=(S=8060128,E=8060159,C=4)
	CT(30690)=(S=8060160,E=8060415,C=2)
	CT(30691)=(S=8060416,E=8060479,C=9)
	CT(30692)=(S=8060480,E=8060543,C=92)
	CT(30693)=(S=8060544,E=8060671,C=6)
	CT(30694)=(S=8060672,E=8060735,C=156)
	CT(30695)=(S=8060736,E=8060743,C=1)
	CT(30696)=(S=8060744,E=8060747,C=4)
	CT(30697)=(S=8060748,E=8060751,C=2)
	CT(30698)=(S=8060752,E=8060767,C=9)
	CT(30699)=(S=8060768,E=8060799,C=5)
	CT(30700)=(S=8060800,E=8060927,C=2)
	CT(30701)=(S=8060928,E=8060943,C=6)
	CT(30702)=(S=8060944,E=8060959,C=84)
	CT(30703)=(S=8060960,E=8060991,C=9)
	CT(30704)=(S=8060992,E=8061055,C=2)
	CT(30705)=(S=8061056,E=8061119,C=1)
	CT(30706)=(S=8061120,E=8061183,C=9)
	CT(30707)=(S=8061184,E=8061311,C=2)
	CT(30708)=(S=8061312,E=8061439,C=8)
	CT(30709)=(S=8061440,E=8061951,C=0)
	CT(30710)=(S=8061952,E=8065023,C=1)
	CT(30711)=(S=8065024,E=8069119,C=11)
	CT(30712)=(S=8069120,E=8073215,C=6)
	CT(30713)=(S=8073216,E=8073471,C=2)
	CT(30714)=(S=8073472,E=8073535,C=84)
	CT(30715)=(S=8073536,E=8073599,C=7)
	CT(30716)=(S=8073600,E=8073727,C=1)
	CT(30717)=(S=8073728,E=8073759,C=2)
	CT(30718)=(S=8073760,E=8073791,C=9)
	CT(30719)=(S=8073792,E=8073855,C=164)
	CT(30720)=(S=8073856,E=8073887,C=0)
	CT(30721)=(S=8073888,E=8073919,C=1)
	CT(30722)=(S=8073920,E=8073983,C=2)
	CT(30723)=(S=8073984,E=8074111,C=0)
	CT(30724)=(S=8074112,E=8074239,C=9)
	CT(30725)=(S=8074240,E=8077055,C=1)
	CT(30726)=(S=8077056,E=8077311,C=4)
	CT(30727)=(S=8077312,E=8086143,C=1)
	CT(30728)=(S=8086144,E=8086159,C=0)
	CT(30729)=(S=8086160,E=8086175,C=2)
	CT(30730)=(S=8086176,E=8086239,C=6)
	CT(30731)=(S=8086240,E=8086271,C=2)
	CT(30732)=(S=8086272,E=8086335,C=9)
	CT(30733)=(S=8086336,E=8086399,C=6)
	CT(30734)=(S=8086400,E=8086559,C=1)
	CT(30735)=(S=8086560,E=8086687,C=0)
	CT(30736)=(S=8086688,E=8086719,C=6)
	CT(30737)=(S=8086720,E=8086751,C=2)
	CT(30738)=(S=8086752,E=8086783,C=7)
	CT(30739)=(S=8086784,E=8087039,C=1)
	CT(30740)=(S=8087040,E=8087295,C=0)
	CT(30741)=(S=8087296,E=8087423,C=1)
	CT(30742)=(S=8087424,E=8087487,C=2)
	CT(30743)=(S=8087488,E=8087519,C=0)
	CT(30744)=(S=8087520,E=8087535,C=2)
	CT(30745)=(S=8087536,E=8087551,C=8)
	CT(30746)=(S=8087552,E=8088583,C=2)
	CT(30747)=(S=8088584,E=8088591,C=74)
	CT(30748)=(S=8088592,E=8088607,C=6)
	CT(30749)=(S=8088608,E=8088639,C=4)
	CT(30750)=(S=8088640,E=8088671,C=0)
	CT(30751)=(S=8088672,E=8088679,C=74)
	CT(30752)=(S=8088680,E=8088683,C=0)
	CT(30753)=(S=8088684,E=8088687,C=8)
	CT(30754)=(S=8088688,E=8088703,C=10)
	CT(30755)=(S=8088704,E=8088719,C=1)
	CT(30756)=(S=8088720,E=8088727,C=73)
	CT(30757)=(S=8088728,E=8088735,C=2)
	CT(30758)=(S=8088736,E=8088767,C=6)
	CT(30759)=(S=8088768,E=8088775,C=2)
	CT(30760)=(S=8088776,E=8088783,C=4)
	CT(30761)=(S=8088784,E=8088799,C=1)
	CT(30762)=(S=8088800,E=8088807,C=4)
	CT(30763)=(S=8088808,E=8088815,C=2)
	CT(30764)=(S=8088816,E=8088823,C=84)
	CT(30765)=(S=8088824,E=8088831,C=86)
	CT(30766)=(S=8088832,E=8089087,C=6)
	CT(30767)=(S=8089088,E=8089343,C=9)
	CT(30768)=(S=8089344,E=8089599,C=6)
	CT(30769)=(S=8089600,E=8095743,C=1)
	CT(30770)=(S=8095744,E=8095759,C=8)
	CT(30771)=(S=8095760,E=8095767,C=4)
	CT(30772)=(S=8095768,E=8095775,C=84)
	CT(30773)=(S=8095776,E=8095807,C=0)
	CT(30774)=(S=8095808,E=8095815,C=7)
	CT(30775)=(S=8095816,E=8095823,C=10)
	CT(30776)=(S=8095824,E=8095839,C=1)
	CT(30777)=(S=8095840,E=8095871,C=5)
	CT(30778)=(S=8095872,E=8095999,C=4)
	CT(30779)=(S=8096000,E=8096767,C=1)
	CT(30780)=(S=8096768,E=8097791,C=6)
	CT(30781)=(S=8097792,E=8105983,C=1)
	CT(30782)=(S=8105984,E=8106015,C=88)
	CT(30783)=(S=8106016,E=8106031,C=4)
	CT(30784)=(S=8106032,E=8106039,C=2)
	CT(30785)=(S=8106040,E=8106043,C=84)
	CT(30786)=(S=8106044,E=8106047,C=1)
	CT(30787)=(S=8106048,E=8106055,C=10)
	CT(30788)=(S=8106056,E=8106063,C=162)
	CT(30789)=(S=8106064,E=8106079,C=1)
	CT(30790)=(S=8106080,E=8106087,C=8)
	CT(30791)=(S=8106088,E=8106095,C=2)
	CT(30792)=(S=8106096,E=8106103,C=0)
	CT(30793)=(S=8106104,E=8106111,C=74)
	CT(30794)=(S=8106112,E=8106239,C=2)
	CT(30795)=(S=8106240,E=8110079,C=1)
	CT(30796)=(S=8110080,E=8111103,C=9)
	CT(30797)=(S=8111104,E=8111615,C=1)
	CT(30798)=(S=8111616,E=8111871,C=2)
	CT(30799)=(S=8111872,E=8111999,C=6)
	CT(30800)=(S=8112000,E=8112127,C=1)
	CT(30801)=(S=8112128,E=8112159,C=84)
	CT(30802)=(S=8112160,E=8112191,C=2)
	CT(30803)=(S=8112192,E=8112255,C=6)
	CT(30804)=(S=8112256,E=8112383,C=0)
	CT(30805)=(S=8112384,E=8112639,C=4)
	CT(30806)=(S=8112640,E=8113151,C=8)
	CT(30807)=(S=8113152,E=8113663,C=9)
	CT(30808)=(S=8113664,E=8114175,C=1)
	CT(30809)=(S=8114176,E=8115199,C=0)
	CT(30810)=(S=8115200,E=8116223,C=6)
	CT(30811)=(S=8116224,E=8119295,C=2)
	CT(30812)=(S=8119296,E=8119807,C=6)
	CT(30813)=(S=8119808,E=8120063,C=2)
	CT(30814)=(S=8120064,E=8120191,C=156)
	CT(30815)=(S=8120192,E=8120319,C=74)
	CT(30816)=(S=8120320,E=8121343,C=1)
	CT(30817)=(S=8121344,E=8122367,C=4)
	CT(30818)=(S=8122368,E=8122879,C=9)
	CT(30819)=(S=8122880,E=8123007,C=1)
	CT(30820)=(S=8123008,E=8123071,C=3)
	CT(30821)=(S=8123080,E=8123087,C=10)
	CT(30822)=(S=8123088,E=8123103,C=2)
	CT(30823)=(S=8123104,E=8123111,C=8)
	CT(30824)=(S=8123112,E=8123119,C=2)
	CT(30825)=(S=8123120,E=8123135,C=4)
	CT(30826)=(S=8123136,E=8123391,C=0)
	CT(30827)=(S=8123392,E=8124415,C=1)
	CT(30828)=(S=8124416,E=8124671,C=6)
	CT(30829)=(S=8124672,E=8124927,C=1)
	CT(30830)=(S=8124928,E=8125439,C=6)
	CT(30831)=(S=8125440,E=8125567,C=9)
	CT(30832)=(S=8125568,E=8125695,C=4)
	CT(30833)=(S=8125696,E=8125951,C=1)
	CT(30834)=(S=8125952,E=8126015,C=2)
	CT(30835)=(S=8126016,E=8126047,C=6)
	CT(30836)=(S=8126056,E=8126063,C=8)
	CT(30837)=(S=8126064,E=8126079,C=0)
	CT(30838)=(S=8126080,E=8126207,C=6)
	CT(30839)=(S=8126208,E=8126271,C=73)
	CT(30840)=(S=8126272,E=8126335,C=8)
	CT(30841)=(S=8126336,E=8126407,C=2)
	CT(30842)=(S=8126408,E=8126415,C=74)
	CT(30843)=(S=8126416,E=8126431,C=0)
	CT(30844)=(S=8126432,E=8126439,C=9)
	CT(30845)=(S=8126440,E=8126455,C=2)
	CT(30846)=(S=8126456,E=8126463,C=4)
	CT(30847)=(S=8126464,E=8127487,C=6)
	CT(30848)=(S=8127488,E=8127743,C=4)
	CT(30849)=(S=8127744,E=8127999,C=6)
	CT(30850)=(S=8128000,E=8128031,C=9)
	CT(30851)=(S=8128032,E=8128063,C=7)
	CT(30852)=(S=8128064,E=8128127,C=1)
	CT(30853)=(S=8128128,E=8128191,C=10)
	CT(30854)=(S=8128192,E=8128223,C=73)
	CT(30855)=(S=8128224,E=8128239,C=84)
	CT(30856)=(S=8128240,E=8128255,C=156)
	CT(30857)=(S=8128256,E=8128511,C=4)
	CT(30858)=(S=8128512,E=8129791,C=9)
	CT(30859)=(S=8129792,E=8130047,C=5)
	CT(30860)=(S=8130048,E=8131071,C=1)
	CT(30861)=(S=8131072,E=8131327,C=2)
	CT(30862)=(S=8131328,E=8131455,C=0)
	CT(30863)=(S=8131456,E=8131583,C=10)
	CT(30864)=(S=8131584,E=8132607,C=1)
	CT(30865)=(S=8132608,E=8133631,C=2)
	CT(30866)=(S=8133632,E=8133823,C=6)
	CT(30867)=(S=8133824,E=8134015,C=1)
	CT(30868)=(S=8134016,E=8134079,C=9)
	CT(30869)=(S=8134080,E=8134143,C=77)
	CT(30870)=(S=8134144,E=8134399,C=4)
	CT(30871)=(S=8134400,E=8134655,C=1)
	CT(30872)=(S=8134656,E=8136815,C=2)
	CT(30873)=(S=8136816,E=8136927,C=1)
	CT(30874)=(S=8136928,E=8136943,C=3)
	CT(30875)=(S=8136948,E=8136951,C=4)
	CT(30876)=(S=8136952,E=8136959,C=74)
	CT(30877)=(S=8136960,E=8137151,C=2)
	CT(30878)=(S=8137152,E=8137215,C=75)
	CT(30879)=(S=8137216,E=8137471,C=1)
	CT(30880)=(S=8137472,E=8137727,C=156)
	CT(30881)=(S=8137728,E=8138239,C=2)
	CT(30882)=(S=8138240,E=8138495,C=6)
	CT(30883)=(S=8138496,E=8138559,C=1)
	CT(30884)=(S=8138560,E=8138623,C=2)
	CT(30885)=(S=8138624,E=8138687,C=0)
	CT(30886)=(S=8138688,E=8138751,C=2)
	CT(30887)=(S=8138752,E=8142847,C=6)
	CT(30888)=(S=8142848,E=8143487,C=1)
	CT(30889)=(S=8143488,E=8143519,C=7)
	CT(30890)=(S=8143520,E=8143527,C=74)
	CT(30891)=(S=8143536,E=8143551,C=6)
	CT(30892)=(S=8143552,E=8143567,C=2)
	CT(30893)=(S=8143568,E=8143583,C=6)
	CT(30894)=(S=8143584,E=8143615,C=2)
	CT(30895)=(S=8143616,E=8146943,C=1)
	CT(30896)=(S=8146944,E=8147199,C=6)
	CT(30897)=(S=8147200,E=8147455,C=74)
	CT(30898)=(S=8147456,E=8147711,C=5)
	CT(30899)=(S=8147712,E=8147839,C=10)
	CT(30900)=(S=8147840,E=8148991,C=2)
	CT(30901)=(S=8148992,E=8151039,C=1)
	CT(30902)=(S=8151040,E=8153087,C=2)
	CT(30903)=(S=8153088,E=8154111,C=10)
	CT(30904)=(S=8154112,E=8154115,C=0)
	CT(30905)=(S=8154116,E=8154119,C=86)
	CT(30906)=(S=8154120,E=8154127,C=1)
	CT(30907)=(S=8154136,E=8154143,C=91)
	CT(30908)=(S=8154144,E=8154151,C=2)
	CT(30909)=(S=8154152,E=8154159,C=1)
	CT(30910)=(S=8154160,E=8154167,C=86)
	CT(30911)=(S=8154168,E=8154175,C=2)
	CT(30912)=(S=8154176,E=8154223,C=8)
	CT(30913)=(S=8154224,E=8154239,C=7)
	CT(30914)=(S=8154240,E=8154303,C=9)
	CT(30915)=(S=8154304,E=8154367,C=2)
	CT(30916)=(S=8154368,E=8154371,C=3)
	CT(30917)=(S=8154372,E=8154375,C=7)
	CT(30918)=(S=8154376,E=8154383,C=163)
	CT(30919)=(S=8154392,E=8154399,C=3)
	CT(30920)=(S=8154400,E=8154431,C=77)
	CT(30921)=(S=8154432,E=8154463,C=0)
	CT(30922)=(S=8154464,E=8154471,C=1)
	CT(30923)=(S=8154472,E=8154475,C=84)
	CT(30924)=(S=8154476,E=8154479,C=2)
	CT(30925)=(S=8154480,E=8154495,C=9)
	CT(30926)=(S=8154496,E=8154879,C=2)
	CT(30927)=(S=8154880,E=8155135,C=6)
	CT(30928)=(S=8155136,E=8157183,C=1)
	CT(30929)=(S=8157184,E=8157951,C=3)
	CT(30930)=(S=8157952,E=8158719,C=4)
	CT(30931)=(S=8158720,E=8161279,C=1)
	CT(30932)=(S=8161280,E=8162303,C=6)
	CT(30933)=(S=8162304,E=8163839,C=2)
	CT(30934)=(S=8163840,E=8163903,C=6)
	CT(30935)=(S=8163904,E=8164223,C=2)
	CT(30936)=(S=8164224,E=8164351,C=1)
	CT(30937)=(S=8164352,E=8164991,C=0)
	CT(30938)=(S=8164992,E=8164999,C=9)
	CT(30939)=(S=8165000,E=8165000,C=4)
	CT(30940)=(S=8165004,E=8165007,C=5)
	CT(30941)=(S=8165008,E=8165023,C=2)
	CT(30942)=(S=8165024,E=8165039,C=73)
	CT(30943)=(S=8165040,E=8165119,C=2)
	CT(30944)=(S=8165120,E=8165631,C=1)
	CT(30945)=(S=8165632,E=8165695,C=74)
	CT(30946)=(S=8165696,E=8165759,C=4)
	CT(30947)=(S=8165760,E=8165887,C=6)
	CT(30948)=(S=8165888,E=8166271,C=2)
	CT(30949)=(S=8166272,E=8166335,C=9)
	CT(30950)=(S=8166336,E=8166367,C=7)
	CT(30951)=(S=8166368,E=8166383,C=73)
	CT(30952)=(S=8166384,E=8166399,C=4)
	CT(30953)=(S=8166400,E=8166655,C=1)
	CT(30954)=(S=8166656,E=8166719,C=11)
	CT(30955)=(S=8166720,E=8166783,C=73)
	CT(30956)=(S=8166784,E=8166911,C=3)
	CT(30957)=(S=8166912,E=8166927,C=11)
	CT(30958)=(S=8166928,E=8166939,C=0)
	CT(30959)=(S=8166940,E=8166943,C=4)
	CT(30960)=(S=8166944,E=8166975,C=10)
	CT(30961)=(S=8166976,E=8167039,C=92)
	CT(30962)=(S=8167040,E=8167103,C=74)
	CT(30963)=(S=8167104,E=8167135,C=8)
	CT(30964)=(S=8167136,E=8167167,C=5)
	CT(30965)=(S=8167168,E=8167423,C=2)
	CT(30966)=(S=8167424,E=8169471,C=1)
	CT(30967)=(S=8169472,E=8170495,C=0)
	CT(30968)=(S=8170496,E=8171519,C=1)
	CT(30969)=(S=8171520,E=8175615,C=0)
	CT(30970)=(S=8175616,E=8176127,C=1)
	CT(30971)=(S=8176128,E=8176383,C=6)
	CT(30972)=(S=8176384,E=8176511,C=74)
	CT(30973)=(S=8176512,E=8176527,C=5)
	CT(30974)=(S=8176528,E=8176543,C=2)
	CT(30975)=(S=8176544,E=8176575,C=6)
	CT(30976)=(S=8176576,E=8176607,C=88)
	CT(30977)=(S=8176608,E=8176639,C=6)
	CT(30978)=(S=8176640,E=8176895,C=1)
	CT(30979)=(S=8176896,E=8176959,C=73)
	CT(30980)=(S=8176960,E=8177023,C=7)
	CT(30981)=(S=8177024,E=8177119,C=6)
	CT(30982)=(S=8177120,E=8177151,C=5)
	CT(30983)=(S=8177152,E=8177279,C=6)
	CT(30984)=(S=8177280,E=8177407,C=0)
	CT(30985)=(S=8177408,E=8177471,C=6)
	CT(30986)=(S=8177472,E=8177519,C=9)
	CT(30987)=(S=8177520,E=8177535,C=89)
	CT(30988)=(S=8177536,E=8177663,C=6)
	CT(30989)=(S=8177664,E=8179711,C=1)
	CT(30990)=(S=8179712,E=8181759,C=2)
	CT(30991)=(S=8181760,E=8182015,C=6)
	CT(30992)=(S=8182016,E=8182143,C=10)
	CT(30993)=(S=8182144,E=8182207,C=8)
	CT(30994)=(S=8182208,E=8182239,C=6)
	CT(30995)=(S=8182240,E=8182271,C=5)
	CT(30996)=(S=8182272,E=8182655,C=9)
	CT(30997)=(S=8182656,E=8182783,C=2)
	CT(30998)=(S=8182784,E=8188095,C=1)
	CT(30999)=(S=8188096,E=8188127,C=76)
	CT(31000)=(S=8188128,E=8188415,C=2)
	CT(31001)=(S=8188416,E=8188671,C=1)
	CT(31002)=(S=8188672,E=8188799,C=6)
	CT(31003)=(S=8188800,E=8188863,C=0)
	CT(31004)=(S=8188864,E=8188927,C=1)
	CT(31005)=(S=8188928,E=8189183,C=8)
	CT(31006)=(S=8189184,E=8189439,C=2)
	CT(31007)=(S=8189440,E=8189503,C=0)
	CT(31008)=(S=8189504,E=8189567,C=7)
	CT(31009)=(S=8189568,E=8189887,C=2)
	CT(31010)=(S=8189888,E=8189951,C=4)
	CT(31011)=(S=8189952,E=8190079,C=1)
	CT(31012)=(S=8190080,E=8190095,C=73)
	CT(31013)=(S=8190096,E=8190111,C=2)
	CT(31014)=(S=8190112,E=8190143,C=86)
	CT(31015)=(S=8190144,E=8190207,C=8)
	CT(31016)=(S=8190208,E=8190975,C=1)
	CT(31017)=(S=8190976,E=8191231,C=0)
	CT(31018)=(S=8191232,E=8191487,C=4)
	CT(31019)=(S=8191488,E=8191551,C=1)
	CT(31020)=(S=8191552,E=8191615,C=0)
	CT(31021)=(S=8191616,E=8191743,C=6)
	CT(31022)=(S=8191744,E=8193279,C=2)
	CT(31023)=(S=8193280,E=8193535,C=10)
	CT(31024)=(S=8193536,E=8193791,C=2)
	CT(31025)=(S=8193792,E=8193919,C=0)
	CT(31026)=(S=8193920,E=8194047,C=6)
	CT(31027)=(S=8194048,E=8196095,C=2)
	CT(31028)=(S=8196096,E=8198143,C=4)
	CT(31029)=(S=8198144,E=8199167,C=3)
	CT(31030)=(S=8199168,E=8199935,C=2)
	CT(31031)=(S=8199936,E=8199999,C=87)
	CT(31032)=(S=8200000,E=8200063,C=2)
	CT(31033)=(S=8200064,E=8200127,C=6)
	CT(31034)=(S=8200128,E=8204287,C=1)
	CT(31035)=(S=8204288,E=8206463,C=2)
	CT(31036)=(S=8206464,E=8206591,C=7)
	CT(31037)=(S=8206592,E=8206847,C=6)
	CT(31038)=(S=8206848,E=8206911,C=11)
	CT(31039)=(S=8206912,E=8206975,C=2)
	CT(31040)=(S=8206976,E=8207103,C=1)
	CT(31041)=(S=8207104,E=8207359,C=8)
	CT(31042)=(S=8207360,E=8207487,C=6)
	CT(31043)=(S=8207488,E=8207615,C=10)
	CT(31044)=(S=8207616,E=8207743,C=6)
	CT(31045)=(S=8207744,E=8207935,C=1)
	CT(31046)=(S=8207936,E=8207967,C=0)
	CT(31047)=(S=8207968,E=8208087,C=4)
	CT(31048)=(S=8208088,E=8208095,C=6)
	CT(31049)=(S=8208096,E=8208111,C=9)
	CT(31050)=(S=8208112,E=8208127,C=2)
	CT(31051)=(S=8208128,E=8208159,C=0)
	CT(31052)=(S=8208160,E=8208175,C=2)
	CT(31053)=(S=8208176,E=8208191,C=0)
	CT(31054)=(S=8208192,E=8208255,C=4)
	CT(31055)=(S=8208256,E=8208383,C=0)
	CT(31056)=(S=8208384,E=8217343,C=1)
	CT(31057)=(S=8217344,E=8217599,C=4)
	CT(31058)=(S=8217600,E=8218623,C=2)
	CT(31059)=(S=8218624,E=8224767,C=1)
	CT(31060)=(S=8224768,E=8232959,C=6)
	CT(31061)=(S=8232960,E=8235007,C=74)
	CT(31062)=(S=8235008,E=8235263,C=0)
	CT(31063)=(S=8235264,E=8235519,C=1)
	CT(31064)=(S=8235520,E=8235775,C=2)
	CT(31065)=(S=8235776,E=8236031,C=1)
	CT(31066)=(S=8236032,E=8237055,C=2)
	CT(31067)=(S=8237056,E=8241151,C=6)
	CT(31068)=(S=8241152,E=8245247,C=2)
	CT(31069)=(S=8245248,E=8245311,C=1)
	CT(31070)=(S=8245312,E=8245375,C=6)
	CT(31071)=(S=8245376,E=8245439,C=74)
	CT(31072)=(S=8245440,E=8245567,C=6)
	CT(31073)=(S=8245568,E=8245631,C=77)
	CT(31074)=(S=8245632,E=8245695,C=0)
	CT(31075)=(S=8245696,E=8245759,C=6)
	CT(31076)=(S=8245760,E=8246271,C=1)
	CT(31077)=(S=8246272,E=8246399,C=10)
	CT(31078)=(S=8246400,E=8246527,C=11)
	CT(31079)=(S=8246528,E=8246655,C=1)
	CT(31080)=(S=8246656,E=8246687,C=74)
	CT(31081)=(S=8246688,E=8246719,C=0)
	CT(31082)=(S=8246720,E=8246751,C=89)
	CT(31083)=(S=8246752,E=8246783,C=3)
	CT(31084)=(S=8246784,E=8246847,C=11)
	CT(31085)=(S=8246848,E=8246879,C=0)
	CT(31086)=(S=8246880,E=8246911,C=1)
	CT(31087)=(S=8246912,E=8246943,C=2)
	CT(31088)=(S=8246944,E=8246975,C=156)
	CT(31089)=(S=8246976,E=8247039,C=8)
	CT(31090)=(S=8247040,E=8247103,C=1)
	CT(31091)=(S=8247104,E=8247167,C=2)
	CT(31092)=(S=8247168,E=8247295,C=8)
	CT(31093)=(S=8247296,E=8249343,C=1)
	CT(31094)=(S=8249344,E=8251903,C=9)
	CT(31095)=(S=8251904,E=8252415,C=11)
	CT(31096)=(S=8252416,E=8253439,C=73)
	CT(31097)=(S=8253440,E=8256575,C=6)
	CT(31098)=(S=8256576,E=8256703,C=2)
	CT(31099)=(S=8256704,E=8256767,C=7)
	CT(31100)=(S=8256768,E=8256879,C=0)
	CT(31101)=(S=8256880,E=8256895,C=11)
	CT(31102)=(S=8256896,E=8257023,C=8)
	CT(31103)=(S=8257024,E=8257075,C=0)
	CT(31104)=(S=8257080,E=8257083,C=2)
	CT(31105)=(S=8257084,E=8257151,C=0)
	CT(31106)=(S=8257152,E=8257279,C=1)
	CT(31107)=(S=8257280,E=8323071,C=2)
	CT(31108)=(S=8388608,E=8388615,C=38)
	CT(31109)=(S=8388616,E=8388623,C=24)
	CT(31110)=(S=8388624,E=8388631,C=27)
	CT(31111)=(S=8388632,E=8388639,C=18)
	CT(31112)=(S=8388640,E=8388671,C=38)
	CT(31113)=(S=8388672,E=8388679,C=36)
	CT(31114)=(S=8388680,E=8388687,C=24)
	CT(31115)=(S=8388688,E=8388703,C=18)
	CT(31116)=(S=8388704,E=8388711,C=28)
	CT(31117)=(S=8388712,E=8388713,C=31)
	CT(31118)=(S=8388714,E=8388714,C=18)
	CT(31119)=(S=8388715,E=8388719,C=32)
	CT(31120)=(S=8388720,E=8388735,C=28)
	CT(31121)=(S=8388736,E=8388751,C=18)
	CT(31122)=(S=8388752,E=8388763,C=28)
	CT(31123)=(S=8388768,E=8388775,C=18)
	CT(31124)=(S=8388776,E=8388783,C=31)
	CT(31125)=(S=8388784,E=8388799,C=33)
	CT(31126)=(S=8388800,E=8388807,C=28)
	CT(31127)=(S=8388808,E=8388863,C=32)
	CT(31128)=(S=8388864,E=8390399,C=29)
	CT(31129)=(S=8390400,E=8390655,C=28)
	CT(31130)=(S=8390656,E=8392703,C=29)
	CT(31131)=(S=8392704,E=8392959,C=14)
	CT(31132)=(S=8392960,E=8398591,C=29)
	CT(31133)=(S=8398592,E=8398847,C=27)
	CT(31134)=(S=8398848,E=8399359,C=14)
	CT(31135)=(S=8399360,E=8399615,C=29)
	CT(31136)=(S=8399616,E=8399871,C=78)
	CT(31137)=(S=8399872,E=8400127,C=29)
	CT(31138)=(S=8400128,E=8400383,C=15)
	CT(31139)=(S=8400384,E=8405247,C=29)
	CT(31140)=(S=8405248,E=8405311,C=68)
	CT(31141)=(S=8405312,E=8405343,C=28)
	CT(31142)=(S=8405344,E=8405351,C=14)
	CT(31143)=(S=8405352,E=8405359,C=59)
	CT(31144)=(S=8405360,E=8405375,C=15)
	CT(31145)=(S=8405376,E=8405383,C=42)
	CT(31146)=(S=8405384,E=8405391,C=18)
	CT(31147)=(S=8405392,E=8405399,C=28)
	CT(31148)=(S=8405400,E=8405407,C=25)
	CT(31149)=(S=8405408,E=8405439,C=26)
	CT(31150)=(S=8405440,E=8405447,C=34)
	CT(31151)=(S=8405448,E=8405455,C=15)
	CT(31152)=(S=8405456,E=8405471,C=28)
	CT(31153)=(S=8405472,E=8405503,C=21)
	CT(31154)=(S=8405504,E=8406015,C=15)
	CT(31155)=(S=8406016,E=8408063,C=18)
	CT(31156)=(S=8408064,E=8408575,C=24)
	CT(31157)=(S=8408576,E=8409087,C=12)
	CT(31158)=(S=8409088,E=8410623,C=29)
	CT(31159)=(S=8410624,E=8411135,C=14)
	CT(31160)=(S=8411136,E=8412415,C=29)
	CT(31161)=(S=8412416,E=8412671,C=12)
	CT(31162)=(S=8412672,E=8413695,C=29)
	CT(31163)=(S=8413696,E=8413951,C=14)
	CT(31164)=(S=8413952,E=8414207,C=29)
	CT(31165)=(S=8414208,E=8414463,C=78)
	CT(31166)=(S=8414464,E=8420351,C=29)
	CT(31167)=(S=8420352,E=8420607,C=31)
	CT(31168)=(S=8420608,E=8421119,C=29)
	CT(31169)=(S=8421120,E=8421127,C=15)
	CT(31170)=(S=8421136,E=8421143,C=12)
	CT(31171)=(S=8421144,E=8421151,C=14)
	CT(31172)=(S=8421152,E=8421167,C=36)
	CT(31173)=(S=8421168,E=8421175,C=28)
	CT(31174)=(S=8421176,E=8421183,C=15)
	CT(31175)=(S=8421184,E=8421191,C=28)
	CT(31176)=(S=8421192,E=8421199,C=16)
	CT(31177)=(S=8421200,E=8421215,C=58)
	CT(31178)=(S=8421216,E=8421223,C=18)
	CT(31179)=(S=8421224,E=8421231,C=19)
	CT(31180)=(S=8421232,E=8421247,C=38)
	CT(31181)=(S=8421248,E=8421263,C=12)
	CT(31182)=(S=8421264,E=8421271,C=18)
	CT(31183)=(S=8421272,E=8421279,C=28)
	CT(31184)=(S=8421280,E=8421287,C=25)
	CT(31185)=(S=8421288,E=8421295,C=37)
	CT(31186)=(S=8421304,E=8421311,C=19)
	CT(31187)=(S=8421312,E=8421375,C=23)
	CT(31188)=(S=8421376,E=8421887,C=29)
	CT(31189)=(S=8421888,E=8422399,C=42)
	CT(31190)=(S=8422400,E=8422911,C=29)
	CT(31191)=(S=8422912,E=8423167,C=6)
	CT(31192)=(S=8423168,E=8424191,C=29)
	CT(31193)=(S=8424192,E=8424447,C=17)
	CT(31194)=(S=8424448,E=8424575,C=26)
	CT(31195)=(S=8424576,E=8424599,C=12)
	CT(31196)=(S=8424600,E=8424607,C=32)
	CT(31197)=(S=8424608,E=8424623,C=18)
	CT(31198)=(S=8424624,E=8424639,C=51)
	CT(31199)=(S=8424640,E=8424647,C=18)
	CT(31200)=(S=8424648,E=8424655,C=25)
	CT(31201)=(S=8424656,E=8424663,C=28)
	CT(31202)=(S=8424664,E=8424671,C=36)
	CT(31203)=(S=8424672,E=8424679,C=38)
	CT(31204)=(S=8424680,E=8424687,C=48)
	CT(31205)=(S=8424688,E=8424703,C=68)
	CT(31206)=(S=8424704,E=8425215,C=34)
	CT(31207)=(S=8425216,E=8425471,C=29)
	CT(31208)=(S=8425472,E=8425727,C=78)
	CT(31209)=(S=8425728,E=8433663,C=29)
	CT(31210)=(S=8433664,E=8433919,C=28)
	CT(31211)=(S=8433920,E=8434175,C=29)
	CT(31212)=(S=8434176,E=8434687,C=34)
	CT(31213)=(S=8434688,E=8435711,C=29)
	CT(31214)=(S=8435712,E=8435967,C=0)
	CT(31215)=(S=8435968,E=8436223,C=4)
	CT(31216)=(S=8436224,E=8436991,C=29)
	CT(31217)=(S=8436992,E=8437247,C=78)
	CT(31218)=(S=8437248,E=8439551,C=29)
	CT(31219)=(S=8439552,E=8439807,C=14)
	CT(31220)=(S=8439808,E=8440831,C=29)
	CT(31221)=(S=8440832,E=8440959,C=18)
	CT(31222)=(S=8440960,E=8440991,C=42)
	CT(31223)=(S=8440992,E=8441023,C=18)
	CT(31224)=(S=8441024,E=8441039,C=36)
	CT(31225)=(S=8441040,E=8441047,C=12)
	CT(31226)=(S=8441048,E=8441055,C=58)
	CT(31227)=(S=8441056,E=8441071,C=36)
	CT(31228)=(S=8441072,E=8441087,C=23)
	CT(31229)=(S=8441088,E=8443391,C=29)
	CT(31230)=(S=8443392,E=8443647,C=47)
	CT(31231)=(S=8443648,E=8447999,C=29)
	CT(31232)=(S=8448000,E=8448255,C=14)
	CT(31233)=(S=8448256,E=8448511,C=78)
	CT(31234)=(S=8448512,E=8448767,C=23)
	CT(31235)=(S=8448768,E=8450047,C=29)
	CT(31236)=(S=8450048,E=8450303,C=14)
	CT(31237)=(S=8450304,E=8450815,C=29)
	CT(31238)=(S=8450816,E=8451071,C=14)
	CT(31239)=(S=8451072,E=8451583,C=29)
	CT(31240)=(S=8451584,E=8451839,C=28)
	CT(31241)=(S=8451840,E=8452607,C=29)
	CT(31242)=(S=8452608,E=8452863,C=0)
	CT(31243)=(S=8452864,E=8454143,C=29)
	CT(31244)=(S=8454144,E=8454399,C=120)
	CT(31245)=(S=8454400,E=8456959,C=29)
	CT(31246)=(S=8456960,E=8457471,C=14)
	CT(31247)=(S=8457472,E=8457727,C=28)
	CT(31248)=(S=8457728,E=8458239,C=29)
	CT(31249)=(S=8458240,E=8458495,C=19)
	CT(31250)=(S=8458496,E=8458751,C=29)
	CT(31251)=(S=8458752,E=8459007,C=111)
	CT(31252)=(S=8459008,E=8459263,C=29)
	CT(31253)=(S=8459264,E=8459519,C=12)
	CT(31254)=(S=8459520,E=8460799,C=29)
	CT(31255)=(S=8460800,E=8461055,C=28)
	CT(31256)=(S=8461056,E=8461311,C=42)
	CT(31257)=(S=8461312,E=8462079,C=29)
	CT(31258)=(S=8462080,E=8462335,C=14)
	CT(31259)=(S=8462336,E=8465663,C=29)
	CT(31260)=(S=8465664,E=8465791,C=135)
	CT(31261)=(S=8465920,E=8469503,C=29)
	CT(31262)=(S=8469504,E=8469759,C=2)
	CT(31263)=(S=8469760,E=8471295,C=29)
	CT(31264)=(S=8471296,E=8471551,C=14)
	CT(31265)=(S=8471552,E=8471807,C=29)
	CT(31266)=(S=8471808,E=8472319,C=28)
	CT(31267)=(S=8472320,E=8472831,C=29)
	CT(31268)=(S=8472832,E=8473087,C=28)
	CT(31269)=(S=8473088,E=8474111,C=29)
	CT(31270)=(S=8474112,E=8474367,C=0)
	CT(31271)=(S=8474368,E=8476671,C=29)
	CT(31272)=(S=8476672,E=8476927,C=12)
	CT(31273)=(S=8476928,E=8477183,C=29)
	CT(31274)=(S=8477184,E=8477439,C=200)
	CT(31275)=(S=8477440,E=8478207,C=29)
	CT(31276)=(S=8478208,E=8478463,C=0)
	CT(31277)=(S=8478464,E=8478719,C=29)
	CT(31278)=(S=8478720,E=8478975,C=0)
	CT(31279)=(S=8478976,E=8479231,C=78)
	CT(31280)=(S=8479232,E=8479743,C=29)
	CT(31281)=(S=8479744,E=8479999,C=78)
	CT(31282)=(S=8480000,E=8480255,C=29)
	CT(31283)=(S=8480256,E=8480511,C=12)
	CT(31284)=(S=8480512,E=8480767,C=28)
	CT(31285)=(S=8480768,E=8481023,C=12)
	CT(31286)=(S=8481024,E=8486143,C=29)
	CT(31287)=(S=8486144,E=8486399,C=36)
	CT(31288)=(S=8486400,E=8486655,C=29)
	CT(31289)=(S=8486656,E=8486911,C=0)
	CT(31290)=(S=8486912,E=8487167,C=78)
	CT(31291)=(S=8487168,E=8487423,C=34)
	CT(31292)=(S=8487424,E=8487935,C=29)
	CT(31293)=(S=8487936,E=8488191,C=34)
	CT(31294)=(S=8488192,E=8488959,C=29)
	CT(31295)=(S=8488960,E=8489215,C=2)
	CT(31296)=(S=8489216,E=8490495,C=29)
	CT(31297)=(S=8490496,E=8490751,C=24)
	CT(31298)=(S=8490752,E=8491007,C=28)
	CT(31299)=(S=8491008,E=8497407,C=29)
	CT(31300)=(S=8497408,E=8497663,C=14)
	CT(31301)=(S=8497664,E=8498431,C=29)
	CT(31302)=(S=8498432,E=8498687,C=78)
	CT(31303)=(S=8498688,E=8498943,C=29)
	CT(31304)=(S=8498944,E=8499199,C=12)
	CT(31305)=(S=8499200,E=8499455,C=29)
	CT(31306)=(S=8499456,E=8499711,C=27)
	CT(31307)=(S=8499712,E=8499967,C=19)
	CT(31308)=(S=8499968,E=8500223,C=29)
	CT(31309)=(S=8500224,E=8500479,C=0)
	CT(31310)=(S=8500480,E=8501759,C=12)
	CT(31311)=(S=8501760,E=8502015,C=29)
	CT(31312)=(S=8502016,E=8502271,C=28)
	CT(31313)=(S=8502272,E=8503807,C=29)
	CT(31314)=(S=8503808,E=8504319,C=34)
	CT(31315)=(S=8504320,E=8505087,C=29)
	CT(31316)=(S=8505088,E=8505343,C=12)
	CT(31317)=(S=8505344,E=8506623,C=29)
	CT(31318)=(S=8506624,E=8506655,C=130)
	CT(31319)=(S=8506656,E=8506719,C=105)
	CT(31320)=(S=8506720,E=8506751,C=111)
	CT(31321)=(S=8506752,E=8506815,C=105)
	CT(31322)=(S=8506816,E=8506879,C=126)
	CT(31323)=(S=8506880,E=8507135,C=28)
	CT(31324)=(S=8507136,E=8507391,C=29)
	CT(31325)=(S=8507392,E=8507647,C=23)
	CT(31326)=(S=8507648,E=8508927,C=29)
	CT(31327)=(S=8508928,E=8509183,C=28)
	CT(31328)=(S=8509184,E=8509439,C=14)
	CT(31329)=(S=8509440,E=8509695,C=29)
	CT(31330)=(S=8509696,E=8509951,C=28)
	CT(31331)=(S=8509952,E=8513535,C=29)
	CT(31332)=(S=8513536,E=8513663,C=133)
	CT(31333)=(S=8513664,E=8513791,C=105)
	CT(31334)=(S=8513792,E=8514047,C=28)
	CT(31335)=(S=8514048,E=8514303,C=14)
	CT(31336)=(S=8514304,E=8515583,C=29)
	CT(31337)=(S=8515584,E=8516351,C=27)
	CT(31338)=(S=8516352,E=8517375,C=29)
	CT(31339)=(S=8517376,E=8517631,C=28)
	CT(31340)=(S=8517632,E=8517887,C=29)
	CT(31341)=(S=8517888,E=8518143,C=2)
	CT(31342)=(S=8518144,E=8519167,C=29)
	CT(31343)=(S=8519168,E=8519423,C=6)
	CT(31344)=(S=8519424,E=8519679,C=29)
	CT(31345)=(S=8519680,E=8519687,C=14)
	CT(31346)=(S=8519688,E=8519695,C=41)
	CT(31347)=(S=8519696,E=8519703,C=28)
	CT(31348)=(S=8519704,E=8519711,C=96)
	CT(31349)=(S=8519712,E=8519743,C=31)
	CT(31350)=(S=8519744,E=8519751,C=42)
	CT(31351)=(S=8519752,E=8519759,C=28)
	CT(31352)=(S=8519760,E=8519767,C=14)
	CT(31353)=(S=8519768,E=8519775,C=59)
	CT(31354)=(S=8519776,E=8519807,C=12)
	CT(31355)=(S=8519808,E=8519871,C=15)
	CT(31356)=(S=8519872,E=8519879,C=12)
	CT(31357)=(S=8519880,E=8519887,C=19)
	CT(31358)=(S=8519888,E=8519895,C=15)
	CT(31359)=(S=8519896,E=8519903,C=18)
	CT(31360)=(S=8519904,E=8519911,C=14)
	CT(31361)=(S=8519912,E=8519919,C=31)
	CT(31362)=(S=8519920,E=8519935,C=30)
	CT(31363)=(S=8519936,E=8522751,C=29)
	CT(31364)=(S=8522752,E=8523007,C=78)
	CT(31365)=(S=8523008,E=8523519,C=29)
	CT(31366)=(S=8523520,E=8523775,C=78)
	CT(31367)=(S=8523776,E=8525823,C=29)
	CT(31368)=(S=8525824,E=8526079,C=14)
	CT(31369)=(S=8526080,E=8526335,C=15)
	CT(31370)=(S=8526336,E=8526847,C=29)
	CT(31371)=(S=8526848,E=8527103,C=19)
	CT(31372)=(S=8527104,E=8527871,C=29)
	CT(31373)=(S=8527872,E=8528127,C=14)
	CT(31374)=(S=8528128,E=8528383,C=29)
	CT(31375)=(S=8528384,E=8528639,C=2)
	CT(31376)=(S=8528640,E=8529151,C=29)
	CT(31377)=(S=8529152,E=8529407,C=36)
	CT(31378)=(S=8529408,E=8530687,C=29)
	CT(31379)=(S=8530688,E=8530815,C=22)
	CT(31380)=(S=8530816,E=8530879,C=14)
	CT(31381)=(S=8530880,E=8530943,C=50)
	CT(31382)=(S=8530944,E=8533503,C=29)
	CT(31383)=(S=8533504,E=8533759,C=2)
	CT(31384)=(S=8533760,E=8534015,C=29)
	CT(31385)=(S=8534016,E=8534271,C=0)
	CT(31386)=(S=8534272,E=8534783,C=29)
	CT(31387)=(S=8534784,E=8535295,C=34)
	CT(31388)=(S=8535296,E=8535807,C=29)
	CT(31389)=(S=8535808,E=8536063,C=78)
	CT(31390)=(S=8536064,E=8536575,C=29)
	CT(31391)=(S=8536576,E=8536831,C=12)
	CT(31392)=(S=8536832,E=8537087,C=27)
	CT(31393)=(S=8537088,E=8537343,C=29)
	CT(31394)=(S=8537344,E=8537599,C=2)
	CT(31395)=(S=8537600,E=8538367,C=29)
	CT(31396)=(S=8538368,E=8538623,C=28)
	CT(31397)=(S=8538624,E=8538879,C=29)
	CT(31398)=(S=8538880,E=8539135,C=28)
	CT(31399)=(S=8539136,E=8539647,C=29)
	CT(31400)=(S=8539648,E=8539903,C=36)
	CT(31401)=(S=8539904,E=8540159,C=12)
	CT(31402)=(S=8540160,E=8540671,C=29)
	CT(31403)=(S=8540672,E=8540927,C=34)
	CT(31404)=(S=8540928,E=8541183,C=28)
	CT(31405)=(S=8541184,E=8541439,C=12)
	CT(31406)=(S=8541440,E=8541951,C=29)
	CT(31407)=(S=8541952,E=8542207,C=2)
	CT(31408)=(S=8542208,E=8542463,C=14)
	CT(31409)=(S=8542464,E=8542719,C=36)
	CT(31410)=(S=8542720,E=8543231,C=29)
	CT(31411)=(S=8543232,E=8543487,C=34)
	CT(31412)=(S=8543488,E=8543999,C=29)
	CT(31413)=(S=8544000,E=8544255,C=0)
	CT(31414)=(S=8544256,E=8544767,C=29)
	CT(31415)=(S=8544768,E=8545023,C=12)
	CT(31416)=(S=8545024,E=8545279,C=29)
	CT(31417)=(S=8545280,E=8545535,C=19)
	CT(31418)=(S=8545536,E=8545791,C=29)
	CT(31419)=(S=8545792,E=8546047,C=0)
	CT(31420)=(S=8546048,E=8546303,C=29)
	CT(31421)=(S=8546304,E=8546559,C=49)
	CT(31422)=(S=8546560,E=8546815,C=10)
	CT(31423)=(S=8546816,E=8548351,C=29)
	CT(31424)=(S=8548352,E=8548607,C=36)
	CT(31425)=(S=8548608,E=8548863,C=78)
	CT(31426)=(S=8548864,E=8549119,C=29)
	CT(31427)=(S=8549120,E=8549375,C=36)
	CT(31428)=(S=8549376,E=8549631,C=0)
	CT(31429)=(S=8549632,E=8550399,C=29)
	CT(31430)=(S=8550400,E=8550655,C=12)
	CT(31431)=(S=8550656,E=8551167,C=29)
	CT(31432)=(S=8551168,E=8551423,C=73)
	CT(31433)=(S=8551424,E=8551679,C=29)
	CT(31434)=(S=8551680,E=8551935,C=34)
	CT(31435)=(S=8551936,E=8552959,C=29)
	CT(31436)=(S=8552960,E=8553215,C=0)
	CT(31437)=(S=8553216,E=8553727,C=29)
	CT(31438)=(S=8553728,E=8553983,C=28)
	CT(31439)=(S=8553984,E=8554495,C=29)
	CT(31440)=(S=8554496,E=8554751,C=15)
	CT(31441)=(S=8554752,E=8555007,C=29)
	CT(31442)=(S=8555008,E=8557567,C=36)
	CT(31443)=(S=8557568,E=8557823,C=14)
	CT(31444)=(S=8557824,E=8558079,C=28)
	CT(31445)=(S=8558080,E=8558847,C=29)
	CT(31446)=(S=8558848,E=8559103,C=2)
	CT(31447)=(S=8559104,E=8559359,C=29)
	CT(31448)=(S=8559360,E=8559615,C=0)
	CT(31449)=(S=8559616,E=8560127,C=29)
	CT(31450)=(S=8560128,E=8560383,C=2)
	CT(31451)=(S=8560384,E=8560639,C=14)
	CT(31452)=(S=8560640,E=8560895,C=29)
	CT(31453)=(S=8560896,E=8561151,C=36)
	CT(31454)=(S=8561152,E=8565503,C=29)
	CT(31455)=(S=8565504,E=8565759,C=78)
	CT(31456)=(S=8565760,E=8565887,C=28)
	CT(31457)=(S=8565888,E=8565951,C=30)
	CT(31458)=(S=8565952,E=8565959,C=58)
	CT(31459)=(S=8565960,E=8565967,C=14)
	CT(31460)=(S=8565968,E=8565983,C=31)
	CT(31461)=(S=8565984,E=8566015,C=46)
	CT(31462)=(S=8566016,E=8566527,C=29)
	CT(31463)=(S=8566528,E=8566783,C=28)
	CT(31464)=(S=8566784,E=8567039,C=29)
	CT(31465)=(S=8567040,E=8567103,C=31)
	CT(31466)=(S=8567104,E=8567111,C=14)
	CT(31467)=(S=8567112,E=8567119,C=26)
	CT(31468)=(S=8567120,E=8567127,C=21)
	CT(31469)=(S=8567128,E=8567135,C=25)
	CT(31470)=(S=8567136,E=8567143,C=58)
	CT(31471)=(S=8567144,E=8567151,C=28)
	CT(31472)=(S=8567168,E=8567183,C=24)
	CT(31473)=(S=8567184,E=8567191,C=14)
	CT(31474)=(S=8567192,E=8567199,C=19)
	CT(31475)=(S=8567200,E=8567207,C=14)
	CT(31476)=(S=8567208,E=8567215,C=12)
	CT(31477)=(S=8567216,E=8567223,C=33)
	CT(31478)=(S=8567224,E=8567231,C=12)
	CT(31479)=(S=8567232,E=8567295,C=51)
	CT(31480)=(S=8567296,E=8567551,C=15)
	CT(31481)=(S=8567552,E=8567807,C=29)
	CT(31482)=(S=8567808,E=8568063,C=47)
	CT(31483)=(S=8568064,E=8568319,C=29)
	CT(31484)=(S=8568320,E=8568575,C=12)
	CT(31485)=(S=8568576,E=8568831,C=29)
	CT(31486)=(S=8568832,E=8569087,C=15)
	CT(31487)=(S=8569088,E=8569095,C=12)
	CT(31488)=(S=8569096,E=8569111,C=33)
	CT(31489)=(S=8569112,E=8569119,C=12)
	CT(31490)=(S=8569120,E=8569159,C=18)
	CT(31491)=(S=8569160,E=8569167,C=15)
	CT(31492)=(S=8569168,E=8569183,C=14)
	CT(31493)=(S=8569184,E=8569191,C=28)
	CT(31494)=(S=8569192,E=8569199,C=25)
	CT(31495)=(S=8569200,E=8569207,C=28)
	CT(31496)=(S=8569208,E=8569215,C=66)
	CT(31497)=(S=8569216,E=8569343,C=35)
	CT(31498)=(S=8569344,E=8569599,C=0)
	CT(31499)=(S=8569600,E=8569855,C=73)
	CT(31500)=(S=8569856,E=8571903,C=29)
	CT(31501)=(S=8571904,E=8572159,C=51)
	CT(31502)=(S=8572160,E=8572415,C=29)
	CT(31503)=(S=8572416,E=8572671,C=25)
	CT(31504)=(S=8572672,E=8572927,C=29)
	CT(31505)=(S=8572928,E=8573183,C=48)
	CT(31506)=(S=8573184,E=8573439,C=14)
	CT(31507)=(S=8573440,E=8574975,C=29)
	CT(31508)=(S=8574976,E=8575487,C=73)
	CT(31509)=(S=8575488,E=8575999,C=29)
	CT(31510)=(S=8576000,E=8576255,C=0)
	CT(31511)=(S=8576256,E=8576767,C=29)
	CT(31512)=(S=8576768,E=8577023,C=34)
	CT(31513)=(S=8577024,E=8577279,C=29)
	CT(31514)=(S=8577280,E=8578303,C=24)
	CT(31515)=(S=8578304,E=8578559,C=19)
	CT(31516)=(S=8578560,E=8579839,C=47)
	CT(31517)=(S=8579840,E=8582399,C=19)
	CT(31518)=(S=8582400,E=8582655,C=29)
	CT(31519)=(S=8582656,E=8582911,C=14)
	CT(31520)=(S=8582912,E=8583679,C=29)
	CT(31521)=(S=8583680,E=8583935,C=78)
	CT(31522)=(S=8583936,E=8584191,C=15)
	CT(31523)=(S=8584192,E=8584959,C=29)
	CT(31524)=(S=8584960,E=8584967,C=35)
	CT(31525)=(S=8584968,E=8584975,C=18)
	CT(31526)=(S=8584976,E=8584991,C=14)
	CT(31527)=(S=8584992,E=8585023,C=18)
	CT(31528)=(S=8585024,E=8585031,C=43)
	CT(31529)=(S=8585032,E=8585039,C=28)
	CT(31530)=(S=8585040,E=8585047,C=18)
	CT(31531)=(S=8585048,E=8585055,C=35)
	CT(31532)=(S=8585056,E=8585063,C=15)
	CT(31533)=(S=8585064,E=8585071,C=28)
	CT(31534)=(S=8585072,E=8585079,C=15)
	CT(31535)=(S=8585080,E=8585087,C=28)
	CT(31536)=(S=8585088,E=8585103,C=31)
	CT(31537)=(S=8585104,E=8585111,C=15)
	CT(31538)=(S=8585112,E=8585119,C=58)
	CT(31539)=(S=8585120,E=8585127,C=19)
	CT(31540)=(S=8585128,E=8585135,C=15)
	CT(31541)=(S=8585136,E=8585143,C=23)
	CT(31542)=(S=8585144,E=8585151,C=28)
	CT(31543)=(S=8585152,E=8585215,C=26)
	CT(31544)=(S=8585216,E=8585219,C=201)
	CT(31545)=(S=8585220,E=8585267,C=189)
	CT(31546)=(S=8585268,E=8585271,C=83)
	CT(31547)=(S=8585272,E=8585287,C=189)
	CT(31548)=(S=8585288,E=8585291,C=202)
	CT(31549)=(S=8585292,E=8585319,C=189)
	CT(31550)=(S=8585320,E=8585323,C=83)
	CT(31551)=(S=8585324,E=8585327,C=203)
	CT(31552)=(S=8585328,E=8585339,C=189)
	CT(31553)=(S=8585340,E=8585343,C=204)
	CT(31554)=(S=8585344,E=8585347,C=83)
	CT(31555)=(S=8585348,E=8585351,C=189)
	CT(31556)=(S=8585352,E=8585355,C=181)
	CT(31557)=(S=8585356,E=8585383,C=189)
	CT(31558)=(S=8585384,E=8585387,C=181)
	CT(31559)=(S=8585388,E=8585391,C=203)
	CT(31560)=(S=8585392,E=8585411,C=189)
	CT(31561)=(S=8585412,E=8585415,C=83)
	CT(31562)=(S=8585416,E=8585427,C=189)
	CT(31563)=(S=8585428,E=8585431,C=205)
	CT(31564)=(S=8585432,E=8585447,C=189)
	CT(31565)=(S=8585448,E=8585451,C=83)
	CT(31566)=(S=8585452,E=8585455,C=204)
	CT(31567)=(S=8585456,E=8585471,C=189)
	CT(31568)=(S=8585472,E=8585727,C=15)
	CT(31569)=(S=8585728,E=8603647,C=29)
	CT(31570)=(S=8603648,E=8603651,C=83)
	CT(31571)=(S=8603652,E=8603671,C=189)
	CT(31572)=(S=8603672,E=8603675,C=206)
	CT(31573)=(S=8603676,E=8603679,C=83)
	CT(31574)=(S=8603680,E=8603719,C=189)
	CT(31575)=(S=8603720,E=8603723,C=83)
	CT(31576)=(S=8603724,E=8603727,C=176)
	CT(31577)=(S=8603728,E=8603759,C=189)
	CT(31578)=(S=8603760,E=8603763,C=207)
	CT(31579)=(S=8603764,E=8603779,C=189)
	CT(31580)=(S=8603780,E=8603783,C=83)
	CT(31581)=(S=8603784,E=8603787,C=170)
	CT(31582)=(S=8603788,E=8603803,C=189)
	CT(31583)=(S=8603804,E=8603807,C=83)
	CT(31584)=(S=8603808,E=8603815,C=189)
	CT(31585)=(S=8603816,E=8603819,C=200)
	CT(31586)=(S=8603820,E=8603851,C=189)
	CT(31587)=(S=8603852,E=8603855,C=83)
	CT(31588)=(S=8603856,E=8603859,C=208)
	CT(31589)=(S=8603860,E=8603863,C=204)
	CT(31590)=(S=8603864,E=8603871,C=189)
	CT(31591)=(S=8603872,E=8603875,C=209)
	CT(31592)=(S=8603876,E=8603879,C=204)
	CT(31593)=(S=8603880,E=8603887,C=203)
	CT(31594)=(S=8603888,E=8603891,C=83)
	CT(31595)=(S=8603892,E=8603903,C=189)
	CT(31596)=(S=8603904,E=8610047,C=29)
	CT(31597)=(S=8610048,E=8610303,C=19)
	CT(31598)=(S=8610304,E=8610559,C=29)
	CT(31599)=(S=8610560,E=8610815,C=28)
	CT(31600)=(S=8610816,E=8610819,C=209)
	CT(31601)=(S=8610820,E=8610843,C=189)
	CT(31602)=(S=8610844,E=8610847,C=203)
	CT(31603)=(S=8610848,E=8610851,C=189)
	CT(31604)=(S=8610852,E=8610855,C=176)
	CT(31605)=(S=8610856,E=8610879,C=189)
	CT(31606)=(S=8610880,E=8610883,C=83)
	CT(31607)=(S=8610884,E=8610903,C=189)
	CT(31608)=(S=8610904,E=8610907,C=83)
	CT(31609)=(S=8610908,E=8610915,C=189)
	CT(31610)=(S=8610916,E=8610919,C=83)
	CT(31611)=(S=8610920,E=8610923,C=189)
	CT(31612)=(S=8610924,E=8610927,C=83)
	CT(31613)=(S=8610928,E=8610955,C=189)
	CT(31614)=(S=8610956,E=8610959,C=210)
	CT(31615)=(S=8610960,E=8610975,C=189)
	CT(31616)=(S=8610976,E=8610979,C=176)
	CT(31617)=(S=8610980,E=8610995,C=189)
	CT(31618)=(S=8610996,E=8610999,C=83)
	CT(31619)=(S=8611000,E=8611003,C=206)
	CT(31620)=(S=8611004,E=8611015,C=189)
	CT(31621)=(S=8611016,E=8611019,C=201)
	CT(31622)=(S=8611020,E=8611071,C=189)
	CT(31623)=(S=8611072,E=8611327,C=29)
	CT(31624)=(S=8611328,E=8611583,C=34)
	CT(31625)=(S=8611584,E=8611839,C=29)
	CT(31626)=(S=8611840,E=8612095,C=78)
	CT(31627)=(S=8612096,E=8612863,C=29)
	CT(31628)=(S=8612864,E=8612867,C=83)
	CT(31629)=(S=8612868,E=8612871,C=209)
	CT(31630)=(S=8612872,E=8612879,C=189)
	CT(31631)=(S=8612880,E=8612883,C=208)
	CT(31632)=(S=8612884,E=8612887,C=209)
	CT(31633)=(S=8612888,E=8612899,C=189)
	CT(31634)=(S=8612900,E=8612903,C=202)
	CT(31635)=(S=8612904,E=8612907,C=83)
	CT(31636)=(S=8612908,E=8612931,C=189)
	CT(31637)=(S=8612932,E=8612935,C=203)
	CT(31638)=(S=8612936,E=8612943,C=189)
	CT(31639)=(S=8612944,E=8612947,C=83)
	CT(31640)=(S=8612948,E=8613003,C=189)
	CT(31641)=(S=8613004,E=8613007,C=83)
	CT(31642)=(S=8613008,E=8613031,C=189)
	CT(31643)=(S=8613032,E=8613035,C=181)
	CT(31644)=(S=8613036,E=8613055,C=189)
	CT(31645)=(S=8613056,E=8613059,C=206)
	CT(31646)=(S=8613060,E=8613071,C=189)
	CT(31647)=(S=8613072,E=8613075,C=203)
	CT(31648)=(S=8613076,E=8613095,C=189)
	CT(31649)=(S=8613096,E=8613099,C=208)
	CT(31650)=(S=8613100,E=8613119,C=189)
	CT(31651)=(S=8613120,E=8613631,C=29)
	CT(31652)=(S=8613632,E=8613887,C=14)
	CT(31653)=(S=8613888,E=8614399,C=2)
	CT(31654)=(S=8614400,E=8614655,C=15)
	CT(31655)=(S=8614656,E=8615167,C=19)
	CT(31656)=(S=8615168,E=8615295,C=34)
	CT(31657)=(S=8615296,E=8615311,C=41)
	CT(31658)=(S=8615312,E=8615327,C=28)
	CT(31659)=(S=8615328,E=8615335,C=67)
	CT(31660)=(S=8615336,E=8615343,C=14)
	CT(31661)=(S=8615344,E=8615351,C=19)
	CT(31662)=(S=8615352,E=8615359,C=14)
	CT(31663)=(S=8615360,E=8615367,C=33)
	CT(31664)=(S=8615368,E=8615375,C=12)
	CT(31665)=(S=8615376,E=8615383,C=33)
	CT(31666)=(S=8615384,E=8615391,C=18)
	CT(31667)=(S=8615392,E=8615423,C=35)
	CT(31668)=(S=8615424,E=8618239,C=29)
	CT(31669)=(S=8618240,E=8618495,C=8)
	CT(31670)=(S=8618496,E=8618751,C=42)
	CT(31671)=(S=8618752,E=8619007,C=29)
	CT(31672)=(S=8619008,E=8621567,C=78)
	CT(31673)=(S=8621568,E=8622847,C=29)
	CT(31674)=(S=8622848,E=8623103,C=2)
	CT(31675)=(S=8623104,E=8623622,C=78)
	CT(31676)=(S=8623623,E=8623623,C=29)
	CT(31677)=(S=8623624,E=8623626,C=78)
	CT(31678)=(S=8623627,E=8623631,C=29)
	CT(31679)=(S=8623632,E=8623646,C=78)
	CT(31680)=(S=8623647,E=8623647,C=29)
	CT(31681)=(S=8623648,E=8623670,C=78)
	CT(31682)=(S=8623671,E=8623671,C=29)
	CT(31683)=(S=8623672,E=8623677,C=78)
	CT(31684)=(S=8623678,E=8623678,C=29)
	CT(31685)=(S=8623679,E=8623695,C=78)
	CT(31686)=(S=8623696,E=8623699,C=29)
	CT(31687)=(S=8623700,E=8623745,C=78)
	CT(31688)=(S=8623746,E=8623747,C=29)
	CT(31689)=(S=8623748,E=8623759,C=78)
	CT(31690)=(S=8623760,E=8623775,C=29)
	CT(31691)=(S=8623776,E=8623805,C=78)
	CT(31692)=(S=8623806,E=8623806,C=29)
	CT(31693)=(S=8623807,E=8623871,C=78)
	CT(31694)=(S=8623872,E=8624127,C=29)
	CT(31695)=(S=8624128,E=8624383,C=34)
	CT(31696)=(S=8624384,E=8624639,C=29)
	CT(31697)=(S=8624640,E=8624895,C=15)
	CT(31698)=(S=8624896,E=8625151,C=36)
	CT(31699)=(S=8625152,E=8625919,C=29)
	CT(31700)=(S=8625920,E=8626175,C=28)
	CT(31701)=(S=8626176,E=8626431,C=47)
	CT(31702)=(S=8626432,E=8626483,C=189)
	CT(31703)=(S=8626484,E=8626487,C=208)
	CT(31704)=(S=8626488,E=8626491,C=203)
	CT(31705)=(S=8626492,E=8626511,C=189)
	CT(31706)=(S=8626512,E=8626515,C=208)
	CT(31707)=(S=8626516,E=8626519,C=211)
	CT(31708)=(S=8626520,E=8626523,C=83)
	CT(31709)=(S=8626524,E=8626531,C=189)
	CT(31710)=(S=8626532,E=8626535,C=212)
	CT(31711)=(S=8626536,E=8626579,C=189)
	CT(31712)=(S=8626580,E=8626583,C=170)
	CT(31713)=(S=8626584,E=8626587,C=83)
	CT(31714)=(S=8626588,E=8626615,C=189)
	CT(31715)=(S=8626616,E=8626619,C=83)
	CT(31716)=(S=8626620,E=8626631,C=189)
	CT(31717)=(S=8626632,E=8626635,C=203)
	CT(31718)=(S=8626636,E=8626663,C=189)
	CT(31719)=(S=8626664,E=8626671,C=83)
	CT(31720)=(S=8626672,E=8626675,C=189)
	CT(31721)=(S=8626676,E=8626679,C=204)
	CT(31722)=(S=8626680,E=8626683,C=189)
	CT(31723)=(S=8626684,E=8626687,C=206)
	CT(31724)=(S=8626688,E=8627199,C=78)
	CT(31725)=(S=8627200,E=8627967,C=24)
	CT(31726)=(S=8627968,E=8628479,C=29)
	CT(31727)=(S=8628480,E=8628735,C=28)
	CT(31728)=(S=8628736,E=8628991,C=0)
	CT(31729)=(S=8628992,E=8629247,C=29)
	CT(31730)=(S=8629248,E=8629503,C=0)
	CT(31731)=(S=8629504,E=8629759,C=28)
	CT(31732)=(S=8629760,E=8630015,C=36)
	CT(31733)=(S=8630016,E=8630271,C=15)
	CT(31734)=(S=8630272,E=8630527,C=13)
	CT(31735)=(S=8630528,E=8630783,C=47)
	CT(31736)=(S=8630784,E=8631039,C=204)
	CT(31737)=(S=8631040,E=8631295,C=29)
	CT(31738)=(S=8631296,E=8631551,C=36)
	CT(31739)=(S=8631552,E=8631807,C=0)
	CT(31740)=(S=8631808,E=8632575,C=29)
	CT(31741)=(S=8632576,E=8632831,C=0)
	CT(31742)=(S=8632832,E=8633343,C=29)
	CT(31743)=(S=8633344,E=8633599,C=28)
	CT(31744)=(S=8633600,E=8635135,C=29)
	CT(31745)=(S=8635136,E=8635391,C=78)
	CT(31746)=(S=8635648,E=8636927,C=29)
	CT(31747)=(S=8636928,E=8637183,C=78)
	CT(31748)=(S=8637184,E=8637439,C=0)
	CT(31749)=(S=8637440,E=8637695,C=29)
	CT(31750)=(S=8637696,E=8637951,C=19)
	CT(31751)=(S=8637952,E=8638207,C=2)
	CT(31752)=(S=8638208,E=8638463,C=47)
	CT(31753)=(S=8638464,E=8639231,C=29)
	CT(31754)=(S=8639232,E=8639487,C=36)
	CT(31755)=(S=8639488,E=8639743,C=29)
	CT(31756)=(S=8639744,E=8639999,C=2)
	CT(31757)=(S=8640000,E=8640767,C=29)
	CT(31758)=(S=8640768,E=8641023,C=0)
	CT(31759)=(S=8641024,E=8641535,C=29)
	CT(31760)=(S=8641536,E=8641791,C=28)
	CT(31761)=(S=8641792,E=8641795,C=83)
	CT(31762)=(S=8641796,E=8641799,C=177)
	CT(31763)=(S=8641800,E=8641803,C=208)
	CT(31764)=(S=8641804,E=8641807,C=189)
	CT(31765)=(S=8641808,E=8641811,C=83)
	CT(31766)=(S=8641812,E=8641815,C=189)
	CT(31767)=(S=8641816,E=8641819,C=209)
	CT(31768)=(S=8641820,E=8641823,C=176)
	CT(31769)=(S=8641824,E=8641827,C=203)
	CT(31770)=(S=8641828,E=8641831,C=189)
	CT(31771)=(S=8641832,E=8641835,C=181)
	CT(31772)=(S=8641836,E=8641855,C=189)
	CT(31773)=(S=8641856,E=8641859,C=83)
	CT(31774)=(S=8641860,E=8641903,C=189)
	CT(31775)=(S=8641904,E=8641907,C=200)
	CT(31776)=(S=8641908,E=8641935,C=189)
	CT(31777)=(S=8641936,E=8641939,C=207)
	CT(31778)=(S=8641940,E=8641943,C=189)
	CT(31779)=(S=8641944,E=8641947,C=212)
	CT(31780)=(S=8641948,E=8641955,C=189)
	CT(31781)=(S=8641956,E=8641959,C=203)
	CT(31782)=(S=8641960,E=8642047,C=189)
	CT(31783)=(S=8642048,E=8642559,C=29)
	CT(31784)=(S=8642560,E=8642815,C=36)
	CT(31785)=(S=8642816,E=8643327,C=29)
	CT(31786)=(S=8643328,E=8643583,C=14)
	CT(31787)=(S=8643584,E=8643839,C=47)
	CT(31788)=(S=8643840,E=8644351,C=29)
	CT(31789)=(S=8644352,E=8644607,C=14)
	CT(31790)=(S=8644608,E=8644863,C=78)
	CT(31791)=(S=8644864,E=8645119,C=29)
	CT(31792)=(S=8645120,E=8645375,C=28)
	CT(31793)=(S=8645376,E=8645631,C=78)
	CT(31794)=(S=8645632,E=8645887,C=0)
	CT(31795)=(S=8645888,E=8646143,C=36)
	CT(31796)=(S=8646144,E=8647167,C=29)
	CT(31797)=(S=8647168,E=8647423,C=0)
	CT(31798)=(S=8647424,E=8647679,C=29)
	CT(31799)=(S=8647680,E=8648191,C=0)
	CT(31800)=(S=8648192,E=8648447,C=28)
	CT(31801)=(S=8648448,E=8648703,C=29)
	CT(31802)=(S=8648704,E=8648959,C=2)
	CT(31803)=(S=8648960,E=8649471,C=29)
	CT(31804)=(S=8649472,E=8649727,C=14)
	CT(31805)=(S=8649728,E=8650239,C=29)
	CT(31806)=(S=8650240,E=8650495,C=12)
	CT(31807)=(S=8650496,E=8650499,C=189)
	CT(31808)=(S=8650500,E=8650503,C=83)
	CT(31809)=(S=8650504,E=8650507,C=189)
	CT(31810)=(S=8650508,E=8650511,C=83)
	CT(31811)=(S=8650512,E=8650535,C=189)
	CT(31812)=(S=8650536,E=8650539,C=170)
	CT(31813)=(S=8650540,E=8650555,C=189)
	CT(31814)=(S=8650556,E=8650559,C=83)
	CT(31815)=(S=8650560,E=8650599,C=189)
	CT(31816)=(S=8650600,E=8650603,C=200)
	CT(31817)=(S=8650604,E=8650631,C=189)
	CT(31818)=(S=8650632,E=8650635,C=212)
	CT(31819)=(S=8650636,E=8650675,C=189)
	CT(31820)=(S=8650676,E=8650679,C=83)
	CT(31821)=(S=8650680,E=8650687,C=189)
	CT(31822)=(S=8650688,E=8650691,C=212)
	CT(31823)=(S=8650692,E=8650743,C=189)
	CT(31824)=(S=8650744,E=8650747,C=83)
	CT(31825)=(S=8650748,E=8650751,C=189)
	CT(31826)=(S=8650752,E=8667135,C=29)
	CT(31827)=(S=8667136,E=8670975,C=17)
	CT(31828)=(S=8670976,E=8688127,C=29)
	CT(31829)=(S=8688128,E=8688383,C=14)
	CT(31830)=(S=8688384,E=8688447,C=29)
	CT(31831)=(S=8688448,E=8688511,C=7)
	CT(31832)=(S=8688512,E=8688895,C=29)
	CT(31833)=(S=8688896,E=8689151,C=12)
	CT(31834)=(S=8689152,E=8689407,C=27)
	CT(31835)=(S=8689408,E=8689919,C=29)
	CT(31836)=(S=8689920,E=8690175,C=14)
	CT(31837)=(S=8690176,E=8690431,C=29)
	CT(31838)=(S=8690432,E=8690687,C=47)
	CT(31839)=(S=8690688,E=8690943,C=78)
	CT(31840)=(S=8690944,E=8691199,C=212)
	CT(31841)=(S=8691200,E=8692735,C=29)
	CT(31842)=(S=8692992,E=8694271,C=12)
	CT(31843)=(S=8694272,E=8694527,C=29)
	CT(31844)=(S=8694528,E=8694783,C=47)
	CT(31845)=(S=8694784,E=8695807,C=29)
	CT(31846)=(S=8695808,E=8696063,C=28)
	CT(31847)=(S=8696064,E=8696575,C=29)
	CT(31848)=(S=8696576,E=8696831,C=2)
	CT(31849)=(S=8696832,E=8697087,C=28)
	CT(31850)=(S=8697088,E=8697343,C=73)
	CT(31851)=(S=8697344,E=8697599,C=2)
	CT(31852)=(S=8697600,E=8697855,C=29)
	CT(31853)=(S=8697856,E=8698111,C=212)
	CT(31854)=(S=8698112,E=8698367,C=14)
	CT(31855)=(S=8698368,E=8698623,C=16)
	CT(31856)=(S=8698624,E=8698879,C=28)
	CT(31857)=(S=8698880,E=8699647,C=29)
	CT(31858)=(S=8699648,E=8699903,C=212)
	CT(31859)=(S=8699904,E=8700671,C=29)
	CT(31860)=(S=8700672,E=8700927,C=28)
	CT(31861)=(S=8700928,E=8701183,C=19)
	CT(31862)=(S=8701184,E=8701695,C=29)
	CT(31863)=(S=8701696,E=8701951,C=28)
	CT(31864)=(S=8701952,E=8702463,C=29)
	CT(31865)=(S=8702464,E=8707583,C=78)
	CT(31866)=(S=8707584,E=8707839,C=2)
	CT(31867)=(S=8707840,E=8708351,C=29)
	CT(31868)=(S=8708352,E=8708607,C=78)
	CT(31869)=(S=8708608,E=8708863,C=29)
	CT(31870)=(S=8708864,E=8709119,C=12)
	CT(31871)=(S=8709120,E=8709375,C=29)
	CT(31872)=(S=8709376,E=8709631,C=36)
	CT(31873)=(S=8709632,E=8710143,C=28)
	CT(31874)=(S=8710144,E=8710399,C=14)
	CT(31875)=(S=8710400,E=8710655,C=29)
	CT(31876)=(S=8710656,E=8710911,C=0)
	CT(31877)=(S=8710912,E=8713215,C=29)
	CT(31878)=(S=8713216,E=8713471,C=14)
	CT(31879)=(S=8713472,E=8713727,C=29)
	CT(31880)=(S=8713728,E=8713983,C=78)
	CT(31881)=(S=8713984,E=8714495,C=204)
	CT(31882)=(S=8714496,E=8715007,C=29)
	CT(31883)=(S=8715008,E=8715231,C=212)
	CT(31884)=(S=8715232,E=8715263,C=201)
	CT(31885)=(S=8715264,E=8715519,C=28)
	CT(31886)=(S=8715520,E=8715775,C=29)
	CT(31887)=(S=8715776,E=8716031,C=204)
	CT(31888)=(S=8716032,E=8716035,C=212)
	CT(31889)=(S=8716036,E=8716043,C=83)
	CT(31890)=(S=8716044,E=8716051,C=189)
	CT(31891)=(S=8716052,E=8716055,C=181)
	CT(31892)=(S=8716056,E=8716099,C=189)
	CT(31893)=(S=8716100,E=8716103,C=203)
	CT(31894)=(S=8716104,E=8716143,C=189)
	CT(31895)=(S=8716144,E=8716147,C=83)
	CT(31896)=(S=8716148,E=8716155,C=189)
	CT(31897)=(S=8716156,E=8716159,C=204)
	CT(31898)=(S=8716160,E=8716163,C=189)
	CT(31899)=(S=8716164,E=8716167,C=208)
	CT(31900)=(S=8716168,E=8716195,C=189)
	CT(31901)=(S=8716196,E=8716199,C=206)
	CT(31902)=(S=8716200,E=8716231,C=189)
	CT(31903)=(S=8716232,E=8716235,C=83)
	CT(31904)=(S=8716236,E=8716239,C=189)
	CT(31905)=(S=8716240,E=8716243,C=210)
	CT(31906)=(S=8716244,E=8716255,C=189)
	CT(31907)=(S=8716256,E=8716259,C=83)
	CT(31908)=(S=8716260,E=8716267,C=189)
	CT(31909)=(S=8716268,E=8716271,C=203)
	CT(31910)=(S=8716272,E=8716287,C=189)
	CT(31911)=(S=8716288,E=8781823,C=2)
	CT(31912)=(S=8781824,E=8781831,C=15)
	CT(31913)=(S=8781832,E=8781839,C=25)
	CT(31914)=(S=8781840,E=8781847,C=14)
	CT(31915)=(S=8781848,E=8781855,C=28)
	CT(31916)=(S=8781856,E=8781887,C=96)
	CT(31917)=(S=8781888,E=8781895,C=14)
	CT(31918)=(S=8781896,E=8781903,C=13)
	CT(31919)=(S=8781904,E=8781911,C=63)
	CT(31920)=(S=8781912,E=8781919,C=36)
	CT(31921)=(S=8781920,E=8781943,C=18)
	CT(31922)=(S=8781944,E=8781951,C=28)
	CT(31923)=(S=8781952,E=8782079,C=45)
	CT(31924)=(S=8782080,E=8782847,C=28)
	CT(31925)=(S=8782848,E=8783615,C=29)
	CT(31926)=(S=8783616,E=8783871,C=0)
	CT(31927)=(S=8783872,E=8786175,C=29)
	CT(31928)=(S=8786176,E=8786431,C=68)
	CT(31929)=(S=8786432,E=8786687,C=0)
	CT(31930)=(S=8786688,E=8786815,C=28)
	CT(31931)=(S=8786816,E=8786823,C=18)
	CT(31932)=(S=8786824,E=8786831,C=71)
	CT(31933)=(S=8786832,E=8786847,C=18)
	CT(31934)=(S=8786848,E=8786855,C=14)
	CT(31935)=(S=8786856,E=8786863,C=15)
	CT(31936)=(S=8786864,E=8786879,C=36)
	CT(31937)=(S=8786880,E=8786887,C=12)
	CT(31938)=(S=8786888,E=8786895,C=37)
	CT(31939)=(S=8786896,E=8786911,C=41)
	CT(31940)=(S=8786912,E=8786943,C=40)
	CT(31941)=(S=8786944,E=8787199,C=29)
	CT(31942)=(S=8787200,E=8787455,C=34)
	CT(31943)=(S=8787456,E=8787711,C=78)
	CT(31944)=(S=8787712,E=8788223,C=29)
	CT(31945)=(S=8788224,E=8788479,C=19)
	CT(31946)=(S=8788480,E=8788735,C=29)
	CT(31947)=(S=8788736,E=8788991,C=36)
	CT(31948)=(S=8788992,E=8789247,C=28)
	CT(31949)=(S=8789248,E=8789503,C=29)
	CT(31950)=(S=8789504,E=8789759,C=28)
	CT(31951)=(S=8789760,E=8790015,C=78)
	CT(31952)=(S=8790016,E=8790527,C=29)
	CT(31953)=(S=8790528,E=8790783,C=28)
	CT(31954)=(S=8790784,E=8791039,C=29)
	CT(31955)=(S=8791040,E=8791295,C=14)
	CT(31956)=(S=8791296,E=8791551,C=47)
	CT(31957)=(S=8791552,E=8792799,C=29)
	CT(31958)=(S=8792800,E=8792800,C=8)
	CT(31959)=(S=8792801,E=8792801,C=7)
	CT(31960)=(S=8792802,E=8792802,C=0)
	CT(31961)=(S=8792803,E=8792803,C=2)
	CT(31962)=(S=8792804,E=8793855,C=29)
	CT(31963)=(S=8793856,E=8794111,C=27)
	CT(31964)=(S=8794112,E=8795647,C=29)
	CT(31965)=(S=8795648,E=8795903,C=49)
	CT(31966)=(S=8795904,E=8796671,C=29)
	CT(31967)=(S=8796672,E=8796927,C=49)
	CT(31968)=(S=8796928,E=8797183,C=12)
	CT(31969)=(S=8797184,E=8797695,C=28)
	CT(31970)=(S=8797696,E=8801023,C=29)
	CT(31971)=(S=8801024,E=8801279,C=6)
	CT(31972)=(S=8801280,E=8801535,C=28)
	CT(31973)=(S=8801536,E=8802559,C=29)
	CT(31974)=(S=8802560,E=8802815,C=28)
	CT(31975)=(S=8802816,E=8803071,C=29)
	CT(31976)=(S=8803072,E=8803327,C=14)
	CT(31977)=(S=8803328,E=8804095,C=29)
	CT(31978)=(S=8804096,E=8804351,C=78)
	CT(31979)=(S=8804352,E=8804863,C=29)
	CT(31980)=(S=8804864,E=8804991,C=40)
	CT(31981)=(S=8804992,E=8804999,C=59)
	CT(31982)=(S=8805000,E=8805007,C=12)
	CT(31983)=(S=8805008,E=8805015,C=27)
	CT(31984)=(S=8805016,E=8805055,C=18)
	CT(31985)=(S=8805056,E=8805071,C=14)
	CT(31986)=(S=8805072,E=8805079,C=18)
	CT(31987)=(S=8805080,E=8805087,C=34)
	CT(31988)=(S=8805088,E=8805119,C=15)
	CT(31989)=(S=8805120,E=8810239,C=28)
	CT(31990)=(S=8810240,E=8811263,C=29)
	CT(31991)=(S=8811264,E=8811519,C=0)
	CT(31992)=(S=8811520,E=8811775,C=29)
	CT(31993)=(S=8811776,E=8812031,C=78)
	CT(31994)=(S=8812032,E=8812287,C=29)
	CT(31995)=(S=8812288,E=8812543,C=27)
	CT(31996)=(S=8812544,E=8814591,C=29)
	CT(31997)=(S=8814592,E=8814623,C=14)
	CT(31998)=(S=8814624,E=8814655,C=29)
	CT(31999)=(S=8814656,E=8814847,C=14)
	CT(32000)=(S=8814848,E=8815103,C=29)
	CT(32001)=(S=8815104,E=8815359,C=28)
	CT(32002)=(S=8815360,E=8817151,C=29)
	CT(32003)=(S=8817152,E=8817407,C=19)
	CT(32004)=(S=8817408,E=8818175,C=29)
	CT(32005)=(S=8818176,E=8818431,C=14)
	CT(32006)=(S=8818432,E=8818687,C=36)
	CT(32007)=(S=8818688,E=8818943,C=0)
	CT(32008)=(S=8818944,E=8819455,C=36)
	CT(32009)=(S=8819456,E=8819711,C=28)
	CT(32010)=(S=8819712,E=8819967,C=0)
	CT(32011)=(S=8819968,E=8820223,C=29)
	CT(32012)=(S=8820224,E=8820479,C=78)
	CT(32013)=(S=8820480,E=8820735,C=14)
	CT(32014)=(S=8820736,E=8820991,C=29)
	CT(32015)=(S=8820992,E=8821247,C=78)
	CT(32016)=(S=8821248,E=8821503,C=29)
	CT(32017)=(S=8821504,E=8821759,C=28)
	CT(32018)=(S=8821760,E=8822015,C=29)
	CT(32019)=(S=8822016,E=8822527,C=12)
	CT(32020)=(S=8822528,E=8822783,C=8)
	CT(32021)=(S=8822784,E=8823039,C=2)
	CT(32022)=(S=8823040,E=8825087,C=29)
	CT(32023)=(S=8825088,E=8825343,C=28)
	CT(32024)=(S=8825344,E=8825599,C=29)
	CT(32025)=(S=8825600,E=8825855,C=28)
	CT(32026)=(S=8825856,E=8826623,C=29)
	CT(32027)=(S=8826880,E=8827135,C=28)
	CT(32028)=(S=8827136,E=8827391,C=29)
	CT(32029)=(S=8827392,E=8827647,C=0)
	CT(32030)=(S=8827648,E=8827903,C=29)
	CT(32031)=(S=8827904,E=8828159,C=2)
	CT(32032)=(S=8828160,E=8828671,C=29)
	CT(32033)=(S=8828672,E=8828927,C=14)
	CT(32034)=(S=8828928,E=8829183,C=49)
	CT(32035)=(S=8829184,E=8829951,C=29)
	CT(32036)=(S=8829952,E=8830207,C=36)
	CT(32037)=(S=8830208,E=8830463,C=29)
	CT(32038)=(S=8830464,E=8830719,C=78)
	CT(32039)=(S=8830720,E=8830975,C=14)
	CT(32040)=(S=8830976,E=8831743,C=29)
	CT(32041)=(S=8832000,E=8832255,C=3)
	CT(32042)=(S=8832256,E=8832767,C=29)
	CT(32043)=(S=8832768,E=8833023,C=78)
	CT(32044)=(S=8833024,E=8833535,C=29)
	CT(32045)=(S=8833536,E=8833791,C=80)
	CT(32046)=(S=8833792,E=8834047,C=36)
	CT(32047)=(S=8834048,E=8834559,C=29)
	CT(32048)=(S=8834560,E=8834815,C=12)
	CT(32049)=(S=8834816,E=8835071,C=29)
	CT(32050)=(S=8835072,E=8835327,C=9)
	CT(32051)=(S=8835328,E=8835839,C=29)
	CT(32052)=(S=8835840,E=8836095,C=0)
	CT(32053)=(S=8836096,E=8836351,C=12)
	CT(32054)=(S=8836352,E=8836607,C=63)
	CT(32055)=(S=8836608,E=8836863,C=12)
	CT(32056)=(S=8836864,E=8837887,C=29)
	CT(32057)=(S=8837888,E=8838399,C=14)
	CT(32058)=(S=8838400,E=8838911,C=36)
	CT(32059)=(S=8838912,E=8839423,C=29)
	CT(32060)=(S=8839424,E=8839679,C=14)
	CT(32061)=(S=8839680,E=8839935,C=63)
	CT(32062)=(S=8839936,E=8840191,C=12)
	CT(32063)=(S=8840192,E=8842239,C=29)
	CT(32064)=(S=8842240,E=8842495,C=3)
	CT(32065)=(S=8842496,E=8842751,C=2)
	CT(32066)=(S=8842752,E=8843007,C=29)
	CT(32067)=(S=8843008,E=8843263,C=38)
	CT(32068)=(S=8843264,E=8844543,C=29)
	CT(32069)=(S=8844544,E=8844799,C=28)
	CT(32070)=(S=8844800,E=8845055,C=12)
	CT(32071)=(S=8845056,E=8845311,C=28)
	CT(32072)=(S=8845312,E=8845567,C=29)
	CT(32073)=(S=8845568,E=8845823,C=31)
	CT(32074)=(S=8845824,E=8847103,C=29)
	CT(32075)=(S=8847104,E=8847231,C=50)
	CT(32076)=(S=8847232,E=8847263,C=18)
	CT(32077)=(S=8847264,E=8847287,C=15)
	CT(32078)=(S=8847288,E=8847295,C=25)
	CT(32079)=(S=8847296,E=8847299,C=28)
	CT(32080)=(S=8847300,E=8847301,C=26)
	CT(32081)=(S=8847302,E=8847302,C=28)
	CT(32082)=(S=8847303,E=8847303,C=37)
	CT(32083)=(S=8847304,E=8847311,C=26)
	CT(32084)=(S=8847312,E=8847348,C=28)
	CT(32085)=(S=8847349,E=8847350,C=26)
	CT(32086)=(S=8847351,E=8847351,C=28)
	CT(32087)=(S=8847352,E=8847353,C=26)
	CT(32088)=(S=8847354,E=8847359,C=28)
	CT(32089)=(S=8847360,E=8847615,C=78)
	CT(32090)=(S=8847616,E=8850431,C=29)
	CT(32091)=(S=8850432,E=8850687,C=78)
	CT(32092)=(S=8850688,E=8852223,C=29)
	CT(32093)=(S=8852224,E=8852479,C=78)
	CT(32094)=(S=8852480,E=8853247,C=29)
	CT(32095)=(S=8853248,E=8853503,C=78)
	CT(32096)=(S=8853504,E=8869039,C=29)
	CT(32097)=(S=8869040,E=8869055,C=78)
	CT(32098)=(S=8869056,E=8869095,C=29)
	CT(32099)=(S=8869096,E=8869103,C=5)
	CT(32100)=(S=8869104,E=8869119,C=4)
	CT(32101)=(S=8869120,E=8893695,C=29)
	CT(32102)=(S=8893696,E=8893951,C=78)
	CT(32103)=(S=8893952,E=8897535,C=29)
	CT(32104)=(S=8897536,E=8897791,C=14)
	CT(32105)=(S=8897792,E=8950015,C=29)
	CT(32106)=(S=8950016,E=8950271,C=80)
	CT(32107)=(S=8950272,E=8950783,C=29)
	CT(32108)=(S=8950784,E=8951039,C=14)
	CT(32109)=(S=8951040,E=8952063,C=29)
	CT(32110)=(S=8952064,E=8952575,C=0)
	CT(32111)=(S=8952576,E=8952831,C=19)
	CT(32112)=(S=8952832,E=8953087,C=14)
	CT(32113)=(S=8953088,E=8953343,C=28)
	CT(32114)=(S=8953344,E=8953599,C=27)
	CT(32115)=(S=8953600,E=8953855,C=78)
	CT(32116)=(S=8953856,E=8954623,C=29)
	CT(32117)=(S=8954624,E=8954879,C=19)
	CT(32118)=(S=8954880,E=8955135,C=27)
	CT(32119)=(S=8955136,E=8956159,C=29)
	CT(32120)=(S=8956160,E=8956287,C=60)
	CT(32121)=(S=8956288,E=8956415,C=18)
	CT(32122)=(S=8956416,E=8956671,C=14)
	CT(32123)=(S=8956672,E=8956927,C=29)
	CT(32124)=(S=8956928,E=8957183,C=28)
	CT(32125)=(S=8957184,E=8957439,C=70)
	CT(32126)=(S=8957440,E=8960255,C=29)
	CT(32127)=(S=8960256,E=8960511,C=4)
	CT(32128)=(S=8960512,E=8960767,C=0)
	CT(32129)=(S=8960768,E=8961023,C=2)
	CT(32130)=(S=8961024,E=8963583,C=29)
	CT(32131)=(S=8963584,E=8963839,C=2)
	CT(32132)=(S=8963840,E=8964095,C=28)
	CT(32133)=(S=8964096,E=8964351,C=29)
	CT(32134)=(S=8964352,E=8964607,C=63)
	CT(32135)=(S=8964608,E=8965631,C=29)
	CT(32136)=(S=8965632,E=8965887,C=63)
	CT(32137)=(S=8965888,E=8970495,C=29)
	CT(32138)=(S=8970496,E=8970751,C=19)
	CT(32139)=(S=8970752,E=8971775,C=29)
	CT(32140)=(S=8971776,E=8972031,C=28)
	CT(32141)=(S=8972032,E=8972287,C=36)
	CT(32142)=(S=8972288,E=8975103,C=29)
	CT(32143)=(S=8975104,E=8975359,C=28)
	CT(32144)=(S=8975360,E=8982271,C=29)
	CT(32145)=(S=8982272,E=8982527,C=78)
	CT(32146)=(S=8982528,E=8982783,C=29)
	CT(32147)=(S=8982784,E=8983039,C=36)
	CT(32148)=(S=8983040,E=8986879,C=29)
	CT(32149)=(S=8986880,E=8987135,C=47)
	CT(32150)=(S=8987136,E=8987391,C=34)
	CT(32151)=(S=8987392,E=8989439,C=29)
	CT(32152)=(S=8989440,E=8989695,C=63)
	CT(32153)=(S=8989696,E=8989951,C=14)
	CT(32154)=(S=8989952,E=8990463,C=29)
	CT(32155)=(S=8990464,E=8990719,C=27)
	CT(32156)=(S=8990720,E=8991231,C=29)
	CT(32157)=(S=8991232,E=8991487,C=14)
	CT(32158)=(S=8991488,E=8992511,C=29)
	CT(32159)=(S=8992512,E=8993023,C=36)
	CT(32160)=(S=8993024,E=8993279,C=29)
	CT(32161)=(S=8993280,E=8994303,C=19)
	CT(32162)=(S=8994304,E=8994559,C=34)
	CT(32163)=(S=8994560,E=8994687,C=147)
	CT(32164)=(S=8995072,E=8995839,C=29)
	CT(32165)=(S=8995840,E=8996095,C=6)
	CT(32166)=(S=8996096,E=8997119,C=29)
	CT(32167)=(S=8997120,E=8997375,C=14)
	CT(32168)=(S=8997376,E=8997631,C=8)
	CT(32169)=(S=8997632,E=8997887,C=29)
	CT(32170)=(S=8997888,E=8998143,C=0)
	CT(32171)=(S=8998144,E=8999423,C=29)
	CT(32172)=(S=8999424,E=8999679,C=78)
	CT(32173)=(S=8999680,E=9001983,C=29)
	CT(32174)=(S=9001984,E=9002239,C=0)
	CT(32175)=(S=9002240,E=9002495,C=27)
	CT(32176)=(S=9002496,E=9002751,C=78)
	CT(32177)=(S=9002752,E=9003007,C=29)
	CT(32178)=(S=9003008,E=9003263,C=47)
	CT(32179)=(S=9003264,E=9003519,C=29)
	CT(32180)=(S=9003520,E=9003775,C=33)
	CT(32181)=(S=9003776,E=9004287,C=29)
	CT(32182)=(S=9004288,E=9004543,C=14)
	CT(32183)=(S=9004544,E=9005311,C=29)
	CT(32184)=(S=9005312,E=9005567,C=14)
	CT(32185)=(S=9005568,E=9006079,C=29)
	CT(32186)=(S=9006080,E=9006335,C=14)
	CT(32187)=(S=9006336,E=9006591,C=0)
	CT(32188)=(S=9006592,E=9006847,C=29)
	CT(32189)=(S=9006848,E=9007103,C=0)
	CT(32190)=(S=9007104,E=9009151,C=29)
	CT(32191)=(S=9009152,E=9009407,C=36)
	CT(32192)=(S=9009408,E=9009663,C=12)
	CT(32193)=(S=9009664,E=9009919,C=78)
	CT(32194)=(S=9009920,E=9011455,C=29)
	CT(32195)=(S=9011456,E=9011711,C=12)
	CT(32196)=(S=9011712,E=9012223,C=29)
	CT(32197)=(S=9012224,E=9012479,C=7)
	CT(32198)=(S=9012480,E=9012735,C=27)
	CT(32199)=(S=9012736,E=9013759,C=29)
	CT(32200)=(S=9013760,E=9014015,C=34)
	CT(32201)=(S=9014016,E=9016063,C=29)
	CT(32202)=(S=9016064,E=9016319,C=0)
	CT(32203)=(S=9016320,E=9016575,C=29)
	CT(32204)=(S=9016576,E=9016831,C=78)
	CT(32205)=(S=9016832,E=9017599,C=29)
	CT(32206)=(S=9017600,E=9017855,C=2)
	CT(32207)=(S=9017856,E=9018111,C=0)
	CT(32208)=(S=9018112,E=9018367,C=29)
	CT(32209)=(S=9018368,E=9018623,C=34)
	CT(32210)=(S=9018624,E=9018879,C=0)
	CT(32211)=(S=9018880,E=9019135,C=105)
	CT(32212)=(S=9019136,E=9020159,C=29)
	CT(32213)=(S=9020160,E=9020415,C=47)
	CT(32214)=(S=9020416,E=9020927,C=29)
	CT(32215)=(S=9020928,E=9021183,C=0)
	CT(32216)=(S=9021184,E=9022207,C=29)
	CT(32217)=(S=9022208,E=9022463,C=105)
	CT(32218)=(S=9022464,E=9022719,C=0)
	CT(32219)=(S=9022720,E=9022975,C=29)
	CT(32220)=(S=9022976,E=9023231,C=36)
	CT(32221)=(S=9023232,E=9023359,C=29)
	CT(32222)=(S=9023360,E=9023487,C=78)
	CT(32223)=(S=9023488,E=9026047,C=29)
	CT(32224)=(S=9026048,E=9026303,C=78)
	CT(32225)=(S=9026304,E=9026815,C=29)
	CT(32226)=(S=9026816,E=9027071,C=8)
	CT(32227)=(S=9027072,E=9027327,C=29)
	CT(32228)=(S=9027328,E=9027583,C=63)
	CT(32229)=(S=9027584,E=9027839,C=29)
	CT(32230)=(S=9027840,E=9028095,C=28)
	CT(32231)=(S=9028096,E=9028351,C=12)
	CT(32232)=(S=9028352,E=9028607,C=14)
	CT(32233)=(S=9028864,E=9030655,C=29)
	CT(32234)=(S=9030656,E=9030911,C=15)
	CT(32235)=(S=9030912,E=9031167,C=14)
	CT(32236)=(S=9031168,E=9031423,C=29)
	CT(32237)=(S=9031424,E=9031679,C=78)
	CT(32238)=(S=9031680,E=9031935,C=42)
	CT(32239)=(S=9031936,E=9032959,C=29)
	CT(32240)=(S=9032960,E=9033215,C=14)
	CT(32241)=(S=9033216,E=9033727,C=105)
	CT(32242)=(S=9033728,E=9033983,C=29)
	CT(32243)=(S=9033984,E=9034239,C=27)
	CT(32244)=(S=9034240,E=9034495,C=14)
	CT(32245)=(S=9034496,E=9034751,C=0)
	CT(32246)=(S=9034752,E=9035007,C=29)
	CT(32247)=(S=9035008,E=9035775,C=14)
	CT(32248)=(S=9035776,E=9036031,C=36)
	CT(32249)=(S=9036032,E=9036287,C=29)
	CT(32250)=(S=9036288,E=9036543,C=28)
	CT(32251)=(S=9036544,E=9041919,C=29)
	CT(32252)=(S=9041920,E=9042175,C=28)
	CT(32253)=(S=9042176,E=9042431,C=29)
	CT(32254)=(S=9042432,E=9042943,C=28)
	CT(32255)=(S=9042944,E=9043199,C=29)
	CT(32256)=(S=9043200,E=9043455,C=14)
	CT(32257)=(S=9043456,E=9043711,C=29)
	CT(32258)=(S=9043968,E=9043975,C=189)
	CT(32259)=(S=9043976,E=9043979,C=83)
	CT(32260)=(S=9043980,E=9043983,C=203)
	CT(32261)=(S=9043984,E=9044007,C=189)
	CT(32262)=(S=9044008,E=9044011,C=200)
	CT(32263)=(S=9044012,E=9044023,C=189)
	CT(32264)=(S=9044024,E=9044027,C=83)
	CT(32265)=(S=9044028,E=9044055,C=189)
	CT(32266)=(S=9044056,E=9044059,C=181)
	CT(32267)=(S=9044060,E=9044071,C=189)
	CT(32268)=(S=9044072,E=9044075,C=83)
	CT(32269)=(S=9044076,E=9044083,C=189)
	CT(32270)=(S=9044084,E=9044087,C=209)
	CT(32271)=(S=9044088,E=9044091,C=203)
	CT(32272)=(S=9044092,E=9044119,C=189)
	CT(32273)=(S=9044120,E=9044127,C=83)
	CT(32274)=(S=9044128,E=9044151,C=189)
	CT(32275)=(S=9044152,E=9044155,C=83)
	CT(32276)=(S=9044156,E=9044195,C=189)
	CT(32277)=(S=9044196,E=9044199,C=208)
	CT(32278)=(S=9044200,E=9044207,C=189)
	CT(32279)=(S=9044208,E=9044211,C=208)
	CT(32280)=(S=9044212,E=9044223,C=189)
	CT(32281)=(S=9044224,E=9044735,C=29)
	CT(32282)=(S=9044736,E=9044991,C=19)
	CT(32283)=(S=9044992,E=9045247,C=25)
	CT(32284)=(S=9045248,E=9045503,C=29)
	CT(32285)=(S=9045504,E=9045759,C=19)
	CT(32286)=(S=9045760,E=9046015,C=0)
	CT(32287)=(S=9046016,E=9046527,C=29)
	CT(32288)=(S=9046528,E=9047039,C=78)
	CT(32289)=(S=9047040,E=9047551,C=29)
	CT(32290)=(S=9047552,E=9047807,C=19)
	CT(32291)=(S=9047808,E=9048831,C=29)
	CT(32292)=(S=9048832,E=9049087,C=0)
	CT(32293)=(S=9049088,E=9049343,C=29)
	CT(32294)=(S=9049344,E=9049599,C=12)
	CT(32295)=(S=9049600,E=9049855,C=42)
	CT(32296)=(S=9049856,E=9050111,C=29)
	CT(32297)=(S=9050112,E=9050623,C=0)
	CT(32298)=(S=9050624,E=9051647,C=29)
	CT(32299)=(S=9051648,E=9051903,C=2)
	CT(32300)=(S=9051904,E=9052671,C=29)
	CT(32301)=(S=9052672,E=9052927,C=78)
	CT(32302)=(S=9052928,E=9053183,C=29)
	CT(32303)=(S=9053184,E=9053199,C=189)
	CT(32304)=(S=9053200,E=9053203,C=83)
	CT(32305)=(S=9053204,E=9053207,C=189)
	CT(32306)=(S=9053208,E=9053211,C=177)
	CT(32307)=(S=9053212,E=9053235,C=189)
	CT(32308)=(S=9053236,E=9053239,C=83)
	CT(32309)=(S=9053240,E=9053247,C=189)
	CT(32310)=(S=9053248,E=9053251,C=181)
	CT(32311)=(S=9053252,E=9053259,C=189)
	CT(32312)=(S=9053260,E=9053263,C=201)
	CT(32313)=(S=9053264,E=9053275,C=189)
	CT(32314)=(S=9053276,E=9053279,C=208)
	CT(32315)=(S=9053280,E=9053283,C=83)
	CT(32316)=(S=9053284,E=9053319,C=189)
	CT(32317)=(S=9053320,E=9053323,C=208)
	CT(32318)=(S=9053324,E=9053327,C=189)
	CT(32319)=(S=9053328,E=9053331,C=212)
	CT(32320)=(S=9053332,E=9053335,C=83)
	CT(32321)=(S=9053336,E=9053339,C=204)
	CT(32322)=(S=9053340,E=9053403,C=189)
	CT(32323)=(S=9053404,E=9053407,C=83)
	CT(32324)=(S=9053408,E=9053419,C=189)
	CT(32325)=(S=9053420,E=9053423,C=83)
	CT(32326)=(S=9053424,E=9053435,C=189)
	CT(32327)=(S=9053436,E=9053439,C=203)
	CT(32328)=(S=9053440,E=9053951,C=14)
	CT(32329)=(S=9053952,E=9054207,C=29)
	CT(32330)=(S=9054208,E=9054463,C=14)
	CT(32331)=(S=9054464,E=9054719,C=15)
	CT(32332)=(S=9054720,E=9055231,C=29)
	CT(32333)=(S=9055232,E=9055487,C=0)
	CT(32334)=(S=9055488,E=9056255,C=29)
	CT(32335)=(S=9056256,E=9056511,C=49)
	CT(32336)=(S=9056512,E=9057023,C=29)
	CT(32337)=(S=9057024,E=9057279,C=78)
	CT(32338)=(S=9057280,E=9059071,C=29)
	CT(32339)=(S=9059072,E=9059075,C=83)
	CT(32340)=(S=9059076,E=9059079,C=208)
	CT(32341)=(S=9059080,E=9059083,C=200)
	CT(32342)=(S=9059084,E=9059087,C=205)
	CT(32343)=(S=9059088,E=9059091,C=202)
	CT(32344)=(S=9059092,E=9059095,C=189)
	CT(32345)=(S=9059096,E=9059099,C=176)
	CT(32346)=(S=9059100,E=9059135,C=189)
	CT(32347)=(S=9059136,E=9059139,C=212)
	CT(32348)=(S=9059140,E=9059179,C=189)
	CT(32349)=(S=9059180,E=9059183,C=213)
	CT(32350)=(S=9059184,E=9059187,C=189)
	CT(32351)=(S=9059188,E=9059191,C=200)
	CT(32352)=(S=9059192,E=9059203,C=189)
	CT(32353)=(S=9059204,E=9059207,C=202)
	CT(32354)=(S=9059208,E=9059211,C=189)
	CT(32355)=(S=9059212,E=9059215,C=83)
	CT(32356)=(S=9059216,E=9059219,C=189)
	CT(32357)=(S=9059220,E=9059223,C=209)
	CT(32358)=(S=9059224,E=9059235,C=189)
	CT(32359)=(S=9059236,E=9059239,C=206)
	CT(32360)=(S=9059240,E=9059247,C=83)
	CT(32361)=(S=9059248,E=9059251,C=208)
	CT(32362)=(S=9059252,E=9059275,C=189)
	CT(32363)=(S=9059276,E=9059279,C=208)
	CT(32364)=(S=9059280,E=9059315,C=189)
	CT(32365)=(S=9059316,E=9059319,C=83)
	CT(32366)=(S=9059320,E=9059327,C=189)
	CT(32367)=(S=9059328,E=9059839,C=29)
	CT(32368)=(S=9059840,E=9060095,C=27)
	CT(32369)=(S=9060096,E=9060351,C=12)
	CT(32370)=(S=9060352,E=9060863,C=29)
	CT(32371)=(S=9060864,E=9061119,C=15)
	CT(32372)=(S=9061120,E=9061887,C=29)
	CT(32373)=(S=9061888,E=9062143,C=15)
	CT(32374)=(S=9062144,E=9062399,C=0)
	CT(32375)=(S=9062400,E=9062655,C=29)
	CT(32376)=(S=9062656,E=9062911,C=78)
	CT(32377)=(S=9062912,E=9063167,C=29)
	CT(32378)=(S=9063168,E=9063423,C=7)
	CT(32379)=(S=9063424,E=9063679,C=29)
	CT(32380)=(S=9063680,E=9063935,C=0)
	CT(32381)=(S=9063936,E=9064191,C=29)
	CT(32382)=(S=9064192,E=9064703,C=0)
	CT(32383)=(S=9064704,E=9064959,C=34)
	CT(32384)=(S=9064960,E=9065215,C=78)
	CT(32385)=(S=9065216,E=9068031,C=29)
	CT(32386)=(S=9068032,E=9068035,C=181)
	CT(32387)=(S=9068036,E=9068043,C=189)
	CT(32388)=(S=9068044,E=9068047,C=83)
	CT(32389)=(S=9068048,E=9068087,C=189)
	CT(32390)=(S=9068088,E=9068091,C=202)
	CT(32391)=(S=9068092,E=9068151,C=189)
	CT(32392)=(S=9068152,E=9068155,C=208)
	CT(32393)=(S=9068156,E=9068171,C=189)
	CT(32394)=(S=9068172,E=9068175,C=204)
	CT(32395)=(S=9068176,E=9068179,C=208)
	CT(32396)=(S=9068180,E=9068247,C=189)
	CT(32397)=(S=9068248,E=9068251,C=208)
	CT(32398)=(S=9068252,E=9068271,C=189)
	CT(32399)=(S=9068272,E=9068275,C=176)
	CT(32400)=(S=9068276,E=9068279,C=210)
	CT(32401)=(S=9068280,E=9068283,C=214)
	CT(32402)=(S=9068284,E=9068287,C=215)
	CT(32403)=(S=9068288,E=9068543,C=29)
	CT(32404)=(S=9068544,E=9068799,C=12)
	CT(32405)=(S=9068800,E=9068855,C=189)
	CT(32406)=(S=9068856,E=9068859,C=181)
	CT(32407)=(S=9068860,E=9068879,C=189)
	CT(32408)=(S=9068880,E=9068883,C=181)
	CT(32409)=(S=9068884,E=9068939,C=189)
	CT(32410)=(S=9068940,E=9068943,C=210)
	CT(32411)=(S=9068944,E=9068955,C=189)
	CT(32412)=(S=9068956,E=9068959,C=170)
	CT(32413)=(S=9068960,E=9068963,C=216)
	CT(32414)=(S=9068964,E=9068967,C=189)
	CT(32415)=(S=9068968,E=9068971,C=204)
	CT(32416)=(S=9068972,E=9068975,C=189)
	CT(32417)=(S=9068976,E=9068979,C=83)
	CT(32418)=(S=9068980,E=9068999,C=189)
	CT(32419)=(S=9069000,E=9069003,C=83)
	CT(32420)=(S=9069004,E=9069035,C=189)
	CT(32421)=(S=9069036,E=9069039,C=83)
	CT(32422)=(S=9069040,E=9069047,C=189)
	CT(32423)=(S=9069048,E=9069051,C=177)
	CT(32424)=(S=9069052,E=9069055,C=189)
	CT(32425)=(S=9069056,E=9069311,C=29)
	CT(32426)=(S=9069312,E=9069315,C=210)
	CT(32427)=(S=9069316,E=9069319,C=83)
	CT(32428)=(S=9069320,E=9069347,C=189)
	CT(32429)=(S=9069348,E=9069351,C=208)
	CT(32430)=(S=9069352,E=9069355,C=189)
	CT(32431)=(S=9069356,E=9069359,C=205)
	CT(32432)=(S=9069360,E=9069411,C=189)
	CT(32433)=(S=9069412,E=9069415,C=206)
	CT(32434)=(S=9069416,E=9069431,C=189)
	CT(32435)=(S=9069432,E=9069435,C=170)
	CT(32436)=(S=9069436,E=9069439,C=200)
	CT(32437)=(S=9069440,E=9069443,C=83)
	CT(32438)=(S=9069444,E=9069447,C=189)
	CT(32439)=(S=9069448,E=9069451,C=209)
	CT(32440)=(S=9069452,E=9069455,C=189)
	CT(32441)=(S=9069456,E=9069459,C=209)
	CT(32442)=(S=9069460,E=9069463,C=204)
	CT(32443)=(S=9069464,E=9069483,C=189)
	CT(32444)=(S=9069484,E=9069487,C=83)
	CT(32445)=(S=9069488,E=9069491,C=203)
	CT(32446)=(S=9069492,E=9069495,C=210)
	CT(32447)=(S=9069496,E=9069499,C=83)
	CT(32448)=(S=9069500,E=9069519,C=189)
	CT(32449)=(S=9069520,E=9069523,C=170)
	CT(32450)=(S=9069524,E=9069527,C=207)
	CT(32451)=(S=9069528,E=9069531,C=170)
	CT(32452)=(S=9069532,E=9069535,C=200)
	CT(32453)=(S=9069536,E=9069539,C=203)
	CT(32454)=(S=9069540,E=9069547,C=189)
	CT(32455)=(S=9069548,E=9069551,C=177)
	CT(32456)=(S=9069552,E=9069567,C=189)
	CT(32457)=(S=9069568,E=9069823,C=25)
	CT(32458)=(S=9069824,E=9070079,C=2)
	CT(32459)=(S=9070080,E=9070335,C=12)
	CT(32460)=(S=9070336,E=9070591,C=19)
	CT(32461)=(S=9070592,E=9070847,C=14)
	CT(32462)=(S=9070848,E=9071103,C=29)
	CT(32463)=(S=9071104,E=9071359,C=19)
	CT(32464)=(S=9071360,E=9071615,C=2)
	CT(32465)=(S=9071616,E=9073919,C=29)
	CT(32466)=(S=9073920,E=9073923,C=189)
	CT(32467)=(S=9073924,E=9073927,C=216)
	CT(32468)=(S=9073928,E=9073943,C=83)
	CT(32469)=(S=9073944,E=9073959,C=189)
	CT(32470)=(S=9073960,E=9073963,C=181)
	CT(32471)=(S=9073964,E=9073967,C=200)
	CT(32472)=(S=9073968,E=9073971,C=83)
	CT(32473)=(S=9073972,E=9073987,C=189)
	CT(32474)=(S=9073988,E=9073991,C=203)
	CT(32475)=(S=9073992,E=9073995,C=189)
	CT(32476)=(S=9073996,E=9073999,C=83)
	CT(32477)=(S=9074000,E=9074003,C=189)
	CT(32478)=(S=9074004,E=9074007,C=181)
	CT(32479)=(S=9074008,E=9074015,C=189)
	CT(32480)=(S=9074016,E=9074019,C=208)
	CT(32481)=(S=9074020,E=9074027,C=189)
	CT(32482)=(S=9074028,E=9074031,C=181)
	CT(32483)=(S=9074032,E=9074035,C=83)
	CT(32484)=(S=9074036,E=9074047,C=189)
	CT(32485)=(S=9074048,E=9074051,C=83)
	CT(32486)=(S=9074052,E=9074055,C=189)
	CT(32487)=(S=9074056,E=9074059,C=181)
	CT(32488)=(S=9074060,E=9074063,C=215)
	CT(32489)=(S=9074064,E=9074067,C=189)
	CT(32490)=(S=9074068,E=9074071,C=203)
	CT(32491)=(S=9074072,E=9074075,C=189)
	CT(32492)=(S=9074076,E=9074079,C=83)
	CT(32493)=(S=9074080,E=9074083,C=212)
	CT(32494)=(S=9074084,E=9074119,C=189)
	CT(32495)=(S=9074120,E=9074123,C=83)
	CT(32496)=(S=9074124,E=9074183,C=189)
	CT(32497)=(S=9074184,E=9074187,C=203)
	CT(32498)=(S=9074188,E=9074211,C=189)
	CT(32499)=(S=9074212,E=9074215,C=83)
	CT(32500)=(S=9074216,E=9074223,C=189)
	CT(32501)=(S=9074224,E=9074227,C=83)
	CT(32502)=(S=9074228,E=9074255,C=189)
	CT(32503)=(S=9074256,E=9074259,C=202)
	CT(32504)=(S=9074260,E=9074279,C=189)
	CT(32505)=(S=9074280,E=9074283,C=215)
	CT(32506)=(S=9074284,E=9074287,C=189)
	CT(32507)=(S=9074288,E=9074291,C=83)
	CT(32508)=(S=9074292,E=9074299,C=189)
	CT(32509)=(S=9074300,E=9074303,C=209)
	CT(32510)=(S=9074304,E=9074319,C=189)
	CT(32511)=(S=9074320,E=9074323,C=83)
	CT(32512)=(S=9074324,E=9074331,C=189)
	CT(32513)=(S=9074332,E=9074335,C=202)
	CT(32514)=(S=9074336,E=9074383,C=189)
	CT(32515)=(S=9074384,E=9074395,C=83)
	CT(32516)=(S=9074396,E=9074431,C=189)
	CT(32517)=(S=9074432,E=9074687,C=78)
	CT(32518)=(S=9074688,E=9074943,C=29)
	CT(32519)=(S=9074944,E=9074947,C=189)
	CT(32520)=(S=9074948,E=9074951,C=181)
	CT(32521)=(S=9074952,E=9074955,C=189)
	CT(32522)=(S=9074956,E=9074959,C=181)
	CT(32523)=(S=9074960,E=9075019,C=189)
	CT(32524)=(S=9075020,E=9075035,C=83)
	CT(32525)=(S=9075036,E=9075039,C=189)
	CT(32526)=(S=9075040,E=9075043,C=203)
	CT(32527)=(S=9075044,E=9075051,C=83)
	CT(32528)=(S=9075052,E=9075055,C=203)
	CT(32529)=(S=9075056,E=9075059,C=83)
	CT(32530)=(S=9075060,E=9075067,C=189)
	CT(32531)=(S=9075068,E=9075071,C=83)
	CT(32532)=(S=9075072,E=9075083,C=189)
	CT(32533)=(S=9075084,E=9075087,C=202)
	CT(32534)=(S=9075088,E=9075099,C=189)
	CT(32535)=(S=9075100,E=9075103,C=181)
	CT(32536)=(S=9075104,E=9075107,C=83)
	CT(32537)=(S=9075108,E=9075111,C=189)
	CT(32538)=(S=9075112,E=9075115,C=203)
	CT(32539)=(S=9075116,E=9075119,C=189)
	CT(32540)=(S=9075120,E=9075123,C=203)
	CT(32541)=(S=9075124,E=9075143,C=189)
	CT(32542)=(S=9075144,E=9075147,C=181)
	CT(32543)=(S=9075148,E=9075171,C=189)
	CT(32544)=(S=9075172,E=9075175,C=83)
	CT(32545)=(S=9075176,E=9075183,C=189)
	CT(32546)=(S=9075184,E=9075187,C=83)
	CT(32547)=(S=9075188,E=9075203,C=189)
	CT(32548)=(S=9075204,E=9075207,C=200)
	CT(32549)=(S=9075208,E=9075223,C=189)
	CT(32550)=(S=9075224,E=9075227,C=202)
	CT(32551)=(S=9075228,E=9075295,C=189)
	CT(32552)=(S=9075296,E=9075299,C=204)
	CT(32553)=(S=9075300,E=9075307,C=189)
	CT(32554)=(S=9075308,E=9075311,C=213)
	CT(32555)=(S=9075312,E=9075327,C=189)
	CT(32556)=(S=9075328,E=9075331,C=83)
	CT(32557)=(S=9075332,E=9075343,C=189)
	CT(32558)=(S=9075344,E=9075347,C=204)
	CT(32559)=(S=9075348,E=9075355,C=189)
	CT(32560)=(S=9075356,E=9075359,C=83)
	CT(32561)=(S=9075360,E=9075363,C=206)
	CT(32562)=(S=9075364,E=9075383,C=189)
	CT(32563)=(S=9075384,E=9075387,C=206)
	CT(32564)=(S=9075388,E=9075391,C=83)
	CT(32565)=(S=9075392,E=9075395,C=208)
	CT(32566)=(S=9075396,E=9075399,C=189)
	CT(32567)=(S=9075400,E=9075403,C=181)
	CT(32568)=(S=9075404,E=9075423,C=189)
	CT(32569)=(S=9075424,E=9075427,C=203)
	CT(32570)=(S=9075428,E=9075431,C=83)
	CT(32571)=(S=9075432,E=9075443,C=189)
	CT(32572)=(S=9075444,E=9075447,C=202)
	CT(32573)=(S=9075448,E=9075451,C=189)
	CT(32574)=(S=9075452,E=9075455,C=204)
	CT(32575)=(S=9075456,E=9075711,C=29)
	CT(32576)=(S=9075712,E=9075967,C=27)
	CT(32577)=(S=9075968,E=9076927,C=29)
	CT(32578)=(S=9076928,E=9076959,C=78)
	CT(32579)=(S=9076960,E=9076991,C=95)
	CT(32580)=(S=9076992,E=9077247,C=29)
	CT(32581)=(S=9077248,E=9077503,C=0)
	CT(32582)=(S=9077504,E=9077759,C=34)
	CT(32583)=(S=9077760,E=9078015,C=15)
	CT(32584)=(S=9078016,E=9078271,C=29)
	CT(32585)=(S=9078272,E=9078527,C=17)
	CT(32586)=(S=9078528,E=9091327,C=29)
	CT(32587)=(S=9091328,E=9091331,C=204)
	CT(32588)=(S=9091332,E=9091339,C=189)
	CT(32589)=(S=9091340,E=9091343,C=203)
	CT(32590)=(S=9091344,E=9091355,C=189)
	CT(32591)=(S=9091356,E=9091359,C=216)
	CT(32592)=(S=9091360,E=9091391,C=189)
	CT(32593)=(S=9091392,E=9091399,C=200)
	CT(32594)=(S=9091400,E=9091403,C=189)
	CT(32595)=(S=9091404,E=9091407,C=170)
	CT(32596)=(S=9091408,E=9091411,C=83)
	CT(32597)=(S=9091412,E=9091431,C=189)
	CT(32598)=(S=9091432,E=9091435,C=216)
	CT(32599)=(S=9091436,E=9091439,C=189)
	CT(32600)=(S=9091440,E=9091443,C=83)
	CT(32601)=(S=9091444,E=9091447,C=212)
	CT(32602)=(S=9091448,E=9091463,C=189)
	CT(32603)=(S=9091464,E=9091467,C=213)
	CT(32604)=(S=9091468,E=9091471,C=83)
	CT(32605)=(S=9091472,E=9091531,C=189)
	CT(32606)=(S=9091532,E=9091535,C=83)
	CT(32607)=(S=9091536,E=9091539,C=217)
	CT(32608)=(S=9091540,E=9091551,C=189)
	CT(32609)=(S=9091552,E=9091555,C=204)
	CT(32610)=(S=9091556,E=9091575,C=189)
	CT(32611)=(S=9091576,E=9091579,C=202)
	CT(32612)=(S=9091580,E=9091587,C=189)
	CT(32613)=(S=9091588,E=9091591,C=200)
	CT(32614)=(S=9091592,E=9091595,C=203)
	CT(32615)=(S=9091596,E=9091599,C=209)
	CT(32616)=(S=9091600,E=9091603,C=189)
	CT(32617)=(S=9091604,E=9091607,C=181)
	CT(32618)=(S=9091608,E=9091611,C=189)
	CT(32619)=(S=9091612,E=9091615,C=204)
	CT(32620)=(S=9091616,E=9091639,C=189)
	CT(32621)=(S=9091644,E=9091647,C=206)
	CT(32622)=(S=9091648,E=9091651,C=204)
	CT(32623)=(S=9091652,E=9091671,C=189)
	CT(32624)=(S=9091672,E=9091675,C=209)
	CT(32625)=(S=9091676,E=9091683,C=189)
	CT(32626)=(S=9091684,E=9091687,C=204)
	CT(32627)=(S=9091688,E=9091703,C=189)
	CT(32628)=(S=9091704,E=9091707,C=203)
	CT(32629)=(S=9091708,E=9091719,C=189)
	CT(32630)=(S=9091720,E=9091723,C=208)
	CT(32631)=(S=9091724,E=9091727,C=83)
	CT(32632)=(S=9091728,E=9091731,C=189)
	CT(32633)=(S=9091732,E=9091739,C=83)
	CT(32634)=(S=9091740,E=9091743,C=189)
	CT(32635)=(S=9091744,E=9091747,C=83)
	CT(32636)=(S=9091748,E=9091759,C=189)
	CT(32637)=(S=9091760,E=9091763,C=200)
	CT(32638)=(S=9091764,E=9091771,C=189)
	CT(32639)=(S=9091772,E=9091775,C=181)
	CT(32640)=(S=9091776,E=9091783,C=189)
	CT(32641)=(S=9091784,E=9091787,C=204)
	CT(32642)=(S=9091788,E=9091791,C=189)
	CT(32643)=(S=9091792,E=9091795,C=208)
	CT(32644)=(S=9091796,E=9091803,C=83)
	CT(32645)=(S=9091804,E=9091827,C=189)
	CT(32646)=(S=9091828,E=9091831,C=83)
	CT(32647)=(S=9091832,E=9091835,C=210)
	CT(32648)=(S=9091836,E=9091839,C=83)
	CT(32649)=(S=9091840,E=9093119,C=34)
	CT(32650)=(S=9093120,E=9093631,C=29)
	CT(32651)=(S=9093632,E=9093887,C=0)
	CT(32652)=(S=9093888,E=9094143,C=12)
	CT(32653)=(S=9094144,E=9094655,C=29)
	CT(32654)=(S=9094656,E=9094911,C=14)
	CT(32655)=(S=9094912,E=9095167,C=13)
	CT(32656)=(S=9095168,E=9095679,C=28)
	CT(32657)=(S=9095680,E=9095935,C=29)
	CT(32658)=(S=9095936,E=9096191,C=49)
	CT(32659)=(S=9096192,E=9096199,C=83)
	CT(32660)=(S=9096200,E=9096203,C=189)
	CT(32661)=(S=9096204,E=9096207,C=170)
	CT(32662)=(S=9096208,E=9096231,C=189)
	CT(32663)=(S=9096232,E=9096235,C=83)
	CT(32664)=(S=9096236,E=9096299,C=189)
	CT(32665)=(S=9096300,E=9096303,C=209)
	CT(32666)=(S=9096304,E=9096343,C=189)
	CT(32667)=(S=9096344,E=9096355,C=83)
	CT(32668)=(S=9096356,E=9096359,C=189)
	CT(32669)=(S=9096360,E=9096363,C=170)
	CT(32670)=(S=9096364,E=9096371,C=189)
	CT(32671)=(S=9096372,E=9096375,C=208)
	CT(32672)=(S=9096376,E=9096395,C=189)
	CT(32673)=(S=9096396,E=9096399,C=204)
	CT(32674)=(S=9096400,E=9096407,C=189)
	CT(32675)=(S=9096408,E=9096411,C=83)
	CT(32676)=(S=9096412,E=9096419,C=189)
	CT(32677)=(S=9096420,E=9096423,C=203)
	CT(32678)=(S=9096424,E=9096427,C=189)
	CT(32679)=(S=9096428,E=9096431,C=200)
	CT(32680)=(S=9096432,E=9096435,C=189)
	CT(32681)=(S=9096436,E=9096447,C=83)
	CT(32682)=(S=9096448,E=9096703,C=49)
	CT(32683)=(S=9096704,E=9096959,C=34)
	CT(32684)=(S=9096960,E=9097983,C=29)
	CT(32685)=(S=9097984,E=9098239,C=73)
	CT(32686)=(S=9098240,E=9098495,C=2)
	CT(32687)=(S=9098752,E=9099007,C=78)
	CT(32688)=(S=9099008,E=9099263,C=19)
	CT(32689)=(S=9099264,E=9099519,C=47)
	CT(32690)=(S=9099520,E=9099775,C=0)
	CT(32691)=(S=9099776,E=9100031,C=78)
	CT(32692)=(S=9100032,E=9100039,C=83)
	CT(32693)=(S=9100040,E=9100043,C=206)
	CT(32694)=(S=9100044,E=9100047,C=210)
	CT(32695)=(S=9100048,E=9100051,C=83)
	CT(32696)=(S=9100052,E=9100067,C=189)
	CT(32697)=(S=9100068,E=9100071,C=204)
	CT(32698)=(S=9100072,E=9100075,C=83)
	CT(32699)=(S=9100076,E=9100087,C=189)
	CT(32700)=(S=9100092,E=9100095,C=83)
	CT(32701)=(S=9100096,E=9100103,C=189)
	CT(32702)=(S=9100104,E=9100107,C=208)
	CT(32703)=(S=9100108,E=9100111,C=189)
	CT(32704)=(S=9100112,E=9100115,C=83)
	CT(32705)=(S=9100116,E=9100151,C=189)
	CT(32706)=(S=9100152,E=9100155,C=208)
	CT(32707)=(S=9100156,E=9100171,C=189)
	CT(32708)=(S=9100172,E=9100175,C=207)
	CT(32709)=(S=9100176,E=9100187,C=189)
	CT(32710)=(S=9100188,E=9100191,C=210)
	CT(32711)=(S=9100192,E=9100195,C=83)
	CT(32712)=(S=9100196,E=9100203,C=189)
	CT(32713)=(S=9100204,E=9100207,C=208)
	CT(32714)=(S=9100208,E=9100243,C=189)
	CT(32715)=(S=9100244,E=9100251,C=83)
	CT(32716)=(S=9100252,E=9100259,C=189)
	CT(32717)=(S=9100260,E=9100263,C=203)
	CT(32718)=(S=9100268,E=9100279,C=189)
	CT(32719)=(S=9100280,E=9100283,C=83)
	CT(32720)=(S=9100288,E=9100543,C=29)
	CT(32721)=(S=9100544,E=9102591,C=34)
	CT(32722)=(S=9102592,E=9102607,C=29)
	CT(32723)=(S=9102608,E=9102623,C=78)
	CT(32724)=(S=9102624,E=9103103,C=29)
	CT(32725)=(S=9103104,E=9103359,C=12)
	CT(32726)=(S=9103360,E=9103615,C=42)
	CT(32727)=(S=9103616,E=9103871,C=19)
	CT(32728)=(S=9103872,E=9104127,C=29)
	CT(32729)=(S=9104128,E=9104383,C=73)
	CT(32730)=(S=9104384,E=9106175,C=29)
	CT(32731)=(S=9106176,E=9106431,C=2)
	CT(32732)=(S=9106432,E=9107199,C=28)
	CT(32733)=(S=9107200,E=9107455,C=29)
	CT(32734)=(S=9107456,E=9107711,C=14)
	CT(32735)=(S=9107712,E=9107967,C=47)
	CT(32736)=(S=9107968,E=9108991,C=14)
	CT(32737)=(S=9108992,E=9109247,C=29)
	CT(32738)=(S=9109248,E=9109299,C=189)
	CT(32739)=(S=9109300,E=9109303,C=209)
	CT(32740)=(S=9109304,E=9109307,C=189)
	CT(32741)=(S=9109308,E=9109311,C=204)
	CT(32742)=(S=9109312,E=9109343,C=189)
	CT(32743)=(S=9109344,E=9109351,C=203)
	CT(32744)=(S=9109352,E=9109363,C=189)
	CT(32745)=(S=9109364,E=9109367,C=204)
	CT(32746)=(S=9109368,E=9109399,C=189)
	CT(32747)=(S=9109400,E=9109403,C=210)
	CT(32748)=(S=9109404,E=9109415,C=189)
	CT(32749)=(S=9109416,E=9109419,C=83)
	CT(32750)=(S=9109420,E=9109495,C=189)
	CT(32751)=(S=9109496,E=9109499,C=177)
	CT(32752)=(S=9109504,E=9109759,C=74)
	CT(32753)=(S=9109760,E=9111807,C=28)
	CT(32754)=(S=9111808,E=9112063,C=1)
	CT(32755)=(S=9112064,E=9112319,C=49)
	CT(32756)=(S=9112320,E=9117439,C=28)
	CT(32757)=(S=9117440,E=9120767,C=29)
	CT(32758)=(S=9120768,E=9121023,C=0)
	CT(32759)=(S=9121024,E=9121791,C=29)
	CT(32760)=(S=9121792,E=9122047,C=78)
	CT(32761)=(S=9122048,E=9122303,C=29)
	CT(32762)=(S=9122304,E=9122559,C=28)
	CT(32763)=(S=9122560,E=9123071,C=29)
	CT(32764)=(S=9123072,E=9123327,C=105)
	CT(32765)=(S=9123328,E=9123583,C=12)
	CT(32766)=(S=9123584,E=9124095,C=29)
	CT(32767)=(S=9124096,E=9124351,C=78)
	CT(32768)=(S=9124352,E=9124607,C=19)
	CT(32769)=(S=9124608,E=9124863,C=0)
	CT(32770)=(S=9124864,E=9125631,C=29)
	CT(32771)=(S=9125632,E=9125887,C=36)
	CT(32772)=(S=9126144,E=9126399,C=29)
	CT(32773)=(S=9126400,E=9126655,C=47)
	CT(32774)=(S=9126656,E=9128447,C=29)
	CT(32775)=(S=9128448,E=9128703,C=47)
	CT(32776)=(S=9128704,E=9128959,C=28)
	CT(32777)=(S=9128960,E=9129727,C=29)
	CT(32778)=(S=9129728,E=9129983,C=34)
	CT(32779)=(S=9129984,E=9130239,C=73)
	CT(32780)=(S=9130240,E=9130495,C=29)
	CT(32781)=(S=9130496,E=9130751,C=189)
	CT(32782)=(S=9130752,E=9131007,C=21)
	CT(32783)=(S=9131008,E=9131519,C=29)
	CT(32784)=(S=9131520,E=9131775,C=0)
	CT(32785)=(S=9131776,E=9132287,C=29)
	CT(32786)=(S=9132288,E=9132543,C=28)
	CT(32787)=(S=9132544,E=9132799,C=49)
	CT(32788)=(S=9132800,E=9133055,C=22)
	CT(32789)=(S=9133056,E=9133311,C=13)
	CT(32790)=(S=9133312,E=9134079,C=29)
	CT(32791)=(S=9134080,E=9134335,C=19)
	CT(32792)=(S=9134336,E=9134591,C=47)
	CT(32793)=(S=9134592,E=9134847,C=27)
	CT(32794)=(S=9134848,E=9135103,C=29)
	CT(32795)=(S=9135104,E=9135359,C=12)
	CT(32796)=(S=9135360,E=9135615,C=2)
	CT(32797)=(S=9135616,E=9135871,C=29)
	CT(32798)=(S=9135872,E=9136127,C=78)
	CT(32799)=(S=9136128,E=9136383,C=29)
	CT(32800)=(S=9136384,E=9140479,C=27)
	CT(32801)=(S=9140480,E=9140735,C=29)
	CT(32802)=(S=9140736,E=9140991,C=36)
	CT(32803)=(S=9140992,E=9141247,C=47)
	CT(32804)=(S=9141248,E=9141503,C=12)
	CT(32805)=(S=9141504,E=9142271,C=29)
	CT(32806)=(S=9142272,E=9142527,C=15)
	CT(32807)=(S=9142528,E=9142783,C=1)
	CT(32808)=(S=9142784,E=9143039,C=0)
	CT(32809)=(S=9143040,E=9143295,C=29)
	CT(32810)=(S=9143296,E=9143551,C=0)
	CT(32811)=(S=9143552,E=9143807,C=14)
	CT(32812)=(S=9143808,E=9144063,C=0)
	CT(32813)=(S=9144064,E=9145599,C=29)
	CT(32814)=(S=9145600,E=9145855,C=101)
	CT(32815)=(S=9145856,E=9146111,C=78)
	CT(32816)=(S=9146112,E=9146367,C=14)
	CT(32817)=(S=9146368,E=9146623,C=29)
	CT(32818)=(S=9146624,E=9146879,C=27)
	CT(32819)=(S=9146880,E=9147391,C=29)
	CT(32820)=(S=9147392,E=9147647,C=1)
	CT(32821)=(S=9147648,E=9147903,C=14)
	CT(32822)=(S=9147904,E=9148159,C=6)
	CT(32823)=(S=9148160,E=9148415,C=29)
	CT(32824)=(S=9148416,E=9148671,C=2)
	CT(32825)=(S=9148672,E=9148927,C=14)
	CT(32826)=(S=9148928,E=9149183,C=2)
	CT(32827)=(S=9149184,E=9149439,C=1)
	CT(32828)=(S=9149440,E=9149695,C=36)
	CT(32829)=(S=9149696,E=9149951,C=47)
	CT(32830)=(S=9149952,E=9150207,C=12)
	CT(32831)=(S=9150208,E=9150463,C=1)
	CT(32832)=(S=9150464,E=9150719,C=12)
	CT(32833)=(S=9150720,E=9150975,C=29)
	CT(32834)=(S=9150976,E=9151231,C=36)
	CT(32835)=(S=9151232,E=9151487,C=0)
	CT(32836)=(S=9151488,E=9151743,C=27)
	CT(32837)=(S=9151744,E=9151999,C=49)
	CT(32838)=(S=9152000,E=9152255,C=14)
	CT(32839)=(S=9152256,E=9152511,C=29)
	CT(32840)=(S=9152512,E=9152767,C=0)
	CT(32841)=(S=9152768,E=9153023,C=29)
	CT(32842)=(S=9153024,E=9153279,C=1)
	CT(32843)=(S=9153280,E=9153791,C=29)
	CT(32844)=(S=9153792,E=9154047,C=78)
	CT(32845)=(S=9154048,E=9154303,C=28)
	CT(32846)=(S=9154304,E=9154559,C=9)
	CT(32847)=(S=9154560,E=9154815,C=1)
	CT(32848)=(S=9154816,E=9155071,C=29)
	CT(32849)=(S=9155072,E=9155327,C=36)
	CT(32850)=(S=9155328,E=9155583,C=37)
	CT(32851)=(S=9155584,E=9156351,C=29)
	CT(32852)=(S=9156352,E=9156607,C=1)
	CT(32853)=(S=9156608,E=9156863,C=14)
	CT(32854)=(S=9156864,E=9157119,C=29)
	CT(32855)=(S=9157120,E=9157375,C=1)
	CT(32856)=(S=9157376,E=9157631,C=29)
	CT(32857)=(S=9157632,E=9157887,C=0)
	CT(32858)=(S=9157888,E=9158143,C=1)
	CT(32859)=(S=9158144,E=9158399,C=77)
	CT(32860)=(S=9158400,E=9158655,C=13)
	CT(32861)=(S=9158656,E=9159679,C=74)
	CT(32862)=(S=9159680,E=9164799,C=1)
	CT(32863)=(S=9164800,E=9165055,C=0)
	CT(32864)=(S=9165056,E=9165311,C=1)
	CT(32865)=(S=9165312,E=9165567,C=0)
	CT(32866)=(S=9165568,E=9166335,C=1)
	CT(32867)=(S=9166336,E=9166591,C=14)
	CT(32868)=(S=9166592,E=9166847,C=9)
	CT(32869)=(S=9166848,E=9167103,C=1)
	CT(32870)=(S=9167104,E=9167359,C=29)
	CT(32871)=(S=9167360,E=9167871,C=1)
	CT(32872)=(S=9167872,E=9168127,C=74)
	CT(32873)=(S=9168128,E=9168383,C=29)
	CT(32874)=(S=9168384,E=9168639,C=0)
	CT(32875)=(S=9168640,E=9174783,C=29)
	CT(32876)=(S=9174784,E=9175295,C=74)
	CT(32877)=(S=9175296,E=9194239,C=29)
	CT(32878)=(S=9194240,E=9194495,C=1)
	CT(32879)=(S=9194496,E=9194751,C=29)
	CT(32880)=(S=9194752,E=9195007,C=12)
	CT(32881)=(S=9195008,E=9195263,C=42)
	CT(32882)=(S=9195264,E=9195519,C=0)
	CT(32883)=(S=9195520,E=9195775,C=78)
	CT(32884)=(S=9196032,E=9196287,C=29)
	CT(32885)=(S=9196288,E=9196543,C=0)
	CT(32886)=(S=9196544,E=9197311,C=19)
	CT(32887)=(S=9197312,E=9198591,C=29)
	CT(32888)=(S=9198592,E=9198847,C=9)
	CT(32889)=(S=9198848,E=9199359,C=12)
	CT(32890)=(S=9199360,E=9199615,C=29)
	CT(32891)=(S=9199616,E=9199871,C=9)
	CT(32892)=(S=9199872,E=9200127,C=14)
	CT(32893)=(S=9200128,E=9201919,C=29)
	CT(32894)=(S=9201920,E=9202175,C=15)
	CT(32895)=(S=9202176,E=9202943,C=29)
	CT(32896)=(S=9202944,E=9210623,C=9)
	CT(32897)=(S=9210624,E=9211647,C=29)
	CT(32898)=(S=9211648,E=9211903,C=1)
	CT(32899)=(S=9211904,E=9212927,C=29)
	CT(32900)=(S=9212928,E=9213183,C=204)
	CT(32901)=(S=9213184,E=9213439,C=3)
	CT(32902)=(S=9213440,E=9213695,C=19)
	CT(32903)=(S=9213696,E=9213951,C=47)
	CT(32904)=(S=9213952,E=9215743,C=29)
	CT(32905)=(S=9215744,E=9215999,C=0)
	CT(32906)=(S=9216000,E=9216255,C=29)
	CT(32907)=(S=9216256,E=9216511,C=78)
	CT(32908)=(S=9216512,E=9217023,C=29)
	CT(32909)=(S=9217024,E=9217279,C=15)
	CT(32910)=(S=9217280,E=9217535,C=29)
	CT(32911)=(S=9217536,E=9217791,C=19)
	CT(32912)=(S=9217792,E=9218047,C=78)
	CT(32913)=(S=9218048,E=9218303,C=0)
	CT(32914)=(S=9218304,E=9221375,C=29)
	CT(32915)=(S=9221376,E=9221631,C=28)
	CT(32916)=(S=9221632,E=9222143,C=29)
	CT(32917)=(S=9222144,E=9222399,C=78)
	CT(32918)=(S=9222400,E=9223935,C=29)
	CT(32919)=(S=9223936,E=9224191,C=83)
	CT(32920)=(S=9224192,E=9224447,C=29)
	CT(32921)=(S=9224448,E=9224703,C=78)
	CT(32922)=(S=9224704,E=9226239,C=29)
	CT(32923)=(S=9226240,E=9226495,C=73)
	CT(32924)=(S=9226496,E=9227007,C=29)
	CT(32925)=(S=9227008,E=9227263,C=63)
	CT(32926)=(S=9227264,E=9227519,C=29)
	CT(32927)=(S=9227520,E=9228287,C=1)
	CT(32928)=(S=9228288,E=9228799,C=29)
	CT(32929)=(S=9228800,E=9229055,C=1)
	CT(32930)=(S=9229056,E=9229567,C=29)
	CT(32931)=(S=9229568,E=9229823,C=74)
	CT(32932)=(S=9229824,E=9232383,C=29)
	CT(32933)=(S=9232384,E=9232639,C=1)
	CT(32934)=(S=9232640,E=9233919,C=29)
	CT(32935)=(S=9233920,E=9234175,C=78)
	CT(32936)=(S=9234176,E=9234431,C=28)
	CT(32937)=(S=9234432,E=9235711,C=29)
	CT(32938)=(S=9235712,E=9235967,C=1)
	CT(32939)=(S=9235968,E=9236479,C=29)
	CT(32940)=(S=9236480,E=9236735,C=1)
	CT(32941)=(S=9236736,E=9237247,C=29)
	CT(32942)=(S=9237248,E=9237503,C=1)
	CT(32943)=(S=9237504,E=9238015,C=29)
	CT(32944)=(S=9238016,E=9238271,C=1)
	CT(32945)=(S=9238272,E=9238783,C=29)
	CT(32946)=(S=9238784,E=9239295,C=1)
	CT(32947)=(S=9239296,E=9239807,C=29)
	CT(32948)=(S=9239808,E=9240063,C=0)
	CT(32949)=(S=9240064,E=9240319,C=29)
	CT(32950)=(S=9240320,E=9240575,C=1)
	CT(32951)=(S=9240576,E=9240583,C=61)
	CT(32952)=(S=9240584,E=9240591,C=29)
	CT(32953)=(S=9240592,E=9240599,C=28)
	CT(32954)=(S=9240600,E=9240607,C=36)
	CT(32955)=(S=9240608,E=9240639,C=14)
	CT(32956)=(S=9240640,E=9240719,C=27)
	CT(32957)=(S=9240720,E=9240743,C=14)
	CT(32958)=(S=9240744,E=9240751,C=36)
	CT(32959)=(S=9240752,E=9240759,C=18)
	CT(32960)=(S=9240760,E=9240767,C=12)
	CT(32961)=(S=9240768,E=9240775,C=14)
	CT(32962)=(S=9240776,E=9240783,C=12)
	CT(32963)=(S=9240784,E=9240799,C=14)
	CT(32964)=(S=9240800,E=9240831,C=27)
	CT(32965)=(S=9240832,E=9242623,C=28)
	CT(32966)=(S=9242624,E=9242751,C=72)
	CT(32967)=(S=9242752,E=9242823,C=18)
	CT(32968)=(S=9242824,E=9242831,C=96)
	CT(32969)=(S=9242832,E=9242839,C=40)
	CT(32970)=(S=9242840,E=9242847,C=14)
	CT(32971)=(S=9242848,E=9242855,C=34)
	CT(32972)=(S=9242856,E=9243391,C=28)
	CT(32973)=(S=9243392,E=9243647,C=14)
	CT(32974)=(S=9243648,E=9261311,C=28)
	CT(32975)=(S=9261312,E=9261567,C=19)
	CT(32976)=(S=9261568,E=9262335,C=28)
	CT(32977)=(S=9262336,E=9262591,C=38)
	CT(32978)=(S=9262592,E=9262847,C=29)
	CT(32979)=(S=9262848,E=9264127,C=28)
	CT(32980)=(S=9264128,E=9264383,C=14)
	CT(32981)=(S=9264384,E=9264639,C=36)
	CT(32982)=(S=9264640,E=9265151,C=14)
	CT(32983)=(S=9265152,E=9265407,C=49)
	CT(32984)=(S=9265408,E=9265663,C=14)
	CT(32985)=(S=9265920,E=9266431,C=28)
	CT(32986)=(S=9266432,E=9266463,C=31)
	CT(32987)=(S=9266464,E=9266479,C=28)
	CT(32988)=(S=9266480,E=9266495,C=12)
	CT(32989)=(S=9266496,E=9266559,C=13)
	CT(32990)=(S=9266560,E=9266567,C=36)
	CT(32991)=(S=9266568,E=9266574,C=18)
	CT(32992)=(S=9266575,E=9266606,C=36)
	CT(32993)=(S=9266607,E=9266687,C=18)
	CT(32994)=(S=9266688,E=9266943,C=29)
	CT(32995)=(S=9266944,E=9267199,C=74)
	CT(32996)=(S=9267200,E=9267455,C=29)
	CT(32997)=(S=9267456,E=9267471,C=36)
	CT(32998)=(S=9267472,E=9267479,C=58)
	CT(32999)=(S=9267480,E=9267511,C=18)
	CT(33000)=(S=9267512,E=9267519,C=61)
	CT(33001)=(S=9267520,E=9267527,C=18)
	CT(33002)=(S=9267528,E=9267535,C=12)
	CT(33003)=(S=9267536,E=9267551,C=39)
	CT(33004)=(S=9267552,E=9267567,C=25)
	CT(33005)=(S=9267568,E=9267575,C=63)
	CT(33006)=(S=9267576,E=9267583,C=36)
	CT(33007)=(S=9267584,E=9267599,C=31)
	CT(33008)=(S=9267600,E=9267607,C=36)
	CT(33009)=(S=9267608,E=9267615,C=15)
	CT(33010)=(S=9267616,E=9267623,C=16)
	CT(33011)=(S=9267624,E=9267631,C=52)
	CT(33012)=(S=9267632,E=9267711,C=14)
	CT(33013)=(S=9267712,E=9268223,C=29)
	CT(33014)=(S=9268224,E=9268479,C=15)
	CT(33015)=(S=9268480,E=9268735,C=78)
	CT(33016)=(S=9268736,E=9269503,C=29)
	CT(33017)=(S=9269504,E=9269759,C=28)
	CT(33018)=(S=9269760,E=9270015,C=29)
	CT(33019)=(S=9270016,E=9270271,C=12)
	CT(33020)=(S=9270272,E=9270527,C=29)
	CT(33021)=(S=9270528,E=9271295,C=78)
	CT(33022)=(S=9271296,E=9271551,C=0)
	CT(33023)=(S=9271552,E=9273855,C=29)
	CT(33024)=(S=9273856,E=9274111,C=28)
	CT(33025)=(S=9274112,E=9274367,C=29)
	CT(33026)=(S=9274368,E=9274623,C=0)
	CT(33027)=(S=9274624,E=9274879,C=29)
	CT(33028)=(S=9274880,E=9275391,C=49)
	CT(33029)=(S=9275392,E=9275399,C=60)
	CT(33030)=(S=9275400,E=9275407,C=54)
	CT(33031)=(S=9275408,E=9275423,C=38)
	CT(33032)=(S=9275424,E=9275439,C=44)
	CT(33033)=(S=9275440,E=9275447,C=71)
	CT(33034)=(S=9275448,E=9275455,C=25)
	CT(33035)=(S=9275456,E=9275487,C=66)
	CT(33036)=(S=9275488,E=9275503,C=63)
	CT(33037)=(S=9275504,E=9275511,C=18)
	CT(33038)=(S=9275512,E=9275519,C=36)
	CT(33039)=(S=9275520,E=9275647,C=57)
	CT(33040)=(S=9275648,E=9275903,C=19)
	CT(33041)=(S=9275904,E=9275967,C=57)
	CT(33042)=(S=9275968,E=9275983,C=49)
	CT(33043)=(S=9275984,E=9275991,C=15)
	CT(33044)=(S=9275992,E=9275999,C=12)
	CT(33045)=(S=9276000,E=9276031,C=31)
	CT(33046)=(S=9276032,E=9276039,C=14)
	CT(33047)=(S=9276040,E=9276055,C=36)
	CT(33048)=(S=9276056,E=9276063,C=12)
	CT(33049)=(S=9276064,E=9276071,C=58)
	CT(33050)=(S=9276072,E=9276079,C=36)
	CT(33051)=(S=9276080,E=9276095,C=39)
	CT(33052)=(S=9276096,E=9276111,C=36)
	CT(33053)=(S=9276112,E=9276119,C=14)
	CT(33054)=(S=9276120,E=9276127,C=36)
	CT(33055)=(S=9276128,E=9276159,C=14)
	CT(33056)=(S=9276160,E=9277183,C=29)
	CT(33057)=(S=9277184,E=9278719,C=19)
	CT(33058)=(S=9278720,E=9282303,C=29)
	CT(33059)=(S=9282304,E=9282559,C=14)
	CT(33060)=(S=9282560,E=9283583,C=29)
	CT(33061)=(S=9283584,E=9283839,C=0)
	CT(33062)=(S=9283840,E=9284095,C=28)
	CT(33063)=(S=9284096,E=9284223,C=14)
	CT(33064)=(S=9284224,E=9284287,C=33)
	CT(33065)=(S=9284288,E=9284303,C=59)
	CT(33066)=(S=9284304,E=9284311,C=25)
	CT(33067)=(S=9284312,E=9284319,C=12)
	CT(33068)=(S=9284320,E=9284351,C=31)
	CT(33069)=(S=9284352,E=9284607,C=34)
	CT(33070)=(S=9284608,E=9284863,C=47)
	CT(33071)=(S=9284864,E=9285375,C=29)
	CT(33072)=(S=9285376,E=9285631,C=12)
	CT(33073)=(S=9285632,E=9285887,C=36)
	CT(33074)=(S=9285888,E=9286143,C=29)
	CT(33075)=(S=9286144,E=9286399,C=2)
	CT(33076)=(S=9286400,E=9289727,C=29)
	CT(33077)=(S=9289728,E=9289983,C=47)
	CT(33078)=(S=9289984,E=9290239,C=29)
	CT(33079)=(S=9290240,E=9290495,C=12)
	CT(33080)=(S=9290496,E=9290751,C=29)
	CT(33081)=(S=9290752,E=9291007,C=37)
	CT(33082)=(S=9291008,E=9291775,C=29)
	CT(33083)=(S=9291776,E=9292031,C=28)
	CT(33084)=(S=9292032,E=9292287,C=42)
	CT(33085)=(S=9292288,E=9292543,C=29)
	CT(33086)=(S=9292544,E=9292799,C=42)
	CT(33087)=(S=9292800,E=9293823,C=29)
	CT(33088)=(S=9293824,E=9294079,C=47)
	CT(33089)=(S=9294080,E=9297663,C=29)
	CT(33090)=(S=9297664,E=9297919,C=6)
	CT(33091)=(S=9297920,E=9298431,C=29)
	CT(33092)=(S=9298432,E=9298687,C=17)
	CT(33093)=(S=9298688,E=9298943,C=12)
	CT(33094)=(S=9298944,E=9299199,C=14)
	CT(33095)=(S=9299456,E=9299711,C=29)
	CT(33096)=(S=9299712,E=9299967,C=14)
	CT(33097)=(S=9299968,E=9301247,C=29)
	CT(33098)=(S=9301248,E=9301503,C=22)
	CT(33099)=(S=9301504,E=9302271,C=29)
	CT(33100)=(S=9302272,E=9302527,C=14)
	CT(33101)=(S=9302528,E=9302783,C=29)
	CT(33102)=(S=9302784,E=9303039,C=0)
	CT(33103)=(S=9303040,E=9303295,C=42)
	CT(33104)=(S=9303296,E=9303551,C=14)
	CT(33105)=(S=9303552,E=9304319,C=29)
	CT(33106)=(S=9304320,E=9304575,C=34)
	CT(33107)=(S=9304576,E=9304831,C=15)
	CT(33108)=(S=9304832,E=9305087,C=29)
	CT(33109)=(S=9305088,E=9305343,C=36)
	CT(33110)=(S=9305344,E=9305599,C=49)
	CT(33111)=(S=9305600,E=9305855,C=29)
	CT(33112)=(S=9305856,E=9305983,C=22)
	CT(33113)=(S=9305984,E=9305999,C=12)
	CT(33114)=(S=9306000,E=9306015,C=36)
	CT(33115)=(S=9306016,E=9306023,C=34)
	CT(33116)=(S=9306024,E=9306031,C=42)
	CT(33117)=(S=9306032,E=9306039,C=36)
	CT(33118)=(S=9306040,E=9306047,C=19)
	CT(33119)=(S=9306048,E=9306111,C=52)
	CT(33120)=(S=9306112,E=9306127,C=29)
	CT(33121)=(S=9306128,E=9306143,C=78)
	CT(33122)=(S=9306144,E=9306255,C=29)
	CT(33123)=(S=9306256,E=9306271,C=78)
	CT(33124)=(S=9306272,E=9306335,C=29)
	CT(33125)=(S=9306336,E=9306351,C=175)
	CT(33126)=(S=9306352,E=9306367,C=29)
	CT(33127)=(S=9306368,E=9307135,C=78)
	CT(33128)=(S=9307136,E=9307327,C=29)
	CT(33129)=(S=9307328,E=9307359,C=78)
	CT(33130)=(S=9307360,E=9307391,C=29)
	CT(33131)=(S=9307392,E=9308927,C=78)
	CT(33132)=(S=9308928,E=9309183,C=29)
	CT(33133)=(S=9309184,E=9319935,C=78)
	CT(33134)=(S=9319936,E=9319955,C=29)
	CT(33135)=(S=9319956,E=9319959,C=95)
	CT(33136)=(S=9319960,E=9319983,C=29)
	CT(33137)=(S=9319984,E=9319999,C=78)
	CT(33138)=(S=9320000,E=9320127,C=29)
	CT(33139)=(S=9320128,E=9320135,C=178)
	CT(33140)=(S=9320136,E=9320139,C=29)
	CT(33141)=(S=9320140,E=9320143,C=179)
	CT(33142)=(S=9320144,E=9320191,C=29)
	CT(33143)=(S=9320192,E=9322751,C=78)
	CT(33144)=(S=9322752,E=9323007,C=29)
	CT(33145)=(S=9323008,E=9325823,C=78)
	CT(33146)=(S=9325824,E=9326079,C=29)
	CT(33147)=(S=9326080,E=9326335,C=78)
	CT(33148)=(S=9326336,E=9326591,C=29)
	CT(33149)=(S=9326592,E=9329407,C=78)
	CT(33150)=(S=9329408,E=9329663,C=29)
	CT(33151)=(S=9329664,E=9332991,C=78)
	CT(33152)=(S=9332992,E=9333247,C=29)
	CT(33153)=(S=9333248,E=9334527,C=78)
	CT(33154)=(S=9334528,E=9334783,C=29)
	CT(33155)=(S=9334784,E=9339135,C=78)
	CT(33156)=(S=9339136,E=9339391,C=29)
	CT(33157)=(S=9339392,E=9340927,C=78)
	CT(33158)=(S=9340928,E=9341183,C=29)
	CT(33159)=(S=9341184,E=9343743,C=78)
	CT(33160)=(S=9343744,E=9343758,C=29)
	CT(33161)=(S=9343759,E=9343791,C=78)
	CT(33162)=(S=9343792,E=9343793,C=29)
	CT(33163)=(S=9343794,E=9343798,C=78)
	CT(33164)=(S=9343799,E=9343804,C=29)
	CT(33165)=(S=9343805,E=9343811,C=78)
	CT(33166)=(S=9343812,E=9343817,C=29)
	CT(33167)=(S=9343818,E=9343818,C=78)
	CT(33168)=(S=9343819,E=9343821,C=29)
	CT(33169)=(S=9343822,E=9343822,C=78)
	CT(33170)=(S=9343823,E=9343823,C=29)
	CT(33171)=(S=9343824,E=9343824,C=78)
	CT(33172)=(S=9343825,E=9343825,C=29)
	CT(33173)=(S=9343826,E=9343827,C=78)
	CT(33174)=(S=9343828,E=9343839,C=29)
	CT(33175)=(S=9343840,E=9343844,C=78)
	CT(33176)=(S=9343845,E=9343845,C=29)
	CT(33177)=(S=9343846,E=9343847,C=180)
	CT(33178)=(S=9343848,E=9343999,C=29)
	CT(33179)=(S=9344000,E=9344127,C=78)
	CT(33180)=(S=9344128,E=9344255,C=29)
	CT(33181)=(S=9344256,E=9355263,C=78)
	CT(33182)=(S=9355264,E=9355519,C=29)
	CT(33183)=(S=9355520,E=9356287,C=78)
	CT(33184)=(S=9356288,E=9356799,C=29)
	CT(33185)=(S=9356800,E=9366015,C=78)
	CT(33186)=(S=9366016,E=9366271,C=29)
	CT(33187)=(S=9366272,E=9370111,C=78)
	CT(33188)=(S=9370112,E=9370879,C=29)
	CT(33189)=(S=9370880,E=9371135,C=78)
	CT(33190)=(S=9371136,E=9371647,C=29)
	CT(33191)=(S=9371648,E=9371671,C=189)
	CT(33192)=(S=9371672,E=9371675,C=177)
	CT(33193)=(S=9371676,E=9371723,C=189)
	CT(33194)=(S=9371728,E=9371731,C=83)
	CT(33195)=(S=9371732,E=9371735,C=189)
	CT(33196)=(S=9371740,E=9371743,C=181)
	CT(33197)=(S=9371744,E=9371747,C=200)
	CT(33198)=(S=9371752,E=9371755,C=210)
	CT(33199)=(S=9371756,E=9371759,C=181)
	CT(33200)=(S=9371760,E=9371815,C=189)
	CT(33201)=(S=9371820,E=9371823,C=176)
	CT(33202)=(S=9371824,E=9371883,C=189)
	CT(33203)=(S=9371884,E=9371887,C=208)
	CT(33204)=(S=9371888,E=9371891,C=203)
	CT(33205)=(S=9371892,E=9371895,C=83)
	CT(33206)=(S=9371896,E=9371899,C=212)
	CT(33207)=(S=9371900,E=9371903,C=189)
	CT(33208)=(S=9371904,E=9382143,C=29)
	CT(33209)=(S=9382144,E=9382399,C=14)
	CT(33210)=(S=9382400,E=9383679,C=29)
	CT(33211)=(S=9383680,E=9383935,C=63)
	CT(33212)=(S=9383936,E=9384447,C=29)
	CT(33213)=(S=9384448,E=9384703,C=42)
	CT(33214)=(S=9384704,E=9384959,C=47)
	CT(33215)=(S=9384960,E=9385471,C=14)
	CT(33216)=(S=9385472,E=9385727,C=189)
	CT(33217)=(S=9385728,E=9388287,C=29)
	CT(33218)=(S=9388288,E=9388543,C=14)
	CT(33219)=(S=9388544,E=9394431,C=29)
	CT(33220)=(S=9394432,E=9394687,C=8)
	CT(33221)=(S=9394688,E=9394943,C=2)
	CT(33222)=(S=9394944,E=9395199,C=29)
	CT(33223)=(S=9395200,E=9395455,C=0)
	CT(33224)=(S=9395456,E=9395711,C=28)
	CT(33225)=(S=9395712,E=9395967,C=2)
	CT(33226)=(S=9395968,E=9396223,C=29)
	CT(33227)=(S=9396224,E=9396479,C=73)
	CT(33228)=(S=9396480,E=9396735,C=27)
	CT(33229)=(S=9396736,E=9396991,C=29)
	CT(33230)=(S=9396992,E=9397247,C=34)
	CT(33231)=(S=9397248,E=9398783,C=29)
	CT(33232)=(S=9398784,E=9399551,C=189)
	CT(33233)=(S=9399552,E=9401599,C=29)
	CT(33234)=(S=9401600,E=9401855,C=14)
	CT(33235)=(S=9401856,E=9402111,C=19)
	CT(33236)=(S=9402112,E=9402367,C=0)
	CT(33237)=(S=9402368,E=9402623,C=29)
	CT(33238)=(S=9402624,E=9402879,C=36)
	CT(33239)=(S=9402880,E=9403647,C=29)
	CT(33240)=(S=9403648,E=9403903,C=2)
	CT(33241)=(S=9403904,E=9404159,C=12)
	CT(33242)=(S=9404160,E=9404415,C=29)
	CT(33243)=(S=9404416,E=9404671,C=105)
	CT(33244)=(S=9404672,E=9404927,C=49)
	CT(33245)=(S=9404928,E=9405183,C=42)
	CT(33246)=(S=9405184,E=9406463,C=29)
	CT(33247)=(S=9406464,E=9406719,C=2)
	CT(33248)=(S=9406720,E=9406739,C=189)
	CT(33249)=(S=9406740,E=9406743,C=83)
	CT(33250)=(S=9406744,E=9406747,C=216)
	CT(33251)=(S=9406748,E=9406795,C=189)
	CT(33252)=(S=9406800,E=9406803,C=177)
	CT(33253)=(S=9406804,E=9406811,C=189)
	CT(33254)=(S=9406976,E=9412607,C=29)
	CT(33255)=(S=9412608,E=9412863,C=105)
	CT(33256)=(S=9412864,E=9413119,C=42)
	CT(33257)=(S=9413120,E=9413375,C=29)
	CT(33258)=(S=9413376,E=9413887,C=28)
	CT(33259)=(S=9413888,E=9414399,C=29)
	CT(33260)=(S=9414400,E=9414655,C=14)
	CT(33261)=(S=9414656,E=9414911,C=29)
	CT(33262)=(S=9414912,E=9415167,C=49)
	CT(33263)=(S=9415168,E=9416191,C=29)
	CT(33264)=(S=9416192,E=9416447,C=0)
	CT(33265)=(S=9416448,E=9416703,C=29)
	CT(33266)=(S=9416704,E=9417727,C=36)
	CT(33267)=(S=9417728,E=9417983,C=34)
	CT(33268)=(S=9417984,E=9419775,C=29)
	CT(33269)=(S=9419776,E=9420287,C=0)
	CT(33270)=(S=9420288,E=9420895,C=29)
	CT(33271)=(S=9420896,E=9420911,C=5)
	CT(33272)=(S=9420912,E=9420927,C=7)
	CT(33273)=(S=9420928,E=9421823,C=29)
	CT(33274)=(S=9421824,E=9422079,C=12)
	CT(33275)=(S=9422080,E=9424127,C=29)
	CT(33276)=(S=9424128,E=9424383,C=42)
	CT(33277)=(S=9424384,E=9425407,C=29)
	CT(33278)=(S=9425408,E=9425663,C=14)
	CT(33279)=(S=9425664,E=9426943,C=29)
	CT(33280)=(S=9426944,E=9427199,C=0)
	CT(33281)=(S=9427200,E=9427455,C=19)
	CT(33282)=(S=9427456,E=9427711,C=0)
	CT(33283)=(S=9427712,E=9428223,C=29)
	CT(33284)=(S=9428224,E=9428479,C=2)
	CT(33285)=(S=9428480,E=9428991,C=29)
	CT(33286)=(S=9428992,E=9429247,C=42)
	CT(33287)=(S=9429248,E=9429503,C=15)
	CT(33288)=(S=9429504,E=9431295,C=29)
	CT(33289)=(S=9431296,E=9431551,C=22)
	CT(33290)=(S=9431552,E=9431807,C=14)
	CT(33291)=(S=9431808,E=9432319,C=29)
	CT(33292)=(S=9432320,E=9432575,C=19)
	CT(33293)=(S=9432576,E=9432831,C=0)
	CT(33294)=(S=9432832,E=9433087,C=63)
	CT(33295)=(S=9433088,E=9434367,C=29)
	CT(33296)=(S=9434368,E=9434623,C=42)
	CT(33297)=(S=9434624,E=9435135,C=29)
	CT(33298)=(S=9435136,E=9435391,C=6)
	CT(33299)=(S=9435392,E=9436159,C=29)
	CT(33300)=(S=9436160,E=9436415,C=14)
	CT(33301)=(S=9436416,E=9436671,C=2)
	CT(33302)=(S=9436672,E=9436927,C=29)
	CT(33303)=(S=9437184,E=9437439,C=1)
	CT(33304)=(S=9437696,E=9437951,C=36)
	CT(33305)=(S=9437952,E=9438207,C=29)
	CT(33306)=(S=9438208,E=9438463,C=47)
	CT(33307)=(S=9438464,E=9438719,C=29)
	CT(33308)=(S=9438720,E=9438975,C=0)
	CT(33309)=(S=9438976,E=9439231,C=1)
	CT(33310)=(S=9439232,E=9440255,C=29)
	CT(33311)=(S=9440256,E=9440511,C=1)
	CT(33312)=(S=9440512,E=9441279,C=29)
	CT(33313)=(S=9441280,E=9441535,C=4)
	CT(33314)=(S=9441536,E=9442303,C=29)
	CT(33315)=(S=9442304,E=9442815,C=19)
	CT(33316)=(S=9442816,E=9443327,C=202)
	CT(33317)=(S=9443328,E=9443583,C=22)
	CT(33318)=(S=9443584,E=9444095,C=29)
	CT(33319)=(S=9444096,E=9444351,C=19)
	CT(33320)=(S=9444352,E=9445375,C=29)
	CT(33321)=(S=9445376,E=9445631,C=14)
	CT(33322)=(S=9445632,E=9446469,C=29)
	CT(33323)=(S=9446470,E=9446473,C=4)
	CT(33324)=(S=9446474,E=9446503,C=29)
	CT(33325)=(S=9446504,E=9446504,C=4)
	CT(33326)=(S=9446505,E=9446527,C=29)
	CT(33327)=(S=9446528,E=9446529,C=4)
	CT(33328)=(S=9446530,E=9446533,C=29)
	CT(33329)=(S=9446534,E=9446535,C=4)
	CT(33330)=(S=9446536,E=9446539,C=29)
	CT(33331)=(S=9446540,E=9446541,C=4)
	CT(33332)=(S=9446542,E=9446543,C=29)
	CT(33333)=(S=9446544,E=9446545,C=4)
	CT(33334)=(S=9446546,E=9446547,C=29)
	CT(33335)=(S=9446548,E=9446549,C=4)
	CT(33336)=(S=9446550,E=9446575,C=29)
	CT(33337)=(S=9446576,E=9446576,C=4)
	CT(33338)=(S=9446577,E=9446577,C=29)
	CT(33339)=(S=9446578,E=9446578,C=4)
	CT(33340)=(S=9446579,E=9446581,C=29)
	CT(33341)=(S=9446582,E=9446585,C=4)
	CT(33342)=(S=9446586,E=9446587,C=29)
	CT(33343)=(S=9446588,E=9446589,C=4)
	CT(33344)=(S=9446590,E=9446605,C=29)
	CT(33345)=(S=9446606,E=9446607,C=4)
	CT(33346)=(S=9446608,E=9446609,C=29)
	CT(33347)=(S=9446610,E=9446611,C=4)
	CT(33348)=(S=9446612,E=9446613,C=29)
	CT(33349)=(S=9446614,E=9446617,C=4)
	CT(33350)=(S=9446618,E=9446619,C=29)
	CT(33351)=(S=9446620,E=9446636,C=4)
	CT(33352)=(S=9446637,E=9446643,C=29)
	CT(33353)=(S=9446644,E=9446645,C=4)
	CT(33354)=(S=9446646,E=9446647,C=29)
	CT(33355)=(S=9446648,E=9446651,C=4)
	CT(33356)=(S=9446652,E=9446653,C=29)
	CT(33357)=(S=9446654,E=9446655,C=4)
	CT(33358)=(S=9446656,E=9447679,C=29)
	CT(33359)=(S=9447680,E=9447935,C=28)
	CT(33360)=(S=9447936,E=9448191,C=29)
	CT(33361)=(S=9448192,E=9448447,C=36)
	CT(33362)=(S=9448448,E=9448703,C=13)
	CT(33363)=(S=9448704,E=9450495,C=29)
	CT(33364)=(S=9450496,E=9450751,C=1)
	CT(33365)=(S=9450752,E=9451007,C=0)
	CT(33366)=(S=9451008,E=9451263,C=36)
	CT(33367)=(S=9451264,E=9451519,C=0)
	CT(33368)=(S=9451520,E=9451775,C=12)
	CT(33369)=(S=9451776,E=9452031,C=19)
	CT(33370)=(S=9452032,E=9453311,C=29)
	CT(33371)=(S=9453312,E=9453567,C=19)
	CT(33372)=(S=9453568,E=9453823,C=21)
	CT(33373)=(S=9453824,E=9454079,C=42)
	CT(33374)=(S=9454080,E=9454335,C=73)
	CT(33375)=(S=9454336,E=9456639,C=29)
	CT(33376)=(S=9456640,E=9456895,C=28)
	CT(33377)=(S=9456896,E=9458175,C=29)
	CT(33378)=(S=9458176,E=9458431,C=14)
	CT(33379)=(S=9458432,E=9458687,C=29)
	CT(33380)=(S=9458688,E=9458943,C=27)
	CT(33381)=(S=9458944,E=9459199,C=34)
	CT(33382)=(S=9459200,E=9459455,C=29)
	CT(33383)=(S=9459456,E=9459711,C=14)
	CT(33384)=(S=9459712,E=9461503,C=29)
	CT(33385)=(S=9461504,E=9461759,C=36)
	CT(33386)=(S=9461760,E=9462015,C=29)
	CT(33387)=(S=9462016,E=9462271,C=0)
	CT(33388)=(S=9462272,E=9462527,C=14)
	CT(33389)=(S=9462528,E=9465343,C=29)
	CT(33390)=(S=9465344,E=9465599,C=0)
	CT(33391)=(S=9465600,E=9467647,C=29)
	CT(33392)=(S=9467648,E=9467903,C=36)
	CT(33393)=(S=9467904,E=9468159,C=91)
	CT(33394)=(S=9468160,E=9468415,C=29)
	CT(33395)=(S=9468416,E=9468671,C=37)
	CT(33396)=(S=9468672,E=9468927,C=1)
	CT(33397)=(S=9468928,E=9469183,C=14)
	CT(33398)=(S=9469184,E=9469695,C=29)
	CT(33399)=(S=9469696,E=9469951,C=27)
	CT(33400)=(S=9469952,E=9470463,C=29)
	CT(33401)=(S=9470464,E=9473279,C=0)
	CT(33402)=(S=9473280,E=9474303,C=29)
	CT(33403)=(S=9474304,E=9474559,C=28)
	CT(33404)=(S=9474560,E=9479167,C=29)
	CT(33405)=(S=9479168,E=9479423,C=27)
	CT(33406)=(S=9479424,E=9479679,C=12)
	CT(33407)=(S=9479680,E=9480243,C=29)
	CT(33408)=(S=9480244,E=9480255,C=78)
	CT(33409)=(S=9480256,E=9481343,C=29)
	CT(33410)=(S=9481344,E=9481471,C=78)
	CT(33411)=(S=9481472,E=9481727,C=14)
	CT(33412)=(S=9481728,E=9482239,C=29)
	CT(33413)=(S=9482240,E=9482751,C=27)
	CT(33414)=(S=9482752,E=9483007,C=14)
	CT(33415)=(S=9483008,E=9483775,C=27)
	CT(33416)=(S=9483776,E=9486591,C=29)
	CT(33417)=(S=9486592,E=9486847,C=27)
	CT(33418)=(S=9486848,E=9487103,C=78)
	CT(33419)=(S=9487104,E=9488127,C=29)
	CT(33420)=(S=9488128,E=9488383,C=5)
	CT(33421)=(S=9488384,E=9488639,C=34)
	CT(33422)=(S=9488640,E=9489407,C=29)
	CT(33423)=(S=9489408,E=9489663,C=12)
	CT(33424)=(S=9489664,E=9489919,C=0)
	CT(33425)=(S=9489920,E=9490175,C=18)
	CT(33426)=(S=9490176,E=9490687,C=29)
	CT(33427)=(S=9490688,E=9490943,C=36)
	CT(33428)=(S=9490944,E=9491711,C=29)
	CT(33429)=(S=9491712,E=9491967,C=2)
	CT(33430)=(S=9491968,E=9492223,C=8)
	CT(33431)=(S=9492224,E=9500671,C=29)
	CT(33432)=(S=9500672,E=9500927,C=49)
	CT(33433)=(S=9500928,E=9502463,C=29)
	CT(33434)=(S=9502464,E=9502719,C=1)
	CT(33435)=(S=9502720,E=9516799,C=36)
	CT(33436)=(S=9516800,E=9517055,C=14)
	CT(33437)=(S=9517056,E=9558527,C=36)
	CT(33438)=(S=9558528,E=9558783,C=13)
	CT(33439)=(S=9558784,E=9560063,C=36)
	CT(33440)=(S=9560064,E=9560319,C=14)
	CT(33441)=(S=9560320,E=9560575,C=28)
	CT(33442)=(S=9560576,E=9560831,C=12)
	CT(33443)=(S=9560832,E=9561087,C=14)
	CT(33444)=(S=9561088,E=9561343,C=28)
	CT(33445)=(S=9561344,E=9561599,C=14)
	CT(33446)=(S=9561600,E=9561855,C=28)
	CT(33447)=(S=9561856,E=9562111,C=12)
	CT(33448)=(S=9562112,E=9562367,C=34)
	CT(33449)=(S=9562368,E=9562623,C=14)
	CT(33450)=(S=9562624,E=9562879,C=34)
	CT(33451)=(S=9562880,E=9563135,C=19)
	CT(33452)=(S=9563136,E=9563391,C=50)
	CT(33453)=(S=9563392,E=9563647,C=58)
	CT(33454)=(S=9563648,E=9563903,C=12)
	CT(33455)=(S=9563904,E=9564159,C=14)
	CT(33456)=(S=9564160,E=9564415,C=12)
	CT(33457)=(S=9564416,E=9564671,C=42)
	CT(33458)=(S=9564672,E=9564927,C=12)
	CT(33459)=(S=9564928,E=9565183,C=28)
	CT(33460)=(S=9565184,E=9565439,C=42)
	CT(33461)=(S=9565440,E=9565695,C=34)
	CT(33462)=(S=9565696,E=9565951,C=14)
	CT(33463)=(S=9565952,E=9566207,C=47)
	CT(33464)=(S=9566208,E=9566463,C=12)
	CT(33465)=(S=9566464,E=9566719,C=18)
	CT(33466)=(S=9566720,E=9566975,C=34)
	CT(33467)=(S=9566976,E=9567231,C=19)
	CT(33468)=(S=9567232,E=9567487,C=34)
	CT(33469)=(S=9567488,E=9567999,C=28)
	CT(33470)=(S=9568000,E=9568047,C=18)
	CT(33471)=(S=9568048,E=9568055,C=28)
	CT(33472)=(S=9568056,E=9568063,C=24)
	CT(33473)=(S=9568064,E=9568127,C=45)
	CT(33474)=(S=9568128,E=9568135,C=36)
	CT(33475)=(S=9568136,E=9568143,C=18)
	CT(33476)=(S=9568144,E=9568159,C=71)
	CT(33477)=(S=9568160,E=9568191,C=20)
	CT(33478)=(S=9568192,E=9568223,C=51)
	CT(33479)=(S=9568224,E=9568231,C=14)
	CT(33480)=(S=9568232,E=9568239,C=18)
	CT(33481)=(S=9568240,E=9568247,C=14)
	CT(33482)=(S=9568248,E=9568255,C=18)
	CT(33483)=(S=9568256,E=9568271,C=28)
	CT(33484)=(S=9568272,E=9568279,C=96)
	CT(33485)=(S=9568280,E=9568287,C=12)
	CT(33486)=(S=9568288,E=9568303,C=28)
	CT(33487)=(S=9568304,E=9568311,C=63)
	CT(33488)=(S=9568312,E=9568313,C=14)
	CT(33489)=(S=9568314,E=9568319,C=38)
	CT(33490)=(S=9568320,E=9568327,C=12)
	CT(33491)=(S=9568328,E=9568335,C=36)
	CT(33492)=(S=9568336,E=9568343,C=31)
	CT(33493)=(S=9568344,E=9568351,C=12)
	CT(33494)=(S=9568352,E=9568383,C=28)
	CT(33495)=(S=9568384,E=9568479,C=38)
	CT(33496)=(S=9568480,E=9568511,C=28)
	CT(33497)=(S=9568512,E=9568767,C=29)
	CT(33498)=(S=9568768,E=9569023,C=27)
	CT(33499)=(S=9569024,E=9569279,C=70)
	CT(33500)=(S=9569280,E=9569535,C=34)
	CT(33501)=(S=9569536,E=9571071,C=29)
	CT(33502)=(S=9571072,E=9571327,C=0)
	CT(33503)=(S=9571328,E=9573119,C=29)
	CT(33504)=(S=9573120,E=9573375,C=12)
	CT(33505)=(S=9573376,E=9573631,C=29)
	CT(33506)=(S=9573632,E=9573887,C=19)
	CT(33507)=(S=9573888,E=9580543,C=29)
	CT(33508)=(S=9580544,E=9580799,C=15)
	CT(33509)=(S=9580800,E=9581055,C=29)
	CT(33510)=(S=9581056,E=9581311,C=36)
	CT(33511)=(S=9581312,E=9581567,C=2)
	CT(33512)=(S=9581568,E=9581823,C=28)
	CT(33513)=(S=9581824,E=9582591,C=29)
	CT(33514)=(S=9582592,E=9582847,C=2)
	CT(33515)=(S=9582848,E=9583103,C=29)
	CT(33516)=(S=9583360,E=9583615,C=27)
	CT(33517)=(S=9583616,E=9583871,C=28)
	CT(33518)=(S=9583872,E=9584639,C=29)
	CT(33519)=(S=9584640,E=9584895,C=105)
	CT(33520)=(S=9584896,E=9585151,C=29)
	CT(33521)=(S=9585152,E=9585159,C=14)
	CT(33522)=(S=9585160,E=9585167,C=18)
	CT(33523)=(S=9585168,E=9585175,C=20)
	CT(33524)=(S=9585176,E=9585183,C=47)
	CT(33525)=(S=9585184,E=9585215,C=14)
	CT(33526)=(S=9585216,E=9585279,C=51)
	CT(33527)=(S=9585280,E=9585287,C=26)
	CT(33528)=(S=9585288,E=9585295,C=25)
	CT(33529)=(S=9585296,E=9585303,C=28)
	CT(33530)=(S=9585304,E=9585311,C=13)
	CT(33531)=(S=9585312,E=9585359,C=18)
	CT(33532)=(S=9585360,E=9585375,C=38)
	CT(33533)=(S=9585376,E=9585383,C=28)
	CT(33534)=(S=9585384,E=9585391,C=19)
	CT(33535)=(S=9585392,E=9585407,C=25)
	CT(33536)=(S=9585408,E=9585663,C=34)
	CT(33537)=(S=9585664,E=9586175,C=29)
	CT(33538)=(S=9586176,E=9586431,C=14)
	CT(33539)=(S=9586432,E=9586687,C=29)
	CT(33540)=(S=9586688,E=9586943,C=27)
	CT(33541)=(S=9586944,E=9587455,C=29)
	CT(33542)=(S=9587456,E=9587711,C=19)
	CT(33543)=(S=9587712,E=9587967,C=29)
	CT(33544)=(S=9587968,E=9588223,C=14)
	CT(33545)=(S=9588224,E=9588735,C=29)
	CT(33546)=(S=9588736,E=9588991,C=14)
	CT(33547)=(S=9588992,E=9589247,C=47)
	CT(33548)=(S=9589248,E=9589503,C=29)
	CT(33549)=(S=9589504,E=9589759,C=203)
	CT(33550)=(S=9589760,E=9590527,C=29)
	CT(33551)=(S=9590528,E=9590783,C=14)
	CT(33552)=(S=9590784,E=9590799,C=29)
	CT(33553)=(S=9590800,E=9590802,C=0)
	CT(33554)=(S=9590803,E=9590803,C=7)
	CT(33555)=(S=9590804,E=9590807,C=0)
	CT(33556)=(S=9590808,E=9590811,C=4)
	CT(33557)=(S=9590812,E=9590812,C=0)
	CT(33558)=(S=9590813,E=9590815,C=29)
	CT(33559)=(S=9590816,E=9590847,C=3)
	CT(33560)=(S=9590848,E=9590863,C=10)
	CT(33561)=(S=9590864,E=9590867,C=5)
	CT(33562)=(S=9590868,E=9590871,C=6)
	CT(33563)=(S=9590872,E=9590879,C=10)
	CT(33564)=(S=9590880,E=9590911,C=29)
	CT(33565)=(S=9590912,E=9590919,C=0)
	CT(33566)=(S=9590920,E=9590927,C=29)
	CT(33567)=(S=9590928,E=9590929,C=0)
	CT(33568)=(S=9590930,E=9590931,C=73)
	CT(33569)=(S=9590932,E=9590935,C=29)
	CT(33570)=(S=9590936,E=9590943,C=5)
	CT(33571)=(S=9590944,E=9590959,C=4)
	CT(33572)=(S=9590960,E=9591295,C=29)
	CT(33573)=(S=9591296,E=9591551,C=14)
	CT(33574)=(S=9591552,E=9593087,C=29)
	CT(33575)=(S=9593088,E=9593343,C=14)
	CT(33576)=(S=9593344,E=9593599,C=29)
	CT(33577)=(S=9593600,E=9593855,C=2)
	CT(33578)=(S=9593856,E=9594111,C=15)
	CT(33579)=(S=9594112,E=9594367,C=14)
	CT(33580)=(S=9594368,E=9594623,C=33)
	CT(33581)=(S=9594624,E=9594879,C=49)
	CT(33582)=(S=9594880,E=9595135,C=36)
	CT(33583)=(S=9595136,E=9595391,C=14)
	CT(33584)=(S=9595392,E=9595647,C=13)
	CT(33585)=(S=9595648,E=9595903,C=28)
	CT(33586)=(S=9595904,E=9596159,C=42)
	CT(33587)=(S=9596160,E=9596415,C=34)
	CT(33588)=(S=9596416,E=9596671,C=50)
	CT(33589)=(S=9596672,E=9596927,C=29)
	CT(33590)=(S=9596928,E=9597183,C=42)
	CT(33591)=(S=9597184,E=9597951,C=29)
	CT(33592)=(S=9597952,E=9598719,C=0)
	CT(33593)=(S=9598720,E=9598975,C=47)
	CT(33594)=(S=9598976,E=9599231,C=33)
	CT(33595)=(S=9599232,E=9599999,C=29)
	CT(33596)=(S=9600000,E=9600255,C=22)
	CT(33597)=(S=9600256,E=9601791,C=29)
	CT(33598)=(S=9601792,E=9602047,C=78)
	CT(33599)=(S=9602048,E=9602303,C=29)
	CT(33600)=(S=9602304,E=9602559,C=15)
	CT(33601)=(S=9602560,E=9602815,C=189)
	CT(33602)=(S=9602816,E=9603071,C=29)
	CT(33603)=(S=9603072,E=9603327,C=34)
	CT(33604)=(S=9603328,E=9604095,C=29)
	CT(33605)=(S=9604096,E=9604351,C=28)
	CT(33606)=(S=9604352,E=9604607,C=105)
	CT(33607)=(S=9604608,E=9607935,C=29)
	CT(33608)=(S=9607936,E=9608191,C=203)
	CT(33609)=(S=9608192,E=9608703,C=29)
	CT(33610)=(S=9608704,E=9608831,C=33)
	CT(33611)=(S=9608832,E=9608959,C=25)
	CT(33612)=(S=9608960,E=9609215,C=34)
	CT(33613)=(S=9609216,E=9609471,C=2)
	CT(33614)=(S=9609472,E=9609727,C=47)
	CT(33615)=(S=9609728,E=9609983,C=14)
	CT(33616)=(S=9609984,E=9610239,C=29)
	CT(33617)=(S=9610240,E=9610495,C=189)
	CT(33618)=(S=9610496,E=9611519,C=29)
	CT(33619)=(S=9611520,E=9611775,C=14)
	CT(33620)=(S=9611776,E=9612031,C=29)
	CT(33621)=(S=9612032,E=9612287,C=73)
	CT(33622)=(S=9612288,E=9612543,C=27)
	CT(33623)=(S=9612544,E=9612799,C=19)
	CT(33624)=(S=9612800,E=9613055,C=29)
	CT(33625)=(S=9613056,E=9613311,C=49)
	CT(33626)=(S=9613312,E=9613823,C=14)
	CT(33627)=(S=9613824,E=9614079,C=0)
	CT(33628)=(S=9614080,E=9614335,C=14)
	CT(33629)=(S=9614336,E=9614847,C=29)
	CT(33630)=(S=9614848,E=9615103,C=105)
	CT(33631)=(S=9615104,E=9615615,C=29)
	CT(33632)=(S=9615616,E=9615623,C=34)
	CT(33633)=(S=9615624,E=9615631,C=19)
	CT(33634)=(S=9615632,E=9615647,C=14)
	CT(33635)=(S=9615648,E=9615655,C=39)
	CT(33636)=(S=9615656,E=9615663,C=12)
	CT(33637)=(S=9615664,E=9615671,C=49)
	CT(33638)=(S=9615672,E=9615679,C=17)
	CT(33639)=(S=9615680,E=9615743,C=37)
	CT(33640)=(S=9615744,E=9615807,C=36)
	CT(33641)=(S=9615808,E=9615871,C=18)
	CT(33642)=(S=9615872,E=9616383,C=29)
	CT(33643)=(S=9616384,E=9616639,C=14)
	CT(33644)=(S=9616640,E=9617151,C=29)
	CT(33645)=(S=9617152,E=9617407,C=14)
	CT(33646)=(S=9617408,E=9617663,C=27)
	CT(33647)=(S=9617664,E=9617919,C=21)
	CT(33648)=(S=9617920,E=9618175,C=14)
	CT(33649)=(S=9618176,E=9618463,C=0)
	CT(33650)=(S=9618688,E=9622015,C=29)
	CT(33651)=(S=9622016,E=9622271,C=42)
	CT(33652)=(S=9622272,E=9622527,C=47)
	CT(33653)=(S=9622528,E=9622783,C=52)
	CT(33654)=(S=9622784,E=9623039,C=27)
	CT(33655)=(S=9623040,E=9623551,C=29)
	CT(33656)=(S=9623552,E=9623807,C=34)
	CT(33657)=(S=9623808,E=9624319,C=29)
	CT(33658)=(S=9624320,E=9624575,C=25)
	CT(33659)=(S=9624576,E=9624831,C=70)
	CT(33660)=(S=9624832,E=9625087,C=0)
	CT(33661)=(S=9625088,E=9625599,C=29)
	CT(33662)=(S=9625600,E=9625855,C=27)
	CT(33663)=(S=9625856,E=9626111,C=29)
	CT(33664)=(S=9626112,E=9626367,C=180)
	CT(33665)=(S=9626368,E=9626623,C=14)
	CT(33666)=(S=9626624,E=9626879,C=34)
	CT(33667)=(S=9626880,E=9627135,C=29)
	CT(33668)=(S=9627136,E=9627903,C=105)
	CT(33669)=(S=9627904,E=9628159,C=29)
	CT(33670)=(S=9628160,E=9628415,C=28)
	CT(33671)=(S=9628416,E=9631487,C=29)
	CT(33672)=(S=9631488,E=9631495,C=18)
	CT(33673)=(S=9631496,E=9631503,C=34)
	CT(33674)=(S=9631504,E=9631511,C=15)
	CT(33675)=(S=9631512,E=9631519,C=52)
	CT(33676)=(S=9631520,E=9631527,C=18)
	CT(33677)=(S=9631528,E=9631535,C=63)
	CT(33678)=(S=9631536,E=9631551,C=14)
	CT(33679)=(S=9631552,E=9631559,C=15)
	CT(33680)=(S=9631560,E=9631567,C=36)
	CT(33681)=(S=9631568,E=9631575,C=19)
	CT(33682)=(S=9631576,E=9631583,C=12)
	CT(33683)=(S=9631584,E=9631615,C=42)
	CT(33684)=(S=9631616,E=9631743,C=27)
	CT(33685)=(S=9631744,E=9631999,C=34)
	CT(33686)=(S=9632000,E=9632255,C=12)
	CT(33687)=(S=9632256,E=9632511,C=29)
	CT(33688)=(S=9632512,E=9632767,C=23)
	CT(33689)=(S=9632768,E=9633023,C=29)
	CT(33690)=(S=9633024,E=9633279,C=19)
	CT(33691)=(S=9633280,E=9633535,C=28)
	CT(33692)=(S=9633536,E=9633551,C=14)
	CT(33693)=(S=9633552,E=9633559,C=25)
	CT(33694)=(S=9633560,E=9633567,C=33)
	CT(33695)=(S=9633568,E=9633591,C=36)
	CT(33696)=(S=9633592,E=9633599,C=42)
	CT(33697)=(S=9633600,E=9633631,C=54)
	CT(33698)=(S=9633632,E=9633639,C=25)
	CT(33699)=(S=9633640,E=9633647,C=14)
	CT(33700)=(S=9633656,E=9633663,C=28)
	CT(33701)=(S=9633664,E=9633695,C=59)
	CT(33702)=(S=9633696,E=9633703,C=15)
	CT(33703)=(S=9633704,E=9633711,C=12)
	CT(33704)=(S=9633712,E=9633719,C=55)
	CT(33705)=(S=9633720,E=9633727,C=25)
	CT(33706)=(S=9633728,E=9633759,C=18)
	CT(33707)=(S=9633760,E=9633791,C=40)
	CT(33708)=(S=9633792,E=9635327,C=29)
	CT(33709)=(S=9635328,E=9635583,C=6)
	CT(33710)=(S=9635584,E=9635839,C=50)
	CT(33711)=(S=9635840,E=9636095,C=8)
	CT(33712)=(S=9636096,E=9636351,C=29)
	CT(33713)=(S=9636352,E=9636607,C=0)
	CT(33714)=(S=9636608,E=9636863,C=29)
	CT(33715)=(S=9636864,E=9637119,C=36)
	CT(33716)=(S=9637120,E=9637631,C=19)
	CT(33717)=(S=9637632,E=9640703,C=29)
	CT(33718)=(S=9640704,E=9640959,C=22)
	CT(33719)=(S=9640960,E=9641215,C=29)
	CT(33720)=(S=9641216,E=9641471,C=24)
	CT(33721)=(S=9641472,E=9641727,C=20)
	CT(33722)=(S=9641728,E=9641983,C=29)
	CT(33723)=(S=9641984,E=9642495,C=33)
	CT(33724)=(S=9642496,E=9644287,C=29)
	CT(33725)=(S=9644288,E=9644543,C=0)
	CT(33726)=(S=9644544,E=9644799,C=29)
	CT(33727)=(S=9644800,E=9645055,C=6)
	CT(33728)=(S=9645056,E=9645311,C=19)
	CT(33729)=(S=9645312,E=9645567,C=18)
	CT(33730)=(S=9645568,E=9646079,C=6)
	CT(33731)=(S=9646080,E=9647103,C=29)
	CT(33732)=(S=9647104,E=9647359,C=22)
	CT(33733)=(S=9647360,E=9647615,C=29)
	CT(33734)=(S=9647616,E=9647871,C=28)
	CT(33735)=(S=9647872,E=9649151,C=29)
	CT(33736)=(S=9649152,E=9649407,C=34)
	CT(33737)=(S=9649408,E=9650431,C=29)
	CT(33738)=(S=9650432,E=9650687,C=189)
	CT(33739)=(S=9650688,E=9650943,C=0)
	CT(33740)=(S=9650944,E=9651199,C=70)
	CT(33741)=(S=9651200,E=9651455,C=14)
	CT(33742)=(S=9651456,E=9651711,C=0)
	CT(33743)=(S=9651712,E=9652991,C=29)
	CT(33744)=(S=9652992,E=9653247,C=34)
	CT(33745)=(S=9653248,E=9653503,C=0)
	CT(33746)=(S=9653504,E=9653759,C=14)
	CT(33747)=(S=9654016,E=9655039,C=29)
	CT(33748)=(S=9655040,E=9655551,C=25)
	CT(33749)=(S=9655552,E=9655807,C=29)
	CT(33750)=(S=9655808,E=9656575,C=34)
	CT(33751)=(S=9656576,E=9656831,C=14)
	CT(33752)=(S=9656832,E=9657087,C=29)
	CT(33753)=(S=9657088,E=9657343,C=46)
	CT(33754)=(S=9657344,E=9657599,C=29)
	CT(33755)=(S=9657600,E=9657855,C=49)
	CT(33756)=(S=9657856,E=9658111,C=12)
	CT(33757)=(S=9658112,E=9658367,C=22)
	CT(33758)=(S=9658368,E=9658623,C=25)
	CT(33759)=(S=9658624,E=9658879,C=29)
	CT(33760)=(S=9658880,E=9659647,C=12)
	CT(33761)=(S=9659648,E=9659903,C=29)
	CT(33762)=(S=9659904,E=9660159,C=22)
	CT(33763)=(S=9660160,E=9661695,C=29)
	CT(33764)=(S=9661696,E=9661951,C=0)
	CT(33765)=(S=9661952,E=9662207,C=105)
	CT(33766)=(S=9662208,E=9662719,C=27)
	CT(33767)=(S=9662720,E=9662975,C=29)
	CT(33768)=(S=9662976,E=9663231,C=14)
	CT(33769)=(S=9663232,E=9664255,C=29)
	CT(33770)=(S=9664256,E=9664511,C=14)
	CT(33771)=(S=9664512,E=9665023,C=29)
	CT(33772)=(S=9665024,E=9665535,C=15)
	CT(33773)=(S=9665536,E=9665791,C=29)
	CT(33774)=(S=9665792,E=9666047,C=42)
	CT(33775)=(S=9666048,E=9666303,C=29)
	CT(33776)=(S=9666304,E=9666559,C=12)
	CT(33777)=(S=9666560,E=9667583,C=29)
	CT(33778)=(S=9667584,E=9667839,C=0)
	CT(33779)=(S=9667840,E=9670143,C=29)
	CT(33780)=(S=9670144,E=9670399,C=28)
	CT(33781)=(S=9670400,E=9670655,C=14)
	CT(33782)=(S=9670656,E=9671423,C=29)
	CT(33783)=(S=9671424,E=9672959,C=14)
	CT(33784)=(S=9672960,E=9673727,C=29)
	CT(33785)=(S=9673728,E=9673983,C=25)
	CT(33786)=(S=9673984,E=9674239,C=2)
	CT(33787)=(S=9674240,E=9674495,C=73)
	CT(33788)=(S=9674496,E=9675007,C=29)
	CT(33789)=(S=9675008,E=9675263,C=17)
	CT(33790)=(S=9675264,E=9675775,C=15)
	CT(33791)=(S=9675776,E=9676543,C=29)
	CT(33792)=(S=9676544,E=9676799,C=34)
	CT(33793)=(S=9676800,E=9677567,C=29)
	CT(33794)=(S=9677568,E=9677823,C=12)
	CT(33795)=(S=9677824,E=9678079,C=28)
	CT(33796)=(S=9678080,E=9678335,C=12)
	CT(33797)=(S=9678336,E=9678591,C=29)
	CT(33798)=(S=9678592,E=9678847,C=71)
	CT(33799)=(S=9678848,E=9679103,C=2)
	CT(33800)=(S=9679104,E=9679871,C=29)
	CT(33801)=(S=9679872,E=9680127,C=19)
	CT(33802)=(S=9680128,E=9680383,C=36)
	CT(33803)=(S=9680384,E=9680895,C=29)
	CT(33804)=(S=9680896,E=9681151,C=14)
	CT(33805)=(S=9681152,E=9681407,C=29)
	CT(33806)=(S=9681408,E=9681663,C=19)
	CT(33807)=(S=9681664,E=9681919,C=29)
	CT(33808)=(S=9681920,E=9682431,C=14)
	CT(33809)=(S=9682432,E=9683199,C=29)
	CT(33810)=(S=9683200,E=9683455,C=14)
	CT(33811)=(S=9683456,E=9683711,C=78)
	CT(33812)=(S=9683712,E=9683967,C=29)
	CT(33813)=(S=9683968,E=9684223,C=12)
	CT(33814)=(S=9684224,E=9684479,C=14)
	CT(33815)=(S=9684480,E=9684991,C=29)
	CT(33816)=(S=9684992,E=9685247,C=0)
	CT(33817)=(S=9685248,E=9685503,C=14)
	CT(33818)=(S=9685504,E=9686015,C=29)
	CT(33819)=(S=9686016,E=9686271,C=28)
	CT(33820)=(S=9686272,E=9687295,C=29)
	CT(33821)=(S=9687296,E=9687551,C=0)
	CT(33822)=(S=9687552,E=9687807,C=12)
	CT(33823)=(S=9687808,E=9688063,C=0)
	CT(33824)=(S=9688320,E=9688575,C=71)
	CT(33825)=(S=9688576,E=9688831,C=19)
	CT(33826)=(S=9688832,E=9689087,C=12)
	CT(33827)=(S=9689088,E=9690111,C=29)
	CT(33828)=(S=9690112,E=9690367,C=19)
	CT(33829)=(S=9690368,E=9692159,C=29)
	CT(33830)=(S=9692160,E=9693183,C=33)
	CT(33831)=(S=9693184,E=9693439,C=71)
	CT(33832)=(S=9693440,E=9694719,C=17)
	CT(33833)=(S=9694720,E=9695999,C=29)
	CT(33834)=(S=9696000,E=9696255,C=47)
	CT(33835)=(S=9696256,E=9697791,C=29)
	CT(33836)=(S=9697792,E=9698047,C=12)
	CT(33837)=(S=9698048,E=9698303,C=33)
	CT(33838)=(S=9698304,E=9698559,C=63)
	CT(33839)=(S=9698560,E=9699327,C=29)
	CT(33840)=(S=9699328,E=9699583,C=177)
	CT(33841)=(S=9699584,E=9699839,C=29)
	CT(33842)=(S=9699840,E=9700095,C=19)
	CT(33843)=(S=9700096,E=9700351,C=14)
	CT(33844)=(S=9700352,E=9700863,C=29)
	CT(33845)=(S=9700864,E=9701119,C=50)
	CT(33846)=(S=9701120,E=9713151,C=29)
	CT(33847)=(S=9713152,E=9713407,C=34)
	CT(33848)=(S=9713408,E=9713663,C=29)
	CT(33849)=(S=9713664,E=9713919,C=42)
	CT(33850)=(S=9713920,E=9714687,C=29)
	CT(33851)=(S=9714688,E=9714943,C=12)
	CT(33852)=(S=9714944,E=9716287,C=29)
	CT(33853)=(S=9716288,E=9716351,C=164)
	CT(33854)=(S=9716352,E=9716479,C=29)
	CT(33855)=(S=9716480,E=9716991,C=2)
	CT(33856)=(S=9716992,E=9719551,C=29)
	CT(33857)=(S=9719552,E=9719807,C=14)
	CT(33858)=(S=9719808,E=9720063,C=29)
	CT(33859)=(S=9720064,E=9720319,C=58)
	CT(33860)=(S=9720320,E=9720831,C=27)
	CT(33861)=(S=9720832,E=9721855,C=29)
	CT(33862)=(S=9721856,E=9722111,C=14)
	CT(33863)=(S=9722112,E=9725183,C=29)
	CT(33864)=(S=9725184,E=9725439,C=177)
	CT(33865)=(S=9725440,E=9725567,C=212)
	CT(33866)=(S=9725568,E=9725695,C=218)
	CT(33867)=(S=9725696,E=9725951,C=177)
	CT(33868)=(S=9725952,E=9727487,C=29)
	CT(33869)=(S=9727488,E=9727743,C=70)
	CT(33870)=(S=9727744,E=9729279,C=29)
	CT(33871)=(S=9729280,E=9729535,C=78)
	CT(33872)=(S=9729536,E=9731071,C=27)
	CT(33873)=(S=9731072,E=9733887,C=29)
	CT(33874)=(S=9733888,E=9734399,C=19)
	CT(33875)=(S=9734400,E=9734655,C=29)
	CT(33876)=(S=9734656,E=9734911,C=19)
	CT(33877)=(S=9734912,E=9735167,C=29)
	CT(33878)=(S=9735168,E=9735423,C=27)
	CT(33879)=(S=9735424,E=9735935,C=29)
	CT(33880)=(S=9735936,E=9736191,C=12)
	CT(33881)=(S=9736192,E=9736447,C=29)
	CT(33882)=(S=9736448,E=9736703,C=0)
	CT(33883)=(S=9736704,E=9737215,C=29)
	CT(33884)=(S=9737216,E=9737471,C=36)
	CT(33885)=(S=9737472,E=9737983,C=29)
	CT(33886)=(S=9737984,E=9738239,C=45)
	CT(33887)=(S=9738240,E=9740287,C=29)
	CT(33888)=(S=9740288,E=9740543,C=19)
	CT(33889)=(S=9740544,E=9740799,C=2)
	CT(33890)=(S=9740800,E=9741235,C=29)
	CT(33891)=(S=9741236,E=9741239,C=78)
	CT(33892)=(S=9741240,E=9742591,C=29)
	CT(33893)=(S=9742592,E=9742847,C=12)
	CT(33894)=(S=9742848,E=9744383,C=29)
	CT(33895)=(S=9744384,E=9744639,C=14)
	CT(33896)=(S=9744640,E=9745663,C=29)
	CT(33897)=(S=9745664,E=9745919,C=14)
	CT(33898)=(S=9745920,E=9746175,C=0)
	CT(33899)=(S=9746176,E=9746687,C=29)
	CT(33900)=(S=9746688,E=9746943,C=13)
	CT(33901)=(S=9746944,E=9747199,C=29)
	CT(33902)=(S=9747200,E=9747455,C=34)
	CT(33903)=(S=9747456,E=9749247,C=29)
	CT(33904)=(S=9749248,E=9749503,C=0)
	CT(33905)=(S=9749504,E=9749759,C=34)
	CT(33906)=(S=9749760,E=9750015,C=14)
	CT(33907)=(S=9750016,E=9750271,C=42)
	CT(33908)=(S=9750272,E=9750527,C=29)
	CT(33909)=(S=9750528,E=9750783,C=36)
	CT(33910)=(S=9750784,E=9763583,C=204)
	CT(33911)=(S=9763584,E=9763839,C=28)
	CT(33912)=(S=9763840,E=9764351,C=14)
	CT(33913)=(S=9764352,E=9764607,C=29)
	CT(33914)=(S=9764608,E=9764863,C=177)
	CT(33915)=(S=9764864,E=9765119,C=37)
	CT(33916)=(S=9765120,E=9765631,C=29)
	CT(33917)=(S=9765632,E=9765759,C=40)
	CT(33918)=(S=9765760,E=9765775,C=14)
	CT(33919)=(S=9765776,E=9765783,C=15)
	CT(33920)=(S=9765784,E=9765791,C=39)
	CT(33921)=(S=9765792,E=9765807,C=23)
	CT(33922)=(S=9765808,E=9765815,C=15)
	CT(33923)=(S=9765816,E=9765823,C=53)
	CT(33924)=(S=9765824,E=9765887,C=20)
	CT(33925)=(S=9765888,E=9771775,C=29)
	CT(33926)=(S=9771776,E=9772031,C=20)
	CT(33927)=(S=9772032,E=9777407,C=29)
	CT(33928)=(S=9777408,E=9777663,C=17)
	CT(33929)=(S=9777664,E=9779967,C=29)
	CT(33930)=(S=9779968,E=9780223,C=36)
	CT(33931)=(S=9780224,E=9780735,C=29)
	CT(33932)=(S=9780736,E=9780791,C=18)
	CT(33933)=(S=9780792,E=9780799,C=28)
	CT(33934)=(S=9780800,E=9780863,C=52)
	CT(33935)=(S=9780864,E=9780879,C=15)
	CT(33936)=(S=9780880,E=9780887,C=33)
	CT(33937)=(S=9780888,E=9780895,C=12)
	CT(33938)=(S=9780896,E=9780903,C=18)
	CT(33939)=(S=9780904,E=9780919,C=25)
	CT(33940)=(S=9780920,E=9780927,C=15)
	CT(33941)=(S=9780928,E=9780991,C=51)
	CT(33942)=(S=9780992,E=9790207,C=29)
	CT(33943)=(S=9790208,E=9790463,C=78)
	CT(33944)=(S=9790464,E=9797119,C=29)
	CT(33945)=(S=9797120,E=9797127,C=34)
	CT(33946)=(S=9797128,E=9797135,C=25)
	CT(33947)=(S=9797136,E=9797143,C=18)
	CT(33948)=(S=9797144,E=9797151,C=50)
	CT(33949)=(S=9797152,E=9797167,C=25)
	CT(33950)=(S=9797168,E=9797175,C=34)
	CT(33951)=(S=9797176,E=9797183,C=49)
	CT(33952)=(S=9797184,E=9797191,C=28)
	CT(33953)=(S=9797192,E=9797199,C=17)
	CT(33954)=(S=9797200,E=9797207,C=48)
	CT(33955)=(S=9797208,E=9797215,C=43)
	CT(33956)=(S=9797216,E=9797223,C=18)
	CT(33957)=(S=9797224,E=9797231,C=14)
	CT(33958)=(S=9797232,E=9797247,C=41)
	CT(33959)=(S=9797248,E=9797279,C=52)
	CT(33960)=(S=9797280,E=9797287,C=25)
	CT(33961)=(S=9797288,E=9797295,C=18)
	CT(33962)=(S=9797296,E=9797303,C=47)
	CT(33963)=(S=9797304,E=9797311,C=28)
	CT(33964)=(S=9797312,E=9797375,C=18)
	CT(33965)=(S=9797376,E=9797887,C=29)
	CT(33966)=(S=9797888,E=9798143,C=5)
	CT(33967)=(S=9798144,E=9798655,C=29)
	CT(33968)=(S=9798656,E=9798911,C=15)
	CT(33969)=(S=9798912,E=9799167,C=34)
	CT(33970)=(S=9799168,E=9799423,C=49)
	CT(33971)=(S=9799424,E=9799679,C=0)
	CT(33972)=(S=9799680,E=9800447,C=29)
	CT(33973)=(S=9800448,E=9800703,C=15)
	CT(33974)=(S=9800704,E=9800959,C=37)
	CT(33975)=(S=9800960,E=9801727,C=29)
	CT(33976)=(S=9801728,E=9801983,C=0)
	CT(33977)=(S=9801984,E=9802239,C=29)
	CT(33978)=(S=9802240,E=9802495,C=36)
	CT(33979)=(S=9802496,E=9802751,C=101)
	CT(33980)=(S=9802752,E=9803007,C=42)
	CT(33981)=(S=9803008,E=9804031,C=29)
	CT(33982)=(S=9804032,E=9804287,C=63)
	CT(33983)=(S=9804288,E=9804351,C=29)
	CT(33984)=(S=9804352,E=9804359,C=18)
	CT(33985)=(S=9804360,E=9804367,C=28)
	CT(33986)=(S=9804368,E=9804375,C=12)
	CT(33987)=(S=9804376,E=9804383,C=15)
	CT(33988)=(S=9804384,E=9804391,C=42)
	CT(33989)=(S=9804392,E=9804407,C=18)
	CT(33990)=(S=9804408,E=9804415,C=47)
	CT(33991)=(S=9804416,E=9804431,C=18)
	CT(33992)=(S=9804432,E=9804439,C=28)
	CT(33993)=(S=9804440,E=9804447,C=13)
	CT(33994)=(S=9804448,E=9804463,C=14)
	CT(33995)=(S=9804464,E=9804479,C=31)
	CT(33996)=(S=9804480,E=9804543,C=49)
	CT(33997)=(S=9804544,E=9804799,C=14)
	CT(33998)=(S=9804800,E=9805055,C=58)
	CT(33999)=(S=9805056,E=9805311,C=63)
	CT(34000)=(S=9805312,E=9807615,C=29)
	CT(34001)=(S=9807616,E=9807871,C=0)
	CT(34002)=(S=9807872,E=9808383,C=29)
	CT(34003)=(S=9808384,E=9808639,C=14)
	CT(34004)=(S=9808640,E=9808895,C=0)
	CT(34005)=(S=9808896,E=9809151,C=28)
	CT(34006)=(S=9809152,E=9809919,C=29)
	CT(34007)=(S=9809920,E=9810175,C=0)
	CT(34008)=(S=9810176,E=9815295,C=14)
	CT(34009)=(S=9815296,E=9815551,C=47)
	CT(34010)=(S=9815552,E=9816063,C=29)
	CT(34011)=(S=9816064,E=9816191,C=50)
	CT(34012)=(S=9816192,E=9816319,C=61)
	CT(34013)=(S=9816320,E=9816575,C=28)
	CT(34014)=(S=9816576,E=9816831,C=12)
	CT(34015)=(S=9816832,E=9818367,C=28)
	CT(34016)=(S=9818368,E=9818623,C=27)
	CT(34017)=(S=9818624,E=9818751,C=22)
	CT(34018)=(S=9818752,E=9818879,C=36)
	CT(34019)=(S=9818880,E=9819135,C=28)
	CT(34020)=(S=9819136,E=9819391,C=24)
	CT(34021)=(S=9819392,E=9821951,C=28)
	CT(34022)=(S=9821952,E=9822207,C=14)
	CT(34023)=(S=9822208,E=9826559,C=28)
	CT(34024)=(S=9826560,E=9826815,C=14)
	CT(34025)=(S=9826816,E=9828351,C=28)
	CT(34026)=(S=9828352,E=9828607,C=78)
	CT(34027)=(S=9828608,E=9829119,C=28)
	CT(34028)=(S=9829120,E=9829375,C=12)
	CT(34029)=(S=9829376,E=9829887,C=29)
	CT(34030)=(S=9829888,E=9830143,C=14)
	CT(34031)=(S=9830144,E=9830175,C=18)
	CT(34032)=(S=9830176,E=9830183,C=36)
	CT(34033)=(S=9830184,E=9830191,C=14)
	CT(34034)=(S=9830192,E=9830199,C=34)
	CT(34035)=(S=9830200,E=9830207,C=14)
	CT(34036)=(S=9830208,E=9830239,C=33)
	CT(34037)=(S=9830240,E=9830255,C=14)
	CT(34038)=(S=9830256,E=9830263,C=18)
	CT(34039)=(S=9830264,E=9830271,C=14)
	CT(34040)=(S=9830272,E=9830279,C=18)
	CT(34041)=(S=9830280,E=9830287,C=12)
	CT(34042)=(S=9830288,E=9830303,C=41)
	CT(34043)=(S=9830304,E=9830311,C=18)
	CT(34044)=(S=9830312,E=9830319,C=14)
	CT(34045)=(S=9830320,E=9830327,C=56)
	CT(34046)=(S=9830328,E=9830335,C=14)
	CT(34047)=(S=9830336,E=9830399,C=35)
	CT(34048)=(S=9830400,E=9830655,C=1)
	CT(34049)=(S=9830656,E=9856255,C=2)
	CT(34050)=(S=9856256,E=9856511,C=0)
	CT(34051)=(S=9856512,E=9857279,C=29)
	CT(34052)=(S=9857280,E=9857535,C=78)
	CT(34053)=(S=9857536,E=9857791,C=27)
	CT(34054)=(S=9857792,E=9857795,C=8)
	CT(34055)=(S=9857796,E=9857799,C=86)
	CT(34056)=(S=9857800,E=9857803,C=4)
	CT(34057)=(S=9857804,E=9857807,C=3)
	CT(34058)=(S=9857808,E=9857819,C=4)
	CT(34059)=(S=9857820,E=9857823,C=3)
	CT(34060)=(S=9857824,E=9857827,C=73)
	CT(34061)=(S=9857828,E=9857831,C=0)
	CT(34062)=(S=9857832,E=9857835,C=4)
	CT(34063)=(S=9857836,E=9857839,C=8)
	CT(34064)=(S=9857840,E=9857847,C=84)
	CT(34065)=(S=9857848,E=9857859,C=9)
	CT(34066)=(S=9857860,E=9857863,C=6)
	CT(34067)=(S=9857864,E=9857867,C=0)
	CT(34068)=(S=9857868,E=9857871,C=5)
	CT(34069)=(S=9857872,E=9857879,C=6)
	CT(34070)=(S=9857880,E=9857895,C=4)
	CT(34071)=(S=9857896,E=9857897,C=74)
	CT(34072)=(S=9857898,E=9857899,C=4)
	CT(34073)=(S=9857900,E=9857903,C=74)
	CT(34074)=(S=9857904,E=9857915,C=4)
	CT(34075)=(S=9857916,E=9857923,C=7)
	CT(34076)=(S=9857924,E=9857935,C=74)
	CT(34077)=(S=9857936,E=9857939,C=4)
	CT(34078)=(S=9857940,E=9857943,C=74)
	CT(34079)=(S=9857944,E=9857947,C=7)
	CT(34080)=(S=9857948,E=9857951,C=5)
	CT(34081)=(S=9857952,E=9857955,C=73)
	CT(34082)=(S=9857956,E=9857963,C=4)
	CT(34083)=(S=9857964,E=9857967,C=73)
	CT(34084)=(S=9857968,E=9857991,C=4)
	CT(34085)=(S=9857992,E=9857995,C=8)
	CT(34086)=(S=9857996,E=9857999,C=75)
	CT(34087)=(S=9858000,E=9858035,C=4)
	CT(34088)=(S=9858036,E=9858043,C=74)
	CT(34089)=(S=9858044,E=9858047,C=4)
	CT(34090)=(S=9858048,E=9859839,C=29)
	CT(34091)=(S=9859840,E=9860095,C=1)
	CT(34092)=(S=9860096,E=9860607,C=9)
	CT(34093)=(S=9860608,E=9861375,C=29)
	CT(34094)=(S=9861376,E=9861887,C=1)
	CT(34095)=(S=9861888,E=9863167,C=29)
	CT(34096)=(S=9863168,E=9863423,C=25)
	CT(34097)=(S=9863424,E=9863427,C=4)
	CT(34098)=(S=9863428,E=9863431,C=77)
	CT(34099)=(S=9863432,E=9863435,C=8)
	CT(34100)=(S=9863436,E=9863439,C=0)
	CT(34101)=(S=9863440,E=9863441,C=84)
	CT(34102)=(S=9863442,E=9863442,C=74)
	CT(34103)=(S=9863443,E=9863443,C=4)
	CT(34104)=(S=9863444,E=9863447,C=86)
	CT(34105)=(S=9863448,E=9863455,C=4)
	CT(34106)=(S=9863456,E=9863459,C=8)
	CT(34107)=(S=9863460,E=9863463,C=9)
	CT(34108)=(S=9863464,E=9863467,C=8)
	CT(34109)=(S=9863468,E=9863479,C=4)
	CT(34110)=(S=9863480,E=9863483,C=74)
	CT(34111)=(S=9863484,E=9863491,C=4)
	CT(34112)=(S=9863492,E=9863495,C=5)
	CT(34113)=(S=9863496,E=9863499,C=9)
	CT(34114)=(S=9863500,E=9863507,C=8)
	CT(34115)=(S=9863508,E=9863511,C=0)
	CT(34116)=(S=9863512,E=9863515,C=4)
	CT(34117)=(S=9863516,E=9863519,C=0)
	CT(34118)=(S=9863520,E=9863523,C=9)
	CT(34119)=(S=9863524,E=9863559,C=4)
	CT(34120)=(S=9863560,E=9863563,C=1)
	CT(34121)=(S=9863564,E=9863567,C=92)
	CT(34122)=(S=9863568,E=9863575,C=4)
	CT(34123)=(S=9863576,E=9863579,C=1)
	CT(34124)=(S=9863580,E=9863583,C=4)
	CT(34125)=(S=9863584,E=9863587,C=74)
	CT(34126)=(S=9863588,E=9863607,C=4)
	CT(34127)=(S=9863608,E=9863611,C=5)
	CT(34128)=(S=9863612,E=9863615,C=74)
	CT(34129)=(S=9863616,E=9863619,C=1)
	CT(34130)=(S=9863620,E=9863631,C=4)
	CT(34131)=(S=9863632,E=9863635,C=8)
	CT(34132)=(S=9863636,E=9863639,C=4)
	CT(34133)=(S=9863640,E=9863643,C=1)
	CT(34134)=(S=9863644,E=9863647,C=84)
	CT(34135)=(S=9863648,E=9863651,C=6)
	CT(34136)=(S=9863652,E=9863655,C=9)
	CT(34137)=(S=9863656,E=9863675,C=4)
	CT(34138)=(S=9863676,E=9863679,C=1)
	CT(34139)=(S=9863680,E=9864191,C=29)
	CT(34140)=(S=9864192,E=9864447,C=19)
	CT(34141)=(S=9864448,E=9865727,C=29)
	CT(34142)=(S=9865728,E=9866239,C=1)
	CT(34143)=(S=9866240,E=9866495,C=22)
	CT(34144)=(S=9866496,E=9867519,C=29)
	CT(34145)=(S=9867520,E=9868031,C=15)
	CT(34146)=(S=9868032,E=9868799,C=29)
	CT(34147)=(S=9868800,E=9869055,C=6)
	CT(34148)=(S=9869056,E=9869311,C=0)
	CT(34149)=(S=9869312,E=9870847,C=29)
	CT(34150)=(S=9870848,E=9871103,C=49)
	CT(34151)=(S=9871104,E=9871615,C=29)
	CT(34152)=(S=9871616,E=9872895,C=189)
	CT(34153)=(S=9872896,E=9874687,C=29)
	CT(34154)=(S=9874688,E=9874943,C=0)
	CT(34155)=(S=9874944,E=9875199,C=29)
	CT(34156)=(S=9875200,E=9875455,C=12)
	CT(34157)=(S=9875456,E=9875967,C=29)
	CT(34158)=(S=9875968,E=9876223,C=15)
	CT(34159)=(S=9876224,E=9877247,C=29)
	CT(34160)=(S=9877248,E=9877503,C=6)
	CT(34161)=(S=9877504,E=9877759,C=29)
	CT(34162)=(S=9877760,E=9879039,C=200)
	CT(34163)=(S=9879040,E=9879295,C=29)
	CT(34164)=(S=9879296,E=9879551,C=0)
	CT(34165)=(S=9879552,E=9880831,C=29)
	CT(34166)=(S=9880832,E=9881087,C=6)
	CT(34167)=(S=9881088,E=9882367,C=29)
	CT(34168)=(S=9882368,E=9882623,C=0)
	CT(34169)=(S=9882624,E=9882879,C=14)
	CT(34170)=(S=9882880,E=9883135,C=34)
	CT(34171)=(S=9883136,E=9883391,C=73)
	CT(34172)=(S=9883392,E=9883647,C=0)
	CT(34173)=(S=9883648,E=9884927,C=29)
	CT(34174)=(S=9884928,E=9885183,C=34)
	CT(34175)=(S=9885184,E=9885439,C=25)
	CT(34176)=(S=9885440,E=9885951,C=29)
	CT(34177)=(S=9885952,E=9886207,C=15)
	CT(34178)=(S=9886208,E=9887487,C=29)
	CT(34179)=(S=9887488,E=9887743,C=1)
	CT(34180)=(S=9887744,E=9888511,C=29)
	CT(34181)=(S=9888512,E=9888767,C=19)
	CT(34182)=(S=9888768,E=9889023,C=29)
	CT(34183)=(S=9889024,E=9889279,C=0)
	CT(34184)=(S=9889536,E=9890815,C=29)
	CT(34185)=(S=9890816,E=9891071,C=19)
	CT(34186)=(S=9891072,E=9891327,C=14)
	CT(34187)=(S=9891328,E=9892095,C=29)
	CT(34188)=(S=9892096,E=9892351,C=25)
	CT(34189)=(S=9892352,E=9892363,C=1)
	CT(34190)=(S=9892364,E=9892379,C=4)
	CT(34191)=(S=9892380,E=9892383,C=1)
	CT(34192)=(S=9892384,E=9892387,C=0)
	CT(34193)=(S=9892388,E=9892391,C=8)
	CT(34194)=(S=9892392,E=9892395,C=73)
	CT(34195)=(S=9892396,E=9892411,C=1)
	CT(34196)=(S=9892412,E=9892427,C=4)
	CT(34197)=(S=9892428,E=9892435,C=1)
	CT(34198)=(S=9892436,E=9892443,C=4)
	CT(34199)=(S=9892444,E=9892451,C=1)
	CT(34200)=(S=9892452,E=9892455,C=9)
	CT(34201)=(S=9892456,E=9892459,C=84)
	CT(34202)=(S=9892460,E=9892463,C=165)
	CT(34203)=(S=9892464,E=9892475,C=1)
	CT(34204)=(S=9892476,E=9892479,C=8)
	CT(34205)=(S=9892480,E=9892483,C=73)
	CT(34206)=(S=9892484,E=9892487,C=6)
	CT(34207)=(S=9892488,E=9892491,C=0)
	CT(34208)=(S=9892492,E=9892495,C=4)
	CT(34209)=(S=9892496,E=9892499,C=6)
	CT(34210)=(S=9892500,E=9892503,C=4)
	CT(34211)=(S=9892504,E=9892523,C=1)
	CT(34212)=(S=9892524,E=9892531,C=4)
	CT(34213)=(S=9892532,E=9892535,C=5)
	CT(34214)=(S=9892536,E=9892547,C=1)
	CT(34215)=(S=9892548,E=9892559,C=4)
	CT(34216)=(S=9892560,E=9892563,C=8)
	CT(34217)=(S=9892564,E=9892567,C=1)
	CT(34218)=(S=9892568,E=9892571,C=8)
	CT(34219)=(S=9892572,E=9892575,C=29)
	CT(34220)=(S=9892576,E=9892579,C=1)
	CT(34221)=(S=9892580,E=9892583,C=8)
	CT(34222)=(S=9892584,E=9892603,C=1)
	CT(34223)=(S=9892604,E=9892607,C=4)
	CT(34224)=(S=9892608,E=9892863,C=29)
	CT(34225)=(S=9892864,E=9893119,C=25)
	CT(34226)=(S=9893120,E=9894655,C=29)
	CT(34227)=(S=9894656,E=9894911,C=49)
	CT(34228)=(S=9894912,E=9895423,C=29)
	CT(34229)=(S=9895424,E=9895679,C=58)
	CT(34230)=(S=9895680,E=9895935,C=1)
	CT(34231)=(S=9895936,E=9895999,C=31)
	CT(34232)=(S=9896000,E=9896063,C=50)
	CT(34233)=(S=9896064,E=9921791,C=15)
	CT(34234)=(S=9921792,E=9922815,C=29)
	CT(34235)=(S=9922816,E=9923071,C=47)
	CT(34236)=(S=9923072,E=9925375,C=29)
	CT(34237)=(S=9925376,E=9925631,C=14)
	CT(34238)=(S=9925632,E=9926655,C=29)
	CT(34239)=(S=9926656,E=9926911,C=34)
	CT(34240)=(S=9926912,E=9928447,C=29)
	CT(34241)=(S=9928448,E=9928703,C=34)
	CT(34242)=(S=9928704,E=9928959,C=29)
	CT(34243)=(S=9928960,E=9929215,C=14)
	CT(34244)=(S=9929216,E=9929471,C=0)
	CT(34245)=(S=9929472,E=9929983,C=29)
	CT(34246)=(S=9929984,E=9930239,C=14)
	CT(34247)=(S=9930240,E=9930495,C=29)
	CT(34248)=(S=9930496,E=9930751,C=37)
	CT(34249)=(S=9930752,E=9931007,C=28)
	CT(34250)=(S=9931008,E=9935871,C=29)
	CT(34251)=(S=9935872,E=9936127,C=19)
	CT(34252)=(S=9936128,E=9936383,C=27)
	CT(34253)=(S=9936640,E=9939455,C=29)
	CT(34254)=(S=9939456,E=9939711,C=14)
	CT(34255)=(S=9939712,E=9940735,C=29)
	CT(34256)=(S=9940736,E=9940991,C=13)
	CT(34257)=(S=9940992,E=9941247,C=14)
	CT(34258)=(S=9941248,E=9941503,C=19)
	CT(34259)=(S=9941504,E=9941759,C=0)
	CT(34260)=(S=9941760,E=9942015,C=29)
	CT(34261)=(S=9942016,E=9942271,C=14)
	CT(34262)=(S=9942272,E=9942527,C=29)
	CT(34263)=(S=9942528,E=9942783,C=14)
	CT(34264)=(S=9942784,E=9943039,C=34)
	CT(34265)=(S=9943040,E=9943295,C=36)
	CT(34266)=(S=9943296,E=9943807,C=29)
	CT(34267)=(S=9943808,E=9944063,C=27)
	CT(34268)=(S=9944064,E=9944319,C=29)
	CT(34269)=(S=9944320,E=9944575,C=28)
	CT(34270)=(S=9944576,E=9949951,C=29)
	CT(34271)=(S=9949952,E=9950463,C=14)
	CT(34272)=(S=9950464,E=9951231,C=29)
	CT(34273)=(S=9951265,E=9951266,C=14)
	CT(34274)=(S=9951268,E=9951271,C=26)
	CT(34275)=(S=9951272,E=9951273,C=46)
	CT(34276)=(S=9951296,E=9951359,C=51)
	CT(34277)=(S=9953280,E=9955327,C=14)
	CT(34278)=(S=9955328,E=9956351,C=26)
	CT(34279)=(S=9956352,E=9956383,C=13)
	CT(34280)=(S=9956384,E=9956415,C=14)
	CT(34281)=(S=9956416,E=9956479,C=18)
	CT(34282)=(S=9956480,E=9956511,C=36)
	CT(34283)=(S=9956512,E=9956543,C=35)
	CT(34284)=(S=9956544,E=9956551,C=20)
	CT(34285)=(S=9956552,E=9956559,C=19)
	CT(34286)=(S=9956560,E=9956575,C=14)
	CT(34287)=(S=9956576,E=9956591,C=71)
	CT(34288)=(S=9956592,E=9956607,C=54)
	CT(34289)=(S=9956608,E=9956751,C=51)
	CT(34290)=(S=9956752,E=9956767,C=20)
	CT(34291)=(S=9956768,E=9956775,C=36)
	CT(34292)=(S=9956776,E=9956783,C=18)
	CT(34293)=(S=9956784,E=9956799,C=19)
	CT(34294)=(S=9956800,E=9956831,C=25)
	CT(34295)=(S=9956832,E=9956839,C=33)
	CT(34296)=(S=9956840,E=9956847,C=14)
	CT(34297)=(S=9956848,E=9956855,C=18)
	CT(34298)=(S=9956856,E=9956863,C=28)
	CT(34299)=(S=9956864,E=9959423,C=26)
	CT(34300)=(S=9959424,E=9959431,C=19)
	CT(34301)=(S=9959432,E=9959439,C=15)
	CT(34302)=(S=9959440,E=9959447,C=49)
	CT(34303)=(S=9959448,E=9959455,C=28)
	CT(34304)=(S=9959456,E=9959471,C=58)
	CT(34305)=(S=9959472,E=9959479,C=49)
	CT(34306)=(S=9959480,E=9959487,C=58)
	CT(34307)=(S=9959488,E=9959519,C=38)
	CT(34308)=(S=9959520,E=9959535,C=104)
	CT(34309)=(S=9959536,E=9959551,C=18)
	CT(34310)=(S=9959552,E=9959679,C=34)
	CT(34311)=(S=9959680,E=9959759,C=14)
	CT(34312)=(S=9959760,E=9959767,C=58)
	CT(34313)=(S=9959768,E=9959775,C=13)
	CT(34314)=(S=9959776,E=9959783,C=31)
	CT(34315)=(S=9959784,E=9959791,C=33)
	CT(34316)=(S=9959792,E=9959807,C=27)
	CT(34317)=(S=9959808,E=9959935,C=33)
	CT(34318)=(S=9959936,E=9960191,C=37)
	CT(34319)=(S=9960192,E=9960447,C=51)
	CT(34320)=(S=9960448,E=9960455,C=14)
	CT(34321)=(S=9960456,E=9960463,C=27)
	CT(34322)=(S=9960464,E=9960471,C=28)
	CT(34323)=(S=9960472,E=9960479,C=19)
	CT(34324)=(S=9960480,E=9960487,C=34)
	CT(34325)=(S=9960488,E=9960511,C=28)
	CT(34326)=(S=9960512,E=9960543,C=18)
	CT(34327)=(S=9960544,E=9960551,C=43)
	CT(34328)=(S=9960552,E=9960559,C=18)
	CT(34329)=(S=9960560,E=9960567,C=25)
	CT(34330)=(S=9960568,E=9960575,C=14)
	CT(34331)=(S=9960576,E=9960639,C=19)
	CT(34332)=(S=9960640,E=9960655,C=51)
	CT(34333)=(S=9960656,E=9960663,C=57)
	CT(34334)=(S=9960664,E=9960671,C=28)
	CT(34335)=(S=9960672,E=9960703,C=57)
	CT(34336)=(S=9960704,E=9960959,C=16)
	CT(34337)=(S=9960960,E=9961471,C=23)
	CT(34338)=(S=9961472,E=9961727,C=177)
	CT(34339)=(S=9961728,E=9974783,C=29)
	CT(34340)=(S=9974784,E=9975039,C=4)
	CT(34341)=(S=9975040,E=9978367,C=29)
	CT(34342)=(S=9978368,E=9978623,C=50)
	CT(34343)=(S=9978624,E=9979647,C=29)
	CT(34344)=(S=9979648,E=9979903,C=14)
	CT(34345)=(S=9979904,E=9980159,C=29)
	CT(34346)=(S=9980160,E=9980415,C=24)
	CT(34347)=(S=9980416,E=9980671,C=203)
	CT(34348)=(S=9980672,E=9980927,C=29)
	CT(34349)=(S=9980928,E=9981183,C=0)
	CT(34350)=(S=9981184,E=9981439,C=12)
	CT(34351)=(S=9981440,E=9981695,C=14)
	CT(34352)=(S=9981696,E=9982207,C=29)
	CT(34353)=(S=9982208,E=9982463,C=12)
	CT(34354)=(S=9982464,E=9982719,C=29)
	CT(34355)=(S=9982720,E=9982975,C=0)
	CT(34356)=(S=9982976,E=9983231,C=189)
	CT(34357)=(S=9983232,E=9983999,C=29)
	CT(34358)=(S=9984000,E=9984255,C=34)
	CT(34359)=(S=9984512,E=9984767,C=27)
	CT(34360)=(S=9984768,E=9985023,C=0)
	CT(34361)=(S=9985024,E=9985279,C=189)
	CT(34362)=(S=9985280,E=9985791,C=27)
	CT(34363)=(S=9985792,E=9986047,C=24)
	CT(34364)=(S=9986048,E=9986303,C=34)
	CT(34365)=(S=9986304,E=9986559,C=29)
	CT(34366)=(S=9986560,E=9986815,C=0)
	CT(34367)=(S=9986816,E=9987071,C=6)
	CT(34368)=(S=9987072,E=9987327,C=29)
	CT(34369)=(S=9987328,E=9987583,C=8)
	CT(34370)=(S=9987584,E=9987839,C=7)
	CT(34371)=(S=9987840,E=9988223,C=8)
	CT(34372)=(S=9988224,E=9988351,C=1)
	CT(34373)=(S=9988352,E=9988607,C=14)
	CT(34374)=(S=9988608,E=9990399,C=105)
	CT(34375)=(S=9990400,E=9990655,C=29)
	CT(34376)=(S=9990656,E=9990911,C=14)
	CT(34377)=(S=9990912,E=9991167,C=24)
	CT(34378)=(S=9991168,E=9991679,C=29)
	CT(34379)=(S=9991680,E=9991935,C=74)
	CT(34380)=(S=9991936,E=9995775,C=29)
	CT(34381)=(S=9995776,E=9996031,C=14)
	CT(34382)=(S=9996032,E=9997055,C=29)
	CT(34383)=(S=9997056,E=9997311,C=203)
	CT(34384)=(S=9997312,E=9998079,C=29)
	CT(34385)=(S=9998080,E=9998335,C=28)
	CT(34386)=(S=9998336,E=9999103,C=29)
	CT(34387)=(S=9999104,E=9999359,C=0)
	CT(34388)=(S=9999360,E=9999615,C=29)
	CT(34389)=(S=9999616,E=9999871,C=6)
	CT(34390)=(S=9999872,E=10000127,C=14)
	CT(34391)=(S=10000128,E=10000383,C=29)
	CT(34392)=(S=10000384,E=10000639,C=49)
	CT(34393)=(S=10000896,E=10001407,C=29)
	CT(34394)=(S=10001408,E=10001663,C=205)
	CT(34395)=(S=10001664,E=10003967,C=29)
	CT(34396)=(S=10003968,E=10004479,C=177)
	CT(34397)=(S=10004480,E=10005503,C=83)
	CT(34398)=(S=10005504,E=10006527,C=203)
	CT(34399)=(S=10006528,E=10012671,C=29)
	CT(34400)=(S=10012672,E=10014207,C=181)
	CT(34401)=(S=10014208,E=10014719,C=219)
	CT(34402)=(S=10014720,E=10019327,C=29)
	CT(34403)=(S=10019328,E=10019583,C=7)
	CT(34404)=(S=10019584,E=10020351,C=29)
	CT(34405)=(S=10020352,E=10020607,C=203)
	CT(34406)=(S=10020608,E=10020623,C=208)
	CT(34407)=(S=10020624,E=10020631,C=83)
	CT(34408)=(S=10020632,E=10020639,C=181)
	CT(34409)=(S=10020640,E=10020655,C=216)
	CT(34410)=(S=10020656,E=10020671,C=83)
	CT(34411)=(S=10020672,E=10020735,C=203)
	CT(34412)=(S=10020736,E=10020863,C=202)
	CT(34413)=(S=10020864,E=10027007,C=189)
	CT(34414)=(S=10027008,E=10027263,C=1)
	CT(34415)=(S=10027264,E=10027519,C=47)
	CT(34416)=(S=10027520,E=10027775,C=29)
	CT(34417)=(S=10027776,E=10028031,C=1)
	CT(34418)=(S=10028032,E=10028287,C=29)
	CT(34419)=(S=10028288,E=10028543,C=52)
	CT(34420)=(S=10028544,E=10030847,C=29)
	CT(34421)=(S=10030848,E=10031103,C=27)
	CT(34422)=(S=10031104,E=10031359,C=29)
	CT(34423)=(S=10031360,E=10031615,C=28)
	CT(34424)=(S=10031616,E=10031871,C=29)
	CT(34425)=(S=10031872,E=10032127,C=58)
	CT(34426)=(S=10032128,E=10032383,C=7)
	CT(34427)=(S=10032384,E=10035711,C=29)
	CT(34428)=(S=10035712,E=10036735,C=1)
	CT(34429)=(S=10036736,E=10038271,C=29)
	CT(34430)=(S=10038272,E=10038527,C=27)
	CT(34431)=(S=10038528,E=10049535,C=29)
	CT(34432)=(S=10049536,E=10049791,C=19)
	CT(34433)=(S=10049792,E=10050559,C=29)
	CT(34434)=(S=10050560,E=10050783,C=28)
	CT(34435)=(S=10050784,E=10050815,C=12)
	CT(34436)=(S=10050816,E=10052351,C=28)
	CT(34437)=(S=10052352,E=10052607,C=1)
	CT(34438)=(S=10052608,E=10052863,C=28)
	CT(34439)=(S=10052864,E=10053119,C=1)
	CT(34440)=(S=10053120,E=10054399,C=29)
	CT(34441)=(S=10054400,E=10054655,C=0)
	CT(34442)=(S=10054656,E=10055167,C=34)
	CT(34443)=(S=10055168,E=10055423,C=27)
	CT(34444)=(S=10055424,E=10055679,C=73)
	CT(34445)=(S=10055680,E=10055935,C=19)
	CT(34446)=(S=10055936,E=10057215,C=29)
	CT(34447)=(S=10057216,E=10057727,C=1)
	CT(34448)=(S=10057728,E=10092543,C=2)
	CT(34449)=(S=10092544,E=10092559,C=105)
	CT(34450)=(S=10092560,E=10092567,C=126)
	CT(34451)=(S=10092568,E=10092575,C=118)
	CT(34452)=(S=10092576,E=10092607,C=122)
	CT(34453)=(S=10092608,E=10092671,C=116)
	CT(34454)=(S=10092672,E=10092687,C=130)
	CT(34455)=(S=10092688,E=10092695,C=111)
	CT(34456)=(S=10092696,E=10092703,C=112)
	CT(34457)=(S=10092704,E=10092719,C=105)
	CT(34458)=(S=10092720,E=10092735,C=122)
	CT(34459)=(S=10092736,E=10092799,C=117)
	CT(34460)=(S=10092800,E=10093823,C=29)
	CT(34461)=(S=10093824,E=10094079,C=78)
	CT(34462)=(S=10094080,E=10094591,C=29)
	CT(34463)=(S=10094592,E=10094847,C=14)
	CT(34464)=(S=10094848,E=10095103,C=29)
	CT(34465)=(S=10095104,E=10095359,C=6)
	CT(34466)=(S=10095360,E=10095615,C=78)
	CT(34467)=(S=10095616,E=10096127,C=29)
	CT(34468)=(S=10096128,E=10096639,C=34)
	CT(34469)=(S=10096640,E=10096895,C=113)
	CT(34470)=(S=10096896,E=10097663,C=29)
	CT(34471)=(S=10097664,E=10097919,C=78)
	CT(34472)=(S=10097920,E=10100735,C=29)
	CT(34473)=(S=10100736,E=10100991,C=14)
	CT(34474)=(S=10100992,E=10101503,C=2)
	CT(34475)=(S=10101504,E=10109183,C=29)
	CT(34476)=(S=10109184,E=10109187,C=105)
	CT(34477)=(S=10109188,E=10109191,C=111)
	CT(34478)=(S=10109192,E=10109195,C=113)
	CT(34479)=(S=10109196,E=10109199,C=105)
	CT(34480)=(S=10109200,E=10109215,C=109)
	CT(34481)=(S=10109216,E=10109223,C=148)
	CT(34482)=(S=10109224,E=10109227,C=105)
	CT(34483)=(S=10109228,E=10109235,C=144)
	CT(34484)=(S=10109236,E=10109239,C=105)
	CT(34485)=(S=10109244,E=10109247,C=139)
	CT(34486)=(S=10109248,E=10109279,C=105)
	CT(34487)=(S=10109280,E=10109311,C=110)
	CT(34488)=(S=10109312,E=10109439,C=116)
	CT(34489)=(S=10109440,E=10109503,C=111)
	CT(34490)=(S=10109504,E=10109519,C=105)
	CT(34491)=(S=10109520,E=10109523,C=116)
	CT(34492)=(S=10109524,E=10109527,C=136)
	CT(34493)=(S=10109528,E=10109535,C=105)
	CT(34494)=(S=10109540,E=10109543,C=144)
	CT(34495)=(S=10109544,E=10109547,C=116)
	CT(34496)=(S=10109548,E=10109551,C=133)
	CT(34497)=(S=10109552,E=10109555,C=105)
	CT(34498)=(S=10109556,E=10109559,C=125)
	CT(34499)=(S=10109560,E=10109567,C=121)
	CT(34500)=(S=10109568,E=10109583,C=124)
	CT(34501)=(S=10109584,E=10109599,C=105)
	CT(34502)=(S=10109600,E=10109615,C=139)
	CT(34503)=(S=10109616,E=10109651,C=105)
	CT(34504)=(S=10109652,E=10109655,C=112)
	CT(34505)=(S=10109656,E=10109659,C=130)
	CT(34506)=(S=10109660,E=10109663,C=141)
	CT(34507)=(S=10109664,E=10109667,C=112)
	CT(34508)=(S=10109668,E=10109671,C=113)
	CT(34509)=(S=10109672,E=10109679,C=122)
	CT(34510)=(S=10109680,E=10109687,C=113)
	CT(34511)=(S=10109688,E=10109695,C=105)
	CT(34512)=(S=10109696,E=10109951,C=151)
	CT(34513)=(S=10109952,E=10110015,C=118)
	CT(34514)=(S=10110016,E=10110079,C=128)
	CT(34515)=(S=10110144,E=10110207,C=111)
	CT(34516)=(S=10110208,E=10110463,C=105)
	CT(34517)=(S=10110464,E=10110527,C=109)
	CT(34518)=(S=10110528,E=10110543,C=110)
	CT(34519)=(S=10110544,E=10110559,C=143)
	CT(34520)=(S=10110560,E=10110591,C=120)
	CT(34521)=(S=10110592,E=10110607,C=105)
	CT(34522)=(S=10110608,E=10110615,C=126)
	CT(34523)=(S=10110616,E=10110619,C=113)
	CT(34524)=(S=10110620,E=10110623,C=130)
	CT(34525)=(S=10110624,E=10110655,C=147)
	CT(34526)=(S=10110656,E=10110659,C=123)
	CT(34527)=(S=10110660,E=10110663,C=145)
	CT(34528)=(S=10110664,E=10110671,C=134)
	CT(34529)=(S=10110672,E=10110719,C=105)
	CT(34530)=(S=10110720,E=10110975,C=113)
	CT(34531)=(S=10110976,E=10110979,C=109)
	CT(34532)=(S=10110980,E=10110987,C=105)
	CT(34533)=(S=10110988,E=10110991,C=220)
	CT(34534)=(S=10110992,E=10110995,C=130)
	CT(34535)=(S=10110996,E=10110999,C=112)
	CT(34536)=(S=10111000,E=10111003,C=140)
	CT(34537)=(S=10111004,E=10111007,C=109)
	CT(34538)=(S=10111008,E=10111015,C=111)
	CT(34539)=(S=10111016,E=10111023,C=105)
	CT(34540)=(S=10111024,E=10111027,C=140)
	CT(34541)=(S=10111028,E=10111031,C=137)
	CT(34542)=(S=10111032,E=10111039,C=105)
	CT(34543)=(S=10111040,E=10111071,C=112)
	CT(34544)=(S=10111072,E=10111087,C=105)
	CT(34545)=(S=10111088,E=10111103,C=124)
	CT(34546)=(S=10111104,E=10111167,C=120)
	CT(34547)=(S=10111168,E=10111199,C=130)
	CT(34548)=(S=10111200,E=10111215,C=152)
	CT(34549)=(S=10111216,E=10111231,C=113)
	CT(34550)=(S=10111232,E=10111239,C=105)
	CT(34551)=(S=10111240,E=10111243,C=111)
	CT(34552)=(S=10111244,E=10111247,C=130)
	CT(34553)=(S=10111248,E=10111251,C=105)
	CT(34554)=(S=10111252,E=10111255,C=137)
	CT(34555)=(S=10111256,E=10111259,C=140)
	CT(34556)=(S=10111260,E=10111263,C=65)
	CT(34557)=(S=10111264,E=10111267,C=105)
	CT(34558)=(S=10111268,E=10111271,C=126)
	CT(34559)=(S=10111272,E=10111275,C=145)
	CT(34560)=(S=10111276,E=10111279,C=140)
	CT(34561)=(S=10111280,E=10111283,C=147)
	CT(34562)=(S=10111284,E=10111287,C=65)
	CT(34563)=(S=10111288,E=10111291,C=146)
	CT(34564)=(S=10111292,E=10111295,C=221)
	CT(34565)=(S=10111296,E=10111299,C=112)
	CT(34566)=(S=10111300,E=10111307,C=105)
	CT(34567)=(S=10111308,E=10111311,C=140)
	CT(34568)=(S=10111312,E=10111315,C=107)
	CT(34569)=(S=10111316,E=10111319,C=126)
	CT(34570)=(S=10111320,E=10111323,C=144)
	CT(34571)=(S=10111324,E=10111327,C=135)
	CT(34572)=(S=10111328,E=10111331,C=105)
	CT(34573)=(S=10111332,E=10111335,C=118)
	CT(34574)=(S=10111336,E=10111339,C=145)
	CT(34575)=(S=10111340,E=10111343,C=65)
	CT(34576)=(S=10111344,E=10111347,C=125)
	CT(34577)=(S=10111348,E=10111351,C=129)
	CT(34578)=(S=10111352,E=10111355,C=105)
	CT(34579)=(S=10111356,E=10111359,C=140)
	CT(34580)=(S=10111360,E=10111367,C=65)
	CT(34581)=(S=10111368,E=10111375,C=105)
	CT(34582)=(S=10111376,E=10111383,C=112)
	CT(34583)=(S=10111384,E=10111391,C=105)
	CT(34584)=(S=10111392,E=10111399,C=125)
	CT(34585)=(S=10111400,E=10111403,C=112)
	CT(34586)=(S=10111404,E=10111407,C=148)
	CT(34587)=(S=10111408,E=10111411,C=97)
	CT(34588)=(S=10111412,E=10111431,C=105)
	CT(34589)=(S=10111432,E=10111439,C=120)
	CT(34590)=(S=10111440,E=10111463,C=105)
	CT(34591)=(S=10111464,E=10111467,C=114)
	CT(34592)=(S=10111468,E=10111471,C=111)
	CT(34593)=(S=10111472,E=10111487,C=105)
	CT(34594)=(S=10111488,E=10111999,C=112)
	CT(34595)=(S=10112000,E=10113023,C=109)
	CT(34596)=(S=10113024,E=10117119,C=147)
	CT(34597)=(S=10117120,E=10119167,C=119)
	CT(34598)=(S=10119168,E=10121215,C=152)
	CT(34599)=(S=10121216,E=10121471,C=122)
	CT(34600)=(S=10121472,E=10121727,C=111)
	CT(34601)=(S=10121728,E=10122111,C=105)
	CT(34602)=(S=10122240,E=10122367,C=122)
	CT(34603)=(S=10122368,E=10122495,C=116)
	CT(34604)=(S=10122496,E=10122559,C=110)
	CT(34605)=(S=10122560,E=10122623,C=111)
	CT(34606)=(S=10122624,E=10122687,C=105)
	CT(34607)=(S=10122688,E=10122751,C=145)
	CT(34608)=(S=10122752,E=10122879,C=111)
	CT(34609)=(S=10122880,E=10122943,C=123)
	CT(34610)=(S=10122944,E=10122975,C=116)
	CT(34611)=(S=10122976,E=10122983,C=112)
	CT(34612)=(S=10122992,E=10122995,C=140)
	CT(34613)=(S=10122996,E=10122999,C=105)
	CT(34614)=(S=10123000,E=10123003,C=122)
	CT(34615)=(S=10123004,E=10123007,C=105)
	CT(34616)=(S=10123008,E=10123039,C=145)
	CT(34617)=(S=10123040,E=10123087,C=105)
	CT(34618)=(S=10123088,E=10123103,C=107)
	CT(34619)=(S=10123104,E=10123135,C=138)
	CT(34620)=(S=10123136,E=10123151,C=108)
	CT(34621)=(S=10123152,E=10123199,C=105)
	CT(34622)=(S=10123200,E=10123231,C=122)
	CT(34623)=(S=10123232,E=10123327,C=105)
	CT(34624)=(S=10123328,E=10123391,C=111)
	CT(34625)=(S=10123392,E=10123455,C=115)
	CT(34626)=(S=10123456,E=10123519,C=113)
	CT(34627)=(S=10123520,E=10123775,C=135)
	CT(34628)=(S=10123776,E=10124287,C=109)
	CT(34629)=(S=10124800,E=10124927,C=115)
	CT(34630)=(S=10124928,E=10124991,C=120)
	CT(34631)=(S=10124992,E=10125023,C=105)
	CT(34632)=(S=10125024,E=10125087,C=112)
	CT(34633)=(S=10125088,E=10125103,C=124)
	CT(34634)=(S=10125104,E=10125119,C=113)
	CT(34635)=(S=10125120,E=10125135,C=65)
	CT(34636)=(S=10125136,E=10125151,C=141)
	CT(34637)=(S=10125152,E=10125167,C=135)
	CT(34638)=(S=10125168,E=10125183,C=105)
	CT(34639)=(S=10125184,E=10125311,C=116)
	CT(34640)=(S=10158080,E=10158335,C=114)
	CT(34641)=(S=10158336,E=10158847,C=29)
	CT(34642)=(S=10158848,E=10159103,C=78)
	CT(34643)=(S=10159104,E=10159359,C=27)
	CT(34644)=(S=10159360,E=10160639,C=29)
	CT(34645)=(S=10160640,E=10160895,C=78)
	CT(34646)=(S=10160896,E=10161151,C=106)
	CT(34647)=(S=10161152,E=10161279,C=112)
	CT(34648)=(S=10161408,E=10161919,C=29)
	CT(34649)=(S=10161920,E=10162175,C=78)
	CT(34650)=(S=10162176,E=10166271,C=29)
	CT(34651)=(S=10166272,E=10166527,C=73)
	CT(34652)=(S=10166528,E=10169599,C=29)
	CT(34653)=(S=10169600,E=10169855,C=28)
	CT(34654)=(S=10169856,E=10171903,C=29)
	CT(34655)=(S=10171904,E=10172159,C=25)
	CT(34656)=(S=10172160,E=10172415,C=27)
	CT(34657)=(S=10172416,E=10172671,C=28)
	CT(34658)=(S=10172672,E=10173183,C=29)
	CT(34659)=(S=10173184,E=10173439,C=73)
	CT(34660)=(S=10173440,E=10173695,C=0)
	CT(34661)=(S=10173696,E=10174975,C=29)
	CT(34662)=(S=10174976,E=10175231,C=34)
	CT(34663)=(S=10175232,E=10175743,C=29)
	CT(34664)=(S=10175744,E=10175999,C=7)
	CT(34665)=(S=10176000,E=10176255,C=29)
	CT(34666)=(S=10176256,E=10176511,C=0)
	CT(34667)=(S=10176512,E=10176767,C=29)
	CT(34668)=(S=10176768,E=10177023,C=42)
	CT(34669)=(S=10177024,E=10180863,C=29)
	CT(34670)=(S=10180864,E=10181119,C=116)
	CT(34671)=(S=10181120,E=10184959,C=29)
	CT(34672)=(S=10184960,E=10185215,C=34)
	CT(34673)=(S=10185216,E=10191615,C=29)
	CT(34674)=(S=10191616,E=10191871,C=14)
	CT(34675)=(S=10191872,E=10192127,C=12)
	CT(34676)=(S=10192128,E=10192383,C=28)
	CT(34677)=(S=10192384,E=10192895,C=29)
	CT(34678)=(S=10192896,E=10193151,C=14)
	CT(34679)=(S=10193152,E=10193407,C=24)
	CT(34680)=(S=10193408,E=10193919,C=29)
	CT(34681)=(S=10193920,E=10194175,C=14)
	CT(34682)=(S=10194176,E=10194687,C=29)
	CT(34683)=(S=10194688,E=10195199,C=0)
	CT(34684)=(S=10195200,E=10195455,C=14)
	CT(34685)=(S=10195456,E=10198527,C=29)
	CT(34686)=(S=10198528,E=10198783,C=58)
	CT(34687)=(S=10198784,E=10199039,C=105)
	CT(34688)=(S=10199040,E=10199295,C=2)
	CT(34689)=(S=10199296,E=10205439,C=29)
	CT(34690)=(S=10205440,E=10205695,C=15)
	CT(34691)=(S=10205696,E=10205951,C=29)
	CT(34692)=(S=10205952,E=10206207,C=0)
	CT(34693)=(S=10206208,E=10207231,C=29)
	CT(34694)=(S=10207232,E=10207487,C=14)
	CT(34695)=(S=10207488,E=10207743,C=29)
	CT(34696)=(S=10207744,E=10207999,C=78)
	CT(34697)=(S=10208000,E=10208767,C=29)
	CT(34698)=(S=10208768,E=10209023,C=14)
	CT(34699)=(S=10209024,E=10209791,C=29)
	CT(34700)=(S=10209792,E=10210047,C=14)
	CT(34701)=(S=10210048,E=10210303,C=29)
	CT(34702)=(S=10210304,E=10210559,C=36)
	CT(34703)=(S=10210560,E=10210815,C=0)
	CT(34704)=(S=10210816,E=10211071,C=29)
	CT(34705)=(S=10211072,E=10211327,C=22)
	CT(34706)=(S=10211328,E=10211583,C=29)
	CT(34707)=(S=10211584,E=10211839,C=27)
	CT(34708)=(S=10211840,E=10212095,C=25)
	CT(34709)=(S=10212096,E=10212351,C=189)
	CT(34710)=(S=10212352,E=10215167,C=29)
	CT(34711)=(S=10215168,E=10215423,C=37)
	CT(34712)=(S=10215424,E=10216191,C=29)
	CT(34713)=(S=10216192,E=10216447,C=27)
	CT(34714)=(S=10216448,E=10216703,C=34)
	CT(34715)=(S=10216704,E=10216959,C=29)
	CT(34716)=(S=10216960,E=10217215,C=6)
	CT(34717)=(S=10217216,E=10217471,C=14)
	CT(34718)=(S=10217472,E=10219775,C=105)
	CT(34719)=(S=10219776,E=10220799,C=29)
	CT(34720)=(S=10220800,E=10221055,C=14)
	CT(34721)=(S=10221056,E=10221823,C=29)
	CT(34722)=(S=10221824,E=10222079,C=14)
	CT(34723)=(S=10222080,E=10222335,C=28)
	CT(34724)=(S=10222592,E=10222847,C=29)
	CT(34725)=(S=10222848,E=10223103,C=15)
	CT(34726)=(S=10223104,E=10223105,C=29)
	CT(34727)=(S=10223106,E=10223107,C=78)
	CT(34728)=(S=10223108,E=10223167,C=29)
	CT(34729)=(S=10223168,E=10223199,C=78)
	CT(34730)=(S=10223200,E=10223239,C=29)
	CT(34731)=(S=10223240,E=10223263,C=78)
	CT(34732)=(S=10223264,E=10225663,C=29)
	CT(34733)=(S=10225664,E=10225919,C=105)
	CT(34734)=(S=10225920,E=10226175,C=29)
	CT(34735)=(S=10226176,E=10226431,C=47)
	CT(34736)=(S=10226432,E=10226687,C=78)
	CT(34737)=(S=10226688,E=10226943,C=29)
	CT(34738)=(S=10226944,E=10227199,C=73)
	CT(34739)=(S=10227200,E=10227455,C=15)
	CT(34740)=(S=10227456,E=10227711,C=29)
	CT(34741)=(S=10227968,E=10228223,C=58)
	CT(34742)=(S=10228224,E=10228479,C=12)
	CT(34743)=(S=10228480,E=10229247,C=29)
	CT(34744)=(S=10229248,E=10229503,C=0)
	CT(34745)=(S=10229504,E=10230015,C=29)
	CT(34746)=(S=10230016,E=10230271,C=34)
	CT(34747)=(S=10230272,E=10230783,C=29)
	CT(34748)=(S=10230784,E=10231039,C=12)
	CT(34749)=(S=10231040,E=10231551,C=29)
	CT(34750)=(S=10231552,E=10231807,C=157)
	CT(34751)=(S=10231808,E=10232319,C=29)
	CT(34752)=(S=10232320,E=10232575,C=78)
	CT(34753)=(S=10232576,E=10232831,C=25)
	CT(34754)=(S=10232832,E=10234623,C=29)
	CT(34755)=(S=10234624,E=10234879,C=14)
	CT(34756)=(S=10234880,E=10235135,C=78)
	CT(34757)=(S=10235136,E=10235903,C=29)
	CT(34758)=(S=10235904,E=10236159,C=14)
	CT(34759)=(S=10236160,E=10236415,C=19)
	CT(34760)=(S=10236416,E=10236671,C=0)
	CT(34761)=(S=10236672,E=10236927,C=19)
	CT(34762)=(S=10236928,E=10237183,C=27)
	CT(34763)=(S=10237184,E=10237439,C=29)
	CT(34764)=(S=10237440,E=10237695,C=15)
	CT(34765)=(S=10237696,E=10238207,C=29)
	CT(34766)=(S=10238208,E=10238463,C=78)
	CT(34767)=(S=10238464,E=10238719,C=42)
	CT(34768)=(S=10238720,E=10238975,C=73)
	CT(34769)=(S=10238976,E=10239231,C=29)
	CT(34770)=(S=10239232,E=10239487,C=14)
	CT(34771)=(S=10239488,E=10239743,C=73)
	CT(34772)=(S=10239744,E=10240767,C=29)
	CT(34773)=(S=10240768,E=10241023,C=28)
	CT(34774)=(S=10241024,E=10241279,C=29)
	CT(34775)=(S=10241280,E=10241535,C=73)
	CT(34776)=(S=10241536,E=10244863,C=29)
	CT(34777)=(S=10244864,E=10245119,C=36)
	CT(34778)=(S=10245120,E=10248447,C=29)
	CT(34779)=(S=10248448,E=10248703,C=203)
	CT(34780)=(S=10248704,E=10250751,C=29)
	CT(34781)=(S=10250752,E=10251007,C=34)
	CT(34782)=(S=10251008,E=10252799,C=29)
	CT(34783)=(S=10252800,E=10253055,C=36)
	CT(34784)=(S=10253056,E=10253311,C=34)
	CT(34785)=(S=10253312,E=10253567,C=27)
	CT(34786)=(S=10253568,E=10253823,C=29)
	CT(34787)=(S=10253824,E=10254079,C=12)
	CT(34788)=(S=10254080,E=10257663,C=29)
	CT(34789)=(S=10257664,E=10257919,C=70)
	CT(34790)=(S=10257920,E=10258175,C=29)
	CT(34791)=(S=10258176,E=10258431,C=34)
	CT(34792)=(S=10258432,E=10258687,C=29)
	CT(34793)=(S=10258688,E=10258943,C=33)
	CT(34794)=(S=10258944,E=10261247,C=29)
	CT(34795)=(S=10261248,E=10261503,C=6)
	CT(34796)=(S=10261504,E=10261759,C=15)
	CT(34797)=(S=10261760,E=10262015,C=73)
	CT(34798)=(S=10262016,E=10262271,C=36)
	CT(34799)=(S=10262272,E=10263295,C=29)
	CT(34800)=(S=10263552,E=10264575,C=112)
	CT(34801)=(S=10264576,E=10280959,C=106)
	CT(34802)=(S=10289152,E=10289407,C=1)
	CT(34803)=(S=10289408,E=10293759,C=2)
	CT(34804)=(S=10293760,E=10294015,C=1)
	CT(34805)=(S=10294016,E=10294527,C=2)
	CT(34806)=(S=10294528,E=10295039,C=29)
	CT(34807)=(S=10295040,E=10295295,C=12)
	CT(34808)=(S=10295296,E=10295551,C=47)
	CT(34809)=(S=10295552,E=10295807,C=58)
	CT(34810)=(S=10295808,E=10296063,C=34)
	CT(34811)=(S=10296064,E=10296831,C=15)
	CT(34812)=(S=10296832,E=10302463,C=29)
	CT(34813)=(S=10302464,E=10302495,C=78)
	CT(34814)=(S=10302496,E=10304767,C=29)
	CT(34815)=(S=10304768,E=10305023,C=1)
	CT(34816)=(S=10305024,E=10305279,C=29)
	CT(34817)=(S=10305280,E=10310399,C=2)
	CT(34818)=(S=10310400,E=10310911,C=14)
	CT(34819)=(S=10310912,E=10311167,C=0)
	CT(34820)=(S=10311168,E=10311423,C=189)
	CT(34821)=(S=10311424,E=10311679,C=29)
	CT(34822)=(S=10311680,E=10311935,C=25)
	CT(34823)=(S=10311936,E=10312703,C=29)
	CT(34824)=(S=10312704,E=10312959,C=83)
	CT(34825)=(S=10312960,E=10313215,C=29)
	CT(34826)=(S=10313216,E=10313471,C=47)
	CT(34827)=(S=10313472,E=10313727,C=29)
	CT(34828)=(S=10313728,E=10313983,C=14)
	CT(34829)=(S=10313984,E=10314239,C=78)
	CT(34830)=(S=10314240,E=10314495,C=29)
	CT(34831)=(S=10314496,E=10314751,C=12)
	CT(34832)=(S=10314752,E=10315007,C=213)
	CT(34833)=(S=10315008,E=10320031,C=2)
	CT(34834)=(S=10320032,E=10320063,C=7)
	CT(34835)=(S=10320064,E=10320095,C=29)
	CT(34836)=(S=10320096,E=10320127,C=2)
	CT(34837)=(S=10320128,E=10320383,C=29)
	CT(34838)=(S=10320384,E=10320639,C=1)
	CT(34839)=(S=10320640,E=10320895,C=29)
	CT(34840)=(S=10320896,E=10321151,C=47)
	CT(34841)=(S=10321152,E=10321407,C=19)
	CT(34842)=(S=10321408,E=10321919,C=29)
	CT(34843)=(S=10321920,E=10322175,C=0)
	CT(34844)=(S=10322176,E=10322431,C=47)
	CT(34845)=(S=10322432,E=10323967,C=29)
	CT(34846)=(S=10323968,E=10324223,C=12)
	CT(34847)=(S=10324224,E=10324479,C=29)
	CT(34848)=(S=10324480,E=10324735,C=15)
	CT(34849)=(S=10324736,E=10324991,C=29)
	CT(34850)=(S=10324992,E=10325247,C=14)
	CT(34851)=(S=10325248,E=10326015,C=29)
	CT(34852)=(S=10326016,E=10326271,C=47)
	CT(34853)=(S=10326272,E=10327039,C=29)
	CT(34854)=(S=10327040,E=10327295,C=1)
	CT(34855)=(S=10327296,E=10328831,C=29)
	CT(34856)=(S=10328832,E=10329087,C=0)
	CT(34857)=(S=10329088,E=10329343,C=1)
	CT(34858)=(S=10329344,E=10329599,C=48)
	CT(34859)=(S=10329600,E=10329855,C=58)
	CT(34860)=(S=10329856,E=10330111,C=12)
	CT(34861)=(S=10330112,E=10330367,C=29)
	CT(34862)=(S=10330368,E=10330623,C=34)
	CT(34863)=(S=10330624,E=10331135,C=28)
	CT(34864)=(S=10331136,E=10331391,C=49)
	CT(34865)=(S=10331392,E=10331903,C=29)
	CT(34866)=(S=10331904,E=10332159,C=42)
	CT(34867)=(S=10332160,E=10332415,C=34)
	CT(34868)=(S=10332416,E=10332671,C=12)
	CT(34869)=(S=10332672,E=10332927,C=29)
	CT(34870)=(S=10332928,E=10333183,C=19)
	CT(34871)=(S=10333184,E=10333439,C=12)
	CT(34872)=(S=10333440,E=10333695,C=14)
	CT(34873)=(S=10333696,E=10334463,C=29)
	CT(34874)=(S=10334464,E=10334719,C=42)
	CT(34875)=(S=10334720,E=10334975,C=29)
	CT(34876)=(S=10334976,E=10335231,C=3)
	CT(34877)=(S=10335232,E=10335487,C=19)
	CT(34878)=(S=10335488,E=10335743,C=50)
	CT(34879)=(S=10335744,E=10336767,C=29)
	CT(34880)=(S=10336768,E=10337023,C=18)
	CT(34881)=(S=10337024,E=10337791,C=29)
	CT(34882)=(S=10337792,E=10338047,C=63)
	CT(34883)=(S=10338048,E=10338303,C=29)
	CT(34884)=(S=10338304,E=10338559,C=2)
	CT(34885)=(S=10338560,E=10338815,C=49)
	CT(34886)=(S=10338816,E=10339583,C=29)
	CT(34887)=(S=10339584,E=10339839,C=6)
	CT(34888)=(S=10339840,E=10340351,C=29)
	CT(34889)=(S=10340352,E=10340607,C=47)
	CT(34890)=(S=10340608,E=10341119,C=29)
	CT(34891)=(S=10341120,E=10341375,C=14)
	CT(34892)=(S=10341376,E=10341631,C=29)
	CT(34893)=(S=10341632,E=10341887,C=2)
	CT(34894)=(S=10341888,E=10343167,C=29)
	CT(34895)=(S=10343168,E=10343423,C=0)
	CT(34896)=(S=10343424,E=10347263,C=29)
	CT(34897)=(S=10347264,E=10347519,C=0)
	CT(34898)=(S=10347520,E=10347775,C=14)
	CT(34899)=(S=10347776,E=10349567,C=29)
	CT(34900)=(S=10349568,E=10349823,C=14)
	CT(34901)=(S=10349824,E=10350079,C=27)
	CT(34902)=(S=10350080,E=10351359,C=29)
	CT(34903)=(S=10351360,E=10351615,C=12)
	CT(34904)=(S=10351616,E=10351871,C=78)
	CT(34905)=(S=10351872,E=10352383,C=29)
	CT(34906)=(S=10352384,E=10352639,C=42)
	CT(34907)=(S=10352640,E=10352895,C=29)
	CT(34908)=(S=10352896,E=10353151,C=27)
	CT(34909)=(S=10353152,E=10353215,C=29)
	CT(34910)=(S=10353216,E=10353407,C=2)
	CT(34911)=(S=10353408,E=10353919,C=29)
	CT(34912)=(S=10353920,E=10354175,C=181)
	CT(34913)=(S=10354176,E=10354431,C=29)
	CT(34914)=(S=10354432,E=10354687,C=1)
	CT(34915)=(S=10354688,E=10363903,C=29)
	CT(34916)=(S=10363904,E=10364927,C=27)
	CT(34917)=(S=10364928,E=10365183,C=0)
	CT(34918)=(S=10365184,E=10365439,C=14)
	CT(34919)=(S=10365440,E=10365695,C=25)
	CT(34920)=(S=10365696,E=10365951,C=14)
	CT(34921)=(S=10365952,E=10366207,C=6)
	CT(34922)=(S=10366208,E=10366463,C=0)
	CT(34923)=(S=10366464,E=10366719,C=18)
	CT(34924)=(S=10366720,E=10366975,C=15)
	CT(34925)=(S=10366976,E=10367231,C=29)
	CT(34926)=(S=10367232,E=10367487,C=25)
	CT(34927)=(S=10367488,E=10367743,C=12)
	CT(34928)=(S=10368000,E=10368511,C=29)
	CT(34929)=(S=10368512,E=10368767,C=0)
	CT(34930)=(S=10368768,E=10369535,C=29)
	CT(34931)=(S=10369536,E=10369663,C=26)
	CT(34932)=(S=10369664,E=10369671,C=18)
	CT(34933)=(S=10369672,E=10369679,C=15)
	CT(34934)=(S=10369680,E=10369687,C=28)
	CT(34935)=(S=10369688,E=10369695,C=27)
	CT(34936)=(S=10369696,E=10369703,C=14)
	CT(34937)=(S=10369704,E=10369711,C=15)
	CT(34938)=(S=10369712,E=10369719,C=12)
	CT(34939)=(S=10369720,E=10369727,C=26)
	CT(34940)=(S=10369728,E=10369791,C=51)
	CT(34941)=(S=10369792,E=10371071,C=29)
	CT(34942)=(S=10371072,E=10371327,C=70)
	CT(34943)=(S=10371328,E=10371583,C=29)
	CT(34944)=(S=10371584,E=10371839,C=58)
	CT(34945)=(S=10371840,E=10372095,C=13)
	CT(34946)=(S=10372096,E=10372351,C=29)
	CT(34947)=(S=10372352,E=10372607,C=78)
	CT(34948)=(S=10372608,E=10373887,C=29)
	CT(34949)=(S=10373888,E=10374143,C=58)
	CT(34950)=(S=10374144,E=10377471,C=29)
	CT(34951)=(S=10377472,E=10377727,C=0)
	CT(34952)=(S=10377728,E=10377983,C=47)
	CT(34953)=(S=10377984,E=10378239,C=29)
	CT(34954)=(S=10378240,E=10378495,C=19)
	CT(34955)=(S=10378496,E=10378751,C=29)
	CT(34956)=(S=10378752,E=10379007,C=14)
	CT(34957)=(S=10379008,E=10379519,C=29)
	CT(34958)=(S=10379520,E=10379775,C=204)
	CT(34959)=(S=10379776,E=10380031,C=29)
	CT(34960)=(S=10380032,E=10380287,C=25)
	CT(34961)=(S=10380288,E=10380799,C=29)
	CT(34962)=(S=10380800,E=10381055,C=15)
	CT(34963)=(S=10381056,E=10381567,C=29)
	CT(34964)=(S=10381568,E=10381823,C=19)
	CT(34965)=(S=10381824,E=10381887,C=29)
	CT(34966)=(S=10381888,E=10381951,C=78)
	CT(34967)=(S=10381952,E=10382047,C=29)
	CT(34968)=(S=10382048,E=10382071,C=78)
	CT(34969)=(S=10382072,E=10382335,C=29)
	CT(34970)=(S=10382336,E=10382591,C=3)
	CT(34971)=(S=10382592,E=10382847,C=25)
	CT(34972)=(S=10382848,E=10383103,C=15)
	CT(34973)=(S=10383104,E=10383359,C=29)
	CT(34974)=(S=10383360,E=10383615,C=27)
	CT(34975)=(S=10383616,E=10385151,C=29)
	CT(34976)=(S=10385152,E=10385407,C=14)
	CT(34977)=(S=10385408,E=10386431,C=29)
	CT(34978)=(S=10386432,E=10386943,C=14)
	CT(34979)=(S=10386944,E=10387199,C=19)
	CT(34980)=(S=10387200,E=10387455,C=47)
	CT(34981)=(S=10387456,E=10387711,C=78)
	CT(34982)=(S=10387712,E=10387967,C=44)
	CT(34983)=(S=10387968,E=10388223,C=29)
	CT(34984)=(S=10388224,E=10388479,C=47)
	CT(34985)=(S=10388480,E=10388735,C=8)
	CT(34986)=(S=10388736,E=10388991,C=34)
	CT(34987)=(S=10388992,E=10390783,C=29)
	CT(34988)=(S=10390784,E=10391039,C=78)
	CT(34989)=(S=10391040,E=10391295,C=29)
	CT(34990)=(S=10391296,E=10391551,C=14)
	CT(34991)=(S=10391552,E=10391807,C=4)
	CT(34992)=(S=10391808,E=10392575,C=29)
	CT(34993)=(S=10392576,E=10392831,C=15)
	CT(34994)=(S=10392832,E=10393343,C=27)
	CT(34995)=(S=10393344,E=10393599,C=29)
	CT(34996)=(S=10393600,E=10393855,C=14)
	CT(34997)=(S=10393856,E=10394623,C=29)
	CT(34998)=(S=10394624,E=10394879,C=12)
	CT(34999)=(S=10394880,E=10395647,C=29)
	CT(35000)=(S=10395648,E=10395903,C=200)
	CT(35001)=(S=10395904,E=10396159,C=157)
	CT(35002)=(S=10396160,E=10396415,C=21)
	CT(35003)=(S=10396416,E=10396671,C=78)
	CT(35004)=(S=10396672,E=10397183,C=29)
	CT(35005)=(S=10397184,E=10398207,C=70)
	CT(35006)=(S=10398208,E=10398463,C=203)
	CT(35007)=(S=10398464,E=10398719,C=29)
	CT(35008)=(S=10398720,E=10398975,C=25)
	CT(35009)=(S=10398976,E=10399231,C=24)
	CT(35010)=(S=10399232,E=10401023,C=14)
	CT(35011)=(S=10401024,E=10401055,C=62)
	CT(35012)=(S=10401056,E=10401071,C=41)
	CT(35013)=(S=10401072,E=10401079,C=28)
	CT(35014)=(S=10401080,E=10401087,C=14)
	CT(35015)=(S=10401088,E=10401119,C=28)
	CT(35016)=(S=10401120,E=10401151,C=34)
	CT(35017)=(S=10401152,E=10401279,C=62)
	CT(35018)=(S=10401280,E=10401535,C=8)
	CT(35019)=(S=10401536,E=10403327,C=29)
	CT(35020)=(S=10403328,E=10404095,C=12)
	CT(35021)=(S=10404096,E=10404351,C=71)
	CT(35022)=(S=10404352,E=10404607,C=33)
	CT(35023)=(S=10404608,E=10404863,C=71)
	CT(35024)=(S=10404864,E=10405119,C=33)
	CT(35025)=(S=10405120,E=10405375,C=71)
	CT(35026)=(S=10405376,E=10409983,C=2)
	CT(35027)=(S=10409984,E=10410239,C=34)
	CT(35028)=(S=10410240,E=10410495,C=2)
	CT(35029)=(S=10410496,E=10410751,C=14)
	CT(35030)=(S=10410752,E=10411007,C=29)
	CT(35031)=(S=10411008,E=10411263,C=34)
	CT(35032)=(S=10411264,E=10411559,C=29)
	CT(35033)=(S=10411560,E=10411561,C=185)
	CT(35034)=(S=10411562,E=10411623,C=29)
	CT(35035)=(S=10411624,E=10411631,C=78)
	CT(35036)=(S=10411632,E=10411775,C=29)
	CT(35037)=(S=10411776,E=10412031,C=14)
	CT(35038)=(S=10412032,E=10412287,C=29)
	CT(35039)=(S=10412288,E=10412543,C=28)
	CT(35040)=(S=10412544,E=10412799,C=34)
	CT(35041)=(S=10412800,E=10413055,C=25)
	CT(35042)=(S=10413056,E=10413567,C=29)
	CT(35043)=(S=10413568,E=10413823,C=14)
	CT(35044)=(S=10413824,E=10414079,C=12)
	CT(35045)=(S=10414080,E=10414335,C=34)
	CT(35046)=(S=10414336,E=10414591,C=47)
	CT(35047)=(S=10414592,E=10414847,C=14)
	CT(35048)=(S=10414848,E=10418175,C=29)
	CT(35049)=(S=10418176,E=10418431,C=19)
	CT(35050)=(S=10418432,E=10418687,C=50)
	CT(35051)=(S=10418688,E=10418943,C=18)
	CT(35052)=(S=10418944,E=10419199,C=203)
	CT(35053)=(S=10419200,E=10419967,C=29)
	CT(35054)=(S=10419968,E=10419975,C=18)
	CT(35055)=(S=10419976,E=10419983,C=14)
	CT(35056)=(S=10419984,E=10420007,C=33)
	CT(35057)=(S=10420008,E=10420015,C=14)
	CT(35058)=(S=10420016,E=10420023,C=18)
	CT(35059)=(S=10420024,E=10420031,C=60)
	CT(35060)=(S=10420032,E=10420047,C=12)
	CT(35061)=(S=10420048,E=10420055,C=18)
	CT(35062)=(S=10420056,E=10420063,C=58)
	CT(35063)=(S=10420064,E=10420095,C=12)
	CT(35064)=(S=10420096,E=10420159,C=18)
	CT(35065)=(S=10420160,E=10420175,C=15)
	CT(35066)=(S=10420176,E=10420183,C=13)
	CT(35067)=(S=10420184,E=10420191,C=19)
	CT(35068)=(S=10420192,E=10420199,C=14)
	CT(35069)=(S=10420200,E=10420207,C=25)
	CT(35070)=(S=10420208,E=10420215,C=15)
	CT(35071)=(S=10420216,E=10420223,C=71)
	CT(35072)=(S=10420224,E=10420479,C=23)
	CT(35073)=(S=10420480,E=10421759,C=29)
	CT(35074)=(S=10421760,E=10422015,C=78)
	CT(35075)=(S=10422016,E=10422271,C=19)
	CT(35076)=(S=10422272,E=10423295,C=29)
	CT(35077)=(S=10423296,E=10423551,C=34)
	CT(35078)=(S=10423552,E=10423807,C=0)
	CT(35079)=(S=10423808,E=10424063,C=29)
	CT(35080)=(S=10424064,E=10424319,C=14)
	CT(35081)=(S=10424320,E=10424575,C=204)
	CT(35082)=(S=10424576,E=10424831,C=29)
	CT(35083)=(S=10424832,E=10425087,C=78)
	CT(35084)=(S=10425088,E=10425343,C=29)
	CT(35085)=(S=10425344,E=10425351,C=24)
	CT(35086)=(S=10425352,E=10425359,C=12)
	CT(35087)=(S=10425360,E=10425367,C=59)
	CT(35088)=(S=10425368,E=10425375,C=72)
	CT(35089)=(S=10425376,E=10425407,C=14)
	CT(35090)=(S=10425408,E=10425439,C=37)
	CT(35091)=(S=10425440,E=10425455,C=26)
	CT(35092)=(S=10425456,E=10425463,C=37)
	CT(35093)=(S=10425464,E=10425471,C=25)
	CT(35094)=(S=10425472,E=10425599,C=15)
	CT(35095)=(S=10425600,E=10425855,C=29)
	CT(35096)=(S=10425856,E=10426111,C=34)
	CT(35097)=(S=10426112,E=10426623,C=29)
	CT(35098)=(S=10426624,E=10426879,C=28)
	CT(35099)=(S=10426880,E=10427647,C=29)
	CT(35100)=(S=10427648,E=10427903,C=34)
	CT(35101)=(S=10427904,E=10428159,C=14)
	CT(35102)=(S=10428160,E=10428415,C=12)
	CT(35103)=(S=10428416,E=10428927,C=78)
	CT(35104)=(S=10428928,E=10429183,C=14)
	CT(35105)=(S=10429184,E=10429951,C=29)
	CT(35106)=(S=10429952,E=10430207,C=19)
	CT(35107)=(S=10430208,E=10431999,C=29)
	CT(35108)=(S=10432000,E=10432255,C=36)
	CT(35109)=(S=10432256,E=10433023,C=29)
	CT(35110)=(S=10433024,E=10433279,C=12)
	CT(35111)=(S=10433280,E=10433535,C=28)
	CT(35112)=(S=10433536,E=10433791,C=0)
	CT(35113)=(S=10433792,E=10434303,C=29)
	CT(35114)=(S=10434304,E=10434559,C=78)
	CT(35115)=(S=10434560,E=10435071,C=29)
	CT(35116)=(S=10435072,E=10435327,C=27)
	CT(35117)=(S=10435328,E=10435583,C=29)
	CT(35118)=(S=10435584,E=10435839,C=36)
	CT(35119)=(S=10435840,E=10438655,C=29)
	CT(35120)=(S=10438656,E=10438911,C=19)
	CT(35121)=(S=10438912,E=10439167,C=0)
	CT(35122)=(S=10439168,E=10440703,C=29)
	CT(35123)=(S=10440704,E=10440959,C=2)
	CT(35124)=(S=10440960,E=10441215,C=27)
	CT(35125)=(S=10441216,E=10441727,C=29)
	CT(35126)=(S=10441728,E=10441983,C=12)
	CT(35127)=(S=10441984,E=10442239,C=29)
	CT(35128)=(S=10442240,E=10442495,C=14)
	CT(35129)=(S=10442496,E=10443007,C=29)
	CT(35130)=(S=10443008,E=10443263,C=78)
	CT(35131)=(S=10443264,E=10443519,C=200)
	CT(35132)=(S=10443520,E=10443775,C=29)
	CT(35133)=(S=10443776,E=10444031,C=14)
	CT(35134)=(S=10444032,E=10444287,C=18)
	CT(35135)=(S=10444288,E=10444543,C=29)
	CT(35136)=(S=10444544,E=10444799,C=12)
	CT(35137)=(S=10444800,E=10445823,C=29)
	CT(35138)=(S=10445824,E=10446335,C=14)
	CT(35139)=(S=10446336,E=10446591,C=29)
	CT(35140)=(S=10446592,E=10446847,C=34)
	CT(35141)=(S=10446848,E=10447615,C=29)
	CT(35142)=(S=10447616,E=10447871,C=19)
	CT(35143)=(S=10447872,E=10448127,C=29)
	CT(35144)=(S=10448128,E=10448383,C=49)
	CT(35145)=(S=10448384,E=10448639,C=29)
	CT(35146)=(S=10448640,E=10448895,C=0)
	CT(35147)=(S=10448896,E=10449407,C=29)
	CT(35148)=(S=10449408,E=10449663,C=14)
	CT(35149)=(S=10449664,E=10450175,C=29)
	CT(35150)=(S=10450176,E=10450431,C=73)
	CT(35151)=(S=10450432,E=10452991,C=29)
	CT(35152)=(S=10452992,E=10453247,C=78)
	CT(35153)=(S=10453248,E=10453503,C=29)
	CT(35154)=(S=10453504,E=10453759,C=27)
	CT(35155)=(S=10453760,E=10454527,C=29)
	CT(35156)=(S=10454528,E=10454783,C=63)
	CT(35157)=(S=10454784,E=10457087,C=29)
	CT(35158)=(S=10457088,E=10457343,C=34)
	CT(35159)=(S=10457344,E=10457599,C=29)
	CT(35160)=(S=10457600,E=10457727,C=37)
	CT(35161)=(S=10457728,E=10457855,C=31)
	CT(35162)=(S=10457856,E=10458111,C=14)
	CT(35163)=(S=10458112,E=10458367,C=60)
	CT(35164)=(S=10458368,E=10458623,C=15)
	CT(35165)=(S=10458624,E=10458879,C=29)
	CT(35166)=(S=10458880,E=10459135,C=12)
	CT(35167)=(S=10459136,E=10459391,C=47)
	CT(35168)=(S=10459392,E=10459647,C=29)
	CT(35169)=(S=10459648,E=10459903,C=28)
	CT(35170)=(S=10459904,E=10460159,C=29)
	CT(35171)=(S=10460160,E=10460415,C=34)
	CT(35172)=(S=10460416,E=10460927,C=29)
	CT(35173)=(S=10460928,E=10461183,C=78)
	CT(35174)=(S=10461184,E=10461439,C=31)
	CT(35175)=(S=10461440,E=10461695,C=29)
	CT(35176)=(S=10461696,E=10461951,C=27)
	CT(35177)=(S=10461952,E=10462975,C=29)
	CT(35178)=(S=10462976,E=10463231,C=14)
	CT(35179)=(S=10463232,E=10463487,C=34)
	CT(35180)=(S=10463488,E=10463743,C=29)
	CT(35181)=(S=10463744,E=10463999,C=14)
	CT(35182)=(S=10464000,E=10464255,C=27)
	CT(35183)=(S=10464256,E=10464511,C=29)
	CT(35184)=(S=10464512,E=10464767,C=12)
	CT(35185)=(S=10464768,E=10465023,C=29)
	CT(35186)=(S=10465024,E=10465279,C=78)
	CT(35187)=(S=10465280,E=10465535,C=29)
	CT(35188)=(S=10465536,E=10465791,C=78)
	CT(35189)=(S=10465792,E=10466047,C=29)
	CT(35190)=(S=10466048,E=10466303,C=34)
	CT(35191)=(S=10466304,E=10466559,C=14)
	CT(35192)=(S=10466560,E=10468863,C=29)
	CT(35193)=(S=10468864,E=10469119,C=19)
	CT(35194)=(S=10469120,E=10469375,C=29)
	CT(35195)=(S=10469376,E=10469631,C=3)
	CT(35196)=(S=10469632,E=10470143,C=19)
	CT(35197)=(S=10470144,E=10470399,C=24)
	CT(35198)=(S=10470400,E=10470655,C=0)
	CT(35199)=(S=10470656,E=10470911,C=14)
	CT(35200)=(S=10470912,E=10471423,C=29)
	CT(35201)=(S=10471424,E=10471679,C=19)
	CT(35202)=(S=10471680,E=10472703,C=29)
	CT(35203)=(S=10472704,E=10472959,C=58)
	CT(35204)=(S=10472960,E=10473215,C=78)
	CT(35205)=(S=10473216,E=10473471,C=0)
	CT(35206)=(S=10473472,E=10473727,C=78)
	CT(35207)=(S=10473728,E=10473983,C=29)
	CT(35208)=(S=10473984,E=10474239,C=15)
	CT(35209)=(S=10474240,E=10474495,C=2)
	CT(35210)=(S=10474496,E=10474751,C=29)
	CT(35211)=(S=10474752,E=10475007,C=15)
	CT(35212)=(S=10475008,E=10475519,C=29)
	CT(35213)=(S=10475520,E=10475775,C=27)
	CT(35214)=(S=10475776,E=10476031,C=12)
	CT(35215)=(S=10476032,E=10476287,C=29)
	CT(35216)=(S=10476288,E=10476543,C=14)
	CT(35217)=(S=10476544,E=10477567,C=29)
	CT(35218)=(S=10477568,E=10477823,C=31)
	CT(35219)=(S=10477824,E=10478079,C=29)
	CT(35220)=(S=10478080,E=10478335,C=1)
	CT(35221)=(S=10478336,E=10479359,C=29)
	CT(35222)=(S=10479360,E=10479615,C=78)
	CT(35223)=(S=10479616,E=10479871,C=34)
	CT(35224)=(S=10479872,E=10480639,C=29)
	CT(35225)=(S=10480640,E=10480895,C=14)
	CT(35226)=(S=10480896,E=10481151,C=25)
	CT(35227)=(S=10481152,E=10481407,C=29)
	CT(35228)=(S=10481408,E=10481663,C=14)
	CT(35229)=(S=10481664,E=10481919,C=29)
	CT(35230)=(S=10481920,E=10482175,C=34)
	CT(35231)=(S=10482176,E=10482687,C=29)
	CT(35232)=(S=10482688,E=10482943,C=14)
	CT(35233)=(S=10482944,E=10483199,C=13)
	CT(35234)=(S=10483200,E=10483711,C=29)
	CT(35235)=(S=10483712,E=10483967,C=0)
	CT(35236)=(S=10483968,E=10484991,C=29)
	CT(35237)=(S=10484992,E=10484999,C=36)
	CT(35238)=(S=10485000,E=10485007,C=27)
	CT(35239)=(S=10485008,E=10485015,C=55)
	CT(35240)=(S=10485016,E=10485023,C=19)
	CT(35241)=(S=10485024,E=10485039,C=37)
	CT(35242)=(S=10485040,E=10485071,C=14)
	CT(35243)=(S=10485072,E=10485079,C=37)
	CT(35244)=(S=10485080,E=10485087,C=24)
	CT(35245)=(S=10485088,E=10485095,C=15)
	CT(35246)=(S=10485096,E=10485103,C=71)
	CT(35247)=(S=10485104,E=10485111,C=28)
	CT(35248)=(S=10485112,E=10485119,C=18)
	CT(35249)=(S=10485120,E=10485151,C=36)
	CT(35250)=(S=10485152,E=10485159,C=14)
	CT(35251)=(S=10485160,E=10485167,C=18)
	CT(35252)=(S=10485168,E=10485175,C=14)
	CT(35253)=(S=10485176,E=10485183,C=15)
	CT(35254)=(S=10485184,E=10485207,C=14)
	CT(35255)=(S=10485208,E=10485215,C=19)
	CT(35256)=(S=10485216,E=10485223,C=49)
	CT(35257)=(S=10485224,E=10485231,C=33)
	CT(35258)=(S=10485232,E=10485239,C=58)
	CT(35259)=(S=10485240,E=10485247,C=17)
	CT(35260)=(S=10485248,E=10485503,C=14)
	CT(35261)=(S=10485504,E=10485535,C=18)
	CT(35262)=(S=10485536,E=10485543,C=26)
	CT(35263)=(S=10485544,E=10485551,C=32)
	CT(35264)=(S=10485552,E=10485567,C=21)
	CT(35265)=(S=10485568,E=10485631,C=18)
	CT(35266)=(S=10485632,E=10485663,C=15)
	CT(35267)=(S=10485664,E=10485671,C=35)
	CT(35268)=(S=10485672,E=10485679,C=28)
	CT(35269)=(S=10485680,E=10485695,C=58)
	CT(35270)=(S=10485696,E=10485703,C=25)
	CT(35271)=(S=10485704,E=10485719,C=14)
	CT(35272)=(S=10485720,E=10485727,C=36)
	CT(35273)=(S=10485728,E=10485759,C=47)
	CT(35274)=(S=10486016,E=10486783,C=29)
	CT(35275)=(S=10486784,E=10487039,C=73)
	CT(35276)=(S=10487040,E=10487295,C=14)
	CT(35277)=(S=10487296,E=10487551,C=63)
	CT(35278)=(S=10487552,E=10487807,C=29)
	CT(35279)=(S=10487808,E=10488063,C=19)
	CT(35280)=(S=10488064,E=10488319,C=14)
	CT(35281)=(S=10488320,E=10488575,C=29)
	CT(35282)=(S=10488576,E=10493439,C=2)
	CT(35283)=(S=10493696,E=10495487,C=29)
	CT(35284)=(S=10495488,E=10495743,C=14)
	CT(35285)=(S=10495744,E=10495999,C=29)
	CT(35286)=(S=10496000,E=10496255,C=22)
	CT(35287)=(S=10496256,E=10497023,C=29)
	CT(35288)=(S=10497024,E=10499071,C=28)
	CT(35289)=(S=10499072,E=10499327,C=42)
	CT(35290)=(S=10499328,E=10499583,C=34)
	CT(35291)=(S=10499584,E=10500863,C=28)
	CT(35292)=(S=10500864,E=10501119,C=34)
	CT(35293)=(S=10501120,E=10501375,C=28)
	CT(35294)=(S=10501376,E=10502143,C=34)
	CT(35295)=(S=10502144,E=10502399,C=0)
	CT(35296)=(S=10502400,E=10502655,C=29)
	CT(35297)=(S=10502656,E=10502911,C=15)
	CT(35298)=(S=10502912,E=10503423,C=27)
	CT(35299)=(S=10503424,E=10503679,C=29)
	CT(35300)=(S=10503680,E=10503935,C=28)
	CT(35301)=(S=10503936,E=10504191,C=47)
	CT(35302)=(S=10504192,E=10504447,C=78)
	CT(35303)=(S=10504448,E=10504703,C=29)
	CT(35304)=(S=10504704,E=10504959,C=2)
	CT(35305)=(S=10504960,E=10505215,C=37)
	CT(35306)=(S=10505216,E=10505471,C=29)
	CT(35307)=(S=10505728,E=10505983,C=15)
	CT(35308)=(S=10505984,E=10506239,C=29)
	CT(35309)=(S=10506240,E=10506495,C=15)
	CT(35310)=(S=10506496,E=10507519,C=29)
	CT(35311)=(S=10507520,E=10507775,C=34)
	CT(35312)=(S=10507776,E=10509311,C=29)
	CT(35313)=(S=10509312,E=10509567,C=12)
	CT(35314)=(S=10509568,E=10510335,C=29)
	CT(35315)=(S=10510336,E=10510591,C=7)
	CT(35316)=(S=10510592,E=10510847,C=15)
	CT(35317)=(S=10510848,E=10511103,C=34)
	CT(35318)=(S=10511104,E=10511359,C=46)
	CT(35319)=(S=10511360,E=10511615,C=14)
	CT(35320)=(S=10511616,E=10512127,C=29)
	CT(35321)=(S=10512128,E=10512383,C=12)
	CT(35322)=(S=10512384,E=10512639,C=14)
	CT(35323)=(S=10512896,E=10513151,C=78)
	CT(35324)=(S=10513152,E=10514687,C=29)
	CT(35325)=(S=10514944,E=10515199,C=50)
	CT(35326)=(S=10515200,E=10517759,C=105)
	CT(35327)=(S=10517760,E=10525183,C=29)
	CT(35328)=(S=10525696,E=10526719,C=152)
	CT(35329)=(S=10526720,E=10531839,C=134)
	CT(35330)=(S=10531840,E=10532095,C=15)
	CT(35331)=(S=10532096,E=10533119,C=105)
	CT(35332)=(S=10533120,E=10538239,C=2)
	CT(35333)=(S=10538240,E=10538495,C=29)
	CT(35334)=(S=10538496,E=10538751,C=0)
	CT(35335)=(S=10538752,E=10539519,C=29)
	CT(35336)=(S=10539520,E=10539775,C=48)
	CT(35337)=(S=10539776,E=10540031,C=28)
	CT(35338)=(S=10540032,E=10540287,C=29)
	CT(35339)=(S=10540288,E=10540543,C=14)
	CT(35340)=(S=10540544,E=10540799,C=25)
	CT(35341)=(S=10540800,E=10541055,C=2)
	CT(35342)=(S=10541056,E=10541823,C=33)
	CT(35343)=(S=10541824,E=10542079,C=34)
	CT(35344)=(S=10542080,E=10542335,C=15)
	CT(35345)=(S=10542336,E=10542591,C=49)
	CT(35346)=(S=10542592,E=10542847,C=0)
	CT(35347)=(S=10542848,E=10543103,C=78)
	CT(35348)=(S=10543104,E=10543615,C=116)
	CT(35349)=(S=10543616,E=10543743,C=111)
	CT(35350)=(S=10543872,E=10544127,C=29)
	CT(35351)=(S=10544128,E=10544383,C=12)
	CT(35352)=(S=10544384,E=10545407,C=29)
	CT(35353)=(S=10545408,E=10547711,C=2)
	CT(35354)=(S=10547712,E=10547839,C=117)
	CT(35355)=(S=10547840,E=10547903,C=130)
	CT(35356)=(S=10547968,E=10550527,C=2)
	CT(35357)=(S=10550528,E=10551039,C=29)
	CT(35358)=(S=10551040,E=10551295,C=105)
	CT(35359)=(S=10551296,E=10551327,C=208)
	CT(35360)=(S=10551328,E=10551359,C=216)
	CT(35361)=(S=10551360,E=10551363,C=83)
	CT(35362)=(S=10551364,E=10551367,C=208)
	CT(35363)=(S=10551368,E=10551375,C=83)
	CT(35364)=(S=10551376,E=10551391,C=217)
	CT(35365)=(S=10551392,E=10551407,C=207)
	CT(35366)=(S=10551408,E=10551415,C=176)
	CT(35367)=(S=10551416,E=10551423,C=205)
	CT(35368)=(S=10551424,E=10551455,C=218)
	CT(35369)=(S=10551456,E=10551471,C=203)
	CT(35370)=(S=10551472,E=10551479,C=83)
	CT(35371)=(S=10551480,E=10551487,C=203)
	CT(35372)=(S=10551488,E=10551519,C=208)
	CT(35373)=(S=10551520,E=10551551,C=176)
	CT(35374)=(S=10551552,E=10551807,C=29)
	CT(35375)=(S=10551808,E=10552063,C=14)
	CT(35376)=(S=10552064,E=10552319,C=12)
	CT(35377)=(S=10552320,E=10552575,C=27)
	CT(35378)=(S=10552576,E=10552831,C=42)
	CT(35379)=(S=10552832,E=10553343,C=29)
	CT(35380)=(S=10553344,E=10553599,C=18)
	CT(35381)=(S=10553600,E=10553855,C=37)
	CT(35382)=(S=10553856,E=10554111,C=181)
	CT(35383)=(S=10554112,E=10554367,C=29)
	CT(35384)=(S=10554368,E=10554623,C=14)
	CT(35385)=(S=10554624,E=10555647,C=29)
	CT(35386)=(S=10555648,E=10555903,C=14)
	CT(35387)=(S=10555904,E=10556159,C=181)
	CT(35388)=(S=10556160,E=10556415,C=78)
	CT(35389)=(S=10556416,E=10556671,C=34)
	CT(35390)=(S=10556672,E=10556927,C=29)
	CT(35391)=(S=10556928,E=10556959,C=83)
	CT(35392)=(S=10556960,E=10556975,C=200)
	CT(35393)=(S=10556976,E=10556983,C=207)
	CT(35394)=(S=10556984,E=10556991,C=189)
	CT(35395)=(S=10556992,E=10557055,C=222)
	CT(35396)=(S=10557056,E=10557183,C=201)
	CT(35397)=(S=10557184,E=10557439,C=14)
	CT(35398)=(S=10557440,E=10557695,C=189)
	CT(35399)=(S=10557696,E=10557951,C=203)
	CT(35400)=(S=10557952,E=10558207,C=29)
	CT(35401)=(S=10558208,E=10558463,C=15)
	CT(35402)=(S=10558464,E=10558719,C=29)
	CT(35403)=(S=10558720,E=10558975,C=0)
	CT(35404)=(S=10558976,E=10559231,C=14)
	CT(35405)=(S=10559232,E=10560767,C=29)
	CT(35406)=(S=10560768,E=10561023,C=25)
	CT(35407)=(S=10561024,E=10561791,C=29)
	CT(35408)=(S=10561792,E=10562047,C=47)
	CT(35409)=(S=10562048,E=10562303,C=28)
	CT(35410)=(S=10562304,E=10562559,C=0)
	CT(35411)=(S=10562560,E=10563583,C=29)
	CT(35412)=(S=10563584,E=10563839,C=12)
	CT(35413)=(S=10563840,E=10564095,C=29)
	CT(35414)=(S=10564096,E=10564351,C=0)
	CT(35415)=(S=10564352,E=10564607,C=29)
	CT(35416)=(S=10564608,E=10564863,C=19)
	CT(35417)=(S=10564864,E=10565119,C=57)
	CT(35418)=(S=10565120,E=10565375,C=19)
	CT(35419)=(S=10565376,E=10565631,C=29)
	CT(35420)=(S=10565632,E=10565887,C=201)
	CT(35421)=(S=10565888,E=10566399,C=29)
	CT(35422)=(S=10566400,E=10566655,C=49)
	CT(35423)=(S=10566656,E=10567167,C=29)
	CT(35424)=(S=10567168,E=10567423,C=34)
	CT(35425)=(S=10567424,E=10567679,C=29)
	CT(35426)=(S=10567680,E=10567935,C=87)
	CT(35427)=(S=10567936,E=10567967,C=0)
	CT(35428)=(S=10567968,E=10567999,C=73)
	CT(35429)=(S=10568000,E=10568095,C=0)
	CT(35430)=(S=10568096,E=10568127,C=73)
	CT(35431)=(S=10568128,E=10568191,C=0)
	CT(35432)=(S=10568192,E=10568447,C=73)
	CT(35433)=(S=10568448,E=10568703,C=25)
	CT(35434)=(S=10568704,E=10569471,C=29)
	CT(35435)=(S=10569472,E=10569727,C=14)
	CT(35436)=(S=10569728,E=10569983,C=25)
	CT(35437)=(S=10569984,E=10570495,C=14)
	CT(35438)=(S=10570496,E=10570751,C=2)
	CT(35439)=(S=10570752,E=10571007,C=14)
	CT(35440)=(S=10571008,E=10571263,C=29)
	CT(35441)=(S=10571264,E=10571519,C=34)
	CT(35442)=(S=10571520,E=10571775,C=189)
	CT(35443)=(S=10571776,E=10572543,C=29)
	CT(35444)=(S=10572544,E=10575103,C=36)
	CT(35445)=(S=10575104,E=10576127,C=2)
	CT(35446)=(S=10576128,E=10577919,C=29)
	CT(35447)=(S=10577920,E=10578687,C=12)
	CT(35448)=(S=10578688,E=10579455,C=29)
	CT(35449)=(S=10579456,E=10579711,C=42)
	CT(35450)=(S=10579712,E=10579967,C=25)
	CT(35451)=(S=10579968,E=10580223,C=14)
	CT(35452)=(S=10580224,E=10580991,C=29)
	CT(35453)=(S=10580992,E=10581247,C=25)
	CT(35454)=(S=10581248,E=10581503,C=0)
	CT(35455)=(S=10581504,E=10581759,C=2)
	CT(35456)=(S=10581760,E=10582527,C=29)
	CT(35457)=(S=10582528,E=10582783,C=6)
	CT(35458)=(S=10583040,E=10583551,C=29)
	CT(35459)=(S=10583552,E=10583807,C=7)
	CT(35460)=(S=10583808,E=10584831,C=29)
	CT(35461)=(S=10584832,E=10585087,C=203)
	CT(35462)=(S=10585088,E=10585343,C=212)
	CT(35463)=(S=10585344,E=10585599,C=29)
	CT(35464)=(S=10585600,E=10585855,C=28)
	CT(35465)=(S=10585856,E=10586623,C=29)
	CT(35466)=(S=10586624,E=10586879,C=201)
	CT(35467)=(S=10586880,E=10587135,C=5)
	CT(35468)=(S=10587136,E=10587391,C=200)
	CT(35469)=(S=10587392,E=10587647,C=78)
	CT(35470)=(S=10587648,E=10587903,C=5)
	CT(35471)=(S=10587904,E=10588159,C=0)
	CT(35472)=(S=10588160,E=10588671,C=29)
	CT(35473)=(S=10588672,E=10588927,C=0)
	CT(35474)=(S=10588928,E=10589183,C=15)
	CT(35475)=(S=10589184,E=10589439,C=189)
	CT(35476)=(S=10589440,E=10590207,C=29)
	CT(35477)=(S=10590208,E=10590463,C=0)
	CT(35478)=(S=10590464,E=10591743,C=29)
	CT(35479)=(S=10591744,E=10591999,C=36)
	CT(35480)=(S=10592000,E=10598399,C=29)
	CT(35481)=(S=10598400,E=10598655,C=78)
	CT(35482)=(S=10598656,E=10599167,C=29)
	CT(35483)=(S=10599168,E=10599423,C=78)
	CT(35484)=(S=10599424,E=10599935,C=29)
	CT(35485)=(S=10599936,E=10600191,C=83)
	CT(35486)=(S=10600192,E=10601471,C=29)
	CT(35487)=(S=10601472,E=10601727,C=200)
	CT(35488)=(S=10601728,E=10602495,C=29)
	CT(35489)=(S=10602496,E=10602751,C=3)
	CT(35490)=(S=10602752,E=10604287,C=29)
	CT(35491)=(S=10604288,E=10604543,C=1)
	CT(35492)=(S=10604544,E=10605567,C=29)
	CT(35493)=(S=10605568,E=10605823,C=200)
	CT(35494)=(S=10605824,E=10606591,C=29)
	CT(35495)=(S=10606592,E=10606847,C=78)
	CT(35496)=(S=10606848,E=10607103,C=29)
	CT(35497)=(S=10607104,E=10607359,C=28)
	CT(35498)=(S=10607360,E=10609919,C=29)
	CT(35499)=(S=10609920,E=10610175,C=7)
	CT(35500)=(S=10610176,E=10611199,C=29)
	CT(35501)=(S=10611200,E=10611455,C=200)
	CT(35502)=(S=10611456,E=10612223,C=29)
	CT(35503)=(S=10612224,E=10612479,C=203)
	CT(35504)=(S=10612480,E=10614271,C=29)
	CT(35505)=(S=10614272,E=10614527,C=3)
	CT(35506)=(S=10614528,E=10615807,C=29)
	CT(35507)=(S=10615808,E=10616063,C=101)
	CT(35508)=(S=10616064,E=10616575,C=29)
	CT(35509)=(S=10616576,E=10616831,C=200)
	CT(35510)=(S=10616832,E=10617087,C=78)
	CT(35511)=(S=10617088,E=10619647,C=29)
	CT(35512)=(S=10619648,E=10619903,C=14)
	CT(35513)=(S=10619904,E=10620159,C=29)
	CT(35514)=(S=10620160,E=10620415,C=14)
	CT(35515)=(S=10620416,E=10622207,C=29)
	CT(35516)=(S=10622208,E=10622463,C=34)
	CT(35517)=(S=10622464,E=10622719,C=29)
	CT(35518)=(S=10622720,E=10622975,C=34)
	CT(35519)=(S=10622976,E=10623231,C=29)
	CT(35520)=(S=10623232,E=10623487,C=42)
	CT(35521)=(S=10623488,E=10623743,C=34)
	CT(35522)=(S=10623744,E=10626559,C=29)
	CT(35523)=(S=10626560,E=10626815,C=12)
	CT(35524)=(S=10626816,E=10630399,C=29)
	CT(35525)=(S=10630400,E=10630655,C=78)
	CT(35526)=(S=10630656,E=10631167,C=29)
	CT(35527)=(S=10631168,E=10631423,C=4)
	CT(35528)=(S=10631424,E=10634751,C=29)
	CT(35529)=(S=10634752,E=10635007,C=78)
	CT(35530)=(S=10635008,E=10635519,C=29)
	CT(35531)=(S=10635520,E=10635775,C=78)
	CT(35532)=(S=10635776,E=10638591,C=29)
	CT(35533)=(S=10638592,E=10638847,C=78)
	CT(35534)=(S=10638848,E=10639103,C=34)
	CT(35535)=(S=10639104,E=10643711,C=29)
	CT(35536)=(S=10643712,E=10643967,C=1)
	CT(35537)=(S=10643968,E=10644223,C=78)
	CT(35538)=(S=10644224,E=10645503,C=29)
	CT(35539)=(S=10645504,E=10645759,C=73)
	CT(35540)=(S=10645760,E=10648063,C=29)
	CT(35541)=(S=10648064,E=10648319,C=200)
	CT(35542)=(S=10648320,E=10652415,C=29)
	CT(35543)=(S=10652416,E=10652671,C=78)
	CT(35544)=(S=10652672,E=10653951,C=29)
	CT(35545)=(S=10653952,E=10654719,C=0)
	CT(35546)=(S=10654720,E=10656767,C=29)
	CT(35547)=(S=10656768,E=10657279,C=78)
	CT(35548)=(S=10657280,E=10670091,C=29)
	CT(35549)=(S=10670092,E=10670095,C=78)
	CT(35550)=(S=10670096,E=10670115,C=29)
	CT(35551)=(S=10670116,E=10670119,C=78)
	CT(35552)=(S=10670120,E=10670139,C=29)
	CT(35553)=(S=10670140,E=10670143,C=78)
	CT(35554)=(S=10670144,E=10670159,C=29)
	CT(35555)=(S=10670160,E=10670163,C=78)
	CT(35556)=(S=10670164,E=10670167,C=29)
	CT(35557)=(S=10670168,E=10670171,C=78)
	CT(35558)=(S=10670172,E=10670179,C=29)
	CT(35559)=(S=10670180,E=10670183,C=78)
	CT(35560)=(S=10670184,E=10670207,C=29)
	CT(35561)=(S=10670208,E=10670335,C=78)
	CT(35562)=(S=10670336,E=10670599,C=29)
	CT(35563)=(S=10670600,E=10670603,C=78)
	CT(35564)=(S=10670604,E=10670623,C=29)
	CT(35565)=(S=10670624,E=10670627,C=78)
	CT(35566)=(S=10670628,E=10670631,C=29)
	CT(35567)=(S=10670632,E=10670635,C=80)
	CT(35568)=(S=10670636,E=10670647,C=29)
	CT(35569)=(S=10670648,E=10670651,C=78)
	CT(35570)=(S=10670652,E=10670659,C=29)
	CT(35571)=(S=10670660,E=10670663,C=95)
	CT(35572)=(S=10670664,E=10670747,C=29)
	CT(35573)=(S=10670748,E=10670751,C=183)
	CT(35574)=(S=10670752,E=10670759,C=78)
	CT(35575)=(S=10670760,E=10670771,C=29)
	CT(35576)=(S=10670772,E=10670775,C=78)
	CT(35577)=(S=10670776,E=10670815,C=29)
	CT(35578)=(S=10670816,E=10670819,C=78)
	CT(35579)=(S=10670820,E=10670943,C=29)
	CT(35580)=(S=10670944,E=10670951,C=78)
	CT(35581)=(S=10670952,E=10670959,C=29)
	CT(35582)=(S=10670960,E=10670967,C=78)
	CT(35583)=(S=10670968,E=10670983,C=29)
	CT(35584)=(S=10670984,E=10670987,C=174)
	CT(35585)=(S=10670988,E=10671031,C=29)
	CT(35586)=(S=10671032,E=10671035,C=78)
	CT(35587)=(S=10671036,E=10671039,C=29)
	CT(35588)=(S=10671040,E=10671047,C=78)
	CT(35589)=(S=10671048,E=10671055,C=29)
	CT(35590)=(S=10671056,E=10671063,C=78)
	CT(35591)=(S=10671064,E=10671071,C=29)
	CT(35592)=(S=10671072,E=10671083,C=78)
	CT(35593)=(S=10671088,E=10671095,C=29)
	CT(35594)=(S=10671096,E=10671099,C=78)
	CT(35595)=(S=10671100,E=10671111,C=29)
	CT(35596)=(S=10671112,E=10671115,C=78)
	CT(35597)=(S=10671116,E=10671119,C=79)
	CT(35598)=(S=10671120,E=10671123,C=29)
	CT(35599)=(S=10671124,E=10671127,C=78)
	CT(35600)=(S=10671128,E=10671167,C=29)
	CT(35601)=(S=10671168,E=10671175,C=78)
	CT(35602)=(S=10671176,E=10671203,C=29)
	CT(35603)=(S=10671204,E=10671207,C=78)
	CT(35604)=(S=10671208,E=10671215,C=29)
	CT(35605)=(S=10671216,E=10671219,C=78)
	CT(35606)=(S=10671220,E=10671263,C=29)
	CT(35607)=(S=10671264,E=10671271,C=80)
	CT(35608)=(S=10671272,E=10671303,C=29)
	CT(35609)=(S=10671304,E=10671311,C=78)
	CT(35610)=(S=10671312,E=10671315,C=82)
	CT(35611)=(S=10671316,E=10671335,C=29)
	CT(35612)=(S=10671336,E=10671343,C=78)
	CT(35613)=(S=10671344,E=10671367,C=29)
	CT(35614)=(S=10671368,E=10671371,C=78)
	CT(35615)=(S=10671372,E=10671463,C=29)
	CT(35616)=(S=10671464,E=10671471,C=78)
	CT(35617)=(S=10671472,E=10671495,C=29)
	CT(35618)=(S=10671496,E=10671499,C=78)
	CT(35619)=(S=10671500,E=10671515,C=29)
	CT(35620)=(S=10671516,E=10671519,C=78)
	CT(35621)=(S=10671520,E=10671527,C=29)
	CT(35622)=(S=10671528,E=10671531,C=182)
	CT(35623)=(S=10671532,E=10672211,C=29)
	CT(35624)=(S=10672212,E=10672215,C=78)
	CT(35625)=(S=10672216,E=10672287,C=29)
	CT(35626)=(S=10672288,E=10672295,C=175)
	CT(35627)=(S=10672296,E=10672311,C=29)
	CT(35628)=(S=10672312,E=10672319,C=78)
	CT(35629)=(S=10672320,E=10672615,C=29)
	CT(35630)=(S=10672616,E=10672627,C=78)
	CT(35631)=(S=10672628,E=10672663,C=29)
	CT(35632)=(S=10672664,E=10672667,C=95)
	CT(35633)=(S=10672668,E=10672671,C=29)
	CT(35634)=(S=10672672,E=10672679,C=78)
	CT(35635)=(S=10672680,E=10672763,C=29)
	CT(35636)=(S=10672764,E=10672767,C=78)
	CT(35637)=(S=10672768,E=10672895,C=29)
	CT(35638)=(S=10672896,E=10672911,C=78)
	CT(35639)=(S=10672912,E=10672943,C=29)
	CT(35640)=(S=10672944,E=10672951,C=78)
	CT(35641)=(S=10672952,E=10672963,C=29)
	CT(35642)=(S=10672964,E=10672967,C=78)
	CT(35643)=(S=10672968,E=10673055,C=29)
	CT(35644)=(S=10673056,E=10673059,C=78)
	CT(35645)=(S=10673060,E=10673071,C=29)
	CT(35646)=(S=10673072,E=10673075,C=78)
	CT(35647)=(S=10673076,E=10673079,C=29)
	CT(35648)=(S=10673080,E=10673087,C=78)
	CT(35649)=(S=10673088,E=10673127,C=29)
	CT(35650)=(S=10673128,E=10673128,C=78)
	CT(35651)=(S=10673129,E=10673129,C=178)
	CT(35652)=(S=10673130,E=10673131,C=78)
	CT(35653)=(S=10673132,E=10673132,C=29)
	CT(35654)=(S=10673133,E=10673135,C=78)
	CT(35655)=(S=10673136,E=10673187,C=29)
	CT(35656)=(S=10673188,E=10673191,C=78)
	CT(35657)=(S=10673192,E=10673247,C=29)
	CT(35658)=(S=10673248,E=10673255,C=80)
	CT(35659)=(S=10673256,E=10673287,C=29)
	CT(35660)=(S=10673288,E=10673295,C=79)
	CT(35661)=(S=10673296,E=10673339,C=29)
	CT(35662)=(S=10673340,E=10673343,C=78)
	CT(35663)=(S=10673344,E=10673355,C=29)
	CT(35664)=(S=10673356,E=10673359,C=78)
	CT(35665)=(S=10673360,E=10673399,C=29)
	CT(35666)=(S=10673400,E=10673403,C=78)
	CT(35667)=(S=10673404,E=10673419,C=29)
	CT(35668)=(S=10673420,E=10673423,C=78)
	CT(35669)=(S=10673424,E=10673527,C=29)
	CT(35670)=(S=10673528,E=10673535,C=78)
	CT(35671)=(S=10673536,E=10673607,C=29)
	CT(35672)=(S=10673608,E=10673615,C=78)
	CT(35673)=(S=10673616,E=10673619,C=29)
	CT(35674)=(S=10673620,E=10673623,C=178)
	CT(35675)=(S=10673624,E=10673655,C=29)
	CT(35676)=(S=10673656,E=10673663,C=78)
	CT(35677)=(S=10673664,E=10673719,C=29)
	CT(35678)=(S=10673720,E=10673727,C=78)
	CT(35679)=(S=10673728,E=10673731,C=29)
	CT(35680)=(S=10673732,E=10673735,C=78)
	CT(35681)=(S=10673736,E=10673743,C=29)
	CT(35682)=(S=10673744,E=10673747,C=78)
	CT(35683)=(S=10673748,E=10673751,C=183)
	CT(35684)=(S=10673752,E=10673771,C=29)
	CT(35685)=(S=10673772,E=10673775,C=80)
	CT(35686)=(S=10673776,E=10673783,C=29)
	CT(35687)=(S=10673784,E=10673803,C=78)
	CT(35688)=(S=10673804,E=10673807,C=29)
	CT(35689)=(S=10673808,E=10673811,C=78)
	CT(35690)=(S=10673812,E=10673819,C=29)
	CT(35691)=(S=10673820,E=10673827,C=78)
	CT(35692)=(S=10673828,E=10673887,C=29)
	CT(35693)=(S=10673888,E=10673891,C=95)
	CT(35694)=(S=10673892,E=10673919,C=29)
	CT(35695)=(S=10673920,E=10673927,C=78)
	CT(35696)=(S=10673928,E=10673963,C=29)
	CT(35697)=(S=10673964,E=10673971,C=78)
	CT(35698)=(S=10673972,E=10674019,C=29)
	CT(35699)=(S=10674020,E=10674023,C=78)
	CT(35700)=(S=10674024,E=10674143,C=29)
	CT(35701)=(S=10674144,E=10674151,C=78)
	CT(35702)=(S=10674152,E=10679311,C=29)
	CT(35703)=(S=10679312,E=10679315,C=80)
	CT(35704)=(S=10679316,E=10679327,C=78)
	CT(35705)=(S=10679328,E=10679339,C=29)
	CT(35706)=(S=10679340,E=10679343,C=78)
	CT(35707)=(S=10679344,E=10679383,C=29)
	CT(35708)=(S=10679384,E=10679387,C=78)
	CT(35709)=(S=10679388,E=10679415,C=29)
	CT(35710)=(S=10679416,E=10679419,C=78)
	CT(35711)=(S=10679420,E=10679463,C=29)
	CT(35712)=(S=10679464,E=10679467,C=80)
	CT(35713)=(S=10679468,E=10679503,C=29)
	CT(35714)=(S=10679504,E=10679507,C=78)
	CT(35715)=(S=10679508,E=10679523,C=29)
	CT(35716)=(S=10679524,E=10679527,C=78)
	CT(35717)=(S=10679528,E=10679543,C=29)
	CT(35718)=(S=10679544,E=10679547,C=78)
	CT(35719)=(S=10679548,E=10679607,C=29)
	CT(35720)=(S=10679608,E=10679611,C=78)
	CT(35721)=(S=10679612,E=10679627,C=29)
	CT(35722)=(S=10679628,E=10679631,C=82)
	CT(35723)=(S=10679632,E=10679695,C=29)
	CT(35724)=(S=10679696,E=10679699,C=78)
	CT(35725)=(S=10679700,E=10679703,C=29)
	CT(35726)=(S=10679704,E=10679707,C=186)
	CT(35727)=(S=10679708,E=10679759,C=29)
	CT(35728)=(S=10679760,E=10679767,C=78)
	CT(35729)=(S=10679768,E=10679791,C=29)
	CT(35730)=(S=10679792,E=10679807,C=78)
	CT(35731)=(S=10679808,E=10679811,C=175)
	CT(35732)=(S=10679812,E=10679815,C=29)
	CT(35733)=(S=10679816,E=10679823,C=78)
	CT(35734)=(S=10679824,E=10679859,C=29)
	CT(35735)=(S=10679860,E=10679863,C=78)
	CT(35736)=(S=10679864,E=10679911,C=29)
	CT(35737)=(S=10679912,E=10679919,C=79)
	CT(35738)=(S=10679920,E=10679935,C=29)
	CT(35739)=(S=10679936,E=10679939,C=78)
	CT(35740)=(S=10679940,E=10679963,C=29)
	CT(35741)=(S=10679964,E=10679967,C=78)
	CT(35742)=(S=10679968,E=10679979,C=29)
	CT(35743)=(S=10679980,E=10679983,C=80)
	CT(35744)=(S=10679984,E=10680075,C=29)
	CT(35745)=(S=10680076,E=10680087,C=78)
	CT(35746)=(S=10680088,E=10680095,C=180)
	CT(35747)=(S=10680096,E=10680099,C=29)
	CT(35748)=(S=10680100,E=10680103,C=78)
	CT(35749)=(S=10680104,E=10680155,C=29)
	CT(35750)=(S=10680156,E=10680159,C=78)
	CT(35751)=(S=10680160,E=10680167,C=95)
	CT(35752)=(S=10680168,E=10680175,C=29)
	CT(35753)=(S=10680176,E=10680191,C=78)
	CT(35754)=(S=10680192,E=10680231,C=29)
	CT(35755)=(S=10680232,E=10680235,C=78)
	CT(35756)=(S=10680236,E=10680283,C=29)
	CT(35757)=(S=10680284,E=10680287,C=174)
	CT(35758)=(S=10680288,E=10680295,C=78)
	CT(35759)=(S=10680296,E=10680327,C=29)
	CT(35760)=(S=10680328,E=10680331,C=78)
	CT(35761)=(S=10680332,E=10680351,C=29)
	CT(35762)=(S=10680352,E=10680359,C=78)
	CT(35763)=(S=10680360,E=10680363,C=29)
	CT(35764)=(S=10680364,E=10680367,C=80)
	CT(35765)=(S=10680368,E=10680439,C=29)
	CT(35766)=(S=10680440,E=10680447,C=78)
	CT(35767)=(S=10680448,E=10680459,C=29)
	CT(35768)=(S=10680460,E=10680463,C=78)
	CT(35769)=(S=10680464,E=10680471,C=29)
	CT(35770)=(S=10680472,E=10680491,C=78)
	CT(35771)=(S=10680492,E=10680495,C=29)
	CT(35772)=(S=10680496,E=10680503,C=78)
	CT(35773)=(S=10680504,E=10680583,C=29)
	CT(35774)=(S=10680584,E=10680587,C=78)
	CT(35775)=(S=10680588,E=10680615,C=29)
	CT(35776)=(S=10680616,E=10680619,C=78)
	CT(35777)=(S=10680620,E=10680663,C=29)
	CT(35778)=(S=10680664,E=10680671,C=78)
	CT(35779)=(S=10680672,E=10680703,C=29)
	CT(35780)=(S=10680704,E=10680711,C=174)
	CT(35781)=(S=10680712,E=10680735,C=29)
	CT(35782)=(S=10680736,E=10680739,C=78)
	CT(35783)=(S=10680740,E=10680811,C=29)
	CT(35784)=(S=10680812,E=10680815,C=78)
	CT(35785)=(S=10680816,E=10680843,C=29)
	CT(35786)=(S=10680844,E=10680847,C=78)
	CT(35787)=(S=10680848,E=10680875,C=29)
	CT(35788)=(S=10680876,E=10680879,C=78)
	CT(35789)=(S=10680880,E=10680999,C=29)
	CT(35790)=(S=10681000,E=10681007,C=78)
	CT(35791)=(S=10681008,E=10681019,C=29)
	CT(35792)=(S=10681020,E=10681031,C=78)
	CT(35793)=(S=10681032,E=10681051,C=29)
	CT(35794)=(S=10681052,E=10681055,C=78)
	CT(35795)=(S=10681056,E=10681103,C=29)
	CT(35796)=(S=10681104,E=10681107,C=78)
	CT(35797)=(S=10681108,E=10681111,C=95)
	CT(35798)=(S=10681112,E=10681147,C=29)
	CT(35799)=(S=10681148,E=10681151,C=78)
	CT(35800)=(S=10681152,E=10681159,C=29)
	CT(35801)=(S=10681160,E=10681167,C=78)
	CT(35802)=(S=10681168,E=10681175,C=95)
	CT(35803)=(S=10681176,E=10681195,C=29)
	CT(35804)=(S=10681196,E=10681199,C=197)
	CT(35805)=(S=10681200,E=10681207,C=29)
	CT(35806)=(S=10681208,E=10681211,C=78)
	CT(35807)=(S=10681212,E=10681243,C=29)
	CT(35808)=(S=10681244,E=10681247,C=80)
	CT(35809)=(S=10681248,E=10681279,C=29)
	CT(35810)=(S=10681280,E=10681287,C=78)
	CT(35811)=(S=10681288,E=10681387,C=29)
	CT(35812)=(S=10681388,E=10681391,C=78)
	CT(35813)=(S=10681392,E=10681435,C=29)
	CT(35814)=(S=10681436,E=10681439,C=78)
	CT(35815)=(S=10681440,E=10681455,C=29)
	CT(35816)=(S=10681456,E=10681463,C=78)
	CT(35817)=(S=10681464,E=10681475,C=29)
	CT(35818)=(S=10681476,E=10681479,C=78)
	CT(35819)=(S=10681480,E=10681527,C=29)
	CT(35820)=(S=10681528,E=10681531,C=78)
	CT(35821)=(S=10681532,E=10681535,C=183)
	CT(35822)=(S=10681536,E=10681559,C=29)
	CT(35823)=(S=10681560,E=10681567,C=78)
	CT(35824)=(S=10681568,E=10681583,C=29)
	CT(35825)=(S=10681584,E=10681587,C=78)
	CT(35826)=(S=10681588,E=10681595,C=29)
	CT(35827)=(S=10681596,E=10681599,C=78)
	CT(35828)=(S=10681600,E=10681607,C=29)
	CT(35829)=(S=10681608,E=10681611,C=78)
	CT(35830)=(S=10681612,E=10681651,C=29)
	CT(35831)=(S=10681652,E=10681655,C=78)
	CT(35832)=(S=10681656,E=10681699,C=29)
	CT(35833)=(S=10681700,E=10681703,C=182)
	CT(35834)=(S=10681704,E=10681727,C=29)
	CT(35835)=(S=10681728,E=10681731,C=78)
	CT(35836)=(S=10681732,E=10681735,C=29)
	CT(35837)=(S=10681736,E=10681743,C=78)
	CT(35838)=(S=10681744,E=10681767,C=29)
	CT(35839)=(S=10681768,E=10681771,C=180)
	CT(35840)=(S=10681772,E=10681807,C=29)
	CT(35841)=(S=10681808,E=10681815,C=78)
	CT(35842)=(S=10681816,E=10681847,C=29)
	CT(35843)=(S=10681848,E=10681855,C=78)
	CT(35844)=(S=10681856,E=10681895,C=29)
	CT(35845)=(S=10681896,E=10681899,C=78)
	CT(35846)=(S=10681900,E=10681987,C=29)
	CT(35847)=(S=10681988,E=10681991,C=78)
	CT(35848)=(S=10681992,E=10682027,C=29)
	CT(35849)=(S=10682028,E=10682031,C=78)
	CT(35850)=(S=10682032,E=10682043,C=29)
	CT(35851)=(S=10682044,E=10682047,C=197)
	CT(35852)=(S=10682048,E=10682163,C=29)
	CT(35853)=(S=10682164,E=10682167,C=78)
	CT(35854)=(S=10682168,E=10682223,C=29)
	CT(35855)=(S=10682224,E=10682227,C=78)
	CT(35856)=(S=10682228,E=10682255,C=29)
	CT(35857)=(S=10682256,E=10682259,C=95)
	CT(35858)=(S=10682260,E=10682263,C=78)
	CT(35859)=(S=10682264,E=10682287,C=29)
	CT(35860)=(S=10682288,E=10682291,C=78)
	CT(35861)=(S=10682292,E=10682315,C=29)
	CT(35862)=(S=10682316,E=10682327,C=78)
	CT(35863)=(S=10682328,E=10682335,C=178)
	CT(35864)=(S=10682336,E=10682339,C=29)
	CT(35865)=(S=10682340,E=10682343,C=78)
	CT(35866)=(S=10682344,E=10682359,C=29)
	CT(35867)=(S=10682360,E=10682367,C=78)
	CT(35868)=(S=10682368,E=10682623,C=1)
	CT(35869)=(S=10682624,E=10682879,C=14)
	CT(35870)=(S=10682880,E=10683135,C=29)
	CT(35871)=(S=10683136,E=10683391,C=19)
	CT(35872)=(S=10683392,E=10683647,C=29)
	CT(35873)=(S=10683648,E=10683903,C=12)
	CT(35874)=(S=10683904,E=10684159,C=29)
	CT(35875)=(S=10684160,E=10684415,C=73)
	CT(35876)=(S=10684416,E=10684671,C=0)
	CT(35877)=(S=10684672,E=10684927,C=12)
	CT(35878)=(S=10684928,E=10685183,C=83)
	CT(35879)=(S=10685184,E=10685695,C=29)
	CT(35880)=(S=10685696,E=10690815,C=9)
	CT(35881)=(S=10690816,E=10691071,C=29)
	CT(35882)=(S=10691072,E=10691327,C=27)
	CT(35883)=(S=10691328,E=10693119,C=29)
	CT(35884)=(S=10693120,E=10694403,C=2)
	CT(35885)=(S=10694404,E=10694407,C=1)
	CT(35886)=(S=10694408,E=10694411,C=0)
	CT(35887)=(S=10694412,E=10694415,C=4)
	CT(35888)=(S=10694416,E=10694418,C=0)
	CT(35889)=(S=10694419,E=10694419,C=2)
	CT(35890)=(S=10694420,E=10694421,C=0)
	CT(35891)=(S=10694423,E=10694429,C=2)
	CT(35892)=(S=10694432,E=10694439,C=84)
	CT(35893)=(S=10694440,E=10694443,C=0)
	CT(35894)=(S=10694444,E=10694445,C=8)
	CT(35895)=(S=10694447,E=10694475,C=0)
	CT(35896)=(S=10694476,E=10694487,C=84)
	CT(35897)=(S=10694488,E=10694491,C=5)
	CT(35898)=(S=10694492,E=10694503,C=0)
	CT(35899)=(S=10694504,E=10694507,C=73)
	CT(35900)=(S=10694508,E=10694511,C=0)
	CT(35901)=(S=10694512,E=10694515,C=73)
	CT(35902)=(S=10694516,E=10694527,C=0)
	CT(35903)=(S=10694528,E=10694531,C=73)
	CT(35904)=(S=10694532,E=10694539,C=0)
	CT(35905)=(S=10694540,E=10694543,C=4)
	CT(35906)=(S=10694544,E=10694547,C=84)
	CT(35907)=(S=10694548,E=10694551,C=75)
	CT(35908)=(S=10694552,E=10694555,C=4)
	CT(35909)=(S=10694556,E=10694559,C=84)
	CT(35910)=(S=10694560,E=10694563,C=89)
	CT(35911)=(S=10694564,E=10694567,C=73)
	CT(35912)=(S=10694568,E=10694571,C=0)
	CT(35913)=(S=10694572,E=10694575,C=86)
	CT(35914)=(S=10694576,E=10694579,C=7)
	CT(35915)=(S=10694580,E=10694583,C=0)
	CT(35916)=(S=10694584,E=10694591,C=73)
	CT(35917)=(S=10694592,E=10694595,C=5)
	CT(35918)=(S=10694596,E=10694599,C=0)
	CT(35919)=(S=10694600,E=10694603,C=5)
	CT(35920)=(S=10694604,E=10694607,C=73)
	CT(35921)=(S=10694608,E=10694615,C=4)
	CT(35922)=(S=10694616,E=10694619,C=0)
	CT(35923)=(S=10694620,E=10694623,C=73)
	CT(35924)=(S=10694624,E=10694627,C=90)
	CT(35925)=(S=10694628,E=10694647,C=73)
	CT(35926)=(S=10694648,E=10694651,C=90)
	CT(35927)=(S=10694652,E=10694655,C=0)
	CT(35928)=(S=10694656,E=10695679,C=2)
	CT(35929)=(S=10695936,E=10695951,C=1)
	CT(35930)=(S=10695952,E=10695955,C=8)
	CT(35931)=(S=10695956,E=10695959,C=0)
	CT(35932)=(S=10695960,E=10695963,C=75)
	CT(35933)=(S=10695964,E=10695967,C=86)
	CT(35934)=(S=10695968,E=10695971,C=0)
	CT(35935)=(S=10695972,E=10696003,C=1)
	CT(35936)=(S=10696004,E=10696007,C=0)
	CT(35937)=(S=10696012,E=10696023,C=4)
	CT(35938)=(S=10696024,E=10696075,C=1)
	CT(35939)=(S=10696076,E=10696079,C=84)
	CT(35940)=(S=10696080,E=10696083,C=0)
	CT(35941)=(S=10696084,E=10696087,C=84)
	CT(35942)=(S=10696088,E=10696091,C=5)
	CT(35943)=(S=10696092,E=10696095,C=6)
	CT(35944)=(S=10696096,E=10696111,C=1)
	CT(35945)=(S=10696112,E=10696115,C=4)
	CT(35946)=(S=10696116,E=10696119,C=84)
	CT(35947)=(S=10696120,E=10696123,C=74)
	CT(35948)=(S=10696124,E=10696127,C=1)
	CT(35949)=(S=10696128,E=10696131,C=74)
	CT(35950)=(S=10696132,E=10696135,C=86)
	CT(35951)=(S=10696136,E=10696147,C=4)
	CT(35952)=(S=10696152,E=10696155,C=8)
	CT(35953)=(S=10696156,E=10696159,C=1)
	CT(35954)=(S=10696160,E=10696163,C=8)
	CT(35955)=(S=10696164,E=10696167,C=0)
	CT(35956)=(S=10696168,E=10696171,C=73)
	CT(35957)=(S=10696172,E=10696175,C=77)
	CT(35958)=(S=10696176,E=10696179,C=1)
	CT(35959)=(S=10696180,E=10696183,C=87)
	CT(35960)=(S=10696184,E=10696187,C=0)
	CT(35961)=(S=10696188,E=10696191,C=4)
	CT(35962)=(S=10696192,E=10697983,C=2)
	CT(35963)=(S=10698240,E=10712319,C=12)
	CT(35964)=(S=10712320,E=10712575,C=25)
	CT(35965)=(S=10712576,E=10712831,C=29)
	CT(35966)=(S=10712832,E=10713087,C=14)
	CT(35967)=(S=10713088,E=10713343,C=29)
	CT(35968)=(S=10713344,E=10713599,C=106)
	CT(35969)=(S=10713600,E=10713855,C=0)
	CT(35970)=(S=10713856,E=10713983,C=78)
	CT(35971)=(S=10714112,E=10714367,C=29)
	CT(35972)=(S=10714368,E=10714623,C=1)
	CT(35973)=(S=10714624,E=10715647,C=29)
	CT(35974)=(S=10715648,E=10718719,C=2)
	CT(35975)=(S=10718720,E=10718975,C=1)
	CT(35976)=(S=10718976,E=10720767,C=2)
	CT(35977)=(S=10720768,E=10721279,C=29)
	CT(35978)=(S=10721280,E=10721535,C=6)
	CT(35979)=(S=10721536,E=10722047,C=29)
	CT(35980)=(S=10722048,E=10722303,C=78)
	CT(35981)=(S=10722304,E=10722559,C=14)
	CT(35982)=(S=10722560,E=10722815,C=19)
	CT(35983)=(S=10722816,E=10723071,C=28)
	CT(35984)=(S=10723072,E=10723327,C=52)
	CT(35985)=(S=10723328,E=10723583,C=14)
	CT(35986)=(S=10723584,E=10723839,C=34)
	CT(35987)=(S=10723840,E=10724095,C=15)
	CT(35988)=(S=10724096,E=10724351,C=49)
	CT(35989)=(S=10724352,E=10724607,C=14)
	CT(35990)=(S=10724608,E=10724799,C=49)
	CT(35991)=(S=10724800,E=10724863,C=28)
	CT(35992)=(S=10724864,E=10725375,C=14)
	CT(35993)=(S=10725376,E=10725631,C=34)
	CT(35994)=(S=10725632,E=10726655,C=14)
	CT(35995)=(S=10726656,E=10726911,C=12)
	CT(35996)=(S=10726912,E=10727167,C=27)
	CT(35997)=(S=10727168,E=10727423,C=36)
	CT(35998)=(S=10727424,E=10727679,C=29)
	CT(35999)=(S=10727680,E=10727935,C=1)
	CT(36000)=(S=10727936,E=10728191,C=202)
	CT(36001)=(S=10728192,E=10728447,C=1)
	CT(36002)=(S=10728448,E=10728703,C=6)
	CT(36003)=(S=10728704,E=10730751,C=29)
	CT(36004)=(S=10730752,E=10731007,C=0)
	CT(36005)=(S=10731008,E=10732287,C=29)
	CT(36006)=(S=10732288,E=10734591,C=105)
	CT(36007)=(S=10734592,E=10734847,C=1)
	CT(36008)=(S=10734848,E=10735615,C=29)
	CT(36009)=(S=10735616,E=10736895,C=2)
	CT(36010)=(S=10736896,E=10737151,C=6)
	CT(36011)=(S=10737152,E=10739199,C=2)
	CT(36012)=(S=10739200,E=10739455,C=6)
	CT(36013)=(S=10739456,E=10740734,C=2)
	CT(36014)=(S=10740736,E=10740991,C=29)
	CT(36015)=(S=10740992,E=10741247,C=6)
	CT(36016)=(S=10741248,E=10741759,C=29)
	CT(36017)=(S=10741760,E=10742271,C=0)
	CT(36018)=(S=10742272,E=10743551,C=29)
	CT(36019)=(S=10743552,E=10743807,C=6)
	CT(36020)=(S=10743808,E=10744319,C=29)
	CT(36021)=(S=10744320,E=10744575,C=28)
	CT(36022)=(S=10744576,E=10745599,C=29)
	CT(36023)=(S=10745600,E=10745855,C=203)
	CT(36024)=(S=10745856,E=10746367,C=29)
	CT(36025)=(S=10746368,E=10746623,C=203)
	CT(36026)=(S=10746624,E=10747647,C=29)
	CT(36027)=(S=10747648,E=10747903,C=6)
	CT(36028)=(S=10747904,E=10748159,C=20)
	CT(36029)=(S=10748160,E=10748671,C=12)
	CT(36030)=(S=10748672,E=10748927,C=42)
	CT(36031)=(S=10748928,E=10749183,C=19)
	CT(36032)=(S=10749184,E=10749439,C=47)
	CT(36033)=(S=10749440,E=10749695,C=14)
	CT(36034)=(S=10749696,E=10749951,C=12)
	CT(36035)=(S=10749952,E=10750207,C=52)
	CT(36036)=(S=10750208,E=10750719,C=19)
	CT(36037)=(S=10750720,E=10751231,C=14)
	CT(36038)=(S=10751232,E=10751487,C=47)
	CT(36039)=(S=10751488,E=10751743,C=34)
	CT(36040)=(S=10751744,E=10751999,C=49)
	CT(36041)=(S=10752000,E=10756863,C=28)
	CT(36042)=(S=10756864,E=10757119,C=49)
	CT(36043)=(S=10757120,E=10757375,C=14)
	CT(36044)=(S=10757376,E=10757631,C=19)
	CT(36045)=(S=10757632,E=10758143,C=14)
	CT(36046)=(S=10758144,E=10758271,C=20)
	CT(36047)=(S=10758272,E=10758303,C=36)
	CT(36048)=(S=10758304,E=10758311,C=28)
	CT(36049)=(S=10758312,E=10758319,C=71)
	CT(36050)=(S=10758320,E=10758327,C=19)
	CT(36051)=(S=10758336,E=10758351,C=28)
	CT(36052)=(S=10758352,E=10758367,C=14)
	CT(36053)=(S=10758368,E=10758375,C=59)
	CT(36054)=(S=10758376,E=10758383,C=28)
	CT(36055)=(S=10758384,E=10758391,C=58)
	CT(36056)=(S=10758392,E=10758399,C=54)
	CT(36057)=(S=10758400,E=10758655,C=189)
	CT(36058)=(S=10758656,E=10758911,C=80)
	CT(36059)=(S=10758912,E=10759167,C=2)
	CT(36060)=(S=10759168,E=10759679,C=29)
	CT(36061)=(S=10759680,E=10759935,C=2)
	CT(36062)=(S=10759936,E=10760191,C=29)
	CT(36063)=(S=10760192,E=10760447,C=19)
	CT(36064)=(S=10760448,E=10761471,C=29)
	CT(36065)=(S=10761472,E=10761727,C=0)
	CT(36066)=(S=10761728,E=10763007,C=29)
	CT(36067)=(S=10763008,E=10763263,C=28)
	CT(36068)=(S=10763264,E=10763519,C=12)
	CT(36069)=(S=10763520,E=10763775,C=28)
	CT(36070)=(S=10763776,E=10765567,C=29)
	CT(36071)=(S=10765568,E=10766335,C=2)
	CT(36072)=(S=10766336,E=10766591,C=29)
	CT(36073)=(S=10766592,E=10766847,C=205)
	CT(36074)=(S=10766848,E=10767103,C=29)
	CT(36075)=(S=10767104,E=10767359,C=0)
	CT(36076)=(S=10767360,E=10767615,C=29)
	CT(36077)=(S=10767616,E=10767871,C=203)
	CT(36078)=(S=10767872,E=10768127,C=7)
	CT(36079)=(S=10768128,E=10768383,C=29)
	CT(36080)=(S=10768384,E=10768639,C=0)
	CT(36081)=(S=10768640,E=10768895,C=12)
	CT(36082)=(S=10768896,E=10769663,C=29)
	CT(36083)=(S=10769664,E=10769919,C=189)
	CT(36084)=(S=10769920,E=10770431,C=29)
	CT(36085)=(S=10770432,E=10770687,C=105)
	CT(36086)=(S=10770688,E=10772479,C=29)
	CT(36087)=(S=10772480,E=10772735,C=203)
	CT(36088)=(S=10772736,E=10772991,C=0)
	CT(36089)=(S=10772992,E=10773247,C=203)
	CT(36090)=(S=10773248,E=10773503,C=29)
	CT(36091)=(S=10773504,E=10773759,C=4)
	CT(36092)=(S=10773760,E=10774015,C=29)
	CT(36093)=(S=10774016,E=10774271,C=78)
	CT(36094)=(S=10774272,E=10775551,C=29)
	CT(36095)=(S=10775552,E=10775807,C=0)
	CT(36096)=(S=10775808,E=10776575,C=29)
	CT(36097)=(S=10776576,E=10776831,C=0)
	CT(36098)=(S=10776832,E=10777343,C=29)
	CT(36099)=(S=10777344,E=10777599,C=3)
	CT(36100)=(S=10777600,E=10779647,C=29)
	CT(36101)=(S=10779648,E=10780159,C=6)
	CT(36102)=(S=10780160,E=10780671,C=58)
	CT(36103)=(S=10780672,E=10780927,C=34)
	CT(36104)=(S=10780928,E=10781183,C=12)
	CT(36105)=(S=10781184,E=10781439,C=15)
	CT(36106)=(S=10781440,E=10781695,C=12)
	CT(36107)=(S=10781696,E=10781951,C=15)
	CT(36108)=(S=10781952,E=10782207,C=28)
	CT(36109)=(S=10782208,E=10782463,C=14)
	CT(36110)=(S=10782464,E=10782719,C=19)
	CT(36111)=(S=10782720,E=10783231,C=14)
	CT(36112)=(S=10783232,E=10783247,C=18)
	CT(36113)=(S=10783248,E=10783255,C=26)
	CT(36114)=(S=10783256,E=10783263,C=36)
	CT(36115)=(S=10783264,E=10783271,C=18)
	CT(36116)=(S=10783272,E=10783279,C=20)
	CT(36117)=(S=10783280,E=10783287,C=15)
	CT(36118)=(S=10783288,E=10783295,C=12)
	CT(36119)=(S=10783296,E=10783311,C=34)
	CT(36120)=(S=10783312,E=10783319,C=14)
	CT(36121)=(S=10783320,E=10783327,C=18)
	CT(36122)=(S=10783336,E=10783343,C=49)
	CT(36123)=(S=10783344,E=10783359,C=17)
	CT(36124)=(S=10783360,E=10783423,C=26)
	CT(36125)=(S=10783424,E=10783431,C=28)
	CT(36126)=(S=10783432,E=10783447,C=25)
	CT(36127)=(S=10783448,E=10783455,C=51)
	CT(36128)=(S=10783456,E=10783463,C=14)
	CT(36129)=(S=10783464,E=10783471,C=35)
	CT(36130)=(S=10783472,E=10783479,C=12)
	CT(36131)=(S=10783480,E=10783487,C=36)
	CT(36132)=(S=10783488,E=10783743,C=28)
	CT(36133)=(S=10783744,E=10783999,C=36)
	CT(36134)=(S=10784000,E=10784255,C=47)
	CT(36135)=(S=10784256,E=10784511,C=15)
	CT(36136)=(S=10784512,E=10784767,C=14)
	CT(36137)=(S=10784768,E=10785279,C=29)
	CT(36138)=(S=10785280,E=10786815,C=105)
	CT(36139)=(S=10786816,E=10787583,C=29)
	CT(36140)=(S=10787584,E=10787839,C=105)
	CT(36141)=(S=10787840,E=10788863,C=29)
	CT(36142)=(S=10789120,E=10789631,C=2)
	CT(36143)=(S=10789888,E=10790143,C=4)
	CT(36144)=(S=10790144,E=10790399,C=29)
	CT(36145)=(S=10790400,E=10790655,C=78)
	CT(36146)=(S=10790656,E=10793215,C=29)
	CT(36147)=(S=10793216,E=10793343,C=12)
	CT(36148)=(S=10793344,E=10793375,C=14)
	CT(36149)=(S=10793376,E=10793391,C=28)
	CT(36150)=(S=10793392,E=10793399,C=18)
	CT(36151)=(S=10793400,E=10793407,C=14)
	CT(36152)=(S=10793408,E=10793471,C=68)
	CT(36153)=(S=10793472,E=10793727,C=78)
	CT(36154)=(S=10793728,E=10802943,C=29)
	CT(36155)=(S=10802944,E=10802975,C=32)
	CT(36156)=(S=10802976,E=10802983,C=47)
	CT(36157)=(S=10802984,E=10802991,C=26)
	CT(36158)=(S=10802992,E=10802999,C=18)
	CT(36159)=(S=10803000,E=10803007,C=26)
	CT(36160)=(S=10803008,E=10803039,C=18)
	CT(36161)=(S=10803040,E=10803047,C=41)
	CT(36162)=(S=10803048,E=10803055,C=14)
	CT(36163)=(S=10803056,E=10803071,C=33)
	CT(36164)=(S=10803072,E=10803199,C=26)
	CT(36165)=(S=10803200,E=10814207,C=29)
	CT(36166)=(S=10814208,E=10814975,C=105)
	CT(36167)=(S=10814976,E=10815487,C=29)
	CT(36168)=(S=10815488,E=10816511,C=105)
	CT(36169)=(S=10816512,E=10816767,C=0)
	CT(36170)=(S=10816768,E=10817023,C=29)
	CT(36171)=(S=10817024,E=10817279,C=2)
	CT(36172)=(S=10818304,E=10818815,C=29)
	CT(36173)=(S=10818816,E=10819071,C=7)
	CT(36174)=(S=10819072,E=10819327,C=29)
	CT(36175)=(S=10819328,E=10819583,C=78)
	CT(36176)=(S=10819584,E=10819839,C=29)
	CT(36177)=(S=10819840,E=10820095,C=105)
	CT(36178)=(S=10820096,E=10821375,C=29)
	CT(36179)=(S=10821376,E=10821631,C=0)
	CT(36180)=(S=10821632,E=10824191,C=29)
	CT(36181)=(S=10824192,E=10824703,C=4)
	CT(36182)=(S=10824704,E=10825983,C=29)
	CT(36183)=(S=10826240,E=10826751,C=152)
	CT(36184)=(S=10826752,E=10827775,C=105)
	CT(36185)=(S=10827776,E=10829823,C=114)
	CT(36186)=(S=10829824,E=10831103,C=29)
	CT(36187)=(S=10831104,E=10831359,C=0)
	CT(36188)=(S=10831360,E=10831871,C=29)
	CT(36189)=(S=10831872,E=10832127,C=33)
	CT(36190)=(S=10832384,E=10832895,C=29)
	CT(36191)=(S=10832896,E=10833151,C=2)
	CT(36192)=(S=10833152,E=10834943,C=29)
	CT(36193)=(S=10834944,E=10835135,C=73)
	CT(36194)=(S=10835136,E=10835151,C=4)
	CT(36195)=(S=10835152,E=10835175,C=73)
	CT(36196)=(S=10835176,E=10835183,C=5)
	CT(36197)=(S=10835184,E=10835199,C=4)
	CT(36198)=(S=10835200,E=10835455,C=29)
	CT(36199)=(S=10835456,E=10835711,C=0)
	CT(36200)=(S=10835712,E=10837247,C=29)
	CT(36201)=(S=10837248,E=10837503,C=2)
	CT(36202)=(S=10837504,E=10838015,C=29)
	CT(36203)=(S=10838016,E=10838271,C=2)
	CT(36204)=(S=10838272,E=10838527,C=29)
	CT(36205)=(S=10838528,E=10838783,C=216)
	CT(36206)=(S=10838784,E=10839807,C=2)
	CT(36207)=(S=10839808,E=10841087,C=29)
	CT(36208)=(S=10841088,E=10841343,C=2)
	CT(36209)=(S=10841344,E=10842623,C=29)
	CT(36210)=(S=10842624,E=10842879,C=36)
	CT(36211)=(S=10842880,E=10843135,C=78)
	CT(36212)=(S=10843136,E=10843647,C=29)
	CT(36213)=(S=10843648,E=10843903,C=0)
	CT(36214)=(S=10843904,E=10847231,C=29)
	CT(36215)=(S=10847232,E=10847743,C=6)
	CT(36216)=(S=10847744,E=10849535,C=29)
	CT(36217)=(S=10849536,E=10849791,C=6)
	CT(36218)=(S=10849792,E=10850047,C=0)
	CT(36219)=(S=10850048,E=10851839,C=105)
	CT(36220)=(S=10851840,E=10852863,C=29)
	CT(36221)=(S=10852864,E=10853119,C=78)
	CT(36222)=(S=10853120,E=10855679,C=29)
	CT(36223)=(S=10855680,E=10855935,C=105)
	CT(36224)=(S=10855936,E=10856703,C=29)
	CT(36225)=(S=10856704,E=10856959,C=151)
	CT(36226)=(S=10856960,E=10859007,C=29)
	CT(36227)=(S=10859008,E=10859263,C=78)
	CT(36228)=(S=10859264,E=10859519,C=29)
	CT(36229)=(S=10859520,E=10859775,C=105)
	CT(36230)=(S=10859776,E=10860031,C=29)
	CT(36231)=(S=10860032,E=10860543,C=203)
	CT(36232)=(S=10860544,E=10860799,C=29)
	CT(36233)=(S=10860800,E=10861055,C=78)
	CT(36234)=(S=10861056,E=10861311,C=6)
	CT(36235)=(S=10861312,E=10861567,C=0)
	CT(36236)=(S=10861568,E=10862335,C=29)
	CT(36237)=(S=10862336,E=10862591,C=0)
	CT(36238)=(S=10862592,E=10863103,C=29)
	CT(36239)=(S=10863104,E=10863359,C=6)
	CT(36240)=(S=10863360,E=10865151,C=29)
	CT(36241)=(S=10865152,E=10865407,C=8)
	CT(36242)=(S=10865408,E=10865919,C=29)
	CT(36243)=(S=10865920,E=10866175,C=73)
	CT(36244)=(S=10866176,E=10866943,C=29)
	CT(36245)=(S=10866944,E=10867199,C=3)
	CT(36246)=(S=10867712,E=10867967,C=29)
	CT(36247)=(S=10867968,E=10868223,C=6)
	CT(36248)=(S=10868224,E=10869759,C=29)
	CT(36249)=(S=10869760,E=10870015,C=10)
	CT(36250)=(S=10870016,E=10870271,C=29)
	CT(36251)=(S=10870272,E=10870527,C=34)
	CT(36252)=(S=10870528,E=10871807,C=29)
	CT(36253)=(S=10871808,E=10872063,C=0)
	CT(36254)=(S=10872064,E=10872319,C=6)
	CT(36255)=(S=10872320,E=10872575,C=29)
	CT(36256)=(S=10872576,E=10872831,C=147)
	CT(36257)=(S=10872832,E=10873087,C=29)
	CT(36258)=(S=10873088,E=10873343,C=105)
	CT(36259)=(S=10873344,E=10874879,C=29)
	CT(36260)=(S=10874880,E=10875135,C=0)
	CT(36261)=(S=10875136,E=10875647,C=2)
	CT(36262)=(S=10875648,E=10876159,C=6)
	CT(36263)=(S=10876160,E=10876415,C=29)
	CT(36264)=(S=10876416,E=10876671,C=6)
	CT(36265)=(S=10876672,E=10878719,C=29)
	CT(36266)=(S=10878720,E=10878975,C=105)
	CT(36267)=(S=10878976,E=10881023,C=29)
	CT(36268)=(S=10881024,E=10881279,C=34)
	CT(36269)=(S=10881280,E=10882047,C=29)
	CT(36270)=(S=10882048,E=10883071,C=34)
	CT(36271)=(S=10883072,E=10891263,C=29)
	CT(36272)=(S=10891264,E=10891519,C=78)
	CT(36273)=(S=10891520,E=10891647,C=29)
	CT(36274)=(S=10891648,E=10891775,C=14)
	CT(36275)=(S=10891776,E=10894975,C=29)
	CT(36276)=(S=10894976,E=10895103,C=78)
	CT(36277)=(S=10895104,E=10895359,C=29)
	CT(36278)=(S=10895360,E=10895615,C=0)
	CT(36279)=(S=10895616,E=10895871,C=73)
	CT(36280)=(S=10895872,E=10898175,C=29)
	CT(36281)=(S=10898176,E=10898431,C=203)
	CT(36282)=(S=10898432,E=10899199,C=29)
	CT(36283)=(S=10899200,E=10899455,C=6)
	CT(36284)=(S=10899456,E=10900223,C=29)
	CT(36285)=(S=10900224,E=10900479,C=73)
	CT(36286)=(S=10900480,E=10900735,C=29)
	CT(36287)=(S=10900736,E=10900991,C=105)
	CT(36288)=(S=10900992,E=10901247,C=29)
	CT(36289)=(S=10901248,E=10901503,C=23)
	CT(36290)=(S=10901504,E=10904575,C=29)
	CT(36291)=(S=10904576,E=10904831,C=2)
	CT(36292)=(S=10904832,E=10905343,C=29)
	CT(36293)=(S=10905344,E=10905855,C=6)
	CT(36294)=(S=10905856,E=10907135,C=29)
	CT(36295)=(S=10907136,E=10907391,C=203)
	CT(36296)=(S=10907392,E=10907647,C=1)
	CT(36297)=(S=10907648,E=10908159,C=29)
	CT(36298)=(S=10908160,E=10908415,C=201)
	CT(36299)=(S=10908416,E=10909183,C=29)
	CT(36300)=(S=10909184,E=10909695,C=2)
	CT(36301)=(S=10909696,E=10909951,C=0)
	CT(36302)=(S=10909952,E=10910207,C=7)
	CT(36303)=(S=10910208,E=10910975,C=29)
	CT(36304)=(S=10910976,E=10911231,C=6)
	CT(36305)=(S=10911232,E=10944511,C=29)
	CT(36306)=(S=10944512,E=10944767,C=181)
	CT(36307)=(S=10944768,E=10947327,C=29)
	CT(36308)=(S=10947328,E=10947583,C=78)
	CT(36309)=(S=10947584,E=10948863,C=29)
	CT(36310)=(S=10948864,E=10949119,C=78)
	CT(36311)=(S=10949120,E=10951679,C=29)
	CT(36312)=(S=10951680,E=10951935,C=203)
	CT(36313)=(S=10951936,E=10952191,C=29)
	CT(36314)=(S=10952192,E=10952447,C=0)
	CT(36315)=(S=10952448,E=10952703,C=29)
	CT(36316)=(S=10952704,E=10958847,C=78)
	CT(36317)=(S=10958848,E=10960895,C=205)
	CT(36318)=(S=10960896,E=10963455,C=29)
	CT(36319)=(S=10963456,E=10963711,C=78)
	CT(36320)=(S=10963712,E=10965247,C=29)
	CT(36321)=(S=10965248,E=10965503,C=36)
	CT(36322)=(S=10965504,E=10967055,C=29)
	CT(36323)=(S=10967056,E=10967087,C=78)
	CT(36324)=(S=10967104,E=10967167,C=29)
	CT(36325)=(S=10967168,E=10967183,C=78)
	CT(36326)=(S=10967184,E=10967215,C=29)
	CT(36327)=(S=10967216,E=10967231,C=78)
	CT(36328)=(S=10967232,E=10968063,C=29)
	CT(36329)=(S=10968064,E=10968319,C=78)
	CT(36330)=(S=10968320,E=10970879,C=29)
	CT(36331)=(S=10970880,E=10971135,C=0)
	CT(36332)=(S=10971136,E=10972159,C=29)
	CT(36333)=(S=10972160,E=10972415,C=205)
	CT(36334)=(S=10972416,E=10972927,C=29)
	CT(36335)=(S=10972928,E=10973183,C=23)
	CT(36336)=(S=10973184,E=10973695,C=29)
	CT(36337)=(S=10973696,E=10973951,C=78)
	CT(36338)=(S=10973952,E=10974207,C=29)
	CT(36339)=(S=10974208,E=10974463,C=205)
	CT(36340)=(S=10974464,E=10975999,C=29)
	CT(36341)=(S=10976000,E=10976255,C=0)
	CT(36342)=(S=10976256,E=10976511,C=29)
	CT(36343)=(S=10976512,E=10976767,C=78)
	CT(36344)=(S=10976768,E=10977535,C=29)
	CT(36345)=(S=10977536,E=10977791,C=78)
	CT(36346)=(S=10977792,E=10978047,C=0)
	CT(36347)=(S=10978048,E=10978815,C=29)
	CT(36348)=(S=10978816,E=10979071,C=200)
	CT(36349)=(S=10979072,E=10980095,C=29)
	CT(36350)=(S=10980096,E=10980351,C=1)
	CT(36351)=(S=10980352,E=10982143,C=29)
	CT(36352)=(S=10982144,E=10982399,C=78)
	CT(36353)=(S=10982400,E=10982655,C=29)
	CT(36354)=(S=10982656,E=10982911,C=34)
	CT(36355)=(S=10982912,E=10984703,C=29)
	CT(36356)=(S=10984704,E=10984959,C=201)
	CT(36357)=(S=10984960,E=10985983,C=29)
	CT(36358)=(S=10985984,E=10986239,C=0)
	CT(36359)=(S=10986240,E=10987519,C=29)
	CT(36360)=(S=10987520,E=10987775,C=34)
	CT(36361)=(S=10987776,E=10988031,C=2)
	CT(36362)=(S=10988032,E=10988543,C=29)
	CT(36363)=(S=10988544,E=10988799,C=14)
	CT(36364)=(S=10988800,E=10989311,C=29)
	CT(36365)=(S=10989312,E=10989567,C=2)
	CT(36366)=(S=10989568,E=10990335,C=29)
	CT(36367)=(S=10990336,E=10990591,C=73)
	CT(36368)=(S=10990592,E=10992895,C=29)
	CT(36369)=(S=10992896,E=10993151,C=1)
	CT(36370)=(S=10993152,E=10996415,C=29)
	CT(36371)=(S=10996416,E=10996447,C=36)
	CT(36372)=(S=10996448,E=10996479,C=29)
	CT(36373)=(S=10996480,E=10996735,C=14)
	CT(36374)=(S=10996736,E=10996991,C=29)
	CT(36375)=(S=10996992,E=10997247,C=74)
	CT(36376)=(S=10997248,E=11001075,C=29)
	CT(36377)=(S=11001076,E=11001079,C=1)
	CT(36378)=(S=11001080,E=11002623,C=29)
	CT(36379)=(S=11002624,E=11002879,C=78)
	CT(36380)=(S=11002880,E=11009023,C=29)
	CT(36381)=(S=11009024,E=11009279,C=83)
	CT(36382)=(S=11009280,E=11027967,C=29)
	CT(36383)=(S=11027968,E=11028223,C=8)
	CT(36384)=(S=11028224,E=11029503,C=29)
	CT(36385)=(S=11029504,E=11029759,C=105)
	CT(36386)=(S=11029760,E=11030015,C=209)
	CT(36387)=(S=11030016,E=11030271,C=6)
	CT(36388)=(S=11030528,E=11031039,C=147)
	CT(36389)=(S=11031040,E=11031295,C=29)
	CT(36390)=(S=11031296,E=11031551,C=83)
	CT(36391)=(S=11031552,E=11032831,C=29)
	CT(36392)=(S=11032832,E=11033087,C=105)
	CT(36393)=(S=11033344,E=11034367,C=29)
	CT(36394)=(S=11034368,E=11034623,C=9)
	CT(36395)=(S=11034624,E=11034879,C=83)
	CT(36396)=(S=11034880,E=11035903,C=29)
	CT(36397)=(S=11035904,E=11036159,C=83)
	CT(36398)=(S=11036160,E=11037183,C=29)
	CT(36399)=(S=11037184,E=11037439,C=8)
	CT(36400)=(S=11037440,E=11039487,C=29)
	CT(36401)=(S=11039488,E=11039743,C=6)
	CT(36402)=(S=11039744,E=11040767,C=29)
	CT(36403)=(S=11040768,E=11041023,C=3)
	CT(36404)=(S=11041280,E=11041535,C=29)
	CT(36405)=(S=11041536,E=11041791,C=159)
	CT(36406)=(S=11041792,E=11042303,C=29)
	CT(36407)=(S=11042304,E=11042559,C=6)
	CT(36408)=(S=11042560,E=11042815,C=29)
	CT(36409)=(S=11042816,E=11043071,C=105)
	CT(36410)=(S=11043072,E=11043327,C=29)
	CT(36411)=(S=11043328,E=11043583,C=0)
	CT(36412)=(S=11043584,E=11043839,C=6)
	CT(36413)=(S=11043840,E=11044095,C=0)
	CT(36414)=(S=11044096,E=11044351,C=29)
	CT(36415)=(S=11044352,E=11044607,C=0)
	CT(36416)=(S=11044608,E=11045631,C=29)
	CT(36417)=(S=11045632,E=11045887,C=37)
	CT(36418)=(S=11045888,E=11046143,C=0)
	CT(36419)=(S=11046144,E=11046399,C=29)
	CT(36420)=(S=11046400,E=11046655,C=105)
	CT(36421)=(S=11046656,E=11046911,C=29)
	CT(36422)=(S=11046912,E=11047167,C=78)
	CT(36423)=(S=11047168,E=11048959,C=29)
	CT(36424)=(S=11048960,E=11049471,C=0)
	CT(36425)=(S=11049472,E=11049727,C=6)
	CT(36426)=(S=11049728,E=11049983,C=105)
	CT(36427)=(S=11049984,E=11051007,C=29)
	CT(36428)=(S=11051008,E=11051263,C=1)
	CT(36429)=(S=11051264,E=11051775,C=29)
	CT(36430)=(S=11051776,E=11052031,C=34)
	CT(36431)=(S=11052032,E=11052287,C=105)
	CT(36432)=(S=11052288,E=11052543,C=204)
	CT(36433)=(S=11052544,E=11052799,C=29)
	CT(36434)=(S=11052800,E=11053055,C=126)
	CT(36435)=(S=11053056,E=11054079,C=29)
	CT(36436)=(S=11054080,E=11054335,C=105)
	CT(36437)=(S=11054336,E=11055103,C=29)
	CT(36438)=(S=11055104,E=11055359,C=181)
	CT(36439)=(S=11055360,E=11057663,C=29)
	CT(36440)=(S=11057664,E=11057919,C=0)
	CT(36441)=(S=11057920,E=11058175,C=101)
	CT(36442)=(S=11058176,E=11058431,C=6)
	CT(36443)=(S=11058432,E=11059711,C=29)
	CT(36444)=(S=11060736,E=11060991,C=0)
	CT(36445)=(S=11060992,E=11061759,C=29)
	CT(36446)=(S=11061760,E=11062015,C=15)
	CT(36447)=(S=11062016,E=11062527,C=29)
	CT(36448)=(S=11062784,E=11063039,C=105)
	CT(36449)=(S=11063040,E=11063551,C=29)
	CT(36450)=(S=11063552,E=11064319,C=105)
	CT(36451)=(S=11064320,E=11065599,C=29)
	CT(36452)=(S=11065600,E=11065855,C=0)
	CT(36453)=(S=11065856,E=11066111,C=29)
	CT(36454)=(S=11066112,E=11066367,C=6)
	CT(36455)=(S=11066368,E=11067903,C=29)
	CT(36456)=(S=11067904,E=11068159,C=83)
	CT(36457)=(S=11068672,E=11069183,C=29)
	CT(36458)=(S=11069184,E=11069439,C=203)
	CT(36459)=(S=11069696,E=11069951,C=29)
	CT(36460)=(S=11069952,E=11070207,C=215)
	CT(36461)=(S=11070208,E=11070351,C=29)
	CT(36462)=(S=11070352,E=11070367,C=78)
	CT(36463)=(S=11070368,E=11070431,C=29)
	CT(36464)=(S=11070432,E=11070447,C=78)
	CT(36465)=(S=11070448,E=11072255,C=29)
	CT(36466)=(S=11072256,E=11072511,C=210)
	CT(36467)=(S=11072512,E=11073535,C=29)
	CT(36468)=(S=11073536,E=11074047,C=6)
	CT(36469)=(S=11074048,E=11074559,C=29)
	CT(36470)=(S=11074560,E=11074815,C=0)
	CT(36471)=(S=11074816,E=11074847,C=108)
	CT(36472)=(S=11074848,E=11074879,C=107)
	CT(36473)=(S=11074880,E=11074911,C=105)
	CT(36474)=(S=11074912,E=11074927,C=135)
	CT(36475)=(S=11074928,E=11074935,C=111)
	CT(36476)=(S=11074936,E=11074943,C=105)
	CT(36477)=(S=11074944,E=11075007,C=151)
	CT(36478)=(S=11075008,E=11075023,C=105)
	CT(36479)=(S=11075024,E=11075031,C=111)
	CT(36480)=(S=11075032,E=11075039,C=105)
	CT(36481)=(S=11075040,E=11075071,C=121)
	CT(36482)=(S=11075072,E=11075327,C=29)
	CT(36483)=(S=11075328,E=11075583,C=204)
	CT(36484)=(S=11075584,E=11076095,C=105)
	CT(36485)=(S=11076096,E=11083775,C=29)
	CT(36486)=(S=11083776,E=11085311,C=34)
	CT(36487)=(S=11085312,E=11085567,C=29)
	CT(36488)=(S=11085568,E=11085823,C=34)
	CT(36489)=(S=11085824,E=11086079,C=29)
	CT(36490)=(S=11086080,E=11086847,C=34)
	CT(36491)=(S=11086848,E=11088127,C=29)
	CT(36492)=(S=11088128,E=11088383,C=34)
	CT(36493)=(S=11088384,E=11088895,C=29)
	CT(36494)=(S=11088896,E=11089151,C=34)
	CT(36495)=(S=11089152,E=11108607,C=29)
	CT(36496)=(S=11108608,E=11108863,C=105)
	CT(36497)=(S=11108864,E=11111423,C=29)
	CT(36498)=(S=11111424,E=11111679,C=6)
	CT(36499)=(S=11111680,E=11116031,C=29)
	CT(36500)=(S=11116032,E=11116287,C=219)
	CT(36501)=(S=11116288,E=11116351,C=122)
	CT(36502)=(S=11116352,E=11116415,C=111)
	CT(36503)=(S=11116480,E=11116495,C=105)
	CT(36504)=(S=11116544,E=11127039,C=29)
	CT(36505)=(S=11127296,E=11127551,C=105)
	CT(36506)=(S=11127552,E=11128831,C=29)
	CT(36507)=(S=11128832,E=11132927,C=6)
	CT(36508)=(S=11132928,E=11136767,C=29)
	CT(36509)=(S=11136768,E=11136771,C=112)
	CT(36510)=(S=11136772,E=11136779,C=105)
	CT(36511)=(S=11136780,E=11136783,C=127)
	CT(36512)=(S=11136784,E=11136787,C=111)
	CT(36513)=(S=11136788,E=11136791,C=147)
	CT(36514)=(S=11136792,E=11136795,C=107)
	CT(36515)=(S=11136796,E=11136799,C=105)
	CT(36516)=(S=11136800,E=11136803,C=116)
	CT(36517)=(S=11136804,E=11136807,C=106)
	CT(36518)=(S=11136808,E=11136811,C=120)
	CT(36519)=(S=11136812,E=11136815,C=105)
	CT(36520)=(S=11136816,E=11136819,C=111)
	CT(36521)=(S=11136820,E=11136823,C=110)
	CT(36522)=(S=11136824,E=11136827,C=135)
	CT(36523)=(S=11136828,E=11136831,C=105)
	CT(36524)=(S=11136832,E=11136835,C=118)
	CT(36525)=(S=11136836,E=11136839,C=105)
	CT(36526)=(S=11136840,E=11136843,C=137)
	CT(36527)=(S=11137024,E=11140607,C=29)
	CT(36528)=(S=11140864,E=11140867,C=105)
	CT(36529)=(S=11140868,E=11140871,C=120)
	CT(36530)=(S=11140872,E=11140875,C=109)
	CT(36531)=(S=11140876,E=11140879,C=105)
	CT(36532)=(S=11140880,E=11140883,C=110)
	CT(36533)=(S=11140884,E=11140887,C=105)
	CT(36534)=(S=11140888,E=11140891,C=144)
	CT(36535)=(S=11140892,E=11140903,C=105)
	CT(36536)=(S=11140904,E=11140907,C=111)
	CT(36537)=(S=11140908,E=11140911,C=105)
	CT(36538)=(S=11140912,E=11140915,C=112)
	CT(36539)=(S=11140916,E=11140919,C=121)
	CT(36540)=(S=11140920,E=11140927,C=105)
	CT(36541)=(S=11140928,E=11140931,C=137)
	CT(36542)=(S=11140932,E=11140935,C=152)
	CT(36543)=(S=11140936,E=11140939,C=129)
	CT(36544)=(S=11140940,E=11140943,C=105)
	CT(36545)=(S=11140944,E=11140947,C=126)
	CT(36546)=(S=11140948,E=11140951,C=106)
	CT(36547)=(S=11140952,E=11140955,C=116)
	CT(36548)=(S=11140956,E=11140959,C=152)
	CT(36549)=(S=11140960,E=11140963,C=109)
	CT(36550)=(S=11140964,E=11140967,C=144)
	CT(36551)=(S=11140968,E=11140971,C=109)
	CT(36552)=(S=11140972,E=11140975,C=65)
	CT(36553)=(S=11140976,E=11140979,C=112)
	CT(36554)=(S=11140980,E=11140983,C=105)
	CT(36555)=(S=11140984,E=11140987,C=129)
	CT(36556)=(S=11140988,E=11140991,C=111)
	CT(36557)=(S=11140992,E=11140995,C=105)
	CT(36558)=(S=11140996,E=11140999,C=131)
	CT(36559)=(S=11141000,E=11141003,C=144)
	CT(36560)=(S=11141004,E=11141007,C=111)
	CT(36561)=(S=11141008,E=11141011,C=105)
	CT(36562)=(S=11141012,E=11141015,C=122)
	CT(36563)=(S=11141016,E=11141019,C=125)
	CT(36564)=(S=11141020,E=11141023,C=65)
	CT(36565)=(S=11141024,E=11141027,C=113)
	CT(36566)=(S=11141028,E=11141039,C=105)
	CT(36567)=(S=11141040,E=11141043,C=134)
	CT(36568)=(S=11141044,E=11141047,C=111)
	CT(36569)=(S=11141048,E=11141051,C=112)
	CT(36570)=(S=11141052,E=11141055,C=137)
	CT(36571)=(S=11141056,E=11141063,C=105)
	CT(36572)=(S=11141064,E=11141067,C=109)
	CT(36573)=(S=11141068,E=11141071,C=137)
	CT(36574)=(S=11141072,E=11141075,C=111)
	CT(36575)=(S=11141076,E=11141079,C=109)
	CT(36576)=(S=11141080,E=11141087,C=105)
	CT(36577)=(S=11141088,E=11141091,C=111)
	CT(36578)=(S=11141092,E=11141099,C=105)
	CT(36579)=(S=11141100,E=11141103,C=111)
	CT(36580)=(S=11141104,E=11141119,C=105)
	CT(36581)=(S=11141376,E=11145727,C=29)
	CT(36582)=(S=11145728,E=11145983,C=203)
	CT(36583)=(S=11145984,E=11147519,C=29)
	CT(36584)=(S=11147520,E=11147775,C=204)
	CT(36585)=(S=11147776,E=11149567,C=29)
	CT(36586)=(S=11149568,E=11149823,C=5)
	CT(36587)=(S=11149824,E=11150847,C=29)
	CT(36588)=(S=11150848,E=11151103,C=5)
	CT(36589)=(S=11151360,E=11154175,C=29)
	CT(36590)=(S=11154176,E=11154431,C=83)
	CT(36591)=(S=11154432,E=11156479,C=29)
	CT(36592)=(S=11156480,E=11156735,C=34)
	CT(36593)=(S=11156736,E=11158015,C=29)
	CT(36594)=(S=11158016,E=11158271,C=189)
	CT(36595)=(S=11158272,E=11159039,C=29)
	CT(36596)=(S=11159040,E=11159295,C=204)
	CT(36597)=(S=11159296,E=11160479,C=29)
	CT(36598)=(S=11160480,E=11160495,C=78)
	CT(36599)=(S=11160496,E=11167999,C=29)
	CT(36600)=(S=11168000,E=11168255,C=2)
	CT(36601)=(S=11168256,E=11180799,C=29)
	CT(36602)=(S=11180800,E=11181055,C=83)
	CT(36603)=(S=11181056,E=11181311,C=29)
	CT(36604)=(S=11181312,E=11181567,C=0)
	CT(36605)=(S=11181568,E=11184383,C=29)
	CT(36606)=(S=11184384,E=11184639,C=204)
	CT(36607)=(S=11184640,E=11185919,C=29)
	CT(36608)=(S=11185920,E=11186175,C=78)
	CT(36609)=(S=11186176,E=11186943,C=29)
	CT(36610)=(S=11186944,E=11187199,C=7)
	CT(36611)=(S=11187200,E=11193663,C=29)
	CT(36612)=(S=11193664,E=11193791,C=49)
	CT(36613)=(S=11193792,E=11194879,C=29)
	CT(36614)=(S=11194880,E=11195135,C=83)
	CT(36615)=(S=11195136,E=11201535,C=29)
	CT(36616)=(S=11201536,E=11202047,C=34)
	CT(36617)=(S=11202560,E=11203583,C=29)
	CT(36618)=(S=11204608,E=11204609,C=4)
	CT(36619)=(S=11204610,E=11204619,C=29)
	CT(36620)=(S=11204620,E=11204621,C=4)
	CT(36621)=(S=11204622,E=11204649,C=29)
	CT(36622)=(S=11204650,E=11204651,C=4)
	CT(36623)=(S=11204652,E=11204653,C=29)
	CT(36624)=(S=11204654,E=11204655,C=4)
	CT(36625)=(S=11204656,E=11204745,C=29)
	CT(36626)=(S=11204746,E=11204747,C=4)
	CT(36627)=(S=11204748,E=11204767,C=29)
	CT(36628)=(S=11204768,E=11204779,C=4)
	CT(36629)=(S=11204780,E=11204797,C=29)
	CT(36630)=(S=11204798,E=11204799,C=4)
	CT(36631)=(S=11204800,E=11205385,C=29)
	CT(36632)=(S=11205386,E=11205387,C=4)
	CT(36633)=(S=11205388,E=11205395,C=29)
	CT(36634)=(S=11205396,E=11205397,C=4)
	CT(36635)=(S=11205398,E=11205415,C=29)
	CT(36636)=(S=11205416,E=11205423,C=4)
	CT(36637)=(S=11205424,E=11205435,C=29)
	CT(36638)=(S=11205436,E=11205437,C=4)
	CT(36639)=(S=11205438,E=11205465,C=29)
	CT(36640)=(S=11205466,E=11205467,C=4)
	CT(36641)=(S=11205468,E=11205485,C=29)
	CT(36642)=(S=11205486,E=11205487,C=4)
	CT(36643)=(S=11205488,E=11205517,C=29)
	CT(36644)=(S=11205518,E=11205519,C=4)
	CT(36645)=(S=11205520,E=11205524,C=29)
	CT(36646)=(S=11205525,E=11205527,C=4)
	CT(36647)=(S=11205528,E=11205529,C=29)
	CT(36648)=(S=11205530,E=11205533,C=4)
	CT(36649)=(S=11205534,E=11205545,C=29)
	CT(36650)=(S=11205546,E=11205547,C=4)
	CT(36651)=(S=11205548,E=11205549,C=29)
	CT(36652)=(S=11205550,E=11205551,C=4)
	CT(36653)=(S=11205552,E=11206143,C=29)
	CT(36654)=(S=11206400,E=11206655,C=49)
	CT(36655)=(S=11206656,E=11207167,C=7)
	CT(36656)=(S=11207168,E=11207679,C=2)
	CT(36657)=(S=11207680,E=11208703,C=3)
	CT(36658)=(S=11208704,E=11210751,C=1)
	CT(36659)=(S=11210752,E=11211007,C=12)
	CT(36660)=(S=11211008,E=11211263,C=28)
	CT(36661)=(S=11211264,E=11211519,C=12)
	CT(36662)=(S=11211520,E=11211775,C=50)
	CT(36663)=(S=11211776,E=11212031,C=24)
	CT(36664)=(S=11212032,E=11212287,C=36)
	CT(36665)=(S=11212288,E=11212543,C=14)
	CT(36666)=(S=11212544,E=11212799,C=27)
	CT(36667)=(S=11212800,E=11213055,C=28)
	CT(36668)=(S=11213056,E=11213183,C=34)
	CT(36669)=(S=11213184,E=11213215,C=19)
	CT(36670)=(S=11213216,E=11213219,C=27)
	CT(36671)=(S=11213220,E=11213223,C=18)
	CT(36672)=(S=11213224,E=11213231,C=31)
	CT(36673)=(S=11213232,E=11213232,C=33)
	CT(36674)=(S=11213233,E=11213233,C=18)
	CT(36675)=(S=11213234,E=11213234,C=28)
	CT(36676)=(S=11213235,E=11213235,C=36)
	CT(36677)=(S=11213236,E=11213237,C=50)
	CT(36678)=(S=11213238,E=11213239,C=58)
	CT(36679)=(S=11213240,E=11213245,C=18)
	CT(36680)=(S=11213246,E=11213246,C=60)
	CT(36681)=(S=11213247,E=11213247,C=15)
	CT(36682)=(S=11213248,E=11213248,C=58)
	CT(36683)=(S=11213249,E=11213249,C=19)
	CT(36684)=(S=11213250,E=11213251,C=18)
	CT(36685)=(S=11213252,E=11213255,C=58)
	CT(36686)=(S=11213256,E=11213257,C=14)
	CT(36687)=(S=11213258,E=11213259,C=18)
	CT(36688)=(S=11213260,E=11213263,C=31)
	CT(36689)=(S=11213264,E=11213267,C=12)
	CT(36690)=(S=11213268,E=11213271,C=18)
	CT(36691)=(S=11213272,E=11213272,C=58)
	CT(36692)=(S=11213273,E=11213273,C=49)
	CT(36693)=(S=11213274,E=11213275,C=60)
	CT(36694)=(S=11213276,E=11213278,C=33)
	CT(36695)=(S=11213279,E=11213279,C=38)
	CT(36696)=(S=11213280,E=11213280,C=101)
	CT(36697)=(S=11213281,E=11213281,C=28)
	CT(36698)=(S=11213282,E=11213283,C=58)
	CT(36699)=(S=11213284,E=11213284,C=24)
	CT(36700)=(S=11213285,E=11213285,C=49)
	CT(36701)=(S=11213286,E=11213287,C=58)
	CT(36702)=(S=11213288,E=11213295,C=18)
	CT(36703)=(S=11213296,E=11213296,C=33)
	CT(36704)=(S=11213297,E=11213297,C=31)
	CT(36705)=(S=11213298,E=11213311,C=18)
	CT(36706)=(S=11213312,E=11213567,C=49)
	CT(36707)=(S=11213568,E=11213823,C=34)
	CT(36708)=(S=11213824,E=11214591,C=14)
	CT(36709)=(S=11214592,E=11214847,C=50)
	CT(36710)=(S=11214848,E=11215103,C=19)
	CT(36711)=(S=11215104,E=11215167,C=34)
	CT(36712)=(S=11215168,E=11215231,C=12)
	CT(36713)=(S=11215232,E=11215239,C=36)
	CT(36714)=(S=11215240,E=11215247,C=33)
	CT(36715)=(S=11215248,E=11215255,C=14)
	CT(36716)=(S=11215256,E=11215263,C=12)
	CT(36717)=(S=11215264,E=11215279,C=14)
	CT(36718)=(S=11215280,E=11215295,C=28)
	CT(36719)=(S=11215296,E=11215327,C=14)
	CT(36720)=(S=11215328,E=11215335,C=36)
	CT(36721)=(S=11215336,E=11215343,C=25)
	CT(36722)=(S=11215344,E=11215351,C=104)
	CT(36723)=(S=11215352,E=11215359,C=18)
	CT(36724)=(S=11215360,E=11218943,C=1)
	CT(36725)=(S=11218944,E=11223039,C=4)
	CT(36726)=(S=11223040,E=11226111,C=29)
	CT(36727)=(S=11226112,E=11227135,C=4)
	CT(36728)=(S=11227136,E=11231231,C=1)
	CT(36729)=(S=11231232,E=11233279,C=3)
	CT(36730)=(S=11233280,E=11239423,C=1)
	CT(36731)=(S=11239424,E=11259647,C=29)
	CT(36732)=(S=11259648,E=11259903,C=7)
	CT(36733)=(S=11259904,E=11263999,C=1)
	CT(36734)=(S=11264000,E=11272191,C=11)
	CT(36735)=(S=11272192,E=11293103,C=29)
	CT(36736)=(S=11293104,E=11293111,C=78)
	CT(36737)=(S=11293112,E=11294031,C=29)
	CT(36738)=(S=11294032,E=11294039,C=78)
	CT(36739)=(S=11294040,E=11294071,C=29)
	CT(36740)=(S=11294072,E=11294075,C=80)
	CT(36741)=(S=11294076,E=11294079,C=78)
	CT(36742)=(S=11294080,E=11294391,C=29)
	CT(36743)=(S=11294392,E=11294431,C=78)
	CT(36744)=(S=11294432,E=11294447,C=29)
	CT(36745)=(S=11294448,E=11294455,C=78)
	CT(36746)=(S=11294456,E=11294543,C=29)
	CT(36747)=(S=11294544,E=11294559,C=78)
	CT(36748)=(S=11294560,E=11296079,C=29)
	CT(36749)=(S=11296080,E=11296095,C=78)
	CT(36750)=(S=11296096,E=11296123,C=29)
	CT(36751)=(S=11296124,E=11296127,C=78)
	CT(36752)=(S=11296128,E=11296775,C=29)
	CT(36753)=(S=11296776,E=11296783,C=78)
	CT(36754)=(S=11296784,E=11296807,C=29)
	CT(36755)=(S=11296808,E=11296815,C=78)
	CT(36756)=(S=11296816,E=11296959,C=29)
	CT(36757)=(S=11296960,E=11297023,C=78)
	CT(36758)=(S=11297024,E=11297031,C=29)
	CT(36759)=(S=11297032,E=11297039,C=78)
	CT(36760)=(S=11297040,E=11297071,C=29)
	CT(36761)=(S=11297072,E=11297079,C=78)
	CT(36762)=(S=11297080,E=11297083,C=29)
	CT(36763)=(S=11297084,E=11297087,C=78)
	CT(36764)=(S=11297088,E=11297143,C=29)
	CT(36765)=(S=11297144,E=11297147,C=78)
	CT(36766)=(S=11297148,E=11297151,C=29)
	CT(36767)=(S=11297152,E=11297279,C=78)
	CT(36768)=(S=11297280,E=11297423,C=29)
	CT(36769)=(S=11297424,E=11297439,C=78)
	CT(36770)=(S=11297440,E=11297543,C=29)
	CT(36771)=(S=11297544,E=11297547,C=78)
	CT(36772)=(S=11297548,E=11297555,C=29)
	CT(36773)=(S=11297556,E=11297567,C=78)
	CT(36774)=(S=11297568,E=11298319,C=29)
	CT(36775)=(S=11298320,E=11298335,C=78)
	CT(36776)=(S=11298336,E=11298527,C=29)
	CT(36777)=(S=11298528,E=11298531,C=180)
	CT(36778)=(S=11298532,E=11298535,C=78)
	CT(36779)=(S=11298536,E=11298607,C=29)
	CT(36780)=(S=11298608,E=11298623,C=78)
	CT(36781)=(S=11298624,E=11298687,C=95)
	CT(36782)=(S=11298688,E=11298815,C=78)
	CT(36783)=(S=11298816,E=11300415,C=29)
	CT(36784)=(S=11300416,E=11300431,C=78)
	CT(36785)=(S=11300432,E=11303423,C=29)
	CT(36786)=(S=11303424,E=11303935,C=78)
	CT(36787)=(S=11303936,E=11321343,C=29)
	CT(36788)=(S=11321344,E=11323391,C=0)
	CT(36789)=(S=11323392,E=11327999,C=29)
	CT(36790)=(S=11328000,E=11328511,C=78)
	CT(36791)=(S=11328512,E=11337919,C=29)
	CT(36792)=(S=11337920,E=11337951,C=78)
	CT(36793)=(S=11337952,E=11345919,C=29)
	CT(36794)=(S=11345920,E=11346943,C=78)
	CT(36795)=(S=11346944,E=11349503,C=29)
	CT(36796)=(S=11349504,E=11349535,C=78)
	CT(36797)=(S=11349536,E=11382783,C=29)
	CT(36798)=(S=11382784,E=11384831,C=78)
	CT(36799)=(S=11384832,E=11387695,C=29)
	CT(36800)=(S=11387696,E=11387711,C=78)
	CT(36801)=(S=11387712,E=11390463,C=29)
	CT(36802)=(S=11390464,E=11390719,C=78)
	CT(36803)=(S=11390720,E=11391263,C=29)
	CT(36804)=(S=11391264,E=11391295,C=78)
	CT(36805)=(S=11391296,E=11391343,C=29)
	CT(36806)=(S=11391344,E=11391391,C=78)
	CT(36807)=(S=11391392,E=11391615,C=29)
	CT(36808)=(S=11391616,E=11391743,C=78)
	CT(36809)=(S=11391744,E=11391871,C=29)
	CT(36810)=(S=11391872,E=11391935,C=80)
	CT(36811)=(S=11391936,E=11392063,C=29)
	CT(36812)=(S=11392064,E=11392191,C=78)
	CT(36813)=(S=11392192,E=11392639,C=29)
	CT(36814)=(S=11392640,E=11392655,C=78)
	CT(36815)=(S=11392656,E=11392895,C=29)
	CT(36816)=(S=11392896,E=11393023,C=80)
	CT(36817)=(S=11393024,E=11395199,C=29)
	CT(36818)=(S=11395200,E=11395215,C=78)
	CT(36819)=(S=11395216,E=11395295,C=29)
	CT(36820)=(S=11395296,E=11395327,C=78)
	CT(36821)=(S=11395328,E=11395535,C=29)
	CT(36822)=(S=11395536,E=11395551,C=174)
	CT(36823)=(S=11395552,E=11395567,C=29)
	CT(36824)=(S=11395568,E=11395583,C=175)
	CT(36825)=(S=11395584,E=11396287,C=29)
	CT(36826)=(S=11396288,E=11396351,C=80)
	CT(36827)=(S=11396352,E=11396767,C=29)
	CT(36828)=(S=11396768,E=11396799,C=78)
	CT(36829)=(S=11396800,E=11396959,C=29)
	CT(36830)=(S=11396960,E=11396991,C=78)
	CT(36831)=(S=11396992,E=11398495,C=29)
	CT(36832)=(S=11398496,E=11398527,C=78)
	CT(36833)=(S=11398528,E=11398607,C=29)
	CT(36834)=(S=11398608,E=11398623,C=78)
	CT(36835)=(S=11398624,E=11398639,C=29)
	CT(36836)=(S=11398640,E=11398911,C=78)
	CT(36837)=(S=11398912,E=11399039,C=29)
	CT(36838)=(S=11399040,E=11399103,C=78)
	CT(36839)=(S=11399104,E=11399343,C=29)
	CT(36840)=(S=11399344,E=11399359,C=78)
	CT(36841)=(S=11399360,E=11399455,C=29)
	CT(36842)=(S=11399456,E=11399471,C=78)
	CT(36843)=(S=11399472,E=11399519,C=29)
	CT(36844)=(S=11399520,E=11399535,C=78)
	CT(36845)=(S=11399536,E=11399887,C=29)
	CT(36846)=(S=11399888,E=11399903,C=78)
	CT(36847)=(S=11399904,E=11399967,C=29)
	CT(36848)=(S=11399968,E=11399983,C=78)
	CT(36849)=(S=11399984,E=11400015,C=29)
	CT(36850)=(S=11400016,E=11400031,C=80)
	CT(36851)=(S=11400032,E=11400127,C=29)
	CT(36852)=(S=11400128,E=11400143,C=78)
	CT(36853)=(S=11400144,E=11400271,C=29)
	CT(36854)=(S=11400272,E=11400287,C=78)
	CT(36855)=(S=11400288,E=11400303,C=29)
	CT(36856)=(S=11400304,E=11400319,C=78)
	CT(36857)=(S=11400320,E=11400703,C=29)
	CT(36858)=(S=11400704,E=11400735,C=78)
	CT(36859)=(S=11400736,E=11400767,C=29)
	CT(36860)=(S=11400768,E=11400799,C=78)
	CT(36861)=(S=11400800,E=11400831,C=29)
	CT(36862)=(S=11400832,E=11400863,C=78)
	CT(36863)=(S=11400864,E=11400991,C=29)
	CT(36864)=(S=11400992,E=11401023,C=78)
	CT(36865)=(S=11401024,E=11401087,C=29)
	CT(36866)=(S=11401088,E=11401119,C=78)
	CT(36867)=(S=11401120,E=11401167,C=29)
	CT(36868)=(S=11401168,E=11401183,C=78)
	CT(36869)=(S=11401184,E=11401407,C=29)
	CT(36870)=(S=11401408,E=11401471,C=78)
	CT(36871)=(S=11401472,E=11401727,C=29)
	CT(36872)=(S=11401728,E=11401855,C=78)
	CT(36873)=(S=11401856,E=11402239,C=29)
	CT(36874)=(S=11402240,E=11402303,C=78)
	CT(36875)=(S=11402304,E=11403023,C=29)
	CT(36876)=(S=11403024,E=11403039,C=78)
	CT(36877)=(S=11403040,E=11403071,C=29)
	CT(36878)=(S=11403072,E=11403087,C=78)
	CT(36879)=(S=11403088,E=11403151,C=29)
	CT(36880)=(S=11403152,E=11403167,C=78)
	CT(36881)=(S=11403168,E=11403263,C=29)
	CT(36882)=(S=11403264,E=11405311,C=78)
	CT(36883)=(S=11405312,E=11412351,C=29)
	CT(36884)=(S=11412352,E=11412479,C=78)
	CT(36885)=(S=11412480,E=11425791,C=29)
	CT(36886)=(S=11425792,E=11427839,C=78)
	CT(36887)=(S=11427840,E=11431935,C=29)
	CT(36888)=(S=11431936,E=11433983,C=78)
	CT(36889)=(S=11433984,E=11438191,C=29)
	CT(36890)=(S=11438192,E=11438207,C=78)
	CT(36891)=(S=11438208,E=11438383,C=29)
	CT(36892)=(S=11438384,E=11438399,C=78)
	CT(36893)=(S=11438400,E=11438527,C=29)
	CT(36894)=(S=11438528,E=11438591,C=78)
	CT(36895)=(S=11438592,E=11438719,C=29)
	CT(36896)=(S=11438720,E=11438735,C=78)
	CT(36897)=(S=11438736,E=11438751,C=8)
	CT(36898)=(S=11438752,E=11438783,C=29)
	CT(36899)=(S=11438784,E=11438815,C=78)
	CT(36900)=(S=11438816,E=11439279,C=29)
	CT(36901)=(S=11439280,E=11439295,C=78)
	CT(36902)=(S=11439296,E=11439615,C=29)
	CT(36903)=(S=11439616,E=11439871,C=78)
	CT(36904)=(S=11439872,E=11468799,C=29)
	CT(36905)=(S=11468800,E=11475967,C=1)
	CT(36906)=(S=11475968,E=11475971,C=86)
	CT(36907)=(S=11475972,E=11475975,C=2)
	CT(36908)=(S=11475976,E=11475979,C=7)
	CT(36909)=(S=11475980,E=11475983,C=5)
	CT(36910)=(S=11475984,E=11475999,C=2)
	CT(36911)=(S=11476000,E=11476015,C=6)
	CT(36912)=(S=11476016,E=11476031,C=2)
	CT(36913)=(S=11476032,E=11476095,C=7)
	CT(36914)=(S=11476096,E=11476223,C=2)
	CT(36915)=(S=11476224,E=11476479,C=84)
	CT(36916)=(S=11476480,E=11476991,C=1)
	CT(36917)=(S=11476992,E=11479039,C=0)
	CT(36918)=(S=11479040,E=11479295,C=4)
	CT(36919)=(S=11479296,E=11479311,C=6)
	CT(36920)=(S=11479312,E=11479327,C=8)
	CT(36921)=(S=11479328,E=11479335,C=2)
	CT(36922)=(S=11479336,E=11479343,C=0)
	CT(36923)=(S=11479344,E=11479359,C=9)
	CT(36924)=(S=11479360,E=11479423,C=2)
	CT(36925)=(S=11479424,E=11479487,C=7)
	CT(36926)=(S=11479488,E=11479551,C=2)
	CT(36927)=(S=11479552,E=11480319,C=1)
	CT(36928)=(S=11480320,E=11480383,C=8)
	CT(36929)=(S=11480384,E=11480455,C=0)
	CT(36930)=(S=11480456,E=11480463,C=2)
	CT(36931)=(S=11480464,E=11480479,C=0)
	CT(36932)=(S=11480480,E=11480487,C=6)
	CT(36933)=(S=11480488,E=11480495,C=2)
	CT(36934)=(S=11480496,E=11480499,C=223)
	CT(36935)=(S=11480504,E=11480511,C=74)
	CT(36936)=(S=11480512,E=11480575,C=6)
	CT(36937)=(S=11480576,E=11493375,C=1)
	CT(36938)=(S=11493376,E=11494399,C=9)
	CT(36939)=(S=11494400,E=11494527,C=86)
	CT(36940)=(S=11494528,E=11494563,C=4)
	CT(36941)=(S=11494564,E=11494567,C=8)
	CT(36942)=(S=11494568,E=11494591,C=4)
	CT(36943)=(S=11494592,E=11494607,C=8)
	CT(36944)=(S=11494608,E=11494655,C=2)
	CT(36945)=(S=11494656,E=11494911,C=4)
	CT(36946)=(S=11494912,E=11495167,C=1)
	CT(36947)=(S=11495168,E=11495183,C=2)
	CT(36948)=(S=11495184,E=11495199,C=0)
	CT(36949)=(S=11495200,E=11495231,C=74)
	CT(36950)=(S=11495232,E=11495295,C=11)
	CT(36951)=(S=11495296,E=11495935,C=2)
	CT(36952)=(S=11495936,E=11495939,C=11)
	CT(36953)=(S=11495944,E=11495959,C=74)
	CT(36954)=(S=11495960,E=11495967,C=0)
	CT(36955)=(S=11495968,E=11495999,C=89)
	CT(36956)=(S=11496000,E=11496063,C=6)
	CT(36957)=(S=11496064,E=11496191,C=1)
	CT(36958)=(S=11496192,E=11496255,C=77)
	CT(36959)=(S=11496256,E=11496319,C=6)
	CT(36960)=(S=11496320,E=11496383,C=0)
	CT(36961)=(S=11496384,E=11496447,C=77)
	CT(36962)=(S=11496448,E=11496959,C=2)
	CT(36963)=(S=11496960,E=11497231,C=77)
	CT(36964)=(S=11497232,E=11497247,C=6)
	CT(36965)=(S=11497248,E=11497279,C=9)
	CT(36966)=(S=11497280,E=11497303,C=2)
	CT(36967)=(S=11497304,E=11497307,C=74)
	CT(36968)=(S=11497308,E=11497311,C=2)
	CT(36969)=(S=11497312,E=11497315,C=8)
	CT(36970)=(S=11497316,E=11497319,C=73)
	CT(36971)=(S=11497320,E=11497323,C=7)
	CT(36972)=(S=11497328,E=11497335,C=74)
	CT(36973)=(S=11497336,E=11497343,C=2)
	CT(36974)=(S=11497408,E=11497471,C=9)
	CT(36975)=(S=11497472,E=11501567,C=6)
	CT(36976)=(S=11501568,E=11503615,C=2)
	CT(36977)=(S=11503616,E=11506175,C=5)
	CT(36978)=(S=11506176,E=11508735,C=1)
	CT(36979)=(S=11508736,E=11508991,C=7)
	CT(36980)=(S=11508992,E=11509247,C=156)
	CT(36981)=(S=11509248,E=11509279,C=6)
	CT(36982)=(S=11509280,E=11509311,C=74)
	CT(36983)=(S=11509312,E=11509343,C=4)
	CT(36984)=(S=11509352,E=11509359,C=0)
	CT(36985)=(S=11509360,E=11509375,C=2)
	CT(36986)=(S=11509376,E=11509439,C=90)
	CT(36987)=(S=11509440,E=11509503,C=10)
	CT(36988)=(S=11509504,E=11509759,C=8)
	CT(36989)=(S=11509760,E=11513855,C=1)
	CT(36990)=(S=11513856,E=11513983,C=10)
	CT(36991)=(S=11513984,E=11513999,C=6)
	CT(36992)=(S=11514000,E=11514007,C=166)
	CT(36993)=(S=11514016,E=11514023,C=74)
	CT(36994)=(S=11514024,E=11514031,C=7)
	CT(36995)=(S=11514048,E=11514051,C=4)
	CT(36996)=(S=11514052,E=11514055,C=2)
	CT(36997)=(S=11514056,E=11514071,C=0)
	CT(36998)=(S=11514072,E=11514075,C=73)
	CT(36999)=(S=11514076,E=11514079,C=3)
	CT(37000)=(S=11514080,E=11514111,C=8)
	CT(37001)=(S=11514112,E=11514367,C=2)
	CT(37002)=(S=11514368,E=11514623,C=1)
	CT(37003)=(S=11514624,E=11514879,C=2)
	CT(37004)=(S=11514880,E=11515903,C=9)
	CT(37005)=(S=11515904,E=11516031,C=2)
	CT(37006)=(S=11516032,E=11516095,C=1)
	CT(37007)=(S=11516096,E=11516127,C=0)
	CT(37008)=(S=11516128,E=11516143,C=74)
	CT(37009)=(S=11516144,E=11516151,C=9)
	CT(37010)=(S=11516152,E=11516155,C=74)
	CT(37011)=(S=11516156,E=11516159,C=4)
	CT(37012)=(S=11516160,E=11517951,C=1)
	CT(37013)=(S=11517952,E=11534335,C=6)
	CT(37014)=(S=11534336,E=11536383,C=28)
	CT(37015)=(S=11536384,E=11536639,C=31)
	CT(37016)=(S=11536640,E=11536895,C=28)
	CT(37017)=(S=11536896,E=11536927,C=15)
	CT(37018)=(S=11536928,E=11536935,C=48)
	CT(37019)=(S=11536936,E=11536943,C=71)
	CT(37020)=(S=11536944,E=11536951,C=28)
	CT(37021)=(S=11536952,E=11536959,C=15)
	CT(37022)=(S=11536960,E=11536975,C=25)
	CT(37023)=(S=11536976,E=11536983,C=13)
	CT(37024)=(S=11536984,E=11536991,C=49)
	CT(37025)=(S=11536992,E=11537023,C=34)
	CT(37026)=(S=11537024,E=11537151,C=19)
	CT(37027)=(S=11537152,E=11537407,C=27)
	CT(37028)=(S=11537408,E=11537471,C=51)
	CT(37029)=(S=11537472,E=11537487,C=26)
	CT(37030)=(S=11537488,E=11537495,C=25)
	CT(37031)=(S=11537496,E=11537503,C=15)
	CT(37032)=(S=11537504,E=11537511,C=18)
	CT(37033)=(S=11537512,E=11537519,C=14)
	CT(37034)=(S=11537520,E=11537535,C=33)
	CT(37035)=(S=11537536,E=11537919,C=17)
	CT(37036)=(S=11537920,E=11538431,C=18)
	CT(37037)=(S=11538432,E=11539455,C=23)
	CT(37038)=(S=11539456,E=11540479,C=24)
	CT(37039)=(S=11540480,E=11541503,C=14)
	CT(37040)=(S=11541504,E=11541567,C=28)
	CT(37041)=(S=11541568,E=11541575,C=18)
	CT(37042)=(S=11541576,E=11541583,C=35)
	CT(37043)=(S=11541584,E=11541591,C=41)
	CT(37044)=(S=11541592,E=11541631,C=25)
	CT(37045)=(S=11541632,E=11542015,C=61)
	CT(37046)=(S=11542016,E=11542271,C=37)
	CT(37047)=(S=11542272,E=11542527,C=12)
	CT(37048)=(S=11542528,E=11542543,C=31)
	CT(37049)=(S=11542544,E=11542559,C=15)
	CT(37050)=(S=11542560,E=11542567,C=18)
	CT(37051)=(S=11542568,E=11542575,C=28)
	CT(37052)=(S=11542576,E=11542583,C=25)
	CT(37053)=(S=11542584,E=11542591,C=15)
	CT(37054)=(S=11542592,E=11542655,C=63)
	CT(37055)=(S=11542656,E=11542687,C=18)
	CT(37056)=(S=11542688,E=11542703,C=58)
	CT(37057)=(S=11542704,E=11542719,C=18)
	CT(37058)=(S=11542720,E=11542727,C=66)
	CT(37059)=(S=11542728,E=11542735,C=50)
	CT(37060)=(S=11542736,E=11542751,C=28)
	CT(37061)=(S=11542752,E=11542759,C=14)
	CT(37062)=(S=11542760,E=11542767,C=28)
	CT(37063)=(S=11542768,E=11542775,C=18)
	CT(37064)=(S=11542776,E=11542783,C=25)
	CT(37065)=(S=11542784,E=11543039,C=37)
	CT(37066)=(S=11543040,E=11543295,C=63)
	CT(37067)=(S=11543296,E=11543551,C=14)
	CT(37068)=(S=11543552,E=11544575,C=31)
	CT(37069)=(S=11544576,E=11545599,C=37)
	CT(37070)=(S=11545600,E=11546111,C=23)
	CT(37071)=(S=11546112,E=11546239,C=55)
	CT(37072)=(S=11546240,E=11546271,C=26)
	CT(37073)=(S=11546272,E=11546303,C=14)
	CT(37074)=(S=11546304,E=11546367,C=34)
	CT(37075)=(S=11546368,E=11546623,C=23)
	CT(37076)=(S=11546624,E=11547775,C=18)
	CT(37077)=(S=11547776,E=11547807,C=52)
	CT(37078)=(S=11547808,E=11547823,C=25)
	CT(37079)=(S=11547824,E=11547831,C=19)
	CT(37080)=(S=11547832,E=11547839,C=12)
	CT(37081)=(S=11547840,E=11547855,C=28)
	CT(37082)=(S=11547856,E=11547863,C=14)
	CT(37083)=(S=11547864,E=11547871,C=63)
	CT(37084)=(S=11547872,E=11547887,C=12)
	CT(37085)=(S=11547888,E=11547895,C=28)
	CT(37086)=(S=11547896,E=11547903,C=25)
	CT(37087)=(S=11547904,E=11548031,C=37)
	CT(37088)=(S=11548096,E=11548159,C=18)
	CT(37089)=(S=11548160,E=11548671,C=37)
	CT(37090)=(S=11548672,E=11548703,C=18)
	CT(37091)=(S=11548704,E=11548711,C=60)
	CT(37092)=(S=11548720,E=11548727,C=18)
	CT(37093)=(S=11548728,E=11548735,C=14)
	CT(37094)=(S=11548736,E=11548799,C=25)
	CT(37095)=(S=11548800,E=11548815,C=34)
	CT(37096)=(S=11548816,E=11548831,C=26)
	CT(37097)=(S=11548832,E=11548863,C=36)
	CT(37098)=(S=11548864,E=11548895,C=14)
	CT(37099)=(S=11548896,E=11548911,C=36)
	CT(37100)=(S=11548912,E=11548927,C=19)
	CT(37101)=(S=11548928,E=11548959,C=61)
	CT(37102)=(S=11548960,E=11548967,C=12)
	CT(37103)=(S=11548968,E=11548975,C=60)
	CT(37104)=(S=11548976,E=11548991,C=31)
	CT(37105)=(S=11549000,E=11549007,C=18)
	CT(37106)=(S=11549008,E=11549015,C=12)
	CT(37107)=(S=11549016,E=11549023,C=57)
	CT(37108)=(S=11549024,E=11549039,C=25)
	CT(37109)=(S=11549040,E=11549055,C=15)
	CT(37110)=(S=11549056,E=11549119,C=28)
	CT(37111)=(S=11549120,E=11549127,C=19)
	CT(37112)=(S=11549128,E=11549135,C=25)
	CT(37113)=(S=11549136,E=11549151,C=18)
	CT(37114)=(S=11549152,E=11549167,C=48)
	CT(37115)=(S=11549168,E=11549175,C=12)
	CT(37116)=(S=11549176,E=11549191,C=14)
	CT(37117)=(S=11549192,E=11549199,C=25)
	CT(37118)=(S=11549200,E=11549207,C=58)
	CT(37119)=(S=11549208,E=11549215,C=36)
	CT(37120)=(S=11549216,E=11549247,C=18)
	CT(37121)=(S=11549248,E=11549263,C=43)
	CT(37122)=(S=11549264,E=11549271,C=15)
	CT(37123)=(S=11549272,E=11549279,C=13)
	CT(37124)=(S=11549280,E=11549311,C=14)
	CT(37125)=(S=11549312,E=11549439,C=22)
	CT(37126)=(S=11549440,E=11549695,C=18)
	CT(37127)=(S=11549696,E=11549951,C=68)
	CT(37128)=(S=11549952,E=11550079,C=63)
	CT(37129)=(S=11550080,E=11550087,C=36)
	CT(37130)=(S=11550088,E=11550095,C=19)
	CT(37131)=(S=11550096,E=11550103,C=21)
	CT(37132)=(S=11550104,E=11550111,C=57)
	CT(37133)=(S=11550112,E=11550143,C=42)
	CT(37134)=(S=11550144,E=11550207,C=71)
	CT(37135)=(S=11550208,E=11550255,C=57)
	CT(37136)=(S=11550256,E=11550271,C=15)
	CT(37137)=(S=11550272,E=11550335,C=18)
	CT(37138)=(S=11550336,E=11550343,C=14)
	CT(37139)=(S=11550344,E=11550351,C=49)
	CT(37140)=(S=11550352,E=11550359,C=26)
	CT(37141)=(S=11550360,E=11550367,C=15)
	CT(37142)=(S=11550368,E=11550383,C=49)
	CT(37143)=(S=11550384,E=11550399,C=18)
	CT(37144)=(S=11550400,E=11550407,C=36)
	CT(37145)=(S=11550408,E=11550415,C=24)
	CT(37146)=(S=11550416,E=11550423,C=14)
	CT(37147)=(S=11550424,E=11550431,C=18)
	CT(37148)=(S=11550432,E=11550447,C=33)
	CT(37149)=(S=11550448,E=11550463,C=18)
	CT(37150)=(S=11550464,E=11550719,C=50)
	CT(37151)=(S=11550720,E=11550975,C=19)
	CT(37152)=(S=11550976,E=11551007,C=43)
	CT(37153)=(S=11551008,E=11551039,C=18)
	CT(37154)=(S=11551040,E=11551047,C=14)
	CT(37155)=(S=11551048,E=11551055,C=70)
	CT(37156)=(S=11551056,E=11551071,C=15)
	CT(37157)=(S=11551072,E=11551103,C=18)
	CT(37158)=(S=11551104,E=11551231,C=28)
	CT(37159)=(S=11551232,E=11551487,C=42)
	CT(37160)=(S=11551488,E=11551519,C=31)
	CT(37161)=(S=11551520,E=11551535,C=60)
	CT(37162)=(S=11551536,E=11551543,C=18)
	CT(37163)=(S=11551544,E=11551551,C=43)
	CT(37164)=(S=11551552,E=11551567,C=26)
	CT(37165)=(S=11551568,E=11551575,C=29)
	CT(37166)=(S=11551576,E=11551583,C=12)
	CT(37167)=(S=11551584,E=11551615,C=43)
	CT(37168)=(S=11551616,E=11551647,C=12)
	CT(37169)=(S=11551648,E=11551671,C=14)
	CT(37170)=(S=11551672,E=11551679,C=24)
	CT(37171)=(S=11551680,E=11551711,C=46)
	CT(37172)=(S=11551712,E=11551719,C=33)
	CT(37173)=(S=11551720,E=11551727,C=51)
	CT(37174)=(S=11551728,E=11551735,C=18)
	CT(37175)=(S=11551736,E=11551743,C=50)
	CT(37176)=(S=11551744,E=11552767,C=19)
	CT(37177)=(S=11552768,E=11553023,C=47)
	CT(37178)=(S=11553024,E=11553279,C=40)
	CT(37179)=(S=11553280,E=11553287,C=14)
	CT(37180)=(S=11553288,E=11553295,C=18)
	CT(37181)=(S=11553296,E=11553311,C=14)
	CT(37182)=(S=11553312,E=11553319,C=12)
	CT(37183)=(S=11553320,E=11553327,C=34)
	CT(37184)=(S=11553328,E=11553335,C=16)
	CT(37185)=(S=11553336,E=11553343,C=28)
	CT(37186)=(S=11553344,E=11553407,C=40)
	CT(37187)=(S=11553408,E=11553439,C=33)
	CT(37188)=(S=11553440,E=11553471,C=14)
	CT(37189)=(S=11553472,E=11553479,C=19)
	CT(37190)=(S=11553480,E=11553487,C=18)
	CT(37191)=(S=11553488,E=11553495,C=27)
	CT(37192)=(S=11553496,E=11553503,C=33)
	CT(37193)=(S=11553504,E=11553535,C=36)
	CT(37194)=(S=11553536,E=11553791,C=27)
	CT(37195)=(S=11553792,E=11554047,C=52)
	CT(37196)=(S=11554048,E=11554175,C=18)
	CT(37197)=(S=11554176,E=11554303,C=50)
	CT(37198)=(S=11554304,E=11554815,C=21)
	CT(37199)=(S=11554816,E=11556863,C=25)
	CT(37200)=(S=11556864,E=11557887,C=37)
	CT(37201)=(S=11557888,E=11558143,C=32)
	CT(37202)=(S=11558144,E=11558399,C=47)
	CT(37203)=(S=11558400,E=11558911,C=28)
	CT(37204)=(S=11558912,E=11558943,C=18)
	CT(37205)=(S=11558944,E=11558959,C=58)
	CT(37206)=(S=11558960,E=11558975,C=38)
	CT(37207)=(S=11558976,E=11558983,C=18)
	CT(37208)=(S=11558984,E=11558991,C=31)
	CT(37209)=(S=11558992,E=11558999,C=18)
	CT(37210)=(S=11559000,E=11559007,C=38)
	CT(37211)=(S=11559008,E=11559039,C=42)
	CT(37212)=(S=11559040,E=11559055,C=18)
	CT(37213)=(S=11559056,E=11559063,C=58)
	CT(37214)=(S=11559064,E=11559071,C=42)
	CT(37215)=(S=11559072,E=11559087,C=58)
	CT(37216)=(S=11559088,E=11559095,C=38)
	CT(37217)=(S=11559096,E=11559103,C=18)
	CT(37218)=(S=11559104,E=11559135,C=31)
	CT(37219)=(S=11559136,E=11559167,C=18)
	CT(37220)=(S=11559168,E=11559175,C=31)
	CT(37221)=(S=11559176,E=11559183,C=33)
	CT(37222)=(S=11559184,E=11559199,C=58)
	CT(37223)=(S=11559200,E=11559215,C=18)
	CT(37224)=(S=11559216,E=11559223,C=58)
	CT(37225)=(S=11559224,E=11559231,C=31)
	CT(37226)=(S=11559232,E=11559247,C=14)
	CT(37227)=(S=11559248,E=11559263,C=58)
	CT(37228)=(S=11559264,E=11559279,C=18)
	CT(37229)=(S=11559280,E=11559295,C=31)
	CT(37230)=(S=11559296,E=11559303,C=18)
	CT(37231)=(S=11559304,E=11559311,C=58)
	CT(37232)=(S=11559312,E=11559319,C=38)
	CT(37233)=(S=11559320,E=11559323,C=14)
	CT(37234)=(S=11559324,E=11559325,C=32)
	CT(37235)=(S=11559326,E=11559326,C=42)
	CT(37236)=(S=11559327,E=11559327,C=12)
	CT(37237)=(S=11559328,E=11559391,C=18)
	CT(37238)=(S=11559392,E=11559407,C=38)
	CT(37239)=(S=11559408,E=11559415,C=33)
	CT(37240)=(S=11559416,E=11559423,C=58)
	CT(37241)=(S=11559424,E=11559479,C=31)
	CT(37242)=(S=11559480,E=11559487,C=38)
	CT(37243)=(S=11559488,E=11559519,C=31)
	CT(37244)=(S=11559520,E=11559535,C=33)
	CT(37245)=(S=11559536,E=11559551,C=58)
	CT(37246)=(S=11559552,E=11559583,C=18)
	CT(37247)=(S=11559584,E=11559591,C=28)
	CT(37248)=(S=11559592,E=11559615,C=18)
	CT(37249)=(S=11559616,E=11559679,C=20)
	CT(37250)=(S=11559680,E=11559695,C=18)
	CT(37251)=(S=11559696,E=11559711,C=23)
	CT(37252)=(S=11559712,E=11559727,C=38)
	CT(37253)=(S=11559728,E=11559735,C=58)
	CT(37254)=(S=11559736,E=11559935,C=18)
	CT(37255)=(S=11559936,E=11559967,C=31)
	CT(37256)=(S=11559968,E=11559975,C=28)
	CT(37257)=(S=11559976,E=11559983,C=18)
	CT(37258)=(S=11559984,E=11559991,C=38)
	CT(37259)=(S=11559992,E=11559999,C=31)
	CT(37260)=(S=11560000,E=11560095,C=18)
	CT(37261)=(S=11560096,E=11560127,C=31)
	CT(37262)=(S=11560128,E=11560135,C=58)
	CT(37263)=(S=11560136,E=11560151,C=25)
	CT(37264)=(S=11560152,E=11560159,C=18)
	CT(37265)=(S=11560160,E=11560175,C=25)
	CT(37266)=(S=11560176,E=11560191,C=18)
	CT(37267)=(S=11560192,E=11560207,C=31)
	CT(37268)=(S=11560208,E=11560223,C=25)
	CT(37269)=(S=11560224,E=11560247,C=26)
	CT(37270)=(S=11560248,E=11560263,C=18)
	CT(37271)=(S=11560264,E=11560271,C=25)
	CT(37272)=(S=11560272,E=11560279,C=58)
	CT(37273)=(S=11560280,E=11560287,C=18)
	CT(37274)=(S=11560288,E=11560319,C=31)
	CT(37275)=(S=11560320,E=11560335,C=58)
	CT(37276)=(S=11560336,E=11560351,C=33)
	CT(37277)=(S=11560352,E=11560359,C=58)
	CT(37278)=(S=11560368,E=11560375,C=71)
	CT(37279)=(S=11560376,E=11560415,C=31)
	CT(37280)=(S=11560416,E=11560447,C=18)
	CT(37281)=(S=11560448,E=11560511,C=31)
	CT(37282)=(S=11560512,E=11560519,C=14)
	CT(37283)=(S=11560520,E=11560527,C=33)
	CT(37284)=(S=11560528,E=11560543,C=58)
	CT(37285)=(S=11560544,E=11560567,C=33)
	CT(37286)=(S=11560568,E=11560575,C=38)
	CT(37287)=(S=11560576,E=11560607,C=33)
	CT(37288)=(S=11560608,E=11560615,C=18)
	CT(37289)=(S=11560616,E=11560623,C=28)
	CT(37290)=(S=11560624,E=11560639,C=58)
	CT(37291)=(S=11560640,E=11560671,C=31)
	CT(37292)=(S=11560672,E=11560703,C=26)
	CT(37293)=(S=11560704,E=11560735,C=31)
	CT(37294)=(S=11560736,E=11560751,C=58)
	CT(37295)=(S=11560752,E=11560775,C=31)
	CT(37296)=(S=11560776,E=11560783,C=58)
	CT(37297)=(S=11560784,E=11560815,C=18)
	CT(37298)=(S=11560816,E=11560823,C=31)
	CT(37299)=(S=11560824,E=11560831,C=38)
	CT(37300)=(S=11560832,E=11560863,C=18)
	CT(37301)=(S=11560864,E=11560879,C=58)
	CT(37302)=(S=11560880,E=11560903,C=60)
	CT(37303)=(S=11560904,E=11560919,C=31)
	CT(37304)=(S=11560920,E=11560927,C=46)
	CT(37305)=(S=11560928,E=11560943,C=33)
	CT(37306)=(S=11560944,E=11560951,C=38)
	CT(37307)=(S=11560952,E=11560959,C=15)
	CT(37308)=(S=11560960,E=11561063,C=31)
	CT(37309)=(S=11561064,E=11561071,C=46)
	CT(37310)=(S=11561072,E=11561079,C=58)
	CT(37311)=(S=11561080,E=11561127,C=18)
	CT(37312)=(S=11561128,E=11561135,C=31)
	CT(37313)=(S=11561136,E=11561143,C=58)
	CT(37314)=(S=11561144,E=11561151,C=31)
	CT(37315)=(S=11561152,E=11561183,C=18)
	CT(37316)=(S=11561184,E=11561343,C=31)
	CT(37317)=(S=11561344,E=11561359,C=58)
	CT(37318)=(S=11561376,E=11561479,C=31)
	CT(37319)=(S=11561480,E=11561503,C=18)
	CT(37320)=(S=11561504,E=11561511,C=58)
	CT(37321)=(S=11561512,E=11561519,C=43)
	CT(37322)=(S=11561520,E=11561535,C=60)
	CT(37323)=(S=11561536,E=11561567,C=18)
	CT(37324)=(S=11561568,E=11561575,C=60)
	CT(37325)=(S=11561576,E=11561583,C=58)
	CT(37326)=(S=11561584,E=11561591,C=14)
	CT(37327)=(S=11561592,E=11561599,C=46)
	CT(37328)=(S=11561600,E=11561631,C=18)
	CT(37329)=(S=11561632,E=11561655,C=60)
	CT(37330)=(S=11561656,E=11561663,C=71)
	CT(37331)=(S=11561664,E=11561695,C=31)
	CT(37332)=(S=11561696,E=11561703,C=17)
	CT(37333)=(S=11561704,E=11561743,C=18)
	CT(37334)=(S=11561744,E=11561759,C=71)
	CT(37335)=(S=11561760,E=11561775,C=58)
	CT(37336)=(S=11561776,E=11561791,C=18)
	CT(37337)=(S=11561792,E=11561807,C=38)
	CT(37338)=(S=11561808,E=11561815,C=18)
	CT(37339)=(S=11561816,E=11561823,C=31)
	CT(37340)=(S=11561824,E=11561831,C=18)
	CT(37341)=(S=11561832,E=11561847,C=58)
	CT(37342)=(S=11561848,E=11561855,C=33)
	CT(37343)=(S=11561856,E=11561887,C=31)
	CT(37344)=(S=11561888,E=11561895,C=18)
	CT(37345)=(S=11561896,E=11561903,C=58)
	CT(37346)=(S=11561904,E=11561919,C=31)
	CT(37347)=(S=11561920,E=11561927,C=33)
	CT(37348)=(S=11561928,E=11561983,C=18)
	CT(37349)=(S=11561984,E=11562015,C=31)
	CT(37350)=(S=11562016,E=11562031,C=46)
	CT(37351)=(S=11562032,E=11562039,C=18)
	CT(37352)=(S=11562040,E=11562047,C=59)
	CT(37353)=(S=11562048,E=11562079,C=20)
	CT(37354)=(S=11562080,E=11562111,C=31)
	CT(37355)=(S=11562112,E=11562119,C=58)
	CT(37356)=(S=11562120,E=11562167,C=18)
	CT(37357)=(S=11562168,E=11562175,C=31)
	CT(37358)=(S=11562176,E=11562207,C=18)
	CT(37359)=(S=11562208,E=11562215,C=38)
	CT(37360)=(S=11562216,E=11562223,C=31)
	CT(37361)=(S=11562224,E=11562247,C=18)
	CT(37362)=(S=11562248,E=11562255,C=31)
	CT(37363)=(S=11562256,E=11562287,C=18)
	CT(37364)=(S=11562288,E=11562295,C=58)
	CT(37365)=(S=11562296,E=11562303,C=33)
	CT(37366)=(S=11562304,E=11562367,C=18)
	CT(37367)=(S=11562368,E=11562375,C=58)
	CT(37368)=(S=11562376,E=11562383,C=46)
	CT(37369)=(S=11562384,E=11562399,C=18)
	CT(37370)=(S=11562400,E=11562407,C=58)
	CT(37371)=(S=11562408,E=11562559,C=31)
	CT(37372)=(S=11562560,E=11562591,C=18)
	CT(37373)=(S=11562592,E=11562595,C=15)
	CT(37374)=(S=11562596,E=11562596,C=23)
	CT(37375)=(S=11562597,E=11562597,C=18)
	CT(37376)=(S=11562598,E=11562599,C=31)
	CT(37377)=(S=11562600,E=11562600,C=36)
	CT(37378)=(S=11562601,E=11562601,C=12)
	CT(37379)=(S=11562602,E=11562603,C=38)
	CT(37380)=(S=11562604,E=11562607,C=26)
	CT(37381)=(S=11562608,E=11562611,C=38)
	CT(37382)=(S=11562612,E=11562613,C=27)
	CT(37383)=(S=11562614,E=11562614,C=14)
	CT(37384)=(S=11562615,E=11562615,C=101)
	CT(37385)=(S=11562616,E=11562639,C=18)
	CT(37386)=(S=11562640,E=11562655,C=58)
	CT(37387)=(S=11562656,E=11562711,C=18)
	CT(37388)=(S=11562712,E=11562719,C=38)
	CT(37389)=(S=11562720,E=11562751,C=18)
	CT(37390)=(S=11562752,E=11562759,C=38)
	CT(37391)=(S=11562760,E=11562767,C=18)
	CT(37392)=(S=11562768,E=11562783,C=58)
	CT(37393)=(S=11562784,E=11562815,C=31)
	CT(37394)=(S=11562816,E=11562831,C=18)
	CT(37395)=(S=11562832,E=11562847,C=31)
	CT(37396)=(S=11562848,E=11562855,C=18)
	CT(37397)=(S=11562856,E=11562863,C=21)
	CT(37398)=(S=11562864,E=11562911,C=58)
	CT(37399)=(S=11562912,E=11562919,C=38)
	CT(37400)=(S=11562928,E=11562943,C=31)
	CT(37401)=(S=11562944,E=11562959,C=27)
	CT(37402)=(S=11562960,E=11562967,C=18)
	CT(37403)=(S=11562968,E=11562975,C=34)
	CT(37404)=(S=11562976,E=11562991,C=58)
	CT(37405)=(S=11562992,E=11562999,C=31)
	CT(37406)=(S=11563000,E=11563007,C=18)
	CT(37407)=(S=11563008,E=11563039,C=31)
	CT(37408)=(S=11563040,E=11563079,C=18)
	CT(37409)=(S=11563080,E=11563087,C=58)
	CT(37410)=(S=11563088,E=11563095,C=38)
	CT(37411)=(S=11563096,E=11563119,C=18)
	CT(37412)=(S=11563120,E=11563127,C=58)
	CT(37413)=(S=11563128,E=11563135,C=31)
	CT(37414)=(S=11563136,E=11563167,C=33)
	CT(37415)=(S=11563168,E=11563183,C=18)
	CT(37416)=(S=11563184,E=11563199,C=71)
	CT(37417)=(S=11563200,E=11563231,C=26)
	CT(37418)=(S=11563232,E=11563327,C=18)
	CT(37419)=(S=11563360,E=11563375,C=38)
	CT(37420)=(S=11563376,E=11563391,C=18)
	CT(37421)=(S=11563392,E=11563407,C=103)
	CT(37422)=(S=11563408,E=11563439,C=31)
	CT(37423)=(S=11563440,E=11563455,C=38)
	CT(37424)=(S=11563456,E=11563471,C=58)
	CT(37425)=(S=11563472,E=11563535,C=31)
	CT(37426)=(S=11563536,E=11563551,C=18)
	CT(37427)=(S=11563552,E=11563567,C=31)
	CT(37428)=(S=11563568,E=11563575,C=18)
	CT(37429)=(S=11563576,E=11563583,C=38)
	CT(37430)=(S=11563584,E=11563615,C=18)
	CT(37431)=(S=11563616,E=11563631,C=14)
	CT(37432)=(S=11563632,E=11563751,C=18)
	CT(37433)=(S=11563752,E=11563759,C=58)
	CT(37434)=(S=11563760,E=11563775,C=33)
	CT(37435)=(S=11563776,E=11563807,C=58)
	CT(37436)=(S=11563808,E=11563855,C=18)
	CT(37437)=(S=11563856,E=11563863,C=58)
	CT(37438)=(S=11563864,E=11563871,C=18)
	CT(37439)=(S=11563872,E=11563879,C=31)
	CT(37440)=(S=11563880,E=11563943,C=18)
	CT(37441)=(S=11563944,E=11563951,C=15)
	CT(37442)=(S=11563952,E=11563999,C=18)
	CT(37443)=(S=11564000,E=11564015,C=38)
	CT(37444)=(S=11564016,E=11564031,C=58)
	CT(37445)=(S=11564032,E=11564063,C=18)
	CT(37446)=(S=11564064,E=11564079,C=38)
	CT(37447)=(S=11564080,E=11564095,C=18)
	CT(37448)=(S=11564096,E=11564127,C=31)
	CT(37449)=(S=11564128,E=11564159,C=71)
	CT(37450)=(S=11564160,E=11564167,C=38)
	CT(37451)=(S=11564168,E=11564175,C=18)
	CT(37452)=(S=11564176,E=11564176,C=51)
	CT(37453)=(S=11564177,E=11564177,C=36)
	CT(37454)=(S=11564178,E=11564179,C=31)
	CT(37455)=(S=11564180,E=11564183,C=58)
	CT(37456)=(S=11564184,E=11564223,C=18)
	CT(37457)=(S=11564224,E=11564231,C=31)
	CT(37458)=(S=11564232,E=11564239,C=58)
	CT(37459)=(S=11564240,E=11564255,C=18)
	CT(37460)=(S=11564256,E=11564271,C=38)
	CT(37461)=(S=11564272,E=11564343,C=18)
	CT(37462)=(S=11564344,E=11564351,C=36)
	CT(37463)=(S=11564352,E=11564383,C=31)
	CT(37464)=(S=11564384,E=11564391,C=37)
	CT(37465)=(S=11564392,E=11564399,C=28)
	CT(37466)=(S=11564400,E=11564447,C=18)
	CT(37467)=(S=11564448,E=11564479,C=31)
	CT(37468)=(S=11564480,E=11564575,C=18)
	CT(37469)=(S=11564584,E=11564599,C=31)
	CT(37470)=(S=11564600,E=11564607,C=38)
	CT(37471)=(S=11564608,E=11564623,C=18)
	CT(37472)=(S=11564624,E=11564639,C=38)
	CT(37473)=(S=11564640,E=11564655,C=71)
	CT(37474)=(S=11564656,E=11564679,C=18)
	CT(37475)=(S=11564680,E=11564687,C=58)
	CT(37476)=(S=11564688,E=11564695,C=31)
	CT(37477)=(S=11564696,E=11564703,C=68)
	CT(37478)=(S=11564744,E=11564751,C=38)
	CT(37479)=(S=11564752,E=11564767,C=33)
	CT(37480)=(S=11564768,E=11564799,C=18)
	CT(37481)=(S=11564800,E=11564815,C=31)
	CT(37482)=(S=11564816,E=11564823,C=18)
	CT(37483)=(S=11564824,E=11564831,C=31)
	CT(37484)=(S=11564832,E=11564863,C=58)
	CT(37485)=(S=11564864,E=11564927,C=31)
	CT(37486)=(S=11564960,E=11565023,C=18)
	CT(37487)=(S=11565024,E=11565031,C=20)
	CT(37488)=(S=11565032,E=11565039,C=31)
	CT(37489)=(S=11565040,E=11565047,C=33)
	CT(37490)=(S=11565048,E=11565055,C=43)
	CT(37491)=(S=11565056,E=11565071,C=18)
	CT(37492)=(S=11565072,E=11565079,C=29)
	CT(37493)=(S=11565080,E=11565087,C=18)
	CT(37494)=(S=11565088,E=11565119,C=31)
	CT(37495)=(S=11565120,E=11565151,C=18)
	CT(37496)=(S=11565152,E=11565175,C=31)
	CT(37497)=(S=11565176,E=11565183,C=58)
	CT(37498)=(S=11565184,E=11565215,C=12)
	CT(37499)=(S=11565216,E=11565223,C=18)
	CT(37500)=(S=11565224,E=11565231,C=25)
	CT(37501)=(S=11565232,E=11565311,C=18)
	CT(37502)=(S=11565312,E=11565327,C=33)
	CT(37503)=(S=11565328,E=11565335,C=18)
	CT(37504)=(S=11565336,E=11565359,C=38)
	CT(37505)=(S=11565360,E=11565375,C=18)
	CT(37506)=(S=11565376,E=11565383,C=25)
	CT(37507)=(S=11565384,E=11565391,C=31)
	CT(37508)=(S=11565392,E=11565399,C=58)
	CT(37509)=(S=11565400,E=11565407,C=27)
	CT(37510)=(S=11565408,E=11565423,C=38)
	CT(37511)=(S=11565424,E=11565439,C=58)
	CT(37512)=(S=11565440,E=11565503,C=18)
	CT(37513)=(S=11565504,E=11565519,C=31)
	CT(37514)=(S=11565520,E=11565535,C=18)
	CT(37515)=(S=11565536,E=11565567,C=31)
	CT(37516)=(S=11565568,E=11565599,C=18)
	CT(37517)=(S=11565600,E=11565615,C=33)
	CT(37518)=(S=11565616,E=11565647,C=18)
	CT(37519)=(S=11565648,E=11565655,C=20)
	CT(37520)=(S=11565656,E=11565759,C=31)
	CT(37521)=(S=11565760,E=11565761,C=14)
	CT(37522)=(S=11565762,E=11565763,C=34)
	CT(37523)=(S=11565764,E=11565766,C=15)
	CT(37524)=(S=11565767,E=11565767,C=58)
	CT(37525)=(S=11565768,E=11565775,C=18)
	CT(37526)=(S=11565778,E=11565779,C=26)
	CT(37527)=(S=11565780,E=11565807,C=58)
	CT(37528)=(S=11565808,E=11565811,C=31)
	CT(37529)=(S=11565812,E=11565813,C=58)
	CT(37530)=(S=11565814,E=11565814,C=14)
	CT(37531)=(S=11565815,E=11565823,C=18)
	CT(37532)=(S=11565824,E=11565855,C=53)
	CT(37533)=(S=11565856,E=11565871,C=14)
	CT(37534)=(S=11565872,E=11565872,C=28)
	CT(37535)=(S=11565873,E=11565873,C=18)
	CT(37536)=(S=11565874,E=11565874,C=14)
	CT(37537)=(S=11565875,E=11565875,C=28)
	CT(37538)=(S=11565876,E=11565877,C=14)
	CT(37539)=(S=11565878,E=11565879,C=28)
	CT(37540)=(S=11565880,E=11565880,C=18)
	CT(37541)=(S=11565881,E=11565881,C=57)
	CT(37542)=(S=11565882,E=11565883,C=14)
	CT(37543)=(S=11565884,E=11565885,C=58)
	CT(37544)=(S=11565886,E=11565886,C=14)
	CT(37545)=(S=11565887,E=11565887,C=12)
	CT(37546)=(S=11565888,E=11565951,C=26)
	CT(37547)=(S=11565952,E=11566031,C=18)
	CT(37548)=(S=11566032,E=11566039,C=46)
	CT(37549)=(S=11566040,E=11566043,C=18)
	CT(37550)=(S=11566044,E=11566045,C=31)
	CT(37551)=(S=11566046,E=11566046,C=14)
	CT(37552)=(S=11566047,E=11566047,C=58)
	CT(37553)=(S=11566048,E=11566079,C=62)
	CT(37554)=(S=11566080,E=11566111,C=31)
	CT(37555)=(S=11566144,E=11566147,C=26)
	CT(37556)=(S=11566148,E=11566150,C=14)
	CT(37557)=(S=11566151,E=11566151,C=36)
	CT(37558)=(S=11566152,E=11566175,C=38)
	CT(37559)=(S=11566176,E=11566183,C=66)
	CT(37560)=(S=11566184,E=11566191,C=38)
	CT(37561)=(S=11566192,E=11566193,C=14)
	CT(37562)=(S=11566194,E=11566207,C=18)
	CT(37563)=(S=11566208,E=11566215,C=31)
	CT(37564)=(S=11566216,E=11566217,C=18)
	CT(37565)=(S=11566218,E=11566219,C=31)
	CT(37566)=(S=11566220,E=11566223,C=58)
	CT(37567)=(S=11566224,E=11566227,C=18)
	CT(37568)=(S=11566228,E=11566228,C=19)
	CT(37569)=(S=11566229,E=11566229,C=28)
	CT(37570)=(S=11566230,E=11566251,C=18)
	CT(37571)=(S=11566252,E=11566263,C=38)
	CT(37572)=(S=11566264,E=11566267,C=33)
	CT(37573)=(S=11566268,E=11566269,C=18)
	CT(37574)=(S=11566270,E=11566270,C=58)
	CT(37575)=(S=11566271,E=11566271,C=14)
	CT(37576)=(S=11566272,E=11566303,C=18)
	CT(37577)=(S=11566304,E=11566305,C=19)
	CT(37578)=(S=11566306,E=11566307,C=18)
	CT(37579)=(S=11566308,E=11566319,C=31)
	CT(37580)=(S=11566320,E=11566325,C=58)
	CT(37581)=(S=11566326,E=11566326,C=63)
	CT(37582)=(S=11566327,E=11566327,C=55)
	CT(37583)=(S=11566328,E=11566328,C=18)
	CT(37584)=(S=11566329,E=11566329,C=42)
	CT(37585)=(S=11566330,E=11566331,C=19)
	CT(37586)=(S=11566332,E=11566332,C=21)
	CT(37587)=(S=11566333,E=11566335,C=36)
	CT(37588)=(S=11566336,E=11566367,C=38)
	CT(37589)=(S=11566368,E=11566399,C=31)
	CT(37590)=(S=11566400,E=11566628,C=18)
	CT(37591)=(S=11566629,E=11566629,C=63)
	CT(37592)=(S=11566630,E=11566630,C=22)
	CT(37593)=(S=11566631,E=11566631,C=42)
	CT(37594)=(S=11566632,E=11566647,C=18)
	CT(37595)=(S=11566648,E=11566651,C=58)
	CT(37596)=(S=11566652,E=11566655,C=31)
	CT(37597)=(S=11566656,E=11566687,C=28)
	CT(37598)=(S=11566688,E=11566751,C=31)
	CT(37599)=(S=11566752,E=11566752,C=19)
	CT(37600)=(S=11566753,E=11566753,C=18)
	CT(37601)=(S=11566754,E=11566755,C=31)
	CT(37602)=(S=11566756,E=11566759,C=62)
	CT(37603)=(S=11566760,E=11566783,C=38)
	CT(37604)=(S=11566784,E=11566786,C=14)
	CT(37605)=(S=11566787,E=11566807,C=38)
	CT(37606)=(S=11566808,E=11566811,C=28)
	CT(37607)=(S=11566812,E=11566815,C=38)
	CT(37608)=(S=11566816,E=11566823,C=14)
	CT(37609)=(S=11566824,E=11566831,C=38)
	CT(37610)=(S=11566832,E=11566839,C=14)
	CT(37611)=(S=11566840,E=11566843,C=58)
	CT(37612)=(S=11566844,E=11566847,C=38)
	CT(37613)=(S=11566848,E=11567103,C=34)
	CT(37614)=(S=11567104,E=11583487,C=12)
	CT(37615)=(S=11583488,E=11585023,C=18)
	CT(37616)=(S=11585024,E=11585535,C=28)
	CT(37617)=(S=11585536,E=11586047,C=15)
	CT(37618)=(S=11586048,E=11586559,C=94)
	CT(37619)=(S=11586560,E=11587071,C=16)
	CT(37620)=(S=11587072,E=11587583,C=15)
	CT(37621)=(S=11587584,E=11589631,C=18)
	CT(37622)=(S=11589632,E=11590911,C=37)
	CT(37623)=(S=11590912,E=11590927,C=18)
	CT(37624)=(S=11590928,E=11590943,C=26)
	CT(37625)=(S=11590944,E=11590951,C=21)
	CT(37626)=(S=11590952,E=11590959,C=28)
	CT(37627)=(S=11590960,E=11590967,C=31)
	CT(37628)=(S=11590968,E=11590975,C=14)
	CT(37629)=(S=11590976,E=11590983,C=26)
	CT(37630)=(S=11590984,E=11590991,C=46)
	CT(37631)=(S=11590992,E=11590999,C=19)
	CT(37632)=(S=11591000,E=11591007,C=27)
	CT(37633)=(S=11591008,E=11591039,C=58)
	CT(37634)=(S=11591040,E=11591167,C=40)
	CT(37635)=(S=11591168,E=11591183,C=51)
	CT(37636)=(S=11591184,E=11591191,C=18)
	CT(37637)=(S=11591192,E=11591199,C=25)
	CT(37638)=(S=11591200,E=11591215,C=57)
	CT(37639)=(S=11591216,E=11591231,C=12)
	CT(37640)=(S=11591232,E=11591263,C=25)
	CT(37641)=(S=11591264,E=11591295,C=19)
	CT(37642)=(S=11591296,E=11591359,C=20)
	CT(37643)=(S=11591360,E=11591375,C=18)
	CT(37644)=(S=11591376,E=11591383,C=12)
	CT(37645)=(S=11591384,E=11591391,C=18)
	CT(37646)=(S=11591392,E=11591399,C=33)
	CT(37647)=(S=11591400,E=11591407,C=24)
	CT(37648)=(S=11591408,E=11591423,C=18)
	CT(37649)=(S=11591424,E=11591495,C=38)
	CT(37650)=(S=11591496,E=11591503,C=20)
	CT(37651)=(S=11591504,E=11591531,C=38)
	CT(37652)=(S=11591532,E=11591532,C=14)
	CT(37653)=(S=11591533,E=11591534,C=38)
	CT(37654)=(S=11591535,E=11591535,C=170)
	CT(37655)=(S=11591536,E=11591539,C=38)
	CT(37656)=(S=11591540,E=11591542,C=14)
	CT(37657)=(S=11591543,E=11591583,C=38)
	CT(37658)=(S=11591584,E=11591585,C=14)
	CT(37659)=(S=11591586,E=11591588,C=38)
	CT(37660)=(S=11591589,E=11591589,C=36)
	CT(37661)=(S=11591590,E=11591595,C=38)
	CT(37662)=(S=11591596,E=11591603,C=95)
	CT(37663)=(S=11591604,E=11591679,C=38)
	CT(37664)=(S=11591680,E=11592191,C=23)
	CT(37665)=(S=11592192,E=11592319,C=50)
	CT(37666)=(S=11592320,E=11592447,C=18)
	CT(37667)=(S=11592448,E=11592575,C=37)
	CT(37668)=(S=11592576,E=11592591,C=14)
	CT(37669)=(S=11592592,E=11592607,C=25)
	CT(37670)=(S=11592608,E=11592615,C=58)
	CT(37671)=(S=11592616,E=11592623,C=15)
	CT(37672)=(S=11592624,E=11592639,C=18)
	CT(37673)=(S=11592640,E=11592671,C=14)
	CT(37674)=(S=11592672,E=11592687,C=32)
	CT(37675)=(S=11592696,E=11592703,C=33)
	CT(37676)=(S=11592704,E=11593727,C=17)
	CT(37677)=(S=11593728,E=11596031,C=37)
	CT(37678)=(S=11596032,E=11596095,C=50)
	CT(37679)=(S=11596096,E=11596103,C=35)
	CT(37680)=(S=11596104,E=11596111,C=58)
	CT(37681)=(S=11596112,E=11596127,C=35)
	CT(37682)=(S=11596128,E=11596135,C=18)
	CT(37683)=(S=11596136,E=11596143,C=31)
	CT(37684)=(S=11596144,E=11596151,C=17)
	CT(37685)=(S=11596152,E=11596159,C=12)
	CT(37686)=(S=11596160,E=11596191,C=31)
	CT(37687)=(S=11596192,E=11596207,C=28)
	CT(37688)=(S=11596208,E=11596215,C=39)
	CT(37689)=(S=11596216,E=11596223,C=23)
	CT(37690)=(S=11596224,E=11596263,C=18)
	CT(37691)=(S=11596264,E=11596271,C=27)
	CT(37692)=(S=11596272,E=11596279,C=58)
	CT(37693)=(S=11596280,E=11596287,C=12)
	CT(37694)=(S=11596288,E=11597823,C=15)
	CT(37695)=(S=11597824,E=11599871,C=14)
	CT(37696)=(S=11599872,E=11623415,C=189)
	CT(37697)=(S=11623416,E=11623423,C=212)
	CT(37698)=(S=11623424,E=11657215,C=189)
	CT(37699)=(S=11657216,E=11659903,C=204)
	CT(37700)=(S=11659904,E=11659907,C=83)
	CT(37701)=(S=11659908,E=11659911,C=189)
	CT(37702)=(S=11659912,E=11659919,C=208)
	CT(37703)=(S=11659920,E=11659967,C=189)
	CT(37704)=(S=11659968,E=11660031,C=213)
	CT(37705)=(S=11660032,E=11660287,C=189)
	CT(37706)=(S=11660288,E=11663871,C=204)
	CT(37707)=(S=11663872,E=11664383,C=206)
	CT(37708)=(S=11664384,E=11665407,C=181)
	CT(37709)=(S=11665408,E=11669503,C=28)
	CT(37710)=(S=11669504,E=11669519,C=14)
	CT(37711)=(S=11669520,E=11669535,C=60)
	CT(37712)=(S=11669536,E=11669551,C=44)
	CT(37713)=(S=11669552,E=11669567,C=28)
	CT(37714)=(S=11669568,E=11669583,C=27)
	CT(37715)=(S=11669584,E=11669599,C=31)
	CT(37716)=(S=11669600,E=11669631,C=58)
	CT(37717)=(S=11669632,E=11669647,C=51)
	CT(37718)=(S=11669648,E=11669663,C=18)
	CT(37719)=(S=11669664,E=11669679,C=12)
	CT(37720)=(S=11669680,E=11669695,C=47)
	CT(37721)=(S=11669696,E=11669711,C=63)
	CT(37722)=(S=11669712,E=11669727,C=19)
	CT(37723)=(S=11669728,E=11669743,C=14)
	CT(37724)=(S=11669744,E=11669759,C=12)
	CT(37725)=(S=11669760,E=11669775,C=33)
	CT(37726)=(S=11669776,E=11669791,C=46)
	CT(37727)=(S=11669792,E=11669807,C=14)
	CT(37728)=(S=11669808,E=11669823,C=28)
	CT(37729)=(S=11669824,E=11669839,C=14)
	CT(37730)=(S=11669840,E=11669871,C=33)
	CT(37731)=(S=11669872,E=11669887,C=57)
	CT(37732)=(S=11669888,E=11669903,C=18)
	CT(37733)=(S=11669904,E=11669919,C=27)
	CT(37734)=(S=11669920,E=11669935,C=53)
	CT(37735)=(S=11669936,E=11669967,C=18)
	CT(37736)=(S=11669984,E=11669999,C=28)
	CT(37737)=(S=11670000,E=11670015,C=14)
	CT(37738)=(S=11670016,E=11670031,C=18)
	CT(37739)=(S=11670032,E=11670047,C=36)
	CT(37740)=(S=11670048,E=11670063,C=53)
	CT(37741)=(S=11670064,E=11670079,C=12)
	CT(37742)=(S=11670080,E=11670095,C=71)
	CT(37743)=(S=11670096,E=11670111,C=36)
	CT(37744)=(S=11670112,E=11670127,C=18)
	CT(37745)=(S=11670128,E=11670143,C=14)
	CT(37746)=(S=11670144,E=11670159,C=36)
	CT(37747)=(S=11670160,E=11670175,C=28)
	CT(37748)=(S=11670176,E=11670191,C=42)
	CT(37749)=(S=11670192,E=11670207,C=12)
	CT(37750)=(S=11670208,E=11670223,C=37)
	CT(37751)=(S=11670224,E=11670255,C=18)
	CT(37752)=(S=11670256,E=11670271,C=28)
	CT(37753)=(S=11670272,E=11670287,C=12)
	CT(37754)=(S=11670288,E=11670303,C=44)
	CT(37755)=(S=11670304,E=11670319,C=25)
	CT(37756)=(S=11670320,E=11670335,C=48)
	CT(37757)=(S=11670336,E=11670351,C=28)
	CT(37758)=(S=11670352,E=11670367,C=18)
	CT(37759)=(S=11670368,E=11670383,C=58)
	CT(37760)=(S=11670384,E=11670399,C=36)
	CT(37761)=(S=11670400,E=11670415,C=12)
	CT(37762)=(S=11670416,E=11670447,C=15)
	CT(37763)=(S=11670448,E=11670463,C=58)
	CT(37764)=(S=11670464,E=11670479,C=171)
	CT(37765)=(S=11670480,E=11670511,C=28)
	CT(37766)=(S=11670512,E=11670527,C=18)
	CT(37767)=(S=11670528,E=11670543,C=28)
	CT(37768)=(S=11670544,E=11670551,C=52)
	CT(37769)=(S=11670560,E=11670567,C=15)
	CT(37770)=(S=11670568,E=11670575,C=18)
	CT(37771)=(S=11670576,E=11670583,C=12)
	CT(37772)=(S=11670584,E=11670591,C=36)
	CT(37773)=(S=11670592,E=11670599,C=12)
	CT(37774)=(S=11670600,E=11670607,C=15)
	CT(37775)=(S=11670608,E=11670615,C=63)
	CT(37776)=(S=11670616,E=11670631,C=28)
	CT(37777)=(S=11670632,E=11670639,C=27)
	CT(37778)=(S=11670640,E=11670647,C=15)
	CT(37779)=(S=11670648,E=11670655,C=22)
	CT(37780)=(S=11670656,E=11670663,C=18)
	CT(37781)=(S=11670664,E=11670671,C=33)
	CT(37782)=(S=11670672,E=11670679,C=23)
	CT(37783)=(S=11670680,E=11670687,C=31)
	CT(37784)=(S=11670688,E=11670695,C=46)
	CT(37785)=(S=11670696,E=11670703,C=36)
	CT(37786)=(S=11670704,E=11670711,C=18)
	CT(37787)=(S=11670712,E=11670719,C=61)
	CT(37788)=(S=11670720,E=11670727,C=14)
	CT(37789)=(S=11670728,E=11670735,C=46)
	CT(37790)=(S=11670744,E=11670751,C=24)
	CT(37791)=(S=11670752,E=11670759,C=37)
	CT(37792)=(S=11670760,E=11670767,C=18)
	CT(37793)=(S=11670768,E=11670775,C=12)
	CT(37794)=(S=11670776,E=11670783,C=36)
	CT(37795)=(S=11670784,E=11670791,C=28)
	CT(37796)=(S=11670792,E=11670799,C=18)
	CT(37797)=(S=11670800,E=11670807,C=36)
	CT(37798)=(S=11670808,E=11670815,C=18)
	CT(37799)=(S=11670816,E=11670823,C=14)
	CT(37800)=(S=11670824,E=11670831,C=26)
	CT(37801)=(S=11670832,E=11670839,C=50)
	CT(37802)=(S=11670840,E=11670847,C=25)
	CT(37803)=(S=11670848,E=11670855,C=18)
	CT(37804)=(S=11670856,E=11670871,C=19)
	CT(37805)=(S=11670872,E=11670879,C=24)
	CT(37806)=(S=11670880,E=11670887,C=15)
	CT(37807)=(S=11670888,E=11670895,C=25)
	CT(37808)=(S=11670896,E=11670903,C=36)
	CT(37809)=(S=11670904,E=11670911,C=38)
	CT(37810)=(S=11670912,E=11670919,C=27)
	CT(37811)=(S=11670920,E=11670927,C=58)
	CT(37812)=(S=11670928,E=11670935,C=28)
	CT(37813)=(S=11670936,E=11670943,C=58)
	CT(37814)=(S=11670944,E=11670951,C=26)
	CT(37815)=(S=11670952,E=11670959,C=22)
	CT(37816)=(S=11670960,E=11670967,C=12)
	CT(37817)=(S=11670968,E=11670975,C=33)
	CT(37818)=(S=11670976,E=11670983,C=47)
	CT(37819)=(S=11670984,E=11670991,C=15)
	CT(37820)=(S=11670992,E=11670999,C=12)
	CT(37821)=(S=11671000,E=11671007,C=36)
	CT(37822)=(S=11671008,E=11671015,C=34)
	CT(37823)=(S=11671016,E=11671023,C=14)
	CT(37824)=(S=11671024,E=11671031,C=55)
	CT(37825)=(S=11671032,E=11671039,C=24)
	CT(37826)=(S=11671040,E=11671047,C=12)
	CT(37827)=(S=11671048,E=11671055,C=37)
	CT(37828)=(S=11671056,E=11671063,C=14)
	CT(37829)=(S=11671064,E=11671071,C=34)
	CT(37830)=(S=11671072,E=11671079,C=35)
	CT(37831)=(S=11671080,E=11671095,C=18)
	CT(37832)=(S=11671096,E=11671103,C=36)
	CT(37833)=(S=11671104,E=11671111,C=34)
	CT(37834)=(S=11671112,E=11671119,C=26)
	CT(37835)=(S=11671120,E=11671127,C=36)
	CT(37836)=(S=11671128,E=11671135,C=18)
	CT(37837)=(S=11671136,E=11671143,C=27)
	CT(37838)=(S=11671144,E=11671151,C=34)
	CT(37839)=(S=11671152,E=11671159,C=33)
	CT(37840)=(S=11671160,E=11671167,C=36)
	CT(37841)=(S=11671168,E=11671175,C=12)
	CT(37842)=(S=11671176,E=11671183,C=14)
	CT(37843)=(S=11671184,E=11671199,C=12)
	CT(37844)=(S=11671200,E=11671207,C=14)
	CT(37845)=(S=11671208,E=11671215,C=20)
	CT(37846)=(S=11671216,E=11671223,C=12)
	CT(37847)=(S=11671224,E=11671231,C=15)
	CT(37848)=(S=11671232,E=11671239,C=18)
	CT(37849)=(S=11671240,E=11671247,C=19)
	CT(37850)=(S=11671248,E=11671255,C=14)
	CT(37851)=(S=11671256,E=11671263,C=46)
	CT(37852)=(S=11671264,E=11671271,C=50)
	CT(37853)=(S=11671272,E=11671279,C=36)
	CT(37854)=(S=11671280,E=11671287,C=12)
	CT(37855)=(S=11671288,E=11671295,C=15)
	CT(37856)=(S=11671296,E=11671303,C=18)
	CT(37857)=(S=11671304,E=11671311,C=15)
	CT(37858)=(S=11671312,E=11671319,C=94)
	CT(37859)=(S=11671320,E=11671327,C=28)
	CT(37860)=(S=11671328,E=11671335,C=12)
	CT(37861)=(S=11671336,E=11671343,C=49)
	CT(37862)=(S=11671344,E=11671351,C=28)
	CT(37863)=(S=11671352,E=11671359,C=25)
	CT(37864)=(S=11671360,E=11671367,C=38)
	CT(37865)=(S=11671368,E=11671375,C=22)
	CT(37866)=(S=11671376,E=11671383,C=42)
	CT(37867)=(S=11671384,E=11671391,C=71)
	CT(37868)=(S=11671392,E=11671399,C=27)
	CT(37869)=(S=11671400,E=11671407,C=58)
	CT(37870)=(S=11671408,E=11671415,C=55)
	CT(37871)=(S=11671416,E=11671423,C=28)
	CT(37872)=(S=11671424,E=11671431,C=14)
	CT(37873)=(S=11671432,E=11671439,C=31)
	CT(37874)=(S=11671440,E=11671447,C=18)
	CT(37875)=(S=11671448,E=11671455,C=12)
	CT(37876)=(S=11671456,E=11671463,C=14)
	CT(37877)=(S=11671464,E=11671471,C=34)
	CT(37878)=(S=11671472,E=11671479,C=24)
	CT(37879)=(S=11671480,E=11671487,C=36)
	CT(37880)=(S=11671488,E=11671495,C=58)
	CT(37881)=(S=11671496,E=11671503,C=15)
	CT(37882)=(S=11671504,E=11671511,C=25)
	CT(37883)=(S=11671512,E=11671519,C=33)
	CT(37884)=(S=11671520,E=11671527,C=18)
	CT(37885)=(S=11671528,E=11671535,C=33)
	CT(37886)=(S=11671536,E=11671543,C=24)
	CT(37887)=(S=11671544,E=11671551,C=15)
	CT(37888)=(S=11671552,E=11672575,C=28)
	CT(37889)=(S=11672576,E=11673599,C=19)
	CT(37890)=(S=11673600,E=11674111,C=12)
	CT(37891)=(S=11674112,E=11674623,C=18)
	CT(37892)=(S=11674624,E=11675135,C=58)
	CT(37893)=(S=11675136,E=11675647,C=34)
	CT(37894)=(S=11675648,E=11676159,C=71)
	CT(37895)=(S=11676160,E=11676671,C=58)
	CT(37896)=(S=11676672,E=11677695,C=18)
	CT(37897)=(S=11677696,E=11677951,C=50)
	CT(37898)=(S=11677952,E=11678207,C=18)
	CT(37899)=(S=11678208,E=11678719,C=49)
	CT(37900)=(S=11678720,E=11678975,C=30)
	CT(37901)=(S=11678976,E=11679231,C=101)
	CT(37902)=(S=11679232,E=11679487,C=31)
	CT(37903)=(S=11679488,E=11679743,C=47)
	CT(37904)=(S=11679744,E=11679999,C=58)
	CT(37905)=(S=11680000,E=11680255,C=18)
	CT(37906)=(S=11680256,E=11680511,C=52)
	CT(37907)=(S=11680512,E=11680767,C=32)
	CT(37908)=(S=11680768,E=11681023,C=25)
	CT(37909)=(S=11681024,E=11681279,C=101)
	CT(37910)=(S=11681280,E=11681535,C=13)
	CT(37911)=(S=11681536,E=11681791,C=28)
	CT(37912)=(S=11681792,E=11683839,C=18)
	CT(37913)=(S=11683840,E=11683903,C=19)
	CT(37914)=(S=11683904,E=11683967,C=18)
	CT(37915)=(S=11683968,E=11684031,C=66)
	CT(37916)=(S=11684032,E=11684095,C=33)
	CT(37917)=(S=11684096,E=11684159,C=58)
	CT(37918)=(S=11684160,E=11684223,C=23)
	CT(37919)=(S=11684224,E=11684287,C=12)
	CT(37920)=(S=11684288,E=11684351,C=19)
	CT(37921)=(S=11684352,E=11684415,C=27)
	CT(37922)=(S=11684416,E=11684543,C=18)
	CT(37923)=(S=11684544,E=11684607,C=31)
	CT(37924)=(S=11684608,E=11684735,C=18)
	CT(37925)=(S=11684736,E=11684799,C=47)
	CT(37926)=(S=11684800,E=11684863,C=51)
	CT(37927)=(S=11684864,E=11684927,C=41)
	CT(37928)=(S=11684928,E=11684991,C=53)
	CT(37929)=(S=11684992,E=11685055,C=28)
	CT(37930)=(S=11685056,E=11685119,C=18)
	CT(37931)=(S=11685120,E=11685183,C=59)
	CT(37932)=(S=11685184,E=11685247,C=28)
	CT(37933)=(S=11685248,E=11685311,C=61)
	CT(37934)=(S=11685312,E=11685375,C=33)
	CT(37935)=(S=11685376,E=11685439,C=18)
	CT(37936)=(S=11685440,E=11685503,C=14)
	CT(37937)=(S=11685504,E=11685567,C=66)
	CT(37938)=(S=11685568,E=11685631,C=19)
	CT(37939)=(S=11685632,E=11685695,C=46)
	CT(37940)=(S=11685696,E=11685759,C=52)
	CT(37941)=(S=11685760,E=11685823,C=14)
	CT(37942)=(S=11685824,E=11685887,C=13)
	CT(37943)=(S=11685888,E=11686399,C=23)
	CT(37944)=(S=11686400,E=11686911,C=34)
	CT(37945)=(S=11686912,E=11687423,C=36)
	CT(37946)=(S=11687424,E=11687935,C=23)
	CT(37947)=(S=11687936,E=11688959,C=20)
	CT(37948)=(S=11688960,E=11689983,C=31)
	CT(37949)=(S=11689984,E=11694079,C=14)
	CT(37950)=(S=11694080,E=11695103,C=42)
	CT(37951)=(S=11695104,E=11696127,C=49)
	CT(37952)=(S=11696128,E=11698175,C=68)
	CT(37953)=(S=11698176,E=11698431,C=22)
	CT(37954)=(S=11698432,E=11698943,C=18)
	CT(37955)=(S=11698944,E=11699199,C=26)
	CT(37956)=(S=11699200,E=11699207,C=18)
	CT(37957)=(S=11699208,E=11699215,C=49)
	CT(37958)=(S=11699216,E=11699231,C=12)
	CT(37959)=(S=11699232,E=11699247,C=104)
	CT(37960)=(S=11699248,E=11699255,C=37)
	CT(37961)=(S=11699256,E=11699263,C=25)
	CT(37962)=(S=11699264,E=11699271,C=28)
	CT(37963)=(S=11699272,E=11699279,C=19)
	CT(37964)=(S=11699280,E=11699287,C=51)
	CT(37965)=(S=11699288,E=11699295,C=15)
	CT(37966)=(S=11699296,E=11699303,C=18)
	CT(37967)=(S=11699304,E=11699311,C=15)
	CT(37968)=(S=11699312,E=11699391,C=53)
	CT(37969)=(S=11699392,E=11699399,C=25)
	CT(37970)=(S=11699400,E=11699407,C=18)
	CT(37971)=(S=11699408,E=11699415,C=36)
	CT(37972)=(S=11699416,E=11699423,C=96)
	CT(37973)=(S=11699424,E=11699439,C=24)
	CT(37974)=(S=11699440,E=11699447,C=28)
	CT(37975)=(S=11699448,E=11699455,C=19)
	CT(37976)=(S=11699456,E=11699711,C=31)
	CT(37977)=(S=11699712,E=11699967,C=40)
	CT(37978)=(S=11699968,E=11700223,C=39)
	CT(37979)=(S=11700224,E=11700735,C=31)
	CT(37980)=(S=11700736,E=11700991,C=38)
	CT(37981)=(S=11700992,E=11701247,C=25)
	CT(37982)=(S=11701248,E=11701759,C=18)
	CT(37983)=(S=11701760,E=11702015,C=28)
	CT(37984)=(S=11702016,E=11702271,C=71)
	CT(37985)=(S=11702272,E=11702783,C=49)
	CT(37986)=(S=11702784,E=11703295,C=22)
	CT(37987)=(S=11703296,E=11703807,C=46)
	CT(37988)=(S=11703808,E=11704319,C=31)
	CT(37989)=(S=11704320,E=11704831,C=94)
	CT(37990)=(S=11704832,E=11704959,C=68)
	CT(37991)=(S=11704960,E=11705215,C=18)
	CT(37992)=(S=11705216,E=11705343,C=24)
	CT(37993)=(S=11705344,E=11705471,C=25)
	CT(37994)=(S=11705472,E=11705599,C=38)
	CT(37995)=(S=11705600,E=11705663,C=26)
	CT(37996)=(S=11705664,E=11705727,C=38)
	CT(37997)=(S=11705728,E=11705791,C=18)
	CT(37998)=(S=11705792,E=11705855,C=24)
	CT(37999)=(S=11705856,E=11706111,C=31)
	CT(38000)=(S=11706112,E=11706127,C=36)
	CT(38001)=(S=11706128,E=11706143,C=18)
	CT(38002)=(S=11706144,E=11706159,C=31)
	CT(38003)=(S=11706160,E=11706175,C=18)
	CT(38004)=(S=11706176,E=11706191,C=38)
	CT(38005)=(S=11706192,E=11706207,C=18)
	CT(38006)=(S=11706208,E=11706239,C=31)
	CT(38007)=(S=11706240,E=11706255,C=58)
	CT(38008)=(S=11706256,E=11706271,C=38)
	CT(38009)=(S=11706272,E=11706287,C=58)
	CT(38010)=(S=11706288,E=11706303,C=31)
	CT(38011)=(S=11706304,E=11706319,C=58)
	CT(38012)=(S=11706320,E=11706351,C=31)
	CT(38013)=(S=11706352,E=11706367,C=68)
	CT(38014)=(S=11706368,E=11706495,C=57)
	CT(38015)=(S=11706496,E=11706623,C=66)
	CT(38016)=(S=11706624,E=11706751,C=101)
	CT(38017)=(S=11706752,E=11707007,C=18)
	CT(38018)=(S=11707008,E=11707135,C=28)
	CT(38019)=(S=11707136,E=11707263,C=18)
	CT(38020)=(S=11707264,E=11707391,C=68)
	CT(38021)=(S=11707392,E=11707519,C=27)
	CT(38022)=(S=11707520,E=11707647,C=50)
	CT(38023)=(S=11707648,E=11707775,C=31)
	CT(38024)=(S=11707776,E=11707903,C=42)
	CT(38025)=(S=11707904,E=11708031,C=21)
	CT(38026)=(S=11708032,E=11708287,C=18)
	CT(38027)=(S=11708288,E=11708415,C=63)
	CT(38028)=(S=11708416,E=11708543,C=53)
	CT(38029)=(S=11708544,E=11708671,C=68)
	CT(38030)=(S=11708672,E=11708799,C=18)
	CT(38031)=(S=11708800,E=11708927,C=38)
	CT(38032)=(S=11708928,E=11709055,C=12)
	CT(38033)=(S=11709056,E=11709087,C=36)
	CT(38034)=(S=11709088,E=11709311,C=18)
	CT(38035)=(S=11709312,E=11709439,C=30)
	CT(38036)=(S=11709440,E=11709567,C=52)
	CT(38037)=(S=11709568,E=11709695,C=68)
	CT(38038)=(S=11709696,E=11709823,C=18)
	CT(38039)=(S=11709824,E=11709951,C=26)
	CT(38040)=(S=11709952,E=11710079,C=34)
	CT(38041)=(S=11710080,E=11710207,C=19)
	CT(38042)=(S=11710208,E=11710335,C=102)
	CT(38043)=(S=11710336,E=11710463,C=53)
	CT(38044)=(S=11710464,E=11711487,C=18)
	CT(38045)=(S=11711488,E=11712511,C=58)
	CT(38046)=(S=11712512,E=11713535,C=18)
	CT(38047)=(S=11713536,E=11714559,C=42)
	CT(38048)=(S=11714560,E=11716607,C=34)
	CT(38049)=(S=11716608,E=11717631,C=28)
	CT(38050)=(S=11717632,E=11718655,C=18)
	CT(38051)=(S=11718656,E=11718687,C=12)
	CT(38052)=(S=11718688,E=11718719,C=49)
	CT(38053)=(S=11718720,E=11718751,C=18)
	CT(38054)=(S=11718752,E=11718783,C=28)
	CT(38055)=(S=11718784,E=11718815,C=18)
	CT(38056)=(S=11718816,E=11718847,C=36)
	CT(38057)=(S=11718848,E=11718879,C=69)
	CT(38058)=(S=11718880,E=11718911,C=18)
	CT(38059)=(S=11718912,E=11718943,C=59)
	CT(38060)=(S=11718944,E=11718975,C=34)
	CT(38061)=(S=11718976,E=11719007,C=31)
	CT(38062)=(S=11719008,E=11719039,C=18)
	CT(38063)=(S=11719040,E=11719071,C=33)
	CT(38064)=(S=11719072,E=11719103,C=24)
	CT(38065)=(S=11719104,E=11719263,C=18)
	CT(38066)=(S=11719264,E=11719295,C=28)
	CT(38067)=(S=11719296,E=11719327,C=31)
	CT(38068)=(S=11719328,E=11719359,C=37)
	CT(38069)=(S=11719360,E=11719391,C=31)
	CT(38070)=(S=11719392,E=11719423,C=50)
	CT(38071)=(S=11719424,E=11719455,C=18)
	CT(38072)=(S=11719456,E=11719487,C=37)
	CT(38073)=(S=11719488,E=11719519,C=15)
	CT(38074)=(S=11719520,E=11719583,C=31)
	CT(38075)=(S=11719584,E=11719615,C=18)
	CT(38076)=(S=11719616,E=11719647,C=37)
	CT(38077)=(S=11719648,E=11719679,C=34)
	CT(38078)=(S=11719680,E=11719687,C=20)
	CT(38079)=(S=11719688,E=11719695,C=18)
	CT(38080)=(S=11719696,E=11719703,C=42)
	CT(38081)=(S=11719704,E=11719711,C=58)
	CT(38082)=(S=11719712,E=11719719,C=38)
	CT(38083)=(S=11719720,E=11719735,C=58)
	CT(38084)=(S=11719736,E=11719743,C=38)
	CT(38085)=(S=11719744,E=11719751,C=31)
	CT(38086)=(S=11719752,E=11719767,C=18)
	CT(38087)=(S=11719768,E=11719775,C=46)
	CT(38088)=(S=11719776,E=11719791,C=31)
	CT(38089)=(S=11719792,E=11719799,C=28)
	CT(38090)=(S=11719800,E=11719807,C=58)
	CT(38091)=(S=11719808,E=11719815,C=18)
	CT(38092)=(S=11719816,E=11719823,C=15)
	CT(38093)=(S=11719824,E=11719855,C=58)
	CT(38094)=(S=11719856,E=11719863,C=18)
	CT(38095)=(S=11719864,E=11719871,C=38)
	CT(38096)=(S=11719872,E=11719879,C=31)
	CT(38097)=(S=11719880,E=11719887,C=38)
	CT(38098)=(S=11719888,E=11719895,C=18)
	CT(38099)=(S=11719896,E=11719903,C=58)
	CT(38100)=(S=11719904,E=11719911,C=18)
	CT(38101)=(S=11719912,E=11719927,C=31)
	CT(38102)=(S=11719928,E=11719931,C=18)
	CT(38103)=(S=11719932,E=11719933,C=60)
	CT(38104)=(S=11719934,E=11719934,C=34)
	CT(38105)=(S=11719935,E=11719935,C=58)
	CT(38106)=(S=11719936,E=11719943,C=31)
	CT(38107)=(S=11719944,E=11719975,C=18)
	CT(38108)=(S=11719976,E=11719983,C=33)
	CT(38109)=(S=11719984,E=11719991,C=60)
	CT(38110)=(S=11719992,E=11719999,C=58)
	CT(38111)=(S=11720000,E=11720007,C=12)
	CT(38112)=(S=11720008,E=11720008,C=58)
	CT(38113)=(S=11720009,E=11720009,C=18)
	CT(38114)=(S=11720010,E=11720010,C=28)
	CT(38115)=(S=11720011,E=11720023,C=18)
	CT(38116)=(S=11720024,E=11720039,C=58)
	CT(38117)=(S=11720040,E=11720047,C=31)
	CT(38118)=(S=11720048,E=11720055,C=18)
	CT(38119)=(S=11720056,E=11720063,C=58)
	CT(38120)=(S=11720064,E=11720071,C=66)
	CT(38121)=(S=11720072,E=11720079,C=58)
	CT(38122)=(S=11720080,E=11720087,C=18)
	CT(38123)=(S=11720088,E=11720095,C=33)
	CT(38124)=(S=11720096,E=11720103,C=58)
	CT(38125)=(S=11720104,E=11720111,C=31)
	CT(38126)=(S=11720112,E=11720119,C=38)
	CT(38127)=(S=11720120,E=11720127,C=31)
	CT(38128)=(S=11720128,E=11720151,C=18)
	CT(38129)=(S=11720152,E=11720159,C=58)
	CT(38130)=(S=11720160,E=11720167,C=18)
	CT(38131)=(S=11720168,E=11720175,C=47)
	CT(38132)=(S=11720176,E=11720191,C=18)
	CT(38133)=(S=11720192,E=11720223,C=58)
	CT(38134)=(S=11720224,E=11720255,C=18)
	CT(38135)=(S=11720256,E=11720287,C=43)
	CT(38136)=(S=11720288,E=11720319,C=36)
	CT(38137)=(S=11720320,E=11720351,C=58)
	CT(38138)=(S=11720352,E=11720415,C=31)
	CT(38139)=(S=11720416,E=11720447,C=18)
	CT(38140)=(S=11720448,E=11720511,C=26)
	CT(38141)=(S=11720512,E=11720575,C=18)
	CT(38142)=(S=11720576,E=11720591,C=14)
	CT(38143)=(S=11720592,E=11720607,C=18)
	CT(38144)=(S=11720608,E=11720639,C=31)
	CT(38145)=(S=11720640,E=11720655,C=58)
	CT(38146)=(S=11720656,E=11720671,C=43)
	CT(38147)=(S=11720672,E=11720727,C=31)
	CT(38148)=(S=11720728,E=11720735,C=58)
	CT(38149)=(S=11720736,E=11720743,C=31)
	CT(38150)=(S=11720744,E=11720751,C=58)
	CT(38151)=(S=11720752,E=11720759,C=55)
	CT(38152)=(S=11720760,E=11720767,C=52)
	CT(38153)=(S=11720768,E=11720783,C=18)
	CT(38154)=(S=11720784,E=11720791,C=33)
	CT(38155)=(S=11720792,E=11720799,C=58)
	CT(38156)=(S=11720800,E=11720807,C=31)
	CT(38157)=(S=11720808,E=11720815,C=58)
	CT(38158)=(S=11720816,E=11720823,C=63)
	CT(38159)=(S=11720824,E=11720831,C=31)
	CT(38160)=(S=11720832,E=11720839,C=98)
	CT(38161)=(S=11720840,E=11720847,C=58)
	CT(38162)=(S=11720848,E=11720855,C=33)
	CT(38163)=(S=11720856,E=11720871,C=18)
	CT(38164)=(S=11720872,E=11720879,C=31)
	CT(38165)=(S=11720880,E=11720887,C=18)
	CT(38166)=(S=11720888,E=11720895,C=31)
	CT(38167)=(S=11720896,E=11720903,C=19)
	CT(38168)=(S=11720904,E=11720911,C=58)
	CT(38169)=(S=11720912,E=11720919,C=62)
	CT(38170)=(S=11720920,E=11720927,C=18)
	CT(38171)=(S=11720928,E=11720943,C=31)
	CT(38172)=(S=11720944,E=11720951,C=38)
	CT(38173)=(S=11720952,E=11720959,C=33)
	CT(38174)=(S=11720960,E=11720967,C=18)
	CT(38175)=(S=11720968,E=11720975,C=46)
	CT(38176)=(S=11720976,E=11720983,C=49)
	CT(38177)=(S=11720984,E=11720991,C=18)
	CT(38178)=(S=11720992,E=11720999,C=58)
	CT(38179)=(S=11721000,E=11721039,C=18)
	CT(38180)=(S=11721040,E=11721047,C=36)
	CT(38181)=(S=11721056,E=11721071,C=18)
	CT(38182)=(S=11721072,E=11721079,C=58)
	CT(38183)=(S=11721080,E=11721087,C=18)
	CT(38184)=(S=11721088,E=11721095,C=47)
	CT(38185)=(S=11721096,E=11721103,C=58)
	CT(38186)=(S=11721104,E=11721111,C=33)
	CT(38187)=(S=11721112,E=11721119,C=18)
	CT(38188)=(S=11721120,E=11721127,C=31)
	CT(38189)=(S=11721128,E=11721135,C=62)
	CT(38190)=(S=11721136,E=11721143,C=18)
	CT(38191)=(S=11721144,E=11721159,C=58)
	CT(38192)=(S=11721160,E=11721175,C=31)
	CT(38193)=(S=11721176,E=11721183,C=58)
	CT(38194)=(S=11721184,E=11721191,C=18)
	CT(38195)=(S=11721200,E=11721207,C=33)
	CT(38196)=(S=11721208,E=11721215,C=58)
	CT(38197)=(S=11721216,E=11721279,C=18)
	CT(38198)=(S=11721280,E=11721295,C=31)
	CT(38199)=(S=11721296,E=11721343,C=18)
	CT(38200)=(S=11721344,E=11721359,C=38)
	CT(38201)=(S=11721360,E=11721375,C=18)
	CT(38202)=(S=11721376,E=11721391,C=57)
	CT(38203)=(S=11721392,E=11721407,C=38)
	CT(38204)=(S=11721416,E=11721423,C=98)
	CT(38205)=(S=11721424,E=11721439,C=18)
	CT(38206)=(S=11721440,E=11721455,C=58)
	CT(38207)=(S=11721472,E=11721487,C=46)
	CT(38208)=(S=11721488,E=11721503,C=58)
	CT(38209)=(S=11721504,E=11721519,C=18)
	CT(38210)=(S=11721520,E=11721535,C=66)
	CT(38211)=(S=11721536,E=11721551,C=15)
	CT(38212)=(S=11721552,E=11721567,C=31)
	CT(38213)=(S=11721568,E=11721615,C=58)
	CT(38214)=(S=11721616,E=11721631,C=18)
	CT(38215)=(S=11721632,E=11721647,C=31)
	CT(38216)=(S=11721648,E=11721663,C=18)
	CT(38217)=(S=11721664,E=11721679,C=31)
	CT(38218)=(S=11721680,E=11721695,C=18)
	CT(38219)=(S=11721696,E=11721711,C=26)
	CT(38220)=(S=11721712,E=11721727,C=31)
	CT(38221)=(S=11721728,E=11722751,C=46)
	CT(38222)=(S=11722752,E=11724799,C=36)
	CT(38223)=(S=11724800,E=11725055,C=27)
	CT(38224)=(S=11725056,E=11725311,C=37)
	CT(38225)=(S=11725312,E=11725567,C=18)
	CT(38226)=(S=11725568,E=11725823,C=58)
	CT(38227)=(S=11725824,E=11725839,C=63)
	CT(38228)=(S=11725840,E=11725855,C=18)
	CT(38229)=(S=11725856,E=11725871,C=26)
	CT(38230)=(S=11725872,E=11725887,C=40)
	CT(38231)=(S=11725888,E=11725903,C=19)
	CT(38232)=(S=11725904,E=11725919,C=59)
	CT(38233)=(S=11725920,E=11725935,C=18)
	CT(38234)=(S=11725936,E=11725951,C=25)
	CT(38235)=(S=11725952,E=11725983,C=14)
	CT(38236)=(S=11725984,E=11725999,C=15)
	CT(38237)=(S=11726000,E=11726015,C=36)
	CT(38238)=(S=11726016,E=11726031,C=55)
	CT(38239)=(S=11726032,E=11726047,C=18)
	CT(38240)=(S=11726048,E=11726063,C=12)
	CT(38241)=(S=11726064,E=11726079,C=18)
	CT(38242)=(S=11726080,E=11726095,C=15)
	CT(38243)=(S=11726096,E=11726111,C=18)
	CT(38244)=(S=11726112,E=11726127,C=36)
	CT(38245)=(S=11726144,E=11726159,C=41)
	CT(38246)=(S=11726160,E=11726175,C=34)
	CT(38247)=(S=11726176,E=11726191,C=12)
	CT(38248)=(S=11726192,E=11726207,C=14)
	CT(38249)=(S=11726208,E=11726239,C=25)
	CT(38250)=(S=11726240,E=11726255,C=14)
	CT(38251)=(S=11726256,E=11726287,C=18)
	CT(38252)=(S=11726288,E=11726303,C=46)
	CT(38253)=(S=11726304,E=11726319,C=25)
	CT(38254)=(S=11726320,E=11726367,C=18)
	CT(38255)=(S=11726368,E=11726383,C=33)
	CT(38256)=(S=11726384,E=11726399,C=15)
	CT(38257)=(S=11726400,E=11726415,C=20)
	CT(38258)=(S=11726416,E=11726431,C=14)
	CT(38259)=(S=11726432,E=11726447,C=36)
	CT(38260)=(S=11726448,E=11726463,C=18)
	CT(38261)=(S=11726464,E=11726495,C=14)
	CT(38262)=(S=11726496,E=11726511,C=34)
	CT(38263)=(S=11726512,E=11726527,C=61)
	CT(38264)=(S=11726528,E=11726543,C=26)
	CT(38265)=(S=11726544,E=11726559,C=50)
	CT(38266)=(S=11726560,E=11726575,C=28)
	CT(38267)=(S=11726576,E=11726607,C=58)
	CT(38268)=(S=11726608,E=11726623,C=59)
	CT(38269)=(S=11726624,E=11726639,C=15)
	CT(38270)=(S=11726640,E=11726655,C=36)
	CT(38271)=(S=11726656,E=11726671,C=28)
	CT(38272)=(S=11726672,E=11726703,C=14)
	CT(38273)=(S=11726704,E=11726719,C=51)
	CT(38274)=(S=11726720,E=11726735,C=58)
	CT(38275)=(S=11726736,E=11726751,C=26)
	CT(38276)=(S=11726752,E=11726767,C=14)
	CT(38277)=(S=11726768,E=11726783,C=15)
	CT(38278)=(S=11726784,E=11726799,C=47)
	CT(38279)=(S=11726800,E=11726815,C=25)
	CT(38280)=(S=11726816,E=11726831,C=51)
	CT(38281)=(S=11726832,E=11726847,C=25)
	CT(38282)=(S=11726848,E=11728895,C=37)
	CT(38283)=(S=11728896,E=11728903,C=18)
	CT(38284)=(S=11728904,E=11728911,C=33)
	CT(38285)=(S=11728912,E=11728919,C=48)
	CT(38286)=(S=11728920,E=11728927,C=19)
	CT(38287)=(S=11728928,E=11728935,C=14)
	CT(38288)=(S=11728936,E=11728943,C=26)
	CT(38289)=(S=11728944,E=11728951,C=15)
	CT(38290)=(S=11728952,E=11728959,C=33)
	CT(38291)=(S=11728960,E=11728967,C=18)
	CT(38292)=(S=11728968,E=11728975,C=58)
	CT(38293)=(S=11728976,E=11728983,C=18)
	CT(38294)=(S=11728984,E=11728991,C=15)
	CT(38295)=(S=11728992,E=11728999,C=27)
	CT(38296)=(S=11729000,E=11729015,C=14)
	CT(38297)=(S=11729016,E=11729023,C=27)
	CT(38298)=(S=11729024,E=11729031,C=14)
	CT(38299)=(S=11729032,E=11729039,C=55)
	CT(38300)=(S=11729040,E=11729047,C=15)
	CT(38301)=(S=11729048,E=11729055,C=36)
	CT(38302)=(S=11729056,E=11729063,C=49)
	CT(38303)=(S=11729064,E=11729071,C=36)
	CT(38304)=(S=11729072,E=11729079,C=15)
	CT(38305)=(S=11729080,E=11729095,C=12)
	CT(38306)=(S=11729096,E=11729103,C=50)
	CT(38307)=(S=11729104,E=11729111,C=12)
	CT(38308)=(S=11729112,E=11729119,C=28)
	CT(38309)=(S=11729120,E=11729127,C=36)
	CT(38310)=(S=11729128,E=11729135,C=18)
	CT(38311)=(S=11729136,E=11729143,C=28)
	CT(38312)=(S=11729144,E=11729151,C=33)
	CT(38313)=(S=11729152,E=11729159,C=28)
	CT(38314)=(S=11729160,E=11729167,C=25)
	CT(38315)=(S=11729168,E=11729175,C=40)
	CT(38316)=(S=11729176,E=11729183,C=28)
	CT(38317)=(S=11729184,E=11729191,C=60)
	CT(38318)=(S=11729192,E=11729199,C=14)
	CT(38319)=(S=11729200,E=11729207,C=24)
	CT(38320)=(S=11729208,E=11729231,C=18)
	CT(38321)=(S=11729232,E=11729239,C=28)
	CT(38322)=(S=11729240,E=11729247,C=58)
	CT(38323)=(S=11729248,E=11729255,C=14)
	CT(38324)=(S=11729256,E=11729263,C=17)
	CT(38325)=(S=11729264,E=11729271,C=54)
	CT(38326)=(S=11729272,E=11729279,C=27)
	CT(38327)=(S=11729280,E=11729287,C=18)
	CT(38328)=(S=11729288,E=11729295,C=28)
	CT(38329)=(S=11729296,E=11729303,C=19)
	CT(38330)=(S=11729304,E=11729311,C=36)
	CT(38331)=(S=11729312,E=11729319,C=12)
	CT(38332)=(S=11729320,E=11729327,C=51)
	CT(38333)=(S=11729328,E=11729335,C=101)
	CT(38334)=(S=11729336,E=11729343,C=15)
	CT(38335)=(S=11729344,E=11729351,C=70)
	CT(38336)=(S=11729352,E=11729359,C=15)
	CT(38337)=(S=11729360,E=11729367,C=33)
	CT(38338)=(S=11729368,E=11729383,C=15)
	CT(38339)=(S=11729392,E=11729399,C=18)
	CT(38340)=(S=11729400,E=11729407,C=36)
	CT(38341)=(S=11729408,E=11729415,C=12)
	CT(38342)=(S=11729416,E=11729423,C=28)
	CT(38343)=(S=11729424,E=11729431,C=36)
	CT(38344)=(S=11729432,E=11729439,C=34)
	CT(38345)=(S=11729440,E=11729447,C=44)
	CT(38346)=(S=11729448,E=11729455,C=58)
	CT(38347)=(S=11729456,E=11729463,C=14)
	CT(38348)=(S=11729464,E=11729471,C=56)
	CT(38349)=(S=11729472,E=11729479,C=15)
	CT(38350)=(S=11729480,E=11729487,C=14)
	CT(38351)=(S=11729488,E=11729495,C=28)
	CT(38352)=(S=11729496,E=11729503,C=37)
	CT(38353)=(S=11729504,E=11729511,C=14)
	CT(38354)=(S=11729512,E=11729519,C=12)
	CT(38355)=(S=11729520,E=11729527,C=63)
	CT(38356)=(S=11729528,E=11729535,C=28)
	CT(38357)=(S=11729536,E=11729543,C=34)
	CT(38358)=(S=11729544,E=11729551,C=46)
	CT(38359)=(S=11729552,E=11729559,C=36)
	CT(38360)=(S=11729560,E=11729567,C=18)
	CT(38361)=(S=11729568,E=11729583,C=28)
	CT(38362)=(S=11729584,E=11729591,C=14)
	CT(38363)=(S=11729592,E=11729599,C=38)
	CT(38364)=(S=11729600,E=11729607,C=36)
	CT(38365)=(S=11729608,E=11729615,C=15)
	CT(38366)=(S=11729616,E=11729623,C=12)
	CT(38367)=(S=11729624,E=11729631,C=27)
	CT(38368)=(S=11729632,E=11729647,C=28)
	CT(38369)=(S=11729648,E=11729655,C=18)
	CT(38370)=(S=11729656,E=11729663,C=16)
	CT(38371)=(S=11729664,E=11729671,C=24)
	CT(38372)=(S=11729672,E=11729679,C=28)
	CT(38373)=(S=11729680,E=11729687,C=12)
	CT(38374)=(S=11729688,E=11729695,C=36)
	CT(38375)=(S=11729696,E=11729703,C=14)
	CT(38376)=(S=11729704,E=11729711,C=37)
	CT(38377)=(S=11729712,E=11729719,C=28)
	CT(38378)=(S=11729720,E=11729727,C=47)
	CT(38379)=(S=11729728,E=11729735,C=42)
	CT(38380)=(S=11729736,E=11729743,C=15)
	CT(38381)=(S=11729744,E=11729751,C=12)
	CT(38382)=(S=11729752,E=11729759,C=28)
	CT(38383)=(S=11729760,E=11729767,C=18)
	CT(38384)=(S=11729768,E=11729775,C=31)
	CT(38385)=(S=11729776,E=11729783,C=28)
	CT(38386)=(S=11729784,E=11729791,C=36)
	CT(38387)=(S=11729792,E=11729799,C=19)
	CT(38388)=(S=11729800,E=11729807,C=18)
	CT(38389)=(S=11729808,E=11729823,C=47)
	CT(38390)=(S=11729824,E=11729831,C=70)
	CT(38391)=(S=11729832,E=11729847,C=14)
	CT(38392)=(S=11729848,E=11729855,C=33)
	CT(38393)=(S=11729856,E=11729863,C=36)
	CT(38394)=(S=11729864,E=11729871,C=12)
	CT(38395)=(S=11729872,E=11729879,C=26)
	CT(38396)=(S=11729880,E=11729887,C=18)
	CT(38397)=(S=11729888,E=11729895,C=28)
	CT(38398)=(S=11729896,E=11729903,C=14)
	CT(38399)=(S=11729904,E=11729911,C=19)
	CT(38400)=(S=11729912,E=11729919,C=12)
	CT(38401)=(S=11729920,E=11729983,C=58)
	CT(38402)=(S=11729984,E=11730047,C=18)
	CT(38403)=(S=11730048,E=11730111,C=26)
	CT(38404)=(S=11730112,E=11730175,C=18)
	CT(38405)=(S=11730176,E=11730239,C=26)
	CT(38406)=(S=11730240,E=11730303,C=30)
	CT(38407)=(S=11730304,E=11730367,C=71)
	CT(38408)=(S=11730368,E=11730431,C=46)
	CT(38409)=(S=11730432,E=11730495,C=28)
	CT(38410)=(S=11730496,E=11730559,C=70)
	CT(38411)=(S=11730560,E=11730623,C=46)
	CT(38412)=(S=11730624,E=11730687,C=51)
	CT(38413)=(S=11730688,E=11730695,C=28)
	CT(38414)=(S=11730696,E=11730703,C=55)
	CT(38415)=(S=11730704,E=11730711,C=28)
	CT(38416)=(S=11730712,E=11730719,C=36)
	CT(38417)=(S=11730720,E=11730727,C=38)
	CT(38418)=(S=11730728,E=11730735,C=58)
	CT(38419)=(S=11730736,E=11730743,C=36)
	CT(38420)=(S=11730744,E=11730759,C=14)
	CT(38421)=(S=11730760,E=11730767,C=15)
	CT(38422)=(S=11730768,E=11730783,C=14)
	CT(38423)=(S=11730784,E=11730791,C=12)
	CT(38424)=(S=11730792,E=11730799,C=25)
	CT(38425)=(S=11730808,E=11730815,C=18)
	CT(38426)=(S=11730816,E=11730823,C=12)
	CT(38427)=(S=11730824,E=11730831,C=28)
	CT(38428)=(S=11730832,E=11730839,C=27)
	CT(38429)=(S=11730840,E=11730847,C=42)
	CT(38430)=(S=11730848,E=11730855,C=12)
	CT(38431)=(S=11730856,E=11730863,C=33)
	CT(38432)=(S=11730864,E=11730871,C=31)
	CT(38433)=(S=11730872,E=11730879,C=15)
	CT(38434)=(S=11730880,E=11730887,C=36)
	CT(38435)=(S=11730888,E=11730895,C=18)
	CT(38436)=(S=11730896,E=11730903,C=12)
	CT(38437)=(S=11730904,E=11730911,C=70)
	CT(38438)=(S=11730912,E=11730919,C=25)
	CT(38439)=(S=11730920,E=11730927,C=18)
	CT(38440)=(S=11730928,E=11730935,C=15)
	CT(38441)=(S=11730936,E=11730943,C=18)
	CT(38442)=(S=11730944,E=11730945,C=83)
	CT(38443)=(S=11730946,E=11730947,C=203)
	CT(38444)=(S=11730948,E=11730951,C=83)
	CT(38445)=(S=11730952,E=11730952,C=203)
	CT(38446)=(S=11730953,E=11730954,C=181)
	CT(38447)=(S=11730955,E=11730955,C=83)
	CT(38448)=(S=11730956,E=11730957,C=210)
	CT(38449)=(S=11730958,E=11730958,C=83)
	CT(38450)=(S=11730959,E=11730959,C=181)
	CT(38451)=(S=11730960,E=11730963,C=203)
	CT(38452)=(S=11730964,E=11730967,C=202)
	CT(38453)=(S=11730968,E=11730968,C=181)
	CT(38454)=(S=11730969,E=11730969,C=83)
	CT(38455)=(S=11730970,E=11730970,C=206)
	CT(38456)=(S=11730971,E=11730971,C=181)
	CT(38457)=(S=11730972,E=11730972,C=176)
	CT(38458)=(S=11730973,E=11730973,C=181)
	CT(38459)=(S=11730974,E=11730974,C=201)
	CT(38460)=(S=11730975,E=11730975,C=208)
	CT(38461)=(S=11730976,E=11731071,C=202)
	CT(38462)=(S=11731072,E=11731072,C=203)
	CT(38463)=(S=11731073,E=11731073,C=213)
	CT(38464)=(S=11731074,E=11731089,C=83)
	CT(38465)=(S=11731090,E=11731090,C=181)
	CT(38466)=(S=11731091,E=11731095,C=83)
	CT(38467)=(S=11731096,E=11731099,C=203)
	CT(38468)=(S=11731100,E=11731103,C=205)
	CT(38469)=(S=11731104,E=11731135,C=202)
	CT(38470)=(S=11731136,E=11731136,C=215)
	CT(38471)=(S=11731137,E=11731137,C=83)
	CT(38472)=(S=11731138,E=11731139,C=209)
	CT(38473)=(S=11731140,E=11731143,C=176)
	CT(38474)=(S=11731144,E=11731145,C=209)
	CT(38475)=(S=11731146,E=11731146,C=202)
	CT(38476)=(S=11731147,E=11731147,C=83)
	CT(38477)=(S=11731148,E=11731148,C=213)
	CT(38478)=(S=11731149,E=11731149,C=181)
	CT(38479)=(S=11731150,E=11731151,C=189)
	CT(38480)=(S=11731152,E=11731167,C=202)
	CT(38481)=(S=11731168,E=11731199,C=83)
	CT(38482)=(S=11731200,E=11731455,C=181)
	CT(38483)=(S=11731456,E=11732223,C=203)
	CT(38484)=(S=11732224,E=11732479,C=210)
	CT(38485)=(S=11732480,E=11732991,C=212)
	CT(38486)=(S=11732992,E=11733503,C=203)
	CT(38487)=(S=11733504,E=11734015,C=189)
	CT(38488)=(S=11734016,E=11735039,C=181)
	CT(38489)=(S=11735040,E=11735551,C=189)
	CT(38490)=(S=11735552,E=11736063,C=181)
	CT(38491)=(S=11736064,E=11737087,C=200)
	CT(38492)=(S=11737088,E=11739135,C=205)
	CT(38493)=(S=11739136,E=11739647,C=181)
	CT(38494)=(S=11739648,E=11740159,C=189)
	CT(38495)=(S=11740160,E=11741695,C=83)
	CT(38496)=(S=11741696,E=11741855,C=209)
	CT(38497)=(S=11741856,E=11741867,C=83)
	CT(38498)=(S=11741868,E=11741871,C=181)
	CT(38499)=(S=11741872,E=11741887,C=83)
	CT(38500)=(S=11741888,E=11741951,C=170)
	CT(38501)=(S=11742016,E=11742031,C=83)
	CT(38502)=(S=11742032,E=11742051,C=212)
	CT(38503)=(S=11742052,E=11742055,C=83)
	CT(38504)=(S=11742056,E=11742063,C=181)
	CT(38505)=(S=11742064,E=11742079,C=83)
	CT(38506)=(S=11742080,E=11742143,C=209)
	CT(38507)=(S=11742144,E=11742207,C=177)
	CT(38508)=(S=11742208,E=11743231,C=200)
	CT(38509)=(S=11743232,E=11743479,C=209)
	CT(38510)=(S=11743480,E=11743483,C=202)
	CT(38511)=(S=11743484,E=11743487,C=83)
	CT(38512)=(S=11743488,E=11743551,C=213)
	CT(38513)=(S=11743552,E=11743559,C=212)
	CT(38514)=(S=11743560,E=11743567,C=83)
	CT(38515)=(S=11743568,E=11743583,C=177)
	CT(38516)=(S=11743584,E=11743599,C=83)
	CT(38517)=(S=11743600,E=11743607,C=208)
	CT(38518)=(S=11743608,E=11743615,C=83)
	CT(38519)=(S=11743616,E=11743871,C=181)
	CT(38520)=(S=11743872,E=11743999,C=202)
	CT(38521)=(S=11744000,E=11744063,C=210)
	CT(38522)=(S=11744064,E=11744079,C=177)
	CT(38523)=(S=11744080,E=11744095,C=83)
	CT(38524)=(S=11744096,E=11744127,C=181)
	CT(38525)=(S=11744128,E=11744223,C=202)
	CT(38526)=(S=11744224,E=11744247,C=83)
	CT(38527)=(S=11744252,E=11744255,C=170)
	CT(38528)=(S=11744256,E=11744767,C=177)
	CT(38529)=(S=11744768,E=11745279,C=189)
	CT(38530)=(S=11745280,E=11745791,C=203)
	CT(38531)=(S=11745792,E=11746303,C=201)
	CT(38532)=(S=11746304,E=11746335,C=83)
	CT(38533)=(S=11746336,E=11746351,C=181)
	CT(38534)=(S=11746352,E=11746367,C=213)
	CT(38535)=(S=11746368,E=11746399,C=203)
	CT(38536)=(S=11746400,E=11746415,C=83)
	CT(38537)=(S=11746416,E=11746431,C=201)
	CT(38538)=(S=11746448,E=11746455,C=170)
	CT(38539)=(S=11746456,E=11746463,C=83)
	CT(38540)=(S=11746464,E=11746471,C=202)
	CT(38541)=(S=11746472,E=11746479,C=83)
	CT(38542)=(S=11746480,E=11746487,C=208)
	CT(38543)=(S=11746488,E=11746495,C=83)
	CT(38544)=(S=11746496,E=11746499,C=29)
	CT(38545)=(S=11746500,E=11746507,C=83)
	CT(38546)=(S=11746508,E=11746511,C=212)
	CT(38547)=(S=11746512,E=11746515,C=83)
	CT(38548)=(S=11746516,E=11746519,C=176)
	CT(38549)=(S=11746520,E=11746527,C=83)
	CT(38550)=(S=11746528,E=11746531,C=212)
	CT(38551)=(S=11746532,E=11746543,C=83)
	CT(38552)=(S=11746544,E=11746547,C=181)
	CT(38553)=(S=11746548,E=11746551,C=213)
	CT(38554)=(S=11746552,E=11746571,C=83)
	CT(38555)=(S=11746572,E=11746575,C=203)
	CT(38556)=(S=11746576,E=11746655,C=83)
	CT(38557)=(S=11746656,E=11746671,C=203)
	CT(38558)=(S=11746672,E=11746687,C=181)
	CT(38559)=(S=11746688,E=11746815,C=203)
	CT(38560)=(S=11746816,E=11747071,C=83)
	CT(38561)=(S=11747072,E=11747263,C=202)
	CT(38562)=(S=11747264,E=11747271,C=209)
	CT(38563)=(S=11747272,E=11747287,C=83)
	CT(38564)=(S=11747288,E=11747295,C=170)
	CT(38565)=(S=11747296,E=11747307,C=203)
	CT(38566)=(S=11747308,E=11747315,C=83)
	CT(38567)=(S=11747316,E=11747319,C=203)
	CT(38568)=(S=11747320,E=11747323,C=83)
	CT(38569)=(S=11747324,E=11747327,C=208)
	CT(38570)=(S=11747328,E=11796479,C=189)
	CT(38571)=(S=11796480,E=11812863,C=2)
	CT(38572)=(S=11812864,E=11814911,C=6)
	CT(38573)=(S=11814912,E=11815935,C=5)
	CT(38574)=(S=11815936,E=11816959,C=1)
	CT(38575)=(S=11816960,E=11817983,C=6)
	CT(38576)=(S=11817984,E=11818751,C=1)
	CT(38577)=(S=11818752,E=11819007,C=4)
	CT(38578)=(S=11819008,E=11820031,C=1)
	CT(38579)=(S=11820032,E=11820047,C=9)
	CT(38580)=(S=11820048,E=11820095,C=2)
	CT(38581)=(S=11820096,E=11820159,C=6)
	CT(38582)=(S=11820160,E=11820191,C=77)
	CT(38583)=(S=11820192,E=11820207,C=4)
	CT(38584)=(S=11820208,E=11820211,C=0)
	CT(38585)=(S=11820212,E=11820215,C=8)
	CT(38586)=(S=11820216,E=11820223,C=7)
	CT(38587)=(S=11820224,E=11820231,C=0)
	CT(38588)=(S=11820232,E=11820239,C=4)
	CT(38589)=(S=11820240,E=11820243,C=0)
	CT(38590)=(S=11820244,E=11820247,C=74)
	CT(38591)=(S=11820248,E=11820255,C=0)
	CT(38592)=(S=11820256,E=11820271,C=84)
	CT(38593)=(S=11820272,E=11820287,C=6)
	CT(38594)=(S=11820288,E=11820543,C=11)
	CT(38595)=(S=11820544,E=11820583,C=10)
	CT(38596)=(S=11820584,E=11820591,C=2)
	CT(38597)=(S=11820592,E=11820599,C=8)
	CT(38598)=(S=11820600,E=11820607,C=1)
	CT(38599)=(S=11820608,E=11820639,C=89)
	CT(38600)=(S=11820640,E=11820655,C=1)
	CT(38601)=(S=11820656,E=11820663,C=0)
	CT(38602)=(S=11820672,E=11820735,C=87)
	CT(38603)=(S=11820736,E=11820759,C=2)
	CT(38604)=(S=11820760,E=11820763,C=7)
	CT(38605)=(S=11820764,E=11820767,C=0)
	CT(38606)=(S=11820768,E=11820775,C=8)
	CT(38607)=(S=11820776,E=11820783,C=0)
	CT(38608)=(S=11820784,E=11820799,C=2)
	CT(38609)=(S=11820800,E=11820927,C=0)
	CT(38610)=(S=11820928,E=11829247,C=1)
	CT(38611)=(S=11829248,E=11829503,C=3)
	CT(38612)=(S=11829504,E=11829631,C=7)
	CT(38613)=(S=11829632,E=11830015,C=1)
	CT(38614)=(S=11830016,E=11830079,C=6)
	CT(38615)=(S=11830080,E=11830143,C=2)
	CT(38616)=(S=11830144,E=11830151,C=0)
	CT(38617)=(S=11830152,E=11830159,C=2)
	CT(38618)=(S=11830160,E=11830163,C=74)
	CT(38619)=(S=11830164,E=11830167,C=154)
	CT(38620)=(S=11830168,E=11830175,C=2)
	CT(38621)=(S=11830176,E=11830207,C=8)
	CT(38622)=(S=11830208,E=11830271,C=2)
	CT(38623)=(S=11830272,E=11831295,C=6)
	CT(38624)=(S=11831296,E=11833343,C=1)
	CT(38625)=(S=11833344,E=11834367,C=2)
	CT(38626)=(S=11834368,E=11834375,C=11)
	CT(38627)=(S=11834376,E=11834379,C=0)
	CT(38628)=(S=11834380,E=11834383,C=4)
	CT(38629)=(S=11834384,E=11834391,C=1)
	CT(38630)=(S=11834392,E=11834395,C=0)
	CT(38631)=(S=11834396,E=11834399,C=2)
	CT(38632)=(S=11834400,E=11834431,C=4)
	CT(38633)=(S=11834432,E=11834495,C=0)
	CT(38634)=(S=11834496,E=11834511,C=11)
	CT(38635)=(S=11834512,E=11834519,C=2)
	CT(38636)=(S=11834520,E=11834527,C=1)
	CT(38637)=(S=11834528,E=11834543,C=2)
	CT(38638)=(S=11834544,E=11834547,C=0)
	CT(38639)=(S=11834548,E=11834551,C=6)
	CT(38640)=(S=11834552,E=11834575,C=2)
	CT(38641)=(S=11834576,E=11834583,C=84)
	CT(38642)=(S=11834584,E=11834623,C=1)
	CT(38643)=(S=11834624,E=11834655,C=84)
	CT(38644)=(S=11834656,E=11834687,C=4)
	CT(38645)=(S=11834688,E=11834751,C=92)
	CT(38646)=(S=11834752,E=11834783,C=1)
	CT(38647)=(S=11834784,E=11834815,C=2)
	CT(38648)=(S=11834816,E=11834823,C=0)
	CT(38649)=(S=11834824,E=11834824,C=2)
	CT(38650)=(S=11834825,E=11834825,C=0)
	CT(38651)=(S=11834826,E=11834831,C=2)
	CT(38652)=(S=11834832,E=11834847,C=77)
	CT(38653)=(S=11834864,E=11834871,C=4)
	CT(38654)=(S=11834872,E=11834875,C=0)
	CT(38655)=(S=11834876,E=11834879,C=8)
	CT(38656)=(S=11834880,E=11835007,C=0)
	CT(38657)=(S=11835008,E=11835039,C=8)
	CT(38658)=(S=11835040,E=11835071,C=1)
	CT(38659)=(S=11835072,E=11835111,C=6)
	CT(38660)=(S=11835112,E=11835119,C=74)
	CT(38661)=(S=11835120,E=11835123,C=4)
	CT(38662)=(S=11835124,E=11835127,C=74)
	CT(38663)=(S=11835128,E=11835131,C=4)
	CT(38664)=(S=11835132,E=11835135,C=76)
	CT(38665)=(S=11835136,E=11835391,C=4)
	CT(38666)=(S=11835392,E=11841535,C=1)
	CT(38667)=(S=11841536,E=11842047,C=9)
	CT(38668)=(S=11842048,E=11842079,C=4)
	CT(38669)=(S=11842080,E=11842111,C=8)
	CT(38670)=(S=11842112,E=11842115,C=0)
	CT(38671)=(S=11842116,E=11842119,C=2)
	CT(38672)=(S=11842120,E=11842127,C=7)
	CT(38673)=(S=11842128,E=11842139,C=2)
	CT(38674)=(S=11842140,E=11842159,C=74)
	CT(38675)=(S=11842168,E=11842171,C=0)
	CT(38676)=(S=11842172,E=11842175,C=86)
	CT(38677)=(S=11842176,E=11842239,C=77)
	CT(38678)=(S=11842240,E=11842303,C=1)
	CT(38679)=(S=11842304,E=11842559,C=4)
	CT(38680)=(S=11842560,E=11842815,C=3)
	CT(38681)=(S=11842816,E=11843071,C=0)
	CT(38682)=(S=11843072,E=11843327,C=6)
	CT(38683)=(S=11843328,E=11843583,C=3)
	CT(38684)=(S=11843584,E=11844735,C=1)
	CT(38685)=(S=11844736,E=11844811,C=8)
	CT(38686)=(S=11844812,E=11844815,C=0)
	CT(38687)=(S=11844816,E=11844831,C=2)
	CT(38688)=(S=11844832,E=11844863,C=4)
	CT(38689)=(S=11844864,E=11844879,C=2)
	CT(38690)=(S=11844880,E=11844895,C=0)
	CT(38691)=(S=11844896,E=11844927,C=2)
	CT(38692)=(S=11844928,E=11844991,C=6)
	CT(38693)=(S=11844992,E=11844999,C=2)
	CT(38694)=(S=11845000,E=11845011,C=0)
	CT(38695)=(S=11845012,E=11845015,C=1)
	CT(38696)=(S=11845016,E=11845023,C=0)
	CT(38697)=(S=11845024,E=11845039,C=165)
	CT(38698)=(S=11845040,E=11845055,C=6)
	CT(38699)=(S=11845056,E=11845087,C=73)
	CT(38700)=(S=11845088,E=11845119,C=2)
	CT(38701)=(S=11845120,E=11846655,C=10)
	CT(38702)=(S=11846656,E=11847807,C=2)
	CT(38703)=(S=11847808,E=11847903,C=0)
	CT(38704)=(S=11847904,E=11847911,C=2)
	CT(38705)=(S=11847912,E=11847915,C=5)
	CT(38706)=(S=11847916,E=11847919,C=84)
	CT(38707)=(S=11847920,E=11847923,C=4)
	CT(38708)=(S=11847924,E=11847927,C=0)
	CT(38709)=(S=11847928,E=11847931,C=2)
	CT(38710)=(S=11847932,E=11848703,C=1)
	CT(38711)=(S=11848704,E=11849727,C=9)
	CT(38712)=(S=11849728,E=11850239,C=1)
	CT(38713)=(S=11850240,E=11850367,C=6)
	CT(38714)=(S=11850368,E=11850431,C=84)
	CT(38715)=(S=11850432,E=11850439,C=6)
	CT(38716)=(S=11850440,E=11850447,C=7)
	CT(38717)=(S=11850448,E=11850451,C=73)
	CT(38718)=(S=11850456,E=11850459,C=3)
	CT(38719)=(S=11850464,E=11850495,C=1)
	CT(38720)=(S=11850496,E=11850559,C=6)
	CT(38721)=(S=11850560,E=11850567,C=4)
	CT(38722)=(S=11850568,E=11850575,C=2)
	CT(38723)=(S=11850576,E=11850579,C=4)
	CT(38724)=(S=11850580,E=11850583,C=2)
	CT(38725)=(S=11850584,E=11850591,C=74)
	CT(38726)=(S=11850592,E=11850623,C=4)
	CT(38727)=(S=11850624,E=11850751,C=84)
	CT(38728)=(S=11850752,E=11851263,C=1)
	CT(38729)=(S=11851264,E=11851327,C=2)
	CT(38730)=(S=11851328,E=11851359,C=0)
	CT(38731)=(S=11851360,E=11851391,C=90)
	CT(38732)=(S=11851392,E=11851423,C=4)
	CT(38733)=(S=11851424,E=11851431,C=8)
	CT(38734)=(S=11851432,E=11851439,C=0)
	CT(38735)=(S=11851440,E=11851455,C=9)
	CT(38736)=(S=11851456,E=11851487,C=3)
	CT(38737)=(S=11851488,E=11851495,C=2)
	CT(38738)=(S=11851496,E=11851499,C=74)
	CT(38739)=(S=11851500,E=11851503,C=11)
	CT(38740)=(S=11851504,E=11851519,C=74)
	CT(38741)=(S=11851520,E=11851775,C=4)
	CT(38742)=(S=11851776,E=11852031,C=0)
	CT(38743)=(S=11852032,E=11852543,C=9)
	CT(38744)=(S=11852544,E=11852799,C=8)
	CT(38745)=(S=11852800,E=11853311,C=2)
	CT(38746)=(S=11853312,E=11853343,C=0)
	CT(38747)=(S=11853344,E=11853375,C=2)
	CT(38748)=(S=11853376,E=11853383,C=73)
	CT(38749)=(S=11853384,E=11853407,C=2)
	CT(38750)=(S=11853408,E=11853431,C=4)
	CT(38751)=(S=11853440,E=11853447,C=7)
	CT(38752)=(S=11853448,E=11853455,C=89)
	CT(38753)=(S=11853456,E=11853471,C=3)
	CT(38754)=(S=11853472,E=11853487,C=73)
	CT(38755)=(S=11853488,E=11853503,C=2)
	CT(38756)=(S=11853504,E=11853511,C=0)
	CT(38757)=(S=11853512,E=11853515,C=8)
	CT(38758)=(S=11853516,E=11853519,C=9)
	CT(38759)=(S=11853520,E=11853523,C=160)
	CT(38760)=(S=11853524,E=11853527,C=4)
	CT(38761)=(S=11853528,E=11853531,C=74)
	CT(38762)=(S=11853532,E=11853535,C=6)
	CT(38763)=(S=11853536,E=11853823,C=1)
	CT(38764)=(S=11853824,E=11855871,C=6)
	CT(38765)=(S=11855872,E=11856127,C=10)
	CT(38766)=(S=11856128,E=11856223,C=1)
	CT(38767)=(S=11856224,E=11856239,C=2)
	CT(38768)=(S=11856240,E=11856247,C=8)
	CT(38769)=(S=11856248,E=11856251,C=4)
	CT(38770)=(S=11856252,E=11856259,C=0)
	CT(38771)=(S=11856260,E=11856271,C=2)
	CT(38772)=(S=11856280,E=11856287,C=74)
	CT(38773)=(S=11856288,E=11856319,C=8)
	CT(38774)=(S=11856320,E=11856383,C=6)
	CT(38775)=(S=11856384,E=11856639,C=84)
	CT(38776)=(S=11856640,E=11856703,C=2)
	CT(38777)=(S=11856704,E=11856735,C=1)
	CT(38778)=(S=11856736,E=11856743,C=2)
	CT(38779)=(S=11856744,E=11856771,C=0)
	CT(38780)=(S=11856772,E=11856775,C=8)
	CT(38781)=(S=11856780,E=11856787,C=2)
	CT(38782)=(S=11856788,E=11856791,C=74)
	CT(38783)=(S=11856792,E=11856795,C=7)
	CT(38784)=(S=11856796,E=11856799,C=0)
	CT(38785)=(S=11856800,E=11856831,C=92)
	CT(38786)=(S=11856832,E=11856863,C=0)
	CT(38787)=(S=11856864,E=11856895,C=2)
	CT(38788)=(S=11856896,E=11857919,C=6)
	CT(38789)=(S=11857920,E=11858047,C=74)
	CT(38790)=(S=11858048,E=11858175,C=7)
	CT(38791)=(S=11858176,E=11861759,C=74)
	CT(38792)=(S=11861760,E=11862015,C=7)
	CT(38793)=(S=11862016,E=11866367,C=83)
	CT(38794)=(S=11866368,E=11867135,C=200)
	CT(38795)=(S=11867136,E=11870207,C=83)
	CT(38796)=(S=11870208,E=11870719,C=181)
	CT(38797)=(S=11870720,E=11871231,C=200)
	CT(38798)=(S=11871232,E=11871743,C=177)
	CT(38799)=(S=11871744,E=11871999,C=209)
	CT(38800)=(S=11872000,E=11872255,C=213)
	CT(38801)=(S=11872256,E=11872511,C=206)
	CT(38802)=(S=11872512,E=11872575,C=170)
	CT(38803)=(S=11872576,E=11872639,C=214)
	CT(38804)=(S=11872640,E=11872703,C=170)
	CT(38805)=(S=11872704,E=11872735,C=203)
	CT(38806)=(S=11872736,E=11872767,C=83)
	CT(38807)=(S=11872768,E=11873279,C=203)
	CT(38808)=(S=11873280,E=11874303,C=83)
	CT(38809)=(S=11874304,E=11878399,C=181)
	CT(38810)=(S=11878400,E=11879423,C=212)
	CT(38811)=(S=11879424,E=11880447,C=181)
	CT(38812)=(S=11880448,E=11881471,C=203)
	CT(38813)=(S=11881472,E=11881983,C=189)
	CT(38814)=(S=11881984,E=11890687,C=83)
	CT(38815)=(S=11890688,E=11891199,C=213)
	CT(38816)=(S=11891200,E=11891231,C=215)
	CT(38817)=(S=11891232,E=11891255,C=83)
	CT(38818)=(S=11891256,E=11891259,C=208)
	CT(38819)=(S=11891260,E=11891263,C=83)
	CT(38820)=(S=11891264,E=11891327,C=201)
	CT(38821)=(S=11891328,E=11891359,C=83)
	CT(38822)=(S=11891360,E=11891391,C=201)
	CT(38823)=(S=11891392,E=11891431,C=83)
	CT(38824)=(S=11891432,E=11891439,C=203)
	CT(38825)=(S=11891440,E=11891455,C=170)
	CT(38826)=(S=11891456,E=11891583,C=208)
	CT(38827)=(S=11891584,E=11891711,C=201)
	CT(38828)=(S=11891712,E=11892255,C=83)
	CT(38829)=(S=11892256,E=11892287,C=209)
	CT(38830)=(S=11892288,E=11892367,C=83)
	CT(38831)=(S=11892368,E=11892383,C=181)
	CT(38832)=(S=11892384,E=11892399,C=203)
	CT(38833)=(S=11892400,E=11892735,C=83)
	CT(38834)=(S=11892736,E=11894783,C=206)
	CT(38835)=(S=11894784,E=11902975,C=181)
	CT(38836)=(S=11902976,E=11903999,C=203)
	CT(38837)=(S=11904000,E=11906047,C=83)
	CT(38838)=(S=11906048,E=11906559,C=203)
	CT(38839)=(S=11906560,E=11906623,C=181)
	CT(38840)=(S=11906624,E=11906687,C=215)
	CT(38841)=(S=11906688,E=11906719,C=83)
	CT(38842)=(S=11906720,E=11906723,C=206)
	CT(38843)=(S=11906724,E=11906727,C=209)
	CT(38844)=(S=11906728,E=11906731,C=202)
	CT(38845)=(S=11906732,E=11906743,C=83)
	CT(38846)=(S=11906744,E=11906751,C=200)
	CT(38847)=(S=11906752,E=11906767,C=83)
	CT(38848)=(S=11906768,E=11906815,C=202)
	CT(38849)=(S=11906816,E=11907071,C=213)
	CT(38850)=(S=11907072,E=11907327,C=212)
	CT(38851)=(S=11907328,E=11907391,C=83)
	CT(38852)=(S=11907392,E=11907455,C=170)
	CT(38853)=(S=11907456,E=11907519,C=201)
	CT(38854)=(S=11907520,E=11907543,C=83)
	CT(38855)=(S=11907544,E=11907547,C=214)
	CT(38856)=(S=11907548,E=11907551,C=203)
	CT(38857)=(S=11907552,E=11907583,C=212)
	CT(38858)=(S=11907584,E=11908095,C=209)
	CT(38859)=(S=11908096,E=11910143,C=200)
	CT(38860)=(S=11910144,E=11910271,C=176)
	CT(38861)=(S=11910272,E=11910335,C=201)
	CT(38862)=(S=11910336,E=11910399,C=213)
	CT(38863)=(S=11910400,E=11910527,C=202)
	CT(38864)=(S=11910528,E=11910559,C=215)
	CT(38865)=(S=11910560,E=11910575,C=83)
	CT(38866)=(S=11910576,E=11910591,C=210)
	CT(38867)=(S=11910592,E=11910623,C=83)
	CT(38868)=(S=11910624,E=11910655,C=208)
	CT(38869)=(S=11910656,E=11910911,C=203)
	CT(38870)=(S=11911168,E=11911263,C=83)
	CT(38871)=(S=11911296,E=11911423,C=181)
	CT(38872)=(S=11911424,E=11912191,C=202)
	CT(38873)=(S=11912192,E=11912447,C=213)
	CT(38874)=(S=11912448,E=11912639,C=209)
	CT(38875)=(S=11912640,E=11912703,C=83)
	CT(38876)=(S=11912704,E=11913087,C=213)
	CT(38877)=(S=11913088,E=11913103,C=212)
	CT(38878)=(S=11913104,E=11913135,C=83)
	CT(38879)=(S=11913136,E=11913151,C=208)
	CT(38880)=(S=11913152,E=11913183,C=181)
	CT(38881)=(S=11913184,E=11913215,C=209)
	CT(38882)=(S=11913216,E=11914239,C=203)
	CT(38883)=(S=11914240,E=11915263,C=181)
	CT(38884)=(S=11915264,E=11915519,C=200)
	CT(38885)=(S=11915520,E=11915647,C=83)
	CT(38886)=(S=11915648,E=11915775,C=215)
	CT(38887)=(S=11915776,E=11916031,C=208)
	CT(38888)=(S=11916032,E=11916287,C=213)
	CT(38889)=(S=11916288,E=11916543,C=203)
	CT(38890)=(S=11916544,E=11916799,C=189)
	CT(38891)=(S=11916800,E=11917311,C=203)
	CT(38892)=(S=11917312,E=11919359,C=189)
	CT(38893)=(S=11919360,E=11919455,C=202)
	CT(38894)=(S=11919456,E=11919487,C=83)
	CT(38895)=(S=11919488,E=11919519,C=209)
	CT(38896)=(S=11919520,E=11919523,C=181)
	CT(38897)=(S=11919524,E=11919527,C=83)
	CT(38898)=(S=11919528,E=11919531,C=177)
	CT(38899)=(S=11919532,E=11919535,C=213)
	CT(38900)=(S=11919536,E=11919551,C=83)
	CT(38901)=(S=11919552,E=11919559,C=202)
	CT(38902)=(S=11919560,E=11919563,C=212)
	CT(38903)=(S=11919564,E=11919567,C=177)
	CT(38904)=(S=11919568,E=11919575,C=83)
	CT(38905)=(S=11919576,E=11919583,C=202)
	CT(38906)=(S=11919584,E=11919615,C=212)
	CT(38907)=(S=11919616,E=11919647,C=83)
	CT(38908)=(S=11919648,E=11919679,C=200)
	CT(38909)=(S=11919680,E=11919727,C=181)
	CT(38910)=(S=11919728,E=11919743,C=203)
	CT(38911)=(S=11919744,E=11919751,C=210)
	CT(38912)=(S=11919752,E=11919755,C=83)
	CT(38913)=(S=11919756,E=11919767,C=203)
	CT(38914)=(S=11919776,E=11919807,C=212)
	CT(38915)=(S=11919808,E=11919839,C=83)
	CT(38916)=(S=11919840,E=11919871,C=219)
	CT(38917)=(S=11919872,E=11920127,C=203)
	CT(38918)=(S=11920128,E=11920383,C=201)
	CT(38919)=(S=11920384,E=11921407,C=83)
	CT(38920)=(S=11921408,E=11921535,C=181)
	CT(38921)=(S=11921536,E=11921811,C=202)
	CT(38922)=(S=11921812,E=11921815,C=189)
	CT(38923)=(S=11921816,E=11921855,C=202)
	CT(38924)=(S=11921856,E=11921919,C=212)
	CT(38925)=(S=11921920,E=11922943,C=181)
	CT(38926)=(S=11922944,E=11923455,C=83)
	CT(38927)=(S=11923456,E=11927551,C=181)
	CT(38928)=(S=11927552,E=11931647,C=74)
	CT(38929)=(S=11931648,E=11931775,C=8)
	CT(38930)=(S=11931776,E=11931787,C=10)
	CT(38931)=(S=11931788,E=11931791,C=7)
	CT(38932)=(S=11931800,E=11931803,C=73)
	CT(38933)=(S=11931804,E=11931807,C=84)
	CT(38934)=(S=11931808,E=11931839,C=74)
	CT(38935)=(S=11931840,E=11931871,C=1)
	CT(38936)=(S=11931872,E=11931879,C=2)
	CT(38937)=(S=11931880,E=11931883,C=7)
	CT(38938)=(S=11931884,E=11931887,C=2)
	CT(38939)=(S=11931888,E=11931903,C=74)
	CT(38940)=(S=11931904,E=11932159,C=3)
	CT(38941)=(S=11932160,E=11932287,C=1)
	CT(38942)=(S=11932288,E=11932351,C=4)
	CT(38943)=(S=11932352,E=11932415,C=10)
	CT(38944)=(S=11932416,E=11932543,C=4)
	CT(38945)=(S=11932544,E=11932671,C=7)
	CT(38946)=(S=11932672,E=11933439,C=2)
	CT(38947)=(S=11933440,E=11933567,C=74)
	CT(38948)=(S=11933568,E=11933583,C=4)
	CT(38949)=(S=11933584,E=11933599,C=7)
	CT(38950)=(S=11933600,E=11933615,C=74)
	CT(38951)=(S=11933616,E=11933623,C=73)
	CT(38952)=(S=11933624,E=11933631,C=1)
	CT(38953)=(S=11933632,E=11933639,C=0)
	CT(38954)=(S=11933640,E=11933647,C=1)
	CT(38955)=(S=11933648,E=11933651,C=2)
	CT(38956)=(S=11933652,E=11933655,C=0)
	CT(38957)=(S=11933656,E=11933695,C=2)
	CT(38958)=(S=11933696,E=11935487,C=74)
	CT(38959)=(S=11935488,E=11935743,C=6)
	CT(38960)=(S=11935744,E=11939839,C=1)
	CT(38961)=(S=11939840,E=11939903,C=2)
	CT(38962)=(S=11939904,E=11939935,C=84)
	CT(38963)=(S=11939936,E=11939967,C=1)
	CT(38964)=(S=11939968,E=11939999,C=73)
	CT(38965)=(S=11940000,E=11940031,C=74)
	CT(38966)=(S=11940032,E=11940095,C=4)
	CT(38967)=(S=11940096,E=11940367,C=1)
	CT(38968)=(S=11940368,E=11940415,C=6)
	CT(38969)=(S=11940416,E=11940419,C=75)
	CT(38970)=(S=11940424,E=11940427,C=162)
	CT(38971)=(S=11940428,E=11940431,C=7)
	CT(38972)=(S=11940432,E=11940447,C=3)
	CT(38973)=(S=11940448,E=11940463,C=2)
	CT(38974)=(S=11940464,E=11940479,C=1)
	CT(38975)=(S=11940480,E=11940511,C=7)
	CT(38976)=(S=11940512,E=11940515,C=73)
	CT(38977)=(S=11940516,E=11940519,C=2)
	CT(38978)=(S=11940520,E=11940523,C=162)
	CT(38979)=(S=11940524,E=11940527,C=2)
	CT(38980)=(S=11940528,E=11940543,C=89)
	CT(38981)=(S=11940544,E=11940591,C=2)
	CT(38982)=(S=11940592,E=11940607,C=74)
	CT(38983)=(S=11940608,E=11940863,C=1)
	CT(38984)=(S=11940864,E=11941375,C=3)
	CT(38985)=(S=11941376,E=11941503,C=1)
	CT(38986)=(S=11941536,E=11941551,C=73)
	CT(38987)=(S=11941552,E=11941567,C=2)
	CT(38988)=(S=11941568,E=11941599,C=5)
	CT(38989)=(S=11941600,E=11941603,C=0)
	CT(38990)=(S=11941604,E=11941607,C=7)
	CT(38991)=(S=11941608,E=11941615,C=0)
	CT(38992)=(S=11941616,E=11941631,C=2)
	CT(38993)=(S=11941632,E=11941887,C=7)
	CT(38994)=(S=11941888,E=11943167,C=4)
	CT(38995)=(S=11943168,E=11943423,C=1)
	CT(38996)=(S=11943424,E=11943935,C=5)
	CT(38997)=(S=11943936,E=11948031,C=4)
	CT(38998)=(S=11948032,E=11951359,C=1)
	CT(38999)=(S=11951360,E=11951423,C=87)
	CT(39000)=(S=11951424,E=11951455,C=75)
	CT(39001)=(S=11951456,E=11951487,C=2)
	CT(39002)=(S=11951488,E=11951615,C=3)
	CT(39003)=(S=11951616,E=11952127,C=4)
	CT(39004)=(S=11952128,E=11966463,C=1)
	CT(39005)=(S=11966464,E=11966975,C=8)
	CT(39006)=(S=11966976,E=11967231,C=73)
	CT(39007)=(S=11967232,E=11967487,C=9)
	CT(39008)=(S=11967488,E=11967743,C=4)
	CT(39009)=(S=11967744,E=11967999,C=1)
	CT(39010)=(S=11968000,E=11968511,C=2)
	CT(39011)=(S=11968512,E=11968575,C=92)
	CT(39012)=(S=11968576,E=11968607,C=1)
	CT(39013)=(S=11968608,E=11968639,C=84)
	CT(39014)=(S=11968640,E=11968703,C=0)
	CT(39015)=(S=11968704,E=11968767,C=2)
	CT(39016)=(S=11968768,E=11968799,C=156)
	CT(39017)=(S=11968832,E=11968839,C=2)
	CT(39018)=(S=11968840,E=11968843,C=7)
	CT(39019)=(S=11968844,E=11968847,C=2)
	CT(39020)=(S=11968848,E=11968863,C=11)
	CT(39021)=(S=11968864,E=11969279,C=6)
	CT(39022)=(S=11969280,E=11969375,C=2)
	CT(39023)=(S=11969376,E=11969407,C=84)
	CT(39024)=(S=11969408,E=11969535,C=6)
	CT(39025)=(S=11969536,E=11971583,C=2)
	CT(39026)=(S=11971584,E=11971839,C=6)
	CT(39027)=(S=11971840,E=11971903,C=9)
	CT(39028)=(S=11971904,E=11971907,C=4)
	CT(39029)=(S=11971908,E=11971911,C=9)
	CT(39030)=(S=11971912,E=11971915,C=5)
	CT(39031)=(S=11971916,E=11971919,C=8)
	CT(39032)=(S=11971920,E=11971967,C=6)
	CT(39033)=(S=11971968,E=11971999,C=2)
	CT(39034)=(S=11972000,E=11972031,C=6)
	CT(39035)=(S=11972032,E=11972095,C=159)
	CT(39036)=(S=11972096,E=11972607,C=1)
	CT(39037)=(S=11972608,E=11976703,C=77)
	CT(39038)=(S=11976704,E=11978751,C=6)
	CT(39039)=(S=11978752,E=11980799,C=1)
	CT(39040)=(S=11980800,E=11986943,C=6)
	CT(39041)=(S=11986944,E=11987199,C=3)
	CT(39042)=(S=11987200,E=11987967,C=9)
	CT(39043)=(S=11987968,E=11988079,C=2)
	CT(39044)=(S=11988080,E=11988083,C=11)
	CT(39045)=(S=11988084,E=11988087,C=0)
	CT(39046)=(S=11988088,E=11988095,C=73)
	CT(39047)=(S=11988096,E=11988223,C=1)
	CT(39048)=(S=11988224,E=11988227,C=8)
	CT(39049)=(S=11988228,E=11988231,C=0)
	CT(39050)=(S=11988232,E=11988243,C=4)
	CT(39051)=(S=11988244,E=11988247,C=11)
	CT(39052)=(S=11988256,E=11988351,C=6)
	CT(39053)=(S=11988352,E=11988415,C=4)
	CT(39054)=(S=11988416,E=11988479,C=6)
	CT(39055)=(S=11988480,E=11988767,C=1)
	CT(39056)=(S=11988768,E=11988775,C=10)
	CT(39057)=(S=11988776,E=11988779,C=5)
	CT(39058)=(S=11988780,E=11988783,C=9)
	CT(39059)=(S=11988784,E=11988863,C=8)
	CT(39060)=(S=11988864,E=11988991,C=0)
	CT(39061)=(S=11988992,E=11991039,C=1)
	CT(39062)=(S=11991040,E=11992063,C=2)
	CT(39063)=(S=11992064,E=11992127,C=6)
	CT(39064)=(S=11992128,E=11992191,C=84)
	CT(39065)=(S=11992192,E=11992319,C=6)
	CT(39066)=(S=11992320,E=11992575,C=74)
	CT(39067)=(S=11992576,E=11992831,C=1)
	CT(39068)=(S=11992832,E=11992839,C=74)
	CT(39069)=(S=11992840,E=11992847,C=3)
	CT(39070)=(S=11992848,E=11992851,C=74)
	CT(39071)=(S=11992852,E=11992855,C=5)
	CT(39072)=(S=11992856,E=11992859,C=8)
	CT(39073)=(S=11992860,E=11992863,C=0)
	CT(39074)=(S=11992896,E=11992911,C=2)
	CT(39075)=(S=11992912,E=11992959,C=0)
	CT(39076)=(S=11992960,E=11993087,C=6)
	CT(39077)=(S=11993088,E=12011519,C=1)
	CT(39078)=(S=12011520,E=12013055,C=2)
	CT(39079)=(S=12013056,E=12013183,C=5)
	CT(39080)=(S=12013184,E=12013215,C=6)
	CT(39081)=(S=12013224,E=12013227,C=5)
	CT(39082)=(S=12013228,E=12013231,C=0)
	CT(39083)=(S=12013236,E=12013239,C=1)
	CT(39084)=(S=12013240,E=12013243,C=4)
	CT(39085)=(S=12013244,E=12013247,C=2)
	CT(39086)=(S=12013248,E=12013311,C=6)
	CT(39087)=(S=12013312,E=12013567,C=2)
	CT(39088)=(S=12013568,E=12013951,C=11)
	CT(39089)=(S=12013952,E=12013967,C=91)
	CT(39090)=(S=12013968,E=12013975,C=0)
	CT(39091)=(S=12013976,E=12013983,C=74)
	CT(39092)=(S=12013984,E=12013991,C=5)
	CT(39093)=(S=12013992,E=12013995,C=92)
	CT(39094)=(S=12014000,E=12014003,C=2)
	CT(39095)=(S=12014004,E=12014007,C=1)
	CT(39096)=(S=12014008,E=12014015,C=86)
	CT(39097)=(S=12014016,E=12014079,C=7)
	CT(39098)=(S=12014080,E=12014591,C=4)
	CT(39099)=(S=12014592,E=12015103,C=1)
	CT(39100)=(S=12015104,E=12015295,C=2)
	CT(39101)=(S=12015296,E=12015359,C=6)
	CT(39102)=(S=12015360,E=12015615,C=4)
	CT(39103)=(S=12015616,E=12016127,C=3)
	CT(39104)=(S=12016128,E=12016255,C=7)
	CT(39105)=(S=12016256,E=12016287,C=6)
	CT(39106)=(S=12016288,E=12016295,C=11)
	CT(39107)=(S=12016296,E=12016303,C=3)
	CT(39108)=(S=12016304,E=12016311,C=2)
	CT(39109)=(S=12016312,E=12016319,C=8)
	CT(39110)=(S=12016320,E=12016383,C=2)
	CT(39111)=(S=12016384,E=12016415,C=11)
	CT(39112)=(S=12016416,E=12016447,C=8)
	CT(39113)=(S=12016448,E=12016479,C=74)
	CT(39114)=(S=12016480,E=12016511,C=2)
	CT(39115)=(S=12016512,E=12016515,C=1)
	CT(39116)=(S=12016516,E=12016519,C=75)
	CT(39117)=(S=12016520,E=12016543,C=1)
	CT(39118)=(S=12016544,E=12016575,C=11)
	CT(39119)=(S=12016576,E=12016639,C=6)
	CT(39120)=(S=12016640,E=12017663,C=1)
	CT(39121)=(S=12017664,E=12025855,C=6)
	CT(39122)=(S=12025856,E=12036863,C=1)
	CT(39123)=(S=12036864,E=12037119,C=5)
	CT(39124)=(S=12037120,E=12038143,C=1)
	CT(39125)=(S=12038144,E=12038455,C=2)
	CT(39126)=(S=12038456,E=12038463,C=0)
	CT(39127)=(S=12038464,E=12038495,C=8)
	CT(39128)=(S=12038496,E=12038511,C=92)
	CT(39129)=(S=12038512,E=12038519,C=10)
	CT(39130)=(S=12038520,E=12038523,C=0)
	CT(39131)=(S=12038524,E=12038527,C=4)
	CT(39132)=(S=12038528,E=12038655,C=2)
	CT(39133)=(S=12038656,E=12039167,C=8)
	CT(39134)=(S=12039168,E=12039679,C=2)
	CT(39135)=(S=12039680,E=12039711,C=1)
	CT(39136)=(S=12039712,E=12039743,C=2)
	CT(39137)=(S=12039744,E=12039759,C=10)
	CT(39138)=(S=12039760,E=12039763,C=8)
	CT(39139)=(S=12039764,E=12039771,C=4)
	CT(39140)=(S=12039772,E=12039775,C=74)
	CT(39141)=(S=12039776,E=12039807,C=154)
	CT(39142)=(S=12039808,E=12040191,C=2)
	CT(39143)=(S=12040192,E=12058623,C=1)
	CT(39144)=(S=12058624,E=12064255,C=29)
	CT(39145)=(S=12064256,E=12064511,C=3)
	CT(39146)=(S=12064512,E=12075007,C=29)
	CT(39147)=(S=12075008,E=12077055,C=78)
	CT(39148)=(S=12077056,E=12077951,C=29)
	CT(39149)=(S=12077952,E=12077967,C=78)
	CT(39150)=(S=12077968,E=12078031,C=29)
	CT(39151)=(S=12078032,E=12078047,C=78)
	CT(39152)=(S=12078048,E=12079615,C=29)
	CT(39153)=(S=12079616,E=12079871,C=3)
	CT(39154)=(S=12079872,E=12082687,C=29)
	CT(39155)=(S=12082688,E=12082815,C=78)
	CT(39156)=(S=12082816,E=12082863,C=29)
	CT(39157)=(S=12082864,E=12082879,C=78)
	CT(39158)=(S=12082880,E=12082943,C=29)
	CT(39159)=(S=12082944,E=12082959,C=80)
	CT(39160)=(S=12082960,E=12083135,C=29)
	CT(39161)=(S=12083136,E=12083199,C=78)
	CT(39162)=(S=12083200,E=12086015,C=29)
	CT(39163)=(S=12086016,E=12086271,C=78)
	CT(39164)=(S=12086272,E=12095487,C=29)
	CT(39165)=(S=12095488,E=12097535,C=78)
	CT(39166)=(S=12097536,E=12099583,C=29)
	CT(39167)=(S=12099584,E=12100607,C=78)
	CT(39168)=(S=12100608,E=12101935,C=29)
	CT(39169)=(S=12101936,E=12101951,C=78)
	CT(39170)=(S=12101952,E=12102143,C=29)
	CT(39171)=(S=12102144,E=12102207,C=175)
	CT(39172)=(S=12102208,E=12102271,C=29)
	CT(39173)=(S=12102272,E=12102287,C=78)
	CT(39174)=(S=12102288,E=12102591,C=29)
	CT(39175)=(S=12102592,E=12102623,C=78)
	CT(39176)=(S=12102624,E=12103423,C=29)
	CT(39177)=(S=12103424,E=12103487,C=78)
	CT(39178)=(S=12103488,E=12124159,C=29)
	CT(39179)=(S=12124416,E=12124416,C=18)
	CT(39180)=(S=12124417,E=12124417,C=43)
	CT(39181)=(S=12124418,E=12124419,C=12)
	CT(39182)=(S=12124420,E=12124423,C=58)
	CT(39183)=(S=12124424,E=12124424,C=16)
	CT(39184)=(S=12124425,E=12124427,C=58)
	CT(39185)=(S=12124428,E=12124428,C=27)
	CT(39186)=(S=12124429,E=12124429,C=18)
	CT(39187)=(S=12124430,E=12124430,C=12)
	CT(39188)=(S=12124431,E=12124431,C=16)
	CT(39189)=(S=12124432,E=12124432,C=31)
	CT(39190)=(S=12124433,E=12124433,C=58)
	CT(39191)=(S=12124434,E=12124434,C=28)
	CT(39192)=(S=12124435,E=12124435,C=18)
	CT(39193)=(S=12124436,E=12124436,C=25)
	CT(39194)=(S=12124437,E=12124437,C=42)
	CT(39195)=(S=12124438,E=12124438,C=58)
	CT(39196)=(S=12124439,E=12124439,C=18)
	CT(39197)=(S=12124440,E=12124440,C=50)
	CT(39198)=(S=12124441,E=12124441,C=33)
	CT(39199)=(S=12124442,E=12124442,C=12)
	CT(39200)=(S=12124443,E=12124443,C=46)
	CT(39201)=(S=12124444,E=12124444,C=12)
	CT(39202)=(S=12124445,E=12124445,C=58)
	CT(39203)=(S=12124446,E=12124446,C=51)
	CT(39204)=(S=12124447,E=12124447,C=42)
	CT(39205)=(S=12124448,E=12124448,C=36)
	CT(39206)=(S=12124449,E=12124449,C=14)
	CT(39207)=(S=12124450,E=12124450,C=71)
	CT(39208)=(S=12124451,E=12124451,C=28)
	CT(39209)=(S=12124452,E=12124452,C=38)
	CT(39210)=(S=12124453,E=12124453,C=58)
	CT(39211)=(S=12124454,E=12124454,C=12)
	CT(39212)=(S=12124456,E=12124456,C=51)
	CT(39213)=(S=12124457,E=12124457,C=18)
	CT(39214)=(S=12124458,E=12124458,C=58)
	CT(39215)=(S=12124459,E=12124459,C=42)
	CT(39216)=(S=12124460,E=12124460,C=102)
	CT(39217)=(S=12124461,E=12124461,C=53)
	CT(39218)=(S=12124462,E=12124462,C=15)
	CT(39219)=(S=12124463,E=12124463,C=12)
	CT(39220)=(S=12124464,E=12124464,C=28)
	CT(39221)=(S=12124465,E=12124465,C=18)
	CT(39222)=(S=12124672,E=12124675,C=34)
	CT(39223)=(S=12124676,E=12124679,C=15)
	CT(39224)=(S=12124680,E=12124683,C=28)
	CT(39225)=(S=12124684,E=12124687,C=26)
	CT(39226)=(S=12124688,E=12124691,C=28)
	CT(39227)=(S=12124692,E=12124695,C=34)
	CT(39228)=(S=12124696,E=12124699,C=36)
	CT(39229)=(S=12124700,E=12124703,C=19)
	CT(39230)=(S=12124704,E=12124707,C=18)
	CT(39231)=(S=12124708,E=12124711,C=58)
	CT(39232)=(S=12124712,E=12124715,C=33)
	CT(39233)=(S=12124716,E=12124723,C=36)
	CT(39234)=(S=12124724,E=12124727,C=49)
	CT(39235)=(S=12124728,E=12124731,C=28)
	CT(39236)=(S=12124732,E=12124735,C=58)
	CT(39237)=(S=12124736,E=12124739,C=63)
	CT(39238)=(S=12124740,E=12124743,C=25)
	CT(39239)=(S=12124744,E=12124747,C=18)
	CT(39240)=(S=12124748,E=12124751,C=58)
	CT(39241)=(S=12124752,E=12124755,C=36)
	CT(39242)=(S=12124756,E=12124759,C=21)
	CT(39243)=(S=12124760,E=12124763,C=18)
	CT(39244)=(S=12124764,E=12124767,C=49)
	CT(39245)=(S=12124768,E=12124771,C=32)
	CT(39246)=(S=12124772,E=12124775,C=28)
	CT(39247)=(S=12124776,E=12124779,C=18)
	CT(39248)=(S=12124780,E=12124783,C=31)
	CT(39249)=(S=12124784,E=12124787,C=58)
	CT(39250)=(S=12124788,E=12124795,C=28)
	CT(39251)=(S=12124796,E=12124799,C=18)
	CT(39252)=(S=12124800,E=12124803,C=25)
	CT(39253)=(S=12124804,E=12124807,C=28)
	CT(39254)=(S=12124808,E=12124811,C=14)
	CT(39255)=(S=12124812,E=12124815,C=28)
	CT(39256)=(S=12124816,E=12124819,C=15)
	CT(39257)=(S=12124820,E=12124823,C=36)
	CT(39258)=(S=12124824,E=12124827,C=19)
	CT(39259)=(S=12124828,E=12124831,C=42)
	CT(39260)=(S=12124832,E=12124835,C=34)
	CT(39261)=(S=12124836,E=12124843,C=14)
	CT(39262)=(S=12124844,E=12124847,C=36)
	CT(39263)=(S=12124848,E=12124851,C=42)
	CT(39264)=(S=12124852,E=12124855,C=28)
	CT(39265)=(S=12124856,E=12124859,C=18)
	CT(39266)=(S=12124860,E=12124863,C=54)
	CT(39267)=(S=12124864,E=12124871,C=14)
	CT(39268)=(S=12124872,E=12124875,C=34)
	CT(39269)=(S=12124876,E=12124879,C=14)
	CT(39270)=(S=12124880,E=12124883,C=51)
	CT(39271)=(S=12124884,E=12124887,C=31)
	CT(39272)=(S=12124888,E=12124895,C=14)
	CT(39273)=(S=12124896,E=12124899,C=20)
	CT(39274)=(S=12124900,E=12124903,C=19)
	CT(39275)=(S=12124904,E=12124907,C=23)
	CT(39276)=(S=12124908,E=12124911,C=14)
	CT(39277)=(S=12124912,E=12124915,C=36)
	CT(39278)=(S=12124916,E=12124919,C=94)
	CT(39279)=(S=12124920,E=12124923,C=28)
	CT(39280)=(S=12124924,E=12124927,C=12)
	CT(39281)=(S=12124928,E=12124935,C=27)
	CT(39282)=(S=12124936,E=12124939,C=33)
	CT(39283)=(S=12124940,E=12124943,C=24)
	CT(39284)=(S=12124944,E=12124947,C=18)
	CT(39285)=(S=12124948,E=12124951,C=39)
	CT(39286)=(S=12124952,E=12124955,C=12)
	CT(39287)=(S=12124956,E=12124959,C=13)
	CT(39288)=(S=12124960,E=12124963,C=18)
	CT(39289)=(S=12124964,E=12124967,C=15)
	CT(39290)=(S=12124968,E=12124971,C=28)
	CT(39291)=(S=12124972,E=12124975,C=70)
	CT(39292)=(S=12124976,E=12124979,C=19)
	CT(39293)=(S=12124980,E=12124983,C=99)
	CT(39294)=(S=12124984,E=12124987,C=37)
	CT(39295)=(S=12124988,E=12124991,C=58)
	CT(39296)=(S=12124992,E=12124995,C=12)
	CT(39297)=(S=12124996,E=12124999,C=18)
	CT(39298)=(S=12125000,E=12125007,C=14)
	CT(39299)=(S=12125008,E=12125011,C=28)
	CT(39300)=(S=12125012,E=12125023,C=14)
	CT(39301)=(S=12125024,E=12125027,C=28)
	CT(39302)=(S=12125028,E=12125031,C=14)
	CT(39303)=(S=12125032,E=12125035,C=36)
	CT(39304)=(S=12125036,E=12125039,C=15)
	CT(39305)=(S=12125040,E=12125043,C=58)
	CT(39306)=(S=12125044,E=12125047,C=25)
	CT(39307)=(S=12125048,E=12125051,C=104)
	CT(39308)=(S=12125052,E=12125055,C=26)
	CT(39309)=(S=12125056,E=12125059,C=12)
	CT(39310)=(S=12125060,E=12125063,C=55)
	CT(39311)=(S=12125064,E=12125067,C=50)
	CT(39312)=(S=12125068,E=12125071,C=18)
	CT(39313)=(S=12125072,E=12125075,C=17)
	CT(39314)=(S=12125076,E=12125079,C=31)
	CT(39315)=(S=12125080,E=12125083,C=16)
	CT(39316)=(S=12125084,E=12125087,C=33)
	CT(39317)=(S=12125088,E=12125091,C=49)
	CT(39318)=(S=12125092,E=12125095,C=14)
	CT(39319)=(S=12125096,E=12125099,C=12)
	CT(39320)=(S=12125100,E=12125103,C=18)
	CT(39321)=(S=12125104,E=12125107,C=36)
	CT(39322)=(S=12125108,E=12125111,C=18)
	CT(39323)=(S=12125112,E=12125115,C=28)
	CT(39324)=(S=12125116,E=12125119,C=66)
	CT(39325)=(S=12125120,E=12125123,C=57)
	CT(39326)=(S=12125124,E=12125127,C=12)
	CT(39327)=(S=12125128,E=12125131,C=26)
	CT(39328)=(S=12125132,E=12125139,C=36)
	CT(39329)=(S=12125140,E=12125143,C=26)
	CT(39330)=(S=12125144,E=12125147,C=49)
	CT(39331)=(S=12125148,E=12125151,C=22)
	CT(39332)=(S=12125152,E=12125155,C=58)
	CT(39333)=(S=12125156,E=12125159,C=44)
	CT(39334)=(S=12125160,E=12125163,C=28)
	CT(39335)=(S=12125164,E=12125167,C=25)
	CT(39336)=(S=12125168,E=12125171,C=18)
	CT(39337)=(S=12125172,E=12125175,C=50)
	CT(39338)=(S=12125176,E=12125179,C=28)
	CT(39339)=(S=12125180,E=12125183,C=15)
	CT(39340)=(S=12125184,E=12125187,C=26)
	CT(39341)=(S=12125188,E=12125191,C=14)
	CT(39342)=(S=12125192,E=12125195,C=19)
	CT(39343)=(S=12125196,E=12125199,C=14)
	CT(39344)=(S=12125200,E=12125203,C=26)
	CT(39345)=(S=12125204,E=12125207,C=12)
	CT(39346)=(S=12125208,E=12125211,C=14)
	CT(39347)=(S=12125212,E=12125215,C=26)
	CT(39348)=(S=12125216,E=12125219,C=25)
	CT(39349)=(S=12125220,E=12125223,C=27)
	CT(39350)=(S=12125224,E=12125227,C=31)
	CT(39351)=(S=12125228,E=12125231,C=12)
	CT(39352)=(S=12125232,E=12125235,C=14)
	CT(39353)=(S=12125236,E=12125239,C=188)
	CT(39354)=(S=12125240,E=12125243,C=18)
	CT(39355)=(S=12125244,E=12125247,C=12)
	CT(39356)=(S=12125248,E=12125251,C=14)
	CT(39357)=(S=12125252,E=12125255,C=37)
	CT(39358)=(S=12125256,E=12125259,C=55)
	CT(39359)=(S=12125260,E=12125263,C=12)
	CT(39360)=(S=12125264,E=12125267,C=51)
	CT(39361)=(S=12125268,E=12125271,C=30)
	CT(39362)=(S=12125272,E=12125275,C=22)
	CT(39363)=(S=12125276,E=12125279,C=25)
	CT(39364)=(S=12125280,E=12125287,C=14)
	CT(39365)=(S=12125288,E=12125291,C=26)
	CT(39366)=(S=12125292,E=12125295,C=18)
	CT(39367)=(S=12125296,E=12125299,C=36)
	CT(39368)=(S=12125300,E=12125303,C=14)
	CT(39369)=(S=12125304,E=12125307,C=42)
	CT(39370)=(S=12125308,E=12125311,C=70)
	CT(39371)=(S=12125312,E=12125315,C=14)
	CT(39372)=(S=12125316,E=12125319,C=22)
	CT(39373)=(S=12125320,E=12125323,C=24)
	CT(39374)=(S=12125324,E=12125327,C=15)
	CT(39375)=(S=12125328,E=12125331,C=18)
	CT(39376)=(S=12125332,E=12125335,C=36)
	CT(39377)=(S=12125336,E=12125339,C=15)
	CT(39378)=(S=12125340,E=12125343,C=14)
	CT(39379)=(S=12125344,E=12125347,C=98)
	CT(39380)=(S=12125348,E=12125351,C=14)
	CT(39381)=(S=12125352,E=12125355,C=34)
	CT(39382)=(S=12125356,E=12125359,C=27)
	CT(39383)=(S=12125360,E=12125367,C=18)
	CT(39384)=(S=12125368,E=12125375,C=25)
	CT(39385)=(S=12125376,E=12125379,C=63)
	CT(39386)=(S=12125380,E=12125383,C=14)
	CT(39387)=(S=12125384,E=12125387,C=33)
	CT(39388)=(S=12125388,E=12125391,C=18)
	CT(39389)=(S=12125392,E=12125395,C=37)
	CT(39390)=(S=12125396,E=12125399,C=58)
	CT(39391)=(S=12125400,E=12125403,C=24)
	CT(39392)=(S=12125404,E=12125407,C=26)
	CT(39393)=(S=12125408,E=12125411,C=37)
	CT(39394)=(S=12125412,E=12125415,C=47)
	CT(39395)=(S=12125416,E=12125419,C=28)
	CT(39396)=(S=12125420,E=12125423,C=31)
	CT(39397)=(S=12125424,E=12125427,C=14)
	CT(39398)=(S=12125428,E=12125431,C=25)
	CT(39399)=(S=12125432,E=12125435,C=12)
	CT(39400)=(S=12125436,E=12125439,C=39)
	CT(39401)=(S=12125440,E=12125443,C=14)
	CT(39402)=(S=12125444,E=12125447,C=54)
	CT(39403)=(S=12125448,E=12125455,C=28)
	CT(39404)=(S=12125456,E=12125459,C=18)
	CT(39405)=(S=12125460,E=12125463,C=19)
	CT(39406)=(S=12125464,E=12125471,C=28)
	CT(39407)=(S=12125472,E=12125475,C=34)
	CT(39408)=(S=12125476,E=12125479,C=14)
	CT(39409)=(S=12125480,E=12125483,C=18)
	CT(39410)=(S=12125484,E=12125487,C=19)
	CT(39411)=(S=12125488,E=12125491,C=72)
	CT(39412)=(S=12125492,E=12125495,C=44)
	CT(39413)=(S=12125496,E=12125499,C=34)
	CT(39414)=(S=12125500,E=12125503,C=15)
	CT(39415)=(S=12125504,E=12125507,C=14)
	CT(39416)=(S=12125508,E=12125511,C=33)
	CT(39417)=(S=12125512,E=12125515,C=13)
	CT(39418)=(S=12125516,E=12125519,C=22)
	CT(39419)=(S=12125520,E=12125523,C=28)
	CT(39420)=(S=12125524,E=12125527,C=25)
	CT(39421)=(S=12125528,E=12125531,C=14)
	CT(39422)=(S=12125532,E=12125535,C=15)
	CT(39423)=(S=12125536,E=12125539,C=58)
	CT(39424)=(S=12125540,E=12125543,C=28)
	CT(39425)=(S=12125544,E=12125547,C=31)
	CT(39426)=(S=12125548,E=12125551,C=12)
	CT(39427)=(S=12125552,E=12125555,C=15)
	CT(39428)=(S=12125556,E=12125559,C=51)
	CT(39429)=(S=12125560,E=12125563,C=36)
	CT(39430)=(S=12125564,E=12125567,C=25)
	CT(39431)=(S=12125568,E=12125571,C=18)
	CT(39432)=(S=12125572,E=12125575,C=36)
	CT(39433)=(S=12125576,E=12125583,C=18)
	CT(39434)=(S=12125584,E=12125587,C=14)
	CT(39435)=(S=12125588,E=12125591,C=18)
	CT(39436)=(S=12125592,E=12125595,C=23)
	CT(39437)=(S=12125596,E=12125599,C=26)
	CT(39438)=(S=12125600,E=12125603,C=18)
	CT(39439)=(S=12125604,E=12125607,C=24)
	CT(39440)=(S=12125608,E=12125611,C=28)
	CT(39441)=(S=12125612,E=12125615,C=38)
	CT(39442)=(S=12125616,E=12125619,C=37)
	CT(39443)=(S=12125620,E=12125623,C=15)
	CT(39444)=(S=12125624,E=12125627,C=28)
	CT(39445)=(S=12125628,E=12125631,C=14)
	CT(39446)=(S=12125632,E=12125635,C=18)
	CT(39447)=(S=12125636,E=12125643,C=15)
	CT(39448)=(S=12125644,E=12125647,C=18)
	CT(39449)=(S=12125648,E=12125651,C=15)
	CT(39450)=(S=12125652,E=12125655,C=12)
	CT(39451)=(S=12125656,E=12125659,C=58)
	CT(39452)=(S=12125660,E=12125663,C=43)
	CT(39453)=(S=12125664,E=12125667,C=33)
	CT(39454)=(S=12125668,E=12125671,C=50)
	CT(39455)=(S=12125672,E=12125675,C=27)
	CT(39456)=(S=12125676,E=12125679,C=25)
	CT(39457)=(S=12125680,E=12125683,C=63)
	CT(39458)=(S=12125684,E=12125687,C=15)
	CT(39459)=(S=12125688,E=12125691,C=18)
	CT(39460)=(S=12125692,E=12125695,C=31)
	CT(39461)=(S=12125696,E=12125699,C=58)
	CT(39462)=(S=12125700,E=12125703,C=25)
	CT(39463)=(S=12125704,E=12125707,C=19)
	CT(39464)=(S=12125708,E=12125711,C=18)
	CT(39465)=(S=12125712,E=12125715,C=43)
	CT(39466)=(S=12125716,E=12125719,C=54)
	CT(39467)=(S=12125720,E=12125723,C=68)
	CT(39468)=(S=12125724,E=12125727,C=58)
	CT(39469)=(S=12125728,E=12125731,C=25)
	CT(39470)=(S=12125732,E=12125735,C=63)
	CT(39471)=(S=12125736,E=12125739,C=98)
	CT(39472)=(S=12125740,E=12125743,C=18)
	CT(39473)=(S=12125744,E=12125747,C=36)
	CT(39474)=(S=12125748,E=12125751,C=58)
	CT(39475)=(S=12125752,E=12125755,C=43)
	CT(39476)=(S=12125756,E=12125759,C=12)
	CT(39477)=(S=12125760,E=12125763,C=17)
	CT(39478)=(S=12125764,E=12125767,C=28)
	CT(39479)=(S=12125768,E=12125771,C=15)
	CT(39480)=(S=12125772,E=12125775,C=22)
	CT(39481)=(S=12125776,E=12125779,C=18)
	CT(39482)=(S=12125780,E=12125783,C=36)
	CT(39483)=(S=12125784,E=12125787,C=15)
	CT(39484)=(S=12125788,E=12125791,C=12)
	CT(39485)=(S=12125792,E=12125795,C=28)
	CT(39486)=(S=12125796,E=12125799,C=50)
	CT(39487)=(S=12125800,E=12125803,C=104)
	CT(39488)=(S=12125804,E=12125807,C=36)
	CT(39489)=(S=12125808,E=12125811,C=58)
	CT(39490)=(S=12125812,E=12125815,C=18)
	CT(39491)=(S=12125816,E=12125819,C=28)
	CT(39492)=(S=12125820,E=12125823,C=18)
	CT(39493)=(S=12125824,E=12125827,C=28)
	CT(39494)=(S=12125828,E=12125831,C=14)
	CT(39495)=(S=12125832,E=12125835,C=50)
	CT(39496)=(S=12125836,E=12125839,C=12)
	CT(39497)=(S=12125840,E=12125843,C=28)
	CT(39498)=(S=12125844,E=12125847,C=39)
	CT(39499)=(S=12125848,E=12125851,C=14)
	CT(39500)=(S=12125852,E=12125855,C=50)
	CT(39501)=(S=12125856,E=12125859,C=36)
	CT(39502)=(S=12125860,E=12125863,C=18)
	CT(39503)=(S=12125864,E=12125867,C=34)
	CT(39504)=(S=12125868,E=12125871,C=18)
	CT(39505)=(S=12125872,E=12125875,C=55)
	CT(39506)=(S=12125876,E=12125879,C=47)
	CT(39507)=(S=12125880,E=12125883,C=31)
	CT(39508)=(S=12125884,E=12125887,C=46)
	CT(39509)=(S=12125888,E=12125891,C=15)
	CT(39510)=(S=12125892,E=12125895,C=14)
	CT(39511)=(S=12125896,E=12125899,C=25)
	CT(39512)=(S=12125900,E=12125903,C=36)
	CT(39513)=(S=12125904,E=12125911,C=12)
	CT(39514)=(S=12125912,E=12125915,C=18)
	CT(39515)=(S=12125916,E=12125919,C=45)
	CT(39516)=(S=12125920,E=12125923,C=14)
	CT(39517)=(S=12125924,E=12125927,C=18)
	CT(39518)=(S=12125928,E=12125931,C=70)
	CT(39519)=(S=12125932,E=12125935,C=18)
	CT(39520)=(S=12125936,E=12125939,C=15)
	CT(39521)=(S=12125940,E=12125947,C=18)
	CT(39522)=(S=12125948,E=12125951,C=28)
	CT(39523)=(S=12125952,E=12125955,C=37)
	CT(39524)=(S=12125956,E=12125959,C=12)
	CT(39525)=(S=12125960,E=12125963,C=104)
	CT(39526)=(S=12125964,E=12125967,C=12)
	CT(39527)=(S=12125968,E=12125971,C=42)
	CT(39528)=(S=12125972,E=12125979,C=33)
	CT(39529)=(S=12125980,E=12125983,C=18)
	CT(39530)=(S=12125984,E=12125987,C=14)
	CT(39531)=(S=12125988,E=12125991,C=12)
	CT(39532)=(S=12125992,E=12125995,C=18)
	CT(39533)=(S=12125996,E=12125999,C=33)
	CT(39534)=(S=12126000,E=12126003,C=18)
	CT(39535)=(S=12126004,E=12126007,C=57)
	CT(39536)=(S=12126008,E=12126011,C=28)
	CT(39537)=(S=12126012,E=12126015,C=27)
	CT(39538)=(S=12126016,E=12126027,C=12)
	CT(39539)=(S=12126028,E=12126031,C=36)
	CT(39540)=(S=12126032,E=12126035,C=37)
	CT(39541)=(S=12126036,E=12126039,C=18)
	CT(39542)=(S=12126040,E=12126043,C=14)
	CT(39543)=(S=12126044,E=12126047,C=18)
	CT(39544)=(S=12126048,E=12126055,C=14)
	CT(39545)=(S=12126056,E=12126059,C=58)
	CT(39546)=(S=12126060,E=12126063,C=12)
	CT(39547)=(S=12126064,E=12126067,C=15)
	CT(39548)=(S=12126068,E=12126071,C=18)
	CT(39549)=(S=12126072,E=12126075,C=43)
	CT(39550)=(S=12126076,E=12126079,C=42)
	CT(39551)=(S=12126080,E=12126083,C=39)
	CT(39552)=(S=12126084,E=12126095,C=27)
	CT(39553)=(S=12126096,E=12126099,C=18)
	CT(39554)=(S=12126100,E=12126103,C=14)
	CT(39555)=(S=12126104,E=12126107,C=18)
	CT(39556)=(S=12126108,E=12126111,C=32)
	CT(39557)=(S=12126112,E=12126115,C=12)
	CT(39558)=(S=12126116,E=12126119,C=14)
	CT(39559)=(S=12126120,E=12126123,C=58)
	CT(39560)=(S=12126124,E=12126127,C=18)
	CT(39561)=(S=12126128,E=12126131,C=37)
	CT(39562)=(S=12126132,E=12126135,C=18)
	CT(39563)=(S=12126136,E=12126139,C=63)
	CT(39564)=(S=12126140,E=12126143,C=14)
	CT(39565)=(S=12126144,E=12126147,C=27)
	CT(39566)=(S=12126148,E=12126151,C=28)
	CT(39567)=(S=12126152,E=12126155,C=18)
	CT(39568)=(S=12126156,E=12126159,C=51)
	CT(39569)=(S=12126160,E=12126163,C=28)
	CT(39570)=(S=12126164,E=12126167,C=12)
	CT(39571)=(S=12126168,E=12126171,C=51)
	CT(39572)=(S=12126172,E=12126175,C=36)
	CT(39573)=(S=12126176,E=12126183,C=14)
	CT(39574)=(S=12126184,E=12126187,C=18)
	CT(39575)=(S=12126188,E=12126191,C=60)
	CT(39576)=(S=12126192,E=12126195,C=12)
	CT(39577)=(S=12126196,E=12126199,C=14)
	CT(39578)=(S=12126200,E=12126203,C=36)
	CT(39579)=(S=12126204,E=12126207,C=55)
	CT(39580)=(S=12126208,E=12126211,C=66)
	CT(39581)=(S=12126212,E=12126215,C=18)
	CT(39582)=(S=12126216,E=12126219,C=28)
	CT(39583)=(S=12126220,E=12126223,C=37)
	CT(39584)=(S=12126224,E=12126227,C=27)
	CT(39585)=(S=12126228,E=12126231,C=18)
	CT(39586)=(S=12126232,E=12126235,C=15)
	CT(39587)=(S=12126236,E=12126239,C=12)
	CT(39588)=(S=12126240,E=12126243,C=37)
	CT(39589)=(S=12126244,E=12126247,C=15)
	CT(39590)=(S=12126248,E=12126251,C=31)
	CT(39591)=(S=12126252,E=12126255,C=49)
	CT(39592)=(S=12126256,E=12126259,C=12)
	CT(39593)=(S=12126260,E=12126263,C=49)
	CT(39594)=(S=12126264,E=12126267,C=33)
	CT(39595)=(S=12126268,E=12126271,C=60)
	CT(39596)=(S=12126272,E=12126275,C=12)
	CT(39597)=(S=12126276,E=12126279,C=53)
	CT(39598)=(S=12126280,E=12126283,C=50)
	CT(39599)=(S=12126284,E=12126287,C=12)
	CT(39600)=(S=12126288,E=12126291,C=19)
	CT(39601)=(S=12126292,E=12126295,C=28)
	CT(39602)=(S=12126296,E=12126303,C=14)
	CT(39603)=(S=12126308,E=12126311,C=19)
	CT(39604)=(S=12126312,E=12126315,C=44)
	CT(39605)=(S=12126316,E=12126319,C=14)
	CT(39606)=(S=12126320,E=12126323,C=63)
	CT(39607)=(S=12126324,E=12126327,C=28)
	CT(39608)=(S=12126328,E=12126331,C=15)
	CT(39609)=(S=12126332,E=12126335,C=18)
	CT(39610)=(S=12126336,E=12126339,C=37)
	CT(39611)=(S=12126340,E=12126343,C=14)
	CT(39612)=(S=12126344,E=12126347,C=28)
	CT(39613)=(S=12126348,E=12126351,C=25)
	CT(39614)=(S=12126352,E=12126355,C=12)
	CT(39615)=(S=12126356,E=12126359,C=15)
	CT(39616)=(S=12126360,E=12126363,C=38)
	CT(39617)=(S=12126364,E=12126367,C=18)
	CT(39618)=(S=12126368,E=12126375,C=33)
	CT(39619)=(S=12126376,E=12126379,C=28)
	CT(39620)=(S=12126380,E=12126383,C=26)
	CT(39621)=(S=12126384,E=12126387,C=36)
	CT(39622)=(S=12126388,E=12126391,C=18)
	CT(39623)=(S=12126392,E=12126395,C=42)
	CT(39624)=(S=12126396,E=12126399,C=33)
	CT(39625)=(S=12126400,E=12126403,C=63)
	CT(39626)=(S=12126404,E=12126407,C=15)
	CT(39627)=(S=12126408,E=12126411,C=18)
	CT(39628)=(S=12126412,E=12126415,C=14)
	CT(39629)=(S=12126416,E=12126419,C=34)
	CT(39630)=(S=12126420,E=12126423,C=98)
	CT(39631)=(S=12126424,E=12126431,C=18)
	CT(39632)=(S=12126432,E=12126439,C=28)
	CT(39633)=(S=12126440,E=12126443,C=20)
	CT(39634)=(S=12126444,E=12126447,C=33)
	CT(39635)=(S=12126448,E=12126455,C=25)
	CT(39636)=(S=12126456,E=12126459,C=27)
	CT(39637)=(S=12126460,E=12126463,C=12)
	CT(39638)=(S=12126464,E=12126467,C=19)
	CT(39639)=(S=12126468,E=12126471,C=24)
	CT(39640)=(S=12126472,E=12126475,C=47)
	CT(39641)=(S=12126476,E=12126479,C=28)
	CT(39642)=(S=12126480,E=12126483,C=14)
	CT(39643)=(S=12126484,E=12126487,C=12)
	CT(39644)=(S=12126488,E=12126491,C=24)
	CT(39645)=(S=12126492,E=12126495,C=28)
	CT(39646)=(S=12126496,E=12126499,C=14)
	CT(39647)=(S=12126500,E=12126503,C=37)
	CT(39648)=(S=12126504,E=12126507,C=31)
	CT(39649)=(S=12126508,E=12126511,C=96)
	CT(39650)=(S=12126512,E=12126515,C=14)
	CT(39651)=(S=12126516,E=12126519,C=15)
	CT(39652)=(S=12126520,E=12126523,C=58)
	CT(39653)=(S=12126524,E=12126527,C=19)
	CT(39654)=(S=12126528,E=12126531,C=28)
	CT(39655)=(S=12126532,E=12126535,C=35)
	CT(39656)=(S=12126536,E=12126551,C=18)
	CT(39657)=(S=12126552,E=12126555,C=27)
	CT(39658)=(S=12126556,E=12126559,C=14)
	CT(39659)=(S=12126560,E=12126563,C=12)
	CT(39660)=(S=12126564,E=12126567,C=72)
	CT(39661)=(S=12126568,E=12126575,C=28)
	CT(39662)=(S=12126576,E=12126579,C=71)
	CT(39663)=(S=12126580,E=12126583,C=33)
	CT(39664)=(S=12126584,E=12126587,C=20)
	CT(39665)=(S=12126588,E=12126591,C=58)
	CT(39666)=(S=12126592,E=12126595,C=28)
	CT(39667)=(S=12126596,E=12126599,C=42)
	CT(39668)=(S=12126600,E=12126603,C=39)
	CT(39669)=(S=12126604,E=12126607,C=24)
	CT(39670)=(S=12126608,E=12126611,C=18)
	CT(39671)=(S=12126612,E=12126615,C=15)
	CT(39672)=(S=12126616,E=12126619,C=12)
	CT(39673)=(S=12126620,E=12126623,C=37)
	CT(39674)=(S=12126624,E=12126627,C=31)
	CT(39675)=(S=12126628,E=12126631,C=19)
	CT(39676)=(S=12126632,E=12126639,C=14)
	CT(39677)=(S=12126640,E=12126647,C=28)
	CT(39678)=(S=12126648,E=12126651,C=18)
	CT(39679)=(S=12126652,E=12126655,C=70)
	CT(39680)=(S=12126656,E=12126659,C=25)
	CT(39681)=(S=12126660,E=12126663,C=101)
	CT(39682)=(S=12126664,E=12126671,C=18)
	CT(39683)=(S=12126672,E=12126675,C=15)
	CT(39684)=(S=12126676,E=12126679,C=69)
	CT(39685)=(S=12126680,E=12126683,C=28)
	CT(39686)=(S=12126684,E=12126687,C=37)
	CT(39687)=(S=12126688,E=12126691,C=28)
	CT(39688)=(S=12126692,E=12126695,C=18)
	CT(39689)=(S=12126696,E=12126699,C=14)
	CT(39690)=(S=12126700,E=12126711,C=38)
	CT(39691)=(S=12126712,E=12126715,C=12)
	CT(39692)=(S=12126716,E=12126719,C=36)
	CT(39693)=(S=12126720,E=12126723,C=18)
	CT(39694)=(S=12126724,E=12126727,C=17)
	CT(39695)=(S=12126728,E=12126731,C=24)
	CT(39696)=(S=12126732,E=12126735,C=14)
	CT(39697)=(S=12126736,E=12126739,C=12)
	CT(39698)=(S=12126740,E=12126743,C=53)
	CT(39699)=(S=12126744,E=12126747,C=58)
	CT(39700)=(S=12126748,E=12126751,C=27)
	CT(39701)=(S=12126752,E=12126755,C=15)
	CT(39702)=(S=12126756,E=12126759,C=28)
	CT(39703)=(S=12126760,E=12126763,C=57)
	CT(39704)=(S=12126764,E=12126767,C=18)
	CT(39705)=(S=12126768,E=12126771,C=36)
	CT(39706)=(S=12126772,E=12126775,C=34)
	CT(39707)=(S=12126776,E=12126779,C=36)
	CT(39708)=(S=12126780,E=12126783,C=18)
	CT(39709)=(S=12126784,E=12126787,C=17)
	CT(39710)=(S=12126788,E=12126791,C=28)
	CT(39711)=(S=12126792,E=12126795,C=26)
	CT(39712)=(S=12126796,E=12126799,C=15)
	CT(39713)=(S=12126800,E=12126803,C=25)
	CT(39714)=(S=12126804,E=12126807,C=33)
	CT(39715)=(S=12126808,E=12126811,C=14)
	CT(39716)=(S=12126812,E=12126815,C=28)
	CT(39717)=(S=12126816,E=12126819,C=36)
	CT(39718)=(S=12126820,E=12126823,C=14)
	CT(39719)=(S=12126824,E=12126827,C=13)
	CT(39720)=(S=12126828,E=12126831,C=28)
	CT(39721)=(S=12126832,E=12126835,C=23)
	CT(39722)=(S=12126836,E=12126839,C=14)
	CT(39723)=(S=12126840,E=12126843,C=58)
	CT(39724)=(S=12126844,E=12126847,C=50)
	CT(39725)=(S=12126848,E=12126851,C=18)
	CT(39726)=(S=12126852,E=12126855,C=19)
	CT(39727)=(S=12126856,E=12126859,C=18)
	CT(39728)=(S=12126860,E=12126863,C=46)
	CT(39729)=(S=12126864,E=12126867,C=14)
	CT(39730)=(S=12126868,E=12126871,C=28)
	CT(39731)=(S=12126872,E=12126875,C=18)
	CT(39732)=(S=12126876,E=12126883,C=36)
	CT(39733)=(S=12126884,E=12126891,C=25)
	CT(39734)=(S=12126892,E=12126895,C=18)
	CT(39735)=(S=12126896,E=12126899,C=12)
	CT(39736)=(S=12126900,E=12126907,C=18)
	CT(39737)=(S=12126908,E=12126911,C=15)
	CT(39738)=(S=12126912,E=12126915,C=28)
	CT(39739)=(S=12126916,E=12126919,C=47)
	CT(39740)=(S=12126920,E=12126923,C=14)
	CT(39741)=(S=12126924,E=12126927,C=46)
	CT(39742)=(S=12126928,E=12126931,C=31)
	CT(39743)=(S=12126932,E=12126935,C=24)
	CT(39744)=(S=12126936,E=12126939,C=61)
	CT(39745)=(S=12126940,E=12126943,C=24)
	CT(39746)=(S=12126944,E=12126947,C=34)
	CT(39747)=(S=12126948,E=12126955,C=36)
	CT(39748)=(S=12126956,E=12126959,C=33)
	CT(39749)=(S=12126960,E=12126963,C=65)
	CT(39750)=(S=12126964,E=12126967,C=28)
	CT(39751)=(S=12126968,E=12126971,C=25)
	CT(39752)=(S=12126972,E=12126975,C=12)
	CT(39753)=(S=12126976,E=12126979,C=27)
	CT(39754)=(S=12126980,E=12126983,C=18)
	CT(39755)=(S=12126984,E=12126987,C=67)
	CT(39756)=(S=12126988,E=12126991,C=37)
	CT(39757)=(S=12126992,E=12126995,C=28)
	CT(39758)=(S=12126996,E=12126999,C=15)
	CT(39759)=(S=12127000,E=12127003,C=44)
	CT(39760)=(S=12127004,E=12127007,C=31)
	CT(39761)=(S=12127008,E=12127011,C=18)
	CT(39762)=(S=12127012,E=12127015,C=14)
	CT(39763)=(S=12127016,E=12127019,C=32)
	CT(39764)=(S=12127020,E=12127023,C=17)
	CT(39765)=(S=12127024,E=12127027,C=18)
	CT(39766)=(S=12127028,E=12127031,C=15)
	CT(39767)=(S=12127032,E=12127035,C=34)
	CT(39768)=(S=12127036,E=12127039,C=18)
	CT(39769)=(S=12127040,E=12127043,C=63)
	CT(39770)=(S=12127044,E=12127047,C=26)
	CT(39771)=(S=12127048,E=12127051,C=18)
	CT(39772)=(S=12127052,E=12127055,C=68)
	CT(39773)=(S=12127056,E=12127059,C=31)
	CT(39774)=(S=12127060,E=12127063,C=42)
	CT(39775)=(S=12127064,E=12127067,C=26)
	CT(39776)=(S=12127068,E=12127071,C=14)
	CT(39777)=(S=12127072,E=12127075,C=19)
	CT(39778)=(S=12127076,E=12127079,C=58)
	CT(39779)=(S=12127080,E=12127083,C=14)
	CT(39780)=(S=12127084,E=12127087,C=58)
	CT(39781)=(S=12127088,E=12127091,C=19)
	CT(39782)=(S=12127092,E=12127095,C=25)
	CT(39783)=(S=12127100,E=12127103,C=17)
	CT(39784)=(S=12127104,E=12127107,C=58)
	CT(39785)=(S=12127108,E=12127111,C=18)
	CT(39786)=(S=12127112,E=12127115,C=28)
	CT(39787)=(S=12127116,E=12127119,C=14)
	CT(39788)=(S=12127120,E=12127123,C=36)
	CT(39789)=(S=12127124,E=12127127,C=18)
	CT(39790)=(S=12127128,E=12127131,C=15)
	CT(39791)=(S=12127132,E=12127135,C=28)
	CT(39792)=(S=12127136,E=12127139,C=12)
	CT(39793)=(S=12127140,E=12127143,C=21)
	CT(39794)=(S=12127144,E=12127147,C=34)
	CT(39795)=(S=12127148,E=12127151,C=50)
	CT(39796)=(S=12127152,E=12127155,C=26)
	CT(39797)=(S=12127156,E=12127159,C=27)
	CT(39798)=(S=12127160,E=12127163,C=14)
	CT(39799)=(S=12127164,E=12127167,C=12)
	CT(39800)=(S=12127168,E=12127171,C=15)
	CT(39801)=(S=12127172,E=12127175,C=18)
	CT(39802)=(S=12127176,E=12127179,C=25)
	CT(39803)=(S=12127180,E=12127183,C=14)
	CT(39804)=(S=12127184,E=12127187,C=47)
	CT(39805)=(S=12127188,E=12127191,C=37)
	CT(39806)=(S=12127192,E=12127195,C=12)
	CT(39807)=(S=12127196,E=12127199,C=25)
	CT(39808)=(S=12127200,E=12127203,C=15)
	CT(39809)=(S=12127204,E=12127207,C=18)
	CT(39810)=(S=12127208,E=12127211,C=15)
	CT(39811)=(S=12127212,E=12127215,C=25)
	CT(39812)=(S=12127216,E=12127219,C=14)
	CT(39813)=(S=12127220,E=12127223,C=18)
	CT(39814)=(S=12127224,E=12127227,C=37)
	CT(39815)=(S=12127228,E=12127231,C=28)
	CT(39816)=(S=12127232,E=12127235,C=12)
	CT(39817)=(S=12127236,E=12127239,C=34)
	CT(39818)=(S=12127240,E=12127243,C=31)
	CT(39819)=(S=12127244,E=12127247,C=36)
	CT(39820)=(S=12127248,E=12127251,C=41)
	CT(39821)=(S=12127252,E=12127255,C=58)
	CT(39822)=(S=12127256,E=12127259,C=35)
	CT(39823)=(S=12127260,E=12127263,C=18)
	CT(39824)=(S=12127264,E=12127267,C=12)
	CT(39825)=(S=12127268,E=12127271,C=14)
	CT(39826)=(S=12127272,E=12127275,C=102)
	CT(39827)=(S=12127276,E=12127279,C=34)
	CT(39828)=(S=12127280,E=12127283,C=12)
	CT(39829)=(S=12127284,E=12127287,C=18)
	CT(39830)=(S=12127288,E=12127291,C=27)
	CT(39831)=(S=12127292,E=12127295,C=26)
	CT(39832)=(S=12127296,E=12127299,C=28)
	CT(39833)=(S=12127300,E=12127303,C=18)
	CT(39834)=(S=12127304,E=12127307,C=27)
	CT(39835)=(S=12127308,E=12127311,C=59)
	CT(39836)=(S=12127312,E=12127315,C=63)
	CT(39837)=(S=12127316,E=12127319,C=18)
	CT(39838)=(S=12127320,E=12127323,C=28)
	CT(39839)=(S=12127324,E=12127327,C=18)
	CT(39840)=(S=12127328,E=12127331,C=12)
	CT(39841)=(S=12127332,E=12127335,C=26)
	CT(39842)=(S=12127336,E=12127339,C=25)
	CT(39843)=(S=12127340,E=12127343,C=37)
	CT(39844)=(S=12127344,E=12127347,C=31)
	CT(39845)=(S=12127348,E=12127351,C=21)
	CT(39846)=(S=12127352,E=12127355,C=36)
	CT(39847)=(S=12127356,E=12127359,C=18)
	CT(39848)=(S=12127360,E=12127363,C=34)
	CT(39849)=(S=12127364,E=12127367,C=36)
	CT(39850)=(S=12127368,E=12127371,C=14)
	CT(39851)=(S=12127372,E=12127375,C=31)
	CT(39852)=(S=12127376,E=12127379,C=36)
	CT(39853)=(S=12127380,E=12127383,C=19)
	CT(39854)=(S=12127384,E=12127387,C=18)
	CT(39855)=(S=12127388,E=12127391,C=63)
	CT(39856)=(S=12127392,E=12127395,C=14)
	CT(39857)=(S=12127396,E=12127399,C=23)
	CT(39858)=(S=12127400,E=12127403,C=59)
	CT(39859)=(S=12127404,E=12127407,C=23)
	CT(39860)=(S=12127408,E=12127411,C=12)
	CT(39861)=(S=12127412,E=12127415,C=42)
	CT(39862)=(S=12127416,E=12127419,C=43)
	CT(39863)=(S=12127420,E=12127427,C=14)
	CT(39864)=(S=12127428,E=12127431,C=33)
	CT(39865)=(S=12127432,E=12127435,C=14)
	CT(39866)=(S=12127436,E=12127439,C=12)
	CT(39867)=(S=12127440,E=12127443,C=18)
	CT(39868)=(S=12127444,E=12127447,C=68)
	CT(39869)=(S=12127448,E=12127451,C=36)
	CT(39870)=(S=12127452,E=12127455,C=39)
	CT(39871)=(S=12127456,E=12127459,C=37)
	CT(39872)=(S=12127460,E=12127463,C=18)
	CT(39873)=(S=12127464,E=12127467,C=14)
	CT(39874)=(S=12127468,E=12127471,C=55)
	CT(39875)=(S=12127472,E=12127475,C=42)
	CT(39876)=(S=12127476,E=12127479,C=61)
	CT(39877)=(S=12127480,E=12127483,C=19)
	CT(39878)=(S=12127484,E=12127491,C=18)
	CT(39879)=(S=12127492,E=12127495,C=31)
	CT(39880)=(S=12127496,E=12127499,C=46)
	CT(39881)=(S=12127500,E=12127503,C=14)
	CT(39882)=(S=12127504,E=12127507,C=55)
	CT(39883)=(S=12127508,E=12127511,C=20)
	CT(39884)=(S=12127512,E=12127515,C=18)
	CT(39885)=(S=12127516,E=12127519,C=28)
	CT(39886)=(S=12127520,E=12127523,C=18)
	CT(39887)=(S=12127524,E=12127527,C=12)
	CT(39888)=(S=12127528,E=12127531,C=72)
	CT(39889)=(S=12127532,E=12127535,C=18)
	CT(39890)=(S=12127536,E=12127539,C=14)
	CT(39891)=(S=12127540,E=12127543,C=52)
	CT(39892)=(S=12127544,E=12127547,C=37)
	CT(39893)=(S=12127548,E=12127551,C=25)
	CT(39894)=(S=12127552,E=12127555,C=12)
	CT(39895)=(S=12127556,E=12127559,C=28)
	CT(39896)=(S=12127560,E=12127563,C=14)
	CT(39897)=(S=12127564,E=12127567,C=25)
	CT(39898)=(S=12127568,E=12127571,C=14)
	CT(39899)=(S=12127572,E=12127575,C=61)
	CT(39900)=(S=12127576,E=12127579,C=70)
	CT(39901)=(S=12127580,E=12127583,C=27)
	CT(39902)=(S=12127584,E=12127587,C=19)
	CT(39903)=(S=12127588,E=12127591,C=12)
	CT(39904)=(S=12127592,E=12127599,C=14)
	CT(39905)=(S=12127600,E=12127611,C=18)
	CT(39906)=(S=12127612,E=12127615,C=27)
	CT(39907)=(S=12127616,E=12127619,C=14)
	CT(39908)=(S=12127620,E=12127627,C=18)
	CT(39909)=(S=12127628,E=12127631,C=14)
	CT(39910)=(S=12127632,E=12127635,C=58)
	CT(39911)=(S=12127636,E=12127639,C=28)
	CT(39912)=(S=12127640,E=12127643,C=60)
	CT(39913)=(S=12127644,E=12127647,C=28)
	CT(39914)=(S=12127648,E=12127651,C=18)
	CT(39915)=(S=12127652,E=12127655,C=15)
	CT(39916)=(S=12127656,E=12127663,C=58)
	CT(39917)=(S=12127664,E=12127667,C=18)
	CT(39918)=(S=12127668,E=12127671,C=12)
	CT(39919)=(S=12127672,E=12127675,C=58)
	CT(39920)=(S=12127676,E=12127679,C=14)
	CT(39921)=(S=12127680,E=12127683,C=17)
	CT(39922)=(S=12127684,E=12127687,C=66)
	CT(39923)=(S=12127688,E=12127691,C=25)
	CT(39924)=(S=12127692,E=12127695,C=14)
	CT(39925)=(S=12127696,E=12127699,C=28)
	CT(39926)=(S=12127700,E=12127707,C=12)
	CT(39927)=(S=12127708,E=12127711,C=15)
	CT(39928)=(S=12127712,E=12127715,C=36)
	CT(39929)=(S=12127716,E=12127719,C=26)
	CT(39930)=(S=12127720,E=12127723,C=58)
	CT(39931)=(S=12127724,E=12127727,C=54)
	CT(39932)=(S=12127728,E=12127731,C=59)
	CT(39933)=(S=12127732,E=12127735,C=14)
	CT(39934)=(S=12127736,E=12127739,C=31)
	CT(39935)=(S=12127740,E=12127743,C=15)
	CT(39936)=(S=12127744,E=12127747,C=12)
	CT(39937)=(S=12127748,E=12127751,C=18)
	CT(39938)=(S=12127752,E=12127755,C=15)
	CT(39939)=(S=12127756,E=12127759,C=28)
	CT(39940)=(S=12127760,E=12127763,C=31)
	CT(39941)=(S=12127764,E=12127767,C=37)
	CT(39942)=(S=12127768,E=12127771,C=25)
	CT(39943)=(S=12127772,E=12127775,C=31)
	CT(39944)=(S=12127776,E=12127779,C=18)
	CT(39945)=(S=12127780,E=12127783,C=25)
	CT(39946)=(S=12127784,E=12127791,C=18)
	CT(39947)=(S=12127792,E=12127795,C=28)
	CT(39948)=(S=12127796,E=12127799,C=42)
	CT(39949)=(S=12127800,E=12127803,C=25)
	CT(39950)=(S=12127804,E=12127807,C=14)
	CT(39951)=(S=12127808,E=12127811,C=37)
	CT(39952)=(S=12127812,E=12127815,C=18)
	CT(39953)=(S=12127816,E=12127819,C=58)
	CT(39954)=(S=12127820,E=12127823,C=15)
	CT(39955)=(S=12127824,E=12127827,C=26)
	CT(39956)=(S=12127828,E=12127831,C=14)
	CT(39957)=(S=12127832,E=12127835,C=18)
	CT(39958)=(S=12127836,E=12127839,C=28)
	CT(39959)=(S=12127840,E=12127843,C=32)
	CT(39960)=(S=12127844,E=12127847,C=20)
	CT(39961)=(S=12127848,E=12127851,C=12)
	CT(39962)=(S=12127852,E=12127855,C=34)
	CT(39963)=(S=12127856,E=12127859,C=58)
	CT(39964)=(S=12127860,E=12127863,C=71)
	CT(39965)=(S=12127864,E=12127867,C=12)
	CT(39966)=(S=12127868,E=12127871,C=59)
	CT(39967)=(S=12127872,E=12127875,C=12)
	CT(39968)=(S=12127876,E=12127879,C=61)
	CT(39969)=(S=12127880,E=12127891,C=14)
	CT(39970)=(S=12127892,E=12127895,C=58)
	CT(39971)=(S=12127896,E=12127899,C=14)
	CT(39972)=(S=12127900,E=12127903,C=34)
	CT(39973)=(S=12127904,E=12127907,C=26)
	CT(39974)=(S=12127908,E=12127911,C=14)
	CT(39975)=(S=12127912,E=12127915,C=36)
	CT(39976)=(S=12127916,E=12127919,C=37)
	CT(39977)=(S=12127920,E=12127927,C=12)
	CT(39978)=(S=12127928,E=12127931,C=36)
	CT(39979)=(S=12127932,E=12127935,C=33)
	CT(39980)=(S=12127936,E=12127939,C=31)
	CT(39981)=(S=12127940,E=12127943,C=23)
	CT(39982)=(S=12127944,E=12127947,C=22)
	CT(39983)=(S=12127948,E=12127951,C=54)
	CT(39984)=(S=12127952,E=12127959,C=14)
	CT(39985)=(S=12127960,E=12127963,C=55)
	CT(39986)=(S=12127964,E=12127967,C=14)
	CT(39987)=(S=12127968,E=12127971,C=33)
	CT(39988)=(S=12127972,E=12127975,C=12)
	CT(39989)=(S=12127976,E=12127979,C=33)
	CT(39990)=(S=12127980,E=12127983,C=15)
	CT(39991)=(S=12127984,E=12127987,C=19)
	CT(39992)=(S=12127988,E=12127991,C=12)
	CT(39993)=(S=12127992,E=12127995,C=35)
	CT(39994)=(S=12127996,E=12127999,C=33)
	CT(39995)=(S=12128000,E=12128003,C=58)
	CT(39996)=(S=12128004,E=12128007,C=31)
	CT(39997)=(S=12128008,E=12128011,C=28)
	CT(39998)=(S=12128012,E=12128015,C=36)
	CT(39999)=(S=12128016,E=12128019,C=12)
	CT(40000)=(S=12128020,E=12128023,C=21)
	CT(40001)=(S=12128024,E=12128027,C=36)
	CT(40002)=(S=12128028,E=12128031,C=57)
	CT(40003)=(S=12128032,E=12128035,C=12)
	CT(40004)=(S=12128040,E=12128043,C=37)
	CT(40005)=(S=12128044,E=12128047,C=58)
	CT(40006)=(S=12128048,E=12128055,C=12)
	CT(40007)=(S=12128056,E=12128059,C=36)
	CT(40008)=(S=12128060,E=12128063,C=18)
	CT(40009)=(S=12128064,E=12128067,C=59)
	CT(40010)=(S=12128068,E=12128071,C=12)
	CT(40011)=(S=12128072,E=12128075,C=24)
	CT(40012)=(S=12128076,E=12128079,C=25)
	CT(40013)=(S=12128080,E=12128083,C=58)
	CT(40014)=(S=12128084,E=12128091,C=18)
	CT(40015)=(S=12128092,E=12128095,C=28)
	CT(40016)=(S=12128096,E=12128099,C=18)
	CT(40017)=(S=12128100,E=12128103,C=25)
	CT(40018)=(S=12128104,E=12128107,C=14)
	CT(40019)=(S=12128108,E=12128111,C=33)
	CT(40020)=(S=12128112,E=12128115,C=40)
	CT(40021)=(S=12128116,E=12128119,C=18)
	CT(40022)=(S=12128120,E=12128123,C=19)
	CT(40023)=(S=12128124,E=12128127,C=24)
	CT(40024)=(S=12128128,E=12128131,C=12)
	CT(40025)=(S=12128132,E=12128135,C=58)
	CT(40026)=(S=12128136,E=12128139,C=16)
	CT(40027)=(S=12128140,E=12128143,C=12)
	CT(40028)=(S=12128144,E=12128147,C=33)
	CT(40029)=(S=12128148,E=12128151,C=25)
	CT(40030)=(S=12128152,E=12128155,C=27)
	CT(40031)=(S=12128156,E=12128159,C=68)
	CT(40032)=(S=12128160,E=12128163,C=15)
	CT(40033)=(S=12128164,E=12128167,C=36)
	CT(40034)=(S=12128168,E=12128171,C=15)
	CT(40035)=(S=12128172,E=12128175,C=18)
	CT(40036)=(S=12128176,E=12128187,C=28)
	CT(40037)=(S=12128188,E=12128191,C=18)
	CT(40038)=(S=12128192,E=12128195,C=28)
	CT(40039)=(S=12128196,E=12128199,C=37)
	CT(40040)=(S=12128200,E=12128203,C=34)
	CT(40041)=(S=12128204,E=12128207,C=14)
	CT(40042)=(S=12128208,E=12128211,C=18)
	CT(40043)=(S=12128212,E=12128215,C=47)
	CT(40044)=(S=12128216,E=12128219,C=27)
	CT(40045)=(S=12128220,E=12128223,C=36)
	CT(40046)=(S=12128224,E=12128227,C=14)
	CT(40047)=(S=12128228,E=12128231,C=34)
	CT(40048)=(S=12128232,E=12128235,C=58)
	CT(40049)=(S=12128236,E=12128239,C=42)
	CT(40050)=(S=12128240,E=12128243,C=14)
	CT(40051)=(S=12128244,E=12128247,C=28)
	CT(40052)=(S=12128248,E=12128251,C=36)
	CT(40053)=(S=12128252,E=12128255,C=33)
	CT(40054)=(S=12128256,E=12128259,C=64)
	CT(40055)=(S=12128260,E=12128263,C=101)
	CT(40056)=(S=12128264,E=12128267,C=18)
	CT(40057)=(S=12128268,E=12128271,C=59)
	CT(40058)=(S=12128272,E=12128275,C=24)
	CT(40059)=(S=12128276,E=12128279,C=36)
	CT(40060)=(S=12128280,E=12128283,C=35)
	CT(40061)=(S=12128284,E=12128287,C=31)
	CT(40062)=(S=12128288,E=12128295,C=58)
	CT(40063)=(S=12128296,E=12128299,C=14)
	CT(40064)=(S=12128300,E=12128307,C=12)
	CT(40065)=(S=12128308,E=12128311,C=42)
	CT(40066)=(S=12128312,E=12128315,C=18)
	CT(40067)=(S=12128316,E=12128319,C=28)
	CT(40068)=(S=12128320,E=12128323,C=18)
	CT(40069)=(S=12128324,E=12128327,C=14)
	CT(40070)=(S=12128328,E=12128331,C=28)
	CT(40071)=(S=12128332,E=12128335,C=42)
	CT(40072)=(S=12128336,E=12128339,C=69)
	CT(40073)=(S=12128340,E=12128343,C=19)
	CT(40074)=(S=12128344,E=12128347,C=17)
	CT(40075)=(S=12128348,E=12128351,C=19)
	CT(40076)=(S=12128352,E=12128355,C=47)
	CT(40077)=(S=12128356,E=12128363,C=18)
	CT(40078)=(S=12128364,E=12128367,C=28)
	CT(40079)=(S=12128368,E=12128371,C=42)
	CT(40080)=(S=12128372,E=12128375,C=18)
	CT(40081)=(S=12128376,E=12128379,C=25)
	CT(40082)=(S=12128380,E=12128383,C=31)
	CT(40083)=(S=12128384,E=12128387,C=58)
	CT(40084)=(S=12128388,E=12128391,C=15)
	CT(40085)=(S=12128392,E=12128395,C=18)
	CT(40086)=(S=12128396,E=12128399,C=24)
	CT(40087)=(S=12128400,E=12128403,C=58)
	CT(40088)=(S=12128404,E=12128407,C=32)
	CT(40089)=(S=12128408,E=12128411,C=14)
	CT(40090)=(S=12128412,E=12128415,C=24)
	CT(40091)=(S=12128416,E=12128419,C=14)
	CT(40092)=(S=12128420,E=12128423,C=22)
	CT(40093)=(S=12128424,E=12128427,C=47)
	CT(40094)=(S=12128428,E=12128431,C=34)
	CT(40095)=(S=12128432,E=12128435,C=18)
	CT(40096)=(S=12128436,E=12128439,C=33)
	CT(40097)=(S=12128440,E=12128443,C=28)
	CT(40098)=(S=12128444,E=12128447,C=42)
	CT(40099)=(S=12128448,E=12128451,C=72)
	CT(40100)=(S=12128452,E=12128455,C=28)
	CT(40101)=(S=12128456,E=12128459,C=19)
	CT(40102)=(S=12128460,E=12128463,C=14)
	CT(40103)=(S=12128464,E=12128467,C=25)
	CT(40104)=(S=12128468,E=12128471,C=18)
	CT(40105)=(S=12128472,E=12128475,C=13)
	CT(40106)=(S=12128476,E=12128479,C=12)
	CT(40107)=(S=12128480,E=12128483,C=14)
	CT(40108)=(S=12128484,E=12128487,C=31)
	CT(40109)=(S=12128488,E=12128491,C=26)
	CT(40110)=(S=12128492,E=12128495,C=37)
	CT(40111)=(S=12128496,E=12128499,C=49)
	CT(40112)=(S=12128500,E=12128503,C=18)
	CT(40113)=(S=12128504,E=12128507,C=25)
	CT(40114)=(S=12128508,E=12128511,C=12)
	CT(40115)=(S=12128512,E=12128515,C=18)
	CT(40116)=(S=12128516,E=12128519,C=28)
	CT(40117)=(S=12128520,E=12128523,C=18)
	CT(40118)=(S=12128524,E=12128527,C=42)
	CT(40119)=(S=12128528,E=12128531,C=18)
	CT(40120)=(S=12128532,E=12128535,C=15)
	CT(40121)=(S=12128536,E=12128539,C=14)
	CT(40122)=(S=12128540,E=12128543,C=18)
	CT(40123)=(S=12128544,E=12128547,C=28)
	CT(40124)=(S=12128548,E=12128551,C=18)
	CT(40125)=(S=12128552,E=12128555,C=58)
	CT(40126)=(S=12128556,E=12128559,C=14)
	CT(40127)=(S=12128560,E=12128563,C=15)
	CT(40128)=(S=12128564,E=12128567,C=24)
	CT(40129)=(S=12128568,E=12128571,C=12)
	CT(40130)=(S=12128572,E=12128575,C=36)
	CT(40131)=(S=12128576,E=12128579,C=33)
	CT(40132)=(S=12128580,E=12128583,C=34)
	CT(40133)=(S=12128584,E=12128587,C=36)
	CT(40134)=(S=12128588,E=12128591,C=18)
	CT(40135)=(S=12128592,E=12128595,C=36)
	CT(40136)=(S=12128596,E=12128603,C=18)
	CT(40137)=(S=12128604,E=12128607,C=27)
	CT(40138)=(S=12128612,E=12128615,C=33)
	CT(40139)=(S=12128616,E=12128619,C=15)
	CT(40140)=(S=12128620,E=12128623,C=12)
	CT(40141)=(S=12128624,E=12128627,C=37)
	CT(40142)=(S=12128628,E=12128631,C=33)
	CT(40143)=(S=12128632,E=12128635,C=18)
	CT(40144)=(S=12128636,E=12128639,C=31)
	CT(40145)=(S=12128640,E=12128647,C=18)
	CT(40146)=(S=12128648,E=12128651,C=37)
	CT(40147)=(S=12128652,E=12128655,C=18)
	CT(40148)=(S=12128656,E=12128659,C=28)
	CT(40149)=(S=12128660,E=12128663,C=14)
	CT(40150)=(S=12128664,E=12128667,C=18)
	CT(40151)=(S=12128668,E=12128671,C=15)
	CT(40152)=(S=12128672,E=12128675,C=58)
	CT(40153)=(S=12128676,E=12128679,C=14)
	CT(40154)=(S=12128680,E=12128683,C=18)
	CT(40155)=(S=12128684,E=12128687,C=14)
	CT(40156)=(S=12128688,E=12128691,C=34)
	CT(40157)=(S=12128692,E=12128695,C=14)
	CT(40158)=(S=12128696,E=12128699,C=36)
	CT(40159)=(S=12128700,E=12128703,C=46)
	CT(40160)=(S=12128704,E=12128707,C=24)
	CT(40161)=(S=12128708,E=12128711,C=14)
	CT(40162)=(S=12128712,E=12128715,C=18)
	CT(40163)=(S=12128716,E=12128719,C=28)
	CT(40164)=(S=12128720,E=12128723,C=34)
	CT(40165)=(S=12128724,E=12128727,C=33)
	CT(40166)=(S=12128728,E=12128731,C=24)
	CT(40167)=(S=12128732,E=12128735,C=169)
	CT(40168)=(S=12128736,E=12128739,C=53)
	CT(40169)=(S=12128740,E=12128743,C=21)
	CT(40170)=(S=12128744,E=12128747,C=43)
	CT(40171)=(S=12128748,E=12128755,C=12)
	CT(40172)=(S=12128756,E=12128759,C=28)
	CT(40173)=(S=12128760,E=12128763,C=12)
	CT(40174)=(S=12128764,E=12128767,C=14)
	CT(40175)=(S=12128768,E=12128771,C=24)
	CT(40176)=(S=12128772,E=12128775,C=18)
	CT(40177)=(S=12128776,E=12128779,C=49)
	CT(40178)=(S=12128780,E=12128783,C=46)
	CT(40179)=(S=12128784,E=12128787,C=25)
	CT(40180)=(S=12128788,E=12128791,C=18)
	CT(40181)=(S=12128792,E=12128795,C=15)
	CT(40182)=(S=12128796,E=12128799,C=33)
	CT(40183)=(S=12128800,E=12128803,C=14)
	CT(40184)=(S=12128804,E=12128807,C=28)
	CT(40185)=(S=12128808,E=12128811,C=96)
	CT(40186)=(S=12128812,E=12128815,C=18)
	CT(40187)=(S=12128816,E=12128819,C=14)
	CT(40188)=(S=12128820,E=12128823,C=25)
	CT(40189)=(S=12128824,E=12128827,C=51)
	CT(40190)=(S=12128828,E=12128831,C=57)
	CT(40191)=(S=12128832,E=12128835,C=71)
	CT(40192)=(S=12128836,E=12128839,C=58)
	CT(40193)=(S=12128840,E=12128843,C=66)
	CT(40194)=(S=12128844,E=12128847,C=47)
	CT(40195)=(S=12128848,E=12128851,C=12)
	CT(40196)=(S=12128852,E=12128855,C=19)
	CT(40197)=(S=12128860,E=12128863,C=28)
	CT(40198)=(S=12128864,E=12128867,C=12)
	CT(40199)=(S=12128868,E=12128871,C=28)
	CT(40200)=(S=12128872,E=12128875,C=49)
	CT(40201)=(S=12128876,E=12128887,C=18)
	CT(40202)=(S=12128888,E=12128891,C=14)
	CT(40203)=(S=12128892,E=12128895,C=18)
	CT(40204)=(S=12128896,E=12128899,C=28)
	CT(40205)=(S=12128900,E=12128903,C=58)
	CT(40206)=(S=12128904,E=12128907,C=14)
	CT(40207)=(S=12128908,E=12128911,C=58)
	CT(40208)=(S=12128912,E=12128915,C=14)
	CT(40209)=(S=12128916,E=12128919,C=49)
	CT(40210)=(S=12128920,E=12128923,C=59)
	CT(40211)=(S=12128924,E=12128927,C=21)
	CT(40212)=(S=12128928,E=12128931,C=55)
	CT(40213)=(S=12128932,E=12128935,C=19)
	CT(40214)=(S=12128936,E=12128939,C=12)
	CT(40215)=(S=12128940,E=12128943,C=34)
	CT(40216)=(S=12128944,E=12128947,C=58)
	CT(40217)=(S=12128948,E=12128951,C=42)
	CT(40218)=(S=12128952,E=12128955,C=32)
	CT(40219)=(S=12128956,E=12128959,C=168)
	CT(40220)=(S=12128960,E=12128963,C=19)
	CT(40221)=(S=12128964,E=12128967,C=25)
	CT(40222)=(S=12128968,E=12128971,C=18)
	CT(40223)=(S=12128972,E=12128975,C=17)
	CT(40224)=(S=12128976,E=12128979,C=12)
	CT(40225)=(S=12128980,E=12128983,C=26)
	CT(40226)=(S=12128984,E=12128987,C=14)
	CT(40227)=(S=12128988,E=12128991,C=63)
	CT(40228)=(S=12128992,E=12128995,C=38)
	CT(40229)=(S=12128996,E=12128999,C=51)
	CT(40230)=(S=12129000,E=12129003,C=15)
	CT(40231)=(S=12129004,E=12129007,C=36)
	CT(40232)=(S=12129008,E=12129011,C=15)
	CT(40233)=(S=12129012,E=12129015,C=41)
	CT(40234)=(S=12129016,E=12129019,C=25)
	CT(40235)=(S=12129020,E=12129023,C=20)
	CT(40236)=(S=12129024,E=12129027,C=33)
	CT(40237)=(S=12129028,E=12129031,C=31)
	CT(40238)=(S=12129032,E=12129035,C=28)
	CT(40239)=(S=12129036,E=12129043,C=14)
	CT(40240)=(S=12129044,E=12129047,C=18)
	CT(40241)=(S=12129048,E=12129051,C=25)
	CT(40242)=(S=12129052,E=12129055,C=34)
	CT(40243)=(S=12129056,E=12129063,C=28)
	CT(40244)=(S=12129064,E=12129067,C=14)
	CT(40245)=(S=12129068,E=12129071,C=22)
	CT(40246)=(S=12129072,E=12129075,C=12)
	CT(40247)=(S=12129076,E=12129079,C=28)
	CT(40248)=(S=12129080,E=12129083,C=36)
	CT(40249)=(S=12129084,E=12129087,C=28)
	CT(40250)=(S=12129088,E=12129091,C=27)
	CT(40251)=(S=12129092,E=12129095,C=25)
	CT(40252)=(S=12129096,E=12129099,C=71)
	CT(40253)=(S=12129100,E=12129103,C=101)
	CT(40254)=(S=12129104,E=12129107,C=37)
	CT(40255)=(S=12129108,E=12129111,C=34)
	CT(40256)=(S=12129112,E=12129115,C=14)
	CT(40257)=(S=12129116,E=12129119,C=37)
	CT(40258)=(S=12129120,E=12129123,C=40)
	CT(40259)=(S=12129124,E=12129127,C=36)
	CT(40260)=(S=12129128,E=12129135,C=14)
	CT(40261)=(S=12129136,E=12129139,C=61)
	CT(40262)=(S=12129140,E=12129143,C=58)
	CT(40263)=(S=12129144,E=12129147,C=34)
	CT(40264)=(S=12129148,E=12129151,C=30)
	CT(40265)=(S=12129152,E=12129155,C=15)
	CT(40266)=(S=12129156,E=12129159,C=24)
	CT(40267)=(S=12129160,E=12129163,C=12)
	CT(40268)=(S=12129164,E=12129167,C=15)
	CT(40269)=(S=12129168,E=12129175,C=14)
	CT(40270)=(S=12129176,E=12129179,C=28)
	CT(40271)=(S=12129180,E=12129183,C=25)
	CT(40272)=(S=12129184,E=12129187,C=12)
	CT(40273)=(S=12129188,E=12129191,C=15)
	CT(40274)=(S=12129196,E=12129199,C=28)
	CT(40275)=(S=12129200,E=12129203,C=18)
	CT(40276)=(S=12129204,E=12129207,C=49)
	CT(40277)=(S=12129208,E=12129211,C=15)
	CT(40278)=(S=12129212,E=12129215,C=25)
	CT(40279)=(S=12129216,E=12129219,C=20)
	CT(40280)=(S=12129220,E=12129223,C=28)
	CT(40281)=(S=12129224,E=12129227,C=14)
	CT(40282)=(S=12129228,E=12129231,C=18)
	CT(40283)=(S=12129232,E=12129235,C=14)
	CT(40284)=(S=12129240,E=12129243,C=28)
	CT(40285)=(S=12129244,E=12129247,C=43)
	CT(40286)=(S=12129248,E=12129251,C=12)
	CT(40287)=(S=12129252,E=12129255,C=61)
	CT(40288)=(S=12129256,E=12129259,C=28)
	CT(40289)=(S=12129260,E=12129263,C=36)
	CT(40290)=(S=12129264,E=12129267,C=12)
	CT(40291)=(S=12129268,E=12129271,C=31)
	CT(40292)=(S=12129272,E=12129279,C=19)
	CT(40293)=(S=12129280,E=12129287,C=14)
	CT(40294)=(S=12129288,E=12129291,C=12)
	CT(40295)=(S=12129292,E=12129295,C=19)
	CT(40296)=(S=12129296,E=12129299,C=12)
	CT(40297)=(S=12129300,E=12129303,C=58)
	CT(40298)=(S=12129304,E=12129307,C=35)
	CT(40299)=(S=12129308,E=12129311,C=57)
	CT(40300)=(S=12129312,E=12129315,C=14)
	CT(40301)=(S=12129316,E=12129319,C=32)
	CT(40302)=(S=12129320,E=12129323,C=12)
	CT(40303)=(S=12129324,E=12129327,C=18)
	CT(40304)=(S=12129328,E=12129335,C=14)
	CT(40305)=(S=12129336,E=12129339,C=55)
	CT(40306)=(S=12129340,E=12129343,C=14)
	CT(40307)=(S=12129344,E=12129351,C=15)
	CT(40308)=(S=12129352,E=12129355,C=23)
	CT(40309)=(S=12129356,E=12129363,C=18)
	CT(40310)=(S=12129364,E=12129367,C=12)
	CT(40311)=(S=12129368,E=12129371,C=51)
	CT(40312)=(S=12129372,E=12129375,C=64)
	CT(40313)=(S=12129376,E=12129379,C=14)
	CT(40314)=(S=12129380,E=12129383,C=55)
	CT(40315)=(S=12129384,E=12129387,C=27)
	CT(40316)=(S=12129388,E=12129391,C=25)
	CT(40317)=(S=12129392,E=12129395,C=68)
	CT(40318)=(S=12129396,E=12129399,C=25)
	CT(40319)=(S=12129400,E=12129403,C=62)
	CT(40320)=(S=12129404,E=12129407,C=20)
	CT(40321)=(S=12129408,E=12129411,C=14)
	CT(40322)=(S=12129412,E=12129415,C=18)
	CT(40323)=(S=12129416,E=12129419,C=47)
	CT(40324)=(S=12129420,E=12129423,C=24)
	CT(40325)=(S=12129424,E=12129427,C=34)
	CT(40326)=(S=12129428,E=12129431,C=58)
	CT(40327)=(S=12129432,E=12129435,C=23)
	CT(40328)=(S=12129436,E=12129439,C=51)
	CT(40329)=(S=12129440,E=12129443,C=26)
	CT(40330)=(S=12129444,E=12129447,C=14)
	CT(40331)=(S=12129448,E=12129451,C=27)
	CT(40332)=(S=12129452,E=12129455,C=58)
	CT(40333)=(S=12129456,E=12129463,C=18)
	CT(40334)=(S=12129464,E=12129467,C=31)
	CT(40335)=(S=12129468,E=12129471,C=28)
	CT(40336)=(S=12129472,E=12129475,C=25)
	CT(40337)=(S=12129476,E=12129479,C=33)
	CT(40338)=(S=12129480,E=12129483,C=34)
	CT(40339)=(S=12129484,E=12129487,C=24)
	CT(40340)=(S=12129488,E=12129491,C=34)
	CT(40341)=(S=12129492,E=12129495,C=32)
	CT(40342)=(S=12129496,E=12129499,C=31)
	CT(40343)=(S=12129500,E=12129503,C=36)
	CT(40344)=(S=12129504,E=12129507,C=18)
	CT(40345)=(S=12129508,E=12129515,C=28)
	CT(40346)=(S=12129516,E=12129519,C=42)
	CT(40347)=(S=12129520,E=12129523,C=24)
	CT(40348)=(S=12129524,E=12129527,C=27)
	CT(40349)=(S=12129528,E=12129535,C=14)
	CT(40350)=(S=12129536,E=12129539,C=47)
	CT(40351)=(S=12129540,E=12129543,C=37)
	CT(40352)=(S=12129544,E=12129547,C=17)
	CT(40353)=(S=12129548,E=12129551,C=14)
	CT(40354)=(S=12129552,E=12129555,C=48)
	CT(40355)=(S=12129556,E=12129559,C=14)
	CT(40356)=(S=12129560,E=12129563,C=15)
	CT(40357)=(S=12129564,E=12129567,C=28)
	CT(40358)=(S=12129568,E=12129571,C=25)
	CT(40359)=(S=12129572,E=12129575,C=58)
	CT(40360)=(S=12129576,E=12129579,C=24)
	CT(40361)=(S=12129580,E=12129583,C=25)
	CT(40362)=(S=12129584,E=12129587,C=12)
	CT(40363)=(S=12129588,E=12129599,C=36)
	CT(40364)=(S=12129600,E=12129603,C=18)
	CT(40365)=(S=12129604,E=12129607,C=26)
	CT(40366)=(S=12129608,E=12129611,C=15)
	CT(40367)=(S=12129612,E=12129615,C=26)
	CT(40368)=(S=12129616,E=12129619,C=12)
	CT(40369)=(S=12129620,E=12129623,C=58)
	CT(40370)=(S=12129624,E=12129627,C=19)
	CT(40371)=(S=12129628,E=12129631,C=38)
	CT(40372)=(S=12129632,E=12129635,C=57)
	CT(40373)=(S=12129636,E=12129639,C=28)
	CT(40374)=(S=12129640,E=12129643,C=12)
	CT(40375)=(S=12129644,E=12129647,C=14)
	CT(40376)=(S=12129648,E=12129651,C=12)
	CT(40377)=(S=12129652,E=12129655,C=100)
	CT(40378)=(S=12129656,E=12129659,C=43)
	CT(40379)=(S=12129660,E=12129663,C=28)
	CT(40380)=(S=12129664,E=12129667,C=12)
	CT(40381)=(S=12129668,E=12129671,C=14)
	CT(40382)=(S=12129672,E=12129675,C=35)
	CT(40383)=(S=12129676,E=12129679,C=18)
	CT(40384)=(S=12129680,E=12129683,C=19)
	CT(40385)=(S=12129684,E=12129687,C=58)
	CT(40386)=(S=12129688,E=12129691,C=12)
	CT(40387)=(S=12129692,E=12129695,C=18)
	CT(40388)=(S=12129696,E=12129699,C=33)
	CT(40389)=(S=12129700,E=12129703,C=28)
	CT(40390)=(S=12129704,E=12129707,C=36)
	CT(40391)=(S=12129708,E=12129711,C=15)
	CT(40392)=(S=12129712,E=12129715,C=31)
	CT(40393)=(S=12129716,E=12129719,C=38)
	CT(40394)=(S=12129720,E=12129727,C=36)
	CT(40395)=(S=12129728,E=12129731,C=12)
	CT(40396)=(S=12129732,E=12129735,C=33)
	CT(40397)=(S=12129736,E=12129739,C=36)
	CT(40398)=(S=12129740,E=12129743,C=37)
	CT(40399)=(S=12129744,E=12129747,C=14)
	CT(40400)=(S=12129748,E=12129751,C=12)
	CT(40401)=(S=12129752,E=12129755,C=14)
	CT(40402)=(S=12129756,E=12129759,C=33)
	CT(40403)=(S=12129760,E=12129763,C=28)
	CT(40404)=(S=12129764,E=12129767,C=12)
	CT(40405)=(S=12129768,E=12129771,C=24)
	CT(40406)=(S=12129772,E=12129775,C=42)
	CT(40407)=(S=12129776,E=12129779,C=36)
	CT(40408)=(S=12129780,E=12129783,C=28)
	CT(40409)=(S=12129784,E=12129787,C=15)
	CT(40410)=(S=12129788,E=12129791,C=66)
	CT(40411)=(S=12129792,E=12129795,C=38)
	CT(40412)=(S=12129796,E=12129799,C=36)
	CT(40413)=(S=12129800,E=12129803,C=58)
	CT(40414)=(S=12129804,E=12129807,C=36)
	CT(40415)=(S=12129808,E=12129811,C=31)
	CT(40416)=(S=12129812,E=12129815,C=42)
	CT(40417)=(S=12129816,E=12129819,C=18)
	CT(40418)=(S=12129820,E=12129823,C=26)
	CT(40419)=(S=12129824,E=12129827,C=39)
	CT(40420)=(S=12129828,E=12129831,C=15)
	CT(40421)=(S=12129832,E=12129835,C=14)
	CT(40422)=(S=12129836,E=12129839,C=28)
	CT(40423)=(S=12129840,E=12129843,C=12)
	CT(40424)=(S=12129844,E=12129847,C=34)
	CT(40425)=(S=12129848,E=12129851,C=37)
	CT(40426)=(S=12129852,E=12129855,C=18)
	CT(40427)=(S=12129856,E=12129859,C=20)
	CT(40428)=(S=12129860,E=12129863,C=28)
	CT(40429)=(S=12129864,E=12129867,C=24)
	CT(40430)=(S=12129868,E=12129875,C=14)
	CT(40431)=(S=12129876,E=12129879,C=15)
	CT(40432)=(S=12129880,E=12129883,C=46)
	CT(40433)=(S=12129884,E=12129887,C=25)
	CT(40434)=(S=12129888,E=12129891,C=33)
	CT(40435)=(S=12129892,E=12129895,C=37)
	CT(40436)=(S=12129896,E=12129899,C=34)
	CT(40437)=(S=12129900,E=12129903,C=12)
	CT(40438)=(S=12129904,E=12129907,C=58)
	CT(40439)=(S=12129908,E=12129911,C=12)
	CT(40440)=(S=12129912,E=12129915,C=27)
	CT(40441)=(S=12129916,E=12129919,C=33)
	CT(40442)=(S=12129920,E=12129923,C=12)
	CT(40443)=(S=12129924,E=12129927,C=47)
	CT(40444)=(S=12129928,E=12129931,C=58)
	CT(40445)=(S=12129932,E=12129935,C=28)
	CT(40446)=(S=12129936,E=12129939,C=46)
	CT(40447)=(S=12129940,E=12129943,C=28)
	CT(40448)=(S=12129944,E=12129947,C=18)
	CT(40449)=(S=12129948,E=12129951,C=63)
	CT(40450)=(S=12129952,E=12129955,C=37)
	CT(40451)=(S=12129956,E=12129959,C=14)
	CT(40452)=(S=12129960,E=12129963,C=12)
	CT(40453)=(S=12129964,E=12129967,C=18)
	CT(40454)=(S=12129968,E=12129971,C=71)
	CT(40455)=(S=12129972,E=12129975,C=18)
	CT(40456)=(S=12129976,E=12129979,C=37)
	CT(40457)=(S=12129980,E=12129987,C=36)
	CT(40458)=(S=12129988,E=12129991,C=12)
	CT(40459)=(S=12129992,E=12129995,C=25)
	CT(40460)=(S=12129996,E=12129999,C=18)
	CT(40461)=(S=12130000,E=12130003,C=14)
	CT(40462)=(S=12130004,E=12130007,C=71)
	CT(40463)=(S=12130008,E=12130011,C=12)
	CT(40464)=(S=12130012,E=12130015,C=28)
	CT(40465)=(S=12130016,E=12130019,C=14)
	CT(40466)=(S=12130020,E=12130023,C=15)
	CT(40467)=(S=12130024,E=12130027,C=18)
	CT(40468)=(S=12130028,E=12130031,C=33)
	CT(40469)=(S=12130032,E=12130035,C=14)
	CT(40470)=(S=12130036,E=12130039,C=42)
	CT(40471)=(S=12130040,E=12130043,C=37)
	CT(40472)=(S=12130044,E=12130047,C=34)
	CT(40473)=(S=12130048,E=12130051,C=42)
	CT(40474)=(S=12130052,E=12130055,C=14)
	CT(40475)=(S=12130056,E=12130059,C=18)
	CT(40476)=(S=12130060,E=12130063,C=58)
	CT(40477)=(S=12130064,E=12130067,C=44)
	CT(40478)=(S=12130068,E=12130071,C=58)
	CT(40479)=(S=12130072,E=12130075,C=36)
	CT(40480)=(S=12130076,E=12130079,C=15)
	CT(40481)=(S=12130080,E=12130083,C=18)
	CT(40482)=(S=12130084,E=12130087,C=25)
	CT(40483)=(S=12130088,E=12130091,C=12)
	CT(40484)=(S=12130092,E=12130095,C=63)
	CT(40485)=(S=12130096,E=12130099,C=40)
	CT(40486)=(S=12130100,E=12130111,C=14)
	CT(40487)=(S=12130112,E=12130115,C=18)
	CT(40488)=(S=12130116,E=12130119,C=25)
	CT(40489)=(S=12130120,E=12130123,C=37)
	CT(40490)=(S=12130124,E=12130127,C=23)
	CT(40491)=(S=12130128,E=12130131,C=18)
	CT(40492)=(S=12130132,E=12130135,C=15)
	CT(40493)=(S=12130136,E=12130139,C=22)
	CT(40494)=(S=12130140,E=12130147,C=12)
	CT(40495)=(S=12130148,E=12130151,C=14)
	CT(40496)=(S=12130152,E=12130155,C=31)
	CT(40497)=(S=12130156,E=12130159,C=50)
	CT(40498)=(S=12130160,E=12130163,C=71)
	CT(40499)=(S=12130164,E=12130167,C=14)
	CT(40500)=(S=12130168,E=12130171,C=25)
	CT(40501)=(S=12130172,E=12130175,C=23)
	CT(40502)=(S=12130176,E=12130179,C=26)
	CT(40503)=(S=12130180,E=12130187,C=12)
	CT(40504)=(S=12130188,E=12130191,C=36)
	CT(40505)=(S=12130192,E=12130195,C=27)
	CT(40506)=(S=12130196,E=12130199,C=58)
	CT(40507)=(S=12130200,E=12130203,C=35)
	CT(40508)=(S=12130204,E=12130207,C=28)
	CT(40509)=(S=12130208,E=12130211,C=60)
	CT(40510)=(S=12130212,E=12130215,C=31)
	CT(40511)=(S=12130216,E=12130219,C=22)
	CT(40512)=(S=12130220,E=12130223,C=17)
	CT(40513)=(S=12130224,E=12130231,C=14)
	CT(40514)=(S=12130232,E=12130235,C=30)
	CT(40515)=(S=12130236,E=12130239,C=12)
	CT(40516)=(S=12130240,E=12130243,C=25)
	CT(40517)=(S=12130244,E=12130247,C=16)
	CT(40518)=(S=12130248,E=12130251,C=103)
	CT(40519)=(S=12130252,E=12130255,C=27)
	CT(40520)=(S=12130256,E=12130259,C=28)
	CT(40521)=(S=12130260,E=12130263,C=36)
	CT(40522)=(S=12130264,E=12130267,C=24)
	CT(40523)=(S=12130268,E=12130271,C=25)
	CT(40524)=(S=12130272,E=12130275,C=28)
	CT(40525)=(S=12130276,E=12130279,C=18)
	CT(40526)=(S=12130280,E=12130283,C=14)
	CT(40527)=(S=12130284,E=12130287,C=28)
	CT(40528)=(S=12130288,E=12130291,C=16)
	CT(40529)=(S=12130292,E=12130295,C=49)
	CT(40530)=(S=12130296,E=12130299,C=36)
	CT(40531)=(S=12130300,E=12130303,C=14)
	CT(40532)=(S=12130304,E=12130307,C=48)
	CT(40533)=(S=12130308,E=12130311,C=25)
	CT(40534)=(S=12130312,E=12130315,C=33)
	CT(40535)=(S=12130316,E=12130319,C=14)
	CT(40536)=(S=12130320,E=12130323,C=12)
	CT(40537)=(S=12130324,E=12130327,C=33)
	CT(40538)=(S=12130328,E=12130331,C=58)
	CT(40539)=(S=12130332,E=12130335,C=12)
	CT(40540)=(S=12130336,E=12130339,C=46)
	CT(40541)=(S=12130340,E=12130343,C=52)
	CT(40542)=(S=12130344,E=12130351,C=18)
	CT(40543)=(S=12130352,E=12130355,C=36)
	CT(40544)=(S=12130356,E=12130363,C=18)
	CT(40545)=(S=12130364,E=12130367,C=30)
	CT(40546)=(S=12130368,E=12130371,C=14)
	CT(40547)=(S=12130372,E=12130375,C=28)
	CT(40548)=(S=12130376,E=12130379,C=14)
	CT(40549)=(S=12130380,E=12130383,C=17)
	CT(40550)=(S=12130384,E=12130387,C=37)
	CT(40551)=(S=12130388,E=12130391,C=14)
	CT(40552)=(S=12130392,E=12130395,C=58)
	CT(40553)=(S=12130396,E=12130399,C=18)
	CT(40554)=(S=12130400,E=12130403,C=14)
	CT(40555)=(S=12130404,E=12130407,C=42)
	CT(40556)=(S=12130408,E=12130411,C=15)
	CT(40557)=(S=12130412,E=12130419,C=18)
	CT(40558)=(S=12130420,E=12130423,C=15)
	CT(40559)=(S=12130424,E=12130427,C=14)
	CT(40560)=(S=12130428,E=12130431,C=37)
	CT(40561)=(S=12130432,E=12130435,C=61)
	CT(40562)=(S=12130436,E=12130439,C=36)
	CT(40563)=(S=12130440,E=12130443,C=26)
	CT(40564)=(S=12130444,E=12130451,C=12)
	CT(40565)=(S=12130452,E=12130455,C=26)
	CT(40566)=(S=12130456,E=12130463,C=12)
	CT(40567)=(S=12130464,E=12130467,C=14)
	CT(40568)=(S=12130468,E=12130471,C=58)
	CT(40569)=(S=12130472,E=12130475,C=24)
	CT(40570)=(S=12130476,E=12130479,C=36)
	CT(40571)=(S=12130480,E=12130487,C=18)
	CT(40572)=(S=12130488,E=12130491,C=12)
	CT(40573)=(S=12130492,E=12130495,C=27)
	CT(40574)=(S=12130496,E=12130499,C=12)
	CT(40575)=(S=12130500,E=12130507,C=58)
	CT(40576)=(S=12130508,E=12130511,C=17)
	CT(40577)=(S=12130512,E=12130515,C=59)
	CT(40578)=(S=12130516,E=12130519,C=18)
	CT(40579)=(S=12130520,E=12130523,C=58)
	CT(40580)=(S=12130524,E=12130531,C=36)
	CT(40581)=(S=12130532,E=12130535,C=26)
	CT(40582)=(S=12130536,E=12130539,C=63)
	CT(40583)=(S=12130540,E=12130543,C=33)
	CT(40584)=(S=12130544,E=12130547,C=36)
	CT(40585)=(S=12130548,E=12130551,C=18)
	CT(40586)=(S=12130552,E=12130555,C=36)
	CT(40587)=(S=12130556,E=12130559,C=26)
	CT(40588)=(S=12130560,E=12130563,C=44)
	CT(40589)=(S=12130564,E=12130571,C=18)
	CT(40590)=(S=12130572,E=12130575,C=94)
	CT(40591)=(S=12130576,E=12130579,C=18)
	CT(40592)=(S=12130580,E=12130583,C=22)
	CT(40593)=(S=12130584,E=12130587,C=25)
	CT(40594)=(S=12130588,E=12130591,C=34)
	CT(40595)=(S=12130592,E=12130595,C=14)
	CT(40596)=(S=12130596,E=12130599,C=28)
	CT(40597)=(S=12130600,E=12130603,C=12)
	CT(40598)=(S=12130604,E=12130607,C=19)
	CT(40599)=(S=12130608,E=12130611,C=44)
	CT(40600)=(S=12130612,E=12130615,C=22)
	CT(40601)=(S=12130616,E=12130619,C=14)
	CT(40602)=(S=12130620,E=12130623,C=18)
	CT(40603)=(S=12130624,E=12130627,C=14)
	CT(40604)=(S=12130628,E=12130631,C=18)
	CT(40605)=(S=12130632,E=12130635,C=15)
	CT(40606)=(S=12130636,E=12130639,C=47)
	CT(40607)=(S=12130640,E=12130643,C=36)
	CT(40608)=(S=12130644,E=12130647,C=14)
	CT(40609)=(S=12130648,E=12130651,C=18)
	CT(40610)=(S=12130652,E=12130655,C=34)
	CT(40611)=(S=12130656,E=12130659,C=28)
	CT(40612)=(S=12130660,E=12130663,C=37)
	CT(40613)=(S=12130664,E=12130667,C=16)
	CT(40614)=(S=12130668,E=12130671,C=58)
	CT(40615)=(S=12130672,E=12130675,C=25)
	CT(40616)=(S=12130676,E=12130679,C=31)
	CT(40617)=(S=12130680,E=12130683,C=58)
	CT(40618)=(S=12130684,E=12130687,C=42)
	CT(40619)=(S=12130688,E=12130691,C=15)
	CT(40620)=(S=12130692,E=12130695,C=28)
	CT(40621)=(S=12130696,E=12130699,C=15)
	CT(40622)=(S=12130700,E=12130703,C=24)
	CT(40623)=(S=12130704,E=12130707,C=14)
	CT(40624)=(S=12130708,E=12130711,C=58)
	CT(40625)=(S=12130712,E=12130715,C=18)
	CT(40626)=(S=12130716,E=12130719,C=12)
	CT(40627)=(S=12130720,E=12130723,C=49)
	CT(40628)=(S=12130724,E=12130727,C=28)
	CT(40629)=(S=12130728,E=12130731,C=49)
	CT(40630)=(S=12130732,E=12130735,C=14)
	CT(40631)=(S=12130736,E=12130739,C=18)
	CT(40632)=(S=12130740,E=12130743,C=13)
	CT(40633)=(S=12130744,E=12130747,C=33)
	CT(40634)=(S=12130748,E=12130751,C=14)
	CT(40635)=(S=12130752,E=12130755,C=34)
	CT(40636)=(S=12130756,E=12130759,C=25)
	CT(40637)=(S=12130760,E=12130763,C=47)
	CT(40638)=(S=12130764,E=12130767,C=15)
	CT(40639)=(S=12130768,E=12130771,C=14)
	CT(40640)=(S=12130772,E=12130775,C=24)
	CT(40641)=(S=12130776,E=12130779,C=58)
	CT(40642)=(S=12130780,E=12130783,C=14)
	CT(40643)=(S=12130784,E=12130787,C=34)
	CT(40644)=(S=12130788,E=12130791,C=36)
	CT(40645)=(S=12130792,E=12130795,C=15)
	CT(40646)=(S=12130796,E=12130807,C=14)
	CT(40647)=(S=12130808,E=12130811,C=71)
	CT(40648)=(S=12130812,E=12130815,C=48)
	CT(40649)=(S=12130816,E=12130819,C=51)
	CT(40650)=(S=12130820,E=12130823,C=25)
	CT(40651)=(S=12130824,E=12130827,C=19)
	CT(40652)=(S=12130828,E=12130835,C=14)
	CT(40653)=(S=12130836,E=12130839,C=49)
	CT(40654)=(S=12130840,E=12130843,C=16)
	CT(40655)=(S=12130844,E=12130847,C=18)
	CT(40656)=(S=12130848,E=12130851,C=26)
	CT(40657)=(S=12130852,E=12130855,C=28)
	CT(40658)=(S=12130856,E=12130863,C=18)
	CT(40659)=(S=12130864,E=12130867,C=47)
	CT(40660)=(S=12130868,E=12130871,C=18)
	CT(40661)=(S=12130872,E=12130875,C=36)
	CT(40662)=(S=12130876,E=12130879,C=24)
	CT(40663)=(S=12130880,E=12130883,C=15)
	CT(40664)=(S=12130884,E=12130887,C=37)
	CT(40665)=(S=12130888,E=12130891,C=18)
	CT(40666)=(S=12130892,E=12130899,C=28)
	CT(40667)=(S=12130900,E=12130903,C=23)
	CT(40668)=(S=12130904,E=12130907,C=12)
	CT(40669)=(S=12130908,E=12130911,C=14)
	CT(40670)=(S=12130912,E=12130915,C=28)
	CT(40671)=(S=12130916,E=12130919,C=18)
	CT(40672)=(S=12130920,E=12130923,C=12)
	CT(40673)=(S=12130924,E=12130927,C=36)
	CT(40674)=(S=12130928,E=12130931,C=18)
	CT(40675)=(S=12130932,E=12130935,C=46)
	CT(40676)=(S=12130936,E=12130939,C=18)
	CT(40677)=(S=12130940,E=12130943,C=12)
	CT(40678)=(S=12130944,E=12130947,C=27)
	CT(40679)=(S=12130948,E=12130951,C=44)
	CT(40680)=(S=12130952,E=12130955,C=25)
	CT(40681)=(S=12130956,E=12130959,C=15)
	CT(40682)=(S=12130960,E=12130963,C=37)
	CT(40683)=(S=12130964,E=12130967,C=14)
	CT(40684)=(S=12130968,E=12130971,C=99)
	CT(40685)=(S=12130972,E=12130975,C=28)
	CT(40686)=(S=12130976,E=12130979,C=70)
	CT(40687)=(S=12130980,E=12130983,C=27)
	CT(40688)=(S=12130984,E=12130987,C=18)
	CT(40689)=(S=12130988,E=12130991,C=46)
	CT(40690)=(S=12130992,E=12130995,C=59)
	CT(40691)=(S=12130996,E=12130999,C=27)
	CT(40692)=(S=12131000,E=12131003,C=41)
	CT(40693)=(S=12131004,E=12131007,C=42)
	CT(40694)=(S=12131008,E=12131011,C=18)
	CT(40695)=(S=12131012,E=12131015,C=15)
	CT(40696)=(S=12131016,E=12131019,C=28)
	CT(40697)=(S=12131020,E=12131023,C=18)
	CT(40698)=(S=12131024,E=12131027,C=26)
	CT(40699)=(S=12131028,E=12131031,C=36)
	CT(40700)=(S=12131032,E=12131035,C=18)
	CT(40701)=(S=12131036,E=12131039,C=12)
	CT(40702)=(S=12131040,E=12131043,C=25)
	CT(40703)=(S=12131044,E=12131047,C=14)
	CT(40704)=(S=12131048,E=12131051,C=26)
	CT(40705)=(S=12131052,E=12131055,C=13)
	CT(40706)=(S=12131056,E=12131059,C=14)
	CT(40707)=(S=12131060,E=12131067,C=12)
	CT(40708)=(S=12131068,E=12131071,C=28)
	CT(40709)=(S=12131072,E=12131075,C=24)
	CT(40710)=(S=12131076,E=12131079,C=34)
	CT(40711)=(S=12131080,E=12131083,C=28)
	CT(40712)=(S=12131088,E=12131091,C=36)
	CT(40713)=(S=12131092,E=12131095,C=14)
	CT(40714)=(S=12131096,E=12131099,C=27)
	CT(40715)=(S=12131100,E=12131107,C=36)
	CT(40716)=(S=12131108,E=12131111,C=48)
	CT(40717)=(S=12131112,E=12131115,C=36)
	CT(40718)=(S=12131116,E=12131123,C=18)
	CT(40719)=(S=12131124,E=12131127,C=58)
	CT(40720)=(S=12131128,E=12131131,C=72)
	CT(40721)=(S=12131132,E=12131135,C=50)
	CT(40722)=(S=12131136,E=12131139,C=56)
	CT(40723)=(S=12131140,E=12131143,C=20)
	CT(40724)=(S=12131144,E=12131147,C=15)
	CT(40725)=(S=12131148,E=12131151,C=12)
	CT(40726)=(S=12131152,E=12131155,C=18)
	CT(40727)=(S=12131156,E=12131159,C=28)
	CT(40728)=(S=12131160,E=12131163,C=45)
	CT(40729)=(S=12131164,E=12131167,C=60)
	CT(40730)=(S=12131168,E=12131171,C=19)
	CT(40731)=(S=12131172,E=12131175,C=18)
	CT(40732)=(S=12131176,E=12131179,C=17)
	CT(40733)=(S=12131180,E=12131183,C=34)
	CT(40734)=(S=12131184,E=12131187,C=24)
	CT(40735)=(S=12131188,E=12131191,C=61)
	CT(40736)=(S=12131192,E=12131195,C=42)
	CT(40737)=(S=12131196,E=12131199,C=25)
	CT(40738)=(S=12131200,E=12131203,C=46)
	CT(40739)=(S=12131204,E=12131207,C=14)
	CT(40740)=(S=12131208,E=12131211,C=25)
	CT(40741)=(S=12131212,E=12131215,C=36)
	CT(40742)=(S=12131216,E=12131219,C=14)
	CT(40743)=(S=12131220,E=12131223,C=18)
	CT(40744)=(S=12131224,E=12131231,C=28)
	CT(40745)=(S=12131232,E=12131235,C=18)
	CT(40746)=(S=12131236,E=12131243,C=19)
	CT(40747)=(S=12131244,E=12131247,C=36)
	CT(40748)=(S=12131248,E=12131251,C=14)
	CT(40749)=(S=12131252,E=12131255,C=28)
	CT(40750)=(S=12131256,E=12131259,C=34)
	CT(40751)=(S=12131260,E=12131263,C=58)
	CT(40752)=(S=12131264,E=12131267,C=18)
	CT(40753)=(S=12131268,E=12131271,C=14)
	CT(40754)=(S=12131272,E=12131275,C=25)
	CT(40755)=(S=12131276,E=12131279,C=24)
	CT(40756)=(S=12131280,E=12131283,C=18)
	CT(40757)=(S=12131284,E=12131287,C=28)
	CT(40758)=(S=12131288,E=12131291,C=35)
	CT(40759)=(S=12131292,E=12131295,C=23)
	CT(40760)=(S=12131296,E=12131299,C=12)
	CT(40761)=(S=12131300,E=12131303,C=13)
	CT(40762)=(S=12131304,E=12131307,C=19)
	CT(40763)=(S=12131308,E=12131311,C=36)
	CT(40764)=(S=12131312,E=12131315,C=19)
	CT(40765)=(S=12131316,E=12131319,C=14)
	CT(40766)=(S=12131320,E=12131323,C=36)
	CT(40767)=(S=12131324,E=12131327,C=28)
	CT(40768)=(S=12131328,E=12131331,C=37)
	CT(40769)=(S=12131332,E=12131335,C=28)
	CT(40770)=(S=12131336,E=12131339,C=52)
	CT(40771)=(S=12131340,E=12131343,C=14)
	CT(40772)=(S=12131344,E=12131347,C=58)
	CT(40773)=(S=12131348,E=12131351,C=44)
	CT(40774)=(S=12131352,E=12131355,C=14)
	CT(40775)=(S=12131356,E=12131359,C=12)
	CT(40776)=(S=12131360,E=12131363,C=14)
	CT(40777)=(S=12131364,E=12131367,C=49)
	CT(40778)=(S=12131368,E=12131371,C=12)
	CT(40779)=(S=12131372,E=12131375,C=14)
	CT(40780)=(S=12131376,E=12131379,C=25)
	CT(40781)=(S=12131380,E=12131383,C=15)
	CT(40782)=(S=12131384,E=12131387,C=36)
	CT(40783)=(S=12131388,E=12131391,C=37)
	CT(40784)=(S=12131392,E=12131395,C=15)
	CT(40785)=(S=12131396,E=12131399,C=36)
	CT(40786)=(S=12131400,E=12131403,C=64)
	CT(40787)=(S=12131404,E=12131407,C=28)
	CT(40788)=(S=12131408,E=12131411,C=15)
	CT(40789)=(S=12131412,E=12131415,C=36)
	CT(40790)=(S=12131416,E=12131423,C=14)
	CT(40791)=(S=12131424,E=12131427,C=28)
	CT(40792)=(S=12131428,E=12131431,C=71)
	CT(40793)=(S=12131432,E=12131435,C=53)
	CT(40794)=(S=12131436,E=12131439,C=31)
	CT(40795)=(S=12131440,E=12131443,C=14)
	CT(40796)=(S=12131444,E=12131447,C=58)
	CT(40797)=(S=12131448,E=12131451,C=55)
	CT(40798)=(S=12131452,E=12131455,C=71)
	CT(40799)=(S=12131456,E=12131459,C=27)
	CT(40800)=(S=12131460,E=12131463,C=37)
	CT(40801)=(S=12131464,E=12131467,C=19)
	CT(40802)=(S=12131468,E=12131471,C=25)
	CT(40803)=(S=12131472,E=12131475,C=71)
	CT(40804)=(S=12131476,E=12131479,C=36)
	CT(40805)=(S=12131480,E=12131483,C=17)
	CT(40806)=(S=12131484,E=12131487,C=28)
	CT(40807)=(S=12131488,E=12131491,C=37)
	CT(40808)=(S=12131492,E=12131495,C=14)
	CT(40809)=(S=12131496,E=12131499,C=25)
	CT(40810)=(S=12131500,E=12131503,C=28)
	CT(40811)=(S=12131504,E=12131507,C=58)
	CT(40812)=(S=12131512,E=12131515,C=28)
	CT(40813)=(S=12131516,E=12131519,C=36)
	CT(40814)=(S=12131520,E=12131523,C=33)
	CT(40815)=(S=12131524,E=12131527,C=63)
	CT(40816)=(S=12131528,E=12131535,C=14)
	CT(40817)=(S=12131536,E=12131539,C=28)
	CT(40818)=(S=12131540,E=12131543,C=14)
	CT(40819)=(S=12131544,E=12131547,C=25)
	CT(40820)=(S=12131548,E=12131555,C=36)
	CT(40821)=(S=12131556,E=12131559,C=28)
	CT(40822)=(S=12131560,E=12131563,C=12)
	CT(40823)=(S=12131564,E=12131567,C=31)
	CT(40824)=(S=12131568,E=12131571,C=14)
	CT(40825)=(S=12131572,E=12131575,C=47)
	CT(40826)=(S=12131576,E=12131579,C=58)
	CT(40827)=(S=12131580,E=12131587,C=14)
	CT(40828)=(S=12131588,E=12131591,C=49)
	CT(40829)=(S=12131592,E=12131595,C=13)
	CT(40830)=(S=12131596,E=12131599,C=58)
	CT(40831)=(S=12131600,E=12131603,C=52)
	CT(40832)=(S=12131604,E=12131607,C=19)
	CT(40833)=(S=12131608,E=12131611,C=22)
	CT(40834)=(S=12131612,E=12131615,C=28)
	CT(40835)=(S=12131616,E=12131627,C=12)
	CT(40836)=(S=12131628,E=12131631,C=14)
	CT(40837)=(S=12131632,E=12131635,C=12)
	CT(40838)=(S=12131636,E=12131643,C=18)
	CT(40839)=(S=12131644,E=12131647,C=36)
	CT(40840)=(S=12131648,E=12131651,C=12)
	CT(40841)=(S=12131652,E=12131655,C=25)
	CT(40842)=(S=12131656,E=12131659,C=15)
	CT(40843)=(S=12131660,E=12131663,C=24)
	CT(40844)=(S=12131664,E=12131667,C=50)
	CT(40845)=(S=12131668,E=12131671,C=58)
	CT(40846)=(S=12131672,E=12131675,C=42)
	CT(40847)=(S=12131676,E=12131679,C=23)
	CT(40848)=(S=12131680,E=12131683,C=15)
	CT(40849)=(S=12131684,E=12131687,C=46)
	CT(40850)=(S=12131688,E=12131691,C=15)
	CT(40851)=(S=12131692,E=12131695,C=36)
	CT(40852)=(S=12131696,E=12131699,C=19)
	CT(40853)=(S=12131700,E=12131703,C=28)
	CT(40854)=(S=12131704,E=12131707,C=37)
	CT(40855)=(S=12131708,E=12131715,C=18)
	CT(40856)=(S=12131716,E=12131723,C=14)
	CT(40857)=(S=12131724,E=12131727,C=52)
	CT(40858)=(S=12131728,E=12131731,C=15)
	CT(40859)=(S=12131732,E=12131735,C=12)
	CT(40860)=(S=12131736,E=12131739,C=34)
	CT(40861)=(S=12131740,E=12131743,C=71)
	CT(40862)=(S=12131744,E=12131747,C=15)
	CT(40863)=(S=12131748,E=12131751,C=36)
	CT(40864)=(S=12131752,E=12131755,C=28)
	CT(40865)=(S=12131756,E=12131763,C=14)
	CT(40866)=(S=12131764,E=12131767,C=12)
	CT(40867)=(S=12131768,E=12131771,C=62)
	CT(40868)=(S=12131772,E=12131775,C=28)
	CT(40869)=(S=12131776,E=12131779,C=37)
	CT(40870)=(S=12131780,E=12131783,C=48)
	CT(40871)=(S=12131784,E=12131787,C=36)
	CT(40872)=(S=12131788,E=12131791,C=15)
	CT(40873)=(S=12131792,E=12131795,C=14)
	CT(40874)=(S=12131796,E=12131799,C=25)
	CT(40875)=(S=12131800,E=12131803,C=28)
	CT(40876)=(S=12131804,E=12131807,C=26)
	CT(40877)=(S=12131808,E=12131815,C=14)
	CT(40878)=(S=12131816,E=12131819,C=12)
	CT(40879)=(S=12131820,E=12131823,C=31)
	CT(40880)=(S=12131824,E=12131827,C=28)
	CT(40881)=(S=12131828,E=12131831,C=12)
	CT(40882)=(S=12131832,E=12131835,C=15)
	CT(40883)=(S=12131836,E=12131839,C=31)
	CT(40884)=(S=12131840,E=12131843,C=58)
	CT(40885)=(S=12131844,E=12131847,C=26)
	CT(40886)=(S=12131848,E=12131851,C=14)
	CT(40887)=(S=12131852,E=12131863,C=18)
	CT(40888)=(S=12131864,E=12131867,C=14)
	CT(40889)=(S=12131868,E=12131871,C=12)
	CT(40890)=(S=12131872,E=12131875,C=28)
	CT(40891)=(S=12131876,E=12131879,C=39)
	CT(40892)=(S=12131880,E=12131883,C=18)
	CT(40893)=(S=12131884,E=12131887,C=15)
	CT(40894)=(S=12131888,E=12131891,C=12)
	CT(40895)=(S=12131892,E=12131895,C=49)
	CT(40896)=(S=12131896,E=12131899,C=36)
	CT(40897)=(S=12131900,E=12131915,C=15)
	CT(40898)=(S=12131916,E=12131919,C=26)
	CT(40899)=(S=12131920,E=12131923,C=15)
	CT(40900)=(S=12131924,E=12131927,C=31)
	CT(40901)=(S=12131928,E=12131931,C=41)
	CT(40902)=(S=12131932,E=12131935,C=12)
	CT(40903)=(S=12131936,E=12131939,C=18)
	CT(40904)=(S=12131940,E=12131943,C=24)
	CT(40905)=(S=12131944,E=12131947,C=18)
	CT(40906)=(S=12131948,E=12131955,C=15)
	CT(40907)=(S=12131956,E=12131959,C=18)
	CT(40908)=(S=12131960,E=12131963,C=71)
	CT(40909)=(S=12131964,E=12131967,C=58)
	CT(40910)=(S=12131968,E=12131975,C=12)
	CT(40911)=(S=12131976,E=12131979,C=52)
	CT(40912)=(S=12131980,E=12131983,C=25)
	CT(40913)=(S=12131984,E=12131987,C=96)
	CT(40914)=(S=12131988,E=12131991,C=23)
	CT(40915)=(S=12131992,E=12131995,C=19)
	CT(40916)=(S=12131996,E=12131999,C=28)
	CT(40917)=(S=12132000,E=12132003,C=24)
	CT(40918)=(S=12132004,E=12132007,C=28)
	CT(40919)=(S=12132008,E=12132011,C=14)
	CT(40920)=(S=12132012,E=12132015,C=70)
	CT(40921)=(S=12132016,E=12132019,C=36)
	CT(40922)=(S=12132020,E=12132023,C=15)
	CT(40923)=(S=12132024,E=12132027,C=48)
	CT(40924)=(S=12132028,E=12132031,C=15)
	CT(40925)=(S=12132032,E=12132035,C=18)
	CT(40926)=(S=12132036,E=12132039,C=25)
	CT(40927)=(S=12132040,E=12132043,C=31)
	CT(40928)=(S=12132044,E=12132047,C=36)
	CT(40929)=(S=12132048,E=12132051,C=12)
	CT(40930)=(S=12132052,E=12132055,C=14)
	CT(40931)=(S=12132056,E=12132059,C=12)
	CT(40932)=(S=12132060,E=12132063,C=18)
	CT(40933)=(S=12132064,E=12132067,C=27)
	CT(40934)=(S=12132068,E=12132071,C=18)
	CT(40935)=(S=12132072,E=12132075,C=17)
	CT(40936)=(S=12132076,E=12132079,C=36)
	CT(40937)=(S=12132080,E=12132087,C=25)
	CT(40938)=(S=12132088,E=12132091,C=61)
	CT(40939)=(S=12132092,E=12132095,C=49)
	CT(40940)=(S=12132096,E=12132099,C=12)
	CT(40941)=(S=12132100,E=12132103,C=28)
	CT(40942)=(S=12132104,E=12132107,C=36)
	CT(40943)=(S=12132108,E=12132111,C=14)
	CT(40944)=(S=12132112,E=12132115,C=13)
	CT(40945)=(S=12132116,E=12132119,C=25)
	CT(40946)=(S=12132120,E=12132123,C=58)
	CT(40947)=(S=12132124,E=12132127,C=42)
	CT(40948)=(S=12132128,E=12132131,C=46)
	CT(40949)=(S=12132132,E=12132135,C=33)
	CT(40950)=(S=12132136,E=12132139,C=12)
	CT(40951)=(S=12132140,E=12132143,C=60)
	CT(40952)=(S=12132144,E=12132147,C=58)
	CT(40953)=(S=12132148,E=12132151,C=42)
	CT(40954)=(S=12132152,E=12132155,C=15)
	CT(40955)=(S=12132156,E=12132159,C=28)
	CT(40956)=(S=12132160,E=12132163,C=15)
	CT(40957)=(S=12132164,E=12132167,C=47)
	CT(40958)=(S=12132168,E=12132171,C=20)
	CT(40959)=(S=12132172,E=12132175,C=24)
	CT(40960)=(S=12132176,E=12132179,C=58)
	CT(40961)=(S=12132180,E=12132183,C=31)
	CT(40962)=(S=12132184,E=12132187,C=58)
	CT(40963)=(S=12132188,E=12132191,C=55)
	CT(40964)=(S=12132192,E=12132195,C=63)
	CT(40965)=(S=12132196,E=12132199,C=34)
	CT(40966)=(S=12132200,E=12132203,C=25)
	CT(40967)=(S=12132204,E=12132215,C=18)
	CT(40968)=(S=12132216,E=12132219,C=51)
	CT(40969)=(S=12132220,E=12132223,C=26)
	CT(40970)=(S=12132224,E=12132227,C=36)
	CT(40971)=(S=12132228,E=12132231,C=18)
	CT(40972)=(S=12132232,E=12132235,C=47)
	CT(40973)=(S=12132236,E=12132239,C=19)
	CT(40974)=(S=12132240,E=12132243,C=36)
	CT(40975)=(S=12132244,E=12132247,C=12)
	CT(40976)=(S=12132248,E=12132251,C=14)
	CT(40977)=(S=12132252,E=12132255,C=21)
	CT(40978)=(S=12132256,E=12132263,C=18)
	CT(40979)=(S=12132264,E=12132267,C=14)
	CT(40980)=(S=12132268,E=12132271,C=36)
	CT(40981)=(S=12132272,E=12132275,C=24)
	CT(40982)=(S=12132276,E=12132279,C=34)
	CT(40983)=(S=12132280,E=12132283,C=58)
	CT(40984)=(S=12132284,E=12132287,C=16)
	CT(40985)=(S=12132288,E=12132291,C=18)
	CT(40986)=(S=12132292,E=12132295,C=19)
	CT(40987)=(S=12132296,E=12132299,C=36)
	CT(40988)=(S=12132300,E=12132303,C=14)
	CT(40989)=(S=12132304,E=12132307,C=28)
	CT(40990)=(S=12132308,E=12132311,C=42)
	CT(40991)=(S=12132312,E=12132315,C=58)
	CT(40992)=(S=12132316,E=12132319,C=64)
	CT(40993)=(S=12132320,E=12132323,C=14)
	CT(40994)=(S=12132324,E=12132327,C=36)
	CT(40995)=(S=12132328,E=12132331,C=14)
	CT(40996)=(S=12132332,E=12132335,C=25)
	CT(40997)=(S=12132336,E=12132339,C=55)
	CT(40998)=(S=12132340,E=12132343,C=36)
	CT(40999)=(S=12132344,E=12132347,C=34)
	CT(41000)=(S=12132348,E=12132351,C=14)
	CT(41001)=(S=12132352,E=12132355,C=53)
	CT(41002)=(S=12132356,E=12132359,C=58)
	CT(41003)=(S=12132360,E=12132363,C=19)
	CT(41004)=(S=12132364,E=12132367,C=37)
	CT(41005)=(S=12132368,E=12132371,C=25)
	CT(41006)=(S=12132372,E=12132375,C=96)
	CT(41007)=(S=12132376,E=12132379,C=38)
	CT(41008)=(S=12132380,E=12132383,C=25)
	CT(41009)=(S=12132384,E=12132387,C=28)
	CT(41010)=(S=12132388,E=12132391,C=21)
	CT(41011)=(S=12132392,E=12132395,C=14)
	CT(41012)=(S=12132396,E=12132399,C=41)
	CT(41013)=(S=12132400,E=12132403,C=58)
	CT(41014)=(S=12132404,E=12132407,C=14)
	CT(41015)=(S=12132408,E=12132411,C=18)
	CT(41016)=(S=12132412,E=12132415,C=31)
	CT(41017)=(S=12132416,E=12132419,C=33)
	CT(41018)=(S=12132420,E=12132423,C=18)
	CT(41019)=(S=12132424,E=12132431,C=14)
	CT(41020)=(S=12132432,E=12132435,C=28)
	CT(41021)=(S=12132436,E=12132439,C=18)
	CT(41022)=(S=12132440,E=12132443,C=27)
	CT(41023)=(S=12132444,E=12132447,C=63)
	CT(41024)=(S=12132448,E=12132455,C=12)
	CT(41025)=(S=12132456,E=12132463,C=14)
	CT(41026)=(S=12132464,E=12132467,C=17)
	CT(41027)=(S=12132468,E=12132471,C=28)
	CT(41028)=(S=12132472,E=12132475,C=18)
	CT(41029)=(S=12132476,E=12132479,C=34)
	CT(41030)=(S=12132480,E=12132483,C=26)
	CT(41031)=(S=12132484,E=12132487,C=18)
	CT(41032)=(S=12132488,E=12132491,C=25)
	CT(41033)=(S=12132492,E=12132495,C=36)
	CT(41034)=(S=12132496,E=12132499,C=58)
	CT(41035)=(S=12132500,E=12132503,C=35)
	CT(41036)=(S=12132504,E=12132507,C=63)
	CT(41037)=(S=12132508,E=12132511,C=58)
	CT(41038)=(S=12132512,E=12132515,C=33)
	CT(41039)=(S=12132516,E=12132519,C=18)
	CT(41040)=(S=12132520,E=12132523,C=49)
	CT(41041)=(S=12132524,E=12132527,C=59)
	CT(41042)=(S=12132528,E=12132531,C=17)
	CT(41043)=(S=12132532,E=12132535,C=33)
	CT(41044)=(S=12132536,E=12132539,C=18)
	CT(41045)=(S=12132540,E=12132543,C=21)
	CT(41046)=(S=12132544,E=12132547,C=13)
	CT(41047)=(S=12132548,E=12132551,C=19)
	CT(41048)=(S=12132552,E=12132555,C=14)
	CT(41049)=(S=12132556,E=12132559,C=19)
	CT(41050)=(S=12132560,E=12132563,C=12)
	CT(41051)=(S=12132564,E=12132567,C=34)
	CT(41052)=(S=12132568,E=12132571,C=12)
	CT(41053)=(S=12132572,E=12132575,C=34)
	CT(41054)=(S=12132576,E=12132579,C=68)
	CT(41055)=(S=12132580,E=12132583,C=22)
	CT(41056)=(S=12132584,E=12132587,C=36)
	CT(41057)=(S=12132588,E=12132591,C=70)
	CT(41058)=(S=12132592,E=12132595,C=36)
	CT(41059)=(S=12132596,E=12132599,C=28)
	CT(41060)=(S=12132600,E=12132603,C=18)
	CT(41061)=(S=12132604,E=12132607,C=63)
	CT(41062)=(S=12132608,E=12132611,C=188)
	CT(41063)=(S=12132612,E=12132615,C=13)
	CT(41064)=(S=12132616,E=12132619,C=42)
	CT(41065)=(S=12132620,E=12132623,C=12)
	CT(41066)=(S=12132624,E=12132627,C=21)
	CT(41067)=(S=12132628,E=12132631,C=34)
	CT(41068)=(S=12132632,E=12132635,C=12)
	CT(41069)=(S=12132636,E=12132639,C=61)
	CT(41070)=(S=12132640,E=12132643,C=96)
	CT(41071)=(S=12132644,E=12132647,C=58)
	CT(41072)=(S=12132648,E=12132651,C=12)
	CT(41073)=(S=12132652,E=12132655,C=35)
	CT(41074)=(S=12132656,E=12132659,C=52)
	CT(41075)=(S=12132660,E=12132663,C=50)
	CT(41076)=(S=12132664,E=12132667,C=15)
	CT(41077)=(S=12132668,E=12132671,C=37)
	CT(41078)=(S=12132672,E=12132675,C=25)
	CT(41079)=(S=12132676,E=12132679,C=36)
	CT(41080)=(S=12132680,E=12132683,C=47)
	CT(41081)=(S=12132684,E=12132687,C=14)
	CT(41082)=(S=12132688,E=12132691,C=50)
	CT(41083)=(S=12132692,E=12132695,C=14)
	CT(41084)=(S=12132696,E=12132699,C=12)
	CT(41085)=(S=12132700,E=12132703,C=14)
	CT(41086)=(S=12132704,E=12132707,C=47)
	CT(41087)=(S=12132708,E=12132711,C=36)
	CT(41088)=(S=12132712,E=12132715,C=53)
	CT(41089)=(S=12132716,E=12132719,C=37)
	CT(41090)=(S=12132720,E=12132723,C=39)
	CT(41091)=(S=12132724,E=12132727,C=36)
	CT(41092)=(S=12132728,E=12132731,C=18)
	CT(41093)=(S=12132732,E=12132735,C=101)
	CT(41094)=(S=12132736,E=12132739,C=37)
	CT(41095)=(S=12132740,E=12132743,C=25)
	CT(41096)=(S=12132744,E=12132747,C=33)
	CT(41097)=(S=12132748,E=12132751,C=31)
	CT(41098)=(S=12132752,E=12132755,C=50)
	CT(41099)=(S=12132756,E=12132759,C=14)
	CT(41100)=(S=12132760,E=12132763,C=36)
	CT(41101)=(S=12132764,E=12132767,C=33)
	CT(41102)=(S=12132768,E=12132771,C=18)
	CT(41103)=(S=12132772,E=12132775,C=36)
	CT(41104)=(S=12132776,E=12132779,C=43)
	CT(41105)=(S=12132780,E=12132783,C=23)
	CT(41106)=(S=12132784,E=12132787,C=104)
	CT(41107)=(S=12132788,E=12132791,C=28)
	CT(41108)=(S=12132792,E=12132795,C=14)
	CT(41109)=(S=12132796,E=12132799,C=28)
	CT(41110)=(S=12132800,E=12132803,C=14)
	CT(41111)=(S=12132804,E=12132811,C=18)
	CT(41112)=(S=12132812,E=12132815,C=28)
	CT(41113)=(S=12132816,E=12132819,C=14)
	CT(41114)=(S=12132820,E=12132823,C=12)
	CT(41115)=(S=12132824,E=12132827,C=28)
	CT(41116)=(S=12132828,E=12132831,C=36)
	CT(41117)=(S=12132832,E=12132835,C=34)
	CT(41118)=(S=12132836,E=12132839,C=18)
	CT(41119)=(S=12132840,E=12132843,C=37)
	CT(41120)=(S=12132844,E=12132847,C=18)
	CT(41121)=(S=12132848,E=12132851,C=14)
	CT(41122)=(S=12132852,E=12132855,C=18)
	CT(41123)=(S=12132856,E=12132859,C=34)
	CT(41124)=(S=12132860,E=12132863,C=42)
	CT(41125)=(S=12132864,E=12132867,C=28)
	CT(41126)=(S=12132868,E=12132871,C=27)
	CT(41127)=(S=12132872,E=12132875,C=14)
	CT(41128)=(S=12132876,E=12132879,C=24)
	CT(41129)=(S=12132880,E=12132883,C=35)
	CT(41130)=(S=12132884,E=12132887,C=18)
	CT(41131)=(S=12132888,E=12132891,C=60)
	CT(41132)=(S=12132892,E=12132895,C=58)
	CT(41133)=(S=12132896,E=12132899,C=12)
	CT(41134)=(S=12132900,E=12132903,C=15)
	CT(41135)=(S=12132904,E=12132907,C=36)
	CT(41136)=(S=12132908,E=12132911,C=14)
	CT(41137)=(S=12132912,E=12132915,C=58)
	CT(41138)=(S=12132916,E=12132919,C=36)
	CT(41139)=(S=12132920,E=12132927,C=14)
	CT(41140)=(S=12132928,E=12132931,C=36)
	CT(41141)=(S=12132932,E=12132935,C=60)
	CT(41142)=(S=12132936,E=12132939,C=41)
	CT(41143)=(S=12132940,E=12132943,C=58)
	CT(41144)=(S=12132944,E=12132947,C=14)
	CT(41145)=(S=12132948,E=12132951,C=15)
	CT(41146)=(S=12132952,E=12132955,C=28)
	CT(41147)=(S=12132956,E=12132959,C=46)
	CT(41148)=(S=12132960,E=12132963,C=14)
	CT(41149)=(S=12132964,E=12132967,C=41)
	CT(41150)=(S=12132968,E=12132975,C=63)
	CT(41151)=(S=12132976,E=12132979,C=28)
	CT(41152)=(S=12132980,E=12132983,C=14)
	CT(41153)=(S=12132984,E=12132987,C=63)
	CT(41154)=(S=12132988,E=12132991,C=14)
	CT(41155)=(S=12132992,E=12132995,C=37)
	CT(41156)=(S=12132996,E=12132999,C=24)
	CT(41157)=(S=12133000,E=12133003,C=15)
	CT(41158)=(S=12133004,E=12133007,C=12)
	CT(41159)=(S=12133008,E=12133011,C=36)
	CT(41160)=(S=12133012,E=12133015,C=56)
	CT(41161)=(S=12133016,E=12133019,C=18)
	CT(41162)=(S=12133020,E=12133023,C=36)
	CT(41163)=(S=12133024,E=12133027,C=26)
	CT(41164)=(S=12133028,E=12133031,C=14)
	CT(41165)=(S=12133032,E=12133035,C=36)
	CT(41166)=(S=12133036,E=12133039,C=14)
	CT(41167)=(S=12133040,E=12133043,C=36)
	CT(41168)=(S=12133044,E=12133047,C=18)
	CT(41169)=(S=12133048,E=12133051,C=28)
	CT(41170)=(S=12133052,E=12133055,C=63)
	CT(41171)=(S=12133056,E=12133059,C=25)
	CT(41172)=(S=12133060,E=12133063,C=42)
	CT(41173)=(S=12133064,E=12133067,C=36)
	CT(41174)=(S=12133068,E=12133071,C=15)
	CT(41175)=(S=12133072,E=12133075,C=14)
	CT(41176)=(S=12133076,E=12133079,C=28)
	CT(41177)=(S=12133080,E=12133087,C=36)
	CT(41178)=(S=12133088,E=12133091,C=31)
	CT(41179)=(S=12133092,E=12133095,C=104)
	CT(41180)=(S=12133096,E=12133099,C=14)
	CT(41181)=(S=12133100,E=12133103,C=58)
	CT(41182)=(S=12133104,E=12133111,C=18)
	CT(41183)=(S=12133112,E=12133119,C=14)
	CT(41184)=(S=12133120,E=12133127,C=18)
	CT(41185)=(S=12133128,E=12133131,C=31)
	CT(41186)=(S=12133132,E=12133135,C=28)
	CT(41187)=(S=12133136,E=12133139,C=20)
	CT(41188)=(S=12133140,E=12133143,C=37)
	CT(41189)=(S=12133144,E=12133147,C=14)
	CT(41190)=(S=12133148,E=12133151,C=34)
	CT(41191)=(S=12133152,E=12133155,C=36)
	CT(41192)=(S=12133156,E=12133167,C=18)
	CT(41193)=(S=12133168,E=12133171,C=14)
	CT(41194)=(S=12133172,E=12133175,C=49)
	CT(41195)=(S=12133176,E=12133179,C=14)
	CT(41196)=(S=12133180,E=12133183,C=34)
	CT(41197)=(S=12133184,E=12133187,C=28)
	CT(41198)=(S=12133188,E=12133191,C=60)
	CT(41199)=(S=12133192,E=12133199,C=14)
	CT(41200)=(S=12133200,E=12133203,C=27)
	CT(41201)=(S=12133204,E=12133207,C=19)
	CT(41202)=(S=12133208,E=12133211,C=14)
	CT(41203)=(S=12133212,E=12133215,C=25)
	CT(41204)=(S=12133216,E=12133219,C=36)
	CT(41205)=(S=12133220,E=12133223,C=31)
	CT(41206)=(S=12133224,E=12133227,C=16)
	CT(41207)=(S=12133228,E=12133231,C=28)
	CT(41208)=(S=12133232,E=12133235,C=36)
	CT(41209)=(S=12133236,E=12133243,C=18)
	CT(41210)=(S=12133244,E=12133247,C=14)
	CT(41211)=(S=12133248,E=12133251,C=18)
	CT(41212)=(S=12133252,E=12133255,C=33)
	CT(41213)=(S=12133256,E=12133259,C=36)
	CT(41214)=(S=12133260,E=12133263,C=25)
	CT(41215)=(S=12133264,E=12133267,C=28)
	CT(41216)=(S=12133268,E=12133271,C=14)
	CT(41217)=(S=12133272,E=12133275,C=42)
	CT(41218)=(S=12133276,E=12133279,C=59)
	CT(41219)=(S=12133280,E=12133283,C=18)
	CT(41220)=(S=12133284,E=12133287,C=49)
	CT(41221)=(S=12133288,E=12133291,C=18)
	CT(41222)=(S=12133292,E=12133295,C=49)
	CT(41223)=(S=12133296,E=12133299,C=51)
	CT(41224)=(S=12133300,E=12133303,C=42)
	CT(41225)=(S=12133304,E=12133307,C=27)
	CT(41226)=(S=12133308,E=12133311,C=25)
	CT(41227)=(S=12133312,E=12133315,C=18)
	CT(41228)=(S=12133316,E=12133319,C=58)
	CT(41229)=(S=12133320,E=12133323,C=27)
	CT(41230)=(S=12133324,E=12133327,C=12)
	CT(41231)=(S=12133328,E=12133331,C=28)
	CT(41232)=(S=12133332,E=12133335,C=27)
	CT(41233)=(S=12133336,E=12133339,C=28)
	CT(41234)=(S=12133340,E=12133343,C=18)
	CT(41235)=(S=12133344,E=12133347,C=34)
	CT(41236)=(S=12133348,E=12133351,C=14)
	CT(41237)=(S=12133352,E=12133355,C=28)
	CT(41238)=(S=12133356,E=12133359,C=19)
	CT(41239)=(S=12133360,E=12133363,C=28)
	CT(41240)=(S=12133364,E=12133367,C=48)
	CT(41241)=(S=12133368,E=12133371,C=14)
	CT(41242)=(S=12133372,E=12133375,C=34)
	CT(41243)=(S=12133376,E=12133379,C=25)
	CT(41244)=(S=12133380,E=12133383,C=49)
	CT(41245)=(S=12133384,E=12133387,C=14)
	CT(41246)=(S=12133388,E=12133391,C=12)
	CT(41247)=(S=12133392,E=12133395,C=42)
	CT(41248)=(S=12133396,E=12133399,C=13)
	CT(41249)=(S=12133400,E=12133403,C=14)
	CT(41250)=(S=12133404,E=12133407,C=27)
	CT(41251)=(S=12133408,E=12133411,C=14)
	CT(41252)=(S=12133412,E=12133415,C=66)
	CT(41253)=(S=12133416,E=12133419,C=36)
	CT(41254)=(S=12133420,E=12133423,C=28)
	CT(41255)=(S=12133424,E=12133427,C=27)
	CT(41256)=(S=12133428,E=12133431,C=15)
	CT(41257)=(S=12133432,E=12133435,C=31)
	CT(41258)=(S=12133436,E=12133439,C=18)
	CT(41259)=(S=12133440,E=12133443,C=15)
	CT(41260)=(S=12133444,E=12133447,C=59)
	CT(41261)=(S=12133448,E=12133451,C=15)
	CT(41262)=(S=12133452,E=12133455,C=14)
	CT(41263)=(S=12133456,E=12133459,C=44)
	CT(41264)=(S=12133460,E=12133463,C=59)
	CT(41265)=(S=12133464,E=12133467,C=104)
	CT(41266)=(S=12133468,E=12133471,C=38)
	CT(41267)=(S=12133472,E=12133475,C=47)
	CT(41268)=(S=12133476,E=12133479,C=33)
	CT(41269)=(S=12133480,E=12133483,C=18)
	CT(41270)=(S=12133484,E=12133487,C=14)
	CT(41271)=(S=12133488,E=12133491,C=36)
	CT(41272)=(S=12133492,E=12133499,C=28)
	CT(41273)=(S=12133500,E=12133503,C=27)
	CT(41274)=(S=12133504,E=12133507,C=15)
	CT(41275)=(S=12133508,E=12133511,C=36)
	CT(41276)=(S=12133512,E=12133515,C=28)
	CT(41277)=(S=12133516,E=12133519,C=27)
	CT(41278)=(S=12133520,E=12133523,C=25)
	CT(41279)=(S=12133524,E=12133527,C=47)
	CT(41280)=(S=12133528,E=12133531,C=58)
	CT(41281)=(S=12133532,E=12133535,C=18)
	CT(41282)=(S=12133536,E=12133539,C=33)
	CT(41283)=(S=12133540,E=12133543,C=49)
	CT(41284)=(S=12133544,E=12133547,C=58)
	CT(41285)=(S=12133548,E=12133551,C=18)
	CT(41286)=(S=12133552,E=12133555,C=101)
	CT(41287)=(S=12133556,E=12133559,C=70)
	CT(41288)=(S=12133560,E=12133563,C=24)
	CT(41289)=(S=12133564,E=12133567,C=31)
	CT(41290)=(S=12133568,E=12133571,C=26)
	CT(41291)=(S=12133572,E=12133575,C=31)
	CT(41292)=(S=12133576,E=12133579,C=41)
	CT(41293)=(S=12133580,E=12133587,C=25)
	CT(41294)=(S=12133588,E=12133591,C=27)
	CT(41295)=(S=12133592,E=12133595,C=12)
	CT(41296)=(S=12133596,E=12133603,C=34)
	CT(41297)=(S=12133604,E=12133607,C=18)
	CT(41298)=(S=12133608,E=12133611,C=22)
	CT(41299)=(S=12133612,E=12133615,C=72)
	CT(41300)=(S=12133616,E=12133619,C=27)
	CT(41301)=(S=12133620,E=12133623,C=40)
	CT(41302)=(S=12133624,E=12133627,C=14)
	CT(41303)=(S=12133628,E=12133631,C=38)
	CT(41304)=(S=12133632,E=12133635,C=34)
	CT(41305)=(S=12133636,E=12133639,C=36)
	CT(41306)=(S=12133640,E=12133643,C=49)
	CT(41307)=(S=12133644,E=12133647,C=43)
	CT(41308)=(S=12133648,E=12133651,C=28)
	CT(41309)=(S=12133652,E=12133655,C=36)
	CT(41310)=(S=12133656,E=12133659,C=46)
	CT(41311)=(S=12133660,E=12133663,C=12)
	CT(41312)=(S=12133664,E=12133667,C=102)
	CT(41313)=(S=12133668,E=12133671,C=24)
	CT(41314)=(S=12133672,E=12133675,C=12)
	CT(41315)=(S=12133676,E=12133679,C=15)
	CT(41316)=(S=12133680,E=12133683,C=18)
	CT(41317)=(S=12133684,E=12133687,C=26)
	CT(41318)=(S=12133688,E=12133695,C=18)
	CT(41319)=(S=12133696,E=12133703,C=36)
	CT(41320)=(S=12133704,E=12133707,C=34)
	CT(41321)=(S=12133708,E=12133711,C=14)
	CT(41322)=(S=12133712,E=12133715,C=15)
	CT(41323)=(S=12133716,E=12133719,C=24)
	CT(41324)=(S=12133720,E=12133723,C=35)
	CT(41325)=(S=12133724,E=12133727,C=15)
	CT(41326)=(S=12133728,E=12133731,C=94)
	CT(41327)=(S=12133732,E=12133735,C=51)
	CT(41328)=(S=12133736,E=12133739,C=42)
	CT(41329)=(S=12133740,E=12133743,C=94)
	CT(41330)=(S=12133744,E=12133747,C=58)
	CT(41331)=(S=12133748,E=12133751,C=15)
	CT(41332)=(S=12133752,E=12133755,C=14)
	CT(41333)=(S=12133756,E=12133759,C=36)
	CT(41334)=(S=12133760,E=12133763,C=18)
	CT(41335)=(S=12133764,E=12133767,C=47)
	CT(41336)=(S=12133768,E=12133771,C=28)
	CT(41337)=(S=12133772,E=12133775,C=19)
	CT(41338)=(S=12133776,E=12133779,C=28)
	CT(41339)=(S=12133780,E=12133783,C=17)
	CT(41340)=(S=12133784,E=12133787,C=28)
	CT(41341)=(S=12133788,E=12133791,C=12)
	CT(41342)=(S=12133792,E=12133795,C=35)
	CT(41343)=(S=12133796,E=12133799,C=18)
	CT(41344)=(S=12133800,E=12133803,C=46)
	CT(41345)=(S=12133804,E=12133811,C=14)
	CT(41346)=(S=12133812,E=12133815,C=15)
	CT(41347)=(S=12133816,E=12133819,C=32)
	CT(41348)=(S=12133820,E=12133823,C=18)
	CT(41349)=(S=12133824,E=12133827,C=31)
	CT(41350)=(S=12133828,E=12133831,C=14)
	CT(41351)=(S=12133832,E=12133835,C=42)
	CT(41352)=(S=12133836,E=12133839,C=15)
	CT(41353)=(S=12133840,E=12133843,C=21)
	CT(41354)=(S=12133844,E=12133847,C=25)
	CT(41355)=(S=12133848,E=12133851,C=14)
	CT(41356)=(S=12133852,E=12133855,C=12)
	CT(41357)=(S=12133856,E=12133863,C=25)
	CT(41358)=(S=12133864,E=12133867,C=48)
	CT(41359)=(S=12133868,E=12133871,C=14)
	CT(41360)=(S=12133872,E=12133875,C=71)
	CT(41361)=(S=12133876,E=12133879,C=36)
	CT(41362)=(S=12133880,E=12133883,C=28)
	CT(41363)=(S=12133884,E=12133887,C=34)
	CT(41364)=(S=12133888,E=12133891,C=47)
	CT(41365)=(S=12133892,E=12133895,C=42)
	CT(41366)=(S=12133896,E=12133899,C=28)
	CT(41367)=(S=12133900,E=12133903,C=36)
	CT(41368)=(S=12133904,E=12133907,C=18)
	CT(41369)=(S=12133908,E=12133911,C=12)
	CT(41370)=(S=12133912,E=12133915,C=24)
	CT(41371)=(S=12133916,E=12133919,C=39)
	CT(41372)=(S=12133920,E=12133927,C=14)
	CT(41373)=(S=12133928,E=12133931,C=28)
	CT(41374)=(S=12133932,E=12133935,C=14)
	CT(41375)=(S=12133936,E=12133943,C=28)
	CT(41376)=(S=12133944,E=12133947,C=60)
	CT(41377)=(S=12133948,E=12133951,C=63)
	CT(41378)=(S=12133952,E=12133955,C=34)
	CT(41379)=(S=12133956,E=12133959,C=27)
	CT(41380)=(S=12133960,E=12133963,C=31)
	CT(41381)=(S=12133964,E=12133967,C=28)
	CT(41382)=(S=12133968,E=12133971,C=14)
	CT(41383)=(S=12133972,E=12133975,C=18)
	CT(41384)=(S=12133976,E=12133979,C=36)
	CT(41385)=(S=12133980,E=12133983,C=12)
	CT(41386)=(S=12133984,E=12133987,C=14)
	CT(41387)=(S=12133988,E=12133991,C=36)
	CT(41388)=(S=12133992,E=12133995,C=14)
	CT(41389)=(S=12133996,E=12133999,C=33)
	CT(41390)=(S=12134000,E=12134003,C=14)
	CT(41391)=(S=12134004,E=12134007,C=28)
	CT(41392)=(S=12134008,E=12134011,C=27)
	CT(41393)=(S=12134012,E=12134015,C=50)
	CT(41394)=(S=12134016,E=12134019,C=25)
	CT(41395)=(S=12134020,E=12134023,C=14)
	CT(41396)=(S=12134024,E=12134027,C=15)
	CT(41397)=(S=12134028,E=12134031,C=21)
	CT(41398)=(S=12134032,E=12134035,C=59)
	CT(41399)=(S=12134036,E=12134039,C=14)
	CT(41400)=(S=12134040,E=12134047,C=36)
	CT(41401)=(S=12134048,E=12134051,C=18)
	CT(41402)=(S=12134052,E=12134055,C=44)
	CT(41403)=(S=12134056,E=12134059,C=42)
	CT(41404)=(S=12134060,E=12134063,C=24)
	CT(41405)=(S=12134064,E=12134067,C=57)
	CT(41406)=(S=12134068,E=12134071,C=24)
	CT(41407)=(S=12134072,E=12134075,C=102)
	CT(41408)=(S=12134076,E=12134079,C=33)
	CT(41409)=(S=12134080,E=12134083,C=96)
	CT(41410)=(S=12134084,E=12134087,C=12)
	CT(41411)=(S=12134088,E=12134091,C=17)
	CT(41412)=(S=12134092,E=12134095,C=18)
	CT(41413)=(S=12134096,E=12134099,C=31)
	CT(41414)=(S=12134100,E=12134103,C=35)
	CT(41415)=(S=12134104,E=12134107,C=31)
	CT(41416)=(S=12134108,E=12134115,C=58)
	CT(41417)=(S=12134116,E=12134119,C=14)
	CT(41418)=(S=12134120,E=12134123,C=36)
	CT(41419)=(S=12134124,E=12134127,C=27)
	CT(41420)=(S=12134132,E=12134135,C=14)
	CT(41421)=(S=12134136,E=12134139,C=58)
	CT(41422)=(S=12134140,E=12134143,C=15)
	CT(41423)=(S=12134144,E=12134147,C=59)
	CT(41424)=(S=12134148,E=12134151,C=18)
	CT(41425)=(S=12134152,E=12134155,C=34)
	CT(41426)=(S=12134156,E=12134159,C=28)
	CT(41427)=(S=12134160,E=12134163,C=18)
	CT(41428)=(S=12134164,E=12134167,C=28)
	CT(41429)=(S=12134168,E=12134171,C=15)
	CT(41430)=(S=12134172,E=12134175,C=31)
	CT(41431)=(S=12134176,E=12134179,C=34)
	CT(41432)=(S=12134180,E=12134183,C=23)
	CT(41433)=(S=12134184,E=12134187,C=25)
	CT(41434)=(S=12134188,E=12134191,C=36)
	CT(41435)=(S=12134192,E=12134195,C=28)
	CT(41436)=(S=12134196,E=12134199,C=36)
	CT(41437)=(S=12134200,E=12134203,C=28)
	CT(41438)=(S=12134204,E=12134207,C=25)
	CT(41439)=(S=12134208,E=12134211,C=28)
	CT(41440)=(S=12134212,E=12134215,C=25)
	CT(41441)=(S=12134216,E=12134219,C=31)
	CT(41442)=(S=12134220,E=12134223,C=35)
	CT(41443)=(S=12134224,E=12134227,C=18)
	CT(41444)=(S=12134228,E=12134235,C=28)
	CT(41445)=(S=12134236,E=12134239,C=34)
	CT(41446)=(S=12134240,E=12134243,C=25)
	CT(41447)=(S=12134244,E=12134247,C=15)
	CT(41448)=(S=12134248,E=12134255,C=28)
	CT(41449)=(S=12134256,E=12134263,C=18)
	CT(41450)=(S=12134264,E=12134267,C=36)
	CT(41451)=(S=12134268,E=12134271,C=19)
	CT(41452)=(S=12134272,E=12134275,C=47)
	CT(41453)=(S=12134276,E=12134279,C=18)
	CT(41454)=(S=12134280,E=12134283,C=19)
	CT(41455)=(S=12134284,E=12134287,C=12)
	CT(41456)=(S=12134288,E=12134291,C=19)
	CT(41457)=(S=12134292,E=12134295,C=18)
	CT(41458)=(S=12134296,E=12134299,C=49)
	CT(41459)=(S=12134300,E=12134303,C=12)
	CT(41460)=(S=12134304,E=12134307,C=58)
	CT(41461)=(S=12134308,E=12134311,C=63)
	CT(41462)=(S=12134312,E=12134315,C=12)
	CT(41463)=(S=12134316,E=12134319,C=42)
	CT(41464)=(S=12134320,E=12134323,C=28)
	CT(41465)=(S=12134324,E=12134327,C=26)
	CT(41466)=(S=12134328,E=12134331,C=47)
	CT(41467)=(S=12134332,E=12134335,C=15)
	CT(41468)=(S=12134336,E=12134339,C=18)
	CT(41469)=(S=12134340,E=12134343,C=31)
	CT(41470)=(S=12134344,E=12134347,C=27)
	CT(41471)=(S=12134348,E=12134351,C=18)
	CT(41472)=(S=12134352,E=12134355,C=21)
	CT(41473)=(S=12134356,E=12134359,C=27)
	CT(41474)=(S=12134360,E=12134363,C=12)
	CT(41475)=(S=12134364,E=12134367,C=28)
	CT(41476)=(S=12134368,E=12134371,C=31)
	CT(41477)=(S=12134372,E=12134375,C=34)
	CT(41478)=(S=12134376,E=12134383,C=14)
	CT(41479)=(S=12134384,E=12134387,C=15)
	CT(41480)=(S=12134388,E=12134395,C=14)
	CT(41481)=(S=12134396,E=12134399,C=28)
	CT(41482)=(S=12134400,E=12134407,C=18)
	CT(41483)=(S=12134408,E=12134411,C=14)
	CT(41484)=(S=12134412,E=12134419,C=15)
	CT(41485)=(S=12134420,E=12134423,C=50)
	CT(41486)=(S=12134424,E=12134427,C=14)
	CT(41487)=(S=12134428,E=12134431,C=18)
	CT(41488)=(S=12134432,E=12134435,C=41)
	CT(41489)=(S=12134436,E=12134439,C=25)
	CT(41490)=(S=12134440,E=12134443,C=18)
	CT(41491)=(S=12134444,E=12134447,C=24)
	CT(41492)=(S=12134448,E=12134451,C=31)
	CT(41493)=(S=12134452,E=12134455,C=42)
	CT(41494)=(S=12134456,E=12134459,C=36)
	CT(41495)=(S=12134460,E=12134463,C=70)
	CT(41496)=(S=12134464,E=12134467,C=63)
	CT(41497)=(S=12134468,E=12134471,C=49)
	CT(41498)=(S=12134472,E=12134475,C=37)
	CT(41499)=(S=12134476,E=12134479,C=18)
	CT(41500)=(S=12134480,E=12134483,C=33)
	CT(41501)=(S=12134484,E=12134487,C=37)
	CT(41502)=(S=12134488,E=12134491,C=47)
	CT(41503)=(S=12134492,E=12134499,C=36)
	CT(41504)=(S=12134500,E=12134503,C=12)
	CT(41505)=(S=12134504,E=12134507,C=18)
	CT(41506)=(S=12134508,E=12134511,C=36)
	CT(41507)=(S=12134512,E=12134515,C=58)
	CT(41508)=(S=12134516,E=12134519,C=28)
	CT(41509)=(S=12134520,E=12134523,C=48)
	CT(41510)=(S=12134524,E=12134527,C=14)
	CT(41511)=(S=12134528,E=12134531,C=18)
	CT(41512)=(S=12134532,E=12134535,C=28)
	CT(41513)=(S=12134536,E=12134539,C=50)
	CT(41514)=(S=12134540,E=12134543,C=14)
	CT(41515)=(S=12134544,E=12134551,C=25)
	CT(41516)=(S=12134552,E=12134555,C=18)
	CT(41517)=(S=12134556,E=12134559,C=41)
	CT(41518)=(S=12134560,E=12134563,C=28)
	CT(41519)=(S=12134564,E=12134567,C=51)
	CT(41520)=(S=12134568,E=12134571,C=42)
	CT(41521)=(S=12134572,E=12134575,C=28)
	CT(41522)=(S=12134576,E=12134579,C=41)
	CT(41523)=(S=12134584,E=12134587,C=19)
	CT(41524)=(S=12134588,E=12134591,C=102)
	CT(41525)=(S=12134592,E=12134595,C=43)
	CT(41526)=(S=12134596,E=12134599,C=58)
	CT(41527)=(S=12134600,E=12134603,C=47)
	CT(41528)=(S=12134604,E=12134607,C=14)
	CT(41529)=(S=12134608,E=12134611,C=39)
	CT(41530)=(S=12134612,E=12134615,C=15)
	CT(41531)=(S=12134616,E=12134619,C=34)
	CT(41532)=(S=12134620,E=12134623,C=15)
	CT(41533)=(S=12134624,E=12134627,C=26)
	CT(41534)=(S=12134628,E=12134631,C=58)
	CT(41535)=(S=12134632,E=12134635,C=13)
	CT(41536)=(S=12134636,E=12134639,C=55)
	CT(41537)=(S=12134640,E=12134643,C=26)
	CT(41538)=(S=12134648,E=12134651,C=28)
	CT(41539)=(S=12134652,E=12134655,C=36)
	CT(41540)=(S=12134656,E=12134659,C=26)
	CT(41541)=(S=12134660,E=12134663,C=12)
	CT(41542)=(S=12134664,E=12134667,C=14)
	CT(41543)=(S=12134668,E=12134671,C=28)
	CT(41544)=(S=12134672,E=12134675,C=27)
	CT(41545)=(S=12134676,E=12134679,C=31)
	CT(41546)=(S=12134680,E=12134683,C=14)
	CT(41547)=(S=12134684,E=12134687,C=12)
	CT(41548)=(S=12134688,E=12134691,C=15)
	CT(41549)=(S=12134692,E=12134695,C=36)
	CT(41550)=(S=12134696,E=12134699,C=18)
	CT(41551)=(S=12134700,E=12134703,C=14)
	CT(41552)=(S=12134704,E=12134707,C=36)
	CT(41553)=(S=12134708,E=12134711,C=33)
	CT(41554)=(S=12134712,E=12134719,C=28)
	CT(41555)=(S=12134720,E=12134723,C=27)
	CT(41556)=(S=12134724,E=12134727,C=58)
	CT(41557)=(S=12134728,E=12134731,C=15)
	CT(41558)=(S=12134732,E=12134735,C=34)
	CT(41559)=(S=12134736,E=12134739,C=58)
	CT(41560)=(S=12134740,E=12134743,C=30)
	CT(41561)=(S=12134744,E=12134747,C=19)
	CT(41562)=(S=12134748,E=12134751,C=58)
	CT(41563)=(S=12134752,E=12134755,C=25)
	CT(41564)=(S=12134756,E=12134759,C=18)
	CT(41565)=(S=12134760,E=12134767,C=28)
	CT(41566)=(S=12134768,E=12134771,C=18)
	CT(41567)=(S=12134772,E=12134775,C=21)
	CT(41568)=(S=12134776,E=12134779,C=18)
	CT(41569)=(S=12134780,E=12134783,C=36)
	CT(41570)=(S=12134784,E=12134787,C=58)
	CT(41571)=(S=12134788,E=12134791,C=34)
	CT(41572)=(S=12134792,E=12134803,C=36)
	CT(41573)=(S=12134804,E=12134807,C=19)
	CT(41574)=(S=12134808,E=12134811,C=12)
	CT(41575)=(S=12134812,E=12134815,C=40)
	CT(41576)=(S=12134816,E=12134819,C=18)
	CT(41577)=(S=12134820,E=12134823,C=15)
	CT(41578)=(S=12134824,E=12134827,C=18)
	CT(41579)=(S=12134828,E=12134831,C=19)
	CT(41580)=(S=12134832,E=12134835,C=52)
	CT(41581)=(S=12134836,E=12134839,C=12)
	CT(41582)=(S=12134840,E=12134843,C=18)
	CT(41583)=(S=12134844,E=12134847,C=14)
	CT(41584)=(S=12134848,E=12134851,C=31)
	CT(41585)=(S=12134852,E=12134855,C=18)
	CT(41586)=(S=12134856,E=12134859,C=41)
	CT(41587)=(S=12134860,E=12134863,C=18)
	CT(41588)=(S=12134864,E=12134867,C=32)
	CT(41589)=(S=12134868,E=12134871,C=15)
	CT(41590)=(S=12134872,E=12134875,C=101)
	CT(41591)=(S=12134876,E=12134891,C=14)
	CT(41592)=(S=12134892,E=12134895,C=49)
	CT(41593)=(S=12134896,E=12134899,C=19)
	CT(41594)=(S=12134900,E=12134903,C=25)
	CT(41595)=(S=12134904,E=12134907,C=12)
	CT(41596)=(S=12134908,E=12134911,C=15)
	CT(41597)=(S=12134912,E=12134915,C=14)
	CT(41598)=(S=12134916,E=12134919,C=71)
	CT(41599)=(S=12134920,E=12134923,C=36)
	CT(41600)=(S=12134924,E=12134927,C=18)
	CT(41601)=(S=12134928,E=12134931,C=14)
	CT(41602)=(S=12134932,E=12134935,C=25)
	CT(41603)=(S=12134936,E=12134939,C=26)
	CT(41604)=(S=12134940,E=12134943,C=12)
	CT(41605)=(S=12134944,E=12134947,C=32)
	CT(41606)=(S=12134948,E=12134951,C=54)
	CT(41607)=(S=12134952,E=12134955,C=31)
	CT(41608)=(S=12134956,E=12134959,C=58)
	CT(41609)=(S=12134960,E=12134963,C=18)
	CT(41610)=(S=12134964,E=12134967,C=14)
	CT(41611)=(S=12134968,E=12134971,C=36)
	CT(41612)=(S=12134972,E=12134975,C=18)
	CT(41613)=(S=12134976,E=12134979,C=14)
	CT(41614)=(S=12134980,E=12134987,C=36)
	CT(41615)=(S=12134988,E=12134995,C=18)
	CT(41616)=(S=12134996,E=12134999,C=36)
	CT(41617)=(S=12135000,E=12135007,C=18)
	CT(41618)=(S=12135008,E=12135011,C=103)
	CT(41619)=(S=12135012,E=12135015,C=13)
	CT(41620)=(S=12135016,E=12135019,C=25)
	CT(41621)=(S=12135020,E=12135023,C=18)
	CT(41622)=(S=12135024,E=12135031,C=12)
	CT(41623)=(S=12135032,E=12135035,C=25)
	CT(41624)=(S=12135036,E=12135039,C=18)
	CT(41625)=(S=12135040,E=12135043,C=31)
	CT(41626)=(S=12135044,E=12135047,C=12)
	CT(41627)=(S=12135048,E=12135051,C=19)
	CT(41628)=(S=12135052,E=12135055,C=16)
	CT(41629)=(S=12135056,E=12135059,C=18)
	CT(41630)=(S=12135060,E=12135067,C=14)
	CT(41631)=(S=12135068,E=12135071,C=18)
	CT(41632)=(S=12135072,E=12135075,C=15)
	CT(41633)=(S=12135076,E=12135079,C=18)
	CT(41634)=(S=12135080,E=12135083,C=27)
	CT(41635)=(S=12135084,E=12135087,C=37)
	CT(41636)=(S=12135088,E=12135091,C=12)
	CT(41637)=(S=12135092,E=12135095,C=18)
	CT(41638)=(S=12135096,E=12135099,C=58)
	CT(41639)=(S=12135100,E=12135103,C=12)
	CT(41640)=(S=12135104,E=12135107,C=35)
	CT(41641)=(S=12135108,E=12135115,C=36)
	CT(41642)=(S=12135116,E=12135119,C=13)
	CT(41643)=(S=12135120,E=12135123,C=22)
	CT(41644)=(S=12135124,E=12135127,C=26)
	CT(41645)=(S=12135128,E=12135131,C=70)
	CT(41646)=(S=12135132,E=12135135,C=14)
	CT(41647)=(S=12135136,E=12135139,C=26)
	CT(41648)=(S=12135140,E=12135143,C=33)
	CT(41649)=(S=12135144,E=12135147,C=12)
	CT(41650)=(S=12135148,E=12135151,C=14)
	CT(41651)=(S=12135152,E=12135155,C=25)
	CT(41652)=(S=12135156,E=12135163,C=34)
	CT(41653)=(S=12135164,E=12135167,C=28)
	CT(41654)=(S=12135168,E=12135171,C=49)
	CT(41655)=(S=12135172,E=12135175,C=28)
	CT(41656)=(S=12135176,E=12135179,C=18)
	CT(41657)=(S=12135180,E=12135183,C=42)
	CT(41658)=(S=12135184,E=12135187,C=15)
	CT(41659)=(S=12135188,E=12135191,C=36)
	CT(41660)=(S=12135192,E=12135199,C=28)
	CT(41661)=(S=12135200,E=12135203,C=14)
	CT(41662)=(S=12135204,E=12135207,C=12)
	CT(41663)=(S=12135208,E=12135211,C=14)
	CT(41664)=(S=12135212,E=12135215,C=12)
	CT(41665)=(S=12135216,E=12135219,C=14)
	CT(41666)=(S=12135220,E=12135223,C=13)
	CT(41667)=(S=12135224,E=12135227,C=51)
	CT(41668)=(S=12135228,E=12135231,C=12)
	CT(41669)=(S=12135232,E=12135235,C=36)
	CT(41670)=(S=12135236,E=12135239,C=12)
	CT(41671)=(S=12135240,E=12135243,C=25)
	CT(41672)=(S=12135244,E=12135247,C=14)
	CT(41673)=(S=12135248,E=12135251,C=36)
	CT(41674)=(S=12135252,E=12135255,C=18)
	CT(41675)=(S=12135256,E=12135259,C=47)
	CT(41676)=(S=12135260,E=12135263,C=18)
	CT(41677)=(S=12135264,E=12135267,C=14)
	CT(41678)=(S=12135268,E=12135271,C=18)
	CT(41679)=(S=12135272,E=12135275,C=55)
	CT(41680)=(S=12135276,E=12135279,C=14)
	CT(41681)=(S=12135280,E=12135283,C=71)
	CT(41682)=(S=12135284,E=12135287,C=28)
	CT(41683)=(S=12135288,E=12135291,C=47)
	CT(41684)=(S=12135292,E=12135295,C=49)
	CT(41685)=(S=12135296,E=12135299,C=14)
	CT(41686)=(S=12135300,E=12135303,C=33)
	CT(41687)=(S=12135304,E=12135307,C=58)
	CT(41688)=(S=12135308,E=12135311,C=24)
	CT(41689)=(S=12135312,E=12135315,C=14)
	CT(41690)=(S=12135316,E=12135319,C=15)
	CT(41691)=(S=12135320,E=12135323,C=49)
	CT(41692)=(S=12135324,E=12135327,C=14)
	CT(41693)=(S=12135328,E=12135331,C=58)
	CT(41694)=(S=12135332,E=12135335,C=15)
	CT(41695)=(S=12135336,E=12135339,C=28)
	CT(41696)=(S=12135340,E=12135343,C=50)
	CT(41697)=(S=12135344,E=12135347,C=28)
	CT(41698)=(S=12135348,E=12135351,C=25)
	CT(41699)=(S=12135352,E=12135355,C=14)
	CT(41700)=(S=12135356,E=12135359,C=41)
	CT(41701)=(S=12135360,E=12135363,C=63)
	CT(41702)=(S=12135364,E=12135367,C=18)
	CT(41703)=(S=12135368,E=12135371,C=34)
	CT(41704)=(S=12135372,E=12135375,C=50)
	CT(41705)=(S=12135376,E=12135379,C=14)
	CT(41706)=(S=12135380,E=12135383,C=36)
	CT(41707)=(S=12135384,E=12135387,C=14)
	CT(41708)=(S=12135388,E=12135391,C=33)
	CT(41709)=(S=12135392,E=12135395,C=31)
	CT(41710)=(S=12135396,E=12135399,C=37)
	CT(41711)=(S=12135400,E=12135403,C=63)
	CT(41712)=(S=12135404,E=12135407,C=31)
	CT(41713)=(S=12135408,E=12135411,C=12)
	CT(41714)=(S=12135412,E=12135415,C=34)
	CT(41715)=(S=12135416,E=12135419,C=31)
	CT(41716)=(S=12135420,E=12135423,C=25)
	CT(41717)=(S=12135424,E=12135427,C=18)
	CT(41718)=(S=12135428,E=12135431,C=28)
	CT(41719)=(S=12135432,E=12135439,C=18)
	CT(41720)=(S=12135440,E=12135443,C=25)
	CT(41721)=(S=12135444,E=12135447,C=34)
	CT(41722)=(S=12135448,E=12135455,C=25)
	CT(41723)=(S=12135456,E=12135459,C=28)
	CT(41724)=(S=12135460,E=12135463,C=26)
	CT(41725)=(S=12135464,E=12135471,C=12)
	CT(41726)=(S=12135472,E=12135479,C=42)
	CT(41727)=(S=12135480,E=12135483,C=70)
	CT(41728)=(S=12135484,E=12135487,C=25)
	CT(41729)=(S=12135488,E=12135491,C=26)
	CT(41730)=(S=12135492,E=12135495,C=18)
	CT(41731)=(S=12135496,E=12135499,C=44)
	CT(41732)=(S=12135500,E=12135503,C=14)
	CT(41733)=(S=12135504,E=12135507,C=34)
	CT(41734)=(S=12135508,E=12135511,C=31)
	CT(41735)=(S=12135512,E=12135515,C=12)
	CT(41736)=(S=12135516,E=12135519,C=36)
	CT(41737)=(S=12135520,E=12135523,C=27)
	CT(41738)=(S=12135524,E=12135527,C=26)
	CT(41739)=(S=12135528,E=12135535,C=28)
	CT(41740)=(S=12135536,E=12135539,C=26)
	CT(41741)=(S=12135540,E=12135543,C=51)
	CT(41742)=(S=12135544,E=12135547,C=18)
	CT(41743)=(S=12135548,E=12135551,C=51)
	CT(41744)=(S=12135552,E=12135555,C=36)
	CT(41745)=(S=12135556,E=12135559,C=28)
	CT(41746)=(S=12135560,E=12135563,C=36)
	CT(41747)=(S=12135564,E=12135567,C=12)
	CT(41748)=(S=12135568,E=12135571,C=22)
	CT(41749)=(S=12135572,E=12135575,C=28)
	CT(41750)=(S=12135576,E=12135579,C=39)
	CT(41751)=(S=12135580,E=12135583,C=56)
	CT(41752)=(S=12135584,E=12135587,C=36)
	CT(41753)=(S=12135588,E=12135591,C=18)
	CT(41754)=(S=12135592,E=12135595,C=36)
	CT(41755)=(S=12135596,E=12135599,C=58)
	CT(41756)=(S=12135600,E=12135603,C=28)
	CT(41757)=(S=12135604,E=12135607,C=15)
	CT(41758)=(S=12135608,E=12135611,C=19)
	CT(41759)=(S=12135612,E=12135615,C=33)
	CT(41760)=(S=12135616,E=12135619,C=37)
	CT(41761)=(S=12135620,E=12135623,C=34)
	CT(41762)=(S=12135624,E=12135627,C=28)
	CT(41763)=(S=12135628,E=12135635,C=42)
	CT(41764)=(S=12135636,E=12135639,C=15)
	CT(41765)=(S=12135640,E=12135643,C=58)
	CT(41766)=(S=12135644,E=12135647,C=15)
	CT(41767)=(S=12135648,E=12135651,C=36)
	CT(41768)=(S=12135652,E=12135655,C=66)
	CT(41769)=(S=12135656,E=12135659,C=25)
	CT(41770)=(S=12135660,E=12135663,C=18)
	CT(41771)=(S=12135664,E=12135667,C=48)
	CT(41772)=(S=12135668,E=12135671,C=36)
	CT(41773)=(S=12135672,E=12135683,C=14)
	CT(41774)=(S=12135684,E=12135687,C=63)
	CT(41775)=(S=12135688,E=12135691,C=13)
	CT(41776)=(S=12135692,E=12135695,C=38)
	CT(41777)=(S=12135696,E=12135699,C=28)
	CT(41778)=(S=12135700,E=12135703,C=24)
	CT(41779)=(S=12135704,E=12135707,C=14)
	CT(41780)=(S=12135708,E=12135711,C=36)
	CT(41781)=(S=12135712,E=12135715,C=12)
	CT(41782)=(S=12135716,E=12135719,C=27)
	CT(41783)=(S=12135720,E=12135723,C=100)
	CT(41784)=(S=12135724,E=12135727,C=36)
	CT(41785)=(S=12135728,E=12135731,C=24)
	CT(41786)=(S=12135732,E=12135735,C=49)
	CT(41787)=(S=12135736,E=12135739,C=33)
	CT(41788)=(S=12135740,E=12135743,C=18)
	CT(41789)=(S=12135744,E=12135747,C=51)
	CT(41790)=(S=12135748,E=12135751,C=100)
	CT(41791)=(S=12135752,E=12135755,C=25)
	CT(41792)=(S=12135756,E=12135759,C=48)
	CT(41793)=(S=12135760,E=12135763,C=18)
	CT(41794)=(S=12135764,E=12135767,C=19)
	CT(41795)=(S=12135768,E=12135771,C=28)
	CT(41796)=(S=12135772,E=12135775,C=18)
	CT(41797)=(S=12135776,E=12135779,C=14)
	CT(41798)=(S=12135780,E=12135783,C=18)
	CT(41799)=(S=12135784,E=12135787,C=63)
	CT(41800)=(S=12135792,E=12135795,C=28)
	CT(41801)=(S=12135796,E=12135799,C=36)
	CT(41802)=(S=12135800,E=12135803,C=19)
	CT(41803)=(S=12135804,E=12135807,C=33)
	CT(41804)=(S=12135808,E=12135811,C=19)
	CT(41805)=(S=12135812,E=12135815,C=28)
	CT(41806)=(S=12135816,E=12135819,C=33)
	CT(41807)=(S=12135820,E=12135823,C=55)
	CT(41808)=(S=12135824,E=12135827,C=18)
	CT(41809)=(S=12135828,E=12135831,C=24)
	CT(41810)=(S=12135832,E=12135835,C=51)
	CT(41811)=(S=12135836,E=12135839,C=16)
	CT(41812)=(S=12135840,E=12135843,C=14)
	CT(41813)=(S=12135844,E=12135847,C=25)
	CT(41814)=(S=12135848,E=12135851,C=14)
	CT(41815)=(S=12135852,E=12135855,C=12)
	CT(41816)=(S=12135856,E=12135859,C=27)
	CT(41817)=(S=12135860,E=12135863,C=12)
	CT(41818)=(S=12135864,E=12135867,C=24)
	CT(41819)=(S=12135868,E=12135871,C=26)
	CT(41820)=(S=12135872,E=12135875,C=16)
	CT(41821)=(S=12135876,E=12135879,C=50)
	CT(41822)=(S=12135880,E=12135883,C=33)
	CT(41823)=(S=12135884,E=12135887,C=28)
	CT(41824)=(S=12135888,E=12135891,C=15)
	CT(41825)=(S=12135892,E=12135895,C=42)
	CT(41826)=(S=12135896,E=12135899,C=58)
	CT(41827)=(S=12135900,E=12135907,C=14)
	CT(41828)=(S=12135908,E=12135911,C=12)
	CT(41829)=(S=12135912,E=12135915,C=42)
	CT(41830)=(S=12135916,E=12135919,C=34)
	CT(41831)=(S=12135920,E=12135923,C=28)
	CT(41832)=(S=12135924,E=12135927,C=31)
	CT(41833)=(S=12135928,E=12135931,C=28)
	CT(41834)=(S=12135932,E=12135935,C=12)
	CT(41835)=(S=12135936,E=12135939,C=26)
	CT(41836)=(S=12135940,E=12135943,C=22)
	CT(41837)=(S=12135944,E=12135955,C=18)
	CT(41838)=(S=12135956,E=12135959,C=55)
	CT(41839)=(S=12135960,E=12135963,C=15)
	CT(41840)=(S=12135964,E=12135967,C=36)
	CT(41841)=(S=12135968,E=12135971,C=57)
	CT(41842)=(S=12135972,E=12135975,C=24)
	CT(41843)=(S=12135976,E=12135979,C=37)
	CT(41844)=(S=12135980,E=12135987,C=18)
	CT(41845)=(S=12135988,E=12135991,C=37)
	CT(41846)=(S=12135992,E=12135995,C=34)
	CT(41847)=(S=12135996,E=12135999,C=53)
	CT(41848)=(S=12136000,E=12136003,C=36)
	CT(41849)=(S=12136004,E=12136007,C=15)
	CT(41850)=(S=12136008,E=12136011,C=63)
	CT(41851)=(S=12136012,E=12136015,C=49)
	CT(41852)=(S=12136016,E=12136019,C=34)
	CT(41853)=(S=12136020,E=12136023,C=18)
	CT(41854)=(S=12136024,E=12136027,C=31)
	CT(41855)=(S=12136028,E=12136031,C=58)
	CT(41856)=(S=12136032,E=12136035,C=18)
	CT(41857)=(S=12136036,E=12136039,C=15)
	CT(41858)=(S=12136040,E=12136043,C=58)
	CT(41859)=(S=12136044,E=12136047,C=26)
	CT(41860)=(S=12136048,E=12136051,C=14)
	CT(41861)=(S=12136052,E=12136055,C=36)
	CT(41862)=(S=12136056,E=12136059,C=29)
	CT(41863)=(S=12136060,E=12136063,C=25)
	CT(41864)=(S=12136064,E=12136067,C=28)
	CT(41865)=(S=12136068,E=12136071,C=27)
	CT(41866)=(S=12136072,E=12136075,C=28)
	CT(41867)=(S=12136076,E=12136079,C=15)
	CT(41868)=(S=12136080,E=12136083,C=101)
	CT(41869)=(S=12136084,E=12136087,C=31)
	CT(41870)=(S=12136088,E=12136091,C=18)
	CT(41871)=(S=12136092,E=12136095,C=36)
	CT(41872)=(S=12136096,E=12136099,C=51)
	CT(41873)=(S=12136100,E=12136103,C=12)
	CT(41874)=(S=12136104,E=12136107,C=58)
	CT(41875)=(S=12136108,E=12136111,C=14)
	CT(41876)=(S=12136112,E=12136115,C=40)
	CT(41877)=(S=12136116,E=12136119,C=36)
	CT(41878)=(S=12136120,E=12136123,C=18)
	CT(41879)=(S=12136124,E=12136127,C=31)
	CT(41880)=(S=12136128,E=12136131,C=36)
	CT(41881)=(S=12136132,E=12136135,C=18)
	CT(41882)=(S=12136136,E=12136139,C=14)
	CT(41883)=(S=12136140,E=12136143,C=58)
	CT(41884)=(S=12136144,E=12136147,C=14)
	CT(41885)=(S=12136148,E=12136151,C=34)
	CT(41886)=(S=12136152,E=12136155,C=26)
	CT(41887)=(S=12136156,E=12136159,C=31)
	CT(41888)=(S=12136160,E=12136163,C=27)
	CT(41889)=(S=12136164,E=12136167,C=12)
	CT(41890)=(S=12136168,E=12136171,C=36)
	CT(41891)=(S=12136172,E=12136175,C=31)
	CT(41892)=(S=12136176,E=12136183,C=58)
	CT(41893)=(S=12136184,E=12136187,C=31)
	CT(41894)=(S=12136188,E=12136191,C=63)
	CT(41895)=(S=12136192,E=12136195,C=36)
	CT(41896)=(S=12136196,E=12136199,C=41)
	CT(41897)=(S=12136200,E=12136203,C=60)
	CT(41898)=(S=12136204,E=12136207,C=36)
	CT(41899)=(S=12136208,E=12136211,C=25)
	CT(41900)=(S=12136212,E=12136215,C=36)
	CT(41901)=(S=12136216,E=12136219,C=34)
	CT(41902)=(S=12136220,E=12136223,C=49)
	CT(41903)=(S=12136224,E=12136227,C=25)
	CT(41904)=(S=12136228,E=12136231,C=18)
	CT(41905)=(S=12136232,E=12136235,C=27)
	CT(41906)=(S=12136236,E=12136239,C=28)
	CT(41907)=(S=12136240,E=12136243,C=26)
	CT(41908)=(S=12136244,E=12136247,C=18)
	CT(41909)=(S=12136248,E=12136251,C=54)
	CT(41910)=(S=12136252,E=12136255,C=14)
	CT(41911)=(S=12136256,E=12136259,C=58)
	CT(41912)=(S=12136260,E=12136263,C=49)
	CT(41913)=(S=12136264,E=12136267,C=63)
	CT(41914)=(S=12136268,E=12136271,C=47)
	CT(41915)=(S=12136272,E=12136275,C=12)
	CT(41916)=(S=12136276,E=12136279,C=29)
	CT(41917)=(S=12136280,E=12136283,C=46)
	CT(41918)=(S=12136284,E=12136287,C=60)
	CT(41919)=(S=12136288,E=12136291,C=15)
	CT(41920)=(S=12136292,E=12136295,C=36)
	CT(41921)=(S=12136296,E=12136299,C=14)
	CT(41922)=(S=12136300,E=12136303,C=15)
	CT(41923)=(S=12136304,E=12136307,C=36)
	CT(41924)=(S=12136308,E=12136311,C=14)
	CT(41925)=(S=12136312,E=12136315,C=36)
	CT(41926)=(S=12136316,E=12136319,C=28)
	CT(41927)=(S=12136320,E=12136323,C=25)
	CT(41928)=(S=12136324,E=12136327,C=36)
	CT(41929)=(S=12136328,E=12136331,C=15)
	CT(41930)=(S=12136332,E=12136335,C=28)
	CT(41931)=(S=12136336,E=12136339,C=63)
	CT(41932)=(S=12136340,E=12136343,C=49)
	CT(41933)=(S=12136344,E=12136351,C=18)
	CT(41934)=(S=12136352,E=12136355,C=50)
	CT(41935)=(S=12136356,E=12136359,C=19)
	CT(41936)=(S=12136360,E=12136363,C=36)
	CT(41937)=(S=12136364,E=12136367,C=17)
	CT(41938)=(S=12136368,E=12136371,C=33)
	CT(41939)=(S=12136372,E=12136375,C=35)
	CT(41940)=(S=12136376,E=12136379,C=50)
	CT(41941)=(S=12136380,E=12136383,C=46)
	CT(41942)=(S=12136384,E=12136387,C=19)
	CT(41943)=(S=12136388,E=12136395,C=36)
	CT(41944)=(S=12136396,E=12136399,C=18)
	CT(41945)=(S=12136400,E=12136403,C=46)
	CT(41946)=(S=12136404,E=12136407,C=25)
	CT(41947)=(S=12136408,E=12136411,C=14)
	CT(41948)=(S=12136412,E=12136415,C=33)
	CT(41949)=(S=12136416,E=12136419,C=36)
	CT(41950)=(S=12136420,E=12136423,C=15)
	CT(41951)=(S=12136424,E=12136427,C=28)
	CT(41952)=(S=12136428,E=12136431,C=36)
	CT(41953)=(S=12136432,E=12136435,C=12)
	CT(41954)=(S=12136436,E=12136439,C=25)
	CT(41955)=(S=12136440,E=12136443,C=27)
	CT(41956)=(S=12136444,E=12136447,C=12)
	CT(41957)=(S=12136448,E=12136451,C=28)
	CT(41958)=(S=12136452,E=12136455,C=42)
	CT(41959)=(S=12136456,E=12136459,C=58)
	CT(41960)=(S=12136460,E=12136463,C=49)
	CT(41961)=(S=12136464,E=12136467,C=18)
	CT(41962)=(S=12136468,E=12136471,C=33)
	CT(41963)=(S=12136472,E=12136475,C=37)
	CT(41964)=(S=12136476,E=12136479,C=58)
	CT(41965)=(S=12136480,E=12136483,C=15)
	CT(41966)=(S=12136484,E=12136487,C=18)
	CT(41967)=(S=12136488,E=12136491,C=50)
	CT(41968)=(S=12136492,E=12136499,C=12)
	CT(41969)=(S=12136500,E=12136503,C=15)
	CT(41970)=(S=12136504,E=12136511,C=14)
	CT(41971)=(S=12136512,E=12136515,C=16)
	CT(41972)=(S=12136516,E=12136519,C=36)
	CT(41973)=(S=12136520,E=12136523,C=18)
	CT(41974)=(S=12136524,E=12136527,C=64)
	CT(41975)=(S=12136528,E=12136531,C=19)
	CT(41976)=(S=12136532,E=12136539,C=25)
	CT(41977)=(S=12136540,E=12136543,C=28)
	CT(41978)=(S=12136544,E=12136547,C=15)
	CT(41979)=(S=12136548,E=12136551,C=36)
	CT(41980)=(S=12136552,E=12136555,C=28)
	CT(41981)=(S=12136556,E=12136559,C=14)
	CT(41982)=(S=12136560,E=12136563,C=18)
	CT(41983)=(S=12136564,E=12136567,C=28)
	CT(41984)=(S=12136568,E=12136571,C=63)
	CT(41985)=(S=12136572,E=12136575,C=20)
	CT(41986)=(S=12136576,E=12136579,C=58)
	CT(41987)=(S=12136580,E=12136583,C=12)
	CT(41988)=(S=12136584,E=12136587,C=62)
	CT(41989)=(S=12136588,E=12136591,C=15)
	CT(41990)=(S=12136592,E=12136595,C=34)
	CT(41991)=(S=12136596,E=12136599,C=20)
	CT(41992)=(S=12136600,E=12136603,C=24)
	CT(41993)=(S=12136604,E=12136607,C=28)
	CT(41994)=(S=12136608,E=12136615,C=14)
	CT(41995)=(S=12136616,E=12136619,C=58)
	CT(41996)=(S=12136620,E=12136623,C=59)
	CT(41997)=(S=12136624,E=12136627,C=58)
	CT(41998)=(S=12136628,E=12136631,C=37)
	CT(41999)=(S=12136632,E=12136635,C=18)
	CT(42000)=(S=12136636,E=12136639,C=101)
	CT(42001)=(S=12136640,E=12136643,C=25)
	CT(42002)=(S=12136644,E=12136647,C=31)
	CT(42003)=(S=12136648,E=12136651,C=12)
	CT(42004)=(S=12136652,E=12136655,C=25)
	CT(42005)=(S=12136656,E=12136659,C=28)
	CT(42006)=(S=12136660,E=12136663,C=37)
	CT(42007)=(S=12136664,E=12136675,C=28)
	CT(42008)=(S=12136676,E=12136679,C=34)
	CT(42009)=(S=12136680,E=12136683,C=58)
	CT(42010)=(S=12136684,E=12136687,C=18)
	CT(42011)=(S=12136688,E=12136691,C=66)
	CT(42012)=(S=12136692,E=12136695,C=28)
	CT(42013)=(S=12136696,E=12136699,C=71)
	CT(42014)=(S=12136700,E=12136703,C=12)
	CT(42015)=(S=12136704,E=12136707,C=52)
	CT(42016)=(S=12136708,E=12136711,C=27)
	CT(42017)=(S=12136712,E=12136715,C=15)
	CT(42018)=(S=12136716,E=12136719,C=58)
	CT(42019)=(S=12136720,E=12136723,C=28)
	CT(42020)=(S=12136724,E=12136727,C=12)
	CT(42021)=(S=12136728,E=12136731,C=96)
	CT(42022)=(S=12136732,E=12136735,C=58)
	CT(42023)=(S=12136736,E=12136747,C=14)
	CT(42024)=(S=12136748,E=12136751,C=37)
	CT(42025)=(S=12136752,E=12136755,C=36)
	CT(42026)=(S=12136756,E=12136759,C=42)
	CT(42027)=(S=12136760,E=12136763,C=15)
	CT(42028)=(S=12136764,E=12136767,C=63)
	CT(42029)=(S=12136768,E=12136771,C=18)
	CT(42030)=(S=12136772,E=12136775,C=37)
	CT(42031)=(S=12136776,E=12136779,C=14)
	CT(42032)=(S=12136780,E=12136783,C=28)
	CT(42033)=(S=12136784,E=12136787,C=36)
	CT(42034)=(S=12136788,E=12136791,C=26)
	CT(42035)=(S=12136792,E=12136795,C=51)
	CT(42036)=(S=12136796,E=12136799,C=14)
	CT(42037)=(S=12136800,E=12136803,C=26)
	CT(42038)=(S=12136804,E=12136807,C=28)
	CT(42039)=(S=12136808,E=12136811,C=26)
	CT(42040)=(S=12136812,E=12136815,C=18)
	CT(42041)=(S=12136816,E=12136819,C=14)
	CT(42042)=(S=12136820,E=12136823,C=52)
	CT(42043)=(S=12136824,E=12136827,C=36)
	CT(42044)=(S=12136828,E=12136831,C=14)
	CT(42045)=(S=12136832,E=12136835,C=37)
	CT(42046)=(S=12136836,E=12136839,C=19)
	CT(42047)=(S=12136840,E=12136843,C=28)
	CT(42048)=(S=12136844,E=12136851,C=36)
	CT(42049)=(S=12136852,E=12136855,C=58)
	CT(42050)=(S=12136856,E=12136859,C=34)
	CT(42051)=(S=12136860,E=12136863,C=25)
	CT(42052)=(S=12136864,E=12136867,C=104)
	CT(42053)=(S=12136868,E=12136871,C=40)
	CT(42054)=(S=12136872,E=12136875,C=25)
	CT(42055)=(S=12136876,E=12136879,C=14)
	CT(42056)=(S=12136880,E=12136883,C=42)
	CT(42057)=(S=12136884,E=12136887,C=49)
	CT(42058)=(S=12136888,E=12136891,C=25)
	CT(42059)=(S=12136892,E=12136895,C=33)
	CT(42060)=(S=12136896,E=12136899,C=25)
	CT(42061)=(S=12136900,E=12136903,C=27)
	CT(42062)=(S=12136904,E=12136907,C=58)
	CT(42063)=(S=12136908,E=12136911,C=15)
	CT(42064)=(S=12136912,E=12136915,C=12)
	CT(42065)=(S=12136916,E=12136919,C=15)
	CT(42066)=(S=12136920,E=12136923,C=14)
	CT(42067)=(S=12136924,E=12136927,C=34)
	CT(42068)=(S=12136928,E=12136931,C=25)
	CT(42069)=(S=12136932,E=12136943,C=14)
	CT(42070)=(S=12136944,E=12136947,C=18)
	CT(42071)=(S=12136948,E=12136951,C=70)
	CT(42072)=(S=12136952,E=12136955,C=15)
	CT(42073)=(S=12136956,E=12136959,C=24)
	CT(42074)=(S=12136960,E=12136963,C=68)
	CT(42075)=(S=12136964,E=12136967,C=53)
	CT(42076)=(S=12136968,E=12136971,C=18)
	CT(42077)=(S=12136972,E=12136975,C=57)
	CT(42078)=(S=12136976,E=12136983,C=14)
	CT(42079)=(S=12136984,E=12136987,C=18)
	CT(42080)=(S=12136988,E=12136991,C=50)
	CT(42081)=(S=12136992,E=12136995,C=12)
	CT(42082)=(S=12136996,E=12136996,C=19)
	CT(42083)=(S=12136997,E=12136997,C=63)
	CT(42084)=(S=12136998,E=12136999,C=26)
	CT(42085)=(S=12137000,E=12137003,C=19)
	CT(42086)=(S=12137004,E=12137007,C=25)
	CT(42087)=(S=12137008,E=12137011,C=58)
	CT(42088)=(S=12137012,E=12137015,C=12)
	CT(42089)=(S=12137016,E=12137019,C=59)
	CT(42090)=(S=12137020,E=12137023,C=36)
	CT(42091)=(S=12137024,E=12137027,C=12)
	CT(42092)=(S=12137028,E=12137031,C=37)
	CT(42093)=(S=12137032,E=12137039,C=12)
	CT(42094)=(S=12137040,E=12137043,C=18)
	CT(42095)=(S=12137044,E=12137047,C=49)
	CT(42096)=(S=12137048,E=12137051,C=47)
	CT(42097)=(S=12137052,E=12137055,C=36)
	CT(42098)=(S=12137056,E=12137059,C=55)
	CT(42099)=(S=12137060,E=12137063,C=63)
	CT(42100)=(S=12137064,E=12137067,C=36)
	CT(42101)=(S=12137068,E=12137071,C=28)
	CT(42102)=(S=12137072,E=12137075,C=12)
	CT(42103)=(S=12137076,E=12137079,C=25)
	CT(42104)=(S=12137080,E=12137083,C=28)
	CT(42105)=(S=12137084,E=12137087,C=33)
	CT(42106)=(S=12137088,E=12137091,C=12)
	CT(42107)=(S=12137092,E=12137095,C=32)
	CT(42108)=(S=12137096,E=12137099,C=18)
	CT(42109)=(S=12137100,E=12137103,C=28)
	CT(42110)=(S=12137104,E=12137107,C=25)
	CT(42111)=(S=12137108,E=12137111,C=33)
	CT(42112)=(S=12137116,E=12137119,C=18)
	CT(42113)=(S=12137120,E=12137123,C=14)
	CT(42114)=(S=12137124,E=12137127,C=58)
	CT(42115)=(S=12137128,E=12137131,C=25)
	CT(42116)=(S=12137132,E=12137135,C=36)
	CT(42117)=(S=12137136,E=12137139,C=25)
	CT(42118)=(S=12137140,E=12137143,C=96)
	CT(42119)=(S=12137144,E=12137147,C=46)
	CT(42120)=(S=12137148,E=12137151,C=34)
	CT(42121)=(S=12137152,E=12137155,C=28)
	CT(42122)=(S=12137156,E=12137159,C=25)
	CT(42123)=(S=12137160,E=12137163,C=18)
	CT(42124)=(S=12137164,E=12137167,C=25)
	CT(42125)=(S=12137168,E=12137171,C=36)
	CT(42126)=(S=12137172,E=12137175,C=71)
	CT(42127)=(S=12137176,E=12137179,C=18)
	CT(42128)=(S=12137180,E=12137187,C=34)
	CT(42129)=(S=12137188,E=12137191,C=33)
	CT(42130)=(S=12137192,E=12137195,C=42)
	CT(42131)=(S=12137196,E=12137199,C=34)
	CT(42132)=(S=12137200,E=12137203,C=14)
	CT(42133)=(S=12137204,E=12137207,C=47)
	CT(42134)=(S=12137208,E=12137211,C=31)
	CT(42135)=(S=12137212,E=12137215,C=12)
	CT(42136)=(S=12137216,E=12137219,C=19)
	CT(42137)=(S=12137220,E=12137223,C=29)
	CT(42138)=(S=12137224,E=12137227,C=42)
	CT(42139)=(S=12137228,E=12137231,C=15)
	CT(42140)=(S=12137232,E=12137235,C=40)
	CT(42141)=(S=12137236,E=12137239,C=18)
	CT(42142)=(S=12137240,E=12137243,C=37)
	CT(42143)=(S=12137244,E=12137247,C=33)
	CT(42144)=(S=12137248,E=12137251,C=19)
	CT(42145)=(S=12137252,E=12137255,C=37)
	CT(42146)=(S=12137256,E=12137259,C=26)
	CT(42147)=(S=12137260,E=12137263,C=25)
	CT(42148)=(S=12137264,E=12137275,C=36)
	CT(42149)=(S=12137276,E=12137279,C=18)
	CT(42150)=(S=12137280,E=12137283,C=50)
	CT(42151)=(S=12137284,E=12137287,C=34)
	CT(42152)=(S=12137288,E=12137291,C=63)
	CT(42153)=(S=12137292,E=12137295,C=24)
	CT(42154)=(S=12137296,E=12137299,C=14)
	CT(42155)=(S=12137300,E=12137303,C=28)
	CT(42156)=(S=12137304,E=12137307,C=58)
	CT(42157)=(S=12137308,E=12137311,C=21)
	CT(42158)=(S=12137312,E=12137315,C=58)
	CT(42159)=(S=12137316,E=12137319,C=40)
	CT(42160)=(S=12137320,E=12137323,C=63)
	CT(42161)=(S=12137324,E=12137327,C=25)
	CT(42162)=(S=12137328,E=12137331,C=37)
	CT(42163)=(S=12137332,E=12137335,C=58)
	CT(42164)=(S=12137336,E=12137339,C=15)
	CT(42165)=(S=12137340,E=12137343,C=18)
	CT(42166)=(S=12137344,E=12137347,C=42)
	CT(42167)=(S=12137348,E=12137351,C=22)
	CT(42168)=(S=12137352,E=12137355,C=15)
	CT(42169)=(S=12137356,E=12137359,C=18)
	CT(42170)=(S=12137360,E=12137363,C=19)
	CT(42171)=(S=12137364,E=12137367,C=36)
	CT(42172)=(S=12137368,E=12137371,C=16)
	CT(42173)=(S=12137372,E=12137375,C=18)
	CT(42174)=(S=12137376,E=12137379,C=63)
	CT(42175)=(S=12137380,E=12137383,C=37)
	CT(42176)=(S=12137384,E=12137387,C=15)
	CT(42177)=(S=12137388,E=12137391,C=14)
	CT(42178)=(S=12137392,E=12137395,C=12)
	CT(42179)=(S=12137396,E=12137399,C=58)
	CT(42180)=(S=12137400,E=12137403,C=28)
	CT(42181)=(S=12137404,E=12137407,C=50)
	CT(42182)=(S=12137408,E=12137411,C=36)
	CT(42183)=(S=12137412,E=12137415,C=42)
	CT(42184)=(S=12137416,E=12137419,C=26)
	CT(42185)=(S=12137420,E=12137423,C=23)
	CT(42186)=(S=12137424,E=12137427,C=18)
	CT(42187)=(S=12137428,E=12137431,C=61)
	CT(42188)=(S=12137432,E=12137435,C=12)
	CT(42189)=(S=12137436,E=12137439,C=35)
	CT(42190)=(S=12137440,E=12137443,C=19)
	CT(42191)=(S=12137444,E=12137447,C=33)
	CT(42192)=(S=12137448,E=12137451,C=27)
	CT(42193)=(S=12137452,E=12137455,C=72)
	CT(42194)=(S=12137456,E=12137459,C=33)
	CT(42195)=(S=12137460,E=12137463,C=14)
	CT(42196)=(S=12137464,E=12137467,C=33)
	CT(42197)=(S=12137468,E=12137471,C=14)
	CT(42198)=(S=12137472,E=12137475,C=29)
	CT(42199)=(S=12137476,E=12137479,C=36)
	CT(42200)=(S=12137480,E=12137483,C=15)
	CT(42201)=(S=12137484,E=12137487,C=36)
	CT(42202)=(S=12137488,E=12137491,C=25)
	CT(42203)=(S=12137492,E=12137495,C=42)
	CT(42204)=(S=12137496,E=12137499,C=14)
	CT(42205)=(S=12137500,E=12137503,C=18)
	CT(42206)=(S=12137504,E=12137507,C=12)
	CT(42207)=(S=12137508,E=12137511,C=14)
	CT(42208)=(S=12137512,E=12137515,C=27)
	CT(42209)=(S=12137516,E=12137519,C=12)
	CT(42210)=(S=12137520,E=12137523,C=51)
	CT(42211)=(S=12137524,E=12137535,C=36)
	CT(42212)=(S=12137536,E=12137539,C=15)
	CT(42213)=(S=12137540,E=12137551,C=18)
	CT(42214)=(S=12137552,E=12137555,C=40)
	CT(42215)=(S=12137556,E=12137559,C=12)
	CT(42216)=(S=12137560,E=12137563,C=32)
	CT(42217)=(S=12137564,E=12137567,C=63)
	CT(42218)=(S=12137568,E=12137571,C=33)
	CT(42219)=(S=12137572,E=12137575,C=35)
	CT(42220)=(S=12137576,E=12137579,C=36)
	CT(42221)=(S=12137580,E=12137583,C=14)
	CT(42222)=(S=12137584,E=12137587,C=15)
	CT(42223)=(S=12137588,E=12137591,C=35)
	CT(42224)=(S=12137592,E=12137595,C=42)
	CT(42225)=(S=12137596,E=12137599,C=12)
	CT(42226)=(S=12137600,E=12137603,C=41)
	CT(42227)=(S=12137604,E=12137607,C=18)
	CT(42228)=(S=12137608,E=12137611,C=29)
	CT(42229)=(S=12137612,E=12137615,C=18)
	CT(42230)=(S=12137616,E=12137623,C=14)
	CT(42231)=(S=12137624,E=12137627,C=18)
	CT(42232)=(S=12137628,E=12137631,C=19)
	CT(42233)=(S=12137632,E=12137635,C=28)
	CT(42234)=(S=12137636,E=12137639,C=47)
	CT(42235)=(S=12137640,E=12137643,C=58)
	CT(42236)=(S=12137644,E=12137647,C=33)
	CT(42237)=(S=12137648,E=12137651,C=36)
	CT(42238)=(S=12137652,E=12137655,C=19)
	CT(42239)=(S=12137656,E=12137663,C=42)
	CT(42240)=(S=12137664,E=12137667,C=58)
	CT(42241)=(S=12137668,E=12137675,C=14)
	CT(42242)=(S=12137676,E=12137679,C=51)
	CT(42243)=(S=12137680,E=12137687,C=36)
	CT(42244)=(S=12137688,E=12137691,C=15)
	CT(42245)=(S=12137692,E=12137695,C=14)
	CT(42246)=(S=12137696,E=12137699,C=36)
	CT(42247)=(S=12137700,E=12137703,C=37)
	CT(42248)=(S=12137704,E=12137707,C=30)
	CT(42249)=(S=12137708,E=12137711,C=36)
	CT(42250)=(S=12137712,E=12137715,C=15)
	CT(42251)=(S=12137716,E=12137719,C=28)
	CT(42252)=(S=12137720,E=12137727,C=14)
	CT(42253)=(S=12137728,E=12137731,C=15)
	CT(42254)=(S=12137732,E=12137735,C=33)
	CT(42255)=(S=12137736,E=12137739,C=36)
	CT(42256)=(S=12137740,E=12137743,C=52)
	CT(42257)=(S=12137744,E=12137747,C=14)
	CT(42258)=(S=12137748,E=12137751,C=18)
	CT(42259)=(S=12137752,E=12137755,C=12)
	CT(42260)=(S=12137756,E=12137759,C=14)
	CT(42261)=(S=12137760,E=12137763,C=63)
	CT(42262)=(S=12137764,E=12137767,C=15)
	CT(42263)=(S=12137768,E=12137775,C=28)
	CT(42264)=(S=12137776,E=12137779,C=36)
	CT(42265)=(S=12137780,E=12137783,C=16)
	CT(42266)=(S=12137784,E=12137787,C=14)
	CT(42267)=(S=12137788,E=12137791,C=37)
	CT(42268)=(S=12137792,E=12137795,C=19)
	CT(42269)=(S=12137796,E=12137799,C=15)
	CT(42270)=(S=12137800,E=12137803,C=68)
	CT(42271)=(S=12137804,E=12137807,C=31)
	CT(42272)=(S=12137808,E=12137811,C=12)
	CT(42273)=(S=12137812,E=12137815,C=47)
	CT(42274)=(S=12137816,E=12137819,C=13)
	CT(42275)=(S=12137820,E=12137823,C=14)
	CT(42276)=(S=12137824,E=12137827,C=58)
	CT(42277)=(S=12137828,E=12137831,C=96)
	CT(42278)=(S=12137832,E=12137835,C=62)
	CT(42279)=(S=12137836,E=12137839,C=42)
	CT(42280)=(S=12137840,E=12137843,C=46)
	CT(42281)=(S=12137844,E=12137847,C=27)
	CT(42282)=(S=12137848,E=12137851,C=15)
	CT(42283)=(S=12137852,E=12137855,C=58)
	CT(42284)=(S=12137856,E=12137859,C=13)
	CT(42285)=(S=12137860,E=12137863,C=58)
	CT(42286)=(S=12137864,E=12137867,C=12)
	CT(42287)=(S=12137868,E=12137871,C=28)
	CT(42288)=(S=12137872,E=12137875,C=58)
	CT(42289)=(S=12137876,E=12137879,C=15)
	CT(42290)=(S=12137880,E=12137883,C=58)
	CT(42291)=(S=12137884,E=12137887,C=28)
	CT(42292)=(S=12137888,E=12137891,C=36)
	CT(42293)=(S=12137892,E=12137895,C=19)
	CT(42294)=(S=12137896,E=12137903,C=14)
	CT(42295)=(S=12137904,E=12137907,C=28)
	CT(42296)=(S=12137908,E=12137911,C=42)
	CT(42297)=(S=12137912,E=12137915,C=12)
	CT(42298)=(S=12137916,E=12137919,C=27)
	CT(42299)=(S=12137920,E=12137923,C=15)
	CT(42300)=(S=12137924,E=12137931,C=38)
	CT(42301)=(S=12137932,E=12137935,C=28)
	CT(42302)=(S=12137936,E=12137939,C=17)
	CT(42303)=(S=12137940,E=12137943,C=19)
	CT(42304)=(S=12137944,E=12137951,C=28)
	CT(42305)=(S=12137952,E=12137955,C=14)
	CT(42306)=(S=12137956,E=12137959,C=62)
	CT(42307)=(S=12137960,E=12137963,C=18)
	CT(42308)=(S=12137964,E=12137967,C=36)
	CT(42309)=(S=12137968,E=12137971,C=45)
	CT(42310)=(S=12137972,E=12137975,C=23)
	CT(42311)=(S=12137976,E=12137979,C=36)
	CT(42312)=(S=12137980,E=12137983,C=15)
	CT(42313)=(S=12137984,E=12137987,C=25)
	CT(42314)=(S=12137988,E=12137991,C=34)
	CT(42315)=(S=12137992,E=12137995,C=12)
	CT(42316)=(S=12137996,E=12137999,C=44)
	CT(42317)=(S=12138000,E=12138003,C=16)
	CT(42318)=(S=12138004,E=12138007,C=18)
	CT(42319)=(S=12138008,E=12138011,C=40)
	CT(42320)=(S=12138012,E=12138015,C=27)
	CT(42321)=(S=12138016,E=12138019,C=15)
	CT(42322)=(S=12138020,E=12138023,C=12)
	CT(42323)=(S=12138024,E=12138027,C=18)
	CT(42324)=(S=12138028,E=12138031,C=36)
	CT(42325)=(S=12138032,E=12138035,C=18)
	CT(42326)=(S=12138036,E=12138043,C=58)
	CT(42327)=(S=12138044,E=12138047,C=39)
	CT(42328)=(S=12138048,E=12138051,C=14)
	CT(42329)=(S=12138052,E=12138055,C=28)
	CT(42330)=(S=12138056,E=12138059,C=14)
	CT(42331)=(S=12138060,E=12138063,C=28)
	CT(42332)=(S=12138064,E=12138067,C=34)
	CT(42333)=(S=12138068,E=12138071,C=14)
	CT(42334)=(S=12138072,E=12138075,C=37)
	CT(42335)=(S=12138076,E=12138079,C=36)
	CT(42336)=(S=12138080,E=12138083,C=39)
	CT(42337)=(S=12138084,E=12138087,C=58)
	CT(42338)=(S=12138088,E=12138091,C=63)
	CT(42339)=(S=12138092,E=12138095,C=28)
	CT(42340)=(S=12138096,E=12138099,C=36)
	CT(42341)=(S=12138100,E=12138107,C=28)
	CT(42342)=(S=12138108,E=12138111,C=14)
	CT(42343)=(S=12138112,E=12138115,C=52)
	CT(42344)=(S=12138116,E=12138119,C=30)
	CT(42345)=(S=12138120,E=12138123,C=18)
	CT(42346)=(S=12138124,E=12138127,C=63)
	CT(42347)=(S=12138128,E=12138131,C=42)
	CT(42348)=(S=12138132,E=12138135,C=28)
	CT(42349)=(S=12138136,E=12138139,C=15)
	CT(42350)=(S=12138140,E=12138143,C=35)
	CT(42351)=(S=12138144,E=12138147,C=32)
	CT(42352)=(S=12138148,E=12138151,C=35)
	CT(42353)=(S=12138152,E=12138155,C=14)
	CT(42354)=(S=12138156,E=12138159,C=34)
	CT(42355)=(S=12138160,E=12138163,C=18)
	CT(42356)=(S=12138164,E=12138167,C=36)
	CT(42357)=(S=12138168,E=12138171,C=58)
	CT(42358)=(S=12138172,E=12138175,C=28)
	CT(42359)=(S=12138176,E=12138179,C=25)
	CT(42360)=(S=12138180,E=12138183,C=19)
	CT(42361)=(S=12138184,E=12138187,C=14)
	CT(42362)=(S=12138188,E=12138191,C=36)
	CT(42363)=(S=12138192,E=12138195,C=42)
	CT(42364)=(S=12138196,E=12138199,C=15)
	CT(42365)=(S=12138200,E=12138203,C=36)
	CT(42366)=(S=12138204,E=12138207,C=18)
	CT(42367)=(S=12138208,E=12138211,C=63)
	CT(42368)=(S=12138212,E=12138215,C=14)
	CT(42369)=(S=12138216,E=12138219,C=28)
	CT(42370)=(S=12138220,E=12138223,C=18)
	CT(42371)=(S=12138224,E=12138227,C=28)
	CT(42372)=(S=12138228,E=12138231,C=18)
	CT(42373)=(S=12138232,E=12138235,C=49)
	CT(42374)=(S=12138236,E=12138239,C=62)
	CT(42375)=(S=12138240,E=12138243,C=18)
	CT(42376)=(S=12138244,E=12138247,C=38)
	CT(42377)=(S=12138248,E=12138251,C=19)
	CT(42378)=(S=12138252,E=12138255,C=50)
	CT(42379)=(S=12138256,E=12138259,C=14)
	CT(42380)=(S=12138260,E=12138263,C=36)
	CT(42381)=(S=12138264,E=12138267,C=42)
	CT(42382)=(S=12138268,E=12138271,C=22)
	CT(42383)=(S=12138272,E=12138275,C=47)
	CT(42384)=(S=12138276,E=12138279,C=41)
	CT(42385)=(S=12138280,E=12138283,C=18)
	CT(42386)=(S=12138284,E=12138287,C=96)
	CT(42387)=(S=12138288,E=12138291,C=55)
	CT(42388)=(S=12138292,E=12138295,C=36)
	CT(42389)=(S=12138296,E=12138299,C=25)
	CT(42390)=(S=12138300,E=12138303,C=14)
	CT(42391)=(S=12138304,E=12138307,C=58)
	CT(42392)=(S=12138308,E=12138315,C=28)
	CT(42393)=(S=12138316,E=12138319,C=14)
	CT(42394)=(S=12138320,E=12138323,C=15)
	CT(42395)=(S=12138324,E=12138327,C=47)
	CT(42396)=(S=12138328,E=12138331,C=15)
	CT(42397)=(S=12138332,E=12138335,C=49)
	CT(42398)=(S=12138336,E=12138339,C=12)
	CT(42399)=(S=12138340,E=12138343,C=28)
	CT(42400)=(S=12138344,E=12138347,C=27)
	CT(42401)=(S=12138348,E=12138355,C=36)
	CT(42402)=(S=12138356,E=12138367,C=28)
	CT(42403)=(S=12138368,E=12138371,C=36)
	CT(42404)=(S=12138372,E=12138375,C=14)
	CT(42405)=(S=12138376,E=12138379,C=36)
	CT(42406)=(S=12138380,E=12138383,C=38)
	CT(42407)=(S=12138384,E=12138387,C=12)
	CT(42408)=(S=12138388,E=12138391,C=96)
	CT(42409)=(S=12138392,E=12138395,C=28)
	CT(42410)=(S=12138400,E=12138403,C=14)
	CT(42411)=(S=12138404,E=12138407,C=12)
	CT(42412)=(S=12138408,E=12138411,C=15)
	CT(42413)=(S=12138412,E=12138415,C=18)
	CT(42414)=(S=12138416,E=12138419,C=47)
	CT(42415)=(S=12138420,E=12138423,C=28)
	CT(42416)=(S=12138424,E=12138427,C=35)
	CT(42417)=(S=12138428,E=12138431,C=14)
	CT(42418)=(S=12138432,E=12138435,C=28)
	CT(42419)=(S=12138436,E=12138439,C=24)
	CT(42420)=(S=12138440,E=12138447,C=63)
	CT(42421)=(S=12138448,E=12138451,C=13)
	CT(42422)=(S=12138452,E=12138455,C=15)
	CT(42423)=(S=12138456,E=12138459,C=25)
	CT(42424)=(S=12138460,E=12138463,C=28)
	CT(42425)=(S=12138464,E=12138467,C=26)
	CT(42426)=(S=12138468,E=12138471,C=51)
	CT(42427)=(S=12138472,E=12138475,C=28)
	CT(42428)=(S=12138476,E=12138479,C=42)
	CT(42429)=(S=12138480,E=12138483,C=28)
	CT(42430)=(S=12138484,E=12138487,C=12)
	CT(42431)=(S=12138488,E=12138491,C=25)
	CT(42432)=(S=12138492,E=12138495,C=14)
	CT(42433)=(S=12138496,E=12138499,C=15)
	CT(42434)=(S=12138500,E=12138503,C=14)
	CT(42435)=(S=12138504,E=12138507,C=15)
	CT(42436)=(S=12138508,E=12138511,C=48)
	CT(42437)=(S=12138512,E=12138515,C=31)
	CT(42438)=(S=12138516,E=12138519,C=14)
	CT(42439)=(S=12138520,E=12138523,C=34)
	CT(42440)=(S=12138524,E=12138531,C=36)
	CT(42441)=(S=12138532,E=12138535,C=58)
	CT(42442)=(S=12138536,E=12138543,C=14)
	CT(42443)=(S=12138544,E=12138547,C=12)
	CT(42444)=(S=12138548,E=12138551,C=36)
	CT(42445)=(S=12138552,E=12138555,C=24)
	CT(42446)=(S=12138556,E=12138563,C=36)
	CT(42447)=(S=12138564,E=12138567,C=14)
	CT(42448)=(S=12138568,E=12138571,C=17)
	CT(42449)=(S=12138572,E=12138575,C=25)
	CT(42450)=(S=12138576,E=12138579,C=36)
	CT(42451)=(S=12138580,E=12138583,C=51)
	CT(42452)=(S=12138584,E=12138587,C=16)
	CT(42453)=(S=12138588,E=12138595,C=26)
	CT(42454)=(S=12138596,E=12138599,C=24)
	CT(42455)=(S=12138600,E=12138603,C=28)
	CT(42456)=(S=12138604,E=12138607,C=25)
	CT(42457)=(S=12138608,E=12138611,C=28)
	CT(42458)=(S=12138612,E=12138615,C=18)
	CT(42459)=(S=12138616,E=12138619,C=15)
	CT(42460)=(S=12138620,E=12138623,C=42)
	CT(42461)=(S=12138624,E=12138631,C=28)
	CT(42462)=(S=12138632,E=12138635,C=72)
	CT(42463)=(S=12138636,E=12138639,C=24)
	CT(42464)=(S=12138640,E=12138643,C=36)
	CT(42465)=(S=12138644,E=12138647,C=28)
	CT(42466)=(S=12138648,E=12138651,C=12)
	CT(42467)=(S=12138652,E=12138655,C=15)
	CT(42468)=(S=12138656,E=12138659,C=12)
	CT(42469)=(S=12138660,E=12138663,C=24)
	CT(42470)=(S=12138664,E=12138667,C=15)
	CT(42471)=(S=12138668,E=12138671,C=58)
	CT(42472)=(S=12138672,E=12138675,C=12)
	CT(42473)=(S=12138676,E=12138679,C=25)
	CT(42474)=(S=12138680,E=12138683,C=27)
	CT(42475)=(S=12138684,E=12138687,C=58)
	CT(42476)=(S=12138688,E=12138691,C=35)
	CT(42477)=(S=12138692,E=12138695,C=23)
	CT(42478)=(S=12138696,E=12138699,C=14)
	CT(42479)=(S=12138700,E=12138703,C=12)
	CT(42480)=(S=12138704,E=12138707,C=58)
	CT(42481)=(S=12138708,E=12138711,C=19)
	CT(42482)=(S=12138712,E=12138715,C=15)
	CT(42483)=(S=12138716,E=12138719,C=52)
	CT(42484)=(S=12138720,E=12138723,C=36)
	CT(42485)=(S=12138724,E=12138727,C=28)
	CT(42486)=(S=12138728,E=12138731,C=31)
	CT(42487)=(S=12138732,E=12138735,C=37)
	CT(42488)=(S=12138736,E=12138739,C=12)
	CT(42489)=(S=12138740,E=12138743,C=58)
	CT(42490)=(S=12138744,E=12138747,C=54)
	CT(42491)=(S=12138748,E=12138751,C=72)
	CT(42492)=(S=12138752,E=12138755,C=28)
	CT(42493)=(S=12138756,E=12138759,C=19)
	CT(42494)=(S=12138760,E=12138763,C=36)
	CT(42495)=(S=12138764,E=12138767,C=12)
	CT(42496)=(S=12138768,E=12138771,C=52)
	CT(42497)=(S=12138772,E=12138775,C=15)
	CT(42498)=(S=12138776,E=12138779,C=28)
	CT(42499)=(S=12138780,E=12138783,C=18)
	CT(42500)=(S=12138784,E=12138795,C=14)
	CT(42501)=(S=12138796,E=12138799,C=53)
	CT(42502)=(S=12138800,E=12138803,C=42)
	CT(42503)=(S=12138804,E=12138807,C=12)
	CT(42504)=(S=12138808,E=12138811,C=13)
	CT(42505)=(S=12138812,E=12138815,C=36)
	CT(42506)=(S=12138816,E=12138819,C=37)
	CT(42507)=(S=12138820,E=12138823,C=66)
	CT(42508)=(S=12138824,E=12138827,C=20)
	CT(42509)=(S=12138828,E=12138831,C=14)
	CT(42510)=(S=12138832,E=12138835,C=38)
	CT(42511)=(S=12138836,E=12138839,C=19)
	CT(42512)=(S=12138840,E=12138843,C=28)
	CT(42513)=(S=12138844,E=12138847,C=14)
	CT(42514)=(S=12138848,E=12138851,C=36)
	CT(42515)=(S=12138852,E=12138855,C=14)
	CT(42516)=(S=12138856,E=12138859,C=19)
	CT(42517)=(S=12138860,E=12138863,C=15)
	CT(42518)=(S=12138864,E=12138871,C=63)
	CT(42519)=(S=12138872,E=12138875,C=61)
	CT(42520)=(S=12138876,E=12138879,C=12)
	CT(42521)=(S=12138880,E=12138883,C=49)
	CT(42522)=(S=12138884,E=12138887,C=26)
	CT(42523)=(S=12138888,E=12138891,C=19)
	CT(42524)=(S=12138892,E=12138895,C=36)
	CT(42525)=(S=12138896,E=12138899,C=52)
	CT(42526)=(S=12138900,E=12138903,C=12)
	CT(42527)=(S=12138904,E=12138911,C=14)
	CT(42528)=(S=12138912,E=12138915,C=168)
	CT(42529)=(S=12138916,E=12138919,C=26)
	CT(42530)=(S=12138920,E=12138923,C=19)
	CT(42531)=(S=12138924,E=12138927,C=36)
	CT(42532)=(S=12138928,E=12138931,C=42)
	CT(42533)=(S=12138932,E=12138935,C=47)
	CT(42534)=(S=12138936,E=12138939,C=28)
	CT(42535)=(S=12138940,E=12138943,C=14)
	CT(42536)=(S=12138944,E=12138951,C=25)
	CT(42537)=(S=12138952,E=12138955,C=26)
	CT(42538)=(S=12138956,E=12138959,C=25)
	CT(42539)=(S=12138960,E=12138963,C=15)
	CT(42540)=(S=12138964,E=12138967,C=14)
	CT(42541)=(S=12138968,E=12138971,C=28)
	CT(42542)=(S=12138972,E=12138975,C=12)
	CT(42543)=(S=12138976,E=12138979,C=52)
	CT(42544)=(S=12138980,E=12138983,C=33)
	CT(42545)=(S=12138984,E=12138987,C=15)
	CT(42546)=(S=12138988,E=12138991,C=18)
	CT(42547)=(S=12138992,E=12138995,C=28)
	CT(42548)=(S=12138996,E=12138999,C=37)
	CT(42549)=(S=12139000,E=12139003,C=18)
	CT(42550)=(S=12139004,E=12139007,C=27)
	CT(42551)=(S=12139008,E=12139011,C=28)
	CT(42552)=(S=12139012,E=12139015,C=15)
	CT(42553)=(S=12139016,E=12139019,C=12)
	CT(42554)=(S=12139020,E=12139023,C=35)
	CT(42555)=(S=12139024,E=12139027,C=63)
	CT(42556)=(S=12139028,E=12139031,C=51)
	CT(42557)=(S=12139032,E=12139035,C=14)
	CT(42558)=(S=12139036,E=12139039,C=28)
	CT(42559)=(S=12139040,E=12139043,C=47)
	CT(42560)=(S=12139044,E=12139047,C=28)
	CT(42561)=(S=12139048,E=12139051,C=33)
	CT(42562)=(S=12139052,E=12139055,C=15)
	CT(42563)=(S=12139056,E=12139059,C=58)
	CT(42564)=(S=12139060,E=12139063,C=42)
	CT(42565)=(S=12139064,E=12139067,C=36)
	CT(42566)=(S=12139068,E=12139071,C=24)
	CT(42567)=(S=12139072,E=12139075,C=44)
	CT(42568)=(S=12139076,E=12139079,C=27)
	CT(42569)=(S=12139080,E=12139083,C=57)
	CT(42570)=(S=12139084,E=12139087,C=19)
	CT(42571)=(S=12139088,E=12139091,C=21)
	CT(42572)=(S=12139092,E=12139099,C=14)
	CT(42573)=(S=12139100,E=12139103,C=59)
	CT(42574)=(S=12139104,E=12139107,C=49)
	CT(42575)=(S=12139108,E=12139111,C=71)
	CT(42576)=(S=12139112,E=12139115,C=15)
	CT(42577)=(S=12139116,E=12139119,C=28)
	CT(42578)=(S=12139120,E=12139123,C=18)
	CT(42579)=(S=12139124,E=12139131,C=15)
	CT(42580)=(S=12139132,E=12139135,C=18)
	CT(42581)=(S=12139136,E=12139139,C=34)
	CT(42582)=(S=12139140,E=12139143,C=15)
	CT(42583)=(S=12139144,E=12139147,C=27)
	CT(42584)=(S=12139148,E=12139155,C=19)
	CT(42585)=(S=12139156,E=12139159,C=36)
	CT(42586)=(S=12139160,E=12139163,C=18)
	CT(42587)=(S=12139164,E=12139167,C=28)
	CT(42588)=(S=12139168,E=12139171,C=58)
	CT(42589)=(S=12139172,E=12139175,C=14)
	CT(42590)=(S=12139176,E=12139179,C=28)
	CT(42591)=(S=12139180,E=12139183,C=25)
	CT(42592)=(S=12139184,E=12139187,C=14)
	CT(42593)=(S=12139188,E=12139191,C=52)
	CT(42594)=(S=12139192,E=12139195,C=12)
	CT(42595)=(S=12139196,E=12139199,C=18)
	CT(42596)=(S=12139200,E=12139203,C=15)
	CT(42597)=(S=12139204,E=12139207,C=25)
	CT(42598)=(S=12139208,E=12139211,C=39)
	CT(42599)=(S=12139212,E=12139215,C=18)
	CT(42600)=(S=12139216,E=12139219,C=25)
	CT(42601)=(S=12139220,E=12139223,C=24)
	CT(42602)=(S=12139224,E=12139231,C=18)
	CT(42603)=(S=12139232,E=12139235,C=15)
	CT(42604)=(S=12139236,E=12139239,C=12)
	CT(42605)=(S=12139240,E=12139243,C=63)
	CT(42606)=(S=12139244,E=12139247,C=36)
	CT(42607)=(S=12139248,E=12139251,C=26)
	CT(42608)=(S=12139252,E=12139255,C=37)
	CT(42609)=(S=12139256,E=12139263,C=18)
	CT(42610)=(S=12139264,E=12139267,C=33)
	CT(42611)=(S=12139268,E=12139271,C=58)
	CT(42612)=(S=12139272,E=12139275,C=12)
	CT(42613)=(S=12139276,E=12139279,C=28)
	CT(42614)=(S=12139280,E=12139283,C=49)
	CT(42615)=(S=12139284,E=12139287,C=15)
	CT(42616)=(S=12139288,E=12139291,C=52)
	CT(42617)=(S=12139292,E=12139295,C=37)
	CT(42618)=(S=12139296,E=12139299,C=55)
	CT(42619)=(S=12139300,E=12139303,C=36)
	CT(42620)=(S=12139304,E=12139307,C=12)
	CT(42621)=(S=12139308,E=12139311,C=37)
	CT(42622)=(S=12139312,E=12139315,C=25)
	CT(42623)=(S=12139316,E=12139319,C=34)
	CT(42624)=(S=12139320,E=12139323,C=18)
	CT(42625)=(S=12139324,E=12139327,C=14)
	CT(42626)=(S=12139328,E=12139331,C=25)
	CT(42627)=(S=12139332,E=12139335,C=66)
	CT(42628)=(S=12139336,E=12139339,C=37)
	CT(42629)=(S=12139340,E=12139343,C=36)
	CT(42630)=(S=12139344,E=12139347,C=15)
	CT(42631)=(S=12139348,E=12139351,C=16)
	CT(42632)=(S=12139352,E=12139355,C=58)
	CT(42633)=(S=12139356,E=12139359,C=14)
	CT(42634)=(S=12139360,E=12139367,C=28)
	CT(42635)=(S=12139368,E=12139371,C=24)
	CT(42636)=(S=12139372,E=12139375,C=58)
	CT(42637)=(S=12139376,E=12139379,C=26)
	CT(42638)=(S=12139380,E=12139383,C=47)
	CT(42639)=(S=12139384,E=12139387,C=33)
	CT(42640)=(S=12139388,E=12139391,C=14)
	CT(42641)=(S=12139392,E=12139395,C=28)
	CT(42642)=(S=12139396,E=12139399,C=15)
	CT(42643)=(S=12139400,E=12139407,C=18)
	CT(42644)=(S=12139408,E=12139411,C=33)
	CT(42645)=(S=12139412,E=12139415,C=29)
	CT(42646)=(S=12139416,E=12139419,C=15)
	CT(42647)=(S=12139420,E=12139423,C=31)
	CT(42648)=(S=12139424,E=12139427,C=58)
	CT(42649)=(S=12139428,E=12139431,C=15)
	CT(42650)=(S=12139432,E=12139435,C=49)
	CT(42651)=(S=12139436,E=12139439,C=18)
	CT(42652)=(S=12139440,E=12139447,C=14)
	CT(42653)=(S=12139448,E=12139451,C=71)
	CT(42654)=(S=12139452,E=12139455,C=23)
	CT(42655)=(S=12139456,E=12139459,C=18)
	CT(42656)=(S=12139460,E=12139463,C=49)
	CT(42657)=(S=12139464,E=12139467,C=28)
	CT(42658)=(S=12139468,E=12139471,C=14)
	CT(42659)=(S=12139472,E=12139475,C=33)
	CT(42660)=(S=12139476,E=12139479,C=27)
	CT(42661)=(S=12139480,E=12139483,C=18)
	CT(42662)=(S=12139484,E=12139487,C=14)
	CT(42663)=(S=12139488,E=12139491,C=58)
	CT(42664)=(S=12139492,E=12139495,C=24)
	CT(42665)=(S=12139496,E=12139499,C=18)
	CT(42666)=(S=12139500,E=12139503,C=27)
	CT(42667)=(S=12139504,E=12139507,C=38)
	CT(42668)=(S=12139508,E=12139511,C=18)
	CT(42669)=(S=12139512,E=12139515,C=28)
	CT(42670)=(S=12139516,E=12139519,C=60)
	CT(42671)=(S=12139520,E=12139523,C=28)
	CT(42672)=(S=12139524,E=12139527,C=14)
	CT(42673)=(S=12139528,E=12139531,C=12)
	CT(42674)=(S=12139532,E=12139535,C=18)
	CT(42675)=(S=12139536,E=12139543,C=28)
	CT(42676)=(S=12139544,E=12139547,C=12)
	CT(42677)=(S=12139548,E=12139551,C=14)
	CT(42678)=(S=12139552,E=12139555,C=26)
	CT(42679)=(S=12139556,E=12139559,C=28)
	CT(42680)=(S=12139560,E=12139563,C=25)
	CT(42681)=(S=12139564,E=12139571,C=18)
	CT(42682)=(S=12139572,E=12139575,C=34)
	CT(42683)=(S=12139576,E=12139579,C=28)
	CT(42684)=(S=12139580,E=12139583,C=36)
	CT(42685)=(S=12139584,E=12139587,C=51)
	CT(42686)=(S=12139588,E=12139591,C=50)
	CT(42687)=(S=12139592,E=12139599,C=28)
	CT(42688)=(S=12139600,E=12139603,C=14)
	CT(42689)=(S=12139604,E=12139607,C=13)
	CT(42690)=(S=12139608,E=12139611,C=64)
	CT(42691)=(S=12139612,E=12139615,C=12)
	CT(42692)=(S=12139616,E=12139619,C=28)
	CT(42693)=(S=12139620,E=12139623,C=50)
	CT(42694)=(S=12139624,E=12139627,C=33)
	CT(42695)=(S=12139628,E=12139635,C=12)
	CT(42696)=(S=12139636,E=12139639,C=52)
	CT(42697)=(S=12139640,E=12139643,C=14)
	CT(42698)=(S=12139644,E=12139647,C=63)
	CT(42699)=(S=12139648,E=12139651,C=19)
	CT(42700)=(S=12139652,E=12139655,C=18)
	CT(42701)=(S=12139656,E=12139659,C=26)
	CT(42702)=(S=12139660,E=12139663,C=15)
	CT(42703)=(S=12139664,E=12139667,C=34)
	CT(42704)=(S=12139668,E=12139671,C=12)
	CT(42705)=(S=12139672,E=12139675,C=18)
	CT(42706)=(S=12139676,E=12139679,C=36)
	CT(42707)=(S=12139680,E=12139683,C=24)
	CT(42708)=(S=12139684,E=12139687,C=12)
	CT(42709)=(S=12139688,E=12139691,C=17)
	CT(42710)=(S=12139692,E=12139695,C=27)
	CT(42711)=(S=12139696,E=12139699,C=12)
	CT(42712)=(S=12139700,E=12139703,C=34)
	CT(42713)=(S=12139704,E=12139707,C=58)
	CT(42714)=(S=12139708,E=12139715,C=15)
	CT(42715)=(S=12139716,E=12139719,C=28)
	CT(42716)=(S=12139720,E=12139723,C=16)
	CT(42717)=(S=12139724,E=12139727,C=24)
	CT(42718)=(S=12139728,E=12139731,C=28)
	CT(42719)=(S=12139732,E=12139735,C=18)
	CT(42720)=(S=12139736,E=12139739,C=63)
	CT(42721)=(S=12139740,E=12139743,C=14)
	CT(42722)=(S=12139744,E=12139747,C=37)
	CT(42723)=(S=12139748,E=12139751,C=58)
	CT(42724)=(S=12139752,E=12139755,C=191)
	CT(42725)=(S=12139756,E=12139759,C=13)
	CT(42726)=(S=12139760,E=12139763,C=15)
	CT(42727)=(S=12139764,E=12139767,C=58)
	CT(42728)=(S=12139768,E=12139771,C=28)
	CT(42729)=(S=12139772,E=12139775,C=36)
	CT(42730)=(S=12139776,E=12139779,C=63)
	CT(42731)=(S=12139780,E=12139783,C=58)
	CT(42732)=(S=12139784,E=12139787,C=28)
	CT(42733)=(S=12139788,E=12139791,C=15)
	CT(42734)=(S=12139792,E=12139795,C=71)
	CT(42735)=(S=12139796,E=12139799,C=43)
	CT(42736)=(S=12139800,E=12139803,C=18)
	CT(42737)=(S=12139804,E=12139811,C=28)
	CT(42738)=(S=12139812,E=12139815,C=18)
	CT(42739)=(S=12139816,E=12139819,C=42)
	CT(42740)=(S=12139820,E=12139823,C=37)
	CT(42741)=(S=12139824,E=12139827,C=39)
	CT(42742)=(S=12139828,E=12139831,C=36)
	CT(42743)=(S=12139832,E=12139835,C=46)
	CT(42744)=(S=12139836,E=12139839,C=63)
	CT(42745)=(S=12139840,E=12139843,C=44)
	CT(42746)=(S=12139844,E=12139847,C=36)
	CT(42747)=(S=12139848,E=12139851,C=49)
	CT(42748)=(S=12139852,E=12139855,C=18)
	CT(42749)=(S=12139856,E=12139859,C=25)
	CT(42750)=(S=12139860,E=12139863,C=33)
	CT(42751)=(S=12139864,E=12139867,C=14)
	CT(42752)=(S=12139868,E=12139871,C=18)
	CT(42753)=(S=12139872,E=12139875,C=14)
	CT(42754)=(S=12139876,E=12139879,C=28)
	CT(42755)=(S=12139880,E=12139883,C=68)
	CT(42756)=(S=12139884,E=12139887,C=19)
	CT(42757)=(S=12139888,E=12139891,C=14)
	CT(42758)=(S=12139892,E=12139895,C=12)
	CT(42759)=(S=12139896,E=12139899,C=28)
	CT(42760)=(S=12139900,E=12139903,C=25)
	CT(42761)=(S=12139904,E=12139907,C=18)
	CT(42762)=(S=12139908,E=12139911,C=14)
	CT(42763)=(S=12139912,E=12139915,C=31)
	CT(42764)=(S=12139916,E=12139919,C=14)
	CT(42765)=(S=12139920,E=12139923,C=12)
	CT(42766)=(S=12139924,E=12139927,C=60)
	CT(42767)=(S=12139928,E=12139931,C=14)
	CT(42768)=(S=12139932,E=12139935,C=31)
	CT(42769)=(S=12139936,E=12139939,C=12)
	CT(42770)=(S=12139940,E=12139943,C=14)
	CT(42771)=(S=12139944,E=12139947,C=15)
	CT(42772)=(S=12139948,E=12139951,C=18)
	CT(42773)=(S=12139952,E=12139955,C=12)
	CT(42774)=(S=12139956,E=12139959,C=15)
	CT(42775)=(S=12139960,E=12139967,C=12)
	CT(42776)=(S=12139968,E=12139971,C=18)
	CT(42777)=(S=12139972,E=12139975,C=36)
	CT(42778)=(S=12139976,E=12139979,C=43)
	CT(42779)=(S=12139980,E=12139983,C=25)
	CT(42780)=(S=12139984,E=12139991,C=14)
	CT(42781)=(S=12139992,E=12139999,C=18)
	CT(42782)=(S=12140000,E=12140003,C=36)
	CT(42783)=(S=12140004,E=12140007,C=33)
	CT(42784)=(S=12140008,E=12140011,C=47)
	CT(42785)=(S=12140012,E=12140015,C=19)
	CT(42786)=(S=12140016,E=12140019,C=32)
	CT(42787)=(S=12140020,E=12140023,C=58)
	CT(42788)=(S=12140024,E=12140027,C=13)
	CT(42789)=(S=12140028,E=12140031,C=18)
	CT(42790)=(S=12140032,E=12140035,C=15)
	CT(42791)=(S=12140036,E=12140039,C=49)
	CT(42792)=(S=12140040,E=12140043,C=33)
	CT(42793)=(S=12140044,E=12140051,C=36)
	CT(42794)=(S=12140052,E=12140055,C=25)
	CT(42795)=(S=12140056,E=12140059,C=28)
	CT(42796)=(S=12140060,E=12140063,C=15)
	CT(42797)=(S=12140064,E=12140067,C=100)
	CT(42798)=(S=12140068,E=12140071,C=72)
	CT(42799)=(S=12140072,E=12140075,C=12)
	CT(42800)=(S=12140076,E=12140079,C=14)
	CT(42801)=(S=12140080,E=12140087,C=15)
	CT(42802)=(S=12140088,E=12140091,C=36)
	CT(42803)=(S=12140092,E=12140095,C=48)
	CT(42804)=(S=12140096,E=12140099,C=28)
	CT(42805)=(S=12140100,E=12140103,C=36)
	CT(42806)=(S=12140104,E=12140107,C=57)
	CT(42807)=(S=12140108,E=12140111,C=14)
	CT(42808)=(S=12140112,E=12140115,C=34)
	CT(42809)=(S=12140116,E=12140119,C=14)
	CT(42810)=(S=12140120,E=12140123,C=34)
	CT(42811)=(S=12140124,E=12140127,C=58)
	CT(42812)=(S=12140128,E=12140131,C=60)
	CT(42813)=(S=12140132,E=12140135,C=18)
	CT(42814)=(S=12140136,E=12140139,C=34)
	CT(42815)=(S=12140140,E=12140143,C=33)
	CT(42816)=(S=12140144,E=12140147,C=46)
	CT(42817)=(S=12140148,E=12140151,C=36)
	CT(42818)=(S=12140152,E=12140155,C=17)
	CT(42819)=(S=12140156,E=12140159,C=28)
	CT(42820)=(S=12140160,E=12140163,C=50)
	CT(42821)=(S=12140164,E=12140167,C=46)
	CT(42822)=(S=12140168,E=12140171,C=29)
	CT(42823)=(S=12140172,E=12140175,C=27)
	CT(42824)=(S=12140176,E=12140183,C=28)
	CT(42825)=(S=12140184,E=12140191,C=14)
	CT(42826)=(S=12140192,E=12140195,C=27)
	CT(42827)=(S=12140196,E=12140199,C=63)
	CT(42828)=(S=12140200,E=12140203,C=34)
	CT(42829)=(S=12140204,E=12140207,C=96)
	CT(42830)=(S=12140208,E=12140211,C=23)
	CT(42831)=(S=12140212,E=12140215,C=58)
	CT(42832)=(S=12140216,E=12140219,C=12)
	CT(42833)=(S=12140220,E=12140223,C=36)
	CT(42834)=(S=12140224,E=12140227,C=18)
	CT(42835)=(S=12140228,E=12140231,C=60)
	CT(42836)=(S=12140232,E=12140235,C=18)
	CT(42837)=(S=12140236,E=12140239,C=19)
	CT(42838)=(S=12140240,E=12140243,C=36)
	CT(42839)=(S=12140244,E=12140251,C=14)
	CT(42840)=(S=12140252,E=12140263,C=12)
	CT(42841)=(S=12140264,E=12140267,C=26)
	CT(42842)=(S=12140268,E=12140271,C=51)
	CT(42843)=(S=12140272,E=12140275,C=15)
	CT(42844)=(S=12140276,E=12140279,C=28)
	CT(42845)=(S=12140280,E=12140283,C=15)
	CT(42846)=(S=12140284,E=12140287,C=19)
	CT(42847)=(S=12140288,E=12140291,C=42)
	CT(42848)=(S=12140292,E=12140295,C=38)
	CT(42849)=(S=12140296,E=12140299,C=28)
	CT(42850)=(S=12140300,E=12140303,C=15)
	CT(42851)=(S=12140304,E=12140307,C=50)
	CT(42852)=(S=12140308,E=12140311,C=36)
	CT(42853)=(S=12140312,E=12140315,C=33)
	CT(42854)=(S=12140316,E=12140319,C=25)
	CT(42855)=(S=12140320,E=12140323,C=31)
	CT(42856)=(S=12140324,E=12140327,C=34)
	CT(42857)=(S=12140328,E=12140331,C=15)
	CT(42858)=(S=12140332,E=12140335,C=50)
	CT(42859)=(S=12140336,E=12140339,C=12)
	CT(42860)=(S=12140340,E=12140343,C=15)
	CT(42861)=(S=12140344,E=12140347,C=36)
	CT(42862)=(S=12140348,E=12140351,C=18)
	CT(42863)=(S=12140352,E=12140355,C=34)
	CT(42864)=(S=12140356,E=12140359,C=36)
	CT(42865)=(S=12140360,E=12140363,C=72)
	CT(42866)=(S=12140364,E=12140367,C=13)
	CT(42867)=(S=12140368,E=12140371,C=14)
	CT(42868)=(S=12140372,E=12140375,C=58)
	CT(42869)=(S=12140376,E=12140379,C=15)
	CT(42870)=(S=12140380,E=12140383,C=12)
	CT(42871)=(S=12140384,E=12140387,C=33)
	CT(42872)=(S=12140388,E=12140391,C=15)
	CT(42873)=(S=12140392,E=12140395,C=33)
	CT(42874)=(S=12140396,E=12140399,C=58)
	CT(42875)=(S=12140400,E=12140403,C=18)
	CT(42876)=(S=12140404,E=12140407,C=14)
	CT(42877)=(S=12140408,E=12140411,C=12)
	CT(42878)=(S=12140412,E=12140419,C=28)
	CT(42879)=(S=12140420,E=12140423,C=15)
	CT(42880)=(S=12140424,E=12140427,C=38)
	CT(42881)=(S=12140428,E=12140431,C=14)
	CT(42882)=(S=12140432,E=12140435,C=29)
	CT(42883)=(S=12140436,E=12140439,C=34)
	CT(42884)=(S=12140440,E=12140443,C=36)
	CT(42885)=(S=12140444,E=12140447,C=71)
	CT(42886)=(S=12140448,E=12140451,C=15)
	CT(42887)=(S=12140452,E=12140455,C=36)
	CT(42888)=(S=12140456,E=12140459,C=15)
	CT(42889)=(S=12140460,E=12140463,C=12)
	CT(42890)=(S=12140464,E=12140467,C=96)
	CT(42891)=(S=12140468,E=12140471,C=21)
	CT(42892)=(S=12140472,E=12140475,C=14)
	CT(42893)=(S=12140476,E=12140479,C=18)
	CT(42894)=(S=12140480,E=12140483,C=12)
	CT(42895)=(S=12140484,E=12140487,C=18)
	CT(42896)=(S=12140488,E=12140491,C=58)
	CT(42897)=(S=12140492,E=12140495,C=28)
	CT(42898)=(S=12140496,E=12140499,C=27)
	CT(42899)=(S=12140500,E=12140503,C=28)
	CT(42900)=(S=12140504,E=12140507,C=18)
	CT(42901)=(S=12140508,E=12140511,C=14)
	CT(42902)=(S=12140512,E=12140515,C=98)
	CT(42903)=(S=12140516,E=12140519,C=15)
	CT(42904)=(S=12140520,E=12140523,C=23)
	CT(42905)=(S=12140524,E=12140527,C=26)
	CT(42906)=(S=12140532,E=12140535,C=36)
	CT(42907)=(S=12140536,E=12140539,C=42)
	CT(42908)=(S=12140540,E=12140543,C=4)
	CT(42909)=(S=12140544,E=12140547,C=102)
	CT(42910)=(S=12140548,E=12140551,C=49)
	CT(42911)=(S=12140552,E=12140555,C=17)
	CT(42912)=(S=12140556,E=12140559,C=14)
	CT(42913)=(S=12140560,E=12140563,C=63)
	CT(42914)=(S=12140564,E=12140567,C=51)
	CT(42915)=(S=12140568,E=12140571,C=45)
	CT(42916)=(S=12140572,E=12140575,C=58)
	CT(42917)=(S=12140576,E=12140579,C=36)
	CT(42918)=(S=12140580,E=12140583,C=14)
	CT(42919)=(S=12140584,E=12140587,C=33)
	CT(42920)=(S=12140588,E=12140591,C=18)
	CT(42921)=(S=12140592,E=12140595,C=42)
	CT(42922)=(S=12140596,E=12140599,C=18)
	CT(42923)=(S=12140600,E=12140603,C=72)
	CT(42924)=(S=12140604,E=12140607,C=42)
	CT(42925)=(S=12140608,E=12140611,C=49)
	CT(42926)=(S=12140612,E=12140615,C=15)
	CT(42927)=(S=12140616,E=12140619,C=19)
	CT(42928)=(S=12140620,E=12140623,C=16)
	CT(42929)=(S=12140624,E=12140627,C=37)
	CT(42930)=(S=12140628,E=12140631,C=54)
	CT(42931)=(S=12140632,E=12140635,C=14)
	CT(42932)=(S=12140636,E=12140639,C=38)
	CT(42933)=(S=12140640,E=12140643,C=28)
	CT(42934)=(S=12140644,E=12140647,C=18)
	CT(42935)=(S=12140648,E=12140651,C=44)
	CT(42936)=(S=12140652,E=12140655,C=36)
	CT(42937)=(S=12140656,E=12140659,C=28)
	CT(42938)=(S=12140660,E=12140663,C=34)
	CT(42939)=(S=12140664,E=12140667,C=36)
	CT(42940)=(S=12140668,E=12140671,C=28)
	CT(42941)=(S=12140672,E=12140675,C=12)
	CT(42942)=(S=12140676,E=12140679,C=34)
	CT(42943)=(S=12140680,E=12140683,C=36)
	CT(42944)=(S=12140684,E=12140687,C=63)
	CT(42945)=(S=12140688,E=12140691,C=13)
	CT(42946)=(S=12140692,E=12140695,C=12)
	CT(42947)=(S=12140696,E=12140699,C=42)
	CT(42948)=(S=12140700,E=12140703,C=28)
	CT(42949)=(S=12140704,E=12140707,C=14)
	CT(42950)=(S=12140708,E=12140711,C=42)
	CT(42951)=(S=12140712,E=12140715,C=28)
	CT(42952)=(S=12140716,E=12140719,C=46)
	CT(42953)=(S=12140720,E=12140723,C=26)
	CT(42954)=(S=12140724,E=12140727,C=49)
	CT(42955)=(S=12140728,E=12140731,C=34)
	CT(42956)=(S=12140732,E=12140735,C=14)
	CT(42957)=(S=12140736,E=12140739,C=36)
	CT(42958)=(S=12140740,E=12140743,C=34)
	CT(42959)=(S=12140744,E=12140747,C=101)
	CT(42960)=(S=12140748,E=12140751,C=15)
	CT(42961)=(S=12140752,E=12140755,C=18)
	CT(42962)=(S=12140756,E=12140759,C=14)
	CT(42963)=(S=12140760,E=12140767,C=33)
	CT(42964)=(S=12140768,E=12140771,C=14)
	CT(42965)=(S=12140772,E=12140775,C=18)
	CT(42966)=(S=12140776,E=12140779,C=25)
	CT(42967)=(S=12140780,E=12140783,C=53)
	CT(42968)=(S=12140784,E=12140787,C=25)
	CT(42969)=(S=12140788,E=12140791,C=28)
	CT(42970)=(S=12140792,E=12140799,C=14)
	CT(42971)=(S=12140800,E=12140803,C=70)
	CT(42972)=(S=12140804,E=12140807,C=28)
	CT(42973)=(S=12140808,E=12140811,C=50)
	CT(42974)=(S=12140812,E=12140823,C=18)
	CT(42975)=(S=12140824,E=12140827,C=14)
	CT(42976)=(S=12140828,E=12140831,C=12)
	CT(42977)=(S=12140836,E=12140839,C=60)
	CT(42978)=(S=12140840,E=12140843,C=14)
	CT(42979)=(S=12140844,E=12140847,C=15)
	CT(42980)=(S=12140848,E=12140851,C=44)
	CT(42981)=(S=12140852,E=12140855,C=36)
	CT(42982)=(S=12140856,E=12140859,C=12)
	CT(42983)=(S=12140860,E=12140863,C=34)
	CT(42984)=(S=12140864,E=12140867,C=25)
	CT(42985)=(S=12140868,E=12140871,C=37)
	CT(42986)=(S=12140872,E=12140875,C=49)
	CT(42987)=(S=12140876,E=12140879,C=33)
	CT(42988)=(S=12140880,E=12140883,C=12)
	CT(42989)=(S=12140884,E=12140887,C=14)
	CT(42990)=(S=12140888,E=12140891,C=42)
	CT(42991)=(S=12140892,E=12140895,C=33)
	CT(42992)=(S=12140896,E=12140899,C=12)
	CT(42993)=(S=12140900,E=12140911,C=14)
	CT(42994)=(S=12140912,E=12140915,C=12)
	CT(42995)=(S=12140916,E=12140919,C=24)
	CT(42996)=(S=12140920,E=12140923,C=59)
	CT(42997)=(S=12140924,E=12140927,C=36)
	CT(42998)=(S=12140928,E=12140931,C=14)
	CT(42999)=(S=12140932,E=12140935,C=19)
	CT(43000)=(S=12140936,E=12140939,C=18)
	CT(43001)=(S=12140940,E=12140943,C=19)
	CT(43002)=(S=12140944,E=12140947,C=14)
	CT(43003)=(S=12140948,E=12140951,C=18)
	CT(43004)=(S=12140952,E=12140955,C=27)
	CT(43005)=(S=12140956,E=12140959,C=25)
	CT(43006)=(S=12140960,E=12140963,C=60)
	CT(43007)=(S=12140964,E=12140967,C=14)
	CT(43008)=(S=12140968,E=12140971,C=34)
	CT(43009)=(S=12140972,E=12140975,C=36)
	CT(43010)=(S=12140976,E=12140979,C=19)
	CT(43011)=(S=12140980,E=12140983,C=49)
	CT(43012)=(S=12140984,E=12140987,C=25)
	CT(43013)=(S=12140988,E=12140991,C=19)
	CT(43014)=(S=12140992,E=12140999,C=28)
	CT(43015)=(S=12141000,E=12141003,C=18)
	CT(43016)=(S=12141004,E=12141007,C=37)
	CT(43017)=(S=12141008,E=12141011,C=28)
	CT(43018)=(S=12141012,E=12141015,C=36)
	CT(43019)=(S=12141016,E=12141019,C=15)
	CT(43020)=(S=12141020,E=12141023,C=71)
	CT(43021)=(S=12141024,E=12141027,C=12)
	CT(43022)=(S=12141028,E=12141031,C=52)
	CT(43023)=(S=12141032,E=12141035,C=42)
	CT(43024)=(S=12141036,E=12141039,C=14)
	CT(43025)=(S=12141040,E=12141043,C=28)
	CT(43026)=(S=12141044,E=12141047,C=31)
	CT(43027)=(S=12141048,E=12141051,C=12)
	CT(43028)=(S=12141052,E=12141055,C=35)
	CT(43029)=(S=12141056,E=12141059,C=24)
	CT(43030)=(S=12141060,E=12141063,C=14)
	CT(43031)=(S=12141064,E=12141067,C=31)
	CT(43032)=(S=12141068,E=12141071,C=18)
	CT(43033)=(S=12141072,E=12141075,C=16)
	CT(43034)=(S=12141076,E=12141079,C=14)
	CT(43035)=(S=12141080,E=12141083,C=15)
	CT(43036)=(S=12141084,E=12141087,C=18)
	CT(43037)=(S=12141088,E=12141091,C=14)
	CT(43038)=(S=12141092,E=12141095,C=33)
	CT(43039)=(S=12141096,E=12141103,C=25)
	CT(43040)=(S=12141104,E=12141107,C=42)
	CT(43041)=(S=12141108,E=12141111,C=50)
	CT(43042)=(S=12141112,E=12141115,C=25)
	CT(43043)=(S=12141116,E=12141119,C=34)
	CT(43044)=(S=12141120,E=12141123,C=28)
	CT(43045)=(S=12141124,E=12141127,C=68)
	CT(43046)=(S=12141128,E=12141131,C=25)
	CT(43047)=(S=12141132,E=12141135,C=28)
	CT(43048)=(S=12141136,E=12141139,C=36)
	CT(43049)=(S=12141140,E=12141143,C=18)
	CT(43050)=(S=12141144,E=12141147,C=14)
	CT(43051)=(S=12141148,E=12141151,C=36)
	CT(43052)=(S=12141152,E=12141155,C=70)
	CT(43053)=(S=12141156,E=12141159,C=12)
	CT(43054)=(S=12141160,E=12141163,C=49)
	CT(43055)=(S=12141164,E=12141167,C=28)
	CT(43056)=(S=12141168,E=12141171,C=14)
	CT(43057)=(S=12141172,E=12141175,C=16)
	CT(43058)=(S=12141176,E=12141179,C=58)
	CT(43059)=(S=12141180,E=12141183,C=37)
	CT(43060)=(S=12141184,E=12141187,C=96)
	CT(43061)=(S=12141188,E=12141191,C=28)
	CT(43062)=(S=12141192,E=12141195,C=38)
	CT(43063)=(S=12141196,E=12141199,C=36)
	CT(43064)=(S=12141200,E=12141203,C=19)
	CT(43065)=(S=12141204,E=12141207,C=52)
	CT(43066)=(S=12141208,E=12141211,C=12)
	CT(43067)=(S=12141212,E=12141215,C=34)
	CT(43068)=(S=12141216,E=12141219,C=32)
	CT(43069)=(S=12141220,E=12141223,C=14)
	CT(43070)=(S=12141224,E=12141227,C=36)
	CT(43071)=(S=12141228,E=12141231,C=25)
	CT(43072)=(S=12141232,E=12141235,C=28)
	CT(43073)=(S=12141236,E=12141239,C=14)
	CT(43074)=(S=12141240,E=12141243,C=24)
	CT(43075)=(S=12141244,E=12141247,C=33)
	CT(43076)=(S=12141248,E=12141255,C=28)
	CT(43077)=(S=12141256,E=12141259,C=71)
	CT(43078)=(S=12141260,E=12141267,C=14)
	CT(43079)=(S=12141268,E=12141271,C=58)
	CT(43080)=(S=12141272,E=12141275,C=18)
	CT(43081)=(S=12141276,E=12141279,C=28)
	CT(43082)=(S=12141280,E=12141287,C=26)
	CT(43083)=(S=12141288,E=12141291,C=12)
	CT(43084)=(S=12141292,E=12141295,C=28)
	CT(43085)=(S=12141296,E=12141299,C=14)
	CT(43086)=(S=12141300,E=12141303,C=28)
	CT(43087)=(S=12141304,E=12141307,C=36)
	CT(43088)=(S=12141308,E=12141311,C=62)
	CT(43089)=(S=12141312,E=12141315,C=31)
	CT(43090)=(S=12141316,E=12141319,C=36)
	CT(43091)=(S=12141320,E=12141323,C=58)
	CT(43092)=(S=12141324,E=12141327,C=26)
	CT(43093)=(S=12141328,E=12141331,C=27)
	CT(43094)=(S=12141332,E=12141335,C=63)
	CT(43095)=(S=12141336,E=12141339,C=28)
	CT(43096)=(S=12141340,E=12141343,C=25)
	CT(43097)=(S=12141344,E=12141347,C=37)
	CT(43098)=(S=12141348,E=12141351,C=28)
	CT(43099)=(S=12141352,E=12141355,C=27)
	CT(43100)=(S=12141356,E=12141359,C=36)
	CT(43101)=(S=12141360,E=12141363,C=14)
	CT(43102)=(S=12141364,E=12141367,C=18)
	CT(43103)=(S=12141368,E=12141371,C=19)
	CT(43104)=(S=12141372,E=12141379,C=36)
	CT(43105)=(S=12141380,E=12141383,C=58)
	CT(43106)=(S=12141384,E=12141387,C=12)
	CT(43107)=(S=12141388,E=12141391,C=25)
	CT(43108)=(S=12141392,E=12141395,C=47)
	CT(43109)=(S=12141396,E=12141399,C=48)
	CT(43110)=(S=12141400,E=12141403,C=12)
	CT(43111)=(S=12141404,E=12141407,C=18)
	CT(43112)=(S=12141408,E=12141411,C=25)
	CT(43113)=(S=12141412,E=12141415,C=26)
	CT(43114)=(S=12141416,E=12141419,C=25)
	CT(43115)=(S=12141420,E=12141423,C=42)
	CT(43116)=(S=12141424,E=12141427,C=25)
	CT(43117)=(S=12141428,E=12141431,C=27)
	CT(43118)=(S=12141432,E=12141439,C=37)
	CT(43119)=(S=12141440,E=12141443,C=24)
	CT(43120)=(S=12141444,E=12141447,C=33)
	CT(43121)=(S=12141448,E=12141451,C=36)
	CT(43122)=(S=12141452,E=12141455,C=14)
	CT(43123)=(S=12141456,E=12141467,C=12)
	CT(43124)=(S=12141468,E=12141471,C=28)
	CT(43125)=(S=12141472,E=12141475,C=18)
	CT(43126)=(S=12141476,E=12141479,C=14)
	CT(43127)=(S=12141480,E=12141483,C=18)
	CT(43128)=(S=12141484,E=12141487,C=42)
	CT(43129)=(S=12141488,E=12141491,C=96)
	CT(43130)=(S=12141492,E=12141495,C=48)
	CT(43131)=(S=12141496,E=12141499,C=14)
	CT(43132)=(S=12141500,E=12141503,C=33)
	CT(43133)=(S=12141504,E=12141511,C=34)
	CT(43134)=(S=12141512,E=12141515,C=36)
	CT(43135)=(S=12141516,E=12141519,C=37)
	CT(43136)=(S=12141520,E=12141523,C=25)
	CT(43137)=(S=12141524,E=12141527,C=26)
	CT(43138)=(S=12141528,E=12141531,C=58)
	CT(43139)=(S=12141532,E=12141535,C=28)
	CT(43140)=(S=12141536,E=12141539,C=36)
	CT(43141)=(S=12141540,E=12141543,C=42)
	CT(43142)=(S=12141544,E=12141547,C=14)
	CT(43143)=(S=12141548,E=12141551,C=15)
	CT(43144)=(S=12141552,E=12141555,C=44)
	CT(43145)=(S=12141556,E=12141559,C=18)
	CT(43146)=(S=12141560,E=12141563,C=14)
	CT(43147)=(S=12141564,E=12141567,C=33)
	CT(43148)=(S=12141568,E=12141571,C=14)
	CT(43149)=(S=12141572,E=12141575,C=42)
	CT(43150)=(S=12141576,E=12141579,C=40)
	CT(43151)=(S=12141580,E=12141583,C=58)
	CT(43152)=(S=12141584,E=12141587,C=31)
	CT(43153)=(S=12141588,E=12141591,C=18)
	CT(43154)=(S=12141592,E=12141595,C=58)
	CT(43155)=(S=12141596,E=12141599,C=33)
	CT(43156)=(S=12141600,E=12141603,C=34)
	CT(43157)=(S=12141604,E=12141607,C=44)
	CT(43158)=(S=12141608,E=12141611,C=28)
	CT(43159)=(S=12141612,E=12141615,C=70)
	CT(43160)=(S=12141616,E=12141619,C=102)
	CT(43161)=(S=12141620,E=12141623,C=34)
	CT(43162)=(S=12141624,E=12141627,C=47)
	CT(43163)=(S=12141628,E=12141631,C=46)
	CT(43164)=(S=12141632,E=12141635,C=34)
	CT(43165)=(S=12141636,E=12141639,C=33)
	CT(43166)=(S=12141640,E=12141643,C=50)
	CT(43167)=(S=12141644,E=12141647,C=28)
	CT(43168)=(S=12141648,E=12141651,C=16)
	CT(43169)=(S=12141652,E=12141655,C=14)
	CT(43170)=(S=12141656,E=12141659,C=36)
	CT(43171)=(S=12141660,E=12141663,C=18)
	CT(43172)=(S=12141664,E=12141667,C=36)
	CT(43173)=(S=12141668,E=12141671,C=18)
	CT(43174)=(S=12141672,E=12141675,C=14)
	CT(43175)=(S=12141676,E=12141679,C=25)
	CT(43176)=(S=12141680,E=12141683,C=14)
	CT(43177)=(S=12141684,E=12141687,C=18)
	CT(43178)=(S=12141688,E=12141691,C=38)
	CT(43179)=(S=12141692,E=12141695,C=28)
	CT(43180)=(S=12141696,E=12141699,C=32)
	CT(43181)=(S=12141700,E=12141703,C=57)
	CT(43182)=(S=12141704,E=12141707,C=28)
	CT(43183)=(S=12141708,E=12141711,C=14)
	CT(43184)=(S=12141712,E=12141715,C=18)
	CT(43185)=(S=12141716,E=12141719,C=12)
	CT(43186)=(S=12141720,E=12141723,C=18)
	CT(43187)=(S=12141724,E=12141727,C=28)
	CT(43188)=(S=12141728,E=12141731,C=36)
	CT(43189)=(S=12141732,E=12141735,C=33)
	CT(43190)=(S=12141736,E=12141739,C=27)
	CT(43191)=(S=12141740,E=12141743,C=36)
	CT(43192)=(S=12141744,E=12141747,C=15)
	CT(43193)=(S=12141748,E=12141751,C=36)
	CT(43194)=(S=12141752,E=12141755,C=18)
	CT(43195)=(S=12141756,E=12141759,C=38)
	CT(43196)=(S=12141760,E=12141763,C=15)
	CT(43197)=(S=12141764,E=12141767,C=36)
	CT(43198)=(S=12141768,E=12141771,C=14)
	CT(43199)=(S=12141772,E=12141775,C=34)
	CT(43200)=(S=12141776,E=12141779,C=55)
	CT(43201)=(S=12141780,E=12141787,C=33)
	CT(43202)=(S=12141788,E=12141791,C=37)
	CT(43203)=(S=12141792,E=12141799,C=12)
	CT(43204)=(S=12141800,E=12141803,C=14)
	CT(43205)=(S=12141804,E=12141807,C=34)
	CT(43206)=(S=12141808,E=12141811,C=36)
	CT(43207)=(S=12141812,E=12141815,C=18)
	CT(43208)=(S=12141816,E=12141819,C=14)
	CT(43209)=(S=12141820,E=12141823,C=28)
	CT(43210)=(S=12141824,E=12141827,C=58)
	CT(43211)=(S=12141828,E=12141831,C=35)
	CT(43212)=(S=12141832,E=12141835,C=25)
	CT(43213)=(S=12141836,E=12141843,C=14)
	CT(43214)=(S=12141844,E=12141847,C=58)
	CT(43215)=(S=12141848,E=12141851,C=60)
	CT(43216)=(S=12141852,E=12141855,C=59)
	CT(43217)=(S=12141856,E=12141859,C=36)
	CT(43218)=(S=12141860,E=12141863,C=47)
	CT(43219)=(S=12141864,E=12141867,C=34)
	CT(43220)=(S=12141868,E=12141871,C=19)
	CT(43221)=(S=12141872,E=12141875,C=71)
	CT(43222)=(S=12141876,E=12141879,C=44)
	CT(43223)=(S=12141880,E=12141883,C=38)
	CT(43224)=(S=12141884,E=12141887,C=36)
	CT(43225)=(S=12141888,E=12141891,C=14)
	CT(43226)=(S=12141892,E=12141895,C=33)
	CT(43227)=(S=12141896,E=12141899,C=47)
	CT(43228)=(S=12141900,E=12141903,C=12)
	CT(43229)=(S=12141904,E=12141907,C=18)
	CT(43230)=(S=12141908,E=12141911,C=14)
	CT(43231)=(S=12141912,E=12141915,C=36)
	CT(43232)=(S=12141916,E=12141919,C=34)
	CT(43233)=(S=12141920,E=12141923,C=50)
	CT(43234)=(S=12141924,E=12141927,C=36)
	CT(43235)=(S=12141928,E=12141931,C=71)
	CT(43236)=(S=12141932,E=12141935,C=26)
	CT(43237)=(S=12141936,E=12141939,C=36)
	CT(43238)=(S=12141940,E=12141943,C=14)
	CT(43239)=(S=12141944,E=12141947,C=26)
	CT(43240)=(S=12141948,E=12141951,C=18)
	CT(43241)=(S=12141952,E=12141955,C=36)
	CT(43242)=(S=12141956,E=12141959,C=18)
	CT(43243)=(S=12141960,E=12141963,C=34)
	CT(43244)=(S=12141964,E=12141967,C=47)
	CT(43245)=(S=12141968,E=12141971,C=14)
	CT(43246)=(S=12141972,E=12141975,C=52)
	CT(43247)=(S=12141976,E=12141979,C=31)
	CT(43248)=(S=12141980,E=12141983,C=36)
	CT(43249)=(S=12141984,E=12141987,C=42)
	CT(43250)=(S=12141988,E=12141991,C=49)
	CT(43251)=(S=12141992,E=12141995,C=19)
	CT(43252)=(S=12141996,E=12141999,C=40)
	CT(43253)=(S=12142000,E=12142007,C=28)
	CT(43254)=(S=12142008,E=12142011,C=190)
	CT(43255)=(S=12142012,E=12142015,C=15)
	CT(43256)=(S=12142016,E=12142023,C=58)
	CT(43257)=(S=12142024,E=12142027,C=28)
	CT(43258)=(S=12142028,E=12142031,C=36)
	CT(43259)=(S=12142032,E=12142035,C=23)
	CT(43260)=(S=12142036,E=12142039,C=63)
	CT(43261)=(S=12142040,E=12142043,C=70)
	CT(43262)=(S=12142044,E=12142047,C=18)
	CT(43263)=(S=12142048,E=12142051,C=70)
	CT(43264)=(S=12142052,E=12142055,C=12)
	CT(43265)=(S=12142056,E=12142059,C=36)
	CT(43266)=(S=12142060,E=12142063,C=42)
	CT(43267)=(S=12142064,E=12142067,C=38)
	CT(43268)=(S=12142068,E=12142071,C=42)
	CT(43269)=(S=12142072,E=12142075,C=12)
	CT(43270)=(S=12142076,E=12142079,C=25)
	CT(43271)=(S=12142080,E=12142083,C=71)
	CT(43272)=(S=12142084,E=12142087,C=36)
	CT(43273)=(S=12142088,E=12142091,C=14)
	CT(43274)=(S=12142092,E=12142095,C=68)
	CT(43275)=(S=12142096,E=12142099,C=31)
	CT(43276)=(S=12142100,E=12142103,C=28)
	CT(43277)=(S=12142104,E=12142107,C=33)
	CT(43278)=(S=12142108,E=12142111,C=63)
	CT(43279)=(S=12142112,E=12142115,C=14)
	CT(43280)=(S=12142116,E=12142119,C=58)
	CT(43281)=(S=12142120,E=12142123,C=34)
	CT(43282)=(S=12142124,E=12142127,C=49)
	CT(43283)=(S=12142128,E=12142131,C=34)
	CT(43284)=(S=12142132,E=12142135,C=40)
	CT(43285)=(S=12142136,E=12142139,C=14)
	CT(43286)=(S=12142140,E=12142143,C=26)
	CT(43287)=(S=12142144,E=12142147,C=36)
	CT(43288)=(S=12142148,E=12142151,C=34)
	CT(43289)=(S=12142152,E=12142155,C=49)
	CT(43290)=(S=12142156,E=12142159,C=22)
	CT(43291)=(S=12142160,E=12142163,C=56)
	CT(43292)=(S=12142164,E=12142167,C=37)
	CT(43293)=(S=12142168,E=12142171,C=14)
	CT(43294)=(S=12142172,E=12142175,C=25)
	CT(43295)=(S=12142176,E=12142179,C=37)
	CT(43296)=(S=12142180,E=12142183,C=21)
	CT(43297)=(S=12142184,E=12142187,C=18)
	CT(43298)=(S=12142188,E=12142191,C=31)
	CT(43299)=(S=12142192,E=12142195,C=36)
	CT(43300)=(S=12142196,E=12142199,C=20)
	CT(43301)=(S=12142200,E=12142203,C=28)
	CT(43302)=(S=12142204,E=12142207,C=19)
	CT(43303)=(S=12142208,E=12142211,C=31)
	CT(43304)=(S=12142212,E=12142215,C=14)
	CT(43305)=(S=12142216,E=12142219,C=25)
	CT(43306)=(S=12142220,E=12142223,C=37)
	CT(43307)=(S=12142224,E=12142227,C=28)
	CT(43308)=(S=12142228,E=12142231,C=15)
	CT(43309)=(S=12142232,E=12142235,C=25)
	CT(43310)=(S=12142236,E=12142243,C=14)
	CT(43311)=(S=12142244,E=12142247,C=18)
	CT(43312)=(S=12142248,E=12142251,C=14)
	CT(43313)=(S=12142252,E=12142255,C=25)
	CT(43314)=(S=12142256,E=12142259,C=12)
	CT(43315)=(S=12142260,E=12142263,C=58)
	CT(43316)=(S=12142264,E=12142267,C=36)
	CT(43317)=(S=12142268,E=12142271,C=53)
	CT(43318)=(S=12142272,E=12142275,C=14)
	CT(43319)=(S=12142276,E=12142279,C=26)
	CT(43320)=(S=12142280,E=12142283,C=15)
	CT(43321)=(S=12142284,E=12142287,C=28)
	CT(43322)=(S=12142288,E=12142295,C=36)
	CT(43323)=(S=12142296,E=12142299,C=33)
	CT(43324)=(S=12142300,E=12142303,C=28)
	CT(43325)=(S=12142304,E=12142307,C=39)
	CT(43326)=(S=12142308,E=12142319,C=36)
	CT(43327)=(S=12142320,E=12142323,C=14)
	CT(43328)=(S=12142324,E=12142327,C=34)
	CT(43329)=(S=12142328,E=12142331,C=17)
	CT(43330)=(S=12142332,E=12142335,C=14)
	CT(43331)=(S=12142336,E=12142339,C=31)
	CT(43332)=(S=12142340,E=12142343,C=28)
	CT(43333)=(S=12142344,E=12142347,C=15)
	CT(43334)=(S=12142348,E=12142355,C=34)
	CT(43335)=(S=12142356,E=12142359,C=58)
	CT(43336)=(S=12142360,E=12142363,C=14)
	CT(43337)=(S=12142364,E=12142367,C=25)
	CT(43338)=(S=12142368,E=12142371,C=28)
	CT(43339)=(S=12142372,E=12142375,C=14)
	CT(43340)=(S=12142376,E=12142379,C=33)
	CT(43341)=(S=12142380,E=12142383,C=15)
	CT(43342)=(S=12142384,E=12142387,C=14)
	CT(43343)=(S=12142388,E=12142391,C=18)
	CT(43344)=(S=12142392,E=12142395,C=70)
	CT(43345)=(S=12142396,E=12142399,C=36)
	CT(43346)=(S=12142400,E=12142407,C=18)
	CT(43347)=(S=12142408,E=12142411,C=14)
	CT(43348)=(S=12142412,E=12142419,C=18)
	CT(43349)=(S=12142420,E=12142423,C=28)
	CT(43350)=(S=12142424,E=12142427,C=38)
	CT(43351)=(S=12142428,E=12142431,C=14)
	CT(43352)=(S=12142432,E=12142435,C=18)
	CT(43353)=(S=12142436,E=12142439,C=14)
	CT(43354)=(S=12142440,E=12142443,C=50)
	CT(43355)=(S=12142444,E=12142447,C=14)
	CT(43356)=(S=12142448,E=12142451,C=34)
	CT(43357)=(S=12142452,E=12142455,C=37)
	CT(43358)=(S=12142456,E=12142463,C=28)
	CT(43359)=(S=12142464,E=12142467,C=60)
	CT(43360)=(S=12142468,E=12142471,C=67)
	CT(43361)=(S=12142472,E=12142475,C=14)
	CT(43362)=(S=12142476,E=12142479,C=43)
	CT(43363)=(S=12142480,E=12142483,C=18)
	CT(43364)=(S=12142484,E=12142487,C=12)
	CT(43365)=(S=12142488,E=12142491,C=26)
	CT(43366)=(S=12142492,E=12142495,C=12)
	CT(43367)=(S=12142496,E=12142499,C=63)
	CT(43368)=(S=12142500,E=12142503,C=42)
	CT(43369)=(S=12142504,E=12142507,C=28)
	CT(43370)=(S=12142508,E=12142511,C=52)
	CT(43371)=(S=12142512,E=12142515,C=19)
	CT(43372)=(S=12142516,E=12142519,C=96)
	CT(43373)=(S=12142520,E=12142523,C=14)
	CT(43374)=(S=12142524,E=12142527,C=12)
	CT(43375)=(S=12142528,E=12142531,C=26)
	CT(43376)=(S=12142532,E=12142535,C=18)
	CT(43377)=(S=12142536,E=12142539,C=25)
	CT(43378)=(S=12142540,E=12142543,C=16)
	CT(43379)=(S=12142544,E=12142547,C=27)
	CT(43380)=(S=12142548,E=12142551,C=18)
	CT(43381)=(S=12142552,E=12142555,C=32)
	CT(43382)=(S=12142556,E=12142563,C=36)
	CT(43383)=(S=12142564,E=12142567,C=28)
	CT(43384)=(S=12142568,E=12142571,C=33)
	CT(43385)=(S=12142572,E=12142575,C=57)
	CT(43386)=(S=12142576,E=12142579,C=58)
	CT(43387)=(S=12142580,E=12142583,C=15)
	CT(43388)=(S=12142584,E=12142587,C=28)
	CT(43389)=(S=12142588,E=12142591,C=42)
	CT(43390)=(S=12142592,E=12142595,C=34)
	CT(43391)=(S=12142596,E=12142599,C=12)
	CT(43392)=(S=12142600,E=12142603,C=18)
	CT(43393)=(S=12142604,E=12142607,C=25)
	CT(43394)=(S=12142608,E=12142611,C=50)
	CT(43395)=(S=12142612,E=12142615,C=34)
	CT(43396)=(S=12142616,E=12142619,C=26)
	CT(43397)=(S=12142620,E=12142623,C=52)
	CT(43398)=(S=12142624,E=12142627,C=58)
	CT(43399)=(S=12142628,E=12142631,C=24)
	CT(43400)=(S=12142632,E=12142635,C=15)
	CT(43401)=(S=12142636,E=12142639,C=25)
	CT(43402)=(S=12142640,E=12142643,C=31)
	CT(43403)=(S=12142644,E=12142647,C=25)
	CT(43404)=(S=12142648,E=12142651,C=51)
	CT(43405)=(S=12142652,E=12142655,C=52)
	CT(43406)=(S=12142656,E=12142659,C=34)
	CT(43407)=(S=12142660,E=12142663,C=28)
	CT(43408)=(S=12142664,E=12142667,C=42)
	CT(43409)=(S=12142668,E=12142671,C=15)
	CT(43410)=(S=12142672,E=12142675,C=12)
	CT(43411)=(S=12142676,E=12142679,C=60)
	CT(43412)=(S=12142680,E=12142683,C=12)
	CT(43413)=(S=12142684,E=12142687,C=14)
	CT(43414)=(S=12142688,E=12142691,C=27)
	CT(43415)=(S=12142692,E=12142695,C=61)
	CT(43416)=(S=12142696,E=12142699,C=52)
	CT(43417)=(S=12142700,E=12142703,C=14)
	CT(43418)=(S=12142704,E=12142707,C=25)
	CT(43419)=(S=12142708,E=12142711,C=47)
	CT(43420)=(S=12142712,E=12142719,C=36)
	CT(43421)=(S=12142720,E=12142723,C=14)
	CT(43422)=(S=12142724,E=12142727,C=36)
	CT(43423)=(S=12142728,E=12142731,C=42)
	CT(43424)=(S=12142732,E=12142735,C=24)
	CT(43425)=(S=12142736,E=12142739,C=18)
	CT(43426)=(S=12142740,E=12142743,C=22)
	CT(43427)=(S=12142744,E=12142747,C=21)
	CT(43428)=(S=12142748,E=12142751,C=29)
	CT(43429)=(S=12142752,E=12142759,C=36)
	CT(43430)=(S=12142760,E=12142763,C=49)
	CT(43431)=(S=12142764,E=12142767,C=14)
	CT(43432)=(S=12142768,E=12142771,C=38)
	CT(43433)=(S=12142772,E=12142775,C=36)
	CT(43434)=(S=12142776,E=12142779,C=58)
	CT(43435)=(S=12142780,E=12142783,C=18)
	CT(43436)=(S=12142784,E=12142787,C=34)
	CT(43437)=(S=12142788,E=12142791,C=58)
	CT(43438)=(S=12142792,E=12142795,C=28)
	CT(43439)=(S=12142796,E=12142799,C=60)
	CT(43440)=(S=12142800,E=12142803,C=24)
	CT(43441)=(S=12142804,E=12142807,C=25)
	CT(43442)=(S=12142808,E=12142811,C=39)
	CT(43443)=(S=12142812,E=12142815,C=27)
	CT(43444)=(S=12142816,E=12142823,C=18)
	CT(43445)=(S=12142824,E=12142827,C=28)
	CT(43446)=(S=12142828,E=12142831,C=34)
	CT(43447)=(S=12142832,E=12142835,C=38)
	CT(43448)=(S=12142836,E=12142839,C=28)
	CT(43449)=(S=12142840,E=12142843,C=12)
	CT(43450)=(S=12142844,E=12142847,C=37)
	CT(43451)=(S=12142848,E=12142851,C=26)
	CT(43452)=(S=12142852,E=12142855,C=52)
	CT(43453)=(S=12142856,E=12142859,C=14)
	CT(43454)=(S=12142860,E=12142863,C=27)
	CT(43455)=(S=12142864,E=12142867,C=18)
	CT(43456)=(S=12142868,E=12142871,C=28)
	CT(43457)=(S=12142872,E=12142875,C=27)
	CT(43458)=(S=12142876,E=12142883,C=36)
	CT(43459)=(S=12142884,E=12142887,C=14)
	CT(43460)=(S=12142888,E=12142891,C=22)
	CT(43461)=(S=12142892,E=12142895,C=14)
	CT(43462)=(S=12142896,E=12142899,C=28)
	CT(43463)=(S=12142900,E=12142903,C=12)
	CT(43464)=(S=12142904,E=12142907,C=51)
	CT(43465)=(S=12142908,E=12142911,C=58)
	CT(43466)=(S=12142912,E=12142915,C=14)
	CT(43467)=(S=12142916,E=12142919,C=55)
	CT(43468)=(S=12142920,E=12142923,C=24)
	CT(43469)=(S=12142924,E=12142927,C=26)
	CT(43470)=(S=12142928,E=12142931,C=28)
	CT(43471)=(S=12142932,E=12142935,C=14)
	CT(43472)=(S=12142936,E=12142939,C=58)
	CT(43473)=(S=12142940,E=12142943,C=36)
	CT(43474)=(S=12142944,E=12142947,C=25)
	CT(43475)=(S=12142948,E=12142959,C=33)
	CT(43476)=(S=12142960,E=12142963,C=26)
	CT(43477)=(S=12142964,E=12142971,C=34)
	CT(43478)=(S=12142972,E=12142975,C=36)
	CT(43479)=(S=12142976,E=12142979,C=37)
	CT(43480)=(S=12142980,E=12142983,C=55)
	CT(43481)=(S=12142984,E=12142987,C=15)
	CT(43482)=(S=12142988,E=12142991,C=14)
	CT(43483)=(S=12142992,E=12142995,C=36)
	CT(43484)=(S=12142996,E=12143003,C=14)
	CT(43485)=(S=12143004,E=12143007,C=56)
	CT(43486)=(S=12143008,E=12143011,C=33)
	CT(43487)=(S=12143012,E=12143015,C=15)
	CT(43488)=(S=12143016,E=12143027,C=25)
	CT(43489)=(S=12143028,E=12143031,C=18)
	CT(43490)=(S=12143032,E=12143035,C=14)
	CT(43491)=(S=12143036,E=12143039,C=25)
	CT(43492)=(S=12143040,E=12143043,C=18)
	CT(43493)=(S=12143044,E=12143047,C=24)
	CT(43494)=(S=12143048,E=12143051,C=37)
	CT(43495)=(S=12143052,E=12143055,C=12)
	CT(43496)=(S=12143056,E=12143059,C=58)
	CT(43497)=(S=12143060,E=12143067,C=18)
	CT(43498)=(S=12143068,E=12143071,C=36)
	CT(43499)=(S=12143072,E=12143075,C=33)
	CT(43500)=(S=12143076,E=12143079,C=58)
	CT(43501)=(S=12143080,E=12143083,C=12)
	CT(43502)=(S=12143084,E=12143087,C=51)
	CT(43503)=(S=12143088,E=12143091,C=34)
	CT(43504)=(S=12143092,E=12143099,C=15)
	CT(43505)=(S=12143100,E=12143103,C=12)
	CT(43506)=(S=12143104,E=12143107,C=42)
	CT(43507)=(S=12143108,E=12143111,C=98)
	CT(43508)=(S=12143112,E=12143115,C=12)
	CT(43509)=(S=12143116,E=12143119,C=28)
	CT(43510)=(S=12143120,E=12143123,C=25)
	CT(43511)=(S=12143124,E=12143127,C=50)
	CT(43512)=(S=12143128,E=12143131,C=14)
	CT(43513)=(S=12143132,E=12143135,C=34)
	CT(43514)=(S=12143136,E=12143139,C=12)
	CT(43515)=(S=12143140,E=12143143,C=15)
	CT(43516)=(S=12143144,E=12143147,C=12)
	CT(43517)=(S=12143148,E=12143151,C=36)
	CT(43518)=(S=12143152,E=12143155,C=24)
	CT(43519)=(S=12143156,E=12143159,C=12)
	CT(43520)=(S=12143160,E=12143163,C=64)
	CT(43521)=(S=12143164,E=12143167,C=33)
	CT(43522)=(S=12143168,E=12143171,C=28)
	CT(43523)=(S=12143172,E=12143175,C=34)
	CT(43524)=(S=12143176,E=12143179,C=69)
	CT(43525)=(S=12143180,E=12143183,C=36)
	CT(43526)=(S=12143184,E=12143187,C=25)
	CT(43527)=(S=12143188,E=12143191,C=58)
	CT(43528)=(S=12143192,E=12143195,C=46)
	CT(43529)=(S=12143196,E=12143199,C=40)
	CT(43530)=(S=12143200,E=12143203,C=12)
	CT(43531)=(S=12143204,E=12143207,C=98)
	CT(43532)=(S=12143208,E=12143211,C=18)
	CT(43533)=(S=12143212,E=12143267,C=34)
	CT(43534)=(S=12143268,E=12143271,C=26)
	CT(43535)=(S=12143272,E=12143275,C=14)
	CT(43536)=(S=12143276,E=12143279,C=18)
	CT(43537)=(S=12143280,E=12143283,C=14)
	CT(43538)=(S=12143284,E=12143287,C=28)
	CT(43539)=(S=12143292,E=12143295,C=15)
	CT(43540)=(S=12143296,E=12143299,C=25)
	CT(43541)=(S=12143300,E=12143303,C=71)
	CT(43542)=(S=12143304,E=12143307,C=12)
	CT(43543)=(S=12143308,E=12143311,C=33)
	CT(43544)=(S=12143312,E=12143315,C=171)
	CT(43545)=(S=12143316,E=12143319,C=34)
	CT(43546)=(S=12143320,E=12143323,C=28)
	CT(43547)=(S=12143324,E=12143327,C=18)
	CT(43548)=(S=12143328,E=12143331,C=28)
	CT(43549)=(S=12143332,E=12143335,C=18)
	CT(43550)=(S=12143336,E=12143343,C=14)
	CT(43551)=(S=12143344,E=12143347,C=25)
	CT(43552)=(S=12143348,E=12143351,C=28)
	CT(43553)=(S=12143352,E=12143355,C=15)
	CT(43554)=(S=12143356,E=12143359,C=33)
	CT(43555)=(S=12143360,E=12143363,C=58)
	CT(43556)=(S=12143364,E=12143367,C=18)
	CT(43557)=(S=12143368,E=12143371,C=42)
	CT(43558)=(S=12143372,E=12143375,C=25)
	CT(43559)=(S=12143376,E=12143379,C=23)
	CT(43560)=(S=12143380,E=12143383,C=14)
	CT(43561)=(S=12143384,E=12143387,C=28)
	CT(43562)=(S=12143388,E=12143391,C=14)
	CT(43563)=(S=12143392,E=12143395,C=34)
	CT(43564)=(S=12143396,E=12143399,C=58)
	CT(43565)=(S=12143400,E=12143403,C=19)
	CT(43566)=(S=12143404,E=12143407,C=18)
	CT(43567)=(S=12143408,E=12143411,C=51)
	CT(43568)=(S=12143412,E=12143415,C=38)
	CT(43569)=(S=12143416,E=12143419,C=46)
	CT(43570)=(S=12143420,E=12143423,C=14)
	CT(43571)=(S=12143424,E=12143427,C=18)
	CT(43572)=(S=12143428,E=12143431,C=12)
	CT(43573)=(S=12143432,E=12143435,C=28)
	CT(43574)=(S=12143436,E=12143439,C=15)
	CT(43575)=(S=12143440,E=12143443,C=69)
	CT(43576)=(S=12143444,E=12143451,C=18)
	CT(43577)=(S=12143452,E=12143455,C=14)
	CT(43578)=(S=12143456,E=12143459,C=35)
	CT(43579)=(S=12143460,E=12143463,C=28)
	CT(43580)=(S=12143464,E=12143467,C=14)
	CT(43581)=(S=12143468,E=12143471,C=15)
	CT(43582)=(S=12143472,E=12143475,C=36)
	CT(43583)=(S=12143476,E=12143479,C=33)
	CT(43584)=(S=12143480,E=12143483,C=14)
	CT(43585)=(S=12143484,E=12143487,C=12)
	CT(43586)=(S=12143488,E=12143491,C=47)
	CT(43587)=(S=12143492,E=12143495,C=18)
	CT(43588)=(S=12143496,E=12143499,C=33)
	CT(43589)=(S=12143500,E=12143503,C=12)
	CT(43590)=(S=12143504,E=12143507,C=63)
	CT(43591)=(S=12143508,E=12143511,C=28)
	CT(43592)=(S=12143512,E=12143515,C=34)
	CT(43593)=(S=12143516,E=12143519,C=36)
	CT(43594)=(S=12143520,E=12143523,C=58)
	CT(43595)=(S=12143524,E=12143527,C=28)
	CT(43596)=(S=12143528,E=12143531,C=36)
	CT(43597)=(S=12143532,E=12143535,C=14)
	CT(43598)=(S=12143536,E=12143539,C=25)
	CT(43599)=(S=12143540,E=12143543,C=18)
	CT(43600)=(S=12143544,E=12143547,C=28)
	CT(43601)=(S=12143548,E=12143551,C=18)
	CT(43602)=(S=12143552,E=12143555,C=50)
	CT(43603)=(S=12143556,E=12143559,C=26)
	CT(43604)=(S=12143560,E=12143563,C=18)
	CT(43605)=(S=12143564,E=12143567,C=26)
	CT(43606)=(S=12143568,E=12143571,C=36)
	CT(43607)=(S=12143572,E=12143575,C=14)
	CT(43608)=(S=12143576,E=12143579,C=42)
	CT(43609)=(S=12143580,E=12143583,C=36)
	CT(43610)=(S=12143584,E=12143587,C=12)
	CT(43611)=(S=12143588,E=12143591,C=28)
	CT(43612)=(S=12143592,E=12143595,C=33)
	CT(43613)=(S=12143596,E=12143599,C=60)
	CT(43614)=(S=12143600,E=12143603,C=42)
	CT(43615)=(S=12143604,E=12143607,C=36)
	CT(43616)=(S=12143608,E=12143611,C=18)
	CT(43617)=(S=12143612,E=12143615,C=36)
	CT(43618)=(S=12143616,E=12143619,C=12)
	CT(43619)=(S=12143620,E=12143623,C=21)
	CT(43620)=(S=12143624,E=12143627,C=33)
	CT(43621)=(S=12143628,E=12143631,C=31)
	CT(43622)=(S=12143632,E=12143635,C=36)
	CT(43623)=(S=12143636,E=12143643,C=12)
	CT(43624)=(S=12143644,E=12143647,C=38)
	CT(43625)=(S=12143648,E=12143651,C=35)
	CT(43626)=(S=12143652,E=12143655,C=14)
	CT(43627)=(S=12143656,E=12143659,C=28)
	CT(43628)=(S=12143660,E=12143667,C=14)
	CT(43629)=(S=12143668,E=12143671,C=38)
	CT(43630)=(S=12143672,E=12143675,C=15)
	CT(43631)=(S=12143676,E=12143679,C=38)
	CT(43632)=(S=12143680,E=12143683,C=19)
	CT(43633)=(S=12143684,E=12143687,C=31)
	CT(43634)=(S=12143688,E=12143691,C=36)
	CT(43635)=(S=12143692,E=12143695,C=25)
	CT(43636)=(S=12143696,E=12143699,C=18)
	CT(43637)=(S=12143700,E=12143703,C=27)
	CT(43638)=(S=12143704,E=12143707,C=36)
	CT(43639)=(S=12143708,E=12143711,C=14)
	CT(43640)=(S=12143712,E=12143715,C=28)
	CT(43641)=(S=12143716,E=12143719,C=15)
	CT(43642)=(S=12143720,E=12143723,C=58)
	CT(43643)=(S=12143724,E=12143727,C=18)
	CT(43644)=(S=12143728,E=12143731,C=12)
	CT(43645)=(S=12143732,E=12143735,C=28)
	CT(43646)=(S=12143736,E=12143739,C=12)
	CT(43647)=(S=12143740,E=12143743,C=36)
	CT(43648)=(S=12143744,E=12143747,C=16)
	CT(43649)=(S=12143748,E=12143751,C=36)
	CT(43650)=(S=12143752,E=12143755,C=23)
	CT(43651)=(S=12143756,E=12143759,C=37)
	CT(43652)=(S=12143760,E=12143763,C=18)
	CT(43653)=(S=12143764,E=12143767,C=14)
	CT(43654)=(S=12143768,E=12143771,C=37)
	CT(43655)=(S=12143772,E=12143783,C=28)
	CT(43656)=(S=12143784,E=12143787,C=36)
	CT(43657)=(S=12143788,E=12143791,C=28)
	CT(43658)=(S=12143792,E=12143795,C=39)
	CT(43659)=(S=12143796,E=12143799,C=34)
	CT(43660)=(S=12143800,E=12143803,C=14)
	CT(43661)=(S=12143804,E=12143807,C=28)
	CT(43662)=(S=12143808,E=12143811,C=12)
	CT(43663)=(S=12143812,E=12143819,C=37)
	CT(43664)=(S=12143820,E=12143823,C=16)
	CT(43665)=(S=12143824,E=12143827,C=14)
	CT(43666)=(S=12143828,E=12143835,C=25)
	CT(43667)=(S=12143836,E=12143839,C=18)
	CT(43668)=(S=12143840,E=12143847,C=63)
	CT(43669)=(S=12143848,E=12143851,C=18)
	CT(43670)=(S=12143852,E=12143855,C=36)
	CT(43671)=(S=12143856,E=12143859,C=18)
	CT(43672)=(S=12143860,E=12143863,C=72)
	CT(43673)=(S=12143864,E=12143867,C=26)
	CT(43674)=(S=12143868,E=12143871,C=18)
	CT(43675)=(S=12143872,E=12143875,C=37)
	CT(43676)=(S=12143876,E=12143879,C=14)
	CT(43677)=(S=12143880,E=12143883,C=18)
	CT(43678)=(S=12143884,E=12143887,C=49)
	CT(43679)=(S=12143888,E=12143891,C=18)
	CT(43680)=(S=12143892,E=12143895,C=14)
	CT(43681)=(S=12143896,E=12143899,C=36)
	CT(43682)=(S=12143900,E=12143903,C=40)
	CT(43683)=(S=12143904,E=12143907,C=15)
	CT(43684)=(S=12143908,E=12143911,C=22)
	CT(43685)=(S=12143912,E=12143915,C=37)
	CT(43686)=(S=12143916,E=12143919,C=56)
	CT(43687)=(S=12143920,E=12143923,C=15)
	CT(43688)=(S=12143924,E=12143927,C=56)
	CT(43689)=(S=12143928,E=12143931,C=14)
	CT(43690)=(S=12143932,E=12143935,C=49)
	CT(43691)=(S=12143936,E=12143939,C=14)
	CT(43692)=(S=12143940,E=12143943,C=34)
	CT(43693)=(S=12143944,E=12143947,C=14)
	CT(43694)=(S=12143948,E=12143951,C=58)
	CT(43695)=(S=12143952,E=12143955,C=14)
	CT(43696)=(S=12143956,E=12143959,C=15)
	CT(43697)=(S=12143960,E=12143963,C=37)
	CT(43698)=(S=12143964,E=12143967,C=34)
	CT(43699)=(S=12143968,E=12143971,C=44)
	CT(43700)=(S=12143972,E=12143975,C=31)
	CT(43701)=(S=12143976,E=12143979,C=28)
	CT(43702)=(S=12143980,E=12143983,C=42)
	CT(43703)=(S=12143984,E=12143987,C=12)
	CT(43704)=(S=12143988,E=12143991,C=27)
	CT(43705)=(S=12143992,E=12143995,C=36)
	CT(43706)=(S=12143996,E=12143999,C=51)
	CT(43707)=(S=12144000,E=12144003,C=36)
	CT(43708)=(S=12144004,E=12144007,C=25)
	CT(43709)=(S=12144008,E=12144011,C=18)
	CT(43710)=(S=12144012,E=12144015,C=14)
	CT(43711)=(S=12144016,E=12144019,C=71)
	CT(43712)=(S=12144020,E=12144023,C=18)
	CT(43713)=(S=12144024,E=12144027,C=14)
	CT(43714)=(S=12144028,E=12144031,C=25)
	CT(43715)=(S=12144032,E=12144035,C=15)
	CT(43716)=(S=12144036,E=12144039,C=44)
	CT(43717)=(S=12144040,E=12144043,C=25)
	CT(43718)=(S=12144044,E=12144047,C=14)
	CT(43719)=(S=12144048,E=12144051,C=25)
	CT(43720)=(S=12144052,E=12144055,C=12)
	CT(43721)=(S=12144056,E=12144059,C=25)
	CT(43722)=(S=12144060,E=12144063,C=36)
	CT(43723)=(S=12144064,E=12144067,C=66)
	CT(43724)=(S=12144068,E=12144071,C=49)
	CT(43725)=(S=12144072,E=12144075,C=14)
	CT(43726)=(S=12144076,E=12144079,C=18)
	CT(43727)=(S=12144080,E=12144083,C=36)
	CT(43728)=(S=12144084,E=12144087,C=49)
	CT(43729)=(S=12144088,E=12144095,C=18)
	CT(43730)=(S=12144096,E=12144099,C=169)
	CT(43731)=(S=12144100,E=12144115,C=18)
	CT(43732)=(S=12144116,E=12144119,C=47)
	CT(43733)=(S=12144120,E=12144123,C=38)
	CT(43734)=(S=12144124,E=12144127,C=42)
	CT(43735)=(S=12144128,E=12144131,C=25)
	CT(43736)=(S=12144132,E=12144139,C=14)
	CT(43737)=(S=12144140,E=12144143,C=33)
	CT(43738)=(S=12144144,E=12144147,C=15)
	CT(43739)=(S=12144148,E=12144151,C=26)
	CT(43740)=(S=12144152,E=12144155,C=25)
	CT(43741)=(S=12144156,E=12144163,C=18)
	CT(43742)=(S=12144164,E=12144167,C=51)
	CT(43743)=(S=12144168,E=12144171,C=28)
	CT(43744)=(S=12144172,E=12144175,C=55)
	CT(43745)=(S=12144176,E=12144179,C=15)
	CT(43746)=(S=12144180,E=12144183,C=14)
	CT(43747)=(S=12144184,E=12144187,C=33)
	CT(43748)=(S=12144188,E=12144191,C=28)
	CT(43749)=(S=12144192,E=12144195,C=15)
	CT(43750)=(S=12144196,E=12144199,C=27)
	CT(43751)=(S=12144200,E=12144203,C=58)
	CT(43752)=(S=12144204,E=12144207,C=18)
	CT(43753)=(S=12144208,E=12144211,C=12)
	CT(43754)=(S=12144212,E=12144215,C=37)
	CT(43755)=(S=12144216,E=12144219,C=14)
	CT(43756)=(S=12144220,E=12144223,C=18)
	CT(43757)=(S=12144224,E=12144227,C=36)
	CT(43758)=(S=12144228,E=12144231,C=33)
	CT(43759)=(S=12144232,E=12144235,C=26)
	CT(43760)=(S=12144236,E=12144247,C=18)
	CT(43761)=(S=12144248,E=12144255,C=34)
	CT(43762)=(S=12144256,E=12144259,C=32)
	CT(43763)=(S=12144260,E=12144263,C=58)
	CT(43764)=(S=12144264,E=12144267,C=98)
	CT(43765)=(S=12144268,E=12144271,C=12)
	CT(43766)=(S=12144272,E=12144275,C=66)
	CT(43767)=(S=12144276,E=12144279,C=58)
	CT(43768)=(S=12144280,E=12144283,C=71)
	CT(43769)=(S=12144284,E=12144287,C=12)
	CT(43770)=(S=12144288,E=12144291,C=36)
	CT(43771)=(S=12144292,E=12144299,C=14)
	CT(43772)=(S=12144300,E=12144303,C=19)
	CT(43773)=(S=12144304,E=12144307,C=28)
	CT(43774)=(S=12144308,E=12144311,C=49)
	CT(43775)=(S=12144312,E=12144315,C=28)
	CT(43776)=(S=12144316,E=12144319,C=34)
	CT(43777)=(S=12144320,E=12144323,C=14)
	CT(43778)=(S=12144324,E=12144327,C=36)
	CT(43779)=(S=12144328,E=12144331,C=18)
	CT(43780)=(S=12144332,E=12144335,C=14)
	CT(43781)=(S=12144336,E=12144339,C=27)
	CT(43782)=(S=12144340,E=12144343,C=25)
	CT(43783)=(S=12144344,E=12144347,C=14)
	CT(43784)=(S=12144348,E=12144351,C=22)
	CT(43785)=(S=12144352,E=12144355,C=96)
	CT(43786)=(S=12144356,E=12144359,C=27)
	CT(43787)=(S=12144360,E=12144363,C=33)
	CT(43788)=(S=12144364,E=12144367,C=18)
	CT(43789)=(S=12144368,E=12144371,C=52)
	CT(43790)=(S=12144372,E=12144375,C=16)
	CT(43791)=(S=12144376,E=12144379,C=36)
	CT(43792)=(S=12144380,E=12144383,C=28)
	CT(43793)=(S=12144384,E=12144387,C=66)
	CT(43794)=(S=12144388,E=12144391,C=25)
	CT(43795)=(S=12144392,E=12144395,C=36)
	CT(43796)=(S=12144396,E=12144399,C=37)
	CT(43797)=(S=12144400,E=12144403,C=68)
	CT(43798)=(S=12144404,E=12144407,C=25)
	CT(43799)=(S=12144408,E=12144411,C=28)
	CT(43800)=(S=12144412,E=12144415,C=25)
	CT(43801)=(S=12144416,E=12144419,C=58)
	CT(43802)=(S=12144420,E=12144423,C=25)
	CT(43803)=(S=12144424,E=12144427,C=53)
	CT(43804)=(S=12144428,E=12144431,C=28)
	CT(43805)=(S=12144432,E=12144435,C=18)
	CT(43806)=(S=12144436,E=12144439,C=15)
	CT(43807)=(S=12144440,E=12144443,C=14)
	CT(43808)=(S=12144444,E=12144447,C=26)
	CT(43809)=(S=12144448,E=12144451,C=28)
	CT(43810)=(S=12144452,E=12144455,C=18)
	CT(43811)=(S=12144456,E=12144459,C=28)
	CT(43812)=(S=12144460,E=12144463,C=18)
	CT(43813)=(S=12144464,E=12144467,C=36)
	CT(43814)=(S=12144468,E=12144471,C=27)
	CT(43815)=(S=12144472,E=12144475,C=34)
	CT(43816)=(S=12144476,E=12144479,C=38)
	CT(43817)=(S=12144480,E=12144483,C=26)
	CT(43818)=(S=12144484,E=12144487,C=18)
	CT(43819)=(S=12144488,E=12144491,C=34)
	CT(43820)=(S=12144492,E=12144499,C=14)
	CT(43821)=(S=12144500,E=12144503,C=18)
	CT(43822)=(S=12144504,E=12144507,C=51)
	CT(43823)=(S=12144508,E=12144511,C=28)
	CT(43824)=(S=12144512,E=12144515,C=25)
	CT(43825)=(S=12144516,E=12144523,C=18)
	CT(43826)=(S=12144524,E=12144527,C=14)
	CT(43827)=(S=12144528,E=12144531,C=25)
	CT(43828)=(S=12144532,E=12144535,C=12)
	CT(43829)=(S=12144536,E=12144539,C=36)
	CT(43830)=(S=12144540,E=12144543,C=26)
	CT(43831)=(S=12144544,E=12144547,C=36)
	CT(43832)=(S=12144548,E=12144551,C=19)
	CT(43833)=(S=12144552,E=12144555,C=28)
	CT(43834)=(S=12144556,E=12144559,C=25)
	CT(43835)=(S=12144560,E=12144563,C=14)
	CT(43836)=(S=12144564,E=12144567,C=34)
	CT(43837)=(S=12144568,E=12144571,C=14)
	CT(43838)=(S=12144572,E=12144575,C=22)
	CT(43839)=(S=12144576,E=12144579,C=12)
	CT(43840)=(S=12144580,E=12144583,C=28)
	CT(43841)=(S=12144584,E=12144591,C=34)
	CT(43842)=(S=12144592,E=12144595,C=28)
	CT(43843)=(S=12144596,E=12144599,C=20)
	CT(43844)=(S=12144600,E=12144603,C=28)
	CT(43845)=(S=12144604,E=12144607,C=12)
	CT(43846)=(S=12144608,E=12144611,C=21)
	CT(43847)=(S=12144612,E=12144615,C=52)
	CT(43848)=(S=12144616,E=12144619,C=34)
	CT(43849)=(S=12144620,E=12144623,C=18)
	CT(43850)=(S=12144624,E=12144627,C=58)
	CT(43851)=(S=12144628,E=12144631,C=44)
	CT(43852)=(S=12144632,E=12144635,C=23)
	CT(43853)=(S=12144636,E=12144639,C=34)
	CT(43854)=(S=12144640,E=12144643,C=51)
	CT(43855)=(S=12144644,E=12144647,C=25)
	CT(43856)=(S=12144648,E=12144651,C=36)
	CT(43857)=(S=12144652,E=12144655,C=14)
	CT(43858)=(S=12144656,E=12144659,C=12)
	CT(43859)=(S=12144660,E=12144663,C=37)
	CT(43860)=(S=12144664,E=12144667,C=61)
	CT(43861)=(S=12144668,E=12144671,C=18)
	CT(43862)=(S=12144672,E=12144675,C=58)
	CT(43863)=(S=12144676,E=12144679,C=36)
	CT(43864)=(S=12144680,E=12144683,C=35)
	CT(43865)=(S=12144684,E=12144687,C=67)
	CT(43866)=(S=12144688,E=12144691,C=50)
	CT(43867)=(S=12144692,E=12144695,C=14)
	CT(43868)=(S=12144696,E=12144699,C=58)
	CT(43869)=(S=12144700,E=12144703,C=27)
	CT(43870)=(S=12144704,E=12144707,C=34)
	CT(43871)=(S=12144708,E=12144711,C=46)
	CT(43872)=(S=12144712,E=12144715,C=37)
	CT(43873)=(S=12144716,E=12144719,C=21)
	CT(43874)=(S=12144720,E=12144727,C=14)
	CT(43875)=(S=12144728,E=12144731,C=34)
	CT(43876)=(S=12144732,E=12144735,C=28)
	CT(43877)=(S=12144736,E=12144739,C=102)
	CT(43878)=(S=12144740,E=12144743,C=18)
	CT(43879)=(S=12144744,E=12144747,C=61)
	CT(43880)=(S=12144748,E=12144751,C=17)
	CT(43881)=(S=12144752,E=12144755,C=38)
	CT(43882)=(S=12144756,E=12144759,C=14)
	CT(43883)=(S=12144760,E=12144763,C=28)
	CT(43884)=(S=12144764,E=12144767,C=70)
	CT(43885)=(S=12144768,E=12144771,C=44)
	CT(43886)=(S=12144772,E=12144775,C=14)
	CT(43887)=(S=12144776,E=12144779,C=37)
	CT(43888)=(S=12144780,E=12144783,C=67)
	CT(43889)=(S=12144784,E=12144787,C=28)
	CT(43890)=(S=12144788,E=12144791,C=18)
	CT(43891)=(S=12144792,E=12144795,C=31)
	CT(43892)=(S=12144796,E=12144799,C=27)
	CT(43893)=(S=12144800,E=12144803,C=58)
	CT(43894)=(S=12144804,E=12144807,C=54)
	CT(43895)=(S=12144808,E=12144811,C=28)
	CT(43896)=(S=12144812,E=12144815,C=41)
	CT(43897)=(S=12144816,E=12144819,C=14)
	CT(43898)=(S=12144820,E=12144823,C=27)
	CT(43899)=(S=12144824,E=12144827,C=28)
	CT(43900)=(S=12144828,E=12144831,C=42)
	CT(43901)=(S=12144832,E=12144835,C=57)
	CT(43902)=(S=12144836,E=12144839,C=26)
	CT(43903)=(S=12144840,E=12144843,C=33)
	CT(43904)=(S=12144844,E=12144847,C=18)
	CT(43905)=(S=12144848,E=12144851,C=19)
	CT(43906)=(S=12144852,E=12144855,C=58)
	CT(43907)=(S=12144856,E=12144859,C=19)
	CT(43908)=(S=12144860,E=12144863,C=14)
	CT(43909)=(S=12144864,E=12144867,C=96)
	CT(43910)=(S=12144868,E=12144871,C=18)
	CT(43911)=(S=12144872,E=12144875,C=36)
	CT(43912)=(S=12144876,E=12144879,C=60)
	CT(43913)=(S=12144880,E=12144883,C=42)
	CT(43914)=(S=12144884,E=12144887,C=36)
	CT(43915)=(S=12144888,E=12144891,C=18)
	CT(43916)=(S=12144892,E=12144895,C=19)
	CT(43917)=(S=12144896,E=12144903,C=15)
	CT(43918)=(S=12144904,E=12144907,C=36)
	CT(43919)=(S=12144908,E=12144911,C=27)
	CT(43920)=(S=12144912,E=12144915,C=14)
	CT(43921)=(S=12144916,E=12144919,C=50)
	CT(43922)=(S=12144920,E=12144923,C=18)
	CT(43923)=(S=12144924,E=12144927,C=52)
	CT(43924)=(S=12144928,E=12144931,C=32)
	CT(43925)=(S=12144932,E=12144935,C=24)
	CT(43926)=(S=12144936,E=12144939,C=26)
	CT(43927)=(S=12144940,E=12144943,C=12)
	CT(43928)=(S=12144944,E=12144947,C=60)
	CT(43929)=(S=12144948,E=12144951,C=12)
	CT(43930)=(S=12144952,E=12144955,C=58)
	CT(43931)=(S=12144956,E=12144959,C=46)
	CT(43932)=(S=12144960,E=12144963,C=18)
	CT(43933)=(S=12144964,E=12144967,C=47)
	CT(43934)=(S=12144968,E=12144971,C=63)
	CT(43935)=(S=12144972,E=12144975,C=25)
	CT(43936)=(S=12144976,E=12144983,C=41)
	CT(43937)=(S=12144984,E=12144987,C=0)
	CT(43938)=(S=12144988,E=12144991,C=40)
	CT(43939)=(S=12144992,E=12144995,C=26)
	CT(43940)=(S=12144996,E=12144999,C=28)
	CT(43941)=(S=12145000,E=12145003,C=58)
	CT(43942)=(S=12145004,E=12145007,C=19)
	CT(43943)=(S=12145008,E=12145011,C=14)
	CT(43944)=(S=12145012,E=12145015,C=24)
	CT(43945)=(S=12145016,E=12145023,C=25)
	CT(43946)=(S=12145024,E=12145027,C=60)
	CT(43947)=(S=12145028,E=12145031,C=25)
	CT(43948)=(S=12145032,E=12145035,C=36)
	CT(43949)=(S=12145036,E=12145039,C=39)
	CT(43950)=(S=12145040,E=12145043,C=18)
	CT(43951)=(S=12145044,E=12145047,C=28)
	CT(43952)=(S=12145048,E=12145051,C=37)
	CT(43953)=(S=12145052,E=12145055,C=12)
	CT(43954)=(S=12145056,E=12145059,C=14)
	CT(43955)=(S=12145060,E=12145063,C=44)
	CT(43956)=(S=12145064,E=12145067,C=14)
	CT(43957)=(S=12145068,E=12145071,C=18)
	CT(43958)=(S=12145072,E=12145075,C=14)
	CT(43959)=(S=12145076,E=12145079,C=25)
	CT(43960)=(S=12145080,E=12145083,C=18)
	CT(43961)=(S=12145084,E=12145091,C=14)
	CT(43962)=(S=12145092,E=12145095,C=22)
	CT(43963)=(S=12145096,E=12145099,C=12)
	CT(43964)=(S=12145100,E=12145111,C=42)
	CT(43965)=(S=12145112,E=12145115,C=41)
	CT(43966)=(S=12145116,E=12145119,C=18)
	CT(43967)=(S=12145120,E=12145123,C=49)
	CT(43968)=(S=12145124,E=12145127,C=57)
	CT(43969)=(S=12145128,E=12145131,C=14)
	CT(43970)=(S=12145132,E=12145135,C=37)
	CT(43971)=(S=12145136,E=12145139,C=42)
	CT(43972)=(S=12145140,E=12145143,C=14)
	CT(43973)=(S=12145144,E=12145147,C=18)
	CT(43974)=(S=12145148,E=12145151,C=14)
	CT(43975)=(S=12145152,E=12145155,C=15)
	CT(43976)=(S=12145156,E=12145159,C=28)
	CT(43977)=(S=12145160,E=12145163,C=14)
	CT(43978)=(S=12145164,E=12145167,C=18)
	CT(43979)=(S=12145168,E=12145171,C=42)
	CT(43980)=(S=12145172,E=12145175,C=28)
	CT(43981)=(S=12145176,E=12145179,C=18)
	CT(43982)=(S=12145180,E=12145183,C=26)
	CT(43983)=(S=12145184,E=12145187,C=39)
	CT(43984)=(S=12145188,E=12145191,C=51)
	CT(43985)=(S=12145192,E=12145195,C=28)
	CT(43986)=(S=12145196,E=12145199,C=63)
	CT(43987)=(S=12145200,E=12145203,C=69)
	CT(43988)=(S=12145204,E=12145207,C=17)
	CT(43989)=(S=12145208,E=12145211,C=25)
	CT(43990)=(S=12145212,E=12145215,C=14)
	CT(43991)=(S=12145216,E=12145219,C=26)
	CT(43992)=(S=12145220,E=12145223,C=17)
	CT(43993)=(S=12145224,E=12145227,C=63)
	CT(43994)=(S=12145228,E=12145231,C=14)
	CT(43995)=(S=12145232,E=12145235,C=25)
	CT(43996)=(S=12145236,E=12145239,C=28)
	CT(43997)=(S=12145240,E=12145243,C=36)
	CT(43998)=(S=12145244,E=12145247,C=44)
	CT(43999)=(S=12145248,E=12145251,C=39)
	CT(44000)=(S=12145252,E=12145255,C=34)
	CT(44001)=(S=12145256,E=12145263,C=46)
	CT(44002)=(S=12145264,E=12145267,C=15)
	CT(44003)=(S=12145268,E=12145271,C=23)
	CT(44004)=(S=12145272,E=12145275,C=42)
	CT(44005)=(S=12145276,E=12145279,C=36)
	CT(44006)=(S=12145280,E=12145283,C=34)
	CT(44007)=(S=12145284,E=12145287,C=15)
	CT(44008)=(S=12145288,E=12145291,C=26)
	CT(44009)=(S=12145292,E=12145295,C=36)
	CT(44010)=(S=12145296,E=12145299,C=47)
	CT(44011)=(S=12145300,E=12145303,C=12)
	CT(44012)=(S=12145304,E=12145307,C=36)
	CT(44013)=(S=12145308,E=12145311,C=14)
	CT(44014)=(S=12145312,E=12145315,C=28)
	CT(44015)=(S=12145316,E=12145319,C=26)
	CT(44016)=(S=12145320,E=12145323,C=15)
	CT(44017)=(S=12145324,E=12145327,C=38)
	CT(44018)=(S=12145328,E=12145331,C=18)
	CT(44019)=(S=12145332,E=12145335,C=26)
	CT(44020)=(S=12145336,E=12145347,C=36)
	CT(44021)=(S=12145348,E=12145351,C=14)
	CT(44022)=(S=12145352,E=12145355,C=36)
	CT(44023)=(S=12145356,E=12145359,C=34)
	CT(44024)=(S=12145360,E=12145363,C=36)
	CT(44025)=(S=12145364,E=12145367,C=33)
	CT(44026)=(S=12145368,E=12145371,C=51)
	CT(44027)=(S=12145372,E=12145375,C=37)
	CT(44028)=(S=12145376,E=12145379,C=15)
	CT(44029)=(S=12145380,E=12145383,C=24)
	CT(44030)=(S=12145384,E=12145387,C=50)
	CT(44031)=(S=12145388,E=12145391,C=33)
	CT(44032)=(S=12145392,E=12145395,C=63)
	CT(44033)=(S=12145396,E=12145399,C=14)
	CT(44034)=(S=12145400,E=12145403,C=58)
	CT(44035)=(S=12145404,E=12145407,C=37)
	CT(44036)=(S=12145408,E=12145411,C=18)
	CT(44037)=(S=12145412,E=12145415,C=55)
	CT(44038)=(S=12145416,E=12145419,C=36)
	CT(44039)=(S=12145420,E=12145423,C=14)
	CT(44040)=(S=12145424,E=12145431,C=27)
	CT(44041)=(S=12145432,E=12145435,C=12)
	CT(44042)=(S=12145436,E=12145439,C=26)
	CT(44043)=(S=12145440,E=12145443,C=36)
	CT(44044)=(S=12145444,E=12145447,C=40)
	CT(44045)=(S=12145448,E=12145451,C=14)
	CT(44046)=(S=12145452,E=12145455,C=58)
	CT(44047)=(S=12145456,E=12145459,C=49)
	CT(44048)=(S=12145460,E=12145463,C=14)
	CT(44049)=(S=12145464,E=12145467,C=33)
	CT(44050)=(S=12145468,E=12145471,C=34)
	CT(44051)=(S=12145472,E=12145475,C=14)
	CT(44052)=(S=12145476,E=12145479,C=12)
	CT(44053)=(S=12145480,E=12145491,C=26)
	CT(44054)=(S=12145492,E=12145495,C=14)
	CT(44055)=(S=12145496,E=12145499,C=26)
	CT(44056)=(S=12145500,E=12145503,C=19)
	CT(44057)=(S=12145504,E=12145507,C=17)
	CT(44058)=(S=12145508,E=12145511,C=14)
	CT(44059)=(S=12145512,E=12145515,C=25)
	CT(44060)=(S=12145516,E=12145519,C=34)
	CT(44061)=(S=12145520,E=12145523,C=26)
	CT(44062)=(S=12145524,E=12145527,C=47)
	CT(44063)=(S=12145528,E=12145531,C=14)
	CT(44064)=(S=12145532,E=12145535,C=41)
	CT(44065)=(S=12145536,E=12145539,C=14)
	CT(44066)=(S=12145540,E=12145543,C=25)
	CT(44067)=(S=12145544,E=12145547,C=49)
	CT(44068)=(S=12145548,E=12145551,C=28)
	CT(44069)=(S=12145552,E=12145555,C=37)
	CT(44070)=(S=12145556,E=12145559,C=69)
	CT(44071)=(S=12145560,E=12145563,C=14)
	CT(44072)=(S=12145564,E=12145567,C=29)
	CT(44073)=(S=12145568,E=12145571,C=25)
	CT(44074)=(S=12145572,E=12145575,C=27)
	CT(44075)=(S=12145576,E=12145579,C=14)
	CT(44076)=(S=12145580,E=12145583,C=15)
	CT(44077)=(S=12145584,E=12145587,C=14)
	CT(44078)=(S=12145588,E=12145595,C=26)
	CT(44079)=(S=12145596,E=12145599,C=12)
	CT(44080)=(S=12145600,E=12145603,C=27)
	CT(44081)=(S=12145604,E=12145611,C=26)
	CT(44082)=(S=12145612,E=12145615,C=47)
	CT(44083)=(S=12145616,E=12145619,C=26)
	CT(44084)=(S=12145620,E=12145623,C=21)
	CT(44085)=(S=12145624,E=12145627,C=13)
	CT(44086)=(S=12145628,E=12145631,C=17)
	CT(44087)=(S=12145632,E=12145635,C=50)
	CT(44088)=(S=12145636,E=12145639,C=28)
	CT(44089)=(S=12145640,E=12145643,C=63)
	CT(44090)=(S=12145644,E=12145647,C=12)
	CT(44091)=(S=12145648,E=12145651,C=18)
	CT(44092)=(S=12145652,E=12145655,C=37)
	CT(44093)=(S=12145656,E=12145659,C=21)
	CT(44094)=(S=12145660,E=12145663,C=54)
	CT(44095)=(S=12145664,E=12145667,C=36)
	CT(44096)=(S=12145668,E=12145671,C=28)
	CT(44097)=(S=12145672,E=12145675,C=63)
	CT(44098)=(S=12145676,E=12145683,C=14)
	CT(44099)=(S=12145684,E=12145687,C=28)
	CT(44100)=(S=12145688,E=12145691,C=49)
	CT(44101)=(S=12145692,E=12145695,C=19)
	CT(44102)=(S=12145696,E=12145699,C=18)
	CT(44103)=(S=12145700,E=12145703,C=27)
	CT(44104)=(S=12145704,E=12145707,C=18)
	CT(44105)=(S=12145708,E=12145711,C=25)
	CT(44106)=(S=12145712,E=12145715,C=30)
	CT(44107)=(S=12145716,E=12145719,C=19)
	CT(44108)=(S=12145720,E=12145723,C=28)
	CT(44109)=(S=12145724,E=12145727,C=24)
	CT(44110)=(S=12145728,E=12145731,C=38)
	CT(44111)=(S=12145732,E=12145735,C=35)
	CT(44112)=(S=12145736,E=12145739,C=36)
	CT(44113)=(S=12145740,E=12145743,C=34)
	CT(44114)=(S=12145744,E=12145747,C=28)
	CT(44115)=(S=12145748,E=12145751,C=14)
	CT(44116)=(S=12145752,E=12145755,C=15)
	CT(44117)=(S=12145756,E=12145759,C=14)
	CT(44118)=(S=12145760,E=12145763,C=15)
	CT(44119)=(S=12145764,E=12145767,C=52)
	CT(44120)=(S=12145768,E=12145771,C=39)
	CT(44121)=(S=12145772,E=12145775,C=18)
	CT(44122)=(S=12145776,E=12145779,C=12)
	CT(44123)=(S=12145780,E=12145783,C=25)
	CT(44124)=(S=12145784,E=12145787,C=28)
	CT(44125)=(S=12145788,E=12145791,C=22)
	CT(44126)=(S=12145792,E=12145795,C=25)
	CT(44127)=(S=12145796,E=12145799,C=24)
	CT(44128)=(S=12145800,E=12145803,C=58)
	CT(44129)=(S=12145804,E=12145807,C=36)
	CT(44130)=(S=12145808,E=12145811,C=59)
	CT(44131)=(S=12145812,E=12145815,C=31)
	CT(44132)=(S=12145816,E=12145819,C=33)
	CT(44133)=(S=12145820,E=12145823,C=18)
	CT(44134)=(S=12145824,E=12145827,C=26)
	CT(44135)=(S=12145828,E=12145831,C=23)
	CT(44136)=(S=12145832,E=12145835,C=15)
	CT(44137)=(S=12145836,E=12145839,C=18)
	CT(44138)=(S=12145840,E=12145843,C=15)
	CT(44139)=(S=12145844,E=12145847,C=37)
	CT(44140)=(S=12145848,E=12145855,C=18)
	CT(44141)=(S=12145856,E=12145859,C=34)
	CT(44142)=(S=12145860,E=12145863,C=36)
	CT(44143)=(S=12145864,E=12145867,C=14)
	CT(44144)=(S=12145868,E=12145871,C=25)
	CT(44145)=(S=12145872,E=12145875,C=34)
	CT(44146)=(S=12145876,E=12145879,C=19)
	CT(44147)=(S=12145880,E=12145883,C=28)
	CT(44148)=(S=12145884,E=12145887,C=26)
	CT(44149)=(S=12145888,E=12145891,C=39)
	CT(44150)=(S=12145892,E=12145895,C=15)
	CT(44151)=(S=12145896,E=12145899,C=33)
	CT(44152)=(S=12145900,E=12145903,C=30)
	CT(44153)=(S=12145904,E=12145907,C=19)
	CT(44154)=(S=12145908,E=12145911,C=18)
	CT(44155)=(S=12145912,E=12145915,C=19)
	CT(44156)=(S=12145916,E=12145919,C=63)
	CT(44157)=(S=12145920,E=12145923,C=28)
	CT(44158)=(S=12145924,E=12145927,C=24)
	CT(44159)=(S=12145928,E=12145935,C=18)
	CT(44160)=(S=12145936,E=12145939,C=36)
	CT(44161)=(S=12145940,E=12145947,C=15)
	CT(44162)=(S=12145948,E=12145955,C=42)
	CT(44163)=(S=12145956,E=12145959,C=71)
	CT(44164)=(S=12145960,E=12145963,C=14)
	CT(44165)=(S=12145964,E=12145967,C=24)
	CT(44166)=(S=12145968,E=12145971,C=28)
	CT(44167)=(S=12145972,E=12145975,C=47)
	CT(44168)=(S=12145976,E=12145983,C=14)
	CT(44169)=(S=12145984,E=12145987,C=42)
	CT(44170)=(S=12145988,E=12145991,C=26)
	CT(44171)=(S=12145992,E=12145995,C=37)
	CT(44172)=(S=12145996,E=12145999,C=12)
	CT(44173)=(S=12146000,E=12146003,C=36)
	CT(44174)=(S=12146004,E=12146011,C=14)
	CT(44175)=(S=12146012,E=12146015,C=33)
	CT(44176)=(S=12146016,E=12146019,C=14)
	CT(44177)=(S=12146020,E=12146023,C=19)
	CT(44178)=(S=12146024,E=12146027,C=37)
	CT(44179)=(S=12146028,E=12146031,C=34)
	CT(44180)=(S=12146032,E=12146035,C=36)
	CT(44181)=(S=12146036,E=12146039,C=14)
	CT(44182)=(S=12146040,E=12146043,C=18)
	CT(44183)=(S=12146044,E=12146047,C=34)
	CT(44184)=(S=12146048,E=12146051,C=27)
	CT(44185)=(S=12146052,E=12146055,C=28)
	CT(44186)=(S=12146056,E=12146059,C=26)
	CT(44187)=(S=12146060,E=12146063,C=36)
	CT(44188)=(S=12146064,E=12146067,C=34)
	CT(44189)=(S=12146068,E=12146071,C=52)
	CT(44190)=(S=12146072,E=12146075,C=96)
	CT(44191)=(S=12146076,E=12146079,C=19)
	CT(44192)=(S=12146080,E=12146083,C=28)
	CT(44193)=(S=12146084,E=12146087,C=33)
	CT(44194)=(S=12146088,E=12146091,C=34)
	CT(44195)=(S=12146092,E=12146095,C=49)
	CT(44196)=(S=12146096,E=12146103,C=38)
	CT(44197)=(S=12146104,E=12146107,C=14)
	CT(44198)=(S=12146108,E=12146111,C=37)
	CT(44199)=(S=12146112,E=12146115,C=15)
	CT(44200)=(S=12146116,E=12146119,C=31)
	CT(44201)=(S=12146120,E=12146123,C=38)
	CT(44202)=(S=12146124,E=12146127,C=37)
	CT(44203)=(S=12146128,E=12146131,C=26)
	CT(44204)=(S=12146132,E=12146135,C=15)
	CT(44205)=(S=12146136,E=12146139,C=18)
	CT(44206)=(S=12146140,E=12146143,C=28)
	CT(44207)=(S=12146144,E=12146147,C=12)
	CT(44208)=(S=12146148,E=12146151,C=34)
	CT(44209)=(S=12146152,E=12146155,C=12)
	CT(44210)=(S=12146156,E=12146159,C=37)
	CT(44211)=(S=12146160,E=12146163,C=12)
	CT(44212)=(S=12146164,E=12146167,C=14)
	CT(44213)=(S=12146168,E=12146171,C=34)
	CT(44214)=(S=12146172,E=12146175,C=15)
	CT(44215)=(S=12146176,E=12146179,C=31)
	CT(44216)=(S=12146180,E=12146183,C=37)
	CT(44217)=(S=12146184,E=12146187,C=25)
	CT(44218)=(S=12146188,E=12146191,C=37)
	CT(44219)=(S=12146192,E=12146195,C=49)
	CT(44220)=(S=12146196,E=12146199,C=14)
	CT(44221)=(S=12146200,E=12146203,C=42)
	CT(44222)=(S=12146204,E=12146207,C=34)
	CT(44223)=(S=12146208,E=12146211,C=25)
	CT(44224)=(S=12146212,E=12146215,C=26)
	CT(44225)=(S=12146216,E=12146219,C=17)
	CT(44226)=(S=12146220,E=12146223,C=72)
	CT(44227)=(S=12146224,E=12146231,C=12)
	CT(44228)=(S=12146232,E=12146235,C=31)
	CT(44229)=(S=12146236,E=12146239,C=100)
	CT(44230)=(S=12146240,E=12146243,C=38)
	CT(44231)=(S=12146244,E=12146247,C=24)
	CT(44232)=(S=12146248,E=12146251,C=25)
	CT(44233)=(S=12146252,E=12146255,C=31)
	CT(44234)=(S=12146256,E=12146259,C=37)
	CT(44235)=(S=12146260,E=12146263,C=15)
	CT(44236)=(S=12146264,E=12146267,C=12)
	CT(44237)=(S=12146268,E=12146271,C=18)
	CT(44238)=(S=12146272,E=12146275,C=33)
	CT(44239)=(S=12146276,E=12146283,C=19)
	CT(44240)=(S=12146284,E=12146287,C=14)
	CT(44241)=(S=12146288,E=12146303,C=18)
	CT(44242)=(S=12146304,E=12146307,C=15)
	CT(44243)=(S=12146308,E=12146311,C=18)
	CT(44244)=(S=12146312,E=12146315,C=12)
	CT(44245)=(S=12146316,E=12146319,C=43)
	CT(44246)=(S=12146320,E=12146323,C=18)
	CT(44247)=(S=12146324,E=12146327,C=19)
	CT(44248)=(S=12146328,E=12146331,C=37)
	CT(44249)=(S=12146332,E=12146335,C=28)
	CT(44250)=(S=12146336,E=12146339,C=39)
	CT(44251)=(S=12146340,E=12146343,C=37)
	CT(44252)=(S=12146344,E=12146347,C=12)
	CT(44253)=(S=12146348,E=12146351,C=28)
	CT(44254)=(S=12146352,E=12146355,C=12)
	CT(44255)=(S=12146356,E=12146359,C=26)
	CT(44256)=(S=12146360,E=12146363,C=14)
	CT(44257)=(S=12146364,E=12146367,C=28)
	CT(44258)=(S=12146368,E=12146371,C=66)
	CT(44259)=(S=12146372,E=12146375,C=12)
	CT(44260)=(S=12146376,E=12146379,C=15)
	CT(44261)=(S=12146380,E=12146383,C=17)
	CT(44262)=(S=12146384,E=12146391,C=25)
	CT(44263)=(S=12146392,E=12146395,C=14)
	CT(44264)=(S=12146396,E=12146399,C=48)
	CT(44265)=(S=12146400,E=12146407,C=34)
	CT(44266)=(S=12146408,E=12146411,C=28)
	CT(44267)=(S=12146412,E=12146415,C=54)
	CT(44268)=(S=12146416,E=12146419,C=46)
	CT(44269)=(S=12146420,E=12146423,C=37)
	CT(44270)=(S=12146424,E=12146427,C=25)
	CT(44271)=(S=12146428,E=12146431,C=12)
	CT(44272)=(S=12146432,E=12146435,C=28)
	CT(44273)=(S=12146436,E=12146439,C=49)
	CT(44274)=(S=12146440,E=12146443,C=36)
	CT(44275)=(S=12146444,E=12146447,C=14)
	CT(44276)=(S=12146448,E=12146451,C=42)
	CT(44277)=(S=12146452,E=12146455,C=28)
	CT(44278)=(S=12146456,E=12146459,C=37)
	CT(44279)=(S=12146460,E=12146463,C=15)
	CT(44280)=(S=12146464,E=12146467,C=14)
	CT(44281)=(S=12146468,E=12146479,C=188)
	CT(44282)=(S=12146480,E=12146483,C=18)
	CT(44283)=(S=12146484,E=12146487,C=27)
	CT(44284)=(S=12146488,E=12146495,C=14)
	CT(44285)=(S=12146496,E=12146499,C=39)
	CT(44286)=(S=12146500,E=12146503,C=15)
	CT(44287)=(S=12146504,E=12146507,C=28)
	CT(44288)=(S=12146508,E=12146511,C=15)
	CT(44289)=(S=12146512,E=12146515,C=24)
	CT(44290)=(S=12146516,E=12146519,C=25)
	CT(44291)=(S=12146520,E=12146527,C=12)
	CT(44292)=(S=12146528,E=12146531,C=25)
	CT(44293)=(S=12146532,E=12146535,C=12)
	CT(44294)=(S=12146536,E=12146539,C=36)
	CT(44295)=(S=12146540,E=12146543,C=47)
	CT(44296)=(S=12146544,E=12146547,C=34)
	CT(44297)=(S=12146548,E=12146551,C=36)
	CT(44298)=(S=12146552,E=12146555,C=37)
	CT(44299)=(S=12146556,E=12146559,C=49)
	CT(44300)=(S=12146560,E=12146563,C=25)
	CT(44301)=(S=12146564,E=12146567,C=28)
	CT(44302)=(S=12146568,E=12146571,C=58)
	CT(44303)=(S=12146572,E=12146575,C=33)
	CT(44304)=(S=12146576,E=12146579,C=58)
	CT(44305)=(S=12146580,E=12146583,C=14)
	CT(44306)=(S=12146584,E=12146591,C=36)
	CT(44307)=(S=12146592,E=12146595,C=17)
	CT(44308)=(S=12146596,E=12146599,C=19)
	CT(44309)=(S=12146600,E=12146603,C=39)
	CT(44310)=(S=12146604,E=12146607,C=37)
	CT(44311)=(S=12146608,E=12146611,C=28)
	CT(44312)=(S=12146612,E=12146615,C=26)
	CT(44313)=(S=12146616,E=12146619,C=36)
	CT(44314)=(S=12146620,E=12146623,C=25)
	CT(44315)=(S=12146624,E=12146631,C=18)
	CT(44316)=(S=12146632,E=12146635,C=36)
	CT(44317)=(S=12146636,E=12146639,C=60)
	CT(44318)=(S=12146640,E=12146647,C=14)
	CT(44319)=(S=12146648,E=12146651,C=55)
	CT(44320)=(S=12146652,E=12146655,C=14)
	CT(44321)=(S=12146656,E=12146659,C=28)
	CT(44322)=(S=12146660,E=12146663,C=12)
	CT(44323)=(S=12146664,E=12146667,C=58)
	CT(44324)=(S=12146668,E=12146671,C=42)
	CT(44325)=(S=12146672,E=12146675,C=15)
	CT(44326)=(S=12146676,E=12146679,C=47)
	CT(44327)=(S=12146680,E=12146683,C=36)
	CT(44328)=(S=12146684,E=12146687,C=37)
	CT(44329)=(S=12146688,E=12146691,C=25)
	CT(44330)=(S=12146692,E=12146695,C=37)
	CT(44331)=(S=12146696,E=12146699,C=18)
	CT(44332)=(S=12146700,E=12146703,C=33)
	CT(44333)=(S=12146704,E=12146707,C=21)
	CT(44334)=(S=12146708,E=12146711,C=22)
	CT(44335)=(S=12146712,E=12146715,C=35)
	CT(44336)=(S=12146716,E=12146719,C=38)
	CT(44337)=(S=12146720,E=12146723,C=62)
	CT(44338)=(S=12146724,E=12146727,C=18)
	CT(44339)=(S=12146728,E=12146731,C=27)
	CT(44340)=(S=12146732,E=12146735,C=14)
	CT(44341)=(S=12146736,E=12146739,C=26)
	CT(44342)=(S=12146740,E=12146743,C=25)
	CT(44343)=(S=12146744,E=12146747,C=14)
	CT(44344)=(S=12146748,E=12146751,C=28)
	CT(44345)=(S=12146752,E=12146755,C=36)
	CT(44346)=(S=12146756,E=12146759,C=25)
	CT(44347)=(S=12146760,E=12146763,C=33)
	CT(44348)=(S=12146764,E=12146767,C=34)
	CT(44349)=(S=12146768,E=12146771,C=43)
	CT(44350)=(S=12146772,E=12146775,C=15)
	CT(44351)=(S=12146776,E=12146779,C=43)
	CT(44352)=(S=12146780,E=12146783,C=28)
	CT(44353)=(S=12146784,E=12146791,C=18)
	CT(44354)=(S=12146792,E=12146795,C=12)
	CT(44355)=(S=12146796,E=12146799,C=14)
	CT(44356)=(S=12146800,E=12146803,C=19)
	CT(44357)=(S=12146804,E=12146807,C=34)
	CT(44358)=(S=12146808,E=12146811,C=49)
	CT(44359)=(S=12146812,E=12146815,C=18)
	CT(44360)=(S=12146816,E=12146819,C=38)
	CT(44361)=(S=12146820,E=12146823,C=36)
	CT(44362)=(S=12146824,E=12146827,C=19)
	CT(44363)=(S=12146828,E=12146831,C=36)
	CT(44364)=(S=12146832,E=12146835,C=28)
	CT(44365)=(S=12146836,E=12146839,C=36)
	CT(44366)=(S=12146840,E=12146843,C=26)
	CT(44367)=(S=12146844,E=12146851,C=18)
	CT(44368)=(S=12146852,E=12146855,C=36)
	CT(44369)=(S=12146856,E=12146859,C=15)
	CT(44370)=(S=12146860,E=12146863,C=37)
	CT(44371)=(S=12146864,E=12146867,C=26)
	CT(44372)=(S=12146868,E=12146871,C=36)
	CT(44373)=(S=12146872,E=12146875,C=15)
	CT(44374)=(S=12146876,E=12146879,C=36)
	CT(44375)=(S=12146880,E=12146883,C=14)
	CT(44376)=(S=12146884,E=12146887,C=12)
	CT(44377)=(S=12146888,E=12146891,C=25)
	CT(44378)=(S=12146892,E=12146895,C=15)
	CT(44379)=(S=12146896,E=12146899,C=18)
	CT(44380)=(S=12146900,E=12146903,C=28)
	CT(44381)=(S=12146904,E=12146907,C=14)
	CT(44382)=(S=12146908,E=12146915,C=28)
	CT(44383)=(S=12146916,E=12146919,C=171)
	CT(44384)=(S=12146920,E=12146923,C=25)
	CT(44385)=(S=12146924,E=12146927,C=34)
	CT(44386)=(S=12146928,E=12146931,C=15)
	CT(44387)=(S=12146932,E=12146935,C=28)
	CT(44388)=(S=12146936,E=12146939,C=33)
	CT(44389)=(S=12146940,E=12146943,C=26)
	CT(44390)=(S=12146944,E=12146947,C=27)
	CT(44391)=(S=12146948,E=12146951,C=36)
	CT(44392)=(S=12146952,E=12146959,C=18)
	CT(44393)=(S=12146960,E=12146963,C=36)
	CT(44394)=(S=12146964,E=12146967,C=29)
	CT(44395)=(S=12146968,E=12146971,C=28)
	CT(44396)=(S=12146972,E=12146975,C=25)
	CT(44397)=(S=12146976,E=12146979,C=14)
	CT(44398)=(S=12146980,E=12146983,C=28)
	CT(44399)=(S=12146984,E=12146987,C=18)
	CT(44400)=(S=12146988,E=12146991,C=15)
	CT(44401)=(S=12146992,E=12146995,C=60)
	CT(44402)=(S=12146996,E=12146999,C=47)
	CT(44403)=(S=12147000,E=12147003,C=28)
	CT(44404)=(S=12147004,E=12147007,C=25)
	CT(44405)=(S=12147008,E=12147011,C=58)
	CT(44406)=(S=12147012,E=12147015,C=12)
	CT(44407)=(S=12147016,E=12147019,C=63)
	CT(44408)=(S=12147020,E=12147027,C=36)
	CT(44409)=(S=12147028,E=12147031,C=39)
	CT(44410)=(S=12147032,E=12147035,C=35)
	CT(44411)=(S=12147036,E=12147039,C=27)
	CT(44412)=(S=12147040,E=12147043,C=29)
	CT(44413)=(S=12147044,E=12147047,C=31)
	CT(44414)=(S=12147048,E=12147055,C=18)
	CT(44415)=(S=12147056,E=12147059,C=26)
	CT(44416)=(S=12147060,E=12147063,C=33)
	CT(44417)=(S=12147064,E=12147067,C=14)
	CT(44418)=(S=12147072,E=12147075,C=12)
	CT(44419)=(S=12147076,E=12147079,C=36)
	CT(44420)=(S=12147080,E=12147083,C=72)
	CT(44421)=(S=12147084,E=12147087,C=14)
	CT(44422)=(S=12147088,E=12147091,C=34)
	CT(44423)=(S=12147092,E=12147095,C=42)
	CT(44424)=(S=12147096,E=12147099,C=36)
	CT(44425)=(S=12147100,E=12147103,C=96)
	CT(44426)=(S=12147104,E=12147107,C=42)
	CT(44427)=(S=12147108,E=12147115,C=12)
	CT(44428)=(S=12147116,E=12147119,C=36)
	CT(44429)=(S=12147120,E=12147123,C=58)
	CT(44430)=(S=12147124,E=12147127,C=15)
	CT(44431)=(S=12147128,E=12147131,C=58)
	CT(44432)=(S=12147132,E=12147135,C=34)
	CT(44433)=(S=12147136,E=12147139,C=21)
	CT(44434)=(S=12147140,E=12147143,C=28)
	CT(44435)=(S=12147144,E=12147147,C=34)
	CT(44436)=(S=12147148,E=12147151,C=14)
	CT(44437)=(S=12147152,E=12147155,C=28)
	CT(44438)=(S=12147156,E=12147159,C=57)
	CT(44439)=(S=12147160,E=12147163,C=14)
	CT(44440)=(S=12147164,E=12147167,C=12)
	CT(44441)=(S=12147168,E=12147175,C=28)
	CT(44442)=(S=12147176,E=12147179,C=49)
	CT(44443)=(S=12147180,E=12147183,C=72)
	CT(44444)=(S=12147184,E=12147187,C=38)
	CT(44445)=(S=12147188,E=12147191,C=54)
	CT(44446)=(S=12147192,E=12147195,C=36)
	CT(44447)=(S=12147196,E=12147199,C=34)
	CT(44448)=(S=12147200,E=12147203,C=14)
	CT(44449)=(S=12147204,E=12147207,C=37)
	CT(44450)=(S=12147208,E=12147211,C=15)
	CT(44451)=(S=12147212,E=12147215,C=14)
	CT(44452)=(S=12147216,E=12147219,C=23)
	CT(44453)=(S=12147220,E=12147223,C=42)
	CT(44454)=(S=12147224,E=12147227,C=19)
	CT(44455)=(S=12147228,E=12147231,C=25)
	CT(44456)=(S=12147232,E=12147235,C=14)
	CT(44457)=(S=12147236,E=12147239,C=34)
	CT(44458)=(S=12147240,E=12147243,C=42)
	CT(44459)=(S=12147244,E=12147247,C=32)
	CT(44460)=(S=12147248,E=12147251,C=28)
	CT(44461)=(S=12147252,E=12147255,C=24)
	CT(44462)=(S=12147256,E=12147259,C=21)
	CT(44463)=(S=12147260,E=12147263,C=14)
	CT(44464)=(S=12147264,E=12147267,C=24)
	CT(44465)=(S=12147268,E=12147271,C=15)
	CT(44466)=(S=12147272,E=12147275,C=20)
	CT(44467)=(S=12147276,E=12147279,C=63)
	CT(44468)=(S=12147280,E=12147283,C=37)
	CT(44469)=(S=12147284,E=12147287,C=50)
	CT(44470)=(S=12147288,E=12147291,C=23)
	CT(44471)=(S=12147292,E=12147295,C=42)
	CT(44472)=(S=12147296,E=12147299,C=12)
	CT(44473)=(S=12147300,E=12147303,C=18)
	CT(44474)=(S=12147304,E=12147307,C=35)
	CT(44475)=(S=12147308,E=12147311,C=34)
	CT(44476)=(S=12147312,E=12147315,C=24)
	CT(44477)=(S=12147316,E=12147319,C=15)
	CT(44478)=(S=12147320,E=12147323,C=14)
	CT(44479)=(S=12147324,E=12147327,C=43)
	CT(44480)=(S=12147328,E=12147331,C=28)
	CT(44481)=(S=12147332,E=12147335,C=46)
	CT(44482)=(S=12147336,E=12147339,C=36)
	CT(44483)=(S=12147340,E=12147343,C=14)
	CT(44484)=(S=12147344,E=12147347,C=18)
	CT(44485)=(S=12147348,E=12147351,C=25)
	CT(44486)=(S=12147352,E=12147355,C=34)
	CT(44487)=(S=12147356,E=12147363,C=28)
	CT(44488)=(S=12147364,E=12147367,C=71)
	CT(44489)=(S=12147368,E=12147371,C=39)
	CT(44490)=(S=12147372,E=12147375,C=28)
	CT(44491)=(S=12147376,E=12147379,C=19)
	CT(44492)=(S=12147380,E=12147383,C=27)
	CT(44493)=(S=12147384,E=12147387,C=49)
	CT(44494)=(S=12147388,E=12147391,C=15)
	CT(44495)=(S=12147392,E=12147395,C=14)
	CT(44496)=(S=12147396,E=12147399,C=36)
	CT(44497)=(S=12147400,E=12147403,C=28)
	CT(44498)=(S=12147404,E=12147411,C=25)
	CT(44499)=(S=12147412,E=12147415,C=27)
	CT(44500)=(S=12147416,E=12147419,C=14)
	CT(44501)=(S=12147420,E=12147423,C=36)
	CT(44502)=(S=12147424,E=12147427,C=18)
	CT(44503)=(S=12147428,E=12147435,C=28)
	CT(44504)=(S=12147436,E=12147439,C=50)
	CT(44505)=(S=12147440,E=12147443,C=37)
	CT(44506)=(S=12147444,E=12147447,C=28)
	CT(44507)=(S=12147448,E=12147451,C=41)
	CT(44508)=(S=12147452,E=12147455,C=25)
	CT(44509)=(S=12147456,E=12147459,C=72)
	CT(44510)=(S=12147460,E=12147463,C=28)
	CT(44511)=(S=12147464,E=12147467,C=19)
	CT(44512)=(S=12147468,E=12147471,C=36)
	CT(44513)=(S=12147472,E=12147475,C=35)
	CT(44514)=(S=12147476,E=12147479,C=14)
	CT(44515)=(S=12147480,E=12147483,C=28)
	CT(44516)=(S=12147484,E=12147487,C=36)
	CT(44517)=(S=12147488,E=12147491,C=42)
	CT(44518)=(S=12147492,E=12147495,C=12)
	CT(44519)=(S=12147496,E=12147499,C=14)
	CT(44520)=(S=12147500,E=12147507,C=28)
	CT(44521)=(S=12147508,E=12147511,C=18)
	CT(44522)=(S=12147512,E=12147519,C=15)
	CT(44523)=(S=12147520,E=12147523,C=27)
	CT(44524)=(S=12147524,E=12147527,C=25)
	CT(44525)=(S=12147528,E=12147531,C=31)
	CT(44526)=(S=12147532,E=12147535,C=14)
	CT(44527)=(S=12147536,E=12147539,C=42)
	CT(44528)=(S=12147540,E=12147543,C=14)
	CT(44529)=(S=12147544,E=12147547,C=12)
	CT(44530)=(S=12147548,E=12147551,C=36)
	CT(44531)=(S=12147552,E=12147555,C=39)
	CT(44532)=(S=12147556,E=12147559,C=14)
	CT(44533)=(S=12147560,E=12147563,C=28)
	CT(44534)=(S=12147564,E=12147567,C=25)
	CT(44535)=(S=12147568,E=12147571,C=14)
	CT(44536)=(S=12147572,E=12147575,C=33)
	CT(44537)=(S=12147576,E=12147587,C=14)
	CT(44538)=(S=12147588,E=12147591,C=62)
	CT(44539)=(S=12147592,E=12147595,C=25)
	CT(44540)=(S=12147596,E=12147599,C=50)
	CT(44541)=(S=12147600,E=12147603,C=42)
	CT(44542)=(S=12147604,E=12147607,C=15)
	CT(44543)=(S=12147608,E=12147611,C=27)
	CT(44544)=(S=12147612,E=12147615,C=59)
	CT(44545)=(S=12147616,E=12147619,C=15)
	CT(44546)=(S=12147620,E=12147627,C=33)
	CT(44547)=(S=12147628,E=12147639,C=18)
	CT(44548)=(S=12147640,E=12147643,C=50)
	CT(44549)=(S=12147644,E=12147647,C=15)
	CT(44550)=(S=12147648,E=12147651,C=25)
	CT(44551)=(S=12147652,E=12147655,C=28)
	CT(44552)=(S=12147656,E=12147659,C=71)
	CT(44553)=(S=12147660,E=12147663,C=33)
	CT(44554)=(S=12147664,E=12147667,C=41)
	CT(44555)=(S=12147668,E=12147671,C=58)
	CT(44556)=(S=12147672,E=12147675,C=14)
	CT(44557)=(S=12147676,E=12147683,C=12)
	CT(44558)=(S=12147684,E=12147687,C=25)
	CT(44559)=(S=12147688,E=12147691,C=12)
	CT(44560)=(S=12147692,E=12147695,C=14)
	CT(44561)=(S=12147696,E=12147699,C=15)
	CT(44562)=(S=12147700,E=12147703,C=28)
	CT(44563)=(S=12147704,E=12147707,C=36)
	CT(44564)=(S=12147708,E=12147711,C=18)
	CT(44565)=(S=12147712,E=12147715,C=37)
	CT(44566)=(S=12147716,E=12147723,C=26)
	CT(44567)=(S=12147724,E=12147727,C=37)
	CT(44568)=(S=12147728,E=12147731,C=36)
	CT(44569)=(S=12147732,E=12147735,C=12)
	CT(44570)=(S=12147736,E=12147739,C=14)
	CT(44571)=(S=12147740,E=12147743,C=30)
	CT(44572)=(S=12147744,E=12147747,C=18)
	CT(44573)=(S=12147748,E=12147751,C=12)
	CT(44574)=(S=12147752,E=12147755,C=26)
	CT(44575)=(S=12147756,E=12147759,C=25)
	CT(44576)=(S=12147760,E=12147763,C=58)
	CT(44577)=(S=12147764,E=12147767,C=27)
	CT(44578)=(S=12147768,E=12147771,C=40)
	CT(44579)=(S=12147772,E=12147775,C=36)
	CT(44580)=(S=12147776,E=12147779,C=34)
	CT(44581)=(S=12147780,E=12147787,C=36)
	CT(44582)=(S=12147788,E=12147791,C=28)
	CT(44583)=(S=12147792,E=12147795,C=25)
	CT(44584)=(S=12147796,E=12147799,C=66)
	CT(44585)=(S=12147800,E=12147803,C=30)
	CT(44586)=(S=12147804,E=12147807,C=36)
	CT(44587)=(S=12147808,E=12147811,C=21)
	CT(44588)=(S=12147812,E=12147815,C=42)
	CT(44589)=(S=12147816,E=12147819,C=15)
	CT(44590)=(S=12147820,E=12147823,C=25)
	CT(44591)=(S=12147824,E=12147827,C=47)
	CT(44592)=(S=12147828,E=12147831,C=50)
	CT(44593)=(S=12147832,E=12147835,C=28)
	CT(44594)=(S=12147836,E=12147839,C=14)
	CT(44595)=(S=12147840,E=12147843,C=63)
	CT(44596)=(S=12147844,E=12147847,C=51)
	CT(44597)=(S=12147848,E=12147859,C=18)
	CT(44598)=(S=12147860,E=12147863,C=36)
	CT(44599)=(S=12147864,E=12147867,C=28)
	CT(44600)=(S=12147868,E=12147871,C=58)
	CT(44601)=(S=12147872,E=12147875,C=28)
	CT(44602)=(S=12147876,E=12147879,C=15)
	CT(44603)=(S=12147880,E=12147883,C=28)
	CT(44604)=(S=12147884,E=12147887,C=14)
	CT(44605)=(S=12147888,E=12147891,C=52)
	CT(44606)=(S=12147892,E=12147895,C=28)
	CT(44607)=(S=12147896,E=12147899,C=18)
	CT(44608)=(S=12147900,E=12147903,C=28)
	CT(44609)=(S=12147904,E=12147907,C=38)
	CT(44610)=(S=12147908,E=12147911,C=49)
	CT(44611)=(S=12147912,E=12147915,C=42)
	CT(44612)=(S=12147916,E=12147919,C=28)
	CT(44613)=(S=12147920,E=12147923,C=34)
	CT(44614)=(S=12147924,E=12147927,C=37)
	CT(44615)=(S=12147928,E=12147931,C=33)
	CT(44616)=(S=12147932,E=12147935,C=12)
	CT(44617)=(S=12147936,E=12147939,C=41)
	CT(44618)=(S=12147940,E=12147940,C=52)
	CT(44619)=(S=12147941,E=12147941,C=19)
	CT(44620)=(S=12147942,E=12147947,C=52)
	CT(44621)=(S=12147948,E=12147951,C=25)
	CT(44622)=(S=12147952,E=12147955,C=24)
	CT(44623)=(S=12147956,E=12147959,C=25)
	CT(44624)=(S=12147960,E=12147963,C=58)
	CT(44625)=(S=12147964,E=12147967,C=14)
	CT(44626)=(S=12147968,E=12147971,C=33)
	CT(44627)=(S=12147972,E=12147975,C=12)
	CT(44628)=(S=12147976,E=12147979,C=42)
	CT(44629)=(S=12147980,E=12147983,C=36)
	CT(44630)=(S=12147984,E=12147987,C=49)
	CT(44631)=(S=12147988,E=12147991,C=168)
	CT(44632)=(S=12147992,E=12147995,C=36)
	CT(44633)=(S=12147996,E=12147999,C=14)
	CT(44634)=(S=12148000,E=12148003,C=58)
	CT(44635)=(S=12148004,E=12148007,C=12)
	CT(44636)=(S=12148008,E=12148011,C=18)
	CT(44637)=(S=12148012,E=12148015,C=25)
	CT(44638)=(S=12148016,E=12148019,C=47)
	CT(44639)=(S=12148020,E=12148023,C=37)
	CT(44640)=(S=12148024,E=12148027,C=28)
	CT(44641)=(S=12148028,E=12148031,C=33)
	CT(44642)=(S=12148032,E=12148035,C=50)
	CT(44643)=(S=12148036,E=12148039,C=37)
	CT(44644)=(S=12148040,E=12148043,C=58)
	CT(44645)=(S=12148044,E=12148047,C=36)
	CT(44646)=(S=12148048,E=12148051,C=25)
	CT(44647)=(S=12148052,E=12148055,C=24)
	CT(44648)=(S=12148056,E=12148059,C=14)
	CT(44649)=(S=12148060,E=12148063,C=58)
	CT(44650)=(S=12148064,E=12148067,C=38)
	CT(44651)=(S=12148068,E=12148071,C=46)
	CT(44652)=(S=12148072,E=12148075,C=52)
	CT(44653)=(S=12148076,E=12148079,C=18)
	CT(44654)=(S=12148080,E=12148083,C=25)
	CT(44655)=(S=12148084,E=12148087,C=28)
	CT(44656)=(S=12148088,E=12148091,C=25)
	CT(44657)=(S=12148092,E=12148095,C=98)
	CT(44658)=(S=12148096,E=12148099,C=12)
	CT(44659)=(S=12148100,E=12148103,C=58)
	CT(44660)=(S=12148104,E=12148111,C=28)
	CT(44661)=(S=12148112,E=12148115,C=15)
	CT(44662)=(S=12148116,E=12148119,C=63)
	CT(44663)=(S=12148120,E=12148123,C=25)
	CT(44664)=(S=12148124,E=12148127,C=48)
	CT(44665)=(S=12148128,E=12148131,C=19)
	CT(44666)=(S=12148132,E=12148135,C=26)
	CT(44667)=(S=12148136,E=12148143,C=14)
	CT(44668)=(S=12148144,E=12148147,C=15)
	CT(44669)=(S=12148148,E=12148151,C=14)
	CT(44670)=(S=12148152,E=12148155,C=31)
	CT(44671)=(S=12148156,E=12148159,C=28)
	CT(44672)=(S=12148160,E=12148163,C=24)
	CT(44673)=(S=12148164,E=12148167,C=16)
	CT(44674)=(S=12148168,E=12148171,C=28)
	CT(44675)=(S=12148172,E=12148175,C=15)
	CT(44676)=(S=12148176,E=12148179,C=18)
	CT(44677)=(S=12148180,E=12148183,C=42)
	CT(44678)=(S=12148184,E=12148191,C=34)
	CT(44679)=(S=12148192,E=12148195,C=25)
	CT(44680)=(S=12148196,E=12148199,C=14)
	CT(44681)=(S=12148200,E=12148203,C=25)
	CT(44682)=(S=12148204,E=12148207,C=28)
	CT(44683)=(S=12148208,E=12148211,C=58)
	CT(44684)=(S=12148212,E=12148215,C=16)
	CT(44685)=(S=12148216,E=12148219,C=37)
	CT(44686)=(S=12148220,E=12148223,C=18)
	CT(44687)=(S=12148224,E=12148227,C=70)
	CT(44688)=(S=12148228,E=12148231,C=33)
	CT(44689)=(S=12148232,E=12148235,C=34)
	CT(44690)=(S=12148236,E=12148239,C=19)
	CT(44691)=(S=12148240,E=12148247,C=15)
	CT(44692)=(S=12148248,E=12148251,C=28)
	CT(44693)=(S=12148252,E=12148255,C=14)
	CT(44694)=(S=12148256,E=12148259,C=18)
	CT(44695)=(S=12148260,E=12148263,C=28)
	CT(44696)=(S=12148264,E=12148267,C=49)
	CT(44697)=(S=12148268,E=12148271,C=63)
	CT(44698)=(S=12148272,E=12148275,C=25)
	CT(44699)=(S=12148276,E=12148279,C=33)
	CT(44700)=(S=12148280,E=12148283,C=12)
	CT(44701)=(S=12148284,E=12148287,C=19)
	CT(44702)=(S=12148288,E=12148291,C=18)
	CT(44703)=(S=12148292,E=12148295,C=58)
	CT(44704)=(S=12148296,E=12148299,C=19)
	CT(44705)=(S=12148300,E=12148303,C=69)
	CT(44706)=(S=12148304,E=12148307,C=15)
	CT(44707)=(S=12148308,E=12148311,C=36)
	CT(44708)=(S=12148312,E=12148315,C=15)
	CT(44709)=(S=12148316,E=12148319,C=36)
	CT(44710)=(S=12148320,E=12148323,C=26)
	CT(44711)=(S=12148324,E=12148327,C=34)
	CT(44712)=(S=12148328,E=12148331,C=28)
	CT(44713)=(S=12148332,E=12148335,C=18)
	CT(44714)=(S=12148336,E=12148339,C=55)
	CT(44715)=(S=12148340,E=12148343,C=14)
	CT(44716)=(S=12148344,E=12148347,C=36)
	CT(44717)=(S=12148348,E=12148351,C=14)
	CT(44718)=(S=12148352,E=12148355,C=19)
	CT(44719)=(S=12148356,E=12148359,C=15)
	CT(44720)=(S=12148360,E=12148363,C=49)
	CT(44721)=(S=12148364,E=12148367,C=12)
	CT(44722)=(S=12148368,E=12148375,C=34)
	CT(44723)=(S=12148376,E=12148379,C=18)
	CT(44724)=(S=12148380,E=12148383,C=71)
	CT(44725)=(S=12148384,E=12148387,C=15)
	CT(44726)=(S=12148388,E=12148391,C=18)
	CT(44727)=(S=12148392,E=12148395,C=36)
	CT(44728)=(S=12148396,E=12148399,C=18)
	CT(44729)=(S=12148400,E=12148403,C=14)
	CT(44730)=(S=12148404,E=12148407,C=36)
	CT(44731)=(S=12148408,E=12148419,C=14)
	CT(44732)=(S=12148420,E=12148423,C=36)
	CT(44733)=(S=12148424,E=12148427,C=63)
	CT(44734)=(S=12148428,E=12148431,C=14)
	CT(44735)=(S=12148432,E=12148435,C=36)
	CT(44736)=(S=12148436,E=12148439,C=18)
	CT(44737)=(S=12148440,E=12148443,C=31)
	CT(44738)=(S=12148444,E=12148447,C=12)
	CT(44739)=(S=12148448,E=12148451,C=18)
	CT(44740)=(S=12148452,E=12148455,C=12)
	CT(44741)=(S=12148456,E=12148459,C=14)
	CT(44742)=(S=12148460,E=12148463,C=36)
	CT(44743)=(S=12148464,E=12148467,C=27)
	CT(44744)=(S=12148468,E=12148471,C=36)
	CT(44745)=(S=12148472,E=12148475,C=25)
	CT(44746)=(S=12148476,E=12148479,C=28)
	CT(44747)=(S=12148480,E=12148483,C=37)
	CT(44748)=(S=12148484,E=12148487,C=101)
	CT(44749)=(S=12148488,E=12148491,C=12)
	CT(44750)=(S=12148492,E=12148495,C=36)
	CT(44751)=(S=12148496,E=12148499,C=14)
	CT(44752)=(S=12148500,E=12148503,C=31)
	CT(44753)=(S=12148504,E=12148507,C=36)
	CT(44754)=(S=12148508,E=12148515,C=28)
	CT(44755)=(S=12148516,E=12148519,C=15)
	CT(44756)=(S=12148520,E=12148523,C=63)
	CT(44757)=(S=12148524,E=12148527,C=36)
	CT(44758)=(S=12148528,E=12148531,C=14)
	CT(44759)=(S=12148532,E=12148535,C=15)
	CT(44760)=(S=12148536,E=12148539,C=25)
	CT(44761)=(S=12148540,E=12148543,C=26)
	CT(44762)=(S=12148544,E=12148547,C=49)
	CT(44763)=(S=12148548,E=12148551,C=36)
	CT(44764)=(S=12148552,E=12148555,C=49)
	CT(44765)=(S=12148556,E=12148559,C=14)
	CT(44766)=(S=12148560,E=12148563,C=28)
	CT(44767)=(S=12148564,E=12148567,C=37)
	CT(44768)=(S=12148568,E=12148575,C=19)
	CT(44769)=(S=12148576,E=12148579,C=28)
	CT(44770)=(S=12148580,E=12148587,C=18)
	CT(44771)=(S=12148588,E=12148591,C=14)
	CT(44772)=(S=12148592,E=12148595,C=28)
	CT(44773)=(S=12148596,E=12148599,C=33)
	CT(44774)=(S=12148600,E=12148603,C=37)
	CT(44775)=(S=12148604,E=12148607,C=39)
	CT(44776)=(S=12148608,E=12148611,C=28)
	CT(44777)=(S=12148612,E=12148619,C=15)
	CT(44778)=(S=12148620,E=12148623,C=23)
	CT(44779)=(S=12148624,E=12148627,C=15)
	CT(44780)=(S=12148628,E=12148631,C=19)
	CT(44781)=(S=12148632,E=12148635,C=26)
	CT(44782)=(S=12148636,E=12148639,C=51)
	CT(44783)=(S=12148640,E=12148643,C=19)
	CT(44784)=(S=12148644,E=12148647,C=36)
	CT(44785)=(S=12148648,E=12148651,C=37)
	CT(44786)=(S=12148652,E=12148655,C=14)
	CT(44787)=(S=12148656,E=12148659,C=36)
	CT(44788)=(S=12148660,E=12148663,C=26)
	CT(44789)=(S=12148664,E=12148667,C=35)
	CT(44790)=(S=12148668,E=12148671,C=18)
	CT(44791)=(S=12148672,E=12148675,C=28)
	CT(44792)=(S=12148676,E=12148679,C=58)
	CT(44793)=(S=12148680,E=12148683,C=28)
	CT(44794)=(S=12148684,E=12148687,C=35)
	CT(44795)=(S=12148688,E=12148691,C=42)
	CT(44796)=(S=12148692,E=12148695,C=28)
	CT(44797)=(S=12148696,E=12148699,C=34)
	CT(44798)=(S=12148700,E=12148703,C=21)
	CT(44799)=(S=12148704,E=12148707,C=15)
	CT(44800)=(S=12148708,E=12148711,C=18)
	CT(44801)=(S=12148712,E=12148723,C=15)
	CT(44802)=(S=12148724,E=12148727,C=14)
	CT(44803)=(S=12148728,E=12148731,C=51)
	CT(44804)=(S=12148732,E=12148735,C=39)
	CT(44805)=(S=12148736,E=12148739,C=18)
	CT(44806)=(S=12148740,E=12148743,C=36)
	CT(44807)=(S=12148744,E=12148747,C=58)
	CT(44808)=(S=12148748,E=12148751,C=28)
	CT(44809)=(S=12148752,E=12148755,C=14)
	CT(44810)=(S=12148756,E=12148759,C=15)
	CT(44811)=(S=12148760,E=12148763,C=25)
	CT(44812)=(S=12148764,E=12148767,C=36)
	CT(44813)=(S=12148768,E=12148771,C=22)
	CT(44814)=(S=12148772,E=12148775,C=18)
	CT(44815)=(S=12148776,E=12148779,C=27)
	CT(44816)=(S=12148780,E=12148783,C=36)
	CT(44817)=(S=12148784,E=12148787,C=14)
	CT(44818)=(S=12148788,E=12148791,C=37)
	CT(44819)=(S=12148792,E=12148795,C=28)
	CT(44820)=(S=12148796,E=12148803,C=14)
	CT(44821)=(S=12148804,E=12148807,C=61)
	CT(44822)=(S=12148808,E=12148811,C=27)
	CT(44823)=(S=12148812,E=12148815,C=34)
	CT(44824)=(S=12148816,E=12148823,C=18)
	CT(44825)=(S=12148824,E=12148827,C=24)
	CT(44826)=(S=12148828,E=12148831,C=14)
	CT(44827)=(S=12148832,E=12148835,C=15)
	CT(44828)=(S=12148836,E=12148839,C=42)
	CT(44829)=(S=12148840,E=12148843,C=12)
	CT(44830)=(S=12148844,E=12148847,C=30)
	CT(44831)=(S=12148848,E=12148851,C=37)
	CT(44832)=(S=12148852,E=12148855,C=15)
	CT(44833)=(S=12148856,E=12148859,C=58)
	CT(44834)=(S=12148860,E=12148863,C=41)
	CT(44835)=(S=12148864,E=12148867,C=15)
	CT(44836)=(S=12148868,E=12148871,C=18)
	CT(44837)=(S=12148872,E=12148875,C=16)
	CT(44838)=(S=12148876,E=12148879,C=36)
	CT(44839)=(S=12148880,E=12148883,C=19)
	CT(44840)=(S=12148884,E=12148887,C=36)
	CT(44841)=(S=12148888,E=12148891,C=19)
	CT(44842)=(S=12148892,E=12148895,C=12)
	CT(44843)=(S=12148896,E=12148899,C=38)
	CT(44844)=(S=12148900,E=12148903,C=51)
	CT(44845)=(S=12148904,E=12148907,C=37)
	CT(44846)=(S=12148908,E=12148911,C=21)
	CT(44847)=(S=12148912,E=12148915,C=53)
	CT(44848)=(S=12148916,E=12148919,C=24)
	CT(44849)=(S=12148920,E=12148923,C=12)
	CT(44850)=(S=12148924,E=12148927,C=31)
	CT(44851)=(S=12148928,E=12148931,C=37)
	CT(44852)=(S=12148932,E=12148935,C=28)
	CT(44853)=(S=12148936,E=12148939,C=14)
	CT(44854)=(S=12148940,E=12148943,C=15)
	CT(44855)=(S=12148944,E=12148947,C=46)
	CT(44856)=(S=12148948,E=12148951,C=28)
	CT(44857)=(S=12148952,E=12148955,C=15)
	CT(44858)=(S=12148956,E=12148959,C=14)
	CT(44859)=(S=12148960,E=12148963,C=94)
	CT(44860)=(S=12148964,E=12148967,C=101)
	CT(44861)=(S=12148968,E=12148971,C=25)
	CT(44862)=(S=12148972,E=12148975,C=28)
	CT(44863)=(S=12148976,E=12148979,C=26)
	CT(44864)=(S=12148980,E=12148983,C=14)
	CT(44865)=(S=12148984,E=12148987,C=19)
	CT(44866)=(S=12148988,E=12148991,C=51)
	CT(44867)=(S=12148992,E=12148995,C=18)
	CT(44868)=(S=12148996,E=12148999,C=28)
	CT(44869)=(S=12149000,E=12149003,C=37)
	CT(44870)=(S=12149004,E=12149007,C=15)
	CT(44871)=(S=12149008,E=12149011,C=58)
	CT(44872)=(S=12149012,E=12149015,C=14)
	CT(44873)=(S=12149016,E=12149019,C=33)
	CT(44874)=(S=12149020,E=12149023,C=14)
	CT(44875)=(S=12149024,E=12149027,C=19)
	CT(44876)=(S=12149028,E=12149031,C=55)
	CT(44877)=(S=12149032,E=12149035,C=28)
	CT(44878)=(S=12149036,E=12149039,C=44)
	CT(44879)=(S=12149040,E=12149043,C=39)
	CT(44880)=(S=12149044,E=12149047,C=52)
	CT(44881)=(S=12149048,E=12149051,C=18)
	CT(44882)=(S=12149052,E=12149055,C=14)
	CT(44883)=(S=12149056,E=12149059,C=43)
	CT(44884)=(S=12149060,E=12149063,C=52)
	CT(44885)=(S=12149064,E=12149067,C=51)
	CT(44886)=(S=12149068,E=12149071,C=18)
	CT(44887)=(S=12149072,E=12149075,C=63)
	CT(44888)=(S=12149076,E=12149079,C=12)
	CT(44889)=(S=12149080,E=12149083,C=19)
	CT(44890)=(S=12149084,E=12149087,C=39)
	CT(44891)=(S=12149088,E=12149091,C=28)
	CT(44892)=(S=12149092,E=12149095,C=34)
	CT(44893)=(S=12149096,E=12149099,C=39)
	CT(44894)=(S=12149100,E=12149103,C=15)
	CT(44895)=(S=12149104,E=12149107,C=20)
	CT(44896)=(S=12149108,E=12149111,C=26)
	CT(44897)=(S=12149112,E=12149115,C=15)
	CT(44898)=(S=12149116,E=12149119,C=17)
	CT(44899)=(S=12149120,E=12149123,C=19)
	CT(44900)=(S=12149124,E=12149127,C=43)
	CT(44901)=(S=12149128,E=12149131,C=34)
	CT(44902)=(S=12149132,E=12149135,C=12)
	CT(44903)=(S=12149136,E=12149139,C=14)
	CT(44904)=(S=12149140,E=12149143,C=44)
	CT(44905)=(S=12149144,E=12149147,C=14)
	CT(44906)=(S=12149148,E=12149151,C=15)
	CT(44907)=(S=12149152,E=12149155,C=14)
	CT(44908)=(S=12149156,E=12149159,C=18)
	CT(44909)=(S=12149160,E=12149163,C=25)
	CT(44910)=(S=12149164,E=12149175,C=28)
	CT(44911)=(S=12149176,E=12149179,C=12)
	CT(44912)=(S=12149180,E=12149191,C=27)
	CT(44913)=(S=12149192,E=12149195,C=18)
	CT(44914)=(S=12149196,E=12149199,C=12)
	CT(44915)=(S=12149200,E=12149203,C=50)
	CT(44916)=(S=12149204,E=12149207,C=36)
	CT(44917)=(S=12149208,E=12149215,C=15)
	CT(44918)=(S=12149216,E=12149219,C=28)
	CT(44919)=(S=12149220,E=12149223,C=36)
	CT(44920)=(S=12149224,E=12149227,C=15)
	CT(44921)=(S=12149228,E=12149231,C=63)
	CT(44922)=(S=12149232,E=12149235,C=28)
	CT(44923)=(S=12149236,E=12149239,C=70)
	CT(44924)=(S=12149240,E=12149243,C=55)
	CT(44925)=(S=12149244,E=12149247,C=18)
	CT(44926)=(S=12149248,E=12149251,C=59)
	CT(44927)=(S=12149252,E=12149255,C=20)
	CT(44928)=(S=12149256,E=12149259,C=15)
	CT(44929)=(S=12149260,E=12149263,C=57)
	CT(44930)=(S=12149264,E=12149267,C=25)
	CT(44931)=(S=12149268,E=12149271,C=12)
	CT(44932)=(S=12149272,E=12149275,C=35)
	CT(44933)=(S=12149276,E=12149279,C=34)
	CT(44934)=(S=12149280,E=12149283,C=25)
	CT(44935)=(S=12149284,E=12149287,C=24)
	CT(44936)=(S=12149288,E=12149291,C=18)
	CT(44937)=(S=12149292,E=12149295,C=14)
	CT(44938)=(S=12149296,E=12149299,C=28)
	CT(44939)=(S=12149300,E=12149303,C=36)
	CT(44940)=(S=12149304,E=12149307,C=28)
	CT(44941)=(S=12149308,E=12149311,C=37)
	CT(44942)=(S=12149312,E=12149315,C=12)
	CT(44943)=(S=12149316,E=12149319,C=42)
	CT(44944)=(S=12149320,E=12149323,C=34)
	CT(44945)=(S=12149324,E=12149327,C=20)
	CT(44946)=(S=12149328,E=12149335,C=18)
	CT(44947)=(S=12149336,E=12149339,C=20)
	CT(44948)=(S=12149340,E=12149343,C=28)
	CT(44949)=(S=12149344,E=12149347,C=24)
	CT(44950)=(S=12149348,E=12149355,C=36)
	CT(44951)=(S=12149356,E=12149359,C=14)
	CT(44952)=(S=12149360,E=12149363,C=26)
	CT(44953)=(S=12149364,E=12149367,C=12)
	CT(44954)=(S=12149368,E=12149371,C=34)
	CT(44955)=(S=12149372,E=12149375,C=36)
	CT(44956)=(S=12149376,E=12149379,C=12)
	CT(44957)=(S=12149380,E=12149383,C=22)
	CT(44958)=(S=12149384,E=12149387,C=37)
	CT(44959)=(S=12149388,E=12149391,C=28)
	CT(44960)=(S=12149392,E=12149395,C=36)
	CT(44961)=(S=12149396,E=12149399,C=14)
	CT(44962)=(S=12149400,E=12149403,C=34)
	CT(44963)=(S=12149404,E=12149407,C=19)
	CT(44964)=(S=12149408,E=12149411,C=14)
	CT(44965)=(S=12149412,E=12149415,C=15)
	CT(44966)=(S=12149416,E=12149431,C=14)
	CT(44967)=(S=12149432,E=12149435,C=28)
	CT(44968)=(S=12149436,E=12149439,C=14)
	CT(44969)=(S=12149440,E=12149443,C=19)
	CT(44970)=(S=12149444,E=12149451,C=14)
	CT(44971)=(S=12149452,E=12149455,C=39)
	CT(44972)=(S=12149456,E=12149459,C=71)
	CT(44973)=(S=12149460,E=12149463,C=14)
	CT(44974)=(S=12149464,E=12149467,C=37)
	CT(44975)=(S=12149468,E=12149475,C=61)
	CT(44976)=(S=12149476,E=12149479,C=47)
	CT(44977)=(S=12149480,E=12149483,C=25)
	CT(44978)=(S=12149484,E=12149491,C=28)
	CT(44979)=(S=12149492,E=12149495,C=19)
	CT(44980)=(S=12149496,E=12149499,C=21)
	CT(44981)=(S=12149500,E=12149503,C=37)
	CT(44982)=(S=12149504,E=12149507,C=59)
	CT(44983)=(S=12149508,E=12149519,C=18)
	CT(44984)=(S=12149520,E=12149523,C=39)
	CT(44985)=(S=12149524,E=12149527,C=37)
	CT(44986)=(S=12149528,E=12149531,C=14)
	CT(44987)=(S=12149532,E=12149535,C=18)
	CT(44988)=(S=12149536,E=12149539,C=39)
	CT(44989)=(S=12149540,E=12149543,C=19)
	CT(44990)=(S=12149548,E=12149551,C=43)
	CT(44991)=(S=12149552,E=12149555,C=49)
	CT(44992)=(S=12149556,E=12149563,C=28)
	CT(44993)=(S=12149564,E=12149567,C=25)
	CT(44994)=(S=12149568,E=12149575,C=33)
	CT(44995)=(S=12149576,E=12149579,C=42)
	CT(44996)=(S=12149580,E=12149583,C=14)
	CT(44997)=(S=12149584,E=12149587,C=42)
	CT(44998)=(S=12149588,E=12149591,C=14)
	CT(44999)=(S=12149592,E=12149595,C=38)
	CT(45000)=(S=12149596,E=12149599,C=18)
	CT(45001)=(S=12149600,E=12149603,C=36)
	CT(45002)=(S=12149604,E=12149607,C=38)
	CT(45003)=(S=12149608,E=12149611,C=58)
	CT(45004)=(S=12149612,E=12149615,C=63)
	CT(45005)=(S=12149616,E=12149619,C=12)
	CT(45006)=(S=12149620,E=12149623,C=33)
	CT(45007)=(S=12149624,E=12149627,C=24)
	CT(45008)=(S=12149628,E=12149631,C=20)
	CT(45009)=(S=12149632,E=12149635,C=36)
	CT(45010)=(S=12149636,E=12149639,C=68)
	CT(45011)=(S=12149640,E=12149643,C=63)
	CT(45012)=(S=12149644,E=12149647,C=46)
	CT(45013)=(S=12149648,E=12149651,C=28)
	CT(45014)=(S=12149652,E=12149655,C=12)
	CT(45015)=(S=12149656,E=12149667,C=18)
	CT(45016)=(S=12149668,E=12149671,C=14)
	CT(45017)=(S=12149672,E=12149675,C=18)
	CT(45018)=(S=12149676,E=12149679,C=58)
	CT(45019)=(S=12149680,E=12149683,C=33)
	CT(45020)=(S=12149684,E=12149687,C=29)
	CT(45021)=(S=12149688,E=12149691,C=36)
	CT(45022)=(S=12149692,E=12149695,C=19)
	CT(45023)=(S=12149696,E=12149699,C=58)
	CT(45024)=(S=12149700,E=12149703,C=15)
	CT(45025)=(S=12149704,E=12149707,C=14)
	CT(45026)=(S=12149708,E=12149711,C=36)
	CT(45027)=(S=12149712,E=12149715,C=28)
	CT(45028)=(S=12149716,E=12149719,C=26)
	CT(45029)=(S=12149720,E=12149723,C=12)
	CT(45030)=(S=12149724,E=12149727,C=39)
	CT(45031)=(S=12149728,E=12149731,C=15)
	CT(45032)=(S=12149732,E=12149735,C=28)
	CT(45033)=(S=12149736,E=12149739,C=21)
	CT(45034)=(S=12149740,E=12149743,C=14)
	CT(45035)=(S=12149744,E=12149747,C=25)
	CT(45036)=(S=12149748,E=12149751,C=14)
	CT(45037)=(S=12149752,E=12149755,C=40)
	CT(45038)=(S=12149756,E=12149759,C=29)
	CT(45039)=(S=12149760,E=12149763,C=34)
	CT(45040)=(S=12149764,E=12149767,C=12)
	CT(45041)=(S=12149768,E=12149771,C=28)
	CT(45042)=(S=12149772,E=12149779,C=15)
	CT(45043)=(S=12149780,E=12149783,C=35)
	CT(45044)=(S=12149784,E=12149787,C=18)
	CT(45045)=(S=12149788,E=12149791,C=12)
	CT(45046)=(S=12149792,E=12149795,C=15)
	CT(45047)=(S=12149796,E=12149799,C=36)
	CT(45048)=(S=12149800,E=12149803,C=29)
	CT(45049)=(S=12149804,E=12149807,C=26)
	CT(45050)=(S=12149808,E=12149811,C=28)
	CT(45051)=(S=12149812,E=12149815,C=18)
	CT(45052)=(S=12149816,E=12149819,C=36)
	CT(45053)=(S=12149820,E=12149823,C=14)
	CT(45054)=(S=12149824,E=12149827,C=20)
	CT(45055)=(S=12149828,E=12149831,C=14)
	CT(45056)=(S=12149832,E=12149835,C=25)
	CT(45057)=(S=12149836,E=12149839,C=40)
	CT(45058)=(S=12149840,E=12149843,C=15)
	CT(45059)=(S=12149844,E=12149847,C=38)
	CT(45060)=(S=12149848,E=12149851,C=25)
	CT(45061)=(S=12149852,E=12149855,C=33)
	CT(45062)=(S=12149856,E=12149859,C=101)
	CT(45063)=(S=12149860,E=12149867,C=18)
	CT(45064)=(S=12149868,E=12149887,C=15)
	CT(45065)=(S=12149888,E=12149891,C=36)
	CT(45066)=(S=12149892,E=12149895,C=70)
	CT(45067)=(S=12149896,E=12149899,C=25)
	CT(45068)=(S=12149900,E=12149903,C=36)
	CT(45069)=(S=12149904,E=12149907,C=42)
	CT(45070)=(S=12149908,E=12149911,C=14)
	CT(45071)=(S=12149912,E=12149915,C=23)
	CT(45072)=(S=12149916,E=12149919,C=18)
	CT(45073)=(S=12149920,E=12149923,C=28)
	CT(45074)=(S=12149924,E=12149927,C=15)
	CT(45075)=(S=12149928,E=12149931,C=39)
	CT(45076)=(S=12149932,E=12149935,C=14)
	CT(45077)=(S=12149936,E=12149939,C=15)
	CT(45078)=(S=12149940,E=12149943,C=25)
	CT(45079)=(S=12149944,E=12149947,C=12)
	CT(45080)=(S=12149948,E=12149951,C=14)
	CT(45081)=(S=12149952,E=12149955,C=36)
	CT(45082)=(S=12149956,E=12149959,C=33)
	CT(45083)=(S=12149960,E=12149963,C=39)
	CT(45084)=(S=12149964,E=12149967,C=27)
	CT(45085)=(S=12149968,E=12149971,C=16)
	CT(45086)=(S=12149972,E=12149975,C=14)
	CT(45087)=(S=12149976,E=12149979,C=39)
	CT(45088)=(S=12149980,E=12149983,C=28)
	CT(45089)=(S=12149984,E=12149987,C=20)
	CT(45090)=(S=12149988,E=12149991,C=58)
	CT(45091)=(S=12149992,E=12149995,C=36)
	CT(45092)=(S=12149996,E=12149999,C=15)
	CT(45093)=(S=12150000,E=12150003,C=42)
	CT(45094)=(S=12150004,E=12150007,C=54)
	CT(45095)=(S=12150008,E=12150011,C=14)
	CT(45096)=(S=12150012,E=12150015,C=15)
	CT(45097)=(S=12150016,E=12150019,C=27)
	CT(45098)=(S=12150020,E=12150023,C=28)
	CT(45099)=(S=12150024,E=12150027,C=56)
	CT(45100)=(S=12150028,E=12150031,C=14)
	CT(45101)=(S=12150032,E=12150035,C=39)
	CT(45102)=(S=12150036,E=12150043,C=18)
	CT(45103)=(S=12150044,E=12150047,C=12)
	CT(45104)=(S=12150048,E=12150051,C=27)
	CT(45105)=(S=12150052,E=12150055,C=15)
	CT(45106)=(S=12150056,E=12150059,C=36)
	CT(45107)=(S=12150060,E=12150063,C=14)
	CT(45108)=(S=12150064,E=12150067,C=56)
	CT(45109)=(S=12150068,E=12150071,C=28)
	CT(45110)=(S=12150072,E=12150075,C=19)
	CT(45111)=(S=12150076,E=12150079,C=36)
	CT(45112)=(S=12150080,E=12150083,C=12)
	CT(45113)=(S=12150084,E=12150087,C=18)
	CT(45114)=(S=12150088,E=12150091,C=58)
	CT(45115)=(S=12150092,E=12150099,C=14)
	CT(45116)=(S=12150100,E=12150103,C=63)
	CT(45117)=(S=12150104,E=12150107,C=24)
	CT(45118)=(S=12150108,E=12150111,C=28)
	CT(45119)=(S=12150112,E=12150115,C=36)
	CT(45120)=(S=12150116,E=12150119,C=28)
	CT(45121)=(S=12150120,E=12150123,C=38)
	CT(45122)=(S=12150124,E=12150127,C=36)
	CT(45123)=(S=12150128,E=12150131,C=28)
	CT(45124)=(S=12150132,E=12150135,C=34)
	CT(45125)=(S=12150136,E=12150139,C=14)
	CT(45126)=(S=12150140,E=12150143,C=48)
	CT(45127)=(S=12150144,E=12150147,C=58)
	CT(45128)=(S=12150148,E=12150151,C=28)
	CT(45129)=(S=12150152,E=12150155,C=40)
	CT(45130)=(S=12150156,E=12150159,C=71)
	CT(45131)=(S=12150160,E=12150163,C=49)
	CT(45132)=(S=12150164,E=12150167,C=14)
	CT(45133)=(S=12150168,E=12150171,C=24)
	CT(45134)=(S=12150172,E=12150175,C=34)
	CT(45135)=(S=12150176,E=12150183,C=25)
	CT(45136)=(S=12150184,E=12150187,C=38)
	CT(45137)=(S=12150188,E=12150191,C=28)
	CT(45138)=(S=12150192,E=12150195,C=21)
	CT(45139)=(S=12150196,E=12150199,C=12)
	CT(45140)=(S=12150200,E=12150203,C=28)
	CT(45141)=(S=12150204,E=12150207,C=58)
	CT(45142)=(S=12150208,E=12150211,C=14)
	CT(45143)=(S=12150212,E=12150215,C=36)
	CT(45144)=(S=12150216,E=12150223,C=18)
	CT(45145)=(S=12150224,E=12150227,C=12)
	CT(45146)=(S=12150228,E=12150231,C=15)
	CT(45147)=(S=12150232,E=12150235,C=18)
	CT(45148)=(S=12150236,E=12150239,C=33)
	CT(45149)=(S=12150240,E=12150243,C=14)
	CT(45150)=(S=12150244,E=12150247,C=26)
	CT(45151)=(S=12150248,E=12150251,C=58)
	CT(45152)=(S=12150252,E=12150255,C=35)
	CT(45153)=(S=12150256,E=12150259,C=63)
	CT(45154)=(S=12150260,E=12150263,C=36)
	CT(45155)=(S=12150264,E=12150267,C=28)
	CT(45156)=(S=12150268,E=12150271,C=36)
	CT(45157)=(S=12150272,E=12150275,C=17)
	CT(45158)=(S=12150276,E=12150279,C=36)
	CT(45159)=(S=12150280,E=12150283,C=18)
	CT(45160)=(S=12150284,E=12150287,C=42)
	CT(45161)=(S=12150288,E=12150291,C=19)
	CT(45162)=(S=12150292,E=12150295,C=33)
	CT(45163)=(S=12150296,E=12150299,C=36)
	CT(45164)=(S=12150300,E=12150303,C=27)
	CT(45165)=(S=12150304,E=12150307,C=19)
	CT(45166)=(S=12150308,E=12150311,C=37)
	CT(45167)=(S=12150312,E=12150315,C=15)
	CT(45168)=(S=12150316,E=12150319,C=36)
	CT(45169)=(S=12150320,E=12150323,C=38)
	CT(45170)=(S=12150324,E=12150327,C=27)
	CT(45171)=(S=12150328,E=12150339,C=28)
	CT(45172)=(S=12150340,E=12150343,C=36)
	CT(45173)=(S=12150344,E=12150347,C=20)
	CT(45174)=(S=12150348,E=12150351,C=33)
	CT(45175)=(S=12150352,E=12150355,C=12)
	CT(45176)=(S=12150356,E=12150359,C=14)
	CT(45177)=(S=12150360,E=12150363,C=31)
	CT(45178)=(S=12150364,E=12150367,C=28)
	CT(45179)=(S=12150368,E=12150371,C=33)
	CT(45180)=(S=12150372,E=12150375,C=19)
	CT(45181)=(S=12150376,E=12150379,C=28)
	CT(45182)=(S=12150380,E=12150383,C=27)
	CT(45183)=(S=12150384,E=12150387,C=18)
	CT(45184)=(S=12150388,E=12150391,C=20)
	CT(45185)=(S=12150392,E=12150395,C=18)
	CT(45186)=(S=12150396,E=12150399,C=46)
	CT(45187)=(S=12150400,E=12150403,C=20)
	CT(45188)=(S=12150404,E=12150407,C=14)
	CT(45189)=(S=12150408,E=12150411,C=18)
	CT(45190)=(S=12150412,E=12150415,C=15)
	CT(45191)=(S=12150416,E=12150419,C=40)
	CT(45192)=(S=12150420,E=12150423,C=14)
	CT(45193)=(S=12150424,E=12150427,C=28)
	CT(45194)=(S=12150428,E=12150431,C=38)
	CT(45195)=(S=12150432,E=12150435,C=12)
	CT(45196)=(S=12150436,E=12150439,C=31)
	CT(45197)=(S=12150440,E=12150447,C=36)
	CT(45198)=(S=12150448,E=12150451,C=12)
	CT(45199)=(S=12150452,E=12150455,C=18)
	CT(45200)=(S=12150456,E=12150459,C=31)
	CT(45201)=(S=12150460,E=12150463,C=58)
	CT(45202)=(S=12150464,E=12150475,C=14)
	CT(45203)=(S=12150476,E=12150479,C=25)
	CT(45204)=(S=12150480,E=12150483,C=55)
	CT(45205)=(S=12150484,E=12150487,C=19)
	CT(45206)=(S=12150488,E=12150491,C=29)
	CT(45207)=(S=12150492,E=12150495,C=14)
	CT(45208)=(S=12150496,E=12150499,C=28)
	CT(45209)=(S=12150500,E=12150503,C=42)
	CT(45210)=(S=12150504,E=12150507,C=47)
	CT(45211)=(S=12150508,E=12150511,C=46)
	CT(45212)=(S=12150512,E=12150515,C=28)
	CT(45213)=(S=12150516,E=12150523,C=14)
	CT(45214)=(S=12150524,E=12150527,C=28)
	CT(45215)=(S=12150528,E=12150531,C=70)
	CT(45216)=(S=12150532,E=12150535,C=14)
	CT(45217)=(S=12150536,E=12150539,C=25)
	CT(45218)=(S=12150540,E=12150543,C=39)
	CT(45219)=(S=12150544,E=12150547,C=36)
	CT(45220)=(S=12150548,E=12150551,C=39)
	CT(45221)=(S=12150552,E=12150555,C=18)
	CT(45222)=(S=12150556,E=12150559,C=8)
	CT(45223)=(S=12150560,E=12150567,C=36)
	CT(45224)=(S=12150568,E=12150571,C=31)
	CT(45225)=(S=12150572,E=12150575,C=28)
	CT(45226)=(S=12150576,E=12150579,C=19)
	CT(45227)=(S=12150580,E=12150583,C=15)
	CT(45228)=(S=12150584,E=12150587,C=36)
	CT(45229)=(S=12150588,E=12150591,C=18)
	CT(45230)=(S=12150592,E=12150595,C=12)
	CT(45231)=(S=12150596,E=12150599,C=18)
	CT(45232)=(S=12150600,E=12150603,C=28)
	CT(45233)=(S=12150604,E=12150607,C=36)
	CT(45234)=(S=12150608,E=12150611,C=15)
	CT(45235)=(S=12150612,E=12150615,C=26)
	CT(45236)=(S=12150616,E=12150619,C=39)
	CT(45237)=(S=12150620,E=12150623,C=28)
	CT(45238)=(S=12150624,E=12150627,C=14)
	CT(45239)=(S=12150628,E=12150631,C=18)
	CT(45240)=(S=12150632,E=12150635,C=63)
	CT(45241)=(S=12150636,E=12150639,C=18)
	CT(45242)=(S=12150640,E=12150643,C=15)
	CT(45243)=(S=12150644,E=12150647,C=14)
	CT(45244)=(S=12150648,E=12150651,C=24)
	CT(45245)=(S=12150652,E=12150655,C=25)
	CT(45246)=(S=12150656,E=12150659,C=26)
	CT(45247)=(S=12150660,E=12150663,C=18)
	CT(45248)=(S=12150664,E=12150667,C=46)
	CT(45249)=(S=12150668,E=12150671,C=12)
	CT(45250)=(S=12150672,E=12150675,C=71)
	CT(45251)=(S=12150676,E=12150679,C=25)
	CT(45252)=(S=12150680,E=12150683,C=58)
	CT(45253)=(S=12150684,E=12150687,C=70)
	CT(45254)=(S=12150688,E=12150695,C=18)
	CT(45255)=(S=12150696,E=12150699,C=20)
	CT(45256)=(S=12150700,E=12150703,C=36)
	CT(45257)=(S=12150704,E=12150707,C=20)
	CT(45258)=(S=12150708,E=12150711,C=25)
	CT(45259)=(S=12150712,E=12150715,C=20)
	CT(45260)=(S=12150716,E=12150719,C=19)
	CT(45261)=(S=12150720,E=12150723,C=28)
	CT(45262)=(S=12150724,E=12150727,C=37)
	CT(45263)=(S=12150728,E=12150731,C=36)
	CT(45264)=(S=12150732,E=12150735,C=25)
	CT(45265)=(S=12150736,E=12150743,C=19)
	CT(45266)=(S=12150744,E=12150747,C=57)
	CT(45267)=(S=12150748,E=12150751,C=14)
	CT(45268)=(S=12150752,E=12150755,C=28)
	CT(45269)=(S=12150756,E=12150759,C=14)
	CT(45270)=(S=12150760,E=12150763,C=28)
	CT(45271)=(S=12150764,E=12150767,C=36)
	CT(45272)=(S=12150768,E=12150771,C=19)
	CT(45273)=(S=12150772,E=12150779,C=26)
	CT(45274)=(S=12150780,E=12150783,C=18)
	CT(45275)=(S=12150784,E=12150787,C=22)
	CT(45276)=(S=12150788,E=12150791,C=18)
	CT(45277)=(S=12150792,E=12150795,C=36)
	CT(45278)=(S=12150796,E=12150799,C=18)
	CT(45279)=(S=12150800,E=12150803,C=34)
	CT(45280)=(S=12150804,E=12150807,C=37)
	CT(45281)=(S=12150808,E=12150811,C=15)
	CT(45282)=(S=12150812,E=12150815,C=36)
	CT(45283)=(S=12150816,E=12150827,C=38)
	CT(45284)=(S=12150828,E=12150831,C=14)
	CT(45285)=(S=12150832,E=12150839,C=38)
	CT(45286)=(S=12150840,E=12150843,C=94)
	CT(45287)=(S=12150844,E=12150847,C=25)
	CT(45288)=(S=12150848,E=12150851,C=58)
	CT(45289)=(S=12150852,E=12150855,C=39)
	CT(45290)=(S=12150856,E=12150859,C=28)
	CT(45291)=(S=12150860,E=12150863,C=12)
	CT(45292)=(S=12150864,E=12150867,C=14)
	CT(45293)=(S=12150868,E=12150871,C=34)
	CT(45294)=(S=12150872,E=12150875,C=28)
	CT(45295)=(S=12150876,E=12150879,C=18)
	CT(45296)=(S=12150880,E=12150883,C=14)
	CT(45297)=(S=12150884,E=12150887,C=36)
	CT(45298)=(S=12150888,E=12150891,C=18)
	CT(45299)=(S=12150892,E=12150895,C=70)
	CT(45300)=(S=12150896,E=12150899,C=31)
	CT(45301)=(S=12150900,E=12150903,C=15)
	CT(45302)=(S=12150904,E=12150907,C=14)
	CT(45303)=(S=12150908,E=12150911,C=51)
	CT(45304)=(S=12150912,E=12150923,C=36)
	CT(45305)=(S=12150924,E=12150927,C=28)
	CT(45306)=(S=12150928,E=12150931,C=22)
	CT(45307)=(S=12150932,E=12150935,C=18)
	CT(45308)=(S=12150936,E=12150939,C=25)
	CT(45309)=(S=12150940,E=12150943,C=35)
	CT(45310)=(S=12150944,E=12150947,C=28)
	CT(45311)=(S=12150948,E=12150951,C=25)
	CT(45312)=(S=12150952,E=12150955,C=38)
	CT(45313)=(S=12150956,E=12150959,C=18)
	CT(45314)=(S=12150960,E=12150963,C=44)
	CT(45315)=(S=12150964,E=12150967,C=38)
	CT(45316)=(S=12150968,E=12150971,C=14)
	CT(45317)=(S=12150972,E=12150975,C=36)
	CT(45318)=(S=12150976,E=12150979,C=26)
	CT(45319)=(S=12150980,E=12150983,C=38)
	CT(45320)=(S=12150984,E=12150987,C=172)
	CT(45321)=(S=12150988,E=12150991,C=28)
	CT(45322)=(S=12150992,E=12150995,C=27)
	CT(45323)=(S=12150996,E=12150999,C=18)
	CT(45324)=(S=12151000,E=12151003,C=14)
	CT(45325)=(S=12151004,E=12151007,C=69)
	CT(45326)=(S=12151008,E=12151011,C=42)
	CT(45327)=(S=12151012,E=12151019,C=26)
	CT(45328)=(S=12151020,E=12151023,C=25)
	CT(45329)=(S=12151024,E=12151027,C=26)
	CT(45330)=(S=12151028,E=12151031,C=39)
	CT(45331)=(S=12151032,E=12151035,C=18)
	CT(45332)=(S=12151036,E=12151039,C=39)
	CT(45333)=(S=12151040,E=12151043,C=24)
	CT(45334)=(S=12151044,E=12151047,C=38)
	CT(45335)=(S=12151048,E=12151051,C=58)
	CT(45336)=(S=12151052,E=12151059,C=25)
	CT(45337)=(S=12151060,E=12151063,C=42)
	CT(45338)=(S=12151064,E=12151067,C=25)
	CT(45339)=(S=12151068,E=12151071,C=12)
	CT(45340)=(S=12151072,E=12151079,C=38)
	CT(45341)=(S=12151080,E=12151083,C=28)
	CT(45342)=(S=12151084,E=12151091,C=18)
	CT(45343)=(S=12151092,E=12151095,C=49)
	CT(45344)=(S=12151096,E=12151099,C=14)
	CT(45345)=(S=12151100,E=12151103,C=19)
	CT(45346)=(S=12151104,E=12151107,C=14)
	CT(45347)=(S=12151108,E=12151111,C=37)
	CT(45348)=(S=12151112,E=12151115,C=14)
	CT(45349)=(S=12151116,E=12151119,C=36)
	CT(45350)=(S=12151120,E=12151123,C=49)
	CT(45351)=(S=12151124,E=12151127,C=38)
	CT(45352)=(S=12151128,E=12151131,C=18)
	CT(45353)=(S=12151132,E=12151135,C=51)
	CT(45354)=(S=12151136,E=12151139,C=14)
	CT(45355)=(S=12151140,E=12151143,C=18)
	CT(45356)=(S=12151144,E=12151147,C=50)
	CT(45357)=(S=12151148,E=12151151,C=31)
	CT(45358)=(S=12151152,E=12151155,C=28)
	CT(45359)=(S=12151156,E=12151159,C=18)
	CT(45360)=(S=12151160,E=12151163,C=26)
	CT(45361)=(S=12151164,E=12151167,C=14)
	CT(45362)=(S=12151168,E=12151171,C=50)
	CT(45363)=(S=12151172,E=12151175,C=34)
	CT(45364)=(S=12151176,E=12151179,C=12)
	CT(45365)=(S=12151180,E=12151183,C=58)
	CT(45366)=(S=12151184,E=12151187,C=34)
	CT(45367)=(S=12151188,E=12151191,C=40)
	CT(45368)=(S=12151192,E=12151195,C=12)
	CT(45369)=(S=12151196,E=12151199,C=49)
	CT(45370)=(S=12151200,E=12151203,C=33)
	CT(45371)=(S=12151204,E=12151207,C=14)
	CT(45372)=(S=12151208,E=12151211,C=18)
	CT(45373)=(S=12151212,E=12151215,C=32)
	CT(45374)=(S=12151216,E=12151219,C=17)
	CT(45375)=(S=12151220,E=12151223,C=33)
	CT(45376)=(S=12151224,E=12151227,C=26)
	CT(45377)=(S=12151228,E=12151231,C=38)
	CT(45378)=(S=12151232,E=12151235,C=36)
	CT(45379)=(S=12151236,E=12151239,C=41)
	CT(45380)=(S=12151240,E=12151243,C=49)
	CT(45381)=(S=12151244,E=12151247,C=36)
	CT(45382)=(S=12151248,E=12151251,C=15)
	CT(45383)=(S=12151252,E=12151255,C=33)
	CT(45384)=(S=12151256,E=12151263,C=36)
	CT(45385)=(S=12151264,E=12151267,C=18)
	CT(45386)=(S=12151268,E=12151271,C=103)
	CT(45387)=(S=12151272,E=12151275,C=25)
	CT(45388)=(S=12151276,E=12151279,C=26)
	CT(45389)=(S=12151280,E=12151283,C=15)
	CT(45390)=(S=12151284,E=12151287,C=19)
	CT(45391)=(S=12151288,E=12151291,C=36)
	CT(45392)=(S=12151292,E=12151295,C=28)
	CT(45393)=(S=12151296,E=12151303,C=25)
	CT(45394)=(S=12151304,E=12151307,C=28)
	CT(45395)=(S=12151308,E=12151311,C=35)
	CT(45396)=(S=12151312,E=12151315,C=22)
	CT(45397)=(S=12151316,E=12151319,C=37)
	CT(45398)=(S=12151320,E=12151323,C=70)
	CT(45399)=(S=12151324,E=12151327,C=35)
	CT(45400)=(S=12151328,E=12151331,C=17)
	CT(45401)=(S=12151332,E=12151335,C=25)
	CT(45402)=(S=12151336,E=12151339,C=18)
	CT(45403)=(S=12151340,E=12151343,C=25)
	CT(45404)=(S=12151344,E=12151347,C=42)
	CT(45405)=(S=12151348,E=12151351,C=36)
	CT(45406)=(S=12151352,E=12151355,C=15)
	CT(45407)=(S=12151356,E=12151359,C=33)
	CT(45408)=(S=12151360,E=12151363,C=34)
	CT(45409)=(S=12151364,E=12151367,C=49)
	CT(45410)=(S=12151368,E=12151371,C=36)
	CT(45411)=(S=12151372,E=12151375,C=28)
	CT(45412)=(S=12151376,E=12151379,C=38)
	CT(45413)=(S=12151380,E=12151383,C=28)
	CT(45414)=(S=12151384,E=12151387,C=63)
	CT(45415)=(S=12151388,E=12151391,C=18)
	CT(45416)=(S=12151392,E=12151395,C=50)
	CT(45417)=(S=12151396,E=12151399,C=32)
	CT(45418)=(S=12151400,E=12151403,C=18)
	CT(45419)=(S=12151404,E=12151407,C=42)
	CT(45420)=(S=12151408,E=12151411,C=50)
	CT(45421)=(S=12151412,E=12151415,C=35)
	CT(45422)=(S=12151416,E=12151419,C=36)
	CT(45423)=(S=12151420,E=12151423,C=188)
	CT(45424)=(S=12151424,E=12151427,C=18)
	CT(45425)=(S=12151428,E=12151431,C=58)
	CT(45426)=(S=12151432,E=12151435,C=26)
	CT(45427)=(S=12151436,E=12151439,C=46)
	CT(45428)=(S=12151440,E=12151443,C=26)
	CT(45429)=(S=12151444,E=12151447,C=28)
	CT(45430)=(S=12151448,E=12151451,C=36)
	CT(45431)=(S=12151452,E=12151455,C=20)
	CT(45432)=(S=12151456,E=12151459,C=42)
	CT(45433)=(S=12151460,E=12151463,C=28)
	CT(45434)=(S=12151464,E=12151467,C=151)
	CT(45435)=(S=12151468,E=12151471,C=18)
	CT(45436)=(S=12151472,E=12151475,C=14)
	CT(45437)=(S=12151476,E=12151479,C=19)
	CT(45438)=(S=12151480,E=12151483,C=28)
	CT(45439)=(S=12151484,E=12151487,C=42)
	CT(45440)=(S=12151488,E=12151491,C=43)
	CT(45441)=(S=12151492,E=12151495,C=18)
	CT(45442)=(S=12151496,E=12151499,C=26)
	CT(45443)=(S=12151500,E=12151503,C=18)
	CT(45444)=(S=12151504,E=12151507,C=37)
	CT(45445)=(S=12151508,E=12151511,C=42)
	CT(45446)=(S=12151512,E=12151515,C=12)
	CT(45447)=(S=12151516,E=12151523,C=36)
	CT(45448)=(S=12151524,E=12151527,C=26)
	CT(45449)=(S=12151528,E=12151531,C=28)
	CT(45450)=(S=12151532,E=12151535,C=49)
	CT(45451)=(S=12151536,E=12151539,C=36)
	CT(45452)=(S=12151540,E=12151543,C=28)
	CT(45453)=(S=12151544,E=12151547,C=42)
	CT(45454)=(S=12151548,E=12151551,C=28)
	CT(45455)=(S=12151552,E=12151555,C=25)
	CT(45456)=(S=12151556,E=12151559,C=28)
	CT(45457)=(S=12151560,E=12151563,C=14)
	CT(45458)=(S=12151564,E=12151567,C=24)
	CT(45459)=(S=12151568,E=12151571,C=25)
	CT(45460)=(S=12151572,E=12151575,C=28)
	CT(45461)=(S=12151576,E=12151579,C=36)
	CT(45462)=(S=12151580,E=12151587,C=26)
	CT(45463)=(S=12151588,E=12151599,C=36)
	CT(45464)=(S=12151600,E=12151603,C=39)
	CT(45465)=(S=12151604,E=12151607,C=28)
	CT(45466)=(S=12151608,E=12151611,C=36)
	CT(45467)=(S=12151612,E=12151615,C=48)
	CT(45468)=(S=12151616,E=12151619,C=58)
	CT(45469)=(S=12151620,E=12151623,C=36)
	CT(45470)=(S=12151624,E=12151627,C=18)
	CT(45471)=(S=12151628,E=12151631,C=94)
	CT(45472)=(S=12151632,E=12151635,C=36)
	CT(45473)=(S=12151636,E=12151639,C=12)
	CT(45474)=(S=12151640,E=12151647,C=36)
	CT(45475)=(S=12151648,E=12151651,C=28)
	CT(45476)=(S=12151652,E=12151655,C=36)
	CT(45477)=(S=12151656,E=12151663,C=25)
	CT(45478)=(S=12151664,E=12151675,C=36)
	CT(45479)=(S=12151676,E=12151683,C=18)
	CT(45480)=(S=12151684,E=12151687,C=37)
	CT(45481)=(S=12151688,E=12151691,C=18)
	CT(45482)=(S=12151692,E=12151695,C=58)
	CT(45483)=(S=12151696,E=12151703,C=15)
	CT(45484)=(S=12151704,E=12151707,C=25)
	CT(45485)=(S=12151708,E=12151711,C=18)
	CT(45486)=(S=12151712,E=12151715,C=25)
	CT(45487)=(S=12151716,E=12151719,C=18)
	CT(45488)=(S=12151720,E=12151723,C=24)
	CT(45489)=(S=12151724,E=12151727,C=20)
	CT(45490)=(S=12151728,E=12151731,C=24)
	CT(45491)=(S=12151732,E=12151739,C=15)
	CT(45492)=(S=12151740,E=12151743,C=36)
	CT(45493)=(S=12151744,E=12151747,C=28)
	CT(45494)=(S=12151748,E=12151751,C=14)
	CT(45495)=(S=12151752,E=12151755,C=24)
	CT(45496)=(S=12151756,E=12151759,C=15)
	CT(45497)=(S=12151760,E=12151763,C=49)
	CT(45498)=(S=12151764,E=12151767,C=36)
	CT(45499)=(S=12151768,E=12151771,C=54)
	CT(45500)=(S=12151772,E=12151775,C=28)
	CT(45501)=(S=12151776,E=12151779,C=36)
	CT(45502)=(S=12151780,E=12151783,C=14)
	CT(45503)=(S=12151784,E=12151787,C=12)
	CT(45504)=(S=12151788,E=12151791,C=15)
	CT(45505)=(S=12151792,E=12151795,C=18)
	CT(45506)=(S=12151796,E=12151803,C=26)
	CT(45507)=(S=12151804,E=12151815,C=18)
	CT(45508)=(S=12151816,E=12151819,C=28)
	CT(45509)=(S=12151820,E=12151823,C=14)
	CT(45510)=(S=12151824,E=12151831,C=18)
	CT(45511)=(S=12151832,E=12151835,C=43)
	CT(45512)=(S=12151836,E=12151839,C=33)
	CT(45513)=(S=12151840,E=12151843,C=12)
	CT(45514)=(S=12151844,E=12151847,C=53)
	CT(45515)=(S=12151848,E=12151851,C=63)
	CT(45516)=(S=12151852,E=12151855,C=14)
	CT(45517)=(S=12151856,E=12151859,C=58)
	CT(45518)=(S=12151860,E=12151867,C=14)
	CT(45519)=(S=12151868,E=12151871,C=33)
	CT(45520)=(S=12151872,E=12151875,C=60)
	CT(45521)=(S=12151876,E=12151879,C=58)
	CT(45522)=(S=12151880,E=12151887,C=28)
	CT(45523)=(S=12151888,E=12151891,C=17)
	CT(45524)=(S=12151892,E=12151899,C=18)
	CT(45525)=(S=12151900,E=12151903,C=14)
	CT(45526)=(S=12151904,E=12151907,C=26)
	CT(45527)=(S=12151908,E=12151911,C=24)
	CT(45528)=(S=12151912,E=12151915,C=21)
	CT(45529)=(S=12151916,E=12151919,C=24)
	CT(45530)=(S=12151920,E=12151923,C=38)
	CT(45531)=(S=12151924,E=12151927,C=36)
	CT(45532)=(S=12151928,E=12151931,C=33)
	CT(45533)=(S=12151932,E=12151935,C=37)
	CT(45534)=(S=12151936,E=12151939,C=63)
	CT(45535)=(S=12151940,E=12151943,C=14)
	CT(45536)=(S=12151944,E=12151947,C=53)
	CT(45537)=(S=12151948,E=12151951,C=51)
	CT(45538)=(S=12151952,E=12151955,C=53)
	CT(45539)=(S=12151956,E=12151959,C=37)
	CT(45540)=(S=12151960,E=12151963,C=53)
	CT(45541)=(S=12151964,E=12151967,C=37)
	CT(45542)=(S=12151968,E=12151971,C=18)
	CT(45543)=(S=12151972,E=12151975,C=26)
	CT(45544)=(S=12151976,E=12151979,C=14)
	CT(45545)=(S=12151980,E=12151983,C=28)
	CT(45546)=(S=12151984,E=12151987,C=14)
	CT(45547)=(S=12151988,E=12151991,C=53)
	CT(45548)=(S=12151992,E=12151995,C=28)
	CT(45549)=(S=12151996,E=12151999,C=51)
	CT(45550)=(S=12152000,E=12152007,C=18)
	CT(45551)=(S=12152008,E=12152011,C=42)
	CT(45552)=(S=12152012,E=12152015,C=25)
	CT(45553)=(S=12152016,E=12152023,C=18)
	CT(45554)=(S=12152024,E=12152027,C=28)
	CT(45555)=(S=12152028,E=12152031,C=18)
	CT(45556)=(S=12152032,E=12152035,C=36)
	CT(45557)=(S=12152036,E=12152039,C=28)
	CT(45558)=(S=12152040,E=12152043,C=14)
	CT(45559)=(S=12152044,E=12152047,C=12)
	CT(45560)=(S=12152048,E=12152051,C=23)
	CT(45561)=(S=12152052,E=12152055,C=34)
	CT(45562)=(S=12152056,E=12152059,C=70)
	CT(45563)=(S=12152060,E=12152063,C=24)
	CT(45564)=(S=12152064,E=12152067,C=34)
	CT(45565)=(S=12152068,E=12152071,C=63)
	CT(45566)=(S=12152072,E=12152075,C=14)
	CT(45567)=(S=12152076,E=12152079,C=24)
	CT(45568)=(S=12152080,E=12152083,C=63)
	CT(45569)=(S=12152084,E=12152087,C=18)
	CT(45570)=(S=12152088,E=12152091,C=15)
	CT(45571)=(S=12152092,E=12152095,C=37)
	CT(45572)=(S=12152096,E=12152099,C=28)
	CT(45573)=(S=12152100,E=12152103,C=34)
	CT(45574)=(S=12152104,E=12152115,C=14)
	CT(45575)=(S=12152116,E=12152119,C=31)
	CT(45576)=(S=12152120,E=12152123,C=15)
	CT(45577)=(S=12152124,E=12152127,C=26)
	CT(45578)=(S=12152128,E=12152131,C=24)
	CT(45579)=(S=12152132,E=12152135,C=15)
	CT(45580)=(S=12152136,E=12152139,C=26)
	CT(45581)=(S=12152140,E=12152143,C=24)
	CT(45582)=(S=12152144,E=12152147,C=26)
	CT(45583)=(S=12152148,E=12152151,C=19)
	CT(45584)=(S=12152152,E=12152155,C=24)
	CT(45585)=(S=12152156,E=12152159,C=14)
	CT(45586)=(S=12152160,E=12152163,C=36)
	CT(45587)=(S=12152164,E=12152167,C=48)
	CT(45588)=(S=12152168,E=12152171,C=14)
	CT(45589)=(S=12152172,E=12152175,C=28)
	CT(45590)=(S=12152176,E=12152179,C=18)
	CT(45591)=(S=12152180,E=12152183,C=14)
	CT(45592)=(S=12152184,E=12152187,C=61)
	CT(45593)=(S=12152188,E=12152191,C=14)
	CT(45594)=(S=12152192,E=12152195,C=26)
	CT(45595)=(S=12152196,E=12152199,C=24)
	CT(45596)=(S=12152200,E=12152207,C=14)
	CT(45597)=(S=12152208,E=12152211,C=36)
	CT(45598)=(S=12152212,E=12152215,C=17)
	CT(45599)=(S=12152216,E=12152219,C=14)
	CT(45600)=(S=12152220,E=12152223,C=28)
	CT(45601)=(S=12152224,E=12152227,C=36)
	CT(45602)=(S=12152228,E=12152231,C=34)
	CT(45603)=(S=12152232,E=12152235,C=14)
	CT(45604)=(S=12152236,E=12152239,C=12)
	CT(45605)=(S=12152240,E=12152243,C=23)
	CT(45606)=(S=12152244,E=12152247,C=28)
	CT(45607)=(S=12152248,E=12152255,C=15)
	CT(45608)=(S=12152256,E=12152259,C=61)
	CT(45609)=(S=12152260,E=12152263,C=28)
	CT(45610)=(S=12152264,E=12152267,C=12)
	CT(45611)=(S=12152268,E=12152275,C=18)
	CT(45612)=(S=12152276,E=12152279,C=38)
	CT(45613)=(S=12152280,E=12152283,C=58)
	CT(45614)=(S=12152284,E=12152287,C=15)
	CT(45615)=(S=12152288,E=12152291,C=63)
	CT(45616)=(S=12152292,E=12152295,C=14)
	CT(45617)=(S=12152296,E=12152299,C=36)
	CT(45618)=(S=12152300,E=12152303,C=28)
	CT(45619)=(S=12152304,E=12152307,C=34)
	CT(45620)=(S=12152308,E=12152315,C=26)
	CT(45621)=(S=12152316,E=12152319,C=38)
	CT(45622)=(S=12152320,E=12152323,C=14)
	CT(45623)=(S=12152324,E=12152327,C=58)
	CT(45624)=(S=12152328,E=12152331,C=18)
	CT(45625)=(S=12152332,E=12152335,C=28)
	CT(45626)=(S=12152336,E=12152339,C=25)
	CT(45627)=(S=12152340,E=12152343,C=15)
	CT(45628)=(S=12152344,E=12152347,C=38)
	CT(45629)=(S=12152348,E=12152351,C=26)
	CT(45630)=(S=12152352,E=12152355,C=28)
	CT(45631)=(S=12152356,E=12152359,C=172)
	CT(45632)=(S=12152360,E=12152363,C=28)
	CT(45633)=(S=12152364,E=12152367,C=18)
	CT(45634)=(S=12152368,E=12152371,C=58)
	CT(45635)=(S=12152372,E=12152375,C=14)
	CT(45636)=(S=12152376,E=12152379,C=18)
	CT(45637)=(S=12152380,E=12152383,C=51)
	CT(45638)=(S=12152384,E=12152391,C=25)
	CT(45639)=(S=12152392,E=12152395,C=34)
	CT(45640)=(S=12152396,E=12152399,C=25)
	CT(45641)=(S=12152400,E=12152403,C=24)
	CT(45642)=(S=12152404,E=12152407,C=12)
	CT(45643)=(S=12152408,E=12152411,C=34)
	CT(45644)=(S=12152412,E=12152415,C=36)
	CT(45645)=(S=12152416,E=12152419,C=12)
	CT(45646)=(S=12152420,E=12152423,C=34)
	CT(45647)=(S=12152424,E=12152427,C=30)
	CT(45648)=(S=12152428,E=12152431,C=17)
	CT(45649)=(S=12152432,E=12152435,C=36)
	CT(45650)=(S=12152436,E=12152439,C=15)
	CT(45651)=(S=12152440,E=12152443,C=42)
	CT(45652)=(S=12152444,E=12152447,C=28)
	CT(45653)=(S=12152448,E=12152451,C=18)
	CT(45654)=(S=12152452,E=12152455,C=31)
	CT(45655)=(S=12152456,E=12152459,C=42)
	CT(45656)=(S=12152460,E=12152467,C=71)
	CT(45657)=(S=12152468,E=12152471,C=27)
	CT(45658)=(S=12152472,E=12152475,C=28)
	CT(45659)=(S=12152476,E=12152483,C=25)
	CT(45660)=(S=12152484,E=12152487,C=40)
	CT(45661)=(S=12152488,E=12152491,C=18)
	CT(45662)=(S=12152492,E=12152495,C=36)
	CT(45663)=(S=12152496,E=12152503,C=14)
	CT(45664)=(S=12152504,E=12152507,C=34)
	CT(45665)=(S=12152508,E=12152511,C=14)
	CT(45666)=(S=12152512,E=12152515,C=12)
	CT(45667)=(S=12152516,E=12152519,C=34)
	CT(45668)=(S=12152520,E=12152523,C=36)
	CT(45669)=(S=12152524,E=12152527,C=33)
	CT(45670)=(S=12152528,E=12152531,C=18)
	CT(45671)=(S=12152532,E=12152535,C=25)
	CT(45672)=(S=12152536,E=12152539,C=26)
	CT(45673)=(S=12152540,E=12152543,C=34)
	CT(45674)=(S=12152544,E=12152547,C=25)
	CT(45675)=(S=12152548,E=12152551,C=26)
	CT(45676)=(S=12152552,E=12152555,C=23)
	CT(45677)=(S=12152556,E=12152559,C=26)
	CT(45678)=(S=12152560,E=12152563,C=37)
	CT(45679)=(S=12152564,E=12152567,C=26)
	CT(45680)=(S=12152568,E=12152571,C=14)
	CT(45681)=(S=12152572,E=12152575,C=26)
	CT(45682)=(S=12152576,E=12152579,C=25)
	CT(45683)=(S=12152580,E=12152583,C=39)
	CT(45684)=(S=12152584,E=12152591,C=26)
	CT(45685)=(S=12152592,E=12152595,C=12)
	CT(45686)=(S=12152596,E=12152599,C=14)
	CT(45687)=(S=12152600,E=12152603,C=18)
	CT(45688)=(S=12152604,E=12152607,C=36)
	CT(45689)=(S=12152608,E=12152611,C=28)
	CT(45690)=(S=12152612,E=12152615,C=48)
	CT(45691)=(S=12152616,E=12152619,C=26)
	CT(45692)=(S=12152620,E=12152623,C=12)
	CT(45693)=(S=12152624,E=12152627,C=18)
	CT(45694)=(S=12152628,E=12152635,C=12)
	CT(45695)=(S=12152636,E=12152639,C=28)
	CT(45696)=(S=12152640,E=12152643,C=26)
	CT(45697)=(S=12152644,E=12152647,C=41)
	CT(45698)=(S=12152648,E=12152651,C=36)
	CT(45699)=(S=12152652,E=12152655,C=15)
	CT(45700)=(S=12152656,E=12152659,C=36)
	CT(45701)=(S=12152660,E=12152663,C=18)
	CT(45702)=(S=12152664,E=12152667,C=50)
	CT(45703)=(S=12152668,E=12152671,C=31)
	CT(45704)=(S=12152672,E=12152675,C=33)
	CT(45705)=(S=12152676,E=12152679,C=18)
	CT(45706)=(S=12152680,E=12152683,C=20)
	CT(45707)=(S=12152684,E=12152687,C=33)
	CT(45708)=(S=12152688,E=12152691,C=63)
	CT(45709)=(S=12152692,E=12152695,C=31)
	CT(45710)=(S=12152696,E=12152699,C=36)
	CT(45711)=(S=12152700,E=12152703,C=42)
	CT(45712)=(S=12152704,E=12152707,C=26)
	CT(45713)=(S=12152708,E=12152711,C=58)
	CT(45714)=(S=12152712,E=12152715,C=26)
	CT(45715)=(S=12152716,E=12152719,C=63)
	CT(45716)=(S=12152720,E=12152723,C=42)
	CT(45717)=(S=12152724,E=12152727,C=25)
	CT(45718)=(S=12152728,E=12152731,C=19)
	CT(45719)=(S=12152732,E=12152735,C=34)
	CT(45720)=(S=12152736,E=12152739,C=39)
	CT(45721)=(S=12152740,E=12152743,C=18)
	CT(45722)=(S=12152744,E=12152747,C=28)
	CT(45723)=(S=12152748,E=12152751,C=24)
	CT(45724)=(S=12152752,E=12152755,C=34)
	CT(45725)=(S=12152756,E=12152759,C=44)
	CT(45726)=(S=12152760,E=12152763,C=25)
	CT(45727)=(S=12152764,E=12152767,C=57)
	CT(45728)=(S=12152768,E=12152771,C=28)
	CT(45729)=(S=12152772,E=12152775,C=49)
	CT(45730)=(S=12152776,E=12152779,C=36)
	CT(45731)=(S=12152780,E=12152783,C=49)
	CT(45732)=(S=12152784,E=12152787,C=50)
	CT(45733)=(S=12152788,E=12152791,C=51)
	CT(45734)=(S=12152792,E=12152795,C=18)
	CT(45735)=(S=12152796,E=12152799,C=28)
	CT(45736)=(S=12152800,E=12152803,C=14)
	CT(45737)=(S=12152804,E=12152807,C=25)
	CT(45738)=(S=12152808,E=12152811,C=37)
	CT(45739)=(S=12152812,E=12152815,C=12)
	CT(45740)=(S=12152816,E=12152819,C=19)
	CT(45741)=(S=12152820,E=12152823,C=37)
	CT(45742)=(S=12152824,E=12152827,C=15)
	CT(45743)=(S=12152828,E=12152835,C=25)
	CT(45744)=(S=12152836,E=12152839,C=15)
	CT(45745)=(S=12152840,E=12152843,C=34)
	CT(45746)=(S=12152844,E=12152847,C=31)
	CT(45747)=(S=12152848,E=12152851,C=12)
	CT(45748)=(S=12152852,E=12152855,C=46)
	CT(45749)=(S=12152856,E=12152859,C=27)
	CT(45750)=(S=12152860,E=12152863,C=33)
	CT(45751)=(S=12152864,E=12152871,C=26)
	CT(45752)=(S=12152872,E=12152875,C=58)
	CT(45753)=(S=12152876,E=12152879,C=28)
	CT(45754)=(S=12152880,E=12152883,C=56)
	CT(45755)=(S=12152884,E=12152887,C=34)
	CT(45756)=(S=12152888,E=12152891,C=18)
	CT(45757)=(S=12152892,E=12152895,C=15)
	CT(45758)=(S=12152896,E=12152899,C=36)
	CT(45759)=(S=12152900,E=12152903,C=28)
	CT(45760)=(S=12152904,E=12152907,C=34)
	CT(45761)=(S=12152908,E=12152911,C=25)
	CT(45762)=(S=12152912,E=12152915,C=18)
	CT(45763)=(S=12152916,E=12152927,C=15)
	CT(45764)=(S=12152928,E=12152931,C=14)
	CT(45765)=(S=12152932,E=12152935,C=18)
	CT(45766)=(S=12152936,E=12152939,C=45)
	CT(45767)=(S=12152940,E=12152943,C=36)
	CT(45768)=(S=12152944,E=12152947,C=19)
	CT(45769)=(S=12152948,E=12152951,C=58)
	CT(45770)=(S=12152952,E=12152955,C=18)
	CT(45771)=(S=12152956,E=12152959,C=28)
	CT(45772)=(S=12152960,E=12152963,C=26)
	CT(45773)=(S=12152964,E=12152967,C=34)
	CT(45774)=(S=12152968,E=12152971,C=19)
	CT(45775)=(S=12152972,E=12152975,C=18)
	CT(45776)=(S=12152976,E=12152979,C=48)
	CT(45777)=(S=12152980,E=12152983,C=26)
	CT(45778)=(S=12152984,E=12152987,C=28)
	CT(45779)=(S=12152988,E=12152991,C=50)
	CT(45780)=(S=12152992,E=12152995,C=14)
	CT(45781)=(S=12152996,E=12152999,C=33)
	CT(45782)=(S=12153000,E=12153003,C=26)
	CT(45783)=(S=12153004,E=12153007,C=31)
	CT(45784)=(S=12153008,E=12153011,C=28)
	CT(45785)=(S=12153012,E=12153015,C=58)
	CT(45786)=(S=12153016,E=12153019,C=38)
	CT(45787)=(S=12153020,E=12153023,C=35)
	CT(45788)=(S=12153024,E=12153027,C=41)
	CT(45789)=(S=12153028,E=12153031,C=15)
	CT(45790)=(S=12153032,E=12153035,C=67)
	CT(45791)=(S=12153036,E=12153039,C=48)
	CT(45792)=(S=12153040,E=12153043,C=24)
	CT(45793)=(S=12153044,E=12153047,C=15)
	CT(45794)=(S=12153048,E=12153051,C=28)
	CT(45795)=(S=12153052,E=12153055,C=14)
	CT(45796)=(S=12153056,E=12153059,C=18)
	CT(45797)=(S=12153060,E=12153063,C=12)
	CT(45798)=(S=12153064,E=12153067,C=35)
	CT(45799)=(S=12153068,E=12153071,C=18)
	CT(45800)=(S=12153072,E=12153083,C=14)
	CT(45801)=(S=12153084,E=12153087,C=28)
	CT(45802)=(S=12153088,E=12153091,C=14)
	CT(45803)=(S=12153092,E=12153095,C=28)
	CT(45804)=(S=12153096,E=12153099,C=37)
	CT(45805)=(S=12153100,E=12153103,C=18)
	CT(45806)=(S=12153104,E=12153107,C=14)
	CT(45807)=(S=12153108,E=12153111,C=34)
	CT(45808)=(S=12153112,E=12153115,C=14)
	CT(45809)=(S=12153116,E=12153119,C=18)
	CT(45810)=(S=12153120,E=12153123,C=58)
	CT(45811)=(S=12153124,E=12153127,C=20)
	CT(45812)=(S=12153128,E=12153131,C=25)
	CT(45813)=(S=12153132,E=12153135,C=71)
	CT(45814)=(S=12153136,E=12153139,C=12)
	CT(45815)=(S=12153140,E=12153143,C=36)
	CT(45816)=(S=12153144,E=12153147,C=26)
	CT(45817)=(S=12153148,E=12153151,C=14)
	CT(45818)=(S=12153152,E=12153155,C=12)
	CT(45819)=(S=12153156,E=12153159,C=14)
	CT(45820)=(S=12153160,E=12153167,C=18)
	CT(45821)=(S=12153168,E=12153171,C=14)
	CT(45822)=(S=12153172,E=12153179,C=36)
	CT(45823)=(S=12153180,E=12153183,C=14)
	CT(45824)=(S=12153184,E=12153187,C=19)
	CT(45825)=(S=12153188,E=12153191,C=42)
	CT(45826)=(S=12153192,E=12153195,C=26)
	CT(45827)=(S=12153196,E=12153199,C=18)
	CT(45828)=(S=12153200,E=12153203,C=26)
	CT(45829)=(S=12153204,E=12153207,C=18)
	CT(45830)=(S=12153208,E=12153215,C=28)
	CT(45831)=(S=12153216,E=12153219,C=14)
	CT(45832)=(S=12153220,E=12153223,C=20)
	CT(45833)=(S=12153224,E=12153227,C=18)
	CT(45834)=(S=12153228,E=12153231,C=21)
	CT(45835)=(S=12153232,E=12153235,C=28)
	CT(45836)=(S=12153236,E=12153243,C=26)
	CT(45837)=(S=12153244,E=12153247,C=21)
	CT(45838)=(S=12153248,E=12153251,C=12)
	CT(45839)=(S=12153252,E=12153255,C=28)
	CT(45840)=(S=12153256,E=12153259,C=60)
	CT(45841)=(S=12153260,E=12153263,C=36)
	CT(45842)=(S=12153264,E=12153267,C=19)
	CT(45843)=(S=12153268,E=12153271,C=14)
	CT(45844)=(S=12153272,E=12153275,C=25)
	CT(45845)=(S=12153276,E=12153279,C=14)
	CT(45846)=(S=12153280,E=12153283,C=25)
	CT(45847)=(S=12153284,E=12153287,C=36)
	CT(45848)=(S=12153288,E=12153291,C=18)
	CT(45849)=(S=12153292,E=12153295,C=14)
	CT(45850)=(S=12153296,E=12153299,C=18)
	CT(45851)=(S=12153300,E=12153303,C=36)
	CT(45852)=(S=12153304,E=12153307,C=14)
	CT(45853)=(S=12153308,E=12153311,C=37)
	CT(45854)=(S=12153312,E=12153315,C=36)
	CT(45855)=(S=12153316,E=12153319,C=24)
	CT(45856)=(S=12153320,E=12153323,C=63)
	CT(45857)=(S=12153324,E=12153327,C=25)
	CT(45858)=(S=12153328,E=12153331,C=14)
	CT(45859)=(S=12153332,E=12153335,C=18)
	CT(45860)=(S=12153336,E=12153339,C=63)
	CT(45861)=(S=12153340,E=12153343,C=55)
	CT(45862)=(S=12153344,E=12153347,C=15)
	CT(45863)=(S=12153348,E=12153355,C=12)
	CT(45864)=(S=12153356,E=12153363,C=28)
	CT(45865)=(S=12153364,E=12153367,C=37)
	CT(45866)=(S=12153368,E=12153383,C=36)
	CT(45867)=(S=12153384,E=12153387,C=28)
	CT(45868)=(S=12153388,E=12153391,C=24)
	CT(45869)=(S=12153392,E=12153395,C=49)
	CT(45870)=(S=12153396,E=12153399,C=38)
	CT(45871)=(S=12153400,E=12153403,C=27)
	CT(45872)=(S=12153404,E=12153407,C=36)
	CT(45873)=(S=12153408,E=12153411,C=25)
	CT(45874)=(S=12153412,E=12153415,C=23)
	CT(45875)=(S=12153416,E=12153419,C=26)
	CT(45876)=(S=12153420,E=12153423,C=36)
	CT(45877)=(S=12153424,E=12153427,C=15)
	CT(45878)=(S=12153428,E=12153431,C=38)
	CT(45879)=(S=12153432,E=12153435,C=39)
	CT(45880)=(S=12153436,E=12153439,C=58)
	CT(45881)=(S=12153440,E=12153443,C=14)
	CT(45882)=(S=12153444,E=12153447,C=12)
	CT(45883)=(S=12153448,E=12153451,C=19)
	CT(45884)=(S=12153452,E=12153455,C=15)
	CT(45885)=(S=12153456,E=12153459,C=96)
	CT(45886)=(S=12153460,E=12153463,C=15)
	CT(45887)=(S=12153464,E=12153467,C=17)
	CT(45888)=(S=12153468,E=12153471,C=28)
	CT(45889)=(S=12153472,E=12153475,C=12)
	CT(45890)=(S=12153476,E=12153479,C=28)
	CT(45891)=(S=12153480,E=12153483,C=31)
	CT(45892)=(S=12153484,E=12153487,C=36)
	CT(45893)=(S=12153488,E=12153491,C=52)
	CT(45894)=(S=12153492,E=12153495,C=14)
	CT(45895)=(S=12153496,E=12153499,C=29)
	CT(45896)=(S=12153500,E=12153503,C=36)
	CT(45897)=(S=12153504,E=12153507,C=63)
	CT(45898)=(S=12153508,E=12153511,C=36)
	CT(45899)=(S=12153512,E=12153515,C=28)
	CT(45900)=(S=12153516,E=12153519,C=25)
	CT(45901)=(S=12153520,E=12153523,C=55)
	CT(45902)=(S=12153524,E=12153527,C=51)
	CT(45903)=(S=12153528,E=12153531,C=36)
	CT(45904)=(S=12153532,E=12153535,C=26)
	CT(45905)=(S=12153536,E=12153539,C=37)
	CT(45906)=(S=12153540,E=12153543,C=36)
	CT(45907)=(S=12153544,E=12153547,C=28)
	CT(45908)=(S=12153548,E=12153551,C=14)
	CT(45909)=(S=12153552,E=12153555,C=34)
	CT(45910)=(S=12153556,E=12153567,C=28)
	CT(45911)=(S=12153568,E=12153571,C=42)
	CT(45912)=(S=12153572,E=12153575,C=14)
	CT(45913)=(S=12153576,E=12153579,C=15)
	CT(45914)=(S=12153580,E=12153583,C=36)
	CT(45915)=(S=12153584,E=12153587,C=30)
	CT(45916)=(S=12153588,E=12153591,C=18)
	CT(45917)=(S=12153592,E=12153595,C=19)
	CT(45918)=(S=12153596,E=12153599,C=39)
	CT(45919)=(S=12153600,E=12153603,C=33)
	CT(45920)=(S=12153604,E=12153607,C=40)
	CT(45921)=(S=12153608,E=12153611,C=33)
	CT(45922)=(S=12153612,E=12153615,C=72)
	CT(45923)=(S=12153616,E=12153619,C=47)
	CT(45924)=(S=12153620,E=12153623,C=50)
	CT(45925)=(S=12153624,E=12153627,C=28)
	CT(45926)=(S=12153628,E=12153631,C=38)
	CT(45927)=(S=12153632,E=12153635,C=36)
	CT(45928)=(S=12153636,E=12153639,C=31)
	CT(45929)=(S=12153640,E=12153643,C=37)
	CT(45930)=(S=12153644,E=12153647,C=58)
	CT(45931)=(S=12153648,E=12153651,C=28)
	CT(45932)=(S=12153652,E=12153655,C=25)
	CT(45933)=(S=12153656,E=12153659,C=57)
	CT(45934)=(S=12153660,E=12153663,C=14)
	CT(45935)=(S=12153664,E=12153667,C=38)
	CT(45936)=(S=12153668,E=12153671,C=14)
	CT(45937)=(S=12153672,E=12153675,C=33)
	CT(45938)=(S=12153676,E=12153679,C=26)
	CT(45939)=(S=12153680,E=12153683,C=28)
	CT(45940)=(S=12153684,E=12153687,C=14)
	CT(45941)=(S=12153688,E=12153691,C=36)
	CT(45942)=(S=12153692,E=12153695,C=38)
	CT(45943)=(S=12153696,E=12153699,C=18)
	CT(45944)=(S=12153700,E=12153703,C=39)
	CT(45945)=(S=12153704,E=12153707,C=28)
	CT(45946)=(S=12153708,E=12153711,C=14)
	CT(45947)=(S=12153712,E=12153715,C=58)
	CT(45948)=(S=12153716,E=12153723,C=14)
	CT(45949)=(S=12153724,E=12153727,C=28)
	CT(45950)=(S=12153728,E=12153731,C=14)
	CT(45951)=(S=12153732,E=12153735,C=15)
	CT(45952)=(S=12153736,E=12153739,C=40)
	CT(45953)=(S=12153740,E=12153743,C=18)
	CT(45954)=(S=12153744,E=12153747,C=36)
	CT(45955)=(S=12153748,E=12153751,C=26)
	CT(45956)=(S=12153752,E=12153755,C=58)
	CT(45957)=(S=12153756,E=12153763,C=18)
	CT(45958)=(S=12153764,E=12153771,C=26)
	CT(45959)=(S=12153772,E=12153779,C=28)
	CT(45960)=(S=12153780,E=12153783,C=33)
	CT(45961)=(S=12153784,E=12153787,C=42)
	CT(45962)=(S=12153788,E=12153791,C=12)
	CT(45963)=(S=12153792,E=12153795,C=28)
	CT(45964)=(S=12153796,E=12153803,C=14)
	CT(45965)=(S=12153804,E=12153807,C=22)
	CT(45966)=(S=12153808,E=12153811,C=37)
	CT(45967)=(S=12153812,E=12153815,C=17)
	CT(45968)=(S=12153816,E=12153819,C=49)
	CT(45969)=(S=12153820,E=12153823,C=38)
	CT(45970)=(S=12153824,E=12153827,C=17)
	CT(45971)=(S=12153828,E=12153831,C=14)
	CT(45972)=(S=12153832,E=12153835,C=25)
	CT(45973)=(S=12153836,E=12153839,C=28)
	CT(45974)=(S=12153840,E=12153843,C=47)
	CT(45975)=(S=12153844,E=12153855,C=27)
	CT(45976)=(S=12153856,E=12153859,C=15)
	CT(45977)=(S=12153860,E=12153863,C=27)
	CT(45978)=(S=12153864,E=12153867,C=24)
	CT(45979)=(S=12153868,E=12153871,C=36)
	CT(45980)=(S=12153872,E=12153875,C=28)
	CT(45981)=(S=12153876,E=12153883,C=26)
	CT(45982)=(S=12153884,E=12153887,C=42)
	CT(45983)=(S=12153888,E=12153891,C=47)
	CT(45984)=(S=12153892,E=12153895,C=28)
	CT(45985)=(S=12153896,E=12153903,C=36)
	CT(45986)=(S=12153904,E=12153907,C=33)
	CT(45987)=(S=12153908,E=12153915,C=14)
	CT(45988)=(S=12153916,E=12153919,C=49)
	CT(45989)=(S=12153920,E=12153923,C=63)
	CT(45990)=(S=12153924,E=12153927,C=14)
	CT(45991)=(S=12153928,E=12153931,C=49)
	CT(45992)=(S=12153932,E=12153935,C=33)
	CT(45993)=(S=12153936,E=12153939,C=36)
	CT(45994)=(S=12153940,E=12153943,C=15)
	CT(45995)=(S=12153944,E=12153947,C=12)
	CT(45996)=(S=12153948,E=12153951,C=51)
	CT(45997)=(S=12153952,E=12153955,C=42)
	CT(45998)=(S=12153956,E=12153959,C=17)
	CT(45999)=(S=12153960,E=12153963,C=12)
	CT(46000)=(S=12153964,E=12153971,C=14)
	CT(46001)=(S=12153972,E=12153975,C=39)
	CT(46002)=(S=12153976,E=12153979,C=19)
	CT(46003)=(S=12153980,E=12153983,C=36)
	CT(46004)=(S=12153984,E=12153991,C=12)
	CT(46005)=(S=12153992,E=12153995,C=58)
	CT(46006)=(S=12153996,E=12153999,C=20)
	CT(46007)=(S=12154000,E=12154003,C=36)
	CT(46008)=(S=12154004,E=12154007,C=24)
	CT(46009)=(S=12154008,E=12154011,C=37)
	CT(46010)=(S=12154012,E=12154015,C=28)
	CT(46011)=(S=12154016,E=12154019,C=26)
	CT(46012)=(S=12154020,E=12154023,C=36)
	CT(46013)=(S=12154024,E=12154027,C=63)
	CT(46014)=(S=12154028,E=12154031,C=14)
	CT(46015)=(S=12154032,E=12154035,C=12)
	CT(46016)=(S=12154036,E=12154039,C=25)
	CT(46017)=(S=12154040,E=12154047,C=24)
	CT(46018)=(S=12154048,E=12154051,C=20)
	CT(46019)=(S=12154052,E=12154059,C=18)
	CT(46020)=(S=12154060,E=12154063,C=51)
	CT(46021)=(S=12154064,E=12154067,C=28)
	CT(46022)=(S=12154068,E=12154071,C=14)
	CT(46023)=(S=12154072,E=12154075,C=36)
	CT(46024)=(S=12154076,E=12154079,C=28)
	CT(46025)=(S=12154080,E=12154083,C=23)
	CT(46026)=(S=12154084,E=12154087,C=18)
	CT(46027)=(S=12154088,E=12154091,C=28)
	CT(46028)=(S=12154092,E=12154095,C=14)
	CT(46029)=(S=12154096,E=12154099,C=27)
	CT(46030)=(S=12154100,E=12154107,C=28)
	CT(46031)=(S=12154108,E=12154111,C=58)
	CT(46032)=(S=12154112,E=12154115,C=5)
	CT(46033)=(S=12154116,E=12154119,C=36)
	CT(46034)=(S=12154120,E=12154123,C=35)
	CT(46035)=(S=12154124,E=12154127,C=34)
	CT(46036)=(S=12154128,E=12154135,C=12)
	CT(46037)=(S=12154136,E=12154139,C=36)
	CT(46038)=(S=12154140,E=12154143,C=18)
	CT(46039)=(S=12154144,E=12154147,C=22)
	CT(46040)=(S=12154148,E=12154151,C=25)
	CT(46041)=(S=12154152,E=12154155,C=42)
	CT(46042)=(S=12154156,E=12154159,C=25)
	CT(46043)=(S=12154160,E=12154163,C=26)
	CT(46044)=(S=12154164,E=12154167,C=28)
	CT(46045)=(S=12154168,E=12154175,C=12)
	CT(46046)=(S=12154176,E=12154179,C=55)
	CT(46047)=(S=12154180,E=12154183,C=16)
	CT(46048)=(S=12154184,E=12154191,C=36)
	CT(46049)=(S=12154192,E=12154195,C=51)
	CT(46050)=(S=12154196,E=12154199,C=58)
	CT(46051)=(S=12154200,E=12154203,C=36)
	CT(46052)=(S=12154204,E=12154207,C=12)
	CT(46053)=(S=12154208,E=12154211,C=26)
	CT(46054)=(S=12154212,E=12154215,C=18)
	CT(46055)=(S=12154216,E=12154219,C=19)
	CT(46056)=(S=12154220,E=12154223,C=36)
	CT(46057)=(S=12154224,E=12154227,C=14)
	CT(46058)=(S=12154228,E=12154231,C=18)
	CT(46059)=(S=12154232,E=12154235,C=37)
	CT(46060)=(S=12154236,E=12154239,C=36)
	CT(46061)=(S=12154240,E=12154243,C=23)
	CT(46062)=(S=12154244,E=12154247,C=32)
	CT(46063)=(S=12154248,E=12154251,C=26)
	CT(46064)=(S=12154252,E=12154259,C=18)
	CT(46065)=(S=12154260,E=12154263,C=62)
	CT(46066)=(S=12154264,E=12154271,C=18)
	CT(46067)=(S=12154272,E=12154275,C=28)
	CT(46068)=(S=12154276,E=12154279,C=56)
	CT(46069)=(S=12154280,E=12154283,C=34)
	CT(46070)=(S=12154284,E=12154291,C=25)
	CT(46071)=(S=12154292,E=12154295,C=28)
	CT(46072)=(S=12154296,E=12154299,C=15)
	CT(46073)=(S=12154300,E=12154303,C=51)
	CT(46074)=(S=12154304,E=12154307,C=27)
	CT(46075)=(S=12154308,E=12154315,C=14)
	CT(46076)=(S=12154316,E=12154319,C=26)
	CT(46077)=(S=12154320,E=12154323,C=36)
	CT(46078)=(S=12154324,E=12154327,C=28)
	CT(46079)=(S=12154328,E=12154331,C=15)
	CT(46080)=(S=12154332,E=12154335,C=47)
	CT(46081)=(S=12154336,E=12154339,C=39)
	CT(46082)=(S=12154340,E=12154343,C=28)
	CT(46083)=(S=12154344,E=12154347,C=18)
	CT(46084)=(S=12154348,E=12154351,C=14)
	CT(46085)=(S=12154352,E=12154355,C=31)
	CT(46086)=(S=12154356,E=12154359,C=18)
	CT(46087)=(S=12154360,E=12154371,C=28)
	CT(46088)=(S=12154372,E=12154375,C=38)
	CT(46089)=(S=12154376,E=12154379,C=35)
	CT(46090)=(S=12154380,E=12154383,C=26)
	CT(46091)=(S=12154384,E=12154387,C=12)
	CT(46092)=(S=12189696,E=12189823,C=181)
	CT(46093)=(S=12189824,E=12189851,C=83)
	CT(46094)=(S=12189852,E=12189855,C=201)
	CT(46095)=(S=12189856,E=12189883,C=83)
	CT(46096)=(S=12189884,E=12189887,C=206)
	CT(46097)=(S=12189888,E=12189951,C=83)
	CT(46098)=(S=12189952,E=12190015,C=216)
	CT(46099)=(S=12190016,E=12190079,C=177)
	CT(46100)=(S=12190080,E=12190111,C=209)
	CT(46101)=(S=12190112,E=12190143,C=181)
	CT(46102)=(S=12190144,E=12190159,C=218)
	CT(46103)=(S=12190160,E=12190199,C=83)
	CT(46104)=(S=12190200,E=12190207,C=181)
	CT(46105)=(S=12190208,E=12190335,C=201)
	CT(46106)=(S=12190336,E=12190367,C=208)
	CT(46107)=(S=12190368,E=12190383,C=170)
	CT(46108)=(S=12190384,E=12190399,C=207)
	CT(46109)=(S=12190400,E=12190447,C=206)
	CT(46110)=(S=12190448,E=12190451,C=202)
	CT(46111)=(S=12190452,E=12190455,C=222)
	CT(46112)=(S=12190460,E=12190463,C=203)
	CT(46113)=(S=12190464,E=12190719,C=213)
	CT(46114)=(S=12190720,E=12190783,C=202)
	CT(46115)=(S=12190784,E=12190847,C=83)
	CT(46116)=(S=12190848,E=12191103,C=213)
	CT(46117)=(S=12191104,E=12191135,C=209)
	CT(46118)=(S=12191136,E=12191151,C=202)
	CT(46119)=(S=12191152,E=12191207,C=83)
	CT(46120)=(S=12191208,E=12191215,C=203)
	CT(46121)=(S=12191216,E=12191231,C=83)
	CT(46122)=(S=12191232,E=12191743,C=177)
	CT(46123)=(S=12191744,E=12191999,C=205)
	CT(46124)=(S=12192000,E=12192767,C=203)
	CT(46125)=(S=12192768,E=12193279,C=83)
	CT(46126)=(S=12193280,E=12193535,C=200)
	CT(46127)=(S=12193536,E=12193791,C=202)
	CT(46128)=(S=12193792,E=12194303,C=206)
	CT(46129)=(S=12194304,E=12194815,C=83)
	CT(46130)=(S=12194816,E=12195327,C=203)
	CT(46131)=(S=12195328,E=12195839,C=83)
	CT(46132)=(S=12195840,E=12196415,C=200)
	CT(46133)=(S=12196416,E=12196463,C=83)
	CT(46134)=(S=12196464,E=12196479,C=202)
	CT(46135)=(S=12196480,E=12196607,C=200)
	CT(46136)=(S=12196608,E=12196735,C=201)
	CT(46137)=(S=12196736,E=12197887,C=181)
	CT(46138)=(S=12197888,E=12197951,C=202)
	CT(46139)=(S=12197952,E=12198015,C=210)
	CT(46140)=(S=12198016,E=12198047,C=208)
	CT(46141)=(S=12198048,E=12198111,C=202)
	CT(46142)=(S=12198112,E=12198135,C=208)
	CT(46143)=(S=12198136,E=12198143,C=210)
	CT(46144)=(S=12198144,E=12198207,C=83)
	CT(46145)=(S=12198208,E=12198271,C=177)
	CT(46146)=(S=12198272,E=12198335,C=213)
	CT(46147)=(S=12198336,E=12198399,C=83)
	CT(46148)=(S=12198400,E=12199423,C=203)
	CT(46149)=(S=12199424,E=12199935,C=83)
	CT(46150)=(S=12199936,E=12200447,C=203)
	CT(46151)=(S=12200448,E=12200703,C=213)
	CT(46152)=(S=12200704,E=12200831,C=181)
	CT(46153)=(S=12200832,E=12200959,C=213)
	CT(46154)=(S=12200960,E=12201471,C=176)
	CT(46155)=(S=12201472,E=12201983,C=213)
	CT(46156)=(S=12201984,E=12204031,C=205)
	CT(46157)=(S=12204032,E=12206183,C=83)
	CT(46158)=(S=12206184,E=12206191,C=215)
	CT(46159)=(S=12206192,E=12206203,C=203)
	CT(46160)=(S=12206204,E=12206207,C=212)
	CT(46161)=(S=12206208,E=12206335,C=202)
	CT(46162)=(S=12206336,E=12206399,C=213)
	CT(46163)=(S=12206400,E=12206407,C=83)
	CT(46164)=(S=12206408,E=12206415,C=203)
	CT(46165)=(S=12206416,E=12206423,C=83)
	CT(46166)=(S=12206424,E=12206427,C=170)
	CT(46167)=(S=12206432,E=12206447,C=83)
	CT(46168)=(S=12206448,E=12206463,C=208)
	CT(46169)=(S=12206464,E=12206591,C=203)
	CT(46170)=(S=12206592,E=12206847,C=213)
	CT(46171)=(S=12206848,E=12207103,C=203)
	CT(46172)=(S=12207104,E=12208127,C=213)
	CT(46173)=(S=12208128,E=12209151,C=209)
	CT(46174)=(S=12209152,E=12209663,C=216)
	CT(46175)=(S=12209664,E=12210175,C=203)
	CT(46176)=(S=12210176,E=12212223,C=181)
	CT(46177)=(S=12212224,E=12214271,C=200)
	CT(46178)=(S=12214272,E=12214335,C=83)
	CT(46179)=(S=12214336,E=12214351,C=200)
	CT(46180)=(S=12214352,E=12214367,C=202)
	CT(46181)=(S=12214368,E=12214399,C=209)
	CT(46182)=(S=12214400,E=12214523,C=83)
	CT(46183)=(S=12214524,E=12214527,C=208)
	CT(46184)=(S=12214528,E=12215295,C=181)
	CT(46185)=(S=12215296,E=12215551,C=83)
	CT(46186)=(S=12215552,E=12215807,C=213)
	CT(46187)=(S=12215808,E=12216191,C=181)
	CT(46188)=(S=12216192,E=12217343,C=203)
	CT(46189)=(S=12217344,E=12218367,C=83)
	CT(46190)=(S=12218368,E=12220415,C=181)
	CT(46191)=(S=12220416,E=12220671,C=177)
	CT(46192)=(S=12220672,E=12220799,C=181)
	CT(46193)=(S=12220800,E=12220863,C=83)
	CT(46194)=(S=12220864,E=12220927,C=201)
	CT(46195)=(S=12220928,E=12226559,C=83)
	CT(46196)=(S=12226560,E=12227583,C=181)
	CT(46197)=(S=12227584,E=12227647,C=203)
	CT(46198)=(S=12227648,E=12227671,C=83)
	CT(46199)=(S=12227672,E=12227679,C=177)
	CT(46200)=(S=12227680,E=12227695,C=209)
	CT(46201)=(S=12227696,E=12227743,C=83)
	CT(46202)=(S=12227744,E=12227775,C=209)
	CT(46203)=(S=12227784,E=12227791,C=83)
	CT(46204)=(S=12227792,E=12227795,C=203)
	CT(46205)=(S=12227796,E=12227799,C=202)
	CT(46206)=(S=12227800,E=12227807,C=207)
	CT(46207)=(S=12227816,E=12227839,C=83)
	CT(46208)=(S=12227840,E=12228351,C=177)
	CT(46209)=(S=12228352,E=12228607,C=215)
	CT(46210)=(S=12228608,E=12229119,C=83)
	CT(46211)=(S=12229120,E=12229631,C=181)
	CT(46212)=(S=12229632,E=12229887,C=203)
	CT(46213)=(S=12229888,E=12230399,C=83)
	CT(46214)=(S=12230400,E=12230495,C=181)
	CT(46215)=(S=12230496,E=12230511,C=217)
	CT(46216)=(S=12230512,E=12230515,C=181)
	CT(46217)=(S=12230516,E=12230519,C=209)
	CT(46218)=(S=12230520,E=12230527,C=83)
	CT(46219)=(S=12230528,E=12230655,C=202)
	CT(46220)=(S=12230656,E=12231679,C=212)
	CT(46221)=(S=12231680,E=12232703,C=200)
	CT(46222)=(S=12232704,E=12233727,C=181)
	CT(46223)=(S=12233728,E=12234751,C=203)
	CT(46224)=(S=12234752,E=12235199,C=202)
	CT(46225)=(S=12235200,E=12235263,C=83)
	CT(46226)=(S=12235264,E=12235519,C=213)
	CT(46227)=(S=12235520,E=12235583,C=208)
	CT(46228)=(S=12235584,E=12235599,C=202)
	CT(46229)=(S=12235600,E=12235615,C=203)
	CT(46230)=(S=12235616,E=12235631,C=181)
	CT(46231)=(S=12235632,E=12235647,C=209)
	CT(46232)=(S=12235648,E=12235775,C=224)
	CT(46233)=(S=12235776,E=12236287,C=181)
	CT(46234)=(S=12236288,E=12236671,C=83)
	CT(46235)=(S=12236672,E=12236799,C=181)
	CT(46236)=(S=12236800,E=12237951,C=200)
	CT(46237)=(S=12237952,E=12238079,C=209)
	CT(46238)=(S=12238080,E=12238335,C=83)
	CT(46239)=(S=12238336,E=12238463,C=218)
	CT(46240)=(S=12238464,E=12238543,C=83)
	CT(46241)=(S=12238544,E=12238567,C=209)
	CT(46242)=(S=12238568,E=12238571,C=207)
	CT(46243)=(S=12238572,E=12238575,C=203)
	CT(46244)=(S=12238576,E=12238591,C=207)
	CT(46245)=(S=12238592,E=12238847,C=83)
	CT(46246)=(S=12238848,E=12239499,C=189)
	CT(46247)=(S=12239500,E=12239503,C=204)
	CT(46248)=(S=12239504,E=12253847,C=189)
	CT(46249)=(S=12253848,E=12253851,C=83)
	CT(46250)=(S=12253852,E=12268023,C=189)
	CT(46251)=(S=12268024,E=12268031,C=202)
	CT(46252)=(S=12268032,E=12287999,C=189)
	CT(46253)=(S=12288000,E=12301311,C=204)
	CT(46254)=(S=12301312,E=12302335,C=189)
	CT(46255)=(S=12302336,E=12304223,C=204)
	CT(46256)=(S=12304224,E=12304255,C=189)
	CT(46257)=(S=12304256,E=12320511,C=204)
	CT(46258)=(S=12320512,E=12320767,C=189)
	CT(46259)=(S=12320768,E=12320799,C=18)
	CT(46260)=(S=12320800,E=12320807,C=58)
	CT(46261)=(S=12320808,E=12320815,C=18)
	CT(46262)=(S=12320816,E=12320831,C=55)
	CT(46263)=(S=12320832,E=12320895,C=31)
	CT(46264)=(S=12320896,E=12320927,C=20)
	CT(46265)=(S=12320928,E=12320991,C=18)
	CT(46266)=(S=12320992,E=12321023,C=53)
	CT(46267)=(S=12321024,E=12321279,C=28)
	CT(46268)=(S=12321280,E=12321535,C=46)
	CT(46269)=(S=12321536,E=12321791,C=37)
	CT(46270)=(S=12321792,E=12322047,C=22)
	CT(46271)=(S=12322048,E=12322303,C=49)
	CT(46272)=(S=12322304,E=12322559,C=50)
	CT(46273)=(S=12322560,E=12322815,C=12)
	CT(46274)=(S=12322816,E=12324863,C=15)
	CT(46275)=(S=12324864,E=12325887,C=18)
	CT(46276)=(S=12325888,E=12326911,C=42)
	CT(46277)=(S=12326912,E=12327935,C=38)
	CT(46278)=(S=12327936,E=12328959,C=14)
	CT(46279)=(S=12328960,E=12329215,C=18)
	CT(46280)=(S=12329216,E=12329471,C=58)
	CT(46281)=(S=12329472,E=12329727,C=26)
	CT(46282)=(S=12329728,E=12329983,C=18)
	CT(46283)=(S=12329984,E=12330239,C=50)
	CT(46284)=(S=12330240,E=12330495,C=21)
	CT(46285)=(S=12330496,E=12330751,C=37)
	CT(46286)=(S=12330752,E=12331007,C=14)
	CT(46287)=(S=12331008,E=12331263,C=28)
	CT(46288)=(S=12331264,E=12331519,C=37)
	CT(46289)=(S=12331520,E=12331775,C=70)
	CT(46290)=(S=12331776,E=12332031,C=18)
	CT(46291)=(S=12332032,E=12332063,C=54)
	CT(46292)=(S=12332064,E=12332095,C=18)
	CT(46293)=(S=12332096,E=12332127,C=49)
	CT(46294)=(S=12332128,E=12332159,C=18)
	CT(46295)=(S=12332160,E=12332287,C=50)
	CT(46296)=(S=12332288,E=12332543,C=42)
	CT(46297)=(S=12332544,E=12332799,C=28)
	CT(46298)=(S=12332800,E=12333055,C=58)
	CT(46299)=(S=12333056,E=12335103,C=23)
	CT(46300)=(S=12335104,E=12336127,C=37)
	CT(46301)=(S=12336128,E=12337151,C=34)
	CT(46302)=(S=12337152,E=12337159,C=47)
	CT(46303)=(S=12337160,E=12337167,C=41)
	CT(46304)=(S=12337168,E=12337175,C=28)
	CT(46305)=(S=12337176,E=12337183,C=52)
	CT(46306)=(S=12337184,E=12337191,C=14)
	CT(46307)=(S=12337192,E=12337199,C=28)
	CT(46308)=(S=12337200,E=12337207,C=49)
	CT(46309)=(S=12337208,E=12337215,C=28)
	CT(46310)=(S=12337216,E=12337223,C=36)
	CT(46311)=(S=12337224,E=12337231,C=49)
	CT(46312)=(S=12337232,E=12337239,C=58)
	CT(46313)=(S=12337240,E=12337247,C=15)
	CT(46314)=(S=12337248,E=12337255,C=17)
	CT(46315)=(S=12337256,E=12337263,C=52)
	CT(46316)=(S=12337264,E=12337271,C=18)
	CT(46317)=(S=12337272,E=12337279,C=15)
	CT(46318)=(S=12337280,E=12337287,C=18)
	CT(46319)=(S=12337288,E=12337295,C=32)
	CT(46320)=(S=12337296,E=12337303,C=18)
	CT(46321)=(S=12337304,E=12337311,C=24)
	CT(46322)=(S=12337312,E=12337327,C=18)
	CT(46323)=(S=12337328,E=12337335,C=60)
	CT(46324)=(S=12337336,E=12337343,C=14)
	CT(46325)=(S=12337344,E=12337351,C=28)
	CT(46326)=(S=12337352,E=12337359,C=17)
	CT(46327)=(S=12337360,E=12337367,C=37)
	CT(46328)=(S=12337368,E=12337375,C=18)
	CT(46329)=(S=12337376,E=12337383,C=12)
	CT(46330)=(S=12337384,E=12337391,C=49)
	CT(46331)=(S=12337392,E=12337399,C=12)
	CT(46332)=(S=12337400,E=12337407,C=28)
	CT(46333)=(S=12337408,E=12337415,C=15)
	CT(46334)=(S=12337416,E=12337423,C=18)
	CT(46335)=(S=12337432,E=12337439,C=45)
	CT(46336)=(S=12337440,E=12337447,C=14)
	CT(46337)=(S=12337448,E=12337455,C=58)
	CT(46338)=(S=12337456,E=12337463,C=18)
	CT(46339)=(S=12337464,E=12337471,C=14)
	CT(46340)=(S=12337472,E=12337479,C=18)
	CT(46341)=(S=12337480,E=12337487,C=42)
	CT(46342)=(S=12337488,E=12337495,C=15)
	CT(46343)=(S=12337496,E=12337503,C=25)
	CT(46344)=(S=12337504,E=12337511,C=14)
	CT(46345)=(S=12337512,E=12337519,C=18)
	CT(46346)=(S=12337520,E=12337527,C=14)
	CT(46347)=(S=12337528,E=12337535,C=12)
	CT(46348)=(S=12337536,E=12337543,C=18)
	CT(46349)=(S=12337544,E=12337551,C=47)
	CT(46350)=(S=12337552,E=12337559,C=28)
	CT(46351)=(S=12337560,E=12337567,C=19)
	CT(46352)=(S=12337568,E=12337575,C=36)
	CT(46353)=(S=12337576,E=12337583,C=12)
	CT(46354)=(S=12337584,E=12337591,C=14)
	CT(46355)=(S=12337592,E=12337599,C=72)
	CT(46356)=(S=12337600,E=12337607,C=36)
	CT(46357)=(S=12337608,E=12337615,C=28)
	CT(46358)=(S=12337616,E=12337623,C=18)
	CT(46359)=(S=12337624,E=12337631,C=49)
	CT(46360)=(S=12337632,E=12337639,C=36)
	CT(46361)=(S=12337640,E=12337655,C=18)
	CT(46362)=(S=12337656,E=12337663,C=14)
	CT(46363)=(S=12337664,E=12337671,C=55)
	CT(46364)=(S=12337672,E=12337679,C=49)
	CT(46365)=(S=12337680,E=12337687,C=36)
	CT(46366)=(S=12337696,E=12337703,C=18)
	CT(46367)=(S=12337704,E=12337711,C=28)
	CT(46368)=(S=12337728,E=12337791,C=14)
	CT(46369)=(S=12337792,E=12337919,C=45)
	CT(46370)=(S=12337920,E=12338175,C=47)
	CT(46371)=(S=12338176,E=12338207,C=18)
	CT(46372)=(S=12338208,E=12338239,C=28)
	CT(46373)=(S=12338240,E=12338339,C=18)
	CT(46374)=(S=12338340,E=12338343,C=14)
	CT(46375)=(S=12338344,E=12338399,C=18)
	CT(46376)=(S=12338400,E=12338431,C=58)
	CT(46377)=(S=12338432,E=12338687,C=44)
	CT(46378)=(S=12338688,E=12339199,C=101)
	CT(46379)=(S=12339200,E=12339263,C=35)
	CT(46380)=(S=12339264,E=12339271,C=36)
	CT(46381)=(S=12339272,E=12339279,C=18)
	CT(46382)=(S=12339280,E=12339327,C=36)
	CT(46383)=(S=12339328,E=12339391,C=41)
	CT(46384)=(S=12339392,E=12339455,C=28)
	CT(46385)=(S=12339456,E=12339519,C=12)
	CT(46386)=(S=12339520,E=12339583,C=28)
	CT(46387)=(S=12339584,E=12339647,C=18)
	CT(46388)=(S=12339648,E=12339711,C=22)
	CT(46389)=(S=12339712,E=12339775,C=28)
	CT(46390)=(S=12339776,E=12339839,C=14)
	CT(46391)=(S=12339840,E=12339967,C=38)
	CT(46392)=(S=12339968,E=12340031,C=18)
	CT(46393)=(S=12340032,E=12340095,C=26)
	CT(46394)=(S=12340096,E=12340159,C=33)
	CT(46395)=(S=12340160,E=12340223,C=18)
	CT(46396)=(S=12340224,E=12341247,C=25)
	CT(46397)=(S=12341248,E=12342271,C=21)
	CT(46398)=(S=12342272,E=12343295,C=25)
	CT(46399)=(S=12343296,E=12344319,C=36)
	CT(46400)=(S=12344320,E=12344327,C=18)
	CT(46401)=(S=12344328,E=12344335,C=33)
	CT(46402)=(S=12344336,E=12344343,C=60)
	CT(46403)=(S=12344344,E=12344351,C=15)
	CT(46404)=(S=12344352,E=12344359,C=28)
	CT(46405)=(S=12344360,E=12344367,C=33)
	CT(46406)=(S=12344368,E=12344375,C=34)
	CT(46407)=(S=12344376,E=12344383,C=36)
	CT(46408)=(S=12344384,E=12344391,C=12)
	CT(46409)=(S=12344392,E=12344399,C=60)
	CT(46410)=(S=12344400,E=12344407,C=27)
	CT(46411)=(S=12344408,E=12344415,C=14)
	CT(46412)=(S=12344416,E=12344423,C=33)
	CT(46413)=(S=12344424,E=12344431,C=18)
	CT(46414)=(S=12344432,E=12344455,C=28)
	CT(46415)=(S=12344456,E=12344463,C=14)
	CT(46416)=(S=12344464,E=12344471,C=34)
	CT(46417)=(S=12344472,E=12344479,C=36)
	CT(46418)=(S=12344480,E=12344487,C=55)
	CT(46419)=(S=12344488,E=12344495,C=19)
	CT(46420)=(S=12344496,E=12344503,C=18)
	CT(46421)=(S=12344504,E=12344511,C=15)
	CT(46422)=(S=12344512,E=12344519,C=18)
	CT(46423)=(S=12344520,E=12344527,C=28)
	CT(46424)=(S=12344528,E=12344535,C=40)
	CT(46425)=(S=12344536,E=12344543,C=18)
	CT(46426)=(S=12344544,E=12344551,C=34)
	CT(46427)=(S=12344552,E=12344559,C=14)
	CT(46428)=(S=12344560,E=12344567,C=18)
	CT(46429)=(S=12344568,E=12344575,C=19)
	CT(46430)=(S=12344576,E=12344583,C=63)
	CT(46431)=(S=12344584,E=12344591,C=28)
	CT(46432)=(S=12344592,E=12344607,C=18)
	CT(46433)=(S=12344608,E=12344615,C=25)
	CT(46434)=(S=12344616,E=12344623,C=12)
	CT(46435)=(S=12344624,E=12344639,C=18)
	CT(46436)=(S=12344640,E=12344647,C=26)
	CT(46437)=(S=12344648,E=12344655,C=25)
	CT(46438)=(S=12344656,E=12344663,C=49)
	CT(46439)=(S=12344664,E=12344671,C=40)
	CT(46440)=(S=12344672,E=12344679,C=49)
	CT(46441)=(S=12344680,E=12344687,C=18)
	CT(46442)=(S=12344688,E=12344695,C=49)
	CT(46443)=(S=12344696,E=12344703,C=46)
	CT(46444)=(S=12344704,E=12344711,C=18)
	CT(46445)=(S=12344712,E=12344719,C=15)
	CT(46446)=(S=12344720,E=12344727,C=36)
	CT(46447)=(S=12344728,E=12344735,C=49)
	CT(46448)=(S=12344736,E=12344743,C=58)
	CT(46449)=(S=12344744,E=12344751,C=70)
	CT(46450)=(S=12344752,E=12344759,C=14)
	CT(46451)=(S=12344760,E=12344767,C=27)
	CT(46452)=(S=12344768,E=12344775,C=21)
	CT(46453)=(S=12344776,E=12344783,C=50)
	CT(46454)=(S=12344784,E=12344791,C=18)
	CT(46455)=(S=12344792,E=12344799,C=28)
	CT(46456)=(S=12344800,E=12344815,C=21)
	CT(46457)=(S=12344816,E=12344823,C=18)
	CT(46458)=(S=12344824,E=12344831,C=42)
	CT(46459)=(S=12344832,E=12344839,C=12)
	CT(46460)=(S=12344840,E=12344847,C=25)
	CT(46461)=(S=12344848,E=12344855,C=14)
	CT(46462)=(S=12344856,E=12344863,C=28)
	CT(46463)=(S=12344864,E=12344871,C=18)
	CT(46464)=(S=12344872,E=12344879,C=14)
	CT(46465)=(S=12344880,E=12344887,C=18)
	CT(46466)=(S=12344888,E=12344895,C=25)
	CT(46467)=(S=12344896,E=12344903,C=47)
	CT(46468)=(S=12344904,E=12344911,C=14)
	CT(46469)=(S=12344912,E=12344919,C=53)
	CT(46470)=(S=12344920,E=12344927,C=18)
	CT(46471)=(S=12344928,E=12344935,C=28)
	CT(46472)=(S=12344936,E=12344943,C=14)
	CT(46473)=(S=12344944,E=12344951,C=28)
	CT(46474)=(S=12344952,E=12344959,C=15)
	CT(46475)=(S=12344960,E=12344967,C=14)
	CT(46476)=(S=12344968,E=12344975,C=50)
	CT(46477)=(S=12344976,E=12344983,C=15)
	CT(46478)=(S=12344984,E=12344991,C=20)
	CT(46479)=(S=12344992,E=12345007,C=18)
	CT(46480)=(S=12345008,E=12345015,C=34)
	CT(46481)=(S=12345024,E=12345031,C=15)
	CT(46482)=(S=12345032,E=12345039,C=12)
	CT(46483)=(S=12345040,E=12345047,C=18)
	CT(46484)=(S=12345048,E=12345055,C=27)
	CT(46485)=(S=12345056,E=12345063,C=18)
	CT(46486)=(S=12345064,E=12345071,C=33)
	CT(46487)=(S=12345072,E=12345079,C=14)
	CT(46488)=(S=12345080,E=12345103,C=28)
	CT(46489)=(S=12345104,E=12345111,C=15)
	CT(46490)=(S=12345112,E=12345119,C=58)
	CT(46491)=(S=12345120,E=12345127,C=19)
	CT(46492)=(S=12345128,E=12345135,C=14)
	CT(46493)=(S=12345136,E=12345143,C=36)
	CT(46494)=(S=12345144,E=12345151,C=33)
	CT(46495)=(S=12345152,E=12345159,C=61)
	CT(46496)=(S=12345160,E=12345167,C=15)
	CT(46497)=(S=12345168,E=12345175,C=14)
	CT(46498)=(S=12345184,E=12345191,C=36)
	CT(46499)=(S=12345192,E=12345199,C=18)
	CT(46500)=(S=12345200,E=12345207,C=25)
	CT(46501)=(S=12345208,E=12345215,C=33)
	CT(46502)=(S=12345216,E=12345223,C=14)
	CT(46503)=(S=12345224,E=12345231,C=36)
	CT(46504)=(S=12345232,E=12345239,C=37)
	CT(46505)=(S=12345240,E=12345247,C=31)
	CT(46506)=(S=12345248,E=12345255,C=23)
	CT(46507)=(S=12345256,E=12345263,C=36)
	CT(46508)=(S=12345264,E=12345271,C=46)
	CT(46509)=(S=12345272,E=12345279,C=18)
	CT(46510)=(S=12345280,E=12345295,C=14)
	CT(46511)=(S=12345296,E=12345303,C=55)
	CT(46512)=(S=12345304,E=12345311,C=25)
	CT(46513)=(S=12345312,E=12345319,C=19)
	CT(46514)=(S=12345320,E=12345327,C=28)
	CT(46515)=(S=12345328,E=12345335,C=27)
	CT(46516)=(S=12345336,E=12345343,C=25)
	CT(46517)=(S=12345344,E=12349439,C=28)
	CT(46518)=(S=12349440,E=12349503,C=58)
	CT(46519)=(S=12349504,E=12349567,C=71)
	CT(46520)=(S=12349568,E=12349631,C=60)
	CT(46521)=(S=12349632,E=12349759,C=18)
	CT(46522)=(S=12349760,E=12349823,C=27)
	CT(46523)=(S=12349824,E=12349887,C=18)
	CT(46524)=(S=12349888,E=12349951,C=98)
	CT(46525)=(S=12349952,E=12350015,C=18)
	CT(46526)=(S=12350016,E=12350047,C=58)
	CT(46527)=(S=12350048,E=12350063,C=13)
	CT(46528)=(S=12350064,E=12350071,C=14)
	CT(46529)=(S=12350072,E=12350079,C=35)
	CT(46530)=(S=12350080,E=12350143,C=24)
	CT(46531)=(S=12350144,E=12350207,C=18)
	CT(46532)=(S=12350208,E=12350271,C=70)
	CT(46533)=(S=12350272,E=12350335,C=12)
	CT(46534)=(S=12350336,E=12350399,C=31)
	CT(46535)=(S=12350400,E=12350463,C=66)
	CT(46536)=(S=12350464,E=12350527,C=58)
	CT(46537)=(S=12350528,E=12350591,C=33)
	CT(46538)=(S=12350592,E=12350655,C=18)
	CT(46539)=(S=12350656,E=12350719,C=104)
	CT(46540)=(S=12350720,E=12350783,C=47)
	CT(46541)=(S=12350784,E=12350847,C=23)
	CT(46542)=(S=12350848,E=12350911,C=58)
	CT(46543)=(S=12350912,E=12350975,C=42)
	CT(46544)=(S=12350976,E=12351039,C=49)
	CT(46545)=(S=12351040,E=12351103,C=26)
	CT(46546)=(S=12351104,E=12351167,C=28)
	CT(46547)=(S=12351168,E=12351231,C=42)
	CT(46548)=(S=12351232,E=12351295,C=37)
	CT(46549)=(S=12351360,E=12351423,C=38)
	CT(46550)=(S=12351424,E=12351487,C=25)
	CT(46551)=(S=12351488,E=12351519,C=71)
	CT(46552)=(S=12351520,E=12351551,C=18)
	CT(46553)=(S=12351552,E=12351583,C=24)
	CT(46554)=(S=12351584,E=12351615,C=46)
	CT(46555)=(S=12351616,E=12351647,C=17)
	CT(46556)=(S=12351648,E=12351679,C=19)
	CT(46557)=(S=12351680,E=12351711,C=33)
	CT(46558)=(S=12351712,E=12351743,C=18)
	CT(46559)=(S=12351744,E=12351775,C=58)
	CT(46560)=(S=12351776,E=12351807,C=36)
	CT(46561)=(S=12351808,E=12351839,C=19)
	CT(46562)=(S=12351840,E=12351903,C=26)
	CT(46563)=(S=12351904,E=12351935,C=71)
	CT(46564)=(S=12351936,E=12351967,C=40)
	CT(46565)=(S=12351968,E=12351999,C=36)
	CT(46566)=(S=12352000,E=12352031,C=58)
	CT(46567)=(S=12352032,E=12352063,C=14)
	CT(46568)=(S=12352064,E=12352095,C=36)
	CT(46569)=(S=12352096,E=12352127,C=26)
	CT(46570)=(S=12352128,E=12352159,C=19)
	CT(46571)=(S=12352160,E=12352191,C=18)
	CT(46572)=(S=12352192,E=12352223,C=33)
	CT(46573)=(S=12352224,E=12352255,C=18)
	CT(46574)=(S=12352256,E=12352287,C=47)
	CT(46575)=(S=12352288,E=12352319,C=18)
	CT(46576)=(S=12352320,E=12352351,C=12)
	CT(46577)=(S=12352352,E=12352383,C=71)
	CT(46578)=(S=12352384,E=12352415,C=40)
	CT(46579)=(S=12352416,E=12352447,C=61)
	CT(46580)=(S=12352448,E=12352479,C=58)
	CT(46581)=(S=12352480,E=12352511,C=18)
	CT(46582)=(S=12352512,E=12352543,C=37)
	CT(46583)=(S=12352544,E=12352575,C=18)
	CT(46584)=(S=12352576,E=12352607,C=51)
	CT(46585)=(S=12352608,E=12352639,C=18)
	CT(46586)=(S=12352640,E=12352671,C=27)
	CT(46587)=(S=12352672,E=12352703,C=58)
	CT(46588)=(S=12352704,E=12352735,C=59)
	CT(46589)=(S=12352736,E=12352767,C=18)
	CT(46590)=(S=12352768,E=12352799,C=57)
	CT(46591)=(S=12352800,E=12352831,C=58)
	CT(46592)=(S=12352832,E=12352863,C=14)
	CT(46593)=(S=12352864,E=12352895,C=15)
	CT(46594)=(S=12352896,E=12352927,C=58)
	CT(46595)=(S=12352928,E=12352959,C=37)
	CT(46596)=(S=12352960,E=12352991,C=21)
	CT(46597)=(S=12352992,E=12353023,C=58)
	CT(46598)=(S=12353024,E=12353055,C=51)
	CT(46599)=(S=12353056,E=12353087,C=18)
	CT(46600)=(S=12353088,E=12353119,C=19)
	CT(46601)=(S=12353120,E=12353151,C=49)
	CT(46602)=(S=12353152,E=12353183,C=26)
	CT(46603)=(S=12353184,E=12353215,C=28)
	CT(46604)=(S=12353216,E=12353247,C=27)
	CT(46605)=(S=12353248,E=12353279,C=19)
	CT(46606)=(S=12353280,E=12353311,C=58)
	CT(46607)=(S=12353312,E=12353343,C=18)
	CT(46608)=(S=12353344,E=12353375,C=51)
	CT(46609)=(S=12353376,E=12353407,C=59)
	CT(46610)=(S=12353408,E=12353439,C=50)
	CT(46611)=(S=12353440,E=12353471,C=25)
	CT(46612)=(S=12353472,E=12353503,C=47)
	CT(46613)=(S=12353504,E=12353663,C=18)
	CT(46614)=(S=12353664,E=12353791,C=58)
	CT(46615)=(S=12353792,E=12353919,C=57)
	CT(46616)=(S=12353920,E=12354047,C=40)
	CT(46617)=(S=12354048,E=12354175,C=12)
	CT(46618)=(S=12354176,E=12354303,C=18)
	CT(46619)=(S=12354304,E=12354431,C=53)
	CT(46620)=(S=12354432,E=12354559,C=18)
	CT(46621)=(S=12354560,E=12354687,C=23)
	CT(46622)=(S=12354688,E=12354815,C=37)
	CT(46623)=(S=12354816,E=12354943,C=22)
	CT(46624)=(S=12354944,E=12355199,C=18)
	CT(46625)=(S=12355200,E=12355327,C=33)
	CT(46626)=(S=12355328,E=12355455,C=45)
	CT(46627)=(S=12355456,E=12355583,C=15)
	CT(46628)=(S=12355584,E=12355711,C=28)
	CT(46629)=(S=12355712,E=12355839,C=26)
	CT(46630)=(S=12355840,E=12355967,C=58)
	CT(46631)=(S=12355968,E=12356095,C=104)
	CT(46632)=(S=12356096,E=12356223,C=28)
	CT(46633)=(S=12356224,E=12356351,C=53)
	CT(46634)=(S=12356352,E=12356479,C=23)
	CT(46635)=(S=12356480,E=12356607,C=30)
	CT(46636)=(S=12356608,E=12356735,C=21)
	CT(46637)=(S=12356736,E=12356863,C=45)
	CT(46638)=(S=12356864,E=12356991,C=63)
	CT(46639)=(S=12356992,E=12357119,C=12)
	CT(46640)=(S=12357120,E=12357247,C=36)
	CT(46641)=(S=12357248,E=12357503,C=50)
	CT(46642)=(S=12357504,E=12357631,C=18)
	CT(46643)=(S=12357632,E=12358143,C=28)
	CT(46644)=(S=12358144,E=12358655,C=58)
	CT(46645)=(S=12358656,E=12359679,C=27)
	CT(46646)=(S=12359680,E=12360191,C=15)
	CT(46647)=(S=12360192,E=12360703,C=34)
	CT(46648)=(S=12360704,E=12361215,C=50)
	CT(46649)=(S=12361216,E=12361727,C=26)
	CT(46650)=(S=12361728,E=12361983,C=30)
	CT(46651)=(S=12361984,E=12362239,C=43)
	CT(46652)=(S=12362240,E=12362495,C=18)
	CT(46653)=(S=12362496,E=12362751,C=31)
	CT(46654)=(S=12362752,E=12362879,C=14)
	CT(46655)=(S=12362880,E=12362887,C=15)
	CT(46656)=(S=12362888,E=12362895,C=18)
	CT(46657)=(S=12362896,E=12362903,C=58)
	CT(46658)=(S=12362904,E=12362911,C=66)
	CT(46659)=(S=12362912,E=12362943,C=18)
	CT(46660)=(S=12362944,E=12362951,C=25)
	CT(46661)=(S=12362952,E=12362959,C=22)
	CT(46662)=(S=12362960,E=12362967,C=18)
	CT(46663)=(S=12362968,E=12362975,C=96)
	CT(46664)=(S=12362976,E=12362983,C=14)
	CT(46665)=(S=12362984,E=12362991,C=28)
	CT(46666)=(S=12362992,E=12362999,C=58)
	CT(46667)=(S=12363000,E=12363007,C=36)
	CT(46668)=(S=12363008,E=12363263,C=12)
	CT(46669)=(S=12363264,E=12363519,C=36)
	CT(46670)=(S=12363520,E=12363775,C=71)
	CT(46671)=(S=12363776,E=12364031,C=18)
	CT(46672)=(S=12364032,E=12364287,C=40)
	CT(46673)=(S=12364288,E=12364543,C=18)
	CT(46674)=(S=12364544,E=12364799,C=25)
	CT(46675)=(S=12364800,E=12364911,C=72)
	CT(46676)=(S=12364912,E=12364927,C=28)
	CT(46677)=(S=12364928,E=12364943,C=63)
	CT(46678)=(S=12364944,E=12364991,C=14)
	CT(46679)=(S=12364992,E=12365055,C=42)
	CT(46680)=(S=12365056,E=12365311,C=38)
	CT(46681)=(S=12365312,E=12365567,C=28)
	CT(46682)=(S=12365568,E=12365823,C=33)
	CT(46683)=(S=12365824,E=12367871,C=24)
	CT(46684)=(S=12367872,E=12368383,C=34)
	CT(46685)=(S=12368384,E=12368895,C=18)
	CT(46686)=(S=12368896,E=12369407,C=49)
	CT(46687)=(S=12369408,E=12369503,C=31)
	CT(46688)=(S=12369536,E=12369567,C=18)
	CT(46689)=(S=12369568,E=12369663,C=31)
	CT(46690)=(S=12369664,E=12369679,C=18)
	CT(46691)=(S=12369680,E=12369711,C=31)
	CT(46692)=(S=12369712,E=12369727,C=33)
	CT(46693)=(S=12369728,E=12369743,C=31)
	CT(46694)=(S=12369744,E=12369759,C=18)
	CT(46695)=(S=12369760,E=12369775,C=31)
	CT(46696)=(S=12369776,E=12369791,C=18)
	CT(46697)=(S=12369792,E=12369793,C=58)
	CT(46698)=(S=12369794,E=12369795,C=33)
	CT(46699)=(S=12369796,E=12369807,C=14)
	CT(46700)=(S=12369808,E=12369815,C=31)
	CT(46701)=(S=12369816,E=12369823,C=14)
	CT(46702)=(S=12369824,E=12369839,C=18)
	CT(46703)=(S=12369840,E=12369847,C=26)
	CT(46704)=(S=12369848,E=12369863,C=18)
	CT(46705)=(S=12369864,E=12369871,C=58)
	CT(46706)=(S=12369872,E=12369879,C=53)
	CT(46707)=(S=12369880,E=12369887,C=58)
	CT(46708)=(S=12369888,E=12369895,C=18)
	CT(46709)=(S=12369896,E=12369903,C=31)
	CT(46710)=(S=12369904,E=12369911,C=18)
	CT(46711)=(S=12369912,E=12369919,C=38)
	CT(46712)=(S=12369920,E=12370943,C=28)
	CT(46713)=(S=12370944,E=12371967,C=52)
	CT(46714)=(S=12371968,E=12374015,C=36)
	CT(46715)=(S=12374016,E=12374031,C=95)
	CT(46716)=(S=12374032,E=12374053,C=38)
	CT(46717)=(S=12374054,E=12374055,C=36)
	CT(46718)=(S=12374056,E=12374063,C=95)
	CT(46719)=(S=12374064,E=12374071,C=14)
	CT(46720)=(S=12374072,E=12374079,C=53)
	CT(46721)=(S=12374080,E=12374111,C=26)
	CT(46722)=(S=12374112,E=12374117,C=38)
	CT(46723)=(S=12374118,E=12374118,C=44)
	CT(46724)=(S=12374119,E=12374124,C=38)
	CT(46725)=(S=12374125,E=12374125,C=14)
	CT(46726)=(S=12374126,E=12374159,C=38)
	CT(46727)=(S=12374160,E=12374207,C=26)
	CT(46728)=(S=12374208,E=12374215,C=38)
	CT(46729)=(S=12374216,E=12374219,C=53)
	CT(46730)=(S=12374220,E=12374223,C=38)
	CT(46731)=(S=12374224,E=12374231,C=53)
	CT(46732)=(S=12374232,E=12374235,C=95)
	CT(46733)=(S=12374236,E=12374239,C=14)
	CT(46734)=(S=12374240,E=12374271,C=26)
	CT(46735)=(S=12374272,E=12374279,C=53)
	CT(46736)=(S=12374280,E=12374319,C=26)
	CT(46737)=(S=12374320,E=12374335,C=36)
	CT(46738)=(S=12374336,E=12374351,C=26)
	CT(46739)=(S=12374352,E=12374367,C=15)
	CT(46740)=(S=12374368,E=12374383,C=38)
	CT(46741)=(S=12374384,E=12374387,C=25)
	CT(46742)=(S=12374388,E=12374391,C=26)
	CT(46743)=(S=12374392,E=12374399,C=14)
	CT(46744)=(S=12374400,E=12374479,C=26)
	CT(46745)=(S=12374480,E=12374527,C=38)
	CT(46746)=(S=12374528,E=12374591,C=28)
	CT(46747)=(S=12374592,E=12374615,C=26)
	CT(46748)=(S=12374616,E=12374619,C=38)
	CT(46749)=(S=12374620,E=12374620,C=49)
	CT(46750)=(S=12374621,E=12374621,C=29)
	CT(46751)=(S=12374622,E=12374623,C=38)
	CT(46752)=(S=12374624,E=12374735,C=26)
	CT(46753)=(S=12374736,E=12374743,C=95)
	CT(46754)=(S=12374744,E=12374747,C=25)
	CT(46755)=(S=12374748,E=12374751,C=58)
	CT(46756)=(S=12374752,E=12374759,C=38)
	CT(46757)=(S=12374760,E=12374763,C=53)
	CT(46758)=(S=12374764,E=12374779,C=38)
	CT(46759)=(S=12374780,E=12374781,C=14)
	CT(46760)=(S=12374782,E=12374783,C=38)
	CT(46761)=(S=12374784,E=12374799,C=26)
	CT(46762)=(S=12374800,E=12374809,C=38)
	CT(46763)=(S=12374810,E=12374810,C=44)
	CT(46764)=(S=12374811,E=12374815,C=38)
	CT(46765)=(S=12374816,E=12374943,C=26)
	CT(46766)=(S=12374944,E=12374947,C=95)
	CT(46767)=(S=12374948,E=12374950,C=38)
	CT(46768)=(S=12374951,E=12374951,C=14)
	CT(46769)=(S=12374952,E=12374959,C=38)
	CT(46770)=(S=12374960,E=12375007,C=26)
	CT(46771)=(S=12375008,E=12375023,C=38)
	CT(46772)=(S=12375024,E=12375031,C=19)
	CT(46773)=(S=12375032,E=12375045,C=38)
	CT(46774)=(S=12375046,E=12375047,C=26)
	CT(46775)=(S=12375048,E=12375055,C=38)
	CT(46776)=(S=12375056,E=12375059,C=25)
	CT(46777)=(S=12375060,E=12375063,C=95)
	CT(46778)=(S=12375064,E=12375071,C=38)
	CT(46779)=(S=12375072,E=12375075,C=95)
	CT(46780)=(S=12375076,E=12375087,C=38)
	CT(46781)=(S=12375088,E=12375139,C=26)
	CT(46782)=(S=12375140,E=12375147,C=38)
	CT(46783)=(S=12375148,E=12375149,C=19)
	CT(46784)=(S=12375150,E=12375155,C=38)
	CT(46785)=(S=12375156,E=12375159,C=49)
	CT(46786)=(S=12375160,E=12375183,C=38)
	CT(46787)=(S=12375184,E=12375191,C=26)
	CT(46788)=(S=12375192,E=12375199,C=38)
	CT(46789)=(S=12375200,E=12375231,C=26)
	CT(46790)=(S=12375232,E=12375239,C=38)
	CT(46791)=(S=12375240,E=12375287,C=26)
	CT(46792)=(S=12375288,E=12375289,C=38)
	CT(46793)=(S=12375290,E=12375290,C=14)
	CT(46794)=(S=12375291,E=12375293,C=38)
	CT(46795)=(S=12375294,E=12375294,C=14)
	CT(46796)=(S=12375295,E=12375303,C=38)
	CT(46797)=(S=12375304,E=12375311,C=25)
	CT(46798)=(S=12375312,E=12375319,C=38)
	CT(46799)=(S=12375320,E=12375327,C=12)
	CT(46800)=(S=12375328,E=12375335,C=38)
	CT(46801)=(S=12375336,E=12375343,C=15)
	CT(46802)=(S=12375344,E=12375351,C=38)
	CT(46803)=(S=12375352,E=12375359,C=12)
	CT(46804)=(S=12375360,E=12375375,C=26)
	CT(46805)=(S=12375376,E=12375391,C=38)
	CT(46806)=(S=12375392,E=12375423,C=26)
	CT(46807)=(S=12375424,E=12375430,C=38)
	CT(46808)=(S=12375431,E=12375431,C=14)
	CT(46809)=(S=12375432,E=12375435,C=95)
	CT(46810)=(S=12375436,E=12375439,C=12)
	CT(46811)=(S=12375440,E=12375455,C=26)
	CT(46812)=(S=12375456,E=12375471,C=15)
	CT(46813)=(S=12375472,E=12375495,C=26)
	CT(46814)=(S=12375496,E=12375499,C=38)
	CT(46815)=(S=12375500,E=12375500,C=36)
	CT(46816)=(S=12375501,E=12375501,C=38)
	CT(46817)=(S=12375502,E=12375503,C=14)
	CT(46818)=(S=12375504,E=12375507,C=53)
	CT(46819)=(S=12375508,E=12375509,C=38)
	CT(46820)=(S=12375510,E=12375511,C=14)
	CT(46821)=(S=12375512,E=12375513,C=38)
	CT(46822)=(S=12375514,E=12375514,C=44)
	CT(46823)=(S=12375515,E=12375519,C=38)
	CT(46824)=(S=12375520,E=12375527,C=36)
	CT(46825)=(S=12375528,E=12375529,C=13)
	CT(46826)=(S=12375530,E=12375531,C=33)
	CT(46827)=(S=12375532,E=12375535,C=13)
	CT(46828)=(S=12375536,E=12375536,C=38)
	CT(46829)=(S=12375537,E=12375537,C=13)
	CT(46830)=(S=12375538,E=12375538,C=38)
	CT(46831)=(S=12375539,E=12375539,C=29)
	CT(46832)=(S=12375540,E=12375543,C=13)
	CT(46833)=(S=12375544,E=12375551,C=14)
	CT(46834)=(S=12375552,E=12375555,C=38)
	CT(46835)=(S=12375556,E=12375559,C=26)
	CT(46836)=(S=12375560,E=12375567,C=95)
	CT(46837)=(S=12375568,E=12375575,C=38)
	CT(46838)=(S=12375576,E=12375577,C=12)
	CT(46839)=(S=12375578,E=12375599,C=38)
	CT(46840)=(S=12375600,E=12375607,C=95)
	CT(46841)=(S=12375608,E=12375615,C=38)
	CT(46842)=(S=12375616,E=12375631,C=26)
	CT(46843)=(S=12375632,E=12375635,C=38)
	CT(46844)=(S=12375636,E=12375639,C=26)
	CT(46845)=(S=12375640,E=12375641,C=38)
	CT(46846)=(S=12375642,E=12375643,C=14)
	CT(46847)=(S=12375644,E=12375645,C=38)
	CT(46848)=(S=12375646,E=12375647,C=14)
	CT(46849)=(S=12375648,E=12375651,C=53)
	CT(46850)=(S=12375652,E=12375655,C=12)
	CT(46851)=(S=12375656,E=12375663,C=38)
	CT(46852)=(S=12375664,E=12375671,C=25)
	CT(46853)=(S=12375672,E=12375673,C=26)
	CT(46854)=(S=12375674,E=12375679,C=38)
	CT(46855)=(S=12375680,E=12375687,C=44)
	CT(46856)=(S=12375688,E=12375711,C=38)
	CT(46857)=(S=12375712,E=12375743,C=26)
	CT(46858)=(S=12375744,E=12375783,C=38)
	CT(46859)=(S=12375784,E=12375791,C=26)
	CT(46860)=(S=12375792,E=12375795,C=25)
	CT(46861)=(S=12375796,E=12375825,C=38)
	CT(46862)=(S=12375826,E=12375827,C=33)
	CT(46863)=(S=12375828,E=12375831,C=95)
	CT(46864)=(S=12375832,E=12375835,C=26)
	CT(46865)=(S=12375836,E=12375841,C=38)
	CT(46866)=(S=12375842,E=12375842,C=44)
	CT(46867)=(S=12375843,E=12375895,C=38)
	CT(46868)=(S=12375896,E=12375899,C=26)
	CT(46869)=(S=12375900,E=12375927,C=38)
	CT(46870)=(S=12375928,E=12375935,C=43)
	CT(46871)=(S=12375936,E=12375951,C=26)
	CT(46872)=(S=12375952,E=12375967,C=38)
	CT(46873)=(S=12375968,E=12376031,C=26)
	CT(46874)=(S=12376032,E=12376039,C=95)
	CT(46875)=(S=12376040,E=12376047,C=38)
	CT(46876)=(S=12376048,E=12376051,C=53)
	CT(46877)=(S=12376052,E=12376063,C=38)
	CT(46878)=(S=12376064,E=12377087,C=15)
	CT(46879)=(S=12377088,E=12378111,C=14)
	CT(46880)=(S=12378112,E=12378239,C=12)
	CT(46881)=(S=12378240,E=12378367,C=38)
	CT(46882)=(S=12378368,E=12378495,C=18)
	CT(46883)=(S=12378496,E=12378623,C=43)
	CT(46884)=(S=12378624,E=12378751,C=18)
	CT(46885)=(S=12378752,E=12378879,C=13)
	CT(46886)=(S=12378880,E=12379007,C=18)
	CT(46887)=(S=12379008,E=12379039,C=33)
	CT(46888)=(S=12379040,E=12379071,C=14)
	CT(46889)=(S=12379080,E=12379087,C=36)
	CT(46890)=(S=12379088,E=12379103,C=41)
	CT(46891)=(S=12379104,E=12379111,C=50)
	CT(46892)=(S=12379112,E=12379119,C=43)
	CT(46893)=(S=12379120,E=12379127,C=14)
	CT(46894)=(S=12379128,E=12379135,C=55)
	CT(46895)=(S=12379136,E=12379263,C=24)
	CT(46896)=(S=12379264,E=12379391,C=15)
	CT(46897)=(S=12379392,E=12379519,C=26)
	CT(46898)=(S=12379520,E=12379647,C=30)
	CT(46899)=(S=12379648,E=12379775,C=31)
	CT(46900)=(S=12379776,E=12379903,C=52)
	CT(46901)=(S=12379904,E=12380031,C=101)
	CT(46902)=(S=12380032,E=12380159,C=31)
	CT(46903)=(S=12380160,E=12381183,C=18)
	CT(46904)=(S=12381184,E=12381439,C=101)
	CT(46905)=(S=12381440,E=12381695,C=53)
	CT(46906)=(S=12381696,E=12381951,C=47)
	CT(46907)=(S=12381952,E=12382207,C=31)
	CT(46908)=(S=12382208,E=12382225,C=38)
	CT(46909)=(S=12382226,E=12382227,C=19)
	CT(46910)=(S=12382228,E=12382231,C=38)
	CT(46911)=(S=12382232,E=12382239,C=36)
	CT(46912)=(S=12382240,E=12382243,C=38)
	CT(46913)=(S=12382244,E=12382247,C=25)
	CT(46914)=(S=12382248,E=12382335,C=38)
	CT(46915)=(S=12382336,E=12382351,C=95)
	CT(46916)=(S=12382352,E=12382367,C=19)
	CT(46917)=(S=12382368,E=12382399,C=14)
	CT(46918)=(S=12382400,E=12382403,C=25)
	CT(46919)=(S=12382404,E=12382404,C=26)
	CT(46920)=(S=12382405,E=12382411,C=38)
	CT(46921)=(S=12382412,E=12382415,C=25)
	CT(46922)=(S=12382416,E=12382418,C=38)
	CT(46923)=(S=12382419,E=12382419,C=44)
	CT(46924)=(S=12382420,E=12382420,C=26)
	CT(46925)=(S=12382421,E=12382429,C=38)
	CT(46926)=(S=12382430,E=12382431,C=19)
	CT(46927)=(S=12382432,E=12382443,C=38)
	CT(46928)=(S=12382444,E=12382447,C=25)
	CT(46929)=(S=12382448,E=12382455,C=38)
	CT(46930)=(S=12382456,E=12382463,C=26)
	CT(46931)=(S=12382464,E=12382475,C=38)
	CT(46932)=(S=12382476,E=12382476,C=14)
	CT(46933)=(S=12382477,E=12382503,C=38)
	CT(46934)=(S=12382504,E=12382511,C=14)
	CT(46935)=(S=12382512,E=12382531,C=38)
	CT(46936)=(S=12382532,E=12382532,C=36)
	CT(46937)=(S=12382533,E=12382559,C=38)
	CT(46938)=(S=12382560,E=12382567,C=25)
	CT(46939)=(S=12382568,E=12382623,C=38)
	CT(46940)=(S=12382624,E=12382639,C=19)
	CT(46941)=(S=12382640,E=12382656,C=38)
	CT(46942)=(S=12382657,E=12382657,C=36)
	CT(46943)=(S=12382658,E=12382661,C=38)
	CT(46944)=(S=12382662,E=12382663,C=19)
	CT(46945)=(S=12382664,E=12382674,C=38)
	CT(46946)=(S=12382675,E=12382675,C=14)
	CT(46947)=(S=12382676,E=12382719,C=38)
	CT(46948)=(S=12382720,E=12383247,C=18)
	CT(46949)=(S=12383248,E=12383263,C=53)
	CT(46950)=(S=12383264,E=12383279,C=18)
	CT(46951)=(S=12383280,E=12383295,C=33)
	CT(46952)=(S=12383296,E=12383311,C=24)
	CT(46953)=(S=12383312,E=12383327,C=25)
	CT(46954)=(S=12383344,E=12383359,C=14)
	CT(46955)=(S=12383360,E=12383487,C=18)
	CT(46956)=(S=12383488,E=12383743,C=26)
	CT(46957)=(S=12383744,E=12383775,C=28)
	CT(46958)=(S=12383776,E=12383807,C=46)
	CT(46959)=(S=12383808,E=12383839,C=59)
	CT(46960)=(S=12383840,E=12383871,C=33)
	CT(46961)=(S=12383872,E=12383903,C=58)
	CT(46962)=(S=12383904,E=12383935,C=18)
	CT(46963)=(S=12383936,E=12383967,C=14)
	CT(46964)=(S=12383968,E=12383999,C=18)
	CT(46965)=(S=12384000,E=12384031,C=30)
	CT(46966)=(S=12384032,E=12384063,C=18)
	CT(46967)=(S=12384064,E=12384095,C=61)
	CT(46968)=(S=12384096,E=12384127,C=31)
	CT(46969)=(S=12384128,E=12384135,C=38)
	CT(46970)=(S=12384136,E=12384159,C=25)
	CT(46971)=(S=12384160,E=12384167,C=38)
	CT(46972)=(S=12384168,E=12384175,C=25)
	CT(46973)=(S=12384176,E=12384223,C=20)
	CT(46974)=(S=12384224,E=12384255,C=38)
	CT(46975)=(S=12384256,E=12384767,C=23)
	CT(46976)=(S=12384768,E=12385279,C=21)
	CT(46977)=(S=12385280,E=12385407,C=58)
	CT(46978)=(S=12385408,E=12385535,C=57)
	CT(46979)=(S=12385536,E=12385663,C=26)
	CT(46980)=(S=12385664,E=12385791,C=41)
	CT(46981)=(S=12385792,E=12385919,C=18)
	CT(46982)=(S=12385920,E=12386047,C=51)
	CT(46983)=(S=12386048,E=12386175,C=18)
	CT(46984)=(S=12386176,E=12386303,C=46)
	CT(46985)=(S=12386304,E=12419071,C=189)
	CT(46986)=(S=12419072,E=12437955,C=204)
	CT(46987)=(S=12437956,E=12437983,C=189)
	CT(46988)=(S=12437984,E=12437991,C=204)
	CT(46989)=(S=12437992,E=12437999,C=189)
	CT(46990)=(S=12438000,E=12451839,C=204)
	CT(46991)=(S=12451840,E=12451903,C=181)
	CT(46992)=(S=12451904,E=12451935,C=177)
	CT(46993)=(S=12451936,E=12451967,C=83)
	CT(46994)=(S=12451968,E=12451999,C=205)
	CT(46995)=(S=12452000,E=12452031,C=83)
	CT(46996)=(S=12452032,E=12452063,C=215)
	CT(46997)=(S=12452064,E=12452071,C=202)
	CT(46998)=(S=12452072,E=12452079,C=83)
	CT(46999)=(S=12452080,E=12452087,C=181)
	CT(47000)=(S=12452088,E=12452095,C=201)
	CT(47001)=(S=12452096,E=12452159,C=83)
	CT(47002)=(S=12452160,E=12452191,C=181)
	CT(47003)=(S=12452192,E=12452223,C=83)
	CT(47004)=(S=12452224,E=12452351,C=181)
	CT(47005)=(S=12452352,E=12452415,C=83)
	CT(47006)=(S=12452424,E=12452431,C=189)
	CT(47007)=(S=12452432,E=12452439,C=83)
	CT(47008)=(S=12452440,E=12452447,C=203)
	CT(47009)=(S=12452448,E=12452479,C=83)
	CT(47010)=(S=12452480,E=12452543,C=207)
	CT(47011)=(S=12452544,E=12452559,C=206)
	CT(47012)=(S=12452560,E=12452567,C=209)
	CT(47013)=(S=12452568,E=12452575,C=202)
	CT(47014)=(S=12452576,E=12452591,C=209)
	CT(47015)=(S=12452592,E=12452767,C=83)
	CT(47016)=(S=12452776,E=12452791,C=203)
	CT(47017)=(S=12452792,E=12452799,C=201)
	CT(47018)=(S=12452800,E=12452863,C=181)
	CT(47019)=(S=12452864,E=12452927,C=215)
	CT(47020)=(S=12452928,E=12452943,C=217)
	CT(47021)=(S=12452944,E=12452951,C=203)
	CT(47022)=(S=12452952,E=12452959,C=202)
	CT(47023)=(S=12452960,E=12452975,C=203)
	CT(47024)=(S=12452976,E=12452991,C=83)
	CT(47025)=(S=12452992,E=12453055,C=207)
	CT(47026)=(S=12453056,E=12453119,C=203)
	CT(47027)=(S=12453120,E=12453151,C=83)
	CT(47028)=(S=12453152,E=12453183,C=203)
	CT(47029)=(S=12453184,E=12453247,C=208)
	CT(47030)=(S=12453248,E=12453279,C=210)
	CT(47031)=(S=12453280,E=12453311,C=83)
	CT(47032)=(S=12453312,E=12453327,C=181)
	CT(47033)=(S=12453328,E=12453335,C=202)
	CT(47034)=(S=12453336,E=12453343,C=83)
	CT(47035)=(S=12453344,E=12453375,C=209)
	CT(47036)=(S=12453376,E=12453439,C=200)
	CT(47037)=(S=12453440,E=12453471,C=219)
	CT(47038)=(S=12453472,E=12453495,C=83)
	CT(47039)=(S=12453496,E=12453503,C=203)
	CT(47040)=(S=12453504,E=12453535,C=177)
	CT(47041)=(S=12453536,E=12453567,C=181)
	CT(47042)=(S=12453568,E=12453583,C=208)
	CT(47043)=(S=12453584,E=12453599,C=83)
	CT(47044)=(S=12453600,E=12453615,C=176)
	CT(47045)=(S=12453616,E=12453695,C=83)
	CT(47046)=(S=12453696,E=12453791,C=181)
	CT(47047)=(S=12453792,E=12453807,C=202)
	CT(47048)=(S=12453808,E=12453823,C=189)
	CT(47049)=(S=12453824,E=12453855,C=202)
	CT(47050)=(S=12453856,E=12453919,C=83)
	CT(47051)=(S=12453920,E=12453935,C=177)
	CT(47052)=(S=12453936,E=12453951,C=83)
	CT(47053)=(S=12453952,E=12454015,C=203)
	CT(47054)=(S=12454016,E=12454047,C=212)
	CT(47055)=(S=12454052,E=12454059,C=200)
	CT(47056)=(S=12454060,E=12454063,C=212)
	CT(47057)=(S=12454064,E=12454067,C=181)
	CT(47058)=(S=12454068,E=12454071,C=213)
	CT(47059)=(S=12454072,E=12454079,C=83)
	CT(47060)=(S=12454080,E=12454143,C=181)
	CT(47061)=(S=12454144,E=12454175,C=83)
	CT(47062)=(S=12454176,E=12454199,C=209)
	CT(47063)=(S=12454200,E=12454207,C=203)
	CT(47064)=(S=12454208,E=12454271,C=181)
	CT(47065)=(S=12454272,E=12454303,C=200)
	CT(47066)=(S=12454304,E=12454335,C=213)
	CT(47067)=(S=12454336,E=12454399,C=181)
	CT(47068)=(S=12454400,E=12454527,C=202)
	CT(47069)=(S=12454528,E=12454687,C=213)
	CT(47070)=(S=12454688,E=12454719,C=83)
	CT(47071)=(S=12454720,E=12454751,C=201)
	CT(47072)=(S=12454752,E=12454863,C=83)
	CT(47073)=(S=12454880,E=12454895,C=208)
	CT(47074)=(S=12454896,E=12454975,C=213)
	CT(47075)=(S=12454976,E=12455007,C=212)
	CT(47076)=(S=12455008,E=12455039,C=83)
	CT(47077)=(S=12455040,E=12455071,C=181)
	CT(47078)=(S=12455072,E=12455167,C=83)
	CT(47079)=(S=12455168,E=12455231,C=181)
	CT(47080)=(S=12455232,E=12455247,C=203)
	CT(47081)=(S=12455248,E=12455255,C=209)
	CT(47082)=(S=12455256,E=12455263,C=83)
	CT(47083)=(S=12455264,E=12455279,C=181)
	CT(47084)=(S=12455280,E=12455283,C=83)
	CT(47085)=(S=12455284,E=12455287,C=203)
	CT(47086)=(S=12455288,E=12455295,C=207)
	CT(47087)=(S=12455296,E=12455359,C=203)
	CT(47088)=(S=12455360,E=12455375,C=181)
	CT(47089)=(S=12455376,E=12455459,C=83)
	CT(47090)=(S=12455460,E=12455463,C=209)
	CT(47091)=(S=12455464,E=12455471,C=83)
	CT(47092)=(S=12455472,E=12455487,C=203)
	CT(47093)=(S=12455488,E=12455551,C=201)
	CT(47094)=(S=12455552,E=12455567,C=215)
	CT(47095)=(S=12455568,E=12455575,C=83)
	CT(47096)=(S=12455576,E=12455579,C=202)
	CT(47097)=(S=12455580,E=12455615,C=83)
	CT(47098)=(S=12455616,E=12455647,C=209)
	CT(47099)=(S=12455648,E=12455711,C=181)
	CT(47100)=(S=12455712,E=12455743,C=189)
	CT(47101)=(S=12455744,E=12455759,C=208)
	CT(47102)=(S=12455760,E=12455775,C=83)
	CT(47103)=(S=12455776,E=12455807,C=189)
	CT(47104)=(S=12455808,E=12455823,C=213)
	CT(47105)=(S=12455824,E=12455839,C=219)
	CT(47106)=(S=12455840,E=12455871,C=200)
	CT(47107)=(S=12455872,E=12456959,C=83)
	CT(47108)=(S=12456960,E=12457727,C=203)
	CT(47109)=(S=12457728,E=12457983,C=206)
	CT(47110)=(S=12457984,E=12459519,C=181)
	CT(47111)=(S=12459520,E=12460031,C=83)
	CT(47112)=(S=12460032,E=12461055,C=209)
	CT(47113)=(S=12461056,E=12462079,C=200)
	CT(47114)=(S=12462080,E=12463103,C=212)
	CT(47115)=(S=12463104,E=12464127,C=203)
	CT(47116)=(S=12464128,E=12465151,C=83)
	CT(47117)=(S=12465152,E=12465183,C=181)
	CT(47118)=(S=12465184,E=12465199,C=83)
	CT(47119)=(S=12465200,E=12465207,C=201)
	CT(47120)=(S=12465208,E=12465215,C=83)
	CT(47121)=(S=12465216,E=12465231,C=213)
	CT(47122)=(S=12465232,E=12465247,C=83)
	CT(47123)=(S=12465248,E=12465263,C=200)
	CT(47124)=(S=12465264,E=12465279,C=212)
	CT(47125)=(S=12465280,E=12465343,C=206)
	CT(47126)=(S=12465344,E=12465359,C=213)
	CT(47127)=(S=12465360,E=12465375,C=83)
	CT(47128)=(S=12465376,E=12465407,C=177)
	CT(47129)=(S=12465408,E=12465455,C=210)
	CT(47130)=(S=12465456,E=12465503,C=208)
	CT(47131)=(S=12465504,E=12465599,C=210)
	CT(47132)=(S=12465600,E=12465663,C=208)
	CT(47133)=(S=12465664,E=12465919,C=203)
	CT(47134)=(S=12465920,E=12466175,C=83)
	CT(47135)=(S=12466176,E=12466431,C=215)
	CT(47136)=(S=12466432,E=12466559,C=210)
	CT(47137)=(S=12466560,E=12466623,C=213)
	CT(47138)=(S=12466624,E=12466687,C=83)
	CT(47139)=(S=12466688,E=12467199,C=176)
	CT(47140)=(S=12467200,E=12467711,C=181)
	CT(47141)=(S=12467712,E=12467967,C=210)
	CT(47142)=(S=12467968,E=12468223,C=213)
	CT(47143)=(S=12468224,E=12468479,C=205)
	CT(47144)=(S=12468480,E=12470271,C=181)
	CT(47145)=(S=12470272,E=12472319,C=200)
	CT(47146)=(S=12472320,E=12472447,C=214)
	CT(47147)=(S=12472448,E=12472575,C=177)
	CT(47148)=(S=12472576,E=12472831,C=212)
	CT(47149)=(S=12472832,E=12473087,C=203)
	CT(47150)=(S=12473216,E=12473343,C=176)
	CT(47151)=(S=12473344,E=12473855,C=181)
	CT(47152)=(S=12473856,E=12474367,C=210)
	CT(47153)=(S=12474368,E=12474623,C=207)
	CT(47154)=(S=12474880,E=12475135,C=181)
	CT(47155)=(S=12475136,E=12475391,C=203)
	CT(47156)=(S=12475392,E=12475487,C=208)
	CT(47157)=(S=12475488,E=12475503,C=83)
	CT(47158)=(S=12475504,E=12475519,C=219)
	CT(47159)=(S=12475520,E=12475647,C=83)
	CT(47160)=(S=12475648,E=12475679,C=176)
	CT(47161)=(S=12475684,E=12475687,C=214)
	CT(47162)=(S=12475688,E=12475691,C=83)
	CT(47163)=(S=12475692,E=12475695,C=200)
	CT(47164)=(S=12475696,E=12475711,C=83)
	CT(47165)=(S=12475712,E=12475775,C=176)
	CT(47166)=(S=12475776,E=12475807,C=181)
	CT(47167)=(S=12475808,E=12475823,C=202)
	CT(47168)=(S=12475824,E=12475827,C=206)
	CT(47169)=(S=12475840,E=12475871,C=83)
	CT(47170)=(S=12475872,E=12475887,C=203)
	CT(47171)=(S=12475888,E=12475903,C=202)
	CT(47172)=(S=12475904,E=12476031,C=177)
	CT(47173)=(S=12476032,E=12476063,C=213)
	CT(47174)=(S=12476064,E=12476095,C=83)
	CT(47175)=(S=12476096,E=12476159,C=200)
	CT(47176)=(S=12476160,E=12476287,C=203)
	CT(47177)=(S=12476288,E=12476415,C=213)
	CT(47178)=(S=12476416,E=12476511,C=203)
	CT(47179)=(S=12476512,E=12476527,C=213)
	CT(47180)=(S=12476528,E=12476543,C=83)
	CT(47181)=(S=12476544,E=12476671,C=181)
	CT(47182)=(S=12476672,E=12476735,C=83)
	CT(47183)=(S=12476736,E=12476767,C=181)
	CT(47184)=(S=12476768,E=12476783,C=176)
	CT(47185)=(S=12476784,E=12476791,C=217)
	CT(47186)=(S=12476792,E=12476799,C=83)
	CT(47187)=(S=12476800,E=12476831,C=181)
	CT(47188)=(S=12476832,E=12476839,C=209)
	CT(47189)=(S=12476840,E=12476843,C=203)
	CT(47190)=(S=12476848,E=12476863,C=208)
	CT(47191)=(S=12476864,E=12476895,C=181)
	CT(47192)=(S=12476896,E=12476927,C=200)
	CT(47193)=(S=12476928,E=12477055,C=224)
	CT(47194)=(S=12477056,E=12477119,C=83)
	CT(47195)=(S=12477120,E=12477183,C=203)
	CT(47196)=(S=12477184,E=12477207,C=208)
	CT(47197)=(S=12477208,E=12477247,C=210)
	CT(47198)=(S=12477248,E=12477255,C=83)
	CT(47199)=(S=12477256,E=12477263,C=213)
	CT(47200)=(S=12477264,E=12477271,C=83)
	CT(47201)=(S=12477272,E=12477275,C=203)
	CT(47202)=(S=12477276,E=12477279,C=201)
	CT(47203)=(S=12477280,E=12477295,C=205)
	CT(47204)=(S=12477296,E=12477299,C=83)
	CT(47205)=(S=12477300,E=12477303,C=215)
	CT(47206)=(S=12477304,E=12477311,C=202)
	CT(47207)=(S=12477312,E=12477439,C=181)
	CT(47208)=(S=12477440,E=12477951,C=203)
	CT(47209)=(S=12477952,E=12477983,C=211)
	CT(47210)=(S=12477984,E=12477999,C=202)
	CT(47211)=(S=12478000,E=12478007,C=201)
	CT(47212)=(S=12478008,E=12478015,C=209)
	CT(47213)=(S=12478016,E=12478047,C=218)
	CT(47214)=(S=12478048,E=12478079,C=170)
	CT(47215)=(S=12478080,E=12478111,C=212)
	CT(47216)=(S=12478112,E=12478175,C=181)
	CT(47217)=(S=12478176,E=12478207,C=203)
	CT(47218)=(S=12478208,E=12478235,C=83)
	CT(47219)=(S=12478236,E=12478271,C=200)
	CT(47220)=(S=12478272,E=12478287,C=201)
	CT(47221)=(S=12478288,E=12478303,C=83)
	CT(47222)=(S=12478304,E=12478335,C=181)
	CT(47223)=(S=12478336,E=12478351,C=83)
	CT(47224)=(S=12478352,E=12478367,C=203)
	CT(47225)=(S=12478368,E=12478383,C=189)
	CT(47226)=(S=12478384,E=12478387,C=83)
	CT(47227)=(S=12478388,E=12478391,C=177)
	CT(47228)=(S=12478392,E=12478395,C=29)
	CT(47229)=(S=12478396,E=12478399,C=202)
	CT(47230)=(S=12478400,E=12478463,C=83)
	CT(47231)=(S=12478464,E=12478495,C=201)
	CT(47232)=(S=12478496,E=12478559,C=83)
	CT(47233)=(S=12478560,E=12478575,C=225)
	CT(47234)=(S=12478576,E=12478591,C=215)
	CT(47235)=(S=12478592,E=12478655,C=206)
	CT(47236)=(S=12478656,E=12478847,C=83)
	CT(47237)=(S=12478848,E=12478863,C=208)
	CT(47238)=(S=12478864,E=12478871,C=83)
	CT(47239)=(S=12478872,E=12478875,C=209)
	CT(47240)=(S=12478876,E=12478879,C=214)
	CT(47241)=(S=12478880,E=12478887,C=83)
	CT(47242)=(S=12478888,E=12478891,C=209)
	CT(47243)=(S=12478892,E=12478895,C=218)
	CT(47244)=(S=12478896,E=12478911,C=83)
	CT(47245)=(S=12478912,E=12478915,C=207)
	CT(47246)=(S=12478916,E=12478959,C=83)
	CT(47247)=(S=12478960,E=12478975,C=212)
	CT(47248)=(S=12478976,E=12479007,C=216)
	CT(47249)=(S=12479008,E=12479023,C=83)
	CT(47250)=(S=12479024,E=12479039,C=216)
	CT(47251)=(S=12479040,E=12479055,C=202)
	CT(47252)=(S=12479056,E=12479063,C=83)
	CT(47253)=(S=12479064,E=12479071,C=170)
	CT(47254)=(S=12479072,E=12479087,C=83)
	CT(47255)=(S=12479088,E=12479095,C=177)
	CT(47256)=(S=12479096,E=12479167,C=83)
	CT(47257)=(S=12479168,E=12479199,C=215)
	CT(47258)=(S=12479216,E=12479231,C=201)
	CT(47259)=(S=12479232,E=12479247,C=219)
	CT(47260)=(S=12479248,E=12479263,C=181)
	CT(47261)=(S=12479264,E=12479295,C=201)
	CT(47262)=(S=12479296,E=12479319,C=213)
	CT(47263)=(S=12479320,E=12479327,C=202)
	CT(47264)=(S=12479328,E=12479359,C=83)
	CT(47265)=(S=12479360,E=12479391,C=208)
	CT(47266)=(S=12479408,E=12479411,C=203)
	CT(47267)=(S=12479412,E=12479415,C=212)
	CT(47268)=(S=12479416,E=12479423,C=83)
	CT(47269)=(S=12479424,E=12479439,C=189)
	CT(47270)=(S=12479440,E=12479443,C=216)
	CT(47271)=(S=12479444,E=12479455,C=83)
	CT(47272)=(S=12479456,E=12479463,C=203)
	CT(47273)=(S=12479464,E=12479471,C=213)
	CT(47274)=(S=12479472,E=12479479,C=83)
	CT(47275)=(S=12479480,E=12479487,C=217)
	CT(47276)=(S=12479488,E=12479519,C=205)
	CT(47277)=(S=12479520,E=12479551,C=83)
	CT(47278)=(S=12479552,E=12479559,C=213)
	CT(47279)=(S=12479560,E=12479563,C=202)
	CT(47280)=(S=12479564,E=12479567,C=83)
	CT(47281)=(S=12479568,E=12479583,C=212)
	CT(47282)=(S=12479584,E=12479615,C=189)
	CT(47283)=(S=12479616,E=12479679,C=203)
	CT(47284)=(S=12479680,E=12479683,C=83)
	CT(47285)=(S=12479684,E=12479703,C=214)
	CT(47286)=(S=12479704,E=12479707,C=83)
	CT(47287)=(S=12479708,E=12479711,C=202)
	CT(47288)=(S=12479712,E=12479743,C=83)
	CT(47289)=(S=12479744,E=12479775,C=209)
	CT(47290)=(S=12479776,E=12479807,C=83)
	CT(47291)=(S=12479808,E=12479835,C=189)
	CT(47292)=(S=12479836,E=12479839,C=204)
	CT(47293)=(S=12479840,E=12479935,C=181)
	CT(47294)=(S=12479936,E=12479967,C=208)
	CT(47295)=(S=12479968,E=12479999,C=201)
	CT(47296)=(S=12480000,E=12480063,C=177)
	CT(47297)=(S=12480064,E=12480095,C=181)
	CT(47298)=(S=12480096,E=12480111,C=203)
	CT(47299)=(S=12480112,E=12480119,C=83)
	CT(47300)=(S=12480120,E=12480175,C=203)
	CT(47301)=(S=12480176,E=12480191,C=83)
	CT(47302)=(S=12480192,E=12480223,C=213)
	CT(47303)=(S=12480224,E=12480255,C=83)
	CT(47304)=(S=12480256,E=12480287,C=215)
	CT(47305)=(S=12480288,E=12480319,C=83)
	CT(47306)=(S=12480320,E=12480351,C=213)
	CT(47307)=(S=12480368,E=12480375,C=83)
	CT(47308)=(S=12480376,E=12480379,C=200)
	CT(47309)=(S=12480380,E=12480559,C=83)
	CT(47310)=(S=12480560,E=12480563,C=203)
	CT(47311)=(S=12480564,E=12480567,C=205)
	CT(47312)=(S=12480568,E=12480703,C=83)
	CT(47313)=(S=12480704,E=12480707,C=205)
	CT(47314)=(S=12480708,E=12480711,C=83)
	CT(47315)=(S=12480712,E=12480715,C=205)
	CT(47316)=(S=12480716,E=12480719,C=189)
	CT(47317)=(S=12480720,E=12480727,C=206)
	CT(47318)=(S=12480728,E=12480731,C=83)
	CT(47319)=(S=12480732,E=12480735,C=202)
	CT(47320)=(S=12480736,E=12480767,C=207)
	CT(47321)=(S=12480768,E=12480799,C=203)
	CT(47322)=(S=12480800,E=12480831,C=202)
	CT(47323)=(S=12480832,E=12480847,C=177)
	CT(47324)=(S=12480848,E=12480855,C=202)
	CT(47325)=(S=12480856,E=12480859,C=215)
	CT(47326)=(S=12480860,E=12480863,C=206)
	CT(47327)=(S=12480864,E=12480895,C=202)
	CT(47328)=(S=12480896,E=12480959,C=83)
	CT(47329)=(S=12480960,E=12480991,C=212)
	CT(47330)=(S=12480992,E=12481007,C=83)
	CT(47331)=(S=12481008,E=12481011,C=177)
	CT(47332)=(S=12481012,E=12481015,C=203)
	CT(47333)=(S=12481016,E=12481023,C=202)
	CT(47334)=(S=12481024,E=12481055,C=209)
	CT(47335)=(S=12481056,E=12481087,C=203)
	CT(47336)=(S=12481088,E=12481247,C=83)
	CT(47337)=(S=12481248,E=12481255,C=206)
	CT(47338)=(S=12481256,E=12481263,C=83)
	CT(47339)=(S=12481264,E=12481271,C=200)
	CT(47340)=(S=12481272,E=12481275,C=212)
	CT(47341)=(S=12481276,E=12481279,C=203)
	CT(47342)=(S=12481280,E=12481295,C=215)
	CT(47343)=(S=12481296,E=12481311,C=170)
	CT(47344)=(S=12481312,E=12481343,C=83)
	CT(47345)=(S=12481376,E=12481391,C=202)
	CT(47346)=(S=12481392,E=12481407,C=83)
	CT(47347)=(S=12481408,E=12481471,C=218)
	CT(47348)=(S=12481472,E=12481503,C=83)
	CT(47349)=(S=12481504,E=12481535,C=181)
	CT(47350)=(S=12481536,E=12482559,C=212)
	CT(47351)=(S=12482560,E=12482591,C=210)
	CT(47352)=(S=12482608,E=12482623,C=83)
	CT(47353)=(S=12482624,E=12482655,C=213)
	CT(47354)=(S=12482656,E=12482687,C=83)
	CT(47355)=(S=12482688,E=12482703,C=181)
	CT(47356)=(S=12482704,E=12482719,C=83)
	CT(47357)=(S=12482720,E=12482735,C=203)
	CT(47358)=(S=12482736,E=12482751,C=83)
	CT(47359)=(S=12482752,E=12482783,C=218)
	CT(47360)=(S=12482784,E=12482799,C=209)
	CT(47361)=(S=12482800,E=12482803,C=202)
	CT(47362)=(S=12482804,E=12482807,C=83)
	CT(47363)=(S=12482808,E=12482815,C=200)
	CT(47364)=(S=12482816,E=12482943,C=203)
	CT(47365)=(S=12482944,E=12482975,C=181)
	CT(47366)=(S=12482976,E=12482991,C=206)
	CT(47367)=(S=12482992,E=12483007,C=83)
	CT(47368)=(S=12483008,E=12483023,C=215)
	CT(47369)=(S=12483024,E=12483039,C=207)
	CT(47370)=(S=12483040,E=12483055,C=200)
	CT(47371)=(S=12483056,E=12483071,C=207)
	CT(47372)=(S=12483072,E=12483167,C=83)
	CT(47373)=(S=12483168,E=12483199,C=177)
	CT(47374)=(S=12483200,E=12483231,C=83)
	CT(47375)=(S=12483232,E=12483247,C=209)
	CT(47376)=(S=12483248,E=12483251,C=210)
	CT(47377)=(S=12483252,E=12483255,C=83)
	CT(47378)=(S=12483256,E=12483259,C=215)
	CT(47379)=(S=12483260,E=12483263,C=209)
	CT(47380)=(S=12483264,E=12483287,C=83)
	CT(47381)=(S=12483288,E=12483295,C=200)
	CT(47382)=(S=12483296,E=12483327,C=83)
	CT(47383)=(S=12483328,E=12483343,C=213)
	CT(47384)=(S=12483344,E=12483347,C=217)
	CT(47385)=(S=12483348,E=12483351,C=207)
	CT(47386)=(S=12483352,E=12483359,C=208)
	CT(47387)=(S=12483360,E=12483375,C=209)
	CT(47388)=(S=12483376,E=12483391,C=213)
	CT(47389)=(S=12483392,E=12483407,C=203)
	CT(47390)=(S=12483408,E=12483451,C=83)
	CT(47391)=(S=12483452,E=12483455,C=201)
	CT(47392)=(S=12483456,E=12483471,C=202)
	CT(47393)=(S=12483472,E=12483487,C=83)
	CT(47394)=(S=12483488,E=12483519,C=200)
	CT(47395)=(S=12483520,E=12483535,C=209)
	CT(47396)=(S=12483536,E=12483551,C=208)
	CT(47397)=(S=12483552,E=12483567,C=209)
	CT(47398)=(S=12483568,E=12483599,C=83)
	CT(47399)=(S=12483600,E=12483607,C=208)
	CT(47400)=(S=12483608,E=12483611,C=203)
	CT(47401)=(S=12483612,E=12483615,C=200)
	CT(47402)=(S=12483616,E=12483623,C=216)
	CT(47403)=(S=12483624,E=12483631,C=170)
	CT(47404)=(S=12483632,E=12483639,C=212)
	CT(47405)=(S=12483640,E=12483647,C=83)
	CT(47406)=(S=12483648,E=12483679,C=177)
	CT(47407)=(S=12483680,E=12483711,C=181)
	CT(47408)=(S=12483712,E=12483743,C=83)
	CT(47409)=(S=12483744,E=12483751,C=208)
	CT(47410)=(S=12483752,E=12483759,C=83)
	CT(47411)=(S=12483760,E=12483775,C=189)
	CT(47412)=(S=12483776,E=12483799,C=83)
	CT(47413)=(S=12483800,E=12483803,C=211)
	CT(47414)=(S=12483804,E=12483807,C=214)
	CT(47415)=(S=12483808,E=12483827,C=83)
	CT(47416)=(S=12483828,E=12483835,C=202)
	CT(47417)=(S=12483840,E=12484735,C=181)
	CT(47418)=(S=12484736,E=12484863,C=206)
	CT(47419)=(S=12484864,E=12485119,C=201)
	CT(47420)=(S=12485120,E=12485183,C=208)
	CT(47421)=(S=12485184,E=12485247,C=181)
	CT(47422)=(S=12485248,E=12485567,C=213)
	CT(47423)=(S=12485568,E=12485631,C=181)
	CT(47424)=(S=12485632,E=12486655,C=205)
	CT(47425)=(S=12486656,E=12487679,C=83)
	CT(47426)=(S=12487680,E=12488191,C=209)
	CT(47427)=(S=12488192,E=12488447,C=200)
	CT(47428)=(S=12488448,E=12488575,C=181)
	CT(47429)=(S=12488576,E=12488703,C=215)
	CT(47430)=(S=12488704,E=12489727,C=181)
	CT(47431)=(S=12489728,E=12490239,C=215)
	CT(47432)=(S=12490240,E=12490495,C=210)
	CT(47433)=(S=12490496,E=12490623,C=203)
	CT(47434)=(S=12490624,E=12490627,C=215)
	CT(47435)=(S=12490628,E=12490631,C=202)
	CT(47436)=(S=12490632,E=12490671,C=83)
	CT(47437)=(S=12490672,E=12490687,C=210)
	CT(47438)=(S=12490688,E=12490751,C=181)
	CT(47439)=(S=12490752,E=12491007,C=213)
	CT(47440)=(S=12491008,E=12491135,C=200)
	CT(47441)=(S=12491136,E=12491263,C=203)
	CT(47442)=(S=12491264,E=12491775,C=213)
	CT(47443)=(S=12491776,E=12492799,C=181)
	CT(47444)=(S=12492800,E=12494079,C=203)
	CT(47445)=(S=12494080,E=12494335,C=181)
	CT(47446)=(S=12494336,E=12494847,C=177)
	CT(47447)=(S=12494848,E=12495615,C=200)
	CT(47448)=(S=12495616,E=12495679,C=202)
	CT(47449)=(S=12495680,E=12495711,C=181)
	CT(47450)=(S=12495712,E=12495743,C=202)
	CT(47451)=(S=12495744,E=12495807,C=203)
	CT(47452)=(S=12495808,E=12495871,C=201)
	CT(47453)=(S=12495872,E=12497919,C=83)
	CT(47454)=(S=12497920,E=12498047,C=201)
	CT(47455)=(S=12498048,E=12498063,C=189)
	CT(47456)=(S=12498064,E=12498067,C=83)
	CT(47457)=(S=12498068,E=12498071,C=189)
	CT(47458)=(S=12498072,E=12498079,C=83)
	CT(47459)=(S=12498080,E=12498111,C=189)
	CT(47460)=(S=12498112,E=12498175,C=203)
	CT(47461)=(S=12498176,E=12498239,C=201)
	CT(47462)=(S=12498240,E=12498303,C=83)
	CT(47463)=(S=12498304,E=12498367,C=216)
	CT(47464)=(S=12498368,E=12498431,C=208)
	CT(47465)=(S=12498432,E=12498559,C=181)
	CT(47466)=(S=12498560,E=12498943,C=83)
	CT(47467)=(S=12498944,E=12499071,C=216)
	CT(47468)=(S=12499072,E=12499135,C=181)
	CT(47469)=(S=12499136,E=12499139,C=83)
	CT(47470)=(S=12499140,E=12499143,C=202)
	CT(47471)=(S=12499144,E=12499151,C=181)
	CT(47472)=(S=12499152,E=12499159,C=83)
	CT(47473)=(S=12499160,E=12499163,C=213)
	CT(47474)=(S=12499164,E=12499199,C=83)
	CT(47475)=(S=12499200,E=12499295,C=207)
	CT(47476)=(S=12499296,E=12499299,C=202)
	CT(47477)=(S=12499300,E=12499307,C=83)
	CT(47478)=(S=12499308,E=12499327,C=208)
	CT(47479)=(S=12499328,E=12499455,C=83)
	CT(47480)=(S=12499456,E=12499711,C=201)
	CT(47481)=(S=12499712,E=12499967,C=212)
	CT(47482)=(S=12499968,E=12502015,C=83)
	CT(47483)=(S=12502016,E=12502207,C=203)
	CT(47484)=(S=12502208,E=12502223,C=218)
	CT(47485)=(S=12502224,E=12502239,C=203)
	CT(47486)=(S=12502240,E=12502255,C=83)
	CT(47487)=(S=12502256,E=12502263,C=213)
	CT(47488)=(S=12502264,E=12502271,C=83)
	CT(47489)=(S=12502272,E=12502399,C=170)
	CT(47490)=(S=12502400,E=12502527,C=83)
	CT(47491)=(S=12502528,E=12505087,C=200)
	CT(47492)=(S=12505088,E=12505599,C=203)
	CT(47493)=(S=12505600,E=12505855,C=83)
	CT(47494)=(S=12505856,E=12505919,C=203)
	CT(47495)=(S=12505920,E=12505947,C=83)
	CT(47496)=(S=12505948,E=12505983,C=202)
	CT(47497)=(S=12505984,E=12505991,C=83)
	CT(47498)=(S=12505992,E=12505995,C=208)
	CT(47499)=(S=12505996,E=12505999,C=181)
	CT(47500)=(S=12506000,E=12506015,C=83)
	CT(47501)=(S=12506016,E=12506031,C=203)
	CT(47502)=(S=12506032,E=12506047,C=177)
	CT(47503)=(S=12506048,E=12506079,C=83)
	CT(47504)=(S=12506080,E=12506095,C=201)
	CT(47505)=(S=12506096,E=12506099,C=206)
	CT(47506)=(S=12506100,E=12506103,C=83)
	CT(47507)=(S=12506104,E=12506107,C=203)
	CT(47508)=(S=12506108,E=12506111,C=209)
	CT(47509)=(S=12506112,E=12506367,C=216)
	CT(47510)=(S=12506368,E=12506623,C=176)
	CT(47511)=(S=12506624,E=12506879,C=213)
	CT(47512)=(S=12506880,E=12507135,C=203)
	CT(47513)=(S=12507136,E=12507519,C=83)
	CT(47514)=(S=12507520,E=12507647,C=203)
	CT(47515)=(S=12507648,E=12508159,C=209)
	CT(47516)=(S=12508160,E=12508671,C=83)
	CT(47517)=(S=12508672,E=12509183,C=212)
	CT(47518)=(S=12509184,E=12511231,C=83)
	CT(47519)=(S=12511232,E=12513279,C=212)
	CT(47520)=(S=12513280,E=12513535,C=181)
	CT(47521)=(S=12513536,E=12513791,C=202)
	CT(47522)=(S=12513792,E=12514303,C=181)
	CT(47523)=(S=12514304,E=12515327,C=83)
	CT(47524)=(S=12515328,E=12517375,C=181)
	CT(47525)=(S=12517376,E=12533759,C=189)
	CT(47526)=(S=12533760,E=12537855,C=181)
	CT(47527)=(S=12537856,E=12539391,C=83)
	CT(47528)=(S=12539392,E=12539903,C=189)
	CT(47529)=(S=12539904,E=12541951,C=181)
	CT(47530)=(S=12541952,E=12542207,C=203)
	CT(47531)=(S=12542208,E=12542223,C=209)
	CT(47532)=(S=12542224,E=12542227,C=200)
	CT(47533)=(S=12542228,E=12542231,C=209)
	CT(47534)=(S=12542232,E=12542255,C=83)
	CT(47535)=(S=12542256,E=12542271,C=212)
	CT(47536)=(S=12542272,E=12542275,C=213)
	CT(47537)=(S=12542276,E=12542287,C=83)
	CT(47538)=(S=12542288,E=12542295,C=170)
	CT(47539)=(S=12542296,E=12542303,C=177)
	CT(47540)=(S=12542304,E=12542307,C=83)
	CT(47541)=(S=12542308,E=12542311,C=212)
	CT(47542)=(S=12542312,E=12542315,C=177)
	CT(47543)=(S=12542316,E=12542327,C=83)
	CT(47544)=(S=12542328,E=12542335,C=206)
	CT(47545)=(S=12542336,E=12542463,C=83)
	CT(47546)=(S=12542464,E=12542591,C=181)
	CT(47547)=(S=12542592,E=12542655,C=212)
	CT(47548)=(S=12542656,E=12542659,C=215)
	CT(47549)=(S=12542660,E=12542663,C=209)
	CT(47550)=(S=12542664,E=12542671,C=203)
	CT(47551)=(S=12542672,E=12542679,C=177)
	CT(47552)=(S=12542680,E=12542687,C=83)
	CT(47553)=(S=12542688,E=12542719,C=216)
	CT(47554)=(S=12542720,E=12543231,C=213)
	CT(47555)=(S=12543232,E=12543487,C=203)
	CT(47556)=(S=12543488,E=12543503,C=181)
	CT(47557)=(S=12543504,E=12543519,C=83)
	CT(47558)=(S=12543520,E=12543523,C=203)
	CT(47559)=(S=12543524,E=12543527,C=202)
	CT(47560)=(S=12543528,E=12543535,C=83)
	CT(47561)=(S=12543536,E=12543543,C=216)
	CT(47562)=(S=12543544,E=12543547,C=202)
	CT(47563)=(S=12543548,E=12543615,C=181)
	CT(47564)=(S=12543616,E=12543679,C=208)
	CT(47565)=(S=12543680,E=12543727,C=181)
	CT(47566)=(S=12543728,E=12543731,C=177)
	CT(47567)=(S=12543732,E=12543807,C=83)
	CT(47568)=(S=12543808,E=12543839,C=208)
	CT(47569)=(S=12543840,E=12543855,C=83)
	CT(47570)=(S=12543856,E=12543871,C=216)
	CT(47571)=(S=12543872,E=12546047,C=181)
	CT(47572)=(S=12546048,E=12548095,C=203)
	CT(47573)=(S=12548096,E=12549119,C=189)
	CT(47574)=(S=12549120,E=12550143,C=203)
	CT(47575)=(S=12550144,E=12554239,C=189)
	CT(47576)=(S=12554240,E=12558335,C=181)
	CT(47577)=(S=12558336,E=12582911,C=189)
	CT(47578)=(S=12582915,E=12582915,C=29)
	CT(47579)=(S=12582916,E=12582919,C=174)
	CT(47580)=(S=12582920,E=12583039,C=29)
	CT(47581)=(S=12583040,E=12583167,C=78)
	CT(47582)=(S=12583168,E=12584219,C=29)
	CT(47583)=(S=12584220,E=12584222,C=14)
	CT(47584)=(S=12584223,E=12584227,C=29)
	CT(47585)=(S=12584228,E=12584228,C=19)
	CT(47586)=(S=12584229,E=12584241,C=29)
	CT(47587)=(S=12584242,E=12584242,C=19)
	CT(47588)=(S=12584243,E=12584250,C=29)
	CT(47589)=(S=12584251,E=12584251,C=12)
	CT(47590)=(S=12584252,E=12584252,C=29)
	CT(47591)=(S=12584253,E=12584254,C=12)
	CT(47592)=(S=12584255,E=12584281,C=29)
	CT(47593)=(S=12584282,E=12584282,C=6)
	CT(47594)=(S=12584283,E=12584287,C=29)
	CT(47595)=(S=12584288,E=12584288,C=78)
	CT(47596)=(S=12584289,E=12584289,C=15)
	CT(47597)=(S=12584290,E=12584333,C=29)
	CT(47598)=(S=12584334,E=12584334,C=12)
	CT(47599)=(S=12584335,E=12584335,C=29)
	CT(47600)=(S=12584336,E=12584336,C=78)
	CT(47601)=(S=12584337,E=12584337,C=28)
	CT(47602)=(S=12584338,E=12584353,C=29)
	CT(47603)=(S=12584354,E=12584354,C=42)
	CT(47604)=(S=12584355,E=12584395,C=29)
	CT(47605)=(S=12584396,E=12584397,C=78)
	CT(47606)=(S=12584398,E=12584407,C=29)
	CT(47607)=(S=12584408,E=12584408,C=2)
	CT(47608)=(S=12584409,E=12584430,C=29)
	CT(47609)=(S=12584431,E=12584431,C=14)
	CT(47610)=(S=12584432,E=12584432,C=0)
	CT(47611)=(S=12584433,E=12584445,C=29)
	CT(47612)=(S=12584446,E=12584446,C=14)
	CT(47613)=(S=12584447,E=12584991,C=29)
	CT(47614)=(S=12584992,E=12585003,C=4)
	CT(47615)=(S=12585004,E=12585119,C=29)
	CT(47616)=(S=12585120,E=12585136,C=4)
	CT(47617)=(S=12585137,E=12585138,C=29)
	CT(47618)=(S=12585139,E=12585140,C=4)
	CT(47619)=(S=12585141,E=12585141,C=29)
	CT(47620)=(S=12585142,E=12585142,C=4)
	CT(47621)=(S=12585143,E=12585143,C=29)
	CT(47622)=(S=12585144,E=12585215,C=4)
	CT(47623)=(S=12585216,E=12585983,C=29)
	CT(47624)=(S=12585984,E=12585984,C=78)
	CT(47625)=(S=12585985,E=12585985,C=14)
	CT(47626)=(S=12585986,E=12586030,C=29)
	CT(47627)=(S=12586031,E=12586031,C=15)
	CT(47628)=(S=12586032,E=12586037,C=29)
	CT(47629)=(S=12586038,E=12586038,C=36)
	CT(47630)=(S=12586039,E=12586045,C=29)
	CT(47631)=(S=12586046,E=12586046,C=78)
	CT(47632)=(S=12586047,E=12586055,C=29)
	CT(47633)=(S=12586056,E=12586057,C=14)
	CT(47634)=(S=12586058,E=12586059,C=29)
	CT(47635)=(S=12586060,E=12586060,C=0)
	CT(47636)=(S=12586061,E=12586061,C=15)
	CT(47637)=(S=12586062,E=12586064,C=29)
	CT(47638)=(S=12586065,E=12586065,C=28)
	CT(47639)=(S=12586066,E=12586079,C=29)
	CT(47640)=(S=12586080,E=12586080,C=12)
	CT(47641)=(S=12586081,E=12586081,C=29)
	CT(47642)=(S=12586082,E=12586082,C=78)
	CT(47643)=(S=12586083,E=12586083,C=14)
	CT(47644)=(S=12586084,E=12586112,C=29)
	CT(47645)=(S=12586113,E=12586113,C=78)
	CT(47646)=(S=12586114,E=12586157,C=29)
	CT(47647)=(S=12586158,E=12586167,C=78)
	CT(47648)=(S=12586168,E=12586175,C=29)
	CT(47649)=(S=12586176,E=12586178,C=15)
	CT(47650)=(S=12586179,E=12586198,C=29)
	CT(47651)=(S=12586199,E=12586199,C=78)
	CT(47652)=(S=12586200,E=12586200,C=29)
	CT(47653)=(S=12586201,E=12586202,C=34)
	CT(47654)=(S=12586203,E=12586214,C=29)
	CT(47655)=(S=12586215,E=12586215,C=14)
	CT(47656)=(S=12586216,E=12586216,C=21)
	CT(47657)=(S=12586217,E=12586218,C=29)
	CT(47658)=(S=12586219,E=12586219,C=19)
	CT(47659)=(S=12586220,E=12586230,C=29)
	CT(47660)=(S=12586231,E=12586231,C=34)
	CT(47661)=(S=12586232,E=12587130,C=29)
	CT(47662)=(S=12587131,E=12587144,C=19)
	CT(47663)=(S=12587145,E=12587145,C=28)
	CT(47664)=(S=12587146,E=12587162,C=19)
	CT(47665)=(S=12587163,E=12587174,C=34)
	CT(47666)=(S=12587175,E=12587185,C=29)
	CT(47667)=(S=12587186,E=12587187,C=2)
	CT(47668)=(S=12587188,E=12587188,C=0)
	CT(47669)=(S=12587189,E=12587190,C=29)
	CT(47670)=(S=12587191,E=12587210,C=36)
	CT(47671)=(S=12587211,E=12587212,C=29)
	CT(47672)=(S=12587213,E=12587263,C=78)
	CT(47673)=(S=12587264,E=12587714,C=29)
	CT(47674)=(S=12587715,E=12587715,C=14)
	CT(47675)=(S=12587716,E=12589595,C=29)
	CT(47676)=(S=12589596,E=12589612,C=34)
	CT(47677)=(S=12589613,E=12589613,C=14)
	CT(47678)=(S=12589614,E=12589615,C=34)
	CT(47679)=(S=12589616,E=12589658,C=29)
	CT(47680)=(S=12589659,E=12589659,C=2)
	CT(47681)=(S=12589660,E=12589660,C=29)
	CT(47682)=(S=12589661,E=12589662,C=2)
	CT(47683)=(S=12589663,E=12589663,C=29)
	CT(47684)=(S=12589664,E=12589664,C=2)
	CT(47685)=(S=12589665,E=12589672,C=29)
	CT(47686)=(S=12589673,E=12589696,C=47)
	CT(47687)=(S=12589697,E=12589700,C=29)
	CT(47688)=(S=12589701,E=12589703,C=47)
	CT(47689)=(S=12589704,E=12589721,C=29)
	CT(47690)=(S=12589722,E=12589741,C=36)
	CT(47691)=(S=12589742,E=12589761,C=28)
	CT(47692)=(S=12589762,E=12589777,C=29)
	CT(47693)=(S=12589778,E=12589780,C=78)
	CT(47694)=(S=12589781,E=12589782,C=29)
	CT(47695)=(S=12589784,E=12589798,C=0)
	CT(47696)=(S=12589799,E=12589799,C=21)
	CT(47697)=(S=12589800,E=12589800,C=0)
	CT(47698)=(S=12589801,E=12589801,C=29)
	CT(47699)=(S=12589802,E=12589802,C=14)
	CT(47700)=(S=12589803,E=12589803,C=78)
	CT(47701)=(S=12589804,E=12589804,C=21)
	CT(47702)=(S=12589805,E=12589805,C=36)
	CT(47703)=(S=12589806,E=12589806,C=42)
	CT(47704)=(S=12589807,E=12589807,C=21)
	CT(47705)=(S=12589808,E=12590199,C=29)
	CT(47706)=(S=12590200,E=12590203,C=0)
	CT(47707)=(S=12590204,E=12590631,C=29)
	CT(47708)=(S=12590632,E=12590635,C=80)
	CT(47709)=(S=12590636,E=12590679,C=29)
	CT(47710)=(S=12590680,E=12590681,C=78)
	CT(47711)=(S=12590682,E=12590688,C=29)
	CT(47712)=(S=12590689,E=12590691,C=78)
	CT(47713)=(S=12590692,E=12590715,C=29)
	CT(47714)=(S=12590716,E=12590716,C=197)
	CT(47715)=(S=12590717,E=12590733,C=29)
	CT(47716)=(S=12590734,E=12590734,C=78)
	CT(47717)=(S=12590735,E=12590740,C=29)
	CT(47718)=(S=12590741,E=12590741,C=78)
	CT(47719)=(S=12590742,E=12590793,C=29)
	CT(47720)=(S=12590794,E=12590794,C=78)
	CT(47721)=(S=12590795,E=12590817,C=29)
	CT(47722)=(S=12590818,E=12590818,C=78)
	CT(47723)=(S=12590819,E=12590823,C=29)
	CT(47724)=(S=12590824,E=12590831,C=78)
	CT(47725)=(S=12590832,E=12590853,C=29)
	CT(47726)=(S=12590854,E=12590854,C=78)
	CT(47727)=(S=12590855,E=12590861,C=29)
	CT(47728)=(S=12590862,E=12590862,C=28)
	CT(47729)=(S=12590863,E=12590870,C=29)
	CT(47730)=(S=12590871,E=12590871,C=49)
	CT(47731)=(S=12590872,E=12590873,C=29)
	CT(47732)=(S=12590874,E=12590874,C=14)
	CT(47733)=(S=12590875,E=12590875,C=12)
	CT(47734)=(S=12590876,E=12590878,C=29)
	CT(47735)=(S=12590879,E=12590879,C=14)
	CT(47736)=(S=12590880,E=12590887,C=29)
	CT(47737)=(S=12590888,E=12590888,C=49)
	CT(47738)=(S=12590889,E=12590899,C=29)
	CT(47739)=(S=12590900,E=12590903,C=78)
	CT(47740)=(S=12590904,E=12590909,C=29)
	CT(47741)=(S=12590910,E=12590910,C=14)
	CT(47742)=(S=12590911,E=12590949,C=29)
	CT(47743)=(S=12590950,E=12590950,C=28)
	CT(47744)=(S=12590951,E=12590963,C=29)
	CT(47745)=(S=12590964,E=12590972,C=2)
	CT(47746)=(S=12590973,E=12591013,C=29)
	CT(47747)=(S=12591014,E=12591019,C=36)
	CT(47748)=(S=12591020,E=12591024,C=29)
	CT(47749)=(S=12591025,E=12591025,C=78)
	CT(47750)=(S=12591026,E=12591027,C=29)
	CT(47751)=(S=12591028,E=12591028,C=78)
	CT(47752)=(S=12591029,E=12591043,C=29)
	CT(47753)=(S=12591045,E=12591054,C=2)
	CT(47754)=(S=12591055,E=12591058,C=29)
	CT(47755)=(S=12591059,E=12591059,C=12)
	CT(47756)=(S=12591060,E=12591075,C=29)
	CT(47757)=(S=12591076,E=12591076,C=78)
	CT(47758)=(S=12591077,E=12591078,C=29)
	CT(47759)=(S=12591079,E=12591079,C=36)
	CT(47760)=(S=12591080,E=12591098,C=29)
	CT(47761)=(S=12591099,E=12591099,C=78)
	CT(47762)=(S=12591100,E=12591100,C=12)
	CT(47763)=(S=12591101,E=12591369,C=29)
	CT(47764)=(S=12591370,E=12591370,C=105)
	CT(47765)=(S=12591371,E=12591374,C=29)
	CT(47766)=(S=12591375,E=12591375,C=12)
	CT(47767)=(S=12591376,E=12591376,C=14)
	CT(47768)=(S=12591377,E=12591395,C=29)
	CT(47769)=(S=12591396,E=12591396,C=36)
	CT(47770)=(S=12591397,E=12591446,C=29)
	CT(47771)=(S=12591447,E=12591471,C=34)
	CT(47772)=(S=12591472,E=12591472,C=29)
	CT(47773)=(S=12591473,E=12591474,C=12)
	CT(47774)=(S=12591475,E=12591477,C=29)
	CT(47775)=(S=12591478,E=12591487,C=34)
	CT(47776)=(S=12591488,E=12591489,C=29)
	CT(47777)=(S=12591490,E=12591490,C=14)
	CT(47778)=(S=12591491,E=12591491,C=36)
	CT(47779)=(S=12591492,E=12591494,C=29)
	CT(47780)=(S=12591495,E=12591495,C=2)
	CT(47781)=(S=12591496,E=12591502,C=29)
	CT(47782)=(S=12591503,E=12591505,C=12)
	CT(47783)=(S=12591506,E=12591506,C=29)
	CT(47784)=(S=12591507,E=12591526,C=12)
	CT(47785)=(S=12591527,E=12591528,C=29)
	CT(47786)=(S=12591529,E=12591531,C=12)
	CT(47787)=(S=12591532,E=12591535,C=29)
	CT(47788)=(S=12591536,E=12591542,C=12)
	CT(47789)=(S=12591543,E=12591551,C=29)
	CT(47790)=(S=12591552,E=12591591,C=34)
	CT(47791)=(S=12591592,E=12591592,C=29)
	CT(47792)=(S=12591593,E=12591599,C=36)
	CT(47793)=(S=12591600,E=12591613,C=29)
	CT(47794)=(S=12591614,E=12591614,C=28)
	CT(47795)=(S=12591615,E=12591634,C=29)
	CT(47796)=(S=12591635,E=12591635,C=14)
	CT(47797)=(S=12591636,E=12591641,C=29)
	CT(47798)=(S=12591642,E=12591643,C=78)
	CT(47799)=(S=12591644,E=12591651,C=29)
	CT(47800)=(S=12591652,E=12591655,C=78)
	CT(47801)=(S=12591656,E=12591665,C=29)
	CT(47802)=(S=12591666,E=12591666,C=14)
	CT(47803)=(S=12591667,E=12591667,C=29)
	CT(47804)=(S=12591668,E=12591671,C=78)
	CT(47805)=(S=12591672,E=12591685,C=29)
	CT(47806)=(S=12591686,E=12591686,C=78)
	CT(47807)=(S=12591687,E=12591691,C=29)
	CT(47808)=(S=12591692,E=12591695,C=78)
	CT(47809)=(S=12591696,E=12591707,C=29)
	CT(47810)=(S=12591708,E=12591711,C=78)
	CT(47811)=(S=12591712,E=12591722,C=29)
	CT(47812)=(S=12591723,E=12591723,C=19)
	CT(47813)=(S=12591724,E=12591731,C=29)
	CT(47814)=(S=12591732,E=12591732,C=36)
	CT(47815)=(S=12591734,E=12591794,C=29)
	CT(47816)=(S=12591795,E=12591795,C=47)
	CT(47817)=(S=12591796,E=12591796,C=29)
	CT(47818)=(S=12591797,E=12591811,C=78)
	CT(47819)=(S=12591812,E=12591814,C=29)
	CT(47820)=(S=12591815,E=12591815,C=78)
	CT(47821)=(S=12591816,E=12591831,C=29)
	CT(47822)=(S=12591832,E=12591847,C=78)
	CT(47823)=(S=12591848,E=12591871,C=29)
	CT(47824)=(S=12591872,E=12591891,C=28)
	CT(47825)=(S=12591892,E=12591930,C=29)
	CT(47826)=(S=12591931,E=12591931,C=0)
	CT(47827)=(S=12591932,E=12591934,C=29)
	CT(47828)=(S=12591935,E=12591944,C=28)
	CT(47829)=(S=12591945,E=12591961,C=29)
	CT(47830)=(S=12591962,E=12591962,C=28)
	CT(47831)=(S=12591963,E=12591965,C=29)
	CT(47832)=(S=12591966,E=12591966,C=14)
	CT(47833)=(S=12591967,E=12591979,C=29)
	CT(47834)=(S=12591980,E=12591980,C=14)
	CT(47835)=(S=12591981,E=12592001,C=29)
	CT(47836)=(S=12592002,E=12592005,C=36)
	CT(47837)=(S=12592006,E=12592009,C=29)
	CT(47838)=(S=12592010,E=12592010,C=14)
	CT(47839)=(S=12592011,E=12592015,C=29)
	CT(47840)=(S=12592016,E=12592016,C=78)
	CT(47841)=(S=12592017,E=12592017,C=29)
	CT(47842)=(S=12592018,E=12592018,C=15)
	CT(47843)=(S=12592019,E=12592020,C=29)
	CT(47844)=(S=12592021,E=12592025,C=28)
	CT(47845)=(S=12592026,E=12592043,C=29)
	CT(47846)=(S=12592044,E=12592044,C=14)
	CT(47847)=(S=12592045,E=12592054,C=29)
	CT(47848)=(S=12592055,E=12592064,C=36)
	CT(47849)=(S=12592065,E=12592068,C=29)
	CT(47850)=(S=12592069,E=12592070,C=14)
	CT(47851)=(S=12592071,E=12592076,C=29)
	CT(47852)=(S=12592077,E=12592078,C=14)
	CT(47853)=(S=12592079,E=12592079,C=63)
	CT(47854)=(S=12592080,E=12592100,C=29)
	CT(47855)=(S=12592101,E=12592101,C=28)
	CT(47856)=(S=12592102,E=12592111,C=29)
	CT(47857)=(S=12592112,E=12592116,C=42)
	CT(47858)=(S=12592117,E=12592117,C=29)
	CT(47859)=(S=12592118,E=12592118,C=21)
	CT(47860)=(S=12592119,E=12592119,C=36)
	CT(47861)=(S=12592120,E=12592120,C=0)
	CT(47862)=(S=12592121,E=12592127,C=29)
	CT(47863)=(S=12592128,E=12592383,C=19)
	CT(47864)=(S=12592384,E=12592639,C=34)
	CT(47865)=(S=12592640,E=12592895,C=24)
	CT(47866)=(S=12592896,E=12593183,C=29)
	CT(47867)=(S=12593184,E=12593191,C=78)
	CT(47868)=(S=12593192,E=12593217,C=29)
	CT(47869)=(S=12593218,E=12593219,C=78)
	CT(47870)=(S=12593220,E=12593220,C=29)
	CT(47871)=(S=12593221,E=12593221,C=16)
	CT(47872)=(S=12593222,E=12593223,C=57)
	CT(47873)=(S=12593224,E=12593231,C=47)
	CT(47874)=(S=12593232,E=12593232,C=14)
	CT(47875)=(S=12593233,E=12593233,C=78)
	CT(47876)=(S=12593234,E=12593235,C=29)
	CT(47877)=(S=12593236,E=12593239,C=178)
	CT(47878)=(S=12593240,E=12593247,C=29)
	CT(47879)=(S=12593248,E=12593255,C=78)
	CT(47880)=(S=12593256,E=12593287,C=29)
	CT(47881)=(S=12593288,E=12593291,C=78)
	CT(47882)=(S=12593292,E=12593293,C=29)
	CT(47883)=(S=12593294,E=12593294,C=78)
	CT(47884)=(S=12593295,E=12593368,C=29)
	CT(47885)=(S=12593369,E=12593369,C=78)
	CT(47886)=(S=12593370,E=12593379,C=29)
	CT(47887)=(S=12593380,E=12593380,C=47)
	CT(47888)=(S=12593381,E=12593383,C=29)
	CT(47889)=(S=12593384,E=12593395,C=78)
	CT(47890)=(S=12593396,E=12593405,C=29)
	CT(47891)=(S=12593406,E=12593406,C=78)
	CT(47892)=(S=12593408,E=12593510,C=29)
	CT(47893)=(S=12593511,E=12593539,C=14)
	CT(47894)=(S=12593540,E=12593544,C=34)
	CT(47895)=(S=12593545,E=12593547,C=29)
	CT(47896)=(S=12593548,E=12593548,C=36)
	CT(47897)=(S=12593549,E=12593553,C=34)
	CT(47898)=(S=12593554,E=12593554,C=0)
	CT(47899)=(S=12593555,E=12593555,C=28)
	CT(47900)=(S=12593556,E=12593556,C=78)
	CT(47901)=(S=12593557,E=12593568,C=34)
	CT(47902)=(S=12593569,E=12593577,C=29)
	CT(47903)=(S=12593578,E=12593578,C=3)
	CT(47904)=(S=12593579,E=12593585,C=29)
	CT(47905)=(S=12593586,E=12593605,C=2)
	CT(47906)=(S=12593606,E=12593607,C=29)
	CT(47907)=(S=12593608,E=12593609,C=78)
	CT(47908)=(S=12593610,E=12593610,C=29)
	CT(47909)=(S=12593611,E=12593611,C=0)
	CT(47910)=(S=12593612,E=12593613,C=29)
	CT(47911)=(S=12593614,E=12593614,C=74)
	CT(47912)=(S=12593615,E=12593617,C=29)
	CT(47913)=(S=12593618,E=12593618,C=14)
	CT(47914)=(S=12593619,E=12593623,C=29)
	CT(47915)=(S=12593624,E=12593624,C=12)
	CT(47916)=(S=12593625,E=12593625,C=29)
	CT(47917)=(S=12593626,E=12593626,C=15)
	CT(47918)=(S=12593627,E=12593634,C=29)
	CT(47919)=(S=12593635,E=12593635,C=28)
	CT(47920)=(S=12593636,E=12593664,C=29)
	CT(47921)=(S=12593665,E=12593665,C=28)
	CT(47922)=(S=12593666,E=12593668,C=29)
	CT(47923)=(S=12593669,E=12593670,C=78)
	CT(47924)=(S=12593671,E=12593705,C=29)
	CT(47925)=(S=12593706,E=12593711,C=34)
	CT(47926)=(S=12593712,E=12593715,C=29)
	CT(47927)=(S=12593717,E=12593717,C=14)
	CT(47928)=(S=12593718,E=12593718,C=78)
	CT(47929)=(S=12593719,E=12593723,C=29)
	CT(47930)=(S=12593724,E=12593726,C=0)
	CT(47931)=(S=12593727,E=12593728,C=28)
	CT(47932)=(S=12593730,E=12593731,C=29)
	CT(47933)=(S=12593732,E=12593732,C=78)
	CT(47934)=(S=12593733,E=12593735,C=29)
	CT(47935)=(S=12593736,E=12593737,C=78)
	CT(47936)=(S=12593738,E=12593747,C=29)
	CT(47937)=(S=12593751,E=12593751,C=14)
	CT(47938)=(S=12593752,E=12593753,C=29)
	CT(47939)=(S=12593754,E=12593754,C=78)
	CT(47940)=(S=12593755,E=12593762,C=29)
	CT(47941)=(S=12593763,E=12593763,C=105)
	CT(47942)=(S=12593764,E=12593764,C=14)
	CT(47943)=(S=12593765,E=12593765,C=29)
	CT(47944)=(S=12593766,E=12593766,C=12)
	CT(47945)=(S=12593767,E=12593771,C=2)
	CT(47946)=(S=12593772,E=12593776,C=29)
	CT(47947)=(S=12593777,E=12593796,C=36)
	CT(47948)=(S=12593797,E=12593806,C=29)
	CT(47949)=(S=12593807,E=12593807,C=28)
	CT(47950)=(S=12593808,E=12593814,C=29)
	CT(47951)=(S=12593815,E=12593815,C=0)
	CT(47952)=(S=12593816,E=12593843,C=29)
	CT(47953)=(S=12593844,E=12593865,C=34)
	CT(47954)=(S=12593866,E=12593868,C=78)
	CT(47955)=(S=12593869,E=12593869,C=29)
	CT(47956)=(S=12593870,E=12593870,C=78)
	CT(47957)=(S=12593871,E=12593900,C=29)
	CT(47958)=(S=12593901,E=12593901,C=2)
	CT(47959)=(S=12593902,E=12594081,C=29)
	CT(47960)=(S=12594082,E=12594091,C=19)
	CT(47961)=(S=12594092,E=12594104,C=29)
	CT(47962)=(S=12594105,E=12594105,C=0)
	CT(47963)=(S=12594106,E=12594108,C=29)
	CT(47964)=(S=12594110,E=12594110,C=78)
	CT(47965)=(S=12594111,E=12594111,C=29)
	CT(47966)=(S=12594112,E=12594116,C=34)
	CT(47967)=(S=12594117,E=12594117,C=29)
	CT(47968)=(S=12594118,E=12594118,C=0)
	CT(47969)=(S=12594119,E=12594126,C=29)
	CT(47970)=(S=12594127,E=12594129,C=0)
	CT(47971)=(S=12594130,E=12594130,C=36)
	CT(47972)=(S=12594131,E=12594131,C=29)
	CT(47973)=(S=12594132,E=12594132,C=36)
	CT(47974)=(S=12594133,E=12594138,C=29)
	CT(47975)=(S=12594139,E=12594139,C=78)
	CT(47976)=(S=12594140,E=12594145,C=29)
	CT(47977)=(S=12594146,E=12594149,C=0)
	CT(47978)=(S=12594150,E=12594153,C=29)
	CT(47979)=(S=12594154,E=12594154,C=14)
	CT(47980)=(S=12594155,E=12594158,C=29)
	CT(47981)=(S=12594159,E=12594159,C=0)
	CT(47982)=(S=12594160,E=12594175,C=29)
	CT(47983)=(S=12594176,E=12594216,C=28)
	CT(47984)=(S=12594217,E=12594243,C=12)
	CT(47985)=(S=12594244,E=12594246,C=29)
	CT(47986)=(S=12594247,E=12594256,C=12)
	CT(47987)=(S=12594257,E=12594266,C=28)
	CT(47988)=(S=12594267,E=12594415,C=29)
	CT(47989)=(S=12594416,E=12594427,C=19)
	CT(47990)=(S=12594428,E=12594943,C=29)
	CT(47991)=(S=12594944,E=12595185,C=2)
	CT(47992)=(S=12595186,E=12595187,C=29)
	CT(47993)=(S=12595188,E=12595193,C=34)
	CT(47994)=(S=12595194,E=12595196,C=29)
	CT(47995)=(S=12595197,E=12595197,C=78)
	CT(47996)=(S=12595198,E=12595198,C=29)
	CT(47997)=(S=12595199,E=12595199,C=78)
	CT(47998)=(S=12595200,E=12595230,C=29)
	CT(47999)=(S=12595231,E=12595231,C=28)
	CT(48000)=(S=12595232,E=12595306,C=29)
	CT(48001)=(S=12595307,E=12595307,C=28)
	CT(48002)=(S=12595308,E=12595308,C=29)
	CT(48003)=(S=12595309,E=12595310,C=0)
	CT(48004)=(S=12595311,E=12595340,C=29)
	CT(48005)=(S=12595341,E=12595341,C=78)
	CT(48006)=(S=12595342,E=12595344,C=29)
	CT(48007)=(S=12595345,E=12595345,C=28)
	CT(48008)=(S=12595346,E=12595419,C=29)
	CT(48009)=(S=12595420,E=12595420,C=78)
	CT(48010)=(S=12595421,E=12595423,C=29)
	CT(48011)=(S=12595424,E=12595424,C=28)
	CT(48012)=(S=12595425,E=12595430,C=29)
	CT(48013)=(S=12595431,E=12595431,C=28)
	CT(48014)=(S=12595432,E=12595433,C=29)
	CT(48015)=(S=12595434,E=12595434,C=78)
	CT(48016)=(S=12595435,E=12595452,C=29)
	CT(48017)=(S=12595453,E=12595453,C=105)
	CT(48018)=(S=12595454,E=12595455,C=29)
	CT(48019)=(S=12595456,E=12595711,C=47)
	CT(48020)=(S=12595712,E=12595967,C=2)
	CT(48021)=(S=12595968,E=12595983,C=27)
	CT(48022)=(S=12595984,E=12596015,C=2)
	CT(48023)=(S=12596016,E=12596016,C=29)
	CT(48024)=(S=12596017,E=12596159,C=2)
	CT(48025)=(S=12596160,E=12596161,C=29)
	CT(48026)=(S=12596162,E=12596223,C=2)
	CT(48027)=(S=12596224,E=12596274,C=28)
	CT(48028)=(S=12596275,E=12596295,C=29)
	CT(48029)=(S=12596296,E=12596297,C=0)
	CT(48030)=(S=12596298,E=12596331,C=29)
	CT(48031)=(S=12596332,E=12596332,C=78)
	CT(48032)=(S=12596333,E=12596375,C=29)
	CT(48033)=(S=12596376,E=12596376,C=36)
	CT(48034)=(S=12596377,E=12596382,C=29)
	CT(48035)=(S=12596383,E=12596384,C=28)
	CT(48036)=(S=12596385,E=12596385,C=14)
	CT(48037)=(S=12596386,E=12596394,C=29)
	CT(48038)=(S=12596395,E=12596400,C=0)
	CT(48039)=(S=12596401,E=12596444,C=29)
	CT(48040)=(S=12596445,E=12596447,C=36)
	CT(48041)=(S=12596448,E=12596455,C=29)
	CT(48042)=(S=12596456,E=12596456,C=135)
	CT(48043)=(S=12596457,E=12596462,C=29)
	CT(48044)=(S=12596463,E=12596463,C=78)
	CT(48045)=(S=12596464,E=12596476,C=29)
	CT(48046)=(S=12596477,E=12596477,C=14)
	CT(48047)=(S=12596478,E=12596478,C=12)
	CT(48048)=(S=12596479,E=12596582,C=29)
	CT(48049)=(S=12596583,E=12596583,C=28)
	CT(48050)=(S=12596584,E=12596584,C=34)
	CT(48051)=(S=12596585,E=12596766,C=29)
	CT(48052)=(S=12596767,E=12596788,C=28)
	CT(48053)=(S=12596789,E=12596789,C=29)
	CT(48054)=(S=12596796,E=12596816,C=28)
	CT(48055)=(S=12596817,E=12596839,C=29)
	CT(48056)=(S=12596840,E=12596840,C=28)
	CT(48057)=(S=12596841,E=12596842,C=0)
	CT(48058)=(S=12596843,E=12596848,C=29)
	CT(48059)=(S=12596849,E=12596849,C=28)
	CT(48060)=(S=12596850,E=12596850,C=29)
	CT(48061)=(S=12596851,E=12596856,C=24)
	CT(48062)=(S=12596857,E=12596857,C=78)
	CT(48063)=(S=12596858,E=12596858,C=34)
	CT(48064)=(S=12596859,E=12596860,C=29)
	CT(48065)=(S=12596861,E=12596864,C=28)
	CT(48066)=(S=12596865,E=12596865,C=29)
	CT(48067)=(S=12596866,E=12596866,C=73)
	CT(48068)=(S=12596867,E=12596867,C=29)
	CT(48069)=(S=12596868,E=12596868,C=34)
	CT(48070)=(S=12596869,E=12596870,C=29)
	CT(48071)=(S=12596871,E=12596871,C=0)
	CT(48072)=(S=12596872,E=12596874,C=29)
	CT(48073)=(S=12596875,E=12596875,C=28)
	CT(48074)=(S=12596876,E=12596876,C=29)
	CT(48075)=(S=12596877,E=12596939,C=12)
	CT(48076)=(S=12596940,E=12596940,C=17)
	CT(48077)=(S=12596941,E=12596955,C=12)
	CT(48078)=(S=12596956,E=12596956,C=58)
	CT(48079)=(S=12596957,E=12596957,C=28)
	CT(48080)=(S=12596958,E=12596960,C=29)
	CT(48081)=(S=12596961,E=12596961,C=19)
	CT(48082)=(S=12596962,E=12596977,C=29)
	CT(48083)=(S=12596978,E=12596978,C=78)
	CT(48084)=(S=12596979,E=12596979,C=29)
	CT(48085)=(S=12596981,E=12596981,C=14)
	CT(48086)=(S=12596982,E=12596986,C=29)
	CT(48087)=(S=12596987,E=12596987,C=0)
	CT(48088)=(S=12596988,E=12596989,C=29)
	CT(48089)=(S=12596990,E=12596990,C=28)
	CT(48090)=(S=12596991,E=12597075,C=29)
	CT(48091)=(S=12597076,E=12597076,C=28)
	CT(48092)=(S=12597077,E=12597080,C=29)
	CT(48093)=(S=12597081,E=12597081,C=28)
	CT(48094)=(S=12597082,E=12597089,C=29)
	CT(48095)=(S=12597090,E=12597091,C=0)
	CT(48096)=(S=12597092,E=12597092,C=29)
	CT(48097)=(S=12597093,E=12597093,C=15)
	CT(48098)=(S=12597094,E=12597096,C=29)
	CT(48099)=(S=12597097,E=12597097,C=36)
	CT(48100)=(S=12597098,E=12597100,C=29)
	CT(48101)=(S=12597101,E=12597101,C=58)
	CT(48102)=(S=12597103,E=12597103,C=29)
	CT(48103)=(S=12597104,E=12597104,C=0)
	CT(48104)=(S=12597105,E=12597106,C=29)
	CT(48105)=(S=12597107,E=12597107,C=36)
	CT(48106)=(S=12597108,E=12597109,C=29)
	CT(48107)=(S=12597110,E=12597110,C=2)
	CT(48108)=(S=12597111,E=12597111,C=78)
	CT(48109)=(S=12597112,E=12597120,C=29)
	CT(48110)=(S=12597121,E=12597121,C=36)
	CT(48111)=(S=12597122,E=12597122,C=78)
	CT(48112)=(S=12597123,E=12597123,C=29)
	CT(48113)=(S=12597124,E=12597124,C=14)
	CT(48114)=(S=12597125,E=12597179,C=29)
	CT(48115)=(S=12597180,E=12597180,C=28)
	CT(48116)=(S=12597181,E=12597181,C=29)
	CT(48117)=(S=12597182,E=12597182,C=0)
	CT(48118)=(S=12597183,E=12597184,C=29)
	CT(48119)=(S=12597185,E=12597185,C=14)
	CT(48120)=(S=12597186,E=12597186,C=78)
	CT(48121)=(S=12597187,E=12597188,C=29)
	CT(48122)=(S=12597189,E=12597189,C=28)
	CT(48123)=(S=12597190,E=12597190,C=0)
	CT(48124)=(S=12597191,E=12597197,C=29)
	CT(48125)=(S=12597198,E=12597198,C=78)
	CT(48126)=(S=12597199,E=12597203,C=29)
	CT(48127)=(S=12597204,E=12597204,C=14)
	CT(48128)=(S=12597205,E=12597207,C=29)
	CT(48129)=(S=12597208,E=12597208,C=78)
	CT(48130)=(S=12597209,E=12597210,C=29)
	CT(48131)=(S=12597211,E=12597211,C=0)
	CT(48132)=(S=12597212,E=12597212,C=2)
	CT(48133)=(S=12597213,E=12597215,C=29)
	CT(48134)=(S=12597216,E=12597216,C=78)
	CT(48135)=(S=12597217,E=12597223,C=29)
	CT(48136)=(S=12597224,E=12597224,C=0)
	CT(48137)=(S=12597225,E=12597235,C=29)
	CT(48138)=(S=12597236,E=12597236,C=28)
	CT(48139)=(S=12597237,E=12597768,C=29)
	CT(48140)=(S=12597769,E=12597778,C=2)
	CT(48141)=(S=12597779,E=12597787,C=29)
	CT(48142)=(S=12597788,E=12597788,C=70)
	CT(48143)=(S=12597789,E=12597789,C=16)
	CT(48144)=(S=12597790,E=12597791,C=47)
	CT(48145)=(S=12597792,E=12597795,C=51)
	CT(48146)=(S=12597796,E=12597800,C=29)
	CT(48147)=(S=12597801,E=12597849,C=47)
	CT(48148)=(S=12597850,E=12597880,C=29)
	CT(48149)=(S=12597881,E=12597881,C=78)
	CT(48150)=(S=12597882,E=12597892,C=29)
	CT(48151)=(S=12597893,E=12597896,C=0)
	CT(48152)=(S=12597897,E=12597899,C=29)
	CT(48153)=(S=12597900,E=12597903,C=82)
	CT(48154)=(S=12597904,E=12597953,C=29)
	CT(48155)=(S=12597954,E=12597954,C=78)
	CT(48156)=(S=12597955,E=12597956,C=29)
	CT(48157)=(S=12597957,E=12597957,C=19)
	CT(48158)=(S=12597958,E=12597972,C=29)
	CT(48159)=(S=12597973,E=12597973,C=78)
	CT(48160)=(S=12597974,E=12597977,C=29)
	CT(48161)=(S=12597978,E=12597978,C=15)
	CT(48162)=(S=12597979,E=12597985,C=29)
	CT(48163)=(S=12597986,E=12597989,C=36)
	CT(48164)=(S=12597990,E=12598004,C=29)
	CT(48165)=(S=12598005,E=12598005,C=78)
	CT(48166)=(S=12598006,E=12599303,C=29)
	CT(48167)=(S=12599304,E=12599311,C=78)
	CT(48168)=(S=12599312,E=12599323,C=29)
	CT(48169)=(S=12599324,E=12599324,C=42)
	CT(48170)=(S=12599325,E=12599325,C=29)
	CT(48171)=(S=12599326,E=12599327,C=78)
	CT(48172)=(S=12599328,E=12599331,C=29)
	CT(48173)=(S=12599332,E=12599339,C=78)
	CT(48174)=(S=12599340,E=12599341,C=33)
	CT(48175)=(S=12599342,E=12599351,C=29)
	CT(48176)=(S=12599352,E=12599359,C=78)
	CT(48177)=(S=12599360,E=12599395,C=29)
	CT(48178)=(S=12599396,E=12599399,C=49)
	CT(48179)=(S=12599400,E=12599415,C=29)
	CT(48180)=(S=12599416,E=12599419,C=183)
	CT(48181)=(S=12599420,E=12599423,C=29)
	CT(48182)=(S=12599424,E=12599439,C=78)
	CT(48183)=(S=12599440,E=12599451,C=29)
	CT(48184)=(S=12599452,E=12599452,C=78)
	CT(48185)=(S=12599453,E=12599454,C=29)
	CT(48186)=(S=12599455,E=12599455,C=78)
	CT(48187)=(S=12599456,E=12599456,C=29)
	CT(48188)=(S=12599457,E=12599460,C=78)
	CT(48189)=(S=12599461,E=12599487,C=29)
	CT(48190)=(S=12599488,E=12599493,C=78)
	CT(48191)=(S=12599494,E=12599497,C=29)
	CT(48192)=(S=12599498,E=12599498,C=28)
	CT(48193)=(S=12599499,E=12599500,C=8)
	CT(48194)=(S=12599501,E=12599602,C=29)
	CT(48195)=(S=12599603,E=12599622,C=36)
	CT(48196)=(S=12599623,E=12599633,C=29)
	CT(48197)=(S=12599634,E=12599643,C=0)
	CT(48198)=(S=12599644,E=12599646,C=34)
	CT(48199)=(S=12599647,E=12599647,C=29)
	CT(48200)=(S=12599648,E=12599648,C=36)
	CT(48201)=(S=12599649,E=12599681,C=29)
	CT(48202)=(S=12599682,E=12599682,C=0)
	CT(48203)=(S=12599683,E=12599683,C=15)
	CT(48204)=(S=12599684,E=12599684,C=14)
	CT(48205)=(S=12599685,E=12599690,C=29)
	CT(48206)=(S=12599691,E=12599691,C=28)
	CT(48207)=(S=12599692,E=12599695,C=29)
	CT(48208)=(S=12599696,E=12599698,C=14)
	CT(48209)=(S=12599699,E=12599703,C=29)
	CT(48210)=(S=12599704,E=12599704,C=200)
	CT(48211)=(S=12599705,E=12599705,C=14)
	CT(48212)=(S=12599706,E=12599711,C=29)
	CT(48213)=(S=12599712,E=12599719,C=79)
	CT(48214)=(S=12599720,E=12599721,C=29)
	CT(48215)=(S=12599722,E=12599722,C=180)
	CT(48216)=(S=12599723,E=12599727,C=29)
	CT(48217)=(S=12599728,E=12599728,C=80)
	CT(48218)=(S=12599729,E=12599733,C=29)
	CT(48219)=(S=12599734,E=12599734,C=0)
	CT(48220)=(S=12599735,E=12599749,C=34)
	CT(48221)=(S=12599750,E=12599768,C=29)
	CT(48222)=(S=12599769,E=12599769,C=0)
	CT(48223)=(S=12599770,E=12599770,C=29)
	CT(48224)=(S=12599771,E=12599780,C=14)
	CT(48225)=(S=12599781,E=12599783,C=29)
	CT(48226)=(S=12599784,E=12599791,C=78)
	CT(48227)=(S=12599792,E=12599807,C=29)
	CT(48228)=(S=12599808,E=12600063,C=24)
	CT(48229)=(S=12600064,E=12600066,C=29)
	CT(48230)=(S=12600067,E=12600067,C=63)
	CT(48231)=(S=12600068,E=12600068,C=14)
	CT(48232)=(S=12600069,E=12600072,C=29)
	CT(48233)=(S=12600073,E=12600073,C=78)
	CT(48234)=(S=12600074,E=12600075,C=29)
	CT(48235)=(S=12600076,E=12600076,C=0)
	CT(48236)=(S=12600077,E=12600095,C=29)
	CT(48237)=(S=12600096,E=12600099,C=78)
	CT(48238)=(S=12600100,E=12600102,C=29)
	CT(48239)=(S=12600103,E=12600103,C=14)
	CT(48240)=(S=12600104,E=12600106,C=29)
	CT(48241)=(S=12600107,E=12600107,C=14)
	CT(48242)=(S=12600108,E=12600108,C=29)
	CT(48243)=(S=12600109,E=12600109,C=78)
	CT(48244)=(S=12600110,E=12600110,C=29)
	CT(48245)=(S=12600111,E=12600111,C=14)
	CT(48246)=(S=12600112,E=12600113,C=29)
	CT(48247)=(S=12600114,E=12600114,C=14)
	CT(48248)=(S=12600115,E=12600115,C=29)
	CT(48249)=(S=12600116,E=12600116,C=14)
	CT(48250)=(S=12600117,E=12600118,C=29)
	CT(48251)=(S=12600119,E=12600119,C=28)
	CT(48252)=(S=12600120,E=12600121,C=29)
	CT(48253)=(S=12600122,E=12600122,C=19)
	CT(48254)=(S=12600123,E=12600139,C=29)
	CT(48255)=(S=12600140,E=12600140,C=21)
	CT(48256)=(S=12600141,E=12600142,C=29)
	CT(48257)=(S=12600143,E=12600143,C=25)
	CT(48258)=(S=12600144,E=12600147,C=29)
	CT(48259)=(S=12600148,E=12600148,C=0)
	CT(48260)=(S=12600150,E=12600150,C=29)
	CT(48261)=(S=12600151,E=12600151,C=14)
	CT(48262)=(S=12600152,E=12600157,C=29)
	CT(48263)=(S=12600158,E=12600158,C=14)
	CT(48264)=(S=12600159,E=12600163,C=29)
	CT(48265)=(S=12600164,E=12600168,C=14)
	CT(48266)=(S=12600169,E=12600170,C=78)
	CT(48267)=(S=12600171,E=12600198,C=29)
	CT(48268)=(S=12600199,E=12600199,C=14)
	CT(48269)=(S=12600200,E=12600230,C=29)
	CT(48270)=(S=12600231,E=12600231,C=28)
	CT(48271)=(S=12600233,E=12600233,C=29)
	CT(48272)=(S=12600234,E=12600234,C=28)
	CT(48273)=(S=12600235,E=12600235,C=0)
	CT(48274)=(S=12600236,E=12600252,C=29)
	CT(48275)=(S=12600253,E=12600272,C=28)
	CT(48276)=(S=12600273,E=12600280,C=29)
	CT(48277)=(S=12600282,E=12600282,C=28)
	CT(48278)=(S=12600283,E=12600283,C=29)
	CT(48279)=(S=12600284,E=12600285,C=36)
	CT(48280)=(S=12600286,E=12600286,C=78)
	CT(48281)=(S=12600287,E=12600287,C=63)
	CT(48282)=(S=12600288,E=12600311,C=29)
	CT(48283)=(S=12600312,E=12600312,C=0)
	CT(48284)=(S=12600313,E=12600313,C=22)
	CT(48285)=(S=12600314,E=12600314,C=78)
	CT(48286)=(S=12600315,E=12600318,C=29)
	CT(48287)=(S=12600319,E=12600319,C=78)
	CT(48288)=(S=12600320,E=12600323,C=28)
	CT(48289)=(S=12600324,E=12600324,C=42)
	CT(48290)=(S=12600325,E=12600339,C=28)
	CT(48291)=(S=12600340,E=12600342,C=29)
	CT(48292)=(S=12600343,E=12600343,C=14)
	CT(48293)=(S=12600344,E=12600350,C=29)
	CT(48294)=(S=12600351,E=12600370,C=36)
	CT(48295)=(S=12600371,E=12600371,C=58)
	CT(48296)=(S=12600372,E=12600372,C=29)
	CT(48297)=(S=12600373,E=12600394,C=78)
	CT(48298)=(S=12600395,E=12600395,C=29)
	CT(48299)=(S=12600396,E=12600427,C=27)
	CT(48300)=(S=12600428,E=12600450,C=29)
	CT(48301)=(S=12600451,E=12600451,C=78)
	CT(48302)=(S=12600452,E=12600452,C=0)
	CT(48303)=(S=12600453,E=12600456,C=29)
	CT(48304)=(S=12600457,E=12600457,C=78)
	CT(48305)=(S=12600458,E=12600458,C=152)
	CT(48306)=(S=12600460,E=12600470,C=29)
	CT(48307)=(S=12600471,E=12600473,C=14)
	CT(48308)=(S=12600474,E=12600482,C=29)
	CT(48309)=(S=12600483,E=12600483,C=78)
	CT(48310)=(S=12600484,E=12600484,C=29)
	CT(48311)=(S=12600485,E=12600489,C=28)
	CT(48312)=(S=12600490,E=12600490,C=14)
	CT(48313)=(S=12600491,E=12600492,C=0)
	CT(48314)=(S=12600493,E=12600493,C=29)
	CT(48315)=(S=12600494,E=12600494,C=14)
	CT(48316)=(S=12600495,E=12600495,C=29)
	CT(48317)=(S=12600496,E=12600496,C=15)
	CT(48318)=(S=12600497,E=12600501,C=29)
	CT(48319)=(S=12600502,E=12600502,C=36)
	CT(48320)=(S=12600503,E=12600503,C=29)
	CT(48321)=(S=12600504,E=12600504,C=78)
	CT(48322)=(S=12600506,E=12600506,C=21)
	CT(48323)=(S=12600507,E=12600507,C=29)
	CT(48324)=(S=12600508,E=12600508,C=78)
	CT(48325)=(S=12600509,E=12600528,C=29)
	CT(48326)=(S=12600529,E=12600529,C=21)
	CT(48327)=(S=12600530,E=12600530,C=29)
	CT(48328)=(S=12600531,E=12600535,C=28)
	CT(48329)=(S=12600536,E=12600536,C=21)
	CT(48330)=(S=12600537,E=12600540,C=29)
	CT(48331)=(S=12600541,E=12600541,C=21)
	CT(48332)=(S=12600542,E=12600543,C=29)
	CT(48333)=(S=12600544,E=12600544,C=21)
	CT(48334)=(S=12600545,E=12600548,C=29)
	CT(48335)=(S=12600549,E=12600549,C=78)
	CT(48336)=(S=12600550,E=12600550,C=36)
	CT(48337)=(S=12600551,E=12600554,C=29)
	CT(48338)=(S=12600555,E=12600569,C=2)
	CT(48339)=(S=12600570,E=12600572,C=36)
	CT(48340)=(S=12600573,E=12600573,C=29)
	CT(48341)=(S=12600574,E=12600574,C=28)
	CT(48342)=(S=12600575,E=12600576,C=29)
	CT(48343)=(S=12600577,E=12600577,C=78)
	CT(48344)=(S=12600578,E=12600615,C=29)
	CT(48345)=(S=12600616,E=12600619,C=78)
	CT(48346)=(S=12600620,E=12600639,C=29)
	CT(48347)=(S=12600640,E=12600640,C=49)
	CT(48348)=(S=12600641,E=12600643,C=29)
	CT(48349)=(S=12600644,E=12600647,C=78)
	CT(48350)=(S=12600648,E=12600649,C=29)
	CT(48351)=(S=12600650,E=12600650,C=78)
	CT(48352)=(S=12600651,E=12600661,C=29)
	CT(48353)=(S=12600662,E=12600663,C=78)
	CT(48354)=(S=12600664,E=12600676,C=29)
	CT(48355)=(S=12600677,E=12600677,C=78)
	CT(48356)=(S=12600678,E=12600689,C=29)
	CT(48357)=(S=12600690,E=12600691,C=78)
	CT(48358)=(S=12600692,E=12600692,C=0)
	CT(48359)=(S=12600693,E=12600707,C=29)
	CT(48360)=(S=12600708,E=12600711,C=78)
	CT(48361)=(S=12600712,E=12600724,C=29)
	CT(48362)=(S=12600725,E=12600734,C=78)
	CT(48363)=(S=12600735,E=12600809,C=29)
	CT(48364)=(S=12600810,E=12600810,C=28)
	CT(48365)=(S=12600811,E=12600831,C=29)
	CT(48366)=(S=12600832,E=12600951,C=12)
	CT(48367)=(S=12600952,E=12600964,C=29)
	CT(48368)=(S=12600965,E=12600966,C=37)
	CT(48369)=(S=12600967,E=12600967,C=0)
	CT(48370)=(S=12600968,E=12600968,C=15)
	CT(48371)=(S=12600969,E=12600971,C=29)
	CT(48372)=(S=12600972,E=12600991,C=28)
	CT(48373)=(S=12600992,E=12601003,C=29)
	CT(48374)=(S=12601004,E=12601004,C=78)
	CT(48375)=(S=12601005,E=12601005,C=29)
	CT(48376)=(S=12601006,E=12601006,C=0)
	CT(48377)=(S=12601007,E=12601007,C=29)
	CT(48378)=(S=12601008,E=12601009,C=78)
	CT(48379)=(S=12601010,E=12601010,C=29)
	CT(48380)=(S=12601011,E=12601013,C=2)
	CT(48381)=(S=12601016,E=12601047,C=29)
	CT(48382)=(S=12601048,E=12601048,C=0)
	CT(48383)=(S=12601049,E=12601050,C=29)
	CT(48384)=(S=12601051,E=12601051,C=0)
	CT(48385)=(S=12601052,E=12601052,C=78)
	CT(48386)=(S=12601053,E=12601055,C=29)
	CT(48387)=(S=12601056,E=12601056,C=78)
	CT(48388)=(S=12601057,E=12601068,C=29)
	CT(48389)=(S=12601069,E=12601069,C=105)
	CT(48390)=(S=12601070,E=12601073,C=29)
	CT(48391)=(S=12601074,E=12601074,C=14)
	CT(48392)=(S=12601075,E=12601087,C=29)
	CT(48393)=(S=12601088,E=12601343,C=19)
	CT(48394)=(S=12601344,E=12601346,C=29)
	CT(48395)=(S=12601347,E=12601596,C=9)
	CT(48396)=(S=12601597,E=12601604,C=29)
	CT(48397)=(S=12601605,E=12601605,C=78)
	CT(48398)=(S=12601606,E=12601611,C=29)
	CT(48399)=(S=12601612,E=12601612,C=0)
	CT(48400)=(S=12601613,E=12601618,C=29)
	CT(48401)=(S=12601619,E=12601620,C=14)
	CT(48402)=(S=12601621,E=12601621,C=73)
	CT(48403)=(S=12601622,E=12601622,C=0)
	CT(48404)=(S=12601623,E=12601623,C=29)
	CT(48405)=(S=12601624,E=12601625,C=74)
	CT(48406)=(S=12601626,E=12601633,C=29)
	CT(48407)=(S=12601634,E=12601634,C=28)
	CT(48408)=(S=12601635,E=12601644,C=34)
	CT(48409)=(S=12601645,E=12601665,C=29)
	CT(48410)=(S=12601666,E=12601666,C=0)
	CT(48411)=(S=12601667,E=12601825,C=29)
	CT(48412)=(S=12601826,E=12601826,C=49)
	CT(48413)=(S=12601827,E=12601828,C=29)
	CT(48414)=(S=12601829,E=12601829,C=14)
	CT(48415)=(S=12601830,E=12601994,C=29)
	CT(48416)=(S=12601995,E=12602064,C=0)
	CT(48417)=(S=12602065,E=12602112,C=29)
	CT(48418)=(S=12602113,E=12602122,C=78)
	CT(48419)=(S=12602123,E=12602123,C=29)
	CT(48420)=(S=12602124,E=12602130,C=78)
	CT(48421)=(S=12602131,E=12602131,C=29)
	CT(48422)=(S=12602132,E=12602134,C=78)
	CT(48423)=(S=12602135,E=12602135,C=29)
	CT(48424)=(S=12602136,E=12602173,C=78)
	CT(48425)=(S=12602174,E=12602174,C=29)
	CT(48426)=(S=12602175,E=12602177,C=78)
	CT(48427)=(S=12602178,E=12602178,C=29)
	CT(48428)=(S=12602179,E=12602217,C=78)
	CT(48429)=(S=12602218,E=12602218,C=29)
	CT(48430)=(S=12602219,E=12602246,C=78)
	CT(48431)=(S=12602247,E=12602248,C=29)
	CT(48432)=(S=12602250,E=12602255,C=78)
	CT(48433)=(S=12602256,E=12602259,C=29)
	CT(48434)=(S=12602260,E=12602298,C=78)
	CT(48435)=(S=12602299,E=12602299,C=80)
	CT(48436)=(S=12602300,E=12602300,C=29)
	CT(48437)=(S=12602301,E=12602303,C=78)
	CT(48438)=(S=12602304,E=12602311,C=29)
	CT(48439)=(S=12602312,E=12602320,C=78)
	CT(48440)=(S=12602321,E=12602321,C=29)
	CT(48441)=(S=12602322,E=12602322,C=78)
	CT(48442)=(S=12602323,E=12602323,C=29)
	CT(48443)=(S=12602324,E=12602366,C=78)
	CT(48444)=(S=12602367,E=12602373,C=29)
	CT(48445)=(S=12602374,E=12602403,C=14)
	CT(48446)=(S=12602404,E=12602489,C=29)
	CT(48447)=(S=12602490,E=12602490,C=0)
	CT(48448)=(S=12602491,E=12602500,C=28)
	CT(48449)=(S=12602501,E=12602501,C=29)
	CT(48450)=(S=12602502,E=12602505,C=28)
	CT(48451)=(S=12602506,E=12602506,C=42)
	CT(48452)=(S=12602507,E=12602540,C=28)
	CT(48453)=(S=12602541,E=12602543,C=29)
	CT(48454)=(S=12602544,E=12602544,C=28)
	CT(48455)=(S=12602545,E=12602608,C=29)
	CT(48456)=(S=12602609,E=12602609,C=28)
	CT(48457)=(S=12602610,E=12602610,C=21)
	CT(48458)=(S=12602611,E=12602612,C=42)
	CT(48459)=(S=12602613,E=12602616,C=28)
	CT(48460)=(S=12602617,E=12602634,C=29)
	CT(48461)=(S=12602635,E=12602635,C=34)
	CT(48462)=(S=12602636,E=12602669,C=29)
	CT(48463)=(S=12602670,E=12602699,C=78)
	CT(48464)=(S=12602700,E=12602737,C=29)
	CT(48465)=(S=12602738,E=12602739,C=28)
	CT(48466)=(S=12602740,E=12602743,C=29)
	CT(48467)=(S=12602744,E=12602747,C=47)
	CT(48468)=(S=12602748,E=12602755,C=29)
	CT(48469)=(S=12602756,E=12602762,C=14)
	CT(48470)=(S=12602763,E=12602763,C=29)
	CT(48471)=(S=12602764,E=12602765,C=25)
	CT(48472)=(S=12602766,E=12603411,C=29)
	CT(48473)=(S=12603412,E=12603413,C=21)
	CT(48474)=(S=12603414,E=12603415,C=29)
	CT(48475)=(S=12603416,E=12603416,C=203)
	CT(48476)=(S=12603417,E=12603422,C=29)
	CT(48477)=(S=12603423,E=12603434,C=34)
	CT(48478)=(S=12603435,E=12603437,C=29)
	CT(48479)=(S=12603438,E=12603438,C=14)
	CT(48480)=(S=12603439,E=12603442,C=29)
	CT(48481)=(S=12603443,E=12603443,C=28)
	CT(48482)=(S=12603444,E=12603471,C=29)
	CT(48483)=(S=12603472,E=12603472,C=78)
	CT(48484)=(S=12603473,E=12603600,C=29)
	CT(48485)=(S=12603601,E=12603601,C=189)
	CT(48486)=(S=12603602,E=12603659,C=29)
	CT(48487)=(S=12603660,E=12603679,C=78)
	CT(48488)=(S=12603680,E=12603699,C=29)
	CT(48489)=(S=12603700,E=12603705,C=78)
	CT(48490)=(S=12603706,E=12603706,C=29)
	CT(48491)=(S=12603707,E=12603707,C=49)
	CT(48492)=(S=12603708,E=12603708,C=29)
	CT(48493)=(S=12603709,E=12603710,C=14)
	CT(48494)=(S=12603711,E=12603713,C=29)
	CT(48495)=(S=12603714,E=12603714,C=78)
	CT(48496)=(S=12603715,E=12603717,C=29)
	CT(48497)=(S=12603718,E=12603719,C=78)
	CT(48498)=(S=12603720,E=12603721,C=180)
	CT(48499)=(S=12603722,E=12603731,C=29)
	CT(48500)=(S=12603732,E=12603733,C=78)
	CT(48501)=(S=12603734,E=12603736,C=29)
	CT(48502)=(S=12603737,E=12603737,C=78)
	CT(48503)=(S=12603738,E=12603756,C=29)
	CT(48504)=(S=12603757,E=12603757,C=27)
	CT(48505)=(S=12603758,E=12603768,C=29)
	CT(48506)=(S=12603769,E=12603769,C=28)
	CT(48507)=(S=12603770,E=12603770,C=29)
	CT(48508)=(S=12603771,E=12603771,C=47)
	CT(48509)=(S=12603772,E=12603807,C=29)
	CT(48510)=(S=12603808,E=12603808,C=47)
	CT(48511)=(S=12603809,E=12603809,C=29)
	CT(48512)=(S=12603810,E=12603813,C=78)
	CT(48513)=(S=12603814,E=12603815,C=29)
	CT(48514)=(S=12603816,E=12603823,C=78)
	CT(48515)=(S=12603824,E=12603829,C=29)
	CT(48516)=(S=12603830,E=12603832,C=13)
	CT(48517)=(S=12603833,E=12603840,C=29)
	CT(48518)=(S=12603841,E=12603841,C=78)
	CT(48519)=(S=12603842,E=12603842,C=19)
	CT(48520)=(S=12603843,E=12603871,C=29)
	CT(48521)=(S=12603872,E=12603875,C=78)
	CT(48522)=(S=12603876,E=12603877,C=29)
	CT(48523)=(S=12603878,E=12603878,C=28)
	CT(48524)=(S=12603879,E=12603881,C=29)
	CT(48525)=(S=12603882,E=12603882,C=14)
	CT(48526)=(S=12603883,E=12604007,C=29)
	CT(48527)=(S=12604008,E=12604008,C=78)
	CT(48528)=(S=12604009,E=12604020,C=29)
	CT(48529)=(S=12604021,E=12604021,C=78)
	CT(48530)=(S=12604022,E=12604024,C=29)
	CT(48531)=(S=12604025,E=12604025,C=14)
	CT(48532)=(S=12604026,E=12604027,C=29)
	CT(48533)=(S=12604028,E=12604028,C=34)
	CT(48534)=(S=12604029,E=12604030,C=29)
	CT(48535)=(S=12604031,E=12604031,C=21)
	CT(48536)=(S=12604032,E=12604035,C=78)
	CT(48537)=(S=12604037,E=12604043,C=29)
	CT(48538)=(S=12604044,E=12604044,C=0)
	CT(48539)=(S=12604045,E=12604045,C=29)
	CT(48540)=(S=12604046,E=12604046,C=105)
	CT(48541)=(S=12604047,E=12604047,C=0)
	CT(48542)=(S=12604048,E=12604053,C=29)
	CT(48543)=(S=12604054,E=12604054,C=78)
	CT(48544)=(S=12604055,E=12604056,C=29)
	CT(48545)=(S=12604057,E=12604057,C=14)
	CT(48546)=(S=12604058,E=12604060,C=29)
	CT(48547)=(S=12604061,E=12604062,C=42)
	CT(48548)=(S=12604063,E=12604064,C=29)
	CT(48549)=(S=12604065,E=12604065,C=0)
	CT(48550)=(S=12604066,E=12604117,C=29)
	CT(48551)=(S=12604118,E=12604118,C=21)
	CT(48552)=(S=12604119,E=12604123,C=29)
	CT(48553)=(S=12604124,E=12604125,C=15)
	CT(48554)=(S=12604126,E=12604126,C=0)
	CT(48555)=(S=12604127,E=12604144,C=29)
	CT(48556)=(S=12604145,E=12604145,C=28)
	CT(48557)=(S=12604146,E=12604159,C=29)
	CT(48558)=(S=12604160,E=12604260,C=47)
	CT(48559)=(S=12604261,E=12604261,C=78)
	CT(48560)=(S=12604262,E=12604262,C=14)
	CT(48561)=(S=12604263,E=12604277,C=29)
	CT(48562)=(S=12604278,E=12604279,C=0)
	CT(48563)=(S=12604280,E=12604280,C=29)
	CT(48564)=(S=12604281,E=12604281,C=78)
	CT(48565)=(S=12604283,E=12604283,C=0)
	CT(48566)=(S=12604284,E=12604319,C=29)
	CT(48567)=(S=12604320,E=12604320,C=12)
	CT(48568)=(S=12604321,E=12604324,C=29)
	CT(48569)=(S=12604325,E=12604325,C=58)
	CT(48570)=(S=12604326,E=12604356,C=9)
	CT(48571)=(S=12604357,E=12604357,C=29)
	CT(48572)=(S=12604358,E=12604358,C=0)
	CT(48573)=(S=12604359,E=12604359,C=29)
	CT(48574)=(S=12604360,E=12604360,C=14)
	CT(48575)=(S=12604361,E=12604361,C=29)
	CT(48576)=(S=12604362,E=12604362,C=14)
	CT(48577)=(S=12604363,E=12604366,C=29)
	CT(48578)=(S=12604378,E=12604379,C=2)
	CT(48579)=(S=12604380,E=12604382,C=29)
	CT(48580)=(S=12604383,E=12604383,C=34)
	CT(48581)=(S=12604384,E=12604384,C=0)
	CT(48582)=(S=12604385,E=12604388,C=29)
	CT(48583)=(S=12604389,E=12604389,C=28)
	CT(48584)=(S=12604390,E=12604390,C=29)
	CT(48585)=(S=12604391,E=12604391,C=0)
	CT(48586)=(S=12604392,E=12604396,C=29)
	CT(48587)=(S=12604397,E=12604398,C=0)
	CT(48588)=(S=12604399,E=12604406,C=29)
	CT(48589)=(S=12604407,E=12604407,C=78)
	CT(48590)=(S=12604408,E=12604420,C=29)
	CT(48591)=(S=12604421,E=12604421,C=14)
	CT(48592)=(S=12604422,E=12604427,C=29)
	CT(48593)=(S=12604428,E=12604428,C=204)
	CT(48594)=(S=12604429,E=12604429,C=21)
	CT(48595)=(S=12604430,E=12604430,C=29)
	CT(48596)=(S=12604431,E=12604431,C=21)
	CT(48597)=(S=12604432,E=12604442,C=29)
	CT(48598)=(S=12604443,E=12604443,C=42)
	CT(48599)=(S=12604444,E=12604445,C=29)
	CT(48600)=(S=12604446,E=12604446,C=36)
	CT(48601)=(S=12604447,E=12604447,C=29)
	CT(48602)=(S=12604448,E=12604449,C=27)
	CT(48603)=(S=12604450,E=12604477,C=29)
	CT(48604)=(S=12604478,E=12604478,C=21)
	CT(48605)=(S=12604479,E=12604490,C=29)
	CT(48606)=(S=12604491,E=12604500,C=14)
	CT(48607)=(S=12604501,E=12604502,C=29)
	CT(48608)=(S=12604503,E=12604503,C=15)
	CT(48609)=(S=12604504,E=12604504,C=29)
	CT(48610)=(S=12604505,E=12604505,C=0)
	CT(48611)=(S=12604506,E=12604506,C=52)
	CT(48612)=(S=12604507,E=12604508,C=57)
	CT(48613)=(S=12604509,E=12604515,C=52)
	CT(48614)=(S=12604516,E=12604516,C=29)
	CT(48615)=(S=12604517,E=12604520,C=52)
	CT(48616)=(S=12604521,E=12604522,C=57)
	CT(48617)=(S=12604523,E=12604525,C=52)
	CT(48618)=(S=12604526,E=12604542,C=29)
	CT(48619)=(S=12604543,E=12604572,C=15)
	CT(48620)=(S=12604573,E=12604581,C=29)
	CT(48621)=(S=12604582,E=12604582,C=34)
	CT(48622)=(S=12604583,E=12604588,C=29)
	CT(48623)=(S=12604589,E=12604589,C=14)
	CT(48624)=(S=12604590,E=12604591,C=29)
	CT(48625)=(S=12604592,E=12604623,C=13)
	CT(48626)=(S=12604624,E=12604627,C=29)
	CT(48627)=(S=12604628,E=12604628,C=14)
	CT(48628)=(S=12604629,E=12604634,C=29)
	CT(48629)=(S=12604635,E=12604635,C=0)
	CT(48630)=(S=12604636,E=12604636,C=28)
	CT(48631)=(S=12604637,E=12604637,C=42)
	CT(48632)=(S=12604638,E=12604641,C=29)
	CT(48633)=(S=12604642,E=12604645,C=50)
	CT(48634)=(S=12604646,E=12604646,C=0)
	CT(48635)=(S=12604647,E=12604652,C=29)
	CT(48636)=(S=12604653,E=12604654,C=0)
	CT(48637)=(S=12604655,E=12604656,C=29)
	CT(48638)=(S=12604658,E=12604658,C=0)
	CT(48639)=(S=12604659,E=12604659,C=29)
	CT(48640)=(S=12604660,E=12604660,C=105)
	CT(48641)=(S=12604661,E=12604663,C=28)
	CT(48642)=(S=12604664,E=12604668,C=29)
	CT(48643)=(S=12604669,E=12604669,C=73)
	CT(48644)=(S=12604670,E=12604928,C=29)
	CT(48645)=(S=12604929,E=12604929,C=78)
	CT(48646)=(S=12604930,E=12604938,C=29)
	CT(48647)=(S=12604939,E=12604939,C=18)
	CT(48648)=(S=12604941,E=12604941,C=29)
	CT(48649)=(S=12604942,E=12604942,C=58)
	CT(48650)=(S=12604943,E=12604945,C=29)
	CT(48651)=(S=12604946,E=12604946,C=14)
	CT(48652)=(S=12604947,E=12604952,C=29)
	CT(48653)=(S=12604953,E=12604953,C=17)
	CT(48654)=(S=12604954,E=12604954,C=29)
	CT(48655)=(S=12604955,E=12604955,C=14)
	CT(48656)=(S=12604956,E=12605016,C=29)
	CT(48657)=(S=12605017,E=12605017,C=17)
	CT(48658)=(S=12605018,E=12605052,C=29)
	CT(48659)=(S=12605053,E=12605054,C=36)
	CT(48660)=(S=12605055,E=12605055,C=14)
	CT(48661)=(S=12605056,E=12605059,C=29)
	CT(48662)=(S=12605062,E=12605062,C=14)
	CT(48663)=(S=12605063,E=12605064,C=29)
	CT(48664)=(S=12605065,E=12605065,C=14)
	CT(48665)=(S=12605066,E=12605066,C=21)
	CT(48666)=(S=12605067,E=12605090,C=29)
	CT(48667)=(S=12605091,E=12605091,C=28)
	CT(48668)=(S=12605092,E=12605092,C=29)
	CT(48669)=(S=12605093,E=12605093,C=0)
	CT(48670)=(S=12605094,E=12605094,C=34)
	CT(48671)=(S=12605095,E=12605095,C=12)
	CT(48672)=(S=12605096,E=12605096,C=29)
	CT(48673)=(S=12605097,E=12605097,C=14)
	CT(48674)=(S=12605098,E=12605181,C=29)
	CT(48675)=(S=12605182,E=12605182,C=19)
	CT(48676)=(S=12605183,E=12605183,C=29)
	CT(48677)=(S=12605184,E=12605439,C=36)
	CT(48678)=(S=12605440,E=12605440,C=29)
	CT(48679)=(S=12605441,E=12605441,C=34)
	CT(48680)=(S=12605442,E=12605443,C=29)
	CT(48681)=(S=12605444,E=12605444,C=28)
	CT(48682)=(S=12605445,E=12605445,C=29)
	CT(48683)=(S=12605446,E=12605447,C=0)
	CT(48684)=(S=12605448,E=12605448,C=29)
	CT(48685)=(S=12605449,E=12605450,C=14)
	CT(48686)=(S=12605451,E=12605456,C=29)
	CT(48687)=(S=12605457,E=12605457,C=21)
	CT(48688)=(S=12605458,E=12605462,C=29)
	CT(48689)=(S=12605463,E=12605464,C=42)
	CT(48690)=(S=12605465,E=12605522,C=29)
	CT(48691)=(S=12605523,E=12605524,C=14)
	CT(48692)=(S=12605525,E=12605525,C=73)
	CT(48693)=(S=12605526,E=12605526,C=47)
	CT(48694)=(S=12605527,E=12605536,C=29)
	CT(48695)=(S=12605537,E=12605538,C=28)
	CT(48696)=(S=12605540,E=12605547,C=29)
	CT(48697)=(S=12605548,E=12605548,C=28)
	CT(48698)=(S=12605549,E=12605557,C=29)
	CT(48699)=(S=12605558,E=12605558,C=14)
	CT(48700)=(S=12605560,E=12605562,C=29)
	CT(48701)=(S=12605563,E=12605563,C=21)
	CT(48702)=(S=12605564,E=12605567,C=29)
	CT(48703)=(S=12605568,E=12605568,C=14)
	CT(48704)=(S=12605569,E=12605569,C=29)
	CT(48705)=(S=12605570,E=12605570,C=14)
	CT(48706)=(S=12605571,E=12605572,C=29)
	CT(48707)=(S=12605573,E=12605573,C=14)
	CT(48708)=(S=12605574,E=12605626,C=29)
	CT(48709)=(S=12605627,E=12605627,C=0)
	CT(48710)=(S=12605628,E=12605629,C=29)
	CT(48711)=(S=12605630,E=12605630,C=73)
	CT(48712)=(S=12605631,E=12605635,C=29)
	CT(48713)=(S=12605636,E=12605636,C=15)
	CT(48714)=(S=12605637,E=12605643,C=29)
	CT(48715)=(S=12605644,E=12605644,C=58)
	CT(48716)=(S=12605645,E=12605677,C=29)
	CT(48717)=(S=12605678,E=12605678,C=14)
	CT(48718)=(S=12605679,E=12605679,C=28)
	CT(48719)=(S=12605680,E=12605689,C=29)
	CT(48720)=(S=12605690,E=12605694,C=21)
	CT(48721)=(S=12605695,E=12605695,C=29)
	CT(48722)=(S=12605696,E=12605951,C=47)
	CT(48723)=(S=12605952,E=12606346,C=29)
	CT(48724)=(S=12606348,E=12606349,C=70)
	CT(48725)=(S=12606350,E=12606384,C=29)
	CT(48726)=(S=12606385,E=12606385,C=21)
	CT(48727)=(S=12606386,E=12606386,C=29)
	CT(48728)=(S=12606387,E=12606387,C=0)
	CT(48729)=(S=12606388,E=12606392,C=29)
	CT(48730)=(S=12606393,E=12606393,C=0)
	CT(48731)=(S=12606394,E=12606394,C=18)
	CT(48732)=(S=12606395,E=12606396,C=29)
	CT(48733)=(S=12606397,E=12606397,C=27)
	CT(48734)=(S=12606398,E=12606398,C=29)
	CT(48735)=(S=12606399,E=12606399,C=14)
	CT(48736)=(S=12606400,E=12606406,C=29)
	CT(48737)=(S=12606407,E=12606409,C=14)
	CT(48738)=(S=12606410,E=12606417,C=29)
	CT(48739)=(S=12606418,E=12606418,C=78)
	CT(48740)=(S=12606419,E=12606419,C=14)
	CT(48741)=(S=12606420,E=12606420,C=29)
	CT(48742)=(S=12606422,E=12606441,C=19)
	CT(48743)=(S=12606442,E=12606443,C=29)
	CT(48744)=(S=12606444,E=12606455,C=34)
	CT(48745)=(S=12606456,E=12606549,C=29)
	CT(48746)=(S=12606550,E=12606550,C=12)
	CT(48747)=(S=12606551,E=12606557,C=29)
	CT(48748)=(S=12606558,E=12606558,C=63)
	CT(48749)=(S=12606559,E=12606567,C=29)
	CT(48750)=(S=12606568,E=12606570,C=15)
	CT(48751)=(S=12606571,E=12606571,C=29)
	CT(48752)=(S=12606572,E=12606572,C=36)
	CT(48753)=(S=12606573,E=12606573,C=14)
	CT(48754)=(S=12606574,E=12606579,C=29)
	CT(48755)=(S=12606580,E=12606580,C=47)
	CT(48756)=(S=12606581,E=12606588,C=29)
	CT(48757)=(S=12606589,E=12606589,C=42)
	CT(48758)=(S=12606590,E=12606590,C=15)
	CT(48759)=(S=12606591,E=12606591,C=36)
	CT(48760)=(S=12606592,E=12606592,C=34)
	CT(48761)=(S=12606593,E=12606593,C=51)
	CT(48762)=(S=12606594,E=12606594,C=49)
	CT(48763)=(S=12606595,E=12606596,C=36)
	CT(48764)=(S=12606597,E=12606597,C=21)
	CT(48765)=(S=12606598,E=12606598,C=36)
	CT(48766)=(S=12606599,E=12606599,C=21)
	CT(48767)=(S=12606600,E=12606601,C=36)
	CT(48768)=(S=12606602,E=12606602,C=42)
	CT(48769)=(S=12606603,E=12606603,C=34)
	CT(48770)=(S=12606604,E=12606604,C=29)
	CT(48771)=(S=12606605,E=12606605,C=42)
	CT(48772)=(S=12606606,E=12606617,C=21)
	CT(48773)=(S=12606619,E=12606620,C=22)
	CT(48774)=(S=12606621,E=12606656,C=29)
	CT(48775)=(S=12606657,E=12606657,C=80)
	CT(48776)=(S=12606658,E=12606679,C=29)
	CT(48777)=(S=12606680,E=12606680,C=27)
	CT(48778)=(S=12606681,E=12606719,C=29)
	CT(48779)=(S=12606720,E=12606975,C=12)
	CT(48780)=(S=12606976,E=12606999,C=29)
	CT(48781)=(S=12607000,E=12607000,C=21)
	CT(48782)=(S=12607001,E=12607003,C=29)
	CT(48783)=(S=12607004,E=12607004,C=42)
	CT(48784)=(S=12607005,E=12607016,C=29)
	CT(48785)=(S=12607017,E=12607017,C=0)
	CT(48786)=(S=12607018,E=12607024,C=29)
	CT(48787)=(S=12607025,E=12607025,C=78)
	CT(48788)=(S=12607026,E=12607032,C=29)
	CT(48789)=(S=12607033,E=12607033,C=42)
	CT(48790)=(S=12607034,E=12607034,C=14)
	CT(48791)=(S=12607035,E=12607036,C=29)
	CT(48792)=(S=12607037,E=12607037,C=105)
	CT(48793)=(S=12607038,E=12607040,C=0)
	CT(48794)=(S=12607041,E=12607042,C=29)
	CT(48795)=(S=12607043,E=12607044,C=22)
	CT(48796)=(S=12607045,E=12607051,C=29)
	CT(48797)=(S=12607052,E=12607052,C=28)
	CT(48798)=(S=12607055,E=12607086,C=29)
	CT(48799)=(S=12607087,E=12607093,C=14)
	CT(48800)=(S=12607094,E=12607097,C=29)
	CT(48801)=(S=12607098,E=12607098,C=157)
	CT(48802)=(S=12607099,E=12607101,C=29)
	CT(48803)=(S=12607102,E=12607102,C=78)
	CT(48804)=(S=12607103,E=12607131,C=29)
	CT(48805)=(S=12607132,E=12607138,C=24)
	CT(48806)=(S=12607139,E=12607139,C=25)
	CT(48807)=(S=12607140,E=12607144,C=29)
	CT(48808)=(S=12607145,E=12607145,C=73)
	CT(48809)=(S=12607146,E=12607147,C=29)
	CT(48810)=(S=12607148,E=12607148,C=14)
	CT(48811)=(S=12607149,E=12607149,C=29)
	CT(48812)=(S=12607150,E=12607150,C=8)
	CT(48813)=(S=12607151,E=12607175,C=12)
	CT(48814)=(S=12607177,E=12607183,C=29)
	CT(48815)=(S=12607184,E=12607185,C=0)
	CT(48816)=(S=12607186,E=12607186,C=105)
	CT(48817)=(S=12607187,E=12607187,C=29)
	CT(48818)=(S=12607188,E=12607188,C=15)
	CT(48819)=(S=12607189,E=12607195,C=29)
	CT(48820)=(S=12607197,E=12607197,C=18)
	CT(48821)=(S=12607198,E=12607201,C=29)
	CT(48822)=(S=12607202,E=12607202,C=42)
	CT(48823)=(S=12607203,E=12607203,C=0)
	CT(48824)=(S=12607204,E=12607208,C=29)
	CT(48825)=(S=12607209,E=12607209,C=18)
	CT(48826)=(S=12607210,E=12607210,C=29)
	CT(48827)=(S=12607211,E=12607211,C=14)
	CT(48828)=(S=12607212,E=12607213,C=29)
	CT(48829)=(S=12607215,E=12607215,C=42)
	CT(48830)=(S=12607216,E=12607217,C=105)
	CT(48831)=(S=12607218,E=12607218,C=29)
	CT(48832)=(S=12607219,E=12607220,C=0)
	CT(48833)=(S=12607221,E=12607221,C=29)
	CT(48834)=(S=12607222,E=12607222,C=105)
	CT(48835)=(S=12607223,E=12607226,C=29)
	CT(48836)=(S=12607227,E=12607227,C=78)
	CT(48837)=(S=12607228,E=12607231,C=29)
	CT(48838)=(S=12607232,E=12607295,C=78)
	CT(48839)=(S=12607297,E=12607297,C=29)
	CT(48840)=(S=12607298,E=12607299,C=78)
	CT(48841)=(S=12607300,E=12607359,C=29)
	CT(48842)=(S=12607360,E=12607488,C=78)
	CT(48843)=(S=12607489,E=12607504,C=105)
	CT(48844)=(S=12607505,E=12607506,C=29)
	CT(48845)=(S=12607507,E=12607510,C=105)
	CT(48846)=(S=12607511,E=12607511,C=29)
	CT(48847)=(S=12607512,E=12607523,C=105)
	CT(48848)=(S=12607524,E=12607524,C=29)
	CT(48849)=(S=12607525,E=12607528,C=105)
	CT(48850)=(S=12607529,E=12607530,C=29)
	CT(48851)=(S=12607531,E=12607531,C=105)
	CT(48852)=(S=12607532,E=12607532,C=29)
	CT(48853)=(S=12607533,E=12607546,C=105)
	CT(48854)=(S=12607547,E=12607547,C=29)
	CT(48855)=(S=12607548,E=12607549,C=105)
	CT(48856)=(S=12607550,E=12607555,C=29)
	CT(48857)=(S=12607556,E=12607560,C=105)
	CT(48858)=(S=12607561,E=12607561,C=29)
	CT(48859)=(S=12607562,E=12607565,C=105)
	CT(48860)=(S=12607566,E=12607566,C=78)
	CT(48861)=(S=12607567,E=12607590,C=105)
	CT(48862)=(S=12607591,E=12607591,C=29)
	CT(48863)=(S=12607592,E=12607622,C=105)
	CT(48864)=(S=12607624,E=12607625,C=185)
	CT(48865)=(S=12607626,E=12607643,C=105)
	CT(48866)=(S=12607644,E=12607645,C=29)
	CT(48867)=(S=12607646,E=12607646,C=105)
	CT(48868)=(S=12607647,E=12607651,C=29)
	CT(48869)=(S=12607652,E=12607655,C=105)
	CT(48870)=(S=12607656,E=12607664,C=29)
	CT(48871)=(S=12607665,E=12607679,C=105)
	CT(48872)=(S=12607680,E=12607680,C=29)
	CT(48873)=(S=12607681,E=12607687,C=105)
	CT(48874)=(S=12607688,E=12607711,C=29)
	CT(48875)=(S=12607712,E=12607720,C=105)
	CT(48876)=(S=12607721,E=12607721,C=29)
	CT(48877)=(S=12607722,E=12607731,C=105)
	CT(48878)=(S=12607732,E=12607732,C=29)
	CT(48879)=(S=12607733,E=12607742,C=105)
	CT(48880)=(S=12607743,E=12607999,C=29)
	CT(48881)=(S=12608000,E=12608255,C=47)
	CT(48882)=(S=12608256,E=12608511,C=78)
	CT(48883)=(S=12608512,E=12608512,C=29)
	CT(48884)=(S=12608513,E=12608513,C=105)
	CT(48885)=(S=12608514,E=12608514,C=6)
	CT(48886)=(S=12608515,E=12608529,C=29)
	CT(48887)=(S=12608530,E=12608530,C=12)
	CT(48888)=(S=12608531,E=12608533,C=29)
	CT(48889)=(S=12608534,E=12608534,C=0)
	CT(48890)=(S=12608535,E=12608537,C=42)
	CT(48891)=(S=12608538,E=12608563,C=29)
	CT(48892)=(S=12608564,E=12608564,C=14)
	CT(48893)=(S=12608565,E=12608565,C=73)
	CT(48894)=(S=12608566,E=12608572,C=29)
	CT(48895)=(S=12608573,E=12608573,C=14)
	CT(48896)=(S=12608574,E=12608574,C=29)
	CT(48897)=(S=12608575,E=12608575,C=27)
	CT(48898)=(S=12608576,E=12608588,C=29)
	CT(48899)=(S=12608589,E=12608589,C=3)
	CT(48900)=(S=12608590,E=12608590,C=14)
	CT(48901)=(S=12608591,E=12608591,C=29)
	CT(48902)=(S=12608592,E=12608592,C=0)
	CT(48903)=(S=12608593,E=12608598,C=29)
	CT(48904)=(S=12608599,E=12608599,C=78)
	CT(48905)=(S=12608600,E=12608607,C=29)
	CT(48906)=(S=12608608,E=12608610,C=28)
	CT(48907)=(S=12608611,E=12608611,C=14)
	CT(48908)=(S=12608612,E=12608612,C=29)
	CT(48909)=(S=12608613,E=12608613,C=78)
	CT(48910)=(S=12608614,E=12608645,C=47)
	CT(48911)=(S=12608646,E=12608646,C=29)
	CT(48912)=(S=12608647,E=12608652,C=14)
	CT(48913)=(S=12608653,E=12608655,C=29)
	CT(48914)=(S=12608656,E=12608656,C=14)
	CT(48915)=(S=12608657,E=12608665,C=29)
	CT(48916)=(S=12608666,E=12608666,C=14)
	CT(48917)=(S=12608667,E=12608668,C=204)
	CT(48918)=(S=12608669,E=12608669,C=83)
	CT(48919)=(S=12608670,E=12608682,C=204)
	CT(48920)=(S=12608683,E=12608683,C=208)
	CT(48921)=(S=12608684,E=12608688,C=204)
	CT(48922)=(S=12608689,E=12608689,C=189)
	CT(48923)=(S=12608690,E=12608693,C=204)
	CT(48924)=(S=12608694,E=12608694,C=202)
	CT(48925)=(S=12608695,E=12608697,C=204)
	CT(48926)=(S=12608698,E=12608699,C=83)
	CT(48927)=(S=12608700,E=12608709,C=204)
	CT(48928)=(S=12608710,E=12608710,C=189)
	CT(48929)=(S=12608711,E=12608717,C=204)
	CT(48930)=(S=12608718,E=12608718,C=189)
	CT(48931)=(S=12608719,E=12608725,C=204)
	CT(48932)=(S=12608726,E=12608727,C=189)
	CT(48933)=(S=12608728,E=12608739,C=204)
	CT(48934)=(S=12608740,E=12608741,C=189)
	CT(48935)=(S=12608742,E=12608758,C=204)
	CT(48936)=(S=12608759,E=12608760,C=189)
	CT(48937)=(S=12608761,E=12608762,C=204)
	CT(48938)=(S=12608763,E=12608765,C=189)
	CT(48939)=(S=12608766,E=12608766,C=83)
	CT(48940)=(S=12608767,E=12608767,C=29)
	CT(48941)=(S=12608768,E=12608768,C=78)
	CT(48942)=(S=12608769,E=12608769,C=25)
	CT(48943)=(S=12608770,E=12608771,C=29)
	CT(48944)=(S=12608772,E=12608772,C=18)
	CT(48945)=(S=12608773,E=12608775,C=29)
	CT(48946)=(S=12608776,E=12608776,C=25)
	CT(48947)=(S=12608777,E=12608778,C=29)
	CT(48948)=(S=12608779,E=12608779,C=14)
	CT(48949)=(S=12608780,E=12608783,C=29)
	CT(48950)=(S=12608784,E=12608784,C=73)
	CT(48951)=(S=12608785,E=12608795,C=29)
	CT(48952)=(S=12608796,E=12608796,C=28)
	CT(48953)=(S=12608797,E=12608801,C=29)
	CT(48954)=(S=12608802,E=12608802,C=14)
	CT(48955)=(S=12608803,E=12608842,C=29)
	CT(48956)=(S=12608843,E=12608843,C=28)
	CT(48957)=(S=12608844,E=12608848,C=29)
	CT(48958)=(S=12608849,E=12608858,C=36)
	CT(48959)=(S=12608859,E=12608859,C=3)
	CT(48960)=(S=12608860,E=12608878,C=29)
	CT(48961)=(S=12608879,E=12608882,C=36)
	CT(48962)=(S=12608883,E=12608885,C=29)
	CT(48963)=(S=12608886,E=12608886,C=21)
	CT(48964)=(S=12608887,E=12608901,C=29)
	CT(48965)=(S=12608902,E=12608902,C=0)
	CT(48966)=(S=12608903,E=12608904,C=29)
	CT(48967)=(S=12608905,E=12608905,C=14)
	CT(48968)=(S=12608906,E=12608909,C=29)
	CT(48969)=(S=12608910,E=12608910,C=106)
	CT(48970)=(S=12608911,E=12608928,C=29)
	CT(48971)=(S=12608929,E=12608936,C=25)
	CT(48972)=(S=12608937,E=12608937,C=80)
	CT(48973)=(S=12608938,E=12608938,C=49)
	CT(48974)=(S=12608939,E=12608943,C=29)
	CT(48975)=(S=12608944,E=12608944,C=34)
	CT(48976)=(S=12608946,E=12608946,C=29)
	CT(48977)=(S=12608947,E=12608948,C=28)
	CT(48978)=(S=12608949,E=12608950,C=29)
	CT(48979)=(S=12608951,E=12608951,C=0)
	CT(48980)=(S=12608952,E=12608959,C=29)
	CT(48981)=(S=12608960,E=12608960,C=47)
	CT(48982)=(S=12608961,E=12608964,C=29)
	CT(48983)=(S=12608965,E=12608966,C=28)
	CT(48984)=(S=12608967,E=12609019,C=29)
	CT(48985)=(S=12609020,E=12609020,C=49)
	CT(48986)=(S=12609021,E=12609021,C=29)
	CT(48987)=(S=12609022,E=12609022,C=0)
	CT(48988)=(S=12609023,E=12609024,C=29)
	CT(48989)=(S=12609025,E=12609025,C=42)
	CT(48990)=(S=12609026,E=12609029,C=29)
	CT(48991)=(S=12609030,E=12609031,C=31)
	CT(48992)=(S=12609032,E=12609032,C=14)
	CT(48993)=(S=12609033,E=12609033,C=105)
	CT(48994)=(S=12609034,E=12609034,C=29)
	CT(48995)=(S=12609035,E=12609035,C=78)
	CT(48996)=(S=12609036,E=12609040,C=29)
	CT(48997)=(S=12609041,E=12609041,C=42)
	CT(48998)=(S=12609042,E=12609105,C=47)
	CT(48999)=(S=12609106,E=12609106,C=29)
	CT(49000)=(S=12609107,E=12609107,C=3)
	CT(49001)=(S=12609108,E=12609108,C=212)
	CT(49002)=(S=12609109,E=12609112,C=29)
	CT(49003)=(S=12609113,E=12609113,C=28)
	CT(49004)=(S=12609114,E=12609115,C=29)
	CT(49005)=(S=12609116,E=12609116,C=73)
	CT(49006)=(S=12609117,E=12609118,C=29)
	CT(49007)=(S=12609119,E=12609119,C=34)
	CT(49008)=(S=12609120,E=12609169,C=29)
	CT(49009)=(S=12609170,E=12609201,C=28)
	CT(49010)=(S=12609202,E=12609237,C=29)
	CT(49011)=(S=12609238,E=12609238,C=14)
	CT(49012)=(S=12609239,E=12609247,C=29)
	CT(49013)=(S=12609248,E=12609248,C=12)
	CT(49014)=(S=12609249,E=12609249,C=58)
	CT(49015)=(S=12609250,E=12609250,C=29)
	CT(49016)=(S=12609251,E=12609252,C=14)
	CT(49017)=(S=12609253,E=12609253,C=18)
	CT(49018)=(S=12609254,E=12609262,C=29)
	CT(49019)=(S=12609263,E=12609263,C=0)
	CT(49020)=(S=12609264,E=12609273,C=29)
	CT(49021)=(S=12609274,E=12609275,C=0)
	CT(49022)=(S=12609276,E=12609277,C=29)
	CT(49023)=(S=12609278,E=12609279,C=78)
	CT(49024)=(S=12609280,E=12609281,C=29)
	CT(49025)=(S=12609282,E=12609282,C=70)
	CT(49026)=(S=12609283,E=12609286,C=29)
	CT(49027)=(S=12609287,E=12609287,C=12)
	CT(49028)=(S=12609288,E=12609293,C=29)
	CT(49029)=(S=12609294,E=12609294,C=16)
	CT(49030)=(S=12609295,E=12609299,C=29)
	CT(49031)=(S=12609300,E=12609300,C=22)
	CT(49032)=(S=12609301,E=12609302,C=29)
	CT(49033)=(S=12609303,E=12609303,C=28)
	CT(49034)=(S=12609304,E=12609306,C=29)
	CT(49035)=(S=12609307,E=12609307,C=28)
	CT(49036)=(S=12609308,E=12609319,C=14)
	CT(49037)=(S=12609320,E=12609320,C=28)
	CT(49038)=(S=12609321,E=12609364,C=29)
	CT(49039)=(S=12609365,E=12609396,C=47)
	CT(49040)=(S=12609397,E=12609409,C=29)
	CT(49041)=(S=12609410,E=12609410,C=0)
	CT(49042)=(S=12609411,E=12609416,C=29)
	CT(49043)=(S=12609417,E=12609419,C=14)
	CT(49044)=(S=12609420,E=12609426,C=29)
	CT(49045)=(S=12609427,E=12609427,C=21)
	CT(49046)=(S=12609428,E=12609550,C=29)
	CT(49047)=(S=12609551,E=12609551,C=6)
	CT(49048)=(S=12609552,E=12609558,C=29)
	CT(49049)=(S=12609559,E=12609559,C=15)
	CT(49050)=(S=12609560,E=12609563,C=29)
	CT(49051)=(S=12609564,E=12609565,C=14)
	CT(49052)=(S=12609566,E=12609570,C=29)
	CT(49053)=(S=12609571,E=12609573,C=21)
	CT(49054)=(S=12609574,E=12609574,C=78)
	CT(49055)=(S=12609575,E=12609575,C=29)
	CT(49056)=(S=12609577,E=12609577,C=34)
	CT(49057)=(S=12609578,E=12609578,C=29)
	CT(49058)=(S=12609579,E=12609581,C=0)
	CT(49059)=(S=12609582,E=12609583,C=29)
	CT(49060)=(S=12609584,E=12609584,C=21)
	CT(49061)=(S=12609585,E=12609588,C=29)
	CT(49062)=(S=12609589,E=12609589,C=14)
	CT(49063)=(S=12609590,E=12609590,C=29)
	CT(49064)=(S=12609591,E=12609592,C=36)
	CT(49065)=(S=12609593,E=12609594,C=27)
	CT(49066)=(S=12609595,E=12609607,C=29)
	CT(49067)=(S=12609608,E=12609608,C=34)
	CT(49068)=(S=12609609,E=12609612,C=29)
	CT(49069)=(S=12609613,E=12609613,C=28)
	CT(49070)=(S=12609614,E=12609617,C=29)
	CT(49071)=(S=12609618,E=12609618,C=14)
	CT(49072)=(S=12609619,E=12609675,C=29)
	CT(49073)=(S=12609676,E=12609676,C=36)
	CT(49074)=(S=12609677,E=12609677,C=29)
	CT(49075)=(S=12609678,E=12609678,C=36)
	CT(49076)=(S=12609679,E=12609682,C=29)
	CT(49077)=(S=12609683,E=12609683,C=22)
	CT(49078)=(S=12609684,E=12609689,C=29)
	CT(49079)=(S=12609690,E=12609690,C=63)
	CT(49080)=(S=12609691,E=12609691,C=12)
	CT(49081)=(S=12609692,E=12609702,C=29)
	CT(49082)=(S=12609703,E=12609704,C=49)
	CT(49083)=(S=12609705,E=12609707,C=29)
	CT(49084)=(S=12609708,E=12609708,C=2)
	CT(49085)=(S=12609709,E=12609767,C=29)
	CT(49086)=(S=12609768,E=12609769,C=2)
	CT(49087)=(S=12609770,E=12609773,C=29)
	CT(49088)=(S=12609774,E=12609774,C=14)
	CT(49089)=(S=12609775,E=12609778,C=29)
	CT(49090)=(S=12609779,E=12609779,C=78)
	CT(49091)=(S=12609780,E=12609780,C=29)
	CT(49092)=(S=12609781,E=12609781,C=12)
	CT(49093)=(S=12609782,E=12609783,C=2)
	CT(49094)=(S=12609784,E=12609784,C=24)
	CT(49095)=(S=12609785,E=12609786,C=29)
	CT(49096)=(S=12609787,E=12609787,C=34)
	CT(49097)=(S=12609789,E=12609801,C=29)
	CT(49098)=(S=12609802,E=12609802,C=73)
	CT(49099)=(S=12609803,E=12609866,C=29)
	CT(49100)=(S=12609867,E=12609867,C=18)
	CT(49101)=(S=12609868,E=12610047,C=29)
	CT(49102)=(S=12610048,E=12610303,C=15)
	CT(49103)=(S=12610304,E=12610304,C=29)
	CT(49104)=(S=12610306,E=12610306,C=14)
	CT(49105)=(S=12610307,E=12610307,C=29)
	CT(49106)=(S=12610308,E=12610309,C=14)
	CT(49107)=(S=12610311,E=12610312,C=29)
	CT(49108)=(S=12610313,E=12610313,C=0)
	CT(49109)=(S=12610315,E=12610317,C=14)
	CT(49110)=(S=12610318,E=12610354,C=29)
	CT(49111)=(S=12610355,E=12610404,C=15)
	CT(49112)=(S=12610405,E=12610405,C=0)
	CT(49113)=(S=12610406,E=12610407,C=29)
	CT(49114)=(S=12610408,E=12610408,C=216)
	CT(49115)=(S=12610409,E=12610413,C=29)
	CT(49116)=(S=12610414,E=12610414,C=63)
	CT(49117)=(S=12610415,E=12610415,C=80)
	CT(49118)=(S=12610417,E=12610417,C=73)
	CT(49119)=(S=12610418,E=12610418,C=27)
	CT(49120)=(S=12610419,E=12610425,C=29)
	CT(49121)=(S=12610426,E=12610426,C=21)
	CT(49122)=(S=12610427,E=12610427,C=28)
	CT(49123)=(S=12610428,E=12610429,C=42)
	CT(49124)=(S=12610430,E=12610433,C=14)
	CT(49125)=(S=12610434,E=12610434,C=29)
	CT(49126)=(S=12610435,E=12610435,C=78)
	CT(49127)=(S=12610436,E=12610436,C=28)
	CT(49128)=(S=12610437,E=12610437,C=78)
	CT(49129)=(S=12610438,E=12610447,C=29)
	CT(49130)=(S=12610448,E=12610448,C=78)
	CT(49131)=(S=12610449,E=12610471,C=29)
	CT(49132)=(S=12610472,E=12610472,C=14)
	CT(49133)=(S=12610473,E=12610474,C=29)
	CT(49134)=(S=12610475,E=12610476,C=73)
	CT(49135)=(S=12610477,E=12610477,C=29)
	CT(49136)=(S=12610478,E=12610478,C=14)
	CT(49137)=(S=12610479,E=12610480,C=29)
	CT(49138)=(S=12610481,E=12610481,C=22)
	CT(49139)=(S=12610482,E=12610482,C=14)
	CT(49140)=(S=12610483,E=12610490,C=29)
	CT(49141)=(S=12610491,E=12610491,C=19)
	CT(49142)=(S=12610492,E=12610503,C=29)
	CT(49143)=(S=12610504,E=12610535,C=47)
	CT(49144)=(S=12610536,E=12610537,C=42)
	CT(49145)=(S=12610538,E=12610538,C=29)
	CT(49146)=(S=12610539,E=12610540,C=28)
	CT(49147)=(S=12610541,E=12610582,C=29)
	CT(49148)=(S=12610583,E=12610607,C=13)
	CT(49149)=(S=12610608,E=12610609,C=28)
	CT(49150)=(S=12610610,E=12610610,C=29)
	CT(49151)=(S=12610611,E=12610652,C=28)
	CT(49152)=(S=12610653,E=12610658,C=29)
	CT(49153)=(S=12610659,E=12610659,C=0)
	CT(49154)=(S=12610660,E=12610661,C=27)
	CT(49155)=(S=12610662,E=12610666,C=29)
	CT(49156)=(S=12610667,E=12610668,C=19)
	CT(49157)=(S=12610669,E=12610672,C=29)
	CT(49158)=(S=12610674,E=12610674,C=22)
	CT(49159)=(S=12610675,E=12610679,C=12)
	CT(49160)=(S=12610680,E=12610680,C=14)
	CT(49161)=(S=12610681,E=12610681,C=42)
	CT(49162)=(S=12610682,E=12610684,C=29)
	CT(49163)=(S=12610685,E=12610685,C=71)
	CT(49164)=(S=12610686,E=12610686,C=33)
	CT(49165)=(S=12610687,E=12610687,C=29)
	CT(49166)=(S=12610688,E=12610689,C=33)
	CT(49167)=(S=12610690,E=12610734,C=71)
	CT(49168)=(S=12610735,E=12610754,C=29)
	CT(49169)=(S=12610755,E=12610774,C=19)
	CT(49170)=(S=12610775,E=12610789,C=29)
	CT(49171)=(S=12610790,E=12610790,C=3)
	CT(49172)=(S=12610791,E=12610793,C=29)
	CT(49173)=(S=12610794,E=12610794,C=34)
	CT(49174)=(S=12610795,E=12610797,C=29)
	CT(49175)=(S=12610798,E=12610798,C=42)
	CT(49176)=(S=12610799,E=12610815,C=29)
	CT(49177)=(S=12610816,E=12610860,C=28)
	CT(49178)=(S=12610861,E=12610861,C=29)
	CT(49179)=(S=12610862,E=12610889,C=28)
	CT(49180)=(S=12610891,E=12610891,C=29)
	CT(49181)=(S=12610892,E=12610906,C=28)
	CT(49182)=(S=12610908,E=12610909,C=29)
	CT(49183)=(S=12610910,E=12610912,C=28)
	CT(49184)=(S=12610915,E=12610915,C=29)
	CT(49185)=(S=12610917,E=12610918,C=28)
	CT(49186)=(S=12610919,E=12610920,C=29)
	CT(49187)=(S=12610921,E=12610934,C=28)
	CT(49188)=(S=12610936,E=12610936,C=29)
	CT(49189)=(S=12610937,E=12610953,C=28)
	CT(49190)=(S=12610956,E=12610958,C=12)
	CT(49191)=(S=12610959,E=12610959,C=28)
	CT(49192)=(S=12610960,E=12610964,C=12)
	CT(49193)=(S=12610966,E=12611057,C=28)
	CT(49194)=(S=12611058,E=12611058,C=105)
	CT(49195)=(S=12611062,E=12611071,C=28)
	CT(49196)=(S=12611072,E=12611243,C=29)
	CT(49197)=(S=12611244,E=12611247,C=78)
	CT(49198)=(S=12611248,E=12611359,C=29)
	CT(49199)=(S=12611360,E=12611360,C=0)
	CT(49200)=(S=12611361,E=12611361,C=27)
	CT(49201)=(S=12611362,E=12611366,C=29)
	CT(49202)=(S=12611367,E=12611367,C=63)
	CT(49203)=(S=12611368,E=12611371,C=29)
	CT(49204)=(S=12611372,E=12611372,C=33)
	CT(49205)=(S=12611373,E=12611374,C=29)
	CT(49206)=(S=12611375,E=12611375,C=28)
	CT(49207)=(S=12611376,E=12611376,C=34)
	CT(49208)=(S=12611377,E=12611415,C=29)
	CT(49209)=(S=12611416,E=12611416,C=14)
	CT(49210)=(S=12611417,E=12611428,C=29)
	CT(49211)=(S=12611429,E=12611429,C=21)
	CT(49212)=(S=12611430,E=12611430,C=73)
	CT(49213)=(S=12611431,E=12611431,C=34)
	CT(49214)=(S=12611432,E=12611432,C=42)
	CT(49215)=(S=12611433,E=12611433,C=0)
	CT(49216)=(S=12611434,E=12611451,C=29)
	CT(49217)=(S=12611452,E=12611453,C=27)
	CT(49218)=(S=12611454,E=12611454,C=29)
	CT(49219)=(S=12611455,E=12611455,C=28)
	CT(49220)=(S=12611456,E=12611556,C=29)
	CT(49221)=(S=12611557,E=12611558,C=189)
	CT(49222)=(S=12611559,E=12611579,C=29)
	CT(49223)=(S=12611580,E=12611580,C=42)
	CT(49224)=(S=12611581,E=12611612,C=29)
	CT(49225)=(S=12611613,E=12611613,C=78)
	CT(49226)=(S=12611614,E=12611616,C=21)
	CT(49227)=(S=12611617,E=12611628,C=29)
	CT(49228)=(S=12611629,E=12611629,C=21)
	CT(49229)=(S=12611630,E=12611632,C=29)
	CT(49230)=(S=12611633,E=12611633,C=14)
	CT(49231)=(S=12611634,E=12611644,C=29)
	CT(49232)=(S=12611645,E=12611645,C=42)
	CT(49233)=(S=12611646,E=12611653,C=29)
	CT(49234)=(S=12611654,E=12611663,C=25)
	CT(49235)=(S=12611664,E=12611681,C=29)
	CT(49236)=(S=12611682,E=12611682,C=14)
	CT(49237)=(S=12611683,E=12611683,C=24)
	CT(49238)=(S=12611684,E=12611684,C=14)
	CT(49239)=(S=12611685,E=12611787,C=29)
	CT(49240)=(S=12611788,E=12611788,C=47)
	CT(49241)=(S=12611789,E=12611789,C=29)
	CT(49242)=(S=12611790,E=12611790,C=19)
	CT(49243)=(S=12611791,E=12611791,C=29)
	CT(49244)=(S=12611792,E=12611792,C=28)
	CT(49245)=(S=12611793,E=12611796,C=29)
	CT(49246)=(S=12611797,E=12611797,C=28)
	CT(49247)=(S=12611798,E=12611798,C=34)
	CT(49248)=(S=12611799,E=12611799,C=0)
	CT(49249)=(S=12611800,E=12611830,C=29)
	CT(49250)=(S=12611831,E=12611831,C=25)
	CT(49251)=(S=12611832,E=12611837,C=29)
	CT(49252)=(S=12611838,E=12611838,C=34)
	CT(49253)=(S=12611839,E=12611839,C=29)
	CT(49254)=(S=12611840,E=12612095,C=36)
	CT(49255)=(S=12612096,E=12613375,C=17)
	CT(49256)=(S=12613376,E=12613376,C=78)
	CT(49257)=(S=12613377,E=12613519,C=29)
	CT(49258)=(S=12613520,E=12613535,C=78)
	CT(49259)=(S=12613536,E=12613631,C=29)
	CT(49260)=(S=12613632,E=12613632,C=78)
	CT(49261)=(S=12613633,E=12613887,C=29)
	CT(49262)=(S=12613888,E=12614143,C=19)
	CT(49263)=(S=12614144,E=12614144,C=29)
	CT(49264)=(S=12614145,E=12614274,C=14)
	CT(49265)=(S=12614275,E=12614284,C=7)
	CT(49266)=(S=12614285,E=12614290,C=14)
	CT(49267)=(S=12614291,E=12614294,C=29)
	CT(49268)=(S=12614295,E=12614314,C=14)
	CT(49269)=(S=12614315,E=12614315,C=73)
	CT(49270)=(S=12614316,E=12614319,C=29)
	CT(49271)=(S=12614320,E=12614320,C=0)
	CT(49272)=(S=12614321,E=12614323,C=29)
	CT(49273)=(S=12614324,E=12614324,C=73)
	CT(49274)=(S=12614325,E=12614349,C=29)
	CT(49275)=(S=12614350,E=12614350,C=78)
	CT(49276)=(S=12614351,E=12614357,C=29)
	CT(49277)=(S=12614358,E=12614358,C=14)
	CT(49278)=(S=12614359,E=12614359,C=29)
	CT(49279)=(S=12614360,E=12614360,C=63)
	CT(49280)=(S=12614361,E=12614378,C=14)
	CT(49281)=(S=12614379,E=12614381,C=29)
	CT(49282)=(S=12614382,E=12614386,C=21)
	CT(49283)=(S=12614387,E=12614665,C=29)
	CT(49284)=(S=12614666,E=12614670,C=0)
	CT(49285)=(S=12614671,E=12614679,C=29)
	CT(49286)=(S=12614681,E=12614684,C=28)
	CT(49287)=(S=12614685,E=12614685,C=29)
	CT(49288)=(S=12614686,E=12614686,C=78)
	CT(49289)=(S=12614687,E=12614687,C=29)
	CT(49290)=(S=12614688,E=12614688,C=42)
	CT(49291)=(S=12614689,E=12614694,C=29)
	CT(49292)=(S=12614695,E=12614695,C=49)
	CT(49293)=(S=12614696,E=12614701,C=29)
	CT(49294)=(S=12614702,E=12614702,C=14)
	CT(49295)=(S=12614703,E=12614767,C=29)
	CT(49296)=(S=12614768,E=12614768,C=28)
	CT(49297)=(S=12614769,E=12614770,C=29)
	CT(49298)=(S=12614771,E=12614771,C=28)
	CT(49299)=(S=12614772,E=12614772,C=58)
	CT(49300)=(S=12614773,E=12614773,C=0)
	CT(49301)=(S=12614774,E=12614807,C=29)
	CT(49302)=(S=12614808,E=12614808,C=0)
	CT(49303)=(S=12614809,E=12614809,C=29)
	CT(49304)=(S=12614810,E=12614810,C=1)
	CT(49305)=(S=12614811,E=12614811,C=42)
	CT(49306)=(S=12614812,E=12614825,C=29)
	CT(49307)=(S=12614826,E=12614875,C=18)
	CT(49308)=(S=12614876,E=12614890,C=29)
	CT(49309)=(S=12614891,E=12614891,C=28)
	CT(49310)=(S=12614892,E=12614892,C=29)
	CT(49311)=(S=12614893,E=12614901,C=28)
	CT(49312)=(S=12614902,E=12614902,C=29)
	CT(49313)=(S=12614903,E=12614904,C=28)
	CT(49314)=(S=12614905,E=12614905,C=29)
	CT(49315)=(S=12614906,E=12615167,C=28)
	CT(49316)=(S=12615168,E=12615168,C=29)
	CT(49317)=(S=12615169,E=12615232,C=47)
	CT(49318)=(S=12615233,E=12615936,C=29)
	CT(49319)=(S=12615937,E=12615997,C=28)
	CT(49320)=(S=12615998,E=12616015,C=29)
	CT(49321)=(S=12616016,E=12616016,C=42)
	CT(49322)=(S=12616017,E=12616022,C=29)
	CT(49323)=(S=12616023,E=12616023,C=37)
	CT(49324)=(S=12616024,E=12616033,C=29)
	CT(49325)=(S=12616034,E=12616034,C=28)
	CT(49326)=(S=12616035,E=12616191,C=29)
	CT(49327)=(S=12616192,E=12616447,C=47)
	CT(49328)=(S=12616448,E=12616460,C=29)
	CT(49329)=(S=12616461,E=12616461,C=0)
	CT(49330)=(S=12616462,E=12616467,C=29)
	CT(49331)=(S=12616468,E=12616468,C=28)
	CT(49332)=(S=12616469,E=12616472,C=29)
	CT(49333)=(S=12616473,E=12616474,C=19)
	CT(49334)=(S=12616475,E=12616478,C=0)
	CT(49335)=(S=12616479,E=12616479,C=29)
	CT(49336)=(S=12616480,E=12616487,C=175)
	CT(49337)=(S=12616488,E=12616491,C=29)
	CT(49338)=(S=12616492,E=12616492,C=78)
	CT(49339)=(S=12616493,E=12616526,C=29)
	CT(49340)=(S=12616527,E=12616527,C=14)
	CT(49341)=(S=12616528,E=12616536,C=29)
	CT(49342)=(S=12616537,E=12616537,C=14)
	CT(49343)=(S=12616538,E=12616540,C=0)
	CT(49344)=(S=12616541,E=12616541,C=29)
	CT(49345)=(S=12616542,E=12616542,C=78)
	CT(49346)=(S=12616543,E=12616543,C=29)
	CT(49347)=(S=12616544,E=12616544,C=47)
	CT(49348)=(S=12616545,E=12616545,C=78)
	CT(49349)=(S=12616546,E=12616555,C=29)
	CT(49350)=(S=12616556,E=12616556,C=14)
	CT(49351)=(S=12616557,E=12616557,C=78)
	CT(49352)=(S=12616558,E=12616579,C=29)
	CT(49353)=(S=12616580,E=12616580,C=49)
	CT(49354)=(S=12616581,E=12616583,C=29)
	CT(49355)=(S=12616584,E=12616587,C=78)
	CT(49356)=(S=12616591,E=12616680,C=29)
	CT(49357)=(S=12616682,E=12616682,C=78)
	CT(49358)=(S=12616683,E=12616698,C=29)
	CT(49359)=(S=12616699,E=12616700,C=0)
	CT(49360)=(S=12616701,E=12616703,C=29)
	CT(49361)=(S=12616704,E=12616704,C=78)
	CT(49362)=(S=12616705,E=12616712,C=29)
	CT(49363)=(S=12616713,E=12616713,C=14)
	CT(49364)=(S=12616714,E=12616714,C=15)
	CT(49365)=(S=12616715,E=12616715,C=27)
	CT(49366)=(S=12616716,E=12616718,C=29)
	CT(49367)=(S=12616719,E=12616719,C=6)
	CT(49368)=(S=12616720,E=12616722,C=29)
	CT(49369)=(S=12616723,E=12616723,C=78)
	CT(49370)=(S=12616724,E=12616734,C=29)
	CT(49371)=(S=12616735,E=12616735,C=2)
	CT(49372)=(S=12616736,E=12616737,C=29)
	CT(49373)=(S=12616738,E=12616738,C=15)
	CT(49374)=(S=12616739,E=12616739,C=189)
	CT(49375)=(S=12616740,E=12616744,C=29)
	CT(49376)=(S=12616745,E=12616745,C=0)
	CT(49377)=(S=12616746,E=12616756,C=29)
	CT(49378)=(S=12616757,E=12616757,C=21)
	CT(49379)=(S=12616758,E=12616758,C=29)
	CT(49380)=(S=12616759,E=12616759,C=21)
	CT(49381)=(S=12616760,E=12616763,C=29)
	CT(49382)=(S=12616764,E=12616764,C=78)
	CT(49383)=(S=12616765,E=12616796,C=29)
	CT(49384)=(S=12616797,E=12616797,C=0)
	CT(49385)=(S=12616798,E=12616802,C=29)
	CT(49386)=(S=12616803,E=12616803,C=14)
	CT(49387)=(S=12616804,E=12616942,C=29)
	CT(49388)=(S=12616943,E=12616943,C=14)
	CT(49389)=(S=12616944,E=12616944,C=29)
	CT(49390)=(S=12616945,E=12616945,C=78)
	CT(49391)=(S=12616946,E=12616947,C=29)
	CT(49392)=(S=12616948,E=12616948,C=17)
	CT(49393)=(S=12616949,E=12616949,C=36)
	CT(49394)=(S=12616950,E=12616950,C=29)
	CT(49395)=(S=12616951,E=12616955,C=6)
	CT(49396)=(S=12616956,E=12616956,C=34)
	CT(49397)=(S=12616957,E=12616957,C=0)
	CT(49398)=(S=12616958,E=12616959,C=29)
	CT(49399)=(S=12616960,E=12616961,C=78)
	CT(49400)=(S=12616962,E=12616973,C=29)
	CT(49401)=(S=12616975,E=12616975,C=21)
	CT(49402)=(S=12616976,E=12616980,C=29)
	CT(49403)=(S=12616981,E=12616981,C=0)
	CT(49404)=(S=12616982,E=12616987,C=29)
	CT(49405)=(S=12616988,E=12616988,C=15)
	CT(49406)=(S=12616989,E=12616990,C=29)
	CT(49407)=(S=12616991,E=12616991,C=73)
	CT(49408)=(S=12616992,E=12616992,C=27)
	CT(49409)=(S=12616993,E=12616995,C=29)
	CT(49410)=(S=12616996,E=12616996,C=17)
	CT(49411)=(S=12616997,E=12617000,C=29)
	CT(49412)=(S=12617001,E=12617001,C=0)
	CT(49413)=(S=12617002,E=12617004,C=29)
	CT(49414)=(S=12617005,E=12617005,C=78)
	CT(49415)=(S=12617006,E=12617012,C=29)
	CT(49416)=(S=12617013,E=12617013,C=36)
	CT(49417)=(S=12617014,E=12617015,C=21)
	CT(49418)=(S=12617016,E=12617017,C=29)
	CT(49419)=(S=12617018,E=12617018,C=14)
	CT(49420)=(S=12617019,E=12617023,C=29)
	CT(49421)=(S=12617024,E=12617024,C=27)
	CT(49422)=(S=12617025,E=12617025,C=29)
	CT(49423)=(S=12617027,E=12617027,C=7)
	CT(49424)=(S=12617028,E=12617067,C=29)
	CT(49425)=(S=12617068,E=12617068,C=21)
	CT(49426)=(S=12617069,E=12617069,C=42)
	CT(49427)=(S=12617070,E=12617071,C=47)
	CT(49428)=(S=12617072,E=12617080,C=29)
	CT(49429)=(S=12617081,E=12617081,C=19)
	CT(49430)=(S=12617082,E=12617090,C=29)
	CT(49431)=(S=12617091,E=12617091,C=14)
	CT(49432)=(S=12617092,E=12617203,C=29)
	CT(49433)=(S=12617204,E=12617204,C=14)
	CT(49434)=(S=12617205,E=12617209,C=29)
	CT(49435)=(S=12617210,E=12617210,C=105)
	CT(49436)=(S=12617211,E=12617214,C=29)
	CT(49437)=(S=12617215,E=12617215,C=78)
	CT(49438)=(S=12617216,E=12617471,C=12)
	CT(49439)=(S=12617472,E=12617478,C=29)
	CT(49440)=(S=12617479,E=12617479,C=28)
	CT(49441)=(S=12617480,E=12617509,C=15)
	CT(49442)=(S=12617510,E=12617511,C=29)
	CT(49443)=(S=12617512,E=12617515,C=78)
	CT(49444)=(S=12617516,E=12617517,C=29)
	CT(49445)=(S=12617518,E=12617518,C=27)
	CT(49446)=(S=12617519,E=12617519,C=29)
	CT(49447)=(S=12617520,E=12617520,C=78)
	CT(49448)=(S=12617521,E=12617522,C=29)
	CT(49449)=(S=12617523,E=12617523,C=28)
	CT(49450)=(S=12617524,E=12617525,C=14)
	CT(49451)=(S=12617526,E=12617537,C=29)
	CT(49452)=(S=12617538,E=12617538,C=14)
	CT(49453)=(S=12617539,E=12617539,C=29)
	CT(49454)=(S=12617540,E=12617540,C=14)
	CT(49455)=(S=12617541,E=12617548,C=29)
	CT(49456)=(S=12617549,E=12617549,C=78)
	CT(49457)=(S=12617550,E=12617553,C=29)
	CT(49458)=(S=12617554,E=12617554,C=36)
	CT(49459)=(S=12617555,E=12617555,C=78)
	CT(49460)=(S=12617556,E=12617560,C=29)
	CT(49461)=(S=12617561,E=12617564,C=2)
	CT(49462)=(S=12617565,E=12617565,C=29)
	CT(49463)=(S=12617566,E=12617580,C=2)
	CT(49464)=(S=12617581,E=12617581,C=29)
	CT(49465)=(S=12617582,E=12617583,C=80)
	CT(49466)=(S=12617584,E=12617594,C=29)
	CT(49467)=(S=12617595,E=12617595,C=78)
	CT(49468)=(S=12617596,E=12617600,C=29)
	CT(49469)=(S=12617601,E=12617601,C=21)
	CT(49470)=(S=12617602,E=12617604,C=29)
	CT(49471)=(S=12617605,E=12617605,C=42)
	CT(49472)=(S=12617606,E=12617614,C=29)
	CT(49473)=(S=12617615,E=12617615,C=27)
	CT(49474)=(S=12617616,E=12617616,C=29)
	CT(49475)=(S=12617617,E=12617636,C=34)
	CT(49476)=(S=12617637,E=12617637,C=15)
	CT(49477)=(S=12617638,E=12617638,C=22)
	CT(49478)=(S=12617639,E=12617640,C=49)
	CT(49479)=(S=12617641,E=12617646,C=29)
	CT(49480)=(S=12617647,E=12617647,C=12)
	CT(49481)=(S=12617648,E=12617658,C=29)
	CT(49482)=(S=12617659,E=12617659,C=21)
	CT(49483)=(S=12617660,E=12617678,C=29)
	CT(49484)=(S=12617679,E=12617680,C=0)
	CT(49485)=(S=12617681,E=12617690,C=29)
	CT(49486)=(S=12617691,E=12617691,C=21)
	CT(49487)=(S=12617692,E=12617693,C=29)
	CT(49488)=(S=12617694,E=12617694,C=2)
	CT(49489)=(S=12617695,E=12617696,C=29)
	CT(49490)=(S=12617697,E=12617697,C=63)
	CT(49491)=(S=12617698,E=12617701,C=29)
	CT(49492)=(S=12617702,E=12617702,C=78)
	CT(49493)=(S=12617703,E=12617706,C=14)
	CT(49494)=(S=12617707,E=12617724,C=29)
	CT(49495)=(S=12617725,E=12617725,C=27)
	CT(49496)=(S=12617726,E=12617726,C=58)
	CT(49497)=(S=12617727,E=12617734,C=29)
	CT(49498)=(S=12617735,E=12617735,C=72)
	CT(49499)=(S=12617736,E=12617736,C=29)
	CT(49500)=(S=12617737,E=12617737,C=47)
	CT(49501)=(S=12617738,E=12617745,C=29)
	CT(49502)=(S=12617746,E=12617746,C=14)
	CT(49503)=(S=12617747,E=12617747,C=49)
	CT(49504)=(S=12617748,E=12617750,C=29)
	CT(49505)=(S=12617751,E=12617751,C=36)
	CT(49506)=(S=12617752,E=12617754,C=29)
	CT(49507)=(S=12617755,E=12617755,C=78)
	CT(49508)=(S=12617756,E=12617756,C=29)
	CT(49509)=(S=12617757,E=12617757,C=14)
	CT(49510)=(S=12617758,E=12617758,C=12)
	CT(49511)=(S=12617759,E=12617759,C=21)
	CT(49512)=(S=12617760,E=12617766,C=29)
	CT(49513)=(S=12617767,E=12617767,C=0)
	CT(49514)=(S=12617768,E=12617768,C=14)
	CT(49515)=(S=12617769,E=12617769,C=34)
	CT(49516)=(S=12617770,E=12617775,C=216)
	CT(49517)=(S=12617776,E=12617776,C=29)
	CT(49518)=(S=12617777,E=12617777,C=21)
	CT(49519)=(S=12617778,E=12617778,C=29)
	CT(49520)=(S=12617779,E=12617779,C=36)
	CT(49521)=(S=12617780,E=12617780,C=21)
	CT(49522)=(S=12617781,E=12617781,C=78)
	CT(49523)=(S=12617782,E=12617782,C=12)
	CT(49524)=(S=12617783,E=12617785,C=139)
	CT(49525)=(S=12617786,E=12617786,C=12)
	CT(49526)=(S=12617787,E=12617787,C=164)
	CT(49527)=(S=12617788,E=12617788,C=29)
	CT(49528)=(S=12617789,E=12617789,C=18)
	CT(49529)=(S=12617790,E=12617798,C=29)
	CT(49530)=(S=12617799,E=12617830,C=47)
	CT(49531)=(S=12617831,E=12617841,C=29)
	CT(49532)=(S=12617842,E=12617843,C=78)
	CT(49533)=(S=12617844,E=12617881,C=29)
	CT(49534)=(S=12617882,E=12617883,C=14)
	CT(49535)=(S=12617884,E=12617884,C=34)
	CT(49536)=(S=12617885,E=12618240,C=29)
	CT(49537)=(S=12618241,E=12618248,C=14)
	CT(49538)=(S=12618249,E=12618325,C=29)
	CT(49539)=(S=12618326,E=12618326,C=21)
	CT(49540)=(S=12618327,E=12618339,C=29)
	CT(49541)=(S=12618340,E=12618340,C=0)
	CT(49542)=(S=12618341,E=12618344,C=29)
	CT(49543)=(S=12618345,E=12618348,C=27)
	CT(49544)=(S=12618349,E=12618357,C=19)
	CT(49545)=(S=12618366,E=12618394,C=29)
	CT(49546)=(S=12618395,E=12618395,C=27)
	CT(49547)=(S=12618396,E=12618398,C=14)
	CT(49548)=(S=12618399,E=12618422,C=29)
	CT(49549)=(S=12618423,E=12618423,C=15)
	CT(49550)=(S=12618424,E=12618424,C=29)
	CT(49551)=(S=12618425,E=12618426,C=78)
	CT(49552)=(S=12618427,E=12618431,C=29)
	CT(49553)=(S=12618432,E=12618432,C=14)
	CT(49554)=(S=12618433,E=12618443,C=29)
	CT(49555)=(S=12618444,E=12618444,C=21)
	CT(49556)=(S=12618445,E=12618467,C=29)
	CT(49557)=(S=12618468,E=12618468,C=42)
	CT(49558)=(S=12618469,E=12618469,C=14)
	CT(49559)=(S=12618470,E=12618472,C=34)
	CT(49560)=(S=12618473,E=12618487,C=29)
	CT(49561)=(S=12618488,E=12618488,C=14)
	CT(49562)=(S=12618489,E=12618490,C=29)
	CT(49563)=(S=12618491,E=12618491,C=73)
	CT(49564)=(S=12618492,E=12618496,C=29)
	CT(49565)=(S=12618497,E=12618498,C=78)
	CT(49566)=(S=12618499,E=12618499,C=29)
	CT(49567)=(S=12618500,E=12618500,C=78)
	CT(49568)=(S=12618501,E=12618501,C=29)
	CT(49569)=(S=12618502,E=12618513,C=78)
	CT(49570)=(S=12618514,E=12618514,C=29)
	CT(49571)=(S=12618515,E=12618515,C=78)
	CT(49572)=(S=12618516,E=12618516,C=29)
	CT(49573)=(S=12618517,E=12618557,C=78)
	CT(49574)=(S=12618558,E=12618558,C=29)
	CT(49575)=(S=12618559,E=12618573,C=78)
	CT(49576)=(S=12618574,E=12618574,C=12)
	CT(49577)=(S=12618575,E=12618613,C=78)
	CT(49578)=(S=12618614,E=12618614,C=29)
	CT(49579)=(S=12618615,E=12618623,C=78)
	CT(49580)=(S=12618624,E=12618624,C=29)
	CT(49581)=(S=12618625,E=12618626,C=78)
	CT(49582)=(S=12618627,E=12618633,C=29)
	CT(49583)=(S=12618634,E=12618634,C=78)
	CT(49584)=(S=12618635,E=12618635,C=29)
	CT(49585)=(S=12618636,E=12618642,C=78)
	CT(49586)=(S=12618643,E=12618643,C=29)
	CT(49587)=(S=12618644,E=12618649,C=78)
	CT(49588)=(S=12618650,E=12618651,C=29)
	CT(49589)=(S=12618652,E=12618684,C=78)
	CT(49590)=(S=12618685,E=12618685,C=29)
	CT(49591)=(S=12618686,E=12618687,C=78)
	CT(49592)=(S=12618688,E=12618688,C=185)
	CT(49593)=(S=12618689,E=12618700,C=78)
	CT(49594)=(S=12618701,E=12618701,C=29)
	CT(49595)=(S=12618702,E=12618750,C=78)
	CT(49596)=(S=12618751,E=12618752,C=29)
	CT(49597)=(S=12619850,E=12619853,C=12)
	CT(49598)=(S=12620263,E=12620404,C=29)
	CT(49599)=(S=12620405,E=12620405,C=24)
	CT(49600)=(S=12620406,E=12620406,C=29)
	CT(49601)=(S=12620407,E=12620407,C=12)
	CT(49602)=(S=12620408,E=12620410,C=29)
	CT(49603)=(S=12620411,E=12620411,C=14)
	CT(49604)=(S=12620412,E=12620414,C=25)
	CT(49605)=(S=12620415,E=12620419,C=29)
	CT(49606)=(S=12620420,E=12620420,C=42)
	CT(49607)=(S=12620421,E=12620421,C=73)
	CT(49608)=(S=12620422,E=12620423,C=50)
	CT(49609)=(S=12620424,E=12620425,C=14)
	CT(49610)=(S=12620428,E=12620429,C=18)
	CT(49611)=(S=12620430,E=12620437,C=29)
	CT(49612)=(S=12620438,E=12620438,C=73)
	CT(49613)=(S=12620439,E=12620439,C=29)
	CT(49614)=(S=12620440,E=12620440,C=47)
	CT(49615)=(S=12620441,E=12620443,C=29)
	CT(49616)=(S=12620444,E=12620444,C=78)
	CT(49617)=(S=12620445,E=12620445,C=189)
	CT(49618)=(S=12620446,E=12620450,C=29)
	CT(49619)=(S=12620451,E=12620458,C=27)
	CT(49620)=(S=12620459,E=12620459,C=29)
	CT(49621)=(S=12620460,E=12620460,C=25)
	CT(49622)=(S=12620461,E=12620469,C=15)
	CT(49623)=(S=12620470,E=12620470,C=12)
	CT(49624)=(S=12620471,E=12620471,C=29)
	CT(49625)=(S=12620472,E=12620472,C=2)
	CT(49626)=(S=12620473,E=12620473,C=28)
	CT(49627)=(S=12620474,E=12620474,C=29)
	CT(49628)=(S=12620475,E=12620475,C=25)
	CT(49629)=(S=12620476,E=12620480,C=29)
	CT(49630)=(S=12620481,E=12620481,C=27)
	CT(49631)=(S=12620482,E=12620487,C=29)
	CT(49632)=(S=12620488,E=12620488,C=0)
	CT(49633)=(S=12620489,E=12620491,C=29)
	CT(49634)=(S=12620492,E=12620492,C=33)
	CT(49635)=(S=12620493,E=12620505,C=29)
	CT(49636)=(S=12620506,E=12620507,C=73)
	CT(49637)=(S=12620508,E=12620514,C=29)
	CT(49638)=(S=12620515,E=12620515,C=27)
	CT(49639)=(S=12620516,E=12620516,C=14)
	CT(49640)=(S=12620517,E=12620517,C=189)
	CT(49641)=(S=12620518,E=12620519,C=78)
	CT(49642)=(S=12620521,E=12620522,C=12)
	CT(49643)=(S=12620523,E=12620524,C=29)
	CT(49644)=(S=12620525,E=12620525,C=205)
	CT(49645)=(S=12620526,E=12620527,C=27)
	CT(49646)=(S=12620528,E=12620529,C=29)
	CT(49647)=(S=12620530,E=12620530,C=15)
	CT(49648)=(S=12620531,E=12620538,C=29)
	CT(49649)=(S=12620539,E=12620541,C=0)
	CT(49650)=(S=12620542,E=12620566,C=29)
	CT(49651)=(S=12620567,E=12620567,C=14)
	CT(49652)=(S=12620568,E=12620577,C=29)
	CT(49653)=(S=12620578,E=12620578,C=48)
	CT(49654)=(S=12620579,E=12620579,C=29)
	CT(49655)=(S=12620580,E=12620580,C=14)
	CT(49656)=(S=12620581,E=12620585,C=29)
	CT(49657)=(S=12620586,E=12620586,C=58)
	CT(49658)=(S=12620587,E=12620619,C=29)
	CT(49659)=(S=12620620,E=12620622,C=21)
	CT(49660)=(S=12620623,E=12620623,C=12)
	CT(49661)=(S=12620624,E=12620657,C=29)
	CT(49662)=(S=12620658,E=12620658,C=0)
	CT(49663)=(S=12620659,E=12620684,C=29)
	CT(49664)=(S=12620685,E=12620685,C=58)
	CT(49665)=(S=12620686,E=12620693,C=29)
	CT(49666)=(S=12620694,E=12620694,C=14)
	CT(49667)=(S=12620698,E=12620698,C=29)
	CT(49668)=(S=12620699,E=12620699,C=21)
	CT(49669)=(S=12620700,E=12620753,C=29)
	CT(49670)=(S=12620754,E=12620754,C=189)
	CT(49671)=(S=12620755,E=12620755,C=29)
	CT(49672)=(S=12620756,E=12620756,C=14)
	CT(49673)=(S=12620757,E=12620757,C=78)
	CT(49674)=(S=12620758,E=12620758,C=29)
	CT(49675)=(S=12620759,E=12620760,C=47)
	CT(49676)=(S=12620761,E=12620761,C=14)
	CT(49677)=(S=12620762,E=12620762,C=189)
	CT(49678)=(S=12620763,E=12620763,C=14)
	CT(49679)=(S=12620764,E=12620771,C=29)
	CT(49680)=(S=12620772,E=12620772,C=14)
	CT(49681)=(S=12620773,E=12620774,C=29)
	CT(49682)=(S=12620775,E=12620775,C=226)
	CT(49683)=(S=12620776,E=12620778,C=29)
	CT(49684)=(S=12620779,E=12620779,C=0)
	CT(49685)=(S=12620780,E=12620789,C=29)
	CT(49686)=(S=12620790,E=12620790,C=0)
	CT(49687)=(S=12620791,E=12620791,C=15)
	CT(49688)=(S=12620792,E=12620794,C=29)
	CT(49689)=(S=12620795,E=12620795,C=13)
	CT(49690)=(S=12620796,E=12620832,C=29)
	CT(49691)=(S=12620833,E=12620892,C=19)
	CT(49692)=(S=12620893,E=12620902,C=29)
	CT(49693)=(S=12620903,E=12620903,C=21)
	CT(49694)=(S=12620904,E=12620910,C=29)
	CT(49695)=(S=12620911,E=12620915,C=78)
	CT(49696)=(S=12620916,E=12620965,C=0)
	CT(49697)=(S=12620966,E=12620966,C=18)
	CT(49698)=(S=12620967,E=12620967,C=25)
	CT(49699)=(S=12620968,E=12620974,C=29)
	CT(49700)=(S=12620975,E=12620975,C=204)
	CT(49701)=(S=12620976,E=12620976,C=29)
	CT(49702)=(S=12620977,E=12620986,C=14)
	CT(49703)=(S=12620987,E=12620991,C=29)
	CT(49704)=(S=12620992,E=12620992,C=36)
	CT(49705)=(S=12620993,E=12620993,C=15)
	CT(49706)=(S=12620994,E=12620994,C=14)
	CT(49707)=(S=12620995,E=12620997,C=29)
	CT(49708)=(S=12620998,E=12620999,C=14)
	CT(49709)=(S=12621000,E=12621000,C=36)
	CT(49710)=(S=12621001,E=12621015,C=25)
	CT(49711)=(S=12621016,E=12621016,C=14)
	CT(49712)=(S=12621017,E=12621017,C=78)
	CT(49713)=(S=12621018,E=12621019,C=29)
	CT(49714)=(S=12621020,E=12621020,C=15)
	CT(49715)=(S=12621021,E=12621021,C=73)
	CT(49716)=(S=12621022,E=12621022,C=29)
	CT(49717)=(S=12621023,E=12621030,C=0)
	CT(49718)=(S=12621031,E=12621058,C=29)
	CT(49719)=(S=12621059,E=12621059,C=58)
	CT(49720)=(S=12621060,E=12621060,C=29)
	CT(49721)=(S=12621061,E=12621061,C=47)
	CT(49722)=(S=12621062,E=12621070,C=29)
	CT(49723)=(S=12621071,E=12621071,C=14)
	CT(49724)=(S=12621072,E=12621074,C=29)
	CT(49725)=(S=12621075,E=12621075,C=25)
	CT(49726)=(S=12621076,E=12621076,C=0)
	CT(49727)=(S=12621077,E=12621078,C=29)
	CT(49728)=(S=12621079,E=12621079,C=78)
	CT(49729)=(S=12621080,E=12621082,C=29)
	CT(49730)=(S=12621083,E=12621083,C=42)
	CT(49731)=(S=12621084,E=12621084,C=29)
	CT(49732)=(S=12621085,E=12621089,C=19)
	CT(49733)=(S=12621090,E=12621090,C=29)
	CT(49734)=(S=12621091,E=12621091,C=36)
	CT(49735)=(S=12621092,E=12621092,C=0)
	CT(49736)=(S=12621093,E=12621096,C=29)
	CT(49737)=(S=12621097,E=12621097,C=12)
	CT(49738)=(S=12621098,E=12621108,C=29)
	CT(49739)=(S=12621109,E=12621109,C=78)
	CT(49740)=(S=12621110,E=12621110,C=0)
	CT(49741)=(S=12621111,E=12621112,C=29)
	CT(49742)=(S=12621113,E=12621113,C=14)
	CT(49743)=(S=12621114,E=12621114,C=29)
	CT(49744)=(S=12621115,E=12621115,C=12)
	CT(49745)=(S=12621116,E=12621116,C=14)
	CT(49746)=(S=12621117,E=12621117,C=29)
	CT(49747)=(S=12621118,E=12621122,C=78)
	CT(49748)=(S=12621123,E=12621132,C=29)
	CT(49749)=(S=12621133,E=12621133,C=12)
	CT(49750)=(S=12621134,E=12621135,C=27)
	CT(49751)=(S=12621136,E=12621136,C=78)
	CT(49752)=(S=12621138,E=12621145,C=29)
	CT(49753)=(S=12621146,E=12621147,C=78)
	CT(49754)=(S=12621148,E=12621153,C=29)
	CT(49755)=(S=12621154,E=12621154,C=78)
	CT(49756)=(S=12621155,E=12621155,C=29)
	CT(49757)=(S=12621156,E=12621156,C=27)
	CT(49758)=(S=12621157,E=12621157,C=29)
	CT(49759)=(S=12621158,E=12621158,C=27)
	CT(49760)=(S=12621159,E=12621159,C=29)
	CT(49761)=(S=12621160,E=12621160,C=78)
	CT(49762)=(S=12621161,E=12621165,C=29)
	CT(49763)=(S=12621166,E=12621166,C=15)
	CT(49764)=(S=12621167,E=12621167,C=14)
	CT(49765)=(S=12621168,E=12621172,C=29)
	CT(49766)=(S=12621173,E=12621173,C=14)
	CT(49767)=(S=12621174,E=12621174,C=36)
	CT(49768)=(S=12621175,E=12621177,C=14)
	CT(49769)=(S=12621178,E=12621178,C=29)
	CT(49770)=(S=12621179,E=12621179,C=78)
	CT(49771)=(S=12621180,E=12621181,C=29)
	CT(49772)=(S=12621182,E=12621182,C=28)
	CT(49773)=(S=12621183,E=12621257,C=29)
	CT(49774)=(S=12621258,E=12621258,C=159)
	CT(49775)=(S=12621259,E=12621264,C=29)
	CT(49776)=(S=12621265,E=12621265,C=2)
	CT(49777)=(S=12621266,E=12621282,C=29)
	CT(49778)=(S=12621283,E=12621283,C=36)
	CT(49779)=(S=12621284,E=12621287,C=29)
	CT(49780)=(S=12621288,E=12621288,C=42)
	CT(49781)=(S=12621289,E=12621293,C=29)
	CT(49782)=(S=12621294,E=12621294,C=14)
	CT(49783)=(S=12621296,E=12621369,C=29)
	CT(49784)=(S=12621370,E=12621384,C=19)
	CT(49785)=(S=12621385,E=12621386,C=29)
	CT(49786)=(S=12621387,E=12621388,C=19)
	CT(49787)=(S=12621389,E=12621389,C=29)
	CT(49788)=(S=12621390,E=12621394,C=19)
	CT(49789)=(S=12621395,E=12621395,C=58)
	CT(49790)=(S=12621396,E=12621396,C=19)
	CT(49791)=(S=12621397,E=12621400,C=29)
	CT(49792)=(S=12621401,E=12621401,C=14)
	CT(49793)=(S=12621402,E=12621403,C=78)
	CT(49794)=(S=12621404,E=12621404,C=14)
	CT(49795)=(S=12621405,E=12621405,C=29)
	CT(49796)=(S=12621406,E=12621406,C=36)
	CT(49797)=(S=12621407,E=12621415,C=29)
	CT(49798)=(S=12621416,E=12621416,C=58)
	CT(49799)=(S=12621417,E=12621417,C=29)
	CT(49800)=(S=12621418,E=12621418,C=14)
	CT(49801)=(S=12621420,E=12621435,C=29)
	CT(49802)=(S=12621436,E=12621436,C=27)
	CT(49803)=(S=12621437,E=12621437,C=29)
	CT(49804)=(S=12621438,E=12621438,C=215)
	CT(49805)=(S=12621439,E=12621440,C=29)
	CT(49806)=(S=12621441,E=12621451,C=0)
	CT(49807)=(S=12621452,E=12621452,C=14)
	CT(49808)=(S=12621453,E=12621457,C=29)
	CT(49809)=(S=12621458,E=12621458,C=15)
	CT(49810)=(S=12621459,E=12621488,C=29)
	CT(49811)=(S=12621489,E=12621496,C=14)
	CT(49812)=(S=12621497,E=12621497,C=78)
	CT(49813)=(S=12621498,E=12621499,C=29)
	CT(49814)=(S=12621500,E=12621500,C=27)
	CT(49815)=(S=12621501,E=12621502,C=28)
	CT(49816)=(S=12621503,E=12621503,C=29)
	CT(49817)=(S=12621504,E=12621505,C=21)
	CT(49818)=(S=12621506,E=12621508,C=15)
	CT(49819)=(S=12621509,E=12621509,C=27)
	CT(49820)=(S=12621510,E=12621510,C=14)
	CT(49821)=(S=12621511,E=12621513,C=29)
	CT(49822)=(S=12621514,E=12621514,C=0)
	CT(49823)=(S=12621515,E=12621515,C=42)
	CT(49824)=(S=12621516,E=12621516,C=14)
	CT(49825)=(S=12621517,E=12621517,C=18)
	CT(49826)=(S=12621518,E=12621519,C=29)
	CT(49827)=(S=12621520,E=12621520,C=14)
	CT(49828)=(S=12621521,E=12621521,C=27)
	CT(49829)=(S=12621522,E=12621529,C=29)
	CT(49830)=(S=12621530,E=12621531,C=73)
	CT(49831)=(S=12621533,E=12621534,C=29)
	CT(49832)=(S=12621535,E=12621535,C=27)
	CT(49833)=(S=12621536,E=12621538,C=29)
	CT(49834)=(S=12621539,E=12621545,C=14)
	CT(49835)=(S=12621546,E=12621549,C=0)
	CT(49836)=(S=12621550,E=12621550,C=50)
	CT(49837)=(S=12621551,E=12621557,C=29)
	CT(49838)=(S=12621558,E=12621558,C=0)
	CT(49839)=(S=12621559,E=12621559,C=29)
	CT(49840)=(S=12621560,E=12621560,C=14)
	CT(49841)=(S=12621561,E=12621563,C=3)
	CT(49842)=(S=12621564,E=12621564,C=27)
	CT(49843)=(S=12621565,E=12621565,C=29)
	CT(49844)=(S=12621566,E=12621566,C=27)
	CT(49845)=(S=12621567,E=12621676,C=29)
	CT(49846)=(S=12621677,E=12621677,C=78)
	CT(49847)=(S=12621678,E=12621699,C=29)
	CT(49848)=(S=12621700,E=12621703,C=78)
	CT(49849)=(S=12621704,E=12621829,C=29)
	CT(49850)=(S=12621830,E=12621830,C=27)
	CT(49851)=(S=12621831,E=12621837,C=29)
	CT(49852)=(S=12621838,E=12621838,C=14)
	CT(49853)=(S=12621839,E=12621840,C=29)
	CT(49854)=(S=12621841,E=12621841,C=14)
	CT(49855)=(S=12621842,E=12621849,C=29)
	CT(49856)=(S=12621850,E=12621851,C=28)
	CT(49857)=(S=12621852,E=12621865,C=29)
	CT(49858)=(S=12621866,E=12621866,C=12)
	CT(49859)=(S=12621867,E=12621867,C=29)
	CT(49860)=(S=12621868,E=12621868,C=14)
	CT(49861)=(S=12621869,E=12621870,C=29)
	CT(49862)=(S=12621871,E=12621871,C=21)
	CT(49863)=(S=12621872,E=12621872,C=14)
	CT(49864)=(S=12621873,E=12621875,C=36)
	CT(49865)=(S=12621876,E=12621877,C=29)
	CT(49866)=(S=12621878,E=12621878,C=42)
	CT(49867)=(S=12621879,E=12621884,C=29)
	CT(49868)=(S=12621885,E=12621886,C=14)
	CT(49869)=(S=12621887,E=12621887,C=42)
	CT(49870)=(S=12621888,E=12621891,C=29)
	CT(49871)=(S=12621892,E=12621892,C=47)
	CT(49872)=(S=12621893,E=12621893,C=2)
	CT(49873)=(S=12621894,E=12621902,C=29)
	CT(49874)=(S=12621903,E=12621903,C=78)
	CT(49875)=(S=12621904,E=12621905,C=29)
	CT(49876)=(S=12621906,E=12621906,C=28)
	CT(49877)=(S=12621907,E=12621914,C=29)
	CT(49878)=(S=12621915,E=12621915,C=78)
	CT(49879)=(S=12621916,E=12621920,C=29)
	CT(49880)=(S=12621921,E=12621921,C=78)
	CT(49881)=(S=12621922,E=12621922,C=34)
	CT(49882)=(S=12621923,E=12621930,C=29)
	CT(49883)=(S=12621931,E=12621931,C=78)
	CT(49884)=(S=12621932,E=12621934,C=29)
	CT(49885)=(S=12621935,E=12621935,C=27)
	CT(49886)=(S=12621936,E=12621936,C=14)
	CT(49887)=(S=12621937,E=12621942,C=29)
	CT(49888)=(S=12621943,E=12621943,C=0)
	CT(49889)=(S=12621944,E=12621945,C=29)
	CT(49890)=(S=12621946,E=12621946,C=14)
	CT(49891)=(S=12621947,E=12621947,C=29)
	CT(49892)=(S=12621948,E=12621948,C=49)
	CT(49893)=(S=12621949,E=12621963,C=29)
	CT(49894)=(S=12621965,E=12621965,C=15)
	CT(49895)=(S=12621966,E=12621974,C=29)
	CT(49896)=(S=12621975,E=12621975,C=28)
	CT(49897)=(S=12621976,E=12621979,C=29)
	CT(49898)=(S=12621980,E=12621980,C=14)
	CT(49899)=(S=12621981,E=12621981,C=29)
	CT(49900)=(S=12621982,E=12621982,C=78)
	CT(49901)=(S=12621983,E=12621987,C=29)
	CT(49902)=(S=12621988,E=12621988,C=80)
	CT(49903)=(S=12621989,E=12621989,C=78)
	CT(49904)=(S=12621990,E=12621990,C=15)
	CT(49905)=(S=12621991,E=12621991,C=12)
	CT(49906)=(S=12621992,E=12621997,C=29)
	CT(49907)=(S=12621998,E=12621998,C=21)
	CT(49908)=(S=12621999,E=12622007,C=29)
	CT(49909)=(S=12622008,E=12622011,C=14)
	CT(49910)=(S=12622012,E=12622018,C=29)
	CT(49911)=(S=12622019,E=12622019,C=78)
	CT(49912)=(S=12622020,E=12622035,C=29)
	CT(49913)=(S=12622036,E=12622036,C=2)
	CT(49914)=(S=12622037,E=12622037,C=78)
	CT(49915)=(S=12622038,E=12622047,C=29)
	CT(49916)=(S=12622048,E=12622048,C=78)
	CT(49917)=(S=12622049,E=12622064,C=29)
	CT(49918)=(S=12622065,E=12622065,C=14)
	CT(49919)=(S=12622066,E=12622067,C=29)
	CT(49920)=(S=12622068,E=12622068,C=14)
	CT(49921)=(S=12622069,E=12622071,C=29)
	CT(49922)=(S=12622072,E=12622072,C=78)
	CT(49923)=(S=12622073,E=12622076,C=29)
	CT(49924)=(S=12622077,E=12622077,C=14)
	CT(49925)=(S=12622078,E=12622078,C=42)
	CT(49926)=(S=12622079,E=12622081,C=29)
	CT(49927)=(S=12622082,E=12622083,C=14)
	CT(49928)=(S=12622084,E=12622091,C=29)
	CT(49929)=(S=12622093,E=12622093,C=21)
	CT(49930)=(S=12622094,E=12622097,C=29)
	CT(49931)=(S=12622098,E=12622099,C=50)
	CT(49932)=(S=12622100,E=12622167,C=29)
	CT(49933)=(S=12622168,E=12622168,C=189)
	CT(49934)=(S=12622169,E=12622169,C=34)
	CT(49935)=(S=12622170,E=12622172,C=29)
	CT(49936)=(S=12622173,E=12622173,C=0)
	CT(49937)=(S=12622174,E=12622174,C=29)
	CT(49938)=(S=12622175,E=12622194,C=2)
	CT(49939)=(S=12622195,E=12622195,C=29)
	CT(49940)=(S=12622196,E=12622196,C=50)
	CT(49941)=(S=12622197,E=12622198,C=29)
	CT(49942)=(S=12622199,E=12622199,C=203)
	CT(49943)=(S=12622200,E=12622200,C=189)
	CT(49944)=(S=12622201,E=12622206,C=29)
	CT(49945)=(S=12622207,E=12622207,C=58)
	CT(49946)=(S=12622208,E=12622232,C=29)
	CT(49947)=(S=12622233,E=12622233,C=14)
	CT(49948)=(S=12622234,E=12622234,C=29)
	CT(49949)=(S=12622235,E=12622235,C=204)
	CT(49950)=(S=12622236,E=12622237,C=29)
	CT(49951)=(S=12622238,E=12622238,C=0)
	CT(49952)=(S=12622239,E=12622245,C=29)
	CT(49953)=(S=12622246,E=12622246,C=14)
	CT(49954)=(S=12622247,E=12622247,C=202)
	CT(49955)=(S=12622248,E=12622248,C=49)
	CT(49956)=(S=12622249,E=12622250,C=29)
	CT(49957)=(S=12622251,E=12622251,C=18)
	CT(49958)=(S=12622252,E=12622252,C=29)
	CT(49959)=(S=12622253,E=12622262,C=42)
	CT(49960)=(S=12622263,E=12622265,C=29)
	CT(49961)=(S=12622266,E=12622266,C=78)
	CT(49962)=(S=12622267,E=12622267,C=29)
	CT(49963)=(S=12622268,E=12622268,C=19)
	CT(49964)=(S=12622269,E=12622269,C=14)
	CT(49965)=(S=12622270,E=12622273,C=29)
	CT(49966)=(S=12622274,E=12622274,C=27)
	CT(49967)=(S=12622275,E=12622275,C=29)
	CT(49968)=(S=12622276,E=12622277,C=0)
	CT(49969)=(S=12622278,E=12622292,C=29)
	CT(49970)=(S=12622293,E=12622293,C=14)
	CT(49971)=(S=12622294,E=12622296,C=29)
	CT(49972)=(S=12622297,E=12622297,C=78)
	CT(49973)=(S=12622298,E=12622299,C=29)
	CT(49974)=(S=12622300,E=12622300,C=78)
	CT(49975)=(S=12622309,E=12622330,C=29)
	CT(49976)=(S=12622331,E=12622331,C=73)
	CT(49977)=(S=12622332,E=12622348,C=29)
	CT(49978)=(S=12622349,E=12622349,C=78)
	CT(49979)=(S=12622350,E=12622351,C=29)
	CT(49980)=(S=12622352,E=12622361,C=2)
	CT(49981)=(S=12622362,E=12622363,C=29)
	CT(49982)=(S=12622364,E=12622367,C=78)
	CT(49983)=(S=12622368,E=12622471,C=29)
	CT(49984)=(S=12622472,E=12622495,C=78)
	CT(49985)=(S=12622496,E=12622591,C=29)
	CT(49986)=(S=12622592,E=12622592,C=78)
	CT(49987)=(S=12622593,E=12622598,C=27)
	CT(49988)=(S=12622599,E=12622660,C=29)
	CT(49989)=(S=12622661,E=12622661,C=78)
	CT(49990)=(S=12622662,E=12622749,C=29)
	CT(49991)=(S=12622750,E=12622751,C=80)
	CT(49992)=(S=12622752,E=12622979,C=29)
	CT(49993)=(S=12622980,E=12622980,C=49)
	CT(49994)=(S=12622981,E=12622987,C=29)
	CT(49995)=(S=12622988,E=12623002,C=2)
	CT(49996)=(S=12623005,E=12623005,C=29)
	CT(49997)=(S=12623006,E=12623007,C=2)
	CT(49998)=(S=12623008,E=12623009,C=29)
	CT(49999)=(S=12623010,E=12623010,C=14)
	CT(50000)=(S=12623011,E=12623012,C=29)
	CT(50001)=(S=12623013,E=12623013,C=73)
	CT(50002)=(S=12623014,E=12623014,C=29)
	CT(50003)=(S=12623015,E=12623015,C=14)
	CT(50004)=(S=12623016,E=12623016,C=29)
	CT(50005)=(S=12623017,E=12623017,C=14)
	CT(50006)=(S=12623018,E=12623046,C=29)
	CT(50007)=(S=12623047,E=12623047,C=178)
	CT(50008)=(S=12623048,E=12623054,C=29)
	CT(50009)=(S=12623055,E=12623055,C=0)
	CT(50010)=(S=12623057,E=12623057,C=29)
	CT(50011)=(S=12623058,E=12623058,C=21)
	CT(50012)=(S=12623059,E=12623060,C=29)
	CT(50013)=(S=12623061,E=12623061,C=15)
	CT(50014)=(S=12623062,E=12623072,C=29)
	CT(50015)=(S=12623073,E=12623073,C=73)
	CT(50016)=(S=12623074,E=12623074,C=29)
	CT(50017)=(S=12623075,E=12623075,C=49)
	CT(50018)=(S=12623076,E=12623086,C=29)
	CT(50019)=(S=12623087,E=12623087,C=14)
	CT(50020)=(S=12623088,E=12623095,C=29)
	CT(50021)=(S=12623096,E=12623096,C=70)
	CT(50022)=(S=12623098,E=12623104,C=29)
	CT(50023)=(S=12623105,E=12623107,C=27)
	CT(50024)=(S=12623108,E=12623111,C=29)
	CT(50025)=(S=12623112,E=12623121,C=19)
	CT(50026)=(S=12623122,E=12623199,C=29)
	CT(50027)=(S=12623200,E=12623231,C=78)
	CT(50028)=(S=12623232,E=12623232,C=29)
	CT(50029)=(S=12623233,E=12623233,C=27)
	CT(50030)=(S=12623234,E=12623267,C=29)
	CT(50031)=(S=12623268,E=12623268,C=78)
	CT(50032)=(S=12623269,E=12623272,C=42)
	CT(50033)=(S=12623273,E=12623274,C=14)
	CT(50034)=(S=12623275,E=12623275,C=28)
	CT(50035)=(S=12623276,E=12623276,C=14)
	CT(50036)=(S=12623277,E=12623277,C=78)
	CT(50037)=(S=12623278,E=12623278,C=14)
	CT(50038)=(S=12623279,E=12623279,C=29)
	CT(50039)=(S=12623280,E=12623280,C=14)
	CT(50040)=(S=12623281,E=12623288,C=29)
	CT(50041)=(S=12623289,E=12623289,C=15)
	CT(50042)=(S=12623290,E=12623290,C=29)
	CT(50043)=(S=12623291,E=12623291,C=14)
	CT(50044)=(S=12623292,E=12623292,C=29)
	CT(50045)=(S=12623293,E=12623293,C=34)
	CT(50046)=(S=12623294,E=12623294,C=105)
	CT(50047)=(S=12623296,E=12623371,C=29)
	CT(50048)=(S=12623372,E=12623375,C=78)
	CT(50049)=(S=12623376,E=12623421,C=29)
	CT(50050)=(S=12623422,E=12623423,C=80)
	CT(50051)=(S=12623424,E=12623573,C=29)
	CT(50052)=(S=12623574,E=12623575,C=78)
	CT(50053)=(S=12623576,E=12623631,C=29)
	CT(50054)=(S=12623632,E=12623646,C=7)
	CT(50055)=(S=12623647,E=12623685,C=29)
	CT(50056)=(S=12623686,E=12623686,C=47)
	CT(50057)=(S=12623687,E=12623687,C=78)
	CT(50058)=(S=12623688,E=12623688,C=29)
	CT(50059)=(S=12623689,E=12623689,C=14)
	CT(50060)=(S=12623690,E=12623692,C=29)
	CT(50061)=(S=12623693,E=12623693,C=15)
	CT(50062)=(S=12623694,E=12623699,C=29)
	CT(50063)=(S=12623700,E=12623701,C=25)
	CT(50064)=(S=12623702,E=12623705,C=29)
	CT(50065)=(S=12623706,E=12623706,C=49)
	CT(50066)=(S=12623707,E=12623709,C=29)
	CT(50067)=(S=12623710,E=12623710,C=78)
	CT(50068)=(S=12623711,E=12623711,C=28)
	CT(50069)=(S=12623712,E=12623714,C=29)
	CT(50070)=(S=12623715,E=12623719,C=14)
	CT(50071)=(S=12623720,E=12623720,C=29)
	CT(50072)=(S=12623721,E=12623721,C=36)
	CT(50073)=(S=12623722,E=12623723,C=29)
	CT(50074)=(S=12623724,E=12623724,C=27)
	CT(50075)=(S=12623725,E=12623725,C=63)
	CT(50076)=(S=12623726,E=12623726,C=29)
	CT(50077)=(S=12623727,E=12623727,C=78)
	CT(50078)=(S=12623728,E=12623731,C=29)
	CT(50079)=(S=12623732,E=12623733,C=189)
	CT(50080)=(S=12623734,E=12623734,C=63)
	CT(50081)=(S=12623735,E=12623736,C=29)
	CT(50082)=(S=12623737,E=12623737,C=12)
	CT(50083)=(S=12623738,E=12623738,C=47)
	CT(50084)=(S=12623739,E=12623807,C=29)
	CT(50085)=(S=12623808,E=12623871,C=78)
	CT(50086)=(S=12623873,E=12623878,C=29)
	CT(50087)=(S=12623879,E=12623879,C=78)
	CT(50088)=(S=12623880,E=12623881,C=29)
	CT(50089)=(S=12623882,E=12623882,C=14)
	CT(50090)=(S=12623883,E=12623886,C=29)
	CT(50091)=(S=12623887,E=12623887,C=52)
	CT(50092)=(S=12623888,E=12623888,C=0)
	CT(50093)=(S=12623889,E=12623890,C=29)
	CT(50094)=(S=12623891,E=12623891,C=0)
	CT(50095)=(S=12623892,E=12623892,C=29)
	CT(50096)=(S=12623893,E=12623893,C=37)
	CT(50097)=(S=12623894,E=12623894,C=70)
	CT(50098)=(S=12623895,E=12623895,C=27)
	CT(50099)=(S=12623896,E=12623898,C=29)
	CT(50100)=(S=12623899,E=12623899,C=15)
	CT(50101)=(S=12623900,E=12623904,C=29)
	CT(50102)=(S=12623905,E=12623905,C=25)
	CT(50103)=(S=12623907,E=12623908,C=29)
	CT(50104)=(S=12623909,E=12623909,C=15)
	CT(50105)=(S=12623910,E=12623916,C=29)
	CT(50106)=(S=12623917,E=12623917,C=189)
	CT(50107)=(S=12623918,E=12623921,C=29)
	CT(50108)=(S=12623922,E=12623922,C=189)
	CT(50109)=(S=12623923,E=12623936,C=29)
	CT(50110)=(S=12623937,E=12623937,C=28)
	CT(50111)=(S=12623938,E=12623938,C=29)
	CT(50112)=(S=12623939,E=12623939,C=42)
	CT(50113)=(S=12623940,E=12623942,C=29)
	CT(50114)=(S=12623943,E=12623943,C=0)
	CT(50115)=(S=12623944,E=12623966,C=29)
	CT(50116)=(S=12623967,E=12623968,C=34)
	CT(50117)=(S=12623969,E=12623973,C=29)
	CT(50118)=(S=12623974,E=12623974,C=78)
	CT(50119)=(S=12623975,E=12623977,C=29)
	CT(50120)=(S=12623978,E=12623978,C=36)
	CT(50121)=(S=12623979,E=12623980,C=29)
	CT(50122)=(S=12623981,E=12623981,C=14)
	CT(50123)=(S=12623982,E=12623982,C=58)
	CT(50124)=(S=12623983,E=12623983,C=189)
	CT(50125)=(S=12623984,E=12623994,C=29)
	CT(50126)=(S=12623995,E=12623995,C=12)
	CT(50127)=(S=12623996,E=12623997,C=29)
	CT(50128)=(S=12623998,E=12623998,C=27)
	CT(50129)=(S=12623999,E=12623999,C=29)
	CT(50130)=(S=12624000,E=12624000,C=189)
	CT(50131)=(S=12624001,E=12624013,C=29)
	CT(50132)=(S=12624014,E=12624014,C=28)
	CT(50133)=(S=12624015,E=12624015,C=14)
	CT(50134)=(S=12624016,E=12624023,C=29)
	CT(50135)=(S=12624024,E=12624024,C=36)
	CT(50136)=(S=12624025,E=12624026,C=29)
	CT(50137)=(S=12624028,E=12624028,C=15)
	CT(50138)=(S=12624029,E=12624031,C=29)
	CT(50139)=(S=12624032,E=12624032,C=15)
	CT(50140)=(S=12624033,E=12624033,C=2)
	CT(50141)=(S=12624034,E=12624038,C=29)
	CT(50142)=(S=12624039,E=12624039,C=78)
	CT(50143)=(S=12624040,E=12624040,C=29)
	CT(50144)=(S=12624041,E=12624043,C=0)
	CT(50145)=(S=12624044,E=12624044,C=50)
	CT(50146)=(S=12624045,E=12624045,C=29)
	CT(50147)=(S=12624046,E=12624048,C=80)
	CT(50148)=(S=12624049,E=12624051,C=14)
	CT(50149)=(S=12624052,E=12624059,C=29)
	CT(50150)=(S=12624060,E=12624060,C=189)
	CT(50151)=(S=12624061,E=12624065,C=29)
	CT(50152)=(S=12624066,E=12624067,C=14)
	CT(50153)=(S=12624068,E=12624076,C=29)
	CT(50154)=(S=12624077,E=12624077,C=0)
	CT(50155)=(S=12624078,E=12624095,C=29)
	CT(50156)=(S=12624096,E=12624096,C=18)
	CT(50157)=(S=12624097,E=12624097,C=27)
	CT(50158)=(S=12624099,E=12624100,C=29)
	CT(50159)=(S=12624101,E=12624101,C=0)
	CT(50160)=(S=12624102,E=12624102,C=29)
	CT(50161)=(S=12624103,E=12624104,C=42)
	CT(50162)=(S=12624105,E=12624105,C=18)
	CT(50163)=(S=12624106,E=12624116,C=29)
	CT(50164)=(S=12624117,E=12624121,C=21)
	CT(50165)=(S=12624122,E=12624122,C=174)
	CT(50166)=(S=12624123,E=12624123,C=21)
	CT(50167)=(S=12624124,E=12624124,C=17)
	CT(50168)=(S=12624125,E=12624126,C=29)
	CT(50169)=(S=12624127,E=12624127,C=78)
	CT(50170)=(S=12624128,E=12624191,C=29)
	CT(50171)=(S=12624192,E=12624193,C=49)
	CT(50172)=(S=12624194,E=12624223,C=29)
	CT(50173)=(S=12624224,E=12624255,C=78)
	CT(50174)=(S=12624256,E=12624257,C=73)
	CT(50175)=(S=12624258,E=12624319,C=29)
	CT(50176)=(S=12624320,E=12624383,C=78)
	CT(50177)=(S=12624384,E=12624387,C=18)
	CT(50178)=(S=12624388,E=12624391,C=27)
	CT(50179)=(S=12624392,E=12624395,C=18)
	CT(50180)=(S=12624396,E=12624399,C=25)
	CT(50181)=(S=12624400,E=12624400,C=38)
	CT(50182)=(S=12624401,E=12624401,C=21)
	CT(50183)=(S=12624402,E=12624402,C=14)
	CT(50184)=(S=12624403,E=12624403,C=31)
	CT(50185)=(S=12624404,E=12624407,C=18)
	CT(50186)=(S=12624408,E=12624411,C=25)
	CT(50187)=(S=12624412,E=12624415,C=34)
	CT(50188)=(S=12624416,E=12624419,C=31)
	CT(50189)=(S=12624420,E=12624427,C=18)
	CT(50190)=(S=12624428,E=12624435,C=58)
	CT(50191)=(S=12624436,E=12624439,C=46)
	CT(50192)=(S=12624440,E=12624443,C=18)
	CT(50193)=(S=12624444,E=12624447,C=58)
	CT(50194)=(S=12624448,E=12624451,C=18)
	CT(50195)=(S=12624452,E=12624455,C=12)
	CT(50196)=(S=12624456,E=12624459,C=23)
	CT(50197)=(S=12624460,E=12624467,C=31)
	CT(50198)=(S=12624468,E=12624471,C=28)
	CT(50199)=(S=12624472,E=12624475,C=18)
	CT(50200)=(S=12624476,E=12624479,C=58)
	CT(50201)=(S=12624480,E=12624483,C=33)
	CT(50202)=(S=12624484,E=12624487,C=18)
	CT(50203)=(S=12624488,E=12624491,C=60)
	CT(50204)=(S=12624492,E=12624503,C=31)
	CT(50205)=(S=12624504,E=12624511,C=18)
	CT(50206)=(S=12624512,E=12624515,C=58)
	CT(50207)=(S=12624516,E=12624519,C=31)
	CT(50208)=(S=12624520,E=12624523,C=36)
	CT(50209)=(S=12624524,E=12624527,C=31)
	CT(50210)=(S=12624528,E=12624535,C=58)
	CT(50211)=(S=12624536,E=12624539,C=31)
	CT(50212)=(S=12624540,E=12624543,C=18)
	CT(50213)=(S=12624544,E=12624547,C=31)
	CT(50214)=(S=12624548,E=12624551,C=58)
	CT(50215)=(S=12624552,E=12624555,C=28)
	CT(50216)=(S=12624556,E=12624563,C=58)
	CT(50217)=(S=12624564,E=12624567,C=18)
	CT(50218)=(S=12624568,E=12624571,C=38)
	CT(50219)=(S=12624572,E=12624579,C=18)
	CT(50220)=(S=12624584,E=12624595,C=31)
	CT(50221)=(S=12624596,E=12624599,C=18)
	CT(50222)=(S=12624600,E=12624603,C=63)
	CT(50223)=(S=12624604,E=12624607,C=19)
	CT(50224)=(S=12624608,E=12624611,C=31)
	CT(50225)=(S=12624612,E=12624614,C=28)
	CT(50226)=(S=12624615,E=12624615,C=51)
	CT(50227)=(S=12624616,E=12624623,C=31)
	CT(50228)=(S=12624624,E=12624631,C=18)
	CT(50229)=(S=12624632,E=12624632,C=58)
	CT(50230)=(S=12624633,E=12624639,C=18)
	CT(50231)=(S=12624640,E=12624671,C=29)
	CT(50232)=(S=12624672,E=12624799,C=47)
	CT(50233)=(S=12624800,E=12624895,C=29)
	CT(50234)=(S=12624896,E=12625151,C=42)
	CT(50235)=(S=12625152,E=12625407,C=19)
	CT(50236)=(S=12625408,E=12625415,C=28)
	CT(50237)=(S=12625420,E=12625423,C=18)
	CT(50238)=(S=12625424,E=12625503,C=28)
	CT(50239)=(S=12625504,E=12625507,C=36)
	CT(50240)=(S=12625508,E=12625511,C=31)
	CT(50241)=(S=12625512,E=12625519,C=28)
	CT(50242)=(S=12625520,E=12625521,C=31)
	CT(50243)=(S=12625524,E=12625527,C=18)
	CT(50244)=(S=12625528,E=12625529,C=58)
	CT(50245)=(S=12625530,E=12625530,C=38)
	CT(50246)=(S=12625531,E=12625535,C=18)
	CT(50247)=(S=12625536,E=12625543,C=31)
	CT(50248)=(S=12625544,E=12625547,C=14)
	CT(50249)=(S=12625548,E=12625551,C=12)
	CT(50250)=(S=12625552,E=12625553,C=31)
	CT(50251)=(S=12625554,E=12625555,C=28)
	CT(50252)=(S=12625556,E=12625559,C=25)
	CT(50253)=(S=12625560,E=12625609,C=28)
	CT(50254)=(S=12625610,E=12625611,C=58)
	CT(50255)=(S=12625616,E=12625623,C=28)
	CT(50256)=(S=12625624,E=12625627,C=58)
	CT(50257)=(S=12625640,E=12625647,C=18)
	CT(50258)=(S=12625661,E=12625662,C=28)
	CT(50259)=(S=12625663,E=12625663,C=58)
	CT(50260)=(S=12625664,E=12625919,C=15)
	CT(50261)=(S=12626176,E=12626207,C=29)
	CT(50262)=(S=12626208,E=12626217,C=7)
	CT(50263)=(S=12626218,E=12626688,C=29)
	CT(50264)=(S=12626689,E=12626693,C=13)
	CT(50265)=(S=12626694,E=12626719,C=29)
	CT(50266)=(S=12626720,E=12626751,C=78)
	CT(50267)=(S=12626752,E=12626801,C=29)
	CT(50268)=(S=12626802,E=12626803,C=78)
	CT(50269)=(S=12626804,E=12626807,C=29)
	CT(50270)=(S=12626808,E=12626815,C=79)
	CT(50271)=(S=12626816,E=12626887,C=14)
	CT(50272)=(S=12626888,E=12627170,C=29)
	CT(50273)=(S=12627171,E=12627175,C=2)
	CT(50274)=(S=12627177,E=12627177,C=29)
	CT(50275)=(S=12627178,E=12627183,C=2)
	CT(50276)=(S=12627184,E=12627184,C=29)
	CT(50277)=(S=12627185,E=12627187,C=2)
	CT(50278)=(S=12627191,E=12627193,C=29)
	CT(50279)=(S=12627194,E=12627194,C=78)
	CT(50280)=(S=12627195,E=12627196,C=29)
	CT(50281)=(S=12627197,E=12627197,C=70)
	CT(50282)=(S=12627198,E=12627198,C=78)
	CT(50283)=(S=12627199,E=12627200,C=29)
	CT(50284)=(S=12627201,E=12627204,C=14)
	CT(50285)=(S=12627205,E=12627215,C=29)
	CT(50286)=(S=12627216,E=12627224,C=73)
	CT(50287)=(S=12627225,E=12627227,C=29)
	CT(50288)=(S=12627228,E=12627231,C=78)
	CT(50289)=(S=12627232,E=12627327,C=29)
	CT(50290)=(S=12627328,E=12627335,C=14)
	CT(50291)=(S=12627336,E=12627459,C=29)
	CT(50292)=(S=12627460,E=12627463,C=78)
	CT(50293)=(S=12627464,E=12627519,C=29)
	CT(50294)=(S=12627520,E=12627524,C=42)
	CT(50295)=(S=12627525,E=12627583,C=29)
	CT(50296)=(S=12627584,E=12627711,C=2)
	CT(50297)=(S=12627712,E=12627717,C=29)
	CT(50298)=(S=12627718,E=12627719,C=78)
	CT(50299)=(S=12627720,E=12627743,C=29)
	CT(50300)=(S=12627744,E=12627748,C=47)
	CT(50301)=(S=12627749,E=12627749,C=12)
	CT(50302)=(S=12627750,E=12627751,C=34)
	CT(50303)=(S=12627752,E=12627755,C=14)
	CT(50304)=(S=12627756,E=12627759,C=47)
	CT(50305)=(S=12627760,E=12627799,C=29)
	CT(50306)=(S=12627800,E=12627839,C=78)
	CT(50307)=(S=12627840,E=12627967,C=29)
	CT(50308)=(S=12627968,E=12628223,C=19)
	CT(50309)=(S=12628224,E=12630031,C=29)
	CT(50310)=(S=12630032,E=12630047,C=78)
	CT(50311)=(S=12630048,E=12630591,C=29)
	CT(50312)=(S=12630592,E=12630655,C=78)
	CT(50313)=(S=12630656,E=12630799,C=29)
	CT(50314)=(S=12630800,E=12630809,C=25)
	CT(50315)=(S=12630810,E=12630811,C=29)
	CT(50316)=(S=12630812,E=12630815,C=78)
	CT(50317)=(S=12630816,E=12631041,C=29)
	CT(50318)=(S=12631042,E=12631042,C=78)
	CT(50319)=(S=12631043,E=12631049,C=29)
	CT(50320)=(S=12631050,E=12631050,C=21)
	CT(50321)=(S=12631051,E=12631051,C=189)
	CT(50322)=(S=12631052,E=12631082,C=29)
	CT(50323)=(S=12631083,E=12631083,C=73)
	CT(50324)=(S=12631084,E=12631100,C=213)
	CT(50325)=(S=12631101,E=12631102,C=29)
	CT(50326)=(S=12631103,E=12631104,C=14)
	CT(50327)=(S=12631105,E=12631105,C=12)
	CT(50328)=(S=12631106,E=12631108,C=29)
	CT(50329)=(S=12631109,E=12631109,C=14)
	CT(50330)=(S=12631110,E=12631110,C=29)
	CT(50331)=(S=12631111,E=12631111,C=73)
	CT(50332)=(S=12631112,E=12631112,C=29)
	CT(50333)=(S=12631113,E=12631119,C=2)
	CT(50334)=(S=12631124,E=12631124,C=29)
	CT(50335)=(S=12631125,E=12631127,C=2)
	CT(50336)=(S=12631128,E=12631128,C=78)
	CT(50337)=(S=12631131,E=12631131,C=29)
	CT(50338)=(S=12631132,E=12631132,C=2)
	CT(50339)=(S=12631133,E=12631135,C=29)
	CT(50340)=(S=12631136,E=12631136,C=14)
	CT(50341)=(S=12631137,E=12631140,C=29)
	CT(50342)=(S=12631141,E=12631141,C=0)
	CT(50343)=(S=12631142,E=12631144,C=29)
	CT(50344)=(S=12631145,E=12631145,C=27)
	CT(50345)=(S=12631146,E=12631146,C=29)
	CT(50346)=(S=12631147,E=12631147,C=0)
	CT(50347)=(S=12631148,E=12631154,C=29)
	CT(50348)=(S=12631155,E=12631155,C=0)
	CT(50349)=(S=12631156,E=12631156,C=18)
	CT(50350)=(S=12631157,E=12631157,C=14)
	CT(50351)=(S=12631158,E=12631160,C=29)
	CT(50352)=(S=12631161,E=12631161,C=42)
	CT(50353)=(S=12631162,E=12631162,C=14)
	CT(50354)=(S=12631163,E=12631164,C=29)
	CT(50355)=(S=12631165,E=12631165,C=14)
	CT(50356)=(S=12631166,E=12631166,C=29)
	CT(50357)=(S=12631167,E=12631167,C=63)
	CT(50358)=(S=12631169,E=12631169,C=14)
	CT(50359)=(S=12631170,E=12631171,C=29)
	CT(50360)=(S=12631172,E=12631172,C=14)
	CT(50361)=(S=12631173,E=12631175,C=29)
	CT(50362)=(S=12631176,E=12631176,C=28)
	CT(50363)=(S=12631177,E=12631184,C=29)
	CT(50364)=(S=12631185,E=12631185,C=14)
	CT(50365)=(S=12631186,E=12631190,C=29)
	CT(50366)=(S=12631191,E=12631191,C=0)
	CT(50367)=(S=12631192,E=12631196,C=29)
	CT(50368)=(S=12631197,E=12631198,C=14)
	CT(50369)=(S=12631199,E=12631203,C=29)
	CT(50370)=(S=12631204,E=12631207,C=122)
	CT(50371)=(S=12631208,E=12631209,C=78)
	CT(50372)=(S=12631210,E=12631210,C=1)
	CT(50373)=(S=12631211,E=12631211,C=9)
	CT(50374)=(S=12631212,E=12631212,C=3)
	CT(50375)=(S=12631213,E=12631213,C=29)
	CT(50376)=(S=12631214,E=12631214,C=10)
	CT(50377)=(S=12631215,E=12631215,C=74)
	CT(50378)=(S=12631216,E=12631224,C=29)
	CT(50379)=(S=12631225,E=12631225,C=73)
	CT(50380)=(S=12631226,E=12631226,C=0)
	CT(50381)=(S=12631227,E=12631227,C=18)
	CT(50382)=(S=12631228,E=12631228,C=29)
	CT(50383)=(S=12631229,E=12631229,C=18)
	CT(50384)=(S=12631230,E=12631233,C=29)
	CT(50385)=(S=12631234,E=12631237,C=213)
	CT(50386)=(S=12631238,E=12631272,C=29)
	CT(50387)=(S=12631273,E=12631277,C=58)
	CT(50388)=(S=12631278,E=12631280,C=29)
	CT(50389)=(S=12631281,E=12631281,C=73)
	CT(50390)=(S=12631282,E=12631287,C=50)
	CT(50391)=(S=12631289,E=12631296,C=29)
	CT(50392)=(S=12631297,E=12631297,C=14)
	CT(50393)=(S=12631298,E=12631303,C=29)
	CT(50394)=(S=12631304,E=12631307,C=14)
	CT(50395)=(S=12631308,E=12631309,C=29)
	CT(50396)=(S=12631310,E=12631310,C=28)
	CT(50397)=(S=12631311,E=12631317,C=29)
	CT(50398)=(S=12631318,E=12631318,C=78)
	CT(50399)=(S=12631319,E=12631319,C=14)
	CT(50400)=(S=12631320,E=12631320,C=2)
	CT(50401)=(S=12631321,E=12631329,C=29)
	CT(50402)=(S=12631330,E=12631330,C=73)
	CT(50403)=(S=12631331,E=12631331,C=29)
	CT(50404)=(S=12631332,E=12631335,C=78)
	CT(50405)=(S=12631336,E=12631336,C=29)
	CT(50406)=(S=12631337,E=12631337,C=14)
	CT(50407)=(S=12631338,E=12631346,C=29)
	CT(50408)=(S=12631347,E=12631347,C=42)
	CT(50409)=(S=12631348,E=12631348,C=14)
	CT(50410)=(S=12631349,E=12631349,C=29)
	CT(50411)=(S=12631350,E=12631350,C=0)
	CT(50412)=(S=12631351,E=12631351,C=14)
	CT(50413)=(S=12631352,E=12631360,C=29)
	CT(50414)=(S=12631362,E=12631362,C=12)
	CT(50415)=(S=12631363,E=12631364,C=78)
	CT(50416)=(S=12631365,E=12631365,C=14)
	CT(50417)=(S=12631366,E=12631366,C=17)
	CT(50418)=(S=12631367,E=12631368,C=29)
	CT(50419)=(S=12631369,E=12631369,C=36)
	CT(50420)=(S=12631370,E=12631370,C=29)
	CT(50421)=(S=12631371,E=12631371,C=105)
	CT(50422)=(S=12631372,E=12631372,C=28)
	CT(50423)=(S=12631373,E=12631414,C=29)
	CT(50424)=(S=12631415,E=12631415,C=14)
	CT(50425)=(S=12631416,E=12631431,C=29)
	CT(50426)=(S=12631432,E=12631432,C=0)
	CT(50427)=(S=12631433,E=12631434,C=29)
	CT(50428)=(S=12631435,E=12631436,C=122)
	CT(50429)=(S=12631437,E=12631443,C=29)
	CT(50430)=(S=12631444,E=12631444,C=0)
	CT(50431)=(S=12631445,E=12631446,C=29)
	CT(50432)=(S=12631447,E=12631447,C=28)
	CT(50433)=(S=12631448,E=12631449,C=29)
	CT(50434)=(S=12631450,E=12631450,C=15)
	CT(50435)=(S=12631451,E=12631452,C=29)
	CT(50436)=(S=12631453,E=12631453,C=14)
	CT(50437)=(S=12631454,E=12631454,C=29)
	CT(50438)=(S=12631456,E=12631456,C=14)
	CT(50439)=(S=12631457,E=12631461,C=29)
	CT(50440)=(S=12631462,E=12631466,C=14)
	CT(50441)=(S=12631468,E=12631497,C=29)
	CT(50442)=(S=12631498,E=12631499,C=14)
	CT(50443)=(S=12631500,E=12631500,C=78)
	CT(50444)=(S=12631501,E=12631503,C=29)
	CT(50445)=(S=12631504,E=12631507,C=0)
	CT(50446)=(S=12631508,E=12631518,C=29)
	CT(50447)=(S=12631519,E=12631519,C=10)
	CT(50448)=(S=12631520,E=12631524,C=29)
	CT(50449)=(S=12631525,E=12631525,C=0)
	CT(50450)=(S=12631530,E=12631546,C=29)
	CT(50451)=(S=12631547,E=12631547,C=28)
	CT(50452)=(S=12631548,E=12631551,C=29)
	CT(50453)=(S=12631552,E=12631552,C=78)
	CT(50454)=(S=12631553,E=12631581,C=29)
	CT(50455)=(S=12631582,E=12631583,C=204)
	CT(50456)=(S=12631584,E=12631595,C=29)
	CT(50457)=(S=12631596,E=12631596,C=14)
	CT(50458)=(S=12631597,E=12631609,C=29)
	CT(50459)=(S=12631610,E=12631611,C=27)
	CT(50460)=(S=12631612,E=12631612,C=29)
	CT(50461)=(S=12631613,E=12631613,C=0)
	CT(50462)=(S=12631614,E=12631615,C=29)
	CT(50463)=(S=12631616,E=12631616,C=14)
	CT(50464)=(S=12631617,E=12631617,C=29)
	CT(50465)=(S=12631618,E=12631618,C=0)
	CT(50466)=(S=12631619,E=12631619,C=14)
	CT(50467)=(S=12631620,E=12631620,C=29)
	CT(50468)=(S=12631621,E=12631621,C=12)
	CT(50469)=(S=12631622,E=12631626,C=29)
	CT(50470)=(S=12631627,E=12631627,C=78)
	CT(50471)=(S=12631628,E=12631646,C=29)
	CT(50472)=(S=12631647,E=12631647,C=27)
	CT(50473)=(S=12631648,E=12631659,C=29)
	CT(50474)=(S=12631660,E=12631660,C=73)
	CT(50475)=(S=12631661,E=12631680,C=29)
	CT(50476)=(S=12631681,E=12631684,C=12)
	CT(50477)=(S=12631685,E=12631724,C=29)
	CT(50478)=(S=12631725,E=12631725,C=50)
	CT(50479)=(S=12631726,E=12631726,C=21)
	CT(50480)=(S=12631727,E=12631731,C=29)
	CT(50481)=(S=12631732,E=12631733,C=0)
	CT(50482)=(S=12631734,E=12631735,C=14)
	CT(50483)=(S=12631736,E=12631741,C=29)
	CT(50484)=(S=12631742,E=12631745,C=27)
	CT(50485)=(S=12631746,E=12631752,C=29)
	CT(50486)=(S=12631753,E=12631754,C=14)
	CT(50487)=(S=12631755,E=12631755,C=29)
	CT(50488)=(S=12631756,E=12631757,C=7)
	CT(50489)=(S=12631758,E=12631759,C=29)
	CT(50490)=(S=12631760,E=12631767,C=0)
	CT(50491)=(S=12631768,E=12631784,C=29)
	CT(50492)=(S=12631785,E=12631788,C=12)
	CT(50493)=(S=12631789,E=12631791,C=29)
	CT(50494)=(S=12631792,E=12631792,C=14)
	CT(50495)=(S=12631793,E=12631793,C=29)
	CT(50496)=(S=12631794,E=12631794,C=12)
	CT(50497)=(S=12631795,E=12631798,C=29)
	CT(50498)=(S=12631799,E=12631801,C=14)
	CT(50499)=(S=12631802,E=12632063,C=29)
	CT(50500)=(S=12632064,E=12632319,C=9)
	CT(50501)=(S=12632320,E=12632575,C=29)
	CT(50502)=(S=12632576,E=12632831,C=47)
	CT(50503)=(S=12632832,E=12632832,C=29)
	CT(50504)=(S=12632833,E=12632833,C=14)
	CT(50505)=(S=12632835,E=12632835,C=78)
	CT(50506)=(S=12632836,E=12632839,C=29)
	CT(50507)=(S=12632840,E=12632840,C=18)
	CT(50508)=(S=12632841,E=12632843,C=29)
	CT(50509)=(S=12632844,E=12632846,C=73)
	CT(50510)=(S=12632847,E=12632865,C=29)
	CT(50511)=(S=12632866,E=12632866,C=0)
	CT(50512)=(S=12632867,E=12632870,C=29)
	CT(50513)=(S=12632871,E=12632872,C=6)
	CT(50514)=(S=12632873,E=12632873,C=29)
	CT(50515)=(S=12632874,E=12632874,C=63)
	CT(50516)=(S=12632875,E=12632875,C=14)
	CT(50517)=(S=12632876,E=12632887,C=0)
	CT(50518)=(S=12632890,E=12632903,C=29)
	CT(50519)=(S=12632904,E=12632904,C=58)
	CT(50520)=(S=12632905,E=12632929,C=29)
	CT(50521)=(S=12632930,E=12632930,C=28)
	CT(50522)=(S=12632931,E=12632933,C=29)
	CT(50523)=(S=12632934,E=12632934,C=0)
	CT(50524)=(S=12632935,E=12632936,C=29)
	CT(50525)=(S=12632937,E=12632937,C=14)
	CT(50526)=(S=12632938,E=12632938,C=58)
	CT(50527)=(S=12632939,E=12632941,C=29)
	CT(50528)=(S=12632942,E=12632942,C=15)
	CT(50529)=(S=12632943,E=12632943,C=29)
	CT(50530)=(S=12632944,E=12632944,C=0)
	CT(50531)=(S=12632945,E=12632947,C=29)
	CT(50532)=(S=12632948,E=12632950,C=14)
	CT(50533)=(S=12632951,E=12632965,C=29)
	CT(50534)=(S=12632966,E=12632980,C=19)
	CT(50535)=(S=12632981,E=12632983,C=29)
	CT(50536)=(S=12632984,E=12632984,C=0)
	CT(50537)=(S=12632985,E=12633001,C=29)
	CT(50538)=(S=12633002,E=12633002,C=0)
	CT(50539)=(S=12633003,E=12633014,C=29)
	CT(50540)=(S=12633015,E=12633015,C=36)
	CT(50541)=(S=12633016,E=12633016,C=28)
	CT(50542)=(S=12633017,E=12633026,C=29)
	CT(50543)=(S=12633027,E=12633027,C=21)
	CT(50544)=(S=12633028,E=12633028,C=29)
	CT(50545)=(S=12633029,E=12633029,C=0)
	CT(50546)=(S=12633030,E=12633034,C=29)
	CT(50547)=(S=12633035,E=12633035,C=78)
	CT(50548)=(S=12633036,E=12633050,C=29)
	CT(50549)=(S=12633051,E=12633051,C=73)
	CT(50550)=(S=12633052,E=12633067,C=29)
	CT(50551)=(S=12633068,E=12633068,C=14)
	CT(50552)=(S=12633069,E=12633069,C=189)
	CT(50553)=(S=12633070,E=12633082,C=29)
	CT(50554)=(S=12633083,E=12633083,C=78)
	CT(50555)=(S=12633084,E=12633086,C=29)
	CT(50556)=(S=12633087,E=12633087,C=78)
	CT(50557)=(S=12633088,E=12633088,C=29)
	CT(50558)=(S=12633089,E=12633243,C=12)
	CT(50559)=(S=12633244,E=12633343,C=29)
	CT(50560)=(S=12633344,E=12633401,C=78)
	CT(50561)=(S=12633402,E=12633403,C=29)
	CT(50562)=(S=12633404,E=12633432,C=78)
	CT(50563)=(S=12633433,E=12633433,C=29)
	CT(50564)=(S=12633434,E=12633435,C=78)
	CT(50565)=(S=12633436,E=12633437,C=29)
	CT(50566)=(S=12633438,E=12633465,C=78)
	CT(50567)=(S=12633466,E=12633467,C=29)
	CT(50568)=(S=12633468,E=12633487,C=78)
	CT(50569)=(S=12633488,E=12633491,C=29)
	CT(50570)=(S=12633492,E=12633499,C=78)
	CT(50571)=(S=12633500,E=12633504,C=29)
	CT(50572)=(S=12633505,E=12633542,C=78)
	CT(50573)=(S=12633543,E=12633544,C=29)
	CT(50574)=(S=12633545,E=12633550,C=78)
	CT(50575)=(S=12633551,E=12633551,C=29)
	CT(50576)=(S=12633552,E=12633563,C=78)
	CT(50577)=(S=12633564,E=12633568,C=29)
	CT(50578)=(S=12633569,E=12633588,C=78)
	CT(50579)=(S=12633589,E=12633589,C=29)
	CT(50580)=(S=12633590,E=12633598,C=78)
	CT(50581)=(S=12633599,E=12633607,C=29)
	CT(50582)=(S=12633608,E=12633615,C=83)
	CT(50583)=(S=12633616,E=12633855,C=29)
	CT(50584)=(S=12633856,E=12633856,C=78)
	CT(50585)=(S=12633857,E=12633867,C=29)
	CT(50586)=(S=12633868,E=12633871,C=78)
	CT(50587)=(S=12633872,E=12633887,C=21)
	CT(50588)=(S=12633888,E=12633903,C=0)
	CT(50589)=(S=12633904,E=12633919,C=78)
	CT(50590)=(S=12633920,E=12634047,C=29)
	CT(50591)=(S=12634048,E=12634063,C=78)
	CT(50592)=(S=12634064,E=12634175,C=29)
	CT(50593)=(S=12634176,E=12634191,C=73)
	CT(50594)=(S=12634192,E=12634879,C=29)
	CT(50595)=(S=12634880,E=12634880,C=80)
	CT(50596)=(S=12634881,E=12634916,C=29)
	CT(50597)=(S=12634917,E=12634917,C=179)
	CT(50598)=(S=12634920,E=12634959,C=29)
	CT(50599)=(S=12634960,E=12634960,C=18)
	CT(50600)=(S=12634976,E=12634985,C=29)
	CT(50601)=(S=12634986,E=12634987,C=78)
	CT(50602)=(S=12634988,E=12634989,C=47)
	CT(50603)=(S=12634990,E=12635017,C=29)
	CT(50604)=(S=12635018,E=12635026,C=6)
	CT(50605)=(S=12635027,E=12635033,C=29)
	CT(50606)=(S=12635034,E=12635034,C=73)
	CT(50607)=(S=12635035,E=12635035,C=29)
	CT(50608)=(S=12635038,E=12635038,C=203)
	CT(50609)=(S=12635039,E=12635055,C=29)
	CT(50610)=(S=12635056,E=12635056,C=0)
	CT(50611)=(S=12635057,E=12635057,C=204)
	CT(50612)=(S=12635058,E=12635076,C=29)
	CT(50613)=(S=12635077,E=12635077,C=78)
	CT(50614)=(S=12635078,E=12635082,C=29)
	CT(50615)=(S=12635083,E=12635083,C=78)
	CT(50616)=(S=12635084,E=12635092,C=29)
	CT(50617)=(S=12635093,E=12635093,C=0)
	CT(50618)=(S=12635094,E=12635095,C=29)
	CT(50619)=(S=12635096,E=12635096,C=73)
	CT(50620)=(S=12635097,E=12635106,C=29)
	CT(50621)=(S=12635107,E=12635107,C=23)
	CT(50622)=(S=12635108,E=12635111,C=29)
	CT(50623)=(S=12635112,E=12635112,C=87)
	CT(50624)=(S=12635113,E=12635114,C=29)
	CT(50625)=(S=12635115,E=12635115,C=0)
	CT(50626)=(S=12635116,E=12635118,C=29)
	CT(50627)=(S=12635119,E=12635119,C=78)
	CT(50628)=(S=12635120,E=12635126,C=29)
	CT(50629)=(S=12635127,E=12635127,C=3)
	CT(50630)=(S=12635128,E=12635648,C=29)
	CT(50631)=(S=12635649,E=12635649,C=78)
	CT(50632)=(S=12635650,E=12635651,C=29)
	CT(50633)=(S=12635652,E=12635655,C=78)
	CT(50634)=(S=12635656,E=12635675,C=29)
	CT(50635)=(S=12635676,E=12635676,C=78)
	CT(50636)=(S=12635677,E=12635701,C=29)
	CT(50637)=(S=12635702,E=12635703,C=78)
	CT(50638)=(S=12635704,E=12635724,C=29)
	CT(50639)=(S=12635725,E=12635729,C=14)
	CT(50640)=(S=12635730,E=12635733,C=29)
	CT(50641)=(S=12635734,E=12635734,C=17)
	CT(50642)=(S=12635735,E=12635740,C=29)
	CT(50643)=(S=12635741,E=12635742,C=78)
	CT(50644)=(S=12635744,E=12635787,C=29)
	CT(50645)=(S=12635792,E=12635799,C=78)
	CT(50646)=(S=12635800,E=12635805,C=73)
	CT(50647)=(S=12635806,E=12635828,C=29)
	CT(50648)=(S=12635829,E=12635829,C=78)
	CT(50649)=(S=12635830,E=12635864,C=29)
	CT(50650)=(S=12635865,E=12635866,C=78)
	CT(50651)=(S=12635867,E=12635869,C=29)
	CT(50652)=(S=12635870,E=12635874,C=17)
	CT(50653)=(S=12635875,E=12635915,C=29)
	CT(50654)=(S=12635916,E=12635916,C=78)
	CT(50655)=(S=12635917,E=12635917,C=29)
	CT(50656)=(S=12635918,E=12635918,C=14)
	CT(50657)=(S=12635919,E=12635934,C=29)
	CT(50658)=(S=12635935,E=12635935,C=36)
	CT(50659)=(S=12635936,E=12635936,C=29)
	CT(50660)=(S=12635937,E=12635937,C=0)
	CT(50661)=(S=12635938,E=12635939,C=29)
	CT(50662)=(S=12635940,E=12635940,C=0)
	CT(50663)=(S=12635941,E=12635943,C=29)
	CT(50664)=(S=12635944,E=12635946,C=213)
	CT(50665)=(S=12635947,E=12635951,C=29)
	CT(50666)=(S=12635952,E=12635952,C=80)
	CT(50667)=(S=12635953,E=12635963,C=29)
	CT(50668)=(S=12635964,E=12635965,C=78)
	CT(50669)=(S=12635966,E=12635967,C=29)
	CT(50670)=(S=12635968,E=12635968,C=3)
	CT(50671)=(S=12635969,E=12635972,C=213)
	CT(50672)=(S=12635973,E=12635979,C=29)
	CT(50673)=(S=12635980,E=12635980,C=78)
	CT(50674)=(S=12635981,E=12635994,C=29)
	CT(50675)=(S=12635995,E=12635995,C=8)
	CT(50676)=(S=12635996,E=12635999,C=29)
	CT(50677)=(S=12636000,E=12636002,C=0)
	CT(50678)=(S=12636003,E=12636016,C=29)
	CT(50679)=(S=12636017,E=12636017,C=0)
	CT(50680)=(S=12636018,E=12636022,C=29)
	CT(50681)=(S=12636023,E=12636025,C=0)
	CT(50682)=(S=12636026,E=12636041,C=29)
	CT(50683)=(S=12636042,E=12636042,C=78)
	CT(50684)=(S=12636043,E=12636044,C=29)
	CT(50685)=(S=12636045,E=12636045,C=14)
	CT(50686)=(S=12636046,E=12636046,C=70)
	CT(50687)=(S=12636047,E=12636047,C=73)
	CT(50688)=(S=12636048,E=12636049,C=29)
	CT(50689)=(S=12636050,E=12636050,C=203)
	CT(50690)=(S=12636051,E=12636054,C=29)
	CT(50691)=(S=12636055,E=12636062,C=0)
	CT(50692)=(S=12636063,E=12636064,C=29)
	CT(50693)=(S=12636065,E=12636065,C=0)
	CT(50694)=(S=12636066,E=12636068,C=29)
	CT(50695)=(S=12636069,E=12636069,C=0)
	CT(50696)=(S=12636070,E=12636086,C=29)
	CT(50697)=(S=12636087,E=12636087,C=215)
	CT(50698)=(S=12636088,E=12636090,C=29)
	CT(50699)=(S=12636091,E=12636091,C=0)
	CT(50700)=(S=12636092,E=12636097,C=29)
	CT(50701)=(S=12636098,E=12636099,C=189)
	CT(50702)=(S=12636100,E=12636100,C=21)
	CT(50703)=(S=12636101,E=12636102,C=0)
	CT(50704)=(S=12636103,E=12636103,C=29)
	CT(50705)=(S=12636104,E=12636107,C=189)
	CT(50706)=(S=12636108,E=12636109,C=204)
	CT(50707)=(S=12636110,E=12636110,C=189)
	CT(50708)=(S=12636111,E=12636145,C=29)
	CT(50709)=(S=12636146,E=12636146,C=3)
	CT(50710)=(S=12636147,E=12636147,C=29)
	CT(50711)=(S=12636148,E=12636148,C=213)
	CT(50712)=(S=12636149,E=12636151,C=29)
	CT(50713)=(S=12636152,E=12636152,C=0)
	CT(50714)=(S=12636153,E=12636160,C=29)
	CT(50715)=(S=12636161,E=12636175,C=78)
	CT(50716)=(S=12636176,E=12636672,C=29)
	CT(50717)=(S=12636673,E=12636682,C=78)
	CT(50718)=(S=12636683,E=12636735,C=29)
	CT(50719)=(S=12636736,E=12636799,C=78)
	CT(50720)=(S=12636800,E=12636973,C=29)
	CT(50721)=(S=12636974,E=12636975,C=78)
	CT(50722)=(S=12636976,E=12637051,C=29)
	CT(50723)=(S=12637052,E=12637055,C=78)
	CT(50724)=(S=12637056,E=12637795,C=29)
	CT(50725)=(S=12637796,E=12637799,C=78)
	CT(50726)=(S=12637800,E=12637807,C=29)
	CT(50727)=(S=12637808,E=12637823,C=79)
	CT(50728)=(S=12637824,E=12637951,C=78)
	CT(50729)=(S=12637952,E=12638719,C=29)
	CT(50730)=(S=12638720,E=12638975,C=2)
	CT(50731)=(S=12638976,E=12639230,C=78)
	CT(50732)=(S=12639231,E=12639744,C=29)
	CT(50733)=(S=12639745,E=12639752,C=78)
	CT(50734)=(S=12639753,E=12639845,C=29)
	CT(50735)=(S=12639846,E=12639847,C=78)
	CT(50736)=(S=12639848,E=12639871,C=29)
	CT(50737)=(S=12639872,E=12639999,C=78)
	CT(50738)=(S=12640000,E=12640011,C=29)
	CT(50739)=(S=12640012,E=12640015,C=78)
	CT(50740)=(S=12640016,E=12640063,C=29)
	CT(50741)=(S=12640064,E=12640127,C=201)
	CT(50742)=(S=12640128,E=12640799,C=29)
	CT(50743)=(S=12640800,E=12640831,C=78)
	CT(50744)=(S=12640832,E=12640895,C=29)
	CT(50745)=(S=12640896,E=12641023,C=78)
	CT(50746)=(S=12641024,E=12641024,C=29)
	CT(50747)=(S=12641025,E=12641039,C=10)
	CT(50748)=(S=12641040,E=12641280,C=29)
	CT(50749)=(S=12641281,E=12641311,C=78)
	CT(50750)=(S=12641312,E=12641407,C=29)
	CT(50751)=(S=12641408,E=12641535,C=5)
	CT(50752)=(S=12641536,E=12641559,C=29)
	CT(50753)=(S=12641560,E=12641567,C=78)
	CT(50754)=(S=12641568,E=12642052,C=29)
	CT(50755)=(S=12642053,E=12642053,C=78)
	CT(50756)=(S=12642054,E=12642062,C=29)
	CT(50757)=(S=12642063,E=12642063,C=73)
	CT(50758)=(S=12642064,E=12642081,C=29)
	CT(50759)=(S=12642082,E=12642082,C=0)
	CT(50760)=(S=12642083,E=12642083,C=29)
	CT(50761)=(S=12642084,E=12642084,C=81)
	CT(50762)=(S=12642085,E=12642106,C=29)
	CT(50763)=(S=12642107,E=12642107,C=0)
	CT(50764)=(S=12642108,E=12642113,C=29)
	CT(50765)=(S=12642114,E=12642114,C=78)
	CT(50766)=(S=12642115,E=12642115,C=42)
	CT(50767)=(S=12642116,E=12642129,C=29)
	CT(50768)=(S=12642130,E=12642130,C=28)
	CT(50769)=(S=12642131,E=12642136,C=29)
	CT(50770)=(S=12642137,E=12642137,C=78)
	CT(50771)=(S=12642138,E=12642139,C=29)
	CT(50772)=(S=12642140,E=12642145,C=80)
	CT(50773)=(S=12642146,E=12642157,C=29)
	CT(50774)=(S=12642158,E=12642158,C=203)
	CT(50775)=(S=12642159,E=12642161,C=29)
	CT(50776)=(S=12642162,E=12642163,C=189)
	CT(50777)=(S=12642164,E=12642167,C=181)
	CT(50778)=(S=12642168,E=12642169,C=83)
	CT(50779)=(S=12642170,E=12642174,C=29)
	CT(50780)=(S=12642175,E=12642175,C=0)
	CT(50781)=(S=12642176,E=12642182,C=29)
	CT(50782)=(S=12642183,E=12642184,C=0)
	CT(50783)=(S=12642186,E=12642187,C=29)
	CT(50784)=(S=12642188,E=12642188,C=203)
	CT(50785)=(S=12642189,E=12642213,C=29)
	CT(50786)=(S=12642214,E=12642216,C=0)
	CT(50787)=(S=12642217,E=12642218,C=7)
	CT(50788)=(S=12642219,E=12642219,C=29)
	CT(50789)=(S=12642220,E=12642222,C=78)
	CT(50790)=(S=12642223,E=12642224,C=189)
	CT(50791)=(S=12642225,E=12642235,C=29)
	CT(50792)=(S=12642236,E=12642236,C=78)
	CT(50793)=(S=12642237,E=12642240,C=29)
	CT(50794)=(S=12642241,E=12642241,C=0)
	CT(50795)=(S=12642242,E=12642250,C=29)
	CT(50796)=(S=12642251,E=12642251,C=0)
	CT(50797)=(S=12642252,E=12642258,C=29)
	CT(50798)=(S=12642260,E=12642260,C=0)
	CT(50799)=(S=12642261,E=12642284,C=29)
	CT(50800)=(S=12642285,E=12642285,C=113)
	CT(50801)=(S=12642287,E=12642294,C=29)
	CT(50802)=(S=12642295,E=12642295,C=78)
	CT(50803)=(S=12642297,E=12642301,C=29)
	CT(50804)=(S=12642302,E=12642302,C=105)
	CT(50805)=(S=12642303,E=12642431,C=29)
	CT(50806)=(S=12642432,E=12642463,C=0)
	CT(50807)=(S=12642464,E=12642815,C=29)
	CT(50808)=(S=12642816,E=12642816,C=78)
	CT(50809)=(S=12642817,E=12642838,C=29)
	CT(50810)=(S=12642839,E=12642839,C=78)
	CT(50811)=(S=12642840,E=12642913,C=29)
	CT(50812)=(S=12642914,E=12642914,C=78)
	CT(50813)=(S=12642915,E=12642986,C=29)
	CT(50814)=(S=12642987,E=12642987,C=78)
	CT(50815)=(S=12642988,E=12643011,C=29)
	CT(50816)=(S=12643012,E=12643015,C=78)
	CT(50817)=(S=12643016,E=12643019,C=29)
	CT(50818)=(S=12643020,E=12643020,C=78)
	CT(50819)=(S=12643021,E=12643038,C=29)
	CT(50820)=(S=12643039,E=12643045,C=78)
	CT(50821)=(S=12643046,E=12643060,C=29)
	CT(50822)=(S=12643061,E=12643061,C=0)
	CT(50823)=(S=12643062,E=12643069,C=29)
	CT(50824)=(S=12643070,E=12643070,C=78)
	CT(50825)=(S=12643071,E=12643119,C=29)
	CT(50826)=(S=12643120,E=12643135,C=79)
	CT(50827)=(S=12643136,E=12643199,C=29)
	CT(50828)=(S=12643200,E=12643327,C=78)
	CT(50829)=(S=12643328,E=12643339,C=29)
	CT(50830)=(S=12643340,E=12643343,C=78)
	CT(50831)=(S=12643344,E=12643583,C=29)
	CT(50832)=(S=12643584,E=12643615,C=78)
	CT(50833)=(S=12643616,E=12643840,C=29)
	CT(50834)=(S=12643841,E=12643850,C=34)
	CT(50835)=(S=12643851,E=12643855,C=78)
	CT(50836)=(S=12643856,E=12644367,C=29)
	CT(50837)=(S=12644368,E=12644383,C=0)
	CT(50838)=(S=12644384,E=12644615,C=29)
	CT(50839)=(S=12644616,E=12644623,C=78)
	CT(50840)=(S=12644624,E=12645167,C=29)
	CT(50841)=(S=12645168,E=12645183,C=182)
	CT(50842)=(S=12645184,E=12645231,C=29)
	CT(50843)=(S=12645232,E=12645247,C=78)
	CT(50844)=(S=12645248,E=12645333,C=29)
	CT(50845)=(S=12645334,E=12645335,C=78)
	CT(50846)=(S=12645336,E=12645375,C=29)
	CT(50847)=(S=12645376,E=12645631,C=2)
	CT(50848)=(S=12645632,E=12645644,C=29)
	CT(50849)=(S=12645645,E=12645646,C=0)
	CT(50850)=(S=12645647,E=12645655,C=29)
	CT(50851)=(S=12645656,E=12645656,C=78)
	CT(50852)=(S=12645657,E=12645661,C=29)
	CT(50853)=(S=12645662,E=12645662,C=189)
	CT(50854)=(S=12645663,E=12645665,C=29)
	CT(50855)=(S=12645668,E=12645668,C=73)
	CT(50856)=(S=12645669,E=12645682,C=29)
	CT(50857)=(S=12645683,E=12645683,C=78)
	CT(50858)=(S=12645684,E=12645689,C=29)
	CT(50859)=(S=12645690,E=12645691,C=7)
	CT(50860)=(S=12645692,E=12645692,C=203)
	CT(50861)=(S=12645693,E=12645722,C=29)
	CT(50862)=(S=12645723,E=12645723,C=78)
	CT(50863)=(S=12645724,E=12645736,C=29)
	CT(50864)=(S=12645737,E=12645744,C=0)
	CT(50865)=(S=12645745,E=12645752,C=29)
	CT(50866)=(S=12645753,E=12645753,C=201)
	CT(50867)=(S=12645754,E=12645779,C=29)
	CT(50868)=(S=12645780,E=12645780,C=135)
	CT(50869)=(S=12645781,E=12645783,C=29)
	CT(50870)=(S=12645784,E=12645784,C=31)
	CT(50871)=(S=12645785,E=12645791,C=29)
	CT(50872)=(S=12645792,E=12645793,C=78)
	CT(50873)=(S=12645794,E=12645800,C=29)
	CT(50874)=(S=12645801,E=12645801,C=58)
	CT(50875)=(S=12645802,E=12645803,C=29)
	CT(50876)=(S=12645804,E=12645804,C=78)
	CT(50877)=(S=12645805,E=12645814,C=29)
	CT(50878)=(S=12645815,E=12645815,C=78)
	CT(50879)=(S=12645816,E=12645816,C=29)
	CT(50880)=(S=12645817,E=12645817,C=73)
	CT(50881)=(S=12645819,E=12645823,C=29)
	CT(50882)=(S=12645824,E=12645824,C=0)
	CT(50883)=(S=12645825,E=12645826,C=29)
	CT(50884)=(S=12645828,E=12645828,C=8)
	CT(50885)=(S=12645829,E=12645839,C=29)
	CT(50886)=(S=12645840,E=12645840,C=8)
	CT(50887)=(S=12645841,E=12645841,C=29)
	CT(50888)=(S=12645842,E=12645849,C=0)
	CT(50889)=(S=12645850,E=12645856,C=29)
	CT(50890)=(S=12645857,E=12645857,C=14)
	CT(50891)=(S=12645858,E=12645859,C=29)
	CT(50892)=(S=12645860,E=12645860,C=78)
	CT(50893)=(S=12645861,E=12645880,C=29)
	CT(50894)=(S=12645881,E=12645883,C=6)
	CT(50895)=(S=12645884,E=12646143,C=29)
	CT(50896)=(S=12646144,E=12646144,C=78)
	CT(50897)=(S=12646145,E=12646154,C=14)
	CT(50898)=(S=12646155,E=12646155,C=29)
	CT(50899)=(S=12646156,E=12646159,C=78)
	CT(50900)=(S=12646160,E=12646399,C=29)
	CT(50901)=(S=12646400,E=12646527,C=156)
	CT(50902)=(S=12646528,E=12646671,C=29)
	CT(50903)=(S=12646672,E=12646687,C=6)
	CT(50904)=(S=12646688,E=12646751,C=29)
	CT(50905)=(S=12646752,E=12646767,C=78)
	CT(50906)=(S=12646768,E=12646783,C=29)
	CT(50907)=(S=12646784,E=12646911,C=78)
	CT(50908)=(S=12646912,E=12646927,C=29)
	CT(50909)=(S=12646928,E=12646931,C=78)
	CT(50910)=(S=12646932,E=12646935,C=29)
	CT(50911)=(S=12646936,E=12646940,C=78)
	CT(50912)=(S=12646941,E=12647167,C=29)
	CT(50913)=(S=12647168,E=12647168,C=78)
	CT(50914)=(S=12647169,E=12647193,C=29)
	CT(50915)=(S=12647194,E=12647198,C=78)
	CT(50916)=(S=12647199,E=12647206,C=29)
	CT(50917)=(S=12647207,E=12647207,C=203)
	CT(50918)=(S=12647208,E=12647219,C=29)
	CT(50919)=(S=12647220,E=12647223,C=78)
	CT(50920)=(S=12647224,E=12647228,C=29)
	CT(50921)=(S=12647229,E=12647233,C=23)
	CT(50922)=(S=12647234,E=12647258,C=29)
	CT(50923)=(S=12647259,E=12647259,C=78)
	CT(50924)=(S=12647260,E=12647315,C=29)
	CT(50925)=(S=12647316,E=12647316,C=0)
	CT(50926)=(S=12647317,E=12647331,C=29)
	CT(50927)=(S=12647332,E=12647332,C=78)
	CT(50928)=(S=12647333,E=12647333,C=29)
	CT(50929)=(S=12647334,E=12647334,C=78)
	CT(50930)=(S=12647335,E=12647362,C=29)
	CT(50931)=(S=12647363,E=12647363,C=0)
	CT(50932)=(S=12647364,E=12647364,C=78)
	CT(50933)=(S=12647365,E=12647365,C=29)
	CT(50934)=(S=12647366,E=12647367,C=78)
	CT(50935)=(S=12647368,E=12647368,C=29)
	CT(50936)=(S=12647370,E=12647370,C=110)
	CT(50937)=(S=12647371,E=12647371,C=29)
	CT(50938)=(S=12647373,E=12647375,C=0)
	CT(50939)=(S=12647376,E=12647393,C=29)
	CT(50940)=(S=12647394,E=12647394,C=28)
	CT(50941)=(S=12647399,E=12647399,C=78)
	CT(50942)=(S=12647400,E=12647551,C=29)
	CT(50943)=(S=12647552,E=12647567,C=78)
	CT(50944)=(S=12647568,E=12647583,C=29)
	CT(50945)=(S=12647584,E=12647599,C=78)
	CT(50946)=(S=12647600,E=12647647,C=29)
	CT(50947)=(S=12647648,E=12647663,C=78)
	CT(50948)=(S=12647664,E=12647883,C=29)
	CT(50949)=(S=12647884,E=12647903,C=78)
	CT(50950)=(S=12647904,E=12648031,C=29)
	CT(50951)=(S=12648032,E=12648047,C=80)
	CT(50952)=(S=12648048,E=12648447,C=29)
	CT(50953)=(S=12648448,E=12648479,C=36)
	CT(50954)=(S=12648480,E=12648503,C=34)
	CT(50955)=(S=12648504,E=12648508,C=24)
	CT(50956)=(S=12648509,E=12648509,C=31)
	CT(50957)=(S=12648510,E=12648510,C=12)
	CT(50958)=(S=12648511,E=12648511,C=28)
	CT(50959)=(S=12648512,E=12648576,C=58)
	CT(50960)=(S=12648577,E=12648583,C=18)
	CT(50961)=(S=12648584,E=12648589,C=60)
	CT(50962)=(S=12648590,E=12648591,C=31)
	CT(50963)=(S=12648592,E=12648592,C=58)
	CT(50964)=(S=12648593,E=12648603,C=18)
	CT(50965)=(S=12648604,E=12648604,C=26)
	CT(50966)=(S=12648605,E=12648605,C=15)
	CT(50967)=(S=12648606,E=12648607,C=14)
	CT(50968)=(S=12648608,E=12648608,C=36)
	CT(50969)=(S=12648609,E=12648609,C=24)
	CT(50970)=(S=12648610,E=12648611,C=20)
	CT(50971)=(S=12648612,E=12648613,C=47)
	CT(50972)=(S=12648614,E=12648623,C=18)
	CT(50973)=(S=12648624,E=12648625,C=14)
	CT(50974)=(S=12648626,E=12648627,C=18)
	CT(50975)=(S=12648628,E=12648631,C=34)
	CT(50976)=(S=12648632,E=12648633,C=18)
	CT(50977)=(S=12648634,E=12648635,C=28)
	CT(50978)=(S=12648636,E=12648636,C=42)
	CT(50979)=(S=12648637,E=12648637,C=14)
	CT(50980)=(S=12648638,E=12648638,C=38)
	CT(50981)=(S=12648639,E=12648639,C=14)
	CT(50982)=(S=12648640,E=12648645,C=58)
	CT(50983)=(S=12648646,E=12648646,C=18)
	CT(50984)=(S=12648647,E=12648647,C=28)
	CT(50985)=(S=12648648,E=12648651,C=18)
	CT(50986)=(S=12648652,E=12648655,C=31)
	CT(50987)=(S=12648656,E=12648659,C=58)
	CT(50988)=(S=12648661,E=12648661,C=51)
	CT(50989)=(S=12648662,E=12648663,C=18)
	CT(50990)=(S=12648664,E=12648669,C=31)
	CT(50991)=(S=12648670,E=12648670,C=27)
	CT(50992)=(S=12648671,E=12648671,C=31)
	CT(50993)=(S=12648672,E=12648672,C=34)
	CT(50994)=(S=12648673,E=12648673,C=38)
	CT(50995)=(S=12648674,E=12648674,C=50)
	CT(50996)=(S=12648676,E=12648676,C=31)
	CT(50997)=(S=12648679,E=12648679,C=33)
	CT(50998)=(S=12648680,E=12648681,C=34)
	CT(50999)=(S=12648684,E=12648684,C=58)
	CT(51000)=(S=12648685,E=12648685,C=34)
	CT(51001)=(S=12648686,E=12648686,C=28)
	CT(51002)=(S=12648688,E=12648688,C=31)
	CT(51003)=(S=12648689,E=12648689,C=25)
	CT(51004)=(S=12648690,E=12648690,C=58)
	CT(51005)=(S=12648691,E=12648691,C=24)
	CT(51006)=(S=12648692,E=12648692,C=52)
	CT(51007)=(S=12648694,E=12648694,C=28)
	CT(51008)=(S=12648695,E=12648695,C=31)
	CT(51009)=(S=12648696,E=12648696,C=28)
	CT(51010)=(S=12648697,E=12648697,C=14)
	CT(51011)=(S=12648698,E=12648698,C=28)
	CT(51012)=(S=12648699,E=12648699,C=44)
	CT(51013)=(S=12648700,E=12648700,C=27)
	CT(51014)=(S=12648701,E=12648701,C=19)
	CT(51015)=(S=12648702,E=12648702,C=42)
	CT(51016)=(S=12648703,E=12648703,C=14)
	CT(51017)=(S=12648704,E=12648959,C=63)
	CT(51018)=(S=12648960,E=12649215,C=52)
	CT(51019)=(S=12649216,E=12649471,C=24)
	CT(51020)=(S=12649472,E=12649727,C=48)
	CT(51021)=(S=12649728,E=12649983,C=34)
	CT(51022)=(S=12649984,E=12650239,C=50)
	CT(51023)=(S=12650240,E=12650399,C=28)
	CT(51024)=(S=12650400,E=12650407,C=18)
	CT(51025)=(S=12650408,E=12650495,C=28)
	CT(51026)=(S=12650496,E=12650496,C=18)
	CT(51027)=(S=12650505,E=12650527,C=34)
	CT(51028)=(S=12650528,E=12650531,C=19)
	CT(51029)=(S=12650532,E=12650535,C=24)
	CT(51030)=(S=12650536,E=12650540,C=34)
	CT(51031)=(S=12650541,E=12650541,C=58)
	CT(51032)=(S=12650542,E=12650543,C=18)
	CT(51033)=(S=12650544,E=12650549,C=34)
	CT(51034)=(S=12650553,E=12650553,C=58)
	CT(51035)=(S=12650554,E=12650556,C=34)
	CT(51036)=(S=12650557,E=12650557,C=58)
	CT(51037)=(S=12650560,E=12650567,C=34)
	CT(51038)=(S=12650570,E=12650571,C=18)
	CT(51039)=(S=12650572,E=12650573,C=34)
	CT(51040)=(S=12650574,E=12650575,C=31)
	CT(51041)=(S=12650582,E=12650583,C=33)
	CT(51042)=(S=12650584,E=12650607,C=34)
	CT(51043)=(S=12650622,E=12650622,C=19)
	CT(51044)=(S=12650624,E=12650633,C=34)
	CT(51045)=(S=12650635,E=12650635,C=26)
	CT(51046)=(S=12650636,E=12650677,C=34)
	CT(51047)=(S=12650678,E=12650679,C=58)
	CT(51048)=(S=12650684,E=12650685,C=28)
	CT(51049)=(S=12650686,E=12650696,C=34)
	CT(51050)=(S=12650697,E=12650697,C=31)
	CT(51051)=(S=12650699,E=12650699,C=18)
	CT(51052)=(S=12650700,E=12650741,C=34)
	CT(51053)=(S=12650742,E=12650742,C=44)
	CT(51054)=(S=12650744,E=12650744,C=34)
	CT(51055)=(S=12650746,E=12650746,C=23)
	CT(51056)=(S=12650747,E=12650750,C=34)
	CT(51057)=(S=12650752,E=12650752,C=14)
	CT(51058)=(S=12650753,E=12650759,C=13)
	CT(51059)=(S=12650760,E=12650763,C=49)
	CT(51060)=(S=12650764,E=12650764,C=52)
	CT(51061)=(S=12650765,E=12650766,C=14)
	CT(51062)=(S=12650768,E=12650768,C=51)
	CT(51063)=(S=12650769,E=12650769,C=18)
	CT(51064)=(S=12650770,E=12650770,C=28)
	CT(51065)=(S=12650771,E=12650771,C=52)
	CT(51066)=(S=12650772,E=12650772,C=14)
	CT(51067)=(S=12650773,E=12650773,C=52)
	CT(51068)=(S=12650774,E=12650774,C=18)
	CT(51069)=(S=12650775,E=12650775,C=36)
	CT(51070)=(S=12650776,E=12650776,C=34)
	CT(51071)=(S=12650777,E=12650777,C=58)
	CT(51072)=(S=12650778,E=12650779,C=50)
	CT(51073)=(S=12650780,E=12650780,C=31)
	CT(51074)=(S=12650816,E=12650863,C=19)
	CT(51075)=(S=12650872,E=12650872,C=60)
	CT(51076)=(S=12650873,E=12650873,C=58)
	CT(51077)=(S=12650874,E=12650874,C=34)
	CT(51078)=(S=12650875,E=12650875,C=58)
	CT(51079)=(S=12650876,E=12650879,C=50)
	CT(51080)=(S=12650880,E=12650899,C=70)
	CT(51081)=(S=12650900,E=12650903,C=19)
	CT(51082)=(S=12650910,E=12650910,C=31)
	CT(51083)=(S=12650911,E=12650991,C=24)
	CT(51084)=(S=12650992,E=12650999,C=18)
	CT(51085)=(S=12651000,E=12651000,C=14)
	CT(51086)=(S=12651001,E=12651001,C=18)
	CT(51087)=(S=12651002,E=12651003,C=31)
	CT(51088)=(S=12651004,E=12651005,C=42)
	CT(51089)=(S=12651006,E=12651006,C=24)
	CT(51090)=(S=12651008,E=12652543,C=19)
	CT(51091)=(S=12652545,E=12652575,C=28)
	CT(51092)=(S=12652587,E=12652587,C=25)
	CT(51093)=(S=12652588,E=12652588,C=34)
	CT(51094)=(S=12652589,E=12652589,C=31)
	CT(51095)=(S=12652590,E=12652590,C=50)
	CT(51096)=(S=12652591,E=12652591,C=31)
	CT(51097)=(S=12652592,E=12652639,C=28)
	CT(51098)=(S=12652640,E=12652640,C=72)
	CT(51099)=(S=12652641,E=12652642,C=28)
	CT(51100)=(S=12652643,E=12652643,C=58)
	CT(51101)=(S=12652644,E=12652644,C=36)
	CT(51102)=(S=12652645,E=12652645,C=31)
	CT(51103)=(S=12652646,E=12652646,C=51)
	CT(51104)=(S=12652647,E=12652647,C=21)
	CT(51105)=(S=12652648,E=12652648,C=33)
	CT(51106)=(S=12652649,E=12652649,C=31)
	CT(51107)=(S=12652651,E=12652651,C=58)
	CT(51108)=(S=12652652,E=12652652,C=14)
	CT(51109)=(S=12652653,E=12652653,C=52)
	CT(51110)=(S=12652655,E=12652655,C=53)
	CT(51111)=(S=12652656,E=12652687,C=28)
	CT(51112)=(S=12652690,E=12652690,C=24)
	CT(51113)=(S=12652691,E=12652691,C=17)
	CT(51114)=(S=12652692,E=12652692,C=38)
	CT(51115)=(S=12652693,E=12652694,C=14)
	CT(51116)=(S=12652695,E=12652695,C=28)
	CT(51117)=(S=12652696,E=12652696,C=52)
	CT(51118)=(S=12652697,E=12652697,C=25)
	CT(51119)=(S=12652698,E=12652698,C=36)
	CT(51120)=(S=12652699,E=12652699,C=42)
	CT(51121)=(S=12652701,E=12652701,C=51)
	CT(51122)=(S=12652702,E=12652702,C=31)
	CT(51123)=(S=12652703,E=12652703,C=12)
	CT(51124)=(S=12652704,E=12652751,C=28)
	CT(51125)=(S=12652752,E=12652754,C=18)
	CT(51126)=(S=12652755,E=12652755,C=34)
	CT(51127)=(S=12652756,E=12652756,C=28)
	CT(51128)=(S=12652758,E=12652759,C=42)
	CT(51129)=(S=12652760,E=12652760,C=25)
	CT(51130)=(S=12652761,E=12652761,C=34)
	CT(51131)=(S=12652762,E=12652762,C=38)
	CT(51132)=(S=12652763,E=12652763,C=22)
	CT(51133)=(S=12652764,E=12652764,C=34)
	CT(51134)=(S=12652766,E=12652772,C=28)
	CT(51135)=(S=12652773,E=12652773,C=58)
	CT(51136)=(S=12652774,E=12652775,C=28)
	CT(51137)=(S=12652776,E=12652776,C=18)
	CT(51138)=(S=12652777,E=12652777,C=31)
	CT(51139)=(S=12652778,E=12652778,C=36)
	CT(51140)=(S=12652779,E=12652779,C=28)
	CT(51141)=(S=12652781,E=12652781,C=42)
	CT(51142)=(S=12652782,E=12652782,C=17)
	CT(51143)=(S=12652784,E=12652784,C=58)
	CT(51144)=(S=12652785,E=12652785,C=34)
	CT(51145)=(S=12652786,E=12652786,C=42)
	CT(51146)=(S=12652787,E=12652787,C=25)
	CT(51147)=(S=12652788,E=12652788,C=42)
	CT(51148)=(S=12652789,E=12652789,C=28)
	CT(51149)=(S=12652790,E=12652790,C=51)
	CT(51150)=(S=12652791,E=12652791,C=31)
	CT(51151)=(S=12652792,E=12652798,C=28)
	CT(51152)=(S=12652799,E=12652799,C=58)
	CT(51153)=(S=12652800,E=12652800,C=21)
	CT(51154)=(S=12652801,E=12652801,C=52)
	CT(51155)=(S=12652802,E=12652811,C=28)
	CT(51156)=(S=12652812,E=12652812,C=14)
	CT(51157)=(S=12652813,E=12652813,C=33)
	CT(51158)=(S=12652814,E=12652814,C=58)
	CT(51159)=(S=12652816,E=12652818,C=28)
	CT(51160)=(S=12652819,E=12652819,C=12)
	CT(51161)=(S=12652821,E=12652827,C=28)
	CT(51162)=(S=12652841,E=12652841,C=58)
	CT(51163)=(S=12652842,E=12652842,C=17)
	CT(51164)=(S=12652843,E=12652843,C=18)
	CT(51165)=(S=12652844,E=12652844,C=31)
	CT(51166)=(S=12652845,E=12652845,C=15)
	CT(51167)=(S=12652846,E=12652846,C=31)
	CT(51168)=(S=12652847,E=12652847,C=33)
	CT(51169)=(S=12652848,E=12652851,C=28)
	CT(51170)=(S=12652852,E=12652852,C=18)
	CT(51171)=(S=12652853,E=12652853,C=61)
	CT(51172)=(S=12652854,E=12652854,C=58)
	CT(51173)=(S=12652860,E=12652863,C=28)
	CT(51174)=(S=12652865,E=12652865,C=14)
	CT(51175)=(S=12652866,E=12652866,C=25)
	CT(51176)=(S=12652867,E=12652867,C=19)
	CT(51177)=(S=12652868,E=12652868,C=17)
	CT(51178)=(S=12652869,E=12652869,C=31)
	CT(51179)=(S=12652871,E=12652871,C=58)
	CT(51180)=(S=12652872,E=12652872,C=19)
	CT(51181)=(S=12652873,E=12652873,C=12)
	CT(51182)=(S=12652874,E=12652874,C=17)
	CT(51183)=(S=12652875,E=12652875,C=31)
	CT(51184)=(S=12652876,E=12652876,C=34)
	CT(51185)=(S=12652877,E=12652877,C=28)
	CT(51186)=(S=12652878,E=12652878,C=53)
	CT(51187)=(S=12652879,E=12652879,C=56)
	CT(51188)=(S=12652884,E=12652884,C=58)
	CT(51189)=(S=12652885,E=12652885,C=34)
	CT(51190)=(S=12652896,E=12652927,C=28)
	CT(51191)=(S=12652928,E=12652969,C=14)
	CT(51192)=(S=12652970,E=12652970,C=18)
	CT(51193)=(S=12652971,E=12652971,C=28)
	CT(51194)=(S=12652972,E=12652972,C=58)
	CT(51195)=(S=12652973,E=12652973,C=68)
	CT(51196)=(S=12652974,E=12652974,C=58)
	CT(51197)=(S=12652975,E=12652975,C=50)
	CT(51198)=(S=12652978,E=12652978,C=28)
	CT(51199)=(S=12652984,E=12652984,C=58)
	CT(51200)=(S=12652985,E=12652986,C=28)
	CT(51201)=(S=12652987,E=12652987,C=14)
	CT(51202)=(S=12652992,E=12652992,C=12)
	CT(51203)=(S=12652994,E=12652994,C=34)
	CT(51204)=(S=12652995,E=12652995,C=38)
	CT(51205)=(S=12652996,E=12652996,C=12)
	CT(51206)=(S=12652997,E=12652997,C=28)
	CT(51207)=(S=12652998,E=12652998,C=12)
	CT(51208)=(S=12652999,E=12652999,C=34)
	CT(51209)=(S=12653000,E=12653000,C=58)
	CT(51210)=(S=12653001,E=12653001,C=34)
	CT(51211)=(S=12653002,E=12653002,C=19)
	CT(51212)=(S=12653003,E=12653003,C=58)
	CT(51213)=(S=12653004,E=12653005,C=28)
	CT(51214)=(S=12653006,E=12653006,C=24)
	CT(51215)=(S=12653007,E=12653007,C=19)
	CT(51216)=(S=12653008,E=12653008,C=31)
	CT(51217)=(S=12653009,E=12653009,C=12)
	CT(51218)=(S=12653010,E=12653010,C=14)
	CT(51219)=(S=12653011,E=12653011,C=19)
	CT(51220)=(S=12653012,E=12653012,C=58)
	CT(51221)=(S=12653013,E=12653013,C=31)
	CT(51222)=(S=12653014,E=12653014,C=58)
	CT(51223)=(S=12653016,E=12653017,C=31)
	CT(51224)=(S=12653018,E=12653018,C=19)
	CT(51225)=(S=12653019,E=12653021,C=28)
	CT(51226)=(S=12653022,E=12653022,C=42)
	CT(51227)=(S=12653023,E=12653023,C=58)
	CT(51228)=(S=12653025,E=12653025,C=38)
	CT(51229)=(S=12653026,E=12653026,C=14)
	CT(51230)=(S=12653027,E=12653027,C=52)
	CT(51231)=(S=12653029,E=12653029,C=51)
	CT(51232)=(S=12653030,E=12653047,C=28)
	CT(51233)=(S=12653048,E=12653051,C=33)
	CT(51234)=(S=12653052,E=12653052,C=42)
	CT(51235)=(S=12653054,E=12653375,C=28)
	CT(51236)=(S=12653376,E=12653377,C=27)
	CT(51237)=(S=12653378,E=12653383,C=18)
	CT(51238)=(S=12653386,E=12653387,C=31)
	CT(51239)=(S=12653388,E=12653389,C=12)
	CT(51240)=(S=12653390,E=12653391,C=28)
	CT(51241)=(S=12653392,E=12653393,C=19)
	CT(51242)=(S=12653394,E=12653395,C=18)
	CT(51243)=(S=12653396,E=12653399,C=31)
	CT(51244)=(S=12653400,E=12653401,C=14)
	CT(51245)=(S=12653402,E=12653403,C=23)
	CT(51246)=(S=12653404,E=12653405,C=42)
	CT(51247)=(S=12653406,E=12653407,C=34)
	CT(51248)=(S=12653410,E=12653411,C=14)
	CT(51249)=(S=12653412,E=12653413,C=31)
	CT(51250)=(S=12653414,E=12653415,C=18)
	CT(51251)=(S=12653416,E=12653417,C=58)
	CT(51252)=(S=12653420,E=12653423,C=31)
	CT(51253)=(S=12653424,E=12653425,C=70)
	CT(51254)=(S=12653426,E=12653427,C=28)
	CT(51255)=(S=12653428,E=12653429,C=18)
	CT(51256)=(S=12653430,E=12653431,C=31)
	CT(51257)=(S=12653432,E=12653433,C=18)
	CT(51258)=(S=12653434,E=12653437,C=58)
	CT(51259)=(S=12653438,E=12653439,C=18)
	CT(51260)=(S=12653444,E=12653447,C=31)
	CT(51261)=(S=12653448,E=12653451,C=47)
	CT(51262)=(S=12653452,E=12653455,C=58)
	CT(51263)=(S=12653456,E=12653457,C=26)
	CT(51264)=(S=12653458,E=12653458,C=19)
	CT(51265)=(S=12653459,E=12653459,C=51)
	CT(51266)=(S=12653464,E=12653467,C=31)
	CT(51267)=(S=12653468,E=12653471,C=14)
	CT(51268)=(S=12653472,E=12653475,C=28)
	CT(51269)=(S=12653476,E=12653479,C=58)
	CT(51270)=(S=12653480,E=12653483,C=18)
	CT(51271)=(S=12653484,E=12653487,C=51)
	CT(51272)=(S=12653488,E=12653491,C=33)
	CT(51273)=(S=12653492,E=12653495,C=28)
	CT(51274)=(S=12653496,E=12653499,C=31)
	CT(51275)=(S=12653500,E=12653503,C=14)
	CT(51276)=(S=12653504,E=12653507,C=38)
	CT(51277)=(S=12653508,E=12653511,C=14)
	CT(51278)=(S=12653520,E=12653523,C=12)
	CT(51279)=(S=12653524,E=12653527,C=58)
	CT(51280)=(S=12653528,E=12653531,C=12)
	CT(51281)=(S=12653532,E=12653535,C=57)
	CT(51282)=(S=12653536,E=12653539,C=36)
	CT(51283)=(S=12653540,E=12653543,C=31)
	CT(51284)=(S=12653544,E=12653547,C=106)
	CT(51285)=(S=12653548,E=12653551,C=28)
	CT(51286)=(S=12653552,E=12653567,C=31)
	CT(51287)=(S=12653568,E=12654084,C=28)
	CT(51288)=(S=12654085,E=12654085,C=58)
	CT(51289)=(S=12654086,E=12654086,C=18)
	CT(51290)=(S=12654087,E=12654087,C=12)
	CT(51291)=(S=12654088,E=12654099,C=28)
	CT(51292)=(S=12654108,E=12654108,C=58)
	CT(51293)=(S=12654109,E=12654109,C=28)
	CT(51294)=(S=12654110,E=12654110,C=32)
	CT(51295)=(S=12654111,E=12654111,C=57)
	CT(51296)=(S=12654112,E=12654159,C=28)
	CT(51297)=(S=12654161,E=12654161,C=58)
	CT(51298)=(S=12654162,E=12654162,C=42)
	CT(51299)=(S=12654163,E=12654163,C=58)
	CT(51300)=(S=12654164,E=12654164,C=31)
	CT(51301)=(S=12654165,E=12654165,C=42)
	CT(51302)=(S=12654167,E=12654167,C=14)
	CT(51303)=(S=12654168,E=12654168,C=18)
	CT(51304)=(S=12654169,E=12654169,C=14)
	CT(51305)=(S=12654170,E=12654170,C=15)
	CT(51306)=(S=12654171,E=12654171,C=58)
	CT(51307)=(S=12654172,E=12654172,C=12)
	CT(51308)=(S=12654173,E=12654173,C=38)
	CT(51309)=(S=12654174,E=12654174,C=27)
	CT(51310)=(S=12654175,E=12654175,C=38)
	CT(51311)=(S=12654180,E=12654181,C=28)
	CT(51312)=(S=12654182,E=12654182,C=34)
	CT(51313)=(S=12654183,E=12654183,C=51)
	CT(51314)=(S=12654187,E=12654187,C=12)
	CT(51315)=(S=12654189,E=12654198,C=28)
	CT(51316)=(S=12654199,E=12654199,C=25)
	CT(51317)=(S=12654200,E=12654207,C=28)
	CT(51318)=(S=12654220,E=12654220,C=31)
	CT(51319)=(S=12654222,E=12654222,C=34)
	CT(51320)=(S=12654223,E=12654223,C=12)
	CT(51321)=(S=12654240,E=12654244,C=28)
	CT(51322)=(S=12654245,E=12654245,C=25)
	CT(51323)=(S=12654246,E=12654247,C=28)
	CT(51324)=(S=12654249,E=12654249,C=58)
	CT(51325)=(S=12654251,E=12654251,C=34)
	CT(51326)=(S=12654252,E=12654252,C=101)
	CT(51327)=(S=12654253,E=12654253,C=38)
	CT(51328)=(S=12654254,E=12654254,C=28)
	CT(51329)=(S=12654255,E=12654255,C=58)
	CT(51330)=(S=12654256,E=12654303,C=28)
	CT(51331)=(S=12654322,E=12654322,C=18)
	CT(51332)=(S=12654323,E=12654325,C=14)
	CT(51333)=(S=12654328,E=12654328,C=51)
	CT(51334)=(S=12654329,E=12654329,C=23)
	CT(51335)=(S=12654330,E=12654330,C=27)
	CT(51336)=(S=12654331,E=12654331,C=28)
	CT(51337)=(S=12654332,E=12654332,C=58)
	CT(51338)=(S=12654333,E=12654381,C=28)
	CT(51339)=(S=12654384,E=12654384,C=58)
	CT(51340)=(S=12654385,E=12654385,C=52)
	CT(51341)=(S=12654387,E=12654387,C=58)
	CT(51342)=(S=12654388,E=12654388,C=51)
	CT(51343)=(S=12654389,E=12654389,C=31)
	CT(51344)=(S=12654390,E=12654390,C=58)
	CT(51345)=(S=12654392,E=12654392,C=28)
	CT(51346)=(S=12654393,E=12654393,C=19)
	CT(51347)=(S=12654394,E=12654394,C=31)
	CT(51348)=(S=12654395,E=12654395,C=44)
	CT(51349)=(S=12654397,E=12654397,C=58)
	CT(51350)=(S=12654398,E=12654398,C=52)
	CT(51351)=(S=12654399,E=12654399,C=58)
	CT(51352)=(S=12654400,E=12654447,C=28)
	CT(51353)=(S=12654448,E=12654449,C=36)
	CT(51354)=(S=12654450,E=12654450,C=15)
	CT(51355)=(S=12654451,E=12654451,C=49)
	CT(51356)=(S=12654452,E=12654452,C=14)
	CT(51357)=(S=12654453,E=12654453,C=19)
	CT(51358)=(S=12654455,E=12654455,C=28)
	CT(51359)=(S=12654456,E=12654456,C=14)
	CT(51360)=(S=12654457,E=12654457,C=12)
	CT(51361)=(S=12654458,E=12654458,C=31)
	CT(51362)=(S=12654459,E=12654459,C=12)
	CT(51363)=(S=12654460,E=12654460,C=14)
	CT(51364)=(S=12654462,E=12654462,C=18)
	CT(51365)=(S=12654468,E=12654470,C=28)
	CT(51366)=(S=12654472,E=12654472,C=14)
	CT(51367)=(S=12654473,E=12654473,C=52)
	CT(51368)=(S=12654474,E=12654475,C=50)
	CT(51369)=(S=12654476,E=12654476,C=31)
	CT(51370)=(S=12654477,E=12654477,C=14)
	CT(51371)=(S=12654478,E=12654478,C=18)
	CT(51372)=(S=12654479,E=12654479,C=36)
	CT(51373)=(S=12654484,E=12654491,C=28)
	CT(51374)=(S=12654492,E=12654492,C=37)
	CT(51375)=(S=12654493,E=12654493,C=31)
	CT(51376)=(S=12654494,E=12654494,C=12)
	CT(51377)=(S=12654495,E=12654495,C=27)
	CT(51378)=(S=12654498,E=12654507,C=28)
	CT(51379)=(S=12654508,E=12654508,C=19)
	CT(51380)=(S=12654510,E=12654510,C=58)
	CT(51381)=(S=12654511,E=12654511,C=31)
	CT(51382)=(S=12654516,E=12654516,C=23)
	CT(51383)=(S=12654517,E=12654517,C=31)
	CT(51384)=(S=12654518,E=12654518,C=57)
	CT(51385)=(S=12654519,E=12654519,C=31)
	CT(51386)=(S=12654520,E=12654559,C=28)
	CT(51387)=(S=12654560,E=12654560,C=14)
	CT(51388)=(S=12654561,E=12654561,C=31)
	CT(51389)=(S=12654562,E=12654562,C=28)
	CT(51390)=(S=12654563,E=12654563,C=12)
	CT(51391)=(S=12654564,E=12654564,C=19)
	CT(51392)=(S=12654565,E=12654565,C=27)
	CT(51393)=(S=12654566,E=12654590,C=28)
	CT(51394)=(S=12654591,E=12654591,C=24)
	CT(51395)=(S=12654594,E=12654604,C=28)
	CT(51396)=(S=12654606,E=12654606,C=58)
	CT(51397)=(S=12654607,E=12654615,C=28)
	CT(51398)=(S=12654616,E=12654616,C=58)
	CT(51399)=(S=12654617,E=12654617,C=31)
	CT(51400)=(S=12654618,E=12654618,C=14)
	CT(51401)=(S=12654619,E=12654619,C=28)
	CT(51402)=(S=12654620,E=12654620,C=14)
	CT(51403)=(S=12654621,E=12654621,C=15)
	CT(51404)=(S=12654622,E=12654622,C=31)
	CT(51405)=(S=12654624,E=12654655,C=28)
	CT(51406)=(S=12654656,E=12654663,C=47)
	CT(51407)=(S=12654664,E=12654783,C=28)
	CT(51408)=(S=12654784,E=12654787,C=34)
	CT(51409)=(S=12654788,E=12654791,C=18)
	CT(51410)=(S=12654792,E=12654799,C=58)
	CT(51411)=(S=12654800,E=12654811,C=28)
	CT(51412)=(S=12654812,E=12654815,C=31)
	CT(51413)=(S=12654816,E=12654831,C=28)
	CT(51414)=(S=12654832,E=12654835,C=51)
	CT(51415)=(S=12654836,E=12654839,C=58)
	CT(51416)=(S=12654840,E=12654843,C=52)
	CT(51417)=(S=12654844,E=12654847,C=28)
	CT(51418)=(S=12654848,E=12654855,C=58)
	CT(51419)=(S=12654856,E=12654863,C=18)
	CT(51420)=(S=12654864,E=12654943,C=28)
	CT(51421)=(S=12654946,E=12654947,C=42)
	CT(51422)=(S=12654948,E=12654949,C=28)
	CT(51423)=(S=12654950,E=12654951,C=34)
	CT(51424)=(S=12654952,E=12654961,C=38)
	CT(51425)=(S=12654962,E=12654963,C=28)
	CT(51426)=(S=12654964,E=12654965,C=14)
	CT(51427)=(S=12654966,E=12654967,C=28)
	CT(51428)=(S=12654968,E=12654969,C=31)
	CT(51429)=(S=12654972,E=12654973,C=37)
	CT(51430)=(S=12654974,E=12654975,C=19)
	CT(51431)=(S=12654976,E=12655007,C=28)
	CT(51432)=(S=12655008,E=12655009,C=58)
	CT(51433)=(S=12655010,E=12655011,C=51)
	CT(51434)=(S=12655012,E=12655013,C=21)
	CT(51435)=(S=12655014,E=12655015,C=58)
	CT(51436)=(S=12655018,E=12655023,C=28)
	CT(51437)=(S=12655024,E=12655025,C=31)
	CT(51438)=(S=12655026,E=12655027,C=15)
	CT(51439)=(S=12655028,E=12655029,C=31)
	CT(51440)=(S=12655030,E=12655031,C=28)
	CT(51441)=(S=12655032,E=12655033,C=58)
	CT(51442)=(S=12655034,E=12655035,C=14)
	CT(51443)=(S=12655038,E=12655039,C=18)
	CT(51444)=(S=12655040,E=12655040,C=25)
	CT(51445)=(S=12655041,E=12655041,C=58)
	CT(51446)=(S=12655043,E=12655045,C=12)
	CT(51447)=(S=12655046,E=12655046,C=13)
	CT(51448)=(S=12655047,E=12655047,C=42)
	CT(51449)=(S=12655048,E=12655061,C=28)
	CT(51450)=(S=12655068,E=12655068,C=57)
	CT(51451)=(S=12655069,E=12655069,C=18)
	CT(51452)=(S=12655070,E=12655070,C=58)
	CT(51453)=(S=12655072,E=12655102,C=28)
	CT(51454)=(S=12655105,E=12655105,C=36)
	CT(51455)=(S=12655107,E=12655107,C=31)
	CT(51456)=(S=12655108,E=12655108,C=14)
	CT(51457)=(S=12655109,E=12655109,C=28)
	CT(51458)=(S=12655110,E=12655110,C=19)
	CT(51459)=(S=12655111,E=12655111,C=38)
	CT(51460)=(S=12655112,E=12655112,C=42)
	CT(51461)=(S=12655113,E=12655113,C=36)
	CT(51462)=(S=12655114,E=12655114,C=38)
	CT(51463)=(S=12655115,E=12655115,C=12)
	CT(51464)=(S=12655117,E=12655117,C=31)
	CT(51465)=(S=12655119,E=12655119,C=28)
	CT(51466)=(S=12655120,E=12655120,C=18)
	CT(51467)=(S=12655121,E=12655121,C=19)
	CT(51468)=(S=12655122,E=12655122,C=68)
	CT(51469)=(S=12655123,E=12655123,C=18)
	CT(51470)=(S=12655124,E=12655124,C=31)
	CT(51471)=(S=12655127,E=12655127,C=14)
	CT(51472)=(S=12655128,E=12655128,C=28)
	CT(51473)=(S=12655129,E=12655129,C=58)
	CT(51474)=(S=12655130,E=12655130,C=52)
	CT(51475)=(S=12655131,E=12655131,C=31)
	CT(51476)=(S=12655132,E=12655132,C=28)
	CT(51477)=(S=12655133,E=12655133,C=12)
	CT(51478)=(S=12655134,E=12655134,C=19)
	CT(51479)=(S=12655135,E=12655135,C=50)
	CT(51480)=(S=12655136,E=12655167,C=28)
	CT(51481)=(S=12655168,E=12655199,C=14)
	CT(51482)=(S=12655200,E=12655230,C=28)
	CT(51483)=(S=12655232,E=12655232,C=15)
	CT(51484)=(S=12655233,E=12655233,C=38)
	CT(51485)=(S=12655234,E=12655234,C=34)
	CT(51486)=(S=12655235,E=12655235,C=58)
	CT(51487)=(S=12655236,E=12655236,C=24)
	CT(51488)=(S=12655238,E=12655238,C=31)
	CT(51489)=(S=12655239,E=12655239,C=18)
	CT(51490)=(S=12655240,E=12655311,C=28)
	CT(51491)=(S=12655313,E=12655313,C=31)
	CT(51492)=(S=12655318,E=12655318,C=18)
	CT(51493)=(S=12655320,E=12655320,C=51)
	CT(51494)=(S=12655321,E=12655321,C=18)
	CT(51495)=(S=12655322,E=12655322,C=24)
	CT(51496)=(S=12655324,E=12655324,C=52)
	CT(51497)=(S=12655325,E=12655325,C=19)
	CT(51498)=(S=12655326,E=12655326,C=14)
	CT(51499)=(S=12655328,E=12655359,C=28)
	CT(51500)=(S=12655360,E=12655360,C=36)
	CT(51501)=(S=12655361,E=12655361,C=63)
	CT(51502)=(S=12655362,E=12655363,C=24)
	CT(51503)=(S=12655364,E=12655364,C=12)
	CT(51504)=(S=12655365,E=12655366,C=58)
	CT(51505)=(S=12655367,E=12655367,C=23)
	CT(51506)=(S=12655369,E=12655369,C=25)
	CT(51507)=(S=12655371,E=12655371,C=21)
	CT(51508)=(S=12655378,E=12655382,C=28)
	CT(51509)=(S=12655400,E=12655400,C=42)
	CT(51510)=(S=12655401,E=12655401,C=18)
	CT(51511)=(S=12655402,E=12655402,C=14)
	CT(51512)=(S=12655403,E=12655403,C=34)
	CT(51513)=(S=12655404,E=12655404,C=24)
	CT(51514)=(S=12655406,E=12655406,C=28)
	CT(51515)=(S=12655407,E=12655407,C=25)
	CT(51516)=(S=12655408,E=12655423,C=28)
	CT(51517)=(S=12655424,E=12655425,C=13)
	CT(51518)=(S=12655426,E=12655427,C=28)
	CT(51519)=(S=12655428,E=12655429,C=12)
	CT(51520)=(S=12655430,E=12655431,C=38)
	CT(51521)=(S=12655432,E=12655433,C=60)
	CT(51522)=(S=12655434,E=12655435,C=34)
	CT(51523)=(S=12655438,E=12655439,C=12)
	CT(51524)=(S=12655440,E=12655441,C=31)
	CT(51525)=(S=12655442,E=12655443,C=58)
	CT(51526)=(S=12655444,E=12655445,C=38)
	CT(51527)=(S=12655446,E=12655447,C=36)
	CT(51528)=(S=12655448,E=12655449,C=14)
	CT(51529)=(S=12655450,E=12655451,C=58)
	CT(51530)=(S=12655452,E=12655453,C=17)
	CT(51531)=(S=12655454,E=12655455,C=27)
	CT(51532)=(S=12655456,E=12655551,C=28)
	CT(51533)=(S=12655552,E=12655553,C=19)
	CT(51534)=(S=12655554,E=12655555,C=12)
	CT(51535)=(S=12655556,E=12655557,C=31)
	CT(51536)=(S=12655558,E=12655559,C=58)
	CT(51537)=(S=12655560,E=12655561,C=19)
	CT(51538)=(S=12655562,E=12655563,C=15)
	CT(51539)=(S=12655564,E=12655565,C=58)
	CT(51540)=(S=12655566,E=12655567,C=98)
	CT(51541)=(S=12655568,E=12655569,C=31)
	CT(51542)=(S=12655570,E=12655571,C=58)
	CT(51543)=(S=12655572,E=12655573,C=14)
	CT(51544)=(S=12655574,E=12655575,C=18)
	CT(51545)=(S=12655576,E=12655577,C=24)
	CT(51546)=(S=12655578,E=12655579,C=34)
	CT(51547)=(S=12655580,E=12655581,C=28)
	CT(51548)=(S=12655582,E=12655583,C=18)
	CT(51549)=(S=12655586,E=12655587,C=34)
	CT(51550)=(S=12655590,E=12655601,C=18)
	CT(51551)=(S=12655602,E=12655603,C=31)
	CT(51552)=(S=12655604,E=12655605,C=42)
	CT(51553)=(S=12655610,E=12655611,C=24)
	CT(51554)=(S=12655612,E=12655613,C=28)
	CT(51555)=(S=12655614,E=12655615,C=19)
	CT(51556)=(S=12655616,E=12655616,C=58)
	CT(51557)=(S=12655621,E=12655621,C=28)
	CT(51558)=(S=12655622,E=12655622,C=18)
	CT(51559)=(S=12655624,E=12655624,C=71)
	CT(51560)=(S=12655625,E=12655626,C=23)
	CT(51561)=(S=12655627,E=12655627,C=28)
	CT(51562)=(S=12655628,E=12655628,C=42)
	CT(51563)=(S=12655629,E=12655629,C=36)
	CT(51564)=(S=12655631,E=12655657,C=28)
	CT(51565)=(S=12655658,E=12655658,C=34)
	CT(51566)=(S=12655660,E=12655660,C=18)
	CT(51567)=(S=12655661,E=12655661,C=14)
	CT(51568)=(S=12655662,E=12655662,C=18)
	CT(51569)=(S=12655663,E=12655663,C=36)
	CT(51570)=(S=12655664,E=12655699,C=28)
	CT(51571)=(S=12655700,E=12655700,C=58)
	CT(51572)=(S=12655701,E=12655701,C=31)
	CT(51573)=(S=12655702,E=12655702,C=50)
	CT(51574)=(S=12655703,E=12655703,C=78)
	CT(51575)=(S=12655704,E=12655704,C=28)
	CT(51576)=(S=12655705,E=12655705,C=47)
	CT(51577)=(S=12655706,E=12655707,C=28)
	CT(51578)=(S=12655708,E=12655708,C=31)
	CT(51579)=(S=12655710,E=12655710,C=23)
	CT(51580)=(S=12655711,E=12655711,C=15)
	CT(51581)=(S=12655712,E=12655761,C=28)
	CT(51582)=(S=12655762,E=12655762,C=52)
	CT(51583)=(S=12655763,E=12655765,C=24)
	CT(51584)=(S=12655766,E=12655766,C=28)
	CT(51585)=(S=12655767,E=12655767,C=38)
	CT(51586)=(S=12655768,E=12655768,C=36)
	CT(51587)=(S=12655769,E=12655769,C=28)
	CT(51588)=(S=12655770,E=12655770,C=14)
	CT(51589)=(S=12655771,E=12655771,C=17)
	CT(51590)=(S=12655772,E=12655772,C=31)
	CT(51591)=(S=12655773,E=12655773,C=15)
	CT(51592)=(S=12655774,E=12655774,C=58)
	CT(51593)=(S=12655775,E=12655775,C=19)
	CT(51594)=(S=12655776,E=12655791,C=28)
	CT(51595)=(S=12655792,E=12655792,C=14)
	CT(51596)=(S=12655793,E=12655793,C=31)
	CT(51597)=(S=12655794,E=12655794,C=14)
	CT(51598)=(S=12655795,E=12655795,C=18)
	CT(51599)=(S=12655796,E=12655796,C=28)
	CT(51600)=(S=12655797,E=12655797,C=18)
	CT(51601)=(S=12655799,E=12655799,C=12)
	CT(51602)=(S=12655800,E=12655800,C=31)
	CT(51603)=(S=12655801,E=12655801,C=34)
	CT(51604)=(S=12655802,E=12655802,C=31)
	CT(51605)=(S=12655803,E=12655803,C=58)
	CT(51606)=(S=12655804,E=12655804,C=34)
	CT(51607)=(S=12655805,E=12655805,C=58)
	CT(51608)=(S=12655806,E=12655806,C=18)
	CT(51609)=(S=12655808,E=12655815,C=28)
	CT(51610)=(S=12655816,E=12655816,C=31)
	CT(51611)=(S=12655817,E=12655817,C=14)
	CT(51612)=(S=12655819,E=12655819,C=47)
	CT(51613)=(S=12655820,E=12655820,C=42)
	CT(51614)=(S=12655821,E=12655821,C=12)
	CT(51615)=(S=12655822,E=12655822,C=36)
	CT(51616)=(S=12655823,E=12655835,C=28)
	CT(51617)=(S=12655840,E=12655840,C=24)
	CT(51618)=(S=12655841,E=12655841,C=37)
	CT(51619)=(S=12655842,E=12655842,C=58)
	CT(51620)=(S=12655843,E=12655843,C=24)
	CT(51621)=(S=12655844,E=12655845,C=28)
	CT(51622)=(S=12655846,E=12655846,C=58)
	CT(51623)=(S=12655847,E=12655847,C=50)
	CT(51624)=(S=12655848,E=12655848,C=14)
	CT(51625)=(S=12655849,E=12655849,C=12)
	CT(51626)=(S=12655850,E=12655850,C=18)
	CT(51627)=(S=12655851,E=12655851,C=12)
	CT(51628)=(S=12655852,E=12655852,C=27)
	CT(51629)=(S=12655854,E=12655854,C=28)
	CT(51630)=(S=12655855,E=12655855,C=18)
	CT(51631)=(S=12655856,E=12655865,C=28)
	CT(51632)=(S=12655866,E=12655866,C=51)
	CT(51633)=(S=12655867,E=12655867,C=28)
	CT(51634)=(S=12655870,E=12655870,C=55)
	CT(51635)=(S=12655872,E=12655881,C=28)
	CT(51636)=(S=12655888,E=12655888,C=31)
	CT(51637)=(S=12655891,E=12655891,C=18)
	CT(51638)=(S=12655894,E=12655910,C=28)
	CT(51639)=(S=12655911,E=12655911,C=44)
	CT(51640)=(S=12655915,E=12655915,C=28)
	CT(51641)=(S=12655920,E=12655920,C=58)
	CT(51642)=(S=12655921,E=12655921,C=50)
	CT(51643)=(S=12655922,E=12655922,C=23)
	CT(51644)=(S=12655923,E=12655923,C=58)
	CT(51645)=(S=12655924,E=12655924,C=12)
	CT(51646)=(S=12655925,E=12655925,C=20)
	CT(51647)=(S=12655926,E=12655926,C=28)
	CT(51648)=(S=12655927,E=12655927,C=51)
	CT(51649)=(S=12655936,E=12656010,C=28)
	CT(51650)=(S=12656011,E=12656011,C=36)
	CT(51651)=(S=12656012,E=12656058,C=28)
	CT(51652)=(S=12656059,E=12656059,C=42)
	CT(51653)=(S=12656060,E=12656071,C=28)
	CT(51654)=(S=12656072,E=12656072,C=31)
	CT(51655)=(S=12656073,E=12656073,C=24)
	CT(51656)=(S=12656074,E=12656074,C=25)
	CT(51657)=(S=12656075,E=12656076,C=31)
	CT(51658)=(S=12656077,E=12656077,C=58)
	CT(51659)=(S=12656078,E=12656078,C=33)
	CT(51660)=(S=12656079,E=12656079,C=58)
	CT(51661)=(S=12656080,E=12656091,C=28)
	CT(51662)=(S=12656092,E=12656092,C=31)
	CT(51663)=(S=12656093,E=12656093,C=49)
	CT(51664)=(S=12656094,E=12656094,C=12)
	CT(51665)=(S=12656095,E=12656095,C=14)
	CT(51666)=(S=12656098,E=12656125,C=28)
	CT(51667)=(S=12656127,E=12656127,C=58)
	CT(51668)=(S=12656128,E=12656128,C=27)
	CT(51669)=(S=12656131,E=12656148,C=28)
	CT(51670)=(S=12656152,E=12656155,C=14)
	CT(51671)=(S=12656156,E=12656156,C=38)
	CT(51672)=(S=12656157,E=12656157,C=28)
	CT(51673)=(S=12656158,E=12656159,C=36)
	CT(51674)=(S=12656164,E=12656166,C=28)
	CT(51675)=(S=12656167,E=12656167,C=18)
	CT(51676)=(S=12656169,E=12656169,C=14)
	CT(51677)=(S=12656170,E=12656235,C=28)
	CT(51678)=(S=12656236,E=12656236,C=14)
	CT(51679)=(S=12656238,E=12656238,C=49)
	CT(51680)=(S=12656239,E=12656239,C=14)
	CT(51681)=(S=12656240,E=12656268,C=28)
	CT(51682)=(S=12656269,E=12656269,C=12)
	CT(51683)=(S=12656270,E=12656270,C=19)
	CT(51684)=(S=12656271,E=12656271,C=36)
	CT(51685)=(S=12656272,E=12656287,C=28)
	CT(51686)=(S=12656288,E=12656288,C=58)
	CT(51687)=(S=12656289,E=12656289,C=17)
	CT(51688)=(S=12656291,E=12656291,C=42)
	CT(51689)=(S=12656292,E=12656292,C=50)
	CT(51690)=(S=12656295,E=12656295,C=34)
	CT(51691)=(S=12656296,E=12656351,C=28)
	CT(51692)=(S=12656352,E=12656355,C=12)
	CT(51693)=(S=12656356,E=12656359,C=51)
	CT(51694)=(S=12656360,E=12656363,C=14)
	CT(51695)=(S=12656364,E=12656367,C=18)
	CT(51696)=(S=12656368,E=12656375,C=31)
	CT(51697)=(S=12656376,E=12656379,C=46)
	CT(51698)=(S=12656380,E=12656380,C=14)
	CT(51699)=(S=12656381,E=12656381,C=31)
	CT(51700)=(S=12656382,E=12656383,C=14)
	CT(51701)=(S=12656384,E=12656384,C=18)
	CT(51702)=(S=12656385,E=12656395,C=28)
	CT(51703)=(S=12656398,E=12656398,C=42)
	CT(51704)=(S=12656402,E=12656575,C=28)
	CT(51705)=(S=12656576,E=12656583,C=18)
	CT(51706)=(S=12656584,E=12656591,C=20)
	CT(51707)=(S=12656592,E=12656607,C=13)
	CT(51708)=(S=12656608,E=12656631,C=28)
	CT(51709)=(S=12656636,E=12656637,C=34)
	CT(51710)=(S=12656643,E=12656649,C=14)
	CT(51711)=(S=12656652,E=12656655,C=31)
	CT(51712)=(S=12656657,E=12656658,C=14)
	CT(51713)=(S=12656659,E=12656659,C=36)
	CT(51714)=(S=12656660,E=12656660,C=18)
	CT(51715)=(S=12656661,E=12656661,C=31)
	CT(51716)=(S=12656662,E=12656675,C=14)
	CT(51717)=(S=12656676,E=12656677,C=58)
	CT(51718)=(S=12656678,E=12656689,C=14)
	CT(51719)=(S=12656690,E=12656690,C=58)
	CT(51720)=(S=12656691,E=12656691,C=42)
	CT(51721)=(S=12656692,E=12656692,C=60)
	CT(51722)=(S=12656693,E=12656693,C=37)
	CT(51723)=(S=12656694,E=12656694,C=18)
	CT(51724)=(S=12656695,E=12656695,C=37)
	CT(51725)=(S=12656696,E=12656696,C=33)
	CT(51726)=(S=12656697,E=12656697,C=31)
	CT(51727)=(S=12656698,E=12656698,C=38)
	CT(51728)=(S=12656699,E=12656699,C=58)
	CT(51729)=(S=12656700,E=12656703,C=18)
	CT(51730)=(S=12656704,E=12656704,C=28)
	CT(51731)=(S=12656705,E=12656705,C=24)
	CT(51732)=(S=12656706,E=12656707,C=14)
	CT(51733)=(S=12656708,E=12656711,C=18)
	CT(51734)=(S=12656712,E=12656719,C=25)
	CT(51735)=(S=12656720,E=12656721,C=26)
	CT(51736)=(S=12656722,E=12656731,C=14)
	CT(51737)=(S=12656740,E=12656740,C=38)
	CT(51738)=(S=12656741,E=12656895,C=14)
	CT(51739)=(S=12656896,E=12656897,C=58)
	CT(51740)=(S=12656898,E=12656899,C=25)
	CT(51741)=(S=12656900,E=12656905,C=58)
	CT(51742)=(S=12656906,E=12656907,C=18)
	CT(51743)=(S=12656908,E=12656909,C=12)
	CT(51744)=(S=12656910,E=12656911,C=33)
	CT(51745)=(S=12656912,E=12656913,C=17)
	CT(51746)=(S=12656914,E=12656915,C=15)
	CT(51747)=(S=12656916,E=12656917,C=28)
	CT(51748)=(S=12656918,E=12656919,C=33)
	CT(51749)=(S=12656920,E=12656921,C=38)
	CT(51750)=(S=12656922,E=12656925,C=18)
	CT(51751)=(S=12656926,E=12656927,C=34)
	CT(51752)=(S=12656928,E=12656929,C=38)
	CT(51753)=(S=12656930,E=12656931,C=58)
	CT(51754)=(S=12656932,E=12656933,C=12)
	CT(51755)=(S=12656934,E=12656935,C=18)
	CT(51756)=(S=12656938,E=12656939,C=58)
	CT(51757)=(S=12656940,E=12656941,C=31)
	CT(51758)=(S=12656942,E=12656943,C=12)
	CT(51759)=(S=12656944,E=12656945,C=31)
	CT(51760)=(S=12656946,E=12656947,C=36)
	CT(51761)=(S=12656948,E=12656949,C=49)
	CT(51762)=(S=12656950,E=12656951,C=31)
	CT(51763)=(S=12656952,E=12656953,C=12)
	CT(51764)=(S=12656954,E=12656955,C=58)
	CT(51765)=(S=12656956,E=12656957,C=36)
	CT(51766)=(S=12656958,E=12656959,C=18)
	CT(51767)=(S=12656960,E=12656961,C=31)
	CT(51768)=(S=12656964,E=12656965,C=18)
	CT(51769)=(S=12656966,E=12656967,C=36)
	CT(51770)=(S=12656968,E=12656969,C=14)
	CT(51771)=(S=12656970,E=12656971,C=18)
	CT(51772)=(S=12656972,E=12656973,C=14)
	CT(51773)=(S=12656974,E=12656975,C=12)
	CT(51774)=(S=12656976,E=12656985,C=18)
	CT(51775)=(S=12656986,E=12656987,C=38)
	CT(51776)=(S=12656988,E=12656989,C=53)
	CT(51777)=(S=12656990,E=12656991,C=38)
	CT(51778)=(S=12656992,E=12656993,C=28)
	CT(51779)=(S=12656994,E=12656995,C=15)
	CT(51780)=(S=12656996,E=12656997,C=18)
	CT(51781)=(S=12656998,E=12656999,C=16)
	CT(51782)=(S=12657000,E=12657001,C=31)
	CT(51783)=(S=12657004,E=12657005,C=14)
	CT(51784)=(S=12657006,E=12657007,C=58)
	CT(51785)=(S=12657010,E=12657011,C=42)
	CT(51786)=(S=12657012,E=12657013,C=14)
	CT(51787)=(S=12657014,E=12657015,C=38)
	CT(51788)=(S=12657016,E=12657019,C=18)
	CT(51789)=(S=12657020,E=12657021,C=58)
	CT(51790)=(S=12657022,E=12657023,C=33)
	CT(51791)=(S=12657024,E=12657025,C=34)
	CT(51792)=(S=12657028,E=12657029,C=18)
	CT(51793)=(S=12657030,E=12657031,C=36)
	CT(51794)=(S=12657034,E=12657035,C=14)
	CT(51795)=(S=12657038,E=12657039,C=34)
	CT(51796)=(S=12657040,E=12657041,C=18)
	CT(51797)=(S=12657042,E=12657043,C=14)
	CT(51798)=(S=12657044,E=12657045,C=24)
	CT(51799)=(S=12657046,E=12657047,C=42)
	CT(51800)=(S=12657048,E=12657049,C=19)
	CT(51801)=(S=12657050,E=12657051,C=38)
	CT(51802)=(S=12657052,E=12657053,C=14)
	CT(51803)=(S=12657054,E=12657055,C=28)
	CT(51804)=(S=12657056,E=12657062,C=18)
	CT(51805)=(S=12657063,E=12657065,C=12)
	CT(51806)=(S=12657066,E=12657067,C=14)
	CT(51807)=(S=12657068,E=12657069,C=16)
	CT(51808)=(S=12657070,E=12657073,C=58)
	CT(51809)=(S=12657074,E=12657075,C=14)
	CT(51810)=(S=12657076,E=12657077,C=36)
	CT(51811)=(S=12657078,E=12657079,C=49)
	CT(51812)=(S=12657080,E=12657081,C=18)
	CT(51813)=(S=12657082,E=12657083,C=14)
	CT(51814)=(S=12657084,E=12657085,C=19)
	CT(51815)=(S=12657086,E=12657087,C=24)
	CT(51816)=(S=12657088,E=12657091,C=18)
	CT(51817)=(S=12657092,E=12657093,C=31)
	CT(51818)=(S=12657094,E=12657095,C=14)
	CT(51819)=(S=12657096,E=12657097,C=38)
	CT(51820)=(S=12657098,E=12657103,C=31)
	CT(51821)=(S=12657104,E=12657105,C=36)
	CT(51822)=(S=12657106,E=12657107,C=42)
	CT(51823)=(S=12657108,E=12657109,C=31)
	CT(51824)=(S=12657110,E=12657111,C=36)
	CT(51825)=(S=12657112,E=12657113,C=42)
	CT(51826)=(S=12657114,E=12657115,C=19)
	CT(51827)=(S=12657116,E=12657117,C=36)
	CT(51828)=(S=12657118,E=12657119,C=12)
	CT(51829)=(S=12657120,E=12657121,C=101)
	CT(51830)=(S=12657122,E=12657123,C=18)
	CT(51831)=(S=12657124,E=12657125,C=72)
	CT(51832)=(S=12657126,E=12657129,C=18)
	CT(51833)=(S=12657130,E=12657131,C=17)
	CT(51834)=(S=12657132,E=12657133,C=31)
	CT(51835)=(S=12657134,E=12657135,C=34)
	CT(51836)=(S=12657136,E=12657137,C=18)
	CT(51837)=(S=12657140,E=12657141,C=14)
	CT(51838)=(S=12657142,E=12657143,C=19)
	CT(51839)=(S=12657144,E=12657145,C=14)
	CT(51840)=(S=12657146,E=12657147,C=38)
	CT(51841)=(S=12657150,E=12657151,C=14)
	CT(51842)=(S=12657152,E=12657155,C=58)
	CT(51843)=(S=12657156,E=12657159,C=36)
	CT(51844)=(S=12657160,E=12657167,C=18)
	CT(51845)=(S=12657168,E=12657171,C=12)
	CT(51846)=(S=12657172,E=12657175,C=31)
	CT(51847)=(S=12657176,E=12657183,C=28)
	CT(51848)=(S=12657184,E=12657187,C=12)
	CT(51849)=(S=12657188,E=12657191,C=29)
	CT(51850)=(S=12657192,E=12657195,C=47)
	CT(51851)=(S=12657196,E=12657199,C=58)
	CT(51852)=(S=12657200,E=12657203,C=14)
	CT(51853)=(S=12657204,E=12657207,C=58)
	CT(51854)=(S=12657208,E=12657211,C=17)
	CT(51855)=(S=12657212,E=12657215,C=31)
	CT(51856)=(S=12657216,E=12657219,C=46)
	CT(51857)=(S=12657220,E=12657223,C=28)
	CT(51858)=(S=12657224,E=12657227,C=31)
	CT(51859)=(S=12657232,E=12657235,C=63)
	CT(51860)=(S=12657236,E=12657239,C=44)
	CT(51861)=(S=12657240,E=12657243,C=49)
	CT(51862)=(S=12657244,E=12657251,C=31)
	CT(51863)=(S=12657252,E=12657255,C=28)
	CT(51864)=(S=12657256,E=12657259,C=171)
	CT(51865)=(S=12657260,E=12657263,C=38)
	CT(51866)=(S=12657264,E=12657267,C=58)
	CT(51867)=(S=12657268,E=12657275,C=28)
	CT(51868)=(S=12657276,E=12657279,C=18)
	CT(51869)=(S=12657280,E=12657281,C=31)
	CT(51870)=(S=12657282,E=12657283,C=12)
	CT(51871)=(S=12657284,E=12657285,C=37)
	CT(51872)=(S=12657286,E=12657287,C=50)
	CT(51873)=(S=12657288,E=12657289,C=34)
	CT(51874)=(S=12657290,E=12657291,C=58)
	CT(51875)=(S=12657292,E=12657293,C=31)
	CT(51876)=(S=12657294,E=12657295,C=46)
	CT(51877)=(S=12657296,E=12657297,C=28)
	CT(51878)=(S=12657298,E=12657301,C=14)
	CT(51879)=(S=12657302,E=12657303,C=36)
	CT(51880)=(S=12657304,E=12657305,C=19)
	CT(51881)=(S=12657306,E=12657307,C=31)
	CT(51882)=(S=12657308,E=12657309,C=38)
	CT(51883)=(S=12657310,E=12657311,C=58)
	CT(51884)=(S=12657312,E=12657313,C=18)
	CT(51885)=(S=12657314,E=12657315,C=58)
	CT(51886)=(S=12657316,E=12657317,C=18)
	CT(51887)=(S=12657318,E=12657319,C=36)
	CT(51888)=(S=12657320,E=12657321,C=31)
	CT(51889)=(S=12657322,E=12657323,C=14)
	CT(51890)=(S=12657324,E=12657325,C=31)
	CT(51891)=(S=12657326,E=12657327,C=38)
	CT(51892)=(S=12657328,E=12657329,C=22)
	CT(51893)=(S=12657330,E=12657331,C=58)
	CT(51894)=(S=12657332,E=12657335,C=38)
	CT(51895)=(S=12657336,E=12657337,C=18)
	CT(51896)=(S=12657338,E=12657339,C=14)
	CT(51897)=(S=12657340,E=12657341,C=34)
	CT(51898)=(S=12657342,E=12657343,C=18)
	CT(51899)=(S=12657344,E=12657344,C=19)
	CT(51900)=(S=12657345,E=12657345,C=58)
	CT(51901)=(S=12657347,E=12657348,C=58)
	CT(51902)=(S=12657349,E=12657349,C=12)
	CT(51903)=(S=12657350,E=12657350,C=58)
	CT(51904)=(S=12657351,E=12657351,C=27)
	CT(51905)=(S=12657352,E=12657352,C=31)
	CT(51906)=(S=12657353,E=12657353,C=38)
	CT(51907)=(S=12657354,E=12657354,C=19)
	CT(51908)=(S=12657355,E=12657356,C=58)
	CT(51909)=(S=12657357,E=12657357,C=37)
	CT(51910)=(S=12657358,E=12657358,C=42)
	CT(51911)=(S=12657359,E=12657359,C=28)
	CT(51912)=(S=12657360,E=12657363,C=18)
	CT(51913)=(S=12657364,E=12657367,C=58)
	CT(51914)=(S=12657368,E=12657371,C=31)
	CT(51915)=(S=12657372,E=12657375,C=58)
	CT(51916)=(S=12657382,E=12657407,C=14)
	CT(51917)=(S=12657408,E=12657408,C=38)
	CT(51918)=(S=12657416,E=12657421,C=14)
	CT(51919)=(S=12657433,E=12657433,C=31)
	CT(51920)=(S=12657440,E=12657443,C=18)
	CT(51921)=(S=12657444,E=12657444,C=58)
	CT(51922)=(S=12657445,E=12657445,C=13)
	CT(51923)=(S=12657446,E=12657447,C=18)
	CT(51924)=(S=12657448,E=12657448,C=51)
	CT(51925)=(S=12657449,E=12657449,C=58)
	CT(51926)=(S=12657450,E=12657451,C=18)
	CT(51927)=(S=12657452,E=12657453,C=25)
	CT(51928)=(S=12657454,E=12657454,C=51)
	CT(51929)=(S=12657455,E=12657455,C=58)
	CT(51930)=(S=12657456,E=12657459,C=18)
	CT(51931)=(S=12657460,E=12657463,C=27)
	CT(51932)=(S=12657464,E=12657467,C=14)
	CT(51933)=(S=12657468,E=12657468,C=58)
	CT(51934)=(S=12657469,E=12657469,C=18)
	CT(51935)=(S=12657470,E=12657470,C=26)
	CT(51936)=(S=12657471,E=12657471,C=18)
	CT(51937)=(S=12657472,E=12657487,C=14)
	CT(51938)=(S=12657504,E=12657506,C=18)
	CT(51939)=(S=12657507,E=12657507,C=58)
	CT(51940)=(S=12657508,E=12657508,C=18)
	CT(51941)=(S=12657510,E=12657515,C=33)
	CT(51942)=(S=12657516,E=12657516,C=38)
	CT(51943)=(S=12657517,E=12657517,C=18)
	CT(51944)=(S=12657518,E=12657518,C=58)
	CT(51945)=(S=12657519,E=12657519,C=52)
	CT(51946)=(S=12657520,E=12657663,C=14)
	CT(51947)=(S=12657664,E=12657664,C=37)
	CT(51948)=(S=12657665,E=12657665,C=28)
	CT(51949)=(S=12657666,E=12657695,C=14)
	CT(51950)=(S=12657696,E=12657696,C=34)
	CT(51951)=(S=12657697,E=12657697,C=28)
	CT(51952)=(S=12657698,E=12657698,C=14)
	CT(51953)=(S=12657699,E=12657699,C=18)
	CT(51954)=(S=12657700,E=12657700,C=34)
	CT(51955)=(S=12657701,E=12657701,C=38)
	CT(51956)=(S=12657702,E=12657702,C=18)
	CT(51957)=(S=12657703,E=12657703,C=37)
	CT(51958)=(S=12657704,E=12657704,C=52)
	CT(51959)=(S=12657705,E=12657706,C=18)
	CT(51960)=(S=12657707,E=12657707,C=63)
	CT(51961)=(S=12657708,E=12657708,C=38)
	CT(51962)=(S=12657709,E=12657709,C=12)
	CT(51963)=(S=12657710,E=12657710,C=28)
	CT(51964)=(S=12657711,E=12657837,C=14)
	CT(51965)=(S=12657840,E=12657840,C=17)
	CT(51966)=(S=12657841,E=12657841,C=36)
	CT(51967)=(S=12657842,E=12657842,C=27)
	CT(51968)=(S=12657846,E=12657846,C=12)
	CT(51969)=(S=12657847,E=12657847,C=58)
	CT(51970)=(S=12657848,E=12657848,C=37)
	CT(51971)=(S=12657850,E=12657850,C=36)
	CT(51972)=(S=12657851,E=12657852,C=28)
	CT(51973)=(S=12657853,E=12657853,C=42)
	CT(51974)=(S=12657854,E=12657854,C=21)
	CT(51975)=(S=12657855,E=12657855,C=27)
	CT(51976)=(S=12657856,E=12657887,C=38)
	CT(51977)=(S=12657890,E=12657890,C=42)
	CT(51978)=(S=12657893,E=12657893,C=36)
	CT(51979)=(S=12657894,E=12657955,C=14)
	CT(51980)=(S=12657956,E=12657959,C=63)
	CT(51981)=(S=12658008,E=12658047,C=14)
	CT(51982)=(S=12658048,E=12658050,C=17)
	CT(51983)=(S=12658051,E=12658052,C=28)
	CT(51984)=(S=12658053,E=12658053,C=31)
	CT(51985)=(S=12658054,E=12658054,C=13)
	CT(51986)=(S=12658055,E=12658055,C=37)
	CT(51987)=(S=12658056,E=12658056,C=38)
	CT(51988)=(S=12658057,E=12658057,C=18)
	CT(51989)=(S=12658058,E=12658058,C=38)
	CT(51990)=(S=12658059,E=12658060,C=58)
	CT(51991)=(S=12658061,E=12658061,C=31)
	CT(51992)=(S=12658062,E=12658062,C=14)
	CT(51993)=(S=12658063,E=12658063,C=23)
	CT(51994)=(S=12658064,E=12658064,C=42)
	CT(51995)=(S=12658065,E=12658065,C=12)
	CT(51996)=(S=12658066,E=12658066,C=14)
	CT(51997)=(S=12658067,E=12658067,C=34)
	CT(51998)=(S=12658068,E=12658069,C=28)
	CT(51999)=(S=12658070,E=12658070,C=12)
	CT(52000)=(S=12658071,E=12658072,C=28)
	CT(52001)=(S=12658073,E=12658073,C=21)
	CT(52002)=(S=12658074,E=12658074,C=37)
	CT(52003)=(S=12658076,E=12658076,C=31)
	CT(52004)=(S=12658077,E=12658077,C=18)
	CT(52005)=(S=12658078,E=12658078,C=28)
	CT(52006)=(S=12658079,E=12658079,C=34)
	CT(52007)=(S=12658080,E=12658149,C=14)
	CT(52008)=(S=12658156,E=12658156,C=34)
	CT(52009)=(S=12658157,E=12658157,C=14)
	CT(52010)=(S=12658158,E=12658158,C=51)
	CT(52011)=(S=12658159,E=12658159,C=36)
	CT(52012)=(S=12658160,E=12658167,C=14)
	CT(52013)=(S=12658176,E=12658207,C=19)
	CT(52014)=(S=12658224,E=12658224,C=58)
	CT(52015)=(S=12658225,E=12658497,C=14)
	CT(52016)=(S=12658498,E=12658498,C=38)
	CT(52017)=(S=12658499,E=12658499,C=14)
	CT(52018)=(S=12658500,E=12658500,C=28)
	CT(52019)=(S=12658501,E=12658501,C=31)
	CT(52020)=(S=12658502,E=12658502,C=12)
	CT(52021)=(S=12658503,E=12658503,C=58)
	CT(52022)=(S=12658504,E=12658504,C=31)
	CT(52023)=(S=12658505,E=12658505,C=38)
	CT(52024)=(S=12658506,E=12658506,C=28)
	CT(52025)=(S=12658507,E=12658509,C=31)
	CT(52026)=(S=12658511,E=12658511,C=17)
	CT(52027)=(S=12658512,E=12658543,C=14)
	CT(52028)=(S=12658544,E=12658544,C=36)
	CT(52029)=(S=12658545,E=12658545,C=18)
	CT(52030)=(S=12658546,E=12658546,C=31)
	CT(52031)=(S=12658547,E=12658547,C=28)
	CT(52032)=(S=12658548,E=12658548,C=18)
	CT(52033)=(S=12658549,E=12658549,C=38)
	CT(52034)=(S=12658550,E=12658550,C=31)
	CT(52035)=(S=12658551,E=12658551,C=38)
	CT(52036)=(S=12658552,E=12658567,C=19)
	CT(52037)=(S=12658568,E=12658568,C=14)
	CT(52038)=(S=12658571,E=12658571,C=24)
	CT(52039)=(S=12658572,E=12658615,C=14)
	CT(52040)=(S=12658632,E=12658639,C=28)
	CT(52041)=(S=12658642,E=12658679,C=14)
	CT(52042)=(S=12658680,E=12658681,C=18)
	CT(52043)=(S=12658683,E=12658683,C=14)
	CT(52044)=(S=12658688,E=12658943,C=55)
	CT(52045)=(S=12658944,E=12658945,C=14)
	CT(52046)=(S=12658946,E=12658947,C=37)
	CT(52047)=(S=12658948,E=12658949,C=31)
	CT(52048)=(S=12658950,E=12658951,C=15)
	CT(52049)=(S=12658952,E=12658955,C=28)
	CT(52050)=(S=12658956,E=12658957,C=24)
	CT(52051)=(S=12658958,E=12658959,C=25)
	CT(52052)=(S=12658960,E=12658967,C=49)
	CT(52053)=(S=12658968,E=12658975,C=25)
	CT(52054)=(S=12658977,E=12658977,C=60)
	CT(52055)=(S=12658979,E=12658980,C=52)
	CT(52056)=(S=12658981,E=12658981,C=28)
	CT(52057)=(S=12658984,E=12658984,C=44)
	CT(52058)=(S=12658985,E=12658985,C=42)
	CT(52059)=(S=12658986,E=12658986,C=49)
	CT(52060)=(S=12658987,E=12658987,C=25)
	CT(52061)=(S=12658988,E=12658988,C=58)
	CT(52062)=(S=12658989,E=12658989,C=60)
	CT(52063)=(S=12658990,E=12658990,C=25)
	CT(52064)=(S=12658992,E=12658995,C=31)
	CT(52065)=(S=12659000,E=12659003,C=22)
	CT(52066)=(S=12659004,E=12659007,C=31)
	CT(52067)=(S=12659008,E=12659011,C=51)
	CT(52068)=(S=12659015,E=12659019,C=12)
	CT(52069)=(S=12659020,E=12659023,C=18)
	CT(52070)=(S=12659024,E=12659024,C=31)
	CT(52071)=(S=12659025,E=12659025,C=44)
	CT(52072)=(S=12659027,E=12659027,C=12)
	CT(52073)=(S=12659028,E=12659029,C=15)
	CT(52074)=(S=12659030,E=12659030,C=18)
	CT(52075)=(S=12659032,E=12659032,C=31)
	CT(52076)=(S=12659033,E=12659033,C=52)
	CT(52077)=(S=12659034,E=12659034,C=34)
	CT(52078)=(S=12659037,E=12659037,C=18)
	CT(52079)=(S=12659038,E=12659038,C=42)
	CT(52080)=(S=12659039,E=12659039,C=12)
	CT(52081)=(S=12659040,E=12659047,C=14)
	CT(52082)=(S=12659048,E=12659055,C=13)
	CT(52083)=(S=12659056,E=12659057,C=58)
	CT(52084)=(S=12659060,E=12659061,C=28)
	CT(52085)=(S=12659062,E=12659063,C=19)
	CT(52086)=(S=12659064,E=12659069,C=28)
	CT(52087)=(S=12659070,E=12659071,C=38)
	CT(52088)=(S=12659072,E=12659075,C=31)
	CT(52089)=(S=12659076,E=12659079,C=28)
	CT(52090)=(S=12659080,E=12659081,C=34)
	CT(52091)=(S=12659082,E=12659083,C=42)
	CT(52092)=(S=12659084,E=12659087,C=18)
	CT(52093)=(S=12659088,E=12659089,C=28)
	CT(52094)=(S=12659090,E=12659091,C=135)
	CT(52095)=(S=12659092,E=12659093,C=34)
	CT(52096)=(S=12659094,E=12659095,C=22)
	CT(52097)=(S=12659096,E=12659097,C=14)
	CT(52098)=(S=12659100,E=12659101,C=24)
	CT(52099)=(S=12659102,E=12659103,C=49)
	CT(52100)=(S=12659104,E=12659107,C=25)
	CT(52101)=(S=12659108,E=12659111,C=14)
	CT(52102)=(S=12659112,E=12659115,C=19)
	CT(52103)=(S=12659116,E=12659119,C=58)
	CT(52104)=(S=12659120,E=12659123,C=15)
	CT(52105)=(S=12659124,E=12659125,C=12)
	CT(52106)=(S=12659126,E=12659127,C=51)
	CT(52107)=(S=12659128,E=12659131,C=31)
	CT(52108)=(S=12659132,E=12659135,C=51)
	CT(52109)=(S=12659137,E=12659137,C=34)
	CT(52110)=(S=12659138,E=12659138,C=12)
	CT(52111)=(S=12659139,E=12659139,C=60)
	CT(52112)=(S=12659140,E=12659140,C=28)
	CT(52113)=(S=12659141,E=12659141,C=44)
	CT(52114)=(S=12659142,E=12659142,C=15)
	CT(52115)=(S=12659144,E=12659144,C=28)
	CT(52116)=(S=12659145,E=12659145,C=42)
	CT(52117)=(S=12659146,E=12659146,C=17)
	CT(52118)=(S=12659147,E=12659147,C=18)
	CT(52119)=(S=12659149,E=12659149,C=15)
	CT(52120)=(S=12659150,E=12659150,C=38)
	CT(52121)=(S=12659151,E=12659151,C=42)
	CT(52122)=(S=12659152,E=12659153,C=17)
	CT(52123)=(S=12659154,E=12659155,C=58)
	CT(52124)=(S=12659158,E=12659159,C=19)
	CT(52125)=(S=12659160,E=12659161,C=38)
	CT(52126)=(S=12659162,E=12659163,C=31)
	CT(52127)=(S=12659164,E=12659165,C=14)
	CT(52128)=(S=12659166,E=12659167,C=12)
	CT(52129)=(S=12659168,E=12659168,C=19)
	CT(52130)=(S=12659169,E=12659169,C=37)
	CT(52131)=(S=12659171,E=12659171,C=25)
	CT(52132)=(S=12659172,E=12659172,C=42)
	CT(52133)=(S=12659173,E=12659173,C=22)
	CT(52134)=(S=12659174,E=12659175,C=58)
	CT(52135)=(S=12659176,E=12659176,C=42)
	CT(52136)=(S=12659177,E=12659177,C=49)
	CT(52137)=(S=12659178,E=12659178,C=25)
	CT(52138)=(S=12659179,E=12659180,C=15)
	CT(52139)=(S=12659182,E=12659182,C=12)
	CT(52140)=(S=12659183,E=12659183,C=31)
	CT(52141)=(S=12659184,E=12659187,C=25)
	CT(52142)=(S=12659188,E=12659195,C=38)
	CT(52143)=(S=12659196,E=12659199,C=28)
	CT(52144)=(S=12659202,E=12659341,C=15)
	CT(52145)=(S=12659342,E=12659342,C=18)
	CT(52146)=(S=12659343,E=12659343,C=25)
	CT(52147)=(S=12659344,E=12659344,C=14)
	CT(52148)=(S=12659345,E=12659345,C=18)
	CT(52149)=(S=12659346,E=12659346,C=34)
	CT(52150)=(S=12659347,E=12659347,C=58)
	CT(52151)=(S=12659348,E=12659348,C=31)
	CT(52152)=(S=12659350,E=12659351,C=12)
	CT(52153)=(S=12659352,E=12659352,C=14)
	CT(52154)=(S=12659353,E=12659354,C=58)
	CT(52155)=(S=12659355,E=12659355,C=28)
	CT(52156)=(S=12659356,E=12659357,C=31)
	CT(52157)=(S=12659358,E=12659358,C=34)
	CT(52158)=(S=12659359,E=12659359,C=19)
	CT(52159)=(S=12659360,E=12659409,C=15)
	CT(52160)=(S=12659411,E=12659411,C=58)
	CT(52161)=(S=12659412,E=12659412,C=36)
	CT(52162)=(S=12659413,E=12659413,C=12)
	CT(52163)=(S=12659415,E=12659415,C=19)
	CT(52164)=(S=12659416,E=12659416,C=37)
	CT(52165)=(S=12659418,E=12659418,C=58)
	CT(52166)=(S=12659420,E=12659420,C=23)
	CT(52167)=(S=12659421,E=12659421,C=28)
	CT(52168)=(S=12659422,E=12659422,C=50)
	CT(52169)=(S=12659423,E=12659423,C=101)
	CT(52170)=(S=12659428,E=12659431,C=58)
	CT(52171)=(S=12659432,E=12659439,C=15)
	CT(52172)=(S=12659448,E=12659455,C=36)
	CT(52173)=(S=12659456,E=12659456,C=58)
	CT(52174)=(S=12659457,E=12659463,C=15)
	CT(52175)=(S=12659468,E=12659469,C=58)
	CT(52176)=(S=12659471,E=12659480,C=15)
	CT(52177)=(S=12659481,E=12659481,C=12)
	CT(52178)=(S=12659482,E=12659482,C=51)
	CT(52179)=(S=12659483,E=12659483,C=34)
	CT(52180)=(S=12659485,E=12659485,C=28)
	CT(52181)=(S=12659490,E=12659532,C=15)
	CT(52182)=(S=12659533,E=12659533,C=58)
	CT(52183)=(S=12659534,E=12659534,C=18)
	CT(52184)=(S=12659536,E=12659543,C=15)
	CT(52185)=(S=12659544,E=12659545,C=36)
	CT(52186)=(S=12659546,E=12659546,C=58)
	CT(52187)=(S=12659548,E=12659548,C=36)
	CT(52188)=(S=12659549,E=12659549,C=34)
	CT(52189)=(S=12659551,E=12659551,C=31)
	CT(52190)=(S=12659552,E=12659557,C=15)
	CT(52191)=(S=12659558,E=12659559,C=169)
	CT(52192)=(S=12659560,E=12659580,C=15)
	CT(52193)=(S=12659581,E=12659581,C=14)
	CT(52194)=(S=12659583,E=12659583,C=18)
	CT(52195)=(S=12659584,E=12659587,C=169)
	CT(52196)=(S=12659588,E=12659597,C=15)
	CT(52197)=(S=12659600,E=12659604,C=58)
	CT(52198)=(S=12659606,E=12659606,C=36)
	CT(52199)=(S=12659607,E=12659607,C=19)
	CT(52200)=(S=12659608,E=12659613,C=34)
	CT(52201)=(S=12659614,E=12659614,C=42)
	CT(52202)=(S=12659616,E=12659635,C=15)
	CT(52203)=(S=12659636,E=12659636,C=18)
	CT(52204)=(S=12659637,E=12659637,C=58)
	CT(52205)=(S=12659638,E=12659638,C=18)
	CT(52206)=(S=12659639,E=12659639,C=34)
	CT(52207)=(S=12659640,E=12659663,C=15)
	CT(52208)=(S=12659664,E=12659665,C=18)
	CT(52209)=(S=12659666,E=12659667,C=31)
	CT(52210)=(S=12659668,E=12659669,C=42)
	CT(52211)=(S=12659670,E=12659671,C=12)
	CT(52212)=(S=12659672,E=12659673,C=24)
	CT(52213)=(S=12659674,E=12659675,C=36)
	CT(52214)=(S=12659676,E=12659677,C=28)
	CT(52215)=(S=12659678,E=12659683,C=31)
	CT(52216)=(S=12659684,E=12659687,C=58)
	CT(52217)=(S=12659688,E=12659691,C=18)
	CT(52218)=(S=12659692,E=12659695,C=13)
	CT(52219)=(S=12659696,E=12659699,C=58)
	CT(52220)=(S=12659700,E=12659703,C=17)
	CT(52221)=(S=12659704,E=12659711,C=31)
	CT(52222)=(S=12659712,E=12660223,C=19)
	CT(52223)=(S=12660225,E=12660225,C=42)
	CT(52224)=(S=12660226,E=12660226,C=18)
	CT(52225)=(S=12660227,E=12660269,C=42)
	CT(52226)=(S=12660270,E=12660270,C=31)
	CT(52227)=(S=12660271,E=12660271,C=58)
	CT(52228)=(S=12660272,E=12660283,C=42)
	CT(52229)=(S=12660284,E=12660284,C=18)
	CT(52230)=(S=12660285,E=12660285,C=14)
	CT(52231)=(S=12660286,E=12660287,C=28)
	CT(52232)=(S=12660288,E=12660288,C=17)
	CT(52233)=(S=12660289,E=12660289,C=18)
	CT(52234)=(S=12660290,E=12660290,C=31)
	CT(52235)=(S=12660292,E=12660292,C=58)
	CT(52236)=(S=12660293,E=12660293,C=24)
	CT(52237)=(S=12660294,E=12660294,C=38)
	CT(52238)=(S=12660295,E=12660295,C=52)
	CT(52239)=(S=12660296,E=12660296,C=14)
	CT(52240)=(S=12660297,E=12660297,C=34)
	CT(52241)=(S=12660298,E=12660298,C=18)
	CT(52242)=(S=12660299,E=12660299,C=52)
	CT(52243)=(S=12660300,E=12660300,C=14)
	CT(52244)=(S=12660301,E=12660301,C=18)
	CT(52245)=(S=12660304,E=12660304,C=36)
	CT(52246)=(S=12660305,E=12660305,C=31)
	CT(52247)=(S=12660306,E=12660306,C=28)
	CT(52248)=(S=12660307,E=12660308,C=44)
	CT(52249)=(S=12660309,E=12660309,C=12)
	CT(52250)=(S=12660310,E=12660310,C=31)
	CT(52251)=(S=12660311,E=12660311,C=14)
	CT(52252)=(S=12660312,E=12660312,C=34)
	CT(52253)=(S=12660313,E=12660313,C=31)
	CT(52254)=(S=12660314,E=12660314,C=14)
	CT(52255)=(S=12660315,E=12660315,C=38)
	CT(52256)=(S=12660316,E=12660395,C=42)
	CT(52257)=(S=12660404,E=12660404,C=15)
	CT(52258)=(S=12660405,E=12660405,C=58)
	CT(52259)=(S=12660406,E=12660406,C=28)
	CT(52260)=(S=12660407,E=12660407,C=14)
	CT(52261)=(S=12660408,E=12660408,C=28)
	CT(52262)=(S=12660409,E=12660411,C=58)
	CT(52263)=(S=12660412,E=12660415,C=42)
	CT(52264)=(S=12660425,E=12660426,C=31)
	CT(52265)=(S=12660427,E=12660427,C=12)
	CT(52266)=(S=12660428,E=12660429,C=14)
	CT(52267)=(S=12660430,E=12660430,C=12)
	CT(52268)=(S=12660431,E=12660432,C=18)
	CT(52269)=(S=12660433,E=12660433,C=42)
	CT(52270)=(S=12660434,E=12660434,C=31)
	CT(52271)=(S=12660436,E=12660436,C=58)
	CT(52272)=(S=12660437,E=12660437,C=18)
	CT(52273)=(S=12660439,E=12660439,C=28)
	CT(52274)=(S=12660445,E=12660445,C=14)
	CT(52275)=(S=12660448,E=12660459,C=42)
	CT(52276)=(S=12660460,E=12660460,C=60)
	CT(52277)=(S=12660461,E=12660461,C=58)
	CT(52278)=(S=12660462,E=12660462,C=25)
	CT(52279)=(S=12660463,E=12660463,C=28)
	CT(52280)=(S=12660472,E=12660477,C=42)
	CT(52281)=(S=12660480,E=12660480,C=12)
	CT(52282)=(S=12660481,E=12660511,C=34)
	CT(52283)=(S=12660551,E=12660551,C=24)
	CT(52284)=(S=12660552,E=12660552,C=38)
	CT(52285)=(S=12660553,E=12660553,C=28)
	CT(52286)=(S=12660554,E=12660554,C=51)
	CT(52287)=(S=12660556,E=12660556,C=25)
	CT(52288)=(S=12660557,E=12660557,C=28)
	CT(52289)=(S=12660558,E=12660559,C=36)
	CT(52290)=(S=12660560,E=12660560,C=12)
	CT(52291)=(S=12660561,E=12660561,C=24)
	CT(52292)=(S=12660562,E=12660562,C=28)
	CT(52293)=(S=12660563,E=12660563,C=14)
	CT(52294)=(S=12660564,E=12660564,C=25)
	CT(52295)=(S=12660565,E=12660565,C=31)
	CT(52296)=(S=12660567,E=12660567,C=18)
	CT(52297)=(S=12660568,E=12660578,C=34)
	CT(52298)=(S=12660579,E=12660580,C=28)
	CT(52299)=(S=12660581,E=12660581,C=38)
	CT(52300)=(S=12660582,E=12660582,C=23)
	CT(52301)=(S=12660583,E=12660615,C=34)
	CT(52302)=(S=12660616,E=12660616,C=52)
	CT(52303)=(S=12660617,E=12660617,C=31)
	CT(52304)=(S=12660618,E=12660618,C=42)
	CT(52305)=(S=12660619,E=12660619,C=12)
	CT(52306)=(S=12660620,E=12660620,C=34)
	CT(52307)=(S=12660621,E=12660621,C=12)
	CT(52308)=(S=12660622,E=12660622,C=24)
	CT(52309)=(S=12660623,E=12660623,C=44)
	CT(52310)=(S=12660624,E=12660624,C=38)
	CT(52311)=(S=12660625,E=12660625,C=25)
	CT(52312)=(S=12660626,E=12660626,C=18)
	CT(52313)=(S=12660627,E=12660627,C=14)
	CT(52314)=(S=12660628,E=12660628,C=18)
	CT(52315)=(S=12660629,E=12660629,C=34)
	CT(52316)=(S=12660631,E=12660631,C=58)
	CT(52317)=(S=12660632,E=12660632,C=28)
	CT(52318)=(S=12660633,E=12660633,C=34)
	CT(52319)=(S=12660634,E=12660634,C=18)
	CT(52320)=(S=12660635,E=12660637,C=42)
	CT(52321)=(S=12660638,E=12660640,C=58)
	CT(52322)=(S=12660641,E=12660641,C=28)
	CT(52323)=(S=12660642,E=12660642,C=38)
	CT(52324)=(S=12660643,E=12660643,C=58)
	CT(52325)=(S=12660644,E=12660644,C=14)
	CT(52326)=(S=12660645,E=12660645,C=17)
	CT(52327)=(S=12660646,E=12660646,C=31)
	CT(52328)=(S=12660647,E=12660647,C=19)
	CT(52329)=(S=12660648,E=12660663,C=34)
	CT(52330)=(S=12660664,E=12660664,C=12)
	CT(52331)=(S=12660665,E=12660665,C=21)
	CT(52332)=(S=12660666,E=12660666,C=24)
	CT(52333)=(S=12660667,E=12660667,C=28)
	CT(52334)=(S=12660668,E=12660668,C=36)
	CT(52335)=(S=12660669,E=12660669,C=31)
	CT(52336)=(S=12660670,E=12660670,C=34)
	CT(52337)=(S=12660671,E=12660671,C=24)
	CT(52338)=(S=12660672,E=12660713,C=34)
	CT(52339)=(S=12660726,E=12660726,C=57)
	CT(52340)=(S=12660727,E=12660727,C=58)
	CT(52341)=(S=12660728,E=12660728,C=17)
	CT(52342)=(S=12660729,E=12660729,C=38)
	CT(52343)=(S=12660730,E=12660735,C=34)
	CT(52344)=(S=12660736,E=12662015,C=12)
	CT(52345)=(S=12662019,E=12662020,C=49)
	CT(52346)=(S=12662022,E=12662022,C=14)
	CT(52347)=(S=12662024,E=12662037,C=49)
	CT(52348)=(S=12662038,E=12662038,C=21)
	CT(52349)=(S=12662039,E=12662039,C=28)
	CT(52350)=(S=12662040,E=12662049,C=19)
	CT(52351)=(S=12662050,E=12662095,C=49)
	CT(52352)=(S=12662096,E=12662096,C=42)
	CT(52353)=(S=12662099,E=12662099,C=31)
	CT(52354)=(S=12662100,E=12662100,C=19)
	CT(52355)=(S=12662101,E=12662101,C=24)
	CT(52356)=(S=12662102,E=12662102,C=49)
	CT(52357)=(S=12662104,E=12662104,C=27)
	CT(52358)=(S=12662105,E=12662105,C=31)
	CT(52359)=(S=12662108,E=12662108,C=49)
	CT(52360)=(S=12662109,E=12662109,C=28)
	CT(52361)=(S=12662110,E=12662111,C=14)
	CT(52362)=(S=12662115,E=12662116,C=13)
	CT(52363)=(S=12662117,E=12662118,C=49)
	CT(52364)=(S=12662122,E=12662122,C=46)
	CT(52365)=(S=12662123,E=12662140,C=49)
	CT(52366)=(S=12662141,E=12662141,C=36)
	CT(52367)=(S=12662144,E=12662264,C=49)
	CT(52368)=(S=12662271,E=12662271,C=58)
	CT(52369)=(S=12662272,E=12662887,C=12)
	CT(52370)=(S=12662890,E=12662890,C=19)
	CT(52371)=(S=12662892,E=12662977,C=12)
	CT(52372)=(S=12662986,E=12662986,C=18)
	CT(52373)=(S=12662993,E=12663053,C=12)
	CT(52374)=(S=12663060,E=12663060,C=28)
	CT(52375)=(S=12663064,E=12663136,C=12)
	CT(52376)=(S=12663137,E=12663137,C=28)
	CT(52377)=(S=12663145,E=12663295,C=12)
	CT(52378)=(S=12663296,E=12663296,C=32)
	CT(52379)=(S=12663297,E=12663359,C=49)
	CT(52380)=(S=12663360,E=12663365,C=14)
	CT(52381)=(S=12663366,E=12663367,C=22)
	CT(52382)=(S=12663368,E=12663375,C=33)
	CT(52383)=(S=12663376,E=12663445,C=49)
	CT(52384)=(S=12663451,E=12663451,C=34)
	CT(52385)=(S=12663452,E=12663468,C=49)
	CT(52386)=(S=12663488,E=12663493,C=71)
	CT(52387)=(S=12663494,E=12663495,C=14)
	CT(52388)=(S=12663496,E=12663499,C=28)
	CT(52389)=(S=12663500,E=12663504,C=36)
	CT(52390)=(S=12663505,E=12663505,C=34)
	CT(52391)=(S=12663506,E=12663511,C=42)
	CT(52392)=(S=12663512,E=12663519,C=29)
	CT(52393)=(S=12663520,E=12663527,C=49)
	CT(52394)=(S=12663528,E=12663528,C=58)
	CT(52395)=(S=12663529,E=12663529,C=12)
	CT(52396)=(S=12663530,E=12663530,C=22)
	CT(52397)=(S=12663531,E=12663531,C=14)
	CT(52398)=(S=12663532,E=12663532,C=37)
	CT(52399)=(S=12663534,E=12663534,C=19)
	CT(52400)=(S=12663535,E=12663535,C=15)
	CT(52401)=(S=12663537,E=12663537,C=27)
	CT(52402)=(S=12663538,E=12663538,C=34)
	CT(52403)=(S=12663539,E=12663539,C=38)
	CT(52404)=(S=12663540,E=12663541,C=28)
	CT(52405)=(S=12663542,E=12663542,C=31)
	CT(52406)=(S=12663543,E=12663543,C=12)
	CT(52407)=(S=12663546,E=12663546,C=27)
	CT(52408)=(S=12663547,E=12663547,C=18)
	CT(52409)=(S=12663548,E=12663548,C=57)
	CT(52410)=(S=12663549,E=12663549,C=28)
	CT(52411)=(S=12663551,E=12663551,C=68)
	CT(52412)=(S=12663552,E=12663807,C=58)
	CT(52413)=(S=12663808,E=12664831,C=14)
	CT(52414)=(S=12664832,E=12665599,C=47)
	CT(52415)=(S=12665600,E=12665855,C=36)
	CT(52416)=(S=12665856,E=12665887,C=51)
	CT(52417)=(S=12665888,E=12665919,C=50)
	CT(52418)=(S=12665920,E=12665951,C=60)
	CT(52419)=(S=12665952,E=12666111,C=51)
	CT(52420)=(S=12666112,E=12666367,C=27)
	CT(52421)=(S=12666368,E=12666623,C=15)
	CT(52422)=(S=12666624,E=12666879,C=27)
	CT(52423)=(S=12666880,E=12667391,C=34)
	CT(52424)=(S=12667392,E=12667647,C=49)
	CT(52425)=(S=12667648,E=12667775,C=27)
	CT(52426)=(S=12667776,E=12667903,C=49)
	CT(52427)=(S=12667904,E=12668159,C=15)
	CT(52428)=(S=12668160,E=12668415,C=52)
	CT(52429)=(S=12668416,E=12668927,C=36)
	CT(52430)=(S=12668928,E=12669439,C=42)
	CT(52431)=(S=12669440,E=12669695,C=14)
	CT(52432)=(S=12669696,E=12669951,C=42)
	CT(52433)=(S=12669952,E=12669952,C=14)
	CT(52434)=(S=12669959,E=12669959,C=33)
	CT(52435)=(S=12669960,E=12669960,C=18)
	CT(52436)=(S=12669961,E=12669961,C=36)
	CT(52437)=(S=12669962,E=12669967,C=33)
	CT(52438)=(S=12669969,E=12669969,C=31)
	CT(52439)=(S=12669970,E=12669970,C=12)
	CT(52440)=(S=12669972,E=12669972,C=28)
	CT(52441)=(S=12669973,E=12669973,C=22)
	CT(52442)=(S=12669974,E=12669975,C=31)
	CT(52443)=(S=12669976,E=12669976,C=49)
	CT(52444)=(S=12669977,E=12669977,C=34)
	CT(52445)=(S=12669978,E=12669978,C=63)
	CT(52446)=(S=12669979,E=12669979,C=24)
	CT(52447)=(S=12669980,E=12669980,C=42)
	CT(52448)=(S=12669981,E=12669981,C=58)
	CT(52449)=(S=12669983,E=12669983,C=18)
	CT(52450)=(S=12669984,E=12669999,C=33)
	CT(52451)=(S=12670002,E=12670002,C=31)
	CT(52452)=(S=12670003,E=12670003,C=14)
	CT(52453)=(S=12670004,E=12670004,C=42)
	CT(52454)=(S=12670005,E=12670005,C=33)
	CT(52455)=(S=12670006,E=12670006,C=28)
	CT(52456)=(S=12670007,E=12670015,C=33)
	CT(52457)=(S=12670016,E=12670016,C=38)
	CT(52458)=(S=12670017,E=12670017,C=42)
	CT(52459)=(S=12670018,E=12670018,C=28)
	CT(52460)=(S=12670019,E=12670019,C=36)
	CT(52461)=(S=12670020,E=12670020,C=33)
	CT(52462)=(S=12670021,E=12670021,C=38)
	CT(52463)=(S=12670023,E=12670023,C=58)
	CT(52464)=(S=12670024,E=12670024,C=31)
	CT(52465)=(S=12670025,E=12670025,C=12)
	CT(52466)=(S=12670027,E=12670027,C=19)
	CT(52467)=(S=12670028,E=12670029,C=31)
	CT(52468)=(S=12670031,E=12670031,C=58)
	CT(52469)=(S=12670032,E=12670035,C=33)
	CT(52470)=(S=12670036,E=12670036,C=49)
	CT(52471)=(S=12670038,E=12670038,C=51)
	CT(52472)=(S=12670039,E=12670039,C=14)
	CT(52473)=(S=12670041,E=12670041,C=12)
	CT(52474)=(S=12670043,E=12670043,C=28)
	CT(52475)=(S=12670046,E=12670046,C=42)
	CT(52476)=(S=12670047,E=12670047,C=28)
	CT(52477)=(S=12670048,E=12670059,C=33)
	CT(52478)=(S=12670064,E=12670064,C=42)
	CT(52479)=(S=12670068,E=12670081,C=33)
	CT(52480)=(S=12670094,E=12670094,C=14)
	CT(52481)=(S=12670095,E=12670095,C=36)
	CT(52482)=(S=12670096,E=12670127,C=33)
	CT(52483)=(S=12670128,E=12670128,C=36)
	CT(52484)=(S=12670129,E=12670129,C=12)
	CT(52485)=(S=12670130,E=12670133,C=28)
	CT(52486)=(S=12670134,E=12670134,C=58)
	CT(52487)=(S=12670135,E=12670135,C=53)
	CT(52488)=(S=12670136,E=12670136,C=60)
	CT(52489)=(S=12670137,E=12670137,C=38)
	CT(52490)=(S=12670138,E=12670175,C=33)
	CT(52491)=(S=12670176,E=12670191,C=14)
	CT(52492)=(S=12670197,E=12670197,C=33)
	CT(52493)=(S=12670198,E=12670198,C=58)
	CT(52494)=(S=12670203,E=12670203,C=22)
	CT(52495)=(S=12670204,E=12670204,C=33)
	CT(52496)=(S=12670207,E=12670207,C=18)
	CT(52497)=(S=12670208,E=12670719,C=33)
	CT(52498)=(S=12670720,E=12670975,C=71)
	CT(52499)=(S=12670976,E=12671487,C=24)
	CT(52500)=(S=12671488,E=12671743,C=27)
	CT(52501)=(S=12671744,E=12671775,C=58)
	CT(52502)=(S=12671776,E=12671791,C=70)
	CT(52503)=(S=12671792,E=12671807,C=36)
	CT(52504)=(S=12671808,E=12671839,C=50)
	CT(52505)=(S=12671840,E=12671871,C=49)
	CT(52506)=(S=12671872,E=12671999,C=27)
	CT(52507)=(S=12672000,E=12672255,C=22)
	CT(52508)=(S=12672256,E=12672259,C=28)
	CT(52509)=(S=12672264,E=12672267,C=12)
	CT(52510)=(S=12672268,E=12672275,C=31)
	CT(52511)=(S=12672276,E=12672279,C=34)
	CT(52512)=(S=12672280,E=12672283,C=51)
	CT(52513)=(S=12672288,E=12672295,C=14)
	CT(52514)=(S=12672296,E=12672299,C=38)
	CT(52515)=(S=12672300,E=12672303,C=14)
	CT(52516)=(S=12672304,E=12672311,C=31)
	CT(52517)=(S=12672312,E=12672315,C=20)
	CT(52518)=(S=12672316,E=12672319,C=18)
	CT(52519)=(S=12672320,E=12672323,C=34)
	CT(52520)=(S=12672324,E=12672327,C=58)
	CT(52521)=(S=12672328,E=12672331,C=71)
	CT(52522)=(S=12672332,E=12672335,C=31)
	CT(52523)=(S=12672340,E=12672343,C=14)
	CT(52524)=(S=12672344,E=12672351,C=58)
	CT(52525)=(S=12672352,E=12672355,C=13)
	CT(52526)=(S=12672356,E=12672359,C=31)
	CT(52527)=(S=12672360,E=12672363,C=58)
	CT(52528)=(S=12672364,E=12672367,C=31)
	CT(52529)=(S=12672368,E=12672371,C=49)
	CT(52530)=(S=12672372,E=12672375,C=31)
	CT(52531)=(S=12672376,E=12672379,C=18)
	CT(52532)=(S=12672380,E=12672383,C=12)
	CT(52533)=(S=12672384,E=12672387,C=27)
	CT(52534)=(S=12672388,E=12672395,C=47)
	CT(52535)=(S=12672396,E=12672399,C=38)
	CT(52536)=(S=12672400,E=12672403,C=58)
	CT(52537)=(S=12672404,E=12672411,C=14)
	CT(52538)=(S=12672412,E=12672415,C=58)
	CT(52539)=(S=12672416,E=12672419,C=31)
	CT(52540)=(S=12672420,E=12672423,C=22)
	CT(52541)=(S=12672424,E=12672427,C=31)
	CT(52542)=(S=12672428,E=12672431,C=36)
	CT(52543)=(S=12672432,E=12672435,C=28)
	CT(52544)=(S=12672436,E=12672439,C=12)
	CT(52545)=(S=12672440,E=12672447,C=31)
	CT(52546)=(S=12672448,E=12672451,C=18)
	CT(52547)=(S=12672452,E=12672459,C=14)
	CT(52548)=(S=12672460,E=12672463,C=58)
	CT(52549)=(S=12672464,E=12672467,C=34)
	CT(52550)=(S=12672468,E=12672471,C=20)
	CT(52551)=(S=12672472,E=12672475,C=31)
	CT(52552)=(S=12672476,E=12672479,C=27)
	CT(52553)=(S=12672480,E=12672483,C=14)
	CT(52554)=(S=12672484,E=12672487,C=31)
	CT(52555)=(S=12672492,E=12672495,C=18)
	CT(52556)=(S=12672496,E=12672499,C=28)
	CT(52557)=(S=12672500,E=12672503,C=18)
	CT(52558)=(S=12672504,E=12672511,C=27)
	CT(52559)=(S=12672512,E=12672767,C=47)
	CT(52560)=(S=12672768,E=12672895,C=152)
	CT(52561)=(S=12672896,E=12672959,C=63)
	CT(52562)=(S=12672960,E=12673023,C=52)
	CT(52563)=(S=12673024,E=12675071,C=28)
	CT(52564)=(S=12675072,E=12675072,C=25)
	CT(52565)=(S=12675073,E=12675073,C=42)
	CT(52566)=(S=12675074,E=12675074,C=58)
	CT(52567)=(S=12675075,E=12675075,C=28)
	CT(52568)=(S=12675076,E=12675076,C=14)
	CT(52569)=(S=12675077,E=12675077,C=38)
	CT(52570)=(S=12675078,E=12675078,C=36)
	CT(52571)=(S=12675079,E=12675079,C=31)
	CT(52572)=(S=12675080,E=12675080,C=49)
	CT(52573)=(S=12675081,E=12675081,C=28)
	CT(52574)=(S=12675082,E=12675082,C=52)
	CT(52575)=(S=12675083,E=12675083,C=18)
	CT(52576)=(S=12675085,E=12675085,C=37)
	CT(52577)=(S=12675086,E=12675086,C=18)
	CT(52578)=(S=12675087,E=12675087,C=14)
	CT(52579)=(S=12675088,E=12675088,C=31)
	CT(52580)=(S=12675089,E=12675089,C=33)
	CT(52581)=(S=12675090,E=12675090,C=28)
	CT(52582)=(S=12675091,E=12675091,C=12)
	CT(52583)=(S=12675092,E=12675092,C=40)
	CT(52584)=(S=12675093,E=12675093,C=38)
	CT(52585)=(S=12675094,E=12675094,C=26)
	CT(52586)=(S=12675095,E=12675095,C=52)
	CT(52587)=(S=12675096,E=12675096,C=18)
	CT(52588)=(S=12675097,E=12675097,C=14)
	CT(52589)=(S=12675098,E=12675098,C=28)
	CT(52590)=(S=12675099,E=12675101,C=18)
	CT(52591)=(S=12675102,E=12675102,C=38)
	CT(52592)=(S=12675103,E=12675103,C=34)
	CT(52593)=(S=12675104,E=12675104,C=28)
	CT(52594)=(S=12675105,E=12675105,C=54)
	CT(52595)=(S=12675106,E=12675106,C=42)
	CT(52596)=(S=12675107,E=12675107,C=14)
	CT(52597)=(S=12675108,E=12675108,C=12)
	CT(52598)=(S=12675109,E=12675109,C=49)
	CT(52599)=(S=12675110,E=12675110,C=47)
	CT(52600)=(S=12675111,E=12675111,C=12)
	CT(52601)=(S=12675112,E=12675112,C=23)
	CT(52602)=(S=12675113,E=12675113,C=53)
	CT(52603)=(S=12675114,E=12675114,C=34)
	CT(52604)=(S=12675115,E=12675115,C=24)
	CT(52605)=(S=12675116,E=12675116,C=17)
	CT(52606)=(S=12675117,E=12675118,C=58)
	CT(52607)=(S=12675119,E=12675119,C=34)
	CT(52608)=(S=12675120,E=12675120,C=12)
	CT(52609)=(S=12675121,E=12675121,C=28)
	CT(52610)=(S=12675122,E=12675122,C=58)
	CT(52611)=(S=12675123,E=12675123,C=25)
	CT(52612)=(S=12675124,E=12675124,C=21)
	CT(52613)=(S=12675125,E=12675125,C=50)
	CT(52614)=(S=12675126,E=12675126,C=12)
	CT(52615)=(S=12675127,E=12675127,C=42)
	CT(52616)=(S=12675128,E=12675128,C=12)
	CT(52617)=(S=12675129,E=12675129,C=18)
	CT(52618)=(S=12675130,E=12675130,C=31)
	CT(52619)=(S=12675131,E=12675131,C=18)
	CT(52620)=(S=12675132,E=12675132,C=12)
	CT(52621)=(S=12675133,E=12675133,C=34)
	CT(52622)=(S=12675134,E=12675134,C=17)
	CT(52623)=(S=12675135,E=12675135,C=14)
	CT(52624)=(S=12675136,E=12675136,C=18)
	CT(52625)=(S=12675137,E=12675137,C=42)
	CT(52626)=(S=12675138,E=12675138,C=28)
	CT(52627)=(S=12675140,E=12675140,C=46)
	CT(52628)=(S=12675141,E=12675142,C=18)
	CT(52629)=(S=12675143,E=12675143,C=58)
	CT(52630)=(S=12675144,E=12675144,C=12)
	CT(52631)=(S=12675145,E=12675145,C=38)
	CT(52632)=(S=12675146,E=12675147,C=18)
	CT(52633)=(S=12675148,E=12675148,C=32)
	CT(52634)=(S=12675149,E=12675149,C=17)
	CT(52635)=(S=12675150,E=12675150,C=14)
	CT(52636)=(S=12675151,E=12675151,C=51)
	CT(52637)=(S=12675152,E=12675152,C=37)
	CT(52638)=(S=12675153,E=12675153,C=14)
	CT(52639)=(S=12675154,E=12675154,C=42)
	CT(52640)=(S=12675155,E=12675155,C=24)
	CT(52641)=(S=12675156,E=12675156,C=28)
	CT(52642)=(S=12675157,E=12675157,C=31)
	CT(52643)=(S=12675158,E=12675158,C=15)
	CT(52644)=(S=12675159,E=12675159,C=18)
	CT(52645)=(S=12675160,E=12675160,C=58)
	CT(52646)=(S=12675161,E=12675161,C=14)
	CT(52647)=(S=12675162,E=12675162,C=28)
	CT(52648)=(S=12675163,E=12675164,C=14)
	CT(52649)=(S=12675166,E=12675166,C=18)
	CT(52650)=(S=12675167,E=12675167,C=28)
	CT(52651)=(S=12675168,E=12675168,C=31)
	CT(52652)=(S=12675169,E=12675169,C=18)
	CT(52653)=(S=12675170,E=12675170,C=36)
	CT(52654)=(S=12675171,E=12675171,C=58)
	CT(52655)=(S=12675172,E=12675172,C=19)
	CT(52656)=(S=12675173,E=12675173,C=14)
	CT(52657)=(S=12675174,E=12675174,C=18)
	CT(52658)=(S=12675175,E=12675175,C=14)
	CT(52659)=(S=12675176,E=12675176,C=36)
	CT(52660)=(S=12675177,E=12675177,C=27)
	CT(52661)=(S=12675178,E=12675178,C=38)
	CT(52662)=(S=12675179,E=12675180,C=14)
	CT(52663)=(S=12675181,E=12675181,C=37)
	CT(52664)=(S=12675182,E=12675182,C=33)
	CT(52665)=(S=12675183,E=12675183,C=36)
	CT(52666)=(S=12675184,E=12675184,C=19)
	CT(52667)=(S=12675185,E=12675185,C=14)
	CT(52668)=(S=12675186,E=12675186,C=24)
	CT(52669)=(S=12675187,E=12675187,C=17)
	CT(52670)=(S=12675188,E=12675188,C=34)
	CT(52671)=(S=12675189,E=12675189,C=17)
	CT(52672)=(S=12675190,E=12675190,C=28)
	CT(52673)=(S=12675191,E=12675191,C=17)
	CT(52674)=(S=12675192,E=12675192,C=42)
	CT(52675)=(S=12675193,E=12675193,C=38)
	CT(52676)=(S=12675194,E=12675194,C=19)
	CT(52677)=(S=12675195,E=12675195,C=38)
	CT(52678)=(S=12675196,E=12675196,C=37)
	CT(52679)=(S=12675197,E=12675197,C=42)
	CT(52680)=(S=12675198,E=12675198,C=14)
	CT(52681)=(S=12675199,E=12675200,C=18)
	CT(52682)=(S=12675201,E=12675201,C=36)
	CT(52683)=(S=12675202,E=12675202,C=37)
	CT(52684)=(S=12675203,E=12675203,C=14)
	CT(52685)=(S=12675204,E=12675204,C=34)
	CT(52686)=(S=12675205,E=12675205,C=31)
	CT(52687)=(S=12675206,E=12675206,C=52)
	CT(52688)=(S=12675207,E=12675207,C=24)
	CT(52689)=(S=12675208,E=12675208,C=63)
	CT(52690)=(S=12675209,E=12675209,C=15)
	CT(52691)=(S=12675210,E=12675210,C=37)
	CT(52692)=(S=12675211,E=12675211,C=18)
	CT(52693)=(S=12675212,E=12675212,C=14)
	CT(52694)=(S=12675213,E=12675215,C=18)
	CT(52695)=(S=12675216,E=12675216,C=14)
	CT(52696)=(S=12675217,E=12675217,C=31)
	CT(52697)=(S=12675218,E=12675218,C=28)
	CT(52698)=(S=12675219,E=12675219,C=17)
	CT(52699)=(S=12675220,E=12675220,C=34)
	CT(52700)=(S=12675221,E=12675221,C=18)
	CT(52701)=(S=12675222,E=12675222,C=38)
	CT(52702)=(S=12675223,E=12675223,C=15)
	CT(52703)=(S=12675224,E=12675224,C=14)
	CT(52704)=(S=12675226,E=12675226,C=58)
	CT(52705)=(S=12675227,E=12675227,C=28)
	CT(52706)=(S=12675228,E=12675228,C=58)
	CT(52707)=(S=12675229,E=12675229,C=18)
	CT(52708)=(S=12675230,E=12675230,C=12)
	CT(52709)=(S=12675231,E=12675231,C=49)
	CT(52710)=(S=12675232,E=12675232,C=19)
	CT(52711)=(S=12675233,E=12675233,C=58)
	CT(52712)=(S=12675234,E=12675234,C=12)
	CT(52713)=(S=12675235,E=12675235,C=58)
	CT(52714)=(S=12675236,E=12675236,C=14)
	CT(52715)=(S=12675237,E=12675237,C=51)
	CT(52716)=(S=12675238,E=12675238,C=58)
	CT(52717)=(S=12675239,E=12675239,C=34)
	CT(52718)=(S=12675240,E=12675241,C=14)
	CT(52719)=(S=12675242,E=12675242,C=18)
	CT(52720)=(S=12675243,E=12675243,C=24)
	CT(52721)=(S=12675244,E=12675244,C=33)
	CT(52722)=(S=12675245,E=12675245,C=38)
	CT(52723)=(S=12675246,E=12675246,C=28)
	CT(52724)=(S=12675247,E=12675247,C=14)
	CT(52725)=(S=12675249,E=12675249,C=18)
	CT(52726)=(S=12675250,E=12675250,C=31)
	CT(52727)=(S=12675251,E=12675251,C=14)
	CT(52728)=(S=12675252,E=12675252,C=38)
	CT(52729)=(S=12675253,E=12675253,C=18)
	CT(52730)=(S=12675254,E=12675255,C=31)
	CT(52731)=(S=12675256,E=12675256,C=58)
	CT(52732)=(S=12675257,E=12675257,C=34)
	CT(52733)=(S=12675258,E=12675259,C=36)
	CT(52734)=(S=12675260,E=12675260,C=47)
	CT(52735)=(S=12675261,E=12675261,C=31)
	CT(52736)=(S=12675262,E=12675263,C=18)
	CT(52737)=(S=12675264,E=12675264,C=36)
	CT(52738)=(S=12675265,E=12675265,C=49)
	CT(52739)=(S=12675266,E=12675266,C=28)
	CT(52740)=(S=12675267,E=12675267,C=18)
	CT(52741)=(S=12675268,E=12675268,C=47)
	CT(52742)=(S=12675269,E=12675269,C=18)
	CT(52743)=(S=12675270,E=12675270,C=14)
	CT(52744)=(S=12675271,E=12675271,C=12)
	CT(52745)=(S=12675272,E=12675272,C=22)
	CT(52746)=(S=12675273,E=12675273,C=37)
	CT(52747)=(S=12675274,E=12675274,C=24)
	CT(52748)=(S=12675275,E=12675275,C=31)
	CT(52749)=(S=12675276,E=12675276,C=23)
	CT(52750)=(S=12675277,E=12675277,C=12)
	CT(52751)=(S=12675278,E=12675278,C=44)
	CT(52752)=(S=12675279,E=12675279,C=18)
	CT(52753)=(S=12675280,E=12675282,C=31)
	CT(52754)=(S=12675284,E=12675284,C=26)
	CT(52755)=(S=12675285,E=12675285,C=31)
	CT(52756)=(S=12675286,E=12675286,C=19)
	CT(52757)=(S=12675287,E=12675287,C=63)
	CT(52758)=(S=12675288,E=12675288,C=28)
	CT(52759)=(S=12675289,E=12675289,C=19)
	CT(52760)=(S=12675290,E=12675290,C=25)
	CT(52761)=(S=12675291,E=12675291,C=12)
	CT(52762)=(S=12675292,E=12675292,C=28)
	CT(52763)=(S=12675293,E=12675293,C=58)
	CT(52764)=(S=12675295,E=12675295,C=15)
	CT(52765)=(S=12675296,E=12675299,C=18)
	CT(52766)=(S=12675300,E=12675300,C=14)
	CT(52767)=(S=12675301,E=12675301,C=61)
	CT(52768)=(S=12675302,E=12675302,C=18)
	CT(52769)=(S=12675303,E=12675303,C=12)
	CT(52770)=(S=12675304,E=12675304,C=42)
	CT(52771)=(S=12675305,E=12675305,C=14)
	CT(52772)=(S=12675306,E=12675306,C=18)
	CT(52773)=(S=12675307,E=12675307,C=58)
	CT(52774)=(S=12675308,E=12675308,C=52)
	CT(52775)=(S=12675309,E=12675309,C=18)
	CT(52776)=(S=12675310,E=12675310,C=34)
	CT(52777)=(S=12675311,E=12675311,C=58)
	CT(52778)=(S=12675312,E=12675312,C=52)
	CT(52779)=(S=12675313,E=12675313,C=18)
	CT(52780)=(S=12675314,E=12675314,C=31)
	CT(52781)=(S=12675315,E=12675315,C=18)
	CT(52782)=(S=12675316,E=12675316,C=58)
	CT(52783)=(S=12675317,E=12675317,C=18)
	CT(52784)=(S=12675318,E=12675318,C=24)
	CT(52785)=(S=12675319,E=12675319,C=38)
	CT(52786)=(S=12675320,E=12675320,C=18)
	CT(52787)=(S=12675321,E=12675321,C=31)
	CT(52788)=(S=12675322,E=12675322,C=15)
	CT(52789)=(S=12675323,E=12675323,C=58)
	CT(52790)=(S=12675325,E=12675325,C=18)
	CT(52791)=(S=12675326,E=12675326,C=33)
	CT(52792)=(S=12675327,E=12675327,C=36)
	CT(52793)=(S=12675328,E=12675328,C=28)
	CT(52794)=(S=12675329,E=12675329,C=12)
	CT(52795)=(S=12675330,E=12675330,C=26)
	CT(52796)=(S=12675331,E=12675331,C=36)
	CT(52797)=(S=12675332,E=12675332,C=22)
	CT(52798)=(S=12675333,E=12675333,C=34)
	CT(52799)=(S=12675334,E=12675334,C=26)
	CT(52800)=(S=12675335,E=12675335,C=31)
	CT(52801)=(S=12675336,E=12675336,C=39)
	CT(52802)=(S=12675337,E=12675337,C=34)
	CT(52803)=(S=12675338,E=12675338,C=24)
	CT(52804)=(S=12675339,E=12675339,C=18)
	CT(52805)=(S=12675340,E=12675340,C=36)
	CT(52806)=(S=12675341,E=12675341,C=14)
	CT(52807)=(S=12675342,E=12675342,C=18)
	CT(52808)=(S=12675343,E=12675343,C=49)
	CT(52809)=(S=12675344,E=12675344,C=58)
	CT(52810)=(S=12675345,E=12675345,C=28)
	CT(52811)=(S=12675346,E=12675346,C=12)
	CT(52812)=(S=12675347,E=12675347,C=28)
	CT(52813)=(S=12675348,E=12675348,C=58)
	CT(52814)=(S=12675349,E=12675349,C=38)
	CT(52815)=(S=12675350,E=12675350,C=52)
	CT(52816)=(S=12675351,E=12675351,C=57)
	CT(52817)=(S=12675352,E=12675352,C=58)
	CT(52818)=(S=12675353,E=12675353,C=36)
	CT(52819)=(S=12675354,E=12675354,C=12)
	CT(52820)=(S=12675355,E=12675355,C=18)
	CT(52821)=(S=12675357,E=12675357,C=58)
	CT(52822)=(S=12675358,E=12675358,C=31)
	CT(52823)=(S=12675360,E=12675360,C=58)
	CT(52824)=(S=12675361,E=12675361,C=36)
	CT(52825)=(S=12675362,E=12675362,C=25)
	CT(52826)=(S=12675363,E=12675363,C=58)
	CT(52827)=(S=12675364,E=12675364,C=49)
	CT(52828)=(S=12675365,E=12675365,C=18)
	CT(52829)=(S=12675366,E=12675366,C=28)
	CT(52830)=(S=12675367,E=12675368,C=18)
	CT(52831)=(S=12675369,E=12675369,C=31)
	CT(52832)=(S=12675370,E=12675370,C=22)
	CT(52833)=(S=12675371,E=12675371,C=12)
	CT(52834)=(S=12675372,E=12675372,C=24)
	CT(52835)=(S=12675373,E=12675373,C=58)
	CT(52836)=(S=12675374,E=12675374,C=28)
	CT(52837)=(S=12675375,E=12675376,C=14)
	CT(52838)=(S=12675377,E=12675377,C=44)
	CT(52839)=(S=12675378,E=12675378,C=36)
	CT(52840)=(S=12675379,E=12675379,C=14)
	CT(52841)=(S=12675380,E=12675380,C=28)
	CT(52842)=(S=12675381,E=12675381,C=18)
	CT(52843)=(S=12675382,E=12675382,C=14)
	CT(52844)=(S=12675383,E=12675383,C=28)
	CT(52845)=(S=12675384,E=12675384,C=31)
	CT(52846)=(S=12675385,E=12675385,C=69)
	CT(52847)=(S=12675386,E=12675386,C=38)
	CT(52848)=(S=12675387,E=12675387,C=18)
	CT(52849)=(S=12675388,E=12675388,C=51)
	CT(52850)=(S=12675389,E=12675389,C=14)
	CT(52851)=(S=12675390,E=12675390,C=31)
	CT(52852)=(S=12675391,E=12675391,C=58)
	CT(52853)=(S=12675392,E=12675392,C=38)
	CT(52854)=(S=12675393,E=12675393,C=18)
	CT(52855)=(S=12675394,E=12675394,C=58)
	CT(52856)=(S=12675395,E=12675395,C=52)
	CT(52857)=(S=12675396,E=12675396,C=28)
	CT(52858)=(S=12675397,E=12675397,C=14)
	CT(52859)=(S=12675398,E=12675398,C=31)
	CT(52860)=(S=12675399,E=12675400,C=18)
	CT(52861)=(S=12675401,E=12675401,C=49)
	CT(52862)=(S=12675402,E=12675402,C=64)
	CT(52863)=(S=12675403,E=12675403,C=58)
	CT(52864)=(S=12675404,E=12675404,C=47)
	CT(52865)=(S=12675405,E=12675405,C=27)
	CT(52866)=(S=12675406,E=12675406,C=38)
	CT(52867)=(S=12675407,E=12675407,C=15)
	CT(52868)=(S=12675408,E=12675408,C=22)
	CT(52869)=(S=12675410,E=12675410,C=68)
	CT(52870)=(S=12675411,E=12675411,C=53)
	CT(52871)=(S=12675413,E=12675413,C=22)
	CT(52872)=(S=12675414,E=12675414,C=42)
	CT(52873)=(S=12675415,E=12675415,C=28)
	CT(52874)=(S=12675417,E=12675417,C=23)
	CT(52875)=(S=12675418,E=12675418,C=12)
	CT(52876)=(S=12675419,E=12675419,C=38)
	CT(52877)=(S=12675420,E=12675420,C=31)
	CT(52878)=(S=12675421,E=12675422,C=18)
	CT(52879)=(S=12675425,E=12675425,C=47)
	CT(52880)=(S=12675426,E=12675426,C=14)
	CT(52881)=(S=12675427,E=12675427,C=17)
	CT(52882)=(S=12675428,E=12675428,C=18)
	CT(52883)=(S=12675429,E=12675429,C=36)
	CT(52884)=(S=12675430,E=12675430,C=32)
	CT(52885)=(S=12675431,E=12675431,C=28)
	CT(52886)=(S=12675432,E=12675432,C=14)
	CT(52887)=(S=12675433,E=12675433,C=28)
	CT(52888)=(S=12675434,E=12675434,C=49)
	CT(52889)=(S=12675435,E=12675435,C=18)
	CT(52890)=(S=12675436,E=12675436,C=14)
	CT(52891)=(S=12675437,E=12675437,C=22)
	CT(52892)=(S=12675438,E=12675438,C=58)
	CT(52893)=(S=12675439,E=12675439,C=36)
	CT(52894)=(S=12675440,E=12675440,C=27)
	CT(52895)=(S=12675441,E=12675441,C=19)
	CT(52896)=(S=12675442,E=12675442,C=18)
	CT(52897)=(S=12675443,E=12675443,C=52)
	CT(52898)=(S=12675444,E=12675444,C=24)
	CT(52899)=(S=12675445,E=12675446,C=18)
	CT(52900)=(S=12675447,E=12675447,C=23)
	CT(52901)=(S=12675448,E=12675448,C=14)
	CT(52902)=(S=12675450,E=12675450,C=58)
	CT(52903)=(S=12675451,E=12675451,C=47)
	CT(52904)=(S=12675452,E=12675452,C=31)
	CT(52905)=(S=12675453,E=12675453,C=58)
	CT(52906)=(S=12675454,E=12675454,C=18)
	CT(52907)=(S=12675455,E=12675455,C=52)
	CT(52908)=(S=12675456,E=12675456,C=12)
	CT(52909)=(S=12675457,E=12675457,C=58)
	CT(52910)=(S=12675458,E=12675458,C=19)
	CT(52911)=(S=12675459,E=12675459,C=42)
	CT(52912)=(S=12675460,E=12675460,C=18)
	CT(52913)=(S=12675461,E=12675461,C=49)
	CT(52914)=(S=12675462,E=12675462,C=19)
	CT(52915)=(S=12675463,E=12675463,C=31)
	CT(52916)=(S=12675464,E=12675464,C=34)
	CT(52917)=(S=12675465,E=12675465,C=28)
	CT(52918)=(S=12675466,E=12675466,C=19)
	CT(52919)=(S=12675467,E=12675467,C=28)
	CT(52920)=(S=12675468,E=12675468,C=38)
	CT(52921)=(S=12675470,E=12675470,C=71)
	CT(52922)=(S=12675471,E=12675471,C=58)
	CT(52923)=(S=12675472,E=12675472,C=36)
	CT(52924)=(S=12675473,E=12675473,C=63)
	CT(52925)=(S=12675474,E=12675474,C=44)
	CT(52926)=(S=12675475,E=12675475,C=28)
	CT(52927)=(S=12675476,E=12675476,C=51)
	CT(52928)=(S=12675477,E=12675477,C=24)
	CT(52929)=(S=12675478,E=12675478,C=14)
	CT(52930)=(S=12675480,E=12675480,C=31)
	CT(52931)=(S=12675481,E=12675481,C=42)
	CT(52932)=(S=12675483,E=12675483,C=60)
	CT(52933)=(S=12675484,E=12675484,C=18)
	CT(52934)=(S=12675485,E=12675487,C=33)
	CT(52935)=(S=12675488,E=12675488,C=47)
	CT(52936)=(S=12675489,E=12675489,C=34)
	CT(52937)=(S=12675491,E=12675491,C=46)
	CT(52938)=(S=12675492,E=12675492,C=31)
	CT(52939)=(S=12675493,E=12675493,C=36)
	CT(52940)=(S=12675494,E=12675494,C=60)
	CT(52941)=(S=12675495,E=12675495,C=58)
	CT(52942)=(S=12675496,E=12675496,C=28)
	CT(52943)=(S=12675497,E=12675497,C=18)
	CT(52944)=(S=12675498,E=12675498,C=14)
	CT(52945)=(S=12675499,E=12675499,C=31)
	CT(52946)=(S=12675500,E=12675500,C=14)
	CT(52947)=(S=12675501,E=12675501,C=36)
	CT(52948)=(S=12675502,E=12675503,C=18)
	CT(52949)=(S=12675504,E=12675505,C=38)
	CT(52950)=(S=12675506,E=12675506,C=58)
	CT(52951)=(S=12675507,E=12675507,C=18)
	CT(52952)=(S=12675508,E=12675508,C=58)
	CT(52953)=(S=12675509,E=12675509,C=42)
	CT(52954)=(S=12675510,E=12675510,C=14)
	CT(52955)=(S=12675511,E=12675511,C=36)
	CT(52956)=(S=12675512,E=12675512,C=18)
	CT(52957)=(S=12675513,E=12675513,C=38)
	CT(52958)=(S=12675514,E=12675514,C=12)
	CT(52959)=(S=12675515,E=12675515,C=36)
	CT(52960)=(S=12675516,E=12675516,C=14)
	CT(52961)=(S=12675517,E=12675517,C=31)
	CT(52962)=(S=12675518,E=12675518,C=24)
	CT(52963)=(S=12675519,E=12675520,C=58)
	CT(52964)=(S=12675521,E=12675521,C=57)
	CT(52965)=(S=12675522,E=12675522,C=14)
	CT(52966)=(S=12675523,E=12675523,C=42)
	CT(52967)=(S=12675524,E=12675524,C=51)
	CT(52968)=(S=12675525,E=12675525,C=12)
	CT(52969)=(S=12675526,E=12675526,C=18)
	CT(52970)=(S=12675527,E=12675527,C=17)
	CT(52971)=(S=12675528,E=12675528,C=36)
	CT(52972)=(S=12675529,E=12675529,C=31)
	CT(52973)=(S=12675530,E=12675530,C=58)
	CT(52974)=(S=12675531,E=12675531,C=17)
	CT(52975)=(S=12675532,E=12675534,C=42)
	CT(52976)=(S=12675535,E=12675535,C=20)
	CT(52977)=(S=12675536,E=12675536,C=37)
	CT(52978)=(S=12675537,E=12675537,C=14)
	CT(52979)=(S=12675538,E=12675538,C=31)
	CT(52980)=(S=12675539,E=12675539,C=37)
	CT(52981)=(S=12675540,E=12675540,C=14)
	CT(52982)=(S=12675541,E=12675541,C=18)
	CT(52983)=(S=12675542,E=12675542,C=42)
	CT(52984)=(S=12675543,E=12675543,C=36)
	CT(52985)=(S=12675544,E=12675544,C=31)
	CT(52986)=(S=12675546,E=12675546,C=14)
	CT(52987)=(S=12675547,E=12675547,C=31)
	CT(52988)=(S=12675548,E=12675548,C=24)
	CT(52989)=(S=12675549,E=12675549,C=58)
	CT(52990)=(S=12675550,E=12675550,C=18)
	CT(52991)=(S=12675551,E=12675551,C=28)
	CT(52992)=(S=12675552,E=12675552,C=32)
	CT(52993)=(S=12675553,E=12675553,C=12)
	CT(52994)=(S=12675554,E=12675554,C=19)
	CT(52995)=(S=12675555,E=12675555,C=58)
	CT(52996)=(S=12675556,E=12675556,C=15)
	CT(52997)=(S=12675557,E=12675557,C=58)
	CT(52998)=(S=12675558,E=12675558,C=28)
	CT(52999)=(S=12675559,E=12675559,C=14)
	CT(53000)=(S=12675560,E=12675560,C=12)
	CT(53001)=(S=12675561,E=12675561,C=36)
	CT(53002)=(S=12675562,E=12675562,C=37)
	CT(53003)=(S=12675563,E=12675564,C=18)
	CT(53004)=(S=12675565,E=12675565,C=40)
	CT(53005)=(S=12675566,E=12675566,C=12)
	CT(53006)=(S=12675567,E=12675567,C=38)
	CT(53007)=(S=12675568,E=12675568,C=60)
	CT(53008)=(S=12675569,E=12675569,C=36)
	CT(53009)=(S=12675570,E=12675570,C=58)
	CT(53010)=(S=12675571,E=12675571,C=37)
	CT(53011)=(S=12675573,E=12675573,C=18)
	CT(53012)=(S=12675574,E=12675575,C=28)
	CT(53013)=(S=12675576,E=12675576,C=58)
	CT(53014)=(S=12675577,E=12675577,C=31)
	CT(53015)=(S=12675578,E=12675578,C=28)
	CT(53016)=(S=12675579,E=12675579,C=58)
	CT(53017)=(S=12675580,E=12675580,C=12)
	CT(53018)=(S=12675581,E=12675581,C=63)
	CT(53019)=(S=12675582,E=12675582,C=31)
	CT(53020)=(S=12675583,E=12675591,C=18)
	CT(53021)=(S=12675592,E=12675595,C=47)
	CT(53022)=(S=12675596,E=12675599,C=14)
	CT(53023)=(S=12675600,E=12675603,C=28)
	CT(53024)=(S=12675608,E=12675615,C=31)
	CT(53025)=(S=12675620,E=12675623,C=12)
	CT(53026)=(S=12675624,E=12675627,C=18)
	CT(53027)=(S=12675628,E=12675631,C=38)
	CT(53028)=(S=12675632,E=12675635,C=62)
	CT(53029)=(S=12675636,E=12675639,C=17)
	CT(53030)=(S=12675640,E=12675651,C=31)
	CT(53031)=(S=12675652,E=12675659,C=18)
	CT(53032)=(S=12675660,E=12675663,C=58)
	CT(53033)=(S=12675664,E=12675667,C=31)
	CT(53034)=(S=12675668,E=12675671,C=58)
	CT(53035)=(S=12675672,E=12675683,C=18)
	CT(53036)=(S=12675684,E=12675691,C=58)
	CT(53037)=(S=12675692,E=12675695,C=18)
	CT(53038)=(S=12675696,E=12675699,C=14)
	CT(53039)=(S=12675700,E=12675703,C=12)
	CT(53040)=(S=12675704,E=12675707,C=24)
	CT(53041)=(S=12675708,E=12675711,C=38)
	CT(53042)=(S=12675712,E=12675715,C=58)
	CT(53043)=(S=12675716,E=12675719,C=18)
	CT(53044)=(S=12675720,E=12675723,C=31)
	CT(53045)=(S=12675724,E=12675727,C=28)
	CT(53046)=(S=12675728,E=12675731,C=31)
	CT(53047)=(S=12675732,E=12675735,C=18)
	CT(53048)=(S=12675736,E=12675743,C=58)
	CT(53049)=(S=12675744,E=12675747,C=31)
	CT(53050)=(S=12675748,E=12675751,C=24)
	CT(53051)=(S=12675752,E=12675759,C=18)
	CT(53052)=(S=12675760,E=12675763,C=25)
	CT(53053)=(S=12675764,E=12675767,C=15)
	CT(53054)=(S=12675768,E=12675771,C=31)
	CT(53055)=(S=12675772,E=12675775,C=18)
	CT(53056)=(S=12675776,E=12675779,C=58)
	CT(53057)=(S=12675780,E=12675783,C=35)
	CT(53058)=(S=12675784,E=12675787,C=31)
	CT(53059)=(S=12675788,E=12675791,C=17)
	CT(53060)=(S=12675792,E=12675793,C=58)
	CT(53061)=(S=12675794,E=12675794,C=19)
	CT(53062)=(S=12675795,E=12675795,C=31)
	CT(53063)=(S=12675796,E=12675799,C=18)
	CT(53064)=(S=12675800,E=12675803,C=58)
	CT(53065)=(S=12675804,E=12675807,C=31)
	CT(53066)=(S=12675808,E=12675811,C=28)
	CT(53067)=(S=12675812,E=12675815,C=58)
	CT(53068)=(S=12675816,E=12675819,C=18)
	CT(53069)=(S=12675820,E=12675821,C=31)
	CT(53070)=(S=12675822,E=12675822,C=58)
	CT(53071)=(S=12675823,E=12675823,C=12)
	CT(53072)=(S=12675824,E=12675827,C=18)
	CT(53073)=(S=12675828,E=12675831,C=58)
	CT(53074)=(S=12675832,E=12675835,C=31)
	CT(53075)=(S=12675836,E=12675839,C=22)
	CT(53076)=(S=12675840,E=12675843,C=18)
	CT(53077)=(S=12675844,E=12675847,C=46)
	CT(53078)=(S=12675848,E=12675851,C=58)
	CT(53079)=(S=12675852,E=12675863,C=18)
	CT(53080)=(S=12675864,E=12675867,C=31)
	CT(53081)=(S=12675868,E=12675871,C=27)
	CT(53082)=(S=12675872,E=12675875,C=58)
	CT(53083)=(S=12675876,E=12675879,C=51)
	CT(53084)=(S=12675880,E=12675883,C=31)
	CT(53085)=(S=12675884,E=12675887,C=14)
	CT(53086)=(S=12675888,E=12675895,C=18)
	CT(53087)=(S=12675896,E=12675899,C=12)
	CT(53088)=(S=12675900,E=12675907,C=18)
	CT(53089)=(S=12675908,E=12675911,C=51)
	CT(53090)=(S=12675912,E=12675923,C=31)
	CT(53091)=(S=12675924,E=12675927,C=48)
	CT(53092)=(S=12675928,E=12675931,C=58)
	CT(53093)=(S=12675932,E=12675935,C=18)
	CT(53094)=(S=12675936,E=12675939,C=53)
	CT(53095)=(S=12675940,E=12675943,C=18)
	CT(53096)=(S=12675944,E=12675955,C=31)
	CT(53097)=(S=12675956,E=12675959,C=14)
	CT(53098)=(S=12675960,E=12675963,C=28)
	CT(53099)=(S=12675964,E=12675967,C=12)
	CT(53100)=(S=12675968,E=12675979,C=31)
	CT(53101)=(S=12675980,E=12675983,C=18)
	CT(53102)=(S=12675984,E=12675987,C=28)
	CT(53103)=(S=12675988,E=12675995,C=18)
	CT(53104)=(S=12675996,E=12675999,C=42)
	CT(53105)=(S=12676000,E=12676003,C=33)
	CT(53106)=(S=12676004,E=12676007,C=58)
	CT(53107)=(S=12676008,E=12676019,C=31)
	CT(53108)=(S=12676020,E=12676035,C=18)
	CT(53109)=(S=12676036,E=12676039,C=58)
	CT(53110)=(S=12676040,E=12676043,C=31)
	CT(53111)=(S=12676044,E=12676047,C=14)
	CT(53112)=(S=12676052,E=12676055,C=58)
	CT(53113)=(S=12676056,E=12676059,C=18)
	CT(53114)=(S=12676060,E=12676063,C=36)
	CT(53115)=(S=12676064,E=12676067,C=31)
	CT(53116)=(S=12676068,E=12676079,C=18)
	CT(53117)=(S=12676080,E=12676083,C=12)
	CT(53118)=(S=12676084,E=12676087,C=31)
	CT(53119)=(S=12676088,E=12676091,C=58)
	CT(53120)=(S=12676092,E=12676095,C=33)
	CT(53121)=(S=12676096,E=12676103,C=19)
	CT(53122)=(S=12676104,E=12676112,C=28)
	CT(53123)=(S=12676113,E=12676113,C=49)
	CT(53124)=(S=12676114,E=12676114,C=14)
	CT(53125)=(S=12676115,E=12676115,C=28)
	CT(53126)=(S=12676117,E=12676117,C=12)
	CT(53127)=(S=12676118,E=12676118,C=42)
	CT(53128)=(S=12676119,E=12676119,C=110)
	CT(53129)=(S=12676120,E=12676120,C=51)
	CT(53130)=(S=12676123,E=12676123,C=36)
	CT(53131)=(S=12676124,E=12676124,C=110)
	CT(53132)=(S=12676125,E=12676125,C=60)
	CT(53133)=(S=12676126,E=12676126,C=101)
	CT(53134)=(S=12676127,E=12676127,C=12)
	CT(53135)=(S=12676128,E=12676129,C=51)
	CT(53136)=(S=12676130,E=12676131,C=58)
	CT(53137)=(S=12676132,E=12676133,C=14)
	CT(53138)=(S=12676134,E=12676135,C=31)
	CT(53139)=(S=12676136,E=12676137,C=42)
	CT(53140)=(S=12676138,E=12676139,C=19)
	CT(53141)=(S=12676140,E=12676147,C=31)
	CT(53142)=(S=12676148,E=12676151,C=38)
	CT(53143)=(S=12676152,E=12676155,C=31)
	CT(53144)=(S=12676156,E=12676159,C=15)
	CT(53145)=(S=12676160,E=12676167,C=33)
	CT(53146)=(S=12676168,E=12676183,C=14)
	CT(53147)=(S=12676184,E=12676191,C=13)
	CT(53148)=(S=12676194,E=12676195,C=36)
	CT(53149)=(S=12676196,E=12676197,C=42)
	CT(53150)=(S=12676198,E=12676201,C=31)
	CT(53151)=(S=12676202,E=12676205,C=33)
	CT(53152)=(S=12676206,E=12676207,C=12)
	CT(53153)=(S=12676212,E=12676219,C=31)
	CT(53154)=(S=12676220,E=12676223,C=58)
	CT(53155)=(S=12676224,E=12676225,C=31)
	CT(53156)=(S=12676226,E=12676227,C=18)
	CT(53157)=(S=12676228,E=12676229,C=71)
	CT(53158)=(S=12676230,E=12676231,C=61)
	CT(53159)=(S=12676232,E=12676233,C=34)
	CT(53160)=(S=12676234,E=12676235,C=28)
	CT(53161)=(S=12676236,E=12676237,C=12)
	CT(53162)=(S=12676238,E=12676239,C=101)
	CT(53163)=(S=12676240,E=12676243,C=60)
	CT(53164)=(S=12676244,E=12676247,C=14)
	CT(53165)=(S=12676248,E=12676251,C=13)
	CT(53166)=(S=12676252,E=12676255,C=14)
	CT(53167)=(S=12676256,E=12676257,C=22)
	CT(53168)=(S=12676258,E=12676259,C=31)
	CT(53169)=(S=12676260,E=12676261,C=28)
	CT(53170)=(S=12676262,E=12676263,C=12)
	CT(53171)=(S=12676264,E=12676265,C=14)
	CT(53172)=(S=12676266,E=12676267,C=31)
	CT(53173)=(S=12676268,E=12676269,C=28)
	CT(53174)=(S=12676270,E=12676271,C=25)
	CT(53175)=(S=12676272,E=12676272,C=28)
	CT(53176)=(S=12676273,E=12676273,C=58)
	CT(53177)=(S=12676275,E=12676275,C=28)
	CT(53178)=(S=12676276,E=12676276,C=12)
	CT(53179)=(S=12676277,E=12676277,C=28)
	CT(53180)=(S=12676278,E=12676278,C=18)
	CT(53181)=(S=12676280,E=12676280,C=28)
	CT(53182)=(S=12676281,E=12676281,C=60)
	CT(53183)=(S=12676282,E=12676282,C=15)
	CT(53184)=(S=12676283,E=12676283,C=34)
	CT(53185)=(S=12676284,E=12676286,C=24)
	CT(53186)=(S=12676287,E=12676288,C=14)
	CT(53187)=(S=12676289,E=12676289,C=28)
	CT(53188)=(S=12676290,E=12676290,C=58)
	CT(53189)=(S=12676291,E=12676291,C=17)
	CT(53190)=(S=12676292,E=12676292,C=19)
	CT(53191)=(S=12676293,E=12676293,C=12)
	CT(53192)=(S=12676295,E=12676296,C=14)
	CT(53193)=(S=12676297,E=12676297,C=49)
	CT(53194)=(S=12676298,E=12676298,C=58)
	CT(53195)=(S=12676299,E=12676299,C=12)
	CT(53196)=(S=12676300,E=12676301,C=15)
	CT(53197)=(S=12676305,E=12676305,C=31)
	CT(53198)=(S=12676306,E=12676306,C=36)
	CT(53199)=(S=12676308,E=12676308,C=42)
	CT(53200)=(S=12676309,E=12676309,C=37)
	CT(53201)=(S=12676310,E=12676310,C=130)
	CT(53202)=(S=12676312,E=12676312,C=12)
	CT(53203)=(S=12676313,E=12676313,C=28)
	CT(53204)=(S=12676314,E=12676314,C=14)
	CT(53205)=(S=12676316,E=12676316,C=58)
	CT(53206)=(S=12676317,E=12676317,C=14)
	CT(53207)=(S=12676318,E=12676318,C=28)
	CT(53208)=(S=12676319,E=12676319,C=19)
	CT(53209)=(S=12676320,E=12676321,C=24)
	CT(53210)=(S=12676322,E=12676323,C=31)
	CT(53211)=(S=12676324,E=12676325,C=58)
	CT(53212)=(S=12676328,E=12676329,C=34)
	CT(53213)=(S=12676332,E=12676333,C=31)
	CT(53214)=(S=12676334,E=12676334,C=36)
	CT(53215)=(S=12676335,E=12676335,C=19)
	CT(53216)=(S=12676336,E=12676339,C=31)
	CT(53217)=(S=12676340,E=12676343,C=14)
	CT(53218)=(S=12676344,E=12676347,C=31)
	CT(53219)=(S=12676348,E=12676351,C=130)
	CT(53220)=(S=12676352,E=12676355,C=12)
	CT(53221)=(S=12676356,E=12676359,C=28)
	CT(53222)=(S=12676360,E=12676363,C=31)
	CT(53223)=(S=12676368,E=12676383,C=48)
	CT(53224)=(S=12676384,E=12676399,C=15)
	CT(53225)=(S=12676400,E=12676401,C=34)
	CT(53226)=(S=12676402,E=12676403,C=14)
	CT(53227)=(S=12676404,E=12676405,C=34)
	CT(53228)=(S=12676406,E=12676407,C=51)
	CT(53229)=(S=12676408,E=12676409,C=24)
	CT(53230)=(S=12676410,E=12676411,C=58)
	CT(53231)=(S=12676412,E=12676412,C=18)
	CT(53232)=(S=12676413,E=12676413,C=51)
	CT(53233)=(S=12676414,E=12676415,C=49)
	CT(53234)=(S=12676418,E=12676419,C=109)
	CT(53235)=(S=12676420,E=12676423,C=18)
	CT(53236)=(S=12676424,E=12676425,C=49)
	CT(53237)=(S=12676426,E=12676427,C=42)
	CT(53238)=(S=12676428,E=12676429,C=18)
	CT(53239)=(S=12676432,E=12676432,C=31)
	CT(53240)=(S=12676433,E=12676433,C=14)
	CT(53241)=(S=12676434,E=12676434,C=17)
	CT(53242)=(S=12676435,E=12676435,C=14)
	CT(53243)=(S=12676438,E=12676438,C=19)
	CT(53244)=(S=12676439,E=12676439,C=12)
	CT(53245)=(S=12676440,E=12676440,C=42)
	CT(53246)=(S=12676441,E=12676441,C=18)
	CT(53247)=(S=12676442,E=12676442,C=15)
	CT(53248)=(S=12676443,E=12676443,C=58)
	CT(53249)=(S=12676445,E=12676445,C=21)
	CT(53250)=(S=12676447,E=12676447,C=34)
	CT(53251)=(S=12676448,E=12676451,C=28)
	CT(53252)=(S=12676456,E=12676464,C=15)
	CT(53253)=(S=12676465,E=12676465,C=28)
	CT(53254)=(S=12676466,E=12676466,C=18)
	CT(53255)=(S=12676467,E=12676467,C=58)
	CT(53256)=(S=12676468,E=12676468,C=14)
	CT(53257)=(S=12676469,E=12676469,C=71)
	CT(53258)=(S=12676470,E=12676470,C=31)
	CT(53259)=(S=12676471,E=12676471,C=12)
	CT(53260)=(S=12676473,E=12676473,C=25)
	CT(53261)=(S=12676474,E=12676474,C=36)
	CT(53262)=(S=12676475,E=12676475,C=58)
	CT(53263)=(S=12676476,E=12676476,C=52)
	CT(53264)=(S=12676477,E=12676478,C=49)
	CT(53265)=(S=12676479,E=12676479,C=58)
	CT(53266)=(S=12676480,E=12676481,C=31)
	CT(53267)=(S=12676482,E=12676483,C=70)
	CT(53268)=(S=12676484,E=12676485,C=28)
	CT(53269)=(S=12676486,E=12676487,C=37)
	CT(53270)=(S=12676490,E=12676491,C=28)
	CT(53271)=(S=12676492,E=12676493,C=42)
	CT(53272)=(S=12676494,E=12676495,C=12)
	CT(53273)=(S=12676496,E=12676499,C=31)
	CT(53274)=(S=12676500,E=12676511,C=42)
	CT(53275)=(S=12676512,E=12676523,C=31)
	CT(53276)=(S=12676524,E=12676527,C=25)
	CT(53277)=(S=12676528,E=12676535,C=33)
	CT(53278)=(S=12676536,E=12676543,C=49)
	CT(53279)=(S=12676544,E=12676551,C=14)
	CT(53280)=(S=12676552,E=12676559,C=15)
	CT(53281)=(S=12676560,E=12676560,C=70)
	CT(53282)=(S=12676561,E=12676561,C=19)
	CT(53283)=(S=12676562,E=12676562,C=27)
	CT(53284)=(S=12676563,E=12676563,C=60)
	CT(53285)=(S=12676564,E=12676564,C=58)
	CT(53286)=(S=12676566,E=12676566,C=28)
	CT(53287)=(S=12676567,E=12676567,C=101)
	CT(53288)=(S=12676568,E=12676568,C=31)
	CT(53289)=(S=12676570,E=12676570,C=23)
	CT(53290)=(S=12676572,E=12676572,C=14)
	CT(53291)=(S=12676574,E=12676575,C=28)
	CT(53292)=(S=12676577,E=12676577,C=58)
	CT(53293)=(S=12676579,E=12676579,C=52)
	CT(53294)=(S=12676580,E=12676580,C=49)
	CT(53295)=(S=12676581,E=12676581,C=25)
	CT(53296)=(S=12676583,E=12676583,C=28)
	CT(53297)=(S=12676584,E=12676584,C=42)
	CT(53298)=(S=12676585,E=12676585,C=15)
	CT(53299)=(S=12676586,E=12676586,C=49)
	CT(53300)=(S=12676587,E=12676587,C=44)
	CT(53301)=(S=12676588,E=12676590,C=28)
	CT(53302)=(S=12676591,E=12676591,C=58)
	CT(53303)=(S=12676592,E=12676593,C=31)
	CT(53304)=(S=12676594,E=12676595,C=28)
	CT(53305)=(S=12676596,E=12676597,C=58)
	CT(53306)=(S=12676598,E=12676599,C=95)
	CT(53307)=(S=12676600,E=12676601,C=31)
	CT(53308)=(S=12676602,E=12676603,C=12)
	CT(53309)=(S=12676604,E=12676605,C=50)
	CT(53310)=(S=12676606,E=12676613,C=14)
	CT(53311)=(S=12676614,E=12676615,C=28)
	CT(53312)=(S=12676616,E=12676617,C=60)
	CT(53313)=(S=12676618,E=12676619,C=42)
	CT(53314)=(S=12676620,E=12676621,C=19)
	CT(53315)=(S=12676622,E=12676623,C=52)
	CT(53316)=(S=12676624,E=12676631,C=31)
	CT(53317)=(S=12676632,E=12676635,C=28)
	CT(53318)=(S=12676636,E=12676639,C=42)
	CT(53319)=(S=12676640,E=12676647,C=47)
	CT(53320)=(S=12676648,E=12676655,C=28)
	CT(53321)=(S=12676656,E=12676663,C=14)
	CT(53322)=(S=12676664,E=12676671,C=50)
	CT(53323)=(S=12676672,E=12676675,C=47)
	CT(53324)=(S=12676676,E=12676679,C=28)
	CT(53325)=(S=12676680,E=12676687,C=31)
	CT(53326)=(S=12676688,E=12676688,C=58)
	CT(53327)=(S=12676689,E=12676689,C=49)
	CT(53328)=(S=12676691,E=12676691,C=15)
	CT(53329)=(S=12676692,E=12676692,C=24)
	CT(53330)=(S=12676693,E=12676693,C=34)
	CT(53331)=(S=12676694,E=12676694,C=22)
	CT(53332)=(S=12676695,E=12676695,C=24)
	CT(53333)=(S=12676696,E=12676696,C=14)
	CT(53334)=(S=12676697,E=12676697,C=31)
	CT(53335)=(S=12676698,E=12676698,C=28)
	CT(53336)=(S=12676699,E=12676699,C=18)
	CT(53337)=(S=12676700,E=12676700,C=49)
	CT(53338)=(S=12676702,E=12676702,C=42)
	CT(53339)=(S=12676703,E=12676703,C=34)
	CT(53340)=(S=12676704,E=12676707,C=58)
	CT(53341)=(S=12676708,E=12676709,C=31)
	CT(53342)=(S=12676710,E=12676711,C=28)
	CT(53343)=(S=12676712,E=12676713,C=137)
	CT(53344)=(S=12676714,E=12676715,C=31)
	CT(53345)=(S=12676716,E=12676719,C=47)
	CT(53346)=(S=12676720,E=12676723,C=31)
	CT(53347)=(S=12676724,E=12676727,C=28)
	CT(53348)=(S=12676728,E=12676731,C=58)
	CT(53349)=(S=12676732,E=12676735,C=31)
	CT(53350)=(S=12676736,E=12676736,C=25)
	CT(53351)=(S=12676737,E=12676737,C=42)
	CT(53352)=(S=12676738,E=12676738,C=49)
	CT(53353)=(S=12676740,E=12676740,C=51)
	CT(53354)=(S=12676741,E=12676741,C=28)
	CT(53355)=(S=12676745,E=12676745,C=58)
	CT(53356)=(S=12676746,E=12676746,C=14)
	CT(53357)=(S=12676747,E=12676747,C=28)
	CT(53358)=(S=12676748,E=12676748,C=12)
	CT(53359)=(S=12676750,E=12676750,C=58)
	CT(53360)=(S=12676752,E=12676752,C=31)
	CT(53361)=(S=12676753,E=12676753,C=52)
	CT(53362)=(S=12676754,E=12676754,C=12)
	CT(53363)=(S=12676755,E=12676755,C=36)
	CT(53364)=(S=12676757,E=12676757,C=14)
	CT(53365)=(S=12676758,E=12676758,C=28)
	CT(53366)=(S=12676759,E=12676759,C=15)
	CT(53367)=(S=12676760,E=12676760,C=12)
	CT(53368)=(S=12676761,E=12676761,C=28)
	CT(53369)=(S=12676762,E=12676762,C=25)
	CT(53370)=(S=12676763,E=12676763,C=72)
	CT(53371)=(S=12676764,E=12676764,C=31)
	CT(53372)=(S=12676765,E=12676765,C=36)
	CT(53373)=(S=12676767,E=12676767,C=51)
	CT(53374)=(S=12676768,E=12676771,C=31)
	CT(53375)=(S=12676772,E=12676773,C=60)
	CT(53376)=(S=12676774,E=12676775,C=24)
	CT(53377)=(S=12676776,E=12676777,C=31)
	CT(53378)=(S=12676778,E=12676779,C=37)
	CT(53379)=(S=12676780,E=12676785,C=31)
	CT(53380)=(S=12676786,E=12676787,C=28)
	CT(53381)=(S=12676788,E=12676789,C=14)
	CT(53382)=(S=12676790,E=12676791,C=42)
	CT(53383)=(S=12676792,E=12676793,C=31)
	CT(53384)=(S=12676794,E=12676795,C=71)
	CT(53385)=(S=12676796,E=12676797,C=31)
	CT(53386)=(S=12676798,E=12676799,C=49)
	CT(53387)=(S=12676800,E=12676803,C=28)
	CT(53388)=(S=12676804,E=12676807,C=24)
	CT(53389)=(S=12676812,E=12676815,C=28)
	CT(53390)=(S=12676816,E=12676823,C=37)
	CT(53391)=(S=12676824,E=12676831,C=51)
	CT(53392)=(S=12676832,E=12676835,C=47)
	CT(53393)=(S=12676836,E=12676839,C=58)
	CT(53394)=(S=12676840,E=12676843,C=36)
	CT(53395)=(S=12676844,E=12676847,C=18)
	CT(53396)=(S=12676848,E=12676855,C=14)
	CT(53397)=(S=12676856,E=12676863,C=49)
	CT(53398)=(S=12676864,E=12676865,C=31)
	CT(53399)=(S=12676866,E=12676867,C=18)
	CT(53400)=(S=12676870,E=12676873,C=31)
	CT(53401)=(S=12676874,E=12676875,C=103)
	CT(53402)=(S=12676876,E=12676877,C=49)
	CT(53403)=(S=12676878,E=12676879,C=12)
	CT(53404)=(S=12676880,E=12676881,C=38)
	CT(53405)=(S=12676882,E=12676883,C=25)
	CT(53406)=(S=12676884,E=12676885,C=15)
	CT(53407)=(S=12676886,E=12676887,C=58)
	CT(53408)=(S=12676888,E=12676889,C=14)
	CT(53409)=(S=12676890,E=12676891,C=58)
	CT(53410)=(S=12676892,E=12676897,C=14)
	CT(53411)=(S=12676898,E=12676898,C=28)
	CT(53412)=(S=12676899,E=12676899,C=14)
	CT(53413)=(S=12676900,E=12676903,C=58)
	CT(53414)=(S=12676904,E=12676904,C=28)
	CT(53415)=(S=12676905,E=12676905,C=58)
	CT(53416)=(S=12676906,E=12676906,C=21)
	CT(53417)=(S=12676907,E=12676908,C=28)
	CT(53418)=(S=12676910,E=12676910,C=15)
	CT(53419)=(S=12676911,E=12676911,C=42)
	CT(53420)=(S=12676912,E=12676915,C=31)
	CT(53421)=(S=12676920,E=12676923,C=28)
	CT(53422)=(S=12676924,E=12676927,C=31)
	CT(53423)=(S=12676928,E=12676929,C=47)
	CT(53424)=(S=12676930,E=12676931,C=25)
	CT(53425)=(S=12676932,E=12676933,C=38)
	CT(53426)=(S=12676934,E=12676937,C=15)
	CT(53427)=(S=12676938,E=12676939,C=14)
	CT(53428)=(S=12676945,E=12676945,C=36)
	CT(53429)=(S=12676946,E=12676946,C=63)
	CT(53430)=(S=12676947,E=12676947,C=31)
	CT(53431)=(S=12676948,E=12676948,C=14)
	CT(53432)=(S=12676949,E=12676949,C=31)
	CT(53433)=(S=12676951,E=12676951,C=42)
	CT(53434)=(S=12676952,E=12676952,C=28)
	CT(53435)=(S=12676953,E=12676953,C=51)
	CT(53436)=(S=12676956,E=12676956,C=18)
	CT(53437)=(S=12676957,E=12676957,C=47)
	CT(53438)=(S=12676958,E=12676958,C=24)
	CT(53439)=(S=12676959,E=12676959,C=49)
	CT(53440)=(S=12676960,E=12676963,C=71)
	CT(53441)=(S=12676964,E=12676967,C=28)
	CT(53442)=(S=12676968,E=12676971,C=19)
	CT(53443)=(S=12676972,E=12676977,C=28)
	CT(53444)=(S=12676978,E=12676979,C=31)
	CT(53445)=(S=12676982,E=12676983,C=47)
	CT(53446)=(S=12676984,E=12676985,C=38)
	CT(53447)=(S=12676986,E=12676987,C=25)
	CT(53448)=(S=12676990,E=12676991,C=31)
	CT(53449)=(S=12676992,E=12676999,C=33)
	CT(53450)=(S=12677000,E=12677007,C=14)
	CT(53451)=(S=12677008,E=12677011,C=58)
	CT(53452)=(S=12677020,E=12677023,C=31)
	CT(53453)=(S=12677024,E=12677024,C=15)
	CT(53454)=(S=12677025,E=12677025,C=38)
	CT(53455)=(S=12677026,E=12677026,C=24)
	CT(53456)=(S=12677027,E=12677027,C=28)
	CT(53457)=(S=12677028,E=12677028,C=57)
	CT(53458)=(S=12677029,E=12677029,C=19)
	CT(53459)=(S=12677030,E=12677030,C=58)
	CT(53460)=(S=12677031,E=12677033,C=28)
	CT(53461)=(S=12677036,E=12677036,C=36)
	CT(53462)=(S=12677037,E=12677037,C=31)
	CT(53463)=(S=12677039,E=12677039,C=28)
	CT(53464)=(S=12677040,E=12677043,C=18)
	CT(53465)=(S=12677048,E=12677051,C=14)
	CT(53466)=(S=12677052,E=12677055,C=31)
	CT(53467)=(S=12677056,E=12677057,C=52)
	CT(53468)=(S=12677058,E=12677059,C=51)
	CT(53469)=(S=12677062,E=12677063,C=28)
	CT(53470)=(S=12677064,E=12677065,C=14)
	CT(53471)=(S=12677066,E=12677067,C=34)
	CT(53472)=(S=12677068,E=12677069,C=31)
	CT(53473)=(S=12677070,E=12677083,C=28)
	CT(53474)=(S=12677084,E=12677087,C=52)
	CT(53475)=(S=12677088,E=12677088,C=50)
	CT(53476)=(S=12677089,E=12677089,C=49)
	CT(53477)=(S=12677091,E=12677091,C=14)
	CT(53478)=(S=12677092,E=12677092,C=36)
	CT(53479)=(S=12677094,E=12677095,C=14)
	CT(53480)=(S=12677096,E=12677096,C=38)
	CT(53481)=(S=12677100,E=12677100,C=58)
	CT(53482)=(S=12677101,E=12677102,C=28)
	CT(53483)=(S=12677103,E=12677107,C=31)
	CT(53484)=(S=12677108,E=12677111,C=60)
	CT(53485)=(S=12677116,E=12677119,C=18)
	CT(53486)=(S=12677120,E=12679167,C=14)
	CT(53487)=(S=12679168,E=12679423,C=63)
	CT(53488)=(S=12679424,E=12679679,C=49)
	CT(53489)=(S=12679680,E=12680191,C=14)
	CT(53490)=(S=12680192,E=12680703,C=18)
	CT(53491)=(S=12680704,E=12680959,C=21)
	CT(53492)=(S=12680960,E=12681215,C=13)
	CT(53493)=(S=12681216,E=12682751,C=14)
	CT(53494)=(S=12682752,E=12683263,C=34)
	CT(53495)=(S=12683264,E=12683775,C=21)
	CT(53496)=(S=12683776,E=12683776,C=42)
	CT(53497)=(S=12683777,E=12683779,C=52)
	CT(53498)=(S=12683782,E=12683783,C=27)
	CT(53499)=(S=12683784,E=12683784,C=55)
	CT(53500)=(S=12683785,E=12683785,C=52)
	CT(53501)=(S=12683786,E=12683803,C=28)
	CT(53502)=(S=12683805,E=12683805,C=34)
	CT(53503)=(S=12683806,E=12683806,C=44)
	CT(53504)=(S=12683807,E=12683807,C=28)
	CT(53505)=(S=12683808,E=12683839,C=52)
	CT(53506)=(S=12683840,E=12683840,C=12)
	CT(53507)=(S=12683841,E=12683841,C=58)
	CT(53508)=(S=12683842,E=12683842,C=28)
	CT(53509)=(S=12683843,E=12683843,C=51)
	CT(53510)=(S=12683844,E=12683844,C=28)
	CT(53511)=(S=12683845,E=12683845,C=34)
	CT(53512)=(S=12683846,E=12683846,C=18)
	CT(53513)=(S=12683847,E=12683847,C=19)
	CT(53514)=(S=12683849,E=12683849,C=34)
	CT(53515)=(S=12683850,E=12683850,C=19)
	CT(53516)=(S=12683852,E=12683852,C=24)
	CT(53517)=(S=12683853,E=12683853,C=60)
	CT(53518)=(S=12683854,E=12683854,C=33)
	CT(53519)=(S=12683855,E=12683855,C=18)
	CT(53520)=(S=12683856,E=12683857,C=28)
	CT(53521)=(S=12683858,E=12683858,C=18)
	CT(53522)=(S=12683859,E=12683859,C=34)
	CT(53523)=(S=12683860,E=12683860,C=31)
	CT(53524)=(S=12683862,E=12683862,C=14)
	CT(53525)=(S=12683863,E=12683863,C=31)
	CT(53526)=(S=12683864,E=12683864,C=28)
	CT(53527)=(S=12683865,E=12683865,C=18)
	CT(53528)=(S=12683866,E=12683866,C=12)
	CT(53529)=(S=12683867,E=12683867,C=28)
	CT(53530)=(S=12683868,E=12683868,C=17)
	CT(53531)=(S=12683869,E=12683869,C=31)
	CT(53532)=(S=12683870,E=12683870,C=42)
	CT(53533)=(S=12683871,E=12683871,C=14)
	CT(53534)=(S=12683872,E=12683872,C=28)
	CT(53535)=(S=12683873,E=12683875,C=38)
	CT(53536)=(S=12683876,E=12683876,C=63)
	CT(53537)=(S=12683877,E=12683877,C=49)
	CT(53538)=(S=12683878,E=12683878,C=19)
	CT(53539)=(S=12683879,E=12683879,C=38)
	CT(53540)=(S=12683880,E=12683880,C=28)
	CT(53541)=(S=12683881,E=12683881,C=42)
	CT(53542)=(S=12683882,E=12683882,C=33)
	CT(53543)=(S=12683883,E=12683883,C=14)
	CT(53544)=(S=12683884,E=12683884,C=28)
	CT(53545)=(S=12683885,E=12683885,C=19)
	CT(53546)=(S=12683886,E=12683886,C=58)
	CT(53547)=(S=12683888,E=12683890,C=28)
	CT(53548)=(S=12683891,E=12683891,C=24)
	CT(53549)=(S=12683892,E=12683892,C=37)
	CT(53550)=(S=12683893,E=12683893,C=28)
	CT(53551)=(S=12683894,E=12683894,C=58)
	CT(53552)=(S=12683895,E=12683895,C=28)
	CT(53553)=(S=12683899,E=12683899,C=42)
	CT(53554)=(S=12683900,E=12683900,C=24)
	CT(53555)=(S=12683901,E=12683901,C=50)
	CT(53556)=(S=12683902,E=12683902,C=47)
	CT(53557)=(S=12683903,E=12683903,C=50)
	CT(53558)=(S=12683904,E=12683907,C=18)
	CT(53559)=(S=12683908,E=12683911,C=31)
	CT(53560)=(S=12683912,E=12683919,C=58)
	CT(53561)=(S=12683920,E=12683923,C=31)
	CT(53562)=(S=12683924,E=12683927,C=18)
	CT(53563)=(S=12683928,E=12683931,C=33)
	CT(53564)=(S=12683932,E=12683935,C=28)
	CT(53565)=(S=12683936,E=12683939,C=15)
	CT(53566)=(S=12683940,E=12683943,C=28)
	CT(53567)=(S=12683952,E=12683955,C=18)
	CT(53568)=(S=12683956,E=12683959,C=28)
	CT(53569)=(S=12683960,E=12683963,C=31)
	CT(53570)=(S=12683964,E=12683967,C=60)
	CT(53571)=(S=12683968,E=12683975,C=38)
	CT(53572)=(S=12683976,E=12683979,C=47)
	CT(53573)=(S=12683980,E=12683983,C=36)
	CT(53574)=(S=12683984,E=12683987,C=58)
	CT(53575)=(S=12683988,E=12683991,C=34)
	CT(53576)=(S=12683992,E=12683995,C=19)
	CT(53577)=(S=12683996,E=12684007,C=36)
	CT(53578)=(S=12684008,E=12684011,C=18)
	CT(53579)=(S=12684012,E=12684015,C=31)
	CT(53580)=(S=12684016,E=12684019,C=58)
	CT(53581)=(S=12684020,E=12684023,C=31)
	CT(53582)=(S=12684024,E=12684027,C=36)
	CT(53583)=(S=12684032,E=12684287,C=12)
	CT(53584)=(S=12684288,E=12684543,C=37)
	CT(53585)=(S=12684544,E=12684799,C=28)
	CT(53586)=(S=12684800,E=12684800,C=51)
	CT(53587)=(S=12684801,E=12684817,C=47)
	CT(53588)=(S=12684819,E=12684819,C=66)
	CT(53589)=(S=12684823,E=12684826,C=47)
	CT(53590)=(S=12684830,E=12684831,C=18)
	CT(53591)=(S=12684850,E=12684851,C=47)
	CT(53592)=(S=12684853,E=12684853,C=28)
	CT(53593)=(S=12684855,E=12684855,C=47)
	CT(53594)=(S=12684856,E=12684857,C=18)
	CT(53595)=(S=12684861,E=12684861,C=19)
	CT(53596)=(S=12684862,E=12684862,C=34)
	CT(53597)=(S=12684864,E=12684896,C=47)
	CT(53598)=(S=12684897,E=12684897,C=28)
	CT(53599)=(S=12684898,E=12684899,C=47)
	CT(53600)=(S=12684900,E=12684900,C=27)
	CT(53601)=(S=12684901,E=12684902,C=47)
	CT(53602)=(S=12684903,E=12684903,C=24)
	CT(53603)=(S=12684904,E=12684907,C=28)
	CT(53604)=(S=12684908,E=12684910,C=27)
	CT(53605)=(S=12684911,E=12684911,C=34)
	CT(53606)=(S=12684912,E=12684913,C=58)
	CT(53607)=(S=12684914,E=12684914,C=31)
	CT(53608)=(S=12684915,E=12684915,C=58)
	CT(53609)=(S=12684916,E=12684916,C=52)
	CT(53610)=(S=12684918,E=12684919,C=14)
	CT(53611)=(S=12684920,E=12684923,C=47)
	CT(53612)=(S=12684924,E=12684924,C=31)
	CT(53613)=(S=12684925,E=12684925,C=34)
	CT(53614)=(S=12684927,E=12684927,C=28)
	CT(53615)=(S=12684936,E=12684943,C=47)
	CT(53616)=(S=12684945,E=12684945,C=50)
	CT(53617)=(S=12684949,E=12684949,C=52)
	CT(53618)=(S=12684950,E=12684950,C=47)
	CT(53619)=(S=12684951,E=12684951,C=17)
	CT(53620)=(S=12684953,E=12684953,C=50)
	CT(53621)=(S=12684957,E=12684957,C=28)
	CT(53622)=(S=12684958,E=12684958,C=12)
	CT(53623)=(S=12684959,E=12684959,C=31)
	CT(53624)=(S=12684960,E=12684975,C=19)
	CT(53625)=(S=12684976,E=12685004,C=47)
	CT(53626)=(S=12685008,E=12685008,C=38)
	CT(53627)=(S=12685009,E=12685009,C=50)
	CT(53628)=(S=12685010,E=12685010,C=58)
	CT(53629)=(S=12685011,E=12685011,C=24)
	CT(53630)=(S=12685012,E=12685012,C=14)
	CT(53631)=(S=12685013,E=12685013,C=31)
	CT(53632)=(S=12685014,E=12685014,C=50)
	CT(53633)=(S=12685015,E=12685015,C=36)
	CT(53634)=(S=12685016,E=12685016,C=14)
	CT(53635)=(S=12685017,E=12685017,C=25)
	CT(53636)=(S=12685018,E=12685019,C=31)
	CT(53637)=(S=12685020,E=12685020,C=14)
	CT(53638)=(S=12685021,E=12685021,C=31)
	CT(53639)=(S=12685022,E=12685022,C=23)
	CT(53640)=(S=12685024,E=12685040,C=47)
	CT(53641)=(S=12685043,E=12685043,C=58)
	CT(53642)=(S=12685044,E=12685044,C=19)
	CT(53643)=(S=12685045,E=12685045,C=14)
	CT(53644)=(S=12685046,E=12685046,C=18)
	CT(53645)=(S=12685047,E=12685047,C=47)
	CT(53646)=(S=12685048,E=12685050,C=38)
	CT(53647)=(S=12685051,E=12685052,C=47)
	CT(53648)=(S=12685056,E=12685056,C=18)
	CT(53649)=(S=12685060,E=12685130,C=47)
	CT(53650)=(S=12685133,E=12685133,C=58)
	CT(53651)=(S=12685135,E=12685147,C=47)
	CT(53652)=(S=12685148,E=12685148,C=18)
	CT(53653)=(S=12685149,E=12685151,C=47)
	CT(53654)=(S=12685154,E=12685154,C=18)
	CT(53655)=(S=12685155,E=12685174,C=47)
	CT(53656)=(S=12685175,E=12685175,C=18)
	CT(53657)=(S=12685176,E=12685176,C=38)
	CT(53658)=(S=12685177,E=12685177,C=58)
	CT(53659)=(S=12685178,E=12685178,C=28)
	CT(53660)=(S=12685180,E=12685281,C=47)
	CT(53661)=(S=12685282,E=12685282,C=37)
	CT(53662)=(S=12685284,E=12685303,C=47)
	CT(53663)=(S=12685305,E=12685305,C=15)
	CT(53664)=(S=12685309,E=12685310,C=47)
	CT(53665)=(S=12685312,E=12686335,C=25)
	CT(53666)=(S=12686336,E=12686336,C=38)
	CT(53667)=(S=12686338,E=12686340,C=25)
	CT(53668)=(S=12686342,E=12686343,C=12)
	CT(53669)=(S=12686351,E=12686351,C=18)
	CT(53670)=(S=12686360,E=12686373,C=25)
	CT(53671)=(S=12686374,E=12686375,C=15)
	CT(53672)=(S=12686380,E=12686380,C=18)
	CT(53673)=(S=12686381,E=12686381,C=49)
	CT(53674)=(S=12686382,E=12686382,C=71)
	CT(53675)=(S=12686432,E=12686495,C=25)
	CT(53676)=(S=12686496,E=12686527,C=28)
	CT(53677)=(S=12686528,E=12686599,C=25)
	CT(53678)=(S=12686600,E=12686655,C=28)
	CT(53679)=(S=12686656,E=12686687,C=14)
	CT(53680)=(S=12686688,E=12686719,C=12)
	CT(53681)=(S=12686720,E=12686783,C=24)
	CT(53682)=(S=12686784,E=12686847,C=13)
	CT(53683)=(S=12686848,E=12686849,C=18)
	CT(53684)=(S=12686850,E=12686851,C=58)
	CT(53685)=(S=12686852,E=12686853,C=28)
	CT(53686)=(S=12686854,E=12686855,C=31)
	CT(53687)=(S=12686856,E=12686859,C=13)
	CT(53688)=(S=12686860,E=12686863,C=33)
	CT(53689)=(S=12686864,E=12686867,C=25)
	CT(53690)=(S=12686868,E=12686868,C=18)
	CT(53691)=(S=12686869,E=12686869,C=58)
	CT(53692)=(S=12686870,E=12686871,C=27)
	CT(53693)=(S=12686872,E=12686873,C=33)
	CT(53694)=(S=12686874,E=12686875,C=14)
	CT(53695)=(S=12686876,E=12686879,C=31)
	CT(53696)=(S=12686880,E=12686881,C=18)
	CT(53697)=(S=12686882,E=12686882,C=14)
	CT(53698)=(S=12686883,E=12686883,C=33)
	CT(53699)=(S=12686884,E=12686885,C=58)
	CT(53700)=(S=12686886,E=12686887,C=33)
	CT(53701)=(S=12686888,E=12686888,C=44)
	CT(53702)=(S=12686889,E=12686895,C=58)
	CT(53703)=(S=12686896,E=12686896,C=34)
	CT(53704)=(S=12686897,E=12686897,C=31)
	CT(53705)=(S=12686898,E=12686899,C=58)
	CT(53706)=(S=12686900,E=12686903,C=18)
	CT(53707)=(S=12686904,E=12686907,C=31)
	CT(53708)=(S=12686908,E=12686911,C=15)
	CT(53709)=(S=12686912,E=12686912,C=28)
	CT(53710)=(S=12686913,E=12686913,C=18)
	CT(53711)=(S=12686914,E=12686914,C=19)
	CT(53712)=(S=12686915,E=12686915,C=51)
	CT(53713)=(S=12686916,E=12686916,C=18)
	CT(53714)=(S=12686917,E=12686917,C=58)
	CT(53715)=(S=12686918,E=12686919,C=18)
	CT(53716)=(S=12686920,E=12686923,C=71)
	CT(53717)=(S=12686924,E=12686927,C=58)
	CT(53718)=(S=12686928,E=12686931,C=18)
	CT(53719)=(S=12686932,E=12686935,C=24)
	CT(53720)=(S=12686936,E=12686937,C=31)
	CT(53721)=(S=12686938,E=12686939,C=25)
	CT(53722)=(S=12686940,E=12686943,C=18)
	CT(53723)=(S=12686944,E=12686945,C=31)
	CT(53724)=(S=12686946,E=12686947,C=13)
	CT(53725)=(S=12686948,E=12686951,C=18)
	CT(53726)=(S=12686952,E=12686952,C=19)
	CT(53727)=(S=12686953,E=12686959,C=18)
	CT(53728)=(S=12686960,E=12686960,C=15)
	CT(53729)=(S=12686961,E=12686961,C=31)
	CT(53730)=(S=12686962,E=12686968,C=18)
	CT(53731)=(S=12686969,E=12686969,C=27)
	CT(53732)=(S=12686970,E=12686970,C=31)
	CT(53733)=(S=12686971,E=12686971,C=58)
	CT(53734)=(S=12686972,E=12686973,C=18)
	CT(53735)=(S=12686974,E=12686974,C=72)
	CT(53736)=(S=12686975,E=12686975,C=17)
	CT(53737)=(S=12686976,E=12686999,C=14)
	CT(53738)=(S=12687000,E=12687012,C=36)
	CT(53739)=(S=12687013,E=12687013,C=58)
	CT(53740)=(S=12687014,E=12687015,C=28)
	CT(53741)=(S=12687017,E=12687017,C=12)
	CT(53742)=(S=12687020,E=12687021,C=22)
	CT(53743)=(S=12687022,E=12687027,C=42)
	CT(53744)=(S=12687028,E=12687039,C=14)
	CT(53745)=(S=12687040,E=12687103,C=19)
	CT(53746)=(S=12687104,E=12687107,C=18)
	CT(53747)=(S=12687108,E=12687111,C=28)
	CT(53748)=(S=12687112,E=12687115,C=58)
	CT(53749)=(S=12687116,E=12687119,C=31)
	CT(53750)=(S=12687120,E=12687123,C=14)
	CT(53751)=(S=12687124,E=12687127,C=51)
	CT(53752)=(S=12687128,E=12687131,C=18)
	CT(53753)=(S=12687132,E=12687135,C=38)
	CT(53754)=(S=12687136,E=12687139,C=28)
	CT(53755)=(S=12687140,E=12687143,C=58)
	CT(53756)=(S=12687144,E=12687147,C=68)
	CT(53757)=(S=12687148,E=12687151,C=28)
	CT(53758)=(S=12687152,E=12687159,C=58)
	CT(53759)=(S=12687160,E=12687163,C=31)
	CT(53760)=(S=12687164,E=12687167,C=28)
	CT(53761)=(S=12687168,E=12687175,C=58)
	CT(53762)=(S=12687176,E=12687179,C=36)
	CT(53763)=(S=12687180,E=12687183,C=58)
	CT(53764)=(S=12687184,E=12687187,C=51)
	CT(53765)=(S=12687188,E=12687191,C=12)
	CT(53766)=(S=12687192,E=12687195,C=31)
	CT(53767)=(S=12687196,E=12687199,C=19)
	CT(53768)=(S=12687200,E=12687207,C=58)
	CT(53769)=(S=12687208,E=12687211,C=31)
	CT(53770)=(S=12687212,E=12687215,C=18)
	CT(53771)=(S=12687216,E=12687219,C=58)
	CT(53772)=(S=12687220,E=12687223,C=50)
	CT(53773)=(S=12687224,E=12687231,C=14)
	CT(53774)=(S=12687232,E=12687263,C=26)
	CT(53775)=(S=12687296,E=12687343,C=18)
	CT(53776)=(S=12687344,E=12687351,C=31)
	CT(53777)=(S=12687352,E=12687355,C=28)
	CT(53778)=(S=12687356,E=12687359,C=31)
	CT(53779)=(S=12687360,E=12687871,C=25)
	CT(53780)=(S=12687872,E=12688127,C=42)
	CT(53781)=(S=12688128,E=12688383,C=28)
	CT(53782)=(S=12688384,E=12688895,C=27)
	CT(53783)=(S=12688896,E=12689407,C=28)
	CT(53784)=(S=12689408,E=12689408,C=12)
	CT(53785)=(S=12689409,E=12689417,C=27)
	CT(53786)=(S=12689420,E=12689421,C=18)
	CT(53787)=(S=12689428,E=12689437,C=27)
	CT(53788)=(S=12689441,E=12689441,C=17)
	CT(53789)=(S=12689442,E=12689446,C=27)
	CT(53790)=(S=12689447,E=12689447,C=28)
	CT(53791)=(S=12689448,E=12689471,C=27)
	CT(53792)=(S=12689505,E=12689505,C=31)
	CT(53793)=(S=12689506,E=12689507,C=42)
	CT(53794)=(S=12689511,E=12689535,C=27)
	CT(53795)=(S=12689539,E=12689539,C=18)
	CT(53796)=(S=12689540,E=12689565,C=27)
	CT(53797)=(S=12689566,E=12689566,C=18)
	CT(53798)=(S=12689568,E=12689611,C=27)
	CT(53799)=(S=12689612,E=12689613,C=18)
	CT(53800)=(S=12689614,E=12689615,C=19)
	CT(53801)=(S=12689632,E=12689635,C=31)
	CT(53802)=(S=12689636,E=12689639,C=18)
	CT(53803)=(S=12689647,E=12689647,C=19)
	CT(53804)=(S=12689656,E=12689660,C=27)
	CT(53805)=(S=12689663,E=12689663,C=18)
	CT(53806)=(S=12689664,E=12689664,C=38)
	CT(53807)=(S=12689676,E=12689679,C=31)
	CT(53808)=(S=12689680,E=12689747,C=27)
	CT(53809)=(S=12689748,E=12689751,C=33)
	CT(53810)=(S=12689752,E=12689839,C=27)
	CT(53811)=(S=12689840,E=12689855,C=19)
	CT(53812)=(S=12689856,E=12689856,C=51)
	CT(53813)=(S=12689857,E=12689857,C=18)
	CT(53814)=(S=12689858,E=12689917,C=27)
	CT(53815)=(S=12689920,E=12690431,C=24)
	CT(53816)=(S=12690432,E=12690432,C=21)
	CT(53817)=(S=12690440,E=12690440,C=28)
	CT(53818)=(S=12690441,E=12690441,C=37)
	CT(53819)=(S=12690443,E=12690443,C=14)
	CT(53820)=(S=12690444,E=12690444,C=12)
	CT(53821)=(S=12690445,E=12690446,C=14)
	CT(53822)=(S=12690447,E=12690447,C=42)
	CT(53823)=(S=12690464,E=12690523,C=19)
	CT(53824)=(S=12690524,E=12690527,C=31)
	CT(53825)=(S=12690528,E=12690560,C=14)
	CT(53826)=(S=12690561,E=12690561,C=58)
	CT(53827)=(S=12690562,E=12690562,C=31)
	CT(53828)=(S=12690563,E=12690565,C=28)
	CT(53829)=(S=12690568,E=12690568,C=38)
	CT(53830)=(S=12690569,E=12690573,C=52)
	CT(53831)=(S=12690574,E=12690574,C=58)
	CT(53832)=(S=12690575,E=12690575,C=28)
	CT(53833)=(S=12690576,E=12690576,C=36)
	CT(53834)=(S=12690577,E=12690577,C=34)
	CT(53835)=(S=12690578,E=12690578,C=18)
	CT(53836)=(S=12690579,E=12690579,C=25)
	CT(53837)=(S=12690580,E=12690585,C=12)
	CT(53838)=(S=12690586,E=12690586,C=28)
	CT(53839)=(S=12690587,E=12690587,C=24)
	CT(53840)=(S=12690588,E=12690588,C=12)
	CT(53841)=(S=12690589,E=12690589,C=58)
	CT(53842)=(S=12690590,E=12690590,C=49)
	CT(53843)=(S=12690591,E=12690591,C=28)
	CT(53844)=(S=12690592,E=12690623,C=13)
	CT(53845)=(S=12690624,E=12690625,C=36)
	CT(53846)=(S=12690626,E=12690627,C=18)
	CT(53847)=(S=12690628,E=12690629,C=12)
	CT(53848)=(S=12690630,E=12690631,C=38)
	CT(53849)=(S=12690632,E=12690633,C=18)
	CT(53850)=(S=12690634,E=12690635,C=12)
	CT(53851)=(S=12690636,E=12690637,C=18)
	CT(53852)=(S=12690638,E=12690639,C=14)
	CT(53853)=(S=12690642,E=12690643,C=60)
	CT(53854)=(S=12690644,E=12690645,C=28)
	CT(53855)=(S=12690646,E=12690647,C=18)
	CT(53856)=(S=12690648,E=12690649,C=36)
	CT(53857)=(S=12690650,E=12690651,C=33)
	CT(53858)=(S=12690652,E=12690653,C=29)
	CT(53859)=(S=12690656,E=12690657,C=42)
	CT(53860)=(S=12690658,E=12690659,C=22)
	CT(53861)=(S=12690660,E=12690663,C=12)
	CT(53862)=(S=12690664,E=12690664,C=12)
	CT(53863)=(S=12690665,E=12690665,C=34)
	CT(53864)=(S=12690666,E=12690673,C=42)
	CT(53865)=(S=12690674,E=12690675,C=14)
	CT(53866)=(S=12690676,E=12690677,C=34)
	CT(53867)=(S=12690678,E=12690679,C=28)
	CT(53868)=(S=12690684,E=12690685,C=14)
	CT(53869)=(S=12690686,E=12690687,C=58)
	CT(53870)=(S=12690688,E=12690943,C=33)
	CT(53871)=(S=12690944,E=12691455,C=47)
	CT(53872)=(S=12691456,E=12691456,C=28)
	CT(53873)=(S=12691457,E=12691458,C=70)
	CT(53874)=(S=12691462,E=12691463,C=28)
	CT(53875)=(S=12691464,E=12691501,C=70)
	CT(53876)=(S=12691506,E=12691506,C=12)
	CT(53877)=(S=12691510,E=12691510,C=58)
	CT(53878)=(S=12691514,E=12691583,C=70)
	CT(53879)=(S=12691584,E=12691711,C=28)
	CT(53880)=(S=12691712,E=12691713,C=18)
	CT(53881)=(S=12691714,E=12691715,C=34)
	CT(53882)=(S=12691716,E=12691717,C=18)
	CT(53883)=(S=12691718,E=12691721,C=38)
	CT(53884)=(S=12691722,E=12691723,C=28)
	CT(53885)=(S=12691724,E=12691725,C=60)
	CT(53886)=(S=12691726,E=12691727,C=14)
	CT(53887)=(S=12691728,E=12691729,C=50)
	CT(53888)=(S=12691730,E=12691731,C=58)
	CT(53889)=(S=12691732,E=12691733,C=38)
	CT(53890)=(S=12691734,E=12691741,C=18)
	CT(53891)=(S=12691742,E=12691743,C=69)
	CT(53892)=(S=12691744,E=12691751,C=18)
	CT(53893)=(S=12691752,E=12691753,C=36)
	CT(53894)=(S=12691754,E=12691755,C=27)
	CT(53895)=(S=12691756,E=12691757,C=18)
	CT(53896)=(S=12691758,E=12691759,C=12)
	CT(53897)=(S=12691760,E=12691761,C=52)
	CT(53898)=(S=12691762,E=12691762,C=37)
	CT(53899)=(S=12691763,E=12691763,C=36)
	CT(53900)=(S=12691764,E=12691765,C=18)
	CT(53901)=(S=12691766,E=12691767,C=28)
	CT(53902)=(S=12691768,E=12691769,C=15)
	CT(53903)=(S=12691770,E=12691771,C=14)
	CT(53904)=(S=12691772,E=12691775,C=18)
	CT(53905)=(S=12691776,E=12691779,C=12)
	CT(53906)=(S=12691780,E=12691781,C=36)
	CT(53907)=(S=12691782,E=12691783,C=17)
	CT(53908)=(S=12691784,E=12691785,C=28)
	CT(53909)=(S=12691786,E=12691787,C=24)
	CT(53910)=(S=12691788,E=12691789,C=28)
	CT(53911)=(S=12691790,E=12691791,C=58)
	CT(53912)=(S=12691792,E=12691793,C=31)
	CT(53913)=(S=12691794,E=12691795,C=34)
	CT(53914)=(S=12691796,E=12691797,C=47)
	CT(53915)=(S=12691798,E=12691799,C=31)
	CT(53916)=(S=12691800,E=12691801,C=17)
	CT(53917)=(S=12691802,E=12691803,C=14)
	CT(53918)=(S=12691804,E=12691805,C=18)
	CT(53919)=(S=12691806,E=12691807,C=28)
	CT(53920)=(S=12691808,E=12691809,C=18)
	CT(53921)=(S=12691810,E=12691811,C=28)
	CT(53922)=(S=12691812,E=12691813,C=47)
	CT(53923)=(S=12691814,E=12691815,C=18)
	CT(53924)=(S=12691816,E=12691817,C=17)
	CT(53925)=(S=12691820,E=12691821,C=36)
	CT(53926)=(S=12691822,E=12691823,C=18)
	CT(53927)=(S=12691824,E=12691829,C=58)
	CT(53928)=(S=12691830,E=12691831,C=18)
	CT(53929)=(S=12691832,E=12691833,C=14)
	CT(53930)=(S=12691834,E=12691835,C=18)
	CT(53931)=(S=12691836,E=12691839,C=31)
	CT(53932)=(S=12691840,E=12691841,C=18)
	CT(53933)=(S=12691842,E=12691843,C=101)
	CT(53934)=(S=12691844,E=12691847,C=31)
	CT(53935)=(S=12691848,E=12691849,C=38)
	CT(53936)=(S=12691850,E=12691851,C=36)
	CT(53937)=(S=12691852,E=12691853,C=14)
	CT(53938)=(S=12691854,E=12691857,C=38)
	CT(53939)=(S=12691858,E=12691859,C=31)
	CT(53940)=(S=12691860,E=12691861,C=58)
	CT(53941)=(S=12691862,E=12691863,C=15)
	CT(53942)=(S=12691864,E=12691865,C=34)
	CT(53943)=(S=12691866,E=12691867,C=24)
	CT(53944)=(S=12691868,E=12691869,C=38)
	CT(53945)=(S=12691870,E=12691871,C=70)
	CT(53946)=(S=12691872,E=12691873,C=18)
	CT(53947)=(S=12691874,E=12691875,C=15)
	CT(53948)=(S=12691876,E=12691877,C=19)
	CT(53949)=(S=12691878,E=12691879,C=60)
	CT(53950)=(S=12691880,E=12691881,C=28)
	CT(53951)=(S=12691884,E=12691891,C=18)
	CT(53952)=(S=12691892,E=12691893,C=28)
	CT(53953)=(S=12691894,E=12691895,C=33)
	CT(53954)=(S=12691896,E=12691897,C=28)
	CT(53955)=(S=12691898,E=12691899,C=34)
	CT(53956)=(S=12691900,E=12691901,C=31)
	CT(53957)=(S=12691902,E=12691903,C=23)
	CT(53958)=(S=12691904,E=12691905,C=14)
	CT(53959)=(S=12691906,E=12691907,C=60)
	CT(53960)=(S=12691908,E=12691909,C=28)
	CT(53961)=(S=12691910,E=12691911,C=51)
	CT(53962)=(S=12691912,E=12691915,C=18)
	CT(53963)=(S=12691916,E=12691917,C=28)
	CT(53964)=(S=12691918,E=12691919,C=14)
	CT(53965)=(S=12691920,E=12691921,C=24)
	CT(53966)=(S=12691922,E=12691923,C=18)
	CT(53967)=(S=12691924,E=12691925,C=28)
	CT(53968)=(S=12691926,E=12691927,C=31)
	CT(53969)=(S=12691928,E=12691929,C=14)
	CT(53970)=(S=12691932,E=12691935,C=18)
	CT(53971)=(S=12691936,E=12691937,C=14)
	CT(53972)=(S=12691938,E=12691941,C=18)
	CT(53973)=(S=12691942,E=12691943,C=12)
	CT(53974)=(S=12691944,E=12691945,C=58)
	CT(53975)=(S=12691946,E=12691947,C=18)
	CT(53976)=(S=12691948,E=12691949,C=14)
	CT(53977)=(S=12691952,E=12691953,C=31)
	CT(53978)=(S=12691954,E=12691955,C=15)
	CT(53979)=(S=12691956,E=12691957,C=31)
	CT(53980)=(S=12691958,E=12691959,C=17)
	CT(53981)=(S=12691960,E=12691961,C=18)
	CT(53982)=(S=12691964,E=12691965,C=31)
	CT(53983)=(S=12691966,E=12691967,C=14)
	CT(53984)=(S=12691968,E=12692479,C=42)
	CT(53985)=(S=12692480,E=12692991,C=36)
	CT(53986)=(S=12692992,E=12693503,C=28)
	CT(53987)=(S=12693504,E=12693504,C=21)
	CT(53988)=(S=12693507,E=12693507,C=14)
	CT(53989)=(S=12693509,E=12693557,C=36)
	CT(53990)=(S=12693560,E=12693564,C=14)
	CT(53991)=(S=12693565,E=12693648,C=36)
	CT(53992)=(S=12693649,E=12693649,C=18)
	CT(53993)=(S=12693650,E=12693650,C=14)
	CT(53994)=(S=12693652,E=12693712,C=36)
	CT(53995)=(S=12693713,E=12693713,C=12)
	CT(53996)=(S=12693720,E=12693743,C=36)
	CT(53997)=(S=12693751,E=12693751,C=42)
	CT(53998)=(S=12693752,E=12693755,C=31)
	CT(53999)=(S=12693756,E=12693756,C=18)
	CT(54000)=(S=12693757,E=12694015,C=36)
	CT(54001)=(S=12694017,E=12694046,C=63)
	CT(54002)=(S=12694048,E=12694048,C=14)
	CT(54003)=(S=12694050,E=12694050,C=31)
	CT(54004)=(S=12694051,E=12694051,C=19)
	CT(54005)=(S=12694056,E=12694057,C=63)
	CT(54006)=(S=12694059,E=12694059,C=34)
	CT(54007)=(S=12694069,E=12694069,C=63)
	CT(54008)=(S=12694076,E=12694079,C=19)
	CT(54009)=(S=12694080,E=12694132,C=63)
	CT(54010)=(S=12694133,E=12694133,C=58)
	CT(54011)=(S=12694134,E=12694134,C=18)
	CT(54012)=(S=12694135,E=12694135,C=50)
	CT(54013)=(S=12694145,E=12694146,C=19)
	CT(54014)=(S=12694147,E=12694147,C=18)
	CT(54015)=(S=12694148,E=12694148,C=58)
	CT(54016)=(S=12694149,E=12694149,C=47)
	CT(54017)=(S=12694150,E=12694150,C=12)
	CT(54018)=(S=12694151,E=12694151,C=18)
	CT(54019)=(S=12694152,E=12694156,C=12)
	CT(54020)=(S=12694157,E=12694157,C=38)
	CT(54021)=(S=12694159,E=12694159,C=58)
	CT(54022)=(S=12694160,E=12694163,C=31)
	CT(54023)=(S=12694164,E=12694165,C=42)
	CT(54024)=(S=12694166,E=12694167,C=60)
	CT(54025)=(S=12694168,E=12694169,C=51)
	CT(54026)=(S=12694170,E=12694171,C=12)
	CT(54027)=(S=12694172,E=12694173,C=34)
	CT(54028)=(S=12694174,E=12694175,C=18)
	CT(54029)=(S=12694176,E=12694176,C=42)
	CT(54030)=(S=12694177,E=12694177,C=58)
	CT(54031)=(S=12694178,E=12694178,C=31)
	CT(54032)=(S=12694179,E=12694179,C=28)
	CT(54033)=(S=12694180,E=12694180,C=58)
	CT(54034)=(S=12694181,E=12694181,C=38)
	CT(54035)=(S=12694182,E=12694182,C=51)
	CT(54036)=(S=12694183,E=12694183,C=58)
	CT(54037)=(S=12694184,E=12694184,C=50)
	CT(54038)=(S=12694187,E=12694187,C=42)
	CT(54039)=(S=12694189,E=12694189,C=14)
	CT(54040)=(S=12694191,E=12694191,C=24)
	CT(54041)=(S=12694192,E=12694199,C=60)
	CT(54042)=(S=12694200,E=12694200,C=42)
	CT(54043)=(S=12694201,E=12694201,C=28)
	CT(54044)=(S=12694202,E=12694202,C=36)
	CT(54045)=(S=12694203,E=12694203,C=31)
	CT(54046)=(S=12694204,E=12694204,C=52)
	CT(54047)=(S=12694205,E=12694205,C=36)
	CT(54048)=(S=12694206,E=12694207,C=31)
	CT(54049)=(S=12694208,E=12694211,C=60)
	CT(54050)=(S=12694216,E=12694219,C=26)
	CT(54051)=(S=12694220,E=12694223,C=15)
	CT(54052)=(S=12694224,E=12694224,C=42)
	CT(54053)=(S=12694225,E=12694225,C=49)
	CT(54054)=(S=12694226,E=12694227,C=12)
	CT(54055)=(S=12694229,E=12694230,C=58)
	CT(54056)=(S=12694231,E=12694231,C=18)
	CT(54057)=(S=12694232,E=12694232,C=15)
	CT(54058)=(S=12694233,E=12694233,C=14)
	CT(54059)=(S=12694235,E=12694235,C=70)
	CT(54060)=(S=12694237,E=12694237,C=18)
	CT(54061)=(S=12694238,E=12694238,C=51)
	CT(54062)=(S=12694239,E=12694239,C=14)
	CT(54063)=(S=12694240,E=12694241,C=38)
	CT(54064)=(S=12694242,E=12694243,C=28)
	CT(54065)=(S=12694244,E=12694245,C=31)
	CT(54066)=(S=12694248,E=12694249,C=60)
	CT(54067)=(S=12694250,E=12694251,C=15)
	CT(54068)=(S=12694252,E=12694253,C=31)
	CT(54069)=(S=12694254,E=12694255,C=14)
	CT(54070)=(S=12694256,E=12694259,C=58)
	CT(54071)=(S=12694260,E=12694263,C=15)
	CT(54072)=(S=12694264,E=12694267,C=31)
	CT(54073)=(S=12694268,E=12694271,C=15)
	CT(54074)=(S=12694272,E=12694527,C=33)
	CT(54075)=(S=12694528,E=12695551,C=19)
	CT(54076)=(S=12695552,E=12696063,C=47)
	CT(54077)=(S=12696064,E=12696064,C=42)
	CT(54078)=(S=12696065,E=12696065,C=18)
	CT(54079)=(S=12696066,E=12696066,C=17)
	CT(54080)=(S=12696067,E=12696067,C=28)
	CT(54081)=(S=12696068,E=12696068,C=58)
	CT(54082)=(S=12696069,E=12696070,C=42)
	CT(54083)=(S=12696071,E=12696071,C=28)
	CT(54084)=(S=12696072,E=12696072,C=12)
	CT(54085)=(S=12696073,E=12696073,C=31)
	CT(54086)=(S=12696074,E=12696074,C=38)
	CT(54087)=(S=12696076,E=12696076,C=28)
	CT(54088)=(S=12696077,E=12696077,C=14)
	CT(54089)=(S=12696079,E=12696079,C=31)
	CT(54090)=(S=12696080,E=12696095,C=42)
	CT(54091)=(S=12696096,E=12696096,C=50)
	CT(54092)=(S=12696097,E=12696097,C=38)
	CT(54093)=(S=12696098,E=12696098,C=18)
	CT(54094)=(S=12696099,E=12696099,C=24)
	CT(54095)=(S=12696100,E=12696101,C=36)
	CT(54096)=(S=12696102,E=12696102,C=51)
	CT(54097)=(S=12696103,E=12696103,C=58)
	CT(54098)=(S=12696104,E=12696225,C=42)
	CT(54099)=(S=12696226,E=12696226,C=14)
	CT(54100)=(S=12696227,E=12696227,C=22)
	CT(54101)=(S=12696228,E=12696255,C=42)
	CT(54102)=(S=12696272,E=12696272,C=37)
	CT(54103)=(S=12696273,E=12696273,C=42)
	CT(54104)=(S=12696274,E=12696274,C=14)
	CT(54105)=(S=12696276,E=12696318,C=42)
	CT(54106)=(S=12696319,E=12696319,C=58)
	CT(54107)=(S=12696320,E=12696320,C=13)
	CT(54108)=(S=12696322,E=12696371,C=42)
	CT(54109)=(S=12696372,E=12696373,C=58)
	CT(54110)=(S=12696374,E=12696374,C=39)
	CT(54111)=(S=12696375,E=12696375,C=14)
	CT(54112)=(S=12696376,E=12696383,C=71)
	CT(54113)=(S=12696384,E=12696391,C=58)
	CT(54114)=(S=12696392,E=12696392,C=14)
	CT(54115)=(S=12696393,E=12696393,C=18)
	CT(54116)=(S=12696394,E=12696395,C=15)
	CT(54117)=(S=12696396,E=12696399,C=31)
	CT(54118)=(S=12696400,E=12696403,C=18)
	CT(54119)=(S=12696516,E=12696574,C=42)
	CT(54120)=(S=12696575,E=12696575,C=18)
	CT(54121)=(S=12696576,E=12696576,C=58)
	CT(54122)=(S=12696580,E=12696581,C=14)
	CT(54123)=(S=12696582,E=12696582,C=16)
	CT(54124)=(S=12696583,E=12696583,C=134)
	CT(54125)=(S=12696588,E=12696589,C=104)
	CT(54126)=(S=12696608,E=12696623,C=72)
	CT(54127)=(S=12696624,E=12696639,C=101)
	CT(54128)=(S=12696640,E=12696671,C=61)
	CT(54129)=(S=12696672,E=12696703,C=104)
	CT(54130)=(S=12696704,E=12696707,C=39)
	CT(54131)=(S=12696708,E=12696709,C=14)
	CT(54132)=(S=12696710,E=12696710,C=32)
	CT(54133)=(S=12696711,E=12696712,C=39)
	CT(54134)=(S=12696713,E=12696717,C=50)
	CT(54135)=(S=12696718,E=12696729,C=101)
	CT(54136)=(S=12696730,E=12696731,C=28)
	CT(54137)=(S=12696732,E=12696733,C=19)
	CT(54138)=(S=12696734,E=12696735,C=28)
	CT(54139)=(S=12696736,E=12696767,C=101)
	CT(54140)=(S=12696768,E=12696770,C=50)
	CT(54141)=(S=12696772,E=12696773,C=28)
	CT(54142)=(S=12696774,E=12696775,C=58)
	CT(54143)=(S=12696777,E=12696779,C=19)
	CT(54144)=(S=12696780,E=12696783,C=14)
	CT(54145)=(S=12696784,E=12696799,C=42)
	CT(54146)=(S=12696800,E=12696823,C=34)
	CT(54147)=(S=12696824,E=12696824,C=19)
	CT(54148)=(S=12696825,E=12696825,C=12)
	CT(54149)=(S=12696826,E=12696826,C=28)
	CT(54150)=(S=12696829,E=12696829,C=58)
	CT(54151)=(S=12696830,E=12696830,C=31)
	CT(54152)=(S=12696831,E=12696831,C=12)
	CT(54153)=(S=12696832,E=12696897,C=111)
	CT(54154)=(S=12696898,E=12696899,C=63)
	CT(54155)=(S=12696900,E=12696901,C=18)
	CT(54156)=(S=12696902,E=12696903,C=38)
	CT(54157)=(S=12696904,E=12696905,C=58)
	CT(54158)=(S=12696906,E=12696907,C=14)
	CT(54159)=(S=12696908,E=12696909,C=58)
	CT(54160)=(S=12696910,E=12696911,C=28)
	CT(54161)=(S=12696912,E=12696913,C=14)
	CT(54162)=(S=12696914,E=12696915,C=28)
	CT(54163)=(S=12696918,E=12696919,C=44)
	CT(54164)=(S=12696920,E=12696921,C=18)
	CT(54165)=(S=12696924,E=12696925,C=24)
	CT(54166)=(S=12696926,E=12696927,C=28)
	CT(54167)=(S=12696928,E=12696929,C=31)
	CT(54168)=(S=12696930,E=12696931,C=38)
	CT(54169)=(S=12696932,E=12696933,C=19)
	CT(54170)=(S=12696934,E=12696934,C=36)
	CT(54171)=(S=12696935,E=12696935,C=58)
	CT(54172)=(S=12696936,E=12696937,C=12)
	CT(54173)=(S=12696938,E=12696943,C=28)
	CT(54174)=(S=12696944,E=12696945,C=15)
	CT(54175)=(S=12696946,E=12696947,C=28)
	CT(54176)=(S=12696948,E=12696949,C=58)
	CT(54177)=(S=12696952,E=12696953,C=18)
	CT(54178)=(S=12696954,E=12696955,C=26)
	CT(54179)=(S=12696956,E=12696957,C=12)
	CT(54180)=(S=12696958,E=12696959,C=31)
	CT(54181)=(S=12696960,E=12696960,C=111)
	CT(54182)=(S=12696961,E=12696965,C=14)
	CT(54183)=(S=12696966,E=12696966,C=36)
	CT(54184)=(S=12696967,E=12696967,C=49)
	CT(54185)=(S=12696968,E=12696968,C=42)
	CT(54186)=(S=12696969,E=12696969,C=52)
	CT(54187)=(S=12696970,E=12696970,C=18)
	CT(54188)=(S=12696971,E=12696971,C=36)
	CT(54189)=(S=12696972,E=12696973,C=14)
	CT(54190)=(S=12696974,E=12696974,C=37)
	CT(54191)=(S=12696975,E=12696975,C=12)
	CT(54192)=(S=12696976,E=12696976,C=44)
	CT(54193)=(S=12696978,E=12696978,C=42)
	CT(54194)=(S=12696979,E=12696979,C=49)
	CT(54195)=(S=12696980,E=12696980,C=61)
	CT(54196)=(S=12696981,E=12696981,C=36)
	CT(54197)=(S=12696982,E=12696991,C=28)
	CT(54198)=(S=12696992,E=12697023,C=52)
	CT(54199)=(S=12697024,E=12697055,C=34)
	CT(54200)=(S=12697056,E=12697087,C=28)
	CT(54201)=(S=12697088,E=12697599,C=49)
	CT(54202)=(S=12697600,E=12697600,C=36)
	CT(54203)=(S=12697604,E=12697611,C=21)
	CT(54204)=(S=12697612,E=12697612,C=32)
	CT(54205)=(S=12697613,E=12697613,C=14)
	CT(54206)=(S=12697614,E=12697614,C=28)
	CT(54207)=(S=12697615,E=12697615,C=57)
	CT(54208)=(S=12697616,E=12697631,C=34)
	CT(54209)=(S=12697632,E=12697635,C=14)
	CT(54210)=(S=12697636,E=12697637,C=31)
	CT(54211)=(S=12697638,E=12697639,C=22)
	CT(54212)=(S=12697640,E=12697641,C=28)
	CT(54213)=(S=12697642,E=12697647,C=38)
	CT(54214)=(S=12697648,E=12697649,C=51)
	CT(54215)=(S=12697650,E=12697651,C=14)
	CT(54216)=(S=12697652,E=12697653,C=38)
	CT(54217)=(S=12697654,E=12697655,C=19)
	CT(54218)=(S=12697656,E=12697657,C=51)
	CT(54219)=(S=12697658,E=12697661,C=28)
	CT(54220)=(S=12697662,E=12697663,C=58)
	CT(54221)=(S=12697664,E=12697695,C=14)
	CT(54222)=(S=12697696,E=12697727,C=37)
	CT(54223)=(S=12697728,E=12697759,C=18)
	CT(54224)=(S=12697760,E=12697791,C=58)
	CT(54225)=(S=12697792,E=12697823,C=14)
	CT(54226)=(S=12697824,E=12697855,C=34)
	CT(54227)=(S=12697856,E=12697887,C=15)
	CT(54228)=(S=12697888,E=12697919,C=13)
	CT(54229)=(S=12697920,E=12697951,C=58)
	CT(54230)=(S=12697952,E=12697983,C=14)
	CT(54231)=(S=12697984,E=12698015,C=34)
	CT(54232)=(S=12698017,E=12698017,C=36)
	CT(54233)=(S=12698021,E=12698021,C=31)
	CT(54234)=(S=12698022,E=12698023,C=28)
	CT(54235)=(S=12698024,E=12698024,C=58)
	CT(54236)=(S=12698025,E=12698025,C=14)
	CT(54237)=(S=12698026,E=12698026,C=71)
	CT(54238)=(S=12698027,E=12698027,C=36)
	CT(54239)=(S=12698028,E=12698029,C=15)
	CT(54240)=(S=12698030,E=12698036,C=28)
	CT(54241)=(S=12698037,E=12698037,C=58)
	CT(54242)=(S=12698039,E=12698039,C=15)
	CT(54243)=(S=12698040,E=12698041,C=22)
	CT(54244)=(S=12698042,E=12698043,C=18)
	CT(54245)=(S=12698044,E=12698045,C=14)
	CT(54246)=(S=12698046,E=12698047,C=28)
	CT(54247)=(S=12698048,E=12698079,C=31)
	CT(54248)=(S=12698080,E=12698111,C=20)
	CT(54249)=(S=12698112,E=12698112,C=38)
	CT(54250)=(S=12698113,E=12698117,C=134)
	CT(54251)=(S=12698118,E=12698127,C=28)
	CT(54252)=(S=12698128,E=12698132,C=15)
	CT(54253)=(S=12698133,E=12698133,C=38)
	CT(54254)=(S=12698134,E=12698143,C=14)
	CT(54255)=(S=12698144,E=12698175,C=134)
	CT(54256)=(S=12698176,E=12698207,C=135)
	CT(54257)=(S=12698208,E=12698213,C=18)
	CT(54258)=(S=12698214,E=12698216,C=42)
	CT(54259)=(S=12698217,E=12698217,C=32)
	CT(54260)=(S=12698218,E=12698227,C=31)
	CT(54261)=(S=12698228,E=12698231,C=36)
	CT(54262)=(S=12698232,E=12698233,C=52)
	CT(54263)=(S=12698234,E=12698234,C=23)
	CT(54264)=(S=12698235,E=12698235,C=58)
	CT(54265)=(S=12698236,E=12698239,C=31)
	CT(54266)=(S=12698240,E=12698240,C=110)
	CT(54267)=(S=12698241,E=12698241,C=15)
	CT(54268)=(S=12698244,E=12698247,C=12)
	CT(54269)=(S=12698248,E=12698248,C=28)
	CT(54270)=(S=12698250,E=12698251,C=34)
	CT(54271)=(S=12698252,E=12698253,C=51)
	CT(54272)=(S=12698256,E=12698257,C=28)
	CT(54273)=(S=12698258,E=12698258,C=42)
	CT(54274)=(S=12698260,E=12698261,C=28)
	CT(54275)=(S=12698264,E=12698264,C=12)
	CT(54276)=(S=12698265,E=12698266,C=18)
	CT(54277)=(S=12698268,E=12698268,C=51)
	CT(54278)=(S=12698269,E=12698269,C=28)
	CT(54279)=(S=12698270,E=12698270,C=49)
	CT(54280)=(S=12698271,E=12698271,C=50)
	CT(54281)=(S=12698272,E=12698303,C=110)
	CT(54282)=(S=12698304,E=12698367,C=19)
	CT(54283)=(S=12698368,E=12698623,C=14)
	CT(54284)=(S=12698624,E=12699135,C=28)
	CT(54285)=(S=12699136,E=12699391,C=57)
	CT(54286)=(S=12699392,E=12699647,C=47)
	CT(54287)=(S=12699651,E=12699651,C=15)
	CT(54288)=(S=12699652,E=12699652,C=12)
	CT(54289)=(S=12699653,E=12699653,C=34)
	CT(54290)=(S=12699654,E=12699654,C=42)
	CT(54291)=(S=12699655,E=12699655,C=25)
	CT(54292)=(S=12699657,E=12699657,C=71)
	CT(54293)=(S=12699658,E=12699658,C=18)
	CT(54294)=(S=12699659,E=12699659,C=19)
	CT(54295)=(S=12699662,E=12699663,C=51)
	CT(54296)=(S=12699666,E=12699666,C=18)
	CT(54297)=(S=12699667,E=12699667,C=38)
	CT(54298)=(S=12699668,E=12699668,C=58)
	CT(54299)=(S=12699669,E=12699669,C=18)
	CT(54300)=(S=12699670,E=12699670,C=31)
	CT(54301)=(S=12699671,E=12699671,C=49)
	CT(54302)=(S=12699673,E=12699673,C=14)
	CT(54303)=(S=12699676,E=12699676,C=51)
	CT(54304)=(S=12699677,E=12699677,C=12)
	CT(54305)=(S=12699678,E=12699678,C=17)
	CT(54306)=(S=12699679,E=12699679,C=19)
	CT(54307)=(S=12699680,E=12699681,C=31)
	CT(54308)=(S=12699682,E=12699683,C=18)
	CT(54309)=(S=12699684,E=12699687,C=31)
	CT(54310)=(S=12699688,E=12699689,C=101)
	CT(54311)=(S=12699690,E=12699691,C=12)
	CT(54312)=(S=12699692,E=12699693,C=24)
	CT(54313)=(S=12699694,E=12699695,C=58)
	CT(54314)=(S=12699696,E=12699697,C=34)
	CT(54315)=(S=12699698,E=12699699,C=58)
	CT(54316)=(S=12699702,E=12699703,C=18)
	CT(54317)=(S=12699706,E=12699707,C=12)
	CT(54318)=(S=12699708,E=12699709,C=49)
	CT(54319)=(S=12699712,E=12699713,C=31)
	CT(54320)=(S=12699714,E=12699715,C=58)
	CT(54321)=(S=12699716,E=12699717,C=31)
	CT(54322)=(S=12699718,E=12699719,C=36)
	CT(54323)=(S=12699724,E=12699725,C=31)
	CT(54324)=(S=12699726,E=12699727,C=34)
	CT(54325)=(S=12699728,E=12699729,C=14)
	CT(54326)=(S=12699730,E=12699731,C=58)
	CT(54327)=(S=12699732,E=12699733,C=31)
	CT(54328)=(S=12699734,E=12699735,C=19)
	CT(54329)=(S=12699736,E=12699737,C=18)
	CT(54330)=(S=12699738,E=12699739,C=24)
	CT(54331)=(S=12699740,E=12699741,C=27)
	CT(54332)=(S=12699742,E=12699743,C=18)
	CT(54333)=(S=12699746,E=12699749,C=14)
	CT(54334)=(S=12699752,E=12699753,C=24)
	CT(54335)=(S=12699754,E=12699755,C=28)
	CT(54336)=(S=12699756,E=12699757,C=58)
	CT(54337)=(S=12699758,E=12699759,C=12)
	CT(54338)=(S=12699760,E=12699761,C=42)
	CT(54339)=(S=12699762,E=12699763,C=14)
	CT(54340)=(S=12699764,E=12699765,C=12)
	CT(54341)=(S=12699766,E=12699771,C=58)
	CT(54342)=(S=12699772,E=12699773,C=44)
	CT(54343)=(S=12699774,E=12699775,C=38)
	CT(54344)=(S=12699776,E=12699776,C=28)
	CT(54345)=(S=12699777,E=12699777,C=18)
	CT(54346)=(S=12699779,E=12699779,C=60)
	CT(54347)=(S=12699780,E=12699780,C=36)
	CT(54348)=(S=12699781,E=12699781,C=58)
	CT(54349)=(S=12699783,E=12699783,C=49)
	CT(54350)=(S=12699784,E=12699784,C=50)
	CT(54351)=(S=12699785,E=12699786,C=28)
	CT(54352)=(S=12699787,E=12699787,C=12)
	CT(54353)=(S=12699788,E=12699788,C=36)
	CT(54354)=(S=12699789,E=12699789,C=38)
	CT(54355)=(S=12699790,E=12699790,C=42)
	CT(54356)=(S=12699791,E=12699791,C=71)
	CT(54357)=(S=12699792,E=12699792,C=34)
	CT(54358)=(S=12699793,E=12699793,C=14)
	CT(54359)=(S=12699794,E=12699794,C=58)
	CT(54360)=(S=12699795,E=12699795,C=36)
	CT(54361)=(S=12699796,E=12699796,C=19)
	CT(54362)=(S=12699798,E=12699798,C=13)
	CT(54363)=(S=12699799,E=12699799,C=31)
	CT(54364)=(S=12699801,E=12699801,C=36)
	CT(54365)=(S=12699802,E=12699802,C=28)
	CT(54366)=(S=12699803,E=12699803,C=63)
	CT(54367)=(S=12699804,E=12699804,C=28)
	CT(54368)=(S=12699805,E=12699805,C=34)
	CT(54369)=(S=12699806,E=12699806,C=14)
	CT(54370)=(S=12699808,E=12699808,C=28)
	CT(54371)=(S=12699809,E=12699809,C=31)
	CT(54372)=(S=12699810,E=12699810,C=28)
	CT(54373)=(S=12699812,E=12699812,C=36)
	CT(54374)=(S=12699813,E=12699814,C=38)
	CT(54375)=(S=12699816,E=12699816,C=18)
	CT(54376)=(S=12699818,E=12699818,C=37)
	CT(54377)=(S=12699819,E=12699819,C=12)
	CT(54378)=(S=12699820,E=12699820,C=28)
	CT(54379)=(S=12699821,E=12699821,C=31)
	CT(54380)=(S=12699822,E=12699822,C=36)
	CT(54381)=(S=12699823,E=12699823,C=31)
	CT(54382)=(S=12699824,E=12699825,C=14)
	CT(54383)=(S=12699826,E=12699826,C=36)
	CT(54384)=(S=12699827,E=12699827,C=31)
	CT(54385)=(S=12699828,E=12699828,C=37)
	CT(54386)=(S=12699830,E=12699830,C=12)
	CT(54387)=(S=12699831,E=12699831,C=31)
	CT(54388)=(S=12699832,E=12699832,C=12)
	CT(54389)=(S=12699833,E=12699833,C=49)
	CT(54390)=(S=12699834,E=12699835,C=58)
	CT(54391)=(S=12699836,E=12699836,C=37)
	CT(54392)=(S=12699837,E=12699837,C=18)
	CT(54393)=(S=12699838,E=12699838,C=31)
	CT(54394)=(S=12699839,E=12699840,C=14)
	CT(54395)=(S=12699841,E=12699843,C=28)
	CT(54396)=(S=12699845,E=12699845,C=58)
	CT(54397)=(S=12699846,E=12699846,C=36)
	CT(54398)=(S=12699847,E=12699847,C=18)
	CT(54399)=(S=12699848,E=12699848,C=38)
	CT(54400)=(S=12699849,E=12699849,C=60)
	CT(54401)=(S=12699850,E=12699850,C=19)
	CT(54402)=(S=12699851,E=12699851,C=57)
	CT(54403)=(S=12699852,E=12699852,C=24)
	CT(54404)=(S=12699853,E=12699853,C=31)
	CT(54405)=(S=12699854,E=12699854,C=28)
	CT(54406)=(S=12699855,E=12699855,C=52)
	CT(54407)=(S=12699856,E=12699856,C=19)
	CT(54408)=(S=12699857,E=12699857,C=31)
	CT(54409)=(S=12699858,E=12699858,C=28)
	CT(54410)=(S=12699859,E=12699859,C=18)
	CT(54411)=(S=12699860,E=12699860,C=31)
	CT(54412)=(S=12699861,E=12699861,C=15)
	CT(54413)=(S=12699862,E=12699862,C=36)
	CT(54414)=(S=12699863,E=12699864,C=58)
	CT(54415)=(S=12699866,E=12699866,C=28)
	CT(54416)=(S=12699867,E=12699867,C=31)
	CT(54417)=(S=12699868,E=12699868,C=34)
	CT(54418)=(S=12699870,E=12699870,C=12)
	CT(54419)=(S=12699871,E=12699871,C=34)
	CT(54420)=(S=12699872,E=12699872,C=60)
	CT(54421)=(S=12699873,E=12699873,C=24)
	CT(54422)=(S=12699874,E=12699875,C=58)
	CT(54423)=(S=12699876,E=12699876,C=42)
	CT(54424)=(S=12699877,E=12699877,C=31)
	CT(54425)=(S=12699878,E=12699878,C=28)
	CT(54426)=(S=12699879,E=12699879,C=12)
	CT(54427)=(S=12699880,E=12699882,C=58)
	CT(54428)=(S=12699883,E=12699883,C=27)
	CT(54429)=(S=12699884,E=12699884,C=34)
	CT(54430)=(S=12699885,E=12699885,C=51)
	CT(54431)=(S=12699887,E=12699887,C=18)
	CT(54432)=(S=12699888,E=12699889,C=28)
	CT(54433)=(S=12699890,E=12699890,C=38)
	CT(54434)=(S=12699891,E=12699891,C=25)
	CT(54435)=(S=12699893,E=12699893,C=14)
	CT(54436)=(S=12699895,E=12699895,C=23)
	CT(54437)=(S=12699896,E=12699896,C=31)
	CT(54438)=(S=12699897,E=12699897,C=28)
	CT(54439)=(S=12699898,E=12699898,C=24)
	CT(54440)=(S=12699901,E=12699901,C=28)
	CT(54441)=(S=12699902,E=12699902,C=38)
	CT(54442)=(S=12699903,E=12699903,C=31)
	CT(54443)=(S=12699904,E=12699911,C=12)
	CT(54444)=(S=12699912,E=12699919,C=28)
	CT(54445)=(S=12699920,E=12699921,C=19)
	CT(54446)=(S=12699922,E=12699923,C=58)
	CT(54447)=(S=12699924,E=12699925,C=38)
	CT(54448)=(S=12699926,E=12699927,C=22)
	CT(54449)=(S=12699928,E=12699929,C=28)
	CT(54450)=(S=12699930,E=12699931,C=12)
	CT(54451)=(S=12699932,E=12699933,C=15)
	CT(54452)=(S=12699934,E=12699935,C=18)
	CT(54453)=(S=12699936,E=12699936,C=44)
	CT(54454)=(S=12699937,E=12699937,C=34)
	CT(54455)=(S=12699938,E=12699940,C=58)
	CT(54456)=(S=12699943,E=12699943,C=24)
	CT(54457)=(S=12699944,E=12699944,C=15)
	CT(54458)=(S=12699945,E=12699945,C=42)
	CT(54459)=(S=12699946,E=12699946,C=14)
	CT(54460)=(S=12699947,E=12699947,C=19)
	CT(54461)=(S=12699948,E=12699948,C=38)
	CT(54462)=(S=12699949,E=12699949,C=52)
	CT(54463)=(S=12699951,E=12699951,C=14)
	CT(54464)=(S=12699952,E=12699955,C=36)
	CT(54465)=(S=12699956,E=12699959,C=28)
	CT(54466)=(S=12699964,E=12699967,C=31)
	CT(54467)=(S=12699968,E=12699969,C=14)
	CT(54468)=(S=12699970,E=12699971,C=60)
	CT(54469)=(S=12699972,E=12699973,C=49)
	CT(54470)=(S=12699974,E=12699975,C=14)
	CT(54471)=(S=12699978,E=12699979,C=27)
	CT(54472)=(S=12699980,E=12699981,C=12)
	CT(54473)=(S=12699982,E=12699983,C=33)
	CT(54474)=(S=12699984,E=12699987,C=31)
	CT(54475)=(S=12699988,E=12699991,C=34)
	CT(54476)=(S=12699992,E=12699995,C=18)
	CT(54477)=(S=12699996,E=12699999,C=28)
	CT(54478)=(S=12700000,E=12700001,C=19)
	CT(54479)=(S=12700002,E=12700004,C=31)
	CT(54480)=(S=12700005,E=12700005,C=52)
	CT(54481)=(S=12700006,E=12700006,C=42)
	CT(54482)=(S=12700007,E=12700007,C=12)
	CT(54483)=(S=12700008,E=12700008,C=36)
	CT(54484)=(S=12700009,E=12700009,C=31)
	CT(54485)=(S=12700010,E=12700010,C=42)
	CT(54486)=(S=12700011,E=12700012,C=25)
	CT(54487)=(S=12700013,E=12700013,C=52)
	CT(54488)=(S=12700014,E=12700014,C=33)
	CT(54489)=(S=12700015,E=12700015,C=18)
	CT(54490)=(S=12700020,E=12700021,C=31)
	CT(54491)=(S=12700022,E=12700023,C=18)
	CT(54492)=(S=12700024,E=12700025,C=14)
	CT(54493)=(S=12700026,E=12700027,C=34)
	CT(54494)=(S=12700028,E=12700029,C=14)
	CT(54495)=(S=12700032,E=12700035,C=32)
	CT(54496)=(S=12700036,E=12700039,C=14)
	CT(54497)=(S=12700040,E=12700043,C=58)
	CT(54498)=(S=12700044,E=12700047,C=31)
	CT(54499)=(S=12700048,E=12700049,C=47)
	CT(54500)=(S=12700050,E=12700050,C=38)
	CT(54501)=(S=12700051,E=12700051,C=14)
	CT(54502)=(S=12700052,E=12700052,C=36)
	CT(54503)=(S=12700053,E=12700053,C=32)
	CT(54504)=(S=12700054,E=12700054,C=32)
	CT(54505)=(S=12700055,E=12700055,C=28)
	CT(54506)=(S=12700056,E=12700056,C=31)
	CT(54507)=(S=12700057,E=12700057,C=14)
	CT(54508)=(S=12700058,E=12700058,C=38)
	CT(54509)=(S=12700059,E=12700059,C=17)
	CT(54510)=(S=12700060,E=12700060,C=19)
	CT(54511)=(S=12700061,E=12700061,C=32)
	CT(54512)=(S=12700062,E=12700062,C=38)
	CT(54513)=(S=12700063,E=12700063,C=18)
	CT(54514)=(S=12700064,E=12700064,C=14)
	CT(54515)=(S=12700065,E=12700065,C=38)
	CT(54516)=(S=12700066,E=12700066,C=49)
	CT(54517)=(S=12700067,E=12700067,C=14)
	CT(54518)=(S=12700068,E=12700068,C=58)
	CT(54519)=(S=12700069,E=12700069,C=52)
	CT(54520)=(S=12700070,E=12700070,C=22)
	CT(54521)=(S=12700071,E=12700071,C=58)
	CT(54522)=(S=12700072,E=12700074,C=28)
	CT(54523)=(S=12700075,E=12700075,C=15)
	CT(54524)=(S=12700076,E=12700076,C=51)
	CT(54525)=(S=12700077,E=12700078,C=42)
	CT(54526)=(S=12700079,E=12700079,C=31)
	CT(54527)=(S=12700080,E=12700087,C=28)
	CT(54528)=(S=12700088,E=12700101,C=14)
	CT(54529)=(S=12700102,E=12700103,C=31)
	CT(54530)=(S=12700104,E=12700109,C=14)
	CT(54531)=(S=12700110,E=12700111,C=31)
	CT(54532)=(S=12700116,E=12700119,C=52)
	CT(54533)=(S=12700120,E=12700123,C=31)
	CT(54534)=(S=12700124,E=12700127,C=27)
	CT(54535)=(S=12700128,E=12700131,C=31)
	CT(54536)=(S=12700132,E=12700135,C=18)
	CT(54537)=(S=12700136,E=12700139,C=38)
	CT(54538)=(S=12700140,E=12700143,C=28)
	CT(54539)=(S=12700144,E=12700147,C=51)
	CT(54540)=(S=12700156,E=12700159,C=13)
	CT(54541)=(S=12700160,E=12700160,C=49)
	CT(54542)=(S=12700161,E=12700167,C=14)
	CT(54543)=(S=12700176,E=12700176,C=28)
	CT(54544)=(S=12700177,E=12700177,C=49)
	CT(54545)=(S=12700178,E=12700178,C=37)
	CT(54546)=(S=12700179,E=12700179,C=24)
	CT(54547)=(S=12700180,E=12700180,C=28)
	CT(54548)=(S=12700181,E=12700181,C=31)
	CT(54549)=(S=12700182,E=12700182,C=38)
	CT(54550)=(S=12700183,E=12700183,C=36)
	CT(54551)=(S=12700184,E=12700190,C=13)
	CT(54552)=(S=12700192,E=12700199,C=36)
	CT(54553)=(S=12700206,E=12700221,C=34)
	CT(54554)=(S=12700222,E=12700223,C=14)
	CT(54555)=(S=12700224,E=12700239,C=13)
	CT(54556)=(S=12700251,E=12700255,C=12)
	CT(54557)=(S=12700256,E=12700264,C=36)
	CT(54558)=(S=12700265,E=12700266,C=14)
	CT(54559)=(S=12700267,E=12700267,C=36)
	CT(54560)=(S=12700268,E=12700268,C=51)
	CT(54561)=(S=12700269,E=12700269,C=36)
	CT(54562)=(S=12700270,E=12700270,C=24)
	CT(54563)=(S=12700271,E=12700271,C=12)
	CT(54564)=(S=12700273,E=12700273,C=14)
	CT(54565)=(S=12700274,E=12700274,C=58)
	CT(54566)=(S=12700275,E=12700275,C=27)
	CT(54567)=(S=12700276,E=12700276,C=28)
	CT(54568)=(S=12700277,E=12700277,C=58)
	CT(54569)=(S=12700278,E=12700278,C=31)
	CT(54570)=(S=12700279,E=12700279,C=28)
	CT(54571)=(S=12700280,E=12700280,C=37)
	CT(54572)=(S=12700281,E=12700281,C=12)
	CT(54573)=(S=12700282,E=12700282,C=36)
	CT(54574)=(S=12700283,E=12700283,C=42)
	CT(54575)=(S=12700284,E=12700284,C=28)
	CT(54576)=(S=12700285,E=12700286,C=12)
	CT(54577)=(S=12700287,E=12700351,C=28)
	CT(54578)=(S=12700352,E=12700399,C=14)
	CT(54579)=(S=12700400,E=12700415,C=56)
	CT(54580)=(S=12700416,E=12700417,C=42)
	CT(54581)=(S=12700421,E=12700421,C=14)
	CT(54582)=(S=12700422,E=12700435,C=46)
	CT(54583)=(S=12700436,E=12700437,C=14)
	CT(54584)=(S=12700439,E=12700443,C=46)
	CT(54585)=(S=12700448,E=12700451,C=12)
	CT(54586)=(S=12700452,E=12700459,C=18)
	CT(54587)=(S=12700460,E=12700463,C=58)
	CT(54588)=(S=12700464,E=12700467,C=31)
	CT(54589)=(S=12700472,E=12700475,C=58)
	CT(54590)=(S=12700476,E=12700479,C=18)
	CT(54591)=(S=12700480,E=12700511,C=14)
	CT(54592)=(S=12700512,E=12700513,C=12)
	CT(54593)=(S=12700514,E=12700515,C=58)
	CT(54594)=(S=12700516,E=12700519,C=18)
	CT(54595)=(S=12700520,E=12700521,C=34)
	CT(54596)=(S=12700522,E=12700523,C=21)
	CT(54597)=(S=12700524,E=12700525,C=12)
	CT(54598)=(S=12700528,E=12700529,C=18)
	CT(54599)=(S=12700530,E=12700531,C=38)
	CT(54600)=(S=12700532,E=12700533,C=14)
	CT(54601)=(S=12700534,E=12700535,C=25)
	CT(54602)=(S=12700536,E=12700537,C=34)
	CT(54603)=(S=12700538,E=12700539,C=28)
	CT(54604)=(S=12700540,E=12700543,C=18)
	CT(54605)=(S=12700544,E=12700575,C=63)
	CT(54606)=(S=12700576,E=12700607,C=28)
	CT(54607)=(S=12700608,E=12700609,C=14)
	CT(54608)=(S=12700610,E=12700611,C=42)
	CT(54609)=(S=12700612,E=12700613,C=55)
	CT(54610)=(S=12700614,E=12700615,C=51)
	CT(54611)=(S=12700616,E=12700617,C=25)
	CT(54612)=(S=12700618,E=12700619,C=18)
	CT(54613)=(S=12700620,E=12700621,C=38)
	CT(54614)=(S=12700622,E=12700625,C=58)
	CT(54615)=(S=12700626,E=12700627,C=14)
	CT(54616)=(S=12700628,E=12700629,C=12)
	CT(54617)=(S=12700632,E=12700633,C=42)
	CT(54618)=(S=12700634,E=12700635,C=31)
	CT(54619)=(S=12700636,E=12700637,C=36)
	CT(54620)=(S=12700638,E=12700639,C=58)
	CT(54621)=(S=12700641,E=12700642,C=36)
	CT(54622)=(S=12700643,E=12700643,C=38)
	CT(54623)=(S=12700644,E=12700651,C=15)
	CT(54624)=(S=12700652,E=12700653,C=31)
	CT(54625)=(S=12700654,E=12700655,C=28)
	CT(54626)=(S=12700656,E=12700671,C=14)
	CT(54627)=(S=12700672,E=12701695,C=15)
	CT(54628)=(S=12701696,E=12702719,C=47)
	CT(54629)=(S=12702720,E=12703743,C=27)
	CT(54630)=(S=12703744,E=12704255,C=19)
	CT(54631)=(S=12704256,E=12704256,C=51)
	CT(54632)=(S=12704257,E=12704262,C=22)
	CT(54633)=(S=12704264,E=12704271,C=12)
	CT(54634)=(S=12704272,E=12704287,C=28)
	CT(54635)=(S=12704292,E=12704328,C=22)
	CT(54636)=(S=12704329,E=12704329,C=28)
	CT(54637)=(S=12704330,E=12704349,C=22)
	CT(54638)=(S=12704350,E=12704350,C=52)
	CT(54639)=(S=12704351,E=12704351,C=28)
	CT(54640)=(S=12704352,E=12704353,C=22)
	CT(54641)=(S=12704354,E=12704354,C=50)
	CT(54642)=(S=12704355,E=12704355,C=14)
	CT(54643)=(S=12704356,E=12704360,C=34)
	CT(54644)=(S=12704362,E=12704368,C=22)
	CT(54645)=(S=12704369,E=12704369,C=37)
	CT(54646)=(S=12704370,E=12704370,C=198)
	CT(54647)=(S=12704371,E=12704371,C=58)
	CT(54648)=(S=12704372,E=12704373,C=42)
	CT(54649)=(S=12704374,E=12704374,C=18)
	CT(54650)=(S=12704376,E=12704376,C=33)
	CT(54651)=(S=12704378,E=12704381,C=22)
	CT(54652)=(S=12704382,E=12704383,C=58)
	CT(54653)=(S=12704384,E=12704387,C=27)
	CT(54654)=(S=12704388,E=12704391,C=58)
	CT(54655)=(S=12704392,E=12704399,C=18)
	CT(54656)=(S=12704400,E=12704403,C=31)
	CT(54657)=(S=12704404,E=12704411,C=58)
	CT(54658)=(S=12704412,E=12704415,C=31)
	CT(54659)=(S=12704416,E=12704431,C=14)
	CT(54660)=(S=12704432,E=12704435,C=58)
	CT(54661)=(S=12704436,E=12704439,C=18)
	CT(54662)=(S=12704440,E=12704443,C=31)
	CT(54663)=(S=12704444,E=12704447,C=18)
	CT(54664)=(S=12704448,E=12704455,C=49)
	CT(54665)=(S=12704458,E=12704458,C=28)
	CT(54666)=(S=12704461,E=12704461,C=34)
	CT(54667)=(S=12704462,E=12704462,C=19)
	CT(54668)=(S=12704463,E=12704463,C=17)
	CT(54669)=(S=12704464,E=12704477,C=28)
	CT(54670)=(S=12704479,E=12704511,C=22)
	CT(54671)=(S=12704513,E=12704516,C=44)
	CT(54672)=(S=12704517,E=12704518,C=13)
	CT(54673)=(S=12704520,E=12704526,C=44)
	CT(54674)=(S=12704527,E=12704527,C=28)
	CT(54675)=(S=12704528,E=12704539,C=24)
	CT(54676)=(S=12704540,E=12704540,C=58)
	CT(54677)=(S=12704541,E=12704541,C=15)
	CT(54678)=(S=12704542,E=12704543,C=24)
	CT(54679)=(S=12704544,E=12704607,C=44)
	CT(54680)=(S=12704608,E=12704608,C=25)
	CT(54681)=(S=12704609,E=12704609,C=14)
	CT(54682)=(S=12704610,E=12704610,C=49)
	CT(54683)=(S=12704611,E=12704611,C=31)
	CT(54684)=(S=12704612,E=12704612,C=52)
	CT(54685)=(S=12704614,E=12704614,C=15)
	CT(54686)=(S=12704615,E=12704615,C=25)
	CT(54687)=(S=12704616,E=12704616,C=18)
	CT(54688)=(S=12704617,E=12704617,C=28)
	CT(54689)=(S=12704618,E=12704618,C=52)
	CT(54690)=(S=12704619,E=12704619,C=58)
	CT(54691)=(S=12704620,E=12704620,C=14)
	CT(54692)=(S=12704621,E=12704621,C=15)
	CT(54693)=(S=12704622,E=12704623,C=14)
	CT(54694)=(S=12704624,E=12704624,C=19)
	CT(54695)=(S=12704625,E=12704626,C=58)
	CT(54696)=(S=12704627,E=12704627,C=14)
	CT(54697)=(S=12704628,E=12704628,C=36)
	CT(54698)=(S=12704629,E=12704629,C=38)
	CT(54699)=(S=12704630,E=12704630,C=14)
	CT(54700)=(S=12704631,E=12704631,C=38)
	CT(54701)=(S=12704632,E=12704634,C=14)
	CT(54702)=(S=12704635,E=12704635,C=49)
	CT(54703)=(S=12704636,E=12704636,C=31)
	CT(54704)=(S=12704637,E=12704637,C=18)
	CT(54705)=(S=12704638,E=12704638,C=58)
	CT(54706)=(S=12704639,E=12704639,C=18)
	CT(54707)=(S=12704640,E=12705023,C=44)
	CT(54708)=(S=12705024,E=12705039,C=49)
	CT(54709)=(S=12705040,E=12705105,C=19)
	CT(54710)=(S=12705106,E=12705107,C=38)
	CT(54711)=(S=12705108,E=12705116,C=14)
	CT(54712)=(S=12705120,E=12705135,C=12)
	CT(54713)=(S=12705136,E=12705136,C=52)
	CT(54714)=(S=12705137,E=12705137,C=14)
	CT(54715)=(S=12705138,E=12705138,C=36)
	CT(54716)=(S=12705139,E=12705139,C=52)
	CT(54717)=(S=12705140,E=12705140,C=34)
	CT(54718)=(S=12705141,E=12705141,C=12)
	CT(54719)=(S=12705142,E=12705142,C=14)
	CT(54720)=(S=12705143,E=12705143,C=19)
	CT(54721)=(S=12705145,E=12705145,C=13)
	CT(54722)=(S=12705146,E=12705146,C=58)
	CT(54723)=(S=12705148,E=12705148,C=24)
	CT(54724)=(S=12705149,E=12705149,C=36)
	CT(54725)=(S=12705150,E=12705150,C=58)
	CT(54726)=(S=12705151,E=12705151,C=28)
	CT(54727)=(S=12705152,E=12705183,C=36)
	CT(54728)=(S=12705184,E=12705215,C=49)
	CT(54729)=(S=12705242,E=12705242,C=118)
	CT(54730)=(S=12705248,E=12705279,C=49)
	CT(54731)=(S=12705280,E=12705327,C=34)
	CT(54732)=(S=12705328,E=12705329,C=18)
	CT(54733)=(S=12705330,E=12705330,C=14)
	CT(54734)=(S=12705331,E=12705331,C=26)
	CT(54735)=(S=12705332,E=12705335,C=31)
	CT(54736)=(S=12705336,E=12705336,C=28)
	CT(54737)=(S=12705337,E=12705337,C=38)
	CT(54738)=(S=12705338,E=12705338,C=18)
	CT(54739)=(S=12705339,E=12705339,C=34)
	CT(54740)=(S=12705341,E=12705341,C=14)
	CT(54741)=(S=12705342,E=12705342,C=18)
	CT(54742)=(S=12705343,E=12705343,C=38)
	CT(54743)=(S=12705344,E=12705387,C=34)
	CT(54744)=(S=12705388,E=12705388,C=12)
	CT(54745)=(S=12705389,E=12705389,C=18)
	CT(54746)=(S=12705390,E=12705390,C=19)
	CT(54747)=(S=12705392,E=12705407,C=56)
	CT(54748)=(S=12705408,E=12705408,C=12)
	CT(54749)=(S=12705409,E=12705409,C=17)
	CT(54750)=(S=12705410,E=12705410,C=33)
	CT(54751)=(S=12705411,E=12705411,C=58)
	CT(54752)=(S=12705412,E=12705412,C=36)
	CT(54753)=(S=12705413,E=12705413,C=47)
	CT(54754)=(S=12705414,E=12705414,C=12)
	CT(54755)=(S=12705415,E=12705415,C=58)
	CT(54756)=(S=12705416,E=12705416,C=15)
	CT(54757)=(S=12705417,E=12705417,C=36)
	CT(54758)=(S=12705418,E=12705418,C=34)
	CT(54759)=(S=12705419,E=12705419,C=42)
	CT(54760)=(S=12705420,E=12705420,C=31)
	CT(54761)=(S=12705421,E=12705421,C=19)
	CT(54762)=(S=12705428,E=12705467,C=34)
	CT(54763)=(S=12705470,E=12705470,C=36)
	CT(54764)=(S=12705471,E=12705471,C=18)
	CT(54765)=(S=12705472,E=12705530,C=34)
	CT(54766)=(S=12705534,E=12705534,C=25)
	CT(54767)=(S=12705535,E=12705535,C=34)
	CT(54768)=(S=12705536,E=12705536,C=58)
	CT(54769)=(S=12705537,E=12705599,C=34)
	CT(54770)=(S=12705604,E=12705604,C=18)
	CT(54771)=(S=12705612,E=12705612,C=37)
	CT(54772)=(S=12705613,E=12705613,C=28)
	CT(54773)=(S=12705614,E=12705615,C=14)
	CT(54774)=(S=12705616,E=12705631,C=34)
	CT(54775)=(S=12705634,E=12705634,C=31)
	CT(54776)=(S=12705635,E=12705635,C=24)
	CT(54777)=(S=12705636,E=12705636,C=36)
	CT(54778)=(S=12705637,E=12705637,C=38)
	CT(54779)=(S=12705638,E=12705791,C=34)
	CT(54780)=(S=12705792,E=12706303,C=50)
	CT(54781)=(S=12706304,E=12706495,C=38)
	CT(54782)=(S=12706496,E=12706559,C=50)
	CT(54783)=(S=12706560,E=12706623,C=106)
	CT(54784)=(S=12706624,E=12706655,C=15)
	CT(54785)=(S=12706656,E=12706657,C=31)
	CT(54786)=(S=12706658,E=12706658,C=18)
	CT(54787)=(S=12706659,E=12706659,C=28)
	CT(54788)=(S=12706660,E=12706660,C=58)
	CT(54789)=(S=12706661,E=12706661,C=34)
	CT(54790)=(S=12706662,E=12706662,C=44)
	CT(54791)=(S=12706664,E=12706664,C=15)
	CT(54792)=(S=12706665,E=12706665,C=58)
	CT(54793)=(S=12706666,E=12706666,C=14)
	CT(54794)=(S=12706667,E=12706667,C=12)
	CT(54795)=(S=12706668,E=12706668,C=42)
	CT(54796)=(S=12706670,E=12706670,C=38)
	CT(54797)=(S=12706671,E=12706671,C=14)
	CT(54798)=(S=12706672,E=12706672,C=58)
	CT(54799)=(S=12706673,E=12706673,C=42)
	CT(54800)=(S=12706675,E=12706675,C=31)
	CT(54801)=(S=12706676,E=12706676,C=58)
	CT(54802)=(S=12706677,E=12706677,C=28)
	CT(54803)=(S=12706678,E=12706678,C=14)
	CT(54804)=(S=12706679,E=12706680,C=31)
	CT(54805)=(S=12706683,E=12706683,C=58)
	CT(54806)=(S=12706684,E=12706684,C=28)
	CT(54807)=(S=12706686,E=12706686,C=19)
	CT(54808)=(S=12706687,E=12706687,C=23)
	CT(54809)=(S=12706688,E=12706688,C=106)
	CT(54810)=(S=12706690,E=12706690,C=12)
	CT(54811)=(S=12706691,E=12706691,C=58)
	CT(54812)=(S=12706692,E=12706692,C=25)
	CT(54813)=(S=12706693,E=12706693,C=12)
	CT(54814)=(S=12706694,E=12706694,C=18)
	CT(54815)=(S=12706696,E=12706703,C=12)
	CT(54816)=(S=12706704,E=12706719,C=28)
	CT(54817)=(S=12706720,E=12706751,C=39)
	CT(54818)=(S=12706752,E=12706755,C=28)
	CT(54819)=(S=12706756,E=12706759,C=50)
	CT(54820)=(S=12706760,E=12706761,C=14)
	CT(54821)=(S=12706762,E=12706763,C=28)
	CT(54822)=(S=12706764,E=12706765,C=27)
	CT(54823)=(S=12706766,E=12706771,C=31)
	CT(54824)=(S=12706772,E=12706775,C=15)
	CT(54825)=(S=12706776,E=12706783,C=58)
	CT(54826)=(S=12706784,E=12706785,C=34)
	CT(54827)=(S=12706786,E=12706787,C=38)
	CT(54828)=(S=12706788,E=12706789,C=12)
	CT(54829)=(S=12706790,E=12706791,C=31)
	CT(54830)=(S=12706792,E=12706793,C=18)
	CT(54831)=(S=12706794,E=12706795,C=28)
	CT(54832)=(S=12706798,E=12706799,C=21)
	CT(54833)=(S=12706800,E=12706801,C=18)
	CT(54834)=(S=12706804,E=12706807,C=14)
	CT(54835)=(S=12706808,E=12706809,C=12)
	CT(54836)=(S=12706810,E=12706811,C=31)
	CT(54837)=(S=12706812,E=12706813,C=18)
	CT(54838)=(S=12706814,E=12706815,C=28)
	CT(54839)=(S=12706816,E=12706816,C=42)
	CT(54840)=(S=12706817,E=12706817,C=14)
	CT(54841)=(S=12706818,E=12706818,C=31)
	CT(54842)=(S=12706820,E=12706938,C=42)
	CT(54843)=(S=12706957,E=12706957,C=14)
	CT(54844)=(S=12706958,E=12706958,C=18)
	CT(54845)=(S=12706959,E=12706959,C=19)
	CT(54846)=(S=12706960,E=12706962,C=28)
	CT(54847)=(S=12706963,E=12706963,C=12)
	CT(54848)=(S=12706965,E=12706966,C=28)
	CT(54849)=(S=12706967,E=12706967,C=52)
	CT(54850)=(S=12706968,E=12706968,C=51)
	CT(54851)=(S=12706969,E=12706969,C=38)
	CT(54852)=(S=12706970,E=12706970,C=42)
	CT(54853)=(S=12706971,E=12706971,C=14)
	CT(54854)=(S=12706972,E=12706972,C=12)
	CT(54855)=(S=12706973,E=12706973,C=19)
	CT(54856)=(S=12706974,E=12706975,C=18)
	CT(54857)=(S=12706986,E=12706987,C=42)
	CT(54858)=(S=12706990,E=12706990,C=58)
	CT(54859)=(S=12706992,E=12707008,C=42)
	CT(54860)=(S=12707013,E=12707014,C=34)
	CT(54861)=(S=12707016,E=12707049,C=42)
	CT(54862)=(S=12707050,E=12707050,C=33)
	CT(54863)=(S=12707056,E=12707071,C=42)
	CT(54864)=(S=12707072,E=12707327,C=47)
	CT(54865)=(S=12707328,E=12707839,C=38)
	CT(54866)=(S=12707840,E=12708351,C=18)
	CT(54867)=(S=12708352,E=12708863,C=19)
	CT(54868)=(S=12708864,E=12709119,C=21)
	CT(54869)=(S=12709120,E=12709375,C=14)
	CT(54870)=(S=12709380,E=12709383,C=47)
	CT(54871)=(S=12709384,E=12709387,C=28)
	CT(54872)=(S=12709388,E=12709395,C=58)
	CT(54873)=(S=12709396,E=12709399,C=31)
	CT(54874)=(S=12709400,E=12709400,C=58)
	CT(54875)=(S=12709401,E=12709401,C=37)
	CT(54876)=(S=12709402,E=12709403,C=28)
	CT(54877)=(S=12709404,E=12709407,C=111)
	CT(54878)=(S=12709408,E=12709415,C=31)
	CT(54879)=(S=12709416,E=12709419,C=58)
	CT(54880)=(S=12709424,E=12709427,C=18)
	CT(54881)=(S=12709428,E=12709431,C=25)
	CT(54882)=(S=12709432,E=12709435,C=38)
	CT(54883)=(S=12709436,E=12709439,C=28)
	CT(54884)=(S=12709440,E=12709443,C=58)
	CT(54885)=(S=12709444,E=12709447,C=14)
	CT(54886)=(S=12709448,E=12709451,C=31)
	CT(54887)=(S=12709452,E=12709455,C=15)
	CT(54888)=(S=12709456,E=12709463,C=14)
	CT(54889)=(S=12709464,E=12709467,C=28)
	CT(54890)=(S=12709468,E=12709471,C=58)
	CT(54891)=(S=12709472,E=12709475,C=31)
	CT(54892)=(S=12709476,E=12709479,C=15)
	CT(54893)=(S=12709480,E=12709483,C=28)
	CT(54894)=(S=12709484,E=12709487,C=31)
	CT(54895)=(S=12709488,E=12709491,C=22)
	CT(54896)=(S=12709492,E=12709495,C=31)
	CT(54897)=(S=12709500,E=12709503,C=28)
	CT(54898)=(S=12709504,E=12709511,C=18)
	CT(54899)=(S=12709512,E=12709515,C=15)
	CT(54900)=(S=12709516,E=12709519,C=34)
	CT(54901)=(S=12709520,E=12709523,C=25)
	CT(54902)=(S=12709524,E=12709527,C=12)
	CT(54903)=(S=12709528,E=12709531,C=31)
	CT(54904)=(S=12709532,E=12709535,C=42)
	CT(54905)=(S=12709536,E=12709539,C=36)
	CT(54906)=(S=12709540,E=12709543,C=14)
	CT(54907)=(S=12709544,E=12709551,C=58)
	CT(54908)=(S=12709552,E=12709555,C=18)
	CT(54909)=(S=12709556,E=12709559,C=58)
	CT(54910)=(S=12709560,E=12709563,C=24)
	CT(54911)=(S=12709564,E=12709567,C=17)
	CT(54912)=(S=12709572,E=12709575,C=28)
	CT(54913)=(S=12709576,E=12709579,C=58)
	CT(54914)=(S=12709580,E=12709583,C=56)
	CT(54915)=(S=12709584,E=12709587,C=17)
	CT(54916)=(S=12709588,E=12709599,C=60)
	CT(54917)=(S=12709600,E=12709603,C=31)
	CT(54918)=(S=12709604,E=12709607,C=12)
	CT(54919)=(S=12709608,E=12709611,C=14)
	CT(54920)=(S=12709616,E=12709627,C=36)
	CT(54921)=(S=12709628,E=12709631,C=28)
	CT(54922)=(S=12709632,E=12709635,C=33)
	CT(54923)=(S=12709636,E=12709639,C=13)
	CT(54924)=(S=12709640,E=12709643,C=14)
	CT(54925)=(S=12709644,E=12709647,C=25)
	CT(54926)=(S=12709648,E=12709651,C=28)
	CT(54927)=(S=12709652,E=12709655,C=34)
	CT(54928)=(S=12709656,E=12709659,C=31)
	CT(54929)=(S=12709660,E=12709663,C=28)
	CT(54930)=(S=12709668,E=12709671,C=58)
	CT(54931)=(S=12709672,E=12709675,C=42)
	CT(54932)=(S=12709676,E=12709679,C=58)
	CT(54933)=(S=12709680,E=12709683,C=24)
	CT(54934)=(S=12709684,E=12709687,C=15)
	CT(54935)=(S=12709688,E=12709695,C=58)
	CT(54936)=(S=12709696,E=12709699,C=38)
	CT(54937)=(S=12709700,E=12709707,C=31)
	CT(54938)=(S=12709708,E=12709711,C=12)
	CT(54939)=(S=12709712,E=12709715,C=58)
	CT(54940)=(S=12709720,E=12709727,C=36)
	CT(54941)=(S=12709728,E=12709731,C=24)
	CT(54942)=(S=12709732,E=12709735,C=58)
	CT(54943)=(S=12709736,E=12709739,C=28)
	CT(54944)=(S=12709740,E=12709743,C=17)
	CT(54945)=(S=12709744,E=12709747,C=14)
	CT(54946)=(S=12709748,E=12709751,C=36)
	CT(54947)=(S=12709752,E=12709755,C=12)
	CT(54948)=(S=12709756,E=12709759,C=58)
	CT(54949)=(S=12709760,E=12709761,C=31)
	CT(54950)=(S=12709762,E=12709763,C=38)
	CT(54951)=(S=12709764,E=12709764,C=36)
	CT(54952)=(S=12709765,E=12709765,C=18)
	CT(54953)=(S=12709766,E=12709767,C=38)
	CT(54954)=(S=12709768,E=12709769,C=58)
	CT(54955)=(S=12709770,E=12709773,C=38)
	CT(54956)=(S=12709774,E=12709775,C=31)
	CT(54957)=(S=12709776,E=12709777,C=58)
	CT(54958)=(S=12709780,E=12709781,C=50)
	CT(54959)=(S=12709784,E=12709785,C=31)
	CT(54960)=(S=12709788,E=12709789,C=28)
	CT(54961)=(S=12709790,E=12709791,C=38)
	CT(54962)=(S=12709794,E=12709797,C=28)
	CT(54963)=(S=12709800,E=12709801,C=24)
	CT(54964)=(S=12709802,E=12709803,C=31)
	CT(54965)=(S=12709804,E=12709805,C=38)
	CT(54966)=(S=12709806,E=12709807,C=14)
	CT(54967)=(S=12709808,E=12709809,C=24)
	CT(54968)=(S=12709812,E=12709813,C=31)
	CT(54969)=(S=12709814,E=12709815,C=53)
	CT(54970)=(S=12709816,E=12709816,C=58)
	CT(54971)=(S=12709817,E=12709817,C=28)
	CT(54972)=(S=12709818,E=12709819,C=36)
	CT(54973)=(S=12709820,E=12709821,C=42)
	CT(54974)=(S=12709824,E=12709825,C=12)
	CT(54975)=(S=12709826,E=12709827,C=38)
	CT(54976)=(S=12709830,E=12709831,C=12)
	CT(54977)=(S=12709832,E=12709833,C=34)
	CT(54978)=(S=12709834,E=12709835,C=36)
	CT(54979)=(S=12709836,E=12709841,C=58)
	CT(54980)=(S=12709842,E=12709843,C=49)
	CT(54981)=(S=12709844,E=12709845,C=38)
	CT(54982)=(S=12709846,E=12709847,C=22)
	CT(54983)=(S=12709848,E=12709849,C=31)
	CT(54984)=(S=12709850,E=12709851,C=38)
	CT(54985)=(S=12709852,E=12709853,C=34)
	CT(54986)=(S=12709854,E=12709855,C=36)
	CT(54987)=(S=12709856,E=12709857,C=58)
	CT(54988)=(S=12709858,E=12709859,C=42)
	CT(54989)=(S=12709860,E=12709861,C=31)
	CT(54990)=(S=12709862,E=12709863,C=38)
	CT(54991)=(S=12709864,E=12709864,C=19)
	CT(54992)=(S=12709865,E=12709865,C=18)
	CT(54993)=(S=12709866,E=12709871,C=31)
	CT(54994)=(S=12709872,E=12709873,C=12)
	CT(54995)=(S=12709874,E=12709875,C=18)
	CT(54996)=(S=12709878,E=12709879,C=38)
	CT(54997)=(S=12709880,E=12709881,C=42)
	CT(54998)=(S=12709882,E=12709883,C=31)
	CT(54999)=(S=12709884,E=12709885,C=38)
	CT(55000)=(S=12709886,E=12709887,C=31)
	CT(55001)=(S=12709888,E=12710143,C=14)
	CT(55002)=(S=12710144,E=12710399,C=19)
	CT(55003)=(S=12710400,E=12710400,C=14)
	CT(55004)=(S=12710401,E=12710479,C=12)
	CT(55005)=(S=12710480,E=12710495,C=36)
	CT(55006)=(S=12710496,E=12710496,C=14)
	CT(55007)=(S=12710497,E=12710497,C=36)
	CT(55008)=(S=12710498,E=12710498,C=25)
	CT(55009)=(S=12710500,E=12710500,C=58)
	CT(55010)=(S=12710501,E=12710501,C=49)
	CT(55011)=(S=12710502,E=12710502,C=14)
	CT(55012)=(S=12710503,E=12710503,C=19)
	CT(55013)=(S=12710504,E=12710504,C=38)
	CT(55014)=(S=12710505,E=12710505,C=34)
	CT(55015)=(S=12710507,E=12710507,C=24)
	CT(55016)=(S=12710508,E=12710508,C=36)
	CT(55017)=(S=12710510,E=12710510,C=14)
	CT(55018)=(S=12710511,E=12710511,C=63)
	CT(55019)=(S=12710512,E=12710512,C=28)
	CT(55020)=(S=12710513,E=12710513,C=14)
	CT(55021)=(S=12710514,E=12710514,C=31)
	CT(55022)=(S=12710515,E=12710516,C=14)
	CT(55023)=(S=12710517,E=12710517,C=36)
	CT(55024)=(S=12710518,E=12710518,C=42)
	CT(55025)=(S=12710519,E=12710519,C=36)
	CT(55026)=(S=12710520,E=12710520,C=38)
	CT(55027)=(S=12710521,E=12710521,C=28)
	CT(55028)=(S=12710522,E=12710522,C=19)
	CT(55029)=(S=12710523,E=12710523,C=18)
	CT(55030)=(S=12710524,E=12710524,C=21)
	CT(55031)=(S=12710525,E=12710525,C=36)
	CT(55032)=(S=12710526,E=12710526,C=47)
	CT(55033)=(S=12710527,E=12710533,C=18)
	CT(55034)=(S=12710534,E=12710534,C=42)
	CT(55035)=(S=12710535,E=12710535,C=18)
	CT(55036)=(S=12710536,E=12710539,C=58)
	CT(55037)=(S=12710540,E=12710540,C=14)
	CT(55038)=(S=12710541,E=12710541,C=57)
	CT(55039)=(S=12710542,E=12710543,C=58)
	CT(55040)=(S=12710544,E=12710544,C=53)
	CT(55041)=(S=12710545,E=12710545,C=18)
	CT(55042)=(S=12710546,E=12710547,C=58)
	CT(55043)=(S=12710548,E=12710551,C=18)
	CT(55044)=(S=12710552,E=12710552,C=58)
	CT(55045)=(S=12710553,E=12710553,C=18)
	CT(55046)=(S=12710554,E=12710554,C=52)
	CT(55047)=(S=12710555,E=12710555,C=24)
	CT(55048)=(S=12710556,E=12710563,C=31)
	CT(55049)=(S=12710564,E=12710565,C=25)
	CT(55050)=(S=12710566,E=12710566,C=31)
	CT(55051)=(S=12710567,E=12710567,C=18)
	CT(55052)=(S=12710568,E=12710569,C=58)
	CT(55053)=(S=12710570,E=12710571,C=18)
	CT(55054)=(S=12710572,E=12710572,C=58)
	CT(55055)=(S=12710573,E=12710573,C=18)
	CT(55056)=(S=12710574,E=12710575,C=12)
	CT(55057)=(S=12710576,E=12710579,C=18)
	CT(55058)=(S=12710580,E=12710581,C=33)
	CT(55059)=(S=12710582,E=12710582,C=19)
	CT(55060)=(S=12710583,E=12710583,C=36)
	CT(55061)=(S=12710584,E=12710587,C=19)
	CT(55062)=(S=12710588,E=12710589,C=25)
	CT(55063)=(S=12710590,E=12710591,C=18)
	CT(55064)=(S=12710592,E=12710592,C=12)
	CT(55065)=(S=12710593,E=12710593,C=18)
	CT(55066)=(S=12710594,E=12710595,C=26)
	CT(55067)=(S=12710596,E=12710599,C=38)
	CT(55068)=(S=12710600,E=12710607,C=18)
	CT(55069)=(S=12710608,E=12710609,C=26)
	CT(55070)=(S=12710610,E=12710611,C=28)
	CT(55071)=(S=12710612,E=12710613,C=58)
	CT(55072)=(S=12710614,E=12710614,C=32)
	CT(55073)=(S=12710615,E=12710615,C=58)
	CT(55074)=(S=12710616,E=12710617,C=18)
	CT(55075)=(S=12710618,E=12710619,C=34)
	CT(55076)=(S=12710620,E=12710623,C=17)
	CT(55077)=(S=12710624,E=12710639,C=14)
	CT(55078)=(S=12710640,E=12710655,C=22)
	CT(55079)=(S=12710656,E=12710787,C=14)
	CT(55080)=(S=12710788,E=12710789,C=53)
	CT(55081)=(S=12710792,E=12710793,C=49)
	CT(55082)=(S=12710794,E=12710795,C=18)
	CT(55083)=(S=12710796,E=12710797,C=52)
	CT(55084)=(S=12710798,E=12710799,C=58)
	CT(55085)=(S=12710800,E=12710801,C=19)
	CT(55086)=(S=12710802,E=12710805,C=58)
	CT(55087)=(S=12710808,E=12710809,C=31)
	CT(55088)=(S=12710810,E=12710811,C=58)
	CT(55089)=(S=12710812,E=12710815,C=31)
	CT(55090)=(S=12710816,E=12710816,C=28)
	CT(55091)=(S=12710818,E=12710819,C=42)
	CT(55092)=(S=12710823,E=12710823,C=31)
	CT(55093)=(S=12710824,E=12710827,C=46)
	CT(55094)=(S=12710828,E=12710831,C=18)
	CT(55095)=(S=12710832,E=12710837,C=14)
	CT(55096)=(S=12710839,E=12710839,C=19)
	CT(55097)=(S=12710840,E=12710847,C=28)
	CT(55098)=(S=12710848,E=12710879,C=37)
	CT(55099)=(S=12710880,E=12710911,C=14)
	CT(55100)=(S=12710912,E=12711423,C=49)
	CT(55101)=(S=12711424,E=12711935,C=34)
	CT(55102)=(S=12711936,E=12713471,C=12)
	CT(55103)=(S=12713473,E=12713474,C=96)
	CT(55104)=(S=12713476,E=12713479,C=19)
	CT(55105)=(S=12713490,E=12713494,C=28)
	CT(55106)=(S=12713495,E=12713495,C=24)
	CT(55107)=(S=12713496,E=12713498,C=15)
	CT(55108)=(S=12713501,E=12713501,C=51)
	CT(55109)=(S=12713502,E=12713502,C=28)
	CT(55110)=(S=12713503,E=12713503,C=14)
	CT(55111)=(S=12713504,E=12713519,C=25)
	CT(55112)=(S=12713520,E=12713535,C=38)
	CT(55113)=(S=12713536,E=12713599,C=13)
	CT(55114)=(S=12713600,E=12713663,C=28)
	CT(55115)=(S=12713664,E=12713665,C=19)
	CT(55116)=(S=12713666,E=12713667,C=14)
	CT(55117)=(S=12713668,E=12713669,C=31)
	CT(55118)=(S=12713670,E=12713671,C=13)
	CT(55119)=(S=12713672,E=12713673,C=22)
	CT(55120)=(S=12713674,E=12713675,C=28)
	CT(55121)=(S=12713678,E=12713679,C=17)
	CT(55122)=(S=12713680,E=12713681,C=18)
	CT(55123)=(S=12713682,E=12713683,C=14)
	CT(55124)=(S=12713684,E=12713685,C=28)
	CT(55125)=(S=12713686,E=12713687,C=36)
	CT(55126)=(S=12713688,E=12713693,C=31)
	CT(55127)=(S=12713694,E=12713695,C=22)
	CT(55128)=(S=12713696,E=12713699,C=18)
	CT(55129)=(S=12713700,E=12713701,C=37)
	CT(55130)=(S=12713702,E=12713703,C=38)
	CT(55131)=(S=12713704,E=12713705,C=31)
	CT(55132)=(S=12713708,E=12713709,C=28)
	CT(55133)=(S=12713710,E=12713711,C=58)
	CT(55134)=(S=12713712,E=12713713,C=15)
	CT(55135)=(S=12713714,E=12713715,C=38)
	CT(55136)=(S=12713718,E=12713719,C=58)
	CT(55137)=(S=12713720,E=12713721,C=18)
	CT(55138)=(S=12713722,E=12713723,C=20)
	CT(55139)=(S=12713724,E=12713725,C=37)
	CT(55140)=(S=12713726,E=12713727,C=36)
	CT(55141)=(S=12713728,E=12713983,C=37)
	CT(55142)=(S=12713984,E=12713984,C=28)
	CT(55143)=(S=12713985,E=12713986,C=14)
	CT(55144)=(S=12713987,E=12713987,C=44)
	CT(55145)=(S=12713988,E=12713988,C=22)
	CT(55146)=(S=12713989,E=12713989,C=36)
	CT(55147)=(S=12713990,E=12713991,C=49)
	CT(55148)=(S=12713992,E=12713992,C=60)
	CT(55149)=(S=12713993,E=12713993,C=12)
	CT(55150)=(S=12713994,E=12713994,C=42)
	CT(55151)=(S=12713995,E=12713995,C=28)
	CT(55152)=(S=12713996,E=12713998,C=33)
	CT(55153)=(S=12713999,E=12713999,C=23)
	CT(55154)=(S=12714000,E=12714000,C=15)
	CT(55155)=(S=12714001,E=12714001,C=34)
	CT(55156)=(S=12714002,E=12714004,C=44)
	CT(55157)=(S=12714005,E=12714005,C=20)
	CT(55158)=(S=12714006,E=12714006,C=24)
	CT(55159)=(S=12714007,E=12714007,C=70)
	CT(55160)=(S=12714008,E=12714010,C=42)
	CT(55161)=(S=12714011,E=12714011,C=100)
	CT(55162)=(S=12714012,E=12714015,C=36)
	CT(55163)=(S=12714016,E=12714016,C=51)
	CT(55164)=(S=12714017,E=12714019,C=25)
	CT(55165)=(S=12714020,E=12714020,C=12)
	CT(55166)=(S=12714021,E=12714021,C=49)
	CT(55167)=(S=12714022,E=12714025,C=13)
	CT(55168)=(S=12714026,E=12714026,C=70)
	CT(55169)=(S=12714027,E=12714028,C=49)
	CT(55170)=(S=12714029,E=12714029,C=71)
	CT(55171)=(S=12714030,E=12714031,C=24)
	CT(55172)=(S=12714032,E=12714034,C=60)
	CT(55173)=(S=12714035,E=12714035,C=55)
	CT(55174)=(S=12714036,E=12714041,C=31)
	CT(55175)=(S=12714042,E=12714042,C=14)
	CT(55176)=(S=12714043,E=12714043,C=19)
	CT(55177)=(S=12714044,E=12714044,C=20)
	CT(55178)=(S=12714045,E=12714045,C=19)
	CT(55179)=(S=12714046,E=12714046,C=58)
	CT(55180)=(S=12714047,E=12714047,C=42)
	CT(55181)=(S=12714048,E=12714051,C=19)
	CT(55182)=(S=12714052,E=12714055,C=18)
	CT(55183)=(S=12714056,E=12714059,C=42)
	CT(55184)=(S=12714060,E=12714063,C=63)
	CT(55185)=(S=12714064,E=12714067,C=47)
	CT(55186)=(S=12714068,E=12714071,C=57)
	CT(55187)=(S=12714072,E=12714075,C=31)
	CT(55188)=(S=12714076,E=12714079,C=28)
	CT(55189)=(S=12714080,E=12714087,C=58)
	CT(55190)=(S=12714092,E=12714095,C=20)
	CT(55191)=(S=12714096,E=12714107,C=18)
	CT(55192)=(S=12714108,E=12714111,C=38)
	CT(55193)=(S=12714112,E=12714112,C=58)
	CT(55194)=(S=12714113,E=12714113,C=38)
	CT(55195)=(S=12714114,E=12714114,C=37)
	CT(55196)=(S=12714115,E=12714115,C=31)
	CT(55197)=(S=12714116,E=12714116,C=24)
	CT(55198)=(S=12714118,E=12714118,C=19)
	CT(55199)=(S=12714119,E=12714119,C=28)
	CT(55200)=(S=12714120,E=12714120,C=18)
	CT(55201)=(S=12714121,E=12714121,C=34)
	CT(55202)=(S=12714122,E=12714122,C=31)
	CT(55203)=(S=12714124,E=12714124,C=15)
	CT(55204)=(S=12714126,E=12714126,C=37)
	CT(55205)=(S=12714127,E=12714127,C=28)
	CT(55206)=(S=12714128,E=12714129,C=18)
	CT(55207)=(S=12714130,E=12714130,C=32)
	CT(55208)=(S=12714131,E=12714131,C=14)
	CT(55209)=(S=12714132,E=12714132,C=31)
	CT(55210)=(S=12714133,E=12714133,C=28)
	CT(55211)=(S=12714134,E=12714134,C=31)
	CT(55212)=(S=12714135,E=12714135,C=28)
	CT(55213)=(S=12714136,E=12714137,C=12)
	CT(55214)=(S=12714138,E=12714138,C=58)
	CT(55215)=(S=12714139,E=12714139,C=24)
	CT(55216)=(S=12714140,E=12714140,C=14)
	CT(55217)=(S=12714141,E=12714141,C=31)
	CT(55218)=(S=12714142,E=12714142,C=14)
	CT(55219)=(S=12714144,E=12714144,C=44)
	CT(55220)=(S=12714145,E=12714145,C=24)
	CT(55221)=(S=12714146,E=12714146,C=18)
	CT(55222)=(S=12714147,E=12714147,C=28)
	CT(55223)=(S=12714148,E=12714148,C=50)
	CT(55224)=(S=12714149,E=12714149,C=28)
	CT(55225)=(S=12714150,E=12714150,C=12)
	CT(55226)=(S=12714151,E=12714151,C=58)
	CT(55227)=(S=12714153,E=12714153,C=27)
	CT(55228)=(S=12714154,E=12714154,C=36)
	CT(55229)=(S=12714155,E=12714155,C=58)
	CT(55230)=(S=12714156,E=12714156,C=34)
	CT(55231)=(S=12714157,E=12714157,C=38)
	CT(55232)=(S=12714158,E=12714159,C=52)
	CT(55233)=(S=12714160,E=12714160,C=12)
	CT(55234)=(S=12714161,E=12714161,C=38)
	CT(55235)=(S=12714162,E=12714162,C=37)
	CT(55236)=(S=12714163,E=12714163,C=12)
	CT(55237)=(S=12714165,E=12714165,C=58)
	CT(55238)=(S=12714166,E=12714168,C=28)
	CT(55239)=(S=12714169,E=12714169,C=22)
	CT(55240)=(S=12714171,E=12714171,C=31)
	CT(55241)=(S=12714174,E=12714174,C=38)
	CT(55242)=(S=12714175,E=12714175,C=52)
	CT(55243)=(S=12714176,E=12714176,C=27)
	CT(55244)=(S=12714178,E=12714178,C=36)
	CT(55245)=(S=12714179,E=12714179,C=12)
	CT(55246)=(S=12714180,E=12714180,C=63)
	CT(55247)=(S=12714181,E=12714181,C=36)
	CT(55248)=(S=12714182,E=12714182,C=34)
	CT(55249)=(S=12714183,E=12714183,C=58)
	CT(55250)=(S=12714184,E=12714184,C=31)
	CT(55251)=(S=12714185,E=12714185,C=28)
	CT(55252)=(S=12714186,E=12714186,C=37)
	CT(55253)=(S=12714187,E=12714187,C=44)
	CT(55254)=(S=12714188,E=12714188,C=38)
	CT(55255)=(S=12714189,E=12714189,C=49)
	CT(55256)=(S=12714190,E=12714190,C=31)
	CT(55257)=(S=12714192,E=12714192,C=28)
	CT(55258)=(S=12714193,E=12714193,C=18)
	CT(55259)=(S=12714194,E=12714194,C=14)
	CT(55260)=(S=12714195,E=12714195,C=58)
	CT(55261)=(S=12714196,E=12714197,C=34)
	CT(55262)=(S=12714198,E=12714198,C=14)
	CT(55263)=(S=12714199,E=12714199,C=36)
	CT(55264)=(S=12714200,E=12714200,C=28)
	CT(55265)=(S=12714201,E=12714201,C=19)
	CT(55266)=(S=12714202,E=12714202,C=31)
	CT(55267)=(S=12714203,E=12714203,C=18)
	CT(55268)=(S=12714204,E=12714204,C=58)
	CT(55269)=(S=12714206,E=12714206,C=42)
	CT(55270)=(S=12714208,E=12714208,C=57)
	CT(55271)=(S=12714209,E=12714209,C=18)
	CT(55272)=(S=12714210,E=12714210,C=63)
	CT(55273)=(S=12714213,E=12714213,C=34)
	CT(55274)=(S=12714214,E=12714214,C=42)
	CT(55275)=(S=12714216,E=12714216,C=28)
	CT(55276)=(S=12714217,E=12714217,C=58)
	CT(55277)=(S=12714218,E=12714218,C=49)
	CT(55278)=(S=12714219,E=12714219,C=51)
	CT(55279)=(S=12714220,E=12714220,C=24)
	CT(55280)=(S=12714221,E=12714221,C=31)
	CT(55281)=(S=12714222,E=12714223,C=14)
	CT(55282)=(S=12714224,E=12714224,C=58)
	CT(55283)=(S=12714225,E=12714225,C=63)
	CT(55284)=(S=12714226,E=12714226,C=60)
	CT(55285)=(S=12714230,E=12714230,C=19)
	CT(55286)=(S=12714231,E=12714231,C=28)
	CT(55287)=(S=12714233,E=12714233,C=12)
	CT(55288)=(S=12714234,E=12714234,C=36)
	CT(55289)=(S=12714235,E=12714235,C=58)
	CT(55290)=(S=12714236,E=12714236,C=14)
	CT(55291)=(S=12714237,E=12714237,C=38)
	CT(55292)=(S=12714238,E=12714238,C=28)
	CT(55293)=(S=12714239,E=12714239,C=12)
	CT(55294)=(S=12714240,E=12714383,C=71)
	CT(55295)=(S=12714384,E=12714384,C=58)
	CT(55296)=(S=12714385,E=12714385,C=17)
	CT(55297)=(S=12714386,E=12714386,C=58)
	CT(55298)=(S=12714387,E=12714388,C=14)
	CT(55299)=(S=12714389,E=12714389,C=96)
	CT(55300)=(S=12714390,E=12714391,C=14)
	CT(55301)=(S=12714392,E=12714392,C=18)
	CT(55302)=(S=12714393,E=12714393,C=58)
	CT(55303)=(S=12714394,E=12714394,C=19)
	CT(55304)=(S=12714395,E=12714395,C=28)
	CT(55305)=(S=12714396,E=12714396,C=18)
	CT(55306)=(S=12714397,E=12714397,C=71)
	CT(55307)=(S=12714398,E=12714398,C=42)
	CT(55308)=(S=12714399,E=12714399,C=31)
	CT(55309)=(S=12714400,E=12714400,C=15)
	CT(55310)=(S=12714401,E=12714401,C=18)
	CT(55311)=(S=12714402,E=12714403,C=34)
	CT(55312)=(S=12714404,E=12714406,C=14)
	CT(55313)=(S=12714407,E=12714407,C=38)
	CT(55314)=(S=12714408,E=12714408,C=18)
	CT(55315)=(S=12714409,E=12714409,C=38)
	CT(55316)=(S=12714410,E=12714411,C=18)
	CT(55317)=(S=12714412,E=12714415,C=71)
	CT(55318)=(S=12714416,E=12714416,C=28)
	CT(55319)=(S=12714418,E=12714418,C=38)
	CT(55320)=(S=12714420,E=12714420,C=14)
	CT(55321)=(S=12714421,E=12714421,C=28)
	CT(55322)=(S=12714422,E=12714422,C=14)
	CT(55323)=(S=12714423,E=12714423,C=49)
	CT(55324)=(S=12714425,E=12714425,C=71)
	CT(55325)=(S=12714428,E=12714429,C=33)
	CT(55326)=(S=12714430,E=12714430,C=34)
	CT(55327)=(S=12714431,E=12714431,C=71)
	CT(55328)=(S=12714432,E=12714432,C=24)
	CT(55329)=(S=12714433,E=12714433,C=31)
	CT(55330)=(S=12714434,E=12714434,C=14)
	CT(55331)=(S=12714435,E=12714435,C=31)
	CT(55332)=(S=12714436,E=12714436,C=58)
	CT(55333)=(S=12714437,E=12714437,C=19)
	CT(55334)=(S=12714438,E=12714438,C=18)
	CT(55335)=(S=12714439,E=12714439,C=58)
	CT(55336)=(S=12714440,E=12714440,C=24)
	CT(55337)=(S=12714441,E=12714441,C=58)
	CT(55338)=(S=12714442,E=12714442,C=18)
	CT(55339)=(S=12714443,E=12714443,C=12)
	CT(55340)=(S=12714444,E=12714444,C=36)
	CT(55341)=(S=12714445,E=12714445,C=49)
	CT(55342)=(S=12714446,E=12714446,C=42)
	CT(55343)=(S=12714447,E=12714447,C=28)
	CT(55344)=(S=12714448,E=12714448,C=15)
	CT(55345)=(S=12714449,E=12714449,C=55)
	CT(55346)=(S=12714450,E=12714450,C=14)
	CT(55347)=(S=12714451,E=12714452,C=19)
	CT(55348)=(S=12714453,E=12714454,C=18)
	CT(55349)=(S=12714455,E=12714456,C=71)
	CT(55350)=(S=12714457,E=12714457,C=58)
	CT(55351)=(S=12714459,E=12714459,C=24)
	CT(55352)=(S=12714460,E=12714461,C=31)
	CT(55353)=(S=12714462,E=12714471,C=71)
	CT(55354)=(S=12714472,E=12714483,C=18)
	CT(55355)=(S=12714485,E=12714485,C=34)
	CT(55356)=(S=12714486,E=12714486,C=58)
	CT(55357)=(S=12714487,E=12714487,C=19)
	CT(55358)=(S=12714488,E=12714491,C=18)
	CT(55359)=(S=12714492,E=12714494,C=58)
	CT(55360)=(S=12714495,E=12714495,C=57)
	CT(55361)=(S=12714496,E=12715303,C=12)
	CT(55362)=(S=12715315,E=12715315,C=28)
	CT(55363)=(S=12715319,E=12715319,C=12)
	CT(55364)=(S=12715327,E=12715327,C=28)
	CT(55365)=(S=12715336,E=12715345,C=12)
	CT(55366)=(S=12715351,E=12715351,C=57)
	CT(55367)=(S=12715352,E=12715503,C=12)
	CT(55368)=(S=12715517,E=12715517,C=28)
	CT(55369)=(S=12715520,E=12715615,C=13)
	CT(55370)=(S=12715616,E=12715647,C=14)
	CT(55371)=(S=12715648,E=12715679,C=12)
	CT(55372)=(S=12715680,E=12715711,C=34)
	CT(55373)=(S=12715712,E=12715715,C=28)
	CT(55374)=(S=12715716,E=12715719,C=18)
	CT(55375)=(S=12715720,E=12715723,C=38)
	CT(55376)=(S=12715724,E=12715727,C=58)
	CT(55377)=(S=12715728,E=12715731,C=28)
	CT(55378)=(S=12715732,E=12715735,C=38)
	CT(55379)=(S=12715736,E=12715743,C=18)
	CT(55380)=(S=12715744,E=12715744,C=134)
	CT(55381)=(S=12715745,E=12715745,C=43)
	CT(55382)=(S=12715746,E=12715746,C=28)
	CT(55383)=(S=12715747,E=12715747,C=49)
	CT(55384)=(S=12715748,E=12715748,C=28)
	CT(55385)=(S=12715749,E=12715749,C=21)
	CT(55386)=(S=12715750,E=12715750,C=38)
	CT(55387)=(S=12715751,E=12715753,C=31)
	CT(55388)=(S=12715755,E=12715755,C=14)
	CT(55389)=(S=12715756,E=12715756,C=19)
	CT(55390)=(S=12715757,E=12715757,C=52)
	CT(55391)=(S=12715758,E=12715758,C=27)
	CT(55392)=(S=12715759,E=12715759,C=28)
	CT(55393)=(S=12715760,E=12715760,C=12)
	CT(55394)=(S=12715761,E=12715761,C=58)
	CT(55395)=(S=12715762,E=12715762,C=52)
	CT(55396)=(S=12715763,E=12715763,C=19)
	CT(55397)=(S=12715764,E=12715764,C=14)
	CT(55398)=(S=12715765,E=12715766,C=58)
	CT(55399)=(S=12715768,E=12715768,C=14)
	CT(55400)=(S=12715769,E=12715769,C=28)
	CT(55401)=(S=12715770,E=12715770,C=42)
	CT(55402)=(S=12715773,E=12715773,C=25)
	CT(55403)=(S=12715775,E=12715775,C=94)
	CT(55404)=(S=12715776,E=12716031,C=49)
	CT(55405)=(S=12716032,E=12716032,C=28)
	CT(55406)=(S=12716033,E=12716035,C=60)
	CT(55407)=(S=12716036,E=12716036,C=51)
	CT(55408)=(S=12716037,E=12716043,C=60)
	CT(55409)=(S=12716046,E=12716046,C=58)
	CT(55410)=(S=12716047,E=12716047,C=34)
	CT(55411)=(S=12716048,E=12716059,C=60)
	CT(55412)=(S=12716061,E=12716061,C=19)
	CT(55413)=(S=12716062,E=12716062,C=21)
	CT(55414)=(S=12716063,E=12716063,C=19)
	CT(55415)=(S=12716064,E=12716076,C=60)
	CT(55416)=(S=12716077,E=12716078,C=58)
	CT(55417)=(S=12716079,E=12716079,C=18)
	CT(55418)=(S=12716081,E=12716081,C=14)
	CT(55419)=(S=12716082,E=12716082,C=12)
	CT(55420)=(S=12716083,E=12716083,C=31)
	CT(55421)=(S=12716084,E=12716084,C=49)
	CT(55422)=(S=12716085,E=12716085,C=51)
	CT(55423)=(S=12716086,E=12716086,C=14)
	CT(55424)=(S=12716087,E=12716087,C=18)
	CT(55425)=(S=12716088,E=12716088,C=31)
	CT(55426)=(S=12716089,E=12716090,C=28)
	CT(55427)=(S=12716092,E=12716092,C=51)
	CT(55428)=(S=12716093,E=12716093,C=42)
	CT(55429)=(S=12716094,E=12716094,C=49)
	CT(55430)=(S=12716095,E=12716095,C=42)
	CT(55431)=(S=12716096,E=12716097,C=31)
	CT(55432)=(S=12716098,E=12716101,C=58)
	CT(55433)=(S=12716102,E=12716105,C=18)
	CT(55434)=(S=12716106,E=12716107,C=14)
	CT(55435)=(S=12716108,E=12716109,C=17)
	CT(55436)=(S=12716110,E=12716111,C=34)
	CT(55437)=(S=12716112,E=12716115,C=38)
	CT(55438)=(S=12716116,E=12716117,C=18)
	CT(55439)=(S=12716120,E=12716121,C=17)
	CT(55440)=(S=12716122,E=12716123,C=28)
	CT(55441)=(S=12716124,E=12716125,C=14)
	CT(55442)=(S=12716126,E=12716127,C=52)
	CT(55443)=(S=12716128,E=12716159,C=28)
	CT(55444)=(S=12716160,E=12716171,C=18)
	CT(55445)=(S=12716176,E=12716179,C=31)
	CT(55446)=(S=12716180,E=12716183,C=12)
	CT(55447)=(S=12716184,E=12716187,C=18)
	CT(55448)=(S=12716188,E=12716191,C=31)
	CT(55449)=(S=12716192,E=12716223,C=18)
	CT(55450)=(S=12716224,E=12716255,C=28)
	CT(55451)=(S=12716256,E=12716257,C=18)
	CT(55452)=(S=12716258,E=12716259,C=47)
	CT(55453)=(S=12716260,E=12716267,C=18)
	CT(55454)=(S=12716268,E=12716269,C=36)
	CT(55455)=(S=12716270,E=12716271,C=71)
	CT(55456)=(S=12716272,E=12716273,C=31)
	CT(55457)=(S=12716274,E=12716277,C=24)
	CT(55458)=(S=12716278,E=12716279,C=18)
	CT(55459)=(S=12716280,E=12716281,C=31)
	CT(55460)=(S=12716282,E=12716283,C=12)
	CT(55461)=(S=12716284,E=12716285,C=33)
	CT(55462)=(S=12716286,E=12716287,C=14)
	CT(55463)=(S=12716290,E=12716291,C=18)
	CT(55464)=(S=12716292,E=12716293,C=28)
	CT(55465)=(S=12716294,E=12716295,C=51)
	CT(55466)=(S=12716298,E=12716299,C=28)
	CT(55467)=(S=12716300,E=12716301,C=187)
	CT(55468)=(S=12716302,E=12716303,C=31)
	CT(55469)=(S=12716304,E=12716305,C=58)
	CT(55470)=(S=12716308,E=12716309,C=32)
	CT(55471)=(S=12716310,E=12716311,C=38)
	CT(55472)=(S=12716312,E=12716313,C=58)
	CT(55473)=(S=12716314,E=12716315,C=31)
	CT(55474)=(S=12716316,E=12716317,C=14)
	CT(55475)=(S=12716318,E=12716319,C=20)
	CT(55476)=(S=12716320,E=12716321,C=14)
	CT(55477)=(S=12716322,E=12716323,C=34)
	CT(55478)=(S=12716324,E=12716325,C=31)
	CT(55479)=(S=12716326,E=12716327,C=18)
	CT(55480)=(S=12716328,E=12716329,C=14)
	CT(55481)=(S=12716330,E=12716331,C=36)
	CT(55482)=(S=12716332,E=12716333,C=34)
	CT(55483)=(S=12716334,E=12716335,C=58)
	CT(55484)=(S=12716336,E=12716337,C=36)
	CT(55485)=(S=12716338,E=12716339,C=31)
	CT(55486)=(S=12716340,E=12716341,C=58)
	CT(55487)=(S=12716342,E=12716343,C=34)
	CT(55488)=(S=12716346,E=12716347,C=42)
	CT(55489)=(S=12716348,E=12716349,C=38)
	CT(55490)=(S=12716350,E=12716351,C=12)
	CT(55491)=(S=12716352,E=12716353,C=109)
	CT(55492)=(S=12716354,E=12716356,C=58)
	CT(55493)=(S=12716357,E=12716357,C=18)
	CT(55494)=(S=12716358,E=12716359,C=31)
	CT(55495)=(S=12716360,E=12716361,C=28)
	CT(55496)=(S=12716362,E=12716363,C=19)
	CT(55497)=(S=12716364,E=12716365,C=14)
	CT(55498)=(S=12716366,E=12716367,C=58)
	CT(55499)=(S=12716370,E=12716371,C=109)
	CT(55500)=(S=12716372,E=12716373,C=36)
	CT(55501)=(S=12716374,E=12716377,C=28)
	CT(55502)=(S=12716378,E=12716381,C=38)
	CT(55503)=(S=12716382,E=12716404,C=19)
	CT(55504)=(S=12716405,E=12716407,C=28)
	CT(55505)=(S=12716408,E=12716411,C=34)
	CT(55506)=(S=12716412,E=12716454,C=28)
	CT(55507)=(S=12716455,E=12716455,C=38)
	CT(55508)=(S=12716456,E=12716457,C=28)
	CT(55509)=(S=12716458,E=12716458,C=60)
	CT(55510)=(S=12716459,E=12716459,C=98)
	CT(55511)=(S=12716463,E=12716463,C=60)
	CT(55512)=(S=12716468,E=12716469,C=42)
	CT(55513)=(S=12716470,E=12716471,C=28)
	CT(55514)=(S=12716476,E=12716479,C=14)
	CT(55515)=(S=12716480,E=12716495,C=28)
	CT(55516)=(S=12716500,E=12716503,C=60)
	CT(55517)=(S=12716508,E=12716509,C=15)
	CT(55518)=(S=12716511,E=12716511,C=58)
	CT(55519)=(S=12716512,E=12716527,C=18)
	CT(55520)=(S=12716528,E=12716543,C=14)
	CT(55521)=(S=12716544,E=12716799,C=13)
	CT(55522)=(S=12716800,E=12716815,C=34)
	CT(55523)=(S=12716816,E=12716816,C=18)
	CT(55524)=(S=12716817,E=12716818,C=14)
	CT(55525)=(S=12716820,E=12716821,C=18)
	CT(55526)=(S=12716822,E=12716822,C=58)
	CT(55527)=(S=12716823,E=12716823,C=21)
	CT(55528)=(S=12716824,E=12716824,C=58)
	CT(55529)=(S=12716826,E=12716826,C=12)
	CT(55530)=(S=12716827,E=12716827,C=42)
	CT(55531)=(S=12716828,E=12716828,C=31)
	CT(55532)=(S=12716829,E=12716829,C=18)
	CT(55533)=(S=12716830,E=12716969,C=34)
	CT(55534)=(S=12716970,E=12716970,C=28)
	CT(55535)=(S=12716971,E=12716971,C=42)
	CT(55536)=(S=12716972,E=12717023,C=34)
	CT(55537)=(S=12717025,E=12717025,C=31)
	CT(55538)=(S=12717027,E=12717046,C=34)
	CT(55539)=(S=12717047,E=12717047,C=18)
	CT(55540)=(S=12717052,E=12717054,C=34)
	CT(55541)=(S=12717056,E=12717071,C=19)
	CT(55542)=(S=12717072,E=12717087,C=34)
	CT(55543)=(S=12717088,E=12717103,C=33)
	CT(55544)=(S=12717104,E=12717119,C=32)
	CT(55545)=(S=12717120,E=12717183,C=31)
	CT(55546)=(S=12717184,E=12717247,C=34)
	CT(55547)=(S=12717248,E=12717279,C=28)
	CT(55548)=(S=12717280,E=12717311,C=51)
	CT(55549)=(S=12717312,E=12717419,C=36)
	CT(55550)=(S=12717423,E=12717423,C=22)
	CT(55551)=(S=12717424,E=12717445,C=36)
	CT(55552)=(S=12717447,E=12717447,C=28)
	CT(55553)=(S=12717454,E=12717567,C=36)
	CT(55554)=(S=12717568,E=12717823,C=19)
	CT(55555)=(S=12717824,E=12717919,C=28)
	CT(55556)=(S=12717936,E=12717939,C=31)
	CT(55557)=(S=12717940,E=12717943,C=18)
	CT(55558)=(S=12717944,E=12717948,C=58)
	CT(55559)=(S=12717949,E=12717951,C=18)
	CT(55560)=(S=12717952,E=12717963,C=28)
	CT(55561)=(S=12717965,E=12717965,C=19)
	CT(55562)=(S=12717966,E=12717969,C=28)
	CT(55563)=(S=12717971,E=12717971,C=31)
	CT(55564)=(S=12717972,E=12718035,C=28)
	CT(55565)=(S=12718036,E=12718036,C=19)
	CT(55566)=(S=12718037,E=12718037,C=25)
	CT(55567)=(S=12718038,E=12718047,C=28)
	CT(55568)=(S=12718052,E=12718055,C=34)
	CT(55569)=(S=12718057,E=12718079,C=28)
	CT(55570)=(S=12718080,E=12718847,C=19)
	CT(55571)=(S=12718848,E=12718975,C=27)
	CT(55572)=(S=12718976,E=12719071,C=24)
	CT(55573)=(S=12719072,E=12719103,C=60)
	CT(55574)=(S=12719104,E=12719615,C=15)
	CT(55575)=(S=12719616,E=12720127,C=19)
	CT(55576)=(S=12720128,E=12720255,C=34)
	CT(55577)=(S=12720256,E=12720287,C=42)
	CT(55578)=(S=12720288,E=12720289,C=28)
	CT(55579)=(S=12720292,E=12720293,C=58)
	CT(55580)=(S=12720294,E=12720295,C=14)
	CT(55581)=(S=12720296,E=12720297,C=37)
	CT(55582)=(S=12720298,E=12720299,C=22)
	CT(55583)=(S=12720300,E=12720301,C=28)
	CT(55584)=(S=12720302,E=12720303,C=58)
	CT(55585)=(S=12720306,E=12720307,C=38)
	CT(55586)=(S=12720308,E=12720309,C=58)
	CT(55587)=(S=12720310,E=12720315,C=31)
	CT(55588)=(S=12720316,E=12720317,C=14)
	CT(55589)=(S=12720318,E=12720319,C=31)
	CT(55590)=(S=12720320,E=12720351,C=28)
	CT(55591)=(S=12720352,E=12720353,C=37)
	CT(55592)=(S=12720354,E=12720355,C=22)
	CT(55593)=(S=12720356,E=12720357,C=12)
	CT(55594)=(S=12720358,E=12720359,C=28)
	CT(55595)=(S=12720362,E=12720363,C=38)
	CT(55596)=(S=12720364,E=12720365,C=31)
	CT(55597)=(S=12720366,E=12720367,C=58)
	CT(55598)=(S=12720368,E=12720369,C=18)
	CT(55599)=(S=12720370,E=12720371,C=28)
	CT(55600)=(S=12720372,E=12720373,C=58)
	CT(55601)=(S=12720374,E=12720375,C=31)
	CT(55602)=(S=12720376,E=12720377,C=46)
	CT(55603)=(S=12720380,E=12720381,C=27)
	CT(55604)=(S=12720382,E=12720383,C=14)
	CT(55605)=(S=12720384,E=12720639,C=28)
	CT(55606)=(S=12720640,E=12720655,C=36)
	CT(55607)=(S=12720662,E=12720663,C=22)
	CT(55608)=(S=12720664,E=12720664,C=49)
	CT(55609)=(S=12720666,E=12720666,C=34)
	CT(55610)=(S=12720668,E=12720797,C=36)
	CT(55611)=(S=12720798,E=12720799,C=28)
	CT(55612)=(S=12720800,E=12720895,C=36)
	CT(55613)=(S=12720896,E=12721151,C=37)
	CT(55614)=(S=12721152,E=12721155,C=47)
	CT(55615)=(S=12721156,E=12721159,C=31)
	CT(55616)=(S=12721160,E=12721163,C=14)
	CT(55617)=(S=12721164,E=12721167,C=58)
	CT(55618)=(S=12721168,E=12721171,C=31)
	CT(55619)=(S=12721172,E=12721175,C=18)
	CT(55620)=(S=12721176,E=12721179,C=24)
	CT(55621)=(S=12721180,E=12721183,C=18)
	CT(55622)=(S=12721184,E=12721187,C=58)
	CT(55623)=(S=12721188,E=12721191,C=31)
	CT(55624)=(S=12721192,E=12721203,C=58)
	CT(55625)=(S=12721204,E=12721207,C=31)
	CT(55626)=(S=12721208,E=12721211,C=18)
	CT(55627)=(S=12721212,E=12721223,C=31)
	CT(55628)=(S=12721224,E=12721227,C=18)
	CT(55629)=(S=12721228,E=12721231,C=53)
	CT(55630)=(S=12721232,E=12721235,C=58)
	CT(55631)=(S=12721236,E=12721243,C=31)
	CT(55632)=(S=12721244,E=12721251,C=18)
	CT(55633)=(S=12721252,E=12721255,C=31)
	CT(55634)=(S=12721256,E=12721259,C=58)
	CT(55635)=(S=12721260,E=12721263,C=52)
	CT(55636)=(S=12721264,E=12721267,C=53)
	CT(55637)=(S=12721268,E=12721271,C=15)
	CT(55638)=(S=12721272,E=12721275,C=19)
	CT(55639)=(S=12721276,E=12721279,C=14)
	CT(55640)=(S=12721280,E=12721283,C=46)
	CT(55641)=(S=12721284,E=12721287,C=31)
	CT(55642)=(S=12721288,E=12721291,C=14)
	CT(55643)=(S=12721292,E=12721299,C=31)
	CT(55644)=(S=12721300,E=12721303,C=28)
	CT(55645)=(S=12721304,E=12721307,C=18)
	CT(55646)=(S=12721308,E=12721311,C=14)
	CT(55647)=(S=12721312,E=12721315,C=18)
	CT(55648)=(S=12721316,E=12721319,C=14)
	CT(55649)=(S=12721320,E=12721323,C=58)
	CT(55650)=(S=12721324,E=12721335,C=31)
	CT(55651)=(S=12721336,E=12721339,C=18)
	CT(55652)=(S=12721340,E=12721343,C=58)
	CT(55653)=(S=12721344,E=12721347,C=18)
	CT(55654)=(S=12721348,E=12721351,C=31)
	CT(55655)=(S=12721352,E=12721355,C=58)
	CT(55656)=(S=12721356,E=12721371,C=18)
	CT(55657)=(S=12721372,E=12721375,C=58)
	CT(55658)=(S=12721376,E=12721379,C=18)
	CT(55659)=(S=12721380,E=12721383,C=58)
	CT(55660)=(S=12721384,E=12721387,C=44)
	CT(55661)=(S=12721388,E=12721395,C=18)
	CT(55662)=(S=12721396,E=12721399,C=14)
	CT(55663)=(S=12721400,E=12721407,C=24)
	CT(55664)=(S=12721408,E=12721439,C=34)
	CT(55665)=(S=12721440,E=12721455,C=17)
	CT(55666)=(S=12721456,E=12721459,C=22)
	CT(55667)=(S=12721464,E=12721467,C=34)
	CT(55668)=(S=12721468,E=12721471,C=31)
	CT(55669)=(S=12721472,E=12721535,C=13)
	CT(55670)=(S=12721536,E=12721591,C=58)
	CT(55671)=(S=12721592,E=12721592,C=28)
	CT(55672)=(S=12721593,E=12721593,C=50)
	CT(55673)=(S=12721594,E=12721595,C=18)
	CT(55674)=(S=12721596,E=12721599,C=28)
	CT(55675)=(S=12721600,E=12721607,C=47)
	CT(55676)=(S=12721608,E=12721611,C=27)
	CT(55677)=(S=12721612,E=12721612,C=15)
	CT(55678)=(S=12721613,E=12721613,C=31)
	CT(55679)=(S=12721614,E=12721614,C=12)
	CT(55680)=(S=12721615,E=12721615,C=24)
	CT(55681)=(S=12721616,E=12721623,C=37)
	CT(55682)=(S=12721624,E=12721631,C=14)
	CT(55683)=(S=12721632,E=12721663,C=28)
	CT(55684)=(S=12721664,E=12721791,C=25)
	CT(55685)=(S=12721792,E=12721823,C=32)
	CT(55686)=(S=12721824,E=12721824,C=36)
	CT(55687)=(S=12721826,E=12721826,C=28)
	CT(55688)=(S=12721827,E=12721827,C=31)
	CT(55689)=(S=12721828,E=12721828,C=58)
	CT(55690)=(S=12721830,E=12721830,C=14)
	CT(55691)=(S=12721831,E=12721831,C=28)
	CT(55692)=(S=12721832,E=12721832,C=38)
	CT(55693)=(S=12721833,E=12721833,C=19)
	CT(55694)=(S=12721834,E=12721834,C=31)
	CT(55695)=(S=12721835,E=12721835,C=14)
	CT(55696)=(S=12721836,E=12721836,C=31)
	CT(55697)=(S=12721837,E=12721838,C=28)
	CT(55698)=(S=12721839,E=12721839,C=36)
	CT(55699)=(S=12721842,E=12721842,C=22)
	CT(55700)=(S=12721843,E=12721843,C=58)
	CT(55701)=(S=12721844,E=12721844,C=28)
	CT(55702)=(S=12721845,E=12721845,C=34)
	CT(55703)=(S=12721846,E=12721846,C=36)
	CT(55704)=(S=12721848,E=12721848,C=14)
	CT(55705)=(S=12721849,E=12721849,C=28)
	CT(55706)=(S=12721850,E=12721850,C=14)
	CT(55707)=(S=12721851,E=12721851,C=71)
	CT(55708)=(S=12721852,E=12721852,C=36)
	CT(55709)=(S=12721853,E=12721853,C=14)
	CT(55710)=(S=12721854,E=12721854,C=18)
	CT(55711)=(S=12721855,E=12721855,C=42)
	CT(55712)=(S=12721856,E=12721919,C=22)
	CT(55713)=(S=12721920,E=12721951,C=28)
	CT(55714)=(S=12721952,E=12721958,C=58)
	CT(55715)=(S=12721959,E=12721959,C=27)
	CT(55716)=(S=12721960,E=12721961,C=58)
	CT(55717)=(S=12721962,E=12721962,C=53)
	CT(55718)=(S=12721963,E=12721963,C=58)
	CT(55719)=(S=12721964,E=12721967,C=31)
	CT(55720)=(S=12721968,E=12721969,C=58)
	CT(55721)=(S=12721970,E=12721971,C=14)
	CT(55722)=(S=12721976,E=12721976,C=55)
	CT(55723)=(S=12721977,E=12721977,C=101)
	CT(55724)=(S=12721978,E=12721978,C=17)
	CT(55725)=(S=12721979,E=12721979,C=18)
	CT(55726)=(S=12721980,E=12721980,C=58)
	CT(55727)=(S=12721981,E=12721981,C=48)
	CT(55728)=(S=12721982,E=12721982,C=14)
	CT(55729)=(S=12721983,E=12721983,C=58)
	CT(55730)=(S=12721985,E=12722126,C=28)
	CT(55731)=(S=12722127,E=12722127,C=34)
	CT(55732)=(S=12722128,E=12722151,C=28)
	CT(55733)=(S=12722152,E=12722152,C=18)
	CT(55734)=(S=12722153,E=12722166,C=28)
	CT(55735)=(S=12722168,E=12722169,C=31)
	CT(55736)=(S=12722170,E=12722171,C=18)
	CT(55737)=(S=12722172,E=12722172,C=58)
	CT(55738)=(S=12722173,E=12722173,C=18)
	CT(55739)=(S=12722174,E=12722174,C=34)
	CT(55740)=(S=12722176,E=12722258,C=14)
	CT(55741)=(S=12722260,E=12722261,C=37)
	CT(55742)=(S=12722262,E=12722263,C=18)
	CT(55743)=(S=12722326,E=12722326,C=46)
	CT(55744)=(S=12722327,E=12722327,C=19)
	CT(55745)=(S=12722336,E=12722443,C=14)
	CT(55746)=(S=12722444,E=12722444,C=52)
	CT(55747)=(S=12722445,E=12722445,C=14)
	CT(55748)=(S=12722446,E=12722446,C=38)
	CT(55749)=(S=12722447,E=12722447,C=31)
	CT(55750)=(S=12722449,E=12722451,C=28)
	CT(55751)=(S=12722456,E=12722495,C=14)
	CT(55752)=(S=12722496,E=12722499,C=38)
	CT(55753)=(S=12722500,E=12722500,C=23)
	CT(55754)=(S=12722501,E=12722501,C=50)
	CT(55755)=(S=12722502,E=12722503,C=23)
	CT(55756)=(S=12722504,E=12722509,C=58)
	CT(55757)=(S=12722510,E=12722510,C=12)
	CT(55758)=(S=12722511,E=12722511,C=18)
	CT(55759)=(S=12722512,E=12722512,C=19)
	CT(55760)=(S=12722513,E=12722514,C=18)
	CT(55761)=(S=12722515,E=12722515,C=58)
	CT(55762)=(S=12722516,E=12722527,C=18)
	CT(55763)=(S=12722528,E=12722528,C=36)
	CT(55764)=(S=12722529,E=12722529,C=18)
	CT(55765)=(S=12722530,E=12722531,C=34)
	CT(55766)=(S=12722532,E=12722535,C=38)
	CT(55767)=(S=12722536,E=12722539,C=26)
	CT(55768)=(S=12722540,E=12722540,C=42)
	CT(55769)=(S=12722541,E=12722541,C=14)
	CT(55770)=(S=12722542,E=12722542,C=18)
	CT(55771)=(S=12722543,E=12722543,C=58)
	CT(55772)=(S=12722544,E=12722545,C=36)
	CT(55773)=(S=12722546,E=12722546,C=18)
	CT(55774)=(S=12722547,E=12722547,C=28)
	CT(55775)=(S=12722548,E=12722551,C=58)
	CT(55776)=(S=12722552,E=12722553,C=28)
	CT(55777)=(S=12722554,E=12722559,C=26)
	CT(55778)=(S=12722560,E=12722611,C=14)
	CT(55779)=(S=12722612,E=12722613,C=31)
	CT(55780)=(S=12722615,E=12722615,C=14)
	CT(55781)=(S=12722616,E=12722616,C=50)
	CT(55782)=(S=12722617,E=12722617,C=58)
	CT(55783)=(S=12722618,E=12722618,C=34)
	CT(55784)=(S=12722619,E=12722619,C=58)
	CT(55785)=(S=12722620,E=12722621,C=31)
	CT(55786)=(S=12722622,E=12722622,C=19)
	CT(55787)=(S=12722623,E=12722623,C=18)
	CT(55788)=(S=12722624,E=12722699,C=14)
	CT(55789)=(S=12722700,E=12722700,C=51)
	CT(55790)=(S=12722701,E=12722879,C=14)
	CT(55791)=(S=12722886,E=12722886,C=58)
	CT(55792)=(S=12722892,E=12723016,C=14)
	CT(55793)=(S=12723018,E=12723018,C=38)
	CT(55794)=(S=12723023,E=12723199,C=14)
	CT(55795)=(S=12723200,E=12723200,C=24)
	CT(55796)=(S=12723201,E=12723215,C=14)
	CT(55797)=(S=12723216,E=12723217,C=18)
	CT(55798)=(S=12723218,E=12723218,C=28)
	CT(55799)=(S=12723219,E=12723219,C=18)
	CT(55800)=(S=12723220,E=12723220,C=28)
	CT(55801)=(S=12723221,E=12723221,C=14)
	CT(55802)=(S=12723222,E=12723222,C=12)
	CT(55803)=(S=12723223,E=12723359,C=14)
	CT(55804)=(S=12723360,E=12723360,C=37)
	CT(55805)=(S=12723361,E=12723361,C=18)
	CT(55806)=(S=12723362,E=12723362,C=55)
	CT(55807)=(S=12723363,E=12723363,C=14)
	CT(55808)=(S=12723364,E=12723364,C=23)
	CT(55809)=(S=12723365,E=12723365,C=58)
	CT(55810)=(S=12723366,E=12723366,C=12)
	CT(55811)=(S=12723368,E=12723368,C=58)
	CT(55812)=(S=12723410,E=12723420,C=14)
	CT(55813)=(S=12723424,E=12723439,C=70)
	CT(55814)=(S=12723440,E=12723455,C=14)
	CT(55815)=(S=12723456,E=12723703,C=42)
	CT(55816)=(S=12723704,E=12723704,C=31)
	CT(55817)=(S=12723705,E=12723705,C=19)
	CT(55818)=(S=12723708,E=12723708,C=15)
	CT(55819)=(S=12723709,E=12723709,C=42)
	CT(55820)=(S=12723711,E=12723711,C=28)
	CT(55821)=(S=12723776,E=12723807,C=34)
	CT(55822)=(S=12723808,E=12723839,C=50)
	CT(55823)=(S=12723840,E=12723871,C=21)
	CT(55824)=(S=12723872,E=12723967,C=34)
	CT(55825)=(S=12723968,E=12724049,C=28)
	CT(55826)=(S=12724052,E=12724052,C=21)
	CT(55827)=(S=12724053,E=12724053,C=52)
	CT(55828)=(S=12724054,E=12724108,C=28)
	CT(55829)=(S=12724109,E=12724109,C=49)
	CT(55830)=(S=12724111,E=12724111,C=14)
	CT(55831)=(S=12724118,E=12724151,C=28)
	CT(55832)=(S=12724152,E=12724152,C=36)
	CT(55833)=(S=12724153,E=12724223,C=28)
	CT(55834)=(S=12724224,E=12724427,C=34)
	CT(55835)=(S=12724428,E=12724431,C=38)
	CT(55836)=(S=12724432,E=12724433,C=15)
	CT(55837)=(S=12724434,E=12724434,C=38)
	CT(55838)=(S=12724435,E=12724435,C=52)
	CT(55839)=(S=12724436,E=12724473,C=34)
	CT(55840)=(S=12724480,E=12724480,C=28)
	CT(55841)=(S=12724482,E=12724483,C=19)
	CT(55842)=(S=12724484,E=12724489,C=15)
	CT(55843)=(S=12724492,E=12724495,C=58)
	CT(55844)=(S=12724496,E=12724511,C=47)
	CT(55845)=(S=12724519,E=12724537,C=28)
	CT(55846)=(S=12724540,E=12724541,C=14)
	CT(55847)=(S=12724542,E=12724542,C=36)
	CT(55848)=(S=12724543,E=12724575,C=34)
	CT(55849)=(S=12724576,E=12724591,C=49)
	CT(55850)=(S=12724608,E=12724735,C=34)
	CT(55851)=(S=12724736,E=12724775,C=32)
	CT(55852)=(S=12724776,E=12724776,C=58)
	CT(55853)=(S=12724777,E=12724778,C=15)
	CT(55854)=(S=12724779,E=12724779,C=21)
	CT(55855)=(S=12724780,E=12724781,C=38)
	CT(55856)=(S=12724782,E=12724783,C=28)
	CT(55857)=(S=12724784,E=12724784,C=34)
	CT(55858)=(S=12724785,E=12724785,C=24)
	CT(55859)=(S=12724786,E=12724790,C=22)
	CT(55860)=(S=12724791,E=12724791,C=24)
	CT(55861)=(S=12724792,E=12724799,C=47)
	CT(55862)=(S=12724800,E=12724831,C=14)
	CT(55863)=(S=12724834,E=12724835,C=70)
	CT(55864)=(S=12724836,E=12724839,C=38)
	CT(55865)=(S=12724842,E=12724843,C=31)
	CT(55866)=(S=12724844,E=12724845,C=28)
	CT(55867)=(S=12724846,E=12724847,C=58)
	CT(55868)=(S=12724848,E=12724849,C=31)
	CT(55869)=(S=12724850,E=12724851,C=28)
	CT(55870)=(S=12724852,E=12724857,C=58)
	CT(55871)=(S=12724858,E=12724859,C=36)
	CT(55872)=(S=12724860,E=12724861,C=14)
	CT(55873)=(S=12724862,E=12724863,C=38)
	CT(55874)=(S=12724864,E=12724895,C=32)
	CT(55875)=(S=12724896,E=12724927,C=13)
	CT(55876)=(S=12724928,E=12724943,C=31)
	CT(55877)=(S=12724944,E=12724945,C=49)
	CT(55878)=(S=12724946,E=12724947,C=36)
	CT(55879)=(S=12724948,E=12724948,C=70)
	CT(55880)=(S=12724949,E=12724950,C=25)
	CT(55881)=(S=12724951,E=12724951,C=14)
	CT(55882)=(S=12724952,E=12724952,C=29)
	CT(55883)=(S=12724953,E=12724953,C=12)
	CT(55884)=(S=12724954,E=12724955,C=49)
	CT(55885)=(S=12724956,E=12724959,C=36)
	CT(55886)=(S=12724960,E=12725247,C=14)
	CT(55887)=(S=12725248,E=12725503,C=31)
	CT(55888)=(S=12725504,E=12725759,C=28)
	CT(55889)=(S=12725760,E=12725951,C=14)
	CT(55890)=(S=12725952,E=12726015,C=63)
	CT(55891)=(S=12726016,E=12726271,C=19)
	CT(55892)=(S=12726272,E=12726272,C=42)
	CT(55893)=(S=12726273,E=12726273,C=19)
	CT(55894)=(S=12726274,E=12726475,C=42)
	CT(55895)=(S=12726476,E=12726476,C=14)
	CT(55896)=(S=12726477,E=12726477,C=19)
	CT(55897)=(S=12726478,E=12726478,C=51)
	CT(55898)=(S=12726479,E=12726479,C=25)
	CT(55899)=(S=12726480,E=12726480,C=12)
	CT(55900)=(S=12726481,E=12726481,C=31)
	CT(55901)=(S=12726482,E=12726482,C=14)
	CT(55902)=(S=12726484,E=12726484,C=31)
	CT(55903)=(S=12726485,E=12726485,C=19)
	CT(55904)=(S=12726486,E=12726486,C=36)
	CT(55905)=(S=12726487,E=12726487,C=34)
	CT(55906)=(S=12726488,E=12726488,C=58)
	CT(55907)=(S=12726489,E=12726489,C=14)
	CT(55908)=(S=12726490,E=12726490,C=18)
	CT(55909)=(S=12726491,E=12726491,C=58)
	CT(55910)=(S=12726508,E=12726512,C=42)
	CT(55911)=(S=12726513,E=12726513,C=33)
	CT(55912)=(S=12726515,E=12726526,C=42)
	CT(55913)=(S=12726535,E=12726547,C=28)
	CT(55914)=(S=12726549,E=12726549,C=34)
	CT(55915)=(S=12726550,E=12726577,C=28)
	CT(55916)=(S=12726578,E=12726579,C=18)
	CT(55917)=(S=12726582,E=12726647,C=28)
	CT(55918)=(S=12726648,E=12726651,C=18)
	CT(55919)=(S=12726652,E=12726652,C=58)
	CT(55920)=(S=12726653,E=12726653,C=28)
	CT(55921)=(S=12726654,E=12726654,C=37)
	CT(55922)=(S=12726655,E=12726655,C=18)
	CT(55923)=(S=12726656,E=12726783,C=28)
	CT(55924)=(S=12726786,E=12726786,C=33)
	CT(55925)=(S=12726790,E=12726790,C=18)
	CT(55926)=(S=12726791,E=12726791,C=38)
	CT(55927)=(S=12726793,E=12726793,C=31)
	CT(55928)=(S=12726794,E=12726794,C=18)
	CT(55929)=(S=12726795,E=12726795,C=38)
	CT(55930)=(S=12726808,E=12726808,C=18)
	CT(55931)=(S=12726810,E=12726811,C=33)
	CT(55932)=(S=12726812,E=12726812,C=42)
	CT(55933)=(S=12726813,E=12726813,C=12)
	CT(55934)=(S=12726814,E=12726816,C=14)
	CT(55935)=(S=12726817,E=12726817,C=28)
	CT(55936)=(S=12726818,E=12726818,C=56)
	CT(55937)=(S=12726819,E=12726819,C=23)
	CT(55938)=(S=12726820,E=12726820,C=28)
	CT(55939)=(S=12726821,E=12726821,C=58)
	CT(55940)=(S=12726822,E=12726823,C=12)
	CT(55941)=(S=12726824,E=12726824,C=24)
	CT(55942)=(S=12726825,E=12726825,C=14)
	CT(55943)=(S=12726826,E=12726827,C=22)
	CT(55944)=(S=12726828,E=12726828,C=38)
	CT(55945)=(S=12726829,E=12726829,C=34)
	CT(55946)=(S=12726831,E=12726831,C=28)
	CT(55947)=(S=12726833,E=12726833,C=58)
	CT(55948)=(S=12726834,E=12726834,C=38)
	CT(55949)=(S=12726835,E=12726835,C=31)
	CT(55950)=(S=12726837,E=12726837,C=58)
	CT(55951)=(S=12726838,E=12726840,C=14)
	CT(55952)=(S=12726841,E=12726841,C=24)
	CT(55953)=(S=12726842,E=12726842,C=28)
	CT(55954)=(S=12726844,E=12726844,C=12)
	CT(55955)=(S=12726845,E=12726845,C=58)
	CT(55956)=(S=12726846,E=12726846,C=49)
	CT(55957)=(S=12726847,E=12726847,C=38)
	CT(55958)=(S=12726848,E=12726851,C=33)
	CT(55959)=(S=12726852,E=12726854,C=28)
	CT(55960)=(S=12726855,E=12726855,C=17)
	CT(55961)=(S=12726859,E=12726859,C=58)
	CT(55962)=(S=12726860,E=12726860,C=51)
	CT(55963)=(S=12726861,E=12726862,C=12)
	CT(55964)=(S=12726863,E=12726863,C=31)
	CT(55965)=(S=12726864,E=12726864,C=14)
	CT(55966)=(S=12726865,E=12726865,C=31)
	CT(55967)=(S=12726867,E=12726867,C=58)
	CT(55968)=(S=12726868,E=12726868,C=37)
	CT(55969)=(S=12726869,E=12726869,C=31)
	CT(55970)=(S=12726870,E=12726870,C=14)
	CT(55971)=(S=12726871,E=12726871,C=18)
	CT(55972)=(S=12726872,E=12726872,C=14)
	CT(55973)=(S=12726873,E=12726873,C=58)
	CT(55974)=(S=12726874,E=12726874,C=14)
	CT(55975)=(S=12726875,E=12726875,C=49)
	CT(55976)=(S=12726877,E=12726877,C=28)
	CT(55977)=(S=12726879,E=12726879,C=12)
	CT(55978)=(S=12726880,E=12726880,C=36)
	CT(55979)=(S=12726881,E=12726881,C=49)
	CT(55980)=(S=12726882,E=12726882,C=31)
	CT(55981)=(S=12726883,E=12726883,C=36)
	CT(55982)=(S=12726884,E=12726884,C=33)
	CT(55983)=(S=12726885,E=12726885,C=50)
	CT(55984)=(S=12726886,E=12726886,C=36)
	CT(55985)=(S=12726887,E=12726888,C=18)
	CT(55986)=(S=12726889,E=12726889,C=60)
	CT(55987)=(S=12726890,E=12726891,C=19)
	CT(55988)=(S=12726892,E=12726892,C=14)
	CT(55989)=(S=12726893,E=12726893,C=42)
	CT(55990)=(S=12726894,E=12726894,C=58)
	CT(55991)=(S=12726896,E=12726896,C=36)
	CT(55992)=(S=12726897,E=12726897,C=28)
	CT(55993)=(S=12726898,E=12726898,C=31)
	CT(55994)=(S=12726899,E=12726899,C=42)
	CT(55995)=(S=12726900,E=12726900,C=18)
	CT(55996)=(S=12726901,E=12726901,C=58)
	CT(55997)=(S=12726902,E=12726902,C=14)
	CT(55998)=(S=12726903,E=12726903,C=31)
	CT(55999)=(S=12726904,E=12726904,C=18)
	CT(56000)=(S=12726905,E=12726905,C=14)
	CT(56001)=(S=12726906,E=12726906,C=51)
	CT(56002)=(S=12726907,E=12726907,C=58)
	CT(56003)=(S=12726908,E=12726908,C=49)
	CT(56004)=(S=12726909,E=12726909,C=31)
	CT(56005)=(S=12726910,E=12726910,C=38)
	CT(56006)=(S=12726911,E=12726911,C=12)
	CT(56007)=(S=12726912,E=12726915,C=18)
	CT(56008)=(S=12726916,E=12726919,C=58)
	CT(56009)=(S=12726920,E=12726922,C=52)
	CT(56010)=(S=12726923,E=12726923,C=18)
	CT(56011)=(S=12726924,E=12726924,C=19)
	CT(56012)=(S=12726925,E=12726927,C=18)
	CT(56013)=(S=12726928,E=12726931,C=31)
	CT(56014)=(S=12726932,E=12726934,C=18)
	CT(56015)=(S=12726935,E=12726935,C=42)
	CT(56016)=(S=12726936,E=12726936,C=18)
	CT(56017)=(S=12726937,E=12726938,C=14)
	CT(56018)=(S=12726939,E=12726943,C=58)
	CT(56019)=(S=12726944,E=12726944,C=28)
	CT(56020)=(S=12726945,E=12726945,C=31)
	CT(56021)=(S=12726946,E=12726946,C=28)
	CT(56022)=(S=12726947,E=12726947,C=36)
	CT(56023)=(S=12726948,E=12726948,C=28)
	CT(56024)=(S=12726949,E=12726949,C=14)
	CT(56025)=(S=12726950,E=12726950,C=52)
	CT(56026)=(S=12726951,E=12726951,C=31)
	CT(56027)=(S=12726952,E=12726952,C=47)
	CT(56028)=(S=12726953,E=12726953,C=31)
	CT(56029)=(S=12726956,E=12726956,C=34)
	CT(56030)=(S=12726957,E=12726957,C=42)
	CT(56031)=(S=12726958,E=12726958,C=38)
	CT(56032)=(S=12726959,E=12726959,C=17)
	CT(56033)=(S=12726960,E=12726960,C=34)
	CT(56034)=(S=12726961,E=12726961,C=49)
	CT(56035)=(S=12726962,E=12726962,C=34)
	CT(56036)=(S=12726963,E=12726963,C=37)
	CT(56037)=(S=12726965,E=12726965,C=19)
	CT(56038)=(S=12726966,E=12726966,C=28)
	CT(56039)=(S=12726967,E=12726967,C=16)
	CT(56040)=(S=12726970,E=12726970,C=58)
	CT(56041)=(S=12726971,E=12726971,C=28)
	CT(56042)=(S=12726973,E=12726973,C=33)
	CT(56043)=(S=12726975,E=12726975,C=28)
	CT(56044)=(S=12726986,E=12726989,C=33)
	CT(56045)=(S=12726992,E=12726992,C=24)
	CT(56046)=(S=12726994,E=12726995,C=33)
	CT(56047)=(S=12726998,E=12726998,C=58)
	CT(56048)=(S=12726999,E=12727024,C=33)
	CT(56049)=(S=12727025,E=12727025,C=31)
	CT(56050)=(S=12727026,E=12727033,C=33)
	CT(56051)=(S=12727036,E=12727036,C=58)
	CT(56052)=(S=12727038,E=12727038,C=31)
	CT(56053)=(S=12727039,E=12727039,C=18)
	CT(56054)=(S=12727040,E=12727295,C=12)
	CT(56055)=(S=12727296,E=12727551,C=19)
	CT(56056)=(S=12727552,E=12727627,C=36)
	CT(56057)=(S=12727628,E=12727631,C=12)
	CT(56058)=(S=12727632,E=12727723,C=36)
	CT(56059)=(S=12727724,E=12727724,C=49)
	CT(56060)=(S=12727725,E=12727726,C=36)
	CT(56061)=(S=12727727,E=12727727,C=14)
	CT(56062)=(S=12727760,E=12727807,C=36)
	CT(56063)=(S=12727808,E=12727815,C=28)
	CT(56064)=(S=12727816,E=12727819,C=14)
	CT(56065)=(S=12727820,E=12727823,C=18)
	CT(56066)=(S=12727824,E=12727835,C=58)
	CT(56067)=(S=12727836,E=12727839,C=19)
	CT(56068)=(S=12727840,E=12727871,C=37)
	CT(56069)=(S=12727872,E=12727875,C=18)
	CT(56070)=(S=12727876,E=12727879,C=27)
	CT(56071)=(S=12727880,E=12727883,C=28)
	CT(56072)=(S=12727888,E=12727891,C=31)
	CT(56073)=(S=12727892,E=12727895,C=58)
	CT(56074)=(S=12727896,E=12727899,C=31)
	CT(56075)=(S=12727900,E=12727903,C=28)
	CT(56076)=(S=12727904,E=12727935,C=27)
	CT(56077)=(S=12727936,E=12727939,C=38)
	CT(56078)=(S=12727940,E=12727943,C=18)
	CT(56079)=(S=12727948,E=12727951,C=51)
	CT(56080)=(S=12727952,E=12727955,C=36)
	CT(56081)=(S=12727956,E=12727959,C=18)
	CT(56082)=(S=12727960,E=12727971,C=31)
	CT(56083)=(S=12727972,E=12727975,C=19)
	CT(56084)=(S=12727976,E=12727979,C=17)
	CT(56085)=(S=12727980,E=12727983,C=64)
	CT(56086)=(S=12727984,E=12727987,C=18)
	CT(56087)=(S=12727988,E=12727991,C=46)
	CT(56088)=(S=12727992,E=12727995,C=31)
	CT(56089)=(S=12727996,E=12727999,C=58)
	CT(56090)=(S=12728000,E=12728063,C=101)
	CT(56091)=(S=12728065,E=12728226,C=28)
	CT(56092)=(S=12728233,E=12728233,C=38)
	CT(56093)=(S=12728234,E=12728318,C=28)
	CT(56094)=(S=12728320,E=12728530,C=34)
	CT(56095)=(S=12728531,E=12728531,C=58)
	CT(56096)=(S=12728534,E=12728534,C=28)
	CT(56097)=(S=12728535,E=12728535,C=17)
	CT(56098)=(S=12728538,E=12728540,C=34)
	CT(56099)=(S=12728541,E=12728541,C=28)
	CT(56100)=(S=12728548,E=12728574,C=34)
	CT(56101)=(S=12728576,E=12728831,C=12)
	CT(56102)=(S=12728832,E=12729087,C=18)
	CT(56103)=(S=12729090,E=12729239,C=28)
	CT(56104)=(S=12729240,E=12729242,C=34)
	CT(56105)=(S=12729244,E=12729263,C=28)
	CT(56106)=(S=12729265,E=12729266,C=42)
	CT(56107)=(S=12729267,E=12729267,C=28)
	CT(56108)=(S=12729268,E=12729268,C=12)
	CT(56109)=(S=12729270,E=12729270,C=52)
	CT(56110)=(S=12729271,E=12729271,C=28)
	CT(56111)=(S=12729272,E=12729272,C=18)
	CT(56112)=(S=12729273,E=12729273,C=52)
	CT(56113)=(S=12729274,E=12729274,C=18)
	CT(56114)=(S=12729276,E=12729276,C=14)
	CT(56115)=(S=12729277,E=12729335,C=28)
	CT(56116)=(S=12729344,E=12729407,C=14)
	CT(56117)=(S=12729408,E=12729411,C=12)
	CT(56118)=(S=12729412,E=12729412,C=28)
	CT(56119)=(S=12729413,E=12729413,C=31)
	CT(56120)=(S=12729414,E=12729414,C=34)
	CT(56121)=(S=12729415,E=12729415,C=36)
	CT(56122)=(S=12729416,E=12729416,C=38)
	CT(56123)=(S=12729417,E=12729417,C=37)
	CT(56124)=(S=12729418,E=12729418,C=52)
	CT(56125)=(S=12729419,E=12729419,C=27)
	CT(56126)=(S=12729421,E=12729421,C=31)
	CT(56127)=(S=12729422,E=12729422,C=28)
	CT(56128)=(S=12729423,E=12729423,C=58)
	CT(56129)=(S=12729424,E=12729424,C=19)
	CT(56130)=(S=12729425,E=12729425,C=34)
	CT(56131)=(S=12729426,E=12729426,C=38)
	CT(56132)=(S=12729427,E=12729547,C=14)
	CT(56133)=(S=12729548,E=12729548,C=28)
	CT(56134)=(S=12729550,E=12729550,C=49)
	CT(56135)=(S=12729551,E=12729551,C=36)
	CT(56136)=(S=12729560,E=12729560,C=14)
	CT(56137)=(S=12729561,E=12729561,C=34)
	CT(56138)=(S=12729562,E=12729569,C=14)
	CT(56139)=(S=12729570,E=12729571,C=58)
	CT(56140)=(S=12729572,E=12729575,C=26)
	CT(56141)=(S=12729576,E=12729579,C=20)
	CT(56142)=(S=12729580,E=12729583,C=18)
	CT(56143)=(S=12729584,E=12729585,C=12)
	CT(56144)=(S=12729586,E=12729586,C=31)
	CT(56145)=(S=12729587,E=12729587,C=24)
	CT(56146)=(S=12729588,E=12729593,C=18)
	CT(56147)=(S=12729594,E=12729594,C=40)
	CT(56148)=(S=12729595,E=12729595,C=58)
	CT(56149)=(S=12729596,E=12729599,C=31)
	CT(56150)=(S=12729648,E=12729651,C=33)
	CT(56151)=(S=12729652,E=12729656,C=18)
	CT(56152)=(S=12729657,E=12729657,C=58)
	CT(56153)=(S=12729658,E=12729658,C=54)
	CT(56154)=(S=12729659,E=12729659,C=36)
	CT(56155)=(S=12729660,E=12729663,C=18)
	CT(56156)=(S=12729664,E=12729665,C=36)
	CT(56157)=(S=12729666,E=12729666,C=58)
	CT(56158)=(S=12729667,E=12729667,C=18)
	CT(56159)=(S=12729668,E=12729671,C=58)
	CT(56160)=(S=12729678,E=12729695,C=14)
	CT(56161)=(S=12729696,E=12729711,C=70)
	CT(56162)=(S=12729722,E=12729751,C=14)
	CT(56163)=(S=12729758,E=12729758,C=19)
	CT(56164)=(S=12729760,E=12729855,C=14)
	CT(56165)=(S=12729856,E=12729856,C=18)
	CT(56166)=(S=12729863,E=12729877,C=14)
	CT(56167)=(S=12729878,E=12729878,C=38)
	CT(56168)=(S=12729879,E=12729879,C=28)
	CT(56169)=(S=12729880,E=12729907,C=14)
	CT(56170)=(S=12729920,E=12729920,C=68)
	CT(56171)=(S=12729921,E=12730088,C=14)
	CT(56172)=(S=12730090,E=12730090,C=12)
	CT(56173)=(S=12730091,E=12730091,C=14)
	CT(56174)=(S=12730092,E=12730092,C=28)
	CT(56175)=(S=12730093,E=12730093,C=42)
	CT(56176)=(S=12730094,E=12730239,C=14)
	CT(56177)=(S=12730240,E=12730247,C=58)
	CT(56178)=(S=12730248,E=12730251,C=51)
	CT(56179)=(S=12730252,E=12730255,C=18)
	CT(56180)=(S=12730260,E=12730263,C=34)
	CT(56181)=(S=12730268,E=12730271,C=18)
	CT(56182)=(S=12730272,E=12730359,C=22)
	CT(56183)=(S=12730360,E=12730367,C=27)
	CT(56184)=(S=12730368,E=12730623,C=28)
	CT(56185)=(S=12730624,E=12730879,C=21)
	CT(56186)=(S=12730880,E=12731135,C=14)
	CT(56187)=(S=12731136,E=12731391,C=18)
	CT(56188)=(S=12731392,E=12731647,C=19)
	CT(56189)=(S=12731648,E=12731648,C=28)
	CT(56190)=(S=12731649,E=12731649,C=16)
	CT(56191)=(S=12731650,E=12731653,C=34)
	CT(56192)=(S=12731654,E=12731663,C=19)
	CT(56193)=(S=12731664,E=12731679,C=36)
	CT(56194)=(S=12731680,E=12731711,C=13)
	CT(56195)=(S=12731712,E=12731775,C=14)
	CT(56196)=(S=12731776,E=12731791,C=28)
	CT(56197)=(S=12731792,E=12731839,C=13)
	CT(56198)=(S=12731840,E=12731841,C=15)
	CT(56199)=(S=12731842,E=12731843,C=12)
	CT(56200)=(S=12731844,E=12731845,C=34)
	CT(56201)=(S=12731846,E=12731847,C=63)
	CT(56202)=(S=12731852,E=12731852,C=36)
	CT(56203)=(S=12731853,E=12731853,C=28)
	CT(56204)=(S=12731855,E=12731855,C=58)
	CT(56205)=(S=12731856,E=12731871,C=27)
	CT(56206)=(S=12731872,E=12731903,C=25)
	CT(56207)=(S=12731904,E=12732159,C=14)
	CT(56208)=(S=12732160,E=12732415,C=19)
	CT(56209)=(S=12732416,E=12733439,C=14)
	CT(56210)=(S=12733442,E=12733649,C=28)
	CT(56211)=(S=12733650,E=12733650,C=42)
	CT(56212)=(S=12733651,E=12733675,C=28)
	CT(56213)=(S=12733679,E=12733679,C=42)
	CT(56214)=(S=12733682,E=12733683,C=28)
	CT(56215)=(S=12733684,E=12733685,C=38)
	CT(56216)=(S=12733686,E=12733686,C=27)
	CT(56217)=(S=12733687,E=12733687,C=63)
	CT(56218)=(S=12733693,E=12733951,C=28)
	CT(56219)=(S=12733952,E=12734207,C=49)
	CT(56220)=(S=12734208,E=12734211,C=28)
	CT(56221)=(S=12734212,E=12734215,C=18)
	CT(56222)=(S=12734216,E=12734219,C=31)
	CT(56223)=(S=12734220,E=12734223,C=51)
	CT(56224)=(S=12734224,E=12734227,C=47)
	CT(56225)=(S=12734228,E=12734231,C=31)
	CT(56226)=(S=12734232,E=12734235,C=58)
	CT(56227)=(S=12734236,E=12734239,C=14)
	CT(56228)=(S=12734240,E=12734243,C=18)
	CT(56229)=(S=12734248,E=12734251,C=46)
	CT(56230)=(S=12734252,E=12734255,C=18)
	CT(56231)=(S=12734260,E=12734263,C=33)
	CT(56232)=(S=12734264,E=12734267,C=15)
	CT(56233)=(S=12734268,E=12734271,C=31)
	CT(56234)=(S=12734272,E=12734303,C=21)
	CT(56235)=(S=12734304,E=12734335,C=106)
	CT(56236)=(S=12734336,E=12734399,C=12)
	CT(56237)=(S=12734400,E=12734431,C=15)
	CT(56238)=(S=12734432,E=12734447,C=36)
	CT(56239)=(S=12734448,E=12734451,C=14)
	CT(56240)=(S=12734452,E=12734457,C=28)
	CT(56241)=(S=12734458,E=12734459,C=18)
	CT(56242)=(S=12734460,E=12734461,C=22)
	CT(56243)=(S=12734462,E=12734463,C=36)
	CT(56244)=(S=12734464,E=12735487,C=14)
	CT(56245)=(S=12735488,E=12735999,C=18)
	CT(56246)=(S=12736000,E=12736255,C=47)
	CT(56247)=(S=12736256,E=12736511,C=18)
	CT(56248)=(S=12736512,E=12736512,C=28)
	CT(56249)=(S=12736513,E=12736513,C=31)
	CT(56250)=(S=12736514,E=12736514,C=36)
	CT(56251)=(S=12736515,E=12736515,C=58)
	CT(56252)=(S=12736516,E=12736516,C=63)
	CT(56253)=(S=12736517,E=12736517,C=24)
	CT(56254)=(S=12736520,E=12736520,C=42)
	CT(56255)=(S=12736521,E=12736521,C=34)
	CT(56256)=(S=12736522,E=12736522,C=28)
	CT(56257)=(S=12736526,E=12736527,C=18)
	CT(56258)=(S=12736528,E=12736538,C=28)
	CT(56259)=(S=12736539,E=12736539,C=14)
	CT(56260)=(S=12736540,E=12736543,C=19)
	CT(56261)=(S=12736544,E=12736575,C=50)
	CT(56262)=(S=12736576,E=12736607,C=14)
	CT(56263)=(S=12736608,E=12736639,C=49)
	CT(56264)=(S=12736640,E=12736641,C=58)
	CT(56265)=(S=12736642,E=12736643,C=50)
	CT(56266)=(S=12736644,E=12736645,C=14)
	CT(56267)=(S=12736646,E=12736647,C=38)
	CT(56268)=(S=12736648,E=12736649,C=17)
	CT(56269)=(S=12736650,E=12736653,C=18)
	CT(56270)=(S=12736654,E=12736655,C=21)
	CT(56271)=(S=12736656,E=12736657,C=27)
	CT(56272)=(S=12736658,E=12736661,C=38)
	CT(56273)=(S=12736662,E=12736665,C=31)
	CT(56274)=(S=12736666,E=12736667,C=58)
	CT(56275)=(S=12736668,E=12736669,C=12)
	CT(56276)=(S=12736670,E=12736671,C=58)
	CT(56277)=(S=12736672,E=12736703,C=28)
	CT(56278)=(S=12736704,E=12736705,C=34)
	CT(56279)=(S=12736706,E=12736707,C=71)
	CT(56280)=(S=12736708,E=12736709,C=34)
	CT(56281)=(S=12736710,E=12736711,C=28)
	CT(56282)=(S=12736712,E=12736713,C=18)
	CT(56283)=(S=12736714,E=12736715,C=38)
	CT(56284)=(S=12736716,E=12736717,C=18)
	CT(56285)=(S=12736718,E=12736719,C=31)
	CT(56286)=(S=12736720,E=12736723,C=18)
	CT(56287)=(S=12736724,E=12736725,C=34)
	CT(56288)=(S=12736726,E=12736727,C=58)
	CT(56289)=(S=12736728,E=12736729,C=24)
	CT(56290)=(S=12736730,E=12736733,C=31)
	CT(56291)=(S=12736734,E=12736735,C=12)
	CT(56292)=(S=12736736,E=12736737,C=58)
	CT(56293)=(S=12736740,E=12736741,C=51)
	CT(56294)=(S=12736742,E=12736743,C=36)
	CT(56295)=(S=12736744,E=12736747,C=19)
	CT(56296)=(S=12736748,E=12736749,C=36)
	CT(56297)=(S=12736750,E=12736751,C=38)
	CT(56298)=(S=12736752,E=12736753,C=63)
	CT(56299)=(S=12736754,E=12736755,C=14)
	CT(56300)=(S=12736756,E=12736757,C=58)
	CT(56301)=(S=12736758,E=12736759,C=12)
	CT(56302)=(S=12736760,E=12736761,C=38)
	CT(56303)=(S=12736762,E=12736763,C=51)
	CT(56304)=(S=12736766,E=12736767,C=19)
	CT(56305)=(S=12736768,E=12737023,C=47)
	CT(56306)=(S=12737024,E=12737279,C=17)
	CT(56307)=(S=12737280,E=12737535,C=15)
	CT(56308)=(S=12737536,E=12737791,C=58)
	CT(56309)=(S=12737865,E=12737867,C=19)
	CT(56310)=(S=12737868,E=12737871,C=42)
	CT(56311)=(S=12737872,E=12737883,C=19)
	CT(56312)=(S=12737884,E=12737887,C=25)
	CT(56313)=(S=12737888,E=12737890,C=13)
	CT(56314)=(S=12737891,E=12737891,C=18)
	CT(56315)=(S=12737892,E=12737892,C=38)
	CT(56316)=(S=12737893,E=12737894,C=36)
	CT(56317)=(S=12737895,E=12737903,C=28)
	CT(56318)=(S=12737904,E=12737914,C=34)
	CT(56319)=(S=12737915,E=12737915,C=28)
	CT(56320)=(S=12737916,E=12737919,C=58)
	CT(56321)=(S=12737920,E=12737951,C=14)
	CT(56322)=(S=12737952,E=12737983,C=31)
	CT(56323)=(S=12737984,E=12738559,C=28)
	CT(56324)=(S=12738560,E=12738815,C=42)
	CT(56325)=(S=12738816,E=12739071,C=28)
	CT(56326)=(S=12739072,E=12739327,C=12)
	CT(56327)=(S=12739328,E=12739441,C=28)
	CT(56328)=(S=12739444,E=12739444,C=18)
	CT(56329)=(S=12739445,E=12739445,C=58)
	CT(56330)=(S=12739446,E=12739583,C=28)
	CT(56331)=(S=12739584,E=12739839,C=47)
	CT(56332)=(S=12739840,E=12740127,C=14)
	CT(56333)=(S=12740128,E=12740351,C=38)
	CT(56334)=(S=12740352,E=12740607,C=19)
	CT(56335)=(S=12740608,E=12740863,C=36)
	CT(56336)=(S=12740864,E=12740895,C=38)
	CT(56337)=(S=12740896,E=12740911,C=72)
	CT(56338)=(S=12740912,E=12740919,C=15)
	CT(56339)=(S=12740920,E=12740920,C=60)
	CT(56340)=(S=12740921,E=12740925,C=15)
	CT(56341)=(S=12740928,E=12740959,C=14)
	CT(56342)=(S=12740960,E=12740967,C=28)
	CT(56343)=(S=12740968,E=12740975,C=38)
	CT(56344)=(S=12740976,E=12740983,C=14)
	CT(56345)=(S=12740984,E=12740991,C=13)
	CT(56346)=(S=12740992,E=12740993,C=36)
	CT(56347)=(S=12740994,E=12740995,C=18)
	CT(56348)=(S=12740996,E=12740997,C=58)
	CT(56349)=(S=12740998,E=12740999,C=14)
	CT(56350)=(S=12741000,E=12741001,C=31)
	CT(56351)=(S=12741002,E=12741003,C=36)
	CT(56352)=(S=12741004,E=12741007,C=38)
	CT(56353)=(S=12741008,E=12741009,C=31)
	CT(56354)=(S=12741010,E=12741011,C=28)
	CT(56355)=(S=12741012,E=12741012,C=18)
	CT(56356)=(S=12741013,E=12741013,C=14)
	CT(56357)=(S=12741014,E=12741015,C=36)
	CT(56358)=(S=12741016,E=12741017,C=12)
	CT(56359)=(S=12741018,E=12741019,C=14)
	CT(56360)=(S=12741020,E=12741021,C=18)
	CT(56361)=(S=12741024,E=12741055,C=14)
	CT(56362)=(S=12741056,E=12741087,C=18)
	CT(56363)=(S=12741088,E=12741119,C=48)
	CT(56364)=(S=12741120,E=12741151,C=25)
	CT(56365)=(S=12741152,E=12741183,C=14)
	CT(56366)=(S=12741184,E=12741215,C=72)
	CT(56367)=(S=12741216,E=12741247,C=55)
	CT(56368)=(S=12741248,E=12741279,C=63)
	CT(56369)=(S=12741280,E=12741311,C=46)
	CT(56370)=(S=12741312,E=12741313,C=58)
	CT(56371)=(S=12741314,E=12741317,C=18)
	CT(56372)=(S=12741318,E=12741318,C=26)
	CT(56373)=(S=12741319,E=12741319,C=14)
	CT(56374)=(S=12741320,E=12741321,C=58)
	CT(56375)=(S=12741324,E=12741325,C=38)
	CT(56376)=(S=12741326,E=12741327,C=28)
	CT(56377)=(S=12741328,E=12741329,C=31)
	CT(56378)=(S=12741330,E=12741331,C=58)
	CT(56379)=(S=12741332,E=12741333,C=38)
	CT(56380)=(S=12741334,E=12741335,C=58)
	CT(56381)=(S=12741336,E=12741339,C=31)
	CT(56382)=(S=12741340,E=12741343,C=14)
	CT(56383)=(S=12741344,E=12741391,C=42)
	CT(56384)=(S=12741392,E=12741392,C=14)
	CT(56385)=(S=12741393,E=12741393,C=18)
	CT(56386)=(S=12741394,E=12741394,C=58)
	CT(56387)=(S=12741395,E=12741395,C=36)
	CT(56388)=(S=12741396,E=12741396,C=42)
	CT(56389)=(S=12741397,E=12741397,C=18)
	CT(56390)=(S=12741398,E=12741398,C=37)
	CT(56391)=(S=12741399,E=12741399,C=18)
	CT(56392)=(S=12741400,E=12741445,C=42)
	CT(56393)=(S=12741456,E=12741456,C=18)
	CT(56394)=(S=12741457,E=12741470,C=42)
	CT(56395)=(S=12741471,E=12741471,C=28)
	CT(56396)=(S=12741472,E=12741487,C=42)
	CT(56397)=(S=12741488,E=12741488,C=38)
	CT(56398)=(S=12741489,E=12741489,C=42)
	CT(56399)=(S=12741490,E=12741490,C=36)
	CT(56400)=(S=12741491,E=12741491,C=31)
	CT(56401)=(S=12741492,E=12741492,C=18)
	CT(56402)=(S=12741493,E=12741493,C=31)
	CT(56403)=(S=12741494,E=12741494,C=14)
	CT(56404)=(S=12741495,E=12741495,C=19)
	CT(56405)=(S=12741496,E=12741496,C=58)
	CT(56406)=(S=12741497,E=12741497,C=36)
	CT(56407)=(S=12741498,E=12741498,C=38)
	CT(56408)=(S=12741499,E=12741499,C=49)
	CT(56409)=(S=12741500,E=12741500,C=18)
	CT(56410)=(S=12741502,E=12741502,C=12)
	CT(56411)=(S=12741503,E=12741503,C=21)
	CT(56412)=(S=12741506,E=12741623,C=42)
	CT(56413)=(S=12741632,E=12741887,C=33)
	CT(56414)=(S=12741888,E=12742143,C=36)
	CT(56415)=(S=12742144,E=12742144,C=19)
	CT(56416)=(S=12742150,E=12742150,C=31)
	CT(56417)=(S=12742162,E=12742162,C=18)
	CT(56418)=(S=12742167,E=12742168,C=47)
	CT(56419)=(S=12742169,E=12742169,C=28)
	CT(56420)=(S=12742170,E=12742170,C=58)
	CT(56421)=(S=12742175,E=12742207,C=47)
	CT(56422)=(S=12742208,E=12742209,C=58)
	CT(56423)=(S=12742210,E=12742210,C=18)
	CT(56424)=(S=12742211,E=12742211,C=36)
	CT(56425)=(S=12742212,E=12742212,C=38)
	CT(56426)=(S=12742213,E=12742213,C=12)
	CT(56427)=(S=12742214,E=12742214,C=19)
	CT(56428)=(S=12742215,E=12742215,C=15)
	CT(56429)=(S=12742217,E=12742217,C=22)
	CT(56430)=(S=12742219,E=12742219,C=14)
	CT(56431)=(S=12742220,E=12742220,C=21)
	CT(56432)=(S=12742221,E=12742221,C=58)
	CT(56433)=(S=12742222,E=12742222,C=42)
	CT(56434)=(S=12742223,E=12742223,C=31)
	CT(56435)=(S=12742240,E=12742255,C=19)
	CT(56436)=(S=12742260,E=12742260,C=58)
	CT(56437)=(S=12742270,E=12742270,C=31)
	CT(56438)=(S=12742271,E=12742271,C=58)
	CT(56439)=(S=12742272,E=12742272,C=34)
	CT(56440)=(S=12742273,E=12742273,C=31)
	CT(56441)=(S=12742277,E=12742277,C=28)
	CT(56442)=(S=12742280,E=12742280,C=24)
	CT(56443)=(S=12742286,E=12742286,C=47)
	CT(56444)=(S=12742294,E=12742294,C=37)
	CT(56445)=(S=12742295,E=12742295,C=49)
	CT(56446)=(S=12742296,E=12742297,C=18)
	CT(56447)=(S=12742298,E=12742298,C=12)
	CT(56448)=(S=12742299,E=12742299,C=16)
	CT(56449)=(S=12742300,E=12742303,C=28)
	CT(56450)=(S=12742304,E=12742307,C=31)
	CT(56451)=(S=12742308,E=12742311,C=12)
	CT(56452)=(S=12742320,E=12742335,C=47)
	CT(56453)=(S=12742336,E=12742336,C=12)
	CT(56454)=(S=12742338,E=12742338,C=14)
	CT(56455)=(S=12742339,E=12742339,C=27)
	CT(56456)=(S=12742340,E=12742340,C=28)
	CT(56457)=(S=12742341,E=12742341,C=14)
	CT(56458)=(S=12742342,E=12742342,C=22)
	CT(56459)=(S=12742343,E=12742343,C=52)
	CT(56460)=(S=12742345,E=12742345,C=25)
	CT(56461)=(S=12742346,E=12742346,C=18)
	CT(56462)=(S=12742347,E=12742348,C=14)
	CT(56463)=(S=12742349,E=12742349,C=51)
	CT(56464)=(S=12742350,E=12742350,C=24)
	CT(56465)=(S=12742351,E=12742352,C=12)
	CT(56466)=(S=12742353,E=12742353,C=14)
	CT(56467)=(S=12742355,E=12742355,C=36)
	CT(56468)=(S=12742356,E=12742356,C=38)
	CT(56469)=(S=12742357,E=12742357,C=37)
	CT(56470)=(S=12742358,E=12742358,C=36)
	CT(56471)=(S=12742359,E=12742359,C=14)
	CT(56472)=(S=12742360,E=12742360,C=46)
	CT(56473)=(S=12742361,E=12742361,C=28)
	CT(56474)=(S=12742362,E=12742362,C=22)
	CT(56475)=(S=12742363,E=12742363,C=31)
	CT(56476)=(S=12742364,E=12742364,C=44)
	CT(56477)=(S=12742365,E=12742365,C=36)
	CT(56478)=(S=12742366,E=12742366,C=52)
	CT(56479)=(S=12742367,E=12742367,C=18)
	CT(56480)=(S=12742368,E=12742379,C=47)
	CT(56481)=(S=12742380,E=12742380,C=61)
	CT(56482)=(S=12742381,E=12742381,C=14)
	CT(56483)=(S=12742382,E=12742384,C=58)
	CT(56484)=(S=12742385,E=12742385,C=14)
	CT(56485)=(S=12742386,E=12742386,C=34)
	CT(56486)=(S=12742387,E=12742388,C=14)
	CT(56487)=(S=12742389,E=12742389,C=12)
	CT(56488)=(S=12742393,E=12742393,C=17)
	CT(56489)=(S=12742394,E=12742394,C=14)
	CT(56490)=(S=12742395,E=12742395,C=18)
	CT(56491)=(S=12742396,E=12742396,C=31)
	CT(56492)=(S=12742397,E=12742399,C=18)
	CT(56493)=(S=12742400,E=12742671,C=47)
	CT(56494)=(S=12742672,E=12742687,C=28)
	CT(56495)=(S=12742688,E=12742719,C=14)
	CT(56496)=(S=12742720,E=12742783,C=13)
	CT(56497)=(S=12742784,E=12742911,C=42)
	CT(56498)=(S=12742912,E=12743061,C=28)
	CT(56499)=(S=12743066,E=12743066,C=42)
	CT(56500)=(S=12743072,E=12743075,C=28)
	CT(56501)=(S=12743077,E=12743077,C=38)
	CT(56502)=(S=12743080,E=12743080,C=28)
	CT(56503)=(S=12743083,E=12743083,C=36)
	CT(56504)=(S=12743085,E=12743295,C=28)
	CT(56505)=(S=12743296,E=12743299,C=18)
	CT(56506)=(S=12743300,E=12743303,C=31)
	CT(56507)=(S=12743308,E=12743311,C=19)
	CT(56508)=(S=12743312,E=12743312,C=53)
	CT(56509)=(S=12743313,E=12743313,C=12)
	CT(56510)=(S=12743314,E=12743315,C=17)
	CT(56511)=(S=12743316,E=12743319,C=58)
	CT(56512)=(S=12743320,E=12743423,C=19)
	CT(56513)=(S=12743424,E=12743679,C=28)
	CT(56514)=(S=12743680,E=12743807,C=15)
	CT(56515)=(S=12743808,E=12743809,C=28)
	CT(56516)=(S=12743810,E=12743815,C=58)
	CT(56517)=(S=12743816,E=12743817,C=38)
	CT(56518)=(S=12743818,E=12743819,C=58)
	CT(56519)=(S=12743820,E=12743821,C=18)
	CT(56520)=(S=12743822,E=12743825,C=12)
	CT(56521)=(S=12743826,E=12743827,C=36)
	CT(56522)=(S=12743828,E=12743829,C=12)
	CT(56523)=(S=12743830,E=12743831,C=34)
	CT(56524)=(S=12743832,E=12743833,C=25)
	CT(56525)=(S=12743836,E=12743837,C=19)
	CT(56526)=(S=12743838,E=12743839,C=42)
	CT(56527)=(S=12743840,E=12743841,C=14)
	CT(56528)=(S=12743842,E=12743843,C=31)
	CT(56529)=(S=12743844,E=12743845,C=15)
	CT(56530)=(S=12743846,E=12743846,C=58)
	CT(56531)=(S=12743847,E=12743847,C=14)
	CT(56532)=(S=12743848,E=12743848,C=22)
	CT(56533)=(S=12743849,E=12743849,C=18)
	CT(56534)=(S=12743850,E=12743851,C=31)
	CT(56535)=(S=12743854,E=12743855,C=14)
	CT(56536)=(S=12743856,E=12743857,C=12)
	CT(56537)=(S=12743858,E=12743859,C=24)
	CT(56538)=(S=12743860,E=12743861,C=34)
	CT(56539)=(S=12743862,E=12743863,C=58)
	CT(56540)=(S=12743864,E=12743865,C=25)
	CT(56541)=(S=12743866,E=12743867,C=28)
	CT(56542)=(S=12743868,E=12743869,C=55)
	CT(56543)=(S=12743872,E=12743873,C=58)
	CT(56544)=(S=12743874,E=12743875,C=31)
	CT(56545)=(S=12743876,E=12743877,C=34)
	CT(56546)=(S=12743878,E=12743879,C=14)
	CT(56547)=(S=12743880,E=12743881,C=38)
	CT(56548)=(S=12743882,E=12743883,C=12)
	CT(56549)=(S=12743884,E=12743885,C=52)
	CT(56550)=(S=12743886,E=12743889,C=28)
	CT(56551)=(S=12743890,E=12743891,C=34)
	CT(56552)=(S=12743892,E=12743893,C=19)
	CT(56553)=(S=12743898,E=12743899,C=12)
	CT(56554)=(S=12743900,E=12743901,C=36)
	CT(56555)=(S=12743902,E=12743903,C=24)
	CT(56556)=(S=12743904,E=12743905,C=42)
	CT(56557)=(S=12743906,E=12743911,C=17)
	CT(56558)=(S=12743912,E=12743913,C=31)
	CT(56559)=(S=12743914,E=12743915,C=28)
	CT(56560)=(S=12743918,E=12743919,C=31)
	CT(56561)=(S=12743920,E=12743921,C=25)
	CT(56562)=(S=12743922,E=12743923,C=42)
	CT(56563)=(S=12743924,E=12743925,C=31)
	CT(56564)=(S=12743926,E=12743927,C=38)
	CT(56565)=(S=12743932,E=12743933,C=58)
	CT(56566)=(S=12743934,E=12743935,C=14)
	CT(56567)=(S=12743936,E=12743985,C=21)
	CT(56568)=(S=12743986,E=12743986,C=18)
	CT(56569)=(S=12743987,E=12743987,C=16)
	CT(56570)=(S=12743988,E=12743988,C=24)
	CT(56571)=(S=12743989,E=12743989,C=27)
	CT(56572)=(S=12743990,E=12743990,C=14)
	CT(56573)=(S=12743991,E=12743991,C=38)
	CT(56574)=(S=12743992,E=12743999,C=35)
	CT(56575)=(S=12744000,E=12744031,C=18)
	CT(56576)=(S=12744032,E=12744063,C=28)
	CT(56577)=(S=12744064,E=12744095,C=14)
	CT(56578)=(S=12744096,E=12744127,C=19)
	CT(56579)=(S=12744128,E=12744159,C=12)
	CT(56580)=(S=12744162,E=12744163,C=14)
	CT(56581)=(S=12744164,E=12744165,C=58)
	CT(56582)=(S=12744166,E=12744167,C=38)
	CT(56583)=(S=12744168,E=12744169,C=24)
	CT(56584)=(S=12744170,E=12744171,C=95)
	CT(56585)=(S=12744172,E=12744173,C=38)
	CT(56586)=(S=12744176,E=12744177,C=58)
	CT(56587)=(S=12744178,E=12744179,C=38)
	CT(56588)=(S=12744180,E=12744183,C=12)
	CT(56589)=(S=12744184,E=12744185,C=14)
	CT(56590)=(S=12744188,E=12744189,C=31)
	CT(56591)=(S=12744190,E=12744191,C=28)
	CT(56592)=(S=12744192,E=12744447,C=42)
	CT(56593)=(S=12744448,E=12744575,C=12)
	CT(56594)=(S=12744576,E=12744639,C=14)
	CT(56595)=(S=12744640,E=12744671,C=15)
	CT(56596)=(S=12744672,E=12744703,C=49)
	CT(56597)=(S=12744704,E=12745727,C=28)
	CT(56598)=(S=12745728,E=12745954,C=34)
	CT(56599)=(S=12745957,E=12745957,C=14)
	CT(56600)=(S=12745958,E=12745982,C=34)
	CT(56601)=(S=12745984,E=12746207,C=63)
	CT(56602)=(S=12746208,E=12746211,C=31)
	CT(56603)=(S=12746216,E=12746219,C=37)
	CT(56604)=(S=12746220,E=12746223,C=18)
	CT(56605)=(S=12746224,E=12746225,C=60)
	CT(56606)=(S=12746226,E=12746227,C=15)
	CT(56607)=(S=12746228,E=12746229,C=31)
	CT(56608)=(S=12746230,E=12746231,C=14)
	CT(56609)=(S=12746236,E=12746237,C=22)
	CT(56610)=(S=12746238,E=12746239,C=18)
	CT(56611)=(S=12746240,E=12746271,C=39)
	CT(56612)=(S=12746272,E=12746303,C=101)
	CT(56613)=(S=12746304,E=12746335,C=14)
	CT(56614)=(S=12746336,E=12746367,C=55)
	CT(56615)=(S=12746368,E=12746369,C=12)
	CT(56616)=(S=12746370,E=12746370,C=36)
	CT(56617)=(S=12746376,E=12746383,C=39)
	CT(56618)=(S=12746384,E=12746385,C=34)
	CT(56619)=(S=12746386,E=12746387,C=38)
	CT(56620)=(S=12746388,E=12746389,C=15)
	CT(56621)=(S=12746390,E=12746391,C=19)
	CT(56622)=(S=12746396,E=12746397,C=12)
	CT(56623)=(S=12746398,E=12746399,C=28)
	CT(56624)=(S=12746404,E=12746407,C=58)
	CT(56625)=(S=12746408,E=12746411,C=18)
	CT(56626)=(S=12746412,E=12746415,C=36)
	CT(56627)=(S=12746420,E=12746423,C=31)
	CT(56628)=(S=12746424,E=12746427,C=38)
	CT(56629)=(S=12746428,E=12746431,C=34)
	CT(56630)=(S=12746432,E=12746432,C=24)
	CT(56631)=(S=12746433,E=12746433,C=36)
	CT(56632)=(S=12746435,E=12746436,C=28)
	CT(56633)=(S=12746437,E=12746437,C=52)
	CT(56634)=(S=12746438,E=12746438,C=28)
	CT(56635)=(S=12746439,E=12746439,C=60)
	CT(56636)=(S=12746440,E=12746441,C=34)
	CT(56637)=(S=12746443,E=12746443,C=18)
	CT(56638)=(S=12746444,E=12746444,C=31)
	CT(56639)=(S=12746445,E=12746445,C=38)
	CT(56640)=(S=12746446,E=12746446,C=28)
	CT(56641)=(S=12746447,E=12746447,C=58)
	CT(56642)=(S=12746448,E=12746448,C=28)
	CT(56643)=(S=12746449,E=12746449,C=14)
	CT(56644)=(S=12746450,E=12746450,C=58)
	CT(56645)=(S=12746451,E=12746451,C=27)
	CT(56646)=(S=12746452,E=12746452,C=47)
	CT(56647)=(S=12746453,E=12746454,C=57)
	CT(56648)=(S=12746456,E=12746456,C=58)
	CT(56649)=(S=12746457,E=12746457,C=12)
	CT(56650)=(S=12746458,E=12746458,C=34)
	CT(56651)=(S=12746460,E=12746460,C=38)
	CT(56652)=(S=12746461,E=12746462,C=58)
	CT(56653)=(S=12746463,E=12746463,C=24)
	CT(56654)=(S=12746464,E=12746464,C=31)
	CT(56655)=(S=12746465,E=12746465,C=25)
	CT(56656)=(S=12746466,E=12746466,C=12)
	CT(56657)=(S=12746468,E=12746468,C=28)
	CT(56658)=(S=12746469,E=12746469,C=58)
	CT(56659)=(S=12746470,E=12746470,C=37)
	CT(56660)=(S=12746471,E=12746471,C=23)
	CT(56661)=(S=12746472,E=12746472,C=58)
	CT(56662)=(S=12746474,E=12746474,C=24)
	CT(56663)=(S=12746475,E=12746475,C=36)
	CT(56664)=(S=12746476,E=12746476,C=12)
	CT(56665)=(S=12746477,E=12746477,C=28)
	CT(56666)=(S=12746478,E=12746478,C=58)
	CT(56667)=(S=12746479,E=12746479,C=28)
	CT(56668)=(S=12746480,E=12746480,C=14)
	CT(56669)=(S=12746481,E=12746481,C=36)
	CT(56670)=(S=12746482,E=12746483,C=28)
	CT(56671)=(S=12746484,E=12746484,C=38)
	CT(56672)=(S=12746486,E=12746486,C=28)
	CT(56673)=(S=12746487,E=12746487,C=14)
	CT(56674)=(S=12746488,E=12746488,C=42)
	CT(56675)=(S=12746489,E=12746489,C=19)
	CT(56676)=(S=12746490,E=12746490,C=24)
	CT(56677)=(S=12746492,E=12746492,C=44)
	CT(56678)=(S=12746493,E=12746493,C=14)
	CT(56679)=(S=12746494,E=12746494,C=58)
	CT(56680)=(S=12746495,E=12746495,C=22)
	CT(56681)=(S=12746496,E=12746630,C=28)
	CT(56682)=(S=12746632,E=12746633,C=58)
	CT(56683)=(S=12746634,E=12746634,C=28)
	CT(56684)=(S=12746635,E=12746635,C=42)
	CT(56685)=(S=12746636,E=12746750,C=28)
	CT(56686)=(S=12746752,E=12747775,C=14)
	CT(56687)=(S=12747776,E=12748031,C=19)
	CT(56688)=(S=12748032,E=12748287,C=13)
	CT(56689)=(S=12748288,E=12748543,C=36)
	CT(56690)=(S=12748544,E=12748623,C=18)
	CT(56691)=(S=12748624,E=12748639,C=44)
	CT(56692)=(S=12748640,E=12748695,C=18)
	CT(56693)=(S=12748696,E=12748735,C=41)
	CT(56694)=(S=12748736,E=12748743,C=18)
	CT(56695)=(S=12748744,E=12748751,C=28)
	CT(56696)=(S=12748752,E=12748799,C=18)
	CT(56697)=(S=12748800,E=12749311,C=47)
	CT(56698)=(S=12749312,E=12749823,C=28)
	CT(56699)=(S=12749824,E=12749919,C=25)
	CT(56700)=(S=12749920,E=12749951,C=28)
	CT(56701)=(S=12749952,E=12750015,C=25)
	CT(56702)=(S=12750016,E=12750047,C=50)
	CT(56703)=(S=12750049,E=12750049,C=12)
	CT(56704)=(S=12750050,E=12750050,C=14)
	CT(56705)=(S=12750051,E=12750051,C=37)
	CT(56706)=(S=12750052,E=12750052,C=31)
	CT(56707)=(S=12750054,E=12750054,C=36)
	CT(56708)=(S=12750055,E=12750055,C=28)
	CT(56709)=(S=12750056,E=12750056,C=21)
	CT(56710)=(S=12750057,E=12750057,C=58)
	CT(56711)=(S=12750058,E=12750058,C=63)
	CT(56712)=(S=12750059,E=12750059,C=38)
	CT(56713)=(S=12750060,E=12750060,C=24)
	CT(56714)=(S=12750062,E=12750062,C=21)
	CT(56715)=(S=12750063,E=12750063,C=31)
	CT(56716)=(S=12750064,E=12750064,C=19)
	CT(56717)=(S=12750065,E=12750065,C=58)
	CT(56718)=(S=12750067,E=12750068,C=19)
	CT(56719)=(S=12750069,E=12750069,C=38)
	CT(56720)=(S=12750070,E=12750070,C=36)
	CT(56721)=(S=12750071,E=12750071,C=12)
	CT(56722)=(S=12750072,E=12750072,C=50)
	CT(56723)=(S=12750074,E=12750074,C=58)
	CT(56724)=(S=12750075,E=12750077,C=14)
	CT(56725)=(S=12750078,E=12750078,C=42)
	CT(56726)=(S=12750079,E=12750079,C=58)
	CT(56727)=(S=12750080,E=12750335,C=51)
	CT(56728)=(S=12750336,E=12750591,C=47)
	CT(56729)=(S=12750592,E=12750719,C=27)
	CT(56730)=(S=12750720,E=12750721,C=53)
	CT(56731)=(S=12750722,E=12750723,C=36)
	CT(56732)=(S=12750724,E=12750725,C=12)
	CT(56733)=(S=12750726,E=12750727,C=28)
	CT(56734)=(S=12750728,E=12750729,C=31)
	CT(56735)=(S=12750730,E=12750731,C=18)
	CT(56736)=(S=12750732,E=12750733,C=26)
	CT(56737)=(S=12750734,E=12750735,C=15)
	CT(56738)=(S=12750736,E=12750739,C=31)
	CT(56739)=(S=12750740,E=12750743,C=18)
	CT(56740)=(S=12750744,E=12750745,C=60)
	CT(56741)=(S=12750746,E=12750747,C=14)
	CT(56742)=(S=12750748,E=12750749,C=28)
	CT(56743)=(S=12750750,E=12750783,C=14)
	CT(56744)=(S=12750784,E=12750815,C=25)
	CT(56745)=(S=12750816,E=12750847,C=50)
	CT(56746)=(S=12750848,E=12751103,C=48)
	CT(56747)=(S=12751104,E=12751135,C=14)
	CT(56748)=(S=12751136,E=12751156,C=47)
	CT(56749)=(S=12751157,E=12751162,C=12)
	CT(56750)=(S=12751163,E=12751166,C=21)
	CT(56751)=(S=12751167,E=12751167,C=51)
	CT(56752)=(S=12751168,E=12751199,C=28)
	CT(56753)=(S=12751200,E=12751215,C=58)
	CT(56754)=(S=12751218,E=12751218,C=28)
	CT(56755)=(S=12751219,E=12751219,C=34)
	CT(56756)=(S=12751221,E=12751221,C=31)
	CT(56757)=(S=12751222,E=12751222,C=28)
	CT(56758)=(S=12751223,E=12751223,C=36)
	CT(56759)=(S=12751224,E=12751224,C=38)
	CT(56760)=(S=12751225,E=12751225,C=21)
	CT(56761)=(S=12751226,E=12751228,C=28)
	CT(56762)=(S=12751230,E=12751230,C=14)
	CT(56763)=(S=12751232,E=12751239,C=63)
	CT(56764)=(S=12751244,E=12751245,C=25)
	CT(56765)=(S=12751248,E=12751248,C=12)
	CT(56766)=(S=12751249,E=12751249,C=19)
	CT(56767)=(S=12751250,E=12751250,C=28)
	CT(56768)=(S=12751251,E=12751253,C=14)
	CT(56769)=(S=12751254,E=12751255,C=28)
	CT(56770)=(S=12751257,E=12751257,C=46)
	CT(56771)=(S=12751258,E=12751258,C=12)
	CT(56772)=(S=12751260,E=12751260,C=50)
	CT(56773)=(S=12751261,E=12751261,C=34)
	CT(56774)=(S=12751262,E=12751262,C=18)
	CT(56775)=(S=12751263,E=12751263,C=38)
	CT(56776)=(S=12751264,E=12751267,C=51)
	CT(56777)=(S=12751268,E=12751271,C=19)
	CT(56778)=(S=12751272,E=12751275,C=18)
	CT(56779)=(S=12751276,E=12751279,C=19)
	CT(56780)=(S=12751280,E=12751283,C=42)
	CT(56781)=(S=12751284,E=12751287,C=33)
	CT(56782)=(S=12751288,E=12751291,C=58)
	CT(56783)=(S=12751292,E=12751295,C=14)
	CT(56784)=(S=12751296,E=12751297,C=28)
	CT(56785)=(S=12751298,E=12751299,C=36)
	CT(56786)=(S=12751300,E=12751301,C=14)
	CT(56787)=(S=12751302,E=12751303,C=31)
	CT(56788)=(S=12751304,E=12751305,C=36)
	CT(56789)=(S=12751306,E=12751307,C=31)
	CT(56790)=(S=12751308,E=12751309,C=58)
	CT(56791)=(S=12751310,E=12751311,C=71)
	CT(56792)=(S=12751312,E=12751313,C=170)
	CT(56793)=(S=12751318,E=12751321,C=31)
	CT(56794)=(S=12751322,E=12751323,C=28)
	CT(56795)=(S=12751324,E=12751325,C=31)
	CT(56796)=(S=12751326,E=12751327,C=14)
	CT(56797)=(S=12751328,E=12751328,C=28)
	CT(56798)=(S=12751329,E=12751329,C=58)
	CT(56799)=(S=12751330,E=12751330,C=28)
	CT(56800)=(S=12751331,E=12751331,C=31)
	CT(56801)=(S=12751332,E=12751333,C=58)
	CT(56802)=(S=12751334,E=12751334,C=28)
	CT(56803)=(S=12751335,E=12751335,C=15)
	CT(56804)=(S=12751336,E=12751338,C=28)
	CT(56805)=(S=12751339,E=12751339,C=12)
	CT(56806)=(S=12751340,E=12751340,C=28)
	CT(56807)=(S=12751341,E=12751341,C=14)
	CT(56808)=(S=12751342,E=12751342,C=38)
	CT(56809)=(S=12751343,E=12751343,C=28)
	CT(56810)=(S=12751344,E=12751347,C=14)
	CT(56811)=(S=12751348,E=12751351,C=36)
	CT(56812)=(S=12751352,E=12751355,C=19)
	CT(56813)=(S=12751356,E=12751359,C=28)
	CT(56814)=(S=12751376,E=12751387,C=12)
	CT(56815)=(S=12751397,E=12751397,C=28)
	CT(56816)=(S=12751406,E=12751441,C=12)
	CT(56817)=(S=12751454,E=12751455,C=13)
	CT(56818)=(S=12751456,E=12751458,C=12)
	CT(56819)=(S=12751465,E=12751468,C=19)
	CT(56820)=(S=12751469,E=12751469,C=57)
	CT(56821)=(S=12751470,E=12751470,C=31)
	CT(56822)=(S=12751471,E=12751471,C=34)
	CT(56823)=(S=12751472,E=12751472,C=31)
	CT(56824)=(S=12751473,E=12751475,C=18)
	CT(56825)=(S=12751477,E=12751477,C=19)
	CT(56826)=(S=12751478,E=12751478,C=28)
	CT(56827)=(S=12751479,E=12751479,C=18)
	CT(56828)=(S=12751480,E=12751480,C=58)
	CT(56829)=(S=12751481,E=12751481,C=49)
	CT(56830)=(S=12751482,E=12751482,C=18)
	CT(56831)=(S=12751483,E=12751483,C=58)
	CT(56832)=(S=12751485,E=12751485,C=38)
	CT(56833)=(S=12751487,E=12751487,C=36)
	CT(56834)=(S=12751488,E=12751491,C=58)
	CT(56835)=(S=12751492,E=12751503,C=31)
	CT(56836)=(S=12751504,E=12751507,C=28)
	CT(56837)=(S=12751508,E=12751511,C=26)
	CT(56838)=(S=12751512,E=12751515,C=39)
	CT(56839)=(S=12751520,E=12751523,C=18)
	CT(56840)=(S=12751528,E=12751531,C=19)
	CT(56841)=(S=12751532,E=12751535,C=12)
	CT(56842)=(S=12751536,E=12751539,C=28)
	CT(56843)=(S=12751540,E=12751543,C=31)
	CT(56844)=(S=12751544,E=12751547,C=42)
	CT(56845)=(S=12751548,E=12751559,C=31)
	CT(56846)=(S=12751560,E=12751563,C=18)
	CT(56847)=(S=12751568,E=12751571,C=28)
	CT(56848)=(S=12751572,E=12751575,C=34)
	CT(56849)=(S=12751576,E=12751579,C=58)
	CT(56850)=(S=12751584,E=12751587,C=12)
	CT(56851)=(S=12751588,E=12751591,C=31)
	CT(56852)=(S=12751592,E=12751595,C=51)
	CT(56853)=(S=12751596,E=12751599,C=16)
	CT(56854)=(S=12751600,E=12751603,C=18)
	CT(56855)=(S=12751604,E=12751607,C=27)
	CT(56856)=(S=12751608,E=12751615,C=58)
	CT(56857)=(S=12751634,E=12751682,C=34)
	CT(56858)=(S=12751684,E=12751685,C=56)
	CT(56859)=(S=12751686,E=12751720,C=34)
	CT(56860)=(S=12751724,E=12751724,C=58)
	CT(56861)=(S=12751728,E=12751748,C=34)
	CT(56862)=(S=12751749,E=12751749,C=28)
	CT(56863)=(S=12751750,E=12751785,C=34)
	CT(56864)=(S=12751787,E=12751787,C=42)
	CT(56865)=(S=12751789,E=12751811,C=34)
	CT(56866)=(S=12751812,E=12751823,C=56)
	CT(56867)=(S=12751824,E=12751861,C=34)
	CT(56868)=(S=12751862,E=12751863,C=18)
	CT(56869)=(S=12751864,E=12751864,C=58)
	CT(56870)=(S=12751865,E=12751865,C=18)
	CT(56871)=(S=12751870,E=12752127,C=34)
	CT(56872)=(S=12752128,E=12752191,C=50)
	CT(56873)=(S=12752192,E=12752199,C=18)
	CT(56874)=(S=12752200,E=12752203,C=25)
	CT(56875)=(S=12752204,E=12752207,C=63)
	CT(56876)=(S=12752208,E=12752215,C=36)
	CT(56877)=(S=12752216,E=12752216,C=58)
	CT(56878)=(S=12752218,E=12752218,C=18)
	CT(56879)=(S=12752220,E=12752220,C=31)
	CT(56880)=(S=12752222,E=12752222,C=12)
	CT(56881)=(S=12752223,E=12752223,C=18)
	CT(56882)=(S=12752224,E=12752255,C=33)
	CT(56883)=(S=12752256,E=12752287,C=54)
	CT(56884)=(S=12752288,E=12752319,C=12)
	CT(56885)=(S=12752320,E=12752351,C=25)
	CT(56886)=(S=12752352,E=12752355,C=18)
	CT(56887)=(S=12752356,E=12752359,C=58)
	CT(56888)=(S=12752360,E=12752361,C=15)
	CT(56889)=(S=12752362,E=12752363,C=18)
	CT(56890)=(S=12752364,E=12752365,C=22)
	CT(56891)=(S=12752366,E=12752367,C=24)
	CT(56892)=(S=12752368,E=12752368,C=58)
	CT(56893)=(S=12752369,E=12752369,C=28)
	CT(56894)=(S=12752370,E=12752370,C=18)
	CT(56895)=(S=12752371,E=12752371,C=49)
	CT(56896)=(S=12752372,E=12752372,C=15)
	CT(56897)=(S=12752373,E=12752383,C=28)
	CT(56898)=(S=12752384,E=12752447,C=14)
	CT(56899)=(S=12752448,E=12752451,C=55)
	CT(56900)=(S=12752456,E=12752459,C=31)
	CT(56901)=(S=12752464,E=12752467,C=28)
	CT(56902)=(S=12752468,E=12752471,C=38)
	CT(56903)=(S=12752472,E=12752475,C=18)
	CT(56904)=(S=12752480,E=12752487,C=58)
	CT(56905)=(S=12752488,E=12752491,C=18)
	CT(56906)=(S=12752492,E=12752499,C=24)
	CT(56907)=(S=12752500,E=12752503,C=51)
	CT(56908)=(S=12752504,E=12752507,C=14)
	CT(56909)=(S=12752512,E=12752543,C=18)
	CT(56910)=(S=12752544,E=12752545,C=34)
	CT(56911)=(S=12752546,E=12752547,C=15)
	CT(56912)=(S=12752548,E=12752549,C=18)
	CT(56913)=(S=12752550,E=12752551,C=58)
	CT(56914)=(S=12752552,E=12752553,C=28)
	CT(56915)=(S=12752554,E=12752555,C=58)
	CT(56916)=(S=12752556,E=12752557,C=28)
	CT(56917)=(S=12752558,E=12752559,C=31)
	CT(56918)=(S=12752560,E=12752561,C=14)
	CT(56919)=(S=12752562,E=12752563,C=18)
	CT(56920)=(S=12752566,E=12752567,C=14)
	CT(56921)=(S=12752568,E=12752569,C=58)
	CT(56922)=(S=12752570,E=12752571,C=38)
	CT(56923)=(S=12752572,E=12752575,C=28)
	CT(56924)=(S=12752576,E=12752577,C=31)
	CT(56925)=(S=12752578,E=12752579,C=14)
	CT(56926)=(S=12752580,E=12752581,C=58)
	CT(56927)=(S=12752582,E=12752583,C=34)
	CT(56928)=(S=12752584,E=12752585,C=14)
	CT(56929)=(S=12752586,E=12752587,C=18)
	CT(56930)=(S=12752590,E=12752591,C=58)
	CT(56931)=(S=12752592,E=12752593,C=36)
	CT(56932)=(S=12752594,E=12752595,C=28)
	CT(56933)=(S=12752596,E=12752597,C=27)
	CT(56934)=(S=12752600,E=12752601,C=38)
	CT(56935)=(S=12752602,E=12752603,C=17)
	CT(56936)=(S=12752604,E=12752605,C=31)
	CT(56937)=(S=12752606,E=12752607,C=18)
	CT(56938)=(S=12752608,E=12752609,C=49)
	CT(56939)=(S=12752610,E=12752613,C=28)
	CT(56940)=(S=12752614,E=12752615,C=58)
	CT(56941)=(S=12752616,E=12752617,C=31)
	CT(56942)=(S=12752620,E=12752621,C=12)
	CT(56943)=(S=12752622,E=12752623,C=58)
	CT(56944)=(S=12752624,E=12752625,C=28)
	CT(56945)=(S=12752626,E=12752627,C=38)
	CT(56946)=(S=12752628,E=12752629,C=34)
	CT(56947)=(S=12752630,E=12752631,C=58)
	CT(56948)=(S=12752632,E=12752633,C=34)
	CT(56949)=(S=12752634,E=12752635,C=58)
	CT(56950)=(S=12752636,E=12752637,C=14)
	CT(56951)=(S=12752638,E=12752639,C=18)
	CT(56952)=(S=12752640,E=12752895,C=36)
	CT(56953)=(S=12752896,E=12752927,C=52)
	CT(56954)=(S=12752928,E=12752929,C=60)
	CT(56955)=(S=12752930,E=12752931,C=18)
	CT(56956)=(S=12752932,E=12752933,C=62)
	CT(56957)=(S=12752934,E=12752935,C=24)
	CT(56958)=(S=12752936,E=12752937,C=14)
	CT(56959)=(S=12752938,E=12752939,C=38)
	CT(56960)=(S=12752940,E=12752941,C=16)
	CT(56961)=(S=12752942,E=12752943,C=58)
	CT(56962)=(S=12752944,E=12752945,C=15)
	CT(56963)=(S=12752946,E=12752947,C=58)
	CT(56964)=(S=12752948,E=12752948,C=70)
	CT(56965)=(S=12752949,E=12752949,C=58)
	CT(56966)=(S=12752950,E=12752951,C=28)
	CT(56967)=(S=12752952,E=12752957,C=58)
	CT(56968)=(S=12752958,E=12752959,C=18)
	CT(56969)=(S=12752960,E=12752991,C=14)
	CT(56970)=(S=12752992,E=12753023,C=42)
	CT(56971)=(S=12753024,E=12753055,C=50)
	CT(56972)=(S=12753056,E=12753087,C=42)
	CT(56973)=(S=12753088,E=12753151,C=57)
	CT(56974)=(S=12753152,E=12753183,C=14)
	CT(56975)=(S=12753184,E=12753223,C=19)
	CT(56976)=(S=12753226,E=12753226,C=36)
	CT(56977)=(S=12753227,E=12753227,C=14)
	CT(56978)=(S=12753230,E=12753230,C=36)
	CT(56979)=(S=12753231,E=12753231,C=60)
	CT(56980)=(S=12753233,E=12753234,C=22)
	CT(56981)=(S=12753235,E=12753235,C=34)
	CT(56982)=(S=12753236,E=12753236,C=19)
	CT(56983)=(S=12753237,E=12753237,C=72)
	CT(56984)=(S=12753238,E=12753239,C=28)
	CT(56985)=(S=12753240,E=12753242,C=12)
	CT(56986)=(S=12753243,E=12753243,C=19)
	CT(56987)=(S=12753244,E=12753244,C=12)
	CT(56988)=(S=12753245,E=12753245,C=42)
	CT(56989)=(S=12753246,E=12753246,C=34)
	CT(56990)=(S=12753247,E=12753247,C=58)
	CT(56991)=(S=12753248,E=12753248,C=34)
	CT(56992)=(S=12753249,E=12753249,C=12)
	CT(56993)=(S=12753250,E=12753250,C=63)
	CT(56994)=(S=12753252,E=12753252,C=28)
	CT(56995)=(S=12753253,E=12753253,C=17)
	CT(56996)=(S=12753256,E=12753257,C=28)
	CT(56997)=(S=12753258,E=12753258,C=14)
	CT(56998)=(S=12753259,E=12753259,C=58)
	CT(56999)=(S=12753260,E=12753262,C=12)
	CT(57000)=(S=12753263,E=12753263,C=42)
	CT(57001)=(S=12753264,E=12753264,C=18)
	CT(57002)=(S=12753265,E=12753266,C=28)
	CT(57003)=(S=12753267,E=12753267,C=58)
	CT(57004)=(S=12753268,E=12753268,C=28)
	CT(57005)=(S=12753269,E=12753269,C=36)
	CT(57006)=(S=12753270,E=12753270,C=51)
	CT(57007)=(S=12753271,E=12753271,C=58)
	CT(57008)=(S=12753272,E=12753279,C=42)
	CT(57009)=(S=12753280,E=12753281,C=31)
	CT(57010)=(S=12753282,E=12753282,C=28)
	CT(57011)=(S=12753283,E=12753283,C=15)
	CT(57012)=(S=12753284,E=12753284,C=21)
	CT(57013)=(S=12753286,E=12753286,C=58)
	CT(57014)=(S=12753287,E=12753287,C=60)
	CT(57015)=(S=12753288,E=12753296,C=14)
	CT(57016)=(S=12753297,E=12753297,C=51)
	CT(57017)=(S=12753298,E=12753299,C=28)
	CT(57018)=(S=12753300,E=12753301,C=31)
	CT(57019)=(S=12753302,E=12753303,C=28)
	CT(57020)=(S=12753304,E=12753304,C=14)
	CT(57021)=(S=12753305,E=12753305,C=16)
	CT(57022)=(S=12753306,E=12753306,C=27)
	CT(57023)=(S=12753308,E=12753308,C=32)
	CT(57024)=(S=12753309,E=12753309,C=32)
	CT(57025)=(S=12753310,E=12753310,C=18)
	CT(57026)=(S=12753311,E=12753311,C=47)
	CT(57027)=(S=12753312,E=12753319,C=28)
	CT(57028)=(S=12753320,E=12753321,C=14)
	CT(57029)=(S=12753323,E=12753323,C=60)
	CT(57030)=(S=12753324,E=12753327,C=15)
	CT(57031)=(S=12753328,E=12753335,C=34)
	CT(57032)=(S=12753336,E=12753336,C=14)
	CT(57033)=(S=12753338,E=12753338,C=28)
	CT(57034)=(S=12753339,E=12753339,C=15)
	CT(57035)=(S=12753340,E=12753341,C=34)
	CT(57036)=(S=12753342,E=12753343,C=28)
	CT(57037)=(S=12753344,E=12753365,C=15)
	CT(57038)=(S=12753366,E=12753367,C=12)
	CT(57039)=(S=12753368,E=12753368,C=58)
	CT(57040)=(S=12753369,E=12753369,C=42)
	CT(57041)=(S=12753370,E=12753370,C=14)
	CT(57042)=(S=12753371,E=12753371,C=28)
	CT(57043)=(S=12753372,E=12753375,C=42)
	CT(57044)=(S=12753376,E=12753407,C=38)
	CT(57045)=(S=12753408,E=12753471,C=14)
	CT(57046)=(S=12753472,E=12753503,C=18)
	CT(57047)=(S=12753504,E=12753535,C=14)
	CT(57048)=(S=12753536,E=12753567,C=32)
	CT(57049)=(S=12753568,E=12753599,C=14)
	CT(57050)=(S=12753600,E=12753631,C=70)
	CT(57051)=(S=12753632,E=12753663,C=71)
	CT(57052)=(S=12753664,E=12753919,C=14)
	CT(57053)=(S=12753923,E=12753927,C=28)
	CT(57054)=(S=12753944,E=12753944,C=12)
	CT(57055)=(S=12753945,E=12754081,C=28)
	CT(57056)=(S=12754085,E=12754085,C=37)
	CT(57057)=(S=12754088,E=12754117,C=28)
	CT(57058)=(S=12754122,E=12754123,C=12)
	CT(57059)=(S=12754126,E=12754167,C=28)
	CT(57060)=(S=12754176,E=12754431,C=47)
	CT(57061)=(S=12754432,E=12754463,C=34)
	CT(57062)=(S=12754464,E=12754495,C=72)
	CT(57063)=(S=12754496,E=12754527,C=188)
	CT(57064)=(S=12754528,E=12754559,C=12)
	CT(57065)=(S=12754560,E=12754591,C=42)
	CT(57066)=(S=12754592,E=12754623,C=36)
	CT(57067)=(S=12754624,E=12754655,C=68)
	CT(57068)=(S=12754656,E=12754687,C=34)
	CT(57069)=(S=12754688,E=12754943,C=14)
	CT(57070)=(S=12754944,E=12755199,C=71)
	CT(57071)=(S=12755200,E=12755455,C=13)
	CT(57072)=(S=12755456,E=12755967,C=28)
	CT(57073)=(S=12755968,E=12756223,C=14)
	CT(57074)=(S=12756224,E=12756227,C=18)
	CT(57075)=(S=12756230,E=12756231,C=31)
	CT(57076)=(S=12756232,E=12756233,C=58)
	CT(57077)=(S=12756236,E=12756237,C=14)
	CT(57078)=(S=12756238,E=12756239,C=12)
	CT(57079)=(S=12756240,E=12756249,C=18)
	CT(57080)=(S=12756252,E=12756253,C=14)
	CT(57081)=(S=12756254,E=12756255,C=18)
	CT(57082)=(S=12756256,E=12756256,C=19)
	CT(57083)=(S=12756257,E=12756257,C=34)
	CT(57084)=(S=12756258,E=12756258,C=36)
	CT(57085)=(S=12756260,E=12756260,C=15)
	CT(57086)=(S=12756261,E=12756261,C=63)
	CT(57087)=(S=12756263,E=12756263,C=53)
	CT(57088)=(S=12756264,E=12756264,C=28)
	CT(57089)=(S=12756265,E=12756265,C=58)
	CT(57090)=(S=12756266,E=12756266,C=14)
	CT(57091)=(S=12756267,E=12756267,C=12)
	CT(57092)=(S=12756268,E=12756268,C=34)
	CT(57093)=(S=12756269,E=12756269,C=18)
	CT(57094)=(S=12756270,E=12756270,C=31)
	CT(57095)=(S=12756271,E=12756271,C=33)
	CT(57096)=(S=12756272,E=12756272,C=58)
	CT(57097)=(S=12756273,E=12756273,C=12)
	CT(57098)=(S=12756274,E=12756274,C=18)
	CT(57099)=(S=12756275,E=12756275,C=36)
	CT(57100)=(S=12756276,E=12756276,C=42)
	CT(57101)=(S=12756277,E=12756277,C=18)
	CT(57102)=(S=12756278,E=12756278,C=58)
	CT(57103)=(S=12756279,E=12756279,C=24)
	CT(57104)=(S=12756281,E=12756281,C=33)
	CT(57105)=(S=12756283,E=12756283,C=19)
	CT(57106)=(S=12756284,E=12756284,C=14)
	CT(57107)=(S=12756285,E=12756285,C=18)
	CT(57108)=(S=12756286,E=12756287,C=31)
	CT(57109)=(S=12756288,E=12756319,C=36)
	CT(57110)=(S=12756320,E=12756351,C=52)
	CT(57111)=(S=12756352,E=12756383,C=61)
	CT(57112)=(S=12756384,E=12756415,C=63)
	CT(57113)=(S=12756416,E=12756447,C=14)
	CT(57114)=(S=12756448,E=12756479,C=19)
	CT(57115)=(S=12756480,E=12756735,C=42)
	CT(57116)=(S=12756736,E=12756991,C=12)
	CT(57117)=(S=12756992,E=12757298,C=14)
	CT(57118)=(S=12757300,E=12757300,C=28)
	CT(57119)=(S=12757312,E=12757372,C=14)
	CT(57120)=(S=12757373,E=12757373,C=21)
	CT(57121)=(S=12757374,E=12757375,C=58)
	CT(57122)=(S=12757376,E=12757406,C=14)
	CT(57123)=(S=12757431,E=12757431,C=12)
	CT(57124)=(S=12757439,E=12757439,C=38)
	CT(57125)=(S=12757440,E=12757440,C=28)
	CT(57126)=(S=12757442,E=12757445,C=14)
	CT(57127)=(S=12757446,E=12757446,C=28)
	CT(57128)=(S=12757447,E=12757447,C=34)
	CT(57129)=(S=12757448,E=12757448,C=42)
	CT(57130)=(S=12757449,E=12757449,C=25)
	CT(57131)=(S=12757450,E=12757450,C=28)
	CT(57132)=(S=12757451,E=12757451,C=12)
	CT(57133)=(S=12757452,E=12757455,C=31)
	CT(57134)=(S=12757456,E=12757456,C=36)
	CT(57135)=(S=12757457,E=12757457,C=38)
	CT(57136)=(S=12757458,E=12757458,C=31)
	CT(57137)=(S=12757459,E=12757459,C=28)
	CT(57138)=(S=12757460,E=12757460,C=38)
	CT(57139)=(S=12757462,E=12757462,C=12)
	CT(57140)=(S=12757463,E=12757463,C=21)
	CT(57141)=(S=12757464,E=12757464,C=28)
	CT(57142)=(S=12757465,E=12757466,C=14)
	CT(57143)=(S=12757467,E=12757467,C=34)
	CT(57144)=(S=12757468,E=12757468,C=58)
	CT(57145)=(S=12757469,E=12757469,C=14)
	CT(57146)=(S=12757470,E=12757470,C=28)
	CT(57147)=(S=12757472,E=12757472,C=33)
	CT(57148)=(S=12757473,E=12757473,C=19)
	CT(57149)=(S=12757474,E=12757477,C=58)
	CT(57150)=(S=12757479,E=12757479,C=51)
	CT(57151)=(S=12757480,E=12757480,C=22)
	CT(57152)=(S=12757481,E=12757481,C=38)
	CT(57153)=(S=12757482,E=12757482,C=14)
	CT(57154)=(S=12757483,E=12757483,C=38)
	CT(57155)=(S=12757484,E=12757484,C=58)
	CT(57156)=(S=12757486,E=12757486,C=31)
	CT(57157)=(S=12757487,E=12757487,C=28)
	CT(57158)=(S=12757488,E=12757488,C=12)
	CT(57159)=(S=12757489,E=12757489,C=58)
	CT(57160)=(S=12757491,E=12757491,C=19)
	CT(57161)=(S=12757492,E=12757492,C=14)
	CT(57162)=(S=12757494,E=12757494,C=42)
	CT(57163)=(S=12757495,E=12757495,C=18)
	CT(57164)=(S=12757496,E=12757496,C=36)
	CT(57165)=(S=12757497,E=12757497,C=12)
	CT(57166)=(S=12757498,E=12757498,C=18)
	CT(57167)=(S=12757499,E=12757499,C=28)
	CT(57168)=(S=12757500,E=12757500,C=33)
	CT(57169)=(S=12757502,E=12757502,C=14)
	CT(57170)=(S=12757503,E=12757503,C=18)
	CT(57171)=(S=12757504,E=12757759,C=16)
	CT(57172)=(S=12757760,E=12758015,C=36)
	CT(57173)=(S=12758016,E=12759071,C=28)
	CT(57174)=(S=12759072,E=12759103,C=44)
	CT(57175)=(S=12759104,E=12759135,C=14)
	CT(57176)=(S=12759136,E=12759136,C=18)
	CT(57177)=(S=12759137,E=12759137,C=31)
	CT(57178)=(S=12759140,E=12759140,C=18)
	CT(57179)=(S=12759141,E=12759142,C=58)
	CT(57180)=(S=12759143,E=12759143,C=47)
	CT(57181)=(S=12759144,E=12759144,C=34)
	CT(57182)=(S=12759145,E=12759145,C=14)
	CT(57183)=(S=12759146,E=12759146,C=28)
	CT(57184)=(S=12759148,E=12759148,C=101)
	CT(57185)=(S=12759149,E=12759149,C=34)
	CT(57186)=(S=12759150,E=12759150,C=49)
	CT(57187)=(S=12759151,E=12759151,C=62)
	CT(57188)=(S=12759152,E=12759152,C=52)
	CT(57189)=(S=12759153,E=12759153,C=22)
	CT(57190)=(S=12759154,E=12759154,C=18)
	CT(57191)=(S=12759155,E=12759155,C=38)
	CT(57192)=(S=12759156,E=12759156,C=34)
	CT(57193)=(S=12759158,E=12759158,C=18)
	CT(57194)=(S=12759159,E=12759159,C=25)
	CT(57195)=(S=12759160,E=12759162,C=28)
	CT(57196)=(S=12759164,E=12759165,C=14)
	CT(57197)=(S=12759166,E=12759166,C=15)
	CT(57198)=(S=12759167,E=12759167,C=12)
	CT(57199)=(S=12759168,E=12759199,C=13)
	CT(57200)=(S=12759200,E=12759231,C=38)
	CT(57201)=(S=12759232,E=12759263,C=14)
	CT(57202)=(S=12759264,E=12759295,C=50)
	CT(57203)=(S=12759296,E=12759299,C=31)
	CT(57204)=(S=12759300,E=12759303,C=58)
	CT(57205)=(S=12759304,E=12759307,C=12)
	CT(57206)=(S=12759308,E=12759311,C=18)
	CT(57207)=(S=12759312,E=12759315,C=17)
	CT(57208)=(S=12759316,E=12759319,C=18)
	CT(57209)=(S=12759320,E=12759323,C=31)
	CT(57210)=(S=12759324,E=12759327,C=58)
	CT(57211)=(S=12759328,E=12759359,C=12)
	CT(57212)=(S=12759360,E=12759423,C=15)
	CT(57213)=(S=12759424,E=12759455,C=42)
	CT(57214)=(S=12759456,E=12759487,C=14)
	CT(57215)=(S=12759488,E=12759519,C=22)
	CT(57216)=(S=12759520,E=12759551,C=168)
	CT(57217)=(S=12759552,E=12759807,C=36)
	CT(57218)=(S=12759808,E=12759935,C=25)
	CT(57219)=(S=12759936,E=12760063,C=15)
	CT(57220)=(S=12760064,E=12760220,C=28)
	CT(57221)=(S=12760223,E=12760223,C=14)
	CT(57222)=(S=12760224,E=12760282,C=28)
	CT(57223)=(S=12760290,E=12760290,C=55)
	CT(57224)=(S=12760298,E=12760319,C=28)
	CT(57225)=(S=12760320,E=12760575,C=58)
	CT(57226)=(S=12760576,E=12760831,C=24)
	CT(57227)=(S=12760832,E=12760863,C=15)
	CT(57228)=(S=12760864,E=12760895,C=58)
	CT(57229)=(S=12760896,E=12760927,C=100)
	CT(57230)=(S=12760928,E=12760959,C=14)
	CT(57231)=(S=12760960,E=12760991,C=42)
	CT(57232)=(S=12760992,E=12761023,C=31)
	CT(57233)=(S=12761024,E=12761055,C=12)
	CT(57234)=(S=12761056,E=12761087,C=49)
	CT(57235)=(S=12761088,E=12761599,C=15)
	CT(57236)=(S=12761600,E=12761855,C=18)
	CT(57237)=(S=12761856,E=12761871,C=27)
	CT(57238)=(S=12761872,E=12761875,C=28)
	CT(57239)=(S=12761876,E=12761879,C=38)
	CT(57240)=(S=12761884,E=12761887,C=31)
	CT(57241)=(S=12761888,E=12761891,C=14)
	CT(57242)=(S=12761896,E=12761899,C=58)
	CT(57243)=(S=12761900,E=12761903,C=36)
	CT(57244)=(S=12761904,E=12761907,C=31)
	CT(57245)=(S=12761908,E=12761911,C=58)
	CT(57246)=(S=12761912,E=12761915,C=14)
	CT(57247)=(S=12761916,E=12761919,C=19)
	CT(57248)=(S=12761920,E=12761923,C=28)
	CT(57249)=(S=12761924,E=12761927,C=60)
	CT(57250)=(S=12761928,E=12761931,C=58)
	CT(57251)=(S=12761932,E=12761935,C=36)
	CT(57252)=(S=12761936,E=12761939,C=43)
	CT(57253)=(S=12761940,E=12761943,C=17)
	CT(57254)=(S=12761944,E=12761947,C=34)
	CT(57255)=(S=12761948,E=12761951,C=42)
	CT(57256)=(S=12761952,E=12761955,C=36)
	CT(57257)=(S=12761956,E=12761959,C=60)
	CT(57258)=(S=12761960,E=12761967,C=31)
	CT(57259)=(S=12761968,E=12761971,C=28)
	CT(57260)=(S=12761972,E=12761975,C=58)
	CT(57261)=(S=12761980,E=12761983,C=15)
	CT(57262)=(S=12761984,E=12761987,C=31)
	CT(57263)=(S=12761988,E=12761991,C=51)
	CT(57264)=(S=12761992,E=12761995,C=38)
	CT(57265)=(S=12761996,E=12761999,C=28)
	CT(57266)=(S=12762000,E=12762003,C=58)
	CT(57267)=(S=12762004,E=12762011,C=31)
	CT(57268)=(S=12762012,E=12762015,C=34)
	CT(57269)=(S=12762016,E=12762023,C=28)
	CT(57270)=(S=12762024,E=12762027,C=58)
	CT(57271)=(S=12762028,E=12762031,C=15)
	CT(57272)=(S=12762032,E=12762035,C=42)
	CT(57273)=(S=12762036,E=12762039,C=58)
	CT(57274)=(S=12762040,E=12762043,C=28)
	CT(57275)=(S=12762048,E=12762051,C=12)
	CT(57276)=(S=12762052,E=12762055,C=18)
	CT(57277)=(S=12762056,E=12762059,C=58)
	CT(57278)=(S=12762060,E=12762063,C=18)
	CT(57279)=(S=12762064,E=12762067,C=31)
	CT(57280)=(S=12762068,E=12762071,C=47)
	CT(57281)=(S=12762072,E=12762075,C=31)
	CT(57282)=(S=12762076,E=12762079,C=28)
	CT(57283)=(S=12762080,E=12762083,C=12)
	CT(57284)=(S=12762084,E=12762087,C=31)
	CT(57285)=(S=12762088,E=12762091,C=47)
	CT(57286)=(S=12762092,E=12762095,C=58)
	CT(57287)=(S=12762096,E=12762099,C=28)
	CT(57288)=(S=12762100,E=12762103,C=20)
	CT(57289)=(S=12762104,E=12762107,C=14)
	CT(57290)=(S=12762108,E=12762111,C=15)
	CT(57291)=(S=12762112,E=12762367,C=47)
	CT(57292)=(S=12762368,E=12762623,C=14)
	CT(57293)=(S=12762624,E=12762879,C=18)
	CT(57294)=(S=12762880,E=12763135,C=34)
	CT(57295)=(S=12763136,E=12763391,C=24)
	CT(57296)=(S=12763392,E=12763647,C=14)
	CT(57297)=(S=12763648,E=12763903,C=13)
	CT(57298)=(S=12763904,E=12764159,C=28)
	CT(57299)=(S=12764160,E=12764415,C=13)
	CT(57300)=(S=12764416,E=12764671,C=47)
	CT(57301)=(S=12764672,E=12764927,C=19)
	CT(57302)=(S=12764928,E=12765183,C=12)
	CT(57303)=(S=12765184,E=12766207,C=14)
	CT(57304)=(S=12766208,E=12766271,C=55)
	CT(57305)=(S=12766272,E=12766303,C=34)
	CT(57306)=(S=12766304,E=12766335,C=72)
	CT(57307)=(S=12766336,E=12766399,C=58)
	CT(57308)=(S=12766400,E=12766463,C=134)
	CT(57309)=(S=12766464,E=12766719,C=13)
	CT(57310)=(S=12766720,E=12766975,C=12)
	CT(57311)=(S=12766976,E=12767231,C=14)
	CT(57312)=(S=12767232,E=12767487,C=42)
	CT(57313)=(S=12767488,E=12767743,C=34)
	CT(57314)=(S=12767744,E=12767999,C=21)
	CT(57315)=(S=12768000,E=12768255,C=47)
	CT(57316)=(S=12768256,E=12768511,C=33)
	CT(57317)=(S=12768512,E=12768512,C=28)
	CT(57318)=(S=12768513,E=12768513,C=58)
	CT(57319)=(S=12768514,E=12768514,C=28)
	CT(57320)=(S=12768515,E=12768515,C=14)
	CT(57321)=(S=12768516,E=12768516,C=17)
	CT(57322)=(S=12768517,E=12768517,C=28)
	CT(57323)=(S=12768518,E=12768518,C=38)
	CT(57324)=(S=12768519,E=12768519,C=34)
	CT(57325)=(S=12768521,E=12768521,C=60)
	CT(57326)=(S=12768523,E=12768523,C=42)
	CT(57327)=(S=12768524,E=12768524,C=18)
	CT(57328)=(S=12768526,E=12768526,C=49)
	CT(57329)=(S=12768527,E=12768527,C=36)
	CT(57330)=(S=12768528,E=12768528,C=38)
	CT(57331)=(S=12768530,E=12768530,C=14)
	CT(57332)=(S=12768531,E=12768531,C=25)
	CT(57333)=(S=12768533,E=12768533,C=18)
	CT(57334)=(S=12768534,E=12768534,C=58)
	CT(57335)=(S=12768535,E=12768535,C=31)
	CT(57336)=(S=12768537,E=12768537,C=18)
	CT(57337)=(S=12768538,E=12768538,C=58)
	CT(57338)=(S=12768539,E=12768539,C=28)
	CT(57339)=(S=12768540,E=12768540,C=14)
	CT(57340)=(S=12768541,E=12768541,C=63)
	CT(57341)=(S=12768542,E=12768542,C=12)
	CT(57342)=(S=12768543,E=12768543,C=28)
	CT(57343)=(S=12768544,E=12768575,C=33)
	CT(57344)=(S=12768576,E=12768607,C=19)
	CT(57345)=(S=12768608,E=12768609,C=18)
	CT(57346)=(S=12768610,E=12768611,C=28)
	CT(57347)=(S=12768612,E=12768613,C=60)
	CT(57348)=(S=12768614,E=12768615,C=18)
	CT(57349)=(S=12768616,E=12768619,C=31)
	CT(57350)=(S=12768620,E=12768621,C=17)
	CT(57351)=(S=12768622,E=12768623,C=18)
	CT(57352)=(S=12768624,E=12768625,C=36)
	CT(57353)=(S=12768626,E=12768627,C=49)
	CT(57354)=(S=12768628,E=12768629,C=18)
	CT(57355)=(S=12768630,E=12768631,C=19)
	CT(57356)=(S=12768632,E=12768633,C=31)
	CT(57357)=(S=12768634,E=12768635,C=18)
	CT(57358)=(S=12768636,E=12768637,C=12)
	CT(57359)=(S=12768638,E=12768639,C=36)
	CT(57360)=(S=12768640,E=12768703,C=27)
	CT(57361)=(S=12768704,E=12768767,C=33)
	CT(57362)=(S=12768768,E=12769023,C=12)
	CT(57363)=(S=12769024,E=12769279,C=47)
	CT(57364)=(S=12769280,E=12769791,C=14)
	CT(57365)=(S=12769792,E=12770047,C=19)
	CT(57366)=(S=12770048,E=12770303,C=22)
	CT(57367)=(S=12770304,E=12770559,C=18)
	CT(57368)=(S=12770560,E=12770815,C=13)
	CT(57369)=(S=12770816,E=12771327,C=14)
	CT(57370)=(S=12771328,E=12771583,C=25)
	CT(57371)=(S=12771584,E=12771839,C=26)
	CT(57372)=(S=12771840,E=12772095,C=18)
	CT(57373)=(S=12772096,E=12772351,C=14)
	CT(57374)=(S=12772352,E=12772607,C=33)
	CT(57375)=(S=12772608,E=12772863,C=36)
	CT(57376)=(S=12772864,E=12773119,C=34)
	CT(57377)=(S=12773120,E=12773375,C=28)
	CT(57378)=(S=12773376,E=12773631,C=42)
	CT(57379)=(S=12773632,E=12773887,C=28)
	CT(57380)=(S=12773888,E=12774399,C=13)
	CT(57381)=(S=12774400,E=12774911,C=19)
	CT(57382)=(S=12774912,E=12775167,C=14)
	CT(57383)=(S=12775168,E=12775423,C=24)
	CT(57384)=(S=12775424,E=12775935,C=47)
	CT(57385)=(S=12775936,E=12775937,C=58)
	CT(57386)=(S=12775940,E=12775945,C=18)
	CT(57387)=(S=12775946,E=12775947,C=27)
	CT(57388)=(S=12775948,E=12775949,C=28)
	CT(57389)=(S=12775950,E=12775951,C=18)
	CT(57390)=(S=12775952,E=12775953,C=42)
	CT(57391)=(S=12775954,E=12775955,C=36)
	CT(57392)=(S=12775956,E=12775957,C=28)
	CT(57393)=(S=12775960,E=12775961,C=17)
	CT(57394)=(S=12775962,E=12775967,C=18)
	CT(57395)=(S=12775968,E=12775968,C=37)
	CT(57396)=(S=12775969,E=12775969,C=18)
	CT(57397)=(S=12775970,E=12775970,C=34)
	CT(57398)=(S=12775971,E=12775971,C=42)
	CT(57399)=(S=12775972,E=12775972,C=52)
	CT(57400)=(S=12775973,E=12775973,C=42)
	CT(57401)=(S=12775975,E=12775975,C=58)
	CT(57402)=(S=12775976,E=12775976,C=24)
	CT(57403)=(S=12775977,E=12775977,C=42)
	CT(57404)=(S=12775978,E=12775978,C=24)
	CT(57405)=(S=12775979,E=12775983,C=12)
	CT(57406)=(S=12775986,E=12775986,C=24)
	CT(57407)=(S=12775987,E=12775987,C=34)
	CT(57408)=(S=12775988,E=12775988,C=58)
	CT(57409)=(S=12775989,E=12775989,C=31)
	CT(57410)=(S=12775990,E=12775990,C=36)
	CT(57411)=(S=12775991,E=12775991,C=14)
	CT(57412)=(S=12775993,E=12775993,C=38)
	CT(57413)=(S=12775994,E=12775994,C=49)
	CT(57414)=(S=12775995,E=12775996,C=31)
	CT(57415)=(S=12775997,E=12775997,C=15)
	CT(57416)=(S=12775998,E=12775998,C=58)
	CT(57417)=(S=12775999,E=12775999,C=34)
	CT(57418)=(S=12776000,E=12776031,C=36)
	CT(57419)=(S=12776032,E=12776039,C=31)
	CT(57420)=(S=12776040,E=12776043,C=58)
	CT(57421)=(S=12776044,E=12776047,C=19)
	CT(57422)=(S=12776048,E=12776051,C=12)
	CT(57423)=(S=12776052,E=12776055,C=31)
	CT(57424)=(S=12776056,E=12776063,C=18)
	CT(57425)=(S=12776064,E=12776095,C=14)
	CT(57426)=(S=12776096,E=12776127,C=12)
	CT(57427)=(S=12776128,E=12776703,C=15)
	CT(57428)=(S=12776704,E=12776959,C=28)
	CT(57429)=(S=12776960,E=12776991,C=15)
	CT(57430)=(S=12776999,E=12776999,C=36)
	CT(57431)=(S=12777004,E=12777033,C=28)
	CT(57432)=(S=12777034,E=12777035,C=31)
	CT(57433)=(S=12777036,E=12777036,C=24)
	CT(57434)=(S=12777037,E=12777037,C=31)
	CT(57435)=(S=12777038,E=12777039,C=14)
	CT(57436)=(S=12777045,E=12777047,C=28)
	CT(57437)=(S=12777056,E=12777056,C=42)
	CT(57438)=(S=12777057,E=12777057,C=34)
	CT(57439)=(S=12777058,E=12777058,C=58)
	CT(57440)=(S=12777059,E=12777059,C=31)
	CT(57441)=(S=12777061,E=12777061,C=12)
	CT(57442)=(S=12777062,E=12777062,C=50)
	CT(57443)=(S=12777064,E=12777065,C=31)
	CT(57444)=(S=12777066,E=12777067,C=58)
	CT(57445)=(S=12777068,E=12777069,C=14)
	CT(57446)=(S=12777070,E=12777071,C=51)
	CT(57447)=(S=12777072,E=12777075,C=18)
	CT(57448)=(S=12777076,E=12777077,C=31)
	CT(57449)=(S=12777078,E=12777079,C=34)
	CT(57450)=(S=12777080,E=12777081,C=31)
	CT(57451)=(S=12777082,E=12777083,C=28)
	CT(57452)=(S=12777084,E=12777085,C=58)
	CT(57453)=(S=12777086,E=12777087,C=15)
	CT(57454)=(S=12777088,E=12777215,C=28)
	CT(57455)=(S=12777216,E=12777219,C=58)
	CT(57456)=(S=12777220,E=12777221,C=28)
	CT(57457)=(S=12777222,E=12777223,C=19)
	CT(57458)=(S=12777224,E=12777225,C=34)
	CT(57459)=(S=12777226,E=12777227,C=42)
	CT(57460)=(S=12777228,E=12777229,C=31)
	CT(57461)=(S=12777230,E=12777231,C=18)
	CT(57462)=(S=12777232,E=12777235,C=31)
	CT(57463)=(S=12777236,E=12777241,C=18)
	CT(57464)=(S=12777242,E=12777243,C=12)
	CT(57465)=(S=12777244,E=12777245,C=15)
	CT(57466)=(S=12777246,E=12777247,C=36)
	CT(57467)=(S=12777248,E=12777248,C=18)
	CT(57468)=(S=12777249,E=12777249,C=31)
	CT(57469)=(S=12777250,E=12777250,C=12)
	CT(57470)=(S=12777251,E=12777251,C=31)
	CT(57471)=(S=12777252,E=12777252,C=50)
	CT(57472)=(S=12777253,E=12777253,C=31)
	CT(57473)=(S=12777254,E=12777254,C=36)
	CT(57474)=(S=12777255,E=12777255,C=46)
	CT(57475)=(S=12777256,E=12777256,C=18)
	CT(57476)=(S=12777257,E=12777257,C=58)
	CT(57477)=(S=12777258,E=12777258,C=31)
	CT(57478)=(S=12777259,E=12777261,C=47)
	CT(57479)=(S=12777262,E=12777262,C=58)
	CT(57480)=(S=12777263,E=12777263,C=42)
	CT(57481)=(S=12777264,E=12777264,C=27)
	CT(57482)=(S=12777265,E=12777265,C=14)
	CT(57483)=(S=12777266,E=12777266,C=66)
	CT(57484)=(S=12777267,E=12777267,C=18)
	CT(57485)=(S=12777268,E=12777268,C=53)
	CT(57486)=(S=12777269,E=12777271,C=18)
	CT(57487)=(S=12777272,E=12777272,C=15)
	CT(57488)=(S=12777273,E=12777273,C=42)
	CT(57489)=(S=12777275,E=12777275,C=37)
	CT(57490)=(S=12777276,E=12777276,C=31)
	CT(57491)=(S=12777277,E=12777277,C=24)
	CT(57492)=(S=12777278,E=12777278,C=58)
	CT(57493)=(S=12777279,E=12777279,C=36)
	CT(57494)=(S=12777280,E=12777311,C=14)
	CT(57495)=(S=12777312,E=12777343,C=36)
	CT(57496)=(S=12777344,E=12777375,C=18)
	CT(57497)=(S=12777376,E=12777377,C=49)
	CT(57498)=(S=12777378,E=12777379,C=52)
	CT(57499)=(S=12777380,E=12777381,C=53)
	CT(57500)=(S=12777382,E=12777383,C=17)
	CT(57501)=(S=12777384,E=12777385,C=18)
	CT(57502)=(S=12777386,E=12777387,C=14)
	CT(57503)=(S=12777388,E=12777395,C=31)
	CT(57504)=(S=12777396,E=12777397,C=12)
	CT(57505)=(S=12777400,E=12777401,C=18)
	CT(57506)=(S=12777404,E=12777405,C=24)
	CT(57507)=(S=12777406,E=12777407,C=18)
	CT(57508)=(S=12777408,E=12777439,C=46)
	CT(57509)=(S=12777440,E=12777471,C=14)
	CT(57510)=(S=12777472,E=12777727,C=27)
	CT(57511)=(S=12777728,E=12777983,C=52)
	CT(57512)=(S=12777984,E=12778239,C=12)
	CT(57513)=(S=12778240,E=12778751,C=47)
	CT(57514)=(S=12778752,E=12779007,C=13)
	CT(57515)=(S=12779008,E=12779263,C=12)
	CT(57516)=(S=12779264,E=12779519,C=24)
	CT(57517)=(S=12779520,E=12779647,C=49)
	CT(57518)=(S=12779648,E=12780031,C=27)
	CT(57519)=(S=12780032,E=12780095,C=13)
	CT(57520)=(S=12780096,E=12780127,C=18)
	CT(57521)=(S=12780128,E=12780159,C=60)
	CT(57522)=(S=12780160,E=12780191,C=14)
	CT(57523)=(S=12780192,E=12780223,C=28)
	CT(57524)=(S=12780224,E=12780227,C=18)
	CT(57525)=(S=12780228,E=12780229,C=38)
	CT(57526)=(S=12780230,E=12780231,C=18)
	CT(57527)=(S=12780232,E=12780233,C=42)
	CT(57528)=(S=12780234,E=12780235,C=63)
	CT(57529)=(S=12780236,E=12780237,C=31)
	CT(57530)=(S=12780238,E=12780239,C=18)
	CT(57531)=(S=12780240,E=12780241,C=58)
	CT(57532)=(S=12780242,E=12780249,C=18)
	CT(57533)=(S=12780250,E=12780251,C=38)
	CT(57534)=(S=12780252,E=12780253,C=42)
	CT(57535)=(S=12780254,E=12780255,C=58)
	CT(57536)=(S=12780256,E=12780257,C=14)
	CT(57537)=(S=12780258,E=12780259,C=18)
	CT(57538)=(S=12780260,E=12780261,C=34)
	CT(57539)=(S=12780262,E=12780263,C=20)
	CT(57540)=(S=12780264,E=12780265,C=18)
	CT(57541)=(S=12780266,E=12780267,C=22)
	CT(57542)=(S=12780268,E=12780269,C=31)
	CT(57543)=(S=12780270,E=12780273,C=18)
	CT(57544)=(S=12780276,E=12780277,C=14)
	CT(57545)=(S=12780278,E=12780279,C=38)
	CT(57546)=(S=12780280,E=12780281,C=36)
	CT(57547)=(S=12780282,E=12780283,C=14)
	CT(57548)=(S=12780284,E=12780285,C=18)
	CT(57549)=(S=12780286,E=12780287,C=58)
	CT(57550)=(S=12780288,E=12780351,C=12)
	CT(57551)=(S=12780352,E=12780415,C=42)
	CT(57552)=(S=12780416,E=12780423,C=31)
	CT(57553)=(S=12780428,E=12780431,C=18)
	CT(57554)=(S=12780432,E=12780435,C=60)
	CT(57555)=(S=12780436,E=12780439,C=31)
	CT(57556)=(S=12780440,E=12780443,C=58)
	CT(57557)=(S=12780444,E=12780447,C=31)
	CT(57558)=(S=12780448,E=12780451,C=15)
	CT(57559)=(S=12780452,E=12780455,C=36)
	CT(57560)=(S=12780456,E=12780459,C=71)
	CT(57561)=(S=12780460,E=12780463,C=14)
	CT(57562)=(S=12780464,E=12780467,C=36)
	CT(57563)=(S=12780468,E=12780471,C=18)
	CT(57564)=(S=12780472,E=12780475,C=14)
	CT(57565)=(S=12780476,E=12780479,C=43)
	CT(57566)=(S=12780480,E=12780483,C=28)
	CT(57567)=(S=12780484,E=12780487,C=31)
	CT(57568)=(S=12780488,E=12780491,C=58)
	CT(57569)=(S=12780492,E=12780495,C=31)
	CT(57570)=(S=12780496,E=12780499,C=13)
	CT(57571)=(S=12780500,E=12780507,C=28)
	CT(57572)=(S=12780508,E=12780511,C=58)
	CT(57573)=(S=12780512,E=12780515,C=31)
	CT(57574)=(S=12780516,E=12780519,C=25)
	CT(57575)=(S=12780520,E=12780523,C=58)
	CT(57576)=(S=12780524,E=12780527,C=31)
	CT(57577)=(S=12780528,E=12780531,C=18)
	CT(57578)=(S=12780532,E=12780535,C=31)
	CT(57579)=(S=12780536,E=12780539,C=28)
	CT(57580)=(S=12780540,E=12780543,C=18)
	CT(57581)=(S=12780544,E=12780799,C=28)
	CT(57582)=(S=12780800,E=12780863,C=31)
	CT(57583)=(S=12780864,E=12780895,C=25)
	CT(57584)=(S=12780898,E=12780899,C=28)
	CT(57585)=(S=12780900,E=12780901,C=24)
	CT(57586)=(S=12780902,E=12780903,C=60)
	CT(57587)=(S=12780904,E=12780905,C=56)
	CT(57588)=(S=12780906,E=12780909,C=31)
	CT(57589)=(S=12780910,E=12780910,C=71)
	CT(57590)=(S=12780911,E=12780911,C=19)
	CT(57591)=(S=12780912,E=12780913,C=27)
	CT(57592)=(S=12780914,E=12780915,C=38)
	CT(57593)=(S=12780916,E=12780917,C=25)
	CT(57594)=(S=12780918,E=12780919,C=19)
	CT(57595)=(S=12780920,E=12780921,C=28)
	CT(57596)=(S=12780922,E=12780923,C=14)
	CT(57597)=(S=12780924,E=12780926,C=31)
	CT(57598)=(S=12780927,E=12780959,C=18)
	CT(57599)=(S=12780960,E=12780960,C=24)
	CT(57600)=(S=12780962,E=12780962,C=18)
	CT(57601)=(S=12780963,E=12780963,C=36)
	CT(57602)=(S=12780964,E=12780965,C=52)
	CT(57603)=(S=12780966,E=12780966,C=27)
	CT(57604)=(S=12780967,E=12780969,C=14)
	CT(57605)=(S=12780971,E=12780971,C=34)
	CT(57606)=(S=12780972,E=12780972,C=63)
	CT(57607)=(S=12780973,E=12780973,C=39)
	CT(57608)=(S=12780974,E=12780974,C=27)
	CT(57609)=(S=12780975,E=12780975,C=14)
	CT(57610)=(S=12780977,E=12780977,C=50)
	CT(57611)=(S=12780978,E=12780978,C=18)
	CT(57612)=(S=12780979,E=12780979,C=40)
	CT(57613)=(S=12780980,E=12780980,C=34)
	CT(57614)=(S=12780981,E=12780982,C=58)
	CT(57615)=(S=12780983,E=12780983,C=27)
	CT(57616)=(S=12780984,E=12780984,C=31)
	CT(57617)=(S=12780985,E=12780985,C=19)
	CT(57618)=(S=12780986,E=12780986,C=33)
	CT(57619)=(S=12780987,E=12780987,C=28)
	CT(57620)=(S=12780988,E=12780988,C=58)
	CT(57621)=(S=12780989,E=12780989,C=14)
	CT(57622)=(S=12780990,E=12780990,C=19)
	CT(57623)=(S=12780991,E=12780991,C=28)
	CT(57624)=(S=12780992,E=12781311,C=12)
	CT(57625)=(S=12781328,E=12781331,C=15)
	CT(57626)=(S=12781332,E=12781343,C=24)
	CT(57627)=(S=12781344,E=12781375,C=63)
	CT(57628)=(S=12781376,E=12781407,C=19)
	CT(57629)=(S=12781408,E=12781439,C=12)
	CT(57630)=(S=12781440,E=12781471,C=36)
	CT(57631)=(S=12781472,E=12781503,C=18)
	CT(57632)=(S=12781504,E=12781567,C=14)
	CT(57633)=(S=12781568,E=12781599,C=21)
	CT(57634)=(S=12781600,E=12781601,C=27)
	CT(57635)=(S=12781602,E=12781607,C=18)
	CT(57636)=(S=12781608,E=12781611,C=31)
	CT(57637)=(S=12781612,E=12781613,C=18)
	CT(57638)=(S=12781614,E=12781615,C=38)
	CT(57639)=(S=12781618,E=12781619,C=66)
	CT(57640)=(S=12781620,E=12781621,C=58)
	CT(57641)=(S=12781622,E=12781623,C=47)
	CT(57642)=(S=12781624,E=12781625,C=31)
	CT(57643)=(S=12781626,E=12781627,C=21)
	CT(57644)=(S=12781628,E=12781629,C=47)
	CT(57645)=(S=12781630,E=12781631,C=18)
	CT(57646)=(S=12781632,E=12781664,C=14)
	CT(57647)=(S=12781665,E=12781665,C=58)
	CT(57648)=(S=12781666,E=12781666,C=19)
	CT(57649)=(S=12781667,E=12781667,C=58)
	CT(57650)=(S=12781668,E=12781668,C=14)
	CT(57651)=(S=12781669,E=12781669,C=58)
	CT(57652)=(S=12781671,E=12781671,C=14)
	CT(57653)=(S=12781672,E=12781672,C=28)
	CT(57654)=(S=12781673,E=12781673,C=38)
	CT(57655)=(S=12781674,E=12781675,C=58)
	CT(57656)=(S=12781676,E=12781676,C=34)
	CT(57657)=(S=12781677,E=12781677,C=37)
	CT(57658)=(S=12781679,E=12781679,C=12)
	CT(57659)=(S=12781681,E=12781681,C=34)
	CT(57660)=(S=12781683,E=12781683,C=60)
	CT(57661)=(S=12781684,E=12781685,C=14)
	CT(57662)=(S=12781688,E=12781688,C=38)
	CT(57663)=(S=12781689,E=12781691,C=28)
	CT(57664)=(S=12781692,E=12781692,C=12)
	CT(57665)=(S=12781693,E=12781694,C=14)
	CT(57666)=(S=12781695,E=12781695,C=42)
	CT(57667)=(S=12781696,E=12781727,C=58)
	CT(57668)=(S=12781728,E=12781761,C=14)
	CT(57669)=(S=12781762,E=12781763,C=38)
	CT(57670)=(S=12781764,E=12781765,C=14)
	CT(57671)=(S=12781766,E=12781767,C=34)
	CT(57672)=(S=12781768,E=12781769,C=31)
	CT(57673)=(S=12781770,E=12781771,C=14)
	CT(57674)=(S=12781772,E=12781773,C=55)
	CT(57675)=(S=12781774,E=12781775,C=38)
	CT(57676)=(S=12781776,E=12781777,C=36)
	CT(57677)=(S=12781778,E=12781779,C=34)
	CT(57678)=(S=12781780,E=12781781,C=42)
	CT(57679)=(S=12781782,E=12781783,C=12)
	CT(57680)=(S=12781784,E=12781785,C=58)
	CT(57681)=(S=12781786,E=12781787,C=44)
	CT(57682)=(S=12781788,E=12781789,C=58)
	CT(57683)=(S=12781790,E=12781791,C=51)
	CT(57684)=(S=12781792,E=12781823,C=28)
	CT(57685)=(S=12781824,E=12782079,C=18)
	CT(57686)=(S=12782080,E=12782143,C=13)
	CT(57687)=(S=12782144,E=12782175,C=70)
	CT(57688)=(S=12782176,E=12782207,C=14)
	CT(57689)=(S=12782208,E=12782271,C=47)
	CT(57690)=(S=12782272,E=12782272,C=72)
	CT(57691)=(S=12782273,E=12782273,C=51)
	CT(57692)=(S=12782274,E=12782274,C=17)
	CT(57693)=(S=12782275,E=12782275,C=28)
	CT(57694)=(S=12782276,E=12782276,C=27)
	CT(57695)=(S=12782277,E=12782277,C=23)
	CT(57696)=(S=12782278,E=12782278,C=18)
	CT(57697)=(S=12782279,E=12782280,C=14)
	CT(57698)=(S=12782281,E=12782281,C=25)
	CT(57699)=(S=12782282,E=12782282,C=36)
	CT(57700)=(S=12782283,E=12782283,C=28)
	CT(57701)=(S=12782284,E=12782284,C=60)
	CT(57702)=(S=12782285,E=12782285,C=18)
	CT(57703)=(S=12782286,E=12782286,C=42)
	CT(57704)=(S=12782287,E=12782287,C=24)
	CT(57705)=(S=12782288,E=12782288,C=28)
	CT(57706)=(S=12782290,E=12782290,C=31)
	CT(57707)=(S=12782291,E=12782291,C=19)
	CT(57708)=(S=12782292,E=12782292,C=36)
	CT(57709)=(S=12782293,E=12782293,C=14)
	CT(57710)=(S=12782294,E=12782294,C=18)
	CT(57711)=(S=12782296,E=12782296,C=60)
	CT(57712)=(S=12782297,E=12782297,C=72)
	CT(57713)=(S=12782298,E=12782298,C=31)
	CT(57714)=(S=12782300,E=12782300,C=17)
	CT(57715)=(S=12782301,E=12782623,C=14)
	CT(57716)=(S=12782624,E=12782627,C=58)
	CT(57717)=(S=12782628,E=12782631,C=31)
	CT(57718)=(S=12782632,E=12782639,C=34)
	CT(57719)=(S=12782640,E=12782643,C=14)
	CT(57720)=(S=12782644,E=12782647,C=38)
	CT(57721)=(S=12782648,E=12782651,C=31)
	CT(57722)=(S=12782652,E=12782655,C=36)
	CT(57723)=(S=12782656,E=12782687,C=18)
	CT(57724)=(S=12782688,E=12782719,C=20)
	CT(57725)=(S=12782720,E=12782751,C=71)
	CT(57726)=(S=12782752,E=12782783,C=44)
	CT(57727)=(S=12782784,E=12782815,C=42)
	CT(57728)=(S=12782816,E=12782847,C=13)
	CT(57729)=(S=12782848,E=12782879,C=49)
	CT(57730)=(S=12782880,E=12782885,C=12)
	CT(57731)=(S=12782886,E=12782887,C=58)
	CT(57732)=(S=12782888,E=12782891,C=28)
	CT(57733)=(S=12782894,E=12782894,C=24)
	CT(57734)=(S=12782896,E=12782897,C=38)
	CT(57735)=(S=12782898,E=12782898,C=18)
	CT(57736)=(S=12782899,E=12782899,C=14)
	CT(57737)=(S=12782900,E=12782901,C=34)
	CT(57738)=(S=12782902,E=12782903,C=19)
	CT(57739)=(S=12782904,E=12782905,C=49)
	CT(57740)=(S=12782906,E=12782907,C=12)
	CT(57741)=(S=12782908,E=12782975,C=14)
	CT(57742)=(S=12782976,E=12783103,C=60)
	CT(57743)=(S=12783104,E=12783104,C=12)
	CT(57744)=(S=12783105,E=12783105,C=36)
	CT(57745)=(S=12783106,E=12783106,C=18)
	CT(57746)=(S=12783107,E=12783108,C=58)
	CT(57747)=(S=12783109,E=12783109,C=12)
	CT(57748)=(S=12783110,E=12783110,C=38)
	CT(57749)=(S=12783112,E=12783112,C=28)
	CT(57750)=(S=12783113,E=12783113,C=22)
	CT(57751)=(S=12783115,E=12783115,C=58)
	CT(57752)=(S=12783116,E=12783116,C=18)
	CT(57753)=(S=12783117,E=12783117,C=38)
	CT(57754)=(S=12783118,E=12783118,C=24)
	CT(57755)=(S=12783119,E=12783119,C=47)
	CT(57756)=(S=12783120,E=12783120,C=14)
	CT(57757)=(S=12783121,E=12783121,C=31)
	CT(57758)=(S=12783122,E=12783122,C=14)
	CT(57759)=(S=12783123,E=12783123,C=23)
	CT(57760)=(S=12783124,E=12783125,C=19)
	CT(57761)=(S=12783126,E=12783126,C=12)
	CT(57762)=(S=12783127,E=12783127,C=34)
	CT(57763)=(S=12783129,E=12783129,C=19)
	CT(57764)=(S=12783132,E=12783132,C=15)
	CT(57765)=(S=12783133,E=12783133,C=36)
	CT(57766)=(S=12783134,E=12783134,C=58)
	CT(57767)=(S=12783135,E=12783135,C=49)
	CT(57768)=(S=12783136,E=12783167,C=18)
	CT(57769)=(S=12783168,E=12783199,C=14)
	CT(57770)=(S=12783200,E=12783201,C=18)
	CT(57771)=(S=12783202,E=12783203,C=36)
	CT(57772)=(S=12783204,E=12783205,C=18)
	CT(57773)=(S=12783206,E=12783207,C=15)
	CT(57774)=(S=12783208,E=12783213,C=18)
	CT(57775)=(S=12783214,E=12783215,C=101)
	CT(57776)=(S=12783218,E=12783219,C=18)
	CT(57777)=(S=12783220,E=12783221,C=58)
	CT(57778)=(S=12783222,E=12783223,C=18)
	CT(57779)=(S=12783224,E=12783225,C=42)
	CT(57780)=(S=12783226,E=12783229,C=31)
	CT(57781)=(S=12783230,E=12783231,C=34)
	CT(57782)=(S=12783232,E=12783263,C=32)
	CT(57783)=(S=12783264,E=12783295,C=44)
	CT(57784)=(S=12783296,E=12783359,C=28)
	CT(57785)=(S=12783360,E=12783615,C=34)
	CT(57786)=(S=12783616,E=12783647,C=49)
	CT(57787)=(S=12783648,E=12783679,C=18)
	CT(57788)=(S=12783684,E=12783687,C=28)
	CT(57789)=(S=12783688,E=12783691,C=18)
	CT(57790)=(S=12783692,E=12783695,C=31)
	CT(57791)=(S=12783696,E=12783699,C=28)
	CT(57792)=(S=12783700,E=12783703,C=36)
	CT(57793)=(S=12783704,E=12783707,C=33)
	CT(57794)=(S=12783708,E=12783743,C=18)
	CT(57795)=(S=12783744,E=12783775,C=25)
	CT(57796)=(S=12783776,E=12783807,C=14)
	CT(57797)=(S=12783808,E=12783839,C=47)
	CT(57798)=(S=12783840,E=12783871,C=42)
	CT(57799)=(S=12783872,E=12784127,C=19)
	CT(57800)=(S=12784132,E=12784135,C=14)
	CT(57801)=(S=12784136,E=12784139,C=17)
	CT(57802)=(S=12784140,E=12784147,C=31)
	CT(57803)=(S=12784148,E=12784155,C=18)
	CT(57804)=(S=12784156,E=12784159,C=31)
	CT(57805)=(S=12784160,E=12784191,C=18)
	CT(57806)=(S=12784192,E=12784255,C=36)
	CT(57807)=(S=12784256,E=12784383,C=27)
	CT(57808)=(S=12784384,E=12784639,C=18)
	CT(57809)=(S=12784640,E=12784643,C=58)
	CT(57810)=(S=12784644,E=12784647,C=31)
	CT(57811)=(S=12784648,E=12784651,C=36)
	CT(57812)=(S=12784652,E=12784655,C=12)
	CT(57813)=(S=12784656,E=12784659,C=47)
	CT(57814)=(S=12784660,E=12784667,C=51)
	CT(57815)=(S=12784668,E=12784671,C=31)
	CT(57816)=(S=12784672,E=12784703,C=36)
	CT(57817)=(S=12784704,E=12784735,C=28)
	CT(57818)=(S=12784736,E=12784737,C=31)
	CT(57819)=(S=12784740,E=12784741,C=19)
	CT(57820)=(S=12784742,E=12784743,C=31)
	CT(57821)=(S=12784746,E=12784747,C=38)
	CT(57822)=(S=12784748,E=12784749,C=14)
	CT(57823)=(S=12784750,E=12784751,C=58)
	CT(57824)=(S=12784752,E=12784755,C=28)
	CT(57825)=(S=12784756,E=12784757,C=47)
	CT(57826)=(S=12784758,E=12784759,C=31)
	CT(57827)=(S=12784760,E=12784762,C=28)
	CT(57828)=(S=12784763,E=12784763,C=58)
	CT(57829)=(S=12784764,E=12784765,C=31)
	CT(57830)=(S=12784766,E=12784767,C=60)
	CT(57831)=(S=12784768,E=12784768,C=18)
	CT(57832)=(S=12784769,E=12784769,C=42)
	CT(57833)=(S=12784770,E=12784770,C=31)
	CT(57834)=(S=12784771,E=12784771,C=58)
	CT(57835)=(S=12784772,E=12784772,C=42)
	CT(57836)=(S=12784773,E=12784773,C=28)
	CT(57837)=(S=12784774,E=12784774,C=58)
	CT(57838)=(S=12784775,E=12784775,C=18)
	CT(57839)=(S=12784776,E=12784776,C=26)
	CT(57840)=(S=12784777,E=12784777,C=18)
	CT(57841)=(S=12784778,E=12784778,C=58)
	CT(57842)=(S=12784779,E=12784779,C=42)
	CT(57843)=(S=12784780,E=12784781,C=18)
	CT(57844)=(S=12784782,E=12784782,C=52)
	CT(57845)=(S=12784784,E=12784784,C=36)
	CT(57846)=(S=12784785,E=12784785,C=38)
	CT(57847)=(S=12784786,E=12784787,C=34)
	CT(57848)=(S=12784788,E=12784788,C=31)
	CT(57849)=(S=12784789,E=12784789,C=47)
	CT(57850)=(S=12784790,E=12784790,C=58)
	CT(57851)=(S=12784791,E=12784791,C=55)
	CT(57852)=(S=12784792,E=12784792,C=50)
	CT(57853)=(S=12784793,E=12784793,C=57)
	CT(57854)=(S=12784794,E=12784794,C=31)
	CT(57855)=(S=12784795,E=12784795,C=14)
	CT(57856)=(S=12784796,E=12784796,C=38)
	CT(57857)=(S=12784797,E=12784797,C=34)
	CT(57858)=(S=12784798,E=12784798,C=42)
	CT(57859)=(S=12784799,E=12784799,C=28)
	CT(57860)=(S=12784800,E=12784831,C=71)
	CT(57861)=(S=12784832,E=12784833,C=187)
	CT(57862)=(S=12784834,E=12784837,C=18)
	CT(57863)=(S=12784838,E=12784839,C=12)
	CT(57864)=(S=12784840,E=12784841,C=58)
	CT(57865)=(S=12784842,E=12784843,C=38)
	CT(57866)=(S=12784844,E=12784845,C=36)
	CT(57867)=(S=12784846,E=12784847,C=19)
	CT(57868)=(S=12784848,E=12784849,C=34)
	CT(57869)=(S=12784850,E=12784851,C=28)
	CT(57870)=(S=12784852,E=12784853,C=58)
	CT(57871)=(S=12784854,E=12784855,C=14)
	CT(57872)=(S=12784856,E=12784857,C=25)
	CT(57873)=(S=12784858,E=12784859,C=58)
	CT(57874)=(S=12784860,E=12784861,C=28)
	CT(57875)=(S=12784862,E=12784863,C=14)
	CT(57876)=(S=12784864,E=12784895,C=28)
	CT(57877)=(S=12784896,E=12785151,C=14)
	CT(57878)=(S=12785152,E=12785183,C=21)
	CT(57879)=(S=12785184,E=12785215,C=33)
	CT(57880)=(S=12785216,E=12785247,C=19)
	CT(57881)=(S=12785248,E=12785251,C=58)
	CT(57882)=(S=12785252,E=12785255,C=36)
	CT(57883)=(S=12785256,E=12785259,C=18)
	CT(57884)=(S=12785260,E=12785267,C=31)
	CT(57885)=(S=12785268,E=12785271,C=58)
	CT(57886)=(S=12785272,E=12785272,C=34)
	CT(57887)=(S=12785273,E=12785273,C=14)
	CT(57888)=(S=12785275,E=12785275,C=18)
	CT(57889)=(S=12785276,E=12785279,C=58)
	CT(57890)=(S=12785280,E=12785281,C=14)
	CT(57891)=(S=12785282,E=12785285,C=31)
	CT(57892)=(S=12785286,E=12785287,C=14)
	CT(57893)=(S=12785288,E=12785289,C=28)
	CT(57894)=(S=12785290,E=12785291,C=49)
	CT(57895)=(S=12785292,E=12785293,C=31)
	CT(57896)=(S=12785294,E=12785295,C=28)
	CT(57897)=(S=12785296,E=12785297,C=12)
	CT(57898)=(S=12785298,E=12785299,C=51)
	CT(57899)=(S=12785302,E=12785303,C=14)
	CT(57900)=(S=12785306,E=12785307,C=22)
	CT(57901)=(S=12785310,E=12785311,C=14)
	CT(57902)=(S=12785312,E=12785343,C=44)
	CT(57903)=(S=12785344,E=12785375,C=15)
	CT(57904)=(S=12785376,E=12785407,C=53)
	CT(57905)=(S=12785408,E=12785663,C=21)
	CT(57906)=(S=12785664,E=12785695,C=24)
	CT(57907)=(S=12785696,E=12785727,C=51)
	CT(57908)=(S=12785728,E=12785735,C=18)
	CT(57909)=(S=12785736,E=12785743,C=70)
	CT(57910)=(S=12785744,E=12785751,C=135)
	CT(57911)=(S=12785752,E=12785759,C=51)
	CT(57912)=(S=12785760,E=12785791,C=28)
	CT(57913)=(S=12785792,E=12785823,C=31)
	CT(57914)=(S=12785824,E=12785855,C=19)
	CT(57915)=(S=12785856,E=12785887,C=120)
	CT(57916)=(S=12785888,E=12785889,C=19)
	CT(57917)=(S=12785890,E=12785891,C=24)
	CT(57918)=(S=12785892,E=12785893,C=18)
	CT(57919)=(S=12785894,E=12785897,C=14)
	CT(57920)=(S=12785898,E=12785899,C=31)
	CT(57921)=(S=12785900,E=12785905,C=38)
	CT(57922)=(S=12785906,E=12785907,C=31)
	CT(57923)=(S=12785908,E=12785909,C=24)
	CT(57924)=(S=12785910,E=12785911,C=12)
	CT(57925)=(S=12785912,E=12785913,C=58)
	CT(57926)=(S=12785914,E=12785915,C=13)
	CT(57927)=(S=12785916,E=12785917,C=31)
	CT(57928)=(S=12785918,E=12785919,C=18)
	CT(57929)=(S=12785920,E=12786175,C=12)
	CT(57930)=(S=12786176,E=12786177,C=27)
	CT(57931)=(S=12786178,E=12786179,C=49)
	CT(57932)=(S=12786180,E=12786181,C=70)
	CT(57933)=(S=12786186,E=12786187,C=19)
	CT(57934)=(S=12786188,E=12786189,C=58)
	CT(57935)=(S=12786190,E=12786191,C=42)
	CT(57936)=(S=12786192,E=12786195,C=31)
	CT(57937)=(S=12786196,E=12786197,C=60)
	CT(57938)=(S=12786198,E=12786200,C=58)
	CT(57939)=(S=12786201,E=12786201,C=33)
	CT(57940)=(S=12786202,E=12786203,C=14)
	CT(57941)=(S=12786204,E=12786205,C=19)
	CT(57942)=(S=12786206,E=12786207,C=18)
	CT(57943)=(S=12786208,E=12786239,C=14)
	CT(57944)=(S=12786240,E=12786243,C=31)
	CT(57945)=(S=12786248,E=12786255,C=58)
	CT(57946)=(S=12786256,E=12786263,C=31)
	CT(57947)=(S=12786264,E=12786267,C=14)
	CT(57948)=(S=12786268,E=12786271,C=31)
	CT(57949)=(S=12786272,E=12786303,C=14)
	CT(57950)=(S=12786304,E=12786335,C=54)
	CT(57951)=(S=12786336,E=12786367,C=18)
	CT(57952)=(S=12786368,E=12786399,C=42)
	CT(57953)=(S=12786400,E=12786431,C=14)
	CT(57954)=(S=12786432,E=12786687,C=13)
	CT(57955)=(S=12786688,E=12786689,C=58)
	CT(57956)=(S=12786690,E=12786697,C=38)
	CT(57957)=(S=12786700,E=12786701,C=28)
	CT(57958)=(S=12786702,E=12786703,C=18)
	CT(57959)=(S=12786704,E=12786705,C=58)
	CT(57960)=(S=12786706,E=12786707,C=38)
	CT(57961)=(S=12786708,E=12786709,C=18)
	CT(57962)=(S=12786710,E=12786711,C=36)
	CT(57963)=(S=12786712,E=12786713,C=38)
	CT(57964)=(S=12786714,E=12786715,C=60)
	CT(57965)=(S=12786716,E=12786717,C=19)
	CT(57966)=(S=12786718,E=12786719,C=31)
	CT(57967)=(S=12786720,E=12786751,C=18)
	CT(57968)=(S=12786752,E=12786847,C=71)
	CT(57969)=(S=12786848,E=12786849,C=70)
	CT(57970)=(S=12786850,E=12786851,C=38)
	CT(57971)=(S=12786852,E=12786853,C=49)
	CT(57972)=(S=12786854,E=12786855,C=17)
	CT(57973)=(S=12786858,E=12786859,C=58)
	CT(57974)=(S=12786860,E=12786861,C=27)
	CT(57975)=(S=12786862,E=12786863,C=28)
	CT(57976)=(S=12786864,E=12786865,C=38)
	CT(57977)=(S=12786868,E=12786869,C=17)
	CT(57978)=(S=12786870,E=12786871,C=14)
	CT(57979)=(S=12786872,E=12786873,C=38)
	CT(57980)=(S=12786874,E=12786875,C=28)
	CT(57981)=(S=12786876,E=12786877,C=38)
	CT(57982)=(S=12786878,E=12786879,C=58)
	CT(57983)=(S=12786880,E=12786911,C=12)
	CT(57984)=(S=12786912,E=12786943,C=34)
	CT(57985)=(S=12786944,E=12787199,C=57)
	CT(57986)=(S=12787200,E=12787455,C=28)
	CT(57987)=(S=12787456,E=12787839,C=15)
	CT(57988)=(S=12787840,E=12787967,C=28)
	CT(57989)=(S=12787968,E=12788159,C=36)
	CT(57990)=(S=12788160,E=12788223,C=37)
	CT(57991)=(S=12788224,E=12788287,C=18)
	CT(57992)=(S=12788288,E=12788289,C=28)
	CT(57993)=(S=12788292,E=12788293,C=23)
	CT(57994)=(S=12788296,E=12788297,C=34)
	CT(57995)=(S=12788298,E=12788299,C=31)
	CT(57996)=(S=12788300,E=12788301,C=38)
	CT(57997)=(S=12788302,E=12788303,C=69)
	CT(57998)=(S=12788304,E=12788305,C=58)
	CT(57999)=(S=12788306,E=12788307,C=28)
	CT(58000)=(S=12788308,E=12788309,C=19)
	CT(58001)=(S=12788310,E=12788311,C=70)
	CT(58002)=(S=12788312,E=12788313,C=34)
	CT(58003)=(S=12788314,E=12788315,C=31)
	CT(58004)=(S=12788316,E=12788317,C=14)
	CT(58005)=(S=12788318,E=12788319,C=31)
	CT(58006)=(S=12788320,E=12788351,C=51)
	CT(58007)=(S=12788352,E=12788383,C=42)
	CT(58008)=(S=12788384,E=12788415,C=28)
	CT(58009)=(S=12788416,E=12788419,C=18)
	CT(58010)=(S=12788420,E=12788431,C=31)
	CT(58011)=(S=12788432,E=12788435,C=58)
	CT(58012)=(S=12788436,E=12788439,C=44)
	CT(58013)=(S=12788440,E=12788443,C=58)
	CT(58014)=(S=12788444,E=12788447,C=42)
	CT(58015)=(S=12788448,E=12788479,C=18)
	CT(58016)=(S=12788480,E=12788543,C=28)
	CT(58017)=(S=12788544,E=12788544,C=49)
	CT(58018)=(S=12788545,E=12788545,C=31)
	CT(58019)=(S=12788546,E=12788546,C=21)
	CT(58020)=(S=12788548,E=12788551,C=18)
	CT(58021)=(S=12788552,E=12788559,C=28)
	CT(58022)=(S=12788560,E=12788560,C=58)
	CT(58023)=(S=12788561,E=12788561,C=70)
	CT(58024)=(S=12788562,E=12788563,C=19)
	CT(58025)=(S=12788564,E=12788564,C=51)
	CT(58026)=(S=12788565,E=12788565,C=43)
	CT(58027)=(S=12788566,E=12788566,C=63)
	CT(58028)=(S=12788567,E=12788569,C=28)
	CT(58029)=(S=12788570,E=12788571,C=14)
	CT(58030)=(S=12788572,E=12788575,C=22)
	CT(58031)=(S=12788576,E=12788576,C=21)
	CT(58032)=(S=12788579,E=12788579,C=15)
	CT(58033)=(S=12788580,E=12788581,C=19)
	CT(58034)=(S=12788583,E=12788583,C=28)
	CT(58035)=(S=12788584,E=12788584,C=15)
	CT(58036)=(S=12788585,E=12788585,C=22)
	CT(58037)=(S=12788586,E=12788586,C=12)
	CT(58038)=(S=12788587,E=12788587,C=60)
	CT(58039)=(S=12788588,E=12788588,C=18)
	CT(58040)=(S=12788589,E=12788589,C=27)
	CT(58041)=(S=12788590,E=12788591,C=49)
	CT(58042)=(S=12788592,E=12788593,C=14)
	CT(58043)=(S=12788594,E=12788595,C=36)
	CT(58044)=(S=12788596,E=12788597,C=18)
	CT(58045)=(S=12788600,E=12788600,C=14)
	CT(58046)=(S=12788601,E=12788601,C=34)
	CT(58047)=(S=12788602,E=12788602,C=52)
	CT(58048)=(S=12788603,E=12788605,C=14)
	CT(58049)=(S=12788606,E=12788606,C=22)
	CT(58050)=(S=12788607,E=12788607,C=28)
	CT(58051)=(S=12788608,E=12788735,C=36)
	CT(58052)=(S=12788736,E=12788799,C=15)
	CT(58053)=(S=12788800,E=12788863,C=28)
	CT(58054)=(S=12788864,E=12788991,C=12)
	CT(58055)=(S=12788992,E=12789247,C=28)
	CT(58056)=(S=12789260,E=12789263,C=58)
	CT(58057)=(S=12789264,E=12789266,C=31)
	CT(58058)=(S=12789267,E=12789267,C=14)
	CT(58059)=(S=12789268,E=12789268,C=28)
	CT(58060)=(S=12789270,E=12789270,C=20)
	CT(58061)=(S=12789271,E=12789271,C=18)
	CT(58062)=(S=12789272,E=12789275,C=28)
	CT(58063)=(S=12789276,E=12789277,C=14)
	CT(58064)=(S=12789278,E=12789278,C=72)
	CT(58065)=(S=12789279,E=12789279,C=58)
	CT(58066)=(S=12789280,E=12789311,C=18)
	CT(58067)=(S=12789312,E=12789343,C=14)
	CT(58068)=(S=12789344,E=12789375,C=50)
	CT(58069)=(S=12789376,E=12789407,C=28)
	CT(58070)=(S=12789408,E=12789439,C=62)
	CT(58071)=(S=12789440,E=12789503,C=36)
	CT(58072)=(S=12789504,E=12789631,C=33)
	CT(58073)=(S=12789632,E=12789695,C=101)
	CT(58074)=(S=12789696,E=12789697,C=14)
	CT(58075)=(S=12789700,E=12789701,C=31)
	CT(58076)=(S=12789702,E=12789703,C=51)
	CT(58077)=(S=12789704,E=12789705,C=42)
	CT(58078)=(S=12789710,E=12789711,C=18)
	CT(58079)=(S=12789712,E=12789713,C=42)
	CT(58080)=(S=12789714,E=12789715,C=31)
	CT(58081)=(S=12789716,E=12789717,C=51)
	CT(58082)=(S=12789720,E=12789721,C=58)
	CT(58083)=(S=12789722,E=12789723,C=135)
	CT(58084)=(S=12789726,E=12789727,C=28)
	CT(58085)=(S=12789728,E=12789729,C=37)
	CT(58086)=(S=12789730,E=12789731,C=28)
	CT(58087)=(S=12789732,E=12789733,C=18)
	CT(58088)=(S=12789734,E=12789735,C=28)
	CT(58089)=(S=12789736,E=12789737,C=31)
	CT(58090)=(S=12789738,E=12789739,C=28)
	CT(58091)=(S=12789740,E=12789741,C=22)
	CT(58092)=(S=12789743,E=12789743,C=33)
	CT(58093)=(S=12789744,E=12789747,C=31)
	CT(58094)=(S=12789748,E=12789749,C=49)
	CT(58095)=(S=12789752,E=12789753,C=31)
	CT(58096)=(S=12789758,E=12789759,C=36)
	CT(58097)=(S=12789760,E=12790015,C=14)
	CT(58098)=(S=12790016,E=12790271,C=24)
	CT(58099)=(S=12790272,E=12790335,C=12)
	CT(58100)=(S=12790336,E=12790369,C=18)
	CT(58101)=(S=12790370,E=12790371,C=58)
	CT(58102)=(S=12790372,E=12790373,C=28)
	CT(58103)=(S=12790374,E=12790375,C=36)
	CT(58104)=(S=12790376,E=12790377,C=19)
	CT(58105)=(S=12790378,E=12790379,C=42)
	CT(58106)=(S=12790380,E=12790381,C=28)
	CT(58107)=(S=12790382,E=12790383,C=19)
	CT(58108)=(S=12790384,E=12790385,C=58)
	CT(58109)=(S=12790386,E=12790387,C=28)
	CT(58110)=(S=12790388,E=12790389,C=38)
	CT(58111)=(S=12790390,E=12790391,C=18)
	CT(58112)=(S=12790392,E=12790393,C=28)
	CT(58113)=(S=12790394,E=12790395,C=18)
	CT(58114)=(S=12790396,E=12790397,C=12)
	CT(58115)=(S=12790398,E=12790399,C=31)
	CT(58116)=(S=12790400,E=12790401,C=38)
	CT(58117)=(S=12790402,E=12790403,C=31)
	CT(58118)=(S=12790404,E=12790405,C=29)
	CT(58119)=(S=12790406,E=12790407,C=36)
	CT(58120)=(S=12790408,E=12790409,C=31)
	CT(58121)=(S=12790410,E=12790411,C=38)
	CT(58122)=(S=12790412,E=12790413,C=58)
	CT(58123)=(S=12790414,E=12790415,C=28)
	CT(58124)=(S=12790416,E=12790417,C=12)
	CT(58125)=(S=12790418,E=12790419,C=18)
	CT(58126)=(S=12790420,E=12790421,C=12)
	CT(58127)=(S=12790422,E=12790423,C=24)
	CT(58128)=(S=12790424,E=12790425,C=58)
	CT(58129)=(S=12790426,E=12790463,C=18)
	CT(58130)=(S=12790464,E=12790495,C=19)
	CT(58131)=(S=12790496,E=12790501,C=36)
	CT(58132)=(S=12790502,E=12790502,C=42)
	CT(58133)=(S=12790503,E=12790503,C=12)
	CT(58134)=(S=12790508,E=12790511,C=14)
	CT(58135)=(S=12790521,E=12790521,C=58)
	CT(58136)=(S=12790522,E=12790523,C=12)
	CT(58137)=(S=12790524,E=12790527,C=28)
	CT(58138)=(S=12790528,E=12790559,C=106)
	CT(58139)=(S=12790560,E=12790563,C=18)
	CT(58140)=(S=12790564,E=12790567,C=19)
	CT(58141)=(S=12790568,E=12790571,C=31)
	CT(58142)=(S=12790572,E=12790579,C=14)
	CT(58143)=(S=12790580,E=12790583,C=28)
	CT(58144)=(S=12790584,E=12790587,C=34)
	CT(58145)=(S=12790588,E=12790591,C=27)
	CT(58146)=(S=12790592,E=12790593,C=12)
	CT(58147)=(S=12790594,E=12790595,C=58)
	CT(58148)=(S=12790596,E=12790597,C=18)
	CT(58149)=(S=12790598,E=12790599,C=31)
	CT(58150)=(S=12790600,E=12790601,C=58)
	CT(58151)=(S=12790602,E=12790603,C=66)
	CT(58152)=(S=12790604,E=12790605,C=33)
	CT(58153)=(S=12790606,E=12790607,C=24)
	CT(58154)=(S=12790610,E=12790611,C=60)
	CT(58155)=(S=12790612,E=12790613,C=58)
	CT(58156)=(S=12790614,E=12790615,C=55)
	CT(58157)=(S=12790616,E=12790617,C=28)
	CT(58158)=(S=12790618,E=12790623,C=18)
	CT(58159)=(S=12790624,E=12790655,C=70)
	CT(58160)=(S=12790657,E=12790657,C=15)
	CT(58161)=(S=12790658,E=12790658,C=36)
	CT(58162)=(S=12790659,E=12790659,C=52)
	CT(58163)=(S=12790660,E=12790660,C=22)
	CT(58164)=(S=12790661,E=12790661,C=52)
	CT(58165)=(S=12790662,E=12790662,C=34)
	CT(58166)=(S=12790663,E=12790663,C=18)
	CT(58167)=(S=12790664,E=12790664,C=44)
	CT(58168)=(S=12790665,E=12790665,C=23)
	CT(58169)=(S=12790666,E=12790666,C=36)
	CT(58170)=(S=12790667,E=12790667,C=42)
	CT(58171)=(S=12790668,E=12790668,C=34)
	CT(58172)=(S=12790669,E=12790669,C=28)
	CT(58173)=(S=12790670,E=12790671,C=58)
	CT(58174)=(S=12790672,E=12790672,C=18)
	CT(58175)=(S=12790673,E=12790673,C=27)
	CT(58176)=(S=12790674,E=12790674,C=31)
	CT(58177)=(S=12790675,E=12790675,C=18)
	CT(58178)=(S=12790676,E=12790676,C=31)
	CT(58179)=(S=12790677,E=12790677,C=42)
	CT(58180)=(S=12790678,E=12790678,C=12)
	CT(58181)=(S=12790679,E=12790679,C=19)
	CT(58182)=(S=12790680,E=12790680,C=42)
	CT(58183)=(S=12790681,E=12790681,C=34)
	CT(58184)=(S=12790682,E=12790682,C=36)
	CT(58185)=(S=12790683,E=12790683,C=63)
	CT(58186)=(S=12790684,E=12790684,C=58)
	CT(58187)=(S=12790685,E=12790685,C=14)
	CT(58188)=(S=12790686,E=12790686,C=36)
	CT(58189)=(S=12790687,E=12790687,C=52)
	CT(58190)=(S=12790688,E=12790719,C=15)
	CT(58191)=(S=12790720,E=12790783,C=19)
	CT(58192)=(S=12790784,E=12791039,C=13)
	CT(58193)=(S=12791040,E=12791295,C=15)
	CT(58194)=(S=12791296,E=12791327,C=22)
	CT(58195)=(S=12791328,E=12791331,C=31)
	CT(58196)=(S=12791332,E=12791333,C=58)
	CT(58197)=(S=12791334,E=12791334,C=36)
	CT(58198)=(S=12791335,E=12791335,C=18)
	CT(58199)=(S=12791336,E=12791339,C=58)
	CT(58200)=(S=12791340,E=12791343,C=28)
	CT(58201)=(S=12791344,E=12791347,C=14)
	CT(58202)=(S=12791348,E=12791351,C=46)
	CT(58203)=(S=12791352,E=12791355,C=31)
	CT(58204)=(S=12791356,E=12791359,C=14)
	CT(58205)=(S=12791360,E=12791391,C=71)
	CT(58206)=(S=12791392,E=12791423,C=18)
	CT(58207)=(S=12791424,E=12791455,C=37)
	CT(58208)=(S=12791456,E=12791487,C=18)
	CT(58209)=(S=12791488,E=12791519,C=12)
	CT(58210)=(S=12791520,E=12791551,C=70)
	CT(58211)=(S=12791552,E=12791679,C=33)
	CT(58212)=(S=12791680,E=12791743,C=24)
	CT(58213)=(S=12791744,E=12791744,C=18)
	CT(58214)=(S=12791745,E=12791745,C=51)
	CT(58215)=(S=12791746,E=12791746,C=38)
	CT(58216)=(S=12791747,E=12791747,C=28)
	CT(58217)=(S=12791748,E=12791748,C=18)
	CT(58218)=(S=12791749,E=12791749,C=52)
	CT(58219)=(S=12791750,E=12791750,C=14)
	CT(58220)=(S=12791751,E=12791751,C=15)
	CT(58221)=(S=12791752,E=12791752,C=21)
	CT(58222)=(S=12791753,E=12791753,C=58)
	CT(58223)=(S=12791754,E=12791754,C=31)
	CT(58224)=(S=12791755,E=12791755,C=58)
	CT(58225)=(S=12791756,E=12791756,C=18)
	CT(58226)=(S=12791757,E=12791757,C=19)
	CT(58227)=(S=12791758,E=12791758,C=25)
	CT(58228)=(S=12791759,E=12791759,C=14)
	CT(58229)=(S=12791760,E=12791760,C=22)
	CT(58230)=(S=12791761,E=12791761,C=25)
	CT(58231)=(S=12791763,E=12791765,C=52)
	CT(58232)=(S=12791766,E=12791766,C=38)
	CT(58233)=(S=12791767,E=12791767,C=49)
	CT(58234)=(S=12791769,E=12791769,C=42)
	CT(58235)=(S=12791771,E=12791771,C=31)
	CT(58236)=(S=12791772,E=12791772,C=18)
	CT(58237)=(S=12791773,E=12791773,C=15)
	CT(58238)=(S=12791774,E=12791774,C=24)
	CT(58239)=(S=12791775,E=12791775,C=14)
	CT(58240)=(S=12791776,E=12791776,C=52)
	CT(58241)=(S=12791777,E=12791777,C=14)
	CT(58242)=(S=12791778,E=12791778,C=52)
	CT(58243)=(S=12791779,E=12791779,C=58)
	CT(58244)=(S=12791780,E=12791780,C=52)
	CT(58245)=(S=12791781,E=12791781,C=28)
	CT(58246)=(S=12791783,E=12791783,C=34)
	CT(58247)=(S=12791784,E=12791784,C=15)
	CT(58248)=(S=12791785,E=12791785,C=22)
	CT(58249)=(S=12791786,E=12791786,C=23)
	CT(58250)=(S=12791787,E=12791787,C=33)
	CT(58251)=(S=12791788,E=12791788,C=25)
	CT(58252)=(S=12791789,E=12791789,C=14)
	CT(58253)=(S=12791790,E=12791790,C=19)
	CT(58254)=(S=12791792,E=12791792,C=14)
	CT(58255)=(S=12791793,E=12791793,C=12)
	CT(58256)=(S=12791795,E=12791795,C=52)
	CT(58257)=(S=12791796,E=12791796,C=14)
	CT(58258)=(S=12791797,E=12791797,C=34)
	CT(58259)=(S=12791798,E=12791798,C=58)
	CT(58260)=(S=12791799,E=12791799,C=24)
	CT(58261)=(S=12791800,E=12791800,C=31)
	CT(58262)=(S=12791801,E=12791801,C=28)
	CT(58263)=(S=12791803,E=12791803,C=15)
	CT(58264)=(S=12791804,E=12791804,C=17)
	CT(58265)=(S=12791805,E=12791805,C=31)
	CT(58266)=(S=12791806,E=12791806,C=19)
	CT(58267)=(S=12791807,E=12791807,C=20)
	CT(58268)=(S=12791808,E=12792191,C=34)
	CT(58269)=(S=12792196,E=12792199,C=12)
	CT(58270)=(S=12792200,E=12792203,C=28)
	CT(58271)=(S=12792204,E=12792207,C=12)
	CT(58272)=(S=12792208,E=12792211,C=14)
	CT(58273)=(S=12792212,E=12792215,C=31)
	CT(58274)=(S=12792216,E=12792223,C=28)
	CT(58275)=(S=12792224,E=12792227,C=19)
	CT(58276)=(S=12792228,E=12792231,C=31)
	CT(58277)=(S=12792232,E=12792235,C=18)
	CT(58278)=(S=12792236,E=12792239,C=19)
	CT(58279)=(S=12792240,E=12792243,C=25)
	CT(58280)=(S=12792244,E=12792247,C=14)
	CT(58281)=(S=12792248,E=12792251,C=18)
	CT(58282)=(S=12792252,E=12792255,C=71)
	CT(58283)=(S=12792256,E=12792263,C=18)
	CT(58284)=(S=12792264,E=12792271,C=31)
	CT(58285)=(S=12792272,E=12792279,C=20)
	CT(58286)=(S=12792280,E=12792287,C=37)
	CT(58287)=(S=12792288,E=12792295,C=28)
	CT(58288)=(S=12792304,E=12792311,C=19)
	CT(58289)=(S=12792312,E=12792319,C=38)
	CT(58290)=(S=12792320,E=12792351,C=68)
	CT(58291)=(S=12792352,E=12792383,C=24)
	CT(58292)=(S=12792384,E=12792447,C=14)
	CT(58293)=(S=12792448,E=12792511,C=28)
	CT(58294)=(S=12792512,E=12792575,C=55)
	CT(58295)=(S=12792576,E=12792831,C=13)
	CT(58296)=(S=12792832,E=12793087,C=28)
	CT(58297)=(S=12793088,E=12793343,C=25)
	CT(58298)=(S=12793344,E=12793377,C=18)
	CT(58299)=(S=12793378,E=12793379,C=28)
	CT(58300)=(S=12793380,E=12793381,C=14)
	CT(58301)=(S=12793382,E=12793383,C=36)
	CT(58302)=(S=12793384,E=12793385,C=18)
	CT(58303)=(S=12793386,E=12793387,C=31)
	CT(58304)=(S=12793388,E=12793389,C=51)
	CT(58305)=(S=12793390,E=12793391,C=58)
	CT(58306)=(S=12793392,E=12793393,C=12)
	CT(58307)=(S=12793394,E=12793395,C=58)
	CT(58308)=(S=12793396,E=12793397,C=31)
	CT(58309)=(S=12793398,E=12793399,C=38)
	CT(58310)=(S=12793400,E=12793401,C=36)
	CT(58311)=(S=12793402,E=12793403,C=14)
	CT(58312)=(S=12793404,E=12793405,C=15)
	CT(58313)=(S=12793406,E=12793407,C=12)
	CT(58314)=(S=12793408,E=12793439,C=24)
	CT(58315)=(S=12793440,E=12793503,C=19)
	CT(58316)=(S=12793506,E=12793507,C=31)
	CT(58317)=(S=12793508,E=12793509,C=28)
	CT(58318)=(S=12793512,E=12793513,C=21)
	CT(58319)=(S=12793516,E=12793517,C=42)
	CT(58320)=(S=12793518,E=12793519,C=36)
	CT(58321)=(S=12793520,E=12793521,C=18)
	CT(58322)=(S=12793522,E=12793523,C=40)
	CT(58323)=(S=12793524,E=12793525,C=62)
	CT(58324)=(S=12793526,E=12793527,C=19)
	CT(58325)=(S=12793528,E=12793533,C=58)
	CT(58326)=(S=12793534,E=12793535,C=57)
	CT(58327)=(S=12793536,E=12793567,C=18)
	CT(58328)=(S=12793568,E=12793599,C=14)
	CT(58329)=(S=12793600,E=12793855,C=25)
	CT(58330)=(S=12793856,E=12794111,C=50)
	CT(58331)=(S=12794112,E=12794367,C=25)
	CT(58332)=(S=12794368,E=12794431,C=18)
	CT(58333)=(S=12794432,E=12794463,C=14)
	CT(58334)=(S=12794464,E=12794495,C=19)
	CT(58335)=(S=12794496,E=12794527,C=14)
	CT(58336)=(S=12794528,E=12794559,C=42)
	CT(58337)=(S=12794560,E=12794591,C=14)
	CT(58338)=(S=12794592,E=12794623,C=31)
	CT(58339)=(S=12794624,E=12794879,C=13)
	CT(58340)=(S=12794880,E=12794911,C=14)
	CT(58341)=(S=12794912,E=12794943,C=28)
	CT(58342)=(S=12794944,E=12794945,C=58)
	CT(58343)=(S=12794946,E=12794947,C=31)
	CT(58344)=(S=12794948,E=12794949,C=19)
	CT(58345)=(S=12794950,E=12794951,C=31)
	CT(58346)=(S=12794952,E=12794959,C=38)
	CT(58347)=(S=12794960,E=12794960,C=21)
	CT(58348)=(S=12794961,E=12794961,C=32)
	CT(58349)=(S=12794962,E=12794962,C=14)
	CT(58350)=(S=12794963,E=12794963,C=36)
	CT(58351)=(S=12794964,E=12794964,C=32)
	CT(58352)=(S=12794965,E=12794966,C=14)
	CT(58353)=(S=12794967,E=12794967,C=32)
	CT(58354)=(S=12794968,E=12794968,C=12)
	CT(58355)=(S=12794969,E=12794969,C=15)
	CT(58356)=(S=12794970,E=12794970,C=24)
	CT(58357)=(S=12794971,E=12794971,C=49)
	CT(58358)=(S=12794972,E=12794972,C=31)
	CT(58359)=(S=12794973,E=12794973,C=32)
	CT(58360)=(S=12794974,E=12794974,C=58)
	CT(58361)=(S=12794975,E=12794975,C=14)
	CT(58362)=(S=12794976,E=12795007,C=28)
	CT(58363)=(S=12795008,E=12795039,C=15)
	CT(58364)=(S=12795040,E=12795041,C=24)
	CT(58365)=(S=12795042,E=12795043,C=42)
	CT(58366)=(S=12795044,E=12795045,C=14)
	CT(58367)=(S=12795048,E=12795049,C=31)
	CT(58368)=(S=12795050,E=12795051,C=14)
	CT(58369)=(S=12795054,E=12795055,C=31)
	CT(58370)=(S=12795058,E=12795059,C=38)
	CT(58371)=(S=12795060,E=12795061,C=28)
	CT(58372)=(S=12795062,E=12795063,C=19)
	CT(58373)=(S=12795064,E=12795065,C=31)
	CT(58374)=(S=12795066,E=12795067,C=28)
	CT(58375)=(S=12795068,E=12795069,C=12)
	CT(58376)=(S=12795070,E=12795071,C=15)
	CT(58377)=(S=12795072,E=12795075,C=34)
	CT(58378)=(S=12795076,E=12795079,C=14)
	CT(58379)=(S=12795080,E=12795083,C=31)
	CT(58380)=(S=12795088,E=12795091,C=28)
	CT(58381)=(S=12795092,E=12795095,C=36)
	CT(58382)=(S=12795096,E=12795099,C=58)
	CT(58383)=(S=12795100,E=12795103,C=18)
	CT(58384)=(S=12795104,E=12795111,C=31)
	CT(58385)=(S=12795112,E=12795115,C=17)
	CT(58386)=(S=12795116,E=12795119,C=30)
	CT(58387)=(S=12795120,E=12795123,C=58)
	CT(58388)=(S=12795124,E=12795127,C=18)
	CT(58389)=(S=12795128,E=12795131,C=60)
	CT(58390)=(S=12795132,E=12795135,C=47)
	CT(58391)=(S=12795136,E=12795391,C=13)
	CT(58392)=(S=12795392,E=12795399,C=18)
	CT(58393)=(S=12795400,E=12795403,C=14)
	CT(58394)=(S=12795404,E=12795405,C=58)
	CT(58395)=(S=12795406,E=12795407,C=31)
	CT(58396)=(S=12795408,E=12795409,C=71)
	CT(58397)=(S=12795410,E=12795411,C=17)
	CT(58398)=(S=12795412,E=12795413,C=28)
	CT(58399)=(S=12795416,E=12795417,C=31)
	CT(58400)=(S=12795418,E=12795419,C=42)
	CT(58401)=(S=12795420,E=12795421,C=14)
	CT(58402)=(S=12795422,E=12795423,C=17)
	CT(58403)=(S=12795424,E=12795426,C=18)
	CT(58404)=(S=12795427,E=12795427,C=37)
	CT(58405)=(S=12795428,E=12795429,C=31)
	CT(58406)=(S=12795430,E=12795431,C=18)
	CT(58407)=(S=12795432,E=12795433,C=58)
	CT(58408)=(S=12795434,E=12795435,C=12)
	CT(58409)=(S=12795436,E=12795437,C=28)
	CT(58410)=(S=12795438,E=12795441,C=18)
	CT(58411)=(S=12795442,E=12795443,C=12)
	CT(58412)=(S=12795444,E=12795447,C=18)
	CT(58413)=(S=12795448,E=12795449,C=38)
	CT(58414)=(S=12795450,E=12795455,C=18)
	CT(58415)=(S=12795456,E=12795457,C=58)
	CT(58416)=(S=12795458,E=12795459,C=19)
	CT(58417)=(S=12795460,E=12795461,C=49)
	CT(58418)=(S=12795462,E=12795463,C=18)
	CT(58419)=(S=12795464,E=12795467,C=12)
	CT(58420)=(S=12795468,E=12795469,C=19)
	CT(58421)=(S=12795470,E=12795471,C=18)
	CT(58422)=(S=12795472,E=12795473,C=28)
	CT(58423)=(S=12795474,E=12795475,C=36)
	CT(58424)=(S=12795476,E=12795477,C=42)
	CT(58425)=(S=12795480,E=12795481,C=38)
	CT(58426)=(S=12795482,E=12795483,C=49)
	CT(58427)=(S=12795484,E=12795485,C=18)
	CT(58428)=(S=12795486,E=12795487,C=38)
	CT(58429)=(S=12795488,E=12795519,C=28)
	CT(58430)=(S=12795520,E=12795551,C=60)
	CT(58431)=(S=12795552,E=12795583,C=15)
	CT(58432)=(S=12795584,E=12795615,C=14)
	CT(58433)=(S=12795616,E=12795647,C=15)
	CT(58434)=(S=12795648,E=12795903,C=28)
	CT(58435)=(S=12795904,E=12795935,C=42)
	CT(58436)=(S=12795936,E=12795999,C=36)
	CT(58437)=(S=12796000,E=12796031,C=28)
	CT(58438)=(S=12796034,E=12796035,C=58)
	CT(58439)=(S=12796036,E=12796037,C=38)
	CT(58440)=(S=12796038,E=12796039,C=34)
	CT(58441)=(S=12796040,E=12796041,C=31)
	CT(58442)=(S=12796042,E=12796043,C=34)
	CT(58443)=(S=12796044,E=12796045,C=18)
	CT(58444)=(S=12796046,E=12796047,C=31)
	CT(58445)=(S=12796048,E=12796049,C=19)
	CT(58446)=(S=12796050,E=12796051,C=42)
	CT(58447)=(S=12796052,E=12796053,C=31)
	CT(58448)=(S=12796054,E=12796055,C=58)
	CT(58449)=(S=12796058,E=12796059,C=31)
	CT(58450)=(S=12796060,E=12796061,C=18)
	CT(58451)=(S=12796062,E=12796063,C=58)
	CT(58452)=(S=12796064,E=12796065,C=51)
	CT(58453)=(S=12796066,E=12796067,C=31)
	CT(58454)=(S=12796068,E=12796069,C=12)
	CT(58455)=(S=12796070,E=12796071,C=31)
	CT(58456)=(S=12796074,E=12796075,C=46)
	CT(58457)=(S=12796076,E=12796077,C=22)
	CT(58458)=(S=12796078,E=12796079,C=58)
	CT(58459)=(S=12796080,E=12796081,C=28)
	CT(58460)=(S=12796082,E=12796083,C=38)
	CT(58461)=(S=12796084,E=12796085,C=28)
	CT(58462)=(S=12796086,E=12796089,C=31)
	CT(58463)=(S=12796090,E=12796091,C=25)
	CT(58464)=(S=12796092,E=12796093,C=15)
	CT(58465)=(S=12796094,E=12796095,C=31)
	CT(58466)=(S=12796096,E=12796127,C=18)
	CT(58467)=(S=12796128,E=12796159,C=31)
	CT(58468)=(S=12796160,E=12796415,C=34)
	CT(58469)=(S=12796416,E=12796423,C=28)
	CT(58470)=(S=12796424,E=12796431,C=15)
	CT(58471)=(S=12796432,E=12796439,C=58)
	CT(58472)=(S=12796440,E=12796447,C=14)
	CT(58473)=(S=12796448,E=12796479,C=19)
	CT(58474)=(S=12796480,E=12796480,C=60)
	CT(58475)=(S=12796481,E=12796482,C=31)
	CT(58476)=(S=12796483,E=12796483,C=18)
	CT(58477)=(S=12796484,E=12796484,C=58)
	CT(58478)=(S=12796485,E=12796485,C=52)
	CT(58479)=(S=12796486,E=12796486,C=28)
	CT(58480)=(S=12796487,E=12796488,C=18)
	CT(58481)=(S=12796489,E=12796489,C=58)
	CT(58482)=(S=12796490,E=12796490,C=28)
	CT(58483)=(S=12796491,E=12796491,C=63)
	CT(58484)=(S=12796492,E=12796492,C=28)
	CT(58485)=(S=12796493,E=12796494,C=18)
	CT(58486)=(S=12796495,E=12796495,C=31)
	CT(58487)=(S=12796496,E=12796496,C=12)
	CT(58488)=(S=12796497,E=12796497,C=14)
	CT(58489)=(S=12796498,E=12796498,C=12)
	CT(58490)=(S=12796499,E=12796499,C=28)
	CT(58491)=(S=12796500,E=12796500,C=18)
	CT(58492)=(S=12796501,E=12796501,C=34)
	CT(58493)=(S=12796502,E=12796504,C=58)
	CT(58494)=(S=12796505,E=12796505,C=18)
	CT(58495)=(S=12796506,E=12796506,C=36)
	CT(58496)=(S=12796507,E=12796508,C=42)
	CT(58497)=(S=12796509,E=12796509,C=31)
	CT(58498)=(S=12796510,E=12796511,C=19)
	CT(58499)=(S=12796512,E=12796512,C=18)
	CT(58500)=(S=12796513,E=12796513,C=34)
	CT(58501)=(S=12796514,E=12796514,C=58)
	CT(58502)=(S=12796515,E=12796515,C=28)
	CT(58503)=(S=12796516,E=12796516,C=23)
	CT(58504)=(S=12796517,E=12796517,C=19)
	CT(58505)=(S=12796518,E=12796518,C=29)
	CT(58506)=(S=12796519,E=12796519,C=58)
	CT(58507)=(S=12796521,E=12796521,C=31)
	CT(58508)=(S=12796522,E=12796522,C=55)
	CT(58509)=(S=12796523,E=12796523,C=52)
	CT(58510)=(S=12796524,E=12796524,C=18)
	CT(58511)=(S=12796525,E=12796525,C=24)
	CT(58512)=(S=12796526,E=12796526,C=31)
	CT(58513)=(S=12796527,E=12796527,C=28)
	CT(58514)=(S=12796528,E=12796528,C=12)
	CT(58515)=(S=12796529,E=12796529,C=38)
	CT(58516)=(S=12796531,E=12796531,C=49)
	CT(58517)=(S=12796532,E=12796532,C=58)
	CT(58518)=(S=12796533,E=12796533,C=18)
	CT(58519)=(S=12796534,E=12796534,C=17)
	CT(58520)=(S=12796535,E=12796535,C=12)
	CT(58521)=(S=12796537,E=12796538,C=28)
	CT(58522)=(S=12796539,E=12796539,C=34)
	CT(58523)=(S=12796540,E=12796540,C=38)
	CT(58524)=(S=12796541,E=12796541,C=51)
	CT(58525)=(S=12796542,E=12796542,C=18)
	CT(58526)=(S=12796543,E=12796543,C=21)
	CT(58527)=(S=12796544,E=12796545,C=23)
	CT(58528)=(S=12796546,E=12796547,C=19)
	CT(58529)=(S=12796548,E=12796549,C=29)
	CT(58530)=(S=12796550,E=12796551,C=36)
	CT(58531)=(S=12796552,E=12796557,C=31)
	CT(58532)=(S=12796558,E=12796559,C=38)
	CT(58533)=(S=12796560,E=12796561,C=58)
	CT(58534)=(S=12796562,E=12796563,C=31)
	CT(58535)=(S=12796564,E=12796565,C=14)
	CT(58536)=(S=12796566,E=12796567,C=25)
	CT(58537)=(S=12796570,E=12796571,C=28)
	CT(58538)=(S=12796572,E=12796573,C=31)
	CT(58539)=(S=12796574,E=12796575,C=22)
	CT(58540)=(S=12796576,E=12796607,C=102)
	CT(58541)=(S=12796608,E=12796639,C=31)
	CT(58542)=(S=12796640,E=12796671,C=14)
	CT(58543)=(S=12796672,E=12796927,C=19)
	CT(58544)=(S=12796928,E=12797055,C=14)
	CT(58545)=(S=12797056,E=12797119,C=18)
	CT(58546)=(S=12797120,E=12797121,C=58)
	CT(58547)=(S=12797122,E=12797123,C=12)
	CT(58548)=(S=12797124,E=12797125,C=31)
	CT(58549)=(S=12797126,E=12797127,C=28)
	CT(58550)=(S=12797128,E=12797129,C=51)
	CT(58551)=(S=12797130,E=12797131,C=31)
	CT(58552)=(S=12797132,E=12797133,C=28)
	CT(58553)=(S=12797134,E=12797135,C=14)
	CT(58554)=(S=12797136,E=12797137,C=43)
	CT(58555)=(S=12797138,E=12797139,C=31)
	CT(58556)=(S=12797140,E=12797141,C=42)
	CT(58557)=(S=12797142,E=12797143,C=51)
	CT(58558)=(S=12797144,E=12797147,C=31)
	CT(58559)=(S=12797148,E=12797149,C=14)
	CT(58560)=(S=12797150,E=12797151,C=31)
	CT(58561)=(S=12797156,E=12797157,C=14)
	CT(58562)=(S=12797158,E=12797159,C=71)
	CT(58563)=(S=12797160,E=12797161,C=58)
	CT(58564)=(S=12797162,E=12797163,C=71)
	CT(58565)=(S=12797164,E=12797165,C=28)
	CT(58566)=(S=12797166,E=12797167,C=14)
	CT(58567)=(S=12797168,E=12797169,C=18)
	CT(58568)=(S=12797170,E=12797171,C=28)
	CT(58569)=(S=12797172,E=12797172,C=36)
	CT(58570)=(S=12797173,E=12797173,C=18)
	CT(58571)=(S=12797174,E=12797175,C=28)
	CT(58572)=(S=12797178,E=12797179,C=12)
	CT(58573)=(S=12797180,E=12797181,C=18)
	CT(58574)=(S=12797184,E=12797247,C=27)
	CT(58575)=(S=12797248,E=12797251,C=18)
	CT(58576)=(S=12797252,E=12797255,C=15)
	CT(58577)=(S=12797256,E=12797259,C=36)
	CT(58578)=(S=12797260,E=12797263,C=31)
	CT(58579)=(S=12797264,E=12797267,C=58)
	CT(58580)=(S=12797268,E=12797271,C=31)
	CT(58581)=(S=12797272,E=12797275,C=60)
	CT(58582)=(S=12797276,E=12797279,C=14)
	CT(58583)=(S=12797280,E=12797283,C=52)
	CT(58584)=(S=12797288,E=12797291,C=42)
	CT(58585)=(S=12797292,E=12797295,C=51)
	CT(58586)=(S=12797296,E=12797299,C=28)
	CT(58587)=(S=12797300,E=12797303,C=19)
	CT(58588)=(S=12797304,E=12797307,C=51)
	CT(58589)=(S=12797308,E=12797311,C=58)
	CT(58590)=(S=12797312,E=12797315,C=24)
	CT(58591)=(S=12797316,E=12797319,C=31)
	CT(58592)=(S=12797328,E=12797331,C=36)
	CT(58593)=(S=12797332,E=12797335,C=18)
	CT(58594)=(S=12797336,E=12797339,C=14)
	CT(58595)=(S=12797340,E=12797343,C=18)
	CT(58596)=(S=12797344,E=12797347,C=14)
	CT(58597)=(S=12797348,E=12797351,C=51)
	CT(58598)=(S=12797352,E=12797355,C=18)
	CT(58599)=(S=12797356,E=12797359,C=42)
	CT(58600)=(S=12797360,E=12797363,C=31)
	CT(58601)=(S=12797364,E=12797367,C=42)
	CT(58602)=(S=12797368,E=12797371,C=31)
	CT(58603)=(S=12797372,E=12797375,C=98)
	CT(58604)=(S=12797376,E=12797379,C=42)
	CT(58605)=(S=12797380,E=12797387,C=31)
	CT(58606)=(S=12797388,E=12797391,C=18)
	CT(58607)=(S=12797392,E=12797395,C=58)
	CT(58608)=(S=12797396,E=12797399,C=14)
	CT(58609)=(S=12797400,E=12797403,C=18)
	CT(58610)=(S=12797404,E=12797407,C=31)
	CT(58611)=(S=12797408,E=12797419,C=36)
	CT(58612)=(S=12797420,E=12797423,C=18)
	CT(58613)=(S=12797424,E=12797427,C=28)
	CT(58614)=(S=12797428,E=12797435,C=31)
	CT(58615)=(S=12797440,E=12797471,C=34)
	CT(58616)=(S=12797472,E=12797503,C=50)
	CT(58617)=(S=12797504,E=12797535,C=14)
	CT(58618)=(S=12797536,E=12797567,C=42)
	CT(58619)=(S=12797568,E=12797599,C=33)
	CT(58620)=(S=12797600,E=12797631,C=27)
	CT(58621)=(S=12797632,E=12797663,C=18)
	CT(58622)=(S=12797664,E=12797695,C=42)
	CT(58623)=(S=12797696,E=12797951,C=28)
	CT(58624)=(S=12797952,E=12797983,C=71)
	CT(58625)=(S=12797984,E=12798015,C=14)
	CT(58626)=(S=12798016,E=12798047,C=49)
	CT(58627)=(S=12798048,E=12798063,C=28)
	CT(58628)=(S=12798064,E=12798064,C=51)
	CT(58629)=(S=12798065,E=12798067,C=14)
	CT(58630)=(S=12798068,E=12798075,C=17)
	CT(58631)=(S=12798076,E=12798079,C=36)
	CT(58632)=(S=12798080,E=12798083,C=14)
	CT(58633)=(S=12798084,E=12798087,C=42)
	CT(58634)=(S=12798088,E=12798091,C=21)
	CT(58635)=(S=12798092,E=12798095,C=18)
	CT(58636)=(S=12798096,E=12798099,C=31)
	CT(58637)=(S=12798100,E=12798103,C=58)
	CT(58638)=(S=12798104,E=12798107,C=12)
	CT(58639)=(S=12798108,E=12798111,C=31)
	CT(58640)=(S=12798112,E=12798143,C=14)
	CT(58641)=(S=12798144,E=12798175,C=15)
	CT(58642)=(S=12798176,E=12798207,C=18)
	CT(58643)=(S=12798208,E=12798463,C=28)
	CT(58644)=(S=12798464,E=12798495,C=47)
	CT(58645)=(S=12798496,E=12798499,C=24)
	CT(58646)=(S=12798500,E=12798503,C=19)
	CT(58647)=(S=12798504,E=12798511,C=28)
	CT(58648)=(S=12798512,E=12798515,C=58)
	CT(58649)=(S=12798516,E=12798519,C=14)
	CT(58650)=(S=12798520,E=12798523,C=58)
	CT(58651)=(S=12798524,E=12798525,C=14)
	CT(58652)=(S=12798526,E=12798527,C=18)
	CT(58653)=(S=12798528,E=12798528,C=58)
	CT(58654)=(S=12798529,E=12798529,C=36)
	CT(58655)=(S=12798530,E=12798530,C=19)
	CT(58656)=(S=12798531,E=12798531,C=31)
	CT(58657)=(S=12798532,E=12798532,C=14)
	CT(58658)=(S=12798533,E=12798533,C=58)
	CT(58659)=(S=12798534,E=12798534,C=28)
	CT(58660)=(S=12798535,E=12798535,C=38)
	CT(58661)=(S=12798536,E=12798536,C=31)
	CT(58662)=(S=12798537,E=12798537,C=28)
	CT(58663)=(S=12798538,E=12798538,C=18)
	CT(58664)=(S=12798539,E=12798539,C=15)
	CT(58665)=(S=12798540,E=12798540,C=33)
	CT(58666)=(S=12798541,E=12798541,C=27)
	CT(58667)=(S=12798542,E=12798542,C=38)
	CT(58668)=(S=12798543,E=12798543,C=58)
	CT(58669)=(S=12798544,E=12798544,C=12)
	CT(58670)=(S=12798545,E=12798545,C=15)
	CT(58671)=(S=12798546,E=12798546,C=18)
	CT(58672)=(S=12798547,E=12798547,C=14)
	CT(58673)=(S=12798548,E=12798548,C=18)
	CT(58674)=(S=12798549,E=12798549,C=51)
	CT(58675)=(S=12798550,E=12798550,C=28)
	CT(58676)=(S=12798551,E=12798551,C=36)
	CT(58677)=(S=12798555,E=12798556,C=58)
	CT(58678)=(S=12798557,E=12798557,C=14)
	CT(58679)=(S=12798558,E=12798558,C=28)
	CT(58680)=(S=12798559,E=12798559,C=18)
	CT(58681)=(S=12798560,E=12798623,C=14)
	CT(58682)=(S=12798624,E=12798655,C=13)
	CT(58683)=(S=12798656,E=12798687,C=49)
	CT(58684)=(S=12798688,E=12798719,C=22)
	CT(58685)=(S=12798720,E=12798975,C=13)
	CT(58686)=(S=12798976,E=12799487,C=25)
	CT(58687)=(S=12799488,E=12799519,C=187)
	CT(58688)=(S=12799520,E=12799523,C=57)
	CT(58689)=(S=12799524,E=12799525,C=19)
	CT(58690)=(S=12799526,E=12799527,C=31)
	CT(58691)=(S=12799528,E=12799529,C=28)
	CT(58692)=(S=12799530,E=12799531,C=31)
	CT(58693)=(S=12799532,E=12799533,C=71)
	CT(58694)=(S=12799534,E=12799535,C=42)
	CT(58695)=(S=12799538,E=12799539,C=49)
	CT(58696)=(S=12799540,E=12799541,C=42)
	CT(58697)=(S=12799546,E=12799547,C=31)
	CT(58698)=(S=12799548,E=12799549,C=18)
	CT(58699)=(S=12799550,E=12799551,C=28)
	CT(58700)=(S=12799552,E=12799553,C=38)
	CT(58701)=(S=12799554,E=12799555,C=58)
	CT(58702)=(S=12799556,E=12799557,C=31)
	CT(58703)=(S=12799558,E=12799559,C=47)
	CT(58704)=(S=12799560,E=12799561,C=18)
	CT(58705)=(S=12799562,E=12799563,C=101)
	CT(58706)=(S=12799564,E=12799565,C=28)
	CT(58707)=(S=12799566,E=12799567,C=25)
	CT(58708)=(S=12799568,E=12799571,C=14)
	CT(58709)=(S=12799572,E=12799573,C=36)
	CT(58710)=(S=12799574,E=12799575,C=22)
	CT(58711)=(S=12799576,E=12799577,C=38)
	CT(58712)=(S=12799578,E=12799579,C=18)
	CT(58713)=(S=12799580,E=12799581,C=31)
	CT(58714)=(S=12799582,E=12799583,C=14)
	CT(58715)=(S=12799584,E=12799585,C=31)
	CT(58716)=(S=12799586,E=12799587,C=42)
	CT(58717)=(S=12799588,E=12799589,C=31)
	CT(58718)=(S=12799590,E=12799591,C=18)
	CT(58719)=(S=12799592,E=12799593,C=31)
	CT(58720)=(S=12799594,E=12799595,C=38)
	CT(58721)=(S=12799596,E=12799597,C=13)
	CT(58722)=(S=12799598,E=12799599,C=14)
	CT(58723)=(S=12799600,E=12799601,C=31)
	CT(58724)=(S=12799602,E=12799607,C=18)
	CT(58725)=(S=12799608,E=12799609,C=36)
	CT(58726)=(S=12799610,E=12799611,C=33)
	CT(58727)=(S=12799612,E=12799613,C=38)
	CT(58728)=(S=12799614,E=12799615,C=18)
	CT(58729)=(S=12799616,E=12799647,C=19)
	CT(58730)=(S=12799648,E=12799679,C=28)
	CT(58731)=(S=12799680,E=12799711,C=15)
	CT(58732)=(S=12799712,E=12799715,C=19)
	CT(58733)=(S=12799716,E=12799719,C=25)
	CT(58734)=(S=12799720,E=12799723,C=31)
	CT(58735)=(S=12799724,E=12799727,C=58)
	CT(58736)=(S=12799728,E=12799731,C=28)
	CT(58737)=(S=12799732,E=12799735,C=31)
	CT(58738)=(S=12799736,E=12799739,C=28)
	CT(58739)=(S=12799740,E=12799743,C=31)
	CT(58740)=(S=12799744,E=12799999,C=13)
	CT(58741)=(S=12800000,E=12800095,C=14)
	CT(58742)=(S=12800096,E=12800127,C=55)
	CT(58743)=(S=12800128,E=12800135,C=58)
	CT(58744)=(S=12800136,E=12800139,C=18)
	CT(58745)=(S=12800140,E=12800141,C=31)
	CT(58746)=(S=12800142,E=12800143,C=38)
	CT(58747)=(S=12800144,E=12800147,C=18)
	CT(58748)=(S=12800148,E=12800151,C=58)
	CT(58749)=(S=12800152,E=12800159,C=12)
	CT(58750)=(S=12800160,E=12800191,C=71)
	CT(58751)=(S=12800192,E=12800223,C=28)
	CT(58752)=(S=12800224,E=12800224,C=18)
	CT(58753)=(S=12800225,E=12800225,C=52)
	CT(58754)=(S=12800226,E=12800226,C=36)
	CT(58755)=(S=12800227,E=12800227,C=101)
	CT(58756)=(S=12800228,E=12800228,C=18)
	CT(58757)=(S=12800229,E=12800229,C=58)
	CT(58758)=(S=12800232,E=12800232,C=31)
	CT(58759)=(S=12800233,E=12800233,C=34)
	CT(58760)=(S=12800234,E=12800234,C=42)
	CT(58761)=(S=12800235,E=12800235,C=34)
	CT(58762)=(S=12800236,E=12800237,C=58)
	CT(58763)=(S=12800238,E=12800238,C=49)
	CT(58764)=(S=12800239,E=12800239,C=28)
	CT(58765)=(S=12800240,E=12800255,C=24)
	CT(58766)=(S=12800256,E=12800511,C=14)
	CT(58767)=(S=12800512,E=12800543,C=20)
	CT(58768)=(S=12800544,E=12800607,C=28)
	CT(58769)=(S=12800608,E=12800639,C=14)
	CT(58770)=(S=12800640,E=12800641,C=17)
	CT(58771)=(S=12800642,E=12800643,C=38)
	CT(58772)=(S=12800646,E=12800649,C=18)
	CT(58773)=(S=12800650,E=12800651,C=104)
	CT(58774)=(S=12800652,E=12800659,C=18)
	CT(58775)=(S=12800660,E=12800661,C=38)
	CT(58776)=(S=12800662,E=12800663,C=31)
	CT(58777)=(S=12800664,E=12800665,C=36)
	CT(58778)=(S=12800666,E=12800669,C=18)
	CT(58779)=(S=12800670,E=12800671,C=28)
	CT(58780)=(S=12800672,E=12800703,C=58)
	CT(58781)=(S=12800704,E=12800735,C=24)
	CT(58782)=(S=12800736,E=12800767,C=42)
	CT(58783)=(S=12800768,E=12801023,C=12)
	CT(58784)=(S=12801024,E=12801279,C=19)
	CT(58785)=(S=12801280,E=12801407,C=24)
	CT(58786)=(S=12801408,E=12801471,C=36)
	CT(58787)=(S=12801472,E=12801472,C=52)
	CT(58788)=(S=12801473,E=12801473,C=14)
	CT(58789)=(S=12801475,E=12801476,C=58)
	CT(58790)=(S=12801478,E=12801478,C=31)
	CT(58791)=(S=12801479,E=12801479,C=58)
	CT(58792)=(S=12801480,E=12801480,C=25)
	CT(58793)=(S=12801482,E=12801482,C=28)
	CT(58794)=(S=12801483,E=12801484,C=49)
	CT(58795)=(S=12801486,E=12801486,C=18)
	CT(58796)=(S=12801488,E=12801488,C=31)
	CT(58797)=(S=12801489,E=12801489,C=58)
	CT(58798)=(S=12801490,E=12801490,C=28)
	CT(58799)=(S=12801491,E=12801491,C=38)
	CT(58800)=(S=12801492,E=12801492,C=15)
	CT(58801)=(S=12801493,E=12801493,C=18)
	CT(58802)=(S=12801494,E=12801494,C=31)
	CT(58803)=(S=12801495,E=12801495,C=51)
	CT(58804)=(S=12801497,E=12801497,C=28)
	CT(58805)=(S=12801499,E=12801499,C=31)
	CT(58806)=(S=12801500,E=12801500,C=28)
	CT(58807)=(S=12801501,E=12801501,C=21)
	CT(58808)=(S=12801502,E=12801502,C=28)
	CT(58809)=(S=12801504,E=12801504,C=23)
	CT(58810)=(S=12801505,E=12801505,C=18)
	CT(58811)=(S=12801507,E=12801507,C=58)
	CT(58812)=(S=12801508,E=12801508,C=15)
	CT(58813)=(S=12801509,E=12801509,C=28)
	CT(58814)=(S=12801510,E=12801510,C=58)
	CT(58815)=(S=12801511,E=12801511,C=18)
	CT(58816)=(S=12801512,E=12801512,C=14)
	CT(58817)=(S=12801514,E=12801516,C=18)
	CT(58818)=(S=12801517,E=12801517,C=28)
	CT(58819)=(S=12801518,E=12801518,C=18)
	CT(58820)=(S=12801519,E=12801519,C=38)
	CT(58821)=(S=12801520,E=12801520,C=28)
	CT(58822)=(S=12801521,E=12801521,C=58)
	CT(58823)=(S=12801522,E=12801522,C=37)
	CT(58824)=(S=12801523,E=12801523,C=58)
	CT(58825)=(S=12801525,E=12801525,C=14)
	CT(58826)=(S=12801526,E=12801526,C=49)
	CT(58827)=(S=12801527,E=12801527,C=12)
	CT(58828)=(S=12801529,E=12801529,C=58)
	CT(58829)=(S=12801530,E=12801530,C=31)
	CT(58830)=(S=12801531,E=12801531,C=24)
	CT(58831)=(S=12801532,E=12801532,C=58)
	CT(58832)=(S=12801533,E=12801533,C=24)
	CT(58833)=(S=12801534,E=12801534,C=28)
	CT(58834)=(S=12801535,E=12801535,C=37)
	CT(58835)=(S=12801536,E=12801791,C=36)
	CT(58836)=(S=12801792,E=12802047,C=37)
	CT(58837)=(S=12802048,E=12802051,C=18)
	CT(58838)=(S=12802052,E=12802055,C=15)
	CT(58839)=(S=12802056,E=12802057,C=14)
	CT(58840)=(S=12802058,E=12802059,C=42)
	CT(58841)=(S=12802060,E=12802061,C=46)
	CT(58842)=(S=12802062,E=12802063,C=31)
	CT(58843)=(S=12802064,E=12802065,C=18)
	CT(58844)=(S=12802066,E=12802067,C=36)
	CT(58845)=(S=12802068,E=12802069,C=14)
	CT(58846)=(S=12802070,E=12802071,C=18)
	CT(58847)=(S=12802074,E=12802075,C=31)
	CT(58848)=(S=12802076,E=12802079,C=58)
	CT(58849)=(S=12802080,E=12802081,C=36)
	CT(58850)=(S=12802082,E=12802083,C=42)
	CT(58851)=(S=12802084,E=12802085,C=24)
	CT(58852)=(S=12802086,E=12802087,C=38)
	CT(58853)=(S=12802088,E=12802089,C=18)
	CT(58854)=(S=12802090,E=12802091,C=31)
	CT(58855)=(S=12802092,E=12802093,C=28)
	CT(58856)=(S=12802094,E=12802095,C=57)
	CT(58857)=(S=12802096,E=12802097,C=42)
	CT(58858)=(S=12802098,E=12802099,C=58)
	CT(58859)=(S=12802100,E=12802101,C=31)
	CT(58860)=(S=12802102,E=12802103,C=27)
	CT(58861)=(S=12802106,E=12802107,C=18)
	CT(58862)=(S=12802108,E=12802109,C=28)
	CT(58863)=(S=12802110,E=12802111,C=18)
	CT(58864)=(S=12802112,E=12802113,C=38)
	CT(58865)=(S=12802114,E=12802115,C=66)
	CT(58866)=(S=12802116,E=12802117,C=18)
	CT(58867)=(S=12802118,E=12802119,C=14)
	CT(58868)=(S=12802120,E=12802121,C=31)
	CT(58869)=(S=12802122,E=12802123,C=51)
	CT(58870)=(S=12802124,E=12802127,C=31)
	CT(58871)=(S=12802128,E=12802129,C=38)
	CT(58872)=(S=12802130,E=12802131,C=52)
	CT(58873)=(S=12802132,E=12802133,C=12)
	CT(58874)=(S=12802134,E=12802135,C=14)
	CT(58875)=(S=12802136,E=12802137,C=28)
	CT(58876)=(S=12802138,E=12802139,C=58)
	CT(58877)=(S=12802140,E=12802141,C=18)
	CT(58878)=(S=12802142,E=12802143,C=12)
	CT(58879)=(S=12802144,E=12802147,C=38)
	CT(58880)=(S=12802148,E=12802149,C=14)
	CT(58881)=(S=12802150,E=12802151,C=18)
	CT(58882)=(S=12802154,E=12802155,C=27)
	CT(58883)=(S=12802156,E=12802157,C=28)
	CT(58884)=(S=12802158,E=12802159,C=19)
	CT(58885)=(S=12802160,E=12802161,C=31)
	CT(58886)=(S=12802162,E=12802163,C=58)
	CT(58887)=(S=12802164,E=12802165,C=28)
	CT(58888)=(S=12802166,E=12802167,C=14)
	CT(58889)=(S=12802168,E=12802169,C=18)
	CT(58890)=(S=12802170,E=12802171,C=54)
	CT(58891)=(S=12802172,E=12802173,C=31)
	CT(58892)=(S=12802174,E=12802175,C=18)
	CT(58893)=(S=12802176,E=12802177,C=58)
	CT(58894)=(S=12802178,E=12802179,C=19)
	CT(58895)=(S=12802180,E=12802183,C=18)
	CT(58896)=(S=12802184,E=12802185,C=36)
	CT(58897)=(S=12802186,E=12802187,C=31)
	CT(58898)=(S=12802188,E=12802189,C=51)
	CT(58899)=(S=12802190,E=12802191,C=33)
	CT(58900)=(S=12802192,E=12802193,C=58)
	CT(58901)=(S=12802194,E=12802195,C=18)
	CT(58902)=(S=12802196,E=12802201,C=25)
	CT(58903)=(S=12802202,E=12802203,C=18)
	CT(58904)=(S=12802204,E=12802205,C=36)
	CT(58905)=(S=12802206,E=12802207,C=31)
	CT(58906)=(S=12802208,E=12802209,C=36)
	CT(58907)=(S=12802210,E=12802211,C=22)
	CT(58908)=(S=12802212,E=12802213,C=33)
	CT(58909)=(S=12802214,E=12802215,C=42)
	CT(58910)=(S=12802218,E=12802219,C=18)
	CT(58911)=(S=12802220,E=12802223,C=38)
	CT(58912)=(S=12802224,E=12802225,C=32)
	CT(58913)=(S=12802226,E=12802227,C=18)
	CT(58914)=(S=12802228,E=12802228,C=58)
	CT(58915)=(S=12802229,E=12802229,C=18)
	CT(58916)=(S=12802230,E=12802231,C=38)
	CT(58917)=(S=12802232,E=12802233,C=18)
	CT(58918)=(S=12802234,E=12802235,C=58)
	CT(58919)=(S=12802236,E=12802237,C=26)
	CT(58920)=(S=12802238,E=12802241,C=18)
	CT(58921)=(S=12802242,E=12802243,C=12)
	CT(58922)=(S=12802244,E=12802249,C=18)
	CT(58923)=(S=12802250,E=12802251,C=36)
	CT(58924)=(S=12802254,E=12802257,C=18)
	CT(58925)=(S=12802260,E=12802261,C=57)
	CT(58926)=(S=12802262,E=12802263,C=98)
	CT(58927)=(S=12802264,E=12802265,C=46)
	CT(58928)=(S=12802268,E=12802269,C=18)
	CT(58929)=(S=12802270,E=12802271,C=101)
	CT(58930)=(S=12802272,E=12802273,C=18)
	CT(58931)=(S=12802276,E=12802277,C=14)
	CT(58932)=(S=12802280,E=12802281,C=58)
	CT(58933)=(S=12802284,E=12802285,C=14)
	CT(58934)=(S=12802286,E=12802287,C=42)
	CT(58935)=(S=12802288,E=12802289,C=18)
	CT(58936)=(S=12802290,E=12802291,C=31)
	CT(58937)=(S=12802292,E=12802293,C=23)
	CT(58938)=(S=12802294,E=12802297,C=18)
	CT(58939)=(S=12802298,E=12802298,C=38)
	CT(58940)=(S=12802299,E=12802299,C=50)
	CT(58941)=(S=12802300,E=12802301,C=18)
	CT(58942)=(S=12802302,E=12802303,C=66)
	CT(58943)=(S=12802304,E=12802559,C=14)
	CT(58944)=(S=12802560,E=12802591,C=28)
	CT(58945)=(S=12802592,E=12802623,C=13)
	CT(58946)=(S=12802624,E=12802655,C=36)
	CT(58947)=(S=12802656,E=12802657,C=14)
	CT(58948)=(S=12802660,E=12802661,C=24)
	CT(58949)=(S=12802662,E=12802663,C=58)
	CT(58950)=(S=12802664,E=12802665,C=19)
	CT(58951)=(S=12802666,E=12802667,C=58)
	CT(58952)=(S=12802668,E=12802669,C=15)
	CT(58953)=(S=12802670,E=12802671,C=38)
	CT(58954)=(S=12802672,E=12802673,C=34)
	CT(58955)=(S=12802680,E=12802681,C=14)
	CT(58956)=(S=12802684,E=12802685,C=38)
	CT(58957)=(S=12802686,E=12802687,C=58)
	CT(58958)=(S=12802688,E=12802751,C=18)
	CT(58959)=(S=12802752,E=12802815,C=28)
	CT(58960)=(S=12802816,E=12802943,C=71)
	CT(58961)=(S=12802944,E=12803071,C=18)
	CT(58962)=(S=12803072,E=12803455,C=14)
	CT(58963)=(S=12803456,E=12803457,C=18)
	CT(58964)=(S=12803458,E=12803459,C=28)
	CT(58965)=(S=12803460,E=12803461,C=63)
	CT(58966)=(S=12803462,E=12803463,C=58)
	CT(58967)=(S=12803464,E=12803465,C=38)
	CT(58968)=(S=12803466,E=12803467,C=31)
	CT(58969)=(S=12803468,E=12803469,C=38)
	CT(58970)=(S=12803470,E=12803471,C=58)
	CT(58971)=(S=12803472,E=12803473,C=18)
	CT(58972)=(S=12803474,E=12803475,C=42)
	CT(58973)=(S=12803476,E=12803479,C=18)
	CT(58974)=(S=12803480,E=12803481,C=20)
	CT(58975)=(S=12803482,E=12803483,C=31)
	CT(58976)=(S=12803484,E=12803485,C=14)
	CT(58977)=(S=12803486,E=12803487,C=28)
	CT(58978)=(S=12803490,E=12803491,C=58)
	CT(58979)=(S=12803492,E=12803493,C=18)
	CT(58980)=(S=12803494,E=12803495,C=28)
	CT(58981)=(S=12803496,E=12803497,C=15)
	CT(58982)=(S=12803498,E=12803499,C=25)
	CT(58983)=(S=12803500,E=12803501,C=31)
	CT(58984)=(S=12803502,E=12803503,C=42)
	CT(58985)=(S=12803506,E=12803507,C=58)
	CT(58986)=(S=12803508,E=12803509,C=18)
	CT(58987)=(S=12803510,E=12803511,C=28)
	CT(58988)=(S=12803514,E=12803517,C=18)
	CT(58989)=(S=12803518,E=12803519,C=31)
	CT(58990)=(S=12803520,E=12803521,C=24)
	CT(58991)=(S=12803522,E=12803523,C=42)
	CT(58992)=(S=12803524,E=12803525,C=13)
	CT(58993)=(S=12803526,E=12803529,C=28)
	CT(58994)=(S=12803530,E=12803531,C=58)
	CT(58995)=(S=12803532,E=12803533,C=31)
	CT(58996)=(S=12803534,E=12803535,C=18)
	CT(58997)=(S=12803538,E=12803539,C=25)
	CT(58998)=(S=12803540,E=12803541,C=31)
	CT(58999)=(S=12803544,E=12803545,C=33)
	CT(59000)=(S=12803546,E=12803547,C=53)
	CT(59001)=(S=12803548,E=12803549,C=42)
	CT(59002)=(S=12803550,E=12803551,C=58)
	CT(59003)=(S=12803552,E=12803553,C=36)
	CT(59004)=(S=12803554,E=12803555,C=14)
	CT(59005)=(S=12803556,E=12803557,C=18)
	CT(59006)=(S=12803558,E=12803559,C=58)
	CT(59007)=(S=12803560,E=12803561,C=27)
	CT(59008)=(S=12803562,E=12803563,C=17)
	CT(59009)=(S=12803564,E=12803565,C=58)
	CT(59010)=(S=12803566,E=12803567,C=36)
	CT(59011)=(S=12803568,E=12803569,C=18)
	CT(59012)=(S=12803570,E=12803573,C=28)
	CT(59013)=(S=12803574,E=12803577,C=18)
	CT(59014)=(S=12803578,E=12803579,C=58)
	CT(59015)=(S=12803580,E=12803581,C=19)
	CT(59016)=(S=12803582,E=12803583,C=15)
	CT(59017)=(S=12803584,E=12803615,C=67)
	CT(59018)=(S=12803616,E=12803647,C=34)
	CT(59019)=(S=12803648,E=12803679,C=28)
	CT(59020)=(S=12803680,E=12803691,C=47)
	CT(59021)=(S=12803692,E=12803695,C=34)
	CT(59022)=(S=12803696,E=12803711,C=14)
	CT(59023)=(S=12803712,E=12803775,C=15)
	CT(59024)=(S=12803776,E=12803807,C=58)
	CT(59025)=(S=12803808,E=12803839,C=18)
	CT(59026)=(S=12803840,E=12803967,C=49)
	CT(59027)=(S=12803969,E=12803969,C=58)
	CT(59028)=(S=12803971,E=12803971,C=14)
	CT(59029)=(S=12803972,E=12803972,C=34)
	CT(59030)=(S=12803973,E=12803973,C=14)
	CT(59031)=(S=12803975,E=12803975,C=12)
	CT(59032)=(S=12803976,E=12803976,C=49)
	CT(59033)=(S=12803977,E=12803977,C=28)
	CT(59034)=(S=12803978,E=12803978,C=32)
	CT(59035)=(S=12803979,E=12803979,C=31)
	CT(59036)=(S=12803980,E=12803980,C=19)
	CT(59037)=(S=12803981,E=12803981,C=34)
	CT(59038)=(S=12803982,E=12803982,C=15)
	CT(59039)=(S=12803983,E=12803983,C=14)
	CT(59040)=(S=12803984,E=12803984,C=58)
	CT(59041)=(S=12803985,E=12803985,C=38)
	CT(59042)=(S=12803986,E=12803986,C=28)
	CT(59043)=(S=12803987,E=12803987,C=31)
	CT(59044)=(S=12803988,E=12803988,C=34)
	CT(59045)=(S=12803989,E=12803989,C=37)
	CT(59046)=(S=12803990,E=12803990,C=36)
	CT(59047)=(S=12803991,E=12803991,C=31)
	CT(59048)=(S=12803992,E=12803992,C=38)
	CT(59049)=(S=12803993,E=12803993,C=25)
	CT(59050)=(S=12803994,E=12803994,C=58)
	CT(59051)=(S=12803996,E=12803996,C=25)
	CT(59052)=(S=12803997,E=12803997,C=31)
	CT(59053)=(S=12803998,E=12803998,C=52)
	CT(59054)=(S=12803999,E=12803999,C=31)
	CT(59055)=(S=12804000,E=12804000,C=37)
	CT(59056)=(S=12804001,E=12804001,C=52)
	CT(59057)=(S=12804002,E=12804003,C=42)
	CT(59058)=(S=12804004,E=12804004,C=58)
	CT(59059)=(S=12804005,E=12804005,C=31)
	CT(59060)=(S=12804006,E=12804007,C=38)
	CT(59061)=(S=12804008,E=12804008,C=12)
	CT(59062)=(S=12804009,E=12804010,C=58)
	CT(59063)=(S=12804011,E=12804011,C=31)
	CT(59064)=(S=12804012,E=12804012,C=24)
	CT(59065)=(S=12804013,E=12804013,C=52)
	CT(59066)=(S=12804014,E=12804015,C=36)
	CT(59067)=(S=12804016,E=12804016,C=58)
	CT(59068)=(S=12804017,E=12804017,C=36)
	CT(59069)=(S=12804018,E=12804018,C=38)
	CT(59070)=(S=12804019,E=12804019,C=37)
	CT(59071)=(S=12804020,E=12804020,C=38)
	CT(59072)=(S=12804021,E=12804021,C=24)
	CT(59073)=(S=12804022,E=12804022,C=58)
	CT(59074)=(S=12804023,E=12804023,C=17)
	CT(59075)=(S=12804024,E=12804024,C=19)
	CT(59076)=(S=12804026,E=12804026,C=58)
	CT(59077)=(S=12804027,E=12804027,C=14)
	CT(59078)=(S=12804028,E=12804028,C=58)
	CT(59079)=(S=12804029,E=12804029,C=31)
	CT(59080)=(S=12804030,E=12804031,C=63)
	CT(59081)=(S=12804032,E=12804035,C=13)
	CT(59082)=(S=12804036,E=12804037,C=49)
	CT(59083)=(S=12804038,E=12804039,C=36)
	CT(59084)=(S=12804042,E=12804043,C=31)
	CT(59085)=(S=12804044,E=12804045,C=71)
	CT(59086)=(S=12804046,E=12804047,C=31)
	CT(59087)=(S=12804048,E=12804049,C=47)
	CT(59088)=(S=12804050,E=12804051,C=31)
	CT(59089)=(S=12804052,E=12804058,C=18)
	CT(59090)=(S=12804059,E=12804059,C=16)
	CT(59091)=(S=12804060,E=12804061,C=28)
	CT(59092)=(S=12804062,E=12804063,C=31)
	CT(59093)=(S=12804064,E=12804065,C=12)
	CT(59094)=(S=12804066,E=12804067,C=22)
	CT(59095)=(S=12804068,E=12804069,C=38)
	CT(59096)=(S=12804070,E=12804071,C=15)
	CT(59097)=(S=12804072,E=12804073,C=31)
	CT(59098)=(S=12804074,E=12804075,C=58)
	CT(59099)=(S=12804076,E=12804077,C=21)
	CT(59100)=(S=12804078,E=12804083,C=24)
	CT(59101)=(S=12804084,E=12804089,C=36)
	CT(59102)=(S=12804092,E=12804093,C=18)
	CT(59103)=(S=12804094,E=12804095,C=38)
	CT(59104)=(S=12804096,E=12804127,C=42)
	CT(59105)=(S=12804128,E=12804159,C=28)
	CT(59106)=(S=12804160,E=12804191,C=18)
	CT(59107)=(S=12804192,E=12804223,C=36)
	CT(59108)=(S=12804224,E=12804255,C=15)
	CT(59109)=(S=12804256,E=12804287,C=18)
	CT(59110)=(S=12804288,E=12804319,C=15)
	CT(59111)=(S=12804320,E=12804351,C=51)
	CT(59112)=(S=12804352,E=12804479,C=22)
	CT(59113)=(S=12804480,E=12804543,C=24)
	CT(59114)=(S=12804544,E=12804607,C=14)
	CT(59115)=(S=12804608,E=12804639,C=71)
	CT(59116)=(S=12804640,E=12804703,C=18)
	CT(59117)=(S=12804704,E=12804735,C=12)
	CT(59118)=(S=12804736,E=12804767,C=71)
	CT(59119)=(S=12804768,E=12804799,C=18)
	CT(59120)=(S=12804800,E=12804831,C=28)
	CT(59121)=(S=12804832,E=12804863,C=12)
	CT(59122)=(S=12804864,E=12805119,C=14)
	CT(59123)=(S=12805120,E=12805375,C=19)
	CT(59124)=(S=12805376,E=12805631,C=12)
	CT(59125)=(S=12805632,E=12805887,C=14)
	CT(59126)=(S=12805888,E=12806143,C=15)
	CT(59127)=(S=12806144,E=12807167,C=14)
	CT(59128)=(S=12807168,E=12807679,C=36)
	CT(59129)=(S=12807684,E=12807685,C=38)
	CT(59130)=(S=12807686,E=12807687,C=31)
	CT(59131)=(S=12807688,E=12807691,C=14)
	CT(59132)=(S=12807692,E=12807693,C=12)
	CT(59133)=(S=12807694,E=12807695,C=28)
	CT(59134)=(S=12807696,E=12807697,C=24)
	CT(59135)=(S=12807698,E=12807701,C=28)
	CT(59136)=(S=12807704,E=12807705,C=51)
	CT(59137)=(S=12807706,E=12807707,C=28)
	CT(59138)=(S=12807708,E=12807709,C=27)
	CT(59139)=(S=12807710,E=12807711,C=14)
	CT(59140)=(S=12807712,E=12807713,C=18)
	CT(59141)=(S=12807714,E=12807715,C=12)
	CT(59142)=(S=12807716,E=12807717,C=14)
	CT(59143)=(S=12807718,E=12807719,C=26)
	CT(59144)=(S=12807720,E=12807721,C=17)
	CT(59145)=(S=12807722,E=12807723,C=28)
	CT(59146)=(S=12807724,E=12807725,C=19)
	CT(59147)=(S=12807726,E=12807727,C=18)
	CT(59148)=(S=12807728,E=12807731,C=58)
	CT(59149)=(S=12807732,E=12807733,C=18)
	CT(59150)=(S=12807734,E=12807735,C=49)
	CT(59151)=(S=12807736,E=12807737,C=31)
	CT(59152)=(S=12807738,E=12807739,C=44)
	CT(59153)=(S=12807740,E=12807741,C=28)
	CT(59154)=(S=12807742,E=12807743,C=18)
	CT(59155)=(S=12807744,E=12807775,C=14)
	CT(59156)=(S=12807776,E=12807839,C=15)
	CT(59157)=(S=12807840,E=12807871,C=14)
	CT(59158)=(S=12807872,E=12807903,C=42)
	CT(59159)=(S=12807904,E=12807935,C=12)
	CT(59160)=(S=12807936,E=12808191,C=50)
	CT(59161)=(S=12808192,E=12808255,C=14)
	CT(59162)=(S=12808256,E=12808287,C=34)
	CT(59163)=(S=12808288,E=12808319,C=18)
	CT(59164)=(S=12808320,E=12808351,C=37)
	CT(59165)=(S=12808352,E=12808383,C=28)
	CT(59166)=(S=12808384,E=12808415,C=39)
	CT(59167)=(S=12808416,E=12808447,C=18)
	CT(59168)=(S=12808448,E=12808703,C=33)
	CT(59169)=(S=12808704,E=12808705,C=58)
	CT(59170)=(S=12808706,E=12808707,C=18)
	CT(59171)=(S=12808710,E=12808711,C=31)
	CT(59172)=(S=12808714,E=12808715,C=28)
	CT(59173)=(S=12808716,E=12808717,C=36)
	CT(59174)=(S=12808720,E=12808721,C=50)
	CT(59175)=(S=12808722,E=12808723,C=12)
	CT(59176)=(S=12808724,E=12808725,C=18)
	CT(59177)=(S=12808726,E=12808727,C=28)
	CT(59178)=(S=12808728,E=12808731,C=12)
	CT(59179)=(S=12808732,E=12808733,C=43)
	CT(59180)=(S=12808734,E=12808735,C=31)
	CT(59181)=(S=12808736,E=12808767,C=60)
	CT(59182)=(S=12808768,E=12808799,C=12)
	CT(59183)=(S=12808800,E=12808801,C=31)
	CT(59184)=(S=12808802,E=12808803,C=28)
	CT(59185)=(S=12808804,E=12808805,C=36)
	CT(59186)=(S=12808806,E=12808807,C=14)
	CT(59187)=(S=12808808,E=12808809,C=18)
	CT(59188)=(S=12808810,E=12808811,C=23)
	CT(59189)=(S=12808816,E=12808816,C=18)
	CT(59190)=(S=12808817,E=12808817,C=14)
	CT(59191)=(S=12808818,E=12808819,C=12)
	CT(59192)=(S=12808824,E=12808825,C=31)
	CT(59193)=(S=12808826,E=12808829,C=18)
	CT(59194)=(S=12808832,E=12808863,C=31)
	CT(59195)=(S=12808864,E=12808895,C=58)
	CT(59196)=(S=12808896,E=12808927,C=25)
	CT(59197)=(S=12808928,E=12808943,C=36)
	CT(59198)=(S=12808944,E=12808959,C=62)
	CT(59199)=(S=12808960,E=12809215,C=12)
	CT(59200)=(S=12809216,E=12809727,C=58)
	CT(59201)=(S=12809728,E=12810239,C=13)
	CT(59202)=(S=12810240,E=12810495,C=15)
	CT(59203)=(S=12810496,E=12810751,C=36)
	CT(59204)=(S=12810752,E=12810783,C=60)
	CT(59205)=(S=12810784,E=12810815,C=14)
	CT(59206)=(S=12810816,E=12810847,C=15)
	CT(59207)=(S=12810848,E=12810879,C=49)
	CT(59208)=(S=12810880,E=12810943,C=14)
	CT(59209)=(S=12810944,E=12810975,C=33)
	CT(59210)=(S=12810976,E=12811007,C=18)
	CT(59211)=(S=12811008,E=12811263,C=31)
	CT(59212)=(S=12811264,E=12812291,C=28)
	CT(59213)=(S=12812292,E=12812295,C=19)
	CT(59214)=(S=12812296,E=12812299,C=36)
	CT(59215)=(S=12812304,E=12812307,C=31)
	CT(59216)=(S=12812308,E=12812311,C=28)
	CT(59217)=(S=12812316,E=12812319,C=58)
	CT(59218)=(S=12812320,E=12812327,C=37)
	CT(59219)=(S=12812328,E=12812335,C=28)
	CT(59220)=(S=12812336,E=12812383,C=18)
	CT(59221)=(S=12812392,E=12812399,C=42)
	CT(59222)=(S=12812400,E=12812407,C=58)
	CT(59223)=(S=12812408,E=12812415,C=18)
	CT(59224)=(S=12812416,E=12812416,C=52)
	CT(59225)=(S=12812417,E=12812417,C=36)
	CT(59226)=(S=12812418,E=12812418,C=38)
	CT(59227)=(S=12812419,E=12812419,C=23)
	CT(59228)=(S=12812420,E=12812420,C=51)
	CT(59229)=(S=12812421,E=12812421,C=18)
	CT(59230)=(S=12812422,E=12812422,C=51)
	CT(59231)=(S=12812423,E=12812423,C=58)
	CT(59232)=(S=12812424,E=12812424,C=38)
	CT(59233)=(S=12812425,E=12812425,C=18)
	CT(59234)=(S=12812426,E=12812426,C=14)
	CT(59235)=(S=12812427,E=12812427,C=31)
	CT(59236)=(S=12812428,E=12812428,C=58)
	CT(59237)=(S=12812429,E=12812429,C=52)
	CT(59238)=(S=12812430,E=12812430,C=58)
	CT(59239)=(S=12812431,E=12812431,C=12)
	CT(59240)=(S=12812432,E=12812432,C=28)
	CT(59241)=(S=12812433,E=12812433,C=17)
	CT(59242)=(S=12812434,E=12812435,C=12)
	CT(59243)=(S=12812436,E=12812436,C=58)
	CT(59244)=(S=12812437,E=12812437,C=12)
	CT(59245)=(S=12812438,E=12812438,C=42)
	CT(59246)=(S=12812439,E=12812439,C=38)
	CT(59247)=(S=12812440,E=12812441,C=18)
	CT(59248)=(S=12812442,E=12812442,C=58)
	CT(59249)=(S=12812443,E=12812443,C=33)
	CT(59250)=(S=12812444,E=12812444,C=28)
	CT(59251)=(S=12812445,E=12812447,C=18)
	CT(59252)=(S=12812448,E=12812449,C=36)
	CT(59253)=(S=12812452,E=12812453,C=38)
	CT(59254)=(S=12812454,E=12812455,C=66)
	CT(59255)=(S=12812456,E=12812457,C=38)
	CT(59256)=(S=12812458,E=12812459,C=42)
	CT(59257)=(S=12812460,E=12812461,C=58)
	CT(59258)=(S=12812462,E=12812463,C=24)
	CT(59259)=(S=12812464,E=12812465,C=17)
	CT(59260)=(S=12812468,E=12812469,C=58)
	CT(59261)=(S=12812470,E=12812471,C=31)
	CT(59262)=(S=12812472,E=12812475,C=36)
	CT(59263)=(S=12812476,E=12812477,C=38)
	CT(59264)=(S=12812478,E=12812479,C=14)
	CT(59265)=(S=12812480,E=12812511,C=33)
	CT(59266)=(S=12812512,E=12812513,C=51)
	CT(59267)=(S=12812514,E=12812515,C=14)
	CT(59268)=(S=12812516,E=12812517,C=58)
	CT(59269)=(S=12812518,E=12812519,C=31)
	CT(59270)=(S=12812520,E=12812521,C=50)
	CT(59271)=(S=12812522,E=12812523,C=15)
	CT(59272)=(S=12812524,E=12812525,C=24)
	CT(59273)=(S=12812526,E=12812527,C=14)
	CT(59274)=(S=12812530,E=12812531,C=58)
	CT(59275)=(S=12812532,E=12812535,C=18)
	CT(59276)=(S=12812536,E=12812537,C=31)
	CT(59277)=(S=12812538,E=12812539,C=14)
	CT(59278)=(S=12812540,E=12812541,C=31)
	CT(59279)=(S=12812544,E=12812799,C=13)
	CT(59280)=(S=12812800,E=12812831,C=14)
	CT(59281)=(S=12812832,E=12812863,C=59)
	CT(59282)=(S=12812864,E=12812927,C=22)
	CT(59283)=(S=12812928,E=12812959,C=49)
	CT(59284)=(S=12812960,E=12812991,C=34)
	CT(59285)=(S=12812993,E=12812993,C=48)
	CT(59286)=(S=12812994,E=12812994,C=52)
	CT(59287)=(S=12812995,E=12812995,C=15)
	CT(59288)=(S=12812996,E=12812996,C=34)
	CT(59289)=(S=12812997,E=12812997,C=58)
	CT(59290)=(S=12812998,E=12812998,C=42)
	CT(59291)=(S=12812999,E=12812999,C=13)
	CT(59292)=(S=12813000,E=12813000,C=19)
	CT(59293)=(S=12813001,E=12813001,C=14)
	CT(59294)=(S=12813002,E=12813003,C=50)
	CT(59295)=(S=12813005,E=12813005,C=60)
	CT(59296)=(S=12813007,E=12813007,C=12)
	CT(59297)=(S=12813009,E=12813009,C=18)
	CT(59298)=(S=12813010,E=12813010,C=14)
	CT(59299)=(S=12813011,E=12813011,C=48)
	CT(59300)=(S=12813012,E=12813012,C=58)
	CT(59301)=(S=12813013,E=12813013,C=15)
	CT(59302)=(S=12813014,E=12813014,C=18)
	CT(59303)=(S=12813015,E=12813015,C=38)
	CT(59304)=(S=12813016,E=12813016,C=18)
	CT(59305)=(S=12813017,E=12813017,C=14)
	CT(59306)=(S=12813018,E=12813018,C=34)
	CT(59307)=(S=12813019,E=12813019,C=31)
	CT(59308)=(S=12813020,E=12813020,C=15)
	CT(59309)=(S=12813021,E=12813021,C=28)
	CT(59310)=(S=12813022,E=12813022,C=12)
	CT(59311)=(S=12813023,E=12813023,C=36)
	CT(59312)=(S=12813024,E=12813055,C=15)
	CT(59313)=(S=12813056,E=12813311,C=18)
	CT(59314)=(S=12813312,E=12813567,C=12)
	CT(59315)=(S=12813568,E=12813823,C=18)
	CT(59316)=(S=12813824,E=12813855,C=14)
	CT(59317)=(S=12813856,E=12813887,C=27)
	CT(59318)=(S=12813888,E=12813951,C=22)
	CT(59319)=(S=12813952,E=12813983,C=34)
	CT(59320)=(S=12813984,E=12813991,C=12)
	CT(59321)=(S=12813992,E=12813999,C=36)
	CT(59322)=(S=12814000,E=12814007,C=15)
	CT(59323)=(S=12814008,E=12814015,C=23)
	CT(59324)=(S=12814016,E=12814047,C=14)
	CT(59325)=(S=12814048,E=12814079,C=47)
	CT(59326)=(S=12814080,E=12814207,C=15)
	CT(59327)=(S=12814208,E=12814271,C=28)
	CT(59328)=(S=12814272,E=12814275,C=38)
	CT(59329)=(S=12814276,E=12814279,C=31)
	CT(59330)=(S=12814280,E=12814283,C=15)
	CT(59331)=(S=12814284,E=12814287,C=18)
	CT(59332)=(S=12814288,E=12814291,C=14)
	CT(59333)=(S=12814292,E=12814295,C=18)
	CT(59334)=(S=12814296,E=12814299,C=24)
	CT(59335)=(S=12814300,E=12814307,C=28)
	CT(59336)=(S=12814308,E=12814311,C=58)
	CT(59337)=(S=12814312,E=12814323,C=18)
	CT(59338)=(S=12814328,E=12814331,C=25)
	CT(59339)=(S=12814336,E=12814591,C=58)
	CT(59340)=(S=12814592,E=12814719,C=14)
	CT(59341)=(S=12814720,E=12814751,C=15)
	CT(59342)=(S=12814752,E=12814752,C=68)
	CT(59343)=(S=12814753,E=12814754,C=58)
	CT(59344)=(S=12814755,E=12814755,C=28)
	CT(59345)=(S=12814757,E=12814757,C=36)
	CT(59346)=(S=12814758,E=12814758,C=38)
	CT(59347)=(S=12814759,E=12814759,C=31)
	CT(59348)=(S=12814761,E=12814761,C=36)
	CT(59349)=(S=12814762,E=12814762,C=28)
	CT(59350)=(S=12814763,E=12814763,C=18)
	CT(59351)=(S=12814764,E=12814764,C=34)
	CT(59352)=(S=12814765,E=12814765,C=57)
	CT(59353)=(S=12814766,E=12814766,C=101)
	CT(59354)=(S=12814767,E=12814767,C=34)
	CT(59355)=(S=12814768,E=12814768,C=14)
	CT(59356)=(S=12814769,E=12814769,C=28)
	CT(59357)=(S=12814770,E=12814770,C=24)
	CT(59358)=(S=12814771,E=12814771,C=55)
	CT(59359)=(S=12814772,E=12814772,C=68)
	CT(59360)=(S=12814773,E=12814773,C=12)
	CT(59361)=(S=12814774,E=12814774,C=33)
	CT(59362)=(S=12814775,E=12814775,C=24)
	CT(59363)=(S=12814776,E=12814776,C=12)
	CT(59364)=(S=12814777,E=12814777,C=31)
	CT(59365)=(S=12814779,E=12814779,C=18)
	CT(59366)=(S=12814781,E=12814781,C=58)
	CT(59367)=(S=12814782,E=12814782,C=101)
	CT(59368)=(S=12814783,E=12814783,C=28)
	CT(59369)=(S=12814786,E=12814787,C=24)
	CT(59370)=(S=12814788,E=12814789,C=31)
	CT(59371)=(S=12814790,E=12814791,C=28)
	CT(59372)=(S=12814792,E=12814793,C=18)
	CT(59373)=(S=12814794,E=12814799,C=31)
	CT(59374)=(S=12814800,E=12814801,C=58)
	CT(59375)=(S=12814802,E=12814805,C=28)
	CT(59376)=(S=12814806,E=12814807,C=36)
	CT(59377)=(S=12814808,E=12814809,C=28)
	CT(59378)=(S=12814810,E=12814811,C=31)
	CT(59379)=(S=12814812,E=12814813,C=19)
	CT(59380)=(S=12814814,E=12814815,C=37)
	CT(59381)=(S=12814816,E=12814817,C=28)
	CT(59382)=(S=12814818,E=12814819,C=31)
	CT(59383)=(S=12814820,E=12814821,C=70)
	CT(59384)=(S=12814822,E=12814823,C=36)
	CT(59385)=(S=12814824,E=12814825,C=31)
	CT(59386)=(S=12814826,E=12814827,C=28)
	CT(59387)=(S=12814828,E=12814829,C=24)
	CT(59388)=(S=12814830,E=12814831,C=42)
	CT(59389)=(S=12814832,E=12814833,C=31)
	CT(59390)=(S=12814834,E=12814834,C=36)
	CT(59391)=(S=12814835,E=12814835,C=49)
	CT(59392)=(S=12814836,E=12814837,C=31)
	CT(59393)=(S=12814838,E=12814839,C=58)
	CT(59394)=(S=12814840,E=12814841,C=12)
	CT(59395)=(S=12814842,E=12814843,C=31)
	CT(59396)=(S=12814844,E=12814845,C=51)
	CT(59397)=(S=12814846,E=12814847,C=31)
	CT(59398)=(S=12814848,E=12814879,C=21)
	CT(59399)=(S=12814880,E=12814911,C=28)
	CT(59400)=(S=12814912,E=12814943,C=31)
	CT(59401)=(S=12814944,E=12814975,C=53)
	CT(59402)=(S=12814976,E=12815007,C=51)
	CT(59403)=(S=12815008,E=12815039,C=31)
	CT(59404)=(S=12815040,E=12815040,C=38)
	CT(59405)=(S=12815041,E=12815041,C=31)
	CT(59406)=(S=12815042,E=12815043,C=14)
	CT(59407)=(S=12815044,E=12815044,C=52)
	CT(59408)=(S=12815045,E=12815045,C=12)
	CT(59409)=(S=12815046,E=12815046,C=31)
	CT(59410)=(S=12815047,E=12815047,C=27)
	CT(59411)=(S=12815048,E=12815048,C=58)
	CT(59412)=(S=12815049,E=12815049,C=52)
	CT(59413)=(S=12815050,E=12815050,C=12)
	CT(59414)=(S=12815051,E=12815051,C=34)
	CT(59415)=(S=12815052,E=12815053,C=14)
	CT(59416)=(S=12815054,E=12815054,C=18)
	CT(59417)=(S=12815055,E=12815056,C=58)
	CT(59418)=(S=12815057,E=12815057,C=25)
	CT(59419)=(S=12815058,E=12815058,C=18)
	CT(59420)=(S=12815059,E=12815059,C=58)
	CT(59421)=(S=12815060,E=12815060,C=19)
	CT(59422)=(S=12815061,E=12815061,C=47)
	CT(59423)=(S=12815062,E=12815062,C=38)
	CT(59424)=(S=12815063,E=12815064,C=34)
	CT(59425)=(S=12815065,E=12815067,C=31)
	CT(59426)=(S=12815069,E=12815069,C=19)
	CT(59427)=(S=12815070,E=12815070,C=38)
	CT(59428)=(S=12815071,E=12815071,C=28)
	CT(59429)=(S=12815072,E=12815079,C=72)
	CT(59430)=(S=12815080,E=12815087,C=18)
	CT(59431)=(S=12815088,E=12815103,C=28)
	CT(59432)=(S=12815104,E=12815359,C=27)
	CT(59433)=(S=12815360,E=12815487,C=28)
	CT(59434)=(S=12815492,E=12815495,C=24)
	CT(59435)=(S=12815496,E=12815499,C=42)
	CT(59436)=(S=12815500,E=12815511,C=12)
	CT(59437)=(S=12815512,E=12815515,C=58)
	CT(59438)=(S=12815516,E=12815519,C=25)
	CT(59439)=(S=12815520,E=12815523,C=31)
	CT(59440)=(S=12815524,E=12815527,C=102)
	CT(59441)=(S=12815528,E=12815531,C=31)
	CT(59442)=(S=12815532,E=12815535,C=19)
	CT(59443)=(S=12815536,E=12815539,C=31)
	CT(59444)=(S=12815540,E=12815543,C=18)
	CT(59445)=(S=12815544,E=12815547,C=28)
	CT(59446)=(S=12815548,E=12815551,C=58)
	CT(59447)=(S=12815552,E=12815555,C=47)
	CT(59448)=(S=12815556,E=12815559,C=37)
	CT(59449)=(S=12815560,E=12815563,C=19)
	CT(59450)=(S=12815564,E=12815567,C=18)
	CT(59451)=(S=12815568,E=12815575,C=28)
	CT(59452)=(S=12815576,E=12815579,C=14)
	CT(59453)=(S=12815580,E=12815583,C=58)
	CT(59454)=(S=12815588,E=12815591,C=31)
	CT(59455)=(S=12815592,E=12815595,C=28)
	CT(59456)=(S=12815596,E=12815599,C=58)
	CT(59457)=(S=12815600,E=12815603,C=36)
	CT(59458)=(S=12815604,E=12815607,C=31)
	CT(59459)=(S=12815608,E=12815611,C=131)
	CT(59460)=(S=12815612,E=12815615,C=33)
	CT(59461)=(S=12815616,E=12815871,C=34)
	CT(59462)=(S=12815872,E=12816127,C=37)
	CT(59463)=(S=12816128,E=12816384,C=14)
	CT(59464)=(S=12816385,E=12816385,C=18)
	CT(59465)=(S=12816386,E=12816386,C=36)
	CT(59466)=(S=12816387,E=12816387,C=28)
	CT(59467)=(S=12816388,E=12816388,C=19)
	CT(59468)=(S=12816389,E=12816389,C=14)
	CT(59469)=(S=12816390,E=12816390,C=18)
	CT(59470)=(S=12816391,E=12816391,C=34)
	CT(59471)=(S=12816392,E=12816392,C=14)
	CT(59472)=(S=12816393,E=12816393,C=58)
	CT(59473)=(S=12816394,E=12816394,C=36)
	CT(59474)=(S=12816395,E=12816395,C=12)
	CT(59475)=(S=12816396,E=12816396,C=58)
	CT(59476)=(S=12816397,E=12816397,C=19)
	CT(59477)=(S=12816398,E=12816398,C=31)
	CT(59478)=(S=12816399,E=12816399,C=28)
	CT(59479)=(S=12816400,E=12816400,C=22)
	CT(59480)=(S=12816401,E=12816403,C=14)
	CT(59481)=(S=12816404,E=12816404,C=50)
	CT(59482)=(S=12816405,E=12816405,C=31)
	CT(59483)=(S=12816406,E=12816406,C=28)
	CT(59484)=(S=12816408,E=12816408,C=12)
	CT(59485)=(S=12816409,E=12816409,C=31)
	CT(59486)=(S=12816410,E=12816410,C=52)
	CT(59487)=(S=12816412,E=12816412,C=31)
	CT(59488)=(S=12816413,E=12816413,C=18)
	CT(59489)=(S=12816414,E=12816414,C=34)
	CT(59490)=(S=12816415,E=12816415,C=28)
	CT(59491)=(S=12816416,E=12816447,C=34)
	CT(59492)=(S=12816448,E=12816479,C=49)
	CT(59493)=(S=12816480,E=12816511,C=33)
	CT(59494)=(S=12816512,E=12816543,C=13)
	CT(59495)=(S=12816544,E=12816575,C=34)
	CT(59496)=(S=12816576,E=12816639,C=18)
	CT(59497)=(S=12816640,E=12816895,C=28)
	CT(59498)=(S=12816896,E=12816927,C=71)
	CT(59499)=(S=12816928,E=12816959,C=26)
	CT(59500)=(S=12816960,E=12816991,C=18)
	CT(59501)=(S=12816992,E=12817023,C=33)
	CT(59502)=(S=12817024,E=12817055,C=71)
	CT(59503)=(S=12817056,E=12817087,C=14)
	CT(59504)=(S=12817088,E=12817151,C=12)
	CT(59505)=(S=12817152,E=12817407,C=14)
	CT(59506)=(S=12817408,E=12817663,C=47)
	CT(59507)=(S=12817664,E=12817727,C=14)
	CT(59508)=(S=12817728,E=12817728,C=58)
	CT(59509)=(S=12817729,E=12817729,C=23)
	CT(59510)=(S=12817730,E=12817730,C=12)
	CT(59511)=(S=12817732,E=12817732,C=58)
	CT(59512)=(S=12817733,E=12817733,C=14)
	CT(59513)=(S=12817734,E=12817734,C=31)
	CT(59514)=(S=12817735,E=12817735,C=51)
	CT(59515)=(S=12817736,E=12817736,C=38)
	CT(59516)=(S=12817738,E=12817738,C=28)
	CT(59517)=(S=12817739,E=12817739,C=42)
	CT(59518)=(S=12817740,E=12817740,C=28)
	CT(59519)=(S=12817741,E=12817741,C=58)
	CT(59520)=(S=12817742,E=12817742,C=24)
	CT(59521)=(S=12817743,E=12817746,C=28)
	CT(59522)=(S=12817747,E=12817747,C=47)
	CT(59523)=(S=12817748,E=12817748,C=14)
	CT(59524)=(S=12817749,E=12817749,C=37)
	CT(59525)=(S=12817750,E=12817750,C=36)
	CT(59526)=(S=12817751,E=12817751,C=18)
	CT(59527)=(S=12817752,E=12817752,C=31)
	CT(59528)=(S=12817753,E=12817753,C=18)
	CT(59529)=(S=12817755,E=12817755,C=23)
	CT(59530)=(S=12817758,E=12817758,C=52)
	CT(59531)=(S=12817759,E=12817759,C=58)
	CT(59532)=(S=12817760,E=12817760,C=31)
	CT(59533)=(S=12817761,E=12817761,C=28)
	CT(59534)=(S=12817762,E=12817762,C=58)
	CT(59535)=(S=12817763,E=12817763,C=28)
	CT(59536)=(S=12817764,E=12817764,C=58)
	CT(59537)=(S=12817765,E=12817765,C=19)
	CT(59538)=(S=12817766,E=12817766,C=28)
	CT(59539)=(S=12817767,E=12817767,C=14)
	CT(59540)=(S=12817768,E=12817769,C=18)
	CT(59541)=(S=12817770,E=12817770,C=31)
	CT(59542)=(S=12817771,E=12817771,C=36)
	CT(59543)=(S=12817772,E=12817773,C=31)
	CT(59544)=(S=12817774,E=12817775,C=18)
	CT(59545)=(S=12817776,E=12817776,C=31)
	CT(59546)=(S=12817777,E=12817777,C=25)
	CT(59547)=(S=12817778,E=12817778,C=31)
	CT(59548)=(S=12817779,E=12817779,C=42)
	CT(59549)=(S=12817780,E=12817780,C=37)
	CT(59550)=(S=12817781,E=12817781,C=31)
	CT(59551)=(S=12817782,E=12817782,C=58)
	CT(59552)=(S=12817784,E=12817784,C=28)
	CT(59553)=(S=12817785,E=12817785,C=14)
	CT(59554)=(S=12817786,E=12817786,C=28)
	CT(59555)=(S=12817787,E=12817787,C=19)
	CT(59556)=(S=12817788,E=12817788,C=58)
	CT(59557)=(S=12817789,E=12817789,C=31)
	CT(59558)=(S=12817790,E=12817791,C=14)
	CT(59559)=(S=12817792,E=12817855,C=19)
	CT(59560)=(S=12817856,E=12817859,C=46)
	CT(59561)=(S=12817860,E=12817861,C=18)
	CT(59562)=(S=12817862,E=12817863,C=58)
	CT(59563)=(S=12817864,E=12817871,C=18)
	CT(59564)=(S=12817872,E=12817879,C=25)
	CT(59565)=(S=12817880,E=12817887,C=42)
	CT(59566)=(S=12817888,E=12817903,C=58)
	CT(59567)=(S=12817904,E=12817911,C=42)
	CT(59568)=(S=12817912,E=12817919,C=51)
	CT(59569)=(S=12817920,E=12818175,C=58)
	CT(59570)=(S=12818176,E=12818431,C=18)
	CT(59571)=(S=12818432,E=12818943,C=14)
	CT(59572)=(S=12818944,E=12819199,C=12)
	CT(59573)=(S=12819200,E=12819455,C=37)
	CT(59574)=(S=12819456,E=12819711,C=47)
	CT(59575)=(S=12819712,E=12819967,C=14)
	CT(59576)=(S=12819968,E=12819999,C=98)
	CT(59577)=(S=12820000,E=12820031,C=28)
	CT(59578)=(S=12820032,E=12820095,C=72)
	CT(59579)=(S=12820096,E=12820159,C=28)
	CT(59580)=(S=12820160,E=12820191,C=61)
	CT(59581)=(S=12820192,E=12820193,C=18)
	CT(59582)=(S=12820194,E=12820195,C=58)
	CT(59583)=(S=12820196,E=12820197,C=18)
	CT(59584)=(S=12820198,E=12820199,C=34)
	CT(59585)=(S=12820200,E=12820201,C=18)
	CT(59586)=(S=12820202,E=12820203,C=38)
	CT(59587)=(S=12820204,E=12820205,C=28)
	CT(59588)=(S=12820206,E=12820207,C=18)
	CT(59589)=(S=12820208,E=12820209,C=12)
	CT(59590)=(S=12820210,E=12820211,C=24)
	CT(59591)=(S=12820214,E=12820215,C=58)
	CT(59592)=(S=12820216,E=12820219,C=18)
	CT(59593)=(S=12820220,E=12820221,C=28)
	CT(59594)=(S=12820222,E=12820223,C=18)
	CT(59595)=(S=12820224,E=12820479,C=27)
	CT(59596)=(S=12820480,E=12820639,C=34)
	CT(59597)=(S=12820640,E=12820641,C=28)
	CT(59598)=(S=12820642,E=12820643,C=38)
	CT(59599)=(S=12820644,E=12820645,C=18)
	CT(59600)=(S=12820646,E=12820647,C=49)
	CT(59601)=(S=12820648,E=12820649,C=28)
	CT(59602)=(S=12820650,E=12820651,C=60)
	CT(59603)=(S=12820652,E=12820653,C=28)
	CT(59604)=(S=12820654,E=12820655,C=58)
	CT(59605)=(S=12820656,E=12820657,C=19)
	CT(59606)=(S=12820658,E=12820661,C=58)
	CT(59607)=(S=12820662,E=12820663,C=71)
	CT(59608)=(S=12820664,E=12820665,C=58)
	CT(59609)=(S=12820668,E=12820669,C=12)
	CT(59610)=(S=12820670,E=12820671,C=34)
	CT(59611)=(S=12820672,E=12820675,C=31)
	CT(59612)=(S=12820676,E=12820683,C=28)
	CT(59613)=(S=12820684,E=12820687,C=31)
	CT(59614)=(S=12820688,E=12820691,C=15)
	CT(59615)=(S=12820692,E=12820695,C=14)
	CT(59616)=(S=12820696,E=12820699,C=36)
	CT(59617)=(S=12820700,E=12820703,C=31)
	CT(59618)=(S=12820704,E=12820707,C=25)
	CT(59619)=(S=12820708,E=12820711,C=28)
	CT(59620)=(S=12820712,E=12820715,C=31)
	CT(59621)=(S=12820716,E=12820719,C=14)
	CT(59622)=(S=12820720,E=12820723,C=17)
	CT(59623)=(S=12820724,E=12820727,C=18)
	CT(59624)=(S=12820728,E=12820731,C=28)
	CT(59625)=(S=12820732,E=12820735,C=36)
	CT(59626)=(S=12820736,E=12820991,C=18)
	CT(59627)=(S=12820992,E=12820993,C=31)
	CT(59628)=(S=12820994,E=12820995,C=14)
	CT(59629)=(S=12820996,E=12820997,C=28)
	CT(59630)=(S=12821000,E=12821003,C=18)
	CT(59631)=(S=12821004,E=12821005,C=14)
	CT(59632)=(S=12821006,E=12821007,C=20)
	CT(59633)=(S=12821008,E=12821009,C=58)
	CT(59634)=(S=12821010,E=12821011,C=25)
	CT(59635)=(S=12821012,E=12821013,C=14)
	CT(59636)=(S=12821014,E=12821015,C=18)
	CT(59637)=(S=12821016,E=12821017,C=58)
	CT(59638)=(S=12821018,E=12821019,C=20)
	CT(59639)=(S=12821020,E=12821021,C=14)
	CT(59640)=(S=12821022,E=12821023,C=28)
	CT(59641)=(S=12821024,E=12821055,C=18)
	CT(59642)=(S=12821056,E=12821057,C=31)
	CT(59643)=(S=12821058,E=12821059,C=17)
	CT(59644)=(S=12821060,E=12821061,C=18)
	CT(59645)=(S=12821062,E=12821063,C=31)
	CT(59646)=(S=12821064,E=12821065,C=51)
	CT(59647)=(S=12821066,E=12821067,C=31)
	CT(59648)=(S=12821068,E=12821069,C=58)
	CT(59649)=(S=12821070,E=12821071,C=18)
	CT(59650)=(S=12821072,E=12821075,C=31)
	CT(59651)=(S=12821076,E=12821079,C=63)
	CT(59652)=(S=12821080,E=12821083,C=31)
	CT(59653)=(S=12821084,E=12821087,C=18)
	CT(59654)=(S=12821088,E=12821119,C=14)
	CT(59655)=(S=12821120,E=12821183,C=34)
	CT(59656)=(S=12821184,E=12821215,C=49)
	CT(59657)=(S=12821216,E=12821247,C=28)
	CT(59658)=(S=12821248,E=12821503,C=19)
	CT(59659)=(S=12821504,E=12821759,C=58)
	CT(59660)=(S=12821760,E=12822015,C=47)
	CT(59661)=(S=12822016,E=12822175,C=14)
	CT(59662)=(S=12822176,E=12822207,C=18)
	CT(59663)=(S=12822208,E=12822239,C=69)
	CT(59664)=(S=12822272,E=12822399,C=22)
	CT(59665)=(S=12822400,E=12822463,C=14)
	CT(59666)=(S=12822464,E=12822479,C=12)
	CT(59667)=(S=12822480,E=12822495,C=28)
	CT(59668)=(S=12822496,E=12822511,C=12)
	CT(59669)=(S=12822512,E=12822527,C=14)
	CT(59670)=(S=12822528,E=12822783,C=71)
	CT(59671)=(S=12822784,E=12823039,C=36)
	CT(59672)=(S=12823040,E=12823071,C=22)
	CT(59673)=(S=12823072,E=12823103,C=18)
	CT(59674)=(S=12823104,E=12823135,C=42)
	CT(59675)=(S=12823136,E=12823167,C=28)
	CT(59676)=(S=12823168,E=12823199,C=47)
	CT(59677)=(S=12823200,E=12823200,C=34)
	CT(59678)=(S=12823201,E=12823201,C=38)
	CT(59679)=(S=12823202,E=12823202,C=12)
	CT(59680)=(S=12823203,E=12823203,C=26)
	CT(59681)=(S=12823204,E=12823204,C=12)
	CT(59682)=(S=12823206,E=12823206,C=51)
	CT(59683)=(S=12823208,E=12823208,C=21)
	CT(59684)=(S=12823209,E=12823209,C=14)
	CT(59685)=(S=12823210,E=12823210,C=25)
	CT(59686)=(S=12823211,E=12823212,C=38)
	CT(59687)=(S=12823213,E=12823213,C=14)
	CT(59688)=(S=12823214,E=12823214,C=18)
	CT(59689)=(S=12823215,E=12823215,C=34)
	CT(59690)=(S=12823216,E=12823216,C=32)
	CT(59691)=(S=12823217,E=12823217,C=52)
	CT(59692)=(S=12823218,E=12823218,C=51)
	CT(59693)=(S=12823219,E=12823219,C=31)
	CT(59694)=(S=12823220,E=12823220,C=23)
	CT(59695)=(S=12823221,E=12823221,C=44)
	CT(59696)=(S=12823222,E=12823222,C=58)
	CT(59697)=(S=12823223,E=12823223,C=46)
	CT(59698)=(S=12823224,E=12823224,C=50)
	CT(59699)=(S=12823225,E=12823225,C=28)
	CT(59700)=(S=12823226,E=12823226,C=58)
	CT(59701)=(S=12823227,E=12823227,C=19)
	CT(59702)=(S=12823228,E=12823228,C=58)
	CT(59703)=(S=12823229,E=12823229,C=19)
	CT(59704)=(S=12823231,E=12823231,C=58)
	CT(59705)=(S=12823232,E=12823295,C=18)
	CT(59706)=(S=12823296,E=12824063,C=14)
	CT(59707)=(S=12824064,E=12824575,C=37)
	CT(59708)=(S=12824576,E=12824831,C=34)
	CT(59709)=(S=12824832,E=12824895,C=28)
	CT(59710)=(S=12824896,E=12824899,C=58)
	CT(59711)=(S=12824900,E=12824903,C=14)
	CT(59712)=(S=12824904,E=12824907,C=31)
	CT(59713)=(S=12824912,E=12824915,C=49)
	CT(59714)=(S=12824916,E=12824919,C=58)
	CT(59715)=(S=12824924,E=12824927,C=31)
	CT(59716)=(S=12824928,E=12824931,C=27)
	CT(59717)=(S=12824932,E=12824935,C=58)
	CT(59718)=(S=12824936,E=12824939,C=18)
	CT(59719)=(S=12824948,E=12824951,C=31)
	CT(59720)=(S=12824952,E=12824955,C=18)
	CT(59721)=(S=12824956,E=12824959,C=31)
	CT(59722)=(S=12824960,E=12825023,C=28)
	CT(59723)=(S=12825024,E=12825025,C=14)
	CT(59724)=(S=12825028,E=12825029,C=58)
	CT(59725)=(S=12825030,E=12825031,C=42)
	CT(59726)=(S=12825032,E=12825033,C=28)
	CT(59727)=(S=12825034,E=12825035,C=27)
	CT(59728)=(S=12825036,E=12825037,C=18)
	CT(59729)=(S=12825038,E=12825039,C=37)
	CT(59730)=(S=12825040,E=12825041,C=31)
	CT(59731)=(S=12825042,E=12825043,C=58)
	CT(59732)=(S=12825044,E=12825045,C=14)
	CT(59733)=(S=12825046,E=12825047,C=36)
	CT(59734)=(S=12825048,E=12825049,C=58)
	CT(59735)=(S=12825050,E=12825051,C=51)
	CT(59736)=(S=12825052,E=12825053,C=25)
	CT(59737)=(S=12825054,E=12825055,C=31)
	CT(59738)=(S=12825056,E=12825057,C=38)
	CT(59739)=(S=12825058,E=12825059,C=18)
	CT(59740)=(S=12825060,E=12825061,C=42)
	CT(59741)=(S=12825064,E=12825065,C=28)
	CT(59742)=(S=12825066,E=12825067,C=34)
	CT(59743)=(S=12825068,E=12825073,C=31)
	CT(59744)=(S=12825074,E=12825075,C=36)
	CT(59745)=(S=12825076,E=12825077,C=14)
	CT(59746)=(S=12825078,E=12825079,C=49)
	CT(59747)=(S=12825080,E=12825081,C=51)
	CT(59748)=(S=12825082,E=12825083,C=42)
	CT(59749)=(S=12825084,E=12825085,C=14)
	CT(59750)=(S=12825086,E=12825089,C=28)
	CT(59751)=(S=12825090,E=12825091,C=18)
	CT(59752)=(S=12825092,E=12825093,C=36)
	CT(59753)=(S=12825094,E=12825095,C=13)
	CT(59754)=(S=12825096,E=12825097,C=51)
	CT(59755)=(S=12825098,E=12825099,C=31)
	CT(59756)=(S=12825100,E=12825101,C=18)
	CT(59757)=(S=12825102,E=12825103,C=24)
	CT(59758)=(S=12825104,E=12825105,C=18)
	CT(59759)=(S=12825106,E=12825107,C=31)
	CT(59760)=(S=12825108,E=12825113,C=18)
	CT(59761)=(S=12825114,E=12825115,C=58)
	CT(59762)=(S=12825116,E=12825117,C=15)
	CT(59763)=(S=12825118,E=12825119,C=31)
	CT(59764)=(S=12825120,E=12825151,C=46)
	CT(59765)=(S=12825152,E=12825183,C=33)
	CT(59766)=(S=12825184,E=12825185,C=60)
	CT(59767)=(S=12825186,E=12825187,C=51)
	CT(59768)=(S=12825188,E=12825189,C=28)
	CT(59769)=(S=12825190,E=12825191,C=38)
	CT(59770)=(S=12825192,E=12825193,C=31)
	CT(59771)=(S=12825194,E=12825195,C=38)
	CT(59772)=(S=12825196,E=12825197,C=18)
	CT(59773)=(S=12825200,E=12825201,C=12)
	CT(59774)=(S=12825202,E=12825203,C=58)
	CT(59775)=(S=12825204,E=12825205,C=51)
	CT(59776)=(S=12825206,E=12825207,C=52)
	CT(59777)=(S=12825210,E=12825211,C=58)
	CT(59778)=(S=12825216,E=12825247,C=31)
	CT(59779)=(S=12825248,E=12825279,C=19)
	CT(59780)=(S=12825280,E=12825311,C=18)
	CT(59781)=(S=12825312,E=12825343,C=19)
	CT(59782)=(S=12825344,E=12825855,C=28)
	CT(59783)=(S=12825856,E=12826111,C=24)
	CT(59784)=(S=12826112,E=12826112,C=31)
	CT(59785)=(S=12826113,E=12826113,C=36)
	CT(59786)=(S=12826114,E=12826114,C=28)
	CT(59787)=(S=12826115,E=12826115,C=18)
	CT(59788)=(S=12826116,E=12826116,C=14)
	CT(59789)=(S=12826117,E=12826117,C=19)
	CT(59790)=(S=12826118,E=12826118,C=42)
	CT(59791)=(S=12826119,E=12826119,C=31)
	CT(59792)=(S=12826120,E=12826120,C=18)
	CT(59793)=(S=12826121,E=12826121,C=58)
	CT(59794)=(S=12826123,E=12826123,C=28)
	CT(59795)=(S=12826124,E=12826124,C=34)
	CT(59796)=(S=12826125,E=12826125,C=36)
	CT(59797)=(S=12826126,E=12826126,C=58)
	CT(59798)=(S=12826128,E=12826129,C=12)
	CT(59799)=(S=12826130,E=12826130,C=28)
	CT(59800)=(S=12826132,E=12826132,C=18)
	CT(59801)=(S=12826133,E=12826134,C=31)
	CT(59802)=(S=12826135,E=12826135,C=58)
	CT(59803)=(S=12826137,E=12826137,C=37)
	CT(59804)=(S=12826138,E=12826138,C=42)
	CT(59805)=(S=12826140,E=12826140,C=18)
	CT(59806)=(S=12826141,E=12826141,C=42)
	CT(59807)=(S=12826142,E=12826142,C=25)
	CT(59808)=(S=12826143,E=12826143,C=23)
	CT(59809)=(S=12826144,E=12826144,C=24)
	CT(59810)=(S=12826145,E=12826145,C=17)
	CT(59811)=(S=12826146,E=12826147,C=58)
	CT(59812)=(S=12826148,E=12826151,C=24)
	CT(59813)=(S=12826152,E=12826152,C=58)
	CT(59814)=(S=12826153,E=12826153,C=51)
	CT(59815)=(S=12826154,E=12826154,C=37)
	CT(59816)=(S=12826155,E=12826155,C=36)
	CT(59817)=(S=12826156,E=12826156,C=51)
	CT(59818)=(S=12826157,E=12826157,C=28)
	CT(59819)=(S=12826158,E=12826158,C=57)
	CT(59820)=(S=12826159,E=12826159,C=60)
	CT(59821)=(S=12826160,E=12826160,C=28)
	CT(59822)=(S=12826161,E=12826161,C=58)
	CT(59823)=(S=12826162,E=12826162,C=28)
	CT(59824)=(S=12826163,E=12826164,C=58)
	CT(59825)=(S=12826165,E=12826165,C=34)
	CT(59826)=(S=12826166,E=12826166,C=58)
	CT(59827)=(S=12826167,E=12826167,C=42)
	CT(59828)=(S=12826168,E=12826168,C=28)
	CT(59829)=(S=12826169,E=12826169,C=18)
	CT(59830)=(S=12826170,E=12826170,C=28)
	CT(59831)=(S=12826171,E=12826171,C=14)
	CT(59832)=(S=12826172,E=12826172,C=28)
	CT(59833)=(S=12826173,E=12826173,C=52)
	CT(59834)=(S=12826174,E=12826174,C=14)
	CT(59835)=(S=12826175,E=12826175,C=31)
	CT(59836)=(S=12826176,E=12826207,C=44)
	CT(59837)=(S=12826208,E=12826239,C=13)
	CT(59838)=(S=12826240,E=12826271,C=18)
	CT(59839)=(S=12826272,E=12826303,C=14)
	CT(59840)=(S=12826304,E=12826307,C=31)
	CT(59841)=(S=12826308,E=12826309,C=28)
	CT(59842)=(S=12826314,E=12826315,C=31)
	CT(59843)=(S=12826318,E=12826319,C=58)
	CT(59844)=(S=12826320,E=12826321,C=38)
	CT(59845)=(S=12826322,E=12826323,C=15)
	CT(59846)=(S=12826324,E=12826325,C=38)
	CT(59847)=(S=12826326,E=12826327,C=46)
	CT(59848)=(S=12826328,E=12826329,C=24)
	CT(59849)=(S=12826330,E=12826331,C=58)
	CT(59850)=(S=12826332,E=12826333,C=38)
	CT(59851)=(S=12826334,E=12826335,C=34)
	CT(59852)=(S=12826336,E=12826367,C=15)
	CT(59853)=(S=12826368,E=12826623,C=13)
	CT(59854)=(S=12826624,E=12826655,C=50)
	CT(59855)=(S=12826656,E=12826687,C=24)
	CT(59856)=(S=12826688,E=12826689,C=36)
	CT(59857)=(S=12826690,E=12826691,C=14)
	CT(59858)=(S=12826692,E=12826693,C=19)
	CT(59859)=(S=12826694,E=12826695,C=31)
	CT(59860)=(S=12826698,E=12826699,C=18)
	CT(59861)=(S=12826700,E=12826701,C=14)
	CT(59862)=(S=12826702,E=12826703,C=18)
	CT(59863)=(S=12826704,E=12826705,C=31)
	CT(59864)=(S=12826706,E=12826713,C=58)
	CT(59865)=(S=12826716,E=12826717,C=34)
	CT(59866)=(S=12826718,E=12826719,C=28)
	CT(59867)=(S=12826720,E=12826751,C=24)
	CT(59868)=(S=12826752,E=12826783,C=49)
	CT(59869)=(S=12826784,E=12826815,C=50)
	CT(59870)=(S=12826816,E=12826847,C=31)
	CT(59871)=(S=12826848,E=12826879,C=14)
	CT(59872)=(S=12826880,E=12827135,C=28)
	CT(59873)=(S=12827136,E=12827391,C=34)
	CT(59874)=(S=12827392,E=12827647,C=58)
	CT(59875)=(S=12827648,E=12827903,C=14)
	CT(59876)=(S=12827904,E=12827907,C=33)
	CT(59877)=(S=12827908,E=12827911,C=42)
	CT(59878)=(S=12827912,E=12827915,C=31)
	CT(59879)=(S=12827916,E=12827919,C=14)
	CT(59880)=(S=12827920,E=12827923,C=31)
	CT(59881)=(S=12827924,E=12827927,C=36)
	CT(59882)=(S=12827928,E=12827931,C=19)
	CT(59883)=(S=12827932,E=12827935,C=20)
	CT(59884)=(S=12827936,E=12827939,C=58)
	CT(59885)=(S=12827940,E=12827943,C=31)
	CT(59886)=(S=12827944,E=12827947,C=58)
	CT(59887)=(S=12827948,E=12827955,C=31)
	CT(59888)=(S=12827956,E=12827959,C=58)
	CT(59889)=(S=12827960,E=12827963,C=22)
	CT(59890)=(S=12827964,E=12827967,C=31)
	CT(59891)=(S=12827968,E=12827971,C=12)
	CT(59892)=(S=12827972,E=12827975,C=20)
	CT(59893)=(S=12827980,E=12827983,C=58)
	CT(59894)=(S=12827984,E=12827987,C=51)
	CT(59895)=(S=12827988,E=12827995,C=58)
	CT(59896)=(S=12827996,E=12827999,C=28)
	CT(59897)=(S=12828000,E=12828003,C=31)
	CT(59898)=(S=12828004,E=12828007,C=18)
	CT(59899)=(S=12828008,E=12828011,C=31)
	CT(59900)=(S=12828012,E=12828015,C=18)
	CT(59901)=(S=12828016,E=12828019,C=19)
	CT(59902)=(S=12828020,E=12828023,C=36)
	CT(59903)=(S=12828024,E=12828027,C=18)
	CT(59904)=(S=12828028,E=12828031,C=36)
	CT(59905)=(S=12828032,E=12828033,C=63)
	CT(59906)=(S=12828034,E=12828035,C=24)
	CT(59907)=(S=12828036,E=12828037,C=31)
	CT(59908)=(S=12828038,E=12828041,C=18)
	CT(59909)=(S=12828042,E=12828043,C=38)
	CT(59910)=(S=12828044,E=12828045,C=17)
	CT(59911)=(S=12828046,E=12828047,C=27)
	CT(59912)=(S=12828048,E=12828049,C=38)
	CT(59913)=(S=12828050,E=12828053,C=34)
	CT(59914)=(S=12828054,E=12828059,C=38)
	CT(59915)=(S=12828060,E=12828061,C=32)
	CT(59916)=(S=12828062,E=12828063,C=14)
	CT(59917)=(S=12828064,E=12828065,C=19)
	CT(59918)=(S=12828066,E=12828067,C=22)
	CT(59919)=(S=12828070,E=12828075,C=42)
	CT(59920)=(S=12828076,E=12828077,C=14)
	CT(59921)=(S=12828080,E=12828081,C=38)
	CT(59922)=(S=12828082,E=12828083,C=12)
	CT(59923)=(S=12828084,E=12828085,C=18)
	CT(59924)=(S=12828086,E=12828087,C=19)
	CT(59925)=(S=12828090,E=12828091,C=38)
	CT(59926)=(S=12828092,E=12828093,C=19)
	CT(59927)=(S=12828096,E=12828097,C=17)
	CT(59928)=(S=12828098,E=12828099,C=38)
	CT(59929)=(S=12828100,E=12828101,C=31)
	CT(59930)=(S=12828102,E=12828103,C=42)
	CT(59931)=(S=12828104,E=12828105,C=31)
	CT(59932)=(S=12828106,E=12828107,C=49)
	CT(59933)=(S=12828108,E=12828109,C=34)
	CT(59934)=(S=12828110,E=12828111,C=54)
	CT(59935)=(S=12828112,E=12828113,C=38)
	CT(59936)=(S=12828114,E=12828115,C=43)
	CT(59937)=(S=12828116,E=12828117,C=23)
	CT(59938)=(S=12828118,E=12828119,C=31)
	CT(59939)=(S=12828120,E=12828121,C=36)
	CT(59940)=(S=12828122,E=12828123,C=18)
	CT(59941)=(S=12828124,E=12828125,C=14)
	CT(59942)=(S=12828126,E=12828127,C=18)
	CT(59943)=(S=12828128,E=12828129,C=51)
	CT(59944)=(S=12828130,E=12828133,C=31)
	CT(59945)=(S=12828134,E=12828135,C=58)
	CT(59946)=(S=12828136,E=12828137,C=24)
	CT(59947)=(S=12828138,E=12828139,C=31)
	CT(59948)=(S=12828140,E=12828141,C=28)
	CT(59949)=(S=12828142,E=12828143,C=18)
	CT(59950)=(S=12828144,E=12828145,C=31)
	CT(59951)=(S=12828146,E=12828147,C=209)
	CT(59952)=(S=12828148,E=12828149,C=28)
	CT(59953)=(S=12828150,E=12828153,C=31)
	CT(59954)=(S=12828154,E=12828155,C=38)
	CT(59955)=(S=12828156,E=12828157,C=34)
	CT(59956)=(S=12828158,E=12828159,C=27)
	CT(59957)=(S=12828160,E=12828160,C=36)
	CT(59958)=(S=12828161,E=12828161,C=58)
	CT(59959)=(S=12828162,E=12828162,C=28)
	CT(59960)=(S=12828163,E=12828163,C=12)
	CT(59961)=(S=12828164,E=12828164,C=34)
	CT(59962)=(S=12828165,E=12828166,C=12)
	CT(59963)=(S=12828167,E=12828169,C=28)
	CT(59964)=(S=12828170,E=12828170,C=57)
	CT(59965)=(S=12828171,E=12828171,C=28)
	CT(59966)=(S=12828172,E=12828172,C=18)
	CT(59967)=(S=12828173,E=12828173,C=31)
	CT(59968)=(S=12828174,E=12828174,C=18)
	CT(59969)=(S=12828175,E=12828176,C=42)
	CT(59970)=(S=12828177,E=12828177,C=58)
	CT(59971)=(S=12828178,E=12828178,C=24)
	CT(59972)=(S=12828179,E=12828179,C=17)
	CT(59973)=(S=12828180,E=12828180,C=37)
	CT(59974)=(S=12828181,E=12828181,C=15)
	CT(59975)=(S=12828182,E=12828182,C=14)
	CT(59976)=(S=12828183,E=12828183,C=17)
	CT(59977)=(S=12828184,E=12828184,C=29)
	CT(59978)=(S=12828185,E=12828185,C=24)
	CT(59979)=(S=12828186,E=12828186,C=42)
	CT(59980)=(S=12828187,E=12828187,C=12)
	CT(59981)=(S=12828188,E=12828188,C=36)
	CT(59982)=(S=12828189,E=12828189,C=34)
	CT(59983)=(S=12828190,E=12828191,C=24)
	CT(59984)=(S=12828192,E=12828223,C=22)
	CT(59985)=(S=12828224,E=12828255,C=14)
	CT(59986)=(S=12828256,E=12828287,C=18)
	CT(59987)=(S=12828288,E=12828288,C=24)
	CT(59988)=(S=12828289,E=12828289,C=52)
	CT(59989)=(S=12828291,E=12828291,C=42)
	CT(59990)=(S=12828292,E=12828292,C=18)
	CT(59991)=(S=12828293,E=12828293,C=3)
	CT(59992)=(S=12828294,E=12828294,C=55)
	CT(59993)=(S=12828295,E=12828295,C=28)
	CT(59994)=(S=12828296,E=12828296,C=57)
	CT(59995)=(S=12828297,E=12828297,C=12)
	CT(59996)=(S=12828298,E=12828298,C=28)
	CT(59997)=(S=12828300,E=12828300,C=33)
	CT(59998)=(S=12828301,E=12828301,C=44)
	CT(59999)=(S=12828302,E=12828302,C=28)
	CT(60000)=(S=12828303,E=12828303,C=42)
	CT(60001)=(S=12828305,E=12828305,C=28)
	CT(60002)=(S=12828306,E=12828306,C=36)
	CT(60003)=(S=12828307,E=12828307,C=19)
	CT(60004)=(S=12828308,E=12828308,C=28)
	CT(60005)=(S=12828311,E=12828311,C=36)
	CT(60006)=(S=12828312,E=12828312,C=31)
	CT(60007)=(S=12828313,E=12828313,C=24)
	CT(60008)=(S=12828315,E=12828316,C=12)
	CT(60009)=(S=12828318,E=12828318,C=52)
	CT(60010)=(S=12828319,E=12828319,C=38)
	CT(60011)=(S=12828320,E=12828351,C=34)
	CT(60012)=(S=12828352,E=12828383,C=19)
	CT(60013)=(S=12828384,E=12828417,C=36)
	CT(60014)=(S=12828418,E=12828419,C=28)
	CT(60015)=(S=12828420,E=12828421,C=18)
	CT(60016)=(S=12828422,E=12828423,C=23)
	CT(60017)=(S=12828424,E=12828425,C=58)
	CT(60018)=(S=12828426,E=12828427,C=18)
	CT(60019)=(S=12828428,E=12828429,C=31)
	CT(60020)=(S=12828430,E=12828431,C=28)
	CT(60021)=(S=12828432,E=12828433,C=36)
	CT(60022)=(S=12828434,E=12828435,C=18)
	CT(60023)=(S=12828436,E=12828437,C=28)
	CT(60024)=(S=12828440,E=12828441,C=31)
	CT(60025)=(S=12828442,E=12828443,C=28)
	CT(60026)=(S=12828444,E=12828445,C=63)
	CT(60027)=(S=12828446,E=12828447,C=27)
	CT(60028)=(S=12828448,E=12828449,C=18)
	CT(60029)=(S=12828450,E=12828451,C=51)
	CT(60030)=(S=12828452,E=12828453,C=18)
	CT(60031)=(S=12828454,E=12828455,C=31)
	CT(60032)=(S=12828456,E=12828457,C=18)
	CT(60033)=(S=12828458,E=12828459,C=28)
	CT(60034)=(S=12828460,E=12828463,C=38)
	CT(60035)=(S=12828464,E=12828465,C=33)
	CT(60036)=(S=12828466,E=12828467,C=18)
	CT(60037)=(S=12828468,E=12828469,C=17)
	CT(60038)=(S=12828472,E=12828475,C=31)
	CT(60039)=(S=12828476,E=12828477,C=51)
	CT(60040)=(S=12828478,E=12828479,C=58)
	CT(60041)=(S=12828480,E=12828481,C=18)
	CT(60042)=(S=12828482,E=12828483,C=14)
	CT(60043)=(S=12828484,E=12828485,C=28)
	CT(60044)=(S=12828486,E=12828487,C=14)
	CT(60045)=(S=12828488,E=12828489,C=31)
	CT(60046)=(S=12828490,E=12828491,C=26)
	CT(60047)=(S=12828492,E=12828495,C=18)
	CT(60048)=(S=12828496,E=12828497,C=31)
	CT(60049)=(S=12828498,E=12828499,C=18)
	CT(60050)=(S=12828500,E=12828501,C=15)
	CT(60051)=(S=12828502,E=12828503,C=14)
	CT(60052)=(S=12828504,E=12828505,C=18)
	CT(60053)=(S=12828506,E=12828507,C=31)
	CT(60054)=(S=12828508,E=12828509,C=28)
	CT(60055)=(S=12828510,E=12828511,C=51)
	CT(60056)=(S=12828512,E=12828512,C=52)
	CT(60057)=(S=12828513,E=12828513,C=50)
	CT(60058)=(S=12828514,E=12828515,C=22)
	CT(60059)=(S=12828516,E=12828517,C=66)
	CT(60060)=(S=12828518,E=12828519,C=13)
	CT(60061)=(S=12828520,E=12828521,C=31)
	CT(60062)=(S=12828522,E=12828523,C=14)
	CT(60063)=(S=12828524,E=12828525,C=24)
	CT(60064)=(S=12828526,E=12828527,C=42)
	CT(60065)=(S=12828528,E=12828529,C=36)
	CT(60066)=(S=12828530,E=12828531,C=28)
	CT(60067)=(S=12828532,E=12828533,C=27)
	CT(60068)=(S=12828534,E=12828535,C=37)
	CT(60069)=(S=12828536,E=12828537,C=36)
	CT(60070)=(S=12828538,E=12828539,C=47)
	CT(60071)=(S=12828540,E=12828541,C=38)
	CT(60072)=(S=12828542,E=12828543,C=31)
	CT(60073)=(S=12828544,E=12828545,C=28)
	CT(60074)=(S=12828546,E=12828547,C=18)
	CT(60075)=(S=12828548,E=12828549,C=34)
	CT(60076)=(S=12828550,E=12828551,C=12)
	CT(60077)=(S=12828552,E=12828553,C=18)
	CT(60078)=(S=12828554,E=12828555,C=31)
	CT(60079)=(S=12828558,E=12828559,C=24)
	CT(60080)=(S=12828560,E=12828561,C=19)
	CT(60081)=(S=12828562,E=12828563,C=18)
	CT(60082)=(S=12828564,E=12828565,C=51)
	CT(60083)=(S=12828566,E=12828569,C=36)
	CT(60084)=(S=12828570,E=12828571,C=66)
	CT(60085)=(S=12828572,E=12828573,C=14)
	CT(60086)=(S=12828574,E=12828575,C=18)
	CT(60087)=(S=12828576,E=12828577,C=38)
	CT(60088)=(S=12828578,E=12828579,C=58)
	CT(60089)=(S=12828580,E=12828581,C=36)
	CT(60090)=(S=12828582,E=12828583,C=18)
	CT(60091)=(S=12828584,E=12828585,C=31)
	CT(60092)=(S=12828586,E=12828587,C=58)
	CT(60093)=(S=12828588,E=12828589,C=12)
	CT(60094)=(S=12828590,E=12828591,C=18)
	CT(60095)=(S=12828594,E=12828595,C=31)
	CT(60096)=(S=12828596,E=12828597,C=58)
	CT(60097)=(S=12828598,E=12828599,C=18)
	CT(60098)=(S=12828600,E=12828601,C=58)
	CT(60099)=(S=12828602,E=12828603,C=66)
	CT(60100)=(S=12828604,E=12828605,C=34)
	CT(60101)=(S=12828606,E=12828609,C=18)
	CT(60102)=(S=12828610,E=12828611,C=31)
	CT(60103)=(S=12828612,E=12828613,C=28)
	CT(60104)=(S=12828616,E=12828617,C=31)
	CT(60105)=(S=12828618,E=12828621,C=33)
	CT(60106)=(S=12828624,E=12828625,C=18)
	CT(60107)=(S=12828626,E=12828627,C=14)
	CT(60108)=(S=12828628,E=12828629,C=19)
	CT(60109)=(S=12828630,E=12828631,C=31)
	CT(60110)=(S=12828632,E=12828633,C=28)
	CT(60111)=(S=12828634,E=12828635,C=14)
	CT(60112)=(S=12828636,E=12828637,C=18)
	CT(60113)=(S=12828638,E=12828639,C=50)
	CT(60114)=(S=12828640,E=12828641,C=28)
	CT(60115)=(S=12828642,E=12828643,C=31)
	CT(60116)=(S=12828644,E=12828645,C=50)
	CT(60117)=(S=12828646,E=12828647,C=190)
	CT(60118)=(S=12828648,E=12828649,C=58)
	CT(60119)=(S=12828650,E=12828651,C=31)
	CT(60120)=(S=12828652,E=12828655,C=18)
	CT(60121)=(S=12828656,E=12828657,C=34)
	CT(60122)=(S=12828658,E=12828659,C=18)
	CT(60123)=(S=12828660,E=12828661,C=12)
	CT(60124)=(S=12828662,E=12828663,C=31)
	CT(60125)=(S=12828664,E=12828665,C=58)
	CT(60126)=(S=12828666,E=12828667,C=18)
	CT(60127)=(S=12828668,E=12828671,C=42)
	CT(60128)=(S=12828672,E=12828927,C=24)
	CT(60129)=(S=12828928,E=12829183,C=36)
	CT(60130)=(S=12829184,E=12829695,C=14)
	CT(60131)=(S=12829696,E=12829951,C=19)
	CT(60132)=(S=12829952,E=12830207,C=47)
	CT(60133)=(S=12830208,E=12830463,C=19)
	CT(60134)=(S=12830464,E=12830719,C=50)
	CT(60135)=(S=12830720,E=12830751,C=14)
	CT(60136)=(S=12830752,E=12830783,C=28)
	CT(60137)=(S=12830784,E=12830785,C=31)
	CT(60138)=(S=12830788,E=12830789,C=47)
	CT(60139)=(S=12830790,E=12830791,C=28)
	CT(60140)=(S=12830792,E=12830793,C=19)
	CT(60141)=(S=12830794,E=12830795,C=20)
	CT(60142)=(S=12830796,E=12830797,C=14)
	CT(60143)=(S=12830798,E=12830799,C=12)
	CT(60144)=(S=12830800,E=12830801,C=28)
	CT(60145)=(S=12830802,E=12830803,C=36)
	CT(60146)=(S=12830804,E=12830805,C=29)
	CT(60147)=(S=12830806,E=12830807,C=38)
	CT(60148)=(S=12830808,E=12830809,C=58)
	CT(60149)=(S=12830810,E=12830811,C=31)
	CT(60150)=(S=12830812,E=12830813,C=17)
	CT(60151)=(S=12830814,E=12830815,C=15)
	CT(60152)=(S=12830816,E=12830847,C=12)
	CT(60153)=(S=12830848,E=12830879,C=14)
	CT(60154)=(S=12830880,E=12830911,C=12)
	CT(60155)=(S=12830912,E=12830912,C=28)
	CT(60156)=(S=12830913,E=12830913,C=22)
	CT(60157)=(S=12830914,E=12830914,C=28)
	CT(60158)=(S=12830915,E=12830915,C=12)
	CT(60159)=(S=12830916,E=12830916,C=31)
	CT(60160)=(S=12830917,E=12830917,C=42)
	CT(60161)=(S=12830918,E=12830918,C=14)
	CT(60162)=(S=12830919,E=12830919,C=58)
	CT(60163)=(S=12830920,E=12830920,C=52)
	CT(60164)=(S=12830921,E=12830921,C=49)
	CT(60165)=(S=12830922,E=12830922,C=15)
	CT(60166)=(S=12830923,E=12830923,C=18)
	CT(60167)=(S=12830924,E=12830924,C=22)
	CT(60168)=(S=12830925,E=12830925,C=17)
	CT(60169)=(S=12830926,E=12830926,C=58)
	CT(60170)=(S=12830927,E=12830927,C=28)
	CT(60171)=(S=12830928,E=12830928,C=12)
	CT(60172)=(S=12830929,E=12830929,C=18)
	CT(60173)=(S=12830931,E=12830931,C=14)
	CT(60174)=(S=12830932,E=12830932,C=58)
	CT(60175)=(S=12830933,E=12830933,C=18)
	CT(60176)=(S=12830934,E=12830934,C=58)
	CT(60177)=(S=12830936,E=12830936,C=18)
	CT(60178)=(S=12830937,E=12830937,C=12)
	CT(60179)=(S=12830938,E=12830938,C=34)
	CT(60180)=(S=12830939,E=12830940,C=19)
	CT(60181)=(S=12830942,E=12830942,C=37)
	CT(60182)=(S=12830943,E=12830943,C=36)
	CT(60183)=(S=12830944,E=12830945,C=49)
	CT(60184)=(S=12830946,E=12830947,C=18)
	CT(60185)=(S=12830948,E=12830949,C=12)
	CT(60186)=(S=12830950,E=12830951,C=14)
	CT(60187)=(S=12830952,E=12830953,C=18)
	CT(60188)=(S=12830954,E=12830955,C=58)
	CT(60189)=(S=12830956,E=12830957,C=18)
	CT(60190)=(S=12830958,E=12830959,C=22)
	CT(60191)=(S=12830960,E=12830961,C=70)
	CT(60192)=(S=12830962,E=12830963,C=28)
	CT(60193)=(S=12830964,E=12830965,C=31)
	CT(60194)=(S=12830966,E=12830967,C=54)
	CT(60195)=(S=12830968,E=12830969,C=18)
	CT(60196)=(S=12830970,E=12830971,C=31)
	CT(60197)=(S=12830972,E=12830973,C=21)
	CT(60198)=(S=12830974,E=12830975,C=12)
	CT(60199)=(S=12830976,E=12831231,C=18)
	CT(60200)=(S=12831232,E=12831263,C=12)
	CT(60201)=(S=12831264,E=12831295,C=28)
	CT(60202)=(S=12831296,E=12831327,C=109)
	CT(60203)=(S=12831328,E=12831359,C=14)
	CT(60204)=(S=12831360,E=12831423,C=42)
	CT(60205)=(S=12831424,E=12831487,C=34)
	CT(60206)=(S=12831488,E=12831743,C=28)
	CT(60207)=(S=12831744,E=12831999,C=27)
	CT(60208)=(S=12832000,E=12832255,C=58)
	CT(60209)=(S=12832256,E=12832287,C=15)
	CT(60210)=(S=12832288,E=12832319,C=18)
	CT(60211)=(S=12832320,E=12832351,C=36)
	CT(60212)=(S=12832352,E=12832359,C=42)
	CT(60213)=(S=12832368,E=12832375,C=58)
	CT(60214)=(S=12832376,E=12832376,C=38)
	CT(60215)=(S=12832377,E=12832377,C=24)
	CT(60216)=(S=12832378,E=12832379,C=18)
	CT(60217)=(S=12832380,E=12832383,C=58)
	CT(60218)=(S=12832384,E=12832415,C=28)
	CT(60219)=(S=12832416,E=12832479,C=14)
	CT(60220)=(S=12832480,E=12832483,C=18)
	CT(60221)=(S=12832484,E=12832485,C=52)
	CT(60222)=(S=12832488,E=12832489,C=31)
	CT(60223)=(S=12832492,E=12832493,C=12)
	CT(60224)=(S=12832494,E=12832495,C=31)
	CT(60225)=(S=12832496,E=12832497,C=18)
	CT(60226)=(S=12832498,E=12832499,C=38)
	CT(60227)=(S=12832500,E=12832501,C=18)
	CT(60228)=(S=12832502,E=12832503,C=38)
	CT(60229)=(S=12832504,E=12832505,C=19)
	CT(60230)=(S=12832506,E=12832507,C=32)
	CT(60231)=(S=12832508,E=12832511,C=18)
	CT(60232)=(S=12832512,E=12832767,C=49)
	CT(60233)=(S=12832768,E=12833279,C=18)
	CT(60234)=(S=12833280,E=12833281,C=50)
	CT(60235)=(S=12833282,E=12833283,C=18)
	CT(60236)=(S=12833284,E=12833285,C=38)
	CT(60237)=(S=12833286,E=12833287,C=28)
	CT(60238)=(S=12833288,E=12833289,C=18)
	CT(60239)=(S=12833290,E=12833291,C=28)
	CT(60240)=(S=12833292,E=12833293,C=58)
	CT(60241)=(S=12833294,E=12833295,C=33)
	CT(60242)=(S=12833296,E=12833297,C=34)
	CT(60243)=(S=12833298,E=12833299,C=24)
	CT(60244)=(S=12833300,E=12833301,C=18)
	CT(60245)=(S=12833302,E=12833303,C=14)
	CT(60246)=(S=12833304,E=12833305,C=58)
	CT(60247)=(S=12833306,E=12833307,C=28)
	CT(60248)=(S=12833308,E=12833309,C=71)
	CT(60249)=(S=12833310,E=12833313,C=18)
	CT(60250)=(S=12833314,E=12833315,C=58)
	CT(60251)=(S=12833316,E=12833317,C=19)
	CT(60252)=(S=12833318,E=12833319,C=14)
	CT(60253)=(S=12833320,E=12833321,C=38)
	CT(60254)=(S=12833322,E=12833323,C=12)
	CT(60255)=(S=12833324,E=12833325,C=38)
	CT(60256)=(S=12833326,E=12833327,C=20)
	CT(60257)=(S=12833328,E=12833331,C=28)
	CT(60258)=(S=12833332,E=12833333,C=19)
	CT(60259)=(S=12833334,E=12833335,C=14)
	CT(60260)=(S=12833336,E=12833337,C=36)
	CT(60261)=(S=12833338,E=12833339,C=28)
	CT(60262)=(S=12833340,E=12833341,C=58)
	CT(60263)=(S=12833342,E=12833343,C=19)
	CT(60264)=(S=12833344,E=12833407,C=15)
	CT(60265)=(S=12833408,E=12833471,C=18)
	CT(60266)=(S=12833472,E=12833535,C=52)
	CT(60267)=(S=12833536,E=12833539,C=58)
	CT(60268)=(S=12833540,E=12833543,C=18)
	CT(60269)=(S=12833544,E=12833547,C=58)
	CT(60270)=(S=12833548,E=12833551,C=34)
	CT(60271)=(S=12833552,E=12833555,C=47)
	CT(60272)=(S=12833556,E=12833567,C=18)
	CT(60273)=(S=12833568,E=12833571,C=42)
	CT(60274)=(S=12833572,E=12833579,C=18)
	CT(60275)=(S=12833580,E=12833583,C=26)
	CT(60276)=(S=12833584,E=12833587,C=36)
	CT(60277)=(S=12833588,E=12833591,C=33)
	CT(60278)=(S=12833592,E=12833595,C=36)
	CT(60279)=(S=12833596,E=12833599,C=31)
	CT(60280)=(S=12833600,E=12833603,C=18)
	CT(60281)=(S=12833604,E=12833607,C=17)
	CT(60282)=(S=12833608,E=12833611,C=36)
	CT(60283)=(S=12833612,E=12833619,C=18)
	CT(60284)=(S=12833620,E=12833623,C=31)
	CT(60285)=(S=12833624,E=12833627,C=18)
	CT(60286)=(S=12833628,E=12833631,C=14)
	CT(60287)=(S=12833632,E=12833635,C=31)
	CT(60288)=(S=12833636,E=12833643,C=18)
	CT(60289)=(S=12833644,E=12833647,C=31)
	CT(60290)=(S=12833648,E=12833651,C=18)
	CT(60291)=(S=12833652,E=12833655,C=31)
	CT(60292)=(S=12833656,E=12833667,C=18)
	CT(60293)=(S=12833668,E=12833671,C=19)
	CT(60294)=(S=12833672,E=12833691,C=31)
	CT(60295)=(S=12833692,E=12833695,C=60)
	CT(60296)=(S=12833696,E=12833699,C=31)
	CT(60297)=(S=12833700,E=12833703,C=14)
	CT(60298)=(S=12833704,E=12833707,C=18)
	CT(60299)=(S=12833708,E=12833711,C=31)
	CT(60300)=(S=12833712,E=12833715,C=24)
	CT(60301)=(S=12833716,E=12833719,C=98)
	CT(60302)=(S=12833720,E=12833727,C=31)
	CT(60303)=(S=12833728,E=12833735,C=18)
	CT(60304)=(S=12833736,E=12833739,C=22)
	CT(60305)=(S=12833740,E=12833743,C=18)
	CT(60306)=(S=12833744,E=12833751,C=31)
	CT(60307)=(S=12833752,E=12833759,C=18)
	CT(60308)=(S=12833760,E=12833763,C=63)
	CT(60309)=(S=12833764,E=12833767,C=31)
	CT(60310)=(S=12833768,E=12833775,C=18)
	CT(60311)=(S=12833776,E=12833779,C=31)
	CT(60312)=(S=12833780,E=12833787,C=18)
	CT(60313)=(S=12833788,E=12833791,C=31)
	CT(60314)=(S=12833792,E=12834047,C=13)
	CT(60315)=(S=12834048,E=12834303,C=49)
	CT(60316)=(S=12834304,E=12834431,C=28)
	CT(60317)=(S=12834432,E=12834495,C=37)
	CT(60318)=(S=12834496,E=12834503,C=31)
	CT(60319)=(S=12834504,E=12834511,C=27)
	CT(60320)=(S=12834512,E=12834519,C=31)
	CT(60321)=(S=12834520,E=12834523,C=28)
	CT(60322)=(S=12834524,E=12834527,C=31)
	CT(60323)=(S=12834528,E=12834535,C=12)
	CT(60324)=(S=12834536,E=12834539,C=18)
	CT(60325)=(S=12834540,E=12834543,C=31)
	CT(60326)=(S=12834544,E=12834551,C=12)
	CT(60327)=(S=12834552,E=12834559,C=51)
	CT(60328)=(S=12834560,E=12834815,C=24)
	CT(60329)=(S=12834816,E=12834847,C=14)
	CT(60330)=(S=12834848,E=12834879,C=19)
	CT(60331)=(S=12834880,E=12834911,C=34)
	CT(60332)=(S=12834912,E=12834943,C=58)
	CT(60333)=(S=12834976,E=12835007,C=60)
	CT(60334)=(S=12835008,E=12835009,C=12)
	CT(60335)=(S=12835010,E=12835011,C=18)
	CT(60336)=(S=12835012,E=12835013,C=14)
	CT(60337)=(S=12835014,E=12835015,C=28)
	CT(60338)=(S=12835016,E=12835017,C=18)
	CT(60339)=(S=12835020,E=12835029,C=31)
	CT(60340)=(S=12835030,E=12835031,C=12)
	CT(60341)=(S=12835032,E=12835033,C=18)
	CT(60342)=(S=12835034,E=12835034,C=38)
	CT(60343)=(S=12835035,E=12835035,C=58)
	CT(60344)=(S=12835036,E=12835037,C=28)
	CT(60345)=(S=12835038,E=12835039,C=38)
	CT(60346)=(S=12835040,E=12835040,C=36)
	CT(60347)=(S=12835041,E=12835041,C=29)
	CT(60348)=(S=12835042,E=12835042,C=31)
	CT(60349)=(S=12835043,E=12835043,C=34)
	CT(60350)=(S=12835044,E=12835044,C=51)
	CT(60351)=(S=12835045,E=12835045,C=42)
	CT(60352)=(S=12835046,E=12835046,C=38)
	CT(60353)=(S=12835047,E=12835047,C=19)
	CT(60354)=(S=12835048,E=12835048,C=37)
	CT(60355)=(S=12835049,E=12835049,C=44)
	CT(60356)=(S=12835051,E=12835051,C=28)
	CT(60357)=(S=12835052,E=12835052,C=60)
	CT(60358)=(S=12835053,E=12835053,C=62)
	CT(60359)=(S=12835055,E=12835055,C=38)
	CT(60360)=(S=12835056,E=12835056,C=33)
	CT(60361)=(S=12835057,E=12835057,C=18)
	CT(60362)=(S=12835058,E=12835058,C=12)
	CT(60363)=(S=12835059,E=12835059,C=14)
	CT(60364)=(S=12835060,E=12835060,C=18)
	CT(60365)=(S=12835062,E=12835062,C=36)
	CT(60366)=(S=12835064,E=12835064,C=31)
	CT(60367)=(S=12835065,E=12835065,C=12)
	CT(60368)=(S=12835066,E=12835066,C=58)
	CT(60369)=(S=12835067,E=12835067,C=50)
	CT(60370)=(S=12835068,E=12835068,C=17)
	CT(60371)=(S=12835069,E=12835069,C=52)
	CT(60372)=(S=12835070,E=12835070,C=58)
	CT(60373)=(S=12835071,E=12835071,C=28)
	CT(60374)=(S=12835072,E=12835327,C=14)
	CT(60375)=(S=12835328,E=12835359,C=70)
	CT(60376)=(S=12835360,E=12835391,C=14)
	CT(60377)=(S=12835392,E=12835423,C=47)
	CT(60378)=(S=12835424,E=12835455,C=63)
	CT(60379)=(S=12835456,E=12835583,C=18)
	CT(60380)=(S=12835584,E=12835839,C=13)
	CT(60381)=(S=12835840,E=12836351,C=12)
	CT(60382)=(S=12836352,E=12836383,C=55)
	CT(60383)=(S=12836384,E=12836415,C=59)
	CT(60384)=(S=12836416,E=12836447,C=68)
	CT(60385)=(S=12836448,E=12836451,C=46)
	CT(60386)=(S=12836452,E=12836455,C=58)
	CT(60387)=(S=12836456,E=12836459,C=38)
	CT(60388)=(S=12836460,E=12836463,C=63)
	CT(60389)=(S=12836464,E=12836467,C=58)
	CT(60390)=(S=12836468,E=12836471,C=36)
	CT(60391)=(S=12836472,E=12836475,C=42)
	CT(60392)=(S=12836476,E=12836479,C=31)
	CT(60393)=(S=12836480,E=12836543,C=18)
	CT(60394)=(S=12836544,E=12836607,C=28)
	CT(60395)=(S=12836608,E=12836863,C=15)
	CT(60396)=(S=12836864,E=12837119,C=14)
	CT(60397)=(S=12837120,E=12837151,C=27)
	CT(60398)=(S=12837152,E=12837153,C=34)
	CT(60399)=(S=12837154,E=12837155,C=36)
	CT(60400)=(S=12837156,E=12837157,C=58)
	CT(60401)=(S=12837158,E=12837159,C=18)
	CT(60402)=(S=12837160,E=12837163,C=38)
	CT(60403)=(S=12837164,E=12837165,C=34)
	CT(60404)=(S=12837166,E=12837167,C=17)
	CT(60405)=(S=12837168,E=12837169,C=31)
	CT(60406)=(S=12837170,E=12837170,C=52)
	CT(60407)=(S=12837171,E=12837171,C=51)
	CT(60408)=(S=12837172,E=12837173,C=31)
	CT(60409)=(S=12837174,E=12837175,C=36)
	CT(60410)=(S=12837176,E=12837177,C=18)
	CT(60411)=(S=12837178,E=12837179,C=38)
	CT(60412)=(S=12837180,E=12837181,C=34)
	CT(60413)=(S=12837182,E=12837183,C=13)
	CT(60414)=(S=12837188,E=12837191,C=58)
	CT(60415)=(S=12837192,E=12837195,C=33)
	CT(60416)=(S=12837196,E=12837199,C=58)
	CT(60417)=(S=12837200,E=12837203,C=14)
	CT(60418)=(S=12837204,E=12837207,C=38)
	CT(60419)=(S=12837208,E=12837211,C=24)
	CT(60420)=(S=12837212,E=12837215,C=58)
	CT(60421)=(S=12837216,E=12837223,C=36)
	CT(60422)=(S=12837224,E=12837227,C=28)
	CT(60423)=(S=12837228,E=12837231,C=18)
	CT(60424)=(S=12837232,E=12837235,C=31)
	CT(60425)=(S=12837236,E=12837239,C=34)
	CT(60426)=(S=12837240,E=12837243,C=58)
	CT(60427)=(S=12837244,E=12837247,C=51)
	CT(60428)=(S=12837248,E=12837251,C=18)
	CT(60429)=(S=12837252,E=12837255,C=28)
	CT(60430)=(S=12837256,E=12837259,C=58)
	CT(60431)=(S=12837260,E=12837263,C=38)
	CT(60432)=(S=12837264,E=12837267,C=31)
	CT(60433)=(S=12837268,E=12837275,C=28)
	CT(60434)=(S=12837276,E=12837279,C=31)
	CT(60435)=(S=12837280,E=12837283,C=18)
	CT(60436)=(S=12837284,E=12837287,C=49)
	CT(60437)=(S=12837288,E=12837291,C=15)
	CT(60438)=(S=12837292,E=12837295,C=31)
	CT(60439)=(S=12837296,E=12837299,C=209)
	CT(60440)=(S=12837304,E=12837307,C=19)
	CT(60441)=(S=12837308,E=12837311,C=14)
	CT(60442)=(S=12837312,E=12837315,C=34)
	CT(60443)=(S=12837316,E=12837319,C=28)
	CT(60444)=(S=12837320,E=12837323,C=56)
	CT(60445)=(S=12837324,E=12837327,C=12)
	CT(60446)=(S=12837328,E=12837331,C=28)
	CT(60447)=(S=12837332,E=12837335,C=58)
	CT(60448)=(S=12837336,E=12837339,C=14)
	CT(60449)=(S=12837344,E=12837347,C=15)
	CT(60450)=(S=12837348,E=12837351,C=31)
	CT(60451)=(S=12837352,E=12837355,C=18)
	CT(60452)=(S=12837356,E=12837359,C=42)
	CT(60453)=(S=12837360,E=12837363,C=28)
	CT(60454)=(S=12837364,E=12837371,C=58)
	CT(60455)=(S=12837372,E=12837375,C=51)
	CT(60456)=(S=12837376,E=12837407,C=34)
	CT(60457)=(S=12837408,E=12837439,C=14)
	CT(60458)=(S=12837440,E=12837503,C=28)
	CT(60459)=(S=12837504,E=12837535,C=14)
	CT(60460)=(S=12837536,E=12837567,C=28)
	CT(60461)=(S=12837568,E=12837568,C=31)
	CT(60462)=(S=12837569,E=12837569,C=15)
	CT(60463)=(S=12837570,E=12837570,C=28)
	CT(60464)=(S=12837571,E=12837571,C=42)
	CT(60465)=(S=12837572,E=12837572,C=37)
	CT(60466)=(S=12837573,E=12837573,C=31)
	CT(60467)=(S=12837574,E=12837574,C=58)
	CT(60468)=(S=12837575,E=12837575,C=24)
	CT(60469)=(S=12837576,E=12837576,C=28)
	CT(60470)=(S=12837577,E=12837577,C=14)
	CT(60471)=(S=12837578,E=12837578,C=47)
	CT(60472)=(S=12837579,E=12837580,C=18)
	CT(60473)=(S=12837581,E=12837581,C=36)
	CT(60474)=(S=12837582,E=12837582,C=38)
	CT(60475)=(S=12837583,E=12837583,C=58)
	CT(60476)=(S=12837584,E=12837584,C=20)
	CT(60477)=(S=12837585,E=12837585,C=18)
	CT(60478)=(S=12837586,E=12837586,C=53)
	CT(60479)=(S=12837587,E=12837587,C=12)
	CT(60480)=(S=12837588,E=12837588,C=38)
	CT(60481)=(S=12837589,E=12837589,C=31)
	CT(60482)=(S=12837590,E=12837590,C=38)
	CT(60483)=(S=12837592,E=12837592,C=18)
	CT(60484)=(S=12837593,E=12837593,C=33)
	CT(60485)=(S=12837594,E=12837594,C=60)
	CT(60486)=(S=12837595,E=12837595,C=12)
	CT(60487)=(S=12837597,E=12837597,C=37)
	CT(60488)=(S=12837598,E=12837598,C=18)
	CT(60489)=(S=12837599,E=12837599,C=15)
	CT(60490)=(S=12837600,E=12837631,C=101)
	CT(60491)=(S=12837632,E=12837887,C=28)
	CT(60492)=(S=12837888,E=12838143,C=50)
	CT(60493)=(S=12838144,E=12838399,C=16)
	CT(60494)=(S=12838400,E=12838431,C=51)
	CT(60495)=(S=12838432,E=12838463,C=42)
	CT(60496)=(S=12838464,E=12838495,C=18)
	CT(60497)=(S=12838496,E=12838496,C=31)
	CT(60498)=(S=12838497,E=12838497,C=20)
	CT(60499)=(S=12838498,E=12838498,C=44)
	CT(60500)=(S=12838499,E=12838499,C=18)
	CT(60501)=(S=12838500,E=12838500,C=34)
	CT(60502)=(S=12838501,E=12838501,C=18)
	CT(60503)=(S=12838502,E=12838502,C=22)
	CT(60504)=(S=12838503,E=12838503,C=31)
	CT(60505)=(S=12838504,E=12838504,C=49)
	CT(60506)=(S=12838505,E=12838505,C=25)
	CT(60507)=(S=12838506,E=12838506,C=50)
	CT(60508)=(S=12838508,E=12838508,C=29)
	CT(60509)=(S=12838509,E=12838509,C=18)
	CT(60510)=(S=12838510,E=12838510,C=36)
	CT(60511)=(S=12838511,E=12838511,C=18)
	CT(60512)=(S=12838512,E=12838512,C=31)
	CT(60513)=(S=12838513,E=12838513,C=58)
	CT(60514)=(S=12838514,E=12838514,C=28)
	CT(60515)=(S=12838515,E=12838515,C=31)
	CT(60516)=(S=12838516,E=12838516,C=28)
	CT(60517)=(S=12838517,E=12838517,C=34)
	CT(60518)=(S=12838518,E=12838518,C=14)
	CT(60519)=(S=12838519,E=12838519,C=31)
	CT(60520)=(S=12838520,E=12838520,C=58)
	CT(60521)=(S=12838521,E=12838521,C=52)
	CT(60522)=(S=12838522,E=12838522,C=14)
	CT(60523)=(S=12838523,E=12838523,C=18)
	CT(60524)=(S=12838524,E=12838524,C=71)
	CT(60525)=(S=12838525,E=12838525,C=31)
	CT(60526)=(S=12838526,E=12838526,C=28)
	CT(60527)=(S=12838527,E=12838559,C=31)
	CT(60528)=(S=12838560,E=12838591,C=42)
	CT(60529)=(S=12838592,E=12838655,C=15)
	CT(60530)=(S=12838656,E=12838911,C=24)
	CT(60531)=(S=12838912,E=12839039,C=14)
	CT(60532)=(S=12839040,E=12839423,C=28)
	CT(60533)=(S=12839424,E=12839425,C=56)
	CT(60534)=(S=12839426,E=12839427,C=18)
	CT(60535)=(S=12839428,E=12839431,C=31)
	CT(60536)=(S=12839432,E=12839433,C=58)
	CT(60537)=(S=12839434,E=12839435,C=14)
	CT(60538)=(S=12839436,E=12839437,C=28)
	CT(60539)=(S=12839438,E=12839439,C=19)
	CT(60540)=(S=12839440,E=12839441,C=58)
	CT(60541)=(S=12839442,E=12839443,C=42)
	CT(60542)=(S=12839444,E=12839445,C=58)
	CT(60543)=(S=12839446,E=12839447,C=57)
	CT(60544)=(S=12839448,E=12839449,C=13)
	CT(60545)=(S=12839450,E=12839451,C=17)
	CT(60546)=(S=12839452,E=12839453,C=28)
	CT(60547)=(S=12839454,E=12839455,C=38)
	CT(60548)=(S=12839456,E=12839456,C=49)
	CT(60549)=(S=12839457,E=12839458,C=28)
	CT(60550)=(S=12839459,E=12839460,C=12)
	CT(60551)=(S=12839461,E=12839462,C=34)
	CT(60552)=(S=12839463,E=12839463,C=14)
	CT(60553)=(S=12839464,E=12839464,C=58)
	CT(60554)=(S=12839465,E=12839466,C=12)
	CT(60555)=(S=12839467,E=12839467,C=34)
	CT(60556)=(S=12839468,E=12839468,C=42)
	CT(60557)=(S=12839469,E=12839469,C=49)
	CT(60558)=(S=12839470,E=12839470,C=12)
	CT(60559)=(S=12839471,E=12839471,C=21)
	CT(60560)=(S=12839472,E=12839472,C=14)
	CT(60561)=(S=12839473,E=12839473,C=22)
	CT(60562)=(S=12839474,E=12839474,C=28)
	CT(60563)=(S=12839475,E=12839475,C=72)
	CT(60564)=(S=12839476,E=12839476,C=37)
	CT(60565)=(S=12839477,E=12839477,C=49)
	CT(60566)=(S=12839478,E=12839479,C=38)
	CT(60567)=(S=12839480,E=12839480,C=18)
	CT(60568)=(S=12839481,E=12839481,C=19)
	CT(60569)=(S=12839482,E=12839482,C=42)
	CT(60570)=(S=12839483,E=12839483,C=25)
	CT(60571)=(S=12839484,E=12839484,C=34)
	CT(60572)=(S=12839485,E=12839485,C=25)
	CT(60573)=(S=12839488,E=12839491,C=34)
	CT(60574)=(S=12839492,E=12839503,C=31)
	CT(60575)=(S=12839504,E=12839505,C=14)
	CT(60576)=(S=12839508,E=12839511,C=51)
	CT(60577)=(S=12839512,E=12839515,C=12)
	CT(60578)=(S=12839520,E=12839523,C=18)
	CT(60579)=(S=12839524,E=12839527,C=42)
	CT(60580)=(S=12839528,E=12839531,C=28)
	CT(60581)=(S=12839536,E=12839539,C=31)
	CT(60582)=(S=12839540,E=12839543,C=14)
	CT(60583)=(S=12839552,E=12839552,C=28)
	CT(60584)=(S=12839553,E=12839553,C=34)
	CT(60585)=(S=12839555,E=12839555,C=60)
	CT(60586)=(S=12839556,E=12839556,C=31)
	CT(60587)=(S=12839557,E=12839557,C=28)
	CT(60588)=(S=12839558,E=12839558,C=21)
	CT(60589)=(S=12839559,E=12839559,C=47)
	CT(60590)=(S=12839560,E=12839560,C=28)
	CT(60591)=(S=12839561,E=12839561,C=52)
	CT(60592)=(S=12839562,E=12839562,C=51)
	CT(60593)=(S=12839563,E=12839563,C=28)
	CT(60594)=(S=12839565,E=12839565,C=58)
	CT(60595)=(S=12839566,E=12839566,C=47)
	CT(60596)=(S=12839568,E=12839568,C=60)
	CT(60597)=(S=12839569,E=12839569,C=49)
	CT(60598)=(S=12839570,E=12839570,C=19)
	CT(60599)=(S=12839572,E=12839572,C=31)
	CT(60600)=(S=12839573,E=12839573,C=28)
	CT(60601)=(S=12839574,E=12839574,C=19)
	CT(60602)=(S=12839575,E=12839575,C=42)
	CT(60603)=(S=12839576,E=12839576,C=28)
	CT(60604)=(S=12839577,E=12839577,C=14)
	CT(60605)=(S=12839578,E=12839578,C=12)
	CT(60606)=(S=12839579,E=12839579,C=24)
	CT(60607)=(S=12839581,E=12839581,C=42)
	CT(60608)=(S=12839582,E=12839582,C=17)
	CT(60609)=(S=12839584,E=12839584,C=24)
	CT(60610)=(S=12839586,E=12839586,C=49)
	CT(60611)=(S=12839587,E=12839587,C=34)
	CT(60612)=(S=12839588,E=12839588,C=58)
	CT(60613)=(S=12839589,E=12839589,C=37)
	CT(60614)=(S=12839590,E=12839591,C=25)
	CT(60615)=(S=12839592,E=12839592,C=106)
	CT(60616)=(S=12839594,E=12839594,C=18)
	CT(60617)=(S=12839595,E=12839595,C=15)
	CT(60618)=(S=12839596,E=12839596,C=38)
	CT(60619)=(S=12839597,E=12839597,C=36)
	CT(60620)=(S=12839598,E=12839598,C=31)
	CT(60621)=(S=12839599,E=12839599,C=14)
	CT(60622)=(S=12839600,E=12839600,C=34)
	CT(60623)=(S=12839601,E=12839601,C=38)
	CT(60624)=(S=12839602,E=12839602,C=28)
	CT(60625)=(S=12839604,E=12839604,C=34)
	CT(60626)=(S=12839606,E=12839606,C=38)
	CT(60627)=(S=12839607,E=12839607,C=28)
	CT(60628)=(S=12839608,E=12839608,C=49)
	CT(60629)=(S=12839609,E=12839609,C=106)
	CT(60630)=(S=12839610,E=12839610,C=49)
	CT(60631)=(S=12839611,E=12839611,C=36)
	CT(60632)=(S=12839612,E=12839612,C=38)
	CT(60633)=(S=12839614,E=12839614,C=18)
	CT(60634)=(S=12839616,E=12839619,C=28)
	CT(60635)=(S=12839620,E=12839623,C=47)
	CT(60636)=(S=12839624,E=12839627,C=31)
	CT(60637)=(S=12839628,E=12839631,C=33)
	CT(60638)=(S=12839632,E=12839635,C=18)
	CT(60639)=(S=12839636,E=12839639,C=31)
	CT(60640)=(S=12839640,E=12839643,C=28)
	CT(60641)=(S=12839644,E=12839647,C=31)
	CT(60642)=(S=12839648,E=12839651,C=34)
	CT(60643)=(S=12839652,E=12839655,C=28)
	CT(60644)=(S=12839656,E=12839659,C=15)
	CT(60645)=(S=12839660,E=12839663,C=51)
	CT(60646)=(S=12839664,E=12839667,C=14)
	CT(60647)=(S=12839668,E=12839671,C=28)
	CT(60648)=(S=12839672,E=12839675,C=12)
	CT(60649)=(S=12839676,E=12839679,C=106)
	CT(60650)=(S=12839680,E=12839935,C=25)
	CT(60651)=(S=12839936,E=12840447,C=47)
	CT(60652)=(S=12840448,E=12840479,C=49)
	CT(60653)=(S=12840480,E=12840511,C=28)
	CT(60654)=(S=12840512,E=12840513,C=44)
	CT(60655)=(S=12840514,E=12840519,C=58)
	CT(60656)=(S=12840520,E=12840521,C=18)
	CT(60657)=(S=12840522,E=12840523,C=36)
	CT(60658)=(S=12840524,E=12840525,C=19)
	CT(60659)=(S=12840528,E=12840531,C=38)
	CT(60660)=(S=12840532,E=12840533,C=51)
	CT(60661)=(S=12840534,E=12840535,C=36)
	CT(60662)=(S=12840536,E=12840539,C=38)
	CT(60663)=(S=12840540,E=12840541,C=31)
	CT(60664)=(S=12840544,E=12840547,C=12)
	CT(60665)=(S=12840548,E=12840551,C=18)
	CT(60666)=(S=12840552,E=12840555,C=98)
	CT(60667)=(S=12840556,E=12840567,C=31)
	CT(60668)=(S=12840576,E=12840611,C=28)
	CT(60669)=(S=12840612,E=12840619,C=58)
	CT(60670)=(S=12840620,E=12840623,C=14)
	CT(60671)=(S=12840624,E=12840631,C=31)
	CT(60672)=(S=12840632,E=12840635,C=58)
	CT(60673)=(S=12840636,E=12840639,C=31)
	CT(60674)=(S=12840640,E=12840671,C=47)
	CT(60675)=(S=12840672,E=12840673,C=28)
	CT(60676)=(S=12840674,E=12840674,C=12)
	CT(60677)=(S=12840675,E=12840676,C=58)
	CT(60678)=(S=12840677,E=12840677,C=12)
	CT(60679)=(S=12840678,E=12840678,C=18)
	CT(60680)=(S=12840680,E=12840680,C=14)
	CT(60681)=(S=12840681,E=12840681,C=28)
	CT(60682)=(S=12840682,E=12840682,C=27)
	CT(60683)=(S=12840683,E=12840683,C=15)
	CT(60684)=(S=12840684,E=12840684,C=58)
	CT(60685)=(S=12840685,E=12840685,C=14)
	CT(60686)=(S=12840686,E=12840686,C=28)
	CT(60687)=(S=12840687,E=12840687,C=18)
	CT(60688)=(S=12840688,E=12840689,C=14)
	CT(60689)=(S=12840692,E=12840692,C=36)
	CT(60690)=(S=12840693,E=12840693,C=58)
	CT(60691)=(S=12840694,E=12840694,C=18)
	CT(60692)=(S=12840697,E=12840697,C=14)
	CT(60693)=(S=12840698,E=12840698,C=18)
	CT(60694)=(S=12840699,E=12840699,C=12)
	CT(60695)=(S=12840701,E=12840701,C=34)
	CT(60696)=(S=12840702,E=12840703,C=19)
	CT(60697)=(S=12840704,E=12840959,C=18)
	CT(60698)=(S=12840960,E=12841471,C=36)
	CT(60699)=(S=12841472,E=12841503,C=18)
	CT(60700)=(S=12841504,E=12841535,C=19)
	CT(60701)=(S=12841536,E=12841537,C=58)
	CT(60702)=(S=12841538,E=12841539,C=28)
	CT(60703)=(S=12841540,E=12841541,C=58)
	CT(60704)=(S=12841542,E=12841543,C=14)
	CT(60705)=(S=12841544,E=12841545,C=36)
	CT(60706)=(S=12841548,E=12841549,C=46)
	CT(60707)=(S=12841550,E=12841551,C=38)
	CT(60708)=(S=12841552,E=12841553,C=31)
	CT(60709)=(S=12841554,E=12841555,C=18)
	CT(60710)=(S=12841556,E=12841557,C=34)
	CT(60711)=(S=12841558,E=12841559,C=14)
	CT(60712)=(S=12841560,E=12841561,C=38)
	CT(60713)=(S=12841562,E=12841565,C=58)
	CT(60714)=(S=12841566,E=12841567,C=31)
	CT(60715)=(S=12841568,E=12841571,C=36)
	CT(60716)=(S=12841576,E=12841577,C=28)
	CT(60717)=(S=12841578,E=12841579,C=51)
	CT(60718)=(S=12841580,E=12841581,C=58)
	CT(60719)=(S=12841584,E=12841587,C=31)
	CT(60720)=(S=12841588,E=12841589,C=58)
	CT(60721)=(S=12841592,E=12841593,C=24)
	CT(60722)=(S=12841594,E=12841595,C=14)
	CT(60723)=(S=12841596,E=12841597,C=58)
	CT(60724)=(S=12841598,E=12841599,C=51)
	CT(60725)=(S=12841600,E=12841601,C=58)
	CT(60726)=(S=12841602,E=12841603,C=24)
	CT(60727)=(S=12841604,E=12841605,C=19)
	CT(60728)=(S=12841606,E=12841607,C=58)
	CT(60729)=(S=12841608,E=12841609,C=25)
	CT(60730)=(S=12841610,E=12841611,C=58)
	CT(60731)=(S=12841612,E=12841613,C=34)
	CT(60732)=(S=12841614,E=12841617,C=18)
	CT(60733)=(S=12841618,E=12841619,C=19)
	CT(60734)=(S=12841622,E=12841623,C=31)
	CT(60735)=(S=12841624,E=12841625,C=36)
	CT(60736)=(S=12841626,E=12841629,C=14)
	CT(60737)=(S=12841630,E=12841631,C=38)
	CT(60738)=(S=12841632,E=12841632,C=36)
	CT(60739)=(S=12841633,E=12841633,C=31)
	CT(60740)=(S=12841634,E=12841634,C=28)
	CT(60741)=(S=12841635,E=12841637,C=34)
	CT(60742)=(S=12841638,E=12841638,C=42)
	CT(60743)=(S=12841639,E=12841639,C=28)
	CT(60744)=(S=12841640,E=12841640,C=42)
	CT(60745)=(S=12841641,E=12841641,C=102)
	CT(60746)=(S=12841642,E=12841642,C=32)
	CT(60747)=(S=12841643,E=12841643,C=36)
	CT(60748)=(S=12841644,E=12841644,C=49)
	CT(60749)=(S=12841645,E=12841645,C=28)
	CT(60750)=(S=12841646,E=12841646,C=34)
	CT(60751)=(S=12841647,E=12841647,C=42)
	CT(60752)=(S=12841648,E=12841648,C=58)
	CT(60753)=(S=12841650,E=12841650,C=49)
	CT(60754)=(S=12841651,E=12841651,C=31)
	CT(60755)=(S=12841652,E=12841652,C=12)
	CT(60756)=(S=12841653,E=12841653,C=15)
	CT(60757)=(S=12841655,E=12841655,C=58)
	CT(60758)=(S=12841656,E=12841656,C=42)
	CT(60759)=(S=12841657,E=12841657,C=71)
	CT(60760)=(S=12841658,E=12841658,C=58)
	CT(60761)=(S=12841660,E=12841660,C=23)
	CT(60762)=(S=12841661,E=12841661,C=19)
	CT(60763)=(S=12841662,E=12841662,C=28)
	CT(60764)=(S=12841663,E=12841663,C=12)
	CT(60765)=(S=12841664,E=12841667,C=24)
	CT(60766)=(S=12841668,E=12841671,C=23)
	CT(60767)=(S=12841672,E=12841675,C=31)
	CT(60768)=(S=12841676,E=12841679,C=28)
	CT(60769)=(S=12841684,E=12841687,C=14)
	CT(60770)=(S=12841688,E=12841689,C=18)
	CT(60771)=(S=12841690,E=12841691,C=31)
	CT(60772)=(S=12841692,E=12841695,C=14)
	CT(60773)=(S=12841696,E=12841703,C=28)
	CT(60774)=(S=12841704,E=12841707,C=58)
	CT(60775)=(S=12841708,E=12841711,C=14)
	CT(60776)=(S=12841716,E=12841719,C=38)
	CT(60777)=(S=12841720,E=12841723,C=56)
	CT(60778)=(S=12841724,E=12841727,C=58)
	CT(60779)=(S=12841728,E=12841985,C=28)
	CT(60780)=(S=12841988,E=12841989,C=31)
	CT(60781)=(S=12841990,E=12841991,C=19)
	CT(60782)=(S=12841992,E=12841993,C=31)
	CT(60783)=(S=12841994,E=12841995,C=28)
	CT(60784)=(S=12841996,E=12841997,C=44)
	CT(60785)=(S=12842000,E=12842001,C=14)
	CT(60786)=(S=12842004,E=12842005,C=12)
	CT(60787)=(S=12842006,E=12842007,C=17)
	CT(60788)=(S=12842008,E=12842009,C=18)
	CT(60789)=(S=12842010,E=12842011,C=58)
	CT(60790)=(S=12842012,E=12842013,C=14)
	CT(60791)=(S=12842014,E=12842015,C=12)
	CT(60792)=(S=12842016,E=12842047,C=37)
	CT(60793)=(S=12842048,E=12842079,C=47)
	CT(60794)=(S=12842080,E=12842111,C=28)
	CT(60795)=(S=12842112,E=12842143,C=60)
	CT(60796)=(S=12842144,E=12842175,C=49)
	CT(60797)=(S=12842176,E=12842207,C=69)
	CT(60798)=(S=12842208,E=12842305,C=28)
	CT(60799)=(S=12842306,E=12842309,C=58)
	CT(60800)=(S=12842310,E=12842311,C=26)
	CT(60801)=(S=12842312,E=12842313,C=18)
	CT(60802)=(S=12842314,E=12842315,C=19)
	CT(60803)=(S=12842316,E=12842317,C=18)
	CT(60804)=(S=12842318,E=12842319,C=71)
	CT(60805)=(S=12842320,E=12842321,C=31)
	CT(60806)=(S=12842322,E=12842323,C=38)
	CT(60807)=(S=12842324,E=12842325,C=58)
	CT(60808)=(S=12842326,E=12842327,C=28)
	CT(60809)=(S=12842330,E=12842331,C=63)
	CT(60810)=(S=12842332,E=12842333,C=42)
	CT(60811)=(S=12842334,E=12842335,C=58)
	CT(60812)=(S=12842340,E=12842341,C=14)
	CT(60813)=(S=12842344,E=12842345,C=58)
	CT(60814)=(S=12842346,E=12842347,C=52)
	CT(60815)=(S=12842348,E=12842349,C=13)
	CT(60816)=(S=12842352,E=12842353,C=14)
	CT(60817)=(S=12842354,E=12842355,C=49)
	CT(60818)=(S=12842356,E=12842357,C=15)
	CT(60819)=(S=12842358,E=12842361,C=31)
	CT(60820)=(S=12842362,E=12842363,C=36)
	CT(60821)=(S=12842364,E=12842367,C=13)
	CT(60822)=(S=12842368,E=12842431,C=21)
	CT(60823)=(S=12842432,E=12842432,C=28)
	CT(60824)=(S=12842433,E=12842433,C=24)
	CT(60825)=(S=12842434,E=12842434,C=60)
	CT(60826)=(S=12842436,E=12842436,C=25)
	CT(60827)=(S=12842437,E=12842437,C=21)
	CT(60828)=(S=12842438,E=12842438,C=58)
	CT(60829)=(S=12842439,E=12842439,C=36)
	CT(60830)=(S=12842440,E=12842440,C=31)
	CT(60831)=(S=12842441,E=12842441,C=14)
	CT(60832)=(S=12842442,E=12842442,C=27)
	CT(60833)=(S=12842443,E=12842444,C=58)
	CT(60834)=(S=12842445,E=12842446,C=18)
	CT(60835)=(S=12842448,E=12842448,C=52)
	CT(60836)=(S=12842449,E=12842449,C=33)
	CT(60837)=(S=12842450,E=12842450,C=24)
	CT(60838)=(S=12842451,E=12842451,C=18)
	CT(60839)=(S=12842452,E=12842452,C=25)
	CT(60840)=(S=12842453,E=12842453,C=58)
	CT(60841)=(S=12842454,E=12842454,C=18)
	CT(60842)=(S=12842455,E=12842455,C=31)
	CT(60843)=(S=12842456,E=12842456,C=52)
	CT(60844)=(S=12842457,E=12842457,C=58)
	CT(60845)=(S=12842458,E=12842458,C=56)
	CT(60846)=(S=12842459,E=12842459,C=53)
	CT(60847)=(S=12842460,E=12842460,C=38)
	CT(60848)=(S=12842461,E=12842461,C=31)
	CT(60849)=(S=12842462,E=12842462,C=58)
	CT(60850)=(S=12842463,E=12842463,C=14)
	CT(60851)=(S=12842464,E=12842464,C=58)
	CT(60852)=(S=12842465,E=12842465,C=42)
	CT(60853)=(S=12842466,E=12842466,C=58)
	CT(60854)=(S=12842467,E=12842467,C=37)
	CT(60855)=(S=12842468,E=12842468,C=34)
	CT(60856)=(S=12842470,E=12842470,C=28)
	CT(60857)=(S=12842471,E=12842471,C=36)
	CT(60858)=(S=12842472,E=12842472,C=18)
	CT(60859)=(S=12842473,E=12842473,C=71)
	CT(60860)=(S=12842474,E=12842474,C=21)
	CT(60861)=(S=12842475,E=12842475,C=14)
	CT(60862)=(S=12842476,E=12842476,C=36)
	CT(60863)=(S=12842477,E=12842477,C=34)
	CT(60864)=(S=12842480,E=12842480,C=19)
	CT(60865)=(S=12842483,E=12842483,C=28)
	CT(60866)=(S=12842484,E=12842484,C=49)
	CT(60867)=(S=12842485,E=12842485,C=28)
	CT(60868)=(S=12842487,E=12842487,C=14)
	CT(60869)=(S=12842488,E=12842488,C=25)
	CT(60870)=(S=12842489,E=12842489,C=18)
	CT(60871)=(S=12842490,E=12842490,C=52)
	CT(60872)=(S=12842491,E=12842491,C=38)
	CT(60873)=(S=12842492,E=12842492,C=14)
	CT(60874)=(S=12842493,E=12842493,C=31)
	CT(60875)=(S=12842494,E=12842494,C=28)
	CT(60876)=(S=12842495,E=12842495,C=57)
	CT(60877)=(S=12842496,E=12842527,C=52)
	CT(60878)=(S=12842528,E=12842559,C=106)
	CT(60879)=(S=12842560,E=12842591,C=34)
	CT(60880)=(S=12842592,E=12842593,C=33)
	CT(60881)=(S=12842594,E=12842595,C=36)
	CT(60882)=(S=12842596,E=12842597,C=70)
	CT(60883)=(S=12842598,E=12842599,C=18)
	CT(60884)=(S=12842600,E=12842601,C=23)
	CT(60885)=(S=12842602,E=12842603,C=18)
	CT(60886)=(S=12842604,E=12842605,C=14)
	CT(60887)=(S=12842608,E=12842609,C=49)
	CT(60888)=(S=12842610,E=12842611,C=36)
	CT(60889)=(S=12842612,E=12842613,C=13)
	CT(60890)=(S=12842614,E=12842615,C=18)
	CT(60891)=(S=12842616,E=12842617,C=19)
	CT(60892)=(S=12842618,E=12842619,C=18)
	CT(60893)=(S=12842620,E=12842621,C=31)
	CT(60894)=(S=12842622,E=12842623,C=58)
	CT(60895)=(S=12842624,E=12842655,C=14)
	CT(60896)=(S=12842656,E=12842687,C=28)
	CT(60897)=(S=12842688,E=12842692,C=15)
	CT(60898)=(S=12842694,E=12842694,C=24)
	CT(60899)=(S=12842695,E=12842695,C=50)
	CT(60900)=(S=12842704,E=12842711,C=15)
	CT(60901)=(S=12842712,E=12842712,C=58)
	CT(60902)=(S=12842713,E=12842713,C=31)
	CT(60903)=(S=12842714,E=12842714,C=18)
	CT(60904)=(S=12842716,E=12842719,C=15)
	CT(60905)=(S=12842720,E=12842721,C=58)
	CT(60906)=(S=12842722,E=12842723,C=60)
	CT(60907)=(S=12842724,E=12842725,C=14)
	CT(60908)=(S=12842726,E=12842727,C=12)
	CT(60909)=(S=12842728,E=12842729,C=14)
	CT(60910)=(S=12842730,E=12842731,C=28)
	CT(60911)=(S=12842732,E=12842733,C=36)
	CT(60912)=(S=12842734,E=12842735,C=21)
	CT(60913)=(S=12842736,E=12842737,C=51)
	CT(60914)=(S=12842738,E=12842739,C=38)
	CT(60915)=(S=12842740,E=12842741,C=34)
	CT(60916)=(S=12842746,E=12842747,C=38)
	CT(60917)=(S=12842748,E=12843007,C=28)
	CT(60918)=(S=12843008,E=12843039,C=12)
	CT(60919)=(S=12843040,E=12843071,C=42)
	CT(60920)=(S=12843072,E=12843072,C=60)
	CT(60921)=(S=12843073,E=12843073,C=38)
	CT(60922)=(S=12843074,E=12843074,C=18)
	CT(60923)=(S=12843075,E=12843075,C=28)
	CT(60924)=(S=12843076,E=12843076,C=52)
	CT(60925)=(S=12843077,E=12843077,C=18)
	CT(60926)=(S=12843078,E=12843078,C=14)
	CT(60927)=(S=12843079,E=12843079,C=28)
	CT(60928)=(S=12843080,E=12843080,C=58)
	CT(60929)=(S=12843081,E=12843081,C=42)
	CT(60930)=(S=12843082,E=12843082,C=28)
	CT(60931)=(S=12843083,E=12843083,C=14)
	CT(60932)=(S=12843084,E=12843084,C=38)
	CT(60933)=(S=12843085,E=12843085,C=36)
	CT(60934)=(S=12843086,E=12843086,C=27)
	CT(60935)=(S=12843087,E=12843087,C=38)
	CT(60936)=(S=12843088,E=12843088,C=63)
	CT(60937)=(S=12843089,E=12843089,C=14)
	CT(60938)=(S=12843090,E=12843090,C=18)
	CT(60939)=(S=12843091,E=12843091,C=34)
	CT(60940)=(S=12843093,E=12843093,C=42)
	CT(60941)=(S=12843094,E=12843094,C=47)
	CT(60942)=(S=12843095,E=12843095,C=36)
	CT(60943)=(S=12843096,E=12843096,C=58)
	CT(60944)=(S=12843097,E=12843097,C=28)
	CT(60945)=(S=12843098,E=12843098,C=14)
	CT(60946)=(S=12843099,E=12843100,C=34)
	CT(60947)=(S=12843101,E=12843101,C=31)
	CT(60948)=(S=12843102,E=12843102,C=18)
	CT(60949)=(S=12843103,E=12843103,C=50)
	CT(60950)=(S=12843104,E=12843135,C=14)
	CT(60951)=(S=12843136,E=12843167,C=28)
	CT(60952)=(S=12843168,E=12843199,C=31)
	CT(60953)=(S=12843200,E=12843231,C=13)
	CT(60954)=(S=12843232,E=12843233,C=31)
	CT(60955)=(S=12843234,E=12843235,C=58)
	CT(60956)=(S=12843236,E=12843237,C=38)
	CT(60957)=(S=12843238,E=12843239,C=25)
	CT(60958)=(S=12843240,E=12843241,C=18)
	CT(60959)=(S=12843242,E=12843243,C=31)
	CT(60960)=(S=12843244,E=12843245,C=18)
	CT(60961)=(S=12843246,E=12843247,C=36)
	CT(60962)=(S=12843254,E=12843255,C=58)
	CT(60963)=(S=12843256,E=12843257,C=18)
	CT(60964)=(S=12843258,E=12843259,C=12)
	CT(60965)=(S=12843260,E=12843261,C=28)
	CT(60966)=(S=12843262,E=12843263,C=58)
	CT(60967)=(S=12843264,E=12843519,C=24)
	CT(60968)=(S=12843520,E=12843551,C=15)
	CT(60969)=(S=12843552,E=12843552,C=42)
	CT(60970)=(S=12843553,E=12843553,C=17)
	CT(60971)=(S=12843554,E=12843554,C=15)
	CT(60972)=(S=12843555,E=12843555,C=28)
	CT(60973)=(S=12843556,E=12843556,C=31)
	CT(60974)=(S=12843557,E=12843558,C=58)
	CT(60975)=(S=12843559,E=12843559,C=50)
	CT(60976)=(S=12843560,E=12843560,C=12)
	CT(60977)=(S=12843561,E=12843561,C=34)
	CT(60978)=(S=12843562,E=12843562,C=18)
	CT(60979)=(S=12843563,E=12843563,C=31)
	CT(60980)=(S=12843564,E=12843566,C=34)
	CT(60981)=(S=12843567,E=12843567,C=31)
	CT(60982)=(S=12843568,E=12843568,C=28)
	CT(60983)=(S=12843569,E=12843569,C=14)
	CT(60984)=(S=12843570,E=12843570,C=28)
	CT(60985)=(S=12843571,E=12843571,C=18)
	CT(60986)=(S=12843572,E=12843572,C=34)
	CT(60987)=(S=12843573,E=12843573,C=38)
	CT(60988)=(S=12843574,E=12843574,C=58)
	CT(60989)=(S=12843576,E=12843576,C=18)
	CT(60990)=(S=12843577,E=12843577,C=28)
	CT(60991)=(S=12843578,E=12843578,C=52)
	CT(60992)=(S=12843579,E=12843579,C=38)
	CT(60993)=(S=12843580,E=12843580,C=51)
	CT(60994)=(S=12843581,E=12843581,C=19)
	CT(60995)=(S=12843582,E=12843582,C=31)
	CT(60996)=(S=12843584,E=12843615,C=66)
	CT(60997)=(S=12843616,E=12843647,C=46)
	CT(60998)=(S=12843648,E=12843679,C=33)
	CT(60999)=(S=12843680,E=12843711,C=55)
	CT(61000)=(S=12843712,E=12843735,C=52)
	CT(61001)=(S=12843736,E=12843743,C=59)
	CT(61002)=(S=12843744,E=12843775,C=15)
	CT(61003)=(S=12843776,E=12844031,C=22)
	CT(61004)=(S=12844032,E=12844095,C=19)
	CT(61005)=(S=12844096,E=12844159,C=46)
	CT(61006)=(S=12844160,E=12844671,C=28)
	CT(61007)=(S=12844672,E=12844673,C=56)
	CT(61008)=(S=12844674,E=12844675,C=12)
	CT(61009)=(S=12844676,E=12844681,C=38)
	CT(61010)=(S=12844682,E=12844683,C=58)
	CT(61011)=(S=12844684,E=12844685,C=38)
	CT(61012)=(S=12844688,E=12844689,C=34)
	CT(61013)=(S=12844690,E=12844691,C=12)
	CT(61014)=(S=12844692,E=12844693,C=25)
	CT(61015)=(S=12844694,E=12844695,C=31)
	CT(61016)=(S=12844696,E=12844697,C=38)
	CT(61017)=(S=12844700,E=12844701,C=58)
	CT(61018)=(S=12844702,E=12844705,C=12)
	CT(61019)=(S=12844706,E=12844707,C=28)
	CT(61020)=(S=12844708,E=12844709,C=14)
	CT(61021)=(S=12844710,E=12844711,C=38)
	CT(61022)=(S=12844712,E=12844713,C=24)
	CT(61023)=(S=12844714,E=12844715,C=101)
	CT(61024)=(S=12844716,E=12844717,C=27)
	CT(61025)=(S=12844718,E=12844721,C=58)
	CT(61026)=(S=12844722,E=12844725,C=14)
	CT(61027)=(S=12844726,E=12844727,C=38)
	CT(61028)=(S=12844728,E=12844729,C=58)
	CT(61029)=(S=12844730,E=12844733,C=18)
	CT(61030)=(S=12844734,E=12844735,C=42)
	CT(61031)=(S=12844736,E=12844767,C=27)
	CT(61032)=(S=12844768,E=12844799,C=15)
	CT(61033)=(S=12844800,E=12845055,C=47)
	CT(61034)=(S=12845056,E=12845311,C=130)
	CT(61035)=(S=12845312,E=12845312,C=113)
	CT(61036)=(S=12845313,E=12845313,C=4)
	CT(61037)=(S=12845314,E=12845314,C=80)
	CT(61038)=(S=12845315,E=12845315,C=23)
	CT(61039)=(S=12845316,E=12845316,C=109)
	CT(61040)=(S=12845317,E=12845317,C=18)
	CT(61041)=(S=12845318,E=12845318,C=23)
	CT(61042)=(S=12845319,E=12845319,C=113)
	CT(61043)=(S=12845320,E=12845326,C=105)
	CT(61044)=(S=12845327,E=12845327,C=116)
	CT(61045)=(S=12845328,E=12845337,C=105)
	CT(61046)=(S=12845338,E=12845339,C=109)
	CT(61047)=(S=12845340,E=12845343,C=117)
	CT(61048)=(S=12845344,E=12845363,C=105)
	CT(61049)=(S=12845364,E=12845367,C=112)
	CT(61050)=(S=12845368,E=12845375,C=105)
	CT(61051)=(S=12845376,E=12845379,C=23)
	CT(61052)=(S=12845380,E=12845380,C=4)
	CT(61053)=(S=12845381,E=12845382,C=101)
	CT(61054)=(S=12845383,E=12845383,C=105)
	CT(61055)=(S=12845384,E=12845385,C=29)
	CT(61056)=(S=12845386,E=12845403,C=105)
	CT(61057)=(S=12845404,E=12845412,C=148)
	CT(61058)=(S=12845413,E=12845415,C=105)
	CT(61059)=(S=12845416,E=12845418,C=4)
	CT(61060)=(S=12845419,E=12845419,C=178)
	CT(61061)=(S=12845421,E=12845423,C=4)
	CT(61062)=(S=12845424,E=12845424,C=219)
	CT(61063)=(S=12845425,E=12845426,C=4)
	CT(61064)=(S=12845427,E=12845427,C=105)
	CT(61065)=(S=12845430,E=12845430,C=110)
	CT(61066)=(S=12845431,E=12845431,C=106)
	CT(61067)=(S=12845432,E=12845439,C=147)
	CT(61068)=(S=12845440,E=12845442,C=105)
	CT(61069)=(S=12845443,E=12845444,C=109)
	CT(61070)=(S=12845445,E=12845445,C=111)
	CT(61071)=(S=12845446,E=12845446,C=4)
	CT(61072)=(S=12845447,E=12845447,C=219)
	CT(61073)=(S=12845448,E=12845448,C=175)
	CT(61074)=(S=12845449,E=12845449,C=137)
	CT(61075)=(S=12845450,E=12845451,C=175)
	CT(61076)=(S=12845452,E=12845452,C=29)
	CT(61077)=(S=12845453,E=12845453,C=80)
	CT(61078)=(S=12845454,E=12845454,C=29)
	CT(61079)=(S=12845455,E=12845455,C=106)
	CT(61080)=(S=12845456,E=12845471,C=105)
	CT(61081)=(S=12845472,E=12845487,C=79)
	CT(61082)=(S=12845504,E=12845567,C=119)
	CT(61083)=(S=12845568,E=12845569,C=175)
	CT(61084)=(S=12845570,E=12845570,C=126)
	CT(61085)=(S=12845571,E=12845571,C=113)
	CT(61086)=(S=12845572,E=12845575,C=30)
	CT(61087)=(S=12845576,E=12845583,C=145)
	CT(61088)=(S=12845584,E=12845647,C=105)
	CT(61089)=(S=12845648,E=12845663,C=134)
	CT(61090)=(S=12845664,E=12845759,C=105)
	CT(61091)=(S=12845760,E=12845791,C=106)
	CT(61092)=(S=12845792,E=12845823,C=105)
	CT(61093)=(S=12845824,E=12845831,C=175)
	CT(61094)=(S=12845832,E=12845837,C=80)
	CT(61095)=(S=12845838,E=12845839,C=106)
	CT(61096)=(S=12845840,E=12845855,C=207)
	CT(61097)=(S=12845856,E=12845880,C=29)
	CT(61098)=(S=12845881,E=12845881,C=105)
	CT(61099)=(S=12845882,E=12845883,C=109)
	CT(61100)=(S=12845884,E=12845887,C=111)
	CT(61101)=(S=12845888,E=12845888,C=110)
	CT(61102)=(S=12845889,E=12845889,C=4)
	CT(61103)=(S=12845890,E=12845890,C=23)
	CT(61104)=(S=12845891,E=12845895,C=16)
	CT(61105)=(S=12845896,E=12845896,C=77)
	CT(61106)=(S=12845897,E=12845897,C=186)
	CT(61107)=(S=12845898,E=12845913,C=177)
	CT(61108)=(S=12845914,E=12845914,C=120)
	CT(61109)=(S=12845915,E=12845915,C=26)
	CT(61110)=(S=12845916,E=12845917,C=105)
	CT(61111)=(S=12845918,E=12845918,C=117)
	CT(61112)=(S=12845919,E=12845919,C=175)
	CT(61113)=(S=12845920,E=12845927,C=131)
	CT(61114)=(S=12845928,E=12845928,C=175)
	CT(61115)=(S=12845929,E=12845929,C=105)
	CT(61116)=(S=12845930,E=12845930,C=177)
	CT(61117)=(S=12845931,E=12845932,C=105)
	CT(61118)=(S=12845933,E=12845933,C=113)
	CT(61119)=(S=12845934,E=12845955,C=105)
	CT(61120)=(S=12845956,E=12845971,C=176)
	CT(61121)=(S=12845972,E=12845975,C=105)
	CT(61122)=(S=12845976,E=12845976,C=219)
	CT(61123)=(S=12845977,E=12845977,C=175)
	CT(61124)=(S=12845978,E=12846003,C=105)
	CT(61125)=(S=12846008,E=12846015,C=175)
	CT(61126)=(S=12846016,E=12846047,C=79)
	CT(61127)=(S=12846048,E=12846115,C=105)
	CT(61128)=(S=12846116,E=12846125,C=147)
	CT(61129)=(S=12846126,E=12846150,C=29)
	CT(61130)=(S=12846151,E=12846159,C=105)
	CT(61131)=(S=12846160,E=12846160,C=107)
	CT(61132)=(S=12846161,E=12846334,C=105)
	CT(61133)=(S=12846335,E=12846335,C=133)
	CT(61134)=(S=12846336,E=12846694,C=105)
	CT(61135)=(S=12846695,E=12846695,C=111)
	CT(61136)=(S=12846712,E=12846712,C=106)
	CT(61137)=(S=12846713,E=12846776,C=105)
	CT(61138)=(S=12846777,E=12846777,C=106)
	CT(61139)=(S=12846778,E=12846787,C=111)
	CT(61140)=(S=12846788,E=12846789,C=112)
	CT(61141)=(S=12846790,E=12846790,C=105)
	CT(61142)=(S=12846791,E=12846791,C=113)
	CT(61143)=(S=12846792,E=12846792,C=106)
	CT(61144)=(S=12846793,E=12846793,C=105)
	CT(61145)=(S=12846794,E=12846795,C=109)
	CT(61146)=(S=12846796,E=12846799,C=111)
	CT(61147)=(S=12846800,E=12846806,C=105)
	CT(61148)=(S=12846807,E=12846812,C=109)
	CT(61149)=(S=12846813,E=12846815,C=105)
	CT(61150)=(S=12846828,E=12846828,C=106)
	CT(61151)=(S=12846829,E=12846846,C=105)
	CT(61152)=(S=12846847,E=12846847,C=116)
	CT(61153)=(S=12846848,E=12847666,C=105)
	CT(61154)=(S=12847667,E=12847667,C=116)
	CT(61155)=(S=12847668,E=12847671,C=105)
	CT(61156)=(S=12847672,E=12847675,C=111)
	CT(61157)=(S=12847676,E=12847676,C=112)
	CT(61158)=(S=12847677,E=12847712,C=105)
	CT(61159)=(S=12847713,E=12847713,C=106)
	CT(61160)=(S=12847714,E=12847715,C=105)
	CT(61161)=(S=12847716,E=12847716,C=120)
	CT(61162)=(S=12847717,E=12847734,C=105)
	CT(61163)=(S=12847735,E=12847735,C=130)
	CT(61164)=(S=12847736,E=12847736,C=106)
	CT(61165)=(S=12847737,E=12847737,C=105)
	CT(61166)=(S=12847738,E=12847739,C=118)
	CT(61167)=(S=12847740,E=12847752,C=105)
	CT(61168)=(S=12847753,E=12847753,C=110)
	CT(61169)=(S=12847754,E=12847755,C=130)
	CT(61170)=(S=12847756,E=12847762,C=105)
	CT(61171)=(S=12847763,E=12847763,C=112)
	CT(61172)=(S=12847764,E=12847764,C=131)
	CT(61173)=(S=12847765,E=12847817,C=105)
	CT(61174)=(S=12847818,E=12847818,C=111)
	CT(61175)=(S=12847819,E=12847830,C=105)
	CT(61176)=(S=12847831,E=12847831,C=110)
	CT(61177)=(S=12847832,E=12847839,C=123)
	CT(61178)=(S=12847840,E=12847843,C=105)
	CT(61179)=(S=12847844,E=12847844,C=130)
	CT(61180)=(S=12847845,E=12847846,C=105)
	CT(61181)=(S=12847847,E=12847847,C=111)
	CT(61182)=(S=12847848,E=12847866,C=105)
	CT(61183)=(S=12847867,E=12847871,C=111)
	CT(61184)=(S=12847872,E=12847933,C=105)
	CT(61185)=(S=12847934,E=12847934,C=140)
	CT(61186)=(S=12847936,E=12847946,C=105)
	CT(61187)=(S=12847947,E=12847947,C=135)
	CT(61188)=(S=12847948,E=12847951,C=105)
	CT(61189)=(S=12847960,E=12847961,C=109)
	CT(61190)=(S=12847962,E=12847962,C=110)
	CT(61191)=(S=12847963,E=12847974,C=105)
	CT(61192)=(S=12847976,E=12847976,C=111)
	CT(61193)=(S=12847977,E=12847995,C=105)
	CT(61194)=(S=12847996,E=12847996,C=138)
	CT(61195)=(S=12847997,E=12848006,C=105)
	CT(61196)=(S=12848007,E=12848007,C=131)
	CT(61197)=(S=12848008,E=12848021,C=105)
	CT(61198)=(S=12848022,E=12848023,C=111)
	CT(61199)=(S=12848032,E=12848046,C=105)
	CT(61200)=(S=12848047,E=12848047,C=133)
	CT(61201)=(S=12848056,E=12848059,C=111)
	CT(61202)=(S=12848060,E=12848060,C=105)
	CT(61203)=(S=12848061,E=12848061,C=116)
	CT(61204)=(S=12848062,E=12848063,C=109)
	CT(61205)=(S=12848064,E=12848077,C=105)
	CT(61206)=(S=12848078,E=12848079,C=110)
	CT(61207)=(S=12848080,E=12848102,C=105)
	CT(61208)=(S=12848103,E=12848103,C=111)
	CT(61209)=(S=12848104,E=12848105,C=105)
	CT(61210)=(S=12848106,E=12848106,C=116)
	CT(61211)=(S=12848107,E=12848126,C=105)
	CT(61212)=(S=12848127,E=12848127,C=112)
	CT(61213)=(S=12848128,E=12848137,C=80)
	CT(61214)=(S=12848138,E=12848138,C=117)
	CT(61215)=(S=12848139,E=12848139,C=106)
	CT(61216)=(S=12848140,E=12848143,C=114)
	CT(61217)=(S=12848144,E=12848159,C=105)
	CT(61218)=(S=12848160,E=12848191,C=4)
	CT(61219)=(S=12848192,E=12848255,C=178)
	CT(61220)=(S=12848256,E=12848287,C=113)
	CT(61221)=(S=12848288,E=12848319,C=80)
	CT(61222)=(S=12848320,E=12848383,C=134)
	CT(61223)=(S=12848384,E=12848484,C=105)
	CT(61224)=(S=12848485,E=12848485,C=131)
	CT(61225)=(S=12848486,E=12848487,C=111)
	CT(61226)=(S=12848488,E=12848488,C=114)
	CT(61227)=(S=12848489,E=12848489,C=112)
	CT(61228)=(S=12848490,E=12848491,C=105)
	CT(61229)=(S=12848492,E=12848492,C=134)
	CT(61230)=(S=12848493,E=12848493,C=116)
	CT(61231)=(S=12848496,E=12848496,C=111)
	CT(61232)=(S=12848497,E=12848504,C=105)
	CT(61233)=(S=12848505,E=12848505,C=109)
	CT(61234)=(S=12848506,E=12848506,C=130)
	CT(61235)=(S=12848507,E=12848507,C=110)
	CT(61236)=(S=12848508,E=12848508,C=105)
	CT(61237)=(S=12848509,E=12848509,C=113)
	CT(61238)=(S=12848510,E=12848516,C=105)
	CT(61239)=(S=12848517,E=12848517,C=112)
	CT(61240)=(S=12848518,E=12848519,C=105)
	CT(61241)=(S=12848520,E=12848521,C=109)
	CT(61242)=(S=12848522,E=12848544,C=105)
	CT(61243)=(S=12848545,E=12848545,C=111)
	CT(61244)=(S=12848546,E=12848551,C=105)
	CT(61245)=(S=12848552,E=12848552,C=138)
	CT(61246)=(S=12848553,E=12848556,C=105)
	CT(61247)=(S=12848557,E=12848557,C=109)
	CT(61248)=(S=12848558,E=12848559,C=110)
	CT(61249)=(S=12848560,E=12848569,C=105)
	CT(61250)=(S=12848570,E=12848570,C=112)
	CT(61251)=(S=12848571,E=12848574,C=105)
	CT(61252)=(S=12848575,E=12848575,C=109)
	CT(61253)=(S=12848576,E=12848583,C=110)
	CT(61254)=(S=12848584,E=12848584,C=105)
	CT(61255)=(S=12848588,E=12848588,C=116)
	CT(61256)=(S=12848589,E=12848589,C=105)
	CT(61257)=(S=12848590,E=12848590,C=106)
	CT(61258)=(S=12848591,E=12848591,C=139)
	CT(61259)=(S=12848592,E=12848592,C=147)
	CT(61260)=(S=12848593,E=12848593,C=109)
	CT(61261)=(S=12848594,E=12848606,C=105)
	CT(61262)=(S=12848607,E=12848607,C=145)
	CT(61263)=(S=12848608,E=12848625,C=105)
	CT(61264)=(S=12848626,E=12848626,C=111)
	CT(61265)=(S=12848627,E=12848627,C=116)
	CT(61266)=(S=12848628,E=12848628,C=106)
	CT(61267)=(S=12848629,E=12848629,C=111)
	CT(61268)=(S=12848630,E=12848635,C=105)
	CT(61269)=(S=12848636,E=12848636,C=110)
	CT(61270)=(S=12848637,E=12848637,C=106)
	CT(61271)=(S=12848638,E=12848638,C=105)
	CT(61272)=(S=12848639,E=12848639,C=109)
	CT(61273)=(S=12848640,E=12848911,C=105)
	CT(61274)=(S=12848912,E=12848927,C=4)
	CT(61275)=(S=12848928,E=12848959,C=23)
	CT(61276)=(S=12848960,E=12849151,C=105)
	CT(61277)=(S=12849152,E=12850175,C=147)
	CT(61278)=(S=12850176,E=12850207,C=117)
	CT(61279)=(S=12850208,E=12850239,C=106)
	CT(61280)=(S=12850240,E=12850303,C=135)
	CT(61281)=(S=12850304,E=12850431,C=113)
	CT(61282)=(S=12850432,E=12850692,C=105)
	CT(61283)=(S=12850693,E=12850693,C=106)
	CT(61284)=(S=12850694,E=12850694,C=111)
	CT(61285)=(S=12850695,E=12850695,C=106)
	CT(61286)=(S=12850700,E=12850703,C=221)
	CT(61287)=(S=12850704,E=12850735,C=105)
	CT(61288)=(S=12850736,E=12850751,C=131)
	CT(61289)=(S=12850752,E=12850817,C=105)
	CT(61290)=(S=12850818,E=12850818,C=106)
	CT(61291)=(S=12850819,E=12850819,C=109)
	CT(61292)=(S=12850820,E=12850831,C=105)
	CT(61293)=(S=12850832,E=12850847,C=113)
	CT(61294)=(S=12850848,E=12851967,C=105)
	CT(61295)=(S=12851968,E=12852031,C=29)
	CT(61296)=(S=12852032,E=12852095,C=113)
	CT(61297)=(S=12852096,E=12852223,C=111)
	CT(61298)=(S=12852224,E=12852230,C=105)
	CT(61299)=(S=12852231,E=12852231,C=138)
	CT(61300)=(S=12852232,E=12852233,C=105)
	CT(61301)=(S=12852234,E=12852234,C=111)
	CT(61302)=(S=12852235,E=12852235,C=109)
	CT(61303)=(S=12852236,E=12852271,C=105)
	CT(61304)=(S=12852272,E=12852287,C=80)
	CT(61305)=(S=12852288,E=12852447,C=105)
	CT(61306)=(S=12852448,E=12852463,C=131)
	CT(61307)=(S=12852464,E=12852479,C=139)
	CT(61308)=(S=12852480,E=12852511,C=105)
	CT(61309)=(S=12852512,E=12852519,C=107)
	CT(61310)=(S=12852520,E=12852523,C=135)
	CT(61311)=(S=12852524,E=12852527,C=111)
	CT(61312)=(S=12852528,E=12852535,C=112)
	CT(61313)=(S=12852536,E=12852543,C=105)
	CT(61314)=(S=12852544,E=12852575,C=176)
	CT(61315)=(S=12852576,E=12852607,C=110)
	CT(61316)=(S=12852608,E=12852639,C=105)
	CT(61317)=(S=12852640,E=12852671,C=119)
	CT(61318)=(S=12852672,E=12852687,C=116)
	CT(61319)=(S=12852704,E=12852719,C=110)
	CT(61320)=(S=12852720,E=12853247,C=105)
	CT(61321)=(S=12853248,E=12853255,C=175)
	CT(61322)=(S=12853256,E=12853263,C=105)
	CT(61323)=(S=12853264,E=12853279,C=135)
	CT(61324)=(S=12853280,E=12853311,C=176)
	CT(61325)=(S=12853312,E=12853343,C=83)
	CT(61326)=(S=12853360,E=12853375,C=123)
	CT(61327)=(S=12853376,E=12853407,C=80)
	CT(61328)=(S=12853440,E=12853447,C=116)
	CT(61329)=(S=12853448,E=12853455,C=142)
	CT(61330)=(S=12853456,E=12853463,C=105)
	CT(61331)=(S=12853472,E=12853473,C=111)
	CT(61332)=(S=12853474,E=12853475,C=109)
	CT(61333)=(S=12853476,E=12853477,C=106)
	CT(61334)=(S=12853478,E=12853487,C=105)
	CT(61335)=(S=12853488,E=12853495,C=112)
	CT(61336)=(S=12853504,E=12855295,C=105)
	CT(61337)=(S=12855296,E=12855391,C=202)
	CT(61338)=(S=12855392,E=12855407,C=105)
	CT(61339)=(S=12855408,E=12855423,C=131)
	CT(61340)=(S=12855424,E=12855439,C=109)
	CT(61341)=(S=12855440,E=12855455,C=110)
	CT(61342)=(S=12855456,E=12855471,C=111)
	CT(61343)=(S=12855472,E=12855487,C=105)
	CT(61344)=(S=12855488,E=12855551,C=111)
	CT(61345)=(S=12855552,E=12855583,C=105)
	CT(61346)=(S=12855584,E=12855615,C=112)
	CT(61347)=(S=12855648,E=12855775,C=105)
	CT(61348)=(S=12855776,E=12855807,C=135)
	CT(61349)=(S=12855808,E=12855871,C=80)
	CT(61350)=(S=12855872,E=12856127,C=105)
	CT(61351)=(S=12856128,E=12856159,C=112)
	CT(61352)=(S=12856160,E=12856191,C=107)
	CT(61353)=(S=12856192,E=12856255,C=130)
	CT(61354)=(S=12856256,E=12856256,C=109)
	CT(61355)=(S=12856257,E=12856257,C=114)
	CT(61356)=(S=12856258,E=12856258,C=110)
	CT(61357)=(S=12856259,E=12856259,C=105)
	CT(61358)=(S=12856260,E=12856260,C=110)
	CT(61359)=(S=12856261,E=12856261,C=116)
	CT(61360)=(S=12856262,E=12856262,C=106)
	CT(61361)=(S=12856263,E=12856263,C=107)
	CT(61362)=(S=12856264,E=12856264,C=105)
	CT(61363)=(S=12856265,E=12856265,C=106)
	CT(61364)=(S=12856266,E=12856266,C=109)
	CT(61365)=(S=12856267,E=12856268,C=105)
	CT(61366)=(S=12856269,E=12856269,C=109)
	CT(61367)=(S=12856270,E=12856271,C=110)
	CT(61368)=(S=12856272,E=12856273,C=105)
	CT(61369)=(S=12856274,E=12856274,C=116)
	CT(61370)=(S=12856275,E=12856276,C=109)
	CT(61371)=(S=12856277,E=12856277,C=105)
	CT(61372)=(S=12856278,E=12856278,C=115)
	CT(61373)=(S=12856279,E=12856279,C=111)
	CT(61374)=(S=12856280,E=12856280,C=126)
	CT(61375)=(S=12856281,E=12856282,C=109)
	CT(61376)=(S=12856283,E=12856283,C=106)
	CT(61377)=(S=12856284,E=12856284,C=109)
	CT(61378)=(S=12856285,E=12856285,C=112)
	CT(61379)=(S=12856286,E=12856286,C=109)
	CT(61380)=(S=12856287,E=12856287,C=116)
	CT(61381)=(S=12856288,E=12856288,C=111)
	CT(61382)=(S=12856289,E=12856289,C=110)
	CT(61383)=(S=12856290,E=12856290,C=130)
	CT(61384)=(S=12856291,E=12856291,C=105)
	CT(61385)=(S=12856292,E=12856292,C=109)
	CT(61386)=(S=12856293,E=12856293,C=97)
	CT(61387)=(S=12856294,E=12856294,C=112)
	CT(61388)=(S=12856295,E=12856295,C=105)
	CT(61389)=(S=12856296,E=12856297,C=111)
	CT(61390)=(S=12856298,E=12856298,C=131)
	CT(61391)=(S=12856299,E=12856299,C=111)
	CT(61392)=(S=12856300,E=12856300,C=134)
	CT(61393)=(S=12856301,E=12856301,C=106)
	CT(61394)=(S=12856302,E=12856302,C=111)
	CT(61395)=(S=12856303,E=12856303,C=109)
	CT(61396)=(S=12856304,E=12856304,C=129)
	CT(61397)=(S=12856305,E=12856305,C=131)
	CT(61398)=(S=12856306,E=12856306,C=110)
	CT(61399)=(S=12856307,E=12856307,C=105)
	CT(61400)=(S=12856308,E=12856308,C=111)
	CT(61401)=(S=12856309,E=12856310,C=109)
	CT(61402)=(S=12856311,E=12856311,C=139)
	CT(61403)=(S=12856312,E=12856312,C=109)
	CT(61404)=(S=12856313,E=12856313,C=133)
	CT(61405)=(S=12856314,E=12856314,C=105)
	CT(61406)=(S=12856315,E=12856316,C=111)
	CT(61407)=(S=12856317,E=12856317,C=105)
	CT(61408)=(S=12856318,E=12856319,C=112)
	CT(61409)=(S=12856320,E=12856415,C=105)
	CT(61410)=(S=12856416,E=12856447,C=110)
	CT(61411)=(S=12856448,E=12856479,C=117)
	CT(61412)=(S=12856480,E=12856495,C=112)
	CT(61413)=(S=12856496,E=12856511,C=107)
	CT(61414)=(S=12856512,E=12856559,C=105)
	CT(61415)=(S=12856560,E=12856575,C=128)
	CT(61416)=(S=12856592,E=12856607,C=105)
	CT(61417)=(S=12856608,E=12856623,C=112)
	CT(61418)=(S=12856624,E=12856639,C=111)
	CT(61419)=(S=12856640,E=12856703,C=105)
	CT(61420)=(S=12856704,E=12856735,C=112)
	CT(61421)=(S=12856736,E=12856739,C=116)
	CT(61422)=(S=12856740,E=12856743,C=126)
	CT(61423)=(S=12856744,E=12856759,C=105)
	CT(61424)=(S=12856760,E=12856763,C=112)
	CT(61425)=(S=12856764,E=12856767,C=121)
	CT(61426)=(S=12856768,E=12856831,C=105)
	CT(61427)=(S=12856832,E=12856847,C=131)
	CT(61428)=(S=12856848,E=12856848,C=109)
	CT(61429)=(S=12856849,E=12856849,C=106)
	CT(61430)=(S=12856850,E=12856851,C=114)
	CT(61431)=(S=12856852,E=12856852,C=111)
	CT(61432)=(S=12856853,E=12856853,C=116)
	CT(61433)=(S=12856854,E=12856854,C=106)
	CT(61434)=(S=12856855,E=12856855,C=105)
	CT(61435)=(S=12856856,E=12856856,C=106)
	CT(61436)=(S=12856857,E=12856857,C=105)
	CT(61437)=(S=12856858,E=12856858,C=106)
	CT(61438)=(S=12856859,E=12856859,C=111)
	CT(61439)=(S=12856860,E=12856860,C=117)
	CT(61440)=(S=12856861,E=12856861,C=106)
	CT(61441)=(S=12856862,E=12856863,C=105)
	CT(61442)=(S=12856864,E=12856895,C=113)
	CT(61443)=(S=12856896,E=12856903,C=105)
	CT(61444)=(S=12856904,E=12856911,C=116)
	CT(61445)=(S=12856912,E=12856927,C=110)
	CT(61446)=(S=12856928,E=12856943,C=112)
	CT(61447)=(S=12856944,E=12856947,C=137)
	CT(61448)=(S=12856948,E=12856951,C=105)
	CT(61449)=(S=12856952,E=12856955,C=112)
	CT(61450)=(S=12856956,E=12856959,C=113)
	CT(61451)=(S=12856960,E=12856967,C=112)
	CT(61452)=(S=12856968,E=12856975,C=105)
	CT(61453)=(S=12856976,E=12856979,C=111)
	CT(61454)=(S=12856980,E=12856983,C=147)
	CT(61455)=(S=12856984,E=12856987,C=124)
	CT(61456)=(S=12856988,E=12856991,C=122)
	CT(61457)=(S=12856992,E=12857019,C=105)
	CT(61458)=(S=12857020,E=12857023,C=106)
	CT(61459)=(S=12857024,E=12857055,C=114)
	CT(61460)=(S=12857056,E=12857063,C=105)
	CT(61461)=(S=12857064,E=12857071,C=132)
	CT(61462)=(S=12857072,E=12857079,C=111)
	CT(61463)=(S=12857080,E=12857087,C=135)
	CT(61464)=(S=12857088,E=12857183,C=105)
	CT(61465)=(S=12857184,E=12857215,C=111)
	CT(61466)=(S=12857216,E=12857279,C=118)
	CT(61467)=(S=12857280,E=12857343,C=105)
	CT(61468)=(S=12857344,E=12857599,C=147)
	CT(61469)=(S=12857600,E=12857600,C=118)
	CT(61470)=(S=12857601,E=12857601,C=132)
	CT(61471)=(S=12857602,E=12857602,C=117)
	CT(61472)=(S=12857603,E=12857603,C=145)
	CT(61473)=(S=12857604,E=12857604,C=138)
	CT(61474)=(S=12857605,E=12857605,C=112)
	CT(61475)=(S=12857606,E=12857606,C=116)
	CT(61476)=(S=12857607,E=12857607,C=128)
	CT(61477)=(S=12857608,E=12857608,C=124)
	CT(61478)=(S=12857609,E=12857609,C=113)
	CT(61479)=(S=12857610,E=12857610,C=150)
	CT(61480)=(S=12857611,E=12857611,C=147)
	CT(61481)=(S=12857612,E=12857612,C=112)
	CT(61482)=(S=12857613,E=12857613,C=115)
	CT(61483)=(S=12857614,E=12857614,C=110)
	CT(61484)=(S=12857615,E=12857615,C=130)
	CT(61485)=(S=12857616,E=12857616,C=108)
	CT(61486)=(S=12857856,E=12859391,C=105)
	CT(61487)=(S=12859392,E=12860415,C=147)
	CT(61488)=(S=12860928,E=12861439,C=105)
	CT(61489)=(S=12894208,E=12894223,C=113)
	CT(61490)=(S=12894224,E=12894239,C=124)
	CT(61491)=(S=12894240,E=12894255,C=115)
	CT(61492)=(S=12894272,E=12894351,C=113)
	CT(61493)=(S=12894352,E=12894367,C=111)
	CT(61494)=(S=12894400,E=12895231,C=105)
	CT(61495)=(S=12895232,E=12896255,C=147)
	CT(61496)=(S=12896256,E=12896271,C=105)
	CT(61497)=(S=12896272,E=12896303,C=109)
	CT(61498)=(S=12896304,E=12896319,C=123)
	CT(61499)=(S=12896320,E=12896335,C=111)
	CT(61500)=(S=12896336,E=12896351,C=123)
	CT(61501)=(S=12896352,E=12896367,C=227)
	CT(61502)=(S=12896368,E=12896383,C=111)
	CT(61503)=(S=12896384,E=12896447,C=134)
	CT(61504)=(S=12896448,E=12896463,C=105)
	CT(61505)=(S=12896464,E=12896479,C=113)
	CT(61506)=(S=12896496,E=12896511,C=134)
	CT(61507)=(S=12896513,E=12896513,C=113)
	CT(61508)=(S=12896514,E=12896514,C=110)
	CT(61509)=(S=12896515,E=12896515,C=106)
	CT(61510)=(S=12896518,E=12896519,C=105)
	CT(61511)=(S=12896528,E=12896535,C=113)
	CT(61512)=(S=12896536,E=12896543,C=106)
	CT(61513)=(S=12896544,E=12896575,C=110)
	CT(61514)=(S=12896576,E=12896607,C=118)
	CT(61515)=(S=12896640,E=12896671,C=109)
	CT(61516)=(S=12896704,E=12896719,C=150)
	CT(61517)=(S=12896720,E=12896739,C=109)
	CT(61518)=(S=12896740,E=12896743,C=113)
	CT(61519)=(S=12896752,E=12896895,C=106)
	CT(61520)=(S=12896896,E=12896927,C=119)
	CT(61521)=(S=12896960,E=12896991,C=109)
	CT(61522)=(S=12897000,E=12897007,C=120)
	CT(61523)=(S=12897008,E=12897015,C=133)
	CT(61524)=(S=12897016,E=12897019,C=105)
	CT(61525)=(S=12897020,E=12897023,C=116)
	CT(61526)=(S=12897024,E=12897279,C=152)
	CT(61527)=(S=12897280,E=12897791,C=106)
	CT(61528)=(S=12897792,E=12898047,C=134)
	CT(61529)=(S=12898048,E=12898063,C=111)
	CT(61530)=(S=12898064,E=12898079,C=109)
	CT(61531)=(S=12898080,E=12898095,C=105)
	CT(61532)=(S=12898096,E=12898111,C=113)
	CT(61533)=(S=12898112,E=12898175,C=105)
	CT(61534)=(S=12898240,E=12898303,C=148)
	CT(61535)=(S=12898304,E=12900351,C=105)
	CT(61536)=(S=12900352,E=12900352,C=109)
	CT(61537)=(S=12900353,E=12900353,C=29)
	CT(61538)=(S=12900354,E=12900355,C=113)
	CT(61539)=(S=12900356,E=12900359,C=111)
	CT(61540)=(S=12900360,E=12900367,C=121)
	CT(61541)=(S=12900368,E=12900375,C=105)
	CT(61542)=(S=12900376,E=12900383,C=106)
	CT(61543)=(S=12900384,E=12900415,C=117)
	CT(61544)=(S=12900416,E=12900447,C=113)
	CT(61545)=(S=12900480,E=12900483,C=109)
	CT(61546)=(S=12900484,E=12900491,C=105)
	CT(61547)=(S=12900492,E=12900495,C=106)
	CT(61548)=(S=12900496,E=12900503,C=111)
	CT(61549)=(S=12900504,E=12900507,C=105)
	CT(61550)=(S=12900508,E=12900511,C=152)
	CT(61551)=(S=12900512,E=12900512,C=142)
	CT(61552)=(S=12900513,E=12900513,C=116)
	CT(61553)=(S=12900514,E=12900514,C=112)
	CT(61554)=(S=12900515,E=12900515,C=126)
	CT(61555)=(S=12900516,E=12900519,C=117)
	CT(61556)=(S=12900520,E=12900521,C=113)
	CT(61557)=(S=12900522,E=12900522,C=105)
	CT(61558)=(S=12900523,E=12900523,C=113)
	CT(61559)=(S=12900524,E=12900527,C=105)
	CT(61560)=(S=12900528,E=12900531,C=130)
	CT(61561)=(S=12900532,E=12900535,C=110)
	CT(61562)=(S=12900536,E=12900539,C=111)
	CT(61563)=(S=12900540,E=12900543,C=148)
	CT(61564)=(S=12900544,E=12900547,C=105)
	CT(61565)=(S=12900548,E=12900549,C=130)
	CT(61566)=(S=12900550,E=12900551,C=116)
	CT(61567)=(S=12900552,E=12900555,C=111)
	CT(61568)=(S=12900556,E=12900557,C=105)
	CT(61569)=(S=12900558,E=12900558,C=106)
	CT(61570)=(S=12900559,E=12900559,C=117)
	CT(61571)=(S=12900560,E=12900563,C=111)
	CT(61572)=(S=12900564,E=12900564,C=120)
	CT(61573)=(S=12900565,E=12900565,C=113)
	CT(61574)=(S=12900566,E=12900566,C=110)
	CT(61575)=(S=12900567,E=12900567,C=105)
	CT(61576)=(S=12900568,E=12900569,C=137)
	CT(61577)=(S=12900570,E=12900571,C=112)
	CT(61578)=(S=12900572,E=12900573,C=97)
	CT(61579)=(S=12900574,E=12900575,C=109)
	CT(61580)=(S=12900580,E=12900581,C=110)
	CT(61581)=(S=12900582,E=12900583,C=105)
	CT(61582)=(S=12900584,E=12900585,C=109)
	CT(61583)=(S=12900586,E=12900587,C=112)
	CT(61584)=(S=12900588,E=12900589,C=105)
	CT(61585)=(S=12900590,E=12900591,C=113)
	CT(61586)=(S=12900592,E=12900593,C=106)
	CT(61587)=(S=12900594,E=12900597,C=109)
	CT(61588)=(S=12900598,E=12900598,C=106)
	CT(61589)=(S=12900599,E=12900599,C=112)
	CT(61590)=(S=12900600,E=12900601,C=116)
	CT(61591)=(S=12900602,E=12900602,C=105)
	CT(61592)=(S=12900603,E=12900603,C=111)
	CT(61593)=(S=12900604,E=12900604,C=106)
	CT(61594)=(S=12900605,E=12900607,C=111)
	CT(61595)=(S=12900608,E=12900863,C=134)
	CT(61596)=(S=12900864,E=12901375,C=106)
	CT(61597)=(S=12901376,E=12901407,C=111)
	CT(61598)=(S=12901408,E=12901439,C=105)
	CT(61599)=(S=12901440,E=12901471,C=111)
	CT(61600)=(S=12901472,E=12901503,C=107)
	CT(61601)=(S=12901504,E=12901583,C=111)
	CT(61602)=(S=12901584,E=12901599,C=121)
	CT(61603)=(S=12901600,E=12901631,C=111)
	CT(61604)=(S=12901632,E=12901887,C=106)
	CT(61605)=(S=12901888,E=12902143,C=111)
	CT(61606)=(S=12902144,E=12902144,C=113)
	CT(61607)=(S=12902145,E=12902145,C=116)
	CT(61608)=(S=12902146,E=12902146,C=114)
	CT(61609)=(S=12902147,E=12902147,C=145)
	CT(61610)=(S=12902148,E=12902148,C=118)
	CT(61611)=(S=12902149,E=12902150,C=112)
	CT(61612)=(S=12902151,E=12902151,C=106)
	CT(61613)=(S=12902152,E=12902153,C=105)
	CT(61614)=(S=12902154,E=12902154,C=97)
	CT(61615)=(S=12902155,E=12902155,C=109)
	CT(61616)=(S=12902156,E=12902156,C=128)
	CT(61617)=(S=12902157,E=12902157,C=116)
	CT(61618)=(S=12902158,E=12902159,C=105)
	CT(61619)=(S=12902160,E=12902161,C=106)
	CT(61620)=(S=12902162,E=12902162,C=117)
	CT(61621)=(S=12902163,E=12902163,C=116)
	CT(61622)=(S=12902164,E=12902164,C=119)
	CT(61623)=(S=12902165,E=12902165,C=109)
	CT(61624)=(S=12902166,E=12902167,C=105)
	CT(61625)=(S=12902168,E=12902168,C=133)
	CT(61626)=(S=12902169,E=12902169,C=130)
	CT(61627)=(S=12902170,E=12902170,C=107)
	CT(61628)=(S=12902171,E=12902171,C=121)
	CT(61629)=(S=12902172,E=12902172,C=137)
	CT(61630)=(S=12902174,E=12902175,C=105)
	CT(61631)=(S=12902176,E=12902176,C=129)
	CT(61632)=(S=12902177,E=12902177,C=131)
	CT(61633)=(S=12902178,E=12902178,C=132)
	CT(61634)=(S=12902179,E=12902179,C=117)
	CT(61635)=(S=12902180,E=12902180,C=145)
	CT(61636)=(S=12902181,E=12902181,C=138)
	CT(61637)=(S=12902182,E=12902182,C=150)
	CT(61638)=(S=12902183,E=12902183,C=122)
	CT(61639)=(S=12902184,E=12902184,C=124)
	CT(61640)=(S=12902185,E=12902185,C=115)
	CT(61641)=(S=12902186,E=12902186,C=125)
	CT(61642)=(S=12902187,E=12902187,C=147)
	CT(61643)=(S=12902188,E=12902188,C=108)
	CT(61644)=(S=12906496,E=12908031,C=147)
	CT(61645)=(S=12908032,E=12908287,C=105)
	CT(61646)=(S=12908288,E=12908543,C=147)
	CT(61647)=(S=12908544,E=12910591,C=105)
	CT(61648)=(S=12910592,E=12918783,C=152)
	CT(61649)=(S=12918784,E=12926975,C=106)
	CT(61650)=(S=12926976,E=12939263,C=105)
	CT(61651)=(S=12939264,E=12941311,C=135)
	CT(61652)=(S=12941312,E=12943359,C=106)
	CT(61653)=(S=12943360,E=12944383,C=134)
	CT(61654)=(S=12944384,E=12945407,C=106)
	CT(61655)=(S=12945408,E=12946431,C=109)
	CT(61656)=(S=12946432,E=12947455,C=135)
	CT(61657)=(S=12947456,E=12948479,C=134)
	CT(61658)=(S=12948480,E=12948543,C=116)
	CT(61659)=(S=12948544,E=12948551,C=105)
	CT(61660)=(S=12948552,E=12948559,C=132)
	CT(61661)=(S=12948560,E=12948567,C=111)
	CT(61662)=(S=12948568,E=12948575,C=116)
	CT(61663)=(S=12948576,E=12948607,C=143)
	CT(61664)=(S=12948608,E=12948671,C=115)
	CT(61665)=(S=12948672,E=12948703,C=118)
	CT(61666)=(S=12948704,E=12948735,C=110)
	CT(61667)=(S=12948736,E=12948799,C=115)
	CT(61668)=(S=12948800,E=12948863,C=111)
	CT(61669)=(S=12948864,E=12948867,C=125)
	CT(61670)=(S=12948868,E=12948871,C=111)
	CT(61671)=(S=12948872,E=12948875,C=129)
	CT(61672)=(S=12948876,E=12948879,C=145)
	CT(61673)=(S=12948880,E=12948883,C=105)
	CT(61674)=(S=12948884,E=12948887,C=116)
	CT(61675)=(S=12948888,E=12948891,C=121)
	CT(61676)=(S=12948892,E=12948895,C=131)
	CT(61677)=(S=12948896,E=12948899,C=105)
	CT(61678)=(S=12948900,E=12948903,C=111)
	CT(61679)=(S=12948904,E=12948907,C=146)
	CT(61680)=(S=12948912,E=12948915,C=112)
	CT(61681)=(S=12948916,E=12948919,C=105)
	CT(61682)=(S=12948920,E=12948923,C=137)
	CT(61683)=(S=12948924,E=12948927,C=111)
	CT(61684)=(S=12948928,E=12948991,C=136)
	CT(61685)=(S=12948992,E=12949503,C=106)
	CT(61686)=(S=12949504,E=12949759,C=112)
	CT(61687)=(S=12949760,E=12950015,C=134)
	CT(61688)=(S=12950016,E=12950271,C=228)
	CT(61689)=(S=12950272,E=12950335,C=105)
	CT(61690)=(S=12950336,E=12950367,C=113)
	CT(61691)=(S=12950368,E=12950399,C=105)
	CT(61692)=(S=12950400,E=12950463,C=123)
	CT(61693)=(S=12950464,E=12950479,C=133)
	CT(61694)=(S=12950496,E=12950511,C=113)
	CT(61695)=(S=12950512,E=12950519,C=110)
	CT(61696)=(S=12950520,E=12950591,C=105)
	CT(61697)=(S=12950592,E=12950655,C=228)
	CT(61698)=(S=12950656,E=12950719,C=109)
	CT(61699)=(S=12950720,E=12950783,C=111)
	CT(61700)=(S=12950784,E=12950847,C=130)
	CT(61701)=(S=12950848,E=12950879,C=105)
	CT(61702)=(S=12950880,E=12950911,C=147)
	CT(61703)=(S=12950912,E=12950975,C=128)
	CT(61704)=(S=12950976,E=12950979,C=145)
	CT(61705)=(S=12950980,E=12950983,C=116)
	CT(61706)=(S=12950984,E=12950987,C=133)
	CT(61707)=(S=12950988,E=12950991,C=107)
	CT(61708)=(S=12950992,E=12950995,C=137)
	CT(61709)=(S=12950996,E=12950999,C=128)
	CT(61710)=(S=12951000,E=12951003,C=111)
	CT(61711)=(S=12951004,E=12951007,C=105)
	CT(61712)=(S=12951008,E=12951011,C=97)
	CT(61713)=(S=12951012,E=12951015,C=109)
	CT(61714)=(S=12951016,E=12951019,C=97)
	CT(61715)=(S=12951024,E=12951027,C=105)
	CT(61716)=(S=12951028,E=12951031,C=140)
	CT(61717)=(S=12951032,E=12951035,C=137)
	CT(61718)=(S=12951036,E=12951039,C=129)
	CT(61719)=(S=12951040,E=12951103,C=131)
	CT(61720)=(S=12951104,E=12951167,C=115)
	CT(61721)=(S=12951168,E=12951231,C=114)
	CT(61722)=(S=12951232,E=12951295,C=147)
	CT(61723)=(S=12951296,E=12951327,C=120)
	CT(61724)=(S=12951328,E=12951359,C=105)
	CT(61725)=(S=12951360,E=12951375,C=111)
	CT(61726)=(S=12951392,E=12951407,C=109)
	CT(61727)=(S=12951408,E=12951423,C=112)
	CT(61728)=(S=12951424,E=12951439,C=110)
	CT(61729)=(S=12951440,E=12951455,C=115)
	CT(61730)=(S=12951456,E=12951487,C=220)
	CT(61731)=(S=12951488,E=12951519,C=118)
	CT(61732)=(S=12951552,E=12953599,C=106)
	CT(61733)=(S=12953600,E=12955647,C=105)
	CT(61734)=(S=12955648,E=12957695,C=109)
	CT(61735)=(S=12957696,E=12958207,C=105)
	CT(61736)=(S=12958208,E=12958719,C=112)
	CT(61737)=(S=12958720,E=12958975,C=117)
	CT(61738)=(S=12958976,E=12959103,C=137)
	CT(61739)=(S=12959104,E=12959167,C=133)
	CT(61740)=(S=12959168,E=12959231,C=105)
	CT(61741)=(S=12959232,E=12959743,C=110)
	CT(61742)=(S=12959744,E=12961791,C=106)
	CT(61743)=(S=12961792,E=12963839,C=135)
	CT(61744)=(S=12963840,E=12964351,C=119)
	CT(61745)=(S=12964352,E=12964607,C=111)
	CT(61746)=(S=12964608,E=12964639,C=109)
	CT(61747)=(S=12964640,E=12964671,C=111)
	CT(61748)=(S=12964672,E=12964703,C=105)
	CT(61749)=(S=12964704,E=12964735,C=121)
	CT(61750)=(S=12964736,E=12964799,C=105)
	CT(61751)=(S=12964800,E=12964863,C=113)
	CT(61752)=(S=12964864,E=12965375,C=114)
	CT(61753)=(S=12965376,E=12965439,C=141)
	CT(61754)=(S=12965440,E=12965455,C=146)
	CT(61755)=(S=12965456,E=12965471,C=221)
	CT(61756)=(S=12965472,E=12965487,C=111)
	CT(61757)=(S=12965488,E=12965495,C=105)
	CT(61758)=(S=12965496,E=12965503,C=107)
	CT(61759)=(S=12965504,E=12965631,C=129)
	CT(61760)=(S=12965632,E=12965759,C=97)
	CT(61761)=(S=12965760,E=12965791,C=65)
	CT(61762)=(S=12965792,E=12965823,C=105)
	CT(61763)=(S=12965824,E=12965839,C=115)
	CT(61764)=(S=12965840,E=12965847,C=105)
	CT(61765)=(S=12965848,E=12965851,C=108)
	CT(61766)=(S=12965852,E=12965855,C=137)
	CT(61767)=(S=12965856,E=12965871,C=105)
	CT(61768)=(S=12965872,E=12965887,C=112)
	CT(61769)=(S=12965888,E=12966399,C=116)
	CT(61770)=(S=12966400,E=12966911,C=131)
	CT(61771)=(S=12966912,E=12966943,C=114)
	CT(61772)=(S=12966976,E=12967007,C=140)
	CT(61773)=(S=12967008,E=12967039,C=109)
	CT(61774)=(S=12967040,E=12967071,C=133)
	CT(61775)=(S=12967072,E=12967103,C=110)
	CT(61776)=(S=12967104,E=12967167,C=114)
	CT(61777)=(S=12967168,E=12967231,C=105)
	CT(61778)=(S=12967232,E=12967263,C=110)
	CT(61779)=(S=12967264,E=12967295,C=105)
	CT(61780)=(S=12967296,E=12967327,C=130)
	CT(61781)=(S=12967328,E=12967359,C=105)
	CT(61782)=(S=12967360,E=12967391,C=112)
	CT(61783)=(S=12967392,E=12967423,C=107)
	CT(61784)=(S=12967424,E=12967935,C=106)
	CT(61785)=(S=12967936,E=12968959,C=113)
	CT(61786)=(S=12968960,E=12969471,C=105)
	CT(61787)=(S=12969472,E=12969727,C=134)
	CT(61788)=(S=12969728,E=12969759,C=149)
	CT(61789)=(S=12969760,E=12969791,C=133)
	CT(61790)=(S=12969792,E=12969855,C=122)
	CT(61791)=(S=12969856,E=12969863,C=132)
	CT(61792)=(S=12969864,E=12969871,C=122)
	CT(61793)=(S=12969872,E=12969879,C=105)
	CT(61794)=(S=12969880,E=12969887,C=108)
	CT(61795)=(S=12969888,E=12969903,C=105)
	CT(61796)=(S=12969904,E=12969911,C=109)
	CT(61797)=(S=12969912,E=12969919,C=113)
	CT(61798)=(S=12969920,E=12969923,C=126)
	CT(61799)=(S=12969924,E=12969927,C=105)
	CT(61800)=(S=12969928,E=12969931,C=140)
	CT(61801)=(S=12969932,E=12969935,C=132)
	CT(61802)=(S=12969936,E=12969939,C=147)
	CT(61803)=(S=12969940,E=12969943,C=107)
	CT(61804)=(S=12969944,E=12969947,C=131)
	CT(61805)=(S=12969948,E=12969951,C=108)
	CT(61806)=(S=12969952,E=12969955,C=127)
	CT(61807)=(S=12969956,E=12969959,C=65)
	CT(61808)=(S=12969960,E=12969963,C=105)
	CT(61809)=(S=12969964,E=12969967,C=144)
	CT(61810)=(S=12969972,E=12969975,C=114)
	CT(61811)=(S=12969976,E=12969979,C=145)
	CT(61812)=(S=12969980,E=12969983,C=137)
	CT(61813)=(S=12969984,E=12970239,C=109)
	CT(61814)=(S=12970240,E=12970495,C=117)
	CT(61815)=(S=12970496,E=12970511,C=147)
	CT(61816)=(S=12970512,E=12970527,C=105)
	CT(61817)=(S=12970528,E=12970559,C=111)
	CT(61818)=(S=12970560,E=12970623,C=117)
	CT(61819)=(S=12970624,E=12970703,C=105)
	CT(61820)=(S=12970704,E=12970711,C=126)
	CT(61821)=(S=12970712,E=12970719,C=124)
	CT(61822)=(S=12970720,E=12970723,C=112)
	CT(61823)=(S=12970724,E=12970727,C=111)
	CT(61824)=(S=12970728,E=12970731,C=105)
	CT(61825)=(S=12970732,E=12970735,C=109)
	CT(61826)=(S=12970736,E=12970739,C=147)
	CT(61827)=(S=12970740,E=12970743,C=128)
	CT(61828)=(S=12970744,E=12970747,C=65)
	CT(61829)=(S=12970752,E=12971007,C=131)
	CT(61830)=(S=12971008,E=12971263,C=105)
	CT(61831)=(S=12971264,E=12971519,C=109)
	CT(61832)=(S=12971776,E=12971839,C=130)
	CT(61833)=(S=12971840,E=12971903,C=139)
	CT(61834)=(S=12971904,E=12971967,C=105)
	CT(61835)=(S=12971968,E=12972031,C=109)
	CT(61836)=(S=12972288,E=12972415,C=150)
	CT(61837)=(S=12972416,E=12972543,C=116)
	CT(61838)=(S=12972544,E=12972575,C=122)
	CT(61839)=(S=12972576,E=12972639,C=105)
	CT(61840)=(S=12972640,E=12972671,C=111)
	CT(61841)=(S=12972672,E=12972687,C=132)
	CT(61842)=(S=12972688,E=12972703,C=105)
	CT(61843)=(S=12972704,E=12972719,C=131)
	CT(61844)=(S=12972720,E=12972727,C=142)
	CT(61845)=(S=12972728,E=12972735,C=113)
	CT(61846)=(S=12972736,E=12972743,C=112)
	CT(61847)=(S=12972744,E=12972751,C=105)
	CT(61848)=(S=12972752,E=12972767,C=109)
	CT(61849)=(S=12972768,E=12972783,C=118)
	CT(61850)=(S=12972784,E=12972799,C=111)
	CT(61851)=(S=12972800,E=12972927,C=128)
	CT(61852)=(S=12972928,E=12973055,C=117)
	CT(61853)=(S=12973312,E=12973567,C=105)
	CT(61854)=(S=12973568,E=12973823,C=106)
	CT(61855)=(S=12973824,E=12974079,C=134)
	CT(61856)=(S=12974080,E=12974335,C=109)
	CT(61857)=(S=12974336,E=12974591,C=131)
	CT(61858)=(S=12974592,E=12974847,C=112)
	CT(61859)=(S=12974848,E=12974975,C=119)
	CT(61860)=(S=12974976,E=12975103,C=110)
	CT(61861)=(S=12975104,E=12975359,C=119)
	CT(61862)=(S=12975360,E=12975423,C=111)
	CT(61863)=(S=12975424,E=12975487,C=110)
	CT(61864)=(S=12975488,E=12975615,C=134)
	CT(61865)=(S=12975616,E=12975743,C=109)
	CT(61866)=(S=12975744,E=12975807,C=133)
	CT(61867)=(S=12975808,E=12975871,C=119)
	CT(61868)=(S=12975872,E=12975935,C=111)
	CT(61869)=(S=12975936,E=12975999,C=105)
	CT(61870)=(S=12976000,E=12976015,C=127)
	CT(61871)=(S=12976016,E=12976031,C=105)
	CT(61872)=(S=12976032,E=12976047,C=111)
	CT(61873)=(S=12976048,E=12976063,C=129)
	CT(61874)=(S=12976064,E=12976079,C=132)
	CT(61875)=(S=12976080,E=12976095,C=111)
	CT(61876)=(S=12976096,E=12976111,C=229)
	CT(61877)=(S=12976112,E=12976115,C=105)
	CT(61878)=(S=12976116,E=12976119,C=111)
	CT(61879)=(S=12976120,E=12976123,C=122)
	CT(61880)=(S=12976124,E=12976127,C=111)
	CT(61881)=(S=12976128,E=12976439,C=29)
	CT(61882)=(S=12976440,E=12976447,C=78)
	CT(61883)=(S=12976448,E=12976575,C=29)
	CT(61884)=(S=12976576,E=12976639,C=78)
	CT(61885)=(S=12976640,E=12976655,C=29)
	CT(61886)=(S=12976656,E=12976671,C=203)
	CT(61887)=(S=12976672,E=12976703,C=29)
	CT(61888)=(S=12976704,E=12976767,C=78)
	CT(61889)=(S=12976768,E=12978943,C=29)
	CT(61890)=(S=12978944,E=12978947,C=23)
	CT(61891)=(S=12978948,E=12979231,C=29)
	CT(61892)=(S=12979232,E=12979263,C=83)
	CT(61893)=(S=12979264,E=12979983,C=29)
	CT(61894)=(S=12980000,E=12980031,C=0)
	CT(61895)=(S=12980032,E=12980351,C=29)
	CT(61896)=(S=12980352,E=12980479,C=78)
	CT(61897)=(S=12980480,E=12980522,C=29)
	CT(61898)=(S=12980523,E=12980523,C=78)
	CT(61899)=(S=12980524,E=12980537,C=29)
	CT(61900)=(S=12980538,E=12980538,C=78)
	CT(61901)=(S=12980539,E=12980549,C=29)
	CT(61902)=(S=12980550,E=12980550,C=7)
	CT(61903)=(S=12980551,E=12980556,C=29)
	CT(61904)=(S=12980557,E=12980557,C=14)
	CT(61905)=(S=12980558,E=12980575,C=29)
	CT(61906)=(S=12980576,E=12980576,C=78)
	CT(61907)=(S=12980577,E=12980596,C=29)
	CT(61908)=(S=12980597,E=12980597,C=15)
	CT(61909)=(S=12980598,E=12980598,C=2)
	CT(61910)=(S=12980599,E=12980599,C=29)
	CT(61911)=(S=12980600,E=12980601,C=189)
	CT(61912)=(S=12980602,E=12980619,C=29)
	CT(61913)=(S=12980620,E=12980620,C=78)
	CT(61914)=(S=12980621,E=12980659,C=29)
	CT(61915)=(S=12980660,E=12980662,C=36)
	CT(61916)=(S=12980663,E=12980710,C=29)
	CT(61917)=(S=12980711,E=12980712,C=189)
	CT(61918)=(S=12980713,E=12981255,C=29)
	CT(61919)=(S=12981256,E=12981263,C=78)
	CT(61920)=(S=12981264,E=12981279,C=29)
	CT(61921)=(S=12981280,E=12981311,C=78)
	CT(61922)=(S=12981312,E=12981793,C=29)
	CT(61923)=(S=12981796,E=12981796,C=78)
	CT(61924)=(S=12981798,E=12981806,C=29)
	CT(61925)=(S=12981807,E=12981807,C=0)
	CT(61926)=(S=12981811,E=12981811,C=28)
	CT(61927)=(S=12981812,E=12981820,C=29)
	CT(61928)=(S=12981821,E=12981821,C=0)
	CT(61929)=(S=12981822,E=12981852,C=29)
	CT(61930)=(S=12981853,E=12981857,C=28)
	CT(61931)=(S=12981858,E=12981861,C=29)
	CT(61932)=(S=12981862,E=12981862,C=78)
	CT(61933)=(S=12981863,E=12981947,C=29)
	CT(61934)=(S=12981948,E=12981948,C=78)
	CT(61935)=(S=12981949,E=12981963,C=29)
	CT(61936)=(S=12981964,E=12981965,C=78)
	CT(61937)=(S=12981966,E=12981996,C=29)
	CT(61938)=(S=12981997,E=12981997,C=78)
	CT(61939)=(S=12981998,E=12982009,C=29)
	CT(61940)=(S=12982010,E=12982011,C=203)
	CT(61941)=(S=12982012,E=12983043,C=29)
	CT(61942)=(S=12983044,E=12983048,C=73)
	CT(61943)=(S=12983049,E=12983051,C=29)
	CT(61944)=(S=12983052,E=12983055,C=78)
	CT(61945)=(S=12983056,E=12983103,C=29)
	CT(61946)=(S=12983104,E=12983167,C=78)
	CT(61947)=(S=12983168,E=12984327,C=29)
	CT(61948)=(S=12984328,E=12984418,C=230)
	CT(61949)=(S=12984419,E=12984445,C=29)
	CT(61950)=(S=12984446,E=12984447,C=230)
	CT(61951)=(S=12984448,E=12984449,C=29)
	CT(61952)=(S=12984450,E=12984451,C=230)
	CT(61953)=(S=12984452,E=12984455,C=29)
	CT(61954)=(S=12984456,E=12984463,C=230)
	CT(61955)=(S=12984464,E=12984464,C=29)
	CT(61956)=(S=12984465,E=12984465,C=230)
	CT(61957)=(S=12984466,E=12984466,C=29)
	CT(61958)=(S=12984467,E=12984479,C=230)
	CT(61959)=(S=12984480,E=12984481,C=29)
	CT(61960)=(S=12984482,E=12984494,C=230)
	CT(61961)=(S=12984495,E=12984497,C=29)
	CT(61962)=(S=12984498,E=12984500,C=230)
	CT(61963)=(S=12984501,E=12984503,C=29)
	CT(61964)=(S=12984504,E=12984505,C=230)
	CT(61965)=(S=12984506,E=12984506,C=29)
	CT(61966)=(S=12984507,E=12984509,C=230)
	CT(61967)=(S=12984510,E=12984512,C=29)
	CT(61968)=(S=12984513,E=12984514,C=230)
	CT(61969)=(S=12984515,E=12984516,C=29)
	CT(61970)=(S=12984517,E=12984543,C=230)
	CT(61971)=(S=12984544,E=12984559,C=29)
	CT(61972)=(S=12984560,E=12984560,C=230)
	CT(61973)=(S=12984561,E=12984563,C=29)
	CT(61974)=(S=12984564,E=12984575,C=230)
	CT(61975)=(S=12984576,E=12985087,C=78)
	CT(61976)=(S=12985088,E=12985129,C=29)
	CT(61977)=(S=12985130,E=12985135,C=78)
	CT(61978)=(S=12985136,E=12985371,C=29)
	CT(61979)=(S=12985372,E=12985375,C=180)
	CT(61980)=(S=12985376,E=12985390,C=23)
	CT(61981)=(S=12985391,E=12985459,C=29)
	CT(61982)=(S=12985460,E=12985463,C=78)
	CT(61983)=(S=12985464,E=12985759,C=29)
	CT(61984)=(S=12985760,E=12985791,C=78)
	CT(61985)=(S=12985792,E=12985855,C=29)
	CT(61986)=(S=12985856,E=12985859,C=78)
	CT(61987)=(S=12985860,E=12985862,C=29)
	CT(61988)=(S=12985863,E=12985863,C=78)
	CT(61989)=(S=12985864,E=12986639,C=29)
	CT(61990)=(S=12986652,E=12986655,C=78)
	CT(61991)=(S=12986656,E=12986670,C=203)
	CT(61992)=(S=12986671,E=12988203,C=29)
	CT(61993)=(S=12988204,E=12988207,C=78)
	CT(61994)=(S=12988208,E=12988415,C=29)
	CT(61995)=(S=12988416,E=12988419,C=73)
	CT(61996)=(S=12988420,E=12988543,C=29)
	CT(61997)=(S=12988544,E=12988672,C=78)
	CT(61998)=(S=12988673,E=12988783,C=29)
	CT(61999)=(S=12988784,E=12988785,C=3)
	CT(62000)=(S=12988786,E=12988795,C=29)
	CT(62001)=(S=12988796,E=12988796,C=0)
	CT(62002)=(S=12988797,E=12988799,C=29)
	CT(62003)=(S=12988800,E=12988803,C=181)
	CT(62004)=(S=12988804,E=12988805,C=189)
	CT(62005)=(S=12988806,E=12988812,C=29)
	CT(62006)=(S=12988813,E=12988813,C=78)
	CT(62007)=(S=12988814,E=12988835,C=29)
	CT(62008)=(S=12988836,E=12988836,C=202)
	CT(62009)=(S=12988837,E=12988846,C=29)
	CT(62010)=(S=12988847,E=12988847,C=78)
	CT(62011)=(S=12988848,E=12988943,C=29)
	CT(62012)=(S=12988944,E=12988951,C=83)
	CT(62013)=(S=12988952,E=12989023,C=29)
	CT(62014)=(S=12989024,E=12989183,C=78)
	CT(62015)=(S=12989184,E=12989195,C=29)
	CT(62016)=(S=12989196,E=12989196,C=23)
	CT(62017)=(S=12989197,E=12989254,C=29)
	CT(62018)=(S=12989255,E=12989255,C=181)
	CT(62019)=(S=12989256,E=12989258,C=29)
	CT(62020)=(S=12989259,E=12989259,C=78)
	CT(62021)=(S=12989260,E=12989287,C=29)
	CT(62022)=(S=12989288,E=12989291,C=78)
	CT(62023)=(S=12989292,E=12989295,C=29)
	CT(62024)=(S=12989296,E=12989301,C=78)
	CT(62025)=(S=12989302,E=12989319,C=29)
	CT(62026)=(S=12989320,E=12989323,C=78)
	CT(62027)=(S=12989324,E=12989326,C=29)
	CT(62028)=(S=12989327,E=12989330,C=30)
	CT(62029)=(S=12989331,E=12989334,C=29)
	CT(62030)=(S=12989335,E=12989335,C=78)
	CT(62031)=(S=12989336,E=12989339,C=29)
	CT(62032)=(S=12989340,E=12989340,C=78)
	CT(62033)=(S=12989341,E=12989378,C=29)
	CT(62034)=(S=12989379,E=12989379,C=73)
	CT(62035)=(S=12989381,E=12989393,C=29)
	CT(62036)=(S=12989394,E=12989394,C=0)
	CT(62037)=(S=12989396,E=12989438,C=29)
	CT(62038)=(S=12989439,E=12989439,C=78)
	CT(62039)=(S=12989440,E=12989471,C=29)
	CT(62040)=(S=12989472,E=12989503,C=78)
	CT(62041)=(S=12989504,E=12989567,C=29)
	CT(62042)=(S=12989568,E=12989951,C=78)
	CT(62043)=(S=12989952,E=12989952,C=29)
	CT(62044)=(S=12989953,E=12989962,C=105)
	CT(62045)=(S=12989963,E=12989966,C=29)
	CT(62046)=(S=12989967,E=12990013,C=105)
	CT(62047)=(S=12990014,E=12990015,C=29)
	CT(62048)=(S=12990016,E=12990032,C=105)
	CT(62049)=(S=12990033,E=12990034,C=29)
	CT(62050)=(S=12990035,E=12990039,C=105)
	CT(62051)=(S=12990040,E=12990042,C=29)
	CT(62052)=(S=12990043,E=12990043,C=105)
	CT(62053)=(S=12990044,E=12990049,C=29)
	CT(62054)=(S=12990050,E=12990051,C=105)
	CT(62055)=(S=12990052,E=12990055,C=78)
	CT(62056)=(S=12990056,E=12990092,C=29)
	CT(62057)=(S=12990093,E=12990103,C=105)
	CT(62058)=(S=12990104,E=12990105,C=29)
	CT(62059)=(S=12990106,E=12990115,C=105)
	CT(62060)=(S=12990116,E=12990116,C=29)
	CT(62061)=(S=12990117,E=12990117,C=105)
	CT(62062)=(S=12990118,E=12990121,C=29)
	CT(62063)=(S=12990122,E=12990122,C=105)
	CT(62064)=(S=12990123,E=12990124,C=29)
	CT(62065)=(S=12990125,E=12990133,C=105)
	CT(62066)=(S=12990134,E=12990134,C=29)
	CT(62067)=(S=12990135,E=12990151,C=105)
	CT(62068)=(S=12990152,E=12990153,C=29)
	CT(62069)=(S=12990154,E=12990162,C=105)
	CT(62070)=(S=12990163,E=12990163,C=29)
	CT(62071)=(S=12990164,E=12990167,C=78)
	CT(62072)=(S=12990168,E=12990170,C=29)
	CT(62073)=(S=12990171,E=12990172,C=105)
	CT(62074)=(S=12990173,E=12990174,C=29)
	CT(62075)=(S=12990175,E=12990175,C=105)
	CT(62076)=(S=12990176,E=12990176,C=29)
	CT(62077)=(S=12990177,E=12990181,C=105)
	CT(62078)=(S=12990182,E=12990182,C=29)
	CT(62079)=(S=12990183,E=12990184,C=105)
	CT(62080)=(S=12990185,E=12990185,C=29)
	CT(62081)=(S=12990186,E=12990186,C=105)
	CT(62082)=(S=12990187,E=12990198,C=29)
	CT(62083)=(S=12990199,E=12990206,C=105)
	CT(62084)=(S=12990207,E=12990223,C=29)
	CT(62085)=(S=12990224,E=12990233,C=80)
	CT(62086)=(S=12990234,E=12990255,C=29)
	CT(62087)=(S=12990256,E=12990271,C=78)
	CT(62088)=(S=12990272,E=12990435,C=29)
	CT(62089)=(S=12990436,E=12990439,C=78)
	CT(62090)=(S=12990440,E=12990519,C=29)
	CT(62091)=(S=12990520,E=12990527,C=79)
	CT(62092)=(S=12990528,E=12990745,C=29)
	CT(62093)=(S=12990746,E=12990747,C=78)
	CT(62094)=(S=12990748,E=12990783,C=29)
	CT(62095)=(S=12990784,E=12990799,C=78)
	CT(62096)=(S=12990800,E=12990975,C=29)
	CT(62097)=(S=12990976,E=12990977,C=175)
	CT(62098)=(S=12990978,E=12990982,C=29)
	CT(62099)=(S=12990983,E=12990983,C=78)
	CT(62100)=(S=12990984,E=12990988,C=189)
	CT(62101)=(S=12990989,E=12991103,C=29)
	CT(62102)=(S=12991104,E=12991231,C=78)
	CT(62103)=(S=12991232,E=12992015,C=29)
	CT(62104)=(S=12992016,E=12992031,C=78)
	CT(62105)=(S=12992032,E=12992093,C=29)
	CT(62106)=(S=12992094,E=12992094,C=78)
	CT(62107)=(S=12992095,E=12992111,C=29)
	CT(62108)=(S=12992112,E=12992112,C=78)
	CT(62109)=(S=12992113,E=12992157,C=29)
	CT(62110)=(S=12992158,E=12992159,C=78)
	CT(62111)=(S=12992160,E=12992163,C=29)
	CT(62112)=(S=12992164,E=12992164,C=78)
	CT(62113)=(S=12992165,E=12992174,C=29)
	CT(62114)=(S=12992175,E=12992175,C=78)
	CT(62115)=(S=12992176,E=12994583,C=29)
	CT(62116)=(S=12994584,E=12994591,C=78)
	CT(62117)=(S=12994592,E=12994655,C=29)
	CT(62118)=(S=12994656,E=12994687,C=78)
	CT(62119)=(S=12994688,E=12994846,C=29)
	CT(62120)=(S=12994847,E=12994847,C=78)
	CT(62121)=(S=12994848,E=12994848,C=29)
	CT(62122)=(S=12994849,E=12994853,C=78)
	CT(62123)=(S=12994854,E=12994854,C=29)
	CT(62124)=(S=12994855,E=12994910,C=78)
	CT(62125)=(S=12994911,E=12994919,C=29)
	CT(62126)=(S=12994920,E=12994920,C=78)
	CT(62127)=(S=12994921,E=12994924,C=29)
	CT(62128)=(S=12994925,E=12994973,C=78)
	CT(62129)=(S=12994974,E=12994976,C=29)
	CT(62130)=(S=12994977,E=12994987,C=78)
	CT(62131)=(S=12994988,E=12994991,C=29)
	CT(62132)=(S=12994992,E=12995007,C=78)
	CT(62133)=(S=12995008,E=12995023,C=29)
	CT(62134)=(S=12995024,E=12995024,C=78)
	CT(62135)=(S=12995025,E=12995025,C=29)
	CT(62136)=(S=12995026,E=12995035,C=78)
	CT(62137)=(S=12995036,E=12995036,C=2)
	CT(62138)=(S=12995037,E=12995037,C=78)
	CT(62139)=(S=12995038,E=12995055,C=29)
	CT(62140)=(S=12995056,E=12995068,C=78)
	CT(62141)=(S=12995069,E=12995069,C=29)
	CT(62142)=(S=12995070,E=12995070,C=78)
	CT(62143)=(S=12995071,E=12997645,C=29)
	CT(62144)=(S=12997646,E=12997647,C=78)
	CT(62145)=(S=12997648,E=12997759,C=29)
	CT(62146)=(S=12997760,E=12997887,C=78)
	CT(62147)=(S=12997888,E=12998919,C=29)
	CT(62148)=(S=12998920,E=12998926,C=78)
	CT(62149)=(S=12998927,E=12999172,C=29)
	CT(62150)=(S=12999173,E=12999175,C=78)
	CT(62151)=(S=12999176,E=12999243,C=29)
	CT(62152)=(S=12999244,E=12999263,C=78)
	CT(62153)=(S=12999264,E=12999551,C=29)
	CT(62154)=(S=12999552,E=12999679,C=78)
	CT(62155)=(S=12999680,E=13000704,C=29)
	CT(62156)=(S=13000705,E=13000707,C=78)
	CT(62157)=(S=13000708,E=13000709,C=29)
	CT(62158)=(S=13000710,E=13000725,C=78)
	CT(62159)=(S=13000726,E=13000726,C=29)
	CT(62160)=(S=13000727,E=13000732,C=78)
	CT(62161)=(S=13000733,E=13000733,C=29)
	CT(62162)=(S=13000734,E=13000766,C=78)
	CT(62163)=(S=13000767,E=13000767,C=29)
	CT(62164)=(S=13000768,E=13000791,C=78)
	CT(62165)=(S=13000792,E=13000799,C=29)
	CT(62166)=(S=13000800,E=13000917,C=78)
	CT(62167)=(S=13000918,E=13000919,C=29)
	CT(62168)=(S=13000920,E=13000923,C=78)
	CT(62169)=(S=13000924,E=13000926,C=29)
	CT(62170)=(S=13000927,E=13000943,C=78)
	CT(62171)=(S=13000944,E=13000949,C=29)
	CT(62172)=(S=13000950,E=13000959,C=78)
	CT(62173)=(S=13000960,E=13001170,C=29)
	CT(62174)=(S=13001171,E=13001175,C=78)
	CT(62175)=(S=13001176,E=13001196,C=29)
	CT(62176)=(S=13001197,E=13001197,C=78)
	CT(62177)=(S=13001198,E=13001399,C=29)
	CT(62178)=(S=13001400,E=13001407,C=78)
	CT(62179)=(S=13001408,E=13001499,C=29)
	CT(62180)=(S=13001500,E=13001503,C=78)
	CT(62181)=(S=13001504,E=13001619,C=29)
	CT(62182)=(S=13001620,E=13001620,C=34)
	CT(62183)=(S=13001621,E=13001641,C=29)
	CT(62184)=(S=13001642,E=13001642,C=78)
	CT(62185)=(S=13001643,E=13001644,C=29)
	CT(62186)=(S=13001645,E=13001645,C=8)
	CT(62187)=(S=13001646,E=13001689,C=29)
	CT(62188)=(S=13001690,E=13001693,C=78)
	CT(62189)=(S=13001694,E=13001694,C=34)
	CT(62190)=(S=13001695,E=13001855,C=29)
	CT(62191)=(S=13001856,E=13001863,C=78)
	CT(62192)=(S=13001864,E=13001871,C=29)
	CT(62193)=(S=13001872,E=13001887,C=78)
	CT(62194)=(S=13001888,E=13002011,C=29)
	CT(62195)=(S=13002012,E=13002015,C=182)
	CT(62196)=(S=13002016,E=13002413,C=29)
	CT(62197)=(S=13002414,E=13002414,C=78)
	CT(62198)=(S=13002415,E=13002460,C=29)
	CT(62199)=(S=13002461,E=13002461,C=78)
	CT(62200)=(S=13002462,E=13002463,C=29)
	CT(62201)=(S=13002464,E=13002467,C=78)
	CT(62202)=(S=13002468,E=13002495,C=29)
	CT(62203)=(S=13002496,E=13002751,C=78)
	CT(62204)=(S=13002752,E=13010239,C=29)
	CT(62205)=(S=13010240,E=13010248,C=78)
	CT(62206)=(S=13010249,E=13010250,C=29)
	CT(62207)=(S=13010251,E=13010251,C=78)
	CT(62208)=(S=13010252,E=13010255,C=29)
	CT(62209)=(S=13010256,E=13010261,C=14)
	CT(62210)=(S=13010262,E=13010315,C=29)
	CT(62211)=(S=13010316,E=13010316,C=34)
	CT(62212)=(S=13010317,E=13010337,C=29)
	CT(62213)=(S=13010338,E=13010340,C=78)
	CT(62214)=(S=13010341,E=13010349,C=29)
	CT(62215)=(S=13010350,E=13010350,C=78)
	CT(62216)=(S=13010351,E=13010401,C=29)
	CT(62217)=(S=13010402,E=13010402,C=14)
	CT(62218)=(S=13010403,E=13010406,C=29)
	CT(62219)=(S=13010407,E=13010407,C=78)
	CT(62220)=(S=13010408,E=13010459,C=29)
	CT(62221)=(S=13010460,E=13010463,C=78)
	CT(62222)=(S=13010464,E=13010602,C=29)
	CT(62223)=(S=13010603,E=13010603,C=78)
	CT(62224)=(S=13010604,E=13010650,C=29)
	CT(62225)=(S=13010651,E=13010651,C=78)
	CT(62226)=(S=13010652,E=13010759,C=29)
	CT(62227)=(S=13010760,E=13010760,C=78)
	CT(62228)=(S=13010761,E=13010794,C=29)
	CT(62229)=(S=13010795,E=13010795,C=78)
	CT(62230)=(S=13010796,E=13010803,C=29)
	CT(62231)=(S=13010804,E=13010805,C=78)
	CT(62232)=(S=13010806,E=13010824,C=29)
	CT(62233)=(S=13010825,E=13010826,C=34)
	CT(62234)=(S=13010827,E=13010854,C=29)
	CT(62235)=(S=13010855,E=13010855,C=14)
	CT(62236)=(S=13010856,E=13010879,C=29)
	CT(62237)=(S=13010880,E=13010881,C=78)
	CT(62238)=(S=13010882,E=13010903,C=29)
	CT(62239)=(S=13010904,E=13010904,C=78)
	CT(62240)=(S=13010905,E=13010923,C=29)
	CT(62241)=(S=13010924,E=13010924,C=78)
	CT(62242)=(S=13010925,E=13010987,C=29)
	CT(62243)=(S=13010988,E=13010991,C=78)
	CT(62244)=(S=13010992,E=13011198,C=29)
	CT(62245)=(S=13011199,E=13011199,C=78)
	CT(62246)=(S=13011200,E=13011345,C=29)
	CT(62247)=(S=13011346,E=13011346,C=78)
	CT(62248)=(S=13011347,E=13012101,C=29)
	CT(62249)=(S=13012102,E=13012102,C=78)
	CT(62250)=(S=13012103,E=13012118,C=29)
	CT(62251)=(S=13012119,E=13012119,C=78)
	CT(62252)=(S=13012120,E=13012223,C=29)
	CT(62253)=(S=13012224,E=13012479,C=33)
	CT(62254)=(S=13012480,E=13012735,C=0)
	CT(62255)=(S=13012736,E=13013135,C=29)
	CT(62256)=(S=13013136,E=13013151,C=78)
	CT(62257)=(S=13013152,E=13013919,C=29)
	CT(62258)=(S=13013920,E=13013920,C=34)
	CT(62259)=(S=13013921,E=13013927,C=29)
	CT(62260)=(S=13013928,E=13013929,C=78)
	CT(62261)=(S=13013930,E=13014189,C=29)
	CT(62262)=(S=13014190,E=13014190,C=78)
	CT(62263)=(S=13014191,E=13014192,C=29)
	CT(62264)=(S=13014193,E=13014194,C=34)
	CT(62265)=(S=13014195,E=13014204,C=29)
	CT(62266)=(S=13014205,E=13014205,C=0)
	CT(62267)=(S=13014206,E=13014207,C=29)
	CT(62268)=(S=13014208,E=13014211,C=6)
	CT(62269)=(S=13014212,E=13014213,C=0)
	CT(62270)=(S=13014214,E=13014246,C=29)
	CT(62271)=(S=13014247,E=13014247,C=78)
	CT(62272)=(S=13014248,E=13014254,C=29)
	CT(62273)=(S=13014255,E=13014255,C=78)
	CT(62274)=(S=13014256,E=13014915,C=29)
	CT(62275)=(S=13014916,E=13014916,C=204)
	CT(62276)=(S=13014917,E=13015371,C=29)
	CT(62277)=(S=13015372,E=13015375,C=78)
	CT(62278)=(S=13015376,E=13015558,C=29)
	CT(62279)=(S=13015559,E=13015559,C=78)
	CT(62280)=(S=13015560,E=13015579,C=29)
	CT(62281)=(S=13015580,E=13015583,C=78)
	CT(62282)=(S=13015584,E=13015735,C=29)
	CT(62283)=(S=13015736,E=13015743,C=78)
	CT(62284)=(S=13015744,E=13017278,C=29)
	CT(62285)=(S=13017279,E=13017279,C=78)
	CT(62286)=(S=13017280,E=13017344,C=29)
	CT(62287)=(S=13017345,E=13017423,C=78)
	CT(62288)=(S=13017424,E=13017424,C=29)
	CT(62289)=(S=13017425,E=13017426,C=78)
	CT(62290)=(S=13017427,E=13017427,C=29)
	CT(62291)=(S=13017428,E=13017430,C=78)
	CT(62292)=(S=13017431,E=13017431,C=29)
	CT(62293)=(S=13017432,E=13017475,C=78)
	CT(62294)=(S=13017476,E=13017491,C=29)
	CT(62295)=(S=13017492,E=13017508,C=78)
	CT(62296)=(S=13017509,E=13017515,C=29)
	CT(62297)=(S=13017516,E=13017543,C=78)
	CT(62298)=(S=13017544,E=13017546,C=29)
	CT(62299)=(S=13017547,E=13017580,C=78)
	CT(62300)=(S=13017581,E=13017581,C=29)
	CT(62301)=(S=13017582,E=13017582,C=78)
	CT(62302)=(S=13017583,E=13017583,C=29)
	CT(62303)=(S=13017584,E=13017589,C=78)
	CT(62304)=(S=13017590,E=13017591,C=29)
	CT(62305)=(S=13017592,E=13017598,C=78)
	CT(62306)=(S=13017599,E=13017600,C=29)
	CT(62307)=(S=13017601,E=13017611,C=78)
	CT(62308)=(S=13017612,E=13017612,C=29)
	CT(62309)=(S=13017613,E=13017676,C=78)
	CT(62310)=(S=13017677,E=13017679,C=29)
	CT(62311)=(S=13017680,E=13017681,C=78)
	CT(62312)=(S=13017682,E=13017683,C=29)
	CT(62313)=(S=13017684,E=13017687,C=78)
	CT(62314)=(S=13017688,E=13017693,C=29)
	CT(62315)=(S=13017694,E=13017791,C=78)
	CT(62316)=(S=13017792,E=13017807,C=29)
	CT(62317)=(S=13017808,E=13017823,C=78)
	CT(62318)=(S=13017824,E=13017825,C=29)
	CT(62319)=(S=13017826,E=13017829,C=78)
	CT(62320)=(S=13017830,E=13017830,C=29)
	CT(62321)=(S=13017831,E=13017838,C=78)
	CT(62322)=(S=13017839,E=13017839,C=29)
	CT(62323)=(S=13017840,E=13017854,C=78)
	CT(62324)=(S=13017855,E=13017856,C=29)
	CT(62325)=(S=13017857,E=13017887,C=78)
	CT(62326)=(S=13017888,E=13017895,C=29)
	CT(62327)=(S=13017896,E=13017929,C=78)
	CT(62328)=(S=13017930,E=13017931,C=29)
	CT(62329)=(S=13017932,E=13017947,C=78)
	CT(62330)=(S=13017948,E=13017948,C=29)
	CT(62331)=(S=13017949,E=13017950,C=78)
	CT(62332)=(S=13017951,E=13017951,C=29)
	CT(62333)=(S=13017952,E=13018009,C=78)
	CT(62334)=(S=13018010,E=13018011,C=29)
	CT(62335)=(S=13018012,E=13018012,C=78)
	CT(62336)=(S=13018013,E=13018013,C=29)
	CT(62337)=(S=13018014,E=13018031,C=78)
	CT(62338)=(S=13018032,E=13018034,C=29)
	CT(62339)=(S=13018035,E=13018037,C=78)
	CT(62340)=(S=13018038,E=13018063,C=29)
	CT(62341)=(S=13018064,E=13018110,C=78)
	CT(62342)=(S=13018111,E=13018111,C=29)
	CT(62343)=(S=13018112,E=13018879,C=78)
	CT(62344)=(S=13018880,E=13018880,C=29)
	CT(62345)=(S=13018881,E=13018911,C=78)
	CT(62346)=(S=13018912,E=13018912,C=29)
	CT(62347)=(S=13018913,E=13019008,C=78)
	CT(62348)=(S=13019009,E=13019039,C=29)
	CT(62349)=(S=13019040,E=13019042,C=78)
	CT(62350)=(S=13019043,E=13019043,C=29)
	CT(62351)=(S=13019044,E=13019047,C=78)
	CT(62352)=(S=13019048,E=13019071,C=29)
	CT(62353)=(S=13019072,E=13019103,C=230)
	CT(62354)=(S=13019104,E=13019129,C=29)
	CT(62355)=(S=13019130,E=13019134,C=78)
	CT(62356)=(S=13019136,E=13019136,C=29)
	CT(62357)=(S=13019137,E=13019165,C=78)
	CT(62358)=(S=13019166,E=13019177,C=29)
	CT(62359)=(S=13019178,E=13019178,C=78)
	CT(62360)=(S=13019179,E=13019179,C=29)
	CT(62361)=(S=13019180,E=13019229,C=78)
	CT(62362)=(S=13019230,E=13019231,C=29)
	CT(62363)=(S=13019232,E=13019240,C=78)
	CT(62364)=(S=13019241,E=13019253,C=29)
	CT(62365)=(S=13019254,E=13019257,C=78)
	CT(62366)=(S=13019260,E=13019262,C=29)
	CT(62367)=(S=13019263,E=13019327,C=78)
	CT(62368)=(S=13019328,E=13019328,C=29)
	CT(62369)=(S=13019329,E=13019391,C=78)
	CT(62370)=(S=13019392,E=13019392,C=29)
	CT(62371)=(S=13019393,E=13019398,C=78)
	CT(62372)=(S=13019399,E=13019399,C=29)
	CT(62373)=(S=13019400,E=13019452,C=78)
	CT(62374)=(S=13019453,E=13019455,C=29)
	CT(62375)=(S=13019456,E=13019487,C=78)
	CT(62376)=(S=13019488,E=13019491,C=29)
	CT(62377)=(S=13019492,E=13019519,C=78)
	CT(62378)=(S=13019520,E=13019520,C=29)
	CT(62379)=(S=13019521,E=13019563,C=78)
	CT(62380)=(S=13019564,E=13019565,C=29)
	CT(62381)=(S=13019566,E=13019579,C=78)
	CT(62382)=(S=13019580,E=13019581,C=29)
	CT(62383)=(S=13019582,E=13019585,C=78)
	CT(62384)=(S=13019586,E=13019587,C=29)
	CT(62385)=(S=13019588,E=13019599,C=78)
	CT(62386)=(S=13019600,E=13019601,C=29)
	CT(62387)=(S=13019602,E=13019621,C=78)
	CT(62388)=(S=13019622,E=13019629,C=29)
	CT(62389)=(S=13019630,E=13019646,C=78)
	CT(62390)=(S=13019647,E=13021406,C=29)
	CT(62391)=(S=13021407,E=13021411,C=28)
	CT(62392)=(S=13021412,E=13021811,C=29)
	CT(62393)=(S=13021812,E=13021815,C=78)
	CT(62394)=(S=13021816,E=13021850,C=29)
	CT(62395)=(S=13021851,E=13021851,C=78)
	CT(62396)=(S=13021852,E=13021969,C=29)
	CT(62397)=(S=13021970,E=13021970,C=78)
	CT(62398)=(S=13021971,E=13022170,C=29)
	CT(62399)=(S=13022171,E=13022171,C=78)
	CT(62400)=(S=13022172,E=13022176,C=29)
	CT(62401)=(S=13022177,E=13022177,C=8)
	CT(62402)=(S=13022178,E=13022349,C=29)
	CT(62403)=(S=13022350,E=13022350,C=2)
	CT(62404)=(S=13022351,E=13022583,C=29)
	CT(62405)=(S=13022584,E=13022591,C=78)
	CT(62406)=(S=13022592,E=13022662,C=29)
	CT(62407)=(S=13022663,E=13022663,C=78)
	CT(62408)=(S=13022664,E=13022889,C=29)
	CT(62409)=(S=13022890,E=13022890,C=178)
	CT(62410)=(S=13022891,E=13023200,C=29)
	CT(62411)=(S=13023201,E=13023201,C=78)
	CT(62412)=(S=13023202,E=13023392,C=29)
	CT(62413)=(S=13023393,E=13023393,C=189)
	CT(62414)=(S=13023394,E=13023428,C=29)
	CT(62415)=(S=13023429,E=13023429,C=78)
	CT(62416)=(S=13023430,E=13023515,C=29)
	CT(62417)=(S=13023516,E=13023519,C=78)
	CT(62418)=(S=13023520,E=13023704,C=29)
	CT(62419)=(S=13023705,E=13023705,C=78)
	CT(62420)=(S=13023706,E=13023978,C=29)
	CT(62421)=(S=13023979,E=13023979,C=178)
	CT(62422)=(S=13023980,E=13024170,C=29)
	CT(62423)=(S=13024171,E=13024171,C=178)
	CT(62424)=(S=13024172,E=13027078,C=29)
	CT(62425)=(S=13027079,E=13027079,C=78)
	CT(62426)=(S=13027080,E=13027085,C=29)
	CT(62427)=(S=13027086,E=13027087,C=78)
	CT(62428)=(S=13027088,E=13027327,C=29)
	CT(62429)=(S=13027328,E=13027337,C=10)
	CT(62430)=(S=13027338,E=13027391,C=29)
	CT(62431)=(S=13027392,E=13027455,C=78)
	CT(62432)=(S=13027456,E=13027569,C=29)
	CT(62433)=(S=13027570,E=13027570,C=78)
	CT(62434)=(S=13027571,E=13027969,C=29)
	CT(62435)=(S=13027970,E=13027970,C=78)
	CT(62436)=(S=13027971,E=13028330,C=29)
	CT(62437)=(S=13028331,E=13028331,C=78)
	CT(62438)=(S=13028332,E=13028422,C=29)
	CT(62439)=(S=13028423,E=13028424,C=80)
	CT(62440)=(S=13028425,E=13029060,C=29)
	CT(62441)=(S=13029061,E=13029061,C=34)
	CT(62442)=(S=13029062,E=13029063,C=29)
	CT(62443)=(S=13029064,E=13029066,C=34)
	CT(62444)=(S=13029067,E=13029127,C=29)
	CT(62445)=(S=13029136,E=13029143,C=178)
	CT(62446)=(S=13029144,E=13030406,C=29)
	CT(62447)=(S=13030407,E=13030407,C=78)
	CT(62448)=(S=13030408,E=13031795,C=29)
	CT(62449)=(S=13031796,E=13031799,C=78)
	CT(62450)=(S=13031800,E=13036288,C=29)
	CT(62451)=(S=13036289,E=13036319,C=78)
	CT(62452)=(S=13036320,E=13036335,C=29)
	CT(62453)=(S=13036336,E=13036341,C=78)
	CT(62454)=(S=13036342,E=13036343,C=29)
	CT(62455)=(S=13036344,E=13036415,C=78)
	CT(62456)=(S=13036416,E=13036416,C=29)
	CT(62457)=(S=13036417,E=13036417,C=78)
	CT(62458)=(S=13036418,E=13036423,C=29)
	CT(62459)=(S=13036424,E=13036465,C=78)
	CT(62460)=(S=13036466,E=13036467,C=29)
	CT(62461)=(S=13036468,E=13036491,C=78)
	CT(62462)=(S=13036492,E=13036493,C=29)
	CT(62463)=(S=13036494,E=13036542,C=78)
	CT(62464)=(S=13036543,E=13037695,C=29)
	CT(62465)=(S=13037696,E=13037823,C=34)
	CT(62466)=(S=13037824,E=13038655,C=29)
	CT(62467)=(S=13038656,E=13038671,C=80)
	CT(62468)=(S=13038672,E=13038861,C=29)
	CT(62469)=(S=13038862,E=13038863,C=78)
	CT(62470)=(S=13038864,E=13038895,C=29)
	CT(62471)=(S=13038896,E=13038911,C=78)
	CT(62472)=(S=13038912,E=13038943,C=29)
	CT(62473)=(S=13038944,E=13038959,C=80)
	CT(62474)=(S=13038960,E=13038975,C=78)
	CT(62475)=(S=13038976,E=13038986,C=29)
	CT(62476)=(S=13038987,E=13038987,C=80)
	CT(62477)=(S=13038988,E=13039007,C=29)
	CT(62478)=(S=13039008,E=13039008,C=79)
	CT(62479)=(S=13039009,E=13039023,C=29)
	CT(62480)=(S=13039024,E=13039025,C=78)
	CT(62481)=(S=13039026,E=13039102,C=29)
	CT(62482)=(S=13039103,E=13039103,C=80)
	CT(62483)=(S=13039104,E=13039131,C=29)
	CT(62484)=(S=13039132,E=13039135,C=78)
	CT(62485)=(S=13039136,E=13039147,C=29)
	CT(62486)=(S=13039148,E=13039151,C=78)
	CT(62487)=(S=13039152,E=13039332,C=29)
	CT(62488)=(S=13039333,E=13039334,C=79)
	CT(62489)=(S=13039335,E=13040431,C=29)
	CT(62490)=(S=13040432,E=13040447,C=78)
	CT(62491)=(S=13040448,E=13040511,C=29)
	CT(62492)=(S=13040512,E=13040639,C=78)
	CT(62493)=(S=13040640,E=13040719,C=29)
	CT(62494)=(S=13040720,E=13040751,C=78)
	CT(62495)=(S=13040752,E=13040767,C=29)
	CT(62496)=(S=13040768,E=13040771,C=78)
	CT(62497)=(S=13040772,E=13041279,C=29)
	CT(62498)=(S=13041280,E=13041407,C=78)
	CT(62499)=(S=13041408,E=13042831,C=29)
	CT(62500)=(S=13042832,E=13042832,C=78)
	CT(62501)=(S=13042833,E=13042877,C=29)
	CT(62502)=(S=13042878,E=13042878,C=78)
	CT(62503)=(S=13042879,E=13042942,C=29)
	CT(62504)=(S=13042943,E=13042943,C=78)
	CT(62505)=(S=13042944,E=13043137,C=29)
	CT(62506)=(S=13043138,E=13043138,C=78)
	CT(62507)=(S=13043139,E=13043525,C=29)
	CT(62508)=(S=13043526,E=13043526,C=36)
	CT(62509)=(S=13043527,E=13043542,C=29)
	CT(62510)=(S=13043543,E=13043545,C=78)
	CT(62511)=(S=13043546,E=13043546,C=231)
	CT(62512)=(S=13043547,E=13043567,C=29)
	CT(62513)=(S=13043568,E=13043575,C=79)
	CT(62514)=(S=13043576,E=13043591,C=29)
	CT(62515)=(S=13043592,E=13043595,C=78)
	CT(62516)=(S=13043596,E=13043611,C=29)
	CT(62517)=(S=13043612,E=13043615,C=78)
	CT(62518)=(S=13043616,E=13043623,C=29)
	CT(62519)=(S=13043624,E=13043627,C=80)
	CT(62520)=(S=13043628,E=13043663,C=29)
	CT(62521)=(S=13043664,E=13043667,C=80)
	CT(62522)=(S=13043668,E=13043671,C=29)
	CT(62523)=(S=13043672,E=13043675,C=78)
	CT(62524)=(S=13043676,E=13043687,C=29)
	CT(62525)=(S=13043688,E=13043695,C=78)
	CT(62526)=(S=13043696,E=13043991,C=29)
	CT(62527)=(S=13043992,E=13043999,C=8)
	CT(62528)=(S=13044000,E=13044024,C=29)
	CT(62529)=(S=13044025,E=13044025,C=78)
	CT(62530)=(S=13044028,E=13044249,C=29)
	CT(62531)=(S=13044252,E=13044255,C=78)
	CT(62532)=(S=13044256,E=13044289,C=29)
	CT(62533)=(S=13044290,E=13044290,C=78)
	CT(62534)=(S=13044291,E=13044307,C=29)
	CT(62535)=(S=13044308,E=13044308,C=78)
	CT(62536)=(S=13044309,E=13045583,C=29)
	CT(62537)=(S=13045584,E=13045592,C=78)
	CT(62538)=(S=13045593,E=13045731,C=29)
	CT(62539)=(S=13045732,E=13045735,C=178)
	CT(62540)=(S=13045736,E=13045815,C=29)
	CT(62541)=(S=13045816,E=13045819,C=183)
	CT(62542)=(S=13045820,E=13045871,C=29)
	CT(62543)=(S=13045872,E=13045875,C=78)
	CT(62544)=(S=13045876,E=13045887,C=29)
	CT(62545)=(S=13045888,E=13045891,C=78)
	CT(62546)=(S=13045892,E=13045911,C=29)
	CT(62547)=(S=13045912,E=13045915,C=78)
	CT(62548)=(S=13045916,E=13045939,C=29)
	CT(62549)=(S=13045940,E=13045943,C=78)
	CT(62550)=(S=13045944,E=13046575,C=29)
	CT(62551)=(S=13046576,E=13046591,C=78)
	CT(62552)=(S=13046592,E=13046619,C=29)
	CT(62553)=(S=13046620,E=13046623,C=78)
	CT(62554)=(S=13046624,E=13046687,C=29)
	CT(62555)=(S=13046688,E=13046703,C=78)
	CT(62556)=(S=13046704,E=13046739,C=29)
	CT(62557)=(S=13046740,E=13046751,C=78)
	CT(62558)=(S=13046752,E=13047187,C=29)
	CT(62559)=(S=13047188,E=13047191,C=78)
	CT(62560)=(S=13047192,E=13047203,C=29)
	CT(62561)=(S=13047204,E=13047207,C=230)
	CT(62562)=(S=13047208,E=13047219,C=29)
	CT(62563)=(S=13047220,E=13047223,C=78)
	CT(62564)=(S=13047224,E=13047255,C=29)
	CT(62565)=(S=13047256,E=13047259,C=78)
	CT(62566)=(S=13047260,E=13047295,C=29)
	CT(62567)=(S=13047296,E=13048319,C=78)
	CT(62568)=(S=13048320,E=13048384,C=29)
	CT(62569)=(S=13048388,E=13048391,C=78)
	CT(62570)=(S=13048392,E=13048399,C=29)
	CT(62571)=(S=13048400,E=13048403,C=78)
	CT(62572)=(S=13048404,E=13048509,C=29)
	CT(62573)=(S=13048510,E=13048511,C=78)
	CT(62574)=(S=13048512,E=13048553,C=29)
	CT(62575)=(S=13048554,E=13048554,C=209)
	CT(62576)=(S=13048555,E=13048671,C=29)
	CT(62577)=(S=13048672,E=13048679,C=80)
	CT(62578)=(S=13048680,E=13048755,C=29)
	CT(62579)=(S=13048756,E=13048759,C=78)
	CT(62580)=(S=13048760,E=13048791,C=29)
	CT(62581)=(S=13048792,E=13048799,C=78)
	CT(62582)=(S=13048800,E=13049519,C=29)
	CT(62583)=(S=13049520,E=13049527,C=78)
	CT(62584)=(S=13049528,E=13050200,C=29)
	CT(62585)=(S=13050201,E=13050201,C=78)
	CT(62586)=(S=13050202,E=13050205,C=29)
	CT(62587)=(S=13050206,E=13050207,C=78)
	CT(62588)=(S=13050208,E=13050223,C=29)
	CT(62589)=(S=13050224,E=13050227,C=78)
	CT(62590)=(S=13050228,E=13051103,C=29)
	CT(62591)=(S=13051104,E=13051111,C=78)
	CT(62592)=(S=13051112,E=13051487,C=29)
	CT(62593)=(S=13051488,E=13051495,C=78)
	CT(62594)=(S=13051496,E=13051583,C=29)
	CT(62595)=(S=13051584,E=13051591,C=82)
	CT(62596)=(S=13051592,E=13051627,C=29)
	CT(62597)=(S=13051628,E=13051631,C=78)
	CT(62598)=(S=13051632,E=13051903,C=29)
	CT(62599)=(S=13051904,E=13052415,C=33)
	CT(62600)=(S=13052416,E=13052672,C=29)
	CT(62601)=(S=13052673,E=13052675,C=78)
	CT(62602)=(S=13052676,E=13052689,C=29)
	CT(62603)=(S=13052690,E=13052690,C=2)
	CT(62604)=(S=13052691,E=13052691,C=8)
	CT(62605)=(S=13052692,E=13052716,C=29)
	CT(62606)=(S=13052717,E=13052717,C=8)
	CT(62607)=(S=13052718,E=13052734,C=29)
	CT(62608)=(S=13052735,E=13052735,C=8)
	CT(62609)=(S=13052736,E=13052779,C=29)
	CT(62610)=(S=13052780,E=13052803,C=78)
	CT(62611)=(S=13052804,E=13052807,C=29)
	CT(62612)=(S=13052808,E=13052855,C=78)
	CT(62613)=(S=13052856,E=13052866,C=29)
	CT(62614)=(S=13052867,E=13052867,C=78)
	CT(62615)=(S=13052870,E=13052870,C=29)
	CT(62616)=(S=13052871,E=13052877,C=78)
	CT(62617)=(S=13052878,E=13052914,C=29)
	CT(62618)=(S=13052915,E=13052917,C=78)
	CT(62619)=(S=13052918,E=13052919,C=34)
	CT(62620)=(S=13052920,E=13052926,C=78)
	CT(62621)=(S=13052927,E=13053551,C=29)
	CT(62622)=(S=13053552,E=13053555,C=78)
	CT(62623)=(S=13053556,E=13053743,C=29)
	CT(62624)=(S=13053744,E=13053747,C=78)
	CT(62625)=(S=13053748,E=13053751,C=79)
	CT(62626)=(S=13053752,E=13053759,C=78)
	CT(62627)=(S=13053760,E=13053819,C=29)
	CT(62628)=(S=13053820,E=13053823,C=78)
	CT(62629)=(S=13053824,E=13053935,C=29)
	CT(62630)=(S=13053936,E=13053939,C=78)
	CT(62631)=(S=13053940,E=13054015,C=29)
	CT(62632)=(S=13054016,E=13054019,C=78)
	CT(62633)=(S=13054020,E=13054051,C=29)
	CT(62634)=(S=13054052,E=13054055,C=78)
	CT(62635)=(S=13054056,E=13054155,C=29)
	CT(62636)=(S=13054156,E=13054159,C=183)
	CT(62637)=(S=13054160,E=13054167,C=29)
	CT(62638)=(S=13054168,E=13054175,C=78)
	CT(62639)=(S=13054176,E=13055231,C=29)
	CT(62640)=(S=13055232,E=13055487,C=34)
	CT(62641)=(S=13055488,E=13056591,C=29)
	CT(62642)=(S=13056592,E=13056595,C=78)
	CT(62643)=(S=13056596,E=13056627,C=29)
	CT(62644)=(S=13056628,E=13056631,C=78)
	CT(62645)=(S=13056632,E=13056639,C=186)
	CT(62646)=(S=13056640,E=13056663,C=29)
	CT(62647)=(S=13056664,E=13056667,C=79)
	CT(62648)=(S=13056668,E=13056679,C=29)
	CT(62649)=(S=13056680,E=13056687,C=78)
	CT(62650)=(S=13056688,E=13056703,C=29)
	CT(62651)=(S=13056704,E=13056707,C=78)
	CT(62652)=(S=13056708,E=13056743,C=29)
	CT(62653)=(S=13056744,E=13056751,C=78)
	CT(62654)=(S=13056752,E=13056843,C=29)
	CT(62655)=(S=13056844,E=13056847,C=78)
	CT(62656)=(S=13056848,E=13056895,C=29)
	CT(62657)=(S=13056896,E=13056903,C=78)
	CT(62658)=(S=13056904,E=13056951,C=29)
	CT(62659)=(S=13056952,E=13056959,C=78)
	CT(62660)=(S=13056960,E=13057011,C=29)
	CT(62661)=(S=13057012,E=13057015,C=78)
	CT(62662)=(S=13057016,E=13057023,C=29)
	CT(62663)=(S=13057024,E=13057043,C=78)
	CT(62664)=(S=13057044,E=13057049,C=29)
	CT(62665)=(S=13057050,E=13057051,C=78)
	CT(62666)=(S=13057052,E=13057052,C=29)
	CT(62667)=(S=13057053,E=13057146,C=78)
	CT(62668)=(S=13057147,E=13057147,C=29)
	CT(62669)=(S=13057148,E=13057255,C=78)
	CT(62670)=(S=13057256,E=13057256,C=29)
	CT(62671)=(S=13057257,E=13057278,C=78)
	CT(62672)=(S=13057279,E=13058647,C=29)
	CT(62673)=(S=13058648,E=13058655,C=78)
	CT(62674)=(S=13058656,E=13058811,C=29)
	CT(62675)=(S=13058812,E=13058815,C=78)
	CT(62676)=(S=13058816,E=13059191,C=29)
	CT(62677)=(S=13059192,E=13059199,C=78)
	CT(62678)=(S=13059200,E=13059247,C=29)
	CT(62679)=(S=13059248,E=13059255,C=78)
	CT(62680)=(S=13059256,E=13059263,C=29)
	CT(62681)=(S=13059264,E=13059267,C=178)
	CT(62682)=(S=13059268,E=13059840,C=29)
	CT(62683)=(S=13059841,E=13059945,C=78)
	CT(62684)=(S=13059946,E=13059947,C=29)
	CT(62685)=(S=13059948,E=13059981,C=78)
	CT(62686)=(S=13059982,E=13059983,C=29)
	CT(62687)=(S=13059984,E=13059998,C=78)
	CT(62688)=(S=13059999,E=13059999,C=29)
	CT(62689)=(S=13060000,E=13060015,C=78)
	CT(62690)=(S=13060016,E=13060021,C=29)
	CT(62691)=(S=13060022,E=13060025,C=78)
	CT(62692)=(S=13060026,E=13060026,C=29)
	CT(62693)=(S=13060027,E=13060048,C=78)
	CT(62694)=(S=13060049,E=13060079,C=29)
	CT(62695)=(S=13060080,E=13060094,C=78)
	CT(62696)=(S=13060096,E=13060393,C=29)
	CT(62697)=(S=13060394,E=13060395,C=78)
	CT(62698)=(S=13060396,E=13060411,C=29)
	CT(62699)=(S=13060412,E=13060415,C=175)
	CT(62700)=(S=13060416,E=13061947,C=29)
	CT(62701)=(S=13061948,E=13061951,C=95)
	CT(62702)=(S=13061952,E=13062953,C=29)
	CT(62703)=(S=13062954,E=13062955,C=78)
	CT(62704)=(S=13062956,E=13062995,C=29)
	CT(62705)=(S=13062996,E=13062999,C=78)
	CT(62706)=(S=13063000,E=13063047,C=29)
	CT(62707)=(S=13063048,E=13063051,C=78)
	CT(62708)=(S=13063052,E=13063063,C=29)
	CT(62709)=(S=13063064,E=13063071,C=78)
	CT(62710)=(S=13063072,E=13063103,C=29)
	CT(62711)=(S=13063104,E=13063111,C=186)
	CT(62712)=(S=13063112,E=13063119,C=78)
	CT(62713)=(S=13063120,E=13063151,C=29)
	CT(62714)=(S=13063152,E=13063159,C=78)
	CT(62715)=(S=13063160,E=13063168,C=29)
	CT(62716)=(S=13063169,E=13063263,C=78)
	CT(62717)=(S=13063264,E=13063295,C=29)
	CT(62718)=(S=13063296,E=13063422,C=78)
	CT(62719)=(S=13063423,E=13063425,C=29)
	CT(62720)=(S=13063426,E=13063430,C=78)
	CT(62721)=(S=13063431,E=13063431,C=29)
	CT(62722)=(S=13063440,E=13063445,C=78)
	CT(62723)=(S=13063446,E=13063447,C=29)
	CT(62724)=(S=13063448,E=13063486,C=78)
	CT(62725)=(S=13063487,E=13063487,C=29)
	CT(62726)=(S=13063488,E=13063499,C=78)
	CT(62727)=(S=13063500,E=13063521,C=29)
	CT(62728)=(S=13063523,E=13063539,C=78)
	CT(62729)=(S=13063540,E=13063541,C=29)
	CT(62730)=(S=13063542,E=13063548,C=78)
	CT(62731)=(S=13063549,E=13063551,C=29)
	CT(62732)=(S=13063552,E=13063623,C=78)
	CT(62733)=(S=13063624,E=13063624,C=29)
	CT(62734)=(S=13063625,E=13063627,C=78)
	CT(62735)=(S=13063628,E=13063631,C=80)
	CT(62736)=(S=13063632,E=13063643,C=29)
	CT(62737)=(S=13063644,E=13063657,C=78)
	CT(62738)=(S=13063658,E=13063659,C=29)
	CT(62739)=(S=13063660,E=13063663,C=186)
	CT(62740)=(S=13063664,E=13063667,C=29)
	CT(62741)=(S=13063669,E=13063669,C=78)
	CT(62742)=(S=13063670,E=13063670,C=29)
	CT(62743)=(S=13063671,E=13063673,C=78)
	CT(62744)=(S=13063674,E=13063674,C=29)
	CT(62745)=(S=13063675,E=13063678,C=78)
	CT(62746)=(S=13063679,E=13063983,C=29)
	CT(62747)=(S=13063984,E=13063987,C=78)
	CT(62748)=(S=13063988,E=13064087,C=29)
	CT(62749)=(S=13064088,E=13064095,C=78)
	CT(62750)=(S=13064096,E=13064175,C=29)
	CT(62751)=(S=13064176,E=13064183,C=78)
	CT(62752)=(S=13064184,E=13064399,C=29)
	CT(62753)=(S=13064400,E=13064407,C=36)
	CT(62754)=(S=13064408,E=13064975,C=29)
	CT(62755)=(S=13064976,E=13064991,C=34)
	CT(62756)=(S=13064992,E=13065035,C=29)
	CT(62757)=(S=13065036,E=13065039,C=80)
	CT(62758)=(S=13065040,E=13065047,C=78)
	CT(62759)=(S=13065048,E=13065055,C=29)
	CT(62760)=(S=13065056,E=13065063,C=95)
	CT(62761)=(S=13065064,E=13065071,C=29)
	CT(62762)=(S=13065072,E=13065079,C=78)
	CT(62763)=(S=13065080,E=13065119,C=29)
	CT(62764)=(S=13065120,E=13065123,C=198)
	CT(62765)=(S=13065124,E=13065143,C=29)
	CT(62766)=(S=13065144,E=13065151,C=78)
	CT(62767)=(S=13065152,E=13065163,C=29)
	CT(62768)=(S=13065164,E=13065175,C=78)
	CT(62769)=(S=13065176,E=13065199,C=29)
	CT(62770)=(S=13065200,E=13065207,C=78)
	CT(62771)=(S=13065208,E=13065211,C=29)
	CT(62772)=(S=13065212,E=13065215,C=78)
	CT(62773)=(S=13065216,E=13066267,C=29)
	CT(62774)=(S=13066268,E=13066271,C=78)
	CT(62775)=(S=13066272,E=13066303,C=29)
	CT(62776)=(S=13066304,E=13066307,C=178)
	CT(62777)=(S=13066308,E=13066327,C=29)
	CT(62778)=(S=13066328,E=13066335,C=78)
	CT(62779)=(S=13066336,E=13066435,C=29)
	CT(62780)=(S=13066436,E=13066439,C=78)
	CT(62781)=(S=13066440,E=13066447,C=29)
	CT(62782)=(S=13066448,E=13066451,C=78)
	CT(62783)=(S=13066452,E=13067575,C=29)
	CT(62784)=(S=13067576,E=13067583,C=78)
	CT(62785)=(S=13067584,E=13067591,C=29)
	CT(62786)=(S=13067592,E=13067595,C=78)
	CT(62787)=(S=13067596,E=13067683,C=29)
	CT(62788)=(S=13067684,E=13067687,C=78)
	CT(62789)=(S=13067688,E=13067707,C=29)
	CT(62790)=(S=13067708,E=13067711,C=185)
	CT(62791)=(S=13067712,E=13067731,C=29)
	CT(62792)=(S=13067732,E=13067735,C=78)
	CT(62793)=(S=13067736,E=13067871,C=29)
	CT(62794)=(S=13067872,E=13067879,C=78)
	CT(62795)=(S=13067880,E=13067903,C=29)
	CT(62796)=(S=13067904,E=13067907,C=78)
	CT(62797)=(S=13067908,E=13067931,C=29)
	CT(62798)=(S=13067932,E=13067935,C=78)
	CT(62799)=(S=13067936,E=13067963,C=29)
	CT(62800)=(S=13067964,E=13067967,C=81)
	CT(62801)=(S=13067968,E=13067975,C=29)
	CT(62802)=(S=13067976,E=13067983,C=78)
	CT(62803)=(S=13067984,E=13068039,C=29)
	CT(62804)=(S=13068040,E=13068044,C=36)
	CT(62805)=(S=13068045,E=13068057,C=29)
	CT(62806)=(S=13068060,E=13068063,C=179)
	CT(62807)=(S=13068064,E=13068087,C=29)
	CT(62808)=(S=13068088,E=13068095,C=78)
	CT(62809)=(S=13068096,E=13068099,C=14)
	CT(62810)=(S=13068100,E=13068100,C=7)
	CT(62811)=(S=13068101,E=13068104,C=29)
	CT(62812)=(S=13068105,E=13068105,C=25)
	CT(62813)=(S=13068106,E=13068106,C=70)
	CT(62814)=(S=13068107,E=13068109,C=28)
	CT(62815)=(S=13068110,E=13068112,C=29)
	CT(62816)=(S=13068113,E=13068113,C=25)
	CT(62817)=(S=13068114,E=13068115,C=14)
	CT(62818)=(S=13068116,E=13068116,C=29)
	CT(62819)=(S=13068117,E=13068117,C=78)
	CT(62820)=(S=13068118,E=13068118,C=14)
	CT(62821)=(S=13068119,E=13068120,C=12)
	CT(62822)=(S=13068122,E=13068123,C=29)
	CT(62823)=(S=13068124,E=13068124,C=12)
	CT(62824)=(S=13068125,E=13068125,C=28)
	CT(62825)=(S=13068126,E=13068128,C=29)
	CT(62826)=(S=13068129,E=13068131,C=28)
	CT(62827)=(S=13068134,E=13068134,C=203)
	CT(62828)=(S=13068135,E=13068135,C=2)
	CT(62829)=(S=13068136,E=13068137,C=42)
	CT(62830)=(S=13068138,E=13068138,C=34)
	CT(62831)=(S=13068139,E=13068139,C=28)
	CT(62832)=(S=13068140,E=13068142,C=29)
	CT(62833)=(S=13068143,E=13068144,C=34)
	CT(62834)=(S=13068145,E=13068150,C=29)
	CT(62835)=(S=13068151,E=13068151,C=15)
	CT(62836)=(S=13068152,E=13068154,C=29)
	CT(62837)=(S=13068155,E=13068159,C=49)
	CT(62838)=(S=13068160,E=13071079,C=29)
	CT(62839)=(S=13071080,E=13071083,C=78)
	CT(62840)=(S=13071084,E=13071099,C=29)
	CT(62841)=(S=13071100,E=13071103,C=78)
	CT(62842)=(S=13071104,E=13071167,C=29)
	CT(62843)=(S=13071168,E=13071175,C=78)
	CT(62844)=(S=13071176,E=13071191,C=29)
	CT(62845)=(S=13071192,E=13071199,C=78)
	CT(62846)=(S=13071200,E=13071227,C=29)
	CT(62847)=(S=13071228,E=13071231,C=78)
	CT(62848)=(S=13071232,E=13071255,C=29)
	CT(62849)=(S=13071256,E=13071262,C=78)
	CT(62850)=(S=13071263,E=13071415,C=29)
	CT(62851)=(S=13071416,E=13071423,C=80)
	CT(62852)=(S=13071424,E=13071571,C=29)
	CT(62853)=(S=13071576,E=13071583,C=78)
	CT(62854)=(S=13071584,E=13071591,C=29)
	CT(62855)=(S=13071592,E=13071595,C=78)
	CT(62856)=(S=13071596,E=13071599,C=29)
	CT(62857)=(S=13071600,E=13071603,C=78)
	CT(62858)=(S=13071604,E=13072255,C=29)
	CT(62859)=(S=13072256,E=13072259,C=78)
	CT(62860)=(S=13072260,E=13072343,C=29)
	CT(62861)=(S=13072344,E=13072347,C=78)
	CT(62862)=(S=13072348,E=13072359,C=29)
	CT(62863)=(S=13072360,E=13072363,C=78)
	CT(62864)=(S=13072364,E=13072375,C=29)
	CT(62865)=(S=13072376,E=13072379,C=80)
	CT(62866)=(S=13072380,E=13072383,C=78)
	CT(62867)=(S=13072384,E=13072621,C=29)
	CT(62868)=(S=13072623,E=13072623,C=78)
	CT(62869)=(S=13072624,E=13072902,C=29)
	CT(62870)=(S=13072903,E=13072903,C=78)
	CT(62871)=(S=13072904,E=13073011,C=29)
	CT(62872)=(S=13073012,E=13073015,C=78)
	CT(62873)=(S=13073016,E=13073410,C=29)
	CT(62874)=(S=13073411,E=13073411,C=78)
	CT(62875)=(S=13073412,E=13073919,C=29)
	CT(62876)=(S=13073920,E=13074175,C=78)
	CT(62877)=(S=13074176,E=13074283,C=29)
	CT(62878)=(S=13074284,E=13074287,C=78)
	CT(62879)=(S=13074288,E=13074303,C=29)
	CT(62880)=(S=13074304,E=13074307,C=78)
	CT(62881)=(S=13074308,E=13074347,C=29)
	CT(62882)=(S=13074348,E=13074351,C=78)
	CT(62883)=(S=13074352,E=13074371,C=29)
	CT(62884)=(S=13074372,E=13074375,C=182)
	CT(62885)=(S=13074376,E=13083776,C=29)
	CT(62886)=(S=13083777,E=13083777,C=78)
	CT(62887)=(S=13083778,E=13083839,C=29)
	CT(62888)=(S=13083840,E=13083840,C=78)
	CT(62889)=(S=13083841,E=13083902,C=29)
	CT(62890)=(S=13083903,E=13083903,C=78)
	CT(62891)=(S=13083904,E=13084115,C=29)
	CT(62892)=(S=13084116,E=13084116,C=78)
	CT(62893)=(S=13084117,E=13084160,C=29)
	CT(62894)=(S=13084161,E=13084162,C=78)
	CT(62895)=(S=13084163,E=13084163,C=29)
	CT(62896)=(S=13084164,E=13084183,C=78)
	CT(62897)=(S=13084184,E=13084193,C=29)
	CT(62898)=(S=13084194,E=13084197,C=4)
	CT(62899)=(S=13084198,E=13084375,C=78)
	CT(62900)=(S=13084376,E=13084378,C=29)
	CT(62901)=(S=13084379,E=13084407,C=78)
	CT(62902)=(S=13084408,E=13084411,C=29)
	CT(62903)=(S=13084412,E=13084414,C=78)
	CT(62904)=(S=13084415,E=13084431,C=29)
	CT(62905)=(S=13084432,E=13084443,C=78)
	CT(62906)=(S=13084444,E=13084519,C=29)
	CT(62907)=(S=13084520,E=13084527,C=78)
	CT(62908)=(S=13084528,E=13084531,C=29)
	CT(62909)=(S=13084532,E=13084535,C=78)
	CT(62910)=(S=13084536,E=13084543,C=29)
	CT(62911)=(S=13084544,E=13084547,C=78)
	CT(62912)=(S=13084548,E=13084551,C=29)
	CT(62913)=(S=13084552,E=13084559,C=78)
	CT(62914)=(S=13084560,E=13084567,C=29)
	CT(62915)=(S=13084568,E=13084583,C=78)
	CT(62916)=(S=13084584,E=13084887,C=29)
	CT(62917)=(S=13084888,E=13084895,C=78)
	CT(62918)=(S=13084896,E=13084919,C=29)
	CT(62919)=(S=13084920,E=13084923,C=78)
	CT(62920)=(S=13084924,E=13086463,C=29)
	CT(62921)=(S=13086464,E=13086507,C=78)
	CT(62922)=(S=13086508,E=13086523,C=29)
	CT(62923)=(S=13086524,E=13086564,C=78)
	CT(62924)=(S=13086565,E=13086565,C=29)
	CT(62925)=(S=13086566,E=13086647,C=78)
	CT(62926)=(S=13086648,E=13086652,C=29)
	CT(62927)=(S=13086653,E=13086718,C=78)
	CT(62928)=(S=13086719,E=13087769,C=29)
	CT(62929)=(S=13087772,E=13087775,C=78)
	CT(62930)=(S=13087776,E=13087795,C=29)
	CT(62931)=(S=13087796,E=13087799,C=78)
	CT(62932)=(S=13087800,E=13087839,C=29)
	CT(62933)=(S=13087840,E=13087843,C=78)
	CT(62934)=(S=13087844,E=13087911,C=29)
	CT(62935)=(S=13087912,E=13087919,C=78)
	CT(62936)=(S=13087920,E=13088080,C=29)
	CT(62937)=(S=13088081,E=13088081,C=78)
	CT(62938)=(S=13088082,E=13088255,C=29)
	CT(62939)=(S=13088256,E=13088303,C=78)
	CT(62940)=(S=13088304,E=13088307,C=29)
	CT(62941)=(S=13088308,E=13088311,C=78)
	CT(62942)=(S=13088312,E=13088351,C=29)
	CT(62943)=(S=13088352,E=13088355,C=78)
	CT(62944)=(S=13088356,E=13088371,C=29)
	CT(62945)=(S=13088372,E=13088375,C=78)
	CT(62946)=(S=13088376,E=13088379,C=29)
	CT(62947)=(S=13088384,E=13088391,C=78)
	CT(62948)=(S=13088392,E=13088447,C=29)
	CT(62949)=(S=13088448,E=13088451,C=179)
	CT(62950)=(S=13088452,E=13088471,C=29)
	CT(62951)=(S=13088472,E=13088479,C=78)
	CT(62952)=(S=13088480,E=13089024,C=29)
	CT(62953)=(S=13089025,E=13089120,C=78)
	CT(62954)=(S=13089121,E=13089121,C=29)
	CT(62955)=(S=13089122,E=13089153,C=78)
	CT(62956)=(S=13089154,E=13089155,C=29)
	CT(62957)=(S=13089156,E=13089162,C=78)
	CT(62958)=(S=13089163,E=13089163,C=14)
	CT(62959)=(S=13089164,E=13089197,C=78)
	CT(62960)=(S=13089198,E=13089199,C=29)
	CT(62961)=(S=13089200,E=13089205,C=78)
	CT(62962)=(S=13089206,E=13089206,C=29)
	CT(62963)=(S=13089207,E=13089207,C=78)
	CT(62964)=(S=13089208,E=13089209,C=29)
	CT(62965)=(S=13089210,E=13089231,C=78)
	CT(62966)=(S=13089232,E=13089232,C=29)
	CT(62967)=(S=13089233,E=13089279,C=78)
	CT(62968)=(S=13089280,E=13089727,C=29)
	CT(62969)=(S=13089728,E=13089731,C=78)
	CT(62970)=(S=13089732,E=13089759,C=29)
	CT(62971)=(S=13089764,E=13089767,C=78)
	CT(62972)=(S=13089768,E=13089823,C=29)
	CT(62973)=(S=13089824,E=13089835,C=78)
	CT(62974)=(S=13089836,E=13089851,C=29)
	CT(62975)=(S=13089852,E=13089855,C=78)
	CT(62976)=(S=13089856,E=13089859,C=29)
	CT(62977)=(S=13089860,E=13089863,C=78)
	CT(62978)=(S=13089864,E=13089903,C=29)
	CT(62979)=(S=13089904,E=13089907,C=78)
	CT(62980)=(S=13089908,E=13089963,C=29)
	CT(62981)=(S=13089964,E=13089967,C=78)
	CT(62982)=(S=13089968,E=13089979,C=29)
	CT(62983)=(S=13089980,E=13089983,C=78)
	CT(62984)=(S=13089984,E=13090011,C=29)
	CT(62985)=(S=13090012,E=13090015,C=78)
	CT(62986)=(S=13090016,E=13090073,C=29)
	CT(62987)=(S=13090074,E=13090075,C=78)
	CT(62988)=(S=13090076,E=13090123,C=29)
	CT(62989)=(S=13090124,E=13090127,C=78)
	CT(62990)=(S=13090128,E=13090135,C=29)
	CT(62991)=(S=13090136,E=13090143,C=78)
	CT(62992)=(S=13090144,E=13090159,C=29)
	CT(62993)=(S=13090160,E=13090163,C=183)
	CT(62994)=(S=13090164,E=13090179,C=29)
	CT(62995)=(S=13090180,E=13090183,C=78)
	CT(62996)=(S=13090184,E=13090247,C=29)
	CT(62997)=(S=13090248,E=13090251,C=78)
	CT(62998)=(S=13090252,E=13090283,C=29)
	CT(62999)=(S=13090284,E=13090287,C=78)
	CT(63000)=(S=13090288,E=13090609,C=29)
	CT(63001)=(S=13090610,E=13090611,C=95)
	CT(63002)=(S=13090612,E=13090919,C=29)
	CT(63003)=(S=13090920,E=13090923,C=78)
	CT(63004)=(S=13090924,E=13090991,C=29)
	CT(63005)=(S=13090992,E=13090995,C=78)
	CT(63006)=(S=13090996,E=13090999,C=29)
	CT(63007)=(S=13091000,E=13091007,C=78)
	CT(63008)=(S=13091008,E=13091027,C=29)
	CT(63009)=(S=13091028,E=13091031,C=78)
	CT(63010)=(S=13091032,E=13091039,C=29)
	CT(63011)=(S=13091040,E=13091043,C=82)
	CT(63012)=(S=13091044,E=13091047,C=29)
	CT(63013)=(S=13091048,E=13091055,C=78)
	CT(63014)=(S=13091056,E=13091119,C=29)
	CT(63015)=(S=13091120,E=13091127,C=78)
	CT(63016)=(S=13091128,E=13091159,C=29)
	CT(63017)=(S=13091160,E=13091167,C=78)
	CT(63018)=(S=13091168,E=13091239,C=29)
	CT(63019)=(S=13091240,E=13091247,C=78)
	CT(63020)=(S=13091248,E=13091299,C=29)
	CT(63021)=(S=13091300,E=13091303,C=178)
	CT(63022)=(S=13091304,E=13091697,C=29)
	CT(63023)=(S=13091698,E=13091699,C=78)
	CT(63024)=(S=13091700,E=13091731,C=29)
	CT(63025)=(S=13091732,E=13091735,C=78)
	CT(63026)=(S=13091736,E=13091739,C=29)
	CT(63027)=(S=13091740,E=13091751,C=78)
	CT(63028)=(S=13091752,E=13091787,C=29)
	CT(63029)=(S=13091788,E=13091791,C=78)
	CT(63030)=(S=13091792,E=13091803,C=29)
	CT(63031)=(S=13091804,E=13091807,C=175)
	CT(63032)=(S=13091808,E=13091811,C=29)
	CT(63033)=(S=13091812,E=13091815,C=78)
	CT(63034)=(S=13091816,E=13092351,C=29)
	CT(63035)=(S=13092352,E=13092561,C=78)
	CT(63036)=(S=13092562,E=13092563,C=29)
	CT(63037)=(S=13092564,E=13092564,C=78)
	CT(63038)=(S=13092565,E=13092567,C=29)
	CT(63039)=(S=13092568,E=13092571,C=78)
	CT(63040)=(S=13092572,E=13093203,C=29)
	CT(63041)=(S=13093204,E=13093207,C=174)
	CT(63042)=(S=13093208,E=13093239,C=29)
	CT(63043)=(S=13093240,E=13093247,C=78)
	CT(63044)=(S=13093248,E=13093341,C=29)
	CT(63045)=(S=13093342,E=13093342,C=78)
	CT(63046)=(S=13093343,E=13093375,C=29)
	CT(63047)=(S=13093376,E=13093407,C=78)
	CT(63048)=(S=13093408,E=13093439,C=29)
	CT(63049)=(S=13093440,E=13093491,C=78)
	CT(63050)=(S=13093492,E=13093511,C=29)
	CT(63051)=(S=13093512,E=13093591,C=78)
	CT(63052)=(S=13093592,E=13093599,C=29)
	CT(63053)=(S=13093600,E=13093607,C=78)
	CT(63054)=(S=13093608,E=13093887,C=29)
	CT(63055)=(S=13093888,E=13093891,C=78)
	CT(63056)=(S=13093892,E=13093931,C=29)
	CT(63057)=(S=13093932,E=13093943,C=78)
	CT(63058)=(S=13093944,E=13093951,C=29)
	CT(63059)=(S=13093952,E=13093967,C=78)
	CT(63060)=(S=13093968,E=13094007,C=29)
	CT(63061)=(S=13094008,E=13094011,C=78)
	CT(63062)=(S=13094012,E=13094015,C=29)
	CT(63063)=(S=13094016,E=13094019,C=78)
	CT(63064)=(S=13094020,E=13094095,C=29)
	CT(63065)=(S=13094096,E=13094103,C=78)
	CT(63066)=(S=13094104,E=13094119,C=29)
	CT(63067)=(S=13094120,E=13094127,C=95)
	CT(63068)=(S=13094128,E=13095936,C=29)
	CT(63069)=(S=13095937,E=13095949,C=78)
	CT(63070)=(S=13095950,E=13095950,C=29)
	CT(63071)=(S=13095951,E=13095991,C=78)
	CT(63072)=(S=13095992,E=13095992,C=29)
	CT(63073)=(S=13095994,E=13096021,C=78)
	CT(63074)=(S=13096022,E=13096023,C=29)
	CT(63075)=(S=13096024,E=13096030,C=78)
	CT(63076)=(S=13096031,E=13096031,C=29)
	CT(63077)=(S=13096032,E=13096059,C=78)
	CT(63078)=(S=13096060,E=13096060,C=29)
	CT(63079)=(S=13096061,E=13096153,C=78)
	CT(63080)=(S=13096154,E=13096159,C=29)
	CT(63081)=(S=13096160,E=13097215,C=78)
	CT(63082)=(S=13097216,E=13097335,C=29)
	CT(63083)=(S=13097336,E=13097339,C=78)
	CT(63084)=(S=13097340,E=13100507,C=29)
	CT(63085)=(S=13100508,E=13100511,C=78)
	CT(63086)=(S=13100512,E=13100999,C=29)
	CT(63087)=(S=13101000,E=13101007,C=78)
	CT(63088)=(S=13101008,E=13101019,C=29)
	CT(63089)=(S=13101020,E=13101023,C=78)
	CT(63090)=(S=13101024,E=13101337,C=29)
	CT(63091)=(S=13101338,E=13101339,C=78)
	CT(63092)=(S=13101340,E=13101406,C=29)
	CT(63093)=(S=13101407,E=13101407,C=78)
	CT(63094)=(S=13101408,E=13101482,C=29)
	CT(63095)=(S=13101483,E=13101483,C=78)
	CT(63096)=(S=13101484,E=13101494,C=29)
	CT(63097)=(S=13101495,E=13101496,C=78)
	CT(63098)=(S=13101497,E=13101823,C=29)
	CT(63099)=(S=13101824,E=13102079,C=78)
	CT(63100)=(S=13102080,E=13103487,C=29)
	CT(63101)=(S=13103488,E=13103495,C=78)
	CT(63102)=(S=13103496,E=13103519,C=29)
	CT(63103)=(S=13103520,E=13103527,C=78)
	CT(63104)=(S=13103528,E=13103871,C=29)
	CT(63105)=(S=13103872,E=13104127,C=78)
	CT(63106)=(S=13104128,E=13104203,C=29)
	CT(63107)=(S=13104204,E=13104207,C=78)
	CT(63108)=(S=13104208,E=13104247,C=29)
	CT(63109)=(S=13104248,E=13104251,C=78)
	CT(63110)=(S=13104252,E=13104325,C=29)
	CT(63111)=(S=13104326,E=13104327,C=78)
	CT(63112)=(S=13104328,E=13104572,C=29)
	CT(63113)=(S=13104573,E=13104573,C=78)
	CT(63114)=(S=13104574,E=13104640,C=29)
	CT(63115)=(S=13104641,E=13104716,C=78)
	CT(63116)=(S=13104717,E=13104727,C=29)
	CT(63117)=(S=13104728,E=13104745,C=78)
	CT(63118)=(S=13104746,E=13104746,C=29)
	CT(63119)=(S=13104747,E=13104778,C=78)
	CT(63120)=(S=13104779,E=13104779,C=29)
	CT(63121)=(S=13104780,E=13104894,C=78)
	CT(63122)=(S=13104895,E=13104895,C=29)
	CT(63123)=(S=13104896,E=13105151,C=78)
	CT(63124)=(S=13105152,E=13105295,C=29)
	CT(63125)=(S=13105296,E=13105296,C=83)
	CT(63126)=(S=13105297,E=13105359,C=29)
	CT(63127)=(S=13105360,E=13105360,C=78)
	CT(63128)=(S=13105361,E=13105517,C=29)
	CT(63129)=(S=13105518,E=13105535,C=78)
	CT(63130)=(S=13105536,E=13105671,C=29)
	CT(63131)=(S=13105672,E=13105677,C=78)
	CT(63132)=(S=13105678,E=13106441,C=29)
	CT(63133)=(S=13106442,E=13106443,C=78)
	CT(63134)=(S=13106444,E=13106487,C=29)
	CT(63135)=(S=13106488,E=13106494,C=78)
	CT(63136)=(S=13106495,E=13106681,C=29)
	CT(63137)=(S=13106682,E=13106683,C=78)
	CT(63138)=(S=13106684,E=13106715,C=29)
	CT(63139)=(S=13106716,E=13106719,C=78)
	CT(63140)=(S=13106720,E=13106735,C=29)
	CT(63141)=(S=13106736,E=13106751,C=78)
	CT(63142)=(S=13106752,E=13106791,C=29)
	CT(63143)=(S=13106792,E=13106799,C=79)
	CT(63144)=(S=13106800,E=13106839,C=29)
	CT(63145)=(S=13106840,E=13106840,C=78)
	CT(63146)=(S=13106841,E=13106922,C=29)
	CT(63147)=(S=13106923,E=13106923,C=78)
	CT(63148)=(S=13106924,E=13106971,C=29)
	CT(63149)=(S=13106972,E=13106975,C=78)
	CT(63150)=(S=13106976,E=13107059,C=29)
	CT(63151)=(S=13107060,E=13107063,C=80)
	CT(63152)=(S=13107064,E=13107159,C=29)
	CT(63153)=(S=13107160,E=13107167,C=78)
	CT(63154)=(S=13107168,E=13107199,C=29)
	CT(63155)=(S=13107200,E=13107207,C=181)
	CT(63156)=(S=13107208,E=13107215,C=189)
	CT(63157)=(S=13107216,E=13107216,C=219)
	CT(63158)=(S=13107217,E=13107217,C=83)
	CT(63159)=(S=13107218,E=13107218,C=218)
	CT(63160)=(S=13107219,E=13107219,C=83)
	CT(63161)=(S=13107220,E=13107221,C=207)
	CT(63162)=(S=13107222,E=13107223,C=211)
	CT(63163)=(S=13107224,E=13107227,C=219)
	CT(63164)=(S=13107228,E=13107231,C=213)
	CT(63165)=(S=13107232,E=13107259,C=189)
	CT(63166)=(S=13107260,E=13107261,C=208)
	CT(63167)=(S=13107262,E=13107262,C=189)
	CT(63168)=(S=13107263,E=13107263,C=213)
	CT(63169)=(S=13107265,E=13107265,C=200)
	CT(63170)=(S=13107266,E=13107266,C=203)
	CT(63171)=(S=13107267,E=13107267,C=189)
	CT(63172)=(S=13107268,E=13107269,C=205)
	CT(63173)=(S=13107270,E=13107272,C=189)
	CT(63174)=(S=13107273,E=13107278,C=213)
	CT(63175)=(S=13107279,E=13107280,C=83)
	CT(63176)=(S=13107281,E=13107281,C=189)
	CT(63177)=(S=13107282,E=13107282,C=203)
	CT(63178)=(S=13107283,E=13107284,C=204)
	CT(63179)=(S=13107285,E=13107285,C=189)
	CT(63180)=(S=13107286,E=13107288,C=205)
	CT(63181)=(S=13107289,E=13107289,C=189)
	CT(63182)=(S=13107290,E=13107293,C=204)
	CT(63183)=(S=13107294,E=13107294,C=29)
	CT(63184)=(S=13107295,E=13107295,C=204)
	CT(63185)=(S=13107296,E=13107299,C=83)
	CT(63186)=(S=13107300,E=13107302,C=189)
	CT(63187)=(S=13107303,E=13107306,C=204)
	CT(63188)=(S=13107307,E=13107307,C=212)
	CT(63189)=(S=13107308,E=13107308,C=203)
	CT(63190)=(S=13107309,E=13107309,C=204)
	CT(63191)=(S=13107310,E=13107310,C=212)
	CT(63192)=(S=13107311,E=13107313,C=204)
	CT(63193)=(S=13107314,E=13107314,C=189)
	CT(63194)=(S=13107315,E=13107317,C=203)
	CT(63195)=(S=13107318,E=13107319,C=212)
	CT(63196)=(S=13107320,E=13107324,C=203)
	CT(63197)=(S=13107325,E=13107333,C=204)
	CT(63198)=(S=13107334,E=13107335,C=189)
	CT(63199)=(S=13107336,E=13107336,C=212)
	CT(63200)=(S=13107337,E=13107338,C=29)
	CT(63201)=(S=13107339,E=13107339,C=200)
	CT(63202)=(S=13107340,E=13107341,C=203)
	CT(63203)=(S=13107342,E=13107342,C=212)
	CT(63204)=(S=13107343,E=13107343,C=204)
	CT(63205)=(S=13107344,E=13107363,C=203)
	CT(63206)=(S=13107364,E=13107373,C=212)
	CT(63207)=(S=13107374,E=13107374,C=203)
	CT(63208)=(S=13107375,E=13107375,C=83)
	CT(63209)=(S=13107376,E=13107377,C=215)
	CT(63210)=(S=13107378,E=13107378,C=209)
	CT(63211)=(S=13107379,E=13107379,C=202)
	CT(63212)=(S=13107380,E=13107380,C=210)
	CT(63213)=(S=13107381,E=13107386,C=83)
	CT(63214)=(S=13107387,E=13107387,C=181)
	CT(63215)=(S=13107388,E=13107400,C=83)
	CT(63216)=(S=13107401,E=13107401,C=181)
	CT(63217)=(S=13107404,E=13107407,C=205)
	CT(63218)=(S=13107408,E=13107415,C=83)
	CT(63219)=(S=13107416,E=13107423,C=29)
	CT(63220)=(S=13107424,E=13107455,C=83)
	CT(63221)=(S=13107456,E=13107461,C=200)
	CT(63222)=(S=13107462,E=13107462,C=213)
	CT(63223)=(S=13107463,E=13107463,C=83)
	CT(63224)=(S=13107464,E=13107471,C=200)
	CT(63225)=(S=13107472,E=13107487,C=203)
	CT(63226)=(S=13107488,E=13107519,C=83)
	CT(63227)=(S=13107520,E=13107559,C=181)
	CT(63228)=(S=13107560,E=13107567,C=176)
	CT(63229)=(S=13107568,E=13107568,C=189)
	CT(63230)=(S=13107569,E=13107569,C=83)
	CT(63231)=(S=13107571,E=13107571,C=203)
	CT(63232)=(S=13107572,E=13107574,C=83)
	CT(63233)=(S=13107575,E=13107575,C=200)
	CT(63234)=(S=13107576,E=13107579,C=203)
	CT(63235)=(S=13107580,E=13107583,C=181)
	CT(63236)=(S=13107584,E=13107599,C=203)
	CT(63237)=(S=13107600,E=13107607,C=200)
	CT(63238)=(S=13107608,E=13107608,C=216)
	CT(63239)=(S=13107609,E=13107609,C=215)
	CT(63240)=(S=13107610,E=13107611,C=177)
	CT(63241)=(S=13107612,E=13107615,C=224)
	CT(63242)=(S=13107616,E=13107616,C=178)
	CT(63243)=(S=13107617,E=13107617,C=213)
	CT(63244)=(S=13107618,E=13107627,C=200)
	CT(63245)=(S=13107628,E=13107628,C=213)
	CT(63246)=(S=13107629,E=13107629,C=181)
	CT(63247)=(S=13107630,E=13107630,C=203)
	CT(63248)=(S=13107631,E=13107631,C=181)
	CT(63249)=(S=13107632,E=13107639,C=212)
	CT(63250)=(S=13107640,E=13107647,C=200)
	CT(63251)=(S=13107648,E=13107655,C=181)
	CT(63252)=(S=13107656,E=13107663,C=206)
	CT(63253)=(S=13107664,E=13107671,C=224)
	CT(63254)=(S=13107672,E=13107679,C=189)
	CT(63255)=(S=13107680,E=13107711,C=83)
	CT(63256)=(S=13107712,E=13107719,C=203)
	CT(63257)=(S=13107720,E=13107727,C=200)
	CT(63258)=(S=13107728,E=13107743,C=203)
	CT(63259)=(S=13107744,E=13107775,C=205)
	CT(63260)=(S=13107776,E=13107783,C=181)
	CT(63261)=(S=13107784,E=13107811,C=203)
	CT(63262)=(S=13107812,E=13107815,C=189)
	CT(63263)=(S=13107816,E=13107831,C=203)
	CT(63264)=(S=13107832,E=13107839,C=83)
	CT(63265)=(S=13107840,E=13107871,C=218)
	CT(63266)=(S=13107872,E=13107903,C=224)
	CT(63267)=(S=13107904,E=13107967,C=203)
	CT(63268)=(S=13107968,E=13107977,C=200)
	CT(63269)=(S=13107980,E=13107983,C=205)
	CT(63270)=(S=13107984,E=13107999,C=189)
	CT(63271)=(S=13108000,E=13108081,C=83)
	CT(63272)=(S=13108082,E=13108082,C=212)
	CT(63273)=(S=13108083,E=13108083,C=83)
	CT(63274)=(S=13108084,E=13108087,C=200)
	CT(63275)=(S=13108088,E=13108095,C=83)
	CT(63276)=(S=13108096,E=13108113,C=181)
	CT(63277)=(S=13108114,E=13108114,C=215)
	CT(63278)=(S=13108115,E=13108135,C=181)
	CT(63279)=(S=13108136,E=13108143,C=83)
	CT(63280)=(S=13108144,E=13108151,C=176)
	CT(63281)=(S=13108152,E=13108159,C=83)
	CT(63282)=(S=13108160,E=13108161,C=181)
	CT(63283)=(S=13108162,E=13108162,C=208)
	CT(63284)=(S=13108163,E=13108163,C=83)
	CT(63285)=(S=13108168,E=13108175,C=209)
	CT(63286)=(S=13108176,E=13108183,C=203)
	CT(63287)=(S=13108184,E=13108191,C=83)
	CT(63288)=(S=13108192,E=13108208,C=208)
	CT(63289)=(S=13108209,E=13108210,C=212)
	CT(63290)=(S=13108211,E=13108211,C=83)
	CT(63291)=(S=13108212,E=13108215,C=181)
	CT(63292)=(S=13108216,E=13108223,C=206)
	CT(63293)=(S=13108224,E=13108239,C=204)
	CT(63294)=(S=13108240,E=13108255,C=181)
	CT(63295)=(S=13108256,E=13108282,C=204)
	CT(63296)=(S=13108283,E=13108283,C=83)
	CT(63297)=(S=13108284,E=13108291,C=204)
	CT(63298)=(S=13108292,E=13108293,C=83)
	CT(63299)=(S=13108294,E=13108383,C=204)
	CT(63300)=(S=13108384,E=13108415,C=218)
	CT(63301)=(S=13108416,E=13108479,C=212)
	CT(63302)=(S=13108480,E=13108488,C=80)
	CT(63303)=(S=13108489,E=13108489,C=189)
	CT(63304)=(S=13108490,E=13108490,C=83)
	CT(63305)=(S=13108491,E=13108491,C=80)
	CT(63306)=(S=13108492,E=13108495,C=219)
	CT(63307)=(S=13108496,E=13108511,C=200)
	CT(63308)=(S=13108512,E=13108519,C=189)
	CT(63309)=(S=13108520,E=13108527,C=200)
	CT(63310)=(S=13108528,E=13108543,C=202)
	CT(63311)=(S=13108544,E=13108607,C=83)
	CT(63312)=(S=13108608,E=13108671,C=200)
	CT(63313)=(S=13108672,E=13108735,C=83)
	CT(63314)=(S=13108736,E=13108753,C=213)
	CT(63315)=(S=13108754,E=13108754,C=203)
	CT(63316)=(S=13108755,E=13108762,C=213)
	CT(63317)=(S=13108763,E=13108763,C=200)
	CT(63318)=(S=13108764,E=13108768,C=213)
	CT(63319)=(S=13108769,E=13108769,C=203)
	CT(63320)=(S=13108770,E=13108770,C=213)
	CT(63321)=(S=13108771,E=13108784,C=189)
	CT(63322)=(S=13108788,E=13108788,C=209)
	CT(63323)=(S=13108789,E=13108790,C=203)
	CT(63324)=(S=13108791,E=13108791,C=216)
	CT(63325)=(S=13108792,E=13108799,C=207)
	CT(63326)=(S=13108800,E=13108814,C=203)
	CT(63327)=(S=13108815,E=13108815,C=209)
	CT(63328)=(S=13108832,E=13108863,C=203)
	CT(63329)=(S=13108864,E=13108869,C=189)
	CT(63330)=(S=13108870,E=13108871,C=202)
	CT(63331)=(S=13108872,E=13108879,C=189)
	CT(63332)=(S=13108880,E=13108887,C=217)
	CT(63333)=(S=13108888,E=13108895,C=209)
	CT(63334)=(S=13108896,E=13108927,C=181)
	CT(63335)=(S=13108928,E=13108991,C=215)
	CT(63336)=(S=13108992,E=13108995,C=189)
	CT(63337)=(S=13108996,E=13108999,C=203)
	CT(63338)=(S=13109000,E=13109005,C=189)
	CT(63339)=(S=13109006,E=13109006,C=206)
	CT(63340)=(S=13109007,E=13109007,C=83)
	CT(63341)=(S=13109008,E=13109023,C=203)
	CT(63342)=(S=13109024,E=13109055,C=211)
	CT(63343)=(S=13109056,E=13109075,C=213)
	CT(63344)=(S=13109076,E=13109079,C=205)
	CT(63345)=(S=13109080,E=13109087,C=176)
	CT(63346)=(S=13109088,E=13109103,C=209)
	CT(63347)=(S=13109104,E=13109119,C=189)
	CT(63348)=(S=13109120,E=13109139,C=83)
	CT(63349)=(S=13109140,E=13109143,C=224)
	CT(63350)=(S=13109144,E=13109151,C=83)
	CT(63351)=(S=13109152,E=13109167,C=201)
	CT(63352)=(S=13109168,E=13109175,C=83)
	CT(63353)=(S=13109176,E=13109183,C=207)
	CT(63354)=(S=13109184,E=13109247,C=213)
	CT(63355)=(S=13109248,E=13109503,C=200)
	CT(63356)=(S=13109504,E=13109506,C=189)
	CT(63357)=(S=13109507,E=13109507,C=203)
	CT(63358)=(S=13109508,E=13109511,C=206)
	CT(63359)=(S=13109512,E=13109519,C=204)
	CT(63360)=(S=13109536,E=13109567,C=202)
	CT(63361)=(S=13109568,E=13109568,C=203)
	CT(63362)=(S=13109569,E=13109575,C=189)
	CT(63363)=(S=13109576,E=13109576,C=181)
	CT(63364)=(S=13109577,E=13109577,C=203)
	CT(63365)=(S=13109578,E=13109578,C=215)
	CT(63366)=(S=13109579,E=13109579,C=83)
	CT(63367)=(S=13109580,E=13109582,C=189)
	CT(63368)=(S=13109583,E=13109585,C=203)
	CT(63369)=(S=13109586,E=13109587,C=29)
	CT(63370)=(S=13109588,E=13109595,C=189)
	CT(63371)=(S=13109598,E=13109598,C=181)
	CT(63372)=(S=13109599,E=13109599,C=189)
	CT(63373)=(S=13109600,E=13109600,C=213)
	CT(63374)=(S=13109601,E=13109605,C=203)
	CT(63375)=(S=13109608,E=13109611,C=189)
	CT(63376)=(S=13109612,E=13109618,C=203)
	CT(63377)=(S=13109619,E=13109619,C=175)
	CT(63378)=(S=13109620,E=13109623,C=202)
	CT(63379)=(S=13109626,E=13109626,C=203)
	CT(63380)=(S=13109627,E=13109631,C=189)
	CT(63381)=(S=13109632,E=13109632,C=203)
	CT(63382)=(S=13109633,E=13109643,C=189)
	CT(63383)=(S=13109645,E=13109646,C=203)
	CT(63384)=(S=13109647,E=13109648,C=189)
	CT(63385)=(S=13109649,E=13109650,C=203)
	CT(63386)=(S=13109651,E=13109651,C=83)
	CT(63387)=(S=13109653,E=13109653,C=189)
	CT(63388)=(S=13109654,E=13109657,C=203)
	CT(63389)=(S=13109661,E=13109661,C=83)
	CT(63390)=(S=13109664,E=13109668,C=189)
	CT(63391)=(S=13109669,E=13109672,C=201)
	CT(63392)=(S=13109674,E=13109677,C=204)
	CT(63393)=(S=13109679,E=13109679,C=189)
	CT(63394)=(S=13109680,E=13109680,C=213)
	CT(63395)=(S=13109681,E=13109684,C=203)
	CT(63396)=(S=13109686,E=13109689,C=204)
	CT(63397)=(S=13109691,E=13109696,C=216)
	CT(63398)=(S=13109697,E=13109697,C=203)
	CT(63399)=(S=13109698,E=13109698,C=83)
	CT(63400)=(S=13109699,E=13109702,C=203)
	CT(63401)=(S=13109703,E=13109703,C=189)
	CT(63402)=(S=13109705,E=13109709,C=203)
	CT(63403)=(S=13109712,E=13109712,C=83)
	CT(63404)=(S=13109713,E=13109715,C=202)
	CT(63405)=(S=13109716,E=13109717,C=83)
	CT(63406)=(S=13109719,E=13109719,C=203)
	CT(63407)=(S=13109720,E=13109722,C=176)
	CT(63408)=(S=13109723,E=13109723,C=83)
	CT(63409)=(S=13109724,E=13109728,C=189)
	CT(63410)=(S=13109729,E=13109729,C=203)
	CT(63411)=(S=13109731,E=13109731,C=83)
	CT(63412)=(S=13109732,E=13109737,C=203)
	CT(63413)=(S=13109739,E=13109741,C=83)
	CT(63414)=(S=13109742,E=13109745,C=203)
	CT(63415)=(S=13109746,E=13109750,C=83)
	CT(63416)=(S=13109751,E=13109751,C=203)
	CT(63417)=(S=13109752,E=13109752,C=213)
	CT(63418)=(S=13109757,E=13109758,C=83)
	CT(63419)=(S=13109759,E=13109759,C=215)
	CT(63420)=(S=13109760,E=13109763,C=203)
	CT(63421)=(S=13109764,E=13109767,C=189)
	CT(63422)=(S=13109768,E=13109774,C=203)
	CT(63423)=(S=13109775,E=13109775,C=209)
	CT(63424)=(S=13109776,E=13109787,C=203)
	CT(63425)=(S=13109788,E=13109791,C=210)
	CT(63426)=(S=13109792,E=13109807,C=208)
	CT(63427)=(S=13109808,E=13109819,C=189)
	CT(63428)=(S=13109820,E=13109823,C=205)
	CT(63429)=(S=13109824,E=13109855,C=212)
	CT(63430)=(S=13109856,E=13109889,C=83)
	CT(63431)=(S=13109890,E=13109891,C=212)
	CT(63432)=(S=13109892,E=13109895,C=189)
	CT(63433)=(S=13109899,E=13109900,C=83)
	CT(63434)=(S=13109902,E=13109902,C=177)
	CT(63435)=(S=13109903,E=13109903,C=204)
	CT(63436)=(S=13109905,E=13109905,C=202)
	CT(63437)=(S=13109907,E=13109911,C=213)
	CT(63438)=(S=13109912,E=13109912,C=175)
	CT(63439)=(S=13109915,E=13109915,C=29)
	CT(63440)=(S=13109919,E=13109919,C=189)
	CT(63441)=(S=13109920,E=13109920,C=29)
	CT(63442)=(S=13109921,E=13109921,C=83)
	CT(63443)=(S=13109922,E=13109922,C=177)
	CT(63444)=(S=13109925,E=13109926,C=178)
	CT(63445)=(S=13109927,E=13109932,C=203)
	CT(63446)=(S=13109938,E=13109939,C=189)
	CT(63447)=(S=13109942,E=13109944,C=203)
	CT(63448)=(S=13109946,E=13109946,C=83)
	CT(63449)=(S=13109948,E=13109948,C=212)
	CT(63450)=(S=13109950,E=13109950,C=203)
	CT(63451)=(S=13109954,E=13109954,C=170)
	CT(63452)=(S=13109955,E=13109955,C=203)
	CT(63453)=(S=13109956,E=13109956,C=83)
	CT(63454)=(S=13109957,E=13109958,C=204)
	CT(63455)=(S=13109959,E=13109959,C=83)
	CT(63456)=(S=13109960,E=13109960,C=204)
	CT(63457)=(S=13109961,E=13109962,C=83)
	CT(63458)=(S=13109963,E=13109963,C=209)
	CT(63459)=(S=13109964,E=13109964,C=83)
	CT(63460)=(S=13109965,E=13109965,C=216)
	CT(63461)=(S=13109967,E=13109967,C=83)
	CT(63462)=(S=13109968,E=13109971,C=203)
	CT(63463)=(S=13109972,E=13109978,C=204)
	CT(63464)=(S=13109979,E=13109979,C=83)
	CT(63465)=(S=13109980,E=13109986,C=203)
	CT(63466)=(S=13109988,E=13109991,C=206)
	CT(63467)=(S=13109992,E=13109992,C=203)
	CT(63468)=(S=13109993,E=13109993,C=204)
	CT(63469)=(S=13109994,E=13109998,C=203)
	CT(63470)=(S=13110000,E=13110000,C=204)
	CT(63471)=(S=13110001,E=13110001,C=215)
	CT(63472)=(S=13110002,E=13110002,C=203)
	CT(63473)=(S=13110003,E=13110009,C=204)
	CT(63474)=(S=13110010,E=13110010,C=212)
	CT(63475)=(S=13110011,E=13110013,C=203)
	CT(63476)=(S=13110014,E=13110014,C=200)
	CT(63477)=(S=13110015,E=13110015,C=203)
	CT(63478)=(S=13110016,E=13110044,C=189)
	CT(63479)=(S=13110045,E=13110045,C=83)
	CT(63480)=(S=13110046,E=13110046,C=216)
	CT(63481)=(S=13110047,E=13110047,C=210)
	CT(63482)=(S=13110048,E=13110055,C=212)
	CT(63483)=(S=13110056,E=13110063,C=181)
	CT(63484)=(S=13110064,E=13110079,C=212)
	CT(63485)=(S=13110080,E=13110119,C=203)
	CT(63486)=(S=13110128,E=13110135,C=83)
	CT(63487)=(S=13110144,E=13110271,C=200)
	CT(63488)=(S=13110272,E=13110287,C=189)
	CT(63489)=(S=13110288,E=13110303,C=203)
	CT(63490)=(S=13110304,E=13110335,C=215)
	CT(63491)=(S=13110336,E=13110400,C=204)
	CT(63492)=(S=13110401,E=13110404,C=203)
	CT(63493)=(S=13110405,E=13110410,C=83)
	CT(63494)=(S=13110412,E=13110417,C=203)
	CT(63495)=(S=13110418,E=13110418,C=206)
	CT(63496)=(S=13110419,E=13110424,C=203)
	CT(63497)=(S=13110425,E=13110425,C=209)
	CT(63498)=(S=13110426,E=13110426,C=83)
	CT(63499)=(S=13110427,E=13110428,C=205)
	CT(63500)=(S=13110430,E=13110438,C=204)
	CT(63501)=(S=13110439,E=13110439,C=202)
	CT(63502)=(S=13110440,E=13110440,C=29)
	CT(63503)=(S=13110441,E=13110441,C=213)
	CT(63504)=(S=13110442,E=13110442,C=181)
	CT(63505)=(S=13110443,E=13110443,C=203)
	CT(63506)=(S=13110444,E=13110446,C=210)
	CT(63507)=(S=13110447,E=13110463,C=181)
	CT(63508)=(S=13110464,E=13110464,C=83)
	CT(63509)=(S=13110465,E=13110465,C=200)
	CT(63510)=(S=13110466,E=13110467,C=203)
	CT(63511)=(S=13110468,E=13110471,C=213)
	CT(63512)=(S=13110472,E=13110479,C=83)
	CT(63513)=(S=13110480,E=13110487,C=209)
	CT(63514)=(S=13110488,E=13110495,C=83)
	CT(63515)=(S=13110496,E=13110511,C=29)
	CT(63516)=(S=13110512,E=13110519,C=176)
	CT(63517)=(S=13110520,E=13110527,C=201)
	CT(63518)=(S=13110528,E=13110535,C=203)
	CT(63519)=(S=13110536,E=13110543,C=189)
	CT(63520)=(S=13110544,E=13110568,C=204)
	CT(63521)=(S=13110569,E=13110575,C=83)
	CT(63522)=(S=13110576,E=13110610,C=204)
	CT(63523)=(S=13110611,E=13110611,C=83)
	CT(63524)=(S=13110616,E=13110659,C=204)
	CT(63525)=(S=13110660,E=13110663,C=208)
	CT(63526)=(S=13110664,E=13110671,C=222)
	CT(63527)=(S=13110672,E=13110679,C=219)
	CT(63528)=(S=13110680,E=13110687,C=201)
	CT(63529)=(S=13110688,E=13110719,C=210)
	CT(63530)=(S=13110720,E=13110783,C=181)
	CT(63531)=(S=13110784,E=13110791,C=203)
	CT(63532)=(S=13110792,E=13110799,C=200)
	CT(63533)=(S=13110800,E=13110815,C=204)
	CT(63534)=(S=13110816,E=13110817,C=189)
	CT(63535)=(S=13110818,E=13110818,C=213)
	CT(63536)=(S=13110819,E=13110819,C=189)
	CT(63537)=(S=13110821,E=13110823,C=83)
	CT(63538)=(S=13110824,E=13110831,C=181)
	CT(63539)=(S=13110832,E=13110839,C=219)
	CT(63540)=(S=13110848,E=13110857,C=203)
	CT(63541)=(S=13110858,E=13110859,C=83)
	CT(63542)=(S=13110860,E=13110862,C=203)
	CT(63543)=(S=13110863,E=13110863,C=177)
	CT(63544)=(S=13110864,E=13110866,C=203)
	CT(63545)=(S=13110867,E=13110867,C=213)
	CT(63546)=(S=13110868,E=13110895,C=203)
	CT(63547)=(S=13110896,E=13110897,C=181)
	CT(63548)=(S=13110898,E=13110899,C=203)
	CT(63549)=(S=13110900,E=13110900,C=83)
	CT(63550)=(S=13110901,E=13110902,C=203)
	CT(63551)=(S=13110903,E=13110903,C=209)
	CT(63552)=(S=13110912,E=13111039,C=203)
	CT(63553)=(S=13111040,E=13111295,C=29)
	CT(63554)=(S=13111296,E=13111303,C=212)
	CT(63555)=(S=13111304,E=13111311,C=105)
	CT(63556)=(S=13111312,E=13111327,C=83)
	CT(63557)=(S=13111328,E=13111359,C=204)
	CT(63558)=(S=13111360,E=13111361,C=83)
	CT(63559)=(S=13111362,E=13111363,C=202)
	CT(63560)=(S=13111364,E=13111367,C=200)
	CT(63561)=(S=13111368,E=13111374,C=83)
	CT(63562)=(S=13111375,E=13111375,C=181)
	CT(63563)=(S=13111376,E=13111383,C=83)
	CT(63564)=(S=13111384,E=13111384,C=29)
	CT(63565)=(S=13111385,E=13111385,C=83)
	CT(63566)=(S=13111386,E=13111387,C=177)
	CT(63567)=(S=13111388,E=13111389,C=207)
	CT(63568)=(S=13111390,E=13111390,C=213)
	CT(63569)=(S=13111391,E=13111391,C=200)
	CT(63570)=(S=13111392,E=13111407,C=83)
	CT(63571)=(S=13111408,E=13111411,C=203)
	CT(63572)=(S=13111412,E=13111412,C=83)
	CT(63573)=(S=13111413,E=13111415,C=181)
	CT(63574)=(S=13111416,E=13111551,C=83)
	CT(63575)=(S=13111552,E=13112575,C=189)
	CT(63576)=(S=13112576,E=13112831,C=181)
	CT(63577)=(S=13112832,E=13113087,C=203)
	CT(63578)=(S=13113088,E=13113155,C=204)
	CT(63579)=(S=13113156,E=13113159,C=189)
	CT(63580)=(S=13113160,E=13113267,C=204)
	CT(63581)=(S=13113268,E=13113271,C=189)
	CT(63582)=(S=13113272,E=13113311,C=204)
	CT(63583)=(S=13113312,E=13113327,C=189)
	CT(63584)=(S=13113328,E=13113341,C=204)
	CT(63585)=(S=13113344,E=13113353,C=181)
	CT(63586)=(S=13113354,E=13113355,C=189)
	CT(63587)=(S=13113356,E=13113359,C=203)
	CT(63588)=(S=13113360,E=13113407,C=181)
	CT(63589)=(S=13113408,E=13113439,C=189)
	CT(63590)=(S=13113440,E=13113471,C=181)
	CT(63591)=(S=13113472,E=13113503,C=213)
	CT(63592)=(S=13113504,E=13113535,C=212)
	CT(63593)=(S=13113536,E=13113567,C=213)
	CT(63594)=(S=13113568,E=13113583,C=203)
	CT(63595)=(S=13113584,E=13113599,C=83)
	CT(63596)=(S=13113600,E=13113727,C=181)
	CT(63597)=(S=13113728,E=13113823,C=213)
	CT(63598)=(S=13113824,E=13113855,C=181)
	CT(63599)=(S=13113856,E=13113983,C=83)
	CT(63600)=(S=13113984,E=13114015,C=181)
	CT(63601)=(S=13114016,E=13114023,C=200)
	CT(63602)=(S=13114024,E=13114031,C=177)
	CT(63603)=(S=13114032,E=13114039,C=206)
	CT(63604)=(S=13114040,E=13114047,C=83)
	CT(63605)=(S=13114048,E=13114079,C=207)
	CT(63606)=(S=13114112,E=13114719,C=203)
	CT(63607)=(S=13114720,E=13114751,C=181)
	CT(63608)=(S=13114752,E=13114855,C=203)
	CT(63609)=(S=13114856,E=13114863,C=181)
	CT(63610)=(S=13114864,E=13114871,C=213)
	CT(63611)=(S=13114872,E=13114879,C=83)
	CT(63612)=(S=13114880,E=13114943,C=29)
	CT(63613)=(S=13114944,E=13115007,C=181)
	CT(63614)=(S=13115008,E=13115071,C=215)
	CT(63615)=(S=13115072,E=13115135,C=203)
	CT(63616)=(S=13115136,E=13115167,C=83)
	CT(63617)=(S=13115168,E=13115199,C=203)
	CT(63618)=(S=13115200,E=13115231,C=181)
	CT(63619)=(S=13115232,E=13115263,C=212)
	CT(63620)=(S=13115264,E=13115295,C=200)
	CT(63621)=(S=13115296,E=13115327,C=210)
	CT(63622)=(S=13115328,E=13115359,C=181)
	CT(63623)=(S=13115360,E=13115551,C=83)
	CT(63624)=(S=13115552,E=13115583,C=203)
	CT(63625)=(S=13115584,E=13115647,C=170)
	CT(63626)=(S=13115648,E=13115651,C=204)
	CT(63627)=(S=13115652,E=13115652,C=83)
	CT(63628)=(S=13115653,E=13115675,C=204)
	CT(63629)=(S=13115676,E=13115676,C=83)
	CT(63630)=(S=13115677,E=13115714,C=204)
	CT(63631)=(S=13115715,E=13115715,C=83)
	CT(63632)=(S=13115716,E=13115735,C=204)
	CT(63633)=(S=13115736,E=13115743,C=189)
	CT(63634)=(S=13115745,E=13115772,C=204)
	CT(63635)=(S=13115776,E=13115783,C=189)
	CT(63636)=(S=13115784,E=13115799,C=204)
	CT(63637)=(S=13115800,E=13115807,C=189)
	CT(63638)=(S=13115808,E=13115811,C=204)
	CT(63639)=(S=13115812,E=13115815,C=189)
	CT(63640)=(S=13115816,E=13115903,C=204)
	CT(63641)=(S=13115904,E=13115904,C=181)
	CT(63642)=(S=13115905,E=13116063,C=204)
	CT(63643)=(S=13116064,E=13116067,C=189)
	CT(63644)=(S=13116068,E=13116074,C=204)
	CT(63645)=(S=13116075,E=13116075,C=181)
	CT(63646)=(S=13116076,E=13116118,C=204)
	CT(63647)=(S=13116119,E=13116119,C=83)
	CT(63648)=(S=13116120,E=13116123,C=189)
	CT(63649)=(S=13116124,E=13116127,C=204)
	CT(63650)=(S=13116128,E=13116159,C=189)
	CT(63651)=(S=13116160,E=13116191,C=200)
	CT(63652)=(S=13116192,E=13116223,C=181)
	CT(63653)=(S=13116224,E=13116303,C=200)
	CT(63654)=(S=13116304,E=13116311,C=209)
	CT(63655)=(S=13116312,E=13116315,C=205)
	CT(63656)=(S=13116316,E=13116319,C=203)
	CT(63657)=(S=13116320,E=13116351,C=215)
	CT(63658)=(S=13116352,E=13116415,C=200)
	CT(63659)=(S=13116416,E=13116671,C=204)
	CT(63660)=(S=13116672,E=13116927,C=212)
	CT(63661)=(S=13116928,E=13117247,C=204)
	CT(63662)=(S=13117248,E=13117279,C=189)
	CT(63663)=(S=13117280,E=13117439,C=204)
	CT(63664)=(S=13117440,E=13117695,C=205)
	CT(63665)=(S=13117696,E=13118111,C=83)
	CT(63666)=(S=13118112,E=13118143,C=203)
	CT(63667)=(S=13118144,E=13118207,C=177)
	CT(63668)=(S=13118208,E=13118463,C=83)
	CT(63669)=(S=13118464,E=13118719,C=200)
	CT(63670)=(S=13118720,E=13118975,C=83)
	CT(63671)=(S=13118976,E=13119231,C=209)
	CT(63672)=(S=13119232,E=13119403,C=200)
	CT(63673)=(S=13119404,E=13119407,C=181)
	CT(63674)=(S=13119408,E=13119447,C=200)
	CT(63675)=(S=13119448,E=13119451,C=181)
	CT(63676)=(S=13119452,E=13119487,C=200)
	CT(63677)=(S=13119488,E=13119743,C=212)
	CT(63678)=(S=13119744,E=13119759,C=83)
	CT(63679)=(S=13119760,E=13119767,C=203)
	CT(63680)=(S=13119768,E=13119775,C=83)
	CT(63681)=(S=13119776,E=13119807,C=189)
	CT(63682)=(S=13119808,E=13119903,C=83)
	CT(63683)=(S=13119904,E=13119935,C=215)
	CT(63684)=(S=13119936,E=13119983,C=83)
	CT(63685)=(S=13119984,E=13119991,C=213)
	CT(63686)=(S=13119992,E=13119999,C=29)
	CT(63687)=(S=13120000,E=13120031,C=80)
	CT(63688)=(S=13120032,E=13120063,C=203)
	CT(63689)=(S=13120064,E=13120095,C=79)
	CT(63690)=(S=13120096,E=13120127,C=203)
	CT(63691)=(S=13120128,E=13120143,C=79)
	CT(63692)=(S=13120144,E=13120159,C=203)
	CT(63693)=(S=13120160,E=13120191,C=83)
	CT(63694)=(S=13120224,E=13120231,C=202)
	CT(63695)=(S=13120232,E=13120239,C=213)
	CT(63696)=(S=13120240,E=13120511,C=83)
	CT(63697)=(S=13120512,E=13120527,C=204)
	CT(63698)=(S=13120528,E=13120543,C=189)
	CT(63699)=(S=13120544,E=13120655,C=204)
	CT(63700)=(S=13120656,E=13120671,C=208)
	CT(63701)=(S=13120672,E=13120767,C=204)
	CT(63702)=(S=13120776,E=13120799,C=189)
	CT(63703)=(S=13120800,E=13120959,C=204)
	CT(63704)=(S=13120960,E=13120991,C=189)
	CT(63705)=(S=13120992,E=13121023,C=204)
	CT(63706)=(S=13121024,E=13121279,C=203)
	CT(63707)=(S=13121280,E=13121407,C=83)
	CT(63708)=(S=13121408,E=13121471,C=219)
	CT(63709)=(S=13121472,E=13121503,C=203)
	CT(63710)=(S=13121504,E=13121519,C=213)
	CT(63711)=(S=13121520,E=13121527,C=83)
	CT(63712)=(S=13121528,E=13121535,C=213)
	CT(63713)=(S=13121536,E=13122047,C=204)
	CT(63714)=(S=13122048,E=13122111,C=29)
	CT(63715)=(S=13122112,E=13122143,C=201)
	CT(63716)=(S=13122144,E=13122175,C=83)
	CT(63717)=(S=13122176,E=13122207,C=205)
	CT(63718)=(S=13122208,E=13122239,C=201)
	CT(63719)=(S=13122240,E=13122287,C=181)
	CT(63720)=(S=13122288,E=13122295,C=177)
	CT(63721)=(S=13122304,E=13122319,C=83)
	CT(63722)=(S=13122320,E=13122327,C=202)
	CT(63723)=(S=13122328,E=13122335,C=208)
	CT(63724)=(S=13122336,E=13122487,C=83)
	CT(63725)=(S=13122488,E=13122495,C=200)
	CT(63726)=(S=13122496,E=13122559,C=83)
	CT(63727)=(S=13122560,E=13122815,C=212)
	CT(63728)=(S=13122816,E=13122943,C=83)
	CT(63729)=(S=13122944,E=13122975,C=181)
	CT(63730)=(S=13122976,E=13123039,C=83)
	CT(63731)=(S=13123040,E=13123055,C=206)
	CT(63732)=(S=13123056,E=13123067,C=83)
	CT(63733)=(S=13123068,E=13123068,C=200)
	CT(63734)=(S=13123069,E=13123069,C=207)
	CT(63735)=(S=13123070,E=13123071,C=83)
	CT(63736)=(S=13123072,E=13123135,C=29)
	CT(63737)=(S=13123136,E=13123199,C=216)
	CT(63738)=(S=13123200,E=13123327,C=212)
	CT(63739)=(S=13123328,E=13123367,C=83)
	CT(63740)=(S=13123368,E=13123375,C=209)
	CT(63741)=(S=13123376,E=13123383,C=83)
	CT(63742)=(S=13123384,E=13123391,C=203)
	CT(63743)=(S=13123392,E=13123423,C=83)
	CT(63744)=(S=13123424,E=13123431,C=203)
	CT(63745)=(S=13123432,E=13123435,C=213)
	CT(63746)=(S=13123436,E=13123519,C=83)
	CT(63747)=(S=13123520,E=13123583,C=213)
	CT(63748)=(S=13123584,E=13124207,C=204)
	CT(63749)=(S=13124208,E=13124223,C=189)
	CT(63750)=(S=13124224,E=13124607,C=204)
	CT(63751)=(S=13124608,E=13124671,C=203)
	CT(63752)=(S=13124672,E=13124735,C=83)
	CT(63753)=(S=13124736,E=13124799,C=204)
	CT(63754)=(S=13124800,E=13124959,C=83)
	CT(63755)=(S=13124960,E=13124991,C=181)
	CT(63756)=(S=13124992,E=13125023,C=83)
	CT(63757)=(S=13125024,E=13125055,C=213)
	CT(63758)=(S=13125056,E=13125375,C=83)
	CT(63759)=(S=13125376,E=13125407,C=205)
	CT(63760)=(S=13125408,E=13125455,C=181)
	CT(63761)=(S=13125456,E=13125463,C=201)
	CT(63762)=(S=13125464,E=13125471,C=83)
	CT(63763)=(S=13125488,E=13125491,C=200)
	CT(63764)=(S=13125492,E=13125499,C=189)
	CT(63765)=(S=13125500,E=13125503,C=208)
	CT(63766)=(S=13125504,E=13125567,C=200)
	CT(63767)=(S=13125568,E=13125599,C=203)
	CT(63768)=(S=13125600,E=13125615,C=83)
	CT(63769)=(S=13125616,E=13125623,C=200)
	CT(63770)=(S=13125624,E=13125631,C=217)
	CT(63771)=(S=13125632,E=13125887,C=203)
	CT(63772)=(S=13125888,E=13125983,C=181)
	CT(63773)=(S=13125984,E=13125991,C=201)
	CT(63774)=(S=13125992,E=13125999,C=210)
	CT(63775)=(S=13126000,E=13126015,C=203)
	CT(63776)=(S=13126016,E=13126079,C=83)
	CT(63777)=(S=13126080,E=13126087,C=200)
	CT(63778)=(S=13126088,E=13126095,C=213)
	CT(63779)=(S=13126096,E=13126271,C=203)
	CT(63780)=(S=13126272,E=13126303,C=181)
	CT(63781)=(S=13126304,E=13126335,C=203)
	CT(63782)=(S=13126336,E=13126383,C=200)
	CT(63783)=(S=13126384,E=13126391,C=209)
	CT(63784)=(S=13126392,E=13126399,C=202)
	CT(63785)=(S=13126400,E=13126431,C=203)
	CT(63786)=(S=13126432,E=13126495,C=181)
	CT(63787)=(S=13126496,E=13126559,C=200)
	CT(63788)=(S=13126560,E=13126575,C=201)
	CT(63789)=(S=13126576,E=13126583,C=189)
	CT(63790)=(S=13126584,E=13126591,C=208)
	CT(63791)=(S=13126592,E=13126655,C=209)
	CT(63792)=(S=13126656,E=13127087,C=204)
	CT(63793)=(S=13127088,E=13127095,C=189)
	CT(63794)=(S=13127096,E=13127099,C=203)
	CT(63795)=(S=13127100,E=13127103,C=213)
	CT(63796)=(S=13127104,E=13127607,C=204)
	CT(63797)=(S=13127608,E=13127615,C=189)
	CT(63798)=(S=13127616,E=13127679,C=204)
	CT(63799)=(S=13127680,E=13127743,C=181)
	CT(63800)=(S=13127744,E=13127775,C=83)
	CT(63801)=(S=13127776,E=13127807,C=218)
	CT(63802)=(S=13127808,E=13127983,C=83)
	CT(63803)=(S=13127992,E=13127999,C=181)
	CT(63804)=(S=13128000,E=13128079,C=83)
	CT(63805)=(S=13128080,E=13128087,C=203)
	CT(63806)=(S=13128088,E=13128319,C=83)
	CT(63807)=(S=13128320,E=13128447,C=200)
	CT(63808)=(S=13128448,E=13128703,C=203)
	CT(63809)=(S=13128704,E=13128959,C=200)
	CT(63810)=(S=13128960,E=13128991,C=210)
	CT(63811)=(S=13128992,E=13129023,C=206)
	CT(63812)=(S=13129024,E=13129039,C=200)
	CT(63813)=(S=13129040,E=13129047,C=213)
	CT(63814)=(S=13129048,E=13129087,C=83)
	CT(63815)=(S=13129088,E=13129111,C=201)
	CT(63816)=(S=13129112,E=13129119,C=83)
	CT(63817)=(S=13129120,E=13129135,C=216)
	CT(63818)=(S=13129136,E=13129151,C=83)
	CT(63819)=(S=13129152,E=13129183,C=203)
	CT(63820)=(S=13129184,E=13129215,C=181)
	CT(63821)=(S=13129216,E=13129471,C=203)
	CT(63822)=(S=13129472,E=13129727,C=201)
	CT(63823)=(S=13129728,E=13129983,C=177)
	CT(63824)=(S=13129984,E=13130015,C=212)
	CT(63825)=(S=13130016,E=13130063,C=203)
	CT(63826)=(S=13130064,E=13130071,C=210)
	CT(63827)=(S=13130072,E=13130079,C=177)
	CT(63828)=(S=13130080,E=13130111,C=181)
	CT(63829)=(S=13130112,E=13130175,C=83)
	CT(63830)=(S=13130176,E=13130239,C=181)
	CT(63831)=(S=13130240,E=13130367,C=200)
	CT(63832)=(S=13130368,E=13130383,C=209)
	CT(63833)=(S=13130384,E=13130391,C=201)
	CT(63834)=(S=13130392,E=13130399,C=213)
	CT(63835)=(S=13130400,E=13130527,C=203)
	CT(63836)=(S=13130528,E=13130535,C=83)
	CT(63837)=(S=13130536,E=13130543,C=203)
	CT(63838)=(S=13130544,E=13130559,C=83)
	CT(63839)=(S=13130560,E=13130687,C=202)
	CT(63840)=(S=13130688,E=13130751,C=181)
	CT(63841)=(S=13130752,E=13131007,C=204)
	CT(63842)=(S=13131008,E=13131135,C=200)
	CT(63843)=(S=13131136,E=13131199,C=181)
	CT(63844)=(S=13131200,E=13131247,C=213)
	CT(63845)=(S=13131248,E=13131255,C=205)
	CT(63846)=(S=13131256,E=13131263,C=213)
	CT(63847)=(S=13131264,E=13131503,C=204)
	CT(63848)=(S=13131504,E=13131511,C=189)
	CT(63849)=(S=13131512,E=13131515,C=215)
	CT(63850)=(S=13131516,E=13131519,C=189)
	CT(63851)=(S=13131520,E=13131691,C=204)
	CT(63852)=(S=13131692,E=13131703,C=189)
	CT(63853)=(S=13131704,E=13131707,C=203)
	CT(63854)=(S=13131708,E=13131743,C=189)
	CT(63855)=(S=13131744,E=13131775,C=204)
	CT(63856)=(S=13131776,E=13133823,C=189)
	CT(63857)=(S=13133824,E=13134079,C=203)
	CT(63858)=(S=13134080,E=13134175,C=83)
	CT(63859)=(S=13134176,E=13134183,C=202)
	CT(63860)=(S=13134184,E=13134191,C=83)
	CT(63861)=(S=13134192,E=13134199,C=213)
	CT(63862)=(S=13134200,E=13134207,C=83)
	CT(63863)=(S=13134208,E=13134303,C=201)
	CT(63864)=(S=13134304,E=13134335,C=213)
	CT(63865)=(S=13134336,E=13134463,C=212)
	CT(63866)=(S=13134480,E=13134487,C=209)
	CT(63867)=(S=13134488,E=13134495,C=203)
	CT(63868)=(S=13134496,E=13134559,C=181)
	CT(63869)=(S=13134560,E=13134575,C=210)
	CT(63870)=(S=13134576,E=13134579,C=83)
	CT(63871)=(S=13134580,E=13134581,C=201)
	CT(63872)=(S=13134582,E=13134582,C=208)
	CT(63873)=(S=13134583,E=13134583,C=216)
	CT(63874)=(S=13134584,E=13134591,C=83)
	CT(63875)=(S=13134592,E=13134655,C=213)
	CT(63876)=(S=13134656,E=13134671,C=203)
	CT(63877)=(S=13134672,E=13134672,C=208)
	CT(63878)=(S=13134673,E=13134673,C=83)
	CT(63879)=(S=13134674,E=13134675,C=202)
	CT(63880)=(S=13134676,E=13134679,C=217)
	CT(63881)=(S=13134680,E=13134703,C=83)
	CT(63882)=(S=13134712,E=13134719,C=208)
	CT(63883)=(S=13134720,E=13134783,C=212)
	CT(63884)=(S=13134784,E=13134791,C=200)
	CT(63885)=(S=13134792,E=13134799,C=83)
	CT(63886)=(S=13134800,E=13134807,C=208)
	CT(63887)=(S=13134816,E=13134823,C=83)
	CT(63888)=(S=13134824,E=13134831,C=208)
	CT(63889)=(S=13134832,E=13134839,C=201)
	CT(63890)=(S=13134840,E=13134847,C=213)
	CT(63891)=(S=13134848,E=13134879,C=176)
	CT(63892)=(S=13134880,E=13134911,C=209)
	CT(63893)=(S=13134912,E=13134943,C=83)
	CT(63894)=(S=13134944,E=13134959,C=212)
	CT(63895)=(S=13134960,E=13134975,C=83)
	CT(63896)=(S=13134976,E=13134991,C=206)
	CT(63897)=(S=13134992,E=13135007,C=83)
	CT(63898)=(S=13135040,E=13135103,C=205)
	CT(63899)=(S=13135104,E=13135359,C=200)
	CT(63900)=(S=13135360,E=13135407,C=212)
	CT(63901)=(S=13135408,E=13135423,C=201)
	CT(63902)=(S=13135424,E=13135487,C=213)
	CT(63903)=(S=13135488,E=13135519,C=181)
	CT(63904)=(S=13135520,E=13135527,C=83)
	CT(63905)=(S=13135528,E=13135535,C=181)
	CT(63906)=(S=13135536,E=13135591,C=83)
	CT(63907)=(S=13135592,E=13135599,C=213)
	CT(63908)=(S=13135600,E=13135615,C=200)
	CT(63909)=(S=13135616,E=13135999,C=203)
	CT(63910)=(S=13136000,E=13136063,C=83)
	CT(63911)=(S=13136064,E=13136087,C=181)
	CT(63912)=(S=13136088,E=13136095,C=213)
	CT(63913)=(S=13136096,E=13136319,C=203)
	CT(63914)=(S=13136320,E=13136383,C=218)
	CT(63915)=(S=13136384,E=13136447,C=181)
	CT(63916)=(S=13136448,E=13136463,C=203)
	CT(63917)=(S=13136464,E=13136471,C=83)
	CT(63918)=(S=13136472,E=13136479,C=203)
	CT(63919)=(S=13136480,E=13136495,C=83)
	CT(63920)=(S=13136496,E=13136503,C=215)
	CT(63921)=(S=13136504,E=13136655,C=83)
	CT(63922)=(S=13136656,E=13136657,C=206)
	CT(63923)=(S=13136658,E=13136658,C=202)
	CT(63924)=(S=13136659,E=13136659,C=209)
	CT(63925)=(S=13136660,E=13136663,C=212)
	CT(63926)=(S=13136664,E=13136671,C=83)
	CT(63927)=(S=13136672,E=13136687,C=213)
	CT(63928)=(S=13136688,E=13136703,C=83)
	CT(63929)=(S=13136704,E=13136719,C=205)
	CT(63930)=(S=13136720,E=13136735,C=83)
	CT(63931)=(S=13136768,E=13136815,C=209)
	CT(63932)=(S=13136816,E=13136816,C=208)
	CT(63933)=(S=13136817,E=13136817,C=209)
	CT(63934)=(S=13136818,E=13136818,C=181)
	CT(63935)=(S=13136819,E=13136819,C=207)
	CT(63936)=(S=13136820,E=13136821,C=181)
	CT(63937)=(S=13136822,E=13136823,C=218)
	CT(63938)=(S=13136824,E=13136824,C=203)
	CT(63939)=(S=13136825,E=13136827,C=83)
	CT(63940)=(S=13136828,E=13136831,C=208)
	CT(63941)=(S=13136832,E=13136895,C=83)
	CT(63942)=(S=13136896,E=13137151,C=181)
	CT(63943)=(S=13137152,E=13137407,C=83)
	CT(63944)=(S=13137408,E=13137791,C=181)
	CT(63945)=(S=13137792,E=13137855,C=215)
	CT(63946)=(S=13137856,E=13137887,C=201)
	CT(63947)=(S=13137888,E=13138175,C=203)
	CT(63948)=(S=13138176,E=13138431,C=212)
	CT(63949)=(S=13138432,E=13138559,C=83)
	CT(63950)=(S=13138560,E=13138623,C=202)
	CT(63951)=(S=13138624,E=13138687,C=181)
	CT(63952)=(S=13138688,E=13138719,C=212)
	CT(63953)=(S=13138720,E=13138895,C=83)
	CT(63954)=(S=13138896,E=13138903,C=170)
	CT(63955)=(S=13138904,E=13138943,C=83)
	CT(63956)=(S=13138944,E=13138975,C=209)
	CT(63957)=(S=13138976,E=13139007,C=203)
	CT(63958)=(S=13139040,E=13139063,C=83)
	CT(63959)=(S=13139064,E=13139064,C=206)
	CT(63960)=(S=13139065,E=13139065,C=200)
	CT(63961)=(S=13139066,E=13139067,C=83)
	CT(63962)=(S=13139068,E=13139069,C=181)
	CT(63963)=(S=13139070,E=13139070,C=83)
	CT(63964)=(S=13139071,E=13139071,C=209)
	CT(63965)=(S=13139072,E=13139103,C=207)
	CT(63966)=(S=13139120,E=13139131,C=83)
	CT(63967)=(S=13139132,E=13139135,C=212)
	CT(63968)=(S=13139136,E=13139167,C=205)
	CT(63969)=(S=13139168,E=13139199,C=213)
	CT(63970)=(S=13139200,E=13139263,C=205)
	CT(63971)=(S=13139264,E=13139327,C=83)
	CT(63972)=(S=13139328,E=13139359,C=213)
	CT(63973)=(S=13139360,E=13139367,C=176)
	CT(63974)=(S=13139384,E=13139391,C=200)
	CT(63975)=(S=13139392,E=13139487,C=213)
	CT(63976)=(S=13139488,E=13139503,C=203)
	CT(63977)=(S=13139504,E=13139504,C=213)
	CT(63978)=(S=13139505,E=13139507,C=203)
	CT(63979)=(S=13139508,E=13139519,C=212)
	CT(63980)=(S=13139520,E=13139583,C=203)
	CT(63981)=(S=13139584,E=13139967,C=83)
	CT(63982)=(S=13139968,E=13155327,C=189)
	CT(63983)=(S=13155328,E=13155487,C=204)
	CT(63984)=(S=13155488,E=13155627,C=189)
	CT(63985)=(S=13155628,E=13155631,C=208)
	CT(63986)=(S=13155632,E=13156456,C=189)
	CT(63987)=(S=13156457,E=13156457,C=212)
	CT(63988)=(S=13156460,E=13156587,C=189)
	CT(63989)=(S=13156588,E=13156591,C=202)
	CT(63990)=(S=13156592,E=13156863,C=189)
	CT(63991)=(S=13156864,E=13156927,C=204)
	CT(63992)=(S=13156928,E=13164799,C=189)
	CT(63993)=(S=13164800,E=13164831,C=204)
	CT(63994)=(S=13164832,E=13164899,C=189)
	CT(63995)=(S=13164900,E=13164903,C=83)
	CT(63996)=(S=13164904,E=13165967,C=189)
	CT(63997)=(S=13165968,E=13165971,C=202)
	CT(63998)=(S=13165972,E=13165975,C=83)
	CT(63999)=(S=13165976,E=13165979,C=200)
	CT(64000)=(S=13165980,E=13166039,C=189)
	CT(64001)=(S=13166040,E=13166043,C=200)
	CT(64002)=(S=13166044,E=13166073,C=189)
	CT(64003)=(S=13166076,E=13166079,C=83)
	CT(64004)=(S=13166080,E=13167679,C=189)
	CT(64005)=(S=13167680,E=13167743,C=204)
	CT(64006)=(S=13167744,E=13168383,C=189)
	CT(64007)=(S=13168384,E=13168447,C=204)
	CT(64008)=(S=13168448,E=13197311,C=189)
	CT(64009)=(S=13197312,E=13206031,C=204)
	CT(64010)=(S=13206036,E=13206039,C=189)
	CT(64011)=(S=13206040,E=13206043,C=212)
	CT(64012)=(S=13206044,E=13206047,C=189)
	CT(64013)=(S=13206048,E=13206097,C=204)
	CT(64014)=(S=13206098,E=13206099,C=200)
	CT(64015)=(S=13206100,E=13206111,C=189)
	CT(64016)=(S=13206112,E=13206281,C=204)
	CT(64017)=(S=13206282,E=13206283,C=202)
	CT(64018)=(S=13206284,E=13206295,C=204)
	CT(64019)=(S=13206296,E=13206303,C=189)
	CT(64020)=(S=13206304,E=13206308,C=204)
	CT(64021)=(S=13206309,E=13206309,C=202)
	CT(64022)=(S=13206310,E=13206311,C=203)
	CT(64023)=(S=13206312,E=13206312,C=204)
	CT(64024)=(S=13206313,E=13206313,C=201)
	CT(64025)=(S=13206314,E=13206314,C=210)
	CT(64026)=(S=13206315,E=13206315,C=207)
	CT(64027)=(S=13206316,E=13206316,C=204)
	CT(64028)=(S=13206317,E=13206317,C=201)
	CT(64029)=(S=13206318,E=13206318,C=181)
	CT(64030)=(S=13206319,E=13206319,C=83)
	CT(64031)=(S=13206320,E=13206320,C=204)
	CT(64032)=(S=13206321,E=13206321,C=209)
	CT(64033)=(S=13206322,E=13206322,C=177)
	CT(64034)=(S=13206323,E=13206323,C=206)
	CT(64035)=(S=13206324,E=13206337,C=204)
	CT(64036)=(S=13206338,E=13206338,C=216)
	CT(64037)=(S=13206339,E=13206339,C=212)
	CT(64038)=(S=13206340,E=13206343,C=189)
	CT(64039)=(S=13206344,E=13206344,C=204)
	CT(64040)=(S=13206345,E=13206345,C=215)
	CT(64041)=(S=13206346,E=13206347,C=83)
	CT(64042)=(S=13206348,E=13206348,C=204)
	CT(64043)=(S=13206349,E=13206349,C=218)
	CT(64044)=(S=13206350,E=13206350,C=181)
	CT(64045)=(S=13206351,E=13206351,C=83)
	CT(64046)=(S=13206352,E=13206359,C=189)
	CT(64047)=(S=13206362,E=13206363,C=181)
	CT(64048)=(S=13206364,E=13206367,C=189)
	CT(64049)=(S=13206368,E=13206368,C=204)
	CT(64050)=(S=13206369,E=13206369,C=181)
	CT(64051)=(S=13206370,E=13206371,C=209)
	CT(64052)=(S=13206372,E=13206373,C=204)
	CT(64053)=(S=13206374,E=13206374,C=83)
	CT(64054)=(S=13206375,E=13206375,C=224)
	CT(64055)=(S=13206376,E=13206377,C=204)
	CT(64056)=(S=13206378,E=13206379,C=177)
	CT(64057)=(S=13206380,E=13206381,C=204)
	CT(64058)=(S=13206382,E=13206383,C=210)
	CT(64059)=(S=13206384,E=13206385,C=204)
	CT(64060)=(S=13206386,E=13206386,C=181)
	CT(64061)=(S=13206387,E=13206387,C=216)
	CT(64062)=(S=13206388,E=13206389,C=204)
	CT(64063)=(S=13206390,E=13206391,C=83)
	CT(64064)=(S=13206392,E=13206392,C=204)
	CT(64065)=(S=13206393,E=13206393,C=212)
	CT(64066)=(S=13206394,E=13206395,C=210)
	CT(64067)=(S=13206396,E=13206407,C=204)
	CT(64068)=(S=13206408,E=13206419,C=189)
	CT(64069)=(S=13206420,E=13206423,C=209)
	CT(64070)=(S=13206424,E=13206459,C=189)
	CT(64071)=(S=13206460,E=13206463,C=181)
	CT(64072)=(S=13206464,E=13206479,C=204)
	CT(64073)=(S=13206480,E=13206487,C=189)
	CT(64074)=(S=13206488,E=13206491,C=181)
	CT(64075)=(S=13206492,E=13206495,C=212)
	CT(64076)=(S=13206496,E=13206503,C=189)
	CT(64077)=(S=13206504,E=13206511,C=204)
	CT(64078)=(S=13206512,E=13206519,C=189)
	CT(64079)=(S=13206520,E=13208399,C=204)
	CT(64080)=(S=13208400,E=13208403,C=83)
	CT(64081)=(S=13208404,E=13208415,C=189)
	CT(64082)=(S=13208416,E=13208491,C=204)
	CT(64083)=(S=13208492,E=13208495,C=189)
	CT(64084)=(S=13208496,E=13208503,C=204)
	CT(64085)=(S=13208504,E=13208511,C=189)
	CT(64086)=(S=13208512,E=13208535,C=204)
	CT(64087)=(S=13208536,E=13208543,C=189)
	CT(64088)=(S=13208544,E=13208783,C=204)
	CT(64089)=(S=13208784,E=13208799,C=189)
	CT(64090)=(S=13208800,E=13208803,C=83)
	CT(64091)=(S=13208804,E=13208811,C=189)
	CT(64092)=(S=13208812,E=13208815,C=181)
	CT(64093)=(S=13208816,E=13208831,C=189)
	CT(64094)=(S=13208832,E=13210719,C=204)
	CT(64095)=(S=13210720,E=13210727,C=189)
	CT(64096)=(S=13210728,E=13210731,C=203)
	CT(64097)=(S=13210732,E=13210751,C=189)
	CT(64098)=(S=13210752,E=13210783,C=204)
	CT(64099)=(S=13210784,E=13210791,C=189)
	CT(64100)=(S=13210792,E=13210795,C=208)
	CT(64101)=(S=13210796,E=13210815,C=189)
	CT(64102)=(S=13210816,E=13210831,C=204)
	CT(64103)=(S=13210832,E=13210835,C=189)
	CT(64104)=(S=13210836,E=13210839,C=203)
	CT(64105)=(S=13210840,E=13210855,C=189)
	CT(64106)=(S=13210856,E=13210859,C=202)
	CT(64107)=(S=13210860,E=13210879,C=189)
	CT(64108)=(S=13210880,E=13210975,C=204)
	CT(64109)=(S=13210976,E=13211007,C=189)
	CT(64110)=(S=13211008,E=13211147,C=204)
	CT(64111)=(S=13211148,E=13211163,C=189)
	CT(64112)=(S=13211164,E=13211167,C=203)
	CT(64113)=(S=13211168,E=13211183,C=204)
	CT(64114)=(S=13211184,E=13211195,C=189)
	CT(64115)=(S=13211196,E=13211199,C=212)
	CT(64116)=(S=13211200,E=13211215,C=204)
	CT(64117)=(S=13211216,E=13211219,C=189)
	CT(64118)=(S=13211220,E=13211223,C=210)
	CT(64119)=(S=13211224,E=13211231,C=189)
	CT(64120)=(S=13211232,E=13211235,C=181)
	CT(64121)=(S=13211236,E=13211239,C=189)
	CT(64122)=(S=13211240,E=13211243,C=218)
	CT(64123)=(S=13211244,E=13211263,C=189)
	CT(64124)=(S=13211264,E=13211279,C=204)
	CT(64125)=(S=13211280,E=13211295,C=189)
	CT(64126)=(S=13211296,E=13211327,C=201)
	CT(64127)=(S=13211328,E=13213119,C=204)
	CT(64128)=(S=13213120,E=13213183,C=189)
	CT(64129)=(S=13213184,E=13213191,C=204)
	CT(64130)=(S=13213192,E=13213199,C=189)
	CT(64131)=(S=13213200,E=13213203,C=204)
	CT(64132)=(S=13213204,E=13213215,C=189)
	CT(64133)=(S=13213216,E=13213223,C=204)
	CT(64134)=(S=13213224,E=13213231,C=189)
	CT(64135)=(S=13213232,E=13213235,C=208)
	CT(64136)=(S=13213236,E=13213239,C=212)
	CT(64137)=(S=13213240,E=13213247,C=189)
	CT(64138)=(S=13213248,E=13213291,C=204)
	CT(64139)=(S=13213292,E=13213295,C=189)
	CT(64140)=(S=13213296,E=13213303,C=204)
	CT(64141)=(S=13213304,E=13213311,C=83)
	CT(64142)=(S=13213312,E=13213447,C=204)
	CT(64143)=(S=13213448,E=13213455,C=83)
	CT(64144)=(S=13213456,E=13213463,C=204)
	CT(64145)=(S=13213464,E=13213471,C=189)
	CT(64146)=(S=13213472,E=13213483,C=204)
	CT(64147)=(S=13213484,E=13213487,C=189)
	CT(64148)=(S=13213488,E=13213491,C=204)
	CT(64149)=(S=13213492,E=13213495,C=189)
	CT(64150)=(S=13213496,E=13213503,C=83)
	CT(64151)=(S=13213504,E=13213511,C=204)
	CT(64152)=(S=13213512,E=13213519,C=189)
	CT(64153)=(S=13213520,E=13213523,C=204)
	CT(64154)=(S=13213524,E=13213535,C=189)
	CT(64155)=(S=13213536,E=13213551,C=204)
	CT(64156)=(S=13213552,E=13213555,C=215)
	CT(64157)=(S=13213556,E=13213563,C=189)
	CT(64158)=(S=13213564,E=13213583,C=204)
	CT(64159)=(S=13213584,E=13213599,C=189)
	CT(64160)=(S=13213600,E=13213607,C=204)
	CT(64161)=(S=13213608,E=13213615,C=203)
	CT(64162)=(S=13213616,E=13213619,C=204)
	CT(64163)=(S=13213620,E=13213631,C=189)
	CT(64164)=(S=13213632,E=13213655,C=204)
	CT(64165)=(S=13213656,E=13213659,C=83)
	CT(64166)=(S=13213660,E=13213663,C=213)
	CT(64167)=(S=13213664,E=13213687,C=204)
	CT(64168)=(S=13213688,E=13213695,C=189)
	CT(64169)=(S=13213696,E=13214271,C=204)
	CT(64170)=(S=13214272,E=13214335,C=189)
	CT(64171)=(S=13214336,E=13217583,C=204)
	CT(64172)=(S=13217584,E=13217599,C=189)
	CT(64173)=(S=13217600,E=13217791,C=204)
	CT(64174)=(S=13217792,E=13219327,C=83)
	CT(64175)=(S=13219584,E=13219839,C=213)
	CT(64176)=(S=13219840,E=13220351,C=181)
	CT(64177)=(S=13220352,E=13221375,C=203)
	CT(64178)=(S=13221376,E=13221439,C=208)
	CT(64179)=(S=13221440,E=13221503,C=181)
	CT(64180)=(S=13221504,E=13221631,C=209)
	CT(64181)=(S=13221632,E=13225983,C=202)
	CT(64182)=(S=13225984,E=13227007,C=200)
	CT(64183)=(S=13227008,E=13227519,C=83)
	CT(64184)=(S=13227520,E=13228031,C=203)
	CT(64185)=(S=13228032,E=13228095,C=181)
	CT(64186)=(S=13228160,E=13228223,C=215)
	CT(64187)=(S=13228224,E=13228287,C=83)
	CT(64188)=(S=13228288,E=13228351,C=206)
	CT(64189)=(S=13228352,E=13228415,C=213)
	CT(64190)=(S=13228416,E=13228479,C=205)
	CT(64191)=(S=13228480,E=13228511,C=181)
	CT(64192)=(S=13228512,E=13228527,C=83)
	CT(64193)=(S=13228528,E=13228531,C=203)
	CT(64194)=(S=13228532,E=13228535,C=83)
	CT(64195)=(S=13228536,E=13228543,C=203)
	CT(64196)=(S=13228544,E=13228607,C=213)
	CT(64197)=(S=13228608,E=13228671,C=209)
	CT(64198)=(S=13228672,E=13228703,C=203)
	CT(64199)=(S=13228704,E=13228735,C=202)
	CT(64200)=(S=13228736,E=13228799,C=209)
	CT(64201)=(S=13228800,E=13228863,C=213)
	CT(64202)=(S=13228864,E=13228911,C=83)
	CT(64203)=(S=13228912,E=13228927,C=181)
	CT(64204)=(S=13228928,E=13228959,C=203)
	CT(64205)=(S=13228960,E=13228991,C=83)
	CT(64206)=(S=13228992,E=13229023,C=181)
	CT(64207)=(S=13229024,E=13229031,C=205)
	CT(64208)=(S=13229032,E=13229039,C=203)
	CT(64209)=(S=13229040,E=13229047,C=181)
	CT(64210)=(S=13229048,E=13229051,C=189)
	CT(64211)=(S=13229052,E=13229055,C=205)
	CT(64212)=(S=13229056,E=13229071,C=211)
	CT(64213)=(S=13229072,E=13229080,C=83)
	CT(64214)=(S=13229081,E=13229081,C=206)
	CT(64215)=(S=13229082,E=13229082,C=202)
	CT(64216)=(S=13229083,E=13229083,C=203)
	CT(64217)=(S=13229084,E=13229084,C=83)
	CT(64218)=(S=13229085,E=13229085,C=202)
	CT(64219)=(S=13229086,E=13229151,C=181)
	CT(64220)=(S=13229152,E=13229183,C=203)
	CT(64221)=(S=13229184,E=13229199,C=208)
	CT(64222)=(S=13229200,E=13229247,C=83)
	CT(64223)=(S=13229248,E=13229279,C=219)
	CT(64224)=(S=13229280,E=13229311,C=203)
	CT(64225)=(S=13229312,E=13229375,C=205)
	CT(64226)=(S=13229376,E=13229407,C=176)
	CT(64227)=(S=13229408,E=13229423,C=83)
	CT(64228)=(S=13229424,E=13229431,C=203)
	CT(64229)=(S=13229432,E=13229432,C=83)
	CT(64230)=(S=13229433,E=13229433,C=203)
	CT(64231)=(S=13229434,E=13229434,C=181)
	CT(64232)=(S=13229435,E=13229435,C=203)
	CT(64233)=(S=13229436,E=13229437,C=181)
	CT(64234)=(S=13229438,E=13229438,C=177)
	CT(64235)=(S=13229439,E=13229439,C=213)
	CT(64236)=(S=13229440,E=13229503,C=181)
	CT(64237)=(S=13229504,E=13229535,C=203)
	CT(64238)=(S=13229536,E=13229567,C=209)
	CT(64239)=(S=13229568,E=13229583,C=200)
	CT(64240)=(S=13229584,E=13229587,C=83)
	CT(64241)=(S=13229588,E=13229599,C=189)
	CT(64242)=(S=13229616,E=13229623,C=206)
	CT(64243)=(S=13229624,E=13229627,C=83)
	CT(64244)=(S=13229628,E=13229631,C=215)
	CT(64245)=(S=13229632,E=13229695,C=201)
	CT(64246)=(S=13229696,E=13230079,C=203)
	CT(64247)=(S=13230080,E=13231103,C=209)
	CT(64248)=(S=13231104,E=13231359,C=181)
	CT(64249)=(S=13231360,E=13231487,C=225)
	CT(64250)=(S=13231488,E=13231615,C=177)
	CT(64251)=(S=13231616,E=13231871,C=212)
	CT(64252)=(S=13231872,E=13232127,C=83)
	CT(64253)=(S=13232128,E=13232639,C=181)
	CT(64254)=(S=13232640,E=13233151,C=83)
	CT(64255)=(S=13233152,E=13233343,C=203)
	CT(64256)=(S=13233344,E=13233407,C=181)
	CT(64257)=(S=13233408,E=13233663,C=202)
	CT(64258)=(S=13233664,E=13233727,C=200)
	CT(64259)=(S=13233728,E=13233791,C=176)
	CT(64260)=(S=13233792,E=13233855,C=213)
	CT(64261)=(S=13233856,E=13234175,C=203)
	CT(64262)=(S=13234176,E=13234431,C=212)
	CT(64263)=(S=13234432,E=13234687,C=203)
	CT(64264)=(S=13234688,E=13235199,C=200)
	CT(64265)=(S=13235200,E=13235711,C=181)
	CT(64266)=(S=13235712,E=13235967,C=203)
	CT(64267)=(S=13235968,E=13236223,C=210)
	CT(64268)=(S=13236224,E=13236735,C=200)
	CT(64269)=(S=13236736,E=13238271,C=83)
	CT(64270)=(S=13238272,E=13238287,C=0)
	CT(64271)=(S=13238288,E=13238303,C=10)
	CT(64272)=(S=13238304,E=13238335,C=73)
	CT(64273)=(S=13238336,E=13238336,C=0)
	CT(64274)=(S=13238337,E=13238338,C=2)
	CT(64275)=(S=13238339,E=13238342,C=0)
	CT(64276)=(S=13238343,E=13238343,C=7)
	CT(64277)=(S=13238344,E=13238345,C=2)
	CT(64278)=(S=13238346,E=13238347,C=0)
	CT(64279)=(S=13238348,E=13238348,C=2)
	CT(64280)=(S=13238349,E=13238349,C=8)
	CT(64281)=(S=13238350,E=13238350,C=0)
	CT(64282)=(S=13238351,E=13238351,C=11)
	CT(64283)=(S=13238352,E=13238352,C=76)
	CT(64284)=(S=13238353,E=13238355,C=0)
	CT(64285)=(S=13238356,E=13238356,C=73)
	CT(64286)=(S=13238357,E=13238359,C=0)
	CT(64287)=(S=13238360,E=13238361,C=74)
	CT(64288)=(S=13238362,E=13238362,C=0)
	CT(64289)=(S=13238363,E=13238363,C=10)
	CT(64290)=(S=13238364,E=13238365,C=74)
	CT(64291)=(S=13238367,E=13238371,C=0)
	CT(64292)=(S=13238372,E=13238373,C=1)
	CT(64293)=(S=13238374,E=13238374,C=0)
	CT(64294)=(S=13238375,E=13238375,C=4)
	CT(64295)=(S=13238376,E=13238376,C=7)
	CT(64296)=(S=13238377,E=13238378,C=0)
	CT(64297)=(S=13238379,E=13238379,C=74)
	CT(64298)=(S=13238380,E=13238380,C=0)
	CT(64299)=(S=13238381,E=13238381,C=4)
	CT(64300)=(S=13238382,E=13238382,C=77)
	CT(64301)=(S=13238383,E=13238383,C=3)
	CT(64302)=(S=13238384,E=13238384,C=4)
	CT(64303)=(S=13238385,E=13238386,C=0)
	CT(64304)=(S=13238387,E=13238388,C=4)
	CT(64305)=(S=13238389,E=13238392,C=3)
	CT(64306)=(S=13238393,E=13238393,C=73)
	CT(64307)=(S=13238394,E=13238395,C=1)
	CT(64308)=(S=13238396,E=13238397,C=73)
	CT(64309)=(S=13238398,E=13238399,C=7)
	CT(64310)=(S=13238400,E=13238415,C=8)
	CT(64311)=(S=13238416,E=13238419,C=10)
	CT(64312)=(S=13238421,E=13238421,C=7)
	CT(64313)=(S=13238423,E=13238423,C=73)
	CT(64314)=(S=13238426,E=13238427,C=0)
	CT(64315)=(S=13238428,E=13238431,C=90)
	CT(64316)=(S=13238432,E=13238447,C=8)
	CT(64317)=(S=13238448,E=13238451,C=1)
	CT(64318)=(S=13238452,E=13238455,C=8)
	CT(64319)=(S=13238456,E=13238529,C=0)
	CT(64320)=(S=13238530,E=13238530,C=8)
	CT(64321)=(S=13238531,E=13238531,C=0)
	CT(64322)=(S=13238532,E=13238532,C=4)
	CT(64323)=(S=13238533,E=13238533,C=7)
	CT(64324)=(S=13238534,E=13238535,C=8)
	CT(64325)=(S=13238536,E=13238543,C=73)
	CT(64326)=(S=13238544,E=13238559,C=0)
	CT(64327)=(S=13238560,E=13238591,C=76)
	CT(64328)=(S=13238592,E=13238641,C=7)
	CT(64329)=(S=13238642,E=13238643,C=10)
	CT(64330)=(S=13238644,E=13238647,C=0)
	CT(64331)=(S=13238648,E=13238655,C=4)
	CT(64332)=(S=13238656,E=13238687,C=7)
	CT(64333)=(S=13238688,E=13238719,C=192)
	CT(64334)=(S=13238720,E=13238735,C=88)
	CT(64335)=(S=13238736,E=13238751,C=2)
	CT(64336)=(S=13238760,E=13238760,C=8)
	CT(64337)=(S=13238761,E=13238763,C=0)
	CT(64338)=(S=13238764,E=13238764,C=74)
	CT(64339)=(S=13238765,E=13238766,C=0)
	CT(64340)=(S=13238767,E=13238767,C=7)
	CT(64341)=(S=13238768,E=13238783,C=76)
	CT(64342)=(S=13238784,E=13238787,C=0)
	CT(64343)=(S=13238788,E=13238799,C=73)
	CT(64344)=(S=13238800,E=13238815,C=0)
	CT(64345)=(S=13238816,E=13238835,C=8)
	CT(64346)=(S=13238836,E=13238839,C=9)
	CT(64347)=(S=13238840,E=13238847,C=0)
	CT(64348)=(S=13238848,E=13238871,C=8)
	CT(64349)=(S=13238872,E=13238875,C=2)
	CT(64350)=(S=13238876,E=13238879,C=0)
	CT(64351)=(S=13238880,E=13238911,C=232)
	CT(64352)=(S=13238912,E=13239039,C=2)
	CT(64353)=(S=13239040,E=13239045,C=0)
	CT(64354)=(S=13239046,E=13239047,C=8)
	CT(64355)=(S=13239048,E=13239052,C=0)
	CT(64356)=(S=13239053,E=13239053,C=4)
	CT(64357)=(S=13239054,E=13239054,C=74)
	CT(64358)=(S=13239055,E=13239055,C=4)
	CT(64359)=(S=13239056,E=13239071,C=6)
	CT(64360)=(S=13239072,E=13239103,C=73)
	CT(64361)=(S=13239104,E=13239104,C=4)
	CT(64362)=(S=13239105,E=13239105,C=0)
	CT(64363)=(S=13239106,E=13239106,C=4)
	CT(64364)=(S=13239107,E=13239107,C=0)
	CT(64365)=(S=13239108,E=13239111,C=3)
	CT(64366)=(S=13239114,E=13239114,C=0)
	CT(64367)=(S=13239117,E=13239117,C=4)
	CT(64368)=(S=13239118,E=13239118,C=7)
	CT(64369)=(S=13239119,E=13239119,C=0)
	CT(64370)=(S=13239120,E=13239135,C=73)
	CT(64371)=(S=13239136,E=13239151,C=0)
	CT(64372)=(S=13239152,E=13239167,C=4)
	CT(64373)=(S=13239168,E=13239169,C=1)
	CT(64374)=(S=13239170,E=13239171,C=77)
	CT(64375)=(S=13239172,E=13239174,C=0)
	CT(64376)=(S=13239175,E=13239175,C=7)
	CT(64377)=(S=13239176,E=13239176,C=2)
	CT(64378)=(S=13239177,E=13239177,C=0)
	CT(64379)=(S=13239178,E=13239179,C=7)
	CT(64380)=(S=13239180,E=13239199,C=2)
	CT(64381)=(S=13239200,E=13239231,C=9)
	CT(64382)=(S=13239232,E=13239247,C=0)
	CT(64383)=(S=13239248,E=13239263,C=74)
	CT(64384)=(S=13239264,E=13239295,C=164)
	CT(64385)=(S=13239296,E=13239311,C=10)
	CT(64386)=(S=13239312,E=13239320,C=73)
	CT(64387)=(S=13239321,E=13239321,C=4)
	CT(64388)=(S=13239322,E=13239323,C=8)
	CT(64389)=(S=13239324,E=13239327,C=0)
	CT(64390)=(S=13239328,E=13239359,C=162)
	CT(64391)=(S=13239360,E=13239391,C=0)
	CT(64392)=(S=13239392,E=13239423,C=84)
	CT(64393)=(S=13239424,E=13239455,C=1)
	CT(64394)=(S=13239456,E=13239457,C=74)
	CT(64395)=(S=13239458,E=13239459,C=0)
	CT(64396)=(S=13239460,E=13239460,C=4)
	CT(64397)=(S=13239461,E=13239462,C=0)
	CT(64398)=(S=13239463,E=13239463,C=77)
	CT(64399)=(S=13239464,E=13239464,C=11)
	CT(64400)=(S=13239466,E=13239467,C=74)
	CT(64401)=(S=13239468,E=13239468,C=4)
	CT(64402)=(S=13239469,E=13239469,C=84)
	CT(64403)=(S=13239470,E=13239470,C=10)
	CT(64404)=(S=13239471,E=13239471,C=0)
	CT(64405)=(S=13239472,E=13239472,C=11)
	CT(64406)=(S=13239473,E=13239474,C=4)
	CT(64407)=(S=13239475,E=13239475,C=74)
	CT(64408)=(S=13239476,E=13239476,C=4)
	CT(64409)=(S=13239477,E=13239477,C=0)
	CT(64410)=(S=13239478,E=13239478,C=4)
	CT(64411)=(S=13239479,E=13239479,C=0)
	CT(64412)=(S=13239480,E=13239480,C=7)
	CT(64413)=(S=13239481,E=13239482,C=74)
	CT(64414)=(S=13239484,E=13239486,C=0)
	CT(64415)=(S=13239487,E=13239487,C=10)
	CT(64416)=(S=13239488,E=13239519,C=8)
	CT(64417)=(S=13239520,E=13239539,C=0)
	CT(64418)=(S=13239540,E=13239546,C=2)
	CT(64419)=(S=13239547,E=13239547,C=160)
	CT(64420)=(S=13239548,E=13239551,C=1)
	CT(64421)=(S=13239552,E=13239555,C=0)
	CT(64422)=(S=13239556,E=13239557,C=9)
	CT(64423)=(S=13239558,E=13239559,C=0)
	CT(64424)=(S=13239560,E=13239567,C=9)
	CT(64425)=(S=13239584,E=13239615,C=84)
	CT(64426)=(S=13239616,E=13239631,C=2)
	CT(64427)=(S=13239632,E=13239647,C=3)
	CT(64428)=(S=13239648,E=13239679,C=0)
	CT(64429)=(S=13239680,E=13239711,C=77)
	CT(64430)=(S=13239712,E=13239743,C=0)
	CT(64431)=(S=13239744,E=13239759,C=92)
	CT(64432)=(S=13239776,E=13239807,C=9)
	CT(64433)=(S=13239808,E=13239809,C=8)
	CT(64434)=(S=13239810,E=13239810,C=11)
	CT(64435)=(S=13239811,E=13239812,C=0)
	CT(64436)=(S=13239813,E=13239813,C=73)
	CT(64437)=(S=13239814,E=13239815,C=1)
	CT(64438)=(S=13239816,E=13239823,C=2)
	CT(64439)=(S=13239824,E=13239839,C=3)
	CT(64440)=(S=13239840,E=13239873,C=0)
	CT(64441)=(S=13239874,E=13239875,C=1)
	CT(64442)=(S=13239876,E=13239879,C=2)
	CT(64443)=(S=13239880,E=13239881,C=1)
	CT(64444)=(S=13239882,E=13239891,C=0)
	CT(64445)=(S=13239892,E=13239892,C=73)
	CT(64446)=(S=13239893,E=13239893,C=10)
	CT(64447)=(S=13239894,E=13239894,C=0)
	CT(64448)=(S=13239895,E=13239897,C=1)
	CT(64449)=(S=13239898,E=13239898,C=3)
	CT(64450)=(S=13239899,E=13239899,C=0)
	CT(64451)=(S=13239900,E=13239901,C=1)
	CT(64452)=(S=13239902,E=13239902,C=8)
	CT(64453)=(S=13239903,E=13239903,C=6)
	CT(64454)=(S=13239904,E=13239905,C=11)
	CT(64455)=(S=13239906,E=13239906,C=2)
	CT(64456)=(S=13239907,E=13239907,C=4)
	CT(64457)=(S=13239908,E=13239909,C=3)
	CT(64458)=(S=13239910,E=13239910,C=7)
	CT(64459)=(S=13239911,E=13239911,C=1)
	CT(64460)=(S=13239912,E=13239913,C=9)
	CT(64461)=(S=13239914,E=13239914,C=0)
	CT(64462)=(S=13239915,E=13239915,C=3)
	CT(64463)=(S=13239916,E=13239916,C=1)
	CT(64464)=(S=13239917,E=13239917,C=73)
	CT(64465)=(S=13239918,E=13239919,C=1)
	CT(64466)=(S=13239920,E=13239921,C=0)
	CT(64467)=(S=13239922,E=13239922,C=1)
	CT(64468)=(S=13239923,E=13239923,C=0)
	CT(64469)=(S=13239924,E=13239925,C=73)
	CT(64470)=(S=13239927,E=13239927,C=2)
	CT(64471)=(S=13239928,E=13239931,C=193)
	CT(64472)=(S=13239932,E=13239967,C=0)
	CT(64473)=(S=13239968,E=13239983,C=7)
	CT(64474)=(S=13239984,E=13239999,C=1)
	CT(64475)=(S=13240000,E=13240015,C=8)
	CT(64476)=(S=13240016,E=13240047,C=74)
	CT(64477)=(S=13240048,E=13240051,C=7)
	CT(64478)=(S=13240052,E=13240055,C=2)
	CT(64479)=(S=13240056,E=13240059,C=0)
	CT(64480)=(S=13240060,E=13240063,C=2)
	CT(64481)=(S=13240064,E=13240065,C=73)
	CT(64482)=(S=13240066,E=13240067,C=0)
	CT(64483)=(S=13240068,E=13240071,C=73)
	CT(64484)=(S=13240072,E=13240079,C=0)
	CT(64485)=(S=13240080,E=13240095,C=2)
	CT(64486)=(S=13240096,E=13240115,C=73)
	CT(64487)=(S=13240122,E=13240122,C=4)
	CT(64488)=(S=13240124,E=13240127,C=73)
	CT(64489)=(S=13240128,E=13240191,C=2)
	CT(64490)=(S=13240192,E=13240207,C=8)
	CT(64491)=(S=13240208,E=13240319,C=0)
	CT(64492)=(S=13240320,E=13240320,C=1)
	CT(64493)=(S=13240321,E=13240321,C=7)
	CT(64494)=(S=13240322,E=13240325,C=1)
	CT(64495)=(S=13240326,E=13240327,C=2)
	CT(64496)=(S=13240328,E=13240331,C=0)
	CT(64497)=(S=13240332,E=13240332,C=1)
	CT(64498)=(S=13240333,E=13240333,C=73)
	CT(64499)=(S=13240334,E=13240335,C=9)
	CT(64500)=(S=13240336,E=13240343,C=2)
	CT(64501)=(S=13240344,E=13240344,C=1)
	CT(64502)=(S=13240346,E=13240347,C=2)
	CT(64503)=(S=13240348,E=13240349,C=74)
	CT(64504)=(S=13240350,E=13240350,C=10)
	CT(64505)=(S=13240351,E=13240351,C=8)
	CT(64506)=(S=13240352,E=13240359,C=0)
	CT(64507)=(S=13240360,E=13240363,C=7)
	CT(64508)=(S=13240364,E=13240367,C=73)
	CT(64509)=(S=13240368,E=13240383,C=2)
	CT(64510)=(S=13240384,E=13240384,C=0)
	CT(64511)=(S=13240385,E=13240391,C=3)
	CT(64512)=(S=13240392,E=13240395,C=86)
	CT(64513)=(S=13240396,E=13240396,C=0)
	CT(64514)=(S=13240397,E=13240397,C=1)
	CT(64515)=(S=13240398,E=13240399,C=3)
	CT(64516)=(S=13240400,E=13240403,C=2)
	CT(64517)=(S=13240404,E=13240407,C=3)
	CT(64518)=(S=13240408,E=13240411,C=8)
	CT(64519)=(S=13240412,E=13240412,C=0)
	CT(64520)=(S=13240413,E=13240413,C=8)
	CT(64521)=(S=13240414,E=13240415,C=7)
	CT(64522)=(S=13240448,E=13240479,C=1)
	CT(64523)=(S=13240480,E=13240511,C=6)
	CT(64524)=(S=13240512,E=13240527,C=1)
	CT(64525)=(S=13240528,E=13240543,C=2)
	CT(64526)=(S=13240544,E=13240575,C=10)
	CT(64527)=(S=13240576,E=13240607,C=0)
	CT(64528)=(S=13240608,E=13240608,C=1)
	CT(64529)=(S=13240609,E=13240609,C=0)
	CT(64530)=(S=13240610,E=13240611,C=1)
	CT(64531)=(S=13240612,E=13240615,C=5)
	CT(64532)=(S=13240616,E=13240623,C=92)
	CT(64533)=(S=13240624,E=13240625,C=1)
	CT(64534)=(S=13240626,E=13240626,C=0)
	CT(64535)=(S=13240627,E=13240630,C=1)
	CT(64536)=(S=13240632,E=13240632,C=0)
	CT(64537)=(S=13240633,E=13240635,C=1)
	CT(64538)=(S=13240641,E=13240641,C=0)
	CT(64539)=(S=13240642,E=13240643,C=7)
	CT(64540)=(S=13240645,E=13240645,C=74)
	CT(64541)=(S=13240646,E=13240647,C=7)
	CT(64542)=(S=13240648,E=13240649,C=74)
	CT(64543)=(S=13240650,E=13240650,C=0)
	CT(64544)=(S=13240651,E=13240651,C=4)
	CT(64545)=(S=13240652,E=13240653,C=154)
	CT(64546)=(S=13240654,E=13240654,C=0)
	CT(64547)=(S=13240655,E=13240656,C=11)
	CT(64548)=(S=13240657,E=13240657,C=3)
	CT(64549)=(S=13240658,E=13240658,C=8)
	CT(64550)=(S=13240659,E=13240659,C=0)
	CT(64551)=(S=13240660,E=13240660,C=11)
	CT(64552)=(S=13240661,E=13240661,C=74)
	CT(64553)=(S=13240662,E=13240662,C=7)
	CT(64554)=(S=13240663,E=13240663,C=0)
	CT(64555)=(S=13240664,E=13240664,C=73)
	CT(64556)=(S=13240667,E=13240669,C=4)
	CT(64557)=(S=13240670,E=13240671,C=0)
	CT(64558)=(S=13240672,E=13240687,C=5)
	CT(64559)=(S=13240704,E=13240783,C=4)
	CT(64560)=(S=13240784,E=13240799,C=2)
	CT(64561)=(S=13240800,E=13240831,C=73)
	CT(64562)=(S=13240832,E=13240863,C=0)
	CT(64563)=(S=13240864,E=13240895,C=74)
	CT(64564)=(S=13240896,E=13240911,C=1)
	CT(64565)=(S=13240912,E=13240927,C=0)
	CT(64566)=(S=13240928,E=13240943,C=2)
	CT(64567)=(S=13240960,E=13241087,C=0)
	CT(64568)=(S=13241088,E=13241343,C=2)
	CT(64569)=(S=13241344,E=13241344,C=73)
	CT(64570)=(S=13241345,E=13241346,C=1)
	CT(64571)=(S=13241347,E=13241347,C=73)
	CT(64572)=(S=13241348,E=13241351,C=8)
	CT(64573)=(S=13241352,E=13241359,C=2)
	CT(64574)=(S=13241360,E=13241360,C=4)
	CT(64575)=(S=13241361,E=13241363,C=1)
	CT(64576)=(S=13241364,E=13241365,C=3)
	CT(64577)=(S=13241366,E=13241369,C=0)
	CT(64578)=(S=13241370,E=13241370,C=157)
	CT(64579)=(S=13241371,E=13241371,C=2)
	CT(64580)=(S=13241372,E=13241373,C=0)
	CT(64581)=(S=13241374,E=13241374,C=2)
	CT(64582)=(S=13241375,E=13241407,C=0)
	CT(64583)=(S=13241408,E=13241411,C=2)
	CT(64584)=(S=13241412,E=13241413,C=0)
	CT(64585)=(S=13241414,E=13241414,C=73)
	CT(64586)=(S=13241415,E=13241415,C=0)
	CT(64587)=(S=13241416,E=13241416,C=1)
	CT(64588)=(S=13241417,E=13241418,C=3)
	CT(64589)=(S=13241419,E=13241419,C=74)
	CT(64590)=(S=13241424,E=13241427,C=4)
	CT(64591)=(S=13241428,E=13241429,C=1)
	CT(64592)=(S=13241430,E=13241434,C=0)
	CT(64593)=(S=13241435,E=13241435,C=73)
	CT(64594)=(S=13241436,E=13241437,C=0)
	CT(64595)=(S=13241438,E=13241439,C=7)
	CT(64596)=(S=13241440,E=13241440,C=1)
	CT(64597)=(S=13241441,E=13241441,C=3)
	CT(64598)=(S=13241442,E=13241443,C=1)
	CT(64599)=(S=13241444,E=13241444,C=0)
	CT(64600)=(S=13241445,E=13241445,C=4)
	CT(64601)=(S=13241446,E=13241446,C=0)
	CT(64602)=(S=13241447,E=13241447,C=4)
	CT(64603)=(S=13241448,E=13241449,C=73)
	CT(64604)=(S=13241450,E=13241450,C=1)
	CT(64605)=(S=13241451,E=13241451,C=0)
	CT(64606)=(S=13241452,E=13241452,C=73)
	CT(64607)=(S=13241453,E=13241454,C=0)
	CT(64608)=(S=13241455,E=13241455,C=1)
	CT(64609)=(S=13241456,E=13241459,C=0)
	CT(64610)=(S=13241460,E=13241460,C=1)
	CT(64611)=(S=13241461,E=13241462,C=3)
	CT(64612)=(S=13241463,E=13241586,C=0)
	CT(64613)=(S=13241588,E=13241591,C=2)
	CT(64614)=(S=13241592,E=13241599,C=73)
	CT(64615)=(S=13241600,E=13241855,C=2)
	CT(64616)=(S=13241856,E=13241859,C=0)
	CT(64617)=(S=13241864,E=13241871,C=7)
	CT(64618)=(S=13241872,E=13241919,C=73)
	CT(64619)=(S=13241920,E=13241921,C=1)
	CT(64620)=(S=13241922,E=13241922,C=0)
	CT(64621)=(S=13241923,E=13241924,C=2)
	CT(64622)=(S=13241925,E=13241925,C=1)
	CT(64623)=(S=13241926,E=13241927,C=77)
	CT(64624)=(S=13241928,E=13241928,C=4)
	CT(64625)=(S=13241929,E=13241932,C=1)
	CT(64626)=(S=13241933,E=13241933,C=0)
	CT(64627)=(S=13241934,E=13241935,C=1)
	CT(64628)=(S=13241936,E=13241936,C=8)
	CT(64629)=(S=13241937,E=13241937,C=0)
	CT(64630)=(S=13241938,E=13241940,C=73)
	CT(64631)=(S=13241941,E=13241943,C=10)
	CT(64632)=(S=13241944,E=13241944,C=1)
	CT(64633)=(S=13241945,E=13241945,C=0)
	CT(64634)=(S=13241946,E=13241947,C=6)
	CT(64635)=(S=13241948,E=13241949,C=74)
	CT(64636)=(S=13241950,E=13241951,C=0)
	CT(64637)=(S=13241952,E=13241952,C=73)
	CT(64638)=(S=13241953,E=13241953,C=1)
	CT(64639)=(S=13241954,E=13241954,C=0)
	CT(64640)=(S=13241955,E=13241956,C=73)
	CT(64641)=(S=13241957,E=13241957,C=0)
	CT(64642)=(S=13241958,E=13241958,C=73)
	CT(64643)=(S=13241959,E=13241959,C=6)
	CT(64644)=(S=13241960,E=13241961,C=1)
	CT(64645)=(S=13241962,E=13241963,C=73)
	CT(64646)=(S=13241964,E=13241965,C=1)
	CT(64647)=(S=13241966,E=13241966,C=0)
	CT(64648)=(S=13241967,E=13241967,C=1)
	CT(64649)=(S=13241968,E=13241969,C=0)
	CT(64650)=(S=13241970,E=13241971,C=1)
	CT(64651)=(S=13241972,E=13241972,C=0)
	CT(64652)=(S=13241973,E=13241973,C=3)
	CT(64653)=(S=13241974,E=13241981,C=1)
	CT(64654)=(S=13241982,E=13241982,C=0)
	CT(64655)=(S=13241983,E=13241983,C=1)
	CT(64656)=(S=13241984,E=13241984,C=0)
	CT(64657)=(S=13241985,E=13241985,C=1)
	CT(64658)=(S=13241986,E=13241990,C=0)
	CT(64659)=(S=13241991,E=13241992,C=1)
	CT(64660)=(S=13241993,E=13241995,C=0)
	CT(64661)=(S=13241996,E=13241998,C=73)
	CT(64662)=(S=13241999,E=13242004,C=0)
	CT(64663)=(S=13242005,E=13242005,C=1)
	CT(64664)=(S=13242006,E=13242006,C=0)
	CT(64665)=(S=13242007,E=13242007,C=1)
	CT(64666)=(S=13242008,E=13242008,C=0)
	CT(64667)=(S=13242009,E=13242010,C=7)
	CT(64668)=(S=13242011,E=13242012,C=0)
	CT(64669)=(S=13242013,E=13242015,C=1)
	CT(64670)=(S=13242016,E=13242019,C=4)
	CT(64671)=(S=13242020,E=13242020,C=3)
	CT(64672)=(S=13242021,E=13242021,C=6)
	CT(64673)=(S=13242022,E=13242022,C=0)
	CT(64674)=(S=13242023,E=13242024,C=73)
	CT(64675)=(S=13242025,E=13242032,C=1)
	CT(64676)=(S=13242033,E=13242039,C=0)
	CT(64677)=(S=13242040,E=13242041,C=1)
	CT(64678)=(S=13242042,E=13242055,C=0)
	CT(64679)=(S=13242056,E=13242063,C=7)
	CT(64680)=(S=13242064,E=13242065,C=1)
	CT(64681)=(S=13242066,E=13242068,C=0)
	CT(64682)=(S=13242069,E=13242069,C=1)
	CT(64683)=(S=13242070,E=13242071,C=0)
	CT(64684)=(S=13242072,E=13242074,C=73)
	CT(64685)=(S=13242075,E=13242076,C=1)
	CT(64686)=(S=13242077,E=13242077,C=0)
	CT(64687)=(S=13242078,E=13242083,C=1)
	CT(64688)=(S=13242084,E=13242084,C=73)
	CT(64689)=(S=13242085,E=13242086,C=0)
	CT(64690)=(S=13242087,E=13242087,C=1)
	CT(64691)=(S=13242088,E=13242090,C=0)
	CT(64692)=(S=13242091,E=13242095,C=1)
	CT(64693)=(S=13242096,E=13242101,C=0)
	CT(64694)=(S=13242102,E=13242102,C=1)
	CT(64695)=(S=13242103,E=13242106,C=0)
	CT(64696)=(S=13242107,E=13242107,C=1)
	CT(64697)=(S=13242108,E=13242110,C=73)
	CT(64698)=(S=13242111,E=13242111,C=74)
	CT(64699)=(S=13242112,E=13243391,C=2)
	CT(64700)=(S=13243392,E=13243399,C=73)
	CT(64701)=(S=13243400,E=13243423,C=0)
	CT(64702)=(S=13243424,E=13243457,C=73)
	CT(64703)=(S=13243458,E=13243458,C=1)
	CT(64704)=(S=13243459,E=13243467,C=0)
	CT(64705)=(S=13243468,E=13243470,C=73)
	CT(64706)=(S=13243471,E=13243471,C=1)
	CT(64707)=(S=13243472,E=13243472,C=73)
	CT(64708)=(S=13243473,E=13243473,C=0)
	CT(64709)=(S=13243474,E=13243478,C=6)
	CT(64710)=(S=13243479,E=13243482,C=1)
	CT(64711)=(S=13243483,E=13243485,C=73)
	CT(64712)=(S=13243486,E=13243487,C=1)
	CT(64713)=(S=13243488,E=13243489,C=73)
	CT(64714)=(S=13243490,E=13243490,C=8)
	CT(64715)=(S=13243491,E=13243491,C=6)
	CT(64716)=(S=13243492,E=13243493,C=8)
	CT(64717)=(S=13243494,E=13243496,C=73)
	CT(64718)=(S=13243497,E=13243497,C=3)
	CT(64719)=(S=13243498,E=13243501,C=74)
	CT(64720)=(S=13243502,E=13243502,C=2)
	CT(64721)=(S=13243503,E=13243503,C=8)
	CT(64722)=(S=13243504,E=13243504,C=159)
	CT(64723)=(S=13243505,E=13243505,C=73)
	CT(64724)=(S=13243506,E=13243506,C=1)
	CT(64725)=(S=13243507,E=13243508,C=2)
	CT(64726)=(S=13243509,E=13243509,C=1)
	CT(64727)=(S=13243510,E=13243510,C=8)
	CT(64728)=(S=13243511,E=13243511,C=6)
	CT(64729)=(S=13243512,E=13243512,C=1)
	CT(64730)=(S=13243513,E=13243514,C=73)
	CT(64731)=(S=13243515,E=13243516,C=2)
	CT(64732)=(S=13243517,E=13243519,C=1)
	CT(64733)=(S=13243520,E=13243655,C=6)
	CT(64734)=(S=13243656,E=13243663,C=0)
	CT(64735)=(S=13243664,E=13243679,C=73)
	CT(64736)=(S=13243680,E=13243687,C=156)
	CT(64737)=(S=13243712,E=13243743,C=73)
	CT(64738)=(S=13243744,E=13243775,C=92)
	CT(64739)=(S=13243776,E=13243776,C=73)
	CT(64740)=(S=13243777,E=13243777,C=7)
	CT(64741)=(S=13243778,E=13243778,C=73)
	CT(64742)=(S=13243779,E=13243780,C=1)
	CT(64743)=(S=13243781,E=13243781,C=73)
	CT(64744)=(S=13243782,E=13243783,C=4)
	CT(64745)=(S=13243784,E=13243787,C=73)
	CT(64746)=(S=13243788,E=13243788,C=3)
	CT(64747)=(S=13243789,E=13243790,C=1)
	CT(64748)=(S=13243791,E=13243791,C=73)
	CT(64749)=(S=13243792,E=13243792,C=3)
	CT(64750)=(S=13243793,E=13243794,C=2)
	CT(64751)=(S=13243795,E=13243796,C=1)
	CT(64752)=(S=13243797,E=13243797,C=3)
	CT(64753)=(S=13243798,E=13243802,C=1)
	CT(64754)=(S=13243803,E=13243803,C=73)
	CT(64755)=(S=13243804,E=13243804,C=1)
	CT(64756)=(S=13243805,E=13243805,C=73)
	CT(64757)=(S=13243806,E=13243807,C=7)
	CT(64758)=(S=13243808,E=13243823,C=2)
	CT(64759)=(S=13243824,E=13243839,C=88)
	CT(64760)=(S=13243840,E=13243903,C=7)
	CT(64761)=(S=13243904,E=13243911,C=73)
	CT(64762)=(S=13243912,E=13243919,C=3)
	CT(64763)=(S=13243920,E=13243934,C=73)
	CT(64764)=(S=13243935,E=13243935,C=74)
	CT(64765)=(S=13243936,E=13243967,C=6)
	CT(64766)=(S=13243968,E=13244031,C=2)
	CT(64767)=(S=13244032,E=13244063,C=90)
	CT(64768)=(S=13244064,E=13244079,C=0)
	CT(64769)=(S=13244080,E=13244095,C=159)
	CT(64770)=(S=13244096,E=13244111,C=84)
	CT(64771)=(S=13244112,E=13244127,C=2)
	CT(64772)=(S=13244128,E=13244143,C=90)
	CT(64773)=(S=13244144,E=13244151,C=8)
	CT(64774)=(S=13244152,E=13244159,C=1)
	CT(64775)=(S=13244160,E=13245183,C=2)
	CT(64776)=(S=13245184,E=13245192,C=73)
	CT(64777)=(S=13245193,E=13245193,C=4)
	CT(64778)=(S=13245194,E=13245195,C=3)
	CT(64779)=(S=13245197,E=13245199,C=0)
	CT(64780)=(S=13245200,E=13245319,C=73)
	CT(64781)=(S=13245320,E=13245321,C=1)
	CT(64782)=(S=13245322,E=13245439,C=73)
	CT(64783)=(S=13245440,E=13245951,C=3)
	CT(64784)=(S=13245952,E=13246463,C=6)
	CT(64785)=(S=13246464,E=13247487,C=2)
	CT(64786)=(S=13247488,E=13247817,C=73)
	CT(64787)=(S=13247818,E=13247818,C=10)
	CT(64788)=(S=13247819,E=13247829,C=73)
	CT(64789)=(S=13247830,E=13247831,C=11)
	CT(64790)=(S=13247832,E=13247863,C=73)
	CT(64791)=(S=13247864,E=13247864,C=74)
	CT(64792)=(S=13247865,E=13247999,C=73)
	CT(64793)=(S=13248000,E=13248003,C=1)
	CT(64794)=(S=13248004,E=13248007,C=75)
	CT(64795)=(S=13248008,E=13248015,C=1)
	CT(64796)=(S=13248016,E=13248031,C=8)
	CT(64797)=(S=13248032,E=13248047,C=7)
	CT(64798)=(S=13248048,E=13248138,C=1)
	CT(64799)=(S=13248139,E=13248139,C=0)
	CT(64800)=(S=13248140,E=13248143,C=1)
	CT(64801)=(S=13248144,E=13248145,C=0)
	CT(64802)=(S=13248146,E=13248147,C=1)
	CT(64803)=(S=13248148,E=13248148,C=0)
	CT(64804)=(S=13248149,E=13248156,C=1)
	CT(64805)=(S=13248157,E=13248157,C=0)
	CT(64806)=(S=13248158,E=13248161,C=1)
	CT(64807)=(S=13248162,E=13248163,C=0)
	CT(64808)=(S=13248164,E=13248171,C=1)
	CT(64809)=(S=13248172,E=13248175,C=4)
	CT(64810)=(S=13248176,E=13248177,C=1)
	CT(64811)=(S=13248178,E=13248179,C=8)
	CT(64812)=(S=13248180,E=13248183,C=4)
	CT(64813)=(S=13248184,E=13248255,C=1)
	CT(64814)=(S=13248256,E=13248511,C=9)
	CT(64815)=(S=13248512,E=13248515,C=0)
	CT(64816)=(S=13248516,E=13248517,C=1)
	CT(64817)=(S=13248518,E=13248518,C=4)
	CT(64818)=(S=13248519,E=13248519,C=1)
	CT(64819)=(S=13248520,E=13248526,C=8)
	CT(64820)=(S=13248527,E=13248527,C=1)
	CT(64821)=(S=13248528,E=13248543,C=8)
	CT(64822)=(S=13248544,E=13248639,C=4)
	CT(64823)=(S=13248640,E=13248646,C=8)
	CT(64824)=(S=13248647,E=13248648,C=1)
	CT(64825)=(S=13248649,E=13248651,C=8)
	CT(64826)=(S=13248652,E=13248652,C=1)
	CT(64827)=(S=13248653,E=13248654,C=8)
	CT(64828)=(S=13248655,E=13248657,C=1)
	CT(64829)=(S=13248658,E=13248661,C=8)
	CT(64830)=(S=13248662,E=13248662,C=1)
	CT(64831)=(S=13248663,E=13248666,C=8)
	CT(64832)=(S=13248667,E=13248668,C=1)
	CT(64833)=(S=13248669,E=13248669,C=8)
	CT(64834)=(S=13248670,E=13248671,C=1)
	CT(64835)=(S=13248672,E=13248673,C=8)
	CT(64836)=(S=13248674,E=13248674,C=1)
	CT(64837)=(S=13248675,E=13248679,C=0)
	CT(64838)=(S=13248680,E=13248683,C=8)
	CT(64839)=(S=13248684,E=13248687,C=10)
	CT(64840)=(S=13248688,E=13248703,C=84)
	CT(64841)=(S=13248704,E=13248735,C=8)
	CT(64842)=(S=13248736,E=13248767,C=7)
	CT(64843)=(S=13248768,E=13248771,C=4)
	CT(64844)=(S=13248772,E=13248775,C=0)
	CT(64845)=(S=13248776,E=13248777,C=1)
	CT(64846)=(S=13248778,E=13248778,C=4)
	CT(64847)=(S=13248779,E=13248781,C=1)
	CT(64848)=(S=13248782,E=13248783,C=0)
	CT(64849)=(S=13248784,E=13248895,C=4)
	CT(64850)=(S=13248896,E=13248896,C=1)
	CT(64851)=(S=13248897,E=13248897,C=4)
	CT(64852)=(S=13248898,E=13248899,C=1)
	CT(64853)=(S=13248900,E=13248901,C=4)
	CT(64854)=(S=13248902,E=13248903,C=2)
	CT(64855)=(S=13248904,E=13248911,C=73)
	CT(64856)=(S=13248912,E=13248913,C=0)
	CT(64857)=(S=13248914,E=13248914,C=9)
	CT(64858)=(S=13248915,E=13248915,C=10)
	CT(64859)=(S=13248916,E=13248919,C=4)
	CT(64860)=(S=13248920,E=13248927,C=1)
	CT(64861)=(S=13248928,E=13248959,C=3)
	CT(64862)=(S=13248960,E=13248960,C=1)
	CT(64863)=(S=13248961,E=13248963,C=0)
	CT(64864)=(S=13248972,E=13248975,C=4)
	CT(64865)=(S=13248976,E=13248991,C=2)
	CT(64866)=(S=13248992,E=13249007,C=8)
	CT(64867)=(S=13249008,E=13249023,C=1)
	CT(64868)=(S=13249024,E=13249279,C=2)
	CT(64869)=(S=13249280,E=13249287,C=0)
	CT(64870)=(S=13249288,E=13249295,C=166)
	CT(64871)=(S=13249296,E=13249311,C=2)
	CT(64872)=(S=13249312,E=13249327,C=3)
	CT(64873)=(S=13249328,E=13249343,C=6)
	CT(64874)=(S=13249344,E=13249345,C=74)
	CT(64875)=(S=13249346,E=13249346,C=0)
	CT(64876)=(S=13249347,E=13249347,C=4)
	CT(64877)=(S=13249348,E=13249351,C=2)
	CT(64878)=(S=13249352,E=13249355,C=74)
	CT(64879)=(S=13249356,E=13249359,C=1)
	CT(64880)=(S=13249360,E=13249361,C=0)
	CT(64881)=(S=13249362,E=13249363,C=8)
	CT(64882)=(S=13249364,E=13249367,C=0)
	CT(64883)=(S=13249368,E=13249369,C=74)
	CT(64884)=(S=13249370,E=13249370,C=4)
	CT(64885)=(S=13249372,E=13249375,C=74)
	CT(64886)=(S=13249376,E=13249379,C=2)
	CT(64887)=(S=13249380,E=13249383,C=5)
	CT(64888)=(S=13249384,E=13249387,C=2)
	CT(64889)=(S=13249388,E=13249391,C=11)
	CT(64890)=(S=13249392,E=13249397,C=74)
	CT(64891)=(S=13249398,E=13249399,C=77)
	CT(64892)=(S=13249408,E=13249423,C=0)
	CT(64893)=(S=13249424,E=13249439,C=1)
	CT(64894)=(S=13249440,E=13249471,C=74)
	CT(64895)=(S=13249472,E=13249503,C=8)
	CT(64896)=(S=13249504,E=13249519,C=0)
	CT(64897)=(S=13249520,E=13249527,C=2)
	CT(64898)=(S=13249528,E=13249535,C=74)
	CT(64899)=(S=13249536,E=13249551,C=3)
	CT(64900)=(S=13249552,E=13249567,C=1)
	CT(64901)=(S=13249568,E=13249591,C=3)
	CT(64902)=(S=13249592,E=13249595,C=4)
	CT(64903)=(S=13249596,E=13249599,C=73)
	CT(64904)=(S=13249600,E=13249602,C=3)
	CT(64905)=(S=13249603,E=13249603,C=1)
	CT(64906)=(S=13249604,E=13249609,C=3)
	CT(64907)=(S=13249610,E=13249610,C=1)
	CT(64908)=(S=13249611,E=13249615,C=0)
	CT(64909)=(S=13249616,E=13249631,C=77)
	CT(64910)=(S=13249632,E=13249632,C=3)
	CT(64911)=(S=13249634,E=13249635,C=0)
	CT(64912)=(S=13249636,E=13249639,C=10)
	CT(64913)=(S=13249640,E=13249647,C=84)
	CT(64914)=(S=13249648,E=13249651,C=233)
	CT(64915)=(S=13249652,E=13249655,C=0)
	CT(64916)=(S=13249656,E=13249663,C=8)
	CT(64917)=(S=13249664,E=13249664,C=3)
	CT(64918)=(S=13249665,E=13249665,C=1)
	CT(64919)=(S=13249666,E=13249667,C=3)
	CT(64920)=(S=13249668,E=13249669,C=1)
	CT(64921)=(S=13249670,E=13249670,C=0)
	CT(64922)=(S=13249671,E=13249672,C=3)
	CT(64923)=(S=13249673,E=13249673,C=11)
	CT(64924)=(S=13249674,E=13249681,C=3)
	CT(64925)=(S=13249682,E=13249683,C=1)
	CT(64926)=(S=13249684,E=13249687,C=3)
	CT(64927)=(S=13249688,E=13249727,C=0)
	CT(64928)=(S=13249728,E=13249791,C=3)
	CT(64929)=(S=13249792,E=13249794,C=1)
	CT(64930)=(S=13249795,E=13249795,C=0)
	CT(64931)=(S=13249796,E=13249797,C=8)
	CT(64932)=(S=13249798,E=13249799,C=4)
	CT(64933)=(S=13249800,E=13249801,C=8)
	CT(64934)=(S=13249804,E=13249804,C=2)
	CT(64935)=(S=13249805,E=13249805,C=0)
	CT(64936)=(S=13249806,E=13249806,C=8)
	CT(64937)=(S=13249807,E=13249823,C=1)
	CT(64938)=(S=13249824,E=13249887,C=8)
	CT(64939)=(S=13249888,E=13249919,C=0)
	CT(64940)=(S=13249920,E=13249920,C=8)
	CT(64941)=(S=13249921,E=13249921,C=2)
	CT(64942)=(S=13249922,E=13249923,C=8)
	CT(64943)=(S=13249924,E=13249935,C=5)
	CT(64944)=(S=13249936,E=13249939,C=75)
	CT(64945)=(S=13249940,E=13249940,C=8)
	CT(64946)=(S=13249941,E=13249941,C=74)
	CT(64947)=(S=13249942,E=13249947,C=0)
	CT(64948)=(S=13249948,E=13249949,C=77)
	CT(64949)=(S=13249950,E=13249950,C=0)
	CT(64950)=(S=13249951,E=13249951,C=84)
	CT(64951)=(S=13249952,E=13249967,C=2)
	CT(64952)=(S=13249968,E=13250047,C=8)
	CT(64953)=(S=13250048,E=13250063,C=74)
	CT(64954)=(S=13250064,E=13250066,C=1)
	CT(64955)=(S=13250067,E=13250067,C=4)
	CT(64956)=(S=13250068,E=13250069,C=1)
	CT(64957)=(S=13250070,E=13250071,C=4)
	CT(64958)=(S=13250072,E=13250079,C=74)
	CT(64959)=(S=13250080,E=13250111,C=1)
	CT(64960)=(S=13250112,E=13250159,C=74)
	CT(64961)=(S=13250160,E=13250175,C=5)
	CT(64962)=(S=13250176,E=13250176,C=1)
	CT(64963)=(S=13250177,E=13250179,C=74)
	CT(64964)=(S=13250180,E=13250191,C=0)
	CT(64965)=(S=13250192,E=13250207,C=74)
	CT(64966)=(S=13250208,E=13250239,C=73)
	CT(64967)=(S=13250240,E=13250271,C=4)
	CT(64968)=(S=13250272,E=13250287,C=1)
	CT(64969)=(S=13250288,E=13250303,C=74)
	CT(64970)=(S=13250304,E=13250311,C=0)
	CT(64971)=(S=13250312,E=13250335,C=2)
	CT(64972)=(S=13250336,E=13250367,C=0)
	CT(64973)=(S=13250368,E=13250384,C=74)
	CT(64974)=(S=13250385,E=13250385,C=0)
	CT(64975)=(S=13250386,E=13250387,C=1)
	CT(64976)=(S=13250388,E=13250390,C=0)
	CT(64977)=(S=13250391,E=13250391,C=11)
	CT(64978)=(S=13250392,E=13250392,C=74)
	CT(64979)=(S=13250393,E=13250393,C=7)
	CT(64980)=(S=13250394,E=13250397,C=74)
	CT(64981)=(S=13250398,E=13250398,C=77)
	CT(64982)=(S=13250399,E=13250399,C=0)
	CT(64983)=(S=13250416,E=13250423,C=4)
	CT(64984)=(S=13250424,E=13250427,C=0)
	CT(64985)=(S=13250429,E=13250429,C=10)
	CT(64986)=(S=13250430,E=13250430,C=1)
	CT(64987)=(S=13250431,E=13250431,C=4)
	CT(64988)=(S=13250432,E=13250432,C=1)
	CT(64989)=(S=13250433,E=13250433,C=10)
	CT(64990)=(S=13250434,E=13250435,C=1)
	CT(64991)=(S=13250436,E=13250437,C=10)
	CT(64992)=(S=13250438,E=13250439,C=0)
	CT(64993)=(S=13250440,E=13250443,C=2)
	CT(64994)=(S=13250444,E=13250445,C=10)
	CT(64995)=(S=13250446,E=13250446,C=11)
	CT(64996)=(S=13250447,E=13250447,C=6)
	CT(64997)=(S=13250448,E=13250463,C=159)
	CT(64998)=(S=13250464,E=13250495,C=5)
	CT(64999)=(S=13250496,E=13250527,C=8)
	CT(65000)=(S=13250528,E=13250559,C=3)
	CT(65001)=(S=13250560,E=13250815,C=2)
	CT(65002)=(S=13250816,E=13251123,C=73)
	CT(65003)=(S=13251124,E=13251127,C=75)
	CT(65004)=(S=13251128,E=13251267,C=73)
	CT(65005)=(S=13251268,E=13251269,C=0)
	CT(65006)=(S=13251270,E=13251271,C=73)
	CT(65007)=(S=13251272,E=13251275,C=74)
	CT(65008)=(S=13251276,E=13251327,C=73)
	CT(65009)=(S=13251328,E=13251331,C=75)
	CT(65010)=(S=13251332,E=13251343,C=2)
	CT(65011)=(S=13251344,E=13251359,C=74)
	CT(65012)=(S=13251360,E=13251383,C=2)
	CT(65013)=(S=13251384,E=13251391,C=74)
	CT(65014)=(S=13251392,E=13251423,C=75)
	CT(65015)=(S=13251424,E=13251455,C=74)
	CT(65016)=(S=13251456,E=13251487,C=8)
	CT(65017)=(S=13251488,E=13251503,C=0)
	CT(65018)=(S=13251504,E=13251519,C=84)
	CT(65019)=(S=13251520,E=13251567,C=74)
	CT(65020)=(S=13251568,E=13251575,C=7)
	CT(65021)=(S=13251580,E=13251583,C=74)
	CT(65022)=(S=13251584,E=13251585,C=75)
	CT(65023)=(S=13251586,E=13251587,C=4)
	CT(65024)=(S=13251588,E=13251591,C=3)
	CT(65025)=(S=13251592,E=13251599,C=74)
	CT(65026)=(S=13251600,E=13251615,C=2)
	CT(65027)=(S=13251616,E=13251616,C=77)
	CT(65028)=(S=13251619,E=13251619,C=8)
	CT(65029)=(S=13251620,E=13251621,C=0)
	CT(65030)=(S=13251622,E=13251622,C=2)
	CT(65031)=(S=13251623,E=13251623,C=11)
	CT(65032)=(S=13251624,E=13251624,C=4)
	CT(65033)=(S=13251625,E=13251625,C=0)
	CT(65034)=(S=13251626,E=13251627,C=8)
	CT(65035)=(S=13251628,E=13251628,C=4)
	CT(65036)=(S=13251629,E=13251630,C=0)
	CT(65037)=(S=13251632,E=13251634,C=74)
	CT(65038)=(S=13251635,E=13251635,C=7)
	CT(65039)=(S=13251636,E=13251636,C=74)
	CT(65040)=(S=13251637,E=13251637,C=4)
	CT(65041)=(S=13251638,E=13251639,C=10)
	CT(65042)=(S=13251640,E=13251641,C=4)
	CT(65043)=(S=13251642,E=13251643,C=74)
	CT(65044)=(S=13251644,E=13251644,C=0)
	CT(65045)=(S=13251645,E=13251645,C=5)
	CT(65046)=(S=13251646,E=13251647,C=4)
	CT(65047)=(S=13251648,E=13251711,C=7)
	CT(65048)=(S=13251712,E=13251712,C=4)
	CT(65049)=(S=13251713,E=13251713,C=0)
	CT(65050)=(S=13251714,E=13251714,C=4)
	CT(65051)=(S=13251715,E=13251716,C=74)
	CT(65052)=(S=13251717,E=13251717,C=76)
	CT(65053)=(S=13251718,E=13251718,C=4)
	CT(65054)=(S=13251719,E=13251719,C=0)
	CT(65055)=(S=13251720,E=13251720,C=8)
	CT(65056)=(S=13251721,E=13251721,C=74)
	CT(65057)=(S=13251722,E=13251723,C=84)
	CT(65058)=(S=13251724,E=13251725,C=74)
	CT(65059)=(S=13251726,E=13251729,C=0)
	CT(65060)=(S=13251730,E=13251731,C=74)
	CT(65061)=(S=13251732,E=13251732,C=4)
	CT(65062)=(S=13251733,E=13251733,C=5)
	CT(65063)=(S=13251734,E=13251734,C=4)
	CT(65064)=(S=13251735,E=13251735,C=2)
	CT(65065)=(S=13251736,E=13251736,C=73)
	CT(65066)=(S=13251737,E=13251737,C=4)
	CT(65067)=(S=13251738,E=13251738,C=2)
	CT(65068)=(S=13251739,E=13251739,C=73)
	CT(65069)=(S=13251740,E=13251740,C=74)
	CT(65070)=(S=13251742,E=13251743,C=0)
	CT(65071)=(S=13251744,E=13251759,C=10)
	CT(65072)=(S=13251760,E=13251775,C=2)
	CT(65073)=(S=13251776,E=13251807,C=8)
	CT(65074)=(S=13251808,E=13251843,C=75)
	CT(65075)=(S=13251844,E=13251847,C=0)
	CT(65076)=(S=13251848,E=13251855,C=4)
	CT(65077)=(S=13251856,E=13251871,C=2)
	CT(65078)=(S=13251872,E=13251903,C=0)
	CT(65079)=(S=13251904,E=13251951,C=4)
	CT(65080)=(S=13251952,E=13251967,C=2)
	CT(65081)=(S=13251968,E=13251982,C=8)
	CT(65082)=(S=13252000,E=13252015,C=84)
	CT(65083)=(S=13252016,E=13252047,C=0)
	CT(65084)=(S=13252048,E=13252063,C=2)
	CT(65085)=(S=13252064,E=13252095,C=74)
	CT(65086)=(S=13252096,E=13252351,C=4)
	CT(65087)=(S=13252352,E=13252415,C=8)
	CT(65088)=(S=13252416,E=13252447,C=7)
	CT(65089)=(S=13252448,E=13252463,C=73)
	CT(65090)=(S=13252464,E=13252479,C=2)
	CT(65091)=(S=13252480,E=13252483,C=77)
	CT(65092)=(S=13252484,E=13252487,C=11)
	CT(65093)=(S=13252488,E=13252491,C=2)
	CT(65094)=(S=13252492,E=13252495,C=3)
	CT(65095)=(S=13252496,E=13252511,C=0)
	CT(65096)=(S=13252512,E=13252527,C=74)
	CT(65097)=(S=13252528,E=13252543,C=92)
	CT(65098)=(S=13252544,E=13252575,C=2)
	CT(65099)=(S=13252576,E=13252607,C=9)
	CT(65100)=(S=13252608,E=13252611,C=4)
	CT(65101)=(S=13252612,E=13252615,C=84)
	CT(65102)=(S=13252616,E=13252623,C=8)
	CT(65103)=(S=13252624,E=13252639,C=2)
	CT(65104)=(S=13252640,E=13252663,C=73)
	CT(65105)=(S=13252664,E=13252664,C=7)
	CT(65106)=(S=13252665,E=13252665,C=11)
	CT(65107)=(S=13252666,E=13252667,C=4)
	CT(65108)=(S=13252668,E=13252671,C=0)
	CT(65109)=(S=13252672,E=13252687,C=2)
	CT(65110)=(S=13252688,E=13252703,C=5)
	CT(65111)=(S=13252704,E=13252735,C=4)
	CT(65112)=(S=13252736,E=13252767,C=7)
	CT(65113)=(S=13252768,E=13252783,C=74)
	CT(65114)=(S=13252784,E=13252799,C=89)
	CT(65115)=(S=13252800,E=13252863,C=4)
	CT(65116)=(S=13252864,E=13252895,C=74)
	CT(65117)=(S=13252896,E=13252991,C=10)
	CT(65118)=(S=13252992,E=13253055,C=3)
	CT(65119)=(S=13253084,E=13253087,C=8)
	CT(65120)=(S=13253088,E=13253103,C=2)
	CT(65121)=(S=13253104,E=13253120,C=1)
	CT(65122)=(S=13253121,E=13253121,C=2)
	CT(65123)=(S=13253122,E=13253122,C=10)
	CT(65124)=(S=13253123,E=13253123,C=0)
	CT(65125)=(S=13253124,E=13253127,C=2)
	CT(65126)=(S=13253128,E=13253135,C=7)
	CT(65127)=(S=13253136,E=13253151,C=2)
	CT(65128)=(S=13253152,E=13253183,C=0)
	CT(65129)=(S=13253184,E=13253199,C=74)
	CT(65130)=(S=13253200,E=13253215,C=5)
	CT(65131)=(S=13253216,E=13253219,C=86)
	CT(65132)=(S=13253220,E=13253221,C=0)
	CT(65133)=(S=13253222,E=13253223,C=4)
	CT(65134)=(S=13253236,E=13253239,C=2)
	CT(65135)=(S=13253240,E=13253243,C=4)
	CT(65136)=(S=13253244,E=13253244,C=74)
	CT(65137)=(S=13253245,E=13253245,C=4)
	CT(65138)=(S=13253246,E=13253246,C=3)
	CT(65139)=(S=13253247,E=13253247,C=4)
	CT(65140)=(S=13253248,E=13253251,C=76)
	CT(65141)=(S=13253252,E=13253255,C=4)
	CT(65142)=(S=13253256,E=13253263,C=0)
	CT(65143)=(S=13253264,E=13253279,C=2)
	CT(65144)=(S=13253280,E=13253303,C=74)
	CT(65145)=(S=13253304,E=13253311,C=8)
	CT(65146)=(S=13253312,E=13253343,C=74)
	CT(65147)=(S=13253344,E=13253347,C=0)
	CT(65148)=(S=13253348,E=13253348,C=4)
	CT(65149)=(S=13253349,E=13253351,C=0)
	CT(65150)=(S=13253352,E=13253352,C=4)
	CT(65151)=(S=13253354,E=13253354,C=2)
	CT(65152)=(S=13253355,E=13253355,C=0)
	CT(65153)=(S=13253356,E=13253357,C=10)
	CT(65154)=(S=13253358,E=13253359,C=74)
	CT(65155)=(S=13253360,E=13253361,C=0)
	CT(65156)=(S=13253362,E=13253363,C=74)
	CT(65157)=(S=13253364,E=13253364,C=3)
	CT(65158)=(S=13253365,E=13253365,C=11)
	CT(65159)=(S=13253366,E=13253367,C=0)
	CT(65160)=(S=13253376,E=13253376,C=1)
	CT(65161)=(S=13253378,E=13253379,C=4)
	CT(65162)=(S=13253380,E=13253383,C=173)
	CT(65163)=(S=13253384,E=13253439,C=0)
	CT(65164)=(S=13253440,E=13253471,C=77)
	CT(65165)=(S=13253472,E=13253487,C=0)
	CT(65166)=(S=13253488,E=13253503,C=2)
	CT(65167)=(S=13253504,E=13253519,C=84)
	CT(65168)=(S=13253520,E=13253527,C=2)
	CT(65169)=(S=13253528,E=13253535,C=8)
	CT(65170)=(S=13253536,E=13253551,C=74)
	CT(65171)=(S=13253552,E=13253567,C=2)
	CT(65172)=(S=13253568,E=13253583,C=74)
	CT(65173)=(S=13253584,E=13253587,C=84)
	CT(65174)=(S=13253588,E=13253591,C=1)
	CT(65175)=(S=13253592,E=13253599,C=6)
	CT(65176)=(S=13253600,E=13253600,C=73)
	CT(65177)=(S=13253601,E=13253603,C=0)
	CT(65178)=(S=13253604,E=13253604,C=4)
	CT(65179)=(S=13253605,E=13253605,C=0)
	CT(65180)=(S=13253606,E=13253606,C=157)
	CT(65181)=(S=13253607,E=13253607,C=4)
	CT(65182)=(S=13253608,E=13253609,C=1)
	CT(65183)=(S=13253610,E=13253611,C=2)
	CT(65184)=(S=13253612,E=13253612,C=1)
	CT(65185)=(S=13253613,E=13253613,C=0)
	CT(65186)=(S=13253614,E=13253615,C=11)
	CT(65187)=(S=13253617,E=13253619,C=0)
	CT(65188)=(S=13253620,E=13253621,C=4)
	CT(65189)=(S=13253622,E=13253623,C=0)
	CT(65190)=(S=13253624,E=13253624,C=4)
	CT(65191)=(S=13253625,E=13253625,C=8)
	CT(65192)=(S=13253626,E=13253627,C=9)
	CT(65193)=(S=13253628,E=13253629,C=11)
	CT(65194)=(S=13253630,E=13253630,C=77)
	CT(65195)=(S=13253631,E=13253631,C=74)
	CT(65196)=(S=13253632,E=13253639,C=113)
	CT(65197)=(S=13253640,E=13253647,C=10)
	CT(65198)=(S=13253648,E=13253679,C=2)
	CT(65199)=(S=13253680,E=13253687,C=1)
	CT(65200)=(S=13253688,E=13253691,C=5)
	CT(65201)=(S=13253692,E=13253695,C=4)
	CT(65202)=(S=13253696,E=13253727,C=0)
	CT(65203)=(S=13253728,E=13253735,C=1)
	CT(65204)=(S=13253736,E=13253743,C=11)
	CT(65205)=(S=13253744,E=13253759,C=1)
	CT(65206)=(S=13253760,E=13253763,C=4)
	CT(65207)=(S=13253764,E=13253791,C=1)
	CT(65208)=(S=13253792,E=13253823,C=2)
	CT(65209)=(S=13253824,E=13253839,C=3)
	CT(65210)=(S=13253840,E=13253855,C=4)
	CT(65211)=(S=13253856,E=13253887,C=8)
	CT(65212)=(S=13253888,E=13253888,C=76)
	CT(65213)=(S=13253889,E=13253889,C=2)
	CT(65214)=(S=13253890,E=13253891,C=73)
	CT(65215)=(S=13253892,E=13253895,C=0)
	CT(65216)=(S=13253896,E=13253899,C=4)
	CT(65217)=(S=13253900,E=13253903,C=0)
	CT(65218)=(S=13253904,E=13253919,C=2)
	CT(65219)=(S=13253920,E=13253951,C=77)
	CT(65220)=(S=13253984,E=13253984,C=74)
	CT(65221)=(S=13253985,E=13253985,C=92)
	CT(65222)=(S=13253986,E=13253989,C=74)
	CT(65223)=(S=13253990,E=13253990,C=4)
	CT(65224)=(S=13253991,E=13253991,C=8)
	CT(65225)=(S=13253992,E=13253993,C=74)
	CT(65226)=(S=13253994,E=13253995,C=7)
	CT(65227)=(S=13253996,E=13253996,C=10)
	CT(65228)=(S=13253997,E=13253997,C=0)
	CT(65229)=(S=13253998,E=13253998,C=7)
	CT(65230)=(S=13253999,E=13253999,C=4)
	CT(65231)=(S=13254000,E=13254001,C=74)
	CT(65232)=(S=13254002,E=13254003,C=73)
	CT(65233)=(S=13254004,E=13254004,C=0)
	CT(65234)=(S=13254005,E=13254005,C=4)
	CT(65235)=(S=13254006,E=13254006,C=10)
	CT(65236)=(S=13254007,E=13254010,C=0)
	CT(65237)=(S=13254012,E=13254014,C=74)
	CT(65238)=(S=13254016,E=13254143,C=0)
	CT(65239)=(S=13254144,E=13254151,C=91)
	CT(65240)=(S=13254152,E=13254175,C=74)
	CT(65241)=(S=13254176,E=13254207,C=0)
	CT(65242)=(S=13254208,E=13254239,C=4)
	CT(65243)=(S=13254240,E=13254255,C=154)
	CT(65244)=(S=13254256,E=13254259,C=1)
	CT(65245)=(S=13254260,E=13254261,C=4)
	CT(65246)=(S=13254262,E=13254271,C=91)
	CT(65247)=(S=13254272,E=13254303,C=0)
	CT(65248)=(S=13254304,E=13254335,C=2)
	CT(65249)=(S=13254336,E=13254367,C=8)
	CT(65250)=(S=13254368,E=13254383,C=4)
	CT(65251)=(S=13254384,E=13254387,C=0)
	CT(65252)=(S=13254388,E=13254388,C=4)
	CT(65253)=(S=13254389,E=13254389,C=73)
	CT(65254)=(S=13254390,E=13254391,C=4)
	CT(65255)=(S=13254392,E=13254396,C=1)
	CT(65256)=(S=13254397,E=13254397,C=2)
	CT(65257)=(S=13254398,E=13254398,C=0)
	CT(65258)=(S=13254399,E=13254399,C=1)
	CT(65259)=(S=13254400,E=13254431,C=8)
	CT(65260)=(S=13254432,E=13254479,C=0)
	CT(65261)=(S=13254481,E=13254495,C=1)
	CT(65262)=(S=13254496,E=13254527,C=4)
	CT(65263)=(S=13254528,E=13254559,C=8)
	CT(65264)=(S=13254560,E=13254591,C=1)
	CT(65265)=(S=13254592,E=13254623,C=77)
	CT(65266)=(S=13254624,E=13254639,C=7)
	CT(65267)=(S=13254640,E=13254647,C=75)
	CT(65268)=(S=13254648,E=13254651,C=1)
	CT(65269)=(S=13254652,E=13254652,C=4)
	CT(65270)=(S=13254654,E=13254655,C=192)
	CT(65271)=(S=13254656,E=13254911,C=2)
	CT(65272)=(S=13254912,E=13254921,C=1)
	CT(65273)=(S=13254922,E=13254923,C=84)
	CT(65274)=(S=13254924,E=13254943,C=0)
	CT(65275)=(S=13254944,E=13254975,C=234)
	CT(65276)=(S=13254976,E=13255007,C=0)
	CT(65277)=(S=13255024,E=13255039,C=74)
	CT(65278)=(S=13255040,E=13255071,C=4)
	CT(65279)=(S=13255072,E=13255079,C=73)
	CT(65280)=(S=13255080,E=13255087,C=84)
	CT(65281)=(S=13255088,E=13255103,C=10)
	CT(65282)=(S=13255104,E=13255135,C=8)
	CT(65283)=(S=13255136,E=13255151,C=74)
	CT(65284)=(S=13255152,E=13255159,C=7)
	CT(65285)=(S=13255160,E=13255423,C=8)
	CT(65286)=(S=13255424,E=13255427,C=1)
	CT(65287)=(S=13255428,E=13255431,C=4)
	CT(65288)=(S=13255432,E=13255439,C=74)
	CT(65289)=(S=13255440,E=13255455,C=2)
	CT(65290)=(S=13255456,E=13255471,C=74)
	CT(65291)=(S=13255472,E=13255487,C=2)
	CT(65292)=(S=13255488,E=13255551,C=0)
	CT(65293)=(S=13255552,E=13255743,C=8)
	CT(65294)=(S=13255744,E=13255759,C=0)
	CT(65295)=(S=13255760,E=13255775,C=73)
	CT(65296)=(S=13255776,E=13255807,C=0)
	CT(65297)=(S=13255808,E=13255839,C=4)
	CT(65298)=(S=13255840,E=13255855,C=0)
	CT(65299)=(S=13255856,E=13255871,C=7)
	CT(65300)=(S=13255872,E=13255887,C=8)
	CT(65301)=(S=13255888,E=13255903,C=7)
	CT(65302)=(S=13255904,E=13255935,C=6)
	CT(65303)=(S=13255936,E=13255939,C=8)
	CT(65304)=(S=13255940,E=13255943,C=1)
	CT(65305)=(S=13255944,E=13255951,C=77)
	CT(65306)=(S=13255952,E=13255967,C=1)
	CT(65307)=(S=13255968,E=13255999,C=77)
	CT(65308)=(S=13256000,E=13256031,C=8)
	CT(65309)=(S=13256032,E=13256047,C=74)
	CT(65310)=(S=13256048,E=13256063,C=2)
	CT(65311)=(S=13256064,E=13256067,C=8)
	CT(65312)=(S=13256068,E=13256071,C=0)
	CT(65313)=(S=13256072,E=13256079,C=3)
	CT(65314)=(S=13256080,E=13256095,C=7)
	CT(65315)=(S=13256096,E=13256127,C=10)
	CT(65316)=(S=13256128,E=13256143,C=156)
	CT(65317)=(S=13256144,E=13256159,C=0)
	CT(65318)=(S=13256160,E=13256175,C=2)
	CT(65319)=(S=13256176,E=13256191,C=8)
	CT(65320)=(S=13256192,E=13256223,C=1)
	CT(65321)=(S=13256224,E=13256239,C=92)
	CT(65322)=(S=13256240,E=13256255,C=74)
	CT(65323)=(S=13256256,E=13256287,C=75)
	CT(65324)=(S=13256288,E=13256303,C=1)
	CT(65325)=(S=13256304,E=13256319,C=196)
	CT(65326)=(S=13256320,E=13256321,C=0)
	CT(65327)=(S=13256322,E=13256322,C=73)
	CT(65328)=(S=13256323,E=13256323,C=4)
	CT(65329)=(S=13256324,E=13256328,C=74)
	CT(65330)=(S=13256329,E=13256329,C=0)
	CT(65331)=(S=13256330,E=13256330,C=74)
	CT(65332)=(S=13256331,E=13256331,C=0)
	CT(65333)=(S=13256332,E=13256332,C=2)
	CT(65334)=(S=13256333,E=13256335,C=0)
	CT(65335)=(S=13256336,E=13256351,C=77)
	CT(65336)=(S=13256352,E=13256367,C=8)
	CT(65337)=(S=13256368,E=13256383,C=2)
	CT(65338)=(S=13256384,E=13256399,C=1)
	CT(65339)=(S=13256400,E=13256447,C=2)
	CT(65340)=(S=13256448,E=13256479,C=4)
	CT(65341)=(S=13256496,E=13256543,C=2)
	CT(65342)=(S=13256544,E=13256559,C=5)
	CT(65343)=(S=13256560,E=13256575,C=3)
	CT(65344)=(S=13256576,E=13256607,C=4)
	CT(65345)=(S=13256608,E=13256623,C=0)
	CT(65346)=(S=13256624,E=13256639,C=10)
	CT(65347)=(S=13256640,E=13256735,C=8)
	CT(65348)=(S=13256736,E=13256743,C=0)
	CT(65349)=(S=13256744,E=13256751,C=1)
	CT(65350)=(S=13256752,E=13256783,C=2)
	CT(65351)=(S=13256784,E=13256799,C=1)
	CT(65352)=(S=13256800,E=13256815,C=29)
	CT(65353)=(S=13256832,E=13256895,C=0)
	CT(65354)=(S=13256896,E=13256927,C=74)
	CT(65355)=(S=13256936,E=13256943,C=84)
	CT(65356)=(S=13256944,E=13256951,C=92)
	CT(65357)=(S=13256952,E=13256955,C=4)
	CT(65358)=(S=13256956,E=13256963,C=8)
	CT(65359)=(S=13256964,E=13256967,C=84)
	CT(65360)=(S=13256968,E=13256975,C=5)
	CT(65361)=(S=13256976,E=13256991,C=74)
	CT(65362)=(S=13256992,E=13257023,C=7)
	CT(65363)=(S=13257024,E=13257055,C=2)
	CT(65364)=(S=13257056,E=13257087,C=74)
	CT(65365)=(S=13257088,E=13257091,C=1)
	CT(65366)=(S=13257092,E=13257095,C=6)
	CT(65367)=(S=13257096,E=13257103,C=2)
	CT(65368)=(S=13257104,E=13257119,C=0)
	CT(65369)=(S=13257120,E=13257151,C=10)
	CT(65370)=(S=13257152,E=13257167,C=73)
	CT(65371)=(S=13257168,E=13257183,C=2)
	CT(65372)=(S=13257184,E=13257199,C=74)
	CT(65373)=(S=13257216,E=13257219,C=8)
	CT(65374)=(S=13257220,E=13257223,C=2)
	CT(65375)=(S=13257224,E=13257231,C=1)
	CT(65376)=(S=13257232,E=13257247,C=2)
	CT(65377)=(S=13257248,E=13257248,C=0)
	CT(65378)=(S=13257249,E=13257249,C=73)
	CT(65379)=(S=13257250,E=13257250,C=0)
	CT(65380)=(S=13257251,E=13257251,C=8)
	CT(65381)=(S=13257253,E=13257253,C=7)
	CT(65382)=(S=13257254,E=13257255,C=73)
	CT(65383)=(S=13257256,E=13257257,C=8)
	CT(65384)=(S=13257259,E=13257259,C=74)
	CT(65385)=(S=13257260,E=13257260,C=0)
	CT(65386)=(S=13257261,E=13257261,C=74)
	CT(65387)=(S=13257262,E=13257262,C=0)
	CT(65388)=(S=13257263,E=13257263,C=2)
	CT(65389)=(S=13257264,E=13257265,C=0)
	CT(65390)=(S=13257266,E=13257267,C=8)
	CT(65391)=(S=13257269,E=13257270,C=4)
	CT(65392)=(S=13257272,E=13257272,C=11)
	CT(65393)=(S=13257273,E=13257273,C=0)
	CT(65394)=(S=13257274,E=13257275,C=11)
	CT(65395)=(S=13257276,E=13257276,C=73)
	CT(65396)=(S=13257277,E=13257277,C=0)
	CT(65397)=(S=13257278,E=13257279,C=7)
	CT(65398)=(S=13257280,E=13257287,C=0)
	CT(65399)=(S=13257288,E=13257295,C=74)
	CT(65400)=(S=13257296,E=13257311,C=1)
	CT(65401)=(S=13257312,E=13257343,C=8)
	CT(65402)=(S=13257344,E=13257407,C=0)
	CT(65403)=(S=13257408,E=13257447,C=73)
	CT(65404)=(S=13257456,E=13257463,C=84)
	CT(65405)=(S=13257464,E=13257467,C=10)
	CT(65406)=(S=13257468,E=13257468,C=0)
	CT(65407)=(S=13257469,E=13257469,C=2)
	CT(65408)=(S=13257470,E=13257471,C=1)
	CT(65409)=(S=13257472,E=13257475,C=8)
	CT(65410)=(S=13257476,E=13257479,C=5)
	CT(65411)=(S=13257480,E=13257487,C=2)
	CT(65412)=(S=13257488,E=13257503,C=74)
	CT(65413)=(S=13257504,E=13257535,C=5)
	CT(65414)=(S=13257536,E=13257567,C=8)
	CT(65415)=(S=13257568,E=13257583,C=74)
	CT(65416)=(S=13257584,E=13257599,C=2)
	CT(65417)=(S=13257600,E=13257663,C=5)
	CT(65418)=(S=13257664,E=13257679,C=4)
	CT(65419)=(S=13257680,E=13257695,C=1)
	CT(65420)=(S=13257696,E=13257711,C=2)
	CT(65421)=(S=13257712,E=13257719,C=5)
	CT(65422)=(S=13257720,E=13257723,C=87)
	CT(65423)=(S=13257724,E=13257727,C=1)
	CT(65424)=(S=13257728,E=13257855,C=8)
	CT(65425)=(S=13257856,E=13257919,C=0)
	CT(65426)=(S=13257920,E=13257951,C=2)
	CT(65427)=(S=13257952,E=13257967,C=5)
	CT(65428)=(S=13257968,E=13257975,C=7)
	CT(65429)=(S=13257976,E=13257979,C=0)
	CT(65430)=(S=13257980,E=13257983,C=1)
	CT(65431)=(S=13257984,E=13258047,C=8)
	CT(65432)=(S=13258048,E=13258063,C=74)
	CT(65433)=(S=13258064,E=13258071,C=1)
	CT(65434)=(S=13258072,E=13258075,C=0)
	CT(65435)=(S=13258076,E=13258079,C=1)
	CT(65436)=(S=13258080,E=13258111,C=74)
	CT(65437)=(S=13258112,E=13258119,C=8)
	CT(65438)=(S=13258120,E=13258120,C=0)
	CT(65439)=(S=13258121,E=13258121,C=4)
	CT(65440)=(S=13258122,E=13258123,C=10)
	CT(65441)=(S=13258124,E=13258247,C=8)
	CT(65442)=(S=13258248,E=13258255,C=1)
	CT(65443)=(S=13258256,E=13258271,C=7)
	CT(65444)=(S=13258272,E=13258287,C=0)
	CT(65445)=(S=13258288,E=13258303,C=7)
	CT(65446)=(S=13258304,E=13258367,C=10)
	CT(65447)=(S=13258368,E=13258399,C=73)
	CT(65448)=(S=13258400,E=13258415,C=4)
	CT(65449)=(S=13258416,E=13258431,C=2)
	CT(65450)=(S=13258432,E=13258447,C=74)
	CT(65451)=(S=13258448,E=13258463,C=2)
	CT(65452)=(S=13258464,E=13258471,C=11)
	CT(65453)=(S=13258472,E=13258479,C=4)
	CT(65454)=(S=13258480,E=13258487,C=73)
	CT(65455)=(S=13258488,E=13258490,C=2)
	CT(65456)=(S=13258491,E=13258491,C=4)
	CT(65457)=(S=13258492,E=13258495,C=8)
	CT(65458)=(S=13258496,E=13258511,C=2)
	CT(65459)=(S=13258512,E=13258519,C=84)
	CT(65460)=(S=13258520,E=13258527,C=86)
	CT(65461)=(S=13258528,E=13258559,C=75)
	CT(65462)=(S=13258560,E=13258623,C=8)
	CT(65463)=(S=13258624,E=13258655,C=2)
	CT(65464)=(S=13258656,E=13258671,C=0)
	CT(65465)=(S=13258672,E=13258719,C=7)
	CT(65466)=(S=13258720,E=13258727,C=1)
	CT(65467)=(S=13258728,E=13258735,C=11)
	CT(65468)=(S=13258736,E=13258743,C=2)
	CT(65469)=(S=13258744,E=13258747,C=1)
	CT(65470)=(S=13258748,E=13258783,C=8)
	CT(65471)=(S=13258784,E=13258799,C=160)
	CT(65472)=(S=13258800,E=13258815,C=4)
	CT(65473)=(S=13258816,E=13258847,C=0)
	CT(65474)=(S=13258848,E=13258855,C=2)
	CT(65475)=(S=13258856,E=13258863,C=9)
	CT(65476)=(S=13258864,E=13258879,C=74)
	CT(65477)=(S=13258880,E=13258895,C=8)
	CT(65478)=(S=13258896,E=13258903,C=0)
	CT(65479)=(S=13258904,E=13258911,C=10)
	CT(65480)=(S=13258912,E=13258927,C=7)
	CT(65481)=(S=13258928,E=13258943,C=0)
	CT(65482)=(S=13258944,E=13258959,C=1)
	CT(65483)=(S=13258960,E=13258975,C=74)
	CT(65484)=(S=13258976,E=13259007,C=3)
	CT(65485)=(S=13259008,E=13259011,C=1)
	CT(65486)=(S=13259012,E=13259015,C=0)
	CT(65487)=(S=13259016,E=13259023,C=2)
	CT(65488)=(S=13259024,E=13259039,C=0)
	CT(65489)=(S=13259040,E=13259071,C=74)
	CT(65490)=(S=13259072,E=13259087,C=0)
	CT(65491)=(S=13259088,E=13259103,C=2)
	CT(65492)=(S=13259104,E=13259135,C=0)
	CT(65493)=(S=13259136,E=13259167,C=4)
	CT(65494)=(S=13259168,E=13259183,C=10)
	CT(65495)=(S=13259200,E=13259215,C=4)
	CT(65496)=(S=13259216,E=13259231,C=0)
	CT(65497)=(S=13259232,E=13259535,C=8)
	CT(65498)=(S=13259536,E=13259583,C=4)
	CT(65499)=(S=13259584,E=13259615,C=0)
	CT(65500)=(S=13259616,E=13259631,C=7)
	CT(65501)=(S=13259632,E=13259639,C=0)
	CT(65502)=(S=13259648,E=13259679,C=2)
	CT(65503)=(S=13259680,E=13259695,C=77)
	CT(65504)=(S=13259696,E=13259703,C=0)
	CT(65505)=(S=13259704,E=13259711,C=2)
	CT(65506)=(S=13259712,E=13259743,C=8)
	CT(65507)=(S=13259744,E=13259759,C=2)
	CT(65508)=(S=13259760,E=13259767,C=8)
	CT(65509)=(S=13259768,E=13259771,C=7)
	CT(65510)=(S=13259772,E=13259793,C=1)
	CT(65511)=(S=13259794,E=13259795,C=2)
	CT(65512)=(S=13259799,E=13259799,C=10)
	CT(65513)=(S=13259800,E=13259807,C=1)
	CT(65514)=(S=13259808,E=13259823,C=84)
	CT(65515)=(S=13259824,E=13259839,C=2)
	CT(65516)=(S=13259840,E=13259847,C=84)
	CT(65517)=(S=13259848,E=13259855,C=86)
	CT(65518)=(S=13259856,E=13259871,C=2)
	CT(65519)=(S=13259872,E=13259903,C=10)
	CT(65520)=(S=13259904,E=13260239,C=8)
	CT(65521)=(S=13260240,E=13260255,C=1)
	CT(65522)=(S=13260256,E=13260271,C=8)
	CT(65523)=(S=13260272,E=13260279,C=0)
	CT(65524)=(S=13260280,E=13260291,C=8)
	CT(65525)=(S=13260292,E=13260295,C=0)
	CT(65526)=(S=13260296,E=13260303,C=6)
	CT(65527)=(S=13260304,E=13260319,C=89)
	CT(65528)=(S=13260320,E=13260343,C=0)
	CT(65529)=(S=13260344,E=13260351,C=2)
	CT(65530)=(S=13260352,E=13260383,C=8)
	CT(65531)=(S=13260384,E=13260399,C=73)
	CT(65532)=(S=13260400,E=13260407,C=0)
	CT(65533)=(S=13260408,E=13260415,C=2)
	CT(65534)=(S=13260416,E=13260479,C=87)
	CT(65535)=(S=13260480,E=13260495,C=10)
	CT(65536)=(S=13260496,E=13260503,C=0)
	CT(65537)=(S=13260504,E=13260511,C=84)
	CT(65538)=(S=13260512,E=13260535,C=2)
	CT(65539)=(S=13260536,E=13260536,C=7)
	CT(65540)=(S=13260537,E=13260537,C=1)
	CT(65541)=(S=13260538,E=13260538,C=0)
	CT(65542)=(S=13260539,E=13260539,C=4)
	CT(65543)=(S=13260540,E=13260543,C=1)
	CT(65544)=(S=13260544,E=13260547,C=8)
	CT(65545)=(S=13260548,E=13260575,C=0)
	CT(65546)=(S=13260576,E=13260607,C=4)
	CT(65547)=(S=13260608,E=13260623,C=2)
	CT(65548)=(S=13260624,E=13260639,C=1)
	CT(65549)=(S=13260640,E=13260671,C=5)
	CT(65550)=(S=13260672,E=13260703,C=90)
	CT(65551)=(S=13260704,E=13260719,C=0)
	CT(65552)=(S=13260720,E=13260751,C=74)
	CT(65553)=(S=13260752,E=13260755,C=162)
	CT(65554)=(S=13260756,E=13260759,C=11)
	CT(65555)=(S=13260760,E=13260767,C=5)
	CT(65556)=(S=13260768,E=13260783,C=2)
	CT(65557)=(S=13260784,E=13260799,C=74)
	CT(65558)=(S=13260800,E=13260831,C=8)
	CT(65559)=(S=13260848,E=13260863,C=2)
	CT(65560)=(S=13260864,E=13260895,C=167)
	CT(65561)=(S=13260896,E=13260927,C=8)
	CT(65562)=(S=13260928,E=13260991,C=4)
	CT(65563)=(S=13260992,E=13260999,C=2)
	CT(65564)=(S=13261000,E=13261007,C=8)
	CT(65565)=(S=13261008,E=13261055,C=4)
	CT(65566)=(S=13261056,E=13261059,C=8)
	CT(65567)=(S=13261060,E=13261063,C=73)
	CT(65568)=(S=13261064,E=13261071,C=1)
	CT(65569)=(S=13261072,E=13261079,C=8)
	CT(65570)=(S=13261080,E=13261087,C=161)
	CT(65571)=(S=13261088,E=13261119,C=0)
	CT(65572)=(S=13261120,E=13261135,C=4)
	CT(65573)=(S=13261136,E=13261151,C=2)
	CT(65574)=(S=13261156,E=13261159,C=0)
	CT(65575)=(S=13261160,E=13261163,C=4)
	CT(65576)=(S=13261168,E=13261171,C=2)
	CT(65577)=(S=13261172,E=13261173,C=74)
	CT(65578)=(S=13261174,E=13261174,C=4)
	CT(65579)=(S=13261176,E=13261179,C=8)
	CT(65580)=(S=13261180,E=13261183,C=6)
	CT(65581)=(S=13261184,E=13261215,C=73)
	CT(65582)=(S=13261216,E=13261247,C=0)
	CT(65583)=(S=13261248,E=13261263,C=10)
	CT(65584)=(S=13261264,E=13261279,C=74)
	CT(65585)=(S=13261280,E=13261287,C=2)
	CT(65586)=(S=13261288,E=13261295,C=1)
	CT(65587)=(S=13261296,E=13261303,C=2)
	CT(65588)=(S=13261304,E=13261307,C=6)
	CT(65589)=(S=13261308,E=13261311,C=8)
	CT(65590)=(S=13261312,E=13261315,C=1)
	CT(65591)=(S=13261316,E=13261319,C=3)
	CT(65592)=(S=13261320,E=13261327,C=2)
	CT(65593)=(S=13261344,E=13261344,C=0)
	CT(65594)=(S=13261345,E=13261345,C=4)
	CT(65595)=(S=13261346,E=13261347,C=0)
	CT(65596)=(S=13261348,E=13261348,C=157)
	CT(65597)=(S=13261350,E=13261351,C=0)
	CT(65598)=(S=13261352,E=13261352,C=4)
	CT(65599)=(S=13261353,E=13261353,C=0)
	CT(65600)=(S=13261354,E=13261355,C=4)
	CT(65601)=(S=13261356,E=13261359,C=73)
	CT(65602)=(S=13261360,E=13261367,C=0)
	CT(65603)=(S=13261368,E=13261375,C=73)
	CT(65604)=(S=13261376,E=13261407,C=164)
	CT(65605)=(S=13261424,E=13261439,C=1)
	CT(65606)=(S=13261440,E=13261471,C=10)
	CT(65607)=(S=13261472,E=13261487,C=0)
	CT(65608)=(S=13261488,E=13261503,C=2)
	CT(65609)=(S=13261504,E=13261504,C=4)
	CT(65610)=(S=13261506,E=13261507,C=74)
	CT(65611)=(S=13261508,E=13261508,C=1)
	CT(65612)=(S=13261509,E=13261509,C=0)
	CT(65613)=(S=13261510,E=13261511,C=74)
	CT(65614)=(S=13261512,E=13261515,C=84)
	CT(65615)=(S=13261516,E=13261516,C=4)
	CT(65616)=(S=13261518,E=13261519,C=0)
	CT(65617)=(S=13261520,E=13261535,C=2)
	CT(65618)=(S=13261536,E=13261551,C=1)
	CT(65619)=(S=13261552,E=13261559,C=0)
	CT(65620)=(S=13261560,E=13261563,C=8)
	CT(65621)=(S=13261568,E=13261571,C=1)
	CT(65622)=(S=13261572,E=13261575,C=0)
	CT(65623)=(S=13261576,E=13261583,C=74)
	CT(65624)=(S=13261584,E=13261591,C=3)
	CT(65625)=(S=13261592,E=13261599,C=74)
	CT(65626)=(S=13261600,E=13261615,C=7)
	CT(65627)=(S=13261616,E=13261631,C=2)
	CT(65628)=(S=13261632,E=13261663,C=4)
	CT(65629)=(S=13261664,E=13261679,C=1)
	CT(65630)=(S=13261680,E=13261695,C=2)
	CT(65631)=(S=13261696,E=13261699,C=1)
	CT(65632)=(S=13261700,E=13261711,C=4)
	CT(65633)=(S=13261712,E=13261727,C=2)
	CT(65634)=(S=13261728,E=13261743,C=10)
	CT(65635)=(S=13261744,E=13261759,C=1)
	CT(65636)=(S=13261760,E=13261775,C=0)
	CT(65637)=(S=13261776,E=13261791,C=2)
	CT(65638)=(S=13261792,E=13261827,C=1)
	CT(65639)=(S=13261828,E=13261831,C=11)
	CT(65640)=(S=13261832,E=13261839,C=1)
	CT(65641)=(S=13261840,E=13261855,C=77)
	CT(65642)=(S=13261856,E=13261871,C=0)
	CT(65643)=(S=13261872,E=13261887,C=1)
	CT(65644)=(S=13261888,E=13261951,C=0)
	CT(65645)=(S=13261952,E=13261983,C=10)
	CT(65646)=(S=13261984,E=13262015,C=8)
	CT(65647)=(S=13262016,E=13262031,C=74)
	CT(65648)=(S=13262032,E=13262047,C=0)
	CT(65649)=(S=13262048,E=13262063,C=4)
	CT(65650)=(S=13262064,E=13262075,C=0)
	CT(65651)=(S=13262076,E=13262083,C=1)
	CT(65652)=(S=13262084,E=13262085,C=0)
	CT(65653)=(S=13262086,E=13262087,C=6)
	CT(65654)=(S=13262088,E=13262095,C=86)
	CT(65655)=(S=13262096,E=13262127,C=74)
	CT(65656)=(S=13262128,E=13262143,C=3)
	CT(65657)=(S=13262144,E=13262175,C=2)
	CT(65658)=(S=13262176,E=13262191,C=0)
	CT(65659)=(S=13262192,E=13262223,C=74)
	CT(65660)=(S=13262224,E=13262231,C=2)
	CT(65661)=(S=13262232,E=13262232,C=0)
	CT(65662)=(S=13262233,E=13262233,C=86)
	CT(65663)=(S=13262234,E=13262234,C=4)
	CT(65664)=(S=13262235,E=13262235,C=0)
	CT(65665)=(S=13262236,E=13262239,C=11)
	CT(65666)=(S=13262240,E=13262271,C=0)
	CT(65667)=(S=13262272,E=13262287,C=8)
	CT(65668)=(S=13262288,E=13262303,C=157)
	CT(65669)=(S=13262304,E=13262327,C=74)
	CT(65670)=(S=13262328,E=13262331,C=0)
	CT(65671)=(S=13262332,E=13262335,C=1)
	CT(65672)=(S=13262368,E=13262383,C=0)
	CT(65673)=(S=13262384,E=13262399,C=2)
	CT(65674)=(S=13262400,E=13262401,C=0)
	CT(65675)=(S=13262402,E=13262402,C=75)
	CT(65676)=(S=13262403,E=13262404,C=4)
	CT(65677)=(S=13262405,E=13262405,C=0)
	CT(65678)=(S=13262406,E=13262406,C=7)
	CT(65679)=(S=13262407,E=13262408,C=0)
	CT(65680)=(S=13262409,E=13262409,C=3)
	CT(65681)=(S=13262411,E=13262411,C=8)
	CT(65682)=(S=13262412,E=13262413,C=3)
	CT(65683)=(S=13262414,E=13262415,C=0)
	CT(65684)=(S=13262416,E=13262416,C=74)
	CT(65685)=(S=13262418,E=13262418,C=11)
	CT(65686)=(S=13262419,E=13262421,C=74)
	CT(65687)=(S=13262422,E=13262422,C=2)
	CT(65688)=(S=13262423,E=13262423,C=8)
	CT(65689)=(S=13262424,E=13262425,C=11)
	CT(65690)=(S=13262428,E=13262431,C=1)
	CT(65691)=(S=13262432,E=13262463,C=7)
	CT(65692)=(S=13262464,E=13262495,C=2)
	CT(65693)=(S=13262496,E=13262511,C=5)
	CT(65694)=(S=13262512,E=13262559,C=2)
	CT(65695)=(S=13262560,E=13262575,C=8)
	CT(65696)=(S=13262576,E=13262591,C=3)
	CT(65697)=(S=13262592,E=13262623,C=1)
	CT(65698)=(S=13262624,E=13262655,C=2)
	CT(65699)=(S=13262656,E=13262719,C=7)
	CT(65700)=(S=13262720,E=13262751,C=74)
	CT(65701)=(S=13262752,E=13262783,C=2)
	CT(65702)=(S=13262784,E=13262799,C=76)
	CT(65703)=(S=13262800,E=13262815,C=2)
	CT(65704)=(S=13262816,E=13262831,C=10)
	CT(65705)=(S=13262832,E=13262839,C=1)
	CT(65706)=(S=13262840,E=13262841,C=2)
	CT(65707)=(S=13262842,E=13262842,C=73)
	CT(65708)=(S=13262843,E=13262843,C=4)
	CT(65709)=(S=13262844,E=13269511,C=1)
	CT(65710)=(S=13269512,E=13269519,C=74)
	CT(65711)=(S=13269520,E=13269527,C=4)
	CT(65712)=(S=13269528,E=13269535,C=2)
	CT(65713)=(S=13269536,E=13269543,C=1)
	CT(65714)=(S=13269544,E=13269551,C=3)
	CT(65715)=(S=13269552,E=13269567,C=2)
	CT(65716)=(S=13269568,E=13269599,C=1)
	CT(65717)=(S=13269600,E=13269615,C=8)
	CT(65718)=(S=13269616,E=13269632,C=1)
	CT(65719)=(S=13269633,E=13269633,C=0)
	CT(65720)=(S=13269634,E=13269635,C=3)
	CT(65721)=(S=13269636,E=13269636,C=1)
	CT(65722)=(S=13269637,E=13269637,C=10)
	CT(65723)=(S=13269640,E=13269647,C=2)
	CT(65724)=(S=13269648,E=13269663,C=5)
	CT(65725)=(S=13269664,E=13269679,C=74)
	CT(65726)=(S=13269680,E=13269759,C=2)
	CT(65727)=(S=13269760,E=13269791,C=113)
	CT(65728)=(S=13269792,E=13269807,C=4)
	CT(65729)=(S=13269808,E=13269823,C=10)
	CT(65730)=(S=13269824,E=13269855,C=8)
	CT(65731)=(S=13269856,E=13269871,C=1)
	CT(65732)=(S=13269888,E=13269919,C=159)
	CT(65733)=(S=13269920,E=13269935,C=8)
	CT(65734)=(S=13269936,E=13269943,C=154)
	CT(65735)=(S=13269944,E=13269951,C=2)
	CT(65736)=(S=13269952,E=13269983,C=8)
	CT(65737)=(S=13269984,E=13269999,C=74)
	CT(65738)=(S=13270000,E=13270015,C=77)
	CT(65739)=(S=13270016,E=13270031,C=2)
	CT(65740)=(S=13270032,E=13270043,C=1)
	CT(65741)=(S=13270044,E=13270047,C=8)
	CT(65742)=(S=13270048,E=13270063,C=0)
	CT(65743)=(S=13270064,E=13270079,C=2)
	CT(65744)=(S=13270080,E=13270111,C=0)
	CT(65745)=(S=13270112,E=13270143,C=73)
	CT(65746)=(S=13270144,E=13270175,C=10)
	CT(65747)=(S=13270176,E=13270207,C=156)
	CT(65748)=(S=13270208,E=13270208,C=0)
	CT(65749)=(S=13270209,E=13270209,C=10)
	CT(65750)=(S=13270210,E=13270210,C=4)
	CT(65751)=(S=13270211,E=13270211,C=0)
	CT(65752)=(S=13270212,E=13270215,C=74)
	CT(65753)=(S=13270216,E=13270217,C=0)
	CT(65754)=(S=13270218,E=13270218,C=4)
	CT(65755)=(S=13270219,E=13270219,C=74)
	CT(65756)=(S=13270220,E=13270220,C=11)
	CT(65757)=(S=13270221,E=13270221,C=74)
	CT(65758)=(S=13270222,E=13270223,C=0)
	CT(65759)=(S=13270224,E=13270239,C=2)
	CT(65760)=(S=13270240,E=13270255,C=194)
	CT(65761)=(S=13270256,E=13270263,C=0)
	CT(65762)=(S=13270264,E=13270271,C=4)
	CT(65763)=(S=13270272,E=13270319,C=0)
	CT(65764)=(S=13270320,E=13270335,C=2)
	CT(65765)=(S=13270336,E=13270351,C=84)
	CT(65766)=(S=13270352,E=13270355,C=74)
	CT(65767)=(S=13270356,E=13270357,C=3)
	CT(65768)=(S=13270358,E=13270359,C=0)
	CT(65769)=(S=13270360,E=13270361,C=74)
	CT(65770)=(S=13270362,E=13270363,C=8)
	CT(65771)=(S=13270364,E=13270364,C=4)
	CT(65772)=(S=13270365,E=13270365,C=0)
	CT(65773)=(S=13270366,E=13270367,C=74)
	CT(65774)=(S=13270368,E=13270369,C=84)
	CT(65775)=(S=13270370,E=13270371,C=0)
	CT(65776)=(S=13270372,E=13270373,C=74)
	CT(65777)=(S=13270374,E=13270375,C=10)
	CT(65778)=(S=13270376,E=13270377,C=0)
	CT(65779)=(S=13270378,E=13270378,C=4)
	CT(65780)=(S=13270379,E=13270379,C=7)
	CT(65781)=(S=13270380,E=13270380,C=0)
	CT(65782)=(S=13270382,E=13270382,C=73)
	CT(65783)=(S=13270383,E=13270383,C=0)
	CT(65784)=(S=13270384,E=13270399,C=1)
	CT(65785)=(S=13270400,E=13270431,C=77)
	CT(65786)=(S=13270432,E=13270447,C=0)
	CT(65787)=(S=13270448,E=13270463,C=1)
	CT(65788)=(S=13270464,E=13270527,C=8)
	CT(65789)=(S=13270528,E=13270543,C=7)
	CT(65790)=(S=13270544,E=13270559,C=2)
	CT(65791)=(S=13270560,E=13270575,C=10)
	CT(65792)=(S=13270576,E=13270591,C=8)
	CT(65793)=(S=13270592,E=13270607,C=9)
	CT(65794)=(S=13270608,E=13270615,C=73)
	CT(65795)=(S=13270616,E=13270623,C=92)
	CT(65796)=(S=13270624,E=13270639,C=0)
	CT(65797)=(S=13270640,E=13270647,C=6)
	CT(65798)=(S=13270648,E=13270655,C=84)
	CT(65799)=(S=13270656,E=13270719,C=8)
	CT(65800)=(S=13270720,E=13270735,C=73)
	CT(65801)=(S=13270736,E=13270751,C=8)
	CT(65802)=(S=13270752,E=13270783,C=2)
	CT(65803)=(S=13270784,E=13270791,C=1)
	CT(65804)=(S=13270792,E=13270795,C=73)
	CT(65805)=(S=13270796,E=13270815,C=1)
	CT(65806)=(S=13270816,E=13270823,C=74)
	CT(65807)=(S=13270824,E=13270847,C=1)
	CT(65808)=(S=13270848,E=13270863,C=8)
	CT(65809)=(S=13270864,E=13270879,C=2)
	CT(65810)=(S=13270880,E=13270895,C=74)
	CT(65811)=(S=13270896,E=13270951,C=1)
	CT(65812)=(S=13270952,E=13270959,C=8)
	CT(65813)=(S=13270960,E=13270975,C=2)
	CT(65814)=(S=13270976,E=13270993,C=1)
	CT(65815)=(S=13270994,E=13270995,C=0)
	CT(65816)=(S=13270996,E=13271039,C=1)
	CT(65817)=(S=13271040,E=13271071,C=159)
	CT(65818)=(S=13271072,E=13271103,C=10)
	CT(65819)=(S=13271104,E=13271135,C=159)
	CT(65820)=(S=13271136,E=13271137,C=0)
	CT(65821)=(S=13271138,E=13271138,C=7)
	CT(65822)=(S=13271139,E=13271139,C=8)
	CT(65823)=(S=13271140,E=13271141,C=6)
	CT(65824)=(S=13271142,E=13271143,C=0)
	CT(65825)=(S=13271144,E=13271145,C=8)
	CT(65826)=(S=13271146,E=13271146,C=0)
	CT(65827)=(S=13271147,E=13271147,C=4)
	CT(65828)=(S=13271148,E=13271149,C=73)
	CT(65829)=(S=13271150,E=13271151,C=4)
	CT(65830)=(S=13271152,E=13271167,C=0)
	CT(65831)=(S=13271168,E=13271199,C=8)
	CT(65832)=(S=13271200,E=13271263,C=7)
	CT(65833)=(S=13271264,E=13271295,C=8)
	CT(65834)=(S=13271296,E=13271359,C=3)
	CT(65835)=(S=13271360,E=13271455,C=0)
	CT(65836)=(S=13271456,E=13271471,C=5)
	CT(65837)=(S=13271472,E=13271479,C=2)
	CT(65838)=(S=13271480,E=13271487,C=74)
	CT(65839)=(S=13271488,E=13271491,C=84)
	CT(65840)=(S=13271492,E=13271499,C=4)
	CT(65841)=(S=13271500,E=13271503,C=3)
	CT(65842)=(S=13271505,E=13271509,C=4)
	CT(65843)=(S=13271510,E=13271510,C=0)
	CT(65844)=(S=13271511,E=13271511,C=4)
	CT(65845)=(S=13271512,E=13271513,C=74)
	CT(65846)=(S=13271514,E=13271514,C=0)
	CT(65847)=(S=13271515,E=13271515,C=4)
	CT(65848)=(S=13271516,E=13271519,C=10)
	CT(65849)=(S=13271520,E=13271523,C=74)
	CT(65850)=(S=13271524,E=13271527,C=91)
	CT(65851)=(S=13271528,E=13271531,C=156)
	CT(65852)=(S=13271532,E=13271532,C=86)
	CT(65853)=(S=13271533,E=13271533,C=0)
	CT(65854)=(S=13271534,E=13271534,C=10)
	CT(65855)=(S=13271535,E=13271537,C=4)
	CT(65856)=(S=13271538,E=13271539,C=7)
	CT(65857)=(S=13271540,E=13271543,C=0)
	CT(65858)=(S=13271544,E=13271547,C=75)
	CT(65859)=(S=13271548,E=13271548,C=0)
	CT(65860)=(S=13271549,E=13271549,C=73)
	CT(65861)=(S=13271550,E=13271550,C=2)
	CT(65862)=(S=13271551,E=13271551,C=0)
	CT(65863)=(S=13271552,E=13271583,C=1)
	CT(65864)=(S=13271584,E=13271587,C=0)
	CT(65865)=(S=13271588,E=13271589,C=11)
	CT(65866)=(S=13271590,E=13271590,C=0)
	CT(65867)=(S=13271592,E=13271595,C=4)
	CT(65868)=(S=13271596,E=13271603,C=0)
	CT(65869)=(S=13271604,E=13271607,C=74)
	CT(65870)=(S=13271608,E=13271611,C=7)
	CT(65871)=(S=13271612,E=13271615,C=2)
	CT(65872)=(S=13271616,E=13271743,C=8)
	CT(65873)=(S=13271744,E=13271775,C=0)
	CT(65874)=(S=13271776,E=13271807,C=1)
	CT(65875)=(S=13271808,E=13271815,C=92)
	CT(65876)=(S=13271816,E=13271823,C=2)
	CT(65877)=(S=13271824,E=13271831,C=1)
	CT(65878)=(S=13271832,E=13271839,C=6)
	CT(65879)=(S=13271840,E=13271855,C=8)
	CT(65880)=(S=13271856,E=13271871,C=1)
	CT(65881)=(S=13271872,E=13271887,C=8)
	CT(65882)=(S=13271888,E=13271895,C=86)
	CT(65883)=(S=13271896,E=13271903,C=0)
	CT(65884)=(S=13271904,E=13271967,C=4)
	CT(65885)=(S=13271968,E=13271999,C=159)
	CT(65886)=(S=13272000,E=13272015,C=2)
	CT(65887)=(S=13272016,E=13272031,C=1)
	CT(65888)=(S=13272032,E=13272063,C=92)
	CT(65889)=(S=13272064,E=13272319,C=9)
	CT(65890)=(S=13272320,E=13272327,C=74)
	CT(65891)=(S=13272328,E=13272335,C=8)
	CT(65892)=(S=13272336,E=13272351,C=6)
	CT(65893)=(S=13272352,E=13272367,C=1)
	CT(65894)=(S=13272368,E=13272383,C=4)
	CT(65895)=(S=13272416,E=13272431,C=5)
	CT(65896)=(S=13272432,E=13272447,C=2)
	CT(65897)=(S=13272448,E=13272511,C=3)
	CT(65898)=(S=13272544,E=13272575,C=9)
	CT(65899)=(S=13272576,E=13272583,C=74)
	CT(65900)=(S=13272584,E=13272591,C=84)
	CT(65901)=(S=13272592,E=13272599,C=11)
	CT(65902)=(S=13272600,E=13272607,C=166)
	CT(65903)=(S=13272608,E=13272623,C=0)
	CT(65904)=(S=13272624,E=13272624,C=73)
	CT(65905)=(S=13272625,E=13272627,C=0)
	CT(65906)=(S=13272628,E=13272628,C=4)
	CT(65907)=(S=13272629,E=13272629,C=73)
	CT(65908)=(S=13272630,E=13272630,C=11)
	CT(65909)=(S=13272631,E=13272631,C=8)
	CT(65910)=(S=13272632,E=13272633,C=10)
	CT(65911)=(S=13272634,E=13272634,C=1)
	CT(65912)=(S=13272635,E=13272635,C=4)
	CT(65913)=(S=13272636,E=13272703,C=8)
	CT(65914)=(S=13272704,E=13272719,C=1)
	CT(65915)=(S=13272720,E=13272783,C=4)
	CT(65916)=(S=13272800,E=13272831,C=0)
	CT(65917)=(S=13272832,E=13273087,C=2)
	CT(65918)=(S=13273088,E=13273119,C=7)
	CT(65919)=(S=13273120,E=13273135,C=0)
	CT(65920)=(S=13273136,E=13273151,C=1)
	CT(65921)=(S=13273184,E=13273199,C=0)
	CT(65922)=(S=13273200,E=13273247,C=6)
	CT(65923)=(S=13273248,E=13273279,C=7)
	CT(65924)=(S=13273280,E=13273295,C=2)
	CT(65925)=(S=13273296,E=13273327,C=1)
	CT(65926)=(S=13273328,E=13273335,C=154)
	CT(65927)=(S=13273340,E=13273343,C=0)
	CT(65928)=(S=13273344,E=13273375,C=74)
	CT(65929)=(S=13273376,E=13273407,C=2)
	CT(65930)=(S=13273408,E=13273455,C=0)
	CT(65931)=(S=13273456,E=13273471,C=10)
	CT(65932)=(S=13273472,E=13273503,C=154)
	CT(65933)=(S=13273504,E=13273519,C=0)
	CT(65934)=(S=13273520,E=13273527,C=91)
	CT(65935)=(S=13273528,E=13273535,C=2)
	CT(65936)=(S=13273536,E=13273551,C=0)
	CT(65937)=(S=13273552,E=13273567,C=4)
	CT(65938)=(S=13273568,E=13273568,C=73)
	CT(65939)=(S=13273569,E=13273569,C=74)
	CT(65940)=(S=13273570,E=13273573,C=4)
	CT(65941)=(S=13273574,E=13273574,C=74)
	CT(65942)=(S=13273575,E=13273575,C=1)
	CT(65943)=(S=13273576,E=13273583,C=4)
	CT(65944)=(S=13273584,E=13273591,C=73)
	CT(65945)=(S=13273592,E=13273595,C=4)
	CT(65946)=(S=13273596,E=13273599,C=5)
	CT(65947)=(S=13273600,E=13273679,C=0)
	CT(65948)=(S=13273680,E=13273695,C=2)
	CT(65949)=(S=13273696,E=13273727,C=4)
	CT(65950)=(S=13273728,E=13273791,C=10)
	CT(65951)=(S=13273792,E=13273823,C=0)
	CT(65952)=(S=13273824,E=13273855,C=74)
	CT(65953)=(S=13273856,E=13274047,C=0)
	CT(65954)=(S=13274048,E=13274079,C=3)
	CT(65955)=(S=13274080,E=13274111,C=0)
	CT(65956)=(S=13274112,E=13274143,C=2)
	CT(65957)=(S=13274144,E=13274175,C=4)
	CT(65958)=(S=13274176,E=13274239,C=8)
	CT(65959)=(S=13274248,E=13274251,C=4)
	CT(65960)=(S=13274272,E=13274303,C=9)
	CT(65961)=(S=13274304,E=13274335,C=2)
	CT(65962)=(S=13274336,E=13274367,C=0)
	CT(65963)=(S=13274368,E=13274527,C=4)
	CT(65964)=(S=13274528,E=13274559,C=1)
	CT(65965)=(S=13274560,E=13274575,C=2)
	CT(65966)=(S=13274576,E=13274591,C=0)
	CT(65967)=(S=13274592,E=13274631,C=4)
	CT(65968)=(S=13274632,E=13274639,C=2)
	CT(65969)=(S=13274640,E=13274655,C=1)
	CT(65970)=(S=13274656,E=13274687,C=0)
	CT(65971)=(S=13274688,E=13274751,C=4)
	CT(65972)=(S=13274752,E=13274767,C=0)
	CT(65973)=(S=13274768,E=13274815,C=77)
	CT(65974)=(S=13274816,E=13274847,C=3)
	CT(65975)=(S=13274848,E=13274879,C=2)
	CT(65976)=(S=13274880,E=13274881,C=4)
	CT(65977)=(S=13274882,E=13274883,C=2)
	CT(65978)=(S=13274884,E=13274887,C=1)
	CT(65979)=(S=13274888,E=13274895,C=2)
	CT(65980)=(S=13274896,E=13274927,C=1)
	CT(65981)=(S=13274928,E=13274935,C=0)
	CT(65982)=(S=13274936,E=13274943,C=1)
	CT(65983)=(S=13274944,E=13274975,C=2)
	CT(65984)=(S=13274992,E=13275007,C=77)
	CT(65985)=(S=13275008,E=13275071,C=3)
	CT(65986)=(S=13275072,E=13275135,C=2)
	CT(65987)=(S=13275136,E=13275263,C=4)
	CT(65988)=(S=13275264,E=13275295,C=161)
	CT(65989)=(S=13275296,E=13275319,C=0)
	CT(65990)=(S=13275320,E=13275327,C=154)
	CT(65991)=(S=13275328,E=13275343,C=5)
	CT(65992)=(S=13275344,E=13275359,C=9)
	CT(65993)=(S=13275360,E=13275391,C=2)
	CT(65994)=(S=13275392,E=13275407,C=74)
	CT(65995)=(S=13275408,E=13275423,C=2)
	CT(65996)=(S=13275424,E=13275647,C=9)
	CT(65997)=(S=13275648,E=13275653,C=74)
	CT(65998)=(S=13275654,E=13275663,C=0)
	CT(65999)=(S=13275664,E=13275671,C=2)
	CT(66000)=(S=13275672,E=13275679,C=0)
	CT(66001)=(S=13275680,E=13275711,C=74)
	CT(66002)=(S=13275712,E=13275743,C=5)
	CT(66003)=(S=13275744,E=13275775,C=8)
	CT(66004)=(S=13275776,E=13275807,C=74)
	CT(66005)=(S=13275808,E=13275823,C=1)
	CT(66006)=(S=13275824,E=13275831,C=74)
	CT(66007)=(S=13275832,E=13275833,C=10)
	CT(66008)=(S=13275835,E=13275835,C=0)
	CT(66009)=(S=13275836,E=13275839,C=1)
	CT(66010)=(S=13275840,E=13275843,C=4)
	CT(66011)=(S=13275844,E=13275855,C=1)
	CT(66012)=(S=13275856,E=13275863,C=0)
	CT(66013)=(S=13275864,E=13275871,C=8)
	CT(66014)=(S=13275872,E=13275903,C=74)
	CT(66015)=(S=13275904,E=13275967,C=2)
	CT(66016)=(S=13275968,E=13275999,C=4)
	CT(66017)=(S=13276000,E=13276047,C=0)
	CT(66018)=(S=13276048,E=13276063,C=1)
	CT(66019)=(S=13276064,E=13276095,C=77)
	CT(66020)=(S=13276096,E=13276111,C=74)
	CT(66021)=(S=13276112,E=13276127,C=2)
	CT(66022)=(S=13276128,E=13276191,C=74)
	CT(66023)=(S=13276192,E=13276287,C=1)
	CT(66024)=(S=13276288,E=13276319,C=0)
	CT(66025)=(S=13276320,E=13276335,C=7)
	CT(66026)=(S=13276336,E=13276351,C=3)
	CT(66027)=(S=13276352,E=13276367,C=4)
	CT(66028)=(S=13276368,E=13276383,C=9)
	CT(66029)=(S=13276384,E=13276399,C=0)
	CT(66030)=(S=13276400,E=13276439,C=2)
	CT(66031)=(S=13276440,E=13276447,C=3)
	CT(66032)=(S=13276448,E=13276479,C=1)
	CT(66033)=(S=13276480,E=13276511,C=74)
	CT(66034)=(S=13276512,E=13276543,C=3)
	CT(66035)=(S=13276544,E=13276575,C=74)
	CT(66036)=(S=13276576,E=13276607,C=1)
	CT(66037)=(S=13276608,E=13276639,C=4)
	CT(66038)=(S=13276640,E=13276671,C=1)
	CT(66039)=(S=13276672,E=13276675,C=86)
	CT(66040)=(S=13276676,E=13276679,C=4)
	CT(66041)=(S=13276680,E=13276683,C=86)
	CT(66042)=(S=13276684,E=13276687,C=2)
	CT(66043)=(S=13276688,E=13276719,C=1)
	CT(66044)=(S=13276720,E=13276727,C=2)
	CT(66045)=(S=13276728,E=13276731,C=1)
	CT(66046)=(S=13276732,E=13276735,C=0)
	CT(66047)=(S=13276736,E=13276767,C=74)
	CT(66048)=(S=13276768,E=13276799,C=73)
	CT(66049)=(S=13276800,E=13276847,C=74)
	CT(66050)=(S=13276848,E=13276863,C=6)
	CT(66051)=(S=13276864,E=13276879,C=1)
	CT(66052)=(S=13276880,E=13276895,C=7)
	CT(66053)=(S=13276896,E=13276931,C=1)
	CT(66054)=(S=13276932,E=13276935,C=3)
	CT(66055)=(S=13276936,E=13276943,C=74)
	CT(66056)=(S=13276944,E=13276959,C=91)
	CT(66057)=(S=13276960,E=13276960,C=0)
	CT(66058)=(S=13276962,E=13276963,C=10)
	CT(66059)=(S=13276964,E=13276967,C=0)
	CT(66060)=(S=13276968,E=13276975,C=3)
	CT(66061)=(S=13276976,E=13276991,C=9)
	CT(66062)=(S=13276992,E=13277023,C=159)
	CT(66063)=(S=13277024,E=13277055,C=2)
	CT(66064)=(S=13277056,E=13277087,C=1)
	CT(66065)=(S=13277088,E=13277103,C=11)
	CT(66066)=(S=13277104,E=13277119,C=3)
	CT(66067)=(S=13277120,E=13277183,C=5)
	CT(66068)=(S=13277184,E=13277247,C=74)
	CT(66069)=(S=13277248,E=13277279,C=157)
	CT(66070)=(S=13277280,E=13277311,C=2)
	CT(66071)=(S=13277312,E=13277359,C=74)
	CT(66072)=(S=13277360,E=13277375,C=1)
	CT(66073)=(S=13277376,E=13277391,C=74)
	CT(66074)=(S=13277392,E=13277407,C=2)
	CT(66075)=(S=13277408,E=13277439,C=74)
	CT(66076)=(S=13277440,E=13277443,C=1)
	CT(66077)=(S=13277446,E=13277446,C=7)
	CT(66078)=(S=13277448,E=13277455,C=8)
	CT(66079)=(S=13277456,E=13277471,C=74)
	CT(66080)=(S=13277472,E=13277487,C=4)
	CT(66081)=(S=13277488,E=13277503,C=1)
	CT(66082)=(S=13277504,E=13277519,C=0)
	CT(66083)=(S=13277520,E=13277527,C=88)
	CT(66084)=(S=13277528,E=13277567,C=8)
	CT(66085)=(S=13277568,E=13277599,C=74)
	CT(66086)=(S=13277600,E=13277647,C=9)
	CT(66087)=(S=13277648,E=13277663,C=0)
	CT(66088)=(S=13277664,E=13277759,C=74)
	CT(66089)=(S=13277760,E=13277823,C=0)
	CT(66090)=(S=13277824,E=13277855,C=73)
	CT(66091)=(S=13277856,E=13277871,C=4)
	CT(66092)=(S=13277872,E=13277887,C=74)
	CT(66093)=(S=13277888,E=13277919,C=9)
	CT(66094)=(S=13277920,E=13277951,C=77)
	CT(66095)=(S=13277952,E=13278111,C=74)
	CT(66096)=(S=13278112,E=13278143,C=0)
	CT(66097)=(S=13278144,E=13278207,C=8)
	CT(66098)=(S=13278208,E=13278463,C=7)
	CT(66099)=(S=13278464,E=13278527,C=2)
	CT(66100)=(S=13278528,E=13278559,C=4)
	CT(66101)=(S=13278560,E=13278591,C=2)
	CT(66102)=(S=13278592,E=13278655,C=7)
	CT(66103)=(S=13278656,E=13278687,C=1)
	CT(66104)=(S=13278688,E=13278719,C=2)
	CT(66105)=(S=13278720,E=13278863,C=74)
	CT(66106)=(S=13278864,E=13278879,C=6)
	CT(66107)=(S=13278880,E=13278911,C=1)
	CT(66108)=(S=13278912,E=13278943,C=0)
	CT(66109)=(S=13278944,E=13278959,C=2)
	CT(66110)=(S=13278960,E=13278960,C=7)
	CT(66111)=(S=13278961,E=13278963,C=0)
	CT(66112)=(S=13278964,E=13278967,C=11)
	CT(66113)=(S=13278972,E=13278972,C=74)
	CT(66114)=(S=13278973,E=13278975,C=0)
	CT(66115)=(S=13278976,E=13279103,C=74)
	CT(66116)=(S=13279104,E=13279167,C=0)
	CT(66117)=(S=13279168,E=13279231,C=4)
	CT(66118)=(S=13279232,E=13279279,C=157)
	CT(66119)=(S=13279280,E=13279295,C=73)
	CT(66120)=(S=13279296,E=13279327,C=9)
	CT(66121)=(S=13279328,E=13279343,C=0)
	CT(66122)=(S=13279344,E=13279351,C=73)
	CT(66123)=(S=13279352,E=13279355,C=7)
	CT(66124)=(S=13279356,E=13279357,C=11)
	CT(66125)=(S=13279358,E=13279359,C=0)
	CT(66126)=(S=13279392,E=13279407,C=4)
	CT(66127)=(S=13279408,E=13279423,C=1)
	CT(66128)=(S=13279424,E=13279487,C=7)
	CT(66129)=(S=13279488,E=13279519,C=0)
	CT(66130)=(S=13279520,E=13279551,C=7)
	CT(66131)=(S=13279552,E=13279615,C=0)
	CT(66132)=(S=13279616,E=13279663,C=8)
	CT(66133)=(S=13279664,E=13279679,C=84)
	CT(66134)=(S=13279680,E=13279743,C=2)
	CT(66135)=(S=13279744,E=13279775,C=5)
	CT(66136)=(S=13279776,E=13279791,C=74)
	CT(66137)=(S=13279792,E=13279807,C=4)
	CT(66138)=(S=13279808,E=13279810,C=0)
	CT(66139)=(S=13279811,E=13279811,C=1)
	CT(66140)=(S=13279812,E=13279815,C=7)
	CT(66141)=(S=13279816,E=13279817,C=73)
	CT(66142)=(S=13279818,E=13279818,C=0)
	CT(66143)=(S=13279819,E=13279819,C=1)
	CT(66144)=(S=13279820,E=13279823,C=3)
	CT(66145)=(S=13279824,E=13279839,C=2)
	CT(66146)=(S=13279840,E=13279855,C=0)
	CT(66147)=(S=13279856,E=13279903,C=2)
	CT(66148)=(S=13279904,E=13279919,C=10)
	CT(66149)=(S=13279920,E=13279927,C=73)
	CT(66150)=(S=13279928,E=13279931,C=8)
	CT(66151)=(S=13279932,E=13279935,C=4)
	CT(66152)=(S=13279936,E=13279967,C=74)
	CT(66153)=(S=13279968,E=13279999,C=4)
	CT(66154)=(S=13280000,E=13280031,C=8)
	CT(66155)=(S=13280032,E=13280063,C=7)
	CT(66156)=(S=13280064,E=13280127,C=77)
	CT(66157)=(S=13280128,E=13280159,C=6)
	CT(66158)=(S=13280160,E=13280191,C=2)
	CT(66159)=(S=13280192,E=13280255,C=10)
	CT(66160)=(S=13280256,E=13280271,C=1)
	CT(66161)=(S=13280272,E=13280279,C=0)
	CT(66162)=(S=13280280,E=13280281,C=4)
	CT(66163)=(S=13280282,E=13280282,C=0)
	CT(66164)=(S=13280283,E=13280283,C=8)
	CT(66165)=(S=13280284,E=13280287,C=73)
	CT(66166)=(S=13280288,E=13280319,C=4)
	CT(66167)=(S=13280320,E=13280351,C=2)
	CT(66168)=(S=13280352,E=13280383,C=1)
	CT(66169)=(S=13280384,E=13280415,C=4)
	CT(66170)=(S=13280416,E=13280447,C=10)
	CT(66171)=(S=13280448,E=13280463,C=0)
	CT(66172)=(S=13280464,E=13280471,C=84)
	CT(66173)=(S=13280472,E=13280479,C=74)
	CT(66174)=(S=13280480,E=13280511,C=2)
	CT(66175)=(S=13280512,E=13280543,C=5)
	CT(66176)=(S=13280544,E=13280559,C=74)
	CT(66177)=(S=13280560,E=13280567,C=2)
	CT(66178)=(S=13280568,E=13280571,C=6)
	CT(66179)=(S=13280572,E=13280573,C=10)
	CT(66180)=(S=13280574,E=13280575,C=4)
	CT(66181)=(S=13280576,E=13280607,C=0)
	CT(66182)=(S=13280608,E=13280623,C=1)
	CT(66183)=(S=13280624,E=13280631,C=2)
	CT(66184)=(S=13280640,E=13280671,C=9)
	CT(66185)=(S=13280672,E=13280687,C=2)
	CT(66186)=(S=13280688,E=13280703,C=1)
	CT(66187)=(S=13280704,E=13280719,C=76)
	CT(66188)=(S=13280720,E=13280735,C=1)
	CT(66189)=(S=13280736,E=13280750,C=77)
	CT(66190)=(S=13280751,E=13280753,C=1)
	CT(66191)=(S=13280754,E=13280754,C=77)
	CT(66192)=(S=13280755,E=13280755,C=1)
	CT(66193)=(S=13280756,E=13280756,C=77)
	CT(66194)=(S=13280757,E=13280757,C=1)
	CT(66195)=(S=13280758,E=13280762,C=77)
	CT(66196)=(S=13280763,E=13280767,C=1)
	CT(66197)=(S=13280768,E=13280927,C=7)
	CT(66198)=(S=13280928,E=13280943,C=77)
	CT(66199)=(S=13280944,E=13280951,C=90)
	CT(66200)=(S=13280952,E=13280952,C=4)
	CT(66201)=(S=13280953,E=13280953,C=2)
	CT(66202)=(S=13280954,E=13280955,C=0)
	CT(66203)=(S=13280956,E=13280959,C=10)
	CT(66204)=(S=13280960,E=13280991,C=75)
	CT(66205)=(S=13280992,E=13281023,C=1)
	CT(66206)=(S=13281024,E=13281231,C=5)
	CT(66207)=(S=13281232,E=13281247,C=6)
	CT(66208)=(S=13281248,E=13281279,C=7)
	CT(66209)=(S=13281280,E=13281343,C=0)
	CT(66210)=(S=13281344,E=13281359,C=5)
	CT(66211)=(S=13281368,E=13281371,C=4)
	CT(66212)=(S=13281372,E=13281373,C=7)
	CT(66213)=(S=13281374,E=13281374,C=4)
	CT(66214)=(S=13281375,E=13281375,C=8)
	CT(66215)=(S=13281376,E=13281407,C=0)
	CT(66216)=(S=13281440,E=13281471,C=1)
	CT(66217)=(S=13281472,E=13281487,C=9)
	CT(66218)=(S=13281488,E=13281495,C=8)
	CT(66219)=(S=13281496,E=13281503,C=2)
	CT(66220)=(S=13281504,E=13281535,C=84)
	CT(66221)=(S=13281536,E=13281551,C=8)
	CT(66222)=(S=13281552,E=13281559,C=0)
	CT(66223)=(S=13281560,E=13281567,C=5)
	CT(66224)=(S=13281568,E=13281599,C=74)
	CT(66225)=(S=13281600,E=13281631,C=2)
	CT(66226)=(S=13281632,E=13281663,C=0)
	CT(66227)=(S=13281664,E=13281695,C=2)
	CT(66228)=(S=13281696,E=13281711,C=9)
	CT(66229)=(S=13281712,E=13281719,C=0)
	CT(66230)=(S=13281720,E=13281727,C=2)
	CT(66231)=(S=13281728,E=13281759,C=73)
	CT(66232)=(S=13281760,E=13281791,C=74)
	CT(66233)=(S=13281792,E=13281823,C=8)
	CT(66234)=(S=13281824,E=13281839,C=91)
	CT(66235)=(S=13281840,E=13281855,C=5)
	CT(66236)=(S=13281856,E=13281887,C=92)
	CT(66237)=(S=13281888,E=13281903,C=2)
	CT(66238)=(S=13281904,E=13281919,C=3)
	CT(66239)=(S=13281920,E=13281951,C=1)
	CT(66240)=(S=13281952,E=13281967,C=73)
	CT(66241)=(S=13281968,E=13281983,C=2)
	CT(66242)=(S=13281984,E=13281991,C=0)
	CT(66243)=(S=13282000,E=13282007,C=2)
	CT(66244)=(S=13282008,E=13282047,C=1)
	CT(66245)=(S=13282048,E=13282079,C=74)
	CT(66246)=(S=13282080,E=13282111,C=5)
	CT(66247)=(S=13282112,E=13282127,C=90)
	CT(66248)=(S=13282128,E=13282143,C=2)
	CT(66249)=(S=13282144,E=13282159,C=0)
	CT(66250)=(S=13282160,E=13282207,C=2)
	CT(66251)=(S=13282208,E=13282239,C=0)
	CT(66252)=(S=13282240,E=13282255,C=5)
	CT(66253)=(S=13282256,E=13282263,C=8)
	CT(66254)=(S=13282264,E=13282271,C=1)
	CT(66255)=(S=13282272,E=13282279,C=2)
	CT(66256)=(S=13282281,E=13282281,C=74)
	CT(66257)=(S=13282282,E=13282282,C=4)
	CT(66258)=(S=13282283,E=13282283,C=1)
	CT(66259)=(S=13282284,E=13282285,C=74)
	CT(66260)=(S=13282286,E=13282286,C=0)
	CT(66261)=(S=13282287,E=13282287,C=4)
	CT(66262)=(S=13282288,E=13282295,C=76)
	CT(66263)=(S=13282296,E=13282299,C=6)
	CT(66264)=(S=13282300,E=13282303,C=87)
	CT(66265)=(S=13282304,E=13282307,C=1)
	CT(66266)=(S=13282308,E=13282309,C=11)
	CT(66267)=(S=13282310,E=13282319,C=0)
	CT(66268)=(S=13282320,E=13282327,C=3)
	CT(66269)=(S=13282328,E=13282335,C=2)
	CT(66270)=(S=13282336,E=13282367,C=7)
	CT(66271)=(S=13282368,E=13282399,C=2)
	CT(66272)=(S=13282400,E=13282463,C=0)
	CT(66273)=(S=13282464,E=13282559,C=7)
	CT(66274)=(S=13282560,E=13282563,C=1)
	CT(66275)=(S=13282564,E=13282564,C=10)
	CT(66276)=(S=13282565,E=13282565,C=2)
	CT(66277)=(S=13282567,E=13282567,C=10)
	CT(66278)=(S=13282568,E=13282575,C=1)
	CT(66279)=(S=13282576,E=13282583,C=74)
	CT(66280)=(S=13282584,E=13282591,C=0)
	CT(66281)=(S=13282592,E=13282623,C=9)
	CT(66282)=(S=13282624,E=13282655,C=74)
	CT(66283)=(S=13282656,E=13282671,C=2)
	CT(66284)=(S=13282688,E=13282767,C=0)
	CT(66285)=(S=13282768,E=13282783,C=3)
	CT(66286)=(S=13282784,E=13282815,C=1)
	CT(66287)=(S=13282816,E=13282819,C=87)
	CT(66288)=(S=13282820,E=13282820,C=9)
	CT(66289)=(S=13282821,E=13282821,C=4)
	CT(66290)=(S=13282822,E=13282823,C=73)
	CT(66291)=(S=13282824,E=13282831,C=8)
	CT(66292)=(S=13282832,E=13282847,C=9)
	CT(66293)=(S=13282848,E=13282863,C=0)
	CT(66294)=(S=13282864,E=13282879,C=2)
	CT(66295)=(S=13282880,E=13282895,C=1)
	CT(66296)=(S=13282896,E=13282903,C=0)
	CT(66297)=(S=13282904,E=13282907,C=6)
	CT(66298)=(S=13282908,E=13282911,C=4)
	CT(66299)=(S=13282912,E=13282927,C=0)
	CT(66300)=(S=13282928,E=13282935,C=73)
	CT(66301)=(S=13282936,E=13282936,C=4)
	CT(66302)=(S=13282937,E=13282937,C=8)
	CT(66303)=(S=13282938,E=13282939,C=4)
	CT(66304)=(S=13282944,E=13282975,C=8)
	CT(66305)=(S=13282976,E=13283007,C=73)
	CT(66306)=(S=13283008,E=13283039,C=2)
	CT(66307)=(S=13283040,E=13283071,C=0)
	CT(66308)=(S=13283072,E=13283199,C=87)
	CT(66309)=(S=13283200,E=13283215,C=73)
	CT(66310)=(S=13283216,E=13283231,C=2)
	CT(66311)=(S=13283232,E=13283263,C=87)
	CT(66312)=(S=13283264,E=13283327,C=10)
	CT(66313)=(S=13283336,E=13283339,C=2)
	CT(66314)=(S=13283340,E=13283341,C=5)
	CT(66315)=(S=13283342,E=13283343,C=0)
	CT(66316)=(S=13283344,E=13283391,C=2)
	CT(66317)=(S=13283392,E=13283519,C=3)
	CT(66318)=(S=13283520,E=13283551,C=7)
	CT(66319)=(S=13283552,E=13283583,C=1)
	CT(66320)=(S=13283584,E=13283615,C=8)
	CT(66321)=(S=13283616,E=13283623,C=2)
	CT(66322)=(S=13283624,E=13283627,C=0)
	CT(66323)=(S=13283628,E=13283631,C=4)
	CT(66324)=(S=13283632,E=13283711,C=2)
	CT(66325)=(S=13283712,E=13283775,C=4)
	CT(66326)=(S=13283776,E=13283791,C=7)
	CT(66327)=(S=13283792,E=13283807,C=0)
	CT(66328)=(S=13283808,E=13283839,C=4)
	CT(66329)=(S=13283840,E=13283919,C=5)
	CT(66330)=(S=13283920,E=13283951,C=2)
	CT(66331)=(S=13283952,E=13283967,C=86)
	CT(66332)=(S=13283968,E=13284095,C=9)
	CT(66333)=(S=13284096,E=13284127,C=92)
	CT(66334)=(S=13284128,E=13284159,C=2)
	CT(66335)=(S=13284160,E=13284191,C=4)
	CT(66336)=(S=13284192,E=13284223,C=5)
	CT(66337)=(S=13284272,E=13284279,C=6)
	CT(66338)=(S=13284280,E=13284287,C=74)
	CT(66339)=(S=13284288,E=13284335,C=2)
	CT(66340)=(S=13284336,E=13284351,C=1)
	CT(66341)=(S=13284352,E=13284383,C=74)
	CT(66342)=(S=13284384,E=13284399,C=2)
	CT(66343)=(S=13284400,E=13284407,C=74)
	CT(66344)=(S=13284408,E=13284415,C=2)
	CT(66345)=(S=13284416,E=13284479,C=73)
	CT(66346)=(S=13284480,E=13284511,C=1)
	CT(66347)=(S=13284512,E=13284527,C=8)
	CT(66348)=(S=13284528,E=13284559,C=2)
	CT(66349)=(S=13284560,E=13284567,C=1)
	CT(66350)=(S=13284568,E=13284575,C=92)
	CT(66351)=(S=13284576,E=13284703,C=0)
	CT(66352)=(S=13284704,E=13284719,C=2)
	CT(66353)=(S=13284720,E=13284735,C=1)
	CT(66354)=(S=13284736,E=13284863,C=8)
	CT(66355)=(S=13284864,E=13284895,C=3)
	CT(66356)=(S=13284896,E=13284911,C=1)
	CT(66357)=(S=13284912,E=13284927,C=74)
	CT(66358)=(S=13284928,E=13285023,C=0)
	CT(66359)=(S=13285024,E=13285055,C=74)
	CT(66360)=(S=13285056,E=13285087,C=1)
	CT(66361)=(S=13285088,E=13285119,C=8)
	CT(66362)=(S=13285120,E=13285151,C=74)
	CT(66363)=(S=13285152,E=13285167,C=10)
	CT(66364)=(S=13285168,E=13285183,C=2)
	CT(66365)=(S=13285184,E=13285199,C=4)
	CT(66366)=(S=13285200,E=13285215,C=2)
	CT(66367)=(S=13285216,E=13285247,C=0)
	CT(66368)=(S=13285248,E=13285375,C=3)
	CT(66369)=(S=13285376,E=13286655,C=5)
	CT(66370)=(S=13286656,E=13286719,C=1)
	CT(66371)=(S=13286720,E=13286735,C=0)
	CT(66372)=(S=13286736,E=13286751,C=1)
	CT(66373)=(S=13286752,E=13286783,C=8)
	CT(66374)=(S=13286784,E=13286784,C=6)
	CT(66375)=(S=13286785,E=13286799,C=235)
	CT(66376)=(S=13286800,E=13286815,C=2)
	CT(66377)=(S=13286816,E=13286831,C=73)
	CT(66378)=(S=13286832,E=13286839,C=2)
	CT(66379)=(S=13286840,E=13286847,C=1)
	CT(66380)=(S=13286848,E=13286879,C=2)
	CT(66381)=(S=13286880,E=13286911,C=4)
	CT(66382)=(S=13286912,E=13287167,C=5)
	CT(66383)=(S=13287168,E=13287168,C=1)
	CT(66384)=(S=13287169,E=13287169,C=0)
	CT(66385)=(S=13287170,E=13287171,C=74)
	CT(66386)=(S=13287172,E=13287183,C=0)
	CT(66387)=(S=13287184,E=13287199,C=2)
	CT(66388)=(S=13287200,E=13287215,C=73)
	CT(66389)=(S=13287216,E=13287223,C=0)
	CT(66390)=(S=13287224,E=13287227,C=11)
	CT(66391)=(S=13287228,E=13287231,C=0)
	CT(66392)=(S=13287232,E=13287235,C=4)
	CT(66393)=(S=13287236,E=13287263,C=1)
	CT(66394)=(S=13287264,E=13287279,C=0)
	CT(66395)=(S=13287280,E=13287287,C=2)
	CT(66396)=(S=13287288,E=13287295,C=84)
	CT(66397)=(S=13287296,E=13287423,C=4)
	CT(66398)=(S=13287424,E=13291519,C=1)
	CT(66399)=(S=13291520,E=13303807,C=2)
	CT(66400)=(S=13303808,E=13303811,C=0)
	CT(66401)=(S=13303812,E=13303815,C=1)
	CT(66402)=(S=13303816,E=13303817,C=0)
	CT(66403)=(S=13303818,E=13303819,C=1)
	CT(66404)=(S=13303820,E=13303825,C=0)
	CT(66405)=(S=13303826,E=13303826,C=1)
	CT(66406)=(S=13303827,E=13303831,C=0)
	CT(66407)=(S=13303832,E=13303832,C=1)
	CT(66408)=(S=13303833,E=13303849,C=0)
	CT(66409)=(S=13303850,E=13303851,C=1)
	CT(66410)=(S=13303852,E=13303852,C=0)
	CT(66411)=(S=13303853,E=13303855,C=1)
	CT(66412)=(S=13303856,E=13303888,C=0)
	CT(66413)=(S=13303889,E=13303891,C=1)
	CT(66414)=(S=13303892,E=13303897,C=0)
	CT(66415)=(S=13303898,E=13303899,C=1)
	CT(66416)=(S=13303900,E=13303903,C=0)
	CT(66417)=(S=13303904,E=13303905,C=1)
	CT(66418)=(S=13303906,E=13303911,C=0)
	CT(66419)=(S=13303912,E=13303919,C=1)
	CT(66420)=(S=13303920,E=13303920,C=0)
	CT(66421)=(S=13303922,E=13303923,C=1)
	CT(66422)=(S=13303924,E=13303929,C=0)
	CT(66423)=(S=13303930,E=13303930,C=1)
	CT(66424)=(S=13303931,E=13303935,C=0)
	CT(66425)=(S=13303936,E=13303936,C=1)
	CT(66426)=(S=13303937,E=13303937,C=0)
	CT(66427)=(S=13303938,E=13303943,C=1)
	CT(66428)=(S=13303944,E=13303944,C=0)
	CT(66429)=(S=13303945,E=13303945,C=1)
	CT(66430)=(S=13303946,E=13303949,C=0)
	CT(66431)=(S=13303950,E=13303950,C=1)
	CT(66432)=(S=13303951,E=13303951,C=0)
	CT(66433)=(S=13303952,E=13303952,C=1)
	CT(66434)=(S=13303953,E=13303953,C=0)
	CT(66435)=(S=13303954,E=13303954,C=1)
	CT(66436)=(S=13303955,E=13303955,C=0)
	CT(66437)=(S=13303956,E=13303956,C=1)
	CT(66438)=(S=13303957,E=13303957,C=0)
	CT(66439)=(S=13303958,E=13303960,C=1)
	CT(66440)=(S=13303961,E=13303984,C=0)
	CT(66441)=(S=13303985,E=13303985,C=1)
	CT(66442)=(S=13303986,E=13304031,C=0)
	CT(66443)=(S=13304032,E=13304032,C=1)
	CT(66444)=(S=13304033,E=13304067,C=0)
	CT(66445)=(S=13304068,E=13304071,C=1)
	CT(66446)=(S=13304072,E=13304081,C=0)
	CT(66447)=(S=13304082,E=13304082,C=1)
	CT(66448)=(S=13304083,E=13304088,C=0)
	CT(66449)=(S=13304089,E=13304089,C=74)
	CT(66450)=(S=13304090,E=13304091,C=1)
	CT(66451)=(S=13304092,E=13304128,C=0)
	CT(66452)=(S=13304129,E=13304131,C=1)
	CT(66453)=(S=13304132,E=13304133,C=0)
	CT(66454)=(S=13304134,E=13304135,C=1)
	CT(66455)=(S=13304136,E=13304139,C=0)
	CT(66456)=(S=13304140,E=13304141,C=1)
	CT(66457)=(S=13304142,E=13304153,C=0)
	CT(66458)=(S=13304154,E=13304154,C=1)
	CT(66459)=(S=13304155,E=13304160,C=0)
	CT(66460)=(S=13304161,E=13304167,C=1)
	CT(66461)=(S=13304168,E=13304171,C=0)
	CT(66462)=(S=13304172,E=13304172,C=1)
	CT(66463)=(S=13304173,E=13304316,C=0)
	CT(66464)=(S=13304317,E=13304318,C=1)
	CT(66465)=(S=13304319,E=13304383,C=0)
	CT(66466)=(S=13304384,E=13304391,C=1)
	CT(66467)=(S=13304392,E=13304392,C=0)
	CT(66468)=(S=13304393,E=13304393,C=1)
	CT(66469)=(S=13304394,E=13304431,C=0)
	CT(66470)=(S=13304432,E=13304439,C=1)
	CT(66471)=(S=13304440,E=13304445,C=0)
	CT(66472)=(S=13304446,E=13304447,C=1)
	CT(66473)=(S=13304448,E=13304459,C=0)
	CT(66474)=(S=13304460,E=13304460,C=1)
	CT(66475)=(S=13304461,E=13304469,C=0)
	CT(66476)=(S=13304470,E=13304470,C=1)
	CT(66477)=(S=13304471,E=13304471,C=0)
	CT(66478)=(S=13304472,E=13304477,C=1)
	CT(66479)=(S=13304478,E=13304479,C=0)
	CT(66480)=(S=13304480,E=13304487,C=1)
	CT(66481)=(S=13304488,E=13304499,C=0)
	CT(66482)=(S=13304500,E=13304501,C=1)
	CT(66483)=(S=13304502,E=13304515,C=0)
	CT(66484)=(S=13304516,E=13304517,C=1)
	CT(66485)=(S=13304518,E=13304528,C=0)
	CT(66486)=(S=13304529,E=13304529,C=1)
	CT(66487)=(S=13304530,E=13304533,C=0)
	CT(66488)=(S=13304534,E=13304535,C=1)
	CT(66489)=(S=13304536,E=13304545,C=0)
	CT(66490)=(S=13304546,E=13304547,C=1)
	CT(66491)=(S=13304548,E=13304548,C=2)
	CT(66492)=(S=13304549,E=13304549,C=1)
	CT(66493)=(S=13304550,E=13304555,C=0)
	CT(66494)=(S=13304556,E=13304557,C=1)
	CT(66495)=(S=13304558,E=13304643,C=0)
	CT(66496)=(S=13304644,E=13304644,C=1)
	CT(66497)=(S=13304645,E=13304647,C=0)
	CT(66498)=(S=13304648,E=13304649,C=1)
	CT(66499)=(S=13304650,E=13304650,C=0)
	CT(66500)=(S=13304651,E=13304651,C=1)
	CT(66501)=(S=13304652,E=13304655,C=0)
	CT(66502)=(S=13304656,E=13304663,C=1)
	CT(66503)=(S=13304664,E=13304671,C=0)
	CT(66504)=(S=13304672,E=13304675,C=1)
	CT(66505)=(S=13304676,E=13304680,C=0)
	CT(66506)=(S=13304681,E=13304681,C=1)
	CT(66507)=(S=13304682,E=13304687,C=0)
	CT(66508)=(S=13304688,E=13304696,C=1)
	CT(66509)=(S=13304697,E=13304698,C=0)
	CT(66510)=(S=13304699,E=13304699,C=1)
	CT(66511)=(S=13304700,E=13304710,C=0)
	CT(66512)=(S=13304711,E=13304711,C=1)
	CT(66513)=(S=13304712,E=13304714,C=0)
	CT(66514)=(S=13304715,E=13304715,C=1)
	CT(66515)=(S=13304716,E=13304718,C=0)
	CT(66516)=(S=13304719,E=13304719,C=1)
	CT(66517)=(S=13304720,E=13304963,C=0)
	CT(66518)=(S=13304964,E=13304966,C=1)
	CT(66519)=(S=13304967,E=13304982,C=0)
	CT(66520)=(S=13304983,E=13304987,C=1)
	CT(66521)=(S=13304988,E=13304991,C=0)
	CT(66522)=(S=13304992,E=13304992,C=3)
	CT(66523)=(S=13304993,E=13305005,C=0)
	CT(66524)=(S=13305006,E=13305007,C=1)
	CT(66525)=(S=13305008,E=13305011,C=0)
	CT(66526)=(S=13305012,E=13305012,C=1)
	CT(66527)=(S=13305013,E=13305017,C=0)
	CT(66528)=(S=13305018,E=13305018,C=1)
	CT(66529)=(S=13305019,E=13305036,C=0)
	CT(66530)=(S=13305037,E=13305037,C=1)
	CT(66531)=(S=13305038,E=13305039,C=0)
	CT(66532)=(S=13305040,E=13305043,C=1)
	CT(66533)=(S=13305044,E=13305058,C=0)
	CT(66534)=(S=13305059,E=13305059,C=1)
	CT(66535)=(S=13305060,E=13305061,C=0)
	CT(66536)=(S=13305062,E=13305063,C=1)
	CT(66537)=(S=13305064,E=13305091,C=0)
	CT(66538)=(S=13305092,E=13305093,C=1)
	CT(66539)=(S=13305094,E=13305094,C=0)
	CT(66540)=(S=13305095,E=13305097,C=1)
	CT(66541)=(S=13305098,E=13305098,C=0)
	CT(66542)=(S=13305099,E=13305099,C=1)
	CT(66543)=(S=13305100,E=13305108,C=0)
	CT(66544)=(S=13305109,E=13305110,C=1)
	CT(66545)=(S=13305111,E=13305115,C=0)
	CT(66546)=(S=13305116,E=13305116,C=73)
	CT(66547)=(S=13305117,E=13305131,C=0)
	CT(66548)=(S=13305132,E=13305132,C=1)
	CT(66549)=(S=13305133,E=13305133,C=0)
	CT(66550)=(S=13305134,E=13305135,C=1)
	CT(66551)=(S=13305136,E=13305139,C=0)
	CT(66552)=(S=13305140,E=13305145,C=1)
	CT(66553)=(S=13305146,E=13305147,C=0)
	CT(66554)=(S=13305148,E=13305149,C=1)
	CT(66555)=(S=13305150,E=13305201,C=0)
	CT(66556)=(S=13305202,E=13305203,C=1)
	CT(66557)=(S=13305204,E=13305205,C=0)
	CT(66558)=(S=13305206,E=13305206,C=1)
	CT(66559)=(S=13305207,E=13305207,C=0)
	CT(66560)=(S=13305208,E=13305208,C=1)
	CT(66561)=(S=13305209,E=13305259,C=0)
	CT(66562)=(S=13305260,E=13305260,C=1)
	CT(66563)=(S=13305261,E=13305267,C=0)
	CT(66564)=(S=13305268,E=13305270,C=1)
	CT(66565)=(S=13305271,E=13305272,C=0)
	CT(66566)=(S=13305273,E=13305274,C=1)
	CT(66567)=(S=13305275,E=13305275,C=0)
	CT(66568)=(S=13305276,E=13305278,C=1)
	CT(66569)=(S=13305279,E=13305282,C=0)
	CT(66570)=(S=13305283,E=13305283,C=1)
	CT(66571)=(S=13305284,E=13305301,C=0)
	CT(66572)=(S=13305302,E=13305303,C=1)
	CT(66573)=(S=13305304,E=13305305,C=0)
	CT(66574)=(S=13305306,E=13305307,C=1)
	CT(66575)=(S=13305308,E=13305474,C=0)
	CT(66576)=(S=13305475,E=13305475,C=1)
	CT(66577)=(S=13305476,E=13305479,C=0)
	CT(66578)=(S=13305480,E=13305480,C=1)
	CT(66579)=(S=13305481,E=13305481,C=0)
	CT(66580)=(S=13305482,E=13305483,C=1)
	CT(66581)=(S=13305484,E=13305485,C=0)
	CT(66582)=(S=13305486,E=13305486,C=1)
	CT(66583)=(S=13305487,E=13305491,C=0)
	CT(66584)=(S=13305492,E=13305493,C=74)
	CT(66585)=(S=13305494,E=13305495,C=1)
	CT(66586)=(S=13305496,E=13305500,C=0)
	CT(66587)=(S=13305501,E=13305501,C=1)
	CT(66588)=(S=13305502,E=13305502,C=0)
	CT(66589)=(S=13305503,E=13305503,C=1)
	CT(66590)=(S=13305504,E=13305567,C=0)
	CT(66591)=(S=13305568,E=13305583,C=1)
	CT(66592)=(S=13305584,E=13305589,C=0)
	CT(66593)=(S=13305590,E=13305590,C=73)
	CT(66594)=(S=13305591,E=13305591,C=0)
	CT(66595)=(S=13305592,E=13305593,C=1)
	CT(66596)=(S=13305594,E=13305728,C=0)
	CT(66597)=(S=13305729,E=13305729,C=1)
	CT(66598)=(S=13305730,E=13305737,C=0)
	CT(66599)=(S=13305738,E=13305739,C=1)
	CT(66600)=(S=13305740,E=13305746,C=0)
	CT(66601)=(S=13305747,E=13305747,C=1)
	CT(66602)=(S=13305748,E=13305749,C=0)
	CT(66603)=(S=13305750,E=13305751,C=1)
	CT(66604)=(S=13305752,E=13305757,C=0)
	CT(66605)=(S=13305758,E=13305758,C=1)
	CT(66606)=(S=13305759,E=13305759,C=0)
	CT(66607)=(S=13305760,E=13305767,C=8)
	CT(66608)=(S=13305768,E=13305770,C=0)
	CT(66609)=(S=13305771,E=13305771,C=74)
	CT(66610)=(S=13305772,E=13305791,C=0)
	CT(66611)=(S=13305792,E=13305793,C=1)
	CT(66612)=(S=13305794,E=13305799,C=0)
	CT(66613)=(S=13305800,E=13305800,C=1)
	CT(66614)=(S=13305801,E=13305855,C=0)
	CT(66615)=(S=13305856,E=13305856,C=1)
	CT(66616)=(S=13305857,E=13305863,C=0)
	CT(66617)=(S=13305864,E=13305864,C=1)
	CT(66618)=(S=13305865,E=13305878,C=0)
	CT(66619)=(S=13305879,E=13305887,C=1)
	CT(66620)=(S=13305888,E=13305925,C=0)
	CT(66621)=(S=13305926,E=13305926,C=1)
	CT(66622)=(S=13305927,E=13305937,C=0)
	CT(66623)=(S=13305938,E=13305938,C=1)
	CT(66624)=(S=13305939,E=13305941,C=0)
	CT(66625)=(S=13305942,E=13305943,C=1)
	CT(66626)=(S=13305944,E=13305946,C=0)
	CT(66627)=(S=13305947,E=13305947,C=1)
	CT(66628)=(S=13305948,E=13305964,C=0)
	CT(66629)=(S=13305965,E=13305965,C=4)
	CT(66630)=(S=13305966,E=13305967,C=1)
	CT(66631)=(S=13305968,E=13305970,C=0)
	CT(66632)=(S=13305971,E=13305971,C=1)
	CT(66633)=(S=13305972,E=13305973,C=4)
	CT(66634)=(S=13305974,E=13305982,C=0)
	CT(66635)=(S=13305983,E=13305983,C=11)
	CT(66636)=(S=13305984,E=13306021,C=0)
	CT(66637)=(S=13306022,E=13306023,C=1)
	CT(66638)=(S=13306024,E=13306024,C=0)
	CT(66639)=(S=13306025,E=13306025,C=1)
	CT(66640)=(S=13306026,E=13306027,C=0)
	CT(66641)=(S=13306028,E=13306028,C=11)
	CT(66642)=(S=13306029,E=13306029,C=1)
	CT(66643)=(S=13306030,E=13306039,C=0)
	CT(66644)=(S=13306040,E=13306040,C=1)
	CT(66645)=(S=13306041,E=13306041,C=0)
	CT(66646)=(S=13306042,E=13306043,C=1)
	CT(66647)=(S=13306044,E=13306045,C=0)
	CT(66648)=(S=13306046,E=13306048,C=1)
	CT(66649)=(S=13306049,E=13306052,C=0)
	CT(66650)=(S=13306053,E=13306055,C=1)
	CT(66651)=(S=13306056,E=13306056,C=0)
	CT(66652)=(S=13306057,E=13306057,C=4)
	CT(66653)=(S=13306058,E=13306058,C=0)
	CT(66654)=(S=13306059,E=13306059,C=1)
	CT(66655)=(S=13306060,E=13306064,C=0)
	CT(66656)=(S=13306065,E=13306071,C=1)
	CT(66657)=(S=13306072,E=13306072,C=0)
	CT(66658)=(S=13306073,E=13306073,C=1)
	CT(66659)=(S=13306074,E=13306075,C=0)
	CT(66660)=(S=13306076,E=13306076,C=1)
	CT(66661)=(S=13306077,E=13306143,C=0)
	CT(66662)=(S=13306144,E=13306144,C=1)
	CT(66663)=(S=13306145,E=13306147,C=0)
	CT(66664)=(S=13306148,E=13306149,C=1)
	CT(66665)=(S=13306150,E=13306168,C=0)
	CT(66666)=(S=13306169,E=13306169,C=1)
	CT(66667)=(S=13306170,E=13306174,C=0)
	CT(66668)=(S=13306175,E=13306175,C=1)
	CT(66669)=(S=13306176,E=13306176,C=0)
	CT(66670)=(S=13306177,E=13306177,C=1)
	CT(66671)=(S=13306178,E=13306181,C=0)
	CT(66672)=(S=13306182,E=13306184,C=1)
	CT(66673)=(S=13306185,E=13306186,C=0)
	CT(66674)=(S=13306187,E=13306189,C=1)
	CT(66675)=(S=13306190,E=13306191,C=7)
	CT(66676)=(S=13306192,E=13306207,C=0)
	CT(66677)=(S=13306208,E=13306213,C=1)
	CT(66678)=(S=13306214,E=13306219,C=0)
	CT(66679)=(S=13306220,E=13306220,C=1)
	CT(66680)=(S=13306221,E=13306269,C=0)
	CT(66681)=(S=13306270,E=13306270,C=1)
	CT(66682)=(S=13306271,E=13306287,C=0)
	CT(66683)=(S=13306288,E=13306295,C=73)
	CT(66684)=(S=13306296,E=13306301,C=0)
	CT(66685)=(S=13306302,E=13306303,C=2)
	CT(66686)=(S=13306304,E=13306383,C=0)
	CT(66687)=(S=13306384,E=13306391,C=5)
	CT(66688)=(S=13306392,E=13306401,C=0)
	CT(66689)=(S=13306402,E=13306402,C=1)
	CT(66690)=(S=13306403,E=13306423,C=0)
	CT(66691)=(S=13306424,E=13306424,C=1)
	CT(66692)=(S=13306425,E=13306425,C=0)
	CT(66693)=(S=13306426,E=13306426,C=5)
	CT(66694)=(S=13306427,E=13306441,C=0)
	CT(66695)=(S=13306442,E=13306443,C=1)
	CT(66696)=(S=13306444,E=13306451,C=0)
	CT(66697)=(S=13306452,E=13306456,C=1)
	CT(66698)=(S=13306457,E=13306462,C=0)
	CT(66699)=(S=13306463,E=13306463,C=1)
	CT(66700)=(S=13306464,E=13306492,C=0)
	CT(66701)=(S=13306493,E=13306493,C=1)
	CT(66702)=(S=13306494,E=13306693,C=0)
	CT(66703)=(S=13306694,E=13306694,C=1)
	CT(66704)=(S=13306695,E=13306695,C=0)
	CT(66705)=(S=13306696,E=13306696,C=4)
	CT(66706)=(S=13306697,E=13306697,C=0)
	CT(66707)=(S=13306698,E=13306698,C=3)
	CT(66708)=(S=13306699,E=13306699,C=0)
	CT(66709)=(S=13306700,E=13306703,C=1)
	CT(66710)=(S=13306704,E=13306705,C=0)
	CT(66711)=(S=13306706,E=13306706,C=1)
	CT(66712)=(S=13306707,E=13306707,C=0)
	CT(66713)=(S=13306708,E=13306711,C=1)
	CT(66714)=(S=13306712,E=13306723,C=0)
	CT(66715)=(S=13306724,E=13306727,C=1)
	CT(66716)=(S=13306728,E=13306732,C=0)
	CT(66717)=(S=13306733,E=13306733,C=1)
	CT(66718)=(S=13306734,E=13306740,C=0)
	CT(66719)=(S=13306741,E=13306741,C=1)
	CT(66720)=(S=13306742,E=13306745,C=0)
	CT(66721)=(S=13306746,E=13306746,C=1)
	CT(66722)=(S=13306747,E=13306749,C=0)
	CT(66723)=(S=13306750,E=13306750,C=1)
	CT(66724)=(S=13306751,E=13306751,C=0)
	CT(66725)=(S=13306752,E=13306759,C=8)
	CT(66726)=(S=13306760,E=13306763,C=1)
	CT(66727)=(S=13306764,E=13306764,C=0)
	CT(66728)=(S=13306765,E=13306767,C=1)
	CT(66729)=(S=13306768,E=13306803,C=0)
	CT(66730)=(S=13306804,E=13306807,C=1)
	CT(66731)=(S=13306808,E=13306831,C=0)
	CT(66732)=(S=13306832,E=13306835,C=1)
	CT(66733)=(S=13306836,E=13306895,C=0)
	CT(66734)=(S=13306896,E=13306896,C=1)
	CT(66735)=(S=13306897,E=13306897,C=0)
	CT(66736)=(S=13306898,E=13306898,C=10)
	CT(66737)=(S=13306899,E=13306899,C=1)
	CT(66738)=(S=13306900,E=13306901,C=74)
	CT(66739)=(S=13306902,E=13306903,C=0)
	CT(66740)=(S=13306904,E=13306904,C=1)
	CT(66741)=(S=13306905,E=13306921,C=0)
	CT(66742)=(S=13306922,E=13306922,C=2)
	CT(66743)=(S=13306923,E=13306936,C=0)
	CT(66744)=(S=13306937,E=13306937,C=1)
	CT(66745)=(S=13306938,E=13306944,C=0)
	CT(66746)=(S=13306945,E=13306946,C=1)
	CT(66747)=(S=13306947,E=13306949,C=0)
	CT(66748)=(S=13306950,E=13306951,C=1)
	CT(66749)=(S=13306952,E=13306965,C=0)
	CT(66750)=(S=13306967,E=13306975,C=1)
	CT(66751)=(S=13306976,E=13306976,C=0)
	CT(66752)=(S=13306977,E=13306977,C=2)
	CT(66753)=(S=13306978,E=13306979,C=0)
	CT(66754)=(S=13306980,E=13306981,C=1)
	CT(66755)=(S=13306982,E=13306982,C=0)
	CT(66756)=(S=13306983,E=13306983,C=1)
	CT(66757)=(S=13306984,E=13306993,C=0)
	CT(66758)=(S=13306994,E=13306994,C=1)
	CT(66759)=(S=13306995,E=13306997,C=0)
	CT(66760)=(S=13306998,E=13306998,C=1)
	CT(66761)=(S=13306999,E=13307009,C=0)
	CT(66762)=(S=13307010,E=13307010,C=1)
	CT(66763)=(S=13307011,E=13307016,C=0)
	CT(66764)=(S=13307017,E=13307017,C=1)
	CT(66765)=(S=13307018,E=13307023,C=0)
	CT(66766)=(S=13307024,E=13307031,C=2)
	CT(66767)=(S=13307032,E=13307075,C=0)
	CT(66768)=(S=13307076,E=13307087,C=1)
	CT(66769)=(S=13307088,E=13307090,C=0)
	CT(66770)=(S=13307091,E=13307091,C=1)
	CT(66771)=(S=13307092,E=13307098,C=0)
	CT(66772)=(S=13307099,E=13307099,C=1)
	CT(66773)=(S=13307100,E=13307100,C=5)
	CT(66774)=(S=13307101,E=13307101,C=0)
	CT(66775)=(S=13307102,E=13307103,C=4)
	CT(66776)=(S=13307104,E=13307105,C=0)
	CT(66777)=(S=13307106,E=13307106,C=1)
	CT(66778)=(S=13307107,E=13307119,C=0)
	CT(66779)=(S=13307120,E=13307123,C=1)
	CT(66780)=(S=13307124,E=13307128,C=0)
	CT(66781)=(S=13307129,E=13307129,C=195)
	CT(66782)=(S=13307130,E=13307153,C=0)
	CT(66783)=(S=13307154,E=13307154,C=1)
	CT(66784)=(S=13307155,E=13307159,C=0)
	CT(66785)=(S=13307160,E=13307160,C=1)
	CT(66786)=(S=13307161,E=13307175,C=0)
	CT(66787)=(S=13307176,E=13307176,C=4)
	CT(66788)=(S=13307177,E=13307179,C=0)
	CT(66789)=(S=13307180,E=13307181,C=1)
	CT(66790)=(S=13307182,E=13307215,C=0)
	CT(66791)=(S=13307216,E=13307225,C=1)
	CT(66792)=(S=13307226,E=13307227,C=0)
	CT(66793)=(S=13307228,E=13307231,C=1)
	CT(66794)=(S=13307232,E=13307303,C=0)
	CT(66795)=(S=13307304,E=13307307,C=4)
	CT(66796)=(S=13307308,E=13307308,C=0)
	CT(66797)=(S=13307309,E=13307309,C=1)
	CT(66798)=(S=13307310,E=13307359,C=0)
	CT(66799)=(S=13307360,E=13307361,C=1)
	CT(66800)=(S=13307362,E=13307362,C=0)
	CT(66801)=(S=13307363,E=13307363,C=1)
	CT(66802)=(S=13307364,E=13307368,C=0)
	CT(66803)=(S=13307369,E=13307369,C=1)
	CT(66804)=(S=13307370,E=13307409,C=0)
	CT(66805)=(S=13307410,E=13307410,C=4)
	CT(66806)=(S=13307411,E=13307415,C=0)
	CT(66807)=(S=13307416,E=13307419,C=1)
	CT(66808)=(S=13307420,E=13307423,C=0)
	CT(66809)=(S=13307424,E=13307424,C=4)
	CT(66810)=(S=13307425,E=13307425,C=1)
	CT(66811)=(S=13307426,E=13307447,C=0)
	CT(66812)=(S=13307448,E=13307448,C=1)
	CT(66813)=(S=13307449,E=13307452,C=0)
	CT(66814)=(S=13307453,E=13307454,C=1)
	CT(66815)=(S=13307455,E=13307495,C=0)
	CT(66816)=(S=13307496,E=13307496,C=1)
	CT(66817)=(S=13307497,E=13307505,C=0)
	CT(66818)=(S=13307506,E=13307507,C=1)
	CT(66819)=(S=13307508,E=13307509,C=0)
	CT(66820)=(S=13307510,E=13307510,C=1)
	CT(66821)=(S=13307511,E=13307553,C=0)
	CT(66822)=(S=13307554,E=13307554,C=1)
	CT(66823)=(S=13307555,E=13307557,C=0)
	CT(66824)=(S=13307558,E=13307558,C=8)
	CT(66825)=(S=13307559,E=13307567,C=0)
	CT(66826)=(S=13307568,E=13307568,C=74)
	CT(66827)=(S=13307569,E=13307574,C=0)
	CT(66828)=(S=13307575,E=13307575,C=74)
	CT(66829)=(S=13307576,E=13307584,C=1)
	CT(66830)=(S=13307585,E=13307585,C=0)
	CT(66831)=(S=13307586,E=13307587,C=1)
	CT(66832)=(S=13307588,E=13307603,C=0)
	CT(66833)=(S=13307604,E=13307604,C=2)
	CT(66834)=(S=13307605,E=13307605,C=0)
	CT(66835)=(S=13307606,E=13307606,C=1)
	CT(66836)=(S=13307607,E=13307622,C=0)
	CT(66837)=(S=13307623,E=13307623,C=1)
	CT(66838)=(S=13307624,E=13307637,C=0)
	CT(66839)=(S=13307638,E=13307638,C=1)
	CT(66840)=(S=13307639,E=13307647,C=0)
	CT(66841)=(S=13307648,E=13307663,C=1)
	CT(66842)=(S=13307664,E=13307667,C=0)
	CT(66843)=(S=13307668,E=13307670,C=1)
	CT(66844)=(S=13307671,E=13307715,C=0)
	CT(66845)=(S=13307716,E=13307716,C=2)
	CT(66846)=(S=13307717,E=13307734,C=0)
	CT(66847)=(S=13307735,E=13307737,C=1)
	CT(66848)=(S=13307738,E=13307752,C=0)
	CT(66849)=(S=13307753,E=13307753,C=1)
	CT(66850)=(S=13307754,E=13307759,C=0)
	CT(66851)=(S=13307760,E=13307767,C=1)
	CT(66852)=(S=13307768,E=13307777,C=0)
	CT(66853)=(S=13307778,E=13307779,C=1)
	CT(66854)=(S=13307780,E=13307796,C=0)
	CT(66855)=(S=13307797,E=13307797,C=1)
	CT(66856)=(S=13307798,E=13307798,C=0)
	CT(66857)=(S=13307799,E=13307799,C=1)
	CT(66858)=(S=13307800,E=13307803,C=0)
	CT(66859)=(S=13307804,E=13307807,C=1)
	CT(66860)=(S=13307808,E=13307821,C=0)
	CT(66861)=(S=13307822,E=13307822,C=1)
	CT(66862)=(S=13307823,E=13307874,C=0)
	CT(66863)=(S=13307875,E=13307875,C=1)
	CT(66864)=(S=13307876,E=13307879,C=0)
	CT(66865)=(S=13307880,E=13307889,C=1)
	CT(66866)=(S=13307890,E=13307893,C=0)
	CT(66867)=(S=13307894,E=13307894,C=1)
	CT(66868)=(S=13307895,E=13307913,C=0)
	CT(66869)=(S=13307914,E=13307917,C=1)
	CT(66870)=(S=13307918,E=13307919,C=0)
	CT(66871)=(S=13307920,E=13307927,C=1)
	CT(66872)=(S=13307928,E=13307930,C=0)
	CT(66873)=(S=13307931,E=13307931,C=1)
	CT(66874)=(S=13307932,E=13307937,C=0)
	CT(66875)=(S=13307938,E=13307938,C=77)
	CT(66876)=(S=13307939,E=13307941,C=0)
	CT(66877)=(S=13307942,E=13307942,C=1)
	CT(66878)=(S=13307943,E=13307952,C=0)
	CT(66879)=(S=13307953,E=13307955,C=1)
	CT(66880)=(S=13307956,E=13307961,C=0)
	CT(66881)=(S=13307962,E=13307962,C=1)
	CT(66882)=(S=13307963,E=13308036,C=0)
	CT(66883)=(S=13308037,E=13308037,C=1)
	CT(66884)=(S=13308038,E=13308064,C=0)
	CT(66885)=(S=13308065,E=13308066,C=1)
	CT(66886)=(S=13308067,E=13308068,C=0)
	CT(66887)=(S=13308069,E=13308069,C=4)
	CT(66888)=(S=13308070,E=13308083,C=0)
	CT(66889)=(S=13308084,E=13308087,C=2)
	CT(66890)=(S=13308088,E=13308089,C=0)
	CT(66891)=(S=13308090,E=13308091,C=1)
	CT(66892)=(S=13308092,E=13308131,C=0)
	CT(66893)=(S=13308132,E=13308132,C=1)
	CT(66894)=(S=13308133,E=13308141,C=0)
	CT(66895)=(S=13308142,E=13308142,C=1)
	CT(66896)=(S=13308143,E=13308143,C=8)
	CT(66897)=(S=13308144,E=13308144,C=1)
	CT(66898)=(S=13308145,E=13308148,C=0)
	CT(66899)=(S=13308149,E=13308149,C=1)
	CT(66900)=(S=13308150,E=13308161,C=0)
	CT(66901)=(S=13308162,E=13308162,C=1)
	CT(66902)=(S=13308163,E=13308177,C=0)
	CT(66903)=(S=13308178,E=13308178,C=1)
	CT(66904)=(S=13308179,E=13308180,C=0)
	CT(66905)=(S=13308181,E=13308181,C=74)
	CT(66906)=(S=13308182,E=13308187,C=0)
	CT(66907)=(S=13308188,E=13308188,C=1)
	CT(66908)=(S=13308189,E=13308190,C=0)
	CT(66909)=(S=13308191,E=13308191,C=4)
	CT(66910)=(S=13308192,E=13308198,C=0)
	CT(66911)=(S=13308199,E=13308199,C=1)
	CT(66912)=(S=13308200,E=13308207,C=2)
	CT(66913)=(S=13308208,E=13308215,C=0)
	CT(66914)=(S=13308216,E=13308216,C=1)
	CT(66915)=(S=13308217,E=13308233,C=0)
	CT(66916)=(S=13308234,E=13308235,C=1)
	CT(66917)=(S=13308236,E=13308247,C=0)
	CT(66918)=(S=13308248,E=13308249,C=1)
	CT(66919)=(S=13308250,E=13308295,C=0)
	CT(66920)=(S=13308296,E=13308296,C=1)
	CT(66921)=(S=13308297,E=13308321,C=0)
	CT(66922)=(S=13308322,E=13308322,C=2)
	CT(66923)=(S=13308323,E=13308323,C=0)
	CT(66924)=(S=13308324,E=13308324,C=1)
	CT(66925)=(S=13308325,E=13308346,C=0)
	CT(66926)=(S=13308347,E=13308347,C=1)
	CT(66927)=(S=13308348,E=13308348,C=0)
	CT(66928)=(S=13308350,E=13308351,C=1)
	CT(66929)=(S=13308352,E=13308353,C=0)
	CT(66930)=(S=13308354,E=13308354,C=4)
	CT(66931)=(S=13308355,E=13308385,C=0)
	CT(66932)=(S=13308386,E=13308386,C=6)
	CT(66933)=(S=13308387,E=13308390,C=0)
	CT(66934)=(S=13308391,E=13308391,C=1)
	CT(66935)=(S=13308392,E=13308392,C=0)
	CT(66936)=(S=13308393,E=13308393,C=1)
	CT(66937)=(S=13308394,E=13308399,C=0)
	CT(66938)=(S=13308400,E=13308403,C=195)
	CT(66939)=(S=13308404,E=13308407,C=0)
	CT(66940)=(S=13308408,E=13308408,C=1)
	CT(66941)=(S=13308410,E=13308414,C=0)
	CT(66942)=(S=13308415,E=13308415,C=1)
	CT(66943)=(S=13308416,E=13308417,C=0)
	CT(66944)=(S=13308418,E=13308420,C=1)
	CT(66945)=(S=13308421,E=13308422,C=0)
	CT(66946)=(S=13308423,E=13308423,C=1)
	CT(66947)=(S=13308424,E=13308441,C=0)
	CT(66948)=(S=13308442,E=13308442,C=4)
	CT(66949)=(S=13308443,E=13308446,C=0)
	CT(66950)=(S=13308447,E=13308447,C=1)
	CT(66951)=(S=13308448,E=13308452,C=0)
	CT(66952)=(S=13308453,E=13308453,C=1)
	CT(66953)=(S=13308454,E=13308463,C=0)
	CT(66954)=(S=13308464,E=13308465,C=1)
	CT(66955)=(S=13308466,E=13308466,C=8)
	CT(66956)=(S=13308467,E=13308467,C=4)
	CT(66957)=(S=13308468,E=13308468,C=1)
	CT(66958)=(S=13308469,E=13308487,C=0)
	CT(66959)=(S=13308488,E=13308491,C=1)
	CT(66960)=(S=13308492,E=13308495,C=0)
	CT(66961)=(S=13308496,E=13308497,C=1)
	CT(66962)=(S=13308498,E=13308502,C=0)
	CT(66963)=(S=13308503,E=13308503,C=1)
	CT(66964)=(S=13308504,E=13308515,C=0)
	CT(66965)=(S=13308516,E=13308517,C=1)
	CT(66966)=(S=13308518,E=13308520,C=0)
	CT(66967)=(S=13308521,E=13308521,C=1)
	CT(66968)=(S=13308522,E=13308522,C=0)
	CT(66969)=(S=13308523,E=13308523,C=1)
	CT(66970)=(S=13308524,E=13308525,C=0)
	CT(66971)=(S=13308526,E=13308526,C=1)
	CT(66972)=(S=13308528,E=13308544,C=0)
	CT(66973)=(S=13308545,E=13308545,C=1)
	CT(66974)=(S=13308546,E=13308546,C=0)
	CT(66975)=(S=13308547,E=13308549,C=1)
	CT(66976)=(S=13308550,E=13308552,C=0)
	CT(66977)=(S=13308553,E=13308553,C=4)
	CT(66978)=(S=13308554,E=13308559,C=0)
	CT(66979)=(S=13308560,E=13308560,C=1)
	CT(66980)=(S=13308561,E=13308568,C=0)
	CT(66981)=(S=13308569,E=13308569,C=1)
	CT(66982)=(S=13308570,E=13308614,C=0)
	CT(66983)=(S=13308615,E=13308615,C=1)
	CT(66984)=(S=13308616,E=13308623,C=0)
	CT(66985)=(S=13308624,E=13308624,C=1)
	CT(66986)=(S=13308625,E=13308626,C=0)
	CT(66987)=(S=13308627,E=13308627,C=1)
	CT(66988)=(S=13308628,E=13308630,C=0)
	CT(66989)=(S=13308631,E=13308631,C=1)
	CT(66990)=(S=13308632,E=13308674,C=0)
	CT(66991)=(S=13308675,E=13308675,C=4)
	CT(66992)=(S=13308676,E=13308676,C=74)
	CT(66993)=(S=13308677,E=13308689,C=0)
	CT(66994)=(S=13308690,E=13308690,C=1)
	CT(66995)=(S=13308691,E=13308695,C=0)
	CT(66996)=(S=13308696,E=13308696,C=1)
	CT(66997)=(S=13308697,E=13308701,C=0)
	CT(66998)=(S=13308702,E=13308702,C=1)
	CT(66999)=(S=13308703,E=13308703,C=0)
	CT(67000)=(S=13308704,E=13308711,C=1)
	CT(67001)=(S=13308712,E=13308712,C=0)
	CT(67002)=(S=13308713,E=13308713,C=1)
	CT(67003)=(S=13308714,E=13308715,C=0)
	CT(67004)=(S=13308716,E=13308718,C=1)
	CT(67005)=(S=13308719,E=13308729,C=0)
	CT(67006)=(S=13308730,E=13308730,C=1)
	CT(67007)=(S=13308731,E=13308731,C=0)
	CT(67008)=(S=13308732,E=13308733,C=1)
	CT(67009)=(S=13308734,E=13308735,C=0)
	CT(67010)=(S=13308736,E=13308736,C=1)
	CT(67011)=(S=13308737,E=13308739,C=0)
	CT(67012)=(S=13308740,E=13308740,C=1)
	CT(67013)=(S=13308741,E=13308743,C=0)
	CT(67014)=(S=13308744,E=13308744,C=1)
	CT(67015)=(S=13308745,E=13308772,C=0)
	CT(67016)=(S=13308773,E=13308773,C=1)
	CT(67017)=(S=13308774,E=13308782,C=0)
	CT(67018)=(S=13308783,E=13308783,C=1)
	CT(67019)=(S=13308784,E=13308802,C=0)
	CT(67020)=(S=13308803,E=13308803,C=1)
	CT(67021)=(S=13308804,E=13308804,C=0)
	CT(67022)=(S=13308805,E=13308805,C=1)
	CT(67023)=(S=13308806,E=13308815,C=0)
	CT(67024)=(S=13308816,E=13308816,C=1)
	CT(67025)=(S=13308817,E=13308820,C=0)
	CT(67026)=(S=13308821,E=13308821,C=1)
	CT(67027)=(S=13308822,E=13308827,C=0)
	CT(67028)=(S=13308828,E=13308828,C=1)
	CT(67029)=(S=13308829,E=13308847,C=0)
	CT(67030)=(S=13308848,E=13308848,C=1)
	CT(67031)=(S=13308849,E=13308849,C=0)
	CT(67032)=(S=13308850,E=13308851,C=1)
	CT(67033)=(S=13308852,E=13308879,C=0)
	CT(67034)=(S=13308880,E=13308880,C=1)
	CT(67035)=(S=13308881,E=13308899,C=0)
	CT(67036)=(S=13308900,E=13308903,C=1)
	CT(67037)=(S=13308904,E=13308904,C=0)
	CT(67038)=(S=13308905,E=13308905,C=1)
	CT(67039)=(S=13308906,E=13308913,C=0)
	CT(67040)=(S=13308914,E=13308914,C=1)
	CT(67041)=(S=13308915,E=13308919,C=0)
	CT(67042)=(S=13308920,E=13308921,C=1)
	CT(67043)=(S=13308922,E=13308926,C=0)
	CT(67044)=(S=13308927,E=13308927,C=1)
	CT(67045)=(S=13308928,E=13308944,C=0)
	CT(67046)=(S=13308945,E=13308945,C=1)
	CT(67047)=(S=13308946,E=13308967,C=0)
	CT(67048)=(S=13308968,E=13308969,C=1)
	CT(67049)=(S=13308971,E=13308975,C=0)
	CT(67050)=(S=13308976,E=13308976,C=1)
	CT(67051)=(S=13308977,E=13308988,C=0)
	CT(67052)=(S=13308989,E=13308989,C=1)
	CT(67053)=(S=13308990,E=13308992,C=0)
	CT(67054)=(S=13308993,E=13308993,C=1)
	CT(67055)=(S=13308994,E=13309011,C=0)
	CT(67056)=(S=13309012,E=13309013,C=1)
	CT(67057)=(S=13309014,E=13309016,C=0)
	CT(67058)=(S=13309017,E=13309017,C=1)
	CT(67059)=(S=13309018,E=13309032,C=0)
	CT(67060)=(S=13309033,E=13309033,C=7)
	CT(67061)=(S=13309034,E=13309035,C=1)
	CT(67062)=(S=13309036,E=13309042,C=0)
	CT(67063)=(S=13309043,E=13309043,C=1)
	CT(67064)=(S=13309044,E=13309044,C=0)
	CT(67065)=(S=13309045,E=13309047,C=1)
	CT(67066)=(S=13309048,E=13309049,C=0)
	CT(67067)=(S=13309050,E=13309050,C=1)
	CT(67068)=(S=13309051,E=13309053,C=0)
	CT(67069)=(S=13309054,E=13309055,C=1)
	CT(67070)=(S=13309056,E=13309062,C=0)
	CT(67071)=(S=13309063,E=13309071,C=1)
	CT(67072)=(S=13309072,E=13309077,C=0)
	CT(67073)=(S=13309078,E=13309078,C=1)
	CT(67074)=(S=13309079,E=13309079,C=0)
	CT(67075)=(S=13309080,E=13309087,C=8)
	CT(67076)=(S=13309088,E=13309157,C=0)
	CT(67077)=(S=13309158,E=13309158,C=1)
	CT(67078)=(S=13309159,E=13309159,C=0)
	CT(67079)=(S=13309160,E=13309160,C=1)
	CT(67080)=(S=13309161,E=13309163,C=0)
	CT(67081)=(S=13309164,E=13309164,C=1)
	CT(67082)=(S=13309165,E=13309168,C=0)
	CT(67083)=(S=13309169,E=13309169,C=73)
	CT(67084)=(S=13309170,E=13309183,C=0)
	CT(67085)=(S=13309184,E=13309186,C=1)
	CT(67086)=(S=13309187,E=13309190,C=0)
	CT(67087)=(S=13309191,E=13309191,C=4)
	CT(67088)=(S=13309192,E=13309192,C=1)
	CT(67089)=(S=13309193,E=13309193,C=0)
	CT(67090)=(S=13309194,E=13309194,C=1)
	CT(67091)=(S=13309195,E=13309201,C=0)
	CT(67092)=(S=13309202,E=13309202,C=1)
	CT(67093)=(S=13309203,E=13309216,C=0)
	CT(67094)=(S=13309217,E=13309218,C=1)
	CT(67095)=(S=13309219,E=13309224,C=0)
	CT(67096)=(S=13309225,E=13309225,C=1)
	CT(67097)=(S=13309226,E=13309227,C=3)
	CT(67098)=(S=13309228,E=13309228,C=1)
	CT(67099)=(S=13309229,E=13309251,C=0)
	CT(67100)=(S=13309252,E=13309252,C=1)
	CT(67101)=(S=13309253,E=13309257,C=0)
	CT(67102)=(S=13309258,E=13309259,C=74)
	CT(67103)=(S=13309260,E=13309265,C=0)
	CT(67104)=(S=13309266,E=13309266,C=1)
	CT(67105)=(S=13309267,E=13309279,C=0)
	CT(67106)=(S=13309280,E=13309283,C=1)
	CT(67107)=(S=13309284,E=13309307,C=0)
	CT(67108)=(S=13309308,E=13309308,C=1)
	CT(67109)=(S=13309309,E=13309319,C=0)
	CT(67110)=(S=13309320,E=13309321,C=1)
	CT(67111)=(S=13309322,E=13309328,C=0)
	CT(67112)=(S=13309329,E=13309329,C=1)
	CT(67113)=(S=13309330,E=13309389,C=0)
	CT(67114)=(S=13309390,E=13309390,C=1)
	CT(67115)=(S=13309391,E=13309463,C=0)
	CT(67116)=(S=13309464,E=13309464,C=1)
	CT(67117)=(S=13309465,E=13309467,C=0)
	CT(67118)=(S=13309468,E=13309469,C=1)
	CT(67119)=(S=13309470,E=13309470,C=0)
	CT(67120)=(S=13309471,E=13309471,C=1)
	CT(67121)=(S=13309472,E=13309507,C=0)
	CT(67122)=(S=13309508,E=13309508,C=1)
	CT(67123)=(S=13309509,E=13309515,C=0)
	CT(67124)=(S=13309516,E=13309516,C=1)
	CT(67125)=(S=13309517,E=13309517,C=0)
	CT(67126)=(S=13309518,E=13309518,C=1)
	CT(67127)=(S=13309519,E=13309523,C=0)
	CT(67128)=(S=13309524,E=13309524,C=1)
	CT(67129)=(S=13309525,E=13309526,C=0)
	CT(67130)=(S=13309527,E=13309527,C=1)
	CT(67131)=(S=13309528,E=13309531,C=0)
	CT(67132)=(S=13309532,E=13309535,C=1)
	CT(67133)=(S=13309536,E=13309538,C=0)
	CT(67134)=(S=13309539,E=13309539,C=1)
	CT(67135)=(S=13309540,E=13309545,C=0)
	CT(67136)=(S=13309546,E=13309546,C=1)
	CT(67137)=(S=13309547,E=13309561,C=0)
	CT(67138)=(S=13309562,E=13309563,C=1)
	CT(67139)=(S=13309564,E=13309570,C=0)
	CT(67140)=(S=13309571,E=13309571,C=1)
	CT(67141)=(S=13309572,E=13309602,C=0)
	CT(67142)=(S=13309603,E=13309603,C=1)
	CT(67143)=(S=13309604,E=13309605,C=0)
	CT(67144)=(S=13309606,E=13309606,C=1)
	CT(67145)=(S=13309607,E=13309609,C=0)
	CT(67146)=(S=13309610,E=13309610,C=1)
	CT(67147)=(S=13309611,E=13309615,C=0)
	CT(67148)=(S=13309616,E=13309623,C=1)
	CT(67149)=(S=13309624,E=13309633,C=0)
	CT(67150)=(S=13309634,E=13309634,C=1)
	CT(67151)=(S=13309635,E=13309681,C=0)
	CT(67152)=(S=13309682,E=13309683,C=1)
	CT(67153)=(S=13309684,E=13309684,C=0)
	CT(67154)=(S=13309685,E=13309686,C=1)
	CT(67155)=(S=13309687,E=13309691,C=0)
	CT(67156)=(S=13309692,E=13309693,C=1)
	CT(67157)=(S=13309694,E=13309695,C=0)
	CT(67158)=(S=13309696,E=13309696,C=1)
	CT(67159)=(S=13309698,E=13309742,C=0)
	CT(67160)=(S=13309743,E=13309743,C=1)
	CT(67161)=(S=13309744,E=13309756,C=0)
	CT(67162)=(S=13309757,E=13309759,C=1)
	CT(67163)=(S=13309760,E=13309768,C=0)
	CT(67164)=(S=13309769,E=13309769,C=1)
	CT(67165)=(S=13309770,E=13309780,C=0)
	CT(67166)=(S=13309781,E=13309781,C=1)
	CT(67167)=(S=13309782,E=13309787,C=0)
	CT(67168)=(S=13309788,E=13309791,C=1)
	CT(67169)=(S=13309792,E=13309793,C=0)
	CT(67170)=(S=13309794,E=13309794,C=1)
	CT(67171)=(S=13309795,E=13309802,C=0)
	CT(67172)=(S=13309803,E=13309803,C=1)
	CT(67173)=(S=13309804,E=13309807,C=0)
	CT(67174)=(S=13309808,E=13309808,C=1)
	CT(67175)=(S=13309809,E=13309825,C=0)
	CT(67176)=(S=13309826,E=13309826,C=1)
	CT(67177)=(S=13309827,E=13309835,C=0)
	CT(67178)=(S=13309836,E=13309837,C=1)
	CT(67179)=(S=13309838,E=13309867,C=0)
	CT(67180)=(S=13309868,E=13309868,C=1)
	CT(67181)=(S=13309869,E=13309877,C=0)
	CT(67182)=(S=13309878,E=13309878,C=1)
	CT(67183)=(S=13309879,E=13309881,C=0)
	CT(67184)=(S=13309882,E=13309883,C=1)
	CT(67185)=(S=13309884,E=13309887,C=0)
	CT(67186)=(S=13309888,E=13309888,C=1)
	CT(67187)=(S=13309889,E=13309892,C=0)
	CT(67188)=(S=13309893,E=13309894,C=1)
	CT(67189)=(S=13309895,E=13309899,C=0)
	CT(67190)=(S=13309900,E=13309903,C=1)
	CT(67191)=(S=13309904,E=13309919,C=0)
	CT(67192)=(S=13309920,E=13309920,C=1)
	CT(67193)=(S=13309921,E=13309921,C=0)
	CT(67194)=(S=13309922,E=13309927,C=1)
	CT(67195)=(S=13309928,E=13309944,C=0)
	CT(67196)=(S=13309945,E=13309945,C=1)
	CT(67197)=(S=13309946,E=13309946,C=0)
	CT(67198)=(S=13309947,E=13309947,C=1)
	CT(67199)=(S=13309948,E=13309964,C=0)
	CT(67200)=(S=13309965,E=13309965,C=1)
	CT(67201)=(S=13309966,E=13309969,C=0)
	CT(67202)=(S=13309970,E=13309970,C=1)
	CT(67203)=(S=13309971,E=13309978,C=0)
	CT(67204)=(S=13309979,E=13309979,C=1)
	CT(67205)=(S=13309980,E=13309994,C=0)
	CT(67206)=(S=13309995,E=13309995,C=1)
	CT(67207)=(S=13309996,E=13310001,C=0)
	CT(67208)=(S=13310002,E=13310003,C=74)
	CT(67209)=(S=13310004,E=13310007,C=0)
	CT(67210)=(S=13310008,E=13310008,C=1)
	CT(67211)=(S=13310009,E=13310009,C=0)
	CT(67212)=(S=13310010,E=13310010,C=1)
	CT(67213)=(S=13310011,E=13310018,C=0)
	CT(67214)=(S=13310019,E=13310019,C=1)
	CT(67215)=(S=13310020,E=13310025,C=0)
	CT(67216)=(S=13310026,E=13310026,C=1)
	CT(67217)=(S=13310027,E=13310027,C=0)
	CT(67218)=(S=13310028,E=13310029,C=74)
	CT(67219)=(S=13310030,E=13310030,C=0)
	CT(67220)=(S=13310031,E=13310033,C=1)
	CT(67221)=(S=13310034,E=13310035,C=0)
	CT(67222)=(S=13310036,E=13310038,C=1)
	CT(67223)=(S=13310039,E=13310039,C=4)
	CT(67224)=(S=13310040,E=13310041,C=0)
	CT(67225)=(S=13310042,E=13310042,C=1)
	CT(67226)=(S=13310043,E=13310062,C=0)
	CT(67227)=(S=13310063,E=13310064,C=1)
	CT(67228)=(S=13310065,E=13310067,C=0)
	CT(67229)=(S=13310068,E=13310068,C=1)
	CT(67230)=(S=13310069,E=13310073,C=0)
	CT(67231)=(S=13310074,E=13310075,C=1)
	CT(67232)=(S=13310076,E=13310085,C=0)
	CT(67233)=(S=13310086,E=13310087,C=2)
	CT(67234)=(S=13310088,E=13310096,C=0)
	CT(67235)=(S=13310097,E=13310097,C=1)
	CT(67236)=(S=13310098,E=13310103,C=0)
	CT(67237)=(S=13310104,E=13310105,C=1)
	CT(67238)=(S=13310106,E=13310108,C=0)
	CT(67239)=(S=13310109,E=13310109,C=1)
	CT(67240)=(S=13310110,E=13310112,C=0)
	CT(67241)=(S=13310113,E=13310113,C=1)
	CT(67242)=(S=13310114,E=13310118,C=0)
	CT(67243)=(S=13310119,E=13310119,C=1)
	CT(67244)=(S=13310120,E=13310137,C=0)
	CT(67245)=(S=13310138,E=13310139,C=1)
	CT(67246)=(S=13310140,E=13310150,C=0)
	CT(67247)=(S=13310151,E=13310151,C=1)
	CT(67248)=(S=13310152,E=13310153,C=0)
	CT(67249)=(S=13310154,E=13310154,C=1)
	CT(67250)=(S=13310155,E=13310163,C=0)
	CT(67251)=(S=13310164,E=13310165,C=1)
	CT(67252)=(S=13310166,E=13310168,C=0)
	CT(67253)=(S=13310169,E=13310169,C=1)
	CT(67254)=(S=13310170,E=13310170,C=0)
	CT(67255)=(S=13310171,E=13310171,C=1)
	CT(67256)=(S=13310172,E=13310195,C=0)
	CT(67257)=(S=13310196,E=13310196,C=1)
	CT(67258)=(S=13310197,E=13310226,C=0)
	CT(67259)=(S=13310227,E=13310229,C=1)
	CT(67260)=(S=13310230,E=13310253,C=0)
	CT(67261)=(S=13310254,E=13310254,C=1)
	CT(67262)=(S=13310255,E=13310255,C=0)
	CT(67263)=(S=13310256,E=13310263,C=1)
	CT(67264)=(S=13310264,E=13310271,C=0)
	CT(67265)=(S=13310272,E=13310273,C=1)
	CT(67266)=(S=13310274,E=13310298,C=0)
	CT(67267)=(S=13310299,E=13310299,C=1)
	CT(67268)=(S=13310300,E=13310306,C=0)
	CT(67269)=(S=13310307,E=13310308,C=1)
	CT(67270)=(S=13310309,E=13310313,C=0)
	CT(67271)=(S=13310314,E=13310314,C=1)
	CT(67272)=(S=13310315,E=13310315,C=0)
	CT(67273)=(S=13310316,E=13310316,C=4)
	CT(67274)=(S=13310317,E=13310338,C=0)
	CT(67275)=(S=13310339,E=13310339,C=1)
	CT(67276)=(S=13310340,E=13310342,C=0)
	CT(67277)=(S=13310343,E=13310343,C=1)
	CT(67278)=(S=13310345,E=13310345,C=0)
	CT(67279)=(S=13310346,E=13310346,C=1)
	CT(67280)=(S=13310347,E=13310354,C=0)
	CT(67281)=(S=13310355,E=13310355,C=1)
	CT(67282)=(S=13310356,E=13310360,C=0)
	CT(67283)=(S=13310361,E=13310363,C=1)
	CT(67284)=(S=13310364,E=13310371,C=0)
	CT(67285)=(S=13310372,E=13310372,C=1)
	CT(67286)=(S=13310373,E=13310373,C=0)
	CT(67287)=(S=13310374,E=13310374,C=1)
	CT(67288)=(S=13310375,E=13310381,C=0)
	CT(67289)=(S=13310382,E=13310383,C=1)
	CT(67290)=(S=13310384,E=13310387,C=0)
	CT(67291)=(S=13310388,E=13310388,C=1)
	CT(67292)=(S=13310389,E=13310389,C=0)
	CT(67293)=(S=13310390,E=13310390,C=1)
	CT(67294)=(S=13310391,E=13310398,C=0)
	CT(67295)=(S=13310399,E=13310399,C=1)
	CT(67296)=(S=13310400,E=13310406,C=0)
	CT(67297)=(S=13310407,E=13310408,C=1)
	CT(67298)=(S=13310409,E=13310409,C=0)
	CT(67299)=(S=13310410,E=13310411,C=1)
	CT(67300)=(S=13310412,E=13310415,C=0)
	CT(67301)=(S=13310416,E=13310431,C=1)
	CT(67302)=(S=13310432,E=13310436,C=0)
	CT(67303)=(S=13310437,E=13310437,C=1)
	CT(67304)=(S=13310438,E=13310442,C=0)
	CT(67305)=(S=13310443,E=13310444,C=1)
	CT(67306)=(S=13310445,E=13310449,C=0)
	CT(67307)=(S=13310450,E=13310450,C=1)
	CT(67308)=(S=13310451,E=13310471,C=0)
	CT(67309)=(S=13310472,E=13310475,C=2)
	CT(67310)=(S=13310476,E=13310476,C=1)
	CT(67311)=(S=13310477,E=13310497,C=0)
	CT(67312)=(S=13310498,E=13310498,C=1)
	CT(67313)=(S=13310499,E=13310512,C=0)
	CT(67314)=(S=13310513,E=13310514,C=1)
	CT(67315)=(S=13310515,E=13310518,C=0)
	CT(67316)=(S=13310519,E=13310521,C=1)
	CT(67317)=(S=13310522,E=13310523,C=0)
	CT(67318)=(S=13310524,E=13310524,C=1)
	CT(67319)=(S=13310525,E=13310528,C=0)
	CT(67320)=(S=13310529,E=13310529,C=1)
	CT(67321)=(S=13310530,E=13310531,C=0)
	CT(67322)=(S=13310532,E=13310532,C=1)
	CT(67323)=(S=13310533,E=13310539,C=0)
	CT(67324)=(S=13310540,E=13310540,C=1)
	CT(67325)=(S=13310541,E=13310541,C=77)
	CT(67326)=(S=13310542,E=13310543,C=0)
	CT(67327)=(S=13310544,E=13310544,C=1)
	CT(67328)=(S=13310545,E=13310547,C=0)
	CT(67329)=(S=13310548,E=13310548,C=1)
	CT(67330)=(S=13310549,E=13310560,C=0)
	CT(67331)=(S=13310561,E=13310561,C=1)
	CT(67332)=(S=13310562,E=13310565,C=0)
	CT(67333)=(S=13310566,E=13310567,C=1)
	CT(67334)=(S=13310568,E=13310578,C=0)
	CT(67335)=(S=13310579,E=13310580,C=1)
	CT(67336)=(S=13310581,E=13310592,C=0)
	CT(67337)=(S=13310593,E=13310593,C=1)
	CT(67338)=(S=13310594,E=13310606,C=0)
	CT(67339)=(S=13310607,E=13310608,C=1)
	CT(67340)=(S=13310609,E=13310611,C=0)
	CT(67341)=(S=13310612,E=13310613,C=1)
	CT(67342)=(S=13310614,E=13310617,C=0)
	CT(67343)=(S=13310618,E=13310618,C=1)
	CT(67344)=(S=13310619,E=13310621,C=0)
	CT(67345)=(S=13310622,E=13310623,C=1)
	CT(67346)=(S=13310624,E=13310633,C=0)
	CT(67347)=(S=13310634,E=13310634,C=1)
	CT(67348)=(S=13310635,E=13310636,C=0)
	CT(67349)=(S=13310637,E=13310637,C=1)
	CT(67350)=(S=13310638,E=13310638,C=0)
	CT(67351)=(S=13310639,E=13310639,C=2)
	CT(67352)=(S=13310640,E=13310640,C=1)
	CT(67353)=(S=13310641,E=13310648,C=0)
	CT(67354)=(S=13310649,E=13310649,C=1)
	CT(67355)=(S=13310650,E=13310665,C=0)
	CT(67356)=(S=13310666,E=13310667,C=1)
	CT(67357)=(S=13310668,E=13310673,C=0)
	CT(67358)=(S=13310674,E=13310674,C=1)
	CT(67359)=(S=13310675,E=13310677,C=0)
	CT(67360)=(S=13310678,E=13310678,C=1)
	CT(67361)=(S=13310679,E=13310685,C=0)
	CT(67362)=(S=13310686,E=13310686,C=1)
	CT(67363)=(S=13310687,E=13310687,C=0)
	CT(67364)=(S=13310688,E=13310688,C=1)
	CT(67365)=(S=13310689,E=13310691,C=0)
	CT(67366)=(S=13310692,E=13310692,C=1)
	CT(67367)=(S=13310693,E=13310695,C=0)
	CT(67368)=(S=13310696,E=13310696,C=1)
	CT(67369)=(S=13310697,E=13310719,C=0)
	CT(67370)=(S=13310720,E=13310720,C=1)
	CT(67371)=(S=13310721,E=13310725,C=0)
	CT(67372)=(S=13310726,E=13310727,C=74)
	CT(67373)=(S=13310728,E=13310729,C=0)
	CT(67374)=(S=13310730,E=13310730,C=1)
	CT(67375)=(S=13310731,E=13310734,C=0)
	CT(67376)=(S=13310735,E=13310736,C=1)
	CT(67377)=(S=13310737,E=13310739,C=0)
	CT(67378)=(S=13310740,E=13310740,C=1)
	CT(67379)=(S=13310741,E=13310741,C=0)
	CT(67380)=(S=13310742,E=13310743,C=1)
	CT(67381)=(S=13310744,E=13310759,C=0)
	CT(67382)=(S=13310760,E=13310760,C=1)
	CT(67383)=(S=13310761,E=13310764,C=0)
	CT(67384)=(S=13310765,E=13310765,C=1)
	CT(67385)=(S=13310766,E=13310772,C=0)
	CT(67386)=(S=13310773,E=13310773,C=1)
	CT(67387)=(S=13310774,E=13310784,C=0)
	CT(67388)=(S=13310785,E=13310786,C=1)
	CT(67389)=(S=13310787,E=13310787,C=73)
	CT(67390)=(S=13310788,E=13310788,C=0)
	CT(67391)=(S=13310789,E=13310789,C=2)
	CT(67392)=(S=13310790,E=13310800,C=0)
	CT(67393)=(S=13310801,E=13310801,C=1)
	CT(67394)=(S=13310802,E=13310807,C=0)
	CT(67395)=(S=13310808,E=13310808,C=1)
	CT(67396)=(S=13310809,E=13310821,C=0)
	CT(67397)=(S=13310822,E=13310822,C=1)
	CT(67398)=(S=13310823,E=13310828,C=0)
	CT(67399)=(S=13310829,E=13310829,C=1)
	CT(67400)=(S=13310830,E=13310836,C=0)
	CT(67401)=(S=13310837,E=13310837,C=1)
	CT(67402)=(S=13310838,E=13310840,C=0)
	CT(67403)=(S=13310841,E=13310843,C=1)
	CT(67404)=(S=13310844,E=13310844,C=0)
	CT(67405)=(S=13310845,E=13310845,C=1)
	CT(67406)=(S=13310846,E=13310919,C=0)
	CT(67407)=(S=13310920,E=13310920,C=1)
	CT(67408)=(S=13310921,E=13310921,C=0)
	CT(67409)=(S=13310922,E=13310922,C=1)
	CT(67410)=(S=13310923,E=13310952,C=0)
	CT(67411)=(S=13310953,E=13310953,C=1)
	CT(67412)=(S=13310954,E=13310954,C=0)
	CT(67413)=(S=13310955,E=13310955,C=4)
	CT(67414)=(S=13310956,E=13310960,C=0)
	CT(67415)=(S=13310961,E=13310961,C=1)
	CT(67416)=(S=13310962,E=13310969,C=0)
	CT(67417)=(S=13310970,E=13310970,C=1)
	CT(67418)=(S=13310971,E=13310985,C=0)
	CT(67419)=(S=13310986,E=13310986,C=1)
	CT(67420)=(S=13310987,E=13310987,C=0)
	CT(67421)=(S=13310988,E=13310988,C=1)
	CT(67422)=(S=13310989,E=13311008,C=0)
	CT(67423)=(S=13311009,E=13311011,C=1)
	CT(67424)=(S=13311012,E=13311018,C=0)
	CT(67425)=(S=13311019,E=13311020,C=1)
	CT(67426)=(S=13311021,E=13311029,C=0)
	CT(67427)=(S=13311030,E=13311030,C=1)
	CT(67428)=(S=13311031,E=13311031,C=0)
	CT(67429)=(S=13311032,E=13311032,C=1)
	CT(67430)=(S=13311033,E=13311048,C=0)
	CT(67431)=(S=13311049,E=13311050,C=1)
	CT(67432)=(S=13311051,E=13311051,C=0)
	CT(67433)=(S=13311052,E=13311052,C=1)
	CT(67434)=(S=13311053,E=13311061,C=0)
	CT(67435)=(S=13311062,E=13311062,C=1)
	CT(67436)=(S=13311063,E=13311063,C=0)
	CT(67437)=(S=13311064,E=13311064,C=1)
	CT(67438)=(S=13311065,E=13311087,C=0)
	CT(67439)=(S=13311088,E=13311088,C=1)
	CT(67440)=(S=13311089,E=13311103,C=0)
	CT(67441)=(S=13311104,E=13311105,C=3)
	CT(67442)=(S=13311106,E=13311106,C=0)
	CT(67443)=(S=13311107,E=13311107,C=1)
	CT(67444)=(S=13311108,E=13311111,C=0)
	CT(67445)=(S=13311112,E=13311112,C=1)
	CT(67446)=(S=13311113,E=13311115,C=0)
	CT(67447)=(S=13311116,E=13311116,C=1)
	CT(67448)=(S=13311117,E=13311120,C=0)
	CT(67449)=(S=13311121,E=13311121,C=1)
	CT(67450)=(S=13311122,E=13311140,C=0)
	CT(67451)=(S=13311141,E=13311141,C=1)
	CT(67452)=(S=13311142,E=13311144,C=0)
	CT(67453)=(S=13311145,E=13311146,C=1)
	CT(67454)=(S=13311147,E=13311153,C=0)
	CT(67455)=(S=13311154,E=13311155,C=1)
	CT(67456)=(S=13311156,E=13311160,C=0)
	CT(67457)=(S=13311161,E=13311161,C=1)
	CT(67458)=(S=13311162,E=13311162,C=0)
	CT(67459)=(S=13311163,E=13311163,C=1)
	CT(67460)=(S=13311164,E=13311171,C=0)
	CT(67461)=(S=13311172,E=13311172,C=1)
	CT(67462)=(S=13311173,E=13311201,C=0)
	CT(67463)=(S=13311202,E=13311203,C=1)
	CT(67464)=(S=13311204,E=13311214,C=0)
	CT(67465)=(S=13311215,E=13311215,C=1)
	CT(67466)=(S=13311216,E=13311233,C=0)
	CT(67467)=(S=13311234,E=13311234,C=1)
	CT(67468)=(S=13311235,E=13311239,C=0)
	CT(67469)=(S=13311240,E=13311241,C=1)
	CT(67470)=(S=13311242,E=13311244,C=0)
	CT(67471)=(S=13311245,E=13311246,C=1)
	CT(67472)=(S=13311247,E=13311257,C=0)
	CT(67473)=(S=13311258,E=13311259,C=74)
	CT(67474)=(S=13311260,E=13311260,C=1)
	CT(67475)=(S=13311261,E=13311277,C=0)
	CT(67476)=(S=13311278,E=13311278,C=1)
	CT(67477)=(S=13311279,E=13311288,C=0)
	CT(67478)=(S=13311289,E=13311289,C=1)
	CT(67479)=(S=13311290,E=13311292,C=0)
	CT(67480)=(S=13311293,E=13311293,C=1)
	CT(67481)=(S=13311294,E=13311294,C=0)
	CT(67482)=(S=13311295,E=13311295,C=1)
	CT(67483)=(S=13311296,E=13311300,C=0)
	CT(67484)=(S=13311301,E=13311301,C=1)
	CT(67485)=(S=13311302,E=13311304,C=0)
	CT(67486)=(S=13311305,E=13311305,C=1)
	CT(67487)=(S=13311306,E=13311312,C=0)
	CT(67488)=(S=13311313,E=13311313,C=1)
	CT(67489)=(S=13311314,E=13311321,C=0)
	CT(67490)=(S=13311322,E=13311322,C=1)
	CT(67491)=(S=13311323,E=13311326,C=0)
	CT(67492)=(S=13311327,E=13311327,C=1)
	CT(67493)=(S=13311328,E=13311331,C=0)
	CT(67494)=(S=13311332,E=13311332,C=1)
	CT(67495)=(S=13311333,E=13311334,C=0)
	CT(67496)=(S=13311335,E=13311335,C=1)
	CT(67497)=(S=13311336,E=13311343,C=0)
	CT(67498)=(S=13311344,E=13311344,C=1)
	CT(67499)=(S=13311345,E=13311351,C=0)
	CT(67500)=(S=13311352,E=13311355,C=1)
	CT(67501)=(S=13311356,E=13311413,C=0)
	CT(67502)=(S=13311414,E=13311415,C=1)
	CT(67503)=(S=13311416,E=13311418,C=0)
	CT(67504)=(S=13311419,E=13311419,C=1)
	CT(67505)=(S=13311420,E=13311420,C=0)
	CT(67506)=(S=13311421,E=13311422,C=1)
	CT(67507)=(S=13311423,E=13311436,C=0)
	CT(67508)=(S=13311437,E=13311437,C=1)
	CT(67509)=(S=13311438,E=13311441,C=0)
	CT(67510)=(S=13311442,E=13311442,C=1)
	CT(67511)=(S=13311443,E=13311448,C=0)
	CT(67512)=(S=13311449,E=13311449,C=1)
	CT(67513)=(S=13311450,E=13311453,C=0)
	CT(67514)=(S=13311454,E=13311454,C=8)
	CT(67515)=(S=13311455,E=13311458,C=0)
	CT(67516)=(S=13311459,E=13311459,C=1)
	CT(67517)=(S=13311460,E=13311462,C=0)
	CT(67518)=(S=13311463,E=13311463,C=1)
	CT(67519)=(S=13311464,E=13311464,C=0)
	CT(67520)=(S=13311465,E=13311466,C=1)
	CT(67521)=(S=13311467,E=13311479,C=0)
	CT(67522)=(S=13311480,E=13311480,C=1)
	CT(67523)=(S=13311481,E=13311485,C=0)
	CT(67524)=(S=13311486,E=13311487,C=1)
	CT(67525)=(S=13311488,E=13311503,C=0)
	CT(67526)=(S=13311504,E=13311505,C=1)
	CT(67527)=(S=13311506,E=13311512,C=0)
	CT(67528)=(S=13311513,E=13311513,C=1)
	CT(67529)=(S=13311514,E=13311514,C=0)
	CT(67530)=(S=13311515,E=13311515,C=1)
	CT(67531)=(S=13311516,E=13311516,C=0)
	CT(67532)=(S=13311517,E=13311517,C=1)
	CT(67533)=(S=13311518,E=13311525,C=0)
	CT(67534)=(S=13311526,E=13311527,C=7)
	CT(67535)=(S=13311528,E=13311535,C=0)
	CT(67536)=(S=13311536,E=13311537,C=7)
	CT(67537)=(S=13311538,E=13311553,C=0)
	CT(67538)=(S=13311554,E=13311554,C=1)
	CT(67539)=(S=13311555,E=13311568,C=0)
	CT(67540)=(S=13311569,E=13311569,C=1)
	CT(67541)=(S=13311570,E=13311574,C=0)
	CT(67542)=(S=13311575,E=13311575,C=1)
	CT(67543)=(S=13311576,E=13311598,C=0)
	CT(67544)=(S=13311599,E=13311599,C=1)
	CT(67545)=(S=13311600,E=13311608,C=0)
	CT(67546)=(S=13311609,E=13311609,C=1)
	CT(67547)=(S=13311610,E=13311610,C=0)
	CT(67548)=(S=13311611,E=13311611,C=1)
	CT(67549)=(S=13311612,E=13311639,C=0)
	CT(67550)=(S=13311640,E=13311640,C=1)
	CT(67551)=(S=13311641,E=13311643,C=0)
	CT(67552)=(S=13311644,E=13311644,C=1)
	CT(67553)=(S=13311645,E=13311649,C=0)
	CT(67554)=(S=13311650,E=13311650,C=1)
	CT(67555)=(S=13311651,E=13311660,C=0)
	CT(67556)=(S=13311661,E=13311661,C=1)
	CT(67557)=(S=13311662,E=13311662,C=73)
	CT(67558)=(S=13311663,E=13311663,C=1)
	CT(67559)=(S=13311664,E=13311674,C=0)
	CT(67560)=(S=13311675,E=13311675,C=1)
	CT(67561)=(S=13311676,E=13311681,C=0)
	CT(67562)=(S=13311682,E=13311682,C=1)
	CT(67563)=(S=13311683,E=13311704,C=0)
	CT(67564)=(S=13311705,E=13311705,C=1)
	CT(67565)=(S=13311706,E=13311707,C=0)
	CT(67566)=(S=13311708,E=13311710,C=1)
	CT(67567)=(S=13311711,E=13311719,C=0)
	CT(67568)=(S=13311720,E=13311721,C=1)
	CT(67569)=(S=13311722,E=13311722,C=0)
	CT(67570)=(S=13311723,E=13311723,C=1)
	CT(67571)=(S=13311724,E=13311725,C=74)
	CT(67572)=(S=13311726,E=13311727,C=0)
	CT(67573)=(S=13311728,E=13311729,C=1)
	CT(67574)=(S=13311730,E=13311733,C=0)
	CT(67575)=(S=13311734,E=13311734,C=1)
	CT(67576)=(S=13311735,E=13311737,C=0)
	CT(67577)=(S=13311738,E=13311739,C=1)
	CT(67578)=(S=13311740,E=13311741,C=0)
	CT(67579)=(S=13311742,E=13311743,C=74)
	CT(67580)=(S=13311744,E=13311751,C=0)
	CT(67581)=(S=13311752,E=13311753,C=4)
	CT(67582)=(S=13311754,E=13311775,C=0)
	CT(67583)=(S=13311776,E=13311777,C=8)
	CT(67584)=(S=13311778,E=13311788,C=0)
	CT(67585)=(S=13311789,E=13311790,C=1)
	CT(67586)=(S=13311791,E=13311792,C=0)
	CT(67587)=(S=13311793,E=13311793,C=1)
	CT(67588)=(S=13311794,E=13311794,C=0)
	CT(67589)=(S=13311795,E=13311795,C=1)
	CT(67590)=(S=13311796,E=13311797,C=0)
	CT(67591)=(S=13311798,E=13311799,C=1)
	CT(67592)=(S=13311800,E=13311812,C=0)
	CT(67593)=(S=13311813,E=13311813,C=1)
	CT(67594)=(S=13311814,E=13311815,C=0)
	CT(67595)=(S=13311816,E=13311816,C=1)
	CT(67596)=(S=13311817,E=13311823,C=0)
	CT(67597)=(S=13311824,E=13311824,C=1)
	CT(67598)=(S=13311825,E=13311828,C=0)
	CT(67599)=(S=13311829,E=13311829,C=1)
	CT(67600)=(S=13311830,E=13311840,C=0)
	CT(67601)=(S=13311841,E=13311841,C=1)
	CT(67602)=(S=13311842,E=13311848,C=0)
	CT(67603)=(S=13311849,E=13311850,C=1)
	CT(67604)=(S=13311851,E=13311851,C=0)
	CT(67605)=(S=13311852,E=13311853,C=1)
	CT(67606)=(S=13311854,E=13311867,C=0)
	CT(67607)=(S=13311868,E=13311868,C=1)
	CT(67608)=(S=13311869,E=13311905,C=0)
	CT(67609)=(S=13311906,E=13311906,C=1)
	CT(67610)=(S=13311907,E=13311907,C=0)
	CT(67611)=(S=13311908,E=13311909,C=74)
	CT(67612)=(S=13311910,E=13311917,C=0)
	CT(67613)=(S=13311918,E=13311918,C=1)
	CT(67614)=(S=13311919,E=13311920,C=0)
	CT(67615)=(S=13311921,E=13311921,C=1)
	CT(67616)=(S=13311922,E=13311924,C=0)
	CT(67617)=(S=13311925,E=13311925,C=1)
	CT(67618)=(S=13311926,E=13311930,C=0)
	CT(67619)=(S=13311931,E=13311931,C=1)
	CT(67620)=(S=13311932,E=13311932,C=0)
	CT(67621)=(S=13311933,E=13311933,C=1)
	CT(67622)=(S=13311934,E=13311947,C=0)
	CT(67623)=(S=13311948,E=13311948,C=1)
	CT(67624)=(S=13311949,E=13311963,C=0)
	CT(67625)=(S=13311964,E=13311964,C=1)
	CT(67626)=(S=13311965,E=13311965,C=0)
	CT(67627)=(S=13311966,E=13311967,C=1)
	CT(67628)=(S=13311968,E=13311968,C=0)
	CT(67629)=(S=13311969,E=13311969,C=1)
	CT(67630)=(S=13311970,E=13311972,C=0)
	CT(67631)=(S=13311973,E=13311973,C=1)
	CT(67632)=(S=13311974,E=13311977,C=0)
	CT(67633)=(S=13311978,E=13311978,C=14)
	CT(67634)=(S=13311979,E=13311991,C=0)
	CT(67635)=(S=13311992,E=13311993,C=1)
	CT(67636)=(S=13311994,E=13311996,C=0)
	CT(67637)=(S=13311997,E=13311997,C=1)
	CT(67638)=(S=13311998,E=13312003,C=0)
	CT(67639)=(S=13312004,E=13312005,C=4)
	CT(67640)=(S=13312006,E=13312019,C=0)
	CT(67641)=(S=13312020,E=13312020,C=1)
	CT(67642)=(S=13312021,E=13312047,C=0)
	CT(67643)=(S=13312048,E=13312049,C=1)
	CT(67644)=(S=13312050,E=13312055,C=0)
	CT(67645)=(S=13312056,E=13312056,C=1)
	CT(67646)=(S=13312057,E=13312059,C=0)
	CT(67647)=(S=13312060,E=13312060,C=1)
	CT(67648)=(S=13312061,E=13312061,C=0)
	CT(67649)=(S=13312062,E=13312062,C=1)
	CT(67650)=(S=13312063,E=13312067,C=0)
	CT(67651)=(S=13312068,E=13312069,C=1)
	CT(67652)=(S=13312070,E=13312075,C=0)
	CT(67653)=(S=13312076,E=13312076,C=1)
	CT(67654)=(S=13312077,E=13312080,C=0)
	CT(67655)=(S=13312081,E=13312081,C=1)
	CT(67656)=(S=13312082,E=13312083,C=0)
	CT(67657)=(S=13312084,E=13312085,C=1)
	CT(67658)=(S=13312088,E=13312094,C=0)
	CT(67659)=(S=13312095,E=13312095,C=1)
	CT(67660)=(S=13312096,E=13312101,C=0)
	CT(67661)=(S=13312102,E=13312102,C=1)
	CT(67662)=(S=13312103,E=13312104,C=0)
	CT(67663)=(S=13312105,E=13312105,C=1)
	CT(67664)=(S=13312106,E=13312129,C=0)
	CT(67665)=(S=13312130,E=13312130,C=1)
	CT(67666)=(S=13312131,E=13312132,C=0)
	CT(67667)=(S=13312133,E=13312133,C=1)
	CT(67668)=(S=13312134,E=13312139,C=0)
	CT(67669)=(S=13312140,E=13312140,C=1)
	CT(67670)=(S=13312141,E=13312151,C=0)
	CT(67671)=(S=13312152,E=13312152,C=1)
	CT(67672)=(S=13312153,E=13312185,C=0)
	CT(67673)=(S=13312186,E=13312187,C=1)
	CT(67674)=(S=13312189,E=13312191,C=0)
	CT(67675)=(S=13312192,E=13312192,C=1)
	CT(67676)=(S=13312193,E=13312195,C=0)
	CT(67677)=(S=13312196,E=13312196,C=1)
	CT(67678)=(S=13312197,E=13312202,C=0)
	CT(67679)=(S=13312203,E=13312205,C=1)
	CT(67680)=(S=13312206,E=13312211,C=0)
	CT(67681)=(S=13312212,E=13312212,C=1)
	CT(67682)=(S=13312213,E=13312220,C=0)
	CT(67683)=(S=13312221,E=13312221,C=4)
	CT(67684)=(S=13312222,E=13312259,C=0)
	CT(67685)=(S=13312260,E=13312260,C=1)
	CT(67686)=(S=13312261,E=13312262,C=0)
	CT(67687)=(S=13312263,E=13312271,C=1)
	CT(67688)=(S=13312272,E=13312276,C=0)
	CT(67689)=(S=13312277,E=13312277,C=1)
	CT(67690)=(S=13312278,E=13312281,C=0)
	CT(67691)=(S=13312282,E=13312282,C=1)
	CT(67692)=(S=13312283,E=13312287,C=0)
	CT(67693)=(S=13312288,E=13312288,C=1)
	CT(67694)=(S=13312289,E=13312318,C=0)
	CT(67695)=(S=13312319,E=13312320,C=1)
	CT(67696)=(S=13312321,E=13312322,C=0)
	CT(67697)=(S=13312323,E=13312324,C=1)
	CT(67698)=(S=13312325,E=13312328,C=0)
	CT(67699)=(S=13312329,E=13312329,C=1)
	CT(67700)=(S=13312330,E=13312334,C=0)
	CT(67701)=(S=13312335,E=13312335,C=1)
	CT(67702)=(S=13312336,E=13312355,C=0)
	CT(67703)=(S=13312356,E=13312356,C=1)
	CT(67704)=(S=13312357,E=13312377,C=0)
	CT(67705)=(S=13312378,E=13312378,C=1)
	CT(67706)=(S=13312379,E=13312384,C=0)
	CT(67707)=(S=13312385,E=13312385,C=1)
	CT(67708)=(S=13312386,E=13312386,C=0)
	CT(67709)=(S=13312387,E=13312387,C=1)
	CT(67710)=(S=13312388,E=13312400,C=0)
	CT(67711)=(S=13312401,E=13312401,C=1)
	CT(67712)=(S=13312402,E=13312411,C=0)
	CT(67713)=(S=13312412,E=13312412,C=1)
	CT(67714)=(S=13312413,E=13312413,C=0)
	CT(67715)=(S=13312414,E=13312415,C=1)
	CT(67716)=(S=13312416,E=13312429,C=0)
	CT(67717)=(S=13312430,E=13312430,C=1)
	CT(67718)=(S=13312431,E=13312440,C=0)
	CT(67719)=(S=13312441,E=13312441,C=1)
	CT(67720)=(S=13312442,E=13312444,C=0)
	CT(67721)=(S=13312445,E=13312445,C=86)
	CT(67722)=(S=13312446,E=13312451,C=0)
	CT(67723)=(S=13312452,E=13312452,C=10)
	CT(67724)=(S=13312453,E=13312453,C=0)
	CT(67725)=(S=13312454,E=13312455,C=4)
	CT(67726)=(S=13312456,E=13312456,C=1)
	CT(67727)=(S=13312457,E=13312457,C=0)
	CT(67728)=(S=13312458,E=13312460,C=1)
	CT(67729)=(S=13312461,E=13312461,C=0)
	CT(67730)=(S=13312462,E=13312463,C=1)
	CT(67731)=(S=13312464,E=13312469,C=0)
	CT(67732)=(S=13312470,E=13312471,C=1)
	CT(67733)=(S=13312472,E=13312479,C=0)
	CT(67734)=(S=13312480,E=13312482,C=1)
	CT(67735)=(S=13312483,E=13312488,C=0)
	CT(67736)=(S=13312489,E=13312489,C=1)
	CT(67737)=(S=13312490,E=13312495,C=0)
	CT(67738)=(S=13312496,E=13312496,C=3)
	CT(67739)=(S=13312497,E=13312498,C=0)
	CT(67740)=(S=13312499,E=13312499,C=1)
	CT(67741)=(S=13312500,E=13312505,C=0)
	CT(67742)=(S=13312506,E=13312506,C=1)
	CT(67743)=(S=13312507,E=13312515,C=0)
	CT(67744)=(S=13312516,E=13312516,C=1)
	CT(67745)=(S=13312517,E=13312532,C=0)
	CT(67746)=(S=13312533,E=13312533,C=1)
	CT(67747)=(S=13312534,E=13312538,C=0)
	CT(67748)=(S=13312539,E=13312539,C=1)
	CT(67749)=(S=13312540,E=13312548,C=0)
	CT(67750)=(S=13312549,E=13312549,C=92)
	CT(67751)=(S=13312550,E=13312550,C=0)
	CT(67752)=(S=13312551,E=13312551,C=1)
	CT(67753)=(S=13312552,E=13312559,C=0)
	CT(67754)=(S=13312560,E=13312561,C=1)
	CT(67755)=(S=13312562,E=13312565,C=0)
	CT(67756)=(S=13312566,E=13312566,C=1)
	CT(67757)=(S=13312567,E=13312567,C=0)
	CT(67758)=(S=13312568,E=13312569,C=1)
	CT(67759)=(S=13312570,E=13312578,C=0)
	CT(67760)=(S=13312579,E=13312579,C=1)
	CT(67761)=(S=13312580,E=13312580,C=2)
	CT(67762)=(S=13312581,E=13312581,C=1)
	CT(67763)=(S=13312582,E=13312587,C=0)
	CT(67764)=(S=13312588,E=13312588,C=1)
	CT(67765)=(S=13312589,E=13312592,C=0)
	CT(67766)=(S=13312593,E=13312593,C=2)
	CT(67767)=(S=13312595,E=13312603,C=0)
	CT(67768)=(S=13312604,E=13312604,C=1)
	CT(67769)=(S=13312605,E=13312617,C=0)
	CT(67770)=(S=13312618,E=13312618,C=1)
	CT(67771)=(S=13312619,E=13312624,C=0)
	CT(67772)=(S=13312625,E=13312625,C=1)
	CT(67773)=(S=13312626,E=13312627,C=0)
	CT(67774)=(S=13312628,E=13312628,C=156)
	CT(67775)=(S=13312629,E=13312629,C=4)
	CT(67776)=(S=13312630,E=13312631,C=74)
	CT(67777)=(S=13312632,E=13312655,C=0)
	CT(67778)=(S=13312656,E=13312656,C=11)
	CT(67779)=(S=13312657,E=13312658,C=0)
	CT(67780)=(S=13312659,E=13312659,C=1)
	CT(67781)=(S=13312660,E=13312661,C=0)
	CT(67782)=(S=13312662,E=13312662,C=1)
	CT(67783)=(S=13312663,E=13312663,C=0)
	CT(67784)=(S=13312664,E=13312665,C=1)
	CT(67785)=(S=13312667,E=13312672,C=0)
	CT(67786)=(S=13312673,E=13312674,C=1)
	CT(67787)=(S=13312675,E=13312698,C=0)
	CT(67788)=(S=13312699,E=13312699,C=1)
	CT(67789)=(S=13312700,E=13312703,C=0)
	CT(67790)=(S=13312704,E=13312711,C=1)
	CT(67791)=(S=13312712,E=13312715,C=0)
	CT(67792)=(S=13312716,E=13312719,C=1)
	CT(67793)=(S=13312720,E=13312743,C=0)
	CT(67794)=(S=13312744,E=13312744,C=1)
	CT(67795)=(S=13312745,E=13312751,C=0)
	CT(67796)=(S=13312752,E=13312754,C=1)
	CT(67797)=(S=13312755,E=13312756,C=0)
	CT(67798)=(S=13312757,E=13312757,C=1)
	CT(67799)=(S=13312758,E=13312758,C=10)
	CT(67800)=(S=13312759,E=13312762,C=0)
	CT(67801)=(S=13312763,E=13312763,C=1)
	CT(67802)=(S=13312764,E=13317889,C=0)
	CT(67803)=(S=13317890,E=13317892,C=1)
	CT(67804)=(S=13317893,E=13317897,C=0)
	CT(67805)=(S=13317898,E=13317898,C=1)
	CT(67806)=(S=13317899,E=13317900,C=0)
	CT(67807)=(S=13317901,E=13317901,C=1)
	CT(67808)=(S=13317902,E=13317909,C=0)
	CT(67809)=(S=13317910,E=13317910,C=1)
	CT(67810)=(S=13317911,E=13317917,C=0)
	CT(67811)=(S=13317918,E=13317918,C=1)
	CT(67812)=(S=13317919,E=13317954,C=0)
	CT(67813)=(S=13317955,E=13317955,C=10)
	CT(67814)=(S=13317957,E=13317980,C=0)
	CT(67815)=(S=13317981,E=13317981,C=1)
	CT(67816)=(S=13317982,E=13317988,C=0)
	CT(67817)=(S=13317989,E=13317989,C=1)
	CT(67818)=(S=13317990,E=13317996,C=0)
	CT(67819)=(S=13317997,E=13317998,C=1)
	CT(67820)=(S=13317999,E=13318003,C=0)
	CT(67821)=(S=13318004,E=13318005,C=1)
	CT(67822)=(S=13318006,E=13318006,C=0)
	CT(67823)=(S=13318007,E=13318007,C=1)
	CT(67824)=(S=13318008,E=13318015,C=0)
	CT(67825)=(S=13318016,E=13318017,C=1)
	CT(67826)=(S=13318018,E=13318033,C=0)
	CT(67827)=(S=13318034,E=13318035,C=1)
	CT(67828)=(S=13318036,E=13318047,C=0)
	CT(67829)=(S=13318048,E=13318048,C=4)
	CT(67830)=(S=13318049,E=13318060,C=0)
	CT(67831)=(S=13318061,E=13318061,C=4)
	CT(67832)=(S=13318062,E=13318079,C=0)
	CT(67833)=(S=13318080,E=13318080,C=1)
	CT(67834)=(S=13318081,E=13318083,C=0)
	CT(67835)=(S=13318084,E=13318084,C=1)
	CT(67836)=(S=13318085,E=13318105,C=0)
	CT(67837)=(S=13318106,E=13318107,C=1)
	CT(67838)=(S=13318108,E=13318108,C=0)
	CT(67839)=(S=13318109,E=13318109,C=1)
	CT(67840)=(S=13318110,E=13318111,C=0)
	CT(67841)=(S=13318112,E=13318112,C=1)
	CT(67842)=(S=13318113,E=13318144,C=0)
	CT(67843)=(S=13318145,E=13318145,C=1)
	CT(67844)=(S=13318146,E=13318147,C=0)
	CT(67845)=(S=13318148,E=13318148,C=1)
	CT(67846)=(S=13318149,E=13318155,C=0)
	CT(67847)=(S=13318156,E=13318156,C=1)
	CT(67848)=(S=13318157,E=13318167,C=0)
	CT(67849)=(S=13318168,E=13318168,C=1)
	CT(67850)=(S=13318169,E=13318175,C=0)
	CT(67851)=(S=13318176,E=13318177,C=73)
	CT(67852)=(S=13318178,E=13318181,C=0)
	CT(67853)=(S=13318182,E=13318182,C=1)
	CT(67854)=(S=13318183,E=13318183,C=0)
	CT(67855)=(S=13318184,E=13318184,C=1)
	CT(67856)=(S=13318185,E=13318189,C=0)
	CT(67857)=(S=13318190,E=13318190,C=1)
	CT(67858)=(S=13318191,E=13318191,C=0)
	CT(67859)=(S=13318192,E=13318199,C=1)
	CT(67860)=(S=13318200,E=13318211,C=0)
	CT(67861)=(S=13318212,E=13318213,C=1)
	CT(67862)=(S=13318214,E=13318225,C=0)
	CT(67863)=(S=13318226,E=13318229,C=1)
	CT(67864)=(S=13318230,E=13318230,C=0)
	CT(67865)=(S=13318231,E=13318231,C=8)
	CT(67866)=(S=13318232,E=13318233,C=0)
	CT(67867)=(S=13318234,E=13318235,C=8)
	CT(67868)=(S=13318236,E=13318238,C=0)
	CT(67869)=(S=13318239,E=13318239,C=1)
	CT(67870)=(S=13318240,E=13318253,C=0)
	CT(67871)=(S=13318254,E=13318254,C=1)
	CT(67872)=(S=13318255,E=13318264,C=0)
	CT(67873)=(S=13318265,E=13318265,C=1)
	CT(67874)=(S=13318266,E=13318304,C=0)
	CT(67875)=(S=13318305,E=13318305,C=1)
	CT(67876)=(S=13318306,E=13318312,C=0)
	CT(67877)=(S=13318313,E=13318313,C=1)
	CT(67878)=(S=13318314,E=13318315,C=0)
	CT(67879)=(S=13318316,E=13318317,C=1)
	CT(67880)=(S=13318318,E=13318318,C=0)
	CT(67881)=(S=13318319,E=13318319,C=1)
	CT(67882)=(S=13318320,E=13318326,C=0)
	CT(67883)=(S=13318327,E=13318327,C=1)
	CT(67884)=(S=13318328,E=13318328,C=0)
	CT(67885)=(S=13318329,E=13318329,C=1)
	CT(67886)=(S=13318330,E=13318330,C=0)
	CT(67887)=(S=13318331,E=13318331,C=1)
	CT(67888)=(S=13318332,E=13318335,C=0)
	CT(67889)=(S=13318336,E=13318336,C=1)
	CT(67890)=(S=13318337,E=13318341,C=0)
	CT(67891)=(S=13318342,E=13318342,C=1)
	CT(67892)=(S=13318343,E=13318344,C=0)
	CT(67893)=(S=13318345,E=13318345,C=1)
	CT(67894)=(S=13318346,E=13318351,C=0)
	CT(67895)=(S=13318352,E=13318354,C=1)
	CT(67896)=(S=13318355,E=13318357,C=0)
	CT(67897)=(S=13318358,E=13318358,C=1)
	CT(67898)=(S=13318359,E=13318359,C=0)
	CT(67899)=(S=13318360,E=13318360,C=1)
	CT(67900)=(S=13318361,E=13318370,C=0)
	CT(67901)=(S=13318371,E=13318372,C=1)
	CT(67902)=(S=13318373,E=13318374,C=0)
	CT(67903)=(S=13318376,E=13318376,C=1)
	CT(67904)=(S=13318377,E=13318383,C=0)
	CT(67905)=(S=13318384,E=13318384,C=1)
	CT(67906)=(S=13318385,E=13318385,C=10)
	CT(67907)=(S=13318386,E=13318395,C=0)
	CT(67908)=(S=13318396,E=13318396,C=1)
	CT(67909)=(S=13318397,E=13318397,C=8)
	CT(67910)=(S=13318398,E=13318398,C=1)
	CT(67911)=(S=13318399,E=13318404,C=0)
	CT(67912)=(S=13318405,E=13318406,C=1)
	CT(67913)=(S=13318407,E=13318411,C=0)
	CT(67914)=(S=13318412,E=13318413,C=1)
	CT(67915)=(S=13318414,E=13318423,C=0)
	CT(67916)=(S=13318424,E=13318425,C=74)
	CT(67917)=(S=13318426,E=13318427,C=0)
	CT(67918)=(S=13318428,E=13318428,C=1)
	CT(67919)=(S=13318429,E=13318438,C=0)
	CT(67920)=(S=13318439,E=13318439,C=1)
	CT(67921)=(S=13318440,E=13318445,C=0)
	CT(67922)=(S=13318446,E=13318446,C=1)
	CT(67923)=(S=13318447,E=13318457,C=0)
	CT(67924)=(S=13318458,E=13318458,C=1)
	CT(67925)=(S=13318459,E=13318460,C=0)
	CT(67926)=(S=13318461,E=13318461,C=1)
	CT(67927)=(S=13318462,E=13318465,C=0)
	CT(67928)=(S=13318466,E=13318466,C=1)
	CT(67929)=(S=13318467,E=13318468,C=0)
	CT(67930)=(S=13318469,E=13318471,C=1)
	CT(67931)=(S=13318472,E=13318472,C=0)
	CT(67932)=(S=13318473,E=13318473,C=1)
	CT(67933)=(S=13318474,E=13318489,C=0)
	CT(67934)=(S=13318490,E=13318490,C=1)
	CT(67935)=(S=13318491,E=13318500,C=0)
	CT(67936)=(S=13318501,E=13318501,C=1)
	CT(67937)=(S=13318502,E=13318508,C=0)
	CT(67938)=(S=13318509,E=13318509,C=1)
	CT(67939)=(S=13318510,E=13318522,C=0)
	CT(67940)=(S=13318523,E=13318523,C=1)
	CT(67941)=(S=13318524,E=13318544,C=0)
	CT(67942)=(S=13318545,E=13318545,C=73)
	CT(67943)=(S=13318546,E=13318556,C=0)
	CT(67944)=(S=13318557,E=13318557,C=1)
	CT(67945)=(S=13318558,E=13318599,C=0)
	CT(67946)=(S=13318600,E=13318600,C=1)
	CT(67947)=(S=13318601,E=13318601,C=0)
	CT(67948)=(S=13318602,E=13318602,C=1)
	CT(67949)=(S=13318603,E=13318605,C=0)
	CT(67950)=(S=13318606,E=13318606,C=1)
	CT(67951)=(S=13318607,E=13318621,C=0)
	CT(67952)=(S=13318622,E=13318622,C=1)
	CT(67953)=(S=13318623,E=13318623,C=0)
	CT(67954)=(S=13318624,E=13318639,C=1)
	CT(67955)=(S=13318640,E=13318645,C=0)
	CT(67956)=(S=13318646,E=13318647,C=1)
	CT(67957)=(S=13318648,E=13318648,C=0)
	CT(67958)=(S=13318649,E=13318649,C=1)
	CT(67959)=(S=13318650,E=13318652,C=0)
	CT(67960)=(S=13318653,E=13318655,C=1)
	CT(67961)=(S=13318656,E=13319167,C=0)
	CT(67962)=(S=13319168,E=13319423,C=8)
	CT(67963)=(S=13319424,E=13319680,C=0)
	CT(67964)=(S=13319681,E=13319681,C=5)
	CT(67965)=(S=13319682,E=13319682,C=1)
	CT(67966)=(S=13319683,E=13319810,C=0)
	CT(67967)=(S=13319811,E=13319811,C=1)
	CT(67968)=(S=13319812,E=13319818,C=0)
	CT(67969)=(S=13319819,E=13319819,C=1)
	CT(67970)=(S=13319820,E=13319827,C=0)
	CT(67971)=(S=13319828,E=13319831,C=2)
	CT(67972)=(S=13319832,E=13319835,C=3)
	CT(67973)=(S=13319836,E=13319840,C=0)
	CT(67974)=(S=13319841,E=13319841,C=1)
	CT(67975)=(S=13319842,E=13319851,C=0)
	CT(67976)=(S=13319852,E=13319855,C=4)
	CT(67977)=(S=13319856,E=13319874,C=0)
	CT(67978)=(S=13319875,E=13319875,C=7)
	CT(67979)=(S=13319876,E=13319876,C=0)
	CT(67980)=(S=13319877,E=13319877,C=1)
	CT(67981)=(S=13319878,E=13319907,C=0)
	CT(67982)=(S=13319908,E=13319911,C=1)
	CT(67983)=(S=13319912,E=13319913,C=0)
	CT(67984)=(S=13319914,E=13319914,C=1)
	CT(67985)=(S=13319915,E=13319925,C=0)
	CT(67986)=(S=13319926,E=13319926,C=1)
	CT(67987)=(S=13319927,E=13320191,C=0)
	CT(67988)=(S=13320192,E=13323263,C=9)
	CT(67989)=(S=13323264,E=13323327,C=0)
	CT(67990)=(S=13323328,E=13323359,C=2)
	CT(67991)=(S=13323360,E=13323391,C=84)
	CT(67992)=(S=13323392,E=13323407,C=4)
	CT(67993)=(S=13323408,E=13323423,C=84)
	CT(67994)=(S=13323424,E=13323427,C=1)
	CT(67995)=(S=13323428,E=13323431,C=2)
	CT(67996)=(S=13323432,E=13323435,C=1)
	CT(67997)=(S=13323436,E=13323439,C=7)
	CT(67998)=(S=13323440,E=13323455,C=4)
	CT(67999)=(S=13323456,E=13323471,C=10)
	CT(68000)=(S=13323520,E=13323647,C=9)
	CT(68001)=(S=13323648,E=13323679,C=8)
	CT(68002)=(S=13323680,E=13323695,C=0)
	CT(68003)=(S=13323696,E=13323696,C=6)
	CT(68004)=(S=13323697,E=13323697,C=4)
	CT(68005)=(S=13323698,E=13323698,C=11)
	CT(68006)=(S=13323699,E=13323699,C=0)
	CT(68007)=(S=13323700,E=13323703,C=1)
	CT(68008)=(S=13323704,E=13323711,C=2)
	CT(68009)=(S=13323712,E=13323727,C=4)
	CT(68010)=(S=13323728,E=13323775,C=74)
	CT(68011)=(S=13323776,E=13323779,C=9)
	CT(68012)=(S=13323780,E=13323780,C=4)
	CT(68013)=(S=13323781,E=13323783,C=8)
	CT(68014)=(S=13323784,E=13323791,C=7)
	CT(68015)=(S=13323792,E=13323807,C=0)
	CT(68016)=(S=13323808,E=13323823,C=8)
	CT(68017)=(S=13323824,E=13323839,C=1)
	CT(68018)=(S=13323840,E=13323871,C=8)
	CT(68019)=(S=13323872,E=13323887,C=3)
	CT(68020)=(S=13323888,E=13323903,C=74)
	CT(68021)=(S=13323936,E=13323951,C=75)
	CT(68022)=(S=13323952,E=13323967,C=9)
	CT(68023)=(S=13323968,E=13323983,C=5)
	CT(68024)=(S=13323984,E=13323999,C=4)
	CT(68025)=(S=13324000,E=13324031,C=2)
	CT(68026)=(S=13324032,E=13324047,C=1)
	CT(68027)=(S=13324048,E=13324055,C=0)
	CT(68028)=(S=13324056,E=13324057,C=4)
	CT(68029)=(S=13324058,E=13324059,C=74)
	CT(68030)=(S=13324060,E=13324060,C=11)
	CT(68031)=(S=13324061,E=13324061,C=74)
	CT(68032)=(S=13324062,E=13324063,C=0)
	CT(68033)=(S=13324064,E=13324079,C=1)
	CT(68034)=(S=13324080,E=13324095,C=2)
	CT(68035)=(S=13324096,E=13324159,C=73)
	CT(68036)=(S=13324160,E=13324287,C=9)
	CT(68037)=(S=13324288,E=13324291,C=8)
	CT(68038)=(S=13324292,E=13324293,C=1)
	CT(68039)=(S=13324294,E=13324295,C=2)
	CT(68040)=(S=13324296,E=13324303,C=74)
	CT(68041)=(S=13324304,E=13324311,C=5)
	CT(68042)=(S=13324312,E=13324319,C=2)
	CT(68043)=(S=13324320,E=13324335,C=1)
	CT(68044)=(S=13324336,E=13324343,C=90)
	CT(68045)=(S=13324344,E=13324344,C=3)
	CT(68046)=(S=13324345,E=13324345,C=1)
	CT(68047)=(S=13324346,E=13324347,C=0)
	CT(68048)=(S=13324348,E=13324351,C=73)
	CT(68049)=(S=13324352,E=13324415,C=8)
	CT(68050)=(S=13324416,E=13324418,C=77)
	CT(68051)=(S=13324419,E=13324419,C=7)
	CT(68052)=(S=13324420,E=13324447,C=1)
	CT(68053)=(S=13324448,E=13324463,C=0)
	CT(68054)=(S=13324464,E=13324543,C=8)
	CT(68055)=(S=13324544,E=13324551,C=1)
	CT(68056)=(S=13324552,E=13324555,C=6)
	CT(68057)=(S=13324556,E=13324559,C=2)
	CT(68058)=(S=13324560,E=13324575,C=1)
	CT(68059)=(S=13324576,E=13324607,C=2)
	CT(68060)=(S=13324608,E=13324639,C=155)
	CT(68061)=(S=13324640,E=13324655,C=156)
	CT(68062)=(S=13324656,E=13324671,C=2)
	CT(68063)=(S=13324672,E=13324703,C=6)
	CT(68064)=(S=13324704,E=13324719,C=155)
	CT(68065)=(S=13324720,E=13324727,C=8)
	CT(68066)=(S=13324728,E=13324735,C=74)
	CT(68067)=(S=13324736,E=13324783,C=77)
	CT(68068)=(S=13324792,E=13324795,C=74)
	CT(68069)=(S=13324796,E=13324799,C=8)
	CT(68070)=(S=13324800,E=13324801,C=1)
	CT(68071)=(S=13324802,E=13324803,C=88)
	CT(68072)=(S=13324804,E=13324807,C=4)
	CT(68073)=(S=13324808,E=13324815,C=8)
	CT(68074)=(S=13324816,E=13324823,C=1)
	CT(68075)=(S=13324832,E=13324847,C=10)
	CT(68076)=(S=13324848,E=13324863,C=77)
	CT(68077)=(S=13324864,E=13324895,C=5)
	CT(68078)=(S=13324896,E=13324911,C=0)
	CT(68079)=(S=13324928,E=13324959,C=2)
	CT(68080)=(S=13324960,E=13324991,C=0)
	CT(68081)=(S=13324992,E=13325007,C=84)
	CT(68082)=(S=13325008,E=13325017,C=0)
	CT(68083)=(S=13325018,E=13325018,C=4)
	CT(68084)=(S=13325019,E=13325021,C=6)
	CT(68085)=(S=13325022,E=13325023,C=8)
	CT(68086)=(S=13325040,E=13325047,C=6)
	CT(68087)=(S=13325048,E=13325051,C=4)
	CT(68088)=(S=13325052,E=13325055,C=8)
	CT(68089)=(S=13325056,E=13325059,C=1)
	CT(68090)=(S=13325060,E=13325063,C=0)
	CT(68091)=(S=13325072,E=13325079,C=76)
	CT(68092)=(S=13325080,E=13325103,C=74)
	CT(68093)=(S=13325104,E=13325111,C=233)
	CT(68094)=(S=13325112,E=13325119,C=1)
	CT(68095)=(S=13325120,E=13325183,C=8)
	CT(68096)=(S=13325184,E=13325215,C=6)
	CT(68097)=(S=13325216,E=13325247,C=84)
	CT(68098)=(S=13325248,E=13325271,C=0)
	CT(68099)=(S=13325272,E=13325273,C=8)
	CT(68100)=(S=13325274,E=13325274,C=73)
	CT(68101)=(S=13325275,E=13325275,C=0)
	CT(68102)=(S=13325276,E=13325279,C=7)
	CT(68103)=(S=13325280,E=13325295,C=1)
	CT(68104)=(S=13325296,E=13325303,C=2)
	CT(68105)=(S=13325304,E=13325305,C=8)
	CT(68106)=(S=13325306,E=13325306,C=7)
	CT(68107)=(S=13325307,E=13325307,C=91)
	CT(68108)=(S=13325308,E=13325375,C=8)
	CT(68109)=(S=13325376,E=13325439,C=0)
	CT(68110)=(S=13325440,E=13325441,C=8)
	CT(68111)=(S=13325442,E=13325443,C=9)
	CT(68112)=(S=13325444,E=13325444,C=0)
	CT(68113)=(S=13325445,E=13325445,C=8)
	CT(68114)=(S=13325446,E=13325447,C=73)
	CT(68115)=(S=13325448,E=13325455,C=74)
	CT(68116)=(S=13325456,E=13325463,C=9)
	CT(68117)=(S=13325464,E=13325471,C=74)
	CT(68118)=(S=13325472,E=13325503,C=10)
	CT(68119)=(S=13325504,E=13325535,C=8)
	CT(68120)=(S=13325536,E=13325551,C=0)
	CT(68121)=(S=13325552,E=13325567,C=6)
	CT(68122)=(S=13325568,E=13325823,C=8)
	CT(68123)=(S=13325824,E=13325951,C=1)
	CT(68124)=(S=13325952,E=13326015,C=8)
	CT(68125)=(S=13326016,E=13326031,C=73)
	CT(68126)=(S=13326032,E=13326047,C=0)
	CT(68127)=(S=13326048,E=13326055,C=2)
	CT(68128)=(S=13326056,E=13326063,C=8)
	CT(68129)=(S=13326064,E=13326073,C=2)
	CT(68130)=(S=13326075,E=13326075,C=0)
	CT(68131)=(S=13326076,E=13326077,C=8)
	CT(68132)=(S=13326078,E=13326079,C=1)
	CT(68133)=(S=13326080,E=13326207,C=0)
	CT(68134)=(S=13326208,E=13326335,C=10)
	CT(68135)=(S=13326336,E=13326367,C=4)
	CT(68136)=(S=13326368,E=13326399,C=1)
	CT(68137)=(S=13326400,E=13326431,C=8)
	CT(68138)=(S=13326448,E=13326463,C=0)
	CT(68139)=(S=13326464,E=13326495,C=4)
	CT(68140)=(S=13326496,E=13326511,C=8)
	CT(68141)=(S=13326512,E=13326527,C=2)
	CT(68142)=(S=13326528,E=13326559,C=1)
	CT(68143)=(S=13326560,E=13326591,C=0)
	CT(68144)=(S=13326592,E=13326595,C=1)
	CT(68145)=(S=13326596,E=13326596,C=4)
	CT(68146)=(S=13326597,E=13326597,C=0)
	CT(68147)=(S=13326598,E=13326599,C=8)
	CT(68148)=(S=13326600,E=13326607,C=1)
	CT(68149)=(S=13326608,E=13326623,C=74)
	CT(68150)=(S=13326624,E=13326655,C=2)
	CT(68151)=(S=13326656,E=13326719,C=0)
	CT(68152)=(S=13326720,E=13326723,C=2)
	CT(68153)=(S=13326724,E=13326727,C=4)
	CT(68154)=(S=13326728,E=13326731,C=1)
	CT(68155)=(S=13326732,E=13326735,C=11)
	CT(68156)=(S=13326737,E=13326737,C=0)
	CT(68157)=(S=13326738,E=13326741,C=74)
	CT(68158)=(S=13326742,E=13326742,C=7)
	CT(68159)=(S=13326743,E=13326747,C=4)
	CT(68160)=(S=13326748,E=13326847,C=0)
	CT(68161)=(S=13326848,E=13326851,C=1)
	CT(68162)=(S=13326852,E=13326853,C=4)
	CT(68163)=(S=13326854,E=13326855,C=8)
	CT(68164)=(S=13326856,E=13326859,C=1)
	CT(68165)=(S=13326864,E=13326871,C=2)
	CT(68166)=(S=13326872,E=13326879,C=0)
	CT(68167)=(S=13326880,E=13326911,C=6)
	CT(68168)=(S=13326912,E=13326975,C=4)
	CT(68169)=(S=13326976,E=13327071,C=1)
	CT(68170)=(S=13327072,E=13327103,C=8)
	CT(68171)=(S=13327104,E=13327135,C=9)
	CT(68172)=(S=13327136,E=13327167,C=1)
	CT(68173)=(S=13327168,E=13327199,C=0)
	CT(68174)=(S=13327200,E=13327215,C=1)
	CT(68175)=(S=13327216,E=13327223,C=92)
	CT(68176)=(S=13327224,E=13327231,C=1)
	CT(68177)=(S=13327232,E=13327263,C=8)
	CT(68178)=(S=13327264,E=13327295,C=2)
	CT(68179)=(S=13327296,E=13327327,C=4)
	CT(68180)=(S=13327328,E=13327359,C=0)
	CT(68181)=(S=13327360,E=13327363,C=1)
	CT(68182)=(S=13327364,E=13327365,C=77)
	CT(68183)=(S=13327367,E=13327367,C=0)
	CT(68184)=(S=13327368,E=13327375,C=10)
	CT(68185)=(S=13327376,E=13327383,C=2)
	CT(68186)=(S=13327384,E=13327385,C=73)
	CT(68187)=(S=13327386,E=13327391,C=0)
	CT(68188)=(S=13327392,E=13327423,C=4)
	CT(68189)=(S=13327424,E=13327487,C=7)
	CT(68190)=(S=13327488,E=13327519,C=5)
	CT(68191)=(S=13327520,E=13327551,C=1)
	CT(68192)=(S=13327552,E=13327567,C=4)
	CT(68193)=(S=13327568,E=13327583,C=2)
	CT(68194)=(S=13327584,E=13327615,C=0)
	CT(68195)=(S=13327616,E=13327903,C=1)
	CT(68196)=(S=13327904,E=13327935,C=73)
	CT(68197)=(S=13327936,E=13327999,C=156)
	CT(68198)=(S=13328000,E=13328063,C=0)
	CT(68199)=(S=13328064,E=13328127,C=4)
	CT(68200)=(S=13328128,E=13328135,C=1)
	CT(68201)=(S=13328136,E=13328143,C=159)
	CT(68202)=(S=13328144,E=13328191,C=2)
	CT(68203)=(S=13328192,E=13328223,C=7)
	CT(68204)=(S=13328224,E=13328383,C=1)
	CT(68205)=(S=13328384,E=13328959,C=73)
	CT(68206)=(S=13328960,E=13328991,C=0)
	CT(68207)=(S=13328992,E=13329023,C=4)
	CT(68208)=(S=13329024,E=13329087,C=8)
	CT(68209)=(S=13329088,E=13329119,C=0)
	CT(68210)=(S=13329120,E=13329151,C=158)
	CT(68211)=(S=13329152,E=13329159,C=2)
	CT(68212)=(S=13329160,E=13329183,C=1)
	CT(68213)=(S=13329184,E=13329191,C=73)
	CT(68214)=(S=13329192,E=13329199,C=4)
	CT(68215)=(S=13329200,E=13329215,C=77)
	CT(68216)=(S=13329216,E=13329223,C=73)
	CT(68217)=(S=13329224,E=13329231,C=2)
	CT(68218)=(S=13329232,E=13329247,C=1)
	CT(68219)=(S=13329248,E=13329279,C=74)
	CT(68220)=(S=13329280,E=13329281,C=73)
	CT(68221)=(S=13329282,E=13329283,C=74)
	CT(68222)=(S=13329284,E=13329287,C=73)
	CT(68223)=(S=13329288,E=13329291,C=5)
	CT(68224)=(S=13329294,E=13329294,C=0)
	CT(68225)=(S=13329295,E=13329295,C=8)
	CT(68226)=(S=13329296,E=13329307,C=0)
	CT(68227)=(S=13329308,E=13329311,C=162)
	CT(68228)=(S=13329312,E=13329343,C=77)
	CT(68229)=(S=13329344,E=13329375,C=4)
	CT(68230)=(S=13329376,E=13329383,C=2)
	CT(68231)=(S=13329384,E=13329391,C=10)
	CT(68232)=(S=13329392,E=13329399,C=2)
	CT(68233)=(S=13329400,E=13329403,C=11)
	CT(68234)=(S=13329404,E=13329405,C=3)
	CT(68235)=(S=13329406,E=13329406,C=8)
	CT(68236)=(S=13329407,E=13329407,C=162)
	CT(68237)=(S=13329408,E=13329439,C=0)
	CT(68238)=(S=13329440,E=13329463,C=1)
	CT(68239)=(S=13329464,E=13329464,C=0)
	CT(68240)=(S=13329465,E=13329465,C=8)
	CT(68241)=(S=13329467,E=13329469,C=0)
	CT(68242)=(S=13329470,E=13329470,C=7)
	CT(68243)=(S=13329471,E=13329471,C=1)
	CT(68244)=(S=13329472,E=13329487,C=4)
	CT(68245)=(S=13329488,E=13329535,C=1)
	CT(68246)=(S=13329536,E=13329567,C=8)
	CT(68247)=(S=13329568,E=13329599,C=6)
	CT(68248)=(S=13329600,E=13329615,C=1)
	CT(68249)=(S=13329616,E=13329631,C=73)
	CT(68250)=(S=13329632,E=13329663,C=2)
	CT(68251)=(S=13329664,E=13329791,C=4)
	CT(68252)=(S=13329792,E=13329823,C=8)
	CT(68253)=(S=13329824,E=13329855,C=77)
	CT(68254)=(S=13329856,E=13329887,C=7)
	CT(68255)=(S=13329888,E=13330447,C=0)
	CT(68256)=(S=13330448,E=13330455,C=4)
	CT(68257)=(S=13330456,E=13330463,C=88)
	CT(68258)=(S=13330464,E=13330479,C=1)
	CT(68259)=(S=13330480,E=13330495,C=90)
	CT(68260)=(S=13330496,E=13330527,C=10)
	CT(68261)=(S=13330528,E=13330687,C=2)
	CT(68262)=(S=13330688,E=13330751,C=8)
	CT(68263)=(S=13330752,E=13330783,C=2)
	CT(68264)=(S=13330784,E=13330847,C=1)
	CT(68265)=(S=13330848,E=13330879,C=4)
	CT(68266)=(S=13330880,E=13330911,C=0)
	CT(68267)=(S=13330912,E=13330943,C=9)
	CT(68268)=(S=13330944,E=13331199,C=5)
	CT(68269)=(S=13331200,E=13331327,C=1)
	CT(68270)=(S=13331328,E=13331455,C=3)
	CT(68271)=(S=13331456,E=13331711,C=0)
	CT(68272)=(S=13331712,E=13331743,C=6)
	CT(68273)=(S=13331744,E=13331775,C=5)
	CT(68274)=(S=13331776,E=13331839,C=4)
	CT(68275)=(S=13331840,E=13332031,C=73)
	CT(68276)=(S=13332032,E=13332047,C=154)
	CT(68277)=(S=13332048,E=13332063,C=4)
	CT(68278)=(S=13332064,E=13332095,C=2)
	CT(68279)=(S=13332096,E=13332127,C=0)
	CT(68280)=(S=13332128,E=13332159,C=1)
	CT(68281)=(S=13332160,E=13332175,C=0)
	CT(68282)=(S=13332176,E=13332191,C=1)
	CT(68283)=(S=13332192,E=13332199,C=2)
	CT(68284)=(S=13332200,E=13332202,C=1)
	CT(68285)=(S=13332203,E=13332203,C=0)
	CT(68286)=(S=13332204,E=13332207,C=8)
	CT(68287)=(S=13332208,E=13332215,C=4)
	CT(68288)=(S=13332216,E=13332415,C=0)
	CT(68289)=(S=13332416,E=13332431,C=2)
	CT(68290)=(S=13332432,E=13332447,C=9)
	CT(68291)=(S=13332448,E=13332463,C=10)
	CT(68292)=(S=13332464,E=13332471,C=2)
	CT(68293)=(S=13332472,E=13332475,C=4)
	CT(68294)=(S=13332480,E=13332551,C=2)
	CT(68295)=(S=13332552,E=13332559,C=84)
	CT(68296)=(S=13332560,E=13332575,C=8)
	CT(68297)=(S=13332576,E=13332607,C=0)
	CT(68298)=(S=13332608,E=13332639,C=4)
	CT(68299)=(S=13332640,E=13332671,C=7)
	CT(68300)=(S=13332672,E=13332703,C=84)
	CT(68301)=(S=13332704,E=13332735,C=8)
	CT(68302)=(S=13332736,E=13332863,C=3)
	CT(68303)=(S=13332864,E=13332927,C=11)
	CT(68304)=(S=13332928,E=13332991,C=0)
	CT(68305)=(S=13332992,E=13333055,C=2)
	CT(68306)=(S=13333056,E=13333059,C=10)
	CT(68307)=(S=13333088,E=13333119,C=3)
	CT(68308)=(S=13333120,E=13333183,C=73)
	CT(68309)=(S=13333184,E=13333215,C=2)
	CT(68310)=(S=13333216,E=13333219,C=74)
	CT(68311)=(S=13333220,E=13333223,C=2)
	CT(68312)=(S=13333224,E=13333227,C=4)
	CT(68313)=(S=13333228,E=13333231,C=10)
	CT(68314)=(S=13333232,E=13333235,C=4)
	CT(68315)=(S=13333236,E=13333239,C=1)
	CT(68316)=(S=13333240,E=13333243,C=2)
	CT(68317)=(S=13333244,E=13333247,C=8)
	CT(68318)=(S=13333248,E=13333311,C=156)
	CT(68319)=(S=13333312,E=13333375,C=4)
	CT(68320)=(S=13333376,E=13333439,C=10)
	CT(68321)=(S=13333440,E=13333503,C=5)
	CT(68322)=(S=13333504,E=13334079,C=7)
	CT(68323)=(S=13334080,E=13334143,C=3)
	CT(68324)=(S=13334144,E=13334207,C=73)
	CT(68325)=(S=13334208,E=13334239,C=1)
	CT(68326)=(S=13334240,E=13334255,C=9)
	CT(68327)=(S=13334256,E=13334256,C=7)
	CT(68328)=(S=13334257,E=13334257,C=1)
	CT(68329)=(S=13334258,E=13334258,C=86)
	CT(68330)=(S=13334259,E=13334259,C=0)
	CT(68331)=(S=13334260,E=13334263,C=10)
	CT(68332)=(S=13334264,E=13334267,C=1)
	CT(68333)=(S=13334268,E=13334271,C=2)
	CT(68334)=(S=13334272,E=13334272,C=0)
	CT(68335)=(S=13334273,E=13334273,C=2)
	CT(68336)=(S=13334274,E=13334274,C=8)
	CT(68337)=(S=13334275,E=13334275,C=9)
	CT(68338)=(S=13334276,E=13334279,C=10)
	CT(68339)=(S=13334280,E=13334283,C=11)
	CT(68340)=(S=13334284,E=13334284,C=0)
	CT(68341)=(S=13334285,E=13334285,C=74)
	CT(68342)=(S=13334286,E=13334287,C=0)
	CT(68343)=(S=13334288,E=13334291,C=2)
	CT(68344)=(S=13334292,E=13334292,C=77)
	CT(68345)=(S=13334293,E=13334293,C=0)
	CT(68346)=(S=13334294,E=13334295,C=2)
	CT(68347)=(S=13334296,E=13334307,C=1)
	CT(68348)=(S=13334308,E=13334311,C=11)
	CT(68349)=(S=13334312,E=13334312,C=2)
	CT(68350)=(S=13334313,E=13334313,C=74)
	CT(68351)=(S=13334314,E=13334315,C=0)
	CT(68352)=(S=13334316,E=13334319,C=11)
	CT(68353)=(S=13334320,E=13334320,C=74)
	CT(68354)=(S=13334321,E=13334322,C=4)
	CT(68355)=(S=13334323,E=13334323,C=91)
	CT(68356)=(S=13334324,E=13334327,C=74)
	CT(68357)=(S=13334328,E=13334328,C=73)
	CT(68358)=(S=13334329,E=13334329,C=5)
	CT(68359)=(S=13334330,E=13334347,C=11)
	CT(68360)=(S=13334348,E=13334349,C=0)
	CT(68361)=(S=13334350,E=13334350,C=7)
	CT(68362)=(S=13334351,E=13334351,C=86)
	CT(68363)=(S=13334352,E=13334355,C=1)
	CT(68364)=(S=13334356,E=13334356,C=73)
	CT(68365)=(S=13334357,E=13334357,C=1)
	CT(68366)=(S=13334358,E=13334358,C=0)
	CT(68367)=(S=13334359,E=13334359,C=8)
	CT(68368)=(S=13334360,E=13334363,C=75)
	CT(68369)=(S=13334364,E=13334365,C=0)
	CT(68370)=(S=13334366,E=13334366,C=9)
	CT(68371)=(S=13334367,E=13334383,C=0)
	CT(68372)=(S=13334384,E=13334384,C=74)
	CT(68373)=(S=13334385,E=13334527,C=1)
	CT(68374)=(S=13334528,E=13334783,C=2)
	CT(68375)=(S=13334784,E=13334911,C=5)
	CT(68376)=(S=13334912,E=13334975,C=2)
	CT(68377)=(S=13334976,E=13335007,C=0)
	CT(68378)=(S=13335008,E=13335039,C=9)
	CT(68379)=(S=13335040,E=13335103,C=4)
	CT(68380)=(S=13335104,E=13335295,C=0)
	CT(68381)=(S=13335296,E=13335327,C=7)
	CT(68382)=(S=13335328,E=13335343,C=4)
	CT(68383)=(S=13335344,E=13335351,C=7)
	CT(68384)=(S=13335355,E=13335355,C=0)
	CT(68385)=(S=13335356,E=13335359,C=74)
	CT(68386)=(S=13335360,E=13335423,C=0)
	CT(68387)=(S=13335424,E=13335487,C=4)
	CT(68388)=(S=13335488,E=13335519,C=6)
	CT(68389)=(S=13335520,E=13335551,C=74)
	CT(68390)=(S=13335552,E=13335557,C=7)
	CT(68391)=(S=13335558,E=13335559,C=2)
	CT(68392)=(S=13335560,E=13335567,C=8)
	CT(68393)=(S=13335568,E=13335575,C=4)
	CT(68394)=(S=13335576,E=13335615,C=77)
	CT(68395)=(S=13335616,E=13335647,C=2)
	CT(68396)=(S=13335648,E=13335679,C=7)
	CT(68397)=(S=13335680,E=13335727,C=4)
	CT(68398)=(S=13335728,E=13335743,C=0)
	CT(68399)=(S=13335744,E=13335807,C=4)
	CT(68400)=(S=13335808,E=13336575,C=7)
	CT(68401)=(S=13336576,E=13336607,C=77)
	CT(68402)=(S=13336608,E=13336639,C=1)
	CT(68403)=(S=13336640,E=13336671,C=74)
	CT(68404)=(S=13336672,E=13336703,C=1)
	CT(68405)=(S=13336704,E=13336735,C=0)
	CT(68406)=(S=13336736,E=13336799,C=6)
	CT(68407)=(S=13336800,E=13336807,C=1)
	CT(68408)=(S=13336808,E=13336811,C=0)
	CT(68409)=(S=13336812,E=13336815,C=6)
	CT(68410)=(S=13336816,E=13336823,C=11)
	CT(68411)=(S=13336824,E=13336827,C=74)
	CT(68412)=(S=13336828,E=13336835,C=77)
	CT(68413)=(S=13336836,E=13336837,C=10)
	CT(68414)=(S=13336838,E=13336839,C=6)
	CT(68415)=(S=13336840,E=13336847,C=1)
	CT(68416)=(S=13336848,E=13336895,C=0)
	CT(68417)=(S=13336896,E=13336927,C=8)
	CT(68418)=(S=13336928,E=13336959,C=2)
	CT(68419)=(S=13336960,E=13336991,C=0)
	CT(68420)=(S=13336992,E=13337023,C=2)
	CT(68421)=(S=13337024,E=13337087,C=4)
	CT(68422)=(S=13337088,E=13337119,C=77)
	CT(68423)=(S=13337120,E=13337151,C=1)
	CT(68424)=(S=13337152,E=13337215,C=6)
	CT(68425)=(S=13337216,E=13337247,C=3)
	CT(68426)=(S=13337248,E=13337263,C=0)
	CT(68427)=(S=13337264,E=13337279,C=6)
	CT(68428)=(S=13337280,E=13337343,C=74)
	CT(68429)=(S=13337344,E=13337407,C=0)
	CT(68430)=(S=13337408,E=13337535,C=10)
	CT(68431)=(S=13337536,E=13337551,C=2)
	CT(68432)=(S=13337552,E=13337567,C=3)
	CT(68433)=(S=13337568,E=13337599,C=8)
	CT(68434)=(S=13337600,E=13337631,C=0)
	CT(68435)=(S=13337632,E=13337663,C=1)
	CT(68436)=(S=13337664,E=13337695,C=0)
	CT(68437)=(S=13337696,E=13337727,C=2)
	CT(68438)=(S=13337728,E=13337759,C=4)
	CT(68439)=(S=13337760,E=13337791,C=6)
	CT(68440)=(S=13337792,E=13337807,C=8)
	CT(68441)=(S=13337808,E=13337823,C=4)
	CT(68442)=(S=13337824,E=13337855,C=0)
	CT(68443)=(S=13337856,E=13337983,C=9)
	CT(68444)=(S=13337984,E=13338015,C=2)
	CT(68445)=(S=13338016,E=13338047,C=6)
	CT(68446)=(S=13338048,E=13338107,C=2)
	CT(68447)=(S=13338108,E=13338111,C=77)
	CT(68448)=(S=13338112,E=13338303,C=0)
	CT(68449)=(S=13338304,E=13338343,C=4)
	CT(68450)=(S=13338344,E=13338351,C=74)
	CT(68451)=(S=13338352,E=13338359,C=1)
	CT(68452)=(S=13338362,E=13338363,C=4)
	CT(68453)=(S=13338364,E=13338431,C=77)
	CT(68454)=(S=13338432,E=13338463,C=9)
	CT(68455)=(S=13338464,E=13338495,C=1)
	CT(68456)=(S=13338496,E=13338527,C=8)
	CT(68457)=(S=13338528,E=13338543,C=1)
	CT(68458)=(S=13338544,E=13338551,C=74)
	CT(68459)=(S=13338552,E=13338555,C=0)
	CT(68460)=(S=13338556,E=13338556,C=4)
	CT(68461)=(S=13338557,E=13338557,C=7)
	CT(68462)=(S=13338558,E=13338559,C=5)
	CT(68463)=(S=13338560,E=13340159,C=2)
	CT(68464)=(S=13340160,E=13340191,C=7)
	CT(68465)=(S=13340192,E=13340223,C=5)
	CT(68466)=(S=13340224,E=13340247,C=74)
	CT(68467)=(S=13340248,E=13340287,C=8)
	CT(68468)=(S=13340288,E=13340319,C=0)
	CT(68469)=(S=13340320,E=13340351,C=6)
	CT(68470)=(S=13340352,E=13340375,C=2)
	CT(68471)=(S=13340376,E=13340377,C=6)
	CT(68472)=(S=13340380,E=13340380,C=4)
	CT(68473)=(S=13340381,E=13340381,C=195)
	CT(68474)=(S=13340382,E=13340383,C=0)
	CT(68475)=(S=13340384,E=13340415,C=1)
	CT(68476)=(S=13340416,E=13340479,C=156)
	CT(68477)=(S=13340480,E=13340511,C=0)
	CT(68478)=(S=13340512,E=13340543,C=2)
	CT(68479)=(S=13340544,E=13340575,C=7)
	CT(68480)=(S=13340576,E=13340591,C=0)
	CT(68481)=(S=13340592,E=13340607,C=4)
	CT(68482)=(S=13340608,E=13340671,C=2)
	CT(68483)=(S=13340672,E=13340703,C=0)
	CT(68484)=(S=13340704,E=13340719,C=73)
	CT(68485)=(S=13340720,E=13340735,C=29)
	CT(68486)=(S=13340736,E=13340767,C=86)
	CT(68487)=(S=13340768,E=13340799,C=1)
	CT(68488)=(S=13340800,E=13340927,C=3)
	CT(68489)=(S=13340928,E=13340959,C=1)
	CT(68490)=(S=13340960,E=13340991,C=0)
	CT(68491)=(S=13340992,E=13341023,C=8)
	CT(68492)=(S=13341024,E=13341039,C=2)
	CT(68493)=(S=13341040,E=13341047,C=3)
	CT(68494)=(S=13341048,E=13341055,C=2)
	CT(68495)=(S=13341056,E=13341119,C=4)
	CT(68496)=(S=13341120,E=13341151,C=9)
	CT(68497)=(S=13341152,E=13341183,C=2)
	CT(68498)=(S=13341184,E=13341503,C=3)
	CT(68499)=(S=13341504,E=13341527,C=90)
	CT(68500)=(S=13341528,E=13341531,C=4)
	CT(68501)=(S=13341532,E=13341543,C=0)
	CT(68502)=(S=13341544,E=13341547,C=10)
	CT(68503)=(S=13341548,E=13341549,C=0)
	CT(68504)=(S=13341550,E=13341551,C=8)
	CT(68505)=(S=13341552,E=13341567,C=2)
	CT(68506)=(S=13341568,E=13341695,C=0)
	CT(68507)=(S=13341696,E=13341779,C=1)
	CT(68508)=(S=13341780,E=13341781,C=74)
	CT(68509)=(S=13341782,E=13341783,C=1)
	CT(68510)=(S=13341788,E=13341788,C=9)
	CT(68511)=(S=13341789,E=13341791,C=0)
	CT(68512)=(S=13341792,E=13341823,C=2)
	CT(68513)=(S=13341824,E=13342015,C=3)
	CT(68514)=(S=13342016,E=13342043,C=0)
	CT(68515)=(S=13342044,E=13342047,C=1)
	CT(68516)=(S=13342048,E=13342063,C=2)
	CT(68517)=(S=13342064,E=13342079,C=6)
	CT(68518)=(S=13342080,E=13342207,C=7)
	CT(68519)=(S=13342208,E=13342719,C=3)
	CT(68520)=(S=13342720,E=13342783,C=2)
	CT(68521)=(S=13342784,E=13342815,C=1)
	CT(68522)=(S=13342816,E=13342847,C=73)
	CT(68523)=(S=13342848,E=13342879,C=1)
	CT(68524)=(S=13342880,E=13342911,C=6)
	CT(68525)=(S=13342912,E=13342943,C=2)
	CT(68526)=(S=13342944,E=13342975,C=0)
	CT(68527)=(S=13342976,E=13342979,C=1)
	CT(68528)=(S=13342980,E=13342981,C=8)
	CT(68529)=(S=13342982,E=13342991,C=10)
	CT(68530)=(S=13343000,E=13343007,C=74)
	CT(68531)=(S=13343008,E=13343024,C=4)
	CT(68532)=(S=13343025,E=13343025,C=74)
	CT(68533)=(S=13343026,E=13343027,C=3)
	CT(68534)=(S=13343028,E=13343031,C=4)
	CT(68535)=(S=13343032,E=13343055,C=2)
	CT(68536)=(S=13343056,E=13343071,C=5)
	CT(68537)=(S=13343072,E=13343103,C=74)
	CT(68538)=(S=13343104,E=13343119,C=3)
	CT(68539)=(S=13343120,E=13343135,C=6)
	CT(68540)=(S=13343136,E=13343167,C=3)
	CT(68541)=(S=13343168,E=13343183,C=0)
	CT(68542)=(S=13343186,E=13343187,C=4)
	CT(68543)=(S=13343188,E=13343195,C=74)
	CT(68544)=(S=13343196,E=13343199,C=156)
	CT(68545)=(S=13343200,E=13343231,C=0)
	CT(68546)=(S=13343232,E=13343935,C=3)
	CT(68547)=(S=13343936,E=13343999,C=1)
	CT(68548)=(S=13344000,E=13344263,C=3)
	CT(68549)=(S=13344264,E=13344271,C=10)
	CT(68550)=(S=13344272,E=13344279,C=1)
	CT(68551)=(S=13344280,E=13344287,C=5)
	CT(68552)=(S=13344288,E=13344319,C=0)
	CT(68553)=(S=13344320,E=13344351,C=4)
	CT(68554)=(S=13344352,E=13344767,C=3)
	CT(68555)=(S=13344768,E=13344769,C=11)
	CT(68556)=(S=13344770,E=13344771,C=0)
	CT(68557)=(S=13344772,E=13344775,C=3)
	CT(68558)=(S=13344776,E=13344783,C=0)
	CT(68559)=(S=13344784,E=13344799,C=2)
	CT(68560)=(S=13344800,E=13344815,C=8)
	CT(68561)=(S=13344824,E=13344831,C=74)
	CT(68562)=(S=13344832,E=13344863,C=8)
	CT(68563)=(S=13344864,E=13344871,C=11)
	CT(68564)=(S=13344872,E=13344879,C=1)
	CT(68565)=(S=13344880,E=13344895,C=73)
	CT(68566)=(S=13344896,E=13344896,C=74)
	CT(68567)=(S=13344897,E=13344897,C=1)
	CT(68568)=(S=13344898,E=13344899,C=10)
	CT(68569)=(S=13344904,E=13344904,C=4)
	CT(68570)=(S=13344905,E=13344905,C=3)
	CT(68571)=(S=13344906,E=13344907,C=4)
	CT(68572)=(S=13344908,E=13344911,C=6)
	CT(68573)=(S=13344912,E=13344927,C=9)
	CT(68574)=(S=13344928,E=13344959,C=10)
	CT(68575)=(S=13344960,E=13344991,C=1)
	CT(68576)=(S=13344992,E=13345023,C=9)
	CT(68577)=(S=13345024,E=13345027,C=1)
	CT(68578)=(S=13345028,E=13345031,C=8)
	CT(68579)=(S=13345032,E=13345039,C=0)
	CT(68580)=(S=13345040,E=13345055,C=74)
	CT(68581)=(S=13345056,E=13345087,C=5)
	CT(68582)=(S=13345088,E=13345199,C=0)
	CT(68583)=(S=13345200,E=13345201,C=10)
	CT(68584)=(S=13345202,E=13345202,C=11)
	CT(68585)=(S=13345204,E=13345204,C=1)
	CT(68586)=(S=13345205,E=13345205,C=7)
	CT(68587)=(S=13345206,E=13345211,C=0)
	CT(68588)=(S=13345212,E=13345212,C=10)
	CT(68589)=(S=13345213,E=13345213,C=4)
	CT(68590)=(S=13345214,E=13345215,C=74)
	CT(68591)=(S=13345216,E=13345247,C=1)
	CT(68592)=(S=13345248,E=13345279,C=8)
	CT(68593)=(S=13345280,E=13345535,C=11)
	CT(68594)=(S=13345536,E=13345663,C=2)
	CT(68595)=(S=13345664,E=13345727,C=11)
	CT(68596)=(S=13345728,E=13345759,C=9)
	CT(68597)=(S=13345760,E=13345791,C=4)
	CT(68598)=(S=13345792,E=13346047,C=0)
	CT(68599)=(S=13346048,E=13346303,C=2)
	CT(68600)=(S=13346304,E=13346431,C=0)
	CT(68601)=(S=13346432,E=13346463,C=7)
	CT(68602)=(S=13346464,E=13346495,C=1)
	CT(68603)=(S=13346496,E=13346511,C=74)
	CT(68604)=(S=13346512,E=13346519,C=6)
	CT(68605)=(S=13346520,E=13346520,C=0)
	CT(68606)=(S=13346521,E=13346521,C=74)
	CT(68607)=(S=13346522,E=13346523,C=0)
	CT(68608)=(S=13346524,E=13346527,C=8)
	CT(68609)=(S=13346528,E=13346559,C=0)
	CT(68610)=(S=13346560,E=13346567,C=10)
	CT(68611)=(S=13346592,E=13346623,C=2)
	CT(68612)=(S=13346624,E=13346687,C=10)
	CT(68613)=(S=13346688,E=13346815,C=73)
	CT(68614)=(S=13346816,E=13346847,C=1)
	CT(68615)=(S=13346848,E=13346879,C=0)
	CT(68616)=(S=13346880,E=13346943,C=2)
	CT(68617)=(S=13346944,E=13347071,C=8)
	CT(68618)=(S=13347072,E=13347075,C=10)
	CT(68619)=(S=13347076,E=13347077,C=6)
	CT(68620)=(S=13347078,E=13347079,C=7)
	CT(68621)=(S=13347080,E=13347087,C=2)
	CT(68622)=(S=13347088,E=13347095,C=0)
	CT(68623)=(S=13347096,E=13347103,C=84)
	CT(68624)=(S=13347104,E=13347119,C=8)
	CT(68625)=(S=13347120,E=13347127,C=92)
	CT(68626)=(S=13347128,E=13347135,C=8)
	CT(68627)=(S=13347136,E=13347199,C=7)
	CT(68628)=(S=13347200,E=13347327,C=8)
	CT(68629)=(S=13347328,E=13347343,C=0)
	CT(68630)=(S=13347344,E=13347347,C=3)
	CT(68631)=(S=13347348,E=13347351,C=10)
	CT(68632)=(S=13347352,E=13347352,C=0)
	CT(68633)=(S=13347353,E=13347353,C=4)
	CT(68634)=(S=13347354,E=13347355,C=11)
	CT(68635)=(S=13347356,E=13347359,C=8)
	CT(68636)=(S=13347360,E=13347375,C=2)
	CT(68637)=(S=13347376,E=13347376,C=4)
	CT(68638)=(S=13347377,E=13347377,C=8)
	CT(68639)=(S=13347378,E=13347379,C=3)
	CT(68640)=(S=13347380,E=13347383,C=2)
	CT(68641)=(S=13347384,E=13347384,C=73)
	CT(68642)=(S=13347385,E=13347385,C=0)
	CT(68643)=(S=13347386,E=13347387,C=1)
	CT(68644)=(S=13347388,E=13347391,C=4)
	CT(68645)=(S=13347392,E=13347407,C=77)
	CT(68646)=(S=13347408,E=13347415,C=0)
	CT(68647)=(S=13347418,E=13347419,C=8)
	CT(68648)=(S=13347420,E=13347423,C=0)
	CT(68649)=(S=13347424,E=13347455,C=6)
	CT(68650)=(S=13347456,E=13347583,C=3)
	CT(68651)=(S=13347584,E=13347587,C=1)
	CT(68652)=(S=13347588,E=13347588,C=73)
	CT(68653)=(S=13347589,E=13347589,C=0)
	CT(68654)=(S=13347590,E=13347591,C=10)
	CT(68655)=(S=13347592,E=13347599,C=2)
	CT(68656)=(S=13347600,E=13347615,C=11)
	CT(68657)=(S=13347616,E=13347647,C=73)
	CT(68658)=(S=13347648,E=13347711,C=0)
	CT(68659)=(S=13347712,E=13347743,C=2)
	CT(68660)=(S=13347744,E=13347775,C=6)
	CT(68661)=(S=13347776,E=13347791,C=0)
	CT(68662)=(S=13347793,E=13347795,C=4)
	CT(68663)=(S=13347796,E=13347799,C=2)
	CT(68664)=(S=13347800,E=13347803,C=74)
	CT(68665)=(S=13347804,E=13347804,C=0)
	CT(68666)=(S=13347805,E=13347805,C=74)
	CT(68667)=(S=13347806,E=13347806,C=4)
	CT(68668)=(S=13347807,E=13347807,C=73)
	CT(68669)=(S=13347808,E=13347823,C=1)
	CT(68670)=(S=13347824,E=13347831,C=4)
	CT(68671)=(S=13347832,E=13347839,C=0)
	CT(68672)=(S=13347840,E=13347871,C=10)
	CT(68673)=(S=13347872,E=13348095,C=3)
	CT(68674)=(S=13348096,E=13348159,C=0)
	CT(68675)=(S=13348160,E=13348183,C=2)
	CT(68676)=(S=13348184,E=13348191,C=74)
	CT(68677)=(S=13348192,E=13348223,C=6)
	CT(68678)=(S=13348224,E=13348351,C=73)
	CT(68679)=(S=13348352,E=13348355,C=10)
	CT(68680)=(S=13348356,E=13348356,C=1)
	CT(68681)=(S=13348357,E=13348357,C=74)
	CT(68682)=(S=13348359,E=13348359,C=1)
	CT(68683)=(S=13348360,E=13348367,C=74)
	CT(68684)=(S=13348368,E=13348377,C=0)
	CT(68685)=(S=13348378,E=13348378,C=92)
	CT(68686)=(S=13348379,E=13348379,C=89)
	CT(68687)=(S=13348380,E=13348381,C=7)
	CT(68688)=(S=13348382,E=13348383,C=73)
	CT(68689)=(S=13348384,E=13348415,C=8)
	CT(68690)=(S=13348416,E=13348431,C=2)
	CT(68691)=(S=13348432,E=13348439,C=7)
	CT(68692)=(S=13348440,E=13348447,C=2)
	CT(68693)=(S=13348448,E=13348479,C=1)
	CT(68694)=(S=13348480,E=13348543,C=0)
	CT(68695)=(S=13348544,E=13348607,C=2)
	CT(68696)=(S=13348608,E=13348611,C=10)
	CT(68697)=(S=13348612,E=13348615,C=4)
	CT(68698)=(S=13348624,E=13348639,C=2)
	CT(68699)=(S=13348640,E=13348671,C=6)
	CT(68700)=(S=13348672,E=13348687,C=77)
	CT(68701)=(S=13348688,E=13348703,C=2)
	CT(68702)=(S=13348720,E=13348727,C=73)
	CT(68703)=(S=13348728,E=13348731,C=0)
	CT(68704)=(S=13348732,E=13348735,C=10)
	CT(68705)=(S=13348736,E=13348767,C=1)
	CT(68706)=(S=13348768,E=13348783,C=7)
	CT(68707)=(S=13348784,E=13348785,C=4)
	CT(68708)=(S=13348786,E=13348787,C=0)
	CT(68709)=(S=13348788,E=13348793,C=4)
	CT(68710)=(S=13348794,E=13348795,C=0)
	CT(68711)=(S=13348796,E=13348799,C=6)
	CT(68712)=(S=13348800,E=13348959,C=1)
	CT(68713)=(S=13348960,E=13348975,C=0)
	CT(68714)=(S=13348976,E=13348977,C=4)
	CT(68715)=(S=13348978,E=13348979,C=8)
	CT(68716)=(S=13348980,E=13348983,C=10)
	CT(68717)=(S=13348984,E=13348985,C=0)
	CT(68718)=(S=13348986,E=13348987,C=73)
	CT(68719)=(S=13348992,E=13349007,C=86)
	CT(68720)=(S=13349008,E=13349015,C=5)
	CT(68721)=(S=13349016,E=13349016,C=0)
	CT(68722)=(S=13349017,E=13349017,C=4)
	CT(68723)=(S=13349018,E=13349019,C=73)
	CT(68724)=(S=13349020,E=13349021,C=4)
	CT(68725)=(S=13349022,E=13349023,C=0)
	CT(68726)=(S=13349024,E=13349031,C=11)
	CT(68727)=(S=13349032,E=13349039,C=1)
	CT(68728)=(S=13349040,E=13349047,C=74)
	CT(68729)=(S=13349048,E=13349051,C=0)
	CT(68730)=(S=13349052,E=13349052,C=73)
	CT(68731)=(S=13349053,E=13349053,C=0)
	CT(68732)=(S=13349054,E=13349055,C=77)
	CT(68733)=(S=13349056,E=13349119,C=8)
	CT(68734)=(S=13349120,E=13349375,C=10)
	CT(68735)=(S=13349376,E=13350911,C=2)
	CT(68736)=(S=13350912,E=13350975,C=73)
	CT(68737)=(S=13350976,E=13350991,C=0)
	CT(68738)=(S=13350992,E=13351007,C=1)
	CT(68739)=(S=13351008,E=13351039,C=2)
	CT(68740)=(S=13351040,E=13351231,C=8)
	CT(68741)=(S=13351232,E=13351327,C=3)
	CT(68742)=(S=13351328,E=13351351,C=164)
	CT(68743)=(S=13351352,E=13351358,C=0)
	CT(68744)=(S=13351359,E=13351359,C=4)
	CT(68745)=(S=13351360,E=13351423,C=0)
	CT(68746)=(S=13351424,E=13351679,C=8)
	CT(68747)=(S=13351680,E=13351807,C=9)
	CT(68748)=(S=13351808,E=13351839,C=29)
	CT(68749)=(S=13351840,E=13351871,C=1)
	CT(68750)=(S=13351872,E=13351935,C=4)
	CT(68751)=(S=13351936,E=13351999,C=3)
	CT(68752)=(S=13352000,E=13352063,C=8)
	CT(68753)=(S=13352064,E=13352095,C=0)
	CT(68754)=(S=13352096,E=13352127,C=84)
	CT(68755)=(S=13352128,E=13352143,C=9)
	CT(68756)=(S=13352144,E=13352151,C=2)
	CT(68757)=(S=13352152,E=13352159,C=0)
	CT(68758)=(S=13352160,E=13352167,C=4)
	CT(68759)=(S=13352168,E=13352175,C=5)
	CT(68760)=(S=13352176,E=13352191,C=84)
	CT(68761)=(S=13352192,E=13352193,C=1)
	CT(68762)=(S=13352194,E=13352196,C=0)
	CT(68763)=(S=13352197,E=13352197,C=4)
	CT(68764)=(S=13352198,E=13352199,C=1)
	CT(68765)=(S=13352200,E=13352207,C=10)
	CT(68766)=(S=13352208,E=13352215,C=5)
	CT(68767)=(S=13352216,E=13352216,C=4)
	CT(68768)=(S=13352217,E=13352217,C=0)
	CT(68769)=(S=13352218,E=13352219,C=8)
	CT(68770)=(S=13352220,E=13352223,C=11)
	CT(68771)=(S=13352224,E=13352255,C=2)
	CT(68772)=(S=13352256,E=13352271,C=156)
	CT(68773)=(S=13352272,E=13352279,C=0)
	CT(68774)=(S=13352280,E=13352281,C=74)
	CT(68775)=(S=13352282,E=13352284,C=4)
	CT(68776)=(S=13352285,E=13352287,C=0)
	CT(68777)=(S=13352288,E=13352303,C=2)
	CT(68778)=(S=13352304,E=13352307,C=1)
	CT(68779)=(S=13352308,E=13352311,C=3)
	CT(68780)=(S=13352312,E=13352315,C=74)
	CT(68781)=(S=13352316,E=13352319,C=0)
	CT(68782)=(S=13352320,E=13352351,C=86)
	CT(68783)=(S=13352352,E=13352367,C=8)
	CT(68784)=(S=13352368,E=13352375,C=4)
	CT(68785)=(S=13352376,E=13352383,C=156)
	CT(68786)=(S=13352384,E=13352415,C=1)
	CT(68787)=(S=13352416,E=13352451,C=84)
	CT(68788)=(S=13352452,E=13352455,C=6)
	CT(68789)=(S=13352456,E=13352463,C=84)
	CT(68790)=(S=13352464,E=13352471,C=9)
	CT(68791)=(S=13352472,E=13352473,C=0)
	CT(68792)=(S=13352474,E=13352475,C=6)
	CT(68793)=(S=13352476,E=13352479,C=2)
	CT(68794)=(S=13352484,E=13352485,C=74)
	CT(68795)=(S=13352486,E=13352487,C=8)
	CT(68796)=(S=13352488,E=13352503,C=74)
	CT(68797)=(S=13352504,E=13352511,C=2)
	CT(68798)=(S=13352512,E=13352543,C=8)
	CT(68799)=(S=13352544,E=13352559,C=1)
	CT(68800)=(S=13352560,E=13352567,C=74)
	CT(68801)=(S=13352568,E=13352575,C=8)
	CT(68802)=(S=13352576,E=13352607,C=4)
	CT(68803)=(S=13352608,E=13352623,C=11)
	CT(68804)=(S=13352624,E=13352631,C=7)
	CT(68805)=(S=13352632,E=13352639,C=74)
	CT(68806)=(S=13352640,E=13352655,C=0)
	CT(68807)=(S=13352656,E=13352663,C=73)
	CT(68808)=(S=13352665,E=13352665,C=0)
	CT(68809)=(S=13352666,E=13352669,C=10)
	CT(68810)=(S=13352670,E=13352671,C=0)
	CT(68811)=(S=13352672,E=13352687,C=7)
	CT(68812)=(S=13352688,E=13352695,C=74)
	CT(68813)=(S=13352696,E=13352696,C=4)
	CT(68814)=(S=13352697,E=13352697,C=1)
	CT(68815)=(S=13352698,E=13352699,C=3)
	CT(68816)=(S=13352700,E=13352701,C=0)
	CT(68817)=(S=13352702,E=13352703,C=84)
	CT(68818)=(S=13352704,E=13352705,C=1)
	CT(68819)=(S=13352707,E=13352708,C=8)
	CT(68820)=(S=13352710,E=13352710,C=0)
	CT(68821)=(S=13352712,E=13352719,C=11)
	CT(68822)=(S=13352720,E=13352735,C=1)
	CT(68823)=(S=13352736,E=13352737,C=84)
	CT(68824)=(S=13352738,E=13352739,C=4)
	CT(68825)=(S=13352740,E=13352743,C=0)
	CT(68826)=(S=13352744,E=13352751,C=74)
	CT(68827)=(S=13352752,E=13352759,C=11)
	CT(68828)=(S=13352768,E=13352831,C=1)
	CT(68829)=(S=13352832,E=13352835,C=160)
	CT(68830)=(S=13352836,E=13352836,C=0)
	CT(68831)=(S=13352838,E=13352839,C=10)
	CT(68832)=(S=13352840,E=13352847,C=2)
	CT(68833)=(S=13352848,E=13352863,C=1)
	CT(68834)=(S=13352864,E=13352911,C=0)
	CT(68835)=(S=13352912,E=13352959,C=2)
	CT(68836)=(S=13352960,E=13352991,C=1)
	CT(68837)=(S=13352992,E=13353023,C=3)
	CT(68838)=(S=13353024,E=13353055,C=0)
	CT(68839)=(S=13353056,E=13353151,C=2)
	CT(68840)=(S=13353152,E=13353215,C=4)
	CT(68841)=(S=13353216,E=13353343,C=8)
	CT(68842)=(S=13353344,E=13353407,C=4)
	CT(68843)=(S=13353408,E=13353439,C=0)
	CT(68844)=(S=13353440,E=13353471,C=1)
	CT(68845)=(S=13353472,E=13353535,C=0)
	CT(68846)=(S=13353536,E=13353567,C=2)
	CT(68847)=(S=13353568,E=13353583,C=4)
	CT(68848)=(S=13353584,E=13353591,C=92)
	CT(68849)=(S=13353592,E=13353599,C=1)
	CT(68850)=(S=13353600,E=13353727,C=8)
	CT(68851)=(S=13353728,E=13353791,C=11)
	CT(68852)=(S=13353792,E=13353823,C=1)
	CT(68853)=(S=13353824,E=13353839,C=3)
	CT(68854)=(S=13353840,E=13353847,C=1)
	CT(68855)=(S=13353848,E=13353851,C=7)
	CT(68856)=(S=13353852,E=13353852,C=0)
	CT(68857)=(S=13353853,E=13353854,C=7)
	CT(68858)=(S=13353855,E=13353855,C=0)
	CT(68859)=(S=13353856,E=13353999,C=1)
	CT(68860)=(S=13354000,E=13354007,C=2)
	CT(68861)=(S=13354016,E=13354047,C=0)
	CT(68862)=(S=13354048,E=13354073,C=2)
	CT(68863)=(S=13354074,E=13354074,C=74)
	CT(68864)=(S=13354075,E=13354075,C=2)
	CT(68865)=(S=13354076,E=13354079,C=0)
	CT(68866)=(S=13354080,E=13354111,C=2)
	CT(68867)=(S=13354112,E=13354495,C=4)
	CT(68868)=(S=13354496,E=13354751,C=8)
	CT(68869)=(S=13354752,E=13355263,C=4)
	CT(68870)=(S=13355264,E=13355295,C=0)
	CT(68871)=(S=13355296,E=13355311,C=9)
	CT(68872)=(S=13355312,E=13355313,C=84)
	CT(68873)=(S=13355314,E=13355315,C=74)
	CT(68874)=(S=13355316,E=13355319,C=2)
	CT(68875)=(S=13355320,E=13355323,C=11)
	CT(68876)=(S=13355324,E=13355327,C=4)
	CT(68877)=(S=13355328,E=13355423,C=0)
	CT(68878)=(S=13355424,E=13355439,C=74)
	CT(68879)=(S=13355440,E=13355443,C=7)
	CT(68880)=(S=13355444,E=13355444,C=10)
	CT(68881)=(S=13355447,E=13355447,C=0)
	CT(68882)=(S=13355448,E=13355451,C=5)
	CT(68883)=(S=13355452,E=13355455,C=2)
	CT(68884)=(S=13355456,E=13355519,C=4)
	CT(68885)=(S=13355520,E=13355711,C=0)
	CT(68886)=(S=13355712,E=13355743,C=2)
	CT(68887)=(S=13355744,E=13355751,C=8)
	CT(68888)=(S=13355754,E=13355754,C=7)
	CT(68889)=(S=13355755,E=13355755,C=91)
	CT(68890)=(S=13355756,E=13355759,C=1)
	CT(68891)=(S=13355760,E=13355775,C=84)
	CT(68892)=(S=13355776,E=13356287,C=9)
	CT(68893)=(S=13356288,E=13356351,C=11)
	CT(68894)=(S=13356352,E=13356383,C=1)
	CT(68895)=(S=13356384,E=13356415,C=2)
	CT(68896)=(S=13356416,E=13356543,C=1)
	CT(68897)=(S=13356544,E=13356799,C=0)
	CT(68898)=(S=13356800,E=13356815,C=9)
	CT(68899)=(S=13356816,E=13356831,C=6)
	CT(68900)=(S=13356832,E=13356847,C=9)
	CT(68901)=(S=13356848,E=13356851,C=2)
	CT(68902)=(S=13356852,E=13356859,C=74)
	CT(68903)=(S=13356860,E=13356861,C=73)
	CT(68904)=(S=13356862,E=13356862,C=4)
	CT(68905)=(S=13356864,E=13357075,C=1)
	CT(68906)=(S=13357076,E=13357076,C=8)
	CT(68907)=(S=13357077,E=13357077,C=73)
	CT(68908)=(S=13357078,E=13357078,C=10)
	CT(68909)=(S=13357079,E=13357079,C=2)
	CT(68910)=(S=13357080,E=13357087,C=10)
	CT(68911)=(S=13357088,E=13357119,C=1)
	CT(68912)=(S=13357120,E=13357183,C=0)
	CT(68913)=(S=13357184,E=13357311,C=7)
	CT(68914)=(S=13357312,E=13357439,C=3)
	CT(68915)=(S=13357440,E=13357455,C=8)
	CT(68916)=(S=13357456,E=13357471,C=2)
	CT(68917)=(S=13357472,E=13357487,C=0)
	CT(68918)=(S=13357488,E=13357491,C=2)
	CT(68919)=(S=13357492,E=13357495,C=11)
	CT(68920)=(S=13357496,E=13357499,C=2)
	CT(68921)=(S=13357500,E=13357502,C=74)
	CT(68922)=(S=13357503,E=13357503,C=3)
	CT(68923)=(S=13357504,E=13357535,C=0)
	CT(68924)=(S=13357536,E=13357567,C=1)
	CT(68925)=(S=13357568,E=13357583,C=8)
	CT(68926)=(S=13357584,E=13357631,C=6)
	CT(68927)=(S=13357632,E=13357647,C=0)
	CT(68928)=(S=13357648,E=13357649,C=74)
	CT(68929)=(S=13357650,E=13357650,C=0)
	CT(68930)=(S=13357651,E=13357655,C=74)
	CT(68931)=(S=13357656,E=13357659,C=2)
	CT(68932)=(S=13357660,E=13357663,C=4)
	CT(68933)=(S=13357664,E=13357695,C=0)
	CT(68934)=(S=13357696,E=13357823,C=11)
	CT(68935)=(S=13357824,E=13357855,C=9)
	CT(68936)=(S=13357856,E=13357879,C=2)
	CT(68937)=(S=13357880,E=13357883,C=0)
	CT(68938)=(S=13357884,E=13357887,C=7)
	CT(68939)=(S=13357888,E=13357951,C=73)
	CT(68940)=(S=13357952,E=13357983,C=7)
	CT(68941)=(S=13357984,E=13358079,C=2)
	CT(68942)=(S=13358080,E=13358095,C=1)
	CT(68943)=(S=13358096,E=13358103,C=0)
	CT(68944)=(S=13358104,E=13358107,C=4)
	CT(68945)=(S=13358112,E=13358143,C=2)
	CT(68946)=(S=13358144,E=13358159,C=4)
	CT(68947)=(S=13358160,E=13358175,C=1)
	CT(68948)=(S=13358176,E=13358207,C=6)
	CT(68949)=(S=13358208,E=13358239,C=0)
	CT(68950)=(S=13358240,E=13358271,C=6)
	CT(68951)=(S=13358272,E=13358335,C=4)
	CT(68952)=(S=13358336,E=13358495,C=0)
	CT(68953)=(S=13358496,E=13358527,C=2)
	CT(68954)=(S=13358528,E=13358559,C=10)
	CT(68955)=(S=13358560,E=13358879,C=0)
	CT(68956)=(S=13358880,E=13358895,C=89)
	CT(68957)=(S=13358896,E=13358896,C=74)
	CT(68958)=(S=13358897,E=13358897,C=86)
	CT(68959)=(S=13358898,E=13358898,C=74)
	CT(68960)=(S=13358899,E=13358899,C=4)
	CT(68961)=(S=13358900,E=13358903,C=159)
	CT(68962)=(S=13358904,E=13358907,C=2)
	CT(68963)=(S=13358908,E=13358911,C=155)
	CT(68964)=(S=13358912,E=13358975,C=10)
	CT(68965)=(S=13358976,E=13359007,C=0)
	CT(68966)=(S=13359008,E=13359039,C=77)
	CT(68967)=(S=13359040,E=13359071,C=6)
	CT(68968)=(S=13359072,E=13359079,C=2)
	CT(68969)=(S=13359080,E=13359087,C=1)
	CT(68970)=(S=13359088,E=13359103,C=8)
	CT(68971)=(S=13359104,E=13359263,C=2)
	CT(68972)=(S=13359264,E=13359295,C=6)
	CT(68973)=(S=13359296,E=13359359,C=2)
	CT(68974)=(S=13359360,E=13359455,C=0)
	CT(68975)=(S=13359456,E=13359487,C=9)
	CT(68976)=(S=13359488,E=13359491,C=5)
	CT(68977)=(S=13359492,E=13359495,C=74)
	CT(68978)=(S=13359496,E=13359499,C=4)
	CT(68979)=(S=13359500,E=13359500,C=74)
	CT(68980)=(S=13359501,E=13359501,C=0)
	CT(68981)=(S=13359502,E=13359503,C=73)
	CT(68982)=(S=13359504,E=13359507,C=4)
	CT(68983)=(S=13359508,E=13359511,C=0)
	CT(68984)=(S=13359516,E=13359519,C=2)
	CT(68985)=(S=13359520,E=13359521,C=0)
	CT(68986)=(S=13359528,E=13359531,C=86)
	CT(68987)=(S=13359532,E=13359533,C=74)
	CT(68988)=(S=13359534,E=13359534,C=4)
	CT(68989)=(S=13359535,E=13359535,C=8)
	CT(68990)=(S=13359536,E=13359539,C=5)
	CT(68991)=(S=13359540,E=13359543,C=2)
	CT(68992)=(S=13359544,E=13359547,C=7)
	CT(68993)=(S=13359548,E=13359551,C=74)
	CT(68994)=(S=13359552,E=13359615,C=6)
	CT(68995)=(S=13359616,E=13359871,C=8)
	CT(68996)=(S=13359872,E=13360639,C=0)
	CT(68997)=(S=13360640,E=13360671,C=9)
	CT(68998)=(S=13360672,E=13360831,C=0)
	CT(68999)=(S=13360832,E=13360847,C=1)
	CT(69000)=(S=13360848,E=13360895,C=2)
	CT(69001)=(S=13360896,E=13360919,C=1)
	CT(69002)=(S=13360920,E=13360927,C=2)
	CT(69003)=(S=13360928,E=13360943,C=86)
	CT(69004)=(S=13360944,E=13360983,C=2)
	CT(69005)=(S=13360984,E=13360985,C=3)
	CT(69006)=(S=13360986,E=13360987,C=74)
	CT(69007)=(S=13360988,E=13360991,C=84)
	CT(69008)=(S=13360992,E=13361023,C=6)
	CT(69009)=(S=13361024,E=13361055,C=5)
	CT(69010)=(S=13361056,E=13361071,C=77)
	CT(69011)=(S=13361072,E=13361072,C=4)
	CT(69012)=(S=13361073,E=13361073,C=6)
	CT(69013)=(S=13361074,E=13361074,C=8)
	CT(69014)=(S=13361075,E=13361075,C=0)
	CT(69015)=(S=13361076,E=13361079,C=2)
	CT(69016)=(S=13361080,E=13361083,C=0)
	CT(69017)=(S=13361084,E=13361087,C=4)
	CT(69018)=(S=13361088,E=13361151,C=8)
	CT(69019)=(S=13361152,E=13369343,C=6)
	CT(69020)=(S=13369344,E=13371391,C=29)
	CT(69021)=(S=13371392,E=13371395,C=78)
	CT(69022)=(S=13371396,E=13371455,C=29)
	CT(69023)=(S=13371456,E=13371459,C=180)
	CT(69024)=(S=13371460,E=13371463,C=29)
	CT(69025)=(S=13371464,E=13371467,C=78)
	CT(69026)=(S=13371468,E=13371655,C=29)
	CT(69027)=(S=13371656,E=13371663,C=78)
	CT(69028)=(S=13371664,E=13371727,C=29)
	CT(69029)=(S=13371728,E=13371731,C=78)
	CT(69030)=(S=13371732,E=13371807,C=29)
	CT(69031)=(S=13371808,E=13371815,C=78)
	CT(69032)=(S=13371816,E=13371975,C=29)
	CT(69033)=(S=13371976,E=13371979,C=78)
	CT(69034)=(S=13371980,E=13372063,C=29)
	CT(69035)=(S=13372064,E=13372067,C=78)
	CT(69036)=(S=13372068,E=13372103,C=29)
	CT(69037)=(S=13372104,E=13372111,C=78)
	CT(69038)=(S=13372112,E=13372143,C=29)
	CT(69039)=(S=13372144,E=13372147,C=78)
	CT(69040)=(S=13372148,E=13372207,C=29)
	CT(69041)=(S=13372208,E=13372215,C=78)
	CT(69042)=(S=13372216,E=13372219,C=95)
	CT(69043)=(S=13372220,E=13372279,C=29)
	CT(69044)=(S=13372280,E=13372283,C=78)
	CT(69045)=(S=13372284,E=13372311,C=29)
	CT(69046)=(S=13372312,E=13372319,C=180)
	CT(69047)=(S=13372320,E=13372543,C=29)
	CT(69048)=(S=13372544,E=13372559,C=105)
	CT(69049)=(S=13372560,E=13372575,C=78)
	CT(69050)=(S=13372576,E=13372719,C=29)
	CT(69051)=(S=13372720,E=13372723,C=78)
	CT(69052)=(S=13372724,E=13372775,C=29)
	CT(69053)=(S=13372776,E=13372779,C=179)
	CT(69054)=(S=13372780,E=13372923,C=29)
	CT(69055)=(S=13372924,E=13372927,C=78)
	CT(69056)=(S=13372928,E=13372943,C=29)
	CT(69057)=(S=13372944,E=13372947,C=78)
	CT(69058)=(S=13372948,E=13373175,C=29)
	CT(69059)=(S=13373176,E=13373183,C=197)
	CT(69060)=(S=13373184,E=13373215,C=29)
	CT(69061)=(S=13373216,E=13373223,C=78)
	CT(69062)=(S=13373224,E=13373327,C=29)
	CT(69063)=(S=13373328,E=13373335,C=80)
	CT(69064)=(S=13373336,E=13373355,C=29)
	CT(69065)=(S=13373356,E=13373359,C=78)
	CT(69066)=(S=13373360,E=13373367,C=29)
	CT(69067)=(S=13373368,E=13373383,C=78)
	CT(69068)=(S=13373384,E=13373447,C=29)
	CT(69069)=(S=13373448,E=13373451,C=230)
	CT(69070)=(S=13373452,E=13373535,C=29)
	CT(69071)=(S=13373540,E=13373543,C=78)
	CT(69072)=(S=13373544,E=13373551,C=29)
	CT(69073)=(S=13373552,E=13373555,C=183)
	CT(69074)=(S=13373556,E=13373583,C=29)
	CT(69075)=(S=13373584,E=13373591,C=78)
	CT(69076)=(S=13373592,E=13373639,C=29)
	CT(69077)=(S=13373640,E=13373647,C=78)
	CT(69078)=(S=13373648,E=13373849,C=29)
	CT(69079)=(S=13373850,E=13373850,C=78)
	CT(69080)=(S=13373851,E=13374208,C=29)
	CT(69081)=(S=13374209,E=13374222,C=78)
	CT(69082)=(S=13374223,E=13374223,C=29)
	CT(69083)=(S=13374224,E=13374326,C=78)
	CT(69084)=(S=13374327,E=13374327,C=29)
	CT(69085)=(S=13374328,E=13374336,C=78)
	CT(69086)=(S=13374337,E=13374337,C=29)
	CT(69087)=(S=13374338,E=13374369,C=78)
	CT(69088)=(S=13374370,E=13374370,C=29)
	CT(69089)=(S=13374371,E=13374407,C=78)
	CT(69090)=(S=13374408,E=13374411,C=230)
	CT(69091)=(S=13374412,E=13374413,C=29)
	CT(69092)=(S=13374414,E=13374415,C=78)
	CT(69093)=(S=13374416,E=13374416,C=29)
	CT(69094)=(S=13374417,E=13374430,C=78)
	CT(69095)=(S=13374431,E=13374431,C=29)
	CT(69096)=(S=13374432,E=13374445,C=78)
	CT(69097)=(S=13374446,E=13374447,C=29)
	CT(69098)=(S=13374448,E=13374448,C=78)
	CT(69099)=(S=13374449,E=13374450,C=29)
	CT(69100)=(S=13374451,E=13374451,C=78)
	CT(69101)=(S=13374452,E=13374452,C=29)
	CT(69102)=(S=13374453,E=13374462,C=78)
	CT(69103)=(S=13374463,E=13376307,C=29)
	CT(69104)=(S=13376308,E=13376311,C=78)
	CT(69105)=(S=13376312,E=13376434,C=29)
	CT(69106)=(S=13376435,E=13376435,C=14)
	CT(69107)=(S=13376436,E=13376479,C=29)
	CT(69108)=(S=13376480,E=13376480,C=80)
	CT(69109)=(S=13376481,E=13376610,C=29)
	CT(69110)=(S=13376611,E=13376611,C=78)
	CT(69111)=(S=13376612,E=13376928,C=29)
	CT(69112)=(S=13376929,E=13376929,C=78)
	CT(69113)=(S=13376930,E=13376980,C=29)
	CT(69114)=(S=13376981,E=13376981,C=78)
	CT(69115)=(S=13376982,E=13379583,C=29)
	CT(69116)=(S=13379584,E=13380095,C=78)
	CT(69117)=(S=13380096,E=13380747,C=29)
	CT(69118)=(S=13380748,E=13380751,C=78)
	CT(69119)=(S=13380752,E=13381631,C=29)
	CT(69120)=(S=13381632,E=13381639,C=78)
	CT(69121)=(S=13381640,E=13381695,C=29)
	CT(69122)=(S=13381696,E=13381727,C=78)
	CT(69123)=(S=13381728,E=13382143,C=29)
	CT(69124)=(S=13382144,E=13382399,C=78)
	CT(69125)=(S=13382400,E=13382863,C=29)
	CT(69126)=(S=13382864,E=13382864,C=17)
	CT(69127)=(S=13382865,E=13384253,C=29)
	CT(69128)=(S=13384254,E=13384255,C=78)
	CT(69129)=(S=13384256,E=13384327,C=29)
	CT(69130)=(S=13384328,E=13384331,C=204)
	CT(69131)=(S=13384332,E=13385225,C=29)
	CT(69132)=(S=13385226,E=13385227,C=78)
	CT(69133)=(S=13385228,E=13385661,C=29)
	CT(69134)=(S=13385662,E=13385663,C=78)
	CT(69135)=(S=13385664,E=13385719,C=29)
	CT(69136)=(S=13385720,E=13385727,C=78)
	CT(69137)=(S=13385728,E=13387175,C=29)
	CT(69138)=(S=13387176,E=13387176,C=78)
	CT(69139)=(S=13387177,E=13387215,C=29)
	CT(69140)=(S=13387216,E=13387216,C=78)
	CT(69141)=(S=13387217,E=13388685,C=29)
	CT(69142)=(S=13388686,E=13388686,C=0)
	CT(69143)=(S=13388687,E=13389261,C=29)
	CT(69144)=(S=13389262,E=13389262,C=78)
	CT(69145)=(S=13389263,E=13389695,C=29)
	CT(69146)=(S=13389696,E=13389696,C=28)
	CT(69147)=(S=13389697,E=13389697,C=78)
	CT(69148)=(S=13389698,E=13389700,C=15)
	CT(69149)=(S=13389701,E=13389701,C=0)
	CT(69150)=(S=13389702,E=13389702,C=14)
	CT(69151)=(S=13389703,E=13389703,C=29)
	CT(69152)=(S=13389704,E=13389704,C=12)
	CT(69153)=(S=13389705,E=13389708,C=29)
	CT(69154)=(S=13389709,E=13389709,C=14)
	CT(69155)=(S=13389710,E=13389711,C=29)
	CT(69156)=(S=13389712,E=13389713,C=14)
	CT(69157)=(S=13389714,E=13389714,C=19)
	CT(69158)=(S=13389715,E=13389717,C=28)
	CT(69159)=(S=13389718,E=13389724,C=29)
	CT(69160)=(S=13389725,E=13389726,C=2)
	CT(69161)=(S=13389727,E=13389727,C=27)
	CT(69162)=(S=13389728,E=13389728,C=29)
	CT(69163)=(S=13389729,E=13389731,C=34)
	CT(69164)=(S=13389732,E=13389732,C=29)
	CT(69165)=(S=13389733,E=13389733,C=78)
	CT(69166)=(S=13389734,E=13389735,C=34)
	CT(69167)=(S=13389736,E=13389736,C=28)
	CT(69168)=(S=13389737,E=13389737,C=8)
	CT(69169)=(S=13389738,E=13389738,C=49)
	CT(69170)=(S=13389739,E=13389739,C=28)
	CT(69171)=(S=13389740,E=13389740,C=29)
	CT(69172)=(S=13389741,E=13389741,C=12)
	CT(69173)=(S=13389742,E=13389744,C=29)
	CT(69174)=(S=13389745,E=13389746,C=28)
	CT(69175)=(S=13389747,E=13389751,C=29)
	CT(69176)=(S=13389752,E=13389752,C=33)
	CT(69177)=(S=13389753,E=13389754,C=29)
	CT(69178)=(S=13389755,E=13389755,C=28)
	CT(69179)=(S=13389756,E=13389756,C=29)
	CT(69180)=(S=13389757,E=13389757,C=8)
	CT(69181)=(S=13389758,E=13389758,C=29)
	CT(69182)=(S=13389759,E=13389759,C=19)
	CT(69183)=(S=13389760,E=13389761,C=27)
	CT(69184)=(S=13389762,E=13389762,C=14)
	CT(69185)=(S=13389763,E=13389765,C=29)
	CT(69186)=(S=13389766,E=13389766,C=14)
	CT(69187)=(S=13389767,E=13389768,C=28)
	CT(69188)=(S=13389769,E=13389771,C=29)
	CT(69189)=(S=13389772,E=13389772,C=49)
	CT(69190)=(S=13389773,E=13389774,C=29)
	CT(69191)=(S=13389775,E=13389775,C=14)
	CT(69192)=(S=13389776,E=13389778,C=12)
	CT(69193)=(S=13389779,E=13389779,C=14)
	CT(69194)=(S=13389780,E=13389780,C=29)
	CT(69195)=(S=13389781,E=13389783,C=12)
	CT(69196)=(S=13389784,E=13389785,C=29)
	CT(69197)=(S=13389786,E=13389786,C=2)
	CT(69198)=(S=13389787,E=13389787,C=27)
	CT(69199)=(S=13389788,E=13389791,C=29)
	CT(69200)=(S=13389792,E=13389793,C=70)
	CT(69201)=(S=13389794,E=13389795,C=29)
	CT(69202)=(S=13389796,E=13389796,C=12)
	CT(69203)=(S=13389797,E=13389797,C=34)
	CT(69204)=(S=13389798,E=13389799,C=14)
	CT(69205)=(S=13389800,E=13389802,C=27)
	CT(69206)=(S=13389803,E=13389804,C=28)
	CT(69207)=(S=13389805,E=13389807,C=29)
	CT(69208)=(S=13389808,E=13389808,C=15)
	CT(69209)=(S=13389809,E=13389810,C=28)
	CT(69210)=(S=13389811,E=13389811,C=29)
	CT(69211)=(S=13389812,E=13389813,C=78)
	CT(69212)=(S=13389814,E=13389819,C=36)
	CT(69213)=(S=13389820,E=13389820,C=29)
	CT(69214)=(S=13389821,E=13389823,C=36)
	CT(69215)=(S=13389824,E=13389831,C=78)
	CT(69216)=(S=13389832,E=13389966,C=29)
	CT(69217)=(S=13389967,E=13389967,C=204)
	CT(69218)=(S=13389968,E=13390079,C=29)
	CT(69219)=(S=13390080,E=13390847,C=78)
	CT(69220)=(S=13390848,E=13391744,C=29)
	CT(69221)=(S=13391745,E=13391745,C=78)
	CT(69222)=(S=13391746,E=13391784,C=29)
	CT(69223)=(S=13391785,E=13391785,C=203)
	CT(69224)=(S=13391786,E=13391794,C=29)
	CT(69225)=(S=13391795,E=13391795,C=105)
	CT(69226)=(S=13391796,E=13391804,C=29)
	CT(69227)=(S=13391805,E=13391805,C=78)
	CT(69228)=(S=13391806,E=13391820,C=29)
	CT(69229)=(S=13391821,E=13391821,C=212)
	CT(69230)=(S=13391822,E=13391870,C=29)
	CT(69231)=(S=13391871,E=13391871,C=78)
	CT(69232)=(S=13391872,E=13392184,C=29)
	CT(69233)=(S=13392185,E=13392185,C=78)
	CT(69234)=(S=13392186,E=13392323,C=29)
	CT(69235)=(S=13392324,E=13392324,C=78)
	CT(69236)=(S=13392325,E=13392333,C=29)
	CT(69237)=(S=13392334,E=13392334,C=204)
	CT(69238)=(S=13392335,E=13395199,C=29)
	CT(69239)=(S=13395200,E=13395455,C=78)
	CT(69240)=(S=13395456,E=13396487,C=29)
	CT(69241)=(S=13396488,E=13396495,C=78)
	CT(69242)=(S=13396497,E=13397559,C=29)
	CT(69243)=(S=13397560,E=13397567,C=179)
	CT(69244)=(S=13397568,E=13398015,C=29)
	CT(69245)=(S=13398016,E=13398271,C=78)
	CT(69246)=(S=13398272,E=13401186,C=29)
	CT(69247)=(S=13401187,E=13401187,C=80)
	CT(69248)=(S=13401188,E=13401191,C=29)
	CT(69249)=(S=13401192,E=13401195,C=204)
	CT(69250)=(S=13401196,E=13401258,C=29)
	CT(69251)=(S=13401259,E=13401259,C=78)
	CT(69252)=(S=13401260,E=13401611,C=29)
	CT(69253)=(S=13401612,E=13401613,C=204)
	CT(69254)=(S=13401614,E=13401727,C=29)
	CT(69255)=(S=13401728,E=13401729,C=177)
	CT(69256)=(S=13401730,E=13401739,C=29)
	CT(69257)=(S=13401740,E=13401741,C=204)
	CT(69258)=(S=13401742,E=13401755,C=29)
	CT(69259)=(S=13401756,E=13401757,C=78)
	CT(69260)=(S=13401758,E=13402790,C=29)
	CT(69261)=(S=13402791,E=13402791,C=78)
	CT(69262)=(S=13402792,E=13404171,C=29)
	CT(69263)=(S=13404172,E=13404175,C=78)
	CT(69264)=(S=13404176,E=13404672,C=29)
	CT(69265)=(S=13404673,E=13404697,C=78)
	CT(69266)=(S=13404698,E=13404698,C=29)
	CT(69267)=(S=13404699,E=13404739,C=78)
	CT(69268)=(S=13404740,E=13404740,C=29)
	CT(69269)=(S=13404741,E=13404765,C=78)
	CT(69270)=(S=13404766,E=13404766,C=29)
	CT(69271)=(S=13404767,E=13404770,C=78)
	CT(69272)=(S=13404771,E=13404774,C=29)
	CT(69273)=(S=13404775,E=13404775,C=78)
	CT(69274)=(S=13404776,E=13404776,C=29)
	CT(69275)=(S=13404777,E=13404783,C=78)
	CT(69276)=(S=13404784,E=13404784,C=29)
	CT(69277)=(S=13404785,E=13404838,C=78)
	CT(69278)=(S=13404839,E=13404839,C=29)
	CT(69279)=(S=13404840,E=13404850,C=78)
	CT(69280)=(S=13404851,E=13404853,C=29)
	CT(69281)=(S=13404854,E=13404911,C=78)
	CT(69282)=(S=13404912,E=13404915,C=29)
	CT(69283)=(S=13404917,E=13404926,C=78)
	CT(69284)=(S=13404927,E=13406267,C=29)
	CT(69285)=(S=13406268,E=13406271,C=78)
	CT(69286)=(S=13406272,E=13406285,C=29)
	CT(69287)=(S=13406286,E=13406286,C=80)
	CT(69288)=(S=13406287,E=13406664,C=29)
	CT(69289)=(S=13406665,E=13406665,C=204)
	CT(69290)=(S=13406666,E=13406692,C=29)
	CT(69291)=(S=13406693,E=13406693,C=78)
	CT(69292)=(S=13406694,E=13407051,C=29)
	CT(69293)=(S=13407052,E=13407055,C=78)
	CT(69294)=(S=13407056,E=13408269,C=29)
	CT(69295)=(S=13408270,E=13408271,C=117)
	CT(69296)=(S=13408272,E=13408479,C=29)
	CT(69297)=(S=13408480,E=13408487,C=8)
	CT(69298)=(S=13408488,E=13408535,C=29)
	CT(69299)=(S=13408536,E=13408537,C=204)
	CT(69300)=(S=13408538,E=13408646,C=29)
	CT(69301)=(S=13408647,E=13408647,C=78)
	CT(69302)=(S=13408648,E=13408726,C=29)
	CT(69303)=(S=13408727,E=13408727,C=78)
	CT(69304)=(S=13408728,E=13408831,C=29)
	CT(69305)=(S=13408832,E=13408839,C=28)
	CT(69306)=(S=13408840,E=13408941,C=29)
	CT(69307)=(S=13408942,E=13408943,C=78)
	CT(69308)=(S=13408944,E=13408967,C=29)
	CT(69309)=(S=13408968,E=13408975,C=78)
	CT(69310)=(S=13408976,E=13409403,C=29)
	CT(69311)=(S=13409404,E=13409407,C=78)
	CT(69312)=(S=13409408,E=13413888,C=29)
	CT(69313)=(S=13413889,E=13413924,C=78)
	CT(69314)=(S=13413925,E=13413925,C=29)
	CT(69315)=(S=13413926,E=13413934,C=78)
	CT(69316)=(S=13413935,E=13413935,C=29)
	CT(69317)=(S=13413936,E=13413991,C=78)
	CT(69318)=(S=13413992,E=13413999,C=29)
	CT(69319)=(S=13414000,E=13414111,C=78)
	CT(69320)=(S=13414112,E=13414112,C=29)
	CT(69321)=(S=13414113,E=13414119,C=78)
	CT(69322)=(S=13414120,E=13414121,C=29)
	CT(69323)=(S=13414122,E=13414142,C=78)
	CT(69324)=(S=13414143,E=13417216,C=29)
	CT(69325)=(S=13417217,E=13417309,C=78)
	CT(69326)=(S=13417310,E=13417310,C=29)
	CT(69327)=(S=13417311,E=13417470,C=78)
	CT(69328)=(S=13417471,E=13418239,C=29)
	CT(69329)=(S=13418240,E=13418495,C=78)
	CT(69330)=(S=13418496,E=13419028,C=29)
	CT(69331)=(S=13419029,E=13419029,C=78)
	CT(69332)=(S=13419030,E=13419951,C=29)
	CT(69333)=(S=13419952,E=13419967,C=78)
	CT(69334)=(S=13419968,E=13422848,C=29)
	CT(69335)=(S=13422849,E=13422873,C=78)
	CT(69336)=(S=13422874,E=13422875,C=29)
	CT(69337)=(S=13422876,E=13422961,C=78)
	CT(69338)=(S=13422962,E=13422962,C=29)
	CT(69339)=(S=13422963,E=13422977,C=78)
	CT(69340)=(S=13422978,E=13422978,C=29)
	CT(69341)=(S=13422979,E=13423006,C=78)
	CT(69342)=(S=13423007,E=13423007,C=29)
	CT(69343)=(S=13423008,E=13423102,C=78)
	CT(69344)=(S=13423104,E=13426944,C=29)
	CT(69345)=(S=13426945,E=13426959,C=78)
	CT(69346)=(S=13426960,E=13426975,C=29)
	CT(69347)=(S=13426976,E=13426981,C=78)
	CT(69348)=(S=13426982,E=13426983,C=29)
	CT(69349)=(S=13426984,E=13426995,C=78)
	CT(69350)=(S=13426996,E=13426997,C=29)
	CT(69351)=(S=13426998,E=13427076,C=78)
	CT(69352)=(S=13427077,E=13427077,C=29)
	CT(69353)=(S=13427078,E=13427079,C=78)
	CT(69354)=(S=13427080,E=13427080,C=29)
	CT(69355)=(S=13427081,E=13427085,C=78)
	CT(69356)=(S=13427086,E=13427086,C=29)
	CT(69357)=(S=13427087,E=13427088,C=78)
	CT(69358)=(S=13427089,E=13427091,C=29)
	CT(69359)=(S=13427092,E=13427093,C=78)
	CT(69360)=(S=13427094,E=13427094,C=29)
	CT(69361)=(S=13427095,E=13427115,C=78)
	CT(69362)=(S=13427116,E=13427116,C=29)
	CT(69363)=(S=13427118,E=13427121,C=78)
	CT(69364)=(S=13427122,E=13427122,C=29)
	CT(69365)=(S=13427123,E=13427151,C=78)
	CT(69366)=(S=13427152,E=13427153,C=29)
	CT(69367)=(S=13427154,E=13427182,C=78)
	CT(69368)=(S=13427183,E=13427183,C=29)
	CT(69369)=(S=13427184,E=13427455,C=78)
	CT(69370)=(S=13427456,E=13428703,C=29)
	CT(69371)=(S=13428704,E=13428704,C=14)
	CT(69372)=(S=13428705,E=13428706,C=36)
	CT(69373)=(S=13428707,E=13428707,C=29)
	CT(69374)=(S=13428708,E=13428708,C=28)
	CT(69375)=(S=13428709,E=13428709,C=29)
	CT(69376)=(S=13428710,E=13428710,C=2)
	CT(69377)=(S=13428711,E=13428711,C=49)
	CT(69378)=(S=13428712,E=13428712,C=14)
	CT(69379)=(S=13428713,E=13428716,C=29)
	CT(69380)=(S=13428717,E=13428717,C=15)
	CT(69381)=(S=13428718,E=13428718,C=14)
	CT(69382)=(S=13428719,E=13428719,C=25)
	CT(69383)=(S=13428720,E=13428720,C=63)
	CT(69384)=(S=13428721,E=13428730,C=29)
	CT(69385)=(S=13428731,E=13428732,C=2)
	CT(69386)=(S=13428733,E=13428733,C=14)
	CT(69387)=(S=13428734,E=13428734,C=29)
	CT(69388)=(S=13428735,E=13428735,C=63)
	CT(69389)=(S=13428736,E=13428799,C=29)
	CT(69390)=(S=13428800,E=13428815,C=78)
	CT(69391)=(S=13428816,E=13429535,C=29)
	CT(69392)=(S=13429536,E=13429547,C=105)
	CT(69393)=(S=13429548,E=13429711,C=29)
	CT(69394)=(S=13429712,E=13429727,C=78)
	CT(69395)=(S=13429728,E=13429823,C=29)
	CT(69396)=(S=13429824,E=13429887,C=81)
	CT(69397)=(S=13429888,E=13430015,C=29)
	CT(69398)=(S=13430016,E=13430143,C=78)
	CT(69399)=(S=13430144,E=13430528,C=29)
	CT(69400)=(S=13430529,E=13430530,C=78)
	CT(69401)=(S=13430531,E=13430531,C=29)
	CT(69402)=(S=13430532,E=13430538,C=78)
	CT(69403)=(S=13430539,E=13430539,C=29)
	CT(69404)=(S=13430540,E=13430541,C=78)
	CT(69405)=(S=13430542,E=13430543,C=29)
	CT(69406)=(S=13430544,E=13430544,C=78)
	CT(69407)=(S=13430545,E=13430545,C=29)
	CT(69408)=(S=13430546,E=13430651,C=78)
	CT(69409)=(S=13430652,E=13430652,C=29)
	CT(69410)=(S=13430653,E=13430673,C=78)
	CT(69411)=(S=13430674,E=13430674,C=29)
	CT(69412)=(S=13430675,E=13430709,C=78)
	CT(69413)=(S=13430710,E=13430710,C=29)
	CT(69414)=(S=13430711,E=13430782,C=78)
	CT(69415)=(S=13430784,E=13431807,C=29)
	CT(69416)=(S=13431808,E=13432063,C=78)
	CT(69417)=(S=13432064,E=13472607,C=29)
	CT(69418)=(S=13472608,E=13472615,C=4)
	CT(69419)=(S=13472616,E=13472619,C=8)
	CT(69420)=(S=13472620,E=13472623,C=4)
	CT(69421)=(S=13472624,E=13473536,C=29)
	CT(69422)=(S=13473537,E=13473648,C=78)
	CT(69423)=(S=13473649,E=13473651,C=29)
	CT(69424)=(S=13473652,E=13473724,C=78)
	CT(69425)=(S=13473725,E=13473725,C=29)
	CT(69426)=(S=13473726,E=13473745,C=78)
	CT(69427)=(S=13473746,E=13473747,C=29)
	CT(69428)=(S=13473748,E=13473790,C=78)
	CT(69429)=(S=13473791,E=13475662,C=29)
	CT(69430)=(S=13475663,E=13475663,C=105)
	CT(69431)=(S=13475664,E=13477410,C=29)
	CT(69432)=(S=13477411,E=13477411,C=226)
	CT(69433)=(S=13477412,E=13477552,C=29)
	CT(69434)=(S=13477553,E=13477553,C=0)
	CT(69435)=(S=13477554,E=13477554,C=29)
	CT(69436)=(S=13477555,E=13477555,C=78)
	CT(69437)=(S=13477556,E=13477626,C=29)
	CT(69438)=(S=13477627,E=13477627,C=204)
	CT(69439)=(S=13477628,E=13478954,C=29)
	CT(69440)=(S=13478955,E=13478955,C=78)
	CT(69441)=(S=13478956,E=13479031,C=29)
	CT(69442)=(S=13479032,E=13479039,C=78)
	CT(69443)=(S=13479040,E=13479238,C=29)
	CT(69444)=(S=13479239,E=13479239,C=78)
	CT(69445)=(S=13479240,E=13479292,C=29)
	CT(69446)=(S=13479293,E=13479293,C=78)
	CT(69447)=(S=13479294,E=13479419,C=29)
	CT(69448)=(S=13479420,E=13479423,C=78)
	CT(69449)=(S=13479424,E=13479583,C=29)
	CT(69450)=(S=13479584,E=13479599,C=78)
	CT(69451)=(S=13479600,E=13479891,C=29)
	CT(69452)=(S=13479892,E=13479895,C=78)
	CT(69453)=(S=13479896,E=13483264,C=29)
	CT(69454)=(S=13483265,E=13483298,C=78)
	CT(69455)=(S=13483299,E=13483302,C=29)
	CT(69456)=(S=13483303,E=13483307,C=78)
	CT(69457)=(S=13483308,E=13483309,C=29)
	CT(69458)=(S=13483310,E=13483348,C=78)
	CT(69459)=(S=13483349,E=13483349,C=29)
	CT(69460)=(S=13483350,E=13483362,C=78)
	CT(69461)=(S=13483363,E=13483363,C=29)
	CT(69462)=(S=13483364,E=13483365,C=78)
	CT(69463)=(S=13483366,E=13483367,C=29)
	CT(69464)=(S=13483368,E=13483375,C=78)
	CT(69465)=(S=13483376,E=13483380,C=29)
	CT(69466)=(S=13483381,E=13483397,C=78)
	CT(69467)=(S=13483398,E=13483398,C=29)
	CT(69468)=(S=13483399,E=13483463,C=78)
	CT(69469)=(S=13483464,E=13483465,C=29)
	CT(69470)=(S=13483466,E=13483466,C=78)
	CT(69471)=(S=13483467,E=13483467,C=29)
	CT(69472)=(S=13483468,E=13483518,C=78)
	CT(69473)=(S=13483519,E=13484031,C=29)
	CT(69474)=(S=13484032,E=13485055,C=78)
	CT(69475)=(S=13485056,E=13486079,C=29)
	CT(69476)=(S=13486080,E=13486335,C=78)
	CT(69477)=(S=13486336,E=13487167,C=29)
	CT(69478)=(S=13487168,E=13487199,C=78)
	CT(69479)=(S=13487200,E=13487615,C=29)
	CT(69480)=(S=13487616,E=13487871,C=78)
	CT(69481)=(S=13487872,E=13487872,C=29)
	CT(69482)=(S=13487873,E=13487899,C=78)
	CT(69483)=(S=13487900,E=13487900,C=29)
	CT(69484)=(S=13487901,E=13487945,C=78)
	CT(69485)=(S=13487946,E=13487946,C=29)
	CT(69486)=(S=13487947,E=13487966,C=78)
	CT(69487)=(S=13487967,E=13487967,C=29)
	CT(69488)=(S=13487968,E=13487971,C=78)
	CT(69489)=(S=13487972,E=13487981,C=29)
	CT(69490)=(S=13487982,E=13488012,C=78)
	CT(69491)=(S=13488013,E=13488013,C=29)
	CT(69492)=(S=13488014,E=13488019,C=78)
	CT(69493)=(S=13488020,E=13488021,C=29)
	CT(69494)=(S=13488022,E=13488026,C=78)
	CT(69495)=(S=13488027,E=13488031,C=29)
	CT(69496)=(S=13488032,E=13488041,C=78)
	CT(69497)=(S=13488042,E=13488043,C=29)
	CT(69498)=(S=13488044,E=13488057,C=78)
	CT(69499)=(S=13488058,E=13488058,C=29)
	CT(69500)=(S=13488059,E=13488097,C=78)
	CT(69501)=(S=13488098,E=13488098,C=29)
	CT(69502)=(S=13488099,E=13488126,C=78)
	CT(69503)=(S=13488127,E=13488384,C=29)
	CT(69504)=(S=13488385,E=13488390,C=78)
	CT(69505)=(S=13488391,E=13488391,C=29)
	CT(69506)=(S=13488392,E=13488392,C=78)
	CT(69507)=(S=13488393,E=13488393,C=29)
	CT(69508)=(S=13488394,E=13488395,C=78)
	CT(69509)=(S=13488396,E=13488640,C=29)
	CT(69510)=(S=13488641,E=13488659,C=78)
	CT(69511)=(S=13488660,E=13488663,C=29)
	CT(69512)=(S=13488664,E=13488675,C=78)
	CT(69513)=(S=13488676,E=13488676,C=29)
	CT(69514)=(S=13488677,E=13488703,C=78)
	CT(69515)=(S=13488704,E=13488709,C=29)
	CT(69516)=(S=13488710,E=13488764,C=78)
	CT(69517)=(S=13488765,E=13488765,C=29)
	CT(69518)=(S=13488766,E=13488784,C=78)
	CT(69519)=(S=13488785,E=13488785,C=29)
	CT(69520)=(S=13488786,E=13488788,C=78)
	CT(69521)=(S=13488789,E=13488789,C=29)
	CT(69522)=(S=13488790,E=13488825,C=78)
	CT(69523)=(S=13488826,E=13488831,C=29)
	CT(69524)=(S=13488832,E=13488863,C=78)
	CT(69525)=(S=13488864,E=13488864,C=29)
	CT(69526)=(S=13488865,E=13488873,C=78)
	CT(69527)=(S=13488874,E=13488874,C=29)
	CT(69528)=(S=13488875,E=13488887,C=78)
	CT(69529)=(S=13488888,E=13488888,C=29)
	CT(69530)=(S=13488889,E=13488894,C=78)
	CT(69531)=(S=13488895,E=13488896,C=29)
	CT(69532)=(S=13488897,E=13488903,C=78)
	CT(69533)=(S=13488904,E=13488905,C=178)
	CT(69534)=(S=13488906,E=13488919,C=78)
	CT(69535)=(S=13488920,E=13488921,C=29)
	CT(69536)=(S=13488922,E=13488986,C=78)
	CT(69537)=(S=13488987,E=13488987,C=29)
	CT(69538)=(S=13488988,E=13489043,C=78)
	CT(69539)=(S=13489044,E=13489044,C=29)
	CT(69540)=(S=13489045,E=13489087,C=78)
	CT(69541)=(S=13489088,E=13489855,C=29)
	CT(69542)=(S=13489856,E=13489887,C=79)
	CT(69543)=(S=13489888,E=13490655,C=29)
	CT(69544)=(S=13490656,E=13490687,C=183)
	CT(69545)=(S=13490688,E=13494528,C=29)
	CT(69546)=(S=13494529,E=13494535,C=78)
	CT(69547)=(S=13494536,E=13494542,C=29)
	CT(69548)=(S=13494543,E=13494544,C=78)
	CT(69549)=(S=13494545,E=13494546,C=29)
	CT(69550)=(S=13494547,E=13494549,C=78)
	CT(69551)=(S=13494550,E=13494559,C=29)
	CT(69552)=(S=13494560,E=13494562,C=78)
	CT(69553)=(S=13494563,E=13494563,C=29)
	CT(69554)=(S=13494564,E=13494566,C=78)
	CT(69555)=(S=13494567,E=13494575,C=29)
	CT(69556)=(S=13494576,E=13494598,C=78)
	CT(69557)=(S=13494599,E=13494599,C=29)
	CT(69558)=(S=13494600,E=13494600,C=78)
	CT(69559)=(S=13494601,E=13494601,C=29)
	CT(69560)=(S=13494602,E=13494602,C=78)
	CT(69561)=(S=13494603,E=13494603,C=29)
	CT(69562)=(S=13494604,E=13494607,C=177)
	CT(69563)=(S=13494608,E=13494653,C=78)
	CT(69564)=(S=13494654,E=13494655,C=29)
	CT(69565)=(S=13494656,E=13494666,C=78)
	CT(69566)=(S=13494667,E=13494671,C=29)
	CT(69567)=(S=13494672,E=13494674,C=78)
	CT(69568)=(S=13494675,E=13494675,C=29)
	CT(69569)=(S=13494676,E=13494734,C=78)
	CT(69570)=(S=13494735,E=13494735,C=29)
	CT(69571)=(S=13494736,E=13494771,C=78)
	CT(69572)=(S=13494772,E=13494775,C=29)
	CT(69573)=(S=13494776,E=13494782,C=78)
	CT(69574)=(S=13494783,E=13494815,C=29)
	CT(69575)=(S=13494816,E=13494847,C=78)
	CT(69576)=(S=13494848,E=13495296,C=29)
	CT(69577)=(S=13495297,E=13495346,C=78)
	CT(69578)=(S=13495347,E=13495347,C=29)
	CT(69579)=(S=13495348,E=13495351,C=78)
	CT(69580)=(S=13495352,E=13495352,C=29)
	CT(69581)=(S=13495353,E=13495370,C=78)
	CT(69582)=(S=13495371,E=13495371,C=29)
	CT(69583)=(S=13495372,E=13495373,C=78)
	CT(69584)=(S=13495374,E=13495375,C=29)
	CT(69585)=(S=13495376,E=13495376,C=78)
	CT(69586)=(S=13495377,E=13495377,C=29)
	CT(69587)=(S=13495378,E=13495422,C=78)
	CT(69588)=(S=13495423,E=13495423,C=29)
	CT(69589)=(S=13495424,E=13495550,C=78)
	CT(69590)=(S=13495551,E=13495552,C=29)
	CT(69591)=(S=13495553,E=13495576,C=78)
	CT(69592)=(S=13495577,E=13495577,C=29)
	CT(69593)=(S=13495578,E=13495630,C=78)
	CT(69594)=(S=13495631,E=13495631,C=29)
	CT(69595)=(S=13495632,E=13495639,C=78)
	CT(69596)=(S=13495640,E=13495761,C=29)
	CT(69597)=(S=13495762,E=13495806,C=78)
	CT(69598)=(S=13495807,E=13498879,C=29)
	CT(69599)=(S=13498880,E=13499135,C=78)
	CT(69600)=(S=13499136,E=13501183,C=29)
	CT(69601)=(S=13501184,E=13501215,C=2)
	CT(69602)=(S=13501216,E=13503487,C=29)
	CT(69603)=(S=13503488,E=13503743,C=78)
	CT(69604)=(S=13503744,E=13510991,C=29)
	CT(69605)=(S=13510992,E=13511007,C=78)
	CT(69606)=(S=13511008,E=13511015,C=29)
	CT(69607)=(S=13511016,E=13511016,C=78)
	CT(69608)=(S=13511017,E=13511018,C=29)
	CT(69609)=(S=13511019,E=13511019,C=175)
	CT(69610)=(S=13511020,E=13511022,C=29)
	CT(69611)=(S=13511023,E=13511023,C=78)
	CT(69612)=(S=13511024,E=13511935,C=29)
	CT(69613)=(S=13511936,E=13512191,C=78)
	CT(69614)=(S=13512192,E=13512447,C=29)
	CT(69615)=(S=13512448,E=13512703,C=78)
	CT(69616)=(S=13512704,E=13513471,C=29)
	CT(69617)=(S=13513472,E=13513472,C=80)
	CT(69618)=(S=13513473,E=13513478,C=78)
	CT(69619)=(S=13513479,E=13513479,C=29)
	CT(69620)=(S=13513480,E=13513501,C=78)
	CT(69621)=(S=13513502,E=13513502,C=29)
	CT(69622)=(S=13513503,E=13513504,C=78)
	CT(69623)=(S=13513505,E=13513505,C=29)
	CT(69624)=(S=13513506,E=13513506,C=78)
	CT(69625)=(S=13513507,E=13513719,C=29)
	CT(69626)=(S=13513720,E=13513725,C=78)
	CT(69627)=(S=13513726,E=13514031,C=29)
	CT(69628)=(S=13514032,E=13514047,C=78)
	CT(69629)=(S=13514048,E=13514123,C=29)
	CT(69630)=(S=13514124,E=13514124,C=78)
	CT(69631)=(S=13514125,E=13514125,C=182)
	CT(69632)=(S=13514127,E=13514127,C=29)
	CT(69633)=(S=13514128,E=13514143,C=78)
	CT(69634)=(S=13514144,E=13514151,C=29)
	CT(69635)=(S=13514160,E=13514175,C=178)
	CT(69636)=(S=13514176,E=13514575,C=29)
	CT(69637)=(S=13514576,E=13514591,C=78)
	CT(69638)=(S=13514592,E=13519055,C=29)
	CT(69639)=(S=13519056,E=13519056,C=78)
	CT(69640)=(S=13519057,E=13519615,C=29)
	CT(69641)=(S=13519616,E=13519871,C=78)
	CT(69642)=(S=13519872,E=13521127,C=29)
	CT(69643)=(S=13521136,E=13521151,C=78)
	CT(69644)=(S=13521152,E=13522687,C=29)
	CT(69645)=(S=13522688,E=13522943,C=78)
	CT(69646)=(S=13522944,E=13528063,C=29)
	CT(69647)=(S=13528064,E=13528095,C=78)
	CT(69648)=(S=13528096,E=13528096,C=29)
	CT(69649)=(S=13528097,E=13528103,C=78)
	CT(69650)=(S=13528104,E=13528119,C=29)
	CT(69651)=(S=13528120,E=13528176,C=78)
	CT(69652)=(S=13528177,E=13528183,C=29)
	CT(69653)=(S=13528184,E=13528295,C=78)
	CT(69654)=(S=13528296,E=13528300,C=29)
	CT(69655)=(S=13528301,E=13528318,C=78)
	CT(69656)=(S=13528319,E=13530111,C=29)
	CT(69657)=(S=13530112,E=13530367,C=78)
	CT(69658)=(S=13530368,E=13531904,C=29)
	CT(69659)=(S=13531905,E=13531910,C=78)
	CT(69660)=(S=13531911,E=13531911,C=29)
	CT(69661)=(S=13531912,E=13531920,C=78)
	CT(69662)=(S=13531921,E=13531921,C=29)
	CT(69663)=(S=13531922,E=13531927,C=78)
	CT(69664)=(S=13531928,E=13531928,C=29)
	CT(69665)=(S=13531929,E=13531933,C=78)
	CT(69666)=(S=13531934,E=13531934,C=29)
	CT(69667)=(S=13531935,E=13531955,C=78)
	CT(69668)=(S=13531956,E=13532063,C=29)
	CT(69669)=(S=13532064,E=13532095,C=78)
	CT(69670)=(S=13532096,E=13532579,C=29)
	CT(69671)=(S=13532580,E=13532583,C=78)
	CT(69672)=(S=13532584,E=13532615,C=29)
	CT(69673)=(S=13532616,E=13532623,C=78)
	CT(69674)=(S=13532624,E=13532751,C=29)
	CT(69675)=(S=13532752,E=13532767,C=78)
	CT(69676)=(S=13532768,E=13532780,C=29)
	CT(69677)=(S=13532781,E=13532781,C=78)
	CT(69678)=(S=13532782,E=13532783,C=29)
	CT(69679)=(S=13532784,E=13532784,C=78)
	CT(69680)=(S=13532785,E=13532791,C=29)
	CT(69681)=(S=13532792,E=13532799,C=82)
	CT(69682)=(S=13532800,E=13532896,C=29)
	CT(69683)=(S=13532897,E=13532897,C=78)
	CT(69684)=(S=13532898,E=13532915,C=29)
	CT(69685)=(S=13532916,E=13532916,C=186)
	CT(69686)=(S=13532917,E=13533695,C=29)
	CT(69687)=(S=13533697,E=13533697,C=78)
	CT(69688)=(S=13533698,E=13533701,C=29)
	CT(69689)=(S=13533703,E=13533705,C=78)
	CT(69690)=(S=13533706,E=13533706,C=29)
	CT(69691)=(S=13533707,E=13533711,C=78)
	CT(69692)=(S=13533712,E=13533715,C=29)
	CT(69693)=(S=13533716,E=13533757,C=78)
	CT(69694)=(S=13533758,E=13533759,C=29)
	CT(69695)=(S=13533760,E=13533783,C=78)
	CT(69696)=(S=13533784,E=13533785,C=29)
	CT(69697)=(S=13533786,E=13533789,C=78)
	CT(69698)=(S=13533790,E=13533843,C=29)
	CT(69699)=(S=13533845,E=13533951,C=78)
	CT(69700)=(S=13533952,E=13534143,C=29)
	CT(69701)=(S=13534144,E=13534159,C=10)
	CT(69702)=(S=13534160,E=13534255,C=29)
	CT(69703)=(S=13534256,E=13534271,C=78)
	CT(69704)=(S=13534272,E=13534383,C=29)
	CT(69705)=(S=13534384,E=13534399,C=78)
	CT(69706)=(S=13534400,E=13537151,C=29)
	CT(69707)=(S=13537152,E=13537279,C=2)
	CT(69708)=(S=13537280,E=13542015,C=29)
	CT(69709)=(S=13542016,E=13542079,C=78)
	CT(69710)=(S=13542080,E=13542367,C=29)
	CT(69711)=(S=13542368,E=13542399,C=78)
	CT(69712)=(S=13542400,E=13543167,C=29)
	CT(69713)=(S=13543168,E=13543168,C=78)
	CT(69714)=(S=13543169,E=13543169,C=29)
	CT(69715)=(S=13543170,E=13543189,C=78)
	CT(69716)=(S=13543190,E=13543190,C=29)
	CT(69717)=(S=13543191,E=13543215,C=78)
	CT(69718)=(S=13543216,E=13543216,C=29)
	CT(69719)=(S=13543217,E=13543237,C=78)
	CT(69720)=(S=13543238,E=13543239,C=29)
	CT(69721)=(S=13543240,E=13543423,C=78)
	CT(69722)=(S=13543424,E=13544447,C=29)
	CT(69723)=(S=13544448,E=13544703,C=78)
	CT(69724)=(S=13544704,E=13545135,C=29)
	CT(69725)=(S=13545136,E=13545183,C=78)
	CT(69726)=(S=13545184,E=13545727,C=29)
	CT(69727)=(S=13545728,E=13546495,C=78)
	CT(69728)=(S=13546496,E=13546735,C=29)
	CT(69729)=(S=13546736,E=13546751,C=78)
	CT(69730)=(S=13546752,E=13548031,C=29)
	CT(69731)=(S=13548032,E=13548287,C=78)
	CT(69732)=(S=13548288,E=13548607,C=29)
	CT(69733)=(S=13548608,E=13548671,C=78)
	CT(69734)=(S=13548672,E=13548703,C=178)
	CT(69735)=(S=13548704,E=13549311,C=29)
	CT(69736)=(S=13549312,E=13549439,C=78)
	CT(69737)=(S=13549440,E=13550998,C=29)
	CT(69738)=(S=13550999,E=13550999,C=78)
	CT(69739)=(S=13551000,E=13551055,C=29)
	CT(69740)=(S=13551056,E=13551056,C=78)
	CT(69741)=(S=13551057,E=13554271,C=29)
	CT(69742)=(S=13554272,E=13554303,C=78)
	CT(69743)=(S=13554304,E=13554647,C=29)
	CT(69744)=(S=13554648,E=13554649,C=78)
	CT(69745)=(S=13554650,E=13555199,C=29)
	CT(69746)=(S=13555200,E=13555231,C=183)
	CT(69747)=(S=13555232,E=13555439,C=29)
	CT(69748)=(S=13555440,E=13555455,C=78)
	CT(69749)=(S=13555456,E=13556479,C=29)
	CT(69750)=(S=13556480,E=13556543,C=6)
	CT(69751)=(S=13556544,E=13556867,C=29)
	CT(69752)=(S=13556868,E=13556871,C=8)
	CT(69753)=(S=13556872,E=13556927,C=29)
	CT(69754)=(S=13556928,E=13556935,C=78)
	CT(69755)=(S=13556936,E=13557071,C=29)
	CT(69756)=(S=13557072,E=13557087,C=83)
	CT(69757)=(S=13557088,E=13557231,C=29)
	CT(69758)=(S=13557232,E=13557247,C=78)
	CT(69759)=(S=13557248,E=13557311,C=29)
	CT(69760)=(S=13557312,E=13557343,C=78)
	CT(69761)=(S=13557344,E=13557593,C=29)
	CT(69762)=(S=13557594,E=13557595,C=78)
	CT(69763)=(S=13557596,E=13557600,C=29)
	CT(69764)=(S=13557601,E=13557601,C=78)
	CT(69765)=(S=13557602,E=13557616,C=29)
	CT(69766)=(S=13557617,E=13557617,C=78)
	CT(69767)=(S=13557618,E=13557627,C=29)
	CT(69768)=(S=13557628,E=13557628,C=181)
	CT(69769)=(S=13557629,E=13557632,C=29)
	CT(69770)=(S=13557634,E=13557634,C=212)
	CT(69771)=(S=13557635,E=13557639,C=29)
	CT(69772)=(S=13557640,E=13557640,C=105)
	CT(69773)=(S=13557641,E=13557663,C=29)
	CT(69774)=(S=13557664,E=13557695,C=78)
	CT(69775)=(S=13557696,E=13557791,C=29)
	CT(69776)=(S=13557792,E=13557823,C=209)
	CT(69777)=(S=13557824,E=13558143,C=29)
	CT(69778)=(S=13558144,E=13558147,C=8)
	CT(69779)=(S=13558148,E=13563967,C=29)
	CT(69780)=(S=13563968,E=13564031,C=80)
	CT(69781)=(S=13564032,E=13564095,C=78)
	CT(69782)=(S=13564096,E=13565319,C=29)
	CT(69783)=(S=13565320,E=13565327,C=78)
	CT(69784)=(S=13565328,E=13566559,C=29)
	CT(69785)=(S=13566560,E=13566567,C=178)
	CT(69786)=(S=13566568,E=13567487,C=29)
	CT(69787)=(S=13567488,E=13567743,C=78)
	CT(69788)=(S=13567744,E=13571839,C=29)
	CT(69789)=(S=13571840,E=13572095,C=78)
	CT(69790)=(S=13572096,E=13574656,C=29)
	CT(69791)=(S=13574657,E=13574660,C=78)
	CT(69792)=(S=13574661,E=13574667,C=29)
	CT(69793)=(S=13574668,E=13574687,C=78)
	CT(69794)=(S=13574688,E=13574703,C=29)
	CT(69795)=(S=13574704,E=13575167,C=78)
	CT(69796)=(S=13575168,E=13578111,C=29)
	CT(69797)=(S=13578112,E=13578239,C=78)
	CT(69798)=(S=13578240,E=13579759,C=29)
	CT(69799)=(S=13579760,E=13579775,C=78)
	CT(69800)=(S=13579776,E=13579871,C=29)
	CT(69801)=(S=13579872,E=13579903,C=78)
	CT(69802)=(S=13579904,E=13581567,C=29)
	CT(69803)=(S=13581568,E=13581823,C=78)
	CT(69804)=(S=13581824,E=13586687,C=29)
	CT(69805)=(S=13586688,E=13586943,C=78)
	CT(69806)=(S=13586944,E=13590655,C=29)
	CT(69807)=(S=13590656,E=13590783,C=78)
	CT(69808)=(S=13590784,E=13592063,C=29)
	CT(69809)=(S=13592064,E=13592319,C=78)
	CT(69810)=(S=13592320,E=13593343,C=29)
	CT(69811)=(S=13593344,E=13593599,C=78)
	CT(69812)=(S=13593600,E=13594623,C=29)
	CT(69813)=(S=13594624,E=13594751,C=78)
	CT(69814)=(S=13594752,E=13595487,C=29)
	CT(69815)=(S=13595488,E=13595503,C=78)
	CT(69816)=(S=13595520,E=13600255,C=29)
	CT(69817)=(S=13600256,E=13600511,C=78)
	CT(69818)=(S=13600512,E=13600831,C=29)
	CT(69819)=(S=13600832,E=13600895,C=78)
	CT(69820)=(S=13600896,E=13603839,C=29)
	CT(69821)=(S=13603840,E=13604031,C=78)
	CT(69822)=(S=13604032,E=13604591,C=29)
	CT(69823)=(S=13604592,E=13604607,C=80)
	CT(69824)=(S=13604608,E=13605119,C=29)
	CT(69825)=(S=13605120,E=13605183,C=78)
	CT(69826)=(S=13605184,E=13607167,C=29)
	CT(69827)=(S=13607168,E=13607551,C=78)
	CT(69828)=(S=13607552,E=13607935,C=29)
	CT(69829)=(S=13607936,E=13608191,C=78)
	CT(69830)=(S=13608192,E=13608559,C=29)
	CT(69831)=(S=13608560,E=13608575,C=80)
	CT(69832)=(S=13608576,E=13608703,C=29)
	CT(69833)=(S=13608704,E=13608735,C=78)
	CT(69834)=(S=13608736,E=13608895,C=29)
	CT(69835)=(S=13608896,E=13608959,C=78)
	CT(69836)=(S=13608960,E=13610626,C=29)
	CT(69837)=(S=13610632,E=13610632,C=80)
	CT(69838)=(S=13610633,E=13610653,C=29)
	CT(69839)=(S=13610654,E=13610655,C=78)
	CT(69840)=(S=13610656,E=13610707,C=29)
	CT(69841)=(S=13610708,E=13610711,C=95)
	CT(69842)=(S=13610712,E=13611903,C=29)
	CT(69843)=(S=13611904,E=13611967,C=78)
	CT(69844)=(S=13611968,E=13612351,C=29)
	CT(69845)=(S=13612352,E=13612415,C=78)
	CT(69846)=(S=13612416,E=13612831,C=29)
	CT(69847)=(S=13612832,E=13612863,C=78)
	CT(69848)=(S=13612864,E=13614143,C=29)
	CT(69849)=(S=13614144,E=13614175,C=78)
	CT(69850)=(S=13614176,E=13614519,C=29)
	CT(69851)=(S=13614520,E=13614523,C=4)
	CT(69852)=(S=13614524,E=13614559,C=29)
	CT(69853)=(S=13614560,E=13614591,C=78)
	CT(69854)=(S=13614592,E=13615087,C=29)
	CT(69855)=(S=13615088,E=13615103,C=82)
	CT(69856)=(S=13615104,E=13615615,C=29)
	CT(69857)=(S=13615616,E=13615999,C=78)
	CT(69858)=(S=13616000,E=13617279,C=29)
	CT(69859)=(S=13617280,E=13617311,C=78)
	CT(69860)=(S=13617312,E=13618239,C=29)
	CT(69861)=(S=13618240,E=13618303,C=175)
	CT(69862)=(S=13618304,E=13618367,C=80)
	CT(69863)=(S=13618368,E=13618399,C=78)
	CT(69864)=(S=13618400,E=13619711,C=29)
	CT(69865)=(S=13619712,E=13619775,C=78)
	CT(69866)=(S=13619776,E=13621247,C=29)
	CT(69867)=(S=13621248,E=13621503,C=78)
	CT(69868)=(S=13621504,E=13622015,C=29)
	CT(69869)=(S=13622016,E=13622271,C=78)
	CT(69870)=(S=13622272,E=13624383,C=29)
	CT(69871)=(S=13624384,E=13624447,C=78)
	CT(69872)=(S=13624448,E=13624511,C=178)
	CT(69873)=(S=13624512,E=13624575,C=29)
	CT(69874)=(S=13624576,E=13624639,C=78)
	CT(69875)=(S=13624640,E=13625055,C=29)
	CT(69876)=(S=13625056,E=13625087,C=78)
	CT(69877)=(S=13625088,E=13625279,C=29)
	CT(69878)=(S=13625280,E=13625283,C=78)
	CT(69879)=(S=13625284,E=13625311,C=29)
	CT(69880)=(S=13625312,E=13625327,C=78)
	CT(69881)=(S=13625328,E=13625439,C=29)
	CT(69882)=(S=13625440,E=13625471,C=78)
	CT(69883)=(S=13625472,E=13626367,C=29)
	CT(69884)=(S=13626368,E=13626623,C=78)
	CT(69885)=(S=13626624,E=13628555,C=29)
	CT(69886)=(S=13628556,E=13628559,C=78)
	CT(69887)=(S=13628560,E=13628671,C=29)
	CT(69888)=(S=13628672,E=13628735,C=78)
	CT(69889)=(S=13628736,E=13628863,C=29)
	CT(69890)=(S=13628864,E=13628927,C=78)
	CT(69891)=(S=13628928,E=13629439,C=29)
	CT(69892)=(S=13629440,E=13629505,C=204)
	CT(69893)=(S=13629506,E=13629507,C=208)
	CT(69894)=(S=13629508,E=13629513,C=204)
	CT(69895)=(S=13629514,E=13629517,C=83)
	CT(69896)=(S=13629518,E=13629524,C=204)
	CT(69897)=(S=13629525,E=13629525,C=83)
	CT(69898)=(S=13629528,E=13629531,C=204)
	CT(69899)=(S=13629532,E=13629535,C=83)
	CT(69900)=(S=13629536,E=13629540,C=204)
	CT(69901)=(S=13629541,E=13629541,C=215)
	CT(69902)=(S=13629544,E=13629560,C=204)
	CT(69903)=(S=13629561,E=13629561,C=212)
	CT(69904)=(S=13629562,E=13629562,C=204)
	CT(69905)=(S=13629563,E=13629563,C=212)
	CT(69906)=(S=13629564,E=13629567,C=83)
	CT(69907)=(S=13629568,E=13629631,C=204)
	CT(69908)=(S=13629632,E=13629663,C=203)
	CT(69909)=(S=13629664,E=13629951,C=204)
	CT(69910)=(S=13629952,E=13630239,C=29)
	CT(69911)=(S=13630240,E=13630255,C=78)
	CT(69912)=(S=13630256,E=13630719,C=29)
	CT(69913)=(S=13630720,E=13630975,C=78)
	CT(69914)=(S=13630976,E=13641215,C=29)
	CT(69915)=(S=13641216,E=13641279,C=78)
	CT(69916)=(S=13641280,E=13645487,C=29)
	CT(69917)=(S=13645488,E=13645503,C=80)
	CT(69918)=(S=13645504,E=13647915,C=29)
	CT(69919)=(S=13647916,E=13647919,C=95)
	CT(69920)=(S=13647920,E=13647975,C=29)
	CT(69921)=(S=13647976,E=13647979,C=78)
	CT(69922)=(S=13647980,E=13648187,C=29)
	CT(69923)=(S=13648188,E=13648191,C=78)
	CT(69924)=(S=13648192,E=13648199,C=29)
	CT(69925)=(S=13648200,E=13648207,C=78)
	CT(69926)=(S=13648208,E=13648359,C=29)
	CT(69927)=(S=13648360,E=13648363,C=78)
	CT(69928)=(S=13648364,E=13648367,C=29)
	CT(69929)=(S=13648368,E=13648375,C=78)
	CT(69930)=(S=13648376,E=13648383,C=29)
	CT(69931)=(S=13648384,E=13648387,C=78)
	CT(69932)=(S=13648388,E=13648399,C=29)
	CT(69933)=(S=13648400,E=13648403,C=78)
	CT(69934)=(S=13648404,E=13648431,C=29)
	CT(69935)=(S=13648432,E=13648439,C=197)
	CT(69936)=(S=13648440,E=13648447,C=29)
	CT(69937)=(S=13648452,E=13648455,C=78)
	CT(69938)=(S=13648456,E=13648475,C=29)
	CT(69939)=(S=13648476,E=13648479,C=78)
	CT(69940)=(S=13648480,E=13648495,C=29)
	CT(69941)=(S=13648496,E=13648503,C=78)
	CT(69942)=(S=13648504,E=13648607,C=29)
	CT(69943)=(S=13648608,E=13648615,C=78)
	CT(69944)=(S=13648616,E=13648631,C=29)
	CT(69945)=(S=13648632,E=13648639,C=78)
	CT(69946)=(S=13648640,E=13648731,C=29)
	CT(69947)=(S=13648732,E=13648735,C=78)
	CT(69948)=(S=13648736,E=13648783,C=29)
	CT(69949)=(S=13648784,E=13648791,C=78)
	CT(69950)=(S=13648792,E=13648807,C=29)
	CT(69951)=(S=13648808,E=13648811,C=78)
	CT(69952)=(S=13648812,E=13648911,C=29)
	CT(69953)=(S=13648912,E=13648915,C=78)
	CT(69954)=(S=13648916,E=13648927,C=29)
	CT(69955)=(S=13648928,E=13648931,C=78)
	CT(69956)=(S=13648932,E=13648967,C=29)
	CT(69957)=(S=13648968,E=13648971,C=78)
	CT(69958)=(S=13648972,E=13648983,C=29)
	CT(69959)=(S=13648984,E=13648991,C=78)
	CT(69960)=(S=13648992,E=13648999,C=29)
	CT(69961)=(S=13649000,E=13649003,C=78)
	CT(69962)=(S=13649004,E=13649095,C=29)
	CT(69963)=(S=13649096,E=13649103,C=78)
	CT(69964)=(S=13649104,E=13649159,C=29)
	CT(69965)=(S=13649160,E=13649167,C=78)
	CT(69966)=(S=13649168,E=13649207,C=29)
	CT(69967)=(S=13649208,E=13649211,C=78)
	CT(69968)=(S=13649212,E=13649391,C=29)
	CT(69969)=(S=13649392,E=13649399,C=78)
	CT(69970)=(S=13649400,E=13649403,C=29)
	CT(69971)=(S=13649404,E=13649419,C=78)
	CT(69972)=(S=13649424,E=13649463,C=29)
	CT(69973)=(S=13649464,E=13649471,C=78)
	CT(69974)=(S=13649472,E=13649499,C=29)
	CT(69975)=(S=13649500,E=13649503,C=230)
	CT(69976)=(S=13649504,E=13649511,C=29)
	CT(69977)=(S=13649512,E=13649519,C=78)
	CT(69978)=(S=13649520,E=13649619,C=29)
	CT(69979)=(S=13649620,E=13649623,C=78)
	CT(69980)=(S=13649624,E=13649651,C=29)
	CT(69981)=(S=13649652,E=13649655,C=78)
	CT(69982)=(S=13649656,E=13649671,C=29)
	CT(69983)=(S=13649672,E=13649675,C=78)
	CT(69984)=(S=13649676,E=13649751,C=29)
	CT(69985)=(S=13649752,E=13649755,C=78)
	CT(69986)=(S=13649756,E=13649775,C=29)
	CT(69987)=(S=13649776,E=13649783,C=78)
	CT(69988)=(S=13649784,E=13649839,C=29)
	CT(69989)=(S=13649840,E=13649843,C=78)
	CT(69990)=(S=13649844,E=13649847,C=29)
	CT(69991)=(S=13649848,E=13649851,C=78)
	CT(69992)=(S=13649852,E=13649923,C=29)
	CT(69993)=(S=13649924,E=13649927,C=78)
	CT(69994)=(S=13649928,E=13650039,C=29)
	CT(69995)=(S=13650040,E=13650047,C=78)
	CT(69996)=(S=13650048,E=13650087,C=29)
	CT(69997)=(S=13650088,E=13650099,C=78)
	CT(69998)=(S=13650100,E=13650127,C=29)
	CT(69999)=(S=13650128,E=13650131,C=78)
	CT(70000)=(S=13650132,E=13650147,C=29)
	CT(70001)=(S=13650148,E=13650151,C=198)
	CT(70002)=(S=13650152,E=13650231,C=29)
	CT(70003)=(S=13650232,E=13650235,C=78)
	CT(70004)=(S=13650236,E=13650251,C=29)
	CT(70005)=(S=13650252,E=13650255,C=78)
	CT(70006)=(S=13650256,E=13650431,C=29)
	CT(70007)=(S=13650432,E=13650439,C=78)
	CT(70008)=(S=13650440,E=13650455,C=29)
	CT(70009)=(S=13650456,E=13650459,C=78)
	CT(70010)=(S=13650460,E=13650639,C=29)
	CT(70011)=(S=13650640,E=13650647,C=78)
	CT(70012)=(S=13650648,E=13650759,C=29)
	CT(70013)=(S=13650760,E=13650763,C=78)
	CT(70014)=(S=13650764,E=13650803,C=29)
	CT(70015)=(S=13650804,E=13650807,C=78)
	CT(70016)=(S=13650808,E=13650823,C=29)
	CT(70017)=(S=13650824,E=13650827,C=78)
	CT(70018)=(S=13650828,E=13650875,C=29)
	CT(70019)=(S=13650876,E=13650879,C=78)
	CT(70020)=(S=13650880,E=13650887,C=29)
	CT(70021)=(S=13650888,E=13650891,C=178)
	CT(70022)=(S=13650892,E=13650983,C=29)
	CT(70023)=(S=13650984,E=13650987,C=78)
	CT(70024)=(S=13650988,E=13651015,C=29)
	CT(70025)=(S=13651016,E=13651019,C=78)
	CT(70026)=(S=13651020,E=13651047,C=29)
	CT(70027)=(S=13651048,E=13651055,C=78)
	CT(70028)=(S=13651056,E=13651279,C=29)
	CT(70029)=(S=13651280,E=13651287,C=78)
	CT(70030)=(S=13651288,E=13651291,C=29)
	CT(70031)=(S=13651292,E=13651295,C=78)
	CT(70032)=(S=13651296,E=13651339,C=29)
	CT(70033)=(S=13651340,E=13651343,C=78)
	CT(70034)=(S=13651344,E=13651355,C=29)
	CT(70035)=(S=13651356,E=13651359,C=78)
	CT(70036)=(S=13651360,E=13651395,C=29)
	CT(70037)=(S=13651396,E=13651399,C=78)
	CT(70038)=(S=13651400,E=13651471,C=29)
	CT(70039)=(S=13651472,E=13651475,C=78)
	CT(70040)=(S=13651476,E=13651503,C=29)
	CT(70041)=(S=13651504,E=13651511,C=185)
	CT(70042)=(S=13651512,E=13651519,C=78)
	CT(70043)=(S=13651520,E=13651767,C=29)
	CT(70044)=(S=13651768,E=13651775,C=78)
	CT(70045)=(S=13651776,E=13651807,C=29)
	CT(70046)=(S=13651808,E=13651811,C=198)
	CT(70047)=(S=13651812,E=13651823,C=29)
	CT(70048)=(S=13651824,E=13651827,C=78)
	CT(70049)=(S=13651828,E=13651927,C=29)
	CT(70050)=(S=13651928,E=13651931,C=78)
	CT(70051)=(S=13651932,E=13652019,C=29)
	CT(70052)=(S=13652020,E=13652023,C=78)
	CT(70053)=(S=13652024,E=13652063,C=29)
	CT(70054)=(S=13652064,E=13652071,C=78)
	CT(70055)=(S=13652072,E=13652103,C=29)
	CT(70056)=(S=13652104,E=13652107,C=78)
	CT(70057)=(S=13652112,E=13652203,C=29)
	CT(70058)=(S=13652204,E=13652207,C=80)
	CT(70059)=(S=13652208,E=13652219,C=29)
	CT(70060)=(S=13652220,E=13652235,C=78)
	CT(70061)=(S=13652236,E=13652263,C=29)
	CT(70062)=(S=13652264,E=13652271,C=78)
	CT(70063)=(S=13652272,E=13652343,C=29)
	CT(70064)=(S=13652344,E=13652347,C=78)
	CT(70065)=(S=13652348,E=13652367,C=29)
	CT(70066)=(S=13652368,E=13652371,C=78)
	CT(70067)=(S=13652372,E=13652383,C=29)
	CT(70068)=(S=13652384,E=13652387,C=230)
	CT(70069)=(S=13652388,E=13652391,C=29)
	CT(70070)=(S=13652392,E=13652399,C=80)
	CT(70071)=(S=13652400,E=13652435,C=29)
	CT(70072)=(S=13652436,E=13652439,C=78)
	CT(70073)=(S=13652440,E=13652567,C=29)
	CT(70074)=(S=13652568,E=13652575,C=78)
	CT(70075)=(S=13652576,E=13652599,C=29)
	CT(70076)=(S=13652600,E=13652603,C=78)
	CT(70077)=(S=13652604,E=13652643,C=29)
	CT(70078)=(S=13652644,E=13652647,C=178)
	CT(70079)=(S=13652648,E=13652695,C=29)
	CT(70080)=(S=13652696,E=13652699,C=174)
	CT(70081)=(S=13652700,E=13652727,C=29)
	CT(70082)=(S=13652728,E=13652735,C=78)
	CT(70083)=(S=13652736,E=13652815,C=29)
	CT(70084)=(S=13652816,E=13652823,C=183)
	CT(70085)=(S=13652824,E=13652943,C=29)
	CT(70086)=(S=13652944,E=13652951,C=78)
	CT(70087)=(S=13652952,E=13653095,C=29)
	CT(70088)=(S=13653096,E=13653099,C=78)
	CT(70089)=(S=13653100,E=13653183,C=29)
	CT(70090)=(S=13653184,E=13653191,C=180)
	CT(70091)=(S=13653192,E=13653199,C=82)
	CT(70092)=(S=13653200,E=13653327,C=29)
	CT(70093)=(S=13653328,E=13653335,C=78)
	CT(70094)=(S=13653336,E=13653359,C=29)
	CT(70095)=(S=13653360,E=13653363,C=78)
	CT(70096)=(S=13653364,E=13653459,C=29)
	CT(70097)=(S=13653460,E=13653471,C=78)
	CT(70098)=(S=13653472,E=13653643,C=29)
	CT(70099)=(S=13653644,E=13653647,C=78)
	CT(70100)=(S=13653648,E=13653707,C=29)
	CT(70101)=(S=13653708,E=13653711,C=78)
	CT(70102)=(S=13653712,E=13653743,C=29)
	CT(70103)=(S=13653744,E=13653747,C=78)
	CT(70104)=(S=13653748,E=13653779,C=29)
	CT(70105)=(S=13653780,E=13653783,C=78)
	CT(70106)=(S=13653784,E=13653791,C=29)
	CT(70107)=(S=13653792,E=13653799,C=81)
	CT(70108)=(S=13653800,E=13653811,C=29)
	CT(70109)=(S=13653812,E=13653815,C=78)
	CT(70110)=(S=13653816,E=13653903,C=29)
	CT(70111)=(S=13653904,E=13653907,C=230)
	CT(70112)=(S=13653908,E=13653925,C=29)
	CT(70113)=(S=13653926,E=13653927,C=8)
	CT(70114)=(S=13653928,E=13653955,C=29)
	CT(70115)=(S=13653956,E=13653959,C=78)
	CT(70116)=(S=13653960,E=13654019,C=29)
	CT(70117)=(S=13654020,E=13654023,C=78)
	CT(70118)=(S=13654024,E=13654123,C=29)
	CT(70119)=(S=13654124,E=13654127,C=78)
	CT(70120)=(S=13654128,E=13654155,C=29)
	CT(70121)=(S=13654156,E=13654159,C=78)
	CT(70122)=(S=13654160,E=13654199,C=29)
	CT(70123)=(S=13654200,E=13654207,C=78)
	CT(70124)=(S=13654208,E=13654211,C=29)
	CT(70125)=(S=13654212,E=13654215,C=78)
	CT(70126)=(S=13654216,E=13654319,C=29)
	CT(70127)=(S=13654320,E=13654327,C=78)
	CT(70128)=(S=13654328,E=13654355,C=29)
	CT(70129)=(S=13654356,E=13654359,C=78)
	CT(70130)=(S=13654360,E=13654399,C=29)
	CT(70131)=(S=13654400,E=13654403,C=78)
	CT(70132)=(S=13654404,E=13654499,C=29)
	CT(70133)=(S=13654500,E=13654503,C=178)
	CT(70134)=(S=13654504,E=13654595,C=29)
	CT(70135)=(S=13654596,E=13654599,C=78)
	CT(70136)=(S=13654600,E=13654607,C=29)
	CT(70137)=(S=13654608,E=13654615,C=78)
	CT(70138)=(S=13654616,E=13654623,C=29)
	CT(70139)=(S=13654624,E=13654631,C=78)
	CT(70140)=(S=13654632,E=13654639,C=29)
	CT(70141)=(S=13654640,E=13654643,C=231)
	CT(70142)=(S=13654644,E=13654759,C=29)
	CT(70143)=(S=13654760,E=13654763,C=78)
	CT(70144)=(S=13654764,E=13654767,C=29)
	CT(70145)=(S=13654768,E=13654775,C=78)
	CT(70146)=(S=13654776,E=13654975,C=29)
	CT(70147)=(S=13654976,E=13654979,C=185)
	CT(70148)=(S=13654980,E=13654983,C=95)
	CT(70149)=(S=13654984,E=13655031,C=29)
	CT(70150)=(S=13655032,E=13655035,C=78)
	CT(70151)=(S=13655036,E=13655055,C=29)
	CT(70152)=(S=13655056,E=13655063,C=78)
	CT(70153)=(S=13655064,E=13655091,C=29)
	CT(70154)=(S=13655092,E=13655095,C=78)
	CT(70155)=(S=13655096,E=13655151,C=29)
	CT(70156)=(S=13655152,E=13655159,C=78)
	CT(70157)=(S=13655160,E=13655171,C=29)
	CT(70158)=(S=13655172,E=13655183,C=78)
	CT(70159)=(S=13655184,E=13655231,C=29)
	CT(70160)=(S=13655232,E=13655235,C=78)
	CT(70161)=(S=13655236,E=13655271,C=29)
	CT(70162)=(S=13655272,E=13655275,C=78)
	CT(70163)=(S=13655276,E=13655527,C=29)
	CT(70164)=(S=13655528,E=13655535,C=78)
	CT(70165)=(S=13655536,E=13655579,C=29)
	CT(70166)=(S=13655580,E=13655583,C=78)
	CT(70167)=(S=13655584,E=13655655,C=29)
	CT(70168)=(S=13655656,E=13655663,C=78)
	CT(70169)=(S=13655664,E=13655711,C=29)
	CT(70170)=(S=13655712,E=13655719,C=78)
	CT(70171)=(S=13655720,E=13655723,C=198)
	CT(70172)=(S=13655724,E=13655727,C=29)
	CT(70173)=(S=13655728,E=13655735,C=82)
	CT(70174)=(S=13655736,E=13655747,C=29)
	CT(70175)=(S=13655748,E=13655751,C=78)
	CT(70176)=(S=13655752,E=13655843,C=29)
	CT(70177)=(S=13655844,E=13655847,C=80)
	CT(70178)=(S=13655848,E=13655979,C=29)
	CT(70179)=(S=13655980,E=13655983,C=78)
	CT(70180)=(S=13655984,E=13656127,C=29)
	CT(70181)=(S=13656128,E=13656191,C=78)
	CT(70182)=(S=13656192,E=13656383,C=29)
	CT(70183)=(S=13656384,E=13656447,C=78)
	CT(70184)=(S=13656448,E=13656767,C=29)
	CT(70185)=(S=13656768,E=13656831,C=78)
	CT(70186)=(S=13656832,E=13656959,C=29)
	CT(70187)=(S=13656960,E=13656991,C=78)
	CT(70188)=(S=13656992,E=13657407,C=29)
	CT(70189)=(S=13657408,E=13657471,C=78)
	CT(70190)=(S=13657472,E=13658079,C=29)
	CT(70191)=(S=13658080,E=13658111,C=78)
	CT(70192)=(S=13658112,E=13659679,C=29)
	CT(70193)=(S=13659680,E=13659695,C=78)
	CT(70194)=(S=13659696,E=13659743,C=29)
	CT(70195)=(S=13659744,E=13659775,C=78)
	CT(70196)=(S=13659776,E=13659951,C=29)
	CT(70197)=(S=13659952,E=13659959,C=2)
	CT(70198)=(S=13659960,E=13659967,C=29)
	CT(70199)=(S=13659968,E=13659999,C=78)
	CT(70200)=(S=13660000,E=13660415,C=29)
	CT(70201)=(S=13660416,E=13660479,C=78)
	CT(70202)=(S=13660480,E=13660799,C=29)
	CT(70203)=(S=13660800,E=13660863,C=78)
	CT(70204)=(S=13660864,E=13661759,C=29)
	CT(70205)=(S=13661760,E=13661839,C=78)
	CT(70206)=(S=13661840,E=13661903,C=29)
	CT(70207)=(S=13661904,E=13661919,C=78)
	CT(70208)=(S=13661920,E=13662959,C=29)
	CT(70209)=(S=13662960,E=13662975,C=78)
	CT(70210)=(S=13662976,E=13663359,C=29)
	CT(70211)=(S=13663360,E=13663487,C=78)
	CT(70212)=(S=13663488,E=13665183,C=29)
	CT(70213)=(S=13665184,E=13665215,C=175)
	CT(70214)=(S=13665216,E=13666831,C=29)
	CT(70215)=(S=13666832,E=13666863,C=175)
	CT(70216)=(S=13666864,E=13671823,C=29)
	CT(70217)=(S=13671824,E=13671831,C=174)
	CT(70218)=(S=13671832,E=13673247,C=29)
	CT(70219)=(S=13673248,E=13673279,C=175)
	CT(70220)=(S=13673280,E=13674719,C=29)
	CT(70221)=(S=13674720,E=13674751,C=174)
	CT(70222)=(S=13674752,E=13677823,C=29)
	CT(70223)=(S=13677824,E=13678079,C=78)
	CT(70224)=(S=13678080,E=13698303,C=29)
	CT(70225)=(S=13698304,E=13698559,C=78)
	CT(70226)=(S=13698560,E=13700351,C=29)
	CT(70227)=(S=13700352,E=13700607,C=83)
	CT(70228)=(S=13700608,E=13701503,C=29)
	CT(70229)=(S=13701504,E=13701567,C=78)
	CT(70230)=(S=13701568,E=13702143,C=29)
	CT(70231)=(S=13702144,E=13702207,C=78)
	CT(70232)=(S=13702208,E=13703987,C=29)
	CT(70233)=(S=13703988,E=13703991,C=174)
	CT(70234)=(S=13703992,E=13703995,C=29)
	CT(70235)=(S=13703996,E=13703999,C=174)
	CT(70236)=(S=13704000,E=13704447,C=29)
	CT(70237)=(S=13704448,E=13704703,C=78)
	CT(70238)=(S=13704704,E=13705823,C=29)
	CT(70239)=(S=13705824,E=13705839,C=78)
	CT(70240)=(S=13705840,E=13707647,C=29)
	CT(70241)=(S=13707648,E=13707679,C=78)
	CT(70242)=(S=13707680,E=13707871,C=29)
	CT(70243)=(S=13707872,E=13707903,C=78)
	CT(70244)=(S=13707904,E=13708383,C=29)
	CT(70245)=(S=13708384,E=13708415,C=78)
	CT(70246)=(S=13708416,E=13708543,C=29)
	CT(70247)=(S=13708544,E=13708671,C=212)
	CT(70248)=(S=13708672,E=13709887,C=29)
	CT(70249)=(S=13709888,E=13709919,C=78)
	CT(70250)=(S=13709920,E=13710335,C=29)
	CT(70251)=(S=13710336,E=13710847,C=78)
	CT(70252)=(S=13710848,E=13710911,C=29)
	CT(70253)=(S=13710912,E=13710927,C=78)
	CT(70254)=(S=13710928,E=13710959,C=29)
	CT(70255)=(S=13710960,E=13710975,C=78)
	CT(70256)=(S=13710976,E=13712047,C=29)
	CT(70257)=(S=13712048,E=13712063,C=7)
	CT(70258)=(S=13712064,E=13712191,C=29)
	CT(70259)=(S=13712192,E=13712255,C=183)
	CT(70260)=(S=13712256,E=13713071,C=29)
	CT(70261)=(S=13713072,E=13713087,C=78)
	CT(70262)=(S=13713088,E=13714591,C=29)
	CT(70263)=(S=13714592,E=13714623,C=78)
	CT(70264)=(S=13714624,E=13715391,C=29)
	CT(70265)=(S=13715392,E=13715455,C=78)
	CT(70266)=(S=13715456,E=13718015,C=29)
	CT(70267)=(S=13718016,E=13718143,C=78)
	CT(70268)=(S=13718144,E=13719295,C=29)
	CT(70269)=(S=13719296,E=13719359,C=78)
	CT(70270)=(S=13719360,E=13719519,C=29)
	CT(70271)=(S=13719520,E=13719551,C=78)
	CT(70272)=(S=13719552,E=13719807,C=29)
	CT(70273)=(S=13719808,E=13720063,C=78)
	CT(70274)=(S=13720064,E=13720191,C=29)
	CT(70275)=(S=13720192,E=13720255,C=78)
	CT(70276)=(S=13720256,E=13720303,C=29)
	CT(70277)=(S=13720304,E=13720319,C=78)
	CT(70278)=(S=13720320,E=13720383,C=29)
	CT(70279)=(S=13720384,E=13720511,C=78)
	CT(70280)=(S=13720512,E=13720575,C=80)
	CT(70281)=(S=13720576,E=13721279,C=29)
	CT(70282)=(S=13721280,E=13721311,C=176)
	CT(70283)=(S=13721312,E=13722047,C=29)
	CT(70284)=(S=13722048,E=13722079,C=78)
	CT(70285)=(S=13722080,E=13722591,C=29)
	CT(70286)=(S=13722592,E=13722607,C=83)
	CT(70287)=(S=13722608,E=13723391,C=29)
	CT(70288)=(S=13723392,E=13723455,C=78)
	CT(70289)=(S=13723456,E=13723711,C=29)
	CT(70290)=(S=13723712,E=13723775,C=78)
	CT(70291)=(S=13723776,E=13723807,C=29)
	CT(70292)=(S=13723808,E=13723839,C=78)
	CT(70293)=(S=13723840,E=13724095,C=29)
	CT(70294)=(S=13724096,E=13724127,C=78)
	CT(70295)=(S=13724128,E=13724511,C=29)
	CT(70296)=(S=13724512,E=13724543,C=78)
	CT(70297)=(S=13724608,E=13725695,C=29)
	CT(70298)=(S=13725696,E=13725759,C=78)
	CT(70299)=(S=13725760,E=13726591,C=29)
	CT(70300)=(S=13726592,E=13726719,C=78)
	CT(70301)=(S=13726720,E=13727999,C=29)
	CT(70302)=(S=13728000,E=13728255,C=78)
	CT(70303)=(S=13728256,E=13729791,C=29)
	CT(70304)=(S=13729792,E=13729855,C=78)
	CT(70305)=(S=13729856,E=13731295,C=29)
	CT(70306)=(S=13731296,E=13731327,C=78)
	CT(70307)=(S=13731328,E=13731647,C=29)
	CT(70308)=(S=13731648,E=13731711,C=78)
	CT(70309)=(S=13731712,E=13732799,C=29)
	CT(70310)=(S=13732800,E=13732863,C=78)
	CT(70311)=(S=13732864,E=13733247,C=29)
	CT(70312)=(S=13733248,E=13733327,C=78)
	CT(70313)=(S=13733328,E=13733343,C=29)
	CT(70314)=(S=13733344,E=13733375,C=28)
	CT(70315)=(S=13733376,E=13733439,C=78)
	CT(70316)=(S=13733440,E=13734239,C=29)
	CT(70317)=(S=13734240,E=13734271,C=78)
	CT(70318)=(S=13734272,E=13734335,C=29)
	CT(70319)=(S=13734336,E=13734399,C=78)
	CT(70320)=(S=13734400,E=13734527,C=29)
	CT(70321)=(S=13734528,E=13734655,C=78)
	CT(70322)=(S=13734656,E=13735039,C=29)
	CT(70323)=(S=13735040,E=13735167,C=78)
	CT(70324)=(S=13735168,E=13735679,C=29)
	CT(70325)=(S=13735680,E=13735711,C=78)
	CT(70326)=(S=13735712,E=13735775,C=29)
	CT(70327)=(S=13735808,E=13735823,C=78)
	CT(70328)=(S=13735824,E=13736383,C=29)
	CT(70329)=(S=13736384,E=13736447,C=78)
	CT(70330)=(S=13736448,E=13737903,C=29)
	CT(70331)=(S=13737904,E=13737919,C=78)
	CT(70332)=(S=13737920,E=13738431,C=29)
	CT(70333)=(S=13738432,E=13738495,C=78)
	CT(70334)=(S=13738496,E=13738655,C=29)
	CT(70335)=(S=13738656,E=13738671,C=78)
	CT(70336)=(S=13738672,E=13738719,C=29)
	CT(70337)=(S=13738720,E=13738751,C=78)
	CT(70338)=(S=13738752,E=13740415,C=29)
	CT(70339)=(S=13740416,E=13740479,C=78)
	CT(70340)=(S=13740480,E=13740799,C=29)
	CT(70341)=(S=13740800,E=13741055,C=78)
	CT(70342)=(S=13741056,E=13741087,C=29)
	CT(70343)=(S=13741088,E=13741119,C=78)
	CT(70344)=(S=13741120,E=13742431,C=29)
	CT(70345)=(S=13742432,E=13742447,C=78)
	CT(70346)=(S=13742448,E=13743871,C=29)
	CT(70347)=(S=13743872,E=13743903,C=78)
	CT(70348)=(S=13743904,E=13743999,C=29)
	CT(70349)=(S=13744000,E=13744031,C=78)
	CT(70350)=(S=13744032,E=13747007,C=29)
	CT(70351)=(S=13747008,E=13747071,C=78)
	CT(70352)=(S=13747072,E=13747423,C=29)
	CT(70353)=(S=13747424,E=13747439,C=78)
	CT(70354)=(S=13747440,E=13747583,C=29)
	CT(70355)=(S=13747584,E=13747647,C=78)
	CT(70356)=(S=13747648,E=13747951,C=29)
	CT(70357)=(S=13747952,E=13747959,C=28)
	CT(70358)=(S=13747960,E=13748735,C=29)
	CT(70359)=(S=13748736,E=13748863,C=78)
	CT(70360)=(S=13748864,E=13748991,C=29)
	CT(70361)=(S=13748992,E=13749055,C=105)
	CT(70362)=(S=13749056,E=13749567,C=29)
	CT(70363)=(S=13749568,E=13749615,C=78)
	CT(70364)=(S=13749616,E=13749727,C=29)
	CT(70365)=(S=13749744,E=13749759,C=78)
	CT(70366)=(S=13749760,E=13750079,C=29)
	CT(70367)=(S=13750080,E=13750143,C=78)
	CT(70368)=(S=13750144,E=13751391,C=29)
	CT(70369)=(S=13751392,E=13751423,C=105)
	CT(70370)=(S=13751424,E=13751775,C=29)
	CT(70371)=(S=13751776,E=13751807,C=78)
	CT(70372)=(S=13751808,E=13752447,C=29)
	CT(70373)=(S=13752448,E=13752479,C=78)
	CT(70374)=(S=13752480,E=13752639,C=29)
	CT(70375)=(S=13752640,E=13752703,C=78)
	CT(70376)=(S=13752704,E=13753663,C=29)
	CT(70377)=(S=13753664,E=13753727,C=78)
	CT(70378)=(S=13753728,E=13753903,C=29)
	CT(70379)=(S=13753904,E=13753919,C=78)
	CT(70380)=(S=13753920,E=13754015,C=29)
	CT(70381)=(S=13754016,E=13754031,C=78)
	CT(70382)=(S=13754032,E=13754079,C=29)
	CT(70383)=(S=13754080,E=13754095,C=78)
	CT(70384)=(S=13754096,E=13754751,C=29)
	CT(70385)=(S=13754752,E=13754815,C=78)
	CT(70386)=(S=13754816,E=13754879,C=29)
	CT(70387)=(S=13754880,E=13755135,C=78)
	CT(70388)=(S=13755136,E=13755263,C=29)
	CT(70389)=(S=13755264,E=13755327,C=78)
	CT(70390)=(S=13755328,E=13757439,C=29)
	CT(70391)=(S=13757440,E=13757503,C=175)
	CT(70392)=(S=13757504,E=13758207,C=29)
	CT(70393)=(S=13758208,E=13758239,C=78)
	CT(70394)=(S=13758240,E=13758495,C=29)
	CT(70395)=(S=13758496,E=13758511,C=178)
	CT(70396)=(S=13758512,E=13758575,C=29)
	CT(70397)=(S=13758576,E=13758591,C=78)
	CT(70398)=(S=13758592,E=13761151,C=29)
	CT(70399)=(S=13761152,E=13761215,C=78)
	CT(70400)=(S=13761216,E=13761303,C=29)
	CT(70401)=(S=13761304,E=13761307,C=78)
	CT(70402)=(S=13761308,E=13762559,C=29)
	CT(70403)=(S=13762560,E=13762591,C=0)
	CT(70404)=(S=13762592,E=13762623,C=6)
	CT(70405)=(S=13762624,E=13762687,C=0)
	CT(70406)=(S=13762688,E=13762815,C=8)
	CT(70407)=(S=13762816,E=13762879,C=3)
	CT(70408)=(S=13762880,E=13762959,C=10)
	CT(70409)=(S=13762960,E=13763007,C=2)
	CT(70410)=(S=13763008,E=13763039,C=0)
	CT(70411)=(S=13763040,E=13763047,C=5)
	CT(70412)=(S=13763048,E=13763051,C=0)
	CT(70413)=(S=13763052,E=13763055,C=2)
	CT(70414)=(S=13763056,E=13763071,C=84)
	CT(70415)=(S=13763072,E=13763103,C=1)
	CT(70416)=(S=13763104,E=13763135,C=6)
	CT(70417)=(S=13763136,E=13763199,C=11)
	CT(70418)=(S=13763200,E=13763263,C=77)
	CT(70419)=(S=13763264,E=13763327,C=2)
	CT(70420)=(S=13763328,E=13763583,C=8)
	CT(70421)=(S=13763584,E=13763647,C=10)
	CT(70422)=(S=13763648,E=13763663,C=84)
	CT(70423)=(S=13763664,E=13763671,C=0)
	CT(70424)=(S=13763672,E=13763679,C=6)
	CT(70425)=(S=13763680,E=13763711,C=10)
	CT(70426)=(S=13763712,E=13763743,C=3)
	CT(70427)=(S=13763744,E=13763775,C=2)
	CT(70428)=(S=13763776,E=13763791,C=7)
	CT(70429)=(S=13763792,E=13763799,C=73)
	CT(70430)=(S=13763800,E=13763807,C=6)
	CT(70431)=(S=13763808,E=13763823,C=0)
	CT(70432)=(S=13763824,E=13763839,C=2)
	CT(70433)=(S=13763840,E=13763871,C=1)
	CT(70434)=(S=13763872,E=13763879,C=0)
	CT(70435)=(S=13763880,E=13763887,C=5)
	CT(70436)=(S=13763888,E=13763895,C=73)
	CT(70437)=(S=13763896,E=13763903,C=1)
	CT(70438)=(S=13763904,E=13763967,C=10)
	CT(70439)=(S=13763968,E=13763999,C=1)
	CT(70440)=(S=13764000,E=13764351,C=8)
	CT(70441)=(S=13764352,E=13764383,C=91)
	CT(70442)=(S=13764384,E=13764399,C=73)
	CT(70443)=(S=13764400,E=13764407,C=0)
	CT(70444)=(S=13764416,E=13764447,C=4)
	CT(70445)=(S=13764448,E=13764607,C=2)
	CT(70446)=(S=13764608,E=13765631,C=0)
	CT(70447)=(S=13765632,E=13766143,C=1)
	CT(70448)=(S=13766144,E=13766191,C=10)
	CT(70449)=(S=13766192,E=13766207,C=2)
	CT(70450)=(S=13766208,E=13766239,C=1)
	CT(70451)=(S=13766240,E=13766255,C=0)
	CT(70452)=(S=13766256,E=13766591,C=1)
	CT(70453)=(S=13766592,E=13766655,C=0)
	CT(70454)=(S=13766656,E=13766719,C=10)
	CT(70455)=(S=13766784,E=13766847,C=1)
	CT(70456)=(S=13766848,E=13766911,C=6)
	CT(70457)=(S=13766912,E=13767039,C=9)
	CT(70458)=(S=13767040,E=13767167,C=2)
	CT(70459)=(S=13767168,E=13767359,C=4)
	CT(70460)=(S=13767360,E=13767423,C=0)
	CT(70461)=(S=13767424,E=13767679,C=5)
	CT(70462)=(S=13767680,E=13767935,C=2)
	CT(70463)=(S=13767936,E=13768447,C=1)
	CT(70464)=(S=13768448,E=13768479,C=7)
	CT(70465)=(S=13768480,E=13768511,C=1)
	CT(70466)=(S=13768512,E=13768527,C=74)
	CT(70467)=(S=13768528,E=13768543,C=167)
	CT(70468)=(S=13768544,E=13768575,C=10)
	CT(70469)=(S=13768576,E=13768607,C=2)
	CT(70470)=(S=13768608,E=13768703,C=10)
	CT(70471)=(S=13768704,E=13768959,C=2)
	CT(70472)=(S=13768960,E=13774847,C=1)
	CT(70473)=(S=13774848,E=13774975,C=73)
	CT(70474)=(S=13774976,E=13774983,C=2)
	CT(70475)=(S=13774984,E=13774991,C=8)
	CT(70476)=(S=13774992,E=13775007,C=5)
	CT(70477)=(S=13775008,E=13775039,C=73)
	CT(70478)=(S=13775040,E=13775055,C=5)
	CT(70479)=(S=13775056,E=13775063,C=0)
	CT(70480)=(S=13775064,E=13775071,C=5)
	CT(70481)=(S=13775072,E=13775103,C=2)
	CT(70482)=(S=13775104,E=13775615,C=0)
	CT(70483)=(S=13775616,E=13776383,C=1)
	CT(70484)=(S=13776384,E=13776895,C=73)
	CT(70485)=(S=13776896,E=13776927,C=77)
	CT(70486)=(S=13776928,E=13776943,C=73)
	CT(70487)=(S=13776944,E=13776959,C=8)
	CT(70488)=(S=13776960,E=13776991,C=0)
	CT(70489)=(S=13776992,E=13777023,C=4)
	CT(70490)=(S=13777024,E=13777039,C=7)
	CT(70491)=(S=13777040,E=13777047,C=4)
	CT(70492)=(S=13777048,E=13777055,C=0)
	CT(70493)=(S=13777056,E=13777087,C=2)
	CT(70494)=(S=13777088,E=13777119,C=1)
	CT(70495)=(S=13777120,E=13777151,C=0)
	CT(70496)=(S=13777152,E=13777359,C=8)
	CT(70497)=(S=13777360,E=13777375,C=74)
	CT(70498)=(S=13777376,E=13777407,C=6)
	CT(70499)=(S=13777408,E=13780991,C=9)
	CT(70500)=(S=13780992,E=13782783,C=1)
	CT(70501)=(S=13782784,E=13782799,C=2)
	CT(70502)=(S=13782800,E=13782815,C=0)
	CT(70503)=(S=13782816,E=13782831,C=2)
	CT(70504)=(S=13782832,E=13782839,C=73)
	CT(70505)=(S=13782840,E=13782847,C=0)
	CT(70506)=(S=13782848,E=13782911,C=1)
	CT(70507)=(S=13782976,E=13782991,C=2)
	CT(70508)=(S=13782992,E=13783007,C=74)
	CT(70509)=(S=13783008,E=13783039,C=1)
	CT(70510)=(S=13783040,E=13783103,C=89)
	CT(70511)=(S=13783104,E=13783135,C=9)
	CT(70512)=(S=13783136,E=13783167,C=6)
	CT(70513)=(S=13783168,E=13783231,C=0)
	CT(70514)=(S=13783232,E=13783551,C=2)
	CT(70515)=(S=13783552,E=13784063,C=1)
	CT(70516)=(S=13784064,E=13784319,C=0)
	CT(70517)=(S=13784320,E=13784575,C=9)
	CT(70518)=(S=13784576,E=13784703,C=73)
	CT(70519)=(S=13784704,E=13784799,C=3)
	CT(70520)=(S=13784800,E=13784815,C=11)
	CT(70521)=(S=13784816,E=13784895,C=0)
	CT(70522)=(S=13784960,E=13785023,C=1)
	CT(70523)=(S=13785024,E=13785055,C=6)
	CT(70524)=(S=13785056,E=13785071,C=2)
	CT(70525)=(S=13785072,E=13785087,C=8)
	CT(70526)=(S=13785088,E=13785375,C=2)
	CT(70527)=(S=13785376,E=13785407,C=4)
	CT(70528)=(S=13785408,E=13785439,C=8)
	CT(70529)=(S=13785440,E=13785471,C=2)
	CT(70530)=(S=13785472,E=13785503,C=0)
	CT(70531)=(S=13785504,E=13785535,C=6)
	CT(70532)=(S=13785536,E=13785599,C=2)
	CT(70533)=(S=13785600,E=13795327,C=6)
	CT(70534)=(S=13795328,E=13807615,C=2)
	CT(70535)=(S=13807616,E=13808127,C=8)
	CT(70536)=(S=13808128,E=13809663,C=6)
	CT(70537)=(S=13809664,E=13809919,C=8)
	CT(70538)=(S=13809920,E=13809983,C=73)
	CT(70539)=(S=13809984,E=13810047,C=0)
	CT(70540)=(S=13810048,E=13810079,C=2)
	CT(70541)=(S=13810080,E=13810111,C=10)
	CT(70542)=(S=13810112,E=13810175,C=1)
	CT(70543)=(S=13810176,E=13810687,C=5)
	CT(70544)=(S=13810688,E=13811711,C=2)
	CT(70545)=(S=13811712,E=13811775,C=9)
	CT(70546)=(S=13811776,E=13811807,C=6)
	CT(70547)=(S=13811808,E=13811839,C=1)
	CT(70548)=(S=13811840,E=13811967,C=9)
	CT(70549)=(S=13811968,E=13812031,C=7)
	CT(70550)=(S=13812032,E=13812095,C=2)
	CT(70551)=(S=13812096,E=13812223,C=0)
	CT(70552)=(S=13812224,E=13812479,C=2)
	CT(70553)=(S=13812480,E=13812735,C=5)
	CT(70554)=(S=13812736,E=13813759,C=2)
	CT(70555)=(S=13813760,E=13814655,C=9)
	CT(70556)=(S=13814656,E=13814719,C=3)
	CT(70557)=(S=13814720,E=13814783,C=2)
	CT(70558)=(S=13814784,E=13815807,C=6)
	CT(70559)=(S=13815808,E=13816127,C=9)
	CT(70560)=(S=13816128,E=13816191,C=8)
	CT(70561)=(S=13816192,E=13816319,C=9)
	CT(70562)=(S=13816320,E=13816447,C=4)
	CT(70563)=(S=13816448,E=13816511,C=74)
	CT(70564)=(S=13816512,E=13816591,C=6)
	CT(70565)=(S=13816592,E=13816599,C=74)
	CT(70566)=(S=13816600,E=13816607,C=8)
	CT(70567)=(S=13816608,E=13816639,C=2)
	CT(70568)=(S=13816640,E=13816671,C=6)
	CT(70569)=(S=13816672,E=13816703,C=11)
	CT(70570)=(S=13816704,E=13817087,C=4)
	CT(70571)=(S=13817088,E=13817151,C=3)
	CT(70572)=(S=13817152,E=13817343,C=10)
	CT(70573)=(S=13817344,E=13817599,C=4)
	CT(70574)=(S=13817600,E=13817855,C=0)
	CT(70575)=(S=13817856,E=13819903,C=6)
	CT(70576)=(S=13819904,E=13823999,C=2)
	CT(70577)=(S=13824000,E=13825279,C=9)
	CT(70578)=(S=13825280,E=13825407,C=11)
	CT(70579)=(S=13825408,E=13825535,C=8)
	CT(70580)=(S=13825536,E=13825599,C=73)
	CT(70581)=(S=13825600,E=13825791,C=3)
	CT(70582)=(S=13825792,E=13825919,C=2)
	CT(70583)=(S=13825920,E=13826031,C=0)
	CT(70584)=(S=13826032,E=13826047,C=74)
	CT(70585)=(S=13826048,E=13833215,C=2)
	CT(70586)=(S=13833216,E=13834239,C=9)
	CT(70587)=(S=13834240,E=13834751,C=5)
	CT(70588)=(S=13834752,E=13836287,C=0)
	CT(70589)=(S=13836288,E=13844479,C=6)
	CT(70590)=(S=13844480,E=13846527,C=1)
	CT(70591)=(S=13846528,E=13848575,C=9)
	CT(70592)=(S=13848576,E=13854719,C=1)
	CT(70593)=(S=13854720,E=13858815,C=6)
	CT(70594)=(S=13858816,E=13862911,C=2)
	CT(70595)=(S=13862912,E=13871103,C=1)
	CT(70596)=(S=13871104,E=13893631,C=6)
	CT(70597)=(S=13893632,E=13893695,C=13)
	CT(70598)=(S=13893696,E=13893727,C=18)
	CT(70599)=(S=13893728,E=13893759,C=25)
	CT(70600)=(S=13893760,E=13893791,C=119)
	CT(70601)=(S=13893792,E=13893823,C=21)
	CT(70602)=(S=13893824,E=13893855,C=53)
	CT(70603)=(S=13893856,E=13893919,C=36)
	CT(70604)=(S=13893920,E=13893951,C=28)
	CT(70605)=(S=13893952,E=13894015,C=31)
	CT(70606)=(S=13894016,E=13894047,C=14)
	CT(70607)=(S=13894048,E=13894079,C=36)
	CT(70608)=(S=13894080,E=13894087,C=26)
	CT(70609)=(S=13894088,E=13894095,C=14)
	CT(70610)=(S=13894096,E=13894103,C=29)
	CT(70611)=(S=13894104,E=13894111,C=14)
	CT(70612)=(S=13894112,E=13894143,C=18)
	CT(70613)=(S=13894144,E=13894175,C=14)
	CT(70614)=(S=13894176,E=13894239,C=28)
	CT(70615)=(S=13894240,E=13894271,C=58)
	CT(70616)=(S=13894272,E=13894303,C=31)
	CT(70617)=(S=13894304,E=13894335,C=63)
	CT(70618)=(S=13894336,E=13894367,C=37)
	CT(70619)=(S=13894368,E=13894399,C=62)
	CT(70620)=(S=13894400,E=13894431,C=19)
	CT(70621)=(S=13894432,E=13894463,C=25)
	CT(70622)=(S=13894464,E=13894495,C=28)
	CT(70623)=(S=13894496,E=13894527,C=31)
	CT(70624)=(S=13894528,E=13894559,C=18)
	CT(70625)=(S=13894560,E=13894591,C=15)
	CT(70626)=(S=13894592,E=13894623,C=28)
	CT(70627)=(S=13894624,E=13894655,C=14)
	CT(70628)=(S=13894656,E=13894687,C=15)
	CT(70629)=(S=13894688,E=13894719,C=27)
	CT(70630)=(S=13894720,E=13894751,C=34)
	CT(70631)=(S=13894752,E=13894783,C=25)
	CT(70632)=(S=13894784,E=13894815,C=33)
	CT(70633)=(S=13894816,E=13894847,C=28)
	CT(70634)=(S=13894848,E=13894879,C=36)
	CT(70635)=(S=13894880,E=13894943,C=28)
	CT(70636)=(S=13894944,E=13894975,C=51)
	CT(70637)=(S=13894976,E=13895039,C=18)
	CT(70638)=(S=13895040,E=13895071,C=51)
	CT(70639)=(S=13895072,E=13895103,C=18)
	CT(70640)=(S=13895104,E=13895135,C=71)
	CT(70641)=(S=13895136,E=13895199,C=18)
	CT(70642)=(S=13895200,E=13895231,C=15)
	CT(70643)=(S=13895232,E=13895423,C=28)
	CT(70644)=(S=13895424,E=13895455,C=55)
	CT(70645)=(S=13895456,E=13895487,C=25)
	CT(70646)=(S=13895488,E=13895519,C=15)
	CT(70647)=(S=13895520,E=13895551,C=18)
	CT(70648)=(S=13895552,E=13895615,C=28)
	CT(70649)=(S=13895616,E=13895647,C=36)
	CT(70650)=(S=13895648,E=13895679,C=18)
	CT(70651)=(S=13895680,E=13895711,C=28)
	CT(70652)=(S=13895712,E=13895743,C=31)
	CT(70653)=(S=13895744,E=13895807,C=25)
	CT(70654)=(S=13895808,E=13895839,C=28)
	CT(70655)=(S=13895840,E=13895871,C=13)
	CT(70656)=(S=13895872,E=13895903,C=28)
	CT(70657)=(S=13895904,E=13895935,C=18)
	CT(70658)=(S=13895936,E=13895967,C=99)
	CT(70659)=(S=13895968,E=13895999,C=28)
	CT(70660)=(S=13896000,E=13896031,C=25)
	CT(70661)=(S=13896032,E=13896063,C=14)
	CT(70662)=(S=13896064,E=13896095,C=42)
	CT(70663)=(S=13896096,E=13896127,C=28)
	CT(70664)=(S=13896128,E=13896159,C=14)
	CT(70665)=(S=13896160,E=13896191,C=31)
	CT(70666)=(S=13896192,E=13896447,C=24)
	CT(70667)=(S=13896448,E=13896511,C=12)
	CT(70668)=(S=13896512,E=13896543,C=14)
	CT(70669)=(S=13896544,E=13896575,C=33)
	CT(70670)=(S=13896576,E=13896607,C=18)
	CT(70671)=(S=13896608,E=13896639,C=23)
	CT(70672)=(S=13896640,E=13896671,C=30)
	CT(70673)=(S=13896672,E=13896703,C=28)
	CT(70674)=(S=13896704,E=13896735,C=18)
	CT(70675)=(S=13896736,E=13896767,C=28)
	CT(70676)=(S=13896768,E=13896799,C=18)
	CT(70677)=(S=13896800,E=13896831,C=28)
	CT(70678)=(S=13896832,E=13896863,C=37)
	CT(70679)=(S=13896864,E=13896895,C=23)
	CT(70680)=(S=13896896,E=13896927,C=44)
	CT(70681)=(S=13896928,E=13896959,C=106)
	CT(70682)=(S=13896960,E=13896991,C=18)
	CT(70683)=(S=13896992,E=13897023,C=21)
	CT(70684)=(S=13897024,E=13897055,C=54)
	CT(70685)=(S=13897056,E=13897087,C=18)
	CT(70686)=(S=13897088,E=13897151,C=20)
	CT(70687)=(S=13897152,E=13897183,C=14)
	CT(70688)=(S=13897184,E=13897215,C=52)
	CT(70689)=(S=13897216,E=13897279,C=58)
	CT(70690)=(S=13897280,E=13897311,C=28)
	CT(70691)=(S=13897312,E=13897343,C=25)
	CT(70692)=(S=13897344,E=13897375,C=15)
	CT(70693)=(S=13897376,E=13897439,C=18)
	CT(70694)=(S=13897440,E=13897471,C=43)
	CT(70695)=(S=13897472,E=13897503,C=37)
	CT(70696)=(S=13897504,E=13897535,C=18)
	CT(70697)=(S=13897536,E=13897567,C=14)
	CT(70698)=(S=13897568,E=13897599,C=18)
	CT(70699)=(S=13897600,E=13897631,C=31)
	CT(70700)=(S=13897632,E=13897663,C=57)
	CT(70701)=(S=13897664,E=13897695,C=28)
	CT(70702)=(S=13897696,E=13897727,C=25)
	CT(70703)=(S=13897728,E=13897759,C=18)
	CT(70704)=(S=13897760,E=13897791,C=42)
	CT(70705)=(S=13897792,E=13897823,C=26)
	CT(70706)=(S=13897824,E=13897855,C=47)
	CT(70707)=(S=13897856,E=13897887,C=50)
	CT(70708)=(S=13897888,E=13897919,C=19)
	CT(70709)=(S=13897920,E=13897951,C=18)
	CT(70710)=(S=13897952,E=13897983,C=28)
	CT(70711)=(S=13897984,E=13898015,C=18)
	CT(70712)=(S=13898016,E=13898047,C=63)
	CT(70713)=(S=13898048,E=13898111,C=42)
	CT(70714)=(S=13898112,E=13898143,C=27)
	CT(70715)=(S=13898144,E=13898175,C=47)
	CT(70716)=(S=13898176,E=13898207,C=15)
	CT(70717)=(S=13898208,E=13898271,C=28)
	CT(70718)=(S=13898272,E=13898303,C=52)
	CT(70719)=(S=13898304,E=13898335,C=28)
	CT(70720)=(S=13898336,E=13898367,C=34)
	CT(70721)=(S=13898368,E=13898399,C=27)
	CT(70722)=(S=13898400,E=13898431,C=21)
	CT(70723)=(S=13898432,E=13898463,C=28)
	CT(70724)=(S=13898464,E=13898479,C=14)
	CT(70725)=(S=13898480,E=13898487,C=12)
	CT(70726)=(S=13898488,E=13898491,C=14)
	CT(70727)=(S=13898492,E=13898495,C=17)
	CT(70728)=(S=13898496,E=13898527,C=18)
	CT(70729)=(S=13898528,E=13898559,C=28)
	CT(70730)=(S=13898560,E=13898591,C=14)
	CT(70731)=(S=13898592,E=13898623,C=15)
	CT(70732)=(S=13898624,E=13898655,C=20)
	CT(70733)=(S=13898656,E=13898687,C=12)
	CT(70734)=(S=13898688,E=13898751,C=36)
	CT(70735)=(S=13898752,E=13898815,C=18)
	CT(70736)=(S=13898816,E=13898879,C=33)
	CT(70737)=(S=13898880,E=13898943,C=28)
	CT(70738)=(S=13898944,E=13898975,C=27)
	CT(70739)=(S=13898976,E=13899007,C=14)
	CT(70740)=(S=13899008,E=13899039,C=18)
	CT(70741)=(S=13899040,E=13899071,C=14)
	CT(70742)=(S=13899072,E=13899103,C=28)
	CT(70743)=(S=13899104,E=13899135,C=14)
	CT(70744)=(S=13899136,E=13899167,C=51)
	CT(70745)=(S=13899168,E=13899199,C=28)
	CT(70746)=(S=13899200,E=13899263,C=25)
	CT(70747)=(S=13899264,E=13899295,C=14)
	CT(70748)=(S=13899296,E=13899327,C=25)
	CT(70749)=(S=13899328,E=13899359,C=18)
	CT(70750)=(S=13899360,E=13899391,C=14)
	CT(70751)=(S=13899392,E=13899423,C=15)
	CT(70752)=(S=13899424,E=13899455,C=109)
	CT(70753)=(S=13899456,E=13899487,C=31)
	CT(70754)=(S=13899488,E=13899519,C=69)
	CT(70755)=(S=13899520,E=13899583,C=14)
	CT(70756)=(S=13899584,E=13899615,C=18)
	CT(70757)=(S=13899616,E=13899679,C=28)
	CT(70758)=(S=13899680,E=13899743,C=12)
	CT(70759)=(S=13899744,E=13899775,C=34)
	CT(70760)=(S=13899776,E=13899807,C=15)
	CT(70761)=(S=13899808,E=13899839,C=18)
	CT(70762)=(S=13899840,E=13899871,C=14)
	CT(70763)=(S=13899872,E=13899903,C=42)
	CT(70764)=(S=13899904,E=13899935,C=33)
	CT(70765)=(S=13899936,E=13899967,C=50)
	CT(70766)=(S=13899968,E=13899999,C=70)
	CT(70767)=(S=13900000,E=13900031,C=23)
	CT(70768)=(S=13900032,E=13900063,C=34)
	CT(70769)=(S=13900064,E=13900095,C=51)
	CT(70770)=(S=13900096,E=13900159,C=17)
	CT(70771)=(S=13900160,E=13900191,C=19)
	CT(70772)=(S=13900192,E=13900223,C=15)
	CT(70773)=(S=13900224,E=13900255,C=36)
	CT(70774)=(S=13900256,E=13900287,C=14)
	CT(70775)=(S=13900288,E=13900415,C=23)
	CT(70776)=(S=13900416,E=13900447,C=31)
	CT(70777)=(S=13900448,E=13900479,C=71)
	CT(70778)=(S=13900480,E=13900511,C=36)
	CT(70779)=(S=13900512,E=13900543,C=18)
	CT(70780)=(S=13900544,E=13900575,C=19)
	CT(70781)=(S=13900576,E=13900607,C=12)
	CT(70782)=(S=13900608,E=13900671,C=42)
	CT(70783)=(S=13900672,E=13900703,C=28)
	CT(70784)=(S=13900704,E=13900735,C=34)
	CT(70785)=(S=13900736,E=13900767,C=33)
	CT(70786)=(S=13900768,E=13900799,C=55)
	CT(70787)=(S=13900800,E=13900831,C=14)
	CT(70788)=(S=13900832,E=13900863,C=28)
	CT(70789)=(S=13900864,E=13900895,C=53)
	CT(70790)=(S=13900896,E=13900927,C=28)
	CT(70791)=(S=13900928,E=13900959,C=34)
	CT(70792)=(S=13900960,E=13900991,C=15)
	CT(70793)=(S=13900992,E=13901023,C=19)
	CT(70794)=(S=13901024,E=13901055,C=39)
	CT(70795)=(S=13901056,E=13901119,C=28)
	CT(70796)=(S=13901120,E=13901183,C=37)
	CT(70797)=(S=13901184,E=13901215,C=15)
	CT(70798)=(S=13901216,E=13901247,C=36)
	CT(70799)=(S=13901248,E=13901311,C=17)
	CT(70800)=(S=13901312,E=13901343,C=14)
	CT(70801)=(S=13901344,E=13901375,C=39)
	CT(70802)=(S=13901376,E=13901407,C=52)
	CT(70803)=(S=13901408,E=13901439,C=12)
	CT(70804)=(S=13901440,E=13901503,C=18)
	CT(70805)=(S=13901504,E=13901567,C=48)
	CT(70806)=(S=13901568,E=13901599,C=37)
	CT(70807)=(S=13901600,E=13901631,C=25)
	CT(70808)=(S=13901632,E=13901663,C=42)
	CT(70809)=(S=13901664,E=13901695,C=32)
	CT(70810)=(S=13901696,E=13901727,C=14)
	CT(70811)=(S=13901728,E=13901759,C=19)
	CT(70812)=(S=13901760,E=13901791,C=25)
	CT(70813)=(S=13901792,E=13901823,C=15)
	CT(70814)=(S=13901824,E=13901951,C=14)
	CT(70815)=(S=13901952,E=13902015,C=19)
	CT(70816)=(S=13902016,E=13902047,C=18)
	CT(70817)=(S=13902048,E=13902079,C=36)
	CT(70818)=(S=13902080,E=13902111,C=18)
	CT(70819)=(S=13902112,E=13902143,C=42)
	CT(70820)=(S=13902144,E=13902175,C=58)
	CT(70821)=(S=13902176,E=13902207,C=43)
	CT(70822)=(S=13902208,E=13902239,C=27)
	CT(70823)=(S=13902240,E=13902271,C=23)
	CT(70824)=(S=13902272,E=13902303,C=26)
	CT(70825)=(S=13902304,E=13902335,C=18)
	CT(70826)=(S=13902336,E=13902367,C=61)
	CT(70827)=(S=13902368,E=13902399,C=18)
	CT(70828)=(S=13902400,E=13902431,C=28)
	CT(70829)=(S=13902432,E=13902463,C=18)
	CT(70830)=(S=13902464,E=13902495,C=25)
	CT(70831)=(S=13902496,E=13902527,C=28)
	CT(70832)=(S=13902528,E=13902559,C=15)
	CT(70833)=(S=13902560,E=13902591,C=66)
	CT(70834)=(S=13902592,E=13902655,C=34)
	CT(70835)=(S=13902656,E=13902687,C=61)
	CT(70836)=(S=13902688,E=13902719,C=14)
	CT(70837)=(S=13902720,E=13902751,C=38)
	CT(70838)=(S=13902752,E=13902783,C=18)
	CT(70839)=(S=13902784,E=13902815,C=15)
	CT(70840)=(S=13902816,E=13902847,C=14)
	CT(70841)=(S=13902848,E=13902879,C=51)
	CT(70842)=(S=13902880,E=13902911,C=14)
	CT(70843)=(S=13902912,E=13902943,C=25)
	CT(70844)=(S=13902944,E=13903039,C=14)
	CT(70845)=(S=13903040,E=13903071,C=39)
	CT(70846)=(S=13903072,E=13903103,C=18)
	CT(70847)=(S=13903104,E=13903135,C=19)
	CT(70848)=(S=13903136,E=13903167,C=28)
	CT(70849)=(S=13903168,E=13903199,C=71)
	CT(70850)=(S=13903200,E=13903231,C=19)
	CT(70851)=(S=13903232,E=13903263,C=24)
	CT(70852)=(S=13903264,E=13903295,C=28)
	CT(70853)=(S=13903296,E=13903327,C=12)
	CT(70854)=(S=13903328,E=13903359,C=27)
	CT(70855)=(S=13903360,E=13903391,C=28)
	CT(70856)=(S=13903392,E=13903423,C=15)
	CT(70857)=(S=13903424,E=13903455,C=14)
	CT(70858)=(S=13903456,E=13903487,C=18)
	CT(70859)=(S=13903488,E=13903519,C=61)
	CT(70860)=(S=13903520,E=13903551,C=14)
	CT(70861)=(S=13903552,E=13903583,C=58)
	CT(70862)=(S=13903584,E=13903615,C=47)
	CT(70863)=(S=13903616,E=13903647,C=15)
	CT(70864)=(S=13903648,E=13903679,C=19)
	CT(70865)=(S=13903680,E=13903711,C=51)
	CT(70866)=(S=13903712,E=13903743,C=59)
	CT(70867)=(S=13903744,E=13903775,C=12)
	CT(70868)=(S=13903776,E=13903807,C=14)
	CT(70869)=(S=13903808,E=13903839,C=42)
	CT(70870)=(S=13903840,E=13903871,C=14)
	CT(70871)=(S=13903872,E=13903903,C=34)
	CT(70872)=(S=13903904,E=13903935,C=31)
	CT(70873)=(S=13903936,E=13903999,C=50)
	CT(70874)=(S=13904000,E=13904031,C=39)
	CT(70875)=(S=13904032,E=13904063,C=28)
	CT(70876)=(S=13904064,E=13904095,C=18)
	CT(70877)=(S=13904096,E=13904127,C=25)
	CT(70878)=(S=13904128,E=13904191,C=18)
	CT(70879)=(S=13904192,E=13904255,C=34)
	CT(70880)=(S=13904256,E=13904287,C=25)
	CT(70881)=(S=13904288,E=13904319,C=14)
	CT(70882)=(S=13904320,E=13904351,C=34)
	CT(70883)=(S=13904352,E=13904383,C=42)
	CT(70884)=(S=13904384,E=13904415,C=14)
	CT(70885)=(S=13904416,E=13904447,C=18)
	CT(70886)=(S=13904448,E=13904479,C=31)
	CT(70887)=(S=13904480,E=13904511,C=62)
	CT(70888)=(S=13904512,E=13904543,C=34)
	CT(70889)=(S=13904544,E=13904575,C=14)
	CT(70890)=(S=13904576,E=13904607,C=66)
	CT(70891)=(S=13904608,E=13904639,C=28)
	CT(70892)=(S=13904640,E=13904671,C=101)
	CT(70893)=(S=13904672,E=13904703,C=51)
	CT(70894)=(S=13904704,E=13904735,C=28)
	CT(70895)=(S=13904736,E=13904767,C=15)
	CT(70896)=(S=13904768,E=13904799,C=34)
	CT(70897)=(S=13904800,E=13904831,C=14)
	CT(70898)=(S=13904832,E=13904895,C=12)
	CT(70899)=(S=13904896,E=13904959,C=14)
	CT(70900)=(S=13904960,E=13904991,C=18)
	CT(70901)=(S=13904992,E=13905023,C=52)
	CT(70902)=(S=13905024,E=13905055,C=18)
	CT(70903)=(S=13905056,E=13905119,C=28)
	CT(70904)=(S=13905120,E=13905151,C=12)
	CT(70905)=(S=13905152,E=13905183,C=18)
	CT(70906)=(S=13905184,E=13905215,C=36)
	CT(70907)=(S=13905216,E=13905247,C=51)
	CT(70908)=(S=13905248,E=13905255,C=15)
	CT(70909)=(S=13905256,E=13905263,C=28)
	CT(70910)=(S=13905264,E=13905279,C=24)
	CT(70911)=(S=13905280,E=13905311,C=15)
	CT(70912)=(S=13905312,E=13905343,C=27)
	CT(70913)=(S=13905344,E=13905375,C=34)
	CT(70914)=(S=13905376,E=13905439,C=18)
	CT(70915)=(S=13905440,E=13905471,C=23)
	CT(70916)=(S=13905472,E=13905503,C=15)
	CT(70917)=(S=13905504,E=13905535,C=28)
	CT(70918)=(S=13905536,E=13905567,C=14)
	CT(70919)=(S=13905568,E=13905599,C=15)
	CT(70920)=(S=13905600,E=13905663,C=18)
	CT(70921)=(S=13905664,E=13905695,C=33)
	CT(70922)=(S=13905696,E=13905727,C=15)
	CT(70923)=(S=13905728,E=13905759,C=14)
	CT(70924)=(S=13905760,E=13905791,C=44)
	CT(70925)=(S=13905792,E=13905823,C=41)
	CT(70926)=(S=13905824,E=13905855,C=34)
	CT(70927)=(S=13905856,E=13905887,C=28)
	CT(70928)=(S=13905888,E=13905919,C=12)
	CT(70929)=(S=13905920,E=13905951,C=15)
	CT(70930)=(S=13905952,E=13905983,C=18)
	CT(70931)=(S=13905984,E=13906015,C=14)
	CT(70932)=(S=13906016,E=13906047,C=28)
	CT(70933)=(S=13906048,E=13906079,C=18)
	CT(70934)=(S=13906080,E=13906111,C=15)
	CT(70935)=(S=13906112,E=13906159,C=18)
	CT(70936)=(S=13906160,E=13906175,C=50)
	CT(70937)=(S=13906176,E=13906207,C=55)
	CT(70938)=(S=13906208,E=13906239,C=58)
	CT(70939)=(S=13906240,E=13906271,C=109)
	CT(70940)=(S=13906272,E=13906303,C=18)
	CT(70941)=(S=13906304,E=13906367,C=25)
	CT(70942)=(S=13906368,E=13906431,C=14)
	CT(70943)=(S=13906432,E=13906463,C=51)
	CT(70944)=(S=13906464,E=13906495,C=37)
	CT(70945)=(S=13906496,E=13906527,C=51)
	CT(70946)=(S=13906528,E=13906559,C=32)
	CT(70947)=(S=13906560,E=13906591,C=47)
	CT(70948)=(S=13906592,E=13906623,C=14)
	CT(70949)=(S=13906624,E=13906655,C=47)
	CT(70950)=(S=13906656,E=13906687,C=26)
	CT(70951)=(S=13906688,E=13906719,C=28)
	CT(70952)=(S=13906720,E=13906751,C=25)
	CT(70953)=(S=13906752,E=13906815,C=50)
	CT(70954)=(S=13906816,E=13906847,C=34)
	CT(70955)=(S=13906848,E=13906879,C=12)
	CT(70956)=(S=13906880,E=13906911,C=58)
	CT(70957)=(S=13906912,E=13906943,C=42)
	CT(70958)=(S=13906944,E=13906975,C=36)
	CT(70959)=(S=13906976,E=13907007,C=44)
	CT(70960)=(S=13907008,E=13907071,C=15)
	CT(70961)=(S=13907072,E=13907103,C=139)
	CT(70962)=(S=13907104,E=13907135,C=50)
	CT(70963)=(S=13907136,E=13907167,C=42)
	CT(70964)=(S=13907168,E=13907199,C=28)
	CT(70965)=(S=13907200,E=13907231,C=49)
	CT(70966)=(S=13907232,E=13907263,C=18)
	CT(70967)=(S=13907264,E=13907295,C=14)
	CT(70968)=(S=13907296,E=13907327,C=34)
	CT(70969)=(S=13907328,E=13907455,C=28)
	CT(70970)=(S=13907456,E=13907487,C=47)
	CT(70971)=(S=13907488,E=13907519,C=36)
	CT(70972)=(S=13907520,E=13907551,C=24)
	CT(70973)=(S=13907552,E=13907583,C=38)
	CT(70974)=(S=13907584,E=13907647,C=14)
	CT(70975)=(S=13907648,E=13907679,C=22)
	CT(70976)=(S=13907680,E=13907711,C=34)
	CT(70977)=(S=13907712,E=13907743,C=25)
	CT(70978)=(S=13907744,E=13907775,C=171)
	CT(70979)=(S=13907776,E=13907807,C=31)
	CT(70980)=(S=13907808,E=13907839,C=18)
	CT(70981)=(S=13907840,E=13907903,C=21)
	CT(70982)=(S=13907904,E=13907935,C=34)
	CT(70983)=(S=13907936,E=13907967,C=71)
	CT(70984)=(S=13907968,E=13907999,C=51)
	CT(70985)=(S=13908000,E=13908095,C=14)
	CT(70986)=(S=13908096,E=13908127,C=72)
	CT(70987)=(S=13908128,E=13908159,C=15)
	CT(70988)=(S=13908160,E=13908191,C=53)
	CT(70989)=(S=13908192,E=13908223,C=70)
	CT(70990)=(S=13908224,E=13908255,C=37)
	CT(70991)=(S=13908256,E=13908263,C=71)
	CT(70992)=(S=13908264,E=13908271,C=46)
	CT(70993)=(S=13908272,E=13908287,C=36)
	CT(70994)=(S=13908288,E=13908319,C=21)
	CT(70995)=(S=13908320,E=13908415,C=18)
	CT(70996)=(S=13908416,E=13908447,C=23)
	CT(70997)=(S=13908448,E=13908479,C=14)
	CT(70998)=(S=13908480,E=13908511,C=37)
	CT(70999)=(S=13908512,E=13908543,C=14)
	CT(71000)=(S=13908544,E=13908575,C=28)
	CT(71001)=(S=13908576,E=13908607,C=40)
	CT(71002)=(S=13908608,E=13908639,C=14)
	CT(71003)=(S=13908640,E=13908671,C=31)
	CT(71004)=(S=13908672,E=13908703,C=18)
	CT(71005)=(S=13908704,E=13908735,C=14)
	CT(71006)=(S=13908736,E=13908767,C=44)
	CT(71007)=(S=13908768,E=13908799,C=28)
	CT(71008)=(S=13908800,E=13908831,C=14)
	CT(71009)=(S=13908832,E=13908863,C=18)
	CT(71010)=(S=13908864,E=13908927,C=34)
	CT(71011)=(S=13908928,E=13908959,C=25)
	CT(71012)=(S=13908960,E=13908991,C=58)
	CT(71013)=(S=13909024,E=13909055,C=34)
	CT(71014)=(S=13909056,E=13909087,C=132)
	CT(71015)=(S=13909088,E=13909119,C=24)
	CT(71016)=(S=13909120,E=13909151,C=28)
	CT(71017)=(S=13909152,E=13909183,C=42)
	CT(71018)=(S=13909184,E=13909247,C=28)
	CT(71019)=(S=13909248,E=13909503,C=36)
	CT(71020)=(S=13909504,E=13909535,C=14)
	CT(71021)=(S=13909536,E=13909567,C=46)
	CT(71022)=(S=13909568,E=13909599,C=28)
	CT(71023)=(S=13909600,E=13909631,C=23)
	CT(71024)=(S=13909632,E=13909695,C=12)
	CT(71025)=(S=13909696,E=13909727,C=28)
	CT(71026)=(S=13909728,E=13909759,C=27)
	CT(71027)=(S=13909760,E=13909791,C=47)
	CT(71028)=(S=13909792,E=13909855,C=28)
	CT(71029)=(S=13909856,E=13909887,C=25)
	CT(71030)=(S=13909888,E=13909951,C=28)
	CT(71031)=(S=13909952,E=13909983,C=19)
	CT(71032)=(S=13909984,E=13910015,C=49)
	CT(71033)=(S=13910016,E=13910143,C=36)
	CT(71034)=(S=13910144,E=13910175,C=14)
	CT(71035)=(S=13910176,E=13910207,C=25)
	CT(71036)=(S=13910208,E=13910239,C=37)
	CT(71037)=(S=13910240,E=13910271,C=42)
	CT(71038)=(S=13910272,E=13910303,C=28)
	CT(71039)=(S=13910304,E=13910335,C=49)
	CT(71040)=(S=13910336,E=13910367,C=18)
	CT(71041)=(S=13910368,E=13910399,C=72)
	CT(71042)=(S=13910400,E=13910431,C=37)
	CT(71043)=(S=13910432,E=13910463,C=34)
	CT(71044)=(S=13910464,E=13910527,C=33)
	CT(71045)=(S=13910528,E=13910559,C=28)
	CT(71046)=(S=13910560,E=13910591,C=31)
	CT(71047)=(S=13910592,E=13910623,C=70)
	CT(71048)=(S=13910624,E=13910655,C=15)
	CT(71049)=(S=13910656,E=13910687,C=28)
	CT(71050)=(S=13910688,E=13910719,C=25)
	CT(71051)=(S=13910720,E=13910751,C=47)
	CT(71052)=(S=13910752,E=13910783,C=15)
	CT(71053)=(S=13910784,E=13910815,C=18)
	CT(71054)=(S=13910816,E=13910831,C=12)
	CT(71055)=(S=13910832,E=13910847,C=27)
	CT(71056)=(S=13910848,E=13910879,C=33)
	CT(71057)=(S=13910880,E=13910911,C=14)
	CT(71058)=(S=13910912,E=13910943,C=58)
	CT(71059)=(S=13910944,E=13910975,C=36)
	CT(71060)=(S=13910976,E=13911007,C=14)
	CT(71061)=(S=13911008,E=13911039,C=42)
	CT(71062)=(S=13911040,E=13911071,C=47)
	CT(71063)=(S=13911072,E=13911103,C=37)
	CT(71064)=(S=13911104,E=13911167,C=28)
	CT(71065)=(S=13911168,E=13911199,C=17)
	CT(71066)=(S=13911200,E=13911231,C=31)
	CT(71067)=(S=13911232,E=13911295,C=49)
	CT(71068)=(S=13911296,E=13911327,C=46)
	CT(71069)=(S=13911328,E=13911359,C=14)
	CT(71070)=(S=13911360,E=13911391,C=58)
	CT(71071)=(S=13911392,E=13911423,C=18)
	CT(71072)=(S=13911424,E=13911427,C=15)
	CT(71073)=(S=13911428,E=13911431,C=13)
	CT(71074)=(S=13911432,E=13911455,C=15)
	CT(71075)=(S=13911456,E=13911487,C=12)
	CT(71076)=(S=13911488,E=13911551,C=14)
	CT(71077)=(S=13911552,E=13911583,C=24)
	CT(71078)=(S=13911584,E=13911615,C=23)
	CT(71079)=(S=13911616,E=13911647,C=14)
	CT(71080)=(S=13911648,E=13911679,C=94)
	CT(71081)=(S=13911680,E=13911711,C=51)
	CT(71082)=(S=13911712,E=13911743,C=60)
	CT(71083)=(S=13911744,E=13911775,C=22)
	CT(71084)=(S=13911776,E=13911807,C=15)
	CT(71085)=(S=13911808,E=13911839,C=49)
	CT(71086)=(S=13911840,E=13911871,C=23)
	CT(71087)=(S=13911872,E=13911903,C=27)
	CT(71088)=(S=13911904,E=13911935,C=34)
	CT(71089)=(S=13911936,E=13911999,C=33)
	CT(71090)=(S=13912000,E=13912031,C=28)
	CT(71091)=(S=13912032,E=13912063,C=14)
	CT(71092)=(S=13912064,E=13912095,C=45)
	CT(71093)=(S=13912096,E=13912127,C=14)
	CT(71094)=(S=13912128,E=13912159,C=28)
	CT(71095)=(S=13912160,E=13912191,C=52)
	CT(71096)=(S=13912192,E=13912223,C=40)
	CT(71097)=(S=13912224,E=13912255,C=28)
	CT(71098)=(S=13912256,E=13912287,C=51)
	CT(71099)=(S=13912288,E=13912319,C=36)
	CT(71100)=(S=13912320,E=13912351,C=19)
	CT(71101)=(S=13912352,E=13912383,C=25)
	CT(71102)=(S=13912384,E=13912415,C=66)
	CT(71103)=(S=13912416,E=13912447,C=18)
	CT(71104)=(S=13912448,E=13912479,C=51)
	CT(71105)=(S=13912480,E=13912511,C=19)
	CT(71106)=(S=13912512,E=13912703,C=14)
	CT(71107)=(S=13912704,E=13912767,C=34)
	CT(71108)=(S=13912768,E=13912831,C=18)
	CT(71109)=(S=13912832,E=13912863,C=51)
	CT(71110)=(S=13912864,E=13912895,C=28)
	CT(71111)=(S=13912896,E=13912927,C=19)
	CT(71112)=(S=13912928,E=13912959,C=58)
	CT(71113)=(S=13912960,E=13912991,C=18)
	CT(71114)=(S=13912992,E=13913023,C=25)
	CT(71115)=(S=13913024,E=13913087,C=18)
	CT(71116)=(S=13913088,E=13913119,C=20)
	CT(71117)=(S=13913120,E=13913151,C=58)
	CT(71118)=(S=13913152,E=13913183,C=23)
	CT(71119)=(S=13913184,E=13913215,C=17)
	CT(71120)=(S=13913216,E=13913279,C=18)
	CT(71121)=(S=13913280,E=13913311,C=28)
	CT(71122)=(S=13913312,E=13913343,C=49)
	CT(71123)=(S=13913344,E=13913375,C=169)
	CT(71124)=(S=13913376,E=13913407,C=56)
	CT(71125)=(S=13913408,E=13913439,C=15)
	CT(71126)=(S=13913440,E=13913471,C=58)
	CT(71127)=(S=13913472,E=13913503,C=18)
	CT(71128)=(S=13913504,E=13913535,C=28)
	CT(71129)=(S=13913536,E=13913567,C=94)
	CT(71130)=(S=13913568,E=13913599,C=28)
	CT(71131)=(S=13913600,E=13913631,C=15)
	CT(71132)=(S=13913632,E=13913663,C=12)
	CT(71133)=(S=13913664,E=13913695,C=14)
	CT(71134)=(S=13913696,E=13913727,C=28)
	CT(71135)=(S=13913728,E=13913759,C=25)
	CT(71136)=(S=13913760,E=13913791,C=14)
	CT(71137)=(S=13913792,E=13913823,C=36)
	CT(71138)=(S=13913824,E=13913855,C=63)
	CT(71139)=(S=13913856,E=13913919,C=28)
	CT(71140)=(S=13913920,E=13913951,C=49)
	CT(71141)=(S=13913952,E=13913967,C=33)
	CT(71142)=(S=13913968,E=13913983,C=31)
	CT(71143)=(S=13913984,E=13914015,C=25)
	CT(71144)=(S=13914016,E=13914079,C=28)
	CT(71145)=(S=13914080,E=13914111,C=36)
	CT(71146)=(S=13914112,E=13914143,C=26)
	CT(71147)=(S=13914144,E=13914175,C=31)
	CT(71148)=(S=13914176,E=13914207,C=33)
	CT(71149)=(S=13914208,E=13914239,C=34)
	CT(71150)=(S=13914240,E=13914303,C=25)
	CT(71151)=(S=13914336,E=13914367,C=28)
	CT(71152)=(S=13914368,E=13914399,C=71)
	CT(71153)=(S=13914432,E=13914495,C=14)
	CT(71154)=(S=13914496,E=13914623,C=25)
	CT(71155)=(S=13914624,E=13914655,C=14)
	CT(71156)=(S=13914656,E=13914687,C=28)
	CT(71157)=(S=13914688,E=13914751,C=14)
	CT(71158)=(S=13914752,E=13914783,C=12)
	CT(71159)=(S=13914784,E=13914815,C=28)
	CT(71160)=(S=13914816,E=13914847,C=31)
	CT(71161)=(S=13914848,E=13914879,C=28)
	CT(71162)=(S=13914880,E=13914911,C=18)
	CT(71163)=(S=13914912,E=13914943,C=28)
	CT(71164)=(S=13914944,E=13914975,C=36)
	CT(71165)=(S=13914976,E=13915007,C=47)
	CT(71166)=(S=13915008,E=13915071,C=12)
	CT(71167)=(S=13915072,E=13915167,C=36)
	CT(71168)=(S=13915168,E=13915175,C=15)
	CT(71169)=(S=13915176,E=13915183,C=63)
	CT(71170)=(S=13915184,E=13915191,C=15)
	CT(71171)=(S=13915192,E=13915199,C=12)
	CT(71172)=(S=13915200,E=13915263,C=14)
	CT(71173)=(S=13915264,E=13915295,C=36)
	CT(71174)=(S=13915296,E=13915327,C=14)
	CT(71175)=(S=13915328,E=13915391,C=28)
	CT(71176)=(S=13915392,E=13915423,C=14)
	CT(71177)=(S=13915424,E=13915455,C=25)
	CT(71178)=(S=13915456,E=13915487,C=19)
	CT(71179)=(S=13915488,E=13915519,C=58)
	CT(71180)=(S=13915520,E=13915551,C=12)
	CT(71181)=(S=13915552,E=13915583,C=52)
	CT(71182)=(S=13915584,E=13915615,C=110)
	CT(71183)=(S=13915616,E=13915647,C=14)
	CT(71184)=(S=13915648,E=13915679,C=47)
	CT(71185)=(S=13915680,E=13915711,C=28)
	CT(71186)=(S=13915712,E=13915743,C=26)
	CT(71187)=(S=13915744,E=13915775,C=31)
	CT(71188)=(S=13915776,E=13915871,C=28)
	CT(71189)=(S=13915872,E=13915903,C=31)
	CT(71190)=(S=13915904,E=13915935,C=58)
	CT(71191)=(S=13915936,E=13915967,C=28)
	CT(71192)=(S=13915968,E=13915999,C=14)
	CT(71193)=(S=13916000,E=13916031,C=49)
	CT(71194)=(S=13916032,E=13916063,C=28)
	CT(71195)=(S=13916064,E=13916095,C=31)
	CT(71196)=(S=13916128,E=13916159,C=58)
	CT(71197)=(S=13916160,E=13916191,C=42)
	CT(71198)=(S=13916192,E=13916223,C=14)
	CT(71199)=(S=13916224,E=13916255,C=24)
	CT(71200)=(S=13916256,E=13916287,C=130)
	CT(71201)=(S=13916288,E=13916319,C=28)
	CT(71202)=(S=13916320,E=13916351,C=42)
	CT(71203)=(S=13916352,E=13916383,C=28)
	CT(71204)=(S=13916384,E=13916415,C=49)
	CT(71205)=(S=13916416,E=13916447,C=25)
	CT(71206)=(S=13916448,E=13916479,C=27)
	CT(71207)=(S=13916480,E=13916511,C=14)
	CT(71208)=(S=13916512,E=13916575,C=28)
	CT(71209)=(S=13916576,E=13916607,C=42)
	CT(71210)=(S=13916608,E=13916639,C=28)
	CT(71211)=(S=13916640,E=13916671,C=71)
	CT(71212)=(S=13916672,E=13916703,C=15)
	CT(71213)=(S=13916704,E=13916735,C=31)
	CT(71214)=(S=13916736,E=13916767,C=47)
	CT(71215)=(S=13916800,E=13916831,C=28)
	CT(71216)=(S=13916832,E=13916863,C=31)
	CT(71217)=(S=13916864,E=13916895,C=34)
	CT(71218)=(S=13916896,E=13916927,C=33)
	CT(71219)=(S=13916928,E=13916959,C=58)
	CT(71220)=(S=13916960,E=13917023,C=15)
	CT(71221)=(S=13917024,E=13917055,C=57)
	CT(71222)=(S=13917056,E=13917087,C=19)
	CT(71223)=(S=13917088,E=13917119,C=51)
	CT(71224)=(S=13917120,E=13917151,C=18)
	CT(71225)=(S=13917152,E=13917183,C=28)
	CT(71226)=(S=13917184,E=13917215,C=50)
	CT(71227)=(S=13917216,E=13917247,C=25)
	CT(71228)=(S=13917248,E=13917279,C=36)
	CT(71229)=(S=13917280,E=13917375,C=18)
	CT(71230)=(S=13917376,E=13917407,C=57)
	CT(71231)=(S=13917408,E=13917439,C=31)
	CT(71232)=(S=13917440,E=13917471,C=28)
	CT(71233)=(S=13917472,E=13917503,C=24)
	CT(71234)=(S=13917504,E=13917535,C=14)
	CT(71235)=(S=13917536,E=13917567,C=60)
	CT(71236)=(S=13917568,E=13917599,C=38)
	CT(71237)=(S=13917600,E=13917663,C=23)
	CT(71238)=(S=13917664,E=13917695,C=52)
	CT(71239)=(S=13917696,E=13917727,C=18)
	CT(71240)=(S=13917728,E=13917759,C=34)
	CT(71241)=(S=13917760,E=13917791,C=47)
	CT(71242)=(S=13917792,E=13917823,C=18)
	CT(71243)=(S=13917824,E=13917855,C=15)
	CT(71244)=(S=13917856,E=13917919,C=12)
	CT(71245)=(S=13917920,E=13917951,C=28)
	CT(71246)=(S=13917952,E=13917983,C=42)
	CT(71247)=(S=13917984,E=13918015,C=28)
	CT(71248)=(S=13918016,E=13918047,C=12)
	CT(71249)=(S=13918048,E=13918079,C=28)
	CT(71250)=(S=13918080,E=13918111,C=26)
	CT(71251)=(S=13918112,E=13918143,C=51)
	CT(71252)=(S=13918144,E=13918175,C=25)
	CT(71253)=(S=13918176,E=13918207,C=14)
	CT(71254)=(S=13918208,E=13918239,C=110)
	CT(71255)=(S=13918240,E=13918271,C=50)
	CT(71256)=(S=13918272,E=13918303,C=19)
	CT(71257)=(S=13918304,E=13918335,C=18)
	CT(71258)=(S=13918336,E=13918367,C=28)
	CT(71259)=(S=13918368,E=13918399,C=33)
	CT(71260)=(S=13918400,E=13918431,C=18)
	CT(71261)=(S=13918432,E=13918463,C=58)
	CT(71262)=(S=13918464,E=13918495,C=62)
	CT(71263)=(S=13918496,E=13918527,C=15)
	CT(71264)=(S=13918528,E=13918559,C=14)
	CT(71265)=(S=13918560,E=13918591,C=28)
	CT(71266)=(S=13918592,E=13918623,C=24)
	CT(71267)=(S=13918624,E=13918655,C=25)
	CT(71268)=(S=13918656,E=13918719,C=24)
	CT(71269)=(S=13918720,E=13918751,C=37)
	CT(71270)=(S=13918752,E=13918783,C=34)
	CT(71271)=(S=13918784,E=13918847,C=24)
	CT(71272)=(S=13918848,E=13918879,C=39)
	CT(71273)=(S=13918880,E=13918911,C=68)
	CT(71274)=(S=13918912,E=13918975,C=37)
	CT(71275)=(S=13918976,E=13919103,C=12)
	CT(71276)=(S=13919104,E=13919199,C=28)
	CT(71277)=(S=13919200,E=13919231,C=24)
	CT(71278)=(S=13919232,E=13919263,C=13)
	CT(71279)=(S=13919264,E=13919295,C=28)
	CT(71280)=(S=13919296,E=13919327,C=111)
	CT(71281)=(S=13919328,E=13919359,C=14)
	CT(71282)=(S=13919360,E=13919391,C=18)
	CT(71283)=(S=13919392,E=13919423,C=49)
	CT(71284)=(S=13919424,E=13919455,C=23)
	CT(71285)=(S=13919456,E=13919487,C=14)
	CT(71286)=(S=13919488,E=13919519,C=34)
	CT(71287)=(S=13919520,E=13919551,C=28)
	CT(71288)=(S=13919552,E=13919583,C=25)
	CT(71289)=(S=13919584,E=13919615,C=37)
	CT(71290)=(S=13919616,E=13919679,C=52)
	CT(71291)=(S=13919680,E=13919711,C=28)
	CT(71292)=(S=13919712,E=13919743,C=39)
	CT(71293)=(S=13919744,E=13919775,C=23)
	CT(71294)=(S=13919776,E=13919839,C=15)
	CT(71295)=(S=13919840,E=13919871,C=14)
	CT(71296)=(S=13919872,E=13919903,C=46)
	CT(71297)=(S=13919904,E=13919935,C=28)
	CT(71298)=(S=13919936,E=13919967,C=14)
	CT(71299)=(S=13919968,E=13919999,C=28)
	CT(71300)=(S=13920000,E=13920031,C=12)
	CT(71301)=(S=13920064,E=13920095,C=34)
	CT(71302)=(S=13920096,E=13920127,C=18)
	CT(71303)=(S=13920128,E=13920159,C=52)
	CT(71304)=(S=13920160,E=13920191,C=106)
	CT(71305)=(S=13920192,E=13920223,C=15)
	CT(71306)=(S=13920224,E=13920255,C=14)
	CT(71307)=(S=13920256,E=13920319,C=15)
	CT(71308)=(S=13920320,E=13920351,C=18)
	CT(71309)=(S=13920352,E=13920383,C=51)
	CT(71310)=(S=13920384,E=13920415,C=14)
	CT(71311)=(S=13920416,E=13920447,C=25)
	CT(71312)=(S=13920448,E=13920511,C=36)
	CT(71313)=(S=13920512,E=13920639,C=19)
	CT(71314)=(S=13920640,E=13920671,C=49)
	CT(71315)=(S=13920672,E=13920703,C=14)
	CT(71316)=(S=13920704,E=13920735,C=28)
	CT(71317)=(S=13920736,E=13920767,C=50)
	CT(71318)=(S=13920768,E=13920799,C=58)
	CT(71319)=(S=13920800,E=13920831,C=18)
	CT(71320)=(S=13920832,E=13920863,C=43)
	CT(71321)=(S=13920864,E=13920895,C=14)
	CT(71322)=(S=13920896,E=13920959,C=58)
	CT(71323)=(S=13920960,E=13921023,C=25)
	CT(71324)=(S=13921056,E=13921087,C=55)
	CT(71325)=(S=13921088,E=13921119,C=12)
	CT(71326)=(S=13921120,E=13921151,C=23)
	CT(71327)=(S=13921152,E=13921183,C=19)
	CT(71328)=(S=13921184,E=13921215,C=28)
	CT(71329)=(S=13921216,E=13921279,C=18)
	CT(71330)=(S=13921280,E=13921311,C=25)
	CT(71331)=(S=13921312,E=13921343,C=42)
	CT(71332)=(S=13921344,E=13921375,C=14)
	CT(71333)=(S=13921376,E=13921407,C=18)
	CT(71334)=(S=13921408,E=13921439,C=37)
	CT(71335)=(S=13921440,E=13921471,C=28)
	CT(71336)=(S=13921472,E=13921535,C=50)
	CT(71337)=(S=13921536,E=13921567,C=18)
	CT(71338)=(S=13921568,E=13921599,C=31)
	CT(71339)=(S=13921600,E=13921631,C=34)
	CT(71340)=(S=13921632,E=13921663,C=37)
	CT(71341)=(S=13921664,E=13921703,C=58)
	CT(71342)=(S=13921704,E=13921711,C=18)
	CT(71343)=(S=13921712,E=13921727,C=33)
	CT(71344)=(S=13921728,E=13921759,C=18)
	CT(71345)=(S=13921760,E=13921791,C=37)
	CT(71346)=(S=13921792,E=13921855,C=15)
	CT(71347)=(S=13921856,E=13921887,C=54)
	CT(71348)=(S=13921888,E=13921919,C=28)
	CT(71349)=(S=13921920,E=13921951,C=31)
	CT(71350)=(S=13921952,E=13921983,C=14)
	CT(71351)=(S=13921984,E=13922047,C=28)
	CT(71352)=(S=13922048,E=13922079,C=33)
	CT(71353)=(S=13922080,E=13922111,C=14)
	CT(71354)=(S=13922112,E=13922143,C=18)
	CT(71355)=(S=13922144,E=13922175,C=25)
	CT(71356)=(S=13922176,E=13922239,C=14)
	CT(71357)=(S=13922240,E=13922271,C=31)
	CT(71358)=(S=13922272,E=13922303,C=28)
	CT(71359)=(S=13922304,E=13922367,C=19)
	CT(71360)=(S=13922368,E=13922399,C=15)
	CT(71361)=(S=13922400,E=13922431,C=62)
	CT(71362)=(S=13922432,E=13922463,C=24)
	CT(71363)=(S=13922464,E=13922495,C=19)
	CT(71364)=(S=13922496,E=13922559,C=28)
	CT(71365)=(S=13922560,E=13922591,C=14)
	CT(71366)=(S=13922592,E=13922623,C=31)
	CT(71367)=(S=13922624,E=13922655,C=49)
	CT(71368)=(S=13922656,E=13922687,C=18)
	CT(71369)=(S=13922688,E=13922719,C=14)
	CT(71370)=(S=13922720,E=13922751,C=21)
	CT(71371)=(S=13922752,E=13922783,C=14)
	CT(71372)=(S=13922784,E=13922847,C=18)
	CT(71373)=(S=13922848,E=13922911,C=28)
	CT(71374)=(S=13922912,E=13922943,C=36)
	CT(71375)=(S=13922944,E=13923071,C=28)
	CT(71376)=(S=13923072,E=13923103,C=37)
	CT(71377)=(S=13923136,E=13923167,C=15)
	CT(71378)=(S=13923200,E=13923263,C=14)
	CT(71379)=(S=13923264,E=13923295,C=36)
	CT(71380)=(S=13923296,E=13923327,C=31)
	CT(71381)=(S=13923328,E=13923359,C=28)
	CT(71382)=(S=13923360,E=13923391,C=47)
	CT(71383)=(S=13923392,E=13923423,C=19)
	CT(71384)=(S=13923424,E=13923455,C=18)
	CT(71385)=(S=13923456,E=13923487,C=51)
	CT(71386)=(S=13923488,E=13923519,C=17)
	CT(71387)=(S=13923520,E=13923551,C=23)
	CT(71388)=(S=13923552,E=13923583,C=20)
	CT(71389)=(S=13923584,E=13923615,C=44)
	CT(71390)=(S=13923616,E=13923647,C=51)
	CT(71391)=(S=13923648,E=13923679,C=28)
	CT(71392)=(S=13923680,E=13923711,C=34)
	CT(71393)=(S=13923712,E=13923743,C=17)
	CT(71394)=(S=13923744,E=13923775,C=70)
	CT(71395)=(S=13923776,E=13923807,C=34)
	CT(71396)=(S=13923808,E=13923839,C=13)
	CT(71397)=(S=13923840,E=13923871,C=61)
	CT(71398)=(S=13923872,E=13923903,C=18)
	CT(71399)=(S=13923904,E=13923935,C=52)
	CT(71400)=(S=13923936,E=13923999,C=23)
	CT(71401)=(S=13924000,E=13924063,C=28)
	CT(71402)=(S=13924064,E=13924127,C=14)
	CT(71403)=(S=13924160,E=13924191,C=23)
	CT(71404)=(S=13924192,E=13924223,C=18)
	CT(71405)=(S=13924224,E=13924255,C=42)
	CT(71406)=(S=13924256,E=13924351,C=18)
	CT(71407)=(S=13924352,E=13924383,C=54)
	CT(71408)=(S=13924384,E=13924415,C=34)
	CT(71409)=(S=13924416,E=13924479,C=36)
	CT(71410)=(S=13924480,E=13924511,C=63)
	CT(71411)=(S=13924512,E=13924543,C=18)
	CT(71412)=(S=13924544,E=13924575,C=26)
	CT(71413)=(S=13924576,E=13924607,C=69)
	CT(71414)=(S=13924608,E=13924671,C=14)
	CT(71415)=(S=13924672,E=13924703,C=15)
	CT(71416)=(S=13924704,E=13924735,C=36)
	CT(71417)=(S=13924736,E=13924767,C=14)
	CT(71418)=(S=13924768,E=13924799,C=12)
	CT(71419)=(S=13924800,E=13924831,C=14)
	CT(71420)=(S=13924832,E=13924863,C=25)
	CT(71421)=(S=13924864,E=13924895,C=18)
	CT(71422)=(S=13924896,E=13924927,C=28)
	CT(71423)=(S=13924928,E=13924959,C=44)
	CT(71424)=(S=13924960,E=13924991,C=25)
	CT(71425)=(S=13924992,E=13925023,C=28)
	CT(71426)=(S=13925024,E=13925055,C=51)
	CT(71427)=(S=13925056,E=13925087,C=58)
	CT(71428)=(S=13925088,E=13925119,C=106)
	CT(71429)=(S=13925120,E=13925151,C=49)
	CT(71430)=(S=13925152,E=13925183,C=28)
	CT(71431)=(S=13925184,E=13925215,C=15)
	CT(71432)=(S=13925216,E=13925247,C=28)
	CT(71433)=(S=13925248,E=13925311,C=36)
	CT(71434)=(S=13925312,E=13925375,C=14)
	CT(71435)=(S=13925376,E=13925407,C=18)
	CT(71436)=(S=13925408,E=13925439,C=28)
	CT(71437)=(S=13925440,E=13925471,C=36)
	CT(71438)=(S=13925472,E=13925503,C=18)
	CT(71439)=(S=13925504,E=13925535,C=42)
	CT(71440)=(S=13925536,E=13925567,C=15)
	CT(71441)=(S=13925568,E=13925599,C=14)
	CT(71442)=(S=13925600,E=13925631,C=42)
	CT(71443)=(S=13925632,E=13925663,C=37)
	CT(71444)=(S=13925664,E=13925695,C=28)
	CT(71445)=(S=13925696,E=13925727,C=14)
	CT(71446)=(S=13925728,E=13925759,C=28)
	CT(71447)=(S=13925760,E=13925791,C=36)
	CT(71448)=(S=13925792,E=13925887,C=27)
	CT(71449)=(S=13925888,E=13925919,C=58)
	CT(71450)=(S=13925920,E=13925951,C=63)
	CT(71451)=(S=13925952,E=13925983,C=42)
	CT(71452)=(S=13925984,E=13926015,C=35)
	CT(71453)=(S=13926016,E=13926047,C=14)
	CT(71454)=(S=13926048,E=13926079,C=34)
	CT(71455)=(S=13926080,E=13926111,C=28)
	CT(71456)=(S=13926112,E=13926143,C=48)
	CT(71457)=(S=13926144,E=13926175,C=14)
	CT(71458)=(S=13926176,E=13926207,C=28)
	CT(71459)=(S=13926208,E=13926239,C=58)
	CT(71460)=(S=13926240,E=13926271,C=37)
	CT(71461)=(S=13926272,E=13926399,C=36)
	CT(71462)=(S=13926400,E=13926655,C=25)
	CT(71463)=(S=13926656,E=13926719,C=12)
	CT(71464)=(S=13926720,E=13926751,C=63)
	CT(71465)=(S=13926752,E=13926783,C=18)
	CT(71466)=(S=13926784,E=13926911,C=36)
	CT(71467)=(S=13926912,E=13927167,C=24)
	CT(71468)=(S=13927168,E=13927423,C=15)
	CT(71469)=(S=13927424,E=13927807,C=14)
	CT(71470)=(S=13927808,E=13927935,C=37)
	CT(71471)=(S=13927936,E=13928447,C=14)
	CT(71472)=(S=13928448,E=13928703,C=36)
	CT(71473)=(S=13928704,E=13928959,C=13)
	CT(71474)=(S=13928960,E=13929215,C=23)
	CT(71475)=(S=13929216,E=13929727,C=14)
	CT(71476)=(S=13929728,E=13929983,C=15)
	CT(71477)=(S=13929984,E=13930047,C=36)
	CT(71478)=(S=13930048,E=13930111,C=60)
	CT(71479)=(S=13930112,E=13930239,C=25)
	CT(71480)=(S=13930240,E=13930495,C=17)
	CT(71481)=(S=13930496,E=13930751,C=28)
	CT(71482)=(S=13930752,E=13931007,C=25)
	CT(71483)=(S=13931008,E=13931071,C=47)
	CT(71484)=(S=13931072,E=13931135,C=38)
	CT(71485)=(S=13931136,E=13931263,C=37)
	CT(71486)=(S=13931264,E=13931391,C=34)
	CT(71487)=(S=13931392,E=13931519,C=63)
	CT(71488)=(S=13931520,E=13931775,C=14)
	CT(71489)=(S=13931776,E=13931839,C=28)
	CT(71490)=(S=13931840,E=13932031,C=47)
	CT(71491)=(S=13932032,E=13932287,C=17)
	CT(71492)=(S=13932288,E=13932575,C=19)
	CT(71493)=(S=13932576,E=13932607,C=18)
	CT(71494)=(S=13932608,E=13932671,C=22)
	CT(71495)=(S=13932672,E=13932799,C=42)
	CT(71496)=(S=13932800,E=13933055,C=36)
	CT(71497)=(S=13933056,E=13933183,C=37)
	CT(71498)=(S=13933184,E=13933311,C=20)
	CT(71499)=(S=13933312,E=13933567,C=12)
	CT(71500)=(S=13933568,E=13933823,C=37)
	CT(71501)=(S=13933824,E=13934079,C=12)
	CT(71502)=(S=13934080,E=13934207,C=13)
	CT(71503)=(S=13934208,E=13934239,C=33)
	CT(71504)=(S=13934240,E=13934255,C=18)
	CT(71505)=(S=13934256,E=13934271,C=54)
	CT(71506)=(S=13934272,E=13934527,C=14)
	CT(71507)=(S=13934528,E=13934591,C=13)
	CT(71508)=(S=13934592,E=13934847,C=58)
	CT(71509)=(S=13934848,E=13934975,C=14)
	CT(71510)=(S=13934976,E=13935103,C=34)
	CT(71511)=(S=13935104,E=13935167,C=14)
	CT(71512)=(S=13935168,E=13935231,C=15)
	CT(71513)=(S=13935232,E=13935263,C=23)
	CT(71514)=(S=13935264,E=13935295,C=19)
	CT(71515)=(S=13935296,E=13935359,C=14)
	CT(71516)=(S=13935360,E=13935615,C=25)
	CT(71517)=(S=13935616,E=13935871,C=18)
	CT(71518)=(S=13935872,E=13935903,C=13)
	CT(71519)=(S=13935904,E=13935935,C=15)
	CT(71520)=(S=13935936,E=13935999,C=36)
	CT(71521)=(S=13936000,E=13936127,C=14)
	CT(71522)=(S=13936128,E=13936191,C=49)
	CT(71523)=(S=13936192,E=13936223,C=25)
	CT(71524)=(S=13936224,E=13936255,C=42)
	CT(71525)=(S=13936256,E=13936383,C=25)
	CT(71526)=(S=13936384,E=13936639,C=13)
	CT(71527)=(S=13936640,E=13936895,C=28)
	CT(71528)=(S=13936896,E=13936959,C=14)
	CT(71529)=(S=13936960,E=13937023,C=27)
	CT(71530)=(S=13937024,E=13937407,C=25)
	CT(71531)=(S=13937408,E=13937663,C=15)
	CT(71532)=(S=13937664,E=13937919,C=28)
	CT(71533)=(S=13937920,E=13938175,C=14)
	CT(71534)=(S=13938176,E=13938687,C=37)
	CT(71535)=(S=13938688,E=13938943,C=18)
	CT(71536)=(S=13938944,E=13939199,C=15)
	CT(71537)=(S=13939200,E=13939231,C=31)
	CT(71538)=(S=13939232,E=13939359,C=36)
	CT(71539)=(S=13939360,E=13939391,C=24)
	CT(71540)=(S=13939392,E=13939423,C=36)
	CT(71541)=(S=13939424,E=13939455,C=46)
	CT(71542)=(S=13939456,E=13939711,C=17)
	CT(71543)=(S=13939712,E=13939839,C=12)
	CT(71544)=(S=13939840,E=13939967,C=58)
	CT(71545)=(S=13939968,E=13940223,C=19)
	CT(71546)=(S=13940224,E=13940351,C=58)
	CT(71547)=(S=13940352,E=13940415,C=36)
	CT(71548)=(S=13940416,E=13940479,C=47)
	CT(71549)=(S=13940480,E=13940607,C=42)
	CT(71550)=(S=13940608,E=13940639,C=14)
	CT(71551)=(S=13940640,E=13940671,C=15)
	CT(71552)=(S=13940672,E=13940735,C=25)
	CT(71553)=(S=13940736,E=13941247,C=28)
	CT(71554)=(S=13941248,E=13941503,C=42)
	CT(71555)=(S=13941504,E=13941631,C=36)
	CT(71556)=(S=13941632,E=13941759,C=14)
	CT(71557)=(S=13941760,E=13941887,C=18)
	CT(71558)=(S=13941888,E=13942015,C=14)
	CT(71559)=(S=13942016,E=13942143,C=36)
	CT(71560)=(S=13942144,E=13942271,C=15)
	CT(71561)=(S=13942272,E=13942527,C=49)
	CT(71562)=(S=13942528,E=13942783,C=58)
	CT(71563)=(S=13942784,E=13943295,C=18)
	CT(71564)=(S=13943296,E=13943807,C=12)
	CT(71565)=(S=13943808,E=13944063,C=14)
	CT(71566)=(S=13944064,E=13944191,C=71)
	CT(71567)=(S=13944192,E=13944255,C=42)
	CT(71568)=(S=13944256,E=13944575,C=12)
	CT(71569)=(S=13944576,E=13944831,C=17)
	CT(71570)=(S=13944832,E=13945087,C=46)
	CT(71571)=(S=13945088,E=13945599,C=28)
	CT(71572)=(S=13945600,E=13945631,C=36)
	CT(71573)=(S=13945632,E=13945727,C=34)
	CT(71574)=(S=13945728,E=13945855,C=58)
	CT(71575)=(S=13945856,E=13945983,C=28)
	CT(71576)=(S=13945984,E=13946111,C=36)
	CT(71577)=(S=13946112,E=13946367,C=22)
	CT(71578)=(S=13946368,E=13946623,C=36)
	CT(71579)=(S=13946624,E=13946879,C=14)
	CT(71580)=(S=13946880,E=13947135,C=12)
	CT(71581)=(S=13947136,E=13947391,C=19)
	CT(71582)=(S=13947392,E=13947647,C=15)
	CT(71583)=(S=13947648,E=13947775,C=14)
	CT(71584)=(S=13947776,E=13947903,C=28)
	CT(71585)=(S=13947904,E=13948159,C=14)
	CT(71586)=(S=13948160,E=13948415,C=47)
	CT(71587)=(S=13948416,E=13948671,C=19)
	CT(71588)=(S=13948672,E=13948799,C=34)
	CT(71589)=(S=13948800,E=13948927,C=23)
	CT(71590)=(S=13948928,E=13949183,C=15)
	CT(71591)=(S=13949184,E=13949311,C=134)
	CT(71592)=(S=13949312,E=13949439,C=19)
	CT(71593)=(S=13949440,E=13949695,C=28)
	CT(71594)=(S=13949696,E=13949951,C=14)
	CT(71595)=(S=13949952,E=13950207,C=18)
	CT(71596)=(S=13950208,E=13950335,C=49)
	CT(71597)=(S=13950336,E=13950463,C=28)
	CT(71598)=(S=13950464,E=13950719,C=14)
	CT(71599)=(S=13950720,E=13951103,C=28)
	CT(71600)=(S=13951104,E=13951231,C=49)
	CT(71601)=(S=13951232,E=13951359,C=14)
	CT(71602)=(S=13951360,E=13951487,C=25)
	CT(71603)=(S=13951488,E=13951743,C=47)
	CT(71604)=(S=13951744,E=13951999,C=28)
	CT(71605)=(S=13952000,E=13952511,C=14)
	CT(71606)=(S=13952512,E=13953023,C=25)
	CT(71607)=(S=13953024,E=13953039,C=18)
	CT(71608)=(S=13953040,E=13953047,C=71)
	CT(71609)=(S=13953048,E=13953055,C=42)
	CT(71610)=(S=13953056,E=13953151,C=18)
	CT(71611)=(S=13953152,E=13953279,C=34)
	CT(71612)=(S=13953280,E=13953311,C=49)
	CT(71613)=(S=13953312,E=13953343,C=70)
	CT(71614)=(S=13953344,E=13953407,C=18)
	CT(71615)=(S=13953408,E=13953535,C=51)
	CT(71616)=(S=13953536,E=13953791,C=12)
	CT(71617)=(S=13953792,E=13953919,C=17)
	CT(71618)=(S=13953920,E=13954047,C=52)
	CT(71619)=(S=13954048,E=13954303,C=42)
	CT(71620)=(S=13954304,E=13954559,C=24)
	CT(71621)=(S=13954560,E=13954815,C=36)
	CT(71622)=(S=13954816,E=13954943,C=15)
	CT(71623)=(S=13954944,E=13955071,C=49)
	CT(71624)=(S=13955072,E=13955327,C=14)
	CT(71625)=(S=13955328,E=13955359,C=62)
	CT(71626)=(S=13955360,E=13955391,C=36)
	CT(71627)=(S=13955392,E=13955455,C=42)
	CT(71628)=(S=13955456,E=13955583,C=14)
	CT(71629)=(S=13955584,E=13955839,C=24)
	CT(71630)=(S=13955840,E=13956095,C=34)
	CT(71631)=(S=13956096,E=13956351,C=58)
	CT(71632)=(S=13956352,E=13956607,C=15)
	CT(71633)=(S=13956608,E=13956863,C=47)
	CT(71634)=(S=13956864,E=13957119,C=19)
	CT(71635)=(S=13957120,E=13957247,C=18)
	CT(71636)=(S=13957248,E=13957375,C=14)
	CT(71637)=(S=13957376,E=13957631,C=34)
	CT(71638)=(S=13957632,E=13957887,C=14)
	CT(71639)=(S=13957888,E=13958015,C=22)
	CT(71640)=(S=13958016,E=13958143,C=27)
	CT(71641)=(S=13958144,E=13958655,C=37)
	CT(71642)=(S=13958656,E=13958911,C=34)
	CT(71643)=(S=13958912,E=13959167,C=28)
	CT(71644)=(S=13959168,E=13959423,C=25)
	CT(71645)=(S=13959424,E=13959935,C=14)
	CT(71646)=(S=13959936,E=13960191,C=34)
	CT(71647)=(S=13960192,E=13960447,C=25)
	CT(71648)=(S=13960448,E=13960463,C=58)
	CT(71649)=(S=13960464,E=13960467,C=18)
	CT(71650)=(S=13960468,E=13960471,C=53)
	CT(71651)=(S=13960472,E=13960479,C=18)
	CT(71652)=(S=13960480,E=13960487,C=24)
	CT(71653)=(S=13960488,E=13960495,C=58)
	CT(71654)=(S=13960496,E=13960503,C=18)
	CT(71655)=(S=13960504,E=13960511,C=57)
	CT(71656)=(S=13960512,E=13960519,C=36)
	CT(71657)=(S=13960520,E=13960535,C=18)
	CT(71658)=(S=13960536,E=13960543,C=14)
	CT(71659)=(S=13960544,E=13960559,C=18)
	CT(71660)=(S=13960560,E=13960567,C=52)
	CT(71661)=(S=13960568,E=13960583,C=18)
	CT(71662)=(S=13960584,E=13960599,C=58)
	CT(71663)=(S=13960600,E=13960607,C=34)
	CT(71664)=(S=13960608,E=13960615,C=18)
	CT(71665)=(S=13960616,E=13960623,C=23)
	CT(71666)=(S=13960624,E=13960631,C=14)
	CT(71667)=(S=13960632,E=13960639,C=18)
	CT(71668)=(S=13960640,E=13960647,C=31)
	CT(71669)=(S=13960648,E=13960663,C=58)
	CT(71670)=(S=13960664,E=13960695,C=18)
	CT(71671)=(S=13960696,E=13960703,C=58)
	CT(71672)=(S=13960704,E=13960959,C=43)
	CT(71673)=(S=13960960,E=13961215,C=32)
	CT(71674)=(S=13961216,E=13961471,C=17)
	CT(71675)=(S=13961472,E=13961599,C=28)
	CT(71676)=(S=13961600,E=13961727,C=25)
	CT(71677)=(S=13961728,E=13961983,C=36)
	CT(71678)=(S=13961984,E=13962239,C=12)
	CT(71679)=(S=13962240,E=13962495,C=13)
	CT(71680)=(S=13962496,E=13962751,C=21)
	CT(71681)=(S=13962752,E=13963007,C=37)
	CT(71682)=(S=13963008,E=13963263,C=19)
	CT(71683)=(S=13963264,E=13963295,C=12)
	CT(71684)=(S=13963296,E=13963327,C=51)
	CT(71685)=(S=13963328,E=13963391,C=50)
	CT(71686)=(S=13963392,E=13963519,C=22)
	CT(71687)=(S=13963520,E=13963647,C=36)
	CT(71688)=(S=13963648,E=13963775,C=58)
	CT(71689)=(S=13963776,E=13964031,C=14)
	CT(71690)=(S=13964032,E=13964159,C=33)
	CT(71691)=(S=13964160,E=13964287,C=14)
	CT(71692)=(S=13964288,E=13964543,C=28)
	CT(71693)=(S=13964544,E=13964607,C=18)
	CT(71694)=(S=13964608,E=13964671,C=19)
	CT(71695)=(S=13964672,E=13964735,C=15)
	CT(71696)=(S=13964736,E=13964799,C=60)
	CT(71697)=(S=13964800,E=13965055,C=21)
	CT(71698)=(S=13965056,E=13965311,C=28)
	CT(71699)=(S=13965312,E=13965567,C=18)
	CT(71700)=(S=13965568,E=13965823,C=58)
	CT(71701)=(S=13965824,E=13966079,C=15)
	CT(71702)=(S=13966080,E=13966207,C=18)
	CT(71703)=(S=13966208,E=13966335,C=25)
	CT(71704)=(S=13966336,E=13966591,C=47)
	CT(71705)=(S=13966592,E=13966847,C=33)
	CT(71706)=(S=13966848,E=13966975,C=21)
	CT(71707)=(S=13966976,E=13967039,C=12)
	CT(71708)=(S=13967040,E=13967103,C=28)
	CT(71709)=(S=13967104,E=13967359,C=49)
	CT(71710)=(S=13967360,E=13967615,C=24)
	CT(71711)=(S=13967616,E=13967743,C=42)
	CT(71712)=(S=13967744,E=13967871,C=18)
	CT(71713)=(S=13967872,E=13968063,C=36)
	CT(71714)=(S=13968064,E=13968095,C=101)
	CT(71715)=(S=13968096,E=13968127,C=36)
	CT(71716)=(S=13968128,E=13968255,C=14)
	CT(71717)=(S=13968256,E=13968383,C=55)
	CT(71718)=(S=13968384,E=13968639,C=12)
	CT(71719)=(S=13968640,E=13968895,C=25)
	CT(71720)=(S=13968896,E=13969151,C=13)
	CT(71721)=(S=13969152,E=13969279,C=14)
	CT(71722)=(S=13969280,E=13969407,C=28)
	CT(71723)=(S=13969408,E=13969791,C=14)
	CT(71724)=(S=13969792,E=13969919,C=12)
	CT(71725)=(S=13969920,E=13970175,C=16)
	CT(71726)=(S=13970176,E=13970431,C=37)
	CT(71727)=(S=13970432,E=13970687,C=12)
	CT(71728)=(S=13970688,E=13970943,C=15)
	CT(71729)=(S=13970944,E=13971199,C=36)
	CT(71730)=(S=13971200,E=13971455,C=42)
	CT(71731)=(S=13971456,E=13971711,C=14)
	CT(71732)=(S=13971712,E=13971967,C=49)
	CT(71733)=(S=13971968,E=13972223,C=19)
	CT(71734)=(S=13972224,E=13972479,C=36)
	CT(71735)=(S=13972480,E=13972607,C=27)
	CT(71736)=(S=13972608,E=13972735,C=14)
	CT(71737)=(S=13972736,E=13972991,C=36)
	CT(71738)=(S=13972992,E=13973247,C=28)
	CT(71739)=(S=13973248,E=13973311,C=12)
	CT(71740)=(S=13973312,E=13973375,C=228)
	CT(71741)=(S=13973376,E=13973503,C=34)
	CT(71742)=(S=13973504,E=13973759,C=12)
	CT(71743)=(S=13973760,E=13974015,C=17)
	CT(71744)=(S=13974016,E=13974271,C=21)
	CT(71745)=(S=13974272,E=13974527,C=18)
	CT(71746)=(S=13974528,E=13974783,C=25)
	CT(71747)=(S=13974784,E=13975039,C=28)
	CT(71748)=(S=13975040,E=13975295,C=13)
	CT(71749)=(S=13975296,E=13975551,C=21)
	CT(71750)=(S=13975552,E=13976575,C=19)
	CT(71751)=(S=13976576,E=13977599,C=28)
	CT(71752)=(S=13977600,E=13977855,C=14)
	CT(71753)=(S=13977856,E=13977887,C=58)
	CT(71754)=(S=13977888,E=13977919,C=25)
	CT(71755)=(S=13977920,E=13977983,C=28)
	CT(71756)=(S=13977984,E=13978111,C=36)
	CT(71757)=(S=13978112,E=13978367,C=37)
	CT(71758)=(S=13978368,E=13978623,C=36)
	CT(71759)=(S=13978624,E=13979135,C=58)
	CT(71760)=(S=13979136,E=13979391,C=14)
	CT(71761)=(S=13979392,E=13979423,C=18)
	CT(71762)=(S=13979424,E=13979455,C=63)
	CT(71763)=(S=13979456,E=13979519,C=18)
	CT(71764)=(S=13979520,E=13979775,C=19)
	CT(71765)=(S=13979776,E=13979903,C=18)
	CT(71766)=(S=13979904,E=13980031,C=14)
	CT(71767)=(S=13980032,E=13980159,C=71)
	CT(71768)=(S=13980160,E=13980415,C=15)
	CT(71769)=(S=13980416,E=13980479,C=28)
	CT(71770)=(S=13980480,E=13980543,C=14)
	CT(71771)=(S=13980544,E=13980671,C=24)
	CT(71772)=(S=13980672,E=13980927,C=36)
	CT(71773)=(S=13980928,E=13981183,C=18)
	CT(71774)=(S=13981184,E=13981439,C=14)
	CT(71775)=(S=13981440,E=13981823,C=18)
	CT(71776)=(S=13981824,E=13982207,C=19)
	CT(71777)=(S=13982208,E=13982335,C=42)
	CT(71778)=(S=13982336,E=13982527,C=12)
	CT(71779)=(S=13982528,E=13982591,C=59)
	CT(71780)=(S=13982592,E=13982719,C=51)
	CT(71781)=(S=13982720,E=13982847,C=15)
	CT(71782)=(S=13982848,E=13982975,C=58)
	CT(71783)=(S=13982976,E=13983231,C=36)
	CT(71784)=(S=13983232,E=13983295,C=25)
	CT(71785)=(S=13983296,E=13983359,C=42)
	CT(71786)=(S=13983360,E=13983487,C=63)
	CT(71787)=(S=13983488,E=13983743,C=28)
	CT(71788)=(S=13983744,E=13984767,C=25)
	CT(71789)=(S=13984768,E=13985791,C=19)
	CT(71790)=(S=13985792,E=13986815,C=14)
	CT(71791)=(S=13986816,E=13986823,C=18)
	CT(71792)=(S=13986824,E=13986831,C=58)
	CT(71793)=(S=13986832,E=13986839,C=18)
	CT(71794)=(S=13986840,E=13986847,C=36)
	CT(71795)=(S=13986848,E=13986855,C=18)
	CT(71796)=(S=13986856,E=13986863,C=31)
	CT(71797)=(S=13986872,E=13986879,C=58)
	CT(71798)=(S=13986880,E=13986887,C=12)
	CT(71799)=(S=13986888,E=13986895,C=31)
	CT(71800)=(S=13986896,E=13986903,C=58)
	CT(71801)=(S=13986904,E=13986911,C=28)
	CT(71802)=(S=13986912,E=13986919,C=19)
	CT(71803)=(S=13986920,E=13986927,C=36)
	CT(71804)=(S=13986928,E=13986935,C=58)
	CT(71805)=(S=13986936,E=13986943,C=71)
	CT(71806)=(S=13986944,E=13986967,C=18)
	CT(71807)=(S=13986968,E=13986975,C=58)
	CT(71808)=(S=13986976,E=13986983,C=33)
	CT(71809)=(S=13986984,E=13986991,C=18)
	CT(71810)=(S=13986992,E=13986999,C=31)
	CT(71811)=(S=13987000,E=13987039,C=18)
	CT(71812)=(S=13987040,E=13987047,C=58)
	CT(71813)=(S=13987048,E=13987055,C=12)
	CT(71814)=(S=13987064,E=13987103,C=18)
	CT(71815)=(S=13987104,E=13987119,C=58)
	CT(71816)=(S=13987120,E=13987151,C=18)
	CT(71817)=(S=13987152,E=13987167,C=31)
	CT(71818)=(S=13987168,E=13987183,C=18)
	CT(71819)=(S=13987184,E=13987199,C=36)
	CT(71820)=(S=13987200,E=13987215,C=31)
	CT(71821)=(S=13987216,E=13987279,C=18)
	CT(71822)=(S=13987280,E=13987295,C=38)
	CT(71823)=(S=13987296,E=13987311,C=31)
	CT(71824)=(S=13987312,E=13987327,C=26)
	CT(71825)=(S=13987328,E=13987391,C=18)
	CT(71826)=(S=13987392,E=13987423,C=60)
	CT(71827)=(S=13987424,E=13987519,C=31)
	CT(71828)=(S=13987520,E=13987583,C=18)
	CT(71829)=(S=13987584,E=13987647,C=12)
	CT(71830)=(S=13987648,E=13987839,C=31)
	CT(71831)=(S=13987840,E=13988863,C=19)
	CT(71832)=(S=13988864,E=13989375,C=13)
	CT(71833)=(S=13989376,E=13989887,C=49)
	CT(71834)=(S=13989888,E=13990911,C=14)
	CT(71835)=(S=13990912,E=13991935,C=36)
	CT(71836)=(S=13991936,E=13991967,C=18)
	CT(71837)=(S=13991968,E=13991999,C=12)
	CT(71838)=(S=13992000,E=13992031,C=37)
	CT(71839)=(S=13992032,E=13992095,C=28)
	CT(71840)=(S=13992096,E=13992127,C=27)
	CT(71841)=(S=13992128,E=13992159,C=18)
	CT(71842)=(S=13992160,E=13992191,C=14)
	CT(71843)=(S=13992192,E=13992223,C=24)
	CT(71844)=(S=13992224,E=13992255,C=18)
	CT(71845)=(S=13992256,E=13992287,C=14)
	CT(71846)=(S=13992288,E=13992319,C=18)
	CT(71847)=(S=13992320,E=13992351,C=33)
	CT(71848)=(S=13992352,E=13992383,C=25)
	CT(71849)=(S=13992384,E=13992415,C=36)
	CT(71850)=(S=13992416,E=13992447,C=42)
	CT(71851)=(S=13992448,E=13992479,C=31)
	CT(71852)=(S=13992480,E=13992511,C=14)
	CT(71853)=(S=13992512,E=13992543,C=51)
	CT(71854)=(S=13992544,E=13992575,C=94)
	CT(71855)=(S=13992576,E=13992607,C=14)
	CT(71856)=(S=13992608,E=13992639,C=36)
	CT(71857)=(S=13992640,E=13992671,C=19)
	CT(71858)=(S=13992672,E=13992703,C=47)
	CT(71859)=(S=13992704,E=13992735,C=18)
	CT(71860)=(S=13992736,E=13992767,C=40)
	CT(71861)=(S=13992768,E=13992799,C=106)
	CT(71862)=(S=13992800,E=13992831,C=14)
	CT(71863)=(S=13992832,E=13992863,C=19)
	CT(71864)=(S=13992864,E=13992895,C=14)
	CT(71865)=(S=13992896,E=13992959,C=28)
	CT(71866)=(S=13992960,E=13992991,C=24)
	CT(71867)=(S=13992992,E=13993023,C=16)
	CT(71868)=(S=13993024,E=13993055,C=18)
	CT(71869)=(S=13993056,E=13993087,C=19)
	CT(71870)=(S=13993088,E=13993119,C=49)
	CT(71871)=(S=13993120,E=13993183,C=36)
	CT(71872)=(S=13993184,E=13993215,C=101)
	CT(71873)=(S=13993216,E=13993247,C=102)
	CT(71874)=(S=13993248,E=13993279,C=36)
	CT(71875)=(S=13993280,E=13993295,C=14)
	CT(71876)=(S=13993296,E=13993311,C=32)
	CT(71877)=(S=13993312,E=13993343,C=28)
	CT(71878)=(S=13993344,E=13993375,C=14)
	CT(71879)=(S=13993376,E=13993407,C=31)
	CT(71880)=(S=13993408,E=13993439,C=99)
	CT(71881)=(S=13993440,E=13993471,C=34)
	CT(71882)=(S=13993472,E=13993503,C=50)
	CT(71883)=(S=13993504,E=13993535,C=25)
	CT(71884)=(S=13993536,E=13993567,C=15)
	CT(71885)=(S=13993568,E=13993599,C=19)
	CT(71886)=(S=13993600,E=13993663,C=58)
	CT(71887)=(S=13993664,E=13993695,C=18)
	CT(71888)=(S=13993696,E=13993727,C=36)
	CT(71889)=(S=13993728,E=13993759,C=28)
	CT(71890)=(S=13993760,E=13993791,C=58)
	CT(71891)=(S=13993792,E=13993887,C=18)
	CT(71892)=(S=13993888,E=13993919,C=54)
	CT(71893)=(S=13993920,E=13993951,C=28)
	CT(71894)=(S=13993952,E=13993983,C=70)
	CT(71895)=(S=13993984,E=13994015,C=36)
	CT(71896)=(S=13994016,E=13994047,C=19)
	CT(71897)=(S=13994048,E=13994079,C=28)
	CT(71898)=(S=13994080,E=13994111,C=118)
	CT(71899)=(S=13994112,E=13994175,C=15)
	CT(71900)=(S=13994176,E=13994207,C=23)
	CT(71901)=(S=13994208,E=13994239,C=58)
	CT(71902)=(S=13994240,E=13994271,C=14)
	CT(71903)=(S=13994272,E=13994303,C=15)
	CT(71904)=(S=13994304,E=13994335,C=29)
	CT(71905)=(S=13994336,E=13994367,C=46)
	CT(71906)=(S=13994368,E=13994399,C=187)
	CT(71907)=(S=13994400,E=13994431,C=13)
	CT(71908)=(S=13994432,E=13994463,C=28)
	CT(71909)=(S=13994464,E=13994495,C=18)
	CT(71910)=(S=13994496,E=13994527,C=12)
	CT(71911)=(S=13994528,E=13994559,C=28)
	CT(71912)=(S=13994560,E=13994591,C=18)
	CT(71913)=(S=13994592,E=13994623,C=14)
	CT(71914)=(S=13994624,E=13994655,C=47)
	CT(71915)=(S=13994656,E=13994687,C=27)
	CT(71916)=(S=13994688,E=13994719,C=18)
	CT(71917)=(S=13994720,E=13994751,C=21)
	CT(71918)=(S=13994752,E=13994783,C=25)
	CT(71919)=(S=13994784,E=13994815,C=61)
	CT(71920)=(S=13994816,E=13994847,C=28)
	CT(71921)=(S=13994848,E=13994879,C=12)
	CT(71922)=(S=13994880,E=13994911,C=28)
	CT(71923)=(S=13994912,E=13994943,C=47)
	CT(71924)=(S=13994944,E=13995007,C=37)
	CT(71925)=(S=13995008,E=13995039,C=15)
	CT(71926)=(S=13995040,E=13995071,C=25)
	CT(71927)=(S=13995072,E=13995103,C=24)
	CT(71928)=(S=13995104,E=13995135,C=18)
	CT(71929)=(S=13995136,E=13995167,C=22)
	CT(71930)=(S=13995168,E=13995199,C=47)
	CT(71931)=(S=13995200,E=13995231,C=32)
	CT(71932)=(S=13995232,E=13995263,C=18)
	CT(71933)=(S=13995264,E=13995295,C=21)
	CT(71934)=(S=13995296,E=13995327,C=25)
	CT(71935)=(S=13995328,E=13995359,C=19)
	CT(71936)=(S=13995360,E=13995391,C=47)
	CT(71937)=(S=13995392,E=13995423,C=18)
	CT(71938)=(S=13995424,E=13995455,C=20)
	CT(71939)=(S=13995456,E=13995519,C=18)
	CT(71940)=(S=13995520,E=13995551,C=19)
	CT(71941)=(S=13995552,E=13995583,C=18)
	CT(71942)=(S=13995584,E=13995615,C=27)
	CT(71943)=(S=13995616,E=13995647,C=42)
	CT(71944)=(S=13995648,E=13995679,C=37)
	CT(71945)=(S=13995680,E=13995711,C=34)
	CT(71946)=(S=13995712,E=13995743,C=18)
	CT(71947)=(S=13995744,E=13995775,C=52)
	CT(71948)=(S=13995776,E=13995807,C=14)
	CT(71949)=(S=13995808,E=13995839,C=25)
	CT(71950)=(S=13995840,E=13995871,C=52)
	CT(71951)=(S=13995872,E=13995903,C=42)
	CT(71952)=(S=13995904,E=13995935,C=14)
	CT(71953)=(S=13995936,E=13995967,C=47)
	CT(71954)=(S=13995968,E=13995999,C=28)
	CT(71955)=(S=13996000,E=13996031,C=37)
	CT(71956)=(S=13996032,E=13996063,C=28)
	CT(71957)=(S=13996064,E=13996095,C=25)
	CT(71958)=(S=13996096,E=13996127,C=15)
	CT(71959)=(S=13996128,E=13996159,C=37)
	CT(71960)=(S=13996160,E=13996191,C=34)
	CT(71961)=(S=13996192,E=13996223,C=15)
	CT(71962)=(S=13996224,E=13996255,C=12)
	CT(71963)=(S=13996256,E=13996287,C=36)
	CT(71964)=(S=13996288,E=13996351,C=18)
	CT(71965)=(S=13996352,E=13996383,C=28)
	CT(71966)=(S=13996384,E=13996415,C=51)
	CT(71967)=(S=13996416,E=13996447,C=62)
	CT(71968)=(S=13996448,E=13996479,C=27)
	CT(71969)=(S=13996480,E=13996511,C=47)
	CT(71970)=(S=13996512,E=13996543,C=42)
	CT(71971)=(S=13996544,E=13996575,C=34)
	CT(71972)=(S=13996576,E=13996607,C=58)
	CT(71973)=(S=13996608,E=13996639,C=19)
	CT(71974)=(S=13996640,E=13996671,C=28)
	CT(71975)=(S=13996672,E=13996735,C=14)
	CT(71976)=(S=13996736,E=13996767,C=21)
	CT(71977)=(S=13996768,E=13996831,C=28)
	CT(71978)=(S=13996832,E=13996863,C=18)
	CT(71979)=(S=13996864,E=13996895,C=109)
	CT(71980)=(S=13996896,E=13996927,C=57)
	CT(71981)=(S=13996928,E=13996959,C=26)
	CT(71982)=(S=13996960,E=13996991,C=42)
	CT(71983)=(S=13996992,E=13997023,C=18)
	CT(71984)=(S=13997024,E=13997055,C=14)
	CT(71985)=(S=13997056,E=13997087,C=18)
	CT(71986)=(S=13997088,E=13997119,C=14)
	CT(71987)=(S=13997120,E=13997151,C=37)
	CT(71988)=(S=13997152,E=13997215,C=28)
	CT(71989)=(S=13997216,E=13997247,C=18)
	CT(71990)=(S=13997248,E=13997279,C=25)
	CT(71991)=(S=13997280,E=13997311,C=36)
	CT(71992)=(S=13997312,E=13997343,C=18)
	CT(71993)=(S=13997344,E=13997375,C=57)
	CT(71994)=(S=13997376,E=13997407,C=28)
	CT(71995)=(S=13997408,E=13997439,C=102)
	CT(71996)=(S=13997440,E=13997471,C=51)
	CT(71997)=(S=13997472,E=13997503,C=32)
	CT(71998)=(S=13997504,E=13997535,C=15)
	CT(71999)=(S=13997536,E=13997567,C=25)
	CT(72000)=(S=13997568,E=13997599,C=42)
	CT(72001)=(S=13997600,E=13997631,C=24)
	CT(72002)=(S=13997632,E=13997663,C=18)
	CT(72003)=(S=13997664,E=13997695,C=109)
	CT(72004)=(S=13997696,E=13997727,C=19)
	CT(72005)=(S=13997728,E=13997759,C=152)
	CT(72006)=(S=13997760,E=13997767,C=127)
	CT(72007)=(S=13997768,E=13997791,C=105)
	CT(72008)=(S=13997792,E=13997823,C=42)
	CT(72009)=(S=13997824,E=13997855,C=18)
	CT(72010)=(S=13997856,E=13997887,C=17)
	CT(72011)=(S=13997888,E=13997919,C=33)
	CT(72012)=(S=13997920,E=13997951,C=25)
	CT(72013)=(S=13997952,E=13997983,C=27)
	CT(72014)=(S=13997984,E=13998015,C=12)
	CT(72015)=(S=13998016,E=13998079,C=71)
	CT(72016)=(S=13998080,E=13998111,C=12)
	CT(72017)=(S=13998112,E=13998143,C=14)
	CT(72018)=(S=13998144,E=13998175,C=106)
	CT(72019)=(S=13998176,E=13998207,C=28)
	CT(72020)=(S=13998208,E=13998239,C=18)
	CT(72021)=(S=13998240,E=13998271,C=36)
	CT(72022)=(S=13998272,E=13998303,C=15)
	CT(72023)=(S=13998304,E=13998335,C=14)
	CT(72024)=(S=13998336,E=13998367,C=27)
	CT(72025)=(S=13998368,E=13998399,C=42)
	CT(72026)=(S=13998400,E=13998431,C=28)
	CT(72027)=(S=13998432,E=13998463,C=19)
	CT(72028)=(S=13998464,E=13998591,C=37)
	CT(72029)=(S=13998592,E=13998623,C=41)
	CT(72030)=(S=13998624,E=13998655,C=106)
	CT(72031)=(S=13998656,E=13998687,C=148)
	CT(72032)=(S=13998688,E=13998751,C=38)
	CT(72033)=(S=13998752,E=13998783,C=18)
	CT(72034)=(S=13998784,E=13998815,C=31)
	CT(72035)=(S=13998816,E=13998847,C=36)
	CT(72036)=(S=13998848,E=13998879,C=31)
	CT(72037)=(S=13998880,E=13998911,C=33)
	CT(72038)=(S=13998912,E=13998943,C=28)
	CT(72039)=(S=13998944,E=13998975,C=37)
	CT(72040)=(S=13998976,E=13999007,C=13)
	CT(72041)=(S=13999008,E=13999039,C=58)
	CT(72042)=(S=13999040,E=13999071,C=18)
	CT(72043)=(S=13999072,E=13999103,C=33)
	CT(72044)=(S=13999104,E=13999135,C=36)
	CT(72045)=(S=13999136,E=13999167,C=15)
	CT(72046)=(S=13999168,E=13999199,C=31)
	CT(72047)=(S=13999200,E=13999231,C=58)
	CT(72048)=(S=13999232,E=13999263,C=18)
	CT(72049)=(S=13999264,E=13999295,C=28)
	CT(72050)=(S=13999296,E=13999327,C=18)
	CT(72051)=(S=13999328,E=13999359,C=34)
	CT(72052)=(S=13999360,E=13999391,C=28)
	CT(72053)=(S=13999392,E=13999423,C=20)
	CT(72054)=(S=13999424,E=13999455,C=47)
	CT(72055)=(S=13999456,E=13999487,C=50)
	CT(72056)=(S=13999488,E=13999519,C=42)
	CT(72057)=(S=13999520,E=13999551,C=38)
	CT(72058)=(S=13999552,E=13999583,C=40)
	CT(72059)=(S=13999584,E=13999615,C=52)
	CT(72060)=(S=13999616,E=13999647,C=18)
	CT(72061)=(S=13999648,E=13999679,C=19)
	CT(72062)=(S=13999680,E=13999711,C=15)
	CT(72063)=(S=13999712,E=13999743,C=28)
	CT(72064)=(S=13999744,E=13999775,C=34)
	CT(72065)=(S=13999776,E=13999807,C=106)
	CT(72066)=(S=13999808,E=13999839,C=58)
	CT(72067)=(S=13999840,E=13999871,C=27)
	CT(72068)=(S=13999872,E=13999903,C=36)
	CT(72069)=(S=13999904,E=13999935,C=44)
	CT(72070)=(S=13999936,E=13999999,C=18)
	CT(72071)=(S=14000000,E=14000031,C=63)
	CT(72072)=(S=14000032,E=14000063,C=24)
	CT(72073)=(S=14000064,E=14000095,C=18)
	CT(72074)=(S=14000096,E=14000127,C=31)
	CT(72075)=(S=14000128,E=14000159,C=28)
	CT(72076)=(S=14000160,E=14000191,C=34)
	CT(72077)=(S=14000192,E=14000223,C=28)
	CT(72078)=(S=14000224,E=14000255,C=14)
	CT(72079)=(S=14000256,E=14000287,C=31)
	CT(72080)=(S=14000288,E=14000319,C=71)
	CT(72081)=(S=14000320,E=14000383,C=36)
	CT(72082)=(S=14000384,E=14000415,C=52)
	CT(72083)=(S=14000416,E=14000447,C=47)
	CT(72084)=(S=14000448,E=14000479,C=14)
	CT(72085)=(S=14000480,E=14000511,C=58)
	CT(72086)=(S=14000512,E=14000543,C=37)
	CT(72087)=(S=14000544,E=14000575,C=27)
	CT(72088)=(S=14000576,E=14000607,C=12)
	CT(72089)=(S=14000608,E=14000639,C=27)
	CT(72090)=(S=14000640,E=14000671,C=34)
	CT(72091)=(S=14000672,E=14000703,C=12)
	CT(72092)=(S=14000704,E=14000735,C=42)
	CT(72093)=(S=14000736,E=14000767,C=14)
	CT(72094)=(S=14000768,E=14000799,C=28)
	CT(72095)=(S=14000800,E=14000831,C=36)
	CT(72096)=(S=14000832,E=14000863,C=25)
	CT(72097)=(S=14000864,E=14000895,C=27)
	CT(72098)=(S=14000896,E=14000959,C=50)
	CT(72099)=(S=14000960,E=14000991,C=36)
	CT(72100)=(S=14000992,E=14001023,C=102)
	CT(72101)=(S=14001024,E=14001055,C=19)
	CT(72102)=(S=14001056,E=14001087,C=12)
	CT(72103)=(S=14001088,E=14001119,C=14)
	CT(72104)=(S=14001120,E=14001183,C=42)
	CT(72105)=(S=14001184,E=14001215,C=25)
	CT(72106)=(S=14001216,E=14001247,C=28)
	CT(72107)=(S=14001248,E=14001279,C=44)
	CT(72108)=(S=14001280,E=14001311,C=28)
	CT(72109)=(S=14001312,E=14001343,C=25)
	CT(72110)=(S=14001344,E=14001375,C=19)
	CT(72111)=(S=14001376,E=14001407,C=38)
	CT(72112)=(S=14001408,E=14001439,C=14)
	CT(72113)=(S=14001440,E=14001471,C=39)
	CT(72114)=(S=14001472,E=14001535,C=28)
	CT(72115)=(S=14001536,E=14001567,C=14)
	CT(72116)=(S=14001568,E=14001599,C=72)
	CT(72117)=(S=14001600,E=14001631,C=18)
	CT(72118)=(S=14001632,E=14001695,C=14)
	CT(72119)=(S=14001696,E=14001727,C=70)
	CT(72120)=(S=14001728,E=14001759,C=14)
	CT(72121)=(S=14001760,E=14001791,C=13)
	CT(72122)=(S=14001792,E=14001823,C=23)
	CT(72123)=(S=14001824,E=14001855,C=27)
	CT(72124)=(S=14001856,E=14001887,C=12)
	CT(72125)=(S=14001888,E=14001919,C=28)
	CT(72126)=(S=14001920,E=14001951,C=51)
	CT(72127)=(S=14001952,E=14001983,C=18)
	CT(72128)=(S=14001984,E=14002015,C=14)
	CT(72129)=(S=14002016,E=14002047,C=27)
	CT(72130)=(S=14002048,E=14002079,C=48)
	CT(72131)=(S=14002080,E=14002111,C=28)
	CT(72132)=(S=14002112,E=14002143,C=18)
	CT(72133)=(S=14002148,E=14002151,C=12)
	CT(72134)=(S=14002152,E=14002159,C=15)
	CT(72135)=(S=14002160,E=14002175,C=12)
	CT(72136)=(S=14002176,E=14002207,C=55)
	CT(72137)=(S=14002208,E=14002239,C=18)
	CT(72138)=(S=14002240,E=14002303,C=28)
	CT(72139)=(S=14002304,E=14002335,C=18)
	CT(72140)=(S=14002336,E=14002367,C=33)
	CT(72141)=(S=14002368,E=14002399,C=28)
	CT(72142)=(S=14002400,E=14002423,C=63)
	CT(72143)=(S=14002424,E=14002427,C=13)
	CT(72144)=(S=14002428,E=14002431,C=63)
	CT(72145)=(S=14002432,E=14002463,C=47)
	CT(72146)=(S=14002464,E=14002495,C=51)
	CT(72147)=(S=14002496,E=14002527,C=31)
	CT(72148)=(S=14002528,E=14002559,C=70)
	CT(72149)=(S=14002560,E=14002591,C=32)
	CT(72150)=(S=14002592,E=14002623,C=12)
	CT(72151)=(S=14002624,E=14002687,C=36)
	CT(72152)=(S=14002688,E=14002719,C=14)
	CT(72153)=(S=14002720,E=14002751,C=25)
	CT(72154)=(S=14002752,E=14002815,C=18)
	CT(72155)=(S=14002816,E=14002847,C=14)
	CT(72156)=(S=14002848,E=14002879,C=28)
	CT(72157)=(S=14002912,E=14002943,C=24)
	CT(72158)=(S=14002944,E=14003007,C=18)
	CT(72159)=(S=14003008,E=14003039,C=36)
	CT(72160)=(S=14003040,E=14003071,C=15)
	CT(72161)=(S=14003072,E=14003103,C=36)
	CT(72162)=(S=14003104,E=14003135,C=15)
	CT(72163)=(S=14003136,E=14003167,C=14)
	CT(72164)=(S=14003168,E=14003199,C=25)
	CT(72165)=(S=14003200,E=14003231,C=18)
	CT(72166)=(S=14003232,E=14003263,C=25)
	CT(72167)=(S=14003264,E=14003295,C=41)
	CT(72168)=(S=14003296,E=14003327,C=28)
	CT(72169)=(S=14003328,E=14003359,C=105)
	CT(72170)=(S=14003360,E=14003391,C=58)
	CT(72171)=(S=14003392,E=14003423,C=27)
	CT(72172)=(S=14003424,E=14003455,C=52)
	CT(72173)=(S=14003456,E=14003487,C=28)
	CT(72174)=(S=14003520,E=14003551,C=42)
	CT(72175)=(S=14003552,E=14003583,C=15)
	CT(72176)=(S=14003584,E=14003615,C=47)
	CT(72177)=(S=14003616,E=14003647,C=34)
	CT(72178)=(S=14003648,E=14003679,C=19)
	CT(72179)=(S=14003680,E=14003711,C=24)
	CT(72180)=(S=14003712,E=14003743,C=31)
	CT(72181)=(S=14003744,E=14003775,C=28)
	CT(72182)=(S=14003776,E=14003807,C=24)
	CT(72183)=(S=14003808,E=14003839,C=12)
	CT(72184)=(S=14003840,E=14003871,C=36)
	CT(72185)=(S=14003872,E=14003903,C=15)
	CT(72186)=(S=14003904,E=14003935,C=14)
	CT(72187)=(S=14003936,E=14003967,C=42)
	CT(72188)=(S=14003968,E=14003999,C=15)
	CT(72189)=(S=14004000,E=14004031,C=33)
	CT(72190)=(S=14004032,E=14004095,C=60)
	CT(72191)=(S=14004096,E=14004127,C=14)
	CT(72192)=(S=14004128,E=14004159,C=39)
	CT(72193)=(S=14004160,E=14004223,C=14)
	CT(72194)=(S=14004224,E=14004351,C=26)
	CT(72195)=(S=14004352,E=14004383,C=48)
	CT(72196)=(S=14004384,E=14004415,C=25)
	CT(72197)=(S=14004416,E=14004447,C=12)
	CT(72198)=(S=14004448,E=14004479,C=18)
	CT(72199)=(S=14004480,E=14004511,C=38)
	CT(72200)=(S=14004512,E=14004543,C=14)
	CT(72201)=(S=14004544,E=14004575,C=49)
	CT(72202)=(S=14004576,E=14004607,C=18)
	CT(72203)=(S=14004608,E=14004671,C=49)
	CT(72204)=(S=14004672,E=14004703,C=25)
	CT(72205)=(S=14004704,E=14004735,C=14)
	CT(72206)=(S=14004736,E=14004767,C=28)
	CT(72207)=(S=14004768,E=14004799,C=18)
	CT(72208)=(S=14004800,E=14004831,C=28)
	CT(72209)=(S=14004832,E=14004863,C=50)
	CT(72210)=(S=14004864,E=14004895,C=14)
	CT(72211)=(S=14004896,E=14004927,C=28)
	CT(72212)=(S=14004928,E=14004959,C=15)
	CT(72213)=(S=14004960,E=14004991,C=30)
	CT(72214)=(S=14004992,E=14005023,C=19)
	CT(72215)=(S=14005024,E=14005055,C=27)
	CT(72216)=(S=14005056,E=14005087,C=28)
	CT(72217)=(S=14005088,E=14005119,C=25)
	CT(72218)=(S=14005120,E=14005151,C=28)
	CT(72219)=(S=14005152,E=14005183,C=135)
	CT(72220)=(S=14005184,E=14005215,C=36)
	CT(72221)=(S=14005216,E=14005247,C=31)
	CT(72222)=(S=14005248,E=14005279,C=55)
	CT(72223)=(S=14005280,E=14005311,C=33)
	CT(72224)=(S=14005312,E=14005343,C=19)
	CT(72225)=(S=14005344,E=14005375,C=60)
	CT(72226)=(S=14005376,E=14005407,C=58)
	CT(72227)=(S=14005408,E=14005439,C=34)
	CT(72228)=(S=14005440,E=14005471,C=18)
	CT(72229)=(S=14005472,E=14005503,C=12)
	CT(72230)=(S=14005504,E=14005535,C=18)
	CT(72231)=(S=14005536,E=14005567,C=49)
	CT(72232)=(S=14005568,E=14005599,C=25)
	CT(72233)=(S=14005600,E=14005631,C=48)
	CT(72234)=(S=14005632,E=14005663,C=71)
	CT(72235)=(S=14005664,E=14005695,C=23)
	CT(72236)=(S=14005696,E=14005727,C=50)
	CT(72237)=(S=14005728,E=14005759,C=106)
	CT(72238)=(S=14005760,E=14005791,C=28)
	CT(72239)=(S=14005792,E=14005823,C=12)
	CT(72240)=(S=14005824,E=14005855,C=15)
	CT(72241)=(S=14005856,E=14005919,C=28)
	CT(72242)=(S=14005920,E=14005951,C=18)
	CT(72243)=(S=14005952,E=14005983,C=60)
	CT(72244)=(S=14005984,E=14006015,C=42)
	CT(72245)=(S=14006016,E=14006047,C=28)
	CT(72246)=(S=14006048,E=14006079,C=18)
	CT(72247)=(S=14006080,E=14006111,C=28)
	CT(72248)=(S=14006112,E=14006143,C=18)
	CT(72249)=(S=14006144,E=14006175,C=15)
	CT(72250)=(S=14006176,E=14006239,C=28)
	CT(72251)=(S=14006240,E=14006271,C=18)
	CT(72252)=(S=14006272,E=14006303,C=58)
	CT(72253)=(S=14006304,E=14006335,C=55)
	CT(72254)=(S=14006368,E=14006399,C=17)
	CT(72255)=(S=14006400,E=14006431,C=18)
	CT(72256)=(S=14006432,E=14006463,C=23)
	CT(72257)=(S=14006464,E=14006495,C=27)
	CT(72258)=(S=14006496,E=14006527,C=68)
	CT(72259)=(S=14006528,E=14006559,C=19)
	CT(72260)=(S=14006560,E=14006591,C=47)
	CT(72261)=(S=14006592,E=14006623,C=28)
	CT(72262)=(S=14006624,E=14006655,C=14)
	CT(72263)=(S=14006656,E=14006687,C=28)
	CT(72264)=(S=14006688,E=14006719,C=42)
	CT(72265)=(S=14006720,E=14006751,C=14)
	CT(72266)=(S=14006752,E=14006783,C=19)
	CT(72267)=(S=14006784,E=14006815,C=57)
	CT(72268)=(S=14006816,E=14006847,C=12)
	CT(72269)=(S=14006848,E=14006879,C=58)
	CT(72270)=(S=14006880,E=14006911,C=31)
	CT(72271)=(S=14006912,E=14006943,C=37)
	CT(72272)=(S=14006944,E=14006975,C=61)
	CT(72273)=(S=14006976,E=14007007,C=31)
	CT(72274)=(S=14007008,E=14007039,C=47)
	CT(72275)=(S=14007040,E=14007071,C=15)
	CT(72276)=(S=14007072,E=14007103,C=14)
	CT(72277)=(S=14007104,E=14007135,C=28)
	CT(72278)=(S=14007136,E=14007167,C=18)
	CT(72279)=(S=14007168,E=14007199,C=36)
	CT(72280)=(S=14007200,E=14007231,C=27)
	CT(72281)=(S=14007232,E=14007263,C=19)
	CT(72282)=(S=14007264,E=14007279,C=55)
	CT(72283)=(S=14007280,E=14007287,C=36)
	CT(72284)=(S=14007288,E=14007295,C=55)
	CT(72285)=(S=14007296,E=14007327,C=27)
	CT(72286)=(S=14007328,E=14007359,C=34)
	CT(72287)=(S=14007360,E=14007391,C=35)
	CT(72288)=(S=14007392,E=14007423,C=28)
	CT(72289)=(S=14007424,E=14007455,C=27)
	CT(72290)=(S=14007456,E=14007487,C=12)
	CT(72291)=(S=14007488,E=14007519,C=15)
	CT(72292)=(S=14007520,E=14007551,C=34)
	CT(72293)=(S=14007552,E=14007583,C=36)
	CT(72294)=(S=14007584,E=14007615,C=58)
	CT(72295)=(S=14007616,E=14007647,C=101)
	CT(72296)=(S=14007648,E=14007679,C=19)
	CT(72297)=(S=14007680,E=14007711,C=34)
	CT(72298)=(S=14007712,E=14007743,C=49)
	CT(72299)=(S=14007744,E=14007807,C=18)
	CT(72300)=(S=14007808,E=14007839,C=25)
	CT(72301)=(S=14007840,E=14007871,C=44)
	CT(72302)=(S=14007872,E=14007903,C=12)
	CT(72303)=(S=14007904,E=14007935,C=48)
	CT(72304)=(S=14007936,E=14007967,C=63)
	CT(72305)=(S=14007968,E=14007999,C=14)
	CT(72306)=(S=14008000,E=14008031,C=21)
	CT(72307)=(S=14008032,E=14008095,C=18)
	CT(72308)=(S=14008096,E=14008159,C=28)
	CT(72309)=(S=14008160,E=14008191,C=33)
	CT(72310)=(S=14008192,E=14008223,C=57)
	CT(72311)=(S=14008224,E=14008287,C=51)
	CT(72312)=(S=14008288,E=14008319,C=63)
	CT(72313)=(S=14008320,E=14008383,C=33)
	CT(72314)=(S=14008384,E=14008447,C=58)
	CT(72315)=(S=14008448,E=14008511,C=47)
	CT(72316)=(S=14008512,E=14008575,C=25)
	CT(72317)=(S=14008576,E=14008607,C=18)
	CT(72318)=(S=14008608,E=14008639,C=105)
	CT(72319)=(S=14008640,E=14008703,C=34)
	CT(72320)=(S=14008704,E=14008767,C=49)
	CT(72321)=(S=14008768,E=14008831,C=36)
	CT(72322)=(S=14008832,E=14008895,C=25)
	CT(72323)=(S=14008896,E=14008959,C=37)
	CT(72324)=(S=14008960,E=14009023,C=25)
	CT(72325)=(S=14009024,E=14009087,C=33)
	CT(72326)=(S=14009088,E=14009151,C=26)
	CT(72327)=(S=14009152,E=14009215,C=25)
	CT(72328)=(S=14009216,E=14009279,C=58)
	CT(72329)=(S=14009280,E=14009343,C=33)
	CT(72330)=(S=14009344,E=14009407,C=36)
	CT(72331)=(S=14009408,E=14009471,C=59)
	CT(72332)=(S=14009472,E=14009535,C=34)
	CT(72333)=(S=14009536,E=14009599,C=28)
	CT(72334)=(S=14009600,E=14009663,C=36)
	CT(72335)=(S=14009664,E=14009727,C=50)
	CT(72336)=(S=14009728,E=14009791,C=44)
	CT(72337)=(S=14009792,E=14009855,C=36)
	CT(72338)=(S=14009856,E=14009983,C=28)
	CT(72339)=(S=14009984,E=14010047,C=15)
	CT(72340)=(S=14010048,E=14010111,C=46)
	CT(72341)=(S=14010112,E=14010143,C=15)
	CT(72342)=(S=14010144,E=14010175,C=14)
	CT(72343)=(S=14010176,E=14010239,C=19)
	CT(72344)=(S=14010240,E=14010303,C=14)
	CT(72345)=(S=14010304,E=14010367,C=58)
	CT(72346)=(S=14010368,E=14010399,C=40)
	CT(72347)=(S=14010400,E=14010431,C=31)
	CT(72348)=(S=14010432,E=14010495,C=63)
	CT(72349)=(S=14010496,E=14010559,C=19)
	CT(72350)=(S=14010560,E=14010623,C=34)
	CT(72351)=(S=14010624,E=14010751,C=25)
	CT(72352)=(S=14010752,E=14010879,C=36)
	CT(72353)=(S=14010880,E=14010911,C=45)
	CT(72354)=(S=14010912,E=14010943,C=34)
	CT(72355)=(S=14010944,E=14011007,C=57)
	CT(72356)=(S=14011008,E=14011071,C=63)
	CT(72357)=(S=14011072,E=14011135,C=28)
	CT(72358)=(S=14011136,E=14011199,C=27)
	CT(72359)=(S=14011200,E=14011263,C=12)
	CT(72360)=(S=14011264,E=14011327,C=15)
	CT(72361)=(S=14011328,E=14011391,C=28)
	CT(72362)=(S=14011392,E=14011423,C=15)
	CT(72363)=(S=14011424,E=14011455,C=47)
	CT(72364)=(S=14011456,E=14011519,C=39)
	CT(72365)=(S=14011520,E=14011583,C=19)
	CT(72366)=(S=14011584,E=14011647,C=36)
	CT(72367)=(S=14011648,E=14011711,C=15)
	CT(72368)=(S=14011712,E=14011743,C=21)
	CT(72369)=(S=14011744,E=14011775,C=12)
	CT(72370)=(S=14011776,E=14011935,C=14)
	CT(72371)=(S=14011936,E=14011967,C=98)
	CT(72372)=(S=14011968,E=14012031,C=36)
	CT(72373)=(S=14012032,E=14012063,C=14)
	CT(72374)=(S=14012064,E=14012095,C=28)
	CT(72375)=(S=14012096,E=14012191,C=36)
	CT(72376)=(S=14012192,E=14012223,C=96)
	CT(72377)=(S=14012224,E=14012287,C=36)
	CT(72378)=(S=14012288,E=14012351,C=32)
	CT(72379)=(S=14012352,E=14012415,C=26)
	CT(72380)=(S=14012416,E=14012447,C=42)
	CT(72381)=(S=14012448,E=14012479,C=28)
	CT(72382)=(S=14012480,E=14012543,C=14)
	CT(72383)=(S=14012544,E=14012575,C=42)
	CT(72384)=(S=14012576,E=14012586,C=18)
	CT(72385)=(S=14012587,E=14012587,C=38)
	CT(72386)=(S=14012588,E=14012607,C=18)
	CT(72387)=(S=14012608,E=14012671,C=14)
	CT(72388)=(S=14012672,E=14012735,C=15)
	CT(72389)=(S=14012736,E=14012831,C=28)
	CT(72390)=(S=14012832,E=14012927,C=15)
	CT(72391)=(S=14012928,E=14012991,C=14)
	CT(72392)=(S=14012992,E=14013055,C=18)
	CT(72393)=(S=14013056,E=14013119,C=33)
	CT(72394)=(S=14013120,E=14013183,C=23)
	CT(72395)=(S=14013184,E=14013215,C=37)
	CT(72396)=(S=14013216,E=14013247,C=33)
	CT(72397)=(S=14013248,E=14013311,C=20)
	CT(72398)=(S=14013312,E=14013375,C=49)
	CT(72399)=(S=14013376,E=14013439,C=28)
	CT(72400)=(S=14013440,E=14013503,C=19)
	CT(72401)=(S=14013504,E=14013567,C=14)
	CT(72402)=(S=14013568,E=14013631,C=15)
	CT(72403)=(S=14013632,E=14013695,C=106)
	CT(72404)=(S=14013696,E=14013823,C=15)
	CT(72405)=(S=14013824,E=14013855,C=48)
	CT(72406)=(S=14013856,E=14013887,C=34)
	CT(72407)=(S=14013888,E=14013951,C=49)
	CT(72408)=(S=14013952,E=14013983,C=28)
	CT(72409)=(S=14013984,E=14014015,C=49)
	CT(72410)=(S=14014016,E=14014047,C=51)
	CT(72411)=(S=14014048,E=14014079,C=36)
	CT(72412)=(S=14014080,E=14014143,C=47)
	CT(72413)=(S=14014144,E=14014207,C=19)
	CT(72414)=(S=14014208,E=14014271,C=12)
	CT(72415)=(S=14014272,E=14014335,C=71)
	CT(72416)=(S=14014336,E=14014463,C=15)
	CT(72417)=(S=14014464,E=14014495,C=28)
	CT(72418)=(S=14014496,E=14014527,C=33)
	CT(72419)=(S=14014528,E=14014591,C=58)
	CT(72420)=(S=14014592,E=14014655,C=14)
	CT(72421)=(S=14014656,E=14014719,C=47)
	CT(72422)=(S=14014752,E=14014783,C=26)
	CT(72423)=(S=14014784,E=14014847,C=28)
	CT(72424)=(S=14014848,E=14014911,C=15)
	CT(72425)=(S=14014912,E=14014975,C=72)
	CT(72426)=(S=14014976,E=14015007,C=28)
	CT(72427)=(S=14015008,E=14015039,C=15)
	CT(72428)=(S=14015040,E=14015103,C=58)
	CT(72429)=(S=14015104,E=14015135,C=12)
	CT(72430)=(S=14015136,E=14015167,C=28)
	CT(72431)=(S=14015168,E=14015267,C=14)
	CT(72432)=(S=14015268,E=14015271,C=13)
	CT(72433)=(S=14015272,E=14015295,C=14)
	CT(72434)=(S=14015296,E=14015359,C=55)
	CT(72435)=(S=14015360,E=14015423,C=49)
	CT(72436)=(S=14015424,E=14015487,C=18)
	CT(72437)=(S=14015488,E=14015551,C=25)
	CT(72438)=(S=14015552,E=14015615,C=48)
	CT(72439)=(S=14015616,E=14015679,C=28)
	CT(72440)=(S=14015680,E=14015743,C=33)
	CT(72441)=(S=14015744,E=14015807,C=18)
	CT(72442)=(S=14015808,E=14015871,C=28)
	CT(72443)=(S=14015872,E=14015903,C=34)
	CT(72444)=(S=14015904,E=14015935,C=14)
	CT(72445)=(S=14015936,E=14015999,C=34)
	CT(72446)=(S=14016000,E=14016031,C=36)
	CT(72447)=(S=14016032,E=14016063,C=51)
	CT(72448)=(S=14016064,E=14016127,C=12)
	CT(72449)=(S=14016128,E=14016191,C=50)
	CT(72450)=(S=14016192,E=14016223,C=58)
	CT(72451)=(S=14016224,E=14016255,C=18)
	CT(72452)=(S=14016256,E=14016511,C=12)
	CT(72453)=(S=14016512,E=14016767,C=49)
	CT(72454)=(S=14016768,E=14016831,C=42)
	CT(72455)=(S=14016832,E=14016895,C=27)
	CT(72456)=(S=14016896,E=14016927,C=14)
	CT(72457)=(S=14016928,E=14016959,C=12)
	CT(72458)=(S=14016960,E=14017023,C=15)
	CT(72459)=(S=14017024,E=14017087,C=51)
	CT(72460)=(S=14017152,E=14017215,C=44)
	CT(72461)=(S=14017216,E=14017279,C=33)
	CT(72462)=(S=14017280,E=14017343,C=25)
	CT(72463)=(S=14017344,E=14017407,C=58)
	CT(72464)=(S=14017408,E=14017439,C=36)
	CT(72465)=(S=14017440,E=14017471,C=42)
	CT(72466)=(S=14017472,E=14017535,C=31)
	CT(72467)=(S=14017536,E=14017599,C=12)
	CT(72468)=(S=14017600,E=14017663,C=18)
	CT(72469)=(S=14017664,E=14017727,C=21)
	CT(72470)=(S=14017728,E=14017791,C=14)
	CT(72471)=(S=14017792,E=14017855,C=42)
	CT(72472)=(S=14017856,E=14017919,C=14)
	CT(72473)=(S=14017920,E=14017983,C=25)
	CT(72474)=(S=14017984,E=14018047,C=52)
	CT(72475)=(S=14018048,E=14018079,C=23)
	CT(72476)=(S=14018080,E=14018111,C=34)
	CT(72477)=(S=14018112,E=14018175,C=98)
	CT(72478)=(S=14018176,E=14018207,C=15)
	CT(72479)=(S=14018208,E=14018239,C=27)
	CT(72480)=(S=14018240,E=14018303,C=14)
	CT(72481)=(S=14018304,E=14018367,C=31)
	CT(72482)=(S=14018368,E=14018431,C=25)
	CT(72483)=(S=14018432,E=14018495,C=51)
	CT(72484)=(S=14018496,E=14018559,C=58)
	CT(72485)=(S=14018560,E=14018623,C=37)
	CT(72486)=(S=14018624,E=14018687,C=14)
	CT(72487)=(S=14018688,E=14018751,C=34)
	CT(72488)=(S=14018752,E=14018815,C=18)
	CT(72489)=(S=14018816,E=14018879,C=15)
	CT(72490)=(S=14018880,E=14018943,C=38)
	CT(72491)=(S=14018944,E=14018975,C=63)
	CT(72492)=(S=14018976,E=14019007,C=26)
	CT(72493)=(S=14019008,E=14019071,C=36)
	CT(72494)=(S=14019072,E=14019103,C=18)
	CT(72495)=(S=14019104,E=14019135,C=14)
	CT(72496)=(S=14019136,E=14019199,C=27)
	CT(72497)=(S=14019200,E=14019263,C=15)
	CT(72498)=(S=14019264,E=14019327,C=18)
	CT(72499)=(S=14019328,E=14019391,C=14)
	CT(72500)=(S=14019392,E=14019519,C=33)
	CT(72501)=(S=14019520,E=14019583,C=42)
	CT(72502)=(S=14019584,E=14019615,C=25)
	CT(72503)=(S=14019616,E=14019647,C=23)
	CT(72504)=(S=14019648,E=14019711,C=36)
	CT(72505)=(S=14019712,E=14019839,C=27)
	CT(72506)=(S=14019840,E=14020031,C=24)
	CT(72507)=(S=14020032,E=14020095,C=14)
	CT(72508)=(S=14020096,E=14020127,C=31)
	CT(72509)=(S=14020128,E=14020159,C=28)
	CT(72510)=(S=14020160,E=14020223,C=58)
	CT(72511)=(S=14020224,E=14020287,C=37)
	CT(72512)=(S=14020288,E=14020351,C=19)
	CT(72513)=(S=14020352,E=14020415,C=36)
	CT(72514)=(S=14020416,E=14020479,C=27)
	CT(72515)=(S=14020480,E=14020543,C=36)
	CT(72516)=(S=14020544,E=14020607,C=28)
	CT(72517)=(S=14020608,E=14020671,C=46)
	CT(72518)=(S=14020672,E=14020735,C=42)
	CT(72519)=(S=14020736,E=14020799,C=28)
	CT(72520)=(S=14020800,E=14020863,C=51)
	CT(72521)=(S=14020864,E=14020991,C=58)
	CT(72522)=(S=14020992,E=14021055,C=28)
	CT(72523)=(S=14021056,E=14021183,C=18)
	CT(72524)=(S=14021184,E=14021247,C=14)
	CT(72525)=(S=14021248,E=14021311,C=19)
	CT(72526)=(S=14021312,E=14021375,C=18)
	CT(72527)=(S=14021376,E=14021439,C=37)
	CT(72528)=(S=14021440,E=14021503,C=18)
	CT(72529)=(S=14021504,E=14021567,C=47)
	CT(72530)=(S=14021568,E=14021631,C=15)
	CT(72531)=(S=14021632,E=14021663,C=12)
	CT(72532)=(S=14021664,E=14021695,C=28)
	CT(72533)=(S=14021696,E=14021759,C=43)
	CT(72534)=(S=14021760,E=14021823,C=14)
	CT(72535)=(S=14021824,E=14021824,C=28)
	CT(72536)=(S=14021825,E=14021825,C=46)
	CT(72537)=(S=14021826,E=14021827,C=34)
	CT(72538)=(S=14021828,E=14021831,C=14)
	CT(72539)=(S=14021832,E=14021871,C=46)
	CT(72540)=(S=14021872,E=14021879,C=19)
	CT(72541)=(S=14021880,E=14021887,C=46)
	CT(72542)=(S=14021888,E=14022143,C=12)
	CT(72543)=(S=14022144,E=14022175,C=18)
	CT(72544)=(S=14022176,E=14022207,C=12)
	CT(72545)=(S=14022208,E=14022335,C=14)
	CT(72546)=(S=14022336,E=14022399,C=49)
	CT(72547)=(S=14022400,E=14022431,C=105)
	CT(72548)=(S=14022432,E=14022527,C=36)
	CT(72549)=(S=14022528,E=14022719,C=18)
	CT(72550)=(S=14022720,E=14022783,C=13)
	CT(72551)=(S=14022784,E=14022847,C=37)
	CT(72552)=(S=14022848,E=14022911,C=14)
	CT(72553)=(S=14022912,E=14022975,C=22)
	CT(72554)=(S=14022976,E=14023007,C=36)
	CT(72555)=(S=14023008,E=14023167,C=14)
	CT(72556)=(S=14023168,E=14023231,C=52)
	CT(72557)=(S=14023232,E=14023295,C=47)
	CT(72558)=(S=14023296,E=14023359,C=25)
	CT(72559)=(S=14023360,E=14023423,C=33)
	CT(72560)=(S=14023424,E=14023487,C=14)
	CT(72561)=(S=14023488,E=14023551,C=49)
	CT(72562)=(S=14023552,E=14023615,C=12)
	CT(72563)=(S=14023616,E=14023679,C=18)
	CT(72564)=(S=14023680,E=14023743,C=28)
	CT(72565)=(S=14023744,E=14023807,C=18)
	CT(72566)=(S=14023808,E=14023871,C=28)
	CT(72567)=(S=14023872,E=14023935,C=44)
	CT(72568)=(S=14023936,E=14023999,C=14)
	CT(72569)=(S=14024000,E=14024063,C=52)
	CT(72570)=(S=14024064,E=14024127,C=14)
	CT(72571)=(S=14024128,E=14024191,C=50)
	CT(72572)=(S=14024192,E=14024223,C=15)
	CT(72573)=(S=14024224,E=14024255,C=28)
	CT(72574)=(S=14024256,E=14024319,C=25)
	CT(72575)=(S=14024320,E=14024351,C=37)
	CT(72576)=(S=14024352,E=14024383,C=14)
	CT(72577)=(S=14024384,E=14024447,C=63)
	CT(72578)=(S=14024448,E=14024575,C=15)
	CT(72579)=(S=14024608,E=14024639,C=47)
	CT(72580)=(S=14024640,E=14024703,C=14)
	CT(72581)=(S=14024704,E=14157503,C=29)
	CT(72582)=(S=14157504,E=14157519,C=78)
	CT(72583)=(S=14157520,E=14157759,C=29)
	CT(72584)=(S=14157760,E=14157791,C=78)
	CT(72585)=(S=14157792,E=14157823,C=29)
	CT(72586)=(S=14157824,E=14157855,C=2)
	CT(72587)=(S=14157856,E=14157951,C=29)
	CT(72588)=(S=14157952,E=14158015,C=78)
	CT(72589)=(S=14158016,E=14158127,C=29)
	CT(72590)=(S=14158128,E=14158143,C=78)
	CT(72591)=(S=14158144,E=14158319,C=29)
	CT(72592)=(S=14158320,E=14158335,C=78)
	CT(72593)=(S=14158336,E=14158543,C=29)
	CT(72594)=(S=14158544,E=14158559,C=175)
	CT(72595)=(S=14158560,E=14158563,C=29)
	CT(72596)=(S=14158564,E=14158569,C=78)
	CT(72597)=(S=14158571,E=14159103,C=29)
	CT(72598)=(S=14159104,E=14159359,C=78)
	CT(72599)=(S=14159360,E=14159487,C=29)
	CT(72600)=(S=14159488,E=14159503,C=78)
	CT(72601)=(S=14159504,E=14159551,C=29)
	CT(72602)=(S=14159552,E=14159567,C=0)
	CT(72603)=(S=14159568,E=14160095,C=29)
	CT(72604)=(S=14160096,E=14160127,C=78)
	CT(72605)=(S=14160128,E=14160383,C=29)
	CT(72606)=(S=14160384,E=14160511,C=78)
	CT(72607)=(S=14160512,E=14160703,C=29)
	CT(72608)=(S=14160704,E=14160735,C=78)
	CT(72609)=(S=14160736,E=14160815,C=29)
	CT(72610)=(S=14160816,E=14160831,C=78)
	CT(72611)=(S=14160832,E=14161159,C=29)
	CT(72612)=(S=14161160,E=14161163,C=78)
	CT(72613)=(S=14161164,E=14161279,C=29)
	CT(72614)=(S=14161280,E=14161311,C=78)
	CT(72615)=(S=14161312,E=14161791,C=29)
	CT(72616)=(S=14161792,E=14161823,C=78)
	CT(72617)=(S=14161824,E=14162143,C=29)
	CT(72618)=(S=14162144,E=14162159,C=78)
	CT(72619)=(S=14162160,E=14162623,C=29)
	CT(72620)=(S=14162624,E=14162655,C=78)
	CT(72621)=(S=14162656,E=14165119,C=29)
	CT(72622)=(S=14165120,E=14165183,C=78)
	CT(72623)=(S=14165184,E=14165359,C=29)
	CT(72624)=(S=14165360,E=14165375,C=78)
	CT(72625)=(S=14165376,E=14166047,C=29)
	CT(72626)=(S=14166048,E=14166063,C=78)
	CT(72627)=(S=14166064,E=14166111,C=29)
	CT(72628)=(S=14166112,E=14166143,C=78)
	CT(72629)=(S=14166144,E=14167391,C=29)
	CT(72630)=(S=14167392,E=14167423,C=78)
	CT(72631)=(S=14167424,E=14167551,C=29)
	CT(72632)=(S=14167552,E=14167599,C=78)
	CT(72633)=(S=14167600,E=14167679,C=29)
	CT(72634)=(S=14167680,E=14167711,C=78)
	CT(72635)=(S=14167712,E=14168143,C=29)
	CT(72636)=(S=14168144,E=14168159,C=78)
	CT(72637)=(S=14168176,E=14168223,C=29)
	CT(72638)=(S=14168224,E=14168239,C=78)
	CT(72639)=(S=14168240,E=14169951,C=29)
	CT(72640)=(S=14169952,E=14169967,C=78)
	CT(72641)=(S=14169968,E=14170047,C=29)
	CT(72642)=(S=14170048,E=14170095,C=78)
	CT(72643)=(S=14170096,E=14170543,C=29)
	CT(72644)=(S=14170544,E=14170559,C=78)
	CT(72645)=(S=14170560,E=14170623,C=29)
	CT(72646)=(S=14170624,E=14170751,C=78)
	CT(72647)=(S=14170752,E=14171103,C=29)
	CT(72648)=(S=14171104,E=14171135,C=78)
	CT(72649)=(S=14171136,E=14172575,C=29)
	CT(72650)=(S=14172576,E=14172607,C=78)
	CT(72651)=(S=14172608,E=14172799,C=29)
	CT(72652)=(S=14172800,E=14172879,C=78)
	CT(72653)=(S=14172880,E=14174143,C=29)
	CT(72654)=(S=14174144,E=14174175,C=78)
	CT(72655)=(S=14174176,E=14174527,C=29)
	CT(72656)=(S=14174528,E=14174543,C=78)
	CT(72657)=(S=14174560,E=14175135,C=29)
	CT(72658)=(S=14175136,E=14175167,C=78)
	CT(72659)=(S=14175168,E=14176511,C=29)
	CT(72660)=(S=14176512,E=14176543,C=78)
	CT(72661)=(S=14176544,E=14176559,C=29)
	CT(72662)=(S=14176560,E=14176575,C=78)
	CT(72663)=(S=14176576,E=14177023,C=29)
	CT(72664)=(S=14177024,E=14177055,C=78)
	CT(72665)=(S=14177056,E=14177887,C=29)
	CT(72666)=(S=14177888,E=14177919,C=78)
	CT(72667)=(S=14177920,E=14180895,C=29)
	CT(72668)=(S=14180896,E=14180927,C=78)
	CT(72669)=(S=14180928,E=14181151,C=29)
	CT(72670)=(S=14181152,E=14181183,C=78)
	CT(72671)=(S=14181184,E=14181215,C=29)
	CT(72672)=(S=14181216,E=14181231,C=78)
	CT(72673)=(S=14181232,E=14181317,C=29)
	CT(72674)=(S=14181318,E=14181318,C=0)
	CT(72675)=(S=14181319,E=14181320,C=29)
	CT(72676)=(S=14181321,E=14181321,C=0)
	CT(72677)=(S=14181322,E=14181323,C=29)
	CT(72678)=(S=14181324,E=14181324,C=0)
	CT(72679)=(S=14181325,E=14182495,C=29)
	CT(72680)=(S=14182496,E=14182527,C=78)
	CT(72681)=(S=14182528,E=14182735,C=29)
	CT(72682)=(S=14182736,E=14182751,C=78)
	CT(72683)=(S=14182752,E=14183007,C=29)
	CT(72684)=(S=14183008,E=14183023,C=78)
	CT(72685)=(S=14183024,E=14183423,C=29)
	CT(72686)=(S=14183424,E=14183615,C=78)
	CT(72687)=(S=14183616,E=14184031,C=29)
	CT(72688)=(S=14184032,E=14184063,C=79)
	CT(72689)=(S=14184064,E=14184159,C=29)
	CT(72690)=(S=14184160,E=14184191,C=78)
	CT(72691)=(S=14184192,E=14184703,C=29)
	CT(72692)=(S=14184704,E=14184831,C=78)
	CT(72693)=(S=14184832,E=14184895,C=29)
	CT(72694)=(S=14184896,E=14184927,C=78)
	CT(72695)=(S=14184928,E=14186111,C=29)
	CT(72696)=(S=14186112,E=14186143,C=78)
	CT(72697)=(S=14186144,E=14186191,C=29)
	CT(72698)=(S=14186192,E=14186207,C=78)
	CT(72699)=(S=14186208,E=14186879,C=29)
	CT(72700)=(S=14186880,E=14187007,C=78)
	CT(72701)=(S=14187008,E=14187263,C=29)
	CT(72702)=(S=14187264,E=14187519,C=78)
	CT(72703)=(S=14187520,E=14188095,C=29)
	CT(72704)=(S=14188096,E=14188159,C=78)
	CT(72705)=(S=14188160,E=14188231,C=29)
	CT(72706)=(S=14188232,E=14188235,C=78)
	CT(72707)=(S=14188236,E=14188271,C=29)
	CT(72708)=(S=14188272,E=14188287,C=78)
	CT(72709)=(S=14188288,E=14188799,C=29)
	CT(72710)=(S=14188800,E=14188895,C=78)
	CT(72711)=(S=14188896,E=14188969,C=29)
	CT(72712)=(S=14188970,E=14188973,C=78)
	CT(72713)=(S=14188974,E=14188991,C=29)
	CT(72714)=(S=14188992,E=14189023,C=78)
	CT(72715)=(S=14189024,E=14189119,C=29)
	CT(72716)=(S=14189120,E=14189151,C=78)
	CT(72717)=(S=14189152,E=14189263,C=29)
	CT(72718)=(S=14189264,E=14189279,C=78)
	CT(72719)=(S=14189280,E=14190847,C=29)
	CT(72720)=(S=14190848,E=14190863,C=81)
	CT(72721)=(S=14190864,E=14190943,C=29)
	CT(72722)=(S=14190944,E=14190975,C=78)
	CT(72723)=(S=14190976,E=14191007,C=29)
	CT(72724)=(S=14191008,E=14191023,C=78)
	CT(72725)=(S=14191040,E=14191295,C=29)
	CT(72726)=(S=14191296,E=14191359,C=78)
	CT(72727)=(S=14191360,E=14192719,C=29)
	CT(72728)=(S=14192720,E=14192735,C=174)
	CT(72729)=(S=14192736,E=14192751,C=29)
	CT(72730)=(S=14192752,E=14192767,C=78)
	CT(72731)=(S=14192768,E=14192991,C=29)
	CT(72732)=(S=14192992,E=14193007,C=78)
	CT(72733)=(S=14193008,E=14193279,C=29)
	CT(72734)=(S=14193280,E=14193295,C=78)
	CT(72735)=(S=14193296,E=14194623,C=29)
	CT(72736)=(S=14194624,E=14194639,C=8)
	CT(72737)=(S=14194640,E=14194847,C=29)
	CT(72738)=(S=14194848,E=14194863,C=207)
	CT(72739)=(S=14194864,E=14194879,C=29)
	CT(72740)=(S=14194880,E=14194895,C=78)
	CT(72741)=(S=14194896,E=14195199,C=29)
	CT(72742)=(S=14195200,E=14195391,C=78)
	CT(72743)=(S=14195392,E=14195519,C=29)
	CT(72744)=(S=14195520,E=14195551,C=203)
	CT(72745)=(S=14195552,E=14195679,C=29)
	CT(72746)=(S=14195680,E=14195695,C=19)
	CT(72747)=(S=14195696,E=14197311,C=29)
	CT(72748)=(S=14197312,E=14197327,C=78)
	CT(72749)=(S=14197328,E=14197471,C=29)
	CT(72750)=(S=14197472,E=14197487,C=78)
	CT(72751)=(S=14197488,E=14198207,C=29)
	CT(72752)=(S=14198208,E=14198239,C=78)
	CT(72753)=(S=14198240,E=14198751,C=29)
	CT(72754)=(S=14198752,E=14198783,C=78)
	CT(72755)=(S=14198784,E=14198879,C=29)
	CT(72756)=(S=14198880,E=14198911,C=78)
	CT(72757)=(S=14198912,E=14198927,C=29)
	CT(72758)=(S=14198928,E=14198943,C=78)
	CT(72759)=(S=14198944,E=14199647,C=29)
	CT(72760)=(S=14199648,E=14199663,C=78)
	CT(72761)=(S=14199664,E=14199775,C=29)
	CT(72762)=(S=14199776,E=14199791,C=78)
	CT(72763)=(S=14199792,E=14199839,C=29)
	CT(72764)=(S=14199840,E=14199855,C=78)
	CT(72765)=(S=14199856,E=14200447,C=29)
	CT(72766)=(S=14200448,E=14200479,C=78)
	CT(72767)=(S=14200480,E=14201039,C=29)
	CT(72768)=(S=14201040,E=14201055,C=78)
	CT(72769)=(S=14201056,E=14202111,C=29)
	CT(72770)=(S=14202112,E=14202112,C=78)
	CT(72771)=(S=14202113,E=14202114,C=29)
	CT(72772)=(S=14202115,E=14202115,C=78)
	CT(72773)=(S=14202116,E=14202116,C=29)
	CT(72774)=(S=14202117,E=14202130,C=78)
	CT(72775)=(S=14202131,E=14202132,C=29)
	CT(72776)=(S=14202133,E=14202209,C=78)
	CT(72777)=(S=14202210,E=14202220,C=29)
	CT(72778)=(S=14202221,E=14202231,C=78)
	CT(72779)=(S=14202232,E=14202232,C=29)
	CT(72780)=(S=14202233,E=14202233,C=78)
	CT(72781)=(S=14202234,E=14202235,C=29)
	CT(72782)=(S=14202236,E=14202236,C=78)
	CT(72783)=(S=14202237,E=14202237,C=29)
	CT(72784)=(S=14202238,E=14202241,C=78)
	CT(72785)=(S=14202242,E=14202242,C=29)
	CT(72786)=(S=14202243,E=14202269,C=78)
	CT(72787)=(S=14202270,E=14202277,C=29)
	CT(72788)=(S=14202278,E=14202341,C=78)
	CT(72789)=(S=14202342,E=14202343,C=29)
	CT(72790)=(S=14202344,E=14202359,C=78)
	CT(72791)=(S=14202360,E=14202361,C=29)
	CT(72792)=(S=14202362,E=14202367,C=78)
	CT(72793)=(S=14202368,E=14202623,C=29)
	CT(72794)=(S=14202624,E=14202655,C=78)
	CT(72795)=(S=14202656,E=14202703,C=29)
	CT(72796)=(S=14202704,E=14202719,C=78)
	CT(72797)=(S=14202720,E=14202751,C=29)
	CT(72798)=(S=14202752,E=14202783,C=78)
	CT(72799)=(S=14202784,E=14203199,C=29)
	CT(72800)=(S=14203200,E=14203231,C=78)
	CT(72801)=(S=14203232,E=14203375,C=29)
	CT(72802)=(S=14203376,E=14203391,C=78)
	CT(72803)=(S=14203392,E=14204671,C=29)
	CT(72804)=(S=14204672,E=14204927,C=78)
	CT(72805)=(S=14204928,E=14205503,C=29)
	CT(72806)=(S=14205504,E=14205551,C=78)
	CT(72807)=(S=14205552,E=14206335,C=29)
	CT(72808)=(S=14206336,E=14206463,C=78)
	CT(72809)=(S=14206464,E=14206591,C=29)
	CT(72810)=(S=14206592,E=14206623,C=78)
	CT(72811)=(S=14206624,E=14207327,C=29)
	CT(72812)=(S=14207328,E=14207343,C=78)
	CT(72813)=(S=14207344,E=14209023,C=29)
	CT(72814)=(S=14209024,E=14209535,C=78)
	CT(72815)=(S=14209536,E=14209631,C=29)
	CT(72816)=(S=14209632,E=14209647,C=78)
	CT(72817)=(S=14209648,E=14209791,C=29)
	CT(72818)=(S=14209792,E=14209919,C=78)
	CT(72819)=(S=14209920,E=14211583,C=29)
	CT(72820)=(S=14211584,E=14211647,C=78)
	CT(72821)=(S=14211648,E=14212127,C=29)
	CT(72822)=(S=14212128,E=14212159,C=78)
	CT(72823)=(S=14212160,E=14212367,C=29)
	CT(72824)=(S=14212384,E=14212447,C=78)
	CT(72825)=(S=14212448,E=14212927,C=29)
	CT(72826)=(S=14212928,E=14213023,C=78)
	CT(72827)=(S=14213024,E=14213663,C=29)
	CT(72828)=(S=14213664,E=14213695,C=78)
	CT(72829)=(S=14213696,E=14214351,C=29)
	CT(72830)=(S=14214352,E=14214367,C=78)
	CT(72831)=(S=14214368,E=14214783,C=29)
	CT(72832)=(S=14214784,E=14214815,C=215)
	CT(72833)=(S=14214816,E=14215167,C=29)
	CT(72834)=(S=14215168,E=14215423,C=78)
	CT(72835)=(S=14215424,E=14215711,C=29)
	CT(72836)=(S=14215712,E=14215743,C=78)
	CT(72837)=(S=14215744,E=14215839,C=29)
	CT(72838)=(S=14215840,E=14215871,C=78)
	CT(72839)=(S=14215872,E=14215935,C=29)
	CT(72840)=(S=14215936,E=14215951,C=78)
	CT(72841)=(S=14215952,E=14216367,C=29)
	CT(72842)=(S=14216368,E=14216383,C=105)
	CT(72843)=(S=14216384,E=14216927,C=29)
	CT(72844)=(S=14216928,E=14216943,C=78)
	CT(72845)=(S=14216944,E=14217023,C=29)
	CT(72846)=(S=14217024,E=14217055,C=78)
	CT(72847)=(S=14217056,E=14217215,C=29)
	CT(72848)=(S=14217216,E=14217231,C=78)
	CT(72849)=(S=14217232,E=14217407,C=29)
	CT(72850)=(S=14217408,E=14217423,C=78)
	CT(72851)=(S=14217424,E=14217439,C=29)
	CT(72852)=(S=14217440,E=14217455,C=78)
	CT(72853)=(S=14217456,E=14217471,C=29)
	CT(72854)=(S=14217472,E=14217503,C=181)
	CT(72855)=(S=14217504,E=14217695,C=29)
	CT(72856)=(S=14217696,E=14217711,C=78)
	CT(72857)=(S=14217712,E=14218367,C=29)
	CT(72858)=(S=14218368,E=14218431,C=212)
	CT(72859)=(S=14218432,E=14218495,C=83)
	CT(72860)=(S=14218496,E=14218975,C=29)
	CT(72861)=(S=14218976,E=14219007,C=78)
	CT(72862)=(S=14219008,E=14219519,C=29)
	CT(72863)=(S=14219520,E=14219535,C=78)
	CT(72864)=(S=14219536,E=14219551,C=29)
	CT(72865)=(S=14219552,E=14219567,C=178)
	CT(72866)=(S=14219568,E=14219583,C=78)
	CT(72867)=(S=14219584,E=14220159,C=29)
	CT(72868)=(S=14220160,E=14220191,C=78)
	CT(72869)=(S=14220192,E=14220351,C=29)
	CT(72870)=(S=14220352,E=14220383,C=78)
	CT(72871)=(S=14220384,E=14220431,C=29)
	CT(72872)=(S=14220432,E=14220439,C=28)
	CT(72873)=(S=14220440,E=14220442,C=29)
	CT(72874)=(S=14220443,E=14220443,C=28)
	CT(72875)=(S=14220444,E=14220463,C=29)
	CT(72876)=(S=14220464,E=14220479,C=28)
	CT(72877)=(S=14220480,E=14220495,C=29)
	CT(72878)=(S=14220496,E=14220503,C=78)
	CT(72879)=(S=14220504,E=14220507,C=28)
	CT(72880)=(S=14220508,E=14220511,C=29)
	CT(72881)=(S=14220512,E=14220543,C=28)
	CT(72882)=(S=14220544,E=14220927,C=29)
	CT(72883)=(S=14220928,E=14221023,C=78)
	CT(72884)=(S=14221024,E=14221279,C=29)
	CT(72885)=(S=14221280,E=14221295,C=2)
	CT(72886)=(S=14221296,E=14221311,C=29)
	CT(72887)=(S=14221312,E=14223359,C=28)
	CT(72888)=(S=14223360,E=14223391,C=14)
	CT(72889)=(S=14223392,E=14223407,C=103)
	CT(72890)=(S=14223408,E=14223423,C=28)
	CT(72891)=(S=14223424,E=14223439,C=23)
	CT(72892)=(S=14223440,E=14223455,C=18)
	CT(72893)=(S=14223456,E=14223471,C=36)
	CT(72894)=(S=14223488,E=14223519,C=27)
	CT(72895)=(S=14223520,E=14223551,C=58)
	CT(72896)=(S=14223552,E=14223583,C=34)
	CT(72897)=(S=14223584,E=14223599,C=18)
	CT(72898)=(S=14223600,E=14223615,C=14)
	CT(72899)=(S=14223616,E=14223631,C=31)
	CT(72900)=(S=14223648,E=14223679,C=28)
	CT(72901)=(S=14223680,E=14223695,C=15)
	CT(72902)=(S=14223696,E=14223711,C=18)
	CT(72903)=(S=14223712,E=14223743,C=28)
	CT(72904)=(S=14223744,E=14223759,C=48)
	CT(72905)=(S=14223760,E=14223775,C=18)
	CT(72906)=(S=14223776,E=14223807,C=58)
	CT(72907)=(S=14223808,E=14223823,C=14)
	CT(72908)=(S=14223824,E=14223839,C=19)
	CT(72909)=(S=14223840,E=14223855,C=51)
	CT(72910)=(S=14223856,E=14223871,C=44)
	CT(72911)=(S=14223872,E=14223887,C=28)
	CT(72912)=(S=14223888,E=14223903,C=24)
	CT(72913)=(S=14223904,E=14223919,C=18)
	CT(72914)=(S=14223920,E=14223951,C=28)
	CT(72915)=(S=14223952,E=14223967,C=25)
	CT(72916)=(S=14223968,E=14223999,C=19)
	CT(72917)=(S=14224000,E=14224063,C=14)
	CT(72918)=(S=14224064,E=14224095,C=38)
	CT(72919)=(S=14224096,E=14224111,C=14)
	CT(72920)=(S=14224112,E=14224127,C=51)
	CT(72921)=(S=14224128,E=14224143,C=15)
	CT(72922)=(S=14224144,E=14224159,C=26)
	CT(72923)=(S=14224160,E=14224175,C=34)
	CT(72924)=(S=14224176,E=14224191,C=28)
	CT(72925)=(S=14224192,E=14224207,C=20)
	CT(72926)=(S=14224208,E=14224223,C=15)
	CT(72927)=(S=14224224,E=14224255,C=25)
	CT(72928)=(S=14224256,E=14224271,C=58)
	CT(72929)=(S=14224272,E=14224287,C=28)
	CT(72930)=(S=14224288,E=14224303,C=40)
	CT(72931)=(S=14224304,E=14224319,C=103)
	CT(72932)=(S=14224320,E=14224335,C=28)
	CT(72933)=(S=14224336,E=14224351,C=34)
	CT(72934)=(S=14224352,E=14224383,C=33)
	CT(72935)=(S=14224384,E=14224399,C=14)
	CT(72936)=(S=14224400,E=14224415,C=25)
	CT(72937)=(S=14224416,E=14224431,C=18)
	CT(72938)=(S=14224432,E=14224447,C=71)
	CT(72939)=(S=14224448,E=14224463,C=18)
	CT(72940)=(S=14224464,E=14224479,C=98)
	CT(72941)=(S=14224480,E=14224495,C=18)
	CT(72942)=(S=14224496,E=14224511,C=53)
	CT(72943)=(S=14224512,E=14224527,C=15)
	CT(72944)=(S=14224528,E=14224543,C=14)
	CT(72945)=(S=14224544,E=14224575,C=15)
	CT(72946)=(S=14224576,E=14224607,C=31)
	CT(72947)=(S=14224608,E=14224623,C=23)
	CT(72948)=(S=14224624,E=14224639,C=18)
	CT(72949)=(S=14224640,E=14224671,C=27)
	CT(72950)=(S=14224672,E=14224687,C=50)
	CT(72951)=(S=14224688,E=14224703,C=12)
	CT(72952)=(S=14224704,E=14224719,C=28)
	CT(72953)=(S=14224720,E=14224735,C=25)
	CT(72954)=(S=14224736,E=14224751,C=50)
	CT(72955)=(S=14224752,E=14224767,C=25)
	CT(72956)=(S=14224768,E=14224799,C=14)
	CT(72957)=(S=14224800,E=14224815,C=28)
	CT(72958)=(S=14224816,E=14224831,C=42)
	CT(72959)=(S=14224832,E=14224847,C=28)
	CT(72960)=(S=14224848,E=14224863,C=18)
	CT(72961)=(S=14224864,E=14224895,C=19)
	CT(72962)=(S=14224896,E=14224911,C=27)
	CT(72963)=(S=14224912,E=14224927,C=18)
	CT(72964)=(S=14224928,E=14224943,C=25)
	CT(72965)=(S=14224944,E=14224959,C=18)
	CT(72966)=(S=14224960,E=14224975,C=34)
	CT(72967)=(S=14224976,E=14224991,C=111)
	CT(72968)=(S=14224992,E=14225007,C=41)
	CT(72969)=(S=14225008,E=14225023,C=28)
	CT(72970)=(S=14225024,E=14225039,C=14)
	CT(72971)=(S=14225040,E=14225071,C=28)
	CT(72972)=(S=14225072,E=14225087,C=14)
	CT(72973)=(S=14225088,E=14225103,C=18)
	CT(72974)=(S=14225104,E=14225119,C=57)
	CT(72975)=(S=14225120,E=14225135,C=42)
	CT(72976)=(S=14225136,E=14225151,C=18)
	CT(72977)=(S=14225152,E=14225167,C=42)
	CT(72978)=(S=14225168,E=14225183,C=18)
	CT(72979)=(S=14225184,E=14225199,C=25)
	CT(72980)=(S=14225200,E=14225215,C=18)
	CT(72981)=(S=14225216,E=14225231,C=14)
	CT(72982)=(S=14225232,E=14225247,C=12)
	CT(72983)=(S=14225248,E=14225263,C=72)
	CT(72984)=(S=14225264,E=14225279,C=14)
	CT(72985)=(S=14225280,E=14225311,C=18)
	CT(72986)=(S=14225312,E=14225327,C=13)
	CT(72987)=(S=14225328,E=14225343,C=20)
	CT(72988)=(S=14225344,E=14225359,C=18)
	CT(72989)=(S=14225360,E=14225375,C=15)
	CT(72990)=(S=14225376,E=14225391,C=49)
	CT(72991)=(S=14225392,E=14225407,C=19)
	CT(72992)=(S=14225408,E=14225423,C=12)
	CT(72993)=(S=14225424,E=14225439,C=18)
	CT(72994)=(S=14225440,E=14225455,C=49)
	CT(72995)=(S=14225456,E=14225471,C=14)
	CT(72996)=(S=14225472,E=14225503,C=54)
	CT(72997)=(S=14225504,E=14225519,C=24)
	CT(72998)=(S=14225520,E=14225535,C=42)
	CT(72999)=(S=14225536,E=14225551,C=46)
	CT(73000)=(S=14225552,E=14225567,C=25)
	CT(73001)=(S=14225568,E=14225583,C=28)
	CT(73002)=(S=14225584,E=14225599,C=33)
	CT(73003)=(S=14225600,E=14225615,C=19)
	CT(73004)=(S=14225616,E=14225631,C=14)
	CT(73005)=(S=14225632,E=14225647,C=32)
	CT(73006)=(S=14225648,E=14225663,C=25)
	CT(73007)=(S=14225664,E=14225679,C=27)
	CT(73008)=(S=14225680,E=14225695,C=28)
	CT(73009)=(S=14225696,E=14225711,C=58)
	CT(73010)=(S=14225712,E=14225727,C=51)
	CT(73011)=(S=14225728,E=14225743,C=19)
	CT(73012)=(S=14225744,E=14225759,C=44)
	CT(73013)=(S=14225760,E=14225775,C=46)
	CT(73014)=(S=14225776,E=14225791,C=18)
	CT(73015)=(S=14225792,E=14225807,C=36)
	CT(73016)=(S=14225808,E=14225823,C=37)
	CT(73017)=(S=14225824,E=14225855,C=18)
	CT(73018)=(S=14225856,E=14225871,C=28)
	CT(73019)=(S=14225872,E=14225887,C=27)
	CT(73020)=(S=14225888,E=14225919,C=104)
	CT(73021)=(S=14225920,E=14225935,C=15)
	CT(73022)=(S=14225936,E=14225951,C=14)
	CT(73023)=(S=14225952,E=14225967,C=25)
	CT(73024)=(S=14225984,E=14225999,C=36)
	CT(73025)=(S=14226000,E=14226015,C=14)
	CT(73026)=(S=14226016,E=14226047,C=15)
	CT(73027)=(S=14226048,E=14226079,C=18)
	CT(73028)=(S=14226080,E=14226095,C=25)
	CT(73029)=(S=14226096,E=14226111,C=28)
	CT(73030)=(S=14226112,E=14226127,C=27)
	CT(73031)=(S=14226128,E=14226143,C=15)
	CT(73032)=(S=14226144,E=14226159,C=25)
	CT(73033)=(S=14226160,E=14226175,C=51)
	CT(73034)=(S=14226176,E=14226191,C=38)
	CT(73035)=(S=14226192,E=14226207,C=36)
	CT(73036)=(S=14226208,E=14226223,C=42)
	CT(73037)=(S=14226224,E=14226239,C=12)
	CT(73038)=(S=14226240,E=14226271,C=22)
	CT(73039)=(S=14226272,E=14226303,C=18)
	CT(73040)=(S=14226304,E=14226319,C=14)
	CT(73041)=(S=14226320,E=14226335,C=15)
	CT(73042)=(S=14226336,E=14226367,C=28)
	CT(73043)=(S=14226368,E=14226383,C=12)
	CT(73044)=(S=14226384,E=14226399,C=53)
	CT(73045)=(S=14226400,E=14226415,C=49)
	CT(73046)=(S=14226416,E=14226431,C=14)
	CT(73047)=(S=14226432,E=14226447,C=15)
	CT(73048)=(S=14226448,E=14226479,C=14)
	CT(73049)=(S=14226480,E=14226495,C=24)
	CT(73050)=(S=14226496,E=14226527,C=18)
	CT(73051)=(S=14226528,E=14226543,C=19)
	CT(73052)=(S=14226544,E=14226559,C=28)
	CT(73053)=(S=14226560,E=14226575,C=50)
	CT(73054)=(S=14226576,E=14226591,C=18)
	CT(73055)=(S=14226592,E=14226623,C=31)
	CT(73056)=(S=14226624,E=14226639,C=34)
	CT(73057)=(S=14226640,E=14226655,C=30)
	CT(73058)=(S=14226656,E=14226671,C=106)
	CT(73059)=(S=14226672,E=14226687,C=14)
	CT(73060)=(S=14226688,E=14226703,C=43)
	CT(73061)=(S=14226704,E=14226719,C=50)
	CT(73062)=(S=14226720,E=14226751,C=68)
	CT(73063)=(S=14226752,E=14226783,C=14)
	CT(73064)=(S=14226784,E=14226799,C=18)
	CT(73065)=(S=14226800,E=14226815,C=113)
	CT(73066)=(S=14226816,E=14226831,C=19)
	CT(73067)=(S=14226832,E=14226847,C=96)
	CT(73068)=(S=14226848,E=14226863,C=60)
	CT(73069)=(S=14226864,E=14226879,C=49)
	CT(73070)=(S=14226880,E=14226895,C=36)
	CT(73071)=(S=14226896,E=14226911,C=18)
	CT(73072)=(S=14226912,E=14226927,C=19)
	CT(73073)=(S=14226928,E=14226943,C=36)
	CT(73074)=(S=14226944,E=14226959,C=14)
	CT(73075)=(S=14226960,E=14226971,C=27)
	CT(73076)=(S=14226972,E=14226975,C=13)
	CT(73077)=(S=14226976,E=14226991,C=27)
	CT(73078)=(S=14226992,E=14227007,C=49)
	CT(73079)=(S=14227008,E=14227023,C=27)
	CT(73080)=(S=14227024,E=14227039,C=14)
	CT(73081)=(S=14227040,E=14227055,C=25)
	CT(73082)=(S=14227056,E=14227071,C=17)
	CT(73083)=(S=14227072,E=14227087,C=28)
	CT(73084)=(S=14227088,E=14227103,C=14)
	CT(73085)=(S=14227104,E=14227119,C=18)
	CT(73086)=(S=14227120,E=14227135,C=72)
	CT(73087)=(S=14227136,E=14227151,C=28)
	CT(73088)=(S=14227152,E=14227199,C=15)
	CT(73089)=(S=14227200,E=14227215,C=36)
	CT(73090)=(S=14227216,E=14227231,C=18)
	CT(73091)=(S=14227232,E=14227247,C=61)
	CT(73092)=(S=14227248,E=14227263,C=28)
	CT(73093)=(S=14227264,E=14227295,C=18)
	CT(73094)=(S=14227296,E=14227311,C=24)
	CT(73095)=(S=14227312,E=14227327,C=68)
	CT(73096)=(S=14227328,E=14227359,C=18)
	CT(73097)=(S=14227360,E=14227375,C=14)
	CT(73098)=(S=14227376,E=14227391,C=18)
	CT(73099)=(S=14227392,E=14227407,C=59)
	CT(73100)=(S=14227408,E=14227423,C=50)
	CT(73101)=(S=14227424,E=14227439,C=14)
	CT(73102)=(S=14227440,E=14227455,C=71)
	CT(73103)=(S=14227456,E=14227471,C=28)
	CT(73104)=(S=14227472,E=14227487,C=46)
	CT(73105)=(S=14227488,E=14227503,C=34)
	CT(73106)=(S=14227504,E=14227519,C=28)
	CT(73107)=(S=14227520,E=14227535,C=60)
	CT(73108)=(S=14227536,E=14227551,C=12)
	CT(73109)=(S=14227552,E=14227567,C=47)
	CT(73110)=(S=14227568,E=14227583,C=18)
	CT(73111)=(S=14227584,E=14227599,C=59)
	CT(73112)=(S=14227600,E=14227615,C=16)
	CT(73113)=(S=14227616,E=14227631,C=31)
	CT(73114)=(S=14227632,E=14227647,C=18)
	CT(73115)=(S=14227648,E=14227695,C=28)
	CT(73116)=(S=14227696,E=14227711,C=96)
	CT(73117)=(S=14227712,E=14227727,C=14)
	CT(73118)=(S=14227728,E=14227743,C=41)
	CT(73119)=(S=14227744,E=14227759,C=19)
	CT(73120)=(S=14227760,E=14227775,C=26)
	CT(73121)=(S=14227776,E=14227791,C=28)
	CT(73122)=(S=14227792,E=14227807,C=18)
	CT(73123)=(S=14227808,E=14227823,C=47)
	CT(73124)=(S=14227824,E=14227839,C=42)
	CT(73125)=(S=14227840,E=14227855,C=28)
	CT(73126)=(S=14227856,E=14227871,C=18)
	CT(73127)=(S=14227872,E=14227887,C=28)
	CT(73128)=(S=14227888,E=14227903,C=12)
	CT(73129)=(S=14227904,E=14227919,C=31)
	CT(73130)=(S=14227920,E=14227951,C=18)
	CT(73131)=(S=14227952,E=14227967,C=19)
	CT(73132)=(S=14227968,E=14227999,C=18)
	CT(73133)=(S=14228000,E=14228031,C=34)
	CT(73134)=(S=14228032,E=14228047,C=46)
	CT(73135)=(S=14228048,E=14228063,C=15)
	CT(73136)=(S=14228064,E=14228095,C=36)
	CT(73137)=(S=14228096,E=14228103,C=42)
	CT(73138)=(S=14228104,E=14228111,C=12)
	CT(73139)=(S=14228112,E=14228143,C=53)
	CT(73140)=(S=14228160,E=14228175,C=12)
	CT(73141)=(S=14228176,E=14228183,C=46)
	CT(73142)=(S=14228184,E=14228191,C=14)
	CT(73143)=(S=14228192,E=14228199,C=28)
	CT(73144)=(S=14228200,E=14228207,C=14)
	CT(73145)=(S=14228208,E=14228239,C=28)
	CT(73146)=(S=14228240,E=14228255,C=14)
	CT(73147)=(S=14228256,E=14228287,C=32)
	CT(73148)=(S=14228288,E=14228319,C=15)
	CT(73149)=(S=14228320,E=14228335,C=34)
	CT(73150)=(S=14228336,E=14228351,C=15)
	CT(73151)=(S=14228352,E=14228367,C=18)
	CT(73152)=(S=14228368,E=14228383,C=31)
	CT(73153)=(S=14228384,E=14228415,C=19)
	CT(73154)=(S=14228416,E=14228431,C=28)
	CT(73155)=(S=14228432,E=14228447,C=50)
	CT(73156)=(S=14228448,E=14228463,C=36)
	CT(73157)=(S=14228464,E=14228511,C=14)
	CT(73158)=(S=14228512,E=14228527,C=19)
	CT(73159)=(S=14228528,E=14228543,C=60)
	CT(73160)=(S=14228544,E=14228551,C=12)
	CT(73161)=(S=14228552,E=14228559,C=18)
	CT(73162)=(S=14228560,E=14228575,C=33)
	CT(73163)=(S=14228576,E=14228591,C=28)
	CT(73164)=(S=14228592,E=14228607,C=15)
	CT(73165)=(S=14228608,E=14228623,C=14)
	CT(73166)=(S=14228624,E=14228639,C=58)
	CT(73167)=(S=14228640,E=14228655,C=24)
	CT(73168)=(S=14228656,E=14228671,C=48)
	CT(73169)=(S=14228672,E=14228687,C=19)
	CT(73170)=(S=14228688,E=14228719,C=18)
	CT(73171)=(S=14228720,E=14228735,C=28)
	CT(73172)=(S=14228736,E=14228751,C=33)
	CT(73173)=(S=14228752,E=14228767,C=62)
	CT(73174)=(S=14228768,E=14228783,C=28)
	CT(73175)=(S=14228784,E=14228799,C=18)
	CT(73176)=(S=14228800,E=14228815,C=15)
	CT(73177)=(S=14228816,E=14228831,C=18)
	CT(73178)=(S=14228832,E=14228847,C=25)
	CT(73179)=(S=14228848,E=14228863,C=98)
	CT(73180)=(S=14228864,E=14228879,C=112)
	CT(73181)=(S=14228880,E=14228895,C=42)
	CT(73182)=(S=14228896,E=14228911,C=15)
	CT(73183)=(S=14228912,E=14228927,C=18)
	CT(73184)=(S=14228928,E=14228943,C=14)
	CT(73185)=(S=14228944,E=14228959,C=105)
	CT(73186)=(S=14228960,E=14228975,C=47)
	CT(73187)=(S=14228976,E=14228991,C=61)
	CT(73188)=(S=14228992,E=14229007,C=15)
	CT(73189)=(S=14229008,E=14229023,C=49)
	CT(73190)=(S=14229024,E=14229039,C=24)
	CT(73191)=(S=14229040,E=14229055,C=28)
	CT(73192)=(S=14229056,E=14229071,C=33)
	CT(73193)=(S=14229072,E=14229087,C=24)
	CT(73194)=(S=14229088,E=14229103,C=72)
	CT(73195)=(S=14229104,E=14229119,C=14)
	CT(73196)=(S=14229120,E=14229151,C=58)
	CT(73197)=(S=14229152,E=14229167,C=98)
	CT(73198)=(S=14229168,E=14229183,C=47)
	CT(73199)=(S=14229184,E=14229199,C=31)
	CT(73200)=(S=14229200,E=14229215,C=51)
	CT(73201)=(S=14229216,E=14229231,C=28)
	CT(73202)=(S=14229232,E=14229247,C=18)
	CT(73203)=(S=14229248,E=14229263,C=14)
	CT(73204)=(S=14229264,E=14229279,C=28)
	CT(73205)=(S=14229280,E=14229295,C=71)
	CT(73206)=(S=14229296,E=14229311,C=33)
	CT(73207)=(S=14229312,E=14229327,C=70)
	CT(73208)=(S=14229328,E=14229343,C=28)
	CT(73209)=(S=14229344,E=14229359,C=27)
	CT(73210)=(S=14229360,E=14229375,C=15)
	CT(73211)=(S=14229376,E=14229407,C=14)
	CT(73212)=(S=14229408,E=14229439,C=19)
	CT(73213)=(S=14229440,E=14229455,C=33)
	CT(73214)=(S=14229456,E=14229471,C=28)
	CT(73215)=(S=14229472,E=14229503,C=37)
	CT(73216)=(S=14229504,E=14233599,C=14)
	CT(73217)=(S=14233600,E=14234623,C=28)
	CT(73218)=(S=14234624,E=14235647,C=106)
	CT(73219)=(S=14235648,E=14236671,C=15)
	CT(73220)=(S=14236672,E=14237183,C=24)
	CT(73221)=(S=14237184,E=14237535,C=36)
	CT(73222)=(S=14237536,E=14237567,C=66)
	CT(73223)=(S=14237568,E=14237695,C=36)
	CT(73224)=(S=14237696,E=14237711,C=33)
	CT(73225)=(S=14237712,E=14237727,C=41)
	CT(73226)=(S=14237728,E=14237743,C=47)
	CT(73227)=(S=14237744,E=14237759,C=12)
	CT(73228)=(S=14237760,E=14237791,C=28)
	CT(73229)=(S=14237792,E=14237807,C=123)
	CT(73230)=(S=14237808,E=14237823,C=14)
	CT(73231)=(S=14237824,E=14237839,C=18)
	CT(73232)=(S=14237840,E=14237855,C=26)
	CT(73233)=(S=14237856,E=14237871,C=28)
	CT(73234)=(S=14237872,E=14237887,C=47)
	CT(73235)=(S=14237888,E=14237903,C=15)
	CT(73236)=(S=14237904,E=14237919,C=37)
	CT(73237)=(S=14237920,E=14237935,C=14)
	CT(73238)=(S=14237936,E=14237951,C=49)
	CT(73239)=(S=14237952,E=14237967,C=18)
	CT(73240)=(S=14237968,E=14237983,C=28)
	CT(73241)=(S=14237984,E=14237999,C=17)
	CT(73242)=(S=14238000,E=14238015,C=69)
	CT(73243)=(S=14238016,E=14238031,C=15)
	CT(73244)=(S=14238032,E=14238047,C=18)
	CT(73245)=(S=14238048,E=14238079,C=50)
	CT(73246)=(S=14238080,E=14238095,C=28)
	CT(73247)=(S=14238096,E=14238127,C=14)
	CT(73248)=(S=14238128,E=14238143,C=37)
	CT(73249)=(S=14238144,E=14238159,C=46)
	CT(73250)=(S=14238160,E=14238175,C=18)
	CT(73251)=(S=14238176,E=14238191,C=27)
	CT(73252)=(S=14238192,E=14238207,C=31)
	CT(73253)=(S=14238208,E=14238223,C=49)
	CT(73254)=(S=14238224,E=14238239,C=18)
	CT(73255)=(S=14238240,E=14238271,C=28)
	CT(73256)=(S=14238272,E=14238303,C=18)
	CT(73257)=(S=14238304,E=14238319,C=31)
	CT(73258)=(S=14238320,E=14238335,C=12)
	CT(73259)=(S=14238336,E=14238351,C=28)
	CT(73260)=(S=14238352,E=14238367,C=18)
	CT(73261)=(S=14238368,E=14238399,C=33)
	CT(73262)=(S=14238400,E=14238431,C=26)
	CT(73263)=(S=14238432,E=14238463,C=43)
	CT(73264)=(S=14238464,E=14238479,C=18)
	CT(73265)=(S=14238480,E=14238495,C=71)
	CT(73266)=(S=14238496,E=14238511,C=28)
	CT(73267)=(S=14238512,E=14238527,C=14)
	CT(73268)=(S=14238528,E=14238543,C=31)
	CT(73269)=(S=14238544,E=14238559,C=19)
	CT(73270)=(S=14238560,E=14238575,C=28)
	CT(73271)=(S=14238576,E=14238591,C=18)
	CT(73272)=(S=14238592,E=14238607,C=63)
	CT(73273)=(S=14238608,E=14238623,C=12)
	CT(73274)=(S=14238624,E=14238639,C=25)
	CT(73275)=(S=14238640,E=14238655,C=18)
	CT(73276)=(S=14238656,E=14238687,C=58)
	CT(73277)=(S=14238688,E=14238719,C=36)
	CT(73278)=(S=14238720,E=14238735,C=28)
	CT(73279)=(S=14238736,E=14238751,C=14)
	CT(73280)=(S=14238752,E=14238767,C=19)
	CT(73281)=(S=14238768,E=14238783,C=36)
	CT(73282)=(S=14238784,E=14238799,C=14)
	CT(73283)=(S=14238800,E=14238815,C=57)
	CT(73284)=(S=14238816,E=14238847,C=27)
	CT(73285)=(S=14238848,E=14238863,C=14)
	CT(73286)=(S=14238864,E=14238927,C=28)
	CT(73287)=(S=14238928,E=14238943,C=37)
	CT(73288)=(S=14238944,E=14238959,C=34)
	CT(73289)=(S=14238960,E=14238975,C=14)
	CT(73290)=(S=14238976,E=14238991,C=22)
	CT(73291)=(S=14238992,E=14239007,C=12)
	CT(73292)=(S=14239008,E=14239023,C=14)
	CT(73293)=(S=14239024,E=14239039,C=47)
	CT(73294)=(S=14239040,E=14239071,C=28)
	CT(73295)=(S=14239072,E=14239087,C=33)
	CT(73296)=(S=14239088,E=14239103,C=60)
	CT(73297)=(S=14239104,E=14239119,C=18)
	CT(73298)=(S=14239120,E=14239135,C=19)
	CT(73299)=(S=14239136,E=14239151,C=28)
	CT(73300)=(S=14239152,E=14239167,C=101)
	CT(73301)=(S=14239168,E=14239199,C=18)
	CT(73302)=(S=14239200,E=14239231,C=28)
	CT(73303)=(S=14239232,E=14239247,C=25)
	CT(73304)=(S=14239248,E=14239263,C=18)
	CT(73305)=(S=14239264,E=14239279,C=19)
	CT(73306)=(S=14239280,E=14239295,C=58)
	CT(73307)=(S=14239296,E=14239311,C=21)
	CT(73308)=(S=14239312,E=14239327,C=12)
	CT(73309)=(S=14239328,E=14239359,C=18)
	CT(73310)=(S=14239360,E=14239375,C=28)
	CT(73311)=(S=14239376,E=14239391,C=15)
	CT(73312)=(S=14239392,E=14239407,C=28)
	CT(73313)=(S=14239408,E=14239423,C=12)
	CT(73314)=(S=14239424,E=14239439,C=28)
	CT(73315)=(S=14239440,E=14239455,C=36)
	CT(73316)=(S=14239456,E=14239471,C=49)
	CT(73317)=(S=14239472,E=14239487,C=25)
	CT(73318)=(S=14239488,E=14239503,C=24)
	CT(73319)=(S=14239504,E=14239519,C=25)
	CT(73320)=(S=14239520,E=14239535,C=55)
	CT(73321)=(S=14239536,E=14239551,C=59)
	CT(73322)=(S=14239552,E=14239567,C=15)
	CT(73323)=(S=14239568,E=14239583,C=34)
	CT(73324)=(S=14239584,E=14239599,C=13)
	CT(73325)=(S=14239600,E=14239615,C=12)
	CT(73326)=(S=14239616,E=14239631,C=18)
	CT(73327)=(S=14239632,E=14239647,C=47)
	CT(73328)=(S=14239648,E=14239663,C=18)
	CT(73329)=(S=14239664,E=14239679,C=72)
	CT(73330)=(S=14239680,E=14239695,C=25)
	CT(73331)=(S=14239696,E=14239703,C=12)
	CT(73332)=(S=14239704,E=14239711,C=28)
	CT(73333)=(S=14239712,E=14239727,C=18)
	CT(73334)=(S=14239728,E=14239743,C=34)
	CT(73335)=(S=14239744,E=14239759,C=18)
	CT(73336)=(S=14239760,E=14239775,C=19)
	CT(73337)=(S=14239776,E=14239791,C=15)
	CT(73338)=(S=14239792,E=14239807,C=19)
	CT(73339)=(S=14239808,E=14239839,C=52)
	CT(73340)=(S=14239840,E=14239855,C=15)
	CT(73341)=(S=14239856,E=14239871,C=14)
	CT(73342)=(S=14239872,E=14239887,C=28)
	CT(73343)=(S=14239888,E=14239903,C=18)
	CT(73344)=(S=14239904,E=14239935,C=14)
	CT(73345)=(S=14239936,E=14239967,C=28)
	CT(73346)=(S=14239968,E=14239983,C=49)
	CT(73347)=(S=14239984,E=14239999,C=14)
	CT(73348)=(S=14240000,E=14240015,C=19)
	CT(73349)=(S=14240016,E=14240031,C=71)
	CT(73350)=(S=14240032,E=14240055,C=28)
	CT(73351)=(S=14240056,E=14240063,C=18)
	CT(73352)=(S=14240064,E=14240079,C=14)
	CT(73353)=(S=14240080,E=14240095,C=31)
	CT(73354)=(S=14240096,E=14240111,C=19)
	CT(73355)=(S=14240112,E=14240127,C=18)
	CT(73356)=(S=14240128,E=14240143,C=96)
	CT(73357)=(S=14240144,E=14240159,C=28)
	CT(73358)=(S=14240160,E=14240175,C=38)
	CT(73359)=(S=14240176,E=14240191,C=58)
	CT(73360)=(S=14240192,E=14240207,C=18)
	CT(73361)=(S=14240208,E=14240239,C=15)
	CT(73362)=(S=14240240,E=14240255,C=58)
	CT(73363)=(S=14240256,E=14240263,C=28)
	CT(73364)=(S=14240264,E=14240271,C=42)
	CT(73365)=(S=14240272,E=14240279,C=36)
	CT(73366)=(S=14240280,E=14240287,C=37)
	CT(73367)=(S=14240288,E=14240303,C=18)
	CT(73368)=(S=14240304,E=14240319,C=63)
	CT(73369)=(S=14240320,E=14240335,C=58)
	CT(73370)=(S=14240336,E=14240351,C=19)
	CT(73371)=(S=14240352,E=14240367,C=12)
	CT(73372)=(S=14240368,E=14240383,C=18)
	CT(73373)=(S=14240384,E=14240415,C=24)
	CT(73374)=(S=14240416,E=14240431,C=18)
	CT(73375)=(S=14240432,E=14240463,C=28)
	CT(73376)=(S=14240464,E=14240479,C=24)
	CT(73377)=(S=14240480,E=14240495,C=14)
	CT(73378)=(S=14240496,E=14240511,C=18)
	CT(73379)=(S=14240512,E=14240527,C=63)
	CT(73380)=(S=14240528,E=14240543,C=34)
	CT(73381)=(S=14240544,E=14240559,C=36)
	CT(73382)=(S=14240560,E=14240575,C=58)
	CT(73383)=(S=14240576,E=14240607,C=71)
	CT(73384)=(S=14240608,E=14240639,C=19)
	CT(73385)=(S=14240640,E=14240671,C=51)
	CT(73386)=(S=14240672,E=14240687,C=14)
	CT(73387)=(S=14240688,E=14240703,C=42)
	CT(73388)=(S=14240704,E=14240719,C=59)
	CT(73389)=(S=14240720,E=14240735,C=33)
	CT(73390)=(S=14240736,E=14240767,C=25)
	CT(73391)=(S=14240768,E=14240783,C=66)
	CT(73392)=(S=14240784,E=14240799,C=36)
	CT(73393)=(S=14240800,E=14240815,C=18)
	CT(73394)=(S=14240816,E=14240831,C=19)
	CT(73395)=(S=14240832,E=14240847,C=20)
	CT(73396)=(S=14240848,E=14240863,C=19)
	CT(73397)=(S=14240864,E=14240879,C=28)
	CT(73398)=(S=14240880,E=14240895,C=58)
	CT(73399)=(S=14240896,E=14240927,C=25)
	CT(73400)=(S=14240928,E=14240943,C=42)
	CT(73401)=(S=14240944,E=14240959,C=18)
	CT(73402)=(S=14240960,E=14240975,C=31)
	CT(73403)=(S=14240976,E=14240991,C=15)
	CT(73404)=(S=14240992,E=14241007,C=49)
	CT(73405)=(S=14241008,E=14241023,C=25)
	CT(73406)=(S=14241024,E=14241039,C=14)
	CT(73407)=(S=14241040,E=14241055,C=44)
	CT(73408)=(S=14241056,E=14241071,C=27)
	CT(73409)=(S=14241072,E=14241087,C=18)
	CT(73410)=(S=14241088,E=14241103,C=122)
	CT(73411)=(S=14241104,E=14241119,C=15)
	CT(73412)=(S=14241120,E=14241151,C=18)
	CT(73413)=(S=14241152,E=14241183,C=36)
	CT(73414)=(S=14241184,E=14241199,C=33)
	CT(73415)=(S=14241200,E=14241215,C=14)
	CT(73416)=(S=14241216,E=14241231,C=47)
	CT(73417)=(S=14241232,E=14241247,C=31)
	CT(73418)=(S=14241248,E=14241263,C=12)
	CT(73419)=(S=14241264,E=14241279,C=28)
	CT(73420)=(S=14241280,E=14241295,C=63)
	CT(73421)=(S=14241296,E=14241327,C=19)
	CT(73422)=(S=14241328,E=14241343,C=43)
	CT(73423)=(S=14241344,E=14241359,C=150)
	CT(73424)=(S=14241360,E=14241375,C=14)
	CT(73425)=(S=14241376,E=14241391,C=37)
	CT(73426)=(S=14241392,E=14241407,C=34)
	CT(73427)=(S=14241408,E=14241423,C=28)
	CT(73428)=(S=14241424,E=14241439,C=14)
	CT(73429)=(S=14241440,E=14241455,C=28)
	CT(73430)=(S=14241456,E=14241471,C=18)
	CT(73431)=(S=14241472,E=14241503,C=47)
	CT(73432)=(S=14241504,E=14241519,C=18)
	CT(73433)=(S=14241520,E=14241535,C=36)
	CT(73434)=(S=14241536,E=14241567,C=18)
	CT(73435)=(S=14241568,E=14241583,C=51)
	CT(73436)=(S=14241584,E=14241599,C=18)
	CT(73437)=(S=14241600,E=14241631,C=51)
	CT(73438)=(S=14241632,E=14241663,C=14)
	CT(73439)=(S=14241664,E=14241679,C=50)
	CT(73440)=(S=14241680,E=14241695,C=58)
	CT(73441)=(S=14241696,E=14241711,C=14)
	CT(73442)=(S=14241712,E=14241727,C=28)
	CT(73443)=(S=14241728,E=14241743,C=34)
	CT(73444)=(S=14241744,E=14241759,C=28)
	CT(73445)=(S=14241760,E=14241775,C=18)
	CT(73446)=(S=14241776,E=14241791,C=36)
	CT(73447)=(S=14241792,E=14245887,C=28)
	CT(73448)=(S=14245888,E=14246911,C=58)
	CT(73449)=(S=14246912,E=14248447,C=36)
	CT(73450)=(S=14248448,E=14248959,C=18)
	CT(73451)=(S=14248960,E=14249471,C=12)
	CT(73452)=(S=14249472,E=14249983,C=28)
	CT(73453)=(S=14249984,E=14250015,C=18)
	CT(73454)=(S=14250016,E=14250031,C=14)
	CT(73455)=(S=14250032,E=14250063,C=12)
	CT(73456)=(S=14250064,E=14250079,C=14)
	CT(73457)=(S=14250080,E=14250095,C=15)
	CT(73458)=(S=14250096,E=14250111,C=36)
	CT(73459)=(S=14250112,E=14250127,C=50)
	CT(73460)=(S=14250128,E=14250143,C=13)
	CT(73461)=(S=14250144,E=14250159,C=33)
	CT(73462)=(S=14250160,E=14250175,C=49)
	CT(73463)=(S=14250176,E=14250191,C=47)
	CT(73464)=(S=14250192,E=14250207,C=31)
	CT(73465)=(S=14250208,E=14250223,C=12)
	CT(73466)=(S=14250224,E=14250239,C=47)
	CT(73467)=(S=14250240,E=14250271,C=66)
	CT(73468)=(S=14250272,E=14250287,C=28)
	CT(73469)=(S=14250288,E=14250303,C=50)
	CT(73470)=(S=14250304,E=14250335,C=49)
	CT(73471)=(S=14250336,E=14250351,C=42)
	CT(73472)=(S=14250352,E=14250367,C=18)
	CT(73473)=(S=14250368,E=14250399,C=58)
	CT(73474)=(S=14250400,E=14250415,C=14)
	CT(73475)=(S=14250416,E=14250431,C=28)
	CT(73476)=(S=14250432,E=14250447,C=12)
	CT(73477)=(S=14250448,E=14250463,C=15)
	CT(73478)=(S=14250464,E=14250479,C=58)
	CT(73479)=(S=14250480,E=14250495,C=25)
	CT(73480)=(S=14250496,E=14250543,C=18)
	CT(73481)=(S=14250544,E=14250559,C=14)
	CT(73482)=(S=14250560,E=14250575,C=28)
	CT(73483)=(S=14250576,E=14250591,C=19)
	CT(73484)=(S=14250592,E=14250607,C=36)
	CT(73485)=(S=14250608,E=14250623,C=34)
	CT(73486)=(S=14250624,E=14250639,C=25)
	CT(73487)=(S=14250640,E=14250655,C=18)
	CT(73488)=(S=14250656,E=14250671,C=63)
	CT(73489)=(S=14250672,E=14250687,C=18)
	CT(73490)=(S=14250688,E=14250695,C=28)
	CT(73491)=(S=14250696,E=14250703,C=12)
	CT(73492)=(S=14250704,E=14250719,C=28)
	CT(73493)=(S=14250720,E=14250751,C=18)
	CT(73494)=(S=14250752,E=14250767,C=28)
	CT(73495)=(S=14250768,E=14250783,C=15)
	CT(73496)=(S=14250784,E=14250815,C=19)
	CT(73497)=(S=14250816,E=14250831,C=28)
	CT(73498)=(S=14250832,E=14250847,C=18)
	CT(73499)=(S=14250848,E=14250863,C=31)
	CT(73500)=(S=14250864,E=14250879,C=63)
	CT(73501)=(S=14250880,E=14250911,C=28)
	CT(73502)=(S=14250912,E=14250927,C=12)
	CT(73503)=(S=14250928,E=14250943,C=18)
	CT(73504)=(S=14250944,E=14250959,C=36)
	CT(73505)=(S=14250960,E=14250975,C=38)
	CT(73506)=(S=14250976,E=14250991,C=36)
	CT(73507)=(S=14250992,E=14251007,C=33)
	CT(73508)=(S=14251008,E=14251039,C=25)
	CT(73509)=(S=14251040,E=14251055,C=50)
	CT(73510)=(S=14251056,E=14251071,C=18)
	CT(73511)=(S=14251072,E=14251087,C=42)
	CT(73512)=(S=14251088,E=14251103,C=27)
	CT(73513)=(S=14251104,E=14251119,C=58)
	CT(73514)=(S=14251120,E=14251135,C=28)
	CT(73515)=(S=14251136,E=14251167,C=18)
	CT(73516)=(S=14251168,E=14251183,C=47)
	CT(73517)=(S=14251184,E=14251199,C=42)
	CT(73518)=(S=14251200,E=14251215,C=37)
	CT(73519)=(S=14251216,E=14251263,C=24)
	CT(73520)=(S=14251264,E=14251279,C=111)
	CT(73521)=(S=14251280,E=14251295,C=44)
	CT(73522)=(S=14251296,E=14251327,C=49)
	CT(73523)=(S=14251328,E=14251343,C=31)
	CT(73524)=(S=14251344,E=14251359,C=18)
	CT(73525)=(S=14251360,E=14251375,C=28)
	CT(73526)=(S=14251376,E=14251391,C=18)
	CT(73527)=(S=14251392,E=14251407,C=58)
	CT(73528)=(S=14251408,E=14251423,C=12)
	CT(73529)=(S=14251424,E=14251439,C=34)
	CT(73530)=(S=14251440,E=14251455,C=18)
	CT(73531)=(S=14251456,E=14251471,C=70)
	CT(73532)=(S=14251472,E=14251487,C=33)
	CT(73533)=(S=14251488,E=14251503,C=36)
	CT(73534)=(S=14251504,E=14251519,C=18)
	CT(73535)=(S=14251520,E=14251535,C=15)
	CT(73536)=(S=14251536,E=14251551,C=28)
	CT(73537)=(S=14251552,E=14251583,C=27)
	CT(73538)=(S=14251584,E=14251615,C=18)
	CT(73539)=(S=14251616,E=14251631,C=71)
	CT(73540)=(S=14251632,E=14251663,C=14)
	CT(73541)=(S=14251664,E=14251679,C=15)
	CT(73542)=(S=14251680,E=14251695,C=36)
	CT(73543)=(S=14251696,E=14251711,C=18)
	CT(73544)=(S=14251712,E=14251727,C=34)
	CT(73545)=(S=14251728,E=14251743,C=19)
	CT(73546)=(S=14251744,E=14251759,C=12)
	CT(73547)=(S=14251760,E=14251775,C=15)
	CT(73548)=(S=14251776,E=14251791,C=36)
	CT(73549)=(S=14251792,E=14251807,C=18)
	CT(73550)=(S=14251808,E=14251823,C=47)
	CT(73551)=(S=14251824,E=14251839,C=28)
	CT(73552)=(S=14251840,E=14251855,C=58)
	CT(73553)=(S=14251856,E=14251871,C=18)
	CT(73554)=(S=14251872,E=14251887,C=34)
	CT(73555)=(S=14251888,E=14251903,C=71)
	CT(73556)=(S=14251904,E=14251919,C=12)
	CT(73557)=(S=14251920,E=14251935,C=34)
	CT(73558)=(S=14251936,E=14251951,C=19)
	CT(73559)=(S=14251952,E=14251967,C=12)
	CT(73560)=(S=14251968,E=14251999,C=28)
	CT(73561)=(S=14252000,E=14252015,C=36)
	CT(73562)=(S=14252016,E=14252031,C=46)
	CT(73563)=(S=14252032,E=14253055,C=36)
	CT(73564)=(S=14253056,E=14254079,C=25)
	CT(73565)=(S=14254080,E=14254335,C=12)
	CT(73566)=(S=14254336,E=14254591,C=21)
	CT(73567)=(S=14254592,E=14254847,C=25)
	CT(73568)=(S=14254848,E=14255103,C=37)
	CT(73569)=(S=14255104,E=14255359,C=17)
	CT(73570)=(S=14255360,E=14255615,C=15)
	CT(73571)=(S=14255616,E=14256127,C=14)
	CT(73572)=(S=14256128,E=14256383,C=49)
	CT(73573)=(S=14256384,E=14256895,C=14)
	CT(73574)=(S=14256896,E=14257151,C=106)
	CT(73575)=(S=14257152,E=14257215,C=36)
	CT(73576)=(S=14257216,E=14257247,C=28)
	CT(73577)=(S=14257248,E=14257263,C=14)
	CT(73578)=(S=14257264,E=14257279,C=19)
	CT(73579)=(S=14257280,E=14257407,C=47)
	CT(73580)=(S=14257408,E=14257663,C=15)
	CT(73581)=(S=14257664,E=14258175,C=19)
	CT(73582)=(S=14258176,E=14258191,C=61)
	CT(73583)=(S=14258192,E=14258207,C=71)
	CT(73584)=(S=14258208,E=14258223,C=28)
	CT(73585)=(S=14258224,E=14258239,C=50)
	CT(73586)=(S=14258240,E=14258255,C=27)
	CT(73587)=(S=14258256,E=14258271,C=14)
	CT(73588)=(S=14258272,E=14258287,C=18)
	CT(73589)=(S=14258288,E=14258303,C=12)
	CT(73590)=(S=14258304,E=14258319,C=28)
	CT(73591)=(S=14258320,E=14258335,C=14)
	CT(73592)=(S=14258336,E=14258367,C=18)
	CT(73593)=(S=14258368,E=14258399,C=58)
	CT(73594)=(S=14258400,E=14258431,C=27)
	CT(73595)=(S=14258432,E=14258447,C=72)
	CT(73596)=(S=14258448,E=14258463,C=18)
	CT(73597)=(S=14258464,E=14258479,C=49)
	CT(73598)=(S=14258480,E=14258495,C=24)
	CT(73599)=(S=14258496,E=14258511,C=14)
	CT(73600)=(S=14258512,E=14258527,C=51)
	CT(73601)=(S=14258528,E=14258543,C=28)
	CT(73602)=(S=14258544,E=14258575,C=14)
	CT(73603)=(S=14258576,E=14258591,C=18)
	CT(73604)=(S=14258592,E=14258607,C=51)
	CT(73605)=(S=14258608,E=14258623,C=25)
	CT(73606)=(S=14258624,E=14258639,C=71)
	CT(73607)=(S=14258640,E=14258655,C=60)
	CT(73608)=(S=14258672,E=14258687,C=23)
	CT(73609)=(S=14258688,E=14258719,C=42)
	CT(73610)=(S=14258720,E=14258735,C=18)
	CT(73611)=(S=14258736,E=14258751,C=14)
	CT(73612)=(S=14258752,E=14258767,C=55)
	CT(73613)=(S=14258768,E=14258815,C=14)
	CT(73614)=(S=14258816,E=14258847,C=28)
	CT(73615)=(S=14258848,E=14258863,C=34)
	CT(73616)=(S=14258864,E=14258879,C=14)
	CT(73617)=(S=14258880,E=14258895,C=15)
	CT(73618)=(S=14258896,E=14258911,C=26)
	CT(73619)=(S=14258912,E=14258927,C=12)
	CT(73620)=(S=14258928,E=14258943,C=31)
	CT(73621)=(S=14258960,E=14258975,C=18)
	CT(73622)=(S=14258976,E=14258991,C=44)
	CT(73623)=(S=14258992,E=14259023,C=28)
	CT(73624)=(S=14259024,E=14259039,C=14)
	CT(73625)=(S=14259040,E=14259055,C=28)
	CT(73626)=(S=14259056,E=14259071,C=69)
	CT(73627)=(S=14259072,E=14259103,C=15)
	CT(73628)=(S=14259104,E=14259119,C=31)
	CT(73629)=(S=14259120,E=14259135,C=14)
	CT(73630)=(S=14259136,E=14259151,C=12)
	CT(73631)=(S=14259152,E=14259167,C=34)
	CT(73632)=(S=14259168,E=14259183,C=40)
	CT(73633)=(S=14259184,E=14259199,C=14)
	CT(73634)=(S=14259200,E=14259215,C=34)
	CT(73635)=(S=14259216,E=14259231,C=36)
	CT(73636)=(S=14259232,E=14259247,C=14)
	CT(73637)=(S=14259248,E=14259263,C=18)
	CT(73638)=(S=14259264,E=14259279,C=25)
	CT(73639)=(S=14259280,E=14259295,C=36)
	CT(73640)=(S=14259296,E=14259311,C=28)
	CT(73641)=(S=14259312,E=14259327,C=15)
	CT(73642)=(S=14259344,E=14259359,C=27)
	CT(73643)=(S=14259360,E=14259391,C=36)
	CT(73644)=(S=14259392,E=14259423,C=18)
	CT(73645)=(S=14259424,E=14259439,C=28)
	CT(73646)=(S=14259440,E=14259455,C=49)
	CT(73647)=(S=14259456,E=14259471,C=25)
	CT(73648)=(S=14259472,E=14259487,C=18)
	CT(73649)=(S=14259488,E=14259503,C=36)
	CT(73650)=(S=14259504,E=14259519,C=47)
	CT(73651)=(S=14259520,E=14259535,C=36)
	CT(73652)=(S=14259536,E=14259551,C=15)
	CT(73653)=(S=14259552,E=14259567,C=14)
	CT(73654)=(S=14259568,E=14259583,C=27)
	CT(73655)=(S=14259584,E=14259599,C=36)
	CT(73656)=(S=14259600,E=14259615,C=25)
	CT(73657)=(S=14259616,E=14259631,C=42)
	CT(73658)=(S=14259632,E=14259647,C=18)
	CT(73659)=(S=14259648,E=14259679,C=36)
	CT(73660)=(S=14259680,E=14259695,C=42)
	CT(73661)=(S=14259696,E=14259711,C=58)
	CT(73662)=(S=14259712,E=14259775,C=18)
	CT(73663)=(S=14259776,E=14259783,C=19)
	CT(73664)=(S=14259784,E=14259791,C=18)
	CT(73665)=(S=14259792,E=14259799,C=39)
	CT(73666)=(S=14259800,E=14259807,C=36)
	CT(73667)=(S=14259808,E=14259839,C=14)
	CT(73668)=(S=14259840,E=14259855,C=50)
	CT(73669)=(S=14259856,E=14259871,C=28)
	CT(73670)=(S=14259872,E=14259903,C=19)
	CT(73671)=(S=14259904,E=14259919,C=18)
	CT(73672)=(S=14259920,E=14259935,C=25)
	CT(73673)=(S=14259936,E=14259967,C=34)
	CT(73674)=(S=14259968,E=14259983,C=12)
	CT(73675)=(S=14259984,E=14259999,C=18)
	CT(73676)=(S=14260000,E=14260015,C=14)
	CT(73677)=(S=14260016,E=14260031,C=19)
	CT(73678)=(S=14260032,E=14260047,C=18)
	CT(73679)=(S=14260048,E=14260063,C=28)
	CT(73680)=(S=14260064,E=14260079,C=14)
	CT(73681)=(S=14260080,E=14260095,C=34)
	CT(73682)=(S=14260096,E=14260111,C=18)
	CT(73683)=(S=14260112,E=14260127,C=28)
	CT(73684)=(S=14260128,E=14260159,C=48)
	CT(73685)=(S=14260160,E=14260175,C=19)
	CT(73686)=(S=14260176,E=14260199,C=28)
	CT(73687)=(S=14260200,E=14260207,C=18)
	CT(73688)=(S=14260208,E=14260223,C=28)
	CT(73689)=(S=14260224,E=14260479,C=47)
	CT(73690)=(S=14260480,E=14260735,C=58)
	CT(73691)=(S=14260736,E=14261247,C=14)
	CT(73692)=(S=14261248,E=14261375,C=38)
	CT(73693)=(S=14261376,E=14261503,C=14)
	CT(73694)=(S=14261504,E=14261759,C=24)
	CT(73695)=(S=14261760,E=14262015,C=14)
	CT(73696)=(S=14262016,E=14262143,C=28)
	CT(73697)=(S=14262144,E=14262271,C=55)
	CT(73698)=(S=14262272,E=14262527,C=28)
	CT(73699)=(S=14262528,E=14262783,C=13)
	CT(73700)=(S=14262784,E=14263039,C=34)
	CT(73701)=(S=14263040,E=14263295,C=14)
	CT(73702)=(S=14263296,E=14263807,C=16)
	CT(73703)=(S=14263808,E=14264063,C=36)
	CT(73704)=(S=14264064,E=14264319,C=12)
	CT(73705)=(S=14264320,E=14264335,C=25)
	CT(73706)=(S=14264336,E=14264351,C=14)
	CT(73707)=(S=14264352,E=14264383,C=34)
	CT(73708)=(S=14264384,E=14264399,C=18)
	CT(73709)=(S=14264400,E=14264415,C=27)
	CT(73710)=(S=14264416,E=14264431,C=12)
	CT(73711)=(S=14264432,E=14264447,C=21)
	CT(73712)=(S=14264448,E=14264463,C=58)
	CT(73713)=(S=14264464,E=14264479,C=14)
	CT(73714)=(S=14264480,E=14264495,C=72)
	CT(73715)=(S=14264496,E=14264511,C=41)
	CT(73716)=(S=14264512,E=14264527,C=58)
	CT(73717)=(S=14264528,E=14264543,C=33)
	CT(73718)=(S=14264544,E=14264559,C=15)
	CT(73719)=(S=14264560,E=14264575,C=18)
	CT(73720)=(S=14264576,E=14264639,C=14)
	CT(73721)=(S=14264640,E=14264655,C=47)
	CT(73722)=(S=14264656,E=14264671,C=18)
	CT(73723)=(S=14264672,E=14264703,C=15)
	CT(73724)=(S=14264704,E=14264735,C=34)
	CT(73725)=(S=14264736,E=14264751,C=14)
	CT(73726)=(S=14264752,E=14264767,C=33)
	CT(73727)=(S=14264768,E=14264783,C=37)
	CT(73728)=(S=14264784,E=14264799,C=46)
	CT(73729)=(S=14264800,E=14264815,C=36)
	CT(73730)=(S=14264816,E=14264831,C=12)
	CT(73731)=(S=14264832,E=14264895,C=36)
	CT(73732)=(S=14264896,E=14264927,C=18)
	CT(73733)=(S=14264928,E=14264943,C=33)
	CT(73734)=(S=14264944,E=14264959,C=18)
	CT(73735)=(S=14264960,E=14264975,C=27)
	CT(73736)=(S=14264976,E=14264991,C=123)
	CT(73737)=(S=14264992,E=14265007,C=58)
	CT(73738)=(S=14265008,E=14265023,C=28)
	CT(73739)=(S=14265024,E=14265039,C=27)
	CT(73740)=(S=14265040,E=14265055,C=18)
	CT(73741)=(S=14265056,E=14265071,C=15)
	CT(73742)=(S=14265072,E=14265087,C=26)
	CT(73743)=(S=14265088,E=14265103,C=18)
	CT(73744)=(S=14265104,E=14265119,C=12)
	CT(73745)=(S=14265120,E=14265135,C=15)
	CT(73746)=(S=14265136,E=14265151,C=58)
	CT(73747)=(S=14265152,E=14265167,C=15)
	CT(73748)=(S=14265168,E=14265183,C=49)
	CT(73749)=(S=14265184,E=14265199,C=14)
	CT(73750)=(S=14265200,E=14265215,C=28)
	CT(73751)=(S=14265216,E=14265231,C=14)
	CT(73752)=(S=14265232,E=14265247,C=18)
	CT(73753)=(S=14265248,E=14265263,C=15)
	CT(73754)=(S=14265264,E=14265279,C=28)
	CT(73755)=(S=14265280,E=14265295,C=27)
	CT(73756)=(S=14265296,E=14265311,C=48)
	CT(73757)=(S=14265312,E=14265319,C=36)
	CT(73758)=(S=14265320,E=14265327,C=15)
	CT(73759)=(S=14265328,E=14265343,C=28)
	CT(73760)=(S=14265344,E=14265359,C=15)
	CT(73761)=(S=14265360,E=14265375,C=18)
	CT(73762)=(S=14265376,E=14265407,C=28)
	CT(73763)=(S=14265408,E=14265423,C=25)
	CT(73764)=(S=14265424,E=14265439,C=171)
	CT(73765)=(S=14265440,E=14265471,C=26)
	CT(73766)=(S=14265472,E=14265487,C=14)
	CT(73767)=(S=14265488,E=14265503,C=71)
	CT(73768)=(S=14265504,E=14265535,C=28)
	CT(73769)=(S=14265536,E=14265567,C=15)
	CT(73770)=(S=14265568,E=14265615,C=58)
	CT(73771)=(S=14265616,E=14265631,C=18)
	CT(73772)=(S=14265632,E=14265647,C=50)
	CT(73773)=(S=14265648,E=14265663,C=14)
	CT(73774)=(S=14265664,E=14265679,C=18)
	CT(73775)=(S=14265680,E=14265695,C=23)
	CT(73776)=(S=14265696,E=14265711,C=63)
	CT(73777)=(S=14265712,E=14265727,C=25)
	CT(73778)=(S=14265728,E=14265759,C=28)
	CT(73779)=(S=14265760,E=14265807,C=58)
	CT(73780)=(S=14265808,E=14265823,C=63)
	CT(73781)=(S=14265824,E=14265839,C=56)
	CT(73782)=(S=14265840,E=14265855,C=27)
	CT(73783)=(S=14265856,E=14265871,C=18)
	CT(73784)=(S=14265872,E=14265887,C=26)
	CT(73785)=(S=14265888,E=14265903,C=37)
	CT(73786)=(S=14265904,E=14265919,C=51)
	CT(73787)=(S=14265920,E=14265951,C=19)
	CT(73788)=(S=14265952,E=14265967,C=18)
	CT(73789)=(S=14265968,E=14265999,C=15)
	CT(73790)=(S=14266000,E=14266015,C=51)
	CT(73791)=(S=14266016,E=14266047,C=18)
	CT(73792)=(S=14266048,E=14266079,C=12)
	CT(73793)=(S=14266080,E=14266095,C=190)
	CT(73794)=(S=14266096,E=14266111,C=14)
	CT(73795)=(S=14266112,E=14266127,C=31)
	CT(73796)=(S=14266128,E=14266159,C=18)
	CT(73797)=(S=14266160,E=14266175,C=15)
	CT(73798)=(S=14266176,E=14266191,C=14)
	CT(73799)=(S=14266192,E=14266207,C=31)
	CT(73800)=(S=14266208,E=14266223,C=28)
	CT(73801)=(S=14266224,E=14266239,C=19)
	CT(73802)=(S=14266240,E=14266271,C=18)
	CT(73803)=(S=14266272,E=14266303,C=12)
	CT(73804)=(S=14266304,E=14266319,C=42)
	CT(73805)=(S=14266320,E=14266335,C=32)
	CT(73806)=(S=14266336,E=14266367,C=28)
	CT(73807)=(S=14266368,E=14268415,C=14)
	CT(73808)=(S=14268416,E=14270463,C=28)
	CT(73809)=(S=14270464,E=14270975,C=34)
	CT(73810)=(S=14270976,E=14270991,C=15)
	CT(73811)=(S=14270992,E=14271007,C=36)
	CT(73812)=(S=14271008,E=14271023,C=28)
	CT(73813)=(S=14271024,E=14271039,C=34)
	CT(73814)=(S=14271040,E=14271055,C=28)
	CT(73815)=(S=14271056,E=14271071,C=12)
	CT(73816)=(S=14271072,E=14271103,C=36)
	CT(73817)=(S=14271104,E=14271135,C=14)
	CT(73818)=(S=14271136,E=14271151,C=33)
	CT(73819)=(S=14271152,E=14271167,C=15)
	CT(73820)=(S=14271168,E=14271183,C=17)
	CT(73821)=(S=14271184,E=14271199,C=14)
	CT(73822)=(S=14271200,E=14271215,C=28)
	CT(73823)=(S=14271216,E=14271231,C=18)
	CT(73824)=(S=14271232,E=14271247,C=28)
	CT(73825)=(S=14271248,E=14271263,C=12)
	CT(73826)=(S=14271264,E=14271279,C=28)
	CT(73827)=(S=14271280,E=14271295,C=60)
	CT(73828)=(S=14271296,E=14271343,C=18)
	CT(73829)=(S=14271344,E=14271359,C=36)
	CT(73830)=(S=14271360,E=14271375,C=22)
	CT(73831)=(S=14271392,E=14271407,C=33)
	CT(73832)=(S=14271408,E=14271423,C=24)
	CT(73833)=(S=14271424,E=14271439,C=37)
	CT(73834)=(S=14271440,E=14271455,C=18)
	CT(73835)=(S=14271456,E=14271487,C=36)
	CT(73836)=(S=14271488,E=14271503,C=14)
	CT(73837)=(S=14271504,E=14271519,C=20)
	CT(73838)=(S=14271520,E=14271535,C=36)
	CT(73839)=(S=14271536,E=14271551,C=27)
	CT(73840)=(S=14271552,E=14271583,C=42)
	CT(73841)=(S=14271584,E=14271599,C=18)
	CT(73842)=(S=14271600,E=14271615,C=33)
	CT(73843)=(S=14271616,E=14271631,C=15)
	CT(73844)=(S=14271632,E=14271647,C=42)
	CT(73845)=(S=14271648,E=14271663,C=31)
	CT(73846)=(S=14271664,E=14271679,C=34)
	CT(73847)=(S=14271680,E=14271695,C=54)
	CT(73848)=(S=14271696,E=14271711,C=33)
	CT(73849)=(S=14271712,E=14271743,C=14)
	CT(73850)=(S=14271744,E=14271759,C=18)
	CT(73851)=(S=14271760,E=14271775,C=25)
	CT(73852)=(S=14271776,E=14271791,C=33)
	CT(73853)=(S=14271792,E=14271807,C=19)
	CT(73854)=(S=14271808,E=14271823,C=58)
	CT(73855)=(S=14271824,E=14271839,C=28)
	CT(73856)=(S=14271840,E=14271855,C=70)
	CT(73857)=(S=14271856,E=14271871,C=18)
	CT(73858)=(S=14271872,E=14271879,C=34)
	CT(73859)=(S=14271880,E=14271887,C=59)
	CT(73860)=(S=14271888,E=14271903,C=33)
	CT(73861)=(S=14271920,E=14271935,C=50)
	CT(73862)=(S=14271936,E=14271951,C=18)
	CT(73863)=(S=14271952,E=14271967,C=34)
	CT(73864)=(S=14271968,E=14272015,C=18)
	CT(73865)=(S=14272016,E=14272031,C=36)
	CT(73866)=(S=14272032,E=14272063,C=14)
	CT(73867)=(S=14272064,E=14272079,C=19)
	CT(73868)=(S=14272080,E=14272095,C=18)
	CT(73869)=(S=14272096,E=14272111,C=14)
	CT(73870)=(S=14272112,E=14272127,C=33)
	CT(73871)=(S=14272128,E=14272143,C=28)
	CT(73872)=(S=14272144,E=14272159,C=19)
	CT(73873)=(S=14272160,E=14272191,C=18)
	CT(73874)=(S=14272192,E=14272207,C=25)
	CT(73875)=(S=14272208,E=14272223,C=24)
	CT(73876)=(S=14272224,E=14272239,C=60)
	CT(73877)=(S=14272240,E=14272255,C=28)
	CT(73878)=(S=14272256,E=14272287,C=15)
	CT(73879)=(S=14272288,E=14272319,C=27)
	CT(73880)=(S=14272320,E=14272335,C=28)
	CT(73881)=(S=14272336,E=14272351,C=42)
	CT(73882)=(S=14272352,E=14272383,C=60)
	CT(73883)=(S=14272384,E=14272399,C=59)
	CT(73884)=(S=14272400,E=14272415,C=109)
	CT(73885)=(S=14272416,E=14272447,C=14)
	CT(73886)=(S=14272448,E=14272463,C=28)
	CT(73887)=(S=14272464,E=14272479,C=18)
	CT(73888)=(S=14272480,E=14272495,C=31)
	CT(73889)=(S=14272496,E=14272511,C=18)
	CT(73890)=(S=14272512,E=14273535,C=15)
	CT(73891)=(S=14273536,E=14274559,C=14)
	CT(73892)=(S=14274560,E=14276607,C=19)
	CT(73893)=(S=14276608,E=14277119,C=25)
	CT(73894)=(S=14277120,E=14277631,C=26)
	CT(73895)=(S=14277632,E=14278655,C=15)
	CT(73896)=(S=14278656,E=14286847,C=28)
	CT(73897)=(S=14286848,E=14295039,C=1)
	CT(73898)=(S=14295040,E=14295295,C=9)
	CT(73899)=(S=14295296,E=14295423,C=74)
	CT(73900)=(S=14295424,E=14295551,C=2)
	CT(73901)=(S=14295552,E=14296063,C=9)
	CT(73902)=(S=14296064,E=14297087,C=6)
	CT(73903)=(S=14297088,E=14299135,C=2)
	CT(73904)=(S=14299136,E=14301183,C=6)
	CT(73905)=(S=14301184,E=14312447,C=1)
	CT(73906)=(S=14312448,E=14312451,C=0)
	CT(73907)=(S=14312452,E=14312452,C=74)
	CT(73908)=(S=14312453,E=14312457,C=2)
	CT(73909)=(S=14312458,E=14312458,C=11)
	CT(73910)=(S=14312459,E=14312459,C=74)
	CT(73911)=(S=14312460,E=14312461,C=0)
	CT(73912)=(S=14312462,E=14312462,C=11)
	CT(73913)=(S=14312463,E=14312463,C=2)
	CT(73914)=(S=14312464,E=14312465,C=8)
	CT(73915)=(S=14312466,E=14312466,C=5)
	CT(73916)=(S=14312467,E=14312467,C=0)
	CT(73917)=(S=14312469,E=14312469,C=73)
	CT(73918)=(S=14312470,E=14312470,C=5)
	CT(73919)=(S=14312471,E=14312474,C=73)
	CT(73920)=(S=14312475,E=14312483,C=74)
	CT(73921)=(S=14312484,E=14312485,C=0)
	CT(73922)=(S=14312486,E=14312486,C=3)
	CT(73923)=(S=14312487,E=14312488,C=0)
	CT(73924)=(S=14312489,E=14312489,C=74)
	CT(73925)=(S=14312490,E=14312490,C=2)
	CT(73926)=(S=14312491,E=14312491,C=0)
	CT(73927)=(S=14312492,E=14312492,C=5)
	CT(73928)=(S=14312493,E=14312493,C=2)
	CT(73929)=(S=14312494,E=14312495,C=3)
	CT(73930)=(S=14312496,E=14312497,C=4)
	CT(73931)=(S=14312498,E=14312502,C=0)
	CT(73932)=(S=14312503,E=14312503,C=74)
	CT(73933)=(S=14312504,E=14312504,C=73)
	CT(73934)=(S=14312505,E=14312505,C=156)
	CT(73935)=(S=14312506,E=14312506,C=5)
	CT(73936)=(S=14312507,E=14312507,C=0)
	CT(73937)=(S=14312508,E=14312508,C=11)
	CT(73938)=(S=14312509,E=14312509,C=156)
	CT(73939)=(S=14312510,E=14312512,C=74)
	CT(73940)=(S=14312513,E=14312513,C=73)
	CT(73941)=(S=14312514,E=14312514,C=3)
	CT(73942)=(S=14312515,E=14312515,C=2)
	CT(73943)=(S=14312517,E=14312517,C=5)
	CT(73944)=(S=14312519,E=14312519,C=86)
	CT(73945)=(S=14312520,E=14312522,C=74)
	CT(73946)=(S=14312523,E=14312523,C=5)
	CT(73947)=(S=14312524,E=14312524,C=0)
	CT(73948)=(S=14312525,E=14312525,C=164)
	CT(73949)=(S=14312526,E=14312526,C=0)
	CT(73950)=(S=14312527,E=14312527,C=73)
	CT(73951)=(S=14312528,E=14312531,C=0)
	CT(73952)=(S=14312532,E=14312532,C=92)
	CT(73953)=(S=14312533,E=14312533,C=77)
	CT(73954)=(S=14312535,E=14312535,C=0)
	CT(73955)=(S=14312536,E=14312703,C=1)
	CT(73956)=(S=14312704,E=14312831,C=73)
	CT(73957)=(S=14312832,E=14312959,C=6)
	CT(73958)=(S=14312960,E=14313471,C=8)
	CT(73959)=(S=14313472,E=14315007,C=1)
	CT(73960)=(S=14315008,E=14315263,C=2)
	CT(73961)=(S=14315264,E=14315519,C=5)
	CT(73962)=(S=14315520,E=14323711,C=2)
	CT(73963)=(S=14323712,E=14327807,C=6)
	CT(73964)=(S=14327808,E=14331903,C=9)
	CT(73965)=(S=14331904,E=14333951,C=2)
	CT(73966)=(S=14333952,E=14334207,C=9)
	CT(73967)=(S=14334208,E=14334335,C=0)
	CT(73968)=(S=14334336,E=14334399,C=2)
	CT(73969)=(S=14334400,E=14334431,C=1)
	CT(73970)=(S=14334432,E=14334439,C=73)
	CT(73971)=(S=14334440,E=14334447,C=0)
	CT(73972)=(S=14334448,E=14334455,C=1)
	CT(73973)=(S=14334464,E=14334719,C=7)
	CT(73974)=(S=14334720,E=14334975,C=9)
	CT(73975)=(S=14334976,E=14335999,C=8)
	CT(73976)=(S=14336000,E=14340095,C=1)
	CT(73977)=(S=14340096,E=14340351,C=5)
	CT(73978)=(S=14340352,E=14340607,C=6)
	CT(73979)=(S=14340608,E=14341119,C=9)
	CT(73980)=(S=14341120,E=14341375,C=7)
	CT(73981)=(S=14341376,E=14341631,C=8)
	CT(73982)=(S=14341632,E=14342143,C=0)
	CT(73983)=(S=14342144,E=14346239,C=2)
	CT(73984)=(S=14346240,E=14348287,C=6)
	CT(73985)=(S=14348288,E=14350335,C=1)
	CT(73986)=(S=14350336,E=14350591,C=4)
	CT(73987)=(S=14350592,E=14350847,C=1)
	CT(73988)=(S=14350848,E=14351103,C=8)
	CT(73989)=(S=14351104,E=14351359,C=2)
	CT(73990)=(S=14351360,E=14352383,C=8)
	CT(73991)=(S=14352384,E=14368767,C=2)
	CT(73992)=(S=14368768,E=14369279,C=4)
	CT(73993)=(S=14369280,E=14369791,C=2)
	CT(73994)=(S=14369792,E=14370815,C=9)
	CT(73995)=(S=14370816,E=14371071,C=1)
	CT(73996)=(S=14371072,E=14371199,C=8)
	CT(73997)=(S=14371200,E=14371327,C=2)
	CT(73998)=(S=14371328,E=14371711,C=7)
	CT(73999)=(S=14371712,E=14371839,C=2)
	CT(74000)=(S=14371840,E=14372863,C=8)
	CT(74001)=(S=14372864,E=14373375,C=9)
	CT(74002)=(S=14373376,E=14373631,C=1)
	CT(74003)=(S=14373632,E=14373759,C=74)
	CT(74004)=(S=14373760,E=14373887,C=1)
	CT(74005)=(S=14373888,E=14374911,C=9)
	CT(74006)=(S=14374912,E=14375423,C=73)
	CT(74007)=(S=14375424,E=14375487,C=9)
	CT(74008)=(S=14375504,E=14375519,C=10)
	CT(74009)=(S=14375520,E=14375535,C=4)
	CT(74010)=(S=14375536,E=14375551,C=8)
	CT(74011)=(S=14375552,E=14375679,C=0)
	CT(74012)=(S=14375680,E=14375807,C=9)
	CT(74013)=(S=14375808,E=14375935,C=4)
	CT(74014)=(S=14375936,E=14376575,C=5)
	CT(74015)=(S=14376576,E=14376703,C=2)
	CT(74016)=(S=14376704,E=14376959,C=5)
	CT(74017)=(S=14376960,E=14385151,C=2)
	CT(74018)=(S=14385152,E=14393343,C=1)
	CT(74019)=(S=14393344,E=14407679,C=2)
	CT(74020)=(S=14407680,E=14413823,C=1)
	CT(74021)=(S=14413824,E=14414335,C=6)
	CT(74022)=(S=14414336,E=14415871,C=1)
	CT(74023)=(S=14415872,E=14417919,C=6)
	CT(74024)=(S=14417920,E=14434303,C=2)
	CT(74025)=(S=14434304,E=14442495,C=6)
	CT(74026)=(S=14442496,E=14443775,C=2)
	CT(74027)=(S=14443776,E=14443967,C=0)
	CT(74028)=(S=14443968,E=14444031,C=1)
	CT(74029)=(S=14444032,E=14444287,C=2)
	CT(74030)=(S=14444288,E=14444543,C=6)
	CT(74031)=(S=14444544,E=14446591,C=2)
	CT(74032)=(S=14446592,E=14447615,C=1)
	CT(74033)=(S=14447616,E=14450687,C=6)
	CT(74034)=(S=14450688,E=14454783,C=9)
	CT(74035)=(S=14454784,E=14456063,C=2)
	CT(74036)=(S=14456064,E=14456319,C=6)
	CT(74037)=(S=14456320,E=14456943,C=2)
	CT(74038)=(S=14456944,E=14456951,C=0)
	CT(74039)=(S=14456952,E=14456959,C=2)
	CT(74040)=(S=14456960,E=14457087,C=1)
	CT(74041)=(S=14457088,E=14457343,C=2)
	CT(74042)=(S=14457344,E=14457855,C=1)
	CT(74043)=(S=14457856,E=14458015,C=2)
	CT(74044)=(S=14458016,E=14458031,C=90)
	CT(74045)=(S=14458032,E=14458039,C=7)
	CT(74046)=(S=14458040,E=14458047,C=4)
	CT(74047)=(S=14458048,E=14458175,C=2)
	CT(74048)=(S=14458176,E=14458207,C=0)
	CT(74049)=(S=14458208,E=14458223,C=74)
	CT(74050)=(S=14458224,E=14458239,C=9)
	CT(74051)=(S=14458240,E=14458879,C=2)
	CT(74052)=(S=14458880,E=14471167,C=1)
	CT(74053)=(S=14471168,E=14475263,C=2)
	CT(74054)=(S=14475264,E=14476287,C=4)
	CT(74055)=(S=14476288,E=14476799,C=9)
	CT(74056)=(S=14476800,E=14477055,C=6)
	CT(74057)=(S=14477056,E=14477119,C=1)
	CT(74058)=(S=14477120,E=14477183,C=11)
	CT(74059)=(S=14477184,E=14477311,C=1)
	CT(74060)=(S=14477312,E=14477375,C=7)
	CT(74061)=(S=14477376,E=14477439,C=1)
	CT(74062)=(S=14477440,E=14477567,C=2)
	CT(74063)=(S=14477568,E=14477823,C=0)
	CT(74064)=(S=14477824,E=14478079,C=1)
	CT(74065)=(S=14478080,E=14479615,C=0)
	CT(74066)=(S=14479616,E=14479871,C=8)
	CT(74067)=(S=14479872,E=14480383,C=1)
	CT(74068)=(S=14480384,E=14480895,C=0)
	CT(74069)=(S=14480896,E=14481151,C=8)
	CT(74070)=(S=14481152,E=14481279,C=2)
	CT(74071)=(S=14481288,E=14481295,C=1)
	CT(74072)=(S=14481296,E=14481311,C=0)
	CT(74073)=(S=14481312,E=14481319,C=84)
	CT(74074)=(S=14481320,E=14481327,C=74)
	CT(74075)=(S=14481328,E=14481335,C=73)
	CT(74076)=(S=14481336,E=14481343,C=2)
	CT(74077)=(S=14481344,E=14481407,C=156)
	CT(74078)=(S=14481408,E=14482687,C=1)
	CT(74079)=(S=14482688,E=14482943,C=0)
	CT(74080)=(S=14482944,E=14483199,C=2)
	CT(74081)=(S=14483200,E=14483455,C=7)
	CT(74082)=(S=14483456,E=14486719,C=1)
	CT(74083)=(S=14486720,E=14486783,C=2)
	CT(74084)=(S=14486784,E=14487551,C=1)
	CT(74085)=(S=14487552,E=14514175,C=2)
	CT(74086)=(S=14514176,E=14514271,C=9)
	CT(74087)=(S=14514272,E=14514279,C=84)
	CT(74088)=(S=14514280,E=14514287,C=4)
	CT(74089)=(S=14514288,E=14514303,C=7)
	CT(74090)=(S=14514304,E=14514343,C=0)
	CT(74091)=(S=14514344,E=14514351,C=2)
	CT(74092)=(S=14514352,E=14514367,C=8)
	CT(74093)=(S=14514368,E=14514431,C=77)
	CT(74094)=(S=14514432,E=14514495,C=11)
	CT(74095)=(S=14514496,E=14514527,C=0)
	CT(74096)=(S=14514528,E=14514559,C=10)
	CT(74097)=(S=14514560,E=14514591,C=0)
	CT(74098)=(S=14514592,E=14514615,C=2)
	CT(74099)=(S=14514616,E=14514623,C=10)
	CT(74100)=(S=14514624,E=14514687,C=2)
	CT(74101)=(S=14514688,E=14515199,C=1)
	CT(74102)=(S=14515200,E=14516223,C=8)
	CT(74103)=(S=14516224,E=14516351,C=2)
	CT(74104)=(S=14516352,E=14517247,C=1)
	CT(74105)=(S=14517248,E=14517311,C=11)
	CT(74106)=(S=14517312,E=14517343,C=6)
	CT(74107)=(S=14517344,E=14517359,C=2)
	CT(74108)=(S=14517360,E=14517367,C=77)
	CT(74109)=(S=14517368,E=14517439,C=2)
	CT(74110)=(S=14517440,E=14517503,C=74)
	CT(74111)=(S=14517504,E=14517535,C=11)
	CT(74112)=(S=14517536,E=14517551,C=5)
	CT(74113)=(S=14517552,E=14517567,C=6)
	CT(74114)=(S=14517568,E=14517631,C=2)
	CT(74115)=(S=14517632,E=14517695,C=6)
	CT(74116)=(S=14517696,E=14517727,C=0)
	CT(74117)=(S=14517728,E=14517759,C=1)
	CT(74118)=(S=14517760,E=14518271,C=4)
	CT(74119)=(S=14518272,E=14518783,C=1)
	CT(74120)=(S=14518784,E=14526719,C=6)
	CT(74121)=(S=14526720,E=14526975,C=9)
	CT(74122)=(S=14526976,E=14527487,C=2)
	CT(74123)=(S=14527488,E=14530559,C=1)
	CT(74124)=(S=14530560,E=14532607,C=2)
	CT(74125)=(S=14532608,E=14534607,C=1)
	CT(74126)=(S=14534608,E=14534623,C=0)
	CT(74127)=(S=14534624,E=14544895,C=1)
	CT(74128)=(S=14544896,E=14553087,C=2)
	CT(74129)=(S=14553088,E=14573567,C=1)
	CT(74130)=(S=14573568,E=14580479,C=6)
	CT(74131)=(S=14580480,E=14580735,C=3)
	CT(74132)=(S=14580736,E=14580991,C=74)
	CT(74133)=(S=14580992,E=14581247,C=1)
	CT(74134)=(S=14581248,E=14581375,C=10)
	CT(74135)=(S=14581376,E=14581503,C=1)
	CT(74136)=(S=14581504,E=14581759,C=10)
	CT(74137)=(S=14581760,E=14585855,C=1)
	CT(74138)=(S=14585856,E=14587903,C=2)
	CT(74139)=(S=14587904,E=14588927,C=73)
	CT(74140)=(S=14588928,E=14589439,C=9)
	CT(74141)=(S=14589440,E=14589951,C=2)
	CT(74142)=(S=14589952,E=14590975,C=1)
	CT(74143)=(S=14590976,E=14591359,C=7)
	CT(74144)=(S=14591360,E=14591423,C=156)
	CT(74145)=(S=14591424,E=14591487,C=74)
	CT(74146)=(S=14591488,E=14591999,C=8)
	CT(74147)=(S=14592000,E=14606335,C=1)
	CT(74148)=(S=14606336,E=14607695,C=2)
	CT(74149)=(S=14607696,E=14607703,C=74)
	CT(74150)=(S=14607704,E=14607711,C=0)
	CT(74151)=(S=14607712,E=14608127,C=2)
	CT(74152)=(S=14608128,E=14608191,C=6)
	CT(74153)=(S=14608192,E=14608383,C=2)
	CT(74154)=(S=14608384,E=14610431,C=6)
	CT(74155)=(S=14610432,E=14612991,C=1)
	CT(74156)=(S=14612992,E=14613375,C=9)
	CT(74157)=(S=14613376,E=14613503,C=6)
	CT(74158)=(S=14613504,E=14614527,C=11)
	CT(74159)=(S=14614528,E=14618623,C=1)
	CT(74160)=(S=14618624,E=14619647,C=8)
	CT(74161)=(S=14619648,E=14620159,C=1)
	CT(74162)=(S=14620160,E=14620671,C=9)
	CT(74163)=(S=14620672,E=14620927,C=3)
	CT(74164)=(S=14620928,E=14620991,C=10)
	CT(74165)=(S=14620992,E=14621023,C=7)
	CT(74166)=(S=14621024,E=14621039,C=74)
	CT(74167)=(S=14621040,E=14621047,C=0)
	CT(74168)=(S=14621048,E=14621055,C=160)
	CT(74169)=(S=14621056,E=14621119,C=2)
	CT(74170)=(S=14621120,E=14621151,C=3)
	CT(74171)=(S=14621152,E=14621159,C=0)
	CT(74172)=(S=14621160,E=14621167,C=7)
	CT(74173)=(S=14621168,E=14621179,C=5)
	CT(74174)=(S=14621184,E=14621199,C=9)
	CT(74175)=(S=14621200,E=14621215,C=7)
	CT(74176)=(S=14621216,E=14621231,C=4)
	CT(74177)=(S=14621232,E=14621247,C=8)
	CT(74178)=(S=14621248,E=14621311,C=9)
	CT(74179)=(S=14621312,E=14621439,C=6)
	CT(74180)=(S=14621440,E=14621471,C=0)
	CT(74181)=(S=14621472,E=14621503,C=9)
	CT(74182)=(S=14621504,E=14621507,C=0)
	CT(74183)=(S=14621508,E=14621519,C=2)
	CT(74184)=(S=14621520,E=14621535,C=84)
	CT(74185)=(S=14621536,E=14621543,C=0)
	CT(74186)=(S=14621544,E=14621551,C=11)
	CT(74187)=(S=14621552,E=14621555,C=84)
	CT(74188)=(S=14621556,E=14621559,C=2)
	CT(74189)=(S=14621560,E=14621563,C=4)
	CT(74190)=(S=14621564,E=14621567,C=2)
	CT(74191)=(S=14621568,E=14621583,C=5)
	CT(74192)=(S=14621584,E=14621599,C=74)
	CT(74193)=(S=14621600,E=14621603,C=8)
	CT(74194)=(S=14621604,E=14621615,C=7)
	CT(74195)=(S=14621616,E=14621619,C=0)
	CT(74196)=(S=14621620,E=14621623,C=2)
	CT(74197)=(S=14621624,E=14621627,C=1)
	CT(74198)=(S=14621628,E=14621631,C=5)
	CT(74199)=(S=14621632,E=14621695,C=3)
	CT(74200)=(S=14621696,E=14621823,C=5)
	CT(74201)=(S=14621824,E=14621951,C=6)
	CT(74202)=(S=14621952,E=14622079,C=2)
	CT(74203)=(S=14622080,E=14622127,C=90)
	CT(74204)=(S=14622128,E=14622143,C=2)
	CT(74205)=(S=14622144,E=14622159,C=4)
	CT(74206)=(S=14622176,E=14622191,C=77)
	CT(74207)=(S=14622192,E=14622195,C=0)
	CT(74208)=(S=14622196,E=14622199,C=2)
	CT(74209)=(S=14622200,E=14622203,C=8)
	CT(74210)=(S=14622208,E=14622719,C=4)
	CT(74211)=(S=14622720,E=14630911,C=6)
	CT(74212)=(S=14630912,E=14644735,C=1)
	CT(74213)=(S=14644736,E=14645247,C=8)
	CT(74214)=(S=14645248,E=14647807,C=1)
	CT(74215)=(S=14647808,E=14647811,C=7)
	CT(74216)=(S=14647812,E=14647815,C=4)
	CT(74217)=(S=14647820,E=14647835,C=0)
	CT(74218)=(S=14647840,E=14647871,C=8)
	CT(74219)=(S=14647872,E=14648319,C=6)
	CT(74220)=(S=14648320,E=14649343,C=2)
	CT(74221)=(S=14649344,E=14651391,C=9)
	CT(74222)=(S=14651392,E=14656511,C=1)
	CT(74223)=(S=14656512,E=14656767,C=74)
	CT(74224)=(S=14656768,E=14656771,C=84)
	CT(74225)=(S=14656772,E=14656775,C=74)
	CT(74226)=(S=14656776,E=14656783,C=9)
	CT(74227)=(S=14656784,E=14656787,C=73)
	CT(74228)=(S=14656788,E=14656791,C=2)
	CT(74229)=(S=14656792,E=14656799,C=4)
	CT(74230)=(S=14656800,E=14656831,C=2)
	CT(74231)=(S=14656832,E=14656847,C=73)
	CT(74232)=(S=14656848,E=14656863,C=2)
	CT(74233)=(S=14656864,E=14656895,C=0)
	CT(74234)=(S=14656896,E=14657023,C=6)
	CT(74235)=(S=14657024,E=14657535,C=1)
	CT(74236)=(S=14657536,E=14659583,C=6)
	CT(74237)=(S=14659584,E=14663679,C=4)
	CT(74238)=(S=14663680,E=14664191,C=1)
	CT(74239)=(S=14664192,E=14664703,C=6)
	CT(74240)=(S=14664704,E=14664959,C=4)
	CT(74241)=(S=14664960,E=14665215,C=8)
	CT(74242)=(S=14665216,E=14665727,C=1)
	CT(74243)=(S=14665728,E=14665983,C=9)
	CT(74244)=(S=14665984,E=14666751,C=1)
	CT(74245)=(S=14666752,E=14667775,C=3)
	CT(74246)=(S=14667776,E=14669823,C=1)
	CT(74247)=(S=14669824,E=14670847,C=2)
	CT(74248)=(S=14670848,E=14671359,C=1)
	CT(74249)=(S=14671360,E=14671615,C=6)
	CT(74250)=(S=14671616,E=14671743,C=2)
	CT(74251)=(S=14671744,E=14671775,C=4)
	CT(74252)=(S=14671776,E=14671783,C=2)
	CT(74253)=(S=14671784,E=14671791,C=86)
	CT(74254)=(S=14671792,E=14671823,C=1)
	CT(74255)=(S=14671824,E=14671831,C=2)
	CT(74256)=(S=14671832,E=14671835,C=8)
	CT(74257)=(S=14671836,E=14671839,C=7)
	CT(74258)=(S=14671840,E=14671871,C=2)
	CT(74259)=(S=14671872,E=14675967,C=4)
	CT(74260)=(S=14675968,E=14679039,C=1)
	CT(74261)=(S=14679040,E=14679167,C=0)
	CT(74262)=(S=14679168,E=14679295,C=1)
	CT(74263)=(S=14679296,E=14679551,C=6)
	CT(74264)=(S=14679552,E=14679935,C=1)
	CT(74265)=(S=14679936,E=14679999,C=8)
	CT(74266)=(S=14680000,E=14680031,C=6)
	CT(74267)=(S=14680032,E=14680039,C=74)
	CT(74268)=(S=14680040,E=14680043,C=0)
	CT(74269)=(S=14680044,E=14680047,C=1)
	CT(74270)=(S=14680048,E=14680051,C=8)
	CT(74271)=(S=14680052,E=14680055,C=4)
	CT(74272)=(S=14680060,E=14680061,C=1)
	CT(74273)=(S=14680062,E=14680062,C=7)
	CT(74274)=(S=14680063,E=14680063,C=0)
}